// Code generated by wit-bindgen-go. DO NOT EDIT.

package run

import (
	"internal/cm"
)

// This file contains wasmimport and wasmexport declarations for "wasi:cli@0.2.0".

//go:wasmexport wasi:cli/run@0.2.0#run
//export wasi:cli/run@0.2.0#run
func wasmexport_Run() (result0 uint32) {
	result := Exports.Run()
	result0 = (uint32)(cm.BoolToU32(result))
	return
}
