// Automatically generated file. DO NOT EDIT.
// Generated by gen-device-svd.go from rp2350.svd, see https://github.com/posborne/cmsis-svd/tree/master/data/RaspberryPi

//go:build rp && rp2350

/*
//
//         Dual Cortex-M33 or Hazard3 processors at 150MHz
//         520kB on-chip SRAM, in 10 independent banks
//         Extended low-power sleep states with optional SRAM retention: as low as 10uA DVDD
//         8kB of one-time-programmable storage (OTP)
//         Up to 16MB of external QSPI flash/PSRAM via dedicated QSPI bus
//             Additional 16MB flash/PSRAM accessible via optional second chip-select
//         On-chip switched-mode power supply to generate core voltage
//             Low-quiescent-current LDO mode can be enabled for sleep states
//         2x on-chip PLLs for internal or external clock generation
//         GPIOs are 5V-tolerant (powered), and 3.3V-failsafe (unpowered)
//         Security features:
//             Optional boot signing, enforced by on-chip mask ROM, with key fingerprint in OTP
//             Protected OTP storage for optional boot decryption key
//             Global bus filtering based on Arm or RISC-V security/privilege levels
//             Peripherals, GPIOs and DMA channels individually assignable to security domains
//             Hardware mitigations for fault injection attacks
//             Hardware SHA-256 accelerator
//         Peripherals:
//             2x UARTs
//             2x SPI controllers
//             2x I2C controllers
//             24x PWM channels
//             USB 1.1 controller and PHY, with host and device support
//             12x PIO state machines
//             1x HSTX peripheral
//
*/
//     Copyright (c) 2024 Raspberry Pi Ltd. SPDX-License-Identifier: BSD-3-Clause
package rp

import (
	"runtime/volatile"
	"unsafe"
)

// Some information about this device.
const (
	Device       = "RP2350"
	CPU          = "CM33"
	FPUPresent   = true
	NVICPrioBits = 4
)

// Interrupt numbers.
const (
	// Controls time and alarms time is a 64 bit value indicating the time since power-on timeh is the top 32 bits of time & timel is the bottom 32 bits to change time write to timelw before timehw to read time read from timelr before timehr An alarm is set by setting alarm_enable and writing to the corresponding alarm register When an alarm is pending, the corresponding alarm_running signal will be high An alarm can be cancelled before it has finished by clearing the alarm_enable When an alarm fires, the corresponding alarm_irq is set and alarm_running is cleared To clear the interrupt write a 1 to the corresponding alarm_irq The timer can be locked to prevent writing
	IRQ_TIMER0_IRQ_0 = 0

	// Controls time and alarms time is a 64 bit value indicating the time since power-on timeh is the top 32 bits of time & timel is the bottom 32 bits to change time write to timelw before timehw to read time read from timelr before timehr An alarm is set by setting alarm_enable and writing to the corresponding alarm register When an alarm is pending, the corresponding alarm_running signal will be high An alarm can be cancelled before it has finished by clearing the alarm_enable When an alarm fires, the corresponding alarm_irq is set and alarm_running is cleared To clear the interrupt write a 1 to the corresponding alarm_irq The timer can be locked to prevent writing
	IRQ_TIMER0_IRQ_1 = 1

	// Controls time and alarms time is a 64 bit value indicating the time since power-on timeh is the top 32 bits of time & timel is the bottom 32 bits to change time write to timelw before timehw to read time read from timelr before timehr An alarm is set by setting alarm_enable and writing to the corresponding alarm register When an alarm is pending, the corresponding alarm_running signal will be high An alarm can be cancelled before it has finished by clearing the alarm_enable When an alarm fires, the corresponding alarm_irq is set and alarm_running is cleared To clear the interrupt write a 1 to the corresponding alarm_irq The timer can be locked to prevent writing
	IRQ_TIMER0_IRQ_2 = 2

	// Controls time and alarms time is a 64 bit value indicating the time since power-on timeh is the top 32 bits of time & timel is the bottom 32 bits to change time write to timelw before timehw to read time read from timelr before timehr An alarm is set by setting alarm_enable and writing to the corresponding alarm register When an alarm is pending, the corresponding alarm_running signal will be high An alarm can be cancelled before it has finished by clearing the alarm_enable When an alarm fires, the corresponding alarm_irq is set and alarm_running is cleared To clear the interrupt write a 1 to the corresponding alarm_irq The timer can be locked to prevent writing
	IRQ_TIMER0_IRQ_3 = 3

	IRQ_TIMER1_IRQ_0 = 4

	IRQ_TIMER1_IRQ_1 = 5

	IRQ_TIMER1_IRQ_2 = 6

	IRQ_TIMER1_IRQ_3 = 7

	// Simple PWM
	IRQ_PWM_IRQ_WRAP_0 = 8

	// Simple PWM
	IRQ_PWM_IRQ_WRAP_1 = 9

	// DMA with separate read and write masters
	IRQ_DMA_IRQ_0 = 10

	// DMA with separate read and write masters
	IRQ_DMA_IRQ_1 = 11

	// DMA with separate read and write masters
	IRQ_DMA_IRQ_2 = 12

	// DMA with separate read and write masters
	IRQ_DMA_IRQ_3 = 13

	// USB FS/LS controller device registers
	IRQ_USBCTRL_IRQ = 14

	// Programmable IO block
	IRQ_PIO0_IRQ_0 = 15

	// Programmable IO block
	IRQ_PIO0_IRQ_1 = 16

	IRQ_PIO1_IRQ_0 = 17

	IRQ_PIO1_IRQ_1 = 18

	IRQ_PIO2_IRQ_0 = 19

	IRQ_PIO2_IRQ_1 = 20

	IRQ_IO_IRQ_BANK0 = 21

	IRQ_IO_IRQ_BANK0_NS = 22

	IRQ_IO_IRQ_QSPI = 23

	IRQ_IO_IRQ_QSPI_NS = 24

	// Single-cycle IO block Provides core-local and inter-core hardware for the two processors, with single-cycle access.
	IRQ_SIO_IRQ_FIFO = 25

	// Single-cycle IO block Provides core-local and inter-core hardware for the two processors, with single-cycle access.
	IRQ_SIO_IRQ_BELL = 26

	// Single-cycle IO block Provides core-local and inter-core hardware for the two processors, with single-cycle access.
	IRQ_SIO_IRQ_FIFO_NS = 27

	// Single-cycle IO block Provides core-local and inter-core hardware for the two processors, with single-cycle access.
	IRQ_SIO_IRQ_BELL_NS = 28

	// Single-cycle IO block Provides core-local and inter-core hardware for the two processors, with single-cycle access.
	IRQ_SIO_IRQ_MTIMECMP = 29

	IRQ_CLOCKS_IRQ = 30

	IRQ_SPI0_IRQ = 31

	IRQ_SPI1_IRQ = 32

	IRQ_UART0_IRQ = 33

	IRQ_UART1_IRQ = 34

	// Control and data interface to SAR ADC
	IRQ_ADC_IRQ_FIFO = 35

	// DW_apb_i2c address block List of configuration constants for the Synopsys I2C hardware (you may see references to these in I2C register header; these are *fixed* values, set at hardware design time): IC_ULTRA_FAST_MODE ................ 0x0 IC_UFM_TBUF_CNT_DEFAULT ........... 0x8 IC_UFM_SCL_LOW_COUNT .............. 0x0008 IC_UFM_SCL_HIGH_COUNT ............. 0x0006 IC_TX_TL .......................... 0x0 IC_TX_CMD_BLOCK ................... 0x1 IC_HAS_DMA ........................ 0x1 IC_HAS_ASYNC_FIFO ................. 0x0 IC_SMBUS_ARP ...................... 0x0 IC_FIRST_DATA_BYTE_STATUS ......... 0x1 IC_INTR_IO ........................ 0x1 IC_MASTER_MODE .................... 0x1 IC_DEFAULT_ACK_GENERAL_CALL ....... 0x1 IC_INTR_POL ....................... 0x1 IC_OPTIONAL_SAR ................... 0x0 IC_DEFAULT_TAR_SLAVE_ADDR ......... 0x055 IC_DEFAULT_SLAVE_ADDR ............. 0x055 IC_DEFAULT_HS_SPKLEN .............. 0x1 IC_FS_SCL_HIGH_COUNT .............. 0x0006 IC_HS_SCL_LOW_COUNT ............... 0x0008 IC_DEVICE_ID_VALUE ................ 0x0 IC_10BITADDR_MASTER ............... 0x0 IC_CLK_FREQ_OPTIMIZATION .......... 0x0 IC_DEFAULT_FS_SPKLEN .............. 0x7 IC_ADD_ENCODED_PARAMS ............. 0x0 IC_DEFAULT_SDA_HOLD ............... 0x000001 IC_DEFAULT_SDA_SETUP .............. 0x64 IC_AVOID_RX_FIFO_FLUSH_ON_TX_ABRT . 0x0 IC_CLOCK_PERIOD ................... 100 IC_EMPTYFIFO_HOLD_MASTER_EN ....... 1 IC_RESTART_EN ..................... 0x1 IC_TX_CMD_BLOCK_DEFAULT ........... 0x0 IC_BUS_CLEAR_FEATURE .............. 0x0 IC_CAP_LOADING .................... 100 IC_FS_SCL_LOW_COUNT ............... 0x000d APB_DATA_WIDTH .................... 32 IC_SDA_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_SLV_DATA_NACK_ONLY ............. 0x1 IC_10BITADDR_SLAVE ................ 0x0 IC_CLK_TYPE ....................... 0x0 IC_SMBUS_UDID_MSB ................. 0x0 IC_SMBUS_SUSPEND_ALERT ............ 0x0 IC_HS_SCL_HIGH_COUNT .............. 0x0006 IC_SLV_RESTART_DET_EN ............. 0x1 IC_SMBUS .......................... 0x0 IC_OPTIONAL_SAR_DEFAULT ........... 0x0 IC_PERSISTANT_SLV_ADDR_DEFAULT .... 0x0 IC_USE_COUNTS ..................... 0x0 IC_RX_BUFFER_DEPTH ................ 16 IC_SCL_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_RX_FULL_HLD_BUS_EN ............. 0x1 IC_SLAVE_DISABLE .................. 0x1 IC_RX_TL .......................... 0x0 IC_DEVICE_ID ...................... 0x0 IC_HC_COUNT_VALUES ................ 0x0 I2C_DYNAMIC_TAR_UPDATE ............ 0 IC_SMBUS_CLK_LOW_MEXT_DEFAULT ..... 0xffffffff IC_SMBUS_CLK_LOW_SEXT_DEFAULT ..... 0xffffffff IC_HS_MASTER_CODE ................. 0x1 IC_SMBUS_RST_IDLE_CNT_DEFAULT ..... 0xffff IC_SMBUS_UDID_LSB_DEFAULT ......... 0xffffffff IC_SS_SCL_HIGH_COUNT .............. 0x0028 IC_SS_SCL_LOW_COUNT ............... 0x002f IC_MAX_SPEED_MODE ................. 0x2 IC_STAT_FOR_CLK_STRETCH ........... 0x0 IC_STOP_DET_IF_MASTER_ACTIVE ...... 0x0 IC_DEFAULT_UFM_SPKLEN ............. 0x1 IC_TX_BUFFER_DEPTH ................ 16
	IRQ_I2C0_IRQ = 36

	IRQ_I2C1_IRQ = 37

	// SNPS OTP control IF (SBPI and RPi wrapper control)
	IRQ_OTP_IRQ = 38

	// ARM TrustZone RNG register block
	IRQ_TRNG_IRQ = 39

	IRQ_PLL_SYS_IRQ = 42

	IRQ_PLL_USB_IRQ = 43

	// Controls vreg, bor, lposc, chip resets & xosc startup, powman and provides scratch register for general use and for bootcode use
	IRQ_POWMAN_IRQ_POW = 44

	// Controls vreg, bor, lposc, chip resets & xosc startup, powman and provides scratch register for general use and for bootcode use
	IRQ_POWMAN_IRQ_TIMER = 45

	// Highest interrupt number on this device.
	IRQ_max = 45
)

// Pseudo function call that is replaced by the compiler with the actual
// functions registered through interrupt.New.
//
//go:linkname callHandlers runtime/interrupt.callHandlers
func callHandlers(num int)

//export TIMER0_IRQ_0_IRQHandler
func interruptTIMER0_IRQ_0() {
	callHandlers(IRQ_TIMER0_IRQ_0)
}

//export TIMER0_IRQ_1_IRQHandler
func interruptTIMER0_IRQ_1() {
	callHandlers(IRQ_TIMER0_IRQ_1)
}

//export TIMER0_IRQ_2_IRQHandler
func interruptTIMER0_IRQ_2() {
	callHandlers(IRQ_TIMER0_IRQ_2)
}

//export TIMER0_IRQ_3_IRQHandler
func interruptTIMER0_IRQ_3() {
	callHandlers(IRQ_TIMER0_IRQ_3)
}

//export TIMER1_IRQ_0_IRQHandler
func interruptTIMER1_IRQ_0() {
	callHandlers(IRQ_TIMER1_IRQ_0)
}

//export TIMER1_IRQ_1_IRQHandler
func interruptTIMER1_IRQ_1() {
	callHandlers(IRQ_TIMER1_IRQ_1)
}

//export TIMER1_IRQ_2_IRQHandler
func interruptTIMER1_IRQ_2() {
	callHandlers(IRQ_TIMER1_IRQ_2)
}

//export TIMER1_IRQ_3_IRQHandler
func interruptTIMER1_IRQ_3() {
	callHandlers(IRQ_TIMER1_IRQ_3)
}

//export PWM_IRQ_WRAP_0_IRQHandler
func interruptPWM_IRQ_WRAP_0() {
	callHandlers(IRQ_PWM_IRQ_WRAP_0)
}

//export PWM_IRQ_WRAP_1_IRQHandler
func interruptPWM_IRQ_WRAP_1() {
	callHandlers(IRQ_PWM_IRQ_WRAP_1)
}

//export DMA_IRQ_0_IRQHandler
func interruptDMA_IRQ_0() {
	callHandlers(IRQ_DMA_IRQ_0)
}

//export DMA_IRQ_1_IRQHandler
func interruptDMA_IRQ_1() {
	callHandlers(IRQ_DMA_IRQ_1)
}

//export DMA_IRQ_2_IRQHandler
func interruptDMA_IRQ_2() {
	callHandlers(IRQ_DMA_IRQ_2)
}

//export DMA_IRQ_3_IRQHandler
func interruptDMA_IRQ_3() {
	callHandlers(IRQ_DMA_IRQ_3)
}

//export USBCTRL_IRQ_IRQHandler
func interruptUSBCTRL_IRQ() {
	callHandlers(IRQ_USBCTRL_IRQ)
}

//export PIO0_IRQ_0_IRQHandler
func interruptPIO0_IRQ_0() {
	callHandlers(IRQ_PIO0_IRQ_0)
}

//export PIO0_IRQ_1_IRQHandler
func interruptPIO0_IRQ_1() {
	callHandlers(IRQ_PIO0_IRQ_1)
}

//export PIO1_IRQ_0_IRQHandler
func interruptPIO1_IRQ_0() {
	callHandlers(IRQ_PIO1_IRQ_0)
}

//export PIO1_IRQ_1_IRQHandler
func interruptPIO1_IRQ_1() {
	callHandlers(IRQ_PIO1_IRQ_1)
}

//export PIO2_IRQ_0_IRQHandler
func interruptPIO2_IRQ_0() {
	callHandlers(IRQ_PIO2_IRQ_0)
}

//export PIO2_IRQ_1_IRQHandler
func interruptPIO2_IRQ_1() {
	callHandlers(IRQ_PIO2_IRQ_1)
}

//export IO_IRQ_BANK0_IRQHandler
func interruptIO_IRQ_BANK0() {
	callHandlers(IRQ_IO_IRQ_BANK0)
}

//export IO_IRQ_BANK0_NS_IRQHandler
func interruptIO_IRQ_BANK0_NS() {
	callHandlers(IRQ_IO_IRQ_BANK0_NS)
}

//export IO_IRQ_QSPI_IRQHandler
func interruptIO_IRQ_QSPI() {
	callHandlers(IRQ_IO_IRQ_QSPI)
}

//export IO_IRQ_QSPI_NS_IRQHandler
func interruptIO_IRQ_QSPI_NS() {
	callHandlers(IRQ_IO_IRQ_QSPI_NS)
}

//export SIO_IRQ_FIFO_IRQHandler
func interruptSIO_IRQ_FIFO() {
	callHandlers(IRQ_SIO_IRQ_FIFO)
}

//export SIO_IRQ_BELL_IRQHandler
func interruptSIO_IRQ_BELL() {
	callHandlers(IRQ_SIO_IRQ_BELL)
}

//export SIO_IRQ_FIFO_NS_IRQHandler
func interruptSIO_IRQ_FIFO_NS() {
	callHandlers(IRQ_SIO_IRQ_FIFO_NS)
}

//export SIO_IRQ_BELL_NS_IRQHandler
func interruptSIO_IRQ_BELL_NS() {
	callHandlers(IRQ_SIO_IRQ_BELL_NS)
}

//export SIO_IRQ_MTIMECMP_IRQHandler
func interruptSIO_IRQ_MTIMECMP() {
	callHandlers(IRQ_SIO_IRQ_MTIMECMP)
}

//export CLOCKS_IRQ_IRQHandler
func interruptCLOCKS_IRQ() {
	callHandlers(IRQ_CLOCKS_IRQ)
}

//export SPI0_IRQ_IRQHandler
func interruptSPI0_IRQ() {
	callHandlers(IRQ_SPI0_IRQ)
}

//export SPI1_IRQ_IRQHandler
func interruptSPI1_IRQ() {
	callHandlers(IRQ_SPI1_IRQ)
}

//export UART0_IRQ_IRQHandler
func interruptUART0_IRQ() {
	callHandlers(IRQ_UART0_IRQ)
}

//export UART1_IRQ_IRQHandler
func interruptUART1_IRQ() {
	callHandlers(IRQ_UART1_IRQ)
}

//export ADC_IRQ_FIFO_IRQHandler
func interruptADC_IRQ_FIFO() {
	callHandlers(IRQ_ADC_IRQ_FIFO)
}

//export I2C0_IRQ_IRQHandler
func interruptI2C0_IRQ() {
	callHandlers(IRQ_I2C0_IRQ)
}

//export I2C1_IRQ_IRQHandler
func interruptI2C1_IRQ() {
	callHandlers(IRQ_I2C1_IRQ)
}

//export OTP_IRQ_IRQHandler
func interruptOTP_IRQ() {
	callHandlers(IRQ_OTP_IRQ)
}

//export TRNG_IRQ_IRQHandler
func interruptTRNG_IRQ() {
	callHandlers(IRQ_TRNG_IRQ)
}

//export PLL_SYS_IRQ_IRQHandler
func interruptPLL_SYS_IRQ() {
	callHandlers(IRQ_PLL_SYS_IRQ)
}

//export PLL_USB_IRQ_IRQHandler
func interruptPLL_USB_IRQ() {
	callHandlers(IRQ_PLL_USB_IRQ)
}

//export POWMAN_IRQ_POW_IRQHandler
func interruptPOWMAN_IRQ_POW() {
	callHandlers(IRQ_POWMAN_IRQ_POW)
}

//export POWMAN_IRQ_TIMER_IRQHandler
func interruptPOWMAN_IRQ_TIMER() {
	callHandlers(IRQ_POWMAN_IRQ_TIMER)
}

// Peripherals.
var (
	RESETS = (*RESETS_Type)(unsafe.Pointer(uintptr(0x40020000)))

	PSM = (*PSM_Type)(unsafe.Pointer(uintptr(0x40018000)))

	CLOCKS = (*CLOCKS_Type)(unsafe.Pointer(uintptr(0x40010000)))

	TICKS = (*TICKS_Type)(unsafe.Pointer(uintptr(0x40108000)))

	PADS_BANK0 = (*PADS_BANK0_Type)(unsafe.Pointer(uintptr(0x40038000)))

	PADS_QSPI = (*PADS_QSPI_Type)(unsafe.Pointer(uintptr(0x40040000)))

	IO_QSPI = (*IO_QSPI_Type)(unsafe.Pointer(uintptr(0x40030000)))

	IO_BANK0 = (*IO_BANK0_Type)(unsafe.Pointer(uintptr(0x40028000)))

	SYSINFO = (*SYSINFO_Type)(unsafe.Pointer(uintptr(0x40000000)))

	// SHA-256 hash function implementation
	SHA256 = (*SHA256_Type)(unsafe.Pointer(uintptr(0x400f8000)))

	// FIFO status and write access for HSTX
	HSTX_FIFO = (*HSTX_FIFO_Type)(unsafe.Pointer(uintptr(0x50600000)))

	// Control interface to HSTX. For FIFO write access and status, see the HSTX_FIFO register block.
	HSTX_CTRL = (*HSTX_CTRL_Type)(unsafe.Pointer(uintptr(0x400c0000)))

	// Cortex-M33 EPPB vendor register block for RP2350
	EPPB = (*EPPB_Type)(unsafe.Pointer(uintptr(0xe0080000)))

	// TEAL registers accessible through the debug interface
	PPB = (*PPB_Type)(unsafe.Pointer(uintptr(0xe0000000)))

	// TEAL registers accessible through the debug interface
	PPB_NS = (*PPB_Type)(unsafe.Pointer(uintptr(0xe0020000)))

	// QSPI Memory Interface. Provides a memory-mapped interface to up to two SPI/DSPI/QSPI flash or PSRAM devices. Also provides a serial interface for programming and configuration of the external device.
	QMI = (*QMI_Type)(unsafe.Pointer(uintptr(0x400d0000)))

	// QSPI flash execute-in-place block
	XIP_CTRL = (*XIP_CTRL_Type)(unsafe.Pointer(uintptr(0x400c8000)))

	// Auxiliary DMA access to XIP FIFOs, via fast AHB bus access
	XIP_AUX = (*XIP_AUX_Type)(unsafe.Pointer(uintptr(0x50500000)))

	// Register block for various chip control signals
	SYSCFG = (*SYSCFG_Type)(unsafe.Pointer(uintptr(0x40008000)))

	// Controls the crystal oscillator
	XOSC = (*XOSC_Type)(unsafe.Pointer(uintptr(0x40048000)))

	PLL_SYS = (*PLL_SYS_Type)(unsafe.Pointer(uintptr(0x40050000)))

	PLL_USB = (*PLL_SYS_Type)(unsafe.Pointer(uintptr(0x40058000)))

	// Hardware access control registers
	ACCESSCTRL = (*ACCESSCTRL_Type)(unsafe.Pointer(uintptr(0x40060000)))

	UART0 = (*UART0_Type)(unsafe.Pointer(uintptr(0x40070000)))

	UART1 = (*UART0_Type)(unsafe.Pointer(uintptr(0x40078000)))

	ROSC = (*ROSC_Type)(unsafe.Pointer(uintptr(0x400e8000)))

	// Controls vreg, bor, lposc, chip resets & xosc startup, powman and provides scratch register for general use and for bootcode use
	POWMAN = (*POWMAN_Type)(unsafe.Pointer(uintptr(0x40100000)))

	WATCHDOG = (*WATCHDOG_Type)(unsafe.Pointer(uintptr(0x400d8000)))

	// DMA with separate read and write masters
	DMA = (*DMA_Type)(unsafe.Pointer(uintptr(0x50000000)))

	// Controls time and alarms time is a 64 bit value indicating the time since power-on timeh is the top 32 bits of time & timel is the bottom 32 bits to change time write to timelw before timehw to read time read from timelr before timehr An alarm is set by setting alarm_enable and writing to the corresponding alarm register When an alarm is pending, the corresponding alarm_running signal will be high An alarm can be cancelled before it has finished by clearing the alarm_enable When an alarm fires, the corresponding alarm_irq is set and alarm_running is cleared To clear the interrupt write a 1 to the corresponding alarm_irq The timer can be locked to prevent writing
	TIMER0 = (*TIMER0_Type)(unsafe.Pointer(uintptr(0x400b0000)))

	// Controls time and alarms time is a 64 bit value indicating the time since power-on timeh is the top 32 bits of time & timel is the bottom 32 bits to change time write to timelw before timehw to read time read from timelr before timehr An alarm is set by setting alarm_enable and writing to the corresponding alarm register When an alarm is pending, the corresponding alarm_running signal will be high An alarm can be cancelled before it has finished by clearing the alarm_enable When an alarm fires, the corresponding alarm_irq is set and alarm_running is cleared To clear the interrupt write a 1 to the corresponding alarm_irq The timer can be locked to prevent writing
	TIMER1 = (*TIMER0_Type)(unsafe.Pointer(uintptr(0x400b8000)))

	// Simple PWM
	PWM = (*PWM_Type)(unsafe.Pointer(uintptr(0x400a8000)))

	// Control and data interface to SAR ADC
	ADC = (*ADC_Type)(unsafe.Pointer(uintptr(0x400a0000)))

	// DW_apb_i2c address block List of configuration constants for the Synopsys I2C hardware (you may see references to these in I2C register header; these are *fixed* values, set at hardware design time): IC_ULTRA_FAST_MODE ................ 0x0 IC_UFM_TBUF_CNT_DEFAULT ........... 0x8 IC_UFM_SCL_LOW_COUNT .............. 0x0008 IC_UFM_SCL_HIGH_COUNT ............. 0x0006 IC_TX_TL .......................... 0x0 IC_TX_CMD_BLOCK ................... 0x1 IC_HAS_DMA ........................ 0x1 IC_HAS_ASYNC_FIFO ................. 0x0 IC_SMBUS_ARP ...................... 0x0 IC_FIRST_DATA_BYTE_STATUS ......... 0x1 IC_INTR_IO ........................ 0x1 IC_MASTER_MODE .................... 0x1 IC_DEFAULT_ACK_GENERAL_CALL ....... 0x1 IC_INTR_POL ....................... 0x1 IC_OPTIONAL_SAR ................... 0x0 IC_DEFAULT_TAR_SLAVE_ADDR ......... 0x055 IC_DEFAULT_SLAVE_ADDR ............. 0x055 IC_DEFAULT_HS_SPKLEN .............. 0x1 IC_FS_SCL_HIGH_COUNT .............. 0x0006 IC_HS_SCL_LOW_COUNT ............... 0x0008 IC_DEVICE_ID_VALUE ................ 0x0 IC_10BITADDR_MASTER ............... 0x0 IC_CLK_FREQ_OPTIMIZATION .......... 0x0 IC_DEFAULT_FS_SPKLEN .............. 0x7 IC_ADD_ENCODED_PARAMS ............. 0x0 IC_DEFAULT_SDA_HOLD ............... 0x000001 IC_DEFAULT_SDA_SETUP .............. 0x64 IC_AVOID_RX_FIFO_FLUSH_ON_TX_ABRT . 0x0 IC_CLOCK_PERIOD ................... 100 IC_EMPTYFIFO_HOLD_MASTER_EN ....... 1 IC_RESTART_EN ..................... 0x1 IC_TX_CMD_BLOCK_DEFAULT ........... 0x0 IC_BUS_CLEAR_FEATURE .............. 0x0 IC_CAP_LOADING .................... 100 IC_FS_SCL_LOW_COUNT ............... 0x000d APB_DATA_WIDTH .................... 32 IC_SDA_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_SLV_DATA_NACK_ONLY ............. 0x1 IC_10BITADDR_SLAVE ................ 0x0 IC_CLK_TYPE ....................... 0x0 IC_SMBUS_UDID_MSB ................. 0x0 IC_SMBUS_SUSPEND_ALERT ............ 0x0 IC_HS_SCL_HIGH_COUNT .............. 0x0006 IC_SLV_RESTART_DET_EN ............. 0x1 IC_SMBUS .......................... 0x0 IC_OPTIONAL_SAR_DEFAULT ........... 0x0 IC_PERSISTANT_SLV_ADDR_DEFAULT .... 0x0 IC_USE_COUNTS ..................... 0x0 IC_RX_BUFFER_DEPTH ................ 16 IC_SCL_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_RX_FULL_HLD_BUS_EN ............. 0x1 IC_SLAVE_DISABLE .................. 0x1 IC_RX_TL .......................... 0x0 IC_DEVICE_ID ...................... 0x0 IC_HC_COUNT_VALUES ................ 0x0 I2C_DYNAMIC_TAR_UPDATE ............ 0 IC_SMBUS_CLK_LOW_MEXT_DEFAULT ..... 0xffffffff IC_SMBUS_CLK_LOW_SEXT_DEFAULT ..... 0xffffffff IC_HS_MASTER_CODE ................. 0x1 IC_SMBUS_RST_IDLE_CNT_DEFAULT ..... 0xffff IC_SMBUS_UDID_LSB_DEFAULT ......... 0xffffffff IC_SS_SCL_HIGH_COUNT .............. 0x0028 IC_SS_SCL_LOW_COUNT ............... 0x002f IC_MAX_SPEED_MODE ................. 0x2 IC_STAT_FOR_CLK_STRETCH ........... 0x0 IC_STOP_DET_IF_MASTER_ACTIVE ...... 0x0 IC_DEFAULT_UFM_SPKLEN ............. 0x1 IC_TX_BUFFER_DEPTH ................ 16
	I2C0 = (*I2C0_Type)(unsafe.Pointer(uintptr(0x40090000)))

	// DW_apb_i2c address block List of configuration constants for the Synopsys I2C hardware (you may see references to these in I2C register header; these are *fixed* values, set at hardware design time): IC_ULTRA_FAST_MODE ................ 0x0 IC_UFM_TBUF_CNT_DEFAULT ........... 0x8 IC_UFM_SCL_LOW_COUNT .............. 0x0008 IC_UFM_SCL_HIGH_COUNT ............. 0x0006 IC_TX_TL .......................... 0x0 IC_TX_CMD_BLOCK ................... 0x1 IC_HAS_DMA ........................ 0x1 IC_HAS_ASYNC_FIFO ................. 0x0 IC_SMBUS_ARP ...................... 0x0 IC_FIRST_DATA_BYTE_STATUS ......... 0x1 IC_INTR_IO ........................ 0x1 IC_MASTER_MODE .................... 0x1 IC_DEFAULT_ACK_GENERAL_CALL ....... 0x1 IC_INTR_POL ....................... 0x1 IC_OPTIONAL_SAR ................... 0x0 IC_DEFAULT_TAR_SLAVE_ADDR ......... 0x055 IC_DEFAULT_SLAVE_ADDR ............. 0x055 IC_DEFAULT_HS_SPKLEN .............. 0x1 IC_FS_SCL_HIGH_COUNT .............. 0x0006 IC_HS_SCL_LOW_COUNT ............... 0x0008 IC_DEVICE_ID_VALUE ................ 0x0 IC_10BITADDR_MASTER ............... 0x0 IC_CLK_FREQ_OPTIMIZATION .......... 0x0 IC_DEFAULT_FS_SPKLEN .............. 0x7 IC_ADD_ENCODED_PARAMS ............. 0x0 IC_DEFAULT_SDA_HOLD ............... 0x000001 IC_DEFAULT_SDA_SETUP .............. 0x64 IC_AVOID_RX_FIFO_FLUSH_ON_TX_ABRT . 0x0 IC_CLOCK_PERIOD ................... 100 IC_EMPTYFIFO_HOLD_MASTER_EN ....... 1 IC_RESTART_EN ..................... 0x1 IC_TX_CMD_BLOCK_DEFAULT ........... 0x0 IC_BUS_CLEAR_FEATURE .............. 0x0 IC_CAP_LOADING .................... 100 IC_FS_SCL_LOW_COUNT ............... 0x000d APB_DATA_WIDTH .................... 32 IC_SDA_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_SLV_DATA_NACK_ONLY ............. 0x1 IC_10BITADDR_SLAVE ................ 0x0 IC_CLK_TYPE ....................... 0x0 IC_SMBUS_UDID_MSB ................. 0x0 IC_SMBUS_SUSPEND_ALERT ............ 0x0 IC_HS_SCL_HIGH_COUNT .............. 0x0006 IC_SLV_RESTART_DET_EN ............. 0x1 IC_SMBUS .......................... 0x0 IC_OPTIONAL_SAR_DEFAULT ........... 0x0 IC_PERSISTANT_SLV_ADDR_DEFAULT .... 0x0 IC_USE_COUNTS ..................... 0x0 IC_RX_BUFFER_DEPTH ................ 16 IC_SCL_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_RX_FULL_HLD_BUS_EN ............. 0x1 IC_SLAVE_DISABLE .................. 0x1 IC_RX_TL .......................... 0x0 IC_DEVICE_ID ...................... 0x0 IC_HC_COUNT_VALUES ................ 0x0 I2C_DYNAMIC_TAR_UPDATE ............ 0 IC_SMBUS_CLK_LOW_MEXT_DEFAULT ..... 0xffffffff IC_SMBUS_CLK_LOW_SEXT_DEFAULT ..... 0xffffffff IC_HS_MASTER_CODE ................. 0x1 IC_SMBUS_RST_IDLE_CNT_DEFAULT ..... 0xffff IC_SMBUS_UDID_LSB_DEFAULT ......... 0xffffffff IC_SS_SCL_HIGH_COUNT .............. 0x0028 IC_SS_SCL_LOW_COUNT ............... 0x002f IC_MAX_SPEED_MODE ................. 0x2 IC_STAT_FOR_CLK_STRETCH ........... 0x0 IC_STOP_DET_IF_MASTER_ACTIVE ...... 0x0 IC_DEFAULT_UFM_SPKLEN ............. 0x1 IC_TX_BUFFER_DEPTH ................ 16
	I2C1 = (*I2C0_Type)(unsafe.Pointer(uintptr(0x40098000)))

	SPI0 = (*SPI0_Type)(unsafe.Pointer(uintptr(0x40080000)))

	SPI1 = (*SPI0_Type)(unsafe.Pointer(uintptr(0x40088000)))

	// Programmable IO block
	PIO0 = (*PIO0_Type)(unsafe.Pointer(uintptr(0x50200000)))

	// Programmable IO block
	PIO1 = (*PIO0_Type)(unsafe.Pointer(uintptr(0x50300000)))

	// Programmable IO block
	PIO2 = (*PIO0_Type)(unsafe.Pointer(uintptr(0x50400000)))

	// Register block for busfabric control signals and performance counters
	BUSCTRL = (*BUSCTRL_Type)(unsafe.Pointer(uintptr(0x40068000)))

	// Single-cycle IO block Provides core-local and inter-core hardware for the two processors, with single-cycle access.
	SIO = (*SIO_Type)(unsafe.Pointer(uintptr(0xd0000000)))

	// Single-cycle IO block Provides core-local and inter-core hardware for the two processors, with single-cycle access.
	SIO_NS = (*SIO_Type)(unsafe.Pointer(uintptr(0xd0020000)))

	// Additional registers mapped adjacent to the bootram, for use by the bootrom.
	BOOTRAM = (*BOOTRAM_Type)(unsafe.Pointer(uintptr(0x400e0000)))

	// Coresight block - RP specific registers
	CORESIGHT_TRACE = (*CORESIGHT_TRACE_Type)(unsafe.Pointer(uintptr(0x50700000)))

	// USB FS/LS controller device registers
	USB = (*USB_Type)(unsafe.Pointer(uintptr(0x50110000)))

	// ARM TrustZone RNG register block
	TRNG = (*TRNG_Type)(unsafe.Pointer(uintptr(0x400f0000)))

	// Glitch detector controls
	GLITCH_DETECTOR = (*GLITCH_DETECTOR_Type)(unsafe.Pointer(uintptr(0x40158000)))

	// SNPS OTP control IF (SBPI and RPi wrapper control)
	OTP = (*OTP_Type)(unsafe.Pointer(uintptr(0x40120000)))

	// Predefined OTP data layout for RP2350
	OTP_DATA = (*OTP_DATA_Type)(unsafe.Pointer(uintptr(0x40130000)))

	// Predefined OTP data layout for RP2350
	OTP_DATA_RAW = (*OTP_DATA_RAW_Type)(unsafe.Pointer(uintptr(0x40134000)))

	// For managing simulation testbenches
	TBMAN = (*TBMAN_Type)(unsafe.Pointer(uintptr(0x40160000)))

	// DPRAM layout for USB device.
	USB_DPRAM = (*USB_DPRAM_Type)(unsafe.Pointer(uintptr(0x50100000)))
)

type RESETS_Type struct {
	RESET      volatile.Register32 // 0x0
	WDSEL      volatile.Register32 // 0x4
	RESET_DONE volatile.Register32 // 0x8
}

// RESETS.RESET
func (o *RESETS_Type) SetRESET_USBCTRL(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x10000000)|value<<28)
}
func (o *RESETS_Type) GetRESET_USBCTRL() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x10000000) >> 28
}
func (o *RESETS_Type) SetRESET_UART1(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x8000000)|value<<27)
}
func (o *RESETS_Type) GetRESET_UART1() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x8000000) >> 27
}
func (o *RESETS_Type) SetRESET_UART0(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x4000000)|value<<26)
}
func (o *RESETS_Type) GetRESET_UART0() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x4000000) >> 26
}
func (o *RESETS_Type) SetRESET_TRNG(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x2000000)|value<<25)
}
func (o *RESETS_Type) GetRESET_TRNG() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x2000000) >> 25
}
func (o *RESETS_Type) SetRESET_TIMER1(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x1000000)|value<<24)
}
func (o *RESETS_Type) GetRESET_TIMER1() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x1000000) >> 24
}
func (o *RESETS_Type) SetRESET_TIMER0(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x800000)|value<<23)
}
func (o *RESETS_Type) GetRESET_TIMER0() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x800000) >> 23
}
func (o *RESETS_Type) SetRESET_TBMAN(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x400000)|value<<22)
}
func (o *RESETS_Type) GetRESET_TBMAN() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x400000) >> 22
}
func (o *RESETS_Type) SetRESET_SYSINFO(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x200000)|value<<21)
}
func (o *RESETS_Type) GetRESET_SYSINFO() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x200000) >> 21
}
func (o *RESETS_Type) SetRESET_SYSCFG(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x100000)|value<<20)
}
func (o *RESETS_Type) GetRESET_SYSCFG() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x100000) >> 20
}
func (o *RESETS_Type) SetRESET_SPI1(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x80000)|value<<19)
}
func (o *RESETS_Type) GetRESET_SPI1() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x80000) >> 19
}
func (o *RESETS_Type) SetRESET_SPI0(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x40000)|value<<18)
}
func (o *RESETS_Type) GetRESET_SPI0() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x40000) >> 18
}
func (o *RESETS_Type) SetRESET_SHA256(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x20000)|value<<17)
}
func (o *RESETS_Type) GetRESET_SHA256() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x20000) >> 17
}
func (o *RESETS_Type) SetRESET_PWM(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x10000)|value<<16)
}
func (o *RESETS_Type) GetRESET_PWM() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x10000) >> 16
}
func (o *RESETS_Type) SetRESET_PLL_USB(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x8000)|value<<15)
}
func (o *RESETS_Type) GetRESET_PLL_USB() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x8000) >> 15
}
func (o *RESETS_Type) SetRESET_PLL_SYS(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x4000)|value<<14)
}
func (o *RESETS_Type) GetRESET_PLL_SYS() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x4000) >> 14
}
func (o *RESETS_Type) SetRESET_PIO2(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x2000)|value<<13)
}
func (o *RESETS_Type) GetRESET_PIO2() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x2000) >> 13
}
func (o *RESETS_Type) SetRESET_PIO1(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x1000)|value<<12)
}
func (o *RESETS_Type) GetRESET_PIO1() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x1000) >> 12
}
func (o *RESETS_Type) SetRESET_PIO0(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x800)|value<<11)
}
func (o *RESETS_Type) GetRESET_PIO0() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x800) >> 11
}
func (o *RESETS_Type) SetRESET_PADS_QSPI(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x400)|value<<10)
}
func (o *RESETS_Type) GetRESET_PADS_QSPI() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x400) >> 10
}
func (o *RESETS_Type) SetRESET_PADS_BANK0(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x200)|value<<9)
}
func (o *RESETS_Type) GetRESET_PADS_BANK0() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x200) >> 9
}
func (o *RESETS_Type) SetRESET_JTAG(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x100)|value<<8)
}
func (o *RESETS_Type) GetRESET_JTAG() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x100) >> 8
}
func (o *RESETS_Type) SetRESET_IO_QSPI(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x80)|value<<7)
}
func (o *RESETS_Type) GetRESET_IO_QSPI() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x80) >> 7
}
func (o *RESETS_Type) SetRESET_IO_BANK0(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x40)|value<<6)
}
func (o *RESETS_Type) GetRESET_IO_BANK0() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x40) >> 6
}
func (o *RESETS_Type) SetRESET_I2C1(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x20)|value<<5)
}
func (o *RESETS_Type) GetRESET_I2C1() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x20) >> 5
}
func (o *RESETS_Type) SetRESET_I2C0(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x10)|value<<4)
}
func (o *RESETS_Type) GetRESET_I2C0() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x10) >> 4
}
func (o *RESETS_Type) SetRESET_HSTX(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x8)|value<<3)
}
func (o *RESETS_Type) GetRESET_HSTX() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x8) >> 3
}
func (o *RESETS_Type) SetRESET_DMA(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x4)|value<<2)
}
func (o *RESETS_Type) GetRESET_DMA() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x4) >> 2
}
func (o *RESETS_Type) SetRESET_BUSCTRL(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x2)|value<<1)
}
func (o *RESETS_Type) GetRESET_BUSCTRL() uint32 {
	return (volatile.LoadUint32(&o.RESET.Reg) & 0x2) >> 1
}
func (o *RESETS_Type) SetRESET_ADC(value uint32) {
	volatile.StoreUint32(&o.RESET.Reg, volatile.LoadUint32(&o.RESET.Reg)&^(0x1)|value)
}
func (o *RESETS_Type) GetRESET_ADC() uint32 {
	return volatile.LoadUint32(&o.RESET.Reg) & 0x1
}

// RESETS.WDSEL
func (o *RESETS_Type) SetWDSEL_USBCTRL(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x10000000)|value<<28)
}
func (o *RESETS_Type) GetWDSEL_USBCTRL() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x10000000) >> 28
}
func (o *RESETS_Type) SetWDSEL_UART1(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x8000000)|value<<27)
}
func (o *RESETS_Type) GetWDSEL_UART1() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x8000000) >> 27
}
func (o *RESETS_Type) SetWDSEL_UART0(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x4000000)|value<<26)
}
func (o *RESETS_Type) GetWDSEL_UART0() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x4000000) >> 26
}
func (o *RESETS_Type) SetWDSEL_TRNG(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x2000000)|value<<25)
}
func (o *RESETS_Type) GetWDSEL_TRNG() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x2000000) >> 25
}
func (o *RESETS_Type) SetWDSEL_TIMER1(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x1000000)|value<<24)
}
func (o *RESETS_Type) GetWDSEL_TIMER1() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x1000000) >> 24
}
func (o *RESETS_Type) SetWDSEL_TIMER0(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x800000)|value<<23)
}
func (o *RESETS_Type) GetWDSEL_TIMER0() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x800000) >> 23
}
func (o *RESETS_Type) SetWDSEL_TBMAN(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x400000)|value<<22)
}
func (o *RESETS_Type) GetWDSEL_TBMAN() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x400000) >> 22
}
func (o *RESETS_Type) SetWDSEL_SYSINFO(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x200000)|value<<21)
}
func (o *RESETS_Type) GetWDSEL_SYSINFO() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x200000) >> 21
}
func (o *RESETS_Type) SetWDSEL_SYSCFG(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x100000)|value<<20)
}
func (o *RESETS_Type) GetWDSEL_SYSCFG() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x100000) >> 20
}
func (o *RESETS_Type) SetWDSEL_SPI1(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x80000)|value<<19)
}
func (o *RESETS_Type) GetWDSEL_SPI1() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x80000) >> 19
}
func (o *RESETS_Type) SetWDSEL_SPI0(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x40000)|value<<18)
}
func (o *RESETS_Type) GetWDSEL_SPI0() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x40000) >> 18
}
func (o *RESETS_Type) SetWDSEL_SHA256(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x20000)|value<<17)
}
func (o *RESETS_Type) GetWDSEL_SHA256() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x20000) >> 17
}
func (o *RESETS_Type) SetWDSEL_PWM(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x10000)|value<<16)
}
func (o *RESETS_Type) GetWDSEL_PWM() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x10000) >> 16
}
func (o *RESETS_Type) SetWDSEL_PLL_USB(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x8000)|value<<15)
}
func (o *RESETS_Type) GetWDSEL_PLL_USB() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x8000) >> 15
}
func (o *RESETS_Type) SetWDSEL_PLL_SYS(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x4000)|value<<14)
}
func (o *RESETS_Type) GetWDSEL_PLL_SYS() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x4000) >> 14
}
func (o *RESETS_Type) SetWDSEL_PIO2(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x2000)|value<<13)
}
func (o *RESETS_Type) GetWDSEL_PIO2() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x2000) >> 13
}
func (o *RESETS_Type) SetWDSEL_PIO1(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x1000)|value<<12)
}
func (o *RESETS_Type) GetWDSEL_PIO1() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x1000) >> 12
}
func (o *RESETS_Type) SetWDSEL_PIO0(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x800)|value<<11)
}
func (o *RESETS_Type) GetWDSEL_PIO0() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x800) >> 11
}
func (o *RESETS_Type) SetWDSEL_PADS_QSPI(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x400)|value<<10)
}
func (o *RESETS_Type) GetWDSEL_PADS_QSPI() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x400) >> 10
}
func (o *RESETS_Type) SetWDSEL_PADS_BANK0(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x200)|value<<9)
}
func (o *RESETS_Type) GetWDSEL_PADS_BANK0() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x200) >> 9
}
func (o *RESETS_Type) SetWDSEL_JTAG(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x100)|value<<8)
}
func (o *RESETS_Type) GetWDSEL_JTAG() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x100) >> 8
}
func (o *RESETS_Type) SetWDSEL_IO_QSPI(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x80)|value<<7)
}
func (o *RESETS_Type) GetWDSEL_IO_QSPI() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x80) >> 7
}
func (o *RESETS_Type) SetWDSEL_IO_BANK0(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x40)|value<<6)
}
func (o *RESETS_Type) GetWDSEL_IO_BANK0() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x40) >> 6
}
func (o *RESETS_Type) SetWDSEL_I2C1(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x20)|value<<5)
}
func (o *RESETS_Type) GetWDSEL_I2C1() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x20) >> 5
}
func (o *RESETS_Type) SetWDSEL_I2C0(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x10)|value<<4)
}
func (o *RESETS_Type) GetWDSEL_I2C0() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x10) >> 4
}
func (o *RESETS_Type) SetWDSEL_HSTX(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x8)|value<<3)
}
func (o *RESETS_Type) GetWDSEL_HSTX() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x8) >> 3
}
func (o *RESETS_Type) SetWDSEL_DMA(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x4)|value<<2)
}
func (o *RESETS_Type) GetWDSEL_DMA() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x4) >> 2
}
func (o *RESETS_Type) SetWDSEL_BUSCTRL(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x2)|value<<1)
}
func (o *RESETS_Type) GetWDSEL_BUSCTRL() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x2) >> 1
}
func (o *RESETS_Type) SetWDSEL_ADC(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x1)|value)
}
func (o *RESETS_Type) GetWDSEL_ADC() uint32 {
	return volatile.LoadUint32(&o.WDSEL.Reg) & 0x1
}

// RESETS.RESET_DONE
func (o *RESETS_Type) SetRESET_DONE_USBCTRL(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x10000000)|value<<28)
}
func (o *RESETS_Type) GetRESET_DONE_USBCTRL() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x10000000) >> 28
}
func (o *RESETS_Type) SetRESET_DONE_UART1(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x8000000)|value<<27)
}
func (o *RESETS_Type) GetRESET_DONE_UART1() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x8000000) >> 27
}
func (o *RESETS_Type) SetRESET_DONE_UART0(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x4000000)|value<<26)
}
func (o *RESETS_Type) GetRESET_DONE_UART0() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x4000000) >> 26
}
func (o *RESETS_Type) SetRESET_DONE_TRNG(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x2000000)|value<<25)
}
func (o *RESETS_Type) GetRESET_DONE_TRNG() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x2000000) >> 25
}
func (o *RESETS_Type) SetRESET_DONE_TIMER1(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x1000000)|value<<24)
}
func (o *RESETS_Type) GetRESET_DONE_TIMER1() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x1000000) >> 24
}
func (o *RESETS_Type) SetRESET_DONE_TIMER0(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x800000)|value<<23)
}
func (o *RESETS_Type) GetRESET_DONE_TIMER0() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x800000) >> 23
}
func (o *RESETS_Type) SetRESET_DONE_TBMAN(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x400000)|value<<22)
}
func (o *RESETS_Type) GetRESET_DONE_TBMAN() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x400000) >> 22
}
func (o *RESETS_Type) SetRESET_DONE_SYSINFO(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x200000)|value<<21)
}
func (o *RESETS_Type) GetRESET_DONE_SYSINFO() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x200000) >> 21
}
func (o *RESETS_Type) SetRESET_DONE_SYSCFG(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x100000)|value<<20)
}
func (o *RESETS_Type) GetRESET_DONE_SYSCFG() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x100000) >> 20
}
func (o *RESETS_Type) SetRESET_DONE_SPI1(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x80000)|value<<19)
}
func (o *RESETS_Type) GetRESET_DONE_SPI1() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x80000) >> 19
}
func (o *RESETS_Type) SetRESET_DONE_SPI0(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x40000)|value<<18)
}
func (o *RESETS_Type) GetRESET_DONE_SPI0() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x40000) >> 18
}
func (o *RESETS_Type) SetRESET_DONE_SHA256(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x20000)|value<<17)
}
func (o *RESETS_Type) GetRESET_DONE_SHA256() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x20000) >> 17
}
func (o *RESETS_Type) SetRESET_DONE_PWM(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x10000)|value<<16)
}
func (o *RESETS_Type) GetRESET_DONE_PWM() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x10000) >> 16
}
func (o *RESETS_Type) SetRESET_DONE_PLL_USB(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x8000)|value<<15)
}
func (o *RESETS_Type) GetRESET_DONE_PLL_USB() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x8000) >> 15
}
func (o *RESETS_Type) SetRESET_DONE_PLL_SYS(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x4000)|value<<14)
}
func (o *RESETS_Type) GetRESET_DONE_PLL_SYS() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x4000) >> 14
}
func (o *RESETS_Type) SetRESET_DONE_PIO2(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x2000)|value<<13)
}
func (o *RESETS_Type) GetRESET_DONE_PIO2() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x2000) >> 13
}
func (o *RESETS_Type) SetRESET_DONE_PIO1(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x1000)|value<<12)
}
func (o *RESETS_Type) GetRESET_DONE_PIO1() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x1000) >> 12
}
func (o *RESETS_Type) SetRESET_DONE_PIO0(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x800)|value<<11)
}
func (o *RESETS_Type) GetRESET_DONE_PIO0() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x800) >> 11
}
func (o *RESETS_Type) SetRESET_DONE_PADS_QSPI(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x400)|value<<10)
}
func (o *RESETS_Type) GetRESET_DONE_PADS_QSPI() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x400) >> 10
}
func (o *RESETS_Type) SetRESET_DONE_PADS_BANK0(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x200)|value<<9)
}
func (o *RESETS_Type) GetRESET_DONE_PADS_BANK0() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x200) >> 9
}
func (o *RESETS_Type) SetRESET_DONE_JTAG(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x100)|value<<8)
}
func (o *RESETS_Type) GetRESET_DONE_JTAG() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x100) >> 8
}
func (o *RESETS_Type) SetRESET_DONE_IO_QSPI(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x80)|value<<7)
}
func (o *RESETS_Type) GetRESET_DONE_IO_QSPI() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x80) >> 7
}
func (o *RESETS_Type) SetRESET_DONE_IO_BANK0(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x40)|value<<6)
}
func (o *RESETS_Type) GetRESET_DONE_IO_BANK0() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x40) >> 6
}
func (o *RESETS_Type) SetRESET_DONE_I2C1(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x20)|value<<5)
}
func (o *RESETS_Type) GetRESET_DONE_I2C1() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x20) >> 5
}
func (o *RESETS_Type) SetRESET_DONE_I2C0(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x10)|value<<4)
}
func (o *RESETS_Type) GetRESET_DONE_I2C0() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x10) >> 4
}
func (o *RESETS_Type) SetRESET_DONE_HSTX(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x8)|value<<3)
}
func (o *RESETS_Type) GetRESET_DONE_HSTX() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x8) >> 3
}
func (o *RESETS_Type) SetRESET_DONE_DMA(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x4)|value<<2)
}
func (o *RESETS_Type) GetRESET_DONE_DMA() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x4) >> 2
}
func (o *RESETS_Type) SetRESET_DONE_BUSCTRL(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x2)|value<<1)
}
func (o *RESETS_Type) GetRESET_DONE_BUSCTRL() uint32 {
	return (volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x2) >> 1
}
func (o *RESETS_Type) SetRESET_DONE_ADC(value uint32) {
	volatile.StoreUint32(&o.RESET_DONE.Reg, volatile.LoadUint32(&o.RESET_DONE.Reg)&^(0x1)|value)
}
func (o *RESETS_Type) GetRESET_DONE_ADC() uint32 {
	return volatile.LoadUint32(&o.RESET_DONE.Reg) & 0x1
}

type PSM_Type struct {
	FRCE_ON  volatile.Register32 // 0x0
	FRCE_OFF volatile.Register32 // 0x4
	WDSEL    volatile.Register32 // 0x8
	DONE     volatile.Register32 // 0xC
}

// PSM.FRCE_ON: Force block out of reset (i.e. power it on)
func (o *PSM_Type) SetFRCE_ON_PROC1(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x1000000)|value<<24)
}
func (o *PSM_Type) GetFRCE_ON_PROC1() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x1000000) >> 24
}
func (o *PSM_Type) SetFRCE_ON_PROC0(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x800000)|value<<23)
}
func (o *PSM_Type) GetFRCE_ON_PROC0() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x800000) >> 23
}
func (o *PSM_Type) SetFRCE_ON_ACCESSCTRL(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x400000)|value<<22)
}
func (o *PSM_Type) GetFRCE_ON_ACCESSCTRL() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x400000) >> 22
}
func (o *PSM_Type) SetFRCE_ON_SIO(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x200000)|value<<21)
}
func (o *PSM_Type) GetFRCE_ON_SIO() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x200000) >> 21
}
func (o *PSM_Type) SetFRCE_ON_XIP(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x100000)|value<<20)
}
func (o *PSM_Type) GetFRCE_ON_XIP() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x100000) >> 20
}
func (o *PSM_Type) SetFRCE_ON_SRAM9(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x80000)|value<<19)
}
func (o *PSM_Type) GetFRCE_ON_SRAM9() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x80000) >> 19
}
func (o *PSM_Type) SetFRCE_ON_SRAM8(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x40000)|value<<18)
}
func (o *PSM_Type) GetFRCE_ON_SRAM8() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x40000) >> 18
}
func (o *PSM_Type) SetFRCE_ON_SRAM7(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x20000)|value<<17)
}
func (o *PSM_Type) GetFRCE_ON_SRAM7() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x20000) >> 17
}
func (o *PSM_Type) SetFRCE_ON_SRAM6(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x10000)|value<<16)
}
func (o *PSM_Type) GetFRCE_ON_SRAM6() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x10000) >> 16
}
func (o *PSM_Type) SetFRCE_ON_SRAM5(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x8000)|value<<15)
}
func (o *PSM_Type) GetFRCE_ON_SRAM5() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x8000) >> 15
}
func (o *PSM_Type) SetFRCE_ON_SRAM4(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x4000)|value<<14)
}
func (o *PSM_Type) GetFRCE_ON_SRAM4() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x4000) >> 14
}
func (o *PSM_Type) SetFRCE_ON_SRAM3(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x2000)|value<<13)
}
func (o *PSM_Type) GetFRCE_ON_SRAM3() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x2000) >> 13
}
func (o *PSM_Type) SetFRCE_ON_SRAM2(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x1000)|value<<12)
}
func (o *PSM_Type) GetFRCE_ON_SRAM2() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x1000) >> 12
}
func (o *PSM_Type) SetFRCE_ON_SRAM1(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x800)|value<<11)
}
func (o *PSM_Type) GetFRCE_ON_SRAM1() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x800) >> 11
}
func (o *PSM_Type) SetFRCE_ON_SRAM0(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x400)|value<<10)
}
func (o *PSM_Type) GetFRCE_ON_SRAM0() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x400) >> 10
}
func (o *PSM_Type) SetFRCE_ON_BOOTRAM(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x200)|value<<9)
}
func (o *PSM_Type) GetFRCE_ON_BOOTRAM() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x200) >> 9
}
func (o *PSM_Type) SetFRCE_ON_ROM(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x100)|value<<8)
}
func (o *PSM_Type) GetFRCE_ON_ROM() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x100) >> 8
}
func (o *PSM_Type) SetFRCE_ON_BUSFABRIC(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x80)|value<<7)
}
func (o *PSM_Type) GetFRCE_ON_BUSFABRIC() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x80) >> 7
}
func (o *PSM_Type) SetFRCE_ON_PSM_READY(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x40)|value<<6)
}
func (o *PSM_Type) GetFRCE_ON_PSM_READY() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x40) >> 6
}
func (o *PSM_Type) SetFRCE_ON_CLOCKS(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x20)|value<<5)
}
func (o *PSM_Type) GetFRCE_ON_CLOCKS() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x20) >> 5
}
func (o *PSM_Type) SetFRCE_ON_RESETS(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x10)|value<<4)
}
func (o *PSM_Type) GetFRCE_ON_RESETS() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x10) >> 4
}
func (o *PSM_Type) SetFRCE_ON_XOSC(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x8)|value<<3)
}
func (o *PSM_Type) GetFRCE_ON_XOSC() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x8) >> 3
}
func (o *PSM_Type) SetFRCE_ON_ROSC(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x4)|value<<2)
}
func (o *PSM_Type) GetFRCE_ON_ROSC() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x4) >> 2
}
func (o *PSM_Type) SetFRCE_ON_OTP(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x2)|value<<1)
}
func (o *PSM_Type) GetFRCE_ON_OTP() uint32 {
	return (volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x2) >> 1
}
func (o *PSM_Type) SetFRCE_ON_PROC_COLD(value uint32) {
	volatile.StoreUint32(&o.FRCE_ON.Reg, volatile.LoadUint32(&o.FRCE_ON.Reg)&^(0x1)|value)
}
func (o *PSM_Type) GetFRCE_ON_PROC_COLD() uint32 {
	return volatile.LoadUint32(&o.FRCE_ON.Reg) & 0x1
}

// PSM.FRCE_OFF: Force into reset (i.e. power it off)
func (o *PSM_Type) SetFRCE_OFF_PROC1(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x1000000)|value<<24)
}
func (o *PSM_Type) GetFRCE_OFF_PROC1() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x1000000) >> 24
}
func (o *PSM_Type) SetFRCE_OFF_PROC0(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x800000)|value<<23)
}
func (o *PSM_Type) GetFRCE_OFF_PROC0() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x800000) >> 23
}
func (o *PSM_Type) SetFRCE_OFF_ACCESSCTRL(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x400000)|value<<22)
}
func (o *PSM_Type) GetFRCE_OFF_ACCESSCTRL() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x400000) >> 22
}
func (o *PSM_Type) SetFRCE_OFF_SIO(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x200000)|value<<21)
}
func (o *PSM_Type) GetFRCE_OFF_SIO() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x200000) >> 21
}
func (o *PSM_Type) SetFRCE_OFF_XIP(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x100000)|value<<20)
}
func (o *PSM_Type) GetFRCE_OFF_XIP() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x100000) >> 20
}
func (o *PSM_Type) SetFRCE_OFF_SRAM9(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x80000)|value<<19)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM9() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x80000) >> 19
}
func (o *PSM_Type) SetFRCE_OFF_SRAM8(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x40000)|value<<18)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM8() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x40000) >> 18
}
func (o *PSM_Type) SetFRCE_OFF_SRAM7(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x20000)|value<<17)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM7() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x20000) >> 17
}
func (o *PSM_Type) SetFRCE_OFF_SRAM6(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x10000)|value<<16)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM6() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x10000) >> 16
}
func (o *PSM_Type) SetFRCE_OFF_SRAM5(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x8000)|value<<15)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM5() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x8000) >> 15
}
func (o *PSM_Type) SetFRCE_OFF_SRAM4(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x4000)|value<<14)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM4() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x4000) >> 14
}
func (o *PSM_Type) SetFRCE_OFF_SRAM3(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x2000)|value<<13)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM3() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x2000) >> 13
}
func (o *PSM_Type) SetFRCE_OFF_SRAM2(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x1000)|value<<12)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM2() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x1000) >> 12
}
func (o *PSM_Type) SetFRCE_OFF_SRAM1(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x800)|value<<11)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM1() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x800) >> 11
}
func (o *PSM_Type) SetFRCE_OFF_SRAM0(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x400)|value<<10)
}
func (o *PSM_Type) GetFRCE_OFF_SRAM0() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x400) >> 10
}
func (o *PSM_Type) SetFRCE_OFF_BOOTRAM(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x200)|value<<9)
}
func (o *PSM_Type) GetFRCE_OFF_BOOTRAM() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x200) >> 9
}
func (o *PSM_Type) SetFRCE_OFF_ROM(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x100)|value<<8)
}
func (o *PSM_Type) GetFRCE_OFF_ROM() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x100) >> 8
}
func (o *PSM_Type) SetFRCE_OFF_BUSFABRIC(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x80)|value<<7)
}
func (o *PSM_Type) GetFRCE_OFF_BUSFABRIC() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x80) >> 7
}
func (o *PSM_Type) SetFRCE_OFF_PSM_READY(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x40)|value<<6)
}
func (o *PSM_Type) GetFRCE_OFF_PSM_READY() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x40) >> 6
}
func (o *PSM_Type) SetFRCE_OFF_CLOCKS(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x20)|value<<5)
}
func (o *PSM_Type) GetFRCE_OFF_CLOCKS() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x20) >> 5
}
func (o *PSM_Type) SetFRCE_OFF_RESETS(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x10)|value<<4)
}
func (o *PSM_Type) GetFRCE_OFF_RESETS() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x10) >> 4
}
func (o *PSM_Type) SetFRCE_OFF_XOSC(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x8)|value<<3)
}
func (o *PSM_Type) GetFRCE_OFF_XOSC() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x8) >> 3
}
func (o *PSM_Type) SetFRCE_OFF_ROSC(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x4)|value<<2)
}
func (o *PSM_Type) GetFRCE_OFF_ROSC() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x4) >> 2
}
func (o *PSM_Type) SetFRCE_OFF_OTP(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x2)|value<<1)
}
func (o *PSM_Type) GetFRCE_OFF_OTP() uint32 {
	return (volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x2) >> 1
}
func (o *PSM_Type) SetFRCE_OFF_PROC_COLD(value uint32) {
	volatile.StoreUint32(&o.FRCE_OFF.Reg, volatile.LoadUint32(&o.FRCE_OFF.Reg)&^(0x1)|value)
}
func (o *PSM_Type) GetFRCE_OFF_PROC_COLD() uint32 {
	return volatile.LoadUint32(&o.FRCE_OFF.Reg) & 0x1
}

// PSM.WDSEL: Set to 1 if the watchdog should reset this
func (o *PSM_Type) SetWDSEL_PROC1(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x1000000)|value<<24)
}
func (o *PSM_Type) GetWDSEL_PROC1() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x1000000) >> 24
}
func (o *PSM_Type) SetWDSEL_PROC0(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x800000)|value<<23)
}
func (o *PSM_Type) GetWDSEL_PROC0() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x800000) >> 23
}
func (o *PSM_Type) SetWDSEL_ACCESSCTRL(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x400000)|value<<22)
}
func (o *PSM_Type) GetWDSEL_ACCESSCTRL() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x400000) >> 22
}
func (o *PSM_Type) SetWDSEL_SIO(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x200000)|value<<21)
}
func (o *PSM_Type) GetWDSEL_SIO() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x200000) >> 21
}
func (o *PSM_Type) SetWDSEL_XIP(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x100000)|value<<20)
}
func (o *PSM_Type) GetWDSEL_XIP() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x100000) >> 20
}
func (o *PSM_Type) SetWDSEL_SRAM9(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x80000)|value<<19)
}
func (o *PSM_Type) GetWDSEL_SRAM9() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x80000) >> 19
}
func (o *PSM_Type) SetWDSEL_SRAM8(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x40000)|value<<18)
}
func (o *PSM_Type) GetWDSEL_SRAM8() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x40000) >> 18
}
func (o *PSM_Type) SetWDSEL_SRAM7(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x20000)|value<<17)
}
func (o *PSM_Type) GetWDSEL_SRAM7() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x20000) >> 17
}
func (o *PSM_Type) SetWDSEL_SRAM6(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x10000)|value<<16)
}
func (o *PSM_Type) GetWDSEL_SRAM6() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x10000) >> 16
}
func (o *PSM_Type) SetWDSEL_SRAM5(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x8000)|value<<15)
}
func (o *PSM_Type) GetWDSEL_SRAM5() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x8000) >> 15
}
func (o *PSM_Type) SetWDSEL_SRAM4(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x4000)|value<<14)
}
func (o *PSM_Type) GetWDSEL_SRAM4() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x4000) >> 14
}
func (o *PSM_Type) SetWDSEL_SRAM3(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x2000)|value<<13)
}
func (o *PSM_Type) GetWDSEL_SRAM3() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x2000) >> 13
}
func (o *PSM_Type) SetWDSEL_SRAM2(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x1000)|value<<12)
}
func (o *PSM_Type) GetWDSEL_SRAM2() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x1000) >> 12
}
func (o *PSM_Type) SetWDSEL_SRAM1(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x800)|value<<11)
}
func (o *PSM_Type) GetWDSEL_SRAM1() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x800) >> 11
}
func (o *PSM_Type) SetWDSEL_SRAM0(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x400)|value<<10)
}
func (o *PSM_Type) GetWDSEL_SRAM0() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x400) >> 10
}
func (o *PSM_Type) SetWDSEL_BOOTRAM(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x200)|value<<9)
}
func (o *PSM_Type) GetWDSEL_BOOTRAM() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x200) >> 9
}
func (o *PSM_Type) SetWDSEL_ROM(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x100)|value<<8)
}
func (o *PSM_Type) GetWDSEL_ROM() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x100) >> 8
}
func (o *PSM_Type) SetWDSEL_BUSFABRIC(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x80)|value<<7)
}
func (o *PSM_Type) GetWDSEL_BUSFABRIC() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x80) >> 7
}
func (o *PSM_Type) SetWDSEL_PSM_READY(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x40)|value<<6)
}
func (o *PSM_Type) GetWDSEL_PSM_READY() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x40) >> 6
}
func (o *PSM_Type) SetWDSEL_CLOCKS(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x20)|value<<5)
}
func (o *PSM_Type) GetWDSEL_CLOCKS() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x20) >> 5
}
func (o *PSM_Type) SetWDSEL_RESETS(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x10)|value<<4)
}
func (o *PSM_Type) GetWDSEL_RESETS() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x10) >> 4
}
func (o *PSM_Type) SetWDSEL_XOSC(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x8)|value<<3)
}
func (o *PSM_Type) GetWDSEL_XOSC() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x8) >> 3
}
func (o *PSM_Type) SetWDSEL_ROSC(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x4)|value<<2)
}
func (o *PSM_Type) GetWDSEL_ROSC() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x4) >> 2
}
func (o *PSM_Type) SetWDSEL_OTP(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x2)|value<<1)
}
func (o *PSM_Type) GetWDSEL_OTP() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x2) >> 1
}
func (o *PSM_Type) SetWDSEL_PROC_COLD(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x1)|value)
}
func (o *PSM_Type) GetWDSEL_PROC_COLD() uint32 {
	return volatile.LoadUint32(&o.WDSEL.Reg) & 0x1
}

// PSM.DONE: Is the subsystem ready?
func (o *PSM_Type) SetDONE_PROC1(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x1000000)|value<<24)
}
func (o *PSM_Type) GetDONE_PROC1() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x1000000) >> 24
}
func (o *PSM_Type) SetDONE_PROC0(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x800000)|value<<23)
}
func (o *PSM_Type) GetDONE_PROC0() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x800000) >> 23
}
func (o *PSM_Type) SetDONE_ACCESSCTRL(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x400000)|value<<22)
}
func (o *PSM_Type) GetDONE_ACCESSCTRL() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x400000) >> 22
}
func (o *PSM_Type) SetDONE_SIO(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x200000)|value<<21)
}
func (o *PSM_Type) GetDONE_SIO() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x200000) >> 21
}
func (o *PSM_Type) SetDONE_XIP(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x100000)|value<<20)
}
func (o *PSM_Type) GetDONE_XIP() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x100000) >> 20
}
func (o *PSM_Type) SetDONE_SRAM9(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x80000)|value<<19)
}
func (o *PSM_Type) GetDONE_SRAM9() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x80000) >> 19
}
func (o *PSM_Type) SetDONE_SRAM8(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x40000)|value<<18)
}
func (o *PSM_Type) GetDONE_SRAM8() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x40000) >> 18
}
func (o *PSM_Type) SetDONE_SRAM7(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x20000)|value<<17)
}
func (o *PSM_Type) GetDONE_SRAM7() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x20000) >> 17
}
func (o *PSM_Type) SetDONE_SRAM6(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x10000)|value<<16)
}
func (o *PSM_Type) GetDONE_SRAM6() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x10000) >> 16
}
func (o *PSM_Type) SetDONE_SRAM5(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x8000)|value<<15)
}
func (o *PSM_Type) GetDONE_SRAM5() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x8000) >> 15
}
func (o *PSM_Type) SetDONE_SRAM4(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x4000)|value<<14)
}
func (o *PSM_Type) GetDONE_SRAM4() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x4000) >> 14
}
func (o *PSM_Type) SetDONE_SRAM3(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x2000)|value<<13)
}
func (o *PSM_Type) GetDONE_SRAM3() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x2000) >> 13
}
func (o *PSM_Type) SetDONE_SRAM2(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x1000)|value<<12)
}
func (o *PSM_Type) GetDONE_SRAM2() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x1000) >> 12
}
func (o *PSM_Type) SetDONE_SRAM1(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x800)|value<<11)
}
func (o *PSM_Type) GetDONE_SRAM1() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x800) >> 11
}
func (o *PSM_Type) SetDONE_SRAM0(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x400)|value<<10)
}
func (o *PSM_Type) GetDONE_SRAM0() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x400) >> 10
}
func (o *PSM_Type) SetDONE_BOOTRAM(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x200)|value<<9)
}
func (o *PSM_Type) GetDONE_BOOTRAM() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x200) >> 9
}
func (o *PSM_Type) SetDONE_ROM(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x100)|value<<8)
}
func (o *PSM_Type) GetDONE_ROM() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x100) >> 8
}
func (o *PSM_Type) SetDONE_BUSFABRIC(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x80)|value<<7)
}
func (o *PSM_Type) GetDONE_BUSFABRIC() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x80) >> 7
}
func (o *PSM_Type) SetDONE_PSM_READY(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x40)|value<<6)
}
func (o *PSM_Type) GetDONE_PSM_READY() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x40) >> 6
}
func (o *PSM_Type) SetDONE_CLOCKS(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x20)|value<<5)
}
func (o *PSM_Type) GetDONE_CLOCKS() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x20) >> 5
}
func (o *PSM_Type) SetDONE_RESETS(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x10)|value<<4)
}
func (o *PSM_Type) GetDONE_RESETS() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x10) >> 4
}
func (o *PSM_Type) SetDONE_XOSC(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x8)|value<<3)
}
func (o *PSM_Type) GetDONE_XOSC() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x8) >> 3
}
func (o *PSM_Type) SetDONE_ROSC(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x4)|value<<2)
}
func (o *PSM_Type) GetDONE_ROSC() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x4) >> 2
}
func (o *PSM_Type) SetDONE_OTP(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x2)|value<<1)
}
func (o *PSM_Type) GetDONE_OTP() uint32 {
	return (volatile.LoadUint32(&o.DONE.Reg) & 0x2) >> 1
}
func (o *PSM_Type) SetDONE_PROC_COLD(value uint32) {
	volatile.StoreUint32(&o.DONE.Reg, volatile.LoadUint32(&o.DONE.Reg)&^(0x1)|value)
}
func (o *PSM_Type) GetDONE_PROC_COLD() uint32 {
	return volatile.LoadUint32(&o.DONE.Reg) & 0x1
}

type CLOCKS_Type struct {
	CLK_GPOUT0_CTRL      volatile.Register32 // 0x0
	CLK_GPOUT0_DIV       volatile.Register32 // 0x4
	CLK_GPOUT0_SELECTED  volatile.Register32 // 0x8
	CLK_GPOUT1_CTRL      volatile.Register32 // 0xC
	CLK_GPOUT1_DIV       volatile.Register32 // 0x10
	CLK_GPOUT1_SELECTED  volatile.Register32 // 0x14
	CLK_GPOUT2_CTRL      volatile.Register32 // 0x18
	CLK_GPOUT2_DIV       volatile.Register32 // 0x1C
	CLK_GPOUT2_SELECTED  volatile.Register32 // 0x20
	CLK_GPOUT3_CTRL      volatile.Register32 // 0x24
	CLK_GPOUT3_DIV       volatile.Register32 // 0x28
	CLK_GPOUT3_SELECTED  volatile.Register32 // 0x2C
	CLK_REF_CTRL         volatile.Register32 // 0x30
	CLK_REF_DIV          volatile.Register32 // 0x34
	CLK_REF_SELECTED     volatile.Register32 // 0x38
	CLK_SYS_CTRL         volatile.Register32 // 0x3C
	CLK_SYS_DIV          volatile.Register32 // 0x40
	CLK_SYS_SELECTED     volatile.Register32 // 0x44
	CLK_PERI_CTRL        volatile.Register32 // 0x48
	CLK_PERI_DIV         volatile.Register32 // 0x4C
	CLK_PERI_SELECTED    volatile.Register32 // 0x50
	CLK_HSTX_CTRL        volatile.Register32 // 0x54
	CLK_HSTX_DIV         volatile.Register32 // 0x58
	CLK_HSTX_SELECTED    volatile.Register32 // 0x5C
	CLK_USB_CTRL         volatile.Register32 // 0x60
	CLK_USB_DIV          volatile.Register32 // 0x64
	CLK_USB_SELECTED     volatile.Register32 // 0x68
	CLK_ADC_CTRL         volatile.Register32 // 0x6C
	CLK_ADC_DIV          volatile.Register32 // 0x70
	CLK_ADC_SELECTED     volatile.Register32 // 0x74
	DFTCLK_XOSC_CTRL     volatile.Register32 // 0x78
	DFTCLK_ROSC_CTRL     volatile.Register32 // 0x7C
	DFTCLK_LPOSC_CTRL    volatile.Register32 // 0x80
	CLK_SYS_RESUS_CTRL   volatile.Register32 // 0x84
	CLK_SYS_RESUS_STATUS volatile.Register32 // 0x88
	FC0_REF_KHZ          volatile.Register32 // 0x8C
	FC0_MIN_KHZ          volatile.Register32 // 0x90
	FC0_MAX_KHZ          volatile.Register32 // 0x94
	FC0_DELAY            volatile.Register32 // 0x98
	FC0_INTERVAL         volatile.Register32 // 0x9C
	FC0_SRC              volatile.Register32 // 0xA0
	FC0_STATUS           volatile.Register32 // 0xA4
	FC0_RESULT           volatile.Register32 // 0xA8
	WAKE_EN0             volatile.Register32 // 0xAC
	WAKE_EN1             volatile.Register32 // 0xB0
	SLEEP_EN0            volatile.Register32 // 0xB4
	SLEEP_EN1            volatile.Register32 // 0xB8
	ENABLED0             volatile.Register32 // 0xBC
	ENABLED1             volatile.Register32 // 0xC0
	INTR                 volatile.Register32 // 0xC4
	INTE                 volatile.Register32 // 0xC8
	INTF                 volatile.Register32 // 0xCC
	INTS                 volatile.Register32 // 0xD0
}

// CLOCKS.CLK_GPOUT0_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_GPOUT0_CTRL_ENABLED(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_CTRL_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetCLK_GPOUT0_CTRL_NUDGE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_CTRL_NUDGE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetCLK_GPOUT0_CTRL_PHASE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_CTRL_PHASE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg) & 0x30000) >> 16
}
func (o *CLOCKS_Type) SetCLK_GPOUT0_CTRL_DC50(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_CTRL_DC50() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetCLK_GPOUT0_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_CTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetCLK_GPOUT0_CTRL_KILL(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_CTRL_KILL() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetCLK_GPOUT0_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg)&^(0x1e0)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT0_CTRL.Reg) & 0x1e0) >> 5
}

// CLOCKS.CLK_GPOUT0_DIV
func (o *CLOCKS_Type) SetCLK_GPOUT0_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_DIV.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_DIV.Reg)&^(0xffff0000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT0_DIV.Reg) & 0xffff0000) >> 16
}
func (o *CLOCKS_Type) SetCLK_GPOUT0_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_DIV.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_DIV.Reg)&^(0xffff)|value)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CLK_GPOUT0_DIV.Reg) & 0xffff
}

// CLOCKS.CLK_GPOUT0_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_GPOUT0_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT0_SELECTED.Reg, volatile.LoadUint32(&o.CLK_GPOUT0_SELECTED.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_GPOUT0_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_GPOUT0_SELECTED.Reg) & 0x1
}

// CLOCKS.CLK_GPOUT1_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_GPOUT1_CTRL_ENABLED(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_CTRL_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetCLK_GPOUT1_CTRL_NUDGE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_CTRL_NUDGE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetCLK_GPOUT1_CTRL_PHASE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_CTRL_PHASE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg) & 0x30000) >> 16
}
func (o *CLOCKS_Type) SetCLK_GPOUT1_CTRL_DC50(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_CTRL_DC50() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetCLK_GPOUT1_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_CTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetCLK_GPOUT1_CTRL_KILL(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_CTRL_KILL() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetCLK_GPOUT1_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg)&^(0x1e0)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT1_CTRL.Reg) & 0x1e0) >> 5
}

// CLOCKS.CLK_GPOUT1_DIV
func (o *CLOCKS_Type) SetCLK_GPOUT1_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_DIV.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_DIV.Reg)&^(0xffff0000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT1_DIV.Reg) & 0xffff0000) >> 16
}
func (o *CLOCKS_Type) SetCLK_GPOUT1_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_DIV.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_DIV.Reg)&^(0xffff)|value)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CLK_GPOUT1_DIV.Reg) & 0xffff
}

// CLOCKS.CLK_GPOUT1_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_GPOUT1_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT1_SELECTED.Reg, volatile.LoadUint32(&o.CLK_GPOUT1_SELECTED.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_GPOUT1_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_GPOUT1_SELECTED.Reg) & 0x1
}

// CLOCKS.CLK_GPOUT2_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_GPOUT2_CTRL_ENABLED(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_CTRL_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetCLK_GPOUT2_CTRL_NUDGE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_CTRL_NUDGE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetCLK_GPOUT2_CTRL_PHASE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_CTRL_PHASE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg) & 0x30000) >> 16
}
func (o *CLOCKS_Type) SetCLK_GPOUT2_CTRL_DC50(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_CTRL_DC50() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetCLK_GPOUT2_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_CTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetCLK_GPOUT2_CTRL_KILL(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_CTRL_KILL() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetCLK_GPOUT2_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg)&^(0x1e0)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT2_CTRL.Reg) & 0x1e0) >> 5
}

// CLOCKS.CLK_GPOUT2_DIV
func (o *CLOCKS_Type) SetCLK_GPOUT2_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_DIV.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_DIV.Reg)&^(0xffff0000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT2_DIV.Reg) & 0xffff0000) >> 16
}
func (o *CLOCKS_Type) SetCLK_GPOUT2_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_DIV.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_DIV.Reg)&^(0xffff)|value)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CLK_GPOUT2_DIV.Reg) & 0xffff
}

// CLOCKS.CLK_GPOUT2_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_GPOUT2_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT2_SELECTED.Reg, volatile.LoadUint32(&o.CLK_GPOUT2_SELECTED.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_GPOUT2_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_GPOUT2_SELECTED.Reg) & 0x1
}

// CLOCKS.CLK_GPOUT3_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_GPOUT3_CTRL_ENABLED(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_CTRL_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetCLK_GPOUT3_CTRL_NUDGE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_CTRL_NUDGE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetCLK_GPOUT3_CTRL_PHASE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_CTRL_PHASE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg) & 0x30000) >> 16
}
func (o *CLOCKS_Type) SetCLK_GPOUT3_CTRL_DC50(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_CTRL_DC50() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetCLK_GPOUT3_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_CTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetCLK_GPOUT3_CTRL_KILL(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_CTRL_KILL() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetCLK_GPOUT3_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_CTRL.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg)&^(0x1e0)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT3_CTRL.Reg) & 0x1e0) >> 5
}

// CLOCKS.CLK_GPOUT3_DIV
func (o *CLOCKS_Type) SetCLK_GPOUT3_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_DIV.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_DIV.Reg)&^(0xffff0000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_GPOUT3_DIV.Reg) & 0xffff0000) >> 16
}
func (o *CLOCKS_Type) SetCLK_GPOUT3_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_DIV.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_DIV.Reg)&^(0xffff)|value)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CLK_GPOUT3_DIV.Reg) & 0xffff
}

// CLOCKS.CLK_GPOUT3_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_GPOUT3_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_GPOUT3_SELECTED.Reg, volatile.LoadUint32(&o.CLK_GPOUT3_SELECTED.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_GPOUT3_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_GPOUT3_SELECTED.Reg) & 0x1
}

// CLOCKS.CLK_REF_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_REF_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_REF_CTRL.Reg, volatile.LoadUint32(&o.CLK_REF_CTRL.Reg)&^(0x60)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_REF_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_REF_CTRL.Reg) & 0x60) >> 5
}
func (o *CLOCKS_Type) SetCLK_REF_CTRL_SRC(value uint32) {
	volatile.StoreUint32(&o.CLK_REF_CTRL.Reg, volatile.LoadUint32(&o.CLK_REF_CTRL.Reg)&^(0x3)|value)
}
func (o *CLOCKS_Type) GetCLK_REF_CTRL_SRC() uint32 {
	return volatile.LoadUint32(&o.CLK_REF_CTRL.Reg) & 0x3
}

// CLOCKS.CLK_REF_DIV
func (o *CLOCKS_Type) SetCLK_REF_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_REF_DIV.Reg, volatile.LoadUint32(&o.CLK_REF_DIV.Reg)&^(0xff0000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_REF_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_REF_DIV.Reg) & 0xff0000) >> 16
}

// CLOCKS.CLK_REF_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_REF_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_REF_SELECTED.Reg, volatile.LoadUint32(&o.CLK_REF_SELECTED.Reg)&^(0xf)|value)
}
func (o *CLOCKS_Type) GetCLK_REF_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_REF_SELECTED.Reg) & 0xf
}

// CLOCKS.CLK_SYS_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_SYS_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_CTRL.Reg, volatile.LoadUint32(&o.CLK_SYS_CTRL.Reg)&^(0xe0)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_SYS_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_SYS_CTRL.Reg) & 0xe0) >> 5
}
func (o *CLOCKS_Type) SetCLK_SYS_CTRL_SRC(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_CTRL.Reg, volatile.LoadUint32(&o.CLK_SYS_CTRL.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_SYS_CTRL_SRC() uint32 {
	return volatile.LoadUint32(&o.CLK_SYS_CTRL.Reg) & 0x1
}

// CLOCKS.CLK_SYS_DIV
func (o *CLOCKS_Type) SetCLK_SYS_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_DIV.Reg, volatile.LoadUint32(&o.CLK_SYS_DIV.Reg)&^(0xffff0000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_SYS_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_SYS_DIV.Reg) & 0xffff0000) >> 16
}
func (o *CLOCKS_Type) SetCLK_SYS_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_DIV.Reg, volatile.LoadUint32(&o.CLK_SYS_DIV.Reg)&^(0xffff)|value)
}
func (o *CLOCKS_Type) GetCLK_SYS_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CLK_SYS_DIV.Reg) & 0xffff
}

// CLOCKS.CLK_SYS_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_SYS_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_SELECTED.Reg, volatile.LoadUint32(&o.CLK_SYS_SELECTED.Reg)&^(0x3)|value)
}
func (o *CLOCKS_Type) GetCLK_SYS_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_SYS_SELECTED.Reg) & 0x3
}

// CLOCKS.CLK_PERI_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_PERI_CTRL_ENABLED(value uint32) {
	volatile.StoreUint32(&o.CLK_PERI_CTRL.Reg, volatile.LoadUint32(&o.CLK_PERI_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetCLK_PERI_CTRL_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.CLK_PERI_CTRL.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetCLK_PERI_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CLK_PERI_CTRL.Reg, volatile.LoadUint32(&o.CLK_PERI_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetCLK_PERI_CTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CLK_PERI_CTRL.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetCLK_PERI_CTRL_KILL(value uint32) {
	volatile.StoreUint32(&o.CLK_PERI_CTRL.Reg, volatile.LoadUint32(&o.CLK_PERI_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetCLK_PERI_CTRL_KILL() uint32 {
	return (volatile.LoadUint32(&o.CLK_PERI_CTRL.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetCLK_PERI_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_PERI_CTRL.Reg, volatile.LoadUint32(&o.CLK_PERI_CTRL.Reg)&^(0xe0)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_PERI_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_PERI_CTRL.Reg) & 0xe0) >> 5
}

// CLOCKS.CLK_PERI_DIV
func (o *CLOCKS_Type) SetCLK_PERI_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_PERI_DIV.Reg, volatile.LoadUint32(&o.CLK_PERI_DIV.Reg)&^(0x30000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_PERI_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_PERI_DIV.Reg) & 0x30000) >> 16
}

// CLOCKS.CLK_PERI_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_PERI_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_PERI_SELECTED.Reg, volatile.LoadUint32(&o.CLK_PERI_SELECTED.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_PERI_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_PERI_SELECTED.Reg) & 0x1
}

// CLOCKS.CLK_HSTX_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_HSTX_CTRL_ENABLED(value uint32) {
	volatile.StoreUint32(&o.CLK_HSTX_CTRL.Reg, volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetCLK_HSTX_CTRL_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetCLK_HSTX_CTRL_NUDGE(value uint32) {
	volatile.StoreUint32(&o.CLK_HSTX_CTRL.Reg, volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetCLK_HSTX_CTRL_NUDGE() uint32 {
	return (volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetCLK_HSTX_CTRL_PHASE(value uint32) {
	volatile.StoreUint32(&o.CLK_HSTX_CTRL.Reg, volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_HSTX_CTRL_PHASE() uint32 {
	return (volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg) & 0x30000) >> 16
}
func (o *CLOCKS_Type) SetCLK_HSTX_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CLK_HSTX_CTRL.Reg, volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetCLK_HSTX_CTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetCLK_HSTX_CTRL_KILL(value uint32) {
	volatile.StoreUint32(&o.CLK_HSTX_CTRL.Reg, volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetCLK_HSTX_CTRL_KILL() uint32 {
	return (volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetCLK_HSTX_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_HSTX_CTRL.Reg, volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg)&^(0xe0)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_HSTX_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_HSTX_CTRL.Reg) & 0xe0) >> 5
}

// CLOCKS.CLK_HSTX_DIV
func (o *CLOCKS_Type) SetCLK_HSTX_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_HSTX_DIV.Reg, volatile.LoadUint32(&o.CLK_HSTX_DIV.Reg)&^(0x30000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_HSTX_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_HSTX_DIV.Reg) & 0x30000) >> 16
}

// CLOCKS.CLK_HSTX_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_HSTX_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_HSTX_SELECTED.Reg, volatile.LoadUint32(&o.CLK_HSTX_SELECTED.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_HSTX_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_HSTX_SELECTED.Reg) & 0x1
}

// CLOCKS.CLK_USB_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_USB_CTRL_ENABLED(value uint32) {
	volatile.StoreUint32(&o.CLK_USB_CTRL.Reg, volatile.LoadUint32(&o.CLK_USB_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetCLK_USB_CTRL_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.CLK_USB_CTRL.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetCLK_USB_CTRL_NUDGE(value uint32) {
	volatile.StoreUint32(&o.CLK_USB_CTRL.Reg, volatile.LoadUint32(&o.CLK_USB_CTRL.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetCLK_USB_CTRL_NUDGE() uint32 {
	return (volatile.LoadUint32(&o.CLK_USB_CTRL.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetCLK_USB_CTRL_PHASE(value uint32) {
	volatile.StoreUint32(&o.CLK_USB_CTRL.Reg, volatile.LoadUint32(&o.CLK_USB_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_USB_CTRL_PHASE() uint32 {
	return (volatile.LoadUint32(&o.CLK_USB_CTRL.Reg) & 0x30000) >> 16
}
func (o *CLOCKS_Type) SetCLK_USB_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CLK_USB_CTRL.Reg, volatile.LoadUint32(&o.CLK_USB_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetCLK_USB_CTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CLK_USB_CTRL.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetCLK_USB_CTRL_KILL(value uint32) {
	volatile.StoreUint32(&o.CLK_USB_CTRL.Reg, volatile.LoadUint32(&o.CLK_USB_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetCLK_USB_CTRL_KILL() uint32 {
	return (volatile.LoadUint32(&o.CLK_USB_CTRL.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetCLK_USB_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_USB_CTRL.Reg, volatile.LoadUint32(&o.CLK_USB_CTRL.Reg)&^(0xe0)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_USB_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_USB_CTRL.Reg) & 0xe0) >> 5
}

// CLOCKS.CLK_USB_DIV
func (o *CLOCKS_Type) SetCLK_USB_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_USB_DIV.Reg, volatile.LoadUint32(&o.CLK_USB_DIV.Reg)&^(0xf0000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_USB_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_USB_DIV.Reg) & 0xf0000) >> 16
}

// CLOCKS.CLK_USB_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_USB_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_USB_SELECTED.Reg, volatile.LoadUint32(&o.CLK_USB_SELECTED.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_USB_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_USB_SELECTED.Reg) & 0x1
}

// CLOCKS.CLK_ADC_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
func (o *CLOCKS_Type) SetCLK_ADC_CTRL_ENABLED(value uint32) {
	volatile.StoreUint32(&o.CLK_ADC_CTRL.Reg, volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetCLK_ADC_CTRL_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetCLK_ADC_CTRL_NUDGE(value uint32) {
	volatile.StoreUint32(&o.CLK_ADC_CTRL.Reg, volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetCLK_ADC_CTRL_NUDGE() uint32 {
	return (volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetCLK_ADC_CTRL_PHASE(value uint32) {
	volatile.StoreUint32(&o.CLK_ADC_CTRL.Reg, volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_ADC_CTRL_PHASE() uint32 {
	return (volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg) & 0x30000) >> 16
}
func (o *CLOCKS_Type) SetCLK_ADC_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CLK_ADC_CTRL.Reg, volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetCLK_ADC_CTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetCLK_ADC_CTRL_KILL(value uint32) {
	volatile.StoreUint32(&o.CLK_ADC_CTRL.Reg, volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetCLK_ADC_CTRL_KILL() uint32 {
	return (volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetCLK_ADC_CTRL_AUXSRC(value uint32) {
	volatile.StoreUint32(&o.CLK_ADC_CTRL.Reg, volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg)&^(0xe0)|value<<5)
}
func (o *CLOCKS_Type) GetCLK_ADC_CTRL_AUXSRC() uint32 {
	return (volatile.LoadUint32(&o.CLK_ADC_CTRL.Reg) & 0xe0) >> 5
}

// CLOCKS.CLK_ADC_DIV
func (o *CLOCKS_Type) SetCLK_ADC_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CLK_ADC_DIV.Reg, volatile.LoadUint32(&o.CLK_ADC_DIV.Reg)&^(0xf0000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_ADC_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CLK_ADC_DIV.Reg) & 0xf0000) >> 16
}

// CLOCKS.CLK_ADC_SELECTED: Indicates which src is currently selected (one-hot)
func (o *CLOCKS_Type) SetCLK_ADC_SELECTED(value uint32) {
	volatile.StoreUint32(&o.CLK_ADC_SELECTED.Reg, volatile.LoadUint32(&o.CLK_ADC_SELECTED.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_ADC_SELECTED() uint32 {
	return volatile.LoadUint32(&o.CLK_ADC_SELECTED.Reg) & 0x1
}

// CLOCKS.DFTCLK_XOSC_CTRL
func (o *CLOCKS_Type) SetDFTCLK_XOSC_CTRL_SRC(value uint32) {
	volatile.StoreUint32(&o.DFTCLK_XOSC_CTRL.Reg, volatile.LoadUint32(&o.DFTCLK_XOSC_CTRL.Reg)&^(0x3)|value)
}
func (o *CLOCKS_Type) GetDFTCLK_XOSC_CTRL_SRC() uint32 {
	return volatile.LoadUint32(&o.DFTCLK_XOSC_CTRL.Reg) & 0x3
}

// CLOCKS.DFTCLK_ROSC_CTRL
func (o *CLOCKS_Type) SetDFTCLK_ROSC_CTRL_SRC(value uint32) {
	volatile.StoreUint32(&o.DFTCLK_ROSC_CTRL.Reg, volatile.LoadUint32(&o.DFTCLK_ROSC_CTRL.Reg)&^(0x3)|value)
}
func (o *CLOCKS_Type) GetDFTCLK_ROSC_CTRL_SRC() uint32 {
	return volatile.LoadUint32(&o.DFTCLK_ROSC_CTRL.Reg) & 0x3
}

// CLOCKS.DFTCLK_LPOSC_CTRL
func (o *CLOCKS_Type) SetDFTCLK_LPOSC_CTRL_SRC(value uint32) {
	volatile.StoreUint32(&o.DFTCLK_LPOSC_CTRL.Reg, volatile.LoadUint32(&o.DFTCLK_LPOSC_CTRL.Reg)&^(0x3)|value)
}
func (o *CLOCKS_Type) GetDFTCLK_LPOSC_CTRL_SRC() uint32 {
	return volatile.LoadUint32(&o.DFTCLK_LPOSC_CTRL.Reg) & 0x3
}

// CLOCKS.CLK_SYS_RESUS_CTRL
func (o *CLOCKS_Type) SetCLK_SYS_RESUS_CTRL_CLEAR(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_RESUS_CTRL.Reg, volatile.LoadUint32(&o.CLK_SYS_RESUS_CTRL.Reg)&^(0x10000)|value<<16)
}
func (o *CLOCKS_Type) GetCLK_SYS_RESUS_CTRL_CLEAR() uint32 {
	return (volatile.LoadUint32(&o.CLK_SYS_RESUS_CTRL.Reg) & 0x10000) >> 16
}
func (o *CLOCKS_Type) SetCLK_SYS_RESUS_CTRL_FRCE(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_RESUS_CTRL.Reg, volatile.LoadUint32(&o.CLK_SYS_RESUS_CTRL.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetCLK_SYS_RESUS_CTRL_FRCE() uint32 {
	return (volatile.LoadUint32(&o.CLK_SYS_RESUS_CTRL.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetCLK_SYS_RESUS_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_RESUS_CTRL.Reg, volatile.LoadUint32(&o.CLK_SYS_RESUS_CTRL.Reg)&^(0x100)|value<<8)
}
func (o *CLOCKS_Type) GetCLK_SYS_RESUS_CTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CLK_SYS_RESUS_CTRL.Reg) & 0x100) >> 8
}
func (o *CLOCKS_Type) SetCLK_SYS_RESUS_CTRL_TIMEOUT(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_RESUS_CTRL.Reg, volatile.LoadUint32(&o.CLK_SYS_RESUS_CTRL.Reg)&^(0xff)|value)
}
func (o *CLOCKS_Type) GetCLK_SYS_RESUS_CTRL_TIMEOUT() uint32 {
	return volatile.LoadUint32(&o.CLK_SYS_RESUS_CTRL.Reg) & 0xff
}

// CLOCKS.CLK_SYS_RESUS_STATUS
func (o *CLOCKS_Type) SetCLK_SYS_RESUS_STATUS_RESUSSED(value uint32) {
	volatile.StoreUint32(&o.CLK_SYS_RESUS_STATUS.Reg, volatile.LoadUint32(&o.CLK_SYS_RESUS_STATUS.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetCLK_SYS_RESUS_STATUS_RESUSSED() uint32 {
	return volatile.LoadUint32(&o.CLK_SYS_RESUS_STATUS.Reg) & 0x1
}

// CLOCKS.FC0_REF_KHZ: Reference clock frequency in kHz
func (o *CLOCKS_Type) SetFC0_REF_KHZ(value uint32) {
	volatile.StoreUint32(&o.FC0_REF_KHZ.Reg, volatile.LoadUint32(&o.FC0_REF_KHZ.Reg)&^(0xfffff)|value)
}
func (o *CLOCKS_Type) GetFC0_REF_KHZ() uint32 {
	return volatile.LoadUint32(&o.FC0_REF_KHZ.Reg) & 0xfffff
}

// CLOCKS.FC0_MIN_KHZ: Minimum pass frequency in kHz. This is optional. Set to 0 if you are not using the pass/fail flags
func (o *CLOCKS_Type) SetFC0_MIN_KHZ(value uint32) {
	volatile.StoreUint32(&o.FC0_MIN_KHZ.Reg, volatile.LoadUint32(&o.FC0_MIN_KHZ.Reg)&^(0x1ffffff)|value)
}
func (o *CLOCKS_Type) GetFC0_MIN_KHZ() uint32 {
	return volatile.LoadUint32(&o.FC0_MIN_KHZ.Reg) & 0x1ffffff
}

// CLOCKS.FC0_MAX_KHZ: Maximum pass frequency in kHz. This is optional. Set to 0x1ffffff if you are not using the pass/fail flags
func (o *CLOCKS_Type) SetFC0_MAX_KHZ(value uint32) {
	volatile.StoreUint32(&o.FC0_MAX_KHZ.Reg, volatile.LoadUint32(&o.FC0_MAX_KHZ.Reg)&^(0x1ffffff)|value)
}
func (o *CLOCKS_Type) GetFC0_MAX_KHZ() uint32 {
	return volatile.LoadUint32(&o.FC0_MAX_KHZ.Reg) & 0x1ffffff
}

// CLOCKS.FC0_DELAY: Delays the start of frequency counting to allow the mux to settle Delay is measured in multiples of the reference clock period
func (o *CLOCKS_Type) SetFC0_DELAY(value uint32) {
	volatile.StoreUint32(&o.FC0_DELAY.Reg, volatile.LoadUint32(&o.FC0_DELAY.Reg)&^(0x7)|value)
}
func (o *CLOCKS_Type) GetFC0_DELAY() uint32 {
	return volatile.LoadUint32(&o.FC0_DELAY.Reg) & 0x7
}

// CLOCKS.FC0_INTERVAL: The test interval is 0.98us * 2**interval, but let's call it 1us * 2**interval The default gives a test interval of 250us
func (o *CLOCKS_Type) SetFC0_INTERVAL(value uint32) {
	volatile.StoreUint32(&o.FC0_INTERVAL.Reg, volatile.LoadUint32(&o.FC0_INTERVAL.Reg)&^(0xf)|value)
}
func (o *CLOCKS_Type) GetFC0_INTERVAL() uint32 {
	return volatile.LoadUint32(&o.FC0_INTERVAL.Reg) & 0xf
}

// CLOCKS.FC0_SRC: Clock sent to frequency counter, set to 0 when not required Writing to this register initiates the frequency count
func (o *CLOCKS_Type) SetFC0_SRC(value uint32) {
	volatile.StoreUint32(&o.FC0_SRC.Reg, volatile.LoadUint32(&o.FC0_SRC.Reg)&^(0xff)|value)
}
func (o *CLOCKS_Type) GetFC0_SRC() uint32 {
	return volatile.LoadUint32(&o.FC0_SRC.Reg) & 0xff
}

// CLOCKS.FC0_STATUS: Frequency counter status
func (o *CLOCKS_Type) SetFC0_STATUS_DIED(value uint32) {
	volatile.StoreUint32(&o.FC0_STATUS.Reg, volatile.LoadUint32(&o.FC0_STATUS.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetFC0_STATUS_DIED() uint32 {
	return (volatile.LoadUint32(&o.FC0_STATUS.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetFC0_STATUS_FAST(value uint32) {
	volatile.StoreUint32(&o.FC0_STATUS.Reg, volatile.LoadUint32(&o.FC0_STATUS.Reg)&^(0x1000000)|value<<24)
}
func (o *CLOCKS_Type) GetFC0_STATUS_FAST() uint32 {
	return (volatile.LoadUint32(&o.FC0_STATUS.Reg) & 0x1000000) >> 24
}
func (o *CLOCKS_Type) SetFC0_STATUS_SLOW(value uint32) {
	volatile.StoreUint32(&o.FC0_STATUS.Reg, volatile.LoadUint32(&o.FC0_STATUS.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetFC0_STATUS_SLOW() uint32 {
	return (volatile.LoadUint32(&o.FC0_STATUS.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetFC0_STATUS_FAIL(value uint32) {
	volatile.StoreUint32(&o.FC0_STATUS.Reg, volatile.LoadUint32(&o.FC0_STATUS.Reg)&^(0x10000)|value<<16)
}
func (o *CLOCKS_Type) GetFC0_STATUS_FAIL() uint32 {
	return (volatile.LoadUint32(&o.FC0_STATUS.Reg) & 0x10000) >> 16
}
func (o *CLOCKS_Type) SetFC0_STATUS_WAITING(value uint32) {
	volatile.StoreUint32(&o.FC0_STATUS.Reg, volatile.LoadUint32(&o.FC0_STATUS.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetFC0_STATUS_WAITING() uint32 {
	return (volatile.LoadUint32(&o.FC0_STATUS.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetFC0_STATUS_RUNNING(value uint32) {
	volatile.StoreUint32(&o.FC0_STATUS.Reg, volatile.LoadUint32(&o.FC0_STATUS.Reg)&^(0x100)|value<<8)
}
func (o *CLOCKS_Type) GetFC0_STATUS_RUNNING() uint32 {
	return (volatile.LoadUint32(&o.FC0_STATUS.Reg) & 0x100) >> 8
}
func (o *CLOCKS_Type) SetFC0_STATUS_DONE(value uint32) {
	volatile.StoreUint32(&o.FC0_STATUS.Reg, volatile.LoadUint32(&o.FC0_STATUS.Reg)&^(0x10)|value<<4)
}
func (o *CLOCKS_Type) GetFC0_STATUS_DONE() uint32 {
	return (volatile.LoadUint32(&o.FC0_STATUS.Reg) & 0x10) >> 4
}
func (o *CLOCKS_Type) SetFC0_STATUS_PASS(value uint32) {
	volatile.StoreUint32(&o.FC0_STATUS.Reg, volatile.LoadUint32(&o.FC0_STATUS.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetFC0_STATUS_PASS() uint32 {
	return volatile.LoadUint32(&o.FC0_STATUS.Reg) & 0x1
}

// CLOCKS.FC0_RESULT: Result of frequency measurement, only valid when status_done=1
func (o *CLOCKS_Type) SetFC0_RESULT_KHZ(value uint32) {
	volatile.StoreUint32(&o.FC0_RESULT.Reg, volatile.LoadUint32(&o.FC0_RESULT.Reg)&^(0x3fffffe0)|value<<5)
}
func (o *CLOCKS_Type) GetFC0_RESULT_KHZ() uint32 {
	return (volatile.LoadUint32(&o.FC0_RESULT.Reg) & 0x3fffffe0) >> 5
}
func (o *CLOCKS_Type) SetFC0_RESULT_FRAC(value uint32) {
	volatile.StoreUint32(&o.FC0_RESULT.Reg, volatile.LoadUint32(&o.FC0_RESULT.Reg)&^(0x1f)|value)
}
func (o *CLOCKS_Type) GetFC0_RESULT_FRAC() uint32 {
	return volatile.LoadUint32(&o.FC0_RESULT.Reg) & 0x1f
}

// CLOCKS.WAKE_EN0: enable clock in wake mode
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_SIO(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x80000000)|value<<31)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_SIO() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x80000000) >> 31
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_SHA256(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x40000000)|value<<30)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_SHA256() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x40000000) >> 30
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_PSM(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x20000000)|value<<29)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_PSM() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x20000000) >> 29
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_ROSC(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_ROSC() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_ROM(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x8000000)|value<<27)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_ROM() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x8000000) >> 27
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_RESETS(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x4000000)|value<<26)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_RESETS() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x4000000) >> 26
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_PWM(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x2000000)|value<<25)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_PWM() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x2000000) >> 25
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_POWMAN(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x1000000)|value<<24)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_POWMAN() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x1000000) >> 24
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_REF_POWMAN(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x800000)|value<<23)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_REF_POWMAN() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x800000) >> 23
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_PLL_USB(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x400000)|value<<22)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_PLL_USB() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x400000) >> 22
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_PLL_SYS(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x200000)|value<<21)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_PLL_SYS() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x200000) >> 21
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_PIO2(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_PIO2() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_PIO1(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x80000)|value<<19)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_PIO1() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x80000) >> 19
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_PIO0(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x40000)|value<<18)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_PIO0() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x40000) >> 18
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_PADS(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x20000)|value<<17)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_PADS() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x20000) >> 17
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_OTP(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x10000)|value<<16)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_OTP() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x10000) >> 16
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_REF_OTP(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x8000)|value<<15)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_REF_OTP() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x8000) >> 15
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_JTAG(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x4000)|value<<14)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_JTAG() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x4000) >> 14
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_IO(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x2000)|value<<13)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_IO() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x2000) >> 13
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_I2C1(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_I2C1() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_I2C0(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_I2C0() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_HSTX(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_HSTX() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_HSTX(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x200)|value<<9)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_HSTX() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x200) >> 9
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_GLITCH_DETECTOR(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x100)|value<<8)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_GLITCH_DETECTOR() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x100) >> 8
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_DMA(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x80)|value<<7)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_DMA() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x80) >> 7
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_BUSFABRIC(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x40)|value<<6)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_BUSFABRIC() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x40) >> 6
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_BUSCTRL(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x20)|value<<5)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_BUSCTRL() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x20) >> 5
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_BOOTRAM(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x10)|value<<4)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_BOOTRAM() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x10) >> 4
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_ADC(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x8)|value<<3)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_ADC() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x8) >> 3
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_ADC(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x4)|value<<2)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_ADC() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x4) >> 2
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_ACCESSCTRL(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x2)|value<<1)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_ACCESSCTRL() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x2) >> 1
}
func (o *CLOCKS_Type) SetWAKE_EN0_CLK_SYS_CLOCKS(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN0.Reg, volatile.LoadUint32(&o.WAKE_EN0.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetWAKE_EN0_CLK_SYS_CLOCKS() uint32 {
	return volatile.LoadUint32(&o.WAKE_EN0.Reg) & 0x1
}

// CLOCKS.WAKE_EN1: enable clock in wake mode
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_XOSC(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x40000000)|value<<30)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_XOSC() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x40000000) >> 30
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_XIP(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x20000000)|value<<29)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_XIP() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x20000000) >> 29
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_WATCHDOG(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_WATCHDOG() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_USB(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x8000000)|value<<27)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_USB() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x8000000) >> 27
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_USBCTRL(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x4000000)|value<<26)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_USBCTRL() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x4000000) >> 26
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_UART1(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x2000000)|value<<25)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_UART1() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x2000000) >> 25
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_PERI_UART1(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x1000000)|value<<24)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_PERI_UART1() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x1000000) >> 24
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_UART0(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x800000)|value<<23)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_UART0() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x800000) >> 23
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_PERI_UART0(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x400000)|value<<22)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_PERI_UART0() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x400000) >> 22
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_TRNG(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x200000)|value<<21)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_TRNG() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x200000) >> 21
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_TIMER1(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_TIMER1() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_TIMER0(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x80000)|value<<19)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_TIMER0() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x80000) >> 19
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_TICKS(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x40000)|value<<18)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_TICKS() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x40000) >> 18
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_REF_TICKS(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x20000)|value<<17)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_REF_TICKS() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x20000) >> 17
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_TBMAN(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x10000)|value<<16)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_TBMAN() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x10000) >> 16
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SYSINFO(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x8000)|value<<15)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SYSINFO() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x8000) >> 15
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SYSCFG(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x4000)|value<<14)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SYSCFG() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x4000) >> 14
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM9(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x2000)|value<<13)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM9() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x2000) >> 13
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM8(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM8() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM7(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM7() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM6(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM6() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM5(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x200)|value<<9)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM5() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x200) >> 9
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM4(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x100)|value<<8)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM4() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x100) >> 8
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM3(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x80)|value<<7)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM3() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x80) >> 7
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM2(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x40)|value<<6)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM2() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x40) >> 6
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM1(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x20)|value<<5)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM1() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x20) >> 5
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SRAM0(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x10)|value<<4)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SRAM0() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x10) >> 4
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SPI1(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x8)|value<<3)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SPI1() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x8) >> 3
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_PERI_SPI1(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x4)|value<<2)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_PERI_SPI1() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x4) >> 2
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_SYS_SPI0(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x2)|value<<1)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_SYS_SPI0() uint32 {
	return (volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x2) >> 1
}
func (o *CLOCKS_Type) SetWAKE_EN1_CLK_PERI_SPI0(value uint32) {
	volatile.StoreUint32(&o.WAKE_EN1.Reg, volatile.LoadUint32(&o.WAKE_EN1.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetWAKE_EN1_CLK_PERI_SPI0() uint32 {
	return volatile.LoadUint32(&o.WAKE_EN1.Reg) & 0x1
}

// CLOCKS.SLEEP_EN0: enable clock in sleep mode
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_SIO(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x80000000)|value<<31)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_SIO() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x80000000) >> 31
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_SHA256(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x40000000)|value<<30)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_SHA256() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x40000000) >> 30
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_PSM(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x20000000)|value<<29)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_PSM() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x20000000) >> 29
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_ROSC(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_ROSC() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_ROM(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x8000000)|value<<27)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_ROM() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x8000000) >> 27
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_RESETS(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x4000000)|value<<26)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_RESETS() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x4000000) >> 26
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_PWM(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x2000000)|value<<25)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_PWM() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x2000000) >> 25
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_POWMAN(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x1000000)|value<<24)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_POWMAN() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x1000000) >> 24
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_REF_POWMAN(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x800000)|value<<23)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_REF_POWMAN() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x800000) >> 23
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_PLL_USB(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x400000)|value<<22)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_PLL_USB() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x400000) >> 22
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_PLL_SYS(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x200000)|value<<21)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_PLL_SYS() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x200000) >> 21
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_PIO2(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_PIO2() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_PIO1(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x80000)|value<<19)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_PIO1() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x80000) >> 19
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_PIO0(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x40000)|value<<18)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_PIO0() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x40000) >> 18
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_PADS(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x20000)|value<<17)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_PADS() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x20000) >> 17
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_OTP(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x10000)|value<<16)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_OTP() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x10000) >> 16
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_REF_OTP(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x8000)|value<<15)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_REF_OTP() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x8000) >> 15
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_JTAG(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x4000)|value<<14)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_JTAG() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x4000) >> 14
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_IO(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x2000)|value<<13)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_IO() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x2000) >> 13
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_I2C1(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_I2C1() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_I2C0(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_I2C0() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_HSTX(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_HSTX() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_HSTX(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x200)|value<<9)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_HSTX() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x200) >> 9
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_GLITCH_DETECTOR(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x100)|value<<8)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_GLITCH_DETECTOR() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x100) >> 8
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_DMA(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x80)|value<<7)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_DMA() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x80) >> 7
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_BUSFABRIC(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x40)|value<<6)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_BUSFABRIC() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x40) >> 6
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_BUSCTRL(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x20)|value<<5)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_BUSCTRL() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x20) >> 5
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_BOOTRAM(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x10)|value<<4)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_BOOTRAM() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x10) >> 4
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_ADC(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x8)|value<<3)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_ADC() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x8) >> 3
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_ADC(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x4)|value<<2)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_ADC() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x4) >> 2
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_ACCESSCTRL(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x2)|value<<1)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_ACCESSCTRL() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x2) >> 1
}
func (o *CLOCKS_Type) SetSLEEP_EN0_CLK_SYS_CLOCKS(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN0.Reg, volatile.LoadUint32(&o.SLEEP_EN0.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetSLEEP_EN0_CLK_SYS_CLOCKS() uint32 {
	return volatile.LoadUint32(&o.SLEEP_EN0.Reg) & 0x1
}

// CLOCKS.SLEEP_EN1: enable clock in sleep mode
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_XOSC(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x40000000)|value<<30)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_XOSC() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x40000000) >> 30
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_XIP(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x20000000)|value<<29)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_XIP() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x20000000) >> 29
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_WATCHDOG(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_WATCHDOG() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_USB(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x8000000)|value<<27)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_USB() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x8000000) >> 27
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_USBCTRL(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x4000000)|value<<26)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_USBCTRL() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x4000000) >> 26
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_UART1(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x2000000)|value<<25)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_UART1() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x2000000) >> 25
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_PERI_UART1(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x1000000)|value<<24)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_PERI_UART1() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x1000000) >> 24
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_UART0(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x800000)|value<<23)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_UART0() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x800000) >> 23
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_PERI_UART0(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x400000)|value<<22)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_PERI_UART0() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x400000) >> 22
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_TRNG(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x200000)|value<<21)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_TRNG() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x200000) >> 21
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_TIMER1(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_TIMER1() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_TIMER0(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x80000)|value<<19)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_TIMER0() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x80000) >> 19
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_TICKS(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x40000)|value<<18)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_TICKS() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x40000) >> 18
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_REF_TICKS(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x20000)|value<<17)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_REF_TICKS() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x20000) >> 17
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_TBMAN(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x10000)|value<<16)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_TBMAN() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x10000) >> 16
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SYSINFO(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x8000)|value<<15)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SYSINFO() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x8000) >> 15
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SYSCFG(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x4000)|value<<14)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SYSCFG() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x4000) >> 14
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM9(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x2000)|value<<13)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM9() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x2000) >> 13
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM8(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM8() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM7(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM7() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM6(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM6() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM5(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x200)|value<<9)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM5() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x200) >> 9
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM4(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x100)|value<<8)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM4() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x100) >> 8
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM3(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x80)|value<<7)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM3() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x80) >> 7
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM2(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x40)|value<<6)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM2() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x40) >> 6
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM1(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x20)|value<<5)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM1() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x20) >> 5
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SRAM0(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x10)|value<<4)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SRAM0() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x10) >> 4
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SPI1(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x8)|value<<3)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SPI1() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x8) >> 3
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_PERI_SPI1(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x4)|value<<2)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_PERI_SPI1() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x4) >> 2
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_SYS_SPI0(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x2)|value<<1)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_SYS_SPI0() uint32 {
	return (volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x2) >> 1
}
func (o *CLOCKS_Type) SetSLEEP_EN1_CLK_PERI_SPI0(value uint32) {
	volatile.StoreUint32(&o.SLEEP_EN1.Reg, volatile.LoadUint32(&o.SLEEP_EN1.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetSLEEP_EN1_CLK_PERI_SPI0() uint32 {
	return volatile.LoadUint32(&o.SLEEP_EN1.Reg) & 0x1
}

// CLOCKS.ENABLED0: indicates the state of the clock enable
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_SIO(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x80000000)|value<<31)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_SIO() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x80000000) >> 31
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_SHA256(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x40000000)|value<<30)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_SHA256() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x40000000) >> 30
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_PSM(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x20000000)|value<<29)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_PSM() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x20000000) >> 29
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_ROSC(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_ROSC() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_ROM(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x8000000)|value<<27)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_ROM() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x8000000) >> 27
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_RESETS(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x4000000)|value<<26)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_RESETS() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x4000000) >> 26
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_PWM(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x2000000)|value<<25)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_PWM() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x2000000) >> 25
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_POWMAN(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x1000000)|value<<24)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_POWMAN() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x1000000) >> 24
}
func (o *CLOCKS_Type) SetENABLED0_CLK_REF_POWMAN(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x800000)|value<<23)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_REF_POWMAN() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x800000) >> 23
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_PLL_USB(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x400000)|value<<22)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_PLL_USB() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x400000) >> 22
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_PLL_SYS(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x200000)|value<<21)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_PLL_SYS() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x200000) >> 21
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_PIO2(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_PIO2() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_PIO1(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x80000)|value<<19)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_PIO1() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x80000) >> 19
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_PIO0(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x40000)|value<<18)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_PIO0() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x40000) >> 18
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_PADS(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x20000)|value<<17)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_PADS() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x20000) >> 17
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_OTP(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x10000)|value<<16)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_OTP() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x10000) >> 16
}
func (o *CLOCKS_Type) SetENABLED0_CLK_REF_OTP(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x8000)|value<<15)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_REF_OTP() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x8000) >> 15
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_JTAG(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x4000)|value<<14)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_JTAG() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x4000) >> 14
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_IO(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x2000)|value<<13)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_IO() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x2000) >> 13
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_I2C1(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_I2C1() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_I2C0(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_I2C0() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_HSTX(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_HSTX() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetENABLED0_CLK_HSTX(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x200)|value<<9)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_HSTX() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x200) >> 9
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_GLITCH_DETECTOR(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x100)|value<<8)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_GLITCH_DETECTOR() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x100) >> 8
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_DMA(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x80)|value<<7)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_DMA() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x80) >> 7
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_BUSFABRIC(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x40)|value<<6)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_BUSFABRIC() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x40) >> 6
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_BUSCTRL(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x20)|value<<5)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_BUSCTRL() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x20) >> 5
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_BOOTRAM(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x10)|value<<4)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_BOOTRAM() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x10) >> 4
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_ADC(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x8)|value<<3)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_ADC() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x8) >> 3
}
func (o *CLOCKS_Type) SetENABLED0_CLK_ADC(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x4)|value<<2)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_ADC() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x4) >> 2
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_ACCESSCTRL(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x2)|value<<1)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_ACCESSCTRL() uint32 {
	return (volatile.LoadUint32(&o.ENABLED0.Reg) & 0x2) >> 1
}
func (o *CLOCKS_Type) SetENABLED0_CLK_SYS_CLOCKS(value uint32) {
	volatile.StoreUint32(&o.ENABLED0.Reg, volatile.LoadUint32(&o.ENABLED0.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetENABLED0_CLK_SYS_CLOCKS() uint32 {
	return volatile.LoadUint32(&o.ENABLED0.Reg) & 0x1
}

// CLOCKS.ENABLED1: indicates the state of the clock enable
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_XOSC(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x40000000)|value<<30)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_XOSC() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x40000000) >> 30
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_XIP(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x20000000)|value<<29)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_XIP() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x20000000) >> 29
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_WATCHDOG(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x10000000)|value<<28)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_WATCHDOG() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x10000000) >> 28
}
func (o *CLOCKS_Type) SetENABLED1_CLK_USB(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x8000000)|value<<27)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_USB() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x8000000) >> 27
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_USBCTRL(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x4000000)|value<<26)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_USBCTRL() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x4000000) >> 26
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_UART1(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x2000000)|value<<25)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_UART1() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x2000000) >> 25
}
func (o *CLOCKS_Type) SetENABLED1_CLK_PERI_UART1(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x1000000)|value<<24)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_PERI_UART1() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x1000000) >> 24
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_UART0(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x800000)|value<<23)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_UART0() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x800000) >> 23
}
func (o *CLOCKS_Type) SetENABLED1_CLK_PERI_UART0(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x400000)|value<<22)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_PERI_UART0() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x400000) >> 22
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_TRNG(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x200000)|value<<21)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_TRNG() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x200000) >> 21
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_TIMER1(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x100000)|value<<20)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_TIMER1() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x100000) >> 20
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_TIMER0(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x80000)|value<<19)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_TIMER0() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x80000) >> 19
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_TICKS(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x40000)|value<<18)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_TICKS() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x40000) >> 18
}
func (o *CLOCKS_Type) SetENABLED1_CLK_REF_TICKS(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x20000)|value<<17)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_REF_TICKS() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x20000) >> 17
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_TBMAN(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x10000)|value<<16)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_TBMAN() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x10000) >> 16
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SYSINFO(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x8000)|value<<15)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SYSINFO() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x8000) >> 15
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SYSCFG(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x4000)|value<<14)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SYSCFG() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x4000) >> 14
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM9(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x2000)|value<<13)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM9() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x2000) >> 13
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM8(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x1000)|value<<12)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM8() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x1000) >> 12
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM7(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x800)|value<<11)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM7() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x800) >> 11
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM6(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x400)|value<<10)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM6() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x400) >> 10
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM5(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x200)|value<<9)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM5() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x200) >> 9
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM4(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x100)|value<<8)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM4() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x100) >> 8
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM3(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x80)|value<<7)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM3() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x80) >> 7
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM2(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x40)|value<<6)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM2() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x40) >> 6
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM1(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x20)|value<<5)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM1() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x20) >> 5
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SRAM0(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x10)|value<<4)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SRAM0() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x10) >> 4
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SPI1(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x8)|value<<3)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SPI1() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x8) >> 3
}
func (o *CLOCKS_Type) SetENABLED1_CLK_PERI_SPI1(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x4)|value<<2)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_PERI_SPI1() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x4) >> 2
}
func (o *CLOCKS_Type) SetENABLED1_CLK_SYS_SPI0(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x2)|value<<1)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_SYS_SPI0() uint32 {
	return (volatile.LoadUint32(&o.ENABLED1.Reg) & 0x2) >> 1
}
func (o *CLOCKS_Type) SetENABLED1_CLK_PERI_SPI0(value uint32) {
	volatile.StoreUint32(&o.ENABLED1.Reg, volatile.LoadUint32(&o.ENABLED1.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetENABLED1_CLK_PERI_SPI0() uint32 {
	return volatile.LoadUint32(&o.ENABLED1.Reg) & 0x1
}

// CLOCKS.INTR: Raw Interrupts
func (o *CLOCKS_Type) SetINTR_CLK_SYS_RESUS(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetINTR_CLK_SYS_RESUS() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// CLOCKS.INTE: Interrupt Enable
func (o *CLOCKS_Type) SetINTE_CLK_SYS_RESUS(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetINTE_CLK_SYS_RESUS() uint32 {
	return volatile.LoadUint32(&o.INTE.Reg) & 0x1
}

// CLOCKS.INTF: Interrupt Force
func (o *CLOCKS_Type) SetINTF_CLK_SYS_RESUS(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetINTF_CLK_SYS_RESUS() uint32 {
	return volatile.LoadUint32(&o.INTF.Reg) & 0x1
}

// CLOCKS.INTS: Interrupt status after masking & forcing
func (o *CLOCKS_Type) SetINTS_CLK_SYS_RESUS(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x1)|value)
}
func (o *CLOCKS_Type) GetINTS_CLK_SYS_RESUS() uint32 {
	return volatile.LoadUint32(&o.INTS.Reg) & 0x1
}

type TICKS_Type struct {
	PROC0_CTRL      volatile.Register32 // 0x0
	PROC0_CYCLES    volatile.Register32 // 0x4
	PROC0_COUNT     volatile.Register32 // 0x8
	PROC1_CTRL      volatile.Register32 // 0xC
	PROC1_CYCLES    volatile.Register32 // 0x10
	PROC1_COUNT     volatile.Register32 // 0x14
	TIMER0_CTRL     volatile.Register32 // 0x18
	TIMER0_CYCLES   volatile.Register32 // 0x1C
	TIMER0_COUNT    volatile.Register32 // 0x20
	TIMER1_CTRL     volatile.Register32 // 0x24
	TIMER1_CYCLES   volatile.Register32 // 0x28
	TIMER1_COUNT    volatile.Register32 // 0x2C
	WATCHDOG_CTRL   volatile.Register32 // 0x30
	WATCHDOG_CYCLES volatile.Register32 // 0x34
	WATCHDOG_COUNT  volatile.Register32 // 0x38
	RISCV_CTRL      volatile.Register32 // 0x3C
	RISCV_CYCLES    volatile.Register32 // 0x40
	RISCV_COUNT     volatile.Register32 // 0x44
}

// TICKS.PROC0_CTRL: Controls the tick generator
func (o *TICKS_Type) SetPROC0_CTRL_RUNNING(value uint32) {
	volatile.StoreUint32(&o.PROC0_CTRL.Reg, volatile.LoadUint32(&o.PROC0_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *TICKS_Type) GetPROC0_CTRL_RUNNING() uint32 {
	return (volatile.LoadUint32(&o.PROC0_CTRL.Reg) & 0x2) >> 1
}
func (o *TICKS_Type) SetPROC0_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.PROC0_CTRL.Reg, volatile.LoadUint32(&o.PROC0_CTRL.Reg)&^(0x1)|value)
}
func (o *TICKS_Type) GetPROC0_CTRL_ENABLE() uint32 {
	return volatile.LoadUint32(&o.PROC0_CTRL.Reg) & 0x1
}

// TICKS.PROC0_CYCLES
func (o *TICKS_Type) SetPROC0_CYCLES(value uint32) {
	volatile.StoreUint32(&o.PROC0_CYCLES.Reg, volatile.LoadUint32(&o.PROC0_CYCLES.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetPROC0_CYCLES() uint32 {
	return volatile.LoadUint32(&o.PROC0_CYCLES.Reg) & 0x1ff
}

// TICKS.PROC0_COUNT
func (o *TICKS_Type) SetPROC0_COUNT(value uint32) {
	volatile.StoreUint32(&o.PROC0_COUNT.Reg, volatile.LoadUint32(&o.PROC0_COUNT.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetPROC0_COUNT() uint32 {
	return volatile.LoadUint32(&o.PROC0_COUNT.Reg) & 0x1ff
}

// TICKS.PROC1_CTRL: Controls the tick generator
func (o *TICKS_Type) SetPROC1_CTRL_RUNNING(value uint32) {
	volatile.StoreUint32(&o.PROC1_CTRL.Reg, volatile.LoadUint32(&o.PROC1_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *TICKS_Type) GetPROC1_CTRL_RUNNING() uint32 {
	return (volatile.LoadUint32(&o.PROC1_CTRL.Reg) & 0x2) >> 1
}
func (o *TICKS_Type) SetPROC1_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.PROC1_CTRL.Reg, volatile.LoadUint32(&o.PROC1_CTRL.Reg)&^(0x1)|value)
}
func (o *TICKS_Type) GetPROC1_CTRL_ENABLE() uint32 {
	return volatile.LoadUint32(&o.PROC1_CTRL.Reg) & 0x1
}

// TICKS.PROC1_CYCLES
func (o *TICKS_Type) SetPROC1_CYCLES(value uint32) {
	volatile.StoreUint32(&o.PROC1_CYCLES.Reg, volatile.LoadUint32(&o.PROC1_CYCLES.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetPROC1_CYCLES() uint32 {
	return volatile.LoadUint32(&o.PROC1_CYCLES.Reg) & 0x1ff
}

// TICKS.PROC1_COUNT
func (o *TICKS_Type) SetPROC1_COUNT(value uint32) {
	volatile.StoreUint32(&o.PROC1_COUNT.Reg, volatile.LoadUint32(&o.PROC1_COUNT.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetPROC1_COUNT() uint32 {
	return volatile.LoadUint32(&o.PROC1_COUNT.Reg) & 0x1ff
}

// TICKS.TIMER0_CTRL: Controls the tick generator
func (o *TICKS_Type) SetTIMER0_CTRL_RUNNING(value uint32) {
	volatile.StoreUint32(&o.TIMER0_CTRL.Reg, volatile.LoadUint32(&o.TIMER0_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *TICKS_Type) GetTIMER0_CTRL_RUNNING() uint32 {
	return (volatile.LoadUint32(&o.TIMER0_CTRL.Reg) & 0x2) >> 1
}
func (o *TICKS_Type) SetTIMER0_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.TIMER0_CTRL.Reg, volatile.LoadUint32(&o.TIMER0_CTRL.Reg)&^(0x1)|value)
}
func (o *TICKS_Type) GetTIMER0_CTRL_ENABLE() uint32 {
	return volatile.LoadUint32(&o.TIMER0_CTRL.Reg) & 0x1
}

// TICKS.TIMER0_CYCLES
func (o *TICKS_Type) SetTIMER0_CYCLES(value uint32) {
	volatile.StoreUint32(&o.TIMER0_CYCLES.Reg, volatile.LoadUint32(&o.TIMER0_CYCLES.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetTIMER0_CYCLES() uint32 {
	return volatile.LoadUint32(&o.TIMER0_CYCLES.Reg) & 0x1ff
}

// TICKS.TIMER0_COUNT
func (o *TICKS_Type) SetTIMER0_COUNT(value uint32) {
	volatile.StoreUint32(&o.TIMER0_COUNT.Reg, volatile.LoadUint32(&o.TIMER0_COUNT.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetTIMER0_COUNT() uint32 {
	return volatile.LoadUint32(&o.TIMER0_COUNT.Reg) & 0x1ff
}

// TICKS.TIMER1_CTRL: Controls the tick generator
func (o *TICKS_Type) SetTIMER1_CTRL_RUNNING(value uint32) {
	volatile.StoreUint32(&o.TIMER1_CTRL.Reg, volatile.LoadUint32(&o.TIMER1_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *TICKS_Type) GetTIMER1_CTRL_RUNNING() uint32 {
	return (volatile.LoadUint32(&o.TIMER1_CTRL.Reg) & 0x2) >> 1
}
func (o *TICKS_Type) SetTIMER1_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.TIMER1_CTRL.Reg, volatile.LoadUint32(&o.TIMER1_CTRL.Reg)&^(0x1)|value)
}
func (o *TICKS_Type) GetTIMER1_CTRL_ENABLE() uint32 {
	return volatile.LoadUint32(&o.TIMER1_CTRL.Reg) & 0x1
}

// TICKS.TIMER1_CYCLES
func (o *TICKS_Type) SetTIMER1_CYCLES(value uint32) {
	volatile.StoreUint32(&o.TIMER1_CYCLES.Reg, volatile.LoadUint32(&o.TIMER1_CYCLES.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetTIMER1_CYCLES() uint32 {
	return volatile.LoadUint32(&o.TIMER1_CYCLES.Reg) & 0x1ff
}

// TICKS.TIMER1_COUNT
func (o *TICKS_Type) SetTIMER1_COUNT(value uint32) {
	volatile.StoreUint32(&o.TIMER1_COUNT.Reg, volatile.LoadUint32(&o.TIMER1_COUNT.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetTIMER1_COUNT() uint32 {
	return volatile.LoadUint32(&o.TIMER1_COUNT.Reg) & 0x1ff
}

// TICKS.WATCHDOG_CTRL: Controls the tick generator
func (o *TICKS_Type) SetWATCHDOG_CTRL_RUNNING(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG_CTRL.Reg, volatile.LoadUint32(&o.WATCHDOG_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *TICKS_Type) GetWATCHDOG_CTRL_RUNNING() uint32 {
	return (volatile.LoadUint32(&o.WATCHDOG_CTRL.Reg) & 0x2) >> 1
}
func (o *TICKS_Type) SetWATCHDOG_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG_CTRL.Reg, volatile.LoadUint32(&o.WATCHDOG_CTRL.Reg)&^(0x1)|value)
}
func (o *TICKS_Type) GetWATCHDOG_CTRL_ENABLE() uint32 {
	return volatile.LoadUint32(&o.WATCHDOG_CTRL.Reg) & 0x1
}

// TICKS.WATCHDOG_CYCLES
func (o *TICKS_Type) SetWATCHDOG_CYCLES(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG_CYCLES.Reg, volatile.LoadUint32(&o.WATCHDOG_CYCLES.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetWATCHDOG_CYCLES() uint32 {
	return volatile.LoadUint32(&o.WATCHDOG_CYCLES.Reg) & 0x1ff
}

// TICKS.WATCHDOG_COUNT
func (o *TICKS_Type) SetWATCHDOG_COUNT(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG_COUNT.Reg, volatile.LoadUint32(&o.WATCHDOG_COUNT.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetWATCHDOG_COUNT() uint32 {
	return volatile.LoadUint32(&o.WATCHDOG_COUNT.Reg) & 0x1ff
}

// TICKS.RISCV_CTRL: Controls the tick generator
func (o *TICKS_Type) SetRISCV_CTRL_RUNNING(value uint32) {
	volatile.StoreUint32(&o.RISCV_CTRL.Reg, volatile.LoadUint32(&o.RISCV_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *TICKS_Type) GetRISCV_CTRL_RUNNING() uint32 {
	return (volatile.LoadUint32(&o.RISCV_CTRL.Reg) & 0x2) >> 1
}
func (o *TICKS_Type) SetRISCV_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.RISCV_CTRL.Reg, volatile.LoadUint32(&o.RISCV_CTRL.Reg)&^(0x1)|value)
}
func (o *TICKS_Type) GetRISCV_CTRL_ENABLE() uint32 {
	return volatile.LoadUint32(&o.RISCV_CTRL.Reg) & 0x1
}

// TICKS.RISCV_CYCLES
func (o *TICKS_Type) SetRISCV_CYCLES(value uint32) {
	volatile.StoreUint32(&o.RISCV_CYCLES.Reg, volatile.LoadUint32(&o.RISCV_CYCLES.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetRISCV_CYCLES() uint32 {
	return volatile.LoadUint32(&o.RISCV_CYCLES.Reg) & 0x1ff
}

// TICKS.RISCV_COUNT
func (o *TICKS_Type) SetRISCV_COUNT(value uint32) {
	volatile.StoreUint32(&o.RISCV_COUNT.Reg, volatile.LoadUint32(&o.RISCV_COUNT.Reg)&^(0x1ff)|value)
}
func (o *TICKS_Type) GetRISCV_COUNT() uint32 {
	return volatile.LoadUint32(&o.RISCV_COUNT.Reg) & 0x1ff
}

type PADS_BANK0_Type struct {
	VOLTAGE_SELECT volatile.Register32 // 0x0
	GPIO0          volatile.Register32 // 0x4
	GPIO1          volatile.Register32 // 0x8
	GPIO2          volatile.Register32 // 0xC
	GPIO3          volatile.Register32 // 0x10
	GPIO4          volatile.Register32 // 0x14
	GPIO5          volatile.Register32 // 0x18
	GPIO6          volatile.Register32 // 0x1C
	GPIO7          volatile.Register32 // 0x20
	GPIO8          volatile.Register32 // 0x24
	GPIO9          volatile.Register32 // 0x28
	GPIO10         volatile.Register32 // 0x2C
	GPIO11         volatile.Register32 // 0x30
	GPIO12         volatile.Register32 // 0x34
	GPIO13         volatile.Register32 // 0x38
	GPIO14         volatile.Register32 // 0x3C
	GPIO15         volatile.Register32 // 0x40
	GPIO16         volatile.Register32 // 0x44
	GPIO17         volatile.Register32 // 0x48
	GPIO18         volatile.Register32 // 0x4C
	GPIO19         volatile.Register32 // 0x50
	GPIO20         volatile.Register32 // 0x54
	GPIO21         volatile.Register32 // 0x58
	GPIO22         volatile.Register32 // 0x5C
	GPIO23         volatile.Register32 // 0x60
	GPIO24         volatile.Register32 // 0x64
	GPIO25         volatile.Register32 // 0x68
	GPIO26         volatile.Register32 // 0x6C
	GPIO27         volatile.Register32 // 0x70
	GPIO28         volatile.Register32 // 0x74
	GPIO29         volatile.Register32 // 0x78
	GPIO30         volatile.Register32 // 0x7C
	GPIO31         volatile.Register32 // 0x80
	GPIO32         volatile.Register32 // 0x84
	GPIO33         volatile.Register32 // 0x88
	GPIO34         volatile.Register32 // 0x8C
	GPIO35         volatile.Register32 // 0x90
	GPIO36         volatile.Register32 // 0x94
	GPIO37         volatile.Register32 // 0x98
	GPIO38         volatile.Register32 // 0x9C
	GPIO39         volatile.Register32 // 0xA0
	GPIO40         volatile.Register32 // 0xA4
	GPIO41         volatile.Register32 // 0xA8
	GPIO42         volatile.Register32 // 0xAC
	GPIO43         volatile.Register32 // 0xB0
	GPIO44         volatile.Register32 // 0xB4
	GPIO45         volatile.Register32 // 0xB8
	GPIO46         volatile.Register32 // 0xBC
	GPIO47         volatile.Register32 // 0xC0
	SWCLK          volatile.Register32 // 0xC4
	SWD            volatile.Register32 // 0xC8
}

// PADS_BANK0.VOLTAGE_SELECT: Voltage select. Per bank control
func (o *PADS_BANK0_Type) SetVOLTAGE_SELECT(value uint32) {
	volatile.StoreUint32(&o.VOLTAGE_SELECT.Reg, volatile.LoadUint32(&o.VOLTAGE_SELECT.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetVOLTAGE_SELECT() uint32 {
	return volatile.LoadUint32(&o.VOLTAGE_SELECT.Reg) & 0x1
}

// PADS_BANK0.GPIO0
func (o *PADS_BANK0_Type) SetGPIO0_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO0.Reg, volatile.LoadUint32(&o.GPIO0.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO0_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO0.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO0_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO0.Reg, volatile.LoadUint32(&o.GPIO0.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO0_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO0.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO0_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO0.Reg, volatile.LoadUint32(&o.GPIO0.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO0_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO0.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO0_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO0.Reg, volatile.LoadUint32(&o.GPIO0.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO0_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO0.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO0_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO0.Reg, volatile.LoadUint32(&o.GPIO0.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO0_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO0.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO0_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO0.Reg, volatile.LoadUint32(&o.GPIO0.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO0_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO0.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO0_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO0.Reg, volatile.LoadUint32(&o.GPIO0.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO0_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO0.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO0_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO0.Reg, volatile.LoadUint32(&o.GPIO0.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO0_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO0.Reg) & 0x1
}

// PADS_BANK0.GPIO1
func (o *PADS_BANK0_Type) SetGPIO1_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO1.Reg, volatile.LoadUint32(&o.GPIO1.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO1_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO1.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO1_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO1.Reg, volatile.LoadUint32(&o.GPIO1.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO1_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO1.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO1_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO1.Reg, volatile.LoadUint32(&o.GPIO1.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO1_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO1.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO1_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO1.Reg, volatile.LoadUint32(&o.GPIO1.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO1_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO1.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO1_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO1.Reg, volatile.LoadUint32(&o.GPIO1.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO1_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO1.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO1_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO1.Reg, volatile.LoadUint32(&o.GPIO1.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO1_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO1.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO1_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO1.Reg, volatile.LoadUint32(&o.GPIO1.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO1_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO1.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO1_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO1.Reg, volatile.LoadUint32(&o.GPIO1.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO1_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO1.Reg) & 0x1
}

// PADS_BANK0.GPIO2
func (o *PADS_BANK0_Type) SetGPIO2_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO2.Reg, volatile.LoadUint32(&o.GPIO2.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO2_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO2.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO2_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO2.Reg, volatile.LoadUint32(&o.GPIO2.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO2_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO2.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO2_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO2.Reg, volatile.LoadUint32(&o.GPIO2.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO2_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO2.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO2_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO2.Reg, volatile.LoadUint32(&o.GPIO2.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO2_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO2.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO2_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO2.Reg, volatile.LoadUint32(&o.GPIO2.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO2_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO2.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO2_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO2.Reg, volatile.LoadUint32(&o.GPIO2.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO2_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO2.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO2_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO2.Reg, volatile.LoadUint32(&o.GPIO2.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO2_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO2.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO2_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO2.Reg, volatile.LoadUint32(&o.GPIO2.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO2_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO2.Reg) & 0x1
}

// PADS_BANK0.GPIO3
func (o *PADS_BANK0_Type) SetGPIO3_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO3.Reg, volatile.LoadUint32(&o.GPIO3.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO3_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO3.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO3_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO3.Reg, volatile.LoadUint32(&o.GPIO3.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO3_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO3.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO3_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO3.Reg, volatile.LoadUint32(&o.GPIO3.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO3_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO3.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO3_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO3.Reg, volatile.LoadUint32(&o.GPIO3.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO3_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO3.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO3_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO3.Reg, volatile.LoadUint32(&o.GPIO3.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO3_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO3.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO3_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO3.Reg, volatile.LoadUint32(&o.GPIO3.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO3_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO3.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO3_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO3.Reg, volatile.LoadUint32(&o.GPIO3.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO3_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO3.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO3_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO3.Reg, volatile.LoadUint32(&o.GPIO3.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO3_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO3.Reg) & 0x1
}

// PADS_BANK0.GPIO4
func (o *PADS_BANK0_Type) SetGPIO4_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO4.Reg, volatile.LoadUint32(&o.GPIO4.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO4_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO4.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO4_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO4.Reg, volatile.LoadUint32(&o.GPIO4.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO4_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO4.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO4_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO4.Reg, volatile.LoadUint32(&o.GPIO4.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO4_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO4.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO4_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO4.Reg, volatile.LoadUint32(&o.GPIO4.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO4_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO4.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO4_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO4.Reg, volatile.LoadUint32(&o.GPIO4.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO4_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO4.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO4_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO4.Reg, volatile.LoadUint32(&o.GPIO4.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO4_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO4.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO4_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO4.Reg, volatile.LoadUint32(&o.GPIO4.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO4_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO4.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO4_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO4.Reg, volatile.LoadUint32(&o.GPIO4.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO4_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO4.Reg) & 0x1
}

// PADS_BANK0.GPIO5
func (o *PADS_BANK0_Type) SetGPIO5_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO5.Reg, volatile.LoadUint32(&o.GPIO5.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO5_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO5.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO5_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO5.Reg, volatile.LoadUint32(&o.GPIO5.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO5_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO5.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO5_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO5.Reg, volatile.LoadUint32(&o.GPIO5.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO5_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO5.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO5_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO5.Reg, volatile.LoadUint32(&o.GPIO5.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO5_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO5.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO5_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO5.Reg, volatile.LoadUint32(&o.GPIO5.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO5_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO5.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO5_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO5.Reg, volatile.LoadUint32(&o.GPIO5.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO5_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO5.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO5_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO5.Reg, volatile.LoadUint32(&o.GPIO5.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO5_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO5.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO5_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO5.Reg, volatile.LoadUint32(&o.GPIO5.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO5_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO5.Reg) & 0x1
}

// PADS_BANK0.GPIO6
func (o *PADS_BANK0_Type) SetGPIO6_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO6.Reg, volatile.LoadUint32(&o.GPIO6.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO6_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO6.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO6_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO6.Reg, volatile.LoadUint32(&o.GPIO6.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO6_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO6.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO6_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO6.Reg, volatile.LoadUint32(&o.GPIO6.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO6_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO6.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO6_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO6.Reg, volatile.LoadUint32(&o.GPIO6.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO6_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO6.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO6_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO6.Reg, volatile.LoadUint32(&o.GPIO6.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO6_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO6.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO6_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO6.Reg, volatile.LoadUint32(&o.GPIO6.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO6_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO6.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO6_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO6.Reg, volatile.LoadUint32(&o.GPIO6.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO6_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO6.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO6_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO6.Reg, volatile.LoadUint32(&o.GPIO6.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO6_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO6.Reg) & 0x1
}

// PADS_BANK0.GPIO7
func (o *PADS_BANK0_Type) SetGPIO7_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO7.Reg, volatile.LoadUint32(&o.GPIO7.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO7_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO7.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO7_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO7.Reg, volatile.LoadUint32(&o.GPIO7.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO7_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO7.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO7_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO7.Reg, volatile.LoadUint32(&o.GPIO7.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO7_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO7.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO7_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO7.Reg, volatile.LoadUint32(&o.GPIO7.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO7_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO7.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO7_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO7.Reg, volatile.LoadUint32(&o.GPIO7.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO7_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO7.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO7_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO7.Reg, volatile.LoadUint32(&o.GPIO7.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO7_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO7.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO7_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO7.Reg, volatile.LoadUint32(&o.GPIO7.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO7_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO7.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO7_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO7.Reg, volatile.LoadUint32(&o.GPIO7.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO7_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO7.Reg) & 0x1
}

// PADS_BANK0.GPIO8
func (o *PADS_BANK0_Type) SetGPIO8_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO8.Reg, volatile.LoadUint32(&o.GPIO8.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO8_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO8.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO8_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO8.Reg, volatile.LoadUint32(&o.GPIO8.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO8_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO8.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO8_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO8.Reg, volatile.LoadUint32(&o.GPIO8.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO8_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO8.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO8_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO8.Reg, volatile.LoadUint32(&o.GPIO8.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO8_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO8.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO8_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO8.Reg, volatile.LoadUint32(&o.GPIO8.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO8_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO8.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO8_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO8.Reg, volatile.LoadUint32(&o.GPIO8.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO8_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO8.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO8_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO8.Reg, volatile.LoadUint32(&o.GPIO8.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO8_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO8.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO8_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO8.Reg, volatile.LoadUint32(&o.GPIO8.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO8_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO8.Reg) & 0x1
}

// PADS_BANK0.GPIO9
func (o *PADS_BANK0_Type) SetGPIO9_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO9.Reg, volatile.LoadUint32(&o.GPIO9.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO9_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO9.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO9_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO9.Reg, volatile.LoadUint32(&o.GPIO9.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO9_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO9.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO9_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO9.Reg, volatile.LoadUint32(&o.GPIO9.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO9_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO9.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO9_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO9.Reg, volatile.LoadUint32(&o.GPIO9.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO9_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO9.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO9_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO9.Reg, volatile.LoadUint32(&o.GPIO9.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO9_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO9.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO9_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO9.Reg, volatile.LoadUint32(&o.GPIO9.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO9_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO9.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO9_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO9.Reg, volatile.LoadUint32(&o.GPIO9.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO9_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO9.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO9_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO9.Reg, volatile.LoadUint32(&o.GPIO9.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO9_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO9.Reg) & 0x1
}

// PADS_BANK0.GPIO10
func (o *PADS_BANK0_Type) SetGPIO10_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO10.Reg, volatile.LoadUint32(&o.GPIO10.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO10_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO10.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO10_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO10.Reg, volatile.LoadUint32(&o.GPIO10.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO10_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO10.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO10_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO10.Reg, volatile.LoadUint32(&o.GPIO10.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO10_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO10.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO10_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO10.Reg, volatile.LoadUint32(&o.GPIO10.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO10_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO10.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO10_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO10.Reg, volatile.LoadUint32(&o.GPIO10.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO10_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO10.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO10_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO10.Reg, volatile.LoadUint32(&o.GPIO10.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO10_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO10.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO10_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO10.Reg, volatile.LoadUint32(&o.GPIO10.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO10_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO10.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO10_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO10.Reg, volatile.LoadUint32(&o.GPIO10.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO10_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO10.Reg) & 0x1
}

// PADS_BANK0.GPIO11
func (o *PADS_BANK0_Type) SetGPIO11_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO11.Reg, volatile.LoadUint32(&o.GPIO11.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO11_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO11.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO11_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO11.Reg, volatile.LoadUint32(&o.GPIO11.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO11_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO11.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO11_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO11.Reg, volatile.LoadUint32(&o.GPIO11.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO11_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO11.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO11_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO11.Reg, volatile.LoadUint32(&o.GPIO11.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO11_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO11.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO11_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO11.Reg, volatile.LoadUint32(&o.GPIO11.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO11_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO11.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO11_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO11.Reg, volatile.LoadUint32(&o.GPIO11.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO11_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO11.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO11_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO11.Reg, volatile.LoadUint32(&o.GPIO11.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO11_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO11.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO11_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO11.Reg, volatile.LoadUint32(&o.GPIO11.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO11_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO11.Reg) & 0x1
}

// PADS_BANK0.GPIO12
func (o *PADS_BANK0_Type) SetGPIO12_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO12.Reg, volatile.LoadUint32(&o.GPIO12.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO12_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO12.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO12_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO12.Reg, volatile.LoadUint32(&o.GPIO12.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO12_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO12.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO12_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO12.Reg, volatile.LoadUint32(&o.GPIO12.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO12_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO12.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO12_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO12.Reg, volatile.LoadUint32(&o.GPIO12.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO12_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO12.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO12_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO12.Reg, volatile.LoadUint32(&o.GPIO12.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO12_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO12.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO12_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO12.Reg, volatile.LoadUint32(&o.GPIO12.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO12_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO12.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO12_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO12.Reg, volatile.LoadUint32(&o.GPIO12.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO12_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO12.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO12_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO12.Reg, volatile.LoadUint32(&o.GPIO12.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO12_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO12.Reg) & 0x1
}

// PADS_BANK0.GPIO13
func (o *PADS_BANK0_Type) SetGPIO13_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO13.Reg, volatile.LoadUint32(&o.GPIO13.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO13_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO13.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO13_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO13.Reg, volatile.LoadUint32(&o.GPIO13.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO13_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO13.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO13_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO13.Reg, volatile.LoadUint32(&o.GPIO13.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO13_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO13.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO13_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO13.Reg, volatile.LoadUint32(&o.GPIO13.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO13_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO13.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO13_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO13.Reg, volatile.LoadUint32(&o.GPIO13.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO13_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO13.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO13_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO13.Reg, volatile.LoadUint32(&o.GPIO13.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO13_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO13.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO13_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO13.Reg, volatile.LoadUint32(&o.GPIO13.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO13_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO13.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO13_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO13.Reg, volatile.LoadUint32(&o.GPIO13.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO13_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO13.Reg) & 0x1
}

// PADS_BANK0.GPIO14
func (o *PADS_BANK0_Type) SetGPIO14_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO14.Reg, volatile.LoadUint32(&o.GPIO14.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO14_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO14.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO14_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO14.Reg, volatile.LoadUint32(&o.GPIO14.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO14_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO14.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO14_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO14.Reg, volatile.LoadUint32(&o.GPIO14.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO14_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO14.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO14_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO14.Reg, volatile.LoadUint32(&o.GPIO14.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO14_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO14.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO14_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO14.Reg, volatile.LoadUint32(&o.GPIO14.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO14_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO14.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO14_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO14.Reg, volatile.LoadUint32(&o.GPIO14.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO14_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO14.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO14_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO14.Reg, volatile.LoadUint32(&o.GPIO14.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO14_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO14.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO14_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO14.Reg, volatile.LoadUint32(&o.GPIO14.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO14_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO14.Reg) & 0x1
}

// PADS_BANK0.GPIO15
func (o *PADS_BANK0_Type) SetGPIO15_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO15.Reg, volatile.LoadUint32(&o.GPIO15.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO15_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO15.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO15_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO15.Reg, volatile.LoadUint32(&o.GPIO15.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO15_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO15.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO15_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO15.Reg, volatile.LoadUint32(&o.GPIO15.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO15_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO15.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO15_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO15.Reg, volatile.LoadUint32(&o.GPIO15.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO15_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO15.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO15_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO15.Reg, volatile.LoadUint32(&o.GPIO15.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO15_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO15.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO15_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO15.Reg, volatile.LoadUint32(&o.GPIO15.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO15_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO15.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO15_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO15.Reg, volatile.LoadUint32(&o.GPIO15.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO15_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO15.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO15_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO15.Reg, volatile.LoadUint32(&o.GPIO15.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO15_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO15.Reg) & 0x1
}

// PADS_BANK0.GPIO16
func (o *PADS_BANK0_Type) SetGPIO16_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO16.Reg, volatile.LoadUint32(&o.GPIO16.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO16_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO16.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO16_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO16.Reg, volatile.LoadUint32(&o.GPIO16.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO16_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO16.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO16_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO16.Reg, volatile.LoadUint32(&o.GPIO16.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO16_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO16.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO16_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO16.Reg, volatile.LoadUint32(&o.GPIO16.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO16_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO16.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO16_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO16.Reg, volatile.LoadUint32(&o.GPIO16.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO16_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO16.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO16_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO16.Reg, volatile.LoadUint32(&o.GPIO16.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO16_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO16.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO16_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO16.Reg, volatile.LoadUint32(&o.GPIO16.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO16_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO16.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO16_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO16.Reg, volatile.LoadUint32(&o.GPIO16.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO16_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO16.Reg) & 0x1
}

// PADS_BANK0.GPIO17
func (o *PADS_BANK0_Type) SetGPIO17_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO17.Reg, volatile.LoadUint32(&o.GPIO17.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO17_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO17.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO17_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO17.Reg, volatile.LoadUint32(&o.GPIO17.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO17_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO17.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO17_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO17.Reg, volatile.LoadUint32(&o.GPIO17.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO17_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO17.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO17_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO17.Reg, volatile.LoadUint32(&o.GPIO17.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO17_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO17.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO17_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO17.Reg, volatile.LoadUint32(&o.GPIO17.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO17_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO17.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO17_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO17.Reg, volatile.LoadUint32(&o.GPIO17.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO17_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO17.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO17_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO17.Reg, volatile.LoadUint32(&o.GPIO17.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO17_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO17.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO17_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO17.Reg, volatile.LoadUint32(&o.GPIO17.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO17_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO17.Reg) & 0x1
}

// PADS_BANK0.GPIO18
func (o *PADS_BANK0_Type) SetGPIO18_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO18.Reg, volatile.LoadUint32(&o.GPIO18.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO18_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO18.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO18_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO18.Reg, volatile.LoadUint32(&o.GPIO18.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO18_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO18.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO18_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO18.Reg, volatile.LoadUint32(&o.GPIO18.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO18_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO18.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO18_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO18.Reg, volatile.LoadUint32(&o.GPIO18.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO18_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO18.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO18_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO18.Reg, volatile.LoadUint32(&o.GPIO18.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO18_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO18.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO18_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO18.Reg, volatile.LoadUint32(&o.GPIO18.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO18_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO18.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO18_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO18.Reg, volatile.LoadUint32(&o.GPIO18.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO18_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO18.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO18_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO18.Reg, volatile.LoadUint32(&o.GPIO18.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO18_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO18.Reg) & 0x1
}

// PADS_BANK0.GPIO19
func (o *PADS_BANK0_Type) SetGPIO19_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO19.Reg, volatile.LoadUint32(&o.GPIO19.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO19_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO19.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO19_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO19.Reg, volatile.LoadUint32(&o.GPIO19.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO19_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO19.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO19_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO19.Reg, volatile.LoadUint32(&o.GPIO19.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO19_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO19.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO19_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO19.Reg, volatile.LoadUint32(&o.GPIO19.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO19_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO19.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO19_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO19.Reg, volatile.LoadUint32(&o.GPIO19.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO19_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO19.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO19_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO19.Reg, volatile.LoadUint32(&o.GPIO19.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO19_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO19.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO19_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO19.Reg, volatile.LoadUint32(&o.GPIO19.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO19_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO19.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO19_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO19.Reg, volatile.LoadUint32(&o.GPIO19.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO19_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO19.Reg) & 0x1
}

// PADS_BANK0.GPIO20
func (o *PADS_BANK0_Type) SetGPIO20_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO20.Reg, volatile.LoadUint32(&o.GPIO20.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO20_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO20.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO20_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO20.Reg, volatile.LoadUint32(&o.GPIO20.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO20_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO20.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO20_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO20.Reg, volatile.LoadUint32(&o.GPIO20.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO20_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO20.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO20_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO20.Reg, volatile.LoadUint32(&o.GPIO20.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO20_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO20.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO20_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO20.Reg, volatile.LoadUint32(&o.GPIO20.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO20_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO20.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO20_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO20.Reg, volatile.LoadUint32(&o.GPIO20.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO20_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO20.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO20_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO20.Reg, volatile.LoadUint32(&o.GPIO20.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO20_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO20.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO20_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO20.Reg, volatile.LoadUint32(&o.GPIO20.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO20_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO20.Reg) & 0x1
}

// PADS_BANK0.GPIO21
func (o *PADS_BANK0_Type) SetGPIO21_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO21.Reg, volatile.LoadUint32(&o.GPIO21.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO21_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO21.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO21_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO21.Reg, volatile.LoadUint32(&o.GPIO21.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO21_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO21.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO21_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO21.Reg, volatile.LoadUint32(&o.GPIO21.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO21_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO21.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO21_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO21.Reg, volatile.LoadUint32(&o.GPIO21.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO21_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO21.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO21_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO21.Reg, volatile.LoadUint32(&o.GPIO21.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO21_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO21.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO21_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO21.Reg, volatile.LoadUint32(&o.GPIO21.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO21_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO21.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO21_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO21.Reg, volatile.LoadUint32(&o.GPIO21.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO21_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO21.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO21_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO21.Reg, volatile.LoadUint32(&o.GPIO21.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO21_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO21.Reg) & 0x1
}

// PADS_BANK0.GPIO22
func (o *PADS_BANK0_Type) SetGPIO22_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO22.Reg, volatile.LoadUint32(&o.GPIO22.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO22_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO22.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO22_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO22.Reg, volatile.LoadUint32(&o.GPIO22.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO22_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO22.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO22_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO22.Reg, volatile.LoadUint32(&o.GPIO22.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO22_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO22.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO22_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO22.Reg, volatile.LoadUint32(&o.GPIO22.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO22_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO22.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO22_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO22.Reg, volatile.LoadUint32(&o.GPIO22.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO22_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO22.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO22_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO22.Reg, volatile.LoadUint32(&o.GPIO22.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO22_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO22.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO22_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO22.Reg, volatile.LoadUint32(&o.GPIO22.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO22_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO22.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO22_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO22.Reg, volatile.LoadUint32(&o.GPIO22.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO22_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO22.Reg) & 0x1
}

// PADS_BANK0.GPIO23
func (o *PADS_BANK0_Type) SetGPIO23_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO23.Reg, volatile.LoadUint32(&o.GPIO23.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO23_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO23.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO23_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO23.Reg, volatile.LoadUint32(&o.GPIO23.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO23_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO23.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO23_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO23.Reg, volatile.LoadUint32(&o.GPIO23.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO23_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO23.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO23_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO23.Reg, volatile.LoadUint32(&o.GPIO23.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO23_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO23.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO23_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO23.Reg, volatile.LoadUint32(&o.GPIO23.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO23_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO23.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO23_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO23.Reg, volatile.LoadUint32(&o.GPIO23.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO23_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO23.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO23_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO23.Reg, volatile.LoadUint32(&o.GPIO23.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO23_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO23.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO23_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO23.Reg, volatile.LoadUint32(&o.GPIO23.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO23_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO23.Reg) & 0x1
}

// PADS_BANK0.GPIO24
func (o *PADS_BANK0_Type) SetGPIO24_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO24.Reg, volatile.LoadUint32(&o.GPIO24.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO24_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO24.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO24_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO24.Reg, volatile.LoadUint32(&o.GPIO24.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO24_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO24.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO24_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO24.Reg, volatile.LoadUint32(&o.GPIO24.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO24_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO24.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO24_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO24.Reg, volatile.LoadUint32(&o.GPIO24.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO24_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO24.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO24_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO24.Reg, volatile.LoadUint32(&o.GPIO24.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO24_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO24.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO24_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO24.Reg, volatile.LoadUint32(&o.GPIO24.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO24_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO24.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO24_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO24.Reg, volatile.LoadUint32(&o.GPIO24.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO24_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO24.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO24_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO24.Reg, volatile.LoadUint32(&o.GPIO24.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO24_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO24.Reg) & 0x1
}

// PADS_BANK0.GPIO25
func (o *PADS_BANK0_Type) SetGPIO25_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO25.Reg, volatile.LoadUint32(&o.GPIO25.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO25_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO25.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO25_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO25.Reg, volatile.LoadUint32(&o.GPIO25.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO25_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO25.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO25_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO25.Reg, volatile.LoadUint32(&o.GPIO25.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO25_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO25.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO25_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO25.Reg, volatile.LoadUint32(&o.GPIO25.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO25_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO25.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO25_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO25.Reg, volatile.LoadUint32(&o.GPIO25.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO25_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO25.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO25_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO25.Reg, volatile.LoadUint32(&o.GPIO25.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO25_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO25.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO25_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO25.Reg, volatile.LoadUint32(&o.GPIO25.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO25_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO25.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO25_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO25.Reg, volatile.LoadUint32(&o.GPIO25.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO25_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO25.Reg) & 0x1
}

// PADS_BANK0.GPIO26
func (o *PADS_BANK0_Type) SetGPIO26_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO26.Reg, volatile.LoadUint32(&o.GPIO26.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO26_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO26.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO26_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO26.Reg, volatile.LoadUint32(&o.GPIO26.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO26_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO26.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO26_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO26.Reg, volatile.LoadUint32(&o.GPIO26.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO26_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO26.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO26_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO26.Reg, volatile.LoadUint32(&o.GPIO26.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO26_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO26.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO26_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO26.Reg, volatile.LoadUint32(&o.GPIO26.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO26_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO26.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO26_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO26.Reg, volatile.LoadUint32(&o.GPIO26.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO26_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO26.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO26_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO26.Reg, volatile.LoadUint32(&o.GPIO26.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO26_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO26.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO26_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO26.Reg, volatile.LoadUint32(&o.GPIO26.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO26_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO26.Reg) & 0x1
}

// PADS_BANK0.GPIO27
func (o *PADS_BANK0_Type) SetGPIO27_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO27.Reg, volatile.LoadUint32(&o.GPIO27.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO27_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO27.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO27_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO27.Reg, volatile.LoadUint32(&o.GPIO27.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO27_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO27.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO27_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO27.Reg, volatile.LoadUint32(&o.GPIO27.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO27_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO27.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO27_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO27.Reg, volatile.LoadUint32(&o.GPIO27.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO27_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO27.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO27_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO27.Reg, volatile.LoadUint32(&o.GPIO27.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO27_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO27.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO27_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO27.Reg, volatile.LoadUint32(&o.GPIO27.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO27_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO27.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO27_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO27.Reg, volatile.LoadUint32(&o.GPIO27.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO27_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO27.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO27_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO27.Reg, volatile.LoadUint32(&o.GPIO27.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO27_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO27.Reg) & 0x1
}

// PADS_BANK0.GPIO28
func (o *PADS_BANK0_Type) SetGPIO28_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO28.Reg, volatile.LoadUint32(&o.GPIO28.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO28_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO28.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO28_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO28.Reg, volatile.LoadUint32(&o.GPIO28.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO28_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO28.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO28_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO28.Reg, volatile.LoadUint32(&o.GPIO28.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO28_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO28.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO28_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO28.Reg, volatile.LoadUint32(&o.GPIO28.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO28_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO28.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO28_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO28.Reg, volatile.LoadUint32(&o.GPIO28.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO28_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO28.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO28_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO28.Reg, volatile.LoadUint32(&o.GPIO28.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO28_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO28.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO28_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO28.Reg, volatile.LoadUint32(&o.GPIO28.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO28_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO28.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO28_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO28.Reg, volatile.LoadUint32(&o.GPIO28.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO28_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO28.Reg) & 0x1
}

// PADS_BANK0.GPIO29
func (o *PADS_BANK0_Type) SetGPIO29_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO29.Reg, volatile.LoadUint32(&o.GPIO29.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO29_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO29.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO29_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO29.Reg, volatile.LoadUint32(&o.GPIO29.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO29_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO29.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO29_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO29.Reg, volatile.LoadUint32(&o.GPIO29.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO29_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO29.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO29_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO29.Reg, volatile.LoadUint32(&o.GPIO29.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO29_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO29.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO29_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO29.Reg, volatile.LoadUint32(&o.GPIO29.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO29_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO29.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO29_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO29.Reg, volatile.LoadUint32(&o.GPIO29.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO29_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO29.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO29_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO29.Reg, volatile.LoadUint32(&o.GPIO29.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO29_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO29.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO29_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO29.Reg, volatile.LoadUint32(&o.GPIO29.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO29_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO29.Reg) & 0x1
}

// PADS_BANK0.GPIO30
func (o *PADS_BANK0_Type) SetGPIO30_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO30.Reg, volatile.LoadUint32(&o.GPIO30.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO30_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO30.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO30_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO30.Reg, volatile.LoadUint32(&o.GPIO30.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO30_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO30.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO30_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO30.Reg, volatile.LoadUint32(&o.GPIO30.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO30_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO30.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO30_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO30.Reg, volatile.LoadUint32(&o.GPIO30.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO30_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO30.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO30_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO30.Reg, volatile.LoadUint32(&o.GPIO30.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO30_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO30.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO30_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO30.Reg, volatile.LoadUint32(&o.GPIO30.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO30_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO30.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO30_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO30.Reg, volatile.LoadUint32(&o.GPIO30.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO30_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO30.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO30_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO30.Reg, volatile.LoadUint32(&o.GPIO30.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO30_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO30.Reg) & 0x1
}

// PADS_BANK0.GPIO31
func (o *PADS_BANK0_Type) SetGPIO31_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO31.Reg, volatile.LoadUint32(&o.GPIO31.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO31_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO31.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO31_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO31.Reg, volatile.LoadUint32(&o.GPIO31.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO31_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO31.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO31_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO31.Reg, volatile.LoadUint32(&o.GPIO31.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO31_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO31.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO31_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO31.Reg, volatile.LoadUint32(&o.GPIO31.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO31_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO31.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO31_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO31.Reg, volatile.LoadUint32(&o.GPIO31.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO31_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO31.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO31_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO31.Reg, volatile.LoadUint32(&o.GPIO31.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO31_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO31.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO31_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO31.Reg, volatile.LoadUint32(&o.GPIO31.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO31_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO31.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO31_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO31.Reg, volatile.LoadUint32(&o.GPIO31.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO31_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO31.Reg) & 0x1
}

// PADS_BANK0.GPIO32
func (o *PADS_BANK0_Type) SetGPIO32_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO32.Reg, volatile.LoadUint32(&o.GPIO32.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO32_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO32.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO32_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO32.Reg, volatile.LoadUint32(&o.GPIO32.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO32_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO32.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO32_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO32.Reg, volatile.LoadUint32(&o.GPIO32.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO32_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO32.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO32_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO32.Reg, volatile.LoadUint32(&o.GPIO32.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO32_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO32.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO32_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO32.Reg, volatile.LoadUint32(&o.GPIO32.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO32_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO32.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO32_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO32.Reg, volatile.LoadUint32(&o.GPIO32.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO32_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO32.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO32_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO32.Reg, volatile.LoadUint32(&o.GPIO32.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO32_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO32.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO32_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO32.Reg, volatile.LoadUint32(&o.GPIO32.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO32_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO32.Reg) & 0x1
}

// PADS_BANK0.GPIO33
func (o *PADS_BANK0_Type) SetGPIO33_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO33.Reg, volatile.LoadUint32(&o.GPIO33.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO33_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO33.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO33_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO33.Reg, volatile.LoadUint32(&o.GPIO33.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO33_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO33.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO33_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO33.Reg, volatile.LoadUint32(&o.GPIO33.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO33_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO33.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO33_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO33.Reg, volatile.LoadUint32(&o.GPIO33.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO33_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO33.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO33_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO33.Reg, volatile.LoadUint32(&o.GPIO33.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO33_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO33.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO33_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO33.Reg, volatile.LoadUint32(&o.GPIO33.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO33_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO33.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO33_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO33.Reg, volatile.LoadUint32(&o.GPIO33.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO33_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO33.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO33_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO33.Reg, volatile.LoadUint32(&o.GPIO33.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO33_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO33.Reg) & 0x1
}

// PADS_BANK0.GPIO34
func (o *PADS_BANK0_Type) SetGPIO34_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO34.Reg, volatile.LoadUint32(&o.GPIO34.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO34_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO34.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO34_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO34.Reg, volatile.LoadUint32(&o.GPIO34.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO34_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO34.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO34_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO34.Reg, volatile.LoadUint32(&o.GPIO34.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO34_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO34.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO34_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO34.Reg, volatile.LoadUint32(&o.GPIO34.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO34_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO34.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO34_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO34.Reg, volatile.LoadUint32(&o.GPIO34.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO34_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO34.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO34_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO34.Reg, volatile.LoadUint32(&o.GPIO34.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO34_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO34.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO34_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO34.Reg, volatile.LoadUint32(&o.GPIO34.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO34_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO34.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO34_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO34.Reg, volatile.LoadUint32(&o.GPIO34.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO34_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO34.Reg) & 0x1
}

// PADS_BANK0.GPIO35
func (o *PADS_BANK0_Type) SetGPIO35_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO35.Reg, volatile.LoadUint32(&o.GPIO35.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO35_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO35.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO35_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO35.Reg, volatile.LoadUint32(&o.GPIO35.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO35_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO35.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO35_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO35.Reg, volatile.LoadUint32(&o.GPIO35.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO35_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO35.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO35_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO35.Reg, volatile.LoadUint32(&o.GPIO35.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO35_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO35.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO35_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO35.Reg, volatile.LoadUint32(&o.GPIO35.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO35_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO35.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO35_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO35.Reg, volatile.LoadUint32(&o.GPIO35.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO35_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO35.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO35_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO35.Reg, volatile.LoadUint32(&o.GPIO35.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO35_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO35.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO35_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO35.Reg, volatile.LoadUint32(&o.GPIO35.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO35_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO35.Reg) & 0x1
}

// PADS_BANK0.GPIO36
func (o *PADS_BANK0_Type) SetGPIO36_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO36.Reg, volatile.LoadUint32(&o.GPIO36.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO36_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO36.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO36_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO36.Reg, volatile.LoadUint32(&o.GPIO36.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO36_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO36.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO36_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO36.Reg, volatile.LoadUint32(&o.GPIO36.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO36_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO36.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO36_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO36.Reg, volatile.LoadUint32(&o.GPIO36.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO36_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO36.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO36_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO36.Reg, volatile.LoadUint32(&o.GPIO36.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO36_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO36.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO36_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO36.Reg, volatile.LoadUint32(&o.GPIO36.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO36_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO36.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO36_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO36.Reg, volatile.LoadUint32(&o.GPIO36.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO36_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO36.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO36_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO36.Reg, volatile.LoadUint32(&o.GPIO36.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO36_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO36.Reg) & 0x1
}

// PADS_BANK0.GPIO37
func (o *PADS_BANK0_Type) SetGPIO37_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO37.Reg, volatile.LoadUint32(&o.GPIO37.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO37_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO37.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO37_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO37.Reg, volatile.LoadUint32(&o.GPIO37.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO37_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO37.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO37_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO37.Reg, volatile.LoadUint32(&o.GPIO37.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO37_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO37.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO37_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO37.Reg, volatile.LoadUint32(&o.GPIO37.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO37_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO37.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO37_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO37.Reg, volatile.LoadUint32(&o.GPIO37.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO37_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO37.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO37_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO37.Reg, volatile.LoadUint32(&o.GPIO37.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO37_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO37.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO37_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO37.Reg, volatile.LoadUint32(&o.GPIO37.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO37_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO37.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO37_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO37.Reg, volatile.LoadUint32(&o.GPIO37.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO37_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO37.Reg) & 0x1
}

// PADS_BANK0.GPIO38
func (o *PADS_BANK0_Type) SetGPIO38_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO38.Reg, volatile.LoadUint32(&o.GPIO38.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO38_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO38.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO38_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO38.Reg, volatile.LoadUint32(&o.GPIO38.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO38_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO38.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO38_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO38.Reg, volatile.LoadUint32(&o.GPIO38.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO38_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO38.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO38_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO38.Reg, volatile.LoadUint32(&o.GPIO38.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO38_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO38.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO38_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO38.Reg, volatile.LoadUint32(&o.GPIO38.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO38_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO38.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO38_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO38.Reg, volatile.LoadUint32(&o.GPIO38.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO38_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO38.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO38_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO38.Reg, volatile.LoadUint32(&o.GPIO38.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO38_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO38.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO38_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO38.Reg, volatile.LoadUint32(&o.GPIO38.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO38_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO38.Reg) & 0x1
}

// PADS_BANK0.GPIO39
func (o *PADS_BANK0_Type) SetGPIO39_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO39.Reg, volatile.LoadUint32(&o.GPIO39.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO39_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO39.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO39_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO39.Reg, volatile.LoadUint32(&o.GPIO39.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO39_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO39.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO39_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO39.Reg, volatile.LoadUint32(&o.GPIO39.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO39_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO39.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO39_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO39.Reg, volatile.LoadUint32(&o.GPIO39.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO39_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO39.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO39_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO39.Reg, volatile.LoadUint32(&o.GPIO39.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO39_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO39.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO39_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO39.Reg, volatile.LoadUint32(&o.GPIO39.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO39_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO39.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO39_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO39.Reg, volatile.LoadUint32(&o.GPIO39.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO39_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO39.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO39_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO39.Reg, volatile.LoadUint32(&o.GPIO39.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO39_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO39.Reg) & 0x1
}

// PADS_BANK0.GPIO40
func (o *PADS_BANK0_Type) SetGPIO40_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO40.Reg, volatile.LoadUint32(&o.GPIO40.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO40_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO40.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO40_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO40.Reg, volatile.LoadUint32(&o.GPIO40.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO40_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO40.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO40_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO40.Reg, volatile.LoadUint32(&o.GPIO40.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO40_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO40.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO40_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO40.Reg, volatile.LoadUint32(&o.GPIO40.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO40_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO40.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO40_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO40.Reg, volatile.LoadUint32(&o.GPIO40.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO40_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO40.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO40_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO40.Reg, volatile.LoadUint32(&o.GPIO40.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO40_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO40.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO40_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO40.Reg, volatile.LoadUint32(&o.GPIO40.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO40_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO40.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO40_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO40.Reg, volatile.LoadUint32(&o.GPIO40.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO40_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO40.Reg) & 0x1
}

// PADS_BANK0.GPIO41
func (o *PADS_BANK0_Type) SetGPIO41_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO41.Reg, volatile.LoadUint32(&o.GPIO41.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO41_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO41.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO41_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO41.Reg, volatile.LoadUint32(&o.GPIO41.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO41_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO41.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO41_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO41.Reg, volatile.LoadUint32(&o.GPIO41.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO41_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO41.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO41_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO41.Reg, volatile.LoadUint32(&o.GPIO41.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO41_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO41.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO41_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO41.Reg, volatile.LoadUint32(&o.GPIO41.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO41_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO41.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO41_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO41.Reg, volatile.LoadUint32(&o.GPIO41.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO41_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO41.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO41_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO41.Reg, volatile.LoadUint32(&o.GPIO41.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO41_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO41.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO41_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO41.Reg, volatile.LoadUint32(&o.GPIO41.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO41_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO41.Reg) & 0x1
}

// PADS_BANK0.GPIO42
func (o *PADS_BANK0_Type) SetGPIO42_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO42.Reg, volatile.LoadUint32(&o.GPIO42.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO42_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO42.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO42_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO42.Reg, volatile.LoadUint32(&o.GPIO42.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO42_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO42.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO42_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO42.Reg, volatile.LoadUint32(&o.GPIO42.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO42_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO42.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO42_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO42.Reg, volatile.LoadUint32(&o.GPIO42.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO42_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO42.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO42_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO42.Reg, volatile.LoadUint32(&o.GPIO42.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO42_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO42.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO42_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO42.Reg, volatile.LoadUint32(&o.GPIO42.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO42_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO42.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO42_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO42.Reg, volatile.LoadUint32(&o.GPIO42.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO42_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO42.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO42_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO42.Reg, volatile.LoadUint32(&o.GPIO42.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO42_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO42.Reg) & 0x1
}

// PADS_BANK0.GPIO43
func (o *PADS_BANK0_Type) SetGPIO43_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO43.Reg, volatile.LoadUint32(&o.GPIO43.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO43_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO43.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO43_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO43.Reg, volatile.LoadUint32(&o.GPIO43.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO43_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO43.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO43_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO43.Reg, volatile.LoadUint32(&o.GPIO43.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO43_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO43.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO43_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO43.Reg, volatile.LoadUint32(&o.GPIO43.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO43_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO43.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO43_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO43.Reg, volatile.LoadUint32(&o.GPIO43.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO43_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO43.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO43_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO43.Reg, volatile.LoadUint32(&o.GPIO43.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO43_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO43.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO43_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO43.Reg, volatile.LoadUint32(&o.GPIO43.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO43_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO43.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO43_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO43.Reg, volatile.LoadUint32(&o.GPIO43.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO43_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO43.Reg) & 0x1
}

// PADS_BANK0.GPIO44
func (o *PADS_BANK0_Type) SetGPIO44_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO44.Reg, volatile.LoadUint32(&o.GPIO44.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO44_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO44.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO44_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO44.Reg, volatile.LoadUint32(&o.GPIO44.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO44_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO44.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO44_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO44.Reg, volatile.LoadUint32(&o.GPIO44.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO44_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO44.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO44_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO44.Reg, volatile.LoadUint32(&o.GPIO44.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO44_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO44.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO44_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO44.Reg, volatile.LoadUint32(&o.GPIO44.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO44_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO44.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO44_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO44.Reg, volatile.LoadUint32(&o.GPIO44.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO44_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO44.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO44_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO44.Reg, volatile.LoadUint32(&o.GPIO44.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO44_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO44.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO44_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO44.Reg, volatile.LoadUint32(&o.GPIO44.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO44_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO44.Reg) & 0x1
}

// PADS_BANK0.GPIO45
func (o *PADS_BANK0_Type) SetGPIO45_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO45.Reg, volatile.LoadUint32(&o.GPIO45.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO45_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO45.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO45_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO45.Reg, volatile.LoadUint32(&o.GPIO45.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO45_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO45.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO45_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO45.Reg, volatile.LoadUint32(&o.GPIO45.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO45_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO45.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO45_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO45.Reg, volatile.LoadUint32(&o.GPIO45.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO45_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO45.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO45_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO45.Reg, volatile.LoadUint32(&o.GPIO45.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO45_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO45.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO45_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO45.Reg, volatile.LoadUint32(&o.GPIO45.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO45_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO45.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO45_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO45.Reg, volatile.LoadUint32(&o.GPIO45.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO45_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO45.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO45_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO45.Reg, volatile.LoadUint32(&o.GPIO45.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO45_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO45.Reg) & 0x1
}

// PADS_BANK0.GPIO46
func (o *PADS_BANK0_Type) SetGPIO46_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO46.Reg, volatile.LoadUint32(&o.GPIO46.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO46_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO46.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO46_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO46.Reg, volatile.LoadUint32(&o.GPIO46.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO46_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO46.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO46_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO46.Reg, volatile.LoadUint32(&o.GPIO46.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO46_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO46.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO46_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO46.Reg, volatile.LoadUint32(&o.GPIO46.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO46_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO46.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO46_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO46.Reg, volatile.LoadUint32(&o.GPIO46.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO46_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO46.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO46_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO46.Reg, volatile.LoadUint32(&o.GPIO46.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO46_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO46.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO46_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO46.Reg, volatile.LoadUint32(&o.GPIO46.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO46_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO46.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO46_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO46.Reg, volatile.LoadUint32(&o.GPIO46.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO46_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO46.Reg) & 0x1
}

// PADS_BANK0.GPIO47
func (o *PADS_BANK0_Type) SetGPIO47_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO47.Reg, volatile.LoadUint32(&o.GPIO47.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetGPIO47_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO47.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetGPIO47_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO47.Reg, volatile.LoadUint32(&o.GPIO47.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetGPIO47_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO47.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetGPIO47_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO47.Reg, volatile.LoadUint32(&o.GPIO47.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetGPIO47_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO47.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetGPIO47_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO47.Reg, volatile.LoadUint32(&o.GPIO47.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetGPIO47_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO47.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetGPIO47_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO47.Reg, volatile.LoadUint32(&o.GPIO47.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetGPIO47_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO47.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetGPIO47_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO47.Reg, volatile.LoadUint32(&o.GPIO47.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetGPIO47_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO47.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetGPIO47_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO47.Reg, volatile.LoadUint32(&o.GPIO47.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetGPIO47_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO47.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetGPIO47_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO47.Reg, volatile.LoadUint32(&o.GPIO47.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetGPIO47_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO47.Reg) & 0x1
}

// PADS_BANK0.SWCLK
func (o *PADS_BANK0_Type) SetSWCLK_ISO(value uint32) {
	volatile.StoreUint32(&o.SWCLK.Reg, volatile.LoadUint32(&o.SWCLK.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetSWCLK_ISO() uint32 {
	return (volatile.LoadUint32(&o.SWCLK.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetSWCLK_OD(value uint32) {
	volatile.StoreUint32(&o.SWCLK.Reg, volatile.LoadUint32(&o.SWCLK.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetSWCLK_OD() uint32 {
	return (volatile.LoadUint32(&o.SWCLK.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetSWCLK_IE(value uint32) {
	volatile.StoreUint32(&o.SWCLK.Reg, volatile.LoadUint32(&o.SWCLK.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetSWCLK_IE() uint32 {
	return (volatile.LoadUint32(&o.SWCLK.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetSWCLK_DRIVE(value uint32) {
	volatile.StoreUint32(&o.SWCLK.Reg, volatile.LoadUint32(&o.SWCLK.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetSWCLK_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.SWCLK.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetSWCLK_PUE(value uint32) {
	volatile.StoreUint32(&o.SWCLK.Reg, volatile.LoadUint32(&o.SWCLK.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetSWCLK_PUE() uint32 {
	return (volatile.LoadUint32(&o.SWCLK.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetSWCLK_PDE(value uint32) {
	volatile.StoreUint32(&o.SWCLK.Reg, volatile.LoadUint32(&o.SWCLK.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetSWCLK_PDE() uint32 {
	return (volatile.LoadUint32(&o.SWCLK.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetSWCLK_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.SWCLK.Reg, volatile.LoadUint32(&o.SWCLK.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetSWCLK_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.SWCLK.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetSWCLK_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.SWCLK.Reg, volatile.LoadUint32(&o.SWCLK.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetSWCLK_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.SWCLK.Reg) & 0x1
}

// PADS_BANK0.SWD
func (o *PADS_BANK0_Type) SetSWD_ISO(value uint32) {
	volatile.StoreUint32(&o.SWD.Reg, volatile.LoadUint32(&o.SWD.Reg)&^(0x100)|value<<8)
}
func (o *PADS_BANK0_Type) GetSWD_ISO() uint32 {
	return (volatile.LoadUint32(&o.SWD.Reg) & 0x100) >> 8
}
func (o *PADS_BANK0_Type) SetSWD_OD(value uint32) {
	volatile.StoreUint32(&o.SWD.Reg, volatile.LoadUint32(&o.SWD.Reg)&^(0x80)|value<<7)
}
func (o *PADS_BANK0_Type) GetSWD_OD() uint32 {
	return (volatile.LoadUint32(&o.SWD.Reg) & 0x80) >> 7
}
func (o *PADS_BANK0_Type) SetSWD_IE(value uint32) {
	volatile.StoreUint32(&o.SWD.Reg, volatile.LoadUint32(&o.SWD.Reg)&^(0x40)|value<<6)
}
func (o *PADS_BANK0_Type) GetSWD_IE() uint32 {
	return (volatile.LoadUint32(&o.SWD.Reg) & 0x40) >> 6
}
func (o *PADS_BANK0_Type) SetSWD_DRIVE(value uint32) {
	volatile.StoreUint32(&o.SWD.Reg, volatile.LoadUint32(&o.SWD.Reg)&^(0x30)|value<<4)
}
func (o *PADS_BANK0_Type) GetSWD_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.SWD.Reg) & 0x30) >> 4
}
func (o *PADS_BANK0_Type) SetSWD_PUE(value uint32) {
	volatile.StoreUint32(&o.SWD.Reg, volatile.LoadUint32(&o.SWD.Reg)&^(0x8)|value<<3)
}
func (o *PADS_BANK0_Type) GetSWD_PUE() uint32 {
	return (volatile.LoadUint32(&o.SWD.Reg) & 0x8) >> 3
}
func (o *PADS_BANK0_Type) SetSWD_PDE(value uint32) {
	volatile.StoreUint32(&o.SWD.Reg, volatile.LoadUint32(&o.SWD.Reg)&^(0x4)|value<<2)
}
func (o *PADS_BANK0_Type) GetSWD_PDE() uint32 {
	return (volatile.LoadUint32(&o.SWD.Reg) & 0x4) >> 2
}
func (o *PADS_BANK0_Type) SetSWD_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.SWD.Reg, volatile.LoadUint32(&o.SWD.Reg)&^(0x2)|value<<1)
}
func (o *PADS_BANK0_Type) GetSWD_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.SWD.Reg) & 0x2) >> 1
}
func (o *PADS_BANK0_Type) SetSWD_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.SWD.Reg, volatile.LoadUint32(&o.SWD.Reg)&^(0x1)|value)
}
func (o *PADS_BANK0_Type) GetSWD_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.SWD.Reg) & 0x1
}

type PADS_QSPI_Type struct {
	VOLTAGE_SELECT volatile.Register32 // 0x0
	GPIO_QSPI_SCLK volatile.Register32 // 0x4
	GPIO_QSPI_SD0  volatile.Register32 // 0x8
	GPIO_QSPI_SD1  volatile.Register32 // 0xC
	GPIO_QSPI_SD2  volatile.Register32 // 0x10
	GPIO_QSPI_SD3  volatile.Register32 // 0x14
	GPIO_QSPI_SS   volatile.Register32 // 0x18
}

// PADS_QSPI.VOLTAGE_SELECT: Voltage select. Per bank control
func (o *PADS_QSPI_Type) SetVOLTAGE_SELECT(value uint32) {
	volatile.StoreUint32(&o.VOLTAGE_SELECT.Reg, volatile.LoadUint32(&o.VOLTAGE_SELECT.Reg)&^(0x1)|value)
}
func (o *PADS_QSPI_Type) GetVOLTAGE_SELECT() uint32 {
	return volatile.LoadUint32(&o.VOLTAGE_SELECT.Reg) & 0x1
}

// PADS_QSPI.GPIO_QSPI_SCLK
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SCLK_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg)&^(0x100)|value<<8)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SCLK_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg) & 0x100) >> 8
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SCLK_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg)&^(0x80)|value<<7)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SCLK_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg) & 0x80) >> 7
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SCLK_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg)&^(0x40)|value<<6)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SCLK_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg) & 0x40) >> 6
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SCLK_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg)&^(0x30)|value<<4)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SCLK_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg) & 0x30) >> 4
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SCLK_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg)&^(0x8)|value<<3)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SCLK_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg) & 0x8) >> 3
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SCLK_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg)&^(0x4)|value<<2)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SCLK_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg) & 0x4) >> 2
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SCLK_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg)&^(0x2)|value<<1)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SCLK_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg) & 0x2) >> 1
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SCLK_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg)&^(0x1)|value)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SCLK_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SCLK.Reg) & 0x1
}

// PADS_QSPI.GPIO_QSPI_SD0
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD0_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg)&^(0x100)|value<<8)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD0_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg) & 0x100) >> 8
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD0_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg)&^(0x80)|value<<7)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD0_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg) & 0x80) >> 7
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD0_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg)&^(0x40)|value<<6)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD0_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg) & 0x40) >> 6
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD0_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg)&^(0x30)|value<<4)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD0_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg) & 0x30) >> 4
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD0_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg)&^(0x8)|value<<3)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD0_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg) & 0x8) >> 3
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD0_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg)&^(0x4)|value<<2)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD0_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg) & 0x4) >> 2
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD0_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg)&^(0x2)|value<<1)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD0_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg) & 0x2) >> 1
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD0_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg)&^(0x1)|value)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD0_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SD0.Reg) & 0x1
}

// PADS_QSPI.GPIO_QSPI_SD1
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD1_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg)&^(0x100)|value<<8)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD1_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg) & 0x100) >> 8
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD1_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg)&^(0x80)|value<<7)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD1_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg) & 0x80) >> 7
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD1_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg)&^(0x40)|value<<6)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD1_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg) & 0x40) >> 6
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD1_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg)&^(0x30)|value<<4)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD1_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg) & 0x30) >> 4
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD1_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg)&^(0x8)|value<<3)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD1_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg) & 0x8) >> 3
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD1_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg)&^(0x4)|value<<2)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD1_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg) & 0x4) >> 2
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD1_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg)&^(0x2)|value<<1)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD1_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg) & 0x2) >> 1
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD1_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg)&^(0x1)|value)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD1_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SD1.Reg) & 0x1
}

// PADS_QSPI.GPIO_QSPI_SD2
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD2_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg)&^(0x100)|value<<8)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD2_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg) & 0x100) >> 8
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD2_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg)&^(0x80)|value<<7)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD2_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg) & 0x80) >> 7
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD2_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg)&^(0x40)|value<<6)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD2_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg) & 0x40) >> 6
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD2_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg)&^(0x30)|value<<4)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD2_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg) & 0x30) >> 4
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD2_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg)&^(0x8)|value<<3)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD2_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg) & 0x8) >> 3
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD2_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg)&^(0x4)|value<<2)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD2_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg) & 0x4) >> 2
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD2_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg)&^(0x2)|value<<1)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD2_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg) & 0x2) >> 1
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD2_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg)&^(0x1)|value)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD2_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SD2.Reg) & 0x1
}

// PADS_QSPI.GPIO_QSPI_SD3
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD3_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg)&^(0x100)|value<<8)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD3_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg) & 0x100) >> 8
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD3_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg)&^(0x80)|value<<7)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD3_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg) & 0x80) >> 7
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD3_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg)&^(0x40)|value<<6)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD3_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg) & 0x40) >> 6
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD3_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg)&^(0x30)|value<<4)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD3_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg) & 0x30) >> 4
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD3_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg)&^(0x8)|value<<3)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD3_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg) & 0x8) >> 3
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD3_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg)&^(0x4)|value<<2)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD3_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg) & 0x4) >> 2
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD3_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg)&^(0x2)|value<<1)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD3_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg) & 0x2) >> 1
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SD3_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg)&^(0x1)|value)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SD3_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SD3.Reg) & 0x1
}

// PADS_QSPI.GPIO_QSPI_SS
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SS_ISO(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg)&^(0x100)|value<<8)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SS_ISO() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg) & 0x100) >> 8
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SS_OD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg)&^(0x80)|value<<7)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SS_OD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg) & 0x80) >> 7
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SS_IE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg)&^(0x40)|value<<6)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SS_IE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg) & 0x40) >> 6
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SS_DRIVE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg)&^(0x30)|value<<4)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SS_DRIVE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg) & 0x30) >> 4
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SS_PUE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg)&^(0x8)|value<<3)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SS_PUE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg) & 0x8) >> 3
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SS_PDE(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg)&^(0x4)|value<<2)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SS_PDE() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg) & 0x4) >> 2
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SS_SCHMITT(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg)&^(0x2)|value<<1)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SS_SCHMITT() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg) & 0x2) >> 1
}
func (o *PADS_QSPI_Type) SetGPIO_QSPI_SS_SLEWFAST(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg)&^(0x1)|value)
}
func (o *PADS_QSPI_Type) GetGPIO_QSPI_SS_SLEWFAST() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SS.Reg) & 0x1
}

type IO_QSPI_Type struct {
	USBPHY_DP_STATUS                  volatile.Register32 // 0x0
	USBPHY_DP_CTRL                    volatile.Register32 // 0x4
	USBPHY_DM_STATUS                  volatile.Register32 // 0x8
	USBPHY_DM_CTRL                    volatile.Register32 // 0xC
	GPIO_QSPI_SCLK_STATUS             volatile.Register32 // 0x10
	GPIO_QSPI_SCLK_CTRL               volatile.Register32 // 0x14
	GPIO_QSPI_SS_STATUS               volatile.Register32 // 0x18
	GPIO_QSPI_SS_CTRL                 volatile.Register32 // 0x1C
	GPIO_QSPI_SD0_STATUS              volatile.Register32 // 0x20
	GPIO_QSPI_SD0_CTRL                volatile.Register32 // 0x24
	GPIO_QSPI_SD1_STATUS              volatile.Register32 // 0x28
	GPIO_QSPI_SD1_CTRL                volatile.Register32 // 0x2C
	GPIO_QSPI_SD2_STATUS              volatile.Register32 // 0x30
	GPIO_QSPI_SD2_CTRL                volatile.Register32 // 0x34
	GPIO_QSPI_SD3_STATUS              volatile.Register32 // 0x38
	GPIO_QSPI_SD3_CTRL                volatile.Register32 // 0x3C
	_                                 [448]byte
	IRQSUMMARY_PROC0_SECURE           volatile.Register32 // 0x200
	IRQSUMMARY_PROC0_NONSECURE        volatile.Register32 // 0x204
	IRQSUMMARY_PROC1_SECURE           volatile.Register32 // 0x208
	IRQSUMMARY_PROC1_NONSECURE        volatile.Register32 // 0x20C
	IRQSUMMARY_DORMANT_WAKE_SECURE    volatile.Register32 // 0x210
	IRQSUMMARY_DORMANT_WAKE_NONSECURE volatile.Register32 // 0x214
	INTR                              volatile.Register32 // 0x218
	PROC0_INTE                        volatile.Register32 // 0x21C
	PROC0_INTF                        volatile.Register32 // 0x220
	PROC0_INTS                        volatile.Register32 // 0x224
	PROC1_INTE                        volatile.Register32 // 0x228
	PROC1_INTF                        volatile.Register32 // 0x22C
	PROC1_INTS                        volatile.Register32 // 0x230
	DORMANT_WAKE_INTE                 volatile.Register32 // 0x234
	DORMANT_WAKE_INTF                 volatile.Register32 // 0x238
	DORMANT_WAKE_INTS                 volatile.Register32 // 0x23C
}

// IO_QSPI.USBPHY_DP_STATUS
func (o *IO_QSPI_Type) SetUSBPHY_DP_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DP_STATUS.Reg, volatile.LoadUint32(&o.USBPHY_DP_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetUSBPHY_DP_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DP_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetUSBPHY_DP_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DP_STATUS.Reg, volatile.LoadUint32(&o.USBPHY_DP_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetUSBPHY_DP_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DP_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetUSBPHY_DP_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DP_STATUS.Reg, volatile.LoadUint32(&o.USBPHY_DP_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetUSBPHY_DP_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DP_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetUSBPHY_DP_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DP_STATUS.Reg, volatile.LoadUint32(&o.USBPHY_DP_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetUSBPHY_DP_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DP_STATUS.Reg) & 0x200) >> 9
}

// IO_QSPI.USBPHY_DP_CTRL
func (o *IO_QSPI_Type) SetUSBPHY_DP_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DP_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_QSPI_Type) GetUSBPHY_DP_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_QSPI_Type) SetUSBPHY_DP_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DP_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_QSPI_Type) GetUSBPHY_DP_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_QSPI_Type) SetUSBPHY_DP_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DP_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_QSPI_Type) GetUSBPHY_DP_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_QSPI_Type) SetUSBPHY_DP_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DP_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_QSPI_Type) GetUSBPHY_DP_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_QSPI_Type) SetUSBPHY_DP_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DP_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_QSPI_Type) GetUSBPHY_DP_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.USBPHY_DP_CTRL.Reg) & 0x1f
}

// IO_QSPI.USBPHY_DM_STATUS
func (o *IO_QSPI_Type) SetUSBPHY_DM_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DM_STATUS.Reg, volatile.LoadUint32(&o.USBPHY_DM_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetUSBPHY_DM_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DM_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetUSBPHY_DM_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DM_STATUS.Reg, volatile.LoadUint32(&o.USBPHY_DM_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetUSBPHY_DM_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DM_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetUSBPHY_DM_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DM_STATUS.Reg, volatile.LoadUint32(&o.USBPHY_DM_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetUSBPHY_DM_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DM_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetUSBPHY_DM_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DM_STATUS.Reg, volatile.LoadUint32(&o.USBPHY_DM_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetUSBPHY_DM_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DM_STATUS.Reg) & 0x200) >> 9
}

// IO_QSPI.USBPHY_DM_CTRL
func (o *IO_QSPI_Type) SetUSBPHY_DM_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DM_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_QSPI_Type) GetUSBPHY_DM_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_QSPI_Type) SetUSBPHY_DM_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DM_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_QSPI_Type) GetUSBPHY_DM_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_QSPI_Type) SetUSBPHY_DM_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DM_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_QSPI_Type) GetUSBPHY_DM_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_QSPI_Type) SetUSBPHY_DM_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DM_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_QSPI_Type) GetUSBPHY_DM_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_QSPI_Type) SetUSBPHY_DM_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DM_CTRL.Reg, volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_QSPI_Type) GetUSBPHY_DM_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.USBPHY_DM_CTRL.Reg) & 0x1f
}

// IO_QSPI.GPIO_QSPI_SCLK_STATUS
func (o *IO_QSPI_Type) SetGPIO_QSPI_SCLK_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SCLK_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SCLK_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SCLK_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SCLK_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SCLK_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SCLK_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SCLK_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK_STATUS.Reg) & 0x200) >> 9
}

// IO_QSPI.GPIO_QSPI_SCLK_CTRL
func (o *IO_QSPI_Type) SetGPIO_QSPI_SCLK_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SCLK_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SCLK_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SCLK_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SCLK_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SCLK_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SCLK_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SCLK_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SCLK_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SCLK_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SCLK_CTRL.Reg) & 0x1f
}

// IO_QSPI.GPIO_QSPI_SS_STATUS
func (o *IO_QSPI_Type) SetGPIO_QSPI_SS_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SS_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SS_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SS_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SS_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SS_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SS_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SS_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS_STATUS.Reg) & 0x200) >> 9
}

// IO_QSPI.GPIO_QSPI_SS_CTRL
func (o *IO_QSPI_Type) SetGPIO_QSPI_SS_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SS_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SS_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SS_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SS_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SS_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SS_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SS_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SS_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SS_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SS_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SS_CTRL.Reg) & 0x1f
}

// IO_QSPI.GPIO_QSPI_SD0_STATUS
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD0_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD0_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD0_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD0_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD0_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD0_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD0_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD0_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0_STATUS.Reg) & 0x200) >> 9
}

// IO_QSPI.GPIO_QSPI_SD0_CTRL
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD0_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD0_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD0_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD0_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD0_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD0_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD0_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD0_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD0_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD0_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD0_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SD0_CTRL.Reg) & 0x1f
}

// IO_QSPI.GPIO_QSPI_SD1_STATUS
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD1_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD1_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD1_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD1_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD1_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD1_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD1_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD1_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1_STATUS.Reg) & 0x200) >> 9
}

// IO_QSPI.GPIO_QSPI_SD1_CTRL
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD1_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD1_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD1_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD1_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD1_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD1_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD1_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD1_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD1_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD1_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD1_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SD1_CTRL.Reg) & 0x1f
}

// IO_QSPI.GPIO_QSPI_SD2_STATUS
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD2_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD2_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD2_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD2_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD2_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD2_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD2_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD2_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2_STATUS.Reg) & 0x200) >> 9
}

// IO_QSPI.GPIO_QSPI_SD2_CTRL
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD2_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD2_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD2_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD2_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD2_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD2_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD2_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD2_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD2_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD2_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD2_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SD2_CTRL.Reg) & 0x1f
}

// IO_QSPI.GPIO_QSPI_SD3_STATUS
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD3_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD3_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD3_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD3_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD3_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD3_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD3_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3_STATUS.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD3_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3_STATUS.Reg) & 0x200) >> 9
}

// IO_QSPI.GPIO_QSPI_SD3_CTRL
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD3_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD3_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD3_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD3_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD3_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD3_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD3_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD3_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_QSPI_Type) SetGPIO_QSPI_SD3_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO_QSPI_SD3_CTRL.Reg, volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_QSPI_Type) GetGPIO_QSPI_SD3_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO_QSPI_SD3_CTRL.Reg) & 0x1f
}

// IO_QSPI.IRQSUMMARY_PROC0_SECURE
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD0() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SS(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SS() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SCLK(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SCLK() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_SECURE_USBPHY_DM(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_SECURE_USBPHY_DM() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_SECURE_USBPHY_DP(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_SECURE_USBPHY_DP() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE.Reg) & 0x1
}

// IO_QSPI.IRQSUMMARY_PROC0_NONSECURE
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD0() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SS(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SS() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SCLK(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SCLK() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_NONSECURE_USBPHY_DM(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_NONSECURE_USBPHY_DM() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC0_NONSECURE_USBPHY_DP(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC0_NONSECURE_USBPHY_DP() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE.Reg) & 0x1
}

// IO_QSPI.IRQSUMMARY_PROC1_SECURE
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD0() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SS(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SS() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SCLK(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SCLK() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_SECURE_USBPHY_DM(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_SECURE_USBPHY_DM() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_SECURE_USBPHY_DP(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_SECURE_USBPHY_DP() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE.Reg) & 0x1
}

// IO_QSPI.IRQSUMMARY_PROC1_NONSECURE
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD0() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SS(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SS() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SCLK(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SCLK() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_NONSECURE_USBPHY_DM(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_NONSECURE_USBPHY_DM() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_PROC1_NONSECURE_USBPHY_DP(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_PROC1_NONSECURE_USBPHY_DP() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE.Reg) & 0x1
}

// IO_QSPI.IRQSUMMARY_DORMANT_WAKE_SECURE
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD0() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SS(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SS() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SCLK(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SCLK() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DM(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DM() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DP(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DP() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE.Reg) & 0x1
}

// IO_QSPI.IRQSUMMARY_DORMANT_WAKE_NONSECURE
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD0() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SS(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SS() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SCLK(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SCLK() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DM(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DM() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DP(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DP() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE.Reg) & 0x1
}

// IO_QSPI.INTR: Raw Interrupts
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetINTR_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetINTR_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetINTR_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetINTR_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetINTR_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetINTR_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetINTR_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetINTR_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetINTR_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetINTR_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetINTR_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetINTR_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetINTR_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetINTR_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetINTR_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetINTR_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetINTR_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetINTR_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// IO_QSPI.PROC0_INTE: Interrupt Enable for proc0
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetPROC0_INTE_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetPROC0_INTE_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetPROC0_INTE_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetPROC0_INTE_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetPROC0_INTE_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetPROC0_INTE_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetPROC0_INTE_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetPROC0_INTE_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetPROC0_INTE_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE.Reg, volatile.LoadUint32(&o.PROC0_INTE.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetPROC0_INTE_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTE.Reg) & 0x1
}

// IO_QSPI.PROC0_INTF: Interrupt Force for proc0
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetPROC0_INTF_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetPROC0_INTF_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetPROC0_INTF_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetPROC0_INTF_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetPROC0_INTF_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetPROC0_INTF_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetPROC0_INTF_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetPROC0_INTF_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetPROC0_INTF_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF.Reg, volatile.LoadUint32(&o.PROC0_INTF.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetPROC0_INTF_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTF.Reg) & 0x1
}

// IO_QSPI.PROC0_INTS: Interrupt status after masking & forcing for proc0
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetPROC0_INTS_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetPROC0_INTS_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetPROC0_INTS_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetPROC0_INTS_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetPROC0_INTS_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetPROC0_INTS_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetPROC0_INTS_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetPROC0_INTS_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetPROC0_INTS_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS.Reg, volatile.LoadUint32(&o.PROC0_INTS.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetPROC0_INTS_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTS.Reg) & 0x1
}

// IO_QSPI.PROC1_INTE: Interrupt Enable for proc1
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetPROC1_INTE_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetPROC1_INTE_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetPROC1_INTE_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetPROC1_INTE_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetPROC1_INTE_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetPROC1_INTE_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetPROC1_INTE_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetPROC1_INTE_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetPROC1_INTE_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE.Reg, volatile.LoadUint32(&o.PROC1_INTE.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetPROC1_INTE_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTE.Reg) & 0x1
}

// IO_QSPI.PROC1_INTF: Interrupt Force for proc1
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetPROC1_INTF_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetPROC1_INTF_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetPROC1_INTF_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetPROC1_INTF_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetPROC1_INTF_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetPROC1_INTF_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetPROC1_INTF_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetPROC1_INTF_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetPROC1_INTF_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF.Reg, volatile.LoadUint32(&o.PROC1_INTF.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetPROC1_INTF_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTF.Reg) & 0x1
}

// IO_QSPI.PROC1_INTS: Interrupt status after masking & forcing for proc1
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetPROC1_INTS_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetPROC1_INTS_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetPROC1_INTS_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetPROC1_INTS_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetPROC1_INTS_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetPROC1_INTS_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetPROC1_INTS_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetPROC1_INTS_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetPROC1_INTS_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS.Reg, volatile.LoadUint32(&o.PROC1_INTS.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetPROC1_INTS_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTS.Reg) & 0x1
}

// IO_QSPI.DORMANT_WAKE_INTE: Interrupt Enable for dormant_wake
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTE_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTE_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTE.Reg) & 0x1
}

// IO_QSPI.DORMANT_WAKE_INTF: Interrupt Force for dormant_wake
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTF_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTF_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTF.Reg) & 0x1
}

// IO_QSPI.DORMANT_WAKE_INTS: Interrupt status after masking & forcing for dormant_wake
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x80000000) >> 31
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x40000000) >> 30
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x20000000) >> 29
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x10000000) >> 28
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x8000000) >> 27
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x4000000) >> 26
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x2000000) >> 25
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x1000000) >> 24
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x800000)|value<<23)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x800000) >> 23
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x400000)|value<<22)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x400000) >> 22
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x200000)|value<<21)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x200000) >> 21
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x100000)|value<<20)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x100000) >> 20
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x80000)|value<<19)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x80000) >> 19
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x40000)|value<<18)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x40000) >> 18
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x20000) >> 17
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x10000)|value<<16)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x10000) >> 16
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x8000)|value<<15)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x8000) >> 15
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x4000)|value<<14)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x4000) >> 14
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x2000) >> 13
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x1000)|value<<12)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x1000) >> 12
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x800)|value<<11)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x800) >> 11
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x400)|value<<10)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x400) >> 10
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x200)|value<<9)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x200) >> 9
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x100)|value<<8)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x100) >> 8
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_USBPHY_DM_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x80)|value<<7)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_USBPHY_DM_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x80) >> 7
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_USBPHY_DM_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x40)|value<<6)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_USBPHY_DM_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x40) >> 6
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_USBPHY_DM_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x20)|value<<5)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_USBPHY_DM_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x20) >> 5
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_USBPHY_DM_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x10)|value<<4)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_USBPHY_DM_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x10) >> 4
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_USBPHY_DP_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x8)|value<<3)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_USBPHY_DP_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x8) >> 3
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_USBPHY_DP_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x4)|value<<2)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_USBPHY_DP_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x4) >> 2
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_USBPHY_DP_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x2)|value<<1)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_USBPHY_DP_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x2) >> 1
}
func (o *IO_QSPI_Type) SetDORMANT_WAKE_INTS_USBPHY_DP_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg)&^(0x1)|value)
}
func (o *IO_QSPI_Type) GetDORMANT_WAKE_INTS_USBPHY_DP_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTS.Reg) & 0x1
}

type IO_BANK0_Type struct {
	GPIO0_STATUS                       volatile.Register32 // 0x0
	GPIO0_CTRL                         volatile.Register32 // 0x4
	GPIO1_STATUS                       volatile.Register32 // 0x8
	GPIO1_CTRL                         volatile.Register32 // 0xC
	GPIO2_STATUS                       volatile.Register32 // 0x10
	GPIO2_CTRL                         volatile.Register32 // 0x14
	GPIO3_STATUS                       volatile.Register32 // 0x18
	GPIO3_CTRL                         volatile.Register32 // 0x1C
	GPIO4_STATUS                       volatile.Register32 // 0x20
	GPIO4_CTRL                         volatile.Register32 // 0x24
	GPIO5_STATUS                       volatile.Register32 // 0x28
	GPIO5_CTRL                         volatile.Register32 // 0x2C
	GPIO6_STATUS                       volatile.Register32 // 0x30
	GPIO6_CTRL                         volatile.Register32 // 0x34
	GPIO7_STATUS                       volatile.Register32 // 0x38
	GPIO7_CTRL                         volatile.Register32 // 0x3C
	GPIO8_STATUS                       volatile.Register32 // 0x40
	GPIO8_CTRL                         volatile.Register32 // 0x44
	GPIO9_STATUS                       volatile.Register32 // 0x48
	GPIO9_CTRL                         volatile.Register32 // 0x4C
	GPIO10_STATUS                      volatile.Register32 // 0x50
	GPIO10_CTRL                        volatile.Register32 // 0x54
	GPIO11_STATUS                      volatile.Register32 // 0x58
	GPIO11_CTRL                        volatile.Register32 // 0x5C
	GPIO12_STATUS                      volatile.Register32 // 0x60
	GPIO12_CTRL                        volatile.Register32 // 0x64
	GPIO13_STATUS                      volatile.Register32 // 0x68
	GPIO13_CTRL                        volatile.Register32 // 0x6C
	GPIO14_STATUS                      volatile.Register32 // 0x70
	GPIO14_CTRL                        volatile.Register32 // 0x74
	GPIO15_STATUS                      volatile.Register32 // 0x78
	GPIO15_CTRL                        volatile.Register32 // 0x7C
	GPIO16_STATUS                      volatile.Register32 // 0x80
	GPIO16_CTRL                        volatile.Register32 // 0x84
	GPIO17_STATUS                      volatile.Register32 // 0x88
	GPIO17_CTRL                        volatile.Register32 // 0x8C
	GPIO18_STATUS                      volatile.Register32 // 0x90
	GPIO18_CTRL                        volatile.Register32 // 0x94
	GPIO19_STATUS                      volatile.Register32 // 0x98
	GPIO19_CTRL                        volatile.Register32 // 0x9C
	GPIO20_STATUS                      volatile.Register32 // 0xA0
	GPIO20_CTRL                        volatile.Register32 // 0xA4
	GPIO21_STATUS                      volatile.Register32 // 0xA8
	GPIO21_CTRL                        volatile.Register32 // 0xAC
	GPIO22_STATUS                      volatile.Register32 // 0xB0
	GPIO22_CTRL                        volatile.Register32 // 0xB4
	GPIO23_STATUS                      volatile.Register32 // 0xB8
	GPIO23_CTRL                        volatile.Register32 // 0xBC
	GPIO24_STATUS                      volatile.Register32 // 0xC0
	GPIO24_CTRL                        volatile.Register32 // 0xC4
	GPIO25_STATUS                      volatile.Register32 // 0xC8
	GPIO25_CTRL                        volatile.Register32 // 0xCC
	GPIO26_STATUS                      volatile.Register32 // 0xD0
	GPIO26_CTRL                        volatile.Register32 // 0xD4
	GPIO27_STATUS                      volatile.Register32 // 0xD8
	GPIO27_CTRL                        volatile.Register32 // 0xDC
	GPIO28_STATUS                      volatile.Register32 // 0xE0
	GPIO28_CTRL                        volatile.Register32 // 0xE4
	GPIO29_STATUS                      volatile.Register32 // 0xE8
	GPIO29_CTRL                        volatile.Register32 // 0xEC
	GPIO30_STATUS                      volatile.Register32 // 0xF0
	GPIO30_CTRL                        volatile.Register32 // 0xF4
	GPIO31_STATUS                      volatile.Register32 // 0xF8
	GPIO31_CTRL                        volatile.Register32 // 0xFC
	GPIO32_STATUS                      volatile.Register32 // 0x100
	GPIO32_CTRL                        volatile.Register32 // 0x104
	GPIO33_STATUS                      volatile.Register32 // 0x108
	GPIO33_CTRL                        volatile.Register32 // 0x10C
	GPIO34_STATUS                      volatile.Register32 // 0x110
	GPIO34_CTRL                        volatile.Register32 // 0x114
	GPIO35_STATUS                      volatile.Register32 // 0x118
	GPIO35_CTRL                        volatile.Register32 // 0x11C
	GPIO36_STATUS                      volatile.Register32 // 0x120
	GPIO36_CTRL                        volatile.Register32 // 0x124
	GPIO37_STATUS                      volatile.Register32 // 0x128
	GPIO37_CTRL                        volatile.Register32 // 0x12C
	GPIO38_STATUS                      volatile.Register32 // 0x130
	GPIO38_CTRL                        volatile.Register32 // 0x134
	GPIO39_STATUS                      volatile.Register32 // 0x138
	GPIO39_CTRL                        volatile.Register32 // 0x13C
	GPIO40_STATUS                      volatile.Register32 // 0x140
	GPIO40_CTRL                        volatile.Register32 // 0x144
	GPIO41_STATUS                      volatile.Register32 // 0x148
	GPIO41_CTRL                        volatile.Register32 // 0x14C
	GPIO42_STATUS                      volatile.Register32 // 0x150
	GPIO42_CTRL                        volatile.Register32 // 0x154
	GPIO43_STATUS                      volatile.Register32 // 0x158
	GPIO43_CTRL                        volatile.Register32 // 0x15C
	GPIO44_STATUS                      volatile.Register32 // 0x160
	GPIO44_CTRL                        volatile.Register32 // 0x164
	GPIO45_STATUS                      volatile.Register32 // 0x168
	GPIO45_CTRL                        volatile.Register32 // 0x16C
	GPIO46_STATUS                      volatile.Register32 // 0x170
	GPIO46_CTRL                        volatile.Register32 // 0x174
	GPIO47_STATUS                      volatile.Register32 // 0x178
	GPIO47_CTRL                        volatile.Register32 // 0x17C
	_                                  [128]byte
	IRQSUMMARY_PROC0_SECURE0           volatile.Register32 // 0x200
	IRQSUMMARY_PROC0_SECURE1           volatile.Register32 // 0x204
	IRQSUMMARY_PROC0_NONSECURE0        volatile.Register32 // 0x208
	IRQSUMMARY_PROC0_NONSECURE1        volatile.Register32 // 0x20C
	IRQSUMMARY_PROC1_SECURE0           volatile.Register32 // 0x210
	IRQSUMMARY_PROC1_SECURE1           volatile.Register32 // 0x214
	IRQSUMMARY_PROC1_NONSECURE0        volatile.Register32 // 0x218
	IRQSUMMARY_PROC1_NONSECURE1        volatile.Register32 // 0x21C
	IRQSUMMARY_DORMANT_WAKE_SECURE0    volatile.Register32 // 0x220
	IRQSUMMARY_DORMANT_WAKE_SECURE1    volatile.Register32 // 0x224
	IRQSUMMARY_DORMANT_WAKE_NONSECURE0 volatile.Register32 // 0x228
	IRQSUMMARY_DORMANT_WAKE_NONSECURE1 volatile.Register32 // 0x22C
	INTR0                              volatile.Register32 // 0x230
	INTR1                              volatile.Register32 // 0x234
	INTR2                              volatile.Register32 // 0x238
	INTR3                              volatile.Register32 // 0x23C
	INTR4                              volatile.Register32 // 0x240
	INTR5                              volatile.Register32 // 0x244
	PROC0_INTE0                        volatile.Register32 // 0x248
	PROC0_INTE1                        volatile.Register32 // 0x24C
	PROC0_INTE2                        volatile.Register32 // 0x250
	PROC0_INTE3                        volatile.Register32 // 0x254
	PROC0_INTE4                        volatile.Register32 // 0x258
	PROC0_INTE5                        volatile.Register32 // 0x25C
	PROC0_INTF0                        volatile.Register32 // 0x260
	PROC0_INTF1                        volatile.Register32 // 0x264
	PROC0_INTF2                        volatile.Register32 // 0x268
	PROC0_INTF3                        volatile.Register32 // 0x26C
	PROC0_INTF4                        volatile.Register32 // 0x270
	PROC0_INTF5                        volatile.Register32 // 0x274
	PROC0_INTS0                        volatile.Register32 // 0x278
	PROC0_INTS1                        volatile.Register32 // 0x27C
	PROC0_INTS2                        volatile.Register32 // 0x280
	PROC0_INTS3                        volatile.Register32 // 0x284
	PROC0_INTS4                        volatile.Register32 // 0x288
	PROC0_INTS5                        volatile.Register32 // 0x28C
	PROC1_INTE0                        volatile.Register32 // 0x290
	PROC1_INTE1                        volatile.Register32 // 0x294
	PROC1_INTE2                        volatile.Register32 // 0x298
	PROC1_INTE3                        volatile.Register32 // 0x29C
	PROC1_INTE4                        volatile.Register32 // 0x2A0
	PROC1_INTE5                        volatile.Register32 // 0x2A4
	PROC1_INTF0                        volatile.Register32 // 0x2A8
	PROC1_INTF1                        volatile.Register32 // 0x2AC
	PROC1_INTF2                        volatile.Register32 // 0x2B0
	PROC1_INTF3                        volatile.Register32 // 0x2B4
	PROC1_INTF4                        volatile.Register32 // 0x2B8
	PROC1_INTF5                        volatile.Register32 // 0x2BC
	PROC1_INTS0                        volatile.Register32 // 0x2C0
	PROC1_INTS1                        volatile.Register32 // 0x2C4
	PROC1_INTS2                        volatile.Register32 // 0x2C8
	PROC1_INTS3                        volatile.Register32 // 0x2CC
	PROC1_INTS4                        volatile.Register32 // 0x2D0
	PROC1_INTS5                        volatile.Register32 // 0x2D4
	DORMANT_WAKE_INTE0                 volatile.Register32 // 0x2D8
	DORMANT_WAKE_INTE1                 volatile.Register32 // 0x2DC
	DORMANT_WAKE_INTE2                 volatile.Register32 // 0x2E0
	DORMANT_WAKE_INTE3                 volatile.Register32 // 0x2E4
	DORMANT_WAKE_INTE4                 volatile.Register32 // 0x2E8
	DORMANT_WAKE_INTE5                 volatile.Register32 // 0x2EC
	DORMANT_WAKE_INTF0                 volatile.Register32 // 0x2F0
	DORMANT_WAKE_INTF1                 volatile.Register32 // 0x2F4
	DORMANT_WAKE_INTF2                 volatile.Register32 // 0x2F8
	DORMANT_WAKE_INTF3                 volatile.Register32 // 0x2FC
	DORMANT_WAKE_INTF4                 volatile.Register32 // 0x300
	DORMANT_WAKE_INTF5                 volatile.Register32 // 0x304
	DORMANT_WAKE_INTS0                 volatile.Register32 // 0x308
	DORMANT_WAKE_INTS1                 volatile.Register32 // 0x30C
	DORMANT_WAKE_INTS2                 volatile.Register32 // 0x310
	DORMANT_WAKE_INTS3                 volatile.Register32 // 0x314
	DORMANT_WAKE_INTS4                 volatile.Register32 // 0x318
	DORMANT_WAKE_INTS5                 volatile.Register32 // 0x31C
}

// IO_BANK0.GPIO0_STATUS
func (o *IO_BANK0_Type) SetGPIO0_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO0_STATUS.Reg, volatile.LoadUint32(&o.GPIO0_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO0_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO0_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO0_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO0_STATUS.Reg, volatile.LoadUint32(&o.GPIO0_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO0_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO0_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO0_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO0_STATUS.Reg, volatile.LoadUint32(&o.GPIO0_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO0_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO0_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO0_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO0_STATUS.Reg, volatile.LoadUint32(&o.GPIO0_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO0_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO0_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO0_CTRL
func (o *IO_BANK0_Type) SetGPIO0_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO0_CTRL.Reg, volatile.LoadUint32(&o.GPIO0_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO0_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO0_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO0_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO0_CTRL.Reg, volatile.LoadUint32(&o.GPIO0_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO0_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO0_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO0_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO0_CTRL.Reg, volatile.LoadUint32(&o.GPIO0_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO0_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO0_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO0_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO0_CTRL.Reg, volatile.LoadUint32(&o.GPIO0_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO0_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO0_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO0_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO0_CTRL.Reg, volatile.LoadUint32(&o.GPIO0_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO0_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO0_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO1_STATUS
func (o *IO_BANK0_Type) SetGPIO1_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO1_STATUS.Reg, volatile.LoadUint32(&o.GPIO1_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO1_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO1_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO1_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO1_STATUS.Reg, volatile.LoadUint32(&o.GPIO1_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO1_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO1_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO1_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO1_STATUS.Reg, volatile.LoadUint32(&o.GPIO1_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO1_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO1_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO1_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO1_STATUS.Reg, volatile.LoadUint32(&o.GPIO1_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO1_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO1_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO1_CTRL
func (o *IO_BANK0_Type) SetGPIO1_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO1_CTRL.Reg, volatile.LoadUint32(&o.GPIO1_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO1_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO1_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO1_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO1_CTRL.Reg, volatile.LoadUint32(&o.GPIO1_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO1_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO1_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO1_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO1_CTRL.Reg, volatile.LoadUint32(&o.GPIO1_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO1_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO1_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO1_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO1_CTRL.Reg, volatile.LoadUint32(&o.GPIO1_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO1_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO1_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO1_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO1_CTRL.Reg, volatile.LoadUint32(&o.GPIO1_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO1_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO1_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO2_STATUS
func (o *IO_BANK0_Type) SetGPIO2_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO2_STATUS.Reg, volatile.LoadUint32(&o.GPIO2_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO2_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO2_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO2_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO2_STATUS.Reg, volatile.LoadUint32(&o.GPIO2_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO2_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO2_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO2_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO2_STATUS.Reg, volatile.LoadUint32(&o.GPIO2_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO2_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO2_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO2_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO2_STATUS.Reg, volatile.LoadUint32(&o.GPIO2_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO2_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO2_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO2_CTRL
func (o *IO_BANK0_Type) SetGPIO2_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO2_CTRL.Reg, volatile.LoadUint32(&o.GPIO2_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO2_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO2_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO2_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO2_CTRL.Reg, volatile.LoadUint32(&o.GPIO2_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO2_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO2_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO2_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO2_CTRL.Reg, volatile.LoadUint32(&o.GPIO2_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO2_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO2_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO2_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO2_CTRL.Reg, volatile.LoadUint32(&o.GPIO2_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO2_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO2_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO2_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO2_CTRL.Reg, volatile.LoadUint32(&o.GPIO2_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO2_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO2_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO3_STATUS
func (o *IO_BANK0_Type) SetGPIO3_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO3_STATUS.Reg, volatile.LoadUint32(&o.GPIO3_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO3_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO3_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO3_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO3_STATUS.Reg, volatile.LoadUint32(&o.GPIO3_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO3_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO3_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO3_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO3_STATUS.Reg, volatile.LoadUint32(&o.GPIO3_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO3_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO3_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO3_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO3_STATUS.Reg, volatile.LoadUint32(&o.GPIO3_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO3_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO3_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO3_CTRL
func (o *IO_BANK0_Type) SetGPIO3_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO3_CTRL.Reg, volatile.LoadUint32(&o.GPIO3_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO3_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO3_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO3_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO3_CTRL.Reg, volatile.LoadUint32(&o.GPIO3_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO3_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO3_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO3_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO3_CTRL.Reg, volatile.LoadUint32(&o.GPIO3_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO3_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO3_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO3_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO3_CTRL.Reg, volatile.LoadUint32(&o.GPIO3_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO3_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO3_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO3_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO3_CTRL.Reg, volatile.LoadUint32(&o.GPIO3_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO3_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO3_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO4_STATUS
func (o *IO_BANK0_Type) SetGPIO4_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO4_STATUS.Reg, volatile.LoadUint32(&o.GPIO4_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO4_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO4_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO4_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO4_STATUS.Reg, volatile.LoadUint32(&o.GPIO4_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO4_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO4_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO4_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO4_STATUS.Reg, volatile.LoadUint32(&o.GPIO4_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO4_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO4_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO4_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO4_STATUS.Reg, volatile.LoadUint32(&o.GPIO4_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO4_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO4_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO4_CTRL
func (o *IO_BANK0_Type) SetGPIO4_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO4_CTRL.Reg, volatile.LoadUint32(&o.GPIO4_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO4_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO4_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO4_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO4_CTRL.Reg, volatile.LoadUint32(&o.GPIO4_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO4_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO4_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO4_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO4_CTRL.Reg, volatile.LoadUint32(&o.GPIO4_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO4_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO4_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO4_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO4_CTRL.Reg, volatile.LoadUint32(&o.GPIO4_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO4_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO4_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO4_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO4_CTRL.Reg, volatile.LoadUint32(&o.GPIO4_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO4_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO4_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO5_STATUS
func (o *IO_BANK0_Type) SetGPIO5_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO5_STATUS.Reg, volatile.LoadUint32(&o.GPIO5_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO5_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO5_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO5_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO5_STATUS.Reg, volatile.LoadUint32(&o.GPIO5_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO5_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO5_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO5_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO5_STATUS.Reg, volatile.LoadUint32(&o.GPIO5_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO5_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO5_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO5_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO5_STATUS.Reg, volatile.LoadUint32(&o.GPIO5_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO5_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO5_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO5_CTRL
func (o *IO_BANK0_Type) SetGPIO5_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO5_CTRL.Reg, volatile.LoadUint32(&o.GPIO5_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO5_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO5_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO5_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO5_CTRL.Reg, volatile.LoadUint32(&o.GPIO5_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO5_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO5_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO5_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO5_CTRL.Reg, volatile.LoadUint32(&o.GPIO5_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO5_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO5_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO5_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO5_CTRL.Reg, volatile.LoadUint32(&o.GPIO5_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO5_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO5_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO5_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO5_CTRL.Reg, volatile.LoadUint32(&o.GPIO5_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO5_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO5_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO6_STATUS
func (o *IO_BANK0_Type) SetGPIO6_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO6_STATUS.Reg, volatile.LoadUint32(&o.GPIO6_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO6_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO6_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO6_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO6_STATUS.Reg, volatile.LoadUint32(&o.GPIO6_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO6_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO6_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO6_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO6_STATUS.Reg, volatile.LoadUint32(&o.GPIO6_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO6_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO6_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO6_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO6_STATUS.Reg, volatile.LoadUint32(&o.GPIO6_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO6_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO6_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO6_CTRL
func (o *IO_BANK0_Type) SetGPIO6_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO6_CTRL.Reg, volatile.LoadUint32(&o.GPIO6_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO6_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO6_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO6_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO6_CTRL.Reg, volatile.LoadUint32(&o.GPIO6_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO6_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO6_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO6_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO6_CTRL.Reg, volatile.LoadUint32(&o.GPIO6_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO6_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO6_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO6_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO6_CTRL.Reg, volatile.LoadUint32(&o.GPIO6_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO6_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO6_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO6_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO6_CTRL.Reg, volatile.LoadUint32(&o.GPIO6_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO6_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO6_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO7_STATUS
func (o *IO_BANK0_Type) SetGPIO7_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO7_STATUS.Reg, volatile.LoadUint32(&o.GPIO7_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO7_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO7_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO7_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO7_STATUS.Reg, volatile.LoadUint32(&o.GPIO7_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO7_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO7_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO7_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO7_STATUS.Reg, volatile.LoadUint32(&o.GPIO7_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO7_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO7_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO7_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO7_STATUS.Reg, volatile.LoadUint32(&o.GPIO7_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO7_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO7_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO7_CTRL
func (o *IO_BANK0_Type) SetGPIO7_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO7_CTRL.Reg, volatile.LoadUint32(&o.GPIO7_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO7_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO7_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO7_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO7_CTRL.Reg, volatile.LoadUint32(&o.GPIO7_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO7_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO7_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO7_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO7_CTRL.Reg, volatile.LoadUint32(&o.GPIO7_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO7_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO7_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO7_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO7_CTRL.Reg, volatile.LoadUint32(&o.GPIO7_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO7_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO7_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO7_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO7_CTRL.Reg, volatile.LoadUint32(&o.GPIO7_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO7_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO7_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO8_STATUS
func (o *IO_BANK0_Type) SetGPIO8_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO8_STATUS.Reg, volatile.LoadUint32(&o.GPIO8_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO8_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO8_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO8_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO8_STATUS.Reg, volatile.LoadUint32(&o.GPIO8_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO8_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO8_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO8_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO8_STATUS.Reg, volatile.LoadUint32(&o.GPIO8_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO8_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO8_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO8_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO8_STATUS.Reg, volatile.LoadUint32(&o.GPIO8_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO8_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO8_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO8_CTRL
func (o *IO_BANK0_Type) SetGPIO8_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO8_CTRL.Reg, volatile.LoadUint32(&o.GPIO8_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO8_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO8_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO8_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO8_CTRL.Reg, volatile.LoadUint32(&o.GPIO8_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO8_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO8_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO8_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO8_CTRL.Reg, volatile.LoadUint32(&o.GPIO8_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO8_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO8_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO8_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO8_CTRL.Reg, volatile.LoadUint32(&o.GPIO8_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO8_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO8_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO8_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO8_CTRL.Reg, volatile.LoadUint32(&o.GPIO8_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO8_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO8_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO9_STATUS
func (o *IO_BANK0_Type) SetGPIO9_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO9_STATUS.Reg, volatile.LoadUint32(&o.GPIO9_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO9_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO9_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO9_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO9_STATUS.Reg, volatile.LoadUint32(&o.GPIO9_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO9_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO9_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO9_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO9_STATUS.Reg, volatile.LoadUint32(&o.GPIO9_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO9_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO9_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO9_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO9_STATUS.Reg, volatile.LoadUint32(&o.GPIO9_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO9_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO9_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO9_CTRL
func (o *IO_BANK0_Type) SetGPIO9_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO9_CTRL.Reg, volatile.LoadUint32(&o.GPIO9_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO9_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO9_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO9_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO9_CTRL.Reg, volatile.LoadUint32(&o.GPIO9_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO9_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO9_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO9_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO9_CTRL.Reg, volatile.LoadUint32(&o.GPIO9_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO9_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO9_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO9_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO9_CTRL.Reg, volatile.LoadUint32(&o.GPIO9_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO9_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO9_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO9_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO9_CTRL.Reg, volatile.LoadUint32(&o.GPIO9_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO9_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO9_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO10_STATUS
func (o *IO_BANK0_Type) SetGPIO10_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO10_STATUS.Reg, volatile.LoadUint32(&o.GPIO10_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO10_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO10_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO10_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO10_STATUS.Reg, volatile.LoadUint32(&o.GPIO10_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO10_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO10_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO10_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO10_STATUS.Reg, volatile.LoadUint32(&o.GPIO10_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO10_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO10_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO10_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO10_STATUS.Reg, volatile.LoadUint32(&o.GPIO10_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO10_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO10_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO10_CTRL
func (o *IO_BANK0_Type) SetGPIO10_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO10_CTRL.Reg, volatile.LoadUint32(&o.GPIO10_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO10_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO10_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO10_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO10_CTRL.Reg, volatile.LoadUint32(&o.GPIO10_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO10_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO10_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO10_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO10_CTRL.Reg, volatile.LoadUint32(&o.GPIO10_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO10_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO10_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO10_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO10_CTRL.Reg, volatile.LoadUint32(&o.GPIO10_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO10_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO10_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO10_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO10_CTRL.Reg, volatile.LoadUint32(&o.GPIO10_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO10_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO10_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO11_STATUS
func (o *IO_BANK0_Type) SetGPIO11_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO11_STATUS.Reg, volatile.LoadUint32(&o.GPIO11_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO11_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO11_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO11_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO11_STATUS.Reg, volatile.LoadUint32(&o.GPIO11_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO11_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO11_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO11_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO11_STATUS.Reg, volatile.LoadUint32(&o.GPIO11_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO11_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO11_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO11_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO11_STATUS.Reg, volatile.LoadUint32(&o.GPIO11_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO11_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO11_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO11_CTRL
func (o *IO_BANK0_Type) SetGPIO11_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO11_CTRL.Reg, volatile.LoadUint32(&o.GPIO11_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO11_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO11_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO11_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO11_CTRL.Reg, volatile.LoadUint32(&o.GPIO11_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO11_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO11_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO11_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO11_CTRL.Reg, volatile.LoadUint32(&o.GPIO11_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO11_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO11_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO11_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO11_CTRL.Reg, volatile.LoadUint32(&o.GPIO11_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO11_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO11_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO11_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO11_CTRL.Reg, volatile.LoadUint32(&o.GPIO11_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO11_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO11_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO12_STATUS
func (o *IO_BANK0_Type) SetGPIO12_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO12_STATUS.Reg, volatile.LoadUint32(&o.GPIO12_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO12_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO12_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO12_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO12_STATUS.Reg, volatile.LoadUint32(&o.GPIO12_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO12_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO12_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO12_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO12_STATUS.Reg, volatile.LoadUint32(&o.GPIO12_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO12_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO12_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO12_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO12_STATUS.Reg, volatile.LoadUint32(&o.GPIO12_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO12_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO12_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO12_CTRL
func (o *IO_BANK0_Type) SetGPIO12_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO12_CTRL.Reg, volatile.LoadUint32(&o.GPIO12_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO12_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO12_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO12_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO12_CTRL.Reg, volatile.LoadUint32(&o.GPIO12_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO12_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO12_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO12_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO12_CTRL.Reg, volatile.LoadUint32(&o.GPIO12_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO12_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO12_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO12_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO12_CTRL.Reg, volatile.LoadUint32(&o.GPIO12_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO12_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO12_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO12_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO12_CTRL.Reg, volatile.LoadUint32(&o.GPIO12_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO12_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO12_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO13_STATUS
func (o *IO_BANK0_Type) SetGPIO13_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO13_STATUS.Reg, volatile.LoadUint32(&o.GPIO13_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO13_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO13_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO13_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO13_STATUS.Reg, volatile.LoadUint32(&o.GPIO13_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO13_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO13_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO13_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO13_STATUS.Reg, volatile.LoadUint32(&o.GPIO13_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO13_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO13_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO13_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO13_STATUS.Reg, volatile.LoadUint32(&o.GPIO13_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO13_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO13_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO13_CTRL
func (o *IO_BANK0_Type) SetGPIO13_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO13_CTRL.Reg, volatile.LoadUint32(&o.GPIO13_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO13_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO13_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO13_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO13_CTRL.Reg, volatile.LoadUint32(&o.GPIO13_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO13_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO13_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO13_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO13_CTRL.Reg, volatile.LoadUint32(&o.GPIO13_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO13_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO13_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO13_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO13_CTRL.Reg, volatile.LoadUint32(&o.GPIO13_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO13_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO13_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO13_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO13_CTRL.Reg, volatile.LoadUint32(&o.GPIO13_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO13_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO13_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO14_STATUS
func (o *IO_BANK0_Type) SetGPIO14_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO14_STATUS.Reg, volatile.LoadUint32(&o.GPIO14_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO14_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO14_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO14_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO14_STATUS.Reg, volatile.LoadUint32(&o.GPIO14_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO14_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO14_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO14_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO14_STATUS.Reg, volatile.LoadUint32(&o.GPIO14_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO14_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO14_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO14_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO14_STATUS.Reg, volatile.LoadUint32(&o.GPIO14_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO14_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO14_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO14_CTRL
func (o *IO_BANK0_Type) SetGPIO14_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO14_CTRL.Reg, volatile.LoadUint32(&o.GPIO14_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO14_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO14_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO14_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO14_CTRL.Reg, volatile.LoadUint32(&o.GPIO14_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO14_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO14_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO14_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO14_CTRL.Reg, volatile.LoadUint32(&o.GPIO14_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO14_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO14_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO14_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO14_CTRL.Reg, volatile.LoadUint32(&o.GPIO14_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO14_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO14_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO14_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO14_CTRL.Reg, volatile.LoadUint32(&o.GPIO14_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO14_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO14_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO15_STATUS
func (o *IO_BANK0_Type) SetGPIO15_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO15_STATUS.Reg, volatile.LoadUint32(&o.GPIO15_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO15_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO15_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO15_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO15_STATUS.Reg, volatile.LoadUint32(&o.GPIO15_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO15_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO15_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO15_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO15_STATUS.Reg, volatile.LoadUint32(&o.GPIO15_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO15_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO15_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO15_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO15_STATUS.Reg, volatile.LoadUint32(&o.GPIO15_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO15_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO15_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO15_CTRL
func (o *IO_BANK0_Type) SetGPIO15_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO15_CTRL.Reg, volatile.LoadUint32(&o.GPIO15_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO15_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO15_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO15_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO15_CTRL.Reg, volatile.LoadUint32(&o.GPIO15_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO15_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO15_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO15_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO15_CTRL.Reg, volatile.LoadUint32(&o.GPIO15_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO15_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO15_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO15_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO15_CTRL.Reg, volatile.LoadUint32(&o.GPIO15_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO15_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO15_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO15_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO15_CTRL.Reg, volatile.LoadUint32(&o.GPIO15_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO15_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO15_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO16_STATUS
func (o *IO_BANK0_Type) SetGPIO16_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO16_STATUS.Reg, volatile.LoadUint32(&o.GPIO16_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO16_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO16_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO16_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO16_STATUS.Reg, volatile.LoadUint32(&o.GPIO16_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO16_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO16_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO16_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO16_STATUS.Reg, volatile.LoadUint32(&o.GPIO16_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO16_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO16_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO16_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO16_STATUS.Reg, volatile.LoadUint32(&o.GPIO16_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO16_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO16_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO16_CTRL
func (o *IO_BANK0_Type) SetGPIO16_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO16_CTRL.Reg, volatile.LoadUint32(&o.GPIO16_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO16_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO16_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO16_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO16_CTRL.Reg, volatile.LoadUint32(&o.GPIO16_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO16_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO16_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO16_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO16_CTRL.Reg, volatile.LoadUint32(&o.GPIO16_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO16_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO16_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO16_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO16_CTRL.Reg, volatile.LoadUint32(&o.GPIO16_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO16_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO16_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO16_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO16_CTRL.Reg, volatile.LoadUint32(&o.GPIO16_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO16_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO16_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO17_STATUS
func (o *IO_BANK0_Type) SetGPIO17_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO17_STATUS.Reg, volatile.LoadUint32(&o.GPIO17_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO17_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO17_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO17_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO17_STATUS.Reg, volatile.LoadUint32(&o.GPIO17_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO17_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO17_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO17_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO17_STATUS.Reg, volatile.LoadUint32(&o.GPIO17_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO17_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO17_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO17_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO17_STATUS.Reg, volatile.LoadUint32(&o.GPIO17_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO17_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO17_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO17_CTRL
func (o *IO_BANK0_Type) SetGPIO17_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO17_CTRL.Reg, volatile.LoadUint32(&o.GPIO17_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO17_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO17_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO17_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO17_CTRL.Reg, volatile.LoadUint32(&o.GPIO17_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO17_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO17_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO17_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO17_CTRL.Reg, volatile.LoadUint32(&o.GPIO17_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO17_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO17_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO17_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO17_CTRL.Reg, volatile.LoadUint32(&o.GPIO17_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO17_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO17_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO17_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO17_CTRL.Reg, volatile.LoadUint32(&o.GPIO17_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO17_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO17_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO18_STATUS
func (o *IO_BANK0_Type) SetGPIO18_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO18_STATUS.Reg, volatile.LoadUint32(&o.GPIO18_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO18_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO18_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO18_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO18_STATUS.Reg, volatile.LoadUint32(&o.GPIO18_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO18_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO18_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO18_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO18_STATUS.Reg, volatile.LoadUint32(&o.GPIO18_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO18_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO18_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO18_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO18_STATUS.Reg, volatile.LoadUint32(&o.GPIO18_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO18_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO18_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO18_CTRL
func (o *IO_BANK0_Type) SetGPIO18_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO18_CTRL.Reg, volatile.LoadUint32(&o.GPIO18_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO18_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO18_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO18_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO18_CTRL.Reg, volatile.LoadUint32(&o.GPIO18_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO18_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO18_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO18_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO18_CTRL.Reg, volatile.LoadUint32(&o.GPIO18_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO18_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO18_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO18_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO18_CTRL.Reg, volatile.LoadUint32(&o.GPIO18_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO18_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO18_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO18_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO18_CTRL.Reg, volatile.LoadUint32(&o.GPIO18_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO18_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO18_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO19_STATUS
func (o *IO_BANK0_Type) SetGPIO19_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO19_STATUS.Reg, volatile.LoadUint32(&o.GPIO19_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO19_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO19_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO19_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO19_STATUS.Reg, volatile.LoadUint32(&o.GPIO19_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO19_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO19_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO19_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO19_STATUS.Reg, volatile.LoadUint32(&o.GPIO19_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO19_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO19_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO19_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO19_STATUS.Reg, volatile.LoadUint32(&o.GPIO19_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO19_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO19_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO19_CTRL
func (o *IO_BANK0_Type) SetGPIO19_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO19_CTRL.Reg, volatile.LoadUint32(&o.GPIO19_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO19_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO19_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO19_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO19_CTRL.Reg, volatile.LoadUint32(&o.GPIO19_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO19_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO19_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO19_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO19_CTRL.Reg, volatile.LoadUint32(&o.GPIO19_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO19_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO19_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO19_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO19_CTRL.Reg, volatile.LoadUint32(&o.GPIO19_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO19_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO19_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO19_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO19_CTRL.Reg, volatile.LoadUint32(&o.GPIO19_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO19_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO19_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO20_STATUS
func (o *IO_BANK0_Type) SetGPIO20_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO20_STATUS.Reg, volatile.LoadUint32(&o.GPIO20_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO20_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO20_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO20_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO20_STATUS.Reg, volatile.LoadUint32(&o.GPIO20_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO20_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO20_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO20_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO20_STATUS.Reg, volatile.LoadUint32(&o.GPIO20_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO20_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO20_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO20_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO20_STATUS.Reg, volatile.LoadUint32(&o.GPIO20_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO20_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO20_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO20_CTRL
func (o *IO_BANK0_Type) SetGPIO20_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO20_CTRL.Reg, volatile.LoadUint32(&o.GPIO20_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO20_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO20_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO20_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO20_CTRL.Reg, volatile.LoadUint32(&o.GPIO20_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO20_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO20_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO20_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO20_CTRL.Reg, volatile.LoadUint32(&o.GPIO20_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO20_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO20_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO20_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO20_CTRL.Reg, volatile.LoadUint32(&o.GPIO20_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO20_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO20_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO20_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO20_CTRL.Reg, volatile.LoadUint32(&o.GPIO20_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO20_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO20_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO21_STATUS
func (o *IO_BANK0_Type) SetGPIO21_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO21_STATUS.Reg, volatile.LoadUint32(&o.GPIO21_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO21_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO21_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO21_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO21_STATUS.Reg, volatile.LoadUint32(&o.GPIO21_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO21_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO21_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO21_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO21_STATUS.Reg, volatile.LoadUint32(&o.GPIO21_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO21_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO21_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO21_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO21_STATUS.Reg, volatile.LoadUint32(&o.GPIO21_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO21_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO21_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO21_CTRL
func (o *IO_BANK0_Type) SetGPIO21_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO21_CTRL.Reg, volatile.LoadUint32(&o.GPIO21_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO21_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO21_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO21_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO21_CTRL.Reg, volatile.LoadUint32(&o.GPIO21_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO21_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO21_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO21_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO21_CTRL.Reg, volatile.LoadUint32(&o.GPIO21_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO21_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO21_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO21_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO21_CTRL.Reg, volatile.LoadUint32(&o.GPIO21_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO21_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO21_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO21_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO21_CTRL.Reg, volatile.LoadUint32(&o.GPIO21_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO21_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO21_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO22_STATUS
func (o *IO_BANK0_Type) SetGPIO22_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO22_STATUS.Reg, volatile.LoadUint32(&o.GPIO22_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO22_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO22_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO22_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO22_STATUS.Reg, volatile.LoadUint32(&o.GPIO22_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO22_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO22_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO22_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO22_STATUS.Reg, volatile.LoadUint32(&o.GPIO22_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO22_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO22_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO22_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO22_STATUS.Reg, volatile.LoadUint32(&o.GPIO22_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO22_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO22_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO22_CTRL
func (o *IO_BANK0_Type) SetGPIO22_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO22_CTRL.Reg, volatile.LoadUint32(&o.GPIO22_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO22_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO22_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO22_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO22_CTRL.Reg, volatile.LoadUint32(&o.GPIO22_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO22_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO22_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO22_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO22_CTRL.Reg, volatile.LoadUint32(&o.GPIO22_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO22_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO22_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO22_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO22_CTRL.Reg, volatile.LoadUint32(&o.GPIO22_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO22_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO22_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO22_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO22_CTRL.Reg, volatile.LoadUint32(&o.GPIO22_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO22_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO22_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO23_STATUS
func (o *IO_BANK0_Type) SetGPIO23_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO23_STATUS.Reg, volatile.LoadUint32(&o.GPIO23_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO23_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO23_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO23_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO23_STATUS.Reg, volatile.LoadUint32(&o.GPIO23_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO23_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO23_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO23_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO23_STATUS.Reg, volatile.LoadUint32(&o.GPIO23_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO23_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO23_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO23_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO23_STATUS.Reg, volatile.LoadUint32(&o.GPIO23_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO23_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO23_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO23_CTRL
func (o *IO_BANK0_Type) SetGPIO23_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO23_CTRL.Reg, volatile.LoadUint32(&o.GPIO23_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO23_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO23_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO23_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO23_CTRL.Reg, volatile.LoadUint32(&o.GPIO23_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO23_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO23_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO23_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO23_CTRL.Reg, volatile.LoadUint32(&o.GPIO23_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO23_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO23_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO23_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO23_CTRL.Reg, volatile.LoadUint32(&o.GPIO23_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO23_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO23_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO23_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO23_CTRL.Reg, volatile.LoadUint32(&o.GPIO23_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO23_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO23_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO24_STATUS
func (o *IO_BANK0_Type) SetGPIO24_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO24_STATUS.Reg, volatile.LoadUint32(&o.GPIO24_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO24_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO24_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO24_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO24_STATUS.Reg, volatile.LoadUint32(&o.GPIO24_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO24_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO24_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO24_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO24_STATUS.Reg, volatile.LoadUint32(&o.GPIO24_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO24_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO24_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO24_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO24_STATUS.Reg, volatile.LoadUint32(&o.GPIO24_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO24_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO24_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO24_CTRL
func (o *IO_BANK0_Type) SetGPIO24_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO24_CTRL.Reg, volatile.LoadUint32(&o.GPIO24_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO24_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO24_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO24_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO24_CTRL.Reg, volatile.LoadUint32(&o.GPIO24_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO24_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO24_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO24_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO24_CTRL.Reg, volatile.LoadUint32(&o.GPIO24_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO24_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO24_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO24_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO24_CTRL.Reg, volatile.LoadUint32(&o.GPIO24_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO24_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO24_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO24_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO24_CTRL.Reg, volatile.LoadUint32(&o.GPIO24_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO24_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO24_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO25_STATUS
func (o *IO_BANK0_Type) SetGPIO25_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO25_STATUS.Reg, volatile.LoadUint32(&o.GPIO25_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO25_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO25_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO25_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO25_STATUS.Reg, volatile.LoadUint32(&o.GPIO25_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO25_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO25_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO25_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO25_STATUS.Reg, volatile.LoadUint32(&o.GPIO25_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO25_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO25_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO25_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO25_STATUS.Reg, volatile.LoadUint32(&o.GPIO25_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO25_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO25_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO25_CTRL
func (o *IO_BANK0_Type) SetGPIO25_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO25_CTRL.Reg, volatile.LoadUint32(&o.GPIO25_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO25_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO25_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO25_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO25_CTRL.Reg, volatile.LoadUint32(&o.GPIO25_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO25_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO25_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO25_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO25_CTRL.Reg, volatile.LoadUint32(&o.GPIO25_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO25_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO25_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO25_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO25_CTRL.Reg, volatile.LoadUint32(&o.GPIO25_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO25_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO25_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO25_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO25_CTRL.Reg, volatile.LoadUint32(&o.GPIO25_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO25_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO25_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO26_STATUS
func (o *IO_BANK0_Type) SetGPIO26_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO26_STATUS.Reg, volatile.LoadUint32(&o.GPIO26_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO26_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO26_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO26_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO26_STATUS.Reg, volatile.LoadUint32(&o.GPIO26_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO26_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO26_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO26_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO26_STATUS.Reg, volatile.LoadUint32(&o.GPIO26_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO26_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO26_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO26_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO26_STATUS.Reg, volatile.LoadUint32(&o.GPIO26_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO26_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO26_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO26_CTRL
func (o *IO_BANK0_Type) SetGPIO26_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO26_CTRL.Reg, volatile.LoadUint32(&o.GPIO26_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO26_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO26_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO26_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO26_CTRL.Reg, volatile.LoadUint32(&o.GPIO26_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO26_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO26_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO26_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO26_CTRL.Reg, volatile.LoadUint32(&o.GPIO26_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO26_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO26_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO26_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO26_CTRL.Reg, volatile.LoadUint32(&o.GPIO26_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO26_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO26_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO26_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO26_CTRL.Reg, volatile.LoadUint32(&o.GPIO26_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO26_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO26_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO27_STATUS
func (o *IO_BANK0_Type) SetGPIO27_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO27_STATUS.Reg, volatile.LoadUint32(&o.GPIO27_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO27_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO27_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO27_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO27_STATUS.Reg, volatile.LoadUint32(&o.GPIO27_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO27_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO27_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO27_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO27_STATUS.Reg, volatile.LoadUint32(&o.GPIO27_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO27_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO27_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO27_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO27_STATUS.Reg, volatile.LoadUint32(&o.GPIO27_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO27_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO27_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO27_CTRL
func (o *IO_BANK0_Type) SetGPIO27_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO27_CTRL.Reg, volatile.LoadUint32(&o.GPIO27_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO27_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO27_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO27_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO27_CTRL.Reg, volatile.LoadUint32(&o.GPIO27_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO27_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO27_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO27_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO27_CTRL.Reg, volatile.LoadUint32(&o.GPIO27_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO27_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO27_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO27_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO27_CTRL.Reg, volatile.LoadUint32(&o.GPIO27_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO27_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO27_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO27_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO27_CTRL.Reg, volatile.LoadUint32(&o.GPIO27_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO27_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO27_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO28_STATUS
func (o *IO_BANK0_Type) SetGPIO28_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO28_STATUS.Reg, volatile.LoadUint32(&o.GPIO28_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO28_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO28_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO28_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO28_STATUS.Reg, volatile.LoadUint32(&o.GPIO28_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO28_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO28_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO28_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO28_STATUS.Reg, volatile.LoadUint32(&o.GPIO28_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO28_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO28_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO28_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO28_STATUS.Reg, volatile.LoadUint32(&o.GPIO28_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO28_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO28_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO28_CTRL
func (o *IO_BANK0_Type) SetGPIO28_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO28_CTRL.Reg, volatile.LoadUint32(&o.GPIO28_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO28_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO28_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO28_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO28_CTRL.Reg, volatile.LoadUint32(&o.GPIO28_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO28_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO28_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO28_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO28_CTRL.Reg, volatile.LoadUint32(&o.GPIO28_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO28_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO28_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO28_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO28_CTRL.Reg, volatile.LoadUint32(&o.GPIO28_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO28_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO28_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO28_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO28_CTRL.Reg, volatile.LoadUint32(&o.GPIO28_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO28_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO28_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO29_STATUS
func (o *IO_BANK0_Type) SetGPIO29_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO29_STATUS.Reg, volatile.LoadUint32(&o.GPIO29_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO29_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO29_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO29_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO29_STATUS.Reg, volatile.LoadUint32(&o.GPIO29_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO29_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO29_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO29_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO29_STATUS.Reg, volatile.LoadUint32(&o.GPIO29_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO29_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO29_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO29_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO29_STATUS.Reg, volatile.LoadUint32(&o.GPIO29_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO29_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO29_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO29_CTRL
func (o *IO_BANK0_Type) SetGPIO29_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO29_CTRL.Reg, volatile.LoadUint32(&o.GPIO29_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO29_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO29_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO29_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO29_CTRL.Reg, volatile.LoadUint32(&o.GPIO29_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO29_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO29_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO29_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO29_CTRL.Reg, volatile.LoadUint32(&o.GPIO29_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO29_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO29_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO29_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO29_CTRL.Reg, volatile.LoadUint32(&o.GPIO29_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO29_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO29_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO29_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO29_CTRL.Reg, volatile.LoadUint32(&o.GPIO29_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO29_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO29_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO30_STATUS
func (o *IO_BANK0_Type) SetGPIO30_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO30_STATUS.Reg, volatile.LoadUint32(&o.GPIO30_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO30_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO30_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO30_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO30_STATUS.Reg, volatile.LoadUint32(&o.GPIO30_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO30_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO30_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO30_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO30_STATUS.Reg, volatile.LoadUint32(&o.GPIO30_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO30_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO30_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO30_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO30_STATUS.Reg, volatile.LoadUint32(&o.GPIO30_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO30_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO30_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO30_CTRL
func (o *IO_BANK0_Type) SetGPIO30_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO30_CTRL.Reg, volatile.LoadUint32(&o.GPIO30_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO30_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO30_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO30_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO30_CTRL.Reg, volatile.LoadUint32(&o.GPIO30_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO30_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO30_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO30_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO30_CTRL.Reg, volatile.LoadUint32(&o.GPIO30_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO30_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO30_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO30_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO30_CTRL.Reg, volatile.LoadUint32(&o.GPIO30_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO30_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO30_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO30_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO30_CTRL.Reg, volatile.LoadUint32(&o.GPIO30_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO30_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO30_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO31_STATUS
func (o *IO_BANK0_Type) SetGPIO31_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO31_STATUS.Reg, volatile.LoadUint32(&o.GPIO31_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO31_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO31_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO31_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO31_STATUS.Reg, volatile.LoadUint32(&o.GPIO31_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO31_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO31_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO31_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO31_STATUS.Reg, volatile.LoadUint32(&o.GPIO31_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO31_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO31_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO31_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO31_STATUS.Reg, volatile.LoadUint32(&o.GPIO31_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO31_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO31_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO31_CTRL
func (o *IO_BANK0_Type) SetGPIO31_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO31_CTRL.Reg, volatile.LoadUint32(&o.GPIO31_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO31_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO31_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO31_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO31_CTRL.Reg, volatile.LoadUint32(&o.GPIO31_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO31_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO31_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO31_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO31_CTRL.Reg, volatile.LoadUint32(&o.GPIO31_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO31_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO31_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO31_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO31_CTRL.Reg, volatile.LoadUint32(&o.GPIO31_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO31_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO31_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO31_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO31_CTRL.Reg, volatile.LoadUint32(&o.GPIO31_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO31_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO31_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO32_STATUS
func (o *IO_BANK0_Type) SetGPIO32_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO32_STATUS.Reg, volatile.LoadUint32(&o.GPIO32_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO32_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO32_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO32_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO32_STATUS.Reg, volatile.LoadUint32(&o.GPIO32_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO32_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO32_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO32_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO32_STATUS.Reg, volatile.LoadUint32(&o.GPIO32_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO32_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO32_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO32_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO32_STATUS.Reg, volatile.LoadUint32(&o.GPIO32_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO32_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO32_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO32_CTRL
func (o *IO_BANK0_Type) SetGPIO32_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO32_CTRL.Reg, volatile.LoadUint32(&o.GPIO32_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO32_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO32_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO32_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO32_CTRL.Reg, volatile.LoadUint32(&o.GPIO32_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO32_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO32_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO32_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO32_CTRL.Reg, volatile.LoadUint32(&o.GPIO32_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO32_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO32_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO32_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO32_CTRL.Reg, volatile.LoadUint32(&o.GPIO32_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO32_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO32_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO32_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO32_CTRL.Reg, volatile.LoadUint32(&o.GPIO32_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO32_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO32_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO33_STATUS
func (o *IO_BANK0_Type) SetGPIO33_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO33_STATUS.Reg, volatile.LoadUint32(&o.GPIO33_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO33_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO33_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO33_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO33_STATUS.Reg, volatile.LoadUint32(&o.GPIO33_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO33_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO33_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO33_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO33_STATUS.Reg, volatile.LoadUint32(&o.GPIO33_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO33_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO33_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO33_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO33_STATUS.Reg, volatile.LoadUint32(&o.GPIO33_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO33_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO33_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO33_CTRL
func (o *IO_BANK0_Type) SetGPIO33_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO33_CTRL.Reg, volatile.LoadUint32(&o.GPIO33_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO33_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO33_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO33_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO33_CTRL.Reg, volatile.LoadUint32(&o.GPIO33_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO33_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO33_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO33_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO33_CTRL.Reg, volatile.LoadUint32(&o.GPIO33_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO33_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO33_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO33_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO33_CTRL.Reg, volatile.LoadUint32(&o.GPIO33_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO33_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO33_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO33_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO33_CTRL.Reg, volatile.LoadUint32(&o.GPIO33_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO33_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO33_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO34_STATUS
func (o *IO_BANK0_Type) SetGPIO34_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO34_STATUS.Reg, volatile.LoadUint32(&o.GPIO34_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO34_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO34_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO34_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO34_STATUS.Reg, volatile.LoadUint32(&o.GPIO34_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO34_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO34_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO34_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO34_STATUS.Reg, volatile.LoadUint32(&o.GPIO34_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO34_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO34_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO34_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO34_STATUS.Reg, volatile.LoadUint32(&o.GPIO34_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO34_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO34_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO34_CTRL
func (o *IO_BANK0_Type) SetGPIO34_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO34_CTRL.Reg, volatile.LoadUint32(&o.GPIO34_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO34_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO34_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO34_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO34_CTRL.Reg, volatile.LoadUint32(&o.GPIO34_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO34_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO34_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO34_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO34_CTRL.Reg, volatile.LoadUint32(&o.GPIO34_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO34_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO34_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO34_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO34_CTRL.Reg, volatile.LoadUint32(&o.GPIO34_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO34_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO34_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO34_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO34_CTRL.Reg, volatile.LoadUint32(&o.GPIO34_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO34_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO34_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO35_STATUS
func (o *IO_BANK0_Type) SetGPIO35_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO35_STATUS.Reg, volatile.LoadUint32(&o.GPIO35_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO35_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO35_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO35_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO35_STATUS.Reg, volatile.LoadUint32(&o.GPIO35_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO35_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO35_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO35_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO35_STATUS.Reg, volatile.LoadUint32(&o.GPIO35_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO35_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO35_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO35_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO35_STATUS.Reg, volatile.LoadUint32(&o.GPIO35_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO35_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO35_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO35_CTRL
func (o *IO_BANK0_Type) SetGPIO35_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO35_CTRL.Reg, volatile.LoadUint32(&o.GPIO35_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO35_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO35_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO35_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO35_CTRL.Reg, volatile.LoadUint32(&o.GPIO35_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO35_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO35_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO35_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO35_CTRL.Reg, volatile.LoadUint32(&o.GPIO35_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO35_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO35_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO35_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO35_CTRL.Reg, volatile.LoadUint32(&o.GPIO35_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO35_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO35_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO35_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO35_CTRL.Reg, volatile.LoadUint32(&o.GPIO35_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO35_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO35_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO36_STATUS
func (o *IO_BANK0_Type) SetGPIO36_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO36_STATUS.Reg, volatile.LoadUint32(&o.GPIO36_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO36_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO36_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO36_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO36_STATUS.Reg, volatile.LoadUint32(&o.GPIO36_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO36_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO36_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO36_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO36_STATUS.Reg, volatile.LoadUint32(&o.GPIO36_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO36_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO36_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO36_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO36_STATUS.Reg, volatile.LoadUint32(&o.GPIO36_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO36_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO36_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO36_CTRL
func (o *IO_BANK0_Type) SetGPIO36_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO36_CTRL.Reg, volatile.LoadUint32(&o.GPIO36_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO36_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO36_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO36_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO36_CTRL.Reg, volatile.LoadUint32(&o.GPIO36_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO36_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO36_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO36_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO36_CTRL.Reg, volatile.LoadUint32(&o.GPIO36_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO36_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO36_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO36_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO36_CTRL.Reg, volatile.LoadUint32(&o.GPIO36_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO36_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO36_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO36_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO36_CTRL.Reg, volatile.LoadUint32(&o.GPIO36_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO36_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO36_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO37_STATUS
func (o *IO_BANK0_Type) SetGPIO37_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO37_STATUS.Reg, volatile.LoadUint32(&o.GPIO37_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO37_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO37_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO37_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO37_STATUS.Reg, volatile.LoadUint32(&o.GPIO37_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO37_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO37_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO37_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO37_STATUS.Reg, volatile.LoadUint32(&o.GPIO37_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO37_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO37_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO37_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO37_STATUS.Reg, volatile.LoadUint32(&o.GPIO37_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO37_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO37_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO37_CTRL
func (o *IO_BANK0_Type) SetGPIO37_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO37_CTRL.Reg, volatile.LoadUint32(&o.GPIO37_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO37_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO37_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO37_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO37_CTRL.Reg, volatile.LoadUint32(&o.GPIO37_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO37_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO37_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO37_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO37_CTRL.Reg, volatile.LoadUint32(&o.GPIO37_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO37_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO37_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO37_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO37_CTRL.Reg, volatile.LoadUint32(&o.GPIO37_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO37_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO37_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO37_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO37_CTRL.Reg, volatile.LoadUint32(&o.GPIO37_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO37_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO37_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO38_STATUS
func (o *IO_BANK0_Type) SetGPIO38_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO38_STATUS.Reg, volatile.LoadUint32(&o.GPIO38_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO38_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO38_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO38_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO38_STATUS.Reg, volatile.LoadUint32(&o.GPIO38_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO38_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO38_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO38_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO38_STATUS.Reg, volatile.LoadUint32(&o.GPIO38_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO38_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO38_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO38_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO38_STATUS.Reg, volatile.LoadUint32(&o.GPIO38_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO38_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO38_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO38_CTRL
func (o *IO_BANK0_Type) SetGPIO38_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO38_CTRL.Reg, volatile.LoadUint32(&o.GPIO38_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO38_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO38_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO38_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO38_CTRL.Reg, volatile.LoadUint32(&o.GPIO38_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO38_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO38_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO38_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO38_CTRL.Reg, volatile.LoadUint32(&o.GPIO38_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO38_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO38_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO38_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO38_CTRL.Reg, volatile.LoadUint32(&o.GPIO38_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO38_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO38_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO38_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO38_CTRL.Reg, volatile.LoadUint32(&o.GPIO38_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO38_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO38_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO39_STATUS
func (o *IO_BANK0_Type) SetGPIO39_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO39_STATUS.Reg, volatile.LoadUint32(&o.GPIO39_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO39_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO39_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO39_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO39_STATUS.Reg, volatile.LoadUint32(&o.GPIO39_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO39_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO39_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO39_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO39_STATUS.Reg, volatile.LoadUint32(&o.GPIO39_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO39_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO39_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO39_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO39_STATUS.Reg, volatile.LoadUint32(&o.GPIO39_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO39_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO39_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO39_CTRL
func (o *IO_BANK0_Type) SetGPIO39_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO39_CTRL.Reg, volatile.LoadUint32(&o.GPIO39_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO39_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO39_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO39_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO39_CTRL.Reg, volatile.LoadUint32(&o.GPIO39_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO39_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO39_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO39_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO39_CTRL.Reg, volatile.LoadUint32(&o.GPIO39_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO39_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO39_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO39_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO39_CTRL.Reg, volatile.LoadUint32(&o.GPIO39_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO39_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO39_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO39_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO39_CTRL.Reg, volatile.LoadUint32(&o.GPIO39_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO39_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO39_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO40_STATUS
func (o *IO_BANK0_Type) SetGPIO40_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO40_STATUS.Reg, volatile.LoadUint32(&o.GPIO40_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO40_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO40_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO40_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO40_STATUS.Reg, volatile.LoadUint32(&o.GPIO40_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO40_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO40_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO40_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO40_STATUS.Reg, volatile.LoadUint32(&o.GPIO40_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO40_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO40_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO40_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO40_STATUS.Reg, volatile.LoadUint32(&o.GPIO40_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO40_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO40_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO40_CTRL
func (o *IO_BANK0_Type) SetGPIO40_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO40_CTRL.Reg, volatile.LoadUint32(&o.GPIO40_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO40_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO40_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO40_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO40_CTRL.Reg, volatile.LoadUint32(&o.GPIO40_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO40_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO40_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO40_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO40_CTRL.Reg, volatile.LoadUint32(&o.GPIO40_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO40_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO40_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO40_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO40_CTRL.Reg, volatile.LoadUint32(&o.GPIO40_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO40_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO40_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO40_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO40_CTRL.Reg, volatile.LoadUint32(&o.GPIO40_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO40_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO40_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO41_STATUS
func (o *IO_BANK0_Type) SetGPIO41_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO41_STATUS.Reg, volatile.LoadUint32(&o.GPIO41_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO41_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO41_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO41_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO41_STATUS.Reg, volatile.LoadUint32(&o.GPIO41_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO41_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO41_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO41_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO41_STATUS.Reg, volatile.LoadUint32(&o.GPIO41_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO41_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO41_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO41_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO41_STATUS.Reg, volatile.LoadUint32(&o.GPIO41_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO41_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO41_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO41_CTRL
func (o *IO_BANK0_Type) SetGPIO41_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO41_CTRL.Reg, volatile.LoadUint32(&o.GPIO41_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO41_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO41_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO41_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO41_CTRL.Reg, volatile.LoadUint32(&o.GPIO41_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO41_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO41_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO41_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO41_CTRL.Reg, volatile.LoadUint32(&o.GPIO41_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO41_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO41_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO41_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO41_CTRL.Reg, volatile.LoadUint32(&o.GPIO41_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO41_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO41_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO41_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO41_CTRL.Reg, volatile.LoadUint32(&o.GPIO41_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO41_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO41_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO42_STATUS
func (o *IO_BANK0_Type) SetGPIO42_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO42_STATUS.Reg, volatile.LoadUint32(&o.GPIO42_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO42_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO42_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO42_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO42_STATUS.Reg, volatile.LoadUint32(&o.GPIO42_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO42_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO42_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO42_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO42_STATUS.Reg, volatile.LoadUint32(&o.GPIO42_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO42_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO42_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO42_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO42_STATUS.Reg, volatile.LoadUint32(&o.GPIO42_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO42_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO42_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO42_CTRL
func (o *IO_BANK0_Type) SetGPIO42_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO42_CTRL.Reg, volatile.LoadUint32(&o.GPIO42_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO42_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO42_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO42_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO42_CTRL.Reg, volatile.LoadUint32(&o.GPIO42_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO42_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO42_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO42_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO42_CTRL.Reg, volatile.LoadUint32(&o.GPIO42_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO42_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO42_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO42_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO42_CTRL.Reg, volatile.LoadUint32(&o.GPIO42_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO42_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO42_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO42_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO42_CTRL.Reg, volatile.LoadUint32(&o.GPIO42_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO42_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO42_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO43_STATUS
func (o *IO_BANK0_Type) SetGPIO43_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO43_STATUS.Reg, volatile.LoadUint32(&o.GPIO43_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO43_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO43_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO43_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO43_STATUS.Reg, volatile.LoadUint32(&o.GPIO43_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO43_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO43_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO43_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO43_STATUS.Reg, volatile.LoadUint32(&o.GPIO43_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO43_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO43_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO43_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO43_STATUS.Reg, volatile.LoadUint32(&o.GPIO43_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO43_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO43_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO43_CTRL
func (o *IO_BANK0_Type) SetGPIO43_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO43_CTRL.Reg, volatile.LoadUint32(&o.GPIO43_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO43_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO43_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO43_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO43_CTRL.Reg, volatile.LoadUint32(&o.GPIO43_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO43_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO43_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO43_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO43_CTRL.Reg, volatile.LoadUint32(&o.GPIO43_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO43_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO43_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO43_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO43_CTRL.Reg, volatile.LoadUint32(&o.GPIO43_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO43_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO43_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO43_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO43_CTRL.Reg, volatile.LoadUint32(&o.GPIO43_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO43_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO43_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO44_STATUS
func (o *IO_BANK0_Type) SetGPIO44_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO44_STATUS.Reg, volatile.LoadUint32(&o.GPIO44_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO44_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO44_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO44_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO44_STATUS.Reg, volatile.LoadUint32(&o.GPIO44_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO44_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO44_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO44_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO44_STATUS.Reg, volatile.LoadUint32(&o.GPIO44_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO44_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO44_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO44_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO44_STATUS.Reg, volatile.LoadUint32(&o.GPIO44_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO44_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO44_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO44_CTRL
func (o *IO_BANK0_Type) SetGPIO44_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO44_CTRL.Reg, volatile.LoadUint32(&o.GPIO44_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO44_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO44_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO44_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO44_CTRL.Reg, volatile.LoadUint32(&o.GPIO44_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO44_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO44_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO44_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO44_CTRL.Reg, volatile.LoadUint32(&o.GPIO44_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO44_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO44_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO44_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO44_CTRL.Reg, volatile.LoadUint32(&o.GPIO44_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO44_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO44_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO44_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO44_CTRL.Reg, volatile.LoadUint32(&o.GPIO44_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO44_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO44_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO45_STATUS
func (o *IO_BANK0_Type) SetGPIO45_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO45_STATUS.Reg, volatile.LoadUint32(&o.GPIO45_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO45_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO45_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO45_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO45_STATUS.Reg, volatile.LoadUint32(&o.GPIO45_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO45_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO45_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO45_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO45_STATUS.Reg, volatile.LoadUint32(&o.GPIO45_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO45_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO45_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO45_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO45_STATUS.Reg, volatile.LoadUint32(&o.GPIO45_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO45_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO45_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO45_CTRL
func (o *IO_BANK0_Type) SetGPIO45_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO45_CTRL.Reg, volatile.LoadUint32(&o.GPIO45_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO45_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO45_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO45_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO45_CTRL.Reg, volatile.LoadUint32(&o.GPIO45_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO45_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO45_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO45_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO45_CTRL.Reg, volatile.LoadUint32(&o.GPIO45_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO45_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO45_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO45_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO45_CTRL.Reg, volatile.LoadUint32(&o.GPIO45_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO45_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO45_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO45_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO45_CTRL.Reg, volatile.LoadUint32(&o.GPIO45_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO45_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO45_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO46_STATUS
func (o *IO_BANK0_Type) SetGPIO46_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO46_STATUS.Reg, volatile.LoadUint32(&o.GPIO46_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO46_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO46_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO46_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO46_STATUS.Reg, volatile.LoadUint32(&o.GPIO46_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO46_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO46_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO46_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO46_STATUS.Reg, volatile.LoadUint32(&o.GPIO46_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO46_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO46_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO46_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO46_STATUS.Reg, volatile.LoadUint32(&o.GPIO46_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO46_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO46_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO46_CTRL
func (o *IO_BANK0_Type) SetGPIO46_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO46_CTRL.Reg, volatile.LoadUint32(&o.GPIO46_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO46_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO46_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO46_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO46_CTRL.Reg, volatile.LoadUint32(&o.GPIO46_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO46_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO46_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO46_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO46_CTRL.Reg, volatile.LoadUint32(&o.GPIO46_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO46_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO46_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO46_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO46_CTRL.Reg, volatile.LoadUint32(&o.GPIO46_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO46_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO46_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO46_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO46_CTRL.Reg, volatile.LoadUint32(&o.GPIO46_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO46_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO46_CTRL.Reg) & 0x1f
}

// IO_BANK0.GPIO47_STATUS
func (o *IO_BANK0_Type) SetGPIO47_STATUS_IRQTOPROC(value uint32) {
	volatile.StoreUint32(&o.GPIO47_STATUS.Reg, volatile.LoadUint32(&o.GPIO47_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetGPIO47_STATUS_IRQTOPROC() uint32 {
	return (volatile.LoadUint32(&o.GPIO47_STATUS.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetGPIO47_STATUS_INFROMPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO47_STATUS.Reg, volatile.LoadUint32(&o.GPIO47_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetGPIO47_STATUS_INFROMPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO47_STATUS.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetGPIO47_STATUS_OETOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO47_STATUS.Reg, volatile.LoadUint32(&o.GPIO47_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetGPIO47_STATUS_OETOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO47_STATUS.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetGPIO47_STATUS_OUTTOPAD(value uint32) {
	volatile.StoreUint32(&o.GPIO47_STATUS.Reg, volatile.LoadUint32(&o.GPIO47_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetGPIO47_STATUS_OUTTOPAD() uint32 {
	return (volatile.LoadUint32(&o.GPIO47_STATUS.Reg) & 0x200) >> 9
}

// IO_BANK0.GPIO47_CTRL
func (o *IO_BANK0_Type) SetGPIO47_CTRL_IRQOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO47_CTRL.Reg, volatile.LoadUint32(&o.GPIO47_CTRL.Reg)&^(0x30000000)|value<<28)
}
func (o *IO_BANK0_Type) GetGPIO47_CTRL_IRQOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO47_CTRL.Reg) & 0x30000000) >> 28
}
func (o *IO_BANK0_Type) SetGPIO47_CTRL_INOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO47_CTRL.Reg, volatile.LoadUint32(&o.GPIO47_CTRL.Reg)&^(0x30000)|value<<16)
}
func (o *IO_BANK0_Type) GetGPIO47_CTRL_INOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO47_CTRL.Reg) & 0x30000) >> 16
}
func (o *IO_BANK0_Type) SetGPIO47_CTRL_OEOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO47_CTRL.Reg, volatile.LoadUint32(&o.GPIO47_CTRL.Reg)&^(0xc000)|value<<14)
}
func (o *IO_BANK0_Type) GetGPIO47_CTRL_OEOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO47_CTRL.Reg) & 0xc000) >> 14
}
func (o *IO_BANK0_Type) SetGPIO47_CTRL_OUTOVER(value uint32) {
	volatile.StoreUint32(&o.GPIO47_CTRL.Reg, volatile.LoadUint32(&o.GPIO47_CTRL.Reg)&^(0x3000)|value<<12)
}
func (o *IO_BANK0_Type) GetGPIO47_CTRL_OUTOVER() uint32 {
	return (volatile.LoadUint32(&o.GPIO47_CTRL.Reg) & 0x3000) >> 12
}
func (o *IO_BANK0_Type) SetGPIO47_CTRL_FUNCSEL(value uint32) {
	volatile.StoreUint32(&o.GPIO47_CTRL.Reg, volatile.LoadUint32(&o.GPIO47_CTRL.Reg)&^(0x1f)|value)
}
func (o *IO_BANK0_Type) GetGPIO47_CTRL_FUNCSEL() uint32 {
	return volatile.LoadUint32(&o.GPIO47_CTRL.Reg) & 0x1f
}

// IO_BANK0.IRQSUMMARY_PROC0_SECURE0
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO31(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO31() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO30(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO30() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO29(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO29() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO28(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO28() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO27(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO27() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO26(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO26() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO25(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO25() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO24(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO24() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO23(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO23() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO22(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO22() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO21(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO21() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO20(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO20() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO19(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO19() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO18(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO18() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO17(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO17() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO16(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO16() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO15(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO15() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO14(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO14() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO13(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO13() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO12(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO12() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO11(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO11() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO10(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO10() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO9(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO9() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO8(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO8() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO7(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO7() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO6(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO6() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO5(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO5() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO4(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO4() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE0_GPIO0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE0_GPIO0() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE0.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_PROC0_SECURE1
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO47(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO47() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO46(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO46() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO45(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO45() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO44(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO44() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO43(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO43() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO42(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO42() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO41(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO41() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO40(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO40() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO39(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO39() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO38(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO38() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO37(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO37() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO36(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO36() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO35(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO35() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO34(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO34() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO33(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO33() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_SECURE1_GPIO32(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_SECURE1_GPIO32() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC0_SECURE1.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_PROC0_NONSECURE0
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO31(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO31() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO30(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO30() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO29(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO29() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO28(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO28() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO27(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO27() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO26(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO26() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO25(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO25() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO24(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO24() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO23(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO23() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO22(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO22() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO21(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO21() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO20(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO20() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO19(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO19() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO18(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO18() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO17(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO17() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO16(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO16() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO15(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO15() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO14(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO14() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO13(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO13() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO12(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO12() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO11(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO11() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO10(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO10() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO9(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO9() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO8(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO8() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO7(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO7() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO6(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO6() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO5(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO5() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO4(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO4() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE0_GPIO0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE0_GPIO0() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE0.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_PROC0_NONSECURE1
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO47(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO47() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO46(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO46() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO45(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO45() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO44(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO44() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO43(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO43() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO42(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO42() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO41(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO41() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO40(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO40() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO39(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO39() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO38(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO38() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO37(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO37() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO36(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO36() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO35(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO35() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO34(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO34() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO33(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO33() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC0_NONSECURE1_GPIO32(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC0_NONSECURE1_GPIO32() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC0_NONSECURE1.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_PROC1_SECURE0
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO31(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO31() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO30(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO30() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO29(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO29() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO28(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO28() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO27(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO27() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO26(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO26() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO25(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO25() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO24(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO24() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO23(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO23() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO22(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO22() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO21(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO21() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO20(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO20() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO19(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO19() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO18(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO18() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO17(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO17() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO16(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO16() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO15(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO15() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO14(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO14() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO13(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO13() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO12(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO12() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO11(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO11() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO10(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO10() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO9(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO9() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO8(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO8() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO7(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO7() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO6(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO6() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO5(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO5() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO4(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO4() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE0_GPIO0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE0_GPIO0() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE0.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_PROC1_SECURE1
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO47(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO47() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO46(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO46() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO45(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO45() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO44(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO44() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO43(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO43() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO42(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO42() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO41(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO41() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO40(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO40() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO39(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO39() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO38(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO38() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO37(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO37() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO36(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO36() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO35(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO35() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO34(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO34() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO33(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO33() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_SECURE1_GPIO32(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_SECURE1_GPIO32() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC1_SECURE1.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_PROC1_NONSECURE0
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO31(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO31() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO30(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO30() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO29(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO29() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO28(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO28() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO27(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO27() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO26(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO26() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO25(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO25() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO24(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO24() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO23(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO23() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO22(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO22() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO21(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO21() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO20(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO20() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO19(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO19() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO18(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO18() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO17(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO17() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO16(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO16() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO15(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO15() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO14(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO14() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO13(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO13() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO12(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO12() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO11(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO11() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO10(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO10() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO9(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO9() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO8(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO8() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO7(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO7() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO6(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO6() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO5(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO5() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO4(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO4() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE0_GPIO0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE0_GPIO0() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE0.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_PROC1_NONSECURE1
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO47(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO47() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO46(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO46() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO45(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO45() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO44(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO44() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO43(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO43() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO42(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO42() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO41(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO41() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO40(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO40() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO39(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO39() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO38(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO38() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO37(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO37() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO36(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO36() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO35(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO35() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO34(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO34() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO33(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO33() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_PROC1_NONSECURE1_GPIO32(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_PROC1_NONSECURE1_GPIO32() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_PROC1_NONSECURE1.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_DORMANT_WAKE_SECURE0
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO31(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO31() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO30(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO30() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO29(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO29() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO28(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO28() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO27(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO27() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO26(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO26() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO25(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO25() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO24(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO24() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO23(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO23() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO22(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO22() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO21(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO21() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO20(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO20() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO19(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO19() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO18(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO18() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO17(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO17() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO16(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO16() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO15(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO15() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO14(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO14() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO13(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO13() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO12(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO12() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO11(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO11() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO10(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO10() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO9(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO9() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO8(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO8() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO7(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO7() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO6(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO6() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO5(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO5() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO4(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO4() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO0() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE0.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_DORMANT_WAKE_SECURE1
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO47(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO47() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO46(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO46() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO45(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO45() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO44(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO44() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO43(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO43() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO42(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO42() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO41(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO41() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO40(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO40() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO39(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO39() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO38(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO38() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO37(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO37() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO36(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO36() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO35(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO35() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO34(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO34() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO33(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO33() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO32(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO32() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_SECURE1.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_DORMANT_WAKE_NONSECURE0
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO31(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO31() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO30(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO30() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO29(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO29() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO28(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO28() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO27(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO27() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO26(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO26() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO25(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO25() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO24(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO24() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO23(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO23() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO22(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO22() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO21(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO21() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO20(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO20() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO19(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO19() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO18(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO18() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO17(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO17() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO16(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO16() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO15(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO15() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO14(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO14() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO13(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO13() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO12(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO12() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO11(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO11() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO10(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO10() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO9(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO9() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO8(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO8() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO7(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO7() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO6(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO6() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO5(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO5() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO4(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO4() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO3(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO3() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO2(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO2() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO1(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO1() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO0(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO0() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE0.Reg) & 0x1
}

// IO_BANK0.IRQSUMMARY_DORMANT_WAKE_NONSECURE1
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO47(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO47() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO46(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO46() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO45(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO45() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO44(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO44() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO43(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO43() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO42(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO42() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO41(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO41() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO40(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO40() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO39(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO39() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO38(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO38() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO37(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO37() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO36(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO36() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO35(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO35() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO34(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO34() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO33(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO33() uint32 {
	return (volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO32(value uint32) {
	volatile.StoreUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg, volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetIRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO32() uint32 {
	return volatile.LoadUint32(&o.IRQSUMMARY_DORMANT_WAKE_NONSECURE1.Reg) & 0x1
}

// IO_BANK0.INTR0: Raw Interrupts
func (o *IO_BANK0_Type) SetINTR0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetINTR0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetINTR0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetINTR0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetINTR0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetINTR0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetINTR0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetINTR0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetINTR0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetINTR0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetINTR0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetINTR0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetINTR0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetINTR0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetINTR0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetINTR0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetINTR0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetINTR0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetINTR0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetINTR0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetINTR0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetINTR0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetINTR0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetINTR0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetINTR0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetINTR0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetINTR0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetINTR0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetINTR0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetINTR0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetINTR0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetINTR0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR0.Reg, volatile.LoadUint32(&o.INTR0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetINTR0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.INTR0.Reg) & 0x1
}

// IO_BANK0.INTR1: Raw Interrupts
func (o *IO_BANK0_Type) SetINTR1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetINTR1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetINTR1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetINTR1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetINTR1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetINTR1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetINTR1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetINTR1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetINTR1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetINTR1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetINTR1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetINTR1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetINTR1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetINTR1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetINTR1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetINTR1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetINTR1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetINTR1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetINTR1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetINTR1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetINTR1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetINTR1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetINTR1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetINTR1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetINTR1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetINTR1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetINTR1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetINTR1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetINTR1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetINTR1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetINTR1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetINTR1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetINTR1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.INTR1.Reg) & 0x1
}

// IO_BANK0.INTR2: Raw Interrupts
func (o *IO_BANK0_Type) SetINTR2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetINTR2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetINTR2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetINTR2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetINTR2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetINTR2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetINTR2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetINTR2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetINTR2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetINTR2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetINTR2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetINTR2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetINTR2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetINTR2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetINTR2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetINTR2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetINTR2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetINTR2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetINTR2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetINTR2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetINTR2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetINTR2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetINTR2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetINTR2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetINTR2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetINTR2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetINTR2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetINTR2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetINTR2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetINTR2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetINTR2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetINTR2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetINTR2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.INTR2.Reg) & 0x1
}

// IO_BANK0.INTR3: Raw Interrupts
func (o *IO_BANK0_Type) SetINTR3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetINTR3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetINTR3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetINTR3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetINTR3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetINTR3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetINTR3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetINTR3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetINTR3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetINTR3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetINTR3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetINTR3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetINTR3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetINTR3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetINTR3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetINTR3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetINTR3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetINTR3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetINTR3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetINTR3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetINTR3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetINTR3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetINTR3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetINTR3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetINTR3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetINTR3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetINTR3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetINTR3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetINTR3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetINTR3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetINTR3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetINTR3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetINTR3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.INTR3.Reg) & 0x1
}

// IO_BANK0.INTR4: Raw Interrupts
func (o *IO_BANK0_Type) SetINTR4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetINTR4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetINTR4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetINTR4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetINTR4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetINTR4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetINTR4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetINTR4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetINTR4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetINTR4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetINTR4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetINTR4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetINTR4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetINTR4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetINTR4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetINTR4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetINTR4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetINTR4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetINTR4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetINTR4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetINTR4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetINTR4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetINTR4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetINTR4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetINTR4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetINTR4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetINTR4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetINTR4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetINTR4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetINTR4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetINTR4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetINTR4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR4.Reg, volatile.LoadUint32(&o.INTR4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetINTR4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.INTR4.Reg) & 0x1
}

// IO_BANK0.INTR5: Raw Interrupts
func (o *IO_BANK0_Type) SetINTR5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetINTR5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetINTR5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetINTR5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetINTR5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetINTR5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetINTR5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetINTR5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetINTR5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetINTR5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetINTR5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetINTR5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetINTR5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetINTR5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetINTR5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetINTR5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetINTR5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetINTR5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetINTR5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetINTR5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetINTR5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetINTR5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetINTR5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetINTR5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetINTR5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetINTR5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetINTR5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetINTR5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetINTR5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetINTR5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetINTR5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.INTR5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetINTR5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR5.Reg, volatile.LoadUint32(&o.INTR5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetINTR5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.INTR5.Reg) & 0x1
}

// IO_BANK0.PROC0_INTE0: Interrupt Enable for proc0
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTE0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE0.Reg, volatile.LoadUint32(&o.PROC0_INTE0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTE0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTE0.Reg) & 0x1
}

// IO_BANK0.PROC0_INTE1: Interrupt Enable for proc0
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTE1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE1.Reg, volatile.LoadUint32(&o.PROC0_INTE1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTE1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTE1.Reg) & 0x1
}

// IO_BANK0.PROC0_INTE2: Interrupt Enable for proc0
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTE2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE2.Reg, volatile.LoadUint32(&o.PROC0_INTE2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTE2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTE2.Reg) & 0x1
}

// IO_BANK0.PROC0_INTE3: Interrupt Enable for proc0
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTE3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE3.Reg, volatile.LoadUint32(&o.PROC0_INTE3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTE3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTE3.Reg) & 0x1
}

// IO_BANK0.PROC0_INTE4: Interrupt Enable for proc0
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTE4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE4.Reg, volatile.LoadUint32(&o.PROC0_INTE4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTE4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTE4.Reg) & 0x1
}

// IO_BANK0.PROC0_INTE5: Interrupt Enable for proc0
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTE5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTE5.Reg, volatile.LoadUint32(&o.PROC0_INTE5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTE5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTE5.Reg) & 0x1
}

// IO_BANK0.PROC0_INTF0: Interrupt Force for proc0
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTF0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF0.Reg, volatile.LoadUint32(&o.PROC0_INTF0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTF0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTF0.Reg) & 0x1
}

// IO_BANK0.PROC0_INTF1: Interrupt Force for proc0
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTF1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF1.Reg, volatile.LoadUint32(&o.PROC0_INTF1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTF1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTF1.Reg) & 0x1
}

// IO_BANK0.PROC0_INTF2: Interrupt Force for proc0
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTF2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF2.Reg, volatile.LoadUint32(&o.PROC0_INTF2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTF2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTF2.Reg) & 0x1
}

// IO_BANK0.PROC0_INTF3: Interrupt Force for proc0
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTF3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF3.Reg, volatile.LoadUint32(&o.PROC0_INTF3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTF3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTF3.Reg) & 0x1
}

// IO_BANK0.PROC0_INTF4: Interrupt Force for proc0
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTF4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF4.Reg, volatile.LoadUint32(&o.PROC0_INTF4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTF4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTF4.Reg) & 0x1
}

// IO_BANK0.PROC0_INTF5: Interrupt Force for proc0
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTF5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTF5.Reg, volatile.LoadUint32(&o.PROC0_INTF5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTF5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTF5.Reg) & 0x1
}

// IO_BANK0.PROC0_INTS0: Interrupt status after masking & forcing for proc0
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTS0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS0.Reg, volatile.LoadUint32(&o.PROC0_INTS0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTS0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTS0.Reg) & 0x1
}

// IO_BANK0.PROC0_INTS1: Interrupt status after masking & forcing for proc0
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTS1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS1.Reg, volatile.LoadUint32(&o.PROC0_INTS1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTS1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTS1.Reg) & 0x1
}

// IO_BANK0.PROC0_INTS2: Interrupt status after masking & forcing for proc0
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTS2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS2.Reg, volatile.LoadUint32(&o.PROC0_INTS2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTS2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTS2.Reg) & 0x1
}

// IO_BANK0.PROC0_INTS3: Interrupt status after masking & forcing for proc0
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTS3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS3.Reg, volatile.LoadUint32(&o.PROC0_INTS3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTS3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTS3.Reg) & 0x1
}

// IO_BANK0.PROC0_INTS4: Interrupt status after masking & forcing for proc0
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTS4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS4.Reg, volatile.LoadUint32(&o.PROC0_INTS4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTS4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTS4.Reg) & 0x1
}

// IO_BANK0.PROC0_INTS5: Interrupt status after masking & forcing for proc0
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC0_INTS5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC0_INTS5.Reg, volatile.LoadUint32(&o.PROC0_INTS5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC0_INTS5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC0_INTS5.Reg) & 0x1
}

// IO_BANK0.PROC1_INTE0: Interrupt Enable for proc1
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTE0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE0.Reg, volatile.LoadUint32(&o.PROC1_INTE0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTE0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTE0.Reg) & 0x1
}

// IO_BANK0.PROC1_INTE1: Interrupt Enable for proc1
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTE1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE1.Reg, volatile.LoadUint32(&o.PROC1_INTE1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTE1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTE1.Reg) & 0x1
}

// IO_BANK0.PROC1_INTE2: Interrupt Enable for proc1
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTE2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE2.Reg, volatile.LoadUint32(&o.PROC1_INTE2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTE2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTE2.Reg) & 0x1
}

// IO_BANK0.PROC1_INTE3: Interrupt Enable for proc1
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTE3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE3.Reg, volatile.LoadUint32(&o.PROC1_INTE3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTE3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTE3.Reg) & 0x1
}

// IO_BANK0.PROC1_INTE4: Interrupt Enable for proc1
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTE4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE4.Reg, volatile.LoadUint32(&o.PROC1_INTE4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTE4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTE4.Reg) & 0x1
}

// IO_BANK0.PROC1_INTE5: Interrupt Enable for proc1
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTE5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTE5.Reg, volatile.LoadUint32(&o.PROC1_INTE5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTE5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTE5.Reg) & 0x1
}

// IO_BANK0.PROC1_INTF0: Interrupt Force for proc1
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTF0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF0.Reg, volatile.LoadUint32(&o.PROC1_INTF0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTF0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTF0.Reg) & 0x1
}

// IO_BANK0.PROC1_INTF1: Interrupt Force for proc1
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTF1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF1.Reg, volatile.LoadUint32(&o.PROC1_INTF1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTF1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTF1.Reg) & 0x1
}

// IO_BANK0.PROC1_INTF2: Interrupt Force for proc1
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTF2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF2.Reg, volatile.LoadUint32(&o.PROC1_INTF2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTF2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTF2.Reg) & 0x1
}

// IO_BANK0.PROC1_INTF3: Interrupt Force for proc1
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTF3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF3.Reg, volatile.LoadUint32(&o.PROC1_INTF3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTF3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTF3.Reg) & 0x1
}

// IO_BANK0.PROC1_INTF4: Interrupt Force for proc1
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTF4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF4.Reg, volatile.LoadUint32(&o.PROC1_INTF4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTF4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTF4.Reg) & 0x1
}

// IO_BANK0.PROC1_INTF5: Interrupt Force for proc1
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTF5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTF5.Reg, volatile.LoadUint32(&o.PROC1_INTF5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTF5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTF5.Reg) & 0x1
}

// IO_BANK0.PROC1_INTS0: Interrupt status after masking & forcing for proc1
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTS0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS0.Reg, volatile.LoadUint32(&o.PROC1_INTS0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTS0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTS0.Reg) & 0x1
}

// IO_BANK0.PROC1_INTS1: Interrupt status after masking & forcing for proc1
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTS1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS1.Reg, volatile.LoadUint32(&o.PROC1_INTS1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTS1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTS1.Reg) & 0x1
}

// IO_BANK0.PROC1_INTS2: Interrupt status after masking & forcing for proc1
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTS2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS2.Reg, volatile.LoadUint32(&o.PROC1_INTS2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTS2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTS2.Reg) & 0x1
}

// IO_BANK0.PROC1_INTS3: Interrupt status after masking & forcing for proc1
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTS3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS3.Reg, volatile.LoadUint32(&o.PROC1_INTS3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTS3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTS3.Reg) & 0x1
}

// IO_BANK0.PROC1_INTS4: Interrupt status after masking & forcing for proc1
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTS4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS4.Reg, volatile.LoadUint32(&o.PROC1_INTS4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTS4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTS4.Reg) & 0x1
}

// IO_BANK0.PROC1_INTS5: Interrupt status after masking & forcing for proc1
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetPROC1_INTS5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.PROC1_INTS5.Reg, volatile.LoadUint32(&o.PROC1_INTS5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetPROC1_INTS5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.PROC1_INTS5.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTE0: Interrupt Enable for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTE0.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTE1: Interrupt Enable for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTE1.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTE2: Interrupt Enable for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTE2.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTE3: Interrupt Enable for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTE3.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTE4: Interrupt Enable for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTE4.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTE5: Interrupt Enable for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTE5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTE5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTE5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTE5.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTF0: Interrupt Force for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTF0.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTF1: Interrupt Force for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTF1.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTF2: Interrupt Force for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTF2.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTF3: Interrupt Force for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTF3.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTF4: Interrupt Force for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTF4.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTF5: Interrupt Force for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTF5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTF5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTF5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTF5.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTS0: Interrupt status after masking & forcing for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO7_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO7_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO7_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO7_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO7_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO7_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO7_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO7_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO6_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO6_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO6_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO6_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO6_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO6_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO6_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO6_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO5_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO5_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO5_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO5_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO5_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO5_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO5_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO5_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO4_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO4_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO4_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO4_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO4_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO4_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO4_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO4_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO3_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO3_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO3_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO3_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO3_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO3_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO3_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO3_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO2_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO2_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO2_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO2_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO2_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO2_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO2_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO2_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO1_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO1_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO1_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO1_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO1_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO1_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO1_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO1_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO0_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO0_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO0_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO0_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO0_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO0_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS0_GPIO0_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS0.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS0_GPIO0_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTS0.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTS1: Interrupt status after masking & forcing for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO15_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO15_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO15_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO15_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO15_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO15_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO15_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO15_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO14_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO14_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO14_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO14_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO14_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO14_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO14_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO14_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO13_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO13_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO13_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO13_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO13_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO13_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO13_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO13_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO12_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO12_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO12_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO12_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO12_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO12_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO12_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO12_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO11_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO11_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO11_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO11_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO11_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO11_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO11_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO11_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO10_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO10_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO10_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO10_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO10_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO10_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO10_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO10_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO9_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO9_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO9_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO9_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO9_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO9_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO9_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO9_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO8_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO8_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO8_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO8_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO8_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO8_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS1_GPIO8_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS1.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS1_GPIO8_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTS1.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTS2: Interrupt status after masking & forcing for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO23_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO23_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO23_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO23_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO23_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO23_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO23_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO23_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO22_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO22_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO22_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO22_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO22_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO22_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO22_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO22_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO21_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO21_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO21_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO21_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO21_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO21_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO21_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO21_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO20_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO20_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO20_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO20_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO20_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO20_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO20_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO20_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO19_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO19_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO19_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO19_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO19_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO19_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO19_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO19_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO18_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO18_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO18_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO18_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO18_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO18_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO18_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO18_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO17_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO17_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO17_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO17_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO17_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO17_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO17_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO17_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO16_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO16_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO16_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO16_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO16_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO16_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS2_GPIO16_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS2.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS2_GPIO16_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTS2.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTS3: Interrupt status after masking & forcing for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO31_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO31_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO31_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO31_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO31_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO31_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO31_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO31_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO30_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO30_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO30_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO30_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO30_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO30_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO30_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO30_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO29_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO29_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO29_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO29_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO29_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO29_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO29_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO29_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO28_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO28_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO28_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO28_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO28_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO28_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO28_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO28_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO27_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO27_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO27_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO27_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO27_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO27_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO27_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO27_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO26_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO26_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO26_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO26_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO26_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO26_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO26_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO26_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO25_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO25_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO25_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO25_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO25_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO25_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO25_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO25_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO24_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO24_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO24_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO24_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO24_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO24_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS3_GPIO24_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS3.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS3_GPIO24_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTS3.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTS4: Interrupt status after masking & forcing for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO39_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO39_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO39_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO39_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO39_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO39_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO39_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO39_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO38_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO38_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO38_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO38_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO38_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO38_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO38_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO38_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO37_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO37_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO37_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO37_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO37_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO37_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO37_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO37_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO36_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO36_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO36_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO36_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO36_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO36_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO36_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO36_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO35_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO35_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO35_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO35_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO35_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO35_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO35_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO35_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO34_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO34_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO34_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO34_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO34_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO34_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO34_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO34_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO33_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO33_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO33_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO33_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO33_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO33_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO33_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO33_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO32_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO32_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO32_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO32_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO32_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO32_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS4_GPIO32_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS4.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS4_GPIO32_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTS4.Reg) & 0x1
}

// IO_BANK0.DORMANT_WAKE_INTS5: Interrupt status after masking & forcing for dormant_wake
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO47_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x80000000)|value<<31)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO47_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x80000000) >> 31
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO47_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x40000000)|value<<30)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO47_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x40000000) >> 30
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO47_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x20000000)|value<<29)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO47_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x20000000) >> 29
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO47_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x10000000)|value<<28)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO47_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x10000000) >> 28
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO46_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x8000000)|value<<27)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO46_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x8000000) >> 27
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO46_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x4000000)|value<<26)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO46_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x4000000) >> 26
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO46_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x2000000)|value<<25)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO46_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x2000000) >> 25
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO46_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x1000000)|value<<24)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO46_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x1000000) >> 24
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO45_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x800000)|value<<23)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO45_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x800000) >> 23
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO45_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x400000)|value<<22)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO45_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x400000) >> 22
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO45_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x200000)|value<<21)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO45_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x200000) >> 21
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO45_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x100000)|value<<20)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO45_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x100000) >> 20
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO44_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x80000)|value<<19)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO44_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x80000) >> 19
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO44_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x40000)|value<<18)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO44_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x40000) >> 18
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO44_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x20000)|value<<17)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO44_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x20000) >> 17
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO44_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x10000)|value<<16)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO44_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x10000) >> 16
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO43_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x8000)|value<<15)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO43_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x8000) >> 15
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO43_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x4000)|value<<14)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO43_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x4000) >> 14
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO43_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x2000)|value<<13)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO43_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x2000) >> 13
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO43_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x1000)|value<<12)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO43_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x1000) >> 12
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO42_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x800)|value<<11)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO42_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x800) >> 11
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO42_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x400)|value<<10)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO42_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x400) >> 10
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO42_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x200)|value<<9)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO42_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x200) >> 9
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO42_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x100)|value<<8)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO42_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x100) >> 8
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO41_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x80)|value<<7)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO41_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x80) >> 7
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO41_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x40)|value<<6)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO41_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x40) >> 6
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO41_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x20)|value<<5)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO41_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x20) >> 5
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO41_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x10)|value<<4)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO41_LEVEL_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x10) >> 4
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO40_EDGE_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x8)|value<<3)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO40_EDGE_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x8) >> 3
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO40_EDGE_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x4)|value<<2)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO40_EDGE_LOW() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x4) >> 2
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO40_LEVEL_HIGH(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x2)|value<<1)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO40_LEVEL_HIGH() uint32 {
	return (volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x2) >> 1
}
func (o *IO_BANK0_Type) SetDORMANT_WAKE_INTS5_GPIO40_LEVEL_LOW(value uint32) {
	volatile.StoreUint32(&o.DORMANT_WAKE_INTS5.Reg, volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg)&^(0x1)|value)
}
func (o *IO_BANK0_Type) GetDORMANT_WAKE_INTS5_GPIO40_LEVEL_LOW() uint32 {
	return volatile.LoadUint32(&o.DORMANT_WAKE_INTS5.Reg) & 0x1
}

type SYSINFO_Type struct {
	CHIP_ID       volatile.Register32 // 0x0
	PACKAGE_SEL   volatile.Register32 // 0x4
	PLATFORM      volatile.Register32 // 0x8
	_             [8]byte
	GITREF_RP2350 volatile.Register32 // 0x14
}

// SYSINFO.CHIP_ID: JEDEC JEP-106 compliant chip identifier.
func (o *SYSINFO_Type) SetCHIP_ID_REVISION(value uint32) {
	volatile.StoreUint32(&o.CHIP_ID.Reg, volatile.LoadUint32(&o.CHIP_ID.Reg)&^(0xf0000000)|value<<28)
}
func (o *SYSINFO_Type) GetCHIP_ID_REVISION() uint32 {
	return (volatile.LoadUint32(&o.CHIP_ID.Reg) & 0xf0000000) >> 28
}
func (o *SYSINFO_Type) SetCHIP_ID_PART(value uint32) {
	volatile.StoreUint32(&o.CHIP_ID.Reg, volatile.LoadUint32(&o.CHIP_ID.Reg)&^(0xffff000)|value<<12)
}
func (o *SYSINFO_Type) GetCHIP_ID_PART() uint32 {
	return (volatile.LoadUint32(&o.CHIP_ID.Reg) & 0xffff000) >> 12
}
func (o *SYSINFO_Type) SetCHIP_ID_MANUFACTURER(value uint32) {
	volatile.StoreUint32(&o.CHIP_ID.Reg, volatile.LoadUint32(&o.CHIP_ID.Reg)&^(0xffe)|value<<1)
}
func (o *SYSINFO_Type) GetCHIP_ID_MANUFACTURER() uint32 {
	return (volatile.LoadUint32(&o.CHIP_ID.Reg) & 0xffe) >> 1
}
func (o *SYSINFO_Type) SetCHIP_ID_STOP_BIT(value uint32) {
	volatile.StoreUint32(&o.CHIP_ID.Reg, volatile.LoadUint32(&o.CHIP_ID.Reg)&^(0x1)|value)
}
func (o *SYSINFO_Type) GetCHIP_ID_STOP_BIT() uint32 {
	return volatile.LoadUint32(&o.CHIP_ID.Reg) & 0x1
}

// SYSINFO.PACKAGE_SEL
func (o *SYSINFO_Type) SetPACKAGE_SEL(value uint32) {
	volatile.StoreUint32(&o.PACKAGE_SEL.Reg, volatile.LoadUint32(&o.PACKAGE_SEL.Reg)&^(0x1)|value)
}
func (o *SYSINFO_Type) GetPACKAGE_SEL() uint32 {
	return volatile.LoadUint32(&o.PACKAGE_SEL.Reg) & 0x1
}

// SYSINFO.PLATFORM: Platform register. Allows software to know what environment it is running in during pre-production development. Post-production, the PLATFORM is always ASIC, non-SIM.
func (o *SYSINFO_Type) SetPLATFORM_GATESIM(value uint32) {
	volatile.StoreUint32(&o.PLATFORM.Reg, volatile.LoadUint32(&o.PLATFORM.Reg)&^(0x10)|value<<4)
}
func (o *SYSINFO_Type) GetPLATFORM_GATESIM() uint32 {
	return (volatile.LoadUint32(&o.PLATFORM.Reg) & 0x10) >> 4
}
func (o *SYSINFO_Type) SetPLATFORM_BATCHSIM(value uint32) {
	volatile.StoreUint32(&o.PLATFORM.Reg, volatile.LoadUint32(&o.PLATFORM.Reg)&^(0x8)|value<<3)
}
func (o *SYSINFO_Type) GetPLATFORM_BATCHSIM() uint32 {
	return (volatile.LoadUint32(&o.PLATFORM.Reg) & 0x8) >> 3
}
func (o *SYSINFO_Type) SetPLATFORM_HDLSIM(value uint32) {
	volatile.StoreUint32(&o.PLATFORM.Reg, volatile.LoadUint32(&o.PLATFORM.Reg)&^(0x4)|value<<2)
}
func (o *SYSINFO_Type) GetPLATFORM_HDLSIM() uint32 {
	return (volatile.LoadUint32(&o.PLATFORM.Reg) & 0x4) >> 2
}
func (o *SYSINFO_Type) SetPLATFORM_ASIC(value uint32) {
	volatile.StoreUint32(&o.PLATFORM.Reg, volatile.LoadUint32(&o.PLATFORM.Reg)&^(0x2)|value<<1)
}
func (o *SYSINFO_Type) GetPLATFORM_ASIC() uint32 {
	return (volatile.LoadUint32(&o.PLATFORM.Reg) & 0x2) >> 1
}
func (o *SYSINFO_Type) SetPLATFORM_FPGA(value uint32) {
	volatile.StoreUint32(&o.PLATFORM.Reg, volatile.LoadUint32(&o.PLATFORM.Reg)&^(0x1)|value)
}
func (o *SYSINFO_Type) GetPLATFORM_FPGA() uint32 {
	return volatile.LoadUint32(&o.PLATFORM.Reg) & 0x1
}

// SYSINFO.GITREF_RP2350: Git hash of the chip source. Used to identify chip version.
func (o *SYSINFO_Type) SetGITREF_RP2350(value uint32) {
	volatile.StoreUint32(&o.GITREF_RP2350.Reg, value)
}
func (o *SYSINFO_Type) GetGITREF_RP2350() uint32 {
	return volatile.LoadUint32(&o.GITREF_RP2350.Reg)
}

// SHA-256 hash function implementation
type SHA256_Type struct {
	CSR   volatile.Register32 // 0x0
	WDATA volatile.Register32 // 0x4
	SUM0  volatile.Register32 // 0x8
	SUM1  volatile.Register32 // 0xC
	SUM2  volatile.Register32 // 0x10
	SUM3  volatile.Register32 // 0x14
	SUM4  volatile.Register32 // 0x18
	SUM5  volatile.Register32 // 0x1C
	SUM6  volatile.Register32 // 0x20
	SUM7  volatile.Register32 // 0x24
}

// SHA256.CSR: Control and status register
func (o *SHA256_Type) SetCSR_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x1000)|value<<12)
}
func (o *SHA256_Type) GetCSR_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x1000) >> 12
}
func (o *SHA256_Type) SetCSR_DMA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x300)|value<<8)
}
func (o *SHA256_Type) GetCSR_DMA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x300) >> 8
}
func (o *SHA256_Type) SetCSR_ERR_WDATA_NOT_RDY(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x10)|value<<4)
}
func (o *SHA256_Type) GetCSR_ERR_WDATA_NOT_RDY() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x10) >> 4
}
func (o *SHA256_Type) SetCSR_SUM_VLD(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x4)|value<<2)
}
func (o *SHA256_Type) GetCSR_SUM_VLD() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x4) >> 2
}
func (o *SHA256_Type) SetCSR_WDATA_RDY(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x2)|value<<1)
}
func (o *SHA256_Type) GetCSR_WDATA_RDY() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x2) >> 1
}
func (o *SHA256_Type) SetCSR_START(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x1)|value)
}
func (o *SHA256_Type) GetCSR_START() uint32 {
	return volatile.LoadUint32(&o.CSR.Reg) & 0x1
}

// SHA256.WDATA: Write data register
func (o *SHA256_Type) SetWDATA(value uint32) {
	volatile.StoreUint32(&o.WDATA.Reg, value)
}
func (o *SHA256_Type) GetWDATA() uint32 {
	return volatile.LoadUint32(&o.WDATA.Reg)
}

// SHA256.SUM0: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
func (o *SHA256_Type) SetSUM0(value uint32) {
	volatile.StoreUint32(&o.SUM0.Reg, value)
}
func (o *SHA256_Type) GetSUM0() uint32 {
	return volatile.LoadUint32(&o.SUM0.Reg)
}

// SHA256.SUM1: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
func (o *SHA256_Type) SetSUM1(value uint32) {
	volatile.StoreUint32(&o.SUM1.Reg, value)
}
func (o *SHA256_Type) GetSUM1() uint32 {
	return volatile.LoadUint32(&o.SUM1.Reg)
}

// SHA256.SUM2: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
func (o *SHA256_Type) SetSUM2(value uint32) {
	volatile.StoreUint32(&o.SUM2.Reg, value)
}
func (o *SHA256_Type) GetSUM2() uint32 {
	return volatile.LoadUint32(&o.SUM2.Reg)
}

// SHA256.SUM3: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
func (o *SHA256_Type) SetSUM3(value uint32) {
	volatile.StoreUint32(&o.SUM3.Reg, value)
}
func (o *SHA256_Type) GetSUM3() uint32 {
	return volatile.LoadUint32(&o.SUM3.Reg)
}

// SHA256.SUM4: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
func (o *SHA256_Type) SetSUM4(value uint32) {
	volatile.StoreUint32(&o.SUM4.Reg, value)
}
func (o *SHA256_Type) GetSUM4() uint32 {
	return volatile.LoadUint32(&o.SUM4.Reg)
}

// SHA256.SUM5: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
func (o *SHA256_Type) SetSUM5(value uint32) {
	volatile.StoreUint32(&o.SUM5.Reg, value)
}
func (o *SHA256_Type) GetSUM5() uint32 {
	return volatile.LoadUint32(&o.SUM5.Reg)
}

// SHA256.SUM6: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
func (o *SHA256_Type) SetSUM6(value uint32) {
	volatile.StoreUint32(&o.SUM6.Reg, value)
}
func (o *SHA256_Type) GetSUM6() uint32 {
	return volatile.LoadUint32(&o.SUM6.Reg)
}

// SHA256.SUM7: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
func (o *SHA256_Type) SetSUM7(value uint32) {
	volatile.StoreUint32(&o.SUM7.Reg, value)
}
func (o *SHA256_Type) GetSUM7() uint32 {
	return volatile.LoadUint32(&o.SUM7.Reg)
}

// FIFO status and write access for HSTX
type HSTX_FIFO_Type struct {
	STAT volatile.Register32 // 0x0
	FIFO volatile.Register32 // 0x4
}

// HSTX_FIFO.STAT: FIFO status
func (o *HSTX_FIFO_Type) SetSTAT_WOF(value uint32) {
	volatile.StoreUint32(&o.STAT.Reg, volatile.LoadUint32(&o.STAT.Reg)&^(0x400)|value<<10)
}
func (o *HSTX_FIFO_Type) GetSTAT_WOF() uint32 {
	return (volatile.LoadUint32(&o.STAT.Reg) & 0x400) >> 10
}
func (o *HSTX_FIFO_Type) SetSTAT_EMPTY(value uint32) {
	volatile.StoreUint32(&o.STAT.Reg, volatile.LoadUint32(&o.STAT.Reg)&^(0x200)|value<<9)
}
func (o *HSTX_FIFO_Type) GetSTAT_EMPTY() uint32 {
	return (volatile.LoadUint32(&o.STAT.Reg) & 0x200) >> 9
}
func (o *HSTX_FIFO_Type) SetSTAT_FULL(value uint32) {
	volatile.StoreUint32(&o.STAT.Reg, volatile.LoadUint32(&o.STAT.Reg)&^(0x100)|value<<8)
}
func (o *HSTX_FIFO_Type) GetSTAT_FULL() uint32 {
	return (volatile.LoadUint32(&o.STAT.Reg) & 0x100) >> 8
}
func (o *HSTX_FIFO_Type) SetSTAT_LEVEL(value uint32) {
	volatile.StoreUint32(&o.STAT.Reg, volatile.LoadUint32(&o.STAT.Reg)&^(0xff)|value)
}
func (o *HSTX_FIFO_Type) GetSTAT_LEVEL() uint32 {
	return volatile.LoadUint32(&o.STAT.Reg) & 0xff
}

// HSTX_FIFO.FIFO: Write access to FIFO
func (o *HSTX_FIFO_Type) SetFIFO(value uint32) {
	volatile.StoreUint32(&o.FIFO.Reg, value)
}
func (o *HSTX_FIFO_Type) GetFIFO() uint32 {
	return volatile.LoadUint32(&o.FIFO.Reg)
}

// Control interface to HSTX. For FIFO write access and status, see the HSTX_FIFO register block.
type HSTX_CTRL_Type struct {
	CSR          volatile.Register32 // 0x0
	BIT0         volatile.Register32 // 0x4
	BIT1         volatile.Register32 // 0x8
	BIT2         volatile.Register32 // 0xC
	BIT3         volatile.Register32 // 0x10
	BIT4         volatile.Register32 // 0x14
	BIT5         volatile.Register32 // 0x18
	BIT6         volatile.Register32 // 0x1C
	BIT7         volatile.Register32 // 0x20
	EXPAND_SHIFT volatile.Register32 // 0x24
	EXPAND_TMDS  volatile.Register32 // 0x28
}

// HSTX_CTRL.CSR
func (o *HSTX_CTRL_Type) SetCSR_CLKDIV(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0xf0000000)|value<<28)
}
func (o *HSTX_CTRL_Type) GetCSR_CLKDIV() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0xf0000000) >> 28
}
func (o *HSTX_CTRL_Type) SetCSR_CLKPHASE(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0xf000000)|value<<24)
}
func (o *HSTX_CTRL_Type) GetCSR_CLKPHASE() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0xf000000) >> 24
}
func (o *HSTX_CTRL_Type) SetCSR_N_SHIFTS(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x1f0000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetCSR_N_SHIFTS() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x1f0000) >> 16
}
func (o *HSTX_CTRL_Type) SetCSR_SHIFT(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetCSR_SHIFT() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetCSR_COUPLED_SEL(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x60)|value<<5)
}
func (o *HSTX_CTRL_Type) GetCSR_COUPLED_SEL() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x60) >> 5
}
func (o *HSTX_CTRL_Type) SetCSR_COUPLED_MODE(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x10)|value<<4)
}
func (o *HSTX_CTRL_Type) GetCSR_COUPLED_MODE() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x10) >> 4
}
func (o *HSTX_CTRL_Type) SetCSR_EXPAND_EN(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x2)|value<<1)
}
func (o *HSTX_CTRL_Type) GetCSR_EXPAND_EN() uint32 {
	return (volatile.LoadUint32(&o.CSR.Reg) & 0x2) >> 1
}
func (o *HSTX_CTRL_Type) SetCSR_EN(value uint32) {
	volatile.StoreUint32(&o.CSR.Reg, volatile.LoadUint32(&o.CSR.Reg)&^(0x1)|value)
}
func (o *HSTX_CTRL_Type) GetCSR_EN() uint32 {
	return volatile.LoadUint32(&o.CSR.Reg) & 0x1
}

// HSTX_CTRL.BIT0: Data control register for output bit 0
func (o *HSTX_CTRL_Type) SetBIT0_CLK(value uint32) {
	volatile.StoreUint32(&o.BIT0.Reg, volatile.LoadUint32(&o.BIT0.Reg)&^(0x20000)|value<<17)
}
func (o *HSTX_CTRL_Type) GetBIT0_CLK() uint32 {
	return (volatile.LoadUint32(&o.BIT0.Reg) & 0x20000) >> 17
}
func (o *HSTX_CTRL_Type) SetBIT0_INV(value uint32) {
	volatile.StoreUint32(&o.BIT0.Reg, volatile.LoadUint32(&o.BIT0.Reg)&^(0x10000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetBIT0_INV() uint32 {
	return (volatile.LoadUint32(&o.BIT0.Reg) & 0x10000) >> 16
}
func (o *HSTX_CTRL_Type) SetBIT0_SEL_N(value uint32) {
	volatile.StoreUint32(&o.BIT0.Reg, volatile.LoadUint32(&o.BIT0.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetBIT0_SEL_N() uint32 {
	return (volatile.LoadUint32(&o.BIT0.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetBIT0_SEL_P(value uint32) {
	volatile.StoreUint32(&o.BIT0.Reg, volatile.LoadUint32(&o.BIT0.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetBIT0_SEL_P() uint32 {
	return volatile.LoadUint32(&o.BIT0.Reg) & 0x1f
}

// HSTX_CTRL.BIT1: Data control register for output bit 1
func (o *HSTX_CTRL_Type) SetBIT1_CLK(value uint32) {
	volatile.StoreUint32(&o.BIT1.Reg, volatile.LoadUint32(&o.BIT1.Reg)&^(0x20000)|value<<17)
}
func (o *HSTX_CTRL_Type) GetBIT1_CLK() uint32 {
	return (volatile.LoadUint32(&o.BIT1.Reg) & 0x20000) >> 17
}
func (o *HSTX_CTRL_Type) SetBIT1_INV(value uint32) {
	volatile.StoreUint32(&o.BIT1.Reg, volatile.LoadUint32(&o.BIT1.Reg)&^(0x10000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetBIT1_INV() uint32 {
	return (volatile.LoadUint32(&o.BIT1.Reg) & 0x10000) >> 16
}
func (o *HSTX_CTRL_Type) SetBIT1_SEL_N(value uint32) {
	volatile.StoreUint32(&o.BIT1.Reg, volatile.LoadUint32(&o.BIT1.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetBIT1_SEL_N() uint32 {
	return (volatile.LoadUint32(&o.BIT1.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetBIT1_SEL_P(value uint32) {
	volatile.StoreUint32(&o.BIT1.Reg, volatile.LoadUint32(&o.BIT1.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetBIT1_SEL_P() uint32 {
	return volatile.LoadUint32(&o.BIT1.Reg) & 0x1f
}

// HSTX_CTRL.BIT2: Data control register for output bit 2
func (o *HSTX_CTRL_Type) SetBIT2_CLK(value uint32) {
	volatile.StoreUint32(&o.BIT2.Reg, volatile.LoadUint32(&o.BIT2.Reg)&^(0x20000)|value<<17)
}
func (o *HSTX_CTRL_Type) GetBIT2_CLK() uint32 {
	return (volatile.LoadUint32(&o.BIT2.Reg) & 0x20000) >> 17
}
func (o *HSTX_CTRL_Type) SetBIT2_INV(value uint32) {
	volatile.StoreUint32(&o.BIT2.Reg, volatile.LoadUint32(&o.BIT2.Reg)&^(0x10000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetBIT2_INV() uint32 {
	return (volatile.LoadUint32(&o.BIT2.Reg) & 0x10000) >> 16
}
func (o *HSTX_CTRL_Type) SetBIT2_SEL_N(value uint32) {
	volatile.StoreUint32(&o.BIT2.Reg, volatile.LoadUint32(&o.BIT2.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetBIT2_SEL_N() uint32 {
	return (volatile.LoadUint32(&o.BIT2.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetBIT2_SEL_P(value uint32) {
	volatile.StoreUint32(&o.BIT2.Reg, volatile.LoadUint32(&o.BIT2.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetBIT2_SEL_P() uint32 {
	return volatile.LoadUint32(&o.BIT2.Reg) & 0x1f
}

// HSTX_CTRL.BIT3: Data control register for output bit 3
func (o *HSTX_CTRL_Type) SetBIT3_CLK(value uint32) {
	volatile.StoreUint32(&o.BIT3.Reg, volatile.LoadUint32(&o.BIT3.Reg)&^(0x20000)|value<<17)
}
func (o *HSTX_CTRL_Type) GetBIT3_CLK() uint32 {
	return (volatile.LoadUint32(&o.BIT3.Reg) & 0x20000) >> 17
}
func (o *HSTX_CTRL_Type) SetBIT3_INV(value uint32) {
	volatile.StoreUint32(&o.BIT3.Reg, volatile.LoadUint32(&o.BIT3.Reg)&^(0x10000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetBIT3_INV() uint32 {
	return (volatile.LoadUint32(&o.BIT3.Reg) & 0x10000) >> 16
}
func (o *HSTX_CTRL_Type) SetBIT3_SEL_N(value uint32) {
	volatile.StoreUint32(&o.BIT3.Reg, volatile.LoadUint32(&o.BIT3.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetBIT3_SEL_N() uint32 {
	return (volatile.LoadUint32(&o.BIT3.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetBIT3_SEL_P(value uint32) {
	volatile.StoreUint32(&o.BIT3.Reg, volatile.LoadUint32(&o.BIT3.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetBIT3_SEL_P() uint32 {
	return volatile.LoadUint32(&o.BIT3.Reg) & 0x1f
}

// HSTX_CTRL.BIT4: Data control register for output bit 4
func (o *HSTX_CTRL_Type) SetBIT4_CLK(value uint32) {
	volatile.StoreUint32(&o.BIT4.Reg, volatile.LoadUint32(&o.BIT4.Reg)&^(0x20000)|value<<17)
}
func (o *HSTX_CTRL_Type) GetBIT4_CLK() uint32 {
	return (volatile.LoadUint32(&o.BIT4.Reg) & 0x20000) >> 17
}
func (o *HSTX_CTRL_Type) SetBIT4_INV(value uint32) {
	volatile.StoreUint32(&o.BIT4.Reg, volatile.LoadUint32(&o.BIT4.Reg)&^(0x10000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetBIT4_INV() uint32 {
	return (volatile.LoadUint32(&o.BIT4.Reg) & 0x10000) >> 16
}
func (o *HSTX_CTRL_Type) SetBIT4_SEL_N(value uint32) {
	volatile.StoreUint32(&o.BIT4.Reg, volatile.LoadUint32(&o.BIT4.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetBIT4_SEL_N() uint32 {
	return (volatile.LoadUint32(&o.BIT4.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetBIT4_SEL_P(value uint32) {
	volatile.StoreUint32(&o.BIT4.Reg, volatile.LoadUint32(&o.BIT4.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetBIT4_SEL_P() uint32 {
	return volatile.LoadUint32(&o.BIT4.Reg) & 0x1f
}

// HSTX_CTRL.BIT5: Data control register for output bit 5
func (o *HSTX_CTRL_Type) SetBIT5_CLK(value uint32) {
	volatile.StoreUint32(&o.BIT5.Reg, volatile.LoadUint32(&o.BIT5.Reg)&^(0x20000)|value<<17)
}
func (o *HSTX_CTRL_Type) GetBIT5_CLK() uint32 {
	return (volatile.LoadUint32(&o.BIT5.Reg) & 0x20000) >> 17
}
func (o *HSTX_CTRL_Type) SetBIT5_INV(value uint32) {
	volatile.StoreUint32(&o.BIT5.Reg, volatile.LoadUint32(&o.BIT5.Reg)&^(0x10000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetBIT5_INV() uint32 {
	return (volatile.LoadUint32(&o.BIT5.Reg) & 0x10000) >> 16
}
func (o *HSTX_CTRL_Type) SetBIT5_SEL_N(value uint32) {
	volatile.StoreUint32(&o.BIT5.Reg, volatile.LoadUint32(&o.BIT5.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetBIT5_SEL_N() uint32 {
	return (volatile.LoadUint32(&o.BIT5.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetBIT5_SEL_P(value uint32) {
	volatile.StoreUint32(&o.BIT5.Reg, volatile.LoadUint32(&o.BIT5.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetBIT5_SEL_P() uint32 {
	return volatile.LoadUint32(&o.BIT5.Reg) & 0x1f
}

// HSTX_CTRL.BIT6: Data control register for output bit 6
func (o *HSTX_CTRL_Type) SetBIT6_CLK(value uint32) {
	volatile.StoreUint32(&o.BIT6.Reg, volatile.LoadUint32(&o.BIT6.Reg)&^(0x20000)|value<<17)
}
func (o *HSTX_CTRL_Type) GetBIT6_CLK() uint32 {
	return (volatile.LoadUint32(&o.BIT6.Reg) & 0x20000) >> 17
}
func (o *HSTX_CTRL_Type) SetBIT6_INV(value uint32) {
	volatile.StoreUint32(&o.BIT6.Reg, volatile.LoadUint32(&o.BIT6.Reg)&^(0x10000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetBIT6_INV() uint32 {
	return (volatile.LoadUint32(&o.BIT6.Reg) & 0x10000) >> 16
}
func (o *HSTX_CTRL_Type) SetBIT6_SEL_N(value uint32) {
	volatile.StoreUint32(&o.BIT6.Reg, volatile.LoadUint32(&o.BIT6.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetBIT6_SEL_N() uint32 {
	return (volatile.LoadUint32(&o.BIT6.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetBIT6_SEL_P(value uint32) {
	volatile.StoreUint32(&o.BIT6.Reg, volatile.LoadUint32(&o.BIT6.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetBIT6_SEL_P() uint32 {
	return volatile.LoadUint32(&o.BIT6.Reg) & 0x1f
}

// HSTX_CTRL.BIT7: Data control register for output bit 7
func (o *HSTX_CTRL_Type) SetBIT7_CLK(value uint32) {
	volatile.StoreUint32(&o.BIT7.Reg, volatile.LoadUint32(&o.BIT7.Reg)&^(0x20000)|value<<17)
}
func (o *HSTX_CTRL_Type) GetBIT7_CLK() uint32 {
	return (volatile.LoadUint32(&o.BIT7.Reg) & 0x20000) >> 17
}
func (o *HSTX_CTRL_Type) SetBIT7_INV(value uint32) {
	volatile.StoreUint32(&o.BIT7.Reg, volatile.LoadUint32(&o.BIT7.Reg)&^(0x10000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetBIT7_INV() uint32 {
	return (volatile.LoadUint32(&o.BIT7.Reg) & 0x10000) >> 16
}
func (o *HSTX_CTRL_Type) SetBIT7_SEL_N(value uint32) {
	volatile.StoreUint32(&o.BIT7.Reg, volatile.LoadUint32(&o.BIT7.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetBIT7_SEL_N() uint32 {
	return (volatile.LoadUint32(&o.BIT7.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetBIT7_SEL_P(value uint32) {
	volatile.StoreUint32(&o.BIT7.Reg, volatile.LoadUint32(&o.BIT7.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetBIT7_SEL_P() uint32 {
	return volatile.LoadUint32(&o.BIT7.Reg) & 0x1f
}

// HSTX_CTRL.EXPAND_SHIFT: Configure the optional shifter inside the command expander
func (o *HSTX_CTRL_Type) SetEXPAND_SHIFT_ENC_N_SHIFTS(value uint32) {
	volatile.StoreUint32(&o.EXPAND_SHIFT.Reg, volatile.LoadUint32(&o.EXPAND_SHIFT.Reg)&^(0x1f000000)|value<<24)
}
func (o *HSTX_CTRL_Type) GetEXPAND_SHIFT_ENC_N_SHIFTS() uint32 {
	return (volatile.LoadUint32(&o.EXPAND_SHIFT.Reg) & 0x1f000000) >> 24
}
func (o *HSTX_CTRL_Type) SetEXPAND_SHIFT_ENC_SHIFT(value uint32) {
	volatile.StoreUint32(&o.EXPAND_SHIFT.Reg, volatile.LoadUint32(&o.EXPAND_SHIFT.Reg)&^(0x1f0000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetEXPAND_SHIFT_ENC_SHIFT() uint32 {
	return (volatile.LoadUint32(&o.EXPAND_SHIFT.Reg) & 0x1f0000) >> 16
}
func (o *HSTX_CTRL_Type) SetEXPAND_SHIFT_RAW_N_SHIFTS(value uint32) {
	volatile.StoreUint32(&o.EXPAND_SHIFT.Reg, volatile.LoadUint32(&o.EXPAND_SHIFT.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetEXPAND_SHIFT_RAW_N_SHIFTS() uint32 {
	return (volatile.LoadUint32(&o.EXPAND_SHIFT.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetEXPAND_SHIFT_RAW_SHIFT(value uint32) {
	volatile.StoreUint32(&o.EXPAND_SHIFT.Reg, volatile.LoadUint32(&o.EXPAND_SHIFT.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetEXPAND_SHIFT_RAW_SHIFT() uint32 {
	return volatile.LoadUint32(&o.EXPAND_SHIFT.Reg) & 0x1f
}

// HSTX_CTRL.EXPAND_TMDS: Configure the optional TMDS encoder inside the command expander
func (o *HSTX_CTRL_Type) SetEXPAND_TMDS_L2_NBITS(value uint32) {
	volatile.StoreUint32(&o.EXPAND_TMDS.Reg, volatile.LoadUint32(&o.EXPAND_TMDS.Reg)&^(0xe00000)|value<<21)
}
func (o *HSTX_CTRL_Type) GetEXPAND_TMDS_L2_NBITS() uint32 {
	return (volatile.LoadUint32(&o.EXPAND_TMDS.Reg) & 0xe00000) >> 21
}
func (o *HSTX_CTRL_Type) SetEXPAND_TMDS_L2_ROT(value uint32) {
	volatile.StoreUint32(&o.EXPAND_TMDS.Reg, volatile.LoadUint32(&o.EXPAND_TMDS.Reg)&^(0x1f0000)|value<<16)
}
func (o *HSTX_CTRL_Type) GetEXPAND_TMDS_L2_ROT() uint32 {
	return (volatile.LoadUint32(&o.EXPAND_TMDS.Reg) & 0x1f0000) >> 16
}
func (o *HSTX_CTRL_Type) SetEXPAND_TMDS_L1_NBITS(value uint32) {
	volatile.StoreUint32(&o.EXPAND_TMDS.Reg, volatile.LoadUint32(&o.EXPAND_TMDS.Reg)&^(0xe000)|value<<13)
}
func (o *HSTX_CTRL_Type) GetEXPAND_TMDS_L1_NBITS() uint32 {
	return (volatile.LoadUint32(&o.EXPAND_TMDS.Reg) & 0xe000) >> 13
}
func (o *HSTX_CTRL_Type) SetEXPAND_TMDS_L1_ROT(value uint32) {
	volatile.StoreUint32(&o.EXPAND_TMDS.Reg, volatile.LoadUint32(&o.EXPAND_TMDS.Reg)&^(0x1f00)|value<<8)
}
func (o *HSTX_CTRL_Type) GetEXPAND_TMDS_L1_ROT() uint32 {
	return (volatile.LoadUint32(&o.EXPAND_TMDS.Reg) & 0x1f00) >> 8
}
func (o *HSTX_CTRL_Type) SetEXPAND_TMDS_L0_NBITS(value uint32) {
	volatile.StoreUint32(&o.EXPAND_TMDS.Reg, volatile.LoadUint32(&o.EXPAND_TMDS.Reg)&^(0xe0)|value<<5)
}
func (o *HSTX_CTRL_Type) GetEXPAND_TMDS_L0_NBITS() uint32 {
	return (volatile.LoadUint32(&o.EXPAND_TMDS.Reg) & 0xe0) >> 5
}
func (o *HSTX_CTRL_Type) SetEXPAND_TMDS_L0_ROT(value uint32) {
	volatile.StoreUint32(&o.EXPAND_TMDS.Reg, volatile.LoadUint32(&o.EXPAND_TMDS.Reg)&^(0x1f)|value)
}
func (o *HSTX_CTRL_Type) GetEXPAND_TMDS_L0_ROT() uint32 {
	return volatile.LoadUint32(&o.EXPAND_TMDS.Reg) & 0x1f
}

// Cortex-M33 EPPB vendor register block for RP2350
type EPPB_Type struct {
	NMI_MASK0 volatile.Register32 // 0x0
	NMI_MASK1 volatile.Register32 // 0x4
	SLEEPCTRL volatile.Register32 // 0x8
}

// EPPB.NMI_MASK0: NMI mask for IRQs 0 through 31. This register is core-local, and is reset by a processor warm reset.
func (o *EPPB_Type) SetNMI_MASK0(value uint32) {
	volatile.StoreUint32(&o.NMI_MASK0.Reg, value)
}
func (o *EPPB_Type) GetNMI_MASK0() uint32 {
	return volatile.LoadUint32(&o.NMI_MASK0.Reg)
}

// EPPB.NMI_MASK1: NMI mask for IRQs 0 though 51. This register is core-local, and is reset by a processor warm reset.
func (o *EPPB_Type) SetNMI_MASK1(value uint32) {
	volatile.StoreUint32(&o.NMI_MASK1.Reg, volatile.LoadUint32(&o.NMI_MASK1.Reg)&^(0xfffff)|value)
}
func (o *EPPB_Type) GetNMI_MASK1() uint32 {
	return volatile.LoadUint32(&o.NMI_MASK1.Reg) & 0xfffff
}

// EPPB.SLEEPCTRL: Nonstandard sleep control register
func (o *EPPB_Type) SetSLEEPCTRL_WICENACK(value uint32) {
	volatile.StoreUint32(&o.SLEEPCTRL.Reg, volatile.LoadUint32(&o.SLEEPCTRL.Reg)&^(0x4)|value<<2)
}
func (o *EPPB_Type) GetSLEEPCTRL_WICENACK() uint32 {
	return (volatile.LoadUint32(&o.SLEEPCTRL.Reg) & 0x4) >> 2
}
func (o *EPPB_Type) SetSLEEPCTRL_WICENREQ(value uint32) {
	volatile.StoreUint32(&o.SLEEPCTRL.Reg, volatile.LoadUint32(&o.SLEEPCTRL.Reg)&^(0x2)|value<<1)
}
func (o *EPPB_Type) GetSLEEPCTRL_WICENREQ() uint32 {
	return (volatile.LoadUint32(&o.SLEEPCTRL.Reg) & 0x2) >> 1
}
func (o *EPPB_Type) SetSLEEPCTRL_LIGHT_SLEEP(value uint32) {
	volatile.StoreUint32(&o.SLEEPCTRL.Reg, volatile.LoadUint32(&o.SLEEPCTRL.Reg)&^(0x1)|value)
}
func (o *EPPB_Type) GetSLEEPCTRL_LIGHT_SLEEP() uint32 {
	return volatile.LoadUint32(&o.SLEEPCTRL.Reg) & 0x1
}

// TEAL registers accessible through the debug interface
type PPB_Type struct {
	ITM_STIM0        volatile.Register32 // 0x0
	ITM_STIM1        volatile.Register32 // 0x4
	ITM_STIM2        volatile.Register32 // 0x8
	ITM_STIM3        volatile.Register32 // 0xC
	ITM_STIM4        volatile.Register32 // 0x10
	ITM_STIM5        volatile.Register32 // 0x14
	ITM_STIM6        volatile.Register32 // 0x18
	ITM_STIM7        volatile.Register32 // 0x1C
	ITM_STIM8        volatile.Register32 // 0x20
	ITM_STIM9        volatile.Register32 // 0x24
	ITM_STIM10       volatile.Register32 // 0x28
	ITM_STIM11       volatile.Register32 // 0x2C
	ITM_STIM12       volatile.Register32 // 0x30
	ITM_STIM13       volatile.Register32 // 0x34
	ITM_STIM14       volatile.Register32 // 0x38
	ITM_STIM15       volatile.Register32 // 0x3C
	ITM_STIM16       volatile.Register32 // 0x40
	ITM_STIM17       volatile.Register32 // 0x44
	ITM_STIM18       volatile.Register32 // 0x48
	ITM_STIM19       volatile.Register32 // 0x4C
	ITM_STIM20       volatile.Register32 // 0x50
	ITM_STIM21       volatile.Register32 // 0x54
	ITM_STIM22       volatile.Register32 // 0x58
	ITM_STIM23       volatile.Register32 // 0x5C
	ITM_STIM24       volatile.Register32 // 0x60
	ITM_STIM25       volatile.Register32 // 0x64
	ITM_STIM26       volatile.Register32 // 0x68
	ITM_STIM27       volatile.Register32 // 0x6C
	ITM_STIM28       volatile.Register32 // 0x70
	ITM_STIM29       volatile.Register32 // 0x74
	ITM_STIM30       volatile.Register32 // 0x78
	ITM_STIM31       volatile.Register32 // 0x7C
	_                [3456]byte
	ITM_TER0         volatile.Register32 // 0xE00
	_                [60]byte
	ITM_TPR          volatile.Register32 // 0xE40
	_                [60]byte
	ITM_TCR          volatile.Register32 // 0xE80
	_                [108]byte
	INT_ATREADY      volatile.Register32 // 0xEF0
	_                [4]byte
	INT_ATVALID      volatile.Register32 // 0xEF8
	_                [4]byte
	ITM_ITCTRL       volatile.Register32 // 0xF00
	_                [184]byte
	ITM_DEVARCH      volatile.Register32 // 0xFBC
	_                [12]byte
	ITM_DEVTYPE      volatile.Register32 // 0xFCC
	ITM_PIDR4        volatile.Register32 // 0xFD0
	ITM_PIDR5        volatile.Register32 // 0xFD4
	ITM_PIDR6        volatile.Register32 // 0xFD8
	ITM_PIDR7        volatile.Register32 // 0xFDC
	ITM_PIDR0        volatile.Register32 // 0xFE0
	ITM_PIDR1        volatile.Register32 // 0xFE4
	ITM_PIDR2        volatile.Register32 // 0xFE8
	ITM_PIDR3        volatile.Register32 // 0xFEC
	ITM_CIDR0        volatile.Register32 // 0xFF0
	ITM_CIDR1        volatile.Register32 // 0xFF4
	ITM_CIDR2        volatile.Register32 // 0xFF8
	ITM_CIDR3        volatile.Register32 // 0xFFC
	DWT_CTRL         volatile.Register32 // 0x1000
	DWT_CYCCNT       volatile.Register32 // 0x1004
	_                [4]byte
	DWT_EXCCNT       volatile.Register32 // 0x100C
	_                [4]byte
	DWT_LSUCNT       volatile.Register32 // 0x1014
	DWT_FOLDCNT      volatile.Register32 // 0x1018
	_                [4]byte
	DWT_COMP0        volatile.Register32 // 0x1020
	_                [4]byte
	DWT_FUNCTION0    volatile.Register32 // 0x1028
	_                [4]byte
	DWT_COMP1        volatile.Register32 // 0x1030
	_                [4]byte
	DWT_FUNCTION1    volatile.Register32 // 0x1038
	_                [4]byte
	DWT_COMP2        volatile.Register32 // 0x1040
	_                [4]byte
	DWT_FUNCTION2    volatile.Register32 // 0x1048
	_                [4]byte
	DWT_COMP3        volatile.Register32 // 0x1050
	_                [4]byte
	DWT_FUNCTION3    volatile.Register32 // 0x1058
	_                [3936]byte
	DWT_DEVARCH      volatile.Register32 // 0x1FBC
	_                [12]byte
	DWT_DEVTYPE      volatile.Register32 // 0x1FCC
	DWT_PIDR4        volatile.Register32 // 0x1FD0
	DWT_PIDR5        volatile.Register32 // 0x1FD4
	DWT_PIDR6        volatile.Register32 // 0x1FD8
	DWT_PIDR7        volatile.Register32 // 0x1FDC
	DWT_PIDR0        volatile.Register32 // 0x1FE0
	DWT_PIDR1        volatile.Register32 // 0x1FE4
	DWT_PIDR2        volatile.Register32 // 0x1FE8
	DWT_PIDR3        volatile.Register32 // 0x1FEC
	DWT_CIDR0        volatile.Register32 // 0x1FF0
	DWT_CIDR1        volatile.Register32 // 0x1FF4
	DWT_CIDR2        volatile.Register32 // 0x1FF8
	DWT_CIDR3        volatile.Register32 // 0x1FFC
	FP_CTRL          volatile.Register32 // 0x2000
	FP_REMAP         volatile.Register32 // 0x2004
	FP_COMP0         volatile.Register32 // 0x2008
	FP_COMP1         volatile.Register32 // 0x200C
	FP_COMP2         volatile.Register32 // 0x2010
	FP_COMP3         volatile.Register32 // 0x2014
	FP_COMP4         volatile.Register32 // 0x2018
	FP_COMP5         volatile.Register32 // 0x201C
	FP_COMP6         volatile.Register32 // 0x2020
	FP_COMP7         volatile.Register32 // 0x2024
	_                [3988]byte
	FP_DEVARCH       volatile.Register32 // 0x2FBC
	_                [12]byte
	FP_DEVTYPE       volatile.Register32 // 0x2FCC
	FP_PIDR4         volatile.Register32 // 0x2FD0
	FP_PIDR5         volatile.Register32 // 0x2FD4
	FP_PIDR6         volatile.Register32 // 0x2FD8
	FP_PIDR7         volatile.Register32 // 0x2FDC
	FP_PIDR0         volatile.Register32 // 0x2FE0
	FP_PIDR1         volatile.Register32 // 0x2FE4
	FP_PIDR2         volatile.Register32 // 0x2FE8
	FP_PIDR3         volatile.Register32 // 0x2FEC
	FP_CIDR0         volatile.Register32 // 0x2FF0
	FP_CIDR1         volatile.Register32 // 0x2FF4
	FP_CIDR2         volatile.Register32 // 0x2FF8
	FP_CIDR3         volatile.Register32 // 0x2FFC
	_                [45060]byte
	ICTR             volatile.Register32 // 0xE004
	ACTLR            volatile.Register32 // 0xE008
	_                [4]byte
	SYST_CSR         volatile.Register32 // 0xE010
	SYST_RVR         volatile.Register32 // 0xE014
	SYST_CVR         volatile.Register32 // 0xE018
	SYST_CALIB       volatile.Register32 // 0xE01C
	_                [224]byte
	NVIC_ISER0       volatile.Register32 // 0xE100
	NVIC_ISER1       volatile.Register32 // 0xE104
	_                [120]byte
	NVIC_ICER0       volatile.Register32 // 0xE180
	NVIC_ICER1       volatile.Register32 // 0xE184
	_                [120]byte
	NVIC_ISPR0       volatile.Register32 // 0xE200
	NVIC_ISPR1       volatile.Register32 // 0xE204
	_                [120]byte
	NVIC_ICPR0       volatile.Register32 // 0xE280
	NVIC_ICPR1       volatile.Register32 // 0xE284
	_                [120]byte
	NVIC_IABR0       volatile.Register32 // 0xE300
	NVIC_IABR1       volatile.Register32 // 0xE304
	_                [120]byte
	NVIC_ITNS0       volatile.Register32 // 0xE380
	NVIC_ITNS1       volatile.Register32 // 0xE384
	_                [120]byte
	NVIC_IPR0        volatile.Register32 // 0xE400
	NVIC_IPR1        volatile.Register32 // 0xE404
	NVIC_IPR2        volatile.Register32 // 0xE408
	NVIC_IPR3        volatile.Register32 // 0xE40C
	NVIC_IPR4        volatile.Register32 // 0xE410
	NVIC_IPR5        volatile.Register32 // 0xE414
	NVIC_IPR6        volatile.Register32 // 0xE418
	NVIC_IPR7        volatile.Register32 // 0xE41C
	NVIC_IPR8        volatile.Register32 // 0xE420
	NVIC_IPR9        volatile.Register32 // 0xE424
	NVIC_IPR10       volatile.Register32 // 0xE428
	NVIC_IPR11       volatile.Register32 // 0xE42C
	NVIC_IPR12       volatile.Register32 // 0xE430
	NVIC_IPR13       volatile.Register32 // 0xE434
	NVIC_IPR14       volatile.Register32 // 0xE438
	NVIC_IPR15       volatile.Register32 // 0xE43C
	_                [2240]byte
	CPUID            volatile.Register32 // 0xED00
	ICSR             volatile.Register32 // 0xED04
	VTOR             volatile.Register32 // 0xED08
	AIRCR            volatile.Register32 // 0xED0C
	SCR              volatile.Register32 // 0xED10
	CCR              volatile.Register32 // 0xED14
	SHPR1            volatile.Register32 // 0xED18
	SHPR2            volatile.Register32 // 0xED1C
	SHPR3            volatile.Register32 // 0xED20
	SHCSR            volatile.Register32 // 0xED24
	CFSR             volatile.Register32 // 0xED28
	HFSR             volatile.Register32 // 0xED2C
	DFSR             volatile.Register32 // 0xED30
	MMFAR            volatile.Register32 // 0xED34
	BFAR             volatile.Register32 // 0xED38
	_                [4]byte
	ID_PFR0          volatile.Register32 // 0xED40
	ID_PFR1          volatile.Register32 // 0xED44
	ID_DFR0          volatile.Register32 // 0xED48
	ID_AFR0          volatile.Register32 // 0xED4C
	ID_MMFR0         volatile.Register32 // 0xED50
	ID_MMFR1         volatile.Register32 // 0xED54
	ID_MMFR2         volatile.Register32 // 0xED58
	ID_MMFR3         volatile.Register32 // 0xED5C
	ID_ISAR0         volatile.Register32 // 0xED60
	ID_ISAR1         volatile.Register32 // 0xED64
	ID_ISAR2         volatile.Register32 // 0xED68
	ID_ISAR3         volatile.Register32 // 0xED6C
	ID_ISAR4         volatile.Register32 // 0xED70
	ID_ISAR5         volatile.Register32 // 0xED74
	_                [4]byte
	CTR              volatile.Register32 // 0xED7C
	_                [8]byte
	CPACR            volatile.Register32 // 0xED88
	NSACR            volatile.Register32 // 0xED8C
	MPU_TYPE         volatile.Register32 // 0xED90
	MPU_CTRL         volatile.Register32 // 0xED94
	MPU_RNR          volatile.Register32 // 0xED98
	MPU_RBAR         volatile.Register32 // 0xED9C
	MPU_RLAR         volatile.Register32 // 0xEDA0
	MPU_RBAR_A1      volatile.Register32 // 0xEDA4
	MPU_RLAR_A1      volatile.Register32 // 0xEDA8
	MPU_RBAR_A2      volatile.Register32 // 0xEDAC
	MPU_RLAR_A2      volatile.Register32 // 0xEDB0
	MPU_RBAR_A3      volatile.Register32 // 0xEDB4
	MPU_RLAR_A3      volatile.Register32 // 0xEDB8
	_                [4]byte
	MPU_MAIR0        volatile.Register32 // 0xEDC0
	MPU_MAIR1        volatile.Register32 // 0xEDC4
	_                [8]byte
	SAU_CTRL         volatile.Register32 // 0xEDD0
	SAU_TYPE         volatile.Register32 // 0xEDD4
	SAU_RNR          volatile.Register32 // 0xEDD8
	SAU_RBAR         volatile.Register32 // 0xEDDC
	SAU_RLAR         volatile.Register32 // 0xEDE0
	SFSR             volatile.Register32 // 0xEDE4
	SFAR             volatile.Register32 // 0xEDE8
	_                [4]byte
	DHCSR            volatile.Register32 // 0xEDF0
	DCRSR            volatile.Register32 // 0xEDF4
	DCRDR            volatile.Register32 // 0xEDF8
	DEMCR            volatile.Register32 // 0xEDFC
	_                [8]byte
	DSCSR            volatile.Register32 // 0xEE08
	_                [244]byte
	STIR             volatile.Register32 // 0xEF00
	_                [48]byte
	FPCCR            volatile.Register32 // 0xEF34
	FPCAR            volatile.Register32 // 0xEF38
	FPDSCR           volatile.Register32 // 0xEF3C
	MVFR0            volatile.Register32 // 0xEF40
	MVFR1            volatile.Register32 // 0xEF44
	MVFR2            volatile.Register32 // 0xEF48
	_                [112]byte
	DDEVARCH         volatile.Register32 // 0xEFBC
	_                [12]byte
	DDEVTYPE         volatile.Register32 // 0xEFCC
	DPIDR4           volatile.Register32 // 0xEFD0
	DPIDR5           volatile.Register32 // 0xEFD4
	DPIDR6           volatile.Register32 // 0xEFD8
	DPIDR7           volatile.Register32 // 0xEFDC
	DPIDR0           volatile.Register32 // 0xEFE0
	DPIDR1           volatile.Register32 // 0xEFE4
	DPIDR2           volatile.Register32 // 0xEFE8
	DPIDR3           volatile.Register32 // 0xEFEC
	DCIDR0           volatile.Register32 // 0xEFF0
	DCIDR1           volatile.Register32 // 0xEFF4
	DCIDR2           volatile.Register32 // 0xEFF8
	DCIDR3           volatile.Register32 // 0xEFFC
	_                [204804]byte
	TRCPRGCTLR       volatile.Register32 // 0x41004
	_                [4]byte
	TRCSTATR         volatile.Register32 // 0x4100C
	TRCCONFIGR       volatile.Register32 // 0x41010
	_                [12]byte
	TRCEVENTCTL0R    volatile.Register32 // 0x41020
	TRCEVENTCTL1R    volatile.Register32 // 0x41024
	_                [4]byte
	TRCSTALLCTLR     volatile.Register32 // 0x4102C
	TRCTSCTLR        volatile.Register32 // 0x41030
	TRCSYNCPR        volatile.Register32 // 0x41034
	TRCCCCTLR        volatile.Register32 // 0x41038
	_                [68]byte
	TRCVICTLR        volatile.Register32 // 0x41080
	_                [188]byte
	TRCCNTRLDVR0     volatile.Register32 // 0x41140
	_                [60]byte
	TRCIDR8          volatile.Register32 // 0x41180
	TRCIDR9          volatile.Register32 // 0x41184
	TRCIDR10         volatile.Register32 // 0x41188
	TRCIDR11         volatile.Register32 // 0x4118C
	TRCIDR12         volatile.Register32 // 0x41190
	TRCIDR13         volatile.Register32 // 0x41194
	_                [40]byte
	TRCIMSPEC        volatile.Register32 // 0x411C0
	_                [28]byte
	TRCIDR0          volatile.Register32 // 0x411E0
	TRCIDR1          volatile.Register32 // 0x411E4
	TRCIDR2          volatile.Register32 // 0x411E8
	TRCIDR3          volatile.Register32 // 0x411EC
	TRCIDR4          volatile.Register32 // 0x411F0
	TRCIDR5          volatile.Register32 // 0x411F4
	TRCIDR6          volatile.Register32 // 0x411F8
	TRCIDR7          volatile.Register32 // 0x411FC
	_                [8]byte
	TRCRSCTLR2       volatile.Register32 // 0x41208
	TRCRSCTLR3       volatile.Register32 // 0x4120C
	_                [144]byte
	TRCSSCSR         volatile.Register32 // 0x412A0
	_                [28]byte
	TRCSSPCICR       volatile.Register32 // 0x412C0
	_                [76]byte
	TRCPDCR          volatile.Register32 // 0x41310
	TRCPDSR          volatile.Register32 // 0x41314
	_                [3020]byte
	TRCITATBIDR      volatile.Register32 // 0x41EE4
	_                [12]byte
	TRCITIATBINR     volatile.Register32 // 0x41EF4
	_                [4]byte
	TRCITIATBOUTR    volatile.Register32 // 0x41EFC
	_                [160]byte
	TRCCLAIMSET      volatile.Register32 // 0x41FA0
	TRCCLAIMCLR      volatile.Register32 // 0x41FA4
	_                [16]byte
	TRCAUTHSTATUS    volatile.Register32 // 0x41FB8
	TRCDEVARCH       volatile.Register32 // 0x41FBC
	_                [8]byte
	TRCDEVID         volatile.Register32 // 0x41FC8
	TRCDEVTYPE       volatile.Register32 // 0x41FCC
	TRCPIDR4         volatile.Register32 // 0x41FD0
	TRCPIDR5         volatile.Register32 // 0x41FD4
	TRCPIDR6         volatile.Register32 // 0x41FD8
	TRCPIDR7         volatile.Register32 // 0x41FDC
	TRCPIDR0         volatile.Register32 // 0x41FE0
	TRCPIDR1         volatile.Register32 // 0x41FE4
	TRCPIDR2         volatile.Register32 // 0x41FE8
	TRCPIDR3         volatile.Register32 // 0x41FEC
	TRCCIDR0         volatile.Register32 // 0x41FF0
	TRCCIDR1         volatile.Register32 // 0x41FF4
	TRCCIDR2         volatile.Register32 // 0x41FF8
	TRCCIDR3         volatile.Register32 // 0x41FFC
	CTICONTROL       volatile.Register32 // 0x42000
	_                [12]byte
	CTIINTACK        volatile.Register32 // 0x42010
	CTIAPPSET        volatile.Register32 // 0x42014
	CTIAPPCLEAR      volatile.Register32 // 0x42018
	CTIAPPPULSE      volatile.Register32 // 0x4201C
	CTIINEN0         volatile.Register32 // 0x42020
	CTIINEN1         volatile.Register32 // 0x42024
	CTIINEN2         volatile.Register32 // 0x42028
	CTIINEN3         volatile.Register32 // 0x4202C
	CTIINEN4         volatile.Register32 // 0x42030
	CTIINEN5         volatile.Register32 // 0x42034
	CTIINEN6         volatile.Register32 // 0x42038
	CTIINEN7         volatile.Register32 // 0x4203C
	_                [96]byte
	CTIOUTEN0        volatile.Register32 // 0x420A0
	CTIOUTEN1        volatile.Register32 // 0x420A4
	CTIOUTEN2        volatile.Register32 // 0x420A8
	CTIOUTEN3        volatile.Register32 // 0x420AC
	CTIOUTEN4        volatile.Register32 // 0x420B0
	CTIOUTEN5        volatile.Register32 // 0x420B4
	CTIOUTEN6        volatile.Register32 // 0x420B8
	CTIOUTEN7        volatile.Register32 // 0x420BC
	_                [112]byte
	CTITRIGINSTATUS  volatile.Register32 // 0x42130
	CTITRIGOUTSTATUS volatile.Register32 // 0x42134
	CTICHINSTATUS    volatile.Register32 // 0x42138
	_                [4]byte
	CTIGATE          volatile.Register32 // 0x42140
	ASICCTL          volatile.Register32 // 0x42144
	_                [3484]byte
	ITCHOUT          volatile.Register32 // 0x42EE4
	ITTRIGOUT        volatile.Register32 // 0x42EE8
	_                [8]byte
	ITCHIN           volatile.Register32 // 0x42EF4
	_                [8]byte
	ITCTRL           volatile.Register32 // 0x42F00
	_                [184]byte
	DEVARCH          volatile.Register32 // 0x42FBC
	_                [8]byte
	DEVID            volatile.Register32 // 0x42FC8
	DEVTYPE          volatile.Register32 // 0x42FCC
	PIDR4            volatile.Register32 // 0x42FD0
	PIDR5            volatile.Register32 // 0x42FD4
	PIDR6            volatile.Register32 // 0x42FD8
	PIDR7            volatile.Register32 // 0x42FDC
	PIDR0            volatile.Register32 // 0x42FE0
	PIDR1            volatile.Register32 // 0x42FE4
	PIDR2            volatile.Register32 // 0x42FE8
	PIDR3            volatile.Register32 // 0x42FEC
	CIDR0            volatile.Register32 // 0x42FF0
	CIDR1            volatile.Register32 // 0x42FF4
	CIDR2            volatile.Register32 // 0x42FF8
	CIDR3            volatile.Register32 // 0x42FFC
}

// PPB.ITM_STIM0: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM0(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM0.Reg, value)
}
func (o *PPB_Type) GetITM_STIM0() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM0.Reg)
}

// PPB.ITM_STIM1: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM1(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM1.Reg, value)
}
func (o *PPB_Type) GetITM_STIM1() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM1.Reg)
}

// PPB.ITM_STIM2: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM2(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM2.Reg, value)
}
func (o *PPB_Type) GetITM_STIM2() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM2.Reg)
}

// PPB.ITM_STIM3: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM3(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM3.Reg, value)
}
func (o *PPB_Type) GetITM_STIM3() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM3.Reg)
}

// PPB.ITM_STIM4: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM4(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM4.Reg, value)
}
func (o *PPB_Type) GetITM_STIM4() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM4.Reg)
}

// PPB.ITM_STIM5: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM5(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM5.Reg, value)
}
func (o *PPB_Type) GetITM_STIM5() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM5.Reg)
}

// PPB.ITM_STIM6: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM6(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM6.Reg, value)
}
func (o *PPB_Type) GetITM_STIM6() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM6.Reg)
}

// PPB.ITM_STIM7: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM7(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM7.Reg, value)
}
func (o *PPB_Type) GetITM_STIM7() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM7.Reg)
}

// PPB.ITM_STIM8: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM8(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM8.Reg, value)
}
func (o *PPB_Type) GetITM_STIM8() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM8.Reg)
}

// PPB.ITM_STIM9: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM9(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM9.Reg, value)
}
func (o *PPB_Type) GetITM_STIM9() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM9.Reg)
}

// PPB.ITM_STIM10: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM10(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM10.Reg, value)
}
func (o *PPB_Type) GetITM_STIM10() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM10.Reg)
}

// PPB.ITM_STIM11: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM11(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM11.Reg, value)
}
func (o *PPB_Type) GetITM_STIM11() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM11.Reg)
}

// PPB.ITM_STIM12: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM12(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM12.Reg, value)
}
func (o *PPB_Type) GetITM_STIM12() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM12.Reg)
}

// PPB.ITM_STIM13: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM13(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM13.Reg, value)
}
func (o *PPB_Type) GetITM_STIM13() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM13.Reg)
}

// PPB.ITM_STIM14: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM14(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM14.Reg, value)
}
func (o *PPB_Type) GetITM_STIM14() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM14.Reg)
}

// PPB.ITM_STIM15: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM15(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM15.Reg, value)
}
func (o *PPB_Type) GetITM_STIM15() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM15.Reg)
}

// PPB.ITM_STIM16: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM16(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM16.Reg, value)
}
func (o *PPB_Type) GetITM_STIM16() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM16.Reg)
}

// PPB.ITM_STIM17: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM17(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM17.Reg, value)
}
func (o *PPB_Type) GetITM_STIM17() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM17.Reg)
}

// PPB.ITM_STIM18: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM18(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM18.Reg, value)
}
func (o *PPB_Type) GetITM_STIM18() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM18.Reg)
}

// PPB.ITM_STIM19: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM19(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM19.Reg, value)
}
func (o *PPB_Type) GetITM_STIM19() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM19.Reg)
}

// PPB.ITM_STIM20: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM20(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM20.Reg, value)
}
func (o *PPB_Type) GetITM_STIM20() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM20.Reg)
}

// PPB.ITM_STIM21: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM21(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM21.Reg, value)
}
func (o *PPB_Type) GetITM_STIM21() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM21.Reg)
}

// PPB.ITM_STIM22: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM22(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM22.Reg, value)
}
func (o *PPB_Type) GetITM_STIM22() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM22.Reg)
}

// PPB.ITM_STIM23: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM23(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM23.Reg, value)
}
func (o *PPB_Type) GetITM_STIM23() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM23.Reg)
}

// PPB.ITM_STIM24: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM24(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM24.Reg, value)
}
func (o *PPB_Type) GetITM_STIM24() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM24.Reg)
}

// PPB.ITM_STIM25: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM25(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM25.Reg, value)
}
func (o *PPB_Type) GetITM_STIM25() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM25.Reg)
}

// PPB.ITM_STIM26: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM26(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM26.Reg, value)
}
func (o *PPB_Type) GetITM_STIM26() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM26.Reg)
}

// PPB.ITM_STIM27: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM27(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM27.Reg, value)
}
func (o *PPB_Type) GetITM_STIM27() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM27.Reg)
}

// PPB.ITM_STIM28: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM28(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM28.Reg, value)
}
func (o *PPB_Type) GetITM_STIM28() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM28.Reg)
}

// PPB.ITM_STIM29: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM29(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM29.Reg, value)
}
func (o *PPB_Type) GetITM_STIM29() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM29.Reg)
}

// PPB.ITM_STIM30: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM30(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM30.Reg, value)
}
func (o *PPB_Type) GetITM_STIM30() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM30.Reg)
}

// PPB.ITM_STIM31: Provides the interface for generating Instrumentation packets
func (o *PPB_Type) SetITM_STIM31(value uint32) {
	volatile.StoreUint32(&o.ITM_STIM31.Reg, value)
}
func (o *PPB_Type) GetITM_STIM31() uint32 {
	return volatile.LoadUint32(&o.ITM_STIM31.Reg)
}

// PPB.ITM_TER0: Provide an individual enable bit for each ITM_STIM register
func (o *PPB_Type) SetITM_TER0(value uint32) {
	volatile.StoreUint32(&o.ITM_TER0.Reg, value)
}
func (o *PPB_Type) GetITM_TER0() uint32 {
	return volatile.LoadUint32(&o.ITM_TER0.Reg)
}

// PPB.ITM_TPR: Controls which stimulus ports can be accessed by unprivileged code
func (o *PPB_Type) SetITM_TPR_PRIVMASK(value uint32) {
	volatile.StoreUint32(&o.ITM_TPR.Reg, volatile.LoadUint32(&o.ITM_TPR.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetITM_TPR_PRIVMASK() uint32 {
	return volatile.LoadUint32(&o.ITM_TPR.Reg) & 0xf
}

// PPB.ITM_TCR: Configures and controls transfers through the ITM interface
func (o *PPB_Type) SetITM_TCR_BUSY(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0x800000)|value<<23)
}
func (o *PPB_Type) GetITM_TCR_BUSY() uint32 {
	return (volatile.LoadUint32(&o.ITM_TCR.Reg) & 0x800000) >> 23
}
func (o *PPB_Type) SetITM_TCR_TRACEBUSID(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0x7f0000)|value<<16)
}
func (o *PPB_Type) GetITM_TCR_TRACEBUSID() uint32 {
	return (volatile.LoadUint32(&o.ITM_TCR.Reg) & 0x7f0000) >> 16
}
func (o *PPB_Type) SetITM_TCR_GTSFREQ(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0xc00)|value<<10)
}
func (o *PPB_Type) GetITM_TCR_GTSFREQ() uint32 {
	return (volatile.LoadUint32(&o.ITM_TCR.Reg) & 0xc00) >> 10
}
func (o *PPB_Type) SetITM_TCR_TSPRESCALE(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0x300)|value<<8)
}
func (o *PPB_Type) GetITM_TCR_TSPRESCALE() uint32 {
	return (volatile.LoadUint32(&o.ITM_TCR.Reg) & 0x300) >> 8
}
func (o *PPB_Type) SetITM_TCR_STALLENA(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0x20)|value<<5)
}
func (o *PPB_Type) GetITM_TCR_STALLENA() uint32 {
	return (volatile.LoadUint32(&o.ITM_TCR.Reg) & 0x20) >> 5
}
func (o *PPB_Type) SetITM_TCR_SWOENA(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetITM_TCR_SWOENA() uint32 {
	return (volatile.LoadUint32(&o.ITM_TCR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetITM_TCR_TXENA(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetITM_TCR_TXENA() uint32 {
	return (volatile.LoadUint32(&o.ITM_TCR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetITM_TCR_SYNCENA(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetITM_TCR_SYNCENA() uint32 {
	return (volatile.LoadUint32(&o.ITM_TCR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetITM_TCR_TSENA(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetITM_TCR_TSENA() uint32 {
	return (volatile.LoadUint32(&o.ITM_TCR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetITM_TCR_ITMENA(value uint32) {
	volatile.StoreUint32(&o.ITM_TCR.Reg, volatile.LoadUint32(&o.ITM_TCR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetITM_TCR_ITMENA() uint32 {
	return volatile.LoadUint32(&o.ITM_TCR.Reg) & 0x1
}

// PPB.INT_ATREADY: Integration Mode: Read ATB Ready
func (o *PPB_Type) SetINT_ATREADY_AFVALID(value uint32) {
	volatile.StoreUint32(&o.INT_ATREADY.Reg, volatile.LoadUint32(&o.INT_ATREADY.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetINT_ATREADY_AFVALID() uint32 {
	return (volatile.LoadUint32(&o.INT_ATREADY.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetINT_ATREADY_ATREADY(value uint32) {
	volatile.StoreUint32(&o.INT_ATREADY.Reg, volatile.LoadUint32(&o.INT_ATREADY.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetINT_ATREADY_ATREADY() uint32 {
	return volatile.LoadUint32(&o.INT_ATREADY.Reg) & 0x1
}

// PPB.INT_ATVALID: Integration Mode: Write ATB Valid
func (o *PPB_Type) SetINT_ATVALID_AFREADY(value uint32) {
	volatile.StoreUint32(&o.INT_ATVALID.Reg, volatile.LoadUint32(&o.INT_ATVALID.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetINT_ATVALID_AFREADY() uint32 {
	return (volatile.LoadUint32(&o.INT_ATVALID.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetINT_ATVALID_ATREADY(value uint32) {
	volatile.StoreUint32(&o.INT_ATVALID.Reg, volatile.LoadUint32(&o.INT_ATVALID.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetINT_ATVALID_ATREADY() uint32 {
	return volatile.LoadUint32(&o.INT_ATVALID.Reg) & 0x1
}

// PPB.ITM_ITCTRL: Integration Mode Control Register
func (o *PPB_Type) SetITM_ITCTRL_IME(value uint32) {
	volatile.StoreUint32(&o.ITM_ITCTRL.Reg, volatile.LoadUint32(&o.ITM_ITCTRL.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetITM_ITCTRL_IME() uint32 {
	return volatile.LoadUint32(&o.ITM_ITCTRL.Reg) & 0x1
}

// PPB.ITM_DEVARCH: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_DEVARCH_ARCHITECT(value uint32) {
	volatile.StoreUint32(&o.ITM_DEVARCH.Reg, volatile.LoadUint32(&o.ITM_DEVARCH.Reg)&^(0xffe00000)|value<<21)
}
func (o *PPB_Type) GetITM_DEVARCH_ARCHITECT() uint32 {
	return (volatile.LoadUint32(&o.ITM_DEVARCH.Reg) & 0xffe00000) >> 21
}
func (o *PPB_Type) SetITM_DEVARCH_PRESENT(value uint32) {
	volatile.StoreUint32(&o.ITM_DEVARCH.Reg, volatile.LoadUint32(&o.ITM_DEVARCH.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetITM_DEVARCH_PRESENT() uint32 {
	return (volatile.LoadUint32(&o.ITM_DEVARCH.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetITM_DEVARCH_REVISION(value uint32) {
	volatile.StoreUint32(&o.ITM_DEVARCH.Reg, volatile.LoadUint32(&o.ITM_DEVARCH.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetITM_DEVARCH_REVISION() uint32 {
	return (volatile.LoadUint32(&o.ITM_DEVARCH.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetITM_DEVARCH_ARCHVER(value uint32) {
	volatile.StoreUint32(&o.ITM_DEVARCH.Reg, volatile.LoadUint32(&o.ITM_DEVARCH.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetITM_DEVARCH_ARCHVER() uint32 {
	return (volatile.LoadUint32(&o.ITM_DEVARCH.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetITM_DEVARCH_ARCHPART(value uint32) {
	volatile.StoreUint32(&o.ITM_DEVARCH.Reg, volatile.LoadUint32(&o.ITM_DEVARCH.Reg)&^(0xfff)|value)
}
func (o *PPB_Type) GetITM_DEVARCH_ARCHPART() uint32 {
	return volatile.LoadUint32(&o.ITM_DEVARCH.Reg) & 0xfff
}

// PPB.ITM_DEVTYPE: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_DEVTYPE_SUB(value uint32) {
	volatile.StoreUint32(&o.ITM_DEVTYPE.Reg, volatile.LoadUint32(&o.ITM_DEVTYPE.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetITM_DEVTYPE_SUB() uint32 {
	return (volatile.LoadUint32(&o.ITM_DEVTYPE.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetITM_DEVTYPE_MAJOR(value uint32) {
	volatile.StoreUint32(&o.ITM_DEVTYPE.Reg, volatile.LoadUint32(&o.ITM_DEVTYPE.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetITM_DEVTYPE_MAJOR() uint32 {
	return volatile.LoadUint32(&o.ITM_DEVTYPE.Reg) & 0xf
}

// PPB.ITM_PIDR4: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_PIDR4_SIZE(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR4.Reg, volatile.LoadUint32(&o.ITM_PIDR4.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetITM_PIDR4_SIZE() uint32 {
	return (volatile.LoadUint32(&o.ITM_PIDR4.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetITM_PIDR4_DES_2(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR4.Reg, volatile.LoadUint32(&o.ITM_PIDR4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetITM_PIDR4_DES_2() uint32 {
	return volatile.LoadUint32(&o.ITM_PIDR4.Reg) & 0xf
}

// PPB.ITM_PIDR5: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_PIDR5(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR5.Reg, value)
}
func (o *PPB_Type) GetITM_PIDR5() uint32 {
	return volatile.LoadUint32(&o.ITM_PIDR5.Reg)
}

// PPB.ITM_PIDR6: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_PIDR6(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR6.Reg, value)
}
func (o *PPB_Type) GetITM_PIDR6() uint32 {
	return volatile.LoadUint32(&o.ITM_PIDR6.Reg)
}

// PPB.ITM_PIDR7: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_PIDR7(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR7.Reg, value)
}
func (o *PPB_Type) GetITM_PIDR7() uint32 {
	return volatile.LoadUint32(&o.ITM_PIDR7.Reg)
}

// PPB.ITM_PIDR0: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_PIDR0_PART_0(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR0.Reg, volatile.LoadUint32(&o.ITM_PIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetITM_PIDR0_PART_0() uint32 {
	return volatile.LoadUint32(&o.ITM_PIDR0.Reg) & 0xff
}

// PPB.ITM_PIDR1: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_PIDR1_DES_0(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR1.Reg, volatile.LoadUint32(&o.ITM_PIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetITM_PIDR1_DES_0() uint32 {
	return (volatile.LoadUint32(&o.ITM_PIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetITM_PIDR1_PART_1(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR1.Reg, volatile.LoadUint32(&o.ITM_PIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetITM_PIDR1_PART_1() uint32 {
	return volatile.LoadUint32(&o.ITM_PIDR1.Reg) & 0xf
}

// PPB.ITM_PIDR2: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_PIDR2_REVISION(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR2.Reg, volatile.LoadUint32(&o.ITM_PIDR2.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetITM_PIDR2_REVISION() uint32 {
	return (volatile.LoadUint32(&o.ITM_PIDR2.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetITM_PIDR2_JEDEC(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR2.Reg, volatile.LoadUint32(&o.ITM_PIDR2.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetITM_PIDR2_JEDEC() uint32 {
	return (volatile.LoadUint32(&o.ITM_PIDR2.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetITM_PIDR2_DES_1(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR2.Reg, volatile.LoadUint32(&o.ITM_PIDR2.Reg)&^(0x7)|value)
}
func (o *PPB_Type) GetITM_PIDR2_DES_1() uint32 {
	return volatile.LoadUint32(&o.ITM_PIDR2.Reg) & 0x7
}

// PPB.ITM_PIDR3: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_PIDR3_REVAND(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR3.Reg, volatile.LoadUint32(&o.ITM_PIDR3.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetITM_PIDR3_REVAND() uint32 {
	return (volatile.LoadUint32(&o.ITM_PIDR3.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetITM_PIDR3_CMOD(value uint32) {
	volatile.StoreUint32(&o.ITM_PIDR3.Reg, volatile.LoadUint32(&o.ITM_PIDR3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetITM_PIDR3_CMOD() uint32 {
	return volatile.LoadUint32(&o.ITM_PIDR3.Reg) & 0xf
}

// PPB.ITM_CIDR0: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_CIDR0_PRMBL_0(value uint32) {
	volatile.StoreUint32(&o.ITM_CIDR0.Reg, volatile.LoadUint32(&o.ITM_CIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetITM_CIDR0_PRMBL_0() uint32 {
	return volatile.LoadUint32(&o.ITM_CIDR0.Reg) & 0xff
}

// PPB.ITM_CIDR1: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_CIDR1_CLASS(value uint32) {
	volatile.StoreUint32(&o.ITM_CIDR1.Reg, volatile.LoadUint32(&o.ITM_CIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetITM_CIDR1_CLASS() uint32 {
	return (volatile.LoadUint32(&o.ITM_CIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetITM_CIDR1_PRMBL_1(value uint32) {
	volatile.StoreUint32(&o.ITM_CIDR1.Reg, volatile.LoadUint32(&o.ITM_CIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetITM_CIDR1_PRMBL_1() uint32 {
	return volatile.LoadUint32(&o.ITM_CIDR1.Reg) & 0xf
}

// PPB.ITM_CIDR2: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_CIDR2_PRMBL_2(value uint32) {
	volatile.StoreUint32(&o.ITM_CIDR2.Reg, volatile.LoadUint32(&o.ITM_CIDR2.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetITM_CIDR2_PRMBL_2() uint32 {
	return volatile.LoadUint32(&o.ITM_CIDR2.Reg) & 0xff
}

// PPB.ITM_CIDR3: Provides CoreSight discovery information for the ITM
func (o *PPB_Type) SetITM_CIDR3_PRMBL_3(value uint32) {
	volatile.StoreUint32(&o.ITM_CIDR3.Reg, volatile.LoadUint32(&o.ITM_CIDR3.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetITM_CIDR3_PRMBL_3() uint32 {
	return volatile.LoadUint32(&o.ITM_CIDR3.Reg) & 0xff
}

// PPB.DWT_CTRL: Provides configuration and status information for the DWT unit, and used to control features of the unit
func (o *PPB_Type) SetDWT_CTRL_NUMCOMP(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetDWT_CTRL_NUMCOMP() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetDWT_CTRL_NOTRCPKT(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x8000000)|value<<27)
}
func (o *PPB_Type) GetDWT_CTRL_NOTRCPKT() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x8000000) >> 27
}
func (o *PPB_Type) SetDWT_CTRL_NOEXTTRIG(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x4000000)|value<<26)
}
func (o *PPB_Type) GetDWT_CTRL_NOEXTTRIG() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x4000000) >> 26
}
func (o *PPB_Type) SetDWT_CTRL_NOCYCCNT(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x2000000)|value<<25)
}
func (o *PPB_Type) GetDWT_CTRL_NOCYCCNT() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x2000000) >> 25
}
func (o *PPB_Type) SetDWT_CTRL_NOPRFCNT(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetDWT_CTRL_NOPRFCNT() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetDWT_CTRL_CYCDISS(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x800000)|value<<23)
}
func (o *PPB_Type) GetDWT_CTRL_CYCDISS() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x800000) >> 23
}
func (o *PPB_Type) SetDWT_CTRL_CYCEVTENA(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x400000)|value<<22)
}
func (o *PPB_Type) GetDWT_CTRL_CYCEVTENA() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x400000) >> 22
}
func (o *PPB_Type) SetDWT_CTRL_FOLDEVTENA(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x200000)|value<<21)
}
func (o *PPB_Type) GetDWT_CTRL_FOLDEVTENA() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x200000) >> 21
}
func (o *PPB_Type) SetDWT_CTRL_LSUEVTENA(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetDWT_CTRL_LSUEVTENA() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetDWT_CTRL_SLEEPEVTENA(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x80000)|value<<19)
}
func (o *PPB_Type) GetDWT_CTRL_SLEEPEVTENA() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x80000) >> 19
}
func (o *PPB_Type) SetDWT_CTRL_EXCEVTENA(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x40000)|value<<18)
}
func (o *PPB_Type) GetDWT_CTRL_EXCEVTENA() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x40000) >> 18
}
func (o *PPB_Type) SetDWT_CTRL_CPIEVTENA(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x20000)|value<<17)
}
func (o *PPB_Type) GetDWT_CTRL_CPIEVTENA() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x20000) >> 17
}
func (o *PPB_Type) SetDWT_CTRL_EXTTRCENA(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetDWT_CTRL_EXTTRCENA() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetDWT_CTRL_PCSAMPLENA(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x1000)|value<<12)
}
func (o *PPB_Type) GetDWT_CTRL_PCSAMPLENA() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x1000) >> 12
}
func (o *PPB_Type) SetDWT_CTRL_SYNCTAP(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0xc00)|value<<10)
}
func (o *PPB_Type) GetDWT_CTRL_SYNCTAP() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0xc00) >> 10
}
func (o *PPB_Type) SetDWT_CTRL_CYCTAP(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x200)|value<<9)
}
func (o *PPB_Type) GetDWT_CTRL_CYCTAP() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x200) >> 9
}
func (o *PPB_Type) SetDWT_CTRL_POSTINIT(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x1e0)|value<<5)
}
func (o *PPB_Type) GetDWT_CTRL_POSTINIT() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x1e0) >> 5
}
func (o *PPB_Type) SetDWT_CTRL_POSTPRESET(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x1e)|value<<1)
}
func (o *PPB_Type) GetDWT_CTRL_POSTPRESET() uint32 {
	return (volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x1e) >> 1
}
func (o *PPB_Type) SetDWT_CTRL_CYCCNTENA(value uint32) {
	volatile.StoreUint32(&o.DWT_CTRL.Reg, volatile.LoadUint32(&o.DWT_CTRL.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetDWT_CTRL_CYCCNTENA() uint32 {
	return volatile.LoadUint32(&o.DWT_CTRL.Reg) & 0x1
}

// PPB.DWT_CYCCNT: Shows or sets the value of the processor cycle counter, CYCCNT
func (o *PPB_Type) SetDWT_CYCCNT(value uint32) {
	volatile.StoreUint32(&o.DWT_CYCCNT.Reg, value)
}
func (o *PPB_Type) GetDWT_CYCCNT() uint32 {
	return volatile.LoadUint32(&o.DWT_CYCCNT.Reg)
}

// PPB.DWT_EXCCNT: Counts the total cycles spent in exception processing
func (o *PPB_Type) SetDWT_EXCCNT_EXCCNT(value uint32) {
	volatile.StoreUint32(&o.DWT_EXCCNT.Reg, volatile.LoadUint32(&o.DWT_EXCCNT.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDWT_EXCCNT_EXCCNT() uint32 {
	return volatile.LoadUint32(&o.DWT_EXCCNT.Reg) & 0xff
}

// PPB.DWT_LSUCNT: Increments on the additional cycles required to execute all load or store instructions
func (o *PPB_Type) SetDWT_LSUCNT_LSUCNT(value uint32) {
	volatile.StoreUint32(&o.DWT_LSUCNT.Reg, volatile.LoadUint32(&o.DWT_LSUCNT.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDWT_LSUCNT_LSUCNT() uint32 {
	return volatile.LoadUint32(&o.DWT_LSUCNT.Reg) & 0xff
}

// PPB.DWT_FOLDCNT: Increments on the additional cycles required to execute all load or store instructions
func (o *PPB_Type) SetDWT_FOLDCNT_FOLDCNT(value uint32) {
	volatile.StoreUint32(&o.DWT_FOLDCNT.Reg, volatile.LoadUint32(&o.DWT_FOLDCNT.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDWT_FOLDCNT_FOLDCNT() uint32 {
	return volatile.LoadUint32(&o.DWT_FOLDCNT.Reg) & 0xff
}

// PPB.DWT_COMP0: Provides a reference value for use by watchpoint comparator 0
func (o *PPB_Type) SetDWT_COMP0(value uint32) {
	volatile.StoreUint32(&o.DWT_COMP0.Reg, value)
}
func (o *PPB_Type) GetDWT_COMP0() uint32 {
	return volatile.LoadUint32(&o.DWT_COMP0.Reg)
}

// PPB.DWT_FUNCTION0: Controls the operation of watchpoint comparator 0
func (o *PPB_Type) SetDWT_FUNCTION0_ID(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION0.Reg, volatile.LoadUint32(&o.DWT_FUNCTION0.Reg)&^(0xf8000000)|value<<27)
}
func (o *PPB_Type) GetDWT_FUNCTION0_ID() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION0.Reg) & 0xf8000000) >> 27
}
func (o *PPB_Type) SetDWT_FUNCTION0_MATCHED(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION0.Reg, volatile.LoadUint32(&o.DWT_FUNCTION0.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetDWT_FUNCTION0_MATCHED() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION0.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetDWT_FUNCTION0_DATAVSIZE(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION0.Reg, volatile.LoadUint32(&o.DWT_FUNCTION0.Reg)&^(0xc00)|value<<10)
}
func (o *PPB_Type) GetDWT_FUNCTION0_DATAVSIZE() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION0.Reg) & 0xc00) >> 10
}
func (o *PPB_Type) SetDWT_FUNCTION0_ACTION(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION0.Reg, volatile.LoadUint32(&o.DWT_FUNCTION0.Reg)&^(0x30)|value<<4)
}
func (o *PPB_Type) GetDWT_FUNCTION0_ACTION() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION0.Reg) & 0x30) >> 4
}
func (o *PPB_Type) SetDWT_FUNCTION0_MATCH(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION0.Reg, volatile.LoadUint32(&o.DWT_FUNCTION0.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDWT_FUNCTION0_MATCH() uint32 {
	return volatile.LoadUint32(&o.DWT_FUNCTION0.Reg) & 0xf
}

// PPB.DWT_COMP1: Provides a reference value for use by watchpoint comparator 1
func (o *PPB_Type) SetDWT_COMP1(value uint32) {
	volatile.StoreUint32(&o.DWT_COMP1.Reg, value)
}
func (o *PPB_Type) GetDWT_COMP1() uint32 {
	return volatile.LoadUint32(&o.DWT_COMP1.Reg)
}

// PPB.DWT_FUNCTION1: Controls the operation of watchpoint comparator 1
func (o *PPB_Type) SetDWT_FUNCTION1_ID(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION1.Reg, volatile.LoadUint32(&o.DWT_FUNCTION1.Reg)&^(0xf8000000)|value<<27)
}
func (o *PPB_Type) GetDWT_FUNCTION1_ID() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION1.Reg) & 0xf8000000) >> 27
}
func (o *PPB_Type) SetDWT_FUNCTION1_MATCHED(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION1.Reg, volatile.LoadUint32(&o.DWT_FUNCTION1.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetDWT_FUNCTION1_MATCHED() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION1.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetDWT_FUNCTION1_DATAVSIZE(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION1.Reg, volatile.LoadUint32(&o.DWT_FUNCTION1.Reg)&^(0xc00)|value<<10)
}
func (o *PPB_Type) GetDWT_FUNCTION1_DATAVSIZE() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION1.Reg) & 0xc00) >> 10
}
func (o *PPB_Type) SetDWT_FUNCTION1_ACTION(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION1.Reg, volatile.LoadUint32(&o.DWT_FUNCTION1.Reg)&^(0x30)|value<<4)
}
func (o *PPB_Type) GetDWT_FUNCTION1_ACTION() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION1.Reg) & 0x30) >> 4
}
func (o *PPB_Type) SetDWT_FUNCTION1_MATCH(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION1.Reg, volatile.LoadUint32(&o.DWT_FUNCTION1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDWT_FUNCTION1_MATCH() uint32 {
	return volatile.LoadUint32(&o.DWT_FUNCTION1.Reg) & 0xf
}

// PPB.DWT_COMP2: Provides a reference value for use by watchpoint comparator 2
func (o *PPB_Type) SetDWT_COMP2(value uint32) {
	volatile.StoreUint32(&o.DWT_COMP2.Reg, value)
}
func (o *PPB_Type) GetDWT_COMP2() uint32 {
	return volatile.LoadUint32(&o.DWT_COMP2.Reg)
}

// PPB.DWT_FUNCTION2: Controls the operation of watchpoint comparator 2
func (o *PPB_Type) SetDWT_FUNCTION2_ID(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION2.Reg, volatile.LoadUint32(&o.DWT_FUNCTION2.Reg)&^(0xf8000000)|value<<27)
}
func (o *PPB_Type) GetDWT_FUNCTION2_ID() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION2.Reg) & 0xf8000000) >> 27
}
func (o *PPB_Type) SetDWT_FUNCTION2_MATCHED(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION2.Reg, volatile.LoadUint32(&o.DWT_FUNCTION2.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetDWT_FUNCTION2_MATCHED() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION2.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetDWT_FUNCTION2_DATAVSIZE(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION2.Reg, volatile.LoadUint32(&o.DWT_FUNCTION2.Reg)&^(0xc00)|value<<10)
}
func (o *PPB_Type) GetDWT_FUNCTION2_DATAVSIZE() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION2.Reg) & 0xc00) >> 10
}
func (o *PPB_Type) SetDWT_FUNCTION2_ACTION(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION2.Reg, volatile.LoadUint32(&o.DWT_FUNCTION2.Reg)&^(0x30)|value<<4)
}
func (o *PPB_Type) GetDWT_FUNCTION2_ACTION() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION2.Reg) & 0x30) >> 4
}
func (o *PPB_Type) SetDWT_FUNCTION2_MATCH(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION2.Reg, volatile.LoadUint32(&o.DWT_FUNCTION2.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDWT_FUNCTION2_MATCH() uint32 {
	return volatile.LoadUint32(&o.DWT_FUNCTION2.Reg) & 0xf
}

// PPB.DWT_COMP3: Provides a reference value for use by watchpoint comparator 3
func (o *PPB_Type) SetDWT_COMP3(value uint32) {
	volatile.StoreUint32(&o.DWT_COMP3.Reg, value)
}
func (o *PPB_Type) GetDWT_COMP3() uint32 {
	return volatile.LoadUint32(&o.DWT_COMP3.Reg)
}

// PPB.DWT_FUNCTION3: Controls the operation of watchpoint comparator 3
func (o *PPB_Type) SetDWT_FUNCTION3_ID(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION3.Reg, volatile.LoadUint32(&o.DWT_FUNCTION3.Reg)&^(0xf8000000)|value<<27)
}
func (o *PPB_Type) GetDWT_FUNCTION3_ID() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION3.Reg) & 0xf8000000) >> 27
}
func (o *PPB_Type) SetDWT_FUNCTION3_MATCHED(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION3.Reg, volatile.LoadUint32(&o.DWT_FUNCTION3.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetDWT_FUNCTION3_MATCHED() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION3.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetDWT_FUNCTION3_DATAVSIZE(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION3.Reg, volatile.LoadUint32(&o.DWT_FUNCTION3.Reg)&^(0xc00)|value<<10)
}
func (o *PPB_Type) GetDWT_FUNCTION3_DATAVSIZE() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION3.Reg) & 0xc00) >> 10
}
func (o *PPB_Type) SetDWT_FUNCTION3_ACTION(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION3.Reg, volatile.LoadUint32(&o.DWT_FUNCTION3.Reg)&^(0x30)|value<<4)
}
func (o *PPB_Type) GetDWT_FUNCTION3_ACTION() uint32 {
	return (volatile.LoadUint32(&o.DWT_FUNCTION3.Reg) & 0x30) >> 4
}
func (o *PPB_Type) SetDWT_FUNCTION3_MATCH(value uint32) {
	volatile.StoreUint32(&o.DWT_FUNCTION3.Reg, volatile.LoadUint32(&o.DWT_FUNCTION3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDWT_FUNCTION3_MATCH() uint32 {
	return volatile.LoadUint32(&o.DWT_FUNCTION3.Reg) & 0xf
}

// PPB.DWT_DEVARCH: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_DEVARCH_ARCHITECT(value uint32) {
	volatile.StoreUint32(&o.DWT_DEVARCH.Reg, volatile.LoadUint32(&o.DWT_DEVARCH.Reg)&^(0xffe00000)|value<<21)
}
func (o *PPB_Type) GetDWT_DEVARCH_ARCHITECT() uint32 {
	return (volatile.LoadUint32(&o.DWT_DEVARCH.Reg) & 0xffe00000) >> 21
}
func (o *PPB_Type) SetDWT_DEVARCH_PRESENT(value uint32) {
	volatile.StoreUint32(&o.DWT_DEVARCH.Reg, volatile.LoadUint32(&o.DWT_DEVARCH.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetDWT_DEVARCH_PRESENT() uint32 {
	return (volatile.LoadUint32(&o.DWT_DEVARCH.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetDWT_DEVARCH_REVISION(value uint32) {
	volatile.StoreUint32(&o.DWT_DEVARCH.Reg, volatile.LoadUint32(&o.DWT_DEVARCH.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetDWT_DEVARCH_REVISION() uint32 {
	return (volatile.LoadUint32(&o.DWT_DEVARCH.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetDWT_DEVARCH_ARCHVER(value uint32) {
	volatile.StoreUint32(&o.DWT_DEVARCH.Reg, volatile.LoadUint32(&o.DWT_DEVARCH.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetDWT_DEVARCH_ARCHVER() uint32 {
	return (volatile.LoadUint32(&o.DWT_DEVARCH.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetDWT_DEVARCH_ARCHPART(value uint32) {
	volatile.StoreUint32(&o.DWT_DEVARCH.Reg, volatile.LoadUint32(&o.DWT_DEVARCH.Reg)&^(0xfff)|value)
}
func (o *PPB_Type) GetDWT_DEVARCH_ARCHPART() uint32 {
	return volatile.LoadUint32(&o.DWT_DEVARCH.Reg) & 0xfff
}

// PPB.DWT_DEVTYPE: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_DEVTYPE_SUB(value uint32) {
	volatile.StoreUint32(&o.DWT_DEVTYPE.Reg, volatile.LoadUint32(&o.DWT_DEVTYPE.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDWT_DEVTYPE_SUB() uint32 {
	return (volatile.LoadUint32(&o.DWT_DEVTYPE.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDWT_DEVTYPE_MAJOR(value uint32) {
	volatile.StoreUint32(&o.DWT_DEVTYPE.Reg, volatile.LoadUint32(&o.DWT_DEVTYPE.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDWT_DEVTYPE_MAJOR() uint32 {
	return volatile.LoadUint32(&o.DWT_DEVTYPE.Reg) & 0xf
}

// PPB.DWT_PIDR4: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_PIDR4_SIZE(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR4.Reg, volatile.LoadUint32(&o.DWT_PIDR4.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDWT_PIDR4_SIZE() uint32 {
	return (volatile.LoadUint32(&o.DWT_PIDR4.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDWT_PIDR4_DES_2(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR4.Reg, volatile.LoadUint32(&o.DWT_PIDR4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDWT_PIDR4_DES_2() uint32 {
	return volatile.LoadUint32(&o.DWT_PIDR4.Reg) & 0xf
}

// PPB.DWT_PIDR5: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_PIDR5(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR5.Reg, value)
}
func (o *PPB_Type) GetDWT_PIDR5() uint32 {
	return volatile.LoadUint32(&o.DWT_PIDR5.Reg)
}

// PPB.DWT_PIDR6: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_PIDR6(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR6.Reg, value)
}
func (o *PPB_Type) GetDWT_PIDR6() uint32 {
	return volatile.LoadUint32(&o.DWT_PIDR6.Reg)
}

// PPB.DWT_PIDR7: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_PIDR7(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR7.Reg, value)
}
func (o *PPB_Type) GetDWT_PIDR7() uint32 {
	return volatile.LoadUint32(&o.DWT_PIDR7.Reg)
}

// PPB.DWT_PIDR0: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_PIDR0_PART_0(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR0.Reg, volatile.LoadUint32(&o.DWT_PIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDWT_PIDR0_PART_0() uint32 {
	return volatile.LoadUint32(&o.DWT_PIDR0.Reg) & 0xff
}

// PPB.DWT_PIDR1: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_PIDR1_DES_0(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR1.Reg, volatile.LoadUint32(&o.DWT_PIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDWT_PIDR1_DES_0() uint32 {
	return (volatile.LoadUint32(&o.DWT_PIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDWT_PIDR1_PART_1(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR1.Reg, volatile.LoadUint32(&o.DWT_PIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDWT_PIDR1_PART_1() uint32 {
	return volatile.LoadUint32(&o.DWT_PIDR1.Reg) & 0xf
}

// PPB.DWT_PIDR2: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_PIDR2_REVISION(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR2.Reg, volatile.LoadUint32(&o.DWT_PIDR2.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDWT_PIDR2_REVISION() uint32 {
	return (volatile.LoadUint32(&o.DWT_PIDR2.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDWT_PIDR2_JEDEC(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR2.Reg, volatile.LoadUint32(&o.DWT_PIDR2.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetDWT_PIDR2_JEDEC() uint32 {
	return (volatile.LoadUint32(&o.DWT_PIDR2.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetDWT_PIDR2_DES_1(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR2.Reg, volatile.LoadUint32(&o.DWT_PIDR2.Reg)&^(0x7)|value)
}
func (o *PPB_Type) GetDWT_PIDR2_DES_1() uint32 {
	return volatile.LoadUint32(&o.DWT_PIDR2.Reg) & 0x7
}

// PPB.DWT_PIDR3: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_PIDR3_REVAND(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR3.Reg, volatile.LoadUint32(&o.DWT_PIDR3.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDWT_PIDR3_REVAND() uint32 {
	return (volatile.LoadUint32(&o.DWT_PIDR3.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDWT_PIDR3_CMOD(value uint32) {
	volatile.StoreUint32(&o.DWT_PIDR3.Reg, volatile.LoadUint32(&o.DWT_PIDR3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDWT_PIDR3_CMOD() uint32 {
	return volatile.LoadUint32(&o.DWT_PIDR3.Reg) & 0xf
}

// PPB.DWT_CIDR0: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_CIDR0_PRMBL_0(value uint32) {
	volatile.StoreUint32(&o.DWT_CIDR0.Reg, volatile.LoadUint32(&o.DWT_CIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDWT_CIDR0_PRMBL_0() uint32 {
	return volatile.LoadUint32(&o.DWT_CIDR0.Reg) & 0xff
}

// PPB.DWT_CIDR1: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_CIDR1_CLASS(value uint32) {
	volatile.StoreUint32(&o.DWT_CIDR1.Reg, volatile.LoadUint32(&o.DWT_CIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDWT_CIDR1_CLASS() uint32 {
	return (volatile.LoadUint32(&o.DWT_CIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDWT_CIDR1_PRMBL_1(value uint32) {
	volatile.StoreUint32(&o.DWT_CIDR1.Reg, volatile.LoadUint32(&o.DWT_CIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDWT_CIDR1_PRMBL_1() uint32 {
	return volatile.LoadUint32(&o.DWT_CIDR1.Reg) & 0xf
}

// PPB.DWT_CIDR2: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_CIDR2_PRMBL_2(value uint32) {
	volatile.StoreUint32(&o.DWT_CIDR2.Reg, volatile.LoadUint32(&o.DWT_CIDR2.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDWT_CIDR2_PRMBL_2() uint32 {
	return volatile.LoadUint32(&o.DWT_CIDR2.Reg) & 0xff
}

// PPB.DWT_CIDR3: Provides CoreSight discovery information for the DWT
func (o *PPB_Type) SetDWT_CIDR3_PRMBL_3(value uint32) {
	volatile.StoreUint32(&o.DWT_CIDR3.Reg, volatile.LoadUint32(&o.DWT_CIDR3.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDWT_CIDR3_PRMBL_3() uint32 {
	return volatile.LoadUint32(&o.DWT_CIDR3.Reg) & 0xff
}

// PPB.FP_CTRL: Provides FPB implementation information, and the global enable for the FPB unit
func (o *PPB_Type) SetFP_CTRL_REV(value uint32) {
	volatile.StoreUint32(&o.FP_CTRL.Reg, volatile.LoadUint32(&o.FP_CTRL.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetFP_CTRL_REV() uint32 {
	return (volatile.LoadUint32(&o.FP_CTRL.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetFP_CTRL_NUM_CODE_14_12_(value uint32) {
	volatile.StoreUint32(&o.FP_CTRL.Reg, volatile.LoadUint32(&o.FP_CTRL.Reg)&^(0x7000)|value<<12)
}
func (o *PPB_Type) GetFP_CTRL_NUM_CODE_14_12_() uint32 {
	return (volatile.LoadUint32(&o.FP_CTRL.Reg) & 0x7000) >> 12
}
func (o *PPB_Type) SetFP_CTRL_NUM_LIT(value uint32) {
	volatile.StoreUint32(&o.FP_CTRL.Reg, volatile.LoadUint32(&o.FP_CTRL.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetFP_CTRL_NUM_LIT() uint32 {
	return (volatile.LoadUint32(&o.FP_CTRL.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetFP_CTRL_NUM_CODE_7_4_(value uint32) {
	volatile.StoreUint32(&o.FP_CTRL.Reg, volatile.LoadUint32(&o.FP_CTRL.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetFP_CTRL_NUM_CODE_7_4_() uint32 {
	return (volatile.LoadUint32(&o.FP_CTRL.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetFP_CTRL_KEY(value uint32) {
	volatile.StoreUint32(&o.FP_CTRL.Reg, volatile.LoadUint32(&o.FP_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetFP_CTRL_KEY() uint32 {
	return (volatile.LoadUint32(&o.FP_CTRL.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetFP_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.FP_CTRL.Reg, volatile.LoadUint32(&o.FP_CTRL.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFP_CTRL_ENABLE() uint32 {
	return volatile.LoadUint32(&o.FP_CTRL.Reg) & 0x1
}

// PPB.FP_REMAP: Indicates whether the implementation supports Flash Patch remap and, if it does, holds the target address for remap
func (o *PPB_Type) SetFP_REMAP_RMPSPT(value uint32) {
	volatile.StoreUint32(&o.FP_REMAP.Reg, volatile.LoadUint32(&o.FP_REMAP.Reg)&^(0x20000000)|value<<29)
}
func (o *PPB_Type) GetFP_REMAP_RMPSPT() uint32 {
	return (volatile.LoadUint32(&o.FP_REMAP.Reg) & 0x20000000) >> 29
}
func (o *PPB_Type) SetFP_REMAP_REMAP(value uint32) {
	volatile.StoreUint32(&o.FP_REMAP.Reg, volatile.LoadUint32(&o.FP_REMAP.Reg)&^(0x1fffffe0)|value<<5)
}
func (o *PPB_Type) GetFP_REMAP_REMAP() uint32 {
	return (volatile.LoadUint32(&o.FP_REMAP.Reg) & 0x1fffffe0) >> 5
}

// PPB.FP_COMP0: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
func (o *PPB_Type) SetFP_COMP0_BE(value uint32) {
	volatile.StoreUint32(&o.FP_COMP0.Reg, volatile.LoadUint32(&o.FP_COMP0.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFP_COMP0_BE() uint32 {
	return volatile.LoadUint32(&o.FP_COMP0.Reg) & 0x1
}

// PPB.FP_COMP1: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
func (o *PPB_Type) SetFP_COMP1_BE(value uint32) {
	volatile.StoreUint32(&o.FP_COMP1.Reg, volatile.LoadUint32(&o.FP_COMP1.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFP_COMP1_BE() uint32 {
	return volatile.LoadUint32(&o.FP_COMP1.Reg) & 0x1
}

// PPB.FP_COMP2: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
func (o *PPB_Type) SetFP_COMP2_BE(value uint32) {
	volatile.StoreUint32(&o.FP_COMP2.Reg, volatile.LoadUint32(&o.FP_COMP2.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFP_COMP2_BE() uint32 {
	return volatile.LoadUint32(&o.FP_COMP2.Reg) & 0x1
}

// PPB.FP_COMP3: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
func (o *PPB_Type) SetFP_COMP3_BE(value uint32) {
	volatile.StoreUint32(&o.FP_COMP3.Reg, volatile.LoadUint32(&o.FP_COMP3.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFP_COMP3_BE() uint32 {
	return volatile.LoadUint32(&o.FP_COMP3.Reg) & 0x1
}

// PPB.FP_COMP4: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
func (o *PPB_Type) SetFP_COMP4_BE(value uint32) {
	volatile.StoreUint32(&o.FP_COMP4.Reg, volatile.LoadUint32(&o.FP_COMP4.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFP_COMP4_BE() uint32 {
	return volatile.LoadUint32(&o.FP_COMP4.Reg) & 0x1
}

// PPB.FP_COMP5: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
func (o *PPB_Type) SetFP_COMP5_BE(value uint32) {
	volatile.StoreUint32(&o.FP_COMP5.Reg, volatile.LoadUint32(&o.FP_COMP5.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFP_COMP5_BE() uint32 {
	return volatile.LoadUint32(&o.FP_COMP5.Reg) & 0x1
}

// PPB.FP_COMP6: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
func (o *PPB_Type) SetFP_COMP6_BE(value uint32) {
	volatile.StoreUint32(&o.FP_COMP6.Reg, volatile.LoadUint32(&o.FP_COMP6.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFP_COMP6_BE() uint32 {
	return volatile.LoadUint32(&o.FP_COMP6.Reg) & 0x1
}

// PPB.FP_COMP7: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
func (o *PPB_Type) SetFP_COMP7_BE(value uint32) {
	volatile.StoreUint32(&o.FP_COMP7.Reg, volatile.LoadUint32(&o.FP_COMP7.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFP_COMP7_BE() uint32 {
	return volatile.LoadUint32(&o.FP_COMP7.Reg) & 0x1
}

// PPB.FP_DEVARCH: Provides CoreSight discovery information for the FPB
func (o *PPB_Type) SetFP_DEVARCH_ARCHITECT(value uint32) {
	volatile.StoreUint32(&o.FP_DEVARCH.Reg, volatile.LoadUint32(&o.FP_DEVARCH.Reg)&^(0xffe00000)|value<<21)
}
func (o *PPB_Type) GetFP_DEVARCH_ARCHITECT() uint32 {
	return (volatile.LoadUint32(&o.FP_DEVARCH.Reg) & 0xffe00000) >> 21
}
func (o *PPB_Type) SetFP_DEVARCH_PRESENT(value uint32) {
	volatile.StoreUint32(&o.FP_DEVARCH.Reg, volatile.LoadUint32(&o.FP_DEVARCH.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetFP_DEVARCH_PRESENT() uint32 {
	return (volatile.LoadUint32(&o.FP_DEVARCH.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetFP_DEVARCH_REVISION(value uint32) {
	volatile.StoreUint32(&o.FP_DEVARCH.Reg, volatile.LoadUint32(&o.FP_DEVARCH.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetFP_DEVARCH_REVISION() uint32 {
	return (volatile.LoadUint32(&o.FP_DEVARCH.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetFP_DEVARCH_ARCHVER(value uint32) {
	volatile.StoreUint32(&o.FP_DEVARCH.Reg, volatile.LoadUint32(&o.FP_DEVARCH.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetFP_DEVARCH_ARCHVER() uint32 {
	return (volatile.LoadUint32(&o.FP_DEVARCH.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetFP_DEVARCH_ARCHPART(value uint32) {
	volatile.StoreUint32(&o.FP_DEVARCH.Reg, volatile.LoadUint32(&o.FP_DEVARCH.Reg)&^(0xfff)|value)
}
func (o *PPB_Type) GetFP_DEVARCH_ARCHPART() uint32 {
	return volatile.LoadUint32(&o.FP_DEVARCH.Reg) & 0xfff
}

// PPB.FP_DEVTYPE: Provides CoreSight discovery information for the FPB
func (o *PPB_Type) SetFP_DEVTYPE_SUB(value uint32) {
	volatile.StoreUint32(&o.FP_DEVTYPE.Reg, volatile.LoadUint32(&o.FP_DEVTYPE.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetFP_DEVTYPE_SUB() uint32 {
	return (volatile.LoadUint32(&o.FP_DEVTYPE.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetFP_DEVTYPE_MAJOR(value uint32) {
	volatile.StoreUint32(&o.FP_DEVTYPE.Reg, volatile.LoadUint32(&o.FP_DEVTYPE.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetFP_DEVTYPE_MAJOR() uint32 {
	return volatile.LoadUint32(&o.FP_DEVTYPE.Reg) & 0xf
}

// PPB.FP_PIDR4: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_PIDR4_SIZE(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR4.Reg, volatile.LoadUint32(&o.FP_PIDR4.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetFP_PIDR4_SIZE() uint32 {
	return (volatile.LoadUint32(&o.FP_PIDR4.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetFP_PIDR4_DES_2(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR4.Reg, volatile.LoadUint32(&o.FP_PIDR4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetFP_PIDR4_DES_2() uint32 {
	return volatile.LoadUint32(&o.FP_PIDR4.Reg) & 0xf
}

// PPB.FP_PIDR5: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_PIDR5(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR5.Reg, value)
}
func (o *PPB_Type) GetFP_PIDR5() uint32 {
	return volatile.LoadUint32(&o.FP_PIDR5.Reg)
}

// PPB.FP_PIDR6: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_PIDR6(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR6.Reg, value)
}
func (o *PPB_Type) GetFP_PIDR6() uint32 {
	return volatile.LoadUint32(&o.FP_PIDR6.Reg)
}

// PPB.FP_PIDR7: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_PIDR7(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR7.Reg, value)
}
func (o *PPB_Type) GetFP_PIDR7() uint32 {
	return volatile.LoadUint32(&o.FP_PIDR7.Reg)
}

// PPB.FP_PIDR0: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_PIDR0_PART_0(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR0.Reg, volatile.LoadUint32(&o.FP_PIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetFP_PIDR0_PART_0() uint32 {
	return volatile.LoadUint32(&o.FP_PIDR0.Reg) & 0xff
}

// PPB.FP_PIDR1: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_PIDR1_DES_0(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR1.Reg, volatile.LoadUint32(&o.FP_PIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetFP_PIDR1_DES_0() uint32 {
	return (volatile.LoadUint32(&o.FP_PIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetFP_PIDR1_PART_1(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR1.Reg, volatile.LoadUint32(&o.FP_PIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetFP_PIDR1_PART_1() uint32 {
	return volatile.LoadUint32(&o.FP_PIDR1.Reg) & 0xf
}

// PPB.FP_PIDR2: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_PIDR2_REVISION(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR2.Reg, volatile.LoadUint32(&o.FP_PIDR2.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetFP_PIDR2_REVISION() uint32 {
	return (volatile.LoadUint32(&o.FP_PIDR2.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetFP_PIDR2_JEDEC(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR2.Reg, volatile.LoadUint32(&o.FP_PIDR2.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetFP_PIDR2_JEDEC() uint32 {
	return (volatile.LoadUint32(&o.FP_PIDR2.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetFP_PIDR2_DES_1(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR2.Reg, volatile.LoadUint32(&o.FP_PIDR2.Reg)&^(0x7)|value)
}
func (o *PPB_Type) GetFP_PIDR2_DES_1() uint32 {
	return volatile.LoadUint32(&o.FP_PIDR2.Reg) & 0x7
}

// PPB.FP_PIDR3: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_PIDR3_REVAND(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR3.Reg, volatile.LoadUint32(&o.FP_PIDR3.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetFP_PIDR3_REVAND() uint32 {
	return (volatile.LoadUint32(&o.FP_PIDR3.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetFP_PIDR3_CMOD(value uint32) {
	volatile.StoreUint32(&o.FP_PIDR3.Reg, volatile.LoadUint32(&o.FP_PIDR3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetFP_PIDR3_CMOD() uint32 {
	return volatile.LoadUint32(&o.FP_PIDR3.Reg) & 0xf
}

// PPB.FP_CIDR0: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_CIDR0_PRMBL_0(value uint32) {
	volatile.StoreUint32(&o.FP_CIDR0.Reg, volatile.LoadUint32(&o.FP_CIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetFP_CIDR0_PRMBL_0() uint32 {
	return volatile.LoadUint32(&o.FP_CIDR0.Reg) & 0xff
}

// PPB.FP_CIDR1: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_CIDR1_CLASS(value uint32) {
	volatile.StoreUint32(&o.FP_CIDR1.Reg, volatile.LoadUint32(&o.FP_CIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetFP_CIDR1_CLASS() uint32 {
	return (volatile.LoadUint32(&o.FP_CIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetFP_CIDR1_PRMBL_1(value uint32) {
	volatile.StoreUint32(&o.FP_CIDR1.Reg, volatile.LoadUint32(&o.FP_CIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetFP_CIDR1_PRMBL_1() uint32 {
	return volatile.LoadUint32(&o.FP_CIDR1.Reg) & 0xf
}

// PPB.FP_CIDR2: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_CIDR2_PRMBL_2(value uint32) {
	volatile.StoreUint32(&o.FP_CIDR2.Reg, volatile.LoadUint32(&o.FP_CIDR2.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetFP_CIDR2_PRMBL_2() uint32 {
	return volatile.LoadUint32(&o.FP_CIDR2.Reg) & 0xff
}

// PPB.FP_CIDR3: Provides CoreSight discovery information for the FP
func (o *PPB_Type) SetFP_CIDR3_PRMBL_3(value uint32) {
	volatile.StoreUint32(&o.FP_CIDR3.Reg, volatile.LoadUint32(&o.FP_CIDR3.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetFP_CIDR3_PRMBL_3() uint32 {
	return volatile.LoadUint32(&o.FP_CIDR3.Reg) & 0xff
}

// PPB.ICTR: Provides information about the interrupt controller
func (o *PPB_Type) SetICTR_INTLINESNUM(value uint32) {
	volatile.StoreUint32(&o.ICTR.Reg, volatile.LoadUint32(&o.ICTR.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetICTR_INTLINESNUM() uint32 {
	return volatile.LoadUint32(&o.ICTR.Reg) & 0xf
}

// PPB.ACTLR: Provides IMPLEMENTATION DEFINED configuration and control options
func (o *PPB_Type) SetACTLR_EXTEXCLALL(value uint32) {
	volatile.StoreUint32(&o.ACTLR.Reg, volatile.LoadUint32(&o.ACTLR.Reg)&^(0x20000000)|value<<29)
}
func (o *PPB_Type) GetACTLR_EXTEXCLALL() uint32 {
	return (volatile.LoadUint32(&o.ACTLR.Reg) & 0x20000000) >> 29
}
func (o *PPB_Type) SetACTLR_DISITMATBFLUSH(value uint32) {
	volatile.StoreUint32(&o.ACTLR.Reg, volatile.LoadUint32(&o.ACTLR.Reg)&^(0x1000)|value<<12)
}
func (o *PPB_Type) GetACTLR_DISITMATBFLUSH() uint32 {
	return (volatile.LoadUint32(&o.ACTLR.Reg) & 0x1000) >> 12
}
func (o *PPB_Type) SetACTLR_FPEXCODIS(value uint32) {
	volatile.StoreUint32(&o.ACTLR.Reg, volatile.LoadUint32(&o.ACTLR.Reg)&^(0x400)|value<<10)
}
func (o *PPB_Type) GetACTLR_FPEXCODIS() uint32 {
	return (volatile.LoadUint32(&o.ACTLR.Reg) & 0x400) >> 10
}
func (o *PPB_Type) SetACTLR_DISOOFP(value uint32) {
	volatile.StoreUint32(&o.ACTLR.Reg, volatile.LoadUint32(&o.ACTLR.Reg)&^(0x200)|value<<9)
}
func (o *PPB_Type) GetACTLR_DISOOFP() uint32 {
	return (volatile.LoadUint32(&o.ACTLR.Reg) & 0x200) >> 9
}
func (o *PPB_Type) SetACTLR_DISFOLD(value uint32) {
	volatile.StoreUint32(&o.ACTLR.Reg, volatile.LoadUint32(&o.ACTLR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetACTLR_DISFOLD() uint32 {
	return (volatile.LoadUint32(&o.ACTLR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetACTLR_DISMCYCINT(value uint32) {
	volatile.StoreUint32(&o.ACTLR.Reg, volatile.LoadUint32(&o.ACTLR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetACTLR_DISMCYCINT() uint32 {
	return volatile.LoadUint32(&o.ACTLR.Reg) & 0x1
}

// PPB.SYST_CSR: Use the SysTick Control and Status Register to enable the SysTick features.
func (o *PPB_Type) SetSYST_CSR_COUNTFLAG(value uint32) {
	volatile.StoreUint32(&o.SYST_CSR.Reg, volatile.LoadUint32(&o.SYST_CSR.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetSYST_CSR_COUNTFLAG() uint32 {
	return (volatile.LoadUint32(&o.SYST_CSR.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetSYST_CSR_CLKSOURCE(value uint32) {
	volatile.StoreUint32(&o.SYST_CSR.Reg, volatile.LoadUint32(&o.SYST_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetSYST_CSR_CLKSOURCE() uint32 {
	return (volatile.LoadUint32(&o.SYST_CSR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetSYST_CSR_TICKINT(value uint32) {
	volatile.StoreUint32(&o.SYST_CSR.Reg, volatile.LoadUint32(&o.SYST_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetSYST_CSR_TICKINT() uint32 {
	return (volatile.LoadUint32(&o.SYST_CSR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetSYST_CSR_ENABLE(value uint32) {
	volatile.StoreUint32(&o.SYST_CSR.Reg, volatile.LoadUint32(&o.SYST_CSR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetSYST_CSR_ENABLE() uint32 {
	return volatile.LoadUint32(&o.SYST_CSR.Reg) & 0x1
}

// PPB.SYST_RVR: Use the SysTick Reload Value Register to specify the start value to load into the current value register when the counter reaches 0. It can be any value between 0 and 0x00FFFFFF. A start value of 0 is possible, but has no effect because the SysTick interrupt and COUNTFLAG are activated when counting from 1 to 0. The reset value of this register is UNKNOWN. To generate a multi-shot timer with a period of N processor clock cycles, use a RELOAD value of N-1. For example, if the SysTick interrupt is required every 100 clock pulses, set RELOAD to 99.
func (o *PPB_Type) SetSYST_RVR_RELOAD(value uint32) {
	volatile.StoreUint32(&o.SYST_RVR.Reg, volatile.LoadUint32(&o.SYST_RVR.Reg)&^(0xffffff)|value)
}
func (o *PPB_Type) GetSYST_RVR_RELOAD() uint32 {
	return volatile.LoadUint32(&o.SYST_RVR.Reg) & 0xffffff
}

// PPB.SYST_CVR: Use the SysTick Current Value Register to find the current value in the register. The reset value of this register is UNKNOWN.
func (o *PPB_Type) SetSYST_CVR_CURRENT(value uint32) {
	volatile.StoreUint32(&o.SYST_CVR.Reg, volatile.LoadUint32(&o.SYST_CVR.Reg)&^(0xffffff)|value)
}
func (o *PPB_Type) GetSYST_CVR_CURRENT() uint32 {
	return volatile.LoadUint32(&o.SYST_CVR.Reg) & 0xffffff
}

// PPB.SYST_CALIB: Use the SysTick Calibration Value Register to enable software to scale to any required speed using divide and multiply.
func (o *PPB_Type) SetSYST_CALIB_NOREF(value uint32) {
	volatile.StoreUint32(&o.SYST_CALIB.Reg, volatile.LoadUint32(&o.SYST_CALIB.Reg)&^(0x80000000)|value<<31)
}
func (o *PPB_Type) GetSYST_CALIB_NOREF() uint32 {
	return (volatile.LoadUint32(&o.SYST_CALIB.Reg) & 0x80000000) >> 31
}
func (o *PPB_Type) SetSYST_CALIB_SKEW(value uint32) {
	volatile.StoreUint32(&o.SYST_CALIB.Reg, volatile.LoadUint32(&o.SYST_CALIB.Reg)&^(0x40000000)|value<<30)
}
func (o *PPB_Type) GetSYST_CALIB_SKEW() uint32 {
	return (volatile.LoadUint32(&o.SYST_CALIB.Reg) & 0x40000000) >> 30
}
func (o *PPB_Type) SetSYST_CALIB_TENMS(value uint32) {
	volatile.StoreUint32(&o.SYST_CALIB.Reg, volatile.LoadUint32(&o.SYST_CALIB.Reg)&^(0xffffff)|value)
}
func (o *PPB_Type) GetSYST_CALIB_TENMS() uint32 {
	return volatile.LoadUint32(&o.SYST_CALIB.Reg) & 0xffffff
}

// PPB.NVIC_ISER0: Enables or reads the enabled state of each group of 32 interrupts
func (o *PPB_Type) SetNVIC_ISER0(value uint32) {
	volatile.StoreUint32(&o.NVIC_ISER0.Reg, value)
}
func (o *PPB_Type) GetNVIC_ISER0() uint32 {
	return volatile.LoadUint32(&o.NVIC_ISER0.Reg)
}

// PPB.NVIC_ISER1: Enables or reads the enabled state of each group of 32 interrupts
func (o *PPB_Type) SetNVIC_ISER1(value uint32) {
	volatile.StoreUint32(&o.NVIC_ISER1.Reg, value)
}
func (o *PPB_Type) GetNVIC_ISER1() uint32 {
	return volatile.LoadUint32(&o.NVIC_ISER1.Reg)
}

// PPB.NVIC_ICER0: Clears or reads the enabled state of each group of 32 interrupts
func (o *PPB_Type) SetNVIC_ICER0(value uint32) {
	volatile.StoreUint32(&o.NVIC_ICER0.Reg, value)
}
func (o *PPB_Type) GetNVIC_ICER0() uint32 {
	return volatile.LoadUint32(&o.NVIC_ICER0.Reg)
}

// PPB.NVIC_ICER1: Clears or reads the enabled state of each group of 32 interrupts
func (o *PPB_Type) SetNVIC_ICER1(value uint32) {
	volatile.StoreUint32(&o.NVIC_ICER1.Reg, value)
}
func (o *PPB_Type) GetNVIC_ICER1() uint32 {
	return volatile.LoadUint32(&o.NVIC_ICER1.Reg)
}

// PPB.NVIC_ISPR0: Enables or reads the pending state of each group of 32 interrupts
func (o *PPB_Type) SetNVIC_ISPR0(value uint32) {
	volatile.StoreUint32(&o.NVIC_ISPR0.Reg, value)
}
func (o *PPB_Type) GetNVIC_ISPR0() uint32 {
	return volatile.LoadUint32(&o.NVIC_ISPR0.Reg)
}

// PPB.NVIC_ISPR1: Enables or reads the pending state of each group of 32 interrupts
func (o *PPB_Type) SetNVIC_ISPR1(value uint32) {
	volatile.StoreUint32(&o.NVIC_ISPR1.Reg, value)
}
func (o *PPB_Type) GetNVIC_ISPR1() uint32 {
	return volatile.LoadUint32(&o.NVIC_ISPR1.Reg)
}

// PPB.NVIC_ICPR0: Clears or reads the pending state of each group of 32 interrupts
func (o *PPB_Type) SetNVIC_ICPR0(value uint32) {
	volatile.StoreUint32(&o.NVIC_ICPR0.Reg, value)
}
func (o *PPB_Type) GetNVIC_ICPR0() uint32 {
	return volatile.LoadUint32(&o.NVIC_ICPR0.Reg)
}

// PPB.NVIC_ICPR1: Clears or reads the pending state of each group of 32 interrupts
func (o *PPB_Type) SetNVIC_ICPR1(value uint32) {
	volatile.StoreUint32(&o.NVIC_ICPR1.Reg, value)
}
func (o *PPB_Type) GetNVIC_ICPR1() uint32 {
	return volatile.LoadUint32(&o.NVIC_ICPR1.Reg)
}

// PPB.NVIC_IABR0: For each group of 32 interrupts, shows the active state of each interrupt
func (o *PPB_Type) SetNVIC_IABR0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IABR0.Reg, value)
}
func (o *PPB_Type) GetNVIC_IABR0() uint32 {
	return volatile.LoadUint32(&o.NVIC_IABR0.Reg)
}

// PPB.NVIC_IABR1: For each group of 32 interrupts, shows the active state of each interrupt
func (o *PPB_Type) SetNVIC_IABR1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IABR1.Reg, value)
}
func (o *PPB_Type) GetNVIC_IABR1() uint32 {
	return volatile.LoadUint32(&o.NVIC_IABR1.Reg)
}

// PPB.NVIC_ITNS0: For each group of 32 interrupts, determines whether each interrupt targets Non-secure or Secure state
func (o *PPB_Type) SetNVIC_ITNS0(value uint32) {
	volatile.StoreUint32(&o.NVIC_ITNS0.Reg, value)
}
func (o *PPB_Type) GetNVIC_ITNS0() uint32 {
	return volatile.LoadUint32(&o.NVIC_ITNS0.Reg)
}

// PPB.NVIC_ITNS1: For each group of 32 interrupts, determines whether each interrupt targets Non-secure or Secure state
func (o *PPB_Type) SetNVIC_ITNS1(value uint32) {
	volatile.StoreUint32(&o.NVIC_ITNS1.Reg, value)
}
func (o *PPB_Type) GetNVIC_ITNS1() uint32 {
	return volatile.LoadUint32(&o.NVIC_ITNS1.Reg)
}

// PPB.NVIC_IPR0: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR0_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR0.Reg, volatile.LoadUint32(&o.NVIC_IPR0.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR0_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR0.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR0_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR0.Reg, volatile.LoadUint32(&o.NVIC_IPR0.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR0_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR0.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR0_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR0.Reg, volatile.LoadUint32(&o.NVIC_IPR0.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR0_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR0.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR0_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR0.Reg, volatile.LoadUint32(&o.NVIC_IPR0.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR0_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR0.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR1: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR1_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR1.Reg, volatile.LoadUint32(&o.NVIC_IPR1.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR1_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR1.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR1_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR1.Reg, volatile.LoadUint32(&o.NVIC_IPR1.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR1_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR1.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR1_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR1.Reg, volatile.LoadUint32(&o.NVIC_IPR1.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR1_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR1.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR1_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR1.Reg, volatile.LoadUint32(&o.NVIC_IPR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR1_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR1.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR2: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR2_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR2.Reg, volatile.LoadUint32(&o.NVIC_IPR2.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR2_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR2.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR2_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR2.Reg, volatile.LoadUint32(&o.NVIC_IPR2.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR2_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR2.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR2_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR2.Reg, volatile.LoadUint32(&o.NVIC_IPR2.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR2_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR2.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR2_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR2.Reg, volatile.LoadUint32(&o.NVIC_IPR2.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR2_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR2.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR3: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR3_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR3.Reg, volatile.LoadUint32(&o.NVIC_IPR3.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR3_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR3.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR3_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR3.Reg, volatile.LoadUint32(&o.NVIC_IPR3.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR3_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR3.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR3_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR3.Reg, volatile.LoadUint32(&o.NVIC_IPR3.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR3_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR3.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR3_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR3.Reg, volatile.LoadUint32(&o.NVIC_IPR3.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR3_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR3.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR4: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR4_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR4.Reg, volatile.LoadUint32(&o.NVIC_IPR4.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR4_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR4.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR4_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR4.Reg, volatile.LoadUint32(&o.NVIC_IPR4.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR4_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR4.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR4_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR4.Reg, volatile.LoadUint32(&o.NVIC_IPR4.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR4_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR4.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR4_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR4.Reg, volatile.LoadUint32(&o.NVIC_IPR4.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR4_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR4.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR5: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR5_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR5.Reg, volatile.LoadUint32(&o.NVIC_IPR5.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR5_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR5.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR5_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR5.Reg, volatile.LoadUint32(&o.NVIC_IPR5.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR5_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR5.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR5_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR5.Reg, volatile.LoadUint32(&o.NVIC_IPR5.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR5_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR5.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR5_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR5.Reg, volatile.LoadUint32(&o.NVIC_IPR5.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR5_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR5.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR6: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR6_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR6.Reg, volatile.LoadUint32(&o.NVIC_IPR6.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR6_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR6.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR6_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR6.Reg, volatile.LoadUint32(&o.NVIC_IPR6.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR6_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR6.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR6_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR6.Reg, volatile.LoadUint32(&o.NVIC_IPR6.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR6_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR6.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR6_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR6.Reg, volatile.LoadUint32(&o.NVIC_IPR6.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR6_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR6.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR7: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR7_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR7.Reg, volatile.LoadUint32(&o.NVIC_IPR7.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR7_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR7.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR7_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR7.Reg, volatile.LoadUint32(&o.NVIC_IPR7.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR7_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR7.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR7_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR7.Reg, volatile.LoadUint32(&o.NVIC_IPR7.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR7_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR7.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR7_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR7.Reg, volatile.LoadUint32(&o.NVIC_IPR7.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR7_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR7.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR8: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR8_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR8.Reg, volatile.LoadUint32(&o.NVIC_IPR8.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR8_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR8.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR8_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR8.Reg, volatile.LoadUint32(&o.NVIC_IPR8.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR8_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR8.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR8_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR8.Reg, volatile.LoadUint32(&o.NVIC_IPR8.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR8_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR8.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR8_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR8.Reg, volatile.LoadUint32(&o.NVIC_IPR8.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR8_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR8.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR9: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR9_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR9.Reg, volatile.LoadUint32(&o.NVIC_IPR9.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR9_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR9.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR9_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR9.Reg, volatile.LoadUint32(&o.NVIC_IPR9.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR9_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR9.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR9_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR9.Reg, volatile.LoadUint32(&o.NVIC_IPR9.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR9_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR9.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR9_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR9.Reg, volatile.LoadUint32(&o.NVIC_IPR9.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR9_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR9.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR10: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR10_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR10.Reg, volatile.LoadUint32(&o.NVIC_IPR10.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR10_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR10.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR10_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR10.Reg, volatile.LoadUint32(&o.NVIC_IPR10.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR10_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR10.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR10_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR10.Reg, volatile.LoadUint32(&o.NVIC_IPR10.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR10_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR10.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR10_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR10.Reg, volatile.LoadUint32(&o.NVIC_IPR10.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR10_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR10.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR11: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR11_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR11.Reg, volatile.LoadUint32(&o.NVIC_IPR11.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR11_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR11.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR11_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR11.Reg, volatile.LoadUint32(&o.NVIC_IPR11.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR11_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR11.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR11_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR11.Reg, volatile.LoadUint32(&o.NVIC_IPR11.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR11_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR11.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR11_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR11.Reg, volatile.LoadUint32(&o.NVIC_IPR11.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR11_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR11.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR12: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR12_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR12.Reg, volatile.LoadUint32(&o.NVIC_IPR12.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR12_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR12.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR12_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR12.Reg, volatile.LoadUint32(&o.NVIC_IPR12.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR12_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR12.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR12_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR12.Reg, volatile.LoadUint32(&o.NVIC_IPR12.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR12_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR12.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR12_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR12.Reg, volatile.LoadUint32(&o.NVIC_IPR12.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR12_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR12.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR13: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR13_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR13.Reg, volatile.LoadUint32(&o.NVIC_IPR13.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR13_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR13.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR13_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR13.Reg, volatile.LoadUint32(&o.NVIC_IPR13.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR13_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR13.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR13_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR13.Reg, volatile.LoadUint32(&o.NVIC_IPR13.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR13_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR13.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR13_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR13.Reg, volatile.LoadUint32(&o.NVIC_IPR13.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR13_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR13.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR14: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR14_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR14.Reg, volatile.LoadUint32(&o.NVIC_IPR14.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR14_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR14.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR14_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR14.Reg, volatile.LoadUint32(&o.NVIC_IPR14.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR14_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR14.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR14_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR14.Reg, volatile.LoadUint32(&o.NVIC_IPR14.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR14_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR14.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR14_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR14.Reg, volatile.LoadUint32(&o.NVIC_IPR14.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR14_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR14.Reg) & 0xf0) >> 4
}

// PPB.NVIC_IPR15: Sets or reads interrupt priorities
func (o *PPB_Type) SetNVIC_IPR15_PRI_N3(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR15.Reg, volatile.LoadUint32(&o.NVIC_IPR15.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetNVIC_IPR15_PRI_N3() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR15.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetNVIC_IPR15_PRI_N2(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR15.Reg, volatile.LoadUint32(&o.NVIC_IPR15.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetNVIC_IPR15_PRI_N2() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR15.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetNVIC_IPR15_PRI_N1(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR15.Reg, volatile.LoadUint32(&o.NVIC_IPR15.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetNVIC_IPR15_PRI_N1() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR15.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetNVIC_IPR15_PRI_N0(value uint32) {
	volatile.StoreUint32(&o.NVIC_IPR15.Reg, volatile.LoadUint32(&o.NVIC_IPR15.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetNVIC_IPR15_PRI_N0() uint32 {
	return (volatile.LoadUint32(&o.NVIC_IPR15.Reg) & 0xf0) >> 4
}

// PPB.CPUID: Provides identification information for the PE, including an implementer code for the device and a device ID number
func (o *PPB_Type) SetCPUID_IMPLEMENTER(value uint32) {
	volatile.StoreUint32(&o.CPUID.Reg, volatile.LoadUint32(&o.CPUID.Reg)&^(0xff000000)|value<<24)
}
func (o *PPB_Type) GetCPUID_IMPLEMENTER() uint32 {
	return (volatile.LoadUint32(&o.CPUID.Reg) & 0xff000000) >> 24
}
func (o *PPB_Type) SetCPUID_VARIANT(value uint32) {
	volatile.StoreUint32(&o.CPUID.Reg, volatile.LoadUint32(&o.CPUID.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetCPUID_VARIANT() uint32 {
	return (volatile.LoadUint32(&o.CPUID.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetCPUID_ARCHITECTURE(value uint32) {
	volatile.StoreUint32(&o.CPUID.Reg, volatile.LoadUint32(&o.CPUID.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetCPUID_ARCHITECTURE() uint32 {
	return (volatile.LoadUint32(&o.CPUID.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetCPUID_PARTNO(value uint32) {
	volatile.StoreUint32(&o.CPUID.Reg, volatile.LoadUint32(&o.CPUID.Reg)&^(0xfff0)|value<<4)
}
func (o *PPB_Type) GetCPUID_PARTNO() uint32 {
	return (volatile.LoadUint32(&o.CPUID.Reg) & 0xfff0) >> 4
}
func (o *PPB_Type) SetCPUID_REVISION(value uint32) {
	volatile.StoreUint32(&o.CPUID.Reg, volatile.LoadUint32(&o.CPUID.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCPUID_REVISION() uint32 {
	return volatile.LoadUint32(&o.CPUID.Reg) & 0xf
}

// PPB.ICSR: Controls and provides status information for NMI, PendSV, SysTick and interrupts
func (o *PPB_Type) SetICSR_PENDNMISET(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x80000000)|value<<31)
}
func (o *PPB_Type) GetICSR_PENDNMISET() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x80000000) >> 31
}
func (o *PPB_Type) SetICSR_PENDNMICLR(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x40000000)|value<<30)
}
func (o *PPB_Type) GetICSR_PENDNMICLR() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x40000000) >> 30
}
func (o *PPB_Type) SetICSR_PENDSVSET(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x10000000)|value<<28)
}
func (o *PPB_Type) GetICSR_PENDSVSET() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x10000000) >> 28
}
func (o *PPB_Type) SetICSR_PENDSVCLR(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x8000000)|value<<27)
}
func (o *PPB_Type) GetICSR_PENDSVCLR() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x8000000) >> 27
}
func (o *PPB_Type) SetICSR_PENDSTSET(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x4000000)|value<<26)
}
func (o *PPB_Type) GetICSR_PENDSTSET() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x4000000) >> 26
}
func (o *PPB_Type) SetICSR_PENDSTCLR(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x2000000)|value<<25)
}
func (o *PPB_Type) GetICSR_PENDSTCLR() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x2000000) >> 25
}
func (o *PPB_Type) SetICSR_STTNS(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetICSR_STTNS() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetICSR_ISRPREEMPT(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x800000)|value<<23)
}
func (o *PPB_Type) GetICSR_ISRPREEMPT() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x800000) >> 23
}
func (o *PPB_Type) SetICSR_ISRPENDING(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x400000)|value<<22)
}
func (o *PPB_Type) GetICSR_ISRPENDING() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x400000) >> 22
}
func (o *PPB_Type) SetICSR_VECTPENDING(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x1ff000)|value<<12)
}
func (o *PPB_Type) GetICSR_VECTPENDING() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x1ff000) >> 12
}
func (o *PPB_Type) SetICSR_RETTOBASE(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x800)|value<<11)
}
func (o *PPB_Type) GetICSR_RETTOBASE() uint32 {
	return (volatile.LoadUint32(&o.ICSR.Reg) & 0x800) >> 11
}
func (o *PPB_Type) SetICSR_VECTACTIVE(value uint32) {
	volatile.StoreUint32(&o.ICSR.Reg, volatile.LoadUint32(&o.ICSR.Reg)&^(0x1ff)|value)
}
func (o *PPB_Type) GetICSR_VECTACTIVE() uint32 {
	return volatile.LoadUint32(&o.ICSR.Reg) & 0x1ff
}

// PPB.VTOR: The VTOR indicates the offset of the vector table base address from memory address 0x00000000.
func (o *PPB_Type) SetVTOR_TBLOFF(value uint32) {
	volatile.StoreUint32(&o.VTOR.Reg, volatile.LoadUint32(&o.VTOR.Reg)&^(0xffffff80)|value<<7)
}
func (o *PPB_Type) GetVTOR_TBLOFF() uint32 {
	return (volatile.LoadUint32(&o.VTOR.Reg) & 0xffffff80) >> 7
}

// PPB.AIRCR: Use the Application Interrupt and Reset Control Register to: determine data endianness, clear all active state information from debug halt mode, request a system reset.
func (o *PPB_Type) SetAIRCR_VECTKEY(value uint32) {
	volatile.StoreUint32(&o.AIRCR.Reg, volatile.LoadUint32(&o.AIRCR.Reg)&^(0xffff0000)|value<<16)
}
func (o *PPB_Type) GetAIRCR_VECTKEY() uint32 {
	return (volatile.LoadUint32(&o.AIRCR.Reg) & 0xffff0000) >> 16
}
func (o *PPB_Type) SetAIRCR_ENDIANESS(value uint32) {
	volatile.StoreUint32(&o.AIRCR.Reg, volatile.LoadUint32(&o.AIRCR.Reg)&^(0x8000)|value<<15)
}
func (o *PPB_Type) GetAIRCR_ENDIANESS() uint32 {
	return (volatile.LoadUint32(&o.AIRCR.Reg) & 0x8000) >> 15
}
func (o *PPB_Type) SetAIRCR_PRIS(value uint32) {
	volatile.StoreUint32(&o.AIRCR.Reg, volatile.LoadUint32(&o.AIRCR.Reg)&^(0x4000)|value<<14)
}
func (o *PPB_Type) GetAIRCR_PRIS() uint32 {
	return (volatile.LoadUint32(&o.AIRCR.Reg) & 0x4000) >> 14
}
func (o *PPB_Type) SetAIRCR_BFHFNMINS(value uint32) {
	volatile.StoreUint32(&o.AIRCR.Reg, volatile.LoadUint32(&o.AIRCR.Reg)&^(0x2000)|value<<13)
}
func (o *PPB_Type) GetAIRCR_BFHFNMINS() uint32 {
	return (volatile.LoadUint32(&o.AIRCR.Reg) & 0x2000) >> 13
}
func (o *PPB_Type) SetAIRCR_PRIGROUP(value uint32) {
	volatile.StoreUint32(&o.AIRCR.Reg, volatile.LoadUint32(&o.AIRCR.Reg)&^(0x700)|value<<8)
}
func (o *PPB_Type) GetAIRCR_PRIGROUP() uint32 {
	return (volatile.LoadUint32(&o.AIRCR.Reg) & 0x700) >> 8
}
func (o *PPB_Type) SetAIRCR_SYSRESETREQS(value uint32) {
	volatile.StoreUint32(&o.AIRCR.Reg, volatile.LoadUint32(&o.AIRCR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetAIRCR_SYSRESETREQS() uint32 {
	return (volatile.LoadUint32(&o.AIRCR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetAIRCR_SYSRESETREQ(value uint32) {
	volatile.StoreUint32(&o.AIRCR.Reg, volatile.LoadUint32(&o.AIRCR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetAIRCR_SYSRESETREQ() uint32 {
	return (volatile.LoadUint32(&o.AIRCR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetAIRCR_VECTCLRACTIVE(value uint32) {
	volatile.StoreUint32(&o.AIRCR.Reg, volatile.LoadUint32(&o.AIRCR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetAIRCR_VECTCLRACTIVE() uint32 {
	return (volatile.LoadUint32(&o.AIRCR.Reg) & 0x2) >> 1
}

// PPB.SCR: System Control Register. Use the System Control Register for power-management functions: signal to the system when the processor can enter a low power state, control how the processor enters and exits low power states.
func (o *PPB_Type) SetSCR_SEVONPEND(value uint32) {
	volatile.StoreUint32(&o.SCR.Reg, volatile.LoadUint32(&o.SCR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetSCR_SEVONPEND() uint32 {
	return (volatile.LoadUint32(&o.SCR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetSCR_SLEEPDEEPS(value uint32) {
	volatile.StoreUint32(&o.SCR.Reg, volatile.LoadUint32(&o.SCR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetSCR_SLEEPDEEPS() uint32 {
	return (volatile.LoadUint32(&o.SCR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetSCR_SLEEPDEEP(value uint32) {
	volatile.StoreUint32(&o.SCR.Reg, volatile.LoadUint32(&o.SCR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetSCR_SLEEPDEEP() uint32 {
	return (volatile.LoadUint32(&o.SCR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetSCR_SLEEPONEXIT(value uint32) {
	volatile.StoreUint32(&o.SCR.Reg, volatile.LoadUint32(&o.SCR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetSCR_SLEEPONEXIT() uint32 {
	return (volatile.LoadUint32(&o.SCR.Reg) & 0x2) >> 1
}

// PPB.CCR: Sets or returns configuration and control data
func (o *PPB_Type) SetCCR_BP(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x40000)|value<<18)
}
func (o *PPB_Type) GetCCR_BP() uint32 {
	return (volatile.LoadUint32(&o.CCR.Reg) & 0x40000) >> 18
}
func (o *PPB_Type) SetCCR_IC(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x20000)|value<<17)
}
func (o *PPB_Type) GetCCR_IC() uint32 {
	return (volatile.LoadUint32(&o.CCR.Reg) & 0x20000) >> 17
}
func (o *PPB_Type) SetCCR_DC(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetCCR_DC() uint32 {
	return (volatile.LoadUint32(&o.CCR.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetCCR_STKOFHFNMIGN(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x400)|value<<10)
}
func (o *PPB_Type) GetCCR_STKOFHFNMIGN() uint32 {
	return (volatile.LoadUint32(&o.CCR.Reg) & 0x400) >> 10
}
func (o *PPB_Type) SetCCR_RES1(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x200)|value<<9)
}
func (o *PPB_Type) GetCCR_RES1() uint32 {
	return (volatile.LoadUint32(&o.CCR.Reg) & 0x200) >> 9
}
func (o *PPB_Type) SetCCR_BFHFNMIGN(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x100)|value<<8)
}
func (o *PPB_Type) GetCCR_BFHFNMIGN() uint32 {
	return (volatile.LoadUint32(&o.CCR.Reg) & 0x100) >> 8
}
func (o *PPB_Type) SetCCR_DIV_0_TRP(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetCCR_DIV_0_TRP() uint32 {
	return (volatile.LoadUint32(&o.CCR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetCCR_UNALIGN_TRP(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetCCR_UNALIGN_TRP() uint32 {
	return (volatile.LoadUint32(&o.CCR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetCCR_USERSETMPEND(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetCCR_USERSETMPEND() uint32 {
	return (volatile.LoadUint32(&o.CCR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetCCR_RES1_1(value uint32) {
	volatile.StoreUint32(&o.CCR.Reg, volatile.LoadUint32(&o.CCR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetCCR_RES1_1() uint32 {
	return volatile.LoadUint32(&o.CCR.Reg) & 0x1
}

// PPB.SHPR1: Sets or returns priority for system handlers 4 - 7
func (o *PPB_Type) SetSHPR1_PRI_7_3(value uint32) {
	volatile.StoreUint32(&o.SHPR1.Reg, volatile.LoadUint32(&o.SHPR1.Reg)&^(0xe0000000)|value<<29)
}
func (o *PPB_Type) GetSHPR1_PRI_7_3() uint32 {
	return (volatile.LoadUint32(&o.SHPR1.Reg) & 0xe0000000) >> 29
}
func (o *PPB_Type) SetSHPR1_PRI_6_3(value uint32) {
	volatile.StoreUint32(&o.SHPR1.Reg, volatile.LoadUint32(&o.SHPR1.Reg)&^(0xe00000)|value<<21)
}
func (o *PPB_Type) GetSHPR1_PRI_6_3() uint32 {
	return (volatile.LoadUint32(&o.SHPR1.Reg) & 0xe00000) >> 21
}
func (o *PPB_Type) SetSHPR1_PRI_5_3(value uint32) {
	volatile.StoreUint32(&o.SHPR1.Reg, volatile.LoadUint32(&o.SHPR1.Reg)&^(0xe000)|value<<13)
}
func (o *PPB_Type) GetSHPR1_PRI_5_3() uint32 {
	return (volatile.LoadUint32(&o.SHPR1.Reg) & 0xe000) >> 13
}
func (o *PPB_Type) SetSHPR1_PRI_4_3(value uint32) {
	volatile.StoreUint32(&o.SHPR1.Reg, volatile.LoadUint32(&o.SHPR1.Reg)&^(0xe0)|value<<5)
}
func (o *PPB_Type) GetSHPR1_PRI_4_3() uint32 {
	return (volatile.LoadUint32(&o.SHPR1.Reg) & 0xe0) >> 5
}

// PPB.SHPR2: Sets or returns priority for system handlers 8 - 11
func (o *PPB_Type) SetSHPR2_PRI_11_3(value uint32) {
	volatile.StoreUint32(&o.SHPR2.Reg, volatile.LoadUint32(&o.SHPR2.Reg)&^(0xe0000000)|value<<29)
}
func (o *PPB_Type) GetSHPR2_PRI_11_3() uint32 {
	return (volatile.LoadUint32(&o.SHPR2.Reg) & 0xe0000000) >> 29
}
func (o *PPB_Type) SetSHPR2_PRI_10(value uint32) {
	volatile.StoreUint32(&o.SHPR2.Reg, volatile.LoadUint32(&o.SHPR2.Reg)&^(0xff0000)|value<<16)
}
func (o *PPB_Type) GetSHPR2_PRI_10() uint32 {
	return (volatile.LoadUint32(&o.SHPR2.Reg) & 0xff0000) >> 16
}
func (o *PPB_Type) SetSHPR2_PRI_9(value uint32) {
	volatile.StoreUint32(&o.SHPR2.Reg, volatile.LoadUint32(&o.SHPR2.Reg)&^(0xff00)|value<<8)
}
func (o *PPB_Type) GetSHPR2_PRI_9() uint32 {
	return (volatile.LoadUint32(&o.SHPR2.Reg) & 0xff00) >> 8
}
func (o *PPB_Type) SetSHPR2_PRI_8(value uint32) {
	volatile.StoreUint32(&o.SHPR2.Reg, volatile.LoadUint32(&o.SHPR2.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetSHPR2_PRI_8() uint32 {
	return volatile.LoadUint32(&o.SHPR2.Reg) & 0xff
}

// PPB.SHPR3: Sets or returns priority for system handlers 12 - 15
func (o *PPB_Type) SetSHPR3_PRI_15_3(value uint32) {
	volatile.StoreUint32(&o.SHPR3.Reg, volatile.LoadUint32(&o.SHPR3.Reg)&^(0xe0000000)|value<<29)
}
func (o *PPB_Type) GetSHPR3_PRI_15_3() uint32 {
	return (volatile.LoadUint32(&o.SHPR3.Reg) & 0xe0000000) >> 29
}
func (o *PPB_Type) SetSHPR3_PRI_14_3(value uint32) {
	volatile.StoreUint32(&o.SHPR3.Reg, volatile.LoadUint32(&o.SHPR3.Reg)&^(0xe00000)|value<<21)
}
func (o *PPB_Type) GetSHPR3_PRI_14_3() uint32 {
	return (volatile.LoadUint32(&o.SHPR3.Reg) & 0xe00000) >> 21
}
func (o *PPB_Type) SetSHPR3_PRI_13(value uint32) {
	volatile.StoreUint32(&o.SHPR3.Reg, volatile.LoadUint32(&o.SHPR3.Reg)&^(0xff00)|value<<8)
}
func (o *PPB_Type) GetSHPR3_PRI_13() uint32 {
	return (volatile.LoadUint32(&o.SHPR3.Reg) & 0xff00) >> 8
}
func (o *PPB_Type) SetSHPR3_PRI_12_3(value uint32) {
	volatile.StoreUint32(&o.SHPR3.Reg, volatile.LoadUint32(&o.SHPR3.Reg)&^(0xe0)|value<<5)
}
func (o *PPB_Type) GetSHPR3_PRI_12_3() uint32 {
	return (volatile.LoadUint32(&o.SHPR3.Reg) & 0xe0) >> 5
}

// PPB.SHCSR: Provides access to the active and pending status of system exceptions
func (o *PPB_Type) SetSHCSR_HARDFAULTPENDED(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x200000)|value<<21)
}
func (o *PPB_Type) GetSHCSR_HARDFAULTPENDED() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x200000) >> 21
}
func (o *PPB_Type) SetSHCSR_SECUREFAULTPENDED(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetSHCSR_SECUREFAULTPENDED() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetSHCSR_SECUREFAULTENA(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x80000)|value<<19)
}
func (o *PPB_Type) GetSHCSR_SECUREFAULTENA() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x80000) >> 19
}
func (o *PPB_Type) SetSHCSR_USGFAULTENA(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x40000)|value<<18)
}
func (o *PPB_Type) GetSHCSR_USGFAULTENA() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x40000) >> 18
}
func (o *PPB_Type) SetSHCSR_BUSFAULTENA(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x20000)|value<<17)
}
func (o *PPB_Type) GetSHCSR_BUSFAULTENA() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x20000) >> 17
}
func (o *PPB_Type) SetSHCSR_MEMFAULTENA(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetSHCSR_MEMFAULTENA() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetSHCSR_SVCALLPENDED(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x8000)|value<<15)
}
func (o *PPB_Type) GetSHCSR_SVCALLPENDED() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x8000) >> 15
}
func (o *PPB_Type) SetSHCSR_BUSFAULTPENDED(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x4000)|value<<14)
}
func (o *PPB_Type) GetSHCSR_BUSFAULTPENDED() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x4000) >> 14
}
func (o *PPB_Type) SetSHCSR_MEMFAULTPENDED(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x2000)|value<<13)
}
func (o *PPB_Type) GetSHCSR_MEMFAULTPENDED() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x2000) >> 13
}
func (o *PPB_Type) SetSHCSR_USGFAULTPENDED(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x1000)|value<<12)
}
func (o *PPB_Type) GetSHCSR_USGFAULTPENDED() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x1000) >> 12
}
func (o *PPB_Type) SetSHCSR_SYSTICKACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x800)|value<<11)
}
func (o *PPB_Type) GetSHCSR_SYSTICKACT() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x800) >> 11
}
func (o *PPB_Type) SetSHCSR_PENDSVACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x400)|value<<10)
}
func (o *PPB_Type) GetSHCSR_PENDSVACT() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x400) >> 10
}
func (o *PPB_Type) SetSHCSR_MONITORACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x100)|value<<8)
}
func (o *PPB_Type) GetSHCSR_MONITORACT() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x100) >> 8
}
func (o *PPB_Type) SetSHCSR_SVCALLACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x80)|value<<7)
}
func (o *PPB_Type) GetSHCSR_SVCALLACT() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x80) >> 7
}
func (o *PPB_Type) SetSHCSR_NMIACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x20)|value<<5)
}
func (o *PPB_Type) GetSHCSR_NMIACT() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x20) >> 5
}
func (o *PPB_Type) SetSHCSR_SECUREFAULTACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetSHCSR_SECUREFAULTACT() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetSHCSR_USGFAULTACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetSHCSR_USGFAULTACT() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetSHCSR_HARDFAULTACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetSHCSR_HARDFAULTACT() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetSHCSR_BUSFAULTACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetSHCSR_BUSFAULTACT() uint32 {
	return (volatile.LoadUint32(&o.SHCSR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetSHCSR_MEMFAULTACT(value uint32) {
	volatile.StoreUint32(&o.SHCSR.Reg, volatile.LoadUint32(&o.SHCSR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetSHCSR_MEMFAULTACT() uint32 {
	return volatile.LoadUint32(&o.SHCSR.Reg) & 0x1
}

// PPB.CFSR: Contains the three Configurable Fault Status Registers. 31:16 UFSR: Provides information on UsageFault exceptions 15:8 BFSR: Provides information on BusFault exceptions 7:0 MMFSR: Provides information on MemManage exceptions
func (o *PPB_Type) SetCFSR_UFSR_DIVBYZERO(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x2000000)|value<<25)
}
func (o *PPB_Type) GetCFSR_UFSR_DIVBYZERO() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x2000000) >> 25
}
func (o *PPB_Type) SetCFSR_UFSR_UNALIGNED(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetCFSR_UFSR_UNALIGNED() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetCFSR_UFSR_STKOF(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetCFSR_UFSR_STKOF() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetCFSR_UFSR_NOCP(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x80000)|value<<19)
}
func (o *PPB_Type) GetCFSR_UFSR_NOCP() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x80000) >> 19
}
func (o *PPB_Type) SetCFSR_UFSR_INVPC(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x40000)|value<<18)
}
func (o *PPB_Type) GetCFSR_UFSR_INVPC() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x40000) >> 18
}
func (o *PPB_Type) SetCFSR_UFSR_INVSTATE(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x20000)|value<<17)
}
func (o *PPB_Type) GetCFSR_UFSR_INVSTATE() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x20000) >> 17
}
func (o *PPB_Type) SetCFSR_UFSR_UNDEFINSTR(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetCFSR_UFSR_UNDEFINSTR() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetCFSR_BFSR_BFARVALID(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x8000)|value<<15)
}
func (o *PPB_Type) GetCFSR_BFSR_BFARVALID() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x8000) >> 15
}
func (o *PPB_Type) SetCFSR_BFSR_LSPERR(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x2000)|value<<13)
}
func (o *PPB_Type) GetCFSR_BFSR_LSPERR() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x2000) >> 13
}
func (o *PPB_Type) SetCFSR_BFSR_STKERR(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x1000)|value<<12)
}
func (o *PPB_Type) GetCFSR_BFSR_STKERR() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x1000) >> 12
}
func (o *PPB_Type) SetCFSR_BFSR_UNSTKERR(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x800)|value<<11)
}
func (o *PPB_Type) GetCFSR_BFSR_UNSTKERR() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x800) >> 11
}
func (o *PPB_Type) SetCFSR_BFSR_IMPRECISERR(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x400)|value<<10)
}
func (o *PPB_Type) GetCFSR_BFSR_IMPRECISERR() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x400) >> 10
}
func (o *PPB_Type) SetCFSR_BFSR_PRECISERR(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x200)|value<<9)
}
func (o *PPB_Type) GetCFSR_BFSR_PRECISERR() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x200) >> 9
}
func (o *PPB_Type) SetCFSR_BFSR_IBUSERR(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0x100)|value<<8)
}
func (o *PPB_Type) GetCFSR_BFSR_IBUSERR() uint32 {
	return (volatile.LoadUint32(&o.CFSR.Reg) & 0x100) >> 8
}
func (o *PPB_Type) SetCFSR_MMFSR(value uint32) {
	volatile.StoreUint32(&o.CFSR.Reg, volatile.LoadUint32(&o.CFSR.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetCFSR_MMFSR() uint32 {
	return volatile.LoadUint32(&o.CFSR.Reg) & 0xff
}

// PPB.HFSR: Shows the cause of any HardFaults
func (o *PPB_Type) SetHFSR_DEBUGEVT(value uint32) {
	volatile.StoreUint32(&o.HFSR.Reg, volatile.LoadUint32(&o.HFSR.Reg)&^(0x80000000)|value<<31)
}
func (o *PPB_Type) GetHFSR_DEBUGEVT() uint32 {
	return (volatile.LoadUint32(&o.HFSR.Reg) & 0x80000000) >> 31
}
func (o *PPB_Type) SetHFSR_FORCED(value uint32) {
	volatile.StoreUint32(&o.HFSR.Reg, volatile.LoadUint32(&o.HFSR.Reg)&^(0x40000000)|value<<30)
}
func (o *PPB_Type) GetHFSR_FORCED() uint32 {
	return (volatile.LoadUint32(&o.HFSR.Reg) & 0x40000000) >> 30
}
func (o *PPB_Type) SetHFSR_VECTTBL(value uint32) {
	volatile.StoreUint32(&o.HFSR.Reg, volatile.LoadUint32(&o.HFSR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetHFSR_VECTTBL() uint32 {
	return (volatile.LoadUint32(&o.HFSR.Reg) & 0x2) >> 1
}

// PPB.DFSR: Shows which debug event occurred
func (o *PPB_Type) SetDFSR_EXTERNAL(value uint32) {
	volatile.StoreUint32(&o.DFSR.Reg, volatile.LoadUint32(&o.DFSR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetDFSR_EXTERNAL() uint32 {
	return (volatile.LoadUint32(&o.DFSR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetDFSR_VCATCH(value uint32) {
	volatile.StoreUint32(&o.DFSR.Reg, volatile.LoadUint32(&o.DFSR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetDFSR_VCATCH() uint32 {
	return (volatile.LoadUint32(&o.DFSR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetDFSR_DWTTRAP(value uint32) {
	volatile.StoreUint32(&o.DFSR.Reg, volatile.LoadUint32(&o.DFSR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetDFSR_DWTTRAP() uint32 {
	return (volatile.LoadUint32(&o.DFSR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetDFSR_BKPT(value uint32) {
	volatile.StoreUint32(&o.DFSR.Reg, volatile.LoadUint32(&o.DFSR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetDFSR_BKPT() uint32 {
	return (volatile.LoadUint32(&o.DFSR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetDFSR_HALTED(value uint32) {
	volatile.StoreUint32(&o.DFSR.Reg, volatile.LoadUint32(&o.DFSR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetDFSR_HALTED() uint32 {
	return volatile.LoadUint32(&o.DFSR.Reg) & 0x1
}

// PPB.MMFAR: Shows the address of the memory location that caused an MPU fault
func (o *PPB_Type) SetMMFAR(value uint32) {
	volatile.StoreUint32(&o.MMFAR.Reg, value)
}
func (o *PPB_Type) GetMMFAR() uint32 {
	return volatile.LoadUint32(&o.MMFAR.Reg)
}

// PPB.BFAR: Shows the address associated with a precise data access BusFault
func (o *PPB_Type) SetBFAR(value uint32) {
	volatile.StoreUint32(&o.BFAR.Reg, value)
}
func (o *PPB_Type) GetBFAR() uint32 {
	return volatile.LoadUint32(&o.BFAR.Reg)
}

// PPB.ID_PFR0: Gives top-level information about the instruction set supported by the PE
func (o *PPB_Type) SetID_PFR0_STATE1(value uint32) {
	volatile.StoreUint32(&o.ID_PFR0.Reg, volatile.LoadUint32(&o.ID_PFR0.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetID_PFR0_STATE1() uint32 {
	return (volatile.LoadUint32(&o.ID_PFR0.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetID_PFR0_STATE0(value uint32) {
	volatile.StoreUint32(&o.ID_PFR0.Reg, volatile.LoadUint32(&o.ID_PFR0.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetID_PFR0_STATE0() uint32 {
	return volatile.LoadUint32(&o.ID_PFR0.Reg) & 0xf
}

// PPB.ID_PFR1: Gives information about the programmers' model and Extensions support
func (o *PPB_Type) SetID_PFR1_MPROGMOD(value uint32) {
	volatile.StoreUint32(&o.ID_PFR1.Reg, volatile.LoadUint32(&o.ID_PFR1.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetID_PFR1_MPROGMOD() uint32 {
	return (volatile.LoadUint32(&o.ID_PFR1.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetID_PFR1_SECURITY(value uint32) {
	volatile.StoreUint32(&o.ID_PFR1.Reg, volatile.LoadUint32(&o.ID_PFR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetID_PFR1_SECURITY() uint32 {
	return (volatile.LoadUint32(&o.ID_PFR1.Reg) & 0xf0) >> 4
}

// PPB.ID_DFR0: Provides top level information about the debug system
func (o *PPB_Type) SetID_DFR0_MPROFDBG(value uint32) {
	volatile.StoreUint32(&o.ID_DFR0.Reg, volatile.LoadUint32(&o.ID_DFR0.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetID_DFR0_MPROFDBG() uint32 {
	return (volatile.LoadUint32(&o.ID_DFR0.Reg) & 0xf00000) >> 20
}

// PPB.ID_AFR0: Provides information about the IMPLEMENTATION DEFINED features of the PE
func (o *PPB_Type) SetID_AFR0_IMPDEF3(value uint32) {
	volatile.StoreUint32(&o.ID_AFR0.Reg, volatile.LoadUint32(&o.ID_AFR0.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetID_AFR0_IMPDEF3() uint32 {
	return (volatile.LoadUint32(&o.ID_AFR0.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetID_AFR0_IMPDEF2(value uint32) {
	volatile.StoreUint32(&o.ID_AFR0.Reg, volatile.LoadUint32(&o.ID_AFR0.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetID_AFR0_IMPDEF2() uint32 {
	return (volatile.LoadUint32(&o.ID_AFR0.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetID_AFR0_IMPDEF1(value uint32) {
	volatile.StoreUint32(&o.ID_AFR0.Reg, volatile.LoadUint32(&o.ID_AFR0.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetID_AFR0_IMPDEF1() uint32 {
	return (volatile.LoadUint32(&o.ID_AFR0.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetID_AFR0_IMPDEF0(value uint32) {
	volatile.StoreUint32(&o.ID_AFR0.Reg, volatile.LoadUint32(&o.ID_AFR0.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetID_AFR0_IMPDEF0() uint32 {
	return volatile.LoadUint32(&o.ID_AFR0.Reg) & 0xf
}

// PPB.ID_MMFR0: Provides information about the implemented memory model and memory management support
func (o *PPB_Type) SetID_MMFR0_AUXREG(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR0.Reg, volatile.LoadUint32(&o.ID_MMFR0.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetID_MMFR0_AUXREG() uint32 {
	return (volatile.LoadUint32(&o.ID_MMFR0.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetID_MMFR0_TCM(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR0.Reg, volatile.LoadUint32(&o.ID_MMFR0.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetID_MMFR0_TCM() uint32 {
	return (volatile.LoadUint32(&o.ID_MMFR0.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetID_MMFR0_SHARELVL(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR0.Reg, volatile.LoadUint32(&o.ID_MMFR0.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetID_MMFR0_SHARELVL() uint32 {
	return (volatile.LoadUint32(&o.ID_MMFR0.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetID_MMFR0_OUTERSHR(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR0.Reg, volatile.LoadUint32(&o.ID_MMFR0.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetID_MMFR0_OUTERSHR() uint32 {
	return (volatile.LoadUint32(&o.ID_MMFR0.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetID_MMFR0_PMSA(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR0.Reg, volatile.LoadUint32(&o.ID_MMFR0.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetID_MMFR0_PMSA() uint32 {
	return (volatile.LoadUint32(&o.ID_MMFR0.Reg) & 0xf0) >> 4
}

// PPB.ID_MMFR1: Provides information about the implemented memory model and memory management support
func (o *PPB_Type) SetID_MMFR1(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR1.Reg, value)
}
func (o *PPB_Type) GetID_MMFR1() uint32 {
	return volatile.LoadUint32(&o.ID_MMFR1.Reg)
}

// PPB.ID_MMFR2: Provides information about the implemented memory model and memory management support
func (o *PPB_Type) SetID_MMFR2_WFISTALL(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR2.Reg, volatile.LoadUint32(&o.ID_MMFR2.Reg)&^(0xf000000)|value<<24)
}
func (o *PPB_Type) GetID_MMFR2_WFISTALL() uint32 {
	return (volatile.LoadUint32(&o.ID_MMFR2.Reg) & 0xf000000) >> 24
}

// PPB.ID_MMFR3: Provides information about the implemented memory model and memory management support
func (o *PPB_Type) SetID_MMFR3_BPMAINT(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR3.Reg, volatile.LoadUint32(&o.ID_MMFR3.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetID_MMFR3_BPMAINT() uint32 {
	return (volatile.LoadUint32(&o.ID_MMFR3.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetID_MMFR3_CMAINTSW(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR3.Reg, volatile.LoadUint32(&o.ID_MMFR3.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetID_MMFR3_CMAINTSW() uint32 {
	return (volatile.LoadUint32(&o.ID_MMFR3.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetID_MMFR3_CMAINTVA(value uint32) {
	volatile.StoreUint32(&o.ID_MMFR3.Reg, volatile.LoadUint32(&o.ID_MMFR3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetID_MMFR3_CMAINTVA() uint32 {
	return volatile.LoadUint32(&o.ID_MMFR3.Reg) & 0xf
}

// PPB.ID_ISAR0: Provides information about the instruction set implemented by the PE
func (o *PPB_Type) SetID_ISAR0_DIVIDE(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR0.Reg, volatile.LoadUint32(&o.ID_ISAR0.Reg)&^(0xf000000)|value<<24)
}
func (o *PPB_Type) GetID_ISAR0_DIVIDE() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR0.Reg) & 0xf000000) >> 24
}
func (o *PPB_Type) SetID_ISAR0_DEBUG(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR0.Reg, volatile.LoadUint32(&o.ID_ISAR0.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetID_ISAR0_DEBUG() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR0.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetID_ISAR0_COPROC(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR0.Reg, volatile.LoadUint32(&o.ID_ISAR0.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetID_ISAR0_COPROC() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR0.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetID_ISAR0_CMPBRANCH(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR0.Reg, volatile.LoadUint32(&o.ID_ISAR0.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetID_ISAR0_CMPBRANCH() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR0.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetID_ISAR0_BITFIELD(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR0.Reg, volatile.LoadUint32(&o.ID_ISAR0.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetID_ISAR0_BITFIELD() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR0.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetID_ISAR0_BITCOUNT(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR0.Reg, volatile.LoadUint32(&o.ID_ISAR0.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetID_ISAR0_BITCOUNT() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR0.Reg) & 0xf0) >> 4
}

// PPB.ID_ISAR1: Provides information about the instruction set implemented by the PE
func (o *PPB_Type) SetID_ISAR1_INTERWORK(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR1.Reg, volatile.LoadUint32(&o.ID_ISAR1.Reg)&^(0xf000000)|value<<24)
}
func (o *PPB_Type) GetID_ISAR1_INTERWORK() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR1.Reg) & 0xf000000) >> 24
}
func (o *PPB_Type) SetID_ISAR1_IMMEDIATE(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR1.Reg, volatile.LoadUint32(&o.ID_ISAR1.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetID_ISAR1_IMMEDIATE() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR1.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetID_ISAR1_IFTHEN(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR1.Reg, volatile.LoadUint32(&o.ID_ISAR1.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetID_ISAR1_IFTHEN() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR1.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetID_ISAR1_EXTEND(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR1.Reg, volatile.LoadUint32(&o.ID_ISAR1.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetID_ISAR1_EXTEND() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR1.Reg) & 0xf000) >> 12
}

// PPB.ID_ISAR2: Provides information about the instruction set implemented by the PE
func (o *PPB_Type) SetID_ISAR2_REVERSAL(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR2.Reg, volatile.LoadUint32(&o.ID_ISAR2.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetID_ISAR2_REVERSAL() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR2.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetID_ISAR2_MULTU(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR2.Reg, volatile.LoadUint32(&o.ID_ISAR2.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetID_ISAR2_MULTU() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR2.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetID_ISAR2_MULTS(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR2.Reg, volatile.LoadUint32(&o.ID_ISAR2.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetID_ISAR2_MULTS() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR2.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetID_ISAR2_MULT(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR2.Reg, volatile.LoadUint32(&o.ID_ISAR2.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetID_ISAR2_MULT() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR2.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetID_ISAR2_MULTIACCESSINT(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR2.Reg, volatile.LoadUint32(&o.ID_ISAR2.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetID_ISAR2_MULTIACCESSINT() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR2.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetID_ISAR2_MEMHINT(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR2.Reg, volatile.LoadUint32(&o.ID_ISAR2.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetID_ISAR2_MEMHINT() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR2.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetID_ISAR2_LOADSTORE(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR2.Reg, volatile.LoadUint32(&o.ID_ISAR2.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetID_ISAR2_LOADSTORE() uint32 {
	return volatile.LoadUint32(&o.ID_ISAR2.Reg) & 0xf
}

// PPB.ID_ISAR3: Provides information about the instruction set implemented by the PE
func (o *PPB_Type) SetID_ISAR3_TRUENOP(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR3.Reg, volatile.LoadUint32(&o.ID_ISAR3.Reg)&^(0xf000000)|value<<24)
}
func (o *PPB_Type) GetID_ISAR3_TRUENOP() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR3.Reg) & 0xf000000) >> 24
}
func (o *PPB_Type) SetID_ISAR3_T32COPY(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR3.Reg, volatile.LoadUint32(&o.ID_ISAR3.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetID_ISAR3_T32COPY() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR3.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetID_ISAR3_TABBRANCH(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR3.Reg, volatile.LoadUint32(&o.ID_ISAR3.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetID_ISAR3_TABBRANCH() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR3.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetID_ISAR3_SYNCHPRIM(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR3.Reg, volatile.LoadUint32(&o.ID_ISAR3.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetID_ISAR3_SYNCHPRIM() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR3.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetID_ISAR3_SVC(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR3.Reg, volatile.LoadUint32(&o.ID_ISAR3.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetID_ISAR3_SVC() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR3.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetID_ISAR3_SIMD(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR3.Reg, volatile.LoadUint32(&o.ID_ISAR3.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetID_ISAR3_SIMD() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR3.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetID_ISAR3_SATURATE(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR3.Reg, volatile.LoadUint32(&o.ID_ISAR3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetID_ISAR3_SATURATE() uint32 {
	return volatile.LoadUint32(&o.ID_ISAR3.Reg) & 0xf
}

// PPB.ID_ISAR4: Provides information about the instruction set implemented by the PE
func (o *PPB_Type) SetID_ISAR4_PSR_M(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR4.Reg, volatile.LoadUint32(&o.ID_ISAR4.Reg)&^(0xf000000)|value<<24)
}
func (o *PPB_Type) GetID_ISAR4_PSR_M() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR4.Reg) & 0xf000000) >> 24
}
func (o *PPB_Type) SetID_ISAR4_SYNCPRIM_FRAC(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR4.Reg, volatile.LoadUint32(&o.ID_ISAR4.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetID_ISAR4_SYNCPRIM_FRAC() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR4.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetID_ISAR4_BARRIER(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR4.Reg, volatile.LoadUint32(&o.ID_ISAR4.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetID_ISAR4_BARRIER() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR4.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetID_ISAR4_WRITEBACK(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR4.Reg, volatile.LoadUint32(&o.ID_ISAR4.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetID_ISAR4_WRITEBACK() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR4.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetID_ISAR4_WITHSHIFTS(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR4.Reg, volatile.LoadUint32(&o.ID_ISAR4.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetID_ISAR4_WITHSHIFTS() uint32 {
	return (volatile.LoadUint32(&o.ID_ISAR4.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetID_ISAR4_UNPRIV(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR4.Reg, volatile.LoadUint32(&o.ID_ISAR4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetID_ISAR4_UNPRIV() uint32 {
	return volatile.LoadUint32(&o.ID_ISAR4.Reg) & 0xf
}

// PPB.ID_ISAR5: Provides information about the instruction set implemented by the PE
func (o *PPB_Type) SetID_ISAR5(value uint32) {
	volatile.StoreUint32(&o.ID_ISAR5.Reg, value)
}
func (o *PPB_Type) GetID_ISAR5() uint32 {
	return volatile.LoadUint32(&o.ID_ISAR5.Reg)
}

// PPB.CTR: Provides information about the architecture of the caches. CTR is RES0 if CLIDR is zero.
func (o *PPB_Type) SetCTR_RES1(value uint32) {
	volatile.StoreUint32(&o.CTR.Reg, volatile.LoadUint32(&o.CTR.Reg)&^(0x80000000)|value<<31)
}
func (o *PPB_Type) GetCTR_RES1() uint32 {
	return (volatile.LoadUint32(&o.CTR.Reg) & 0x80000000) >> 31
}
func (o *PPB_Type) SetCTR_CWG(value uint32) {
	volatile.StoreUint32(&o.CTR.Reg, volatile.LoadUint32(&o.CTR.Reg)&^(0xf000000)|value<<24)
}
func (o *PPB_Type) GetCTR_CWG() uint32 {
	return (volatile.LoadUint32(&o.CTR.Reg) & 0xf000000) >> 24
}
func (o *PPB_Type) SetCTR_ERG(value uint32) {
	volatile.StoreUint32(&o.CTR.Reg, volatile.LoadUint32(&o.CTR.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetCTR_ERG() uint32 {
	return (volatile.LoadUint32(&o.CTR.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetCTR_DMINLINE(value uint32) {
	volatile.StoreUint32(&o.CTR.Reg, volatile.LoadUint32(&o.CTR.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetCTR_DMINLINE() uint32 {
	return (volatile.LoadUint32(&o.CTR.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetCTR_RES1_1(value uint32) {
	volatile.StoreUint32(&o.CTR.Reg, volatile.LoadUint32(&o.CTR.Reg)&^(0xc000)|value<<14)
}
func (o *PPB_Type) GetCTR_RES1_1() uint32 {
	return (volatile.LoadUint32(&o.CTR.Reg) & 0xc000) >> 14
}
func (o *PPB_Type) SetCTR_IMINLINE(value uint32) {
	volatile.StoreUint32(&o.CTR.Reg, volatile.LoadUint32(&o.CTR.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTR_IMINLINE() uint32 {
	return volatile.LoadUint32(&o.CTR.Reg) & 0xf
}

// PPB.CPACR: Specifies the access privileges for coprocessors and the FP Extension
func (o *PPB_Type) SetCPACR_CP11(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0xc00000)|value<<22)
}
func (o *PPB_Type) GetCPACR_CP11() uint32 {
	return (volatile.LoadUint32(&o.CPACR.Reg) & 0xc00000) >> 22
}
func (o *PPB_Type) SetCPACR_CP10(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0x300000)|value<<20)
}
func (o *PPB_Type) GetCPACR_CP10() uint32 {
	return (volatile.LoadUint32(&o.CPACR.Reg) & 0x300000) >> 20
}
func (o *PPB_Type) SetCPACR_CP7(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0xc000)|value<<14)
}
func (o *PPB_Type) GetCPACR_CP7() uint32 {
	return (volatile.LoadUint32(&o.CPACR.Reg) & 0xc000) >> 14
}
func (o *PPB_Type) SetCPACR_CP6(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0x3000)|value<<12)
}
func (o *PPB_Type) GetCPACR_CP6() uint32 {
	return (volatile.LoadUint32(&o.CPACR.Reg) & 0x3000) >> 12
}
func (o *PPB_Type) SetCPACR_CP5(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0xc00)|value<<10)
}
func (o *PPB_Type) GetCPACR_CP5() uint32 {
	return (volatile.LoadUint32(&o.CPACR.Reg) & 0xc00) >> 10
}
func (o *PPB_Type) SetCPACR_CP4(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0x300)|value<<8)
}
func (o *PPB_Type) GetCPACR_CP4() uint32 {
	return (volatile.LoadUint32(&o.CPACR.Reg) & 0x300) >> 8
}
func (o *PPB_Type) SetCPACR_CP3(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0xc0)|value<<6)
}
func (o *PPB_Type) GetCPACR_CP3() uint32 {
	return (volatile.LoadUint32(&o.CPACR.Reg) & 0xc0) >> 6
}
func (o *PPB_Type) SetCPACR_CP2(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0x30)|value<<4)
}
func (o *PPB_Type) GetCPACR_CP2() uint32 {
	return (volatile.LoadUint32(&o.CPACR.Reg) & 0x30) >> 4
}
func (o *PPB_Type) SetCPACR_CP1(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0xc)|value<<2)
}
func (o *PPB_Type) GetCPACR_CP1() uint32 {
	return (volatile.LoadUint32(&o.CPACR.Reg) & 0xc) >> 2
}
func (o *PPB_Type) SetCPACR_CP0(value uint32) {
	volatile.StoreUint32(&o.CPACR.Reg, volatile.LoadUint32(&o.CPACR.Reg)&^(0x3)|value)
}
func (o *PPB_Type) GetCPACR_CP0() uint32 {
	return volatile.LoadUint32(&o.CPACR.Reg) & 0x3
}

// PPB.NSACR: Defines the Non-secure access permissions for both the FP Extension and coprocessors CP0 to CP7
func (o *PPB_Type) SetNSACR_CP11(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x800)|value<<11)
}
func (o *PPB_Type) GetNSACR_CP11() uint32 {
	return (volatile.LoadUint32(&o.NSACR.Reg) & 0x800) >> 11
}
func (o *PPB_Type) SetNSACR_CP10(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x400)|value<<10)
}
func (o *PPB_Type) GetNSACR_CP10() uint32 {
	return (volatile.LoadUint32(&o.NSACR.Reg) & 0x400) >> 10
}
func (o *PPB_Type) SetNSACR_CP7(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x80)|value<<7)
}
func (o *PPB_Type) GetNSACR_CP7() uint32 {
	return (volatile.LoadUint32(&o.NSACR.Reg) & 0x80) >> 7
}
func (o *PPB_Type) SetNSACR_CP6(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x40)|value<<6)
}
func (o *PPB_Type) GetNSACR_CP6() uint32 {
	return (volatile.LoadUint32(&o.NSACR.Reg) & 0x40) >> 6
}
func (o *PPB_Type) SetNSACR_CP5(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x20)|value<<5)
}
func (o *PPB_Type) GetNSACR_CP5() uint32 {
	return (volatile.LoadUint32(&o.NSACR.Reg) & 0x20) >> 5
}
func (o *PPB_Type) SetNSACR_CP4(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetNSACR_CP4() uint32 {
	return (volatile.LoadUint32(&o.NSACR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetNSACR_CP3(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetNSACR_CP3() uint32 {
	return (volatile.LoadUint32(&o.NSACR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetNSACR_CP2(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetNSACR_CP2() uint32 {
	return (volatile.LoadUint32(&o.NSACR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetNSACR_CP1(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetNSACR_CP1() uint32 {
	return (volatile.LoadUint32(&o.NSACR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetNSACR_CP0(value uint32) {
	volatile.StoreUint32(&o.NSACR.Reg, volatile.LoadUint32(&o.NSACR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetNSACR_CP0() uint32 {
	return volatile.LoadUint32(&o.NSACR.Reg) & 0x1
}

// PPB.MPU_TYPE: The MPU Type Register indicates how many regions the MPU `FTSSS supports
func (o *PPB_Type) SetMPU_TYPE_DREGION(value uint32) {
	volatile.StoreUint32(&o.MPU_TYPE.Reg, volatile.LoadUint32(&o.MPU_TYPE.Reg)&^(0xff00)|value<<8)
}
func (o *PPB_Type) GetMPU_TYPE_DREGION() uint32 {
	return (volatile.LoadUint32(&o.MPU_TYPE.Reg) & 0xff00) >> 8
}
func (o *PPB_Type) SetMPU_TYPE_SEPARATE(value uint32) {
	volatile.StoreUint32(&o.MPU_TYPE.Reg, volatile.LoadUint32(&o.MPU_TYPE.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_TYPE_SEPARATE() uint32 {
	return volatile.LoadUint32(&o.MPU_TYPE.Reg) & 0x1
}

// PPB.MPU_CTRL: Enables the MPU and, when the MPU is enabled, controls whether the default memory map is enabled as a background region for privileged accesses, and whether the MPU is enabled for HardFaults, NMIs, and exception handlers when FAULTMASK is set to 1
func (o *PPB_Type) SetMPU_CTRL_PRIVDEFENA(value uint32) {
	volatile.StoreUint32(&o.MPU_CTRL.Reg, volatile.LoadUint32(&o.MPU_CTRL.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetMPU_CTRL_PRIVDEFENA() uint32 {
	return (volatile.LoadUint32(&o.MPU_CTRL.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetMPU_CTRL_HFNMIENA(value uint32) {
	volatile.StoreUint32(&o.MPU_CTRL.Reg, volatile.LoadUint32(&o.MPU_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetMPU_CTRL_HFNMIENA() uint32 {
	return (volatile.LoadUint32(&o.MPU_CTRL.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetMPU_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.MPU_CTRL.Reg, volatile.LoadUint32(&o.MPU_CTRL.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_CTRL_ENABLE() uint32 {
	return volatile.LoadUint32(&o.MPU_CTRL.Reg) & 0x1
}

// PPB.MPU_RNR: Selects the region currently accessed by MPU_RBAR and MPU_RLAR
func (o *PPB_Type) SetMPU_RNR_REGION(value uint32) {
	volatile.StoreUint32(&o.MPU_RNR.Reg, volatile.LoadUint32(&o.MPU_RNR.Reg)&^(0x7)|value)
}
func (o *PPB_Type) GetMPU_RNR_REGION() uint32 {
	return volatile.LoadUint32(&o.MPU_RNR.Reg) & 0x7
}

// PPB.MPU_RBAR: Provides indirect read and write access to the base address of the currently selected MPU region `FTSSS
func (o *PPB_Type) SetMPU_RBAR_BASE(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR.Reg, volatile.LoadUint32(&o.MPU_RBAR.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetMPU_RBAR_BASE() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR.Reg) & 0xffffffe0) >> 5
}
func (o *PPB_Type) SetMPU_RBAR_SH(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR.Reg, volatile.LoadUint32(&o.MPU_RBAR.Reg)&^(0x18)|value<<3)
}
func (o *PPB_Type) GetMPU_RBAR_SH() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR.Reg) & 0x18) >> 3
}
func (o *PPB_Type) SetMPU_RBAR_AP(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR.Reg, volatile.LoadUint32(&o.MPU_RBAR.Reg)&^(0x6)|value<<1)
}
func (o *PPB_Type) GetMPU_RBAR_AP() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR.Reg) & 0x6) >> 1
}
func (o *PPB_Type) SetMPU_RBAR_XN(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR.Reg, volatile.LoadUint32(&o.MPU_RBAR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_RBAR_XN() uint32 {
	return volatile.LoadUint32(&o.MPU_RBAR.Reg) & 0x1
}

// PPB.MPU_RLAR: Provides indirect read and write access to the limit address of the currently selected MPU region `FTSSS
func (o *PPB_Type) SetMPU_RLAR_LIMIT(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR.Reg, volatile.LoadUint32(&o.MPU_RLAR.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetMPU_RLAR_LIMIT() uint32 {
	return (volatile.LoadUint32(&o.MPU_RLAR.Reg) & 0xffffffe0) >> 5
}
func (o *PPB_Type) SetMPU_RLAR_ATTRINDX(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR.Reg, volatile.LoadUint32(&o.MPU_RLAR.Reg)&^(0xe)|value<<1)
}
func (o *PPB_Type) GetMPU_RLAR_ATTRINDX() uint32 {
	return (volatile.LoadUint32(&o.MPU_RLAR.Reg) & 0xe) >> 1
}
func (o *PPB_Type) SetMPU_RLAR_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR.Reg, volatile.LoadUint32(&o.MPU_RLAR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_RLAR_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_RLAR.Reg) & 0x1
}

// PPB.MPU_RBAR_A1: Provides indirect read and write access to the base address of the MPU region selected by MPU_RNR[7:2]:(1[1:0]) `FTSSS
func (o *PPB_Type) SetMPU_RBAR_A1_BASE(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A1.Reg, volatile.LoadUint32(&o.MPU_RBAR_A1.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetMPU_RBAR_A1_BASE() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR_A1.Reg) & 0xffffffe0) >> 5
}
func (o *PPB_Type) SetMPU_RBAR_A1_SH(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A1.Reg, volatile.LoadUint32(&o.MPU_RBAR_A1.Reg)&^(0x18)|value<<3)
}
func (o *PPB_Type) GetMPU_RBAR_A1_SH() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR_A1.Reg) & 0x18) >> 3
}
func (o *PPB_Type) SetMPU_RBAR_A1_AP(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A1.Reg, volatile.LoadUint32(&o.MPU_RBAR_A1.Reg)&^(0x6)|value<<1)
}
func (o *PPB_Type) GetMPU_RBAR_A1_AP() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR_A1.Reg) & 0x6) >> 1
}
func (o *PPB_Type) SetMPU_RBAR_A1_XN(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A1.Reg, volatile.LoadUint32(&o.MPU_RBAR_A1.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_RBAR_A1_XN() uint32 {
	return volatile.LoadUint32(&o.MPU_RBAR_A1.Reg) & 0x1
}

// PPB.MPU_RLAR_A1: Provides indirect read and write access to the limit address of the currently selected MPU region selected by MPU_RNR[7:2]:(1[1:0]) `FTSSS
func (o *PPB_Type) SetMPU_RLAR_A1_LIMIT(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR_A1.Reg, volatile.LoadUint32(&o.MPU_RLAR_A1.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetMPU_RLAR_A1_LIMIT() uint32 {
	return (volatile.LoadUint32(&o.MPU_RLAR_A1.Reg) & 0xffffffe0) >> 5
}
func (o *PPB_Type) SetMPU_RLAR_A1_ATTRINDX(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR_A1.Reg, volatile.LoadUint32(&o.MPU_RLAR_A1.Reg)&^(0xe)|value<<1)
}
func (o *PPB_Type) GetMPU_RLAR_A1_ATTRINDX() uint32 {
	return (volatile.LoadUint32(&o.MPU_RLAR_A1.Reg) & 0xe) >> 1
}
func (o *PPB_Type) SetMPU_RLAR_A1_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR_A1.Reg, volatile.LoadUint32(&o.MPU_RLAR_A1.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_RLAR_A1_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_RLAR_A1.Reg) & 0x1
}

// PPB.MPU_RBAR_A2: Provides indirect read and write access to the base address of the MPU region selected by MPU_RNR[7:2]:(2[1:0]) `FTSSS
func (o *PPB_Type) SetMPU_RBAR_A2_BASE(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A2.Reg, volatile.LoadUint32(&o.MPU_RBAR_A2.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetMPU_RBAR_A2_BASE() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR_A2.Reg) & 0xffffffe0) >> 5
}
func (o *PPB_Type) SetMPU_RBAR_A2_SH(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A2.Reg, volatile.LoadUint32(&o.MPU_RBAR_A2.Reg)&^(0x18)|value<<3)
}
func (o *PPB_Type) GetMPU_RBAR_A2_SH() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR_A2.Reg) & 0x18) >> 3
}
func (o *PPB_Type) SetMPU_RBAR_A2_AP(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A2.Reg, volatile.LoadUint32(&o.MPU_RBAR_A2.Reg)&^(0x6)|value<<1)
}
func (o *PPB_Type) GetMPU_RBAR_A2_AP() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR_A2.Reg) & 0x6) >> 1
}
func (o *PPB_Type) SetMPU_RBAR_A2_XN(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A2.Reg, volatile.LoadUint32(&o.MPU_RBAR_A2.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_RBAR_A2_XN() uint32 {
	return volatile.LoadUint32(&o.MPU_RBAR_A2.Reg) & 0x1
}

// PPB.MPU_RLAR_A2: Provides indirect read and write access to the limit address of the currently selected MPU region selected by MPU_RNR[7:2]:(2[1:0]) `FTSSS
func (o *PPB_Type) SetMPU_RLAR_A2_LIMIT(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR_A2.Reg, volatile.LoadUint32(&o.MPU_RLAR_A2.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetMPU_RLAR_A2_LIMIT() uint32 {
	return (volatile.LoadUint32(&o.MPU_RLAR_A2.Reg) & 0xffffffe0) >> 5
}
func (o *PPB_Type) SetMPU_RLAR_A2_ATTRINDX(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR_A2.Reg, volatile.LoadUint32(&o.MPU_RLAR_A2.Reg)&^(0xe)|value<<1)
}
func (o *PPB_Type) GetMPU_RLAR_A2_ATTRINDX() uint32 {
	return (volatile.LoadUint32(&o.MPU_RLAR_A2.Reg) & 0xe) >> 1
}
func (o *PPB_Type) SetMPU_RLAR_A2_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR_A2.Reg, volatile.LoadUint32(&o.MPU_RLAR_A2.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_RLAR_A2_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_RLAR_A2.Reg) & 0x1
}

// PPB.MPU_RBAR_A3: Provides indirect read and write access to the base address of the MPU region selected by MPU_RNR[7:2]:(3[1:0]) `FTSSS
func (o *PPB_Type) SetMPU_RBAR_A3_BASE(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A3.Reg, volatile.LoadUint32(&o.MPU_RBAR_A3.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetMPU_RBAR_A3_BASE() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR_A3.Reg) & 0xffffffe0) >> 5
}
func (o *PPB_Type) SetMPU_RBAR_A3_SH(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A3.Reg, volatile.LoadUint32(&o.MPU_RBAR_A3.Reg)&^(0x18)|value<<3)
}
func (o *PPB_Type) GetMPU_RBAR_A3_SH() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR_A3.Reg) & 0x18) >> 3
}
func (o *PPB_Type) SetMPU_RBAR_A3_AP(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A3.Reg, volatile.LoadUint32(&o.MPU_RBAR_A3.Reg)&^(0x6)|value<<1)
}
func (o *PPB_Type) GetMPU_RBAR_A3_AP() uint32 {
	return (volatile.LoadUint32(&o.MPU_RBAR_A3.Reg) & 0x6) >> 1
}
func (o *PPB_Type) SetMPU_RBAR_A3_XN(value uint32) {
	volatile.StoreUint32(&o.MPU_RBAR_A3.Reg, volatile.LoadUint32(&o.MPU_RBAR_A3.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_RBAR_A3_XN() uint32 {
	return volatile.LoadUint32(&o.MPU_RBAR_A3.Reg) & 0x1
}

// PPB.MPU_RLAR_A3: Provides indirect read and write access to the limit address of the currently selected MPU region selected by MPU_RNR[7:2]:(3[1:0]) `FTSSS
func (o *PPB_Type) SetMPU_RLAR_A3_LIMIT(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR_A3.Reg, volatile.LoadUint32(&o.MPU_RLAR_A3.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetMPU_RLAR_A3_LIMIT() uint32 {
	return (volatile.LoadUint32(&o.MPU_RLAR_A3.Reg) & 0xffffffe0) >> 5
}
func (o *PPB_Type) SetMPU_RLAR_A3_ATTRINDX(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR_A3.Reg, volatile.LoadUint32(&o.MPU_RLAR_A3.Reg)&^(0xe)|value<<1)
}
func (o *PPB_Type) GetMPU_RLAR_A3_ATTRINDX() uint32 {
	return (volatile.LoadUint32(&o.MPU_RLAR_A3.Reg) & 0xe) >> 1
}
func (o *PPB_Type) SetMPU_RLAR_A3_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_RLAR_A3.Reg, volatile.LoadUint32(&o.MPU_RLAR_A3.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetMPU_RLAR_A3_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_RLAR_A3.Reg) & 0x1
}

// PPB.MPU_MAIR0: Along with MPU_MAIR1, provides the memory attribute encodings corresponding to the AttrIndex values
func (o *PPB_Type) SetMPU_MAIR0_ATTR3(value uint32) {
	volatile.StoreUint32(&o.MPU_MAIR0.Reg, volatile.LoadUint32(&o.MPU_MAIR0.Reg)&^(0xff000000)|value<<24)
}
func (o *PPB_Type) GetMPU_MAIR0_ATTR3() uint32 {
	return (volatile.LoadUint32(&o.MPU_MAIR0.Reg) & 0xff000000) >> 24
}
func (o *PPB_Type) SetMPU_MAIR0_ATTR2(value uint32) {
	volatile.StoreUint32(&o.MPU_MAIR0.Reg, volatile.LoadUint32(&o.MPU_MAIR0.Reg)&^(0xff0000)|value<<16)
}
func (o *PPB_Type) GetMPU_MAIR0_ATTR2() uint32 {
	return (volatile.LoadUint32(&o.MPU_MAIR0.Reg) & 0xff0000) >> 16
}
func (o *PPB_Type) SetMPU_MAIR0_ATTR1(value uint32) {
	volatile.StoreUint32(&o.MPU_MAIR0.Reg, volatile.LoadUint32(&o.MPU_MAIR0.Reg)&^(0xff00)|value<<8)
}
func (o *PPB_Type) GetMPU_MAIR0_ATTR1() uint32 {
	return (volatile.LoadUint32(&o.MPU_MAIR0.Reg) & 0xff00) >> 8
}
func (o *PPB_Type) SetMPU_MAIR0_ATTR0(value uint32) {
	volatile.StoreUint32(&o.MPU_MAIR0.Reg, volatile.LoadUint32(&o.MPU_MAIR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetMPU_MAIR0_ATTR0() uint32 {
	return volatile.LoadUint32(&o.MPU_MAIR0.Reg) & 0xff
}

// PPB.MPU_MAIR1: Along with MPU_MAIR0, provides the memory attribute encodings corresponding to the AttrIndex values
func (o *PPB_Type) SetMPU_MAIR1_ATTR7(value uint32) {
	volatile.StoreUint32(&o.MPU_MAIR1.Reg, volatile.LoadUint32(&o.MPU_MAIR1.Reg)&^(0xff000000)|value<<24)
}
func (o *PPB_Type) GetMPU_MAIR1_ATTR7() uint32 {
	return (volatile.LoadUint32(&o.MPU_MAIR1.Reg) & 0xff000000) >> 24
}
func (o *PPB_Type) SetMPU_MAIR1_ATTR6(value uint32) {
	volatile.StoreUint32(&o.MPU_MAIR1.Reg, volatile.LoadUint32(&o.MPU_MAIR1.Reg)&^(0xff0000)|value<<16)
}
func (o *PPB_Type) GetMPU_MAIR1_ATTR6() uint32 {
	return (volatile.LoadUint32(&o.MPU_MAIR1.Reg) & 0xff0000) >> 16
}
func (o *PPB_Type) SetMPU_MAIR1_ATTR5(value uint32) {
	volatile.StoreUint32(&o.MPU_MAIR1.Reg, volatile.LoadUint32(&o.MPU_MAIR1.Reg)&^(0xff00)|value<<8)
}
func (o *PPB_Type) GetMPU_MAIR1_ATTR5() uint32 {
	return (volatile.LoadUint32(&o.MPU_MAIR1.Reg) & 0xff00) >> 8
}
func (o *PPB_Type) SetMPU_MAIR1_ATTR4(value uint32) {
	volatile.StoreUint32(&o.MPU_MAIR1.Reg, volatile.LoadUint32(&o.MPU_MAIR1.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetMPU_MAIR1_ATTR4() uint32 {
	return volatile.LoadUint32(&o.MPU_MAIR1.Reg) & 0xff
}

// PPB.SAU_CTRL: Allows enabling of the Security Attribution Unit
func (o *PPB_Type) SetSAU_CTRL_ALLNS(value uint32) {
	volatile.StoreUint32(&o.SAU_CTRL.Reg, volatile.LoadUint32(&o.SAU_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetSAU_CTRL_ALLNS() uint32 {
	return (volatile.LoadUint32(&o.SAU_CTRL.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetSAU_CTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.SAU_CTRL.Reg, volatile.LoadUint32(&o.SAU_CTRL.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetSAU_CTRL_ENABLE() uint32 {
	return volatile.LoadUint32(&o.SAU_CTRL.Reg) & 0x1
}

// PPB.SAU_TYPE: Indicates the number of regions implemented by the Security Attribution Unit
func (o *PPB_Type) SetSAU_TYPE_SREGION(value uint32) {
	volatile.StoreUint32(&o.SAU_TYPE.Reg, volatile.LoadUint32(&o.SAU_TYPE.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetSAU_TYPE_SREGION() uint32 {
	return volatile.LoadUint32(&o.SAU_TYPE.Reg) & 0xff
}

// PPB.SAU_RNR: Selects the region currently accessed by SAU_RBAR and SAU_RLAR
func (o *PPB_Type) SetSAU_RNR_REGION(value uint32) {
	volatile.StoreUint32(&o.SAU_RNR.Reg, volatile.LoadUint32(&o.SAU_RNR.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetSAU_RNR_REGION() uint32 {
	return volatile.LoadUint32(&o.SAU_RNR.Reg) & 0xff
}

// PPB.SAU_RBAR: Provides indirect read and write access to the base address of the currently selected SAU region
func (o *PPB_Type) SetSAU_RBAR_BADDR(value uint32) {
	volatile.StoreUint32(&o.SAU_RBAR.Reg, volatile.LoadUint32(&o.SAU_RBAR.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetSAU_RBAR_BADDR() uint32 {
	return (volatile.LoadUint32(&o.SAU_RBAR.Reg) & 0xffffffe0) >> 5
}

// PPB.SAU_RLAR: Provides indirect read and write access to the limit address of the currently selected SAU region
func (o *PPB_Type) SetSAU_RLAR_LADDR(value uint32) {
	volatile.StoreUint32(&o.SAU_RLAR.Reg, volatile.LoadUint32(&o.SAU_RLAR.Reg)&^(0xffffffe0)|value<<5)
}
func (o *PPB_Type) GetSAU_RLAR_LADDR() uint32 {
	return (volatile.LoadUint32(&o.SAU_RLAR.Reg) & 0xffffffe0) >> 5
}
func (o *PPB_Type) SetSAU_RLAR_NSC(value uint32) {
	volatile.StoreUint32(&o.SAU_RLAR.Reg, volatile.LoadUint32(&o.SAU_RLAR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetSAU_RLAR_NSC() uint32 {
	return (volatile.LoadUint32(&o.SAU_RLAR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetSAU_RLAR_ENABLE(value uint32) {
	volatile.StoreUint32(&o.SAU_RLAR.Reg, volatile.LoadUint32(&o.SAU_RLAR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetSAU_RLAR_ENABLE() uint32 {
	return volatile.LoadUint32(&o.SAU_RLAR.Reg) & 0x1
}

// PPB.SFSR: Provides information about any security related faults
func (o *PPB_Type) SetSFSR_LSERR(value uint32) {
	volatile.StoreUint32(&o.SFSR.Reg, volatile.LoadUint32(&o.SFSR.Reg)&^(0x80)|value<<7)
}
func (o *PPB_Type) GetSFSR_LSERR() uint32 {
	return (volatile.LoadUint32(&o.SFSR.Reg) & 0x80) >> 7
}
func (o *PPB_Type) SetSFSR_SFARVALID(value uint32) {
	volatile.StoreUint32(&o.SFSR.Reg, volatile.LoadUint32(&o.SFSR.Reg)&^(0x40)|value<<6)
}
func (o *PPB_Type) GetSFSR_SFARVALID() uint32 {
	return (volatile.LoadUint32(&o.SFSR.Reg) & 0x40) >> 6
}
func (o *PPB_Type) SetSFSR_LSPERR(value uint32) {
	volatile.StoreUint32(&o.SFSR.Reg, volatile.LoadUint32(&o.SFSR.Reg)&^(0x20)|value<<5)
}
func (o *PPB_Type) GetSFSR_LSPERR() uint32 {
	return (volatile.LoadUint32(&o.SFSR.Reg) & 0x20) >> 5
}
func (o *PPB_Type) SetSFSR_INVTRAN(value uint32) {
	volatile.StoreUint32(&o.SFSR.Reg, volatile.LoadUint32(&o.SFSR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetSFSR_INVTRAN() uint32 {
	return (volatile.LoadUint32(&o.SFSR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetSFSR_AUVIOL(value uint32) {
	volatile.StoreUint32(&o.SFSR.Reg, volatile.LoadUint32(&o.SFSR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetSFSR_AUVIOL() uint32 {
	return (volatile.LoadUint32(&o.SFSR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetSFSR_INVER(value uint32) {
	volatile.StoreUint32(&o.SFSR.Reg, volatile.LoadUint32(&o.SFSR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetSFSR_INVER() uint32 {
	return (volatile.LoadUint32(&o.SFSR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetSFSR_INVIS(value uint32) {
	volatile.StoreUint32(&o.SFSR.Reg, volatile.LoadUint32(&o.SFSR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetSFSR_INVIS() uint32 {
	return (volatile.LoadUint32(&o.SFSR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetSFSR_INVEP(value uint32) {
	volatile.StoreUint32(&o.SFSR.Reg, volatile.LoadUint32(&o.SFSR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetSFSR_INVEP() uint32 {
	return volatile.LoadUint32(&o.SFSR.Reg) & 0x1
}

// PPB.SFAR: Shows the address of the memory location that caused a Security violation
func (o *PPB_Type) SetSFAR(value uint32) {
	volatile.StoreUint32(&o.SFAR.Reg, value)
}
func (o *PPB_Type) GetSFAR() uint32 {
	return volatile.LoadUint32(&o.SFAR.Reg)
}

// PPB.DHCSR: Controls halting debug
func (o *PPB_Type) SetDHCSR_S_RESTART_ST(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x4000000)|value<<26)
}
func (o *PPB_Type) GetDHCSR_S_RESTART_ST() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x4000000) >> 26
}
func (o *PPB_Type) SetDHCSR_S_RESET_ST(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x2000000)|value<<25)
}
func (o *PPB_Type) GetDHCSR_S_RESET_ST() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x2000000) >> 25
}
func (o *PPB_Type) SetDHCSR_S_RETIRE_ST(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetDHCSR_S_RETIRE_ST() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetDHCSR_S_SDE(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetDHCSR_S_SDE() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetDHCSR_S_LOCKUP(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x80000)|value<<19)
}
func (o *PPB_Type) GetDHCSR_S_LOCKUP() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x80000) >> 19
}
func (o *PPB_Type) SetDHCSR_S_SLEEP(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x40000)|value<<18)
}
func (o *PPB_Type) GetDHCSR_S_SLEEP() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x40000) >> 18
}
func (o *PPB_Type) SetDHCSR_S_HALT(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x20000)|value<<17)
}
func (o *PPB_Type) GetDHCSR_S_HALT() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x20000) >> 17
}
func (o *PPB_Type) SetDHCSR_S_REGRDY(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetDHCSR_S_REGRDY() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetDHCSR_C_SNAPSTALL(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x20)|value<<5)
}
func (o *PPB_Type) GetDHCSR_C_SNAPSTALL() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x20) >> 5
}
func (o *PPB_Type) SetDHCSR_C_MASKINTS(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetDHCSR_C_MASKINTS() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetDHCSR_C_STEP(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetDHCSR_C_STEP() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetDHCSR_C_HALT(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetDHCSR_C_HALT() uint32 {
	return (volatile.LoadUint32(&o.DHCSR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetDHCSR_C_DEBUGEN(value uint32) {
	volatile.StoreUint32(&o.DHCSR.Reg, volatile.LoadUint32(&o.DHCSR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetDHCSR_C_DEBUGEN() uint32 {
	return volatile.LoadUint32(&o.DHCSR.Reg) & 0x1
}

// PPB.DCRSR: With the DCRDR, provides debug access to the general-purpose registers, special-purpose registers, and the FP extension registers. A write to the DCRSR specifies the register to transfer, whether the transfer is a read or write, and starts the transfer
func (o *PPB_Type) SetDCRSR_REGWNR(value uint32) {
	volatile.StoreUint32(&o.DCRSR.Reg, volatile.LoadUint32(&o.DCRSR.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetDCRSR_REGWNR() uint32 {
	return (volatile.LoadUint32(&o.DCRSR.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetDCRSR_REGSEL(value uint32) {
	volatile.StoreUint32(&o.DCRSR.Reg, volatile.LoadUint32(&o.DCRSR.Reg)&^(0x7f)|value)
}
func (o *PPB_Type) GetDCRSR_REGSEL() uint32 {
	return volatile.LoadUint32(&o.DCRSR.Reg) & 0x7f
}

// PPB.DCRDR: With the DCRSR, provides debug access to the general-purpose registers, special-purpose registers, and the FP Extension registers. If the Main Extension is implemented, it can also be used for message passing between an external debugger and a debug agent running on the PE
func (o *PPB_Type) SetDCRDR(value uint32) {
	volatile.StoreUint32(&o.DCRDR.Reg, value)
}
func (o *PPB_Type) GetDCRDR() uint32 {
	return volatile.LoadUint32(&o.DCRDR.Reg)
}

// PPB.DEMCR: Manages vector catch behavior and DebugMonitor handling when debugging
func (o *PPB_Type) SetDEMCR_TRCENA(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetDEMCR_TRCENA() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetDEMCR_SDME(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetDEMCR_SDME() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetDEMCR_MON_REQ(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x80000)|value<<19)
}
func (o *PPB_Type) GetDEMCR_MON_REQ() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x80000) >> 19
}
func (o *PPB_Type) SetDEMCR_MON_STEP(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x40000)|value<<18)
}
func (o *PPB_Type) GetDEMCR_MON_STEP() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x40000) >> 18
}
func (o *PPB_Type) SetDEMCR_MON_PEND(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x20000)|value<<17)
}
func (o *PPB_Type) GetDEMCR_MON_PEND() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x20000) >> 17
}
func (o *PPB_Type) SetDEMCR_MON_EN(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetDEMCR_MON_EN() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetDEMCR_VC_SFERR(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x800)|value<<11)
}
func (o *PPB_Type) GetDEMCR_VC_SFERR() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x800) >> 11
}
func (o *PPB_Type) SetDEMCR_VC_HARDERR(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x400)|value<<10)
}
func (o *PPB_Type) GetDEMCR_VC_HARDERR() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x400) >> 10
}
func (o *PPB_Type) SetDEMCR_VC_INTERR(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x200)|value<<9)
}
func (o *PPB_Type) GetDEMCR_VC_INTERR() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x200) >> 9
}
func (o *PPB_Type) SetDEMCR_VC_BUSERR(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x100)|value<<8)
}
func (o *PPB_Type) GetDEMCR_VC_BUSERR() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x100) >> 8
}
func (o *PPB_Type) SetDEMCR_VC_STATERR(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x80)|value<<7)
}
func (o *PPB_Type) GetDEMCR_VC_STATERR() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x80) >> 7
}
func (o *PPB_Type) SetDEMCR_VC_CHKERR(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x40)|value<<6)
}
func (o *PPB_Type) GetDEMCR_VC_CHKERR() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x40) >> 6
}
func (o *PPB_Type) SetDEMCR_VC_NOCPERR(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x20)|value<<5)
}
func (o *PPB_Type) GetDEMCR_VC_NOCPERR() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x20) >> 5
}
func (o *PPB_Type) SetDEMCR_VC_MMERR(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetDEMCR_VC_MMERR() uint32 {
	return (volatile.LoadUint32(&o.DEMCR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetDEMCR_VC_CORERESET(value uint32) {
	volatile.StoreUint32(&o.DEMCR.Reg, volatile.LoadUint32(&o.DEMCR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetDEMCR_VC_CORERESET() uint32 {
	return volatile.LoadUint32(&o.DEMCR.Reg) & 0x1
}

// PPB.DSCSR: Provides control and status information for Secure debug
func (o *PPB_Type) SetDSCSR_CDSKEY(value uint32) {
	volatile.StoreUint32(&o.DSCSR.Reg, volatile.LoadUint32(&o.DSCSR.Reg)&^(0x20000)|value<<17)
}
func (o *PPB_Type) GetDSCSR_CDSKEY() uint32 {
	return (volatile.LoadUint32(&o.DSCSR.Reg) & 0x20000) >> 17
}
func (o *PPB_Type) SetDSCSR_CDS(value uint32) {
	volatile.StoreUint32(&o.DSCSR.Reg, volatile.LoadUint32(&o.DSCSR.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetDSCSR_CDS() uint32 {
	return (volatile.LoadUint32(&o.DSCSR.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetDSCSR_SBRSEL(value uint32) {
	volatile.StoreUint32(&o.DSCSR.Reg, volatile.LoadUint32(&o.DSCSR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetDSCSR_SBRSEL() uint32 {
	return (volatile.LoadUint32(&o.DSCSR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetDSCSR_SBRSELEN(value uint32) {
	volatile.StoreUint32(&o.DSCSR.Reg, volatile.LoadUint32(&o.DSCSR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetDSCSR_SBRSELEN() uint32 {
	return volatile.LoadUint32(&o.DSCSR.Reg) & 0x1
}

// PPB.STIR: Provides a mechanism for software to generate an interrupt
func (o *PPB_Type) SetSTIR_INTID(value uint32) {
	volatile.StoreUint32(&o.STIR.Reg, volatile.LoadUint32(&o.STIR.Reg)&^(0x1ff)|value)
}
func (o *PPB_Type) GetSTIR_INTID() uint32 {
	return volatile.LoadUint32(&o.STIR.Reg) & 0x1ff
}

// PPB.FPCCR: Holds control data for the Floating-point extension
func (o *PPB_Type) SetFPCCR_ASPEN(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x80000000)|value<<31)
}
func (o *PPB_Type) GetFPCCR_ASPEN() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x80000000) >> 31
}
func (o *PPB_Type) SetFPCCR_LSPEN(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x40000000)|value<<30)
}
func (o *PPB_Type) GetFPCCR_LSPEN() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x40000000) >> 30
}
func (o *PPB_Type) SetFPCCR_LSPENS(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x20000000)|value<<29)
}
func (o *PPB_Type) GetFPCCR_LSPENS() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x20000000) >> 29
}
func (o *PPB_Type) SetFPCCR_CLRONRET(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x10000000)|value<<28)
}
func (o *PPB_Type) GetFPCCR_CLRONRET() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x10000000) >> 28
}
func (o *PPB_Type) SetFPCCR_CLRONRETS(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x8000000)|value<<27)
}
func (o *PPB_Type) GetFPCCR_CLRONRETS() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x8000000) >> 27
}
func (o *PPB_Type) SetFPCCR_TS(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x4000000)|value<<26)
}
func (o *PPB_Type) GetFPCCR_TS() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x4000000) >> 26
}
func (o *PPB_Type) SetFPCCR_UFRDY(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x400)|value<<10)
}
func (o *PPB_Type) GetFPCCR_UFRDY() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x400) >> 10
}
func (o *PPB_Type) SetFPCCR_SPLIMVIOL(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x200)|value<<9)
}
func (o *PPB_Type) GetFPCCR_SPLIMVIOL() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x200) >> 9
}
func (o *PPB_Type) SetFPCCR_MONRDY(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x100)|value<<8)
}
func (o *PPB_Type) GetFPCCR_MONRDY() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x100) >> 8
}
func (o *PPB_Type) SetFPCCR_SFRDY(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x80)|value<<7)
}
func (o *PPB_Type) GetFPCCR_SFRDY() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x80) >> 7
}
func (o *PPB_Type) SetFPCCR_BFRDY(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x40)|value<<6)
}
func (o *PPB_Type) GetFPCCR_BFRDY() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x40) >> 6
}
func (o *PPB_Type) SetFPCCR_MMRDY(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x20)|value<<5)
}
func (o *PPB_Type) GetFPCCR_MMRDY() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x20) >> 5
}
func (o *PPB_Type) SetFPCCR_HFRDY(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetFPCCR_HFRDY() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetFPCCR_THREAD(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetFPCCR_THREAD() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetFPCCR_S(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetFPCCR_S() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetFPCCR_USER(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetFPCCR_USER() uint32 {
	return (volatile.LoadUint32(&o.FPCCR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetFPCCR_LSPACT(value uint32) {
	volatile.StoreUint32(&o.FPCCR.Reg, volatile.LoadUint32(&o.FPCCR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetFPCCR_LSPACT() uint32 {
	return volatile.LoadUint32(&o.FPCCR.Reg) & 0x1
}

// PPB.FPCAR: Holds the location of the unpopulated floating-point register space allocated on an exception stack frame
func (o *PPB_Type) SetFPCAR_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.FPCAR.Reg, volatile.LoadUint32(&o.FPCAR.Reg)&^(0xfffffff8)|value<<3)
}
func (o *PPB_Type) GetFPCAR_ADDRESS() uint32 {
	return (volatile.LoadUint32(&o.FPCAR.Reg) & 0xfffffff8) >> 3
}

// PPB.FPDSCR: Holds the default values for the floating-point status control data that the PE assigns to the FPSCR when it creates a new floating-point context
func (o *PPB_Type) SetFPDSCR_AHP(value uint32) {
	volatile.StoreUint32(&o.FPDSCR.Reg, volatile.LoadUint32(&o.FPDSCR.Reg)&^(0x4000000)|value<<26)
}
func (o *PPB_Type) GetFPDSCR_AHP() uint32 {
	return (volatile.LoadUint32(&o.FPDSCR.Reg) & 0x4000000) >> 26
}
func (o *PPB_Type) SetFPDSCR_DN(value uint32) {
	volatile.StoreUint32(&o.FPDSCR.Reg, volatile.LoadUint32(&o.FPDSCR.Reg)&^(0x2000000)|value<<25)
}
func (o *PPB_Type) GetFPDSCR_DN() uint32 {
	return (volatile.LoadUint32(&o.FPDSCR.Reg) & 0x2000000) >> 25
}
func (o *PPB_Type) SetFPDSCR_FZ(value uint32) {
	volatile.StoreUint32(&o.FPDSCR.Reg, volatile.LoadUint32(&o.FPDSCR.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetFPDSCR_FZ() uint32 {
	return (volatile.LoadUint32(&o.FPDSCR.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetFPDSCR_RMODE(value uint32) {
	volatile.StoreUint32(&o.FPDSCR.Reg, volatile.LoadUint32(&o.FPDSCR.Reg)&^(0xc00000)|value<<22)
}
func (o *PPB_Type) GetFPDSCR_RMODE() uint32 {
	return (volatile.LoadUint32(&o.FPDSCR.Reg) & 0xc00000) >> 22
}

// PPB.MVFR0: Describes the features provided by the Floating-point Extension
func (o *PPB_Type) SetMVFR0_FPROUND(value uint32) {
	volatile.StoreUint32(&o.MVFR0.Reg, volatile.LoadUint32(&o.MVFR0.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetMVFR0_FPROUND() uint32 {
	return (volatile.LoadUint32(&o.MVFR0.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetMVFR0_FPSQRT(value uint32) {
	volatile.StoreUint32(&o.MVFR0.Reg, volatile.LoadUint32(&o.MVFR0.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetMVFR0_FPSQRT() uint32 {
	return (volatile.LoadUint32(&o.MVFR0.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetMVFR0_FPDIVIDE(value uint32) {
	volatile.StoreUint32(&o.MVFR0.Reg, volatile.LoadUint32(&o.MVFR0.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetMVFR0_FPDIVIDE() uint32 {
	return (volatile.LoadUint32(&o.MVFR0.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetMVFR0_FPDP(value uint32) {
	volatile.StoreUint32(&o.MVFR0.Reg, volatile.LoadUint32(&o.MVFR0.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetMVFR0_FPDP() uint32 {
	return (volatile.LoadUint32(&o.MVFR0.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetMVFR0_FPSP(value uint32) {
	volatile.StoreUint32(&o.MVFR0.Reg, volatile.LoadUint32(&o.MVFR0.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetMVFR0_FPSP() uint32 {
	return (volatile.LoadUint32(&o.MVFR0.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetMVFR0_SIMDREG(value uint32) {
	volatile.StoreUint32(&o.MVFR0.Reg, volatile.LoadUint32(&o.MVFR0.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetMVFR0_SIMDREG() uint32 {
	return volatile.LoadUint32(&o.MVFR0.Reg) & 0xf
}

// PPB.MVFR1: Describes the features provided by the Floating-point Extension
func (o *PPB_Type) SetMVFR1_FMAC(value uint32) {
	volatile.StoreUint32(&o.MVFR1.Reg, volatile.LoadUint32(&o.MVFR1.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetMVFR1_FMAC() uint32 {
	return (volatile.LoadUint32(&o.MVFR1.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetMVFR1_FPHP(value uint32) {
	volatile.StoreUint32(&o.MVFR1.Reg, volatile.LoadUint32(&o.MVFR1.Reg)&^(0xf000000)|value<<24)
}
func (o *PPB_Type) GetMVFR1_FPHP() uint32 {
	return (volatile.LoadUint32(&o.MVFR1.Reg) & 0xf000000) >> 24
}
func (o *PPB_Type) SetMVFR1_FPDNAN(value uint32) {
	volatile.StoreUint32(&o.MVFR1.Reg, volatile.LoadUint32(&o.MVFR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetMVFR1_FPDNAN() uint32 {
	return (volatile.LoadUint32(&o.MVFR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetMVFR1_FPFTZ(value uint32) {
	volatile.StoreUint32(&o.MVFR1.Reg, volatile.LoadUint32(&o.MVFR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetMVFR1_FPFTZ() uint32 {
	return volatile.LoadUint32(&o.MVFR1.Reg) & 0xf
}

// PPB.MVFR2: Describes the features provided by the Floating-point Extension
func (o *PPB_Type) SetMVFR2_FPMISC(value uint32) {
	volatile.StoreUint32(&o.MVFR2.Reg, volatile.LoadUint32(&o.MVFR2.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetMVFR2_FPMISC() uint32 {
	return (volatile.LoadUint32(&o.MVFR2.Reg) & 0xf0) >> 4
}

// PPB.DDEVARCH: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDDEVARCH_ARCHITECT(value uint32) {
	volatile.StoreUint32(&o.DDEVARCH.Reg, volatile.LoadUint32(&o.DDEVARCH.Reg)&^(0xffe00000)|value<<21)
}
func (o *PPB_Type) GetDDEVARCH_ARCHITECT() uint32 {
	return (volatile.LoadUint32(&o.DDEVARCH.Reg) & 0xffe00000) >> 21
}
func (o *PPB_Type) SetDDEVARCH_PRESENT(value uint32) {
	volatile.StoreUint32(&o.DDEVARCH.Reg, volatile.LoadUint32(&o.DDEVARCH.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetDDEVARCH_PRESENT() uint32 {
	return (volatile.LoadUint32(&o.DDEVARCH.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetDDEVARCH_REVISION(value uint32) {
	volatile.StoreUint32(&o.DDEVARCH.Reg, volatile.LoadUint32(&o.DDEVARCH.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetDDEVARCH_REVISION() uint32 {
	return (volatile.LoadUint32(&o.DDEVARCH.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetDDEVARCH_ARCHVER(value uint32) {
	volatile.StoreUint32(&o.DDEVARCH.Reg, volatile.LoadUint32(&o.DDEVARCH.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetDDEVARCH_ARCHVER() uint32 {
	return (volatile.LoadUint32(&o.DDEVARCH.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetDDEVARCH_ARCHPART(value uint32) {
	volatile.StoreUint32(&o.DDEVARCH.Reg, volatile.LoadUint32(&o.DDEVARCH.Reg)&^(0xfff)|value)
}
func (o *PPB_Type) GetDDEVARCH_ARCHPART() uint32 {
	return volatile.LoadUint32(&o.DDEVARCH.Reg) & 0xfff
}

// PPB.DDEVTYPE: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDDEVTYPE_SUB(value uint32) {
	volatile.StoreUint32(&o.DDEVTYPE.Reg, volatile.LoadUint32(&o.DDEVTYPE.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDDEVTYPE_SUB() uint32 {
	return (volatile.LoadUint32(&o.DDEVTYPE.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDDEVTYPE_MAJOR(value uint32) {
	volatile.StoreUint32(&o.DDEVTYPE.Reg, volatile.LoadUint32(&o.DDEVTYPE.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDDEVTYPE_MAJOR() uint32 {
	return volatile.LoadUint32(&o.DDEVTYPE.Reg) & 0xf
}

// PPB.DPIDR4: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDPIDR4_SIZE(value uint32) {
	volatile.StoreUint32(&o.DPIDR4.Reg, volatile.LoadUint32(&o.DPIDR4.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDPIDR4_SIZE() uint32 {
	return (volatile.LoadUint32(&o.DPIDR4.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDPIDR4_DES_2(value uint32) {
	volatile.StoreUint32(&o.DPIDR4.Reg, volatile.LoadUint32(&o.DPIDR4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDPIDR4_DES_2() uint32 {
	return volatile.LoadUint32(&o.DPIDR4.Reg) & 0xf
}

// PPB.DPIDR5: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDPIDR5(value uint32) {
	volatile.StoreUint32(&o.DPIDR5.Reg, value)
}
func (o *PPB_Type) GetDPIDR5() uint32 {
	return volatile.LoadUint32(&o.DPIDR5.Reg)
}

// PPB.DPIDR6: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDPIDR6(value uint32) {
	volatile.StoreUint32(&o.DPIDR6.Reg, value)
}
func (o *PPB_Type) GetDPIDR6() uint32 {
	return volatile.LoadUint32(&o.DPIDR6.Reg)
}

// PPB.DPIDR7: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDPIDR7(value uint32) {
	volatile.StoreUint32(&o.DPIDR7.Reg, value)
}
func (o *PPB_Type) GetDPIDR7() uint32 {
	return volatile.LoadUint32(&o.DPIDR7.Reg)
}

// PPB.DPIDR0: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDPIDR0_PART_0(value uint32) {
	volatile.StoreUint32(&o.DPIDR0.Reg, volatile.LoadUint32(&o.DPIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDPIDR0_PART_0() uint32 {
	return volatile.LoadUint32(&o.DPIDR0.Reg) & 0xff
}

// PPB.DPIDR1: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDPIDR1_DES_0(value uint32) {
	volatile.StoreUint32(&o.DPIDR1.Reg, volatile.LoadUint32(&o.DPIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDPIDR1_DES_0() uint32 {
	return (volatile.LoadUint32(&o.DPIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDPIDR1_PART_1(value uint32) {
	volatile.StoreUint32(&o.DPIDR1.Reg, volatile.LoadUint32(&o.DPIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDPIDR1_PART_1() uint32 {
	return volatile.LoadUint32(&o.DPIDR1.Reg) & 0xf
}

// PPB.DPIDR2: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDPIDR2_REVISION(value uint32) {
	volatile.StoreUint32(&o.DPIDR2.Reg, volatile.LoadUint32(&o.DPIDR2.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDPIDR2_REVISION() uint32 {
	return (volatile.LoadUint32(&o.DPIDR2.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDPIDR2_JEDEC(value uint32) {
	volatile.StoreUint32(&o.DPIDR2.Reg, volatile.LoadUint32(&o.DPIDR2.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetDPIDR2_JEDEC() uint32 {
	return (volatile.LoadUint32(&o.DPIDR2.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetDPIDR2_DES_1(value uint32) {
	volatile.StoreUint32(&o.DPIDR2.Reg, volatile.LoadUint32(&o.DPIDR2.Reg)&^(0x7)|value)
}
func (o *PPB_Type) GetDPIDR2_DES_1() uint32 {
	return volatile.LoadUint32(&o.DPIDR2.Reg) & 0x7
}

// PPB.DPIDR3: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDPIDR3_REVAND(value uint32) {
	volatile.StoreUint32(&o.DPIDR3.Reg, volatile.LoadUint32(&o.DPIDR3.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDPIDR3_REVAND() uint32 {
	return (volatile.LoadUint32(&o.DPIDR3.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDPIDR3_CMOD(value uint32) {
	volatile.StoreUint32(&o.DPIDR3.Reg, volatile.LoadUint32(&o.DPIDR3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDPIDR3_CMOD() uint32 {
	return volatile.LoadUint32(&o.DPIDR3.Reg) & 0xf
}

// PPB.DCIDR0: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDCIDR0_PRMBL_0(value uint32) {
	volatile.StoreUint32(&o.DCIDR0.Reg, volatile.LoadUint32(&o.DCIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDCIDR0_PRMBL_0() uint32 {
	return volatile.LoadUint32(&o.DCIDR0.Reg) & 0xff
}

// PPB.DCIDR1: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDCIDR1_CLASS(value uint32) {
	volatile.StoreUint32(&o.DCIDR1.Reg, volatile.LoadUint32(&o.DCIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDCIDR1_CLASS() uint32 {
	return (volatile.LoadUint32(&o.DCIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDCIDR1_PRMBL_1(value uint32) {
	volatile.StoreUint32(&o.DCIDR1.Reg, volatile.LoadUint32(&o.DCIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDCIDR1_PRMBL_1() uint32 {
	return volatile.LoadUint32(&o.DCIDR1.Reg) & 0xf
}

// PPB.DCIDR2: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDCIDR2_PRMBL_2(value uint32) {
	volatile.StoreUint32(&o.DCIDR2.Reg, volatile.LoadUint32(&o.DCIDR2.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDCIDR2_PRMBL_2() uint32 {
	return volatile.LoadUint32(&o.DCIDR2.Reg) & 0xff
}

// PPB.DCIDR3: Provides CoreSight discovery information for the SCS
func (o *PPB_Type) SetDCIDR3_PRMBL_3(value uint32) {
	volatile.StoreUint32(&o.DCIDR3.Reg, volatile.LoadUint32(&o.DCIDR3.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetDCIDR3_PRMBL_3() uint32 {
	return volatile.LoadUint32(&o.DCIDR3.Reg) & 0xff
}

// PPB.TRCPRGCTLR: Programming Control Register
func (o *PPB_Type) SetTRCPRGCTLR_EN(value uint32) {
	volatile.StoreUint32(&o.TRCPRGCTLR.Reg, volatile.LoadUint32(&o.TRCPRGCTLR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCPRGCTLR_EN() uint32 {
	return volatile.LoadUint32(&o.TRCPRGCTLR.Reg) & 0x1
}

// PPB.TRCSTATR: The TRCSTATR indicates the ETM-Teal status
func (o *PPB_Type) SetTRCSTATR_PMSTABLE(value uint32) {
	volatile.StoreUint32(&o.TRCSTATR.Reg, volatile.LoadUint32(&o.TRCSTATR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetTRCSTATR_PMSTABLE() uint32 {
	return (volatile.LoadUint32(&o.TRCSTATR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetTRCSTATR_IDLE(value uint32) {
	volatile.StoreUint32(&o.TRCSTATR.Reg, volatile.LoadUint32(&o.TRCSTATR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCSTATR_IDLE() uint32 {
	return volatile.LoadUint32(&o.TRCSTATR.Reg) & 0x1
}

// PPB.TRCCONFIGR: The TRCCONFIGR sets the basic tracing options for the trace unit
func (o *PPB_Type) SetTRCCONFIGR_RS(value uint32) {
	volatile.StoreUint32(&o.TRCCONFIGR.Reg, volatile.LoadUint32(&o.TRCCONFIGR.Reg)&^(0x1000)|value<<12)
}
func (o *PPB_Type) GetTRCCONFIGR_RS() uint32 {
	return (volatile.LoadUint32(&o.TRCCONFIGR.Reg) & 0x1000) >> 12
}
func (o *PPB_Type) SetTRCCONFIGR_TS(value uint32) {
	volatile.StoreUint32(&o.TRCCONFIGR.Reg, volatile.LoadUint32(&o.TRCCONFIGR.Reg)&^(0x800)|value<<11)
}
func (o *PPB_Type) GetTRCCONFIGR_TS() uint32 {
	return (volatile.LoadUint32(&o.TRCCONFIGR.Reg) & 0x800) >> 11
}
func (o *PPB_Type) SetTRCCONFIGR_COND(value uint32) {
	volatile.StoreUint32(&o.TRCCONFIGR.Reg, volatile.LoadUint32(&o.TRCCONFIGR.Reg)&^(0x7e0)|value<<5)
}
func (o *PPB_Type) GetTRCCONFIGR_COND() uint32 {
	return (volatile.LoadUint32(&o.TRCCONFIGR.Reg) & 0x7e0) >> 5
}
func (o *PPB_Type) SetTRCCONFIGR_CCI(value uint32) {
	volatile.StoreUint32(&o.TRCCONFIGR.Reg, volatile.LoadUint32(&o.TRCCONFIGR.Reg)&^(0x10)|value<<4)
}
func (o *PPB_Type) GetTRCCONFIGR_CCI() uint32 {
	return (volatile.LoadUint32(&o.TRCCONFIGR.Reg) & 0x10) >> 4
}
func (o *PPB_Type) SetTRCCONFIGR_BB(value uint32) {
	volatile.StoreUint32(&o.TRCCONFIGR.Reg, volatile.LoadUint32(&o.TRCCONFIGR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetTRCCONFIGR_BB() uint32 {
	return (volatile.LoadUint32(&o.TRCCONFIGR.Reg) & 0x8) >> 3
}

// PPB.TRCEVENTCTL0R: The TRCEVENTCTL0R controls the tracing of events in the trace stream. The events also drive the ETM-Teal external outputs.
func (o *PPB_Type) SetTRCEVENTCTL0R_TYPE1(value uint32) {
	volatile.StoreUint32(&o.TRCEVENTCTL0R.Reg, volatile.LoadUint32(&o.TRCEVENTCTL0R.Reg)&^(0x8000)|value<<15)
}
func (o *PPB_Type) GetTRCEVENTCTL0R_TYPE1() uint32 {
	return (volatile.LoadUint32(&o.TRCEVENTCTL0R.Reg) & 0x8000) >> 15
}
func (o *PPB_Type) SetTRCEVENTCTL0R_SEL1(value uint32) {
	volatile.StoreUint32(&o.TRCEVENTCTL0R.Reg, volatile.LoadUint32(&o.TRCEVENTCTL0R.Reg)&^(0x700)|value<<8)
}
func (o *PPB_Type) GetTRCEVENTCTL0R_SEL1() uint32 {
	return (volatile.LoadUint32(&o.TRCEVENTCTL0R.Reg) & 0x700) >> 8
}
func (o *PPB_Type) SetTRCEVENTCTL0R_TYPE0(value uint32) {
	volatile.StoreUint32(&o.TRCEVENTCTL0R.Reg, volatile.LoadUint32(&o.TRCEVENTCTL0R.Reg)&^(0x80)|value<<7)
}
func (o *PPB_Type) GetTRCEVENTCTL0R_TYPE0() uint32 {
	return (volatile.LoadUint32(&o.TRCEVENTCTL0R.Reg) & 0x80) >> 7
}
func (o *PPB_Type) SetTRCEVENTCTL0R_SEL0(value uint32) {
	volatile.StoreUint32(&o.TRCEVENTCTL0R.Reg, volatile.LoadUint32(&o.TRCEVENTCTL0R.Reg)&^(0x7)|value)
}
func (o *PPB_Type) GetTRCEVENTCTL0R_SEL0() uint32 {
	return volatile.LoadUint32(&o.TRCEVENTCTL0R.Reg) & 0x7
}

// PPB.TRCEVENTCTL1R: The TRCEVENTCTL1R controls how the events selected by TRCEVENTCTL0R behave
func (o *PPB_Type) SetTRCEVENTCTL1R_LPOVERRIDE(value uint32) {
	volatile.StoreUint32(&o.TRCEVENTCTL1R.Reg, volatile.LoadUint32(&o.TRCEVENTCTL1R.Reg)&^(0x1000)|value<<12)
}
func (o *PPB_Type) GetTRCEVENTCTL1R_LPOVERRIDE() uint32 {
	return (volatile.LoadUint32(&o.TRCEVENTCTL1R.Reg) & 0x1000) >> 12
}
func (o *PPB_Type) SetTRCEVENTCTL1R_ATB(value uint32) {
	volatile.StoreUint32(&o.TRCEVENTCTL1R.Reg, volatile.LoadUint32(&o.TRCEVENTCTL1R.Reg)&^(0x800)|value<<11)
}
func (o *PPB_Type) GetTRCEVENTCTL1R_ATB() uint32 {
	return (volatile.LoadUint32(&o.TRCEVENTCTL1R.Reg) & 0x800) >> 11
}
func (o *PPB_Type) SetTRCEVENTCTL1R_INSTEN1(value uint32) {
	volatile.StoreUint32(&o.TRCEVENTCTL1R.Reg, volatile.LoadUint32(&o.TRCEVENTCTL1R.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetTRCEVENTCTL1R_INSTEN1() uint32 {
	return (volatile.LoadUint32(&o.TRCEVENTCTL1R.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetTRCEVENTCTL1R_INSTEN0(value uint32) {
	volatile.StoreUint32(&o.TRCEVENTCTL1R.Reg, volatile.LoadUint32(&o.TRCEVENTCTL1R.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCEVENTCTL1R_INSTEN0() uint32 {
	return volatile.LoadUint32(&o.TRCEVENTCTL1R.Reg) & 0x1
}

// PPB.TRCSTALLCTLR: The TRCSTALLCTLR enables ETM-Teal to stall the processor if the ETM-Teal FIFO goes over the programmed level to minimize risk of overflow
func (o *PPB_Type) SetTRCSTALLCTLR_INSTPRIORITY(value uint32) {
	volatile.StoreUint32(&o.TRCSTALLCTLR.Reg, volatile.LoadUint32(&o.TRCSTALLCTLR.Reg)&^(0x400)|value<<10)
}
func (o *PPB_Type) GetTRCSTALLCTLR_INSTPRIORITY() uint32 {
	return (volatile.LoadUint32(&o.TRCSTALLCTLR.Reg) & 0x400) >> 10
}
func (o *PPB_Type) SetTRCSTALLCTLR_ISTALL(value uint32) {
	volatile.StoreUint32(&o.TRCSTALLCTLR.Reg, volatile.LoadUint32(&o.TRCSTALLCTLR.Reg)&^(0x100)|value<<8)
}
func (o *PPB_Type) GetTRCSTALLCTLR_ISTALL() uint32 {
	return (volatile.LoadUint32(&o.TRCSTALLCTLR.Reg) & 0x100) >> 8
}
func (o *PPB_Type) SetTRCSTALLCTLR_LEVEL(value uint32) {
	volatile.StoreUint32(&o.TRCSTALLCTLR.Reg, volatile.LoadUint32(&o.TRCSTALLCTLR.Reg)&^(0xc)|value<<2)
}
func (o *PPB_Type) GetTRCSTALLCTLR_LEVEL() uint32 {
	return (volatile.LoadUint32(&o.TRCSTALLCTLR.Reg) & 0xc) >> 2
}

// PPB.TRCTSCTLR: The TRCTSCTLR controls the insertion of global timestamps into the trace stream. A timestamp is always inserted into the instruction trace stream
func (o *PPB_Type) SetTRCTSCTLR_TYPE0(value uint32) {
	volatile.StoreUint32(&o.TRCTSCTLR.Reg, volatile.LoadUint32(&o.TRCTSCTLR.Reg)&^(0x80)|value<<7)
}
func (o *PPB_Type) GetTRCTSCTLR_TYPE0() uint32 {
	return (volatile.LoadUint32(&o.TRCTSCTLR.Reg) & 0x80) >> 7
}
func (o *PPB_Type) SetTRCTSCTLR_SEL0(value uint32) {
	volatile.StoreUint32(&o.TRCTSCTLR.Reg, volatile.LoadUint32(&o.TRCTSCTLR.Reg)&^(0x3)|value)
}
func (o *PPB_Type) GetTRCTSCTLR_SEL0() uint32 {
	return volatile.LoadUint32(&o.TRCTSCTLR.Reg) & 0x3
}

// PPB.TRCSYNCPR: The TRCSYNCPR specifies the period of trace synchronization of the trace streams. TRCSYNCPR defines a number of bytes of trace between requests for trace synchronization. This value is always a power of two
func (o *PPB_Type) SetTRCSYNCPR_PERIOD(value uint32) {
	volatile.StoreUint32(&o.TRCSYNCPR.Reg, volatile.LoadUint32(&o.TRCSYNCPR.Reg)&^(0x1f)|value)
}
func (o *PPB_Type) GetTRCSYNCPR_PERIOD() uint32 {
	return volatile.LoadUint32(&o.TRCSYNCPR.Reg) & 0x1f
}

// PPB.TRCCCCTLR: The TRCCCCTLR sets the threshold value for instruction trace cycle counting. The threshold represents the minimum interval between cycle count trace packets
func (o *PPB_Type) SetTRCCCCTLR_THRESHOLD(value uint32) {
	volatile.StoreUint32(&o.TRCCCCTLR.Reg, volatile.LoadUint32(&o.TRCCCCTLR.Reg)&^(0xfff)|value)
}
func (o *PPB_Type) GetTRCCCCTLR_THRESHOLD() uint32 {
	return volatile.LoadUint32(&o.TRCCCCTLR.Reg) & 0xfff
}

// PPB.TRCVICTLR: The TRCVICTLR controls instruction trace filtering
func (o *PPB_Type) SetTRCVICTLR_EXLEVEL_S3(value uint32) {
	volatile.StoreUint32(&o.TRCVICTLR.Reg, volatile.LoadUint32(&o.TRCVICTLR.Reg)&^(0x80000)|value<<19)
}
func (o *PPB_Type) GetTRCVICTLR_EXLEVEL_S3() uint32 {
	return (volatile.LoadUint32(&o.TRCVICTLR.Reg) & 0x80000) >> 19
}
func (o *PPB_Type) SetTRCVICTLR_EXLEVEL_S0(value uint32) {
	volatile.StoreUint32(&o.TRCVICTLR.Reg, volatile.LoadUint32(&o.TRCVICTLR.Reg)&^(0x10000)|value<<16)
}
func (o *PPB_Type) GetTRCVICTLR_EXLEVEL_S0() uint32 {
	return (volatile.LoadUint32(&o.TRCVICTLR.Reg) & 0x10000) >> 16
}
func (o *PPB_Type) SetTRCVICTLR_TRCERR(value uint32) {
	volatile.StoreUint32(&o.TRCVICTLR.Reg, volatile.LoadUint32(&o.TRCVICTLR.Reg)&^(0x800)|value<<11)
}
func (o *PPB_Type) GetTRCVICTLR_TRCERR() uint32 {
	return (volatile.LoadUint32(&o.TRCVICTLR.Reg) & 0x800) >> 11
}
func (o *PPB_Type) SetTRCVICTLR_TRCRESET(value uint32) {
	volatile.StoreUint32(&o.TRCVICTLR.Reg, volatile.LoadUint32(&o.TRCVICTLR.Reg)&^(0x400)|value<<10)
}
func (o *PPB_Type) GetTRCVICTLR_TRCRESET() uint32 {
	return (volatile.LoadUint32(&o.TRCVICTLR.Reg) & 0x400) >> 10
}
func (o *PPB_Type) SetTRCVICTLR_SSSTATUS(value uint32) {
	volatile.StoreUint32(&o.TRCVICTLR.Reg, volatile.LoadUint32(&o.TRCVICTLR.Reg)&^(0x200)|value<<9)
}
func (o *PPB_Type) GetTRCVICTLR_SSSTATUS() uint32 {
	return (volatile.LoadUint32(&o.TRCVICTLR.Reg) & 0x200) >> 9
}
func (o *PPB_Type) SetTRCVICTLR_TYPE0(value uint32) {
	volatile.StoreUint32(&o.TRCVICTLR.Reg, volatile.LoadUint32(&o.TRCVICTLR.Reg)&^(0x80)|value<<7)
}
func (o *PPB_Type) GetTRCVICTLR_TYPE0() uint32 {
	return (volatile.LoadUint32(&o.TRCVICTLR.Reg) & 0x80) >> 7
}
func (o *PPB_Type) SetTRCVICTLR_SEL0(value uint32) {
	volatile.StoreUint32(&o.TRCVICTLR.Reg, volatile.LoadUint32(&o.TRCVICTLR.Reg)&^(0x3)|value)
}
func (o *PPB_Type) GetTRCVICTLR_SEL0() uint32 {
	return volatile.LoadUint32(&o.TRCVICTLR.Reg) & 0x3
}

// PPB.TRCCNTRLDVR0: The TRCCNTRLDVR defines the reload value for the reduced function counter
func (o *PPB_Type) SetTRCCNTRLDVR0_VALUE(value uint32) {
	volatile.StoreUint32(&o.TRCCNTRLDVR0.Reg, volatile.LoadUint32(&o.TRCCNTRLDVR0.Reg)&^(0xffff)|value)
}
func (o *PPB_Type) GetTRCCNTRLDVR0_VALUE() uint32 {
	return volatile.LoadUint32(&o.TRCCNTRLDVR0.Reg) & 0xffff
}

// PPB.TRCIDR8: TRCIDR8
func (o *PPB_Type) SetTRCIDR8(value uint32) {
	volatile.StoreUint32(&o.TRCIDR8.Reg, value)
}
func (o *PPB_Type) GetTRCIDR8() uint32 {
	return volatile.LoadUint32(&o.TRCIDR8.Reg)
}

// PPB.TRCIDR9: TRCIDR9
func (o *PPB_Type) SetTRCIDR9(value uint32) {
	volatile.StoreUint32(&o.TRCIDR9.Reg, value)
}
func (o *PPB_Type) GetTRCIDR9() uint32 {
	return volatile.LoadUint32(&o.TRCIDR9.Reg)
}

// PPB.TRCIDR10: TRCIDR10
func (o *PPB_Type) SetTRCIDR10(value uint32) {
	volatile.StoreUint32(&o.TRCIDR10.Reg, value)
}
func (o *PPB_Type) GetTRCIDR10() uint32 {
	return volatile.LoadUint32(&o.TRCIDR10.Reg)
}

// PPB.TRCIDR11: TRCIDR11
func (o *PPB_Type) SetTRCIDR11(value uint32) {
	volatile.StoreUint32(&o.TRCIDR11.Reg, value)
}
func (o *PPB_Type) GetTRCIDR11() uint32 {
	return volatile.LoadUint32(&o.TRCIDR11.Reg)
}

// PPB.TRCIDR12: TRCIDR12
func (o *PPB_Type) SetTRCIDR12(value uint32) {
	volatile.StoreUint32(&o.TRCIDR12.Reg, value)
}
func (o *PPB_Type) GetTRCIDR12() uint32 {
	return volatile.LoadUint32(&o.TRCIDR12.Reg)
}

// PPB.TRCIDR13: TRCIDR13
func (o *PPB_Type) SetTRCIDR13(value uint32) {
	volatile.StoreUint32(&o.TRCIDR13.Reg, value)
}
func (o *PPB_Type) GetTRCIDR13() uint32 {
	return volatile.LoadUint32(&o.TRCIDR13.Reg)
}

// PPB.TRCIMSPEC: The TRCIMSPEC shows the presence of any IMPLEMENTATION SPECIFIC features, and enables any features that are provided
func (o *PPB_Type) SetTRCIMSPEC_SUPPORT(value uint32) {
	volatile.StoreUint32(&o.TRCIMSPEC.Reg, volatile.LoadUint32(&o.TRCIMSPEC.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetTRCIMSPEC_SUPPORT() uint32 {
	return volatile.LoadUint32(&o.TRCIMSPEC.Reg) & 0xf
}

// PPB.TRCIDR0: TRCIDR0
func (o *PPB_Type) SetTRCIDR0_COMMOPT(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x20000000)|value<<29)
}
func (o *PPB_Type) GetTRCIDR0_COMMOPT() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x20000000) >> 29
}
func (o *PPB_Type) SetTRCIDR0_TSSIZE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x1f000000)|value<<24)
}
func (o *PPB_Type) GetTRCIDR0_TSSIZE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x1f000000) >> 24
}
func (o *PPB_Type) SetTRCIDR0_TRCEXDATA(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x20000)|value<<17)
}
func (o *PPB_Type) GetTRCIDR0_TRCEXDATA() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x20000) >> 17
}
func (o *PPB_Type) SetTRCIDR0_QSUPP(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x18000)|value<<15)
}
func (o *PPB_Type) GetTRCIDR0_QSUPP() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x18000) >> 15
}
func (o *PPB_Type) SetTRCIDR0_QFILT(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x4000)|value<<14)
}
func (o *PPB_Type) GetTRCIDR0_QFILT() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x4000) >> 14
}
func (o *PPB_Type) SetTRCIDR0_CONDTYPE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x3000)|value<<12)
}
func (o *PPB_Type) GetTRCIDR0_CONDTYPE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x3000) >> 12
}
func (o *PPB_Type) SetTRCIDR0_NUMEVENT(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0xc00)|value<<10)
}
func (o *PPB_Type) GetTRCIDR0_NUMEVENT() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0xc00) >> 10
}
func (o *PPB_Type) SetTRCIDR0_RETSTACK(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x200)|value<<9)
}
func (o *PPB_Type) GetTRCIDR0_RETSTACK() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x200) >> 9
}
func (o *PPB_Type) SetTRCIDR0_TRCCCI(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x80)|value<<7)
}
func (o *PPB_Type) GetTRCIDR0_TRCCCI() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x80) >> 7
}
func (o *PPB_Type) SetTRCIDR0_TRCCOND(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x40)|value<<6)
}
func (o *PPB_Type) GetTRCIDR0_TRCCOND() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x40) >> 6
}
func (o *PPB_Type) SetTRCIDR0_TRCBB(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x20)|value<<5)
}
func (o *PPB_Type) GetTRCIDR0_TRCBB() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x20) >> 5
}
func (o *PPB_Type) SetTRCIDR0_TRCDATA(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x18)|value<<3)
}
func (o *PPB_Type) GetTRCIDR0_TRCDATA() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x18) >> 3
}
func (o *PPB_Type) SetTRCIDR0_INSTP0(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x6)|value<<1)
}
func (o *PPB_Type) GetTRCIDR0_INSTP0() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x6) >> 1
}
func (o *PPB_Type) SetTRCIDR0_RES1(value uint32) {
	volatile.StoreUint32(&o.TRCIDR0.Reg, volatile.LoadUint32(&o.TRCIDR0.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCIDR0_RES1() uint32 {
	return volatile.LoadUint32(&o.TRCIDR0.Reg) & 0x1
}

// PPB.TRCIDR1: TRCIDR1
func (o *PPB_Type) SetTRCIDR1_DESIGNER(value uint32) {
	volatile.StoreUint32(&o.TRCIDR1.Reg, volatile.LoadUint32(&o.TRCIDR1.Reg)&^(0xff000000)|value<<24)
}
func (o *PPB_Type) GetTRCIDR1_DESIGNER() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR1.Reg) & 0xff000000) >> 24
}
func (o *PPB_Type) SetTRCIDR1_RES1(value uint32) {
	volatile.StoreUint32(&o.TRCIDR1.Reg, volatile.LoadUint32(&o.TRCIDR1.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetTRCIDR1_RES1() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR1.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetTRCIDR1_TRCARCHMAJ(value uint32) {
	volatile.StoreUint32(&o.TRCIDR1.Reg, volatile.LoadUint32(&o.TRCIDR1.Reg)&^(0xf00)|value<<8)
}
func (o *PPB_Type) GetTRCIDR1_TRCARCHMAJ() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR1.Reg) & 0xf00) >> 8
}
func (o *PPB_Type) SetTRCIDR1_TRCARCHMIN(value uint32) {
	volatile.StoreUint32(&o.TRCIDR1.Reg, volatile.LoadUint32(&o.TRCIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetTRCIDR1_TRCARCHMIN() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetTRCIDR1_REVISION(value uint32) {
	volatile.StoreUint32(&o.TRCIDR1.Reg, volatile.LoadUint32(&o.TRCIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetTRCIDR1_REVISION() uint32 {
	return volatile.LoadUint32(&o.TRCIDR1.Reg) & 0xf
}

// PPB.TRCIDR2: TRCIDR2
func (o *PPB_Type) SetTRCIDR2_CCSIZE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR2.Reg, volatile.LoadUint32(&o.TRCIDR2.Reg)&^(0x1e000000)|value<<25)
}
func (o *PPB_Type) GetTRCIDR2_CCSIZE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR2.Reg) & 0x1e000000) >> 25
}
func (o *PPB_Type) SetTRCIDR2_DVSIZE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR2.Reg, volatile.LoadUint32(&o.TRCIDR2.Reg)&^(0x1f00000)|value<<20)
}
func (o *PPB_Type) GetTRCIDR2_DVSIZE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR2.Reg) & 0x1f00000) >> 20
}
func (o *PPB_Type) SetTRCIDR2_DASIZE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR2.Reg, volatile.LoadUint32(&o.TRCIDR2.Reg)&^(0xf8000)|value<<15)
}
func (o *PPB_Type) GetTRCIDR2_DASIZE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR2.Reg) & 0xf8000) >> 15
}
func (o *PPB_Type) SetTRCIDR2_VMIDSIZE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR2.Reg, volatile.LoadUint32(&o.TRCIDR2.Reg)&^(0x7c00)|value<<10)
}
func (o *PPB_Type) GetTRCIDR2_VMIDSIZE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR2.Reg) & 0x7c00) >> 10
}
func (o *PPB_Type) SetTRCIDR2_CIDSIZE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR2.Reg, volatile.LoadUint32(&o.TRCIDR2.Reg)&^(0x3e0)|value<<5)
}
func (o *PPB_Type) GetTRCIDR2_CIDSIZE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR2.Reg) & 0x3e0) >> 5
}
func (o *PPB_Type) SetTRCIDR2_IASIZE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR2.Reg, volatile.LoadUint32(&o.TRCIDR2.Reg)&^(0x1f)|value)
}
func (o *PPB_Type) GetTRCIDR2_IASIZE() uint32 {
	return volatile.LoadUint32(&o.TRCIDR2.Reg) & 0x1f
}

// PPB.TRCIDR3: TRCIDR3
func (o *PPB_Type) SetTRCIDR3_NOOVERFLOW(value uint32) {
	volatile.StoreUint32(&o.TRCIDR3.Reg, volatile.LoadUint32(&o.TRCIDR3.Reg)&^(0x80000000)|value<<31)
}
func (o *PPB_Type) GetTRCIDR3_NOOVERFLOW() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR3.Reg) & 0x80000000) >> 31
}
func (o *PPB_Type) SetTRCIDR3_NUMPROC(value uint32) {
	volatile.StoreUint32(&o.TRCIDR3.Reg, volatile.LoadUint32(&o.TRCIDR3.Reg)&^(0x70000000)|value<<28)
}
func (o *PPB_Type) GetTRCIDR3_NUMPROC() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR3.Reg) & 0x70000000) >> 28
}
func (o *PPB_Type) SetTRCIDR3_SYSSTALL(value uint32) {
	volatile.StoreUint32(&o.TRCIDR3.Reg, volatile.LoadUint32(&o.TRCIDR3.Reg)&^(0x8000000)|value<<27)
}
func (o *PPB_Type) GetTRCIDR3_SYSSTALL() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR3.Reg) & 0x8000000) >> 27
}
func (o *PPB_Type) SetTRCIDR3_STALLCTL(value uint32) {
	volatile.StoreUint32(&o.TRCIDR3.Reg, volatile.LoadUint32(&o.TRCIDR3.Reg)&^(0x4000000)|value<<26)
}
func (o *PPB_Type) GetTRCIDR3_STALLCTL() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR3.Reg) & 0x4000000) >> 26
}
func (o *PPB_Type) SetTRCIDR3_SYNCPR(value uint32) {
	volatile.StoreUint32(&o.TRCIDR3.Reg, volatile.LoadUint32(&o.TRCIDR3.Reg)&^(0x2000000)|value<<25)
}
func (o *PPB_Type) GetTRCIDR3_SYNCPR() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR3.Reg) & 0x2000000) >> 25
}
func (o *PPB_Type) SetTRCIDR3_TRCERR(value uint32) {
	volatile.StoreUint32(&o.TRCIDR3.Reg, volatile.LoadUint32(&o.TRCIDR3.Reg)&^(0x1000000)|value<<24)
}
func (o *PPB_Type) GetTRCIDR3_TRCERR() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR3.Reg) & 0x1000000) >> 24
}
func (o *PPB_Type) SetTRCIDR3_EXLEVEL_NS(value uint32) {
	volatile.StoreUint32(&o.TRCIDR3.Reg, volatile.LoadUint32(&o.TRCIDR3.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetTRCIDR3_EXLEVEL_NS() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR3.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetTRCIDR3_EXLEVEL_S(value uint32) {
	volatile.StoreUint32(&o.TRCIDR3.Reg, volatile.LoadUint32(&o.TRCIDR3.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetTRCIDR3_EXLEVEL_S() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR3.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetTRCIDR3_CCITMIN(value uint32) {
	volatile.StoreUint32(&o.TRCIDR3.Reg, volatile.LoadUint32(&o.TRCIDR3.Reg)&^(0xfff)|value)
}
func (o *PPB_Type) GetTRCIDR3_CCITMIN() uint32 {
	return volatile.LoadUint32(&o.TRCIDR3.Reg) & 0xfff
}

// PPB.TRCIDR4: TRCIDR4
func (o *PPB_Type) SetTRCIDR4_NUMVMIDC(value uint32) {
	volatile.StoreUint32(&o.TRCIDR4.Reg, volatile.LoadUint32(&o.TRCIDR4.Reg)&^(0xf0000000)|value<<28)
}
func (o *PPB_Type) GetTRCIDR4_NUMVMIDC() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR4.Reg) & 0xf0000000) >> 28
}
func (o *PPB_Type) SetTRCIDR4_NUMCIDC(value uint32) {
	volatile.StoreUint32(&o.TRCIDR4.Reg, volatile.LoadUint32(&o.TRCIDR4.Reg)&^(0xf000000)|value<<24)
}
func (o *PPB_Type) GetTRCIDR4_NUMCIDC() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR4.Reg) & 0xf000000) >> 24
}
func (o *PPB_Type) SetTRCIDR4_NUMSSCC(value uint32) {
	volatile.StoreUint32(&o.TRCIDR4.Reg, volatile.LoadUint32(&o.TRCIDR4.Reg)&^(0xf00000)|value<<20)
}
func (o *PPB_Type) GetTRCIDR4_NUMSSCC() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR4.Reg) & 0xf00000) >> 20
}
func (o *PPB_Type) SetTRCIDR4_NUMRSPAIR(value uint32) {
	volatile.StoreUint32(&o.TRCIDR4.Reg, volatile.LoadUint32(&o.TRCIDR4.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetTRCIDR4_NUMRSPAIR() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR4.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetTRCIDR4_NUMPC(value uint32) {
	volatile.StoreUint32(&o.TRCIDR4.Reg, volatile.LoadUint32(&o.TRCIDR4.Reg)&^(0xf000)|value<<12)
}
func (o *PPB_Type) GetTRCIDR4_NUMPC() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR4.Reg) & 0xf000) >> 12
}
func (o *PPB_Type) SetTRCIDR4_SUPPDAC(value uint32) {
	volatile.StoreUint32(&o.TRCIDR4.Reg, volatile.LoadUint32(&o.TRCIDR4.Reg)&^(0x100)|value<<8)
}
func (o *PPB_Type) GetTRCIDR4_SUPPDAC() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR4.Reg) & 0x100) >> 8
}
func (o *PPB_Type) SetTRCIDR4_NUMDVC(value uint32) {
	volatile.StoreUint32(&o.TRCIDR4.Reg, volatile.LoadUint32(&o.TRCIDR4.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetTRCIDR4_NUMDVC() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR4.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetTRCIDR4_NUMACPAIRS(value uint32) {
	volatile.StoreUint32(&o.TRCIDR4.Reg, volatile.LoadUint32(&o.TRCIDR4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetTRCIDR4_NUMACPAIRS() uint32 {
	return volatile.LoadUint32(&o.TRCIDR4.Reg) & 0xf
}

// PPB.TRCIDR5: TRCIDR5
func (o *PPB_Type) SetTRCIDR5_REDFUNCNTR(value uint32) {
	volatile.StoreUint32(&o.TRCIDR5.Reg, volatile.LoadUint32(&o.TRCIDR5.Reg)&^(0x80000000)|value<<31)
}
func (o *PPB_Type) GetTRCIDR5_REDFUNCNTR() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR5.Reg) & 0x80000000) >> 31
}
func (o *PPB_Type) SetTRCIDR5_NUMCNTR(value uint32) {
	volatile.StoreUint32(&o.TRCIDR5.Reg, volatile.LoadUint32(&o.TRCIDR5.Reg)&^(0x70000000)|value<<28)
}
func (o *PPB_Type) GetTRCIDR5_NUMCNTR() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR5.Reg) & 0x70000000) >> 28
}
func (o *PPB_Type) SetTRCIDR5_NUMSEQSTATE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR5.Reg, volatile.LoadUint32(&o.TRCIDR5.Reg)&^(0xe000000)|value<<25)
}
func (o *PPB_Type) GetTRCIDR5_NUMSEQSTATE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR5.Reg) & 0xe000000) >> 25
}
func (o *PPB_Type) SetTRCIDR5_LPOVERRIDE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR5.Reg, volatile.LoadUint32(&o.TRCIDR5.Reg)&^(0x800000)|value<<23)
}
func (o *PPB_Type) GetTRCIDR5_LPOVERRIDE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR5.Reg) & 0x800000) >> 23
}
func (o *PPB_Type) SetTRCIDR5_ATBTRIG(value uint32) {
	volatile.StoreUint32(&o.TRCIDR5.Reg, volatile.LoadUint32(&o.TRCIDR5.Reg)&^(0x400000)|value<<22)
}
func (o *PPB_Type) GetTRCIDR5_ATBTRIG() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR5.Reg) & 0x400000) >> 22
}
func (o *PPB_Type) SetTRCIDR5_TRACEIDSIZE(value uint32) {
	volatile.StoreUint32(&o.TRCIDR5.Reg, volatile.LoadUint32(&o.TRCIDR5.Reg)&^(0x3f0000)|value<<16)
}
func (o *PPB_Type) GetTRCIDR5_TRACEIDSIZE() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR5.Reg) & 0x3f0000) >> 16
}
func (o *PPB_Type) SetTRCIDR5_NUMEXTINSEL(value uint32) {
	volatile.StoreUint32(&o.TRCIDR5.Reg, volatile.LoadUint32(&o.TRCIDR5.Reg)&^(0xe00)|value<<9)
}
func (o *PPB_Type) GetTRCIDR5_NUMEXTINSEL() uint32 {
	return (volatile.LoadUint32(&o.TRCIDR5.Reg) & 0xe00) >> 9
}
func (o *PPB_Type) SetTRCIDR5_NUMEXTIN(value uint32) {
	volatile.StoreUint32(&o.TRCIDR5.Reg, volatile.LoadUint32(&o.TRCIDR5.Reg)&^(0x1ff)|value)
}
func (o *PPB_Type) GetTRCIDR5_NUMEXTIN() uint32 {
	return volatile.LoadUint32(&o.TRCIDR5.Reg) & 0x1ff
}

// PPB.TRCIDR6: TRCIDR6
func (o *PPB_Type) SetTRCIDR6(value uint32) {
	volatile.StoreUint32(&o.TRCIDR6.Reg, value)
}
func (o *PPB_Type) GetTRCIDR6() uint32 {
	return volatile.LoadUint32(&o.TRCIDR6.Reg)
}

// PPB.TRCIDR7: TRCIDR7
func (o *PPB_Type) SetTRCIDR7(value uint32) {
	volatile.StoreUint32(&o.TRCIDR7.Reg, value)
}
func (o *PPB_Type) GetTRCIDR7() uint32 {
	return volatile.LoadUint32(&o.TRCIDR7.Reg)
}

// PPB.TRCRSCTLR2: The TRCRSCTLR controls the trace resources
func (o *PPB_Type) SetTRCRSCTLR2_PAIRINV(value uint32) {
	volatile.StoreUint32(&o.TRCRSCTLR2.Reg, volatile.LoadUint32(&o.TRCRSCTLR2.Reg)&^(0x200000)|value<<21)
}
func (o *PPB_Type) GetTRCRSCTLR2_PAIRINV() uint32 {
	return (volatile.LoadUint32(&o.TRCRSCTLR2.Reg) & 0x200000) >> 21
}
func (o *PPB_Type) SetTRCRSCTLR2_INV(value uint32) {
	volatile.StoreUint32(&o.TRCRSCTLR2.Reg, volatile.LoadUint32(&o.TRCRSCTLR2.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetTRCRSCTLR2_INV() uint32 {
	return (volatile.LoadUint32(&o.TRCRSCTLR2.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetTRCRSCTLR2_GROUP(value uint32) {
	volatile.StoreUint32(&o.TRCRSCTLR2.Reg, volatile.LoadUint32(&o.TRCRSCTLR2.Reg)&^(0x70000)|value<<16)
}
func (o *PPB_Type) GetTRCRSCTLR2_GROUP() uint32 {
	return (volatile.LoadUint32(&o.TRCRSCTLR2.Reg) & 0x70000) >> 16
}
func (o *PPB_Type) SetTRCRSCTLR2_SELECT(value uint32) {
	volatile.StoreUint32(&o.TRCRSCTLR2.Reg, volatile.LoadUint32(&o.TRCRSCTLR2.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetTRCRSCTLR2_SELECT() uint32 {
	return volatile.LoadUint32(&o.TRCRSCTLR2.Reg) & 0xff
}

// PPB.TRCRSCTLR3: The TRCRSCTLR controls the trace resources
func (o *PPB_Type) SetTRCRSCTLR3_PAIRINV(value uint32) {
	volatile.StoreUint32(&o.TRCRSCTLR3.Reg, volatile.LoadUint32(&o.TRCRSCTLR3.Reg)&^(0x200000)|value<<21)
}
func (o *PPB_Type) GetTRCRSCTLR3_PAIRINV() uint32 {
	return (volatile.LoadUint32(&o.TRCRSCTLR3.Reg) & 0x200000) >> 21
}
func (o *PPB_Type) SetTRCRSCTLR3_INV(value uint32) {
	volatile.StoreUint32(&o.TRCRSCTLR3.Reg, volatile.LoadUint32(&o.TRCRSCTLR3.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetTRCRSCTLR3_INV() uint32 {
	return (volatile.LoadUint32(&o.TRCRSCTLR3.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetTRCRSCTLR3_GROUP(value uint32) {
	volatile.StoreUint32(&o.TRCRSCTLR3.Reg, volatile.LoadUint32(&o.TRCRSCTLR3.Reg)&^(0x70000)|value<<16)
}
func (o *PPB_Type) GetTRCRSCTLR3_GROUP() uint32 {
	return (volatile.LoadUint32(&o.TRCRSCTLR3.Reg) & 0x70000) >> 16
}
func (o *PPB_Type) SetTRCRSCTLR3_SELECT(value uint32) {
	volatile.StoreUint32(&o.TRCRSCTLR3.Reg, volatile.LoadUint32(&o.TRCRSCTLR3.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetTRCRSCTLR3_SELECT() uint32 {
	return volatile.LoadUint32(&o.TRCRSCTLR3.Reg) & 0xff
}

// PPB.TRCSSCSR: Controls the corresponding single-shot comparator resource
func (o *PPB_Type) SetTRCSSCSR_STATUS(value uint32) {
	volatile.StoreUint32(&o.TRCSSCSR.Reg, volatile.LoadUint32(&o.TRCSSCSR.Reg)&^(0x80000000)|value<<31)
}
func (o *PPB_Type) GetTRCSSCSR_STATUS() uint32 {
	return (volatile.LoadUint32(&o.TRCSSCSR.Reg) & 0x80000000) >> 31
}
func (o *PPB_Type) SetTRCSSCSR_PC(value uint32) {
	volatile.StoreUint32(&o.TRCSSCSR.Reg, volatile.LoadUint32(&o.TRCSSCSR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetTRCSSCSR_PC() uint32 {
	return (volatile.LoadUint32(&o.TRCSSCSR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetTRCSSCSR_DV(value uint32) {
	volatile.StoreUint32(&o.TRCSSCSR.Reg, volatile.LoadUint32(&o.TRCSSCSR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetTRCSSCSR_DV() uint32 {
	return (volatile.LoadUint32(&o.TRCSSCSR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetTRCSSCSR_DA(value uint32) {
	volatile.StoreUint32(&o.TRCSSCSR.Reg, volatile.LoadUint32(&o.TRCSSCSR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetTRCSSCSR_DA() uint32 {
	return (volatile.LoadUint32(&o.TRCSSCSR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetTRCSSCSR_INST(value uint32) {
	volatile.StoreUint32(&o.TRCSSCSR.Reg, volatile.LoadUint32(&o.TRCSSCSR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCSSCSR_INST() uint32 {
	return volatile.LoadUint32(&o.TRCSSCSR.Reg) & 0x1
}

// PPB.TRCSSPCICR: Selects the PE comparator inputs for Single-shot control
func (o *PPB_Type) SetTRCSSPCICR_PC(value uint32) {
	volatile.StoreUint32(&o.TRCSSPCICR.Reg, volatile.LoadUint32(&o.TRCSSPCICR.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetTRCSSPCICR_PC() uint32 {
	return volatile.LoadUint32(&o.TRCSSPCICR.Reg) & 0xf
}

// PPB.TRCPDCR: Requests the system to provide power to the trace unit
func (o *PPB_Type) SetTRCPDCR_PU(value uint32) {
	volatile.StoreUint32(&o.TRCPDCR.Reg, volatile.LoadUint32(&o.TRCPDCR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetTRCPDCR_PU() uint32 {
	return (volatile.LoadUint32(&o.TRCPDCR.Reg) & 0x8) >> 3
}

// PPB.TRCPDSR: Returns the following information about the trace unit: - OS Lock status. - Core power domain status. - Power interruption status
func (o *PPB_Type) SetTRCPDSR_OSLK(value uint32) {
	volatile.StoreUint32(&o.TRCPDSR.Reg, volatile.LoadUint32(&o.TRCPDSR.Reg)&^(0x20)|value<<5)
}
func (o *PPB_Type) GetTRCPDSR_OSLK() uint32 {
	return (volatile.LoadUint32(&o.TRCPDSR.Reg) & 0x20) >> 5
}
func (o *PPB_Type) SetTRCPDSR_STICKYPD(value uint32) {
	volatile.StoreUint32(&o.TRCPDSR.Reg, volatile.LoadUint32(&o.TRCPDSR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetTRCPDSR_STICKYPD() uint32 {
	return (volatile.LoadUint32(&o.TRCPDSR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetTRCPDSR_POWER(value uint32) {
	volatile.StoreUint32(&o.TRCPDSR.Reg, volatile.LoadUint32(&o.TRCPDSR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCPDSR_POWER() uint32 {
	return volatile.LoadUint32(&o.TRCPDSR.Reg) & 0x1
}

// PPB.TRCITATBIDR: Trace Integration ATB Identification Register
func (o *PPB_Type) SetTRCITATBIDR_ID(value uint32) {
	volatile.StoreUint32(&o.TRCITATBIDR.Reg, volatile.LoadUint32(&o.TRCITATBIDR.Reg)&^(0x7f)|value)
}
func (o *PPB_Type) GetTRCITATBIDR_ID() uint32 {
	return volatile.LoadUint32(&o.TRCITATBIDR.Reg) & 0x7f
}

// PPB.TRCITIATBINR: Trace Integration Instruction ATB In Register
func (o *PPB_Type) SetTRCITIATBINR_AFVALIDM(value uint32) {
	volatile.StoreUint32(&o.TRCITIATBINR.Reg, volatile.LoadUint32(&o.TRCITIATBINR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetTRCITIATBINR_AFVALIDM() uint32 {
	return (volatile.LoadUint32(&o.TRCITIATBINR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetTRCITIATBINR_ATREADYM(value uint32) {
	volatile.StoreUint32(&o.TRCITIATBINR.Reg, volatile.LoadUint32(&o.TRCITIATBINR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCITIATBINR_ATREADYM() uint32 {
	return volatile.LoadUint32(&o.TRCITIATBINR.Reg) & 0x1
}

// PPB.TRCITIATBOUTR: Trace Integration Instruction ATB Out Register
func (o *PPB_Type) SetTRCITIATBOUTR_AFREADY(value uint32) {
	volatile.StoreUint32(&o.TRCITIATBOUTR.Reg, volatile.LoadUint32(&o.TRCITIATBOUTR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetTRCITIATBOUTR_AFREADY() uint32 {
	return (volatile.LoadUint32(&o.TRCITIATBOUTR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetTRCITIATBOUTR_ATVALID(value uint32) {
	volatile.StoreUint32(&o.TRCITIATBOUTR.Reg, volatile.LoadUint32(&o.TRCITIATBOUTR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCITIATBOUTR_ATVALID() uint32 {
	return volatile.LoadUint32(&o.TRCITIATBOUTR.Reg) & 0x1
}

// PPB.TRCCLAIMSET: Claim Tag Set Register
func (o *PPB_Type) SetTRCCLAIMSET_SET3(value uint32) {
	volatile.StoreUint32(&o.TRCCLAIMSET.Reg, volatile.LoadUint32(&o.TRCCLAIMSET.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetTRCCLAIMSET_SET3() uint32 {
	return (volatile.LoadUint32(&o.TRCCLAIMSET.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetTRCCLAIMSET_SET2(value uint32) {
	volatile.StoreUint32(&o.TRCCLAIMSET.Reg, volatile.LoadUint32(&o.TRCCLAIMSET.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetTRCCLAIMSET_SET2() uint32 {
	return (volatile.LoadUint32(&o.TRCCLAIMSET.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetTRCCLAIMSET_SET1(value uint32) {
	volatile.StoreUint32(&o.TRCCLAIMSET.Reg, volatile.LoadUint32(&o.TRCCLAIMSET.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetTRCCLAIMSET_SET1() uint32 {
	return (volatile.LoadUint32(&o.TRCCLAIMSET.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetTRCCLAIMSET_SET0(value uint32) {
	volatile.StoreUint32(&o.TRCCLAIMSET.Reg, volatile.LoadUint32(&o.TRCCLAIMSET.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCCLAIMSET_SET0() uint32 {
	return volatile.LoadUint32(&o.TRCCLAIMSET.Reg) & 0x1
}

// PPB.TRCCLAIMCLR: Claim Tag Clear Register
func (o *PPB_Type) SetTRCCLAIMCLR_CLR3(value uint32) {
	volatile.StoreUint32(&o.TRCCLAIMCLR.Reg, volatile.LoadUint32(&o.TRCCLAIMCLR.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetTRCCLAIMCLR_CLR3() uint32 {
	return (volatile.LoadUint32(&o.TRCCLAIMCLR.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetTRCCLAIMCLR_CLR2(value uint32) {
	volatile.StoreUint32(&o.TRCCLAIMCLR.Reg, volatile.LoadUint32(&o.TRCCLAIMCLR.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetTRCCLAIMCLR_CLR2() uint32 {
	return (volatile.LoadUint32(&o.TRCCLAIMCLR.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetTRCCLAIMCLR_CLR1(value uint32) {
	volatile.StoreUint32(&o.TRCCLAIMCLR.Reg, volatile.LoadUint32(&o.TRCCLAIMCLR.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetTRCCLAIMCLR_CLR1() uint32 {
	return (volatile.LoadUint32(&o.TRCCLAIMCLR.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetTRCCLAIMCLR_CLR0(value uint32) {
	volatile.StoreUint32(&o.TRCCLAIMCLR.Reg, volatile.LoadUint32(&o.TRCCLAIMCLR.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetTRCCLAIMCLR_CLR0() uint32 {
	return volatile.LoadUint32(&o.TRCCLAIMCLR.Reg) & 0x1
}

// PPB.TRCAUTHSTATUS: Returns the level of tracing that the trace unit can support
func (o *PPB_Type) SetTRCAUTHSTATUS_SNID(value uint32) {
	volatile.StoreUint32(&o.TRCAUTHSTATUS.Reg, volatile.LoadUint32(&o.TRCAUTHSTATUS.Reg)&^(0xc0)|value<<6)
}
func (o *PPB_Type) GetTRCAUTHSTATUS_SNID() uint32 {
	return (volatile.LoadUint32(&o.TRCAUTHSTATUS.Reg) & 0xc0) >> 6
}
func (o *PPB_Type) SetTRCAUTHSTATUS_SID(value uint32) {
	volatile.StoreUint32(&o.TRCAUTHSTATUS.Reg, volatile.LoadUint32(&o.TRCAUTHSTATUS.Reg)&^(0x30)|value<<4)
}
func (o *PPB_Type) GetTRCAUTHSTATUS_SID() uint32 {
	return (volatile.LoadUint32(&o.TRCAUTHSTATUS.Reg) & 0x30) >> 4
}
func (o *PPB_Type) SetTRCAUTHSTATUS_NSNID(value uint32) {
	volatile.StoreUint32(&o.TRCAUTHSTATUS.Reg, volatile.LoadUint32(&o.TRCAUTHSTATUS.Reg)&^(0xc)|value<<2)
}
func (o *PPB_Type) GetTRCAUTHSTATUS_NSNID() uint32 {
	return (volatile.LoadUint32(&o.TRCAUTHSTATUS.Reg) & 0xc) >> 2
}
func (o *PPB_Type) SetTRCAUTHSTATUS_NSID(value uint32) {
	volatile.StoreUint32(&o.TRCAUTHSTATUS.Reg, volatile.LoadUint32(&o.TRCAUTHSTATUS.Reg)&^(0x3)|value)
}
func (o *PPB_Type) GetTRCAUTHSTATUS_NSID() uint32 {
	return volatile.LoadUint32(&o.TRCAUTHSTATUS.Reg) & 0x3
}

// PPB.TRCDEVARCH: TRCDEVARCH
func (o *PPB_Type) SetTRCDEVARCH_ARCHITECT(value uint32) {
	volatile.StoreUint32(&o.TRCDEVARCH.Reg, volatile.LoadUint32(&o.TRCDEVARCH.Reg)&^(0xffe00000)|value<<21)
}
func (o *PPB_Type) GetTRCDEVARCH_ARCHITECT() uint32 {
	return (volatile.LoadUint32(&o.TRCDEVARCH.Reg) & 0xffe00000) >> 21
}
func (o *PPB_Type) SetTRCDEVARCH_PRESENT(value uint32) {
	volatile.StoreUint32(&o.TRCDEVARCH.Reg, volatile.LoadUint32(&o.TRCDEVARCH.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetTRCDEVARCH_PRESENT() uint32 {
	return (volatile.LoadUint32(&o.TRCDEVARCH.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetTRCDEVARCH_REVISION(value uint32) {
	volatile.StoreUint32(&o.TRCDEVARCH.Reg, volatile.LoadUint32(&o.TRCDEVARCH.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetTRCDEVARCH_REVISION() uint32 {
	return (volatile.LoadUint32(&o.TRCDEVARCH.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetTRCDEVARCH_ARCHID(value uint32) {
	volatile.StoreUint32(&o.TRCDEVARCH.Reg, volatile.LoadUint32(&o.TRCDEVARCH.Reg)&^(0xffff)|value)
}
func (o *PPB_Type) GetTRCDEVARCH_ARCHID() uint32 {
	return volatile.LoadUint32(&o.TRCDEVARCH.Reg) & 0xffff
}

// PPB.TRCDEVID: TRCDEVID
func (o *PPB_Type) SetTRCDEVID(value uint32) {
	volatile.StoreUint32(&o.TRCDEVID.Reg, value)
}
func (o *PPB_Type) GetTRCDEVID() uint32 {
	return volatile.LoadUint32(&o.TRCDEVID.Reg)
}

// PPB.TRCDEVTYPE: TRCDEVTYPE
func (o *PPB_Type) SetTRCDEVTYPE_SUB(value uint32) {
	volatile.StoreUint32(&o.TRCDEVTYPE.Reg, volatile.LoadUint32(&o.TRCDEVTYPE.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetTRCDEVTYPE_SUB() uint32 {
	return (volatile.LoadUint32(&o.TRCDEVTYPE.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetTRCDEVTYPE_MAJOR(value uint32) {
	volatile.StoreUint32(&o.TRCDEVTYPE.Reg, volatile.LoadUint32(&o.TRCDEVTYPE.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetTRCDEVTYPE_MAJOR() uint32 {
	return volatile.LoadUint32(&o.TRCDEVTYPE.Reg) & 0xf
}

// PPB.TRCPIDR4: TRCPIDR4
func (o *PPB_Type) SetTRCPIDR4_SIZE(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR4.Reg, volatile.LoadUint32(&o.TRCPIDR4.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetTRCPIDR4_SIZE() uint32 {
	return (volatile.LoadUint32(&o.TRCPIDR4.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetTRCPIDR4_DES_2(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR4.Reg, volatile.LoadUint32(&o.TRCPIDR4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetTRCPIDR4_DES_2() uint32 {
	return volatile.LoadUint32(&o.TRCPIDR4.Reg) & 0xf
}

// PPB.TRCPIDR5: TRCPIDR5
func (o *PPB_Type) SetTRCPIDR5(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR5.Reg, value)
}
func (o *PPB_Type) GetTRCPIDR5() uint32 {
	return volatile.LoadUint32(&o.TRCPIDR5.Reg)
}

// PPB.TRCPIDR6: TRCPIDR6
func (o *PPB_Type) SetTRCPIDR6(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR6.Reg, value)
}
func (o *PPB_Type) GetTRCPIDR6() uint32 {
	return volatile.LoadUint32(&o.TRCPIDR6.Reg)
}

// PPB.TRCPIDR7: TRCPIDR7
func (o *PPB_Type) SetTRCPIDR7(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR7.Reg, value)
}
func (o *PPB_Type) GetTRCPIDR7() uint32 {
	return volatile.LoadUint32(&o.TRCPIDR7.Reg)
}

// PPB.TRCPIDR0: TRCPIDR0
func (o *PPB_Type) SetTRCPIDR0_PART_0(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR0.Reg, volatile.LoadUint32(&o.TRCPIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetTRCPIDR0_PART_0() uint32 {
	return volatile.LoadUint32(&o.TRCPIDR0.Reg) & 0xff
}

// PPB.TRCPIDR1: TRCPIDR1
func (o *PPB_Type) SetTRCPIDR1_DES_0(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR1.Reg, volatile.LoadUint32(&o.TRCPIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetTRCPIDR1_DES_0() uint32 {
	return (volatile.LoadUint32(&o.TRCPIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetTRCPIDR1_PART_0(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR1.Reg, volatile.LoadUint32(&o.TRCPIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetTRCPIDR1_PART_0() uint32 {
	return volatile.LoadUint32(&o.TRCPIDR1.Reg) & 0xf
}

// PPB.TRCPIDR2: TRCPIDR2
func (o *PPB_Type) SetTRCPIDR2_REVISION(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR2.Reg, volatile.LoadUint32(&o.TRCPIDR2.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetTRCPIDR2_REVISION() uint32 {
	return (volatile.LoadUint32(&o.TRCPIDR2.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetTRCPIDR2_JEDEC(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR2.Reg, volatile.LoadUint32(&o.TRCPIDR2.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetTRCPIDR2_JEDEC() uint32 {
	return (volatile.LoadUint32(&o.TRCPIDR2.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetTRCPIDR2_DES_0(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR2.Reg, volatile.LoadUint32(&o.TRCPIDR2.Reg)&^(0x7)|value)
}
func (o *PPB_Type) GetTRCPIDR2_DES_0() uint32 {
	return volatile.LoadUint32(&o.TRCPIDR2.Reg) & 0x7
}

// PPB.TRCPIDR3: TRCPIDR3
func (o *PPB_Type) SetTRCPIDR3_REVAND(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR3.Reg, volatile.LoadUint32(&o.TRCPIDR3.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetTRCPIDR3_REVAND() uint32 {
	return (volatile.LoadUint32(&o.TRCPIDR3.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetTRCPIDR3_CMOD(value uint32) {
	volatile.StoreUint32(&o.TRCPIDR3.Reg, volatile.LoadUint32(&o.TRCPIDR3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetTRCPIDR3_CMOD() uint32 {
	return volatile.LoadUint32(&o.TRCPIDR3.Reg) & 0xf
}

// PPB.TRCCIDR0: TRCCIDR0
func (o *PPB_Type) SetTRCCIDR0_PRMBL_0(value uint32) {
	volatile.StoreUint32(&o.TRCCIDR0.Reg, volatile.LoadUint32(&o.TRCCIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetTRCCIDR0_PRMBL_0() uint32 {
	return volatile.LoadUint32(&o.TRCCIDR0.Reg) & 0xff
}

// PPB.TRCCIDR1: TRCCIDR1
func (o *PPB_Type) SetTRCCIDR1_CLASS(value uint32) {
	volatile.StoreUint32(&o.TRCCIDR1.Reg, volatile.LoadUint32(&o.TRCCIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetTRCCIDR1_CLASS() uint32 {
	return (volatile.LoadUint32(&o.TRCCIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetTRCCIDR1_PRMBL_1(value uint32) {
	volatile.StoreUint32(&o.TRCCIDR1.Reg, volatile.LoadUint32(&o.TRCCIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetTRCCIDR1_PRMBL_1() uint32 {
	return volatile.LoadUint32(&o.TRCCIDR1.Reg) & 0xf
}

// PPB.TRCCIDR2: TRCCIDR2
func (o *PPB_Type) SetTRCCIDR2_PRMBL_2(value uint32) {
	volatile.StoreUint32(&o.TRCCIDR2.Reg, volatile.LoadUint32(&o.TRCCIDR2.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetTRCCIDR2_PRMBL_2() uint32 {
	return volatile.LoadUint32(&o.TRCCIDR2.Reg) & 0xff
}

// PPB.TRCCIDR3: TRCCIDR3
func (o *PPB_Type) SetTRCCIDR3_PRMBL_3(value uint32) {
	volatile.StoreUint32(&o.TRCCIDR3.Reg, volatile.LoadUint32(&o.TRCCIDR3.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetTRCCIDR3_PRMBL_3() uint32 {
	return volatile.LoadUint32(&o.TRCCIDR3.Reg) & 0xff
}

// PPB.CTICONTROL: CTI Control Register
func (o *PPB_Type) SetCTICONTROL_GLBEN(value uint32) {
	volatile.StoreUint32(&o.CTICONTROL.Reg, volatile.LoadUint32(&o.CTICONTROL.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetCTICONTROL_GLBEN() uint32 {
	return volatile.LoadUint32(&o.CTICONTROL.Reg) & 0x1
}

// PPB.CTIINTACK: CTI Interrupt Acknowledge Register
func (o *PPB_Type) SetCTIINTACK_INTACK(value uint32) {
	volatile.StoreUint32(&o.CTIINTACK.Reg, volatile.LoadUint32(&o.CTIINTACK.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetCTIINTACK_INTACK() uint32 {
	return volatile.LoadUint32(&o.CTIINTACK.Reg) & 0xff
}

// PPB.CTIAPPSET: CTI Application Trigger Set Register
func (o *PPB_Type) SetCTIAPPSET_APPSET(value uint32) {
	volatile.StoreUint32(&o.CTIAPPSET.Reg, volatile.LoadUint32(&o.CTIAPPSET.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIAPPSET_APPSET() uint32 {
	return volatile.LoadUint32(&o.CTIAPPSET.Reg) & 0xf
}

// PPB.CTIAPPCLEAR: CTI Application Trigger Clear Register
func (o *PPB_Type) SetCTIAPPCLEAR_APPCLEAR(value uint32) {
	volatile.StoreUint32(&o.CTIAPPCLEAR.Reg, volatile.LoadUint32(&o.CTIAPPCLEAR.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIAPPCLEAR_APPCLEAR() uint32 {
	return volatile.LoadUint32(&o.CTIAPPCLEAR.Reg) & 0xf
}

// PPB.CTIAPPPULSE: CTI Application Pulse Register
func (o *PPB_Type) SetCTIAPPPULSE_APPULSE(value uint32) {
	volatile.StoreUint32(&o.CTIAPPPULSE.Reg, volatile.LoadUint32(&o.CTIAPPPULSE.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIAPPPULSE_APPULSE() uint32 {
	return volatile.LoadUint32(&o.CTIAPPPULSE.Reg) & 0xf
}

// PPB.CTIINEN0: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIINEN0_TRIGINEN(value uint32) {
	volatile.StoreUint32(&o.CTIINEN0.Reg, volatile.LoadUint32(&o.CTIINEN0.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIINEN0_TRIGINEN() uint32 {
	return volatile.LoadUint32(&o.CTIINEN0.Reg) & 0xf
}

// PPB.CTIINEN1: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIINEN1_TRIGINEN(value uint32) {
	volatile.StoreUint32(&o.CTIINEN1.Reg, volatile.LoadUint32(&o.CTIINEN1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIINEN1_TRIGINEN() uint32 {
	return volatile.LoadUint32(&o.CTIINEN1.Reg) & 0xf
}

// PPB.CTIINEN2: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIINEN2_TRIGINEN(value uint32) {
	volatile.StoreUint32(&o.CTIINEN2.Reg, volatile.LoadUint32(&o.CTIINEN2.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIINEN2_TRIGINEN() uint32 {
	return volatile.LoadUint32(&o.CTIINEN2.Reg) & 0xf
}

// PPB.CTIINEN3: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIINEN3_TRIGINEN(value uint32) {
	volatile.StoreUint32(&o.CTIINEN3.Reg, volatile.LoadUint32(&o.CTIINEN3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIINEN3_TRIGINEN() uint32 {
	return volatile.LoadUint32(&o.CTIINEN3.Reg) & 0xf
}

// PPB.CTIINEN4: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIINEN4_TRIGINEN(value uint32) {
	volatile.StoreUint32(&o.CTIINEN4.Reg, volatile.LoadUint32(&o.CTIINEN4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIINEN4_TRIGINEN() uint32 {
	return volatile.LoadUint32(&o.CTIINEN4.Reg) & 0xf
}

// PPB.CTIINEN5: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIINEN5_TRIGINEN(value uint32) {
	volatile.StoreUint32(&o.CTIINEN5.Reg, volatile.LoadUint32(&o.CTIINEN5.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIINEN5_TRIGINEN() uint32 {
	return volatile.LoadUint32(&o.CTIINEN5.Reg) & 0xf
}

// PPB.CTIINEN6: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIINEN6_TRIGINEN(value uint32) {
	volatile.StoreUint32(&o.CTIINEN6.Reg, volatile.LoadUint32(&o.CTIINEN6.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIINEN6_TRIGINEN() uint32 {
	return volatile.LoadUint32(&o.CTIINEN6.Reg) & 0xf
}

// PPB.CTIINEN7: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIINEN7_TRIGINEN(value uint32) {
	volatile.StoreUint32(&o.CTIINEN7.Reg, volatile.LoadUint32(&o.CTIINEN7.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIINEN7_TRIGINEN() uint32 {
	return volatile.LoadUint32(&o.CTIINEN7.Reg) & 0xf
}

// PPB.CTIOUTEN0: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIOUTEN0_TRIGOUTEN(value uint32) {
	volatile.StoreUint32(&o.CTIOUTEN0.Reg, volatile.LoadUint32(&o.CTIOUTEN0.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIOUTEN0_TRIGOUTEN() uint32 {
	return volatile.LoadUint32(&o.CTIOUTEN0.Reg) & 0xf
}

// PPB.CTIOUTEN1: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIOUTEN1_TRIGOUTEN(value uint32) {
	volatile.StoreUint32(&o.CTIOUTEN1.Reg, volatile.LoadUint32(&o.CTIOUTEN1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIOUTEN1_TRIGOUTEN() uint32 {
	return volatile.LoadUint32(&o.CTIOUTEN1.Reg) & 0xf
}

// PPB.CTIOUTEN2: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIOUTEN2_TRIGOUTEN(value uint32) {
	volatile.StoreUint32(&o.CTIOUTEN2.Reg, volatile.LoadUint32(&o.CTIOUTEN2.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIOUTEN2_TRIGOUTEN() uint32 {
	return volatile.LoadUint32(&o.CTIOUTEN2.Reg) & 0xf
}

// PPB.CTIOUTEN3: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIOUTEN3_TRIGOUTEN(value uint32) {
	volatile.StoreUint32(&o.CTIOUTEN3.Reg, volatile.LoadUint32(&o.CTIOUTEN3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIOUTEN3_TRIGOUTEN() uint32 {
	return volatile.LoadUint32(&o.CTIOUTEN3.Reg) & 0xf
}

// PPB.CTIOUTEN4: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIOUTEN4_TRIGOUTEN(value uint32) {
	volatile.StoreUint32(&o.CTIOUTEN4.Reg, volatile.LoadUint32(&o.CTIOUTEN4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIOUTEN4_TRIGOUTEN() uint32 {
	return volatile.LoadUint32(&o.CTIOUTEN4.Reg) & 0xf
}

// PPB.CTIOUTEN5: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIOUTEN5_TRIGOUTEN(value uint32) {
	volatile.StoreUint32(&o.CTIOUTEN5.Reg, volatile.LoadUint32(&o.CTIOUTEN5.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIOUTEN5_TRIGOUTEN() uint32 {
	return volatile.LoadUint32(&o.CTIOUTEN5.Reg) & 0xf
}

// PPB.CTIOUTEN6: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIOUTEN6_TRIGOUTEN(value uint32) {
	volatile.StoreUint32(&o.CTIOUTEN6.Reg, volatile.LoadUint32(&o.CTIOUTEN6.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIOUTEN6_TRIGOUTEN() uint32 {
	return volatile.LoadUint32(&o.CTIOUTEN6.Reg) & 0xf
}

// PPB.CTIOUTEN7: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTIOUTEN7_TRIGOUTEN(value uint32) {
	volatile.StoreUint32(&o.CTIOUTEN7.Reg, volatile.LoadUint32(&o.CTIOUTEN7.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTIOUTEN7_TRIGOUTEN() uint32 {
	return volatile.LoadUint32(&o.CTIOUTEN7.Reg) & 0xf
}

// PPB.CTITRIGINSTATUS: CTI Trigger to Channel Enable Registers
func (o *PPB_Type) SetCTITRIGINSTATUS_TRIGINSTATUS(value uint32) {
	volatile.StoreUint32(&o.CTITRIGINSTATUS.Reg, volatile.LoadUint32(&o.CTITRIGINSTATUS.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetCTITRIGINSTATUS_TRIGINSTATUS() uint32 {
	return volatile.LoadUint32(&o.CTITRIGINSTATUS.Reg) & 0xff
}

// PPB.CTITRIGOUTSTATUS: CTI Trigger In Status Register
func (o *PPB_Type) SetCTITRIGOUTSTATUS_TRIGOUTSTATUS(value uint32) {
	volatile.StoreUint32(&o.CTITRIGOUTSTATUS.Reg, volatile.LoadUint32(&o.CTITRIGOUTSTATUS.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetCTITRIGOUTSTATUS_TRIGOUTSTATUS() uint32 {
	return volatile.LoadUint32(&o.CTITRIGOUTSTATUS.Reg) & 0xff
}

// PPB.CTICHINSTATUS: CTI Channel In Status Register
func (o *PPB_Type) SetCTICHINSTATUS_CTICHOUTSTATUS(value uint32) {
	volatile.StoreUint32(&o.CTICHINSTATUS.Reg, volatile.LoadUint32(&o.CTICHINSTATUS.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCTICHINSTATUS_CTICHOUTSTATUS() uint32 {
	return volatile.LoadUint32(&o.CTICHINSTATUS.Reg) & 0xf
}

// PPB.CTIGATE: Enable CTI Channel Gate register
func (o *PPB_Type) SetCTIGATE_CTIGATEEN3(value uint32) {
	volatile.StoreUint32(&o.CTIGATE.Reg, volatile.LoadUint32(&o.CTIGATE.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetCTIGATE_CTIGATEEN3() uint32 {
	return (volatile.LoadUint32(&o.CTIGATE.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetCTIGATE_CTIGATEEN2(value uint32) {
	volatile.StoreUint32(&o.CTIGATE.Reg, volatile.LoadUint32(&o.CTIGATE.Reg)&^(0x4)|value<<2)
}
func (o *PPB_Type) GetCTIGATE_CTIGATEEN2() uint32 {
	return (volatile.LoadUint32(&o.CTIGATE.Reg) & 0x4) >> 2
}
func (o *PPB_Type) SetCTIGATE_CTIGATEEN1(value uint32) {
	volatile.StoreUint32(&o.CTIGATE.Reg, volatile.LoadUint32(&o.CTIGATE.Reg)&^(0x2)|value<<1)
}
func (o *PPB_Type) GetCTIGATE_CTIGATEEN1() uint32 {
	return (volatile.LoadUint32(&o.CTIGATE.Reg) & 0x2) >> 1
}
func (o *PPB_Type) SetCTIGATE_CTIGATEEN0(value uint32) {
	volatile.StoreUint32(&o.CTIGATE.Reg, volatile.LoadUint32(&o.CTIGATE.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetCTIGATE_CTIGATEEN0() uint32 {
	return volatile.LoadUint32(&o.CTIGATE.Reg) & 0x1
}

// PPB.ASICCTL: External Multiplexer Control register
func (o *PPB_Type) SetASICCTL(value uint32) {
	volatile.StoreUint32(&o.ASICCTL.Reg, value)
}
func (o *PPB_Type) GetASICCTL() uint32 {
	return volatile.LoadUint32(&o.ASICCTL.Reg)
}

// PPB.ITCHOUT: Integration Test Channel Output register
func (o *PPB_Type) SetITCHOUT_CTCHOUT(value uint32) {
	volatile.StoreUint32(&o.ITCHOUT.Reg, volatile.LoadUint32(&o.ITCHOUT.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetITCHOUT_CTCHOUT() uint32 {
	return volatile.LoadUint32(&o.ITCHOUT.Reg) & 0xf
}

// PPB.ITTRIGOUT: Integration Test Trigger Output register
func (o *PPB_Type) SetITTRIGOUT_CTTRIGOUT(value uint32) {
	volatile.StoreUint32(&o.ITTRIGOUT.Reg, volatile.LoadUint32(&o.ITTRIGOUT.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetITTRIGOUT_CTTRIGOUT() uint32 {
	return volatile.LoadUint32(&o.ITTRIGOUT.Reg) & 0xff
}

// PPB.ITCHIN: Integration Test Channel Input register
func (o *PPB_Type) SetITCHIN_CTCHIN(value uint32) {
	volatile.StoreUint32(&o.ITCHIN.Reg, volatile.LoadUint32(&o.ITCHIN.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetITCHIN_CTCHIN() uint32 {
	return volatile.LoadUint32(&o.ITCHIN.Reg) & 0xf
}

// PPB.ITCTRL: Integration Mode Control register
func (o *PPB_Type) SetITCTRL_IME(value uint32) {
	volatile.StoreUint32(&o.ITCTRL.Reg, volatile.LoadUint32(&o.ITCTRL.Reg)&^(0x1)|value)
}
func (o *PPB_Type) GetITCTRL_IME() uint32 {
	return volatile.LoadUint32(&o.ITCTRL.Reg) & 0x1
}

// PPB.DEVARCH: Device Architecture register
func (o *PPB_Type) SetDEVARCH_ARCHITECT(value uint32) {
	volatile.StoreUint32(&o.DEVARCH.Reg, volatile.LoadUint32(&o.DEVARCH.Reg)&^(0xffe00000)|value<<21)
}
func (o *PPB_Type) GetDEVARCH_ARCHITECT() uint32 {
	return (volatile.LoadUint32(&o.DEVARCH.Reg) & 0xffe00000) >> 21
}
func (o *PPB_Type) SetDEVARCH_PRESENT(value uint32) {
	volatile.StoreUint32(&o.DEVARCH.Reg, volatile.LoadUint32(&o.DEVARCH.Reg)&^(0x100000)|value<<20)
}
func (o *PPB_Type) GetDEVARCH_PRESENT() uint32 {
	return (volatile.LoadUint32(&o.DEVARCH.Reg) & 0x100000) >> 20
}
func (o *PPB_Type) SetDEVARCH_REVISION(value uint32) {
	volatile.StoreUint32(&o.DEVARCH.Reg, volatile.LoadUint32(&o.DEVARCH.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetDEVARCH_REVISION() uint32 {
	return (volatile.LoadUint32(&o.DEVARCH.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetDEVARCH_ARCHID(value uint32) {
	volatile.StoreUint32(&o.DEVARCH.Reg, volatile.LoadUint32(&o.DEVARCH.Reg)&^(0xffff)|value)
}
func (o *PPB_Type) GetDEVARCH_ARCHID() uint32 {
	return volatile.LoadUint32(&o.DEVARCH.Reg) & 0xffff
}

// PPB.DEVID: Device Configuration register
func (o *PPB_Type) SetDEVID_NUMCH(value uint32) {
	volatile.StoreUint32(&o.DEVID.Reg, volatile.LoadUint32(&o.DEVID.Reg)&^(0xf0000)|value<<16)
}
func (o *PPB_Type) GetDEVID_NUMCH() uint32 {
	return (volatile.LoadUint32(&o.DEVID.Reg) & 0xf0000) >> 16
}
func (o *PPB_Type) SetDEVID_NUMTRIG(value uint32) {
	volatile.StoreUint32(&o.DEVID.Reg, volatile.LoadUint32(&o.DEVID.Reg)&^(0xff00)|value<<8)
}
func (o *PPB_Type) GetDEVID_NUMTRIG() uint32 {
	return (volatile.LoadUint32(&o.DEVID.Reg) & 0xff00) >> 8
}
func (o *PPB_Type) SetDEVID_EXTMUXNUM(value uint32) {
	volatile.StoreUint32(&o.DEVID.Reg, volatile.LoadUint32(&o.DEVID.Reg)&^(0x1f)|value)
}
func (o *PPB_Type) GetDEVID_EXTMUXNUM() uint32 {
	return volatile.LoadUint32(&o.DEVID.Reg) & 0x1f
}

// PPB.DEVTYPE: Device Type Identifier register
func (o *PPB_Type) SetDEVTYPE_SUB(value uint32) {
	volatile.StoreUint32(&o.DEVTYPE.Reg, volatile.LoadUint32(&o.DEVTYPE.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetDEVTYPE_SUB() uint32 {
	return (volatile.LoadUint32(&o.DEVTYPE.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetDEVTYPE_MAJOR(value uint32) {
	volatile.StoreUint32(&o.DEVTYPE.Reg, volatile.LoadUint32(&o.DEVTYPE.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetDEVTYPE_MAJOR() uint32 {
	return volatile.LoadUint32(&o.DEVTYPE.Reg) & 0xf
}

// PPB.PIDR4: CoreSight Peripheral ID4
func (o *PPB_Type) SetPIDR4_SIZE(value uint32) {
	volatile.StoreUint32(&o.PIDR4.Reg, volatile.LoadUint32(&o.PIDR4.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetPIDR4_SIZE() uint32 {
	return (volatile.LoadUint32(&o.PIDR4.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetPIDR4_DES_2(value uint32) {
	volatile.StoreUint32(&o.PIDR4.Reg, volatile.LoadUint32(&o.PIDR4.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetPIDR4_DES_2() uint32 {
	return volatile.LoadUint32(&o.PIDR4.Reg) & 0xf
}

// PPB.PIDR5: CoreSight Peripheral ID5
func (o *PPB_Type) SetPIDR5(value uint32) {
	volatile.StoreUint32(&o.PIDR5.Reg, value)
}
func (o *PPB_Type) GetPIDR5() uint32 {
	return volatile.LoadUint32(&o.PIDR5.Reg)
}

// PPB.PIDR6: CoreSight Peripheral ID6
func (o *PPB_Type) SetPIDR6(value uint32) {
	volatile.StoreUint32(&o.PIDR6.Reg, value)
}
func (o *PPB_Type) GetPIDR6() uint32 {
	return volatile.LoadUint32(&o.PIDR6.Reg)
}

// PPB.PIDR7: CoreSight Peripheral ID7
func (o *PPB_Type) SetPIDR7(value uint32) {
	volatile.StoreUint32(&o.PIDR7.Reg, value)
}
func (o *PPB_Type) GetPIDR7() uint32 {
	return volatile.LoadUint32(&o.PIDR7.Reg)
}

// PPB.PIDR0: CoreSight Peripheral ID0
func (o *PPB_Type) SetPIDR0_PART_0(value uint32) {
	volatile.StoreUint32(&o.PIDR0.Reg, volatile.LoadUint32(&o.PIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetPIDR0_PART_0() uint32 {
	return volatile.LoadUint32(&o.PIDR0.Reg) & 0xff
}

// PPB.PIDR1: CoreSight Peripheral ID1
func (o *PPB_Type) SetPIDR1_DES_0(value uint32) {
	volatile.StoreUint32(&o.PIDR1.Reg, volatile.LoadUint32(&o.PIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetPIDR1_DES_0() uint32 {
	return (volatile.LoadUint32(&o.PIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetPIDR1_PART_1(value uint32) {
	volatile.StoreUint32(&o.PIDR1.Reg, volatile.LoadUint32(&o.PIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetPIDR1_PART_1() uint32 {
	return volatile.LoadUint32(&o.PIDR1.Reg) & 0xf
}

// PPB.PIDR2: CoreSight Peripheral ID2
func (o *PPB_Type) SetPIDR2_REVISION(value uint32) {
	volatile.StoreUint32(&o.PIDR2.Reg, volatile.LoadUint32(&o.PIDR2.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetPIDR2_REVISION() uint32 {
	return (volatile.LoadUint32(&o.PIDR2.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetPIDR2_JEDEC(value uint32) {
	volatile.StoreUint32(&o.PIDR2.Reg, volatile.LoadUint32(&o.PIDR2.Reg)&^(0x8)|value<<3)
}
func (o *PPB_Type) GetPIDR2_JEDEC() uint32 {
	return (volatile.LoadUint32(&o.PIDR2.Reg) & 0x8) >> 3
}
func (o *PPB_Type) SetPIDR2_DES_1(value uint32) {
	volatile.StoreUint32(&o.PIDR2.Reg, volatile.LoadUint32(&o.PIDR2.Reg)&^(0x7)|value)
}
func (o *PPB_Type) GetPIDR2_DES_1() uint32 {
	return volatile.LoadUint32(&o.PIDR2.Reg) & 0x7
}

// PPB.PIDR3: CoreSight Peripheral ID3
func (o *PPB_Type) SetPIDR3_REVAND(value uint32) {
	volatile.StoreUint32(&o.PIDR3.Reg, volatile.LoadUint32(&o.PIDR3.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetPIDR3_REVAND() uint32 {
	return (volatile.LoadUint32(&o.PIDR3.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetPIDR3_CMOD(value uint32) {
	volatile.StoreUint32(&o.PIDR3.Reg, volatile.LoadUint32(&o.PIDR3.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetPIDR3_CMOD() uint32 {
	return volatile.LoadUint32(&o.PIDR3.Reg) & 0xf
}

// PPB.CIDR0: CoreSight Component ID0
func (o *PPB_Type) SetCIDR0_PRMBL_0(value uint32) {
	volatile.StoreUint32(&o.CIDR0.Reg, volatile.LoadUint32(&o.CIDR0.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetCIDR0_PRMBL_0() uint32 {
	return volatile.LoadUint32(&o.CIDR0.Reg) & 0xff
}

// PPB.CIDR1: CoreSight Component ID1
func (o *PPB_Type) SetCIDR1_CLASS(value uint32) {
	volatile.StoreUint32(&o.CIDR1.Reg, volatile.LoadUint32(&o.CIDR1.Reg)&^(0xf0)|value<<4)
}
func (o *PPB_Type) GetCIDR1_CLASS() uint32 {
	return (volatile.LoadUint32(&o.CIDR1.Reg) & 0xf0) >> 4
}
func (o *PPB_Type) SetCIDR1_PRMBL_1(value uint32) {
	volatile.StoreUint32(&o.CIDR1.Reg, volatile.LoadUint32(&o.CIDR1.Reg)&^(0xf)|value)
}
func (o *PPB_Type) GetCIDR1_PRMBL_1() uint32 {
	return volatile.LoadUint32(&o.CIDR1.Reg) & 0xf
}

// PPB.CIDR2: CoreSight Component ID2
func (o *PPB_Type) SetCIDR2_PRMBL_2(value uint32) {
	volatile.StoreUint32(&o.CIDR2.Reg, volatile.LoadUint32(&o.CIDR2.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetCIDR2_PRMBL_2() uint32 {
	return volatile.LoadUint32(&o.CIDR2.Reg) & 0xff
}

// PPB.CIDR3: CoreSight Component ID3
func (o *PPB_Type) SetCIDR3_PRMBL_3(value uint32) {
	volatile.StoreUint32(&o.CIDR3.Reg, volatile.LoadUint32(&o.CIDR3.Reg)&^(0xff)|value)
}
func (o *PPB_Type) GetCIDR3_PRMBL_3() uint32 {
	return volatile.LoadUint32(&o.CIDR3.Reg) & 0xff
}

// QSPI Memory Interface. Provides a memory-mapped interface to up to two SPI/DSPI/QSPI flash or PSRAM devices. Also provides a serial interface for programming and configuration of the external device.
type QMI_Type struct {
	DIRECT_CSR volatile.Register32 // 0x0
	DIRECT_TX  volatile.Register32 // 0x4
	DIRECT_RX  volatile.Register32 // 0x8
	M0_TIMING  volatile.Register32 // 0xC
	M0_RFMT    volatile.Register32 // 0x10
	M0_RCMD    volatile.Register32 // 0x14
	M0_WFMT    volatile.Register32 // 0x18
	M0_WCMD    volatile.Register32 // 0x1C
	M1_TIMING  volatile.Register32 // 0x20
	M1_RFMT    volatile.Register32 // 0x24
	M1_RCMD    volatile.Register32 // 0x28
	M1_WFMT    volatile.Register32 // 0x2C
	M1_WCMD    volatile.Register32 // 0x30
	ATRANS0    volatile.Register32 // 0x34
	ATRANS1    volatile.Register32 // 0x38
	ATRANS2    volatile.Register32 // 0x3C
	ATRANS3    volatile.Register32 // 0x40
	ATRANS4    volatile.Register32 // 0x44
	ATRANS5    volatile.Register32 // 0x48
	ATRANS6    volatile.Register32 // 0x4C
	ATRANS7    volatile.Register32 // 0x50
}

// QMI.DIRECT_CSR: Control and status for direct serial mode Direct serial mode allows the processor to send and receive raw serial frames, for programming, configuration and control of the external memory devices. Only SPI mode 0 (CPOL=0 CPHA=0) is supported.
func (o *QMI_Type) SetDIRECT_CSR_RXDELAY(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0xc0000000)|value<<30)
}
func (o *QMI_Type) GetDIRECT_CSR_RXDELAY() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0xc0000000) >> 30
}
func (o *QMI_Type) SetDIRECT_CSR_CLKDIV(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x3fc00000)|value<<22)
}
func (o *QMI_Type) GetDIRECT_CSR_CLKDIV() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x3fc00000) >> 22
}
func (o *QMI_Type) SetDIRECT_CSR_RXLEVEL(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x1c0000)|value<<18)
}
func (o *QMI_Type) GetDIRECT_CSR_RXLEVEL() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x1c0000) >> 18
}
func (o *QMI_Type) SetDIRECT_CSR_RXFULL(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x20000)|value<<17)
}
func (o *QMI_Type) GetDIRECT_CSR_RXFULL() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x20000) >> 17
}
func (o *QMI_Type) SetDIRECT_CSR_RXEMPTY(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x10000)|value<<16)
}
func (o *QMI_Type) GetDIRECT_CSR_RXEMPTY() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x10000) >> 16
}
func (o *QMI_Type) SetDIRECT_CSR_TXLEVEL(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x7000)|value<<12)
}
func (o *QMI_Type) GetDIRECT_CSR_TXLEVEL() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x7000) >> 12
}
func (o *QMI_Type) SetDIRECT_CSR_TXEMPTY(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x800)|value<<11)
}
func (o *QMI_Type) GetDIRECT_CSR_TXEMPTY() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x800) >> 11
}
func (o *QMI_Type) SetDIRECT_CSR_TXFULL(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x400)|value<<10)
}
func (o *QMI_Type) GetDIRECT_CSR_TXFULL() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x400) >> 10
}
func (o *QMI_Type) SetDIRECT_CSR_AUTO_CS1N(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x80)|value<<7)
}
func (o *QMI_Type) GetDIRECT_CSR_AUTO_CS1N() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x80) >> 7
}
func (o *QMI_Type) SetDIRECT_CSR_AUTO_CS0N(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x40)|value<<6)
}
func (o *QMI_Type) GetDIRECT_CSR_AUTO_CS0N() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x40) >> 6
}
func (o *QMI_Type) SetDIRECT_CSR_ASSERT_CS1N(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x8)|value<<3)
}
func (o *QMI_Type) GetDIRECT_CSR_ASSERT_CS1N() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x8) >> 3
}
func (o *QMI_Type) SetDIRECT_CSR_ASSERT_CS0N(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x4)|value<<2)
}
func (o *QMI_Type) GetDIRECT_CSR_ASSERT_CS0N() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x4) >> 2
}
func (o *QMI_Type) SetDIRECT_CSR_BUSY(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x2)|value<<1)
}
func (o *QMI_Type) GetDIRECT_CSR_BUSY() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x2) >> 1
}
func (o *QMI_Type) SetDIRECT_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.DIRECT_CSR.Reg, volatile.LoadUint32(&o.DIRECT_CSR.Reg)&^(0x1)|value)
}
func (o *QMI_Type) GetDIRECT_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.DIRECT_CSR.Reg) & 0x1
}

// QMI.DIRECT_TX: Transmit FIFO for direct mode
func (o *QMI_Type) SetDIRECT_TX_NOPUSH(value uint32) {
	volatile.StoreUint32(&o.DIRECT_TX.Reg, volatile.LoadUint32(&o.DIRECT_TX.Reg)&^(0x100000)|value<<20)
}
func (o *QMI_Type) GetDIRECT_TX_NOPUSH() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_TX.Reg) & 0x100000) >> 20
}
func (o *QMI_Type) SetDIRECT_TX_OE(value uint32) {
	volatile.StoreUint32(&o.DIRECT_TX.Reg, volatile.LoadUint32(&o.DIRECT_TX.Reg)&^(0x80000)|value<<19)
}
func (o *QMI_Type) GetDIRECT_TX_OE() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_TX.Reg) & 0x80000) >> 19
}
func (o *QMI_Type) SetDIRECT_TX_DWIDTH(value uint32) {
	volatile.StoreUint32(&o.DIRECT_TX.Reg, volatile.LoadUint32(&o.DIRECT_TX.Reg)&^(0x40000)|value<<18)
}
func (o *QMI_Type) GetDIRECT_TX_DWIDTH() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_TX.Reg) & 0x40000) >> 18
}
func (o *QMI_Type) SetDIRECT_TX_IWIDTH(value uint32) {
	volatile.StoreUint32(&o.DIRECT_TX.Reg, volatile.LoadUint32(&o.DIRECT_TX.Reg)&^(0x30000)|value<<16)
}
func (o *QMI_Type) GetDIRECT_TX_IWIDTH() uint32 {
	return (volatile.LoadUint32(&o.DIRECT_TX.Reg) & 0x30000) >> 16
}
func (o *QMI_Type) SetDIRECT_TX_DATA(value uint32) {
	volatile.StoreUint32(&o.DIRECT_TX.Reg, volatile.LoadUint32(&o.DIRECT_TX.Reg)&^(0xffff)|value)
}
func (o *QMI_Type) GetDIRECT_TX_DATA() uint32 {
	return volatile.LoadUint32(&o.DIRECT_TX.Reg) & 0xffff
}

// QMI.DIRECT_RX: Receive FIFO for direct mode
func (o *QMI_Type) SetDIRECT_RX(value uint32) {
	volatile.StoreUint32(&o.DIRECT_RX.Reg, volatile.LoadUint32(&o.DIRECT_RX.Reg)&^(0xffff)|value)
}
func (o *QMI_Type) GetDIRECT_RX() uint32 {
	return volatile.LoadUint32(&o.DIRECT_RX.Reg) & 0xffff
}

// QMI.M0_TIMING: Timing configuration register for memory address window 0.
func (o *QMI_Type) SetM0_TIMING_COOLDOWN(value uint32) {
	volatile.StoreUint32(&o.M0_TIMING.Reg, volatile.LoadUint32(&o.M0_TIMING.Reg)&^(0xc0000000)|value<<30)
}
func (o *QMI_Type) GetM0_TIMING_COOLDOWN() uint32 {
	return (volatile.LoadUint32(&o.M0_TIMING.Reg) & 0xc0000000) >> 30
}
func (o *QMI_Type) SetM0_TIMING_PAGEBREAK(value uint32) {
	volatile.StoreUint32(&o.M0_TIMING.Reg, volatile.LoadUint32(&o.M0_TIMING.Reg)&^(0x30000000)|value<<28)
}
func (o *QMI_Type) GetM0_TIMING_PAGEBREAK() uint32 {
	return (volatile.LoadUint32(&o.M0_TIMING.Reg) & 0x30000000) >> 28
}
func (o *QMI_Type) SetM0_TIMING_SELECT_SETUP(value uint32) {
	volatile.StoreUint32(&o.M0_TIMING.Reg, volatile.LoadUint32(&o.M0_TIMING.Reg)&^(0x2000000)|value<<25)
}
func (o *QMI_Type) GetM0_TIMING_SELECT_SETUP() uint32 {
	return (volatile.LoadUint32(&o.M0_TIMING.Reg) & 0x2000000) >> 25
}
func (o *QMI_Type) SetM0_TIMING_SELECT_HOLD(value uint32) {
	volatile.StoreUint32(&o.M0_TIMING.Reg, volatile.LoadUint32(&o.M0_TIMING.Reg)&^(0x1800000)|value<<23)
}
func (o *QMI_Type) GetM0_TIMING_SELECT_HOLD() uint32 {
	return (volatile.LoadUint32(&o.M0_TIMING.Reg) & 0x1800000) >> 23
}
func (o *QMI_Type) SetM0_TIMING_MAX_SELECT(value uint32) {
	volatile.StoreUint32(&o.M0_TIMING.Reg, volatile.LoadUint32(&o.M0_TIMING.Reg)&^(0x7e0000)|value<<17)
}
func (o *QMI_Type) GetM0_TIMING_MAX_SELECT() uint32 {
	return (volatile.LoadUint32(&o.M0_TIMING.Reg) & 0x7e0000) >> 17
}
func (o *QMI_Type) SetM0_TIMING_MIN_DESELECT(value uint32) {
	volatile.StoreUint32(&o.M0_TIMING.Reg, volatile.LoadUint32(&o.M0_TIMING.Reg)&^(0x1f000)|value<<12)
}
func (o *QMI_Type) GetM0_TIMING_MIN_DESELECT() uint32 {
	return (volatile.LoadUint32(&o.M0_TIMING.Reg) & 0x1f000) >> 12
}
func (o *QMI_Type) SetM0_TIMING_RXDELAY(value uint32) {
	volatile.StoreUint32(&o.M0_TIMING.Reg, volatile.LoadUint32(&o.M0_TIMING.Reg)&^(0x700)|value<<8)
}
func (o *QMI_Type) GetM0_TIMING_RXDELAY() uint32 {
	return (volatile.LoadUint32(&o.M0_TIMING.Reg) & 0x700) >> 8
}
func (o *QMI_Type) SetM0_TIMING_CLKDIV(value uint32) {
	volatile.StoreUint32(&o.M0_TIMING.Reg, volatile.LoadUint32(&o.M0_TIMING.Reg)&^(0xff)|value)
}
func (o *QMI_Type) GetM0_TIMING_CLKDIV() uint32 {
	return volatile.LoadUint32(&o.M0_TIMING.Reg) & 0xff
}

// QMI.M0_RFMT: Read transfer format configuration for memory address window 0. Configure the bus width of each transfer phase individually, and configure the length or presence of the command prefix, command suffix and dummy/turnaround transfer phases. Only 24-bit addresses are supported. The reset value of the M0_RFMT register is configured to support a basic 03h serial read transfer with no additional configuration.
func (o *QMI_Type) SetM0_RFMT_DTR(value uint32) {
	volatile.StoreUint32(&o.M0_RFMT.Reg, volatile.LoadUint32(&o.M0_RFMT.Reg)&^(0x10000000)|value<<28)
}
func (o *QMI_Type) GetM0_RFMT_DTR() uint32 {
	return (volatile.LoadUint32(&o.M0_RFMT.Reg) & 0x10000000) >> 28
}
func (o *QMI_Type) SetM0_RFMT_DUMMY_LEN(value uint32) {
	volatile.StoreUint32(&o.M0_RFMT.Reg, volatile.LoadUint32(&o.M0_RFMT.Reg)&^(0x70000)|value<<16)
}
func (o *QMI_Type) GetM0_RFMT_DUMMY_LEN() uint32 {
	return (volatile.LoadUint32(&o.M0_RFMT.Reg) & 0x70000) >> 16
}
func (o *QMI_Type) SetM0_RFMT_SUFFIX_LEN(value uint32) {
	volatile.StoreUint32(&o.M0_RFMT.Reg, volatile.LoadUint32(&o.M0_RFMT.Reg)&^(0xc000)|value<<14)
}
func (o *QMI_Type) GetM0_RFMT_SUFFIX_LEN() uint32 {
	return (volatile.LoadUint32(&o.M0_RFMT.Reg) & 0xc000) >> 14
}
func (o *QMI_Type) SetM0_RFMT_PREFIX_LEN(value uint32) {
	volatile.StoreUint32(&o.M0_RFMT.Reg, volatile.LoadUint32(&o.M0_RFMT.Reg)&^(0x1000)|value<<12)
}
func (o *QMI_Type) GetM0_RFMT_PREFIX_LEN() uint32 {
	return (volatile.LoadUint32(&o.M0_RFMT.Reg) & 0x1000) >> 12
}
func (o *QMI_Type) SetM0_RFMT_DATA_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_RFMT.Reg, volatile.LoadUint32(&o.M0_RFMT.Reg)&^(0x300)|value<<8)
}
func (o *QMI_Type) GetM0_RFMT_DATA_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M0_RFMT.Reg) & 0x300) >> 8
}
func (o *QMI_Type) SetM0_RFMT_DUMMY_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_RFMT.Reg, volatile.LoadUint32(&o.M0_RFMT.Reg)&^(0xc0)|value<<6)
}
func (o *QMI_Type) GetM0_RFMT_DUMMY_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M0_RFMT.Reg) & 0xc0) >> 6
}
func (o *QMI_Type) SetM0_RFMT_SUFFIX_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_RFMT.Reg, volatile.LoadUint32(&o.M0_RFMT.Reg)&^(0x30)|value<<4)
}
func (o *QMI_Type) GetM0_RFMT_SUFFIX_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M0_RFMT.Reg) & 0x30) >> 4
}
func (o *QMI_Type) SetM0_RFMT_ADDR_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_RFMT.Reg, volatile.LoadUint32(&o.M0_RFMT.Reg)&^(0xc)|value<<2)
}
func (o *QMI_Type) GetM0_RFMT_ADDR_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M0_RFMT.Reg) & 0xc) >> 2
}
func (o *QMI_Type) SetM0_RFMT_PREFIX_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_RFMT.Reg, volatile.LoadUint32(&o.M0_RFMT.Reg)&^(0x3)|value)
}
func (o *QMI_Type) GetM0_RFMT_PREFIX_WIDTH() uint32 {
	return volatile.LoadUint32(&o.M0_RFMT.Reg) & 0x3
}

// QMI.M0_RCMD: Command constants used for reads from memory address window 0. The reset value of the M0_RCMD register is configured to support a basic 03h serial read transfer with no additional configuration.
func (o *QMI_Type) SetM0_RCMD_SUFFIX(value uint32) {
	volatile.StoreUint32(&o.M0_RCMD.Reg, volatile.LoadUint32(&o.M0_RCMD.Reg)&^(0xff00)|value<<8)
}
func (o *QMI_Type) GetM0_RCMD_SUFFIX() uint32 {
	return (volatile.LoadUint32(&o.M0_RCMD.Reg) & 0xff00) >> 8
}
func (o *QMI_Type) SetM0_RCMD_PREFIX(value uint32) {
	volatile.StoreUint32(&o.M0_RCMD.Reg, volatile.LoadUint32(&o.M0_RCMD.Reg)&^(0xff)|value)
}
func (o *QMI_Type) GetM0_RCMD_PREFIX() uint32 {
	return volatile.LoadUint32(&o.M0_RCMD.Reg) & 0xff
}

// QMI.M0_WFMT: Write transfer format configuration for memory address window 0. Configure the bus width of each transfer phase individually, and configure the length or presence of the command prefix, command suffix and dummy/turnaround transfer phases. Only 24-bit addresses are supported. The reset value of the M0_WFMT register is configured to support a basic 02h serial write transfer. However, writes to this window must first be enabled via the XIP_CTRL_WRITABLE_M0 bit, as XIP memory is read-only by default.
func (o *QMI_Type) SetM0_WFMT_DTR(value uint32) {
	volatile.StoreUint32(&o.M0_WFMT.Reg, volatile.LoadUint32(&o.M0_WFMT.Reg)&^(0x10000000)|value<<28)
}
func (o *QMI_Type) GetM0_WFMT_DTR() uint32 {
	return (volatile.LoadUint32(&o.M0_WFMT.Reg) & 0x10000000) >> 28
}
func (o *QMI_Type) SetM0_WFMT_DUMMY_LEN(value uint32) {
	volatile.StoreUint32(&o.M0_WFMT.Reg, volatile.LoadUint32(&o.M0_WFMT.Reg)&^(0x70000)|value<<16)
}
func (o *QMI_Type) GetM0_WFMT_DUMMY_LEN() uint32 {
	return (volatile.LoadUint32(&o.M0_WFMT.Reg) & 0x70000) >> 16
}
func (o *QMI_Type) SetM0_WFMT_SUFFIX_LEN(value uint32) {
	volatile.StoreUint32(&o.M0_WFMT.Reg, volatile.LoadUint32(&o.M0_WFMT.Reg)&^(0xc000)|value<<14)
}
func (o *QMI_Type) GetM0_WFMT_SUFFIX_LEN() uint32 {
	return (volatile.LoadUint32(&o.M0_WFMT.Reg) & 0xc000) >> 14
}
func (o *QMI_Type) SetM0_WFMT_PREFIX_LEN(value uint32) {
	volatile.StoreUint32(&o.M0_WFMT.Reg, volatile.LoadUint32(&o.M0_WFMT.Reg)&^(0x1000)|value<<12)
}
func (o *QMI_Type) GetM0_WFMT_PREFIX_LEN() uint32 {
	return (volatile.LoadUint32(&o.M0_WFMT.Reg) & 0x1000) >> 12
}
func (o *QMI_Type) SetM0_WFMT_DATA_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_WFMT.Reg, volatile.LoadUint32(&o.M0_WFMT.Reg)&^(0x300)|value<<8)
}
func (o *QMI_Type) GetM0_WFMT_DATA_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M0_WFMT.Reg) & 0x300) >> 8
}
func (o *QMI_Type) SetM0_WFMT_DUMMY_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_WFMT.Reg, volatile.LoadUint32(&o.M0_WFMT.Reg)&^(0xc0)|value<<6)
}
func (o *QMI_Type) GetM0_WFMT_DUMMY_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M0_WFMT.Reg) & 0xc0) >> 6
}
func (o *QMI_Type) SetM0_WFMT_SUFFIX_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_WFMT.Reg, volatile.LoadUint32(&o.M0_WFMT.Reg)&^(0x30)|value<<4)
}
func (o *QMI_Type) GetM0_WFMT_SUFFIX_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M0_WFMT.Reg) & 0x30) >> 4
}
func (o *QMI_Type) SetM0_WFMT_ADDR_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_WFMT.Reg, volatile.LoadUint32(&o.M0_WFMT.Reg)&^(0xc)|value<<2)
}
func (o *QMI_Type) GetM0_WFMT_ADDR_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M0_WFMT.Reg) & 0xc) >> 2
}
func (o *QMI_Type) SetM0_WFMT_PREFIX_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M0_WFMT.Reg, volatile.LoadUint32(&o.M0_WFMT.Reg)&^(0x3)|value)
}
func (o *QMI_Type) GetM0_WFMT_PREFIX_WIDTH() uint32 {
	return volatile.LoadUint32(&o.M0_WFMT.Reg) & 0x3
}

// QMI.M0_WCMD: Command constants used for writes to memory address window 0. The reset value of the M0_WCMD register is configured to support a basic 02h serial write transfer with no additional configuration.
func (o *QMI_Type) SetM0_WCMD_SUFFIX(value uint32) {
	volatile.StoreUint32(&o.M0_WCMD.Reg, volatile.LoadUint32(&o.M0_WCMD.Reg)&^(0xff00)|value<<8)
}
func (o *QMI_Type) GetM0_WCMD_SUFFIX() uint32 {
	return (volatile.LoadUint32(&o.M0_WCMD.Reg) & 0xff00) >> 8
}
func (o *QMI_Type) SetM0_WCMD_PREFIX(value uint32) {
	volatile.StoreUint32(&o.M0_WCMD.Reg, volatile.LoadUint32(&o.M0_WCMD.Reg)&^(0xff)|value)
}
func (o *QMI_Type) GetM0_WCMD_PREFIX() uint32 {
	return volatile.LoadUint32(&o.M0_WCMD.Reg) & 0xff
}

// QMI.M1_TIMING: Timing configuration register for memory address window 1.
func (o *QMI_Type) SetM1_TIMING_COOLDOWN(value uint32) {
	volatile.StoreUint32(&o.M1_TIMING.Reg, volatile.LoadUint32(&o.M1_TIMING.Reg)&^(0xc0000000)|value<<30)
}
func (o *QMI_Type) GetM1_TIMING_COOLDOWN() uint32 {
	return (volatile.LoadUint32(&o.M1_TIMING.Reg) & 0xc0000000) >> 30
}
func (o *QMI_Type) SetM1_TIMING_PAGEBREAK(value uint32) {
	volatile.StoreUint32(&o.M1_TIMING.Reg, volatile.LoadUint32(&o.M1_TIMING.Reg)&^(0x30000000)|value<<28)
}
func (o *QMI_Type) GetM1_TIMING_PAGEBREAK() uint32 {
	return (volatile.LoadUint32(&o.M1_TIMING.Reg) & 0x30000000) >> 28
}
func (o *QMI_Type) SetM1_TIMING_SELECT_SETUP(value uint32) {
	volatile.StoreUint32(&o.M1_TIMING.Reg, volatile.LoadUint32(&o.M1_TIMING.Reg)&^(0x2000000)|value<<25)
}
func (o *QMI_Type) GetM1_TIMING_SELECT_SETUP() uint32 {
	return (volatile.LoadUint32(&o.M1_TIMING.Reg) & 0x2000000) >> 25
}
func (o *QMI_Type) SetM1_TIMING_SELECT_HOLD(value uint32) {
	volatile.StoreUint32(&o.M1_TIMING.Reg, volatile.LoadUint32(&o.M1_TIMING.Reg)&^(0x1800000)|value<<23)
}
func (o *QMI_Type) GetM1_TIMING_SELECT_HOLD() uint32 {
	return (volatile.LoadUint32(&o.M1_TIMING.Reg) & 0x1800000) >> 23
}
func (o *QMI_Type) SetM1_TIMING_MAX_SELECT(value uint32) {
	volatile.StoreUint32(&o.M1_TIMING.Reg, volatile.LoadUint32(&o.M1_TIMING.Reg)&^(0x7e0000)|value<<17)
}
func (o *QMI_Type) GetM1_TIMING_MAX_SELECT() uint32 {
	return (volatile.LoadUint32(&o.M1_TIMING.Reg) & 0x7e0000) >> 17
}
func (o *QMI_Type) SetM1_TIMING_MIN_DESELECT(value uint32) {
	volatile.StoreUint32(&o.M1_TIMING.Reg, volatile.LoadUint32(&o.M1_TIMING.Reg)&^(0x1f000)|value<<12)
}
func (o *QMI_Type) GetM1_TIMING_MIN_DESELECT() uint32 {
	return (volatile.LoadUint32(&o.M1_TIMING.Reg) & 0x1f000) >> 12
}
func (o *QMI_Type) SetM1_TIMING_RXDELAY(value uint32) {
	volatile.StoreUint32(&o.M1_TIMING.Reg, volatile.LoadUint32(&o.M1_TIMING.Reg)&^(0x700)|value<<8)
}
func (o *QMI_Type) GetM1_TIMING_RXDELAY() uint32 {
	return (volatile.LoadUint32(&o.M1_TIMING.Reg) & 0x700) >> 8
}
func (o *QMI_Type) SetM1_TIMING_CLKDIV(value uint32) {
	volatile.StoreUint32(&o.M1_TIMING.Reg, volatile.LoadUint32(&o.M1_TIMING.Reg)&^(0xff)|value)
}
func (o *QMI_Type) GetM1_TIMING_CLKDIV() uint32 {
	return volatile.LoadUint32(&o.M1_TIMING.Reg) & 0xff
}

// QMI.M1_RFMT: Read transfer format configuration for memory address window 1. Configure the bus width of each transfer phase individually, and configure the length or presence of the command prefix, command suffix and dummy/turnaround transfer phases. Only 24-bit addresses are supported. The reset value of the M1_RFMT register is configured to support a basic 03h serial read transfer with no additional configuration.
func (o *QMI_Type) SetM1_RFMT_DTR(value uint32) {
	volatile.StoreUint32(&o.M1_RFMT.Reg, volatile.LoadUint32(&o.M1_RFMT.Reg)&^(0x10000000)|value<<28)
}
func (o *QMI_Type) GetM1_RFMT_DTR() uint32 {
	return (volatile.LoadUint32(&o.M1_RFMT.Reg) & 0x10000000) >> 28
}
func (o *QMI_Type) SetM1_RFMT_DUMMY_LEN(value uint32) {
	volatile.StoreUint32(&o.M1_RFMT.Reg, volatile.LoadUint32(&o.M1_RFMT.Reg)&^(0x70000)|value<<16)
}
func (o *QMI_Type) GetM1_RFMT_DUMMY_LEN() uint32 {
	return (volatile.LoadUint32(&o.M1_RFMT.Reg) & 0x70000) >> 16
}
func (o *QMI_Type) SetM1_RFMT_SUFFIX_LEN(value uint32) {
	volatile.StoreUint32(&o.M1_RFMT.Reg, volatile.LoadUint32(&o.M1_RFMT.Reg)&^(0xc000)|value<<14)
}
func (o *QMI_Type) GetM1_RFMT_SUFFIX_LEN() uint32 {
	return (volatile.LoadUint32(&o.M1_RFMT.Reg) & 0xc000) >> 14
}
func (o *QMI_Type) SetM1_RFMT_PREFIX_LEN(value uint32) {
	volatile.StoreUint32(&o.M1_RFMT.Reg, volatile.LoadUint32(&o.M1_RFMT.Reg)&^(0x1000)|value<<12)
}
func (o *QMI_Type) GetM1_RFMT_PREFIX_LEN() uint32 {
	return (volatile.LoadUint32(&o.M1_RFMT.Reg) & 0x1000) >> 12
}
func (o *QMI_Type) SetM1_RFMT_DATA_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_RFMT.Reg, volatile.LoadUint32(&o.M1_RFMT.Reg)&^(0x300)|value<<8)
}
func (o *QMI_Type) GetM1_RFMT_DATA_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M1_RFMT.Reg) & 0x300) >> 8
}
func (o *QMI_Type) SetM1_RFMT_DUMMY_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_RFMT.Reg, volatile.LoadUint32(&o.M1_RFMT.Reg)&^(0xc0)|value<<6)
}
func (o *QMI_Type) GetM1_RFMT_DUMMY_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M1_RFMT.Reg) & 0xc0) >> 6
}
func (o *QMI_Type) SetM1_RFMT_SUFFIX_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_RFMT.Reg, volatile.LoadUint32(&o.M1_RFMT.Reg)&^(0x30)|value<<4)
}
func (o *QMI_Type) GetM1_RFMT_SUFFIX_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M1_RFMT.Reg) & 0x30) >> 4
}
func (o *QMI_Type) SetM1_RFMT_ADDR_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_RFMT.Reg, volatile.LoadUint32(&o.M1_RFMT.Reg)&^(0xc)|value<<2)
}
func (o *QMI_Type) GetM1_RFMT_ADDR_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M1_RFMT.Reg) & 0xc) >> 2
}
func (o *QMI_Type) SetM1_RFMT_PREFIX_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_RFMT.Reg, volatile.LoadUint32(&o.M1_RFMT.Reg)&^(0x3)|value)
}
func (o *QMI_Type) GetM1_RFMT_PREFIX_WIDTH() uint32 {
	return volatile.LoadUint32(&o.M1_RFMT.Reg) & 0x3
}

// QMI.M1_RCMD: Command constants used for reads from memory address window 1. The reset value of the M1_RCMD register is configured to support a basic 03h serial read transfer with no additional configuration.
func (o *QMI_Type) SetM1_RCMD_SUFFIX(value uint32) {
	volatile.StoreUint32(&o.M1_RCMD.Reg, volatile.LoadUint32(&o.M1_RCMD.Reg)&^(0xff00)|value<<8)
}
func (o *QMI_Type) GetM1_RCMD_SUFFIX() uint32 {
	return (volatile.LoadUint32(&o.M1_RCMD.Reg) & 0xff00) >> 8
}
func (o *QMI_Type) SetM1_RCMD_PREFIX(value uint32) {
	volatile.StoreUint32(&o.M1_RCMD.Reg, volatile.LoadUint32(&o.M1_RCMD.Reg)&^(0xff)|value)
}
func (o *QMI_Type) GetM1_RCMD_PREFIX() uint32 {
	return volatile.LoadUint32(&o.M1_RCMD.Reg) & 0xff
}

// QMI.M1_WFMT: Write transfer format configuration for memory address window 1. Configure the bus width of each transfer phase individually, and configure the length or presence of the command prefix, command suffix and dummy/turnaround transfer phases. Only 24-bit addresses are supported. The reset value of the M1_WFMT register is configured to support a basic 02h serial write transfer. However, writes to this window must first be enabled via the XIP_CTRL_WRITABLE_M1 bit, as XIP memory is read-only by default.
func (o *QMI_Type) SetM1_WFMT_DTR(value uint32) {
	volatile.StoreUint32(&o.M1_WFMT.Reg, volatile.LoadUint32(&o.M1_WFMT.Reg)&^(0x10000000)|value<<28)
}
func (o *QMI_Type) GetM1_WFMT_DTR() uint32 {
	return (volatile.LoadUint32(&o.M1_WFMT.Reg) & 0x10000000) >> 28
}
func (o *QMI_Type) SetM1_WFMT_DUMMY_LEN(value uint32) {
	volatile.StoreUint32(&o.M1_WFMT.Reg, volatile.LoadUint32(&o.M1_WFMT.Reg)&^(0x70000)|value<<16)
}
func (o *QMI_Type) GetM1_WFMT_DUMMY_LEN() uint32 {
	return (volatile.LoadUint32(&o.M1_WFMT.Reg) & 0x70000) >> 16
}
func (o *QMI_Type) SetM1_WFMT_SUFFIX_LEN(value uint32) {
	volatile.StoreUint32(&o.M1_WFMT.Reg, volatile.LoadUint32(&o.M1_WFMT.Reg)&^(0xc000)|value<<14)
}
func (o *QMI_Type) GetM1_WFMT_SUFFIX_LEN() uint32 {
	return (volatile.LoadUint32(&o.M1_WFMT.Reg) & 0xc000) >> 14
}
func (o *QMI_Type) SetM1_WFMT_PREFIX_LEN(value uint32) {
	volatile.StoreUint32(&o.M1_WFMT.Reg, volatile.LoadUint32(&o.M1_WFMT.Reg)&^(0x1000)|value<<12)
}
func (o *QMI_Type) GetM1_WFMT_PREFIX_LEN() uint32 {
	return (volatile.LoadUint32(&o.M1_WFMT.Reg) & 0x1000) >> 12
}
func (o *QMI_Type) SetM1_WFMT_DATA_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_WFMT.Reg, volatile.LoadUint32(&o.M1_WFMT.Reg)&^(0x300)|value<<8)
}
func (o *QMI_Type) GetM1_WFMT_DATA_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M1_WFMT.Reg) & 0x300) >> 8
}
func (o *QMI_Type) SetM1_WFMT_DUMMY_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_WFMT.Reg, volatile.LoadUint32(&o.M1_WFMT.Reg)&^(0xc0)|value<<6)
}
func (o *QMI_Type) GetM1_WFMT_DUMMY_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M1_WFMT.Reg) & 0xc0) >> 6
}
func (o *QMI_Type) SetM1_WFMT_SUFFIX_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_WFMT.Reg, volatile.LoadUint32(&o.M1_WFMT.Reg)&^(0x30)|value<<4)
}
func (o *QMI_Type) GetM1_WFMT_SUFFIX_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M1_WFMT.Reg) & 0x30) >> 4
}
func (o *QMI_Type) SetM1_WFMT_ADDR_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_WFMT.Reg, volatile.LoadUint32(&o.M1_WFMT.Reg)&^(0xc)|value<<2)
}
func (o *QMI_Type) GetM1_WFMT_ADDR_WIDTH() uint32 {
	return (volatile.LoadUint32(&o.M1_WFMT.Reg) & 0xc) >> 2
}
func (o *QMI_Type) SetM1_WFMT_PREFIX_WIDTH(value uint32) {
	volatile.StoreUint32(&o.M1_WFMT.Reg, volatile.LoadUint32(&o.M1_WFMT.Reg)&^(0x3)|value)
}
func (o *QMI_Type) GetM1_WFMT_PREFIX_WIDTH() uint32 {
	return volatile.LoadUint32(&o.M1_WFMT.Reg) & 0x3
}

// QMI.M1_WCMD: Command constants used for writes to memory address window 1. The reset value of the M1_WCMD register is configured to support a basic 02h serial write transfer with no additional configuration.
func (o *QMI_Type) SetM1_WCMD_SUFFIX(value uint32) {
	volatile.StoreUint32(&o.M1_WCMD.Reg, volatile.LoadUint32(&o.M1_WCMD.Reg)&^(0xff00)|value<<8)
}
func (o *QMI_Type) GetM1_WCMD_SUFFIX() uint32 {
	return (volatile.LoadUint32(&o.M1_WCMD.Reg) & 0xff00) >> 8
}
func (o *QMI_Type) SetM1_WCMD_PREFIX(value uint32) {
	volatile.StoreUint32(&o.M1_WCMD.Reg, volatile.LoadUint32(&o.M1_WCMD.Reg)&^(0xff)|value)
}
func (o *QMI_Type) GetM1_WCMD_PREFIX() uint32 {
	return volatile.LoadUint32(&o.M1_WCMD.Reg) & 0xff
}

// QMI.ATRANS0: Configure address translation for XIP virtual addresses 0x000000 through 0x3fffff (a 4 MiB window starting at +0 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
func (o *QMI_Type) SetATRANS0_SIZE(value uint32) {
	volatile.StoreUint32(&o.ATRANS0.Reg, volatile.LoadUint32(&o.ATRANS0.Reg)&^(0x7ff0000)|value<<16)
}
func (o *QMI_Type) GetATRANS0_SIZE() uint32 {
	return (volatile.LoadUint32(&o.ATRANS0.Reg) & 0x7ff0000) >> 16
}
func (o *QMI_Type) SetATRANS0_BASE(value uint32) {
	volatile.StoreUint32(&o.ATRANS0.Reg, volatile.LoadUint32(&o.ATRANS0.Reg)&^(0xfff)|value)
}
func (o *QMI_Type) GetATRANS0_BASE() uint32 {
	return volatile.LoadUint32(&o.ATRANS0.Reg) & 0xfff
}

// QMI.ATRANS1: Configure address translation for XIP virtual addresses 0x400000 through 0x7fffff (a 4 MiB window starting at +4 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
func (o *QMI_Type) SetATRANS1_SIZE(value uint32) {
	volatile.StoreUint32(&o.ATRANS1.Reg, volatile.LoadUint32(&o.ATRANS1.Reg)&^(0x7ff0000)|value<<16)
}
func (o *QMI_Type) GetATRANS1_SIZE() uint32 {
	return (volatile.LoadUint32(&o.ATRANS1.Reg) & 0x7ff0000) >> 16
}
func (o *QMI_Type) SetATRANS1_BASE(value uint32) {
	volatile.StoreUint32(&o.ATRANS1.Reg, volatile.LoadUint32(&o.ATRANS1.Reg)&^(0xfff)|value)
}
func (o *QMI_Type) GetATRANS1_BASE() uint32 {
	return volatile.LoadUint32(&o.ATRANS1.Reg) & 0xfff
}

// QMI.ATRANS2: Configure address translation for XIP virtual addresses 0x800000 through 0xbfffff (a 4 MiB window starting at +8 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
func (o *QMI_Type) SetATRANS2_SIZE(value uint32) {
	volatile.StoreUint32(&o.ATRANS2.Reg, volatile.LoadUint32(&o.ATRANS2.Reg)&^(0x7ff0000)|value<<16)
}
func (o *QMI_Type) GetATRANS2_SIZE() uint32 {
	return (volatile.LoadUint32(&o.ATRANS2.Reg) & 0x7ff0000) >> 16
}
func (o *QMI_Type) SetATRANS2_BASE(value uint32) {
	volatile.StoreUint32(&o.ATRANS2.Reg, volatile.LoadUint32(&o.ATRANS2.Reg)&^(0xfff)|value)
}
func (o *QMI_Type) GetATRANS2_BASE() uint32 {
	return volatile.LoadUint32(&o.ATRANS2.Reg) & 0xfff
}

// QMI.ATRANS3: Configure address translation for XIP virtual addresses 0xc00000 through 0xffffff (a 4 MiB window starting at +12 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
func (o *QMI_Type) SetATRANS3_SIZE(value uint32) {
	volatile.StoreUint32(&o.ATRANS3.Reg, volatile.LoadUint32(&o.ATRANS3.Reg)&^(0x7ff0000)|value<<16)
}
func (o *QMI_Type) GetATRANS3_SIZE() uint32 {
	return (volatile.LoadUint32(&o.ATRANS3.Reg) & 0x7ff0000) >> 16
}
func (o *QMI_Type) SetATRANS3_BASE(value uint32) {
	volatile.StoreUint32(&o.ATRANS3.Reg, volatile.LoadUint32(&o.ATRANS3.Reg)&^(0xfff)|value)
}
func (o *QMI_Type) GetATRANS3_BASE() uint32 {
	return volatile.LoadUint32(&o.ATRANS3.Reg) & 0xfff
}

// QMI.ATRANS4: Configure address translation for XIP virtual addresses 0x1000000 through 0x13fffff (a 4 MiB window starting at +16 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
func (o *QMI_Type) SetATRANS4_SIZE(value uint32) {
	volatile.StoreUint32(&o.ATRANS4.Reg, volatile.LoadUint32(&o.ATRANS4.Reg)&^(0x7ff0000)|value<<16)
}
func (o *QMI_Type) GetATRANS4_SIZE() uint32 {
	return (volatile.LoadUint32(&o.ATRANS4.Reg) & 0x7ff0000) >> 16
}
func (o *QMI_Type) SetATRANS4_BASE(value uint32) {
	volatile.StoreUint32(&o.ATRANS4.Reg, volatile.LoadUint32(&o.ATRANS4.Reg)&^(0xfff)|value)
}
func (o *QMI_Type) GetATRANS4_BASE() uint32 {
	return volatile.LoadUint32(&o.ATRANS4.Reg) & 0xfff
}

// QMI.ATRANS5: Configure address translation for XIP virtual addresses 0x1400000 through 0x17fffff (a 4 MiB window starting at +20 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
func (o *QMI_Type) SetATRANS5_SIZE(value uint32) {
	volatile.StoreUint32(&o.ATRANS5.Reg, volatile.LoadUint32(&o.ATRANS5.Reg)&^(0x7ff0000)|value<<16)
}
func (o *QMI_Type) GetATRANS5_SIZE() uint32 {
	return (volatile.LoadUint32(&o.ATRANS5.Reg) & 0x7ff0000) >> 16
}
func (o *QMI_Type) SetATRANS5_BASE(value uint32) {
	volatile.StoreUint32(&o.ATRANS5.Reg, volatile.LoadUint32(&o.ATRANS5.Reg)&^(0xfff)|value)
}
func (o *QMI_Type) GetATRANS5_BASE() uint32 {
	return volatile.LoadUint32(&o.ATRANS5.Reg) & 0xfff
}

// QMI.ATRANS6: Configure address translation for XIP virtual addresses 0x1800000 through 0x1bfffff (a 4 MiB window starting at +24 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
func (o *QMI_Type) SetATRANS6_SIZE(value uint32) {
	volatile.StoreUint32(&o.ATRANS6.Reg, volatile.LoadUint32(&o.ATRANS6.Reg)&^(0x7ff0000)|value<<16)
}
func (o *QMI_Type) GetATRANS6_SIZE() uint32 {
	return (volatile.LoadUint32(&o.ATRANS6.Reg) & 0x7ff0000) >> 16
}
func (o *QMI_Type) SetATRANS6_BASE(value uint32) {
	volatile.StoreUint32(&o.ATRANS6.Reg, volatile.LoadUint32(&o.ATRANS6.Reg)&^(0xfff)|value)
}
func (o *QMI_Type) GetATRANS6_BASE() uint32 {
	return volatile.LoadUint32(&o.ATRANS6.Reg) & 0xfff
}

// QMI.ATRANS7: Configure address translation for XIP virtual addresses 0x1c00000 through 0x1ffffff (a 4 MiB window starting at +28 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
func (o *QMI_Type) SetATRANS7_SIZE(value uint32) {
	volatile.StoreUint32(&o.ATRANS7.Reg, volatile.LoadUint32(&o.ATRANS7.Reg)&^(0x7ff0000)|value<<16)
}
func (o *QMI_Type) GetATRANS7_SIZE() uint32 {
	return (volatile.LoadUint32(&o.ATRANS7.Reg) & 0x7ff0000) >> 16
}
func (o *QMI_Type) SetATRANS7_BASE(value uint32) {
	volatile.StoreUint32(&o.ATRANS7.Reg, volatile.LoadUint32(&o.ATRANS7.Reg)&^(0xfff)|value)
}
func (o *QMI_Type) GetATRANS7_BASE() uint32 {
	return volatile.LoadUint32(&o.ATRANS7.Reg) & 0xfff
}

// QSPI flash execute-in-place block
type XIP_CTRL_Type struct {
	CTRL        volatile.Register32 // 0x0
	_           [4]byte
	STAT        volatile.Register32 // 0x8
	CTR_HIT     volatile.Register32 // 0xC
	CTR_ACC     volatile.Register32 // 0x10
	STREAM_ADDR volatile.Register32 // 0x14
	STREAM_CTR  volatile.Register32 // 0x18
	STREAM_FIFO volatile.Register32 // 0x1C
}

// XIP_CTRL.CTRL: Cache control register. Read-only from a Non-secure context.
func (o *XIP_CTRL_Type) SetCTRL_WRITABLE_M1(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x800)|value<<11)
}
func (o *XIP_CTRL_Type) GetCTRL_WRITABLE_M1() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x800) >> 11
}
func (o *XIP_CTRL_Type) SetCTRL_WRITABLE_M0(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x400)|value<<10)
}
func (o *XIP_CTRL_Type) GetCTRL_WRITABLE_M0() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x400) >> 10
}
func (o *XIP_CTRL_Type) SetCTRL_SPLIT_WAYS(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x200)|value<<9)
}
func (o *XIP_CTRL_Type) GetCTRL_SPLIT_WAYS() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x200) >> 9
}
func (o *XIP_CTRL_Type) SetCTRL_MAINT_NONSEC(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x100)|value<<8)
}
func (o *XIP_CTRL_Type) GetCTRL_MAINT_NONSEC() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x100) >> 8
}
func (o *XIP_CTRL_Type) SetCTRL_NO_UNTRANSLATED_NONSEC(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x80)|value<<7)
}
func (o *XIP_CTRL_Type) GetCTRL_NO_UNTRANSLATED_NONSEC() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x80) >> 7
}
func (o *XIP_CTRL_Type) SetCTRL_NO_UNTRANSLATED_SEC(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x40)|value<<6)
}
func (o *XIP_CTRL_Type) GetCTRL_NO_UNTRANSLATED_SEC() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x40) >> 6
}
func (o *XIP_CTRL_Type) SetCTRL_NO_UNCACHED_NONSEC(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x20)|value<<5)
}
func (o *XIP_CTRL_Type) GetCTRL_NO_UNCACHED_NONSEC() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x20) >> 5
}
func (o *XIP_CTRL_Type) SetCTRL_NO_UNCACHED_SEC(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x10)|value<<4)
}
func (o *XIP_CTRL_Type) GetCTRL_NO_UNCACHED_SEC() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x10) >> 4
}
func (o *XIP_CTRL_Type) SetCTRL_POWER_DOWN(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x8)|value<<3)
}
func (o *XIP_CTRL_Type) GetCTRL_POWER_DOWN() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x8) >> 3
}
func (o *XIP_CTRL_Type) SetCTRL_EN_NONSECURE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x2)|value<<1)
}
func (o *XIP_CTRL_Type) GetCTRL_EN_NONSECURE() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x2) >> 1
}
func (o *XIP_CTRL_Type) SetCTRL_EN_SECURE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x1)|value)
}
func (o *XIP_CTRL_Type) GetCTRL_EN_SECURE() uint32 {
	return volatile.LoadUint32(&o.CTRL.Reg) & 0x1
}

// XIP_CTRL.STAT
func (o *XIP_CTRL_Type) SetSTAT_FIFO_FULL(value uint32) {
	volatile.StoreUint32(&o.STAT.Reg, volatile.LoadUint32(&o.STAT.Reg)&^(0x4)|value<<2)
}
func (o *XIP_CTRL_Type) GetSTAT_FIFO_FULL() uint32 {
	return (volatile.LoadUint32(&o.STAT.Reg) & 0x4) >> 2
}
func (o *XIP_CTRL_Type) SetSTAT_FIFO_EMPTY(value uint32) {
	volatile.StoreUint32(&o.STAT.Reg, volatile.LoadUint32(&o.STAT.Reg)&^(0x2)|value<<1)
}
func (o *XIP_CTRL_Type) GetSTAT_FIFO_EMPTY() uint32 {
	return (volatile.LoadUint32(&o.STAT.Reg) & 0x2) >> 1
}

// XIP_CTRL.CTR_HIT: Cache Hit counter
func (o *XIP_CTRL_Type) SetCTR_HIT(value uint32) {
	volatile.StoreUint32(&o.CTR_HIT.Reg, value)
}
func (o *XIP_CTRL_Type) GetCTR_HIT() uint32 {
	return volatile.LoadUint32(&o.CTR_HIT.Reg)
}

// XIP_CTRL.CTR_ACC: Cache Access counter
func (o *XIP_CTRL_Type) SetCTR_ACC(value uint32) {
	volatile.StoreUint32(&o.CTR_ACC.Reg, value)
}
func (o *XIP_CTRL_Type) GetCTR_ACC() uint32 {
	return volatile.LoadUint32(&o.CTR_ACC.Reg)
}

// XIP_CTRL.STREAM_ADDR: FIFO stream address
func (o *XIP_CTRL_Type) SetSTREAM_ADDR(value uint32) {
	volatile.StoreUint32(&o.STREAM_ADDR.Reg, volatile.LoadUint32(&o.STREAM_ADDR.Reg)&^(0xfffffffc)|value<<2)
}
func (o *XIP_CTRL_Type) GetSTREAM_ADDR() uint32 {
	return (volatile.LoadUint32(&o.STREAM_ADDR.Reg) & 0xfffffffc) >> 2
}

// XIP_CTRL.STREAM_CTR: FIFO stream control
func (o *XIP_CTRL_Type) SetSTREAM_CTR(value uint32) {
	volatile.StoreUint32(&o.STREAM_CTR.Reg, volatile.LoadUint32(&o.STREAM_CTR.Reg)&^(0x3fffff)|value)
}
func (o *XIP_CTRL_Type) GetSTREAM_CTR() uint32 {
	return volatile.LoadUint32(&o.STREAM_CTR.Reg) & 0x3fffff
}

// XIP_CTRL.STREAM_FIFO: FIFO stream data
func (o *XIP_CTRL_Type) SetSTREAM_FIFO(value uint32) {
	volatile.StoreUint32(&o.STREAM_FIFO.Reg, value)
}
func (o *XIP_CTRL_Type) GetSTREAM_FIFO() uint32 {
	return volatile.LoadUint32(&o.STREAM_FIFO.Reg)
}

// Auxiliary DMA access to XIP FIFOs, via fast AHB bus access
type XIP_AUX_Type struct {
	STREAM        volatile.Register32 // 0x0
	QMI_DIRECT_TX volatile.Register32 // 0x4
	QMI_DIRECT_RX volatile.Register32 // 0x8
}

// XIP_AUX.STREAM: Read the XIP stream FIFO (fast bus access to XIP_CTRL_STREAM_FIFO)
func (o *XIP_AUX_Type) SetSTREAM(value uint32) {
	volatile.StoreUint32(&o.STREAM.Reg, value)
}
func (o *XIP_AUX_Type) GetSTREAM() uint32 {
	return volatile.LoadUint32(&o.STREAM.Reg)
}

// XIP_AUX.QMI_DIRECT_TX: Write to the QMI direct-mode TX FIFO (fast bus access to QMI_DIRECT_TX)
func (o *XIP_AUX_Type) SetQMI_DIRECT_TX_NOPUSH(value uint32) {
	volatile.StoreUint32(&o.QMI_DIRECT_TX.Reg, volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg)&^(0x100000)|value<<20)
}
func (o *XIP_AUX_Type) GetQMI_DIRECT_TX_NOPUSH() uint32 {
	return (volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg) & 0x100000) >> 20
}
func (o *XIP_AUX_Type) SetQMI_DIRECT_TX_OE(value uint32) {
	volatile.StoreUint32(&o.QMI_DIRECT_TX.Reg, volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg)&^(0x80000)|value<<19)
}
func (o *XIP_AUX_Type) GetQMI_DIRECT_TX_OE() uint32 {
	return (volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg) & 0x80000) >> 19
}
func (o *XIP_AUX_Type) SetQMI_DIRECT_TX_DWIDTH(value uint32) {
	volatile.StoreUint32(&o.QMI_DIRECT_TX.Reg, volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg)&^(0x40000)|value<<18)
}
func (o *XIP_AUX_Type) GetQMI_DIRECT_TX_DWIDTH() uint32 {
	return (volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg) & 0x40000) >> 18
}
func (o *XIP_AUX_Type) SetQMI_DIRECT_TX_IWIDTH(value uint32) {
	volatile.StoreUint32(&o.QMI_DIRECT_TX.Reg, volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg)&^(0x30000)|value<<16)
}
func (o *XIP_AUX_Type) GetQMI_DIRECT_TX_IWIDTH() uint32 {
	return (volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg) & 0x30000) >> 16
}
func (o *XIP_AUX_Type) SetQMI_DIRECT_TX_DATA(value uint32) {
	volatile.StoreUint32(&o.QMI_DIRECT_TX.Reg, volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg)&^(0xffff)|value)
}
func (o *XIP_AUX_Type) GetQMI_DIRECT_TX_DATA() uint32 {
	return volatile.LoadUint32(&o.QMI_DIRECT_TX.Reg) & 0xffff
}

// XIP_AUX.QMI_DIRECT_RX: Read from the QMI direct-mode RX FIFO (fast bus access to QMI_DIRECT_RX)
func (o *XIP_AUX_Type) SetQMI_DIRECT_RX(value uint32) {
	volatile.StoreUint32(&o.QMI_DIRECT_RX.Reg, volatile.LoadUint32(&o.QMI_DIRECT_RX.Reg)&^(0xffff)|value)
}
func (o *XIP_AUX_Type) GetQMI_DIRECT_RX() uint32 {
	return volatile.LoadUint32(&o.QMI_DIRECT_RX.Reg) & 0xffff
}

// Register block for various chip control signals
type SYSCFG_Type struct {
	PROC_CONFIG            volatile.Register32 // 0x0
	PROC_IN_SYNC_BYPASS    volatile.Register32 // 0x4
	PROC_IN_SYNC_BYPASS_HI volatile.Register32 // 0x8
	DBGFORCE               volatile.Register32 // 0xC
	MEMPOWERDOWN           volatile.Register32 // 0x10
	AUXCTRL                volatile.Register32 // 0x14
}

// SYSCFG.PROC_CONFIG: Configuration for processors
func (o *SYSCFG_Type) SetPROC_CONFIG_PROC1_HALTED(value uint32) {
	volatile.StoreUint32(&o.PROC_CONFIG.Reg, volatile.LoadUint32(&o.PROC_CONFIG.Reg)&^(0x2)|value<<1)
}
func (o *SYSCFG_Type) GetPROC_CONFIG_PROC1_HALTED() uint32 {
	return (volatile.LoadUint32(&o.PROC_CONFIG.Reg) & 0x2) >> 1
}
func (o *SYSCFG_Type) SetPROC_CONFIG_PROC0_HALTED(value uint32) {
	volatile.StoreUint32(&o.PROC_CONFIG.Reg, volatile.LoadUint32(&o.PROC_CONFIG.Reg)&^(0x1)|value)
}
func (o *SYSCFG_Type) GetPROC_CONFIG_PROC0_HALTED() uint32 {
	return volatile.LoadUint32(&o.PROC_CONFIG.Reg) & 0x1
}

// SYSCFG.PROC_IN_SYNC_BYPASS: For each bit, if 1, bypass the input synchronizer between that GPIO and the GPIO input register in the SIO. The input synchronizers should generally be unbypassed, to avoid injecting metastabilities into processors. If you're feeling brave, you can bypass to save two cycles of input latency. This register applies to GPIO 0...31.
func (o *SYSCFG_Type) SetPROC_IN_SYNC_BYPASS(value uint32) {
	volatile.StoreUint32(&o.PROC_IN_SYNC_BYPASS.Reg, value)
}
func (o *SYSCFG_Type) GetPROC_IN_SYNC_BYPASS() uint32 {
	return volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS.Reg)
}

// SYSCFG.PROC_IN_SYNC_BYPASS_HI: For each bit, if 1, bypass the input synchronizer between that GPIO and the GPIO input register in the SIO. The input synchronizers should generally be unbypassed, to avoid injecting metastabilities into processors. If you're feeling brave, you can bypass to save two cycles of input latency. This register applies to GPIO 32...47. USB GPIO 56..57 QSPI GPIO 58..63
func (o *SYSCFG_Type) SetPROC_IN_SYNC_BYPASS_HI_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg, volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg)&^(0xf0000000)|value<<28)
}
func (o *SYSCFG_Type) GetPROC_IN_SYNC_BYPASS_HI_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg) & 0xf0000000) >> 28
}
func (o *SYSCFG_Type) SetPROC_IN_SYNC_BYPASS_HI_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg, volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg)&^(0x8000000)|value<<27)
}
func (o *SYSCFG_Type) GetPROC_IN_SYNC_BYPASS_HI_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg) & 0x8000000) >> 27
}
func (o *SYSCFG_Type) SetPROC_IN_SYNC_BYPASS_HI_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg, volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg)&^(0x4000000)|value<<26)
}
func (o *SYSCFG_Type) GetPROC_IN_SYNC_BYPASS_HI_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg) & 0x4000000) >> 26
}
func (o *SYSCFG_Type) SetPROC_IN_SYNC_BYPASS_HI_USB_DM(value uint32) {
	volatile.StoreUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg, volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg)&^(0x2000000)|value<<25)
}
func (o *SYSCFG_Type) GetPROC_IN_SYNC_BYPASS_HI_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg) & 0x2000000) >> 25
}
func (o *SYSCFG_Type) SetPROC_IN_SYNC_BYPASS_HI_USB_DP(value uint32) {
	volatile.StoreUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg, volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg)&^(0x1000000)|value<<24)
}
func (o *SYSCFG_Type) GetPROC_IN_SYNC_BYPASS_HI_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg) & 0x1000000) >> 24
}
func (o *SYSCFG_Type) SetPROC_IN_SYNC_BYPASS_HI_GPIO(value uint32) {
	volatile.StoreUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg, volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg)&^(0xffff)|value)
}
func (o *SYSCFG_Type) GetPROC_IN_SYNC_BYPASS_HI_GPIO() uint32 {
	return volatile.LoadUint32(&o.PROC_IN_SYNC_BYPASS_HI.Reg) & 0xffff
}

// SYSCFG.DBGFORCE: Directly control the chip SWD debug port
func (o *SYSCFG_Type) SetDBGFORCE_ATTACH(value uint32) {
	volatile.StoreUint32(&o.DBGFORCE.Reg, volatile.LoadUint32(&o.DBGFORCE.Reg)&^(0x8)|value<<3)
}
func (o *SYSCFG_Type) GetDBGFORCE_ATTACH() uint32 {
	return (volatile.LoadUint32(&o.DBGFORCE.Reg) & 0x8) >> 3
}
func (o *SYSCFG_Type) SetDBGFORCE_SWCLK(value uint32) {
	volatile.StoreUint32(&o.DBGFORCE.Reg, volatile.LoadUint32(&o.DBGFORCE.Reg)&^(0x4)|value<<2)
}
func (o *SYSCFG_Type) GetDBGFORCE_SWCLK() uint32 {
	return (volatile.LoadUint32(&o.DBGFORCE.Reg) & 0x4) >> 2
}
func (o *SYSCFG_Type) SetDBGFORCE_SWDI(value uint32) {
	volatile.StoreUint32(&o.DBGFORCE.Reg, volatile.LoadUint32(&o.DBGFORCE.Reg)&^(0x2)|value<<1)
}
func (o *SYSCFG_Type) GetDBGFORCE_SWDI() uint32 {
	return (volatile.LoadUint32(&o.DBGFORCE.Reg) & 0x2) >> 1
}
func (o *SYSCFG_Type) SetDBGFORCE_SWDO(value uint32) {
	volatile.StoreUint32(&o.DBGFORCE.Reg, volatile.LoadUint32(&o.DBGFORCE.Reg)&^(0x1)|value)
}
func (o *SYSCFG_Type) GetDBGFORCE_SWDO() uint32 {
	return volatile.LoadUint32(&o.DBGFORCE.Reg) & 0x1
}

// SYSCFG.MEMPOWERDOWN: Control PD pins to memories. Set high to put memories to a low power state. In this state the memories will retain contents but not be accessible Use with caution
func (o *SYSCFG_Type) SetMEMPOWERDOWN_BOOTRAM(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x1000)|value<<12)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_BOOTRAM() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x1000) >> 12
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_ROM(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x800)|value<<11)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_ROM() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x800) >> 11
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_USB(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x400)|value<<10)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_USB() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x400) >> 10
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM9(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x200)|value<<9)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM9() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x200) >> 9
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM8(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x100)|value<<8)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM8() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x100) >> 8
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM7(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x80)|value<<7)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM7() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x80) >> 7
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM6(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x40)|value<<6)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM6() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x40) >> 6
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM5(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x20)|value<<5)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM5() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x20) >> 5
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM4(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x10)|value<<4)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM4() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x10) >> 4
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM3(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x8)|value<<3)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM3() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x8) >> 3
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM2(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x4)|value<<2)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM2() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x4) >> 2
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM1(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x2)|value<<1)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM1() uint32 {
	return (volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x2) >> 1
}
func (o *SYSCFG_Type) SetMEMPOWERDOWN_SRAM0(value uint32) {
	volatile.StoreUint32(&o.MEMPOWERDOWN.Reg, volatile.LoadUint32(&o.MEMPOWERDOWN.Reg)&^(0x1)|value)
}
func (o *SYSCFG_Type) GetMEMPOWERDOWN_SRAM0() uint32 {
	return volatile.LoadUint32(&o.MEMPOWERDOWN.Reg) & 0x1
}

// SYSCFG.AUXCTRL: Auxiliary system control register
func (o *SYSCFG_Type) SetAUXCTRL(value uint32) {
	volatile.StoreUint32(&o.AUXCTRL.Reg, volatile.LoadUint32(&o.AUXCTRL.Reg)&^(0xff)|value)
}
func (o *SYSCFG_Type) GetAUXCTRL() uint32 {
	return volatile.LoadUint32(&o.AUXCTRL.Reg) & 0xff
}

// Controls the crystal oscillator
type XOSC_Type struct {
	CTRL    volatile.Register32 // 0x0
	STATUS  volatile.Register32 // 0x4
	DORMANT volatile.Register32 // 0x8
	STARTUP volatile.Register32 // 0xC
	COUNT   volatile.Register32 // 0x10
}

// XOSC.CTRL: Crystal Oscillator Control
func (o *XOSC_Type) SetCTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xfff000)|value<<12)
}
func (o *XOSC_Type) GetCTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0xfff000) >> 12
}
func (o *XOSC_Type) SetCTRL_FREQ_RANGE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xfff)|value)
}
func (o *XOSC_Type) GetCTRL_FREQ_RANGE() uint32 {
	return volatile.LoadUint32(&o.CTRL.Reg) & 0xfff
}

// XOSC.STATUS: Crystal Oscillator Status
func (o *XOSC_Type) SetSTATUS_STABLE(value uint32) {
	volatile.StoreUint32(&o.STATUS.Reg, volatile.LoadUint32(&o.STATUS.Reg)&^(0x80000000)|value<<31)
}
func (o *XOSC_Type) GetSTATUS_STABLE() uint32 {
	return (volatile.LoadUint32(&o.STATUS.Reg) & 0x80000000) >> 31
}
func (o *XOSC_Type) SetSTATUS_BADWRITE(value uint32) {
	volatile.StoreUint32(&o.STATUS.Reg, volatile.LoadUint32(&o.STATUS.Reg)&^(0x1000000)|value<<24)
}
func (o *XOSC_Type) GetSTATUS_BADWRITE() uint32 {
	return (volatile.LoadUint32(&o.STATUS.Reg) & 0x1000000) >> 24
}
func (o *XOSC_Type) SetSTATUS_ENABLED(value uint32) {
	volatile.StoreUint32(&o.STATUS.Reg, volatile.LoadUint32(&o.STATUS.Reg)&^(0x1000)|value<<12)
}
func (o *XOSC_Type) GetSTATUS_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.STATUS.Reg) & 0x1000) >> 12
}
func (o *XOSC_Type) SetSTATUS_FREQ_RANGE(value uint32) {
	volatile.StoreUint32(&o.STATUS.Reg, volatile.LoadUint32(&o.STATUS.Reg)&^(0x3)|value)
}
func (o *XOSC_Type) GetSTATUS_FREQ_RANGE() uint32 {
	return volatile.LoadUint32(&o.STATUS.Reg) & 0x3
}

// XOSC.DORMANT: Crystal Oscillator pause control
func (o *XOSC_Type) SetDORMANT(value uint32) {
	volatile.StoreUint32(&o.DORMANT.Reg, value)
}
func (o *XOSC_Type) GetDORMANT() uint32 {
	return volatile.LoadUint32(&o.DORMANT.Reg)
}

// XOSC.STARTUP: Controls the startup delay
func (o *XOSC_Type) SetSTARTUP_X4(value uint32) {
	volatile.StoreUint32(&o.STARTUP.Reg, volatile.LoadUint32(&o.STARTUP.Reg)&^(0x100000)|value<<20)
}
func (o *XOSC_Type) GetSTARTUP_X4() uint32 {
	return (volatile.LoadUint32(&o.STARTUP.Reg) & 0x100000) >> 20
}
func (o *XOSC_Type) SetSTARTUP_DELAY(value uint32) {
	volatile.StoreUint32(&o.STARTUP.Reg, volatile.LoadUint32(&o.STARTUP.Reg)&^(0x3fff)|value)
}
func (o *XOSC_Type) GetSTARTUP_DELAY() uint32 {
	return volatile.LoadUint32(&o.STARTUP.Reg) & 0x3fff
}

// XOSC.COUNT: A down counter running at the xosc frequency which counts to zero and stops. Can be used for short software pauses when setting up time sensitive hardware. To start the counter, write a non-zero value. Reads will return 1 while the count is running and 0 when it has finished. Minimum count value is 4. Count values <4 will be treated as count value =4. Note that synchronisation to the register clock domain costs 2 register clock cycles and the counter cannot compensate for that.
func (o *XOSC_Type) SetCOUNT(value uint32) {
	volatile.StoreUint32(&o.COUNT.Reg, volatile.LoadUint32(&o.COUNT.Reg)&^(0xffff)|value)
}
func (o *XOSC_Type) GetCOUNT() uint32 {
	return volatile.LoadUint32(&o.COUNT.Reg) & 0xffff
}

type PLL_SYS_Type struct {
	CS        volatile.Register32 // 0x0
	PWR       volatile.Register32 // 0x4
	FBDIV_INT volatile.Register32 // 0x8
	PRIM      volatile.Register32 // 0xC
	INTR      volatile.Register32 // 0x10
	INTE      volatile.Register32 // 0x14
	INTF      volatile.Register32 // 0x18
	INTS      volatile.Register32 // 0x1C
}

// PLL_SYS.CS: Control and Status GENERAL CONSTRAINTS: Reference clock frequency min=5MHz, max=800MHz Feedback divider min=16, max=320 VCO frequency min=750MHz, max=1600MHz
func (o *PLL_SYS_Type) SetCS_LOCK(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x80000000)|value<<31)
}
func (o *PLL_SYS_Type) GetCS_LOCK() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x80000000) >> 31
}
func (o *PLL_SYS_Type) SetCS_LOCK_N(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x40000000)|value<<30)
}
func (o *PLL_SYS_Type) GetCS_LOCK_N() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x40000000) >> 30
}
func (o *PLL_SYS_Type) SetCS_BYPASS(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x100)|value<<8)
}
func (o *PLL_SYS_Type) GetCS_BYPASS() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x100) >> 8
}
func (o *PLL_SYS_Type) SetCS_REFDIV(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x3f)|value)
}
func (o *PLL_SYS_Type) GetCS_REFDIV() uint32 {
	return volatile.LoadUint32(&o.CS.Reg) & 0x3f
}

// PLL_SYS.PWR: Controls the PLL power modes.
func (o *PLL_SYS_Type) SetPWR_VCOPD(value uint32) {
	volatile.StoreUint32(&o.PWR.Reg, volatile.LoadUint32(&o.PWR.Reg)&^(0x20)|value<<5)
}
func (o *PLL_SYS_Type) GetPWR_VCOPD() uint32 {
	return (volatile.LoadUint32(&o.PWR.Reg) & 0x20) >> 5
}
func (o *PLL_SYS_Type) SetPWR_POSTDIVPD(value uint32) {
	volatile.StoreUint32(&o.PWR.Reg, volatile.LoadUint32(&o.PWR.Reg)&^(0x8)|value<<3)
}
func (o *PLL_SYS_Type) GetPWR_POSTDIVPD() uint32 {
	return (volatile.LoadUint32(&o.PWR.Reg) & 0x8) >> 3
}
func (o *PLL_SYS_Type) SetPWR_DSMPD(value uint32) {
	volatile.StoreUint32(&o.PWR.Reg, volatile.LoadUint32(&o.PWR.Reg)&^(0x4)|value<<2)
}
func (o *PLL_SYS_Type) GetPWR_DSMPD() uint32 {
	return (volatile.LoadUint32(&o.PWR.Reg) & 0x4) >> 2
}
func (o *PLL_SYS_Type) SetPWR_PD(value uint32) {
	volatile.StoreUint32(&o.PWR.Reg, volatile.LoadUint32(&o.PWR.Reg)&^(0x1)|value)
}
func (o *PLL_SYS_Type) GetPWR_PD() uint32 {
	return volatile.LoadUint32(&o.PWR.Reg) & 0x1
}

// PLL_SYS.FBDIV_INT: Feedback divisor (note: this PLL does not support fractional division)
func (o *PLL_SYS_Type) SetFBDIV_INT(value uint32) {
	volatile.StoreUint32(&o.FBDIV_INT.Reg, volatile.LoadUint32(&o.FBDIV_INT.Reg)&^(0xfff)|value)
}
func (o *PLL_SYS_Type) GetFBDIV_INT() uint32 {
	return volatile.LoadUint32(&o.FBDIV_INT.Reg) & 0xfff
}

// PLL_SYS.PRIM: Controls the PLL post dividers for the primary output (note: this PLL does not have a secondary output) the primary output is driven from VCO divided by postdiv1*postdiv2
func (o *PLL_SYS_Type) SetPRIM_POSTDIV1(value uint32) {
	volatile.StoreUint32(&o.PRIM.Reg, volatile.LoadUint32(&o.PRIM.Reg)&^(0x70000)|value<<16)
}
func (o *PLL_SYS_Type) GetPRIM_POSTDIV1() uint32 {
	return (volatile.LoadUint32(&o.PRIM.Reg) & 0x70000) >> 16
}
func (o *PLL_SYS_Type) SetPRIM_POSTDIV2(value uint32) {
	volatile.StoreUint32(&o.PRIM.Reg, volatile.LoadUint32(&o.PRIM.Reg)&^(0x7000)|value<<12)
}
func (o *PLL_SYS_Type) GetPRIM_POSTDIV2() uint32 {
	return (volatile.LoadUint32(&o.PRIM.Reg) & 0x7000) >> 12
}

// PLL_SYS.INTR: Raw Interrupts
func (o *PLL_SYS_Type) SetINTR_LOCK_N_STICKY(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *PLL_SYS_Type) GetINTR_LOCK_N_STICKY() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// PLL_SYS.INTE: Interrupt Enable
func (o *PLL_SYS_Type) SetINTE_LOCK_N_STICKY(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x1)|value)
}
func (o *PLL_SYS_Type) GetINTE_LOCK_N_STICKY() uint32 {
	return volatile.LoadUint32(&o.INTE.Reg) & 0x1
}

// PLL_SYS.INTF: Interrupt Force
func (o *PLL_SYS_Type) SetINTF_LOCK_N_STICKY(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x1)|value)
}
func (o *PLL_SYS_Type) GetINTF_LOCK_N_STICKY() uint32 {
	return volatile.LoadUint32(&o.INTF.Reg) & 0x1
}

// PLL_SYS.INTS: Interrupt status after masking & forcing
func (o *PLL_SYS_Type) SetINTS_LOCK_N_STICKY(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x1)|value)
}
func (o *PLL_SYS_Type) GetINTS_LOCK_N_STICKY() uint32 {
	return volatile.LoadUint32(&o.INTS.Reg) & 0x1
}

// Hardware access control registers
type ACCESSCTRL_Type struct {
	LOCK             volatile.Register32 // 0x0
	FORCE_CORE_NS    volatile.Register32 // 0x4
	CFGRESET         volatile.Register32 // 0x8
	GPIO_NSMASK0     volatile.Register32 // 0xC
	GPIO_NSMASK1     volatile.Register32 // 0x10
	ROM              volatile.Register32 // 0x14
	XIP_MAIN         volatile.Register32 // 0x18
	SRAM0            volatile.Register32 // 0x1C
	SRAM1            volatile.Register32 // 0x20
	SRAM2            volatile.Register32 // 0x24
	SRAM3            volatile.Register32 // 0x28
	SRAM4            volatile.Register32 // 0x2C
	SRAM5            volatile.Register32 // 0x30
	SRAM6            volatile.Register32 // 0x34
	SRAM7            volatile.Register32 // 0x38
	SRAM8            volatile.Register32 // 0x3C
	SRAM9            volatile.Register32 // 0x40
	DMA              volatile.Register32 // 0x44
	USBCTRL          volatile.Register32 // 0x48
	PIO0             volatile.Register32 // 0x4C
	PIO1             volatile.Register32 // 0x50
	PIO2             volatile.Register32 // 0x54
	CORESIGHT_TRACE  volatile.Register32 // 0x58
	CORESIGHT_PERIPH volatile.Register32 // 0x5C
	SYSINFO          volatile.Register32 // 0x60
	RESETS           volatile.Register32 // 0x64
	IO_BANK0         volatile.Register32 // 0x68
	IO_BANK1         volatile.Register32 // 0x6C
	PADS_BANK0       volatile.Register32 // 0x70
	PADS_QSPI        volatile.Register32 // 0x74
	BUSCTRL          volatile.Register32 // 0x78
	ADC0             volatile.Register32 // 0x7C
	HSTX             volatile.Register32 // 0x80
	I2C0             volatile.Register32 // 0x84
	I2C1             volatile.Register32 // 0x88
	PWM              volatile.Register32 // 0x8C
	SPI0             volatile.Register32 // 0x90
	SPI1             volatile.Register32 // 0x94
	TIMER0           volatile.Register32 // 0x98
	TIMER1           volatile.Register32 // 0x9C
	UART0            volatile.Register32 // 0xA0
	UART1            volatile.Register32 // 0xA4
	OTP              volatile.Register32 // 0xA8
	TBMAN            volatile.Register32 // 0xAC
	POWMAN           volatile.Register32 // 0xB0
	TRNG             volatile.Register32 // 0xB4
	SHA256           volatile.Register32 // 0xB8
	SYSCFG           volatile.Register32 // 0xBC
	CLOCKS           volatile.Register32 // 0xC0
	XOSC             volatile.Register32 // 0xC4
	ROSC             volatile.Register32 // 0xC8
	PLL_SYS          volatile.Register32 // 0xCC
	PLL_USB          volatile.Register32 // 0xD0
	TICKS            volatile.Register32 // 0xD4
	WATCHDOG         volatile.Register32 // 0xD8
	RSM              volatile.Register32 // 0xDC
	XIP_CTRL         volatile.Register32 // 0xE0
	XIP_QMI          volatile.Register32 // 0xE4
	XIP_AUX          volatile.Register32 // 0xE8
}

// ACCESSCTRL.LOCK: Once a LOCK bit is written to 1, ACCESSCTRL silently ignores writes from that master. LOCK is writable only by a Secure, Privileged processor or debugger. LOCK bits are only writable when their value is zero. Once set, they can never be cleared, except by a full reset of ACCESSCTRL Setting the LOCK bit does not affect whether an access raises a bus error. Unprivileged writes, or writes from the DMA, will continue to raise bus errors. All other accesses will continue not to.
func (o *ACCESSCTRL_Type) SetLOCK_DEBUG(value uint32) {
	volatile.StoreUint32(&o.LOCK.Reg, volatile.LoadUint32(&o.LOCK.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetLOCK_DEBUG() uint32 {
	return (volatile.LoadUint32(&o.LOCK.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetLOCK_DMA(value uint32) {
	volatile.StoreUint32(&o.LOCK.Reg, volatile.LoadUint32(&o.LOCK.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetLOCK_DMA() uint32 {
	return (volatile.LoadUint32(&o.LOCK.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetLOCK_CORE1(value uint32) {
	volatile.StoreUint32(&o.LOCK.Reg, volatile.LoadUint32(&o.LOCK.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetLOCK_CORE1() uint32 {
	return (volatile.LoadUint32(&o.LOCK.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetLOCK_CORE0(value uint32) {
	volatile.StoreUint32(&o.LOCK.Reg, volatile.LoadUint32(&o.LOCK.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetLOCK_CORE0() uint32 {
	return volatile.LoadUint32(&o.LOCK.Reg) & 0x1
}

// ACCESSCTRL.FORCE_CORE_NS: Force core 1's bus accesses to always be Non-secure, no matter the core's internal state. Useful for schemes where one core is designated as the Non-secure core, since some peripherals may filter individual registers internally based on security state but not on master ID.
func (o *ACCESSCTRL_Type) SetFORCE_CORE_NS_CORE1(value uint32) {
	volatile.StoreUint32(&o.FORCE_CORE_NS.Reg, volatile.LoadUint32(&o.FORCE_CORE_NS.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetFORCE_CORE_NS_CORE1() uint32 {
	return (volatile.LoadUint32(&o.FORCE_CORE_NS.Reg) & 0x2) >> 1
}

// ACCESSCTRL.CFGRESET: Write 1 to reset all ACCESSCTRL configuration, except for the LOCK and FORCE_CORE_NS registers. This bit is used in the RP2350 bootrom to quickly restore ACCESSCTRL to a known state during the boot path. Note that, like all registers in ACCESSCTRL, this register is not writable when the writer's corresponding LOCK bit is set, therefore a master which has been locked out of ACCESSCTRL can not use the CFGRESET register to disturb its contents.
func (o *ACCESSCTRL_Type) SetCFGRESET(value uint32) {
	volatile.StoreUint32(&o.CFGRESET.Reg, volatile.LoadUint32(&o.CFGRESET.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetCFGRESET() uint32 {
	return volatile.LoadUint32(&o.CFGRESET.Reg) & 0x1
}

// ACCESSCTRL.GPIO_NSMASK0: Control whether GPIO0...31 are accessible to Non-secure code. Writable only by a Secure, Privileged processor or debugger. 0 -> Secure access only 1 -> Secure + Non-secure access
func (o *ACCESSCTRL_Type) SetGPIO_NSMASK0(value uint32) {
	volatile.StoreUint32(&o.GPIO_NSMASK0.Reg, value)
}
func (o *ACCESSCTRL_Type) GetGPIO_NSMASK0() uint32 {
	return volatile.LoadUint32(&o.GPIO_NSMASK0.Reg)
}

// ACCESSCTRL.GPIO_NSMASK1: Control whether GPIO32..47 are accessible to Non-secure code, and whether QSPI and USB bitbang are accessible through the Non-secure SIO. Writable only by a Secure, Privileged processor or debugger.
func (o *ACCESSCTRL_Type) SetGPIO_NSMASK1_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_NSMASK1.Reg, volatile.LoadUint32(&o.GPIO_NSMASK1.Reg)&^(0xf0000000)|value<<28)
}
func (o *ACCESSCTRL_Type) GetGPIO_NSMASK1_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_NSMASK1.Reg) & 0xf0000000) >> 28
}
func (o *ACCESSCTRL_Type) SetGPIO_NSMASK1_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_NSMASK1.Reg, volatile.LoadUint32(&o.GPIO_NSMASK1.Reg)&^(0x8000000)|value<<27)
}
func (o *ACCESSCTRL_Type) GetGPIO_NSMASK1_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_NSMASK1.Reg) & 0x8000000) >> 27
}
func (o *ACCESSCTRL_Type) SetGPIO_NSMASK1_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_NSMASK1.Reg, volatile.LoadUint32(&o.GPIO_NSMASK1.Reg)&^(0x4000000)|value<<26)
}
func (o *ACCESSCTRL_Type) GetGPIO_NSMASK1_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_NSMASK1.Reg) & 0x4000000) >> 26
}
func (o *ACCESSCTRL_Type) SetGPIO_NSMASK1_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_NSMASK1.Reg, volatile.LoadUint32(&o.GPIO_NSMASK1.Reg)&^(0x2000000)|value<<25)
}
func (o *ACCESSCTRL_Type) GetGPIO_NSMASK1_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_NSMASK1.Reg) & 0x2000000) >> 25
}
func (o *ACCESSCTRL_Type) SetGPIO_NSMASK1_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_NSMASK1.Reg, volatile.LoadUint32(&o.GPIO_NSMASK1.Reg)&^(0x1000000)|value<<24)
}
func (o *ACCESSCTRL_Type) GetGPIO_NSMASK1_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_NSMASK1.Reg) & 0x1000000) >> 24
}
func (o *ACCESSCTRL_Type) SetGPIO_NSMASK1_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_NSMASK1.Reg, volatile.LoadUint32(&o.GPIO_NSMASK1.Reg)&^(0xffff)|value)
}
func (o *ACCESSCTRL_Type) GetGPIO_NSMASK1_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_NSMASK1.Reg) & 0xffff
}

// ACCESSCTRL.ROM: Control whether debugger, DMA, core 0 and core 1 can access ROM, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetROM_DBG(value uint32) {
	volatile.StoreUint32(&o.ROM.Reg, volatile.LoadUint32(&o.ROM.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetROM_DBG() uint32 {
	return (volatile.LoadUint32(&o.ROM.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetROM_DMA(value uint32) {
	volatile.StoreUint32(&o.ROM.Reg, volatile.LoadUint32(&o.ROM.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetROM_DMA() uint32 {
	return (volatile.LoadUint32(&o.ROM.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetROM_CORE1(value uint32) {
	volatile.StoreUint32(&o.ROM.Reg, volatile.LoadUint32(&o.ROM.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetROM_CORE1() uint32 {
	return (volatile.LoadUint32(&o.ROM.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetROM_CORE0(value uint32) {
	volatile.StoreUint32(&o.ROM.Reg, volatile.LoadUint32(&o.ROM.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetROM_CORE0() uint32 {
	return (volatile.LoadUint32(&o.ROM.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetROM_SP(value uint32) {
	volatile.StoreUint32(&o.ROM.Reg, volatile.LoadUint32(&o.ROM.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetROM_SP() uint32 {
	return (volatile.LoadUint32(&o.ROM.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetROM_SU(value uint32) {
	volatile.StoreUint32(&o.ROM.Reg, volatile.LoadUint32(&o.ROM.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetROM_SU() uint32 {
	return (volatile.LoadUint32(&o.ROM.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetROM_NSP(value uint32) {
	volatile.StoreUint32(&o.ROM.Reg, volatile.LoadUint32(&o.ROM.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetROM_NSP() uint32 {
	return (volatile.LoadUint32(&o.ROM.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetROM_NSU(value uint32) {
	volatile.StoreUint32(&o.ROM.Reg, volatile.LoadUint32(&o.ROM.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetROM_NSU() uint32 {
	return volatile.LoadUint32(&o.ROM.Reg) & 0x1
}

// ACCESSCTRL.XIP_MAIN: Control whether debugger, DMA, core 0 and core 1 can access XIP_MAIN, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetXIP_MAIN_DBG(value uint32) {
	volatile.StoreUint32(&o.XIP_MAIN.Reg, volatile.LoadUint32(&o.XIP_MAIN.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetXIP_MAIN_DBG() uint32 {
	return (volatile.LoadUint32(&o.XIP_MAIN.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetXIP_MAIN_DMA(value uint32) {
	volatile.StoreUint32(&o.XIP_MAIN.Reg, volatile.LoadUint32(&o.XIP_MAIN.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetXIP_MAIN_DMA() uint32 {
	return (volatile.LoadUint32(&o.XIP_MAIN.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetXIP_MAIN_CORE1(value uint32) {
	volatile.StoreUint32(&o.XIP_MAIN.Reg, volatile.LoadUint32(&o.XIP_MAIN.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetXIP_MAIN_CORE1() uint32 {
	return (volatile.LoadUint32(&o.XIP_MAIN.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetXIP_MAIN_CORE0(value uint32) {
	volatile.StoreUint32(&o.XIP_MAIN.Reg, volatile.LoadUint32(&o.XIP_MAIN.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetXIP_MAIN_CORE0() uint32 {
	return (volatile.LoadUint32(&o.XIP_MAIN.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetXIP_MAIN_SP(value uint32) {
	volatile.StoreUint32(&o.XIP_MAIN.Reg, volatile.LoadUint32(&o.XIP_MAIN.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetXIP_MAIN_SP() uint32 {
	return (volatile.LoadUint32(&o.XIP_MAIN.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetXIP_MAIN_SU(value uint32) {
	volatile.StoreUint32(&o.XIP_MAIN.Reg, volatile.LoadUint32(&o.XIP_MAIN.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetXIP_MAIN_SU() uint32 {
	return (volatile.LoadUint32(&o.XIP_MAIN.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetXIP_MAIN_NSP(value uint32) {
	volatile.StoreUint32(&o.XIP_MAIN.Reg, volatile.LoadUint32(&o.XIP_MAIN.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetXIP_MAIN_NSP() uint32 {
	return (volatile.LoadUint32(&o.XIP_MAIN.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetXIP_MAIN_NSU(value uint32) {
	volatile.StoreUint32(&o.XIP_MAIN.Reg, volatile.LoadUint32(&o.XIP_MAIN.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetXIP_MAIN_NSU() uint32 {
	return volatile.LoadUint32(&o.XIP_MAIN.Reg) & 0x1
}

// ACCESSCTRL.SRAM0: Control whether debugger, DMA, core 0 and core 1 can access SRAM0, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM0_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM0.Reg, volatile.LoadUint32(&o.SRAM0.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM0_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM0.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM0_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM0.Reg, volatile.LoadUint32(&o.SRAM0.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM0_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM0.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM0_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM0.Reg, volatile.LoadUint32(&o.SRAM0.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM0_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM0.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM0_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM0.Reg, volatile.LoadUint32(&o.SRAM0.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM0_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM0.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM0_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM0.Reg, volatile.LoadUint32(&o.SRAM0.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM0_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM0.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM0_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM0.Reg, volatile.LoadUint32(&o.SRAM0.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM0_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM0.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM0_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM0.Reg, volatile.LoadUint32(&o.SRAM0.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM0_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM0.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM0_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM0.Reg, volatile.LoadUint32(&o.SRAM0.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM0_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM0.Reg) & 0x1
}

// ACCESSCTRL.SRAM1: Control whether debugger, DMA, core 0 and core 1 can access SRAM1, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM1_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM1.Reg, volatile.LoadUint32(&o.SRAM1.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM1_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM1.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM1_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM1.Reg, volatile.LoadUint32(&o.SRAM1.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM1_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM1.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM1_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM1.Reg, volatile.LoadUint32(&o.SRAM1.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM1_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM1.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM1_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM1.Reg, volatile.LoadUint32(&o.SRAM1.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM1_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM1.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM1_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM1.Reg, volatile.LoadUint32(&o.SRAM1.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM1_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM1.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM1_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM1.Reg, volatile.LoadUint32(&o.SRAM1.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM1_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM1.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM1_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM1.Reg, volatile.LoadUint32(&o.SRAM1.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM1_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM1.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM1_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM1.Reg, volatile.LoadUint32(&o.SRAM1.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM1_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM1.Reg) & 0x1
}

// ACCESSCTRL.SRAM2: Control whether debugger, DMA, core 0 and core 1 can access SRAM2, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM2_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM2.Reg, volatile.LoadUint32(&o.SRAM2.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM2_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM2.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM2_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM2.Reg, volatile.LoadUint32(&o.SRAM2.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM2_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM2.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM2_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM2.Reg, volatile.LoadUint32(&o.SRAM2.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM2_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM2.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM2_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM2.Reg, volatile.LoadUint32(&o.SRAM2.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM2_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM2.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM2_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM2.Reg, volatile.LoadUint32(&o.SRAM2.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM2_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM2.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM2_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM2.Reg, volatile.LoadUint32(&o.SRAM2.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM2_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM2.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM2_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM2.Reg, volatile.LoadUint32(&o.SRAM2.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM2_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM2.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM2_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM2.Reg, volatile.LoadUint32(&o.SRAM2.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM2_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM2.Reg) & 0x1
}

// ACCESSCTRL.SRAM3: Control whether debugger, DMA, core 0 and core 1 can access SRAM3, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM3_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM3.Reg, volatile.LoadUint32(&o.SRAM3.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM3_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM3.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM3_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM3.Reg, volatile.LoadUint32(&o.SRAM3.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM3_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM3.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM3_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM3.Reg, volatile.LoadUint32(&o.SRAM3.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM3_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM3.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM3_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM3.Reg, volatile.LoadUint32(&o.SRAM3.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM3_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM3.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM3_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM3.Reg, volatile.LoadUint32(&o.SRAM3.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM3_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM3.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM3_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM3.Reg, volatile.LoadUint32(&o.SRAM3.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM3_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM3.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM3_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM3.Reg, volatile.LoadUint32(&o.SRAM3.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM3_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM3.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM3_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM3.Reg, volatile.LoadUint32(&o.SRAM3.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM3_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM3.Reg) & 0x1
}

// ACCESSCTRL.SRAM4: Control whether debugger, DMA, core 0 and core 1 can access SRAM4, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM4_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM4.Reg, volatile.LoadUint32(&o.SRAM4.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM4_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM4.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM4_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM4.Reg, volatile.LoadUint32(&o.SRAM4.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM4_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM4.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM4_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM4.Reg, volatile.LoadUint32(&o.SRAM4.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM4_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM4.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM4_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM4.Reg, volatile.LoadUint32(&o.SRAM4.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM4_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM4.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM4_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM4.Reg, volatile.LoadUint32(&o.SRAM4.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM4_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM4.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM4_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM4.Reg, volatile.LoadUint32(&o.SRAM4.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM4_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM4.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM4_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM4.Reg, volatile.LoadUint32(&o.SRAM4.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM4_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM4.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM4_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM4.Reg, volatile.LoadUint32(&o.SRAM4.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM4_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM4.Reg) & 0x1
}

// ACCESSCTRL.SRAM5: Control whether debugger, DMA, core 0 and core 1 can access SRAM5, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM5_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM5.Reg, volatile.LoadUint32(&o.SRAM5.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM5_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM5.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM5_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM5.Reg, volatile.LoadUint32(&o.SRAM5.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM5_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM5.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM5_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM5.Reg, volatile.LoadUint32(&o.SRAM5.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM5_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM5.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM5_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM5.Reg, volatile.LoadUint32(&o.SRAM5.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM5_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM5.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM5_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM5.Reg, volatile.LoadUint32(&o.SRAM5.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM5_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM5.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM5_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM5.Reg, volatile.LoadUint32(&o.SRAM5.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM5_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM5.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM5_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM5.Reg, volatile.LoadUint32(&o.SRAM5.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM5_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM5.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM5_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM5.Reg, volatile.LoadUint32(&o.SRAM5.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM5_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM5.Reg) & 0x1
}

// ACCESSCTRL.SRAM6: Control whether debugger, DMA, core 0 and core 1 can access SRAM6, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM6_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM6.Reg, volatile.LoadUint32(&o.SRAM6.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM6_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM6.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM6_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM6.Reg, volatile.LoadUint32(&o.SRAM6.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM6_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM6.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM6_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM6.Reg, volatile.LoadUint32(&o.SRAM6.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM6_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM6.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM6_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM6.Reg, volatile.LoadUint32(&o.SRAM6.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM6_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM6.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM6_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM6.Reg, volatile.LoadUint32(&o.SRAM6.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM6_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM6.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM6_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM6.Reg, volatile.LoadUint32(&o.SRAM6.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM6_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM6.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM6_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM6.Reg, volatile.LoadUint32(&o.SRAM6.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM6_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM6.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM6_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM6.Reg, volatile.LoadUint32(&o.SRAM6.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM6_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM6.Reg) & 0x1
}

// ACCESSCTRL.SRAM7: Control whether debugger, DMA, core 0 and core 1 can access SRAM7, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM7_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM7.Reg, volatile.LoadUint32(&o.SRAM7.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM7_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM7.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM7_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM7.Reg, volatile.LoadUint32(&o.SRAM7.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM7_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM7.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM7_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM7.Reg, volatile.LoadUint32(&o.SRAM7.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM7_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM7.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM7_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM7.Reg, volatile.LoadUint32(&o.SRAM7.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM7_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM7.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM7_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM7.Reg, volatile.LoadUint32(&o.SRAM7.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM7_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM7.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM7_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM7.Reg, volatile.LoadUint32(&o.SRAM7.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM7_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM7.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM7_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM7.Reg, volatile.LoadUint32(&o.SRAM7.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM7_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM7.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM7_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM7.Reg, volatile.LoadUint32(&o.SRAM7.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM7_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM7.Reg) & 0x1
}

// ACCESSCTRL.SRAM8: Control whether debugger, DMA, core 0 and core 1 can access SRAM8, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM8_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM8.Reg, volatile.LoadUint32(&o.SRAM8.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM8_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM8.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM8_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM8.Reg, volatile.LoadUint32(&o.SRAM8.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM8_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM8.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM8_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM8.Reg, volatile.LoadUint32(&o.SRAM8.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM8_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM8.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM8_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM8.Reg, volatile.LoadUint32(&o.SRAM8.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM8_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM8.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM8_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM8.Reg, volatile.LoadUint32(&o.SRAM8.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM8_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM8.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM8_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM8.Reg, volatile.LoadUint32(&o.SRAM8.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM8_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM8.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM8_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM8.Reg, volatile.LoadUint32(&o.SRAM8.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM8_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM8.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM8_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM8.Reg, volatile.LoadUint32(&o.SRAM8.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM8_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM8.Reg) & 0x1
}

// ACCESSCTRL.SRAM9: Control whether debugger, DMA, core 0 and core 1 can access SRAM9, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSRAM9_DBG(value uint32) {
	volatile.StoreUint32(&o.SRAM9.Reg, volatile.LoadUint32(&o.SRAM9.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSRAM9_DBG() uint32 {
	return (volatile.LoadUint32(&o.SRAM9.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSRAM9_DMA(value uint32) {
	volatile.StoreUint32(&o.SRAM9.Reg, volatile.LoadUint32(&o.SRAM9.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSRAM9_DMA() uint32 {
	return (volatile.LoadUint32(&o.SRAM9.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSRAM9_CORE1(value uint32) {
	volatile.StoreUint32(&o.SRAM9.Reg, volatile.LoadUint32(&o.SRAM9.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSRAM9_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SRAM9.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSRAM9_CORE0(value uint32) {
	volatile.StoreUint32(&o.SRAM9.Reg, volatile.LoadUint32(&o.SRAM9.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSRAM9_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SRAM9.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSRAM9_SP(value uint32) {
	volatile.StoreUint32(&o.SRAM9.Reg, volatile.LoadUint32(&o.SRAM9.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSRAM9_SP() uint32 {
	return (volatile.LoadUint32(&o.SRAM9.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSRAM9_SU(value uint32) {
	volatile.StoreUint32(&o.SRAM9.Reg, volatile.LoadUint32(&o.SRAM9.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSRAM9_SU() uint32 {
	return (volatile.LoadUint32(&o.SRAM9.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSRAM9_NSP(value uint32) {
	volatile.StoreUint32(&o.SRAM9.Reg, volatile.LoadUint32(&o.SRAM9.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSRAM9_NSP() uint32 {
	return (volatile.LoadUint32(&o.SRAM9.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSRAM9_NSU(value uint32) {
	volatile.StoreUint32(&o.SRAM9.Reg, volatile.LoadUint32(&o.SRAM9.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSRAM9_NSU() uint32 {
	return volatile.LoadUint32(&o.SRAM9.Reg) & 0x1
}

// ACCESSCTRL.DMA: Control whether debugger, DMA, core 0 and core 1 can access DMA, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetDMA_DBG(value uint32) {
	volatile.StoreUint32(&o.DMA.Reg, volatile.LoadUint32(&o.DMA.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetDMA_DBG() uint32 {
	return (volatile.LoadUint32(&o.DMA.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetDMA(value uint32) {
	volatile.StoreUint32(&o.DMA.Reg, volatile.LoadUint32(&o.DMA.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetDMA() uint32 {
	return (volatile.LoadUint32(&o.DMA.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetDMA_CORE1(value uint32) {
	volatile.StoreUint32(&o.DMA.Reg, volatile.LoadUint32(&o.DMA.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetDMA_CORE1() uint32 {
	return (volatile.LoadUint32(&o.DMA.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetDMA_CORE0(value uint32) {
	volatile.StoreUint32(&o.DMA.Reg, volatile.LoadUint32(&o.DMA.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetDMA_CORE0() uint32 {
	return (volatile.LoadUint32(&o.DMA.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetDMA_SP(value uint32) {
	volatile.StoreUint32(&o.DMA.Reg, volatile.LoadUint32(&o.DMA.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetDMA_SP() uint32 {
	return (volatile.LoadUint32(&o.DMA.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetDMA_SU(value uint32) {
	volatile.StoreUint32(&o.DMA.Reg, volatile.LoadUint32(&o.DMA.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetDMA_SU() uint32 {
	return (volatile.LoadUint32(&o.DMA.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetDMA_NSP(value uint32) {
	volatile.StoreUint32(&o.DMA.Reg, volatile.LoadUint32(&o.DMA.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetDMA_NSP() uint32 {
	return (volatile.LoadUint32(&o.DMA.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetDMA_NSU(value uint32) {
	volatile.StoreUint32(&o.DMA.Reg, volatile.LoadUint32(&o.DMA.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetDMA_NSU() uint32 {
	return volatile.LoadUint32(&o.DMA.Reg) & 0x1
}

// ACCESSCTRL.USBCTRL: Control whether debugger, DMA, core 0 and core 1 can access USBCTRL, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetUSBCTRL_DBG(value uint32) {
	volatile.StoreUint32(&o.USBCTRL.Reg, volatile.LoadUint32(&o.USBCTRL.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetUSBCTRL_DBG() uint32 {
	return (volatile.LoadUint32(&o.USBCTRL.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetUSBCTRL_DMA(value uint32) {
	volatile.StoreUint32(&o.USBCTRL.Reg, volatile.LoadUint32(&o.USBCTRL.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetUSBCTRL_DMA() uint32 {
	return (volatile.LoadUint32(&o.USBCTRL.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetUSBCTRL_CORE1(value uint32) {
	volatile.StoreUint32(&o.USBCTRL.Reg, volatile.LoadUint32(&o.USBCTRL.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetUSBCTRL_CORE1() uint32 {
	return (volatile.LoadUint32(&o.USBCTRL.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetUSBCTRL_CORE0(value uint32) {
	volatile.StoreUint32(&o.USBCTRL.Reg, volatile.LoadUint32(&o.USBCTRL.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetUSBCTRL_CORE0() uint32 {
	return (volatile.LoadUint32(&o.USBCTRL.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetUSBCTRL_SP(value uint32) {
	volatile.StoreUint32(&o.USBCTRL.Reg, volatile.LoadUint32(&o.USBCTRL.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetUSBCTRL_SP() uint32 {
	return (volatile.LoadUint32(&o.USBCTRL.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetUSBCTRL_SU(value uint32) {
	volatile.StoreUint32(&o.USBCTRL.Reg, volatile.LoadUint32(&o.USBCTRL.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetUSBCTRL_SU() uint32 {
	return (volatile.LoadUint32(&o.USBCTRL.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetUSBCTRL_NSP(value uint32) {
	volatile.StoreUint32(&o.USBCTRL.Reg, volatile.LoadUint32(&o.USBCTRL.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetUSBCTRL_NSP() uint32 {
	return (volatile.LoadUint32(&o.USBCTRL.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetUSBCTRL_NSU(value uint32) {
	volatile.StoreUint32(&o.USBCTRL.Reg, volatile.LoadUint32(&o.USBCTRL.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetUSBCTRL_NSU() uint32 {
	return volatile.LoadUint32(&o.USBCTRL.Reg) & 0x1
}

// ACCESSCTRL.PIO0: Control whether debugger, DMA, core 0 and core 1 can access PIO0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetPIO0_DBG(value uint32) {
	volatile.StoreUint32(&o.PIO0.Reg, volatile.LoadUint32(&o.PIO0.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetPIO0_DBG() uint32 {
	return (volatile.LoadUint32(&o.PIO0.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetPIO0_DMA(value uint32) {
	volatile.StoreUint32(&o.PIO0.Reg, volatile.LoadUint32(&o.PIO0.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetPIO0_DMA() uint32 {
	return (volatile.LoadUint32(&o.PIO0.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetPIO0_CORE1(value uint32) {
	volatile.StoreUint32(&o.PIO0.Reg, volatile.LoadUint32(&o.PIO0.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetPIO0_CORE1() uint32 {
	return (volatile.LoadUint32(&o.PIO0.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetPIO0_CORE0(value uint32) {
	volatile.StoreUint32(&o.PIO0.Reg, volatile.LoadUint32(&o.PIO0.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetPIO0_CORE0() uint32 {
	return (volatile.LoadUint32(&o.PIO0.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetPIO0_SP(value uint32) {
	volatile.StoreUint32(&o.PIO0.Reg, volatile.LoadUint32(&o.PIO0.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetPIO0_SP() uint32 {
	return (volatile.LoadUint32(&o.PIO0.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetPIO0_SU(value uint32) {
	volatile.StoreUint32(&o.PIO0.Reg, volatile.LoadUint32(&o.PIO0.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetPIO0_SU() uint32 {
	return (volatile.LoadUint32(&o.PIO0.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetPIO0_NSP(value uint32) {
	volatile.StoreUint32(&o.PIO0.Reg, volatile.LoadUint32(&o.PIO0.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetPIO0_NSP() uint32 {
	return (volatile.LoadUint32(&o.PIO0.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetPIO0_NSU(value uint32) {
	volatile.StoreUint32(&o.PIO0.Reg, volatile.LoadUint32(&o.PIO0.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetPIO0_NSU() uint32 {
	return volatile.LoadUint32(&o.PIO0.Reg) & 0x1
}

// ACCESSCTRL.PIO1: Control whether debugger, DMA, core 0 and core 1 can access PIO1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetPIO1_DBG(value uint32) {
	volatile.StoreUint32(&o.PIO1.Reg, volatile.LoadUint32(&o.PIO1.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetPIO1_DBG() uint32 {
	return (volatile.LoadUint32(&o.PIO1.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetPIO1_DMA(value uint32) {
	volatile.StoreUint32(&o.PIO1.Reg, volatile.LoadUint32(&o.PIO1.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetPIO1_DMA() uint32 {
	return (volatile.LoadUint32(&o.PIO1.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetPIO1_CORE1(value uint32) {
	volatile.StoreUint32(&o.PIO1.Reg, volatile.LoadUint32(&o.PIO1.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetPIO1_CORE1() uint32 {
	return (volatile.LoadUint32(&o.PIO1.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetPIO1_CORE0(value uint32) {
	volatile.StoreUint32(&o.PIO1.Reg, volatile.LoadUint32(&o.PIO1.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetPIO1_CORE0() uint32 {
	return (volatile.LoadUint32(&o.PIO1.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetPIO1_SP(value uint32) {
	volatile.StoreUint32(&o.PIO1.Reg, volatile.LoadUint32(&o.PIO1.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetPIO1_SP() uint32 {
	return (volatile.LoadUint32(&o.PIO1.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetPIO1_SU(value uint32) {
	volatile.StoreUint32(&o.PIO1.Reg, volatile.LoadUint32(&o.PIO1.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetPIO1_SU() uint32 {
	return (volatile.LoadUint32(&o.PIO1.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetPIO1_NSP(value uint32) {
	volatile.StoreUint32(&o.PIO1.Reg, volatile.LoadUint32(&o.PIO1.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetPIO1_NSP() uint32 {
	return (volatile.LoadUint32(&o.PIO1.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetPIO1_NSU(value uint32) {
	volatile.StoreUint32(&o.PIO1.Reg, volatile.LoadUint32(&o.PIO1.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetPIO1_NSU() uint32 {
	return volatile.LoadUint32(&o.PIO1.Reg) & 0x1
}

// ACCESSCTRL.PIO2: Control whether debugger, DMA, core 0 and core 1 can access PIO2, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetPIO2_DBG(value uint32) {
	volatile.StoreUint32(&o.PIO2.Reg, volatile.LoadUint32(&o.PIO2.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetPIO2_DBG() uint32 {
	return (volatile.LoadUint32(&o.PIO2.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetPIO2_DMA(value uint32) {
	volatile.StoreUint32(&o.PIO2.Reg, volatile.LoadUint32(&o.PIO2.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetPIO2_DMA() uint32 {
	return (volatile.LoadUint32(&o.PIO2.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetPIO2_CORE1(value uint32) {
	volatile.StoreUint32(&o.PIO2.Reg, volatile.LoadUint32(&o.PIO2.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetPIO2_CORE1() uint32 {
	return (volatile.LoadUint32(&o.PIO2.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetPIO2_CORE0(value uint32) {
	volatile.StoreUint32(&o.PIO2.Reg, volatile.LoadUint32(&o.PIO2.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetPIO2_CORE0() uint32 {
	return (volatile.LoadUint32(&o.PIO2.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetPIO2_SP(value uint32) {
	volatile.StoreUint32(&o.PIO2.Reg, volatile.LoadUint32(&o.PIO2.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetPIO2_SP() uint32 {
	return (volatile.LoadUint32(&o.PIO2.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetPIO2_SU(value uint32) {
	volatile.StoreUint32(&o.PIO2.Reg, volatile.LoadUint32(&o.PIO2.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetPIO2_SU() uint32 {
	return (volatile.LoadUint32(&o.PIO2.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetPIO2_NSP(value uint32) {
	volatile.StoreUint32(&o.PIO2.Reg, volatile.LoadUint32(&o.PIO2.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetPIO2_NSP() uint32 {
	return (volatile.LoadUint32(&o.PIO2.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetPIO2_NSU(value uint32) {
	volatile.StoreUint32(&o.PIO2.Reg, volatile.LoadUint32(&o.PIO2.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetPIO2_NSU() uint32 {
	return volatile.LoadUint32(&o.PIO2.Reg) & 0x1
}

// ACCESSCTRL.CORESIGHT_TRACE: Control whether debugger, DMA, core 0 and core 1 can access CORESIGHT_TRACE, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetCORESIGHT_TRACE_DBG(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_TRACE.Reg, volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_TRACE_DBG() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_TRACE_DMA(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_TRACE.Reg, volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_TRACE_DMA() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_TRACE_CORE1(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_TRACE.Reg, volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_TRACE_CORE1() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_TRACE_CORE0(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_TRACE.Reg, volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_TRACE_CORE0() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_TRACE_SP(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_TRACE.Reg, volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_TRACE_SP() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_TRACE_SU(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_TRACE.Reg, volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_TRACE_SU() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_TRACE_NSP(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_TRACE.Reg, volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_TRACE_NSP() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_TRACE_NSU(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_TRACE.Reg, volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_TRACE_NSU() uint32 {
	return volatile.LoadUint32(&o.CORESIGHT_TRACE.Reg) & 0x1
}

// ACCESSCTRL.CORESIGHT_PERIPH: Control whether debugger, DMA, core 0 and core 1 can access CORESIGHT_PERIPH, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetCORESIGHT_PERIPH_DBG(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_PERIPH.Reg, volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_PERIPH_DBG() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_PERIPH_DMA(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_PERIPH.Reg, volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_PERIPH_DMA() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_PERIPH_CORE1(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_PERIPH.Reg, volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_PERIPH_CORE1() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_PERIPH_CORE0(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_PERIPH.Reg, volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_PERIPH_CORE0() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_PERIPH_SP(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_PERIPH.Reg, volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_PERIPH_SP() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_PERIPH_SU(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_PERIPH.Reg, volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_PERIPH_SU() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_PERIPH_NSP(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_PERIPH.Reg, volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_PERIPH_NSP() uint32 {
	return (volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetCORESIGHT_PERIPH_NSU(value uint32) {
	volatile.StoreUint32(&o.CORESIGHT_PERIPH.Reg, volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetCORESIGHT_PERIPH_NSU() uint32 {
	return volatile.LoadUint32(&o.CORESIGHT_PERIPH.Reg) & 0x1
}

// ACCESSCTRL.SYSINFO: Control whether debugger, DMA, core 0 and core 1 can access SYSINFO, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSYSINFO_DBG(value uint32) {
	volatile.StoreUint32(&o.SYSINFO.Reg, volatile.LoadUint32(&o.SYSINFO.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSYSINFO_DBG() uint32 {
	return (volatile.LoadUint32(&o.SYSINFO.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSYSINFO_DMA(value uint32) {
	volatile.StoreUint32(&o.SYSINFO.Reg, volatile.LoadUint32(&o.SYSINFO.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSYSINFO_DMA() uint32 {
	return (volatile.LoadUint32(&o.SYSINFO.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSYSINFO_CORE1(value uint32) {
	volatile.StoreUint32(&o.SYSINFO.Reg, volatile.LoadUint32(&o.SYSINFO.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSYSINFO_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SYSINFO.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSYSINFO_CORE0(value uint32) {
	volatile.StoreUint32(&o.SYSINFO.Reg, volatile.LoadUint32(&o.SYSINFO.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSYSINFO_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SYSINFO.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSYSINFO_SP(value uint32) {
	volatile.StoreUint32(&o.SYSINFO.Reg, volatile.LoadUint32(&o.SYSINFO.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSYSINFO_SP() uint32 {
	return (volatile.LoadUint32(&o.SYSINFO.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSYSINFO_SU(value uint32) {
	volatile.StoreUint32(&o.SYSINFO.Reg, volatile.LoadUint32(&o.SYSINFO.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSYSINFO_SU() uint32 {
	return (volatile.LoadUint32(&o.SYSINFO.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSYSINFO_NSP(value uint32) {
	volatile.StoreUint32(&o.SYSINFO.Reg, volatile.LoadUint32(&o.SYSINFO.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSYSINFO_NSP() uint32 {
	return (volatile.LoadUint32(&o.SYSINFO.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSYSINFO_NSU(value uint32) {
	volatile.StoreUint32(&o.SYSINFO.Reg, volatile.LoadUint32(&o.SYSINFO.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSYSINFO_NSU() uint32 {
	return volatile.LoadUint32(&o.SYSINFO.Reg) & 0x1
}

// ACCESSCTRL.RESETS: Control whether debugger, DMA, core 0 and core 1 can access RESETS, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetRESETS_DBG(value uint32) {
	volatile.StoreUint32(&o.RESETS.Reg, volatile.LoadUint32(&o.RESETS.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetRESETS_DBG() uint32 {
	return (volatile.LoadUint32(&o.RESETS.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetRESETS_DMA(value uint32) {
	volatile.StoreUint32(&o.RESETS.Reg, volatile.LoadUint32(&o.RESETS.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetRESETS_DMA() uint32 {
	return (volatile.LoadUint32(&o.RESETS.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetRESETS_CORE1(value uint32) {
	volatile.StoreUint32(&o.RESETS.Reg, volatile.LoadUint32(&o.RESETS.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetRESETS_CORE1() uint32 {
	return (volatile.LoadUint32(&o.RESETS.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetRESETS_CORE0(value uint32) {
	volatile.StoreUint32(&o.RESETS.Reg, volatile.LoadUint32(&o.RESETS.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetRESETS_CORE0() uint32 {
	return (volatile.LoadUint32(&o.RESETS.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetRESETS_SP(value uint32) {
	volatile.StoreUint32(&o.RESETS.Reg, volatile.LoadUint32(&o.RESETS.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetRESETS_SP() uint32 {
	return (volatile.LoadUint32(&o.RESETS.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetRESETS_SU(value uint32) {
	volatile.StoreUint32(&o.RESETS.Reg, volatile.LoadUint32(&o.RESETS.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetRESETS_SU() uint32 {
	return (volatile.LoadUint32(&o.RESETS.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetRESETS_NSP(value uint32) {
	volatile.StoreUint32(&o.RESETS.Reg, volatile.LoadUint32(&o.RESETS.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetRESETS_NSP() uint32 {
	return (volatile.LoadUint32(&o.RESETS.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetRESETS_NSU(value uint32) {
	volatile.StoreUint32(&o.RESETS.Reg, volatile.LoadUint32(&o.RESETS.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetRESETS_NSU() uint32 {
	return volatile.LoadUint32(&o.RESETS.Reg) & 0x1
}

// ACCESSCTRL.IO_BANK0: Control whether debugger, DMA, core 0 and core 1 can access IO_BANK0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetIO_BANK0_DBG(value uint32) {
	volatile.StoreUint32(&o.IO_BANK0.Reg, volatile.LoadUint32(&o.IO_BANK0.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetIO_BANK0_DBG() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK0.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetIO_BANK0_DMA(value uint32) {
	volatile.StoreUint32(&o.IO_BANK0.Reg, volatile.LoadUint32(&o.IO_BANK0.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetIO_BANK0_DMA() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK0.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetIO_BANK0_CORE1(value uint32) {
	volatile.StoreUint32(&o.IO_BANK0.Reg, volatile.LoadUint32(&o.IO_BANK0.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetIO_BANK0_CORE1() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK0.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetIO_BANK0_CORE0(value uint32) {
	volatile.StoreUint32(&o.IO_BANK0.Reg, volatile.LoadUint32(&o.IO_BANK0.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetIO_BANK0_CORE0() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK0.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetIO_BANK0_SP(value uint32) {
	volatile.StoreUint32(&o.IO_BANK0.Reg, volatile.LoadUint32(&o.IO_BANK0.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetIO_BANK0_SP() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK0.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetIO_BANK0_SU(value uint32) {
	volatile.StoreUint32(&o.IO_BANK0.Reg, volatile.LoadUint32(&o.IO_BANK0.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetIO_BANK0_SU() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK0.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetIO_BANK0_NSP(value uint32) {
	volatile.StoreUint32(&o.IO_BANK0.Reg, volatile.LoadUint32(&o.IO_BANK0.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetIO_BANK0_NSP() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK0.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetIO_BANK0_NSU(value uint32) {
	volatile.StoreUint32(&o.IO_BANK0.Reg, volatile.LoadUint32(&o.IO_BANK0.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetIO_BANK0_NSU() uint32 {
	return volatile.LoadUint32(&o.IO_BANK0.Reg) & 0x1
}

// ACCESSCTRL.IO_BANK1: Control whether debugger, DMA, core 0 and core 1 can access IO_BANK1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetIO_BANK1_DBG(value uint32) {
	volatile.StoreUint32(&o.IO_BANK1.Reg, volatile.LoadUint32(&o.IO_BANK1.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetIO_BANK1_DBG() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK1.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetIO_BANK1_DMA(value uint32) {
	volatile.StoreUint32(&o.IO_BANK1.Reg, volatile.LoadUint32(&o.IO_BANK1.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetIO_BANK1_DMA() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK1.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetIO_BANK1_CORE1(value uint32) {
	volatile.StoreUint32(&o.IO_BANK1.Reg, volatile.LoadUint32(&o.IO_BANK1.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetIO_BANK1_CORE1() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK1.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetIO_BANK1_CORE0(value uint32) {
	volatile.StoreUint32(&o.IO_BANK1.Reg, volatile.LoadUint32(&o.IO_BANK1.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetIO_BANK1_CORE0() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK1.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetIO_BANK1_SP(value uint32) {
	volatile.StoreUint32(&o.IO_BANK1.Reg, volatile.LoadUint32(&o.IO_BANK1.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetIO_BANK1_SP() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK1.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetIO_BANK1_SU(value uint32) {
	volatile.StoreUint32(&o.IO_BANK1.Reg, volatile.LoadUint32(&o.IO_BANK1.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetIO_BANK1_SU() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK1.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetIO_BANK1_NSP(value uint32) {
	volatile.StoreUint32(&o.IO_BANK1.Reg, volatile.LoadUint32(&o.IO_BANK1.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetIO_BANK1_NSP() uint32 {
	return (volatile.LoadUint32(&o.IO_BANK1.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetIO_BANK1_NSU(value uint32) {
	volatile.StoreUint32(&o.IO_BANK1.Reg, volatile.LoadUint32(&o.IO_BANK1.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetIO_BANK1_NSU() uint32 {
	return volatile.LoadUint32(&o.IO_BANK1.Reg) & 0x1
}

// ACCESSCTRL.PADS_BANK0: Control whether debugger, DMA, core 0 and core 1 can access PADS_BANK0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetPADS_BANK0_DBG(value uint32) {
	volatile.StoreUint32(&o.PADS_BANK0.Reg, volatile.LoadUint32(&o.PADS_BANK0.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetPADS_BANK0_DBG() uint32 {
	return (volatile.LoadUint32(&o.PADS_BANK0.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetPADS_BANK0_DMA(value uint32) {
	volatile.StoreUint32(&o.PADS_BANK0.Reg, volatile.LoadUint32(&o.PADS_BANK0.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetPADS_BANK0_DMA() uint32 {
	return (volatile.LoadUint32(&o.PADS_BANK0.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetPADS_BANK0_CORE1(value uint32) {
	volatile.StoreUint32(&o.PADS_BANK0.Reg, volatile.LoadUint32(&o.PADS_BANK0.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetPADS_BANK0_CORE1() uint32 {
	return (volatile.LoadUint32(&o.PADS_BANK0.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetPADS_BANK0_CORE0(value uint32) {
	volatile.StoreUint32(&o.PADS_BANK0.Reg, volatile.LoadUint32(&o.PADS_BANK0.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetPADS_BANK0_CORE0() uint32 {
	return (volatile.LoadUint32(&o.PADS_BANK0.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetPADS_BANK0_SP(value uint32) {
	volatile.StoreUint32(&o.PADS_BANK0.Reg, volatile.LoadUint32(&o.PADS_BANK0.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetPADS_BANK0_SP() uint32 {
	return (volatile.LoadUint32(&o.PADS_BANK0.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetPADS_BANK0_SU(value uint32) {
	volatile.StoreUint32(&o.PADS_BANK0.Reg, volatile.LoadUint32(&o.PADS_BANK0.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetPADS_BANK0_SU() uint32 {
	return (volatile.LoadUint32(&o.PADS_BANK0.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetPADS_BANK0_NSP(value uint32) {
	volatile.StoreUint32(&o.PADS_BANK0.Reg, volatile.LoadUint32(&o.PADS_BANK0.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetPADS_BANK0_NSP() uint32 {
	return (volatile.LoadUint32(&o.PADS_BANK0.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetPADS_BANK0_NSU(value uint32) {
	volatile.StoreUint32(&o.PADS_BANK0.Reg, volatile.LoadUint32(&o.PADS_BANK0.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetPADS_BANK0_NSU() uint32 {
	return volatile.LoadUint32(&o.PADS_BANK0.Reg) & 0x1
}

// ACCESSCTRL.PADS_QSPI: Control whether debugger, DMA, core 0 and core 1 can access PADS_QSPI, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetPADS_QSPI_DBG(value uint32) {
	volatile.StoreUint32(&o.PADS_QSPI.Reg, volatile.LoadUint32(&o.PADS_QSPI.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetPADS_QSPI_DBG() uint32 {
	return (volatile.LoadUint32(&o.PADS_QSPI.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetPADS_QSPI_DMA(value uint32) {
	volatile.StoreUint32(&o.PADS_QSPI.Reg, volatile.LoadUint32(&o.PADS_QSPI.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetPADS_QSPI_DMA() uint32 {
	return (volatile.LoadUint32(&o.PADS_QSPI.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetPADS_QSPI_CORE1(value uint32) {
	volatile.StoreUint32(&o.PADS_QSPI.Reg, volatile.LoadUint32(&o.PADS_QSPI.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetPADS_QSPI_CORE1() uint32 {
	return (volatile.LoadUint32(&o.PADS_QSPI.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetPADS_QSPI_CORE0(value uint32) {
	volatile.StoreUint32(&o.PADS_QSPI.Reg, volatile.LoadUint32(&o.PADS_QSPI.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetPADS_QSPI_CORE0() uint32 {
	return (volatile.LoadUint32(&o.PADS_QSPI.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetPADS_QSPI_SP(value uint32) {
	volatile.StoreUint32(&o.PADS_QSPI.Reg, volatile.LoadUint32(&o.PADS_QSPI.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetPADS_QSPI_SP() uint32 {
	return (volatile.LoadUint32(&o.PADS_QSPI.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetPADS_QSPI_SU(value uint32) {
	volatile.StoreUint32(&o.PADS_QSPI.Reg, volatile.LoadUint32(&o.PADS_QSPI.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetPADS_QSPI_SU() uint32 {
	return (volatile.LoadUint32(&o.PADS_QSPI.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetPADS_QSPI_NSP(value uint32) {
	volatile.StoreUint32(&o.PADS_QSPI.Reg, volatile.LoadUint32(&o.PADS_QSPI.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetPADS_QSPI_NSP() uint32 {
	return (volatile.LoadUint32(&o.PADS_QSPI.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetPADS_QSPI_NSU(value uint32) {
	volatile.StoreUint32(&o.PADS_QSPI.Reg, volatile.LoadUint32(&o.PADS_QSPI.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetPADS_QSPI_NSU() uint32 {
	return volatile.LoadUint32(&o.PADS_QSPI.Reg) & 0x1
}

// ACCESSCTRL.BUSCTRL: Control whether debugger, DMA, core 0 and core 1 can access BUSCTRL, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetBUSCTRL_DBG(value uint32) {
	volatile.StoreUint32(&o.BUSCTRL.Reg, volatile.LoadUint32(&o.BUSCTRL.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetBUSCTRL_DBG() uint32 {
	return (volatile.LoadUint32(&o.BUSCTRL.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetBUSCTRL_DMA(value uint32) {
	volatile.StoreUint32(&o.BUSCTRL.Reg, volatile.LoadUint32(&o.BUSCTRL.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetBUSCTRL_DMA() uint32 {
	return (volatile.LoadUint32(&o.BUSCTRL.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetBUSCTRL_CORE1(value uint32) {
	volatile.StoreUint32(&o.BUSCTRL.Reg, volatile.LoadUint32(&o.BUSCTRL.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetBUSCTRL_CORE1() uint32 {
	return (volatile.LoadUint32(&o.BUSCTRL.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetBUSCTRL_CORE0(value uint32) {
	volatile.StoreUint32(&o.BUSCTRL.Reg, volatile.LoadUint32(&o.BUSCTRL.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetBUSCTRL_CORE0() uint32 {
	return (volatile.LoadUint32(&o.BUSCTRL.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetBUSCTRL_SP(value uint32) {
	volatile.StoreUint32(&o.BUSCTRL.Reg, volatile.LoadUint32(&o.BUSCTRL.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetBUSCTRL_SP() uint32 {
	return (volatile.LoadUint32(&o.BUSCTRL.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetBUSCTRL_SU(value uint32) {
	volatile.StoreUint32(&o.BUSCTRL.Reg, volatile.LoadUint32(&o.BUSCTRL.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetBUSCTRL_SU() uint32 {
	return (volatile.LoadUint32(&o.BUSCTRL.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetBUSCTRL_NSP(value uint32) {
	volatile.StoreUint32(&o.BUSCTRL.Reg, volatile.LoadUint32(&o.BUSCTRL.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetBUSCTRL_NSP() uint32 {
	return (volatile.LoadUint32(&o.BUSCTRL.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetBUSCTRL_NSU(value uint32) {
	volatile.StoreUint32(&o.BUSCTRL.Reg, volatile.LoadUint32(&o.BUSCTRL.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetBUSCTRL_NSU() uint32 {
	return volatile.LoadUint32(&o.BUSCTRL.Reg) & 0x1
}

// ACCESSCTRL.ADC0: Control whether debugger, DMA, core 0 and core 1 can access ADC0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetADC0_DBG(value uint32) {
	volatile.StoreUint32(&o.ADC0.Reg, volatile.LoadUint32(&o.ADC0.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetADC0_DBG() uint32 {
	return (volatile.LoadUint32(&o.ADC0.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetADC0_DMA(value uint32) {
	volatile.StoreUint32(&o.ADC0.Reg, volatile.LoadUint32(&o.ADC0.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetADC0_DMA() uint32 {
	return (volatile.LoadUint32(&o.ADC0.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetADC0_CORE1(value uint32) {
	volatile.StoreUint32(&o.ADC0.Reg, volatile.LoadUint32(&o.ADC0.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetADC0_CORE1() uint32 {
	return (volatile.LoadUint32(&o.ADC0.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetADC0_CORE0(value uint32) {
	volatile.StoreUint32(&o.ADC0.Reg, volatile.LoadUint32(&o.ADC0.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetADC0_CORE0() uint32 {
	return (volatile.LoadUint32(&o.ADC0.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetADC0_SP(value uint32) {
	volatile.StoreUint32(&o.ADC0.Reg, volatile.LoadUint32(&o.ADC0.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetADC0_SP() uint32 {
	return (volatile.LoadUint32(&o.ADC0.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetADC0_SU(value uint32) {
	volatile.StoreUint32(&o.ADC0.Reg, volatile.LoadUint32(&o.ADC0.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetADC0_SU() uint32 {
	return (volatile.LoadUint32(&o.ADC0.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetADC0_NSP(value uint32) {
	volatile.StoreUint32(&o.ADC0.Reg, volatile.LoadUint32(&o.ADC0.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetADC0_NSP() uint32 {
	return (volatile.LoadUint32(&o.ADC0.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetADC0_NSU(value uint32) {
	volatile.StoreUint32(&o.ADC0.Reg, volatile.LoadUint32(&o.ADC0.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetADC0_NSU() uint32 {
	return volatile.LoadUint32(&o.ADC0.Reg) & 0x1
}

// ACCESSCTRL.HSTX: Control whether debugger, DMA, core 0 and core 1 can access HSTX, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetHSTX_DBG(value uint32) {
	volatile.StoreUint32(&o.HSTX.Reg, volatile.LoadUint32(&o.HSTX.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetHSTX_DBG() uint32 {
	return (volatile.LoadUint32(&o.HSTX.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetHSTX_DMA(value uint32) {
	volatile.StoreUint32(&o.HSTX.Reg, volatile.LoadUint32(&o.HSTX.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetHSTX_DMA() uint32 {
	return (volatile.LoadUint32(&o.HSTX.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetHSTX_CORE1(value uint32) {
	volatile.StoreUint32(&o.HSTX.Reg, volatile.LoadUint32(&o.HSTX.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetHSTX_CORE1() uint32 {
	return (volatile.LoadUint32(&o.HSTX.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetHSTX_CORE0(value uint32) {
	volatile.StoreUint32(&o.HSTX.Reg, volatile.LoadUint32(&o.HSTX.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetHSTX_CORE0() uint32 {
	return (volatile.LoadUint32(&o.HSTX.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetHSTX_SP(value uint32) {
	volatile.StoreUint32(&o.HSTX.Reg, volatile.LoadUint32(&o.HSTX.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetHSTX_SP() uint32 {
	return (volatile.LoadUint32(&o.HSTX.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetHSTX_SU(value uint32) {
	volatile.StoreUint32(&o.HSTX.Reg, volatile.LoadUint32(&o.HSTX.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetHSTX_SU() uint32 {
	return (volatile.LoadUint32(&o.HSTX.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetHSTX_NSP(value uint32) {
	volatile.StoreUint32(&o.HSTX.Reg, volatile.LoadUint32(&o.HSTX.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetHSTX_NSP() uint32 {
	return (volatile.LoadUint32(&o.HSTX.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetHSTX_NSU(value uint32) {
	volatile.StoreUint32(&o.HSTX.Reg, volatile.LoadUint32(&o.HSTX.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetHSTX_NSU() uint32 {
	return volatile.LoadUint32(&o.HSTX.Reg) & 0x1
}

// ACCESSCTRL.I2C0: Control whether debugger, DMA, core 0 and core 1 can access I2C0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetI2C0_DBG(value uint32) {
	volatile.StoreUint32(&o.I2C0.Reg, volatile.LoadUint32(&o.I2C0.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetI2C0_DBG() uint32 {
	return (volatile.LoadUint32(&o.I2C0.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetI2C0_DMA(value uint32) {
	volatile.StoreUint32(&o.I2C0.Reg, volatile.LoadUint32(&o.I2C0.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetI2C0_DMA() uint32 {
	return (volatile.LoadUint32(&o.I2C0.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetI2C0_CORE1(value uint32) {
	volatile.StoreUint32(&o.I2C0.Reg, volatile.LoadUint32(&o.I2C0.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetI2C0_CORE1() uint32 {
	return (volatile.LoadUint32(&o.I2C0.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetI2C0_CORE0(value uint32) {
	volatile.StoreUint32(&o.I2C0.Reg, volatile.LoadUint32(&o.I2C0.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetI2C0_CORE0() uint32 {
	return (volatile.LoadUint32(&o.I2C0.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetI2C0_SP(value uint32) {
	volatile.StoreUint32(&o.I2C0.Reg, volatile.LoadUint32(&o.I2C0.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetI2C0_SP() uint32 {
	return (volatile.LoadUint32(&o.I2C0.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetI2C0_SU(value uint32) {
	volatile.StoreUint32(&o.I2C0.Reg, volatile.LoadUint32(&o.I2C0.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetI2C0_SU() uint32 {
	return (volatile.LoadUint32(&o.I2C0.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetI2C0_NSP(value uint32) {
	volatile.StoreUint32(&o.I2C0.Reg, volatile.LoadUint32(&o.I2C0.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetI2C0_NSP() uint32 {
	return (volatile.LoadUint32(&o.I2C0.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetI2C0_NSU(value uint32) {
	volatile.StoreUint32(&o.I2C0.Reg, volatile.LoadUint32(&o.I2C0.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetI2C0_NSU() uint32 {
	return volatile.LoadUint32(&o.I2C0.Reg) & 0x1
}

// ACCESSCTRL.I2C1: Control whether debugger, DMA, core 0 and core 1 can access I2C1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetI2C1_DBG(value uint32) {
	volatile.StoreUint32(&o.I2C1.Reg, volatile.LoadUint32(&o.I2C1.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetI2C1_DBG() uint32 {
	return (volatile.LoadUint32(&o.I2C1.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetI2C1_DMA(value uint32) {
	volatile.StoreUint32(&o.I2C1.Reg, volatile.LoadUint32(&o.I2C1.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetI2C1_DMA() uint32 {
	return (volatile.LoadUint32(&o.I2C1.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetI2C1_CORE1(value uint32) {
	volatile.StoreUint32(&o.I2C1.Reg, volatile.LoadUint32(&o.I2C1.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetI2C1_CORE1() uint32 {
	return (volatile.LoadUint32(&o.I2C1.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetI2C1_CORE0(value uint32) {
	volatile.StoreUint32(&o.I2C1.Reg, volatile.LoadUint32(&o.I2C1.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetI2C1_CORE0() uint32 {
	return (volatile.LoadUint32(&o.I2C1.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetI2C1_SP(value uint32) {
	volatile.StoreUint32(&o.I2C1.Reg, volatile.LoadUint32(&o.I2C1.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetI2C1_SP() uint32 {
	return (volatile.LoadUint32(&o.I2C1.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetI2C1_SU(value uint32) {
	volatile.StoreUint32(&o.I2C1.Reg, volatile.LoadUint32(&o.I2C1.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetI2C1_SU() uint32 {
	return (volatile.LoadUint32(&o.I2C1.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetI2C1_NSP(value uint32) {
	volatile.StoreUint32(&o.I2C1.Reg, volatile.LoadUint32(&o.I2C1.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetI2C1_NSP() uint32 {
	return (volatile.LoadUint32(&o.I2C1.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetI2C1_NSU(value uint32) {
	volatile.StoreUint32(&o.I2C1.Reg, volatile.LoadUint32(&o.I2C1.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetI2C1_NSU() uint32 {
	return volatile.LoadUint32(&o.I2C1.Reg) & 0x1
}

// ACCESSCTRL.PWM: Control whether debugger, DMA, core 0 and core 1 can access PWM, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetPWM_DBG(value uint32) {
	volatile.StoreUint32(&o.PWM.Reg, volatile.LoadUint32(&o.PWM.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetPWM_DBG() uint32 {
	return (volatile.LoadUint32(&o.PWM.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetPWM_DMA(value uint32) {
	volatile.StoreUint32(&o.PWM.Reg, volatile.LoadUint32(&o.PWM.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetPWM_DMA() uint32 {
	return (volatile.LoadUint32(&o.PWM.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetPWM_CORE1(value uint32) {
	volatile.StoreUint32(&o.PWM.Reg, volatile.LoadUint32(&o.PWM.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetPWM_CORE1() uint32 {
	return (volatile.LoadUint32(&o.PWM.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetPWM_CORE0(value uint32) {
	volatile.StoreUint32(&o.PWM.Reg, volatile.LoadUint32(&o.PWM.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetPWM_CORE0() uint32 {
	return (volatile.LoadUint32(&o.PWM.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetPWM_SP(value uint32) {
	volatile.StoreUint32(&o.PWM.Reg, volatile.LoadUint32(&o.PWM.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetPWM_SP() uint32 {
	return (volatile.LoadUint32(&o.PWM.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetPWM_SU(value uint32) {
	volatile.StoreUint32(&o.PWM.Reg, volatile.LoadUint32(&o.PWM.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetPWM_SU() uint32 {
	return (volatile.LoadUint32(&o.PWM.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetPWM_NSP(value uint32) {
	volatile.StoreUint32(&o.PWM.Reg, volatile.LoadUint32(&o.PWM.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetPWM_NSP() uint32 {
	return (volatile.LoadUint32(&o.PWM.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetPWM_NSU(value uint32) {
	volatile.StoreUint32(&o.PWM.Reg, volatile.LoadUint32(&o.PWM.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetPWM_NSU() uint32 {
	return volatile.LoadUint32(&o.PWM.Reg) & 0x1
}

// ACCESSCTRL.SPI0: Control whether debugger, DMA, core 0 and core 1 can access SPI0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSPI0_DBG(value uint32) {
	volatile.StoreUint32(&o.SPI0.Reg, volatile.LoadUint32(&o.SPI0.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSPI0_DBG() uint32 {
	return (volatile.LoadUint32(&o.SPI0.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSPI0_DMA(value uint32) {
	volatile.StoreUint32(&o.SPI0.Reg, volatile.LoadUint32(&o.SPI0.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSPI0_DMA() uint32 {
	return (volatile.LoadUint32(&o.SPI0.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSPI0_CORE1(value uint32) {
	volatile.StoreUint32(&o.SPI0.Reg, volatile.LoadUint32(&o.SPI0.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSPI0_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SPI0.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSPI0_CORE0(value uint32) {
	volatile.StoreUint32(&o.SPI0.Reg, volatile.LoadUint32(&o.SPI0.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSPI0_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SPI0.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSPI0_SP(value uint32) {
	volatile.StoreUint32(&o.SPI0.Reg, volatile.LoadUint32(&o.SPI0.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSPI0_SP() uint32 {
	return (volatile.LoadUint32(&o.SPI0.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSPI0_SU(value uint32) {
	volatile.StoreUint32(&o.SPI0.Reg, volatile.LoadUint32(&o.SPI0.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSPI0_SU() uint32 {
	return (volatile.LoadUint32(&o.SPI0.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSPI0_NSP(value uint32) {
	volatile.StoreUint32(&o.SPI0.Reg, volatile.LoadUint32(&o.SPI0.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSPI0_NSP() uint32 {
	return (volatile.LoadUint32(&o.SPI0.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSPI0_NSU(value uint32) {
	volatile.StoreUint32(&o.SPI0.Reg, volatile.LoadUint32(&o.SPI0.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSPI0_NSU() uint32 {
	return volatile.LoadUint32(&o.SPI0.Reg) & 0x1
}

// ACCESSCTRL.SPI1: Control whether debugger, DMA, core 0 and core 1 can access SPI1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSPI1_DBG(value uint32) {
	volatile.StoreUint32(&o.SPI1.Reg, volatile.LoadUint32(&o.SPI1.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSPI1_DBG() uint32 {
	return (volatile.LoadUint32(&o.SPI1.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSPI1_DMA(value uint32) {
	volatile.StoreUint32(&o.SPI1.Reg, volatile.LoadUint32(&o.SPI1.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSPI1_DMA() uint32 {
	return (volatile.LoadUint32(&o.SPI1.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSPI1_CORE1(value uint32) {
	volatile.StoreUint32(&o.SPI1.Reg, volatile.LoadUint32(&o.SPI1.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSPI1_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SPI1.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSPI1_CORE0(value uint32) {
	volatile.StoreUint32(&o.SPI1.Reg, volatile.LoadUint32(&o.SPI1.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSPI1_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SPI1.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSPI1_SP(value uint32) {
	volatile.StoreUint32(&o.SPI1.Reg, volatile.LoadUint32(&o.SPI1.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSPI1_SP() uint32 {
	return (volatile.LoadUint32(&o.SPI1.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSPI1_SU(value uint32) {
	volatile.StoreUint32(&o.SPI1.Reg, volatile.LoadUint32(&o.SPI1.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSPI1_SU() uint32 {
	return (volatile.LoadUint32(&o.SPI1.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSPI1_NSP(value uint32) {
	volatile.StoreUint32(&o.SPI1.Reg, volatile.LoadUint32(&o.SPI1.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSPI1_NSP() uint32 {
	return (volatile.LoadUint32(&o.SPI1.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSPI1_NSU(value uint32) {
	volatile.StoreUint32(&o.SPI1.Reg, volatile.LoadUint32(&o.SPI1.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSPI1_NSU() uint32 {
	return volatile.LoadUint32(&o.SPI1.Reg) & 0x1
}

// ACCESSCTRL.TIMER0: Control whether debugger, DMA, core 0 and core 1 can access TIMER0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetTIMER0_DBG(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetTIMER0_DBG() uint32 {
	return (volatile.LoadUint32(&o.TIMER0.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetTIMER0_DMA(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetTIMER0_DMA() uint32 {
	return (volatile.LoadUint32(&o.TIMER0.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetTIMER0_CORE1(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetTIMER0_CORE1() uint32 {
	return (volatile.LoadUint32(&o.TIMER0.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetTIMER0_CORE0(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetTIMER0_CORE0() uint32 {
	return (volatile.LoadUint32(&o.TIMER0.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetTIMER0_SP(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetTIMER0_SP() uint32 {
	return (volatile.LoadUint32(&o.TIMER0.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetTIMER0_SU(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetTIMER0_SU() uint32 {
	return (volatile.LoadUint32(&o.TIMER0.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetTIMER0_NSP(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetTIMER0_NSP() uint32 {
	return (volatile.LoadUint32(&o.TIMER0.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetTIMER0_NSU(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetTIMER0_NSU() uint32 {
	return volatile.LoadUint32(&o.TIMER0.Reg) & 0x1
}

// ACCESSCTRL.TIMER1: Control whether debugger, DMA, core 0 and core 1 can access TIMER1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetTIMER1_DBG(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetTIMER1_DBG() uint32 {
	return (volatile.LoadUint32(&o.TIMER1.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetTIMER1_DMA(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetTIMER1_DMA() uint32 {
	return (volatile.LoadUint32(&o.TIMER1.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetTIMER1_CORE1(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetTIMER1_CORE1() uint32 {
	return (volatile.LoadUint32(&o.TIMER1.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetTIMER1_CORE0(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetTIMER1_CORE0() uint32 {
	return (volatile.LoadUint32(&o.TIMER1.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetTIMER1_SP(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetTIMER1_SP() uint32 {
	return (volatile.LoadUint32(&o.TIMER1.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetTIMER1_SU(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetTIMER1_SU() uint32 {
	return (volatile.LoadUint32(&o.TIMER1.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetTIMER1_NSP(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetTIMER1_NSP() uint32 {
	return (volatile.LoadUint32(&o.TIMER1.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetTIMER1_NSU(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetTIMER1_NSU() uint32 {
	return volatile.LoadUint32(&o.TIMER1.Reg) & 0x1
}

// ACCESSCTRL.UART0: Control whether debugger, DMA, core 0 and core 1 can access UART0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetUART0_DBG(value uint32) {
	volatile.StoreUint32(&o.UART0.Reg, volatile.LoadUint32(&o.UART0.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetUART0_DBG() uint32 {
	return (volatile.LoadUint32(&o.UART0.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetUART0_DMA(value uint32) {
	volatile.StoreUint32(&o.UART0.Reg, volatile.LoadUint32(&o.UART0.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetUART0_DMA() uint32 {
	return (volatile.LoadUint32(&o.UART0.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetUART0_CORE1(value uint32) {
	volatile.StoreUint32(&o.UART0.Reg, volatile.LoadUint32(&o.UART0.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetUART0_CORE1() uint32 {
	return (volatile.LoadUint32(&o.UART0.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetUART0_CORE0(value uint32) {
	volatile.StoreUint32(&o.UART0.Reg, volatile.LoadUint32(&o.UART0.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetUART0_CORE0() uint32 {
	return (volatile.LoadUint32(&o.UART0.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetUART0_SP(value uint32) {
	volatile.StoreUint32(&o.UART0.Reg, volatile.LoadUint32(&o.UART0.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetUART0_SP() uint32 {
	return (volatile.LoadUint32(&o.UART0.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetUART0_SU(value uint32) {
	volatile.StoreUint32(&o.UART0.Reg, volatile.LoadUint32(&o.UART0.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetUART0_SU() uint32 {
	return (volatile.LoadUint32(&o.UART0.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetUART0_NSP(value uint32) {
	volatile.StoreUint32(&o.UART0.Reg, volatile.LoadUint32(&o.UART0.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetUART0_NSP() uint32 {
	return (volatile.LoadUint32(&o.UART0.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetUART0_NSU(value uint32) {
	volatile.StoreUint32(&o.UART0.Reg, volatile.LoadUint32(&o.UART0.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetUART0_NSU() uint32 {
	return volatile.LoadUint32(&o.UART0.Reg) & 0x1
}

// ACCESSCTRL.UART1: Control whether debugger, DMA, core 0 and core 1 can access UART1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetUART1_DBG(value uint32) {
	volatile.StoreUint32(&o.UART1.Reg, volatile.LoadUint32(&o.UART1.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetUART1_DBG() uint32 {
	return (volatile.LoadUint32(&o.UART1.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetUART1_DMA(value uint32) {
	volatile.StoreUint32(&o.UART1.Reg, volatile.LoadUint32(&o.UART1.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetUART1_DMA() uint32 {
	return (volatile.LoadUint32(&o.UART1.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetUART1_CORE1(value uint32) {
	volatile.StoreUint32(&o.UART1.Reg, volatile.LoadUint32(&o.UART1.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetUART1_CORE1() uint32 {
	return (volatile.LoadUint32(&o.UART1.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetUART1_CORE0(value uint32) {
	volatile.StoreUint32(&o.UART1.Reg, volatile.LoadUint32(&o.UART1.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetUART1_CORE0() uint32 {
	return (volatile.LoadUint32(&o.UART1.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetUART1_SP(value uint32) {
	volatile.StoreUint32(&o.UART1.Reg, volatile.LoadUint32(&o.UART1.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetUART1_SP() uint32 {
	return (volatile.LoadUint32(&o.UART1.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetUART1_SU(value uint32) {
	volatile.StoreUint32(&o.UART1.Reg, volatile.LoadUint32(&o.UART1.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetUART1_SU() uint32 {
	return (volatile.LoadUint32(&o.UART1.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetUART1_NSP(value uint32) {
	volatile.StoreUint32(&o.UART1.Reg, volatile.LoadUint32(&o.UART1.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetUART1_NSP() uint32 {
	return (volatile.LoadUint32(&o.UART1.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetUART1_NSU(value uint32) {
	volatile.StoreUint32(&o.UART1.Reg, volatile.LoadUint32(&o.UART1.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetUART1_NSU() uint32 {
	return volatile.LoadUint32(&o.UART1.Reg) & 0x1
}

// ACCESSCTRL.OTP: Control whether debugger, DMA, core 0 and core 1 can access OTP, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetOTP_DBG(value uint32) {
	volatile.StoreUint32(&o.OTP.Reg, volatile.LoadUint32(&o.OTP.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetOTP_DBG() uint32 {
	return (volatile.LoadUint32(&o.OTP.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetOTP_DMA(value uint32) {
	volatile.StoreUint32(&o.OTP.Reg, volatile.LoadUint32(&o.OTP.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetOTP_DMA() uint32 {
	return (volatile.LoadUint32(&o.OTP.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetOTP_CORE1(value uint32) {
	volatile.StoreUint32(&o.OTP.Reg, volatile.LoadUint32(&o.OTP.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetOTP_CORE1() uint32 {
	return (volatile.LoadUint32(&o.OTP.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetOTP_CORE0(value uint32) {
	volatile.StoreUint32(&o.OTP.Reg, volatile.LoadUint32(&o.OTP.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetOTP_CORE0() uint32 {
	return (volatile.LoadUint32(&o.OTP.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetOTP_SP(value uint32) {
	volatile.StoreUint32(&o.OTP.Reg, volatile.LoadUint32(&o.OTP.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetOTP_SP() uint32 {
	return (volatile.LoadUint32(&o.OTP.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetOTP_SU(value uint32) {
	volatile.StoreUint32(&o.OTP.Reg, volatile.LoadUint32(&o.OTP.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetOTP_SU() uint32 {
	return (volatile.LoadUint32(&o.OTP.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetOTP_NSP(value uint32) {
	volatile.StoreUint32(&o.OTP.Reg, volatile.LoadUint32(&o.OTP.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetOTP_NSP() uint32 {
	return (volatile.LoadUint32(&o.OTP.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetOTP_NSU(value uint32) {
	volatile.StoreUint32(&o.OTP.Reg, volatile.LoadUint32(&o.OTP.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetOTP_NSU() uint32 {
	return volatile.LoadUint32(&o.OTP.Reg) & 0x1
}

// ACCESSCTRL.TBMAN: Control whether debugger, DMA, core 0 and core 1 can access TBMAN, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetTBMAN_DBG(value uint32) {
	volatile.StoreUint32(&o.TBMAN.Reg, volatile.LoadUint32(&o.TBMAN.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetTBMAN_DBG() uint32 {
	return (volatile.LoadUint32(&o.TBMAN.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetTBMAN_DMA(value uint32) {
	volatile.StoreUint32(&o.TBMAN.Reg, volatile.LoadUint32(&o.TBMAN.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetTBMAN_DMA() uint32 {
	return (volatile.LoadUint32(&o.TBMAN.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetTBMAN_CORE1(value uint32) {
	volatile.StoreUint32(&o.TBMAN.Reg, volatile.LoadUint32(&o.TBMAN.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetTBMAN_CORE1() uint32 {
	return (volatile.LoadUint32(&o.TBMAN.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetTBMAN_CORE0(value uint32) {
	volatile.StoreUint32(&o.TBMAN.Reg, volatile.LoadUint32(&o.TBMAN.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetTBMAN_CORE0() uint32 {
	return (volatile.LoadUint32(&o.TBMAN.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetTBMAN_SP(value uint32) {
	volatile.StoreUint32(&o.TBMAN.Reg, volatile.LoadUint32(&o.TBMAN.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetTBMAN_SP() uint32 {
	return (volatile.LoadUint32(&o.TBMAN.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetTBMAN_SU(value uint32) {
	volatile.StoreUint32(&o.TBMAN.Reg, volatile.LoadUint32(&o.TBMAN.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetTBMAN_SU() uint32 {
	return (volatile.LoadUint32(&o.TBMAN.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetTBMAN_NSP(value uint32) {
	volatile.StoreUint32(&o.TBMAN.Reg, volatile.LoadUint32(&o.TBMAN.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetTBMAN_NSP() uint32 {
	return (volatile.LoadUint32(&o.TBMAN.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetTBMAN_NSU(value uint32) {
	volatile.StoreUint32(&o.TBMAN.Reg, volatile.LoadUint32(&o.TBMAN.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetTBMAN_NSU() uint32 {
	return volatile.LoadUint32(&o.TBMAN.Reg) & 0x1
}

// ACCESSCTRL.POWMAN: Control whether debugger, DMA, core 0 and core 1 can access POWMAN, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetPOWMAN_DBG(value uint32) {
	volatile.StoreUint32(&o.POWMAN.Reg, volatile.LoadUint32(&o.POWMAN.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetPOWMAN_DBG() uint32 {
	return (volatile.LoadUint32(&o.POWMAN.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetPOWMAN_DMA(value uint32) {
	volatile.StoreUint32(&o.POWMAN.Reg, volatile.LoadUint32(&o.POWMAN.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetPOWMAN_DMA() uint32 {
	return (volatile.LoadUint32(&o.POWMAN.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetPOWMAN_CORE1(value uint32) {
	volatile.StoreUint32(&o.POWMAN.Reg, volatile.LoadUint32(&o.POWMAN.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetPOWMAN_CORE1() uint32 {
	return (volatile.LoadUint32(&o.POWMAN.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetPOWMAN_CORE0(value uint32) {
	volatile.StoreUint32(&o.POWMAN.Reg, volatile.LoadUint32(&o.POWMAN.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetPOWMAN_CORE0() uint32 {
	return (volatile.LoadUint32(&o.POWMAN.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetPOWMAN_SP(value uint32) {
	volatile.StoreUint32(&o.POWMAN.Reg, volatile.LoadUint32(&o.POWMAN.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetPOWMAN_SP() uint32 {
	return (volatile.LoadUint32(&o.POWMAN.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetPOWMAN_SU(value uint32) {
	volatile.StoreUint32(&o.POWMAN.Reg, volatile.LoadUint32(&o.POWMAN.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetPOWMAN_SU() uint32 {
	return (volatile.LoadUint32(&o.POWMAN.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetPOWMAN_NSP(value uint32) {
	volatile.StoreUint32(&o.POWMAN.Reg, volatile.LoadUint32(&o.POWMAN.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetPOWMAN_NSP() uint32 {
	return (volatile.LoadUint32(&o.POWMAN.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetPOWMAN_NSU(value uint32) {
	volatile.StoreUint32(&o.POWMAN.Reg, volatile.LoadUint32(&o.POWMAN.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetPOWMAN_NSU() uint32 {
	return volatile.LoadUint32(&o.POWMAN.Reg) & 0x1
}

// ACCESSCTRL.TRNG: Control whether debugger, DMA, core 0 and core 1 can access TRNG, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetTRNG_DBG(value uint32) {
	volatile.StoreUint32(&o.TRNG.Reg, volatile.LoadUint32(&o.TRNG.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetTRNG_DBG() uint32 {
	return (volatile.LoadUint32(&o.TRNG.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetTRNG_DMA(value uint32) {
	volatile.StoreUint32(&o.TRNG.Reg, volatile.LoadUint32(&o.TRNG.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetTRNG_DMA() uint32 {
	return (volatile.LoadUint32(&o.TRNG.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetTRNG_CORE1(value uint32) {
	volatile.StoreUint32(&o.TRNG.Reg, volatile.LoadUint32(&o.TRNG.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetTRNG_CORE1() uint32 {
	return (volatile.LoadUint32(&o.TRNG.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetTRNG_CORE0(value uint32) {
	volatile.StoreUint32(&o.TRNG.Reg, volatile.LoadUint32(&o.TRNG.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetTRNG_CORE0() uint32 {
	return (volatile.LoadUint32(&o.TRNG.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetTRNG_SP(value uint32) {
	volatile.StoreUint32(&o.TRNG.Reg, volatile.LoadUint32(&o.TRNG.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetTRNG_SP() uint32 {
	return (volatile.LoadUint32(&o.TRNG.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetTRNG_SU(value uint32) {
	volatile.StoreUint32(&o.TRNG.Reg, volatile.LoadUint32(&o.TRNG.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetTRNG_SU() uint32 {
	return (volatile.LoadUint32(&o.TRNG.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetTRNG_NSP(value uint32) {
	volatile.StoreUint32(&o.TRNG.Reg, volatile.LoadUint32(&o.TRNG.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetTRNG_NSP() uint32 {
	return (volatile.LoadUint32(&o.TRNG.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetTRNG_NSU(value uint32) {
	volatile.StoreUint32(&o.TRNG.Reg, volatile.LoadUint32(&o.TRNG.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetTRNG_NSU() uint32 {
	return volatile.LoadUint32(&o.TRNG.Reg) & 0x1
}

// ACCESSCTRL.SHA256: Control whether debugger, DMA, core 0 and core 1 can access SHA256, and at what security/privilege levels they can do so. Defaults to Secure, Privileged access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSHA256_DBG(value uint32) {
	volatile.StoreUint32(&o.SHA256.Reg, volatile.LoadUint32(&o.SHA256.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSHA256_DBG() uint32 {
	return (volatile.LoadUint32(&o.SHA256.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSHA256_DMA(value uint32) {
	volatile.StoreUint32(&o.SHA256.Reg, volatile.LoadUint32(&o.SHA256.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSHA256_DMA() uint32 {
	return (volatile.LoadUint32(&o.SHA256.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSHA256_CORE1(value uint32) {
	volatile.StoreUint32(&o.SHA256.Reg, volatile.LoadUint32(&o.SHA256.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSHA256_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SHA256.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSHA256_CORE0(value uint32) {
	volatile.StoreUint32(&o.SHA256.Reg, volatile.LoadUint32(&o.SHA256.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSHA256_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SHA256.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSHA256_SP(value uint32) {
	volatile.StoreUint32(&o.SHA256.Reg, volatile.LoadUint32(&o.SHA256.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSHA256_SP() uint32 {
	return (volatile.LoadUint32(&o.SHA256.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSHA256_SU(value uint32) {
	volatile.StoreUint32(&o.SHA256.Reg, volatile.LoadUint32(&o.SHA256.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSHA256_SU() uint32 {
	return (volatile.LoadUint32(&o.SHA256.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSHA256_NSP(value uint32) {
	volatile.StoreUint32(&o.SHA256.Reg, volatile.LoadUint32(&o.SHA256.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSHA256_NSP() uint32 {
	return (volatile.LoadUint32(&o.SHA256.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSHA256_NSU(value uint32) {
	volatile.StoreUint32(&o.SHA256.Reg, volatile.LoadUint32(&o.SHA256.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSHA256_NSU() uint32 {
	return volatile.LoadUint32(&o.SHA256.Reg) & 0x1
}

// ACCESSCTRL.SYSCFG: Control whether debugger, DMA, core 0 and core 1 can access SYSCFG, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetSYSCFG_DBG(value uint32) {
	volatile.StoreUint32(&o.SYSCFG.Reg, volatile.LoadUint32(&o.SYSCFG.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetSYSCFG_DBG() uint32 {
	return (volatile.LoadUint32(&o.SYSCFG.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetSYSCFG_DMA(value uint32) {
	volatile.StoreUint32(&o.SYSCFG.Reg, volatile.LoadUint32(&o.SYSCFG.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetSYSCFG_DMA() uint32 {
	return (volatile.LoadUint32(&o.SYSCFG.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetSYSCFG_CORE1(value uint32) {
	volatile.StoreUint32(&o.SYSCFG.Reg, volatile.LoadUint32(&o.SYSCFG.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetSYSCFG_CORE1() uint32 {
	return (volatile.LoadUint32(&o.SYSCFG.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetSYSCFG_CORE0(value uint32) {
	volatile.StoreUint32(&o.SYSCFG.Reg, volatile.LoadUint32(&o.SYSCFG.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetSYSCFG_CORE0() uint32 {
	return (volatile.LoadUint32(&o.SYSCFG.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetSYSCFG_SP(value uint32) {
	volatile.StoreUint32(&o.SYSCFG.Reg, volatile.LoadUint32(&o.SYSCFG.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetSYSCFG_SP() uint32 {
	return (volatile.LoadUint32(&o.SYSCFG.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetSYSCFG_SU(value uint32) {
	volatile.StoreUint32(&o.SYSCFG.Reg, volatile.LoadUint32(&o.SYSCFG.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetSYSCFG_SU() uint32 {
	return (volatile.LoadUint32(&o.SYSCFG.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetSYSCFG_NSP(value uint32) {
	volatile.StoreUint32(&o.SYSCFG.Reg, volatile.LoadUint32(&o.SYSCFG.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetSYSCFG_NSP() uint32 {
	return (volatile.LoadUint32(&o.SYSCFG.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetSYSCFG_NSU(value uint32) {
	volatile.StoreUint32(&o.SYSCFG.Reg, volatile.LoadUint32(&o.SYSCFG.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetSYSCFG_NSU() uint32 {
	return volatile.LoadUint32(&o.SYSCFG.Reg) & 0x1
}

// ACCESSCTRL.CLOCKS: Control whether debugger, DMA, core 0 and core 1 can access CLOCKS, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetCLOCKS_DBG(value uint32) {
	volatile.StoreUint32(&o.CLOCKS.Reg, volatile.LoadUint32(&o.CLOCKS.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetCLOCKS_DBG() uint32 {
	return (volatile.LoadUint32(&o.CLOCKS.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetCLOCKS_DMA(value uint32) {
	volatile.StoreUint32(&o.CLOCKS.Reg, volatile.LoadUint32(&o.CLOCKS.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetCLOCKS_DMA() uint32 {
	return (volatile.LoadUint32(&o.CLOCKS.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetCLOCKS_CORE1(value uint32) {
	volatile.StoreUint32(&o.CLOCKS.Reg, volatile.LoadUint32(&o.CLOCKS.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetCLOCKS_CORE1() uint32 {
	return (volatile.LoadUint32(&o.CLOCKS.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetCLOCKS_CORE0(value uint32) {
	volatile.StoreUint32(&o.CLOCKS.Reg, volatile.LoadUint32(&o.CLOCKS.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetCLOCKS_CORE0() uint32 {
	return (volatile.LoadUint32(&o.CLOCKS.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetCLOCKS_SP(value uint32) {
	volatile.StoreUint32(&o.CLOCKS.Reg, volatile.LoadUint32(&o.CLOCKS.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetCLOCKS_SP() uint32 {
	return (volatile.LoadUint32(&o.CLOCKS.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetCLOCKS_SU(value uint32) {
	volatile.StoreUint32(&o.CLOCKS.Reg, volatile.LoadUint32(&o.CLOCKS.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetCLOCKS_SU() uint32 {
	return (volatile.LoadUint32(&o.CLOCKS.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetCLOCKS_NSP(value uint32) {
	volatile.StoreUint32(&o.CLOCKS.Reg, volatile.LoadUint32(&o.CLOCKS.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetCLOCKS_NSP() uint32 {
	return (volatile.LoadUint32(&o.CLOCKS.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetCLOCKS_NSU(value uint32) {
	volatile.StoreUint32(&o.CLOCKS.Reg, volatile.LoadUint32(&o.CLOCKS.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetCLOCKS_NSU() uint32 {
	return volatile.LoadUint32(&o.CLOCKS.Reg) & 0x1
}

// ACCESSCTRL.XOSC: Control whether debugger, DMA, core 0 and core 1 can access XOSC, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetXOSC_DBG(value uint32) {
	volatile.StoreUint32(&o.XOSC.Reg, volatile.LoadUint32(&o.XOSC.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetXOSC_DBG() uint32 {
	return (volatile.LoadUint32(&o.XOSC.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetXOSC_DMA(value uint32) {
	volatile.StoreUint32(&o.XOSC.Reg, volatile.LoadUint32(&o.XOSC.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetXOSC_DMA() uint32 {
	return (volatile.LoadUint32(&o.XOSC.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetXOSC_CORE1(value uint32) {
	volatile.StoreUint32(&o.XOSC.Reg, volatile.LoadUint32(&o.XOSC.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetXOSC_CORE1() uint32 {
	return (volatile.LoadUint32(&o.XOSC.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetXOSC_CORE0(value uint32) {
	volatile.StoreUint32(&o.XOSC.Reg, volatile.LoadUint32(&o.XOSC.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetXOSC_CORE0() uint32 {
	return (volatile.LoadUint32(&o.XOSC.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetXOSC_SP(value uint32) {
	volatile.StoreUint32(&o.XOSC.Reg, volatile.LoadUint32(&o.XOSC.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetXOSC_SP() uint32 {
	return (volatile.LoadUint32(&o.XOSC.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetXOSC_SU(value uint32) {
	volatile.StoreUint32(&o.XOSC.Reg, volatile.LoadUint32(&o.XOSC.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetXOSC_SU() uint32 {
	return (volatile.LoadUint32(&o.XOSC.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetXOSC_NSP(value uint32) {
	volatile.StoreUint32(&o.XOSC.Reg, volatile.LoadUint32(&o.XOSC.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetXOSC_NSP() uint32 {
	return (volatile.LoadUint32(&o.XOSC.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetXOSC_NSU(value uint32) {
	volatile.StoreUint32(&o.XOSC.Reg, volatile.LoadUint32(&o.XOSC.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetXOSC_NSU() uint32 {
	return volatile.LoadUint32(&o.XOSC.Reg) & 0x1
}

// ACCESSCTRL.ROSC: Control whether debugger, DMA, core 0 and core 1 can access ROSC, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetROSC_DBG(value uint32) {
	volatile.StoreUint32(&o.ROSC.Reg, volatile.LoadUint32(&o.ROSC.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetROSC_DBG() uint32 {
	return (volatile.LoadUint32(&o.ROSC.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetROSC_DMA(value uint32) {
	volatile.StoreUint32(&o.ROSC.Reg, volatile.LoadUint32(&o.ROSC.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetROSC_DMA() uint32 {
	return (volatile.LoadUint32(&o.ROSC.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetROSC_CORE1(value uint32) {
	volatile.StoreUint32(&o.ROSC.Reg, volatile.LoadUint32(&o.ROSC.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetROSC_CORE1() uint32 {
	return (volatile.LoadUint32(&o.ROSC.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetROSC_CORE0(value uint32) {
	volatile.StoreUint32(&o.ROSC.Reg, volatile.LoadUint32(&o.ROSC.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetROSC_CORE0() uint32 {
	return (volatile.LoadUint32(&o.ROSC.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetROSC_SP(value uint32) {
	volatile.StoreUint32(&o.ROSC.Reg, volatile.LoadUint32(&o.ROSC.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetROSC_SP() uint32 {
	return (volatile.LoadUint32(&o.ROSC.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetROSC_SU(value uint32) {
	volatile.StoreUint32(&o.ROSC.Reg, volatile.LoadUint32(&o.ROSC.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetROSC_SU() uint32 {
	return (volatile.LoadUint32(&o.ROSC.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetROSC_NSP(value uint32) {
	volatile.StoreUint32(&o.ROSC.Reg, volatile.LoadUint32(&o.ROSC.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetROSC_NSP() uint32 {
	return (volatile.LoadUint32(&o.ROSC.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetROSC_NSU(value uint32) {
	volatile.StoreUint32(&o.ROSC.Reg, volatile.LoadUint32(&o.ROSC.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetROSC_NSU() uint32 {
	return volatile.LoadUint32(&o.ROSC.Reg) & 0x1
}

// ACCESSCTRL.PLL_SYS: Control whether debugger, DMA, core 0 and core 1 can access PLL_SYS, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetPLL_SYS_DBG(value uint32) {
	volatile.StoreUint32(&o.PLL_SYS.Reg, volatile.LoadUint32(&o.PLL_SYS.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetPLL_SYS_DBG() uint32 {
	return (volatile.LoadUint32(&o.PLL_SYS.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetPLL_SYS_DMA(value uint32) {
	volatile.StoreUint32(&o.PLL_SYS.Reg, volatile.LoadUint32(&o.PLL_SYS.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetPLL_SYS_DMA() uint32 {
	return (volatile.LoadUint32(&o.PLL_SYS.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetPLL_SYS_CORE1(value uint32) {
	volatile.StoreUint32(&o.PLL_SYS.Reg, volatile.LoadUint32(&o.PLL_SYS.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetPLL_SYS_CORE1() uint32 {
	return (volatile.LoadUint32(&o.PLL_SYS.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetPLL_SYS_CORE0(value uint32) {
	volatile.StoreUint32(&o.PLL_SYS.Reg, volatile.LoadUint32(&o.PLL_SYS.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetPLL_SYS_CORE0() uint32 {
	return (volatile.LoadUint32(&o.PLL_SYS.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetPLL_SYS_SP(value uint32) {
	volatile.StoreUint32(&o.PLL_SYS.Reg, volatile.LoadUint32(&o.PLL_SYS.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetPLL_SYS_SP() uint32 {
	return (volatile.LoadUint32(&o.PLL_SYS.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetPLL_SYS_SU(value uint32) {
	volatile.StoreUint32(&o.PLL_SYS.Reg, volatile.LoadUint32(&o.PLL_SYS.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetPLL_SYS_SU() uint32 {
	return (volatile.LoadUint32(&o.PLL_SYS.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetPLL_SYS_NSP(value uint32) {
	volatile.StoreUint32(&o.PLL_SYS.Reg, volatile.LoadUint32(&o.PLL_SYS.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetPLL_SYS_NSP() uint32 {
	return (volatile.LoadUint32(&o.PLL_SYS.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetPLL_SYS_NSU(value uint32) {
	volatile.StoreUint32(&o.PLL_SYS.Reg, volatile.LoadUint32(&o.PLL_SYS.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetPLL_SYS_NSU() uint32 {
	return volatile.LoadUint32(&o.PLL_SYS.Reg) & 0x1
}

// ACCESSCTRL.PLL_USB: Control whether debugger, DMA, core 0 and core 1 can access PLL_USB, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetPLL_USB_DBG(value uint32) {
	volatile.StoreUint32(&o.PLL_USB.Reg, volatile.LoadUint32(&o.PLL_USB.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetPLL_USB_DBG() uint32 {
	return (volatile.LoadUint32(&o.PLL_USB.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetPLL_USB_DMA(value uint32) {
	volatile.StoreUint32(&o.PLL_USB.Reg, volatile.LoadUint32(&o.PLL_USB.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetPLL_USB_DMA() uint32 {
	return (volatile.LoadUint32(&o.PLL_USB.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetPLL_USB_CORE1(value uint32) {
	volatile.StoreUint32(&o.PLL_USB.Reg, volatile.LoadUint32(&o.PLL_USB.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetPLL_USB_CORE1() uint32 {
	return (volatile.LoadUint32(&o.PLL_USB.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetPLL_USB_CORE0(value uint32) {
	volatile.StoreUint32(&o.PLL_USB.Reg, volatile.LoadUint32(&o.PLL_USB.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetPLL_USB_CORE0() uint32 {
	return (volatile.LoadUint32(&o.PLL_USB.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetPLL_USB_SP(value uint32) {
	volatile.StoreUint32(&o.PLL_USB.Reg, volatile.LoadUint32(&o.PLL_USB.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetPLL_USB_SP() uint32 {
	return (volatile.LoadUint32(&o.PLL_USB.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetPLL_USB_SU(value uint32) {
	volatile.StoreUint32(&o.PLL_USB.Reg, volatile.LoadUint32(&o.PLL_USB.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetPLL_USB_SU() uint32 {
	return (volatile.LoadUint32(&o.PLL_USB.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetPLL_USB_NSP(value uint32) {
	volatile.StoreUint32(&o.PLL_USB.Reg, volatile.LoadUint32(&o.PLL_USB.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetPLL_USB_NSP() uint32 {
	return (volatile.LoadUint32(&o.PLL_USB.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetPLL_USB_NSU(value uint32) {
	volatile.StoreUint32(&o.PLL_USB.Reg, volatile.LoadUint32(&o.PLL_USB.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetPLL_USB_NSU() uint32 {
	return volatile.LoadUint32(&o.PLL_USB.Reg) & 0x1
}

// ACCESSCTRL.TICKS: Control whether debugger, DMA, core 0 and core 1 can access TICKS, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetTICKS_DBG(value uint32) {
	volatile.StoreUint32(&o.TICKS.Reg, volatile.LoadUint32(&o.TICKS.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetTICKS_DBG() uint32 {
	return (volatile.LoadUint32(&o.TICKS.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetTICKS_DMA(value uint32) {
	volatile.StoreUint32(&o.TICKS.Reg, volatile.LoadUint32(&o.TICKS.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetTICKS_DMA() uint32 {
	return (volatile.LoadUint32(&o.TICKS.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetTICKS_CORE1(value uint32) {
	volatile.StoreUint32(&o.TICKS.Reg, volatile.LoadUint32(&o.TICKS.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetTICKS_CORE1() uint32 {
	return (volatile.LoadUint32(&o.TICKS.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetTICKS_CORE0(value uint32) {
	volatile.StoreUint32(&o.TICKS.Reg, volatile.LoadUint32(&o.TICKS.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetTICKS_CORE0() uint32 {
	return (volatile.LoadUint32(&o.TICKS.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetTICKS_SP(value uint32) {
	volatile.StoreUint32(&o.TICKS.Reg, volatile.LoadUint32(&o.TICKS.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetTICKS_SP() uint32 {
	return (volatile.LoadUint32(&o.TICKS.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetTICKS_SU(value uint32) {
	volatile.StoreUint32(&o.TICKS.Reg, volatile.LoadUint32(&o.TICKS.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetTICKS_SU() uint32 {
	return (volatile.LoadUint32(&o.TICKS.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetTICKS_NSP(value uint32) {
	volatile.StoreUint32(&o.TICKS.Reg, volatile.LoadUint32(&o.TICKS.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetTICKS_NSP() uint32 {
	return (volatile.LoadUint32(&o.TICKS.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetTICKS_NSU(value uint32) {
	volatile.StoreUint32(&o.TICKS.Reg, volatile.LoadUint32(&o.TICKS.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetTICKS_NSU() uint32 {
	return volatile.LoadUint32(&o.TICKS.Reg) & 0x1
}

// ACCESSCTRL.WATCHDOG: Control whether debugger, DMA, core 0 and core 1 can access WATCHDOG, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetWATCHDOG_DBG(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG.Reg, volatile.LoadUint32(&o.WATCHDOG.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetWATCHDOG_DBG() uint32 {
	return (volatile.LoadUint32(&o.WATCHDOG.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetWATCHDOG_DMA(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG.Reg, volatile.LoadUint32(&o.WATCHDOG.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetWATCHDOG_DMA() uint32 {
	return (volatile.LoadUint32(&o.WATCHDOG.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetWATCHDOG_CORE1(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG.Reg, volatile.LoadUint32(&o.WATCHDOG.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetWATCHDOG_CORE1() uint32 {
	return (volatile.LoadUint32(&o.WATCHDOG.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetWATCHDOG_CORE0(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG.Reg, volatile.LoadUint32(&o.WATCHDOG.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetWATCHDOG_CORE0() uint32 {
	return (volatile.LoadUint32(&o.WATCHDOG.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetWATCHDOG_SP(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG.Reg, volatile.LoadUint32(&o.WATCHDOG.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetWATCHDOG_SP() uint32 {
	return (volatile.LoadUint32(&o.WATCHDOG.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetWATCHDOG_SU(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG.Reg, volatile.LoadUint32(&o.WATCHDOG.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetWATCHDOG_SU() uint32 {
	return (volatile.LoadUint32(&o.WATCHDOG.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetWATCHDOG_NSP(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG.Reg, volatile.LoadUint32(&o.WATCHDOG.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetWATCHDOG_NSP() uint32 {
	return (volatile.LoadUint32(&o.WATCHDOG.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetWATCHDOG_NSU(value uint32) {
	volatile.StoreUint32(&o.WATCHDOG.Reg, volatile.LoadUint32(&o.WATCHDOG.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetWATCHDOG_NSU() uint32 {
	return volatile.LoadUint32(&o.WATCHDOG.Reg) & 0x1
}

// ACCESSCTRL.RSM: Control whether debugger, DMA, core 0 and core 1 can access RSM, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetRSM_DBG(value uint32) {
	volatile.StoreUint32(&o.RSM.Reg, volatile.LoadUint32(&o.RSM.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetRSM_DBG() uint32 {
	return (volatile.LoadUint32(&o.RSM.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetRSM_DMA(value uint32) {
	volatile.StoreUint32(&o.RSM.Reg, volatile.LoadUint32(&o.RSM.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetRSM_DMA() uint32 {
	return (volatile.LoadUint32(&o.RSM.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetRSM_CORE1(value uint32) {
	volatile.StoreUint32(&o.RSM.Reg, volatile.LoadUint32(&o.RSM.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetRSM_CORE1() uint32 {
	return (volatile.LoadUint32(&o.RSM.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetRSM_CORE0(value uint32) {
	volatile.StoreUint32(&o.RSM.Reg, volatile.LoadUint32(&o.RSM.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetRSM_CORE0() uint32 {
	return (volatile.LoadUint32(&o.RSM.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetRSM_SP(value uint32) {
	volatile.StoreUint32(&o.RSM.Reg, volatile.LoadUint32(&o.RSM.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetRSM_SP() uint32 {
	return (volatile.LoadUint32(&o.RSM.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetRSM_SU(value uint32) {
	volatile.StoreUint32(&o.RSM.Reg, volatile.LoadUint32(&o.RSM.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetRSM_SU() uint32 {
	return (volatile.LoadUint32(&o.RSM.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetRSM_NSP(value uint32) {
	volatile.StoreUint32(&o.RSM.Reg, volatile.LoadUint32(&o.RSM.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetRSM_NSP() uint32 {
	return (volatile.LoadUint32(&o.RSM.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetRSM_NSU(value uint32) {
	volatile.StoreUint32(&o.RSM.Reg, volatile.LoadUint32(&o.RSM.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetRSM_NSU() uint32 {
	return volatile.LoadUint32(&o.RSM.Reg) & 0x1
}

// ACCESSCTRL.XIP_CTRL: Control whether debugger, DMA, core 0 and core 1 can access XIP_CTRL, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetXIP_CTRL_DBG(value uint32) {
	volatile.StoreUint32(&o.XIP_CTRL.Reg, volatile.LoadUint32(&o.XIP_CTRL.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetXIP_CTRL_DBG() uint32 {
	return (volatile.LoadUint32(&o.XIP_CTRL.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetXIP_CTRL_DMA(value uint32) {
	volatile.StoreUint32(&o.XIP_CTRL.Reg, volatile.LoadUint32(&o.XIP_CTRL.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetXIP_CTRL_DMA() uint32 {
	return (volatile.LoadUint32(&o.XIP_CTRL.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetXIP_CTRL_CORE1(value uint32) {
	volatile.StoreUint32(&o.XIP_CTRL.Reg, volatile.LoadUint32(&o.XIP_CTRL.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetXIP_CTRL_CORE1() uint32 {
	return (volatile.LoadUint32(&o.XIP_CTRL.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetXIP_CTRL_CORE0(value uint32) {
	volatile.StoreUint32(&o.XIP_CTRL.Reg, volatile.LoadUint32(&o.XIP_CTRL.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetXIP_CTRL_CORE0() uint32 {
	return (volatile.LoadUint32(&o.XIP_CTRL.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetXIP_CTRL_SP(value uint32) {
	volatile.StoreUint32(&o.XIP_CTRL.Reg, volatile.LoadUint32(&o.XIP_CTRL.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetXIP_CTRL_SP() uint32 {
	return (volatile.LoadUint32(&o.XIP_CTRL.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetXIP_CTRL_SU(value uint32) {
	volatile.StoreUint32(&o.XIP_CTRL.Reg, volatile.LoadUint32(&o.XIP_CTRL.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetXIP_CTRL_SU() uint32 {
	return (volatile.LoadUint32(&o.XIP_CTRL.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetXIP_CTRL_NSP(value uint32) {
	volatile.StoreUint32(&o.XIP_CTRL.Reg, volatile.LoadUint32(&o.XIP_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetXIP_CTRL_NSP() uint32 {
	return (volatile.LoadUint32(&o.XIP_CTRL.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetXIP_CTRL_NSU(value uint32) {
	volatile.StoreUint32(&o.XIP_CTRL.Reg, volatile.LoadUint32(&o.XIP_CTRL.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetXIP_CTRL_NSU() uint32 {
	return volatile.LoadUint32(&o.XIP_CTRL.Reg) & 0x1
}

// ACCESSCTRL.XIP_QMI: Control whether debugger, DMA, core 0 and core 1 can access XIP_QMI, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetXIP_QMI_DBG(value uint32) {
	volatile.StoreUint32(&o.XIP_QMI.Reg, volatile.LoadUint32(&o.XIP_QMI.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetXIP_QMI_DBG() uint32 {
	return (volatile.LoadUint32(&o.XIP_QMI.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetXIP_QMI_DMA(value uint32) {
	volatile.StoreUint32(&o.XIP_QMI.Reg, volatile.LoadUint32(&o.XIP_QMI.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetXIP_QMI_DMA() uint32 {
	return (volatile.LoadUint32(&o.XIP_QMI.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetXIP_QMI_CORE1(value uint32) {
	volatile.StoreUint32(&o.XIP_QMI.Reg, volatile.LoadUint32(&o.XIP_QMI.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetXIP_QMI_CORE1() uint32 {
	return (volatile.LoadUint32(&o.XIP_QMI.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetXIP_QMI_CORE0(value uint32) {
	volatile.StoreUint32(&o.XIP_QMI.Reg, volatile.LoadUint32(&o.XIP_QMI.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetXIP_QMI_CORE0() uint32 {
	return (volatile.LoadUint32(&o.XIP_QMI.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetXIP_QMI_SP(value uint32) {
	volatile.StoreUint32(&o.XIP_QMI.Reg, volatile.LoadUint32(&o.XIP_QMI.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetXIP_QMI_SP() uint32 {
	return (volatile.LoadUint32(&o.XIP_QMI.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetXIP_QMI_SU(value uint32) {
	volatile.StoreUint32(&o.XIP_QMI.Reg, volatile.LoadUint32(&o.XIP_QMI.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetXIP_QMI_SU() uint32 {
	return (volatile.LoadUint32(&o.XIP_QMI.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetXIP_QMI_NSP(value uint32) {
	volatile.StoreUint32(&o.XIP_QMI.Reg, volatile.LoadUint32(&o.XIP_QMI.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetXIP_QMI_NSP() uint32 {
	return (volatile.LoadUint32(&o.XIP_QMI.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetXIP_QMI_NSU(value uint32) {
	volatile.StoreUint32(&o.XIP_QMI.Reg, volatile.LoadUint32(&o.XIP_QMI.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetXIP_QMI_NSU() uint32 {
	return volatile.LoadUint32(&o.XIP_QMI.Reg) & 0x1
}

// ACCESSCTRL.XIP_AUX: Control whether debugger, DMA, core 0 and core 1 can access XIP_AUX, and at what security/privilege levels they can do so. Defaults to Secure, Privileged access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
func (o *ACCESSCTRL_Type) SetXIP_AUX_DBG(value uint32) {
	volatile.StoreUint32(&o.XIP_AUX.Reg, volatile.LoadUint32(&o.XIP_AUX.Reg)&^(0x80)|value<<7)
}
func (o *ACCESSCTRL_Type) GetXIP_AUX_DBG() uint32 {
	return (volatile.LoadUint32(&o.XIP_AUX.Reg) & 0x80) >> 7
}
func (o *ACCESSCTRL_Type) SetXIP_AUX_DMA(value uint32) {
	volatile.StoreUint32(&o.XIP_AUX.Reg, volatile.LoadUint32(&o.XIP_AUX.Reg)&^(0x40)|value<<6)
}
func (o *ACCESSCTRL_Type) GetXIP_AUX_DMA() uint32 {
	return (volatile.LoadUint32(&o.XIP_AUX.Reg) & 0x40) >> 6
}
func (o *ACCESSCTRL_Type) SetXIP_AUX_CORE1(value uint32) {
	volatile.StoreUint32(&o.XIP_AUX.Reg, volatile.LoadUint32(&o.XIP_AUX.Reg)&^(0x20)|value<<5)
}
func (o *ACCESSCTRL_Type) GetXIP_AUX_CORE1() uint32 {
	return (volatile.LoadUint32(&o.XIP_AUX.Reg) & 0x20) >> 5
}
func (o *ACCESSCTRL_Type) SetXIP_AUX_CORE0(value uint32) {
	volatile.StoreUint32(&o.XIP_AUX.Reg, volatile.LoadUint32(&o.XIP_AUX.Reg)&^(0x10)|value<<4)
}
func (o *ACCESSCTRL_Type) GetXIP_AUX_CORE0() uint32 {
	return (volatile.LoadUint32(&o.XIP_AUX.Reg) & 0x10) >> 4
}
func (o *ACCESSCTRL_Type) SetXIP_AUX_SP(value uint32) {
	volatile.StoreUint32(&o.XIP_AUX.Reg, volatile.LoadUint32(&o.XIP_AUX.Reg)&^(0x8)|value<<3)
}
func (o *ACCESSCTRL_Type) GetXIP_AUX_SP() uint32 {
	return (volatile.LoadUint32(&o.XIP_AUX.Reg) & 0x8) >> 3
}
func (o *ACCESSCTRL_Type) SetXIP_AUX_SU(value uint32) {
	volatile.StoreUint32(&o.XIP_AUX.Reg, volatile.LoadUint32(&o.XIP_AUX.Reg)&^(0x4)|value<<2)
}
func (o *ACCESSCTRL_Type) GetXIP_AUX_SU() uint32 {
	return (volatile.LoadUint32(&o.XIP_AUX.Reg) & 0x4) >> 2
}
func (o *ACCESSCTRL_Type) SetXIP_AUX_NSP(value uint32) {
	volatile.StoreUint32(&o.XIP_AUX.Reg, volatile.LoadUint32(&o.XIP_AUX.Reg)&^(0x2)|value<<1)
}
func (o *ACCESSCTRL_Type) GetXIP_AUX_NSP() uint32 {
	return (volatile.LoadUint32(&o.XIP_AUX.Reg) & 0x2) >> 1
}
func (o *ACCESSCTRL_Type) SetXIP_AUX_NSU(value uint32) {
	volatile.StoreUint32(&o.XIP_AUX.Reg, volatile.LoadUint32(&o.XIP_AUX.Reg)&^(0x1)|value)
}
func (o *ACCESSCTRL_Type) GetXIP_AUX_NSU() uint32 {
	return volatile.LoadUint32(&o.XIP_AUX.Reg) & 0x1
}

type UART0_Type struct {
	UARTDR        volatile.Register32 // 0x0
	UARTRSR       volatile.Register32 // 0x4
	_             [16]byte
	UARTFR        volatile.Register32 // 0x18
	_             [4]byte
	UARTILPR      volatile.Register32 // 0x20
	UARTIBRD      volatile.Register32 // 0x24
	UARTFBRD      volatile.Register32 // 0x28
	UARTLCR_H     volatile.Register32 // 0x2C
	UARTCR        volatile.Register32 // 0x30
	UARTIFLS      volatile.Register32 // 0x34
	UARTIMSC      volatile.Register32 // 0x38
	UARTRIS       volatile.Register32 // 0x3C
	UARTMIS       volatile.Register32 // 0x40
	UARTICR       volatile.Register32 // 0x44
	UARTDMACR     volatile.Register32 // 0x48
	_             [3988]byte
	UARTPERIPHID0 volatile.Register32 // 0xFE0
	UARTPERIPHID1 volatile.Register32 // 0xFE4
	UARTPERIPHID2 volatile.Register32 // 0xFE8
	UARTPERIPHID3 volatile.Register32 // 0xFEC
	UARTPCELLID0  volatile.Register32 // 0xFF0
	UARTPCELLID1  volatile.Register32 // 0xFF4
	UARTPCELLID2  volatile.Register32 // 0xFF8
	UARTPCELLID3  volatile.Register32 // 0xFFC
}

// UART0.UARTDR: Data Register, UARTDR
func (o *UART0_Type) SetUARTDR_OE(value uint32) {
	volatile.StoreUint32(&o.UARTDR.Reg, volatile.LoadUint32(&o.UARTDR.Reg)&^(0x800)|value<<11)
}
func (o *UART0_Type) GetUARTDR_OE() uint32 {
	return (volatile.LoadUint32(&o.UARTDR.Reg) & 0x800) >> 11
}
func (o *UART0_Type) SetUARTDR_BE(value uint32) {
	volatile.StoreUint32(&o.UARTDR.Reg, volatile.LoadUint32(&o.UARTDR.Reg)&^(0x400)|value<<10)
}
func (o *UART0_Type) GetUARTDR_BE() uint32 {
	return (volatile.LoadUint32(&o.UARTDR.Reg) & 0x400) >> 10
}
func (o *UART0_Type) SetUARTDR_PE(value uint32) {
	volatile.StoreUint32(&o.UARTDR.Reg, volatile.LoadUint32(&o.UARTDR.Reg)&^(0x200)|value<<9)
}
func (o *UART0_Type) GetUARTDR_PE() uint32 {
	return (volatile.LoadUint32(&o.UARTDR.Reg) & 0x200) >> 9
}
func (o *UART0_Type) SetUARTDR_FE(value uint32) {
	volatile.StoreUint32(&o.UARTDR.Reg, volatile.LoadUint32(&o.UARTDR.Reg)&^(0x100)|value<<8)
}
func (o *UART0_Type) GetUARTDR_FE() uint32 {
	return (volatile.LoadUint32(&o.UARTDR.Reg) & 0x100) >> 8
}
func (o *UART0_Type) SetUARTDR_DATA(value uint32) {
	volatile.StoreUint32(&o.UARTDR.Reg, volatile.LoadUint32(&o.UARTDR.Reg)&^(0xff)|value)
}
func (o *UART0_Type) GetUARTDR_DATA() uint32 {
	return volatile.LoadUint32(&o.UARTDR.Reg) & 0xff
}

// UART0.UARTRSR: Receive Status Register/Error Clear Register, UARTRSR/UARTECR
func (o *UART0_Type) SetUARTRSR_OE(value uint32) {
	volatile.StoreUint32(&o.UARTRSR.Reg, volatile.LoadUint32(&o.UARTRSR.Reg)&^(0x8)|value<<3)
}
func (o *UART0_Type) GetUARTRSR_OE() uint32 {
	return (volatile.LoadUint32(&o.UARTRSR.Reg) & 0x8) >> 3
}
func (o *UART0_Type) SetUARTRSR_BE(value uint32) {
	volatile.StoreUint32(&o.UARTRSR.Reg, volatile.LoadUint32(&o.UARTRSR.Reg)&^(0x4)|value<<2)
}
func (o *UART0_Type) GetUARTRSR_BE() uint32 {
	return (volatile.LoadUint32(&o.UARTRSR.Reg) & 0x4) >> 2
}
func (o *UART0_Type) SetUARTRSR_PE(value uint32) {
	volatile.StoreUint32(&o.UARTRSR.Reg, volatile.LoadUint32(&o.UARTRSR.Reg)&^(0x2)|value<<1)
}
func (o *UART0_Type) GetUARTRSR_PE() uint32 {
	return (volatile.LoadUint32(&o.UARTRSR.Reg) & 0x2) >> 1
}
func (o *UART0_Type) SetUARTRSR_FE(value uint32) {
	volatile.StoreUint32(&o.UARTRSR.Reg, volatile.LoadUint32(&o.UARTRSR.Reg)&^(0x1)|value)
}
func (o *UART0_Type) GetUARTRSR_FE() uint32 {
	return volatile.LoadUint32(&o.UARTRSR.Reg) & 0x1
}

// UART0.UARTFR: Flag Register, UARTFR
func (o *UART0_Type) SetUARTFR_RI(value uint32) {
	volatile.StoreUint32(&o.UARTFR.Reg, volatile.LoadUint32(&o.UARTFR.Reg)&^(0x100)|value<<8)
}
func (o *UART0_Type) GetUARTFR_RI() uint32 {
	return (volatile.LoadUint32(&o.UARTFR.Reg) & 0x100) >> 8
}
func (o *UART0_Type) SetUARTFR_TXFE(value uint32) {
	volatile.StoreUint32(&o.UARTFR.Reg, volatile.LoadUint32(&o.UARTFR.Reg)&^(0x80)|value<<7)
}
func (o *UART0_Type) GetUARTFR_TXFE() uint32 {
	return (volatile.LoadUint32(&o.UARTFR.Reg) & 0x80) >> 7
}
func (o *UART0_Type) SetUARTFR_RXFF(value uint32) {
	volatile.StoreUint32(&o.UARTFR.Reg, volatile.LoadUint32(&o.UARTFR.Reg)&^(0x40)|value<<6)
}
func (o *UART0_Type) GetUARTFR_RXFF() uint32 {
	return (volatile.LoadUint32(&o.UARTFR.Reg) & 0x40) >> 6
}
func (o *UART0_Type) SetUARTFR_TXFF(value uint32) {
	volatile.StoreUint32(&o.UARTFR.Reg, volatile.LoadUint32(&o.UARTFR.Reg)&^(0x20)|value<<5)
}
func (o *UART0_Type) GetUARTFR_TXFF() uint32 {
	return (volatile.LoadUint32(&o.UARTFR.Reg) & 0x20) >> 5
}
func (o *UART0_Type) SetUARTFR_RXFE(value uint32) {
	volatile.StoreUint32(&o.UARTFR.Reg, volatile.LoadUint32(&o.UARTFR.Reg)&^(0x10)|value<<4)
}
func (o *UART0_Type) GetUARTFR_RXFE() uint32 {
	return (volatile.LoadUint32(&o.UARTFR.Reg) & 0x10) >> 4
}
func (o *UART0_Type) SetUARTFR_BUSY(value uint32) {
	volatile.StoreUint32(&o.UARTFR.Reg, volatile.LoadUint32(&o.UARTFR.Reg)&^(0x8)|value<<3)
}
func (o *UART0_Type) GetUARTFR_BUSY() uint32 {
	return (volatile.LoadUint32(&o.UARTFR.Reg) & 0x8) >> 3
}
func (o *UART0_Type) SetUARTFR_DCD(value uint32) {
	volatile.StoreUint32(&o.UARTFR.Reg, volatile.LoadUint32(&o.UARTFR.Reg)&^(0x4)|value<<2)
}
func (o *UART0_Type) GetUARTFR_DCD() uint32 {
	return (volatile.LoadUint32(&o.UARTFR.Reg) & 0x4) >> 2
}
func (o *UART0_Type) SetUARTFR_DSR(value uint32) {
	volatile.StoreUint32(&o.UARTFR.Reg, volatile.LoadUint32(&o.UARTFR.Reg)&^(0x2)|value<<1)
}
func (o *UART0_Type) GetUARTFR_DSR() uint32 {
	return (volatile.LoadUint32(&o.UARTFR.Reg) & 0x2) >> 1
}
func (o *UART0_Type) SetUARTFR_CTS(value uint32) {
	volatile.StoreUint32(&o.UARTFR.Reg, volatile.LoadUint32(&o.UARTFR.Reg)&^(0x1)|value)
}
func (o *UART0_Type) GetUARTFR_CTS() uint32 {
	return volatile.LoadUint32(&o.UARTFR.Reg) & 0x1
}

// UART0.UARTILPR: IrDA Low-Power Counter Register, UARTILPR
func (o *UART0_Type) SetUARTILPR_ILPDVSR(value uint32) {
	volatile.StoreUint32(&o.UARTILPR.Reg, volatile.LoadUint32(&o.UARTILPR.Reg)&^(0xff)|value)
}
func (o *UART0_Type) GetUARTILPR_ILPDVSR() uint32 {
	return volatile.LoadUint32(&o.UARTILPR.Reg) & 0xff
}

// UART0.UARTIBRD: Integer Baud Rate Register, UARTIBRD
func (o *UART0_Type) SetUARTIBRD_BAUD_DIVINT(value uint32) {
	volatile.StoreUint32(&o.UARTIBRD.Reg, volatile.LoadUint32(&o.UARTIBRD.Reg)&^(0xffff)|value)
}
func (o *UART0_Type) GetUARTIBRD_BAUD_DIVINT() uint32 {
	return volatile.LoadUint32(&o.UARTIBRD.Reg) & 0xffff
}

// UART0.UARTFBRD: Fractional Baud Rate Register, UARTFBRD
func (o *UART0_Type) SetUARTFBRD_BAUD_DIVFRAC(value uint32) {
	volatile.StoreUint32(&o.UARTFBRD.Reg, volatile.LoadUint32(&o.UARTFBRD.Reg)&^(0x3f)|value)
}
func (o *UART0_Type) GetUARTFBRD_BAUD_DIVFRAC() uint32 {
	return volatile.LoadUint32(&o.UARTFBRD.Reg) & 0x3f
}

// UART0.UARTLCR_H: Line Control Register, UARTLCR_H
func (o *UART0_Type) SetUARTLCR_H_SPS(value uint32) {
	volatile.StoreUint32(&o.UARTLCR_H.Reg, volatile.LoadUint32(&o.UARTLCR_H.Reg)&^(0x80)|value<<7)
}
func (o *UART0_Type) GetUARTLCR_H_SPS() uint32 {
	return (volatile.LoadUint32(&o.UARTLCR_H.Reg) & 0x80) >> 7
}
func (o *UART0_Type) SetUARTLCR_H_WLEN(value uint32) {
	volatile.StoreUint32(&o.UARTLCR_H.Reg, volatile.LoadUint32(&o.UARTLCR_H.Reg)&^(0x60)|value<<5)
}
func (o *UART0_Type) GetUARTLCR_H_WLEN() uint32 {
	return (volatile.LoadUint32(&o.UARTLCR_H.Reg) & 0x60) >> 5
}
func (o *UART0_Type) SetUARTLCR_H_FEN(value uint32) {
	volatile.StoreUint32(&o.UARTLCR_H.Reg, volatile.LoadUint32(&o.UARTLCR_H.Reg)&^(0x10)|value<<4)
}
func (o *UART0_Type) GetUARTLCR_H_FEN() uint32 {
	return (volatile.LoadUint32(&o.UARTLCR_H.Reg) & 0x10) >> 4
}
func (o *UART0_Type) SetUARTLCR_H_STP2(value uint32) {
	volatile.StoreUint32(&o.UARTLCR_H.Reg, volatile.LoadUint32(&o.UARTLCR_H.Reg)&^(0x8)|value<<3)
}
func (o *UART0_Type) GetUARTLCR_H_STP2() uint32 {
	return (volatile.LoadUint32(&o.UARTLCR_H.Reg) & 0x8) >> 3
}
func (o *UART0_Type) SetUARTLCR_H_EPS(value uint32) {
	volatile.StoreUint32(&o.UARTLCR_H.Reg, volatile.LoadUint32(&o.UARTLCR_H.Reg)&^(0x4)|value<<2)
}
func (o *UART0_Type) GetUARTLCR_H_EPS() uint32 {
	return (volatile.LoadUint32(&o.UARTLCR_H.Reg) & 0x4) >> 2
}
func (o *UART0_Type) SetUARTLCR_H_PEN(value uint32) {
	volatile.StoreUint32(&o.UARTLCR_H.Reg, volatile.LoadUint32(&o.UARTLCR_H.Reg)&^(0x2)|value<<1)
}
func (o *UART0_Type) GetUARTLCR_H_PEN() uint32 {
	return (volatile.LoadUint32(&o.UARTLCR_H.Reg) & 0x2) >> 1
}
func (o *UART0_Type) SetUARTLCR_H_BRK(value uint32) {
	volatile.StoreUint32(&o.UARTLCR_H.Reg, volatile.LoadUint32(&o.UARTLCR_H.Reg)&^(0x1)|value)
}
func (o *UART0_Type) GetUARTLCR_H_BRK() uint32 {
	return volatile.LoadUint32(&o.UARTLCR_H.Reg) & 0x1
}

// UART0.UARTCR: Control Register, UARTCR
func (o *UART0_Type) SetUARTCR_CTSEN(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x8000)|value<<15)
}
func (o *UART0_Type) GetUARTCR_CTSEN() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x8000) >> 15
}
func (o *UART0_Type) SetUARTCR_RTSEN(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x4000)|value<<14)
}
func (o *UART0_Type) GetUARTCR_RTSEN() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x4000) >> 14
}
func (o *UART0_Type) SetUARTCR_OUT2(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x2000)|value<<13)
}
func (o *UART0_Type) GetUARTCR_OUT2() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x2000) >> 13
}
func (o *UART0_Type) SetUARTCR_OUT1(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x1000)|value<<12)
}
func (o *UART0_Type) GetUARTCR_OUT1() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x1000) >> 12
}
func (o *UART0_Type) SetUARTCR_RTS(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x800)|value<<11)
}
func (o *UART0_Type) GetUARTCR_RTS() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x800) >> 11
}
func (o *UART0_Type) SetUARTCR_DTR(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x400)|value<<10)
}
func (o *UART0_Type) GetUARTCR_DTR() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x400) >> 10
}
func (o *UART0_Type) SetUARTCR_RXE(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x200)|value<<9)
}
func (o *UART0_Type) GetUARTCR_RXE() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x200) >> 9
}
func (o *UART0_Type) SetUARTCR_TXE(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x100)|value<<8)
}
func (o *UART0_Type) GetUARTCR_TXE() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x100) >> 8
}
func (o *UART0_Type) SetUARTCR_LBE(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x80)|value<<7)
}
func (o *UART0_Type) GetUARTCR_LBE() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x80) >> 7
}
func (o *UART0_Type) SetUARTCR_SIRLP(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x4)|value<<2)
}
func (o *UART0_Type) GetUARTCR_SIRLP() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x4) >> 2
}
func (o *UART0_Type) SetUARTCR_SIREN(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x2)|value<<1)
}
func (o *UART0_Type) GetUARTCR_SIREN() uint32 {
	return (volatile.LoadUint32(&o.UARTCR.Reg) & 0x2) >> 1
}
func (o *UART0_Type) SetUARTCR_UARTEN(value uint32) {
	volatile.StoreUint32(&o.UARTCR.Reg, volatile.LoadUint32(&o.UARTCR.Reg)&^(0x1)|value)
}
func (o *UART0_Type) GetUARTCR_UARTEN() uint32 {
	return volatile.LoadUint32(&o.UARTCR.Reg) & 0x1
}

// UART0.UARTIFLS: Interrupt FIFO Level Select Register, UARTIFLS
func (o *UART0_Type) SetUARTIFLS_RXIFLSEL(value uint32) {
	volatile.StoreUint32(&o.UARTIFLS.Reg, volatile.LoadUint32(&o.UARTIFLS.Reg)&^(0x38)|value<<3)
}
func (o *UART0_Type) GetUARTIFLS_RXIFLSEL() uint32 {
	return (volatile.LoadUint32(&o.UARTIFLS.Reg) & 0x38) >> 3
}
func (o *UART0_Type) SetUARTIFLS_TXIFLSEL(value uint32) {
	volatile.StoreUint32(&o.UARTIFLS.Reg, volatile.LoadUint32(&o.UARTIFLS.Reg)&^(0x7)|value)
}
func (o *UART0_Type) GetUARTIFLS_TXIFLSEL() uint32 {
	return volatile.LoadUint32(&o.UARTIFLS.Reg) & 0x7
}

// UART0.UARTIMSC: Interrupt Mask Set/Clear Register, UARTIMSC
func (o *UART0_Type) SetUARTIMSC_OEIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x400)|value<<10)
}
func (o *UART0_Type) GetUARTIMSC_OEIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x400) >> 10
}
func (o *UART0_Type) SetUARTIMSC_BEIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x200)|value<<9)
}
func (o *UART0_Type) GetUARTIMSC_BEIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x200) >> 9
}
func (o *UART0_Type) SetUARTIMSC_PEIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x100)|value<<8)
}
func (o *UART0_Type) GetUARTIMSC_PEIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x100) >> 8
}
func (o *UART0_Type) SetUARTIMSC_FEIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x80)|value<<7)
}
func (o *UART0_Type) GetUARTIMSC_FEIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x80) >> 7
}
func (o *UART0_Type) SetUARTIMSC_RTIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x40)|value<<6)
}
func (o *UART0_Type) GetUARTIMSC_RTIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x40) >> 6
}
func (o *UART0_Type) SetUARTIMSC_TXIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x20)|value<<5)
}
func (o *UART0_Type) GetUARTIMSC_TXIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x20) >> 5
}
func (o *UART0_Type) SetUARTIMSC_RXIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x10)|value<<4)
}
func (o *UART0_Type) GetUARTIMSC_RXIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x10) >> 4
}
func (o *UART0_Type) SetUARTIMSC_DSRMIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x8)|value<<3)
}
func (o *UART0_Type) GetUARTIMSC_DSRMIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x8) >> 3
}
func (o *UART0_Type) SetUARTIMSC_DCDMIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x4)|value<<2)
}
func (o *UART0_Type) GetUARTIMSC_DCDMIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x4) >> 2
}
func (o *UART0_Type) SetUARTIMSC_CTSMIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x2)|value<<1)
}
func (o *UART0_Type) GetUARTIMSC_CTSMIM() uint32 {
	return (volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x2) >> 1
}
func (o *UART0_Type) SetUARTIMSC_RIMIM(value uint32) {
	volatile.StoreUint32(&o.UARTIMSC.Reg, volatile.LoadUint32(&o.UARTIMSC.Reg)&^(0x1)|value)
}
func (o *UART0_Type) GetUARTIMSC_RIMIM() uint32 {
	return volatile.LoadUint32(&o.UARTIMSC.Reg) & 0x1
}

// UART0.UARTRIS: Raw Interrupt Status Register, UARTRIS
func (o *UART0_Type) SetUARTRIS_OERIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x400)|value<<10)
}
func (o *UART0_Type) GetUARTRIS_OERIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x400) >> 10
}
func (o *UART0_Type) SetUARTRIS_BERIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x200)|value<<9)
}
func (o *UART0_Type) GetUARTRIS_BERIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x200) >> 9
}
func (o *UART0_Type) SetUARTRIS_PERIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x100)|value<<8)
}
func (o *UART0_Type) GetUARTRIS_PERIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x100) >> 8
}
func (o *UART0_Type) SetUARTRIS_FERIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x80)|value<<7)
}
func (o *UART0_Type) GetUARTRIS_FERIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x80) >> 7
}
func (o *UART0_Type) SetUARTRIS_RTRIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x40)|value<<6)
}
func (o *UART0_Type) GetUARTRIS_RTRIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x40) >> 6
}
func (o *UART0_Type) SetUARTRIS_TXRIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x20)|value<<5)
}
func (o *UART0_Type) GetUARTRIS_TXRIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x20) >> 5
}
func (o *UART0_Type) SetUARTRIS_RXRIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x10)|value<<4)
}
func (o *UART0_Type) GetUARTRIS_RXRIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x10) >> 4
}
func (o *UART0_Type) SetUARTRIS_DSRRMIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x8)|value<<3)
}
func (o *UART0_Type) GetUARTRIS_DSRRMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x8) >> 3
}
func (o *UART0_Type) SetUARTRIS_DCDRMIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x4)|value<<2)
}
func (o *UART0_Type) GetUARTRIS_DCDRMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x4) >> 2
}
func (o *UART0_Type) SetUARTRIS_CTSRMIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x2)|value<<1)
}
func (o *UART0_Type) GetUARTRIS_CTSRMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTRIS.Reg) & 0x2) >> 1
}
func (o *UART0_Type) SetUARTRIS_RIRMIS(value uint32) {
	volatile.StoreUint32(&o.UARTRIS.Reg, volatile.LoadUint32(&o.UARTRIS.Reg)&^(0x1)|value)
}
func (o *UART0_Type) GetUARTRIS_RIRMIS() uint32 {
	return volatile.LoadUint32(&o.UARTRIS.Reg) & 0x1
}

// UART0.UARTMIS: Masked Interrupt Status Register, UARTMIS
func (o *UART0_Type) SetUARTMIS_OEMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x400)|value<<10)
}
func (o *UART0_Type) GetUARTMIS_OEMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x400) >> 10
}
func (o *UART0_Type) SetUARTMIS_BEMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x200)|value<<9)
}
func (o *UART0_Type) GetUARTMIS_BEMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x200) >> 9
}
func (o *UART0_Type) SetUARTMIS_PEMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x100)|value<<8)
}
func (o *UART0_Type) GetUARTMIS_PEMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x100) >> 8
}
func (o *UART0_Type) SetUARTMIS_FEMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x80)|value<<7)
}
func (o *UART0_Type) GetUARTMIS_FEMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x80) >> 7
}
func (o *UART0_Type) SetUARTMIS_RTMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x40)|value<<6)
}
func (o *UART0_Type) GetUARTMIS_RTMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x40) >> 6
}
func (o *UART0_Type) SetUARTMIS_TXMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x20)|value<<5)
}
func (o *UART0_Type) GetUARTMIS_TXMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x20) >> 5
}
func (o *UART0_Type) SetUARTMIS_RXMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x10)|value<<4)
}
func (o *UART0_Type) GetUARTMIS_RXMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x10) >> 4
}
func (o *UART0_Type) SetUARTMIS_DSRMMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x8)|value<<3)
}
func (o *UART0_Type) GetUARTMIS_DSRMMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x8) >> 3
}
func (o *UART0_Type) SetUARTMIS_DCDMMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x4)|value<<2)
}
func (o *UART0_Type) GetUARTMIS_DCDMMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x4) >> 2
}
func (o *UART0_Type) SetUARTMIS_CTSMMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x2)|value<<1)
}
func (o *UART0_Type) GetUARTMIS_CTSMMIS() uint32 {
	return (volatile.LoadUint32(&o.UARTMIS.Reg) & 0x2) >> 1
}
func (o *UART0_Type) SetUARTMIS_RIMMIS(value uint32) {
	volatile.StoreUint32(&o.UARTMIS.Reg, volatile.LoadUint32(&o.UARTMIS.Reg)&^(0x1)|value)
}
func (o *UART0_Type) GetUARTMIS_RIMMIS() uint32 {
	return volatile.LoadUint32(&o.UARTMIS.Reg) & 0x1
}

// UART0.UARTICR: Interrupt Clear Register, UARTICR
func (o *UART0_Type) SetUARTICR_OEIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x400)|value<<10)
}
func (o *UART0_Type) GetUARTICR_OEIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x400) >> 10
}
func (o *UART0_Type) SetUARTICR_BEIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x200)|value<<9)
}
func (o *UART0_Type) GetUARTICR_BEIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x200) >> 9
}
func (o *UART0_Type) SetUARTICR_PEIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x100)|value<<8)
}
func (o *UART0_Type) GetUARTICR_PEIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x100) >> 8
}
func (o *UART0_Type) SetUARTICR_FEIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x80)|value<<7)
}
func (o *UART0_Type) GetUARTICR_FEIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x80) >> 7
}
func (o *UART0_Type) SetUARTICR_RTIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x40)|value<<6)
}
func (o *UART0_Type) GetUARTICR_RTIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x40) >> 6
}
func (o *UART0_Type) SetUARTICR_TXIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x20)|value<<5)
}
func (o *UART0_Type) GetUARTICR_TXIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x20) >> 5
}
func (o *UART0_Type) SetUARTICR_RXIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x10)|value<<4)
}
func (o *UART0_Type) GetUARTICR_RXIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x10) >> 4
}
func (o *UART0_Type) SetUARTICR_DSRMIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x8)|value<<3)
}
func (o *UART0_Type) GetUARTICR_DSRMIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x8) >> 3
}
func (o *UART0_Type) SetUARTICR_DCDMIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x4)|value<<2)
}
func (o *UART0_Type) GetUARTICR_DCDMIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x4) >> 2
}
func (o *UART0_Type) SetUARTICR_CTSMIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x2)|value<<1)
}
func (o *UART0_Type) GetUARTICR_CTSMIC() uint32 {
	return (volatile.LoadUint32(&o.UARTICR.Reg) & 0x2) >> 1
}
func (o *UART0_Type) SetUARTICR_RIMIC(value uint32) {
	volatile.StoreUint32(&o.UARTICR.Reg, volatile.LoadUint32(&o.UARTICR.Reg)&^(0x1)|value)
}
func (o *UART0_Type) GetUARTICR_RIMIC() uint32 {
	return volatile.LoadUint32(&o.UARTICR.Reg) & 0x1
}

// UART0.UARTDMACR: DMA Control Register, UARTDMACR
func (o *UART0_Type) SetUARTDMACR_DMAONERR(value uint32) {
	volatile.StoreUint32(&o.UARTDMACR.Reg, volatile.LoadUint32(&o.UARTDMACR.Reg)&^(0x4)|value<<2)
}
func (o *UART0_Type) GetUARTDMACR_DMAONERR() uint32 {
	return (volatile.LoadUint32(&o.UARTDMACR.Reg) & 0x4) >> 2
}
func (o *UART0_Type) SetUARTDMACR_TXDMAE(value uint32) {
	volatile.StoreUint32(&o.UARTDMACR.Reg, volatile.LoadUint32(&o.UARTDMACR.Reg)&^(0x2)|value<<1)
}
func (o *UART0_Type) GetUARTDMACR_TXDMAE() uint32 {
	return (volatile.LoadUint32(&o.UARTDMACR.Reg) & 0x2) >> 1
}
func (o *UART0_Type) SetUARTDMACR_RXDMAE(value uint32) {
	volatile.StoreUint32(&o.UARTDMACR.Reg, volatile.LoadUint32(&o.UARTDMACR.Reg)&^(0x1)|value)
}
func (o *UART0_Type) GetUARTDMACR_RXDMAE() uint32 {
	return volatile.LoadUint32(&o.UARTDMACR.Reg) & 0x1
}

// UART0.UARTPERIPHID0: UARTPeriphID0 Register
func (o *UART0_Type) SetUARTPERIPHID0_PARTNUMBER0(value uint32) {
	volatile.StoreUint32(&o.UARTPERIPHID0.Reg, volatile.LoadUint32(&o.UARTPERIPHID0.Reg)&^(0xff)|value)
}
func (o *UART0_Type) GetUARTPERIPHID0_PARTNUMBER0() uint32 {
	return volatile.LoadUint32(&o.UARTPERIPHID0.Reg) & 0xff
}

// UART0.UARTPERIPHID1: UARTPeriphID1 Register
func (o *UART0_Type) SetUARTPERIPHID1_DESIGNER0(value uint32) {
	volatile.StoreUint32(&o.UARTPERIPHID1.Reg, volatile.LoadUint32(&o.UARTPERIPHID1.Reg)&^(0xf0)|value<<4)
}
func (o *UART0_Type) GetUARTPERIPHID1_DESIGNER0() uint32 {
	return (volatile.LoadUint32(&o.UARTPERIPHID1.Reg) & 0xf0) >> 4
}
func (o *UART0_Type) SetUARTPERIPHID1_PARTNUMBER1(value uint32) {
	volatile.StoreUint32(&o.UARTPERIPHID1.Reg, volatile.LoadUint32(&o.UARTPERIPHID1.Reg)&^(0xf)|value)
}
func (o *UART0_Type) GetUARTPERIPHID1_PARTNUMBER1() uint32 {
	return volatile.LoadUint32(&o.UARTPERIPHID1.Reg) & 0xf
}

// UART0.UARTPERIPHID2: UARTPeriphID2 Register
func (o *UART0_Type) SetUARTPERIPHID2_REVISION(value uint32) {
	volatile.StoreUint32(&o.UARTPERIPHID2.Reg, volatile.LoadUint32(&o.UARTPERIPHID2.Reg)&^(0xf0)|value<<4)
}
func (o *UART0_Type) GetUARTPERIPHID2_REVISION() uint32 {
	return (volatile.LoadUint32(&o.UARTPERIPHID2.Reg) & 0xf0) >> 4
}
func (o *UART0_Type) SetUARTPERIPHID2_DESIGNER1(value uint32) {
	volatile.StoreUint32(&o.UARTPERIPHID2.Reg, volatile.LoadUint32(&o.UARTPERIPHID2.Reg)&^(0xf)|value)
}
func (o *UART0_Type) GetUARTPERIPHID2_DESIGNER1() uint32 {
	return volatile.LoadUint32(&o.UARTPERIPHID2.Reg) & 0xf
}

// UART0.UARTPERIPHID3: UARTPeriphID3 Register
func (o *UART0_Type) SetUARTPERIPHID3_CONFIGURATION(value uint32) {
	volatile.StoreUint32(&o.UARTPERIPHID3.Reg, volatile.LoadUint32(&o.UARTPERIPHID3.Reg)&^(0xff)|value)
}
func (o *UART0_Type) GetUARTPERIPHID3_CONFIGURATION() uint32 {
	return volatile.LoadUint32(&o.UARTPERIPHID3.Reg) & 0xff
}

// UART0.UARTPCELLID0: UARTPCellID0 Register
func (o *UART0_Type) SetUARTPCELLID0(value uint32) {
	volatile.StoreUint32(&o.UARTPCELLID0.Reg, volatile.LoadUint32(&o.UARTPCELLID0.Reg)&^(0xff)|value)
}
func (o *UART0_Type) GetUARTPCELLID0() uint32 {
	return volatile.LoadUint32(&o.UARTPCELLID0.Reg) & 0xff
}

// UART0.UARTPCELLID1: UARTPCellID1 Register
func (o *UART0_Type) SetUARTPCELLID1(value uint32) {
	volatile.StoreUint32(&o.UARTPCELLID1.Reg, volatile.LoadUint32(&o.UARTPCELLID1.Reg)&^(0xff)|value)
}
func (o *UART0_Type) GetUARTPCELLID1() uint32 {
	return volatile.LoadUint32(&o.UARTPCELLID1.Reg) & 0xff
}

// UART0.UARTPCELLID2: UARTPCellID2 Register
func (o *UART0_Type) SetUARTPCELLID2(value uint32) {
	volatile.StoreUint32(&o.UARTPCELLID2.Reg, volatile.LoadUint32(&o.UARTPCELLID2.Reg)&^(0xff)|value)
}
func (o *UART0_Type) GetUARTPCELLID2() uint32 {
	return volatile.LoadUint32(&o.UARTPCELLID2.Reg) & 0xff
}

// UART0.UARTPCELLID3: UARTPCellID3 Register
func (o *UART0_Type) SetUARTPCELLID3(value uint32) {
	volatile.StoreUint32(&o.UARTPCELLID3.Reg, volatile.LoadUint32(&o.UARTPCELLID3.Reg)&^(0xff)|value)
}
func (o *UART0_Type) GetUARTPCELLID3() uint32 {
	return volatile.LoadUint32(&o.UARTPCELLID3.Reg) & 0xff
}

type ROSC_Type struct {
	CTRL      volatile.Register32 // 0x0
	FREQA     volatile.Register32 // 0x4
	FREQB     volatile.Register32 // 0x8
	RANDOM    volatile.Register32 // 0xC
	DORMANT   volatile.Register32 // 0x10
	DIV       volatile.Register32 // 0x14
	PHASE     volatile.Register32 // 0x18
	STATUS    volatile.Register32 // 0x1C
	RANDOMBIT volatile.Register32 // 0x20
	COUNT     volatile.Register32 // 0x24
}

// ROSC.CTRL: Ring Oscillator control
func (o *ROSC_Type) SetCTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xfff000)|value<<12)
}
func (o *ROSC_Type) GetCTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0xfff000) >> 12
}
func (o *ROSC_Type) SetCTRL_FREQ_RANGE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xfff)|value)
}
func (o *ROSC_Type) GetCTRL_FREQ_RANGE() uint32 {
	return volatile.LoadUint32(&o.CTRL.Reg) & 0xfff
}

// ROSC.FREQA: The FREQA & FREQB registers control the frequency by controlling the drive strength of each stage The drive strength has 4 levels determined by the number of bits set Increasing the number of bits set increases the drive strength and increases the oscillation frequency 0 bits set is the default drive strength 1 bit set doubles the drive strength 2 bits set triples drive strength 3 bits set quadruples drive strength For frequency randomisation set both DS0_RANDOM=1 & DS1_RANDOM=1
func (o *ROSC_Type) SetFREQA_PASSWD(value uint32) {
	volatile.StoreUint32(&o.FREQA.Reg, volatile.LoadUint32(&o.FREQA.Reg)&^(0xffff0000)|value<<16)
}
func (o *ROSC_Type) GetFREQA_PASSWD() uint32 {
	return (volatile.LoadUint32(&o.FREQA.Reg) & 0xffff0000) >> 16
}
func (o *ROSC_Type) SetFREQA_DS3(value uint32) {
	volatile.StoreUint32(&o.FREQA.Reg, volatile.LoadUint32(&o.FREQA.Reg)&^(0x7000)|value<<12)
}
func (o *ROSC_Type) GetFREQA_DS3() uint32 {
	return (volatile.LoadUint32(&o.FREQA.Reg) & 0x7000) >> 12
}
func (o *ROSC_Type) SetFREQA_DS2(value uint32) {
	volatile.StoreUint32(&o.FREQA.Reg, volatile.LoadUint32(&o.FREQA.Reg)&^(0x700)|value<<8)
}
func (o *ROSC_Type) GetFREQA_DS2() uint32 {
	return (volatile.LoadUint32(&o.FREQA.Reg) & 0x700) >> 8
}
func (o *ROSC_Type) SetFREQA_DS1_RANDOM(value uint32) {
	volatile.StoreUint32(&o.FREQA.Reg, volatile.LoadUint32(&o.FREQA.Reg)&^(0x80)|value<<7)
}
func (o *ROSC_Type) GetFREQA_DS1_RANDOM() uint32 {
	return (volatile.LoadUint32(&o.FREQA.Reg) & 0x80) >> 7
}
func (o *ROSC_Type) SetFREQA_DS1(value uint32) {
	volatile.StoreUint32(&o.FREQA.Reg, volatile.LoadUint32(&o.FREQA.Reg)&^(0x70)|value<<4)
}
func (o *ROSC_Type) GetFREQA_DS1() uint32 {
	return (volatile.LoadUint32(&o.FREQA.Reg) & 0x70) >> 4
}
func (o *ROSC_Type) SetFREQA_DS0_RANDOM(value uint32) {
	volatile.StoreUint32(&o.FREQA.Reg, volatile.LoadUint32(&o.FREQA.Reg)&^(0x8)|value<<3)
}
func (o *ROSC_Type) GetFREQA_DS0_RANDOM() uint32 {
	return (volatile.LoadUint32(&o.FREQA.Reg) & 0x8) >> 3
}
func (o *ROSC_Type) SetFREQA_DS0(value uint32) {
	volatile.StoreUint32(&o.FREQA.Reg, volatile.LoadUint32(&o.FREQA.Reg)&^(0x7)|value)
}
func (o *ROSC_Type) GetFREQA_DS0() uint32 {
	return volatile.LoadUint32(&o.FREQA.Reg) & 0x7
}

// ROSC.FREQB: For a detailed description see freqa register
func (o *ROSC_Type) SetFREQB_PASSWD(value uint32) {
	volatile.StoreUint32(&o.FREQB.Reg, volatile.LoadUint32(&o.FREQB.Reg)&^(0xffff0000)|value<<16)
}
func (o *ROSC_Type) GetFREQB_PASSWD() uint32 {
	return (volatile.LoadUint32(&o.FREQB.Reg) & 0xffff0000) >> 16
}
func (o *ROSC_Type) SetFREQB_DS7(value uint32) {
	volatile.StoreUint32(&o.FREQB.Reg, volatile.LoadUint32(&o.FREQB.Reg)&^(0x7000)|value<<12)
}
func (o *ROSC_Type) GetFREQB_DS7() uint32 {
	return (volatile.LoadUint32(&o.FREQB.Reg) & 0x7000) >> 12
}
func (o *ROSC_Type) SetFREQB_DS6(value uint32) {
	volatile.StoreUint32(&o.FREQB.Reg, volatile.LoadUint32(&o.FREQB.Reg)&^(0x700)|value<<8)
}
func (o *ROSC_Type) GetFREQB_DS6() uint32 {
	return (volatile.LoadUint32(&o.FREQB.Reg) & 0x700) >> 8
}
func (o *ROSC_Type) SetFREQB_DS5(value uint32) {
	volatile.StoreUint32(&o.FREQB.Reg, volatile.LoadUint32(&o.FREQB.Reg)&^(0x70)|value<<4)
}
func (o *ROSC_Type) GetFREQB_DS5() uint32 {
	return (volatile.LoadUint32(&o.FREQB.Reg) & 0x70) >> 4
}
func (o *ROSC_Type) SetFREQB_DS4(value uint32) {
	volatile.StoreUint32(&o.FREQB.Reg, volatile.LoadUint32(&o.FREQB.Reg)&^(0x7)|value)
}
func (o *ROSC_Type) GetFREQB_DS4() uint32 {
	return volatile.LoadUint32(&o.FREQB.Reg) & 0x7
}

// ROSC.RANDOM: Loads a value to the LFSR randomiser
func (o *ROSC_Type) SetRANDOM(value uint32) {
	volatile.StoreUint32(&o.RANDOM.Reg, value)
}
func (o *ROSC_Type) GetRANDOM() uint32 {
	return volatile.LoadUint32(&o.RANDOM.Reg)
}

// ROSC.DORMANT: Ring Oscillator pause control
func (o *ROSC_Type) SetDORMANT(value uint32) {
	volatile.StoreUint32(&o.DORMANT.Reg, value)
}
func (o *ROSC_Type) GetDORMANT() uint32 {
	return volatile.LoadUint32(&o.DORMANT.Reg)
}

// ROSC.DIV: Controls the output divider
func (o *ROSC_Type) SetDIV(value uint32) {
	volatile.StoreUint32(&o.DIV.Reg, volatile.LoadUint32(&o.DIV.Reg)&^(0xffff)|value)
}
func (o *ROSC_Type) GetDIV() uint32 {
	return volatile.LoadUint32(&o.DIV.Reg) & 0xffff
}

// ROSC.PHASE: Controls the phase shifted output
func (o *ROSC_Type) SetPHASE_PASSWD(value uint32) {
	volatile.StoreUint32(&o.PHASE.Reg, volatile.LoadUint32(&o.PHASE.Reg)&^(0xff0)|value<<4)
}
func (o *ROSC_Type) GetPHASE_PASSWD() uint32 {
	return (volatile.LoadUint32(&o.PHASE.Reg) & 0xff0) >> 4
}
func (o *ROSC_Type) SetPHASE_ENABLE(value uint32) {
	volatile.StoreUint32(&o.PHASE.Reg, volatile.LoadUint32(&o.PHASE.Reg)&^(0x8)|value<<3)
}
func (o *ROSC_Type) GetPHASE_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.PHASE.Reg) & 0x8) >> 3
}
func (o *ROSC_Type) SetPHASE_FLIP(value uint32) {
	volatile.StoreUint32(&o.PHASE.Reg, volatile.LoadUint32(&o.PHASE.Reg)&^(0x4)|value<<2)
}
func (o *ROSC_Type) GetPHASE_FLIP() uint32 {
	return (volatile.LoadUint32(&o.PHASE.Reg) & 0x4) >> 2
}
func (o *ROSC_Type) SetPHASE_SHIFT(value uint32) {
	volatile.StoreUint32(&o.PHASE.Reg, volatile.LoadUint32(&o.PHASE.Reg)&^(0x3)|value)
}
func (o *ROSC_Type) GetPHASE_SHIFT() uint32 {
	return volatile.LoadUint32(&o.PHASE.Reg) & 0x3
}

// ROSC.STATUS: Ring Oscillator Status
func (o *ROSC_Type) SetSTATUS_STABLE(value uint32) {
	volatile.StoreUint32(&o.STATUS.Reg, volatile.LoadUint32(&o.STATUS.Reg)&^(0x80000000)|value<<31)
}
func (o *ROSC_Type) GetSTATUS_STABLE() uint32 {
	return (volatile.LoadUint32(&o.STATUS.Reg) & 0x80000000) >> 31
}
func (o *ROSC_Type) SetSTATUS_BADWRITE(value uint32) {
	volatile.StoreUint32(&o.STATUS.Reg, volatile.LoadUint32(&o.STATUS.Reg)&^(0x1000000)|value<<24)
}
func (o *ROSC_Type) GetSTATUS_BADWRITE() uint32 {
	return (volatile.LoadUint32(&o.STATUS.Reg) & 0x1000000) >> 24
}
func (o *ROSC_Type) SetSTATUS_DIV_RUNNING(value uint32) {
	volatile.StoreUint32(&o.STATUS.Reg, volatile.LoadUint32(&o.STATUS.Reg)&^(0x10000)|value<<16)
}
func (o *ROSC_Type) GetSTATUS_DIV_RUNNING() uint32 {
	return (volatile.LoadUint32(&o.STATUS.Reg) & 0x10000) >> 16
}
func (o *ROSC_Type) SetSTATUS_ENABLED(value uint32) {
	volatile.StoreUint32(&o.STATUS.Reg, volatile.LoadUint32(&o.STATUS.Reg)&^(0x1000)|value<<12)
}
func (o *ROSC_Type) GetSTATUS_ENABLED() uint32 {
	return (volatile.LoadUint32(&o.STATUS.Reg) & 0x1000) >> 12
}

// ROSC.RANDOMBIT: This just reads the state of the oscillator output so randomness is compromised if the ring oscillator is stopped or run at a harmonic of the bus frequency
func (o *ROSC_Type) SetRANDOMBIT(value uint32) {
	volatile.StoreUint32(&o.RANDOMBIT.Reg, volatile.LoadUint32(&o.RANDOMBIT.Reg)&^(0x1)|value)
}
func (o *ROSC_Type) GetRANDOMBIT() uint32 {
	return volatile.LoadUint32(&o.RANDOMBIT.Reg) & 0x1
}

// ROSC.COUNT: A down counter running at the ROSC frequency which counts to zero and stops. To start the counter write a non-zero value. Can be used for short software pauses when setting up time sensitive hardware.
func (o *ROSC_Type) SetCOUNT(value uint32) {
	volatile.StoreUint32(&o.COUNT.Reg, volatile.LoadUint32(&o.COUNT.Reg)&^(0xffff)|value)
}
func (o *ROSC_Type) GetCOUNT() uint32 {
	return volatile.LoadUint32(&o.COUNT.Reg) & 0xffff
}

// Controls vreg, bor, lposc, chip resets & xosc startup, powman and provides scratch register for general use and for bootcode use
type POWMAN_Type struct {
	BADPASSWD           volatile.Register32 // 0x0
	VREG_CTRL           volatile.Register32 // 0x4
	VREG_STS            volatile.Register32 // 0x8
	VREG                volatile.Register32 // 0xC
	VREG_LP_ENTRY       volatile.Register32 // 0x10
	VREG_LP_EXIT        volatile.Register32 // 0x14
	BOD_CTRL            volatile.Register32 // 0x18
	BOD                 volatile.Register32 // 0x1C
	BOD_LP_ENTRY        volatile.Register32 // 0x20
	BOD_LP_EXIT         volatile.Register32 // 0x24
	LPOSC               volatile.Register32 // 0x28
	CHIP_RESET          volatile.Register32 // 0x2C
	WDSEL               volatile.Register32 // 0x30
	SEQ_CFG             volatile.Register32 // 0x34
	STATE               volatile.Register32 // 0x38
	POW_FASTDIV         volatile.Register32 // 0x3C
	POW_DELAY           volatile.Register32 // 0x40
	EXT_CTRL0           volatile.Register32 // 0x44
	EXT_CTRL1           volatile.Register32 // 0x48
	EXT_TIME_REF        volatile.Register32 // 0x4C
	LPOSC_FREQ_KHZ_INT  volatile.Register32 // 0x50
	LPOSC_FREQ_KHZ_FRAC volatile.Register32 // 0x54
	XOSC_FREQ_KHZ_INT   volatile.Register32 // 0x58
	XOSC_FREQ_KHZ_FRAC  volatile.Register32 // 0x5C
	SET_TIME_63TO48     volatile.Register32 // 0x60
	SET_TIME_47TO32     volatile.Register32 // 0x64
	SET_TIME_31TO16     volatile.Register32 // 0x68
	SET_TIME_15TO0      volatile.Register32 // 0x6C
	READ_TIME_UPPER     volatile.Register32 // 0x70
	READ_TIME_LOWER     volatile.Register32 // 0x74
	ALARM_TIME_63TO48   volatile.Register32 // 0x78
	ALARM_TIME_47TO32   volatile.Register32 // 0x7C
	ALARM_TIME_31TO16   volatile.Register32 // 0x80
	ALARM_TIME_15TO0    volatile.Register32 // 0x84
	TIMER               volatile.Register32 // 0x88
	PWRUP0              volatile.Register32 // 0x8C
	PWRUP1              volatile.Register32 // 0x90
	PWRUP2              volatile.Register32 // 0x94
	PWRUP3              volatile.Register32 // 0x98
	CURRENT_PWRUP_REQ   volatile.Register32 // 0x9C
	LAST_SWCORE_PWRUP   volatile.Register32 // 0xA0
	DBG_PWRCFG          volatile.Register32 // 0xA4
	BOOTDIS             volatile.Register32 // 0xA8
	DBGCONFIG           volatile.Register32 // 0xAC
	SCRATCH0            volatile.Register32 // 0xB0
	SCRATCH1            volatile.Register32 // 0xB4
	SCRATCH2            volatile.Register32 // 0xB8
	SCRATCH3            volatile.Register32 // 0xBC
	SCRATCH4            volatile.Register32 // 0xC0
	SCRATCH5            volatile.Register32 // 0xC4
	SCRATCH6            volatile.Register32 // 0xC8
	SCRATCH7            volatile.Register32 // 0xCC
	BOOT0               volatile.Register32 // 0xD0
	BOOT1               volatile.Register32 // 0xD4
	BOOT2               volatile.Register32 // 0xD8
	BOOT3               volatile.Register32 // 0xDC
	INTR                volatile.Register32 // 0xE0
	INTE                volatile.Register32 // 0xE4
	INTF                volatile.Register32 // 0xE8
	INTS                volatile.Register32 // 0xEC
}

// POWMAN.BADPASSWD: Indicates a bad password has been used
func (o *POWMAN_Type) SetBADPASSWD(value uint32) {
	volatile.StoreUint32(&o.BADPASSWD.Reg, volatile.LoadUint32(&o.BADPASSWD.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetBADPASSWD() uint32 {
	return volatile.LoadUint32(&o.BADPASSWD.Reg) & 0x1
}

// POWMAN.VREG_CTRL: Voltage Regulator Control
func (o *POWMAN_Type) SetVREG_CTRL_RST_N(value uint32) {
	volatile.StoreUint32(&o.VREG_CTRL.Reg, volatile.LoadUint32(&o.VREG_CTRL.Reg)&^(0x8000)|value<<15)
}
func (o *POWMAN_Type) GetVREG_CTRL_RST_N() uint32 {
	return (volatile.LoadUint32(&o.VREG_CTRL.Reg) & 0x8000) >> 15
}
func (o *POWMAN_Type) SetVREG_CTRL_UNLOCK(value uint32) {
	volatile.StoreUint32(&o.VREG_CTRL.Reg, volatile.LoadUint32(&o.VREG_CTRL.Reg)&^(0x2000)|value<<13)
}
func (o *POWMAN_Type) GetVREG_CTRL_UNLOCK() uint32 {
	return (volatile.LoadUint32(&o.VREG_CTRL.Reg) & 0x2000) >> 13
}
func (o *POWMAN_Type) SetVREG_CTRL_ISOLATE(value uint32) {
	volatile.StoreUint32(&o.VREG_CTRL.Reg, volatile.LoadUint32(&o.VREG_CTRL.Reg)&^(0x1000)|value<<12)
}
func (o *POWMAN_Type) GetVREG_CTRL_ISOLATE() uint32 {
	return (volatile.LoadUint32(&o.VREG_CTRL.Reg) & 0x1000) >> 12
}
func (o *POWMAN_Type) SetVREG_CTRL_DISABLE_VOLTAGE_LIMIT(value uint32) {
	volatile.StoreUint32(&o.VREG_CTRL.Reg, volatile.LoadUint32(&o.VREG_CTRL.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetVREG_CTRL_DISABLE_VOLTAGE_LIMIT() uint32 {
	return (volatile.LoadUint32(&o.VREG_CTRL.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetVREG_CTRL_HT_TH(value uint32) {
	volatile.StoreUint32(&o.VREG_CTRL.Reg, volatile.LoadUint32(&o.VREG_CTRL.Reg)&^(0x70)|value<<4)
}
func (o *POWMAN_Type) GetVREG_CTRL_HT_TH() uint32 {
	return (volatile.LoadUint32(&o.VREG_CTRL.Reg) & 0x70) >> 4
}

// POWMAN.VREG_STS: Voltage Regulator Status
func (o *POWMAN_Type) SetVREG_STS_VOUT_OK(value uint32) {
	volatile.StoreUint32(&o.VREG_STS.Reg, volatile.LoadUint32(&o.VREG_STS.Reg)&^(0x10)|value<<4)
}
func (o *POWMAN_Type) GetVREG_STS_VOUT_OK() uint32 {
	return (volatile.LoadUint32(&o.VREG_STS.Reg) & 0x10) >> 4
}
func (o *POWMAN_Type) SetVREG_STS_STARTUP(value uint32) {
	volatile.StoreUint32(&o.VREG_STS.Reg, volatile.LoadUint32(&o.VREG_STS.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetVREG_STS_STARTUP() uint32 {
	return volatile.LoadUint32(&o.VREG_STS.Reg) & 0x1
}

// POWMAN.VREG: Voltage Regulator Settings
func (o *POWMAN_Type) SetVREG_UPDATE_IN_PROGRESS(value uint32) {
	volatile.StoreUint32(&o.VREG.Reg, volatile.LoadUint32(&o.VREG.Reg)&^(0x8000)|value<<15)
}
func (o *POWMAN_Type) GetVREG_UPDATE_IN_PROGRESS() uint32 {
	return (volatile.LoadUint32(&o.VREG.Reg) & 0x8000) >> 15
}
func (o *POWMAN_Type) SetVREG_VSEL(value uint32) {
	volatile.StoreUint32(&o.VREG.Reg, volatile.LoadUint32(&o.VREG.Reg)&^(0x1f0)|value<<4)
}
func (o *POWMAN_Type) GetVREG_VSEL() uint32 {
	return (volatile.LoadUint32(&o.VREG.Reg) & 0x1f0) >> 4
}
func (o *POWMAN_Type) SetVREG_HIZ(value uint32) {
	volatile.StoreUint32(&o.VREG.Reg, volatile.LoadUint32(&o.VREG.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetVREG_HIZ() uint32 {
	return (volatile.LoadUint32(&o.VREG.Reg) & 0x2) >> 1
}

// POWMAN.VREG_LP_ENTRY: Voltage Regulator Low Power Entry Settings
func (o *POWMAN_Type) SetVREG_LP_ENTRY_VSEL(value uint32) {
	volatile.StoreUint32(&o.VREG_LP_ENTRY.Reg, volatile.LoadUint32(&o.VREG_LP_ENTRY.Reg)&^(0x1f0)|value<<4)
}
func (o *POWMAN_Type) GetVREG_LP_ENTRY_VSEL() uint32 {
	return (volatile.LoadUint32(&o.VREG_LP_ENTRY.Reg) & 0x1f0) >> 4
}
func (o *POWMAN_Type) SetVREG_LP_ENTRY_MODE(value uint32) {
	volatile.StoreUint32(&o.VREG_LP_ENTRY.Reg, volatile.LoadUint32(&o.VREG_LP_ENTRY.Reg)&^(0x4)|value<<2)
}
func (o *POWMAN_Type) GetVREG_LP_ENTRY_MODE() uint32 {
	return (volatile.LoadUint32(&o.VREG_LP_ENTRY.Reg) & 0x4) >> 2
}
func (o *POWMAN_Type) SetVREG_LP_ENTRY_HIZ(value uint32) {
	volatile.StoreUint32(&o.VREG_LP_ENTRY.Reg, volatile.LoadUint32(&o.VREG_LP_ENTRY.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetVREG_LP_ENTRY_HIZ() uint32 {
	return (volatile.LoadUint32(&o.VREG_LP_ENTRY.Reg) & 0x2) >> 1
}

// POWMAN.VREG_LP_EXIT: Voltage Regulator Low Power Exit Settings
func (o *POWMAN_Type) SetVREG_LP_EXIT_VSEL(value uint32) {
	volatile.StoreUint32(&o.VREG_LP_EXIT.Reg, volatile.LoadUint32(&o.VREG_LP_EXIT.Reg)&^(0x1f0)|value<<4)
}
func (o *POWMAN_Type) GetVREG_LP_EXIT_VSEL() uint32 {
	return (volatile.LoadUint32(&o.VREG_LP_EXIT.Reg) & 0x1f0) >> 4
}
func (o *POWMAN_Type) SetVREG_LP_EXIT_MODE(value uint32) {
	volatile.StoreUint32(&o.VREG_LP_EXIT.Reg, volatile.LoadUint32(&o.VREG_LP_EXIT.Reg)&^(0x4)|value<<2)
}
func (o *POWMAN_Type) GetVREG_LP_EXIT_MODE() uint32 {
	return (volatile.LoadUint32(&o.VREG_LP_EXIT.Reg) & 0x4) >> 2
}
func (o *POWMAN_Type) SetVREG_LP_EXIT_HIZ(value uint32) {
	volatile.StoreUint32(&o.VREG_LP_EXIT.Reg, volatile.LoadUint32(&o.VREG_LP_EXIT.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetVREG_LP_EXIT_HIZ() uint32 {
	return (volatile.LoadUint32(&o.VREG_LP_EXIT.Reg) & 0x2) >> 1
}

// POWMAN.BOD_CTRL: Brown-out Detection Control
func (o *POWMAN_Type) SetBOD_CTRL_ISOLATE(value uint32) {
	volatile.StoreUint32(&o.BOD_CTRL.Reg, volatile.LoadUint32(&o.BOD_CTRL.Reg)&^(0x1000)|value<<12)
}
func (o *POWMAN_Type) GetBOD_CTRL_ISOLATE() uint32 {
	return (volatile.LoadUint32(&o.BOD_CTRL.Reg) & 0x1000) >> 12
}

// POWMAN.BOD: Brown-out Detection Settings
func (o *POWMAN_Type) SetBOD_VSEL(value uint32) {
	volatile.StoreUint32(&o.BOD.Reg, volatile.LoadUint32(&o.BOD.Reg)&^(0x1f0)|value<<4)
}
func (o *POWMAN_Type) GetBOD_VSEL() uint32 {
	return (volatile.LoadUint32(&o.BOD.Reg) & 0x1f0) >> 4
}
func (o *POWMAN_Type) SetBOD_EN(value uint32) {
	volatile.StoreUint32(&o.BOD.Reg, volatile.LoadUint32(&o.BOD.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetBOD_EN() uint32 {
	return volatile.LoadUint32(&o.BOD.Reg) & 0x1
}

// POWMAN.BOD_LP_ENTRY: Brown-out Detection Low Power Entry Settings
func (o *POWMAN_Type) SetBOD_LP_ENTRY_VSEL(value uint32) {
	volatile.StoreUint32(&o.BOD_LP_ENTRY.Reg, volatile.LoadUint32(&o.BOD_LP_ENTRY.Reg)&^(0x1f0)|value<<4)
}
func (o *POWMAN_Type) GetBOD_LP_ENTRY_VSEL() uint32 {
	return (volatile.LoadUint32(&o.BOD_LP_ENTRY.Reg) & 0x1f0) >> 4
}
func (o *POWMAN_Type) SetBOD_LP_ENTRY_EN(value uint32) {
	volatile.StoreUint32(&o.BOD_LP_ENTRY.Reg, volatile.LoadUint32(&o.BOD_LP_ENTRY.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetBOD_LP_ENTRY_EN() uint32 {
	return volatile.LoadUint32(&o.BOD_LP_ENTRY.Reg) & 0x1
}

// POWMAN.BOD_LP_EXIT: Brown-out Detection Low Power Exit Settings
func (o *POWMAN_Type) SetBOD_LP_EXIT_VSEL(value uint32) {
	volatile.StoreUint32(&o.BOD_LP_EXIT.Reg, volatile.LoadUint32(&o.BOD_LP_EXIT.Reg)&^(0x1f0)|value<<4)
}
func (o *POWMAN_Type) GetBOD_LP_EXIT_VSEL() uint32 {
	return (volatile.LoadUint32(&o.BOD_LP_EXIT.Reg) & 0x1f0) >> 4
}
func (o *POWMAN_Type) SetBOD_LP_EXIT_EN(value uint32) {
	volatile.StoreUint32(&o.BOD_LP_EXIT.Reg, volatile.LoadUint32(&o.BOD_LP_EXIT.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetBOD_LP_EXIT_EN() uint32 {
	return volatile.LoadUint32(&o.BOD_LP_EXIT.Reg) & 0x1
}

// POWMAN.LPOSC: Low power oscillator control register.
func (o *POWMAN_Type) SetLPOSC_TRIM(value uint32) {
	volatile.StoreUint32(&o.LPOSC.Reg, volatile.LoadUint32(&o.LPOSC.Reg)&^(0x3f0)|value<<4)
}
func (o *POWMAN_Type) GetLPOSC_TRIM() uint32 {
	return (volatile.LoadUint32(&o.LPOSC.Reg) & 0x3f0) >> 4
}
func (o *POWMAN_Type) SetLPOSC_MODE(value uint32) {
	volatile.StoreUint32(&o.LPOSC.Reg, volatile.LoadUint32(&o.LPOSC.Reg)&^(0x3)|value)
}
func (o *POWMAN_Type) GetLPOSC_MODE() uint32 {
	return volatile.LoadUint32(&o.LPOSC.Reg) & 0x3
}

// POWMAN.CHIP_RESET: Chip reset control and status
func (o *POWMAN_Type) SetCHIP_RESET_HAD_WATCHDOG_RESET_RSM(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x10000000)|value<<28)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_WATCHDOG_RESET_RSM() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x10000000) >> 28
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_HZD_SYS_RESET_REQ(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x8000000)|value<<27)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_HZD_SYS_RESET_REQ() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x8000000) >> 27
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_GLITCH_DETECT(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x4000000)|value<<26)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_GLITCH_DETECT() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x4000000) >> 26
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_SWCORE_PD(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x2000000)|value<<25)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_SWCORE_PD() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x2000000) >> 25
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_WATCHDOG_RESET_SWCORE(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x1000000)|value<<24)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_WATCHDOG_RESET_SWCORE() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x1000000) >> 24
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_WATCHDOG_RESET_POWMAN(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x800000)|value<<23)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_WATCHDOG_RESET_POWMAN() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x800000) >> 23
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_WATCHDOG_RESET_POWMAN_ASYNC(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x400000)|value<<22)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_WATCHDOG_RESET_POWMAN_ASYNC() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x400000) >> 22
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_RESCUE(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x200000)|value<<21)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_RESCUE() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x200000) >> 21
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_DP_RESET_REQ(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x80000)|value<<19)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_DP_RESET_REQ() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x80000) >> 19
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_RUN_LOW(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x40000)|value<<18)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_RUN_LOW() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x40000) >> 18
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_BOR(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x20000)|value<<17)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_BOR() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x20000) >> 17
}
func (o *POWMAN_Type) SetCHIP_RESET_HAD_POR(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x10000)|value<<16)
}
func (o *POWMAN_Type) GetCHIP_RESET_HAD_POR() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x10000) >> 16
}
func (o *POWMAN_Type) SetCHIP_RESET_RESCUE_FLAG(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x10)|value<<4)
}
func (o *POWMAN_Type) GetCHIP_RESET_RESCUE_FLAG() uint32 {
	return (volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x10) >> 4
}
func (o *POWMAN_Type) SetCHIP_RESET_DOUBLE_TAP(value uint32) {
	volatile.StoreUint32(&o.CHIP_RESET.Reg, volatile.LoadUint32(&o.CHIP_RESET.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetCHIP_RESET_DOUBLE_TAP() uint32 {
	return volatile.LoadUint32(&o.CHIP_RESET.Reg) & 0x1
}

// POWMAN.WDSEL: Allows a watchdog reset to reset the internal state of powman in addition to the power-on state machine (PSM). Note that powman ignores watchdog resets that do not select at least the CLOCKS stage or earlier stages in the PSM. If using these bits, it's recommended to set PSM_WDSEL to all-ones in addition to the desired bits in this register. Failing to select CLOCKS or earlier will result in the POWMAN_WDSEL register having no effect.
func (o *POWMAN_Type) SetWDSEL_RESET_RSM(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x1000)|value<<12)
}
func (o *POWMAN_Type) GetWDSEL_RESET_RSM() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x1000) >> 12
}
func (o *POWMAN_Type) SetWDSEL_RESET_SWCORE(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetWDSEL_RESET_SWCORE() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetWDSEL_RESET_POWMAN(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x10)|value<<4)
}
func (o *POWMAN_Type) GetWDSEL_RESET_POWMAN() uint32 {
	return (volatile.LoadUint32(&o.WDSEL.Reg) & 0x10) >> 4
}
func (o *POWMAN_Type) SetWDSEL_RESET_POWMAN_ASYNC(value uint32) {
	volatile.StoreUint32(&o.WDSEL.Reg, volatile.LoadUint32(&o.WDSEL.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetWDSEL_RESET_POWMAN_ASYNC() uint32 {
	return volatile.LoadUint32(&o.WDSEL.Reg) & 0x1
}

// POWMAN.SEQ_CFG: For configuration of the power sequencer Writes are ignored while POWMAN_STATE_CHANGING=1
func (o *POWMAN_Type) SetSEQ_CFG_USING_FAST_POWCK(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x100000)|value<<20)
}
func (o *POWMAN_Type) GetSEQ_CFG_USING_FAST_POWCK() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x100000) >> 20
}
func (o *POWMAN_Type) SetSEQ_CFG_USING_BOD_LP(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x20000)|value<<17)
}
func (o *POWMAN_Type) GetSEQ_CFG_USING_BOD_LP() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x20000) >> 17
}
func (o *POWMAN_Type) SetSEQ_CFG_USING_VREG_LP(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x10000)|value<<16)
}
func (o *POWMAN_Type) GetSEQ_CFG_USING_VREG_LP() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x10000) >> 16
}
func (o *POWMAN_Type) SetSEQ_CFG_USE_FAST_POWCK(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x1000)|value<<12)
}
func (o *POWMAN_Type) GetSEQ_CFG_USE_FAST_POWCK() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x1000) >> 12
}
func (o *POWMAN_Type) SetSEQ_CFG_RUN_LPOSC_IN_LP(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetSEQ_CFG_RUN_LPOSC_IN_LP() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetSEQ_CFG_USE_BOD_HP(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x80)|value<<7)
}
func (o *POWMAN_Type) GetSEQ_CFG_USE_BOD_HP() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x80) >> 7
}
func (o *POWMAN_Type) SetSEQ_CFG_USE_BOD_LP(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x40)|value<<6)
}
func (o *POWMAN_Type) GetSEQ_CFG_USE_BOD_LP() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x40) >> 6
}
func (o *POWMAN_Type) SetSEQ_CFG_USE_VREG_HP(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x20)|value<<5)
}
func (o *POWMAN_Type) GetSEQ_CFG_USE_VREG_HP() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x20) >> 5
}
func (o *POWMAN_Type) SetSEQ_CFG_USE_VREG_LP(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x10)|value<<4)
}
func (o *POWMAN_Type) GetSEQ_CFG_USE_VREG_LP() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x10) >> 4
}
func (o *POWMAN_Type) SetSEQ_CFG_HW_PWRUP_SRAM0(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetSEQ_CFG_HW_PWRUP_SRAM0() uint32 {
	return (volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x2) >> 1
}
func (o *POWMAN_Type) SetSEQ_CFG_HW_PWRUP_SRAM1(value uint32) {
	volatile.StoreUint32(&o.SEQ_CFG.Reg, volatile.LoadUint32(&o.SEQ_CFG.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetSEQ_CFG_HW_PWRUP_SRAM1() uint32 {
	return volatile.LoadUint32(&o.SEQ_CFG.Reg) & 0x1
}

// POWMAN.STATE: This register controls the power state of the 4 power domains. The current power state is indicated in POWMAN_STATE_CURRENT which is read-only. To change the state, write to POWMAN_STATE_REQ. The coding of POWMAN_STATE_CURRENT & POWMAN_STATE_REQ corresponds to the power states defined in the datasheet: bit 3 = SWCORE bit 2 = XIP cache bit 1 = SRAM0 bit 0 = SRAM1 0 = powered up 1 = powered down When POWMAN_STATE_REQ is written, the POWMAN_STATE_WAITING flag is set while the Power Manager determines what is required. If an invalid transition is requested the Power Manager will still register the request in POWMAN_STATE_REQ but will also set the POWMAN_BAD_REQ flag. It will then implement the power-up requests and ignore the power down requests. To do nothing would risk entering an unrecoverable lock-up state. Invalid requests are: any combination of power up and power down requests any request that results in swcore boing powered and xip unpowered If the request is to power down the switched-core domain then POWMAN_STATE_WAITING stays active until the processors halt. During this time the POWMAN_STATE_REQ field can be re-written to change or cancel the request. When the power state transition begins the POWMAN_STATE_WAITING_flag is cleared, the POWMAN_STATE_CHANGING flag is set and POWMAN register writes are ignored until the transition completes.
func (o *POWMAN_Type) SetSTATE_CHANGING(value uint32) {
	volatile.StoreUint32(&o.STATE.Reg, volatile.LoadUint32(&o.STATE.Reg)&^(0x2000)|value<<13)
}
func (o *POWMAN_Type) GetSTATE_CHANGING() uint32 {
	return (volatile.LoadUint32(&o.STATE.Reg) & 0x2000) >> 13
}
func (o *POWMAN_Type) SetSTATE_WAITING(value uint32) {
	volatile.StoreUint32(&o.STATE.Reg, volatile.LoadUint32(&o.STATE.Reg)&^(0x1000)|value<<12)
}
func (o *POWMAN_Type) GetSTATE_WAITING() uint32 {
	return (volatile.LoadUint32(&o.STATE.Reg) & 0x1000) >> 12
}
func (o *POWMAN_Type) SetSTATE_BAD_HW_REQ(value uint32) {
	volatile.StoreUint32(&o.STATE.Reg, volatile.LoadUint32(&o.STATE.Reg)&^(0x800)|value<<11)
}
func (o *POWMAN_Type) GetSTATE_BAD_HW_REQ() uint32 {
	return (volatile.LoadUint32(&o.STATE.Reg) & 0x800) >> 11
}
func (o *POWMAN_Type) SetSTATE_BAD_SW_REQ(value uint32) {
	volatile.StoreUint32(&o.STATE.Reg, volatile.LoadUint32(&o.STATE.Reg)&^(0x400)|value<<10)
}
func (o *POWMAN_Type) GetSTATE_BAD_SW_REQ() uint32 {
	return (volatile.LoadUint32(&o.STATE.Reg) & 0x400) >> 10
}
func (o *POWMAN_Type) SetSTATE_PWRUP_WHILE_WAITING(value uint32) {
	volatile.StoreUint32(&o.STATE.Reg, volatile.LoadUint32(&o.STATE.Reg)&^(0x200)|value<<9)
}
func (o *POWMAN_Type) GetSTATE_PWRUP_WHILE_WAITING() uint32 {
	return (volatile.LoadUint32(&o.STATE.Reg) & 0x200) >> 9
}
func (o *POWMAN_Type) SetSTATE_REQ_IGNORED(value uint32) {
	volatile.StoreUint32(&o.STATE.Reg, volatile.LoadUint32(&o.STATE.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetSTATE_REQ_IGNORED() uint32 {
	return (volatile.LoadUint32(&o.STATE.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetSTATE_REQ(value uint32) {
	volatile.StoreUint32(&o.STATE.Reg, volatile.LoadUint32(&o.STATE.Reg)&^(0xf0)|value<<4)
}
func (o *POWMAN_Type) GetSTATE_REQ() uint32 {
	return (volatile.LoadUint32(&o.STATE.Reg) & 0xf0) >> 4
}
func (o *POWMAN_Type) SetSTATE_CURRENT(value uint32) {
	volatile.StoreUint32(&o.STATE.Reg, volatile.LoadUint32(&o.STATE.Reg)&^(0xf)|value)
}
func (o *POWMAN_Type) GetSTATE_CURRENT() uint32 {
	return volatile.LoadUint32(&o.STATE.Reg) & 0xf
}

// POWMAN.POW_FASTDIV
func (o *POWMAN_Type) SetPOW_FASTDIV(value uint32) {
	volatile.StoreUint32(&o.POW_FASTDIV.Reg, volatile.LoadUint32(&o.POW_FASTDIV.Reg)&^(0x7ff)|value)
}
func (o *POWMAN_Type) GetPOW_FASTDIV() uint32 {
	return volatile.LoadUint32(&o.POW_FASTDIV.Reg) & 0x7ff
}

// POWMAN.POW_DELAY: power state machine delays
func (o *POWMAN_Type) SetPOW_DELAY_SRAM_STEP(value uint32) {
	volatile.StoreUint32(&o.POW_DELAY.Reg, volatile.LoadUint32(&o.POW_DELAY.Reg)&^(0xff00)|value<<8)
}
func (o *POWMAN_Type) GetPOW_DELAY_SRAM_STEP() uint32 {
	return (volatile.LoadUint32(&o.POW_DELAY.Reg) & 0xff00) >> 8
}
func (o *POWMAN_Type) SetPOW_DELAY_XIP_STEP(value uint32) {
	volatile.StoreUint32(&o.POW_DELAY.Reg, volatile.LoadUint32(&o.POW_DELAY.Reg)&^(0xf0)|value<<4)
}
func (o *POWMAN_Type) GetPOW_DELAY_XIP_STEP() uint32 {
	return (volatile.LoadUint32(&o.POW_DELAY.Reg) & 0xf0) >> 4
}
func (o *POWMAN_Type) SetPOW_DELAY_SWCORE_STEP(value uint32) {
	volatile.StoreUint32(&o.POW_DELAY.Reg, volatile.LoadUint32(&o.POW_DELAY.Reg)&^(0xf)|value)
}
func (o *POWMAN_Type) GetPOW_DELAY_SWCORE_STEP() uint32 {
	return volatile.LoadUint32(&o.POW_DELAY.Reg) & 0xf
}

// POWMAN.EXT_CTRL0: Configures a gpio as a power mode aware control output
func (o *POWMAN_Type) SetEXT_CTRL0_LP_EXIT_STATE(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL0.Reg, volatile.LoadUint32(&o.EXT_CTRL0.Reg)&^(0x4000)|value<<14)
}
func (o *POWMAN_Type) GetEXT_CTRL0_LP_EXIT_STATE() uint32 {
	return (volatile.LoadUint32(&o.EXT_CTRL0.Reg) & 0x4000) >> 14
}
func (o *POWMAN_Type) SetEXT_CTRL0_LP_ENTRY_STATE(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL0.Reg, volatile.LoadUint32(&o.EXT_CTRL0.Reg)&^(0x2000)|value<<13)
}
func (o *POWMAN_Type) GetEXT_CTRL0_LP_ENTRY_STATE() uint32 {
	return (volatile.LoadUint32(&o.EXT_CTRL0.Reg) & 0x2000) >> 13
}
func (o *POWMAN_Type) SetEXT_CTRL0_INIT_STATE(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL0.Reg, volatile.LoadUint32(&o.EXT_CTRL0.Reg)&^(0x1000)|value<<12)
}
func (o *POWMAN_Type) GetEXT_CTRL0_INIT_STATE() uint32 {
	return (volatile.LoadUint32(&o.EXT_CTRL0.Reg) & 0x1000) >> 12
}
func (o *POWMAN_Type) SetEXT_CTRL0_INIT(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL0.Reg, volatile.LoadUint32(&o.EXT_CTRL0.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetEXT_CTRL0_INIT() uint32 {
	return (volatile.LoadUint32(&o.EXT_CTRL0.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetEXT_CTRL0_GPIO_SELECT(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL0.Reg, volatile.LoadUint32(&o.EXT_CTRL0.Reg)&^(0x3f)|value)
}
func (o *POWMAN_Type) GetEXT_CTRL0_GPIO_SELECT() uint32 {
	return volatile.LoadUint32(&o.EXT_CTRL0.Reg) & 0x3f
}

// POWMAN.EXT_CTRL1: Configures a gpio as a power mode aware control output
func (o *POWMAN_Type) SetEXT_CTRL1_LP_EXIT_STATE(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL1.Reg, volatile.LoadUint32(&o.EXT_CTRL1.Reg)&^(0x4000)|value<<14)
}
func (o *POWMAN_Type) GetEXT_CTRL1_LP_EXIT_STATE() uint32 {
	return (volatile.LoadUint32(&o.EXT_CTRL1.Reg) & 0x4000) >> 14
}
func (o *POWMAN_Type) SetEXT_CTRL1_LP_ENTRY_STATE(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL1.Reg, volatile.LoadUint32(&o.EXT_CTRL1.Reg)&^(0x2000)|value<<13)
}
func (o *POWMAN_Type) GetEXT_CTRL1_LP_ENTRY_STATE() uint32 {
	return (volatile.LoadUint32(&o.EXT_CTRL1.Reg) & 0x2000) >> 13
}
func (o *POWMAN_Type) SetEXT_CTRL1_INIT_STATE(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL1.Reg, volatile.LoadUint32(&o.EXT_CTRL1.Reg)&^(0x1000)|value<<12)
}
func (o *POWMAN_Type) GetEXT_CTRL1_INIT_STATE() uint32 {
	return (volatile.LoadUint32(&o.EXT_CTRL1.Reg) & 0x1000) >> 12
}
func (o *POWMAN_Type) SetEXT_CTRL1_INIT(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL1.Reg, volatile.LoadUint32(&o.EXT_CTRL1.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetEXT_CTRL1_INIT() uint32 {
	return (volatile.LoadUint32(&o.EXT_CTRL1.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetEXT_CTRL1_GPIO_SELECT(value uint32) {
	volatile.StoreUint32(&o.EXT_CTRL1.Reg, volatile.LoadUint32(&o.EXT_CTRL1.Reg)&^(0x3f)|value)
}
func (o *POWMAN_Type) GetEXT_CTRL1_GPIO_SELECT() uint32 {
	return volatile.LoadUint32(&o.EXT_CTRL1.Reg) & 0x3f
}

// POWMAN.EXT_TIME_REF: Select a GPIO to use as a time reference, the source can be used to drive the low power clock at 32kHz, or to provide a 1ms tick to the timer, or provide a 1Hz tick to the timer. The tick selection is controlled by the POWMAN_TIMER register.
func (o *POWMAN_Type) SetEXT_TIME_REF_DRIVE_LPCK(value uint32) {
	volatile.StoreUint32(&o.EXT_TIME_REF.Reg, volatile.LoadUint32(&o.EXT_TIME_REF.Reg)&^(0x10)|value<<4)
}
func (o *POWMAN_Type) GetEXT_TIME_REF_DRIVE_LPCK() uint32 {
	return (volatile.LoadUint32(&o.EXT_TIME_REF.Reg) & 0x10) >> 4
}
func (o *POWMAN_Type) SetEXT_TIME_REF_SOURCE_SEL(value uint32) {
	volatile.StoreUint32(&o.EXT_TIME_REF.Reg, volatile.LoadUint32(&o.EXT_TIME_REF.Reg)&^(0x3)|value)
}
func (o *POWMAN_Type) GetEXT_TIME_REF_SOURCE_SEL() uint32 {
	return volatile.LoadUint32(&o.EXT_TIME_REF.Reg) & 0x3
}

// POWMAN.LPOSC_FREQ_KHZ_INT: Informs the AON Timer of the integer component of the clock frequency when running off the LPOSC.
func (o *POWMAN_Type) SetLPOSC_FREQ_KHZ_INT(value uint32) {
	volatile.StoreUint32(&o.LPOSC_FREQ_KHZ_INT.Reg, volatile.LoadUint32(&o.LPOSC_FREQ_KHZ_INT.Reg)&^(0x3f)|value)
}
func (o *POWMAN_Type) GetLPOSC_FREQ_KHZ_INT() uint32 {
	return volatile.LoadUint32(&o.LPOSC_FREQ_KHZ_INT.Reg) & 0x3f
}

// POWMAN.LPOSC_FREQ_KHZ_FRAC: Informs the AON Timer of the fractional component of the clock frequency when running off the LPOSC.
func (o *POWMAN_Type) SetLPOSC_FREQ_KHZ_FRAC(value uint32) {
	volatile.StoreUint32(&o.LPOSC_FREQ_KHZ_FRAC.Reg, volatile.LoadUint32(&o.LPOSC_FREQ_KHZ_FRAC.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetLPOSC_FREQ_KHZ_FRAC() uint32 {
	return volatile.LoadUint32(&o.LPOSC_FREQ_KHZ_FRAC.Reg) & 0xffff
}

// POWMAN.XOSC_FREQ_KHZ_INT: Informs the AON Timer of the integer component of the clock frequency when running off the XOSC.
func (o *POWMAN_Type) SetXOSC_FREQ_KHZ_INT(value uint32) {
	volatile.StoreUint32(&o.XOSC_FREQ_KHZ_INT.Reg, volatile.LoadUint32(&o.XOSC_FREQ_KHZ_INT.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetXOSC_FREQ_KHZ_INT() uint32 {
	return volatile.LoadUint32(&o.XOSC_FREQ_KHZ_INT.Reg) & 0xffff
}

// POWMAN.XOSC_FREQ_KHZ_FRAC: Informs the AON Timer of the fractional component of the clock frequency when running off the XOSC.
func (o *POWMAN_Type) SetXOSC_FREQ_KHZ_FRAC(value uint32) {
	volatile.StoreUint32(&o.XOSC_FREQ_KHZ_FRAC.Reg, volatile.LoadUint32(&o.XOSC_FREQ_KHZ_FRAC.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetXOSC_FREQ_KHZ_FRAC() uint32 {
	return volatile.LoadUint32(&o.XOSC_FREQ_KHZ_FRAC.Reg) & 0xffff
}

// POWMAN.SET_TIME_63TO48
func (o *POWMAN_Type) SetSET_TIME_63TO48(value uint32) {
	volatile.StoreUint32(&o.SET_TIME_63TO48.Reg, volatile.LoadUint32(&o.SET_TIME_63TO48.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetSET_TIME_63TO48() uint32 {
	return volatile.LoadUint32(&o.SET_TIME_63TO48.Reg) & 0xffff
}

// POWMAN.SET_TIME_47TO32
func (o *POWMAN_Type) SetSET_TIME_47TO32(value uint32) {
	volatile.StoreUint32(&o.SET_TIME_47TO32.Reg, volatile.LoadUint32(&o.SET_TIME_47TO32.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetSET_TIME_47TO32() uint32 {
	return volatile.LoadUint32(&o.SET_TIME_47TO32.Reg) & 0xffff
}

// POWMAN.SET_TIME_31TO16
func (o *POWMAN_Type) SetSET_TIME_31TO16(value uint32) {
	volatile.StoreUint32(&o.SET_TIME_31TO16.Reg, volatile.LoadUint32(&o.SET_TIME_31TO16.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetSET_TIME_31TO16() uint32 {
	return volatile.LoadUint32(&o.SET_TIME_31TO16.Reg) & 0xffff
}

// POWMAN.SET_TIME_15TO0
func (o *POWMAN_Type) SetSET_TIME_15TO0(value uint32) {
	volatile.StoreUint32(&o.SET_TIME_15TO0.Reg, volatile.LoadUint32(&o.SET_TIME_15TO0.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetSET_TIME_15TO0() uint32 {
	return volatile.LoadUint32(&o.SET_TIME_15TO0.Reg) & 0xffff
}

// POWMAN.READ_TIME_UPPER
func (o *POWMAN_Type) SetREAD_TIME_UPPER(value uint32) {
	volatile.StoreUint32(&o.READ_TIME_UPPER.Reg, value)
}
func (o *POWMAN_Type) GetREAD_TIME_UPPER() uint32 {
	return volatile.LoadUint32(&o.READ_TIME_UPPER.Reg)
}

// POWMAN.READ_TIME_LOWER
func (o *POWMAN_Type) SetREAD_TIME_LOWER(value uint32) {
	volatile.StoreUint32(&o.READ_TIME_LOWER.Reg, value)
}
func (o *POWMAN_Type) GetREAD_TIME_LOWER() uint32 {
	return volatile.LoadUint32(&o.READ_TIME_LOWER.Reg)
}

// POWMAN.ALARM_TIME_63TO48
func (o *POWMAN_Type) SetALARM_TIME_63TO48(value uint32) {
	volatile.StoreUint32(&o.ALARM_TIME_63TO48.Reg, volatile.LoadUint32(&o.ALARM_TIME_63TO48.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetALARM_TIME_63TO48() uint32 {
	return volatile.LoadUint32(&o.ALARM_TIME_63TO48.Reg) & 0xffff
}

// POWMAN.ALARM_TIME_47TO32
func (o *POWMAN_Type) SetALARM_TIME_47TO32(value uint32) {
	volatile.StoreUint32(&o.ALARM_TIME_47TO32.Reg, volatile.LoadUint32(&o.ALARM_TIME_47TO32.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetALARM_TIME_47TO32() uint32 {
	return volatile.LoadUint32(&o.ALARM_TIME_47TO32.Reg) & 0xffff
}

// POWMAN.ALARM_TIME_31TO16
func (o *POWMAN_Type) SetALARM_TIME_31TO16(value uint32) {
	volatile.StoreUint32(&o.ALARM_TIME_31TO16.Reg, volatile.LoadUint32(&o.ALARM_TIME_31TO16.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetALARM_TIME_31TO16() uint32 {
	return volatile.LoadUint32(&o.ALARM_TIME_31TO16.Reg) & 0xffff
}

// POWMAN.ALARM_TIME_15TO0
func (o *POWMAN_Type) SetALARM_TIME_15TO0(value uint32) {
	volatile.StoreUint32(&o.ALARM_TIME_15TO0.Reg, volatile.LoadUint32(&o.ALARM_TIME_15TO0.Reg)&^(0xffff)|value)
}
func (o *POWMAN_Type) GetALARM_TIME_15TO0() uint32 {
	return volatile.LoadUint32(&o.ALARM_TIME_15TO0.Reg) & 0xffff
}

// POWMAN.TIMER
func (o *POWMAN_Type) SetTIMER_USING_GPIO_1HZ(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x80000)|value<<19)
}
func (o *POWMAN_Type) GetTIMER_USING_GPIO_1HZ() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x80000) >> 19
}
func (o *POWMAN_Type) SetTIMER_USING_GPIO_1KHZ(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x40000)|value<<18)
}
func (o *POWMAN_Type) GetTIMER_USING_GPIO_1KHZ() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x40000) >> 18
}
func (o *POWMAN_Type) SetTIMER_USING_LPOSC(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x20000)|value<<17)
}
func (o *POWMAN_Type) GetTIMER_USING_LPOSC() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x20000) >> 17
}
func (o *POWMAN_Type) SetTIMER_USING_XOSC(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x10000)|value<<16)
}
func (o *POWMAN_Type) GetTIMER_USING_XOSC() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x10000) >> 16
}
func (o *POWMAN_Type) SetTIMER_USE_GPIO_1HZ(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x2000)|value<<13)
}
func (o *POWMAN_Type) GetTIMER_USE_GPIO_1HZ() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x2000) >> 13
}
func (o *POWMAN_Type) SetTIMER_USE_GPIO_1KHZ(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x400)|value<<10)
}
func (o *POWMAN_Type) GetTIMER_USE_GPIO_1KHZ() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x400) >> 10
}
func (o *POWMAN_Type) SetTIMER_USE_XOSC(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x200)|value<<9)
}
func (o *POWMAN_Type) GetTIMER_USE_XOSC() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x200) >> 9
}
func (o *POWMAN_Type) SetTIMER_USE_LPOSC(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetTIMER_USE_LPOSC() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetTIMER_ALARM(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x40)|value<<6)
}
func (o *POWMAN_Type) GetTIMER_ALARM() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x40) >> 6
}
func (o *POWMAN_Type) SetTIMER_PWRUP_ON_ALARM(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x20)|value<<5)
}
func (o *POWMAN_Type) GetTIMER_PWRUP_ON_ALARM() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x20) >> 5
}
func (o *POWMAN_Type) SetTIMER_ALARM_ENAB(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x10)|value<<4)
}
func (o *POWMAN_Type) GetTIMER_ALARM_ENAB() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x10) >> 4
}
func (o *POWMAN_Type) SetTIMER_CLEAR(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x4)|value<<2)
}
func (o *POWMAN_Type) GetTIMER_CLEAR() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x4) >> 2
}
func (o *POWMAN_Type) SetTIMER_RUN(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetTIMER_RUN() uint32 {
	return (volatile.LoadUint32(&o.TIMER.Reg) & 0x2) >> 1
}
func (o *POWMAN_Type) SetTIMER_NONSEC_WRITE(value uint32) {
	volatile.StoreUint32(&o.TIMER.Reg, volatile.LoadUint32(&o.TIMER.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetTIMER_NONSEC_WRITE() uint32 {
	return volatile.LoadUint32(&o.TIMER.Reg) & 0x1
}

// POWMAN.PWRUP0: 4 GPIO powerup events can be configured to wake the chip up from a low power state. The pwrups are level/edge sensitive and can be set to trigger on a high/rising or low/falling event The number of gpios available depends on the package option. An invalid selection will be ignored source = 0 selects gpio0 . . source = 47 selects gpio47 source = 48 selects qspi_ss source = 49 selects qspi_sd0 source = 50 selects qspi_sd1 source = 51 selects qspi_sd2 source = 52 selects qspi_sd3 source = 53 selects qspi_sclk level = 0 triggers the pwrup when the source is low level = 1 triggers the pwrup when the source is high
func (o *POWMAN_Type) SetPWRUP0_RAW_STATUS(value uint32) {
	volatile.StoreUint32(&o.PWRUP0.Reg, volatile.LoadUint32(&o.PWRUP0.Reg)&^(0x400)|value<<10)
}
func (o *POWMAN_Type) GetPWRUP0_RAW_STATUS() uint32 {
	return (volatile.LoadUint32(&o.PWRUP0.Reg) & 0x400) >> 10
}
func (o *POWMAN_Type) SetPWRUP0_STATUS(value uint32) {
	volatile.StoreUint32(&o.PWRUP0.Reg, volatile.LoadUint32(&o.PWRUP0.Reg)&^(0x200)|value<<9)
}
func (o *POWMAN_Type) GetPWRUP0_STATUS() uint32 {
	return (volatile.LoadUint32(&o.PWRUP0.Reg) & 0x200) >> 9
}
func (o *POWMAN_Type) SetPWRUP0_MODE(value uint32) {
	volatile.StoreUint32(&o.PWRUP0.Reg, volatile.LoadUint32(&o.PWRUP0.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetPWRUP0_MODE() uint32 {
	return (volatile.LoadUint32(&o.PWRUP0.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetPWRUP0_DIRECTION(value uint32) {
	volatile.StoreUint32(&o.PWRUP0.Reg, volatile.LoadUint32(&o.PWRUP0.Reg)&^(0x80)|value<<7)
}
func (o *POWMAN_Type) GetPWRUP0_DIRECTION() uint32 {
	return (volatile.LoadUint32(&o.PWRUP0.Reg) & 0x80) >> 7
}
func (o *POWMAN_Type) SetPWRUP0_ENABLE(value uint32) {
	volatile.StoreUint32(&o.PWRUP0.Reg, volatile.LoadUint32(&o.PWRUP0.Reg)&^(0x40)|value<<6)
}
func (o *POWMAN_Type) GetPWRUP0_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.PWRUP0.Reg) & 0x40) >> 6
}
func (o *POWMAN_Type) SetPWRUP0_SOURCE(value uint32) {
	volatile.StoreUint32(&o.PWRUP0.Reg, volatile.LoadUint32(&o.PWRUP0.Reg)&^(0x3f)|value)
}
func (o *POWMAN_Type) GetPWRUP0_SOURCE() uint32 {
	return volatile.LoadUint32(&o.PWRUP0.Reg) & 0x3f
}

// POWMAN.PWRUP1: 4 GPIO powerup events can be configured to wake the chip up from a low power state. The pwrups are level/edge sensitive and can be set to trigger on a high/rising or low/falling event The number of gpios available depends on the package option. An invalid selection will be ignored source = 0 selects gpio0 . . source = 47 selects gpio47 source = 48 selects qspi_ss source = 49 selects qspi_sd0 source = 50 selects qspi_sd1 source = 51 selects qspi_sd2 source = 52 selects qspi_sd3 source = 53 selects qspi_sclk level = 0 triggers the pwrup when the source is low level = 1 triggers the pwrup when the source is high
func (o *POWMAN_Type) SetPWRUP1_RAW_STATUS(value uint32) {
	volatile.StoreUint32(&o.PWRUP1.Reg, volatile.LoadUint32(&o.PWRUP1.Reg)&^(0x400)|value<<10)
}
func (o *POWMAN_Type) GetPWRUP1_RAW_STATUS() uint32 {
	return (volatile.LoadUint32(&o.PWRUP1.Reg) & 0x400) >> 10
}
func (o *POWMAN_Type) SetPWRUP1_STATUS(value uint32) {
	volatile.StoreUint32(&o.PWRUP1.Reg, volatile.LoadUint32(&o.PWRUP1.Reg)&^(0x200)|value<<9)
}
func (o *POWMAN_Type) GetPWRUP1_STATUS() uint32 {
	return (volatile.LoadUint32(&o.PWRUP1.Reg) & 0x200) >> 9
}
func (o *POWMAN_Type) SetPWRUP1_MODE(value uint32) {
	volatile.StoreUint32(&o.PWRUP1.Reg, volatile.LoadUint32(&o.PWRUP1.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetPWRUP1_MODE() uint32 {
	return (volatile.LoadUint32(&o.PWRUP1.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetPWRUP1_DIRECTION(value uint32) {
	volatile.StoreUint32(&o.PWRUP1.Reg, volatile.LoadUint32(&o.PWRUP1.Reg)&^(0x80)|value<<7)
}
func (o *POWMAN_Type) GetPWRUP1_DIRECTION() uint32 {
	return (volatile.LoadUint32(&o.PWRUP1.Reg) & 0x80) >> 7
}
func (o *POWMAN_Type) SetPWRUP1_ENABLE(value uint32) {
	volatile.StoreUint32(&o.PWRUP1.Reg, volatile.LoadUint32(&o.PWRUP1.Reg)&^(0x40)|value<<6)
}
func (o *POWMAN_Type) GetPWRUP1_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.PWRUP1.Reg) & 0x40) >> 6
}
func (o *POWMAN_Type) SetPWRUP1_SOURCE(value uint32) {
	volatile.StoreUint32(&o.PWRUP1.Reg, volatile.LoadUint32(&o.PWRUP1.Reg)&^(0x3f)|value)
}
func (o *POWMAN_Type) GetPWRUP1_SOURCE() uint32 {
	return volatile.LoadUint32(&o.PWRUP1.Reg) & 0x3f
}

// POWMAN.PWRUP2: 4 GPIO powerup events can be configured to wake the chip up from a low power state. The pwrups are level/edge sensitive and can be set to trigger on a high/rising or low/falling event The number of gpios available depends on the package option. An invalid selection will be ignored source = 0 selects gpio0 . . source = 47 selects gpio47 source = 48 selects qspi_ss source = 49 selects qspi_sd0 source = 50 selects qspi_sd1 source = 51 selects qspi_sd2 source = 52 selects qspi_sd3 source = 53 selects qspi_sclk level = 0 triggers the pwrup when the source is low level = 1 triggers the pwrup when the source is high
func (o *POWMAN_Type) SetPWRUP2_RAW_STATUS(value uint32) {
	volatile.StoreUint32(&o.PWRUP2.Reg, volatile.LoadUint32(&o.PWRUP2.Reg)&^(0x400)|value<<10)
}
func (o *POWMAN_Type) GetPWRUP2_RAW_STATUS() uint32 {
	return (volatile.LoadUint32(&o.PWRUP2.Reg) & 0x400) >> 10
}
func (o *POWMAN_Type) SetPWRUP2_STATUS(value uint32) {
	volatile.StoreUint32(&o.PWRUP2.Reg, volatile.LoadUint32(&o.PWRUP2.Reg)&^(0x200)|value<<9)
}
func (o *POWMAN_Type) GetPWRUP2_STATUS() uint32 {
	return (volatile.LoadUint32(&o.PWRUP2.Reg) & 0x200) >> 9
}
func (o *POWMAN_Type) SetPWRUP2_MODE(value uint32) {
	volatile.StoreUint32(&o.PWRUP2.Reg, volatile.LoadUint32(&o.PWRUP2.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetPWRUP2_MODE() uint32 {
	return (volatile.LoadUint32(&o.PWRUP2.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetPWRUP2_DIRECTION(value uint32) {
	volatile.StoreUint32(&o.PWRUP2.Reg, volatile.LoadUint32(&o.PWRUP2.Reg)&^(0x80)|value<<7)
}
func (o *POWMAN_Type) GetPWRUP2_DIRECTION() uint32 {
	return (volatile.LoadUint32(&o.PWRUP2.Reg) & 0x80) >> 7
}
func (o *POWMAN_Type) SetPWRUP2_ENABLE(value uint32) {
	volatile.StoreUint32(&o.PWRUP2.Reg, volatile.LoadUint32(&o.PWRUP2.Reg)&^(0x40)|value<<6)
}
func (o *POWMAN_Type) GetPWRUP2_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.PWRUP2.Reg) & 0x40) >> 6
}
func (o *POWMAN_Type) SetPWRUP2_SOURCE(value uint32) {
	volatile.StoreUint32(&o.PWRUP2.Reg, volatile.LoadUint32(&o.PWRUP2.Reg)&^(0x3f)|value)
}
func (o *POWMAN_Type) GetPWRUP2_SOURCE() uint32 {
	return volatile.LoadUint32(&o.PWRUP2.Reg) & 0x3f
}

// POWMAN.PWRUP3: 4 GPIO powerup events can be configured to wake the chip up from a low power state. The pwrups are level/edge sensitive and can be set to trigger on a high/rising or low/falling event The number of gpios available depends on the package option. An invalid selection will be ignored source = 0 selects gpio0 . . source = 47 selects gpio47 source = 48 selects qspi_ss source = 49 selects qspi_sd0 source = 50 selects qspi_sd1 source = 51 selects qspi_sd2 source = 52 selects qspi_sd3 source = 53 selects qspi_sclk level = 0 triggers the pwrup when the source is low level = 1 triggers the pwrup when the source is high
func (o *POWMAN_Type) SetPWRUP3_RAW_STATUS(value uint32) {
	volatile.StoreUint32(&o.PWRUP3.Reg, volatile.LoadUint32(&o.PWRUP3.Reg)&^(0x400)|value<<10)
}
func (o *POWMAN_Type) GetPWRUP3_RAW_STATUS() uint32 {
	return (volatile.LoadUint32(&o.PWRUP3.Reg) & 0x400) >> 10
}
func (o *POWMAN_Type) SetPWRUP3_STATUS(value uint32) {
	volatile.StoreUint32(&o.PWRUP3.Reg, volatile.LoadUint32(&o.PWRUP3.Reg)&^(0x200)|value<<9)
}
func (o *POWMAN_Type) GetPWRUP3_STATUS() uint32 {
	return (volatile.LoadUint32(&o.PWRUP3.Reg) & 0x200) >> 9
}
func (o *POWMAN_Type) SetPWRUP3_MODE(value uint32) {
	volatile.StoreUint32(&o.PWRUP3.Reg, volatile.LoadUint32(&o.PWRUP3.Reg)&^(0x100)|value<<8)
}
func (o *POWMAN_Type) GetPWRUP3_MODE() uint32 {
	return (volatile.LoadUint32(&o.PWRUP3.Reg) & 0x100) >> 8
}
func (o *POWMAN_Type) SetPWRUP3_DIRECTION(value uint32) {
	volatile.StoreUint32(&o.PWRUP3.Reg, volatile.LoadUint32(&o.PWRUP3.Reg)&^(0x80)|value<<7)
}
func (o *POWMAN_Type) GetPWRUP3_DIRECTION() uint32 {
	return (volatile.LoadUint32(&o.PWRUP3.Reg) & 0x80) >> 7
}
func (o *POWMAN_Type) SetPWRUP3_ENABLE(value uint32) {
	volatile.StoreUint32(&o.PWRUP3.Reg, volatile.LoadUint32(&o.PWRUP3.Reg)&^(0x40)|value<<6)
}
func (o *POWMAN_Type) GetPWRUP3_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.PWRUP3.Reg) & 0x40) >> 6
}
func (o *POWMAN_Type) SetPWRUP3_SOURCE(value uint32) {
	volatile.StoreUint32(&o.PWRUP3.Reg, volatile.LoadUint32(&o.PWRUP3.Reg)&^(0x3f)|value)
}
func (o *POWMAN_Type) GetPWRUP3_SOURCE() uint32 {
	return volatile.LoadUint32(&o.PWRUP3.Reg) & 0x3f
}

// POWMAN.CURRENT_PWRUP_REQ: Indicates current powerup request state pwrup events can be cleared by removing the enable from the pwrup register. The alarm pwrup req can be cleared by clearing timer.alarm_enab 0 = chip reset, for the source of the last reset see POWMAN_CHIP_RESET 1 = pwrup0 2 = pwrup1 3 = pwrup2 4 = pwrup3 5 = coresight_pwrup 6 = alarm_pwrup
func (o *POWMAN_Type) SetCURRENT_PWRUP_REQ(value uint32) {
	volatile.StoreUint32(&o.CURRENT_PWRUP_REQ.Reg, volatile.LoadUint32(&o.CURRENT_PWRUP_REQ.Reg)&^(0x7f)|value)
}
func (o *POWMAN_Type) GetCURRENT_PWRUP_REQ() uint32 {
	return volatile.LoadUint32(&o.CURRENT_PWRUP_REQ.Reg) & 0x7f
}

// POWMAN.LAST_SWCORE_PWRUP: Indicates which pwrup source triggered the last switched-core power up 0 = chip reset, for the source of the last reset see POWMAN_CHIP_RESET 1 = pwrup0 2 = pwrup1 3 = pwrup2 4 = pwrup3 5 = coresight_pwrup 6 = alarm_pwrup
func (o *POWMAN_Type) SetLAST_SWCORE_PWRUP(value uint32) {
	volatile.StoreUint32(&o.LAST_SWCORE_PWRUP.Reg, volatile.LoadUint32(&o.LAST_SWCORE_PWRUP.Reg)&^(0x7f)|value)
}
func (o *POWMAN_Type) GetLAST_SWCORE_PWRUP() uint32 {
	return volatile.LoadUint32(&o.LAST_SWCORE_PWRUP.Reg) & 0x7f
}

// POWMAN.DBG_PWRCFG
func (o *POWMAN_Type) SetDBG_PWRCFG_IGNORE(value uint32) {
	volatile.StoreUint32(&o.DBG_PWRCFG.Reg, volatile.LoadUint32(&o.DBG_PWRCFG.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetDBG_PWRCFG_IGNORE() uint32 {
	return volatile.LoadUint32(&o.DBG_PWRCFG.Reg) & 0x1
}

// POWMAN.BOOTDIS: Tell the bootrom to ignore the BOOT0..3 registers following the next RSM reset (e.g. the next core power down/up). If an early boot stage has soft-locked some OTP pages in order to protect their contents from later stages, there is a risk that Secure code running at a later stage can unlock the pages by powering the core up and down. This register can be used to ensure that the bootloader runs as normal on the next power up, preventing Secure code at a later stage from accessing OTP in its unlocked state. Should be used in conjunction with the OTP BOOTDIS register.
func (o *POWMAN_Type) SetBOOTDIS_NEXT(value uint32) {
	volatile.StoreUint32(&o.BOOTDIS.Reg, volatile.LoadUint32(&o.BOOTDIS.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetBOOTDIS_NEXT() uint32 {
	return (volatile.LoadUint32(&o.BOOTDIS.Reg) & 0x2) >> 1
}
func (o *POWMAN_Type) SetBOOTDIS_NOW(value uint32) {
	volatile.StoreUint32(&o.BOOTDIS.Reg, volatile.LoadUint32(&o.BOOTDIS.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetBOOTDIS_NOW() uint32 {
	return volatile.LoadUint32(&o.BOOTDIS.Reg) & 0x1
}

// POWMAN.DBGCONFIG
func (o *POWMAN_Type) SetDBGCONFIG_DP_INSTID(value uint32) {
	volatile.StoreUint32(&o.DBGCONFIG.Reg, volatile.LoadUint32(&o.DBGCONFIG.Reg)&^(0xf)|value)
}
func (o *POWMAN_Type) GetDBGCONFIG_DP_INSTID() uint32 {
	return volatile.LoadUint32(&o.DBGCONFIG.Reg) & 0xf
}

// POWMAN.SCRATCH0: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetSCRATCH0(value uint32) {
	volatile.StoreUint32(&o.SCRATCH0.Reg, value)
}
func (o *POWMAN_Type) GetSCRATCH0() uint32 {
	return volatile.LoadUint32(&o.SCRATCH0.Reg)
}

// POWMAN.SCRATCH1: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetSCRATCH1(value uint32) {
	volatile.StoreUint32(&o.SCRATCH1.Reg, value)
}
func (o *POWMAN_Type) GetSCRATCH1() uint32 {
	return volatile.LoadUint32(&o.SCRATCH1.Reg)
}

// POWMAN.SCRATCH2: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetSCRATCH2(value uint32) {
	volatile.StoreUint32(&o.SCRATCH2.Reg, value)
}
func (o *POWMAN_Type) GetSCRATCH2() uint32 {
	return volatile.LoadUint32(&o.SCRATCH2.Reg)
}

// POWMAN.SCRATCH3: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetSCRATCH3(value uint32) {
	volatile.StoreUint32(&o.SCRATCH3.Reg, value)
}
func (o *POWMAN_Type) GetSCRATCH3() uint32 {
	return volatile.LoadUint32(&o.SCRATCH3.Reg)
}

// POWMAN.SCRATCH4: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetSCRATCH4(value uint32) {
	volatile.StoreUint32(&o.SCRATCH4.Reg, value)
}
func (o *POWMAN_Type) GetSCRATCH4() uint32 {
	return volatile.LoadUint32(&o.SCRATCH4.Reg)
}

// POWMAN.SCRATCH5: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetSCRATCH5(value uint32) {
	volatile.StoreUint32(&o.SCRATCH5.Reg, value)
}
func (o *POWMAN_Type) GetSCRATCH5() uint32 {
	return volatile.LoadUint32(&o.SCRATCH5.Reg)
}

// POWMAN.SCRATCH6: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetSCRATCH6(value uint32) {
	volatile.StoreUint32(&o.SCRATCH6.Reg, value)
}
func (o *POWMAN_Type) GetSCRATCH6() uint32 {
	return volatile.LoadUint32(&o.SCRATCH6.Reg)
}

// POWMAN.SCRATCH7: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetSCRATCH7(value uint32) {
	volatile.StoreUint32(&o.SCRATCH7.Reg, value)
}
func (o *POWMAN_Type) GetSCRATCH7() uint32 {
	return volatile.LoadUint32(&o.SCRATCH7.Reg)
}

// POWMAN.BOOT0: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetBOOT0(value uint32) {
	volatile.StoreUint32(&o.BOOT0.Reg, value)
}
func (o *POWMAN_Type) GetBOOT0() uint32 {
	return volatile.LoadUint32(&o.BOOT0.Reg)
}

// POWMAN.BOOT1: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetBOOT1(value uint32) {
	volatile.StoreUint32(&o.BOOT1.Reg, value)
}
func (o *POWMAN_Type) GetBOOT1() uint32 {
	return volatile.LoadUint32(&o.BOOT1.Reg)
}

// POWMAN.BOOT2: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetBOOT2(value uint32) {
	volatile.StoreUint32(&o.BOOT2.Reg, value)
}
func (o *POWMAN_Type) GetBOOT2() uint32 {
	return volatile.LoadUint32(&o.BOOT2.Reg)
}

// POWMAN.BOOT3: Scratch register. Information persists in low power mode
func (o *POWMAN_Type) SetBOOT3(value uint32) {
	volatile.StoreUint32(&o.BOOT3.Reg, value)
}
func (o *POWMAN_Type) GetBOOT3() uint32 {
	return volatile.LoadUint32(&o.BOOT3.Reg)
}

// POWMAN.INTR: Raw Interrupts
func (o *POWMAN_Type) SetINTR_PWRUP_WHILE_WAITING(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8)|value<<3)
}
func (o *POWMAN_Type) GetINTR_PWRUP_WHILE_WAITING() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8) >> 3
}
func (o *POWMAN_Type) SetINTR_STATE_REQ_IGNORED(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4)|value<<2)
}
func (o *POWMAN_Type) GetINTR_STATE_REQ_IGNORED() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4) >> 2
}
func (o *POWMAN_Type) SetINTR_TIMER(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetINTR_TIMER() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2) >> 1
}
func (o *POWMAN_Type) SetINTR_VREG_OUTPUT_LOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetINTR_VREG_OUTPUT_LOW() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// POWMAN.INTE: Interrupt Enable
func (o *POWMAN_Type) SetINTE_PWRUP_WHILE_WAITING(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x8)|value<<3)
}
func (o *POWMAN_Type) GetINTE_PWRUP_WHILE_WAITING() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x8) >> 3
}
func (o *POWMAN_Type) SetINTE_STATE_REQ_IGNORED(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x4)|value<<2)
}
func (o *POWMAN_Type) GetINTE_STATE_REQ_IGNORED() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x4) >> 2
}
func (o *POWMAN_Type) SetINTE_TIMER(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetINTE_TIMER() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x2) >> 1
}
func (o *POWMAN_Type) SetINTE_VREG_OUTPUT_LOW(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetINTE_VREG_OUTPUT_LOW() uint32 {
	return volatile.LoadUint32(&o.INTE.Reg) & 0x1
}

// POWMAN.INTF: Interrupt Force
func (o *POWMAN_Type) SetINTF_PWRUP_WHILE_WAITING(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x8)|value<<3)
}
func (o *POWMAN_Type) GetINTF_PWRUP_WHILE_WAITING() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x8) >> 3
}
func (o *POWMAN_Type) SetINTF_STATE_REQ_IGNORED(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x4)|value<<2)
}
func (o *POWMAN_Type) GetINTF_STATE_REQ_IGNORED() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x4) >> 2
}
func (o *POWMAN_Type) SetINTF_TIMER(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetINTF_TIMER() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x2) >> 1
}
func (o *POWMAN_Type) SetINTF_VREG_OUTPUT_LOW(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetINTF_VREG_OUTPUT_LOW() uint32 {
	return volatile.LoadUint32(&o.INTF.Reg) & 0x1
}

// POWMAN.INTS: Interrupt status after masking & forcing
func (o *POWMAN_Type) SetINTS_PWRUP_WHILE_WAITING(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x8)|value<<3)
}
func (o *POWMAN_Type) GetINTS_PWRUP_WHILE_WAITING() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x8) >> 3
}
func (o *POWMAN_Type) SetINTS_STATE_REQ_IGNORED(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x4)|value<<2)
}
func (o *POWMAN_Type) GetINTS_STATE_REQ_IGNORED() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x4) >> 2
}
func (o *POWMAN_Type) SetINTS_TIMER(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x2)|value<<1)
}
func (o *POWMAN_Type) GetINTS_TIMER() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x2) >> 1
}
func (o *POWMAN_Type) SetINTS_VREG_OUTPUT_LOW(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x1)|value)
}
func (o *POWMAN_Type) GetINTS_VREG_OUTPUT_LOW() uint32 {
	return volatile.LoadUint32(&o.INTS.Reg) & 0x1
}

type WATCHDOG_Type struct {
	CTRL     volatile.Register32 // 0x0
	LOAD     volatile.Register32 // 0x4
	REASON   volatile.Register32 // 0x8
	SCRATCH0 volatile.Register32 // 0xC
	SCRATCH1 volatile.Register32 // 0x10
	SCRATCH2 volatile.Register32 // 0x14
	SCRATCH3 volatile.Register32 // 0x18
	SCRATCH4 volatile.Register32 // 0x1C
	SCRATCH5 volatile.Register32 // 0x20
	SCRATCH6 volatile.Register32 // 0x24
	SCRATCH7 volatile.Register32 // 0x28
}

// WATCHDOG.CTRL: Watchdog control The rst_wdsel register determines which subsystems are reset when the watchdog is triggered. The watchdog can be triggered in software.
func (o *WATCHDOG_Type) SetCTRL_TRIGGER(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *WATCHDOG_Type) GetCTRL_TRIGGER() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x80000000) >> 31
}
func (o *WATCHDOG_Type) SetCTRL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *WATCHDOG_Type) GetCTRL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x40000000) >> 30
}
func (o *WATCHDOG_Type) SetCTRL_PAUSE_DBG1(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x4000000)|value<<26)
}
func (o *WATCHDOG_Type) GetCTRL_PAUSE_DBG1() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x4000000) >> 26
}
func (o *WATCHDOG_Type) SetCTRL_PAUSE_DBG0(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x2000000)|value<<25)
}
func (o *WATCHDOG_Type) GetCTRL_PAUSE_DBG0() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x2000000) >> 25
}
func (o *WATCHDOG_Type) SetCTRL_PAUSE_JTAG(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x1000000)|value<<24)
}
func (o *WATCHDOG_Type) GetCTRL_PAUSE_JTAG() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x1000000) >> 24
}
func (o *WATCHDOG_Type) SetCTRL_TIME(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xffffff)|value)
}
func (o *WATCHDOG_Type) GetCTRL_TIME() uint32 {
	return volatile.LoadUint32(&o.CTRL.Reg) & 0xffffff
}

// WATCHDOG.LOAD: Load the watchdog timer. The maximum setting is 0xffffff which corresponds to approximately 16 seconds.
func (o *WATCHDOG_Type) SetLOAD(value uint32) {
	volatile.StoreUint32(&o.LOAD.Reg, volatile.LoadUint32(&o.LOAD.Reg)&^(0xffffff)|value)
}
func (o *WATCHDOG_Type) GetLOAD() uint32 {
	return volatile.LoadUint32(&o.LOAD.Reg) & 0xffffff
}

// WATCHDOG.REASON: Logs the reason for the last reset. Both bits are zero for the case of a hardware reset. Additionally, as of RP2350, a debugger warm reset of either core (SYSRESETREQ or hartreset) will also clear the watchdog reason register, so that software loaded under the debugger following a watchdog timeout will not continue to see the timeout condition.
func (o *WATCHDOG_Type) SetREASON_FORCE(value uint32) {
	volatile.StoreUint32(&o.REASON.Reg, volatile.LoadUint32(&o.REASON.Reg)&^(0x2)|value<<1)
}
func (o *WATCHDOG_Type) GetREASON_FORCE() uint32 {
	return (volatile.LoadUint32(&o.REASON.Reg) & 0x2) >> 1
}
func (o *WATCHDOG_Type) SetREASON_TIMER(value uint32) {
	volatile.StoreUint32(&o.REASON.Reg, volatile.LoadUint32(&o.REASON.Reg)&^(0x1)|value)
}
func (o *WATCHDOG_Type) GetREASON_TIMER() uint32 {
	return volatile.LoadUint32(&o.REASON.Reg) & 0x1
}

// WATCHDOG.SCRATCH0: Scratch register. Information persists through soft reset of the chip.
func (o *WATCHDOG_Type) SetSCRATCH0(value uint32) {
	volatile.StoreUint32(&o.SCRATCH0.Reg, value)
}
func (o *WATCHDOG_Type) GetSCRATCH0() uint32 {
	return volatile.LoadUint32(&o.SCRATCH0.Reg)
}

// WATCHDOG.SCRATCH1: Scratch register. Information persists through soft reset of the chip.
func (o *WATCHDOG_Type) SetSCRATCH1(value uint32) {
	volatile.StoreUint32(&o.SCRATCH1.Reg, value)
}
func (o *WATCHDOG_Type) GetSCRATCH1() uint32 {
	return volatile.LoadUint32(&o.SCRATCH1.Reg)
}

// WATCHDOG.SCRATCH2: Scratch register. Information persists through soft reset of the chip.
func (o *WATCHDOG_Type) SetSCRATCH2(value uint32) {
	volatile.StoreUint32(&o.SCRATCH2.Reg, value)
}
func (o *WATCHDOG_Type) GetSCRATCH2() uint32 {
	return volatile.LoadUint32(&o.SCRATCH2.Reg)
}

// WATCHDOG.SCRATCH3: Scratch register. Information persists through soft reset of the chip.
func (o *WATCHDOG_Type) SetSCRATCH3(value uint32) {
	volatile.StoreUint32(&o.SCRATCH3.Reg, value)
}
func (o *WATCHDOG_Type) GetSCRATCH3() uint32 {
	return volatile.LoadUint32(&o.SCRATCH3.Reg)
}

// WATCHDOG.SCRATCH4: Scratch register. Information persists through soft reset of the chip.
func (o *WATCHDOG_Type) SetSCRATCH4(value uint32) {
	volatile.StoreUint32(&o.SCRATCH4.Reg, value)
}
func (o *WATCHDOG_Type) GetSCRATCH4() uint32 {
	return volatile.LoadUint32(&o.SCRATCH4.Reg)
}

// WATCHDOG.SCRATCH5: Scratch register. Information persists through soft reset of the chip.
func (o *WATCHDOG_Type) SetSCRATCH5(value uint32) {
	volatile.StoreUint32(&o.SCRATCH5.Reg, value)
}
func (o *WATCHDOG_Type) GetSCRATCH5() uint32 {
	return volatile.LoadUint32(&o.SCRATCH5.Reg)
}

// WATCHDOG.SCRATCH6: Scratch register. Information persists through soft reset of the chip.
func (o *WATCHDOG_Type) SetSCRATCH6(value uint32) {
	volatile.StoreUint32(&o.SCRATCH6.Reg, value)
}
func (o *WATCHDOG_Type) GetSCRATCH6() uint32 {
	return volatile.LoadUint32(&o.SCRATCH6.Reg)
}

// WATCHDOG.SCRATCH7: Scratch register. Information persists through soft reset of the chip.
func (o *WATCHDOG_Type) SetSCRATCH7(value uint32) {
	volatile.StoreUint32(&o.SCRATCH7.Reg, value)
}
func (o *WATCHDOG_Type) GetSCRATCH7() uint32 {
	return volatile.LoadUint32(&o.SCRATCH7.Reg)
}

// DMA with separate read and write masters
type DMA_Type struct {
	CH0_READ_ADDR             volatile.Register32 // 0x0
	CH0_WRITE_ADDR            volatile.Register32 // 0x4
	CH0_TRANS_COUNT           volatile.Register32 // 0x8
	CH0_CTRL_TRIG             volatile.Register32 // 0xC
	CH0_AL1_CTRL              volatile.Register32 // 0x10
	CH0_AL1_READ_ADDR         volatile.Register32 // 0x14
	CH0_AL1_WRITE_ADDR        volatile.Register32 // 0x18
	CH0_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0x1C
	CH0_AL2_CTRL              volatile.Register32 // 0x20
	CH0_AL2_TRANS_COUNT       volatile.Register32 // 0x24
	CH0_AL2_READ_ADDR         volatile.Register32 // 0x28
	CH0_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0x2C
	CH0_AL3_CTRL              volatile.Register32 // 0x30
	CH0_AL3_WRITE_ADDR        volatile.Register32 // 0x34
	CH0_AL3_TRANS_COUNT       volatile.Register32 // 0x38
	CH0_AL3_READ_ADDR_TRIG    volatile.Register32 // 0x3C
	CH1_READ_ADDR             volatile.Register32 // 0x40
	CH1_WRITE_ADDR            volatile.Register32 // 0x44
	CH1_TRANS_COUNT           volatile.Register32 // 0x48
	CH1_CTRL_TRIG             volatile.Register32 // 0x4C
	CH1_AL1_CTRL              volatile.Register32 // 0x50
	CH1_AL1_READ_ADDR         volatile.Register32 // 0x54
	CH1_AL1_WRITE_ADDR        volatile.Register32 // 0x58
	CH1_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0x5C
	CH1_AL2_CTRL              volatile.Register32 // 0x60
	CH1_AL2_TRANS_COUNT       volatile.Register32 // 0x64
	CH1_AL2_READ_ADDR         volatile.Register32 // 0x68
	CH1_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0x6C
	CH1_AL3_CTRL              volatile.Register32 // 0x70
	CH1_AL3_WRITE_ADDR        volatile.Register32 // 0x74
	CH1_AL3_TRANS_COUNT       volatile.Register32 // 0x78
	CH1_AL3_READ_ADDR_TRIG    volatile.Register32 // 0x7C
	CH2_READ_ADDR             volatile.Register32 // 0x80
	CH2_WRITE_ADDR            volatile.Register32 // 0x84
	CH2_TRANS_COUNT           volatile.Register32 // 0x88
	CH2_CTRL_TRIG             volatile.Register32 // 0x8C
	CH2_AL1_CTRL              volatile.Register32 // 0x90
	CH2_AL1_READ_ADDR         volatile.Register32 // 0x94
	CH2_AL1_WRITE_ADDR        volatile.Register32 // 0x98
	CH2_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0x9C
	CH2_AL2_CTRL              volatile.Register32 // 0xA0
	CH2_AL2_TRANS_COUNT       volatile.Register32 // 0xA4
	CH2_AL2_READ_ADDR         volatile.Register32 // 0xA8
	CH2_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0xAC
	CH2_AL3_CTRL              volatile.Register32 // 0xB0
	CH2_AL3_WRITE_ADDR        volatile.Register32 // 0xB4
	CH2_AL3_TRANS_COUNT       volatile.Register32 // 0xB8
	CH2_AL3_READ_ADDR_TRIG    volatile.Register32 // 0xBC
	CH3_READ_ADDR             volatile.Register32 // 0xC0
	CH3_WRITE_ADDR            volatile.Register32 // 0xC4
	CH3_TRANS_COUNT           volatile.Register32 // 0xC8
	CH3_CTRL_TRIG             volatile.Register32 // 0xCC
	CH3_AL1_CTRL              volatile.Register32 // 0xD0
	CH3_AL1_READ_ADDR         volatile.Register32 // 0xD4
	CH3_AL1_WRITE_ADDR        volatile.Register32 // 0xD8
	CH3_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0xDC
	CH3_AL2_CTRL              volatile.Register32 // 0xE0
	CH3_AL2_TRANS_COUNT       volatile.Register32 // 0xE4
	CH3_AL2_READ_ADDR         volatile.Register32 // 0xE8
	CH3_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0xEC
	CH3_AL3_CTRL              volatile.Register32 // 0xF0
	CH3_AL3_WRITE_ADDR        volatile.Register32 // 0xF4
	CH3_AL3_TRANS_COUNT       volatile.Register32 // 0xF8
	CH3_AL3_READ_ADDR_TRIG    volatile.Register32 // 0xFC
	CH4_READ_ADDR             volatile.Register32 // 0x100
	CH4_WRITE_ADDR            volatile.Register32 // 0x104
	CH4_TRANS_COUNT           volatile.Register32 // 0x108
	CH4_CTRL_TRIG             volatile.Register32 // 0x10C
	CH4_AL1_CTRL              volatile.Register32 // 0x110
	CH4_AL1_READ_ADDR         volatile.Register32 // 0x114
	CH4_AL1_WRITE_ADDR        volatile.Register32 // 0x118
	CH4_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0x11C
	CH4_AL2_CTRL              volatile.Register32 // 0x120
	CH4_AL2_TRANS_COUNT       volatile.Register32 // 0x124
	CH4_AL2_READ_ADDR         volatile.Register32 // 0x128
	CH4_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0x12C
	CH4_AL3_CTRL              volatile.Register32 // 0x130
	CH4_AL3_WRITE_ADDR        volatile.Register32 // 0x134
	CH4_AL3_TRANS_COUNT       volatile.Register32 // 0x138
	CH4_AL3_READ_ADDR_TRIG    volatile.Register32 // 0x13C
	CH5_READ_ADDR             volatile.Register32 // 0x140
	CH5_WRITE_ADDR            volatile.Register32 // 0x144
	CH5_TRANS_COUNT           volatile.Register32 // 0x148
	CH5_CTRL_TRIG             volatile.Register32 // 0x14C
	CH5_AL1_CTRL              volatile.Register32 // 0x150
	CH5_AL1_READ_ADDR         volatile.Register32 // 0x154
	CH5_AL1_WRITE_ADDR        volatile.Register32 // 0x158
	CH5_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0x15C
	CH5_AL2_CTRL              volatile.Register32 // 0x160
	CH5_AL2_TRANS_COUNT       volatile.Register32 // 0x164
	CH5_AL2_READ_ADDR         volatile.Register32 // 0x168
	CH5_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0x16C
	CH5_AL3_CTRL              volatile.Register32 // 0x170
	CH5_AL3_WRITE_ADDR        volatile.Register32 // 0x174
	CH5_AL3_TRANS_COUNT       volatile.Register32 // 0x178
	CH5_AL3_READ_ADDR_TRIG    volatile.Register32 // 0x17C
	CH6_READ_ADDR             volatile.Register32 // 0x180
	CH6_WRITE_ADDR            volatile.Register32 // 0x184
	CH6_TRANS_COUNT           volatile.Register32 // 0x188
	CH6_CTRL_TRIG             volatile.Register32 // 0x18C
	CH6_AL1_CTRL              volatile.Register32 // 0x190
	CH6_AL1_READ_ADDR         volatile.Register32 // 0x194
	CH6_AL1_WRITE_ADDR        volatile.Register32 // 0x198
	CH6_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0x19C
	CH6_AL2_CTRL              volatile.Register32 // 0x1A0
	CH6_AL2_TRANS_COUNT       volatile.Register32 // 0x1A4
	CH6_AL2_READ_ADDR         volatile.Register32 // 0x1A8
	CH6_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0x1AC
	CH6_AL3_CTRL              volatile.Register32 // 0x1B0
	CH6_AL3_WRITE_ADDR        volatile.Register32 // 0x1B4
	CH6_AL3_TRANS_COUNT       volatile.Register32 // 0x1B8
	CH6_AL3_READ_ADDR_TRIG    volatile.Register32 // 0x1BC
	CH7_READ_ADDR             volatile.Register32 // 0x1C0
	CH7_WRITE_ADDR            volatile.Register32 // 0x1C4
	CH7_TRANS_COUNT           volatile.Register32 // 0x1C8
	CH7_CTRL_TRIG             volatile.Register32 // 0x1CC
	CH7_AL1_CTRL              volatile.Register32 // 0x1D0
	CH7_AL1_READ_ADDR         volatile.Register32 // 0x1D4
	CH7_AL1_WRITE_ADDR        volatile.Register32 // 0x1D8
	CH7_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0x1DC
	CH7_AL2_CTRL              volatile.Register32 // 0x1E0
	CH7_AL2_TRANS_COUNT       volatile.Register32 // 0x1E4
	CH7_AL2_READ_ADDR         volatile.Register32 // 0x1E8
	CH7_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0x1EC
	CH7_AL3_CTRL              volatile.Register32 // 0x1F0
	CH7_AL3_WRITE_ADDR        volatile.Register32 // 0x1F4
	CH7_AL3_TRANS_COUNT       volatile.Register32 // 0x1F8
	CH7_AL3_READ_ADDR_TRIG    volatile.Register32 // 0x1FC
	CH8_READ_ADDR             volatile.Register32 // 0x200
	CH8_WRITE_ADDR            volatile.Register32 // 0x204
	CH8_TRANS_COUNT           volatile.Register32 // 0x208
	CH8_CTRL_TRIG             volatile.Register32 // 0x20C
	CH8_AL1_CTRL              volatile.Register32 // 0x210
	CH8_AL1_READ_ADDR         volatile.Register32 // 0x214
	CH8_AL1_WRITE_ADDR        volatile.Register32 // 0x218
	CH8_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0x21C
	CH8_AL2_CTRL              volatile.Register32 // 0x220
	CH8_AL2_TRANS_COUNT       volatile.Register32 // 0x224
	CH8_AL2_READ_ADDR         volatile.Register32 // 0x228
	CH8_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0x22C
	CH8_AL3_CTRL              volatile.Register32 // 0x230
	CH8_AL3_WRITE_ADDR        volatile.Register32 // 0x234
	CH8_AL3_TRANS_COUNT       volatile.Register32 // 0x238
	CH8_AL3_READ_ADDR_TRIG    volatile.Register32 // 0x23C
	CH9_READ_ADDR             volatile.Register32 // 0x240
	CH9_WRITE_ADDR            volatile.Register32 // 0x244
	CH9_TRANS_COUNT           volatile.Register32 // 0x248
	CH9_CTRL_TRIG             volatile.Register32 // 0x24C
	CH9_AL1_CTRL              volatile.Register32 // 0x250
	CH9_AL1_READ_ADDR         volatile.Register32 // 0x254
	CH9_AL1_WRITE_ADDR        volatile.Register32 // 0x258
	CH9_AL1_TRANS_COUNT_TRIG  volatile.Register32 // 0x25C
	CH9_AL2_CTRL              volatile.Register32 // 0x260
	CH9_AL2_TRANS_COUNT       volatile.Register32 // 0x264
	CH9_AL2_READ_ADDR         volatile.Register32 // 0x268
	CH9_AL2_WRITE_ADDR_TRIG   volatile.Register32 // 0x26C
	CH9_AL3_CTRL              volatile.Register32 // 0x270
	CH9_AL3_WRITE_ADDR        volatile.Register32 // 0x274
	CH9_AL3_TRANS_COUNT       volatile.Register32 // 0x278
	CH9_AL3_READ_ADDR_TRIG    volatile.Register32 // 0x27C
	CH10_READ_ADDR            volatile.Register32 // 0x280
	CH10_WRITE_ADDR           volatile.Register32 // 0x284
	CH10_TRANS_COUNT          volatile.Register32 // 0x288
	CH10_CTRL_TRIG            volatile.Register32 // 0x28C
	CH10_AL1_CTRL             volatile.Register32 // 0x290
	CH10_AL1_READ_ADDR        volatile.Register32 // 0x294
	CH10_AL1_WRITE_ADDR       volatile.Register32 // 0x298
	CH10_AL1_TRANS_COUNT_TRIG volatile.Register32 // 0x29C
	CH10_AL2_CTRL             volatile.Register32 // 0x2A0
	CH10_AL2_TRANS_COUNT      volatile.Register32 // 0x2A4
	CH10_AL2_READ_ADDR        volatile.Register32 // 0x2A8
	CH10_AL2_WRITE_ADDR_TRIG  volatile.Register32 // 0x2AC
	CH10_AL3_CTRL             volatile.Register32 // 0x2B0
	CH10_AL3_WRITE_ADDR       volatile.Register32 // 0x2B4
	CH10_AL3_TRANS_COUNT      volatile.Register32 // 0x2B8
	CH10_AL3_READ_ADDR_TRIG   volatile.Register32 // 0x2BC
	CH11_READ_ADDR            volatile.Register32 // 0x2C0
	CH11_WRITE_ADDR           volatile.Register32 // 0x2C4
	CH11_TRANS_COUNT          volatile.Register32 // 0x2C8
	CH11_CTRL_TRIG            volatile.Register32 // 0x2CC
	CH11_AL1_CTRL             volatile.Register32 // 0x2D0
	CH11_AL1_READ_ADDR        volatile.Register32 // 0x2D4
	CH11_AL1_WRITE_ADDR       volatile.Register32 // 0x2D8
	CH11_AL1_TRANS_COUNT_TRIG volatile.Register32 // 0x2DC
	CH11_AL2_CTRL             volatile.Register32 // 0x2E0
	CH11_AL2_TRANS_COUNT      volatile.Register32 // 0x2E4
	CH11_AL2_READ_ADDR        volatile.Register32 // 0x2E8
	CH11_AL2_WRITE_ADDR_TRIG  volatile.Register32 // 0x2EC
	CH11_AL3_CTRL             volatile.Register32 // 0x2F0
	CH11_AL3_WRITE_ADDR       volatile.Register32 // 0x2F4
	CH11_AL3_TRANS_COUNT      volatile.Register32 // 0x2F8
	CH11_AL3_READ_ADDR_TRIG   volatile.Register32 // 0x2FC
	CH12_READ_ADDR            volatile.Register32 // 0x300
	CH12_WRITE_ADDR           volatile.Register32 // 0x304
	CH12_TRANS_COUNT          volatile.Register32 // 0x308
	CH12_CTRL_TRIG            volatile.Register32 // 0x30C
	CH12_AL1_CTRL             volatile.Register32 // 0x310
	CH12_AL1_READ_ADDR        volatile.Register32 // 0x314
	CH12_AL1_WRITE_ADDR       volatile.Register32 // 0x318
	CH12_AL1_TRANS_COUNT_TRIG volatile.Register32 // 0x31C
	CH12_AL2_CTRL             volatile.Register32 // 0x320
	CH12_AL2_TRANS_COUNT      volatile.Register32 // 0x324
	CH12_AL2_READ_ADDR        volatile.Register32 // 0x328
	CH12_AL2_WRITE_ADDR_TRIG  volatile.Register32 // 0x32C
	CH12_AL3_CTRL             volatile.Register32 // 0x330
	CH12_AL3_WRITE_ADDR       volatile.Register32 // 0x334
	CH12_AL3_TRANS_COUNT      volatile.Register32 // 0x338
	CH12_AL3_READ_ADDR_TRIG   volatile.Register32 // 0x33C
	CH13_READ_ADDR            volatile.Register32 // 0x340
	CH13_WRITE_ADDR           volatile.Register32 // 0x344
	CH13_TRANS_COUNT          volatile.Register32 // 0x348
	CH13_CTRL_TRIG            volatile.Register32 // 0x34C
	CH13_AL1_CTRL             volatile.Register32 // 0x350
	CH13_AL1_READ_ADDR        volatile.Register32 // 0x354
	CH13_AL1_WRITE_ADDR       volatile.Register32 // 0x358
	CH13_AL1_TRANS_COUNT_TRIG volatile.Register32 // 0x35C
	CH13_AL2_CTRL             volatile.Register32 // 0x360
	CH13_AL2_TRANS_COUNT      volatile.Register32 // 0x364
	CH13_AL2_READ_ADDR        volatile.Register32 // 0x368
	CH13_AL2_WRITE_ADDR_TRIG  volatile.Register32 // 0x36C
	CH13_AL3_CTRL             volatile.Register32 // 0x370
	CH13_AL3_WRITE_ADDR       volatile.Register32 // 0x374
	CH13_AL3_TRANS_COUNT      volatile.Register32 // 0x378
	CH13_AL3_READ_ADDR_TRIG   volatile.Register32 // 0x37C
	CH14_READ_ADDR            volatile.Register32 // 0x380
	CH14_WRITE_ADDR           volatile.Register32 // 0x384
	CH14_TRANS_COUNT          volatile.Register32 // 0x388
	CH14_CTRL_TRIG            volatile.Register32 // 0x38C
	CH14_AL1_CTRL             volatile.Register32 // 0x390
	CH14_AL1_READ_ADDR        volatile.Register32 // 0x394
	CH14_AL1_WRITE_ADDR       volatile.Register32 // 0x398
	CH14_AL1_TRANS_COUNT_TRIG volatile.Register32 // 0x39C
	CH14_AL2_CTRL             volatile.Register32 // 0x3A0
	CH14_AL2_TRANS_COUNT      volatile.Register32 // 0x3A4
	CH14_AL2_READ_ADDR        volatile.Register32 // 0x3A8
	CH14_AL2_WRITE_ADDR_TRIG  volatile.Register32 // 0x3AC
	CH14_AL3_CTRL             volatile.Register32 // 0x3B0
	CH14_AL3_WRITE_ADDR       volatile.Register32 // 0x3B4
	CH14_AL3_TRANS_COUNT      volatile.Register32 // 0x3B8
	CH14_AL3_READ_ADDR_TRIG   volatile.Register32 // 0x3BC
	CH15_READ_ADDR            volatile.Register32 // 0x3C0
	CH15_WRITE_ADDR           volatile.Register32 // 0x3C4
	CH15_TRANS_COUNT          volatile.Register32 // 0x3C8
	CH15_CTRL_TRIG            volatile.Register32 // 0x3CC
	CH15_AL1_CTRL             volatile.Register32 // 0x3D0
	CH15_AL1_READ_ADDR        volatile.Register32 // 0x3D4
	CH15_AL1_WRITE_ADDR       volatile.Register32 // 0x3D8
	CH15_AL1_TRANS_COUNT_TRIG volatile.Register32 // 0x3DC
	CH15_AL2_CTRL             volatile.Register32 // 0x3E0
	CH15_AL2_TRANS_COUNT      volatile.Register32 // 0x3E4
	CH15_AL2_READ_ADDR        volatile.Register32 // 0x3E8
	CH15_AL2_WRITE_ADDR_TRIG  volatile.Register32 // 0x3EC
	CH15_AL3_CTRL             volatile.Register32 // 0x3F0
	CH15_AL3_WRITE_ADDR       volatile.Register32 // 0x3F4
	CH15_AL3_TRANS_COUNT      volatile.Register32 // 0x3F8
	CH15_AL3_READ_ADDR_TRIG   volatile.Register32 // 0x3FC
	INTR                      volatile.Register32 // 0x400
	INTE0                     volatile.Register32 // 0x404
	INTF0                     volatile.Register32 // 0x408
	INTS0                     volatile.Register32 // 0x40C
	INTR1                     volatile.Register32 // 0x410
	INTE1                     volatile.Register32 // 0x414
	INTF1                     volatile.Register32 // 0x418
	INTS1                     volatile.Register32 // 0x41C
	INTR2                     volatile.Register32 // 0x420
	INTE2                     volatile.Register32 // 0x424
	INTF2                     volatile.Register32 // 0x428
	INTS2                     volatile.Register32 // 0x42C
	INTR3                     volatile.Register32 // 0x430
	INTE3                     volatile.Register32 // 0x434
	INTF3                     volatile.Register32 // 0x438
	INTS3                     volatile.Register32 // 0x43C
	TIMER0                    volatile.Register32 // 0x440
	TIMER1                    volatile.Register32 // 0x444
	TIMER2                    volatile.Register32 // 0x448
	TIMER3                    volatile.Register32 // 0x44C
	MULTI_CHAN_TRIGGER        volatile.Register32 // 0x450
	SNIFF_CTRL                volatile.Register32 // 0x454
	SNIFF_DATA                volatile.Register32 // 0x458
	_                         [4]byte
	FIFO_LEVELS               volatile.Register32 // 0x460
	CHAN_ABORT                volatile.Register32 // 0x464
	N_CHANNELS                volatile.Register32 // 0x468
	_                         [20]byte
	SECCFG_CH0                volatile.Register32 // 0x480
	SECCFG_CH1                volatile.Register32 // 0x484
	SECCFG_CH2                volatile.Register32 // 0x488
	SECCFG_CH3                volatile.Register32 // 0x48C
	SECCFG_CH4                volatile.Register32 // 0x490
	SECCFG_CH5                volatile.Register32 // 0x494
	SECCFG_CH6                volatile.Register32 // 0x498
	SECCFG_CH7                volatile.Register32 // 0x49C
	SECCFG_CH8                volatile.Register32 // 0x4A0
	SECCFG_CH9                volatile.Register32 // 0x4A4
	SECCFG_CH10               volatile.Register32 // 0x4A8
	SECCFG_CH11               volatile.Register32 // 0x4AC
	SECCFG_CH12               volatile.Register32 // 0x4B0
	SECCFG_CH13               volatile.Register32 // 0x4B4
	SECCFG_CH14               volatile.Register32 // 0x4B8
	SECCFG_CH15               volatile.Register32 // 0x4BC
	SECCFG_IRQ0               volatile.Register32 // 0x4C0
	SECCFG_IRQ1               volatile.Register32 // 0x4C4
	SECCFG_IRQ2               volatile.Register32 // 0x4C8
	SECCFG_IRQ3               volatile.Register32 // 0x4CC
	SECCFG_MISC               volatile.Register32 // 0x4D0
	_                         [44]byte
	MPU_CTRL                  volatile.Register32 // 0x500
	MPU_BAR0                  volatile.Register32 // 0x504
	MPU_LAR0                  volatile.Register32 // 0x508
	MPU_BAR1                  volatile.Register32 // 0x50C
	MPU_LAR1                  volatile.Register32 // 0x510
	MPU_BAR2                  volatile.Register32 // 0x514
	MPU_LAR2                  volatile.Register32 // 0x518
	MPU_BAR3                  volatile.Register32 // 0x51C
	MPU_LAR3                  volatile.Register32 // 0x520
	MPU_BAR4                  volatile.Register32 // 0x524
	MPU_LAR4                  volatile.Register32 // 0x528
	MPU_BAR5                  volatile.Register32 // 0x52C
	MPU_LAR5                  volatile.Register32 // 0x530
	MPU_BAR6                  volatile.Register32 // 0x534
	MPU_LAR6                  volatile.Register32 // 0x538
	MPU_BAR7                  volatile.Register32 // 0x53C
	MPU_LAR7                  volatile.Register32 // 0x540
	_                         [700]byte
	CH0_DBG_CTDREQ            volatile.Register32 // 0x800
	CH0_DBG_TCR               volatile.Register32 // 0x804
	_                         [56]byte
	CH1_DBG_CTDREQ            volatile.Register32 // 0x840
	CH1_DBG_TCR               volatile.Register32 // 0x844
	_                         [56]byte
	CH2_DBG_CTDREQ            volatile.Register32 // 0x880
	CH2_DBG_TCR               volatile.Register32 // 0x884
	_                         [56]byte
	CH3_DBG_CTDREQ            volatile.Register32 // 0x8C0
	CH3_DBG_TCR               volatile.Register32 // 0x8C4
	_                         [56]byte
	CH4_DBG_CTDREQ            volatile.Register32 // 0x900
	CH4_DBG_TCR               volatile.Register32 // 0x904
	_                         [56]byte
	CH5_DBG_CTDREQ            volatile.Register32 // 0x940
	CH5_DBG_TCR               volatile.Register32 // 0x944
	_                         [56]byte
	CH6_DBG_CTDREQ            volatile.Register32 // 0x980
	CH6_DBG_TCR               volatile.Register32 // 0x984
	_                         [56]byte
	CH7_DBG_CTDREQ            volatile.Register32 // 0x9C0
	CH7_DBG_TCR               volatile.Register32 // 0x9C4
	_                         [56]byte
	CH8_DBG_CTDREQ            volatile.Register32 // 0xA00
	CH8_DBG_TCR               volatile.Register32 // 0xA04
	_                         [56]byte
	CH9_DBG_CTDREQ            volatile.Register32 // 0xA40
	CH9_DBG_TCR               volatile.Register32 // 0xA44
	_                         [56]byte
	CH10_DBG_CTDREQ           volatile.Register32 // 0xA80
	CH10_DBG_TCR              volatile.Register32 // 0xA84
	_                         [56]byte
	CH11_DBG_CTDREQ           volatile.Register32 // 0xAC0
	CH11_DBG_TCR              volatile.Register32 // 0xAC4
	_                         [56]byte
	CH12_DBG_CTDREQ           volatile.Register32 // 0xB00
	CH12_DBG_TCR              volatile.Register32 // 0xB04
	_                         [56]byte
	CH13_DBG_CTDREQ           volatile.Register32 // 0xB40
	CH13_DBG_TCR              volatile.Register32 // 0xB44
	_                         [56]byte
	CH14_DBG_CTDREQ           volatile.Register32 // 0xB80
	CH14_DBG_TCR              volatile.Register32 // 0xB84
	_                         [56]byte
	CH15_DBG_CTDREQ           volatile.Register32 // 0xBC0
	CH15_DBG_TCR              volatile.Register32 // 0xBC4
}

// DMA.CH0_READ_ADDR: DMA Channel 0 Read Address pointer
func (o *DMA_Type) SetCH0_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH0_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH0_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH0_READ_ADDR.Reg)
}

// DMA.CH0_WRITE_ADDR: DMA Channel 0 Write Address pointer
func (o *DMA_Type) SetCH0_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH0_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH0_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH0_WRITE_ADDR.Reg)
}

// DMA.CH0_TRANS_COUNT: DMA Channel 0 Transfer Count
func (o *DMA_Type) SetCH0_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH0_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH0_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH0_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH0_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH0_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH0_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH0_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH0_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH0_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH0_CTRL_TRIG: DMA Channel 0 Control and Status
func (o *DMA_Type) SetCH0_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH0_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH0_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH0_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH0_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH0_AL1_CTRL: Alias for channel 0 CTRL register
func (o *DMA_Type) SetCH0_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH0_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH0_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH0_AL1_CTRL.Reg)
}

// DMA.CH0_AL1_READ_ADDR: Alias for channel 0 READ_ADDR register
func (o *DMA_Type) SetCH0_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH0_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH0_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH0_AL1_READ_ADDR.Reg)
}

// DMA.CH0_AL1_WRITE_ADDR: Alias for channel 0 WRITE_ADDR register
func (o *DMA_Type) SetCH0_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH0_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH0_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH0_AL1_WRITE_ADDR.Reg)
}

// DMA.CH0_AL1_TRANS_COUNT_TRIG: Alias for channel 0 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH0_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH0_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH0_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH0_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH0_AL2_CTRL: Alias for channel 0 CTRL register
func (o *DMA_Type) SetCH0_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH0_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH0_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH0_AL2_CTRL.Reg)
}

// DMA.CH0_AL2_TRANS_COUNT: Alias for channel 0 TRANS_COUNT register
func (o *DMA_Type) SetCH0_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH0_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH0_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH0_AL2_TRANS_COUNT.Reg)
}

// DMA.CH0_AL2_READ_ADDR: Alias for channel 0 READ_ADDR register
func (o *DMA_Type) SetCH0_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH0_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH0_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH0_AL2_READ_ADDR.Reg)
}

// DMA.CH0_AL2_WRITE_ADDR_TRIG: Alias for channel 0 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH0_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH0_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH0_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH0_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH0_AL3_CTRL: Alias for channel 0 CTRL register
func (o *DMA_Type) SetCH0_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH0_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH0_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH0_AL3_CTRL.Reg)
}

// DMA.CH0_AL3_WRITE_ADDR: Alias for channel 0 WRITE_ADDR register
func (o *DMA_Type) SetCH0_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH0_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH0_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH0_AL3_WRITE_ADDR.Reg)
}

// DMA.CH0_AL3_TRANS_COUNT: Alias for channel 0 TRANS_COUNT register
func (o *DMA_Type) SetCH0_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH0_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH0_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH0_AL3_TRANS_COUNT.Reg)
}

// DMA.CH0_AL3_READ_ADDR_TRIG: Alias for channel 0 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH0_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH0_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH0_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH0_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH1_READ_ADDR: DMA Channel 1 Read Address pointer
func (o *DMA_Type) SetCH1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH1_READ_ADDR.Reg)
}

// DMA.CH1_WRITE_ADDR: DMA Channel 1 Write Address pointer
func (o *DMA_Type) SetCH1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH1_WRITE_ADDR.Reg)
}

// DMA.CH1_TRANS_COUNT: DMA Channel 1 Transfer Count
func (o *DMA_Type) SetCH1_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH1_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH1_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH1_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH1_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH1_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH1_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH1_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH1_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH1_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH1_CTRL_TRIG: DMA Channel 1 Control and Status
func (o *DMA_Type) SetCH1_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH1_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH1_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH1_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH1_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH1_AL1_CTRL: Alias for channel 1 CTRL register
func (o *DMA_Type) SetCH1_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH1_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH1_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH1_AL1_CTRL.Reg)
}

// DMA.CH1_AL1_READ_ADDR: Alias for channel 1 READ_ADDR register
func (o *DMA_Type) SetCH1_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH1_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH1_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH1_AL1_READ_ADDR.Reg)
}

// DMA.CH1_AL1_WRITE_ADDR: Alias for channel 1 WRITE_ADDR register
func (o *DMA_Type) SetCH1_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH1_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH1_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH1_AL1_WRITE_ADDR.Reg)
}

// DMA.CH1_AL1_TRANS_COUNT_TRIG: Alias for channel 1 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH1_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH1_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH1_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH1_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH1_AL2_CTRL: Alias for channel 1 CTRL register
func (o *DMA_Type) SetCH1_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH1_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH1_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH1_AL2_CTRL.Reg)
}

// DMA.CH1_AL2_TRANS_COUNT: Alias for channel 1 TRANS_COUNT register
func (o *DMA_Type) SetCH1_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH1_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH1_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH1_AL2_TRANS_COUNT.Reg)
}

// DMA.CH1_AL2_READ_ADDR: Alias for channel 1 READ_ADDR register
func (o *DMA_Type) SetCH1_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH1_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH1_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH1_AL2_READ_ADDR.Reg)
}

// DMA.CH1_AL2_WRITE_ADDR_TRIG: Alias for channel 1 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH1_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH1_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH1_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH1_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH1_AL3_CTRL: Alias for channel 1 CTRL register
func (o *DMA_Type) SetCH1_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH1_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH1_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH1_AL3_CTRL.Reg)
}

// DMA.CH1_AL3_WRITE_ADDR: Alias for channel 1 WRITE_ADDR register
func (o *DMA_Type) SetCH1_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH1_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH1_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH1_AL3_WRITE_ADDR.Reg)
}

// DMA.CH1_AL3_TRANS_COUNT: Alias for channel 1 TRANS_COUNT register
func (o *DMA_Type) SetCH1_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH1_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH1_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH1_AL3_TRANS_COUNT.Reg)
}

// DMA.CH1_AL3_READ_ADDR_TRIG: Alias for channel 1 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH1_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH1_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH1_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH1_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH2_READ_ADDR: DMA Channel 2 Read Address pointer
func (o *DMA_Type) SetCH2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH2_READ_ADDR.Reg)
}

// DMA.CH2_WRITE_ADDR: DMA Channel 2 Write Address pointer
func (o *DMA_Type) SetCH2_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH2_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH2_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH2_WRITE_ADDR.Reg)
}

// DMA.CH2_TRANS_COUNT: DMA Channel 2 Transfer Count
func (o *DMA_Type) SetCH2_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH2_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH2_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH2_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH2_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH2_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH2_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH2_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH2_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH2_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH2_CTRL_TRIG: DMA Channel 2 Control and Status
func (o *DMA_Type) SetCH2_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH2_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH2_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH2_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH2_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH2_AL1_CTRL: Alias for channel 2 CTRL register
func (o *DMA_Type) SetCH2_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH2_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH2_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH2_AL1_CTRL.Reg)
}

// DMA.CH2_AL1_READ_ADDR: Alias for channel 2 READ_ADDR register
func (o *DMA_Type) SetCH2_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH2_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH2_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH2_AL1_READ_ADDR.Reg)
}

// DMA.CH2_AL1_WRITE_ADDR: Alias for channel 2 WRITE_ADDR register
func (o *DMA_Type) SetCH2_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH2_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH2_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH2_AL1_WRITE_ADDR.Reg)
}

// DMA.CH2_AL1_TRANS_COUNT_TRIG: Alias for channel 2 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH2_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH2_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH2_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH2_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH2_AL2_CTRL: Alias for channel 2 CTRL register
func (o *DMA_Type) SetCH2_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH2_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH2_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH2_AL2_CTRL.Reg)
}

// DMA.CH2_AL2_TRANS_COUNT: Alias for channel 2 TRANS_COUNT register
func (o *DMA_Type) SetCH2_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH2_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH2_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH2_AL2_TRANS_COUNT.Reg)
}

// DMA.CH2_AL2_READ_ADDR: Alias for channel 2 READ_ADDR register
func (o *DMA_Type) SetCH2_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH2_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH2_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH2_AL2_READ_ADDR.Reg)
}

// DMA.CH2_AL2_WRITE_ADDR_TRIG: Alias for channel 2 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH2_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH2_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH2_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH2_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH2_AL3_CTRL: Alias for channel 2 CTRL register
func (o *DMA_Type) SetCH2_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH2_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH2_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH2_AL3_CTRL.Reg)
}

// DMA.CH2_AL3_WRITE_ADDR: Alias for channel 2 WRITE_ADDR register
func (o *DMA_Type) SetCH2_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH2_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH2_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH2_AL3_WRITE_ADDR.Reg)
}

// DMA.CH2_AL3_TRANS_COUNT: Alias for channel 2 TRANS_COUNT register
func (o *DMA_Type) SetCH2_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH2_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH2_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH2_AL3_TRANS_COUNT.Reg)
}

// DMA.CH2_AL3_READ_ADDR_TRIG: Alias for channel 2 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH2_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH2_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH2_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH2_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH3_READ_ADDR: DMA Channel 3 Read Address pointer
func (o *DMA_Type) SetCH3_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH3_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH3_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH3_READ_ADDR.Reg)
}

// DMA.CH3_WRITE_ADDR: DMA Channel 3 Write Address pointer
func (o *DMA_Type) SetCH3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH3_WRITE_ADDR.Reg)
}

// DMA.CH3_TRANS_COUNT: DMA Channel 3 Transfer Count
func (o *DMA_Type) SetCH3_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH3_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH3_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH3_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH3_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH3_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH3_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH3_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH3_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH3_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH3_CTRL_TRIG: DMA Channel 3 Control and Status
func (o *DMA_Type) SetCH3_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH3_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH3_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH3_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH3_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH3_AL1_CTRL: Alias for channel 3 CTRL register
func (o *DMA_Type) SetCH3_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH3_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH3_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH3_AL1_CTRL.Reg)
}

// DMA.CH3_AL1_READ_ADDR: Alias for channel 3 READ_ADDR register
func (o *DMA_Type) SetCH3_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH3_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH3_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH3_AL1_READ_ADDR.Reg)
}

// DMA.CH3_AL1_WRITE_ADDR: Alias for channel 3 WRITE_ADDR register
func (o *DMA_Type) SetCH3_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH3_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH3_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH3_AL1_WRITE_ADDR.Reg)
}

// DMA.CH3_AL1_TRANS_COUNT_TRIG: Alias for channel 3 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH3_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH3_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH3_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH3_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH3_AL2_CTRL: Alias for channel 3 CTRL register
func (o *DMA_Type) SetCH3_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH3_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH3_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH3_AL2_CTRL.Reg)
}

// DMA.CH3_AL2_TRANS_COUNT: Alias for channel 3 TRANS_COUNT register
func (o *DMA_Type) SetCH3_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH3_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH3_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH3_AL2_TRANS_COUNT.Reg)
}

// DMA.CH3_AL2_READ_ADDR: Alias for channel 3 READ_ADDR register
func (o *DMA_Type) SetCH3_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH3_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH3_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH3_AL2_READ_ADDR.Reg)
}

// DMA.CH3_AL2_WRITE_ADDR_TRIG: Alias for channel 3 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH3_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH3_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH3_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH3_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH3_AL3_CTRL: Alias for channel 3 CTRL register
func (o *DMA_Type) SetCH3_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH3_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH3_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH3_AL3_CTRL.Reg)
}

// DMA.CH3_AL3_WRITE_ADDR: Alias for channel 3 WRITE_ADDR register
func (o *DMA_Type) SetCH3_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH3_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH3_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH3_AL3_WRITE_ADDR.Reg)
}

// DMA.CH3_AL3_TRANS_COUNT: Alias for channel 3 TRANS_COUNT register
func (o *DMA_Type) SetCH3_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH3_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH3_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH3_AL3_TRANS_COUNT.Reg)
}

// DMA.CH3_AL3_READ_ADDR_TRIG: Alias for channel 3 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH3_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH3_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH3_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH3_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH4_READ_ADDR: DMA Channel 4 Read Address pointer
func (o *DMA_Type) SetCH4_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH4_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH4_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH4_READ_ADDR.Reg)
}

// DMA.CH4_WRITE_ADDR: DMA Channel 4 Write Address pointer
func (o *DMA_Type) SetCH4_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH4_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH4_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH4_WRITE_ADDR.Reg)
}

// DMA.CH4_TRANS_COUNT: DMA Channel 4 Transfer Count
func (o *DMA_Type) SetCH4_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH4_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH4_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH4_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH4_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH4_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH4_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH4_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH4_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH4_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH4_CTRL_TRIG: DMA Channel 4 Control and Status
func (o *DMA_Type) SetCH4_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH4_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH4_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH4_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH4_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH4_AL1_CTRL: Alias for channel 4 CTRL register
func (o *DMA_Type) SetCH4_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH4_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH4_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH4_AL1_CTRL.Reg)
}

// DMA.CH4_AL1_READ_ADDR: Alias for channel 4 READ_ADDR register
func (o *DMA_Type) SetCH4_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH4_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH4_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH4_AL1_READ_ADDR.Reg)
}

// DMA.CH4_AL1_WRITE_ADDR: Alias for channel 4 WRITE_ADDR register
func (o *DMA_Type) SetCH4_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH4_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH4_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH4_AL1_WRITE_ADDR.Reg)
}

// DMA.CH4_AL1_TRANS_COUNT_TRIG: Alias for channel 4 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH4_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH4_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH4_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH4_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH4_AL2_CTRL: Alias for channel 4 CTRL register
func (o *DMA_Type) SetCH4_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH4_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH4_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH4_AL2_CTRL.Reg)
}

// DMA.CH4_AL2_TRANS_COUNT: Alias for channel 4 TRANS_COUNT register
func (o *DMA_Type) SetCH4_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH4_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH4_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH4_AL2_TRANS_COUNT.Reg)
}

// DMA.CH4_AL2_READ_ADDR: Alias for channel 4 READ_ADDR register
func (o *DMA_Type) SetCH4_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH4_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH4_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH4_AL2_READ_ADDR.Reg)
}

// DMA.CH4_AL2_WRITE_ADDR_TRIG: Alias for channel 4 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH4_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH4_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH4_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH4_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH4_AL3_CTRL: Alias for channel 4 CTRL register
func (o *DMA_Type) SetCH4_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH4_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH4_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH4_AL3_CTRL.Reg)
}

// DMA.CH4_AL3_WRITE_ADDR: Alias for channel 4 WRITE_ADDR register
func (o *DMA_Type) SetCH4_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH4_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH4_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH4_AL3_WRITE_ADDR.Reg)
}

// DMA.CH4_AL3_TRANS_COUNT: Alias for channel 4 TRANS_COUNT register
func (o *DMA_Type) SetCH4_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH4_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH4_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH4_AL3_TRANS_COUNT.Reg)
}

// DMA.CH4_AL3_READ_ADDR_TRIG: Alias for channel 4 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH4_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH4_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH4_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH4_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH5_READ_ADDR: DMA Channel 5 Read Address pointer
func (o *DMA_Type) SetCH5_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH5_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH5_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH5_READ_ADDR.Reg)
}

// DMA.CH5_WRITE_ADDR: DMA Channel 5 Write Address pointer
func (o *DMA_Type) SetCH5_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH5_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH5_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH5_WRITE_ADDR.Reg)
}

// DMA.CH5_TRANS_COUNT: DMA Channel 5 Transfer Count
func (o *DMA_Type) SetCH5_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH5_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH5_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH5_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH5_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH5_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH5_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH5_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH5_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH5_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH5_CTRL_TRIG: DMA Channel 5 Control and Status
func (o *DMA_Type) SetCH5_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH5_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH5_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH5_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH5_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH5_AL1_CTRL: Alias for channel 5 CTRL register
func (o *DMA_Type) SetCH5_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH5_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH5_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH5_AL1_CTRL.Reg)
}

// DMA.CH5_AL1_READ_ADDR: Alias for channel 5 READ_ADDR register
func (o *DMA_Type) SetCH5_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH5_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH5_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH5_AL1_READ_ADDR.Reg)
}

// DMA.CH5_AL1_WRITE_ADDR: Alias for channel 5 WRITE_ADDR register
func (o *DMA_Type) SetCH5_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH5_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH5_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH5_AL1_WRITE_ADDR.Reg)
}

// DMA.CH5_AL1_TRANS_COUNT_TRIG: Alias for channel 5 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH5_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH5_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH5_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH5_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH5_AL2_CTRL: Alias for channel 5 CTRL register
func (o *DMA_Type) SetCH5_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH5_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH5_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH5_AL2_CTRL.Reg)
}

// DMA.CH5_AL2_TRANS_COUNT: Alias for channel 5 TRANS_COUNT register
func (o *DMA_Type) SetCH5_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH5_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH5_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH5_AL2_TRANS_COUNT.Reg)
}

// DMA.CH5_AL2_READ_ADDR: Alias for channel 5 READ_ADDR register
func (o *DMA_Type) SetCH5_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH5_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH5_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH5_AL2_READ_ADDR.Reg)
}

// DMA.CH5_AL2_WRITE_ADDR_TRIG: Alias for channel 5 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH5_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH5_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH5_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH5_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH5_AL3_CTRL: Alias for channel 5 CTRL register
func (o *DMA_Type) SetCH5_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH5_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH5_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH5_AL3_CTRL.Reg)
}

// DMA.CH5_AL3_WRITE_ADDR: Alias for channel 5 WRITE_ADDR register
func (o *DMA_Type) SetCH5_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH5_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH5_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH5_AL3_WRITE_ADDR.Reg)
}

// DMA.CH5_AL3_TRANS_COUNT: Alias for channel 5 TRANS_COUNT register
func (o *DMA_Type) SetCH5_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH5_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH5_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH5_AL3_TRANS_COUNT.Reg)
}

// DMA.CH5_AL3_READ_ADDR_TRIG: Alias for channel 5 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH5_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH5_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH5_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH5_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH6_READ_ADDR: DMA Channel 6 Read Address pointer
func (o *DMA_Type) SetCH6_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH6_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH6_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH6_READ_ADDR.Reg)
}

// DMA.CH6_WRITE_ADDR: DMA Channel 6 Write Address pointer
func (o *DMA_Type) SetCH6_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH6_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH6_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH6_WRITE_ADDR.Reg)
}

// DMA.CH6_TRANS_COUNT: DMA Channel 6 Transfer Count
func (o *DMA_Type) SetCH6_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH6_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH6_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH6_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH6_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH6_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH6_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH6_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH6_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH6_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH6_CTRL_TRIG: DMA Channel 6 Control and Status
func (o *DMA_Type) SetCH6_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH6_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH6_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH6_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH6_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH6_AL1_CTRL: Alias for channel 6 CTRL register
func (o *DMA_Type) SetCH6_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH6_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH6_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH6_AL1_CTRL.Reg)
}

// DMA.CH6_AL1_READ_ADDR: Alias for channel 6 READ_ADDR register
func (o *DMA_Type) SetCH6_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH6_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH6_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH6_AL1_READ_ADDR.Reg)
}

// DMA.CH6_AL1_WRITE_ADDR: Alias for channel 6 WRITE_ADDR register
func (o *DMA_Type) SetCH6_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH6_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH6_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH6_AL1_WRITE_ADDR.Reg)
}

// DMA.CH6_AL1_TRANS_COUNT_TRIG: Alias for channel 6 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH6_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH6_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH6_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH6_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH6_AL2_CTRL: Alias for channel 6 CTRL register
func (o *DMA_Type) SetCH6_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH6_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH6_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH6_AL2_CTRL.Reg)
}

// DMA.CH6_AL2_TRANS_COUNT: Alias for channel 6 TRANS_COUNT register
func (o *DMA_Type) SetCH6_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH6_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH6_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH6_AL2_TRANS_COUNT.Reg)
}

// DMA.CH6_AL2_READ_ADDR: Alias for channel 6 READ_ADDR register
func (o *DMA_Type) SetCH6_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH6_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH6_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH6_AL2_READ_ADDR.Reg)
}

// DMA.CH6_AL2_WRITE_ADDR_TRIG: Alias for channel 6 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH6_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH6_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH6_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH6_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH6_AL3_CTRL: Alias for channel 6 CTRL register
func (o *DMA_Type) SetCH6_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH6_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH6_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH6_AL3_CTRL.Reg)
}

// DMA.CH6_AL3_WRITE_ADDR: Alias for channel 6 WRITE_ADDR register
func (o *DMA_Type) SetCH6_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH6_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH6_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH6_AL3_WRITE_ADDR.Reg)
}

// DMA.CH6_AL3_TRANS_COUNT: Alias for channel 6 TRANS_COUNT register
func (o *DMA_Type) SetCH6_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH6_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH6_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH6_AL3_TRANS_COUNT.Reg)
}

// DMA.CH6_AL3_READ_ADDR_TRIG: Alias for channel 6 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH6_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH6_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH6_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH6_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH7_READ_ADDR: DMA Channel 7 Read Address pointer
func (o *DMA_Type) SetCH7_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH7_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH7_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH7_READ_ADDR.Reg)
}

// DMA.CH7_WRITE_ADDR: DMA Channel 7 Write Address pointer
func (o *DMA_Type) SetCH7_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH7_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH7_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH7_WRITE_ADDR.Reg)
}

// DMA.CH7_TRANS_COUNT: DMA Channel 7 Transfer Count
func (o *DMA_Type) SetCH7_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH7_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH7_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH7_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH7_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH7_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH7_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH7_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH7_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH7_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH7_CTRL_TRIG: DMA Channel 7 Control and Status
func (o *DMA_Type) SetCH7_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH7_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH7_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH7_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH7_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH7_AL1_CTRL: Alias for channel 7 CTRL register
func (o *DMA_Type) SetCH7_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH7_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH7_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH7_AL1_CTRL.Reg)
}

// DMA.CH7_AL1_READ_ADDR: Alias for channel 7 READ_ADDR register
func (o *DMA_Type) SetCH7_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH7_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH7_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH7_AL1_READ_ADDR.Reg)
}

// DMA.CH7_AL1_WRITE_ADDR: Alias for channel 7 WRITE_ADDR register
func (o *DMA_Type) SetCH7_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH7_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH7_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH7_AL1_WRITE_ADDR.Reg)
}

// DMA.CH7_AL1_TRANS_COUNT_TRIG: Alias for channel 7 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH7_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH7_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH7_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH7_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH7_AL2_CTRL: Alias for channel 7 CTRL register
func (o *DMA_Type) SetCH7_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH7_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH7_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH7_AL2_CTRL.Reg)
}

// DMA.CH7_AL2_TRANS_COUNT: Alias for channel 7 TRANS_COUNT register
func (o *DMA_Type) SetCH7_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH7_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH7_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH7_AL2_TRANS_COUNT.Reg)
}

// DMA.CH7_AL2_READ_ADDR: Alias for channel 7 READ_ADDR register
func (o *DMA_Type) SetCH7_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH7_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH7_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH7_AL2_READ_ADDR.Reg)
}

// DMA.CH7_AL2_WRITE_ADDR_TRIG: Alias for channel 7 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH7_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH7_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH7_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH7_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH7_AL3_CTRL: Alias for channel 7 CTRL register
func (o *DMA_Type) SetCH7_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH7_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH7_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH7_AL3_CTRL.Reg)
}

// DMA.CH7_AL3_WRITE_ADDR: Alias for channel 7 WRITE_ADDR register
func (o *DMA_Type) SetCH7_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH7_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH7_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH7_AL3_WRITE_ADDR.Reg)
}

// DMA.CH7_AL3_TRANS_COUNT: Alias for channel 7 TRANS_COUNT register
func (o *DMA_Type) SetCH7_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH7_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH7_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH7_AL3_TRANS_COUNT.Reg)
}

// DMA.CH7_AL3_READ_ADDR_TRIG: Alias for channel 7 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH7_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH7_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH7_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH7_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH8_READ_ADDR: DMA Channel 8 Read Address pointer
func (o *DMA_Type) SetCH8_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH8_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH8_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH8_READ_ADDR.Reg)
}

// DMA.CH8_WRITE_ADDR: DMA Channel 8 Write Address pointer
func (o *DMA_Type) SetCH8_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH8_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH8_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH8_WRITE_ADDR.Reg)
}

// DMA.CH8_TRANS_COUNT: DMA Channel 8 Transfer Count
func (o *DMA_Type) SetCH8_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH8_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH8_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH8_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH8_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH8_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH8_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH8_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH8_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH8_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH8_CTRL_TRIG: DMA Channel 8 Control and Status
func (o *DMA_Type) SetCH8_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH8_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH8_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH8_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH8_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH8_AL1_CTRL: Alias for channel 8 CTRL register
func (o *DMA_Type) SetCH8_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH8_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH8_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH8_AL1_CTRL.Reg)
}

// DMA.CH8_AL1_READ_ADDR: Alias for channel 8 READ_ADDR register
func (o *DMA_Type) SetCH8_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH8_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH8_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH8_AL1_READ_ADDR.Reg)
}

// DMA.CH8_AL1_WRITE_ADDR: Alias for channel 8 WRITE_ADDR register
func (o *DMA_Type) SetCH8_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH8_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH8_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH8_AL1_WRITE_ADDR.Reg)
}

// DMA.CH8_AL1_TRANS_COUNT_TRIG: Alias for channel 8 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH8_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH8_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH8_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH8_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH8_AL2_CTRL: Alias for channel 8 CTRL register
func (o *DMA_Type) SetCH8_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH8_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH8_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH8_AL2_CTRL.Reg)
}

// DMA.CH8_AL2_TRANS_COUNT: Alias for channel 8 TRANS_COUNT register
func (o *DMA_Type) SetCH8_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH8_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH8_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH8_AL2_TRANS_COUNT.Reg)
}

// DMA.CH8_AL2_READ_ADDR: Alias for channel 8 READ_ADDR register
func (o *DMA_Type) SetCH8_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH8_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH8_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH8_AL2_READ_ADDR.Reg)
}

// DMA.CH8_AL2_WRITE_ADDR_TRIG: Alias for channel 8 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH8_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH8_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH8_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH8_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH8_AL3_CTRL: Alias for channel 8 CTRL register
func (o *DMA_Type) SetCH8_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH8_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH8_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH8_AL3_CTRL.Reg)
}

// DMA.CH8_AL3_WRITE_ADDR: Alias for channel 8 WRITE_ADDR register
func (o *DMA_Type) SetCH8_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH8_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH8_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH8_AL3_WRITE_ADDR.Reg)
}

// DMA.CH8_AL3_TRANS_COUNT: Alias for channel 8 TRANS_COUNT register
func (o *DMA_Type) SetCH8_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH8_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH8_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH8_AL3_TRANS_COUNT.Reg)
}

// DMA.CH8_AL3_READ_ADDR_TRIG: Alias for channel 8 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH8_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH8_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH8_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH8_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH9_READ_ADDR: DMA Channel 9 Read Address pointer
func (o *DMA_Type) SetCH9_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH9_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH9_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH9_READ_ADDR.Reg)
}

// DMA.CH9_WRITE_ADDR: DMA Channel 9 Write Address pointer
func (o *DMA_Type) SetCH9_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH9_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH9_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH9_WRITE_ADDR.Reg)
}

// DMA.CH9_TRANS_COUNT: DMA Channel 9 Transfer Count
func (o *DMA_Type) SetCH9_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH9_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH9_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH9_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH9_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH9_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH9_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH9_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH9_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH9_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH9_CTRL_TRIG: DMA Channel 9 Control and Status
func (o *DMA_Type) SetCH9_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH9_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH9_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH9_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH9_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH9_AL1_CTRL: Alias for channel 9 CTRL register
func (o *DMA_Type) SetCH9_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH9_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH9_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH9_AL1_CTRL.Reg)
}

// DMA.CH9_AL1_READ_ADDR: Alias for channel 9 READ_ADDR register
func (o *DMA_Type) SetCH9_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH9_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH9_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH9_AL1_READ_ADDR.Reg)
}

// DMA.CH9_AL1_WRITE_ADDR: Alias for channel 9 WRITE_ADDR register
func (o *DMA_Type) SetCH9_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH9_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH9_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH9_AL1_WRITE_ADDR.Reg)
}

// DMA.CH9_AL1_TRANS_COUNT_TRIG: Alias for channel 9 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH9_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH9_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH9_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH9_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH9_AL2_CTRL: Alias for channel 9 CTRL register
func (o *DMA_Type) SetCH9_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH9_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH9_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH9_AL2_CTRL.Reg)
}

// DMA.CH9_AL2_TRANS_COUNT: Alias for channel 9 TRANS_COUNT register
func (o *DMA_Type) SetCH9_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH9_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH9_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH9_AL2_TRANS_COUNT.Reg)
}

// DMA.CH9_AL2_READ_ADDR: Alias for channel 9 READ_ADDR register
func (o *DMA_Type) SetCH9_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH9_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH9_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH9_AL2_READ_ADDR.Reg)
}

// DMA.CH9_AL2_WRITE_ADDR_TRIG: Alias for channel 9 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH9_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH9_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH9_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH9_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH9_AL3_CTRL: Alias for channel 9 CTRL register
func (o *DMA_Type) SetCH9_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH9_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH9_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH9_AL3_CTRL.Reg)
}

// DMA.CH9_AL3_WRITE_ADDR: Alias for channel 9 WRITE_ADDR register
func (o *DMA_Type) SetCH9_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH9_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH9_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH9_AL3_WRITE_ADDR.Reg)
}

// DMA.CH9_AL3_TRANS_COUNT: Alias for channel 9 TRANS_COUNT register
func (o *DMA_Type) SetCH9_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH9_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH9_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH9_AL3_TRANS_COUNT.Reg)
}

// DMA.CH9_AL3_READ_ADDR_TRIG: Alias for channel 9 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH9_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH9_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH9_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH9_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH10_READ_ADDR: DMA Channel 10 Read Address pointer
func (o *DMA_Type) SetCH10_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH10_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH10_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH10_READ_ADDR.Reg)
}

// DMA.CH10_WRITE_ADDR: DMA Channel 10 Write Address pointer
func (o *DMA_Type) SetCH10_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH10_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH10_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH10_WRITE_ADDR.Reg)
}

// DMA.CH10_TRANS_COUNT: DMA Channel 10 Transfer Count
func (o *DMA_Type) SetCH10_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH10_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH10_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH10_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH10_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH10_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH10_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH10_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH10_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH10_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH10_CTRL_TRIG: DMA Channel 10 Control and Status
func (o *DMA_Type) SetCH10_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH10_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH10_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH10_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH10_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH10_AL1_CTRL: Alias for channel 10 CTRL register
func (o *DMA_Type) SetCH10_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH10_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH10_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH10_AL1_CTRL.Reg)
}

// DMA.CH10_AL1_READ_ADDR: Alias for channel 10 READ_ADDR register
func (o *DMA_Type) SetCH10_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH10_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH10_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH10_AL1_READ_ADDR.Reg)
}

// DMA.CH10_AL1_WRITE_ADDR: Alias for channel 10 WRITE_ADDR register
func (o *DMA_Type) SetCH10_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH10_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH10_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH10_AL1_WRITE_ADDR.Reg)
}

// DMA.CH10_AL1_TRANS_COUNT_TRIG: Alias for channel 10 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH10_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH10_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH10_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH10_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH10_AL2_CTRL: Alias for channel 10 CTRL register
func (o *DMA_Type) SetCH10_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH10_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH10_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH10_AL2_CTRL.Reg)
}

// DMA.CH10_AL2_TRANS_COUNT: Alias for channel 10 TRANS_COUNT register
func (o *DMA_Type) SetCH10_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH10_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH10_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH10_AL2_TRANS_COUNT.Reg)
}

// DMA.CH10_AL2_READ_ADDR: Alias for channel 10 READ_ADDR register
func (o *DMA_Type) SetCH10_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH10_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH10_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH10_AL2_READ_ADDR.Reg)
}

// DMA.CH10_AL2_WRITE_ADDR_TRIG: Alias for channel 10 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH10_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH10_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH10_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH10_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH10_AL3_CTRL: Alias for channel 10 CTRL register
func (o *DMA_Type) SetCH10_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH10_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH10_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH10_AL3_CTRL.Reg)
}

// DMA.CH10_AL3_WRITE_ADDR: Alias for channel 10 WRITE_ADDR register
func (o *DMA_Type) SetCH10_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH10_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH10_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH10_AL3_WRITE_ADDR.Reg)
}

// DMA.CH10_AL3_TRANS_COUNT: Alias for channel 10 TRANS_COUNT register
func (o *DMA_Type) SetCH10_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH10_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH10_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH10_AL3_TRANS_COUNT.Reg)
}

// DMA.CH10_AL3_READ_ADDR_TRIG: Alias for channel 10 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH10_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH10_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH10_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH10_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH11_READ_ADDR: DMA Channel 11 Read Address pointer
func (o *DMA_Type) SetCH11_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH11_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH11_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH11_READ_ADDR.Reg)
}

// DMA.CH11_WRITE_ADDR: DMA Channel 11 Write Address pointer
func (o *DMA_Type) SetCH11_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH11_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH11_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH11_WRITE_ADDR.Reg)
}

// DMA.CH11_TRANS_COUNT: DMA Channel 11 Transfer Count
func (o *DMA_Type) SetCH11_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH11_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH11_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH11_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH11_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH11_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH11_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH11_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH11_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH11_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH11_CTRL_TRIG: DMA Channel 11 Control and Status
func (o *DMA_Type) SetCH11_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH11_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH11_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH11_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH11_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH11_AL1_CTRL: Alias for channel 11 CTRL register
func (o *DMA_Type) SetCH11_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH11_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH11_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH11_AL1_CTRL.Reg)
}

// DMA.CH11_AL1_READ_ADDR: Alias for channel 11 READ_ADDR register
func (o *DMA_Type) SetCH11_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH11_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH11_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH11_AL1_READ_ADDR.Reg)
}

// DMA.CH11_AL1_WRITE_ADDR: Alias for channel 11 WRITE_ADDR register
func (o *DMA_Type) SetCH11_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH11_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH11_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH11_AL1_WRITE_ADDR.Reg)
}

// DMA.CH11_AL1_TRANS_COUNT_TRIG: Alias for channel 11 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH11_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH11_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH11_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH11_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH11_AL2_CTRL: Alias for channel 11 CTRL register
func (o *DMA_Type) SetCH11_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH11_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH11_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH11_AL2_CTRL.Reg)
}

// DMA.CH11_AL2_TRANS_COUNT: Alias for channel 11 TRANS_COUNT register
func (o *DMA_Type) SetCH11_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH11_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH11_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH11_AL2_TRANS_COUNT.Reg)
}

// DMA.CH11_AL2_READ_ADDR: Alias for channel 11 READ_ADDR register
func (o *DMA_Type) SetCH11_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH11_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH11_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH11_AL2_READ_ADDR.Reg)
}

// DMA.CH11_AL2_WRITE_ADDR_TRIG: Alias for channel 11 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH11_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH11_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH11_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH11_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH11_AL3_CTRL: Alias for channel 11 CTRL register
func (o *DMA_Type) SetCH11_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH11_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH11_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH11_AL3_CTRL.Reg)
}

// DMA.CH11_AL3_WRITE_ADDR: Alias for channel 11 WRITE_ADDR register
func (o *DMA_Type) SetCH11_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH11_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH11_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH11_AL3_WRITE_ADDR.Reg)
}

// DMA.CH11_AL3_TRANS_COUNT: Alias for channel 11 TRANS_COUNT register
func (o *DMA_Type) SetCH11_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH11_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH11_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH11_AL3_TRANS_COUNT.Reg)
}

// DMA.CH11_AL3_READ_ADDR_TRIG: Alias for channel 11 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH11_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH11_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH11_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH11_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH12_READ_ADDR: DMA Channel 12 Read Address pointer
func (o *DMA_Type) SetCH12_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH12_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH12_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH12_READ_ADDR.Reg)
}

// DMA.CH12_WRITE_ADDR: DMA Channel 12 Write Address pointer
func (o *DMA_Type) SetCH12_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH12_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH12_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH12_WRITE_ADDR.Reg)
}

// DMA.CH12_TRANS_COUNT: DMA Channel 12 Transfer Count
func (o *DMA_Type) SetCH12_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH12_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH12_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH12_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH12_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH12_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH12_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH12_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH12_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH12_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH12_CTRL_TRIG: DMA Channel 12 Control and Status
func (o *DMA_Type) SetCH12_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH12_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH12_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH12_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH12_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH12_AL1_CTRL: Alias for channel 12 CTRL register
func (o *DMA_Type) SetCH12_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH12_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH12_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH12_AL1_CTRL.Reg)
}

// DMA.CH12_AL1_READ_ADDR: Alias for channel 12 READ_ADDR register
func (o *DMA_Type) SetCH12_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH12_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH12_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH12_AL1_READ_ADDR.Reg)
}

// DMA.CH12_AL1_WRITE_ADDR: Alias for channel 12 WRITE_ADDR register
func (o *DMA_Type) SetCH12_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH12_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH12_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH12_AL1_WRITE_ADDR.Reg)
}

// DMA.CH12_AL1_TRANS_COUNT_TRIG: Alias for channel 12 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH12_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH12_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH12_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH12_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH12_AL2_CTRL: Alias for channel 12 CTRL register
func (o *DMA_Type) SetCH12_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH12_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH12_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH12_AL2_CTRL.Reg)
}

// DMA.CH12_AL2_TRANS_COUNT: Alias for channel 12 TRANS_COUNT register
func (o *DMA_Type) SetCH12_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH12_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH12_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH12_AL2_TRANS_COUNT.Reg)
}

// DMA.CH12_AL2_READ_ADDR: Alias for channel 12 READ_ADDR register
func (o *DMA_Type) SetCH12_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH12_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH12_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH12_AL2_READ_ADDR.Reg)
}

// DMA.CH12_AL2_WRITE_ADDR_TRIG: Alias for channel 12 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH12_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH12_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH12_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH12_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH12_AL3_CTRL: Alias for channel 12 CTRL register
func (o *DMA_Type) SetCH12_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH12_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH12_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH12_AL3_CTRL.Reg)
}

// DMA.CH12_AL3_WRITE_ADDR: Alias for channel 12 WRITE_ADDR register
func (o *DMA_Type) SetCH12_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH12_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH12_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH12_AL3_WRITE_ADDR.Reg)
}

// DMA.CH12_AL3_TRANS_COUNT: Alias for channel 12 TRANS_COUNT register
func (o *DMA_Type) SetCH12_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH12_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH12_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH12_AL3_TRANS_COUNT.Reg)
}

// DMA.CH12_AL3_READ_ADDR_TRIG: Alias for channel 12 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH12_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH12_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH12_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH12_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH13_READ_ADDR: DMA Channel 13 Read Address pointer
func (o *DMA_Type) SetCH13_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH13_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH13_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH13_READ_ADDR.Reg)
}

// DMA.CH13_WRITE_ADDR: DMA Channel 13 Write Address pointer
func (o *DMA_Type) SetCH13_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH13_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH13_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH13_WRITE_ADDR.Reg)
}

// DMA.CH13_TRANS_COUNT: DMA Channel 13 Transfer Count
func (o *DMA_Type) SetCH13_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH13_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH13_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH13_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH13_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH13_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH13_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH13_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH13_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH13_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH13_CTRL_TRIG: DMA Channel 13 Control and Status
func (o *DMA_Type) SetCH13_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH13_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH13_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH13_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH13_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH13_AL1_CTRL: Alias for channel 13 CTRL register
func (o *DMA_Type) SetCH13_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH13_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH13_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH13_AL1_CTRL.Reg)
}

// DMA.CH13_AL1_READ_ADDR: Alias for channel 13 READ_ADDR register
func (o *DMA_Type) SetCH13_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH13_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH13_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH13_AL1_READ_ADDR.Reg)
}

// DMA.CH13_AL1_WRITE_ADDR: Alias for channel 13 WRITE_ADDR register
func (o *DMA_Type) SetCH13_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH13_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH13_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH13_AL1_WRITE_ADDR.Reg)
}

// DMA.CH13_AL1_TRANS_COUNT_TRIG: Alias for channel 13 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH13_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH13_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH13_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH13_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH13_AL2_CTRL: Alias for channel 13 CTRL register
func (o *DMA_Type) SetCH13_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH13_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH13_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH13_AL2_CTRL.Reg)
}

// DMA.CH13_AL2_TRANS_COUNT: Alias for channel 13 TRANS_COUNT register
func (o *DMA_Type) SetCH13_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH13_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH13_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH13_AL2_TRANS_COUNT.Reg)
}

// DMA.CH13_AL2_READ_ADDR: Alias for channel 13 READ_ADDR register
func (o *DMA_Type) SetCH13_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH13_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH13_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH13_AL2_READ_ADDR.Reg)
}

// DMA.CH13_AL2_WRITE_ADDR_TRIG: Alias for channel 13 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH13_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH13_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH13_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH13_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH13_AL3_CTRL: Alias for channel 13 CTRL register
func (o *DMA_Type) SetCH13_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH13_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH13_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH13_AL3_CTRL.Reg)
}

// DMA.CH13_AL3_WRITE_ADDR: Alias for channel 13 WRITE_ADDR register
func (o *DMA_Type) SetCH13_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH13_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH13_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH13_AL3_WRITE_ADDR.Reg)
}

// DMA.CH13_AL3_TRANS_COUNT: Alias for channel 13 TRANS_COUNT register
func (o *DMA_Type) SetCH13_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH13_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH13_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH13_AL3_TRANS_COUNT.Reg)
}

// DMA.CH13_AL3_READ_ADDR_TRIG: Alias for channel 13 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH13_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH13_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH13_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH13_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH14_READ_ADDR: DMA Channel 14 Read Address pointer
func (o *DMA_Type) SetCH14_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH14_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH14_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH14_READ_ADDR.Reg)
}

// DMA.CH14_WRITE_ADDR: DMA Channel 14 Write Address pointer
func (o *DMA_Type) SetCH14_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH14_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH14_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH14_WRITE_ADDR.Reg)
}

// DMA.CH14_TRANS_COUNT: DMA Channel 14 Transfer Count
func (o *DMA_Type) SetCH14_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH14_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH14_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH14_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH14_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH14_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH14_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH14_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH14_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH14_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH14_CTRL_TRIG: DMA Channel 14 Control and Status
func (o *DMA_Type) SetCH14_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH14_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH14_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH14_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH14_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH14_AL1_CTRL: Alias for channel 14 CTRL register
func (o *DMA_Type) SetCH14_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH14_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH14_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH14_AL1_CTRL.Reg)
}

// DMA.CH14_AL1_READ_ADDR: Alias for channel 14 READ_ADDR register
func (o *DMA_Type) SetCH14_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH14_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH14_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH14_AL1_READ_ADDR.Reg)
}

// DMA.CH14_AL1_WRITE_ADDR: Alias for channel 14 WRITE_ADDR register
func (o *DMA_Type) SetCH14_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH14_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH14_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH14_AL1_WRITE_ADDR.Reg)
}

// DMA.CH14_AL1_TRANS_COUNT_TRIG: Alias for channel 14 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH14_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH14_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH14_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH14_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH14_AL2_CTRL: Alias for channel 14 CTRL register
func (o *DMA_Type) SetCH14_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH14_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH14_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH14_AL2_CTRL.Reg)
}

// DMA.CH14_AL2_TRANS_COUNT: Alias for channel 14 TRANS_COUNT register
func (o *DMA_Type) SetCH14_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH14_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH14_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH14_AL2_TRANS_COUNT.Reg)
}

// DMA.CH14_AL2_READ_ADDR: Alias for channel 14 READ_ADDR register
func (o *DMA_Type) SetCH14_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH14_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH14_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH14_AL2_READ_ADDR.Reg)
}

// DMA.CH14_AL2_WRITE_ADDR_TRIG: Alias for channel 14 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH14_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH14_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH14_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH14_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH14_AL3_CTRL: Alias for channel 14 CTRL register
func (o *DMA_Type) SetCH14_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH14_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH14_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH14_AL3_CTRL.Reg)
}

// DMA.CH14_AL3_WRITE_ADDR: Alias for channel 14 WRITE_ADDR register
func (o *DMA_Type) SetCH14_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH14_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH14_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH14_AL3_WRITE_ADDR.Reg)
}

// DMA.CH14_AL3_TRANS_COUNT: Alias for channel 14 TRANS_COUNT register
func (o *DMA_Type) SetCH14_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH14_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH14_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH14_AL3_TRANS_COUNT.Reg)
}

// DMA.CH14_AL3_READ_ADDR_TRIG: Alias for channel 14 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH14_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH14_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH14_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH14_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.CH15_READ_ADDR: DMA Channel 15 Read Address pointer
func (o *DMA_Type) SetCH15_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH15_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH15_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH15_READ_ADDR.Reg)
}

// DMA.CH15_WRITE_ADDR: DMA Channel 15 Write Address pointer
func (o *DMA_Type) SetCH15_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH15_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH15_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH15_WRITE_ADDR.Reg)
}

// DMA.CH15_TRANS_COUNT: DMA Channel 15 Transfer Count
func (o *DMA_Type) SetCH15_TRANS_COUNT_MODE(value uint32) {
	volatile.StoreUint32(&o.CH15_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH15_TRANS_COUNT.Reg)&^(0xf0000000)|value<<28)
}
func (o *DMA_Type) GetCH15_TRANS_COUNT_MODE() uint32 {
	return (volatile.LoadUint32(&o.CH15_TRANS_COUNT.Reg) & 0xf0000000) >> 28
}
func (o *DMA_Type) SetCH15_TRANS_COUNT_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH15_TRANS_COUNT.Reg, volatile.LoadUint32(&o.CH15_TRANS_COUNT.Reg)&^(0xfffffff)|value)
}
func (o *DMA_Type) GetCH15_TRANS_COUNT_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH15_TRANS_COUNT.Reg) & 0xfffffff
}

// DMA.CH15_CTRL_TRIG: DMA Channel 15 Control and Status
func (o *DMA_Type) SetCH15_CTRL_TRIG_AHB_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x80000000)|value<<31)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_AHB_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x80000000) >> 31
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_READ_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x40000000)|value<<30)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_READ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x40000000) >> 30
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_WRITE_ERROR(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x20000000)|value<<29)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_WRITE_ERROR() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x20000000) >> 29
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_BUSY(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x4000000)|value<<26)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_BUSY() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x4000000) >> 26
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_SNIFF_EN(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x2000000)|value<<25)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_SNIFF_EN() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x2000000) >> 25
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_BSWAP(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x1000000)|value<<24)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x1000000) >> 24
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_IRQ_QUIET(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x800000)|value<<23)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_IRQ_QUIET() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x800000) >> 23
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_TREQ_SEL(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x7e0000)|value<<17)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_TREQ_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x7e0000) >> 17
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_CHAIN_TO(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x1e000)|value<<13)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_CHAIN_TO() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x1e000) >> 13
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_RING_SEL(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x1000)|value<<12)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_RING_SEL() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x1000) >> 12
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_RING_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0xf00)|value<<8)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_RING_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0xf00) >> 8
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_INCR_WRITE_REV(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_INCR_WRITE_REV() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_INCR_WRITE(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_INCR_WRITE() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_INCR_READ_REV(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_INCR_READ_REV() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_INCR_READ(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_INCR_READ() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_DATA_SIZE(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0xc)|value<<2)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_DATA_SIZE() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0xc) >> 2
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_HIGH_PRIORITY(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_HIGH_PRIORITY() uint32 {
	return (volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetCH15_CTRL_TRIG_EN(value uint32) {
	volatile.StoreUint32(&o.CH15_CTRL_TRIG.Reg, volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetCH15_CTRL_TRIG_EN() uint32 {
	return volatile.LoadUint32(&o.CH15_CTRL_TRIG.Reg) & 0x1
}

// DMA.CH15_AL1_CTRL: Alias for channel 15 CTRL register
func (o *DMA_Type) SetCH15_AL1_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH15_AL1_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH15_AL1_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH15_AL1_CTRL.Reg)
}

// DMA.CH15_AL1_READ_ADDR: Alias for channel 15 READ_ADDR register
func (o *DMA_Type) SetCH15_AL1_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH15_AL1_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH15_AL1_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH15_AL1_READ_ADDR.Reg)
}

// DMA.CH15_AL1_WRITE_ADDR: Alias for channel 15 WRITE_ADDR register
func (o *DMA_Type) SetCH15_AL1_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH15_AL1_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH15_AL1_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH15_AL1_WRITE_ADDR.Reg)
}

// DMA.CH15_AL1_TRANS_COUNT_TRIG: Alias for channel 15 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH15_AL1_TRANS_COUNT_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH15_AL1_TRANS_COUNT_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH15_AL1_TRANS_COUNT_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH15_AL1_TRANS_COUNT_TRIG.Reg)
}

// DMA.CH15_AL2_CTRL: Alias for channel 15 CTRL register
func (o *DMA_Type) SetCH15_AL2_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH15_AL2_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH15_AL2_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH15_AL2_CTRL.Reg)
}

// DMA.CH15_AL2_TRANS_COUNT: Alias for channel 15 TRANS_COUNT register
func (o *DMA_Type) SetCH15_AL2_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH15_AL2_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH15_AL2_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH15_AL2_TRANS_COUNT.Reg)
}

// DMA.CH15_AL2_READ_ADDR: Alias for channel 15 READ_ADDR register
func (o *DMA_Type) SetCH15_AL2_READ_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH15_AL2_READ_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH15_AL2_READ_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH15_AL2_READ_ADDR.Reg)
}

// DMA.CH15_AL2_WRITE_ADDR_TRIG: Alias for channel 15 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH15_AL2_WRITE_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH15_AL2_WRITE_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH15_AL2_WRITE_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH15_AL2_WRITE_ADDR_TRIG.Reg)
}

// DMA.CH15_AL3_CTRL: Alias for channel 15 CTRL register
func (o *DMA_Type) SetCH15_AL3_CTRL(value uint32) {
	volatile.StoreUint32(&o.CH15_AL3_CTRL.Reg, value)
}
func (o *DMA_Type) GetCH15_AL3_CTRL() uint32 {
	return volatile.LoadUint32(&o.CH15_AL3_CTRL.Reg)
}

// DMA.CH15_AL3_WRITE_ADDR: Alias for channel 15 WRITE_ADDR register
func (o *DMA_Type) SetCH15_AL3_WRITE_ADDR(value uint32) {
	volatile.StoreUint32(&o.CH15_AL3_WRITE_ADDR.Reg, value)
}
func (o *DMA_Type) GetCH15_AL3_WRITE_ADDR() uint32 {
	return volatile.LoadUint32(&o.CH15_AL3_WRITE_ADDR.Reg)
}

// DMA.CH15_AL3_TRANS_COUNT: Alias for channel 15 TRANS_COUNT register
func (o *DMA_Type) SetCH15_AL3_TRANS_COUNT(value uint32) {
	volatile.StoreUint32(&o.CH15_AL3_TRANS_COUNT.Reg, value)
}
func (o *DMA_Type) GetCH15_AL3_TRANS_COUNT() uint32 {
	return volatile.LoadUint32(&o.CH15_AL3_TRANS_COUNT.Reg)
}

// DMA.CH15_AL3_READ_ADDR_TRIG: Alias for channel 15 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
func (o *DMA_Type) SetCH15_AL3_READ_ADDR_TRIG(value uint32) {
	volatile.StoreUint32(&o.CH15_AL3_READ_ADDR_TRIG.Reg, value)
}
func (o *DMA_Type) GetCH15_AL3_READ_ADDR_TRIG() uint32 {
	return volatile.LoadUint32(&o.CH15_AL3_READ_ADDR_TRIG.Reg)
}

// DMA.INTR: Interrupt Status (raw)
func (o *DMA_Type) SetINTR(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTR() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0xffff
}

// DMA.INTE0: Interrupt Enables for IRQ 0
func (o *DMA_Type) SetINTE0(value uint32) {
	volatile.StoreUint32(&o.INTE0.Reg, volatile.LoadUint32(&o.INTE0.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTE0() uint32 {
	return volatile.LoadUint32(&o.INTE0.Reg) & 0xffff
}

// DMA.INTF0: Force Interrupts
func (o *DMA_Type) SetINTF0(value uint32) {
	volatile.StoreUint32(&o.INTF0.Reg, volatile.LoadUint32(&o.INTF0.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTF0() uint32 {
	return volatile.LoadUint32(&o.INTF0.Reg) & 0xffff
}

// DMA.INTS0: Interrupt Status for IRQ 0
func (o *DMA_Type) SetINTS0(value uint32) {
	volatile.StoreUint32(&o.INTS0.Reg, volatile.LoadUint32(&o.INTS0.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTS0() uint32 {
	return volatile.LoadUint32(&o.INTS0.Reg) & 0xffff
}

// DMA.INTR1: Interrupt Status (raw)
func (o *DMA_Type) SetINTR1(value uint32) {
	volatile.StoreUint32(&o.INTR1.Reg, volatile.LoadUint32(&o.INTR1.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTR1() uint32 {
	return volatile.LoadUint32(&o.INTR1.Reg) & 0xffff
}

// DMA.INTE1: Interrupt Enables for IRQ 1
func (o *DMA_Type) SetINTE1(value uint32) {
	volatile.StoreUint32(&o.INTE1.Reg, volatile.LoadUint32(&o.INTE1.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTE1() uint32 {
	return volatile.LoadUint32(&o.INTE1.Reg) & 0xffff
}

// DMA.INTF1: Force Interrupts
func (o *DMA_Type) SetINTF1(value uint32) {
	volatile.StoreUint32(&o.INTF1.Reg, volatile.LoadUint32(&o.INTF1.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTF1() uint32 {
	return volatile.LoadUint32(&o.INTF1.Reg) & 0xffff
}

// DMA.INTS1: Interrupt Status for IRQ 1
func (o *DMA_Type) SetINTS1(value uint32) {
	volatile.StoreUint32(&o.INTS1.Reg, volatile.LoadUint32(&o.INTS1.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTS1() uint32 {
	return volatile.LoadUint32(&o.INTS1.Reg) & 0xffff
}

// DMA.INTR2: Interrupt Status (raw)
func (o *DMA_Type) SetINTR2(value uint32) {
	volatile.StoreUint32(&o.INTR2.Reg, volatile.LoadUint32(&o.INTR2.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTR2() uint32 {
	return volatile.LoadUint32(&o.INTR2.Reg) & 0xffff
}

// DMA.INTE2: Interrupt Enables for IRQ 2
func (o *DMA_Type) SetINTE2(value uint32) {
	volatile.StoreUint32(&o.INTE2.Reg, volatile.LoadUint32(&o.INTE2.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTE2() uint32 {
	return volatile.LoadUint32(&o.INTE2.Reg) & 0xffff
}

// DMA.INTF2: Force Interrupts
func (o *DMA_Type) SetINTF2(value uint32) {
	volatile.StoreUint32(&o.INTF2.Reg, volatile.LoadUint32(&o.INTF2.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTF2() uint32 {
	return volatile.LoadUint32(&o.INTF2.Reg) & 0xffff
}

// DMA.INTS2: Interrupt Status for IRQ 2
func (o *DMA_Type) SetINTS2(value uint32) {
	volatile.StoreUint32(&o.INTS2.Reg, volatile.LoadUint32(&o.INTS2.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTS2() uint32 {
	return volatile.LoadUint32(&o.INTS2.Reg) & 0xffff
}

// DMA.INTR3: Interrupt Status (raw)
func (o *DMA_Type) SetINTR3(value uint32) {
	volatile.StoreUint32(&o.INTR3.Reg, volatile.LoadUint32(&o.INTR3.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTR3() uint32 {
	return volatile.LoadUint32(&o.INTR3.Reg) & 0xffff
}

// DMA.INTE3: Interrupt Enables for IRQ 3
func (o *DMA_Type) SetINTE3(value uint32) {
	volatile.StoreUint32(&o.INTE3.Reg, volatile.LoadUint32(&o.INTE3.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTE3() uint32 {
	return volatile.LoadUint32(&o.INTE3.Reg) & 0xffff
}

// DMA.INTF3: Force Interrupts
func (o *DMA_Type) SetINTF3(value uint32) {
	volatile.StoreUint32(&o.INTF3.Reg, volatile.LoadUint32(&o.INTF3.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTF3() uint32 {
	return volatile.LoadUint32(&o.INTF3.Reg) & 0xffff
}

// DMA.INTS3: Interrupt Status for IRQ 3
func (o *DMA_Type) SetINTS3(value uint32) {
	volatile.StoreUint32(&o.INTS3.Reg, volatile.LoadUint32(&o.INTS3.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetINTS3() uint32 {
	return volatile.LoadUint32(&o.INTS3.Reg) & 0xffff
}

// DMA.TIMER0: Pacing (X/Y) fractional timer The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
func (o *DMA_Type) SetTIMER0_X(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0xffff0000)|value<<16)
}
func (o *DMA_Type) GetTIMER0_X() uint32 {
	return (volatile.LoadUint32(&o.TIMER0.Reg) & 0xffff0000) >> 16
}
func (o *DMA_Type) SetTIMER0_Y(value uint32) {
	volatile.StoreUint32(&o.TIMER0.Reg, volatile.LoadUint32(&o.TIMER0.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetTIMER0_Y() uint32 {
	return volatile.LoadUint32(&o.TIMER0.Reg) & 0xffff
}

// DMA.TIMER1: Pacing (X/Y) fractional timer The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
func (o *DMA_Type) SetTIMER1_X(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0xffff0000)|value<<16)
}
func (o *DMA_Type) GetTIMER1_X() uint32 {
	return (volatile.LoadUint32(&o.TIMER1.Reg) & 0xffff0000) >> 16
}
func (o *DMA_Type) SetTIMER1_Y(value uint32) {
	volatile.StoreUint32(&o.TIMER1.Reg, volatile.LoadUint32(&o.TIMER1.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetTIMER1_Y() uint32 {
	return volatile.LoadUint32(&o.TIMER1.Reg) & 0xffff
}

// DMA.TIMER2: Pacing (X/Y) fractional timer The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
func (o *DMA_Type) SetTIMER2_X(value uint32) {
	volatile.StoreUint32(&o.TIMER2.Reg, volatile.LoadUint32(&o.TIMER2.Reg)&^(0xffff0000)|value<<16)
}
func (o *DMA_Type) GetTIMER2_X() uint32 {
	return (volatile.LoadUint32(&o.TIMER2.Reg) & 0xffff0000) >> 16
}
func (o *DMA_Type) SetTIMER2_Y(value uint32) {
	volatile.StoreUint32(&o.TIMER2.Reg, volatile.LoadUint32(&o.TIMER2.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetTIMER2_Y() uint32 {
	return volatile.LoadUint32(&o.TIMER2.Reg) & 0xffff
}

// DMA.TIMER3: Pacing (X/Y) fractional timer The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
func (o *DMA_Type) SetTIMER3_X(value uint32) {
	volatile.StoreUint32(&o.TIMER3.Reg, volatile.LoadUint32(&o.TIMER3.Reg)&^(0xffff0000)|value<<16)
}
func (o *DMA_Type) GetTIMER3_X() uint32 {
	return (volatile.LoadUint32(&o.TIMER3.Reg) & 0xffff0000) >> 16
}
func (o *DMA_Type) SetTIMER3_Y(value uint32) {
	volatile.StoreUint32(&o.TIMER3.Reg, volatile.LoadUint32(&o.TIMER3.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetTIMER3_Y() uint32 {
	return volatile.LoadUint32(&o.TIMER3.Reg) & 0xffff
}

// DMA.MULTI_CHAN_TRIGGER: Trigger one or more channels simultaneously
func (o *DMA_Type) SetMULTI_CHAN_TRIGGER(value uint32) {
	volatile.StoreUint32(&o.MULTI_CHAN_TRIGGER.Reg, volatile.LoadUint32(&o.MULTI_CHAN_TRIGGER.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetMULTI_CHAN_TRIGGER() uint32 {
	return volatile.LoadUint32(&o.MULTI_CHAN_TRIGGER.Reg) & 0xffff
}

// DMA.SNIFF_CTRL: Sniffer Control
func (o *DMA_Type) SetSNIFF_CTRL_OUT_INV(value uint32) {
	volatile.StoreUint32(&o.SNIFF_CTRL.Reg, volatile.LoadUint32(&o.SNIFF_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *DMA_Type) GetSNIFF_CTRL_OUT_INV() uint32 {
	return (volatile.LoadUint32(&o.SNIFF_CTRL.Reg) & 0x800) >> 11
}
func (o *DMA_Type) SetSNIFF_CTRL_OUT_REV(value uint32) {
	volatile.StoreUint32(&o.SNIFF_CTRL.Reg, volatile.LoadUint32(&o.SNIFF_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *DMA_Type) GetSNIFF_CTRL_OUT_REV() uint32 {
	return (volatile.LoadUint32(&o.SNIFF_CTRL.Reg) & 0x400) >> 10
}
func (o *DMA_Type) SetSNIFF_CTRL_BSWAP(value uint32) {
	volatile.StoreUint32(&o.SNIFF_CTRL.Reg, volatile.LoadUint32(&o.SNIFF_CTRL.Reg)&^(0x200)|value<<9)
}
func (o *DMA_Type) GetSNIFF_CTRL_BSWAP() uint32 {
	return (volatile.LoadUint32(&o.SNIFF_CTRL.Reg) & 0x200) >> 9
}
func (o *DMA_Type) SetSNIFF_CTRL_CALC(value uint32) {
	volatile.StoreUint32(&o.SNIFF_CTRL.Reg, volatile.LoadUint32(&o.SNIFF_CTRL.Reg)&^(0x1e0)|value<<5)
}
func (o *DMA_Type) GetSNIFF_CTRL_CALC() uint32 {
	return (volatile.LoadUint32(&o.SNIFF_CTRL.Reg) & 0x1e0) >> 5
}
func (o *DMA_Type) SetSNIFF_CTRL_DMACH(value uint32) {
	volatile.StoreUint32(&o.SNIFF_CTRL.Reg, volatile.LoadUint32(&o.SNIFF_CTRL.Reg)&^(0x1e)|value<<1)
}
func (o *DMA_Type) GetSNIFF_CTRL_DMACH() uint32 {
	return (volatile.LoadUint32(&o.SNIFF_CTRL.Reg) & 0x1e) >> 1
}
func (o *DMA_Type) SetSNIFF_CTRL_EN(value uint32) {
	volatile.StoreUint32(&o.SNIFF_CTRL.Reg, volatile.LoadUint32(&o.SNIFF_CTRL.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSNIFF_CTRL_EN() uint32 {
	return volatile.LoadUint32(&o.SNIFF_CTRL.Reg) & 0x1
}

// DMA.SNIFF_DATA: Data accumulator for sniff hardware
func (o *DMA_Type) SetSNIFF_DATA(value uint32) {
	volatile.StoreUint32(&o.SNIFF_DATA.Reg, value)
}
func (o *DMA_Type) GetSNIFF_DATA() uint32 {
	return volatile.LoadUint32(&o.SNIFF_DATA.Reg)
}

// DMA.FIFO_LEVELS: Debug RAF, WAF, TDF levels
func (o *DMA_Type) SetFIFO_LEVELS_RAF_LVL(value uint32) {
	volatile.StoreUint32(&o.FIFO_LEVELS.Reg, volatile.LoadUint32(&o.FIFO_LEVELS.Reg)&^(0xff0000)|value<<16)
}
func (o *DMA_Type) GetFIFO_LEVELS_RAF_LVL() uint32 {
	return (volatile.LoadUint32(&o.FIFO_LEVELS.Reg) & 0xff0000) >> 16
}
func (o *DMA_Type) SetFIFO_LEVELS_WAF_LVL(value uint32) {
	volatile.StoreUint32(&o.FIFO_LEVELS.Reg, volatile.LoadUint32(&o.FIFO_LEVELS.Reg)&^(0xff00)|value<<8)
}
func (o *DMA_Type) GetFIFO_LEVELS_WAF_LVL() uint32 {
	return (volatile.LoadUint32(&o.FIFO_LEVELS.Reg) & 0xff00) >> 8
}
func (o *DMA_Type) SetFIFO_LEVELS_TDF_LVL(value uint32) {
	volatile.StoreUint32(&o.FIFO_LEVELS.Reg, volatile.LoadUint32(&o.FIFO_LEVELS.Reg)&^(0xff)|value)
}
func (o *DMA_Type) GetFIFO_LEVELS_TDF_LVL() uint32 {
	return volatile.LoadUint32(&o.FIFO_LEVELS.Reg) & 0xff
}

// DMA.CHAN_ABORT: Abort an in-progress transfer sequence on one or more channels
func (o *DMA_Type) SetCHAN_ABORT(value uint32) {
	volatile.StoreUint32(&o.CHAN_ABORT.Reg, volatile.LoadUint32(&o.CHAN_ABORT.Reg)&^(0xffff)|value)
}
func (o *DMA_Type) GetCHAN_ABORT() uint32 {
	return volatile.LoadUint32(&o.CHAN_ABORT.Reg) & 0xffff
}

// DMA.N_CHANNELS: The number of channels this DMA instance is equipped with. This DMA supports up to 16 hardware channels, but can be configured with as few as one, to minimise silicon area.
func (o *DMA_Type) SetN_CHANNELS(value uint32) {
	volatile.StoreUint32(&o.N_CHANNELS.Reg, volatile.LoadUint32(&o.N_CHANNELS.Reg)&^(0x1f)|value)
}
func (o *DMA_Type) GetN_CHANNELS() uint32 {
	return volatile.LoadUint32(&o.N_CHANNELS.Reg) & 0x1f
}

// DMA.SECCFG_CH0: Security configuration for channel 0. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH0_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH0.Reg, volatile.LoadUint32(&o.SECCFG_CH0.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH0_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH0.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH0_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH0.Reg, volatile.LoadUint32(&o.SECCFG_CH0.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH0_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH0.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH0_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH0.Reg, volatile.LoadUint32(&o.SECCFG_CH0.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH0_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH0.Reg) & 0x1
}

// DMA.SECCFG_CH1: Security configuration for channel 1. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH1_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH1.Reg, volatile.LoadUint32(&o.SECCFG_CH1.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH1_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH1.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH1_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH1.Reg, volatile.LoadUint32(&o.SECCFG_CH1.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH1_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH1.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH1_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH1.Reg, volatile.LoadUint32(&o.SECCFG_CH1.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH1_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH1.Reg) & 0x1
}

// DMA.SECCFG_CH2: Security configuration for channel 2. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH2_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH2.Reg, volatile.LoadUint32(&o.SECCFG_CH2.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH2_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH2.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH2_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH2.Reg, volatile.LoadUint32(&o.SECCFG_CH2.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH2_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH2.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH2_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH2.Reg, volatile.LoadUint32(&o.SECCFG_CH2.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH2_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH2.Reg) & 0x1
}

// DMA.SECCFG_CH3: Security configuration for channel 3. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH3_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH3.Reg, volatile.LoadUint32(&o.SECCFG_CH3.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH3_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH3.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH3_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH3.Reg, volatile.LoadUint32(&o.SECCFG_CH3.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH3_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH3.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH3_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH3.Reg, volatile.LoadUint32(&o.SECCFG_CH3.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH3_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH3.Reg) & 0x1
}

// DMA.SECCFG_CH4: Security configuration for channel 4. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH4_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH4.Reg, volatile.LoadUint32(&o.SECCFG_CH4.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH4_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH4.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH4_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH4.Reg, volatile.LoadUint32(&o.SECCFG_CH4.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH4_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH4.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH4_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH4.Reg, volatile.LoadUint32(&o.SECCFG_CH4.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH4_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH4.Reg) & 0x1
}

// DMA.SECCFG_CH5: Security configuration for channel 5. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH5_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH5.Reg, volatile.LoadUint32(&o.SECCFG_CH5.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH5_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH5.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH5_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH5.Reg, volatile.LoadUint32(&o.SECCFG_CH5.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH5_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH5.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH5_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH5.Reg, volatile.LoadUint32(&o.SECCFG_CH5.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH5_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH5.Reg) & 0x1
}

// DMA.SECCFG_CH6: Security configuration for channel 6. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH6_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH6.Reg, volatile.LoadUint32(&o.SECCFG_CH6.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH6_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH6.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH6_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH6.Reg, volatile.LoadUint32(&o.SECCFG_CH6.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH6_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH6.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH6_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH6.Reg, volatile.LoadUint32(&o.SECCFG_CH6.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH6_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH6.Reg) & 0x1
}

// DMA.SECCFG_CH7: Security configuration for channel 7. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH7_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH7.Reg, volatile.LoadUint32(&o.SECCFG_CH7.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH7_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH7.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH7_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH7.Reg, volatile.LoadUint32(&o.SECCFG_CH7.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH7_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH7.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH7_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH7.Reg, volatile.LoadUint32(&o.SECCFG_CH7.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH7_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH7.Reg) & 0x1
}

// DMA.SECCFG_CH8: Security configuration for channel 8. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH8_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH8.Reg, volatile.LoadUint32(&o.SECCFG_CH8.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH8_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH8.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH8_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH8.Reg, volatile.LoadUint32(&o.SECCFG_CH8.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH8_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH8.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH8_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH8.Reg, volatile.LoadUint32(&o.SECCFG_CH8.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH8_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH8.Reg) & 0x1
}

// DMA.SECCFG_CH9: Security configuration for channel 9. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH9_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH9.Reg, volatile.LoadUint32(&o.SECCFG_CH9.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH9_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH9.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH9_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH9.Reg, volatile.LoadUint32(&o.SECCFG_CH9.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH9_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH9.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH9_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH9.Reg, volatile.LoadUint32(&o.SECCFG_CH9.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH9_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH9.Reg) & 0x1
}

// DMA.SECCFG_CH10: Security configuration for channel 10. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH10_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH10.Reg, volatile.LoadUint32(&o.SECCFG_CH10.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH10_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH10.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH10_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH10.Reg, volatile.LoadUint32(&o.SECCFG_CH10.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH10_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH10.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH10_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH10.Reg, volatile.LoadUint32(&o.SECCFG_CH10.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH10_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH10.Reg) & 0x1
}

// DMA.SECCFG_CH11: Security configuration for channel 11. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH11_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH11.Reg, volatile.LoadUint32(&o.SECCFG_CH11.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH11_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH11.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH11_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH11.Reg, volatile.LoadUint32(&o.SECCFG_CH11.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH11_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH11.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH11_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH11.Reg, volatile.LoadUint32(&o.SECCFG_CH11.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH11_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH11.Reg) & 0x1
}

// DMA.SECCFG_CH12: Security configuration for channel 12. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH12_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH12.Reg, volatile.LoadUint32(&o.SECCFG_CH12.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH12_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH12.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH12_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH12.Reg, volatile.LoadUint32(&o.SECCFG_CH12.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH12_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH12.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH12_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH12.Reg, volatile.LoadUint32(&o.SECCFG_CH12.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH12_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH12.Reg) & 0x1
}

// DMA.SECCFG_CH13: Security configuration for channel 13. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH13_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH13.Reg, volatile.LoadUint32(&o.SECCFG_CH13.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH13_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH13.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH13_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH13.Reg, volatile.LoadUint32(&o.SECCFG_CH13.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH13_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH13.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH13_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH13.Reg, volatile.LoadUint32(&o.SECCFG_CH13.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH13_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH13.Reg) & 0x1
}

// DMA.SECCFG_CH14: Security configuration for channel 14. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH14_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH14.Reg, volatile.LoadUint32(&o.SECCFG_CH14.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH14_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH14.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH14_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH14.Reg, volatile.LoadUint32(&o.SECCFG_CH14.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH14_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH14.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH14_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH14.Reg, volatile.LoadUint32(&o.SECCFG_CH14.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH14_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH14.Reg) & 0x1
}

// DMA.SECCFG_CH15: Security configuration for channel 15. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
func (o *DMA_Type) SetSECCFG_CH15_LOCK(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH15.Reg, volatile.LoadUint32(&o.SECCFG_CH15.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_CH15_LOCK() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH15.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_CH15_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH15.Reg, volatile.LoadUint32(&o.SECCFG_CH15.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_CH15_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_CH15.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_CH15_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_CH15.Reg, volatile.LoadUint32(&o.SECCFG_CH15.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_CH15_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_CH15.Reg) & 0x1
}

// DMA.SECCFG_IRQ0: Security configuration for IRQ 0. Control whether the IRQ permits configuration by Non-secure/Unprivileged contexts, and whether it can observe Secure/Privileged channel interrupt flags.
func (o *DMA_Type) SetSECCFG_IRQ0_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_IRQ0.Reg, volatile.LoadUint32(&o.SECCFG_IRQ0.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_IRQ0_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_IRQ0.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_IRQ0_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_IRQ0.Reg, volatile.LoadUint32(&o.SECCFG_IRQ0.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_IRQ0_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_IRQ0.Reg) & 0x1
}

// DMA.SECCFG_IRQ1: Security configuration for IRQ 1. Control whether the IRQ permits configuration by Non-secure/Unprivileged contexts, and whether it can observe Secure/Privileged channel interrupt flags.
func (o *DMA_Type) SetSECCFG_IRQ1_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_IRQ1.Reg, volatile.LoadUint32(&o.SECCFG_IRQ1.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_IRQ1_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_IRQ1.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_IRQ1_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_IRQ1.Reg, volatile.LoadUint32(&o.SECCFG_IRQ1.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_IRQ1_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_IRQ1.Reg) & 0x1
}

// DMA.SECCFG_IRQ2: Security configuration for IRQ 2. Control whether the IRQ permits configuration by Non-secure/Unprivileged contexts, and whether it can observe Secure/Privileged channel interrupt flags.
func (o *DMA_Type) SetSECCFG_IRQ2_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_IRQ2.Reg, volatile.LoadUint32(&o.SECCFG_IRQ2.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_IRQ2_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_IRQ2.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_IRQ2_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_IRQ2.Reg, volatile.LoadUint32(&o.SECCFG_IRQ2.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_IRQ2_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_IRQ2.Reg) & 0x1
}

// DMA.SECCFG_IRQ3: Security configuration for IRQ 3. Control whether the IRQ permits configuration by Non-secure/Unprivileged contexts, and whether it can observe Secure/Privileged channel interrupt flags.
func (o *DMA_Type) SetSECCFG_IRQ3_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_IRQ3.Reg, volatile.LoadUint32(&o.SECCFG_IRQ3.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_IRQ3_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_IRQ3.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_IRQ3_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_IRQ3.Reg, volatile.LoadUint32(&o.SECCFG_IRQ3.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_IRQ3_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_IRQ3.Reg) & 0x1
}

// DMA.SECCFG_MISC: Miscellaneous security configuration
func (o *DMA_Type) SetSECCFG_MISC_TIMER3_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x200)|value<<9)
}
func (o *DMA_Type) GetSECCFG_MISC_TIMER3_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x200) >> 9
}
func (o *DMA_Type) SetSECCFG_MISC_TIMER3_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x100)|value<<8)
}
func (o *DMA_Type) GetSECCFG_MISC_TIMER3_P() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x100) >> 8
}
func (o *DMA_Type) SetSECCFG_MISC_TIMER2_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x80)|value<<7)
}
func (o *DMA_Type) GetSECCFG_MISC_TIMER2_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x80) >> 7
}
func (o *DMA_Type) SetSECCFG_MISC_TIMER2_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x40)|value<<6)
}
func (o *DMA_Type) GetSECCFG_MISC_TIMER2_P() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x40) >> 6
}
func (o *DMA_Type) SetSECCFG_MISC_TIMER1_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x20)|value<<5)
}
func (o *DMA_Type) GetSECCFG_MISC_TIMER1_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x20) >> 5
}
func (o *DMA_Type) SetSECCFG_MISC_TIMER1_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x10)|value<<4)
}
func (o *DMA_Type) GetSECCFG_MISC_TIMER1_P() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x10) >> 4
}
func (o *DMA_Type) SetSECCFG_MISC_TIMER0_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x8)|value<<3)
}
func (o *DMA_Type) GetSECCFG_MISC_TIMER0_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x8) >> 3
}
func (o *DMA_Type) SetSECCFG_MISC_TIMER0_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetSECCFG_MISC_TIMER0_P() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetSECCFG_MISC_SNIFF_S(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetSECCFG_MISC_SNIFF_S() uint32 {
	return (volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetSECCFG_MISC_SNIFF_P(value uint32) {
	volatile.StoreUint32(&o.SECCFG_MISC.Reg, volatile.LoadUint32(&o.SECCFG_MISC.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetSECCFG_MISC_SNIFF_P() uint32 {
	return volatile.LoadUint32(&o.SECCFG_MISC.Reg) & 0x1
}

// DMA.MPU_CTRL: Control register for DMA MPU. Accessible only from a Privileged context.
func (o *DMA_Type) SetMPU_CTRL_NS_HIDE_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_CTRL.Reg, volatile.LoadUint32(&o.MPU_CTRL.Reg)&^(0x8)|value<<3)
}
func (o *DMA_Type) GetMPU_CTRL_NS_HIDE_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_CTRL.Reg) & 0x8) >> 3
}
func (o *DMA_Type) SetMPU_CTRL_S(value uint32) {
	volatile.StoreUint32(&o.MPU_CTRL.Reg, volatile.LoadUint32(&o.MPU_CTRL.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetMPU_CTRL_S() uint32 {
	return (volatile.LoadUint32(&o.MPU_CTRL.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetMPU_CTRL_P(value uint32) {
	volatile.StoreUint32(&o.MPU_CTRL.Reg, volatile.LoadUint32(&o.MPU_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetMPU_CTRL_P() uint32 {
	return (volatile.LoadUint32(&o.MPU_CTRL.Reg) & 0x2) >> 1
}

// DMA.MPU_BAR0: Base address register for MPU region 0. Writable only from a Secure, Privileged context.
func (o *DMA_Type) SetMPU_BAR0_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_BAR0.Reg, volatile.LoadUint32(&o.MPU_BAR0.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_BAR0_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_BAR0.Reg) & 0xffffffe0) >> 5
}

// DMA.MPU_LAR0: Limit address register for MPU region 0. Writable only from a Secure, Privileged context, with the exception of the P bit.
func (o *DMA_Type) SetMPU_LAR0_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR0.Reg, volatile.LoadUint32(&o.MPU_LAR0.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_LAR0_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR0.Reg) & 0xffffffe0) >> 5
}
func (o *DMA_Type) SetMPU_LAR0_S(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR0.Reg, volatile.LoadUint32(&o.MPU_LAR0.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetMPU_LAR0_S() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR0.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetMPU_LAR0_P(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR0.Reg, volatile.LoadUint32(&o.MPU_LAR0.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetMPU_LAR0_P() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR0.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetMPU_LAR0_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR0.Reg, volatile.LoadUint32(&o.MPU_LAR0.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetMPU_LAR0_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_LAR0.Reg) & 0x1
}

// DMA.MPU_BAR1: Base address register for MPU region 1. Writable only from a Secure, Privileged context.
func (o *DMA_Type) SetMPU_BAR1_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_BAR1.Reg, volatile.LoadUint32(&o.MPU_BAR1.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_BAR1_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_BAR1.Reg) & 0xffffffe0) >> 5
}

// DMA.MPU_LAR1: Limit address register for MPU region 1. Writable only from a Secure, Privileged context, with the exception of the P bit.
func (o *DMA_Type) SetMPU_LAR1_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR1.Reg, volatile.LoadUint32(&o.MPU_LAR1.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_LAR1_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR1.Reg) & 0xffffffe0) >> 5
}
func (o *DMA_Type) SetMPU_LAR1_S(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR1.Reg, volatile.LoadUint32(&o.MPU_LAR1.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetMPU_LAR1_S() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR1.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetMPU_LAR1_P(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR1.Reg, volatile.LoadUint32(&o.MPU_LAR1.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetMPU_LAR1_P() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR1.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetMPU_LAR1_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR1.Reg, volatile.LoadUint32(&o.MPU_LAR1.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetMPU_LAR1_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_LAR1.Reg) & 0x1
}

// DMA.MPU_BAR2: Base address register for MPU region 2. Writable only from a Secure, Privileged context.
func (o *DMA_Type) SetMPU_BAR2_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_BAR2.Reg, volatile.LoadUint32(&o.MPU_BAR2.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_BAR2_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_BAR2.Reg) & 0xffffffe0) >> 5
}

// DMA.MPU_LAR2: Limit address register for MPU region 2. Writable only from a Secure, Privileged context, with the exception of the P bit.
func (o *DMA_Type) SetMPU_LAR2_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR2.Reg, volatile.LoadUint32(&o.MPU_LAR2.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_LAR2_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR2.Reg) & 0xffffffe0) >> 5
}
func (o *DMA_Type) SetMPU_LAR2_S(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR2.Reg, volatile.LoadUint32(&o.MPU_LAR2.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetMPU_LAR2_S() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR2.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetMPU_LAR2_P(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR2.Reg, volatile.LoadUint32(&o.MPU_LAR2.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetMPU_LAR2_P() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR2.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetMPU_LAR2_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR2.Reg, volatile.LoadUint32(&o.MPU_LAR2.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetMPU_LAR2_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_LAR2.Reg) & 0x1
}

// DMA.MPU_BAR3: Base address register for MPU region 3. Writable only from a Secure, Privileged context.
func (o *DMA_Type) SetMPU_BAR3_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_BAR3.Reg, volatile.LoadUint32(&o.MPU_BAR3.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_BAR3_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_BAR3.Reg) & 0xffffffe0) >> 5
}

// DMA.MPU_LAR3: Limit address register for MPU region 3. Writable only from a Secure, Privileged context, with the exception of the P bit.
func (o *DMA_Type) SetMPU_LAR3_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR3.Reg, volatile.LoadUint32(&o.MPU_LAR3.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_LAR3_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR3.Reg) & 0xffffffe0) >> 5
}
func (o *DMA_Type) SetMPU_LAR3_S(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR3.Reg, volatile.LoadUint32(&o.MPU_LAR3.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetMPU_LAR3_S() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR3.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetMPU_LAR3_P(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR3.Reg, volatile.LoadUint32(&o.MPU_LAR3.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetMPU_LAR3_P() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR3.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetMPU_LAR3_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR3.Reg, volatile.LoadUint32(&o.MPU_LAR3.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetMPU_LAR3_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_LAR3.Reg) & 0x1
}

// DMA.MPU_BAR4: Base address register for MPU region 4. Writable only from a Secure, Privileged context.
func (o *DMA_Type) SetMPU_BAR4_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_BAR4.Reg, volatile.LoadUint32(&o.MPU_BAR4.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_BAR4_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_BAR4.Reg) & 0xffffffe0) >> 5
}

// DMA.MPU_LAR4: Limit address register for MPU region 4. Writable only from a Secure, Privileged context, with the exception of the P bit.
func (o *DMA_Type) SetMPU_LAR4_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR4.Reg, volatile.LoadUint32(&o.MPU_LAR4.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_LAR4_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR4.Reg) & 0xffffffe0) >> 5
}
func (o *DMA_Type) SetMPU_LAR4_S(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR4.Reg, volatile.LoadUint32(&o.MPU_LAR4.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetMPU_LAR4_S() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR4.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetMPU_LAR4_P(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR4.Reg, volatile.LoadUint32(&o.MPU_LAR4.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetMPU_LAR4_P() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR4.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetMPU_LAR4_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR4.Reg, volatile.LoadUint32(&o.MPU_LAR4.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetMPU_LAR4_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_LAR4.Reg) & 0x1
}

// DMA.MPU_BAR5: Base address register for MPU region 5. Writable only from a Secure, Privileged context.
func (o *DMA_Type) SetMPU_BAR5_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_BAR5.Reg, volatile.LoadUint32(&o.MPU_BAR5.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_BAR5_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_BAR5.Reg) & 0xffffffe0) >> 5
}

// DMA.MPU_LAR5: Limit address register for MPU region 5. Writable only from a Secure, Privileged context, with the exception of the P bit.
func (o *DMA_Type) SetMPU_LAR5_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR5.Reg, volatile.LoadUint32(&o.MPU_LAR5.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_LAR5_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR5.Reg) & 0xffffffe0) >> 5
}
func (o *DMA_Type) SetMPU_LAR5_S(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR5.Reg, volatile.LoadUint32(&o.MPU_LAR5.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetMPU_LAR5_S() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR5.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetMPU_LAR5_P(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR5.Reg, volatile.LoadUint32(&o.MPU_LAR5.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetMPU_LAR5_P() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR5.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetMPU_LAR5_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR5.Reg, volatile.LoadUint32(&o.MPU_LAR5.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetMPU_LAR5_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_LAR5.Reg) & 0x1
}

// DMA.MPU_BAR6: Base address register for MPU region 6. Writable only from a Secure, Privileged context.
func (o *DMA_Type) SetMPU_BAR6_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_BAR6.Reg, volatile.LoadUint32(&o.MPU_BAR6.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_BAR6_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_BAR6.Reg) & 0xffffffe0) >> 5
}

// DMA.MPU_LAR6: Limit address register for MPU region 6. Writable only from a Secure, Privileged context, with the exception of the P bit.
func (o *DMA_Type) SetMPU_LAR6_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR6.Reg, volatile.LoadUint32(&o.MPU_LAR6.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_LAR6_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR6.Reg) & 0xffffffe0) >> 5
}
func (o *DMA_Type) SetMPU_LAR6_S(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR6.Reg, volatile.LoadUint32(&o.MPU_LAR6.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetMPU_LAR6_S() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR6.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetMPU_LAR6_P(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR6.Reg, volatile.LoadUint32(&o.MPU_LAR6.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetMPU_LAR6_P() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR6.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetMPU_LAR6_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR6.Reg, volatile.LoadUint32(&o.MPU_LAR6.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetMPU_LAR6_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_LAR6.Reg) & 0x1
}

// DMA.MPU_BAR7: Base address register for MPU region 7. Writable only from a Secure, Privileged context.
func (o *DMA_Type) SetMPU_BAR7_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_BAR7.Reg, volatile.LoadUint32(&o.MPU_BAR7.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_BAR7_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_BAR7.Reg) & 0xffffffe0) >> 5
}

// DMA.MPU_LAR7: Limit address register for MPU region 7. Writable only from a Secure, Privileged context, with the exception of the P bit.
func (o *DMA_Type) SetMPU_LAR7_ADDR(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR7.Reg, volatile.LoadUint32(&o.MPU_LAR7.Reg)&^(0xffffffe0)|value<<5)
}
func (o *DMA_Type) GetMPU_LAR7_ADDR() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR7.Reg) & 0xffffffe0) >> 5
}
func (o *DMA_Type) SetMPU_LAR7_S(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR7.Reg, volatile.LoadUint32(&o.MPU_LAR7.Reg)&^(0x4)|value<<2)
}
func (o *DMA_Type) GetMPU_LAR7_S() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR7.Reg) & 0x4) >> 2
}
func (o *DMA_Type) SetMPU_LAR7_P(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR7.Reg, volatile.LoadUint32(&o.MPU_LAR7.Reg)&^(0x2)|value<<1)
}
func (o *DMA_Type) GetMPU_LAR7_P() uint32 {
	return (volatile.LoadUint32(&o.MPU_LAR7.Reg) & 0x2) >> 1
}
func (o *DMA_Type) SetMPU_LAR7_EN(value uint32) {
	volatile.StoreUint32(&o.MPU_LAR7.Reg, volatile.LoadUint32(&o.MPU_LAR7.Reg)&^(0x1)|value)
}
func (o *DMA_Type) GetMPU_LAR7_EN() uint32 {
	return volatile.LoadUint32(&o.MPU_LAR7.Reg) & 0x1
}

// DMA.CH0_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH0_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH0_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH0_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH0_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH0_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH0_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH0_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH0_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH0_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH0_DBG_TCR.Reg)
}

// DMA.CH1_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH1_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH1_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH1_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH1_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH1_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH1_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH1_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH1_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH1_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH1_DBG_TCR.Reg)
}

// DMA.CH2_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH2_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH2_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH2_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH2_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH2_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH2_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH2_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH2_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH2_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH2_DBG_TCR.Reg)
}

// DMA.CH3_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH3_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH3_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH3_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH3_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH3_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH3_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH3_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH3_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH3_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH3_DBG_TCR.Reg)
}

// DMA.CH4_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH4_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH4_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH4_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH4_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH4_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH4_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH4_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH4_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH4_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH4_DBG_TCR.Reg)
}

// DMA.CH5_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH5_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH5_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH5_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH5_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH5_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH5_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH5_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH5_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH5_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH5_DBG_TCR.Reg)
}

// DMA.CH6_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH6_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH6_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH6_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH6_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH6_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH6_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH6_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH6_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH6_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH6_DBG_TCR.Reg)
}

// DMA.CH7_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH7_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH7_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH7_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH7_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH7_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH7_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH7_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH7_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH7_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH7_DBG_TCR.Reg)
}

// DMA.CH8_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH8_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH8_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH8_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH8_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH8_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH8_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH8_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH8_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH8_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH8_DBG_TCR.Reg)
}

// DMA.CH9_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH9_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH9_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH9_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH9_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH9_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH9_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH9_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH9_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH9_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH9_DBG_TCR.Reg)
}

// DMA.CH10_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH10_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH10_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH10_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH10_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH10_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH10_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH10_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH10_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH10_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH10_DBG_TCR.Reg)
}

// DMA.CH11_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH11_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH11_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH11_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH11_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH11_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH11_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH11_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH11_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH11_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH11_DBG_TCR.Reg)
}

// DMA.CH12_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH12_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH12_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH12_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH12_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH12_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH12_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH12_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH12_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH12_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH12_DBG_TCR.Reg)
}

// DMA.CH13_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH13_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH13_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH13_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH13_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH13_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH13_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH13_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH13_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH13_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH13_DBG_TCR.Reg)
}

// DMA.CH14_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH14_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH14_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH14_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH14_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH14_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH14_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH14_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH14_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH14_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH14_DBG_TCR.Reg)
}

// DMA.CH15_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
func (o *DMA_Type) SetCH15_DBG_CTDREQ(value uint32) {
	volatile.StoreUint32(&o.CH15_DBG_CTDREQ.Reg, volatile.LoadUint32(&o.CH15_DBG_CTDREQ.Reg)&^(0x3f)|value)
}
func (o *DMA_Type) GetCH15_DBG_CTDREQ() uint32 {
	return volatile.LoadUint32(&o.CH15_DBG_CTDREQ.Reg) & 0x3f
}

// DMA.CH15_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
func (o *DMA_Type) SetCH15_DBG_TCR(value uint32) {
	volatile.StoreUint32(&o.CH15_DBG_TCR.Reg, value)
}
func (o *DMA_Type) GetCH15_DBG_TCR() uint32 {
	return volatile.LoadUint32(&o.CH15_DBG_TCR.Reg)
}

// Controls time and alarms time is a 64 bit value indicating the time since power-on timeh is the top 32 bits of time & timel is the bottom 32 bits to change time write to timelw before timehw to read time read from timelr before timehr An alarm is set by setting alarm_enable and writing to the corresponding alarm register When an alarm is pending, the corresponding alarm_running signal will be high An alarm can be cancelled before it has finished by clearing the alarm_enable When an alarm fires, the corresponding alarm_irq is set and alarm_running is cleared To clear the interrupt write a 1 to the corresponding alarm_irq The timer can be locked to prevent writing
type TIMER0_Type struct {
	TIMEHW   volatile.Register32 // 0x0
	TIMELW   volatile.Register32 // 0x4
	TIMEHR   volatile.Register32 // 0x8
	TIMELR   volatile.Register32 // 0xC
	ALARM0   volatile.Register32 // 0x10
	ALARM1   volatile.Register32 // 0x14
	ALARM2   volatile.Register32 // 0x18
	ALARM3   volatile.Register32 // 0x1C
	ARMED    volatile.Register32 // 0x20
	TIMERAWH volatile.Register32 // 0x24
	TIMERAWL volatile.Register32 // 0x28
	DBGPAUSE volatile.Register32 // 0x2C
	PAUSE    volatile.Register32 // 0x30
	LOCKED   volatile.Register32 // 0x34
	SOURCE   volatile.Register32 // 0x38
	INTR     volatile.Register32 // 0x3C
	INTE     volatile.Register32 // 0x40
	INTF     volatile.Register32 // 0x44
	INTS     volatile.Register32 // 0x48
}

// TIMER0.TIMEHW: Write to bits 63:32 of time always write timelw before timehw
func (o *TIMER0_Type) SetTIMEHW(value uint32) {
	volatile.StoreUint32(&o.TIMEHW.Reg, value)
}
func (o *TIMER0_Type) GetTIMEHW() uint32 {
	return volatile.LoadUint32(&o.TIMEHW.Reg)
}

// TIMER0.TIMELW: Write to bits 31:0 of time writes do not get copied to time until timehw is written
func (o *TIMER0_Type) SetTIMELW(value uint32) {
	volatile.StoreUint32(&o.TIMELW.Reg, value)
}
func (o *TIMER0_Type) GetTIMELW() uint32 {
	return volatile.LoadUint32(&o.TIMELW.Reg)
}

// TIMER0.TIMEHR: Read from bits 63:32 of time always read timelr before timehr
func (o *TIMER0_Type) SetTIMEHR(value uint32) {
	volatile.StoreUint32(&o.TIMEHR.Reg, value)
}
func (o *TIMER0_Type) GetTIMEHR() uint32 {
	return volatile.LoadUint32(&o.TIMEHR.Reg)
}

// TIMER0.TIMELR: Read from bits 31:0 of time
func (o *TIMER0_Type) SetTIMELR(value uint32) {
	volatile.StoreUint32(&o.TIMELR.Reg, value)
}
func (o *TIMER0_Type) GetTIMELR() uint32 {
	return volatile.LoadUint32(&o.TIMELR.Reg)
}

// TIMER0.ALARM0: Arm alarm 0, and configure the time it will fire. Once armed, the alarm fires when TIMER_ALARM0 == TIMELR. The alarm will disarm itself once it fires, and can be disarmed early using the ARMED status register.
func (o *TIMER0_Type) SetALARM0(value uint32) {
	volatile.StoreUint32(&o.ALARM0.Reg, value)
}
func (o *TIMER0_Type) GetALARM0() uint32 {
	return volatile.LoadUint32(&o.ALARM0.Reg)
}

// TIMER0.ALARM1: Arm alarm 1, and configure the time it will fire. Once armed, the alarm fires when TIMER_ALARM1 == TIMELR. The alarm will disarm itself once it fires, and can be disarmed early using the ARMED status register.
func (o *TIMER0_Type) SetALARM1(value uint32) {
	volatile.StoreUint32(&o.ALARM1.Reg, value)
}
func (o *TIMER0_Type) GetALARM1() uint32 {
	return volatile.LoadUint32(&o.ALARM1.Reg)
}

// TIMER0.ALARM2: Arm alarm 2, and configure the time it will fire. Once armed, the alarm fires when TIMER_ALARM2 == TIMELR. The alarm will disarm itself once it fires, and can be disarmed early using the ARMED status register.
func (o *TIMER0_Type) SetALARM2(value uint32) {
	volatile.StoreUint32(&o.ALARM2.Reg, value)
}
func (o *TIMER0_Type) GetALARM2() uint32 {
	return volatile.LoadUint32(&o.ALARM2.Reg)
}

// TIMER0.ALARM3: Arm alarm 3, and configure the time it will fire. Once armed, the alarm fires when TIMER_ALARM3 == TIMELR. The alarm will disarm itself once it fires, and can be disarmed early using the ARMED status register.
func (o *TIMER0_Type) SetALARM3(value uint32) {
	volatile.StoreUint32(&o.ALARM3.Reg, value)
}
func (o *TIMER0_Type) GetALARM3() uint32 {
	return volatile.LoadUint32(&o.ALARM3.Reg)
}

// TIMER0.ARMED: Indicates the armed/disarmed status of each alarm. A write to the corresponding ALARMx register arms the alarm. Alarms automatically disarm upon firing, but writing ones here will disarm immediately without waiting to fire.
func (o *TIMER0_Type) SetARMED(value uint32) {
	volatile.StoreUint32(&o.ARMED.Reg, volatile.LoadUint32(&o.ARMED.Reg)&^(0xf)|value)
}
func (o *TIMER0_Type) GetARMED() uint32 {
	return volatile.LoadUint32(&o.ARMED.Reg) & 0xf
}

// TIMER0.TIMERAWH: Raw read from bits 63:32 of time (no side effects)
func (o *TIMER0_Type) SetTIMERAWH(value uint32) {
	volatile.StoreUint32(&o.TIMERAWH.Reg, value)
}
func (o *TIMER0_Type) GetTIMERAWH() uint32 {
	return volatile.LoadUint32(&o.TIMERAWH.Reg)
}

// TIMER0.TIMERAWL: Raw read from bits 31:0 of time (no side effects)
func (o *TIMER0_Type) SetTIMERAWL(value uint32) {
	volatile.StoreUint32(&o.TIMERAWL.Reg, value)
}
func (o *TIMER0_Type) GetTIMERAWL() uint32 {
	return volatile.LoadUint32(&o.TIMERAWL.Reg)
}

// TIMER0.DBGPAUSE: Set bits high to enable pause when the corresponding debug ports are active
func (o *TIMER0_Type) SetDBGPAUSE_DBG1(value uint32) {
	volatile.StoreUint32(&o.DBGPAUSE.Reg, volatile.LoadUint32(&o.DBGPAUSE.Reg)&^(0x4)|value<<2)
}
func (o *TIMER0_Type) GetDBGPAUSE_DBG1() uint32 {
	return (volatile.LoadUint32(&o.DBGPAUSE.Reg) & 0x4) >> 2
}
func (o *TIMER0_Type) SetDBGPAUSE_DBG0(value uint32) {
	volatile.StoreUint32(&o.DBGPAUSE.Reg, volatile.LoadUint32(&o.DBGPAUSE.Reg)&^(0x2)|value<<1)
}
func (o *TIMER0_Type) GetDBGPAUSE_DBG0() uint32 {
	return (volatile.LoadUint32(&o.DBGPAUSE.Reg) & 0x2) >> 1
}

// TIMER0.PAUSE: Set high to pause the timer
func (o *TIMER0_Type) SetPAUSE(value uint32) {
	volatile.StoreUint32(&o.PAUSE.Reg, volatile.LoadUint32(&o.PAUSE.Reg)&^(0x1)|value)
}
func (o *TIMER0_Type) GetPAUSE() uint32 {
	return volatile.LoadUint32(&o.PAUSE.Reg) & 0x1
}

// TIMER0.LOCKED: Set locked bit to disable write access to timer Once set, cannot be cleared (without a reset)
func (o *TIMER0_Type) SetLOCKED(value uint32) {
	volatile.StoreUint32(&o.LOCKED.Reg, volatile.LoadUint32(&o.LOCKED.Reg)&^(0x1)|value)
}
func (o *TIMER0_Type) GetLOCKED() uint32 {
	return volatile.LoadUint32(&o.LOCKED.Reg) & 0x1
}

// TIMER0.SOURCE: Selects the source for the timer. Defaults to the normal tick configured in the ticks block (typically configured to 1 microsecond). Writing to 1 will ignore the tick and count clk_sys cycles instead.
func (o *TIMER0_Type) SetSOURCE_CLK_SYS(value uint32) {
	volatile.StoreUint32(&o.SOURCE.Reg, volatile.LoadUint32(&o.SOURCE.Reg)&^(0x1)|value)
}
func (o *TIMER0_Type) GetSOURCE_CLK_SYS() uint32 {
	return volatile.LoadUint32(&o.SOURCE.Reg) & 0x1
}

// TIMER0.INTR: Raw Interrupts
func (o *TIMER0_Type) SetINTR_ALARM_3(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8)|value<<3)
}
func (o *TIMER0_Type) GetINTR_ALARM_3() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8) >> 3
}
func (o *TIMER0_Type) SetINTR_ALARM_2(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4)|value<<2)
}
func (o *TIMER0_Type) GetINTR_ALARM_2() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4) >> 2
}
func (o *TIMER0_Type) SetINTR_ALARM_1(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2)|value<<1)
}
func (o *TIMER0_Type) GetINTR_ALARM_1() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2) >> 1
}
func (o *TIMER0_Type) SetINTR_ALARM_0(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *TIMER0_Type) GetINTR_ALARM_0() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// TIMER0.INTE: Interrupt Enable
func (o *TIMER0_Type) SetINTE_ALARM_3(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x8)|value<<3)
}
func (o *TIMER0_Type) GetINTE_ALARM_3() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x8) >> 3
}
func (o *TIMER0_Type) SetINTE_ALARM_2(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x4)|value<<2)
}
func (o *TIMER0_Type) GetINTE_ALARM_2() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x4) >> 2
}
func (o *TIMER0_Type) SetINTE_ALARM_1(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x2)|value<<1)
}
func (o *TIMER0_Type) GetINTE_ALARM_1() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x2) >> 1
}
func (o *TIMER0_Type) SetINTE_ALARM_0(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x1)|value)
}
func (o *TIMER0_Type) GetINTE_ALARM_0() uint32 {
	return volatile.LoadUint32(&o.INTE.Reg) & 0x1
}

// TIMER0.INTF: Interrupt Force
func (o *TIMER0_Type) SetINTF_ALARM_3(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x8)|value<<3)
}
func (o *TIMER0_Type) GetINTF_ALARM_3() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x8) >> 3
}
func (o *TIMER0_Type) SetINTF_ALARM_2(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x4)|value<<2)
}
func (o *TIMER0_Type) GetINTF_ALARM_2() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x4) >> 2
}
func (o *TIMER0_Type) SetINTF_ALARM_1(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x2)|value<<1)
}
func (o *TIMER0_Type) GetINTF_ALARM_1() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x2) >> 1
}
func (o *TIMER0_Type) SetINTF_ALARM_0(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x1)|value)
}
func (o *TIMER0_Type) GetINTF_ALARM_0() uint32 {
	return volatile.LoadUint32(&o.INTF.Reg) & 0x1
}

// TIMER0.INTS: Interrupt status after masking & forcing
func (o *TIMER0_Type) SetINTS_ALARM_3(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x8)|value<<3)
}
func (o *TIMER0_Type) GetINTS_ALARM_3() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x8) >> 3
}
func (o *TIMER0_Type) SetINTS_ALARM_2(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x4)|value<<2)
}
func (o *TIMER0_Type) GetINTS_ALARM_2() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x4) >> 2
}
func (o *TIMER0_Type) SetINTS_ALARM_1(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x2)|value<<1)
}
func (o *TIMER0_Type) GetINTS_ALARM_1() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x2) >> 1
}
func (o *TIMER0_Type) SetINTS_ALARM_0(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x1)|value)
}
func (o *TIMER0_Type) GetINTS_ALARM_0() uint32 {
	return volatile.LoadUint32(&o.INTS.Reg) & 0x1
}

// Simple PWM
type PWM_Type struct {
	CH0_CSR   volatile.Register32 // 0x0
	CH0_DIV   volatile.Register32 // 0x4
	CH0_CTR   volatile.Register32 // 0x8
	CH0_CC    volatile.Register32 // 0xC
	CH0_TOP   volatile.Register32 // 0x10
	CH1_CSR   volatile.Register32 // 0x14
	CH1_DIV   volatile.Register32 // 0x18
	CH1_CTR   volatile.Register32 // 0x1C
	CH1_CC    volatile.Register32 // 0x20
	CH1_TOP   volatile.Register32 // 0x24
	CH2_CSR   volatile.Register32 // 0x28
	CH2_DIV   volatile.Register32 // 0x2C
	CH2_CTR   volatile.Register32 // 0x30
	CH2_CC    volatile.Register32 // 0x34
	CH2_TOP   volatile.Register32 // 0x38
	CH3_CSR   volatile.Register32 // 0x3C
	CH3_DIV   volatile.Register32 // 0x40
	CH3_CTR   volatile.Register32 // 0x44
	CH3_CC    volatile.Register32 // 0x48
	CH3_TOP   volatile.Register32 // 0x4C
	CH4_CSR   volatile.Register32 // 0x50
	CH4_DIV   volatile.Register32 // 0x54
	CH4_CTR   volatile.Register32 // 0x58
	CH4_CC    volatile.Register32 // 0x5C
	CH4_TOP   volatile.Register32 // 0x60
	CH5_CSR   volatile.Register32 // 0x64
	CH5_DIV   volatile.Register32 // 0x68
	CH5_CTR   volatile.Register32 // 0x6C
	CH5_CC    volatile.Register32 // 0x70
	CH5_TOP   volatile.Register32 // 0x74
	CH6_CSR   volatile.Register32 // 0x78
	CH6_DIV   volatile.Register32 // 0x7C
	CH6_CTR   volatile.Register32 // 0x80
	CH6_CC    volatile.Register32 // 0x84
	CH6_TOP   volatile.Register32 // 0x88
	CH7_CSR   volatile.Register32 // 0x8C
	CH7_DIV   volatile.Register32 // 0x90
	CH7_CTR   volatile.Register32 // 0x94
	CH7_CC    volatile.Register32 // 0x98
	CH7_TOP   volatile.Register32 // 0x9C
	CH8_CSR   volatile.Register32 // 0xA0
	CH8_DIV   volatile.Register32 // 0xA4
	CH8_CTR   volatile.Register32 // 0xA8
	CH8_CC    volatile.Register32 // 0xAC
	CH8_TOP   volatile.Register32 // 0xB0
	CH9_CSR   volatile.Register32 // 0xB4
	CH9_DIV   volatile.Register32 // 0xB8
	CH9_CTR   volatile.Register32 // 0xBC
	CH9_CC    volatile.Register32 // 0xC0
	CH9_TOP   volatile.Register32 // 0xC4
	CH10_CSR  volatile.Register32 // 0xC8
	CH10_DIV  volatile.Register32 // 0xCC
	CH10_CTR  volatile.Register32 // 0xD0
	CH10_CC   volatile.Register32 // 0xD4
	CH10_TOP  volatile.Register32 // 0xD8
	CH11_CSR  volatile.Register32 // 0xDC
	CH11_DIV  volatile.Register32 // 0xE0
	CH11_CTR  volatile.Register32 // 0xE4
	CH11_CC   volatile.Register32 // 0xE8
	CH11_TOP  volatile.Register32 // 0xEC
	EN        volatile.Register32 // 0xF0
	INTR      volatile.Register32 // 0xF4
	IRQ0_INTE volatile.Register32 // 0xF8
	IRQ0_INTF volatile.Register32 // 0xFC
	IRQ0_INTS volatile.Register32 // 0x100
	IRQ1_INTE volatile.Register32 // 0x104
	IRQ1_INTF volatile.Register32 // 0x108
	IRQ1_INTS volatile.Register32 // 0x10C
}

// PWM.CH0_CSR: Control and status register
func (o *PWM_Type) SetCH0_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH0_CSR.Reg, volatile.LoadUint32(&o.CH0_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH0_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH0_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH0_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH0_CSR.Reg, volatile.LoadUint32(&o.CH0_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH0_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH0_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH0_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH0_CSR.Reg, volatile.LoadUint32(&o.CH0_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH0_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH0_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH0_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH0_CSR.Reg, volatile.LoadUint32(&o.CH0_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH0_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH0_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH0_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH0_CSR.Reg, volatile.LoadUint32(&o.CH0_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH0_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH0_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH0_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH0_CSR.Reg, volatile.LoadUint32(&o.CH0_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH0_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH0_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH0_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH0_CSR.Reg, volatile.LoadUint32(&o.CH0_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH0_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH0_CSR.Reg) & 0x1
}

// PWM.CH0_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH0_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH0_DIV.Reg, volatile.LoadUint32(&o.CH0_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH0_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH0_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH0_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH0_DIV.Reg, volatile.LoadUint32(&o.CH0_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH0_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH0_DIV.Reg) & 0xf
}

// PWM.CH0_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH0_CTR(value uint32) {
	volatile.StoreUint32(&o.CH0_CTR.Reg, volatile.LoadUint32(&o.CH0_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH0_CTR() uint32 {
	return volatile.LoadUint32(&o.CH0_CTR.Reg) & 0xffff
}

// PWM.CH0_CC: Counter compare values
func (o *PWM_Type) SetCH0_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH0_CC.Reg, volatile.LoadUint32(&o.CH0_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH0_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH0_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH0_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH0_CC.Reg, volatile.LoadUint32(&o.CH0_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH0_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH0_CC.Reg) & 0xffff
}

// PWM.CH0_TOP: Counter wrap value
func (o *PWM_Type) SetCH0_TOP(value uint32) {
	volatile.StoreUint32(&o.CH0_TOP.Reg, volatile.LoadUint32(&o.CH0_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH0_TOP() uint32 {
	return volatile.LoadUint32(&o.CH0_TOP.Reg) & 0xffff
}

// PWM.CH1_CSR: Control and status register
func (o *PWM_Type) SetCH1_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH1_CSR.Reg, volatile.LoadUint32(&o.CH1_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH1_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH1_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH1_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH1_CSR.Reg, volatile.LoadUint32(&o.CH1_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH1_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH1_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH1_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH1_CSR.Reg, volatile.LoadUint32(&o.CH1_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH1_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH1_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH1_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH1_CSR.Reg, volatile.LoadUint32(&o.CH1_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH1_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH1_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH1_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH1_CSR.Reg, volatile.LoadUint32(&o.CH1_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH1_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH1_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH1_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH1_CSR.Reg, volatile.LoadUint32(&o.CH1_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH1_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH1_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH1_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH1_CSR.Reg, volatile.LoadUint32(&o.CH1_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH1_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH1_CSR.Reg) & 0x1
}

// PWM.CH1_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH1_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH1_DIV.Reg, volatile.LoadUint32(&o.CH1_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH1_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH1_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH1_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH1_DIV.Reg, volatile.LoadUint32(&o.CH1_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH1_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH1_DIV.Reg) & 0xf
}

// PWM.CH1_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH1_CTR(value uint32) {
	volatile.StoreUint32(&o.CH1_CTR.Reg, volatile.LoadUint32(&o.CH1_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH1_CTR() uint32 {
	return volatile.LoadUint32(&o.CH1_CTR.Reg) & 0xffff
}

// PWM.CH1_CC: Counter compare values
func (o *PWM_Type) SetCH1_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH1_CC.Reg, volatile.LoadUint32(&o.CH1_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH1_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH1_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH1_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH1_CC.Reg, volatile.LoadUint32(&o.CH1_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH1_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH1_CC.Reg) & 0xffff
}

// PWM.CH1_TOP: Counter wrap value
func (o *PWM_Type) SetCH1_TOP(value uint32) {
	volatile.StoreUint32(&o.CH1_TOP.Reg, volatile.LoadUint32(&o.CH1_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH1_TOP() uint32 {
	return volatile.LoadUint32(&o.CH1_TOP.Reg) & 0xffff
}

// PWM.CH2_CSR: Control and status register
func (o *PWM_Type) SetCH2_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH2_CSR.Reg, volatile.LoadUint32(&o.CH2_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH2_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH2_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH2_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH2_CSR.Reg, volatile.LoadUint32(&o.CH2_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH2_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH2_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH2_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH2_CSR.Reg, volatile.LoadUint32(&o.CH2_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH2_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH2_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH2_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH2_CSR.Reg, volatile.LoadUint32(&o.CH2_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH2_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH2_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH2_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH2_CSR.Reg, volatile.LoadUint32(&o.CH2_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH2_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH2_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH2_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH2_CSR.Reg, volatile.LoadUint32(&o.CH2_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH2_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH2_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH2_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH2_CSR.Reg, volatile.LoadUint32(&o.CH2_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH2_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH2_CSR.Reg) & 0x1
}

// PWM.CH2_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH2_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH2_DIV.Reg, volatile.LoadUint32(&o.CH2_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH2_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH2_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH2_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH2_DIV.Reg, volatile.LoadUint32(&o.CH2_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH2_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH2_DIV.Reg) & 0xf
}

// PWM.CH2_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH2_CTR(value uint32) {
	volatile.StoreUint32(&o.CH2_CTR.Reg, volatile.LoadUint32(&o.CH2_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH2_CTR() uint32 {
	return volatile.LoadUint32(&o.CH2_CTR.Reg) & 0xffff
}

// PWM.CH2_CC: Counter compare values
func (o *PWM_Type) SetCH2_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH2_CC.Reg, volatile.LoadUint32(&o.CH2_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH2_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH2_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH2_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH2_CC.Reg, volatile.LoadUint32(&o.CH2_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH2_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH2_CC.Reg) & 0xffff
}

// PWM.CH2_TOP: Counter wrap value
func (o *PWM_Type) SetCH2_TOP(value uint32) {
	volatile.StoreUint32(&o.CH2_TOP.Reg, volatile.LoadUint32(&o.CH2_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH2_TOP() uint32 {
	return volatile.LoadUint32(&o.CH2_TOP.Reg) & 0xffff
}

// PWM.CH3_CSR: Control and status register
func (o *PWM_Type) SetCH3_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH3_CSR.Reg, volatile.LoadUint32(&o.CH3_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH3_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH3_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH3_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH3_CSR.Reg, volatile.LoadUint32(&o.CH3_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH3_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH3_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH3_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH3_CSR.Reg, volatile.LoadUint32(&o.CH3_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH3_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH3_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH3_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH3_CSR.Reg, volatile.LoadUint32(&o.CH3_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH3_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH3_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH3_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH3_CSR.Reg, volatile.LoadUint32(&o.CH3_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH3_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH3_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH3_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH3_CSR.Reg, volatile.LoadUint32(&o.CH3_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH3_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH3_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH3_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH3_CSR.Reg, volatile.LoadUint32(&o.CH3_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH3_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH3_CSR.Reg) & 0x1
}

// PWM.CH3_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH3_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH3_DIV.Reg, volatile.LoadUint32(&o.CH3_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH3_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH3_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH3_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH3_DIV.Reg, volatile.LoadUint32(&o.CH3_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH3_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH3_DIV.Reg) & 0xf
}

// PWM.CH3_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH3_CTR(value uint32) {
	volatile.StoreUint32(&o.CH3_CTR.Reg, volatile.LoadUint32(&o.CH3_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH3_CTR() uint32 {
	return volatile.LoadUint32(&o.CH3_CTR.Reg) & 0xffff
}

// PWM.CH3_CC: Counter compare values
func (o *PWM_Type) SetCH3_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH3_CC.Reg, volatile.LoadUint32(&o.CH3_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH3_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH3_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH3_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH3_CC.Reg, volatile.LoadUint32(&o.CH3_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH3_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH3_CC.Reg) & 0xffff
}

// PWM.CH3_TOP: Counter wrap value
func (o *PWM_Type) SetCH3_TOP(value uint32) {
	volatile.StoreUint32(&o.CH3_TOP.Reg, volatile.LoadUint32(&o.CH3_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH3_TOP() uint32 {
	return volatile.LoadUint32(&o.CH3_TOP.Reg) & 0xffff
}

// PWM.CH4_CSR: Control and status register
func (o *PWM_Type) SetCH4_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH4_CSR.Reg, volatile.LoadUint32(&o.CH4_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH4_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH4_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH4_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH4_CSR.Reg, volatile.LoadUint32(&o.CH4_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH4_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH4_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH4_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH4_CSR.Reg, volatile.LoadUint32(&o.CH4_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH4_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH4_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH4_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH4_CSR.Reg, volatile.LoadUint32(&o.CH4_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH4_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH4_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH4_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH4_CSR.Reg, volatile.LoadUint32(&o.CH4_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH4_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH4_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH4_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH4_CSR.Reg, volatile.LoadUint32(&o.CH4_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH4_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH4_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH4_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH4_CSR.Reg, volatile.LoadUint32(&o.CH4_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH4_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH4_CSR.Reg) & 0x1
}

// PWM.CH4_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH4_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH4_DIV.Reg, volatile.LoadUint32(&o.CH4_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH4_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH4_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH4_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH4_DIV.Reg, volatile.LoadUint32(&o.CH4_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH4_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH4_DIV.Reg) & 0xf
}

// PWM.CH4_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH4_CTR(value uint32) {
	volatile.StoreUint32(&o.CH4_CTR.Reg, volatile.LoadUint32(&o.CH4_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH4_CTR() uint32 {
	return volatile.LoadUint32(&o.CH4_CTR.Reg) & 0xffff
}

// PWM.CH4_CC: Counter compare values
func (o *PWM_Type) SetCH4_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH4_CC.Reg, volatile.LoadUint32(&o.CH4_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH4_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH4_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH4_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH4_CC.Reg, volatile.LoadUint32(&o.CH4_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH4_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH4_CC.Reg) & 0xffff
}

// PWM.CH4_TOP: Counter wrap value
func (o *PWM_Type) SetCH4_TOP(value uint32) {
	volatile.StoreUint32(&o.CH4_TOP.Reg, volatile.LoadUint32(&o.CH4_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH4_TOP() uint32 {
	return volatile.LoadUint32(&o.CH4_TOP.Reg) & 0xffff
}

// PWM.CH5_CSR: Control and status register
func (o *PWM_Type) SetCH5_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH5_CSR.Reg, volatile.LoadUint32(&o.CH5_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH5_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH5_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH5_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH5_CSR.Reg, volatile.LoadUint32(&o.CH5_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH5_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH5_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH5_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH5_CSR.Reg, volatile.LoadUint32(&o.CH5_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH5_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH5_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH5_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH5_CSR.Reg, volatile.LoadUint32(&o.CH5_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH5_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH5_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH5_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH5_CSR.Reg, volatile.LoadUint32(&o.CH5_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH5_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH5_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH5_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH5_CSR.Reg, volatile.LoadUint32(&o.CH5_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH5_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH5_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH5_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH5_CSR.Reg, volatile.LoadUint32(&o.CH5_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH5_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH5_CSR.Reg) & 0x1
}

// PWM.CH5_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH5_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH5_DIV.Reg, volatile.LoadUint32(&o.CH5_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH5_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH5_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH5_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH5_DIV.Reg, volatile.LoadUint32(&o.CH5_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH5_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH5_DIV.Reg) & 0xf
}

// PWM.CH5_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH5_CTR(value uint32) {
	volatile.StoreUint32(&o.CH5_CTR.Reg, volatile.LoadUint32(&o.CH5_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH5_CTR() uint32 {
	return volatile.LoadUint32(&o.CH5_CTR.Reg) & 0xffff
}

// PWM.CH5_CC: Counter compare values
func (o *PWM_Type) SetCH5_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH5_CC.Reg, volatile.LoadUint32(&o.CH5_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH5_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH5_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH5_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH5_CC.Reg, volatile.LoadUint32(&o.CH5_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH5_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH5_CC.Reg) & 0xffff
}

// PWM.CH5_TOP: Counter wrap value
func (o *PWM_Type) SetCH5_TOP(value uint32) {
	volatile.StoreUint32(&o.CH5_TOP.Reg, volatile.LoadUint32(&o.CH5_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH5_TOP() uint32 {
	return volatile.LoadUint32(&o.CH5_TOP.Reg) & 0xffff
}

// PWM.CH6_CSR: Control and status register
func (o *PWM_Type) SetCH6_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH6_CSR.Reg, volatile.LoadUint32(&o.CH6_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH6_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH6_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH6_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH6_CSR.Reg, volatile.LoadUint32(&o.CH6_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH6_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH6_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH6_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH6_CSR.Reg, volatile.LoadUint32(&o.CH6_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH6_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH6_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH6_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH6_CSR.Reg, volatile.LoadUint32(&o.CH6_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH6_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH6_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH6_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH6_CSR.Reg, volatile.LoadUint32(&o.CH6_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH6_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH6_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH6_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH6_CSR.Reg, volatile.LoadUint32(&o.CH6_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH6_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH6_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH6_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH6_CSR.Reg, volatile.LoadUint32(&o.CH6_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH6_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH6_CSR.Reg) & 0x1
}

// PWM.CH6_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH6_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH6_DIV.Reg, volatile.LoadUint32(&o.CH6_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH6_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH6_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH6_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH6_DIV.Reg, volatile.LoadUint32(&o.CH6_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH6_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH6_DIV.Reg) & 0xf
}

// PWM.CH6_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH6_CTR(value uint32) {
	volatile.StoreUint32(&o.CH6_CTR.Reg, volatile.LoadUint32(&o.CH6_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH6_CTR() uint32 {
	return volatile.LoadUint32(&o.CH6_CTR.Reg) & 0xffff
}

// PWM.CH6_CC: Counter compare values
func (o *PWM_Type) SetCH6_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH6_CC.Reg, volatile.LoadUint32(&o.CH6_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH6_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH6_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH6_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH6_CC.Reg, volatile.LoadUint32(&o.CH6_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH6_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH6_CC.Reg) & 0xffff
}

// PWM.CH6_TOP: Counter wrap value
func (o *PWM_Type) SetCH6_TOP(value uint32) {
	volatile.StoreUint32(&o.CH6_TOP.Reg, volatile.LoadUint32(&o.CH6_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH6_TOP() uint32 {
	return volatile.LoadUint32(&o.CH6_TOP.Reg) & 0xffff
}

// PWM.CH7_CSR: Control and status register
func (o *PWM_Type) SetCH7_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH7_CSR.Reg, volatile.LoadUint32(&o.CH7_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH7_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH7_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH7_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH7_CSR.Reg, volatile.LoadUint32(&o.CH7_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH7_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH7_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH7_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH7_CSR.Reg, volatile.LoadUint32(&o.CH7_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH7_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH7_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH7_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH7_CSR.Reg, volatile.LoadUint32(&o.CH7_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH7_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH7_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH7_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH7_CSR.Reg, volatile.LoadUint32(&o.CH7_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH7_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH7_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH7_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH7_CSR.Reg, volatile.LoadUint32(&o.CH7_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH7_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH7_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH7_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH7_CSR.Reg, volatile.LoadUint32(&o.CH7_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH7_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH7_CSR.Reg) & 0x1
}

// PWM.CH7_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH7_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH7_DIV.Reg, volatile.LoadUint32(&o.CH7_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH7_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH7_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH7_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH7_DIV.Reg, volatile.LoadUint32(&o.CH7_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH7_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH7_DIV.Reg) & 0xf
}

// PWM.CH7_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH7_CTR(value uint32) {
	volatile.StoreUint32(&o.CH7_CTR.Reg, volatile.LoadUint32(&o.CH7_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH7_CTR() uint32 {
	return volatile.LoadUint32(&o.CH7_CTR.Reg) & 0xffff
}

// PWM.CH7_CC: Counter compare values
func (o *PWM_Type) SetCH7_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH7_CC.Reg, volatile.LoadUint32(&o.CH7_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH7_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH7_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH7_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH7_CC.Reg, volatile.LoadUint32(&o.CH7_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH7_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH7_CC.Reg) & 0xffff
}

// PWM.CH7_TOP: Counter wrap value
func (o *PWM_Type) SetCH7_TOP(value uint32) {
	volatile.StoreUint32(&o.CH7_TOP.Reg, volatile.LoadUint32(&o.CH7_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH7_TOP() uint32 {
	return volatile.LoadUint32(&o.CH7_TOP.Reg) & 0xffff
}

// PWM.CH8_CSR: Control and status register
func (o *PWM_Type) SetCH8_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH8_CSR.Reg, volatile.LoadUint32(&o.CH8_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH8_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH8_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH8_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH8_CSR.Reg, volatile.LoadUint32(&o.CH8_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH8_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH8_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH8_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH8_CSR.Reg, volatile.LoadUint32(&o.CH8_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH8_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH8_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH8_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH8_CSR.Reg, volatile.LoadUint32(&o.CH8_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH8_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH8_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH8_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH8_CSR.Reg, volatile.LoadUint32(&o.CH8_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH8_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH8_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH8_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH8_CSR.Reg, volatile.LoadUint32(&o.CH8_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH8_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH8_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH8_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH8_CSR.Reg, volatile.LoadUint32(&o.CH8_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH8_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH8_CSR.Reg) & 0x1
}

// PWM.CH8_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH8_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH8_DIV.Reg, volatile.LoadUint32(&o.CH8_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH8_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH8_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH8_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH8_DIV.Reg, volatile.LoadUint32(&o.CH8_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH8_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH8_DIV.Reg) & 0xf
}

// PWM.CH8_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH8_CTR(value uint32) {
	volatile.StoreUint32(&o.CH8_CTR.Reg, volatile.LoadUint32(&o.CH8_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH8_CTR() uint32 {
	return volatile.LoadUint32(&o.CH8_CTR.Reg) & 0xffff
}

// PWM.CH8_CC: Counter compare values
func (o *PWM_Type) SetCH8_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH8_CC.Reg, volatile.LoadUint32(&o.CH8_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH8_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH8_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH8_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH8_CC.Reg, volatile.LoadUint32(&o.CH8_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH8_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH8_CC.Reg) & 0xffff
}

// PWM.CH8_TOP: Counter wrap value
func (o *PWM_Type) SetCH8_TOP(value uint32) {
	volatile.StoreUint32(&o.CH8_TOP.Reg, volatile.LoadUint32(&o.CH8_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH8_TOP() uint32 {
	return volatile.LoadUint32(&o.CH8_TOP.Reg) & 0xffff
}

// PWM.CH9_CSR: Control and status register
func (o *PWM_Type) SetCH9_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH9_CSR.Reg, volatile.LoadUint32(&o.CH9_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH9_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH9_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH9_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH9_CSR.Reg, volatile.LoadUint32(&o.CH9_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH9_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH9_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH9_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH9_CSR.Reg, volatile.LoadUint32(&o.CH9_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH9_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH9_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH9_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH9_CSR.Reg, volatile.LoadUint32(&o.CH9_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH9_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH9_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH9_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH9_CSR.Reg, volatile.LoadUint32(&o.CH9_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH9_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH9_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH9_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH9_CSR.Reg, volatile.LoadUint32(&o.CH9_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH9_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH9_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH9_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH9_CSR.Reg, volatile.LoadUint32(&o.CH9_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH9_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH9_CSR.Reg) & 0x1
}

// PWM.CH9_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH9_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH9_DIV.Reg, volatile.LoadUint32(&o.CH9_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH9_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH9_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH9_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH9_DIV.Reg, volatile.LoadUint32(&o.CH9_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH9_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH9_DIV.Reg) & 0xf
}

// PWM.CH9_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH9_CTR(value uint32) {
	volatile.StoreUint32(&o.CH9_CTR.Reg, volatile.LoadUint32(&o.CH9_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH9_CTR() uint32 {
	return volatile.LoadUint32(&o.CH9_CTR.Reg) & 0xffff
}

// PWM.CH9_CC: Counter compare values
func (o *PWM_Type) SetCH9_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH9_CC.Reg, volatile.LoadUint32(&o.CH9_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH9_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH9_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH9_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH9_CC.Reg, volatile.LoadUint32(&o.CH9_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH9_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH9_CC.Reg) & 0xffff
}

// PWM.CH9_TOP: Counter wrap value
func (o *PWM_Type) SetCH9_TOP(value uint32) {
	volatile.StoreUint32(&o.CH9_TOP.Reg, volatile.LoadUint32(&o.CH9_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH9_TOP() uint32 {
	return volatile.LoadUint32(&o.CH9_TOP.Reg) & 0xffff
}

// PWM.CH10_CSR: Control and status register
func (o *PWM_Type) SetCH10_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH10_CSR.Reg, volatile.LoadUint32(&o.CH10_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH10_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH10_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH10_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH10_CSR.Reg, volatile.LoadUint32(&o.CH10_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH10_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH10_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH10_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH10_CSR.Reg, volatile.LoadUint32(&o.CH10_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH10_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH10_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH10_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH10_CSR.Reg, volatile.LoadUint32(&o.CH10_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH10_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH10_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH10_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH10_CSR.Reg, volatile.LoadUint32(&o.CH10_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH10_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH10_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH10_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH10_CSR.Reg, volatile.LoadUint32(&o.CH10_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH10_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH10_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH10_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH10_CSR.Reg, volatile.LoadUint32(&o.CH10_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH10_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH10_CSR.Reg) & 0x1
}

// PWM.CH10_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH10_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH10_DIV.Reg, volatile.LoadUint32(&o.CH10_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH10_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH10_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH10_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH10_DIV.Reg, volatile.LoadUint32(&o.CH10_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH10_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH10_DIV.Reg) & 0xf
}

// PWM.CH10_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH10_CTR(value uint32) {
	volatile.StoreUint32(&o.CH10_CTR.Reg, volatile.LoadUint32(&o.CH10_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH10_CTR() uint32 {
	return volatile.LoadUint32(&o.CH10_CTR.Reg) & 0xffff
}

// PWM.CH10_CC: Counter compare values
func (o *PWM_Type) SetCH10_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH10_CC.Reg, volatile.LoadUint32(&o.CH10_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH10_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH10_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH10_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH10_CC.Reg, volatile.LoadUint32(&o.CH10_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH10_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH10_CC.Reg) & 0xffff
}

// PWM.CH10_TOP: Counter wrap value
func (o *PWM_Type) SetCH10_TOP(value uint32) {
	volatile.StoreUint32(&o.CH10_TOP.Reg, volatile.LoadUint32(&o.CH10_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH10_TOP() uint32 {
	return volatile.LoadUint32(&o.CH10_TOP.Reg) & 0xffff
}

// PWM.CH11_CSR: Control and status register
func (o *PWM_Type) SetCH11_CSR_PH_ADV(value uint32) {
	volatile.StoreUint32(&o.CH11_CSR.Reg, volatile.LoadUint32(&o.CH11_CSR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetCH11_CSR_PH_ADV() uint32 {
	return (volatile.LoadUint32(&o.CH11_CSR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetCH11_CSR_PH_RET(value uint32) {
	volatile.StoreUint32(&o.CH11_CSR.Reg, volatile.LoadUint32(&o.CH11_CSR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetCH11_CSR_PH_RET() uint32 {
	return (volatile.LoadUint32(&o.CH11_CSR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetCH11_CSR_DIVMODE(value uint32) {
	volatile.StoreUint32(&o.CH11_CSR.Reg, volatile.LoadUint32(&o.CH11_CSR.Reg)&^(0x30)|value<<4)
}
func (o *PWM_Type) GetCH11_CSR_DIVMODE() uint32 {
	return (volatile.LoadUint32(&o.CH11_CSR.Reg) & 0x30) >> 4
}
func (o *PWM_Type) SetCH11_CSR_B_INV(value uint32) {
	volatile.StoreUint32(&o.CH11_CSR.Reg, volatile.LoadUint32(&o.CH11_CSR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetCH11_CSR_B_INV() uint32 {
	return (volatile.LoadUint32(&o.CH11_CSR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetCH11_CSR_A_INV(value uint32) {
	volatile.StoreUint32(&o.CH11_CSR.Reg, volatile.LoadUint32(&o.CH11_CSR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetCH11_CSR_A_INV() uint32 {
	return (volatile.LoadUint32(&o.CH11_CSR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetCH11_CSR_PH_CORRECT(value uint32) {
	volatile.StoreUint32(&o.CH11_CSR.Reg, volatile.LoadUint32(&o.CH11_CSR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetCH11_CSR_PH_CORRECT() uint32 {
	return (volatile.LoadUint32(&o.CH11_CSR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetCH11_CSR_EN(value uint32) {
	volatile.StoreUint32(&o.CH11_CSR.Reg, volatile.LoadUint32(&o.CH11_CSR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetCH11_CSR_EN() uint32 {
	return volatile.LoadUint32(&o.CH11_CSR.Reg) & 0x1
}

// PWM.CH11_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
func (o *PWM_Type) SetCH11_DIV_INT(value uint32) {
	volatile.StoreUint32(&o.CH11_DIV.Reg, volatile.LoadUint32(&o.CH11_DIV.Reg)&^(0xff0)|value<<4)
}
func (o *PWM_Type) GetCH11_DIV_INT() uint32 {
	return (volatile.LoadUint32(&o.CH11_DIV.Reg) & 0xff0) >> 4
}
func (o *PWM_Type) SetCH11_DIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.CH11_DIV.Reg, volatile.LoadUint32(&o.CH11_DIV.Reg)&^(0xf)|value)
}
func (o *PWM_Type) GetCH11_DIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.CH11_DIV.Reg) & 0xf
}

// PWM.CH11_CTR: Direct access to the PWM counter
func (o *PWM_Type) SetCH11_CTR(value uint32) {
	volatile.StoreUint32(&o.CH11_CTR.Reg, volatile.LoadUint32(&o.CH11_CTR.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH11_CTR() uint32 {
	return volatile.LoadUint32(&o.CH11_CTR.Reg) & 0xffff
}

// PWM.CH11_CC: Counter compare values
func (o *PWM_Type) SetCH11_CC_B(value uint32) {
	volatile.StoreUint32(&o.CH11_CC.Reg, volatile.LoadUint32(&o.CH11_CC.Reg)&^(0xffff0000)|value<<16)
}
func (o *PWM_Type) GetCH11_CC_B() uint32 {
	return (volatile.LoadUint32(&o.CH11_CC.Reg) & 0xffff0000) >> 16
}
func (o *PWM_Type) SetCH11_CC_A(value uint32) {
	volatile.StoreUint32(&o.CH11_CC.Reg, volatile.LoadUint32(&o.CH11_CC.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH11_CC_A() uint32 {
	return volatile.LoadUint32(&o.CH11_CC.Reg) & 0xffff
}

// PWM.CH11_TOP: Counter wrap value
func (o *PWM_Type) SetCH11_TOP(value uint32) {
	volatile.StoreUint32(&o.CH11_TOP.Reg, volatile.LoadUint32(&o.CH11_TOP.Reg)&^(0xffff)|value)
}
func (o *PWM_Type) GetCH11_TOP() uint32 {
	return volatile.LoadUint32(&o.CH11_TOP.Reg) & 0xffff
}

// PWM.EN: This register aliases the CSR_EN bits for all channels. Writing to this register allows multiple channels to be enabled or disabled simultaneously, so they can run in perfect sync. For each channel, there is only one physical EN register bit, which can be accessed through here or CHx_CSR.
func (o *PWM_Type) SetEN_CH11(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x800)|value<<11)
}
func (o *PWM_Type) GetEN_CH11() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x800) >> 11
}
func (o *PWM_Type) SetEN_CH10(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x400)|value<<10)
}
func (o *PWM_Type) GetEN_CH10() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x400) >> 10
}
func (o *PWM_Type) SetEN_CH9(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x200)|value<<9)
}
func (o *PWM_Type) GetEN_CH9() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x200) >> 9
}
func (o *PWM_Type) SetEN_CH8(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x100)|value<<8)
}
func (o *PWM_Type) GetEN_CH8() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x100) >> 8
}
func (o *PWM_Type) SetEN_CH7(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetEN_CH7() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetEN_CH6(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetEN_CH6() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetEN_CH5(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x20)|value<<5)
}
func (o *PWM_Type) GetEN_CH5() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x20) >> 5
}
func (o *PWM_Type) SetEN_CH4(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x10)|value<<4)
}
func (o *PWM_Type) GetEN_CH4() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x10) >> 4
}
func (o *PWM_Type) SetEN_CH3(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetEN_CH3() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetEN_CH2(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetEN_CH2() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetEN_CH1(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetEN_CH1() uint32 {
	return (volatile.LoadUint32(&o.EN.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetEN_CH0(value uint32) {
	volatile.StoreUint32(&o.EN.Reg, volatile.LoadUint32(&o.EN.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetEN_CH0() uint32 {
	return volatile.LoadUint32(&o.EN.Reg) & 0x1
}

// PWM.INTR: Raw Interrupts
func (o *PWM_Type) SetINTR_CH11(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x800)|value<<11)
}
func (o *PWM_Type) GetINTR_CH11() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x800) >> 11
}
func (o *PWM_Type) SetINTR_CH10(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x400)|value<<10)
}
func (o *PWM_Type) GetINTR_CH10() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x400) >> 10
}
func (o *PWM_Type) SetINTR_CH9(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x200)|value<<9)
}
func (o *PWM_Type) GetINTR_CH9() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x200) >> 9
}
func (o *PWM_Type) SetINTR_CH8(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x100)|value<<8)
}
func (o *PWM_Type) GetINTR_CH8() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x100) >> 8
}
func (o *PWM_Type) SetINTR_CH7(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetINTR_CH7() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetINTR_CH6(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetINTR_CH6() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetINTR_CH5(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x20)|value<<5)
}
func (o *PWM_Type) GetINTR_CH5() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x20) >> 5
}
func (o *PWM_Type) SetINTR_CH4(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x10)|value<<4)
}
func (o *PWM_Type) GetINTR_CH4() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x10) >> 4
}
func (o *PWM_Type) SetINTR_CH3(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetINTR_CH3() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetINTR_CH2(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetINTR_CH2() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetINTR_CH1(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetINTR_CH1() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetINTR_CH0(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetINTR_CH0() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// PWM.IRQ0_INTE: Interrupt Enable for irq0
func (o *PWM_Type) SetIRQ0_INTE_CH11(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x800)|value<<11)
}
func (o *PWM_Type) GetIRQ0_INTE_CH11() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x800) >> 11
}
func (o *PWM_Type) SetIRQ0_INTE_CH10(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x400)|value<<10)
}
func (o *PWM_Type) GetIRQ0_INTE_CH10() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x400) >> 10
}
func (o *PWM_Type) SetIRQ0_INTE_CH9(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x200)|value<<9)
}
func (o *PWM_Type) GetIRQ0_INTE_CH9() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x200) >> 9
}
func (o *PWM_Type) SetIRQ0_INTE_CH8(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x100)|value<<8)
}
func (o *PWM_Type) GetIRQ0_INTE_CH8() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x100) >> 8
}
func (o *PWM_Type) SetIRQ0_INTE_CH7(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetIRQ0_INTE_CH7() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetIRQ0_INTE_CH6(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetIRQ0_INTE_CH6() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetIRQ0_INTE_CH5(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x20)|value<<5)
}
func (o *PWM_Type) GetIRQ0_INTE_CH5() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x20) >> 5
}
func (o *PWM_Type) SetIRQ0_INTE_CH4(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x10)|value<<4)
}
func (o *PWM_Type) GetIRQ0_INTE_CH4() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x10) >> 4
}
func (o *PWM_Type) SetIRQ0_INTE_CH3(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetIRQ0_INTE_CH3() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetIRQ0_INTE_CH2(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetIRQ0_INTE_CH2() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetIRQ0_INTE_CH1(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetIRQ0_INTE_CH1() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetIRQ0_INTE_CH0(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetIRQ0_INTE_CH0() uint32 {
	return volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x1
}

// PWM.IRQ0_INTF: Interrupt Force for irq0
func (o *PWM_Type) SetIRQ0_INTF_CH11(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x800)|value<<11)
}
func (o *PWM_Type) GetIRQ0_INTF_CH11() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x800) >> 11
}
func (o *PWM_Type) SetIRQ0_INTF_CH10(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x400)|value<<10)
}
func (o *PWM_Type) GetIRQ0_INTF_CH10() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x400) >> 10
}
func (o *PWM_Type) SetIRQ0_INTF_CH9(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x200)|value<<9)
}
func (o *PWM_Type) GetIRQ0_INTF_CH9() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x200) >> 9
}
func (o *PWM_Type) SetIRQ0_INTF_CH8(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x100)|value<<8)
}
func (o *PWM_Type) GetIRQ0_INTF_CH8() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x100) >> 8
}
func (o *PWM_Type) SetIRQ0_INTF_CH7(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetIRQ0_INTF_CH7() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetIRQ0_INTF_CH6(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetIRQ0_INTF_CH6() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetIRQ0_INTF_CH5(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x20)|value<<5)
}
func (o *PWM_Type) GetIRQ0_INTF_CH5() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x20) >> 5
}
func (o *PWM_Type) SetIRQ0_INTF_CH4(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x10)|value<<4)
}
func (o *PWM_Type) GetIRQ0_INTF_CH4() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x10) >> 4
}
func (o *PWM_Type) SetIRQ0_INTF_CH3(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetIRQ0_INTF_CH3() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetIRQ0_INTF_CH2(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetIRQ0_INTF_CH2() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetIRQ0_INTF_CH1(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetIRQ0_INTF_CH1() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetIRQ0_INTF_CH0(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetIRQ0_INTF_CH0() uint32 {
	return volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x1
}

// PWM.IRQ0_INTS: Interrupt status after masking & forcing for irq0
func (o *PWM_Type) SetIRQ0_INTS_CH11(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x800)|value<<11)
}
func (o *PWM_Type) GetIRQ0_INTS_CH11() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x800) >> 11
}
func (o *PWM_Type) SetIRQ0_INTS_CH10(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x400)|value<<10)
}
func (o *PWM_Type) GetIRQ0_INTS_CH10() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x400) >> 10
}
func (o *PWM_Type) SetIRQ0_INTS_CH9(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x200)|value<<9)
}
func (o *PWM_Type) GetIRQ0_INTS_CH9() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x200) >> 9
}
func (o *PWM_Type) SetIRQ0_INTS_CH8(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x100)|value<<8)
}
func (o *PWM_Type) GetIRQ0_INTS_CH8() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x100) >> 8
}
func (o *PWM_Type) SetIRQ0_INTS_CH7(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetIRQ0_INTS_CH7() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetIRQ0_INTS_CH6(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetIRQ0_INTS_CH6() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetIRQ0_INTS_CH5(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x20)|value<<5)
}
func (o *PWM_Type) GetIRQ0_INTS_CH5() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x20) >> 5
}
func (o *PWM_Type) SetIRQ0_INTS_CH4(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x10)|value<<4)
}
func (o *PWM_Type) GetIRQ0_INTS_CH4() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x10) >> 4
}
func (o *PWM_Type) SetIRQ0_INTS_CH3(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetIRQ0_INTS_CH3() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetIRQ0_INTS_CH2(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetIRQ0_INTS_CH2() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetIRQ0_INTS_CH1(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetIRQ0_INTS_CH1() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetIRQ0_INTS_CH0(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetIRQ0_INTS_CH0() uint32 {
	return volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x1
}

// PWM.IRQ1_INTE: Interrupt Enable for irq1
func (o *PWM_Type) SetIRQ1_INTE_CH11(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x800)|value<<11)
}
func (o *PWM_Type) GetIRQ1_INTE_CH11() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x800) >> 11
}
func (o *PWM_Type) SetIRQ1_INTE_CH10(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x400)|value<<10)
}
func (o *PWM_Type) GetIRQ1_INTE_CH10() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x400) >> 10
}
func (o *PWM_Type) SetIRQ1_INTE_CH9(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x200)|value<<9)
}
func (o *PWM_Type) GetIRQ1_INTE_CH9() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x200) >> 9
}
func (o *PWM_Type) SetIRQ1_INTE_CH8(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x100)|value<<8)
}
func (o *PWM_Type) GetIRQ1_INTE_CH8() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x100) >> 8
}
func (o *PWM_Type) SetIRQ1_INTE_CH7(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetIRQ1_INTE_CH7() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetIRQ1_INTE_CH6(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetIRQ1_INTE_CH6() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetIRQ1_INTE_CH5(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x20)|value<<5)
}
func (o *PWM_Type) GetIRQ1_INTE_CH5() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x20) >> 5
}
func (o *PWM_Type) SetIRQ1_INTE_CH4(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x10)|value<<4)
}
func (o *PWM_Type) GetIRQ1_INTE_CH4() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x10) >> 4
}
func (o *PWM_Type) SetIRQ1_INTE_CH3(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetIRQ1_INTE_CH3() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetIRQ1_INTE_CH2(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetIRQ1_INTE_CH2() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetIRQ1_INTE_CH1(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetIRQ1_INTE_CH1() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetIRQ1_INTE_CH0(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetIRQ1_INTE_CH0() uint32 {
	return volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x1
}

// PWM.IRQ1_INTF: Interrupt Force for irq1
func (o *PWM_Type) SetIRQ1_INTF_CH11(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x800)|value<<11)
}
func (o *PWM_Type) GetIRQ1_INTF_CH11() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x800) >> 11
}
func (o *PWM_Type) SetIRQ1_INTF_CH10(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x400)|value<<10)
}
func (o *PWM_Type) GetIRQ1_INTF_CH10() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x400) >> 10
}
func (o *PWM_Type) SetIRQ1_INTF_CH9(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x200)|value<<9)
}
func (o *PWM_Type) GetIRQ1_INTF_CH9() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x200) >> 9
}
func (o *PWM_Type) SetIRQ1_INTF_CH8(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x100)|value<<8)
}
func (o *PWM_Type) GetIRQ1_INTF_CH8() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x100) >> 8
}
func (o *PWM_Type) SetIRQ1_INTF_CH7(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetIRQ1_INTF_CH7() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetIRQ1_INTF_CH6(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetIRQ1_INTF_CH6() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetIRQ1_INTF_CH5(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x20)|value<<5)
}
func (o *PWM_Type) GetIRQ1_INTF_CH5() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x20) >> 5
}
func (o *PWM_Type) SetIRQ1_INTF_CH4(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x10)|value<<4)
}
func (o *PWM_Type) GetIRQ1_INTF_CH4() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x10) >> 4
}
func (o *PWM_Type) SetIRQ1_INTF_CH3(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetIRQ1_INTF_CH3() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetIRQ1_INTF_CH2(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetIRQ1_INTF_CH2() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetIRQ1_INTF_CH1(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetIRQ1_INTF_CH1() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetIRQ1_INTF_CH0(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetIRQ1_INTF_CH0() uint32 {
	return volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x1
}

// PWM.IRQ1_INTS: Interrupt status after masking & forcing for irq1
func (o *PWM_Type) SetIRQ1_INTS_CH11(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x800)|value<<11)
}
func (o *PWM_Type) GetIRQ1_INTS_CH11() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x800) >> 11
}
func (o *PWM_Type) SetIRQ1_INTS_CH10(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x400)|value<<10)
}
func (o *PWM_Type) GetIRQ1_INTS_CH10() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x400) >> 10
}
func (o *PWM_Type) SetIRQ1_INTS_CH9(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x200)|value<<9)
}
func (o *PWM_Type) GetIRQ1_INTS_CH9() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x200) >> 9
}
func (o *PWM_Type) SetIRQ1_INTS_CH8(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x100)|value<<8)
}
func (o *PWM_Type) GetIRQ1_INTS_CH8() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x100) >> 8
}
func (o *PWM_Type) SetIRQ1_INTS_CH7(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x80)|value<<7)
}
func (o *PWM_Type) GetIRQ1_INTS_CH7() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x80) >> 7
}
func (o *PWM_Type) SetIRQ1_INTS_CH6(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x40)|value<<6)
}
func (o *PWM_Type) GetIRQ1_INTS_CH6() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x40) >> 6
}
func (o *PWM_Type) SetIRQ1_INTS_CH5(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x20)|value<<5)
}
func (o *PWM_Type) GetIRQ1_INTS_CH5() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x20) >> 5
}
func (o *PWM_Type) SetIRQ1_INTS_CH4(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x10)|value<<4)
}
func (o *PWM_Type) GetIRQ1_INTS_CH4() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x10) >> 4
}
func (o *PWM_Type) SetIRQ1_INTS_CH3(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x8)|value<<3)
}
func (o *PWM_Type) GetIRQ1_INTS_CH3() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x8) >> 3
}
func (o *PWM_Type) SetIRQ1_INTS_CH2(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x4)|value<<2)
}
func (o *PWM_Type) GetIRQ1_INTS_CH2() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x4) >> 2
}
func (o *PWM_Type) SetIRQ1_INTS_CH1(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x2)|value<<1)
}
func (o *PWM_Type) GetIRQ1_INTS_CH1() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x2) >> 1
}
func (o *PWM_Type) SetIRQ1_INTS_CH0(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x1)|value)
}
func (o *PWM_Type) GetIRQ1_INTS_CH0() uint32 {
	return volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x1
}

// Control and data interface to SAR ADC
type ADC_Type struct {
	CS     volatile.Register32 // 0x0
	RESULT volatile.Register32 // 0x4
	FCS    volatile.Register32 // 0x8
	FIFO   volatile.Register32 // 0xC
	DIV    volatile.Register32 // 0x10
	INTR   volatile.Register32 // 0x14
	INTE   volatile.Register32 // 0x18
	INTF   volatile.Register32 // 0x1C
	INTS   volatile.Register32 // 0x20
}

// ADC.CS: ADC Control and Status
func (o *ADC_Type) SetCS_RROBIN(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x1ff0000)|value<<16)
}
func (o *ADC_Type) GetCS_RROBIN() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x1ff0000) >> 16
}
func (o *ADC_Type) SetCS_AINSEL(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0xf000)|value<<12)
}
func (o *ADC_Type) GetCS_AINSEL() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0xf000) >> 12
}
func (o *ADC_Type) SetCS_ERR_STICKY(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x400)|value<<10)
}
func (o *ADC_Type) GetCS_ERR_STICKY() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x400) >> 10
}
func (o *ADC_Type) SetCS_ERR(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x200)|value<<9)
}
func (o *ADC_Type) GetCS_ERR() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x200) >> 9
}
func (o *ADC_Type) SetCS_READY(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x100)|value<<8)
}
func (o *ADC_Type) GetCS_READY() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x100) >> 8
}
func (o *ADC_Type) SetCS_START_MANY(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x8)|value<<3)
}
func (o *ADC_Type) GetCS_START_MANY() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x8) >> 3
}
func (o *ADC_Type) SetCS_START_ONCE(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x4)|value<<2)
}
func (o *ADC_Type) GetCS_START_ONCE() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x4) >> 2
}
func (o *ADC_Type) SetCS_TS_EN(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x2)|value<<1)
}
func (o *ADC_Type) GetCS_TS_EN() uint32 {
	return (volatile.LoadUint32(&o.CS.Reg) & 0x2) >> 1
}
func (o *ADC_Type) SetCS_EN(value uint32) {
	volatile.StoreUint32(&o.CS.Reg, volatile.LoadUint32(&o.CS.Reg)&^(0x1)|value)
}
func (o *ADC_Type) GetCS_EN() uint32 {
	return volatile.LoadUint32(&o.CS.Reg) & 0x1
}

// ADC.RESULT: Result of most recent ADC conversion
func (o *ADC_Type) SetRESULT(value uint32) {
	volatile.StoreUint32(&o.RESULT.Reg, volatile.LoadUint32(&o.RESULT.Reg)&^(0xfff)|value)
}
func (o *ADC_Type) GetRESULT() uint32 {
	return volatile.LoadUint32(&o.RESULT.Reg) & 0xfff
}

// ADC.FCS: FIFO control and status
func (o *ADC_Type) SetFCS_THRESH(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0xf000000)|value<<24)
}
func (o *ADC_Type) GetFCS_THRESH() uint32 {
	return (volatile.LoadUint32(&o.FCS.Reg) & 0xf000000) >> 24
}
func (o *ADC_Type) SetFCS_LEVEL(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0xf0000)|value<<16)
}
func (o *ADC_Type) GetFCS_LEVEL() uint32 {
	return (volatile.LoadUint32(&o.FCS.Reg) & 0xf0000) >> 16
}
func (o *ADC_Type) SetFCS_OVER(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0x800)|value<<11)
}
func (o *ADC_Type) GetFCS_OVER() uint32 {
	return (volatile.LoadUint32(&o.FCS.Reg) & 0x800) >> 11
}
func (o *ADC_Type) SetFCS_UNDER(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0x400)|value<<10)
}
func (o *ADC_Type) GetFCS_UNDER() uint32 {
	return (volatile.LoadUint32(&o.FCS.Reg) & 0x400) >> 10
}
func (o *ADC_Type) SetFCS_FULL(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0x200)|value<<9)
}
func (o *ADC_Type) GetFCS_FULL() uint32 {
	return (volatile.LoadUint32(&o.FCS.Reg) & 0x200) >> 9
}
func (o *ADC_Type) SetFCS_EMPTY(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0x100)|value<<8)
}
func (o *ADC_Type) GetFCS_EMPTY() uint32 {
	return (volatile.LoadUint32(&o.FCS.Reg) & 0x100) >> 8
}
func (o *ADC_Type) SetFCS_DREQ_EN(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0x8)|value<<3)
}
func (o *ADC_Type) GetFCS_DREQ_EN() uint32 {
	return (volatile.LoadUint32(&o.FCS.Reg) & 0x8) >> 3
}
func (o *ADC_Type) SetFCS_ERR(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0x4)|value<<2)
}
func (o *ADC_Type) GetFCS_ERR() uint32 {
	return (volatile.LoadUint32(&o.FCS.Reg) & 0x4) >> 2
}
func (o *ADC_Type) SetFCS_SHIFT(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0x2)|value<<1)
}
func (o *ADC_Type) GetFCS_SHIFT() uint32 {
	return (volatile.LoadUint32(&o.FCS.Reg) & 0x2) >> 1
}
func (o *ADC_Type) SetFCS_EN(value uint32) {
	volatile.StoreUint32(&o.FCS.Reg, volatile.LoadUint32(&o.FCS.Reg)&^(0x1)|value)
}
func (o *ADC_Type) GetFCS_EN() uint32 {
	return volatile.LoadUint32(&o.FCS.Reg) & 0x1
}

// ADC.FIFO: Conversion result FIFO
func (o *ADC_Type) SetFIFO_ERR(value uint32) {
	volatile.StoreUint32(&o.FIFO.Reg, volatile.LoadUint32(&o.FIFO.Reg)&^(0x8000)|value<<15)
}
func (o *ADC_Type) GetFIFO_ERR() uint32 {
	return (volatile.LoadUint32(&o.FIFO.Reg) & 0x8000) >> 15
}
func (o *ADC_Type) SetFIFO_VAL(value uint32) {
	volatile.StoreUint32(&o.FIFO.Reg, volatile.LoadUint32(&o.FIFO.Reg)&^(0xfff)|value)
}
func (o *ADC_Type) GetFIFO_VAL() uint32 {
	return volatile.LoadUint32(&o.FIFO.Reg) & 0xfff
}

// ADC.DIV: Clock divider. If non-zero, CS_START_MANY will start conversions at regular intervals rather than back-to-back. The divider is reset when either of these fields are written. Total period is 1 + INT + FRAC / 256
func (o *ADC_Type) SetDIV_INT(value uint32) {
	volatile.StoreUint32(&o.DIV.Reg, volatile.LoadUint32(&o.DIV.Reg)&^(0xffff00)|value<<8)
}
func (o *ADC_Type) GetDIV_INT() uint32 {
	return (volatile.LoadUint32(&o.DIV.Reg) & 0xffff00) >> 8
}
func (o *ADC_Type) SetDIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.DIV.Reg, volatile.LoadUint32(&o.DIV.Reg)&^(0xff)|value)
}
func (o *ADC_Type) GetDIV_FRAC() uint32 {
	return volatile.LoadUint32(&o.DIV.Reg) & 0xff
}

// ADC.INTR: Raw Interrupts
func (o *ADC_Type) SetINTR_FIFO(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *ADC_Type) GetINTR_FIFO() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// ADC.INTE: Interrupt Enable
func (o *ADC_Type) SetINTE_FIFO(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x1)|value)
}
func (o *ADC_Type) GetINTE_FIFO() uint32 {
	return volatile.LoadUint32(&o.INTE.Reg) & 0x1
}

// ADC.INTF: Interrupt Force
func (o *ADC_Type) SetINTF_FIFO(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x1)|value)
}
func (o *ADC_Type) GetINTF_FIFO() uint32 {
	return volatile.LoadUint32(&o.INTF.Reg) & 0x1
}

// ADC.INTS: Interrupt status after masking & forcing
func (o *ADC_Type) SetINTS_FIFO(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x1)|value)
}
func (o *ADC_Type) GetINTS_FIFO() uint32 {
	return volatile.LoadUint32(&o.INTS.Reg) & 0x1
}

// DW_apb_i2c address block List of configuration constants for the Synopsys I2C hardware (you may see references to these in I2C register header; these are *fixed* values, set at hardware design time): IC_ULTRA_FAST_MODE ................ 0x0 IC_UFM_TBUF_CNT_DEFAULT ........... 0x8 IC_UFM_SCL_LOW_COUNT .............. 0x0008 IC_UFM_SCL_HIGH_COUNT ............. 0x0006 IC_TX_TL .......................... 0x0 IC_TX_CMD_BLOCK ................... 0x1 IC_HAS_DMA ........................ 0x1 IC_HAS_ASYNC_FIFO ................. 0x0 IC_SMBUS_ARP ...................... 0x0 IC_FIRST_DATA_BYTE_STATUS ......... 0x1 IC_INTR_IO ........................ 0x1 IC_MASTER_MODE .................... 0x1 IC_DEFAULT_ACK_GENERAL_CALL ....... 0x1 IC_INTR_POL ....................... 0x1 IC_OPTIONAL_SAR ................... 0x0 IC_DEFAULT_TAR_SLAVE_ADDR ......... 0x055 IC_DEFAULT_SLAVE_ADDR ............. 0x055 IC_DEFAULT_HS_SPKLEN .............. 0x1 IC_FS_SCL_HIGH_COUNT .............. 0x0006 IC_HS_SCL_LOW_COUNT ............... 0x0008 IC_DEVICE_ID_VALUE ................ 0x0 IC_10BITADDR_MASTER ............... 0x0 IC_CLK_FREQ_OPTIMIZATION .......... 0x0 IC_DEFAULT_FS_SPKLEN .............. 0x7 IC_ADD_ENCODED_PARAMS ............. 0x0 IC_DEFAULT_SDA_HOLD ............... 0x000001 IC_DEFAULT_SDA_SETUP .............. 0x64 IC_AVOID_RX_FIFO_FLUSH_ON_TX_ABRT . 0x0 IC_CLOCK_PERIOD ................... 100 IC_EMPTYFIFO_HOLD_MASTER_EN ....... 1 IC_RESTART_EN ..................... 0x1 IC_TX_CMD_BLOCK_DEFAULT ........... 0x0 IC_BUS_CLEAR_FEATURE .............. 0x0 IC_CAP_LOADING .................... 100 IC_FS_SCL_LOW_COUNT ............... 0x000d APB_DATA_WIDTH .................... 32 IC_SDA_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_SLV_DATA_NACK_ONLY ............. 0x1 IC_10BITADDR_SLAVE ................ 0x0 IC_CLK_TYPE ....................... 0x0 IC_SMBUS_UDID_MSB ................. 0x0 IC_SMBUS_SUSPEND_ALERT ............ 0x0 IC_HS_SCL_HIGH_COUNT .............. 0x0006 IC_SLV_RESTART_DET_EN ............. 0x1 IC_SMBUS .......................... 0x0 IC_OPTIONAL_SAR_DEFAULT ........... 0x0 IC_PERSISTANT_SLV_ADDR_DEFAULT .... 0x0 IC_USE_COUNTS ..................... 0x0 IC_RX_BUFFER_DEPTH ................ 16 IC_SCL_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_RX_FULL_HLD_BUS_EN ............. 0x1 IC_SLAVE_DISABLE .................. 0x1 IC_RX_TL .......................... 0x0 IC_DEVICE_ID ...................... 0x0 IC_HC_COUNT_VALUES ................ 0x0 I2C_DYNAMIC_TAR_UPDATE ............ 0 IC_SMBUS_CLK_LOW_MEXT_DEFAULT ..... 0xffffffff IC_SMBUS_CLK_LOW_SEXT_DEFAULT ..... 0xffffffff IC_HS_MASTER_CODE ................. 0x1 IC_SMBUS_RST_IDLE_CNT_DEFAULT ..... 0xffff IC_SMBUS_UDID_LSB_DEFAULT ......... 0xffffffff IC_SS_SCL_HIGH_COUNT .............. 0x0028 IC_SS_SCL_LOW_COUNT ............... 0x002f IC_MAX_SPEED_MODE ................. 0x2 IC_STAT_FOR_CLK_STRETCH ........... 0x0 IC_STOP_DET_IF_MASTER_ACTIVE ...... 0x0 IC_DEFAULT_UFM_SPKLEN ............. 0x1 IC_TX_BUFFER_DEPTH ................ 16
type I2C0_Type struct {
	IC_CON                volatile.Register32 // 0x0
	IC_TAR                volatile.Register32 // 0x4
	IC_SAR                volatile.Register32 // 0x8
	_                     [4]byte
	IC_DATA_CMD           volatile.Register32 // 0x10
	IC_SS_SCL_HCNT        volatile.Register32 // 0x14
	IC_SS_SCL_LCNT        volatile.Register32 // 0x18
	IC_FS_SCL_HCNT        volatile.Register32 // 0x1C
	IC_FS_SCL_LCNT        volatile.Register32 // 0x20
	_                     [8]byte
	IC_INTR_STAT          volatile.Register32 // 0x2C
	IC_INTR_MASK          volatile.Register32 // 0x30
	IC_RAW_INTR_STAT      volatile.Register32 // 0x34
	IC_RX_TL              volatile.Register32 // 0x38
	IC_TX_TL              volatile.Register32 // 0x3C
	IC_CLR_INTR           volatile.Register32 // 0x40
	IC_CLR_RX_UNDER       volatile.Register32 // 0x44
	IC_CLR_RX_OVER        volatile.Register32 // 0x48
	IC_CLR_TX_OVER        volatile.Register32 // 0x4C
	IC_CLR_RD_REQ         volatile.Register32 // 0x50
	IC_CLR_TX_ABRT        volatile.Register32 // 0x54
	IC_CLR_RX_DONE        volatile.Register32 // 0x58
	IC_CLR_ACTIVITY       volatile.Register32 // 0x5C
	IC_CLR_STOP_DET       volatile.Register32 // 0x60
	IC_CLR_START_DET      volatile.Register32 // 0x64
	IC_CLR_GEN_CALL       volatile.Register32 // 0x68
	IC_ENABLE             volatile.Register32 // 0x6C
	IC_STATUS             volatile.Register32 // 0x70
	IC_TXFLR              volatile.Register32 // 0x74
	IC_RXFLR              volatile.Register32 // 0x78
	IC_SDA_HOLD           volatile.Register32 // 0x7C
	IC_TX_ABRT_SOURCE     volatile.Register32 // 0x80
	IC_SLV_DATA_NACK_ONLY volatile.Register32 // 0x84
	IC_DMA_CR             volatile.Register32 // 0x88
	IC_DMA_TDLR           volatile.Register32 // 0x8C
	IC_DMA_RDLR           volatile.Register32 // 0x90
	IC_SDA_SETUP          volatile.Register32 // 0x94
	IC_ACK_GENERAL_CALL   volatile.Register32 // 0x98
	IC_ENABLE_STATUS      volatile.Register32 // 0x9C
	IC_FS_SPKLEN          volatile.Register32 // 0xA0
	_                     [4]byte
	IC_CLR_RESTART_DET    volatile.Register32 // 0xA8
	_                     [72]byte
	IC_COMP_PARAM_1       volatile.Register32 // 0xF4
	IC_COMP_VERSION       volatile.Register32 // 0xF8
	IC_COMP_TYPE          volatile.Register32 // 0xFC
}

// I2C0.IC_CON: I2C Control Register. This register can be written only when the DW_apb_i2c is disabled, which corresponds to the IC_ENABLE[0] register being set to 0. Writes at other times have no effect. Read/Write Access: - bit 10 is read only. - bit 11 is read only - bit 16 is read only - bit 17 is read only - bits 18 and 19 are read only.
func (o *I2C0_Type) SetIC_CON_STOP_DET_IF_MASTER_ACTIVE(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x400)|value<<10)
}
func (o *I2C0_Type) GetIC_CON_STOP_DET_IF_MASTER_ACTIVE() uint32 {
	return (volatile.LoadUint32(&o.IC_CON.Reg) & 0x400) >> 10
}
func (o *I2C0_Type) SetIC_CON_RX_FIFO_FULL_HLD_CTRL(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x200)|value<<9)
}
func (o *I2C0_Type) GetIC_CON_RX_FIFO_FULL_HLD_CTRL() uint32 {
	return (volatile.LoadUint32(&o.IC_CON.Reg) & 0x200) >> 9
}
func (o *I2C0_Type) SetIC_CON_TX_EMPTY_CTRL(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x100)|value<<8)
}
func (o *I2C0_Type) GetIC_CON_TX_EMPTY_CTRL() uint32 {
	return (volatile.LoadUint32(&o.IC_CON.Reg) & 0x100) >> 8
}
func (o *I2C0_Type) SetIC_CON_STOP_DET_IFADDRESSED(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x80)|value<<7)
}
func (o *I2C0_Type) GetIC_CON_STOP_DET_IFADDRESSED() uint32 {
	return (volatile.LoadUint32(&o.IC_CON.Reg) & 0x80) >> 7
}
func (o *I2C0_Type) SetIC_CON_IC_SLAVE_DISABLE(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x40)|value<<6)
}
func (o *I2C0_Type) GetIC_CON_IC_SLAVE_DISABLE() uint32 {
	return (volatile.LoadUint32(&o.IC_CON.Reg) & 0x40) >> 6
}
func (o *I2C0_Type) SetIC_CON_IC_RESTART_EN(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x20)|value<<5)
}
func (o *I2C0_Type) GetIC_CON_IC_RESTART_EN() uint32 {
	return (volatile.LoadUint32(&o.IC_CON.Reg) & 0x20) >> 5
}
func (o *I2C0_Type) SetIC_CON_IC_10BITADDR_MASTER(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x10)|value<<4)
}
func (o *I2C0_Type) GetIC_CON_IC_10BITADDR_MASTER() uint32 {
	return (volatile.LoadUint32(&o.IC_CON.Reg) & 0x10) >> 4
}
func (o *I2C0_Type) SetIC_CON_IC_10BITADDR_SLAVE(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x8)|value<<3)
}
func (o *I2C0_Type) GetIC_CON_IC_10BITADDR_SLAVE() uint32 {
	return (volatile.LoadUint32(&o.IC_CON.Reg) & 0x8) >> 3
}
func (o *I2C0_Type) SetIC_CON_SPEED(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x6)|value<<1)
}
func (o *I2C0_Type) GetIC_CON_SPEED() uint32 {
	return (volatile.LoadUint32(&o.IC_CON.Reg) & 0x6) >> 1
}
func (o *I2C0_Type) SetIC_CON_MASTER_MODE(value uint32) {
	volatile.StoreUint32(&o.IC_CON.Reg, volatile.LoadUint32(&o.IC_CON.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CON_MASTER_MODE() uint32 {
	return volatile.LoadUint32(&o.IC_CON.Reg) & 0x1
}

// I2C0.IC_TAR: I2C Target Address Register This register is 12 bits wide, and bits 31:12 are reserved. This register can be written to only when IC_ENABLE[0] is set to 0. Note: If the software or application is aware that the DW_apb_i2c is not using the TAR address for the pending commands in the Tx FIFO, then it is possible to update the TAR address even while the Tx FIFO has entries (IC_STATUS[2]= 0). - It is not necessary to perform any write to this register if DW_apb_i2c is enabled as an I2C slave only.
func (o *I2C0_Type) SetIC_TAR_SPECIAL(value uint32) {
	volatile.StoreUint32(&o.IC_TAR.Reg, volatile.LoadUint32(&o.IC_TAR.Reg)&^(0x800)|value<<11)
}
func (o *I2C0_Type) GetIC_TAR_SPECIAL() uint32 {
	return (volatile.LoadUint32(&o.IC_TAR.Reg) & 0x800) >> 11
}
func (o *I2C0_Type) SetIC_TAR_GC_OR_START(value uint32) {
	volatile.StoreUint32(&o.IC_TAR.Reg, volatile.LoadUint32(&o.IC_TAR.Reg)&^(0x400)|value<<10)
}
func (o *I2C0_Type) GetIC_TAR_GC_OR_START() uint32 {
	return (volatile.LoadUint32(&o.IC_TAR.Reg) & 0x400) >> 10
}
func (o *I2C0_Type) SetIC_TAR(value uint32) {
	volatile.StoreUint32(&o.IC_TAR.Reg, volatile.LoadUint32(&o.IC_TAR.Reg)&^(0x3ff)|value)
}
func (o *I2C0_Type) GetIC_TAR() uint32 {
	return volatile.LoadUint32(&o.IC_TAR.Reg) & 0x3ff
}

// I2C0.IC_SAR: I2C Slave Address Register
func (o *I2C0_Type) SetIC_SAR(value uint32) {
	volatile.StoreUint32(&o.IC_SAR.Reg, volatile.LoadUint32(&o.IC_SAR.Reg)&^(0x3ff)|value)
}
func (o *I2C0_Type) GetIC_SAR() uint32 {
	return volatile.LoadUint32(&o.IC_SAR.Reg) & 0x3ff
}

// I2C0.IC_DATA_CMD: I2C Rx/Tx Data Buffer and Command Register; this is the register the CPU writes to when filling the TX FIFO and the CPU reads from when retrieving bytes from RX FIFO. The size of the register changes as follows: Write: - 11 bits when IC_EMPTYFIFO_HOLD_MASTER_EN=1 - 9 bits when IC_EMPTYFIFO_HOLD_MASTER_EN=0 Read: - 12 bits when IC_FIRST_DATA_BYTE_STATUS = 1 - 8 bits when IC_FIRST_DATA_BYTE_STATUS = 0 Note: In order for the DW_apb_i2c to continue acknowledging reads, a read command should be written for every byte that is to be received; otherwise the DW_apb_i2c will stop acknowledging.
func (o *I2C0_Type) SetIC_DATA_CMD_FIRST_DATA_BYTE(value uint32) {
	volatile.StoreUint32(&o.IC_DATA_CMD.Reg, volatile.LoadUint32(&o.IC_DATA_CMD.Reg)&^(0x800)|value<<11)
}
func (o *I2C0_Type) GetIC_DATA_CMD_FIRST_DATA_BYTE() uint32 {
	return (volatile.LoadUint32(&o.IC_DATA_CMD.Reg) & 0x800) >> 11
}
func (o *I2C0_Type) SetIC_DATA_CMD_RESTART(value uint32) {
	volatile.StoreUint32(&o.IC_DATA_CMD.Reg, volatile.LoadUint32(&o.IC_DATA_CMD.Reg)&^(0x400)|value<<10)
}
func (o *I2C0_Type) GetIC_DATA_CMD_RESTART() uint32 {
	return (volatile.LoadUint32(&o.IC_DATA_CMD.Reg) & 0x400) >> 10
}
func (o *I2C0_Type) SetIC_DATA_CMD_STOP(value uint32) {
	volatile.StoreUint32(&o.IC_DATA_CMD.Reg, volatile.LoadUint32(&o.IC_DATA_CMD.Reg)&^(0x200)|value<<9)
}
func (o *I2C0_Type) GetIC_DATA_CMD_STOP() uint32 {
	return (volatile.LoadUint32(&o.IC_DATA_CMD.Reg) & 0x200) >> 9
}
func (o *I2C0_Type) SetIC_DATA_CMD_CMD(value uint32) {
	volatile.StoreUint32(&o.IC_DATA_CMD.Reg, volatile.LoadUint32(&o.IC_DATA_CMD.Reg)&^(0x100)|value<<8)
}
func (o *I2C0_Type) GetIC_DATA_CMD_CMD() uint32 {
	return (volatile.LoadUint32(&o.IC_DATA_CMD.Reg) & 0x100) >> 8
}
func (o *I2C0_Type) SetIC_DATA_CMD_DAT(value uint32) {
	volatile.StoreUint32(&o.IC_DATA_CMD.Reg, volatile.LoadUint32(&o.IC_DATA_CMD.Reg)&^(0xff)|value)
}
func (o *I2C0_Type) GetIC_DATA_CMD_DAT() uint32 {
	return volatile.LoadUint32(&o.IC_DATA_CMD.Reg) & 0xff
}

// I2C0.IC_SS_SCL_HCNT: Standard Speed I2C Clock SCL High Count Register
func (o *I2C0_Type) SetIC_SS_SCL_HCNT(value uint32) {
	volatile.StoreUint32(&o.IC_SS_SCL_HCNT.Reg, volatile.LoadUint32(&o.IC_SS_SCL_HCNT.Reg)&^(0xffff)|value)
}
func (o *I2C0_Type) GetIC_SS_SCL_HCNT() uint32 {
	return volatile.LoadUint32(&o.IC_SS_SCL_HCNT.Reg) & 0xffff
}

// I2C0.IC_SS_SCL_LCNT: Standard Speed I2C Clock SCL Low Count Register
func (o *I2C0_Type) SetIC_SS_SCL_LCNT(value uint32) {
	volatile.StoreUint32(&o.IC_SS_SCL_LCNT.Reg, volatile.LoadUint32(&o.IC_SS_SCL_LCNT.Reg)&^(0xffff)|value)
}
func (o *I2C0_Type) GetIC_SS_SCL_LCNT() uint32 {
	return volatile.LoadUint32(&o.IC_SS_SCL_LCNT.Reg) & 0xffff
}

// I2C0.IC_FS_SCL_HCNT: Fast Mode or Fast Mode Plus I2C Clock SCL High Count Register
func (o *I2C0_Type) SetIC_FS_SCL_HCNT(value uint32) {
	volatile.StoreUint32(&o.IC_FS_SCL_HCNT.Reg, volatile.LoadUint32(&o.IC_FS_SCL_HCNT.Reg)&^(0xffff)|value)
}
func (o *I2C0_Type) GetIC_FS_SCL_HCNT() uint32 {
	return volatile.LoadUint32(&o.IC_FS_SCL_HCNT.Reg) & 0xffff
}

// I2C0.IC_FS_SCL_LCNT: Fast Mode or Fast Mode Plus I2C Clock SCL Low Count Register
func (o *I2C0_Type) SetIC_FS_SCL_LCNT(value uint32) {
	volatile.StoreUint32(&o.IC_FS_SCL_LCNT.Reg, volatile.LoadUint32(&o.IC_FS_SCL_LCNT.Reg)&^(0xffff)|value)
}
func (o *I2C0_Type) GetIC_FS_SCL_LCNT() uint32 {
	return volatile.LoadUint32(&o.IC_FS_SCL_LCNT.Reg) & 0xffff
}

// I2C0.IC_INTR_STAT: I2C Interrupt Status Register Each bit in this register has a corresponding mask bit in the IC_INTR_MASK register. These bits are cleared by reading the matching interrupt clear register. The unmasked raw versions of these bits are available in the IC_RAW_INTR_STAT register.
func (o *I2C0_Type) SetIC_INTR_STAT_R_RESTART_DET(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x1000)|value<<12)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_RESTART_DET() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x1000) >> 12
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_GEN_CALL(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x800)|value<<11)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_GEN_CALL() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x800) >> 11
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_START_DET(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x400)|value<<10)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_START_DET() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x400) >> 10
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_STOP_DET(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x200)|value<<9)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_STOP_DET() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x200) >> 9
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_ACTIVITY(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x100)|value<<8)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_ACTIVITY() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x100) >> 8
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_RX_DONE(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x80)|value<<7)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_RX_DONE() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x80) >> 7
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_TX_ABRT(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x40)|value<<6)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_TX_ABRT() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x40) >> 6
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_RD_REQ(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x20)|value<<5)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_RD_REQ() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x20) >> 5
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_TX_EMPTY(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x10)|value<<4)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_TX_EMPTY() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x10) >> 4
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_TX_OVER(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x8)|value<<3)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_TX_OVER() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x8) >> 3
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_RX_FULL(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x4)|value<<2)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_RX_FULL() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x4) >> 2
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_RX_OVER(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x2)|value<<1)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_RX_OVER() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x2) >> 1
}
func (o *I2C0_Type) SetIC_INTR_STAT_R_RX_UNDER(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_INTR_STAT.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_INTR_STAT_R_RX_UNDER() uint32 {
	return volatile.LoadUint32(&o.IC_INTR_STAT.Reg) & 0x1
}

// I2C0.IC_INTR_MASK: I2C Interrupt Mask Register. These bits mask their corresponding interrupt status bits. This register is active low; a value of 0 masks the interrupt, whereas a value of 1 unmasks the interrupt.
func (o *I2C0_Type) SetIC_INTR_MASK_M_RESTART_DET(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x1000)|value<<12)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_RESTART_DET() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x1000) >> 12
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_GEN_CALL(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x800)|value<<11)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_GEN_CALL() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x800) >> 11
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_START_DET(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x400)|value<<10)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_START_DET() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x400) >> 10
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_STOP_DET(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x200)|value<<9)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_STOP_DET() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x200) >> 9
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_ACTIVITY(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x100)|value<<8)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_ACTIVITY() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x100) >> 8
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_RX_DONE(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x80)|value<<7)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_RX_DONE() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x80) >> 7
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_TX_ABRT(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x40)|value<<6)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_TX_ABRT() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x40) >> 6
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_RD_REQ(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x20)|value<<5)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_RD_REQ() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x20) >> 5
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_TX_EMPTY(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x10)|value<<4)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_TX_EMPTY() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x10) >> 4
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_TX_OVER(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x8)|value<<3)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_TX_OVER() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x8) >> 3
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_RX_FULL(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x4)|value<<2)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_RX_FULL() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x4) >> 2
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_RX_OVER(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x2)|value<<1)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_RX_OVER() uint32 {
	return (volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x2) >> 1
}
func (o *I2C0_Type) SetIC_INTR_MASK_M_RX_UNDER(value uint32) {
	volatile.StoreUint32(&o.IC_INTR_MASK.Reg, volatile.LoadUint32(&o.IC_INTR_MASK.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_INTR_MASK_M_RX_UNDER() uint32 {
	return volatile.LoadUint32(&o.IC_INTR_MASK.Reg) & 0x1
}

// I2C0.IC_RAW_INTR_STAT: I2C Raw Interrupt Status Register Unlike the IC_INTR_STAT register, these bits are not masked so they always show the true status of the DW_apb_i2c.
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_RESTART_DET(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x1000)|value<<12)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_RESTART_DET() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x1000) >> 12
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_GEN_CALL(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x800)|value<<11)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_GEN_CALL() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x800) >> 11
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_START_DET(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x400)|value<<10)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_START_DET() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x400) >> 10
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_STOP_DET(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x200)|value<<9)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_STOP_DET() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x200) >> 9
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_ACTIVITY(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x100)|value<<8)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_ACTIVITY() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x100) >> 8
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_RX_DONE(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x80)|value<<7)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_RX_DONE() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x80) >> 7
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_TX_ABRT(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x40)|value<<6)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_TX_ABRT() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x40) >> 6
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_RD_REQ(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x20)|value<<5)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_RD_REQ() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x20) >> 5
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_TX_EMPTY(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x10)|value<<4)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_TX_EMPTY() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x10) >> 4
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_TX_OVER(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x8)|value<<3)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_TX_OVER() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x8) >> 3
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_RX_FULL(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x4)|value<<2)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_RX_FULL() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x4) >> 2
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_RX_OVER(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x2)|value<<1)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_RX_OVER() uint32 {
	return (volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x2) >> 1
}
func (o *I2C0_Type) SetIC_RAW_INTR_STAT_RX_UNDER(value uint32) {
	volatile.StoreUint32(&o.IC_RAW_INTR_STAT.Reg, volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_RAW_INTR_STAT_RX_UNDER() uint32 {
	return volatile.LoadUint32(&o.IC_RAW_INTR_STAT.Reg) & 0x1
}

// I2C0.IC_RX_TL: I2C Receive FIFO Threshold Register
func (o *I2C0_Type) SetIC_RX_TL_RX_TL(value uint32) {
	volatile.StoreUint32(&o.IC_RX_TL.Reg, volatile.LoadUint32(&o.IC_RX_TL.Reg)&^(0xff)|value)
}
func (o *I2C0_Type) GetIC_RX_TL_RX_TL() uint32 {
	return volatile.LoadUint32(&o.IC_RX_TL.Reg) & 0xff
}

// I2C0.IC_TX_TL: I2C Transmit FIFO Threshold Register
func (o *I2C0_Type) SetIC_TX_TL_TX_TL(value uint32) {
	volatile.StoreUint32(&o.IC_TX_TL.Reg, volatile.LoadUint32(&o.IC_TX_TL.Reg)&^(0xff)|value)
}
func (o *I2C0_Type) GetIC_TX_TL_TX_TL() uint32 {
	return volatile.LoadUint32(&o.IC_TX_TL.Reg) & 0xff
}

// I2C0.IC_CLR_INTR: Clear Combined and Individual Interrupt Register
func (o *I2C0_Type) SetIC_CLR_INTR_CLR_INTR(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_INTR.Reg, volatile.LoadUint32(&o.IC_CLR_INTR.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_INTR_CLR_INTR() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_INTR.Reg) & 0x1
}

// I2C0.IC_CLR_RX_UNDER: Clear RX_UNDER Interrupt Register
func (o *I2C0_Type) SetIC_CLR_RX_UNDER_CLR_RX_UNDER(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_RX_UNDER.Reg, volatile.LoadUint32(&o.IC_CLR_RX_UNDER.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_RX_UNDER_CLR_RX_UNDER() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_RX_UNDER.Reg) & 0x1
}

// I2C0.IC_CLR_RX_OVER: Clear RX_OVER Interrupt Register
func (o *I2C0_Type) SetIC_CLR_RX_OVER_CLR_RX_OVER(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_RX_OVER.Reg, volatile.LoadUint32(&o.IC_CLR_RX_OVER.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_RX_OVER_CLR_RX_OVER() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_RX_OVER.Reg) & 0x1
}

// I2C0.IC_CLR_TX_OVER: Clear TX_OVER Interrupt Register
func (o *I2C0_Type) SetIC_CLR_TX_OVER_CLR_TX_OVER(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_TX_OVER.Reg, volatile.LoadUint32(&o.IC_CLR_TX_OVER.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_TX_OVER_CLR_TX_OVER() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_TX_OVER.Reg) & 0x1
}

// I2C0.IC_CLR_RD_REQ: Clear RD_REQ Interrupt Register
func (o *I2C0_Type) SetIC_CLR_RD_REQ_CLR_RD_REQ(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_RD_REQ.Reg, volatile.LoadUint32(&o.IC_CLR_RD_REQ.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_RD_REQ_CLR_RD_REQ() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_RD_REQ.Reg) & 0x1
}

// I2C0.IC_CLR_TX_ABRT: Clear TX_ABRT Interrupt Register
func (o *I2C0_Type) SetIC_CLR_TX_ABRT_CLR_TX_ABRT(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_TX_ABRT.Reg, volatile.LoadUint32(&o.IC_CLR_TX_ABRT.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_TX_ABRT_CLR_TX_ABRT() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_TX_ABRT.Reg) & 0x1
}

// I2C0.IC_CLR_RX_DONE: Clear RX_DONE Interrupt Register
func (o *I2C0_Type) SetIC_CLR_RX_DONE_CLR_RX_DONE(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_RX_DONE.Reg, volatile.LoadUint32(&o.IC_CLR_RX_DONE.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_RX_DONE_CLR_RX_DONE() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_RX_DONE.Reg) & 0x1
}

// I2C0.IC_CLR_ACTIVITY: Clear ACTIVITY Interrupt Register
func (o *I2C0_Type) SetIC_CLR_ACTIVITY_CLR_ACTIVITY(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_ACTIVITY.Reg, volatile.LoadUint32(&o.IC_CLR_ACTIVITY.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_ACTIVITY_CLR_ACTIVITY() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_ACTIVITY.Reg) & 0x1
}

// I2C0.IC_CLR_STOP_DET: Clear STOP_DET Interrupt Register
func (o *I2C0_Type) SetIC_CLR_STOP_DET_CLR_STOP_DET(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_STOP_DET.Reg, volatile.LoadUint32(&o.IC_CLR_STOP_DET.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_STOP_DET_CLR_STOP_DET() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_STOP_DET.Reg) & 0x1
}

// I2C0.IC_CLR_START_DET: Clear START_DET Interrupt Register
func (o *I2C0_Type) SetIC_CLR_START_DET_CLR_START_DET(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_START_DET.Reg, volatile.LoadUint32(&o.IC_CLR_START_DET.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_START_DET_CLR_START_DET() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_START_DET.Reg) & 0x1
}

// I2C0.IC_CLR_GEN_CALL: Clear GEN_CALL Interrupt Register
func (o *I2C0_Type) SetIC_CLR_GEN_CALL_CLR_GEN_CALL(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_GEN_CALL.Reg, volatile.LoadUint32(&o.IC_CLR_GEN_CALL.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_GEN_CALL_CLR_GEN_CALL() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_GEN_CALL.Reg) & 0x1
}

// I2C0.IC_ENABLE: I2C Enable Register
func (o *I2C0_Type) SetIC_ENABLE_TX_CMD_BLOCK(value uint32) {
	volatile.StoreUint32(&o.IC_ENABLE.Reg, volatile.LoadUint32(&o.IC_ENABLE.Reg)&^(0x4)|value<<2)
}
func (o *I2C0_Type) GetIC_ENABLE_TX_CMD_BLOCK() uint32 {
	return (volatile.LoadUint32(&o.IC_ENABLE.Reg) & 0x4) >> 2
}
func (o *I2C0_Type) SetIC_ENABLE_ABORT(value uint32) {
	volatile.StoreUint32(&o.IC_ENABLE.Reg, volatile.LoadUint32(&o.IC_ENABLE.Reg)&^(0x2)|value<<1)
}
func (o *I2C0_Type) GetIC_ENABLE_ABORT() uint32 {
	return (volatile.LoadUint32(&o.IC_ENABLE.Reg) & 0x2) >> 1
}
func (o *I2C0_Type) SetIC_ENABLE_ENABLE(value uint32) {
	volatile.StoreUint32(&o.IC_ENABLE.Reg, volatile.LoadUint32(&o.IC_ENABLE.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_ENABLE_ENABLE() uint32 {
	return volatile.LoadUint32(&o.IC_ENABLE.Reg) & 0x1
}

// I2C0.IC_STATUS: I2C Status Register This is a read-only register used to indicate the current transfer status and FIFO status. The status register may be read at any time. None of the bits in this register request an interrupt. When the I2C is disabled by writing 0 in bit 0 of the IC_ENABLE register: - Bits 1 and 2 are set to 1 - Bits 3 and 10 are set to 0 When the master or slave state machines goes to idle and ic_en=0: - Bits 5 and 6 are set to 0
func (o *I2C0_Type) SetIC_STATUS_SLV_ACTIVITY(value uint32) {
	volatile.StoreUint32(&o.IC_STATUS.Reg, volatile.LoadUint32(&o.IC_STATUS.Reg)&^(0x40)|value<<6)
}
func (o *I2C0_Type) GetIC_STATUS_SLV_ACTIVITY() uint32 {
	return (volatile.LoadUint32(&o.IC_STATUS.Reg) & 0x40) >> 6
}
func (o *I2C0_Type) SetIC_STATUS_MST_ACTIVITY(value uint32) {
	volatile.StoreUint32(&o.IC_STATUS.Reg, volatile.LoadUint32(&o.IC_STATUS.Reg)&^(0x20)|value<<5)
}
func (o *I2C0_Type) GetIC_STATUS_MST_ACTIVITY() uint32 {
	return (volatile.LoadUint32(&o.IC_STATUS.Reg) & 0x20) >> 5
}
func (o *I2C0_Type) SetIC_STATUS_RFF(value uint32) {
	volatile.StoreUint32(&o.IC_STATUS.Reg, volatile.LoadUint32(&o.IC_STATUS.Reg)&^(0x10)|value<<4)
}
func (o *I2C0_Type) GetIC_STATUS_RFF() uint32 {
	return (volatile.LoadUint32(&o.IC_STATUS.Reg) & 0x10) >> 4
}
func (o *I2C0_Type) SetIC_STATUS_RFNE(value uint32) {
	volatile.StoreUint32(&o.IC_STATUS.Reg, volatile.LoadUint32(&o.IC_STATUS.Reg)&^(0x8)|value<<3)
}
func (o *I2C0_Type) GetIC_STATUS_RFNE() uint32 {
	return (volatile.LoadUint32(&o.IC_STATUS.Reg) & 0x8) >> 3
}
func (o *I2C0_Type) SetIC_STATUS_TFE(value uint32) {
	volatile.StoreUint32(&o.IC_STATUS.Reg, volatile.LoadUint32(&o.IC_STATUS.Reg)&^(0x4)|value<<2)
}
func (o *I2C0_Type) GetIC_STATUS_TFE() uint32 {
	return (volatile.LoadUint32(&o.IC_STATUS.Reg) & 0x4) >> 2
}
func (o *I2C0_Type) SetIC_STATUS_TFNF(value uint32) {
	volatile.StoreUint32(&o.IC_STATUS.Reg, volatile.LoadUint32(&o.IC_STATUS.Reg)&^(0x2)|value<<1)
}
func (o *I2C0_Type) GetIC_STATUS_TFNF() uint32 {
	return (volatile.LoadUint32(&o.IC_STATUS.Reg) & 0x2) >> 1
}
func (o *I2C0_Type) SetIC_STATUS_ACTIVITY(value uint32) {
	volatile.StoreUint32(&o.IC_STATUS.Reg, volatile.LoadUint32(&o.IC_STATUS.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_STATUS_ACTIVITY() uint32 {
	return volatile.LoadUint32(&o.IC_STATUS.Reg) & 0x1
}

// I2C0.IC_TXFLR: I2C Transmit FIFO Level Register This register contains the number of valid data entries in the transmit FIFO buffer. It is cleared whenever: - The I2C is disabled - There is a transmit abort - that is, TX_ABRT bit is set in the IC_RAW_INTR_STAT register - The slave bulk transmit mode is aborted The register increments whenever data is placed into the transmit FIFO and decrements when data is taken from the transmit FIFO.
func (o *I2C0_Type) SetIC_TXFLR_TXFLR(value uint32) {
	volatile.StoreUint32(&o.IC_TXFLR.Reg, volatile.LoadUint32(&o.IC_TXFLR.Reg)&^(0x1f)|value)
}
func (o *I2C0_Type) GetIC_TXFLR_TXFLR() uint32 {
	return volatile.LoadUint32(&o.IC_TXFLR.Reg) & 0x1f
}

// I2C0.IC_RXFLR: I2C Receive FIFO Level Register This register contains the number of valid data entries in the receive FIFO buffer. It is cleared whenever: - The I2C is disabled - Whenever there is a transmit abort caused by any of the events tracked in IC_TX_ABRT_SOURCE The register increments whenever data is placed into the receive FIFO and decrements when data is taken from the receive FIFO.
func (o *I2C0_Type) SetIC_RXFLR_RXFLR(value uint32) {
	volatile.StoreUint32(&o.IC_RXFLR.Reg, volatile.LoadUint32(&o.IC_RXFLR.Reg)&^(0x1f)|value)
}
func (o *I2C0_Type) GetIC_RXFLR_RXFLR() uint32 {
	return volatile.LoadUint32(&o.IC_RXFLR.Reg) & 0x1f
}

// I2C0.IC_SDA_HOLD: I2C SDA Hold Time Length Register The bits [15:0] of this register are used to control the hold time of SDA during transmit in both slave and master mode (after SCL goes from HIGH to LOW). The bits [23:16] of this register are used to extend the SDA transition (if any) whenever SCL is HIGH in the receiver in either master or slave mode. Writes to this register succeed only when IC_ENABLE[0]=0. The values in this register are in units of ic_clk period. The value programmed in IC_SDA_TX_HOLD must be greater than the minimum hold time in each mode (one cycle in master mode, seven cycles in slave mode) for the value to be implemented. The programmed SDA hold time during transmit (IC_SDA_TX_HOLD) cannot exceed at any time the duration of the low part of scl. Therefore the programmed value cannot be larger than N_SCL_LOW-2, where N_SCL_LOW is the duration of the low part of the scl period measured in ic_clk cycles.
func (o *I2C0_Type) SetIC_SDA_HOLD_IC_SDA_RX_HOLD(value uint32) {
	volatile.StoreUint32(&o.IC_SDA_HOLD.Reg, volatile.LoadUint32(&o.IC_SDA_HOLD.Reg)&^(0xff0000)|value<<16)
}
func (o *I2C0_Type) GetIC_SDA_HOLD_IC_SDA_RX_HOLD() uint32 {
	return (volatile.LoadUint32(&o.IC_SDA_HOLD.Reg) & 0xff0000) >> 16
}
func (o *I2C0_Type) SetIC_SDA_HOLD_IC_SDA_TX_HOLD(value uint32) {
	volatile.StoreUint32(&o.IC_SDA_HOLD.Reg, volatile.LoadUint32(&o.IC_SDA_HOLD.Reg)&^(0xffff)|value)
}
func (o *I2C0_Type) GetIC_SDA_HOLD_IC_SDA_TX_HOLD() uint32 {
	return volatile.LoadUint32(&o.IC_SDA_HOLD.Reg) & 0xffff
}

// I2C0.IC_TX_ABRT_SOURCE: I2C Transmit Abort Source Register This register has 32 bits that indicate the source of the TX_ABRT bit. Except for Bit 9, this register is cleared whenever the IC_CLR_TX_ABRT register or the IC_CLR_INTR register is read. To clear Bit 9, the source of the ABRT_SBYTE_NORSTRT must be fixed first; RESTART must be enabled (IC_CON[5]=1), the SPECIAL bit must be cleared (IC_TAR[11]), or the GC_OR_START bit must be cleared (IC_TAR[10]). Once the source of the ABRT_SBYTE_NORSTRT is fixed, then this bit can be cleared in the same manner as other bits in this register. If the source of the ABRT_SBYTE_NORSTRT is not fixed before attempting to clear this bit, Bit 9 clears for one cycle and is then re-asserted.
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_TX_FLUSH_CNT(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0xff800000)|value<<23)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_TX_FLUSH_CNT() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0xff800000) >> 23
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_USER_ABRT(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x10000)|value<<16)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_USER_ABRT() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x10000) >> 16
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_SLVRD_INTX(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x8000)|value<<15)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_SLVRD_INTX() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x8000) >> 15
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_SLV_ARBLOST(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x4000)|value<<14)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_SLV_ARBLOST() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x4000) >> 14
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_SLVFLUSH_TXFIFO(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x2000)|value<<13)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_SLVFLUSH_TXFIFO() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x2000) >> 13
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ARB_LOST(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x1000)|value<<12)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ARB_LOST() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x1000) >> 12
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_MASTER_DIS(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x800)|value<<11)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_MASTER_DIS() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x800) >> 11
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_10B_RD_NORSTRT(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x400)|value<<10)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_10B_RD_NORSTRT() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x400) >> 10
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_SBYTE_NORSTRT(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x200)|value<<9)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_SBYTE_NORSTRT() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x200) >> 9
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_HS_NORSTRT(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x100)|value<<8)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_HS_NORSTRT() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x100) >> 8
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_SBYTE_ACKDET(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x80)|value<<7)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_SBYTE_ACKDET() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x80) >> 7
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_HS_ACKDET(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x40)|value<<6)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_HS_ACKDET() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x40) >> 6
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_GCALL_READ(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x20)|value<<5)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_GCALL_READ() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x20) >> 5
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_GCALL_NOACK(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x10)|value<<4)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_GCALL_NOACK() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x10) >> 4
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_TXDATA_NOACK(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x8)|value<<3)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_TXDATA_NOACK() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x8) >> 3
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_10ADDR2_NOACK(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x4)|value<<2)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_10ADDR2_NOACK() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x4) >> 2
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_10ADDR1_NOACK(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x2)|value<<1)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_10ADDR1_NOACK() uint32 {
	return (volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x2) >> 1
}
func (o *I2C0_Type) SetIC_TX_ABRT_SOURCE_ABRT_7B_ADDR_NOACK(value uint32) {
	volatile.StoreUint32(&o.IC_TX_ABRT_SOURCE.Reg, volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_TX_ABRT_SOURCE_ABRT_7B_ADDR_NOACK() uint32 {
	return volatile.LoadUint32(&o.IC_TX_ABRT_SOURCE.Reg) & 0x1
}

// I2C0.IC_SLV_DATA_NACK_ONLY: Generate Slave Data NACK Register The register is used to generate a NACK for the data part of a transfer when DW_apb_i2c is acting as a slave-receiver. This register only exists when the IC_SLV_DATA_NACK_ONLY parameter is set to 1. When this parameter disabled, this register does not exist and writing to the register's address has no effect. A write can occur on this register if both of the following conditions are met: - DW_apb_i2c is disabled (IC_ENABLE[0] = 0) - Slave part is inactive (IC_STATUS[6] = 0) Note: The IC_STATUS[6] is a register read-back location for the internal slv_activity signal; the user should poll this before writing the ic_slv_data_nack_only bit.
func (o *I2C0_Type) SetIC_SLV_DATA_NACK_ONLY_NACK(value uint32) {
	volatile.StoreUint32(&o.IC_SLV_DATA_NACK_ONLY.Reg, volatile.LoadUint32(&o.IC_SLV_DATA_NACK_ONLY.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_SLV_DATA_NACK_ONLY_NACK() uint32 {
	return volatile.LoadUint32(&o.IC_SLV_DATA_NACK_ONLY.Reg) & 0x1
}

// I2C0.IC_DMA_CR: DMA Control Register The register is used to enable the DMA Controller interface operation. There is a separate bit for transmit and receive. This can be programmed regardless of the state of IC_ENABLE.
func (o *I2C0_Type) SetIC_DMA_CR_TDMAE(value uint32) {
	volatile.StoreUint32(&o.IC_DMA_CR.Reg, volatile.LoadUint32(&o.IC_DMA_CR.Reg)&^(0x2)|value<<1)
}
func (o *I2C0_Type) GetIC_DMA_CR_TDMAE() uint32 {
	return (volatile.LoadUint32(&o.IC_DMA_CR.Reg) & 0x2) >> 1
}
func (o *I2C0_Type) SetIC_DMA_CR_RDMAE(value uint32) {
	volatile.StoreUint32(&o.IC_DMA_CR.Reg, volatile.LoadUint32(&o.IC_DMA_CR.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_DMA_CR_RDMAE() uint32 {
	return volatile.LoadUint32(&o.IC_DMA_CR.Reg) & 0x1
}

// I2C0.IC_DMA_TDLR: DMA Transmit Data Level Register
func (o *I2C0_Type) SetIC_DMA_TDLR_DMATDL(value uint32) {
	volatile.StoreUint32(&o.IC_DMA_TDLR.Reg, volatile.LoadUint32(&o.IC_DMA_TDLR.Reg)&^(0xf)|value)
}
func (o *I2C0_Type) GetIC_DMA_TDLR_DMATDL() uint32 {
	return volatile.LoadUint32(&o.IC_DMA_TDLR.Reg) & 0xf
}

// I2C0.IC_DMA_RDLR: I2C Receive Data Level Register
func (o *I2C0_Type) SetIC_DMA_RDLR_DMARDL(value uint32) {
	volatile.StoreUint32(&o.IC_DMA_RDLR.Reg, volatile.LoadUint32(&o.IC_DMA_RDLR.Reg)&^(0xf)|value)
}
func (o *I2C0_Type) GetIC_DMA_RDLR_DMARDL() uint32 {
	return volatile.LoadUint32(&o.IC_DMA_RDLR.Reg) & 0xf
}

// I2C0.IC_SDA_SETUP: I2C SDA Setup Register This register controls the amount of time delay (in terms of number of ic_clk clock periods) introduced in the rising edge of SCL - relative to SDA changing - when DW_apb_i2c services a read request in a slave-transmitter operation. The relevant I2C requirement is tSU:DAT (note 4) as detailed in the I2C Bus Specification. This register must be programmed with a value equal to or greater than 2. Writes to this register succeed only when IC_ENABLE[0] = 0. Note: The length of setup time is calculated using [(IC_SDA_SETUP - 1) * (ic_clk_period)], so if the user requires 10 ic_clk periods of setup time, they should program a value of 11. The IC_SDA_SETUP register is only used by the DW_apb_i2c when operating as a slave transmitter.
func (o *I2C0_Type) SetIC_SDA_SETUP_SDA_SETUP(value uint32) {
	volatile.StoreUint32(&o.IC_SDA_SETUP.Reg, volatile.LoadUint32(&o.IC_SDA_SETUP.Reg)&^(0xff)|value)
}
func (o *I2C0_Type) GetIC_SDA_SETUP_SDA_SETUP() uint32 {
	return volatile.LoadUint32(&o.IC_SDA_SETUP.Reg) & 0xff
}

// I2C0.IC_ACK_GENERAL_CALL: I2C ACK General Call Register The register controls whether DW_apb_i2c responds with a ACK or NACK when it receives an I2C General Call address. This register is applicable only when the DW_apb_i2c is in slave mode.
func (o *I2C0_Type) SetIC_ACK_GENERAL_CALL_ACK_GEN_CALL(value uint32) {
	volatile.StoreUint32(&o.IC_ACK_GENERAL_CALL.Reg, volatile.LoadUint32(&o.IC_ACK_GENERAL_CALL.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_ACK_GENERAL_CALL_ACK_GEN_CALL() uint32 {
	return volatile.LoadUint32(&o.IC_ACK_GENERAL_CALL.Reg) & 0x1
}

// I2C0.IC_ENABLE_STATUS: I2C Enable Status Register The register is used to report the DW_apb_i2c hardware status when the IC_ENABLE[0] register is set from 1 to 0; that is, when DW_apb_i2c is disabled. If IC_ENABLE[0] has been set to 1, bits 2:1 are forced to 0, and bit 0 is forced to 1. If IC_ENABLE[0] has been set to 0, bits 2:1 is only be valid as soon as bit 0 is read as '0'. Note: When IC_ENABLE[0] has been set to 0, a delay occurs for bit 0 to be read as 0 because disabling the DW_apb_i2c depends on I2C bus activities.
func (o *I2C0_Type) SetIC_ENABLE_STATUS_SLV_RX_DATA_LOST(value uint32) {
	volatile.StoreUint32(&o.IC_ENABLE_STATUS.Reg, volatile.LoadUint32(&o.IC_ENABLE_STATUS.Reg)&^(0x4)|value<<2)
}
func (o *I2C0_Type) GetIC_ENABLE_STATUS_SLV_RX_DATA_LOST() uint32 {
	return (volatile.LoadUint32(&o.IC_ENABLE_STATUS.Reg) & 0x4) >> 2
}
func (o *I2C0_Type) SetIC_ENABLE_STATUS_SLV_DISABLED_WHILE_BUSY(value uint32) {
	volatile.StoreUint32(&o.IC_ENABLE_STATUS.Reg, volatile.LoadUint32(&o.IC_ENABLE_STATUS.Reg)&^(0x2)|value<<1)
}
func (o *I2C0_Type) GetIC_ENABLE_STATUS_SLV_DISABLED_WHILE_BUSY() uint32 {
	return (volatile.LoadUint32(&o.IC_ENABLE_STATUS.Reg) & 0x2) >> 1
}
func (o *I2C0_Type) SetIC_ENABLE_STATUS_IC_EN(value uint32) {
	volatile.StoreUint32(&o.IC_ENABLE_STATUS.Reg, volatile.LoadUint32(&o.IC_ENABLE_STATUS.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_ENABLE_STATUS_IC_EN() uint32 {
	return volatile.LoadUint32(&o.IC_ENABLE_STATUS.Reg) & 0x1
}

// I2C0.IC_FS_SPKLEN: I2C SS, FS or FM+ spike suppression limit This register is used to store the duration, measured in ic_clk cycles, of the longest spike that is filtered out by the spike suppression logic when the component is operating in SS, FS or FM+ modes. The relevant I2C requirement is tSP (table 4) as detailed in the I2C Bus Specification. This register must be programmed with a minimum value of 1.
func (o *I2C0_Type) SetIC_FS_SPKLEN(value uint32) {
	volatile.StoreUint32(&o.IC_FS_SPKLEN.Reg, volatile.LoadUint32(&o.IC_FS_SPKLEN.Reg)&^(0xff)|value)
}
func (o *I2C0_Type) GetIC_FS_SPKLEN() uint32 {
	return volatile.LoadUint32(&o.IC_FS_SPKLEN.Reg) & 0xff
}

// I2C0.IC_CLR_RESTART_DET: Clear RESTART_DET Interrupt Register
func (o *I2C0_Type) SetIC_CLR_RESTART_DET_CLR_RESTART_DET(value uint32) {
	volatile.StoreUint32(&o.IC_CLR_RESTART_DET.Reg, volatile.LoadUint32(&o.IC_CLR_RESTART_DET.Reg)&^(0x1)|value)
}
func (o *I2C0_Type) GetIC_CLR_RESTART_DET_CLR_RESTART_DET() uint32 {
	return volatile.LoadUint32(&o.IC_CLR_RESTART_DET.Reg) & 0x1
}

// I2C0.IC_COMP_PARAM_1: Component Parameter Register 1 Note This register is not implemented and therefore reads as 0. If it was implemented it would be a constant read-only register that contains encoded information about the component's parameter settings. Fields shown below are the settings for those parameters
func (o *I2C0_Type) SetIC_COMP_PARAM_1_TX_BUFFER_DEPTH(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_PARAM_1.Reg, volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg)&^(0xff0000)|value<<16)
}
func (o *I2C0_Type) GetIC_COMP_PARAM_1_TX_BUFFER_DEPTH() uint32 {
	return (volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg) & 0xff0000) >> 16
}
func (o *I2C0_Type) SetIC_COMP_PARAM_1_RX_BUFFER_DEPTH(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_PARAM_1.Reg, volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg)&^(0xff00)|value<<8)
}
func (o *I2C0_Type) GetIC_COMP_PARAM_1_RX_BUFFER_DEPTH() uint32 {
	return (volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg) & 0xff00) >> 8
}
func (o *I2C0_Type) SetIC_COMP_PARAM_1_ADD_ENCODED_PARAMS(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_PARAM_1.Reg, volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg)&^(0x80)|value<<7)
}
func (o *I2C0_Type) GetIC_COMP_PARAM_1_ADD_ENCODED_PARAMS() uint32 {
	return (volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg) & 0x80) >> 7
}
func (o *I2C0_Type) SetIC_COMP_PARAM_1_HAS_DMA(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_PARAM_1.Reg, volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg)&^(0x40)|value<<6)
}
func (o *I2C0_Type) GetIC_COMP_PARAM_1_HAS_DMA() uint32 {
	return (volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg) & 0x40) >> 6
}
func (o *I2C0_Type) SetIC_COMP_PARAM_1_INTR_IO(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_PARAM_1.Reg, volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg)&^(0x20)|value<<5)
}
func (o *I2C0_Type) GetIC_COMP_PARAM_1_INTR_IO() uint32 {
	return (volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg) & 0x20) >> 5
}
func (o *I2C0_Type) SetIC_COMP_PARAM_1_HC_COUNT_VALUES(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_PARAM_1.Reg, volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg)&^(0x10)|value<<4)
}
func (o *I2C0_Type) GetIC_COMP_PARAM_1_HC_COUNT_VALUES() uint32 {
	return (volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg) & 0x10) >> 4
}
func (o *I2C0_Type) SetIC_COMP_PARAM_1_MAX_SPEED_MODE(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_PARAM_1.Reg, volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg)&^(0xc)|value<<2)
}
func (o *I2C0_Type) GetIC_COMP_PARAM_1_MAX_SPEED_MODE() uint32 {
	return (volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg) & 0xc) >> 2
}
func (o *I2C0_Type) SetIC_COMP_PARAM_1_APB_DATA_WIDTH(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_PARAM_1.Reg, volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg)&^(0x3)|value)
}
func (o *I2C0_Type) GetIC_COMP_PARAM_1_APB_DATA_WIDTH() uint32 {
	return volatile.LoadUint32(&o.IC_COMP_PARAM_1.Reg) & 0x3
}

// I2C0.IC_COMP_VERSION: I2C Component Version Register
func (o *I2C0_Type) SetIC_COMP_VERSION(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_VERSION.Reg, value)
}
func (o *I2C0_Type) GetIC_COMP_VERSION() uint32 {
	return volatile.LoadUint32(&o.IC_COMP_VERSION.Reg)
}

// I2C0.IC_COMP_TYPE: I2C Component Type Register
func (o *I2C0_Type) SetIC_COMP_TYPE(value uint32) {
	volatile.StoreUint32(&o.IC_COMP_TYPE.Reg, value)
}
func (o *I2C0_Type) GetIC_COMP_TYPE() uint32 {
	return volatile.LoadUint32(&o.IC_COMP_TYPE.Reg)
}

type SPI0_Type struct {
	SSPCR0       volatile.Register32 // 0x0
	SSPCR1       volatile.Register32 // 0x4
	SSPDR        volatile.Register32 // 0x8
	SSPSR        volatile.Register32 // 0xC
	SSPCPSR      volatile.Register32 // 0x10
	SSPIMSC      volatile.Register32 // 0x14
	SSPRIS       volatile.Register32 // 0x18
	SSPMIS       volatile.Register32 // 0x1C
	SSPICR       volatile.Register32 // 0x20
	SSPDMACR     volatile.Register32 // 0x24
	_            [4024]byte
	SSPPERIPHID0 volatile.Register32 // 0xFE0
	SSPPERIPHID1 volatile.Register32 // 0xFE4
	SSPPERIPHID2 volatile.Register32 // 0xFE8
	SSPPERIPHID3 volatile.Register32 // 0xFEC
	SSPPCELLID0  volatile.Register32 // 0xFF0
	SSPPCELLID1  volatile.Register32 // 0xFF4
	SSPPCELLID2  volatile.Register32 // 0xFF8
	SSPPCELLID3  volatile.Register32 // 0xFFC
}

// SPI0.SSPCR0: Control register 0, SSPCR0 on page 3-4
func (o *SPI0_Type) SetSSPCR0_SCR(value uint32) {
	volatile.StoreUint32(&o.SSPCR0.Reg, volatile.LoadUint32(&o.SSPCR0.Reg)&^(0xff00)|value<<8)
}
func (o *SPI0_Type) GetSSPCR0_SCR() uint32 {
	return (volatile.LoadUint32(&o.SSPCR0.Reg) & 0xff00) >> 8
}
func (o *SPI0_Type) SetSSPCR0_SPH(value uint32) {
	volatile.StoreUint32(&o.SSPCR0.Reg, volatile.LoadUint32(&o.SSPCR0.Reg)&^(0x80)|value<<7)
}
func (o *SPI0_Type) GetSSPCR0_SPH() uint32 {
	return (volatile.LoadUint32(&o.SSPCR0.Reg) & 0x80) >> 7
}
func (o *SPI0_Type) SetSSPCR0_SPO(value uint32) {
	volatile.StoreUint32(&o.SSPCR0.Reg, volatile.LoadUint32(&o.SSPCR0.Reg)&^(0x40)|value<<6)
}
func (o *SPI0_Type) GetSSPCR0_SPO() uint32 {
	return (volatile.LoadUint32(&o.SSPCR0.Reg) & 0x40) >> 6
}
func (o *SPI0_Type) SetSSPCR0_FRF(value uint32) {
	volatile.StoreUint32(&o.SSPCR0.Reg, volatile.LoadUint32(&o.SSPCR0.Reg)&^(0x30)|value<<4)
}
func (o *SPI0_Type) GetSSPCR0_FRF() uint32 {
	return (volatile.LoadUint32(&o.SSPCR0.Reg) & 0x30) >> 4
}
func (o *SPI0_Type) SetSSPCR0_DSS(value uint32) {
	volatile.StoreUint32(&o.SSPCR0.Reg, volatile.LoadUint32(&o.SSPCR0.Reg)&^(0xf)|value)
}
func (o *SPI0_Type) GetSSPCR0_DSS() uint32 {
	return volatile.LoadUint32(&o.SSPCR0.Reg) & 0xf
}

// SPI0.SSPCR1: Control register 1, SSPCR1 on page 3-5
func (o *SPI0_Type) SetSSPCR1_SOD(value uint32) {
	volatile.StoreUint32(&o.SSPCR1.Reg, volatile.LoadUint32(&o.SSPCR1.Reg)&^(0x8)|value<<3)
}
func (o *SPI0_Type) GetSSPCR1_SOD() uint32 {
	return (volatile.LoadUint32(&o.SSPCR1.Reg) & 0x8) >> 3
}
func (o *SPI0_Type) SetSSPCR1_MS(value uint32) {
	volatile.StoreUint32(&o.SSPCR1.Reg, volatile.LoadUint32(&o.SSPCR1.Reg)&^(0x4)|value<<2)
}
func (o *SPI0_Type) GetSSPCR1_MS() uint32 {
	return (volatile.LoadUint32(&o.SSPCR1.Reg) & 0x4) >> 2
}
func (o *SPI0_Type) SetSSPCR1_SSE(value uint32) {
	volatile.StoreUint32(&o.SSPCR1.Reg, volatile.LoadUint32(&o.SSPCR1.Reg)&^(0x2)|value<<1)
}
func (o *SPI0_Type) GetSSPCR1_SSE() uint32 {
	return (volatile.LoadUint32(&o.SSPCR1.Reg) & 0x2) >> 1
}
func (o *SPI0_Type) SetSSPCR1_LBM(value uint32) {
	volatile.StoreUint32(&o.SSPCR1.Reg, volatile.LoadUint32(&o.SSPCR1.Reg)&^(0x1)|value)
}
func (o *SPI0_Type) GetSSPCR1_LBM() uint32 {
	return volatile.LoadUint32(&o.SSPCR1.Reg) & 0x1
}

// SPI0.SSPDR: Data register, SSPDR on page 3-6
func (o *SPI0_Type) SetSSPDR_DATA(value uint32) {
	volatile.StoreUint32(&o.SSPDR.Reg, volatile.LoadUint32(&o.SSPDR.Reg)&^(0xffff)|value)
}
func (o *SPI0_Type) GetSSPDR_DATA() uint32 {
	return volatile.LoadUint32(&o.SSPDR.Reg) & 0xffff
}

// SPI0.SSPSR: Status register, SSPSR on page 3-7
func (o *SPI0_Type) SetSSPSR_BSY(value uint32) {
	volatile.StoreUint32(&o.SSPSR.Reg, volatile.LoadUint32(&o.SSPSR.Reg)&^(0x10)|value<<4)
}
func (o *SPI0_Type) GetSSPSR_BSY() uint32 {
	return (volatile.LoadUint32(&o.SSPSR.Reg) & 0x10) >> 4
}
func (o *SPI0_Type) SetSSPSR_RFF(value uint32) {
	volatile.StoreUint32(&o.SSPSR.Reg, volatile.LoadUint32(&o.SSPSR.Reg)&^(0x8)|value<<3)
}
func (o *SPI0_Type) GetSSPSR_RFF() uint32 {
	return (volatile.LoadUint32(&o.SSPSR.Reg) & 0x8) >> 3
}
func (o *SPI0_Type) SetSSPSR_RNE(value uint32) {
	volatile.StoreUint32(&o.SSPSR.Reg, volatile.LoadUint32(&o.SSPSR.Reg)&^(0x4)|value<<2)
}
func (o *SPI0_Type) GetSSPSR_RNE() uint32 {
	return (volatile.LoadUint32(&o.SSPSR.Reg) & 0x4) >> 2
}
func (o *SPI0_Type) SetSSPSR_TNF(value uint32) {
	volatile.StoreUint32(&o.SSPSR.Reg, volatile.LoadUint32(&o.SSPSR.Reg)&^(0x2)|value<<1)
}
func (o *SPI0_Type) GetSSPSR_TNF() uint32 {
	return (volatile.LoadUint32(&o.SSPSR.Reg) & 0x2) >> 1
}
func (o *SPI0_Type) SetSSPSR_TFE(value uint32) {
	volatile.StoreUint32(&o.SSPSR.Reg, volatile.LoadUint32(&o.SSPSR.Reg)&^(0x1)|value)
}
func (o *SPI0_Type) GetSSPSR_TFE() uint32 {
	return volatile.LoadUint32(&o.SSPSR.Reg) & 0x1
}

// SPI0.SSPCPSR: Clock prescale register, SSPCPSR on page 3-8
func (o *SPI0_Type) SetSSPCPSR_CPSDVSR(value uint32) {
	volatile.StoreUint32(&o.SSPCPSR.Reg, volatile.LoadUint32(&o.SSPCPSR.Reg)&^(0xff)|value)
}
func (o *SPI0_Type) GetSSPCPSR_CPSDVSR() uint32 {
	return volatile.LoadUint32(&o.SSPCPSR.Reg) & 0xff
}

// SPI0.SSPIMSC: Interrupt mask set or clear register, SSPIMSC on page 3-9
func (o *SPI0_Type) SetSSPIMSC_TXIM(value uint32) {
	volatile.StoreUint32(&o.SSPIMSC.Reg, volatile.LoadUint32(&o.SSPIMSC.Reg)&^(0x8)|value<<3)
}
func (o *SPI0_Type) GetSSPIMSC_TXIM() uint32 {
	return (volatile.LoadUint32(&o.SSPIMSC.Reg) & 0x8) >> 3
}
func (o *SPI0_Type) SetSSPIMSC_RXIM(value uint32) {
	volatile.StoreUint32(&o.SSPIMSC.Reg, volatile.LoadUint32(&o.SSPIMSC.Reg)&^(0x4)|value<<2)
}
func (o *SPI0_Type) GetSSPIMSC_RXIM() uint32 {
	return (volatile.LoadUint32(&o.SSPIMSC.Reg) & 0x4) >> 2
}
func (o *SPI0_Type) SetSSPIMSC_RTIM(value uint32) {
	volatile.StoreUint32(&o.SSPIMSC.Reg, volatile.LoadUint32(&o.SSPIMSC.Reg)&^(0x2)|value<<1)
}
func (o *SPI0_Type) GetSSPIMSC_RTIM() uint32 {
	return (volatile.LoadUint32(&o.SSPIMSC.Reg) & 0x2) >> 1
}
func (o *SPI0_Type) SetSSPIMSC_RORIM(value uint32) {
	volatile.StoreUint32(&o.SSPIMSC.Reg, volatile.LoadUint32(&o.SSPIMSC.Reg)&^(0x1)|value)
}
func (o *SPI0_Type) GetSSPIMSC_RORIM() uint32 {
	return volatile.LoadUint32(&o.SSPIMSC.Reg) & 0x1
}

// SPI0.SSPRIS: Raw interrupt status register, SSPRIS on page 3-10
func (o *SPI0_Type) SetSSPRIS_TXRIS(value uint32) {
	volatile.StoreUint32(&o.SSPRIS.Reg, volatile.LoadUint32(&o.SSPRIS.Reg)&^(0x8)|value<<3)
}
func (o *SPI0_Type) GetSSPRIS_TXRIS() uint32 {
	return (volatile.LoadUint32(&o.SSPRIS.Reg) & 0x8) >> 3
}
func (o *SPI0_Type) SetSSPRIS_RXRIS(value uint32) {
	volatile.StoreUint32(&o.SSPRIS.Reg, volatile.LoadUint32(&o.SSPRIS.Reg)&^(0x4)|value<<2)
}
func (o *SPI0_Type) GetSSPRIS_RXRIS() uint32 {
	return (volatile.LoadUint32(&o.SSPRIS.Reg) & 0x4) >> 2
}
func (o *SPI0_Type) SetSSPRIS_RTRIS(value uint32) {
	volatile.StoreUint32(&o.SSPRIS.Reg, volatile.LoadUint32(&o.SSPRIS.Reg)&^(0x2)|value<<1)
}
func (o *SPI0_Type) GetSSPRIS_RTRIS() uint32 {
	return (volatile.LoadUint32(&o.SSPRIS.Reg) & 0x2) >> 1
}
func (o *SPI0_Type) SetSSPRIS_RORRIS(value uint32) {
	volatile.StoreUint32(&o.SSPRIS.Reg, volatile.LoadUint32(&o.SSPRIS.Reg)&^(0x1)|value)
}
func (o *SPI0_Type) GetSSPRIS_RORRIS() uint32 {
	return volatile.LoadUint32(&o.SSPRIS.Reg) & 0x1
}

// SPI0.SSPMIS: Masked interrupt status register, SSPMIS on page 3-11
func (o *SPI0_Type) SetSSPMIS_TXMIS(value uint32) {
	volatile.StoreUint32(&o.SSPMIS.Reg, volatile.LoadUint32(&o.SSPMIS.Reg)&^(0x8)|value<<3)
}
func (o *SPI0_Type) GetSSPMIS_TXMIS() uint32 {
	return (volatile.LoadUint32(&o.SSPMIS.Reg) & 0x8) >> 3
}
func (o *SPI0_Type) SetSSPMIS_RXMIS(value uint32) {
	volatile.StoreUint32(&o.SSPMIS.Reg, volatile.LoadUint32(&o.SSPMIS.Reg)&^(0x4)|value<<2)
}
func (o *SPI0_Type) GetSSPMIS_RXMIS() uint32 {
	return (volatile.LoadUint32(&o.SSPMIS.Reg) & 0x4) >> 2
}
func (o *SPI0_Type) SetSSPMIS_RTMIS(value uint32) {
	volatile.StoreUint32(&o.SSPMIS.Reg, volatile.LoadUint32(&o.SSPMIS.Reg)&^(0x2)|value<<1)
}
func (o *SPI0_Type) GetSSPMIS_RTMIS() uint32 {
	return (volatile.LoadUint32(&o.SSPMIS.Reg) & 0x2) >> 1
}
func (o *SPI0_Type) SetSSPMIS_RORMIS(value uint32) {
	volatile.StoreUint32(&o.SSPMIS.Reg, volatile.LoadUint32(&o.SSPMIS.Reg)&^(0x1)|value)
}
func (o *SPI0_Type) GetSSPMIS_RORMIS() uint32 {
	return volatile.LoadUint32(&o.SSPMIS.Reg) & 0x1
}

// SPI0.SSPICR: Interrupt clear register, SSPICR on page 3-11
func (o *SPI0_Type) SetSSPICR_RTIC(value uint32) {
	volatile.StoreUint32(&o.SSPICR.Reg, volatile.LoadUint32(&o.SSPICR.Reg)&^(0x2)|value<<1)
}
func (o *SPI0_Type) GetSSPICR_RTIC() uint32 {
	return (volatile.LoadUint32(&o.SSPICR.Reg) & 0x2) >> 1
}
func (o *SPI0_Type) SetSSPICR_RORIC(value uint32) {
	volatile.StoreUint32(&o.SSPICR.Reg, volatile.LoadUint32(&o.SSPICR.Reg)&^(0x1)|value)
}
func (o *SPI0_Type) GetSSPICR_RORIC() uint32 {
	return volatile.LoadUint32(&o.SSPICR.Reg) & 0x1
}

// SPI0.SSPDMACR: DMA control register, SSPDMACR on page 3-12
func (o *SPI0_Type) SetSSPDMACR_TXDMAE(value uint32) {
	volatile.StoreUint32(&o.SSPDMACR.Reg, volatile.LoadUint32(&o.SSPDMACR.Reg)&^(0x2)|value<<1)
}
func (o *SPI0_Type) GetSSPDMACR_TXDMAE() uint32 {
	return (volatile.LoadUint32(&o.SSPDMACR.Reg) & 0x2) >> 1
}
func (o *SPI0_Type) SetSSPDMACR_RXDMAE(value uint32) {
	volatile.StoreUint32(&o.SSPDMACR.Reg, volatile.LoadUint32(&o.SSPDMACR.Reg)&^(0x1)|value)
}
func (o *SPI0_Type) GetSSPDMACR_RXDMAE() uint32 {
	return volatile.LoadUint32(&o.SSPDMACR.Reg) & 0x1
}

// SPI0.SSPPERIPHID0: Peripheral identification registers, SSPPeriphID0-3 on page 3-13
func (o *SPI0_Type) SetSSPPERIPHID0_PARTNUMBER0(value uint32) {
	volatile.StoreUint32(&o.SSPPERIPHID0.Reg, volatile.LoadUint32(&o.SSPPERIPHID0.Reg)&^(0xff)|value)
}
func (o *SPI0_Type) GetSSPPERIPHID0_PARTNUMBER0() uint32 {
	return volatile.LoadUint32(&o.SSPPERIPHID0.Reg) & 0xff
}

// SPI0.SSPPERIPHID1: Peripheral identification registers, SSPPeriphID0-3 on page 3-13
func (o *SPI0_Type) SetSSPPERIPHID1_DESIGNER0(value uint32) {
	volatile.StoreUint32(&o.SSPPERIPHID1.Reg, volatile.LoadUint32(&o.SSPPERIPHID1.Reg)&^(0xf0)|value<<4)
}
func (o *SPI0_Type) GetSSPPERIPHID1_DESIGNER0() uint32 {
	return (volatile.LoadUint32(&o.SSPPERIPHID1.Reg) & 0xf0) >> 4
}
func (o *SPI0_Type) SetSSPPERIPHID1_PARTNUMBER1(value uint32) {
	volatile.StoreUint32(&o.SSPPERIPHID1.Reg, volatile.LoadUint32(&o.SSPPERIPHID1.Reg)&^(0xf)|value)
}
func (o *SPI0_Type) GetSSPPERIPHID1_PARTNUMBER1() uint32 {
	return volatile.LoadUint32(&o.SSPPERIPHID1.Reg) & 0xf
}

// SPI0.SSPPERIPHID2: Peripheral identification registers, SSPPeriphID0-3 on page 3-13
func (o *SPI0_Type) SetSSPPERIPHID2_REVISION(value uint32) {
	volatile.StoreUint32(&o.SSPPERIPHID2.Reg, volatile.LoadUint32(&o.SSPPERIPHID2.Reg)&^(0xf0)|value<<4)
}
func (o *SPI0_Type) GetSSPPERIPHID2_REVISION() uint32 {
	return (volatile.LoadUint32(&o.SSPPERIPHID2.Reg) & 0xf0) >> 4
}
func (o *SPI0_Type) SetSSPPERIPHID2_DESIGNER1(value uint32) {
	volatile.StoreUint32(&o.SSPPERIPHID2.Reg, volatile.LoadUint32(&o.SSPPERIPHID2.Reg)&^(0xf)|value)
}
func (o *SPI0_Type) GetSSPPERIPHID2_DESIGNER1() uint32 {
	return volatile.LoadUint32(&o.SSPPERIPHID2.Reg) & 0xf
}

// SPI0.SSPPERIPHID3: Peripheral identification registers, SSPPeriphID0-3 on page 3-13
func (o *SPI0_Type) SetSSPPERIPHID3_CONFIGURATION(value uint32) {
	volatile.StoreUint32(&o.SSPPERIPHID3.Reg, volatile.LoadUint32(&o.SSPPERIPHID3.Reg)&^(0xff)|value)
}
func (o *SPI0_Type) GetSSPPERIPHID3_CONFIGURATION() uint32 {
	return volatile.LoadUint32(&o.SSPPERIPHID3.Reg) & 0xff
}

// SPI0.SSPPCELLID0: PrimeCell identification registers, SSPPCellID0-3 on page 3-16
func (o *SPI0_Type) SetSSPPCELLID0(value uint32) {
	volatile.StoreUint32(&o.SSPPCELLID0.Reg, volatile.LoadUint32(&o.SSPPCELLID0.Reg)&^(0xff)|value)
}
func (o *SPI0_Type) GetSSPPCELLID0() uint32 {
	return volatile.LoadUint32(&o.SSPPCELLID0.Reg) & 0xff
}

// SPI0.SSPPCELLID1: PrimeCell identification registers, SSPPCellID0-3 on page 3-16
func (o *SPI0_Type) SetSSPPCELLID1(value uint32) {
	volatile.StoreUint32(&o.SSPPCELLID1.Reg, volatile.LoadUint32(&o.SSPPCELLID1.Reg)&^(0xff)|value)
}
func (o *SPI0_Type) GetSSPPCELLID1() uint32 {
	return volatile.LoadUint32(&o.SSPPCELLID1.Reg) & 0xff
}

// SPI0.SSPPCELLID2: PrimeCell identification registers, SSPPCellID0-3 on page 3-16
func (o *SPI0_Type) SetSSPPCELLID2(value uint32) {
	volatile.StoreUint32(&o.SSPPCELLID2.Reg, volatile.LoadUint32(&o.SSPPCELLID2.Reg)&^(0xff)|value)
}
func (o *SPI0_Type) GetSSPPCELLID2() uint32 {
	return volatile.LoadUint32(&o.SSPPCELLID2.Reg) & 0xff
}

// SPI0.SSPPCELLID3: PrimeCell identification registers, SSPPCellID0-3 on page 3-16
func (o *SPI0_Type) SetSSPPCELLID3(value uint32) {
	volatile.StoreUint32(&o.SSPPCELLID3.Reg, volatile.LoadUint32(&o.SSPPCELLID3.Reg)&^(0xff)|value)
}
func (o *SPI0_Type) GetSSPPCELLID3() uint32 {
	return volatile.LoadUint32(&o.SSPPCELLID3.Reg) & 0xff
}

// Programmable IO block
type PIO0_Type struct {
	CTRL              volatile.Register32 // 0x0
	FSTAT             volatile.Register32 // 0x4
	FDEBUG            volatile.Register32 // 0x8
	FLEVEL            volatile.Register32 // 0xC
	TXF0              volatile.Register32 // 0x10
	TXF1              volatile.Register32 // 0x14
	TXF2              volatile.Register32 // 0x18
	TXF3              volatile.Register32 // 0x1C
	RXF0              volatile.Register32 // 0x20
	RXF1              volatile.Register32 // 0x24
	RXF2              volatile.Register32 // 0x28
	RXF3              volatile.Register32 // 0x2C
	IRQ               volatile.Register32 // 0x30
	IRQ_FORCE         volatile.Register32 // 0x34
	INPUT_SYNC_BYPASS volatile.Register32 // 0x38
	DBG_PADOUT        volatile.Register32 // 0x3C
	DBG_PADOE         volatile.Register32 // 0x40
	DBG_CFGINFO       volatile.Register32 // 0x44
	INSTR_MEM0        volatile.Register32 // 0x48
	INSTR_MEM1        volatile.Register32 // 0x4C
	INSTR_MEM2        volatile.Register32 // 0x50
	INSTR_MEM3        volatile.Register32 // 0x54
	INSTR_MEM4        volatile.Register32 // 0x58
	INSTR_MEM5        volatile.Register32 // 0x5C
	INSTR_MEM6        volatile.Register32 // 0x60
	INSTR_MEM7        volatile.Register32 // 0x64
	INSTR_MEM8        volatile.Register32 // 0x68
	INSTR_MEM9        volatile.Register32 // 0x6C
	INSTR_MEM10       volatile.Register32 // 0x70
	INSTR_MEM11       volatile.Register32 // 0x74
	INSTR_MEM12       volatile.Register32 // 0x78
	INSTR_MEM13       volatile.Register32 // 0x7C
	INSTR_MEM14       volatile.Register32 // 0x80
	INSTR_MEM15       volatile.Register32 // 0x84
	INSTR_MEM16       volatile.Register32 // 0x88
	INSTR_MEM17       volatile.Register32 // 0x8C
	INSTR_MEM18       volatile.Register32 // 0x90
	INSTR_MEM19       volatile.Register32 // 0x94
	INSTR_MEM20       volatile.Register32 // 0x98
	INSTR_MEM21       volatile.Register32 // 0x9C
	INSTR_MEM22       volatile.Register32 // 0xA0
	INSTR_MEM23       volatile.Register32 // 0xA4
	INSTR_MEM24       volatile.Register32 // 0xA8
	INSTR_MEM25       volatile.Register32 // 0xAC
	INSTR_MEM26       volatile.Register32 // 0xB0
	INSTR_MEM27       volatile.Register32 // 0xB4
	INSTR_MEM28       volatile.Register32 // 0xB8
	INSTR_MEM29       volatile.Register32 // 0xBC
	INSTR_MEM30       volatile.Register32 // 0xC0
	INSTR_MEM31       volatile.Register32 // 0xC4
	SM0_CLKDIV        volatile.Register32 // 0xC8
	SM0_EXECCTRL      volatile.Register32 // 0xCC
	SM0_SHIFTCTRL     volatile.Register32 // 0xD0
	SM0_ADDR          volatile.Register32 // 0xD4
	SM0_INSTR         volatile.Register32 // 0xD8
	SM0_PINCTRL       volatile.Register32 // 0xDC
	SM1_CLKDIV        volatile.Register32 // 0xE0
	SM1_EXECCTRL      volatile.Register32 // 0xE4
	SM1_SHIFTCTRL     volatile.Register32 // 0xE8
	SM1_ADDR          volatile.Register32 // 0xEC
	SM1_INSTR         volatile.Register32 // 0xF0
	SM1_PINCTRL       volatile.Register32 // 0xF4
	SM2_CLKDIV        volatile.Register32 // 0xF8
	SM2_EXECCTRL      volatile.Register32 // 0xFC
	SM2_SHIFTCTRL     volatile.Register32 // 0x100
	SM2_ADDR          volatile.Register32 // 0x104
	SM2_INSTR         volatile.Register32 // 0x108
	SM2_PINCTRL       volatile.Register32 // 0x10C
	SM3_CLKDIV        volatile.Register32 // 0x110
	SM3_EXECCTRL      volatile.Register32 // 0x114
	SM3_SHIFTCTRL     volatile.Register32 // 0x118
	SM3_ADDR          volatile.Register32 // 0x11C
	SM3_INSTR         volatile.Register32 // 0x120
	SM3_PINCTRL       volatile.Register32 // 0x124
	RXF0_PUTGET0      volatile.Register32 // 0x128
	RXF0_PUTGET1      volatile.Register32 // 0x12C
	RXF0_PUTGET2      volatile.Register32 // 0x130
	RXF0_PUTGET3      volatile.Register32 // 0x134
	RXF1_PUTGET0      volatile.Register32 // 0x138
	RXF1_PUTGET1      volatile.Register32 // 0x13C
	RXF1_PUTGET2      volatile.Register32 // 0x140
	RXF1_PUTGET3      volatile.Register32 // 0x144
	RXF2_PUTGET0      volatile.Register32 // 0x148
	RXF2_PUTGET1      volatile.Register32 // 0x14C
	RXF2_PUTGET2      volatile.Register32 // 0x150
	RXF2_PUTGET3      volatile.Register32 // 0x154
	RXF3_PUTGET0      volatile.Register32 // 0x158
	RXF3_PUTGET1      volatile.Register32 // 0x15C
	RXF3_PUTGET2      volatile.Register32 // 0x160
	RXF3_PUTGET3      volatile.Register32 // 0x164
	GPIOBASE          volatile.Register32 // 0x168
	INTR              volatile.Register32 // 0x16C
	IRQ0_INTE         volatile.Register32 // 0x170
	IRQ0_INTF         volatile.Register32 // 0x174
	IRQ0_INTS         volatile.Register32 // 0x178
	IRQ1_INTE         volatile.Register32 // 0x17C
	IRQ1_INTF         volatile.Register32 // 0x180
	IRQ1_INTS         volatile.Register32 // 0x184
}

// PIO0.CTRL: PIO control register
func (o *PIO0_Type) SetCTRL_NEXTPREV_CLKDIV_RESTART(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x4000000)|value<<26)
}
func (o *PIO0_Type) GetCTRL_NEXTPREV_CLKDIV_RESTART() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x4000000) >> 26
}
func (o *PIO0_Type) SetCTRL_NEXTPREV_SM_DISABLE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x2000000)|value<<25)
}
func (o *PIO0_Type) GetCTRL_NEXTPREV_SM_DISABLE() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x2000000) >> 25
}
func (o *PIO0_Type) SetCTRL_NEXTPREV_SM_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0x1000000)|value<<24)
}
func (o *PIO0_Type) GetCTRL_NEXTPREV_SM_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0x1000000) >> 24
}
func (o *PIO0_Type) SetCTRL_NEXT_PIO_MASK(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xf00000)|value<<20)
}
func (o *PIO0_Type) GetCTRL_NEXT_PIO_MASK() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0xf00000) >> 20
}
func (o *PIO0_Type) SetCTRL_PREV_PIO_MASK(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xf0000)|value<<16)
}
func (o *PIO0_Type) GetCTRL_PREV_PIO_MASK() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0xf0000) >> 16
}
func (o *PIO0_Type) SetCTRL_CLKDIV_RESTART(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xf00)|value<<8)
}
func (o *PIO0_Type) GetCTRL_CLKDIV_RESTART() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0xf00) >> 8
}
func (o *PIO0_Type) SetCTRL_SM_RESTART(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xf0)|value<<4)
}
func (o *PIO0_Type) GetCTRL_SM_RESTART() uint32 {
	return (volatile.LoadUint32(&o.CTRL.Reg) & 0xf0) >> 4
}
func (o *PIO0_Type) SetCTRL_SM_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CTRL.Reg, volatile.LoadUint32(&o.CTRL.Reg)&^(0xf)|value)
}
func (o *PIO0_Type) GetCTRL_SM_ENABLE() uint32 {
	return volatile.LoadUint32(&o.CTRL.Reg) & 0xf
}

// PIO0.FSTAT: FIFO status register
func (o *PIO0_Type) SetFSTAT_TXEMPTY(value uint32) {
	volatile.StoreUint32(&o.FSTAT.Reg, volatile.LoadUint32(&o.FSTAT.Reg)&^(0xf000000)|value<<24)
}
func (o *PIO0_Type) GetFSTAT_TXEMPTY() uint32 {
	return (volatile.LoadUint32(&o.FSTAT.Reg) & 0xf000000) >> 24
}
func (o *PIO0_Type) SetFSTAT_TXFULL(value uint32) {
	volatile.StoreUint32(&o.FSTAT.Reg, volatile.LoadUint32(&o.FSTAT.Reg)&^(0xf0000)|value<<16)
}
func (o *PIO0_Type) GetFSTAT_TXFULL() uint32 {
	return (volatile.LoadUint32(&o.FSTAT.Reg) & 0xf0000) >> 16
}
func (o *PIO0_Type) SetFSTAT_RXEMPTY(value uint32) {
	volatile.StoreUint32(&o.FSTAT.Reg, volatile.LoadUint32(&o.FSTAT.Reg)&^(0xf00)|value<<8)
}
func (o *PIO0_Type) GetFSTAT_RXEMPTY() uint32 {
	return (volatile.LoadUint32(&o.FSTAT.Reg) & 0xf00) >> 8
}
func (o *PIO0_Type) SetFSTAT_RXFULL(value uint32) {
	volatile.StoreUint32(&o.FSTAT.Reg, volatile.LoadUint32(&o.FSTAT.Reg)&^(0xf)|value)
}
func (o *PIO0_Type) GetFSTAT_RXFULL() uint32 {
	return volatile.LoadUint32(&o.FSTAT.Reg) & 0xf
}

// PIO0.FDEBUG: FIFO debug register
func (o *PIO0_Type) SetFDEBUG_TXSTALL(value uint32) {
	volatile.StoreUint32(&o.FDEBUG.Reg, volatile.LoadUint32(&o.FDEBUG.Reg)&^(0xf000000)|value<<24)
}
func (o *PIO0_Type) GetFDEBUG_TXSTALL() uint32 {
	return (volatile.LoadUint32(&o.FDEBUG.Reg) & 0xf000000) >> 24
}
func (o *PIO0_Type) SetFDEBUG_TXOVER(value uint32) {
	volatile.StoreUint32(&o.FDEBUG.Reg, volatile.LoadUint32(&o.FDEBUG.Reg)&^(0xf0000)|value<<16)
}
func (o *PIO0_Type) GetFDEBUG_TXOVER() uint32 {
	return (volatile.LoadUint32(&o.FDEBUG.Reg) & 0xf0000) >> 16
}
func (o *PIO0_Type) SetFDEBUG_RXUNDER(value uint32) {
	volatile.StoreUint32(&o.FDEBUG.Reg, volatile.LoadUint32(&o.FDEBUG.Reg)&^(0xf00)|value<<8)
}
func (o *PIO0_Type) GetFDEBUG_RXUNDER() uint32 {
	return (volatile.LoadUint32(&o.FDEBUG.Reg) & 0xf00) >> 8
}
func (o *PIO0_Type) SetFDEBUG_RXSTALL(value uint32) {
	volatile.StoreUint32(&o.FDEBUG.Reg, volatile.LoadUint32(&o.FDEBUG.Reg)&^(0xf)|value)
}
func (o *PIO0_Type) GetFDEBUG_RXSTALL() uint32 {
	return volatile.LoadUint32(&o.FDEBUG.Reg) & 0xf
}

// PIO0.FLEVEL: FIFO levels
func (o *PIO0_Type) SetFLEVEL_RX3(value uint32) {
	volatile.StoreUint32(&o.FLEVEL.Reg, volatile.LoadUint32(&o.FLEVEL.Reg)&^(0xf0000000)|value<<28)
}
func (o *PIO0_Type) GetFLEVEL_RX3() uint32 {
	return (volatile.LoadUint32(&o.FLEVEL.Reg) & 0xf0000000) >> 28
}
func (o *PIO0_Type) SetFLEVEL_TX3(value uint32) {
	volatile.StoreUint32(&o.FLEVEL.Reg, volatile.LoadUint32(&o.FLEVEL.Reg)&^(0xf000000)|value<<24)
}
func (o *PIO0_Type) GetFLEVEL_TX3() uint32 {
	return (volatile.LoadUint32(&o.FLEVEL.Reg) & 0xf000000) >> 24
}
func (o *PIO0_Type) SetFLEVEL_RX2(value uint32) {
	volatile.StoreUint32(&o.FLEVEL.Reg, volatile.LoadUint32(&o.FLEVEL.Reg)&^(0xf00000)|value<<20)
}
func (o *PIO0_Type) GetFLEVEL_RX2() uint32 {
	return (volatile.LoadUint32(&o.FLEVEL.Reg) & 0xf00000) >> 20
}
func (o *PIO0_Type) SetFLEVEL_TX2(value uint32) {
	volatile.StoreUint32(&o.FLEVEL.Reg, volatile.LoadUint32(&o.FLEVEL.Reg)&^(0xf0000)|value<<16)
}
func (o *PIO0_Type) GetFLEVEL_TX2() uint32 {
	return (volatile.LoadUint32(&o.FLEVEL.Reg) & 0xf0000) >> 16
}
func (o *PIO0_Type) SetFLEVEL_RX1(value uint32) {
	volatile.StoreUint32(&o.FLEVEL.Reg, volatile.LoadUint32(&o.FLEVEL.Reg)&^(0xf000)|value<<12)
}
func (o *PIO0_Type) GetFLEVEL_RX1() uint32 {
	return (volatile.LoadUint32(&o.FLEVEL.Reg) & 0xf000) >> 12
}
func (o *PIO0_Type) SetFLEVEL_TX1(value uint32) {
	volatile.StoreUint32(&o.FLEVEL.Reg, volatile.LoadUint32(&o.FLEVEL.Reg)&^(0xf00)|value<<8)
}
func (o *PIO0_Type) GetFLEVEL_TX1() uint32 {
	return (volatile.LoadUint32(&o.FLEVEL.Reg) & 0xf00) >> 8
}
func (o *PIO0_Type) SetFLEVEL_RX0(value uint32) {
	volatile.StoreUint32(&o.FLEVEL.Reg, volatile.LoadUint32(&o.FLEVEL.Reg)&^(0xf0)|value<<4)
}
func (o *PIO0_Type) GetFLEVEL_RX0() uint32 {
	return (volatile.LoadUint32(&o.FLEVEL.Reg) & 0xf0) >> 4
}
func (o *PIO0_Type) SetFLEVEL_TX0(value uint32) {
	volatile.StoreUint32(&o.FLEVEL.Reg, volatile.LoadUint32(&o.FLEVEL.Reg)&^(0xf)|value)
}
func (o *PIO0_Type) GetFLEVEL_TX0() uint32 {
	return volatile.LoadUint32(&o.FLEVEL.Reg) & 0xf
}

// PIO0.TXF0: Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
func (o *PIO0_Type) SetTXF0(value uint32) {
	volatile.StoreUint32(&o.TXF0.Reg, value)
}
func (o *PIO0_Type) GetTXF0() uint32 {
	return volatile.LoadUint32(&o.TXF0.Reg)
}

// PIO0.TXF1: Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
func (o *PIO0_Type) SetTXF1(value uint32) {
	volatile.StoreUint32(&o.TXF1.Reg, value)
}
func (o *PIO0_Type) GetTXF1() uint32 {
	return volatile.LoadUint32(&o.TXF1.Reg)
}

// PIO0.TXF2: Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
func (o *PIO0_Type) SetTXF2(value uint32) {
	volatile.StoreUint32(&o.TXF2.Reg, value)
}
func (o *PIO0_Type) GetTXF2() uint32 {
	return volatile.LoadUint32(&o.TXF2.Reg)
}

// PIO0.TXF3: Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
func (o *PIO0_Type) SetTXF3(value uint32) {
	volatile.StoreUint32(&o.TXF3.Reg, value)
}
func (o *PIO0_Type) GetTXF3() uint32 {
	return volatile.LoadUint32(&o.TXF3.Reg)
}

// PIO0.RXF0: Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
func (o *PIO0_Type) SetRXF0(value uint32) {
	volatile.StoreUint32(&o.RXF0.Reg, value)
}
func (o *PIO0_Type) GetRXF0() uint32 {
	return volatile.LoadUint32(&o.RXF0.Reg)
}

// PIO0.RXF1: Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
func (o *PIO0_Type) SetRXF1(value uint32) {
	volatile.StoreUint32(&o.RXF1.Reg, value)
}
func (o *PIO0_Type) GetRXF1() uint32 {
	return volatile.LoadUint32(&o.RXF1.Reg)
}

// PIO0.RXF2: Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
func (o *PIO0_Type) SetRXF2(value uint32) {
	volatile.StoreUint32(&o.RXF2.Reg, value)
}
func (o *PIO0_Type) GetRXF2() uint32 {
	return volatile.LoadUint32(&o.RXF2.Reg)
}

// PIO0.RXF3: Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
func (o *PIO0_Type) SetRXF3(value uint32) {
	volatile.StoreUint32(&o.RXF3.Reg, value)
}
func (o *PIO0_Type) GetRXF3() uint32 {
	return volatile.LoadUint32(&o.RXF3.Reg)
}

// PIO0.IRQ: State machine IRQ flags register. Write 1 to clear. There are eight state machine IRQ flags, which can be set, cleared, and waited on by the state machines. There's no fixed association between flags and state machines -- any state machine can use any flag. Any of the eight flags can be used for timing synchronisation between state machines, using IRQ and WAIT instructions. Any combination of the eight flags can also routed out to either of the two system-level interrupt requests, alongside FIFO status interrupts -- see e.g. IRQ0_INTE.
func (o *PIO0_Type) SetIRQ(value uint32) {
	volatile.StoreUint32(&o.IRQ.Reg, volatile.LoadUint32(&o.IRQ.Reg)&^(0xff)|value)
}
func (o *PIO0_Type) GetIRQ() uint32 {
	return volatile.LoadUint32(&o.IRQ.Reg) & 0xff
}

// PIO0.IRQ_FORCE: Writing a 1 to each of these bits will forcibly assert the corresponding IRQ. Note this is different to the INTF register: writing here affects PIO internal state. INTF just asserts the processor-facing IRQ signal for testing ISRs, and is not visible to the state machines.
func (o *PIO0_Type) SetIRQ_FORCE(value uint32) {
	volatile.StoreUint32(&o.IRQ_FORCE.Reg, volatile.LoadUint32(&o.IRQ_FORCE.Reg)&^(0xff)|value)
}
func (o *PIO0_Type) GetIRQ_FORCE() uint32 {
	return volatile.LoadUint32(&o.IRQ_FORCE.Reg) & 0xff
}

// PIO0.INPUT_SYNC_BYPASS: There is a 2-flipflop synchronizer on each GPIO input, which protects PIO logic from metastabilities. This increases input delay, and for fast synchronous IO (e.g. SPI) these synchronizers may need to be bypassed. Each bit in this register corresponds to one GPIO. 0 -> input is synchronized (default) 1 -> synchronizer is bypassed If in doubt, leave this register as all zeroes.
func (o *PIO0_Type) SetINPUT_SYNC_BYPASS(value uint32) {
	volatile.StoreUint32(&o.INPUT_SYNC_BYPASS.Reg, value)
}
func (o *PIO0_Type) GetINPUT_SYNC_BYPASS() uint32 {
	return volatile.LoadUint32(&o.INPUT_SYNC_BYPASS.Reg)
}

// PIO0.DBG_PADOUT: Read to sample the pad output values PIO is currently driving to the GPIOs. On RP2040 there are 30 GPIOs, so the two most significant bits are hardwired to 0.
func (o *PIO0_Type) SetDBG_PADOUT(value uint32) {
	volatile.StoreUint32(&o.DBG_PADOUT.Reg, value)
}
func (o *PIO0_Type) GetDBG_PADOUT() uint32 {
	return volatile.LoadUint32(&o.DBG_PADOUT.Reg)
}

// PIO0.DBG_PADOE: Read to sample the pad output enables (direction) PIO is currently driving to the GPIOs. On RP2040 there are 30 GPIOs, so the two most significant bits are hardwired to 0.
func (o *PIO0_Type) SetDBG_PADOE(value uint32) {
	volatile.StoreUint32(&o.DBG_PADOE.Reg, value)
}
func (o *PIO0_Type) GetDBG_PADOE() uint32 {
	return volatile.LoadUint32(&o.DBG_PADOE.Reg)
}

// PIO0.DBG_CFGINFO: The PIO hardware has some free parameters that may vary between chip products. These should be provided in the chip datasheet, but are also exposed here.
func (o *PIO0_Type) SetDBG_CFGINFO_VERSION(value uint32) {
	volatile.StoreUint32(&o.DBG_CFGINFO.Reg, volatile.LoadUint32(&o.DBG_CFGINFO.Reg)&^(0xf0000000)|value<<28)
}
func (o *PIO0_Type) GetDBG_CFGINFO_VERSION() uint32 {
	return (volatile.LoadUint32(&o.DBG_CFGINFO.Reg) & 0xf0000000) >> 28
}
func (o *PIO0_Type) SetDBG_CFGINFO_IMEM_SIZE(value uint32) {
	volatile.StoreUint32(&o.DBG_CFGINFO.Reg, volatile.LoadUint32(&o.DBG_CFGINFO.Reg)&^(0x3f0000)|value<<16)
}
func (o *PIO0_Type) GetDBG_CFGINFO_IMEM_SIZE() uint32 {
	return (volatile.LoadUint32(&o.DBG_CFGINFO.Reg) & 0x3f0000) >> 16
}
func (o *PIO0_Type) SetDBG_CFGINFO_SM_COUNT(value uint32) {
	volatile.StoreUint32(&o.DBG_CFGINFO.Reg, volatile.LoadUint32(&o.DBG_CFGINFO.Reg)&^(0xf00)|value<<8)
}
func (o *PIO0_Type) GetDBG_CFGINFO_SM_COUNT() uint32 {
	return (volatile.LoadUint32(&o.DBG_CFGINFO.Reg) & 0xf00) >> 8
}
func (o *PIO0_Type) SetDBG_CFGINFO_FIFO_DEPTH(value uint32) {
	volatile.StoreUint32(&o.DBG_CFGINFO.Reg, volatile.LoadUint32(&o.DBG_CFGINFO.Reg)&^(0x3f)|value)
}
func (o *PIO0_Type) GetDBG_CFGINFO_FIFO_DEPTH() uint32 {
	return volatile.LoadUint32(&o.DBG_CFGINFO.Reg) & 0x3f
}

// PIO0.INSTR_MEM0: Write-only access to instruction memory location 0
func (o *PIO0_Type) SetINSTR_MEM0(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM0.Reg, volatile.LoadUint32(&o.INSTR_MEM0.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM0() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM0.Reg) & 0xffff
}

// PIO0.INSTR_MEM1: Write-only access to instruction memory location 1
func (o *PIO0_Type) SetINSTR_MEM1(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM1.Reg, volatile.LoadUint32(&o.INSTR_MEM1.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM1() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM1.Reg) & 0xffff
}

// PIO0.INSTR_MEM2: Write-only access to instruction memory location 2
func (o *PIO0_Type) SetINSTR_MEM2(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM2.Reg, volatile.LoadUint32(&o.INSTR_MEM2.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM2() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM2.Reg) & 0xffff
}

// PIO0.INSTR_MEM3: Write-only access to instruction memory location 3
func (o *PIO0_Type) SetINSTR_MEM3(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM3.Reg, volatile.LoadUint32(&o.INSTR_MEM3.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM3() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM3.Reg) & 0xffff
}

// PIO0.INSTR_MEM4: Write-only access to instruction memory location 4
func (o *PIO0_Type) SetINSTR_MEM4(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM4.Reg, volatile.LoadUint32(&o.INSTR_MEM4.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM4() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM4.Reg) & 0xffff
}

// PIO0.INSTR_MEM5: Write-only access to instruction memory location 5
func (o *PIO0_Type) SetINSTR_MEM5(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM5.Reg, volatile.LoadUint32(&o.INSTR_MEM5.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM5() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM5.Reg) & 0xffff
}

// PIO0.INSTR_MEM6: Write-only access to instruction memory location 6
func (o *PIO0_Type) SetINSTR_MEM6(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM6.Reg, volatile.LoadUint32(&o.INSTR_MEM6.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM6() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM6.Reg) & 0xffff
}

// PIO0.INSTR_MEM7: Write-only access to instruction memory location 7
func (o *PIO0_Type) SetINSTR_MEM7(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM7.Reg, volatile.LoadUint32(&o.INSTR_MEM7.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM7() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM7.Reg) & 0xffff
}

// PIO0.INSTR_MEM8: Write-only access to instruction memory location 8
func (o *PIO0_Type) SetINSTR_MEM8(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM8.Reg, volatile.LoadUint32(&o.INSTR_MEM8.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM8() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM8.Reg) & 0xffff
}

// PIO0.INSTR_MEM9: Write-only access to instruction memory location 9
func (o *PIO0_Type) SetINSTR_MEM9(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM9.Reg, volatile.LoadUint32(&o.INSTR_MEM9.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM9() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM9.Reg) & 0xffff
}

// PIO0.INSTR_MEM10: Write-only access to instruction memory location 10
func (o *PIO0_Type) SetINSTR_MEM10(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM10.Reg, volatile.LoadUint32(&o.INSTR_MEM10.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM10() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM10.Reg) & 0xffff
}

// PIO0.INSTR_MEM11: Write-only access to instruction memory location 11
func (o *PIO0_Type) SetINSTR_MEM11(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM11.Reg, volatile.LoadUint32(&o.INSTR_MEM11.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM11() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM11.Reg) & 0xffff
}

// PIO0.INSTR_MEM12: Write-only access to instruction memory location 12
func (o *PIO0_Type) SetINSTR_MEM12(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM12.Reg, volatile.LoadUint32(&o.INSTR_MEM12.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM12() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM12.Reg) & 0xffff
}

// PIO0.INSTR_MEM13: Write-only access to instruction memory location 13
func (o *PIO0_Type) SetINSTR_MEM13(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM13.Reg, volatile.LoadUint32(&o.INSTR_MEM13.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM13() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM13.Reg) & 0xffff
}

// PIO0.INSTR_MEM14: Write-only access to instruction memory location 14
func (o *PIO0_Type) SetINSTR_MEM14(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM14.Reg, volatile.LoadUint32(&o.INSTR_MEM14.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM14() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM14.Reg) & 0xffff
}

// PIO0.INSTR_MEM15: Write-only access to instruction memory location 15
func (o *PIO0_Type) SetINSTR_MEM15(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM15.Reg, volatile.LoadUint32(&o.INSTR_MEM15.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM15() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM15.Reg) & 0xffff
}

// PIO0.INSTR_MEM16: Write-only access to instruction memory location 16
func (o *PIO0_Type) SetINSTR_MEM16(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM16.Reg, volatile.LoadUint32(&o.INSTR_MEM16.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM16() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM16.Reg) & 0xffff
}

// PIO0.INSTR_MEM17: Write-only access to instruction memory location 17
func (o *PIO0_Type) SetINSTR_MEM17(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM17.Reg, volatile.LoadUint32(&o.INSTR_MEM17.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM17() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM17.Reg) & 0xffff
}

// PIO0.INSTR_MEM18: Write-only access to instruction memory location 18
func (o *PIO0_Type) SetINSTR_MEM18(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM18.Reg, volatile.LoadUint32(&o.INSTR_MEM18.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM18() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM18.Reg) & 0xffff
}

// PIO0.INSTR_MEM19: Write-only access to instruction memory location 19
func (o *PIO0_Type) SetINSTR_MEM19(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM19.Reg, volatile.LoadUint32(&o.INSTR_MEM19.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM19() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM19.Reg) & 0xffff
}

// PIO0.INSTR_MEM20: Write-only access to instruction memory location 20
func (o *PIO0_Type) SetINSTR_MEM20(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM20.Reg, volatile.LoadUint32(&o.INSTR_MEM20.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM20() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM20.Reg) & 0xffff
}

// PIO0.INSTR_MEM21: Write-only access to instruction memory location 21
func (o *PIO0_Type) SetINSTR_MEM21(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM21.Reg, volatile.LoadUint32(&o.INSTR_MEM21.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM21() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM21.Reg) & 0xffff
}

// PIO0.INSTR_MEM22: Write-only access to instruction memory location 22
func (o *PIO0_Type) SetINSTR_MEM22(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM22.Reg, volatile.LoadUint32(&o.INSTR_MEM22.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM22() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM22.Reg) & 0xffff
}

// PIO0.INSTR_MEM23: Write-only access to instruction memory location 23
func (o *PIO0_Type) SetINSTR_MEM23(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM23.Reg, volatile.LoadUint32(&o.INSTR_MEM23.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM23() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM23.Reg) & 0xffff
}

// PIO0.INSTR_MEM24: Write-only access to instruction memory location 24
func (o *PIO0_Type) SetINSTR_MEM24(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM24.Reg, volatile.LoadUint32(&o.INSTR_MEM24.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM24() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM24.Reg) & 0xffff
}

// PIO0.INSTR_MEM25: Write-only access to instruction memory location 25
func (o *PIO0_Type) SetINSTR_MEM25(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM25.Reg, volatile.LoadUint32(&o.INSTR_MEM25.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM25() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM25.Reg) & 0xffff
}

// PIO0.INSTR_MEM26: Write-only access to instruction memory location 26
func (o *PIO0_Type) SetINSTR_MEM26(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM26.Reg, volatile.LoadUint32(&o.INSTR_MEM26.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM26() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM26.Reg) & 0xffff
}

// PIO0.INSTR_MEM27: Write-only access to instruction memory location 27
func (o *PIO0_Type) SetINSTR_MEM27(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM27.Reg, volatile.LoadUint32(&o.INSTR_MEM27.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM27() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM27.Reg) & 0xffff
}

// PIO0.INSTR_MEM28: Write-only access to instruction memory location 28
func (o *PIO0_Type) SetINSTR_MEM28(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM28.Reg, volatile.LoadUint32(&o.INSTR_MEM28.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM28() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM28.Reg) & 0xffff
}

// PIO0.INSTR_MEM29: Write-only access to instruction memory location 29
func (o *PIO0_Type) SetINSTR_MEM29(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM29.Reg, volatile.LoadUint32(&o.INSTR_MEM29.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM29() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM29.Reg) & 0xffff
}

// PIO0.INSTR_MEM30: Write-only access to instruction memory location 30
func (o *PIO0_Type) SetINSTR_MEM30(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM30.Reg, volatile.LoadUint32(&o.INSTR_MEM30.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM30() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM30.Reg) & 0xffff
}

// PIO0.INSTR_MEM31: Write-only access to instruction memory location 31
func (o *PIO0_Type) SetINSTR_MEM31(value uint32) {
	volatile.StoreUint32(&o.INSTR_MEM31.Reg, volatile.LoadUint32(&o.INSTR_MEM31.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetINSTR_MEM31() uint32 {
	return volatile.LoadUint32(&o.INSTR_MEM31.Reg) & 0xffff
}

// PIO0.SM0_CLKDIV: Clock divisor register for state machine 0 Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
func (o *PIO0_Type) SetSM0_CLKDIV_INT(value uint32) {
	volatile.StoreUint32(&o.SM0_CLKDIV.Reg, volatile.LoadUint32(&o.SM0_CLKDIV.Reg)&^(0xffff0000)|value<<16)
}
func (o *PIO0_Type) GetSM0_CLKDIV_INT() uint32 {
	return (volatile.LoadUint32(&o.SM0_CLKDIV.Reg) & 0xffff0000) >> 16
}
func (o *PIO0_Type) SetSM0_CLKDIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.SM0_CLKDIV.Reg, volatile.LoadUint32(&o.SM0_CLKDIV.Reg)&^(0xff00)|value<<8)
}
func (o *PIO0_Type) GetSM0_CLKDIV_FRAC() uint32 {
	return (volatile.LoadUint32(&o.SM0_CLKDIV.Reg) & 0xff00) >> 8
}

// PIO0.SM0_EXECCTRL: Execution/behavioural settings for state machine 0
func (o *PIO0_Type) SetSM0_EXECCTRL_EXEC_STALLED(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_EXEC_STALLED() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0x80000000) >> 31
}
func (o *PIO0_Type) SetSM0_EXECCTRL_SIDE_EN(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_SIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0x40000000) >> 30
}
func (o *PIO0_Type) SetSM0_EXECCTRL_SIDE_PINDIR(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0x20000000)|value<<29)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_SIDE_PINDIR() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0x20000000) >> 29
}
func (o *PIO0_Type) SetSM0_EXECCTRL_JMP_PIN(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0x1f000000)|value<<24)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_JMP_PIN() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0x1f000000) >> 24
}
func (o *PIO0_Type) SetSM0_EXECCTRL_OUT_EN_SEL(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0xf80000)|value<<19)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_OUT_EN_SEL() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0xf80000) >> 19
}
func (o *PIO0_Type) SetSM0_EXECCTRL_INLINE_OUT_EN(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0x40000)|value<<18)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_INLINE_OUT_EN() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0x40000) >> 18
}
func (o *PIO0_Type) SetSM0_EXECCTRL_OUT_STICKY(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0x20000)|value<<17)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_OUT_STICKY() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0x20000) >> 17
}
func (o *PIO0_Type) SetSM0_EXECCTRL_WRAP_TOP(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0x1f000)|value<<12)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_WRAP_TOP() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0x1f000) >> 12
}
func (o *PIO0_Type) SetSM0_EXECCTRL_WRAP_BOTTOM(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0xf80)|value<<7)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_WRAP_BOTTOM() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0xf80) >> 7
}
func (o *PIO0_Type) SetSM0_EXECCTRL_STATUS_SEL(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0x60)|value<<5)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_STATUS_SEL() uint32 {
	return (volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0x60) >> 5
}
func (o *PIO0_Type) SetSM0_EXECCTRL_STATUS_N(value uint32) {
	volatile.StoreUint32(&o.SM0_EXECCTRL.Reg, volatile.LoadUint32(&o.SM0_EXECCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM0_EXECCTRL_STATUS_N() uint32 {
	return volatile.LoadUint32(&o.SM0_EXECCTRL.Reg) & 0x1f
}

// PIO0.SM0_SHIFTCTRL: Control behaviour of the input/output shift registers for state machine 0
func (o *PIO0_Type) SetSM0_SHIFTCTRL_FJOIN_RX(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_FJOIN_RX() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x80000000) >> 31
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_FJOIN_TX(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_FJOIN_TX() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x40000000) >> 30
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_PULL_THRESH(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x3e000000)|value<<25)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_PULL_THRESH() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x3e000000) >> 25
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_PUSH_THRESH(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x1f00000)|value<<20)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_PUSH_THRESH() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x1f00000) >> 20
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_OUT_SHIFTDIR(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x80000)|value<<19)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_OUT_SHIFTDIR() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x80000) >> 19
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_IN_SHIFTDIR(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x40000)|value<<18)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_IN_SHIFTDIR() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x40000) >> 18
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_AUTOPULL(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x20000)|value<<17)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_AUTOPULL() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x20000) >> 17
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_AUTOPUSH(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x10000)|value<<16)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_AUTOPUSH() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x10000) >> 16
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_FJOIN_RX_PUT(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_FJOIN_RX_PUT() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_FJOIN_RX_GET(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_FJOIN_RX_GET() uint32 {
	return (volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetSM0_SHIFTCTRL_IN_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM0_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM0_SHIFTCTRL_IN_COUNT() uint32 {
	return volatile.LoadUint32(&o.SM0_SHIFTCTRL.Reg) & 0x1f
}

// PIO0.SM0_ADDR: Current instruction address of state machine 0
func (o *PIO0_Type) SetSM0_ADDR(value uint32) {
	volatile.StoreUint32(&o.SM0_ADDR.Reg, volatile.LoadUint32(&o.SM0_ADDR.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM0_ADDR() uint32 {
	return volatile.LoadUint32(&o.SM0_ADDR.Reg) & 0x1f
}

// PIO0.SM0_INSTR: Read to see the instruction currently addressed by state machine 0's program counter Write to execute an instruction immediately (including jumps) and then resume execution.
func (o *PIO0_Type) SetSM0_INSTR(value uint32) {
	volatile.StoreUint32(&o.SM0_INSTR.Reg, volatile.LoadUint32(&o.SM0_INSTR.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetSM0_INSTR() uint32 {
	return volatile.LoadUint32(&o.SM0_INSTR.Reg) & 0xffff
}

// PIO0.SM0_PINCTRL: State machine pin control
func (o *PIO0_Type) SetSM0_PINCTRL_SIDESET_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM0_PINCTRL.Reg, volatile.LoadUint32(&o.SM0_PINCTRL.Reg)&^(0xe0000000)|value<<29)
}
func (o *PIO0_Type) GetSM0_PINCTRL_SIDESET_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM0_PINCTRL.Reg) & 0xe0000000) >> 29
}
func (o *PIO0_Type) SetSM0_PINCTRL_SET_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM0_PINCTRL.Reg, volatile.LoadUint32(&o.SM0_PINCTRL.Reg)&^(0x1c000000)|value<<26)
}
func (o *PIO0_Type) GetSM0_PINCTRL_SET_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM0_PINCTRL.Reg) & 0x1c000000) >> 26
}
func (o *PIO0_Type) SetSM0_PINCTRL_OUT_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM0_PINCTRL.Reg, volatile.LoadUint32(&o.SM0_PINCTRL.Reg)&^(0x3f00000)|value<<20)
}
func (o *PIO0_Type) GetSM0_PINCTRL_OUT_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM0_PINCTRL.Reg) & 0x3f00000) >> 20
}
func (o *PIO0_Type) SetSM0_PINCTRL_IN_BASE(value uint32) {
	volatile.StoreUint32(&o.SM0_PINCTRL.Reg, volatile.LoadUint32(&o.SM0_PINCTRL.Reg)&^(0xf8000)|value<<15)
}
func (o *PIO0_Type) GetSM0_PINCTRL_IN_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM0_PINCTRL.Reg) & 0xf8000) >> 15
}
func (o *PIO0_Type) SetSM0_PINCTRL_SIDESET_BASE(value uint32) {
	volatile.StoreUint32(&o.SM0_PINCTRL.Reg, volatile.LoadUint32(&o.SM0_PINCTRL.Reg)&^(0x7c00)|value<<10)
}
func (o *PIO0_Type) GetSM0_PINCTRL_SIDESET_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM0_PINCTRL.Reg) & 0x7c00) >> 10
}
func (o *PIO0_Type) SetSM0_PINCTRL_SET_BASE(value uint32) {
	volatile.StoreUint32(&o.SM0_PINCTRL.Reg, volatile.LoadUint32(&o.SM0_PINCTRL.Reg)&^(0x3e0)|value<<5)
}
func (o *PIO0_Type) GetSM0_PINCTRL_SET_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM0_PINCTRL.Reg) & 0x3e0) >> 5
}
func (o *PIO0_Type) SetSM0_PINCTRL_OUT_BASE(value uint32) {
	volatile.StoreUint32(&o.SM0_PINCTRL.Reg, volatile.LoadUint32(&o.SM0_PINCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM0_PINCTRL_OUT_BASE() uint32 {
	return volatile.LoadUint32(&o.SM0_PINCTRL.Reg) & 0x1f
}

// PIO0.SM1_CLKDIV: Clock divisor register for state machine 1 Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
func (o *PIO0_Type) SetSM1_CLKDIV_INT(value uint32) {
	volatile.StoreUint32(&o.SM1_CLKDIV.Reg, volatile.LoadUint32(&o.SM1_CLKDIV.Reg)&^(0xffff0000)|value<<16)
}
func (o *PIO0_Type) GetSM1_CLKDIV_INT() uint32 {
	return (volatile.LoadUint32(&o.SM1_CLKDIV.Reg) & 0xffff0000) >> 16
}
func (o *PIO0_Type) SetSM1_CLKDIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.SM1_CLKDIV.Reg, volatile.LoadUint32(&o.SM1_CLKDIV.Reg)&^(0xff00)|value<<8)
}
func (o *PIO0_Type) GetSM1_CLKDIV_FRAC() uint32 {
	return (volatile.LoadUint32(&o.SM1_CLKDIV.Reg) & 0xff00) >> 8
}

// PIO0.SM1_EXECCTRL: Execution/behavioural settings for state machine 1
func (o *PIO0_Type) SetSM1_EXECCTRL_EXEC_STALLED(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_EXEC_STALLED() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0x80000000) >> 31
}
func (o *PIO0_Type) SetSM1_EXECCTRL_SIDE_EN(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_SIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0x40000000) >> 30
}
func (o *PIO0_Type) SetSM1_EXECCTRL_SIDE_PINDIR(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0x20000000)|value<<29)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_SIDE_PINDIR() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0x20000000) >> 29
}
func (o *PIO0_Type) SetSM1_EXECCTRL_JMP_PIN(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0x1f000000)|value<<24)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_JMP_PIN() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0x1f000000) >> 24
}
func (o *PIO0_Type) SetSM1_EXECCTRL_OUT_EN_SEL(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0xf80000)|value<<19)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_OUT_EN_SEL() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0xf80000) >> 19
}
func (o *PIO0_Type) SetSM1_EXECCTRL_INLINE_OUT_EN(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0x40000)|value<<18)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_INLINE_OUT_EN() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0x40000) >> 18
}
func (o *PIO0_Type) SetSM1_EXECCTRL_OUT_STICKY(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0x20000)|value<<17)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_OUT_STICKY() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0x20000) >> 17
}
func (o *PIO0_Type) SetSM1_EXECCTRL_WRAP_TOP(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0x1f000)|value<<12)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_WRAP_TOP() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0x1f000) >> 12
}
func (o *PIO0_Type) SetSM1_EXECCTRL_WRAP_BOTTOM(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0xf80)|value<<7)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_WRAP_BOTTOM() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0xf80) >> 7
}
func (o *PIO0_Type) SetSM1_EXECCTRL_STATUS_SEL(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0x60)|value<<5)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_STATUS_SEL() uint32 {
	return (volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0x60) >> 5
}
func (o *PIO0_Type) SetSM1_EXECCTRL_STATUS_N(value uint32) {
	volatile.StoreUint32(&o.SM1_EXECCTRL.Reg, volatile.LoadUint32(&o.SM1_EXECCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM1_EXECCTRL_STATUS_N() uint32 {
	return volatile.LoadUint32(&o.SM1_EXECCTRL.Reg) & 0x1f
}

// PIO0.SM1_SHIFTCTRL: Control behaviour of the input/output shift registers for state machine 1
func (o *PIO0_Type) SetSM1_SHIFTCTRL_FJOIN_RX(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_FJOIN_RX() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x80000000) >> 31
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_FJOIN_TX(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_FJOIN_TX() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x40000000) >> 30
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_PULL_THRESH(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x3e000000)|value<<25)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_PULL_THRESH() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x3e000000) >> 25
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_PUSH_THRESH(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x1f00000)|value<<20)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_PUSH_THRESH() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x1f00000) >> 20
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_OUT_SHIFTDIR(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x80000)|value<<19)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_OUT_SHIFTDIR() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x80000) >> 19
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_IN_SHIFTDIR(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x40000)|value<<18)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_IN_SHIFTDIR() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x40000) >> 18
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_AUTOPULL(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x20000)|value<<17)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_AUTOPULL() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x20000) >> 17
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_AUTOPUSH(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x10000)|value<<16)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_AUTOPUSH() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x10000) >> 16
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_FJOIN_RX_PUT(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_FJOIN_RX_PUT() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_FJOIN_RX_GET(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_FJOIN_RX_GET() uint32 {
	return (volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetSM1_SHIFTCTRL_IN_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM1_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM1_SHIFTCTRL_IN_COUNT() uint32 {
	return volatile.LoadUint32(&o.SM1_SHIFTCTRL.Reg) & 0x1f
}

// PIO0.SM1_ADDR: Current instruction address of state machine 1
func (o *PIO0_Type) SetSM1_ADDR(value uint32) {
	volatile.StoreUint32(&o.SM1_ADDR.Reg, volatile.LoadUint32(&o.SM1_ADDR.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM1_ADDR() uint32 {
	return volatile.LoadUint32(&o.SM1_ADDR.Reg) & 0x1f
}

// PIO0.SM1_INSTR: Read to see the instruction currently addressed by state machine 1's program counter Write to execute an instruction immediately (including jumps) and then resume execution.
func (o *PIO0_Type) SetSM1_INSTR(value uint32) {
	volatile.StoreUint32(&o.SM1_INSTR.Reg, volatile.LoadUint32(&o.SM1_INSTR.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetSM1_INSTR() uint32 {
	return volatile.LoadUint32(&o.SM1_INSTR.Reg) & 0xffff
}

// PIO0.SM1_PINCTRL: State machine pin control
func (o *PIO0_Type) SetSM1_PINCTRL_SIDESET_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM1_PINCTRL.Reg, volatile.LoadUint32(&o.SM1_PINCTRL.Reg)&^(0xe0000000)|value<<29)
}
func (o *PIO0_Type) GetSM1_PINCTRL_SIDESET_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM1_PINCTRL.Reg) & 0xe0000000) >> 29
}
func (o *PIO0_Type) SetSM1_PINCTRL_SET_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM1_PINCTRL.Reg, volatile.LoadUint32(&o.SM1_PINCTRL.Reg)&^(0x1c000000)|value<<26)
}
func (o *PIO0_Type) GetSM1_PINCTRL_SET_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM1_PINCTRL.Reg) & 0x1c000000) >> 26
}
func (o *PIO0_Type) SetSM1_PINCTRL_OUT_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM1_PINCTRL.Reg, volatile.LoadUint32(&o.SM1_PINCTRL.Reg)&^(0x3f00000)|value<<20)
}
func (o *PIO0_Type) GetSM1_PINCTRL_OUT_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM1_PINCTRL.Reg) & 0x3f00000) >> 20
}
func (o *PIO0_Type) SetSM1_PINCTRL_IN_BASE(value uint32) {
	volatile.StoreUint32(&o.SM1_PINCTRL.Reg, volatile.LoadUint32(&o.SM1_PINCTRL.Reg)&^(0xf8000)|value<<15)
}
func (o *PIO0_Type) GetSM1_PINCTRL_IN_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM1_PINCTRL.Reg) & 0xf8000) >> 15
}
func (o *PIO0_Type) SetSM1_PINCTRL_SIDESET_BASE(value uint32) {
	volatile.StoreUint32(&o.SM1_PINCTRL.Reg, volatile.LoadUint32(&o.SM1_PINCTRL.Reg)&^(0x7c00)|value<<10)
}
func (o *PIO0_Type) GetSM1_PINCTRL_SIDESET_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM1_PINCTRL.Reg) & 0x7c00) >> 10
}
func (o *PIO0_Type) SetSM1_PINCTRL_SET_BASE(value uint32) {
	volatile.StoreUint32(&o.SM1_PINCTRL.Reg, volatile.LoadUint32(&o.SM1_PINCTRL.Reg)&^(0x3e0)|value<<5)
}
func (o *PIO0_Type) GetSM1_PINCTRL_SET_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM1_PINCTRL.Reg) & 0x3e0) >> 5
}
func (o *PIO0_Type) SetSM1_PINCTRL_OUT_BASE(value uint32) {
	volatile.StoreUint32(&o.SM1_PINCTRL.Reg, volatile.LoadUint32(&o.SM1_PINCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM1_PINCTRL_OUT_BASE() uint32 {
	return volatile.LoadUint32(&o.SM1_PINCTRL.Reg) & 0x1f
}

// PIO0.SM2_CLKDIV: Clock divisor register for state machine 2 Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
func (o *PIO0_Type) SetSM2_CLKDIV_INT(value uint32) {
	volatile.StoreUint32(&o.SM2_CLKDIV.Reg, volatile.LoadUint32(&o.SM2_CLKDIV.Reg)&^(0xffff0000)|value<<16)
}
func (o *PIO0_Type) GetSM2_CLKDIV_INT() uint32 {
	return (volatile.LoadUint32(&o.SM2_CLKDIV.Reg) & 0xffff0000) >> 16
}
func (o *PIO0_Type) SetSM2_CLKDIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.SM2_CLKDIV.Reg, volatile.LoadUint32(&o.SM2_CLKDIV.Reg)&^(0xff00)|value<<8)
}
func (o *PIO0_Type) GetSM2_CLKDIV_FRAC() uint32 {
	return (volatile.LoadUint32(&o.SM2_CLKDIV.Reg) & 0xff00) >> 8
}

// PIO0.SM2_EXECCTRL: Execution/behavioural settings for state machine 2
func (o *PIO0_Type) SetSM2_EXECCTRL_EXEC_STALLED(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_EXEC_STALLED() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0x80000000) >> 31
}
func (o *PIO0_Type) SetSM2_EXECCTRL_SIDE_EN(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_SIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0x40000000) >> 30
}
func (o *PIO0_Type) SetSM2_EXECCTRL_SIDE_PINDIR(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0x20000000)|value<<29)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_SIDE_PINDIR() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0x20000000) >> 29
}
func (o *PIO0_Type) SetSM2_EXECCTRL_JMP_PIN(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0x1f000000)|value<<24)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_JMP_PIN() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0x1f000000) >> 24
}
func (o *PIO0_Type) SetSM2_EXECCTRL_OUT_EN_SEL(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0xf80000)|value<<19)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_OUT_EN_SEL() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0xf80000) >> 19
}
func (o *PIO0_Type) SetSM2_EXECCTRL_INLINE_OUT_EN(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0x40000)|value<<18)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_INLINE_OUT_EN() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0x40000) >> 18
}
func (o *PIO0_Type) SetSM2_EXECCTRL_OUT_STICKY(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0x20000)|value<<17)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_OUT_STICKY() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0x20000) >> 17
}
func (o *PIO0_Type) SetSM2_EXECCTRL_WRAP_TOP(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0x1f000)|value<<12)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_WRAP_TOP() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0x1f000) >> 12
}
func (o *PIO0_Type) SetSM2_EXECCTRL_WRAP_BOTTOM(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0xf80)|value<<7)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_WRAP_BOTTOM() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0xf80) >> 7
}
func (o *PIO0_Type) SetSM2_EXECCTRL_STATUS_SEL(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0x60)|value<<5)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_STATUS_SEL() uint32 {
	return (volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0x60) >> 5
}
func (o *PIO0_Type) SetSM2_EXECCTRL_STATUS_N(value uint32) {
	volatile.StoreUint32(&o.SM2_EXECCTRL.Reg, volatile.LoadUint32(&o.SM2_EXECCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM2_EXECCTRL_STATUS_N() uint32 {
	return volatile.LoadUint32(&o.SM2_EXECCTRL.Reg) & 0x1f
}

// PIO0.SM2_SHIFTCTRL: Control behaviour of the input/output shift registers for state machine 2
func (o *PIO0_Type) SetSM2_SHIFTCTRL_FJOIN_RX(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_FJOIN_RX() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x80000000) >> 31
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_FJOIN_TX(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_FJOIN_TX() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x40000000) >> 30
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_PULL_THRESH(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x3e000000)|value<<25)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_PULL_THRESH() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x3e000000) >> 25
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_PUSH_THRESH(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x1f00000)|value<<20)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_PUSH_THRESH() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x1f00000) >> 20
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_OUT_SHIFTDIR(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x80000)|value<<19)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_OUT_SHIFTDIR() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x80000) >> 19
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_IN_SHIFTDIR(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x40000)|value<<18)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_IN_SHIFTDIR() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x40000) >> 18
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_AUTOPULL(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x20000)|value<<17)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_AUTOPULL() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x20000) >> 17
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_AUTOPUSH(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x10000)|value<<16)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_AUTOPUSH() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x10000) >> 16
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_FJOIN_RX_PUT(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_FJOIN_RX_PUT() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_FJOIN_RX_GET(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_FJOIN_RX_GET() uint32 {
	return (volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetSM2_SHIFTCTRL_IN_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM2_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM2_SHIFTCTRL_IN_COUNT() uint32 {
	return volatile.LoadUint32(&o.SM2_SHIFTCTRL.Reg) & 0x1f
}

// PIO0.SM2_ADDR: Current instruction address of state machine 2
func (o *PIO0_Type) SetSM2_ADDR(value uint32) {
	volatile.StoreUint32(&o.SM2_ADDR.Reg, volatile.LoadUint32(&o.SM2_ADDR.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM2_ADDR() uint32 {
	return volatile.LoadUint32(&o.SM2_ADDR.Reg) & 0x1f
}

// PIO0.SM2_INSTR: Read to see the instruction currently addressed by state machine 2's program counter Write to execute an instruction immediately (including jumps) and then resume execution.
func (o *PIO0_Type) SetSM2_INSTR(value uint32) {
	volatile.StoreUint32(&o.SM2_INSTR.Reg, volatile.LoadUint32(&o.SM2_INSTR.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetSM2_INSTR() uint32 {
	return volatile.LoadUint32(&o.SM2_INSTR.Reg) & 0xffff
}

// PIO0.SM2_PINCTRL: State machine pin control
func (o *PIO0_Type) SetSM2_PINCTRL_SIDESET_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM2_PINCTRL.Reg, volatile.LoadUint32(&o.SM2_PINCTRL.Reg)&^(0xe0000000)|value<<29)
}
func (o *PIO0_Type) GetSM2_PINCTRL_SIDESET_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM2_PINCTRL.Reg) & 0xe0000000) >> 29
}
func (o *PIO0_Type) SetSM2_PINCTRL_SET_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM2_PINCTRL.Reg, volatile.LoadUint32(&o.SM2_PINCTRL.Reg)&^(0x1c000000)|value<<26)
}
func (o *PIO0_Type) GetSM2_PINCTRL_SET_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM2_PINCTRL.Reg) & 0x1c000000) >> 26
}
func (o *PIO0_Type) SetSM2_PINCTRL_OUT_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM2_PINCTRL.Reg, volatile.LoadUint32(&o.SM2_PINCTRL.Reg)&^(0x3f00000)|value<<20)
}
func (o *PIO0_Type) GetSM2_PINCTRL_OUT_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM2_PINCTRL.Reg) & 0x3f00000) >> 20
}
func (o *PIO0_Type) SetSM2_PINCTRL_IN_BASE(value uint32) {
	volatile.StoreUint32(&o.SM2_PINCTRL.Reg, volatile.LoadUint32(&o.SM2_PINCTRL.Reg)&^(0xf8000)|value<<15)
}
func (o *PIO0_Type) GetSM2_PINCTRL_IN_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM2_PINCTRL.Reg) & 0xf8000) >> 15
}
func (o *PIO0_Type) SetSM2_PINCTRL_SIDESET_BASE(value uint32) {
	volatile.StoreUint32(&o.SM2_PINCTRL.Reg, volatile.LoadUint32(&o.SM2_PINCTRL.Reg)&^(0x7c00)|value<<10)
}
func (o *PIO0_Type) GetSM2_PINCTRL_SIDESET_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM2_PINCTRL.Reg) & 0x7c00) >> 10
}
func (o *PIO0_Type) SetSM2_PINCTRL_SET_BASE(value uint32) {
	volatile.StoreUint32(&o.SM2_PINCTRL.Reg, volatile.LoadUint32(&o.SM2_PINCTRL.Reg)&^(0x3e0)|value<<5)
}
func (o *PIO0_Type) GetSM2_PINCTRL_SET_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM2_PINCTRL.Reg) & 0x3e0) >> 5
}
func (o *PIO0_Type) SetSM2_PINCTRL_OUT_BASE(value uint32) {
	volatile.StoreUint32(&o.SM2_PINCTRL.Reg, volatile.LoadUint32(&o.SM2_PINCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM2_PINCTRL_OUT_BASE() uint32 {
	return volatile.LoadUint32(&o.SM2_PINCTRL.Reg) & 0x1f
}

// PIO0.SM3_CLKDIV: Clock divisor register for state machine 3 Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
func (o *PIO0_Type) SetSM3_CLKDIV_INT(value uint32) {
	volatile.StoreUint32(&o.SM3_CLKDIV.Reg, volatile.LoadUint32(&o.SM3_CLKDIV.Reg)&^(0xffff0000)|value<<16)
}
func (o *PIO0_Type) GetSM3_CLKDIV_INT() uint32 {
	return (volatile.LoadUint32(&o.SM3_CLKDIV.Reg) & 0xffff0000) >> 16
}
func (o *PIO0_Type) SetSM3_CLKDIV_FRAC(value uint32) {
	volatile.StoreUint32(&o.SM3_CLKDIV.Reg, volatile.LoadUint32(&o.SM3_CLKDIV.Reg)&^(0xff00)|value<<8)
}
func (o *PIO0_Type) GetSM3_CLKDIV_FRAC() uint32 {
	return (volatile.LoadUint32(&o.SM3_CLKDIV.Reg) & 0xff00) >> 8
}

// PIO0.SM3_EXECCTRL: Execution/behavioural settings for state machine 3
func (o *PIO0_Type) SetSM3_EXECCTRL_EXEC_STALLED(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_EXEC_STALLED() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0x80000000) >> 31
}
func (o *PIO0_Type) SetSM3_EXECCTRL_SIDE_EN(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_SIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0x40000000) >> 30
}
func (o *PIO0_Type) SetSM3_EXECCTRL_SIDE_PINDIR(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0x20000000)|value<<29)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_SIDE_PINDIR() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0x20000000) >> 29
}
func (o *PIO0_Type) SetSM3_EXECCTRL_JMP_PIN(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0x1f000000)|value<<24)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_JMP_PIN() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0x1f000000) >> 24
}
func (o *PIO0_Type) SetSM3_EXECCTRL_OUT_EN_SEL(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0xf80000)|value<<19)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_OUT_EN_SEL() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0xf80000) >> 19
}
func (o *PIO0_Type) SetSM3_EXECCTRL_INLINE_OUT_EN(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0x40000)|value<<18)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_INLINE_OUT_EN() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0x40000) >> 18
}
func (o *PIO0_Type) SetSM3_EXECCTRL_OUT_STICKY(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0x20000)|value<<17)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_OUT_STICKY() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0x20000) >> 17
}
func (o *PIO0_Type) SetSM3_EXECCTRL_WRAP_TOP(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0x1f000)|value<<12)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_WRAP_TOP() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0x1f000) >> 12
}
func (o *PIO0_Type) SetSM3_EXECCTRL_WRAP_BOTTOM(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0xf80)|value<<7)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_WRAP_BOTTOM() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0xf80) >> 7
}
func (o *PIO0_Type) SetSM3_EXECCTRL_STATUS_SEL(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0x60)|value<<5)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_STATUS_SEL() uint32 {
	return (volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0x60) >> 5
}
func (o *PIO0_Type) SetSM3_EXECCTRL_STATUS_N(value uint32) {
	volatile.StoreUint32(&o.SM3_EXECCTRL.Reg, volatile.LoadUint32(&o.SM3_EXECCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM3_EXECCTRL_STATUS_N() uint32 {
	return volatile.LoadUint32(&o.SM3_EXECCTRL.Reg) & 0x1f
}

// PIO0.SM3_SHIFTCTRL: Control behaviour of the input/output shift registers for state machine 3
func (o *PIO0_Type) SetSM3_SHIFTCTRL_FJOIN_RX(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_FJOIN_RX() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x80000000) >> 31
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_FJOIN_TX(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_FJOIN_TX() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x40000000) >> 30
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_PULL_THRESH(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x3e000000)|value<<25)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_PULL_THRESH() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x3e000000) >> 25
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_PUSH_THRESH(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x1f00000)|value<<20)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_PUSH_THRESH() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x1f00000) >> 20
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_OUT_SHIFTDIR(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x80000)|value<<19)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_OUT_SHIFTDIR() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x80000) >> 19
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_IN_SHIFTDIR(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x40000)|value<<18)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_IN_SHIFTDIR() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x40000) >> 18
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_AUTOPULL(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x20000)|value<<17)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_AUTOPULL() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x20000) >> 17
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_AUTOPUSH(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x10000)|value<<16)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_AUTOPUSH() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x10000) >> 16
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_FJOIN_RX_PUT(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_FJOIN_RX_PUT() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_FJOIN_RX_GET(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_FJOIN_RX_GET() uint32 {
	return (volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetSM3_SHIFTCTRL_IN_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM3_SHIFTCTRL.Reg, volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM3_SHIFTCTRL_IN_COUNT() uint32 {
	return volatile.LoadUint32(&o.SM3_SHIFTCTRL.Reg) & 0x1f
}

// PIO0.SM3_ADDR: Current instruction address of state machine 3
func (o *PIO0_Type) SetSM3_ADDR(value uint32) {
	volatile.StoreUint32(&o.SM3_ADDR.Reg, volatile.LoadUint32(&o.SM3_ADDR.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM3_ADDR() uint32 {
	return volatile.LoadUint32(&o.SM3_ADDR.Reg) & 0x1f
}

// PIO0.SM3_INSTR: Read to see the instruction currently addressed by state machine 3's program counter Write to execute an instruction immediately (including jumps) and then resume execution.
func (o *PIO0_Type) SetSM3_INSTR(value uint32) {
	volatile.StoreUint32(&o.SM3_INSTR.Reg, volatile.LoadUint32(&o.SM3_INSTR.Reg)&^(0xffff)|value)
}
func (o *PIO0_Type) GetSM3_INSTR() uint32 {
	return volatile.LoadUint32(&o.SM3_INSTR.Reg) & 0xffff
}

// PIO0.SM3_PINCTRL: State machine pin control
func (o *PIO0_Type) SetSM3_PINCTRL_SIDESET_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM3_PINCTRL.Reg, volatile.LoadUint32(&o.SM3_PINCTRL.Reg)&^(0xe0000000)|value<<29)
}
func (o *PIO0_Type) GetSM3_PINCTRL_SIDESET_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM3_PINCTRL.Reg) & 0xe0000000) >> 29
}
func (o *PIO0_Type) SetSM3_PINCTRL_SET_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM3_PINCTRL.Reg, volatile.LoadUint32(&o.SM3_PINCTRL.Reg)&^(0x1c000000)|value<<26)
}
func (o *PIO0_Type) GetSM3_PINCTRL_SET_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM3_PINCTRL.Reg) & 0x1c000000) >> 26
}
func (o *PIO0_Type) SetSM3_PINCTRL_OUT_COUNT(value uint32) {
	volatile.StoreUint32(&o.SM3_PINCTRL.Reg, volatile.LoadUint32(&o.SM3_PINCTRL.Reg)&^(0x3f00000)|value<<20)
}
func (o *PIO0_Type) GetSM3_PINCTRL_OUT_COUNT() uint32 {
	return (volatile.LoadUint32(&o.SM3_PINCTRL.Reg) & 0x3f00000) >> 20
}
func (o *PIO0_Type) SetSM3_PINCTRL_IN_BASE(value uint32) {
	volatile.StoreUint32(&o.SM3_PINCTRL.Reg, volatile.LoadUint32(&o.SM3_PINCTRL.Reg)&^(0xf8000)|value<<15)
}
func (o *PIO0_Type) GetSM3_PINCTRL_IN_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM3_PINCTRL.Reg) & 0xf8000) >> 15
}
func (o *PIO0_Type) SetSM3_PINCTRL_SIDESET_BASE(value uint32) {
	volatile.StoreUint32(&o.SM3_PINCTRL.Reg, volatile.LoadUint32(&o.SM3_PINCTRL.Reg)&^(0x7c00)|value<<10)
}
func (o *PIO0_Type) GetSM3_PINCTRL_SIDESET_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM3_PINCTRL.Reg) & 0x7c00) >> 10
}
func (o *PIO0_Type) SetSM3_PINCTRL_SET_BASE(value uint32) {
	volatile.StoreUint32(&o.SM3_PINCTRL.Reg, volatile.LoadUint32(&o.SM3_PINCTRL.Reg)&^(0x3e0)|value<<5)
}
func (o *PIO0_Type) GetSM3_PINCTRL_SET_BASE() uint32 {
	return (volatile.LoadUint32(&o.SM3_PINCTRL.Reg) & 0x3e0) >> 5
}
func (o *PIO0_Type) SetSM3_PINCTRL_OUT_BASE(value uint32) {
	volatile.StoreUint32(&o.SM3_PINCTRL.Reg, volatile.LoadUint32(&o.SM3_PINCTRL.Reg)&^(0x1f)|value)
}
func (o *PIO0_Type) GetSM3_PINCTRL_OUT_BASE() uint32 {
	return volatile.LoadUint32(&o.SM3_PINCTRL.Reg) & 0x1f
}

// PIO0.RXF0_PUTGET0: Direct read/write access to entry 0 of SM0's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF0_PUTGET0(value uint32) {
	volatile.StoreUint32(&o.RXF0_PUTGET0.Reg, value)
}
func (o *PIO0_Type) GetRXF0_PUTGET0() uint32 {
	return volatile.LoadUint32(&o.RXF0_PUTGET0.Reg)
}

// PIO0.RXF0_PUTGET1: Direct read/write access to entry 1 of SM0's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF0_PUTGET1(value uint32) {
	volatile.StoreUint32(&o.RXF0_PUTGET1.Reg, value)
}
func (o *PIO0_Type) GetRXF0_PUTGET1() uint32 {
	return volatile.LoadUint32(&o.RXF0_PUTGET1.Reg)
}

// PIO0.RXF0_PUTGET2: Direct read/write access to entry 2 of SM0's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF0_PUTGET2(value uint32) {
	volatile.StoreUint32(&o.RXF0_PUTGET2.Reg, value)
}
func (o *PIO0_Type) GetRXF0_PUTGET2() uint32 {
	return volatile.LoadUint32(&o.RXF0_PUTGET2.Reg)
}

// PIO0.RXF0_PUTGET3: Direct read/write access to entry 3 of SM0's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF0_PUTGET3(value uint32) {
	volatile.StoreUint32(&o.RXF0_PUTGET3.Reg, value)
}
func (o *PIO0_Type) GetRXF0_PUTGET3() uint32 {
	return volatile.LoadUint32(&o.RXF0_PUTGET3.Reg)
}

// PIO0.RXF1_PUTGET0: Direct read/write access to entry 0 of SM1's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF1_PUTGET0(value uint32) {
	volatile.StoreUint32(&o.RXF1_PUTGET0.Reg, value)
}
func (o *PIO0_Type) GetRXF1_PUTGET0() uint32 {
	return volatile.LoadUint32(&o.RXF1_PUTGET0.Reg)
}

// PIO0.RXF1_PUTGET1: Direct read/write access to entry 1 of SM1's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF1_PUTGET1(value uint32) {
	volatile.StoreUint32(&o.RXF1_PUTGET1.Reg, value)
}
func (o *PIO0_Type) GetRXF1_PUTGET1() uint32 {
	return volatile.LoadUint32(&o.RXF1_PUTGET1.Reg)
}

// PIO0.RXF1_PUTGET2: Direct read/write access to entry 2 of SM1's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF1_PUTGET2(value uint32) {
	volatile.StoreUint32(&o.RXF1_PUTGET2.Reg, value)
}
func (o *PIO0_Type) GetRXF1_PUTGET2() uint32 {
	return volatile.LoadUint32(&o.RXF1_PUTGET2.Reg)
}

// PIO0.RXF1_PUTGET3: Direct read/write access to entry 3 of SM1's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF1_PUTGET3(value uint32) {
	volatile.StoreUint32(&o.RXF1_PUTGET3.Reg, value)
}
func (o *PIO0_Type) GetRXF1_PUTGET3() uint32 {
	return volatile.LoadUint32(&o.RXF1_PUTGET3.Reg)
}

// PIO0.RXF2_PUTGET0: Direct read/write access to entry 0 of SM2's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF2_PUTGET0(value uint32) {
	volatile.StoreUint32(&o.RXF2_PUTGET0.Reg, value)
}
func (o *PIO0_Type) GetRXF2_PUTGET0() uint32 {
	return volatile.LoadUint32(&o.RXF2_PUTGET0.Reg)
}

// PIO0.RXF2_PUTGET1: Direct read/write access to entry 1 of SM2's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF2_PUTGET1(value uint32) {
	volatile.StoreUint32(&o.RXF2_PUTGET1.Reg, value)
}
func (o *PIO0_Type) GetRXF2_PUTGET1() uint32 {
	return volatile.LoadUint32(&o.RXF2_PUTGET1.Reg)
}

// PIO0.RXF2_PUTGET2: Direct read/write access to entry 2 of SM2's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF2_PUTGET2(value uint32) {
	volatile.StoreUint32(&o.RXF2_PUTGET2.Reg, value)
}
func (o *PIO0_Type) GetRXF2_PUTGET2() uint32 {
	return volatile.LoadUint32(&o.RXF2_PUTGET2.Reg)
}

// PIO0.RXF2_PUTGET3: Direct read/write access to entry 3 of SM2's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF2_PUTGET3(value uint32) {
	volatile.StoreUint32(&o.RXF2_PUTGET3.Reg, value)
}
func (o *PIO0_Type) GetRXF2_PUTGET3() uint32 {
	return volatile.LoadUint32(&o.RXF2_PUTGET3.Reg)
}

// PIO0.RXF3_PUTGET0: Direct read/write access to entry 0 of SM3's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF3_PUTGET0(value uint32) {
	volatile.StoreUint32(&o.RXF3_PUTGET0.Reg, value)
}
func (o *PIO0_Type) GetRXF3_PUTGET0() uint32 {
	return volatile.LoadUint32(&o.RXF3_PUTGET0.Reg)
}

// PIO0.RXF3_PUTGET1: Direct read/write access to entry 1 of SM3's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF3_PUTGET1(value uint32) {
	volatile.StoreUint32(&o.RXF3_PUTGET1.Reg, value)
}
func (o *PIO0_Type) GetRXF3_PUTGET1() uint32 {
	return volatile.LoadUint32(&o.RXF3_PUTGET1.Reg)
}

// PIO0.RXF3_PUTGET2: Direct read/write access to entry 2 of SM3's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF3_PUTGET2(value uint32) {
	volatile.StoreUint32(&o.RXF3_PUTGET2.Reg, value)
}
func (o *PIO0_Type) GetRXF3_PUTGET2() uint32 {
	return volatile.LoadUint32(&o.RXF3_PUTGET2.Reg)
}

// PIO0.RXF3_PUTGET3: Direct read/write access to entry 3 of SM3's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
func (o *PIO0_Type) SetRXF3_PUTGET3(value uint32) {
	volatile.StoreUint32(&o.RXF3_PUTGET3.Reg, value)
}
func (o *PIO0_Type) GetRXF3_PUTGET3() uint32 {
	return volatile.LoadUint32(&o.RXF3_PUTGET3.Reg)
}

// PIO0.GPIOBASE: Relocate GPIO 0 (from PIO's point of view) in the system GPIO numbering, to access more than 32 GPIOs from PIO. Only the values 0 and 16 are supported (only bit 4 is writable).
func (o *PIO0_Type) SetGPIOBASE(value uint32) {
	volatile.StoreUint32(&o.GPIOBASE.Reg, volatile.LoadUint32(&o.GPIOBASE.Reg)&^(0x10)|value<<4)
}
func (o *PIO0_Type) GetGPIOBASE() uint32 {
	return (volatile.LoadUint32(&o.GPIOBASE.Reg) & 0x10) >> 4
}

// PIO0.INTR: Raw Interrupts
func (o *PIO0_Type) SetINTR_SM7(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetINTR_SM7() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetINTR_SM6(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetINTR_SM6() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetINTR_SM5(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2000)|value<<13)
}
func (o *PIO0_Type) GetINTR_SM5() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2000) >> 13
}
func (o *PIO0_Type) SetINTR_SM4(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1000)|value<<12)
}
func (o *PIO0_Type) GetINTR_SM4() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x1000) >> 12
}
func (o *PIO0_Type) SetINTR_SM3(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x800)|value<<11)
}
func (o *PIO0_Type) GetINTR_SM3() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x800) >> 11
}
func (o *PIO0_Type) SetINTR_SM2(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x400)|value<<10)
}
func (o *PIO0_Type) GetINTR_SM2() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x400) >> 10
}
func (o *PIO0_Type) SetINTR_SM1(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x200)|value<<9)
}
func (o *PIO0_Type) GetINTR_SM1() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x200) >> 9
}
func (o *PIO0_Type) SetINTR_SM0(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x100)|value<<8)
}
func (o *PIO0_Type) GetINTR_SM0() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x100) >> 8
}
func (o *PIO0_Type) SetINTR_SM3_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x80)|value<<7)
}
func (o *PIO0_Type) GetINTR_SM3_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x80) >> 7
}
func (o *PIO0_Type) SetINTR_SM2_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x40)|value<<6)
}
func (o *PIO0_Type) GetINTR_SM2_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x40) >> 6
}
func (o *PIO0_Type) SetINTR_SM1_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x20)|value<<5)
}
func (o *PIO0_Type) GetINTR_SM1_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x20) >> 5
}
func (o *PIO0_Type) SetINTR_SM0_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x10)|value<<4)
}
func (o *PIO0_Type) GetINTR_SM0_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x10) >> 4
}
func (o *PIO0_Type) SetINTR_SM3_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8)|value<<3)
}
func (o *PIO0_Type) GetINTR_SM3_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8) >> 3
}
func (o *PIO0_Type) SetINTR_SM2_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4)|value<<2)
}
func (o *PIO0_Type) GetINTR_SM2_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4) >> 2
}
func (o *PIO0_Type) SetINTR_SM1_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2)|value<<1)
}
func (o *PIO0_Type) GetINTR_SM1_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2) >> 1
}
func (o *PIO0_Type) SetINTR_SM0_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *PIO0_Type) GetINTR_SM0_RXNEMPTY() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// PIO0.IRQ0_INTE: Interrupt Enable for irq0
func (o *PIO0_Type) SetIRQ0_INTE_SM7(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM7() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetIRQ0_INTE_SM6(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM6() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetIRQ0_INTE_SM5(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x2000)|value<<13)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM5() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x2000) >> 13
}
func (o *PIO0_Type) SetIRQ0_INTE_SM4(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x1000)|value<<12)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM4() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x1000) >> 12
}
func (o *PIO0_Type) SetIRQ0_INTE_SM3(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x800)|value<<11)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM3() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x800) >> 11
}
func (o *PIO0_Type) SetIRQ0_INTE_SM2(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x400)|value<<10)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM2() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x400) >> 10
}
func (o *PIO0_Type) SetIRQ0_INTE_SM1(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x200)|value<<9)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM1() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x200) >> 9
}
func (o *PIO0_Type) SetIRQ0_INTE_SM0(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x100)|value<<8)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM0() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x100) >> 8
}
func (o *PIO0_Type) SetIRQ0_INTE_SM3_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x80)|value<<7)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM3_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x80) >> 7
}
func (o *PIO0_Type) SetIRQ0_INTE_SM2_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x40)|value<<6)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM2_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x40) >> 6
}
func (o *PIO0_Type) SetIRQ0_INTE_SM1_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x20)|value<<5)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM1_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x20) >> 5
}
func (o *PIO0_Type) SetIRQ0_INTE_SM0_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x10)|value<<4)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM0_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x10) >> 4
}
func (o *PIO0_Type) SetIRQ0_INTE_SM3_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x8)|value<<3)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM3_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x8) >> 3
}
func (o *PIO0_Type) SetIRQ0_INTE_SM2_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x4)|value<<2)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM2_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x4) >> 2
}
func (o *PIO0_Type) SetIRQ0_INTE_SM1_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x2)|value<<1)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM1_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x2) >> 1
}
func (o *PIO0_Type) SetIRQ0_INTE_SM0_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTE.Reg, volatile.LoadUint32(&o.IRQ0_INTE.Reg)&^(0x1)|value)
}
func (o *PIO0_Type) GetIRQ0_INTE_SM0_RXNEMPTY() uint32 {
	return volatile.LoadUint32(&o.IRQ0_INTE.Reg) & 0x1
}

// PIO0.IRQ0_INTF: Interrupt Force for irq0
func (o *PIO0_Type) SetIRQ0_INTF_SM7(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM7() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetIRQ0_INTF_SM6(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM6() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetIRQ0_INTF_SM5(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x2000)|value<<13)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM5() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x2000) >> 13
}
func (o *PIO0_Type) SetIRQ0_INTF_SM4(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x1000)|value<<12)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM4() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x1000) >> 12
}
func (o *PIO0_Type) SetIRQ0_INTF_SM3(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x800)|value<<11)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM3() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x800) >> 11
}
func (o *PIO0_Type) SetIRQ0_INTF_SM2(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x400)|value<<10)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM2() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x400) >> 10
}
func (o *PIO0_Type) SetIRQ0_INTF_SM1(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x200)|value<<9)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM1() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x200) >> 9
}
func (o *PIO0_Type) SetIRQ0_INTF_SM0(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x100)|value<<8)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM0() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x100) >> 8
}
func (o *PIO0_Type) SetIRQ0_INTF_SM3_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x80)|value<<7)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM3_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x80) >> 7
}
func (o *PIO0_Type) SetIRQ0_INTF_SM2_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x40)|value<<6)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM2_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x40) >> 6
}
func (o *PIO0_Type) SetIRQ0_INTF_SM1_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x20)|value<<5)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM1_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x20) >> 5
}
func (o *PIO0_Type) SetIRQ0_INTF_SM0_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x10)|value<<4)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM0_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x10) >> 4
}
func (o *PIO0_Type) SetIRQ0_INTF_SM3_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x8)|value<<3)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM3_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x8) >> 3
}
func (o *PIO0_Type) SetIRQ0_INTF_SM2_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x4)|value<<2)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM2_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x4) >> 2
}
func (o *PIO0_Type) SetIRQ0_INTF_SM1_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x2)|value<<1)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM1_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x2) >> 1
}
func (o *PIO0_Type) SetIRQ0_INTF_SM0_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTF.Reg, volatile.LoadUint32(&o.IRQ0_INTF.Reg)&^(0x1)|value)
}
func (o *PIO0_Type) GetIRQ0_INTF_SM0_RXNEMPTY() uint32 {
	return volatile.LoadUint32(&o.IRQ0_INTF.Reg) & 0x1
}

// PIO0.IRQ0_INTS: Interrupt status after masking & forcing for irq0
func (o *PIO0_Type) SetIRQ0_INTS_SM7(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM7() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetIRQ0_INTS_SM6(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM6() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetIRQ0_INTS_SM5(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x2000)|value<<13)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM5() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x2000) >> 13
}
func (o *PIO0_Type) SetIRQ0_INTS_SM4(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x1000)|value<<12)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM4() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x1000) >> 12
}
func (o *PIO0_Type) SetIRQ0_INTS_SM3(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x800)|value<<11)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM3() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x800) >> 11
}
func (o *PIO0_Type) SetIRQ0_INTS_SM2(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x400)|value<<10)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM2() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x400) >> 10
}
func (o *PIO0_Type) SetIRQ0_INTS_SM1(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x200)|value<<9)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM1() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x200) >> 9
}
func (o *PIO0_Type) SetIRQ0_INTS_SM0(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x100)|value<<8)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM0() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x100) >> 8
}
func (o *PIO0_Type) SetIRQ0_INTS_SM3_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x80)|value<<7)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM3_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x80) >> 7
}
func (o *PIO0_Type) SetIRQ0_INTS_SM2_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x40)|value<<6)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM2_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x40) >> 6
}
func (o *PIO0_Type) SetIRQ0_INTS_SM1_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x20)|value<<5)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM1_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x20) >> 5
}
func (o *PIO0_Type) SetIRQ0_INTS_SM0_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x10)|value<<4)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM0_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x10) >> 4
}
func (o *PIO0_Type) SetIRQ0_INTS_SM3_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x8)|value<<3)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM3_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x8) >> 3
}
func (o *PIO0_Type) SetIRQ0_INTS_SM2_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x4)|value<<2)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM2_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x4) >> 2
}
func (o *PIO0_Type) SetIRQ0_INTS_SM1_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x2)|value<<1)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM1_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x2) >> 1
}
func (o *PIO0_Type) SetIRQ0_INTS_SM0_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ0_INTS.Reg, volatile.LoadUint32(&o.IRQ0_INTS.Reg)&^(0x1)|value)
}
func (o *PIO0_Type) GetIRQ0_INTS_SM0_RXNEMPTY() uint32 {
	return volatile.LoadUint32(&o.IRQ0_INTS.Reg) & 0x1
}

// PIO0.IRQ1_INTE: Interrupt Enable for irq1
func (o *PIO0_Type) SetIRQ1_INTE_SM7(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM7() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetIRQ1_INTE_SM6(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM6() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetIRQ1_INTE_SM5(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x2000)|value<<13)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM5() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x2000) >> 13
}
func (o *PIO0_Type) SetIRQ1_INTE_SM4(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x1000)|value<<12)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM4() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x1000) >> 12
}
func (o *PIO0_Type) SetIRQ1_INTE_SM3(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x800)|value<<11)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM3() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x800) >> 11
}
func (o *PIO0_Type) SetIRQ1_INTE_SM2(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x400)|value<<10)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM2() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x400) >> 10
}
func (o *PIO0_Type) SetIRQ1_INTE_SM1(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x200)|value<<9)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM1() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x200) >> 9
}
func (o *PIO0_Type) SetIRQ1_INTE_SM0(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x100)|value<<8)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM0() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x100) >> 8
}
func (o *PIO0_Type) SetIRQ1_INTE_SM3_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x80)|value<<7)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM3_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x80) >> 7
}
func (o *PIO0_Type) SetIRQ1_INTE_SM2_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x40)|value<<6)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM2_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x40) >> 6
}
func (o *PIO0_Type) SetIRQ1_INTE_SM1_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x20)|value<<5)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM1_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x20) >> 5
}
func (o *PIO0_Type) SetIRQ1_INTE_SM0_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x10)|value<<4)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM0_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x10) >> 4
}
func (o *PIO0_Type) SetIRQ1_INTE_SM3_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x8)|value<<3)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM3_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x8) >> 3
}
func (o *PIO0_Type) SetIRQ1_INTE_SM2_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x4)|value<<2)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM2_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x4) >> 2
}
func (o *PIO0_Type) SetIRQ1_INTE_SM1_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x2)|value<<1)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM1_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x2) >> 1
}
func (o *PIO0_Type) SetIRQ1_INTE_SM0_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTE.Reg, volatile.LoadUint32(&o.IRQ1_INTE.Reg)&^(0x1)|value)
}
func (o *PIO0_Type) GetIRQ1_INTE_SM0_RXNEMPTY() uint32 {
	return volatile.LoadUint32(&o.IRQ1_INTE.Reg) & 0x1
}

// PIO0.IRQ1_INTF: Interrupt Force for irq1
func (o *PIO0_Type) SetIRQ1_INTF_SM7(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM7() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetIRQ1_INTF_SM6(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM6() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetIRQ1_INTF_SM5(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x2000)|value<<13)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM5() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x2000) >> 13
}
func (o *PIO0_Type) SetIRQ1_INTF_SM4(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x1000)|value<<12)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM4() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x1000) >> 12
}
func (o *PIO0_Type) SetIRQ1_INTF_SM3(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x800)|value<<11)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM3() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x800) >> 11
}
func (o *PIO0_Type) SetIRQ1_INTF_SM2(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x400)|value<<10)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM2() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x400) >> 10
}
func (o *PIO0_Type) SetIRQ1_INTF_SM1(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x200)|value<<9)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM1() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x200) >> 9
}
func (o *PIO0_Type) SetIRQ1_INTF_SM0(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x100)|value<<8)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM0() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x100) >> 8
}
func (o *PIO0_Type) SetIRQ1_INTF_SM3_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x80)|value<<7)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM3_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x80) >> 7
}
func (o *PIO0_Type) SetIRQ1_INTF_SM2_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x40)|value<<6)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM2_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x40) >> 6
}
func (o *PIO0_Type) SetIRQ1_INTF_SM1_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x20)|value<<5)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM1_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x20) >> 5
}
func (o *PIO0_Type) SetIRQ1_INTF_SM0_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x10)|value<<4)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM0_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x10) >> 4
}
func (o *PIO0_Type) SetIRQ1_INTF_SM3_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x8)|value<<3)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM3_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x8) >> 3
}
func (o *PIO0_Type) SetIRQ1_INTF_SM2_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x4)|value<<2)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM2_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x4) >> 2
}
func (o *PIO0_Type) SetIRQ1_INTF_SM1_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x2)|value<<1)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM1_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x2) >> 1
}
func (o *PIO0_Type) SetIRQ1_INTF_SM0_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTF.Reg, volatile.LoadUint32(&o.IRQ1_INTF.Reg)&^(0x1)|value)
}
func (o *PIO0_Type) GetIRQ1_INTF_SM0_RXNEMPTY() uint32 {
	return volatile.LoadUint32(&o.IRQ1_INTF.Reg) & 0x1
}

// PIO0.IRQ1_INTS: Interrupt status after masking & forcing for irq1
func (o *PIO0_Type) SetIRQ1_INTS_SM7(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x8000)|value<<15)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM7() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x8000) >> 15
}
func (o *PIO0_Type) SetIRQ1_INTS_SM6(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x4000)|value<<14)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM6() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x4000) >> 14
}
func (o *PIO0_Type) SetIRQ1_INTS_SM5(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x2000)|value<<13)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM5() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x2000) >> 13
}
func (o *PIO0_Type) SetIRQ1_INTS_SM4(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x1000)|value<<12)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM4() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x1000) >> 12
}
func (o *PIO0_Type) SetIRQ1_INTS_SM3(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x800)|value<<11)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM3() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x800) >> 11
}
func (o *PIO0_Type) SetIRQ1_INTS_SM2(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x400)|value<<10)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM2() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x400) >> 10
}
func (o *PIO0_Type) SetIRQ1_INTS_SM1(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x200)|value<<9)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM1() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x200) >> 9
}
func (o *PIO0_Type) SetIRQ1_INTS_SM0(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x100)|value<<8)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM0() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x100) >> 8
}
func (o *PIO0_Type) SetIRQ1_INTS_SM3_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x80)|value<<7)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM3_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x80) >> 7
}
func (o *PIO0_Type) SetIRQ1_INTS_SM2_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x40)|value<<6)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM2_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x40) >> 6
}
func (o *PIO0_Type) SetIRQ1_INTS_SM1_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x20)|value<<5)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM1_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x20) >> 5
}
func (o *PIO0_Type) SetIRQ1_INTS_SM0_TXNFULL(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x10)|value<<4)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM0_TXNFULL() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x10) >> 4
}
func (o *PIO0_Type) SetIRQ1_INTS_SM3_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x8)|value<<3)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM3_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x8) >> 3
}
func (o *PIO0_Type) SetIRQ1_INTS_SM2_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x4)|value<<2)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM2_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x4) >> 2
}
func (o *PIO0_Type) SetIRQ1_INTS_SM1_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x2)|value<<1)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM1_RXNEMPTY() uint32 {
	return (volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x2) >> 1
}
func (o *PIO0_Type) SetIRQ1_INTS_SM0_RXNEMPTY(value uint32) {
	volatile.StoreUint32(&o.IRQ1_INTS.Reg, volatile.LoadUint32(&o.IRQ1_INTS.Reg)&^(0x1)|value)
}
func (o *PIO0_Type) GetIRQ1_INTS_SM0_RXNEMPTY() uint32 {
	return volatile.LoadUint32(&o.IRQ1_INTS.Reg) & 0x1
}

// Register block for busfabric control signals and performance counters
type BUSCTRL_Type struct {
	BUS_PRIORITY     volatile.Register32 // 0x0
	BUS_PRIORITY_ACK volatile.Register32 // 0x4
	PERFCTR_EN       volatile.Register32 // 0x8
	PERFCTR0         volatile.Register32 // 0xC
	PERFSEL0         volatile.Register32 // 0x10
	PERFCTR1         volatile.Register32 // 0x14
	PERFSEL1         volatile.Register32 // 0x18
	PERFCTR2         volatile.Register32 // 0x1C
	PERFSEL2         volatile.Register32 // 0x20
	PERFCTR3         volatile.Register32 // 0x24
	PERFSEL3         volatile.Register32 // 0x28
}

// BUSCTRL.BUS_PRIORITY: Set the priority of each master for bus arbitration.
func (o *BUSCTRL_Type) SetBUS_PRIORITY_DMA_W(value uint32) {
	volatile.StoreUint32(&o.BUS_PRIORITY.Reg, volatile.LoadUint32(&o.BUS_PRIORITY.Reg)&^(0x1000)|value<<12)
}
func (o *BUSCTRL_Type) GetBUS_PRIORITY_DMA_W() uint32 {
	return (volatile.LoadUint32(&o.BUS_PRIORITY.Reg) & 0x1000) >> 12
}
func (o *BUSCTRL_Type) SetBUS_PRIORITY_DMA_R(value uint32) {
	volatile.StoreUint32(&o.BUS_PRIORITY.Reg, volatile.LoadUint32(&o.BUS_PRIORITY.Reg)&^(0x100)|value<<8)
}
func (o *BUSCTRL_Type) GetBUS_PRIORITY_DMA_R() uint32 {
	return (volatile.LoadUint32(&o.BUS_PRIORITY.Reg) & 0x100) >> 8
}
func (o *BUSCTRL_Type) SetBUS_PRIORITY_PROC1(value uint32) {
	volatile.StoreUint32(&o.BUS_PRIORITY.Reg, volatile.LoadUint32(&o.BUS_PRIORITY.Reg)&^(0x10)|value<<4)
}
func (o *BUSCTRL_Type) GetBUS_PRIORITY_PROC1() uint32 {
	return (volatile.LoadUint32(&o.BUS_PRIORITY.Reg) & 0x10) >> 4
}
func (o *BUSCTRL_Type) SetBUS_PRIORITY_PROC0(value uint32) {
	volatile.StoreUint32(&o.BUS_PRIORITY.Reg, volatile.LoadUint32(&o.BUS_PRIORITY.Reg)&^(0x1)|value)
}
func (o *BUSCTRL_Type) GetBUS_PRIORITY_PROC0() uint32 {
	return volatile.LoadUint32(&o.BUS_PRIORITY.Reg) & 0x1
}

// BUSCTRL.BUS_PRIORITY_ACK: Bus priority acknowledge
func (o *BUSCTRL_Type) SetBUS_PRIORITY_ACK(value uint32) {
	volatile.StoreUint32(&o.BUS_PRIORITY_ACK.Reg, volatile.LoadUint32(&o.BUS_PRIORITY_ACK.Reg)&^(0x1)|value)
}
func (o *BUSCTRL_Type) GetBUS_PRIORITY_ACK() uint32 {
	return volatile.LoadUint32(&o.BUS_PRIORITY_ACK.Reg) & 0x1
}

// BUSCTRL.PERFCTR_EN: Enable the performance counters. If 0, the performance counters do not increment. This can be used to precisely start/stop event sampling around the profiled section of code. The performance counters are initially disabled, to save energy.
func (o *BUSCTRL_Type) SetPERFCTR_EN(value uint32) {
	volatile.StoreUint32(&o.PERFCTR_EN.Reg, volatile.LoadUint32(&o.PERFCTR_EN.Reg)&^(0x1)|value)
}
func (o *BUSCTRL_Type) GetPERFCTR_EN() uint32 {
	return volatile.LoadUint32(&o.PERFCTR_EN.Reg) & 0x1
}

// BUSCTRL.PERFCTR0: Bus fabric performance counter 0
func (o *BUSCTRL_Type) SetPERFCTR0(value uint32) {
	volatile.StoreUint32(&o.PERFCTR0.Reg, volatile.LoadUint32(&o.PERFCTR0.Reg)&^(0xffffff)|value)
}
func (o *BUSCTRL_Type) GetPERFCTR0() uint32 {
	return volatile.LoadUint32(&o.PERFCTR0.Reg) & 0xffffff
}

// BUSCTRL.PERFSEL0: Bus fabric performance event select for PERFCTR0
func (o *BUSCTRL_Type) SetPERFSEL0(value uint32) {
	volatile.StoreUint32(&o.PERFSEL0.Reg, volatile.LoadUint32(&o.PERFSEL0.Reg)&^(0x7f)|value)
}
func (o *BUSCTRL_Type) GetPERFSEL0() uint32 {
	return volatile.LoadUint32(&o.PERFSEL0.Reg) & 0x7f
}

// BUSCTRL.PERFCTR1: Bus fabric performance counter 1
func (o *BUSCTRL_Type) SetPERFCTR1(value uint32) {
	volatile.StoreUint32(&o.PERFCTR1.Reg, volatile.LoadUint32(&o.PERFCTR1.Reg)&^(0xffffff)|value)
}
func (o *BUSCTRL_Type) GetPERFCTR1() uint32 {
	return volatile.LoadUint32(&o.PERFCTR1.Reg) & 0xffffff
}

// BUSCTRL.PERFSEL1: Bus fabric performance event select for PERFCTR1
func (o *BUSCTRL_Type) SetPERFSEL1(value uint32) {
	volatile.StoreUint32(&o.PERFSEL1.Reg, volatile.LoadUint32(&o.PERFSEL1.Reg)&^(0x7f)|value)
}
func (o *BUSCTRL_Type) GetPERFSEL1() uint32 {
	return volatile.LoadUint32(&o.PERFSEL1.Reg) & 0x7f
}

// BUSCTRL.PERFCTR2: Bus fabric performance counter 2
func (o *BUSCTRL_Type) SetPERFCTR2(value uint32) {
	volatile.StoreUint32(&o.PERFCTR2.Reg, volatile.LoadUint32(&o.PERFCTR2.Reg)&^(0xffffff)|value)
}
func (o *BUSCTRL_Type) GetPERFCTR2() uint32 {
	return volatile.LoadUint32(&o.PERFCTR2.Reg) & 0xffffff
}

// BUSCTRL.PERFSEL2: Bus fabric performance event select for PERFCTR2
func (o *BUSCTRL_Type) SetPERFSEL2(value uint32) {
	volatile.StoreUint32(&o.PERFSEL2.Reg, volatile.LoadUint32(&o.PERFSEL2.Reg)&^(0x7f)|value)
}
func (o *BUSCTRL_Type) GetPERFSEL2() uint32 {
	return volatile.LoadUint32(&o.PERFSEL2.Reg) & 0x7f
}

// BUSCTRL.PERFCTR3: Bus fabric performance counter 3
func (o *BUSCTRL_Type) SetPERFCTR3(value uint32) {
	volatile.StoreUint32(&o.PERFCTR3.Reg, volatile.LoadUint32(&o.PERFCTR3.Reg)&^(0xffffff)|value)
}
func (o *BUSCTRL_Type) GetPERFCTR3() uint32 {
	return volatile.LoadUint32(&o.PERFCTR3.Reg) & 0xffffff
}

// BUSCTRL.PERFSEL3: Bus fabric performance event select for PERFCTR3
func (o *BUSCTRL_Type) SetPERFSEL3(value uint32) {
	volatile.StoreUint32(&o.PERFSEL3.Reg, volatile.LoadUint32(&o.PERFSEL3.Reg)&^(0x7f)|value)
}
func (o *BUSCTRL_Type) GetPERFSEL3() uint32 {
	return volatile.LoadUint32(&o.PERFSEL3.Reg) & 0x7f
}

// Single-cycle IO block Provides core-local and inter-core hardware for the two processors, with single-cycle access.
type SIO_Type struct {
	CPUID               volatile.Register32 // 0x0
	GPIO_IN             volatile.Register32 // 0x4
	GPIO_HI_IN          volatile.Register32 // 0x8
	_                   [4]byte
	GPIO_OUT            volatile.Register32 // 0x10
	GPIO_HI_OUT         volatile.Register32 // 0x14
	GPIO_OUT_SET        volatile.Register32 // 0x18
	GPIO_HI_OUT_SET     volatile.Register32 // 0x1C
	GPIO_OUT_CLR        volatile.Register32 // 0x20
	GPIO_HI_OUT_CLR     volatile.Register32 // 0x24
	GPIO_OUT_XOR        volatile.Register32 // 0x28
	GPIO_HI_OUT_XOR     volatile.Register32 // 0x2C
	GPIO_OE             volatile.Register32 // 0x30
	GPIO_HI_OE          volatile.Register32 // 0x34
	GPIO_OE_SET         volatile.Register32 // 0x38
	GPIO_HI_OE_SET      volatile.Register32 // 0x3C
	GPIO_OE_CLR         volatile.Register32 // 0x40
	GPIO_HI_OE_CLR      volatile.Register32 // 0x44
	GPIO_OE_XOR         volatile.Register32 // 0x48
	GPIO_HI_OE_XOR      volatile.Register32 // 0x4C
	FIFO_ST             volatile.Register32 // 0x50
	FIFO_WR             volatile.Register32 // 0x54
	FIFO_RD             volatile.Register32 // 0x58
	SPINLOCK_ST         volatile.Register32 // 0x5C
	_                   [32]byte
	INTERP0_ACCUM0      volatile.Register32 // 0x80
	INTERP0_ACCUM1      volatile.Register32 // 0x84
	INTERP0_BASE0       volatile.Register32 // 0x88
	INTERP0_BASE1       volatile.Register32 // 0x8C
	INTERP0_BASE2       volatile.Register32 // 0x90
	INTERP0_POP_LANE0   volatile.Register32 // 0x94
	INTERP0_POP_LANE1   volatile.Register32 // 0x98
	INTERP0_POP_FULL    volatile.Register32 // 0x9C
	INTERP0_PEEK_LANE0  volatile.Register32 // 0xA0
	INTERP0_PEEK_LANE1  volatile.Register32 // 0xA4
	INTERP0_PEEK_FULL   volatile.Register32 // 0xA8
	INTERP0_CTRL_LANE0  volatile.Register32 // 0xAC
	INTERP0_CTRL_LANE1  volatile.Register32 // 0xB0
	INTERP0_ACCUM0_ADD  volatile.Register32 // 0xB4
	INTERP0_ACCUM1_ADD  volatile.Register32 // 0xB8
	INTERP0_BASE_1AND0  volatile.Register32 // 0xBC
	INTERP1_ACCUM0      volatile.Register32 // 0xC0
	INTERP1_ACCUM1      volatile.Register32 // 0xC4
	INTERP1_BASE0       volatile.Register32 // 0xC8
	INTERP1_BASE1       volatile.Register32 // 0xCC
	INTERP1_BASE2       volatile.Register32 // 0xD0
	INTERP1_POP_LANE0   volatile.Register32 // 0xD4
	INTERP1_POP_LANE1   volatile.Register32 // 0xD8
	INTERP1_POP_FULL    volatile.Register32 // 0xDC
	INTERP1_PEEK_LANE0  volatile.Register32 // 0xE0
	INTERP1_PEEK_LANE1  volatile.Register32 // 0xE4
	INTERP1_PEEK_FULL   volatile.Register32 // 0xE8
	INTERP1_CTRL_LANE0  volatile.Register32 // 0xEC
	INTERP1_CTRL_LANE1  volatile.Register32 // 0xF0
	INTERP1_ACCUM0_ADD  volatile.Register32 // 0xF4
	INTERP1_ACCUM1_ADD  volatile.Register32 // 0xF8
	INTERP1_BASE_1AND0  volatile.Register32 // 0xFC
	SPINLOCK0           volatile.Register32 // 0x100
	SPINLOCK1           volatile.Register32 // 0x104
	SPINLOCK2           volatile.Register32 // 0x108
	SPINLOCK3           volatile.Register32 // 0x10C
	SPINLOCK4           volatile.Register32 // 0x110
	SPINLOCK5           volatile.Register32 // 0x114
	SPINLOCK6           volatile.Register32 // 0x118
	SPINLOCK7           volatile.Register32 // 0x11C
	SPINLOCK8           volatile.Register32 // 0x120
	SPINLOCK9           volatile.Register32 // 0x124
	SPINLOCK10          volatile.Register32 // 0x128
	SPINLOCK11          volatile.Register32 // 0x12C
	SPINLOCK12          volatile.Register32 // 0x130
	SPINLOCK13          volatile.Register32 // 0x134
	SPINLOCK14          volatile.Register32 // 0x138
	SPINLOCK15          volatile.Register32 // 0x13C
	SPINLOCK16          volatile.Register32 // 0x140
	SPINLOCK17          volatile.Register32 // 0x144
	SPINLOCK18          volatile.Register32 // 0x148
	SPINLOCK19          volatile.Register32 // 0x14C
	SPINLOCK20          volatile.Register32 // 0x150
	SPINLOCK21          volatile.Register32 // 0x154
	SPINLOCK22          volatile.Register32 // 0x158
	SPINLOCK23          volatile.Register32 // 0x15C
	SPINLOCK24          volatile.Register32 // 0x160
	SPINLOCK25          volatile.Register32 // 0x164
	SPINLOCK26          volatile.Register32 // 0x168
	SPINLOCK27          volatile.Register32 // 0x16C
	SPINLOCK28          volatile.Register32 // 0x170
	SPINLOCK29          volatile.Register32 // 0x174
	SPINLOCK30          volatile.Register32 // 0x178
	SPINLOCK31          volatile.Register32 // 0x17C
	DOORBELL_OUT_SET    volatile.Register32 // 0x180
	DOORBELL_OUT_CLR    volatile.Register32 // 0x184
	DOORBELL_IN_SET     volatile.Register32 // 0x188
	DOORBELL_IN_CLR     volatile.Register32 // 0x18C
	PERI_NONSEC         volatile.Register32 // 0x190
	_                   [12]byte
	RISCV_SOFTIRQ       volatile.Register32 // 0x1A0
	MTIME_CTRL          volatile.Register32 // 0x1A4
	_                   [8]byte
	MTIME               volatile.Register32 // 0x1B0
	MTIMEH              volatile.Register32 // 0x1B4
	MTIMECMP            volatile.Register32 // 0x1B8
	MTIMECMPH           volatile.Register32 // 0x1BC
	TMDS_CTRL           volatile.Register32 // 0x1C0
	TMDS_WDATA          volatile.Register32 // 0x1C4
	TMDS_PEEK_SINGLE    volatile.Register32 // 0x1C8
	TMDS_POP_SINGLE     volatile.Register32 // 0x1CC
	TMDS_PEEK_DOUBLE_L0 volatile.Register32 // 0x1D0
	TMDS_POP_DOUBLE_L0  volatile.Register32 // 0x1D4
	TMDS_PEEK_DOUBLE_L1 volatile.Register32 // 0x1D8
	TMDS_POP_DOUBLE_L1  volatile.Register32 // 0x1DC
	TMDS_PEEK_DOUBLE_L2 volatile.Register32 // 0x1E0
	TMDS_POP_DOUBLE_L2  volatile.Register32 // 0x1E4
}

// SIO.CPUID: Processor core identifier
func (o *SIO_Type) SetCPUID(value uint32) {
	volatile.StoreUint32(&o.CPUID.Reg, value)
}
func (o *SIO_Type) GetCPUID() uint32 {
	return volatile.LoadUint32(&o.CPUID.Reg)
}

// SIO.GPIO_IN: Input value for GPIO0...31. In the Non-secure SIO, Secure-only GPIOs (as per ACCESSCTRL) appear as zero.
func (o *SIO_Type) SetGPIO_IN(value uint32) {
	volatile.StoreUint32(&o.GPIO_IN.Reg, value)
}
func (o *SIO_Type) GetGPIO_IN() uint32 {
	return volatile.LoadUint32(&o.GPIO_IN.Reg)
}

// SIO.GPIO_HI_IN: Input value on GPIO32...47, QSPI IOs and USB pins In the Non-secure SIO, Secure-only GPIOs (as per ACCESSCTRL) appear as zero.
func (o *SIO_Type) SetGPIO_HI_IN_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_IN.Reg, volatile.LoadUint32(&o.GPIO_HI_IN.Reg)&^(0xf0000000)|value<<28)
}
func (o *SIO_Type) GetGPIO_HI_IN_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_IN.Reg) & 0xf0000000) >> 28
}
func (o *SIO_Type) SetGPIO_HI_IN_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_IN.Reg, volatile.LoadUint32(&o.GPIO_HI_IN.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetGPIO_HI_IN_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_IN.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetGPIO_HI_IN_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_IN.Reg, volatile.LoadUint32(&o.GPIO_HI_IN.Reg)&^(0x4000000)|value<<26)
}
func (o *SIO_Type) GetGPIO_HI_IN_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_IN.Reg) & 0x4000000) >> 26
}
func (o *SIO_Type) SetGPIO_HI_IN_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_IN.Reg, volatile.LoadUint32(&o.GPIO_HI_IN.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetGPIO_HI_IN_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_IN.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetGPIO_HI_IN_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_IN.Reg, volatile.LoadUint32(&o.GPIO_HI_IN.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetGPIO_HI_IN_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_IN.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetGPIO_HI_IN_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_IN.Reg, volatile.LoadUint32(&o.GPIO_HI_IN.Reg)&^(0xffff)|value)
}
func (o *SIO_Type) GetGPIO_HI_IN_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_HI_IN.Reg) & 0xffff
}

// SIO.GPIO_OUT: GPIO0...31 output value
func (o *SIO_Type) SetGPIO_OUT(value uint32) {
	volatile.StoreUint32(&o.GPIO_OUT.Reg, value)
}
func (o *SIO_Type) GetGPIO_OUT() uint32 {
	return volatile.LoadUint32(&o.GPIO_OUT.Reg)
}

// SIO.GPIO_HI_OUT: Output value for GPIO32...47, QSPI IOs and USB pins. Write to set output level (1/0 -> high/low). Reading back gives the last value written, NOT the input value from the pins. If core 0 and core 1 both write to GPIO_HI_OUT simultaneously (or to a SET/CLR/XOR alias), the result is as though the write from core 0 took place first, and the write from core 1 was then applied to that intermediate result. In the Non-secure SIO, Secure-only GPIOs (as per ACCESSCTRL) ignore writes, and their output status reads back as zero. This is also true for SET/CLR/XOR aliases of this register.
func (o *SIO_Type) SetGPIO_HI_OUT_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT.Reg)&^(0xf0000000)|value<<28)
}
func (o *SIO_Type) GetGPIO_HI_OUT_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT.Reg) & 0xf0000000) >> 28
}
func (o *SIO_Type) SetGPIO_HI_OUT_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetGPIO_HI_OUT_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetGPIO_HI_OUT_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT.Reg)&^(0x4000000)|value<<26)
}
func (o *SIO_Type) GetGPIO_HI_OUT_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT.Reg) & 0x4000000) >> 26
}
func (o *SIO_Type) SetGPIO_HI_OUT_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetGPIO_HI_OUT_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetGPIO_HI_OUT_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetGPIO_HI_OUT_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetGPIO_HI_OUT_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT.Reg)&^(0xffff)|value)
}
func (o *SIO_Type) GetGPIO_HI_OUT_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_HI_OUT.Reg) & 0xffff
}

// SIO.GPIO_OUT_SET: GPIO0...31 output value set
func (o *SIO_Type) SetGPIO_OUT_SET(value uint32) {
	volatile.StoreUint32(&o.GPIO_OUT_SET.Reg, value)
}
func (o *SIO_Type) GetGPIO_OUT_SET() uint32 {
	return volatile.LoadUint32(&o.GPIO_OUT_SET.Reg)
}

// SIO.GPIO_HI_OUT_SET: Output value set for GPIO32..47, QSPI IOs and USB pins. Perform an atomic bit-set on GPIO_HI_OUT, i.e. `GPIO_HI_OUT |= wdata`
func (o *SIO_Type) SetGPIO_HI_OUT_SET_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg)&^(0xf0000000)|value<<28)
}
func (o *SIO_Type) GetGPIO_HI_OUT_SET_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg) & 0xf0000000) >> 28
}
func (o *SIO_Type) SetGPIO_HI_OUT_SET_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetGPIO_HI_OUT_SET_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetGPIO_HI_OUT_SET_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg)&^(0x4000000)|value<<26)
}
func (o *SIO_Type) GetGPIO_HI_OUT_SET_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg) & 0x4000000) >> 26
}
func (o *SIO_Type) SetGPIO_HI_OUT_SET_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetGPIO_HI_OUT_SET_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetGPIO_HI_OUT_SET_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetGPIO_HI_OUT_SET_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetGPIO_HI_OUT_SET_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg)&^(0xffff)|value)
}
func (o *SIO_Type) GetGPIO_HI_OUT_SET_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_HI_OUT_SET.Reg) & 0xffff
}

// SIO.GPIO_OUT_CLR: GPIO0...31 output value clear
func (o *SIO_Type) SetGPIO_OUT_CLR(value uint32) {
	volatile.StoreUint32(&o.GPIO_OUT_CLR.Reg, value)
}
func (o *SIO_Type) GetGPIO_OUT_CLR() uint32 {
	return volatile.LoadUint32(&o.GPIO_OUT_CLR.Reg)
}

// SIO.GPIO_HI_OUT_CLR: Output value clear for GPIO32..47, QSPI IOs and USB pins. Perform an atomic bit-clear on GPIO_HI_OUT, i.e. `GPIO_HI_OUT &= ~wdata`
func (o *SIO_Type) SetGPIO_HI_OUT_CLR_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg)&^(0xf0000000)|value<<28)
}
func (o *SIO_Type) GetGPIO_HI_OUT_CLR_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg) & 0xf0000000) >> 28
}
func (o *SIO_Type) SetGPIO_HI_OUT_CLR_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetGPIO_HI_OUT_CLR_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetGPIO_HI_OUT_CLR_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg)&^(0x4000000)|value<<26)
}
func (o *SIO_Type) GetGPIO_HI_OUT_CLR_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg) & 0x4000000) >> 26
}
func (o *SIO_Type) SetGPIO_HI_OUT_CLR_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetGPIO_HI_OUT_CLR_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetGPIO_HI_OUT_CLR_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetGPIO_HI_OUT_CLR_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetGPIO_HI_OUT_CLR_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg)&^(0xffff)|value)
}
func (o *SIO_Type) GetGPIO_HI_OUT_CLR_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_HI_OUT_CLR.Reg) & 0xffff
}

// SIO.GPIO_OUT_XOR: GPIO0...31 output value XOR
func (o *SIO_Type) SetGPIO_OUT_XOR(value uint32) {
	volatile.StoreUint32(&o.GPIO_OUT_XOR.Reg, value)
}
func (o *SIO_Type) GetGPIO_OUT_XOR() uint32 {
	return volatile.LoadUint32(&o.GPIO_OUT_XOR.Reg)
}

// SIO.GPIO_HI_OUT_XOR: Output value XOR for GPIO32..47, QSPI IOs and USB pins. Perform an atomic bitwise XOR on GPIO_HI_OUT, i.e. `GPIO_HI_OUT ^= wdata`
func (o *SIO_Type) SetGPIO_HI_OUT_XOR_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg)&^(0xf0000000)|value<<28)
}
func (o *SIO_Type) GetGPIO_HI_OUT_XOR_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg) & 0xf0000000) >> 28
}
func (o *SIO_Type) SetGPIO_HI_OUT_XOR_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetGPIO_HI_OUT_XOR_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetGPIO_HI_OUT_XOR_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg)&^(0x4000000)|value<<26)
}
func (o *SIO_Type) GetGPIO_HI_OUT_XOR_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg) & 0x4000000) >> 26
}
func (o *SIO_Type) SetGPIO_HI_OUT_XOR_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetGPIO_HI_OUT_XOR_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetGPIO_HI_OUT_XOR_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetGPIO_HI_OUT_XOR_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetGPIO_HI_OUT_XOR_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OUT_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg)&^(0xffff)|value)
}
func (o *SIO_Type) GetGPIO_HI_OUT_XOR_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_HI_OUT_XOR.Reg) & 0xffff
}

// SIO.GPIO_OE: GPIO0...31 output enable
func (o *SIO_Type) SetGPIO_OE(value uint32) {
	volatile.StoreUint32(&o.GPIO_OE.Reg, value)
}
func (o *SIO_Type) GetGPIO_OE() uint32 {
	return volatile.LoadUint32(&o.GPIO_OE.Reg)
}

// SIO.GPIO_HI_OE: Output enable value for GPIO32...47, QSPI IOs and USB pins. Write output enable (1/0 -> output/input). Reading back gives the last value written. If core 0 and core 1 both write to GPIO_HI_OE simultaneously (or to a SET/CLR/XOR alias), the result is as though the write from core 0 took place first, and the write from core 1 was then applied to that intermediate result. In the Non-secure SIO, Secure-only GPIOs (as per ACCESSCTRL) ignore writes, and their output status reads back as zero. This is also true for SET/CLR/XOR aliases of this register.
func (o *SIO_Type) SetGPIO_HI_OE_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE.Reg, volatile.LoadUint32(&o.GPIO_HI_OE.Reg)&^(0xf0000000)|value<<28)
}
func (o *SIO_Type) GetGPIO_HI_OE_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE.Reg) & 0xf0000000) >> 28
}
func (o *SIO_Type) SetGPIO_HI_OE_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE.Reg, volatile.LoadUint32(&o.GPIO_HI_OE.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetGPIO_HI_OE_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetGPIO_HI_OE_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE.Reg, volatile.LoadUint32(&o.GPIO_HI_OE.Reg)&^(0x4000000)|value<<26)
}
func (o *SIO_Type) GetGPIO_HI_OE_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE.Reg) & 0x4000000) >> 26
}
func (o *SIO_Type) SetGPIO_HI_OE_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE.Reg, volatile.LoadUint32(&o.GPIO_HI_OE.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetGPIO_HI_OE_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetGPIO_HI_OE_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE.Reg, volatile.LoadUint32(&o.GPIO_HI_OE.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetGPIO_HI_OE_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetGPIO_HI_OE_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE.Reg, volatile.LoadUint32(&o.GPIO_HI_OE.Reg)&^(0xffff)|value)
}
func (o *SIO_Type) GetGPIO_HI_OE_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_HI_OE.Reg) & 0xffff
}

// SIO.GPIO_OE_SET: GPIO0...31 output enable set
func (o *SIO_Type) SetGPIO_OE_SET(value uint32) {
	volatile.StoreUint32(&o.GPIO_OE_SET.Reg, value)
}
func (o *SIO_Type) GetGPIO_OE_SET() uint32 {
	return volatile.LoadUint32(&o.GPIO_OE_SET.Reg)
}

// SIO.GPIO_HI_OE_SET: Output enable set for GPIO32...47, QSPI IOs and USB pins. Perform an atomic bit-set on GPIO_HI_OE, i.e. `GPIO_HI_OE |= wdata`
func (o *SIO_Type) SetGPIO_HI_OE_SET_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg)&^(0xf0000000)|value<<28)
}
func (o *SIO_Type) GetGPIO_HI_OE_SET_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg) & 0xf0000000) >> 28
}
func (o *SIO_Type) SetGPIO_HI_OE_SET_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetGPIO_HI_OE_SET_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetGPIO_HI_OE_SET_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg)&^(0x4000000)|value<<26)
}
func (o *SIO_Type) GetGPIO_HI_OE_SET_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg) & 0x4000000) >> 26
}
func (o *SIO_Type) SetGPIO_HI_OE_SET_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetGPIO_HI_OE_SET_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetGPIO_HI_OE_SET_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetGPIO_HI_OE_SET_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetGPIO_HI_OE_SET_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_SET.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg)&^(0xffff)|value)
}
func (o *SIO_Type) GetGPIO_HI_OE_SET_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_HI_OE_SET.Reg) & 0xffff
}

// SIO.GPIO_OE_CLR: GPIO0...31 output enable clear
func (o *SIO_Type) SetGPIO_OE_CLR(value uint32) {
	volatile.StoreUint32(&o.GPIO_OE_CLR.Reg, value)
}
func (o *SIO_Type) GetGPIO_OE_CLR() uint32 {
	return volatile.LoadUint32(&o.GPIO_OE_CLR.Reg)
}

// SIO.GPIO_HI_OE_CLR: Output enable clear for GPIO32...47, QSPI IOs and USB pins. Perform an atomic bit-clear on GPIO_HI_OE, i.e. `GPIO_HI_OE &= ~wdata`
func (o *SIO_Type) SetGPIO_HI_OE_CLR_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg)&^(0xf0000000)|value<<28)
}
func (o *SIO_Type) GetGPIO_HI_OE_CLR_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg) & 0xf0000000) >> 28
}
func (o *SIO_Type) SetGPIO_HI_OE_CLR_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetGPIO_HI_OE_CLR_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetGPIO_HI_OE_CLR_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg)&^(0x4000000)|value<<26)
}
func (o *SIO_Type) GetGPIO_HI_OE_CLR_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg) & 0x4000000) >> 26
}
func (o *SIO_Type) SetGPIO_HI_OE_CLR_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetGPIO_HI_OE_CLR_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetGPIO_HI_OE_CLR_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetGPIO_HI_OE_CLR_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetGPIO_HI_OE_CLR_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_CLR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg)&^(0xffff)|value)
}
func (o *SIO_Type) GetGPIO_HI_OE_CLR_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_HI_OE_CLR.Reg) & 0xffff
}

// SIO.GPIO_OE_XOR: GPIO0...31 output enable XOR
func (o *SIO_Type) SetGPIO_OE_XOR(value uint32) {
	volatile.StoreUint32(&o.GPIO_OE_XOR.Reg, value)
}
func (o *SIO_Type) GetGPIO_OE_XOR() uint32 {
	return volatile.LoadUint32(&o.GPIO_OE_XOR.Reg)
}

// SIO.GPIO_HI_OE_XOR: Output enable XOR for GPIO32...47, QSPI IOs and USB pins. Perform an atomic bitwise XOR on GPIO_HI_OE, i.e. `GPIO_HI_OE ^= wdata`
func (o *SIO_Type) SetGPIO_HI_OE_XOR_QSPI_SD(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg)&^(0xf0000000)|value<<28)
}
func (o *SIO_Type) GetGPIO_HI_OE_XOR_QSPI_SD() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg) & 0xf0000000) >> 28
}
func (o *SIO_Type) SetGPIO_HI_OE_XOR_QSPI_CSN(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetGPIO_HI_OE_XOR_QSPI_CSN() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetGPIO_HI_OE_XOR_QSPI_SCK(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg)&^(0x4000000)|value<<26)
}
func (o *SIO_Type) GetGPIO_HI_OE_XOR_QSPI_SCK() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg) & 0x4000000) >> 26
}
func (o *SIO_Type) SetGPIO_HI_OE_XOR_USB_DM(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetGPIO_HI_OE_XOR_USB_DM() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetGPIO_HI_OE_XOR_USB_DP(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetGPIO_HI_OE_XOR_USB_DP() uint32 {
	return (volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetGPIO_HI_OE_XOR_GPIO(value uint32) {
	volatile.StoreUint32(&o.GPIO_HI_OE_XOR.Reg, volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg)&^(0xffff)|value)
}
func (o *SIO_Type) GetGPIO_HI_OE_XOR_GPIO() uint32 {
	return volatile.LoadUint32(&o.GPIO_HI_OE_XOR.Reg) & 0xffff
}

// SIO.FIFO_ST: Status register for inter-core FIFOs (mailboxes). There is one FIFO in the core 0 -> core 1 direction, and one core 1 -> core 0. Both are 32 bits wide and 8 words deep. Core 0 can see the read side of the 1->0 FIFO (RX), and the write side of 0->1 FIFO (TX). Core 1 can see the read side of the 0->1 FIFO (RX), and the write side of 1->0 FIFO (TX). The SIO IRQ for each core is the logical OR of the VLD, WOF and ROE fields of its FIFO_ST register.
func (o *SIO_Type) SetFIFO_ST_ROE(value uint32) {
	volatile.StoreUint32(&o.FIFO_ST.Reg, volatile.LoadUint32(&o.FIFO_ST.Reg)&^(0x8)|value<<3)
}
func (o *SIO_Type) GetFIFO_ST_ROE() uint32 {
	return (volatile.LoadUint32(&o.FIFO_ST.Reg) & 0x8) >> 3
}
func (o *SIO_Type) SetFIFO_ST_WOF(value uint32) {
	volatile.StoreUint32(&o.FIFO_ST.Reg, volatile.LoadUint32(&o.FIFO_ST.Reg)&^(0x4)|value<<2)
}
func (o *SIO_Type) GetFIFO_ST_WOF() uint32 {
	return (volatile.LoadUint32(&o.FIFO_ST.Reg) & 0x4) >> 2
}
func (o *SIO_Type) SetFIFO_ST_RDY(value uint32) {
	volatile.StoreUint32(&o.FIFO_ST.Reg, volatile.LoadUint32(&o.FIFO_ST.Reg)&^(0x2)|value<<1)
}
func (o *SIO_Type) GetFIFO_ST_RDY() uint32 {
	return (volatile.LoadUint32(&o.FIFO_ST.Reg) & 0x2) >> 1
}
func (o *SIO_Type) SetFIFO_ST_VLD(value uint32) {
	volatile.StoreUint32(&o.FIFO_ST.Reg, volatile.LoadUint32(&o.FIFO_ST.Reg)&^(0x1)|value)
}
func (o *SIO_Type) GetFIFO_ST_VLD() uint32 {
	return volatile.LoadUint32(&o.FIFO_ST.Reg) & 0x1
}

// SIO.FIFO_WR: Write access to this core's TX FIFO
func (o *SIO_Type) SetFIFO_WR(value uint32) {
	volatile.StoreUint32(&o.FIFO_WR.Reg, value)
}
func (o *SIO_Type) GetFIFO_WR() uint32 {
	return volatile.LoadUint32(&o.FIFO_WR.Reg)
}

// SIO.FIFO_RD: Read access to this core's RX FIFO
func (o *SIO_Type) SetFIFO_RD(value uint32) {
	volatile.StoreUint32(&o.FIFO_RD.Reg, value)
}
func (o *SIO_Type) GetFIFO_RD() uint32 {
	return volatile.LoadUint32(&o.FIFO_RD.Reg)
}

// SIO.SPINLOCK_ST: Spinlock state A bitmap containing the state of all 32 spinlocks (1=locked). Mainly intended for debugging.
func (o *SIO_Type) SetSPINLOCK_ST(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK_ST.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK_ST() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK_ST.Reg)
}

// SIO.INTERP0_ACCUM0: Read/write access to accumulator 0
func (o *SIO_Type) SetINTERP0_ACCUM0(value uint32) {
	volatile.StoreUint32(&o.INTERP0_ACCUM0.Reg, value)
}
func (o *SIO_Type) GetINTERP0_ACCUM0() uint32 {
	return volatile.LoadUint32(&o.INTERP0_ACCUM0.Reg)
}

// SIO.INTERP0_ACCUM1: Read/write access to accumulator 1
func (o *SIO_Type) SetINTERP0_ACCUM1(value uint32) {
	volatile.StoreUint32(&o.INTERP0_ACCUM1.Reg, value)
}
func (o *SIO_Type) GetINTERP0_ACCUM1() uint32 {
	return volatile.LoadUint32(&o.INTERP0_ACCUM1.Reg)
}

// SIO.INTERP0_BASE0: Read/write access to BASE0 register.
func (o *SIO_Type) SetINTERP0_BASE0(value uint32) {
	volatile.StoreUint32(&o.INTERP0_BASE0.Reg, value)
}
func (o *SIO_Type) GetINTERP0_BASE0() uint32 {
	return volatile.LoadUint32(&o.INTERP0_BASE0.Reg)
}

// SIO.INTERP0_BASE1: Read/write access to BASE1 register.
func (o *SIO_Type) SetINTERP0_BASE1(value uint32) {
	volatile.StoreUint32(&o.INTERP0_BASE1.Reg, value)
}
func (o *SIO_Type) GetINTERP0_BASE1() uint32 {
	return volatile.LoadUint32(&o.INTERP0_BASE1.Reg)
}

// SIO.INTERP0_BASE2: Read/write access to BASE2 register.
func (o *SIO_Type) SetINTERP0_BASE2(value uint32) {
	volatile.StoreUint32(&o.INTERP0_BASE2.Reg, value)
}
func (o *SIO_Type) GetINTERP0_BASE2() uint32 {
	return volatile.LoadUint32(&o.INTERP0_BASE2.Reg)
}

// SIO.INTERP0_POP_LANE0: Read LANE0 result, and simultaneously write lane results to both accumulators (POP).
func (o *SIO_Type) SetINTERP0_POP_LANE0(value uint32) {
	volatile.StoreUint32(&o.INTERP0_POP_LANE0.Reg, value)
}
func (o *SIO_Type) GetINTERP0_POP_LANE0() uint32 {
	return volatile.LoadUint32(&o.INTERP0_POP_LANE0.Reg)
}

// SIO.INTERP0_POP_LANE1: Read LANE1 result, and simultaneously write lane results to both accumulators (POP).
func (o *SIO_Type) SetINTERP0_POP_LANE1(value uint32) {
	volatile.StoreUint32(&o.INTERP0_POP_LANE1.Reg, value)
}
func (o *SIO_Type) GetINTERP0_POP_LANE1() uint32 {
	return volatile.LoadUint32(&o.INTERP0_POP_LANE1.Reg)
}

// SIO.INTERP0_POP_FULL: Read FULL result, and simultaneously write lane results to both accumulators (POP).
func (o *SIO_Type) SetINTERP0_POP_FULL(value uint32) {
	volatile.StoreUint32(&o.INTERP0_POP_FULL.Reg, value)
}
func (o *SIO_Type) GetINTERP0_POP_FULL() uint32 {
	return volatile.LoadUint32(&o.INTERP0_POP_FULL.Reg)
}

// SIO.INTERP0_PEEK_LANE0: Read LANE0 result, without altering any internal state (PEEK).
func (o *SIO_Type) SetINTERP0_PEEK_LANE0(value uint32) {
	volatile.StoreUint32(&o.INTERP0_PEEK_LANE0.Reg, value)
}
func (o *SIO_Type) GetINTERP0_PEEK_LANE0() uint32 {
	return volatile.LoadUint32(&o.INTERP0_PEEK_LANE0.Reg)
}

// SIO.INTERP0_PEEK_LANE1: Read LANE1 result, without altering any internal state (PEEK).
func (o *SIO_Type) SetINTERP0_PEEK_LANE1(value uint32) {
	volatile.StoreUint32(&o.INTERP0_PEEK_LANE1.Reg, value)
}
func (o *SIO_Type) GetINTERP0_PEEK_LANE1() uint32 {
	return volatile.LoadUint32(&o.INTERP0_PEEK_LANE1.Reg)
}

// SIO.INTERP0_PEEK_FULL: Read FULL result, without altering any internal state (PEEK).
func (o *SIO_Type) SetINTERP0_PEEK_FULL(value uint32) {
	volatile.StoreUint32(&o.INTERP0_PEEK_FULL.Reg, value)
}
func (o *SIO_Type) GetINTERP0_PEEK_FULL() uint32 {
	return volatile.LoadUint32(&o.INTERP0_PEEK_FULL.Reg)
}

// SIO.INTERP0_CTRL_LANE0: Control register for lane 0
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_OVERF(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_OVERF() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_OVERF1(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_OVERF1() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_OVERF0(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x800000)|value<<23)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_OVERF0() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x800000) >> 23
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_BLEND(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x200000)|value<<21)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_BLEND() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x200000) >> 21
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_FORCE_MSB(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x180000)|value<<19)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_FORCE_MSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x180000) >> 19
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_ADD_RAW(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x40000)|value<<18)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_ADD_RAW() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x40000) >> 18
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_CROSS_RESULT(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x20000)|value<<17)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_CROSS_RESULT() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x20000) >> 17
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_CROSS_INPUT(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x10000)|value<<16)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_CROSS_INPUT() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x10000) >> 16
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_SIGNED(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x8000)|value<<15)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_SIGNED() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x8000) >> 15
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_MASK_MSB(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x7c00)|value<<10)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_MASK_MSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x7c00) >> 10
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_MASK_LSB(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x3e0)|value<<5)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_MASK_LSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x3e0) >> 5
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE0_SHIFT(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg)&^(0x1f)|value)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE0_SHIFT() uint32 {
	return volatile.LoadUint32(&o.INTERP0_CTRL_LANE0.Reg) & 0x1f
}

// SIO.INTERP0_CTRL_LANE1: Control register for lane 1
func (o *SIO_Type) SetINTERP0_CTRL_LANE1_FORCE_MSB(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg)&^(0x180000)|value<<19)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE1_FORCE_MSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg) & 0x180000) >> 19
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE1_ADD_RAW(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg)&^(0x40000)|value<<18)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE1_ADD_RAW() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg) & 0x40000) >> 18
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE1_CROSS_RESULT(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg)&^(0x20000)|value<<17)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE1_CROSS_RESULT() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg) & 0x20000) >> 17
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE1_CROSS_INPUT(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg)&^(0x10000)|value<<16)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE1_CROSS_INPUT() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg) & 0x10000) >> 16
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE1_SIGNED(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg)&^(0x8000)|value<<15)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE1_SIGNED() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg) & 0x8000) >> 15
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE1_MASK_MSB(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg)&^(0x7c00)|value<<10)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE1_MASK_MSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg) & 0x7c00) >> 10
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE1_MASK_LSB(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg)&^(0x3e0)|value<<5)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE1_MASK_LSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg) & 0x3e0) >> 5
}
func (o *SIO_Type) SetINTERP0_CTRL_LANE1_SHIFT(value uint32) {
	volatile.StoreUint32(&o.INTERP0_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg)&^(0x1f)|value)
}
func (o *SIO_Type) GetINTERP0_CTRL_LANE1_SHIFT() uint32 {
	return volatile.LoadUint32(&o.INTERP0_CTRL_LANE1.Reg) & 0x1f
}

// SIO.INTERP0_ACCUM0_ADD: Values written here are atomically added to ACCUM0 Reading yields lane 0's raw shift and mask value (BASE0 not added).
func (o *SIO_Type) SetINTERP0_ACCUM0_ADD(value uint32) {
	volatile.StoreUint32(&o.INTERP0_ACCUM0_ADD.Reg, volatile.LoadUint32(&o.INTERP0_ACCUM0_ADD.Reg)&^(0xffffff)|value)
}
func (o *SIO_Type) GetINTERP0_ACCUM0_ADD() uint32 {
	return volatile.LoadUint32(&o.INTERP0_ACCUM0_ADD.Reg) & 0xffffff
}

// SIO.INTERP0_ACCUM1_ADD: Values written here are atomically added to ACCUM1 Reading yields lane 1's raw shift and mask value (BASE1 not added).
func (o *SIO_Type) SetINTERP0_ACCUM1_ADD(value uint32) {
	volatile.StoreUint32(&o.INTERP0_ACCUM1_ADD.Reg, volatile.LoadUint32(&o.INTERP0_ACCUM1_ADD.Reg)&^(0xffffff)|value)
}
func (o *SIO_Type) GetINTERP0_ACCUM1_ADD() uint32 {
	return volatile.LoadUint32(&o.INTERP0_ACCUM1_ADD.Reg) & 0xffffff
}

// SIO.INTERP0_BASE_1AND0: On write, the lower 16 bits go to BASE0, upper bits to BASE1 simultaneously. Each half is sign-extended to 32 bits if that lane's SIGNED flag is set.
func (o *SIO_Type) SetINTERP0_BASE_1AND0(value uint32) {
	volatile.StoreUint32(&o.INTERP0_BASE_1AND0.Reg, value)
}
func (o *SIO_Type) GetINTERP0_BASE_1AND0() uint32 {
	return volatile.LoadUint32(&o.INTERP0_BASE_1AND0.Reg)
}

// SIO.INTERP1_ACCUM0: Read/write access to accumulator 0
func (o *SIO_Type) SetINTERP1_ACCUM0(value uint32) {
	volatile.StoreUint32(&o.INTERP1_ACCUM0.Reg, value)
}
func (o *SIO_Type) GetINTERP1_ACCUM0() uint32 {
	return volatile.LoadUint32(&o.INTERP1_ACCUM0.Reg)
}

// SIO.INTERP1_ACCUM1: Read/write access to accumulator 1
func (o *SIO_Type) SetINTERP1_ACCUM1(value uint32) {
	volatile.StoreUint32(&o.INTERP1_ACCUM1.Reg, value)
}
func (o *SIO_Type) GetINTERP1_ACCUM1() uint32 {
	return volatile.LoadUint32(&o.INTERP1_ACCUM1.Reg)
}

// SIO.INTERP1_BASE0: Read/write access to BASE0 register.
func (o *SIO_Type) SetINTERP1_BASE0(value uint32) {
	volatile.StoreUint32(&o.INTERP1_BASE0.Reg, value)
}
func (o *SIO_Type) GetINTERP1_BASE0() uint32 {
	return volatile.LoadUint32(&o.INTERP1_BASE0.Reg)
}

// SIO.INTERP1_BASE1: Read/write access to BASE1 register.
func (o *SIO_Type) SetINTERP1_BASE1(value uint32) {
	volatile.StoreUint32(&o.INTERP1_BASE1.Reg, value)
}
func (o *SIO_Type) GetINTERP1_BASE1() uint32 {
	return volatile.LoadUint32(&o.INTERP1_BASE1.Reg)
}

// SIO.INTERP1_BASE2: Read/write access to BASE2 register.
func (o *SIO_Type) SetINTERP1_BASE2(value uint32) {
	volatile.StoreUint32(&o.INTERP1_BASE2.Reg, value)
}
func (o *SIO_Type) GetINTERP1_BASE2() uint32 {
	return volatile.LoadUint32(&o.INTERP1_BASE2.Reg)
}

// SIO.INTERP1_POP_LANE0: Read LANE0 result, and simultaneously write lane results to both accumulators (POP).
func (o *SIO_Type) SetINTERP1_POP_LANE0(value uint32) {
	volatile.StoreUint32(&o.INTERP1_POP_LANE0.Reg, value)
}
func (o *SIO_Type) GetINTERP1_POP_LANE0() uint32 {
	return volatile.LoadUint32(&o.INTERP1_POP_LANE0.Reg)
}

// SIO.INTERP1_POP_LANE1: Read LANE1 result, and simultaneously write lane results to both accumulators (POP).
func (o *SIO_Type) SetINTERP1_POP_LANE1(value uint32) {
	volatile.StoreUint32(&o.INTERP1_POP_LANE1.Reg, value)
}
func (o *SIO_Type) GetINTERP1_POP_LANE1() uint32 {
	return volatile.LoadUint32(&o.INTERP1_POP_LANE1.Reg)
}

// SIO.INTERP1_POP_FULL: Read FULL result, and simultaneously write lane results to both accumulators (POP).
func (o *SIO_Type) SetINTERP1_POP_FULL(value uint32) {
	volatile.StoreUint32(&o.INTERP1_POP_FULL.Reg, value)
}
func (o *SIO_Type) GetINTERP1_POP_FULL() uint32 {
	return volatile.LoadUint32(&o.INTERP1_POP_FULL.Reg)
}

// SIO.INTERP1_PEEK_LANE0: Read LANE0 result, without altering any internal state (PEEK).
func (o *SIO_Type) SetINTERP1_PEEK_LANE0(value uint32) {
	volatile.StoreUint32(&o.INTERP1_PEEK_LANE0.Reg, value)
}
func (o *SIO_Type) GetINTERP1_PEEK_LANE0() uint32 {
	return volatile.LoadUint32(&o.INTERP1_PEEK_LANE0.Reg)
}

// SIO.INTERP1_PEEK_LANE1: Read LANE1 result, without altering any internal state (PEEK).
func (o *SIO_Type) SetINTERP1_PEEK_LANE1(value uint32) {
	volatile.StoreUint32(&o.INTERP1_PEEK_LANE1.Reg, value)
}
func (o *SIO_Type) GetINTERP1_PEEK_LANE1() uint32 {
	return volatile.LoadUint32(&o.INTERP1_PEEK_LANE1.Reg)
}

// SIO.INTERP1_PEEK_FULL: Read FULL result, without altering any internal state (PEEK).
func (o *SIO_Type) SetINTERP1_PEEK_FULL(value uint32) {
	volatile.StoreUint32(&o.INTERP1_PEEK_FULL.Reg, value)
}
func (o *SIO_Type) GetINTERP1_PEEK_FULL() uint32 {
	return volatile.LoadUint32(&o.INTERP1_PEEK_FULL.Reg)
}

// SIO.INTERP1_CTRL_LANE0: Control register for lane 0
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_OVERF(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x2000000)|value<<25)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_OVERF() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x2000000) >> 25
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_OVERF1(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x1000000)|value<<24)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_OVERF1() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x1000000) >> 24
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_OVERF0(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x800000)|value<<23)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_OVERF0() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x800000) >> 23
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_CLAMP(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x400000)|value<<22)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_CLAMP() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x400000) >> 22
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_FORCE_MSB(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x180000)|value<<19)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_FORCE_MSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x180000) >> 19
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_ADD_RAW(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x40000)|value<<18)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_ADD_RAW() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x40000) >> 18
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_CROSS_RESULT(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x20000)|value<<17)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_CROSS_RESULT() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x20000) >> 17
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_CROSS_INPUT(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x10000)|value<<16)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_CROSS_INPUT() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x10000) >> 16
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_SIGNED(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x8000)|value<<15)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_SIGNED() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x8000) >> 15
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_MASK_MSB(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x7c00)|value<<10)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_MASK_MSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x7c00) >> 10
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_MASK_LSB(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x3e0)|value<<5)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_MASK_LSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x3e0) >> 5
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE0_SHIFT(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE0.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg)&^(0x1f)|value)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE0_SHIFT() uint32 {
	return volatile.LoadUint32(&o.INTERP1_CTRL_LANE0.Reg) & 0x1f
}

// SIO.INTERP1_CTRL_LANE1: Control register for lane 1
func (o *SIO_Type) SetINTERP1_CTRL_LANE1_FORCE_MSB(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg)&^(0x180000)|value<<19)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE1_FORCE_MSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg) & 0x180000) >> 19
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE1_ADD_RAW(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg)&^(0x40000)|value<<18)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE1_ADD_RAW() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg) & 0x40000) >> 18
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE1_CROSS_RESULT(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg)&^(0x20000)|value<<17)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE1_CROSS_RESULT() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg) & 0x20000) >> 17
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE1_CROSS_INPUT(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg)&^(0x10000)|value<<16)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE1_CROSS_INPUT() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg) & 0x10000) >> 16
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE1_SIGNED(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg)&^(0x8000)|value<<15)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE1_SIGNED() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg) & 0x8000) >> 15
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE1_MASK_MSB(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg)&^(0x7c00)|value<<10)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE1_MASK_MSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg) & 0x7c00) >> 10
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE1_MASK_LSB(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg)&^(0x3e0)|value<<5)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE1_MASK_LSB() uint32 {
	return (volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg) & 0x3e0) >> 5
}
func (o *SIO_Type) SetINTERP1_CTRL_LANE1_SHIFT(value uint32) {
	volatile.StoreUint32(&o.INTERP1_CTRL_LANE1.Reg, volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg)&^(0x1f)|value)
}
func (o *SIO_Type) GetINTERP1_CTRL_LANE1_SHIFT() uint32 {
	return volatile.LoadUint32(&o.INTERP1_CTRL_LANE1.Reg) & 0x1f
}

// SIO.INTERP1_ACCUM0_ADD: Values written here are atomically added to ACCUM0 Reading yields lane 0's raw shift and mask value (BASE0 not added).
func (o *SIO_Type) SetINTERP1_ACCUM0_ADD(value uint32) {
	volatile.StoreUint32(&o.INTERP1_ACCUM0_ADD.Reg, volatile.LoadUint32(&o.INTERP1_ACCUM0_ADD.Reg)&^(0xffffff)|value)
}
func (o *SIO_Type) GetINTERP1_ACCUM0_ADD() uint32 {
	return volatile.LoadUint32(&o.INTERP1_ACCUM0_ADD.Reg) & 0xffffff
}

// SIO.INTERP1_ACCUM1_ADD: Values written here are atomically added to ACCUM1 Reading yields lane 1's raw shift and mask value (BASE1 not added).
func (o *SIO_Type) SetINTERP1_ACCUM1_ADD(value uint32) {
	volatile.StoreUint32(&o.INTERP1_ACCUM1_ADD.Reg, volatile.LoadUint32(&o.INTERP1_ACCUM1_ADD.Reg)&^(0xffffff)|value)
}
func (o *SIO_Type) GetINTERP1_ACCUM1_ADD() uint32 {
	return volatile.LoadUint32(&o.INTERP1_ACCUM1_ADD.Reg) & 0xffffff
}

// SIO.INTERP1_BASE_1AND0: On write, the lower 16 bits go to BASE0, upper bits to BASE1 simultaneously. Each half is sign-extended to 32 bits if that lane's SIGNED flag is set.
func (o *SIO_Type) SetINTERP1_BASE_1AND0(value uint32) {
	volatile.StoreUint32(&o.INTERP1_BASE_1AND0.Reg, value)
}
func (o *SIO_Type) GetINTERP1_BASE_1AND0() uint32 {
	return volatile.LoadUint32(&o.INTERP1_BASE_1AND0.Reg)
}

// SIO.SPINLOCK0: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK0(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK0.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK0() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK0.Reg)
}

// SIO.SPINLOCK1: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK1(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK1.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK1() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK1.Reg)
}

// SIO.SPINLOCK2: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK2(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK2.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK2() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK2.Reg)
}

// SIO.SPINLOCK3: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK3(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK3.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK3() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK3.Reg)
}

// SIO.SPINLOCK4: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK4(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK4.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK4() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK4.Reg)
}

// SIO.SPINLOCK5: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK5(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK5.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK5() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK5.Reg)
}

// SIO.SPINLOCK6: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK6(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK6.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK6() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK6.Reg)
}

// SIO.SPINLOCK7: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK7(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK7.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK7() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK7.Reg)
}

// SIO.SPINLOCK8: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK8(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK8.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK8() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK8.Reg)
}

// SIO.SPINLOCK9: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK9(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK9.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK9() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK9.Reg)
}

// SIO.SPINLOCK10: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK10(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK10.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK10() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK10.Reg)
}

// SIO.SPINLOCK11: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK11(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK11.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK11() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK11.Reg)
}

// SIO.SPINLOCK12: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK12(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK12.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK12() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK12.Reg)
}

// SIO.SPINLOCK13: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK13(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK13.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK13() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK13.Reg)
}

// SIO.SPINLOCK14: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK14(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK14.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK14() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK14.Reg)
}

// SIO.SPINLOCK15: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK15(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK15.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK15() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK15.Reg)
}

// SIO.SPINLOCK16: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK16(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK16.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK16() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK16.Reg)
}

// SIO.SPINLOCK17: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK17(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK17.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK17() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK17.Reg)
}

// SIO.SPINLOCK18: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK18(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK18.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK18() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK18.Reg)
}

// SIO.SPINLOCK19: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK19(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK19.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK19() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK19.Reg)
}

// SIO.SPINLOCK20: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK20(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK20.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK20() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK20.Reg)
}

// SIO.SPINLOCK21: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK21(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK21.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK21() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK21.Reg)
}

// SIO.SPINLOCK22: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK22(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK22.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK22() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK22.Reg)
}

// SIO.SPINLOCK23: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK23(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK23.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK23() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK23.Reg)
}

// SIO.SPINLOCK24: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK24(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK24.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK24() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK24.Reg)
}

// SIO.SPINLOCK25: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK25(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK25.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK25() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK25.Reg)
}

// SIO.SPINLOCK26: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK26(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK26.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK26() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK26.Reg)
}

// SIO.SPINLOCK27: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK27(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK27.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK27() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK27.Reg)
}

// SIO.SPINLOCK28: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK28(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK28.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK28() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK28.Reg)
}

// SIO.SPINLOCK29: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK29(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK29.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK29() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK29.Reg)
}

// SIO.SPINLOCK30: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK30(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK30.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK30() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK30.Reg)
}

// SIO.SPINLOCK31: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
func (o *SIO_Type) SetSPINLOCK31(value uint32) {
	volatile.StoreUint32(&o.SPINLOCK31.Reg, value)
}
func (o *SIO_Type) GetSPINLOCK31() uint32 {
	return volatile.LoadUint32(&o.SPINLOCK31.Reg)
}

// SIO.DOORBELL_OUT_SET: Trigger a doorbell interrupt on the opposite core. Write 1 to a bit to set the corresponding bit in DOORBELL_IN on the opposite core. This raises the opposite core's doorbell interrupt. Read to get the status of the doorbells currently asserted on the opposite core. This is equivalent to that core reading its own DOORBELL_IN status.
func (o *SIO_Type) SetDOORBELL_OUT_SET(value uint32) {
	volatile.StoreUint32(&o.DOORBELL_OUT_SET.Reg, volatile.LoadUint32(&o.DOORBELL_OUT_SET.Reg)&^(0xff)|value)
}
func (o *SIO_Type) GetDOORBELL_OUT_SET() uint32 {
	return volatile.LoadUint32(&o.DOORBELL_OUT_SET.Reg) & 0xff
}

// SIO.DOORBELL_OUT_CLR: Clear doorbells which have been posted to the opposite core. This register is intended for debugging and initialisation purposes. Writing 1 to a bit in DOORBELL_OUT_CLR clears the corresponding bit in DOORBELL_IN on the opposite core. Clearing all bits will cause that core's doorbell interrupt to deassert. Since the usual order of events is for software to send events using DOORBELL_OUT_SET, and acknowledge incoming events by writing to DOORBELL_IN_CLR, this register should be used with caution to avoid race conditions. Reading returns the status of the doorbells currently asserted on the other core, i.e. is equivalent to that core reading its own DOORBELL_IN status.
func (o *SIO_Type) SetDOORBELL_OUT_CLR(value uint32) {
	volatile.StoreUint32(&o.DOORBELL_OUT_CLR.Reg, volatile.LoadUint32(&o.DOORBELL_OUT_CLR.Reg)&^(0xff)|value)
}
func (o *SIO_Type) GetDOORBELL_OUT_CLR() uint32 {
	return volatile.LoadUint32(&o.DOORBELL_OUT_CLR.Reg) & 0xff
}

// SIO.DOORBELL_IN_SET: Write 1s to trigger doorbell interrupts on this core. Read to get status of doorbells currently asserted on this core.
func (o *SIO_Type) SetDOORBELL_IN_SET(value uint32) {
	volatile.StoreUint32(&o.DOORBELL_IN_SET.Reg, volatile.LoadUint32(&o.DOORBELL_IN_SET.Reg)&^(0xff)|value)
}
func (o *SIO_Type) GetDOORBELL_IN_SET() uint32 {
	return volatile.LoadUint32(&o.DOORBELL_IN_SET.Reg) & 0xff
}

// SIO.DOORBELL_IN_CLR: Check and acknowledge doorbells posted to this core. This core's doorbell interrupt is asserted when any bit in this register is 1. Write 1 to each bit to clear that bit. The doorbell interrupt deasserts once all bits are cleared. Read to get status of doorbells currently asserted on this core.
func (o *SIO_Type) SetDOORBELL_IN_CLR(value uint32) {
	volatile.StoreUint32(&o.DOORBELL_IN_CLR.Reg, volatile.LoadUint32(&o.DOORBELL_IN_CLR.Reg)&^(0xff)|value)
}
func (o *SIO_Type) GetDOORBELL_IN_CLR() uint32 {
	return volatile.LoadUint32(&o.DOORBELL_IN_CLR.Reg) & 0xff
}

// SIO.PERI_NONSEC: Detach certain core-local peripherals from Secure SIO, and attach them to Non-secure SIO, so that Non-secure software can use them. Attempting to access one of these peripherals from the Secure SIO when it is attached to the Non-secure SIO, or vice versa, will generate a bus error. This register is per-core, and is only present on the Secure SIO. Most SIO hardware is duplicated across the Secure and Non-secure SIO, so is not listed in this register.
func (o *SIO_Type) SetPERI_NONSEC_TMDS(value uint32) {
	volatile.StoreUint32(&o.PERI_NONSEC.Reg, volatile.LoadUint32(&o.PERI_NONSEC.Reg)&^(0x20)|value<<5)
}
func (o *SIO_Type) GetPERI_NONSEC_TMDS() uint32 {
	return (volatile.LoadUint32(&o.PERI_NONSEC.Reg) & 0x20) >> 5
}
func (o *SIO_Type) SetPERI_NONSEC_INTERP1(value uint32) {
	volatile.StoreUint32(&o.PERI_NONSEC.Reg, volatile.LoadUint32(&o.PERI_NONSEC.Reg)&^(0x2)|value<<1)
}
func (o *SIO_Type) GetPERI_NONSEC_INTERP1() uint32 {
	return (volatile.LoadUint32(&o.PERI_NONSEC.Reg) & 0x2) >> 1
}
func (o *SIO_Type) SetPERI_NONSEC_INTERP0(value uint32) {
	volatile.StoreUint32(&o.PERI_NONSEC.Reg, volatile.LoadUint32(&o.PERI_NONSEC.Reg)&^(0x1)|value)
}
func (o *SIO_Type) GetPERI_NONSEC_INTERP0() uint32 {
	return volatile.LoadUint32(&o.PERI_NONSEC.Reg) & 0x1
}

// SIO.RISCV_SOFTIRQ: Control the assertion of the standard software interrupt (MIP.MSIP) on the RISC-V cores. Unlike the RISC-V timer, this interrupt is not routed to a normal system-level interrupt line, so can not be used by the Arm cores. It is safe for both cores to write to this register on the same cycle. The set/clear effect is accumulated across both cores, and then applied. If a flag is both set and cleared on the same cycle, only the set takes effect.
func (o *SIO_Type) SetRISCV_SOFTIRQ_CORE1_CLR(value uint32) {
	volatile.StoreUint32(&o.RISCV_SOFTIRQ.Reg, volatile.LoadUint32(&o.RISCV_SOFTIRQ.Reg)&^(0x200)|value<<9)
}
func (o *SIO_Type) GetRISCV_SOFTIRQ_CORE1_CLR() uint32 {
	return (volatile.LoadUint32(&o.RISCV_SOFTIRQ.Reg) & 0x200) >> 9
}
func (o *SIO_Type) SetRISCV_SOFTIRQ_CORE0_CLR(value uint32) {
	volatile.StoreUint32(&o.RISCV_SOFTIRQ.Reg, volatile.LoadUint32(&o.RISCV_SOFTIRQ.Reg)&^(0x100)|value<<8)
}
func (o *SIO_Type) GetRISCV_SOFTIRQ_CORE0_CLR() uint32 {
	return (volatile.LoadUint32(&o.RISCV_SOFTIRQ.Reg) & 0x100) >> 8
}
func (o *SIO_Type) SetRISCV_SOFTIRQ_CORE1_SET(value uint32) {
	volatile.StoreUint32(&o.RISCV_SOFTIRQ.Reg, volatile.LoadUint32(&o.RISCV_SOFTIRQ.Reg)&^(0x2)|value<<1)
}
func (o *SIO_Type) GetRISCV_SOFTIRQ_CORE1_SET() uint32 {
	return (volatile.LoadUint32(&o.RISCV_SOFTIRQ.Reg) & 0x2) >> 1
}
func (o *SIO_Type) SetRISCV_SOFTIRQ_CORE0_SET(value uint32) {
	volatile.StoreUint32(&o.RISCV_SOFTIRQ.Reg, volatile.LoadUint32(&o.RISCV_SOFTIRQ.Reg)&^(0x1)|value)
}
func (o *SIO_Type) GetRISCV_SOFTIRQ_CORE0_SET() uint32 {
	return volatile.LoadUint32(&o.RISCV_SOFTIRQ.Reg) & 0x1
}

// SIO.MTIME_CTRL: Control register for the RISC-V 64-bit Machine-mode timer. This timer is only present in the Secure SIO, so is only accessible to an Arm core in Secure mode or a RISC-V core in Machine mode. Note whilst this timer follows the RISC-V privileged specification, it is equally usable by the Arm cores. The interrupts are routed to normal system-level interrupt lines as well as to the MIP.MTIP inputs on the RISC-V cores.
func (o *SIO_Type) SetMTIME_CTRL_DBGPAUSE_CORE1(value uint32) {
	volatile.StoreUint32(&o.MTIME_CTRL.Reg, volatile.LoadUint32(&o.MTIME_CTRL.Reg)&^(0x8)|value<<3)
}
func (o *SIO_Type) GetMTIME_CTRL_DBGPAUSE_CORE1() uint32 {
	return (volatile.LoadUint32(&o.MTIME_CTRL.Reg) & 0x8) >> 3
}
func (o *SIO_Type) SetMTIME_CTRL_DBGPAUSE_CORE0(value uint32) {
	volatile.StoreUint32(&o.MTIME_CTRL.Reg, volatile.LoadUint32(&o.MTIME_CTRL.Reg)&^(0x4)|value<<2)
}
func (o *SIO_Type) GetMTIME_CTRL_DBGPAUSE_CORE0() uint32 {
	return (volatile.LoadUint32(&o.MTIME_CTRL.Reg) & 0x4) >> 2
}
func (o *SIO_Type) SetMTIME_CTRL_FULLSPEED(value uint32) {
	volatile.StoreUint32(&o.MTIME_CTRL.Reg, volatile.LoadUint32(&o.MTIME_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *SIO_Type) GetMTIME_CTRL_FULLSPEED() uint32 {
	return (volatile.LoadUint32(&o.MTIME_CTRL.Reg) & 0x2) >> 1
}
func (o *SIO_Type) SetMTIME_CTRL_EN(value uint32) {
	volatile.StoreUint32(&o.MTIME_CTRL.Reg, volatile.LoadUint32(&o.MTIME_CTRL.Reg)&^(0x1)|value)
}
func (o *SIO_Type) GetMTIME_CTRL_EN() uint32 {
	return volatile.LoadUint32(&o.MTIME_CTRL.Reg) & 0x1
}

// SIO.MTIME: Read/write access to the high half of RISC-V Machine-mode timer. This register is shared between both cores. If both cores write on the same cycle, core 1 takes precedence.
func (o *SIO_Type) SetMTIME(value uint32) {
	volatile.StoreUint32(&o.MTIME.Reg, value)
}
func (o *SIO_Type) GetMTIME() uint32 {
	return volatile.LoadUint32(&o.MTIME.Reg)
}

// SIO.MTIMEH: Read/write access to the high half of RISC-V Machine-mode timer. This register is shared between both cores. If both cores write on the same cycle, core 1 takes precedence.
func (o *SIO_Type) SetMTIMEH(value uint32) {
	volatile.StoreUint32(&o.MTIMEH.Reg, value)
}
func (o *SIO_Type) GetMTIMEH() uint32 {
	return volatile.LoadUint32(&o.MTIMEH.Reg)
}

// SIO.MTIMECMP: Low half of RISC-V Machine-mode timer comparator. This register is core-local, i.e., each core gets a copy of this register, with the comparison result routed to its own interrupt line. The timer interrupt is asserted whenever MTIME is greater than or equal to MTIMECMP. This comparison is unsigned, and performed on the full 64-bit values.
func (o *SIO_Type) SetMTIMECMP(value uint32) {
	volatile.StoreUint32(&o.MTIMECMP.Reg, value)
}
func (o *SIO_Type) GetMTIMECMP() uint32 {
	return volatile.LoadUint32(&o.MTIMECMP.Reg)
}

// SIO.MTIMECMPH: High half of RISC-V Machine-mode timer comparator. This register is core-local. The timer interrupt is asserted whenever MTIME is greater than or equal to MTIMECMP. This comparison is unsigned, and performed on the full 64-bit values.
func (o *SIO_Type) SetMTIMECMPH(value uint32) {
	volatile.StoreUint32(&o.MTIMECMPH.Reg, value)
}
func (o *SIO_Type) GetMTIMECMPH() uint32 {
	return volatile.LoadUint32(&o.MTIMECMPH.Reg)
}

// SIO.TMDS_CTRL: Control register for TMDS encoder.
func (o *SIO_Type) SetTMDS_CTRL_CLEAR_BALANCE(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *SIO_Type) GetTMDS_CTRL_CLEAR_BALANCE() uint32 {
	return (volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0x10000000) >> 28
}
func (o *SIO_Type) SetTMDS_CTRL_PIX2_NOSHIFT(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0x8000000)|value<<27)
}
func (o *SIO_Type) GetTMDS_CTRL_PIX2_NOSHIFT() uint32 {
	return (volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0x8000000) >> 27
}
func (o *SIO_Type) SetTMDS_CTRL_PIX_SHIFT(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0x7000000)|value<<24)
}
func (o *SIO_Type) GetTMDS_CTRL_PIX_SHIFT() uint32 {
	return (volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0x7000000) >> 24
}
func (o *SIO_Type) SetTMDS_CTRL_INTERLEAVE(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0x800000)|value<<23)
}
func (o *SIO_Type) GetTMDS_CTRL_INTERLEAVE() uint32 {
	return (volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0x800000) >> 23
}
func (o *SIO_Type) SetTMDS_CTRL_L2_NBITS(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0x1c0000)|value<<18)
}
func (o *SIO_Type) GetTMDS_CTRL_L2_NBITS() uint32 {
	return (volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0x1c0000) >> 18
}
func (o *SIO_Type) SetTMDS_CTRL_L1_NBITS(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0x38000)|value<<15)
}
func (o *SIO_Type) GetTMDS_CTRL_L1_NBITS() uint32 {
	return (volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0x38000) >> 15
}
func (o *SIO_Type) SetTMDS_CTRL_L0_NBITS(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0x7000)|value<<12)
}
func (o *SIO_Type) GetTMDS_CTRL_L0_NBITS() uint32 {
	return (volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0x7000) >> 12
}
func (o *SIO_Type) SetTMDS_CTRL_L2_ROT(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0xf00)|value<<8)
}
func (o *SIO_Type) GetTMDS_CTRL_L2_ROT() uint32 {
	return (volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0xf00) >> 8
}
func (o *SIO_Type) SetTMDS_CTRL_L1_ROT(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0xf0)|value<<4)
}
func (o *SIO_Type) GetTMDS_CTRL_L1_ROT() uint32 {
	return (volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0xf0) >> 4
}
func (o *SIO_Type) SetTMDS_CTRL_L0_ROT(value uint32) {
	volatile.StoreUint32(&o.TMDS_CTRL.Reg, volatile.LoadUint32(&o.TMDS_CTRL.Reg)&^(0xf)|value)
}
func (o *SIO_Type) GetTMDS_CTRL_L0_ROT() uint32 {
	return volatile.LoadUint32(&o.TMDS_CTRL.Reg) & 0xf
}

// SIO.TMDS_WDATA: Write-only access to the TMDS colour data register.
func (o *SIO_Type) SetTMDS_WDATA(value uint32) {
	volatile.StoreUint32(&o.TMDS_WDATA.Reg, value)
}
func (o *SIO_Type) GetTMDS_WDATA() uint32 {
	return volatile.LoadUint32(&o.TMDS_WDATA.Reg)
}

// SIO.TMDS_PEEK_SINGLE: Get the encoding of one pixel's worth of colour data, packed into a 32-bit value (3x10-bit symbols). The PEEK alias does not shift the colour register when read, but still advances the running DC balance state of each encoder. This is useful for pixel doubling.
func (o *SIO_Type) SetTMDS_PEEK_SINGLE(value uint32) {
	volatile.StoreUint32(&o.TMDS_PEEK_SINGLE.Reg, value)
}
func (o *SIO_Type) GetTMDS_PEEK_SINGLE() uint32 {
	return volatile.LoadUint32(&o.TMDS_PEEK_SINGLE.Reg)
}

// SIO.TMDS_POP_SINGLE: Get the encoding of one pixel's worth of colour data, packed into a 32-bit value. The packing is 5 chunks of 3 lanes times 2 bits (30 bits total). Each chunk contains two bits of a TMDS symbol per lane. This format is intended for shifting out with the HSTX peripheral on RP2350. The POP alias shifts the colour register when read, as well as advancing the running DC balance state of each encoder.
func (o *SIO_Type) SetTMDS_POP_SINGLE(value uint32) {
	volatile.StoreUint32(&o.TMDS_POP_SINGLE.Reg, value)
}
func (o *SIO_Type) GetTMDS_POP_SINGLE() uint32 {
	return volatile.LoadUint32(&o.TMDS_POP_SINGLE.Reg)
}

// SIO.TMDS_PEEK_DOUBLE_L0: Get lane 0 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The PEEK alias does not shift the colour register when read, but still advances the lane 0 DC balance state. This is useful if all 3 lanes' worth of encode are to be read at once, rather than processing the entire scanline for one lane before moving to the next lane.
func (o *SIO_Type) SetTMDS_PEEK_DOUBLE_L0(value uint32) {
	volatile.StoreUint32(&o.TMDS_PEEK_DOUBLE_L0.Reg, value)
}
func (o *SIO_Type) GetTMDS_PEEK_DOUBLE_L0() uint32 {
	return volatile.LoadUint32(&o.TMDS_PEEK_DOUBLE_L0.Reg)
}

// SIO.TMDS_POP_DOUBLE_L0: Get lane 0 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The POP alias shifts the colour register when read, according to the values of PIX_SHIFT and PIX2_NOSHIFT.
func (o *SIO_Type) SetTMDS_POP_DOUBLE_L0(value uint32) {
	volatile.StoreUint32(&o.TMDS_POP_DOUBLE_L0.Reg, value)
}
func (o *SIO_Type) GetTMDS_POP_DOUBLE_L0() uint32 {
	return volatile.LoadUint32(&o.TMDS_POP_DOUBLE_L0.Reg)
}

// SIO.TMDS_PEEK_DOUBLE_L1: Get lane 1 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The PEEK alias does not shift the colour register when read, but still advances the lane 1 DC balance state. This is useful if all 3 lanes' worth of encode are to be read at once, rather than processing the entire scanline for one lane before moving to the next lane.
func (o *SIO_Type) SetTMDS_PEEK_DOUBLE_L1(value uint32) {
	volatile.StoreUint32(&o.TMDS_PEEK_DOUBLE_L1.Reg, value)
}
func (o *SIO_Type) GetTMDS_PEEK_DOUBLE_L1() uint32 {
	return volatile.LoadUint32(&o.TMDS_PEEK_DOUBLE_L1.Reg)
}

// SIO.TMDS_POP_DOUBLE_L1: Get lane 1 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The POP alias shifts the colour register when read, according to the values of PIX_SHIFT and PIX2_NOSHIFT.
func (o *SIO_Type) SetTMDS_POP_DOUBLE_L1(value uint32) {
	volatile.StoreUint32(&o.TMDS_POP_DOUBLE_L1.Reg, value)
}
func (o *SIO_Type) GetTMDS_POP_DOUBLE_L1() uint32 {
	return volatile.LoadUint32(&o.TMDS_POP_DOUBLE_L1.Reg)
}

// SIO.TMDS_PEEK_DOUBLE_L2: Get lane 2 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The PEEK alias does not shift the colour register when read, but still advances the lane 2 DC balance state. This is useful if all 3 lanes' worth of encode are to be read at once, rather than processing the entire scanline for one lane before moving to the next lane.
func (o *SIO_Type) SetTMDS_PEEK_DOUBLE_L2(value uint32) {
	volatile.StoreUint32(&o.TMDS_PEEK_DOUBLE_L2.Reg, value)
}
func (o *SIO_Type) GetTMDS_PEEK_DOUBLE_L2() uint32 {
	return volatile.LoadUint32(&o.TMDS_PEEK_DOUBLE_L2.Reg)
}

// SIO.TMDS_POP_DOUBLE_L2: Get lane 2 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The POP alias shifts the colour register when read, according to the values of PIX_SHIFT and PIX2_NOSHIFT.
func (o *SIO_Type) SetTMDS_POP_DOUBLE_L2(value uint32) {
	volatile.StoreUint32(&o.TMDS_POP_DOUBLE_L2.Reg, value)
}
func (o *SIO_Type) GetTMDS_POP_DOUBLE_L2() uint32 {
	return volatile.LoadUint32(&o.TMDS_POP_DOUBLE_L2.Reg)
}

// Additional registers mapped adjacent to the bootram, for use by the bootrom.
type BOOTRAM_Type struct {
	_             [2048]byte
	WRITE_ONCE0   volatile.Register32 // 0x800
	WRITE_ONCE1   volatile.Register32 // 0x804
	BOOTLOCK_STAT volatile.Register32 // 0x808
	BOOTLOCK0     volatile.Register32 // 0x80C
	BOOTLOCK1     volatile.Register32 // 0x810
	BOOTLOCK2     volatile.Register32 // 0x814
	BOOTLOCK3     volatile.Register32 // 0x818
	BOOTLOCK4     volatile.Register32 // 0x81C
	BOOTLOCK5     volatile.Register32 // 0x820
	BOOTLOCK6     volatile.Register32 // 0x824
	BOOTLOCK7     volatile.Register32 // 0x828
}

// BOOTRAM.WRITE_ONCE0: This registers always ORs writes into its current contents. Once a bit is set, it can only be cleared by a reset.
func (o *BOOTRAM_Type) SetWRITE_ONCE0(value uint32) {
	volatile.StoreUint32(&o.WRITE_ONCE0.Reg, value)
}
func (o *BOOTRAM_Type) GetWRITE_ONCE0() uint32 {
	return volatile.LoadUint32(&o.WRITE_ONCE0.Reg)
}

// BOOTRAM.WRITE_ONCE1: This registers always ORs writes into its current contents. Once a bit is set, it can only be cleared by a reset.
func (o *BOOTRAM_Type) SetWRITE_ONCE1(value uint32) {
	volatile.StoreUint32(&o.WRITE_ONCE1.Reg, value)
}
func (o *BOOTRAM_Type) GetWRITE_ONCE1() uint32 {
	return volatile.LoadUint32(&o.WRITE_ONCE1.Reg)
}

// BOOTRAM.BOOTLOCK_STAT: Bootlock status register. 1=unclaimed, 0=claimed. These locks function identically to the SIO spinlocks, but are reserved for bootrom use.
func (o *BOOTRAM_Type) SetBOOTLOCK_STAT(value uint32) {
	volatile.StoreUint32(&o.BOOTLOCK_STAT.Reg, volatile.LoadUint32(&o.BOOTLOCK_STAT.Reg)&^(0xff)|value)
}
func (o *BOOTRAM_Type) GetBOOTLOCK_STAT() uint32 {
	return volatile.LoadUint32(&o.BOOTLOCK_STAT.Reg) & 0xff
}

// BOOTRAM.BOOTLOCK0: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
func (o *BOOTRAM_Type) SetBOOTLOCK0(value uint32) {
	volatile.StoreUint32(&o.BOOTLOCK0.Reg, value)
}
func (o *BOOTRAM_Type) GetBOOTLOCK0() uint32 {
	return volatile.LoadUint32(&o.BOOTLOCK0.Reg)
}

// BOOTRAM.BOOTLOCK1: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
func (o *BOOTRAM_Type) SetBOOTLOCK1(value uint32) {
	volatile.StoreUint32(&o.BOOTLOCK1.Reg, value)
}
func (o *BOOTRAM_Type) GetBOOTLOCK1() uint32 {
	return volatile.LoadUint32(&o.BOOTLOCK1.Reg)
}

// BOOTRAM.BOOTLOCK2: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
func (o *BOOTRAM_Type) SetBOOTLOCK2(value uint32) {
	volatile.StoreUint32(&o.BOOTLOCK2.Reg, value)
}
func (o *BOOTRAM_Type) GetBOOTLOCK2() uint32 {
	return volatile.LoadUint32(&o.BOOTLOCK2.Reg)
}

// BOOTRAM.BOOTLOCK3: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
func (o *BOOTRAM_Type) SetBOOTLOCK3(value uint32) {
	volatile.StoreUint32(&o.BOOTLOCK3.Reg, value)
}
func (o *BOOTRAM_Type) GetBOOTLOCK3() uint32 {
	return volatile.LoadUint32(&o.BOOTLOCK3.Reg)
}

// BOOTRAM.BOOTLOCK4: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
func (o *BOOTRAM_Type) SetBOOTLOCK4(value uint32) {
	volatile.StoreUint32(&o.BOOTLOCK4.Reg, value)
}
func (o *BOOTRAM_Type) GetBOOTLOCK4() uint32 {
	return volatile.LoadUint32(&o.BOOTLOCK4.Reg)
}

// BOOTRAM.BOOTLOCK5: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
func (o *BOOTRAM_Type) SetBOOTLOCK5(value uint32) {
	volatile.StoreUint32(&o.BOOTLOCK5.Reg, value)
}
func (o *BOOTRAM_Type) GetBOOTLOCK5() uint32 {
	return volatile.LoadUint32(&o.BOOTLOCK5.Reg)
}

// BOOTRAM.BOOTLOCK6: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
func (o *BOOTRAM_Type) SetBOOTLOCK6(value uint32) {
	volatile.StoreUint32(&o.BOOTLOCK6.Reg, value)
}
func (o *BOOTRAM_Type) GetBOOTLOCK6() uint32 {
	return volatile.LoadUint32(&o.BOOTLOCK6.Reg)
}

// BOOTRAM.BOOTLOCK7: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
func (o *BOOTRAM_Type) SetBOOTLOCK7(value uint32) {
	volatile.StoreUint32(&o.BOOTLOCK7.Reg, value)
}
func (o *BOOTRAM_Type) GetBOOTLOCK7() uint32 {
	return volatile.LoadUint32(&o.BOOTLOCK7.Reg)
}

// Coresight block - RP specific registers
type CORESIGHT_TRACE_Type struct {
	CTRL_STATUS        volatile.Register32 // 0x0
	TRACE_CAPTURE_FIFO volatile.Register32 // 0x4
}

// CORESIGHT_TRACE.CTRL_STATUS: Control and status register
func (o *CORESIGHT_TRACE_Type) SetCTRL_STATUS_TRACE_CAPTURE_FIFO_OVERFLOW(value uint32) {
	volatile.StoreUint32(&o.CTRL_STATUS.Reg, volatile.LoadUint32(&o.CTRL_STATUS.Reg)&^(0x2)|value<<1)
}
func (o *CORESIGHT_TRACE_Type) GetCTRL_STATUS_TRACE_CAPTURE_FIFO_OVERFLOW() uint32 {
	return (volatile.LoadUint32(&o.CTRL_STATUS.Reg) & 0x2) >> 1
}
func (o *CORESIGHT_TRACE_Type) SetCTRL_STATUS_TRACE_CAPTURE_FIFO_FLUSH(value uint32) {
	volatile.StoreUint32(&o.CTRL_STATUS.Reg, volatile.LoadUint32(&o.CTRL_STATUS.Reg)&^(0x1)|value)
}
func (o *CORESIGHT_TRACE_Type) GetCTRL_STATUS_TRACE_CAPTURE_FIFO_FLUSH() uint32 {
	return volatile.LoadUint32(&o.CTRL_STATUS.Reg) & 0x1
}

// CORESIGHT_TRACE.TRACE_CAPTURE_FIFO: FIFO for trace data captured from the TPIU
func (o *CORESIGHT_TRACE_Type) SetTRACE_CAPTURE_FIFO(value uint32) {
	volatile.StoreUint32(&o.TRACE_CAPTURE_FIFO.Reg, value)
}
func (o *CORESIGHT_TRACE_Type) GetTRACE_CAPTURE_FIFO() uint32 {
	return volatile.LoadUint32(&o.TRACE_CAPTURE_FIFO.Reg)
}

// USB FS/LS controller device registers
type USB_Type struct {
	ADDR_ENDP              volatile.Register32 // 0x0
	ADDR_ENDP1             volatile.Register32 // 0x4
	ADDR_ENDP2             volatile.Register32 // 0x8
	ADDR_ENDP3             volatile.Register32 // 0xC
	ADDR_ENDP4             volatile.Register32 // 0x10
	ADDR_ENDP5             volatile.Register32 // 0x14
	ADDR_ENDP6             volatile.Register32 // 0x18
	ADDR_ENDP7             volatile.Register32 // 0x1C
	ADDR_ENDP8             volatile.Register32 // 0x20
	ADDR_ENDP9             volatile.Register32 // 0x24
	ADDR_ENDP10            volatile.Register32 // 0x28
	ADDR_ENDP11            volatile.Register32 // 0x2C
	ADDR_ENDP12            volatile.Register32 // 0x30
	ADDR_ENDP13            volatile.Register32 // 0x34
	ADDR_ENDP14            volatile.Register32 // 0x38
	ADDR_ENDP15            volatile.Register32 // 0x3C
	MAIN_CTRL              volatile.Register32 // 0x40
	SOF_WR                 volatile.Register32 // 0x44
	SOF_RD                 volatile.Register32 // 0x48
	SIE_CTRL               volatile.Register32 // 0x4C
	SIE_STATUS             volatile.Register32 // 0x50
	INT_EP_CTRL            volatile.Register32 // 0x54
	BUFF_STATUS            volatile.Register32 // 0x58
	BUFF_CPU_SHOULD_HANDLE volatile.Register32 // 0x5C
	EP_ABORT               volatile.Register32 // 0x60
	EP_ABORT_DONE          volatile.Register32 // 0x64
	EP_STALL_ARM           volatile.Register32 // 0x68
	NAK_POLL               volatile.Register32 // 0x6C
	EP_STATUS_STALL_NAK    volatile.Register32 // 0x70
	USB_MUXING             volatile.Register32 // 0x74
	USB_PWR                volatile.Register32 // 0x78
	USBPHY_DIRECT          volatile.Register32 // 0x7C
	USBPHY_DIRECT_OVERRIDE volatile.Register32 // 0x80
	USBPHY_TRIM            volatile.Register32 // 0x84
	LINESTATE_TUNING       volatile.Register32 // 0x88
	INTR                   volatile.Register32 // 0x8C
	INTE                   volatile.Register32 // 0x90
	INTF                   volatile.Register32 // 0x94
	INTS                   volatile.Register32 // 0x98
	_                      [100]byte
	SOF_TIMESTAMP_RAW      volatile.Register32 // 0x100
	SOF_TIMESTAMP_LAST     volatile.Register32 // 0x104
	SM_STATE               volatile.Register32 // 0x108
	EP_TX_ERROR            volatile.Register32 // 0x10C
	EP_RX_ERROR            volatile.Register32 // 0x110
	DEV_SM_WATCHDOG        volatile.Register32 // 0x114
}

// USB.ADDR_ENDP: Device address and endpoint control
func (o *USB_Type) SetADDR_ENDP_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP.Reg, volatile.LoadUint32(&o.ADDR_ENDP.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP.Reg, volatile.LoadUint32(&o.ADDR_ENDP.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP.Reg) & 0x7f
}

// USB.ADDR_ENDP1: Interrupt endpoint 1. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP1_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP1.Reg, volatile.LoadUint32(&o.ADDR_ENDP1.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP1_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP1.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP1_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP1.Reg, volatile.LoadUint32(&o.ADDR_ENDP1.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP1_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP1.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP1_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP1.Reg, volatile.LoadUint32(&o.ADDR_ENDP1.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP1_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP1.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP1_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP1.Reg, volatile.LoadUint32(&o.ADDR_ENDP1.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP1_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP1.Reg) & 0x7f
}

// USB.ADDR_ENDP2: Interrupt endpoint 2. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP2_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP2.Reg, volatile.LoadUint32(&o.ADDR_ENDP2.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP2_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP2.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP2_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP2.Reg, volatile.LoadUint32(&o.ADDR_ENDP2.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP2_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP2.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP2_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP2.Reg, volatile.LoadUint32(&o.ADDR_ENDP2.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP2_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP2.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP2_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP2.Reg, volatile.LoadUint32(&o.ADDR_ENDP2.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP2_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP2.Reg) & 0x7f
}

// USB.ADDR_ENDP3: Interrupt endpoint 3. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP3_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP3.Reg, volatile.LoadUint32(&o.ADDR_ENDP3.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP3_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP3.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP3_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP3.Reg, volatile.LoadUint32(&o.ADDR_ENDP3.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP3_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP3.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP3_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP3.Reg, volatile.LoadUint32(&o.ADDR_ENDP3.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP3_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP3.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP3_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP3.Reg, volatile.LoadUint32(&o.ADDR_ENDP3.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP3_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP3.Reg) & 0x7f
}

// USB.ADDR_ENDP4: Interrupt endpoint 4. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP4_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP4.Reg, volatile.LoadUint32(&o.ADDR_ENDP4.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP4_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP4.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP4_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP4.Reg, volatile.LoadUint32(&o.ADDR_ENDP4.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP4_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP4.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP4_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP4.Reg, volatile.LoadUint32(&o.ADDR_ENDP4.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP4_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP4.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP4_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP4.Reg, volatile.LoadUint32(&o.ADDR_ENDP4.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP4_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP4.Reg) & 0x7f
}

// USB.ADDR_ENDP5: Interrupt endpoint 5. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP5_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP5.Reg, volatile.LoadUint32(&o.ADDR_ENDP5.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP5_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP5.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP5_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP5.Reg, volatile.LoadUint32(&o.ADDR_ENDP5.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP5_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP5.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP5_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP5.Reg, volatile.LoadUint32(&o.ADDR_ENDP5.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP5_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP5.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP5_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP5.Reg, volatile.LoadUint32(&o.ADDR_ENDP5.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP5_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP5.Reg) & 0x7f
}

// USB.ADDR_ENDP6: Interrupt endpoint 6. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP6_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP6.Reg, volatile.LoadUint32(&o.ADDR_ENDP6.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP6_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP6.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP6_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP6.Reg, volatile.LoadUint32(&o.ADDR_ENDP6.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP6_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP6.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP6_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP6.Reg, volatile.LoadUint32(&o.ADDR_ENDP6.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP6_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP6.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP6_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP6.Reg, volatile.LoadUint32(&o.ADDR_ENDP6.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP6_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP6.Reg) & 0x7f
}

// USB.ADDR_ENDP7: Interrupt endpoint 7. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP7_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP7.Reg, volatile.LoadUint32(&o.ADDR_ENDP7.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP7_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP7.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP7_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP7.Reg, volatile.LoadUint32(&o.ADDR_ENDP7.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP7_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP7.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP7_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP7.Reg, volatile.LoadUint32(&o.ADDR_ENDP7.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP7_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP7.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP7_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP7.Reg, volatile.LoadUint32(&o.ADDR_ENDP7.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP7_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP7.Reg) & 0x7f
}

// USB.ADDR_ENDP8: Interrupt endpoint 8. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP8_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP8.Reg, volatile.LoadUint32(&o.ADDR_ENDP8.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP8_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP8.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP8_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP8.Reg, volatile.LoadUint32(&o.ADDR_ENDP8.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP8_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP8.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP8_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP8.Reg, volatile.LoadUint32(&o.ADDR_ENDP8.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP8_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP8.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP8_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP8.Reg, volatile.LoadUint32(&o.ADDR_ENDP8.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP8_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP8.Reg) & 0x7f
}

// USB.ADDR_ENDP9: Interrupt endpoint 9. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP9_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP9.Reg, volatile.LoadUint32(&o.ADDR_ENDP9.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP9_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP9.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP9_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP9.Reg, volatile.LoadUint32(&o.ADDR_ENDP9.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP9_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP9.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP9_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP9.Reg, volatile.LoadUint32(&o.ADDR_ENDP9.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP9_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP9.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP9_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP9.Reg, volatile.LoadUint32(&o.ADDR_ENDP9.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP9_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP9.Reg) & 0x7f
}

// USB.ADDR_ENDP10: Interrupt endpoint 10. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP10_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP10.Reg, volatile.LoadUint32(&o.ADDR_ENDP10.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP10_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP10.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP10_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP10.Reg, volatile.LoadUint32(&o.ADDR_ENDP10.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP10_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP10.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP10_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP10.Reg, volatile.LoadUint32(&o.ADDR_ENDP10.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP10_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP10.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP10_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP10.Reg, volatile.LoadUint32(&o.ADDR_ENDP10.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP10_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP10.Reg) & 0x7f
}

// USB.ADDR_ENDP11: Interrupt endpoint 11. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP11_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP11.Reg, volatile.LoadUint32(&o.ADDR_ENDP11.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP11_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP11.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP11_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP11.Reg, volatile.LoadUint32(&o.ADDR_ENDP11.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP11_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP11.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP11_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP11.Reg, volatile.LoadUint32(&o.ADDR_ENDP11.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP11_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP11.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP11_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP11.Reg, volatile.LoadUint32(&o.ADDR_ENDP11.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP11_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP11.Reg) & 0x7f
}

// USB.ADDR_ENDP12: Interrupt endpoint 12. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP12_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP12.Reg, volatile.LoadUint32(&o.ADDR_ENDP12.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP12_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP12.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP12_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP12.Reg, volatile.LoadUint32(&o.ADDR_ENDP12.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP12_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP12.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP12_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP12.Reg, volatile.LoadUint32(&o.ADDR_ENDP12.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP12_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP12.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP12_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP12.Reg, volatile.LoadUint32(&o.ADDR_ENDP12.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP12_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP12.Reg) & 0x7f
}

// USB.ADDR_ENDP13: Interrupt endpoint 13. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP13_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP13.Reg, volatile.LoadUint32(&o.ADDR_ENDP13.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP13_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP13.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP13_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP13.Reg, volatile.LoadUint32(&o.ADDR_ENDP13.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP13_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP13.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP13_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP13.Reg, volatile.LoadUint32(&o.ADDR_ENDP13.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP13_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP13.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP13_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP13.Reg, volatile.LoadUint32(&o.ADDR_ENDP13.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP13_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP13.Reg) & 0x7f
}

// USB.ADDR_ENDP14: Interrupt endpoint 14. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP14_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP14.Reg, volatile.LoadUint32(&o.ADDR_ENDP14.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP14_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP14.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP14_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP14.Reg, volatile.LoadUint32(&o.ADDR_ENDP14.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP14_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP14.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP14_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP14.Reg, volatile.LoadUint32(&o.ADDR_ENDP14.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP14_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP14.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP14_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP14.Reg, volatile.LoadUint32(&o.ADDR_ENDP14.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP14_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP14.Reg) & 0x7f
}

// USB.ADDR_ENDP15: Interrupt endpoint 15. Only valid for HOST mode.
func (o *USB_Type) SetADDR_ENDP15_INTEP_PREAMBLE(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP15.Reg, volatile.LoadUint32(&o.ADDR_ENDP15.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetADDR_ENDP15_INTEP_PREAMBLE() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP15.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetADDR_ENDP15_INTEP_DIR(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP15.Reg, volatile.LoadUint32(&o.ADDR_ENDP15.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetADDR_ENDP15_INTEP_DIR() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP15.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetADDR_ENDP15_ENDPOINT(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP15.Reg, volatile.LoadUint32(&o.ADDR_ENDP15.Reg)&^(0xf0000)|value<<16)
}
func (o *USB_Type) GetADDR_ENDP15_ENDPOINT() uint32 {
	return (volatile.LoadUint32(&o.ADDR_ENDP15.Reg) & 0xf0000) >> 16
}
func (o *USB_Type) SetADDR_ENDP15_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.ADDR_ENDP15.Reg, volatile.LoadUint32(&o.ADDR_ENDP15.Reg)&^(0x7f)|value)
}
func (o *USB_Type) GetADDR_ENDP15_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.ADDR_ENDP15.Reg) & 0x7f
}

// USB.MAIN_CTRL: Main control register
func (o *USB_Type) SetMAIN_CTRL_SIM_TIMING(value uint32) {
	volatile.StoreUint32(&o.MAIN_CTRL.Reg, volatile.LoadUint32(&o.MAIN_CTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetMAIN_CTRL_SIM_TIMING() uint32 {
	return (volatile.LoadUint32(&o.MAIN_CTRL.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetMAIN_CTRL_PHY_ISO(value uint32) {
	volatile.StoreUint32(&o.MAIN_CTRL.Reg, volatile.LoadUint32(&o.MAIN_CTRL.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetMAIN_CTRL_PHY_ISO() uint32 {
	return (volatile.LoadUint32(&o.MAIN_CTRL.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetMAIN_CTRL_HOST_NDEVICE(value uint32) {
	volatile.StoreUint32(&o.MAIN_CTRL.Reg, volatile.LoadUint32(&o.MAIN_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetMAIN_CTRL_HOST_NDEVICE() uint32 {
	return (volatile.LoadUint32(&o.MAIN_CTRL.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetMAIN_CTRL_CONTROLLER_EN(value uint32) {
	volatile.StoreUint32(&o.MAIN_CTRL.Reg, volatile.LoadUint32(&o.MAIN_CTRL.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetMAIN_CTRL_CONTROLLER_EN() uint32 {
	return volatile.LoadUint32(&o.MAIN_CTRL.Reg) & 0x1
}

// USB.SOF_WR: Set the SOF (Start of Frame) frame number in the host controller. The SOF packet is sent every 1ms and the host will increment the frame number by 1 each time.
func (o *USB_Type) SetSOF_WR_COUNT(value uint32) {
	volatile.StoreUint32(&o.SOF_WR.Reg, volatile.LoadUint32(&o.SOF_WR.Reg)&^(0x7ff)|value)
}
func (o *USB_Type) GetSOF_WR_COUNT() uint32 {
	return volatile.LoadUint32(&o.SOF_WR.Reg) & 0x7ff
}

// USB.SOF_RD: Read the last SOF (Start of Frame) frame number seen. In device mode the last SOF received from the host. In host mode the last SOF sent by the host.
func (o *USB_Type) SetSOF_RD_COUNT(value uint32) {
	volatile.StoreUint32(&o.SOF_RD.Reg, volatile.LoadUint32(&o.SOF_RD.Reg)&^(0x7ff)|value)
}
func (o *USB_Type) GetSOF_RD_COUNT() uint32 {
	return volatile.LoadUint32(&o.SOF_RD.Reg) & 0x7ff
}

// USB.SIE_CTRL: SIE control register
func (o *USB_Type) SetSIE_CTRL_EP0_INT_STALL(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetSIE_CTRL_EP0_INT_STALL() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetSIE_CTRL_EP0_DOUBLE_BUF(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_Type) GetSIE_CTRL_EP0_DOUBLE_BUF() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x40000000) >> 30
}
func (o *USB_Type) SetSIE_CTRL_EP0_INT_1BUF(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_Type) GetSIE_CTRL_EP0_INT_1BUF() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x20000000) >> 29
}
func (o *USB_Type) SetSIE_CTRL_EP0_INT_2BUF(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_Type) GetSIE_CTRL_EP0_INT_2BUF() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x10000000) >> 28
}
func (o *USB_Type) SetSIE_CTRL_EP0_INT_NAK(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x8000000)|value<<27)
}
func (o *USB_Type) GetSIE_CTRL_EP0_INT_NAK() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x8000000) >> 27
}
func (o *USB_Type) SetSIE_CTRL_DIRECT_EN(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetSIE_CTRL_DIRECT_EN() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetSIE_CTRL_DIRECT_DP(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetSIE_CTRL_DIRECT_DP() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetSIE_CTRL_DIRECT_DM(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x1000000)|value<<24)
}
func (o *USB_Type) GetSIE_CTRL_DIRECT_DM() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x1000000) >> 24
}
func (o *USB_Type) SetSIE_CTRL_EP0_STOP_ON_SHORT_PACKET(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetSIE_CTRL_EP0_STOP_ON_SHORT_PACKET() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetSIE_CTRL_TRANSCEIVER_PD(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetSIE_CTRL_TRANSCEIVER_PD() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetSIE_CTRL_RPU_OPT(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetSIE_CTRL_RPU_OPT() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetSIE_CTRL_PULLUP_EN(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetSIE_CTRL_PULLUP_EN() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetSIE_CTRL_PULLDOWN_EN(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetSIE_CTRL_PULLDOWN_EN() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetSIE_CTRL_RESET_BUS(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetSIE_CTRL_RESET_BUS() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetSIE_CTRL_RESUME(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetSIE_CTRL_RESUME() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetSIE_CTRL_VBUS_EN(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetSIE_CTRL_VBUS_EN() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetSIE_CTRL_KEEP_ALIVE_EN(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetSIE_CTRL_KEEP_ALIVE_EN() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetSIE_CTRL_SOF_EN(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetSIE_CTRL_SOF_EN() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetSIE_CTRL_SOF_SYNC(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetSIE_CTRL_SOF_SYNC() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetSIE_CTRL_PREAMBLE_EN(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetSIE_CTRL_PREAMBLE_EN() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetSIE_CTRL_STOP_TRANS(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetSIE_CTRL_STOP_TRANS() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetSIE_CTRL_RECEIVE_DATA(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetSIE_CTRL_RECEIVE_DATA() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetSIE_CTRL_SEND_DATA(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetSIE_CTRL_SEND_DATA() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetSIE_CTRL_SEND_SETUP(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetSIE_CTRL_SEND_SETUP() uint32 {
	return (volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetSIE_CTRL_START_TRANS(value uint32) {
	volatile.StoreUint32(&o.SIE_CTRL.Reg, volatile.LoadUint32(&o.SIE_CTRL.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetSIE_CTRL_START_TRANS() uint32 {
	return volatile.LoadUint32(&o.SIE_CTRL.Reg) & 0x1
}

// USB.SIE_STATUS: SIE status register
func (o *USB_Type) SetSIE_STATUS_DATA_SEQ_ERROR(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetSIE_STATUS_DATA_SEQ_ERROR() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetSIE_STATUS_ACK_REC(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_Type) GetSIE_STATUS_ACK_REC() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x40000000) >> 30
}
func (o *USB_Type) SetSIE_STATUS_STALL_REC(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_Type) GetSIE_STATUS_STALL_REC() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x20000000) >> 29
}
func (o *USB_Type) SetSIE_STATUS_NAK_REC(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_Type) GetSIE_STATUS_NAK_REC() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x10000000) >> 28
}
func (o *USB_Type) SetSIE_STATUS_RX_TIMEOUT(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x8000000)|value<<27)
}
func (o *USB_Type) GetSIE_STATUS_RX_TIMEOUT() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x8000000) >> 27
}
func (o *USB_Type) SetSIE_STATUS_RX_OVERFLOW(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetSIE_STATUS_RX_OVERFLOW() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetSIE_STATUS_BIT_STUFF_ERROR(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetSIE_STATUS_BIT_STUFF_ERROR() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetSIE_STATUS_CRC_ERROR(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x1000000)|value<<24)
}
func (o *USB_Type) GetSIE_STATUS_CRC_ERROR() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x1000000) >> 24
}
func (o *USB_Type) SetSIE_STATUS_ENDPOINT_ERROR(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetSIE_STATUS_ENDPOINT_ERROR() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetSIE_STATUS_BUS_RESET(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetSIE_STATUS_BUS_RESET() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetSIE_STATUS_TRANS_COMPLETE(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetSIE_STATUS_TRANS_COMPLETE() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetSIE_STATUS_SETUP_REC(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetSIE_STATUS_SETUP_REC() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetSIE_STATUS_CONNECTED(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetSIE_STATUS_CONNECTED() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetSIE_STATUS_RX_SHORT_PACKET(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetSIE_STATUS_RX_SHORT_PACKET() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetSIE_STATUS_RESUME(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetSIE_STATUS_RESUME() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetSIE_STATUS_VBUS_OVER_CURR(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetSIE_STATUS_VBUS_OVER_CURR() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetSIE_STATUS_SPEED(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x300)|value<<8)
}
func (o *USB_Type) GetSIE_STATUS_SPEED() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x300) >> 8
}
func (o *USB_Type) SetSIE_STATUS_SUSPENDED(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetSIE_STATUS_SUSPENDED() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetSIE_STATUS_LINE_STATE(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0xc)|value<<2)
}
func (o *USB_Type) GetSIE_STATUS_LINE_STATE() uint32 {
	return (volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0xc) >> 2
}
func (o *USB_Type) SetSIE_STATUS_VBUS_DETECTED(value uint32) {
	volatile.StoreUint32(&o.SIE_STATUS.Reg, volatile.LoadUint32(&o.SIE_STATUS.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetSIE_STATUS_VBUS_DETECTED() uint32 {
	return volatile.LoadUint32(&o.SIE_STATUS.Reg) & 0x1
}

// USB.INT_EP_CTRL: interrupt endpoint control register
func (o *USB_Type) SetINT_EP_CTRL_INT_EP_ACTIVE(value uint32) {
	volatile.StoreUint32(&o.INT_EP_CTRL.Reg, volatile.LoadUint32(&o.INT_EP_CTRL.Reg)&^(0xfffe)|value<<1)
}
func (o *USB_Type) GetINT_EP_CTRL_INT_EP_ACTIVE() uint32 {
	return (volatile.LoadUint32(&o.INT_EP_CTRL.Reg) & 0xfffe) >> 1
}

// USB.BUFF_STATUS: Buffer status register. A bit set here indicates that a buffer has completed on the endpoint (if the buffer interrupt is enabled). It is possible for 2 buffers to be completed, so clearing the buffer status bit may instantly re set it on the next clock cycle.
func (o *USB_Type) SetBUFF_STATUS_EP15_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetBUFF_STATUS_EP15_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetBUFF_STATUS_EP15_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_Type) GetBUFF_STATUS_EP15_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x40000000) >> 30
}
func (o *USB_Type) SetBUFF_STATUS_EP14_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_Type) GetBUFF_STATUS_EP14_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x20000000) >> 29
}
func (o *USB_Type) SetBUFF_STATUS_EP14_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_Type) GetBUFF_STATUS_EP14_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x10000000) >> 28
}
func (o *USB_Type) SetBUFF_STATUS_EP13_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x8000000)|value<<27)
}
func (o *USB_Type) GetBUFF_STATUS_EP13_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x8000000) >> 27
}
func (o *USB_Type) SetBUFF_STATUS_EP13_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetBUFF_STATUS_EP13_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetBUFF_STATUS_EP12_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetBUFF_STATUS_EP12_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetBUFF_STATUS_EP12_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x1000000)|value<<24)
}
func (o *USB_Type) GetBUFF_STATUS_EP12_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x1000000) >> 24
}
func (o *USB_Type) SetBUFF_STATUS_EP11_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetBUFF_STATUS_EP11_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetBUFF_STATUS_EP11_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetBUFF_STATUS_EP11_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetBUFF_STATUS_EP10_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetBUFF_STATUS_EP10_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetBUFF_STATUS_EP10_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetBUFF_STATUS_EP10_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetBUFF_STATUS_EP9_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetBUFF_STATUS_EP9_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetBUFF_STATUS_EP9_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetBUFF_STATUS_EP9_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetBUFF_STATUS_EP8_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetBUFF_STATUS_EP8_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetBUFF_STATUS_EP8_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetBUFF_STATUS_EP8_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetBUFF_STATUS_EP7_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetBUFF_STATUS_EP7_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetBUFF_STATUS_EP7_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetBUFF_STATUS_EP7_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetBUFF_STATUS_EP6_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetBUFF_STATUS_EP6_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetBUFF_STATUS_EP6_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetBUFF_STATUS_EP6_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetBUFF_STATUS_EP5_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetBUFF_STATUS_EP5_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetBUFF_STATUS_EP5_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetBUFF_STATUS_EP5_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetBUFF_STATUS_EP4_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetBUFF_STATUS_EP4_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetBUFF_STATUS_EP4_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetBUFF_STATUS_EP4_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetBUFF_STATUS_EP3_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetBUFF_STATUS_EP3_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetBUFF_STATUS_EP3_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetBUFF_STATUS_EP3_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetBUFF_STATUS_EP2_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetBUFF_STATUS_EP2_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetBUFF_STATUS_EP2_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetBUFF_STATUS_EP2_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetBUFF_STATUS_EP1_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetBUFF_STATUS_EP1_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetBUFF_STATUS_EP1_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetBUFF_STATUS_EP1_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetBUFF_STATUS_EP0_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetBUFF_STATUS_EP0_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetBUFF_STATUS_EP0_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_STATUS.Reg, volatile.LoadUint32(&o.BUFF_STATUS.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetBUFF_STATUS_EP0_IN() uint32 {
	return volatile.LoadUint32(&o.BUFF_STATUS.Reg) & 0x1
}

// USB.BUFF_CPU_SHOULD_HANDLE: Which of the double buffers should be handled. Only valid if using an interrupt per buffer (i.e. not per 2 buffers). Not valid for host interrupt endpoint polling because they are only single buffered.
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP15_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP15_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP15_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP15_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x40000000) >> 30
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP14_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP14_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x20000000) >> 29
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP14_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP14_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x10000000) >> 28
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP13_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x8000000)|value<<27)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP13_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x8000000) >> 27
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP13_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP13_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP12_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP12_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP12_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x1000000)|value<<24)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP12_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x1000000) >> 24
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP11_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP11_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP11_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP11_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP10_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP10_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP10_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP10_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP9_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP9_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP9_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP9_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP8_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP8_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP8_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP8_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP7_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP7_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP7_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP7_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP6_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP6_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP6_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP6_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP5_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP5_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP5_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP5_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP4_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP4_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP4_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP4_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP3_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP3_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP3_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP3_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP2_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP2_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP2_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP2_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP1_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP1_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP1_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP1_IN() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP0_OUT(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP0_OUT() uint32 {
	return (volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetBUFF_CPU_SHOULD_HANDLE_EP0_IN(value uint32) {
	volatile.StoreUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg, volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetBUFF_CPU_SHOULD_HANDLE_EP0_IN() uint32 {
	return volatile.LoadUint32(&o.BUFF_CPU_SHOULD_HANDLE.Reg) & 0x1
}

// USB.EP_ABORT: Device only: Can be set to ignore the buffer control register for this endpoint in case you would like to revoke a buffer. A NAK will be sent for every access to the endpoint until this bit is cleared. A corresponding bit in `EP_ABORT_DONE` is set when it is safe to modify the buffer control register.
func (o *USB_Type) SetEP_ABORT_EP15_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetEP_ABORT_EP15_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetEP_ABORT_EP15_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_Type) GetEP_ABORT_EP15_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x40000000) >> 30
}
func (o *USB_Type) SetEP_ABORT_EP14_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_Type) GetEP_ABORT_EP14_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x20000000) >> 29
}
func (o *USB_Type) SetEP_ABORT_EP14_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_Type) GetEP_ABORT_EP14_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x10000000) >> 28
}
func (o *USB_Type) SetEP_ABORT_EP13_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x8000000)|value<<27)
}
func (o *USB_Type) GetEP_ABORT_EP13_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x8000000) >> 27
}
func (o *USB_Type) SetEP_ABORT_EP13_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetEP_ABORT_EP13_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetEP_ABORT_EP12_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetEP_ABORT_EP12_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetEP_ABORT_EP12_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x1000000)|value<<24)
}
func (o *USB_Type) GetEP_ABORT_EP12_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x1000000) >> 24
}
func (o *USB_Type) SetEP_ABORT_EP11_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetEP_ABORT_EP11_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetEP_ABORT_EP11_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetEP_ABORT_EP11_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetEP_ABORT_EP10_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetEP_ABORT_EP10_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetEP_ABORT_EP10_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetEP_ABORT_EP10_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetEP_ABORT_EP9_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetEP_ABORT_EP9_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetEP_ABORT_EP9_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetEP_ABORT_EP9_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetEP_ABORT_EP8_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetEP_ABORT_EP8_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetEP_ABORT_EP8_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetEP_ABORT_EP8_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetEP_ABORT_EP7_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetEP_ABORT_EP7_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetEP_ABORT_EP7_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetEP_ABORT_EP7_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetEP_ABORT_EP6_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetEP_ABORT_EP6_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetEP_ABORT_EP6_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetEP_ABORT_EP6_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetEP_ABORT_EP5_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetEP_ABORT_EP5_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetEP_ABORT_EP5_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetEP_ABORT_EP5_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetEP_ABORT_EP4_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetEP_ABORT_EP4_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetEP_ABORT_EP4_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetEP_ABORT_EP4_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetEP_ABORT_EP3_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetEP_ABORT_EP3_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetEP_ABORT_EP3_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetEP_ABORT_EP3_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetEP_ABORT_EP2_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetEP_ABORT_EP2_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetEP_ABORT_EP2_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetEP_ABORT_EP2_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetEP_ABORT_EP1_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetEP_ABORT_EP1_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetEP_ABORT_EP1_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetEP_ABORT_EP1_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetEP_ABORT_EP0_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetEP_ABORT_EP0_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetEP_ABORT_EP0_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT.Reg, volatile.LoadUint32(&o.EP_ABORT.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetEP_ABORT_EP0_IN() uint32 {
	return volatile.LoadUint32(&o.EP_ABORT.Reg) & 0x1
}

// USB.EP_ABORT_DONE: Device only: Used in conjunction with `EP_ABORT`. Set once an endpoint is idle so the programmer knows it is safe to modify the buffer control register.
func (o *USB_Type) SetEP_ABORT_DONE_EP15_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP15_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetEP_ABORT_DONE_EP15_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP15_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x40000000) >> 30
}
func (o *USB_Type) SetEP_ABORT_DONE_EP14_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP14_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x20000000) >> 29
}
func (o *USB_Type) SetEP_ABORT_DONE_EP14_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP14_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x10000000) >> 28
}
func (o *USB_Type) SetEP_ABORT_DONE_EP13_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x8000000)|value<<27)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP13_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x8000000) >> 27
}
func (o *USB_Type) SetEP_ABORT_DONE_EP13_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP13_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetEP_ABORT_DONE_EP12_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP12_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetEP_ABORT_DONE_EP12_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x1000000)|value<<24)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP12_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x1000000) >> 24
}
func (o *USB_Type) SetEP_ABORT_DONE_EP11_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP11_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetEP_ABORT_DONE_EP11_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP11_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetEP_ABORT_DONE_EP10_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP10_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetEP_ABORT_DONE_EP10_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP10_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetEP_ABORT_DONE_EP9_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP9_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetEP_ABORT_DONE_EP9_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP9_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetEP_ABORT_DONE_EP8_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP8_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetEP_ABORT_DONE_EP8_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP8_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetEP_ABORT_DONE_EP7_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP7_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetEP_ABORT_DONE_EP7_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP7_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetEP_ABORT_DONE_EP6_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP6_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetEP_ABORT_DONE_EP6_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP6_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetEP_ABORT_DONE_EP5_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP5_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetEP_ABORT_DONE_EP5_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP5_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetEP_ABORT_DONE_EP4_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP4_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetEP_ABORT_DONE_EP4_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP4_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetEP_ABORT_DONE_EP3_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP3_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetEP_ABORT_DONE_EP3_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP3_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetEP_ABORT_DONE_EP2_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP2_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetEP_ABORT_DONE_EP2_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP2_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetEP_ABORT_DONE_EP1_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP1_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetEP_ABORT_DONE_EP1_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP1_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetEP_ABORT_DONE_EP0_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP0_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetEP_ABORT_DONE_EP0_IN(value uint32) {
	volatile.StoreUint32(&o.EP_ABORT_DONE.Reg, volatile.LoadUint32(&o.EP_ABORT_DONE.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetEP_ABORT_DONE_EP0_IN() uint32 {
	return volatile.LoadUint32(&o.EP_ABORT_DONE.Reg) & 0x1
}

// USB.EP_STALL_ARM: Device: this bit must be set in conjunction with the `STALL` bit in the buffer control register to send a STALL on EP0. The device controller clears these bits when a SETUP packet is received because the USB spec requires that a STALL condition is cleared when a SETUP packet is received.
func (o *USB_Type) SetEP_STALL_ARM_EP0_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STALL_ARM.Reg, volatile.LoadUint32(&o.EP_STALL_ARM.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetEP_STALL_ARM_EP0_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STALL_ARM.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetEP_STALL_ARM_EP0_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STALL_ARM.Reg, volatile.LoadUint32(&o.EP_STALL_ARM.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetEP_STALL_ARM_EP0_IN() uint32 {
	return volatile.LoadUint32(&o.EP_STALL_ARM.Reg) & 0x1
}

// USB.NAK_POLL: Used by the host controller. Sets the wait time in microseconds before trying again if the device replies with a NAK.
func (o *USB_Type) SetNAK_POLL_RETRY_COUNT_HI(value uint32) {
	volatile.StoreUint32(&o.NAK_POLL.Reg, volatile.LoadUint32(&o.NAK_POLL.Reg)&^(0xf0000000)|value<<28)
}
func (o *USB_Type) GetNAK_POLL_RETRY_COUNT_HI() uint32 {
	return (volatile.LoadUint32(&o.NAK_POLL.Reg) & 0xf0000000) >> 28
}
func (o *USB_Type) SetNAK_POLL_EPX_STOPPED_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.NAK_POLL.Reg, volatile.LoadUint32(&o.NAK_POLL.Reg)&^(0x8000000)|value<<27)
}
func (o *USB_Type) GetNAK_POLL_EPX_STOPPED_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.NAK_POLL.Reg) & 0x8000000) >> 27
}
func (o *USB_Type) SetNAK_POLL_STOP_EPX_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.NAK_POLL.Reg, volatile.LoadUint32(&o.NAK_POLL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetNAK_POLL_STOP_EPX_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.NAK_POLL.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetNAK_POLL_DELAY_FS(value uint32) {
	volatile.StoreUint32(&o.NAK_POLL.Reg, volatile.LoadUint32(&o.NAK_POLL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_Type) GetNAK_POLL_DELAY_FS() uint32 {
	return (volatile.LoadUint32(&o.NAK_POLL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_Type) SetNAK_POLL_RETRY_COUNT_LO(value uint32) {
	volatile.StoreUint32(&o.NAK_POLL.Reg, volatile.LoadUint32(&o.NAK_POLL.Reg)&^(0xfc00)|value<<10)
}
func (o *USB_Type) GetNAK_POLL_RETRY_COUNT_LO() uint32 {
	return (volatile.LoadUint32(&o.NAK_POLL.Reg) & 0xfc00) >> 10
}
func (o *USB_Type) SetNAK_POLL_DELAY_LS(value uint32) {
	volatile.StoreUint32(&o.NAK_POLL.Reg, volatile.LoadUint32(&o.NAK_POLL.Reg)&^(0x3ff)|value)
}
func (o *USB_Type) GetNAK_POLL_DELAY_LS() uint32 {
	return volatile.LoadUint32(&o.NAK_POLL.Reg) & 0x3ff
}

// USB.EP_STATUS_STALL_NAK: Device: bits are set when the `IRQ_ON_NAK` or `IRQ_ON_STALL` bits are set. For EP0 this comes from `SIE_CTRL`. For all other endpoints it comes from the endpoint control register.
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP15_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP15_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP15_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP15_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x40000000) >> 30
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP14_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP14_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x20000000) >> 29
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP14_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP14_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x10000000) >> 28
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP13_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x8000000)|value<<27)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP13_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x8000000) >> 27
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP13_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP13_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP12_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP12_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP12_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x1000000)|value<<24)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP12_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x1000000) >> 24
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP11_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP11_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP11_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP11_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP10_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP10_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP10_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP10_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP9_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP9_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP9_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP9_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP8_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP8_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP8_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP8_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP7_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP7_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP7_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP7_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP6_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP6_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP6_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP6_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP5_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP5_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP5_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP5_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP4_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP4_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP4_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP4_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP3_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP3_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP3_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP3_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP2_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP2_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP2_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP2_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP1_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP1_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP1_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP1_IN() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP0_OUT(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP0_OUT() uint32 {
	return (volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetEP_STATUS_STALL_NAK_EP0_IN(value uint32) {
	volatile.StoreUint32(&o.EP_STATUS_STALL_NAK.Reg, volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetEP_STATUS_STALL_NAK_EP0_IN() uint32 {
	return volatile.LoadUint32(&o.EP_STATUS_STALL_NAK.Reg) & 0x1
}

// USB.USB_MUXING: Where to connect the USB controller. Should be to_phy by default.
func (o *USB_Type) SetUSB_MUXING_SWAP_DPDM(value uint32) {
	volatile.StoreUint32(&o.USB_MUXING.Reg, volatile.LoadUint32(&o.USB_MUXING.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetUSB_MUXING_SWAP_DPDM() uint32 {
	return (volatile.LoadUint32(&o.USB_MUXING.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetUSB_MUXING_USBPHY_AS_GPIO(value uint32) {
	volatile.StoreUint32(&o.USB_MUXING.Reg, volatile.LoadUint32(&o.USB_MUXING.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetUSB_MUXING_USBPHY_AS_GPIO() uint32 {
	return (volatile.LoadUint32(&o.USB_MUXING.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetUSB_MUXING_SOFTCON(value uint32) {
	volatile.StoreUint32(&o.USB_MUXING.Reg, volatile.LoadUint32(&o.USB_MUXING.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetUSB_MUXING_SOFTCON() uint32 {
	return (volatile.LoadUint32(&o.USB_MUXING.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetUSB_MUXING_TO_DIGITAL_PAD(value uint32) {
	volatile.StoreUint32(&o.USB_MUXING.Reg, volatile.LoadUint32(&o.USB_MUXING.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetUSB_MUXING_TO_DIGITAL_PAD() uint32 {
	return (volatile.LoadUint32(&o.USB_MUXING.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetUSB_MUXING_TO_EXTPHY(value uint32) {
	volatile.StoreUint32(&o.USB_MUXING.Reg, volatile.LoadUint32(&o.USB_MUXING.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetUSB_MUXING_TO_EXTPHY() uint32 {
	return (volatile.LoadUint32(&o.USB_MUXING.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetUSB_MUXING_TO_PHY(value uint32) {
	volatile.StoreUint32(&o.USB_MUXING.Reg, volatile.LoadUint32(&o.USB_MUXING.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetUSB_MUXING_TO_PHY() uint32 {
	return volatile.LoadUint32(&o.USB_MUXING.Reg) & 0x1
}

// USB.USB_PWR: Overrides for the power signals in the event that the VBUS signals are not hooked up to GPIO. Set the value of the override and then the override enable to switch over to the override value.
func (o *USB_Type) SetUSB_PWR_OVERCURR_DETECT_EN(value uint32) {
	volatile.StoreUint32(&o.USB_PWR.Reg, volatile.LoadUint32(&o.USB_PWR.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetUSB_PWR_OVERCURR_DETECT_EN() uint32 {
	return (volatile.LoadUint32(&o.USB_PWR.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetUSB_PWR_OVERCURR_DETECT(value uint32) {
	volatile.StoreUint32(&o.USB_PWR.Reg, volatile.LoadUint32(&o.USB_PWR.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetUSB_PWR_OVERCURR_DETECT() uint32 {
	return (volatile.LoadUint32(&o.USB_PWR.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetUSB_PWR_VBUS_DETECT_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USB_PWR.Reg, volatile.LoadUint32(&o.USB_PWR.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetUSB_PWR_VBUS_DETECT_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USB_PWR.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetUSB_PWR_VBUS_DETECT(value uint32) {
	volatile.StoreUint32(&o.USB_PWR.Reg, volatile.LoadUint32(&o.USB_PWR.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetUSB_PWR_VBUS_DETECT() uint32 {
	return (volatile.LoadUint32(&o.USB_PWR.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetUSB_PWR_VBUS_EN_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USB_PWR.Reg, volatile.LoadUint32(&o.USB_PWR.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetUSB_PWR_VBUS_EN_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USB_PWR.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetUSB_PWR_VBUS_EN(value uint32) {
	volatile.StoreUint32(&o.USB_PWR.Reg, volatile.LoadUint32(&o.USB_PWR.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetUSB_PWR_VBUS_EN() uint32 {
	return volatile.LoadUint32(&o.USB_PWR.Reg) & 0x1
}

// USB.USBPHY_DIRECT: This register allows for direct control of the USB phy. Use in conjunction with usbphy_direct_override register to enable each override bit.
func (o *USB_Type) SetUSBPHY_DIRECT_RX_DM_OVERRIDE(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetUSBPHY_DIRECT_RX_DM_OVERRIDE() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetUSBPHY_DIRECT_RX_DP_OVERRIDE(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x1000000)|value<<24)
}
func (o *USB_Type) GetUSBPHY_DIRECT_RX_DP_OVERRIDE() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x1000000) >> 24
}
func (o *USB_Type) SetUSBPHY_DIRECT_RX_DD_OVERRIDE(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetUSBPHY_DIRECT_RX_DD_OVERRIDE() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetUSBPHY_DIRECT_DM_OVV(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DM_OVV() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetUSBPHY_DIRECT_DP_OVV(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DP_OVV() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetUSBPHY_DIRECT_DM_OVCN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DM_OVCN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetUSBPHY_DIRECT_DP_OVCN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DP_OVCN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetUSBPHY_DIRECT_RX_DM(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetUSBPHY_DIRECT_RX_DM() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetUSBPHY_DIRECT_RX_DP(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetUSBPHY_DIRECT_RX_DP() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetUSBPHY_DIRECT_RX_DD(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetUSBPHY_DIRECT_RX_DD() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetUSBPHY_DIRECT_TX_DIFFMODE(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetUSBPHY_DIRECT_TX_DIFFMODE() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetUSBPHY_DIRECT_TX_FSSLEW(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetUSBPHY_DIRECT_TX_FSSLEW() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetUSBPHY_DIRECT_TX_PD(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetUSBPHY_DIRECT_TX_PD() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetUSBPHY_DIRECT_RX_PD(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetUSBPHY_DIRECT_RX_PD() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetUSBPHY_DIRECT_TX_DM(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetUSBPHY_DIRECT_TX_DM() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetUSBPHY_DIRECT_TX_DP(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetUSBPHY_DIRECT_TX_DP() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetUSBPHY_DIRECT_TX_DM_OE(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetUSBPHY_DIRECT_TX_DM_OE() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetUSBPHY_DIRECT_TX_DP_OE(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetUSBPHY_DIRECT_TX_DP_OE() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetUSBPHY_DIRECT_DM_PULLDN_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DM_PULLDN_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetUSBPHY_DIRECT_DM_PULLUP_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DM_PULLUP_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetUSBPHY_DIRECT_DM_PULLUP_HISEL(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DM_PULLUP_HISEL() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetUSBPHY_DIRECT_DP_PULLDN_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DP_PULLDN_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetUSBPHY_DIRECT_DP_PULLUP_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DP_PULLUP_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetUSBPHY_DIRECT_DP_PULLUP_HISEL(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetUSBPHY_DIRECT_DP_PULLUP_HISEL() uint32 {
	return volatile.LoadUint32(&o.USBPHY_DIRECT.Reg) & 0x1
}

// USB.USBPHY_DIRECT_OVERRIDE: Override enable for each control in usbphy_direct
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_RX_DM_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_RX_DM_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_RX_DP_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_RX_DP_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_RX_DD_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_RX_DD_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_TX_DIFFMODE_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_TX_DIFFMODE_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_DM_PULLUP_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_DM_PULLUP_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_TX_FSSLEW_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_TX_FSSLEW_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_TX_PD_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_TX_PD_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_RX_PD_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_RX_PD_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_TX_DM_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_TX_DM_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_TX_DP_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_TX_DP_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_TX_DM_OE_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_TX_DM_OE_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_TX_DP_OE_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_TX_DP_OE_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_DM_PULLDN_EN_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_DM_PULLDN_EN_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_DP_PULLDN_EN_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_DP_PULLDN_EN_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_DP_PULLUP_EN_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_DP_PULLUP_EN_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_DM_PULLUP_HISEL_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_DM_PULLUP_HISEL_OVERRIDE_EN() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetUSBPHY_DIRECT_OVERRIDE_DP_PULLUP_HISEL_OVERRIDE_EN(value uint32) {
	volatile.StoreUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg, volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetUSBPHY_DIRECT_OVERRIDE_DP_PULLUP_HISEL_OVERRIDE_EN() uint32 {
	return volatile.LoadUint32(&o.USBPHY_DIRECT_OVERRIDE.Reg) & 0x1
}

// USB.USBPHY_TRIM: Used to adjust trim values of USB phy pull down resistors.
func (o *USB_Type) SetUSBPHY_TRIM_DM_PULLDN_TRIM(value uint32) {
	volatile.StoreUint32(&o.USBPHY_TRIM.Reg, volatile.LoadUint32(&o.USBPHY_TRIM.Reg)&^(0x1f00)|value<<8)
}
func (o *USB_Type) GetUSBPHY_TRIM_DM_PULLDN_TRIM() uint32 {
	return (volatile.LoadUint32(&o.USBPHY_TRIM.Reg) & 0x1f00) >> 8
}
func (o *USB_Type) SetUSBPHY_TRIM_DP_PULLDN_TRIM(value uint32) {
	volatile.StoreUint32(&o.USBPHY_TRIM.Reg, volatile.LoadUint32(&o.USBPHY_TRIM.Reg)&^(0x1f)|value)
}
func (o *USB_Type) GetUSBPHY_TRIM_DP_PULLDN_TRIM() uint32 {
	return volatile.LoadUint32(&o.USBPHY_TRIM.Reg) & 0x1f
}

// USB.LINESTATE_TUNING: Used for debug only.
func (o *USB_Type) SetLINESTATE_TUNING_SPARE_FIX(value uint32) {
	volatile.StoreUint32(&o.LINESTATE_TUNING.Reg, volatile.LoadUint32(&o.LINESTATE_TUNING.Reg)&^(0xf00)|value<<8)
}
func (o *USB_Type) GetLINESTATE_TUNING_SPARE_FIX() uint32 {
	return (volatile.LoadUint32(&o.LINESTATE_TUNING.Reg) & 0xf00) >> 8
}
func (o *USB_Type) SetLINESTATE_TUNING_DEV_LS_WAKE_FIX(value uint32) {
	volatile.StoreUint32(&o.LINESTATE_TUNING.Reg, volatile.LoadUint32(&o.LINESTATE_TUNING.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetLINESTATE_TUNING_DEV_LS_WAKE_FIX() uint32 {
	return (volatile.LoadUint32(&o.LINESTATE_TUNING.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetLINESTATE_TUNING_DEV_RX_ERR_QUIESCE(value uint32) {
	volatile.StoreUint32(&o.LINESTATE_TUNING.Reg, volatile.LoadUint32(&o.LINESTATE_TUNING.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetLINESTATE_TUNING_DEV_RX_ERR_QUIESCE() uint32 {
	return (volatile.LoadUint32(&o.LINESTATE_TUNING.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetLINESTATE_TUNING_SIE_RX_CHATTER_SE0_FIX(value uint32) {
	volatile.StoreUint32(&o.LINESTATE_TUNING.Reg, volatile.LoadUint32(&o.LINESTATE_TUNING.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetLINESTATE_TUNING_SIE_RX_CHATTER_SE0_FIX() uint32 {
	return (volatile.LoadUint32(&o.LINESTATE_TUNING.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetLINESTATE_TUNING_SIE_RX_BITSTUFF_FIX(value uint32) {
	volatile.StoreUint32(&o.LINESTATE_TUNING.Reg, volatile.LoadUint32(&o.LINESTATE_TUNING.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetLINESTATE_TUNING_SIE_RX_BITSTUFF_FIX() uint32 {
	return (volatile.LoadUint32(&o.LINESTATE_TUNING.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetLINESTATE_TUNING_DEV_BUFF_CONTROL_DOUBLE_READ_FIX(value uint32) {
	volatile.StoreUint32(&o.LINESTATE_TUNING.Reg, volatile.LoadUint32(&o.LINESTATE_TUNING.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetLINESTATE_TUNING_DEV_BUFF_CONTROL_DOUBLE_READ_FIX() uint32 {
	return (volatile.LoadUint32(&o.LINESTATE_TUNING.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetLINESTATE_TUNING_MULTI_HUB_FIX(value uint32) {
	volatile.StoreUint32(&o.LINESTATE_TUNING.Reg, volatile.LoadUint32(&o.LINESTATE_TUNING.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetLINESTATE_TUNING_MULTI_HUB_FIX() uint32 {
	return (volatile.LoadUint32(&o.LINESTATE_TUNING.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetLINESTATE_TUNING_LINESTATE_DELAY(value uint32) {
	volatile.StoreUint32(&o.LINESTATE_TUNING.Reg, volatile.LoadUint32(&o.LINESTATE_TUNING.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetLINESTATE_TUNING_LINESTATE_DELAY() uint32 {
	return (volatile.LoadUint32(&o.LINESTATE_TUNING.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetLINESTATE_TUNING_RCV_DELAY(value uint32) {
	volatile.StoreUint32(&o.LINESTATE_TUNING.Reg, volatile.LoadUint32(&o.LINESTATE_TUNING.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetLINESTATE_TUNING_RCV_DELAY() uint32 {
	return volatile.LoadUint32(&o.LINESTATE_TUNING.Reg) & 0x1
}

// USB.INTR: Raw Interrupts
func (o *USB_Type) SetINTR_EPX_STOPPED_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetINTR_EPX_STOPPED_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetINTR_DEV_SM_WATCHDOG_FIRED(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetINTR_DEV_SM_WATCHDOG_FIRED() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetINTR_ENDPOINT_ERROR(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetINTR_ENDPOINT_ERROR() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetINTR_RX_SHORT_PACKET(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetINTR_RX_SHORT_PACKET() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetINTR_EP_STALL_NAK(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetINTR_EP_STALL_NAK() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetINTR_ABORT_DONE(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetINTR_ABORT_DONE() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetINTR_DEV_SOF(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetINTR_DEV_SOF() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetINTR_SETUP_REQ(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetINTR_SETUP_REQ() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetINTR_DEV_RESUME_FROM_HOST(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetINTR_DEV_RESUME_FROM_HOST() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetINTR_DEV_SUSPEND(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetINTR_DEV_SUSPEND() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetINTR_DEV_CONN_DIS(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetINTR_DEV_CONN_DIS() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetINTR_BUS_RESET(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetINTR_BUS_RESET() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetINTR_VBUS_DETECT(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetINTR_VBUS_DETECT() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetINTR_STALL(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetINTR_STALL() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetINTR_ERROR_CRC(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetINTR_ERROR_CRC() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetINTR_ERROR_BIT_STUFF(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetINTR_ERROR_BIT_STUFF() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetINTR_ERROR_RX_OVERFLOW(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetINTR_ERROR_RX_OVERFLOW() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetINTR_ERROR_RX_TIMEOUT(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetINTR_ERROR_RX_TIMEOUT() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetINTR_ERROR_DATA_SEQ(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetINTR_ERROR_DATA_SEQ() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetINTR_BUFF_STATUS(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetINTR_BUFF_STATUS() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetINTR_TRANS_COMPLETE(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetINTR_TRANS_COMPLETE() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetINTR_HOST_SOF(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetINTR_HOST_SOF() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetINTR_HOST_RESUME(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetINTR_HOST_RESUME() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetINTR_HOST_CONN_DIS(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetINTR_HOST_CONN_DIS() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// USB.INTE: Interrupt Enable
func (o *USB_Type) SetINTE_EPX_STOPPED_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetINTE_EPX_STOPPED_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetINTE_DEV_SM_WATCHDOG_FIRED(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetINTE_DEV_SM_WATCHDOG_FIRED() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetINTE_ENDPOINT_ERROR(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetINTE_ENDPOINT_ERROR() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetINTE_RX_SHORT_PACKET(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetINTE_RX_SHORT_PACKET() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetINTE_EP_STALL_NAK(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetINTE_EP_STALL_NAK() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetINTE_ABORT_DONE(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetINTE_ABORT_DONE() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetINTE_DEV_SOF(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetINTE_DEV_SOF() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetINTE_SETUP_REQ(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetINTE_SETUP_REQ() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetINTE_DEV_RESUME_FROM_HOST(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetINTE_DEV_RESUME_FROM_HOST() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetINTE_DEV_SUSPEND(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetINTE_DEV_SUSPEND() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetINTE_DEV_CONN_DIS(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetINTE_DEV_CONN_DIS() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetINTE_BUS_RESET(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetINTE_BUS_RESET() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetINTE_VBUS_DETECT(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetINTE_VBUS_DETECT() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetINTE_STALL(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetINTE_STALL() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetINTE_ERROR_CRC(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetINTE_ERROR_CRC() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetINTE_ERROR_BIT_STUFF(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetINTE_ERROR_BIT_STUFF() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetINTE_ERROR_RX_OVERFLOW(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetINTE_ERROR_RX_OVERFLOW() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetINTE_ERROR_RX_TIMEOUT(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetINTE_ERROR_RX_TIMEOUT() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetINTE_ERROR_DATA_SEQ(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetINTE_ERROR_DATA_SEQ() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetINTE_BUFF_STATUS(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetINTE_BUFF_STATUS() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetINTE_TRANS_COMPLETE(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetINTE_TRANS_COMPLETE() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetINTE_HOST_SOF(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetINTE_HOST_SOF() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetINTE_HOST_RESUME(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetINTE_HOST_RESUME() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetINTE_HOST_CONN_DIS(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetINTE_HOST_CONN_DIS() uint32 {
	return volatile.LoadUint32(&o.INTE.Reg) & 0x1
}

// USB.INTF: Interrupt Force
func (o *USB_Type) SetINTF_EPX_STOPPED_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetINTF_EPX_STOPPED_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetINTF_DEV_SM_WATCHDOG_FIRED(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetINTF_DEV_SM_WATCHDOG_FIRED() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetINTF_ENDPOINT_ERROR(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetINTF_ENDPOINT_ERROR() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetINTF_RX_SHORT_PACKET(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetINTF_RX_SHORT_PACKET() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetINTF_EP_STALL_NAK(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetINTF_EP_STALL_NAK() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetINTF_ABORT_DONE(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetINTF_ABORT_DONE() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetINTF_DEV_SOF(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetINTF_DEV_SOF() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetINTF_SETUP_REQ(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetINTF_SETUP_REQ() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetINTF_DEV_RESUME_FROM_HOST(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetINTF_DEV_RESUME_FROM_HOST() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetINTF_DEV_SUSPEND(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetINTF_DEV_SUSPEND() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetINTF_DEV_CONN_DIS(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetINTF_DEV_CONN_DIS() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetINTF_BUS_RESET(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetINTF_BUS_RESET() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetINTF_VBUS_DETECT(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetINTF_VBUS_DETECT() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetINTF_STALL(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetINTF_STALL() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetINTF_ERROR_CRC(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetINTF_ERROR_CRC() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetINTF_ERROR_BIT_STUFF(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetINTF_ERROR_BIT_STUFF() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetINTF_ERROR_RX_OVERFLOW(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetINTF_ERROR_RX_OVERFLOW() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetINTF_ERROR_RX_TIMEOUT(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetINTF_ERROR_RX_TIMEOUT() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetINTF_ERROR_DATA_SEQ(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetINTF_ERROR_DATA_SEQ() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetINTF_BUFF_STATUS(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetINTF_BUFF_STATUS() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetINTF_TRANS_COMPLETE(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetINTF_TRANS_COMPLETE() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetINTF_HOST_SOF(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetINTF_HOST_SOF() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetINTF_HOST_RESUME(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetINTF_HOST_RESUME() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetINTF_HOST_CONN_DIS(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetINTF_HOST_CONN_DIS() uint32 {
	return volatile.LoadUint32(&o.INTF.Reg) & 0x1
}

// USB.INTS: Interrupt status after masking & forcing
func (o *USB_Type) SetINTS_EPX_STOPPED_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetINTS_EPX_STOPPED_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetINTS_DEV_SM_WATCHDOG_FIRED(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetINTS_DEV_SM_WATCHDOG_FIRED() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetINTS_ENDPOINT_ERROR(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetINTS_ENDPOINT_ERROR() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetINTS_RX_SHORT_PACKET(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetINTS_RX_SHORT_PACKET() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetINTS_EP_STALL_NAK(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetINTS_EP_STALL_NAK() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetINTS_ABORT_DONE(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetINTS_ABORT_DONE() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetINTS_DEV_SOF(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetINTS_DEV_SOF() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetINTS_SETUP_REQ(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetINTS_SETUP_REQ() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetINTS_DEV_RESUME_FROM_HOST(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetINTS_DEV_RESUME_FROM_HOST() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetINTS_DEV_SUSPEND(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetINTS_DEV_SUSPEND() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetINTS_DEV_CONN_DIS(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetINTS_DEV_CONN_DIS() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetINTS_BUS_RESET(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetINTS_BUS_RESET() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetINTS_VBUS_DETECT(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetINTS_VBUS_DETECT() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetINTS_STALL(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetINTS_STALL() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetINTS_ERROR_CRC(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetINTS_ERROR_CRC() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetINTS_ERROR_BIT_STUFF(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetINTS_ERROR_BIT_STUFF() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetINTS_ERROR_RX_OVERFLOW(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetINTS_ERROR_RX_OVERFLOW() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetINTS_ERROR_RX_TIMEOUT(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetINTS_ERROR_RX_TIMEOUT() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetINTS_ERROR_DATA_SEQ(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetINTS_ERROR_DATA_SEQ() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetINTS_BUFF_STATUS(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetINTS_BUFF_STATUS() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetINTS_TRANS_COMPLETE(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetINTS_TRANS_COMPLETE() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetINTS_HOST_SOF(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetINTS_HOST_SOF() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetINTS_HOST_RESUME(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetINTS_HOST_RESUME() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetINTS_HOST_CONN_DIS(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetINTS_HOST_CONN_DIS() uint32 {
	return volatile.LoadUint32(&o.INTS.Reg) & 0x1
}

// USB.SOF_TIMESTAMP_RAW: Device only. Raw value of free-running PHY clock counter @48MHz. Used to calculate time between SOF events.
func (o *USB_Type) SetSOF_TIMESTAMP_RAW(value uint32) {
	volatile.StoreUint32(&o.SOF_TIMESTAMP_RAW.Reg, volatile.LoadUint32(&o.SOF_TIMESTAMP_RAW.Reg)&^(0x1fffff)|value)
}
func (o *USB_Type) GetSOF_TIMESTAMP_RAW() uint32 {
	return volatile.LoadUint32(&o.SOF_TIMESTAMP_RAW.Reg) & 0x1fffff
}

// USB.SOF_TIMESTAMP_LAST: Device only. Value of free-running PHY clock counter @48MHz when last SOF event occurred.
func (o *USB_Type) SetSOF_TIMESTAMP_LAST(value uint32) {
	volatile.StoreUint32(&o.SOF_TIMESTAMP_LAST.Reg, volatile.LoadUint32(&o.SOF_TIMESTAMP_LAST.Reg)&^(0x1fffff)|value)
}
func (o *USB_Type) GetSOF_TIMESTAMP_LAST() uint32 {
	return volatile.LoadUint32(&o.SOF_TIMESTAMP_LAST.Reg) & 0x1fffff
}

// USB.SM_STATE
func (o *USB_Type) SetSM_STATE_RX_DASM(value uint32) {
	volatile.StoreUint32(&o.SM_STATE.Reg, volatile.LoadUint32(&o.SM_STATE.Reg)&^(0xf00)|value<<8)
}
func (o *USB_Type) GetSM_STATE_RX_DASM() uint32 {
	return (volatile.LoadUint32(&o.SM_STATE.Reg) & 0xf00) >> 8
}
func (o *USB_Type) SetSM_STATE_BC_STATE(value uint32) {
	volatile.StoreUint32(&o.SM_STATE.Reg, volatile.LoadUint32(&o.SM_STATE.Reg)&^(0xe0)|value<<5)
}
func (o *USB_Type) GetSM_STATE_BC_STATE() uint32 {
	return (volatile.LoadUint32(&o.SM_STATE.Reg) & 0xe0) >> 5
}
func (o *USB_Type) SetSM_STATE_STATE(value uint32) {
	volatile.StoreUint32(&o.SM_STATE.Reg, volatile.LoadUint32(&o.SM_STATE.Reg)&^(0x1f)|value)
}
func (o *USB_Type) GetSM_STATE_STATE() uint32 {
	return volatile.LoadUint32(&o.SM_STATE.Reg) & 0x1f
}

// USB.EP_TX_ERROR: TX error count for each endpoint. Write to each field to reset the counter to 0.
func (o *USB_Type) SetEP_TX_ERROR_EP15(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0xc0000000)|value<<30)
}
func (o *USB_Type) GetEP_TX_ERROR_EP15() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0xc0000000) >> 30
}
func (o *USB_Type) SetEP_TX_ERROR_EP14(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0x30000000)|value<<28)
}
func (o *USB_Type) GetEP_TX_ERROR_EP14() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0x30000000) >> 28
}
func (o *USB_Type) SetEP_TX_ERROR_EP13(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_Type) GetEP_TX_ERROR_EP13() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0xc000000) >> 26
}
func (o *USB_Type) SetEP_TX_ERROR_EP12(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0x3000000)|value<<24)
}
func (o *USB_Type) GetEP_TX_ERROR_EP12() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0x3000000) >> 24
}
func (o *USB_Type) SetEP_TX_ERROR_EP11(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0xc00000)|value<<22)
}
func (o *USB_Type) GetEP_TX_ERROR_EP11() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0xc00000) >> 22
}
func (o *USB_Type) SetEP_TX_ERROR_EP10(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0x300000)|value<<20)
}
func (o *USB_Type) GetEP_TX_ERROR_EP10() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0x300000) >> 20
}
func (o *USB_Type) SetEP_TX_ERROR_EP9(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0xc0000)|value<<18)
}
func (o *USB_Type) GetEP_TX_ERROR_EP9() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0xc0000) >> 18
}
func (o *USB_Type) SetEP_TX_ERROR_EP8(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0x30000)|value<<16)
}
func (o *USB_Type) GetEP_TX_ERROR_EP8() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0x30000) >> 16
}
func (o *USB_Type) SetEP_TX_ERROR_EP7(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0xc000)|value<<14)
}
func (o *USB_Type) GetEP_TX_ERROR_EP7() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0xc000) >> 14
}
func (o *USB_Type) SetEP_TX_ERROR_EP6(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0x3000)|value<<12)
}
func (o *USB_Type) GetEP_TX_ERROR_EP6() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0x3000) >> 12
}
func (o *USB_Type) SetEP_TX_ERROR_EP5(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0xc00)|value<<10)
}
func (o *USB_Type) GetEP_TX_ERROR_EP5() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0xc00) >> 10
}
func (o *USB_Type) SetEP_TX_ERROR_EP4(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0x300)|value<<8)
}
func (o *USB_Type) GetEP_TX_ERROR_EP4() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0x300) >> 8
}
func (o *USB_Type) SetEP_TX_ERROR_EP3(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0xc0)|value<<6)
}
func (o *USB_Type) GetEP_TX_ERROR_EP3() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0xc0) >> 6
}
func (o *USB_Type) SetEP_TX_ERROR_EP2(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0x30)|value<<4)
}
func (o *USB_Type) GetEP_TX_ERROR_EP2() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0x30) >> 4
}
func (o *USB_Type) SetEP_TX_ERROR_EP1(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0xc)|value<<2)
}
func (o *USB_Type) GetEP_TX_ERROR_EP1() uint32 {
	return (volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0xc) >> 2
}
func (o *USB_Type) SetEP_TX_ERROR_EP0(value uint32) {
	volatile.StoreUint32(&o.EP_TX_ERROR.Reg, volatile.LoadUint32(&o.EP_TX_ERROR.Reg)&^(0x3)|value)
}
func (o *USB_Type) GetEP_TX_ERROR_EP0() uint32 {
	return volatile.LoadUint32(&o.EP_TX_ERROR.Reg) & 0x3
}

// USB.EP_RX_ERROR: RX error count for each endpoint. Write to each field to reset the counter to 0.
func (o *USB_Type) SetEP_RX_ERROR_EP15_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_Type) GetEP_RX_ERROR_EP15_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x80000000) >> 31
}
func (o *USB_Type) SetEP_RX_ERROR_EP15_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_Type) GetEP_RX_ERROR_EP15_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x40000000) >> 30
}
func (o *USB_Type) SetEP_RX_ERROR_EP14_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_Type) GetEP_RX_ERROR_EP14_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x20000000) >> 29
}
func (o *USB_Type) SetEP_RX_ERROR_EP14_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_Type) GetEP_RX_ERROR_EP14_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x10000000) >> 28
}
func (o *USB_Type) SetEP_RX_ERROR_EP13_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x8000000)|value<<27)
}
func (o *USB_Type) GetEP_RX_ERROR_EP13_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x8000000) >> 27
}
func (o *USB_Type) SetEP_RX_ERROR_EP13_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_Type) GetEP_RX_ERROR_EP13_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x4000000) >> 26
}
func (o *USB_Type) SetEP_RX_ERROR_EP12_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x2000000)|value<<25)
}
func (o *USB_Type) GetEP_RX_ERROR_EP12_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x2000000) >> 25
}
func (o *USB_Type) SetEP_RX_ERROR_EP12_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x1000000)|value<<24)
}
func (o *USB_Type) GetEP_RX_ERROR_EP12_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x1000000) >> 24
}
func (o *USB_Type) SetEP_RX_ERROR_EP11_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x800000)|value<<23)
}
func (o *USB_Type) GetEP_RX_ERROR_EP11_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x800000) >> 23
}
func (o *USB_Type) SetEP_RX_ERROR_EP11_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x400000)|value<<22)
}
func (o *USB_Type) GetEP_RX_ERROR_EP11_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x400000) >> 22
}
func (o *USB_Type) SetEP_RX_ERROR_EP10_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x200000)|value<<21)
}
func (o *USB_Type) GetEP_RX_ERROR_EP10_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x200000) >> 21
}
func (o *USB_Type) SetEP_RX_ERROR_EP10_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetEP_RX_ERROR_EP10_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetEP_RX_ERROR_EP9_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetEP_RX_ERROR_EP9_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetEP_RX_ERROR_EP9_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetEP_RX_ERROR_EP9_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetEP_RX_ERROR_EP8_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x20000)|value<<17)
}
func (o *USB_Type) GetEP_RX_ERROR_EP8_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x20000) >> 17
}
func (o *USB_Type) SetEP_RX_ERROR_EP8_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x10000)|value<<16)
}
func (o *USB_Type) GetEP_RX_ERROR_EP8_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x10000) >> 16
}
func (o *USB_Type) SetEP_RX_ERROR_EP7_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x8000)|value<<15)
}
func (o *USB_Type) GetEP_RX_ERROR_EP7_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x8000) >> 15
}
func (o *USB_Type) SetEP_RX_ERROR_EP7_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x4000)|value<<14)
}
func (o *USB_Type) GetEP_RX_ERROR_EP7_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x4000) >> 14
}
func (o *USB_Type) SetEP_RX_ERROR_EP6_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x2000)|value<<13)
}
func (o *USB_Type) GetEP_RX_ERROR_EP6_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x2000) >> 13
}
func (o *USB_Type) SetEP_RX_ERROR_EP6_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x1000)|value<<12)
}
func (o *USB_Type) GetEP_RX_ERROR_EP6_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x1000) >> 12
}
func (o *USB_Type) SetEP_RX_ERROR_EP5_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x800)|value<<11)
}
func (o *USB_Type) GetEP_RX_ERROR_EP5_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x800) >> 11
}
func (o *USB_Type) SetEP_RX_ERROR_EP5_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x400)|value<<10)
}
func (o *USB_Type) GetEP_RX_ERROR_EP5_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x400) >> 10
}
func (o *USB_Type) SetEP_RX_ERROR_EP4_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x200)|value<<9)
}
func (o *USB_Type) GetEP_RX_ERROR_EP4_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x200) >> 9
}
func (o *USB_Type) SetEP_RX_ERROR_EP4_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x100)|value<<8)
}
func (o *USB_Type) GetEP_RX_ERROR_EP4_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x100) >> 8
}
func (o *USB_Type) SetEP_RX_ERROR_EP3_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x80)|value<<7)
}
func (o *USB_Type) GetEP_RX_ERROR_EP3_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x80) >> 7
}
func (o *USB_Type) SetEP_RX_ERROR_EP3_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x40)|value<<6)
}
func (o *USB_Type) GetEP_RX_ERROR_EP3_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x40) >> 6
}
func (o *USB_Type) SetEP_RX_ERROR_EP2_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x20)|value<<5)
}
func (o *USB_Type) GetEP_RX_ERROR_EP2_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x20) >> 5
}
func (o *USB_Type) SetEP_RX_ERROR_EP2_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x10)|value<<4)
}
func (o *USB_Type) GetEP_RX_ERROR_EP2_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x10) >> 4
}
func (o *USB_Type) SetEP_RX_ERROR_EP1_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x8)|value<<3)
}
func (o *USB_Type) GetEP_RX_ERROR_EP1_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x8) >> 3
}
func (o *USB_Type) SetEP_RX_ERROR_EP1_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x4)|value<<2)
}
func (o *USB_Type) GetEP_RX_ERROR_EP1_TRANSACTION() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x4) >> 2
}
func (o *USB_Type) SetEP_RX_ERROR_EP0_SEQ(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x2)|value<<1)
}
func (o *USB_Type) GetEP_RX_ERROR_EP0_SEQ() uint32 {
	return (volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x2) >> 1
}
func (o *USB_Type) SetEP_RX_ERROR_EP0_TRANSACTION(value uint32) {
	volatile.StoreUint32(&o.EP_RX_ERROR.Reg, volatile.LoadUint32(&o.EP_RX_ERROR.Reg)&^(0x1)|value)
}
func (o *USB_Type) GetEP_RX_ERROR_EP0_TRANSACTION() uint32 {
	return volatile.LoadUint32(&o.EP_RX_ERROR.Reg) & 0x1
}

// USB.DEV_SM_WATCHDOG: Watchdog that forces the device state machine to idle and raises an interrupt if the device stays in a state that isn't idle for the configured limit. The counter is reset on every state transition. Set limit while enable is low and then set the enable.
func (o *USB_Type) SetDEV_SM_WATCHDOG_FIRED(value uint32) {
	volatile.StoreUint32(&o.DEV_SM_WATCHDOG.Reg, volatile.LoadUint32(&o.DEV_SM_WATCHDOG.Reg)&^(0x100000)|value<<20)
}
func (o *USB_Type) GetDEV_SM_WATCHDOG_FIRED() uint32 {
	return (volatile.LoadUint32(&o.DEV_SM_WATCHDOG.Reg) & 0x100000) >> 20
}
func (o *USB_Type) SetDEV_SM_WATCHDOG_RESET(value uint32) {
	volatile.StoreUint32(&o.DEV_SM_WATCHDOG.Reg, volatile.LoadUint32(&o.DEV_SM_WATCHDOG.Reg)&^(0x80000)|value<<19)
}
func (o *USB_Type) GetDEV_SM_WATCHDOG_RESET() uint32 {
	return (volatile.LoadUint32(&o.DEV_SM_WATCHDOG.Reg) & 0x80000) >> 19
}
func (o *USB_Type) SetDEV_SM_WATCHDOG_ENABLE(value uint32) {
	volatile.StoreUint32(&o.DEV_SM_WATCHDOG.Reg, volatile.LoadUint32(&o.DEV_SM_WATCHDOG.Reg)&^(0x40000)|value<<18)
}
func (o *USB_Type) GetDEV_SM_WATCHDOG_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.DEV_SM_WATCHDOG.Reg) & 0x40000) >> 18
}
func (o *USB_Type) SetDEV_SM_WATCHDOG_LIMIT(value uint32) {
	volatile.StoreUint32(&o.DEV_SM_WATCHDOG.Reg, volatile.LoadUint32(&o.DEV_SM_WATCHDOG.Reg)&^(0x3ffff)|value)
}
func (o *USB_Type) GetDEV_SM_WATCHDOG_LIMIT() uint32 {
	return volatile.LoadUint32(&o.DEV_SM_WATCHDOG.Reg) & 0x3ffff
}

// ARM TrustZone RNG register block
type TRNG_Type struct {
	_                  [256]byte
	RNG_IMR            volatile.Register32 // 0x100
	RNG_ISR            volatile.Register32 // 0x104
	RNG_ICR            volatile.Register32 // 0x108
	TRNG_CONFIG        volatile.Register32 // 0x10C
	TRNG_VALID         volatile.Register32 // 0x110
	EHR_DATA0          volatile.Register32 // 0x114
	EHR_DATA1          volatile.Register32 // 0x118
	EHR_DATA2          volatile.Register32 // 0x11C
	EHR_DATA3          volatile.Register32 // 0x120
	EHR_DATA4          volatile.Register32 // 0x124
	EHR_DATA5          volatile.Register32 // 0x128
	RND_SOURCE_ENABLE  volatile.Register32 // 0x12C
	SAMPLE_CNT1        volatile.Register32 // 0x130
	AUTOCORR_STATISTIC volatile.Register32 // 0x134
	TRNG_DEBUG_CONTROL volatile.Register32 // 0x138
	_                  [4]byte
	TRNG_SW_RESET      volatile.Register32 // 0x140
	_                  [112]byte
	RNG_DEBUG_EN_INPUT volatile.Register32 // 0x1B4
	TRNG_BUSY          volatile.Register32 // 0x1B8
	RST_BITS_COUNTER   volatile.Register32 // 0x1BC
	RNG_VERSION        volatile.Register32 // 0x1C0
	_                  [28]byte
	RNG_BIST_CNTR_0    volatile.Register32 // 0x1E0
	RNG_BIST_CNTR_1    volatile.Register32 // 0x1E4
	RNG_BIST_CNTR_2    volatile.Register32 // 0x1E8
}

// TRNG.RNG_IMR: Interrupt masking.
func (o *TRNG_Type) SetRNG_IMR_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RNG_IMR.Reg, volatile.LoadUint32(&o.RNG_IMR.Reg)&^(0xfffffff0)|value<<4)
}
func (o *TRNG_Type) GetRNG_IMR_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RNG_IMR.Reg) & 0xfffffff0) >> 4
}
func (o *TRNG_Type) SetRNG_IMR_VN_ERR_INT_MASK(value uint32) {
	volatile.StoreUint32(&o.RNG_IMR.Reg, volatile.LoadUint32(&o.RNG_IMR.Reg)&^(0x8)|value<<3)
}
func (o *TRNG_Type) GetRNG_IMR_VN_ERR_INT_MASK() uint32 {
	return (volatile.LoadUint32(&o.RNG_IMR.Reg) & 0x8) >> 3
}
func (o *TRNG_Type) SetRNG_IMR_CRNGT_ERR_INT_MASK(value uint32) {
	volatile.StoreUint32(&o.RNG_IMR.Reg, volatile.LoadUint32(&o.RNG_IMR.Reg)&^(0x4)|value<<2)
}
func (o *TRNG_Type) GetRNG_IMR_CRNGT_ERR_INT_MASK() uint32 {
	return (volatile.LoadUint32(&o.RNG_IMR.Reg) & 0x4) >> 2
}
func (o *TRNG_Type) SetRNG_IMR_AUTOCORR_ERR_INT_MASK(value uint32) {
	volatile.StoreUint32(&o.RNG_IMR.Reg, volatile.LoadUint32(&o.RNG_IMR.Reg)&^(0x2)|value<<1)
}
func (o *TRNG_Type) GetRNG_IMR_AUTOCORR_ERR_INT_MASK() uint32 {
	return (volatile.LoadUint32(&o.RNG_IMR.Reg) & 0x2) >> 1
}
func (o *TRNG_Type) SetRNG_IMR_EHR_VALID_INT_MASK(value uint32) {
	volatile.StoreUint32(&o.RNG_IMR.Reg, volatile.LoadUint32(&o.RNG_IMR.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetRNG_IMR_EHR_VALID_INT_MASK() uint32 {
	return volatile.LoadUint32(&o.RNG_IMR.Reg) & 0x1
}

// TRNG.RNG_ISR: RNG status register. If corresponding RNG_IMR bit is unmasked, an interrupt will be generated.
func (o *TRNG_Type) SetRNG_ISR_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RNG_ISR.Reg, volatile.LoadUint32(&o.RNG_ISR.Reg)&^(0xfffffff0)|value<<4)
}
func (o *TRNG_Type) GetRNG_ISR_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RNG_ISR.Reg) & 0xfffffff0) >> 4
}
func (o *TRNG_Type) SetRNG_ISR_VN_ERR(value uint32) {
	volatile.StoreUint32(&o.RNG_ISR.Reg, volatile.LoadUint32(&o.RNG_ISR.Reg)&^(0x8)|value<<3)
}
func (o *TRNG_Type) GetRNG_ISR_VN_ERR() uint32 {
	return (volatile.LoadUint32(&o.RNG_ISR.Reg) & 0x8) >> 3
}
func (o *TRNG_Type) SetRNG_ISR_CRNGT_ERR(value uint32) {
	volatile.StoreUint32(&o.RNG_ISR.Reg, volatile.LoadUint32(&o.RNG_ISR.Reg)&^(0x4)|value<<2)
}
func (o *TRNG_Type) GetRNG_ISR_CRNGT_ERR() uint32 {
	return (volatile.LoadUint32(&o.RNG_ISR.Reg) & 0x4) >> 2
}
func (o *TRNG_Type) SetRNG_ISR_AUTOCORR_ERR(value uint32) {
	volatile.StoreUint32(&o.RNG_ISR.Reg, volatile.LoadUint32(&o.RNG_ISR.Reg)&^(0x2)|value<<1)
}
func (o *TRNG_Type) GetRNG_ISR_AUTOCORR_ERR() uint32 {
	return (volatile.LoadUint32(&o.RNG_ISR.Reg) & 0x2) >> 1
}
func (o *TRNG_Type) SetRNG_ISR_EHR_VALID(value uint32) {
	volatile.StoreUint32(&o.RNG_ISR.Reg, volatile.LoadUint32(&o.RNG_ISR.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetRNG_ISR_EHR_VALID() uint32 {
	return volatile.LoadUint32(&o.RNG_ISR.Reg) & 0x1
}

// TRNG.RNG_ICR: Interrupt/status bit clear Register.
func (o *TRNG_Type) SetRNG_ICR_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RNG_ICR.Reg, volatile.LoadUint32(&o.RNG_ICR.Reg)&^(0xfffffff0)|value<<4)
}
func (o *TRNG_Type) GetRNG_ICR_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RNG_ICR.Reg) & 0xfffffff0) >> 4
}
func (o *TRNG_Type) SetRNG_ICR_VN_ERR(value uint32) {
	volatile.StoreUint32(&o.RNG_ICR.Reg, volatile.LoadUint32(&o.RNG_ICR.Reg)&^(0x8)|value<<3)
}
func (o *TRNG_Type) GetRNG_ICR_VN_ERR() uint32 {
	return (volatile.LoadUint32(&o.RNG_ICR.Reg) & 0x8) >> 3
}
func (o *TRNG_Type) SetRNG_ICR_CRNGT_ERR(value uint32) {
	volatile.StoreUint32(&o.RNG_ICR.Reg, volatile.LoadUint32(&o.RNG_ICR.Reg)&^(0x4)|value<<2)
}
func (o *TRNG_Type) GetRNG_ICR_CRNGT_ERR() uint32 {
	return (volatile.LoadUint32(&o.RNG_ICR.Reg) & 0x4) >> 2
}
func (o *TRNG_Type) SetRNG_ICR_AUTOCORR_ERR(value uint32) {
	volatile.StoreUint32(&o.RNG_ICR.Reg, volatile.LoadUint32(&o.RNG_ICR.Reg)&^(0x2)|value<<1)
}
func (o *TRNG_Type) GetRNG_ICR_AUTOCORR_ERR() uint32 {
	return (volatile.LoadUint32(&o.RNG_ICR.Reg) & 0x2) >> 1
}
func (o *TRNG_Type) SetRNG_ICR_EHR_VALID(value uint32) {
	volatile.StoreUint32(&o.RNG_ICR.Reg, volatile.LoadUint32(&o.RNG_ICR.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetRNG_ICR_EHR_VALID() uint32 {
	return volatile.LoadUint32(&o.RNG_ICR.Reg) & 0x1
}

// TRNG.TRNG_CONFIG: Selecting the inverter-chain length.
func (o *TRNG_Type) SetTRNG_CONFIG_RESERVED(value uint32) {
	volatile.StoreUint32(&o.TRNG_CONFIG.Reg, volatile.LoadUint32(&o.TRNG_CONFIG.Reg)&^(0xfffffffc)|value<<2)
}
func (o *TRNG_Type) GetTRNG_CONFIG_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.TRNG_CONFIG.Reg) & 0xfffffffc) >> 2
}
func (o *TRNG_Type) SetTRNG_CONFIG_RND_SRC_SEL(value uint32) {
	volatile.StoreUint32(&o.TRNG_CONFIG.Reg, volatile.LoadUint32(&o.TRNG_CONFIG.Reg)&^(0x3)|value)
}
func (o *TRNG_Type) GetTRNG_CONFIG_RND_SRC_SEL() uint32 {
	return volatile.LoadUint32(&o.TRNG_CONFIG.Reg) & 0x3
}

// TRNG.TRNG_VALID: 192 bit collection indication.
func (o *TRNG_Type) SetTRNG_VALID_RESERVED(value uint32) {
	volatile.StoreUint32(&o.TRNG_VALID.Reg, volatile.LoadUint32(&o.TRNG_VALID.Reg)&^(0xfffffffe)|value<<1)
}
func (o *TRNG_Type) GetTRNG_VALID_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.TRNG_VALID.Reg) & 0xfffffffe) >> 1
}
func (o *TRNG_Type) SetTRNG_VALID_EHR_VALID(value uint32) {
	volatile.StoreUint32(&o.TRNG_VALID.Reg, volatile.LoadUint32(&o.TRNG_VALID.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetTRNG_VALID_EHR_VALID() uint32 {
	return volatile.LoadUint32(&o.TRNG_VALID.Reg) & 0x1
}

// TRNG.EHR_DATA0: RNG collected bits.
func (o *TRNG_Type) SetEHR_DATA0(value uint32) {
	volatile.StoreUint32(&o.EHR_DATA0.Reg, value)
}
func (o *TRNG_Type) GetEHR_DATA0() uint32 {
	return volatile.LoadUint32(&o.EHR_DATA0.Reg)
}

// TRNG.EHR_DATA1: RNG collected bits.
func (o *TRNG_Type) SetEHR_DATA1(value uint32) {
	volatile.StoreUint32(&o.EHR_DATA1.Reg, value)
}
func (o *TRNG_Type) GetEHR_DATA1() uint32 {
	return volatile.LoadUint32(&o.EHR_DATA1.Reg)
}

// TRNG.EHR_DATA2: RNG collected bits.
func (o *TRNG_Type) SetEHR_DATA2(value uint32) {
	volatile.StoreUint32(&o.EHR_DATA2.Reg, value)
}
func (o *TRNG_Type) GetEHR_DATA2() uint32 {
	return volatile.LoadUint32(&o.EHR_DATA2.Reg)
}

// TRNG.EHR_DATA3: RNG collected bits.
func (o *TRNG_Type) SetEHR_DATA3(value uint32) {
	volatile.StoreUint32(&o.EHR_DATA3.Reg, value)
}
func (o *TRNG_Type) GetEHR_DATA3() uint32 {
	return volatile.LoadUint32(&o.EHR_DATA3.Reg)
}

// TRNG.EHR_DATA4: RNG collected bits.
func (o *TRNG_Type) SetEHR_DATA4(value uint32) {
	volatile.StoreUint32(&o.EHR_DATA4.Reg, value)
}
func (o *TRNG_Type) GetEHR_DATA4() uint32 {
	return volatile.LoadUint32(&o.EHR_DATA4.Reg)
}

// TRNG.EHR_DATA5: RNG collected bits.
func (o *TRNG_Type) SetEHR_DATA5(value uint32) {
	volatile.StoreUint32(&o.EHR_DATA5.Reg, value)
}
func (o *TRNG_Type) GetEHR_DATA5() uint32 {
	return volatile.LoadUint32(&o.EHR_DATA5.Reg)
}

// TRNG.RND_SOURCE_ENABLE: Enable signal for the random source.
func (o *TRNG_Type) SetRND_SOURCE_ENABLE_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RND_SOURCE_ENABLE.Reg, volatile.LoadUint32(&o.RND_SOURCE_ENABLE.Reg)&^(0xfffffffe)|value<<1)
}
func (o *TRNG_Type) GetRND_SOURCE_ENABLE_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RND_SOURCE_ENABLE.Reg) & 0xfffffffe) >> 1
}
func (o *TRNG_Type) SetRND_SOURCE_ENABLE_RND_SRC_EN(value uint32) {
	volatile.StoreUint32(&o.RND_SOURCE_ENABLE.Reg, volatile.LoadUint32(&o.RND_SOURCE_ENABLE.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetRND_SOURCE_ENABLE_RND_SRC_EN() uint32 {
	return volatile.LoadUint32(&o.RND_SOURCE_ENABLE.Reg) & 0x1
}

// TRNG.SAMPLE_CNT1: Counts clocks between sampling of random bit.
func (o *TRNG_Type) SetSAMPLE_CNT1(value uint32) {
	volatile.StoreUint32(&o.SAMPLE_CNT1.Reg, value)
}
func (o *TRNG_Type) GetSAMPLE_CNT1() uint32 {
	return volatile.LoadUint32(&o.SAMPLE_CNT1.Reg)
}

// TRNG.AUTOCORR_STATISTIC: Statistic about Autocorrelation test activations.
func (o *TRNG_Type) SetAUTOCORR_STATISTIC_RESERVED(value uint32) {
	volatile.StoreUint32(&o.AUTOCORR_STATISTIC.Reg, volatile.LoadUint32(&o.AUTOCORR_STATISTIC.Reg)&^(0xffc00000)|value<<22)
}
func (o *TRNG_Type) GetAUTOCORR_STATISTIC_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.AUTOCORR_STATISTIC.Reg) & 0xffc00000) >> 22
}
func (o *TRNG_Type) SetAUTOCORR_STATISTIC_AUTOCORR_FAILS(value uint32) {
	volatile.StoreUint32(&o.AUTOCORR_STATISTIC.Reg, volatile.LoadUint32(&o.AUTOCORR_STATISTIC.Reg)&^(0x3fc000)|value<<14)
}
func (o *TRNG_Type) GetAUTOCORR_STATISTIC_AUTOCORR_FAILS() uint32 {
	return (volatile.LoadUint32(&o.AUTOCORR_STATISTIC.Reg) & 0x3fc000) >> 14
}
func (o *TRNG_Type) SetAUTOCORR_STATISTIC_AUTOCORR_TRYS(value uint32) {
	volatile.StoreUint32(&o.AUTOCORR_STATISTIC.Reg, volatile.LoadUint32(&o.AUTOCORR_STATISTIC.Reg)&^(0x3fff)|value)
}
func (o *TRNG_Type) GetAUTOCORR_STATISTIC_AUTOCORR_TRYS() uint32 {
	return volatile.LoadUint32(&o.AUTOCORR_STATISTIC.Reg) & 0x3fff
}

// TRNG.TRNG_DEBUG_CONTROL: Debug register.
func (o *TRNG_Type) SetTRNG_DEBUG_CONTROL_AUTO_CORRELATE_BYPASS(value uint32) {
	volatile.StoreUint32(&o.TRNG_DEBUG_CONTROL.Reg, volatile.LoadUint32(&o.TRNG_DEBUG_CONTROL.Reg)&^(0x8)|value<<3)
}
func (o *TRNG_Type) GetTRNG_DEBUG_CONTROL_AUTO_CORRELATE_BYPASS() uint32 {
	return (volatile.LoadUint32(&o.TRNG_DEBUG_CONTROL.Reg) & 0x8) >> 3
}
func (o *TRNG_Type) SetTRNG_DEBUG_CONTROL_TRNG_CRNGT_BYPASS(value uint32) {
	volatile.StoreUint32(&o.TRNG_DEBUG_CONTROL.Reg, volatile.LoadUint32(&o.TRNG_DEBUG_CONTROL.Reg)&^(0x4)|value<<2)
}
func (o *TRNG_Type) GetTRNG_DEBUG_CONTROL_TRNG_CRNGT_BYPASS() uint32 {
	return (volatile.LoadUint32(&o.TRNG_DEBUG_CONTROL.Reg) & 0x4) >> 2
}
func (o *TRNG_Type) SetTRNG_DEBUG_CONTROL_VNC_BYPASS(value uint32) {
	volatile.StoreUint32(&o.TRNG_DEBUG_CONTROL.Reg, volatile.LoadUint32(&o.TRNG_DEBUG_CONTROL.Reg)&^(0x2)|value<<1)
}
func (o *TRNG_Type) GetTRNG_DEBUG_CONTROL_VNC_BYPASS() uint32 {
	return (volatile.LoadUint32(&o.TRNG_DEBUG_CONTROL.Reg) & 0x2) >> 1
}
func (o *TRNG_Type) SetTRNG_DEBUG_CONTROL_RESERVED(value uint32) {
	volatile.StoreUint32(&o.TRNG_DEBUG_CONTROL.Reg, volatile.LoadUint32(&o.TRNG_DEBUG_CONTROL.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetTRNG_DEBUG_CONTROL_RESERVED() uint32 {
	return volatile.LoadUint32(&o.TRNG_DEBUG_CONTROL.Reg) & 0x1
}

// TRNG.TRNG_SW_RESET: Generate internal SW reset within the RNG block.
func (o *TRNG_Type) SetTRNG_SW_RESET_RESERVED(value uint32) {
	volatile.StoreUint32(&o.TRNG_SW_RESET.Reg, volatile.LoadUint32(&o.TRNG_SW_RESET.Reg)&^(0xfffffffe)|value<<1)
}
func (o *TRNG_Type) GetTRNG_SW_RESET_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.TRNG_SW_RESET.Reg) & 0xfffffffe) >> 1
}
func (o *TRNG_Type) SetTRNG_SW_RESET(value uint32) {
	volatile.StoreUint32(&o.TRNG_SW_RESET.Reg, volatile.LoadUint32(&o.TRNG_SW_RESET.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetTRNG_SW_RESET() uint32 {
	return volatile.LoadUint32(&o.TRNG_SW_RESET.Reg) & 0x1
}

// TRNG.RNG_DEBUG_EN_INPUT: Enable the RNG debug mode
func (o *TRNG_Type) SetRNG_DEBUG_EN_INPUT_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RNG_DEBUG_EN_INPUT.Reg, volatile.LoadUint32(&o.RNG_DEBUG_EN_INPUT.Reg)&^(0xfffffffe)|value<<1)
}
func (o *TRNG_Type) GetRNG_DEBUG_EN_INPUT_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RNG_DEBUG_EN_INPUT.Reg) & 0xfffffffe) >> 1
}
func (o *TRNG_Type) SetRNG_DEBUG_EN_INPUT_RNG_DEBUG_EN(value uint32) {
	volatile.StoreUint32(&o.RNG_DEBUG_EN_INPUT.Reg, volatile.LoadUint32(&o.RNG_DEBUG_EN_INPUT.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetRNG_DEBUG_EN_INPUT_RNG_DEBUG_EN() uint32 {
	return volatile.LoadUint32(&o.RNG_DEBUG_EN_INPUT.Reg) & 0x1
}

// TRNG.TRNG_BUSY: RNG Busy indication.
func (o *TRNG_Type) SetTRNG_BUSY_RESERVED(value uint32) {
	volatile.StoreUint32(&o.TRNG_BUSY.Reg, volatile.LoadUint32(&o.TRNG_BUSY.Reg)&^(0xfffffffe)|value<<1)
}
func (o *TRNG_Type) GetTRNG_BUSY_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.TRNG_BUSY.Reg) & 0xfffffffe) >> 1
}
func (o *TRNG_Type) SetTRNG_BUSY(value uint32) {
	volatile.StoreUint32(&o.TRNG_BUSY.Reg, volatile.LoadUint32(&o.TRNG_BUSY.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetTRNG_BUSY() uint32 {
	return volatile.LoadUint32(&o.TRNG_BUSY.Reg) & 0x1
}

// TRNG.RST_BITS_COUNTER: Reset the counter of collected bits in the RNG.
func (o *TRNG_Type) SetRST_BITS_COUNTER_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RST_BITS_COUNTER.Reg, volatile.LoadUint32(&o.RST_BITS_COUNTER.Reg)&^(0xfffffffe)|value<<1)
}
func (o *TRNG_Type) GetRST_BITS_COUNTER_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RST_BITS_COUNTER.Reg) & 0xfffffffe) >> 1
}
func (o *TRNG_Type) SetRST_BITS_COUNTER(value uint32) {
	volatile.StoreUint32(&o.RST_BITS_COUNTER.Reg, volatile.LoadUint32(&o.RST_BITS_COUNTER.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetRST_BITS_COUNTER() uint32 {
	return volatile.LoadUint32(&o.RST_BITS_COUNTER.Reg) & 0x1
}

// TRNG.RNG_VERSION: Displays the version settings of the TRNG.
func (o *TRNG_Type) SetRNG_VERSION_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RNG_VERSION.Reg, volatile.LoadUint32(&o.RNG_VERSION.Reg)&^(0xffffff00)|value<<8)
}
func (o *TRNG_Type) GetRNG_VERSION_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RNG_VERSION.Reg) & 0xffffff00) >> 8
}
func (o *TRNG_Type) SetRNG_VERSION_RNG_USE_5_SBOXES(value uint32) {
	volatile.StoreUint32(&o.RNG_VERSION.Reg, volatile.LoadUint32(&o.RNG_VERSION.Reg)&^(0x80)|value<<7)
}
func (o *TRNG_Type) GetRNG_VERSION_RNG_USE_5_SBOXES() uint32 {
	return (volatile.LoadUint32(&o.RNG_VERSION.Reg) & 0x80) >> 7
}
func (o *TRNG_Type) SetRNG_VERSION_RESEEDING_EXISTS(value uint32) {
	volatile.StoreUint32(&o.RNG_VERSION.Reg, volatile.LoadUint32(&o.RNG_VERSION.Reg)&^(0x40)|value<<6)
}
func (o *TRNG_Type) GetRNG_VERSION_RESEEDING_EXISTS() uint32 {
	return (volatile.LoadUint32(&o.RNG_VERSION.Reg) & 0x40) >> 6
}
func (o *TRNG_Type) SetRNG_VERSION_KAT_EXISTS(value uint32) {
	volatile.StoreUint32(&o.RNG_VERSION.Reg, volatile.LoadUint32(&o.RNG_VERSION.Reg)&^(0x20)|value<<5)
}
func (o *TRNG_Type) GetRNG_VERSION_KAT_EXISTS() uint32 {
	return (volatile.LoadUint32(&o.RNG_VERSION.Reg) & 0x20) >> 5
}
func (o *TRNG_Type) SetRNG_VERSION_PRNG_EXISTS(value uint32) {
	volatile.StoreUint32(&o.RNG_VERSION.Reg, volatile.LoadUint32(&o.RNG_VERSION.Reg)&^(0x10)|value<<4)
}
func (o *TRNG_Type) GetRNG_VERSION_PRNG_EXISTS() uint32 {
	return (volatile.LoadUint32(&o.RNG_VERSION.Reg) & 0x10) >> 4
}
func (o *TRNG_Type) SetRNG_VERSION_TRNG_TESTS_BYPASS_EN(value uint32) {
	volatile.StoreUint32(&o.RNG_VERSION.Reg, volatile.LoadUint32(&o.RNG_VERSION.Reg)&^(0x8)|value<<3)
}
func (o *TRNG_Type) GetRNG_VERSION_TRNG_TESTS_BYPASS_EN() uint32 {
	return (volatile.LoadUint32(&o.RNG_VERSION.Reg) & 0x8) >> 3
}
func (o *TRNG_Type) SetRNG_VERSION_AUTOCORR_EXISTS(value uint32) {
	volatile.StoreUint32(&o.RNG_VERSION.Reg, volatile.LoadUint32(&o.RNG_VERSION.Reg)&^(0x4)|value<<2)
}
func (o *TRNG_Type) GetRNG_VERSION_AUTOCORR_EXISTS() uint32 {
	return (volatile.LoadUint32(&o.RNG_VERSION.Reg) & 0x4) >> 2
}
func (o *TRNG_Type) SetRNG_VERSION_CRNGT_EXISTS(value uint32) {
	volatile.StoreUint32(&o.RNG_VERSION.Reg, volatile.LoadUint32(&o.RNG_VERSION.Reg)&^(0x2)|value<<1)
}
func (o *TRNG_Type) GetRNG_VERSION_CRNGT_EXISTS() uint32 {
	return (volatile.LoadUint32(&o.RNG_VERSION.Reg) & 0x2) >> 1
}
func (o *TRNG_Type) SetRNG_VERSION_EHR_WIDTH_192(value uint32) {
	volatile.StoreUint32(&o.RNG_VERSION.Reg, volatile.LoadUint32(&o.RNG_VERSION.Reg)&^(0x1)|value)
}
func (o *TRNG_Type) GetRNG_VERSION_EHR_WIDTH_192() uint32 {
	return volatile.LoadUint32(&o.RNG_VERSION.Reg) & 0x1
}

// TRNG.RNG_BIST_CNTR_0: Collected BIST results.
func (o *TRNG_Type) SetRNG_BIST_CNTR_0_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RNG_BIST_CNTR_0.Reg, volatile.LoadUint32(&o.RNG_BIST_CNTR_0.Reg)&^(0xffc00000)|value<<22)
}
func (o *TRNG_Type) GetRNG_BIST_CNTR_0_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RNG_BIST_CNTR_0.Reg) & 0xffc00000) >> 22
}
func (o *TRNG_Type) SetRNG_BIST_CNTR_0_ROSC_CNTR_VAL(value uint32) {
	volatile.StoreUint32(&o.RNG_BIST_CNTR_0.Reg, volatile.LoadUint32(&o.RNG_BIST_CNTR_0.Reg)&^(0x3fffff)|value)
}
func (o *TRNG_Type) GetRNG_BIST_CNTR_0_ROSC_CNTR_VAL() uint32 {
	return volatile.LoadUint32(&o.RNG_BIST_CNTR_0.Reg) & 0x3fffff
}

// TRNG.RNG_BIST_CNTR_1: Collected BIST results.
func (o *TRNG_Type) SetRNG_BIST_CNTR_1_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RNG_BIST_CNTR_1.Reg, volatile.LoadUint32(&o.RNG_BIST_CNTR_1.Reg)&^(0xffc00000)|value<<22)
}
func (o *TRNG_Type) GetRNG_BIST_CNTR_1_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RNG_BIST_CNTR_1.Reg) & 0xffc00000) >> 22
}
func (o *TRNG_Type) SetRNG_BIST_CNTR_1_ROSC_CNTR_VAL(value uint32) {
	volatile.StoreUint32(&o.RNG_BIST_CNTR_1.Reg, volatile.LoadUint32(&o.RNG_BIST_CNTR_1.Reg)&^(0x3fffff)|value)
}
func (o *TRNG_Type) GetRNG_BIST_CNTR_1_ROSC_CNTR_VAL() uint32 {
	return volatile.LoadUint32(&o.RNG_BIST_CNTR_1.Reg) & 0x3fffff
}

// TRNG.RNG_BIST_CNTR_2: Collected BIST results.
func (o *TRNG_Type) SetRNG_BIST_CNTR_2_RESERVED(value uint32) {
	volatile.StoreUint32(&o.RNG_BIST_CNTR_2.Reg, volatile.LoadUint32(&o.RNG_BIST_CNTR_2.Reg)&^(0xffc00000)|value<<22)
}
func (o *TRNG_Type) GetRNG_BIST_CNTR_2_RESERVED() uint32 {
	return (volatile.LoadUint32(&o.RNG_BIST_CNTR_2.Reg) & 0xffc00000) >> 22
}
func (o *TRNG_Type) SetRNG_BIST_CNTR_2_ROSC_CNTR_VAL(value uint32) {
	volatile.StoreUint32(&o.RNG_BIST_CNTR_2.Reg, volatile.LoadUint32(&o.RNG_BIST_CNTR_2.Reg)&^(0x3fffff)|value)
}
func (o *TRNG_Type) GetRNG_BIST_CNTR_2_ROSC_CNTR_VAL() uint32 {
	return volatile.LoadUint32(&o.RNG_BIST_CNTR_2.Reg) & 0x3fffff
}

// Glitch detector controls
type GLITCH_DETECTOR_Type struct {
	ARM         volatile.Register32 // 0x0
	DISARM      volatile.Register32 // 0x4
	SENSITIVITY volatile.Register32 // 0x8
	LOCK        volatile.Register32 // 0xC
	TRIG_STATUS volatile.Register32 // 0x10
	TRIG_FORCE  volatile.Register32 // 0x14
}

// GLITCH_DETECTOR.ARM: Forcibly arm the glitch detectors, if they are not already armed by OTP. When armed, any individual detector trigger will cause a restart of the switched core power domain's power-on reset state machine. Glitch detector triggers are recorded accumulatively in TRIG_STATUS. If the system is reset by a glitch detector trigger, this is recorded in POWMAN_CHIP_RESET. This register is Secure read/write only.
func (o *GLITCH_DETECTOR_Type) SetARM(value uint32) {
	volatile.StoreUint32(&o.ARM.Reg, volatile.LoadUint32(&o.ARM.Reg)&^(0xffff)|value)
}
func (o *GLITCH_DETECTOR_Type) GetARM() uint32 {
	return volatile.LoadUint32(&o.ARM.Reg) & 0xffff
}

// GLITCH_DETECTOR.DISARM
func (o *GLITCH_DETECTOR_Type) SetDISARM(value uint32) {
	volatile.StoreUint32(&o.DISARM.Reg, volatile.LoadUint32(&o.DISARM.Reg)&^(0xffff)|value)
}
func (o *GLITCH_DETECTOR_Type) GetDISARM() uint32 {
	return volatile.LoadUint32(&o.DISARM.Reg) & 0xffff
}

// GLITCH_DETECTOR.SENSITIVITY: Adjust the sensitivity of glitch detectors to values other than their OTP-provided defaults. This register is Secure read/write only.
func (o *GLITCH_DETECTOR_Type) SetSENSITIVITY_DEFAULT(value uint32) {
	volatile.StoreUint32(&o.SENSITIVITY.Reg, volatile.LoadUint32(&o.SENSITIVITY.Reg)&^(0xff000000)|value<<24)
}
func (o *GLITCH_DETECTOR_Type) GetSENSITIVITY_DEFAULT() uint32 {
	return (volatile.LoadUint32(&o.SENSITIVITY.Reg) & 0xff000000) >> 24
}
func (o *GLITCH_DETECTOR_Type) SetSENSITIVITY_DET3_INV(value uint32) {
	volatile.StoreUint32(&o.SENSITIVITY.Reg, volatile.LoadUint32(&o.SENSITIVITY.Reg)&^(0xc000)|value<<14)
}
func (o *GLITCH_DETECTOR_Type) GetSENSITIVITY_DET3_INV() uint32 {
	return (volatile.LoadUint32(&o.SENSITIVITY.Reg) & 0xc000) >> 14
}
func (o *GLITCH_DETECTOR_Type) SetSENSITIVITY_DET2_INV(value uint32) {
	volatile.StoreUint32(&o.SENSITIVITY.Reg, volatile.LoadUint32(&o.SENSITIVITY.Reg)&^(0x3000)|value<<12)
}
func (o *GLITCH_DETECTOR_Type) GetSENSITIVITY_DET2_INV() uint32 {
	return (volatile.LoadUint32(&o.SENSITIVITY.Reg) & 0x3000) >> 12
}
func (o *GLITCH_DETECTOR_Type) SetSENSITIVITY_DET1_INV(value uint32) {
	volatile.StoreUint32(&o.SENSITIVITY.Reg, volatile.LoadUint32(&o.SENSITIVITY.Reg)&^(0xc00)|value<<10)
}
func (o *GLITCH_DETECTOR_Type) GetSENSITIVITY_DET1_INV() uint32 {
	return (volatile.LoadUint32(&o.SENSITIVITY.Reg) & 0xc00) >> 10
}
func (o *GLITCH_DETECTOR_Type) SetSENSITIVITY_DET0_INV(value uint32) {
	volatile.StoreUint32(&o.SENSITIVITY.Reg, volatile.LoadUint32(&o.SENSITIVITY.Reg)&^(0x300)|value<<8)
}
func (o *GLITCH_DETECTOR_Type) GetSENSITIVITY_DET0_INV() uint32 {
	return (volatile.LoadUint32(&o.SENSITIVITY.Reg) & 0x300) >> 8
}
func (o *GLITCH_DETECTOR_Type) SetSENSITIVITY_DET3(value uint32) {
	volatile.StoreUint32(&o.SENSITIVITY.Reg, volatile.LoadUint32(&o.SENSITIVITY.Reg)&^(0xc0)|value<<6)
}
func (o *GLITCH_DETECTOR_Type) GetSENSITIVITY_DET3() uint32 {
	return (volatile.LoadUint32(&o.SENSITIVITY.Reg) & 0xc0) >> 6
}
func (o *GLITCH_DETECTOR_Type) SetSENSITIVITY_DET2(value uint32) {
	volatile.StoreUint32(&o.SENSITIVITY.Reg, volatile.LoadUint32(&o.SENSITIVITY.Reg)&^(0x30)|value<<4)
}
func (o *GLITCH_DETECTOR_Type) GetSENSITIVITY_DET2() uint32 {
	return (volatile.LoadUint32(&o.SENSITIVITY.Reg) & 0x30) >> 4
}
func (o *GLITCH_DETECTOR_Type) SetSENSITIVITY_DET1(value uint32) {
	volatile.StoreUint32(&o.SENSITIVITY.Reg, volatile.LoadUint32(&o.SENSITIVITY.Reg)&^(0xc)|value<<2)
}
func (o *GLITCH_DETECTOR_Type) GetSENSITIVITY_DET1() uint32 {
	return (volatile.LoadUint32(&o.SENSITIVITY.Reg) & 0xc) >> 2
}
func (o *GLITCH_DETECTOR_Type) SetSENSITIVITY_DET0(value uint32) {
	volatile.StoreUint32(&o.SENSITIVITY.Reg, volatile.LoadUint32(&o.SENSITIVITY.Reg)&^(0x3)|value)
}
func (o *GLITCH_DETECTOR_Type) GetSENSITIVITY_DET0() uint32 {
	return volatile.LoadUint32(&o.SENSITIVITY.Reg) & 0x3
}

// GLITCH_DETECTOR.LOCK
func (o *GLITCH_DETECTOR_Type) SetLOCK(value uint32) {
	volatile.StoreUint32(&o.LOCK.Reg, volatile.LoadUint32(&o.LOCK.Reg)&^(0xff)|value)
}
func (o *GLITCH_DETECTOR_Type) GetLOCK() uint32 {
	return volatile.LoadUint32(&o.LOCK.Reg) & 0xff
}

// GLITCH_DETECTOR.TRIG_STATUS: Set when a detector output triggers. Write-1-clear. (May immediately return high if the detector remains in a failed state. Detectors can only be cleared by a full reset of the switched core power domain.) This register is Secure read/write only.
func (o *GLITCH_DETECTOR_Type) SetTRIG_STATUS_DET3(value uint32) {
	volatile.StoreUint32(&o.TRIG_STATUS.Reg, volatile.LoadUint32(&o.TRIG_STATUS.Reg)&^(0x8)|value<<3)
}
func (o *GLITCH_DETECTOR_Type) GetTRIG_STATUS_DET3() uint32 {
	return (volatile.LoadUint32(&o.TRIG_STATUS.Reg) & 0x8) >> 3
}
func (o *GLITCH_DETECTOR_Type) SetTRIG_STATUS_DET2(value uint32) {
	volatile.StoreUint32(&o.TRIG_STATUS.Reg, volatile.LoadUint32(&o.TRIG_STATUS.Reg)&^(0x4)|value<<2)
}
func (o *GLITCH_DETECTOR_Type) GetTRIG_STATUS_DET2() uint32 {
	return (volatile.LoadUint32(&o.TRIG_STATUS.Reg) & 0x4) >> 2
}
func (o *GLITCH_DETECTOR_Type) SetTRIG_STATUS_DET1(value uint32) {
	volatile.StoreUint32(&o.TRIG_STATUS.Reg, volatile.LoadUint32(&o.TRIG_STATUS.Reg)&^(0x2)|value<<1)
}
func (o *GLITCH_DETECTOR_Type) GetTRIG_STATUS_DET1() uint32 {
	return (volatile.LoadUint32(&o.TRIG_STATUS.Reg) & 0x2) >> 1
}
func (o *GLITCH_DETECTOR_Type) SetTRIG_STATUS_DET0(value uint32) {
	volatile.StoreUint32(&o.TRIG_STATUS.Reg, volatile.LoadUint32(&o.TRIG_STATUS.Reg)&^(0x1)|value)
}
func (o *GLITCH_DETECTOR_Type) GetTRIG_STATUS_DET0() uint32 {
	return volatile.LoadUint32(&o.TRIG_STATUS.Reg) & 0x1
}

// GLITCH_DETECTOR.TRIG_FORCE: Simulate the firing of one or more detectors. Writing ones to this register will set the matching bits in STATUS_TRIG. If the glitch detectors are currently armed, writing ones will also immediately reset the switched core power domain, and set the reset reason latches in POWMAN_CHIP_RESET to indicate a glitch detector resets. This register is Secure read/write only.
func (o *GLITCH_DETECTOR_Type) SetTRIG_FORCE(value uint32) {
	volatile.StoreUint32(&o.TRIG_FORCE.Reg, volatile.LoadUint32(&o.TRIG_FORCE.Reg)&^(0xf)|value)
}
func (o *GLITCH_DETECTOR_Type) GetTRIG_FORCE() uint32 {
	return volatile.LoadUint32(&o.TRIG_FORCE.Reg) & 0xf
}

// SNPS OTP control IF (SBPI and RPi wrapper control)
type OTP_Type struct {
	SW_LOCK0       volatile.Register32 // 0x0
	SW_LOCK1       volatile.Register32 // 0x4
	SW_LOCK2       volatile.Register32 // 0x8
	SW_LOCK3       volatile.Register32 // 0xC
	SW_LOCK4       volatile.Register32 // 0x10
	SW_LOCK5       volatile.Register32 // 0x14
	SW_LOCK6       volatile.Register32 // 0x18
	SW_LOCK7       volatile.Register32 // 0x1C
	SW_LOCK8       volatile.Register32 // 0x20
	SW_LOCK9       volatile.Register32 // 0x24
	SW_LOCK10      volatile.Register32 // 0x28
	SW_LOCK11      volatile.Register32 // 0x2C
	SW_LOCK12      volatile.Register32 // 0x30
	SW_LOCK13      volatile.Register32 // 0x34
	SW_LOCK14      volatile.Register32 // 0x38
	SW_LOCK15      volatile.Register32 // 0x3C
	SW_LOCK16      volatile.Register32 // 0x40
	SW_LOCK17      volatile.Register32 // 0x44
	SW_LOCK18      volatile.Register32 // 0x48
	SW_LOCK19      volatile.Register32 // 0x4C
	SW_LOCK20      volatile.Register32 // 0x50
	SW_LOCK21      volatile.Register32 // 0x54
	SW_LOCK22      volatile.Register32 // 0x58
	SW_LOCK23      volatile.Register32 // 0x5C
	SW_LOCK24      volatile.Register32 // 0x60
	SW_LOCK25      volatile.Register32 // 0x64
	SW_LOCK26      volatile.Register32 // 0x68
	SW_LOCK27      volatile.Register32 // 0x6C
	SW_LOCK28      volatile.Register32 // 0x70
	SW_LOCK29      volatile.Register32 // 0x74
	SW_LOCK30      volatile.Register32 // 0x78
	SW_LOCK31      volatile.Register32 // 0x7C
	SW_LOCK32      volatile.Register32 // 0x80
	SW_LOCK33      volatile.Register32 // 0x84
	SW_LOCK34      volatile.Register32 // 0x88
	SW_LOCK35      volatile.Register32 // 0x8C
	SW_LOCK36      volatile.Register32 // 0x90
	SW_LOCK37      volatile.Register32 // 0x94
	SW_LOCK38      volatile.Register32 // 0x98
	SW_LOCK39      volatile.Register32 // 0x9C
	SW_LOCK40      volatile.Register32 // 0xA0
	SW_LOCK41      volatile.Register32 // 0xA4
	SW_LOCK42      volatile.Register32 // 0xA8
	SW_LOCK43      volatile.Register32 // 0xAC
	SW_LOCK44      volatile.Register32 // 0xB0
	SW_LOCK45      volatile.Register32 // 0xB4
	SW_LOCK46      volatile.Register32 // 0xB8
	SW_LOCK47      volatile.Register32 // 0xBC
	SW_LOCK48      volatile.Register32 // 0xC0
	SW_LOCK49      volatile.Register32 // 0xC4
	SW_LOCK50      volatile.Register32 // 0xC8
	SW_LOCK51      volatile.Register32 // 0xCC
	SW_LOCK52      volatile.Register32 // 0xD0
	SW_LOCK53      volatile.Register32 // 0xD4
	SW_LOCK54      volatile.Register32 // 0xD8
	SW_LOCK55      volatile.Register32 // 0xDC
	SW_LOCK56      volatile.Register32 // 0xE0
	SW_LOCK57      volatile.Register32 // 0xE4
	SW_LOCK58      volatile.Register32 // 0xE8
	SW_LOCK59      volatile.Register32 // 0xEC
	SW_LOCK60      volatile.Register32 // 0xF0
	SW_LOCK61      volatile.Register32 // 0xF4
	SW_LOCK62      volatile.Register32 // 0xF8
	SW_LOCK63      volatile.Register32 // 0xFC
	SBPI_INSTR     volatile.Register32 // 0x100
	SBPI_WDATA_0   volatile.Register32 // 0x104
	SBPI_WDATA_1   volatile.Register32 // 0x108
	SBPI_WDATA_2   volatile.Register32 // 0x10C
	SBPI_WDATA_3   volatile.Register32 // 0x110
	SBPI_RDATA_0   volatile.Register32 // 0x114
	SBPI_RDATA_1   volatile.Register32 // 0x118
	SBPI_RDATA_2   volatile.Register32 // 0x11C
	SBPI_RDATA_3   volatile.Register32 // 0x120
	SBPI_STATUS    volatile.Register32 // 0x124
	USR            volatile.Register32 // 0x128
	DBG            volatile.Register32 // 0x12C
	_              [4]byte
	BIST           volatile.Register32 // 0x134
	CRT_KEY_W0     volatile.Register32 // 0x138
	CRT_KEY_W1     volatile.Register32 // 0x13C
	CRT_KEY_W2     volatile.Register32 // 0x140
	CRT_KEY_W3     volatile.Register32 // 0x144
	CRITICAL       volatile.Register32 // 0x148
	KEY_VALID      volatile.Register32 // 0x14C
	DEBUGEN        volatile.Register32 // 0x150
	DEBUGEN_LOCK   volatile.Register32 // 0x154
	ARCHSEL        volatile.Register32 // 0x158
	ARCHSEL_STATUS volatile.Register32 // 0x15C
	BOOTDIS        volatile.Register32 // 0x160
	INTR           volatile.Register32 // 0x164
	INTE           volatile.Register32 // 0x168
	INTF           volatile.Register32 // 0x16C
	INTS           volatile.Register32 // 0x170
}

// OTP.SW_LOCK0: Software lock register for page 0. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK0_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK0.Reg, volatile.LoadUint32(&o.SW_LOCK0.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK0_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK0.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK0_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK0.Reg, volatile.LoadUint32(&o.SW_LOCK0.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK0_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK0.Reg) & 0x3
}

// OTP.SW_LOCK1: Software lock register for page 1. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK1_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK1.Reg, volatile.LoadUint32(&o.SW_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK1_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK1_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK1.Reg, volatile.LoadUint32(&o.SW_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK1_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK1.Reg) & 0x3
}

// OTP.SW_LOCK2: Software lock register for page 2. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK2_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK2.Reg, volatile.LoadUint32(&o.SW_LOCK2.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK2_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK2.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK2_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK2.Reg, volatile.LoadUint32(&o.SW_LOCK2.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK2_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK2.Reg) & 0x3
}

// OTP.SW_LOCK3: Software lock register for page 3. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK3_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK3.Reg, volatile.LoadUint32(&o.SW_LOCK3.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK3_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK3.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK3_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK3.Reg, volatile.LoadUint32(&o.SW_LOCK3.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK3_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK3.Reg) & 0x3
}

// OTP.SW_LOCK4: Software lock register for page 4. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK4_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK4.Reg, volatile.LoadUint32(&o.SW_LOCK4.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK4_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK4.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK4_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK4.Reg, volatile.LoadUint32(&o.SW_LOCK4.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK4_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK4.Reg) & 0x3
}

// OTP.SW_LOCK5: Software lock register for page 5. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK5_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK5.Reg, volatile.LoadUint32(&o.SW_LOCK5.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK5_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK5.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK5_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK5.Reg, volatile.LoadUint32(&o.SW_LOCK5.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK5_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK5.Reg) & 0x3
}

// OTP.SW_LOCK6: Software lock register for page 6. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK6_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK6.Reg, volatile.LoadUint32(&o.SW_LOCK6.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK6_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK6.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK6_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK6.Reg, volatile.LoadUint32(&o.SW_LOCK6.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK6_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK6.Reg) & 0x3
}

// OTP.SW_LOCK7: Software lock register for page 7. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK7_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK7.Reg, volatile.LoadUint32(&o.SW_LOCK7.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK7_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK7.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK7_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK7.Reg, volatile.LoadUint32(&o.SW_LOCK7.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK7_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK7.Reg) & 0x3
}

// OTP.SW_LOCK8: Software lock register for page 8. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK8_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK8.Reg, volatile.LoadUint32(&o.SW_LOCK8.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK8_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK8.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK8_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK8.Reg, volatile.LoadUint32(&o.SW_LOCK8.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK8_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK8.Reg) & 0x3
}

// OTP.SW_LOCK9: Software lock register for page 9. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK9_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK9.Reg, volatile.LoadUint32(&o.SW_LOCK9.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK9_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK9.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK9_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK9.Reg, volatile.LoadUint32(&o.SW_LOCK9.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK9_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK9.Reg) & 0x3
}

// OTP.SW_LOCK10: Software lock register for page 10. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK10_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK10.Reg, volatile.LoadUint32(&o.SW_LOCK10.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK10_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK10.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK10_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK10.Reg, volatile.LoadUint32(&o.SW_LOCK10.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK10_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK10.Reg) & 0x3
}

// OTP.SW_LOCK11: Software lock register for page 11. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK11_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK11.Reg, volatile.LoadUint32(&o.SW_LOCK11.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK11_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK11.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK11_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK11.Reg, volatile.LoadUint32(&o.SW_LOCK11.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK11_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK11.Reg) & 0x3
}

// OTP.SW_LOCK12: Software lock register for page 12. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK12_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK12.Reg, volatile.LoadUint32(&o.SW_LOCK12.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK12_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK12.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK12_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK12.Reg, volatile.LoadUint32(&o.SW_LOCK12.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK12_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK12.Reg) & 0x3
}

// OTP.SW_LOCK13: Software lock register for page 13. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK13_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK13.Reg, volatile.LoadUint32(&o.SW_LOCK13.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK13_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK13.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK13_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK13.Reg, volatile.LoadUint32(&o.SW_LOCK13.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK13_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK13.Reg) & 0x3
}

// OTP.SW_LOCK14: Software lock register for page 14. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK14_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK14.Reg, volatile.LoadUint32(&o.SW_LOCK14.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK14_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK14.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK14_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK14.Reg, volatile.LoadUint32(&o.SW_LOCK14.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK14_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK14.Reg) & 0x3
}

// OTP.SW_LOCK15: Software lock register for page 15. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK15_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK15.Reg, volatile.LoadUint32(&o.SW_LOCK15.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK15_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK15.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK15_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK15.Reg, volatile.LoadUint32(&o.SW_LOCK15.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK15_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK15.Reg) & 0x3
}

// OTP.SW_LOCK16: Software lock register for page 16. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK16_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK16.Reg, volatile.LoadUint32(&o.SW_LOCK16.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK16_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK16.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK16_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK16.Reg, volatile.LoadUint32(&o.SW_LOCK16.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK16_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK16.Reg) & 0x3
}

// OTP.SW_LOCK17: Software lock register for page 17. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK17_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK17.Reg, volatile.LoadUint32(&o.SW_LOCK17.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK17_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK17.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK17_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK17.Reg, volatile.LoadUint32(&o.SW_LOCK17.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK17_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK17.Reg) & 0x3
}

// OTP.SW_LOCK18: Software lock register for page 18. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK18_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK18.Reg, volatile.LoadUint32(&o.SW_LOCK18.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK18_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK18.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK18_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK18.Reg, volatile.LoadUint32(&o.SW_LOCK18.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK18_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK18.Reg) & 0x3
}

// OTP.SW_LOCK19: Software lock register for page 19. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK19_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK19.Reg, volatile.LoadUint32(&o.SW_LOCK19.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK19_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK19.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK19_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK19.Reg, volatile.LoadUint32(&o.SW_LOCK19.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK19_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK19.Reg) & 0x3
}

// OTP.SW_LOCK20: Software lock register for page 20. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK20_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK20.Reg, volatile.LoadUint32(&o.SW_LOCK20.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK20_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK20.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK20_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK20.Reg, volatile.LoadUint32(&o.SW_LOCK20.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK20_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK20.Reg) & 0x3
}

// OTP.SW_LOCK21: Software lock register for page 21. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK21_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK21.Reg, volatile.LoadUint32(&o.SW_LOCK21.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK21_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK21.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK21_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK21.Reg, volatile.LoadUint32(&o.SW_LOCK21.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK21_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK21.Reg) & 0x3
}

// OTP.SW_LOCK22: Software lock register for page 22. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK22_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK22.Reg, volatile.LoadUint32(&o.SW_LOCK22.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK22_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK22.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK22_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK22.Reg, volatile.LoadUint32(&o.SW_LOCK22.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK22_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK22.Reg) & 0x3
}

// OTP.SW_LOCK23: Software lock register for page 23. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK23_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK23.Reg, volatile.LoadUint32(&o.SW_LOCK23.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK23_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK23.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK23_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK23.Reg, volatile.LoadUint32(&o.SW_LOCK23.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK23_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK23.Reg) & 0x3
}

// OTP.SW_LOCK24: Software lock register for page 24. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK24_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK24.Reg, volatile.LoadUint32(&o.SW_LOCK24.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK24_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK24.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK24_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK24.Reg, volatile.LoadUint32(&o.SW_LOCK24.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK24_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK24.Reg) & 0x3
}

// OTP.SW_LOCK25: Software lock register for page 25. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK25_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK25.Reg, volatile.LoadUint32(&o.SW_LOCK25.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK25_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK25.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK25_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK25.Reg, volatile.LoadUint32(&o.SW_LOCK25.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK25_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK25.Reg) & 0x3
}

// OTP.SW_LOCK26: Software lock register for page 26. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK26_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK26.Reg, volatile.LoadUint32(&o.SW_LOCK26.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK26_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK26.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK26_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK26.Reg, volatile.LoadUint32(&o.SW_LOCK26.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK26_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK26.Reg) & 0x3
}

// OTP.SW_LOCK27: Software lock register for page 27. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK27_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK27.Reg, volatile.LoadUint32(&o.SW_LOCK27.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK27_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK27.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK27_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK27.Reg, volatile.LoadUint32(&o.SW_LOCK27.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK27_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK27.Reg) & 0x3
}

// OTP.SW_LOCK28: Software lock register for page 28. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK28_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK28.Reg, volatile.LoadUint32(&o.SW_LOCK28.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK28_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK28.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK28_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK28.Reg, volatile.LoadUint32(&o.SW_LOCK28.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK28_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK28.Reg) & 0x3
}

// OTP.SW_LOCK29: Software lock register for page 29. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK29_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK29.Reg, volatile.LoadUint32(&o.SW_LOCK29.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK29_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK29.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK29_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK29.Reg, volatile.LoadUint32(&o.SW_LOCK29.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK29_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK29.Reg) & 0x3
}

// OTP.SW_LOCK30: Software lock register for page 30. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK30_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK30.Reg, volatile.LoadUint32(&o.SW_LOCK30.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK30_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK30.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK30_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK30.Reg, volatile.LoadUint32(&o.SW_LOCK30.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK30_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK30.Reg) & 0x3
}

// OTP.SW_LOCK31: Software lock register for page 31. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK31_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK31.Reg, volatile.LoadUint32(&o.SW_LOCK31.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK31_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK31.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK31_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK31.Reg, volatile.LoadUint32(&o.SW_LOCK31.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK31_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK31.Reg) & 0x3
}

// OTP.SW_LOCK32: Software lock register for page 32. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK32_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK32.Reg, volatile.LoadUint32(&o.SW_LOCK32.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK32_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK32.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK32_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK32.Reg, volatile.LoadUint32(&o.SW_LOCK32.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK32_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK32.Reg) & 0x3
}

// OTP.SW_LOCK33: Software lock register for page 33. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK33_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK33.Reg, volatile.LoadUint32(&o.SW_LOCK33.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK33_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK33.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK33_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK33.Reg, volatile.LoadUint32(&o.SW_LOCK33.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK33_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK33.Reg) & 0x3
}

// OTP.SW_LOCK34: Software lock register for page 34. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK34_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK34.Reg, volatile.LoadUint32(&o.SW_LOCK34.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK34_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK34.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK34_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK34.Reg, volatile.LoadUint32(&o.SW_LOCK34.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK34_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK34.Reg) & 0x3
}

// OTP.SW_LOCK35: Software lock register for page 35. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK35_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK35.Reg, volatile.LoadUint32(&o.SW_LOCK35.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK35_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK35.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK35_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK35.Reg, volatile.LoadUint32(&o.SW_LOCK35.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK35_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK35.Reg) & 0x3
}

// OTP.SW_LOCK36: Software lock register for page 36. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK36_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK36.Reg, volatile.LoadUint32(&o.SW_LOCK36.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK36_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK36.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK36_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK36.Reg, volatile.LoadUint32(&o.SW_LOCK36.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK36_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK36.Reg) & 0x3
}

// OTP.SW_LOCK37: Software lock register for page 37. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK37_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK37.Reg, volatile.LoadUint32(&o.SW_LOCK37.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK37_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK37.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK37_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK37.Reg, volatile.LoadUint32(&o.SW_LOCK37.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK37_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK37.Reg) & 0x3
}

// OTP.SW_LOCK38: Software lock register for page 38. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK38_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK38.Reg, volatile.LoadUint32(&o.SW_LOCK38.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK38_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK38.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK38_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK38.Reg, volatile.LoadUint32(&o.SW_LOCK38.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK38_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK38.Reg) & 0x3
}

// OTP.SW_LOCK39: Software lock register for page 39. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK39_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK39.Reg, volatile.LoadUint32(&o.SW_LOCK39.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK39_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK39.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK39_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK39.Reg, volatile.LoadUint32(&o.SW_LOCK39.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK39_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK39.Reg) & 0x3
}

// OTP.SW_LOCK40: Software lock register for page 40. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK40_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK40.Reg, volatile.LoadUint32(&o.SW_LOCK40.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK40_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK40.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK40_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK40.Reg, volatile.LoadUint32(&o.SW_LOCK40.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK40_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK40.Reg) & 0x3
}

// OTP.SW_LOCK41: Software lock register for page 41. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK41_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK41.Reg, volatile.LoadUint32(&o.SW_LOCK41.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK41_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK41.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK41_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK41.Reg, volatile.LoadUint32(&o.SW_LOCK41.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK41_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK41.Reg) & 0x3
}

// OTP.SW_LOCK42: Software lock register for page 42. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK42_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK42.Reg, volatile.LoadUint32(&o.SW_LOCK42.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK42_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK42.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK42_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK42.Reg, volatile.LoadUint32(&o.SW_LOCK42.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK42_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK42.Reg) & 0x3
}

// OTP.SW_LOCK43: Software lock register for page 43. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK43_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK43.Reg, volatile.LoadUint32(&o.SW_LOCK43.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK43_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK43.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK43_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK43.Reg, volatile.LoadUint32(&o.SW_LOCK43.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK43_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK43.Reg) & 0x3
}

// OTP.SW_LOCK44: Software lock register for page 44. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK44_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK44.Reg, volatile.LoadUint32(&o.SW_LOCK44.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK44_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK44.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK44_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK44.Reg, volatile.LoadUint32(&o.SW_LOCK44.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK44_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK44.Reg) & 0x3
}

// OTP.SW_LOCK45: Software lock register for page 45. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK45_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK45.Reg, volatile.LoadUint32(&o.SW_LOCK45.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK45_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK45.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK45_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK45.Reg, volatile.LoadUint32(&o.SW_LOCK45.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK45_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK45.Reg) & 0x3
}

// OTP.SW_LOCK46: Software lock register for page 46. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK46_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK46.Reg, volatile.LoadUint32(&o.SW_LOCK46.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK46_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK46.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK46_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK46.Reg, volatile.LoadUint32(&o.SW_LOCK46.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK46_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK46.Reg) & 0x3
}

// OTP.SW_LOCK47: Software lock register for page 47. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK47_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK47.Reg, volatile.LoadUint32(&o.SW_LOCK47.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK47_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK47.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK47_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK47.Reg, volatile.LoadUint32(&o.SW_LOCK47.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK47_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK47.Reg) & 0x3
}

// OTP.SW_LOCK48: Software lock register for page 48. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK48_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK48.Reg, volatile.LoadUint32(&o.SW_LOCK48.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK48_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK48.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK48_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK48.Reg, volatile.LoadUint32(&o.SW_LOCK48.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK48_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK48.Reg) & 0x3
}

// OTP.SW_LOCK49: Software lock register for page 49. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK49_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK49.Reg, volatile.LoadUint32(&o.SW_LOCK49.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK49_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK49.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK49_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK49.Reg, volatile.LoadUint32(&o.SW_LOCK49.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK49_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK49.Reg) & 0x3
}

// OTP.SW_LOCK50: Software lock register for page 50. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK50_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK50.Reg, volatile.LoadUint32(&o.SW_LOCK50.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK50_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK50.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK50_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK50.Reg, volatile.LoadUint32(&o.SW_LOCK50.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK50_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK50.Reg) & 0x3
}

// OTP.SW_LOCK51: Software lock register for page 51. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK51_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK51.Reg, volatile.LoadUint32(&o.SW_LOCK51.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK51_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK51.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK51_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK51.Reg, volatile.LoadUint32(&o.SW_LOCK51.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK51_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK51.Reg) & 0x3
}

// OTP.SW_LOCK52: Software lock register for page 52. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK52_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK52.Reg, volatile.LoadUint32(&o.SW_LOCK52.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK52_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK52.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK52_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK52.Reg, volatile.LoadUint32(&o.SW_LOCK52.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK52_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK52.Reg) & 0x3
}

// OTP.SW_LOCK53: Software lock register for page 53. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK53_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK53.Reg, volatile.LoadUint32(&o.SW_LOCK53.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK53_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK53.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK53_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK53.Reg, volatile.LoadUint32(&o.SW_LOCK53.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK53_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK53.Reg) & 0x3
}

// OTP.SW_LOCK54: Software lock register for page 54. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK54_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK54.Reg, volatile.LoadUint32(&o.SW_LOCK54.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK54_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK54.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK54_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK54.Reg, volatile.LoadUint32(&o.SW_LOCK54.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK54_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK54.Reg) & 0x3
}

// OTP.SW_LOCK55: Software lock register for page 55. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK55_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK55.Reg, volatile.LoadUint32(&o.SW_LOCK55.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK55_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK55.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK55_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK55.Reg, volatile.LoadUint32(&o.SW_LOCK55.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK55_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK55.Reg) & 0x3
}

// OTP.SW_LOCK56: Software lock register for page 56. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK56_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK56.Reg, volatile.LoadUint32(&o.SW_LOCK56.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK56_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK56.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK56_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK56.Reg, volatile.LoadUint32(&o.SW_LOCK56.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK56_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK56.Reg) & 0x3
}

// OTP.SW_LOCK57: Software lock register for page 57. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK57_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK57.Reg, volatile.LoadUint32(&o.SW_LOCK57.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK57_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK57.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK57_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK57.Reg, volatile.LoadUint32(&o.SW_LOCK57.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK57_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK57.Reg) & 0x3
}

// OTP.SW_LOCK58: Software lock register for page 58. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK58_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK58.Reg, volatile.LoadUint32(&o.SW_LOCK58.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK58_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK58.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK58_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK58.Reg, volatile.LoadUint32(&o.SW_LOCK58.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK58_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK58.Reg) & 0x3
}

// OTP.SW_LOCK59: Software lock register for page 59. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK59_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK59.Reg, volatile.LoadUint32(&o.SW_LOCK59.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK59_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK59.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK59_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK59.Reg, volatile.LoadUint32(&o.SW_LOCK59.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK59_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK59.Reg) & 0x3
}

// OTP.SW_LOCK60: Software lock register for page 60. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK60_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK60.Reg, volatile.LoadUint32(&o.SW_LOCK60.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK60_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK60.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK60_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK60.Reg, volatile.LoadUint32(&o.SW_LOCK60.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK60_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK60.Reg) & 0x3
}

// OTP.SW_LOCK61: Software lock register for page 61. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK61_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK61.Reg, volatile.LoadUint32(&o.SW_LOCK61.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK61_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK61.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK61_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK61.Reg, volatile.LoadUint32(&o.SW_LOCK61.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK61_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK61.Reg) & 0x3
}

// OTP.SW_LOCK62: Software lock register for page 62. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK62_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK62.Reg, volatile.LoadUint32(&o.SW_LOCK62.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK62_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK62.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK62_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK62.Reg, volatile.LoadUint32(&o.SW_LOCK62.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK62_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK62.Reg) & 0x3
}

// OTP.SW_LOCK63: Software lock register for page 63. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
func (o *OTP_Type) SetSW_LOCK63_NSEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK63.Reg, volatile.LoadUint32(&o.SW_LOCK63.Reg)&^(0xc)|value<<2)
}
func (o *OTP_Type) GetSW_LOCK63_NSEC() uint32 {
	return (volatile.LoadUint32(&o.SW_LOCK63.Reg) & 0xc) >> 2
}
func (o *OTP_Type) SetSW_LOCK63_SEC(value uint32) {
	volatile.StoreUint32(&o.SW_LOCK63.Reg, volatile.LoadUint32(&o.SW_LOCK63.Reg)&^(0x3)|value)
}
func (o *OTP_Type) GetSW_LOCK63_SEC() uint32 {
	return volatile.LoadUint32(&o.SW_LOCK63.Reg) & 0x3
}

// OTP.SBPI_INSTR: Dispatch instructions to the SBPI interface, used for programming the OTP fuses.
func (o *OTP_Type) SetSBPI_INSTR_EXEC(value uint32) {
	volatile.StoreUint32(&o.SBPI_INSTR.Reg, volatile.LoadUint32(&o.SBPI_INSTR.Reg)&^(0x40000000)|value<<30)
}
func (o *OTP_Type) GetSBPI_INSTR_EXEC() uint32 {
	return (volatile.LoadUint32(&o.SBPI_INSTR.Reg) & 0x40000000) >> 30
}
func (o *OTP_Type) SetSBPI_INSTR_IS_WR(value uint32) {
	volatile.StoreUint32(&o.SBPI_INSTR.Reg, volatile.LoadUint32(&o.SBPI_INSTR.Reg)&^(0x20000000)|value<<29)
}
func (o *OTP_Type) GetSBPI_INSTR_IS_WR() uint32 {
	return (volatile.LoadUint32(&o.SBPI_INSTR.Reg) & 0x20000000) >> 29
}
func (o *OTP_Type) SetSBPI_INSTR_HAS_PAYLOAD(value uint32) {
	volatile.StoreUint32(&o.SBPI_INSTR.Reg, volatile.LoadUint32(&o.SBPI_INSTR.Reg)&^(0x10000000)|value<<28)
}
func (o *OTP_Type) GetSBPI_INSTR_HAS_PAYLOAD() uint32 {
	return (volatile.LoadUint32(&o.SBPI_INSTR.Reg) & 0x10000000) >> 28
}
func (o *OTP_Type) SetSBPI_INSTR_PAYLOAD_SIZE_M1(value uint32) {
	volatile.StoreUint32(&o.SBPI_INSTR.Reg, volatile.LoadUint32(&o.SBPI_INSTR.Reg)&^(0xf000000)|value<<24)
}
func (o *OTP_Type) GetSBPI_INSTR_PAYLOAD_SIZE_M1() uint32 {
	return (volatile.LoadUint32(&o.SBPI_INSTR.Reg) & 0xf000000) >> 24
}
func (o *OTP_Type) SetSBPI_INSTR_TARGET(value uint32) {
	volatile.StoreUint32(&o.SBPI_INSTR.Reg, volatile.LoadUint32(&o.SBPI_INSTR.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_Type) GetSBPI_INSTR_TARGET() uint32 {
	return (volatile.LoadUint32(&o.SBPI_INSTR.Reg) & 0xff0000) >> 16
}
func (o *OTP_Type) SetSBPI_INSTR_CMD(value uint32) {
	volatile.StoreUint32(&o.SBPI_INSTR.Reg, volatile.LoadUint32(&o.SBPI_INSTR.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_Type) GetSBPI_INSTR_CMD() uint32 {
	return (volatile.LoadUint32(&o.SBPI_INSTR.Reg) & 0xff00) >> 8
}
func (o *OTP_Type) SetSBPI_INSTR_SHORT_WDATA(value uint32) {
	volatile.StoreUint32(&o.SBPI_INSTR.Reg, volatile.LoadUint32(&o.SBPI_INSTR.Reg)&^(0xff)|value)
}
func (o *OTP_Type) GetSBPI_INSTR_SHORT_WDATA() uint32 {
	return volatile.LoadUint32(&o.SBPI_INSTR.Reg) & 0xff
}

// OTP.SBPI_WDATA_0: SBPI write payload bytes 3..0
func (o *OTP_Type) SetSBPI_WDATA_0(value uint32) {
	volatile.StoreUint32(&o.SBPI_WDATA_0.Reg, value)
}
func (o *OTP_Type) GetSBPI_WDATA_0() uint32 {
	return volatile.LoadUint32(&o.SBPI_WDATA_0.Reg)
}

// OTP.SBPI_WDATA_1: SBPI write payload bytes 7..4
func (o *OTP_Type) SetSBPI_WDATA_1(value uint32) {
	volatile.StoreUint32(&o.SBPI_WDATA_1.Reg, value)
}
func (o *OTP_Type) GetSBPI_WDATA_1() uint32 {
	return volatile.LoadUint32(&o.SBPI_WDATA_1.Reg)
}

// OTP.SBPI_WDATA_2: SBPI write payload bytes 11..8
func (o *OTP_Type) SetSBPI_WDATA_2(value uint32) {
	volatile.StoreUint32(&o.SBPI_WDATA_2.Reg, value)
}
func (o *OTP_Type) GetSBPI_WDATA_2() uint32 {
	return volatile.LoadUint32(&o.SBPI_WDATA_2.Reg)
}

// OTP.SBPI_WDATA_3: SBPI write payload bytes 15..12
func (o *OTP_Type) SetSBPI_WDATA_3(value uint32) {
	volatile.StoreUint32(&o.SBPI_WDATA_3.Reg, value)
}
func (o *OTP_Type) GetSBPI_WDATA_3() uint32 {
	return volatile.LoadUint32(&o.SBPI_WDATA_3.Reg)
}

// OTP.SBPI_RDATA_0: Read payload bytes 3..0. Once read, the data in the register will automatically clear to 0.
func (o *OTP_Type) SetSBPI_RDATA_0(value uint32) {
	volatile.StoreUint32(&o.SBPI_RDATA_0.Reg, value)
}
func (o *OTP_Type) GetSBPI_RDATA_0() uint32 {
	return volatile.LoadUint32(&o.SBPI_RDATA_0.Reg)
}

// OTP.SBPI_RDATA_1: Read payload bytes 7..4. Once read, the data in the register will automatically clear to 0.
func (o *OTP_Type) SetSBPI_RDATA_1(value uint32) {
	volatile.StoreUint32(&o.SBPI_RDATA_1.Reg, value)
}
func (o *OTP_Type) GetSBPI_RDATA_1() uint32 {
	return volatile.LoadUint32(&o.SBPI_RDATA_1.Reg)
}

// OTP.SBPI_RDATA_2: Read payload bytes 11..8. Once read, the data in the register will automatically clear to 0.
func (o *OTP_Type) SetSBPI_RDATA_2(value uint32) {
	volatile.StoreUint32(&o.SBPI_RDATA_2.Reg, value)
}
func (o *OTP_Type) GetSBPI_RDATA_2() uint32 {
	return volatile.LoadUint32(&o.SBPI_RDATA_2.Reg)
}

// OTP.SBPI_RDATA_3: Read payload bytes 15..12. Once read, the data in the register will automatically clear to 0.
func (o *OTP_Type) SetSBPI_RDATA_3(value uint32) {
	volatile.StoreUint32(&o.SBPI_RDATA_3.Reg, value)
}
func (o *OTP_Type) GetSBPI_RDATA_3() uint32 {
	return volatile.LoadUint32(&o.SBPI_RDATA_3.Reg)
}

// OTP.SBPI_STATUS
func (o *OTP_Type) SetSBPI_STATUS_MISO(value uint32) {
	volatile.StoreUint32(&o.SBPI_STATUS.Reg, volatile.LoadUint32(&o.SBPI_STATUS.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_Type) GetSBPI_STATUS_MISO() uint32 {
	return (volatile.LoadUint32(&o.SBPI_STATUS.Reg) & 0xff0000) >> 16
}
func (o *OTP_Type) SetSBPI_STATUS_FLAG(value uint32) {
	volatile.StoreUint32(&o.SBPI_STATUS.Reg, volatile.LoadUint32(&o.SBPI_STATUS.Reg)&^(0x1000)|value<<12)
}
func (o *OTP_Type) GetSBPI_STATUS_FLAG() uint32 {
	return (volatile.LoadUint32(&o.SBPI_STATUS.Reg) & 0x1000) >> 12
}
func (o *OTP_Type) SetSBPI_STATUS_INSTR_MISS(value uint32) {
	volatile.StoreUint32(&o.SBPI_STATUS.Reg, volatile.LoadUint32(&o.SBPI_STATUS.Reg)&^(0x100)|value<<8)
}
func (o *OTP_Type) GetSBPI_STATUS_INSTR_MISS() uint32 {
	return (volatile.LoadUint32(&o.SBPI_STATUS.Reg) & 0x100) >> 8
}
func (o *OTP_Type) SetSBPI_STATUS_INSTR_DONE(value uint32) {
	volatile.StoreUint32(&o.SBPI_STATUS.Reg, volatile.LoadUint32(&o.SBPI_STATUS.Reg)&^(0x10)|value<<4)
}
func (o *OTP_Type) GetSBPI_STATUS_INSTR_DONE() uint32 {
	return (volatile.LoadUint32(&o.SBPI_STATUS.Reg) & 0x10) >> 4
}
func (o *OTP_Type) SetSBPI_STATUS_RDATA_VLD(value uint32) {
	volatile.StoreUint32(&o.SBPI_STATUS.Reg, volatile.LoadUint32(&o.SBPI_STATUS.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetSBPI_STATUS_RDATA_VLD() uint32 {
	return volatile.LoadUint32(&o.SBPI_STATUS.Reg) & 0x1
}

// OTP.USR: Controls for APB data read interface (USER interface)
func (o *OTP_Type) SetUSR_PD(value uint32) {
	volatile.StoreUint32(&o.USR.Reg, volatile.LoadUint32(&o.USR.Reg)&^(0x10)|value<<4)
}
func (o *OTP_Type) GetUSR_PD() uint32 {
	return (volatile.LoadUint32(&o.USR.Reg) & 0x10) >> 4
}
func (o *OTP_Type) SetUSR_DCTRL(value uint32) {
	volatile.StoreUint32(&o.USR.Reg, volatile.LoadUint32(&o.USR.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetUSR_DCTRL() uint32 {
	return volatile.LoadUint32(&o.USR.Reg) & 0x1
}

// OTP.DBG: Debug for OTP power-on state machine
func (o *OTP_Type) SetDBG_CUSTOMER_RMA_FLAG(value uint32) {
	volatile.StoreUint32(&o.DBG.Reg, volatile.LoadUint32(&o.DBG.Reg)&^(0x1000)|value<<12)
}
func (o *OTP_Type) GetDBG_CUSTOMER_RMA_FLAG() uint32 {
	return (volatile.LoadUint32(&o.DBG.Reg) & 0x1000) >> 12
}
func (o *OTP_Type) SetDBG_PSM_STATE(value uint32) {
	volatile.StoreUint32(&o.DBG.Reg, volatile.LoadUint32(&o.DBG.Reg)&^(0xf0)|value<<4)
}
func (o *OTP_Type) GetDBG_PSM_STATE() uint32 {
	return (volatile.LoadUint32(&o.DBG.Reg) & 0xf0) >> 4
}
func (o *OTP_Type) SetDBG_ROSC_UP(value uint32) {
	volatile.StoreUint32(&o.DBG.Reg, volatile.LoadUint32(&o.DBG.Reg)&^(0x8)|value<<3)
}
func (o *OTP_Type) GetDBG_ROSC_UP() uint32 {
	return (volatile.LoadUint32(&o.DBG.Reg) & 0x8) >> 3
}
func (o *OTP_Type) SetDBG_ROSC_UP_SEEN(value uint32) {
	volatile.StoreUint32(&o.DBG.Reg, volatile.LoadUint32(&o.DBG.Reg)&^(0x4)|value<<2)
}
func (o *OTP_Type) GetDBG_ROSC_UP_SEEN() uint32 {
	return (volatile.LoadUint32(&o.DBG.Reg) & 0x4) >> 2
}
func (o *OTP_Type) SetDBG_BOOT_DONE(value uint32) {
	volatile.StoreUint32(&o.DBG.Reg, volatile.LoadUint32(&o.DBG.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetDBG_BOOT_DONE() uint32 {
	return (volatile.LoadUint32(&o.DBG.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetDBG_PSM_DONE(value uint32) {
	volatile.StoreUint32(&o.DBG.Reg, volatile.LoadUint32(&o.DBG.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetDBG_PSM_DONE() uint32 {
	return volatile.LoadUint32(&o.DBG.Reg) & 0x1
}

// OTP.BIST: During BIST, count address locations that have at least one leaky bit
func (o *OTP_Type) SetBIST_CNT_FAIL(value uint32) {
	volatile.StoreUint32(&o.BIST.Reg, volatile.LoadUint32(&o.BIST.Reg)&^(0x40000000)|value<<30)
}
func (o *OTP_Type) GetBIST_CNT_FAIL() uint32 {
	return (volatile.LoadUint32(&o.BIST.Reg) & 0x40000000) >> 30
}
func (o *OTP_Type) SetBIST_CNT_CLR(value uint32) {
	volatile.StoreUint32(&o.BIST.Reg, volatile.LoadUint32(&o.BIST.Reg)&^(0x20000000)|value<<29)
}
func (o *OTP_Type) GetBIST_CNT_CLR() uint32 {
	return (volatile.LoadUint32(&o.BIST.Reg) & 0x20000000) >> 29
}
func (o *OTP_Type) SetBIST_CNT_ENA(value uint32) {
	volatile.StoreUint32(&o.BIST.Reg, volatile.LoadUint32(&o.BIST.Reg)&^(0x10000000)|value<<28)
}
func (o *OTP_Type) GetBIST_CNT_ENA() uint32 {
	return (volatile.LoadUint32(&o.BIST.Reg) & 0x10000000) >> 28
}
func (o *OTP_Type) SetBIST_CNT_MAX(value uint32) {
	volatile.StoreUint32(&o.BIST.Reg, volatile.LoadUint32(&o.BIST.Reg)&^(0xfff0000)|value<<16)
}
func (o *OTP_Type) GetBIST_CNT_MAX() uint32 {
	return (volatile.LoadUint32(&o.BIST.Reg) & 0xfff0000) >> 16
}
func (o *OTP_Type) SetBIST_CNT(value uint32) {
	volatile.StoreUint32(&o.BIST.Reg, volatile.LoadUint32(&o.BIST.Reg)&^(0x1fff)|value)
}
func (o *OTP_Type) GetBIST_CNT() uint32 {
	return volatile.LoadUint32(&o.BIST.Reg) & 0x1fff
}

// OTP.CRT_KEY_W0: Word 0 (bits 31..0) of the key. Write only, read returns 0x0
func (o *OTP_Type) SetCRT_KEY_W0(value uint32) {
	volatile.StoreUint32(&o.CRT_KEY_W0.Reg, value)
}
func (o *OTP_Type) GetCRT_KEY_W0() uint32 {
	return volatile.LoadUint32(&o.CRT_KEY_W0.Reg)
}

// OTP.CRT_KEY_W1: Word 1 (bits 63..32) of the key. Write only, read returns 0x0
func (o *OTP_Type) SetCRT_KEY_W1(value uint32) {
	volatile.StoreUint32(&o.CRT_KEY_W1.Reg, value)
}
func (o *OTP_Type) GetCRT_KEY_W1() uint32 {
	return volatile.LoadUint32(&o.CRT_KEY_W1.Reg)
}

// OTP.CRT_KEY_W2: Word 2 (bits 95..64) of the key. Write only, read returns 0x0
func (o *OTP_Type) SetCRT_KEY_W2(value uint32) {
	volatile.StoreUint32(&o.CRT_KEY_W2.Reg, value)
}
func (o *OTP_Type) GetCRT_KEY_W2() uint32 {
	return volatile.LoadUint32(&o.CRT_KEY_W2.Reg)
}

// OTP.CRT_KEY_W3: Word 3 (bits 127..96) of the key. Write only, read returns 0x0
func (o *OTP_Type) SetCRT_KEY_W3(value uint32) {
	volatile.StoreUint32(&o.CRT_KEY_W3.Reg, value)
}
func (o *OTP_Type) GetCRT_KEY_W3() uint32 {
	return volatile.LoadUint32(&o.CRT_KEY_W3.Reg)
}

// OTP.CRITICAL: Quickly check values of critical flags read during boot up
func (o *OTP_Type) SetCRITICAL_RISCV_DISABLE(value uint32) {
	volatile.StoreUint32(&o.CRITICAL.Reg, volatile.LoadUint32(&o.CRITICAL.Reg)&^(0x20000)|value<<17)
}
func (o *OTP_Type) GetCRITICAL_RISCV_DISABLE() uint32 {
	return (volatile.LoadUint32(&o.CRITICAL.Reg) & 0x20000) >> 17
}
func (o *OTP_Type) SetCRITICAL_ARM_DISABLE(value uint32) {
	volatile.StoreUint32(&o.CRITICAL.Reg, volatile.LoadUint32(&o.CRITICAL.Reg)&^(0x10000)|value<<16)
}
func (o *OTP_Type) GetCRITICAL_ARM_DISABLE() uint32 {
	return (volatile.LoadUint32(&o.CRITICAL.Reg) & 0x10000) >> 16
}
func (o *OTP_Type) SetCRITICAL_GLITCH_DETECTOR_SENS(value uint32) {
	volatile.StoreUint32(&o.CRITICAL.Reg, volatile.LoadUint32(&o.CRITICAL.Reg)&^(0x60)|value<<5)
}
func (o *OTP_Type) GetCRITICAL_GLITCH_DETECTOR_SENS() uint32 {
	return (volatile.LoadUint32(&o.CRITICAL.Reg) & 0x60) >> 5
}
func (o *OTP_Type) SetCRITICAL_GLITCH_DETECTOR_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CRITICAL.Reg, volatile.LoadUint32(&o.CRITICAL.Reg)&^(0x10)|value<<4)
}
func (o *OTP_Type) GetCRITICAL_GLITCH_DETECTOR_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CRITICAL.Reg) & 0x10) >> 4
}
func (o *OTP_Type) SetCRITICAL_DEFAULT_ARCHSEL(value uint32) {
	volatile.StoreUint32(&o.CRITICAL.Reg, volatile.LoadUint32(&o.CRITICAL.Reg)&^(0x8)|value<<3)
}
func (o *OTP_Type) GetCRITICAL_DEFAULT_ARCHSEL() uint32 {
	return (volatile.LoadUint32(&o.CRITICAL.Reg) & 0x8) >> 3
}
func (o *OTP_Type) SetCRITICAL_DEBUG_DISABLE(value uint32) {
	volatile.StoreUint32(&o.CRITICAL.Reg, volatile.LoadUint32(&o.CRITICAL.Reg)&^(0x4)|value<<2)
}
func (o *OTP_Type) GetCRITICAL_DEBUG_DISABLE() uint32 {
	return (volatile.LoadUint32(&o.CRITICAL.Reg) & 0x4) >> 2
}
func (o *OTP_Type) SetCRITICAL_SECURE_DEBUG_DISABLE(value uint32) {
	volatile.StoreUint32(&o.CRITICAL.Reg, volatile.LoadUint32(&o.CRITICAL.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetCRITICAL_SECURE_DEBUG_DISABLE() uint32 {
	return (volatile.LoadUint32(&o.CRITICAL.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetCRITICAL_SECURE_BOOT_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CRITICAL.Reg, volatile.LoadUint32(&o.CRITICAL.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetCRITICAL_SECURE_BOOT_ENABLE() uint32 {
	return volatile.LoadUint32(&o.CRITICAL.Reg) & 0x1
}

// OTP.KEY_VALID: Which keys were valid (enrolled) at boot time
func (o *OTP_Type) SetKEY_VALID(value uint32) {
	volatile.StoreUint32(&o.KEY_VALID.Reg, volatile.LoadUint32(&o.KEY_VALID.Reg)&^(0xff)|value)
}
func (o *OTP_Type) GetKEY_VALID() uint32 {
	return volatile.LoadUint32(&o.KEY_VALID.Reg) & 0xff
}

// OTP.DEBUGEN: Enable a debug feature that has been disabled. Debug features are disabled if one of the relevant critical boot flags is set in OTP (DEBUG_DISABLE or SECURE_DEBUG_DISABLE), OR if a debug key is marked valid in OTP, and the matching key value has not been supplied over SWD. Specifically: - The DEBUG_DISABLE flag disables all debug features. This can be fully overridden by setting all bits of this register. - The SECURE_DEBUG_DISABLE flag disables secure processor debug. This can be fully overridden by setting the PROC0_SECURE and PROC1_SECURE bits of this register. - If a single debug key has been registered, and no matching key value has been supplied over SWD, then all debug features are disabled. This can be fully overridden by setting all bits of this register. - If both debug keys have been registered, and the Non-secure key's value (key 6) has been supplied over SWD, secure processor debug is disabled. This can be fully overridden by setting the PROC0_SECURE and PROC1_SECURE bits of this register. - If both debug keys have been registered, and the Secure key's value (key 5) has been supplied over SWD, then no debug features are disabled by the key mechanism. However, note that in this case debug features may still be disabled by the critical boot flags.
func (o *OTP_Type) SetDEBUGEN_MISC(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN.Reg, volatile.LoadUint32(&o.DEBUGEN.Reg)&^(0x100)|value<<8)
}
func (o *OTP_Type) GetDEBUGEN_MISC() uint32 {
	return (volatile.LoadUint32(&o.DEBUGEN.Reg) & 0x100) >> 8
}
func (o *OTP_Type) SetDEBUGEN_PROC1_SECURE(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN.Reg, volatile.LoadUint32(&o.DEBUGEN.Reg)&^(0x8)|value<<3)
}
func (o *OTP_Type) GetDEBUGEN_PROC1_SECURE() uint32 {
	return (volatile.LoadUint32(&o.DEBUGEN.Reg) & 0x8) >> 3
}
func (o *OTP_Type) SetDEBUGEN_PROC1(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN.Reg, volatile.LoadUint32(&o.DEBUGEN.Reg)&^(0x4)|value<<2)
}
func (o *OTP_Type) GetDEBUGEN_PROC1() uint32 {
	return (volatile.LoadUint32(&o.DEBUGEN.Reg) & 0x4) >> 2
}
func (o *OTP_Type) SetDEBUGEN_PROC0_SECURE(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN.Reg, volatile.LoadUint32(&o.DEBUGEN.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetDEBUGEN_PROC0_SECURE() uint32 {
	return (volatile.LoadUint32(&o.DEBUGEN.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetDEBUGEN_PROC0(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN.Reg, volatile.LoadUint32(&o.DEBUGEN.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetDEBUGEN_PROC0() uint32 {
	return volatile.LoadUint32(&o.DEBUGEN.Reg) & 0x1
}

// OTP.DEBUGEN_LOCK: Write 1s to lock corresponding bits in DEBUGEN. This register is reset by the processor cold reset.
func (o *OTP_Type) SetDEBUGEN_LOCK_MISC(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN_LOCK.Reg, volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg)&^(0x100)|value<<8)
}
func (o *OTP_Type) GetDEBUGEN_LOCK_MISC() uint32 {
	return (volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg) & 0x100) >> 8
}
func (o *OTP_Type) SetDEBUGEN_LOCK_PROC1_SECURE(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN_LOCK.Reg, volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg)&^(0x8)|value<<3)
}
func (o *OTP_Type) GetDEBUGEN_LOCK_PROC1_SECURE() uint32 {
	return (volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg) & 0x8) >> 3
}
func (o *OTP_Type) SetDEBUGEN_LOCK_PROC1(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN_LOCK.Reg, volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg)&^(0x4)|value<<2)
}
func (o *OTP_Type) GetDEBUGEN_LOCK_PROC1() uint32 {
	return (volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg) & 0x4) >> 2
}
func (o *OTP_Type) SetDEBUGEN_LOCK_PROC0_SECURE(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN_LOCK.Reg, volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetDEBUGEN_LOCK_PROC0_SECURE() uint32 {
	return (volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetDEBUGEN_LOCK_PROC0(value uint32) {
	volatile.StoreUint32(&o.DEBUGEN_LOCK.Reg, volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetDEBUGEN_LOCK_PROC0() uint32 {
	return volatile.LoadUint32(&o.DEBUGEN_LOCK.Reg) & 0x1
}

// OTP.ARCHSEL: Architecture select (Arm/RISC-V). The default and allowable values of this register are constrained by the critical boot flags. This register is reset by the earliest reset in the switched core power domain (before a processor cold reset). Cores sample their architecture select signal on a warm reset. The source of the warm reset could be the system power-up state machine, the watchdog timer, Arm SYSRESETREQ or from RISC-V hartresetreq. Note that when an Arm core is deselected, its cold reset domain is also held in reset, since in particular the SYSRESETREQ bit becomes inaccessible once the core is deselected. Note also the RISC-V cores do not have a cold reset domain, since their corresponding controls are located in the Debug Module.
func (o *OTP_Type) SetARCHSEL_CORE1(value uint32) {
	volatile.StoreUint32(&o.ARCHSEL.Reg, volatile.LoadUint32(&o.ARCHSEL.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetARCHSEL_CORE1() uint32 {
	return (volatile.LoadUint32(&o.ARCHSEL.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetARCHSEL_CORE0(value uint32) {
	volatile.StoreUint32(&o.ARCHSEL.Reg, volatile.LoadUint32(&o.ARCHSEL.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetARCHSEL_CORE0() uint32 {
	return volatile.LoadUint32(&o.ARCHSEL.Reg) & 0x1
}

// OTP.ARCHSEL_STATUS: Get the current architecture select state of each core. Cores sample the current value of the ARCHSEL register when their warm reset is released, at which point the corresponding bit in this register will also update.
func (o *OTP_Type) SetARCHSEL_STATUS_CORE1(value uint32) {
	volatile.StoreUint32(&o.ARCHSEL_STATUS.Reg, volatile.LoadUint32(&o.ARCHSEL_STATUS.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetARCHSEL_STATUS_CORE1() uint32 {
	return (volatile.LoadUint32(&o.ARCHSEL_STATUS.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetARCHSEL_STATUS_CORE0(value uint32) {
	volatile.StoreUint32(&o.ARCHSEL_STATUS.Reg, volatile.LoadUint32(&o.ARCHSEL_STATUS.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetARCHSEL_STATUS_CORE0() uint32 {
	return volatile.LoadUint32(&o.ARCHSEL_STATUS.Reg) & 0x1
}

// OTP.BOOTDIS: Tell the bootrom to ignore scratch register boot vectors (both power manager and watchdog) on the next power up. If an early boot stage has soft-locked some OTP pages in order to protect their contents from later stages, there is a risk that Secure code running at a later stage can unlock the pages by performing a watchdog reset that resets the OTP. This register can be used to ensure that the bootloader runs as normal on the next power up, preventing Secure code at a later stage from accessing OTP in its unlocked state. Should be used in conjunction with the power manager BOOTDIS register.
func (o *OTP_Type) SetBOOTDIS_NEXT(value uint32) {
	volatile.StoreUint32(&o.BOOTDIS.Reg, volatile.LoadUint32(&o.BOOTDIS.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetBOOTDIS_NEXT() uint32 {
	return (volatile.LoadUint32(&o.BOOTDIS.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetBOOTDIS_NOW(value uint32) {
	volatile.StoreUint32(&o.BOOTDIS.Reg, volatile.LoadUint32(&o.BOOTDIS.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetBOOTDIS_NOW() uint32 {
	return volatile.LoadUint32(&o.BOOTDIS.Reg) & 0x1
}

// OTP.INTR: Raw Interrupts
func (o *OTP_Type) SetINTR_APB_RD_NSEC_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x10)|value<<4)
}
func (o *OTP_Type) GetINTR_APB_RD_NSEC_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x10) >> 4
}
func (o *OTP_Type) SetINTR_APB_RD_SEC_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x8)|value<<3)
}
func (o *OTP_Type) GetINTR_APB_RD_SEC_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x8) >> 3
}
func (o *OTP_Type) SetINTR_APB_DCTRL_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x4)|value<<2)
}
func (o *OTP_Type) GetINTR_APB_DCTRL_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x4) >> 2
}
func (o *OTP_Type) SetINTR_SBPI_WR_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetINTR_SBPI_WR_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTR.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetINTR_SBPI_FLAG_N(value uint32) {
	volatile.StoreUint32(&o.INTR.Reg, volatile.LoadUint32(&o.INTR.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetINTR_SBPI_FLAG_N() uint32 {
	return volatile.LoadUint32(&o.INTR.Reg) & 0x1
}

// OTP.INTE: Interrupt Enable
func (o *OTP_Type) SetINTE_APB_RD_NSEC_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x10)|value<<4)
}
func (o *OTP_Type) GetINTE_APB_RD_NSEC_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x10) >> 4
}
func (o *OTP_Type) SetINTE_APB_RD_SEC_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x8)|value<<3)
}
func (o *OTP_Type) GetINTE_APB_RD_SEC_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x8) >> 3
}
func (o *OTP_Type) SetINTE_APB_DCTRL_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x4)|value<<2)
}
func (o *OTP_Type) GetINTE_APB_DCTRL_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x4) >> 2
}
func (o *OTP_Type) SetINTE_SBPI_WR_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetINTE_SBPI_WR_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTE.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetINTE_SBPI_FLAG_N(value uint32) {
	volatile.StoreUint32(&o.INTE.Reg, volatile.LoadUint32(&o.INTE.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetINTE_SBPI_FLAG_N() uint32 {
	return volatile.LoadUint32(&o.INTE.Reg) & 0x1
}

// OTP.INTF: Interrupt Force
func (o *OTP_Type) SetINTF_APB_RD_NSEC_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x10)|value<<4)
}
func (o *OTP_Type) GetINTF_APB_RD_NSEC_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x10) >> 4
}
func (o *OTP_Type) SetINTF_APB_RD_SEC_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x8)|value<<3)
}
func (o *OTP_Type) GetINTF_APB_RD_SEC_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x8) >> 3
}
func (o *OTP_Type) SetINTF_APB_DCTRL_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x4)|value<<2)
}
func (o *OTP_Type) GetINTF_APB_DCTRL_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x4) >> 2
}
func (o *OTP_Type) SetINTF_SBPI_WR_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetINTF_SBPI_WR_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTF.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetINTF_SBPI_FLAG_N(value uint32) {
	volatile.StoreUint32(&o.INTF.Reg, volatile.LoadUint32(&o.INTF.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetINTF_SBPI_FLAG_N() uint32 {
	return volatile.LoadUint32(&o.INTF.Reg) & 0x1
}

// OTP.INTS: Interrupt status after masking & forcing
func (o *OTP_Type) SetINTS_APB_RD_NSEC_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x10)|value<<4)
}
func (o *OTP_Type) GetINTS_APB_RD_NSEC_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x10) >> 4
}
func (o *OTP_Type) SetINTS_APB_RD_SEC_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x8)|value<<3)
}
func (o *OTP_Type) GetINTS_APB_RD_SEC_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x8) >> 3
}
func (o *OTP_Type) SetINTS_APB_DCTRL_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x4)|value<<2)
}
func (o *OTP_Type) GetINTS_APB_DCTRL_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x4) >> 2
}
func (o *OTP_Type) SetINTS_SBPI_WR_FAIL(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x2)|value<<1)
}
func (o *OTP_Type) GetINTS_SBPI_WR_FAIL() uint32 {
	return (volatile.LoadUint32(&o.INTS.Reg) & 0x2) >> 1
}
func (o *OTP_Type) SetINTS_SBPI_FLAG_N(value uint32) {
	volatile.StoreUint32(&o.INTS.Reg, volatile.LoadUint32(&o.INTS.Reg)&^(0x1)|value)
}
func (o *OTP_Type) GetINTS_SBPI_FLAG_N() uint32 {
	return volatile.LoadUint32(&o.INTS.Reg) & 0x1
}

// Predefined OTP data layout for RP2350
type OTP_DATA_Type struct {
	CHIPID0                   volatile.Register16 // 0x0
	CHIPID1                   volatile.Register16 // 0x2
	CHIPID2                   volatile.Register16 // 0x4
	CHIPID3                   volatile.Register16 // 0x6
	RANDID0                   volatile.Register16 // 0x8
	RANDID1                   volatile.Register16 // 0xA
	RANDID2                   volatile.Register16 // 0xC
	RANDID3                   volatile.Register16 // 0xE
	RANDID4                   volatile.Register16 // 0x10
	RANDID5                   volatile.Register16 // 0x12
	RANDID6                   volatile.Register16 // 0x14
	RANDID7                   volatile.Register16 // 0x16
	_                         [8]byte
	ROSC_CALIB                volatile.Register16 // 0x20
	LPOSC_CALIB               volatile.Register16 // 0x22
	_                         [12]byte
	NUM_GPIOS                 volatile.Register16 // 0x30
	_                         [58]byte
	INFO_CRC0                 volatile.Register16 // 0x6C
	INFO_CRC1                 volatile.Register16 // 0x6E
	_                         [56]byte
	FLASH_DEVINFO             volatile.Register16 // 0xA8
	FLASH_PARTITION_SLOT_SIZE volatile.Register16 // 0xAA
	BOOTSEL_LED_CFG           volatile.Register16 // 0xAC
	BOOTSEL_PLL_CFG           volatile.Register16 // 0xAE
	BOOTSEL_XOSC_CFG          volatile.Register16 // 0xB0
	_                         [6]byte
	USB_WHITE_LABEL_ADDR      volatile.Register16 // 0xB8
	_                         [2]byte
	OTPBOOT_SRC               volatile.Register16 // 0xBC
	OTPBOOT_LEN               volatile.Register16 // 0xBE
	OTPBOOT_DST0              volatile.Register16 // 0xC0
	OTPBOOT_DST1              volatile.Register16 // 0xC2
	_                         [60]byte
	BOOTKEY0_0                volatile.Register16 // 0x100
	BOOTKEY0_1                volatile.Register16 // 0x102
	BOOTKEY0_2                volatile.Register16 // 0x104
	BOOTKEY0_3                volatile.Register16 // 0x106
	BOOTKEY0_4                volatile.Register16 // 0x108
	BOOTKEY0_5                volatile.Register16 // 0x10A
	BOOTKEY0_6                volatile.Register16 // 0x10C
	BOOTKEY0_7                volatile.Register16 // 0x10E
	BOOTKEY0_8                volatile.Register16 // 0x110
	BOOTKEY0_9                volatile.Register16 // 0x112
	BOOTKEY0_10               volatile.Register16 // 0x114
	BOOTKEY0_11               volatile.Register16 // 0x116
	BOOTKEY0_12               volatile.Register16 // 0x118
	BOOTKEY0_13               volatile.Register16 // 0x11A
	BOOTKEY0_14               volatile.Register16 // 0x11C
	BOOTKEY0_15               volatile.Register16 // 0x11E
	BOOTKEY1_0                volatile.Register16 // 0x120
	BOOTKEY1_1                volatile.Register16 // 0x122
	BOOTKEY1_2                volatile.Register16 // 0x124
	BOOTKEY1_3                volatile.Register16 // 0x126
	BOOTKEY1_4                volatile.Register16 // 0x128
	BOOTKEY1_5                volatile.Register16 // 0x12A
	BOOTKEY1_6                volatile.Register16 // 0x12C
	BOOTKEY1_7                volatile.Register16 // 0x12E
	BOOTKEY1_8                volatile.Register16 // 0x130
	BOOTKEY1_9                volatile.Register16 // 0x132
	BOOTKEY1_10               volatile.Register16 // 0x134
	BOOTKEY1_11               volatile.Register16 // 0x136
	BOOTKEY1_12               volatile.Register16 // 0x138
	BOOTKEY1_13               volatile.Register16 // 0x13A
	BOOTKEY1_14               volatile.Register16 // 0x13C
	BOOTKEY1_15               volatile.Register16 // 0x13E
	BOOTKEY2_0                volatile.Register16 // 0x140
	BOOTKEY2_1                volatile.Register16 // 0x142
	BOOTKEY2_2                volatile.Register16 // 0x144
	BOOTKEY2_3                volatile.Register16 // 0x146
	BOOTKEY2_4                volatile.Register16 // 0x148
	BOOTKEY2_5                volatile.Register16 // 0x14A
	BOOTKEY2_6                volatile.Register16 // 0x14C
	BOOTKEY2_7                volatile.Register16 // 0x14E
	BOOTKEY2_8                volatile.Register16 // 0x150
	BOOTKEY2_9                volatile.Register16 // 0x152
	BOOTKEY2_10               volatile.Register16 // 0x154
	BOOTKEY2_11               volatile.Register16 // 0x156
	BOOTKEY2_12               volatile.Register16 // 0x158
	BOOTKEY2_13               volatile.Register16 // 0x15A
	BOOTKEY2_14               volatile.Register16 // 0x15C
	BOOTKEY2_15               volatile.Register16 // 0x15E
	BOOTKEY3_0                volatile.Register16 // 0x160
	BOOTKEY3_1                volatile.Register16 // 0x162
	BOOTKEY3_2                volatile.Register16 // 0x164
	BOOTKEY3_3                volatile.Register16 // 0x166
	BOOTKEY3_4                volatile.Register16 // 0x168
	BOOTKEY3_5                volatile.Register16 // 0x16A
	BOOTKEY3_6                volatile.Register16 // 0x16C
	BOOTKEY3_7                volatile.Register16 // 0x16E
	BOOTKEY3_8                volatile.Register16 // 0x170
	BOOTKEY3_9                volatile.Register16 // 0x172
	BOOTKEY3_10               volatile.Register16 // 0x174
	BOOTKEY3_11               volatile.Register16 // 0x176
	BOOTKEY3_12               volatile.Register16 // 0x178
	BOOTKEY3_13               volatile.Register16 // 0x17A
	BOOTKEY3_14               volatile.Register16 // 0x17C
	BOOTKEY3_15               volatile.Register16 // 0x17E
	_                         [7440]byte
	KEY1_0                    volatile.Register16 // 0x1E90
	KEY1_1                    volatile.Register16 // 0x1E92
	KEY1_2                    volatile.Register16 // 0x1E94
	KEY1_3                    volatile.Register16 // 0x1E96
	KEY1_4                    volatile.Register16 // 0x1E98
	KEY1_5                    volatile.Register16 // 0x1E9A
	KEY1_6                    volatile.Register16 // 0x1E9C
	KEY1_7                    volatile.Register16 // 0x1E9E
	KEY2_0                    volatile.Register16 // 0x1EA0
	KEY2_1                    volatile.Register16 // 0x1EA2
	KEY2_2                    volatile.Register16 // 0x1EA4
	KEY2_3                    volatile.Register16 // 0x1EA6
	KEY2_4                    volatile.Register16 // 0x1EA8
	KEY2_5                    volatile.Register16 // 0x1EAA
	KEY2_6                    volatile.Register16 // 0x1EAC
	KEY2_7                    volatile.Register16 // 0x1EAE
	KEY3_0                    volatile.Register16 // 0x1EB0
	KEY3_1                    volatile.Register16 // 0x1EB2
	KEY3_2                    volatile.Register16 // 0x1EB4
	KEY3_3                    volatile.Register16 // 0x1EB6
	KEY3_4                    volatile.Register16 // 0x1EB8
	KEY3_5                    volatile.Register16 // 0x1EBA
	KEY3_6                    volatile.Register16 // 0x1EBC
	KEY3_7                    volatile.Register16 // 0x1EBE
	KEY4_0                    volatile.Register16 // 0x1EC0
	KEY4_1                    volatile.Register16 // 0x1EC2
	KEY4_2                    volatile.Register16 // 0x1EC4
	KEY4_3                    volatile.Register16 // 0x1EC6
	KEY4_4                    volatile.Register16 // 0x1EC8
	KEY4_5                    volatile.Register16 // 0x1ECA
	KEY4_6                    volatile.Register16 // 0x1ECC
	KEY4_7                    volatile.Register16 // 0x1ECE
	KEY5_0                    volatile.Register16 // 0x1ED0
	KEY5_1                    volatile.Register16 // 0x1ED2
	KEY5_2                    volatile.Register16 // 0x1ED4
	KEY5_3                    volatile.Register16 // 0x1ED6
	KEY5_4                    volatile.Register16 // 0x1ED8
	KEY5_5                    volatile.Register16 // 0x1EDA
	KEY5_6                    volatile.Register16 // 0x1EDC
	KEY5_7                    volatile.Register16 // 0x1EDE
	KEY6_0                    volatile.Register16 // 0x1EE0
	KEY6_1                    volatile.Register16 // 0x1EE2
	KEY6_2                    volatile.Register16 // 0x1EE4
	KEY6_3                    volatile.Register16 // 0x1EE6
	KEY6_4                    volatile.Register16 // 0x1EE8
	KEY6_5                    volatile.Register16 // 0x1EEA
	KEY6_6                    volatile.Register16 // 0x1EEC
	KEY6_7                    volatile.Register16 // 0x1EEE
}

// OTP_DATA.CHIPID0: Bits 15:0 of public device ID. (ECC) The CHIPID0..3 rows contain a 64-bit random identifier for this chip, which can be read from the USB bootloader PICOBOOT interface or from the get_sys_info ROM API. The number of random bits makes the occurrence of twins exceedingly unlikely: for example, a fleet of a hundred million devices has a 99.97% probability of no twinned IDs. This is estimated to be lower than the occurrence of process errors in the assignment of sequential random IDs, and for practical purposes CHIPID may be treated as unique.
func (o *OTP_DATA_Type) SetCHIPID0(value uint16) {
	volatile.StoreUint16(&o.CHIPID0.Reg, value)
}
func (o *OTP_DATA_Type) GetCHIPID0() uint16 {
	return volatile.LoadUint16(&o.CHIPID0.Reg)
}

// OTP_DATA.CHIPID1: Bits 31:16 of public device ID (ECC)
func (o *OTP_DATA_Type) SetCHIPID1(value uint16) {
	volatile.StoreUint16(&o.CHIPID1.Reg, value)
}
func (o *OTP_DATA_Type) GetCHIPID1() uint16 {
	return volatile.LoadUint16(&o.CHIPID1.Reg)
}

// OTP_DATA.CHIPID2: Bits 47:32 of public device ID (ECC)
func (o *OTP_DATA_Type) SetCHIPID2(value uint16) {
	volatile.StoreUint16(&o.CHIPID2.Reg, value)
}
func (o *OTP_DATA_Type) GetCHIPID2() uint16 {
	return volatile.LoadUint16(&o.CHIPID2.Reg)
}

// OTP_DATA.CHIPID3: Bits 63:48 of public device ID (ECC)
func (o *OTP_DATA_Type) SetCHIPID3(value uint16) {
	volatile.StoreUint16(&o.CHIPID3.Reg, value)
}
func (o *OTP_DATA_Type) GetCHIPID3() uint16 {
	return volatile.LoadUint16(&o.CHIPID3.Reg)
}

// OTP_DATA.RANDID0: Bits 15:0 of private per-device random number (ECC) The RANDID0..7 rows form a 128-bit random number generated during device test. This ID is not exposed through the USB PICOBOOT GET_INFO command or the ROM `get_sys_info()` API. However note that the USB PICOBOOT OTP access point can read the entirety of page 0, so this value is not meaningfully private unless the USB PICOBOOT interface is disabled via the DISABLE_BOOTSEL_USB_PICOBOOT_IFC flag in BOOT_FLAGS0.
func (o *OTP_DATA_Type) SetRANDID0(value uint16) {
	volatile.StoreUint16(&o.RANDID0.Reg, value)
}
func (o *OTP_DATA_Type) GetRANDID0() uint16 {
	return volatile.LoadUint16(&o.RANDID0.Reg)
}

// OTP_DATA.RANDID1: Bits 31:16 of private per-device random number (ECC)
func (o *OTP_DATA_Type) SetRANDID1(value uint16) {
	volatile.StoreUint16(&o.RANDID1.Reg, value)
}
func (o *OTP_DATA_Type) GetRANDID1() uint16 {
	return volatile.LoadUint16(&o.RANDID1.Reg)
}

// OTP_DATA.RANDID2: Bits 47:32 of private per-device random number (ECC)
func (o *OTP_DATA_Type) SetRANDID2(value uint16) {
	volatile.StoreUint16(&o.RANDID2.Reg, value)
}
func (o *OTP_DATA_Type) GetRANDID2() uint16 {
	return volatile.LoadUint16(&o.RANDID2.Reg)
}

// OTP_DATA.RANDID3: Bits 63:48 of private per-device random number (ECC)
func (o *OTP_DATA_Type) SetRANDID3(value uint16) {
	volatile.StoreUint16(&o.RANDID3.Reg, value)
}
func (o *OTP_DATA_Type) GetRANDID3() uint16 {
	return volatile.LoadUint16(&o.RANDID3.Reg)
}

// OTP_DATA.RANDID4: Bits 79:64 of private per-device random number (ECC)
func (o *OTP_DATA_Type) SetRANDID4(value uint16) {
	volatile.StoreUint16(&o.RANDID4.Reg, value)
}
func (o *OTP_DATA_Type) GetRANDID4() uint16 {
	return volatile.LoadUint16(&o.RANDID4.Reg)
}

// OTP_DATA.RANDID5: Bits 95:80 of private per-device random number (ECC)
func (o *OTP_DATA_Type) SetRANDID5(value uint16) {
	volatile.StoreUint16(&o.RANDID5.Reg, value)
}
func (o *OTP_DATA_Type) GetRANDID5() uint16 {
	return volatile.LoadUint16(&o.RANDID5.Reg)
}

// OTP_DATA.RANDID6: Bits 111:96 of private per-device random number (ECC)
func (o *OTP_DATA_Type) SetRANDID6(value uint16) {
	volatile.StoreUint16(&o.RANDID6.Reg, value)
}
func (o *OTP_DATA_Type) GetRANDID6() uint16 {
	return volatile.LoadUint16(&o.RANDID6.Reg)
}

// OTP_DATA.RANDID7: Bits 127:112 of private per-device random number (ECC)
func (o *OTP_DATA_Type) SetRANDID7(value uint16) {
	volatile.StoreUint16(&o.RANDID7.Reg, value)
}
func (o *OTP_DATA_Type) GetRANDID7() uint16 {
	return volatile.LoadUint16(&o.RANDID7.Reg)
}

// OTP_DATA.ROSC_CALIB: Ring oscillator frequency in kHz, measured during manufacturing (ECC) This is measured at 1.1 V, at room temperature, with the ROSC configuration registers in their reset state.
func (o *OTP_DATA_Type) SetROSC_CALIB(value uint16) {
	volatile.StoreUint16(&o.ROSC_CALIB.Reg, value)
}
func (o *OTP_DATA_Type) GetROSC_CALIB() uint16 {
	return volatile.LoadUint16(&o.ROSC_CALIB.Reg)
}

// OTP_DATA.LPOSC_CALIB: Low-power oscillator frequency in Hz, measured during manufacturing (ECC) This is measured at 1.1V, at room temperature, with the LPOSC trim register in its reset state.
func (o *OTP_DATA_Type) SetLPOSC_CALIB(value uint16) {
	volatile.StoreUint16(&o.LPOSC_CALIB.Reg, value)
}
func (o *OTP_DATA_Type) GetLPOSC_CALIB() uint16 {
	return volatile.LoadUint16(&o.LPOSC_CALIB.Reg)
}

// OTP_DATA.NUM_GPIOS: The number of main user GPIOs (bank 0). Should read 48 in the QFN80 package, and 30 in the QFN60 package. (ECC)
func (o *OTP_DATA_Type) SetNUM_GPIOS(value uint16) {
	volatile.StoreUint16(&o.NUM_GPIOS.Reg, volatile.LoadUint16(&o.NUM_GPIOS.Reg)&^(0xff)|value)
}
func (o *OTP_DATA_Type) GetNUM_GPIOS() uint16 {
	return volatile.LoadUint16(&o.NUM_GPIOS.Reg) & 0xff
}

// OTP_DATA.INFO_CRC0: Lower 16 bits of CRC32 of OTP addresses 0x00 through 0x6b (polynomial 0x4c11db7, input reflected, output reflected, seed all-ones, final XOR all-ones) (ECC)
func (o *OTP_DATA_Type) SetINFO_CRC0(value uint16) {
	volatile.StoreUint16(&o.INFO_CRC0.Reg, value)
}
func (o *OTP_DATA_Type) GetINFO_CRC0() uint16 {
	return volatile.LoadUint16(&o.INFO_CRC0.Reg)
}

// OTP_DATA.INFO_CRC1: Upper 16 bits of CRC32 of OTP addresses 0x00 through 0x6b (ECC)
func (o *OTP_DATA_Type) SetINFO_CRC1(value uint16) {
	volatile.StoreUint16(&o.INFO_CRC1.Reg, value)
}
func (o *OTP_DATA_Type) GetINFO_CRC1() uint16 {
	return volatile.LoadUint16(&o.INFO_CRC1.Reg)
}

// OTP_DATA.FLASH_DEVINFO: Stores information about external flash device(s). (ECC) Assumed to be valid if BOOT_FLAGS0_FLASH_DEVINFO_ENABLE is set.
func (o *OTP_DATA_Type) SetFLASH_DEVINFO_CS1_SIZE(value uint16) {
	volatile.StoreUint16(&o.FLASH_DEVINFO.Reg, volatile.LoadUint16(&o.FLASH_DEVINFO.Reg)&^(0xf000)|value<<12)
}
func (o *OTP_DATA_Type) GetFLASH_DEVINFO_CS1_SIZE() uint16 {
	return (volatile.LoadUint16(&o.FLASH_DEVINFO.Reg) & 0xf000) >> 12
}
func (o *OTP_DATA_Type) SetFLASH_DEVINFO_CS0_SIZE(value uint16) {
	volatile.StoreUint16(&o.FLASH_DEVINFO.Reg, volatile.LoadUint16(&o.FLASH_DEVINFO.Reg)&^(0xf00)|value<<8)
}
func (o *OTP_DATA_Type) GetFLASH_DEVINFO_CS0_SIZE() uint16 {
	return (volatile.LoadUint16(&o.FLASH_DEVINFO.Reg) & 0xf00) >> 8
}
func (o *OTP_DATA_Type) SetFLASH_DEVINFO_D8H_ERASE_SUPPORTED(value uint16) {
	volatile.StoreUint16(&o.FLASH_DEVINFO.Reg, volatile.LoadUint16(&o.FLASH_DEVINFO.Reg)&^(0x80)|value<<7)
}
func (o *OTP_DATA_Type) GetFLASH_DEVINFO_D8H_ERASE_SUPPORTED() uint16 {
	return (volatile.LoadUint16(&o.FLASH_DEVINFO.Reg) & 0x80) >> 7
}
func (o *OTP_DATA_Type) SetFLASH_DEVINFO_CS1_GPIO(value uint16) {
	volatile.StoreUint16(&o.FLASH_DEVINFO.Reg, volatile.LoadUint16(&o.FLASH_DEVINFO.Reg)&^(0x3f)|value)
}
func (o *OTP_DATA_Type) GetFLASH_DEVINFO_CS1_GPIO() uint16 {
	return volatile.LoadUint16(&o.FLASH_DEVINFO.Reg) & 0x3f
}

// OTP_DATA.FLASH_PARTITION_SLOT_SIZE: Gap between partition table slot 0 and slot 1 at the start of flash (the default size is 4096 bytes) (ECC) Enabled by the OVERRIDE_FLASH_PARTITION_SLOT_SIZE bit in BOOT_FLAGS, the size is 4096 * (value + 1)
func (o *OTP_DATA_Type) SetFLASH_PARTITION_SLOT_SIZE(value uint16) {
	volatile.StoreUint16(&o.FLASH_PARTITION_SLOT_SIZE.Reg, value)
}
func (o *OTP_DATA_Type) GetFLASH_PARTITION_SLOT_SIZE() uint16 {
	return volatile.LoadUint16(&o.FLASH_PARTITION_SLOT_SIZE.Reg)
}

// OTP_DATA.BOOTSEL_LED_CFG: Pin configuration for LED status, used by USB bootloader. (ECC) Must be valid if BOOT_FLAGS0_ENABLE_BOOTSEL_LED is set.
func (o *OTP_DATA_Type) SetBOOTSEL_LED_CFG_ACTIVELOW(value uint16) {
	volatile.StoreUint16(&o.BOOTSEL_LED_CFG.Reg, volatile.LoadUint16(&o.BOOTSEL_LED_CFG.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_Type) GetBOOTSEL_LED_CFG_ACTIVELOW() uint16 {
	return (volatile.LoadUint16(&o.BOOTSEL_LED_CFG.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_Type) SetBOOTSEL_LED_CFG_PIN(value uint16) {
	volatile.StoreUint16(&o.BOOTSEL_LED_CFG.Reg, volatile.LoadUint16(&o.BOOTSEL_LED_CFG.Reg)&^(0x3f)|value)
}
func (o *OTP_DATA_Type) GetBOOTSEL_LED_CFG_PIN() uint16 {
	return volatile.LoadUint16(&o.BOOTSEL_LED_CFG.Reg) & 0x3f
}

// OTP_DATA.BOOTSEL_PLL_CFG: Optional PLL configuration for BOOTSEL mode. (ECC) This should be configured to produce an exact 48 MHz based on the crystal oscillator frequency. User mode software may also use this value to calculate the expected crystal frequency based on an assumed 48 MHz PLL output. If no configuration is given, the crystal is assumed to be 12 MHz. The PLL frequency can be calculated as: PLL out = (XOSC frequency / (REFDIV+1)) x FBDIV / (POSTDIV1 x POSTDIV2) Conversely the crystal frequency can be calculated as: XOSC frequency = 48 MHz x (REFDIV+1) x (POSTDIV1 x POSTDIV2) / FBDIV (Note the +1 on REFDIV is because the value stored in this OTP location is the actual divisor value minus one.) Used if and only if ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG is set in BOOT_FLAGS0. That bit should be set only after this row and BOOTSEL_XOSC_CFG are both correctly programmed.
func (o *OTP_DATA_Type) SetBOOTSEL_PLL_CFG_REFDIV(value uint16) {
	volatile.StoreUint16(&o.BOOTSEL_PLL_CFG.Reg, volatile.LoadUint16(&o.BOOTSEL_PLL_CFG.Reg)&^(0x8000)|value<<15)
}
func (o *OTP_DATA_Type) GetBOOTSEL_PLL_CFG_REFDIV() uint16 {
	return (volatile.LoadUint16(&o.BOOTSEL_PLL_CFG.Reg) & 0x8000) >> 15
}
func (o *OTP_DATA_Type) SetBOOTSEL_PLL_CFG_POSTDIV2(value uint16) {
	volatile.StoreUint16(&o.BOOTSEL_PLL_CFG.Reg, volatile.LoadUint16(&o.BOOTSEL_PLL_CFG.Reg)&^(0x7000)|value<<12)
}
func (o *OTP_DATA_Type) GetBOOTSEL_PLL_CFG_POSTDIV2() uint16 {
	return (volatile.LoadUint16(&o.BOOTSEL_PLL_CFG.Reg) & 0x7000) >> 12
}
func (o *OTP_DATA_Type) SetBOOTSEL_PLL_CFG_POSTDIV1(value uint16) {
	volatile.StoreUint16(&o.BOOTSEL_PLL_CFG.Reg, volatile.LoadUint16(&o.BOOTSEL_PLL_CFG.Reg)&^(0xe00)|value<<9)
}
func (o *OTP_DATA_Type) GetBOOTSEL_PLL_CFG_POSTDIV1() uint16 {
	return (volatile.LoadUint16(&o.BOOTSEL_PLL_CFG.Reg) & 0xe00) >> 9
}
func (o *OTP_DATA_Type) SetBOOTSEL_PLL_CFG_FBDIV(value uint16) {
	volatile.StoreUint16(&o.BOOTSEL_PLL_CFG.Reg, volatile.LoadUint16(&o.BOOTSEL_PLL_CFG.Reg)&^(0x1ff)|value)
}
func (o *OTP_DATA_Type) GetBOOTSEL_PLL_CFG_FBDIV() uint16 {
	return volatile.LoadUint16(&o.BOOTSEL_PLL_CFG.Reg) & 0x1ff
}

// OTP_DATA.BOOTSEL_XOSC_CFG: Non-default crystal oscillator configuration for the USB bootloader. (ECC) These values may also be used by user code configuring the crystal oscillator. Used if and only if ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG is set in BOOT_FLAGS0. That bit should be set only after this row and BOOTSEL_PLL_CFG are both correctly programmed.
func (o *OTP_DATA_Type) SetBOOTSEL_XOSC_CFG_RANGE(value uint16) {
	volatile.StoreUint16(&o.BOOTSEL_XOSC_CFG.Reg, volatile.LoadUint16(&o.BOOTSEL_XOSC_CFG.Reg)&^(0xc000)|value<<14)
}
func (o *OTP_DATA_Type) GetBOOTSEL_XOSC_CFG_RANGE() uint16 {
	return (volatile.LoadUint16(&o.BOOTSEL_XOSC_CFG.Reg) & 0xc000) >> 14
}
func (o *OTP_DATA_Type) SetBOOTSEL_XOSC_CFG_STARTUP(value uint16) {
	volatile.StoreUint16(&o.BOOTSEL_XOSC_CFG.Reg, volatile.LoadUint16(&o.BOOTSEL_XOSC_CFG.Reg)&^(0x3fff)|value)
}
func (o *OTP_DATA_Type) GetBOOTSEL_XOSC_CFG_STARTUP() uint16 {
	return volatile.LoadUint16(&o.BOOTSEL_XOSC_CFG.Reg) & 0x3fff
}

// OTP_DATA.USB_WHITE_LABEL_ADDR: Row index of the USB_WHITE_LABEL structure within OTP (ECC) The table has 16 rows, each of which are also ECC and marked valid by the corresponding valid bit in USB_BOOT_FLAGS (ECC). The entries are either _VALUEs where the 16 bit value is used as is, or _STRDEFs which acts as a pointers to a string value. The value stored in a _STRDEF is two separate bytes: The low seven bits of the first (LSB) byte indicates the number of characters in the string, and the top bit of the first (LSB) byte if set to indicate that each character in the string is two bytes (Unicode) versus one byte if unset. The second (MSB) byte represents the location of the string data, and is encoded as the number of rows from this USB_WHITE_LABEL_ADDR; i.e. the row of the start of the string is USB_WHITE_LABEL_ADDR value + msb_byte. In each case, the corresponding valid bit enables replacing the default value for the corresponding item provided by the boot rom. Note that Unicode _STRDEFs are only supported for USB_DEVICE_PRODUCT_STRDEF, USB_DEVICE_SERIAL_NUMBER_STRDEF and USB_DEVICE_MANUFACTURER_STRDEF. Unicode values will be ignored if specified for other fields, and non-unicode values for these three items will be converted to Unicode characters by setting the upper 8 bits to zero. Note that if the USB_WHITE_LABEL structure or the corresponding strings are not readable by BOOTSEL mode based on OTP permissions, or if alignment requirements are not met, then the corresponding default values are used. The index values indicate where each field is located (row USB_WHITE_LABEL_ADDR value + index):
func (o *OTP_DATA_Type) SetUSB_WHITE_LABEL_ADDR(value uint16) {
	volatile.StoreUint16(&o.USB_WHITE_LABEL_ADDR.Reg, value)
}
func (o *OTP_DATA_Type) GetUSB_WHITE_LABEL_ADDR() uint16 {
	return volatile.LoadUint16(&o.USB_WHITE_LABEL_ADDR.Reg)
}

// OTP_DATA.OTPBOOT_SRC: OTP start row for the OTP boot image. (ECC) If OTP boot is enabled, the bootrom will load from this location into SRAM and then directly enter the loaded image. Note that the image must be signed if SECURE_BOOT_ENABLE is set. The image itself is assumed to be ECC-protected. This must be an even number. Equivalently, the OTP boot image must start at a word-aligned location in the ECC read data address window.
func (o *OTP_DATA_Type) SetOTPBOOT_SRC(value uint16) {
	volatile.StoreUint16(&o.OTPBOOT_SRC.Reg, value)
}
func (o *OTP_DATA_Type) GetOTPBOOT_SRC() uint16 {
	return volatile.LoadUint16(&o.OTPBOOT_SRC.Reg)
}

// OTP_DATA.OTPBOOT_LEN: Length in rows of the OTP boot image. (ECC) OTPBOOT_LEN must be even. The total image size must be a multiple of 4 bytes (32 bits).
func (o *OTP_DATA_Type) SetOTPBOOT_LEN(value uint16) {
	volatile.StoreUint16(&o.OTPBOOT_LEN.Reg, value)
}
func (o *OTP_DATA_Type) GetOTPBOOT_LEN() uint16 {
	return volatile.LoadUint16(&o.OTPBOOT_LEN.Reg)
}

// OTP_DATA.OTPBOOT_DST0: Bits 15:0 of the OTP boot image load destination (and entry point). (ECC) This must be a location in main SRAM (main SRAM is addresses 0x20000000 through 0x20082000) and must be word-aligned.
func (o *OTP_DATA_Type) SetOTPBOOT_DST0(value uint16) {
	volatile.StoreUint16(&o.OTPBOOT_DST0.Reg, value)
}
func (o *OTP_DATA_Type) GetOTPBOOT_DST0() uint16 {
	return volatile.LoadUint16(&o.OTPBOOT_DST0.Reg)
}

// OTP_DATA.OTPBOOT_DST1: Bits 31:16 of the OTP boot image load destination (and entry point). (ECC) This must be a location in main SRAM (main SRAM is addresses 0x20000000 through 0x20082000) and must be word-aligned.
func (o *OTP_DATA_Type) SetOTPBOOT_DST1(value uint16) {
	volatile.StoreUint16(&o.OTPBOOT_DST1.Reg, value)
}
func (o *OTP_DATA_Type) GetOTPBOOT_DST1() uint16 {
	return volatile.LoadUint16(&o.OTPBOOT_DST1.Reg)
}

// OTP_DATA.BOOTKEY0_0: Bits 15:0 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_0(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_0.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_0() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_0.Reg)
}

// OTP_DATA.BOOTKEY0_1: Bits 31:16 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_1(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_1.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_1() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_1.Reg)
}

// OTP_DATA.BOOTKEY0_2: Bits 47:32 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_2(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_2.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_2() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_2.Reg)
}

// OTP_DATA.BOOTKEY0_3: Bits 63:48 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_3(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_3.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_3() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_3.Reg)
}

// OTP_DATA.BOOTKEY0_4: Bits 79:64 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_4(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_4.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_4() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_4.Reg)
}

// OTP_DATA.BOOTKEY0_5: Bits 95:80 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_5(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_5.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_5() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_5.Reg)
}

// OTP_DATA.BOOTKEY0_6: Bits 111:96 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_6(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_6.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_6() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_6.Reg)
}

// OTP_DATA.BOOTKEY0_7: Bits 127:112 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_7(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_7.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_7() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_7.Reg)
}

// OTP_DATA.BOOTKEY0_8: Bits 143:128 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_8(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_8.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_8() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_8.Reg)
}

// OTP_DATA.BOOTKEY0_9: Bits 159:144 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_9(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_9.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_9() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_9.Reg)
}

// OTP_DATA.BOOTKEY0_10: Bits 175:160 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_10(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_10.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_10() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_10.Reg)
}

// OTP_DATA.BOOTKEY0_11: Bits 191:176 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_11(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_11.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_11() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_11.Reg)
}

// OTP_DATA.BOOTKEY0_12: Bits 207:192 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_12(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_12.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_12() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_12.Reg)
}

// OTP_DATA.BOOTKEY0_13: Bits 223:208 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_13(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_13.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_13() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_13.Reg)
}

// OTP_DATA.BOOTKEY0_14: Bits 239:224 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_14(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_14.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_14() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_14.Reg)
}

// OTP_DATA.BOOTKEY0_15: Bits 255:240 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY0_15(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY0_15.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY0_15() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY0_15.Reg)
}

// OTP_DATA.BOOTKEY1_0: Bits 15:0 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_0(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_0.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_0() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_0.Reg)
}

// OTP_DATA.BOOTKEY1_1: Bits 31:16 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_1(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_1.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_1() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_1.Reg)
}

// OTP_DATA.BOOTKEY1_2: Bits 47:32 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_2(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_2.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_2() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_2.Reg)
}

// OTP_DATA.BOOTKEY1_3: Bits 63:48 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_3(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_3.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_3() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_3.Reg)
}

// OTP_DATA.BOOTKEY1_4: Bits 79:64 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_4(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_4.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_4() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_4.Reg)
}

// OTP_DATA.BOOTKEY1_5: Bits 95:80 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_5(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_5.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_5() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_5.Reg)
}

// OTP_DATA.BOOTKEY1_6: Bits 111:96 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_6(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_6.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_6() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_6.Reg)
}

// OTP_DATA.BOOTKEY1_7: Bits 127:112 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_7(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_7.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_7() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_7.Reg)
}

// OTP_DATA.BOOTKEY1_8: Bits 143:128 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_8(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_8.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_8() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_8.Reg)
}

// OTP_DATA.BOOTKEY1_9: Bits 159:144 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_9(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_9.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_9() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_9.Reg)
}

// OTP_DATA.BOOTKEY1_10: Bits 175:160 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_10(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_10.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_10() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_10.Reg)
}

// OTP_DATA.BOOTKEY1_11: Bits 191:176 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_11(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_11.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_11() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_11.Reg)
}

// OTP_DATA.BOOTKEY1_12: Bits 207:192 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_12(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_12.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_12() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_12.Reg)
}

// OTP_DATA.BOOTKEY1_13: Bits 223:208 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_13(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_13.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_13() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_13.Reg)
}

// OTP_DATA.BOOTKEY1_14: Bits 239:224 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_14(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_14.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_14() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_14.Reg)
}

// OTP_DATA.BOOTKEY1_15: Bits 255:240 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY1_15(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY1_15.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY1_15() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY1_15.Reg)
}

// OTP_DATA.BOOTKEY2_0: Bits 15:0 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_0(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_0.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_0() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_0.Reg)
}

// OTP_DATA.BOOTKEY2_1: Bits 31:16 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_1(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_1.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_1() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_1.Reg)
}

// OTP_DATA.BOOTKEY2_2: Bits 47:32 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_2(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_2.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_2() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_2.Reg)
}

// OTP_DATA.BOOTKEY2_3: Bits 63:48 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_3(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_3.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_3() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_3.Reg)
}

// OTP_DATA.BOOTKEY2_4: Bits 79:64 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_4(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_4.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_4() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_4.Reg)
}

// OTP_DATA.BOOTKEY2_5: Bits 95:80 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_5(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_5.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_5() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_5.Reg)
}

// OTP_DATA.BOOTKEY2_6: Bits 111:96 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_6(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_6.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_6() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_6.Reg)
}

// OTP_DATA.BOOTKEY2_7: Bits 127:112 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_7(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_7.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_7() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_7.Reg)
}

// OTP_DATA.BOOTKEY2_8: Bits 143:128 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_8(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_8.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_8() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_8.Reg)
}

// OTP_DATA.BOOTKEY2_9: Bits 159:144 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_9(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_9.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_9() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_9.Reg)
}

// OTP_DATA.BOOTKEY2_10: Bits 175:160 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_10(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_10.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_10() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_10.Reg)
}

// OTP_DATA.BOOTKEY2_11: Bits 191:176 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_11(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_11.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_11() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_11.Reg)
}

// OTP_DATA.BOOTKEY2_12: Bits 207:192 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_12(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_12.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_12() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_12.Reg)
}

// OTP_DATA.BOOTKEY2_13: Bits 223:208 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_13(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_13.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_13() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_13.Reg)
}

// OTP_DATA.BOOTKEY2_14: Bits 239:224 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_14(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_14.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_14() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_14.Reg)
}

// OTP_DATA.BOOTKEY2_15: Bits 255:240 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY2_15(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY2_15.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY2_15() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY2_15.Reg)
}

// OTP_DATA.BOOTKEY3_0: Bits 15:0 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_0(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_0.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_0() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_0.Reg)
}

// OTP_DATA.BOOTKEY3_1: Bits 31:16 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_1(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_1.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_1() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_1.Reg)
}

// OTP_DATA.BOOTKEY3_2: Bits 47:32 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_2(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_2.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_2() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_2.Reg)
}

// OTP_DATA.BOOTKEY3_3: Bits 63:48 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_3(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_3.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_3() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_3.Reg)
}

// OTP_DATA.BOOTKEY3_4: Bits 79:64 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_4(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_4.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_4() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_4.Reg)
}

// OTP_DATA.BOOTKEY3_5: Bits 95:80 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_5(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_5.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_5() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_5.Reg)
}

// OTP_DATA.BOOTKEY3_6: Bits 111:96 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_6(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_6.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_6() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_6.Reg)
}

// OTP_DATA.BOOTKEY3_7: Bits 127:112 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_7(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_7.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_7() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_7.Reg)
}

// OTP_DATA.BOOTKEY3_8: Bits 143:128 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_8(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_8.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_8() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_8.Reg)
}

// OTP_DATA.BOOTKEY3_9: Bits 159:144 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_9(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_9.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_9() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_9.Reg)
}

// OTP_DATA.BOOTKEY3_10: Bits 175:160 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_10(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_10.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_10() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_10.Reg)
}

// OTP_DATA.BOOTKEY3_11: Bits 191:176 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_11(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_11.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_11() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_11.Reg)
}

// OTP_DATA.BOOTKEY3_12: Bits 207:192 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_12(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_12.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_12() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_12.Reg)
}

// OTP_DATA.BOOTKEY3_13: Bits 223:208 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_13(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_13.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_13() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_13.Reg)
}

// OTP_DATA.BOOTKEY3_14: Bits 239:224 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_14(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_14.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_14() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_14.Reg)
}

// OTP_DATA.BOOTKEY3_15: Bits 255:240 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_Type) SetBOOTKEY3_15(value uint16) {
	volatile.StoreUint16(&o.BOOTKEY3_15.Reg, value)
}
func (o *OTP_DATA_Type) GetBOOTKEY3_15() uint16 {
	return volatile.LoadUint16(&o.BOOTKEY3_15.Reg)
}

// OTP_DATA.KEY1_0: Bits 15:0 of OTP access key 1 (ECC)
func (o *OTP_DATA_Type) SetKEY1_0(value uint16) {
	volatile.StoreUint16(&o.KEY1_0.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY1_0() uint16 {
	return volatile.LoadUint16(&o.KEY1_0.Reg)
}

// OTP_DATA.KEY1_1: Bits 31:16 of OTP access key 1 (ECC)
func (o *OTP_DATA_Type) SetKEY1_1(value uint16) {
	volatile.StoreUint16(&o.KEY1_1.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY1_1() uint16 {
	return volatile.LoadUint16(&o.KEY1_1.Reg)
}

// OTP_DATA.KEY1_2: Bits 47:32 of OTP access key 1 (ECC)
func (o *OTP_DATA_Type) SetKEY1_2(value uint16) {
	volatile.StoreUint16(&o.KEY1_2.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY1_2() uint16 {
	return volatile.LoadUint16(&o.KEY1_2.Reg)
}

// OTP_DATA.KEY1_3: Bits 63:48 of OTP access key 1 (ECC)
func (o *OTP_DATA_Type) SetKEY1_3(value uint16) {
	volatile.StoreUint16(&o.KEY1_3.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY1_3() uint16 {
	return volatile.LoadUint16(&o.KEY1_3.Reg)
}

// OTP_DATA.KEY1_4: Bits 79:64 of OTP access key 1 (ECC)
func (o *OTP_DATA_Type) SetKEY1_4(value uint16) {
	volatile.StoreUint16(&o.KEY1_4.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY1_4() uint16 {
	return volatile.LoadUint16(&o.KEY1_4.Reg)
}

// OTP_DATA.KEY1_5: Bits 95:80 of OTP access key 1 (ECC)
func (o *OTP_DATA_Type) SetKEY1_5(value uint16) {
	volatile.StoreUint16(&o.KEY1_5.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY1_5() uint16 {
	return volatile.LoadUint16(&o.KEY1_5.Reg)
}

// OTP_DATA.KEY1_6: Bits 111:96 of OTP access key 1 (ECC)
func (o *OTP_DATA_Type) SetKEY1_6(value uint16) {
	volatile.StoreUint16(&o.KEY1_6.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY1_6() uint16 {
	return volatile.LoadUint16(&o.KEY1_6.Reg)
}

// OTP_DATA.KEY1_7: Bits 127:112 of OTP access key 1 (ECC)
func (o *OTP_DATA_Type) SetKEY1_7(value uint16) {
	volatile.StoreUint16(&o.KEY1_7.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY1_7() uint16 {
	return volatile.LoadUint16(&o.KEY1_7.Reg)
}

// OTP_DATA.KEY2_0: Bits 15:0 of OTP access key 2 (ECC)
func (o *OTP_DATA_Type) SetKEY2_0(value uint16) {
	volatile.StoreUint16(&o.KEY2_0.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY2_0() uint16 {
	return volatile.LoadUint16(&o.KEY2_0.Reg)
}

// OTP_DATA.KEY2_1: Bits 31:16 of OTP access key 2 (ECC)
func (o *OTP_DATA_Type) SetKEY2_1(value uint16) {
	volatile.StoreUint16(&o.KEY2_1.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY2_1() uint16 {
	return volatile.LoadUint16(&o.KEY2_1.Reg)
}

// OTP_DATA.KEY2_2: Bits 47:32 of OTP access key 2 (ECC)
func (o *OTP_DATA_Type) SetKEY2_2(value uint16) {
	volatile.StoreUint16(&o.KEY2_2.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY2_2() uint16 {
	return volatile.LoadUint16(&o.KEY2_2.Reg)
}

// OTP_DATA.KEY2_3: Bits 63:48 of OTP access key 2 (ECC)
func (o *OTP_DATA_Type) SetKEY2_3(value uint16) {
	volatile.StoreUint16(&o.KEY2_3.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY2_3() uint16 {
	return volatile.LoadUint16(&o.KEY2_3.Reg)
}

// OTP_DATA.KEY2_4: Bits 79:64 of OTP access key 2 (ECC)
func (o *OTP_DATA_Type) SetKEY2_4(value uint16) {
	volatile.StoreUint16(&o.KEY2_4.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY2_4() uint16 {
	return volatile.LoadUint16(&o.KEY2_4.Reg)
}

// OTP_DATA.KEY2_5: Bits 95:80 of OTP access key 2 (ECC)
func (o *OTP_DATA_Type) SetKEY2_5(value uint16) {
	volatile.StoreUint16(&o.KEY2_5.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY2_5() uint16 {
	return volatile.LoadUint16(&o.KEY2_5.Reg)
}

// OTP_DATA.KEY2_6: Bits 111:96 of OTP access key 2 (ECC)
func (o *OTP_DATA_Type) SetKEY2_6(value uint16) {
	volatile.StoreUint16(&o.KEY2_6.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY2_6() uint16 {
	return volatile.LoadUint16(&o.KEY2_6.Reg)
}

// OTP_DATA.KEY2_7: Bits 127:112 of OTP access key 2 (ECC)
func (o *OTP_DATA_Type) SetKEY2_7(value uint16) {
	volatile.StoreUint16(&o.KEY2_7.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY2_7() uint16 {
	return volatile.LoadUint16(&o.KEY2_7.Reg)
}

// OTP_DATA.KEY3_0: Bits 15:0 of OTP access key 3 (ECC)
func (o *OTP_DATA_Type) SetKEY3_0(value uint16) {
	volatile.StoreUint16(&o.KEY3_0.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY3_0() uint16 {
	return volatile.LoadUint16(&o.KEY3_0.Reg)
}

// OTP_DATA.KEY3_1: Bits 31:16 of OTP access key 3 (ECC)
func (o *OTP_DATA_Type) SetKEY3_1(value uint16) {
	volatile.StoreUint16(&o.KEY3_1.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY3_1() uint16 {
	return volatile.LoadUint16(&o.KEY3_1.Reg)
}

// OTP_DATA.KEY3_2: Bits 47:32 of OTP access key 3 (ECC)
func (o *OTP_DATA_Type) SetKEY3_2(value uint16) {
	volatile.StoreUint16(&o.KEY3_2.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY3_2() uint16 {
	return volatile.LoadUint16(&o.KEY3_2.Reg)
}

// OTP_DATA.KEY3_3: Bits 63:48 of OTP access key 3 (ECC)
func (o *OTP_DATA_Type) SetKEY3_3(value uint16) {
	volatile.StoreUint16(&o.KEY3_3.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY3_3() uint16 {
	return volatile.LoadUint16(&o.KEY3_3.Reg)
}

// OTP_DATA.KEY3_4: Bits 79:64 of OTP access key 3 (ECC)
func (o *OTP_DATA_Type) SetKEY3_4(value uint16) {
	volatile.StoreUint16(&o.KEY3_4.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY3_4() uint16 {
	return volatile.LoadUint16(&o.KEY3_4.Reg)
}

// OTP_DATA.KEY3_5: Bits 95:80 of OTP access key 3 (ECC)
func (o *OTP_DATA_Type) SetKEY3_5(value uint16) {
	volatile.StoreUint16(&o.KEY3_5.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY3_5() uint16 {
	return volatile.LoadUint16(&o.KEY3_5.Reg)
}

// OTP_DATA.KEY3_6: Bits 111:96 of OTP access key 3 (ECC)
func (o *OTP_DATA_Type) SetKEY3_6(value uint16) {
	volatile.StoreUint16(&o.KEY3_6.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY3_6() uint16 {
	return volatile.LoadUint16(&o.KEY3_6.Reg)
}

// OTP_DATA.KEY3_7: Bits 127:112 of OTP access key 3 (ECC)
func (o *OTP_DATA_Type) SetKEY3_7(value uint16) {
	volatile.StoreUint16(&o.KEY3_7.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY3_7() uint16 {
	return volatile.LoadUint16(&o.KEY3_7.Reg)
}

// OTP_DATA.KEY4_0: Bits 15:0 of OTP access key 4 (ECC)
func (o *OTP_DATA_Type) SetKEY4_0(value uint16) {
	volatile.StoreUint16(&o.KEY4_0.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY4_0() uint16 {
	return volatile.LoadUint16(&o.KEY4_0.Reg)
}

// OTP_DATA.KEY4_1: Bits 31:16 of OTP access key 4 (ECC)
func (o *OTP_DATA_Type) SetKEY4_1(value uint16) {
	volatile.StoreUint16(&o.KEY4_1.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY4_1() uint16 {
	return volatile.LoadUint16(&o.KEY4_1.Reg)
}

// OTP_DATA.KEY4_2: Bits 47:32 of OTP access key 4 (ECC)
func (o *OTP_DATA_Type) SetKEY4_2(value uint16) {
	volatile.StoreUint16(&o.KEY4_2.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY4_2() uint16 {
	return volatile.LoadUint16(&o.KEY4_2.Reg)
}

// OTP_DATA.KEY4_3: Bits 63:48 of OTP access key 4 (ECC)
func (o *OTP_DATA_Type) SetKEY4_3(value uint16) {
	volatile.StoreUint16(&o.KEY4_3.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY4_3() uint16 {
	return volatile.LoadUint16(&o.KEY4_3.Reg)
}

// OTP_DATA.KEY4_4: Bits 79:64 of OTP access key 4 (ECC)
func (o *OTP_DATA_Type) SetKEY4_4(value uint16) {
	volatile.StoreUint16(&o.KEY4_4.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY4_4() uint16 {
	return volatile.LoadUint16(&o.KEY4_4.Reg)
}

// OTP_DATA.KEY4_5: Bits 95:80 of OTP access key 4 (ECC)
func (o *OTP_DATA_Type) SetKEY4_5(value uint16) {
	volatile.StoreUint16(&o.KEY4_5.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY4_5() uint16 {
	return volatile.LoadUint16(&o.KEY4_5.Reg)
}

// OTP_DATA.KEY4_6: Bits 111:96 of OTP access key 4 (ECC)
func (o *OTP_DATA_Type) SetKEY4_6(value uint16) {
	volatile.StoreUint16(&o.KEY4_6.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY4_6() uint16 {
	return volatile.LoadUint16(&o.KEY4_6.Reg)
}

// OTP_DATA.KEY4_7: Bits 127:112 of OTP access key 4 (ECC)
func (o *OTP_DATA_Type) SetKEY4_7(value uint16) {
	volatile.StoreUint16(&o.KEY4_7.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY4_7() uint16 {
	return volatile.LoadUint16(&o.KEY4_7.Reg)
}

// OTP_DATA.KEY5_0: Bits 15:0 of OTP access key 5 (ECC)
func (o *OTP_DATA_Type) SetKEY5_0(value uint16) {
	volatile.StoreUint16(&o.KEY5_0.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY5_0() uint16 {
	return volatile.LoadUint16(&o.KEY5_0.Reg)
}

// OTP_DATA.KEY5_1: Bits 31:16 of OTP access key 5 (ECC)
func (o *OTP_DATA_Type) SetKEY5_1(value uint16) {
	volatile.StoreUint16(&o.KEY5_1.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY5_1() uint16 {
	return volatile.LoadUint16(&o.KEY5_1.Reg)
}

// OTP_DATA.KEY5_2: Bits 47:32 of OTP access key 5 (ECC)
func (o *OTP_DATA_Type) SetKEY5_2(value uint16) {
	volatile.StoreUint16(&o.KEY5_2.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY5_2() uint16 {
	return volatile.LoadUint16(&o.KEY5_2.Reg)
}

// OTP_DATA.KEY5_3: Bits 63:48 of OTP access key 5 (ECC)
func (o *OTP_DATA_Type) SetKEY5_3(value uint16) {
	volatile.StoreUint16(&o.KEY5_3.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY5_3() uint16 {
	return volatile.LoadUint16(&o.KEY5_3.Reg)
}

// OTP_DATA.KEY5_4: Bits 79:64 of OTP access key 5 (ECC)
func (o *OTP_DATA_Type) SetKEY5_4(value uint16) {
	volatile.StoreUint16(&o.KEY5_4.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY5_4() uint16 {
	return volatile.LoadUint16(&o.KEY5_4.Reg)
}

// OTP_DATA.KEY5_5: Bits 95:80 of OTP access key 5 (ECC)
func (o *OTP_DATA_Type) SetKEY5_5(value uint16) {
	volatile.StoreUint16(&o.KEY5_5.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY5_5() uint16 {
	return volatile.LoadUint16(&o.KEY5_5.Reg)
}

// OTP_DATA.KEY5_6: Bits 111:96 of OTP access key 5 (ECC)
func (o *OTP_DATA_Type) SetKEY5_6(value uint16) {
	volatile.StoreUint16(&o.KEY5_6.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY5_6() uint16 {
	return volatile.LoadUint16(&o.KEY5_6.Reg)
}

// OTP_DATA.KEY5_7: Bits 127:112 of OTP access key 5 (ECC)
func (o *OTP_DATA_Type) SetKEY5_7(value uint16) {
	volatile.StoreUint16(&o.KEY5_7.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY5_7() uint16 {
	return volatile.LoadUint16(&o.KEY5_7.Reg)
}

// OTP_DATA.KEY6_0: Bits 15:0 of OTP access key 6 (ECC)
func (o *OTP_DATA_Type) SetKEY6_0(value uint16) {
	volatile.StoreUint16(&o.KEY6_0.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY6_0() uint16 {
	return volatile.LoadUint16(&o.KEY6_0.Reg)
}

// OTP_DATA.KEY6_1: Bits 31:16 of OTP access key 6 (ECC)
func (o *OTP_DATA_Type) SetKEY6_1(value uint16) {
	volatile.StoreUint16(&o.KEY6_1.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY6_1() uint16 {
	return volatile.LoadUint16(&o.KEY6_1.Reg)
}

// OTP_DATA.KEY6_2: Bits 47:32 of OTP access key 6 (ECC)
func (o *OTP_DATA_Type) SetKEY6_2(value uint16) {
	volatile.StoreUint16(&o.KEY6_2.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY6_2() uint16 {
	return volatile.LoadUint16(&o.KEY6_2.Reg)
}

// OTP_DATA.KEY6_3: Bits 63:48 of OTP access key 6 (ECC)
func (o *OTP_DATA_Type) SetKEY6_3(value uint16) {
	volatile.StoreUint16(&o.KEY6_3.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY6_3() uint16 {
	return volatile.LoadUint16(&o.KEY6_3.Reg)
}

// OTP_DATA.KEY6_4: Bits 79:64 of OTP access key 6 (ECC)
func (o *OTP_DATA_Type) SetKEY6_4(value uint16) {
	volatile.StoreUint16(&o.KEY6_4.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY6_4() uint16 {
	return volatile.LoadUint16(&o.KEY6_4.Reg)
}

// OTP_DATA.KEY6_5: Bits 95:80 of OTP access key 6 (ECC)
func (o *OTP_DATA_Type) SetKEY6_5(value uint16) {
	volatile.StoreUint16(&o.KEY6_5.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY6_5() uint16 {
	return volatile.LoadUint16(&o.KEY6_5.Reg)
}

// OTP_DATA.KEY6_6: Bits 111:96 of OTP access key 6 (ECC)
func (o *OTP_DATA_Type) SetKEY6_6(value uint16) {
	volatile.StoreUint16(&o.KEY6_6.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY6_6() uint16 {
	return volatile.LoadUint16(&o.KEY6_6.Reg)
}

// OTP_DATA.KEY6_7: Bits 127:112 of OTP access key 6 (ECC)
func (o *OTP_DATA_Type) SetKEY6_7(value uint16) {
	volatile.StoreUint16(&o.KEY6_7.Reg, value)
}
func (o *OTP_DATA_Type) GetKEY6_7() uint16 {
	return volatile.LoadUint16(&o.KEY6_7.Reg)
}

// Predefined OTP data layout for RP2350
type OTP_DATA_RAW_Type struct {
	CHIPID0                   volatile.Register32 // 0x0
	CHIPID1                   volatile.Register32 // 0x4
	CHIPID2                   volatile.Register32 // 0x8
	CHIPID3                   volatile.Register32 // 0xC
	RANDID0                   volatile.Register32 // 0x10
	RANDID1                   volatile.Register32 // 0x14
	RANDID2                   volatile.Register32 // 0x18
	RANDID3                   volatile.Register32 // 0x1C
	RANDID4                   volatile.Register32 // 0x20
	RANDID5                   volatile.Register32 // 0x24
	RANDID6                   volatile.Register32 // 0x28
	RANDID7                   volatile.Register32 // 0x2C
	_                         [16]byte
	ROSC_CALIB                volatile.Register32 // 0x40
	LPOSC_CALIB               volatile.Register32 // 0x44
	_                         [24]byte
	NUM_GPIOS                 volatile.Register32 // 0x60
	_                         [116]byte
	INFO_CRC0                 volatile.Register32 // 0xD8
	INFO_CRC1                 volatile.Register32 // 0xDC
	CRIT0                     volatile.Register32 // 0xE0
	CRIT0_R1                  volatile.Register32 // 0xE4
	CRIT0_R2                  volatile.Register32 // 0xE8
	CRIT0_R3                  volatile.Register32 // 0xEC
	CRIT0_R4                  volatile.Register32 // 0xF0
	CRIT0_R5                  volatile.Register32 // 0xF4
	CRIT0_R6                  volatile.Register32 // 0xF8
	CRIT0_R7                  volatile.Register32 // 0xFC
	CRIT1                     volatile.Register32 // 0x100
	CRIT1_R1                  volatile.Register32 // 0x104
	CRIT1_R2                  volatile.Register32 // 0x108
	CRIT1_R3                  volatile.Register32 // 0x10C
	CRIT1_R4                  volatile.Register32 // 0x110
	CRIT1_R5                  volatile.Register32 // 0x114
	CRIT1_R6                  volatile.Register32 // 0x118
	CRIT1_R7                  volatile.Register32 // 0x11C
	BOOT_FLAGS0               volatile.Register32 // 0x120
	BOOT_FLAGS0_R1            volatile.Register32 // 0x124
	BOOT_FLAGS0_R2            volatile.Register32 // 0x128
	BOOT_FLAGS1               volatile.Register32 // 0x12C
	BOOT_FLAGS1_R1            volatile.Register32 // 0x130
	BOOT_FLAGS1_R2            volatile.Register32 // 0x134
	DEFAULT_BOOT_VERSION0     volatile.Register32 // 0x138
	DEFAULT_BOOT_VERSION0_R1  volatile.Register32 // 0x13C
	DEFAULT_BOOT_VERSION0_R2  volatile.Register32 // 0x140
	DEFAULT_BOOT_VERSION1     volatile.Register32 // 0x144
	DEFAULT_BOOT_VERSION1_R1  volatile.Register32 // 0x148
	DEFAULT_BOOT_VERSION1_R2  volatile.Register32 // 0x14C
	FLASH_DEVINFO             volatile.Register32 // 0x150
	FLASH_PARTITION_SLOT_SIZE volatile.Register32 // 0x154
	BOOTSEL_LED_CFG           volatile.Register32 // 0x158
	BOOTSEL_PLL_CFG           volatile.Register32 // 0x15C
	BOOTSEL_XOSC_CFG          volatile.Register32 // 0x160
	USB_BOOT_FLAGS            volatile.Register32 // 0x164
	USB_BOOT_FLAGS_R1         volatile.Register32 // 0x168
	USB_BOOT_FLAGS_R2         volatile.Register32 // 0x16C
	USB_WHITE_LABEL_ADDR      volatile.Register32 // 0x170
	_                         [4]byte
	OTPBOOT_SRC               volatile.Register32 // 0x178
	OTPBOOT_LEN               volatile.Register32 // 0x17C
	OTPBOOT_DST0              volatile.Register32 // 0x180
	OTPBOOT_DST1              volatile.Register32 // 0x184
	_                         [120]byte
	BOOTKEY0_0                volatile.Register32 // 0x200
	BOOTKEY0_1                volatile.Register32 // 0x204
	BOOTKEY0_2                volatile.Register32 // 0x208
	BOOTKEY0_3                volatile.Register32 // 0x20C
	BOOTKEY0_4                volatile.Register32 // 0x210
	BOOTKEY0_5                volatile.Register32 // 0x214
	BOOTKEY0_6                volatile.Register32 // 0x218
	BOOTKEY0_7                volatile.Register32 // 0x21C
	BOOTKEY0_8                volatile.Register32 // 0x220
	BOOTKEY0_9                volatile.Register32 // 0x224
	BOOTKEY0_10               volatile.Register32 // 0x228
	BOOTKEY0_11               volatile.Register32 // 0x22C
	BOOTKEY0_12               volatile.Register32 // 0x230
	BOOTKEY0_13               volatile.Register32 // 0x234
	BOOTKEY0_14               volatile.Register32 // 0x238
	BOOTKEY0_15               volatile.Register32 // 0x23C
	BOOTKEY1_0                volatile.Register32 // 0x240
	BOOTKEY1_1                volatile.Register32 // 0x244
	BOOTKEY1_2                volatile.Register32 // 0x248
	BOOTKEY1_3                volatile.Register32 // 0x24C
	BOOTKEY1_4                volatile.Register32 // 0x250
	BOOTKEY1_5                volatile.Register32 // 0x254
	BOOTKEY1_6                volatile.Register32 // 0x258
	BOOTKEY1_7                volatile.Register32 // 0x25C
	BOOTKEY1_8                volatile.Register32 // 0x260
	BOOTKEY1_9                volatile.Register32 // 0x264
	BOOTKEY1_10               volatile.Register32 // 0x268
	BOOTKEY1_11               volatile.Register32 // 0x26C
	BOOTKEY1_12               volatile.Register32 // 0x270
	BOOTKEY1_13               volatile.Register32 // 0x274
	BOOTKEY1_14               volatile.Register32 // 0x278
	BOOTKEY1_15               volatile.Register32 // 0x27C
	BOOTKEY2_0                volatile.Register32 // 0x280
	BOOTKEY2_1                volatile.Register32 // 0x284
	BOOTKEY2_2                volatile.Register32 // 0x288
	BOOTKEY2_3                volatile.Register32 // 0x28C
	BOOTKEY2_4                volatile.Register32 // 0x290
	BOOTKEY2_5                volatile.Register32 // 0x294
	BOOTKEY2_6                volatile.Register32 // 0x298
	BOOTKEY2_7                volatile.Register32 // 0x29C
	BOOTKEY2_8                volatile.Register32 // 0x2A0
	BOOTKEY2_9                volatile.Register32 // 0x2A4
	BOOTKEY2_10               volatile.Register32 // 0x2A8
	BOOTKEY2_11               volatile.Register32 // 0x2AC
	BOOTKEY2_12               volatile.Register32 // 0x2B0
	BOOTKEY2_13               volatile.Register32 // 0x2B4
	BOOTKEY2_14               volatile.Register32 // 0x2B8
	BOOTKEY2_15               volatile.Register32 // 0x2BC
	BOOTKEY3_0                volatile.Register32 // 0x2C0
	BOOTKEY3_1                volatile.Register32 // 0x2C4
	BOOTKEY3_2                volatile.Register32 // 0x2C8
	BOOTKEY3_3                volatile.Register32 // 0x2CC
	BOOTKEY3_4                volatile.Register32 // 0x2D0
	BOOTKEY3_5                volatile.Register32 // 0x2D4
	BOOTKEY3_6                volatile.Register32 // 0x2D8
	BOOTKEY3_7                volatile.Register32 // 0x2DC
	BOOTKEY3_8                volatile.Register32 // 0x2E0
	BOOTKEY3_9                volatile.Register32 // 0x2E4
	BOOTKEY3_10               volatile.Register32 // 0x2E8
	BOOTKEY3_11               volatile.Register32 // 0x2EC
	BOOTKEY3_12               volatile.Register32 // 0x2F0
	BOOTKEY3_13               volatile.Register32 // 0x2F4
	BOOTKEY3_14               volatile.Register32 // 0x2F8
	BOOTKEY3_15               volatile.Register32 // 0x2FC
	_                         [14880]byte
	KEY1_0                    volatile.Register32 // 0x3D20
	KEY1_1                    volatile.Register32 // 0x3D24
	KEY1_2                    volatile.Register32 // 0x3D28
	KEY1_3                    volatile.Register32 // 0x3D2C
	KEY1_4                    volatile.Register32 // 0x3D30
	KEY1_5                    volatile.Register32 // 0x3D34
	KEY1_6                    volatile.Register32 // 0x3D38
	KEY1_7                    volatile.Register32 // 0x3D3C
	KEY2_0                    volatile.Register32 // 0x3D40
	KEY2_1                    volatile.Register32 // 0x3D44
	KEY2_2                    volatile.Register32 // 0x3D48
	KEY2_3                    volatile.Register32 // 0x3D4C
	KEY2_4                    volatile.Register32 // 0x3D50
	KEY2_5                    volatile.Register32 // 0x3D54
	KEY2_6                    volatile.Register32 // 0x3D58
	KEY2_7                    volatile.Register32 // 0x3D5C
	KEY3_0                    volatile.Register32 // 0x3D60
	KEY3_1                    volatile.Register32 // 0x3D64
	KEY3_2                    volatile.Register32 // 0x3D68
	KEY3_3                    volatile.Register32 // 0x3D6C
	KEY3_4                    volatile.Register32 // 0x3D70
	KEY3_5                    volatile.Register32 // 0x3D74
	KEY3_6                    volatile.Register32 // 0x3D78
	KEY3_7                    volatile.Register32 // 0x3D7C
	KEY4_0                    volatile.Register32 // 0x3D80
	KEY4_1                    volatile.Register32 // 0x3D84
	KEY4_2                    volatile.Register32 // 0x3D88
	KEY4_3                    volatile.Register32 // 0x3D8C
	KEY4_4                    volatile.Register32 // 0x3D90
	KEY4_5                    volatile.Register32 // 0x3D94
	KEY4_6                    volatile.Register32 // 0x3D98
	KEY4_7                    volatile.Register32 // 0x3D9C
	KEY5_0                    volatile.Register32 // 0x3DA0
	KEY5_1                    volatile.Register32 // 0x3DA4
	KEY5_2                    volatile.Register32 // 0x3DA8
	KEY5_3                    volatile.Register32 // 0x3DAC
	KEY5_4                    volatile.Register32 // 0x3DB0
	KEY5_5                    volatile.Register32 // 0x3DB4
	KEY5_6                    volatile.Register32 // 0x3DB8
	KEY5_7                    volatile.Register32 // 0x3DBC
	KEY6_0                    volatile.Register32 // 0x3DC0
	KEY6_1                    volatile.Register32 // 0x3DC4
	KEY6_2                    volatile.Register32 // 0x3DC8
	KEY6_3                    volatile.Register32 // 0x3DCC
	KEY6_4                    volatile.Register32 // 0x3DD0
	KEY6_5                    volatile.Register32 // 0x3DD4
	KEY6_6                    volatile.Register32 // 0x3DD8
	KEY6_7                    volatile.Register32 // 0x3DDC
	_                         [4]byte
	KEY1_VALID                volatile.Register32 // 0x3DE4
	KEY2_VALID                volatile.Register32 // 0x3DE8
	KEY3_VALID                volatile.Register32 // 0x3DEC
	KEY4_VALID                volatile.Register32 // 0x3DF0
	KEY5_VALID                volatile.Register32 // 0x3DF4
	KEY6_VALID                volatile.Register32 // 0x3DF8
	_                         [4]byte
	PAGE0_LOCK0               volatile.Register32 // 0x3E00
	PAGE0_LOCK1               volatile.Register32 // 0x3E04
	PAGE1_LOCK0               volatile.Register32 // 0x3E08
	PAGE1_LOCK1               volatile.Register32 // 0x3E0C
	PAGE2_LOCK0               volatile.Register32 // 0x3E10
	PAGE2_LOCK1               volatile.Register32 // 0x3E14
	PAGE3_LOCK0               volatile.Register32 // 0x3E18
	PAGE3_LOCK1               volatile.Register32 // 0x3E1C
	PAGE4_LOCK0               volatile.Register32 // 0x3E20
	PAGE4_LOCK1               volatile.Register32 // 0x3E24
	PAGE5_LOCK0               volatile.Register32 // 0x3E28
	PAGE5_LOCK1               volatile.Register32 // 0x3E2C
	PAGE6_LOCK0               volatile.Register32 // 0x3E30
	PAGE6_LOCK1               volatile.Register32 // 0x3E34
	PAGE7_LOCK0               volatile.Register32 // 0x3E38
	PAGE7_LOCK1               volatile.Register32 // 0x3E3C
	PAGE8_LOCK0               volatile.Register32 // 0x3E40
	PAGE8_LOCK1               volatile.Register32 // 0x3E44
	PAGE9_LOCK0               volatile.Register32 // 0x3E48
	PAGE9_LOCK1               volatile.Register32 // 0x3E4C
	PAGE10_LOCK0              volatile.Register32 // 0x3E50
	PAGE10_LOCK1              volatile.Register32 // 0x3E54
	PAGE11_LOCK0              volatile.Register32 // 0x3E58
	PAGE11_LOCK1              volatile.Register32 // 0x3E5C
	PAGE12_LOCK0              volatile.Register32 // 0x3E60
	PAGE12_LOCK1              volatile.Register32 // 0x3E64
	PAGE13_LOCK0              volatile.Register32 // 0x3E68
	PAGE13_LOCK1              volatile.Register32 // 0x3E6C
	PAGE14_LOCK0              volatile.Register32 // 0x3E70
	PAGE14_LOCK1              volatile.Register32 // 0x3E74
	PAGE15_LOCK0              volatile.Register32 // 0x3E78
	PAGE15_LOCK1              volatile.Register32 // 0x3E7C
	PAGE16_LOCK0              volatile.Register32 // 0x3E80
	PAGE16_LOCK1              volatile.Register32 // 0x3E84
	PAGE17_LOCK0              volatile.Register32 // 0x3E88
	PAGE17_LOCK1              volatile.Register32 // 0x3E8C
	PAGE18_LOCK0              volatile.Register32 // 0x3E90
	PAGE18_LOCK1              volatile.Register32 // 0x3E94
	PAGE19_LOCK0              volatile.Register32 // 0x3E98
	PAGE19_LOCK1              volatile.Register32 // 0x3E9C
	PAGE20_LOCK0              volatile.Register32 // 0x3EA0
	PAGE20_LOCK1              volatile.Register32 // 0x3EA4
	PAGE21_LOCK0              volatile.Register32 // 0x3EA8
	PAGE21_LOCK1              volatile.Register32 // 0x3EAC
	PAGE22_LOCK0              volatile.Register32 // 0x3EB0
	PAGE22_LOCK1              volatile.Register32 // 0x3EB4
	PAGE23_LOCK0              volatile.Register32 // 0x3EB8
	PAGE23_LOCK1              volatile.Register32 // 0x3EBC
	PAGE24_LOCK0              volatile.Register32 // 0x3EC0
	PAGE24_LOCK1              volatile.Register32 // 0x3EC4
	PAGE25_LOCK0              volatile.Register32 // 0x3EC8
	PAGE25_LOCK1              volatile.Register32 // 0x3ECC
	PAGE26_LOCK0              volatile.Register32 // 0x3ED0
	PAGE26_LOCK1              volatile.Register32 // 0x3ED4
	PAGE27_LOCK0              volatile.Register32 // 0x3ED8
	PAGE27_LOCK1              volatile.Register32 // 0x3EDC
	PAGE28_LOCK0              volatile.Register32 // 0x3EE0
	PAGE28_LOCK1              volatile.Register32 // 0x3EE4
	PAGE29_LOCK0              volatile.Register32 // 0x3EE8
	PAGE29_LOCK1              volatile.Register32 // 0x3EEC
	PAGE30_LOCK0              volatile.Register32 // 0x3EF0
	PAGE30_LOCK1              volatile.Register32 // 0x3EF4
	PAGE31_LOCK0              volatile.Register32 // 0x3EF8
	PAGE31_LOCK1              volatile.Register32 // 0x3EFC
	PAGE32_LOCK0              volatile.Register32 // 0x3F00
	PAGE32_LOCK1              volatile.Register32 // 0x3F04
	PAGE33_LOCK0              volatile.Register32 // 0x3F08
	PAGE33_LOCK1              volatile.Register32 // 0x3F0C
	PAGE34_LOCK0              volatile.Register32 // 0x3F10
	PAGE34_LOCK1              volatile.Register32 // 0x3F14
	PAGE35_LOCK0              volatile.Register32 // 0x3F18
	PAGE35_LOCK1              volatile.Register32 // 0x3F1C
	PAGE36_LOCK0              volatile.Register32 // 0x3F20
	PAGE36_LOCK1              volatile.Register32 // 0x3F24
	PAGE37_LOCK0              volatile.Register32 // 0x3F28
	PAGE37_LOCK1              volatile.Register32 // 0x3F2C
	PAGE38_LOCK0              volatile.Register32 // 0x3F30
	PAGE38_LOCK1              volatile.Register32 // 0x3F34
	PAGE39_LOCK0              volatile.Register32 // 0x3F38
	PAGE39_LOCK1              volatile.Register32 // 0x3F3C
	PAGE40_LOCK0              volatile.Register32 // 0x3F40
	PAGE40_LOCK1              volatile.Register32 // 0x3F44
	PAGE41_LOCK0              volatile.Register32 // 0x3F48
	PAGE41_LOCK1              volatile.Register32 // 0x3F4C
	PAGE42_LOCK0              volatile.Register32 // 0x3F50
	PAGE42_LOCK1              volatile.Register32 // 0x3F54
	PAGE43_LOCK0              volatile.Register32 // 0x3F58
	PAGE43_LOCK1              volatile.Register32 // 0x3F5C
	PAGE44_LOCK0              volatile.Register32 // 0x3F60
	PAGE44_LOCK1              volatile.Register32 // 0x3F64
	PAGE45_LOCK0              volatile.Register32 // 0x3F68
	PAGE45_LOCK1              volatile.Register32 // 0x3F6C
	PAGE46_LOCK0              volatile.Register32 // 0x3F70
	PAGE46_LOCK1              volatile.Register32 // 0x3F74
	PAGE47_LOCK0              volatile.Register32 // 0x3F78
	PAGE47_LOCK1              volatile.Register32 // 0x3F7C
	PAGE48_LOCK0              volatile.Register32 // 0x3F80
	PAGE48_LOCK1              volatile.Register32 // 0x3F84
	PAGE49_LOCK0              volatile.Register32 // 0x3F88
	PAGE49_LOCK1              volatile.Register32 // 0x3F8C
	PAGE50_LOCK0              volatile.Register32 // 0x3F90
	PAGE50_LOCK1              volatile.Register32 // 0x3F94
	PAGE51_LOCK0              volatile.Register32 // 0x3F98
	PAGE51_LOCK1              volatile.Register32 // 0x3F9C
	PAGE52_LOCK0              volatile.Register32 // 0x3FA0
	PAGE52_LOCK1              volatile.Register32 // 0x3FA4
	PAGE53_LOCK0              volatile.Register32 // 0x3FA8
	PAGE53_LOCK1              volatile.Register32 // 0x3FAC
	PAGE54_LOCK0              volatile.Register32 // 0x3FB0
	PAGE54_LOCK1              volatile.Register32 // 0x3FB4
	PAGE55_LOCK0              volatile.Register32 // 0x3FB8
	PAGE55_LOCK1              volatile.Register32 // 0x3FBC
	PAGE56_LOCK0              volatile.Register32 // 0x3FC0
	PAGE56_LOCK1              volatile.Register32 // 0x3FC4
	PAGE57_LOCK0              volatile.Register32 // 0x3FC8
	PAGE57_LOCK1              volatile.Register32 // 0x3FCC
	PAGE58_LOCK0              volatile.Register32 // 0x3FD0
	PAGE58_LOCK1              volatile.Register32 // 0x3FD4
	PAGE59_LOCK0              volatile.Register32 // 0x3FD8
	PAGE59_LOCK1              volatile.Register32 // 0x3FDC
	PAGE60_LOCK0              volatile.Register32 // 0x3FE0
	PAGE60_LOCK1              volatile.Register32 // 0x3FE4
	PAGE61_LOCK0              volatile.Register32 // 0x3FE8
	PAGE61_LOCK1              volatile.Register32 // 0x3FEC
	PAGE62_LOCK0              volatile.Register32 // 0x3FF0
	PAGE62_LOCK1              volatile.Register32 // 0x3FF4
	PAGE63_LOCK0              volatile.Register32 // 0x3FF8
	PAGE63_LOCK1              volatile.Register32 // 0x3FFC
}

// OTP_DATA_RAW.CHIPID0: Bits 15:0 of public device ID. (ECC) The CHIPID0..3 rows contain a 64-bit random identifier for this chip, which can be read from the USB bootloader PICOBOOT interface or from the get_sys_info ROM API. The number of random bits makes the occurrence of twins exceedingly unlikely: for example, a fleet of a hundred million devices has a 99.97% probability of no twinned IDs. This is estimated to be lower than the occurrence of process errors in the assignment of sequential random IDs, and for practical purposes CHIPID may be treated as unique.
func (o *OTP_DATA_RAW_Type) SetCHIPID0(value uint32) {
	volatile.StoreUint32(&o.CHIPID0.Reg, volatile.LoadUint32(&o.CHIPID0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCHIPID0() uint32 {
	return volatile.LoadUint32(&o.CHIPID0.Reg) & 0xffff
}

// OTP_DATA_RAW.CHIPID1: Bits 31:16 of public device ID (ECC)
func (o *OTP_DATA_RAW_Type) SetCHIPID1(value uint32) {
	volatile.StoreUint32(&o.CHIPID1.Reg, volatile.LoadUint32(&o.CHIPID1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCHIPID1() uint32 {
	return volatile.LoadUint32(&o.CHIPID1.Reg) & 0xffff
}

// OTP_DATA_RAW.CHIPID2: Bits 47:32 of public device ID (ECC)
func (o *OTP_DATA_RAW_Type) SetCHIPID2(value uint32) {
	volatile.StoreUint32(&o.CHIPID2.Reg, volatile.LoadUint32(&o.CHIPID2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCHIPID2() uint32 {
	return volatile.LoadUint32(&o.CHIPID2.Reg) & 0xffff
}

// OTP_DATA_RAW.CHIPID3: Bits 63:48 of public device ID (ECC)
func (o *OTP_DATA_RAW_Type) SetCHIPID3(value uint32) {
	volatile.StoreUint32(&o.CHIPID3.Reg, volatile.LoadUint32(&o.CHIPID3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCHIPID3() uint32 {
	return volatile.LoadUint32(&o.CHIPID3.Reg) & 0xffff
}

// OTP_DATA_RAW.RANDID0: Bits 15:0 of private per-device random number (ECC) The RANDID0..7 rows form a 128-bit random number generated during device test. This ID is not exposed through the USB PICOBOOT GET_INFO command or the ROM `get_sys_info()` API. However note that the USB PICOBOOT OTP access point can read the entirety of page 0, so this value is not meaningfully private unless the USB PICOBOOT interface is disabled via the DISABLE_BOOTSEL_USB_PICOBOOT_IFC flag in BOOT_FLAGS0.
func (o *OTP_DATA_RAW_Type) SetRANDID0(value uint32) {
	volatile.StoreUint32(&o.RANDID0.Reg, volatile.LoadUint32(&o.RANDID0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetRANDID0() uint32 {
	return volatile.LoadUint32(&o.RANDID0.Reg) & 0xffff
}

// OTP_DATA_RAW.RANDID1: Bits 31:16 of private per-device random number (ECC)
func (o *OTP_DATA_RAW_Type) SetRANDID1(value uint32) {
	volatile.StoreUint32(&o.RANDID1.Reg, volatile.LoadUint32(&o.RANDID1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetRANDID1() uint32 {
	return volatile.LoadUint32(&o.RANDID1.Reg) & 0xffff
}

// OTP_DATA_RAW.RANDID2: Bits 47:32 of private per-device random number (ECC)
func (o *OTP_DATA_RAW_Type) SetRANDID2(value uint32) {
	volatile.StoreUint32(&o.RANDID2.Reg, volatile.LoadUint32(&o.RANDID2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetRANDID2() uint32 {
	return volatile.LoadUint32(&o.RANDID2.Reg) & 0xffff
}

// OTP_DATA_RAW.RANDID3: Bits 63:48 of private per-device random number (ECC)
func (o *OTP_DATA_RAW_Type) SetRANDID3(value uint32) {
	volatile.StoreUint32(&o.RANDID3.Reg, volatile.LoadUint32(&o.RANDID3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetRANDID3() uint32 {
	return volatile.LoadUint32(&o.RANDID3.Reg) & 0xffff
}

// OTP_DATA_RAW.RANDID4: Bits 79:64 of private per-device random number (ECC)
func (o *OTP_DATA_RAW_Type) SetRANDID4(value uint32) {
	volatile.StoreUint32(&o.RANDID4.Reg, volatile.LoadUint32(&o.RANDID4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetRANDID4() uint32 {
	return volatile.LoadUint32(&o.RANDID4.Reg) & 0xffff
}

// OTP_DATA_RAW.RANDID5: Bits 95:80 of private per-device random number (ECC)
func (o *OTP_DATA_RAW_Type) SetRANDID5(value uint32) {
	volatile.StoreUint32(&o.RANDID5.Reg, volatile.LoadUint32(&o.RANDID5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetRANDID5() uint32 {
	return volatile.LoadUint32(&o.RANDID5.Reg) & 0xffff
}

// OTP_DATA_RAW.RANDID6: Bits 111:96 of private per-device random number (ECC)
func (o *OTP_DATA_RAW_Type) SetRANDID6(value uint32) {
	volatile.StoreUint32(&o.RANDID6.Reg, volatile.LoadUint32(&o.RANDID6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetRANDID6() uint32 {
	return volatile.LoadUint32(&o.RANDID6.Reg) & 0xffff
}

// OTP_DATA_RAW.RANDID7: Bits 127:112 of private per-device random number (ECC)
func (o *OTP_DATA_RAW_Type) SetRANDID7(value uint32) {
	volatile.StoreUint32(&o.RANDID7.Reg, volatile.LoadUint32(&o.RANDID7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetRANDID7() uint32 {
	return volatile.LoadUint32(&o.RANDID7.Reg) & 0xffff
}

// OTP_DATA_RAW.ROSC_CALIB: Ring oscillator frequency in kHz, measured during manufacturing (ECC) This is measured at 1.1 V, at room temperature, with the ROSC configuration registers in their reset state.
func (o *OTP_DATA_RAW_Type) SetROSC_CALIB(value uint32) {
	volatile.StoreUint32(&o.ROSC_CALIB.Reg, volatile.LoadUint32(&o.ROSC_CALIB.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetROSC_CALIB() uint32 {
	return volatile.LoadUint32(&o.ROSC_CALIB.Reg) & 0xffff
}

// OTP_DATA_RAW.LPOSC_CALIB: Low-power oscillator frequency in Hz, measured during manufacturing (ECC) This is measured at 1.1V, at room temperature, with the LPOSC trim register in its reset state.
func (o *OTP_DATA_RAW_Type) SetLPOSC_CALIB(value uint32) {
	volatile.StoreUint32(&o.LPOSC_CALIB.Reg, volatile.LoadUint32(&o.LPOSC_CALIB.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetLPOSC_CALIB() uint32 {
	return volatile.LoadUint32(&o.LPOSC_CALIB.Reg) & 0xffff
}

// OTP_DATA_RAW.NUM_GPIOS: The number of main user GPIOs (bank 0). Should read 48 in the QFN80 package, and 30 in the QFN60 package. (ECC)
func (o *OTP_DATA_RAW_Type) SetNUM_GPIOS(value uint32) {
	volatile.StoreUint32(&o.NUM_GPIOS.Reg, volatile.LoadUint32(&o.NUM_GPIOS.Reg)&^(0xff)|value)
}
func (o *OTP_DATA_RAW_Type) GetNUM_GPIOS() uint32 {
	return volatile.LoadUint32(&o.NUM_GPIOS.Reg) & 0xff
}

// OTP_DATA_RAW.INFO_CRC0: Lower 16 bits of CRC32 of OTP addresses 0x00 through 0x6b (polynomial 0x4c11db7, input reflected, output reflected, seed all-ones, final XOR all-ones) (ECC)
func (o *OTP_DATA_RAW_Type) SetINFO_CRC0(value uint32) {
	volatile.StoreUint32(&o.INFO_CRC0.Reg, volatile.LoadUint32(&o.INFO_CRC0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetINFO_CRC0() uint32 {
	return volatile.LoadUint32(&o.INFO_CRC0.Reg) & 0xffff
}

// OTP_DATA_RAW.INFO_CRC1: Upper 16 bits of CRC32 of OTP addresses 0x00 through 0x6b (ECC)
func (o *OTP_DATA_RAW_Type) SetINFO_CRC1(value uint32) {
	volatile.StoreUint32(&o.INFO_CRC1.Reg, volatile.LoadUint32(&o.INFO_CRC1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetINFO_CRC1() uint32 {
	return volatile.LoadUint32(&o.INFO_CRC1.Reg) & 0xffff
}

// OTP_DATA_RAW.CRIT0: Page 0 critical boot flags (RBIT-8)
func (o *OTP_DATA_RAW_Type) SetCRIT0_RISCV_DISABLE(value uint32) {
	volatile.StoreUint32(&o.CRIT0.Reg, volatile.LoadUint32(&o.CRIT0.Reg)&^(0x2)|value<<1)
}
func (o *OTP_DATA_RAW_Type) GetCRIT0_RISCV_DISABLE() uint32 {
	return (volatile.LoadUint32(&o.CRIT0.Reg) & 0x2) >> 1
}
func (o *OTP_DATA_RAW_Type) SetCRIT0_ARM_DISABLE(value uint32) {
	volatile.StoreUint32(&o.CRIT0.Reg, volatile.LoadUint32(&o.CRIT0.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT0_ARM_DISABLE() uint32 {
	return volatile.LoadUint32(&o.CRIT0.Reg) & 0x1
}

// OTP_DATA_RAW.CRIT0_R1: Redundant copy of CRIT0
func (o *OTP_DATA_RAW_Type) SetCRIT0_R1(value uint32) {
	volatile.StoreUint32(&o.CRIT0_R1.Reg, volatile.LoadUint32(&o.CRIT0_R1.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT0_R1() uint32 {
	return volatile.LoadUint32(&o.CRIT0_R1.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT0_R2: Redundant copy of CRIT0
func (o *OTP_DATA_RAW_Type) SetCRIT0_R2(value uint32) {
	volatile.StoreUint32(&o.CRIT0_R2.Reg, volatile.LoadUint32(&o.CRIT0_R2.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT0_R2() uint32 {
	return volatile.LoadUint32(&o.CRIT0_R2.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT0_R3: Redundant copy of CRIT0
func (o *OTP_DATA_RAW_Type) SetCRIT0_R3(value uint32) {
	volatile.StoreUint32(&o.CRIT0_R3.Reg, volatile.LoadUint32(&o.CRIT0_R3.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT0_R3() uint32 {
	return volatile.LoadUint32(&o.CRIT0_R3.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT0_R4: Redundant copy of CRIT0
func (o *OTP_DATA_RAW_Type) SetCRIT0_R4(value uint32) {
	volatile.StoreUint32(&o.CRIT0_R4.Reg, volatile.LoadUint32(&o.CRIT0_R4.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT0_R4() uint32 {
	return volatile.LoadUint32(&o.CRIT0_R4.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT0_R5: Redundant copy of CRIT0
func (o *OTP_DATA_RAW_Type) SetCRIT0_R5(value uint32) {
	volatile.StoreUint32(&o.CRIT0_R5.Reg, volatile.LoadUint32(&o.CRIT0_R5.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT0_R5() uint32 {
	return volatile.LoadUint32(&o.CRIT0_R5.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT0_R6: Redundant copy of CRIT0
func (o *OTP_DATA_RAW_Type) SetCRIT0_R6(value uint32) {
	volatile.StoreUint32(&o.CRIT0_R6.Reg, volatile.LoadUint32(&o.CRIT0_R6.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT0_R6() uint32 {
	return volatile.LoadUint32(&o.CRIT0_R6.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT0_R7: Redundant copy of CRIT0
func (o *OTP_DATA_RAW_Type) SetCRIT0_R7(value uint32) {
	volatile.StoreUint32(&o.CRIT0_R7.Reg, volatile.LoadUint32(&o.CRIT0_R7.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT0_R7() uint32 {
	return volatile.LoadUint32(&o.CRIT0_R7.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT1: Page 1 critical boot flags (RBIT-8)
func (o *OTP_DATA_RAW_Type) SetCRIT1_GLITCH_DETECTOR_SENS(value uint32) {
	volatile.StoreUint32(&o.CRIT1.Reg, volatile.LoadUint32(&o.CRIT1.Reg)&^(0x60)|value<<5)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_GLITCH_DETECTOR_SENS() uint32 {
	return (volatile.LoadUint32(&o.CRIT1.Reg) & 0x60) >> 5
}
func (o *OTP_DATA_RAW_Type) SetCRIT1_GLITCH_DETECTOR_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CRIT1.Reg, volatile.LoadUint32(&o.CRIT1.Reg)&^(0x10)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_GLITCH_DETECTOR_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.CRIT1.Reg) & 0x10) >> 4
}
func (o *OTP_DATA_RAW_Type) SetCRIT1_BOOT_ARCH(value uint32) {
	volatile.StoreUint32(&o.CRIT1.Reg, volatile.LoadUint32(&o.CRIT1.Reg)&^(0x8)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_BOOT_ARCH() uint32 {
	return (volatile.LoadUint32(&o.CRIT1.Reg) & 0x8) >> 3
}
func (o *OTP_DATA_RAW_Type) SetCRIT1_DEBUG_DISABLE(value uint32) {
	volatile.StoreUint32(&o.CRIT1.Reg, volatile.LoadUint32(&o.CRIT1.Reg)&^(0x4)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_DEBUG_DISABLE() uint32 {
	return (volatile.LoadUint32(&o.CRIT1.Reg) & 0x4) >> 2
}
func (o *OTP_DATA_RAW_Type) SetCRIT1_SECURE_DEBUG_DISABLE(value uint32) {
	volatile.StoreUint32(&o.CRIT1.Reg, volatile.LoadUint32(&o.CRIT1.Reg)&^(0x2)|value<<1)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_SECURE_DEBUG_DISABLE() uint32 {
	return (volatile.LoadUint32(&o.CRIT1.Reg) & 0x2) >> 1
}
func (o *OTP_DATA_RAW_Type) SetCRIT1_SECURE_BOOT_ENABLE(value uint32) {
	volatile.StoreUint32(&o.CRIT1.Reg, volatile.LoadUint32(&o.CRIT1.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_SECURE_BOOT_ENABLE() uint32 {
	return volatile.LoadUint32(&o.CRIT1.Reg) & 0x1
}

// OTP_DATA_RAW.CRIT1_R1: Redundant copy of CRIT1
func (o *OTP_DATA_RAW_Type) SetCRIT1_R1(value uint32) {
	volatile.StoreUint32(&o.CRIT1_R1.Reg, volatile.LoadUint32(&o.CRIT1_R1.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_R1() uint32 {
	return volatile.LoadUint32(&o.CRIT1_R1.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT1_R2: Redundant copy of CRIT1
func (o *OTP_DATA_RAW_Type) SetCRIT1_R2(value uint32) {
	volatile.StoreUint32(&o.CRIT1_R2.Reg, volatile.LoadUint32(&o.CRIT1_R2.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_R2() uint32 {
	return volatile.LoadUint32(&o.CRIT1_R2.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT1_R3: Redundant copy of CRIT1
func (o *OTP_DATA_RAW_Type) SetCRIT1_R3(value uint32) {
	volatile.StoreUint32(&o.CRIT1_R3.Reg, volatile.LoadUint32(&o.CRIT1_R3.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_R3() uint32 {
	return volatile.LoadUint32(&o.CRIT1_R3.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT1_R4: Redundant copy of CRIT1
func (o *OTP_DATA_RAW_Type) SetCRIT1_R4(value uint32) {
	volatile.StoreUint32(&o.CRIT1_R4.Reg, volatile.LoadUint32(&o.CRIT1_R4.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_R4() uint32 {
	return volatile.LoadUint32(&o.CRIT1_R4.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT1_R5: Redundant copy of CRIT1
func (o *OTP_DATA_RAW_Type) SetCRIT1_R5(value uint32) {
	volatile.StoreUint32(&o.CRIT1_R5.Reg, volatile.LoadUint32(&o.CRIT1_R5.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_R5() uint32 {
	return volatile.LoadUint32(&o.CRIT1_R5.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT1_R6: Redundant copy of CRIT1
func (o *OTP_DATA_RAW_Type) SetCRIT1_R6(value uint32) {
	volatile.StoreUint32(&o.CRIT1_R6.Reg, volatile.LoadUint32(&o.CRIT1_R6.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_R6() uint32 {
	return volatile.LoadUint32(&o.CRIT1_R6.Reg) & 0xffffff
}

// OTP_DATA_RAW.CRIT1_R7: Redundant copy of CRIT1
func (o *OTP_DATA_RAW_Type) SetCRIT1_R7(value uint32) {
	volatile.StoreUint32(&o.CRIT1_R7.Reg, volatile.LoadUint32(&o.CRIT1_R7.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetCRIT1_R7() uint32 {
	return volatile.LoadUint32(&o.CRIT1_R7.Reg) & 0xffffff
}

// OTP_DATA_RAW.BOOT_FLAGS0: Disable/Enable boot paths/features in the RP2350 mask ROM. Disables always supersede enables. Enables are provided where there are other configurations in OTP that must be valid. (RBIT-3)
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_SRAM_WINDOW_BOOT(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x200000)|value<<21)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_SRAM_WINDOW_BOOT() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x200000) >> 21
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_XIP_ACCESS_ON_SRAM_ENTRY(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x100000)|value<<20)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_XIP_ACCESS_ON_SRAM_ENTRY() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x100000) >> 20
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_BOOTSEL_UART_BOOT(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x80000)|value<<19)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_BOOTSEL_UART_BOOT() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x80000) >> 19
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_BOOTSEL_USB_PICOBOOT_IFC(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x40000)|value<<18)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_BOOTSEL_USB_PICOBOOT_IFC() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x40000) >> 18
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_BOOTSEL_USB_MSD_IFC(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x20000)|value<<17)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_BOOTSEL_USB_MSD_IFC() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x20000) >> 17
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_WATCHDOG_SCRATCH(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x10000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_WATCHDOG_SCRATCH() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x10000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_POWER_SCRATCH(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x8000)|value<<15)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_POWER_SCRATCH() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x8000) >> 15
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_ENABLE_OTP_BOOT(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x4000)|value<<14)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_ENABLE_OTP_BOOT() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x4000) >> 14
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_OTP_BOOT(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x2000)|value<<13)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_OTP_BOOT() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x2000) >> 13
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_FLASH_BOOT(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x1000)|value<<12)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_FLASH_BOOT() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x1000) >> 12
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_ROLLBACK_REQUIRED(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x800)|value<<11)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_ROLLBACK_REQUIRED() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x800) >> 11
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_HASHED_PARTITION_TABLE(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x400)|value<<10)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_HASHED_PARTITION_TABLE() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x400) >> 10
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_SECURE_PARTITION_TABLE(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x200)|value<<9)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_SECURE_PARTITION_TABLE() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x200) >> 9
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_AUTO_SWITCH_ARCH(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_AUTO_SWITCH_ARCH() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_SINGLE_FLASH_BINARY(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x80)|value<<7)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_SINGLE_FLASH_BINARY() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x80) >> 7
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_OVERRIDE_FLASH_PARTITION_SLOT_SIZE(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_OVERRIDE_FLASH_PARTITION_SLOT_SIZE() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_FLASH_DEVINFO_ENABLE(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x20)|value<<5)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_FLASH_DEVINFO_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x20) >> 5
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_FAST_SIGCHECK_ROSC_DIV(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x10)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_FAST_SIGCHECK_ROSC_DIV() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x10) >> 4
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_FLASH_IO_VOLTAGE_1V8(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x8)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_FLASH_IO_VOLTAGE_1V8() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x8) >> 3
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x4)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x4) >> 2
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_ENABLE_BOOTSEL_LED(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x2)|value<<1)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_ENABLE_BOOTSEL_LED() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x2) >> 1
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_DISABLE_BOOTSEL_EXEC2(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_DISABLE_BOOTSEL_EXEC2() uint32 {
	return volatile.LoadUint32(&o.BOOT_FLAGS0.Reg) & 0x1
}

// OTP_DATA_RAW.BOOT_FLAGS0_R1: Redundant copy of BOOT_FLAGS0
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_R1(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0_R1.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0_R1.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_R1() uint32 {
	return volatile.LoadUint32(&o.BOOT_FLAGS0_R1.Reg) & 0xffffff
}

// OTP_DATA_RAW.BOOT_FLAGS0_R2: Redundant copy of BOOT_FLAGS0
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS0_R2(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS0_R2.Reg, volatile.LoadUint32(&o.BOOT_FLAGS0_R2.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS0_R2() uint32 {
	return volatile.LoadUint32(&o.BOOT_FLAGS0_R2.Reg) & 0xffffff
}

// OTP_DATA_RAW.BOOT_FLAGS1: Disable/Enable boot paths/features in the RP2350 mask ROM. Disables always supersede enables. Enables are provided where there are other configurations in OTP that must be valid. (RBIT-3)
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS1_DOUBLE_TAP(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS1.Reg, volatile.LoadUint32(&o.BOOT_FLAGS1.Reg)&^(0x80000)|value<<19)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS1_DOUBLE_TAP() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS1.Reg) & 0x80000) >> 19
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS1_DOUBLE_TAP_DELAY(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS1.Reg, volatile.LoadUint32(&o.BOOT_FLAGS1.Reg)&^(0x70000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS1_DOUBLE_TAP_DELAY() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS1.Reg) & 0x70000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS1_KEY_INVALID(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS1.Reg, volatile.LoadUint32(&o.BOOT_FLAGS1.Reg)&^(0xf00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS1_KEY_INVALID() uint32 {
	return (volatile.LoadUint32(&o.BOOT_FLAGS1.Reg) & 0xf00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS1_KEY_VALID(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS1.Reg, volatile.LoadUint32(&o.BOOT_FLAGS1.Reg)&^(0xf)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS1_KEY_VALID() uint32 {
	return volatile.LoadUint32(&o.BOOT_FLAGS1.Reg) & 0xf
}

// OTP_DATA_RAW.BOOT_FLAGS1_R1: Redundant copy of BOOT_FLAGS1
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS1_R1(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS1_R1.Reg, volatile.LoadUint32(&o.BOOT_FLAGS1_R1.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS1_R1() uint32 {
	return volatile.LoadUint32(&o.BOOT_FLAGS1_R1.Reg) & 0xffffff
}

// OTP_DATA_RAW.BOOT_FLAGS1_R2: Redundant copy of BOOT_FLAGS1
func (o *OTP_DATA_RAW_Type) SetBOOT_FLAGS1_R2(value uint32) {
	volatile.StoreUint32(&o.BOOT_FLAGS1_R2.Reg, volatile.LoadUint32(&o.BOOT_FLAGS1_R2.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOT_FLAGS1_R2() uint32 {
	return volatile.LoadUint32(&o.BOOT_FLAGS1_R2.Reg) & 0xffffff
}

// OTP_DATA_RAW.DEFAULT_BOOT_VERSION0: Default boot version thermometer counter, bits 23:0 (RBIT-3)
func (o *OTP_DATA_RAW_Type) SetDEFAULT_BOOT_VERSION0(value uint32) {
	volatile.StoreUint32(&o.DEFAULT_BOOT_VERSION0.Reg, volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION0.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetDEFAULT_BOOT_VERSION0() uint32 {
	return volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION0.Reg) & 0xffffff
}

// OTP_DATA_RAW.DEFAULT_BOOT_VERSION0_R1: Redundant copy of DEFAULT_BOOT_VERSION0
func (o *OTP_DATA_RAW_Type) SetDEFAULT_BOOT_VERSION0_R1(value uint32) {
	volatile.StoreUint32(&o.DEFAULT_BOOT_VERSION0_R1.Reg, volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION0_R1.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetDEFAULT_BOOT_VERSION0_R1() uint32 {
	return volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION0_R1.Reg) & 0xffffff
}

// OTP_DATA_RAW.DEFAULT_BOOT_VERSION0_R2: Redundant copy of DEFAULT_BOOT_VERSION0
func (o *OTP_DATA_RAW_Type) SetDEFAULT_BOOT_VERSION0_R2(value uint32) {
	volatile.StoreUint32(&o.DEFAULT_BOOT_VERSION0_R2.Reg, volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION0_R2.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetDEFAULT_BOOT_VERSION0_R2() uint32 {
	return volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION0_R2.Reg) & 0xffffff
}

// OTP_DATA_RAW.DEFAULT_BOOT_VERSION1: Default boot version thermometer counter, bits 47:24 (RBIT-3)
func (o *OTP_DATA_RAW_Type) SetDEFAULT_BOOT_VERSION1(value uint32) {
	volatile.StoreUint32(&o.DEFAULT_BOOT_VERSION1.Reg, volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION1.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetDEFAULT_BOOT_VERSION1() uint32 {
	return volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION1.Reg) & 0xffffff
}

// OTP_DATA_RAW.DEFAULT_BOOT_VERSION1_R1: Redundant copy of DEFAULT_BOOT_VERSION1
func (o *OTP_DATA_RAW_Type) SetDEFAULT_BOOT_VERSION1_R1(value uint32) {
	volatile.StoreUint32(&o.DEFAULT_BOOT_VERSION1_R1.Reg, volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION1_R1.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetDEFAULT_BOOT_VERSION1_R1() uint32 {
	return volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION1_R1.Reg) & 0xffffff
}

// OTP_DATA_RAW.DEFAULT_BOOT_VERSION1_R2: Redundant copy of DEFAULT_BOOT_VERSION1
func (o *OTP_DATA_RAW_Type) SetDEFAULT_BOOT_VERSION1_R2(value uint32) {
	volatile.StoreUint32(&o.DEFAULT_BOOT_VERSION1_R2.Reg, volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION1_R2.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetDEFAULT_BOOT_VERSION1_R2() uint32 {
	return volatile.LoadUint32(&o.DEFAULT_BOOT_VERSION1_R2.Reg) & 0xffffff
}

// OTP_DATA_RAW.FLASH_DEVINFO: Stores information about external flash device(s). (ECC) Assumed to be valid if BOOT_FLAGS0_FLASH_DEVINFO_ENABLE is set.
func (o *OTP_DATA_RAW_Type) SetFLASH_DEVINFO_CS1_SIZE(value uint32) {
	volatile.StoreUint32(&o.FLASH_DEVINFO.Reg, volatile.LoadUint32(&o.FLASH_DEVINFO.Reg)&^(0xf000)|value<<12)
}
func (o *OTP_DATA_RAW_Type) GetFLASH_DEVINFO_CS1_SIZE() uint32 {
	return (volatile.LoadUint32(&o.FLASH_DEVINFO.Reg) & 0xf000) >> 12
}
func (o *OTP_DATA_RAW_Type) SetFLASH_DEVINFO_CS0_SIZE(value uint32) {
	volatile.StoreUint32(&o.FLASH_DEVINFO.Reg, volatile.LoadUint32(&o.FLASH_DEVINFO.Reg)&^(0xf00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetFLASH_DEVINFO_CS0_SIZE() uint32 {
	return (volatile.LoadUint32(&o.FLASH_DEVINFO.Reg) & 0xf00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetFLASH_DEVINFO_D8H_ERASE_SUPPORTED(value uint32) {
	volatile.StoreUint32(&o.FLASH_DEVINFO.Reg, volatile.LoadUint32(&o.FLASH_DEVINFO.Reg)&^(0x80)|value<<7)
}
func (o *OTP_DATA_RAW_Type) GetFLASH_DEVINFO_D8H_ERASE_SUPPORTED() uint32 {
	return (volatile.LoadUint32(&o.FLASH_DEVINFO.Reg) & 0x80) >> 7
}
func (o *OTP_DATA_RAW_Type) SetFLASH_DEVINFO_CS1_GPIO(value uint32) {
	volatile.StoreUint32(&o.FLASH_DEVINFO.Reg, volatile.LoadUint32(&o.FLASH_DEVINFO.Reg)&^(0x3f)|value)
}
func (o *OTP_DATA_RAW_Type) GetFLASH_DEVINFO_CS1_GPIO() uint32 {
	return volatile.LoadUint32(&o.FLASH_DEVINFO.Reg) & 0x3f
}

// OTP_DATA_RAW.FLASH_PARTITION_SLOT_SIZE: Gap between partition table slot 0 and slot 1 at the start of flash (the default size is 4096 bytes) (ECC) Enabled by the OVERRIDE_FLASH_PARTITION_SLOT_SIZE bit in BOOT_FLAGS, the size is 4096 * (value + 1)
func (o *OTP_DATA_RAW_Type) SetFLASH_PARTITION_SLOT_SIZE(value uint32) {
	volatile.StoreUint32(&o.FLASH_PARTITION_SLOT_SIZE.Reg, volatile.LoadUint32(&o.FLASH_PARTITION_SLOT_SIZE.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetFLASH_PARTITION_SLOT_SIZE() uint32 {
	return volatile.LoadUint32(&o.FLASH_PARTITION_SLOT_SIZE.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTSEL_LED_CFG: Pin configuration for LED status, used by USB bootloader. (ECC) Must be valid if BOOT_FLAGS0_ENABLE_BOOTSEL_LED is set.
func (o *OTP_DATA_RAW_Type) SetBOOTSEL_LED_CFG_ACTIVELOW(value uint32) {
	volatile.StoreUint32(&o.BOOTSEL_LED_CFG.Reg, volatile.LoadUint32(&o.BOOTSEL_LED_CFG.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetBOOTSEL_LED_CFG_ACTIVELOW() uint32 {
	return (volatile.LoadUint32(&o.BOOTSEL_LED_CFG.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_RAW_Type) SetBOOTSEL_LED_CFG_PIN(value uint32) {
	volatile.StoreUint32(&o.BOOTSEL_LED_CFG.Reg, volatile.LoadUint32(&o.BOOTSEL_LED_CFG.Reg)&^(0x3f)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTSEL_LED_CFG_PIN() uint32 {
	return volatile.LoadUint32(&o.BOOTSEL_LED_CFG.Reg) & 0x3f
}

// OTP_DATA_RAW.BOOTSEL_PLL_CFG: Optional PLL configuration for BOOTSEL mode. (ECC) This should be configured to produce an exact 48 MHz based on the crystal oscillator frequency. User mode software may also use this value to calculate the expected crystal frequency based on an assumed 48 MHz PLL output. If no configuration is given, the crystal is assumed to be 12 MHz. The PLL frequency can be calculated as: PLL out = (XOSC frequency / (REFDIV+1)) x FBDIV / (POSTDIV1 x POSTDIV2) Conversely the crystal frequency can be calculated as: XOSC frequency = 48 MHz x (REFDIV+1) x (POSTDIV1 x POSTDIV2) / FBDIV (Note the +1 on REFDIV is because the value stored in this OTP location is the actual divisor value minus one.) Used if and only if ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG is set in BOOT_FLAGS0. That bit should be set only after this row and BOOTSEL_XOSC_CFG are both correctly programmed.
func (o *OTP_DATA_RAW_Type) SetBOOTSEL_PLL_CFG_REFDIV(value uint32) {
	volatile.StoreUint32(&o.BOOTSEL_PLL_CFG.Reg, volatile.LoadUint32(&o.BOOTSEL_PLL_CFG.Reg)&^(0x8000)|value<<15)
}
func (o *OTP_DATA_RAW_Type) GetBOOTSEL_PLL_CFG_REFDIV() uint32 {
	return (volatile.LoadUint32(&o.BOOTSEL_PLL_CFG.Reg) & 0x8000) >> 15
}
func (o *OTP_DATA_RAW_Type) SetBOOTSEL_PLL_CFG_POSTDIV2(value uint32) {
	volatile.StoreUint32(&o.BOOTSEL_PLL_CFG.Reg, volatile.LoadUint32(&o.BOOTSEL_PLL_CFG.Reg)&^(0x7000)|value<<12)
}
func (o *OTP_DATA_RAW_Type) GetBOOTSEL_PLL_CFG_POSTDIV2() uint32 {
	return (volatile.LoadUint32(&o.BOOTSEL_PLL_CFG.Reg) & 0x7000) >> 12
}
func (o *OTP_DATA_RAW_Type) SetBOOTSEL_PLL_CFG_POSTDIV1(value uint32) {
	volatile.StoreUint32(&o.BOOTSEL_PLL_CFG.Reg, volatile.LoadUint32(&o.BOOTSEL_PLL_CFG.Reg)&^(0xe00)|value<<9)
}
func (o *OTP_DATA_RAW_Type) GetBOOTSEL_PLL_CFG_POSTDIV1() uint32 {
	return (volatile.LoadUint32(&o.BOOTSEL_PLL_CFG.Reg) & 0xe00) >> 9
}
func (o *OTP_DATA_RAW_Type) SetBOOTSEL_PLL_CFG_FBDIV(value uint32) {
	volatile.StoreUint32(&o.BOOTSEL_PLL_CFG.Reg, volatile.LoadUint32(&o.BOOTSEL_PLL_CFG.Reg)&^(0x1ff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTSEL_PLL_CFG_FBDIV() uint32 {
	return volatile.LoadUint32(&o.BOOTSEL_PLL_CFG.Reg) & 0x1ff
}

// OTP_DATA_RAW.BOOTSEL_XOSC_CFG: Non-default crystal oscillator configuration for the USB bootloader. (ECC) These values may also be used by user code configuring the crystal oscillator. Used if and only if ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG is set in BOOT_FLAGS0. That bit should be set only after this row and BOOTSEL_PLL_CFG are both correctly programmed.
func (o *OTP_DATA_RAW_Type) SetBOOTSEL_XOSC_CFG_RANGE(value uint32) {
	volatile.StoreUint32(&o.BOOTSEL_XOSC_CFG.Reg, volatile.LoadUint32(&o.BOOTSEL_XOSC_CFG.Reg)&^(0xc000)|value<<14)
}
func (o *OTP_DATA_RAW_Type) GetBOOTSEL_XOSC_CFG_RANGE() uint32 {
	return (volatile.LoadUint32(&o.BOOTSEL_XOSC_CFG.Reg) & 0xc000) >> 14
}
func (o *OTP_DATA_RAW_Type) SetBOOTSEL_XOSC_CFG_STARTUP(value uint32) {
	volatile.StoreUint32(&o.BOOTSEL_XOSC_CFG.Reg, volatile.LoadUint32(&o.BOOTSEL_XOSC_CFG.Reg)&^(0x3fff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTSEL_XOSC_CFG_STARTUP() uint32 {
	return volatile.LoadUint32(&o.BOOTSEL_XOSC_CFG.Reg) & 0x3fff
}

// OTP_DATA_RAW.USB_BOOT_FLAGS: USB boot specific feature flags (RBIT-3)
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_DP_DM_SWAP(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x800000)|value<<23)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_DP_DM_SWAP() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x800000) >> 23
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WHITE_LABEL_ADDR_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x400000)|value<<22)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WHITE_LABEL_ADDR_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x400000) >> 22
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_INFO_UF2_TXT_BOARD_ID_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x8000)|value<<15)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_INFO_UF2_TXT_BOARD_ID_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x8000) >> 15
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_INFO_UF2_TXT_MODEL_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x4000)|value<<14)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_INFO_UF2_TXT_MODEL_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x4000) >> 14
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_NAME_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x2000)|value<<13)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_NAME_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x2000) >> 13
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_URL_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x1000)|value<<12)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_URL_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x1000) >> 12
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_SCSI_INQUIRY_VERSION_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x800)|value<<11)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_SCSI_INQUIRY_VERSION_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x800) >> 11
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_SCSI_INQUIRY_PRODUCT_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x400)|value<<10)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_SCSI_INQUIRY_PRODUCT_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x400) >> 10
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_SCSI_INQUIRY_VENDOR_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x200)|value<<9)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_SCSI_INQUIRY_VENDOR_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x200) >> 9
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_VOLUME_LABEL_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_VOLUME_LABEL_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x80)|value<<7)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x80) >> 7
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_USB_DEVICE_PRODUCT_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x20)|value<<5)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_USB_DEVICE_PRODUCT_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x20) >> 5
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_USB_DEVICE_MANUFACTURER_STRDEF_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x10)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_USB_DEVICE_MANUFACTURER_STRDEF_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x10) >> 4
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_USB_DEVICE_LANG_ID_VALUE_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x8)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_USB_DEVICE_LANG_ID_VALUE_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x8) >> 3
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_VALUE_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x4)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_VALUE_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x4) >> 2
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_USB_DEVICE_PID_VALUE_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x2)|value<<1)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_USB_DEVICE_PID_VALUE_VALID() uint32 {
	return (volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x2) >> 1
}
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_WL_USB_DEVICE_VID_VALUE_VALID(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_WL_USB_DEVICE_VID_VALUE_VALID() uint32 {
	return volatile.LoadUint32(&o.USB_BOOT_FLAGS.Reg) & 0x1
}

// OTP_DATA_RAW.USB_BOOT_FLAGS_R1: Redundant copy of USB_BOOT_FLAGS
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_R1(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS_R1.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS_R1.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_R1() uint32 {
	return volatile.LoadUint32(&o.USB_BOOT_FLAGS_R1.Reg) & 0xffffff
}

// OTP_DATA_RAW.USB_BOOT_FLAGS_R2: Redundant copy of USB_BOOT_FLAGS
func (o *OTP_DATA_RAW_Type) SetUSB_BOOT_FLAGS_R2(value uint32) {
	volatile.StoreUint32(&o.USB_BOOT_FLAGS_R2.Reg, volatile.LoadUint32(&o.USB_BOOT_FLAGS_R2.Reg)&^(0xffffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetUSB_BOOT_FLAGS_R2() uint32 {
	return volatile.LoadUint32(&o.USB_BOOT_FLAGS_R2.Reg) & 0xffffff
}

// OTP_DATA_RAW.USB_WHITE_LABEL_ADDR: Row index of the USB_WHITE_LABEL structure within OTP (ECC) The table has 16 rows, each of which are also ECC and marked valid by the corresponding valid bit in USB_BOOT_FLAGS (ECC). The entries are either _VALUEs where the 16 bit value is used as is, or _STRDEFs which acts as a pointers to a string value. The value stored in a _STRDEF is two separate bytes: The low seven bits of the first (LSB) byte indicates the number of characters in the string, and the top bit of the first (LSB) byte if set to indicate that each character in the string is two bytes (Unicode) versus one byte if unset. The second (MSB) byte represents the location of the string data, and is encoded as the number of rows from this USB_WHITE_LABEL_ADDR; i.e. the row of the start of the string is USB_WHITE_LABEL_ADDR value + msb_byte. In each case, the corresponding valid bit enables replacing the default value for the corresponding item provided by the boot rom. Note that Unicode _STRDEFs are only supported for USB_DEVICE_PRODUCT_STRDEF, USB_DEVICE_SERIAL_NUMBER_STRDEF and USB_DEVICE_MANUFACTURER_STRDEF. Unicode values will be ignored if specified for other fields, and non-unicode values for these three items will be converted to Unicode characters by setting the upper 8 bits to zero. Note that if the USB_WHITE_LABEL structure or the corresponding strings are not readable by BOOTSEL mode based on OTP permissions, or if alignment requirements are not met, then the corresponding default values are used. The index values indicate where each field is located (row USB_WHITE_LABEL_ADDR value + index):
func (o *OTP_DATA_RAW_Type) SetUSB_WHITE_LABEL_ADDR(value uint32) {
	volatile.StoreUint32(&o.USB_WHITE_LABEL_ADDR.Reg, volatile.LoadUint32(&o.USB_WHITE_LABEL_ADDR.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetUSB_WHITE_LABEL_ADDR() uint32 {
	return volatile.LoadUint32(&o.USB_WHITE_LABEL_ADDR.Reg) & 0xffff
}

// OTP_DATA_RAW.OTPBOOT_SRC: OTP start row for the OTP boot image. (ECC) If OTP boot is enabled, the bootrom will load from this location into SRAM and then directly enter the loaded image. Note that the image must be signed if SECURE_BOOT_ENABLE is set. The image itself is assumed to be ECC-protected. This must be an even number. Equivalently, the OTP boot image must start at a word-aligned location in the ECC read data address window.
func (o *OTP_DATA_RAW_Type) SetOTPBOOT_SRC(value uint32) {
	volatile.StoreUint32(&o.OTPBOOT_SRC.Reg, volatile.LoadUint32(&o.OTPBOOT_SRC.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetOTPBOOT_SRC() uint32 {
	return volatile.LoadUint32(&o.OTPBOOT_SRC.Reg) & 0xffff
}

// OTP_DATA_RAW.OTPBOOT_LEN: Length in rows of the OTP boot image. (ECC) OTPBOOT_LEN must be even. The total image size must be a multiple of 4 bytes (32 bits).
func (o *OTP_DATA_RAW_Type) SetOTPBOOT_LEN(value uint32) {
	volatile.StoreUint32(&o.OTPBOOT_LEN.Reg, volatile.LoadUint32(&o.OTPBOOT_LEN.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetOTPBOOT_LEN() uint32 {
	return volatile.LoadUint32(&o.OTPBOOT_LEN.Reg) & 0xffff
}

// OTP_DATA_RAW.OTPBOOT_DST0: Bits 15:0 of the OTP boot image load destination (and entry point). (ECC) This must be a location in main SRAM (main SRAM is addresses 0x20000000 through 0x20082000) and must be word-aligned.
func (o *OTP_DATA_RAW_Type) SetOTPBOOT_DST0(value uint32) {
	volatile.StoreUint32(&o.OTPBOOT_DST0.Reg, volatile.LoadUint32(&o.OTPBOOT_DST0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetOTPBOOT_DST0() uint32 {
	return volatile.LoadUint32(&o.OTPBOOT_DST0.Reg) & 0xffff
}

// OTP_DATA_RAW.OTPBOOT_DST1: Bits 31:16 of the OTP boot image load destination (and entry point). (ECC) This must be a location in main SRAM (main SRAM is addresses 0x20000000 through 0x20082000) and must be word-aligned.
func (o *OTP_DATA_RAW_Type) SetOTPBOOT_DST1(value uint32) {
	volatile.StoreUint32(&o.OTPBOOT_DST1.Reg, volatile.LoadUint32(&o.OTPBOOT_DST1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetOTPBOOT_DST1() uint32 {
	return volatile.LoadUint32(&o.OTPBOOT_DST1.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_0: Bits 15:0 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_0(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_0.Reg, volatile.LoadUint32(&o.BOOTKEY0_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_0() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_0.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_1: Bits 31:16 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_1(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_1.Reg, volatile.LoadUint32(&o.BOOTKEY0_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_1() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_1.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_2: Bits 47:32 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_2(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_2.Reg, volatile.LoadUint32(&o.BOOTKEY0_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_2() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_2.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_3: Bits 63:48 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_3(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_3.Reg, volatile.LoadUint32(&o.BOOTKEY0_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_3() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_3.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_4: Bits 79:64 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_4(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_4.Reg, volatile.LoadUint32(&o.BOOTKEY0_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_4() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_4.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_5: Bits 95:80 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_5(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_5.Reg, volatile.LoadUint32(&o.BOOTKEY0_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_5() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_5.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_6: Bits 111:96 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_6(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_6.Reg, volatile.LoadUint32(&o.BOOTKEY0_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_6() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_6.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_7: Bits 127:112 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_7(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_7.Reg, volatile.LoadUint32(&o.BOOTKEY0_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_7() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_7.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_8: Bits 143:128 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_8(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_8.Reg, volatile.LoadUint32(&o.BOOTKEY0_8.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_8() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_8.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_9: Bits 159:144 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_9(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_9.Reg, volatile.LoadUint32(&o.BOOTKEY0_9.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_9() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_9.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_10: Bits 175:160 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_10(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_10.Reg, volatile.LoadUint32(&o.BOOTKEY0_10.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_10() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_10.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_11: Bits 191:176 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_11(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_11.Reg, volatile.LoadUint32(&o.BOOTKEY0_11.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_11() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_11.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_12: Bits 207:192 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_12(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_12.Reg, volatile.LoadUint32(&o.BOOTKEY0_12.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_12() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_12.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_13: Bits 223:208 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_13(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_13.Reg, volatile.LoadUint32(&o.BOOTKEY0_13.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_13() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_13.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_14: Bits 239:224 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_14(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_14.Reg, volatile.LoadUint32(&o.BOOTKEY0_14.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_14() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_14.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY0_15: Bits 255:240 of SHA-256 hash of boot key 0 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY0_15(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY0_15.Reg, volatile.LoadUint32(&o.BOOTKEY0_15.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY0_15() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY0_15.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_0: Bits 15:0 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_0(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_0.Reg, volatile.LoadUint32(&o.BOOTKEY1_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_0() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_0.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_1: Bits 31:16 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_1(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_1.Reg, volatile.LoadUint32(&o.BOOTKEY1_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_1() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_1.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_2: Bits 47:32 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_2(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_2.Reg, volatile.LoadUint32(&o.BOOTKEY1_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_2() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_2.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_3: Bits 63:48 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_3(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_3.Reg, volatile.LoadUint32(&o.BOOTKEY1_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_3() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_3.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_4: Bits 79:64 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_4(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_4.Reg, volatile.LoadUint32(&o.BOOTKEY1_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_4() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_4.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_5: Bits 95:80 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_5(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_5.Reg, volatile.LoadUint32(&o.BOOTKEY1_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_5() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_5.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_6: Bits 111:96 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_6(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_6.Reg, volatile.LoadUint32(&o.BOOTKEY1_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_6() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_6.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_7: Bits 127:112 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_7(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_7.Reg, volatile.LoadUint32(&o.BOOTKEY1_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_7() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_7.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_8: Bits 143:128 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_8(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_8.Reg, volatile.LoadUint32(&o.BOOTKEY1_8.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_8() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_8.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_9: Bits 159:144 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_9(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_9.Reg, volatile.LoadUint32(&o.BOOTKEY1_9.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_9() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_9.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_10: Bits 175:160 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_10(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_10.Reg, volatile.LoadUint32(&o.BOOTKEY1_10.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_10() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_10.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_11: Bits 191:176 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_11(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_11.Reg, volatile.LoadUint32(&o.BOOTKEY1_11.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_11() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_11.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_12: Bits 207:192 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_12(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_12.Reg, volatile.LoadUint32(&o.BOOTKEY1_12.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_12() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_12.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_13: Bits 223:208 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_13(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_13.Reg, volatile.LoadUint32(&o.BOOTKEY1_13.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_13() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_13.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_14: Bits 239:224 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_14(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_14.Reg, volatile.LoadUint32(&o.BOOTKEY1_14.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_14() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_14.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY1_15: Bits 255:240 of SHA-256 hash of boot key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY1_15(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY1_15.Reg, volatile.LoadUint32(&o.BOOTKEY1_15.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY1_15() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY1_15.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_0: Bits 15:0 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_0(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_0.Reg, volatile.LoadUint32(&o.BOOTKEY2_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_0() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_0.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_1: Bits 31:16 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_1(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_1.Reg, volatile.LoadUint32(&o.BOOTKEY2_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_1() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_1.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_2: Bits 47:32 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_2(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_2.Reg, volatile.LoadUint32(&o.BOOTKEY2_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_2() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_2.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_3: Bits 63:48 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_3(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_3.Reg, volatile.LoadUint32(&o.BOOTKEY2_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_3() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_3.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_4: Bits 79:64 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_4(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_4.Reg, volatile.LoadUint32(&o.BOOTKEY2_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_4() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_4.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_5: Bits 95:80 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_5(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_5.Reg, volatile.LoadUint32(&o.BOOTKEY2_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_5() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_5.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_6: Bits 111:96 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_6(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_6.Reg, volatile.LoadUint32(&o.BOOTKEY2_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_6() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_6.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_7: Bits 127:112 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_7(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_7.Reg, volatile.LoadUint32(&o.BOOTKEY2_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_7() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_7.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_8: Bits 143:128 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_8(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_8.Reg, volatile.LoadUint32(&o.BOOTKEY2_8.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_8() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_8.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_9: Bits 159:144 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_9(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_9.Reg, volatile.LoadUint32(&o.BOOTKEY2_9.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_9() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_9.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_10: Bits 175:160 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_10(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_10.Reg, volatile.LoadUint32(&o.BOOTKEY2_10.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_10() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_10.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_11: Bits 191:176 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_11(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_11.Reg, volatile.LoadUint32(&o.BOOTKEY2_11.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_11() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_11.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_12: Bits 207:192 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_12(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_12.Reg, volatile.LoadUint32(&o.BOOTKEY2_12.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_12() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_12.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_13: Bits 223:208 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_13(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_13.Reg, volatile.LoadUint32(&o.BOOTKEY2_13.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_13() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_13.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_14: Bits 239:224 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_14(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_14.Reg, volatile.LoadUint32(&o.BOOTKEY2_14.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_14() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_14.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY2_15: Bits 255:240 of SHA-256 hash of boot key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY2_15(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY2_15.Reg, volatile.LoadUint32(&o.BOOTKEY2_15.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY2_15() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY2_15.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_0: Bits 15:0 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_0(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_0.Reg, volatile.LoadUint32(&o.BOOTKEY3_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_0() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_0.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_1: Bits 31:16 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_1(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_1.Reg, volatile.LoadUint32(&o.BOOTKEY3_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_1() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_1.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_2: Bits 47:32 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_2(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_2.Reg, volatile.LoadUint32(&o.BOOTKEY3_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_2() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_2.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_3: Bits 63:48 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_3(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_3.Reg, volatile.LoadUint32(&o.BOOTKEY3_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_3() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_3.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_4: Bits 79:64 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_4(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_4.Reg, volatile.LoadUint32(&o.BOOTKEY3_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_4() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_4.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_5: Bits 95:80 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_5(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_5.Reg, volatile.LoadUint32(&o.BOOTKEY3_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_5() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_5.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_6: Bits 111:96 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_6(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_6.Reg, volatile.LoadUint32(&o.BOOTKEY3_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_6() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_6.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_7: Bits 127:112 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_7(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_7.Reg, volatile.LoadUint32(&o.BOOTKEY3_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_7() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_7.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_8: Bits 143:128 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_8(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_8.Reg, volatile.LoadUint32(&o.BOOTKEY3_8.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_8() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_8.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_9: Bits 159:144 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_9(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_9.Reg, volatile.LoadUint32(&o.BOOTKEY3_9.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_9() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_9.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_10: Bits 175:160 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_10(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_10.Reg, volatile.LoadUint32(&o.BOOTKEY3_10.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_10() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_10.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_11: Bits 191:176 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_11(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_11.Reg, volatile.LoadUint32(&o.BOOTKEY3_11.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_11() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_11.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_12: Bits 207:192 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_12(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_12.Reg, volatile.LoadUint32(&o.BOOTKEY3_12.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_12() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_12.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_13: Bits 223:208 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_13(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_13.Reg, volatile.LoadUint32(&o.BOOTKEY3_13.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_13() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_13.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_14: Bits 239:224 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_14(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_14.Reg, volatile.LoadUint32(&o.BOOTKEY3_14.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_14() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_14.Reg) & 0xffff
}

// OTP_DATA_RAW.BOOTKEY3_15: Bits 255:240 of SHA-256 hash of boot key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetBOOTKEY3_15(value uint32) {
	volatile.StoreUint32(&o.BOOTKEY3_15.Reg, volatile.LoadUint32(&o.BOOTKEY3_15.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetBOOTKEY3_15() uint32 {
	return volatile.LoadUint32(&o.BOOTKEY3_15.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY1_0: Bits 15:0 of OTP access key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY1_0(value uint32) {
	volatile.StoreUint32(&o.KEY1_0.Reg, volatile.LoadUint32(&o.KEY1_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_0() uint32 {
	return volatile.LoadUint32(&o.KEY1_0.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY1_1: Bits 31:16 of OTP access key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY1_1(value uint32) {
	volatile.StoreUint32(&o.KEY1_1.Reg, volatile.LoadUint32(&o.KEY1_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_1() uint32 {
	return volatile.LoadUint32(&o.KEY1_1.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY1_2: Bits 47:32 of OTP access key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY1_2(value uint32) {
	volatile.StoreUint32(&o.KEY1_2.Reg, volatile.LoadUint32(&o.KEY1_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_2() uint32 {
	return volatile.LoadUint32(&o.KEY1_2.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY1_3: Bits 63:48 of OTP access key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY1_3(value uint32) {
	volatile.StoreUint32(&o.KEY1_3.Reg, volatile.LoadUint32(&o.KEY1_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_3() uint32 {
	return volatile.LoadUint32(&o.KEY1_3.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY1_4: Bits 79:64 of OTP access key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY1_4(value uint32) {
	volatile.StoreUint32(&o.KEY1_4.Reg, volatile.LoadUint32(&o.KEY1_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_4() uint32 {
	return volatile.LoadUint32(&o.KEY1_4.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY1_5: Bits 95:80 of OTP access key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY1_5(value uint32) {
	volatile.StoreUint32(&o.KEY1_5.Reg, volatile.LoadUint32(&o.KEY1_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_5() uint32 {
	return volatile.LoadUint32(&o.KEY1_5.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY1_6: Bits 111:96 of OTP access key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY1_6(value uint32) {
	volatile.StoreUint32(&o.KEY1_6.Reg, volatile.LoadUint32(&o.KEY1_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_6() uint32 {
	return volatile.LoadUint32(&o.KEY1_6.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY1_7: Bits 127:112 of OTP access key 1 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY1_7(value uint32) {
	volatile.StoreUint32(&o.KEY1_7.Reg, volatile.LoadUint32(&o.KEY1_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_7() uint32 {
	return volatile.LoadUint32(&o.KEY1_7.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY2_0: Bits 15:0 of OTP access key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY2_0(value uint32) {
	volatile.StoreUint32(&o.KEY2_0.Reg, volatile.LoadUint32(&o.KEY2_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_0() uint32 {
	return volatile.LoadUint32(&o.KEY2_0.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY2_1: Bits 31:16 of OTP access key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY2_1(value uint32) {
	volatile.StoreUint32(&o.KEY2_1.Reg, volatile.LoadUint32(&o.KEY2_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_1() uint32 {
	return volatile.LoadUint32(&o.KEY2_1.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY2_2: Bits 47:32 of OTP access key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY2_2(value uint32) {
	volatile.StoreUint32(&o.KEY2_2.Reg, volatile.LoadUint32(&o.KEY2_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_2() uint32 {
	return volatile.LoadUint32(&o.KEY2_2.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY2_3: Bits 63:48 of OTP access key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY2_3(value uint32) {
	volatile.StoreUint32(&o.KEY2_3.Reg, volatile.LoadUint32(&o.KEY2_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_3() uint32 {
	return volatile.LoadUint32(&o.KEY2_3.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY2_4: Bits 79:64 of OTP access key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY2_4(value uint32) {
	volatile.StoreUint32(&o.KEY2_4.Reg, volatile.LoadUint32(&o.KEY2_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_4() uint32 {
	return volatile.LoadUint32(&o.KEY2_4.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY2_5: Bits 95:80 of OTP access key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY2_5(value uint32) {
	volatile.StoreUint32(&o.KEY2_5.Reg, volatile.LoadUint32(&o.KEY2_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_5() uint32 {
	return volatile.LoadUint32(&o.KEY2_5.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY2_6: Bits 111:96 of OTP access key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY2_6(value uint32) {
	volatile.StoreUint32(&o.KEY2_6.Reg, volatile.LoadUint32(&o.KEY2_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_6() uint32 {
	return volatile.LoadUint32(&o.KEY2_6.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY2_7: Bits 127:112 of OTP access key 2 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY2_7(value uint32) {
	volatile.StoreUint32(&o.KEY2_7.Reg, volatile.LoadUint32(&o.KEY2_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_7() uint32 {
	return volatile.LoadUint32(&o.KEY2_7.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY3_0: Bits 15:0 of OTP access key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY3_0(value uint32) {
	volatile.StoreUint32(&o.KEY3_0.Reg, volatile.LoadUint32(&o.KEY3_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_0() uint32 {
	return volatile.LoadUint32(&o.KEY3_0.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY3_1: Bits 31:16 of OTP access key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY3_1(value uint32) {
	volatile.StoreUint32(&o.KEY3_1.Reg, volatile.LoadUint32(&o.KEY3_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_1() uint32 {
	return volatile.LoadUint32(&o.KEY3_1.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY3_2: Bits 47:32 of OTP access key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY3_2(value uint32) {
	volatile.StoreUint32(&o.KEY3_2.Reg, volatile.LoadUint32(&o.KEY3_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_2() uint32 {
	return volatile.LoadUint32(&o.KEY3_2.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY3_3: Bits 63:48 of OTP access key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY3_3(value uint32) {
	volatile.StoreUint32(&o.KEY3_3.Reg, volatile.LoadUint32(&o.KEY3_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_3() uint32 {
	return volatile.LoadUint32(&o.KEY3_3.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY3_4: Bits 79:64 of OTP access key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY3_4(value uint32) {
	volatile.StoreUint32(&o.KEY3_4.Reg, volatile.LoadUint32(&o.KEY3_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_4() uint32 {
	return volatile.LoadUint32(&o.KEY3_4.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY3_5: Bits 95:80 of OTP access key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY3_5(value uint32) {
	volatile.StoreUint32(&o.KEY3_5.Reg, volatile.LoadUint32(&o.KEY3_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_5() uint32 {
	return volatile.LoadUint32(&o.KEY3_5.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY3_6: Bits 111:96 of OTP access key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY3_6(value uint32) {
	volatile.StoreUint32(&o.KEY3_6.Reg, volatile.LoadUint32(&o.KEY3_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_6() uint32 {
	return volatile.LoadUint32(&o.KEY3_6.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY3_7: Bits 127:112 of OTP access key 3 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY3_7(value uint32) {
	volatile.StoreUint32(&o.KEY3_7.Reg, volatile.LoadUint32(&o.KEY3_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_7() uint32 {
	return volatile.LoadUint32(&o.KEY3_7.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY4_0: Bits 15:0 of OTP access key 4 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY4_0(value uint32) {
	volatile.StoreUint32(&o.KEY4_0.Reg, volatile.LoadUint32(&o.KEY4_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_0() uint32 {
	return volatile.LoadUint32(&o.KEY4_0.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY4_1: Bits 31:16 of OTP access key 4 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY4_1(value uint32) {
	volatile.StoreUint32(&o.KEY4_1.Reg, volatile.LoadUint32(&o.KEY4_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_1() uint32 {
	return volatile.LoadUint32(&o.KEY4_1.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY4_2: Bits 47:32 of OTP access key 4 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY4_2(value uint32) {
	volatile.StoreUint32(&o.KEY4_2.Reg, volatile.LoadUint32(&o.KEY4_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_2() uint32 {
	return volatile.LoadUint32(&o.KEY4_2.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY4_3: Bits 63:48 of OTP access key 4 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY4_3(value uint32) {
	volatile.StoreUint32(&o.KEY4_3.Reg, volatile.LoadUint32(&o.KEY4_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_3() uint32 {
	return volatile.LoadUint32(&o.KEY4_3.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY4_4: Bits 79:64 of OTP access key 4 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY4_4(value uint32) {
	volatile.StoreUint32(&o.KEY4_4.Reg, volatile.LoadUint32(&o.KEY4_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_4() uint32 {
	return volatile.LoadUint32(&o.KEY4_4.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY4_5: Bits 95:80 of OTP access key 4 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY4_5(value uint32) {
	volatile.StoreUint32(&o.KEY4_5.Reg, volatile.LoadUint32(&o.KEY4_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_5() uint32 {
	return volatile.LoadUint32(&o.KEY4_5.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY4_6: Bits 111:96 of OTP access key 4 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY4_6(value uint32) {
	volatile.StoreUint32(&o.KEY4_6.Reg, volatile.LoadUint32(&o.KEY4_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_6() uint32 {
	return volatile.LoadUint32(&o.KEY4_6.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY4_7: Bits 127:112 of OTP access key 4 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY4_7(value uint32) {
	volatile.StoreUint32(&o.KEY4_7.Reg, volatile.LoadUint32(&o.KEY4_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_7() uint32 {
	return volatile.LoadUint32(&o.KEY4_7.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY5_0: Bits 15:0 of OTP access key 5 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY5_0(value uint32) {
	volatile.StoreUint32(&o.KEY5_0.Reg, volatile.LoadUint32(&o.KEY5_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_0() uint32 {
	return volatile.LoadUint32(&o.KEY5_0.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY5_1: Bits 31:16 of OTP access key 5 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY5_1(value uint32) {
	volatile.StoreUint32(&o.KEY5_1.Reg, volatile.LoadUint32(&o.KEY5_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_1() uint32 {
	return volatile.LoadUint32(&o.KEY5_1.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY5_2: Bits 47:32 of OTP access key 5 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY5_2(value uint32) {
	volatile.StoreUint32(&o.KEY5_2.Reg, volatile.LoadUint32(&o.KEY5_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_2() uint32 {
	return volatile.LoadUint32(&o.KEY5_2.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY5_3: Bits 63:48 of OTP access key 5 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY5_3(value uint32) {
	volatile.StoreUint32(&o.KEY5_3.Reg, volatile.LoadUint32(&o.KEY5_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_3() uint32 {
	return volatile.LoadUint32(&o.KEY5_3.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY5_4: Bits 79:64 of OTP access key 5 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY5_4(value uint32) {
	volatile.StoreUint32(&o.KEY5_4.Reg, volatile.LoadUint32(&o.KEY5_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_4() uint32 {
	return volatile.LoadUint32(&o.KEY5_4.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY5_5: Bits 95:80 of OTP access key 5 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY5_5(value uint32) {
	volatile.StoreUint32(&o.KEY5_5.Reg, volatile.LoadUint32(&o.KEY5_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_5() uint32 {
	return volatile.LoadUint32(&o.KEY5_5.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY5_6: Bits 111:96 of OTP access key 5 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY5_6(value uint32) {
	volatile.StoreUint32(&o.KEY5_6.Reg, volatile.LoadUint32(&o.KEY5_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_6() uint32 {
	return volatile.LoadUint32(&o.KEY5_6.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY5_7: Bits 127:112 of OTP access key 5 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY5_7(value uint32) {
	volatile.StoreUint32(&o.KEY5_7.Reg, volatile.LoadUint32(&o.KEY5_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_7() uint32 {
	return volatile.LoadUint32(&o.KEY5_7.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY6_0: Bits 15:0 of OTP access key 6 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY6_0(value uint32) {
	volatile.StoreUint32(&o.KEY6_0.Reg, volatile.LoadUint32(&o.KEY6_0.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_0() uint32 {
	return volatile.LoadUint32(&o.KEY6_0.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY6_1: Bits 31:16 of OTP access key 6 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY6_1(value uint32) {
	volatile.StoreUint32(&o.KEY6_1.Reg, volatile.LoadUint32(&o.KEY6_1.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_1() uint32 {
	return volatile.LoadUint32(&o.KEY6_1.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY6_2: Bits 47:32 of OTP access key 6 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY6_2(value uint32) {
	volatile.StoreUint32(&o.KEY6_2.Reg, volatile.LoadUint32(&o.KEY6_2.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_2() uint32 {
	return volatile.LoadUint32(&o.KEY6_2.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY6_3: Bits 63:48 of OTP access key 6 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY6_3(value uint32) {
	volatile.StoreUint32(&o.KEY6_3.Reg, volatile.LoadUint32(&o.KEY6_3.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_3() uint32 {
	return volatile.LoadUint32(&o.KEY6_3.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY6_4: Bits 79:64 of OTP access key 6 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY6_4(value uint32) {
	volatile.StoreUint32(&o.KEY6_4.Reg, volatile.LoadUint32(&o.KEY6_4.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_4() uint32 {
	return volatile.LoadUint32(&o.KEY6_4.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY6_5: Bits 95:80 of OTP access key 6 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY6_5(value uint32) {
	volatile.StoreUint32(&o.KEY6_5.Reg, volatile.LoadUint32(&o.KEY6_5.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_5() uint32 {
	return volatile.LoadUint32(&o.KEY6_5.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY6_6: Bits 111:96 of OTP access key 6 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY6_6(value uint32) {
	volatile.StoreUint32(&o.KEY6_6.Reg, volatile.LoadUint32(&o.KEY6_6.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_6() uint32 {
	return volatile.LoadUint32(&o.KEY6_6.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY6_7: Bits 127:112 of OTP access key 6 (ECC)
func (o *OTP_DATA_RAW_Type) SetKEY6_7(value uint32) {
	volatile.StoreUint32(&o.KEY6_7.Reg, volatile.LoadUint32(&o.KEY6_7.Reg)&^(0xffff)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_7() uint32 {
	return volatile.LoadUint32(&o.KEY6_7.Reg) & 0xffff
}

// OTP_DATA_RAW.KEY1_VALID: Valid flag for key 1. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
func (o *OTP_DATA_RAW_Type) SetKEY1_VALID_VALID_R2(value uint32) {
	volatile.StoreUint32(&o.KEY1_VALID.Reg, volatile.LoadUint32(&o.KEY1_VALID.Reg)&^(0x10000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_VALID_VALID_R2() uint32 {
	return (volatile.LoadUint32(&o.KEY1_VALID.Reg) & 0x10000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetKEY1_VALID_VALID_R1(value uint32) {
	volatile.StoreUint32(&o.KEY1_VALID.Reg, volatile.LoadUint32(&o.KEY1_VALID.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_VALID_VALID_R1() uint32 {
	return (volatile.LoadUint32(&o.KEY1_VALID.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_RAW_Type) SetKEY1_VALID_VALID(value uint32) {
	volatile.StoreUint32(&o.KEY1_VALID.Reg, volatile.LoadUint32(&o.KEY1_VALID.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY1_VALID_VALID() uint32 {
	return volatile.LoadUint32(&o.KEY1_VALID.Reg) & 0x1
}

// OTP_DATA_RAW.KEY2_VALID: Valid flag for key 2. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
func (o *OTP_DATA_RAW_Type) SetKEY2_VALID_VALID_R2(value uint32) {
	volatile.StoreUint32(&o.KEY2_VALID.Reg, volatile.LoadUint32(&o.KEY2_VALID.Reg)&^(0x10000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_VALID_VALID_R2() uint32 {
	return (volatile.LoadUint32(&o.KEY2_VALID.Reg) & 0x10000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetKEY2_VALID_VALID_R1(value uint32) {
	volatile.StoreUint32(&o.KEY2_VALID.Reg, volatile.LoadUint32(&o.KEY2_VALID.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_VALID_VALID_R1() uint32 {
	return (volatile.LoadUint32(&o.KEY2_VALID.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_RAW_Type) SetKEY2_VALID_VALID(value uint32) {
	volatile.StoreUint32(&o.KEY2_VALID.Reg, volatile.LoadUint32(&o.KEY2_VALID.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY2_VALID_VALID() uint32 {
	return volatile.LoadUint32(&o.KEY2_VALID.Reg) & 0x1
}

// OTP_DATA_RAW.KEY3_VALID: Valid flag for key 3. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
func (o *OTP_DATA_RAW_Type) SetKEY3_VALID_VALID_R2(value uint32) {
	volatile.StoreUint32(&o.KEY3_VALID.Reg, volatile.LoadUint32(&o.KEY3_VALID.Reg)&^(0x10000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_VALID_VALID_R2() uint32 {
	return (volatile.LoadUint32(&o.KEY3_VALID.Reg) & 0x10000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetKEY3_VALID_VALID_R1(value uint32) {
	volatile.StoreUint32(&o.KEY3_VALID.Reg, volatile.LoadUint32(&o.KEY3_VALID.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_VALID_VALID_R1() uint32 {
	return (volatile.LoadUint32(&o.KEY3_VALID.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_RAW_Type) SetKEY3_VALID_VALID(value uint32) {
	volatile.StoreUint32(&o.KEY3_VALID.Reg, volatile.LoadUint32(&o.KEY3_VALID.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY3_VALID_VALID() uint32 {
	return volatile.LoadUint32(&o.KEY3_VALID.Reg) & 0x1
}

// OTP_DATA_RAW.KEY4_VALID: Valid flag for key 4. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
func (o *OTP_DATA_RAW_Type) SetKEY4_VALID_VALID_R2(value uint32) {
	volatile.StoreUint32(&o.KEY4_VALID.Reg, volatile.LoadUint32(&o.KEY4_VALID.Reg)&^(0x10000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_VALID_VALID_R2() uint32 {
	return (volatile.LoadUint32(&o.KEY4_VALID.Reg) & 0x10000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetKEY4_VALID_VALID_R1(value uint32) {
	volatile.StoreUint32(&o.KEY4_VALID.Reg, volatile.LoadUint32(&o.KEY4_VALID.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_VALID_VALID_R1() uint32 {
	return (volatile.LoadUint32(&o.KEY4_VALID.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_RAW_Type) SetKEY4_VALID_VALID(value uint32) {
	volatile.StoreUint32(&o.KEY4_VALID.Reg, volatile.LoadUint32(&o.KEY4_VALID.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY4_VALID_VALID() uint32 {
	return volatile.LoadUint32(&o.KEY4_VALID.Reg) & 0x1
}

// OTP_DATA_RAW.KEY5_VALID: Valid flag for key 5. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
func (o *OTP_DATA_RAW_Type) SetKEY5_VALID_VALID_R2(value uint32) {
	volatile.StoreUint32(&o.KEY5_VALID.Reg, volatile.LoadUint32(&o.KEY5_VALID.Reg)&^(0x10000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_VALID_VALID_R2() uint32 {
	return (volatile.LoadUint32(&o.KEY5_VALID.Reg) & 0x10000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetKEY5_VALID_VALID_R1(value uint32) {
	volatile.StoreUint32(&o.KEY5_VALID.Reg, volatile.LoadUint32(&o.KEY5_VALID.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_VALID_VALID_R1() uint32 {
	return (volatile.LoadUint32(&o.KEY5_VALID.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_RAW_Type) SetKEY5_VALID_VALID(value uint32) {
	volatile.StoreUint32(&o.KEY5_VALID.Reg, volatile.LoadUint32(&o.KEY5_VALID.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY5_VALID_VALID() uint32 {
	return volatile.LoadUint32(&o.KEY5_VALID.Reg) & 0x1
}

// OTP_DATA_RAW.KEY6_VALID: Valid flag for key 6. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
func (o *OTP_DATA_RAW_Type) SetKEY6_VALID_VALID_R2(value uint32) {
	volatile.StoreUint32(&o.KEY6_VALID.Reg, volatile.LoadUint32(&o.KEY6_VALID.Reg)&^(0x10000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_VALID_VALID_R2() uint32 {
	return (volatile.LoadUint32(&o.KEY6_VALID.Reg) & 0x10000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetKEY6_VALID_VALID_R1(value uint32) {
	volatile.StoreUint32(&o.KEY6_VALID.Reg, volatile.LoadUint32(&o.KEY6_VALID.Reg)&^(0x100)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_VALID_VALID_R1() uint32 {
	return (volatile.LoadUint32(&o.KEY6_VALID.Reg) & 0x100) >> 8
}
func (o *OTP_DATA_RAW_Type) SetKEY6_VALID_VALID(value uint32) {
	volatile.StoreUint32(&o.KEY6_VALID.Reg, volatile.LoadUint32(&o.KEY6_VALID.Reg)&^(0x1)|value)
}
func (o *OTP_DATA_RAW_Type) GetKEY6_VALID_VALID() uint32 {
	return volatile.LoadUint32(&o.KEY6_VALID.Reg) & 0x1
}

// OTP_DATA_RAW.PAGE0_LOCK0: Lock configuration LSBs for page 0 (rows 0x0 through 0x3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK0.Reg, volatile.LoadUint32(&o.PAGE0_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE0_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK0.Reg, volatile.LoadUint32(&o.PAGE0_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE0_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK0.Reg, volatile.LoadUint32(&o.PAGE0_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE0_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK0.Reg, volatile.LoadUint32(&o.PAGE0_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE0_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK0.Reg, volatile.LoadUint32(&o.PAGE0_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE0_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE0_LOCK1: Lock configuration MSBs for page 0 (rows 0x0 through 0x3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK1.Reg, volatile.LoadUint32(&o.PAGE0_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE0_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK1.Reg, volatile.LoadUint32(&o.PAGE0_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE0_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK1.Reg, volatile.LoadUint32(&o.PAGE0_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE0_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK1.Reg, volatile.LoadUint32(&o.PAGE0_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE0_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE0_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE0_LOCK1.Reg, volatile.LoadUint32(&o.PAGE0_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE0_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE0_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE1_LOCK0: Lock configuration LSBs for page 1 (rows 0x40 through 0x7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK0.Reg, volatile.LoadUint32(&o.PAGE1_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE1_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK0.Reg, volatile.LoadUint32(&o.PAGE1_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE1_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK0.Reg, volatile.LoadUint32(&o.PAGE1_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE1_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK0.Reg, volatile.LoadUint32(&o.PAGE1_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE1_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK0.Reg, volatile.LoadUint32(&o.PAGE1_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE1_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE1_LOCK1: Lock configuration MSBs for page 1 (rows 0x40 through 0x7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK1.Reg, volatile.LoadUint32(&o.PAGE1_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE1_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK1.Reg, volatile.LoadUint32(&o.PAGE1_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE1_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK1.Reg, volatile.LoadUint32(&o.PAGE1_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE1_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK1.Reg, volatile.LoadUint32(&o.PAGE1_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE1_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE1_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE1_LOCK1.Reg, volatile.LoadUint32(&o.PAGE1_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE1_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE1_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE2_LOCK0: Lock configuration LSBs for page 2 (rows 0x80 through 0xbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK0.Reg, volatile.LoadUint32(&o.PAGE2_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE2_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK0.Reg, volatile.LoadUint32(&o.PAGE2_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE2_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK0.Reg, volatile.LoadUint32(&o.PAGE2_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE2_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK0.Reg, volatile.LoadUint32(&o.PAGE2_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE2_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK0.Reg, volatile.LoadUint32(&o.PAGE2_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE2_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE2_LOCK1: Lock configuration MSBs for page 2 (rows 0x80 through 0xbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK1.Reg, volatile.LoadUint32(&o.PAGE2_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE2_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK1.Reg, volatile.LoadUint32(&o.PAGE2_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE2_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK1.Reg, volatile.LoadUint32(&o.PAGE2_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE2_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK1.Reg, volatile.LoadUint32(&o.PAGE2_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE2_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE2_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE2_LOCK1.Reg, volatile.LoadUint32(&o.PAGE2_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE2_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE2_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE3_LOCK0: Lock configuration LSBs for page 3 (rows 0xc0 through 0xff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK0.Reg, volatile.LoadUint32(&o.PAGE3_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE3_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK0.Reg, volatile.LoadUint32(&o.PAGE3_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE3_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK0.Reg, volatile.LoadUint32(&o.PAGE3_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE3_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK0.Reg, volatile.LoadUint32(&o.PAGE3_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE3_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK0.Reg, volatile.LoadUint32(&o.PAGE3_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE3_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE3_LOCK1: Lock configuration MSBs for page 3 (rows 0xc0 through 0xff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK1.Reg, volatile.LoadUint32(&o.PAGE3_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE3_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK1.Reg, volatile.LoadUint32(&o.PAGE3_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE3_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK1.Reg, volatile.LoadUint32(&o.PAGE3_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE3_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK1.Reg, volatile.LoadUint32(&o.PAGE3_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE3_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE3_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE3_LOCK1.Reg, volatile.LoadUint32(&o.PAGE3_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE3_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE3_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE4_LOCK0: Lock configuration LSBs for page 4 (rows 0x100 through 0x13f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK0.Reg, volatile.LoadUint32(&o.PAGE4_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE4_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK0.Reg, volatile.LoadUint32(&o.PAGE4_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE4_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK0.Reg, volatile.LoadUint32(&o.PAGE4_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE4_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK0.Reg, volatile.LoadUint32(&o.PAGE4_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE4_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK0.Reg, volatile.LoadUint32(&o.PAGE4_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE4_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE4_LOCK1: Lock configuration MSBs for page 4 (rows 0x100 through 0x13f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK1.Reg, volatile.LoadUint32(&o.PAGE4_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE4_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK1.Reg, volatile.LoadUint32(&o.PAGE4_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE4_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK1.Reg, volatile.LoadUint32(&o.PAGE4_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE4_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK1.Reg, volatile.LoadUint32(&o.PAGE4_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE4_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE4_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE4_LOCK1.Reg, volatile.LoadUint32(&o.PAGE4_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE4_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE4_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE5_LOCK0: Lock configuration LSBs for page 5 (rows 0x140 through 0x17f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK0.Reg, volatile.LoadUint32(&o.PAGE5_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE5_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK0.Reg, volatile.LoadUint32(&o.PAGE5_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE5_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK0.Reg, volatile.LoadUint32(&o.PAGE5_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE5_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK0.Reg, volatile.LoadUint32(&o.PAGE5_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE5_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK0.Reg, volatile.LoadUint32(&o.PAGE5_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE5_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE5_LOCK1: Lock configuration MSBs for page 5 (rows 0x140 through 0x17f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK1.Reg, volatile.LoadUint32(&o.PAGE5_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE5_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK1.Reg, volatile.LoadUint32(&o.PAGE5_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE5_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK1.Reg, volatile.LoadUint32(&o.PAGE5_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE5_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK1.Reg, volatile.LoadUint32(&o.PAGE5_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE5_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE5_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE5_LOCK1.Reg, volatile.LoadUint32(&o.PAGE5_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE5_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE5_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE6_LOCK0: Lock configuration LSBs for page 6 (rows 0x180 through 0x1bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK0.Reg, volatile.LoadUint32(&o.PAGE6_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE6_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK0.Reg, volatile.LoadUint32(&o.PAGE6_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE6_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK0.Reg, volatile.LoadUint32(&o.PAGE6_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE6_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK0.Reg, volatile.LoadUint32(&o.PAGE6_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE6_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK0.Reg, volatile.LoadUint32(&o.PAGE6_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE6_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE6_LOCK1: Lock configuration MSBs for page 6 (rows 0x180 through 0x1bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK1.Reg, volatile.LoadUint32(&o.PAGE6_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE6_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK1.Reg, volatile.LoadUint32(&o.PAGE6_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE6_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK1.Reg, volatile.LoadUint32(&o.PAGE6_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE6_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK1.Reg, volatile.LoadUint32(&o.PAGE6_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE6_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE6_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE6_LOCK1.Reg, volatile.LoadUint32(&o.PAGE6_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE6_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE6_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE7_LOCK0: Lock configuration LSBs for page 7 (rows 0x1c0 through 0x1ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK0.Reg, volatile.LoadUint32(&o.PAGE7_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE7_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK0.Reg, volatile.LoadUint32(&o.PAGE7_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE7_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK0.Reg, volatile.LoadUint32(&o.PAGE7_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE7_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK0.Reg, volatile.LoadUint32(&o.PAGE7_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE7_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK0.Reg, volatile.LoadUint32(&o.PAGE7_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE7_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE7_LOCK1: Lock configuration MSBs for page 7 (rows 0x1c0 through 0x1ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK1.Reg, volatile.LoadUint32(&o.PAGE7_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE7_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK1.Reg, volatile.LoadUint32(&o.PAGE7_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE7_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK1.Reg, volatile.LoadUint32(&o.PAGE7_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE7_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK1.Reg, volatile.LoadUint32(&o.PAGE7_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE7_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE7_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE7_LOCK1.Reg, volatile.LoadUint32(&o.PAGE7_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE7_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE7_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE8_LOCK0: Lock configuration LSBs for page 8 (rows 0x200 through 0x23f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK0.Reg, volatile.LoadUint32(&o.PAGE8_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE8_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK0.Reg, volatile.LoadUint32(&o.PAGE8_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE8_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK0.Reg, volatile.LoadUint32(&o.PAGE8_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE8_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK0.Reg, volatile.LoadUint32(&o.PAGE8_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE8_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK0.Reg, volatile.LoadUint32(&o.PAGE8_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE8_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE8_LOCK1: Lock configuration MSBs for page 8 (rows 0x200 through 0x23f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK1.Reg, volatile.LoadUint32(&o.PAGE8_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE8_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK1.Reg, volatile.LoadUint32(&o.PAGE8_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE8_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK1.Reg, volatile.LoadUint32(&o.PAGE8_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE8_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK1.Reg, volatile.LoadUint32(&o.PAGE8_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE8_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE8_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE8_LOCK1.Reg, volatile.LoadUint32(&o.PAGE8_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE8_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE8_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE9_LOCK0: Lock configuration LSBs for page 9 (rows 0x240 through 0x27f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK0.Reg, volatile.LoadUint32(&o.PAGE9_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE9_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK0.Reg, volatile.LoadUint32(&o.PAGE9_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE9_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK0.Reg, volatile.LoadUint32(&o.PAGE9_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE9_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK0.Reg, volatile.LoadUint32(&o.PAGE9_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE9_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK0.Reg, volatile.LoadUint32(&o.PAGE9_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE9_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE9_LOCK1: Lock configuration MSBs for page 9 (rows 0x240 through 0x27f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK1.Reg, volatile.LoadUint32(&o.PAGE9_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE9_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK1.Reg, volatile.LoadUint32(&o.PAGE9_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE9_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK1.Reg, volatile.LoadUint32(&o.PAGE9_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE9_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK1.Reg, volatile.LoadUint32(&o.PAGE9_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE9_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE9_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE9_LOCK1.Reg, volatile.LoadUint32(&o.PAGE9_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE9_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE9_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE10_LOCK0: Lock configuration LSBs for page 10 (rows 0x280 through 0x2bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK0.Reg, volatile.LoadUint32(&o.PAGE10_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE10_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK0.Reg, volatile.LoadUint32(&o.PAGE10_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE10_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK0.Reg, volatile.LoadUint32(&o.PAGE10_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE10_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK0.Reg, volatile.LoadUint32(&o.PAGE10_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE10_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK0.Reg, volatile.LoadUint32(&o.PAGE10_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE10_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE10_LOCK1: Lock configuration MSBs for page 10 (rows 0x280 through 0x2bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK1.Reg, volatile.LoadUint32(&o.PAGE10_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE10_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK1.Reg, volatile.LoadUint32(&o.PAGE10_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE10_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK1.Reg, volatile.LoadUint32(&o.PAGE10_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE10_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK1.Reg, volatile.LoadUint32(&o.PAGE10_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE10_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE10_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE10_LOCK1.Reg, volatile.LoadUint32(&o.PAGE10_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE10_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE10_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE11_LOCK0: Lock configuration LSBs for page 11 (rows 0x2c0 through 0x2ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK0.Reg, volatile.LoadUint32(&o.PAGE11_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE11_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK0.Reg, volatile.LoadUint32(&o.PAGE11_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE11_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK0.Reg, volatile.LoadUint32(&o.PAGE11_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE11_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK0.Reg, volatile.LoadUint32(&o.PAGE11_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE11_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK0.Reg, volatile.LoadUint32(&o.PAGE11_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE11_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE11_LOCK1: Lock configuration MSBs for page 11 (rows 0x2c0 through 0x2ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK1.Reg, volatile.LoadUint32(&o.PAGE11_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE11_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK1.Reg, volatile.LoadUint32(&o.PAGE11_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE11_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK1.Reg, volatile.LoadUint32(&o.PAGE11_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE11_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK1.Reg, volatile.LoadUint32(&o.PAGE11_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE11_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE11_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE11_LOCK1.Reg, volatile.LoadUint32(&o.PAGE11_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE11_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE11_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE12_LOCK0: Lock configuration LSBs for page 12 (rows 0x300 through 0x33f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK0.Reg, volatile.LoadUint32(&o.PAGE12_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE12_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK0.Reg, volatile.LoadUint32(&o.PAGE12_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE12_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK0.Reg, volatile.LoadUint32(&o.PAGE12_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE12_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK0.Reg, volatile.LoadUint32(&o.PAGE12_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE12_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK0.Reg, volatile.LoadUint32(&o.PAGE12_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE12_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE12_LOCK1: Lock configuration MSBs for page 12 (rows 0x300 through 0x33f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK1.Reg, volatile.LoadUint32(&o.PAGE12_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE12_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK1.Reg, volatile.LoadUint32(&o.PAGE12_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE12_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK1.Reg, volatile.LoadUint32(&o.PAGE12_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE12_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK1.Reg, volatile.LoadUint32(&o.PAGE12_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE12_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE12_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE12_LOCK1.Reg, volatile.LoadUint32(&o.PAGE12_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE12_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE12_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE13_LOCK0: Lock configuration LSBs for page 13 (rows 0x340 through 0x37f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK0.Reg, volatile.LoadUint32(&o.PAGE13_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE13_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK0.Reg, volatile.LoadUint32(&o.PAGE13_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE13_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK0.Reg, volatile.LoadUint32(&o.PAGE13_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE13_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK0.Reg, volatile.LoadUint32(&o.PAGE13_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE13_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK0.Reg, volatile.LoadUint32(&o.PAGE13_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE13_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE13_LOCK1: Lock configuration MSBs for page 13 (rows 0x340 through 0x37f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK1.Reg, volatile.LoadUint32(&o.PAGE13_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE13_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK1.Reg, volatile.LoadUint32(&o.PAGE13_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE13_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK1.Reg, volatile.LoadUint32(&o.PAGE13_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE13_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK1.Reg, volatile.LoadUint32(&o.PAGE13_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE13_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE13_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE13_LOCK1.Reg, volatile.LoadUint32(&o.PAGE13_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE13_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE13_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE14_LOCK0: Lock configuration LSBs for page 14 (rows 0x380 through 0x3bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK0.Reg, volatile.LoadUint32(&o.PAGE14_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE14_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK0.Reg, volatile.LoadUint32(&o.PAGE14_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE14_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK0.Reg, volatile.LoadUint32(&o.PAGE14_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE14_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK0.Reg, volatile.LoadUint32(&o.PAGE14_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE14_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK0.Reg, volatile.LoadUint32(&o.PAGE14_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE14_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE14_LOCK1: Lock configuration MSBs for page 14 (rows 0x380 through 0x3bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK1.Reg, volatile.LoadUint32(&o.PAGE14_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE14_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK1.Reg, volatile.LoadUint32(&o.PAGE14_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE14_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK1.Reg, volatile.LoadUint32(&o.PAGE14_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE14_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK1.Reg, volatile.LoadUint32(&o.PAGE14_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE14_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE14_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE14_LOCK1.Reg, volatile.LoadUint32(&o.PAGE14_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE14_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE14_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE15_LOCK0: Lock configuration LSBs for page 15 (rows 0x3c0 through 0x3ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK0.Reg, volatile.LoadUint32(&o.PAGE15_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE15_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK0.Reg, volatile.LoadUint32(&o.PAGE15_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE15_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK0.Reg, volatile.LoadUint32(&o.PAGE15_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE15_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK0.Reg, volatile.LoadUint32(&o.PAGE15_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE15_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK0.Reg, volatile.LoadUint32(&o.PAGE15_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE15_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE15_LOCK1: Lock configuration MSBs for page 15 (rows 0x3c0 through 0x3ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK1.Reg, volatile.LoadUint32(&o.PAGE15_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE15_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK1.Reg, volatile.LoadUint32(&o.PAGE15_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE15_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK1.Reg, volatile.LoadUint32(&o.PAGE15_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE15_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK1.Reg, volatile.LoadUint32(&o.PAGE15_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE15_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE15_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE15_LOCK1.Reg, volatile.LoadUint32(&o.PAGE15_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE15_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE15_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE16_LOCK0: Lock configuration LSBs for page 16 (rows 0x400 through 0x43f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK0.Reg, volatile.LoadUint32(&o.PAGE16_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE16_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK0.Reg, volatile.LoadUint32(&o.PAGE16_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE16_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK0.Reg, volatile.LoadUint32(&o.PAGE16_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE16_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK0.Reg, volatile.LoadUint32(&o.PAGE16_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE16_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK0.Reg, volatile.LoadUint32(&o.PAGE16_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE16_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE16_LOCK1: Lock configuration MSBs for page 16 (rows 0x400 through 0x43f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK1.Reg, volatile.LoadUint32(&o.PAGE16_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE16_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK1.Reg, volatile.LoadUint32(&o.PAGE16_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE16_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK1.Reg, volatile.LoadUint32(&o.PAGE16_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE16_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK1.Reg, volatile.LoadUint32(&o.PAGE16_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE16_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE16_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE16_LOCK1.Reg, volatile.LoadUint32(&o.PAGE16_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE16_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE16_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE17_LOCK0: Lock configuration LSBs for page 17 (rows 0x440 through 0x47f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK0.Reg, volatile.LoadUint32(&o.PAGE17_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE17_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK0.Reg, volatile.LoadUint32(&o.PAGE17_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE17_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK0.Reg, volatile.LoadUint32(&o.PAGE17_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE17_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK0.Reg, volatile.LoadUint32(&o.PAGE17_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE17_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK0.Reg, volatile.LoadUint32(&o.PAGE17_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE17_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE17_LOCK1: Lock configuration MSBs for page 17 (rows 0x440 through 0x47f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK1.Reg, volatile.LoadUint32(&o.PAGE17_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE17_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK1.Reg, volatile.LoadUint32(&o.PAGE17_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE17_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK1.Reg, volatile.LoadUint32(&o.PAGE17_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE17_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK1.Reg, volatile.LoadUint32(&o.PAGE17_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE17_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE17_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE17_LOCK1.Reg, volatile.LoadUint32(&o.PAGE17_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE17_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE17_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE18_LOCK0: Lock configuration LSBs for page 18 (rows 0x480 through 0x4bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK0.Reg, volatile.LoadUint32(&o.PAGE18_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE18_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK0.Reg, volatile.LoadUint32(&o.PAGE18_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE18_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK0.Reg, volatile.LoadUint32(&o.PAGE18_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE18_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK0.Reg, volatile.LoadUint32(&o.PAGE18_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE18_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK0.Reg, volatile.LoadUint32(&o.PAGE18_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE18_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE18_LOCK1: Lock configuration MSBs for page 18 (rows 0x480 through 0x4bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK1.Reg, volatile.LoadUint32(&o.PAGE18_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE18_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK1.Reg, volatile.LoadUint32(&o.PAGE18_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE18_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK1.Reg, volatile.LoadUint32(&o.PAGE18_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE18_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK1.Reg, volatile.LoadUint32(&o.PAGE18_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE18_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE18_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE18_LOCK1.Reg, volatile.LoadUint32(&o.PAGE18_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE18_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE18_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE19_LOCK0: Lock configuration LSBs for page 19 (rows 0x4c0 through 0x4ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK0.Reg, volatile.LoadUint32(&o.PAGE19_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE19_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK0.Reg, volatile.LoadUint32(&o.PAGE19_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE19_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK0.Reg, volatile.LoadUint32(&o.PAGE19_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE19_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK0.Reg, volatile.LoadUint32(&o.PAGE19_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE19_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK0.Reg, volatile.LoadUint32(&o.PAGE19_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE19_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE19_LOCK1: Lock configuration MSBs for page 19 (rows 0x4c0 through 0x4ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK1.Reg, volatile.LoadUint32(&o.PAGE19_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE19_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK1.Reg, volatile.LoadUint32(&o.PAGE19_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE19_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK1.Reg, volatile.LoadUint32(&o.PAGE19_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE19_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK1.Reg, volatile.LoadUint32(&o.PAGE19_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE19_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE19_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE19_LOCK1.Reg, volatile.LoadUint32(&o.PAGE19_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE19_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE19_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE20_LOCK0: Lock configuration LSBs for page 20 (rows 0x500 through 0x53f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK0.Reg, volatile.LoadUint32(&o.PAGE20_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE20_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK0.Reg, volatile.LoadUint32(&o.PAGE20_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE20_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK0.Reg, volatile.LoadUint32(&o.PAGE20_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE20_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK0.Reg, volatile.LoadUint32(&o.PAGE20_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE20_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK0.Reg, volatile.LoadUint32(&o.PAGE20_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE20_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE20_LOCK1: Lock configuration MSBs for page 20 (rows 0x500 through 0x53f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK1.Reg, volatile.LoadUint32(&o.PAGE20_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE20_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK1.Reg, volatile.LoadUint32(&o.PAGE20_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE20_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK1.Reg, volatile.LoadUint32(&o.PAGE20_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE20_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK1.Reg, volatile.LoadUint32(&o.PAGE20_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE20_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE20_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE20_LOCK1.Reg, volatile.LoadUint32(&o.PAGE20_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE20_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE20_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE21_LOCK0: Lock configuration LSBs for page 21 (rows 0x540 through 0x57f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK0.Reg, volatile.LoadUint32(&o.PAGE21_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE21_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK0.Reg, volatile.LoadUint32(&o.PAGE21_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE21_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK0.Reg, volatile.LoadUint32(&o.PAGE21_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE21_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK0.Reg, volatile.LoadUint32(&o.PAGE21_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE21_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK0.Reg, volatile.LoadUint32(&o.PAGE21_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE21_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE21_LOCK1: Lock configuration MSBs for page 21 (rows 0x540 through 0x57f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK1.Reg, volatile.LoadUint32(&o.PAGE21_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE21_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK1.Reg, volatile.LoadUint32(&o.PAGE21_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE21_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK1.Reg, volatile.LoadUint32(&o.PAGE21_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE21_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK1.Reg, volatile.LoadUint32(&o.PAGE21_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE21_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE21_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE21_LOCK1.Reg, volatile.LoadUint32(&o.PAGE21_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE21_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE21_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE22_LOCK0: Lock configuration LSBs for page 22 (rows 0x580 through 0x5bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK0.Reg, volatile.LoadUint32(&o.PAGE22_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE22_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK0.Reg, volatile.LoadUint32(&o.PAGE22_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE22_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK0.Reg, volatile.LoadUint32(&o.PAGE22_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE22_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK0.Reg, volatile.LoadUint32(&o.PAGE22_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE22_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK0.Reg, volatile.LoadUint32(&o.PAGE22_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE22_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE22_LOCK1: Lock configuration MSBs for page 22 (rows 0x580 through 0x5bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK1.Reg, volatile.LoadUint32(&o.PAGE22_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE22_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK1.Reg, volatile.LoadUint32(&o.PAGE22_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE22_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK1.Reg, volatile.LoadUint32(&o.PAGE22_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE22_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK1.Reg, volatile.LoadUint32(&o.PAGE22_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE22_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE22_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE22_LOCK1.Reg, volatile.LoadUint32(&o.PAGE22_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE22_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE22_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE23_LOCK0: Lock configuration LSBs for page 23 (rows 0x5c0 through 0x5ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK0.Reg, volatile.LoadUint32(&o.PAGE23_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE23_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK0.Reg, volatile.LoadUint32(&o.PAGE23_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE23_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK0.Reg, volatile.LoadUint32(&o.PAGE23_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE23_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK0.Reg, volatile.LoadUint32(&o.PAGE23_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE23_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK0.Reg, volatile.LoadUint32(&o.PAGE23_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE23_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE23_LOCK1: Lock configuration MSBs for page 23 (rows 0x5c0 through 0x5ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK1.Reg, volatile.LoadUint32(&o.PAGE23_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE23_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK1.Reg, volatile.LoadUint32(&o.PAGE23_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE23_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK1.Reg, volatile.LoadUint32(&o.PAGE23_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE23_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK1.Reg, volatile.LoadUint32(&o.PAGE23_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE23_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE23_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE23_LOCK1.Reg, volatile.LoadUint32(&o.PAGE23_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE23_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE23_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE24_LOCK0: Lock configuration LSBs for page 24 (rows 0x600 through 0x63f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK0.Reg, volatile.LoadUint32(&o.PAGE24_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE24_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK0.Reg, volatile.LoadUint32(&o.PAGE24_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE24_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK0.Reg, volatile.LoadUint32(&o.PAGE24_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE24_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK0.Reg, volatile.LoadUint32(&o.PAGE24_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE24_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK0.Reg, volatile.LoadUint32(&o.PAGE24_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE24_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE24_LOCK1: Lock configuration MSBs for page 24 (rows 0x600 through 0x63f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK1.Reg, volatile.LoadUint32(&o.PAGE24_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE24_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK1.Reg, volatile.LoadUint32(&o.PAGE24_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE24_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK1.Reg, volatile.LoadUint32(&o.PAGE24_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE24_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK1.Reg, volatile.LoadUint32(&o.PAGE24_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE24_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE24_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE24_LOCK1.Reg, volatile.LoadUint32(&o.PAGE24_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE24_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE24_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE25_LOCK0: Lock configuration LSBs for page 25 (rows 0x640 through 0x67f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK0.Reg, volatile.LoadUint32(&o.PAGE25_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE25_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK0.Reg, volatile.LoadUint32(&o.PAGE25_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE25_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK0.Reg, volatile.LoadUint32(&o.PAGE25_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE25_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK0.Reg, volatile.LoadUint32(&o.PAGE25_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE25_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK0.Reg, volatile.LoadUint32(&o.PAGE25_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE25_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE25_LOCK1: Lock configuration MSBs for page 25 (rows 0x640 through 0x67f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK1.Reg, volatile.LoadUint32(&o.PAGE25_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE25_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK1.Reg, volatile.LoadUint32(&o.PAGE25_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE25_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK1.Reg, volatile.LoadUint32(&o.PAGE25_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE25_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK1.Reg, volatile.LoadUint32(&o.PAGE25_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE25_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE25_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE25_LOCK1.Reg, volatile.LoadUint32(&o.PAGE25_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE25_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE25_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE26_LOCK0: Lock configuration LSBs for page 26 (rows 0x680 through 0x6bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK0.Reg, volatile.LoadUint32(&o.PAGE26_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE26_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK0.Reg, volatile.LoadUint32(&o.PAGE26_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE26_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK0.Reg, volatile.LoadUint32(&o.PAGE26_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE26_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK0.Reg, volatile.LoadUint32(&o.PAGE26_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE26_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK0.Reg, volatile.LoadUint32(&o.PAGE26_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE26_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE26_LOCK1: Lock configuration MSBs for page 26 (rows 0x680 through 0x6bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK1.Reg, volatile.LoadUint32(&o.PAGE26_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE26_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK1.Reg, volatile.LoadUint32(&o.PAGE26_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE26_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK1.Reg, volatile.LoadUint32(&o.PAGE26_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE26_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK1.Reg, volatile.LoadUint32(&o.PAGE26_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE26_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE26_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE26_LOCK1.Reg, volatile.LoadUint32(&o.PAGE26_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE26_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE26_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE27_LOCK0: Lock configuration LSBs for page 27 (rows 0x6c0 through 0x6ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK0.Reg, volatile.LoadUint32(&o.PAGE27_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE27_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK0.Reg, volatile.LoadUint32(&o.PAGE27_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE27_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK0.Reg, volatile.LoadUint32(&o.PAGE27_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE27_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK0.Reg, volatile.LoadUint32(&o.PAGE27_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE27_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK0.Reg, volatile.LoadUint32(&o.PAGE27_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE27_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE27_LOCK1: Lock configuration MSBs for page 27 (rows 0x6c0 through 0x6ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK1.Reg, volatile.LoadUint32(&o.PAGE27_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE27_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK1.Reg, volatile.LoadUint32(&o.PAGE27_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE27_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK1.Reg, volatile.LoadUint32(&o.PAGE27_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE27_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK1.Reg, volatile.LoadUint32(&o.PAGE27_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE27_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE27_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE27_LOCK1.Reg, volatile.LoadUint32(&o.PAGE27_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE27_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE27_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE28_LOCK0: Lock configuration LSBs for page 28 (rows 0x700 through 0x73f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK0.Reg, volatile.LoadUint32(&o.PAGE28_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE28_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK0.Reg, volatile.LoadUint32(&o.PAGE28_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE28_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK0.Reg, volatile.LoadUint32(&o.PAGE28_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE28_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK0.Reg, volatile.LoadUint32(&o.PAGE28_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE28_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK0.Reg, volatile.LoadUint32(&o.PAGE28_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE28_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE28_LOCK1: Lock configuration MSBs for page 28 (rows 0x700 through 0x73f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK1.Reg, volatile.LoadUint32(&o.PAGE28_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE28_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK1.Reg, volatile.LoadUint32(&o.PAGE28_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE28_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK1.Reg, volatile.LoadUint32(&o.PAGE28_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE28_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK1.Reg, volatile.LoadUint32(&o.PAGE28_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE28_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE28_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE28_LOCK1.Reg, volatile.LoadUint32(&o.PAGE28_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE28_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE28_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE29_LOCK0: Lock configuration LSBs for page 29 (rows 0x740 through 0x77f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK0.Reg, volatile.LoadUint32(&o.PAGE29_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE29_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK0.Reg, volatile.LoadUint32(&o.PAGE29_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE29_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK0.Reg, volatile.LoadUint32(&o.PAGE29_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE29_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK0.Reg, volatile.LoadUint32(&o.PAGE29_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE29_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK0.Reg, volatile.LoadUint32(&o.PAGE29_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE29_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE29_LOCK1: Lock configuration MSBs for page 29 (rows 0x740 through 0x77f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK1.Reg, volatile.LoadUint32(&o.PAGE29_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE29_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK1.Reg, volatile.LoadUint32(&o.PAGE29_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE29_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK1.Reg, volatile.LoadUint32(&o.PAGE29_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE29_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK1.Reg, volatile.LoadUint32(&o.PAGE29_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE29_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE29_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE29_LOCK1.Reg, volatile.LoadUint32(&o.PAGE29_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE29_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE29_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE30_LOCK0: Lock configuration LSBs for page 30 (rows 0x780 through 0x7bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK0.Reg, volatile.LoadUint32(&o.PAGE30_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE30_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK0.Reg, volatile.LoadUint32(&o.PAGE30_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE30_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK0.Reg, volatile.LoadUint32(&o.PAGE30_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE30_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK0.Reg, volatile.LoadUint32(&o.PAGE30_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE30_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK0.Reg, volatile.LoadUint32(&o.PAGE30_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE30_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE30_LOCK1: Lock configuration MSBs for page 30 (rows 0x780 through 0x7bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK1.Reg, volatile.LoadUint32(&o.PAGE30_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE30_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK1.Reg, volatile.LoadUint32(&o.PAGE30_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE30_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK1.Reg, volatile.LoadUint32(&o.PAGE30_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE30_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK1.Reg, volatile.LoadUint32(&o.PAGE30_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE30_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE30_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE30_LOCK1.Reg, volatile.LoadUint32(&o.PAGE30_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE30_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE30_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE31_LOCK0: Lock configuration LSBs for page 31 (rows 0x7c0 through 0x7ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK0.Reg, volatile.LoadUint32(&o.PAGE31_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE31_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK0.Reg, volatile.LoadUint32(&o.PAGE31_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE31_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK0.Reg, volatile.LoadUint32(&o.PAGE31_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE31_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK0.Reg, volatile.LoadUint32(&o.PAGE31_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE31_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK0.Reg, volatile.LoadUint32(&o.PAGE31_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE31_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE31_LOCK1: Lock configuration MSBs for page 31 (rows 0x7c0 through 0x7ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK1.Reg, volatile.LoadUint32(&o.PAGE31_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE31_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK1.Reg, volatile.LoadUint32(&o.PAGE31_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE31_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK1.Reg, volatile.LoadUint32(&o.PAGE31_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE31_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK1.Reg, volatile.LoadUint32(&o.PAGE31_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE31_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE31_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE31_LOCK1.Reg, volatile.LoadUint32(&o.PAGE31_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE31_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE31_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE32_LOCK0: Lock configuration LSBs for page 32 (rows 0x800 through 0x83f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK0.Reg, volatile.LoadUint32(&o.PAGE32_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE32_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK0.Reg, volatile.LoadUint32(&o.PAGE32_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE32_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK0.Reg, volatile.LoadUint32(&o.PAGE32_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE32_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK0.Reg, volatile.LoadUint32(&o.PAGE32_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE32_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK0.Reg, volatile.LoadUint32(&o.PAGE32_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE32_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE32_LOCK1: Lock configuration MSBs for page 32 (rows 0x800 through 0x83f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK1.Reg, volatile.LoadUint32(&o.PAGE32_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE32_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK1.Reg, volatile.LoadUint32(&o.PAGE32_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE32_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK1.Reg, volatile.LoadUint32(&o.PAGE32_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE32_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK1.Reg, volatile.LoadUint32(&o.PAGE32_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE32_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE32_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE32_LOCK1.Reg, volatile.LoadUint32(&o.PAGE32_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE32_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE32_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE33_LOCK0: Lock configuration LSBs for page 33 (rows 0x840 through 0x87f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK0.Reg, volatile.LoadUint32(&o.PAGE33_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE33_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK0.Reg, volatile.LoadUint32(&o.PAGE33_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE33_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK0.Reg, volatile.LoadUint32(&o.PAGE33_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE33_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK0.Reg, volatile.LoadUint32(&o.PAGE33_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE33_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK0.Reg, volatile.LoadUint32(&o.PAGE33_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE33_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE33_LOCK1: Lock configuration MSBs for page 33 (rows 0x840 through 0x87f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK1.Reg, volatile.LoadUint32(&o.PAGE33_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE33_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK1.Reg, volatile.LoadUint32(&o.PAGE33_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE33_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK1.Reg, volatile.LoadUint32(&o.PAGE33_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE33_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK1.Reg, volatile.LoadUint32(&o.PAGE33_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE33_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE33_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE33_LOCK1.Reg, volatile.LoadUint32(&o.PAGE33_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE33_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE33_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE34_LOCK0: Lock configuration LSBs for page 34 (rows 0x880 through 0x8bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK0.Reg, volatile.LoadUint32(&o.PAGE34_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE34_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK0.Reg, volatile.LoadUint32(&o.PAGE34_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE34_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK0.Reg, volatile.LoadUint32(&o.PAGE34_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE34_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK0.Reg, volatile.LoadUint32(&o.PAGE34_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE34_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK0.Reg, volatile.LoadUint32(&o.PAGE34_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE34_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE34_LOCK1: Lock configuration MSBs for page 34 (rows 0x880 through 0x8bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK1.Reg, volatile.LoadUint32(&o.PAGE34_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE34_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK1.Reg, volatile.LoadUint32(&o.PAGE34_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE34_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK1.Reg, volatile.LoadUint32(&o.PAGE34_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE34_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK1.Reg, volatile.LoadUint32(&o.PAGE34_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE34_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE34_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE34_LOCK1.Reg, volatile.LoadUint32(&o.PAGE34_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE34_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE34_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE35_LOCK0: Lock configuration LSBs for page 35 (rows 0x8c0 through 0x8ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK0.Reg, volatile.LoadUint32(&o.PAGE35_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE35_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK0.Reg, volatile.LoadUint32(&o.PAGE35_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE35_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK0.Reg, volatile.LoadUint32(&o.PAGE35_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE35_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK0.Reg, volatile.LoadUint32(&o.PAGE35_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE35_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK0.Reg, volatile.LoadUint32(&o.PAGE35_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE35_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE35_LOCK1: Lock configuration MSBs for page 35 (rows 0x8c0 through 0x8ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK1.Reg, volatile.LoadUint32(&o.PAGE35_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE35_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK1.Reg, volatile.LoadUint32(&o.PAGE35_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE35_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK1.Reg, volatile.LoadUint32(&o.PAGE35_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE35_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK1.Reg, volatile.LoadUint32(&o.PAGE35_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE35_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE35_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE35_LOCK1.Reg, volatile.LoadUint32(&o.PAGE35_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE35_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE35_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE36_LOCK0: Lock configuration LSBs for page 36 (rows 0x900 through 0x93f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK0.Reg, volatile.LoadUint32(&o.PAGE36_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE36_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK0.Reg, volatile.LoadUint32(&o.PAGE36_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE36_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK0.Reg, volatile.LoadUint32(&o.PAGE36_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE36_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK0.Reg, volatile.LoadUint32(&o.PAGE36_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE36_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK0.Reg, volatile.LoadUint32(&o.PAGE36_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE36_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE36_LOCK1: Lock configuration MSBs for page 36 (rows 0x900 through 0x93f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK1.Reg, volatile.LoadUint32(&o.PAGE36_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE36_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK1.Reg, volatile.LoadUint32(&o.PAGE36_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE36_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK1.Reg, volatile.LoadUint32(&o.PAGE36_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE36_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK1.Reg, volatile.LoadUint32(&o.PAGE36_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE36_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE36_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE36_LOCK1.Reg, volatile.LoadUint32(&o.PAGE36_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE36_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE36_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE37_LOCK0: Lock configuration LSBs for page 37 (rows 0x940 through 0x97f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK0.Reg, volatile.LoadUint32(&o.PAGE37_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE37_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK0.Reg, volatile.LoadUint32(&o.PAGE37_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE37_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK0.Reg, volatile.LoadUint32(&o.PAGE37_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE37_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK0.Reg, volatile.LoadUint32(&o.PAGE37_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE37_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK0.Reg, volatile.LoadUint32(&o.PAGE37_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE37_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE37_LOCK1: Lock configuration MSBs for page 37 (rows 0x940 through 0x97f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK1.Reg, volatile.LoadUint32(&o.PAGE37_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE37_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK1.Reg, volatile.LoadUint32(&o.PAGE37_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE37_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK1.Reg, volatile.LoadUint32(&o.PAGE37_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE37_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK1.Reg, volatile.LoadUint32(&o.PAGE37_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE37_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE37_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE37_LOCK1.Reg, volatile.LoadUint32(&o.PAGE37_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE37_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE37_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE38_LOCK0: Lock configuration LSBs for page 38 (rows 0x980 through 0x9bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK0.Reg, volatile.LoadUint32(&o.PAGE38_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE38_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK0.Reg, volatile.LoadUint32(&o.PAGE38_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE38_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK0.Reg, volatile.LoadUint32(&o.PAGE38_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE38_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK0.Reg, volatile.LoadUint32(&o.PAGE38_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE38_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK0.Reg, volatile.LoadUint32(&o.PAGE38_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE38_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE38_LOCK1: Lock configuration MSBs for page 38 (rows 0x980 through 0x9bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK1.Reg, volatile.LoadUint32(&o.PAGE38_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE38_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK1.Reg, volatile.LoadUint32(&o.PAGE38_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE38_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK1.Reg, volatile.LoadUint32(&o.PAGE38_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE38_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK1.Reg, volatile.LoadUint32(&o.PAGE38_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE38_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE38_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE38_LOCK1.Reg, volatile.LoadUint32(&o.PAGE38_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE38_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE38_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE39_LOCK0: Lock configuration LSBs for page 39 (rows 0x9c0 through 0x9ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK0.Reg, volatile.LoadUint32(&o.PAGE39_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE39_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK0.Reg, volatile.LoadUint32(&o.PAGE39_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE39_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK0.Reg, volatile.LoadUint32(&o.PAGE39_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE39_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK0.Reg, volatile.LoadUint32(&o.PAGE39_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE39_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK0.Reg, volatile.LoadUint32(&o.PAGE39_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE39_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE39_LOCK1: Lock configuration MSBs for page 39 (rows 0x9c0 through 0x9ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK1.Reg, volatile.LoadUint32(&o.PAGE39_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE39_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK1.Reg, volatile.LoadUint32(&o.PAGE39_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE39_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK1.Reg, volatile.LoadUint32(&o.PAGE39_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE39_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK1.Reg, volatile.LoadUint32(&o.PAGE39_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE39_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE39_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE39_LOCK1.Reg, volatile.LoadUint32(&o.PAGE39_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE39_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE39_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE40_LOCK0: Lock configuration LSBs for page 40 (rows 0xa00 through 0xa3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK0.Reg, volatile.LoadUint32(&o.PAGE40_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE40_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK0.Reg, volatile.LoadUint32(&o.PAGE40_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE40_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK0.Reg, volatile.LoadUint32(&o.PAGE40_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE40_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK0.Reg, volatile.LoadUint32(&o.PAGE40_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE40_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK0.Reg, volatile.LoadUint32(&o.PAGE40_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE40_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE40_LOCK1: Lock configuration MSBs for page 40 (rows 0xa00 through 0xa3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK1.Reg, volatile.LoadUint32(&o.PAGE40_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE40_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK1.Reg, volatile.LoadUint32(&o.PAGE40_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE40_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK1.Reg, volatile.LoadUint32(&o.PAGE40_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE40_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK1.Reg, volatile.LoadUint32(&o.PAGE40_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE40_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE40_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE40_LOCK1.Reg, volatile.LoadUint32(&o.PAGE40_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE40_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE40_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE41_LOCK0: Lock configuration LSBs for page 41 (rows 0xa40 through 0xa7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK0.Reg, volatile.LoadUint32(&o.PAGE41_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE41_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK0.Reg, volatile.LoadUint32(&o.PAGE41_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE41_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK0.Reg, volatile.LoadUint32(&o.PAGE41_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE41_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK0.Reg, volatile.LoadUint32(&o.PAGE41_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE41_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK0.Reg, volatile.LoadUint32(&o.PAGE41_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE41_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE41_LOCK1: Lock configuration MSBs for page 41 (rows 0xa40 through 0xa7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK1.Reg, volatile.LoadUint32(&o.PAGE41_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE41_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK1.Reg, volatile.LoadUint32(&o.PAGE41_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE41_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK1.Reg, volatile.LoadUint32(&o.PAGE41_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE41_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK1.Reg, volatile.LoadUint32(&o.PAGE41_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE41_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE41_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE41_LOCK1.Reg, volatile.LoadUint32(&o.PAGE41_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE41_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE41_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE42_LOCK0: Lock configuration LSBs for page 42 (rows 0xa80 through 0xabf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK0.Reg, volatile.LoadUint32(&o.PAGE42_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE42_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK0.Reg, volatile.LoadUint32(&o.PAGE42_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE42_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK0.Reg, volatile.LoadUint32(&o.PAGE42_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE42_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK0.Reg, volatile.LoadUint32(&o.PAGE42_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE42_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK0.Reg, volatile.LoadUint32(&o.PAGE42_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE42_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE42_LOCK1: Lock configuration MSBs for page 42 (rows 0xa80 through 0xabf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK1.Reg, volatile.LoadUint32(&o.PAGE42_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE42_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK1.Reg, volatile.LoadUint32(&o.PAGE42_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE42_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK1.Reg, volatile.LoadUint32(&o.PAGE42_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE42_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK1.Reg, volatile.LoadUint32(&o.PAGE42_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE42_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE42_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE42_LOCK1.Reg, volatile.LoadUint32(&o.PAGE42_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE42_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE42_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE43_LOCK0: Lock configuration LSBs for page 43 (rows 0xac0 through 0xaff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK0.Reg, volatile.LoadUint32(&o.PAGE43_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE43_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK0.Reg, volatile.LoadUint32(&o.PAGE43_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE43_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK0.Reg, volatile.LoadUint32(&o.PAGE43_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE43_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK0.Reg, volatile.LoadUint32(&o.PAGE43_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE43_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK0.Reg, volatile.LoadUint32(&o.PAGE43_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE43_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE43_LOCK1: Lock configuration MSBs for page 43 (rows 0xac0 through 0xaff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK1.Reg, volatile.LoadUint32(&o.PAGE43_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE43_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK1.Reg, volatile.LoadUint32(&o.PAGE43_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE43_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK1.Reg, volatile.LoadUint32(&o.PAGE43_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE43_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK1.Reg, volatile.LoadUint32(&o.PAGE43_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE43_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE43_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE43_LOCK1.Reg, volatile.LoadUint32(&o.PAGE43_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE43_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE43_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE44_LOCK0: Lock configuration LSBs for page 44 (rows 0xb00 through 0xb3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK0.Reg, volatile.LoadUint32(&o.PAGE44_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE44_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK0.Reg, volatile.LoadUint32(&o.PAGE44_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE44_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK0.Reg, volatile.LoadUint32(&o.PAGE44_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE44_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK0.Reg, volatile.LoadUint32(&o.PAGE44_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE44_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK0.Reg, volatile.LoadUint32(&o.PAGE44_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE44_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE44_LOCK1: Lock configuration MSBs for page 44 (rows 0xb00 through 0xb3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK1.Reg, volatile.LoadUint32(&o.PAGE44_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE44_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK1.Reg, volatile.LoadUint32(&o.PAGE44_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE44_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK1.Reg, volatile.LoadUint32(&o.PAGE44_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE44_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK1.Reg, volatile.LoadUint32(&o.PAGE44_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE44_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE44_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE44_LOCK1.Reg, volatile.LoadUint32(&o.PAGE44_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE44_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE44_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE45_LOCK0: Lock configuration LSBs for page 45 (rows 0xb40 through 0xb7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK0.Reg, volatile.LoadUint32(&o.PAGE45_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE45_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK0.Reg, volatile.LoadUint32(&o.PAGE45_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE45_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK0.Reg, volatile.LoadUint32(&o.PAGE45_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE45_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK0.Reg, volatile.LoadUint32(&o.PAGE45_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE45_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK0.Reg, volatile.LoadUint32(&o.PAGE45_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE45_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE45_LOCK1: Lock configuration MSBs for page 45 (rows 0xb40 through 0xb7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK1.Reg, volatile.LoadUint32(&o.PAGE45_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE45_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK1.Reg, volatile.LoadUint32(&o.PAGE45_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE45_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK1.Reg, volatile.LoadUint32(&o.PAGE45_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE45_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK1.Reg, volatile.LoadUint32(&o.PAGE45_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE45_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE45_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE45_LOCK1.Reg, volatile.LoadUint32(&o.PAGE45_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE45_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE45_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE46_LOCK0: Lock configuration LSBs for page 46 (rows 0xb80 through 0xbbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK0.Reg, volatile.LoadUint32(&o.PAGE46_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE46_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK0.Reg, volatile.LoadUint32(&o.PAGE46_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE46_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK0.Reg, volatile.LoadUint32(&o.PAGE46_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE46_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK0.Reg, volatile.LoadUint32(&o.PAGE46_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE46_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK0.Reg, volatile.LoadUint32(&o.PAGE46_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE46_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE46_LOCK1: Lock configuration MSBs for page 46 (rows 0xb80 through 0xbbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK1.Reg, volatile.LoadUint32(&o.PAGE46_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE46_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK1.Reg, volatile.LoadUint32(&o.PAGE46_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE46_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK1.Reg, volatile.LoadUint32(&o.PAGE46_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE46_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK1.Reg, volatile.LoadUint32(&o.PAGE46_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE46_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE46_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE46_LOCK1.Reg, volatile.LoadUint32(&o.PAGE46_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE46_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE46_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE47_LOCK0: Lock configuration LSBs for page 47 (rows 0xbc0 through 0xbff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK0.Reg, volatile.LoadUint32(&o.PAGE47_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE47_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK0.Reg, volatile.LoadUint32(&o.PAGE47_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE47_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK0.Reg, volatile.LoadUint32(&o.PAGE47_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE47_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK0.Reg, volatile.LoadUint32(&o.PAGE47_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE47_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK0.Reg, volatile.LoadUint32(&o.PAGE47_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE47_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE47_LOCK1: Lock configuration MSBs for page 47 (rows 0xbc0 through 0xbff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK1.Reg, volatile.LoadUint32(&o.PAGE47_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE47_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK1.Reg, volatile.LoadUint32(&o.PAGE47_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE47_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK1.Reg, volatile.LoadUint32(&o.PAGE47_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE47_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK1.Reg, volatile.LoadUint32(&o.PAGE47_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE47_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE47_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE47_LOCK1.Reg, volatile.LoadUint32(&o.PAGE47_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE47_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE47_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE48_LOCK0: Lock configuration LSBs for page 48 (rows 0xc00 through 0xc3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK0.Reg, volatile.LoadUint32(&o.PAGE48_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE48_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK0.Reg, volatile.LoadUint32(&o.PAGE48_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE48_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK0.Reg, volatile.LoadUint32(&o.PAGE48_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE48_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK0.Reg, volatile.LoadUint32(&o.PAGE48_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE48_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK0.Reg, volatile.LoadUint32(&o.PAGE48_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE48_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE48_LOCK1: Lock configuration MSBs for page 48 (rows 0xc00 through 0xc3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK1.Reg, volatile.LoadUint32(&o.PAGE48_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE48_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK1.Reg, volatile.LoadUint32(&o.PAGE48_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE48_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK1.Reg, volatile.LoadUint32(&o.PAGE48_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE48_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK1.Reg, volatile.LoadUint32(&o.PAGE48_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE48_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE48_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE48_LOCK1.Reg, volatile.LoadUint32(&o.PAGE48_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE48_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE48_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE49_LOCK0: Lock configuration LSBs for page 49 (rows 0xc40 through 0xc7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK0.Reg, volatile.LoadUint32(&o.PAGE49_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE49_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK0.Reg, volatile.LoadUint32(&o.PAGE49_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE49_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK0.Reg, volatile.LoadUint32(&o.PAGE49_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE49_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK0.Reg, volatile.LoadUint32(&o.PAGE49_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE49_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK0.Reg, volatile.LoadUint32(&o.PAGE49_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE49_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE49_LOCK1: Lock configuration MSBs for page 49 (rows 0xc40 through 0xc7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK1.Reg, volatile.LoadUint32(&o.PAGE49_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE49_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK1.Reg, volatile.LoadUint32(&o.PAGE49_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE49_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK1.Reg, volatile.LoadUint32(&o.PAGE49_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE49_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK1.Reg, volatile.LoadUint32(&o.PAGE49_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE49_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE49_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE49_LOCK1.Reg, volatile.LoadUint32(&o.PAGE49_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE49_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE49_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE50_LOCK0: Lock configuration LSBs for page 50 (rows 0xc80 through 0xcbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK0.Reg, volatile.LoadUint32(&o.PAGE50_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE50_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK0.Reg, volatile.LoadUint32(&o.PAGE50_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE50_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK0.Reg, volatile.LoadUint32(&o.PAGE50_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE50_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK0.Reg, volatile.LoadUint32(&o.PAGE50_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE50_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK0.Reg, volatile.LoadUint32(&o.PAGE50_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE50_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE50_LOCK1: Lock configuration MSBs for page 50 (rows 0xc80 through 0xcbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK1.Reg, volatile.LoadUint32(&o.PAGE50_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE50_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK1.Reg, volatile.LoadUint32(&o.PAGE50_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE50_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK1.Reg, volatile.LoadUint32(&o.PAGE50_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE50_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK1.Reg, volatile.LoadUint32(&o.PAGE50_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE50_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE50_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE50_LOCK1.Reg, volatile.LoadUint32(&o.PAGE50_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE50_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE50_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE51_LOCK0: Lock configuration LSBs for page 51 (rows 0xcc0 through 0xcff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK0.Reg, volatile.LoadUint32(&o.PAGE51_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE51_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK0.Reg, volatile.LoadUint32(&o.PAGE51_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE51_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK0.Reg, volatile.LoadUint32(&o.PAGE51_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE51_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK0.Reg, volatile.LoadUint32(&o.PAGE51_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE51_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK0.Reg, volatile.LoadUint32(&o.PAGE51_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE51_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE51_LOCK1: Lock configuration MSBs for page 51 (rows 0xcc0 through 0xcff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK1.Reg, volatile.LoadUint32(&o.PAGE51_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE51_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK1.Reg, volatile.LoadUint32(&o.PAGE51_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE51_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK1.Reg, volatile.LoadUint32(&o.PAGE51_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE51_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK1.Reg, volatile.LoadUint32(&o.PAGE51_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE51_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE51_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE51_LOCK1.Reg, volatile.LoadUint32(&o.PAGE51_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE51_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE51_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE52_LOCK0: Lock configuration LSBs for page 52 (rows 0xd00 through 0xd3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK0.Reg, volatile.LoadUint32(&o.PAGE52_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE52_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK0.Reg, volatile.LoadUint32(&o.PAGE52_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE52_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK0.Reg, volatile.LoadUint32(&o.PAGE52_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE52_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK0.Reg, volatile.LoadUint32(&o.PAGE52_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE52_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK0.Reg, volatile.LoadUint32(&o.PAGE52_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE52_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE52_LOCK1: Lock configuration MSBs for page 52 (rows 0xd00 through 0xd3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK1.Reg, volatile.LoadUint32(&o.PAGE52_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE52_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK1.Reg, volatile.LoadUint32(&o.PAGE52_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE52_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK1.Reg, volatile.LoadUint32(&o.PAGE52_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE52_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK1.Reg, volatile.LoadUint32(&o.PAGE52_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE52_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE52_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE52_LOCK1.Reg, volatile.LoadUint32(&o.PAGE52_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE52_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE52_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE53_LOCK0: Lock configuration LSBs for page 53 (rows 0xd40 through 0xd7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK0.Reg, volatile.LoadUint32(&o.PAGE53_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE53_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK0.Reg, volatile.LoadUint32(&o.PAGE53_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE53_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK0.Reg, volatile.LoadUint32(&o.PAGE53_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE53_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK0.Reg, volatile.LoadUint32(&o.PAGE53_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE53_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK0.Reg, volatile.LoadUint32(&o.PAGE53_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE53_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE53_LOCK1: Lock configuration MSBs for page 53 (rows 0xd40 through 0xd7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK1.Reg, volatile.LoadUint32(&o.PAGE53_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE53_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK1.Reg, volatile.LoadUint32(&o.PAGE53_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE53_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK1.Reg, volatile.LoadUint32(&o.PAGE53_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE53_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK1.Reg, volatile.LoadUint32(&o.PAGE53_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE53_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE53_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE53_LOCK1.Reg, volatile.LoadUint32(&o.PAGE53_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE53_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE53_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE54_LOCK0: Lock configuration LSBs for page 54 (rows 0xd80 through 0xdbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK0.Reg, volatile.LoadUint32(&o.PAGE54_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE54_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK0.Reg, volatile.LoadUint32(&o.PAGE54_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE54_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK0.Reg, volatile.LoadUint32(&o.PAGE54_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE54_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK0.Reg, volatile.LoadUint32(&o.PAGE54_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE54_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK0.Reg, volatile.LoadUint32(&o.PAGE54_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE54_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE54_LOCK1: Lock configuration MSBs for page 54 (rows 0xd80 through 0xdbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK1.Reg, volatile.LoadUint32(&o.PAGE54_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE54_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK1.Reg, volatile.LoadUint32(&o.PAGE54_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE54_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK1.Reg, volatile.LoadUint32(&o.PAGE54_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE54_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK1.Reg, volatile.LoadUint32(&o.PAGE54_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE54_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE54_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE54_LOCK1.Reg, volatile.LoadUint32(&o.PAGE54_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE54_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE54_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE55_LOCK0: Lock configuration LSBs for page 55 (rows 0xdc0 through 0xdff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK0.Reg, volatile.LoadUint32(&o.PAGE55_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE55_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK0.Reg, volatile.LoadUint32(&o.PAGE55_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE55_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK0.Reg, volatile.LoadUint32(&o.PAGE55_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE55_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK0.Reg, volatile.LoadUint32(&o.PAGE55_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE55_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK0.Reg, volatile.LoadUint32(&o.PAGE55_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE55_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE55_LOCK1: Lock configuration MSBs for page 55 (rows 0xdc0 through 0xdff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK1.Reg, volatile.LoadUint32(&o.PAGE55_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE55_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK1.Reg, volatile.LoadUint32(&o.PAGE55_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE55_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK1.Reg, volatile.LoadUint32(&o.PAGE55_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE55_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK1.Reg, volatile.LoadUint32(&o.PAGE55_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE55_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE55_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE55_LOCK1.Reg, volatile.LoadUint32(&o.PAGE55_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE55_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE55_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE56_LOCK0: Lock configuration LSBs for page 56 (rows 0xe00 through 0xe3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK0.Reg, volatile.LoadUint32(&o.PAGE56_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE56_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK0.Reg, volatile.LoadUint32(&o.PAGE56_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE56_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK0.Reg, volatile.LoadUint32(&o.PAGE56_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE56_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK0.Reg, volatile.LoadUint32(&o.PAGE56_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE56_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK0.Reg, volatile.LoadUint32(&o.PAGE56_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE56_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE56_LOCK1: Lock configuration MSBs for page 56 (rows 0xe00 through 0xe3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK1.Reg, volatile.LoadUint32(&o.PAGE56_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE56_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK1.Reg, volatile.LoadUint32(&o.PAGE56_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE56_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK1.Reg, volatile.LoadUint32(&o.PAGE56_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE56_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK1.Reg, volatile.LoadUint32(&o.PAGE56_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE56_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE56_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE56_LOCK1.Reg, volatile.LoadUint32(&o.PAGE56_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE56_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE56_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE57_LOCK0: Lock configuration LSBs for page 57 (rows 0xe40 through 0xe7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK0.Reg, volatile.LoadUint32(&o.PAGE57_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE57_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK0.Reg, volatile.LoadUint32(&o.PAGE57_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE57_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK0.Reg, volatile.LoadUint32(&o.PAGE57_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE57_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK0.Reg, volatile.LoadUint32(&o.PAGE57_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE57_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK0.Reg, volatile.LoadUint32(&o.PAGE57_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE57_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE57_LOCK1: Lock configuration MSBs for page 57 (rows 0xe40 through 0xe7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK1.Reg, volatile.LoadUint32(&o.PAGE57_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE57_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK1.Reg, volatile.LoadUint32(&o.PAGE57_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE57_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK1.Reg, volatile.LoadUint32(&o.PAGE57_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE57_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK1.Reg, volatile.LoadUint32(&o.PAGE57_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE57_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE57_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE57_LOCK1.Reg, volatile.LoadUint32(&o.PAGE57_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE57_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE57_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE58_LOCK0: Lock configuration LSBs for page 58 (rows 0xe80 through 0xebf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK0.Reg, volatile.LoadUint32(&o.PAGE58_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE58_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK0.Reg, volatile.LoadUint32(&o.PAGE58_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE58_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK0.Reg, volatile.LoadUint32(&o.PAGE58_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE58_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK0.Reg, volatile.LoadUint32(&o.PAGE58_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE58_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK0.Reg, volatile.LoadUint32(&o.PAGE58_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE58_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE58_LOCK1: Lock configuration MSBs for page 58 (rows 0xe80 through 0xebf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK1.Reg, volatile.LoadUint32(&o.PAGE58_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE58_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK1.Reg, volatile.LoadUint32(&o.PAGE58_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE58_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK1.Reg, volatile.LoadUint32(&o.PAGE58_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE58_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK1.Reg, volatile.LoadUint32(&o.PAGE58_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE58_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE58_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE58_LOCK1.Reg, volatile.LoadUint32(&o.PAGE58_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE58_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE58_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE59_LOCK0: Lock configuration LSBs for page 59 (rows 0xec0 through 0xeff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK0.Reg, volatile.LoadUint32(&o.PAGE59_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE59_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK0.Reg, volatile.LoadUint32(&o.PAGE59_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE59_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK0.Reg, volatile.LoadUint32(&o.PAGE59_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE59_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK0.Reg, volatile.LoadUint32(&o.PAGE59_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE59_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK0.Reg, volatile.LoadUint32(&o.PAGE59_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE59_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE59_LOCK1: Lock configuration MSBs for page 59 (rows 0xec0 through 0xeff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK1.Reg, volatile.LoadUint32(&o.PAGE59_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE59_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK1.Reg, volatile.LoadUint32(&o.PAGE59_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE59_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK1.Reg, volatile.LoadUint32(&o.PAGE59_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE59_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK1.Reg, volatile.LoadUint32(&o.PAGE59_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE59_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE59_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE59_LOCK1.Reg, volatile.LoadUint32(&o.PAGE59_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE59_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE59_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE60_LOCK0: Lock configuration LSBs for page 60 (rows 0xf00 through 0xf3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK0.Reg, volatile.LoadUint32(&o.PAGE60_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE60_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK0.Reg, volatile.LoadUint32(&o.PAGE60_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE60_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK0.Reg, volatile.LoadUint32(&o.PAGE60_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE60_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK0.Reg, volatile.LoadUint32(&o.PAGE60_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE60_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK0.Reg, volatile.LoadUint32(&o.PAGE60_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE60_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE60_LOCK1: Lock configuration MSBs for page 60 (rows 0xf00 through 0xf3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK1.Reg, volatile.LoadUint32(&o.PAGE60_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE60_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK1.Reg, volatile.LoadUint32(&o.PAGE60_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE60_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK1.Reg, volatile.LoadUint32(&o.PAGE60_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE60_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK1.Reg, volatile.LoadUint32(&o.PAGE60_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE60_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE60_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE60_LOCK1.Reg, volatile.LoadUint32(&o.PAGE60_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE60_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE60_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE61_LOCK0: Lock configuration LSBs for page 61 (rows 0xf40 through 0xf7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK0.Reg, volatile.LoadUint32(&o.PAGE61_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE61_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK0.Reg, volatile.LoadUint32(&o.PAGE61_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE61_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK0.Reg, volatile.LoadUint32(&o.PAGE61_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE61_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK0.Reg, volatile.LoadUint32(&o.PAGE61_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE61_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK0.Reg, volatile.LoadUint32(&o.PAGE61_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE61_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE61_LOCK1: Lock configuration MSBs for page 61 (rows 0xf40 through 0xf7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK1.Reg, volatile.LoadUint32(&o.PAGE61_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE61_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK1.Reg, volatile.LoadUint32(&o.PAGE61_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE61_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK1.Reg, volatile.LoadUint32(&o.PAGE61_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE61_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK1.Reg, volatile.LoadUint32(&o.PAGE61_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE61_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE61_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE61_LOCK1.Reg, volatile.LoadUint32(&o.PAGE61_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE61_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE61_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE62_LOCK0: Lock configuration LSBs for page 62 (rows 0xf80 through 0xfbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK0.Reg, volatile.LoadUint32(&o.PAGE62_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE62_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK0.Reg, volatile.LoadUint32(&o.PAGE62_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE62_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK0.Reg, volatile.LoadUint32(&o.PAGE62_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE62_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK0.Reg, volatile.LoadUint32(&o.PAGE62_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE62_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK0.Reg, volatile.LoadUint32(&o.PAGE62_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE62_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE62_LOCK1: Lock configuration MSBs for page 62 (rows 0xf80 through 0xfbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK1.Reg, volatile.LoadUint32(&o.PAGE62_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE62_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK1.Reg, volatile.LoadUint32(&o.PAGE62_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE62_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK1.Reg, volatile.LoadUint32(&o.PAGE62_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE62_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK1.Reg, volatile.LoadUint32(&o.PAGE62_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE62_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE62_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE62_LOCK1.Reg, volatile.LoadUint32(&o.PAGE62_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE62_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE62_LOCK1.Reg) & 0x3
}

// OTP_DATA_RAW.PAGE63_LOCK0: Lock configuration LSBs for page 63 (rows 0xfc0 through 0xfff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK0_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK0.Reg, volatile.LoadUint32(&o.PAGE63_LOCK0.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK0_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE63_LOCK0.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK0_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK0.Reg, volatile.LoadUint32(&o.PAGE63_LOCK0.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK0_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE63_LOCK0.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK0_RMA(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK0.Reg, volatile.LoadUint32(&o.PAGE63_LOCK0.Reg)&^(0x80)|value<<7)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK0_RMA() uint32 {
	return (volatile.LoadUint32(&o.PAGE63_LOCK0.Reg) & 0x80) >> 7
}
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK0_NO_KEY_STATE(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK0.Reg, volatile.LoadUint32(&o.PAGE63_LOCK0.Reg)&^(0x40)|value<<6)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK0_NO_KEY_STATE() uint32 {
	return (volatile.LoadUint32(&o.PAGE63_LOCK0.Reg) & 0x40) >> 6
}
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK0_KEY_R(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK0.Reg, volatile.LoadUint32(&o.PAGE63_LOCK0.Reg)&^(0x38)|value<<3)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK0_KEY_R() uint32 {
	return (volatile.LoadUint32(&o.PAGE63_LOCK0.Reg) & 0x38) >> 3
}
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK0_KEY_W(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK0.Reg, volatile.LoadUint32(&o.PAGE63_LOCK0.Reg)&^(0x7)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK0_KEY_W() uint32 {
	return volatile.LoadUint32(&o.PAGE63_LOCK0.Reg) & 0x7
}

// OTP_DATA_RAW.PAGE63_LOCK1: Lock configuration MSBs for page 63 (rows 0xfc0 through 0xfff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK1_R2(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK1.Reg, volatile.LoadUint32(&o.PAGE63_LOCK1.Reg)&^(0xff0000)|value<<16)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK1_R2() uint32 {
	return (volatile.LoadUint32(&o.PAGE63_LOCK1.Reg) & 0xff0000) >> 16
}
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK1_R1(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK1.Reg, volatile.LoadUint32(&o.PAGE63_LOCK1.Reg)&^(0xff00)|value<<8)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK1_R1() uint32 {
	return (volatile.LoadUint32(&o.PAGE63_LOCK1.Reg) & 0xff00) >> 8
}
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK1_LOCK_BL(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK1.Reg, volatile.LoadUint32(&o.PAGE63_LOCK1.Reg)&^(0x30)|value<<4)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK1_LOCK_BL() uint32 {
	return (volatile.LoadUint32(&o.PAGE63_LOCK1.Reg) & 0x30) >> 4
}
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK1_LOCK_NS(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK1.Reg, volatile.LoadUint32(&o.PAGE63_LOCK1.Reg)&^(0xc)|value<<2)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK1_LOCK_NS() uint32 {
	return (volatile.LoadUint32(&o.PAGE63_LOCK1.Reg) & 0xc) >> 2
}
func (o *OTP_DATA_RAW_Type) SetPAGE63_LOCK1_LOCK_S(value uint32) {
	volatile.StoreUint32(&o.PAGE63_LOCK1.Reg, volatile.LoadUint32(&o.PAGE63_LOCK1.Reg)&^(0x3)|value)
}
func (o *OTP_DATA_RAW_Type) GetPAGE63_LOCK1_LOCK_S() uint32 {
	return volatile.LoadUint32(&o.PAGE63_LOCK1.Reg) & 0x3
}

// For managing simulation testbenches
type TBMAN_Type struct {
	PLATFORM volatile.Register32 // 0x0
}

// TBMAN.PLATFORM: Indicates the type of platform in use
func (o *TBMAN_Type) SetPLATFORM_HDLSIM(value uint32) {
	volatile.StoreUint32(&o.PLATFORM.Reg, volatile.LoadUint32(&o.PLATFORM.Reg)&^(0x4)|value<<2)
}
func (o *TBMAN_Type) GetPLATFORM_HDLSIM() uint32 {
	return (volatile.LoadUint32(&o.PLATFORM.Reg) & 0x4) >> 2
}
func (o *TBMAN_Type) SetPLATFORM_FPGA(value uint32) {
	volatile.StoreUint32(&o.PLATFORM.Reg, volatile.LoadUint32(&o.PLATFORM.Reg)&^(0x2)|value<<1)
}
func (o *TBMAN_Type) GetPLATFORM_FPGA() uint32 {
	return (volatile.LoadUint32(&o.PLATFORM.Reg) & 0x2) >> 1
}
func (o *TBMAN_Type) SetPLATFORM_ASIC(value uint32) {
	volatile.StoreUint32(&o.PLATFORM.Reg, volatile.LoadUint32(&o.PLATFORM.Reg)&^(0x1)|value)
}
func (o *TBMAN_Type) GetPLATFORM_ASIC() uint32 {
	return volatile.LoadUint32(&o.PLATFORM.Reg) & 0x1
}

// DPRAM layout for USB device.
type USB_DPRAM_Type struct {
	SETUP_PACKET_LOW        volatile.Register32 // 0x0
	SETUP_PACKET_HIGH       volatile.Register32 // 0x4
	EP1_IN_CONTROL          volatile.Register32 // 0x8
	EP1_OUT_CONTROL         volatile.Register32 // 0xC
	EP2_IN_CONTROL          volatile.Register32 // 0x10
	EP2_OUT_CONTROL         volatile.Register32 // 0x14
	EP3_IN_CONTROL          volatile.Register32 // 0x18
	EP3_OUT_CONTROL         volatile.Register32 // 0x1C
	EP4_IN_CONTROL          volatile.Register32 // 0x20
	EP4_OUT_CONTROL         volatile.Register32 // 0x24
	EP5_IN_CONTROL          volatile.Register32 // 0x28
	EP5_OUT_CONTROL         volatile.Register32 // 0x2C
	EP6_IN_CONTROL          volatile.Register32 // 0x30
	EP6_OUT_CONTROL         volatile.Register32 // 0x34
	EP7_IN_CONTROL          volatile.Register32 // 0x38
	EP7_OUT_CONTROL         volatile.Register32 // 0x3C
	EP8_IN_CONTROL          volatile.Register32 // 0x40
	EP8_OUT_CONTROL         volatile.Register32 // 0x44
	EP9_IN_CONTROL          volatile.Register32 // 0x48
	EP9_OUT_CONTROL         volatile.Register32 // 0x4C
	EP10_IN_CONTROL         volatile.Register32 // 0x50
	EP10_OUT_CONTROL        volatile.Register32 // 0x54
	EP11_IN_CONTROL         volatile.Register32 // 0x58
	EP11_OUT_CONTROL        volatile.Register32 // 0x5C
	EP12_IN_CONTROL         volatile.Register32 // 0x60
	EP12_OUT_CONTROL        volatile.Register32 // 0x64
	EP13_IN_CONTROL         volatile.Register32 // 0x68
	EP13_OUT_CONTROL        volatile.Register32 // 0x6C
	EP14_IN_CONTROL         volatile.Register32 // 0x70
	EP14_OUT_CONTROL        volatile.Register32 // 0x74
	EP15_IN_CONTROL         volatile.Register32 // 0x78
	EP15_OUT_CONTROL        volatile.Register32 // 0x7C
	EP0_IN_BUFFER_CONTROL   volatile.Register32 // 0x80
	EP0_OUT_BUFFER_CONTROL  volatile.Register32 // 0x84
	EP1_IN_BUFFER_CONTROL   volatile.Register32 // 0x88
	EP1_OUT_BUFFER_CONTROL  volatile.Register32 // 0x8C
	EP2_IN_BUFFER_CONTROL   volatile.Register32 // 0x90
	EP2_OUT_BUFFER_CONTROL  volatile.Register32 // 0x94
	EP3_IN_BUFFER_CONTROL   volatile.Register32 // 0x98
	EP3_OUT_BUFFER_CONTROL  volatile.Register32 // 0x9C
	EP4_IN_BUFFER_CONTROL   volatile.Register32 // 0xA0
	EP4_OUT_BUFFER_CONTROL  volatile.Register32 // 0xA4
	EP5_IN_BUFFER_CONTROL   volatile.Register32 // 0xA8
	EP5_OUT_BUFFER_CONTROL  volatile.Register32 // 0xAC
	EP6_IN_BUFFER_CONTROL   volatile.Register32 // 0xB0
	EP6_OUT_BUFFER_CONTROL  volatile.Register32 // 0xB4
	EP7_IN_BUFFER_CONTROL   volatile.Register32 // 0xB8
	EP7_OUT_BUFFER_CONTROL  volatile.Register32 // 0xBC
	EP8_IN_BUFFER_CONTROL   volatile.Register32 // 0xC0
	EP8_OUT_BUFFER_CONTROL  volatile.Register32 // 0xC4
	EP9_IN_BUFFER_CONTROL   volatile.Register32 // 0xC8
	EP9_OUT_BUFFER_CONTROL  volatile.Register32 // 0xCC
	EP10_IN_BUFFER_CONTROL  volatile.Register32 // 0xD0
	EP10_OUT_BUFFER_CONTROL volatile.Register32 // 0xD4
	EP11_IN_BUFFER_CONTROL  volatile.Register32 // 0xD8
	EP11_OUT_BUFFER_CONTROL volatile.Register32 // 0xDC
	EP12_IN_BUFFER_CONTROL  volatile.Register32 // 0xE0
	EP12_OUT_BUFFER_CONTROL volatile.Register32 // 0xE4
	EP13_IN_BUFFER_CONTROL  volatile.Register32 // 0xE8
	EP13_OUT_BUFFER_CONTROL volatile.Register32 // 0xEC
	EP14_IN_BUFFER_CONTROL  volatile.Register32 // 0xF0
	EP14_OUT_BUFFER_CONTROL volatile.Register32 // 0xF4
	EP15_IN_BUFFER_CONTROL  volatile.Register32 // 0xF8
	EP15_OUT_BUFFER_CONTROL volatile.Register32 // 0xFC
}

// USB_DPRAM.SETUP_PACKET_LOW: Bytes 0-3 of the SETUP packet from the host.
func (o *USB_DPRAM_Type) SetSETUP_PACKET_LOW_WVALUE(value uint32) {
	volatile.StoreUint32(&o.SETUP_PACKET_LOW.Reg, volatile.LoadUint32(&o.SETUP_PACKET_LOW.Reg)&^(0xffff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetSETUP_PACKET_LOW_WVALUE() uint32 {
	return (volatile.LoadUint32(&o.SETUP_PACKET_LOW.Reg) & 0xffff0000) >> 16
}
func (o *USB_DPRAM_Type) SetSETUP_PACKET_LOW_BREQUEST(value uint32) {
	volatile.StoreUint32(&o.SETUP_PACKET_LOW.Reg, volatile.LoadUint32(&o.SETUP_PACKET_LOW.Reg)&^(0xff00)|value<<8)
}
func (o *USB_DPRAM_Type) GetSETUP_PACKET_LOW_BREQUEST() uint32 {
	return (volatile.LoadUint32(&o.SETUP_PACKET_LOW.Reg) & 0xff00) >> 8
}
func (o *USB_DPRAM_Type) SetSETUP_PACKET_LOW_BMREQUESTTYPE(value uint32) {
	volatile.StoreUint32(&o.SETUP_PACKET_LOW.Reg, volatile.LoadUint32(&o.SETUP_PACKET_LOW.Reg)&^(0xff)|value)
}
func (o *USB_DPRAM_Type) GetSETUP_PACKET_LOW_BMREQUESTTYPE() uint32 {
	return volatile.LoadUint32(&o.SETUP_PACKET_LOW.Reg) & 0xff
}

// USB_DPRAM.SETUP_PACKET_HIGH: Bytes 4-7 of the setup packet from the host.
func (o *USB_DPRAM_Type) SetSETUP_PACKET_HIGH_WLENGTH(value uint32) {
	volatile.StoreUint32(&o.SETUP_PACKET_HIGH.Reg, volatile.LoadUint32(&o.SETUP_PACKET_HIGH.Reg)&^(0xffff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetSETUP_PACKET_HIGH_WLENGTH() uint32 {
	return (volatile.LoadUint32(&o.SETUP_PACKET_HIGH.Reg) & 0xffff0000) >> 16
}
func (o *USB_DPRAM_Type) SetSETUP_PACKET_HIGH_WINDEX(value uint32) {
	volatile.StoreUint32(&o.SETUP_PACKET_HIGH.Reg, volatile.LoadUint32(&o.SETUP_PACKET_HIGH.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetSETUP_PACKET_HIGH_WINDEX() uint32 {
	return volatile.LoadUint32(&o.SETUP_PACKET_HIGH.Reg) & 0xffff
}

// USB_DPRAM.EP1_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP1_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP1_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP1_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP1_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP1_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP1_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP1_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP1_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP1_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP1_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP1_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP1_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP1_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP1_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP1_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP1_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP1_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP1_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP1_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP1_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP1_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP1_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP1_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP1_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP1_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP1_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP1_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP2_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP2_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP2_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP2_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP2_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP2_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP2_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP2_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP2_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP2_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP2_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP2_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP2_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP2_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP2_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP2_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP2_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP2_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP2_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP2_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP2_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP2_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP2_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP2_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP2_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP2_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP2_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP2_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP3_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP3_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP3_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP3_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP3_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP3_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP3_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP3_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP3_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP3_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP3_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP3_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP3_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP3_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP3_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP3_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP3_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP3_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP3_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP3_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP3_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP3_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP3_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP3_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP3_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP3_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP3_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP3_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP4_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP4_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP4_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP4_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP4_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP4_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP4_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP4_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP4_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP4_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP4_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP4_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP4_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP4_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP4_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP4_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP4_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP4_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP4_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP4_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP4_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP4_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP4_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP4_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP4_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP4_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP4_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP4_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP5_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP5_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP5_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP5_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP5_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP5_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP5_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP5_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP5_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP5_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP5_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP5_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP5_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP5_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP5_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP5_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP5_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP5_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP5_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP5_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP5_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP5_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP5_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP5_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP5_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP5_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP5_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP5_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP6_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP6_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP6_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP6_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP6_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP6_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP6_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP6_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP6_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP6_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP6_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP6_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP6_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP6_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP6_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP6_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP6_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP6_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP6_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP6_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP6_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP6_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP6_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP6_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP6_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP6_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP6_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP6_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP7_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP7_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP7_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP7_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP7_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP7_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP7_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP7_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP7_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP7_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP7_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP7_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP7_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP7_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP7_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP7_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP7_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP7_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP7_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP7_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP7_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP7_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP7_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP7_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP7_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP7_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP7_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP7_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP8_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP8_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP8_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP8_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP8_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP8_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP8_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP8_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP8_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP8_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP8_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP8_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP8_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP8_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP8_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP8_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP8_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP8_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP8_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP8_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP8_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP8_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP8_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP8_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP8_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP8_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP8_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP8_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP9_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP9_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP9_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP9_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP9_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP9_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP9_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP9_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP9_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP9_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP9_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP9_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP9_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP9_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP9_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP9_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP9_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP9_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP9_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP9_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP9_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP9_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP9_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP9_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP9_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP9_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP9_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP9_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP10_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP10_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP10_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP10_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP10_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP10_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP10_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP10_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP10_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP10_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP10_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP10_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP10_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP10_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP10_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP10_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP10_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP10_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP10_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP10_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP10_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP10_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP10_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP10_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP10_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP10_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP10_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP10_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP11_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP11_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP11_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP11_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP11_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP11_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP11_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP11_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP11_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP11_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP11_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP11_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP11_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP11_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP11_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP11_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP11_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP11_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP11_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP11_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP11_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP11_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP11_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP11_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP11_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP11_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP11_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP11_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP12_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP12_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP12_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP12_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP12_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP12_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP12_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP12_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP12_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP12_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP12_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP12_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP12_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP12_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP12_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP12_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP12_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP12_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP12_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP12_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP12_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP12_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP12_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP12_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP12_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP12_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP12_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP12_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP13_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP13_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP13_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP13_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP13_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP13_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP13_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP13_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP13_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP13_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP13_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP13_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP13_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP13_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP13_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP13_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP13_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP13_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP13_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP13_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP13_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP13_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP13_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP13_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP13_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP13_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP13_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP13_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP14_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP14_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP14_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP14_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP14_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP14_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP14_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP14_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP14_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP14_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP14_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP14_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP14_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP14_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP14_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP14_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP14_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP14_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP14_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP14_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP14_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP14_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP14_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP14_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP14_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP14_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP14_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP14_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP15_IN_CONTROL
func (o *USB_DPRAM_Type) SetEP15_IN_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP15_IN_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP15_IN_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP15_IN_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP15_IN_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP15_IN_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP15_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP15_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP15_IN_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP15_IN_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP15_IN_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP15_IN_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP15_IN_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP15_IN_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP15_IN_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP15_IN_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP15_IN_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP15_OUT_CONTROL
func (o *USB_DPRAM_Type) SetEP15_OUT_CONTROL_ENABLE(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_CONTROL_ENABLE() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP15_OUT_CONTROL_DOUBLE_BUFFERED(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_CONTROL_DOUBLE_BUFFERED() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP15_OUT_CONTROL_INTERRUPT_PER_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_CONTROL_INTERRUPT_PER_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP15_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg)&^(0x10000000)|value<<28)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg) & 0x10000000) >> 28
}
func (o *USB_DPRAM_Type) SetEP15_OUT_CONTROL_ENDPOINT_TYPE(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg)&^(0xc000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_CONTROL_ENDPOINT_TYPE() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg) & 0xc000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP15_OUT_CONTROL_INTERRUPT_ON_STALL(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg)&^(0x20000)|value<<17)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_CONTROL_INTERRUPT_ON_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg) & 0x20000) >> 17
}
func (o *USB_DPRAM_Type) SetEP15_OUT_CONTROL_INTERRUPT_ON_NAK(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg)&^(0x10000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_CONTROL_INTERRUPT_ON_NAK() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg) & 0x10000) >> 16
}
func (o *USB_DPRAM_Type) SetEP15_OUT_CONTROL_BUFFER_ADDRESS(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg)&^(0xffff)|value)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_CONTROL_BUFFER_ADDRESS() uint32 {
	return volatile.LoadUint32(&o.EP15_OUT_CONTROL.Reg) & 0xffff
}

// USB_DPRAM.EP0_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP0_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP0_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP0_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP0_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP0_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP0_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP0_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP0_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP1_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP1_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP1_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP1_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP1_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP1_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP1_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP1_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP1_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP2_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP2_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP2_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP2_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP2_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP2_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP2_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP2_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP2_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP3_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP3_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP3_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP3_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP3_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP3_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP3_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP3_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP3_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP4_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP4_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP4_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP4_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP4_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP4_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP4_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP4_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP4_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP5_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP5_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP5_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP5_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP5_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP5_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP5_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP5_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP5_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP6_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP6_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP6_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP6_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP6_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP6_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP6_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP6_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP6_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP7_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP7_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP7_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP7_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP7_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP7_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP7_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP7_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP7_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP8_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP8_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP8_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP8_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP8_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP8_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP8_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP8_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP8_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP9_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP9_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP9_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP9_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP9_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP9_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP9_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP9_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP9_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP10_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP10_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP10_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP10_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP10_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP10_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP10_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP10_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP10_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP11_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP11_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP11_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP11_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP11_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP11_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP11_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP11_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP11_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP12_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP12_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP12_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP12_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP12_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP12_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP12_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP12_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP12_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP13_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP13_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP13_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP13_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP13_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP13_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP13_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP13_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP13_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP14_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP14_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP14_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP14_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP14_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP14_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP14_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP14_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP14_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP15_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP15_IN_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP15_IN_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP15_IN_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP15_IN_BUFFER_CONTROL.Reg) & 0x3ff
}

// USB_DPRAM.EP15_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_FULL_1(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x80000000)|value<<31)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_FULL_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x80000000) >> 31
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_LAST_1(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x40000000)|value<<30)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_LAST_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x40000000) >> 30
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_PID_1(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x20000000)|value<<29)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_PID_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x20000000) >> 29
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x18000000)|value<<27)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x18000000) >> 27
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_AVAILABLE_1(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x4000000)|value<<26)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_AVAILABLE_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x4000000) >> 26
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_LENGTH_1(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x3ff0000)|value<<16)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_LENGTH_1() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x3ff0000) >> 16
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_FULL_0(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x8000)|value<<15)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_FULL_0() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x8000) >> 15
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_LAST_0(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x4000)|value<<14)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_LAST_0() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x4000) >> 14
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_PID_0(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x2000)|value<<13)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_PID_0() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x2000) >> 13
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_RESET(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x1000)|value<<12)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_RESET() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x1000) >> 12
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_STALL(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x800)|value<<11)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_STALL() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x800) >> 11
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_AVAILABLE_0(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x400)|value<<10)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_AVAILABLE_0() uint32 {
	return (volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x400) >> 10
}
func (o *USB_DPRAM_Type) SetEP15_OUT_BUFFER_CONTROL_LENGTH_0(value uint32) {
	volatile.StoreUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg, volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg)&^(0x3ff)|value)
}
func (o *USB_DPRAM_Type) GetEP15_OUT_BUFFER_CONTROL_LENGTH_0() uint32 {
	return volatile.LoadUint32(&o.EP15_OUT_BUFFER_CONTROL.Reg) & 0x3ff
}

// Constants for RESETS
const (
	// RESET
	// Position of USBCTRL field.
	RESETS_RESET_USBCTRL_Pos = 0x1c
	// Bit mask of USBCTRL field.
	RESETS_RESET_USBCTRL_Msk = 0x10000000
	// Bit USBCTRL.
	RESETS_RESET_USBCTRL = 0x10000000
	// Position of UART1 field.
	RESETS_RESET_UART1_Pos = 0x1b
	// Bit mask of UART1 field.
	RESETS_RESET_UART1_Msk = 0x8000000
	// Bit UART1.
	RESETS_RESET_UART1 = 0x8000000
	// Position of UART0 field.
	RESETS_RESET_UART0_Pos = 0x1a
	// Bit mask of UART0 field.
	RESETS_RESET_UART0_Msk = 0x4000000
	// Bit UART0.
	RESETS_RESET_UART0 = 0x4000000
	// Position of TRNG field.
	RESETS_RESET_TRNG_Pos = 0x19
	// Bit mask of TRNG field.
	RESETS_RESET_TRNG_Msk = 0x2000000
	// Bit TRNG.
	RESETS_RESET_TRNG = 0x2000000
	// Position of TIMER1 field.
	RESETS_RESET_TIMER1_Pos = 0x18
	// Bit mask of TIMER1 field.
	RESETS_RESET_TIMER1_Msk = 0x1000000
	// Bit TIMER1.
	RESETS_RESET_TIMER1 = 0x1000000
	// Position of TIMER0 field.
	RESETS_RESET_TIMER0_Pos = 0x17
	// Bit mask of TIMER0 field.
	RESETS_RESET_TIMER0_Msk = 0x800000
	// Bit TIMER0.
	RESETS_RESET_TIMER0 = 0x800000
	// Position of TBMAN field.
	RESETS_RESET_TBMAN_Pos = 0x16
	// Bit mask of TBMAN field.
	RESETS_RESET_TBMAN_Msk = 0x400000
	// Bit TBMAN.
	RESETS_RESET_TBMAN = 0x400000
	// Position of SYSINFO field.
	RESETS_RESET_SYSINFO_Pos = 0x15
	// Bit mask of SYSINFO field.
	RESETS_RESET_SYSINFO_Msk = 0x200000
	// Bit SYSINFO.
	RESETS_RESET_SYSINFO = 0x200000
	// Position of SYSCFG field.
	RESETS_RESET_SYSCFG_Pos = 0x14
	// Bit mask of SYSCFG field.
	RESETS_RESET_SYSCFG_Msk = 0x100000
	// Bit SYSCFG.
	RESETS_RESET_SYSCFG = 0x100000
	// Position of SPI1 field.
	RESETS_RESET_SPI1_Pos = 0x13
	// Bit mask of SPI1 field.
	RESETS_RESET_SPI1_Msk = 0x80000
	// Bit SPI1.
	RESETS_RESET_SPI1 = 0x80000
	// Position of SPI0 field.
	RESETS_RESET_SPI0_Pos = 0x12
	// Bit mask of SPI0 field.
	RESETS_RESET_SPI0_Msk = 0x40000
	// Bit SPI0.
	RESETS_RESET_SPI0 = 0x40000
	// Position of SHA256 field.
	RESETS_RESET_SHA256_Pos = 0x11
	// Bit mask of SHA256 field.
	RESETS_RESET_SHA256_Msk = 0x20000
	// Bit SHA256.
	RESETS_RESET_SHA256 = 0x20000
	// Position of PWM field.
	RESETS_RESET_PWM_Pos = 0x10
	// Bit mask of PWM field.
	RESETS_RESET_PWM_Msk = 0x10000
	// Bit PWM.
	RESETS_RESET_PWM = 0x10000
	// Position of PLL_USB field.
	RESETS_RESET_PLL_USB_Pos = 0xf
	// Bit mask of PLL_USB field.
	RESETS_RESET_PLL_USB_Msk = 0x8000
	// Bit PLL_USB.
	RESETS_RESET_PLL_USB = 0x8000
	// Position of PLL_SYS field.
	RESETS_RESET_PLL_SYS_Pos = 0xe
	// Bit mask of PLL_SYS field.
	RESETS_RESET_PLL_SYS_Msk = 0x4000
	// Bit PLL_SYS.
	RESETS_RESET_PLL_SYS = 0x4000
	// Position of PIO2 field.
	RESETS_RESET_PIO2_Pos = 0xd
	// Bit mask of PIO2 field.
	RESETS_RESET_PIO2_Msk = 0x2000
	// Bit PIO2.
	RESETS_RESET_PIO2 = 0x2000
	// Position of PIO1 field.
	RESETS_RESET_PIO1_Pos = 0xc
	// Bit mask of PIO1 field.
	RESETS_RESET_PIO1_Msk = 0x1000
	// Bit PIO1.
	RESETS_RESET_PIO1 = 0x1000
	// Position of PIO0 field.
	RESETS_RESET_PIO0_Pos = 0xb
	// Bit mask of PIO0 field.
	RESETS_RESET_PIO0_Msk = 0x800
	// Bit PIO0.
	RESETS_RESET_PIO0 = 0x800
	// Position of PADS_QSPI field.
	RESETS_RESET_PADS_QSPI_Pos = 0xa
	// Bit mask of PADS_QSPI field.
	RESETS_RESET_PADS_QSPI_Msk = 0x400
	// Bit PADS_QSPI.
	RESETS_RESET_PADS_QSPI = 0x400
	// Position of PADS_BANK0 field.
	RESETS_RESET_PADS_BANK0_Pos = 0x9
	// Bit mask of PADS_BANK0 field.
	RESETS_RESET_PADS_BANK0_Msk = 0x200
	// Bit PADS_BANK0.
	RESETS_RESET_PADS_BANK0 = 0x200
	// Position of JTAG field.
	RESETS_RESET_JTAG_Pos = 0x8
	// Bit mask of JTAG field.
	RESETS_RESET_JTAG_Msk = 0x100
	// Bit JTAG.
	RESETS_RESET_JTAG = 0x100
	// Position of IO_QSPI field.
	RESETS_RESET_IO_QSPI_Pos = 0x7
	// Bit mask of IO_QSPI field.
	RESETS_RESET_IO_QSPI_Msk = 0x80
	// Bit IO_QSPI.
	RESETS_RESET_IO_QSPI = 0x80
	// Position of IO_BANK0 field.
	RESETS_RESET_IO_BANK0_Pos = 0x6
	// Bit mask of IO_BANK0 field.
	RESETS_RESET_IO_BANK0_Msk = 0x40
	// Bit IO_BANK0.
	RESETS_RESET_IO_BANK0 = 0x40
	// Position of I2C1 field.
	RESETS_RESET_I2C1_Pos = 0x5
	// Bit mask of I2C1 field.
	RESETS_RESET_I2C1_Msk = 0x20
	// Bit I2C1.
	RESETS_RESET_I2C1 = 0x20
	// Position of I2C0 field.
	RESETS_RESET_I2C0_Pos = 0x4
	// Bit mask of I2C0 field.
	RESETS_RESET_I2C0_Msk = 0x10
	// Bit I2C0.
	RESETS_RESET_I2C0 = 0x10
	// Position of HSTX field.
	RESETS_RESET_HSTX_Pos = 0x3
	// Bit mask of HSTX field.
	RESETS_RESET_HSTX_Msk = 0x8
	// Bit HSTX.
	RESETS_RESET_HSTX = 0x8
	// Position of DMA field.
	RESETS_RESET_DMA_Pos = 0x2
	// Bit mask of DMA field.
	RESETS_RESET_DMA_Msk = 0x4
	// Bit DMA.
	RESETS_RESET_DMA = 0x4
	// Position of BUSCTRL field.
	RESETS_RESET_BUSCTRL_Pos = 0x1
	// Bit mask of BUSCTRL field.
	RESETS_RESET_BUSCTRL_Msk = 0x2
	// Bit BUSCTRL.
	RESETS_RESET_BUSCTRL = 0x2
	// Position of ADC field.
	RESETS_RESET_ADC_Pos = 0x0
	// Bit mask of ADC field.
	RESETS_RESET_ADC_Msk = 0x1
	// Bit ADC.
	RESETS_RESET_ADC = 0x1

	// WDSEL
	// Position of USBCTRL field.
	RESETS_WDSEL_USBCTRL_Pos = 0x1c
	// Bit mask of USBCTRL field.
	RESETS_WDSEL_USBCTRL_Msk = 0x10000000
	// Bit USBCTRL.
	RESETS_WDSEL_USBCTRL = 0x10000000
	// Position of UART1 field.
	RESETS_WDSEL_UART1_Pos = 0x1b
	// Bit mask of UART1 field.
	RESETS_WDSEL_UART1_Msk = 0x8000000
	// Bit UART1.
	RESETS_WDSEL_UART1 = 0x8000000
	// Position of UART0 field.
	RESETS_WDSEL_UART0_Pos = 0x1a
	// Bit mask of UART0 field.
	RESETS_WDSEL_UART0_Msk = 0x4000000
	// Bit UART0.
	RESETS_WDSEL_UART0 = 0x4000000
	// Position of TRNG field.
	RESETS_WDSEL_TRNG_Pos = 0x19
	// Bit mask of TRNG field.
	RESETS_WDSEL_TRNG_Msk = 0x2000000
	// Bit TRNG.
	RESETS_WDSEL_TRNG = 0x2000000
	// Position of TIMER1 field.
	RESETS_WDSEL_TIMER1_Pos = 0x18
	// Bit mask of TIMER1 field.
	RESETS_WDSEL_TIMER1_Msk = 0x1000000
	// Bit TIMER1.
	RESETS_WDSEL_TIMER1 = 0x1000000
	// Position of TIMER0 field.
	RESETS_WDSEL_TIMER0_Pos = 0x17
	// Bit mask of TIMER0 field.
	RESETS_WDSEL_TIMER0_Msk = 0x800000
	// Bit TIMER0.
	RESETS_WDSEL_TIMER0 = 0x800000
	// Position of TBMAN field.
	RESETS_WDSEL_TBMAN_Pos = 0x16
	// Bit mask of TBMAN field.
	RESETS_WDSEL_TBMAN_Msk = 0x400000
	// Bit TBMAN.
	RESETS_WDSEL_TBMAN = 0x400000
	// Position of SYSINFO field.
	RESETS_WDSEL_SYSINFO_Pos = 0x15
	// Bit mask of SYSINFO field.
	RESETS_WDSEL_SYSINFO_Msk = 0x200000
	// Bit SYSINFO.
	RESETS_WDSEL_SYSINFO = 0x200000
	// Position of SYSCFG field.
	RESETS_WDSEL_SYSCFG_Pos = 0x14
	// Bit mask of SYSCFG field.
	RESETS_WDSEL_SYSCFG_Msk = 0x100000
	// Bit SYSCFG.
	RESETS_WDSEL_SYSCFG = 0x100000
	// Position of SPI1 field.
	RESETS_WDSEL_SPI1_Pos = 0x13
	// Bit mask of SPI1 field.
	RESETS_WDSEL_SPI1_Msk = 0x80000
	// Bit SPI1.
	RESETS_WDSEL_SPI1 = 0x80000
	// Position of SPI0 field.
	RESETS_WDSEL_SPI0_Pos = 0x12
	// Bit mask of SPI0 field.
	RESETS_WDSEL_SPI0_Msk = 0x40000
	// Bit SPI0.
	RESETS_WDSEL_SPI0 = 0x40000
	// Position of SHA256 field.
	RESETS_WDSEL_SHA256_Pos = 0x11
	// Bit mask of SHA256 field.
	RESETS_WDSEL_SHA256_Msk = 0x20000
	// Bit SHA256.
	RESETS_WDSEL_SHA256 = 0x20000
	// Position of PWM field.
	RESETS_WDSEL_PWM_Pos = 0x10
	// Bit mask of PWM field.
	RESETS_WDSEL_PWM_Msk = 0x10000
	// Bit PWM.
	RESETS_WDSEL_PWM = 0x10000
	// Position of PLL_USB field.
	RESETS_WDSEL_PLL_USB_Pos = 0xf
	// Bit mask of PLL_USB field.
	RESETS_WDSEL_PLL_USB_Msk = 0x8000
	// Bit PLL_USB.
	RESETS_WDSEL_PLL_USB = 0x8000
	// Position of PLL_SYS field.
	RESETS_WDSEL_PLL_SYS_Pos = 0xe
	// Bit mask of PLL_SYS field.
	RESETS_WDSEL_PLL_SYS_Msk = 0x4000
	// Bit PLL_SYS.
	RESETS_WDSEL_PLL_SYS = 0x4000
	// Position of PIO2 field.
	RESETS_WDSEL_PIO2_Pos = 0xd
	// Bit mask of PIO2 field.
	RESETS_WDSEL_PIO2_Msk = 0x2000
	// Bit PIO2.
	RESETS_WDSEL_PIO2 = 0x2000
	// Position of PIO1 field.
	RESETS_WDSEL_PIO1_Pos = 0xc
	// Bit mask of PIO1 field.
	RESETS_WDSEL_PIO1_Msk = 0x1000
	// Bit PIO1.
	RESETS_WDSEL_PIO1 = 0x1000
	// Position of PIO0 field.
	RESETS_WDSEL_PIO0_Pos = 0xb
	// Bit mask of PIO0 field.
	RESETS_WDSEL_PIO0_Msk = 0x800
	// Bit PIO0.
	RESETS_WDSEL_PIO0 = 0x800
	// Position of PADS_QSPI field.
	RESETS_WDSEL_PADS_QSPI_Pos = 0xa
	// Bit mask of PADS_QSPI field.
	RESETS_WDSEL_PADS_QSPI_Msk = 0x400
	// Bit PADS_QSPI.
	RESETS_WDSEL_PADS_QSPI = 0x400
	// Position of PADS_BANK0 field.
	RESETS_WDSEL_PADS_BANK0_Pos = 0x9
	// Bit mask of PADS_BANK0 field.
	RESETS_WDSEL_PADS_BANK0_Msk = 0x200
	// Bit PADS_BANK0.
	RESETS_WDSEL_PADS_BANK0 = 0x200
	// Position of JTAG field.
	RESETS_WDSEL_JTAG_Pos = 0x8
	// Bit mask of JTAG field.
	RESETS_WDSEL_JTAG_Msk = 0x100
	// Bit JTAG.
	RESETS_WDSEL_JTAG = 0x100
	// Position of IO_QSPI field.
	RESETS_WDSEL_IO_QSPI_Pos = 0x7
	// Bit mask of IO_QSPI field.
	RESETS_WDSEL_IO_QSPI_Msk = 0x80
	// Bit IO_QSPI.
	RESETS_WDSEL_IO_QSPI = 0x80
	// Position of IO_BANK0 field.
	RESETS_WDSEL_IO_BANK0_Pos = 0x6
	// Bit mask of IO_BANK0 field.
	RESETS_WDSEL_IO_BANK0_Msk = 0x40
	// Bit IO_BANK0.
	RESETS_WDSEL_IO_BANK0 = 0x40
	// Position of I2C1 field.
	RESETS_WDSEL_I2C1_Pos = 0x5
	// Bit mask of I2C1 field.
	RESETS_WDSEL_I2C1_Msk = 0x20
	// Bit I2C1.
	RESETS_WDSEL_I2C1 = 0x20
	// Position of I2C0 field.
	RESETS_WDSEL_I2C0_Pos = 0x4
	// Bit mask of I2C0 field.
	RESETS_WDSEL_I2C0_Msk = 0x10
	// Bit I2C0.
	RESETS_WDSEL_I2C0 = 0x10
	// Position of HSTX field.
	RESETS_WDSEL_HSTX_Pos = 0x3
	// Bit mask of HSTX field.
	RESETS_WDSEL_HSTX_Msk = 0x8
	// Bit HSTX.
	RESETS_WDSEL_HSTX = 0x8
	// Position of DMA field.
	RESETS_WDSEL_DMA_Pos = 0x2
	// Bit mask of DMA field.
	RESETS_WDSEL_DMA_Msk = 0x4
	// Bit DMA.
	RESETS_WDSEL_DMA = 0x4
	// Position of BUSCTRL field.
	RESETS_WDSEL_BUSCTRL_Pos = 0x1
	// Bit mask of BUSCTRL field.
	RESETS_WDSEL_BUSCTRL_Msk = 0x2
	// Bit BUSCTRL.
	RESETS_WDSEL_BUSCTRL = 0x2
	// Position of ADC field.
	RESETS_WDSEL_ADC_Pos = 0x0
	// Bit mask of ADC field.
	RESETS_WDSEL_ADC_Msk = 0x1
	// Bit ADC.
	RESETS_WDSEL_ADC = 0x1

	// RESET_DONE
	// Position of USBCTRL field.
	RESETS_RESET_DONE_USBCTRL_Pos = 0x1c
	// Bit mask of USBCTRL field.
	RESETS_RESET_DONE_USBCTRL_Msk = 0x10000000
	// Bit USBCTRL.
	RESETS_RESET_DONE_USBCTRL = 0x10000000
	// Position of UART1 field.
	RESETS_RESET_DONE_UART1_Pos = 0x1b
	// Bit mask of UART1 field.
	RESETS_RESET_DONE_UART1_Msk = 0x8000000
	// Bit UART1.
	RESETS_RESET_DONE_UART1 = 0x8000000
	// Position of UART0 field.
	RESETS_RESET_DONE_UART0_Pos = 0x1a
	// Bit mask of UART0 field.
	RESETS_RESET_DONE_UART0_Msk = 0x4000000
	// Bit UART0.
	RESETS_RESET_DONE_UART0 = 0x4000000
	// Position of TRNG field.
	RESETS_RESET_DONE_TRNG_Pos = 0x19
	// Bit mask of TRNG field.
	RESETS_RESET_DONE_TRNG_Msk = 0x2000000
	// Bit TRNG.
	RESETS_RESET_DONE_TRNG = 0x2000000
	// Position of TIMER1 field.
	RESETS_RESET_DONE_TIMER1_Pos = 0x18
	// Bit mask of TIMER1 field.
	RESETS_RESET_DONE_TIMER1_Msk = 0x1000000
	// Bit TIMER1.
	RESETS_RESET_DONE_TIMER1 = 0x1000000
	// Position of TIMER0 field.
	RESETS_RESET_DONE_TIMER0_Pos = 0x17
	// Bit mask of TIMER0 field.
	RESETS_RESET_DONE_TIMER0_Msk = 0x800000
	// Bit TIMER0.
	RESETS_RESET_DONE_TIMER0 = 0x800000
	// Position of TBMAN field.
	RESETS_RESET_DONE_TBMAN_Pos = 0x16
	// Bit mask of TBMAN field.
	RESETS_RESET_DONE_TBMAN_Msk = 0x400000
	// Bit TBMAN.
	RESETS_RESET_DONE_TBMAN = 0x400000
	// Position of SYSINFO field.
	RESETS_RESET_DONE_SYSINFO_Pos = 0x15
	// Bit mask of SYSINFO field.
	RESETS_RESET_DONE_SYSINFO_Msk = 0x200000
	// Bit SYSINFO.
	RESETS_RESET_DONE_SYSINFO = 0x200000
	// Position of SYSCFG field.
	RESETS_RESET_DONE_SYSCFG_Pos = 0x14
	// Bit mask of SYSCFG field.
	RESETS_RESET_DONE_SYSCFG_Msk = 0x100000
	// Bit SYSCFG.
	RESETS_RESET_DONE_SYSCFG = 0x100000
	// Position of SPI1 field.
	RESETS_RESET_DONE_SPI1_Pos = 0x13
	// Bit mask of SPI1 field.
	RESETS_RESET_DONE_SPI1_Msk = 0x80000
	// Bit SPI1.
	RESETS_RESET_DONE_SPI1 = 0x80000
	// Position of SPI0 field.
	RESETS_RESET_DONE_SPI0_Pos = 0x12
	// Bit mask of SPI0 field.
	RESETS_RESET_DONE_SPI0_Msk = 0x40000
	// Bit SPI0.
	RESETS_RESET_DONE_SPI0 = 0x40000
	// Position of SHA256 field.
	RESETS_RESET_DONE_SHA256_Pos = 0x11
	// Bit mask of SHA256 field.
	RESETS_RESET_DONE_SHA256_Msk = 0x20000
	// Bit SHA256.
	RESETS_RESET_DONE_SHA256 = 0x20000
	// Position of PWM field.
	RESETS_RESET_DONE_PWM_Pos = 0x10
	// Bit mask of PWM field.
	RESETS_RESET_DONE_PWM_Msk = 0x10000
	// Bit PWM.
	RESETS_RESET_DONE_PWM = 0x10000
	// Position of PLL_USB field.
	RESETS_RESET_DONE_PLL_USB_Pos = 0xf
	// Bit mask of PLL_USB field.
	RESETS_RESET_DONE_PLL_USB_Msk = 0x8000
	// Bit PLL_USB.
	RESETS_RESET_DONE_PLL_USB = 0x8000
	// Position of PLL_SYS field.
	RESETS_RESET_DONE_PLL_SYS_Pos = 0xe
	// Bit mask of PLL_SYS field.
	RESETS_RESET_DONE_PLL_SYS_Msk = 0x4000
	// Bit PLL_SYS.
	RESETS_RESET_DONE_PLL_SYS = 0x4000
	// Position of PIO2 field.
	RESETS_RESET_DONE_PIO2_Pos = 0xd
	// Bit mask of PIO2 field.
	RESETS_RESET_DONE_PIO2_Msk = 0x2000
	// Bit PIO2.
	RESETS_RESET_DONE_PIO2 = 0x2000
	// Position of PIO1 field.
	RESETS_RESET_DONE_PIO1_Pos = 0xc
	// Bit mask of PIO1 field.
	RESETS_RESET_DONE_PIO1_Msk = 0x1000
	// Bit PIO1.
	RESETS_RESET_DONE_PIO1 = 0x1000
	// Position of PIO0 field.
	RESETS_RESET_DONE_PIO0_Pos = 0xb
	// Bit mask of PIO0 field.
	RESETS_RESET_DONE_PIO0_Msk = 0x800
	// Bit PIO0.
	RESETS_RESET_DONE_PIO0 = 0x800
	// Position of PADS_QSPI field.
	RESETS_RESET_DONE_PADS_QSPI_Pos = 0xa
	// Bit mask of PADS_QSPI field.
	RESETS_RESET_DONE_PADS_QSPI_Msk = 0x400
	// Bit PADS_QSPI.
	RESETS_RESET_DONE_PADS_QSPI = 0x400
	// Position of PADS_BANK0 field.
	RESETS_RESET_DONE_PADS_BANK0_Pos = 0x9
	// Bit mask of PADS_BANK0 field.
	RESETS_RESET_DONE_PADS_BANK0_Msk = 0x200
	// Bit PADS_BANK0.
	RESETS_RESET_DONE_PADS_BANK0 = 0x200
	// Position of JTAG field.
	RESETS_RESET_DONE_JTAG_Pos = 0x8
	// Bit mask of JTAG field.
	RESETS_RESET_DONE_JTAG_Msk = 0x100
	// Bit JTAG.
	RESETS_RESET_DONE_JTAG = 0x100
	// Position of IO_QSPI field.
	RESETS_RESET_DONE_IO_QSPI_Pos = 0x7
	// Bit mask of IO_QSPI field.
	RESETS_RESET_DONE_IO_QSPI_Msk = 0x80
	// Bit IO_QSPI.
	RESETS_RESET_DONE_IO_QSPI = 0x80
	// Position of IO_BANK0 field.
	RESETS_RESET_DONE_IO_BANK0_Pos = 0x6
	// Bit mask of IO_BANK0 field.
	RESETS_RESET_DONE_IO_BANK0_Msk = 0x40
	// Bit IO_BANK0.
	RESETS_RESET_DONE_IO_BANK0 = 0x40
	// Position of I2C1 field.
	RESETS_RESET_DONE_I2C1_Pos = 0x5
	// Bit mask of I2C1 field.
	RESETS_RESET_DONE_I2C1_Msk = 0x20
	// Bit I2C1.
	RESETS_RESET_DONE_I2C1 = 0x20
	// Position of I2C0 field.
	RESETS_RESET_DONE_I2C0_Pos = 0x4
	// Bit mask of I2C0 field.
	RESETS_RESET_DONE_I2C0_Msk = 0x10
	// Bit I2C0.
	RESETS_RESET_DONE_I2C0 = 0x10
	// Position of HSTX field.
	RESETS_RESET_DONE_HSTX_Pos = 0x3
	// Bit mask of HSTX field.
	RESETS_RESET_DONE_HSTX_Msk = 0x8
	// Bit HSTX.
	RESETS_RESET_DONE_HSTX = 0x8
	// Position of DMA field.
	RESETS_RESET_DONE_DMA_Pos = 0x2
	// Bit mask of DMA field.
	RESETS_RESET_DONE_DMA_Msk = 0x4
	// Bit DMA.
	RESETS_RESET_DONE_DMA = 0x4
	// Position of BUSCTRL field.
	RESETS_RESET_DONE_BUSCTRL_Pos = 0x1
	// Bit mask of BUSCTRL field.
	RESETS_RESET_DONE_BUSCTRL_Msk = 0x2
	// Bit BUSCTRL.
	RESETS_RESET_DONE_BUSCTRL = 0x2
	// Position of ADC field.
	RESETS_RESET_DONE_ADC_Pos = 0x0
	// Bit mask of ADC field.
	RESETS_RESET_DONE_ADC_Msk = 0x1
	// Bit ADC.
	RESETS_RESET_DONE_ADC = 0x1
)

// Constants for PSM
const (
	// FRCE_ON: Force block out of reset (i.e. power it on)
	// Position of PROC1 field.
	PSM_FRCE_ON_PROC1_Pos = 0x18
	// Bit mask of PROC1 field.
	PSM_FRCE_ON_PROC1_Msk = 0x1000000
	// Bit PROC1.
	PSM_FRCE_ON_PROC1 = 0x1000000
	// Position of PROC0 field.
	PSM_FRCE_ON_PROC0_Pos = 0x17
	// Bit mask of PROC0 field.
	PSM_FRCE_ON_PROC0_Msk = 0x800000
	// Bit PROC0.
	PSM_FRCE_ON_PROC0 = 0x800000
	// Position of ACCESSCTRL field.
	PSM_FRCE_ON_ACCESSCTRL_Pos = 0x16
	// Bit mask of ACCESSCTRL field.
	PSM_FRCE_ON_ACCESSCTRL_Msk = 0x400000
	// Bit ACCESSCTRL.
	PSM_FRCE_ON_ACCESSCTRL = 0x400000
	// Position of SIO field.
	PSM_FRCE_ON_SIO_Pos = 0x15
	// Bit mask of SIO field.
	PSM_FRCE_ON_SIO_Msk = 0x200000
	// Bit SIO.
	PSM_FRCE_ON_SIO = 0x200000
	// Position of XIP field.
	PSM_FRCE_ON_XIP_Pos = 0x14
	// Bit mask of XIP field.
	PSM_FRCE_ON_XIP_Msk = 0x100000
	// Bit XIP.
	PSM_FRCE_ON_XIP = 0x100000
	// Position of SRAM9 field.
	PSM_FRCE_ON_SRAM9_Pos = 0x13
	// Bit mask of SRAM9 field.
	PSM_FRCE_ON_SRAM9_Msk = 0x80000
	// Bit SRAM9.
	PSM_FRCE_ON_SRAM9 = 0x80000
	// Position of SRAM8 field.
	PSM_FRCE_ON_SRAM8_Pos = 0x12
	// Bit mask of SRAM8 field.
	PSM_FRCE_ON_SRAM8_Msk = 0x40000
	// Bit SRAM8.
	PSM_FRCE_ON_SRAM8 = 0x40000
	// Position of SRAM7 field.
	PSM_FRCE_ON_SRAM7_Pos = 0x11
	// Bit mask of SRAM7 field.
	PSM_FRCE_ON_SRAM7_Msk = 0x20000
	// Bit SRAM7.
	PSM_FRCE_ON_SRAM7 = 0x20000
	// Position of SRAM6 field.
	PSM_FRCE_ON_SRAM6_Pos = 0x10
	// Bit mask of SRAM6 field.
	PSM_FRCE_ON_SRAM6_Msk = 0x10000
	// Bit SRAM6.
	PSM_FRCE_ON_SRAM6 = 0x10000
	// Position of SRAM5 field.
	PSM_FRCE_ON_SRAM5_Pos = 0xf
	// Bit mask of SRAM5 field.
	PSM_FRCE_ON_SRAM5_Msk = 0x8000
	// Bit SRAM5.
	PSM_FRCE_ON_SRAM5 = 0x8000
	// Position of SRAM4 field.
	PSM_FRCE_ON_SRAM4_Pos = 0xe
	// Bit mask of SRAM4 field.
	PSM_FRCE_ON_SRAM4_Msk = 0x4000
	// Bit SRAM4.
	PSM_FRCE_ON_SRAM4 = 0x4000
	// Position of SRAM3 field.
	PSM_FRCE_ON_SRAM3_Pos = 0xd
	// Bit mask of SRAM3 field.
	PSM_FRCE_ON_SRAM3_Msk = 0x2000
	// Bit SRAM3.
	PSM_FRCE_ON_SRAM3 = 0x2000
	// Position of SRAM2 field.
	PSM_FRCE_ON_SRAM2_Pos = 0xc
	// Bit mask of SRAM2 field.
	PSM_FRCE_ON_SRAM2_Msk = 0x1000
	// Bit SRAM2.
	PSM_FRCE_ON_SRAM2 = 0x1000
	// Position of SRAM1 field.
	PSM_FRCE_ON_SRAM1_Pos = 0xb
	// Bit mask of SRAM1 field.
	PSM_FRCE_ON_SRAM1_Msk = 0x800
	// Bit SRAM1.
	PSM_FRCE_ON_SRAM1 = 0x800
	// Position of SRAM0 field.
	PSM_FRCE_ON_SRAM0_Pos = 0xa
	// Bit mask of SRAM0 field.
	PSM_FRCE_ON_SRAM0_Msk = 0x400
	// Bit SRAM0.
	PSM_FRCE_ON_SRAM0 = 0x400
	// Position of BOOTRAM field.
	PSM_FRCE_ON_BOOTRAM_Pos = 0x9
	// Bit mask of BOOTRAM field.
	PSM_FRCE_ON_BOOTRAM_Msk = 0x200
	// Bit BOOTRAM.
	PSM_FRCE_ON_BOOTRAM = 0x200
	// Position of ROM field.
	PSM_FRCE_ON_ROM_Pos = 0x8
	// Bit mask of ROM field.
	PSM_FRCE_ON_ROM_Msk = 0x100
	// Bit ROM.
	PSM_FRCE_ON_ROM = 0x100
	// Position of BUSFABRIC field.
	PSM_FRCE_ON_BUSFABRIC_Pos = 0x7
	// Bit mask of BUSFABRIC field.
	PSM_FRCE_ON_BUSFABRIC_Msk = 0x80
	// Bit BUSFABRIC.
	PSM_FRCE_ON_BUSFABRIC = 0x80
	// Position of PSM_READY field.
	PSM_FRCE_ON_PSM_READY_Pos = 0x6
	// Bit mask of PSM_READY field.
	PSM_FRCE_ON_PSM_READY_Msk = 0x40
	// Bit PSM_READY.
	PSM_FRCE_ON_PSM_READY = 0x40
	// Position of CLOCKS field.
	PSM_FRCE_ON_CLOCKS_Pos = 0x5
	// Bit mask of CLOCKS field.
	PSM_FRCE_ON_CLOCKS_Msk = 0x20
	// Bit CLOCKS.
	PSM_FRCE_ON_CLOCKS = 0x20
	// Position of RESETS field.
	PSM_FRCE_ON_RESETS_Pos = 0x4
	// Bit mask of RESETS field.
	PSM_FRCE_ON_RESETS_Msk = 0x10
	// Bit RESETS.
	PSM_FRCE_ON_RESETS = 0x10
	// Position of XOSC field.
	PSM_FRCE_ON_XOSC_Pos = 0x3
	// Bit mask of XOSC field.
	PSM_FRCE_ON_XOSC_Msk = 0x8
	// Bit XOSC.
	PSM_FRCE_ON_XOSC = 0x8
	// Position of ROSC field.
	PSM_FRCE_ON_ROSC_Pos = 0x2
	// Bit mask of ROSC field.
	PSM_FRCE_ON_ROSC_Msk = 0x4
	// Bit ROSC.
	PSM_FRCE_ON_ROSC = 0x4
	// Position of OTP field.
	PSM_FRCE_ON_OTP_Pos = 0x1
	// Bit mask of OTP field.
	PSM_FRCE_ON_OTP_Msk = 0x2
	// Bit OTP.
	PSM_FRCE_ON_OTP = 0x2
	// Position of PROC_COLD field.
	PSM_FRCE_ON_PROC_COLD_Pos = 0x0
	// Bit mask of PROC_COLD field.
	PSM_FRCE_ON_PROC_COLD_Msk = 0x1
	// Bit PROC_COLD.
	PSM_FRCE_ON_PROC_COLD = 0x1

	// FRCE_OFF: Force into reset (i.e. power it off)
	// Position of PROC1 field.
	PSM_FRCE_OFF_PROC1_Pos = 0x18
	// Bit mask of PROC1 field.
	PSM_FRCE_OFF_PROC1_Msk = 0x1000000
	// Bit PROC1.
	PSM_FRCE_OFF_PROC1 = 0x1000000
	// Position of PROC0 field.
	PSM_FRCE_OFF_PROC0_Pos = 0x17
	// Bit mask of PROC0 field.
	PSM_FRCE_OFF_PROC0_Msk = 0x800000
	// Bit PROC0.
	PSM_FRCE_OFF_PROC0 = 0x800000
	// Position of ACCESSCTRL field.
	PSM_FRCE_OFF_ACCESSCTRL_Pos = 0x16
	// Bit mask of ACCESSCTRL field.
	PSM_FRCE_OFF_ACCESSCTRL_Msk = 0x400000
	// Bit ACCESSCTRL.
	PSM_FRCE_OFF_ACCESSCTRL = 0x400000
	// Position of SIO field.
	PSM_FRCE_OFF_SIO_Pos = 0x15
	// Bit mask of SIO field.
	PSM_FRCE_OFF_SIO_Msk = 0x200000
	// Bit SIO.
	PSM_FRCE_OFF_SIO = 0x200000
	// Position of XIP field.
	PSM_FRCE_OFF_XIP_Pos = 0x14
	// Bit mask of XIP field.
	PSM_FRCE_OFF_XIP_Msk = 0x100000
	// Bit XIP.
	PSM_FRCE_OFF_XIP = 0x100000
	// Position of SRAM9 field.
	PSM_FRCE_OFF_SRAM9_Pos = 0x13
	// Bit mask of SRAM9 field.
	PSM_FRCE_OFF_SRAM9_Msk = 0x80000
	// Bit SRAM9.
	PSM_FRCE_OFF_SRAM9 = 0x80000
	// Position of SRAM8 field.
	PSM_FRCE_OFF_SRAM8_Pos = 0x12
	// Bit mask of SRAM8 field.
	PSM_FRCE_OFF_SRAM8_Msk = 0x40000
	// Bit SRAM8.
	PSM_FRCE_OFF_SRAM8 = 0x40000
	// Position of SRAM7 field.
	PSM_FRCE_OFF_SRAM7_Pos = 0x11
	// Bit mask of SRAM7 field.
	PSM_FRCE_OFF_SRAM7_Msk = 0x20000
	// Bit SRAM7.
	PSM_FRCE_OFF_SRAM7 = 0x20000
	// Position of SRAM6 field.
	PSM_FRCE_OFF_SRAM6_Pos = 0x10
	// Bit mask of SRAM6 field.
	PSM_FRCE_OFF_SRAM6_Msk = 0x10000
	// Bit SRAM6.
	PSM_FRCE_OFF_SRAM6 = 0x10000
	// Position of SRAM5 field.
	PSM_FRCE_OFF_SRAM5_Pos = 0xf
	// Bit mask of SRAM5 field.
	PSM_FRCE_OFF_SRAM5_Msk = 0x8000
	// Bit SRAM5.
	PSM_FRCE_OFF_SRAM5 = 0x8000
	// Position of SRAM4 field.
	PSM_FRCE_OFF_SRAM4_Pos = 0xe
	// Bit mask of SRAM4 field.
	PSM_FRCE_OFF_SRAM4_Msk = 0x4000
	// Bit SRAM4.
	PSM_FRCE_OFF_SRAM4 = 0x4000
	// Position of SRAM3 field.
	PSM_FRCE_OFF_SRAM3_Pos = 0xd
	// Bit mask of SRAM3 field.
	PSM_FRCE_OFF_SRAM3_Msk = 0x2000
	// Bit SRAM3.
	PSM_FRCE_OFF_SRAM3 = 0x2000
	// Position of SRAM2 field.
	PSM_FRCE_OFF_SRAM2_Pos = 0xc
	// Bit mask of SRAM2 field.
	PSM_FRCE_OFF_SRAM2_Msk = 0x1000
	// Bit SRAM2.
	PSM_FRCE_OFF_SRAM2 = 0x1000
	// Position of SRAM1 field.
	PSM_FRCE_OFF_SRAM1_Pos = 0xb
	// Bit mask of SRAM1 field.
	PSM_FRCE_OFF_SRAM1_Msk = 0x800
	// Bit SRAM1.
	PSM_FRCE_OFF_SRAM1 = 0x800
	// Position of SRAM0 field.
	PSM_FRCE_OFF_SRAM0_Pos = 0xa
	// Bit mask of SRAM0 field.
	PSM_FRCE_OFF_SRAM0_Msk = 0x400
	// Bit SRAM0.
	PSM_FRCE_OFF_SRAM0 = 0x400
	// Position of BOOTRAM field.
	PSM_FRCE_OFF_BOOTRAM_Pos = 0x9
	// Bit mask of BOOTRAM field.
	PSM_FRCE_OFF_BOOTRAM_Msk = 0x200
	// Bit BOOTRAM.
	PSM_FRCE_OFF_BOOTRAM = 0x200
	// Position of ROM field.
	PSM_FRCE_OFF_ROM_Pos = 0x8
	// Bit mask of ROM field.
	PSM_FRCE_OFF_ROM_Msk = 0x100
	// Bit ROM.
	PSM_FRCE_OFF_ROM = 0x100
	// Position of BUSFABRIC field.
	PSM_FRCE_OFF_BUSFABRIC_Pos = 0x7
	// Bit mask of BUSFABRIC field.
	PSM_FRCE_OFF_BUSFABRIC_Msk = 0x80
	// Bit BUSFABRIC.
	PSM_FRCE_OFF_BUSFABRIC = 0x80
	// Position of PSM_READY field.
	PSM_FRCE_OFF_PSM_READY_Pos = 0x6
	// Bit mask of PSM_READY field.
	PSM_FRCE_OFF_PSM_READY_Msk = 0x40
	// Bit PSM_READY.
	PSM_FRCE_OFF_PSM_READY = 0x40
	// Position of CLOCKS field.
	PSM_FRCE_OFF_CLOCKS_Pos = 0x5
	// Bit mask of CLOCKS field.
	PSM_FRCE_OFF_CLOCKS_Msk = 0x20
	// Bit CLOCKS.
	PSM_FRCE_OFF_CLOCKS = 0x20
	// Position of RESETS field.
	PSM_FRCE_OFF_RESETS_Pos = 0x4
	// Bit mask of RESETS field.
	PSM_FRCE_OFF_RESETS_Msk = 0x10
	// Bit RESETS.
	PSM_FRCE_OFF_RESETS = 0x10
	// Position of XOSC field.
	PSM_FRCE_OFF_XOSC_Pos = 0x3
	// Bit mask of XOSC field.
	PSM_FRCE_OFF_XOSC_Msk = 0x8
	// Bit XOSC.
	PSM_FRCE_OFF_XOSC = 0x8
	// Position of ROSC field.
	PSM_FRCE_OFF_ROSC_Pos = 0x2
	// Bit mask of ROSC field.
	PSM_FRCE_OFF_ROSC_Msk = 0x4
	// Bit ROSC.
	PSM_FRCE_OFF_ROSC = 0x4
	// Position of OTP field.
	PSM_FRCE_OFF_OTP_Pos = 0x1
	// Bit mask of OTP field.
	PSM_FRCE_OFF_OTP_Msk = 0x2
	// Bit OTP.
	PSM_FRCE_OFF_OTP = 0x2
	// Position of PROC_COLD field.
	PSM_FRCE_OFF_PROC_COLD_Pos = 0x0
	// Bit mask of PROC_COLD field.
	PSM_FRCE_OFF_PROC_COLD_Msk = 0x1
	// Bit PROC_COLD.
	PSM_FRCE_OFF_PROC_COLD = 0x1

	// WDSEL: Set to 1 if the watchdog should reset this
	// Position of PROC1 field.
	PSM_WDSEL_PROC1_Pos = 0x18
	// Bit mask of PROC1 field.
	PSM_WDSEL_PROC1_Msk = 0x1000000
	// Bit PROC1.
	PSM_WDSEL_PROC1 = 0x1000000
	// Position of PROC0 field.
	PSM_WDSEL_PROC0_Pos = 0x17
	// Bit mask of PROC0 field.
	PSM_WDSEL_PROC0_Msk = 0x800000
	// Bit PROC0.
	PSM_WDSEL_PROC0 = 0x800000
	// Position of ACCESSCTRL field.
	PSM_WDSEL_ACCESSCTRL_Pos = 0x16
	// Bit mask of ACCESSCTRL field.
	PSM_WDSEL_ACCESSCTRL_Msk = 0x400000
	// Bit ACCESSCTRL.
	PSM_WDSEL_ACCESSCTRL = 0x400000
	// Position of SIO field.
	PSM_WDSEL_SIO_Pos = 0x15
	// Bit mask of SIO field.
	PSM_WDSEL_SIO_Msk = 0x200000
	// Bit SIO.
	PSM_WDSEL_SIO = 0x200000
	// Position of XIP field.
	PSM_WDSEL_XIP_Pos = 0x14
	// Bit mask of XIP field.
	PSM_WDSEL_XIP_Msk = 0x100000
	// Bit XIP.
	PSM_WDSEL_XIP = 0x100000
	// Position of SRAM9 field.
	PSM_WDSEL_SRAM9_Pos = 0x13
	// Bit mask of SRAM9 field.
	PSM_WDSEL_SRAM9_Msk = 0x80000
	// Bit SRAM9.
	PSM_WDSEL_SRAM9 = 0x80000
	// Position of SRAM8 field.
	PSM_WDSEL_SRAM8_Pos = 0x12
	// Bit mask of SRAM8 field.
	PSM_WDSEL_SRAM8_Msk = 0x40000
	// Bit SRAM8.
	PSM_WDSEL_SRAM8 = 0x40000
	// Position of SRAM7 field.
	PSM_WDSEL_SRAM7_Pos = 0x11
	// Bit mask of SRAM7 field.
	PSM_WDSEL_SRAM7_Msk = 0x20000
	// Bit SRAM7.
	PSM_WDSEL_SRAM7 = 0x20000
	// Position of SRAM6 field.
	PSM_WDSEL_SRAM6_Pos = 0x10
	// Bit mask of SRAM6 field.
	PSM_WDSEL_SRAM6_Msk = 0x10000
	// Bit SRAM6.
	PSM_WDSEL_SRAM6 = 0x10000
	// Position of SRAM5 field.
	PSM_WDSEL_SRAM5_Pos = 0xf
	// Bit mask of SRAM5 field.
	PSM_WDSEL_SRAM5_Msk = 0x8000
	// Bit SRAM5.
	PSM_WDSEL_SRAM5 = 0x8000
	// Position of SRAM4 field.
	PSM_WDSEL_SRAM4_Pos = 0xe
	// Bit mask of SRAM4 field.
	PSM_WDSEL_SRAM4_Msk = 0x4000
	// Bit SRAM4.
	PSM_WDSEL_SRAM4 = 0x4000
	// Position of SRAM3 field.
	PSM_WDSEL_SRAM3_Pos = 0xd
	// Bit mask of SRAM3 field.
	PSM_WDSEL_SRAM3_Msk = 0x2000
	// Bit SRAM3.
	PSM_WDSEL_SRAM3 = 0x2000
	// Position of SRAM2 field.
	PSM_WDSEL_SRAM2_Pos = 0xc
	// Bit mask of SRAM2 field.
	PSM_WDSEL_SRAM2_Msk = 0x1000
	// Bit SRAM2.
	PSM_WDSEL_SRAM2 = 0x1000
	// Position of SRAM1 field.
	PSM_WDSEL_SRAM1_Pos = 0xb
	// Bit mask of SRAM1 field.
	PSM_WDSEL_SRAM1_Msk = 0x800
	// Bit SRAM1.
	PSM_WDSEL_SRAM1 = 0x800
	// Position of SRAM0 field.
	PSM_WDSEL_SRAM0_Pos = 0xa
	// Bit mask of SRAM0 field.
	PSM_WDSEL_SRAM0_Msk = 0x400
	// Bit SRAM0.
	PSM_WDSEL_SRAM0 = 0x400
	// Position of BOOTRAM field.
	PSM_WDSEL_BOOTRAM_Pos = 0x9
	// Bit mask of BOOTRAM field.
	PSM_WDSEL_BOOTRAM_Msk = 0x200
	// Bit BOOTRAM.
	PSM_WDSEL_BOOTRAM = 0x200
	// Position of ROM field.
	PSM_WDSEL_ROM_Pos = 0x8
	// Bit mask of ROM field.
	PSM_WDSEL_ROM_Msk = 0x100
	// Bit ROM.
	PSM_WDSEL_ROM = 0x100
	// Position of BUSFABRIC field.
	PSM_WDSEL_BUSFABRIC_Pos = 0x7
	// Bit mask of BUSFABRIC field.
	PSM_WDSEL_BUSFABRIC_Msk = 0x80
	// Bit BUSFABRIC.
	PSM_WDSEL_BUSFABRIC = 0x80
	// Position of PSM_READY field.
	PSM_WDSEL_PSM_READY_Pos = 0x6
	// Bit mask of PSM_READY field.
	PSM_WDSEL_PSM_READY_Msk = 0x40
	// Bit PSM_READY.
	PSM_WDSEL_PSM_READY = 0x40
	// Position of CLOCKS field.
	PSM_WDSEL_CLOCKS_Pos = 0x5
	// Bit mask of CLOCKS field.
	PSM_WDSEL_CLOCKS_Msk = 0x20
	// Bit CLOCKS.
	PSM_WDSEL_CLOCKS = 0x20
	// Position of RESETS field.
	PSM_WDSEL_RESETS_Pos = 0x4
	// Bit mask of RESETS field.
	PSM_WDSEL_RESETS_Msk = 0x10
	// Bit RESETS.
	PSM_WDSEL_RESETS = 0x10
	// Position of XOSC field.
	PSM_WDSEL_XOSC_Pos = 0x3
	// Bit mask of XOSC field.
	PSM_WDSEL_XOSC_Msk = 0x8
	// Bit XOSC.
	PSM_WDSEL_XOSC = 0x8
	// Position of ROSC field.
	PSM_WDSEL_ROSC_Pos = 0x2
	// Bit mask of ROSC field.
	PSM_WDSEL_ROSC_Msk = 0x4
	// Bit ROSC.
	PSM_WDSEL_ROSC = 0x4
	// Position of OTP field.
	PSM_WDSEL_OTP_Pos = 0x1
	// Bit mask of OTP field.
	PSM_WDSEL_OTP_Msk = 0x2
	// Bit OTP.
	PSM_WDSEL_OTP = 0x2
	// Position of PROC_COLD field.
	PSM_WDSEL_PROC_COLD_Pos = 0x0
	// Bit mask of PROC_COLD field.
	PSM_WDSEL_PROC_COLD_Msk = 0x1
	// Bit PROC_COLD.
	PSM_WDSEL_PROC_COLD = 0x1

	// DONE: Is the subsystem ready?
	// Position of PROC1 field.
	PSM_DONE_PROC1_Pos = 0x18
	// Bit mask of PROC1 field.
	PSM_DONE_PROC1_Msk = 0x1000000
	// Bit PROC1.
	PSM_DONE_PROC1 = 0x1000000
	// Position of PROC0 field.
	PSM_DONE_PROC0_Pos = 0x17
	// Bit mask of PROC0 field.
	PSM_DONE_PROC0_Msk = 0x800000
	// Bit PROC0.
	PSM_DONE_PROC0 = 0x800000
	// Position of ACCESSCTRL field.
	PSM_DONE_ACCESSCTRL_Pos = 0x16
	// Bit mask of ACCESSCTRL field.
	PSM_DONE_ACCESSCTRL_Msk = 0x400000
	// Bit ACCESSCTRL.
	PSM_DONE_ACCESSCTRL = 0x400000
	// Position of SIO field.
	PSM_DONE_SIO_Pos = 0x15
	// Bit mask of SIO field.
	PSM_DONE_SIO_Msk = 0x200000
	// Bit SIO.
	PSM_DONE_SIO = 0x200000
	// Position of XIP field.
	PSM_DONE_XIP_Pos = 0x14
	// Bit mask of XIP field.
	PSM_DONE_XIP_Msk = 0x100000
	// Bit XIP.
	PSM_DONE_XIP = 0x100000
	// Position of SRAM9 field.
	PSM_DONE_SRAM9_Pos = 0x13
	// Bit mask of SRAM9 field.
	PSM_DONE_SRAM9_Msk = 0x80000
	// Bit SRAM9.
	PSM_DONE_SRAM9 = 0x80000
	// Position of SRAM8 field.
	PSM_DONE_SRAM8_Pos = 0x12
	// Bit mask of SRAM8 field.
	PSM_DONE_SRAM8_Msk = 0x40000
	// Bit SRAM8.
	PSM_DONE_SRAM8 = 0x40000
	// Position of SRAM7 field.
	PSM_DONE_SRAM7_Pos = 0x11
	// Bit mask of SRAM7 field.
	PSM_DONE_SRAM7_Msk = 0x20000
	// Bit SRAM7.
	PSM_DONE_SRAM7 = 0x20000
	// Position of SRAM6 field.
	PSM_DONE_SRAM6_Pos = 0x10
	// Bit mask of SRAM6 field.
	PSM_DONE_SRAM6_Msk = 0x10000
	// Bit SRAM6.
	PSM_DONE_SRAM6 = 0x10000
	// Position of SRAM5 field.
	PSM_DONE_SRAM5_Pos = 0xf
	// Bit mask of SRAM5 field.
	PSM_DONE_SRAM5_Msk = 0x8000
	// Bit SRAM5.
	PSM_DONE_SRAM5 = 0x8000
	// Position of SRAM4 field.
	PSM_DONE_SRAM4_Pos = 0xe
	// Bit mask of SRAM4 field.
	PSM_DONE_SRAM4_Msk = 0x4000
	// Bit SRAM4.
	PSM_DONE_SRAM4 = 0x4000
	// Position of SRAM3 field.
	PSM_DONE_SRAM3_Pos = 0xd
	// Bit mask of SRAM3 field.
	PSM_DONE_SRAM3_Msk = 0x2000
	// Bit SRAM3.
	PSM_DONE_SRAM3 = 0x2000
	// Position of SRAM2 field.
	PSM_DONE_SRAM2_Pos = 0xc
	// Bit mask of SRAM2 field.
	PSM_DONE_SRAM2_Msk = 0x1000
	// Bit SRAM2.
	PSM_DONE_SRAM2 = 0x1000
	// Position of SRAM1 field.
	PSM_DONE_SRAM1_Pos = 0xb
	// Bit mask of SRAM1 field.
	PSM_DONE_SRAM1_Msk = 0x800
	// Bit SRAM1.
	PSM_DONE_SRAM1 = 0x800
	// Position of SRAM0 field.
	PSM_DONE_SRAM0_Pos = 0xa
	// Bit mask of SRAM0 field.
	PSM_DONE_SRAM0_Msk = 0x400
	// Bit SRAM0.
	PSM_DONE_SRAM0 = 0x400
	// Position of BOOTRAM field.
	PSM_DONE_BOOTRAM_Pos = 0x9
	// Bit mask of BOOTRAM field.
	PSM_DONE_BOOTRAM_Msk = 0x200
	// Bit BOOTRAM.
	PSM_DONE_BOOTRAM = 0x200
	// Position of ROM field.
	PSM_DONE_ROM_Pos = 0x8
	// Bit mask of ROM field.
	PSM_DONE_ROM_Msk = 0x100
	// Bit ROM.
	PSM_DONE_ROM = 0x100
	// Position of BUSFABRIC field.
	PSM_DONE_BUSFABRIC_Pos = 0x7
	// Bit mask of BUSFABRIC field.
	PSM_DONE_BUSFABRIC_Msk = 0x80
	// Bit BUSFABRIC.
	PSM_DONE_BUSFABRIC = 0x80
	// Position of PSM_READY field.
	PSM_DONE_PSM_READY_Pos = 0x6
	// Bit mask of PSM_READY field.
	PSM_DONE_PSM_READY_Msk = 0x40
	// Bit PSM_READY.
	PSM_DONE_PSM_READY = 0x40
	// Position of CLOCKS field.
	PSM_DONE_CLOCKS_Pos = 0x5
	// Bit mask of CLOCKS field.
	PSM_DONE_CLOCKS_Msk = 0x20
	// Bit CLOCKS.
	PSM_DONE_CLOCKS = 0x20
	// Position of RESETS field.
	PSM_DONE_RESETS_Pos = 0x4
	// Bit mask of RESETS field.
	PSM_DONE_RESETS_Msk = 0x10
	// Bit RESETS.
	PSM_DONE_RESETS = 0x10
	// Position of XOSC field.
	PSM_DONE_XOSC_Pos = 0x3
	// Bit mask of XOSC field.
	PSM_DONE_XOSC_Msk = 0x8
	// Bit XOSC.
	PSM_DONE_XOSC = 0x8
	// Position of ROSC field.
	PSM_DONE_ROSC_Pos = 0x2
	// Bit mask of ROSC field.
	PSM_DONE_ROSC_Msk = 0x4
	// Bit ROSC.
	PSM_DONE_ROSC = 0x4
	// Position of OTP field.
	PSM_DONE_OTP_Pos = 0x1
	// Bit mask of OTP field.
	PSM_DONE_OTP_Msk = 0x2
	// Bit OTP.
	PSM_DONE_OTP = 0x2
	// Position of PROC_COLD field.
	PSM_DONE_PROC_COLD_Pos = 0x0
	// Bit mask of PROC_COLD field.
	PSM_DONE_PROC_COLD_Msk = 0x1
	// Bit PROC_COLD.
	PSM_DONE_PROC_COLD = 0x1
)

// Constants for CLOCKS
const (
	// CLK_GPOUT0_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of ENABLED field.
	CLOCKS_CLK_GPOUT0_CTRL_ENABLED_Pos = 0x1c
	// Bit mask of ENABLED field.
	CLOCKS_CLK_GPOUT0_CTRL_ENABLED_Msk = 0x10000000
	// Bit ENABLED.
	CLOCKS_CLK_GPOUT0_CTRL_ENABLED = 0x10000000
	// Position of NUDGE field.
	CLOCKS_CLK_GPOUT0_CTRL_NUDGE_Pos = 0x14
	// Bit mask of NUDGE field.
	CLOCKS_CLK_GPOUT0_CTRL_NUDGE_Msk = 0x100000
	// Bit NUDGE.
	CLOCKS_CLK_GPOUT0_CTRL_NUDGE = 0x100000
	// Position of PHASE field.
	CLOCKS_CLK_GPOUT0_CTRL_PHASE_Pos = 0x10
	// Bit mask of PHASE field.
	CLOCKS_CLK_GPOUT0_CTRL_PHASE_Msk = 0x30000
	// Position of DC50 field.
	CLOCKS_CLK_GPOUT0_CTRL_DC50_Pos = 0xc
	// Bit mask of DC50 field.
	CLOCKS_CLK_GPOUT0_CTRL_DC50_Msk = 0x1000
	// Bit DC50.
	CLOCKS_CLK_GPOUT0_CTRL_DC50 = 0x1000
	// Position of ENABLE field.
	CLOCKS_CLK_GPOUT0_CTRL_ENABLE_Pos = 0xb
	// Bit mask of ENABLE field.
	CLOCKS_CLK_GPOUT0_CTRL_ENABLE_Msk = 0x800
	// Bit ENABLE.
	CLOCKS_CLK_GPOUT0_CTRL_ENABLE = 0x800
	// Position of KILL field.
	CLOCKS_CLK_GPOUT0_CTRL_KILL_Pos = 0xa
	// Bit mask of KILL field.
	CLOCKS_CLK_GPOUT0_CTRL_KILL_Msk = 0x400
	// Bit KILL.
	CLOCKS_CLK_GPOUT0_CTRL_KILL = 0x400
	// Position of AUXSRC field.
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_Msk                             = 0x1e0
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLKSRC_PLL_SYS                  = 0x0
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLKSRC_GPIN0                    = 0x1
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLKSRC_GPIN1                    = 0x2
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLKSRC_PLL_USB                  = 0x3
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLKSRC_PLL_USB_PRIMARY_REF_OPCG = 0x4
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_ROSC_CLKSRC                     = 0x5
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_XOSC_CLKSRC                     = 0x6
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_LPOSC_CLKSRC                    = 0x7
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLK_SYS                         = 0x8
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLK_USB                         = 0x9
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLK_ADC                         = 0xa
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLK_REF                         = 0xb
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLK_PERI                        = 0xc
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_CLK_HSTX                        = 0xd
	CLOCKS_CLK_GPOUT0_CTRL_AUXSRC_OTP_CLK2FC                      = 0xe

	// CLK_GPOUT0_DIV
	// Position of INT field.
	CLOCKS_CLK_GPOUT0_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_GPOUT0_DIV_INT_Msk = 0xffff0000
	// Position of FRAC field.
	CLOCKS_CLK_GPOUT0_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	CLOCKS_CLK_GPOUT0_DIV_FRAC_Msk = 0xffff

	// CLK_GPOUT0_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_GPOUT0_SELECTED field.
	CLOCKS_CLK_GPOUT0_SELECTED_CLK_GPOUT0_SELECTED_Pos = 0x0
	// Bit mask of CLK_GPOUT0_SELECTED field.
	CLOCKS_CLK_GPOUT0_SELECTED_CLK_GPOUT0_SELECTED_Msk = 0x1
	// Bit CLK_GPOUT0_SELECTED.
	CLOCKS_CLK_GPOUT0_SELECTED_CLK_GPOUT0_SELECTED = 0x1

	// CLK_GPOUT1_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of ENABLED field.
	CLOCKS_CLK_GPOUT1_CTRL_ENABLED_Pos = 0x1c
	// Bit mask of ENABLED field.
	CLOCKS_CLK_GPOUT1_CTRL_ENABLED_Msk = 0x10000000
	// Bit ENABLED.
	CLOCKS_CLK_GPOUT1_CTRL_ENABLED = 0x10000000
	// Position of NUDGE field.
	CLOCKS_CLK_GPOUT1_CTRL_NUDGE_Pos = 0x14
	// Bit mask of NUDGE field.
	CLOCKS_CLK_GPOUT1_CTRL_NUDGE_Msk = 0x100000
	// Bit NUDGE.
	CLOCKS_CLK_GPOUT1_CTRL_NUDGE = 0x100000
	// Position of PHASE field.
	CLOCKS_CLK_GPOUT1_CTRL_PHASE_Pos = 0x10
	// Bit mask of PHASE field.
	CLOCKS_CLK_GPOUT1_CTRL_PHASE_Msk = 0x30000
	// Position of DC50 field.
	CLOCKS_CLK_GPOUT1_CTRL_DC50_Pos = 0xc
	// Bit mask of DC50 field.
	CLOCKS_CLK_GPOUT1_CTRL_DC50_Msk = 0x1000
	// Bit DC50.
	CLOCKS_CLK_GPOUT1_CTRL_DC50 = 0x1000
	// Position of ENABLE field.
	CLOCKS_CLK_GPOUT1_CTRL_ENABLE_Pos = 0xb
	// Bit mask of ENABLE field.
	CLOCKS_CLK_GPOUT1_CTRL_ENABLE_Msk = 0x800
	// Bit ENABLE.
	CLOCKS_CLK_GPOUT1_CTRL_ENABLE = 0x800
	// Position of KILL field.
	CLOCKS_CLK_GPOUT1_CTRL_KILL_Pos = 0xa
	// Bit mask of KILL field.
	CLOCKS_CLK_GPOUT1_CTRL_KILL_Msk = 0x400
	// Bit KILL.
	CLOCKS_CLK_GPOUT1_CTRL_KILL = 0x400
	// Position of AUXSRC field.
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_Msk                             = 0x1e0
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLKSRC_PLL_SYS                  = 0x0
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLKSRC_GPIN0                    = 0x1
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLKSRC_GPIN1                    = 0x2
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLKSRC_PLL_USB                  = 0x3
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLKSRC_PLL_USB_PRIMARY_REF_OPCG = 0x4
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_ROSC_CLKSRC                     = 0x5
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_XOSC_CLKSRC                     = 0x6
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_LPOSC_CLKSRC                    = 0x7
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLK_SYS                         = 0x8
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLK_USB                         = 0x9
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLK_ADC                         = 0xa
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLK_REF                         = 0xb
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLK_PERI                        = 0xc
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_CLK_HSTX                        = 0xd
	CLOCKS_CLK_GPOUT1_CTRL_AUXSRC_OTP_CLK2FC                      = 0xe

	// CLK_GPOUT1_DIV
	// Position of INT field.
	CLOCKS_CLK_GPOUT1_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_GPOUT1_DIV_INT_Msk = 0xffff0000
	// Position of FRAC field.
	CLOCKS_CLK_GPOUT1_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	CLOCKS_CLK_GPOUT1_DIV_FRAC_Msk = 0xffff

	// CLK_GPOUT1_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_GPOUT1_SELECTED field.
	CLOCKS_CLK_GPOUT1_SELECTED_CLK_GPOUT1_SELECTED_Pos = 0x0
	// Bit mask of CLK_GPOUT1_SELECTED field.
	CLOCKS_CLK_GPOUT1_SELECTED_CLK_GPOUT1_SELECTED_Msk = 0x1
	// Bit CLK_GPOUT1_SELECTED.
	CLOCKS_CLK_GPOUT1_SELECTED_CLK_GPOUT1_SELECTED = 0x1

	// CLK_GPOUT2_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of ENABLED field.
	CLOCKS_CLK_GPOUT2_CTRL_ENABLED_Pos = 0x1c
	// Bit mask of ENABLED field.
	CLOCKS_CLK_GPOUT2_CTRL_ENABLED_Msk = 0x10000000
	// Bit ENABLED.
	CLOCKS_CLK_GPOUT2_CTRL_ENABLED = 0x10000000
	// Position of NUDGE field.
	CLOCKS_CLK_GPOUT2_CTRL_NUDGE_Pos = 0x14
	// Bit mask of NUDGE field.
	CLOCKS_CLK_GPOUT2_CTRL_NUDGE_Msk = 0x100000
	// Bit NUDGE.
	CLOCKS_CLK_GPOUT2_CTRL_NUDGE = 0x100000
	// Position of PHASE field.
	CLOCKS_CLK_GPOUT2_CTRL_PHASE_Pos = 0x10
	// Bit mask of PHASE field.
	CLOCKS_CLK_GPOUT2_CTRL_PHASE_Msk = 0x30000
	// Position of DC50 field.
	CLOCKS_CLK_GPOUT2_CTRL_DC50_Pos = 0xc
	// Bit mask of DC50 field.
	CLOCKS_CLK_GPOUT2_CTRL_DC50_Msk = 0x1000
	// Bit DC50.
	CLOCKS_CLK_GPOUT2_CTRL_DC50 = 0x1000
	// Position of ENABLE field.
	CLOCKS_CLK_GPOUT2_CTRL_ENABLE_Pos = 0xb
	// Bit mask of ENABLE field.
	CLOCKS_CLK_GPOUT2_CTRL_ENABLE_Msk = 0x800
	// Bit ENABLE.
	CLOCKS_CLK_GPOUT2_CTRL_ENABLE = 0x800
	// Position of KILL field.
	CLOCKS_CLK_GPOUT2_CTRL_KILL_Pos = 0xa
	// Bit mask of KILL field.
	CLOCKS_CLK_GPOUT2_CTRL_KILL_Msk = 0x400
	// Bit KILL.
	CLOCKS_CLK_GPOUT2_CTRL_KILL = 0x400
	// Position of AUXSRC field.
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_Msk                             = 0x1e0
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLKSRC_PLL_SYS                  = 0x0
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLKSRC_GPIN0                    = 0x1
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLKSRC_GPIN1                    = 0x2
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLKSRC_PLL_USB                  = 0x3
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLKSRC_PLL_USB_PRIMARY_REF_OPCG = 0x4
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_ROSC_CLKSRC_PH                  = 0x5
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_XOSC_CLKSRC                     = 0x6
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_LPOSC_CLKSRC                    = 0x7
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLK_SYS                         = 0x8
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLK_USB                         = 0x9
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLK_ADC                         = 0xa
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLK_REF                         = 0xb
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLK_PERI                        = 0xc
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_CLK_HSTX                        = 0xd
	CLOCKS_CLK_GPOUT2_CTRL_AUXSRC_OTP_CLK2FC                      = 0xe

	// CLK_GPOUT2_DIV
	// Position of INT field.
	CLOCKS_CLK_GPOUT2_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_GPOUT2_DIV_INT_Msk = 0xffff0000
	// Position of FRAC field.
	CLOCKS_CLK_GPOUT2_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	CLOCKS_CLK_GPOUT2_DIV_FRAC_Msk = 0xffff

	// CLK_GPOUT2_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_GPOUT2_SELECTED field.
	CLOCKS_CLK_GPOUT2_SELECTED_CLK_GPOUT2_SELECTED_Pos = 0x0
	// Bit mask of CLK_GPOUT2_SELECTED field.
	CLOCKS_CLK_GPOUT2_SELECTED_CLK_GPOUT2_SELECTED_Msk = 0x1
	// Bit CLK_GPOUT2_SELECTED.
	CLOCKS_CLK_GPOUT2_SELECTED_CLK_GPOUT2_SELECTED = 0x1

	// CLK_GPOUT3_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of ENABLED field.
	CLOCKS_CLK_GPOUT3_CTRL_ENABLED_Pos = 0x1c
	// Bit mask of ENABLED field.
	CLOCKS_CLK_GPOUT3_CTRL_ENABLED_Msk = 0x10000000
	// Bit ENABLED.
	CLOCKS_CLK_GPOUT3_CTRL_ENABLED = 0x10000000
	// Position of NUDGE field.
	CLOCKS_CLK_GPOUT3_CTRL_NUDGE_Pos = 0x14
	// Bit mask of NUDGE field.
	CLOCKS_CLK_GPOUT3_CTRL_NUDGE_Msk = 0x100000
	// Bit NUDGE.
	CLOCKS_CLK_GPOUT3_CTRL_NUDGE = 0x100000
	// Position of PHASE field.
	CLOCKS_CLK_GPOUT3_CTRL_PHASE_Pos = 0x10
	// Bit mask of PHASE field.
	CLOCKS_CLK_GPOUT3_CTRL_PHASE_Msk = 0x30000
	// Position of DC50 field.
	CLOCKS_CLK_GPOUT3_CTRL_DC50_Pos = 0xc
	// Bit mask of DC50 field.
	CLOCKS_CLK_GPOUT3_CTRL_DC50_Msk = 0x1000
	// Bit DC50.
	CLOCKS_CLK_GPOUT3_CTRL_DC50 = 0x1000
	// Position of ENABLE field.
	CLOCKS_CLK_GPOUT3_CTRL_ENABLE_Pos = 0xb
	// Bit mask of ENABLE field.
	CLOCKS_CLK_GPOUT3_CTRL_ENABLE_Msk = 0x800
	// Bit ENABLE.
	CLOCKS_CLK_GPOUT3_CTRL_ENABLE = 0x800
	// Position of KILL field.
	CLOCKS_CLK_GPOUT3_CTRL_KILL_Pos = 0xa
	// Bit mask of KILL field.
	CLOCKS_CLK_GPOUT3_CTRL_KILL_Msk = 0x400
	// Bit KILL.
	CLOCKS_CLK_GPOUT3_CTRL_KILL = 0x400
	// Position of AUXSRC field.
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_Msk                             = 0x1e0
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLKSRC_PLL_SYS                  = 0x0
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLKSRC_GPIN0                    = 0x1
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLKSRC_GPIN1                    = 0x2
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLKSRC_PLL_USB                  = 0x3
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLKSRC_PLL_USB_PRIMARY_REF_OPCG = 0x4
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_ROSC_CLKSRC_PH                  = 0x5
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_XOSC_CLKSRC                     = 0x6
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_LPOSC_CLKSRC                    = 0x7
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLK_SYS                         = 0x8
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLK_USB                         = 0x9
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLK_ADC                         = 0xa
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLK_REF                         = 0xb
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLK_PERI                        = 0xc
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_CLK_HSTX                        = 0xd
	CLOCKS_CLK_GPOUT3_CTRL_AUXSRC_OTP_CLK2FC                      = 0xe

	// CLK_GPOUT3_DIV
	// Position of INT field.
	CLOCKS_CLK_GPOUT3_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_GPOUT3_DIV_INT_Msk = 0xffff0000
	// Position of FRAC field.
	CLOCKS_CLK_GPOUT3_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	CLOCKS_CLK_GPOUT3_DIV_FRAC_Msk = 0xffff

	// CLK_GPOUT3_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_GPOUT3_SELECTED field.
	CLOCKS_CLK_GPOUT3_SELECTED_CLK_GPOUT3_SELECTED_Pos = 0x0
	// Bit mask of CLK_GPOUT3_SELECTED field.
	CLOCKS_CLK_GPOUT3_SELECTED_CLK_GPOUT3_SELECTED_Msk = 0x1
	// Bit CLK_GPOUT3_SELECTED.
	CLOCKS_CLK_GPOUT3_SELECTED_CLK_GPOUT3_SELECTED = 0x1

	// CLK_REF_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of AUXSRC field.
	CLOCKS_CLK_REF_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_REF_CTRL_AUXSRC_Msk                             = 0x60
	CLOCKS_CLK_REF_CTRL_AUXSRC_CLKSRC_PLL_USB                  = 0x0
	CLOCKS_CLK_REF_CTRL_AUXSRC_CLKSRC_GPIN0                    = 0x1
	CLOCKS_CLK_REF_CTRL_AUXSRC_CLKSRC_GPIN1                    = 0x2
	CLOCKS_CLK_REF_CTRL_AUXSRC_CLKSRC_PLL_USB_PRIMARY_REF_OPCG = 0x3
	// Position of SRC field.
	CLOCKS_CLK_REF_CTRL_SRC_Pos = 0x0
	// Bit mask of SRC field.
	CLOCKS_CLK_REF_CTRL_SRC_Msk                = 0x3
	CLOCKS_CLK_REF_CTRL_SRC_ROSC_CLKSRC_PH     = 0x0
	CLOCKS_CLK_REF_CTRL_SRC_CLKSRC_CLK_REF_AUX = 0x1
	CLOCKS_CLK_REF_CTRL_SRC_XOSC_CLKSRC        = 0x2
	CLOCKS_CLK_REF_CTRL_SRC_LPOSC_CLKSRC       = 0x3

	// CLK_REF_DIV
	// Position of INT field.
	CLOCKS_CLK_REF_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_REF_DIV_INT_Msk = 0xff0000

	// CLK_REF_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_REF_SELECTED field.
	CLOCKS_CLK_REF_SELECTED_CLK_REF_SELECTED_Pos = 0x0
	// Bit mask of CLK_REF_SELECTED field.
	CLOCKS_CLK_REF_SELECTED_CLK_REF_SELECTED_Msk = 0xf

	// CLK_SYS_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of AUXSRC field.
	CLOCKS_CLK_SYS_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_SYS_CTRL_AUXSRC_Msk            = 0xe0
	CLOCKS_CLK_SYS_CTRL_AUXSRC_CLKSRC_PLL_SYS = 0x0
	CLOCKS_CLK_SYS_CTRL_AUXSRC_CLKSRC_PLL_USB = 0x1
	CLOCKS_CLK_SYS_CTRL_AUXSRC_ROSC_CLKSRC    = 0x2
	CLOCKS_CLK_SYS_CTRL_AUXSRC_XOSC_CLKSRC    = 0x3
	CLOCKS_CLK_SYS_CTRL_AUXSRC_CLKSRC_GPIN0   = 0x4
	CLOCKS_CLK_SYS_CTRL_AUXSRC_CLKSRC_GPIN1   = 0x5
	// Position of SRC field.
	CLOCKS_CLK_SYS_CTRL_SRC_Pos = 0x0
	// Bit mask of SRC field.
	CLOCKS_CLK_SYS_CTRL_SRC_Msk = 0x1
	// Bit SRC.
	CLOCKS_CLK_SYS_CTRL_SRC                    = 0x1
	CLOCKS_CLK_SYS_CTRL_SRC_CLK_REF            = 0x0
	CLOCKS_CLK_SYS_CTRL_SRC_CLKSRC_CLK_SYS_AUX = 0x1

	// CLK_SYS_DIV
	// Position of INT field.
	CLOCKS_CLK_SYS_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_SYS_DIV_INT_Msk = 0xffff0000
	// Position of FRAC field.
	CLOCKS_CLK_SYS_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	CLOCKS_CLK_SYS_DIV_FRAC_Msk = 0xffff

	// CLK_SYS_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_SYS_SELECTED field.
	CLOCKS_CLK_SYS_SELECTED_CLK_SYS_SELECTED_Pos = 0x0
	// Bit mask of CLK_SYS_SELECTED field.
	CLOCKS_CLK_SYS_SELECTED_CLK_SYS_SELECTED_Msk = 0x3

	// CLK_PERI_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of ENABLED field.
	CLOCKS_CLK_PERI_CTRL_ENABLED_Pos = 0x1c
	// Bit mask of ENABLED field.
	CLOCKS_CLK_PERI_CTRL_ENABLED_Msk = 0x10000000
	// Bit ENABLED.
	CLOCKS_CLK_PERI_CTRL_ENABLED = 0x10000000
	// Position of ENABLE field.
	CLOCKS_CLK_PERI_CTRL_ENABLE_Pos = 0xb
	// Bit mask of ENABLE field.
	CLOCKS_CLK_PERI_CTRL_ENABLE_Msk = 0x800
	// Bit ENABLE.
	CLOCKS_CLK_PERI_CTRL_ENABLE = 0x800
	// Position of KILL field.
	CLOCKS_CLK_PERI_CTRL_KILL_Pos = 0xa
	// Bit mask of KILL field.
	CLOCKS_CLK_PERI_CTRL_KILL_Msk = 0x400
	// Bit KILL.
	CLOCKS_CLK_PERI_CTRL_KILL = 0x400
	// Position of AUXSRC field.
	CLOCKS_CLK_PERI_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_PERI_CTRL_AUXSRC_Msk            = 0xe0
	CLOCKS_CLK_PERI_CTRL_AUXSRC_CLK_SYS        = 0x0
	CLOCKS_CLK_PERI_CTRL_AUXSRC_CLKSRC_PLL_SYS = 0x1
	CLOCKS_CLK_PERI_CTRL_AUXSRC_CLKSRC_PLL_USB = 0x2
	CLOCKS_CLK_PERI_CTRL_AUXSRC_ROSC_CLKSRC_PH = 0x3
	CLOCKS_CLK_PERI_CTRL_AUXSRC_XOSC_CLKSRC    = 0x4
	CLOCKS_CLK_PERI_CTRL_AUXSRC_CLKSRC_GPIN0   = 0x5
	CLOCKS_CLK_PERI_CTRL_AUXSRC_CLKSRC_GPIN1   = 0x6

	// CLK_PERI_DIV
	// Position of INT field.
	CLOCKS_CLK_PERI_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_PERI_DIV_INT_Msk = 0x30000

	// CLK_PERI_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_PERI_SELECTED field.
	CLOCKS_CLK_PERI_SELECTED_CLK_PERI_SELECTED_Pos = 0x0
	// Bit mask of CLK_PERI_SELECTED field.
	CLOCKS_CLK_PERI_SELECTED_CLK_PERI_SELECTED_Msk = 0x1
	// Bit CLK_PERI_SELECTED.
	CLOCKS_CLK_PERI_SELECTED_CLK_PERI_SELECTED = 0x1

	// CLK_HSTX_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of ENABLED field.
	CLOCKS_CLK_HSTX_CTRL_ENABLED_Pos = 0x1c
	// Bit mask of ENABLED field.
	CLOCKS_CLK_HSTX_CTRL_ENABLED_Msk = 0x10000000
	// Bit ENABLED.
	CLOCKS_CLK_HSTX_CTRL_ENABLED = 0x10000000
	// Position of NUDGE field.
	CLOCKS_CLK_HSTX_CTRL_NUDGE_Pos = 0x14
	// Bit mask of NUDGE field.
	CLOCKS_CLK_HSTX_CTRL_NUDGE_Msk = 0x100000
	// Bit NUDGE.
	CLOCKS_CLK_HSTX_CTRL_NUDGE = 0x100000
	// Position of PHASE field.
	CLOCKS_CLK_HSTX_CTRL_PHASE_Pos = 0x10
	// Bit mask of PHASE field.
	CLOCKS_CLK_HSTX_CTRL_PHASE_Msk = 0x30000
	// Position of ENABLE field.
	CLOCKS_CLK_HSTX_CTRL_ENABLE_Pos = 0xb
	// Bit mask of ENABLE field.
	CLOCKS_CLK_HSTX_CTRL_ENABLE_Msk = 0x800
	// Bit ENABLE.
	CLOCKS_CLK_HSTX_CTRL_ENABLE = 0x800
	// Position of KILL field.
	CLOCKS_CLK_HSTX_CTRL_KILL_Pos = 0xa
	// Bit mask of KILL field.
	CLOCKS_CLK_HSTX_CTRL_KILL_Msk = 0x400
	// Bit KILL.
	CLOCKS_CLK_HSTX_CTRL_KILL = 0x400
	// Position of AUXSRC field.
	CLOCKS_CLK_HSTX_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_HSTX_CTRL_AUXSRC_Msk            = 0xe0
	CLOCKS_CLK_HSTX_CTRL_AUXSRC_CLK_SYS        = 0x0
	CLOCKS_CLK_HSTX_CTRL_AUXSRC_CLKSRC_PLL_SYS = 0x1
	CLOCKS_CLK_HSTX_CTRL_AUXSRC_CLKSRC_PLL_USB = 0x2
	CLOCKS_CLK_HSTX_CTRL_AUXSRC_CLKSRC_GPIN0   = 0x3
	CLOCKS_CLK_HSTX_CTRL_AUXSRC_CLKSRC_GPIN1   = 0x4

	// CLK_HSTX_DIV
	// Position of INT field.
	CLOCKS_CLK_HSTX_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_HSTX_DIV_INT_Msk = 0x30000

	// CLK_HSTX_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_HSTX_SELECTED field.
	CLOCKS_CLK_HSTX_SELECTED_CLK_HSTX_SELECTED_Pos = 0x0
	// Bit mask of CLK_HSTX_SELECTED field.
	CLOCKS_CLK_HSTX_SELECTED_CLK_HSTX_SELECTED_Msk = 0x1
	// Bit CLK_HSTX_SELECTED.
	CLOCKS_CLK_HSTX_SELECTED_CLK_HSTX_SELECTED = 0x1

	// CLK_USB_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of ENABLED field.
	CLOCKS_CLK_USB_CTRL_ENABLED_Pos = 0x1c
	// Bit mask of ENABLED field.
	CLOCKS_CLK_USB_CTRL_ENABLED_Msk = 0x10000000
	// Bit ENABLED.
	CLOCKS_CLK_USB_CTRL_ENABLED = 0x10000000
	// Position of NUDGE field.
	CLOCKS_CLK_USB_CTRL_NUDGE_Pos = 0x14
	// Bit mask of NUDGE field.
	CLOCKS_CLK_USB_CTRL_NUDGE_Msk = 0x100000
	// Bit NUDGE.
	CLOCKS_CLK_USB_CTRL_NUDGE = 0x100000
	// Position of PHASE field.
	CLOCKS_CLK_USB_CTRL_PHASE_Pos = 0x10
	// Bit mask of PHASE field.
	CLOCKS_CLK_USB_CTRL_PHASE_Msk = 0x30000
	// Position of ENABLE field.
	CLOCKS_CLK_USB_CTRL_ENABLE_Pos = 0xb
	// Bit mask of ENABLE field.
	CLOCKS_CLK_USB_CTRL_ENABLE_Msk = 0x800
	// Bit ENABLE.
	CLOCKS_CLK_USB_CTRL_ENABLE = 0x800
	// Position of KILL field.
	CLOCKS_CLK_USB_CTRL_KILL_Pos = 0xa
	// Bit mask of KILL field.
	CLOCKS_CLK_USB_CTRL_KILL_Msk = 0x400
	// Bit KILL.
	CLOCKS_CLK_USB_CTRL_KILL = 0x400
	// Position of AUXSRC field.
	CLOCKS_CLK_USB_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_USB_CTRL_AUXSRC_Msk            = 0xe0
	CLOCKS_CLK_USB_CTRL_AUXSRC_CLKSRC_PLL_USB = 0x0
	CLOCKS_CLK_USB_CTRL_AUXSRC_CLKSRC_PLL_SYS = 0x1
	CLOCKS_CLK_USB_CTRL_AUXSRC_ROSC_CLKSRC_PH = 0x2
	CLOCKS_CLK_USB_CTRL_AUXSRC_XOSC_CLKSRC    = 0x3
	CLOCKS_CLK_USB_CTRL_AUXSRC_CLKSRC_GPIN0   = 0x4
	CLOCKS_CLK_USB_CTRL_AUXSRC_CLKSRC_GPIN1   = 0x5

	// CLK_USB_DIV
	// Position of INT field.
	CLOCKS_CLK_USB_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_USB_DIV_INT_Msk = 0xf0000

	// CLK_USB_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_USB_SELECTED field.
	CLOCKS_CLK_USB_SELECTED_CLK_USB_SELECTED_Pos = 0x0
	// Bit mask of CLK_USB_SELECTED field.
	CLOCKS_CLK_USB_SELECTED_CLK_USB_SELECTED_Msk = 0x1
	// Bit CLK_USB_SELECTED.
	CLOCKS_CLK_USB_SELECTED_CLK_USB_SELECTED = 0x1

	// CLK_ADC_CTRL: Clock control, can be changed on-the-fly (except for auxsrc)
	// Position of ENABLED field.
	CLOCKS_CLK_ADC_CTRL_ENABLED_Pos = 0x1c
	// Bit mask of ENABLED field.
	CLOCKS_CLK_ADC_CTRL_ENABLED_Msk = 0x10000000
	// Bit ENABLED.
	CLOCKS_CLK_ADC_CTRL_ENABLED = 0x10000000
	// Position of NUDGE field.
	CLOCKS_CLK_ADC_CTRL_NUDGE_Pos = 0x14
	// Bit mask of NUDGE field.
	CLOCKS_CLK_ADC_CTRL_NUDGE_Msk = 0x100000
	// Bit NUDGE.
	CLOCKS_CLK_ADC_CTRL_NUDGE = 0x100000
	// Position of PHASE field.
	CLOCKS_CLK_ADC_CTRL_PHASE_Pos = 0x10
	// Bit mask of PHASE field.
	CLOCKS_CLK_ADC_CTRL_PHASE_Msk = 0x30000
	// Position of ENABLE field.
	CLOCKS_CLK_ADC_CTRL_ENABLE_Pos = 0xb
	// Bit mask of ENABLE field.
	CLOCKS_CLK_ADC_CTRL_ENABLE_Msk = 0x800
	// Bit ENABLE.
	CLOCKS_CLK_ADC_CTRL_ENABLE = 0x800
	// Position of KILL field.
	CLOCKS_CLK_ADC_CTRL_KILL_Pos = 0xa
	// Bit mask of KILL field.
	CLOCKS_CLK_ADC_CTRL_KILL_Msk = 0x400
	// Bit KILL.
	CLOCKS_CLK_ADC_CTRL_KILL = 0x400
	// Position of AUXSRC field.
	CLOCKS_CLK_ADC_CTRL_AUXSRC_Pos = 0x5
	// Bit mask of AUXSRC field.
	CLOCKS_CLK_ADC_CTRL_AUXSRC_Msk            = 0xe0
	CLOCKS_CLK_ADC_CTRL_AUXSRC_CLKSRC_PLL_USB = 0x0
	CLOCKS_CLK_ADC_CTRL_AUXSRC_CLKSRC_PLL_SYS = 0x1
	CLOCKS_CLK_ADC_CTRL_AUXSRC_ROSC_CLKSRC_PH = 0x2
	CLOCKS_CLK_ADC_CTRL_AUXSRC_XOSC_CLKSRC    = 0x3
	CLOCKS_CLK_ADC_CTRL_AUXSRC_CLKSRC_GPIN0   = 0x4
	CLOCKS_CLK_ADC_CTRL_AUXSRC_CLKSRC_GPIN1   = 0x5

	// CLK_ADC_DIV
	// Position of INT field.
	CLOCKS_CLK_ADC_DIV_INT_Pos = 0x10
	// Bit mask of INT field.
	CLOCKS_CLK_ADC_DIV_INT_Msk = 0xf0000

	// CLK_ADC_SELECTED: Indicates which src is currently selected (one-hot)
	// Position of CLK_ADC_SELECTED field.
	CLOCKS_CLK_ADC_SELECTED_CLK_ADC_SELECTED_Pos = 0x0
	// Bit mask of CLK_ADC_SELECTED field.
	CLOCKS_CLK_ADC_SELECTED_CLK_ADC_SELECTED_Msk = 0x1
	// Bit CLK_ADC_SELECTED.
	CLOCKS_CLK_ADC_SELECTED_CLK_ADC_SELECTED = 0x1

	// DFTCLK_XOSC_CTRL
	// Position of SRC field.
	CLOCKS_DFTCLK_XOSC_CTRL_SRC_Pos = 0x0
	// Bit mask of SRC field.
	CLOCKS_DFTCLK_XOSC_CTRL_SRC_Msk                    = 0x3
	CLOCKS_DFTCLK_XOSC_CTRL_SRC_NULL                   = 0x0
	CLOCKS_DFTCLK_XOSC_CTRL_SRC_CLKSRC_PLL_USB_PRIMARY = 0x1
	CLOCKS_DFTCLK_XOSC_CTRL_SRC_CLKSRC_GPIN0           = 0x2

	// DFTCLK_ROSC_CTRL
	// Position of SRC field.
	CLOCKS_DFTCLK_ROSC_CTRL_SRC_Pos = 0x0
	// Bit mask of SRC field.
	CLOCKS_DFTCLK_ROSC_CTRL_SRC_Msk                         = 0x3
	CLOCKS_DFTCLK_ROSC_CTRL_SRC_NULL                        = 0x0
	CLOCKS_DFTCLK_ROSC_CTRL_SRC_CLKSRC_PLL_SYS_PRIMARY_ROSC = 0x1
	CLOCKS_DFTCLK_ROSC_CTRL_SRC_CLKSRC_GPIN1                = 0x2

	// DFTCLK_LPOSC_CTRL
	// Position of SRC field.
	CLOCKS_DFTCLK_LPOSC_CTRL_SRC_Pos = 0x0
	// Bit mask of SRC field.
	CLOCKS_DFTCLK_LPOSC_CTRL_SRC_Msk                          = 0x3
	CLOCKS_DFTCLK_LPOSC_CTRL_SRC_NULL                         = 0x0
	CLOCKS_DFTCLK_LPOSC_CTRL_SRC_CLKSRC_PLL_USB_PRIMARY_LPOSC = 0x1
	CLOCKS_DFTCLK_LPOSC_CTRL_SRC_CLKSRC_GPIN1                 = 0x2

	// CLK_SYS_RESUS_CTRL
	// Position of CLEAR field.
	CLOCKS_CLK_SYS_RESUS_CTRL_CLEAR_Pos = 0x10
	// Bit mask of CLEAR field.
	CLOCKS_CLK_SYS_RESUS_CTRL_CLEAR_Msk = 0x10000
	// Bit CLEAR.
	CLOCKS_CLK_SYS_RESUS_CTRL_CLEAR = 0x10000
	// Position of FRCE field.
	CLOCKS_CLK_SYS_RESUS_CTRL_FRCE_Pos = 0xc
	// Bit mask of FRCE field.
	CLOCKS_CLK_SYS_RESUS_CTRL_FRCE_Msk = 0x1000
	// Bit FRCE.
	CLOCKS_CLK_SYS_RESUS_CTRL_FRCE = 0x1000
	// Position of ENABLE field.
	CLOCKS_CLK_SYS_RESUS_CTRL_ENABLE_Pos = 0x8
	// Bit mask of ENABLE field.
	CLOCKS_CLK_SYS_RESUS_CTRL_ENABLE_Msk = 0x100
	// Bit ENABLE.
	CLOCKS_CLK_SYS_RESUS_CTRL_ENABLE = 0x100
	// Position of TIMEOUT field.
	CLOCKS_CLK_SYS_RESUS_CTRL_TIMEOUT_Pos = 0x0
	// Bit mask of TIMEOUT field.
	CLOCKS_CLK_SYS_RESUS_CTRL_TIMEOUT_Msk = 0xff

	// CLK_SYS_RESUS_STATUS
	// Position of RESUSSED field.
	CLOCKS_CLK_SYS_RESUS_STATUS_RESUSSED_Pos = 0x0
	// Bit mask of RESUSSED field.
	CLOCKS_CLK_SYS_RESUS_STATUS_RESUSSED_Msk = 0x1
	// Bit RESUSSED.
	CLOCKS_CLK_SYS_RESUS_STATUS_RESUSSED = 0x1

	// FC0_REF_KHZ: Reference clock frequency in kHz
	// Position of FC0_REF_KHZ field.
	CLOCKS_FC0_REF_KHZ_FC0_REF_KHZ_Pos = 0x0
	// Bit mask of FC0_REF_KHZ field.
	CLOCKS_FC0_REF_KHZ_FC0_REF_KHZ_Msk = 0xfffff

	// FC0_MIN_KHZ: Minimum pass frequency in kHz. This is optional. Set to 0 if you are not using the pass/fail flags
	// Position of FC0_MIN_KHZ field.
	CLOCKS_FC0_MIN_KHZ_FC0_MIN_KHZ_Pos = 0x0
	// Bit mask of FC0_MIN_KHZ field.
	CLOCKS_FC0_MIN_KHZ_FC0_MIN_KHZ_Msk = 0x1ffffff

	// FC0_MAX_KHZ: Maximum pass frequency in kHz. This is optional. Set to 0x1ffffff if you are not using the pass/fail flags
	// Position of FC0_MAX_KHZ field.
	CLOCKS_FC0_MAX_KHZ_FC0_MAX_KHZ_Pos = 0x0
	// Bit mask of FC0_MAX_KHZ field.
	CLOCKS_FC0_MAX_KHZ_FC0_MAX_KHZ_Msk = 0x1ffffff

	// FC0_DELAY: Delays the start of frequency counting to allow the mux to settle Delay is measured in multiples of the reference clock period
	// Position of FC0_DELAY field.
	CLOCKS_FC0_DELAY_FC0_DELAY_Pos = 0x0
	// Bit mask of FC0_DELAY field.
	CLOCKS_FC0_DELAY_FC0_DELAY_Msk = 0x7

	// FC0_INTERVAL: The test interval is 0.98us * 2**interval, but let's call it 1us * 2**interval The default gives a test interval of 250us
	// Position of FC0_INTERVAL field.
	CLOCKS_FC0_INTERVAL_FC0_INTERVAL_Pos = 0x0
	// Bit mask of FC0_INTERVAL field.
	CLOCKS_FC0_INTERVAL_FC0_INTERVAL_Msk = 0xf

	// FC0_SRC: Clock sent to frequency counter, set to 0 when not required Writing to this register initiates the frequency count
	// Position of FC0_SRC field.
	CLOCKS_FC0_SRC_FC0_SRC_Pos = 0x0
	// Bit mask of FC0_SRC field.
	CLOCKS_FC0_SRC_FC0_SRC_Msk                        = 0xff
	CLOCKS_FC0_SRC_FC0_SRC_NULL                       = 0x0
	CLOCKS_FC0_SRC_FC0_SRC_PLL_SYS_CLKSRC_PRIMARY     = 0x1
	CLOCKS_FC0_SRC_FC0_SRC_PLL_USB_CLKSRC_PRIMARY     = 0x2
	CLOCKS_FC0_SRC_FC0_SRC_ROSC_CLKSRC                = 0x3
	CLOCKS_FC0_SRC_FC0_SRC_ROSC_CLKSRC_PH             = 0x4
	CLOCKS_FC0_SRC_FC0_SRC_XOSC_CLKSRC                = 0x5
	CLOCKS_FC0_SRC_FC0_SRC_CLKSRC_GPIN0               = 0x6
	CLOCKS_FC0_SRC_FC0_SRC_CLKSRC_GPIN1               = 0x7
	CLOCKS_FC0_SRC_FC0_SRC_CLK_REF                    = 0x8
	CLOCKS_FC0_SRC_FC0_SRC_CLK_SYS                    = 0x9
	CLOCKS_FC0_SRC_FC0_SRC_CLK_PERI                   = 0xa
	CLOCKS_FC0_SRC_FC0_SRC_CLK_USB                    = 0xb
	CLOCKS_FC0_SRC_FC0_SRC_CLK_ADC                    = 0xc
	CLOCKS_FC0_SRC_FC0_SRC_CLK_HSTX                   = 0xd
	CLOCKS_FC0_SRC_FC0_SRC_LPOSC_CLKSRC               = 0xe
	CLOCKS_FC0_SRC_FC0_SRC_OTP_CLK2FC                 = 0xf
	CLOCKS_FC0_SRC_FC0_SRC_PLL_USB_CLKSRC_PRIMARY_DFT = 0x10

	// FC0_STATUS: Frequency counter status
	// Position of DIED field.
	CLOCKS_FC0_STATUS_DIED_Pos = 0x1c
	// Bit mask of DIED field.
	CLOCKS_FC0_STATUS_DIED_Msk = 0x10000000
	// Bit DIED.
	CLOCKS_FC0_STATUS_DIED = 0x10000000
	// Position of FAST field.
	CLOCKS_FC0_STATUS_FAST_Pos = 0x18
	// Bit mask of FAST field.
	CLOCKS_FC0_STATUS_FAST_Msk = 0x1000000
	// Bit FAST.
	CLOCKS_FC0_STATUS_FAST = 0x1000000
	// Position of SLOW field.
	CLOCKS_FC0_STATUS_SLOW_Pos = 0x14
	// Bit mask of SLOW field.
	CLOCKS_FC0_STATUS_SLOW_Msk = 0x100000
	// Bit SLOW.
	CLOCKS_FC0_STATUS_SLOW = 0x100000
	// Position of FAIL field.
	CLOCKS_FC0_STATUS_FAIL_Pos = 0x10
	// Bit mask of FAIL field.
	CLOCKS_FC0_STATUS_FAIL_Msk = 0x10000
	// Bit FAIL.
	CLOCKS_FC0_STATUS_FAIL = 0x10000
	// Position of WAITING field.
	CLOCKS_FC0_STATUS_WAITING_Pos = 0xc
	// Bit mask of WAITING field.
	CLOCKS_FC0_STATUS_WAITING_Msk = 0x1000
	// Bit WAITING.
	CLOCKS_FC0_STATUS_WAITING = 0x1000
	// Position of RUNNING field.
	CLOCKS_FC0_STATUS_RUNNING_Pos = 0x8
	// Bit mask of RUNNING field.
	CLOCKS_FC0_STATUS_RUNNING_Msk = 0x100
	// Bit RUNNING.
	CLOCKS_FC0_STATUS_RUNNING = 0x100
	// Position of DONE field.
	CLOCKS_FC0_STATUS_DONE_Pos = 0x4
	// Bit mask of DONE field.
	CLOCKS_FC0_STATUS_DONE_Msk = 0x10
	// Bit DONE.
	CLOCKS_FC0_STATUS_DONE = 0x10
	// Position of PASS field.
	CLOCKS_FC0_STATUS_PASS_Pos = 0x0
	// Bit mask of PASS field.
	CLOCKS_FC0_STATUS_PASS_Msk = 0x1
	// Bit PASS.
	CLOCKS_FC0_STATUS_PASS = 0x1

	// FC0_RESULT: Result of frequency measurement, only valid when status_done=1
	// Position of KHZ field.
	CLOCKS_FC0_RESULT_KHZ_Pos = 0x5
	// Bit mask of KHZ field.
	CLOCKS_FC0_RESULT_KHZ_Msk = 0x3fffffe0
	// Position of FRAC field.
	CLOCKS_FC0_RESULT_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	CLOCKS_FC0_RESULT_FRAC_Msk = 0x1f

	// WAKE_EN0: enable clock in wake mode
	// Position of CLK_SYS_SIO field.
	CLOCKS_WAKE_EN0_CLK_SYS_SIO_Pos = 0x1f
	// Bit mask of CLK_SYS_SIO field.
	CLOCKS_WAKE_EN0_CLK_SYS_SIO_Msk = 0x80000000
	// Bit CLK_SYS_SIO.
	CLOCKS_WAKE_EN0_CLK_SYS_SIO = 0x80000000
	// Position of CLK_SYS_SHA256 field.
	CLOCKS_WAKE_EN0_CLK_SYS_SHA256_Pos = 0x1e
	// Bit mask of CLK_SYS_SHA256 field.
	CLOCKS_WAKE_EN0_CLK_SYS_SHA256_Msk = 0x40000000
	// Bit CLK_SYS_SHA256.
	CLOCKS_WAKE_EN0_CLK_SYS_SHA256 = 0x40000000
	// Position of CLK_SYS_PSM field.
	CLOCKS_WAKE_EN0_CLK_SYS_PSM_Pos = 0x1d
	// Bit mask of CLK_SYS_PSM field.
	CLOCKS_WAKE_EN0_CLK_SYS_PSM_Msk = 0x20000000
	// Bit CLK_SYS_PSM.
	CLOCKS_WAKE_EN0_CLK_SYS_PSM = 0x20000000
	// Position of CLK_SYS_ROSC field.
	CLOCKS_WAKE_EN0_CLK_SYS_ROSC_Pos = 0x1c
	// Bit mask of CLK_SYS_ROSC field.
	CLOCKS_WAKE_EN0_CLK_SYS_ROSC_Msk = 0x10000000
	// Bit CLK_SYS_ROSC.
	CLOCKS_WAKE_EN0_CLK_SYS_ROSC = 0x10000000
	// Position of CLK_SYS_ROM field.
	CLOCKS_WAKE_EN0_CLK_SYS_ROM_Pos = 0x1b
	// Bit mask of CLK_SYS_ROM field.
	CLOCKS_WAKE_EN0_CLK_SYS_ROM_Msk = 0x8000000
	// Bit CLK_SYS_ROM.
	CLOCKS_WAKE_EN0_CLK_SYS_ROM = 0x8000000
	// Position of CLK_SYS_RESETS field.
	CLOCKS_WAKE_EN0_CLK_SYS_RESETS_Pos = 0x1a
	// Bit mask of CLK_SYS_RESETS field.
	CLOCKS_WAKE_EN0_CLK_SYS_RESETS_Msk = 0x4000000
	// Bit CLK_SYS_RESETS.
	CLOCKS_WAKE_EN0_CLK_SYS_RESETS = 0x4000000
	// Position of CLK_SYS_PWM field.
	CLOCKS_WAKE_EN0_CLK_SYS_PWM_Pos = 0x19
	// Bit mask of CLK_SYS_PWM field.
	CLOCKS_WAKE_EN0_CLK_SYS_PWM_Msk = 0x2000000
	// Bit CLK_SYS_PWM.
	CLOCKS_WAKE_EN0_CLK_SYS_PWM = 0x2000000
	// Position of CLK_SYS_POWMAN field.
	CLOCKS_WAKE_EN0_CLK_SYS_POWMAN_Pos = 0x18
	// Bit mask of CLK_SYS_POWMAN field.
	CLOCKS_WAKE_EN0_CLK_SYS_POWMAN_Msk = 0x1000000
	// Bit CLK_SYS_POWMAN.
	CLOCKS_WAKE_EN0_CLK_SYS_POWMAN = 0x1000000
	// Position of CLK_REF_POWMAN field.
	CLOCKS_WAKE_EN0_CLK_REF_POWMAN_Pos = 0x17
	// Bit mask of CLK_REF_POWMAN field.
	CLOCKS_WAKE_EN0_CLK_REF_POWMAN_Msk = 0x800000
	// Bit CLK_REF_POWMAN.
	CLOCKS_WAKE_EN0_CLK_REF_POWMAN = 0x800000
	// Position of CLK_SYS_PLL_USB field.
	CLOCKS_WAKE_EN0_CLK_SYS_PLL_USB_Pos = 0x16
	// Bit mask of CLK_SYS_PLL_USB field.
	CLOCKS_WAKE_EN0_CLK_SYS_PLL_USB_Msk = 0x400000
	// Bit CLK_SYS_PLL_USB.
	CLOCKS_WAKE_EN0_CLK_SYS_PLL_USB = 0x400000
	// Position of CLK_SYS_PLL_SYS field.
	CLOCKS_WAKE_EN0_CLK_SYS_PLL_SYS_Pos = 0x15
	// Bit mask of CLK_SYS_PLL_SYS field.
	CLOCKS_WAKE_EN0_CLK_SYS_PLL_SYS_Msk = 0x200000
	// Bit CLK_SYS_PLL_SYS.
	CLOCKS_WAKE_EN0_CLK_SYS_PLL_SYS = 0x200000
	// Position of CLK_SYS_PIO2 field.
	CLOCKS_WAKE_EN0_CLK_SYS_PIO2_Pos = 0x14
	// Bit mask of CLK_SYS_PIO2 field.
	CLOCKS_WAKE_EN0_CLK_SYS_PIO2_Msk = 0x100000
	// Bit CLK_SYS_PIO2.
	CLOCKS_WAKE_EN0_CLK_SYS_PIO2 = 0x100000
	// Position of CLK_SYS_PIO1 field.
	CLOCKS_WAKE_EN0_CLK_SYS_PIO1_Pos = 0x13
	// Bit mask of CLK_SYS_PIO1 field.
	CLOCKS_WAKE_EN0_CLK_SYS_PIO1_Msk = 0x80000
	// Bit CLK_SYS_PIO1.
	CLOCKS_WAKE_EN0_CLK_SYS_PIO1 = 0x80000
	// Position of CLK_SYS_PIO0 field.
	CLOCKS_WAKE_EN0_CLK_SYS_PIO0_Pos = 0x12
	// Bit mask of CLK_SYS_PIO0 field.
	CLOCKS_WAKE_EN0_CLK_SYS_PIO0_Msk = 0x40000
	// Bit CLK_SYS_PIO0.
	CLOCKS_WAKE_EN0_CLK_SYS_PIO0 = 0x40000
	// Position of CLK_SYS_PADS field.
	CLOCKS_WAKE_EN0_CLK_SYS_PADS_Pos = 0x11
	// Bit mask of CLK_SYS_PADS field.
	CLOCKS_WAKE_EN0_CLK_SYS_PADS_Msk = 0x20000
	// Bit CLK_SYS_PADS.
	CLOCKS_WAKE_EN0_CLK_SYS_PADS = 0x20000
	// Position of CLK_SYS_OTP field.
	CLOCKS_WAKE_EN0_CLK_SYS_OTP_Pos = 0x10
	// Bit mask of CLK_SYS_OTP field.
	CLOCKS_WAKE_EN0_CLK_SYS_OTP_Msk = 0x10000
	// Bit CLK_SYS_OTP.
	CLOCKS_WAKE_EN0_CLK_SYS_OTP = 0x10000
	// Position of CLK_REF_OTP field.
	CLOCKS_WAKE_EN0_CLK_REF_OTP_Pos = 0xf
	// Bit mask of CLK_REF_OTP field.
	CLOCKS_WAKE_EN0_CLK_REF_OTP_Msk = 0x8000
	// Bit CLK_REF_OTP.
	CLOCKS_WAKE_EN0_CLK_REF_OTP = 0x8000
	// Position of CLK_SYS_JTAG field.
	CLOCKS_WAKE_EN0_CLK_SYS_JTAG_Pos = 0xe
	// Bit mask of CLK_SYS_JTAG field.
	CLOCKS_WAKE_EN0_CLK_SYS_JTAG_Msk = 0x4000
	// Bit CLK_SYS_JTAG.
	CLOCKS_WAKE_EN0_CLK_SYS_JTAG = 0x4000
	// Position of CLK_SYS_IO field.
	CLOCKS_WAKE_EN0_CLK_SYS_IO_Pos = 0xd
	// Bit mask of CLK_SYS_IO field.
	CLOCKS_WAKE_EN0_CLK_SYS_IO_Msk = 0x2000
	// Bit CLK_SYS_IO.
	CLOCKS_WAKE_EN0_CLK_SYS_IO = 0x2000
	// Position of CLK_SYS_I2C1 field.
	CLOCKS_WAKE_EN0_CLK_SYS_I2C1_Pos = 0xc
	// Bit mask of CLK_SYS_I2C1 field.
	CLOCKS_WAKE_EN0_CLK_SYS_I2C1_Msk = 0x1000
	// Bit CLK_SYS_I2C1.
	CLOCKS_WAKE_EN0_CLK_SYS_I2C1 = 0x1000
	// Position of CLK_SYS_I2C0 field.
	CLOCKS_WAKE_EN0_CLK_SYS_I2C0_Pos = 0xb
	// Bit mask of CLK_SYS_I2C0 field.
	CLOCKS_WAKE_EN0_CLK_SYS_I2C0_Msk = 0x800
	// Bit CLK_SYS_I2C0.
	CLOCKS_WAKE_EN0_CLK_SYS_I2C0 = 0x800
	// Position of CLK_SYS_HSTX field.
	CLOCKS_WAKE_EN0_CLK_SYS_HSTX_Pos = 0xa
	// Bit mask of CLK_SYS_HSTX field.
	CLOCKS_WAKE_EN0_CLK_SYS_HSTX_Msk = 0x400
	// Bit CLK_SYS_HSTX.
	CLOCKS_WAKE_EN0_CLK_SYS_HSTX = 0x400
	// Position of CLK_HSTX field.
	CLOCKS_WAKE_EN0_CLK_HSTX_Pos = 0x9
	// Bit mask of CLK_HSTX field.
	CLOCKS_WAKE_EN0_CLK_HSTX_Msk = 0x200
	// Bit CLK_HSTX.
	CLOCKS_WAKE_EN0_CLK_HSTX = 0x200
	// Position of CLK_SYS_GLITCH_DETECTOR field.
	CLOCKS_WAKE_EN0_CLK_SYS_GLITCH_DETECTOR_Pos = 0x8
	// Bit mask of CLK_SYS_GLITCH_DETECTOR field.
	CLOCKS_WAKE_EN0_CLK_SYS_GLITCH_DETECTOR_Msk = 0x100
	// Bit CLK_SYS_GLITCH_DETECTOR.
	CLOCKS_WAKE_EN0_CLK_SYS_GLITCH_DETECTOR = 0x100
	// Position of CLK_SYS_DMA field.
	CLOCKS_WAKE_EN0_CLK_SYS_DMA_Pos = 0x7
	// Bit mask of CLK_SYS_DMA field.
	CLOCKS_WAKE_EN0_CLK_SYS_DMA_Msk = 0x80
	// Bit CLK_SYS_DMA.
	CLOCKS_WAKE_EN0_CLK_SYS_DMA = 0x80
	// Position of CLK_SYS_BUSFABRIC field.
	CLOCKS_WAKE_EN0_CLK_SYS_BUSFABRIC_Pos = 0x6
	// Bit mask of CLK_SYS_BUSFABRIC field.
	CLOCKS_WAKE_EN0_CLK_SYS_BUSFABRIC_Msk = 0x40
	// Bit CLK_SYS_BUSFABRIC.
	CLOCKS_WAKE_EN0_CLK_SYS_BUSFABRIC = 0x40
	// Position of CLK_SYS_BUSCTRL field.
	CLOCKS_WAKE_EN0_CLK_SYS_BUSCTRL_Pos = 0x5
	// Bit mask of CLK_SYS_BUSCTRL field.
	CLOCKS_WAKE_EN0_CLK_SYS_BUSCTRL_Msk = 0x20
	// Bit CLK_SYS_BUSCTRL.
	CLOCKS_WAKE_EN0_CLK_SYS_BUSCTRL = 0x20
	// Position of CLK_SYS_BOOTRAM field.
	CLOCKS_WAKE_EN0_CLK_SYS_BOOTRAM_Pos = 0x4
	// Bit mask of CLK_SYS_BOOTRAM field.
	CLOCKS_WAKE_EN0_CLK_SYS_BOOTRAM_Msk = 0x10
	// Bit CLK_SYS_BOOTRAM.
	CLOCKS_WAKE_EN0_CLK_SYS_BOOTRAM = 0x10
	// Position of CLK_SYS_ADC field.
	CLOCKS_WAKE_EN0_CLK_SYS_ADC_Pos = 0x3
	// Bit mask of CLK_SYS_ADC field.
	CLOCKS_WAKE_EN0_CLK_SYS_ADC_Msk = 0x8
	// Bit CLK_SYS_ADC.
	CLOCKS_WAKE_EN0_CLK_SYS_ADC = 0x8
	// Position of CLK_ADC field.
	CLOCKS_WAKE_EN0_CLK_ADC_Pos = 0x2
	// Bit mask of CLK_ADC field.
	CLOCKS_WAKE_EN0_CLK_ADC_Msk = 0x4
	// Bit CLK_ADC.
	CLOCKS_WAKE_EN0_CLK_ADC = 0x4
	// Position of CLK_SYS_ACCESSCTRL field.
	CLOCKS_WAKE_EN0_CLK_SYS_ACCESSCTRL_Pos = 0x1
	// Bit mask of CLK_SYS_ACCESSCTRL field.
	CLOCKS_WAKE_EN0_CLK_SYS_ACCESSCTRL_Msk = 0x2
	// Bit CLK_SYS_ACCESSCTRL.
	CLOCKS_WAKE_EN0_CLK_SYS_ACCESSCTRL = 0x2
	// Position of CLK_SYS_CLOCKS field.
	CLOCKS_WAKE_EN0_CLK_SYS_CLOCKS_Pos = 0x0
	// Bit mask of CLK_SYS_CLOCKS field.
	CLOCKS_WAKE_EN0_CLK_SYS_CLOCKS_Msk = 0x1
	// Bit CLK_SYS_CLOCKS.
	CLOCKS_WAKE_EN0_CLK_SYS_CLOCKS = 0x1

	// WAKE_EN1: enable clock in wake mode
	// Position of CLK_SYS_XOSC field.
	CLOCKS_WAKE_EN1_CLK_SYS_XOSC_Pos = 0x1e
	// Bit mask of CLK_SYS_XOSC field.
	CLOCKS_WAKE_EN1_CLK_SYS_XOSC_Msk = 0x40000000
	// Bit CLK_SYS_XOSC.
	CLOCKS_WAKE_EN1_CLK_SYS_XOSC = 0x40000000
	// Position of CLK_SYS_XIP field.
	CLOCKS_WAKE_EN1_CLK_SYS_XIP_Pos = 0x1d
	// Bit mask of CLK_SYS_XIP field.
	CLOCKS_WAKE_EN1_CLK_SYS_XIP_Msk = 0x20000000
	// Bit CLK_SYS_XIP.
	CLOCKS_WAKE_EN1_CLK_SYS_XIP = 0x20000000
	// Position of CLK_SYS_WATCHDOG field.
	CLOCKS_WAKE_EN1_CLK_SYS_WATCHDOG_Pos = 0x1c
	// Bit mask of CLK_SYS_WATCHDOG field.
	CLOCKS_WAKE_EN1_CLK_SYS_WATCHDOG_Msk = 0x10000000
	// Bit CLK_SYS_WATCHDOG.
	CLOCKS_WAKE_EN1_CLK_SYS_WATCHDOG = 0x10000000
	// Position of CLK_USB field.
	CLOCKS_WAKE_EN1_CLK_USB_Pos = 0x1b
	// Bit mask of CLK_USB field.
	CLOCKS_WAKE_EN1_CLK_USB_Msk = 0x8000000
	// Bit CLK_USB.
	CLOCKS_WAKE_EN1_CLK_USB = 0x8000000
	// Position of CLK_SYS_USBCTRL field.
	CLOCKS_WAKE_EN1_CLK_SYS_USBCTRL_Pos = 0x1a
	// Bit mask of CLK_SYS_USBCTRL field.
	CLOCKS_WAKE_EN1_CLK_SYS_USBCTRL_Msk = 0x4000000
	// Bit CLK_SYS_USBCTRL.
	CLOCKS_WAKE_EN1_CLK_SYS_USBCTRL = 0x4000000
	// Position of CLK_SYS_UART1 field.
	CLOCKS_WAKE_EN1_CLK_SYS_UART1_Pos = 0x19
	// Bit mask of CLK_SYS_UART1 field.
	CLOCKS_WAKE_EN1_CLK_SYS_UART1_Msk = 0x2000000
	// Bit CLK_SYS_UART1.
	CLOCKS_WAKE_EN1_CLK_SYS_UART1 = 0x2000000
	// Position of CLK_PERI_UART1 field.
	CLOCKS_WAKE_EN1_CLK_PERI_UART1_Pos = 0x18
	// Bit mask of CLK_PERI_UART1 field.
	CLOCKS_WAKE_EN1_CLK_PERI_UART1_Msk = 0x1000000
	// Bit CLK_PERI_UART1.
	CLOCKS_WAKE_EN1_CLK_PERI_UART1 = 0x1000000
	// Position of CLK_SYS_UART0 field.
	CLOCKS_WAKE_EN1_CLK_SYS_UART0_Pos = 0x17
	// Bit mask of CLK_SYS_UART0 field.
	CLOCKS_WAKE_EN1_CLK_SYS_UART0_Msk = 0x800000
	// Bit CLK_SYS_UART0.
	CLOCKS_WAKE_EN1_CLK_SYS_UART0 = 0x800000
	// Position of CLK_PERI_UART0 field.
	CLOCKS_WAKE_EN1_CLK_PERI_UART0_Pos = 0x16
	// Bit mask of CLK_PERI_UART0 field.
	CLOCKS_WAKE_EN1_CLK_PERI_UART0_Msk = 0x400000
	// Bit CLK_PERI_UART0.
	CLOCKS_WAKE_EN1_CLK_PERI_UART0 = 0x400000
	// Position of CLK_SYS_TRNG field.
	CLOCKS_WAKE_EN1_CLK_SYS_TRNG_Pos = 0x15
	// Bit mask of CLK_SYS_TRNG field.
	CLOCKS_WAKE_EN1_CLK_SYS_TRNG_Msk = 0x200000
	// Bit CLK_SYS_TRNG.
	CLOCKS_WAKE_EN1_CLK_SYS_TRNG = 0x200000
	// Position of CLK_SYS_TIMER1 field.
	CLOCKS_WAKE_EN1_CLK_SYS_TIMER1_Pos = 0x14
	// Bit mask of CLK_SYS_TIMER1 field.
	CLOCKS_WAKE_EN1_CLK_SYS_TIMER1_Msk = 0x100000
	// Bit CLK_SYS_TIMER1.
	CLOCKS_WAKE_EN1_CLK_SYS_TIMER1 = 0x100000
	// Position of CLK_SYS_TIMER0 field.
	CLOCKS_WAKE_EN1_CLK_SYS_TIMER0_Pos = 0x13
	// Bit mask of CLK_SYS_TIMER0 field.
	CLOCKS_WAKE_EN1_CLK_SYS_TIMER0_Msk = 0x80000
	// Bit CLK_SYS_TIMER0.
	CLOCKS_WAKE_EN1_CLK_SYS_TIMER0 = 0x80000
	// Position of CLK_SYS_TICKS field.
	CLOCKS_WAKE_EN1_CLK_SYS_TICKS_Pos = 0x12
	// Bit mask of CLK_SYS_TICKS field.
	CLOCKS_WAKE_EN1_CLK_SYS_TICKS_Msk = 0x40000
	// Bit CLK_SYS_TICKS.
	CLOCKS_WAKE_EN1_CLK_SYS_TICKS = 0x40000
	// Position of CLK_REF_TICKS field.
	CLOCKS_WAKE_EN1_CLK_REF_TICKS_Pos = 0x11
	// Bit mask of CLK_REF_TICKS field.
	CLOCKS_WAKE_EN1_CLK_REF_TICKS_Msk = 0x20000
	// Bit CLK_REF_TICKS.
	CLOCKS_WAKE_EN1_CLK_REF_TICKS = 0x20000
	// Position of CLK_SYS_TBMAN field.
	CLOCKS_WAKE_EN1_CLK_SYS_TBMAN_Pos = 0x10
	// Bit mask of CLK_SYS_TBMAN field.
	CLOCKS_WAKE_EN1_CLK_SYS_TBMAN_Msk = 0x10000
	// Bit CLK_SYS_TBMAN.
	CLOCKS_WAKE_EN1_CLK_SYS_TBMAN = 0x10000
	// Position of CLK_SYS_SYSINFO field.
	CLOCKS_WAKE_EN1_CLK_SYS_SYSINFO_Pos = 0xf
	// Bit mask of CLK_SYS_SYSINFO field.
	CLOCKS_WAKE_EN1_CLK_SYS_SYSINFO_Msk = 0x8000
	// Bit CLK_SYS_SYSINFO.
	CLOCKS_WAKE_EN1_CLK_SYS_SYSINFO = 0x8000
	// Position of CLK_SYS_SYSCFG field.
	CLOCKS_WAKE_EN1_CLK_SYS_SYSCFG_Pos = 0xe
	// Bit mask of CLK_SYS_SYSCFG field.
	CLOCKS_WAKE_EN1_CLK_SYS_SYSCFG_Msk = 0x4000
	// Bit CLK_SYS_SYSCFG.
	CLOCKS_WAKE_EN1_CLK_SYS_SYSCFG = 0x4000
	// Position of CLK_SYS_SRAM9 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM9_Pos = 0xd
	// Bit mask of CLK_SYS_SRAM9 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM9_Msk = 0x2000
	// Bit CLK_SYS_SRAM9.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM9 = 0x2000
	// Position of CLK_SYS_SRAM8 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM8_Pos = 0xc
	// Bit mask of CLK_SYS_SRAM8 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM8_Msk = 0x1000
	// Bit CLK_SYS_SRAM8.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM8 = 0x1000
	// Position of CLK_SYS_SRAM7 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM7_Pos = 0xb
	// Bit mask of CLK_SYS_SRAM7 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM7_Msk = 0x800
	// Bit CLK_SYS_SRAM7.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM7 = 0x800
	// Position of CLK_SYS_SRAM6 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM6_Pos = 0xa
	// Bit mask of CLK_SYS_SRAM6 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM6_Msk = 0x400
	// Bit CLK_SYS_SRAM6.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM6 = 0x400
	// Position of CLK_SYS_SRAM5 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM5_Pos = 0x9
	// Bit mask of CLK_SYS_SRAM5 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM5_Msk = 0x200
	// Bit CLK_SYS_SRAM5.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM5 = 0x200
	// Position of CLK_SYS_SRAM4 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM4_Pos = 0x8
	// Bit mask of CLK_SYS_SRAM4 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM4_Msk = 0x100
	// Bit CLK_SYS_SRAM4.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM4 = 0x100
	// Position of CLK_SYS_SRAM3 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM3_Pos = 0x7
	// Bit mask of CLK_SYS_SRAM3 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM3_Msk = 0x80
	// Bit CLK_SYS_SRAM3.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM3 = 0x80
	// Position of CLK_SYS_SRAM2 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM2_Pos = 0x6
	// Bit mask of CLK_SYS_SRAM2 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM2_Msk = 0x40
	// Bit CLK_SYS_SRAM2.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM2 = 0x40
	// Position of CLK_SYS_SRAM1 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM1_Pos = 0x5
	// Bit mask of CLK_SYS_SRAM1 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM1_Msk = 0x20
	// Bit CLK_SYS_SRAM1.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM1 = 0x20
	// Position of CLK_SYS_SRAM0 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM0_Pos = 0x4
	// Bit mask of CLK_SYS_SRAM0 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM0_Msk = 0x10
	// Bit CLK_SYS_SRAM0.
	CLOCKS_WAKE_EN1_CLK_SYS_SRAM0 = 0x10
	// Position of CLK_SYS_SPI1 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SPI1_Pos = 0x3
	// Bit mask of CLK_SYS_SPI1 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SPI1_Msk = 0x8
	// Bit CLK_SYS_SPI1.
	CLOCKS_WAKE_EN1_CLK_SYS_SPI1 = 0x8
	// Position of CLK_PERI_SPI1 field.
	CLOCKS_WAKE_EN1_CLK_PERI_SPI1_Pos = 0x2
	// Bit mask of CLK_PERI_SPI1 field.
	CLOCKS_WAKE_EN1_CLK_PERI_SPI1_Msk = 0x4
	// Bit CLK_PERI_SPI1.
	CLOCKS_WAKE_EN1_CLK_PERI_SPI1 = 0x4
	// Position of CLK_SYS_SPI0 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SPI0_Pos = 0x1
	// Bit mask of CLK_SYS_SPI0 field.
	CLOCKS_WAKE_EN1_CLK_SYS_SPI0_Msk = 0x2
	// Bit CLK_SYS_SPI0.
	CLOCKS_WAKE_EN1_CLK_SYS_SPI0 = 0x2
	// Position of CLK_PERI_SPI0 field.
	CLOCKS_WAKE_EN1_CLK_PERI_SPI0_Pos = 0x0
	// Bit mask of CLK_PERI_SPI0 field.
	CLOCKS_WAKE_EN1_CLK_PERI_SPI0_Msk = 0x1
	// Bit CLK_PERI_SPI0.
	CLOCKS_WAKE_EN1_CLK_PERI_SPI0 = 0x1

	// SLEEP_EN0: enable clock in sleep mode
	// Position of CLK_SYS_SIO field.
	CLOCKS_SLEEP_EN0_CLK_SYS_SIO_Pos = 0x1f
	// Bit mask of CLK_SYS_SIO field.
	CLOCKS_SLEEP_EN0_CLK_SYS_SIO_Msk = 0x80000000
	// Bit CLK_SYS_SIO.
	CLOCKS_SLEEP_EN0_CLK_SYS_SIO = 0x80000000
	// Position of CLK_SYS_SHA256 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_SHA256_Pos = 0x1e
	// Bit mask of CLK_SYS_SHA256 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_SHA256_Msk = 0x40000000
	// Bit CLK_SYS_SHA256.
	CLOCKS_SLEEP_EN0_CLK_SYS_SHA256 = 0x40000000
	// Position of CLK_SYS_PSM field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PSM_Pos = 0x1d
	// Bit mask of CLK_SYS_PSM field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PSM_Msk = 0x20000000
	// Bit CLK_SYS_PSM.
	CLOCKS_SLEEP_EN0_CLK_SYS_PSM = 0x20000000
	// Position of CLK_SYS_ROSC field.
	CLOCKS_SLEEP_EN0_CLK_SYS_ROSC_Pos = 0x1c
	// Bit mask of CLK_SYS_ROSC field.
	CLOCKS_SLEEP_EN0_CLK_SYS_ROSC_Msk = 0x10000000
	// Bit CLK_SYS_ROSC.
	CLOCKS_SLEEP_EN0_CLK_SYS_ROSC = 0x10000000
	// Position of CLK_SYS_ROM field.
	CLOCKS_SLEEP_EN0_CLK_SYS_ROM_Pos = 0x1b
	// Bit mask of CLK_SYS_ROM field.
	CLOCKS_SLEEP_EN0_CLK_SYS_ROM_Msk = 0x8000000
	// Bit CLK_SYS_ROM.
	CLOCKS_SLEEP_EN0_CLK_SYS_ROM = 0x8000000
	// Position of CLK_SYS_RESETS field.
	CLOCKS_SLEEP_EN0_CLK_SYS_RESETS_Pos = 0x1a
	// Bit mask of CLK_SYS_RESETS field.
	CLOCKS_SLEEP_EN0_CLK_SYS_RESETS_Msk = 0x4000000
	// Bit CLK_SYS_RESETS.
	CLOCKS_SLEEP_EN0_CLK_SYS_RESETS = 0x4000000
	// Position of CLK_SYS_PWM field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PWM_Pos = 0x19
	// Bit mask of CLK_SYS_PWM field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PWM_Msk = 0x2000000
	// Bit CLK_SYS_PWM.
	CLOCKS_SLEEP_EN0_CLK_SYS_PWM = 0x2000000
	// Position of CLK_SYS_POWMAN field.
	CLOCKS_SLEEP_EN0_CLK_SYS_POWMAN_Pos = 0x18
	// Bit mask of CLK_SYS_POWMAN field.
	CLOCKS_SLEEP_EN0_CLK_SYS_POWMAN_Msk = 0x1000000
	// Bit CLK_SYS_POWMAN.
	CLOCKS_SLEEP_EN0_CLK_SYS_POWMAN = 0x1000000
	// Position of CLK_REF_POWMAN field.
	CLOCKS_SLEEP_EN0_CLK_REF_POWMAN_Pos = 0x17
	// Bit mask of CLK_REF_POWMAN field.
	CLOCKS_SLEEP_EN0_CLK_REF_POWMAN_Msk = 0x800000
	// Bit CLK_REF_POWMAN.
	CLOCKS_SLEEP_EN0_CLK_REF_POWMAN = 0x800000
	// Position of CLK_SYS_PLL_USB field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PLL_USB_Pos = 0x16
	// Bit mask of CLK_SYS_PLL_USB field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PLL_USB_Msk = 0x400000
	// Bit CLK_SYS_PLL_USB.
	CLOCKS_SLEEP_EN0_CLK_SYS_PLL_USB = 0x400000
	// Position of CLK_SYS_PLL_SYS field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PLL_SYS_Pos = 0x15
	// Bit mask of CLK_SYS_PLL_SYS field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PLL_SYS_Msk = 0x200000
	// Bit CLK_SYS_PLL_SYS.
	CLOCKS_SLEEP_EN0_CLK_SYS_PLL_SYS = 0x200000
	// Position of CLK_SYS_PIO2 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PIO2_Pos = 0x14
	// Bit mask of CLK_SYS_PIO2 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PIO2_Msk = 0x100000
	// Bit CLK_SYS_PIO2.
	CLOCKS_SLEEP_EN0_CLK_SYS_PIO2 = 0x100000
	// Position of CLK_SYS_PIO1 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PIO1_Pos = 0x13
	// Bit mask of CLK_SYS_PIO1 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PIO1_Msk = 0x80000
	// Bit CLK_SYS_PIO1.
	CLOCKS_SLEEP_EN0_CLK_SYS_PIO1 = 0x80000
	// Position of CLK_SYS_PIO0 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PIO0_Pos = 0x12
	// Bit mask of CLK_SYS_PIO0 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PIO0_Msk = 0x40000
	// Bit CLK_SYS_PIO0.
	CLOCKS_SLEEP_EN0_CLK_SYS_PIO0 = 0x40000
	// Position of CLK_SYS_PADS field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PADS_Pos = 0x11
	// Bit mask of CLK_SYS_PADS field.
	CLOCKS_SLEEP_EN0_CLK_SYS_PADS_Msk = 0x20000
	// Bit CLK_SYS_PADS.
	CLOCKS_SLEEP_EN0_CLK_SYS_PADS = 0x20000
	// Position of CLK_SYS_OTP field.
	CLOCKS_SLEEP_EN0_CLK_SYS_OTP_Pos = 0x10
	// Bit mask of CLK_SYS_OTP field.
	CLOCKS_SLEEP_EN0_CLK_SYS_OTP_Msk = 0x10000
	// Bit CLK_SYS_OTP.
	CLOCKS_SLEEP_EN0_CLK_SYS_OTP = 0x10000
	// Position of CLK_REF_OTP field.
	CLOCKS_SLEEP_EN0_CLK_REF_OTP_Pos = 0xf
	// Bit mask of CLK_REF_OTP field.
	CLOCKS_SLEEP_EN0_CLK_REF_OTP_Msk = 0x8000
	// Bit CLK_REF_OTP.
	CLOCKS_SLEEP_EN0_CLK_REF_OTP = 0x8000
	// Position of CLK_SYS_JTAG field.
	CLOCKS_SLEEP_EN0_CLK_SYS_JTAG_Pos = 0xe
	// Bit mask of CLK_SYS_JTAG field.
	CLOCKS_SLEEP_EN0_CLK_SYS_JTAG_Msk = 0x4000
	// Bit CLK_SYS_JTAG.
	CLOCKS_SLEEP_EN0_CLK_SYS_JTAG = 0x4000
	// Position of CLK_SYS_IO field.
	CLOCKS_SLEEP_EN0_CLK_SYS_IO_Pos = 0xd
	// Bit mask of CLK_SYS_IO field.
	CLOCKS_SLEEP_EN0_CLK_SYS_IO_Msk = 0x2000
	// Bit CLK_SYS_IO.
	CLOCKS_SLEEP_EN0_CLK_SYS_IO = 0x2000
	// Position of CLK_SYS_I2C1 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_I2C1_Pos = 0xc
	// Bit mask of CLK_SYS_I2C1 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_I2C1_Msk = 0x1000
	// Bit CLK_SYS_I2C1.
	CLOCKS_SLEEP_EN0_CLK_SYS_I2C1 = 0x1000
	// Position of CLK_SYS_I2C0 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_I2C0_Pos = 0xb
	// Bit mask of CLK_SYS_I2C0 field.
	CLOCKS_SLEEP_EN0_CLK_SYS_I2C0_Msk = 0x800
	// Bit CLK_SYS_I2C0.
	CLOCKS_SLEEP_EN0_CLK_SYS_I2C0 = 0x800
	// Position of CLK_SYS_HSTX field.
	CLOCKS_SLEEP_EN0_CLK_SYS_HSTX_Pos = 0xa
	// Bit mask of CLK_SYS_HSTX field.
	CLOCKS_SLEEP_EN0_CLK_SYS_HSTX_Msk = 0x400
	// Bit CLK_SYS_HSTX.
	CLOCKS_SLEEP_EN0_CLK_SYS_HSTX = 0x400
	// Position of CLK_HSTX field.
	CLOCKS_SLEEP_EN0_CLK_HSTX_Pos = 0x9
	// Bit mask of CLK_HSTX field.
	CLOCKS_SLEEP_EN0_CLK_HSTX_Msk = 0x200
	// Bit CLK_HSTX.
	CLOCKS_SLEEP_EN0_CLK_HSTX = 0x200
	// Position of CLK_SYS_GLITCH_DETECTOR field.
	CLOCKS_SLEEP_EN0_CLK_SYS_GLITCH_DETECTOR_Pos = 0x8
	// Bit mask of CLK_SYS_GLITCH_DETECTOR field.
	CLOCKS_SLEEP_EN0_CLK_SYS_GLITCH_DETECTOR_Msk = 0x100
	// Bit CLK_SYS_GLITCH_DETECTOR.
	CLOCKS_SLEEP_EN0_CLK_SYS_GLITCH_DETECTOR = 0x100
	// Position of CLK_SYS_DMA field.
	CLOCKS_SLEEP_EN0_CLK_SYS_DMA_Pos = 0x7
	// Bit mask of CLK_SYS_DMA field.
	CLOCKS_SLEEP_EN0_CLK_SYS_DMA_Msk = 0x80
	// Bit CLK_SYS_DMA.
	CLOCKS_SLEEP_EN0_CLK_SYS_DMA = 0x80
	// Position of CLK_SYS_BUSFABRIC field.
	CLOCKS_SLEEP_EN0_CLK_SYS_BUSFABRIC_Pos = 0x6
	// Bit mask of CLK_SYS_BUSFABRIC field.
	CLOCKS_SLEEP_EN0_CLK_SYS_BUSFABRIC_Msk = 0x40
	// Bit CLK_SYS_BUSFABRIC.
	CLOCKS_SLEEP_EN0_CLK_SYS_BUSFABRIC = 0x40
	// Position of CLK_SYS_BUSCTRL field.
	CLOCKS_SLEEP_EN0_CLK_SYS_BUSCTRL_Pos = 0x5
	// Bit mask of CLK_SYS_BUSCTRL field.
	CLOCKS_SLEEP_EN0_CLK_SYS_BUSCTRL_Msk = 0x20
	// Bit CLK_SYS_BUSCTRL.
	CLOCKS_SLEEP_EN0_CLK_SYS_BUSCTRL = 0x20
	// Position of CLK_SYS_BOOTRAM field.
	CLOCKS_SLEEP_EN0_CLK_SYS_BOOTRAM_Pos = 0x4
	// Bit mask of CLK_SYS_BOOTRAM field.
	CLOCKS_SLEEP_EN0_CLK_SYS_BOOTRAM_Msk = 0x10
	// Bit CLK_SYS_BOOTRAM.
	CLOCKS_SLEEP_EN0_CLK_SYS_BOOTRAM = 0x10
	// Position of CLK_SYS_ADC field.
	CLOCKS_SLEEP_EN0_CLK_SYS_ADC_Pos = 0x3
	// Bit mask of CLK_SYS_ADC field.
	CLOCKS_SLEEP_EN0_CLK_SYS_ADC_Msk = 0x8
	// Bit CLK_SYS_ADC.
	CLOCKS_SLEEP_EN0_CLK_SYS_ADC = 0x8
	// Position of CLK_ADC field.
	CLOCKS_SLEEP_EN0_CLK_ADC_Pos = 0x2
	// Bit mask of CLK_ADC field.
	CLOCKS_SLEEP_EN0_CLK_ADC_Msk = 0x4
	// Bit CLK_ADC.
	CLOCKS_SLEEP_EN0_CLK_ADC = 0x4
	// Position of CLK_SYS_ACCESSCTRL field.
	CLOCKS_SLEEP_EN0_CLK_SYS_ACCESSCTRL_Pos = 0x1
	// Bit mask of CLK_SYS_ACCESSCTRL field.
	CLOCKS_SLEEP_EN0_CLK_SYS_ACCESSCTRL_Msk = 0x2
	// Bit CLK_SYS_ACCESSCTRL.
	CLOCKS_SLEEP_EN0_CLK_SYS_ACCESSCTRL = 0x2
	// Position of CLK_SYS_CLOCKS field.
	CLOCKS_SLEEP_EN0_CLK_SYS_CLOCKS_Pos = 0x0
	// Bit mask of CLK_SYS_CLOCKS field.
	CLOCKS_SLEEP_EN0_CLK_SYS_CLOCKS_Msk = 0x1
	// Bit CLK_SYS_CLOCKS.
	CLOCKS_SLEEP_EN0_CLK_SYS_CLOCKS = 0x1

	// SLEEP_EN1: enable clock in sleep mode
	// Position of CLK_SYS_XOSC field.
	CLOCKS_SLEEP_EN1_CLK_SYS_XOSC_Pos = 0x1e
	// Bit mask of CLK_SYS_XOSC field.
	CLOCKS_SLEEP_EN1_CLK_SYS_XOSC_Msk = 0x40000000
	// Bit CLK_SYS_XOSC.
	CLOCKS_SLEEP_EN1_CLK_SYS_XOSC = 0x40000000
	// Position of CLK_SYS_XIP field.
	CLOCKS_SLEEP_EN1_CLK_SYS_XIP_Pos = 0x1d
	// Bit mask of CLK_SYS_XIP field.
	CLOCKS_SLEEP_EN1_CLK_SYS_XIP_Msk = 0x20000000
	// Bit CLK_SYS_XIP.
	CLOCKS_SLEEP_EN1_CLK_SYS_XIP = 0x20000000
	// Position of CLK_SYS_WATCHDOG field.
	CLOCKS_SLEEP_EN1_CLK_SYS_WATCHDOG_Pos = 0x1c
	// Bit mask of CLK_SYS_WATCHDOG field.
	CLOCKS_SLEEP_EN1_CLK_SYS_WATCHDOG_Msk = 0x10000000
	// Bit CLK_SYS_WATCHDOG.
	CLOCKS_SLEEP_EN1_CLK_SYS_WATCHDOG = 0x10000000
	// Position of CLK_USB field.
	CLOCKS_SLEEP_EN1_CLK_USB_Pos = 0x1b
	// Bit mask of CLK_USB field.
	CLOCKS_SLEEP_EN1_CLK_USB_Msk = 0x8000000
	// Bit CLK_USB.
	CLOCKS_SLEEP_EN1_CLK_USB = 0x8000000
	// Position of CLK_SYS_USBCTRL field.
	CLOCKS_SLEEP_EN1_CLK_SYS_USBCTRL_Pos = 0x1a
	// Bit mask of CLK_SYS_USBCTRL field.
	CLOCKS_SLEEP_EN1_CLK_SYS_USBCTRL_Msk = 0x4000000
	// Bit CLK_SYS_USBCTRL.
	CLOCKS_SLEEP_EN1_CLK_SYS_USBCTRL = 0x4000000
	// Position of CLK_SYS_UART1 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_UART1_Pos = 0x19
	// Bit mask of CLK_SYS_UART1 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_UART1_Msk = 0x2000000
	// Bit CLK_SYS_UART1.
	CLOCKS_SLEEP_EN1_CLK_SYS_UART1 = 0x2000000
	// Position of CLK_PERI_UART1 field.
	CLOCKS_SLEEP_EN1_CLK_PERI_UART1_Pos = 0x18
	// Bit mask of CLK_PERI_UART1 field.
	CLOCKS_SLEEP_EN1_CLK_PERI_UART1_Msk = 0x1000000
	// Bit CLK_PERI_UART1.
	CLOCKS_SLEEP_EN1_CLK_PERI_UART1 = 0x1000000
	// Position of CLK_SYS_UART0 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_UART0_Pos = 0x17
	// Bit mask of CLK_SYS_UART0 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_UART0_Msk = 0x800000
	// Bit CLK_SYS_UART0.
	CLOCKS_SLEEP_EN1_CLK_SYS_UART0 = 0x800000
	// Position of CLK_PERI_UART0 field.
	CLOCKS_SLEEP_EN1_CLK_PERI_UART0_Pos = 0x16
	// Bit mask of CLK_PERI_UART0 field.
	CLOCKS_SLEEP_EN1_CLK_PERI_UART0_Msk = 0x400000
	// Bit CLK_PERI_UART0.
	CLOCKS_SLEEP_EN1_CLK_PERI_UART0 = 0x400000
	// Position of CLK_SYS_TRNG field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TRNG_Pos = 0x15
	// Bit mask of CLK_SYS_TRNG field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TRNG_Msk = 0x200000
	// Bit CLK_SYS_TRNG.
	CLOCKS_SLEEP_EN1_CLK_SYS_TRNG = 0x200000
	// Position of CLK_SYS_TIMER1 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TIMER1_Pos = 0x14
	// Bit mask of CLK_SYS_TIMER1 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TIMER1_Msk = 0x100000
	// Bit CLK_SYS_TIMER1.
	CLOCKS_SLEEP_EN1_CLK_SYS_TIMER1 = 0x100000
	// Position of CLK_SYS_TIMER0 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TIMER0_Pos = 0x13
	// Bit mask of CLK_SYS_TIMER0 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TIMER0_Msk = 0x80000
	// Bit CLK_SYS_TIMER0.
	CLOCKS_SLEEP_EN1_CLK_SYS_TIMER0 = 0x80000
	// Position of CLK_SYS_TICKS field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TICKS_Pos = 0x12
	// Bit mask of CLK_SYS_TICKS field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TICKS_Msk = 0x40000
	// Bit CLK_SYS_TICKS.
	CLOCKS_SLEEP_EN1_CLK_SYS_TICKS = 0x40000
	// Position of CLK_REF_TICKS field.
	CLOCKS_SLEEP_EN1_CLK_REF_TICKS_Pos = 0x11
	// Bit mask of CLK_REF_TICKS field.
	CLOCKS_SLEEP_EN1_CLK_REF_TICKS_Msk = 0x20000
	// Bit CLK_REF_TICKS.
	CLOCKS_SLEEP_EN1_CLK_REF_TICKS = 0x20000
	// Position of CLK_SYS_TBMAN field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TBMAN_Pos = 0x10
	// Bit mask of CLK_SYS_TBMAN field.
	CLOCKS_SLEEP_EN1_CLK_SYS_TBMAN_Msk = 0x10000
	// Bit CLK_SYS_TBMAN.
	CLOCKS_SLEEP_EN1_CLK_SYS_TBMAN = 0x10000
	// Position of CLK_SYS_SYSINFO field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SYSINFO_Pos = 0xf
	// Bit mask of CLK_SYS_SYSINFO field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SYSINFO_Msk = 0x8000
	// Bit CLK_SYS_SYSINFO.
	CLOCKS_SLEEP_EN1_CLK_SYS_SYSINFO = 0x8000
	// Position of CLK_SYS_SYSCFG field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SYSCFG_Pos = 0xe
	// Bit mask of CLK_SYS_SYSCFG field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SYSCFG_Msk = 0x4000
	// Bit CLK_SYS_SYSCFG.
	CLOCKS_SLEEP_EN1_CLK_SYS_SYSCFG = 0x4000
	// Position of CLK_SYS_SRAM9 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM9_Pos = 0xd
	// Bit mask of CLK_SYS_SRAM9 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM9_Msk = 0x2000
	// Bit CLK_SYS_SRAM9.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM9 = 0x2000
	// Position of CLK_SYS_SRAM8 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM8_Pos = 0xc
	// Bit mask of CLK_SYS_SRAM8 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM8_Msk = 0x1000
	// Bit CLK_SYS_SRAM8.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM8 = 0x1000
	// Position of CLK_SYS_SRAM7 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM7_Pos = 0xb
	// Bit mask of CLK_SYS_SRAM7 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM7_Msk = 0x800
	// Bit CLK_SYS_SRAM7.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM7 = 0x800
	// Position of CLK_SYS_SRAM6 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM6_Pos = 0xa
	// Bit mask of CLK_SYS_SRAM6 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM6_Msk = 0x400
	// Bit CLK_SYS_SRAM6.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM6 = 0x400
	// Position of CLK_SYS_SRAM5 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM5_Pos = 0x9
	// Bit mask of CLK_SYS_SRAM5 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM5_Msk = 0x200
	// Bit CLK_SYS_SRAM5.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM5 = 0x200
	// Position of CLK_SYS_SRAM4 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM4_Pos = 0x8
	// Bit mask of CLK_SYS_SRAM4 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM4_Msk = 0x100
	// Bit CLK_SYS_SRAM4.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM4 = 0x100
	// Position of CLK_SYS_SRAM3 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM3_Pos = 0x7
	// Bit mask of CLK_SYS_SRAM3 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM3_Msk = 0x80
	// Bit CLK_SYS_SRAM3.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM3 = 0x80
	// Position of CLK_SYS_SRAM2 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM2_Pos = 0x6
	// Bit mask of CLK_SYS_SRAM2 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM2_Msk = 0x40
	// Bit CLK_SYS_SRAM2.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM2 = 0x40
	// Position of CLK_SYS_SRAM1 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM1_Pos = 0x5
	// Bit mask of CLK_SYS_SRAM1 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM1_Msk = 0x20
	// Bit CLK_SYS_SRAM1.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM1 = 0x20
	// Position of CLK_SYS_SRAM0 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM0_Pos = 0x4
	// Bit mask of CLK_SYS_SRAM0 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM0_Msk = 0x10
	// Bit CLK_SYS_SRAM0.
	CLOCKS_SLEEP_EN1_CLK_SYS_SRAM0 = 0x10
	// Position of CLK_SYS_SPI1 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SPI1_Pos = 0x3
	// Bit mask of CLK_SYS_SPI1 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SPI1_Msk = 0x8
	// Bit CLK_SYS_SPI1.
	CLOCKS_SLEEP_EN1_CLK_SYS_SPI1 = 0x8
	// Position of CLK_PERI_SPI1 field.
	CLOCKS_SLEEP_EN1_CLK_PERI_SPI1_Pos = 0x2
	// Bit mask of CLK_PERI_SPI1 field.
	CLOCKS_SLEEP_EN1_CLK_PERI_SPI1_Msk = 0x4
	// Bit CLK_PERI_SPI1.
	CLOCKS_SLEEP_EN1_CLK_PERI_SPI1 = 0x4
	// Position of CLK_SYS_SPI0 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SPI0_Pos = 0x1
	// Bit mask of CLK_SYS_SPI0 field.
	CLOCKS_SLEEP_EN1_CLK_SYS_SPI0_Msk = 0x2
	// Bit CLK_SYS_SPI0.
	CLOCKS_SLEEP_EN1_CLK_SYS_SPI0 = 0x2
	// Position of CLK_PERI_SPI0 field.
	CLOCKS_SLEEP_EN1_CLK_PERI_SPI0_Pos = 0x0
	// Bit mask of CLK_PERI_SPI0 field.
	CLOCKS_SLEEP_EN1_CLK_PERI_SPI0_Msk = 0x1
	// Bit CLK_PERI_SPI0.
	CLOCKS_SLEEP_EN1_CLK_PERI_SPI0 = 0x1

	// ENABLED0: indicates the state of the clock enable
	// Position of CLK_SYS_SIO field.
	CLOCKS_ENABLED0_CLK_SYS_SIO_Pos = 0x1f
	// Bit mask of CLK_SYS_SIO field.
	CLOCKS_ENABLED0_CLK_SYS_SIO_Msk = 0x80000000
	// Bit CLK_SYS_SIO.
	CLOCKS_ENABLED0_CLK_SYS_SIO = 0x80000000
	// Position of CLK_SYS_SHA256 field.
	CLOCKS_ENABLED0_CLK_SYS_SHA256_Pos = 0x1e
	// Bit mask of CLK_SYS_SHA256 field.
	CLOCKS_ENABLED0_CLK_SYS_SHA256_Msk = 0x40000000
	// Bit CLK_SYS_SHA256.
	CLOCKS_ENABLED0_CLK_SYS_SHA256 = 0x40000000
	// Position of CLK_SYS_PSM field.
	CLOCKS_ENABLED0_CLK_SYS_PSM_Pos = 0x1d
	// Bit mask of CLK_SYS_PSM field.
	CLOCKS_ENABLED0_CLK_SYS_PSM_Msk = 0x20000000
	// Bit CLK_SYS_PSM.
	CLOCKS_ENABLED0_CLK_SYS_PSM = 0x20000000
	// Position of CLK_SYS_ROSC field.
	CLOCKS_ENABLED0_CLK_SYS_ROSC_Pos = 0x1c
	// Bit mask of CLK_SYS_ROSC field.
	CLOCKS_ENABLED0_CLK_SYS_ROSC_Msk = 0x10000000
	// Bit CLK_SYS_ROSC.
	CLOCKS_ENABLED0_CLK_SYS_ROSC = 0x10000000
	// Position of CLK_SYS_ROM field.
	CLOCKS_ENABLED0_CLK_SYS_ROM_Pos = 0x1b
	// Bit mask of CLK_SYS_ROM field.
	CLOCKS_ENABLED0_CLK_SYS_ROM_Msk = 0x8000000
	// Bit CLK_SYS_ROM.
	CLOCKS_ENABLED0_CLK_SYS_ROM = 0x8000000
	// Position of CLK_SYS_RESETS field.
	CLOCKS_ENABLED0_CLK_SYS_RESETS_Pos = 0x1a
	// Bit mask of CLK_SYS_RESETS field.
	CLOCKS_ENABLED0_CLK_SYS_RESETS_Msk = 0x4000000
	// Bit CLK_SYS_RESETS.
	CLOCKS_ENABLED0_CLK_SYS_RESETS = 0x4000000
	// Position of CLK_SYS_PWM field.
	CLOCKS_ENABLED0_CLK_SYS_PWM_Pos = 0x19
	// Bit mask of CLK_SYS_PWM field.
	CLOCKS_ENABLED0_CLK_SYS_PWM_Msk = 0x2000000
	// Bit CLK_SYS_PWM.
	CLOCKS_ENABLED0_CLK_SYS_PWM = 0x2000000
	// Position of CLK_SYS_POWMAN field.
	CLOCKS_ENABLED0_CLK_SYS_POWMAN_Pos = 0x18
	// Bit mask of CLK_SYS_POWMAN field.
	CLOCKS_ENABLED0_CLK_SYS_POWMAN_Msk = 0x1000000
	// Bit CLK_SYS_POWMAN.
	CLOCKS_ENABLED0_CLK_SYS_POWMAN = 0x1000000
	// Position of CLK_REF_POWMAN field.
	CLOCKS_ENABLED0_CLK_REF_POWMAN_Pos = 0x17
	// Bit mask of CLK_REF_POWMAN field.
	CLOCKS_ENABLED0_CLK_REF_POWMAN_Msk = 0x800000
	// Bit CLK_REF_POWMAN.
	CLOCKS_ENABLED0_CLK_REF_POWMAN = 0x800000
	// Position of CLK_SYS_PLL_USB field.
	CLOCKS_ENABLED0_CLK_SYS_PLL_USB_Pos = 0x16
	// Bit mask of CLK_SYS_PLL_USB field.
	CLOCKS_ENABLED0_CLK_SYS_PLL_USB_Msk = 0x400000
	// Bit CLK_SYS_PLL_USB.
	CLOCKS_ENABLED0_CLK_SYS_PLL_USB = 0x400000
	// Position of CLK_SYS_PLL_SYS field.
	CLOCKS_ENABLED0_CLK_SYS_PLL_SYS_Pos = 0x15
	// Bit mask of CLK_SYS_PLL_SYS field.
	CLOCKS_ENABLED0_CLK_SYS_PLL_SYS_Msk = 0x200000
	// Bit CLK_SYS_PLL_SYS.
	CLOCKS_ENABLED0_CLK_SYS_PLL_SYS = 0x200000
	// Position of CLK_SYS_PIO2 field.
	CLOCKS_ENABLED0_CLK_SYS_PIO2_Pos = 0x14
	// Bit mask of CLK_SYS_PIO2 field.
	CLOCKS_ENABLED0_CLK_SYS_PIO2_Msk = 0x100000
	// Bit CLK_SYS_PIO2.
	CLOCKS_ENABLED0_CLK_SYS_PIO2 = 0x100000
	// Position of CLK_SYS_PIO1 field.
	CLOCKS_ENABLED0_CLK_SYS_PIO1_Pos = 0x13
	// Bit mask of CLK_SYS_PIO1 field.
	CLOCKS_ENABLED0_CLK_SYS_PIO1_Msk = 0x80000
	// Bit CLK_SYS_PIO1.
	CLOCKS_ENABLED0_CLK_SYS_PIO1 = 0x80000
	// Position of CLK_SYS_PIO0 field.
	CLOCKS_ENABLED0_CLK_SYS_PIO0_Pos = 0x12
	// Bit mask of CLK_SYS_PIO0 field.
	CLOCKS_ENABLED0_CLK_SYS_PIO0_Msk = 0x40000
	// Bit CLK_SYS_PIO0.
	CLOCKS_ENABLED0_CLK_SYS_PIO0 = 0x40000
	// Position of CLK_SYS_PADS field.
	CLOCKS_ENABLED0_CLK_SYS_PADS_Pos = 0x11
	// Bit mask of CLK_SYS_PADS field.
	CLOCKS_ENABLED0_CLK_SYS_PADS_Msk = 0x20000
	// Bit CLK_SYS_PADS.
	CLOCKS_ENABLED0_CLK_SYS_PADS = 0x20000
	// Position of CLK_SYS_OTP field.
	CLOCKS_ENABLED0_CLK_SYS_OTP_Pos = 0x10
	// Bit mask of CLK_SYS_OTP field.
	CLOCKS_ENABLED0_CLK_SYS_OTP_Msk = 0x10000
	// Bit CLK_SYS_OTP.
	CLOCKS_ENABLED0_CLK_SYS_OTP = 0x10000
	// Position of CLK_REF_OTP field.
	CLOCKS_ENABLED0_CLK_REF_OTP_Pos = 0xf
	// Bit mask of CLK_REF_OTP field.
	CLOCKS_ENABLED0_CLK_REF_OTP_Msk = 0x8000
	// Bit CLK_REF_OTP.
	CLOCKS_ENABLED0_CLK_REF_OTP = 0x8000
	// Position of CLK_SYS_JTAG field.
	CLOCKS_ENABLED0_CLK_SYS_JTAG_Pos = 0xe
	// Bit mask of CLK_SYS_JTAG field.
	CLOCKS_ENABLED0_CLK_SYS_JTAG_Msk = 0x4000
	// Bit CLK_SYS_JTAG.
	CLOCKS_ENABLED0_CLK_SYS_JTAG = 0x4000
	// Position of CLK_SYS_IO field.
	CLOCKS_ENABLED0_CLK_SYS_IO_Pos = 0xd
	// Bit mask of CLK_SYS_IO field.
	CLOCKS_ENABLED0_CLK_SYS_IO_Msk = 0x2000
	// Bit CLK_SYS_IO.
	CLOCKS_ENABLED0_CLK_SYS_IO = 0x2000
	// Position of CLK_SYS_I2C1 field.
	CLOCKS_ENABLED0_CLK_SYS_I2C1_Pos = 0xc
	// Bit mask of CLK_SYS_I2C1 field.
	CLOCKS_ENABLED0_CLK_SYS_I2C1_Msk = 0x1000
	// Bit CLK_SYS_I2C1.
	CLOCKS_ENABLED0_CLK_SYS_I2C1 = 0x1000
	// Position of CLK_SYS_I2C0 field.
	CLOCKS_ENABLED0_CLK_SYS_I2C0_Pos = 0xb
	// Bit mask of CLK_SYS_I2C0 field.
	CLOCKS_ENABLED0_CLK_SYS_I2C0_Msk = 0x800
	// Bit CLK_SYS_I2C0.
	CLOCKS_ENABLED0_CLK_SYS_I2C0 = 0x800
	// Position of CLK_SYS_HSTX field.
	CLOCKS_ENABLED0_CLK_SYS_HSTX_Pos = 0xa
	// Bit mask of CLK_SYS_HSTX field.
	CLOCKS_ENABLED0_CLK_SYS_HSTX_Msk = 0x400
	// Bit CLK_SYS_HSTX.
	CLOCKS_ENABLED0_CLK_SYS_HSTX = 0x400
	// Position of CLK_HSTX field.
	CLOCKS_ENABLED0_CLK_HSTX_Pos = 0x9
	// Bit mask of CLK_HSTX field.
	CLOCKS_ENABLED0_CLK_HSTX_Msk = 0x200
	// Bit CLK_HSTX.
	CLOCKS_ENABLED0_CLK_HSTX = 0x200
	// Position of CLK_SYS_GLITCH_DETECTOR field.
	CLOCKS_ENABLED0_CLK_SYS_GLITCH_DETECTOR_Pos = 0x8
	// Bit mask of CLK_SYS_GLITCH_DETECTOR field.
	CLOCKS_ENABLED0_CLK_SYS_GLITCH_DETECTOR_Msk = 0x100
	// Bit CLK_SYS_GLITCH_DETECTOR.
	CLOCKS_ENABLED0_CLK_SYS_GLITCH_DETECTOR = 0x100
	// Position of CLK_SYS_DMA field.
	CLOCKS_ENABLED0_CLK_SYS_DMA_Pos = 0x7
	// Bit mask of CLK_SYS_DMA field.
	CLOCKS_ENABLED0_CLK_SYS_DMA_Msk = 0x80
	// Bit CLK_SYS_DMA.
	CLOCKS_ENABLED0_CLK_SYS_DMA = 0x80
	// Position of CLK_SYS_BUSFABRIC field.
	CLOCKS_ENABLED0_CLK_SYS_BUSFABRIC_Pos = 0x6
	// Bit mask of CLK_SYS_BUSFABRIC field.
	CLOCKS_ENABLED0_CLK_SYS_BUSFABRIC_Msk = 0x40
	// Bit CLK_SYS_BUSFABRIC.
	CLOCKS_ENABLED0_CLK_SYS_BUSFABRIC = 0x40
	// Position of CLK_SYS_BUSCTRL field.
	CLOCKS_ENABLED0_CLK_SYS_BUSCTRL_Pos = 0x5
	// Bit mask of CLK_SYS_BUSCTRL field.
	CLOCKS_ENABLED0_CLK_SYS_BUSCTRL_Msk = 0x20
	// Bit CLK_SYS_BUSCTRL.
	CLOCKS_ENABLED0_CLK_SYS_BUSCTRL = 0x20
	// Position of CLK_SYS_BOOTRAM field.
	CLOCKS_ENABLED0_CLK_SYS_BOOTRAM_Pos = 0x4
	// Bit mask of CLK_SYS_BOOTRAM field.
	CLOCKS_ENABLED0_CLK_SYS_BOOTRAM_Msk = 0x10
	// Bit CLK_SYS_BOOTRAM.
	CLOCKS_ENABLED0_CLK_SYS_BOOTRAM = 0x10
	// Position of CLK_SYS_ADC field.
	CLOCKS_ENABLED0_CLK_SYS_ADC_Pos = 0x3
	// Bit mask of CLK_SYS_ADC field.
	CLOCKS_ENABLED0_CLK_SYS_ADC_Msk = 0x8
	// Bit CLK_SYS_ADC.
	CLOCKS_ENABLED0_CLK_SYS_ADC = 0x8
	// Position of CLK_ADC field.
	CLOCKS_ENABLED0_CLK_ADC_Pos = 0x2
	// Bit mask of CLK_ADC field.
	CLOCKS_ENABLED0_CLK_ADC_Msk = 0x4
	// Bit CLK_ADC.
	CLOCKS_ENABLED0_CLK_ADC = 0x4
	// Position of CLK_SYS_ACCESSCTRL field.
	CLOCKS_ENABLED0_CLK_SYS_ACCESSCTRL_Pos = 0x1
	// Bit mask of CLK_SYS_ACCESSCTRL field.
	CLOCKS_ENABLED0_CLK_SYS_ACCESSCTRL_Msk = 0x2
	// Bit CLK_SYS_ACCESSCTRL.
	CLOCKS_ENABLED0_CLK_SYS_ACCESSCTRL = 0x2
	// Position of CLK_SYS_CLOCKS field.
	CLOCKS_ENABLED0_CLK_SYS_CLOCKS_Pos = 0x0
	// Bit mask of CLK_SYS_CLOCKS field.
	CLOCKS_ENABLED0_CLK_SYS_CLOCKS_Msk = 0x1
	// Bit CLK_SYS_CLOCKS.
	CLOCKS_ENABLED0_CLK_SYS_CLOCKS = 0x1

	// ENABLED1: indicates the state of the clock enable
	// Position of CLK_SYS_XOSC field.
	CLOCKS_ENABLED1_CLK_SYS_XOSC_Pos = 0x1e
	// Bit mask of CLK_SYS_XOSC field.
	CLOCKS_ENABLED1_CLK_SYS_XOSC_Msk = 0x40000000
	// Bit CLK_SYS_XOSC.
	CLOCKS_ENABLED1_CLK_SYS_XOSC = 0x40000000
	// Position of CLK_SYS_XIP field.
	CLOCKS_ENABLED1_CLK_SYS_XIP_Pos = 0x1d
	// Bit mask of CLK_SYS_XIP field.
	CLOCKS_ENABLED1_CLK_SYS_XIP_Msk = 0x20000000
	// Bit CLK_SYS_XIP.
	CLOCKS_ENABLED1_CLK_SYS_XIP = 0x20000000
	// Position of CLK_SYS_WATCHDOG field.
	CLOCKS_ENABLED1_CLK_SYS_WATCHDOG_Pos = 0x1c
	// Bit mask of CLK_SYS_WATCHDOG field.
	CLOCKS_ENABLED1_CLK_SYS_WATCHDOG_Msk = 0x10000000
	// Bit CLK_SYS_WATCHDOG.
	CLOCKS_ENABLED1_CLK_SYS_WATCHDOG = 0x10000000
	// Position of CLK_USB field.
	CLOCKS_ENABLED1_CLK_USB_Pos = 0x1b
	// Bit mask of CLK_USB field.
	CLOCKS_ENABLED1_CLK_USB_Msk = 0x8000000
	// Bit CLK_USB.
	CLOCKS_ENABLED1_CLK_USB = 0x8000000
	// Position of CLK_SYS_USBCTRL field.
	CLOCKS_ENABLED1_CLK_SYS_USBCTRL_Pos = 0x1a
	// Bit mask of CLK_SYS_USBCTRL field.
	CLOCKS_ENABLED1_CLK_SYS_USBCTRL_Msk = 0x4000000
	// Bit CLK_SYS_USBCTRL.
	CLOCKS_ENABLED1_CLK_SYS_USBCTRL = 0x4000000
	// Position of CLK_SYS_UART1 field.
	CLOCKS_ENABLED1_CLK_SYS_UART1_Pos = 0x19
	// Bit mask of CLK_SYS_UART1 field.
	CLOCKS_ENABLED1_CLK_SYS_UART1_Msk = 0x2000000
	// Bit CLK_SYS_UART1.
	CLOCKS_ENABLED1_CLK_SYS_UART1 = 0x2000000
	// Position of CLK_PERI_UART1 field.
	CLOCKS_ENABLED1_CLK_PERI_UART1_Pos = 0x18
	// Bit mask of CLK_PERI_UART1 field.
	CLOCKS_ENABLED1_CLK_PERI_UART1_Msk = 0x1000000
	// Bit CLK_PERI_UART1.
	CLOCKS_ENABLED1_CLK_PERI_UART1 = 0x1000000
	// Position of CLK_SYS_UART0 field.
	CLOCKS_ENABLED1_CLK_SYS_UART0_Pos = 0x17
	// Bit mask of CLK_SYS_UART0 field.
	CLOCKS_ENABLED1_CLK_SYS_UART0_Msk = 0x800000
	// Bit CLK_SYS_UART0.
	CLOCKS_ENABLED1_CLK_SYS_UART0 = 0x800000
	// Position of CLK_PERI_UART0 field.
	CLOCKS_ENABLED1_CLK_PERI_UART0_Pos = 0x16
	// Bit mask of CLK_PERI_UART0 field.
	CLOCKS_ENABLED1_CLK_PERI_UART0_Msk = 0x400000
	// Bit CLK_PERI_UART0.
	CLOCKS_ENABLED1_CLK_PERI_UART0 = 0x400000
	// Position of CLK_SYS_TRNG field.
	CLOCKS_ENABLED1_CLK_SYS_TRNG_Pos = 0x15
	// Bit mask of CLK_SYS_TRNG field.
	CLOCKS_ENABLED1_CLK_SYS_TRNG_Msk = 0x200000
	// Bit CLK_SYS_TRNG.
	CLOCKS_ENABLED1_CLK_SYS_TRNG = 0x200000
	// Position of CLK_SYS_TIMER1 field.
	CLOCKS_ENABLED1_CLK_SYS_TIMER1_Pos = 0x14
	// Bit mask of CLK_SYS_TIMER1 field.
	CLOCKS_ENABLED1_CLK_SYS_TIMER1_Msk = 0x100000
	// Bit CLK_SYS_TIMER1.
	CLOCKS_ENABLED1_CLK_SYS_TIMER1 = 0x100000
	// Position of CLK_SYS_TIMER0 field.
	CLOCKS_ENABLED1_CLK_SYS_TIMER0_Pos = 0x13
	// Bit mask of CLK_SYS_TIMER0 field.
	CLOCKS_ENABLED1_CLK_SYS_TIMER0_Msk = 0x80000
	// Bit CLK_SYS_TIMER0.
	CLOCKS_ENABLED1_CLK_SYS_TIMER0 = 0x80000
	// Position of CLK_SYS_TICKS field.
	CLOCKS_ENABLED1_CLK_SYS_TICKS_Pos = 0x12
	// Bit mask of CLK_SYS_TICKS field.
	CLOCKS_ENABLED1_CLK_SYS_TICKS_Msk = 0x40000
	// Bit CLK_SYS_TICKS.
	CLOCKS_ENABLED1_CLK_SYS_TICKS = 0x40000
	// Position of CLK_REF_TICKS field.
	CLOCKS_ENABLED1_CLK_REF_TICKS_Pos = 0x11
	// Bit mask of CLK_REF_TICKS field.
	CLOCKS_ENABLED1_CLK_REF_TICKS_Msk = 0x20000
	// Bit CLK_REF_TICKS.
	CLOCKS_ENABLED1_CLK_REF_TICKS = 0x20000
	// Position of CLK_SYS_TBMAN field.
	CLOCKS_ENABLED1_CLK_SYS_TBMAN_Pos = 0x10
	// Bit mask of CLK_SYS_TBMAN field.
	CLOCKS_ENABLED1_CLK_SYS_TBMAN_Msk = 0x10000
	// Bit CLK_SYS_TBMAN.
	CLOCKS_ENABLED1_CLK_SYS_TBMAN = 0x10000
	// Position of CLK_SYS_SYSINFO field.
	CLOCKS_ENABLED1_CLK_SYS_SYSINFO_Pos = 0xf
	// Bit mask of CLK_SYS_SYSINFO field.
	CLOCKS_ENABLED1_CLK_SYS_SYSINFO_Msk = 0x8000
	// Bit CLK_SYS_SYSINFO.
	CLOCKS_ENABLED1_CLK_SYS_SYSINFO = 0x8000
	// Position of CLK_SYS_SYSCFG field.
	CLOCKS_ENABLED1_CLK_SYS_SYSCFG_Pos = 0xe
	// Bit mask of CLK_SYS_SYSCFG field.
	CLOCKS_ENABLED1_CLK_SYS_SYSCFG_Msk = 0x4000
	// Bit CLK_SYS_SYSCFG.
	CLOCKS_ENABLED1_CLK_SYS_SYSCFG = 0x4000
	// Position of CLK_SYS_SRAM9 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM9_Pos = 0xd
	// Bit mask of CLK_SYS_SRAM9 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM9_Msk = 0x2000
	// Bit CLK_SYS_SRAM9.
	CLOCKS_ENABLED1_CLK_SYS_SRAM9 = 0x2000
	// Position of CLK_SYS_SRAM8 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM8_Pos = 0xc
	// Bit mask of CLK_SYS_SRAM8 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM8_Msk = 0x1000
	// Bit CLK_SYS_SRAM8.
	CLOCKS_ENABLED1_CLK_SYS_SRAM8 = 0x1000
	// Position of CLK_SYS_SRAM7 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM7_Pos = 0xb
	// Bit mask of CLK_SYS_SRAM7 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM7_Msk = 0x800
	// Bit CLK_SYS_SRAM7.
	CLOCKS_ENABLED1_CLK_SYS_SRAM7 = 0x800
	// Position of CLK_SYS_SRAM6 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM6_Pos = 0xa
	// Bit mask of CLK_SYS_SRAM6 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM6_Msk = 0x400
	// Bit CLK_SYS_SRAM6.
	CLOCKS_ENABLED1_CLK_SYS_SRAM6 = 0x400
	// Position of CLK_SYS_SRAM5 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM5_Pos = 0x9
	// Bit mask of CLK_SYS_SRAM5 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM5_Msk = 0x200
	// Bit CLK_SYS_SRAM5.
	CLOCKS_ENABLED1_CLK_SYS_SRAM5 = 0x200
	// Position of CLK_SYS_SRAM4 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM4_Pos = 0x8
	// Bit mask of CLK_SYS_SRAM4 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM4_Msk = 0x100
	// Bit CLK_SYS_SRAM4.
	CLOCKS_ENABLED1_CLK_SYS_SRAM4 = 0x100
	// Position of CLK_SYS_SRAM3 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM3_Pos = 0x7
	// Bit mask of CLK_SYS_SRAM3 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM3_Msk = 0x80
	// Bit CLK_SYS_SRAM3.
	CLOCKS_ENABLED1_CLK_SYS_SRAM3 = 0x80
	// Position of CLK_SYS_SRAM2 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM2_Pos = 0x6
	// Bit mask of CLK_SYS_SRAM2 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM2_Msk = 0x40
	// Bit CLK_SYS_SRAM2.
	CLOCKS_ENABLED1_CLK_SYS_SRAM2 = 0x40
	// Position of CLK_SYS_SRAM1 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM1_Pos = 0x5
	// Bit mask of CLK_SYS_SRAM1 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM1_Msk = 0x20
	// Bit CLK_SYS_SRAM1.
	CLOCKS_ENABLED1_CLK_SYS_SRAM1 = 0x20
	// Position of CLK_SYS_SRAM0 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM0_Pos = 0x4
	// Bit mask of CLK_SYS_SRAM0 field.
	CLOCKS_ENABLED1_CLK_SYS_SRAM0_Msk = 0x10
	// Bit CLK_SYS_SRAM0.
	CLOCKS_ENABLED1_CLK_SYS_SRAM0 = 0x10
	// Position of CLK_SYS_SPI1 field.
	CLOCKS_ENABLED1_CLK_SYS_SPI1_Pos = 0x3
	// Bit mask of CLK_SYS_SPI1 field.
	CLOCKS_ENABLED1_CLK_SYS_SPI1_Msk = 0x8
	// Bit CLK_SYS_SPI1.
	CLOCKS_ENABLED1_CLK_SYS_SPI1 = 0x8
	// Position of CLK_PERI_SPI1 field.
	CLOCKS_ENABLED1_CLK_PERI_SPI1_Pos = 0x2
	// Bit mask of CLK_PERI_SPI1 field.
	CLOCKS_ENABLED1_CLK_PERI_SPI1_Msk = 0x4
	// Bit CLK_PERI_SPI1.
	CLOCKS_ENABLED1_CLK_PERI_SPI1 = 0x4
	// Position of CLK_SYS_SPI0 field.
	CLOCKS_ENABLED1_CLK_SYS_SPI0_Pos = 0x1
	// Bit mask of CLK_SYS_SPI0 field.
	CLOCKS_ENABLED1_CLK_SYS_SPI0_Msk = 0x2
	// Bit CLK_SYS_SPI0.
	CLOCKS_ENABLED1_CLK_SYS_SPI0 = 0x2
	// Position of CLK_PERI_SPI0 field.
	CLOCKS_ENABLED1_CLK_PERI_SPI0_Pos = 0x0
	// Bit mask of CLK_PERI_SPI0 field.
	CLOCKS_ENABLED1_CLK_PERI_SPI0_Msk = 0x1
	// Bit CLK_PERI_SPI0.
	CLOCKS_ENABLED1_CLK_PERI_SPI0 = 0x1

	// INTR: Raw Interrupts
	// Position of CLK_SYS_RESUS field.
	CLOCKS_INTR_CLK_SYS_RESUS_Pos = 0x0
	// Bit mask of CLK_SYS_RESUS field.
	CLOCKS_INTR_CLK_SYS_RESUS_Msk = 0x1
	// Bit CLK_SYS_RESUS.
	CLOCKS_INTR_CLK_SYS_RESUS = 0x1

	// INTE: Interrupt Enable
	// Position of CLK_SYS_RESUS field.
	CLOCKS_INTE_CLK_SYS_RESUS_Pos = 0x0
	// Bit mask of CLK_SYS_RESUS field.
	CLOCKS_INTE_CLK_SYS_RESUS_Msk = 0x1
	// Bit CLK_SYS_RESUS.
	CLOCKS_INTE_CLK_SYS_RESUS = 0x1

	// INTF: Interrupt Force
	// Position of CLK_SYS_RESUS field.
	CLOCKS_INTF_CLK_SYS_RESUS_Pos = 0x0
	// Bit mask of CLK_SYS_RESUS field.
	CLOCKS_INTF_CLK_SYS_RESUS_Msk = 0x1
	// Bit CLK_SYS_RESUS.
	CLOCKS_INTF_CLK_SYS_RESUS = 0x1

	// INTS: Interrupt status after masking & forcing
	// Position of CLK_SYS_RESUS field.
	CLOCKS_INTS_CLK_SYS_RESUS_Pos = 0x0
	// Bit mask of CLK_SYS_RESUS field.
	CLOCKS_INTS_CLK_SYS_RESUS_Msk = 0x1
	// Bit CLK_SYS_RESUS.
	CLOCKS_INTS_CLK_SYS_RESUS = 0x1
)

// Constants for TICKS
const (
	// PROC0_CTRL: Controls the tick generator
	// Position of RUNNING field.
	TICKS_PROC0_CTRL_RUNNING_Pos = 0x1
	// Bit mask of RUNNING field.
	TICKS_PROC0_CTRL_RUNNING_Msk = 0x2
	// Bit RUNNING.
	TICKS_PROC0_CTRL_RUNNING = 0x2
	// Position of ENABLE field.
	TICKS_PROC0_CTRL_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	TICKS_PROC0_CTRL_ENABLE_Msk = 0x1
	// Bit ENABLE.
	TICKS_PROC0_CTRL_ENABLE = 0x1

	// PROC0_CYCLES
	// Position of PROC0_CYCLES field.
	TICKS_PROC0_CYCLES_PROC0_CYCLES_Pos = 0x0
	// Bit mask of PROC0_CYCLES field.
	TICKS_PROC0_CYCLES_PROC0_CYCLES_Msk = 0x1ff

	// PROC0_COUNT
	// Position of PROC0_COUNT field.
	TICKS_PROC0_COUNT_PROC0_COUNT_Pos = 0x0
	// Bit mask of PROC0_COUNT field.
	TICKS_PROC0_COUNT_PROC0_COUNT_Msk = 0x1ff

	// PROC1_CTRL: Controls the tick generator
	// Position of RUNNING field.
	TICKS_PROC1_CTRL_RUNNING_Pos = 0x1
	// Bit mask of RUNNING field.
	TICKS_PROC1_CTRL_RUNNING_Msk = 0x2
	// Bit RUNNING.
	TICKS_PROC1_CTRL_RUNNING = 0x2
	// Position of ENABLE field.
	TICKS_PROC1_CTRL_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	TICKS_PROC1_CTRL_ENABLE_Msk = 0x1
	// Bit ENABLE.
	TICKS_PROC1_CTRL_ENABLE = 0x1

	// PROC1_CYCLES
	// Position of PROC1_CYCLES field.
	TICKS_PROC1_CYCLES_PROC1_CYCLES_Pos = 0x0
	// Bit mask of PROC1_CYCLES field.
	TICKS_PROC1_CYCLES_PROC1_CYCLES_Msk = 0x1ff

	// PROC1_COUNT
	// Position of PROC1_COUNT field.
	TICKS_PROC1_COUNT_PROC1_COUNT_Pos = 0x0
	// Bit mask of PROC1_COUNT field.
	TICKS_PROC1_COUNT_PROC1_COUNT_Msk = 0x1ff

	// TIMER0_CTRL: Controls the tick generator
	// Position of RUNNING field.
	TICKS_TIMER0_CTRL_RUNNING_Pos = 0x1
	// Bit mask of RUNNING field.
	TICKS_TIMER0_CTRL_RUNNING_Msk = 0x2
	// Bit RUNNING.
	TICKS_TIMER0_CTRL_RUNNING = 0x2
	// Position of ENABLE field.
	TICKS_TIMER0_CTRL_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	TICKS_TIMER0_CTRL_ENABLE_Msk = 0x1
	// Bit ENABLE.
	TICKS_TIMER0_CTRL_ENABLE = 0x1

	// TIMER0_CYCLES
	// Position of TIMER0_CYCLES field.
	TICKS_TIMER0_CYCLES_TIMER0_CYCLES_Pos = 0x0
	// Bit mask of TIMER0_CYCLES field.
	TICKS_TIMER0_CYCLES_TIMER0_CYCLES_Msk = 0x1ff

	// TIMER0_COUNT
	// Position of TIMER0_COUNT field.
	TICKS_TIMER0_COUNT_TIMER0_COUNT_Pos = 0x0
	// Bit mask of TIMER0_COUNT field.
	TICKS_TIMER0_COUNT_TIMER0_COUNT_Msk = 0x1ff

	// TIMER1_CTRL: Controls the tick generator
	// Position of RUNNING field.
	TICKS_TIMER1_CTRL_RUNNING_Pos = 0x1
	// Bit mask of RUNNING field.
	TICKS_TIMER1_CTRL_RUNNING_Msk = 0x2
	// Bit RUNNING.
	TICKS_TIMER1_CTRL_RUNNING = 0x2
	// Position of ENABLE field.
	TICKS_TIMER1_CTRL_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	TICKS_TIMER1_CTRL_ENABLE_Msk = 0x1
	// Bit ENABLE.
	TICKS_TIMER1_CTRL_ENABLE = 0x1

	// TIMER1_CYCLES
	// Position of TIMER1_CYCLES field.
	TICKS_TIMER1_CYCLES_TIMER1_CYCLES_Pos = 0x0
	// Bit mask of TIMER1_CYCLES field.
	TICKS_TIMER1_CYCLES_TIMER1_CYCLES_Msk = 0x1ff

	// TIMER1_COUNT
	// Position of TIMER1_COUNT field.
	TICKS_TIMER1_COUNT_TIMER1_COUNT_Pos = 0x0
	// Bit mask of TIMER1_COUNT field.
	TICKS_TIMER1_COUNT_TIMER1_COUNT_Msk = 0x1ff

	// WATCHDOG_CTRL: Controls the tick generator
	// Position of RUNNING field.
	TICKS_WATCHDOG_CTRL_RUNNING_Pos = 0x1
	// Bit mask of RUNNING field.
	TICKS_WATCHDOG_CTRL_RUNNING_Msk = 0x2
	// Bit RUNNING.
	TICKS_WATCHDOG_CTRL_RUNNING = 0x2
	// Position of ENABLE field.
	TICKS_WATCHDOG_CTRL_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	TICKS_WATCHDOG_CTRL_ENABLE_Msk = 0x1
	// Bit ENABLE.
	TICKS_WATCHDOG_CTRL_ENABLE = 0x1

	// WATCHDOG_CYCLES
	// Position of WATCHDOG_CYCLES field.
	TICKS_WATCHDOG_CYCLES_WATCHDOG_CYCLES_Pos = 0x0
	// Bit mask of WATCHDOG_CYCLES field.
	TICKS_WATCHDOG_CYCLES_WATCHDOG_CYCLES_Msk = 0x1ff

	// WATCHDOG_COUNT
	// Position of WATCHDOG_COUNT field.
	TICKS_WATCHDOG_COUNT_WATCHDOG_COUNT_Pos = 0x0
	// Bit mask of WATCHDOG_COUNT field.
	TICKS_WATCHDOG_COUNT_WATCHDOG_COUNT_Msk = 0x1ff

	// RISCV_CTRL: Controls the tick generator
	// Position of RUNNING field.
	TICKS_RISCV_CTRL_RUNNING_Pos = 0x1
	// Bit mask of RUNNING field.
	TICKS_RISCV_CTRL_RUNNING_Msk = 0x2
	// Bit RUNNING.
	TICKS_RISCV_CTRL_RUNNING = 0x2
	// Position of ENABLE field.
	TICKS_RISCV_CTRL_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	TICKS_RISCV_CTRL_ENABLE_Msk = 0x1
	// Bit ENABLE.
	TICKS_RISCV_CTRL_ENABLE = 0x1

	// RISCV_CYCLES
	// Position of RISCV_CYCLES field.
	TICKS_RISCV_CYCLES_RISCV_CYCLES_Pos = 0x0
	// Bit mask of RISCV_CYCLES field.
	TICKS_RISCV_CYCLES_RISCV_CYCLES_Msk = 0x1ff

	// RISCV_COUNT
	// Position of RISCV_COUNT field.
	TICKS_RISCV_COUNT_RISCV_COUNT_Pos = 0x0
	// Bit mask of RISCV_COUNT field.
	TICKS_RISCV_COUNT_RISCV_COUNT_Msk = 0x1ff
)

// Constants for PADS_BANK0
const (
	// VOLTAGE_SELECT: Voltage select. Per bank control
	// Position of VOLTAGE_SELECT field.
	PADS_BANK0_VOLTAGE_SELECT_VOLTAGE_SELECT_Pos = 0x0
	// Bit mask of VOLTAGE_SELECT field.
	PADS_BANK0_VOLTAGE_SELECT_VOLTAGE_SELECT_Msk = 0x1
	// Bit VOLTAGE_SELECT.
	PADS_BANK0_VOLTAGE_SELECT_VOLTAGE_SELECT = 0x1
	// Set voltage to 3.3V (DVDD >= 2V5)
	PADS_BANK0_VOLTAGE_SELECT_VOLTAGE_SELECT_3v3 = 0x0
	// Set voltage to 1.8V (DVDD <= 1V8)
	PADS_BANK0_VOLTAGE_SELECT_VOLTAGE_SELECT_1v8 = 0x1

	// GPIO0
	// Position of ISO field.
	PADS_BANK0_GPIO0_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO0_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO0_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO0_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO0_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO0_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO0_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO0_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO0_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO0_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO0_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO0_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO0_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO0_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO0_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO0_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO0_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO0_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO0_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO0_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO0_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO0_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO0_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO0_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO0_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO0_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO0_SLEWFAST = 0x1

	// GPIO1
	// Position of ISO field.
	PADS_BANK0_GPIO1_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO1_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO1_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO1_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO1_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO1_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO1_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO1_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO1_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO1_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO1_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO1_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO1_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO1_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO1_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO1_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO1_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO1_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO1_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO1_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO1_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO1_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO1_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO1_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO1_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO1_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO1_SLEWFAST = 0x1

	// GPIO2
	// Position of ISO field.
	PADS_BANK0_GPIO2_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO2_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO2_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO2_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO2_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO2_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO2_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO2_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO2_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO2_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO2_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO2_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO2_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO2_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO2_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO2_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO2_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO2_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO2_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO2_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO2_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO2_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO2_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO2_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO2_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO2_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO2_SLEWFAST = 0x1

	// GPIO3
	// Position of ISO field.
	PADS_BANK0_GPIO3_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO3_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO3_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO3_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO3_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO3_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO3_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO3_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO3_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO3_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO3_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO3_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO3_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO3_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO3_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO3_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO3_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO3_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO3_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO3_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO3_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO3_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO3_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO3_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO3_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO3_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO3_SLEWFAST = 0x1

	// GPIO4
	// Position of ISO field.
	PADS_BANK0_GPIO4_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO4_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO4_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO4_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO4_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO4_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO4_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO4_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO4_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO4_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO4_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO4_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO4_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO4_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO4_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO4_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO4_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO4_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO4_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO4_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO4_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO4_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO4_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO4_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO4_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO4_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO4_SLEWFAST = 0x1

	// GPIO5
	// Position of ISO field.
	PADS_BANK0_GPIO5_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO5_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO5_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO5_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO5_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO5_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO5_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO5_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO5_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO5_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO5_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO5_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO5_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO5_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO5_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO5_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO5_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO5_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO5_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO5_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO5_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO5_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO5_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO5_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO5_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO5_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO5_SLEWFAST = 0x1

	// GPIO6
	// Position of ISO field.
	PADS_BANK0_GPIO6_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO6_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO6_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO6_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO6_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO6_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO6_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO6_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO6_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO6_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO6_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO6_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO6_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO6_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO6_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO6_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO6_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO6_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO6_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO6_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO6_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO6_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO6_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO6_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO6_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO6_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO6_SLEWFAST = 0x1

	// GPIO7
	// Position of ISO field.
	PADS_BANK0_GPIO7_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO7_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO7_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO7_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO7_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO7_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO7_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO7_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO7_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO7_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO7_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO7_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO7_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO7_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO7_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO7_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO7_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO7_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO7_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO7_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO7_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO7_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO7_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO7_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO7_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO7_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO7_SLEWFAST = 0x1

	// GPIO8
	// Position of ISO field.
	PADS_BANK0_GPIO8_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO8_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO8_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO8_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO8_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO8_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO8_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO8_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO8_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO8_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO8_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO8_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO8_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO8_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO8_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO8_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO8_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO8_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO8_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO8_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO8_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO8_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO8_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO8_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO8_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO8_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO8_SLEWFAST = 0x1

	// GPIO9
	// Position of ISO field.
	PADS_BANK0_GPIO9_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO9_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO9_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO9_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO9_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO9_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO9_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO9_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO9_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO9_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO9_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO9_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO9_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO9_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO9_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO9_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO9_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO9_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO9_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO9_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO9_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO9_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO9_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO9_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO9_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO9_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO9_SLEWFAST = 0x1

	// GPIO10
	// Position of ISO field.
	PADS_BANK0_GPIO10_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO10_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO10_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO10_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO10_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO10_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO10_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO10_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO10_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO10_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO10_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO10_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO10_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO10_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO10_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO10_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO10_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO10_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO10_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO10_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO10_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO10_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO10_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO10_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO10_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO10_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO10_SLEWFAST = 0x1

	// GPIO11
	// Position of ISO field.
	PADS_BANK0_GPIO11_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO11_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO11_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO11_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO11_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO11_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO11_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO11_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO11_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO11_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO11_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO11_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO11_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO11_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO11_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO11_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO11_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO11_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO11_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO11_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO11_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO11_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO11_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO11_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO11_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO11_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO11_SLEWFAST = 0x1

	// GPIO12
	// Position of ISO field.
	PADS_BANK0_GPIO12_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO12_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO12_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO12_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO12_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO12_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO12_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO12_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO12_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO12_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO12_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO12_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO12_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO12_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO12_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO12_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO12_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO12_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO12_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO12_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO12_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO12_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO12_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO12_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO12_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO12_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO12_SLEWFAST = 0x1

	// GPIO13
	// Position of ISO field.
	PADS_BANK0_GPIO13_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO13_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO13_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO13_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO13_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO13_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO13_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO13_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO13_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO13_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO13_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO13_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO13_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO13_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO13_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO13_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO13_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO13_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO13_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO13_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO13_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO13_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO13_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO13_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO13_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO13_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO13_SLEWFAST = 0x1

	// GPIO14
	// Position of ISO field.
	PADS_BANK0_GPIO14_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO14_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO14_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO14_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO14_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO14_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO14_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO14_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO14_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO14_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO14_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO14_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO14_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO14_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO14_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO14_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO14_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO14_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO14_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO14_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO14_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO14_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO14_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO14_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO14_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO14_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO14_SLEWFAST = 0x1

	// GPIO15
	// Position of ISO field.
	PADS_BANK0_GPIO15_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO15_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO15_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO15_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO15_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO15_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO15_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO15_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO15_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO15_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO15_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO15_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO15_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO15_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO15_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO15_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO15_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO15_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO15_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO15_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO15_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO15_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO15_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO15_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO15_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO15_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO15_SLEWFAST = 0x1

	// GPIO16
	// Position of ISO field.
	PADS_BANK0_GPIO16_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO16_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO16_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO16_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO16_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO16_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO16_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO16_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO16_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO16_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO16_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO16_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO16_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO16_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO16_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO16_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO16_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO16_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO16_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO16_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO16_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO16_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO16_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO16_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO16_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO16_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO16_SLEWFAST = 0x1

	// GPIO17
	// Position of ISO field.
	PADS_BANK0_GPIO17_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO17_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO17_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO17_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO17_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO17_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO17_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO17_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO17_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO17_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO17_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO17_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO17_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO17_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO17_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO17_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO17_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO17_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO17_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO17_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO17_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO17_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO17_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO17_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO17_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO17_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO17_SLEWFAST = 0x1

	// GPIO18
	// Position of ISO field.
	PADS_BANK0_GPIO18_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO18_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO18_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO18_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO18_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO18_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO18_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO18_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO18_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO18_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO18_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO18_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO18_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO18_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO18_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO18_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO18_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO18_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO18_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO18_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO18_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO18_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO18_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO18_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO18_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO18_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO18_SLEWFAST = 0x1

	// GPIO19
	// Position of ISO field.
	PADS_BANK0_GPIO19_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO19_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO19_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO19_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO19_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO19_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO19_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO19_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO19_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO19_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO19_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO19_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO19_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO19_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO19_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO19_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO19_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO19_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO19_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO19_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO19_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO19_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO19_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO19_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO19_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO19_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO19_SLEWFAST = 0x1

	// GPIO20
	// Position of ISO field.
	PADS_BANK0_GPIO20_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO20_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO20_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO20_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO20_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO20_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO20_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO20_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO20_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO20_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO20_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO20_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO20_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO20_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO20_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO20_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO20_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO20_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO20_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO20_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO20_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO20_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO20_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO20_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO20_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO20_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO20_SLEWFAST = 0x1

	// GPIO21
	// Position of ISO field.
	PADS_BANK0_GPIO21_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO21_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO21_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO21_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO21_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO21_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO21_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO21_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO21_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO21_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO21_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO21_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO21_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO21_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO21_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO21_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO21_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO21_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO21_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO21_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO21_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO21_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO21_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO21_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO21_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO21_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO21_SLEWFAST = 0x1

	// GPIO22
	// Position of ISO field.
	PADS_BANK0_GPIO22_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO22_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO22_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO22_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO22_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO22_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO22_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO22_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO22_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO22_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO22_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO22_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO22_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO22_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO22_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO22_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO22_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO22_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO22_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO22_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO22_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO22_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO22_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO22_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO22_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO22_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO22_SLEWFAST = 0x1

	// GPIO23
	// Position of ISO field.
	PADS_BANK0_GPIO23_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO23_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO23_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO23_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO23_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO23_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO23_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO23_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO23_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO23_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO23_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO23_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO23_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO23_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO23_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO23_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO23_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO23_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO23_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO23_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO23_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO23_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO23_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO23_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO23_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO23_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO23_SLEWFAST = 0x1

	// GPIO24
	// Position of ISO field.
	PADS_BANK0_GPIO24_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO24_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO24_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO24_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO24_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO24_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO24_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO24_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO24_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO24_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO24_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO24_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO24_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO24_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO24_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO24_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO24_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO24_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO24_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO24_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO24_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO24_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO24_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO24_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO24_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO24_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO24_SLEWFAST = 0x1

	// GPIO25
	// Position of ISO field.
	PADS_BANK0_GPIO25_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO25_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO25_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO25_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO25_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO25_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO25_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO25_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO25_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO25_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO25_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO25_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO25_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO25_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO25_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO25_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO25_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO25_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO25_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO25_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO25_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO25_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO25_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO25_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO25_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO25_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO25_SLEWFAST = 0x1

	// GPIO26
	// Position of ISO field.
	PADS_BANK0_GPIO26_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO26_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO26_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO26_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO26_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO26_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO26_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO26_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO26_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO26_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO26_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO26_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO26_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO26_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO26_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO26_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO26_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO26_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO26_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO26_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO26_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO26_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO26_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO26_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO26_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO26_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO26_SLEWFAST = 0x1

	// GPIO27
	// Position of ISO field.
	PADS_BANK0_GPIO27_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO27_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO27_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO27_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO27_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO27_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO27_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO27_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO27_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO27_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO27_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO27_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO27_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO27_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO27_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO27_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO27_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO27_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO27_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO27_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO27_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO27_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO27_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO27_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO27_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO27_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO27_SLEWFAST = 0x1

	// GPIO28
	// Position of ISO field.
	PADS_BANK0_GPIO28_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO28_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO28_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO28_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO28_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO28_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO28_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO28_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO28_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO28_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO28_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO28_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO28_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO28_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO28_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO28_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO28_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO28_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO28_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO28_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO28_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO28_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO28_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO28_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO28_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO28_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO28_SLEWFAST = 0x1

	// GPIO29
	// Position of ISO field.
	PADS_BANK0_GPIO29_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO29_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO29_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO29_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO29_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO29_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO29_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO29_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO29_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO29_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO29_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO29_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO29_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO29_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO29_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO29_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO29_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO29_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO29_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO29_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO29_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO29_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO29_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO29_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO29_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO29_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO29_SLEWFAST = 0x1

	// GPIO30
	// Position of ISO field.
	PADS_BANK0_GPIO30_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO30_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO30_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO30_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO30_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO30_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO30_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO30_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO30_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO30_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO30_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO30_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO30_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO30_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO30_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO30_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO30_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO30_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO30_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO30_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO30_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO30_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO30_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO30_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO30_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO30_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO30_SLEWFAST = 0x1

	// GPIO31
	// Position of ISO field.
	PADS_BANK0_GPIO31_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO31_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO31_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO31_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO31_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO31_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO31_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO31_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO31_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO31_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO31_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO31_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO31_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO31_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO31_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO31_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO31_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO31_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO31_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO31_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO31_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO31_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO31_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO31_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO31_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO31_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO31_SLEWFAST = 0x1

	// GPIO32
	// Position of ISO field.
	PADS_BANK0_GPIO32_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO32_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO32_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO32_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO32_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO32_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO32_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO32_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO32_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO32_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO32_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO32_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO32_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO32_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO32_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO32_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO32_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO32_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO32_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO32_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO32_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO32_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO32_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO32_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO32_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO32_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO32_SLEWFAST = 0x1

	// GPIO33
	// Position of ISO field.
	PADS_BANK0_GPIO33_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO33_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO33_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO33_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO33_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO33_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO33_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO33_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO33_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO33_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO33_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO33_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO33_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO33_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO33_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO33_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO33_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO33_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO33_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO33_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO33_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO33_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO33_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO33_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO33_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO33_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO33_SLEWFAST = 0x1

	// GPIO34
	// Position of ISO field.
	PADS_BANK0_GPIO34_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO34_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO34_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO34_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO34_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO34_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO34_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO34_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO34_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO34_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO34_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO34_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO34_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO34_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO34_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO34_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO34_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO34_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO34_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO34_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO34_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO34_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO34_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO34_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO34_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO34_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO34_SLEWFAST = 0x1

	// GPIO35
	// Position of ISO field.
	PADS_BANK0_GPIO35_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO35_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO35_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO35_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO35_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO35_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO35_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO35_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO35_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO35_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO35_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO35_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO35_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO35_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO35_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO35_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO35_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO35_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO35_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO35_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO35_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO35_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO35_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO35_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO35_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO35_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO35_SLEWFAST = 0x1

	// GPIO36
	// Position of ISO field.
	PADS_BANK0_GPIO36_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO36_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO36_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO36_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO36_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO36_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO36_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO36_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO36_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO36_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO36_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO36_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO36_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO36_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO36_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO36_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO36_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO36_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO36_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO36_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO36_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO36_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO36_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO36_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO36_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO36_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO36_SLEWFAST = 0x1

	// GPIO37
	// Position of ISO field.
	PADS_BANK0_GPIO37_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO37_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO37_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO37_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO37_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO37_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO37_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO37_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO37_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO37_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO37_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO37_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO37_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO37_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO37_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO37_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO37_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO37_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO37_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO37_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO37_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO37_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO37_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO37_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO37_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO37_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO37_SLEWFAST = 0x1

	// GPIO38
	// Position of ISO field.
	PADS_BANK0_GPIO38_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO38_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO38_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO38_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO38_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO38_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO38_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO38_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO38_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO38_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO38_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO38_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO38_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO38_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO38_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO38_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO38_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO38_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO38_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO38_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO38_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO38_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO38_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO38_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO38_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO38_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO38_SLEWFAST = 0x1

	// GPIO39
	// Position of ISO field.
	PADS_BANK0_GPIO39_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO39_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO39_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO39_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO39_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO39_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO39_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO39_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO39_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO39_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO39_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO39_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO39_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO39_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO39_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO39_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO39_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO39_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO39_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO39_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO39_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO39_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO39_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO39_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO39_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO39_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO39_SLEWFAST = 0x1

	// GPIO40
	// Position of ISO field.
	PADS_BANK0_GPIO40_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO40_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO40_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO40_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO40_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO40_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO40_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO40_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO40_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO40_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO40_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO40_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO40_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO40_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO40_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO40_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO40_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO40_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO40_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO40_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO40_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO40_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO40_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO40_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO40_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO40_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO40_SLEWFAST = 0x1

	// GPIO41
	// Position of ISO field.
	PADS_BANK0_GPIO41_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO41_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO41_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO41_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO41_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO41_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO41_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO41_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO41_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO41_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO41_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO41_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO41_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO41_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO41_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO41_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO41_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO41_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO41_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO41_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO41_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO41_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO41_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO41_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO41_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO41_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO41_SLEWFAST = 0x1

	// GPIO42
	// Position of ISO field.
	PADS_BANK0_GPIO42_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO42_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO42_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO42_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO42_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO42_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO42_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO42_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO42_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO42_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO42_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO42_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO42_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO42_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO42_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO42_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO42_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO42_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO42_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO42_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO42_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO42_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO42_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO42_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO42_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO42_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO42_SLEWFAST = 0x1

	// GPIO43
	// Position of ISO field.
	PADS_BANK0_GPIO43_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO43_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO43_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO43_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO43_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO43_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO43_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO43_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO43_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO43_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO43_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO43_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO43_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO43_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO43_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO43_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO43_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO43_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO43_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO43_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO43_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO43_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO43_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO43_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO43_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO43_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO43_SLEWFAST = 0x1

	// GPIO44
	// Position of ISO field.
	PADS_BANK0_GPIO44_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO44_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO44_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO44_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO44_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO44_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO44_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO44_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO44_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO44_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO44_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO44_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO44_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO44_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO44_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO44_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO44_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO44_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO44_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO44_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO44_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO44_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO44_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO44_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO44_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO44_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO44_SLEWFAST = 0x1

	// GPIO45
	// Position of ISO field.
	PADS_BANK0_GPIO45_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO45_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO45_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO45_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO45_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO45_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO45_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO45_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO45_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO45_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO45_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO45_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO45_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO45_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO45_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO45_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO45_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO45_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO45_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO45_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO45_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO45_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO45_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO45_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO45_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO45_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO45_SLEWFAST = 0x1

	// GPIO46
	// Position of ISO field.
	PADS_BANK0_GPIO46_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO46_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO46_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO46_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO46_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO46_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO46_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO46_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO46_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO46_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO46_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO46_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO46_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO46_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO46_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO46_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO46_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO46_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO46_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO46_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO46_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO46_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO46_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO46_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO46_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO46_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO46_SLEWFAST = 0x1

	// GPIO47
	// Position of ISO field.
	PADS_BANK0_GPIO47_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_GPIO47_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_GPIO47_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_GPIO47_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_GPIO47_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_GPIO47_OD = 0x80
	// Position of IE field.
	PADS_BANK0_GPIO47_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_GPIO47_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_GPIO47_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_GPIO47_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_GPIO47_DRIVE_Msk  = 0x30
	PADS_BANK0_GPIO47_DRIVE_2mA  = 0x0
	PADS_BANK0_GPIO47_DRIVE_4mA  = 0x1
	PADS_BANK0_GPIO47_DRIVE_8mA  = 0x2
	PADS_BANK0_GPIO47_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_GPIO47_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_GPIO47_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_GPIO47_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_GPIO47_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_GPIO47_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_GPIO47_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_GPIO47_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_GPIO47_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_GPIO47_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_GPIO47_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_GPIO47_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_GPIO47_SLEWFAST = 0x1

	// SWCLK
	// Position of ISO field.
	PADS_BANK0_SWCLK_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_SWCLK_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_SWCLK_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_SWCLK_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_SWCLK_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_SWCLK_OD = 0x80
	// Position of IE field.
	PADS_BANK0_SWCLK_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_SWCLK_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_SWCLK_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_SWCLK_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_SWCLK_DRIVE_Msk  = 0x30
	PADS_BANK0_SWCLK_DRIVE_2mA  = 0x0
	PADS_BANK0_SWCLK_DRIVE_4mA  = 0x1
	PADS_BANK0_SWCLK_DRIVE_8mA  = 0x2
	PADS_BANK0_SWCLK_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_SWCLK_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_SWCLK_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_SWCLK_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_SWCLK_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_SWCLK_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_SWCLK_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_SWCLK_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_SWCLK_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_SWCLK_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_SWCLK_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_SWCLK_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_SWCLK_SLEWFAST = 0x1

	// SWD
	// Position of ISO field.
	PADS_BANK0_SWD_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_BANK0_SWD_ISO_Msk = 0x100
	// Bit ISO.
	PADS_BANK0_SWD_ISO = 0x100
	// Position of OD field.
	PADS_BANK0_SWD_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_BANK0_SWD_OD_Msk = 0x80
	// Bit OD.
	PADS_BANK0_SWD_OD = 0x80
	// Position of IE field.
	PADS_BANK0_SWD_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_BANK0_SWD_IE_Msk = 0x40
	// Bit IE.
	PADS_BANK0_SWD_IE = 0x40
	// Position of DRIVE field.
	PADS_BANK0_SWD_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_BANK0_SWD_DRIVE_Msk  = 0x30
	PADS_BANK0_SWD_DRIVE_2mA  = 0x0
	PADS_BANK0_SWD_DRIVE_4mA  = 0x1
	PADS_BANK0_SWD_DRIVE_8mA  = 0x2
	PADS_BANK0_SWD_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_BANK0_SWD_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_BANK0_SWD_PUE_Msk = 0x8
	// Bit PUE.
	PADS_BANK0_SWD_PUE = 0x8
	// Position of PDE field.
	PADS_BANK0_SWD_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_BANK0_SWD_PDE_Msk = 0x4
	// Bit PDE.
	PADS_BANK0_SWD_PDE = 0x4
	// Position of SCHMITT field.
	PADS_BANK0_SWD_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_BANK0_SWD_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_BANK0_SWD_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_BANK0_SWD_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_BANK0_SWD_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_BANK0_SWD_SLEWFAST = 0x1
)

// Constants for PADS_QSPI
const (
	// VOLTAGE_SELECT: Voltage select. Per bank control
	// Position of VOLTAGE_SELECT field.
	PADS_QSPI_VOLTAGE_SELECT_VOLTAGE_SELECT_Pos = 0x0
	// Bit mask of VOLTAGE_SELECT field.
	PADS_QSPI_VOLTAGE_SELECT_VOLTAGE_SELECT_Msk = 0x1
	// Bit VOLTAGE_SELECT.
	PADS_QSPI_VOLTAGE_SELECT_VOLTAGE_SELECT = 0x1
	// Set voltage to 3.3V (DVDD >= 2V5)
	PADS_QSPI_VOLTAGE_SELECT_VOLTAGE_SELECT_3v3 = 0x0
	// Set voltage to 1.8V (DVDD <= 1V8)
	PADS_QSPI_VOLTAGE_SELECT_VOLTAGE_SELECT_1v8 = 0x1

	// GPIO_QSPI_SCLK
	// Position of ISO field.
	PADS_QSPI_GPIO_QSPI_SCLK_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_QSPI_GPIO_QSPI_SCLK_ISO_Msk = 0x100
	// Bit ISO.
	PADS_QSPI_GPIO_QSPI_SCLK_ISO = 0x100
	// Position of OD field.
	PADS_QSPI_GPIO_QSPI_SCLK_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_QSPI_GPIO_QSPI_SCLK_OD_Msk = 0x80
	// Bit OD.
	PADS_QSPI_GPIO_QSPI_SCLK_OD = 0x80
	// Position of IE field.
	PADS_QSPI_GPIO_QSPI_SCLK_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_QSPI_GPIO_QSPI_SCLK_IE_Msk = 0x40
	// Bit IE.
	PADS_QSPI_GPIO_QSPI_SCLK_IE = 0x40
	// Position of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SCLK_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SCLK_DRIVE_Msk  = 0x30
	PADS_QSPI_GPIO_QSPI_SCLK_DRIVE_2mA  = 0x0
	PADS_QSPI_GPIO_QSPI_SCLK_DRIVE_4mA  = 0x1
	PADS_QSPI_GPIO_QSPI_SCLK_DRIVE_8mA  = 0x2
	PADS_QSPI_GPIO_QSPI_SCLK_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_QSPI_GPIO_QSPI_SCLK_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_QSPI_GPIO_QSPI_SCLK_PUE_Msk = 0x8
	// Bit PUE.
	PADS_QSPI_GPIO_QSPI_SCLK_PUE = 0x8
	// Position of PDE field.
	PADS_QSPI_GPIO_QSPI_SCLK_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_QSPI_GPIO_QSPI_SCLK_PDE_Msk = 0x4
	// Bit PDE.
	PADS_QSPI_GPIO_QSPI_SCLK_PDE = 0x4
	// Position of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SCLK_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SCLK_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_QSPI_GPIO_QSPI_SCLK_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SCLK_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SCLK_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_QSPI_GPIO_QSPI_SCLK_SLEWFAST = 0x1

	// GPIO_QSPI_SD0
	// Position of ISO field.
	PADS_QSPI_GPIO_QSPI_SD0_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_QSPI_GPIO_QSPI_SD0_ISO_Msk = 0x100
	// Bit ISO.
	PADS_QSPI_GPIO_QSPI_SD0_ISO = 0x100
	// Position of OD field.
	PADS_QSPI_GPIO_QSPI_SD0_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_QSPI_GPIO_QSPI_SD0_OD_Msk = 0x80
	// Bit OD.
	PADS_QSPI_GPIO_QSPI_SD0_OD = 0x80
	// Position of IE field.
	PADS_QSPI_GPIO_QSPI_SD0_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_QSPI_GPIO_QSPI_SD0_IE_Msk = 0x40
	// Bit IE.
	PADS_QSPI_GPIO_QSPI_SD0_IE = 0x40
	// Position of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SD0_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SD0_DRIVE_Msk  = 0x30
	PADS_QSPI_GPIO_QSPI_SD0_DRIVE_2mA  = 0x0
	PADS_QSPI_GPIO_QSPI_SD0_DRIVE_4mA  = 0x1
	PADS_QSPI_GPIO_QSPI_SD0_DRIVE_8mA  = 0x2
	PADS_QSPI_GPIO_QSPI_SD0_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_QSPI_GPIO_QSPI_SD0_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_QSPI_GPIO_QSPI_SD0_PUE_Msk = 0x8
	// Bit PUE.
	PADS_QSPI_GPIO_QSPI_SD0_PUE = 0x8
	// Position of PDE field.
	PADS_QSPI_GPIO_QSPI_SD0_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_QSPI_GPIO_QSPI_SD0_PDE_Msk = 0x4
	// Bit PDE.
	PADS_QSPI_GPIO_QSPI_SD0_PDE = 0x4
	// Position of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SD0_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SD0_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_QSPI_GPIO_QSPI_SD0_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SD0_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SD0_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_QSPI_GPIO_QSPI_SD0_SLEWFAST = 0x1

	// GPIO_QSPI_SD1
	// Position of ISO field.
	PADS_QSPI_GPIO_QSPI_SD1_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_QSPI_GPIO_QSPI_SD1_ISO_Msk = 0x100
	// Bit ISO.
	PADS_QSPI_GPIO_QSPI_SD1_ISO = 0x100
	// Position of OD field.
	PADS_QSPI_GPIO_QSPI_SD1_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_QSPI_GPIO_QSPI_SD1_OD_Msk = 0x80
	// Bit OD.
	PADS_QSPI_GPIO_QSPI_SD1_OD = 0x80
	// Position of IE field.
	PADS_QSPI_GPIO_QSPI_SD1_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_QSPI_GPIO_QSPI_SD1_IE_Msk = 0x40
	// Bit IE.
	PADS_QSPI_GPIO_QSPI_SD1_IE = 0x40
	// Position of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SD1_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SD1_DRIVE_Msk  = 0x30
	PADS_QSPI_GPIO_QSPI_SD1_DRIVE_2mA  = 0x0
	PADS_QSPI_GPIO_QSPI_SD1_DRIVE_4mA  = 0x1
	PADS_QSPI_GPIO_QSPI_SD1_DRIVE_8mA  = 0x2
	PADS_QSPI_GPIO_QSPI_SD1_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_QSPI_GPIO_QSPI_SD1_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_QSPI_GPIO_QSPI_SD1_PUE_Msk = 0x8
	// Bit PUE.
	PADS_QSPI_GPIO_QSPI_SD1_PUE = 0x8
	// Position of PDE field.
	PADS_QSPI_GPIO_QSPI_SD1_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_QSPI_GPIO_QSPI_SD1_PDE_Msk = 0x4
	// Bit PDE.
	PADS_QSPI_GPIO_QSPI_SD1_PDE = 0x4
	// Position of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SD1_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SD1_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_QSPI_GPIO_QSPI_SD1_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SD1_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SD1_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_QSPI_GPIO_QSPI_SD1_SLEWFAST = 0x1

	// GPIO_QSPI_SD2
	// Position of ISO field.
	PADS_QSPI_GPIO_QSPI_SD2_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_QSPI_GPIO_QSPI_SD2_ISO_Msk = 0x100
	// Bit ISO.
	PADS_QSPI_GPIO_QSPI_SD2_ISO = 0x100
	// Position of OD field.
	PADS_QSPI_GPIO_QSPI_SD2_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_QSPI_GPIO_QSPI_SD2_OD_Msk = 0x80
	// Bit OD.
	PADS_QSPI_GPIO_QSPI_SD2_OD = 0x80
	// Position of IE field.
	PADS_QSPI_GPIO_QSPI_SD2_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_QSPI_GPIO_QSPI_SD2_IE_Msk = 0x40
	// Bit IE.
	PADS_QSPI_GPIO_QSPI_SD2_IE = 0x40
	// Position of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SD2_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SD2_DRIVE_Msk  = 0x30
	PADS_QSPI_GPIO_QSPI_SD2_DRIVE_2mA  = 0x0
	PADS_QSPI_GPIO_QSPI_SD2_DRIVE_4mA  = 0x1
	PADS_QSPI_GPIO_QSPI_SD2_DRIVE_8mA  = 0x2
	PADS_QSPI_GPIO_QSPI_SD2_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_QSPI_GPIO_QSPI_SD2_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_QSPI_GPIO_QSPI_SD2_PUE_Msk = 0x8
	// Bit PUE.
	PADS_QSPI_GPIO_QSPI_SD2_PUE = 0x8
	// Position of PDE field.
	PADS_QSPI_GPIO_QSPI_SD2_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_QSPI_GPIO_QSPI_SD2_PDE_Msk = 0x4
	// Bit PDE.
	PADS_QSPI_GPIO_QSPI_SD2_PDE = 0x4
	// Position of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SD2_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SD2_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_QSPI_GPIO_QSPI_SD2_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SD2_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SD2_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_QSPI_GPIO_QSPI_SD2_SLEWFAST = 0x1

	// GPIO_QSPI_SD3
	// Position of ISO field.
	PADS_QSPI_GPIO_QSPI_SD3_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_QSPI_GPIO_QSPI_SD3_ISO_Msk = 0x100
	// Bit ISO.
	PADS_QSPI_GPIO_QSPI_SD3_ISO = 0x100
	// Position of OD field.
	PADS_QSPI_GPIO_QSPI_SD3_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_QSPI_GPIO_QSPI_SD3_OD_Msk = 0x80
	// Bit OD.
	PADS_QSPI_GPIO_QSPI_SD3_OD = 0x80
	// Position of IE field.
	PADS_QSPI_GPIO_QSPI_SD3_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_QSPI_GPIO_QSPI_SD3_IE_Msk = 0x40
	// Bit IE.
	PADS_QSPI_GPIO_QSPI_SD3_IE = 0x40
	// Position of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SD3_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SD3_DRIVE_Msk  = 0x30
	PADS_QSPI_GPIO_QSPI_SD3_DRIVE_2mA  = 0x0
	PADS_QSPI_GPIO_QSPI_SD3_DRIVE_4mA  = 0x1
	PADS_QSPI_GPIO_QSPI_SD3_DRIVE_8mA  = 0x2
	PADS_QSPI_GPIO_QSPI_SD3_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_QSPI_GPIO_QSPI_SD3_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_QSPI_GPIO_QSPI_SD3_PUE_Msk = 0x8
	// Bit PUE.
	PADS_QSPI_GPIO_QSPI_SD3_PUE = 0x8
	// Position of PDE field.
	PADS_QSPI_GPIO_QSPI_SD3_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_QSPI_GPIO_QSPI_SD3_PDE_Msk = 0x4
	// Bit PDE.
	PADS_QSPI_GPIO_QSPI_SD3_PDE = 0x4
	// Position of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SD3_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SD3_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_QSPI_GPIO_QSPI_SD3_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SD3_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SD3_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_QSPI_GPIO_QSPI_SD3_SLEWFAST = 0x1

	// GPIO_QSPI_SS
	// Position of ISO field.
	PADS_QSPI_GPIO_QSPI_SS_ISO_Pos = 0x8
	// Bit mask of ISO field.
	PADS_QSPI_GPIO_QSPI_SS_ISO_Msk = 0x100
	// Bit ISO.
	PADS_QSPI_GPIO_QSPI_SS_ISO = 0x100
	// Position of OD field.
	PADS_QSPI_GPIO_QSPI_SS_OD_Pos = 0x7
	// Bit mask of OD field.
	PADS_QSPI_GPIO_QSPI_SS_OD_Msk = 0x80
	// Bit OD.
	PADS_QSPI_GPIO_QSPI_SS_OD = 0x80
	// Position of IE field.
	PADS_QSPI_GPIO_QSPI_SS_IE_Pos = 0x6
	// Bit mask of IE field.
	PADS_QSPI_GPIO_QSPI_SS_IE_Msk = 0x40
	// Bit IE.
	PADS_QSPI_GPIO_QSPI_SS_IE = 0x40
	// Position of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SS_DRIVE_Pos = 0x4
	// Bit mask of DRIVE field.
	PADS_QSPI_GPIO_QSPI_SS_DRIVE_Msk  = 0x30
	PADS_QSPI_GPIO_QSPI_SS_DRIVE_2mA  = 0x0
	PADS_QSPI_GPIO_QSPI_SS_DRIVE_4mA  = 0x1
	PADS_QSPI_GPIO_QSPI_SS_DRIVE_8mA  = 0x2
	PADS_QSPI_GPIO_QSPI_SS_DRIVE_12mA = 0x3
	// Position of PUE field.
	PADS_QSPI_GPIO_QSPI_SS_PUE_Pos = 0x3
	// Bit mask of PUE field.
	PADS_QSPI_GPIO_QSPI_SS_PUE_Msk = 0x8
	// Bit PUE.
	PADS_QSPI_GPIO_QSPI_SS_PUE = 0x8
	// Position of PDE field.
	PADS_QSPI_GPIO_QSPI_SS_PDE_Pos = 0x2
	// Bit mask of PDE field.
	PADS_QSPI_GPIO_QSPI_SS_PDE_Msk = 0x4
	// Bit PDE.
	PADS_QSPI_GPIO_QSPI_SS_PDE = 0x4
	// Position of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SS_SCHMITT_Pos = 0x1
	// Bit mask of SCHMITT field.
	PADS_QSPI_GPIO_QSPI_SS_SCHMITT_Msk = 0x2
	// Bit SCHMITT.
	PADS_QSPI_GPIO_QSPI_SS_SCHMITT = 0x2
	// Position of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SS_SLEWFAST_Pos = 0x0
	// Bit mask of SLEWFAST field.
	PADS_QSPI_GPIO_QSPI_SS_SLEWFAST_Msk = 0x1
	// Bit SLEWFAST.
	PADS_QSPI_GPIO_QSPI_SS_SLEWFAST = 0x1
)

// Constants for IO_QSPI
const (
	// USBPHY_DP_STATUS
	// Position of IRQTOPROC field.
	IO_QSPI_USBPHY_DP_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_QSPI_USBPHY_DP_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_QSPI_USBPHY_DP_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_QSPI_USBPHY_DP_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_QSPI_USBPHY_DP_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_QSPI_USBPHY_DP_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_QSPI_USBPHY_DP_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_QSPI_USBPHY_DP_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_QSPI_USBPHY_DP_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_QSPI_USBPHY_DP_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_QSPI_USBPHY_DP_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_QSPI_USBPHY_DP_STATUS_OUTTOPAD = 0x200

	// USBPHY_DP_CTRL
	// Position of IRQOVER field.
	IO_QSPI_USBPHY_DP_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_QSPI_USBPHY_DP_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_QSPI_USBPHY_DP_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_QSPI_USBPHY_DP_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_QSPI_USBPHY_DP_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_QSPI_USBPHY_DP_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_QSPI_USBPHY_DP_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_QSPI_USBPHY_DP_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_QSPI_USBPHY_DP_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_QSPI_USBPHY_DP_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_QSPI_USBPHY_DP_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_QSPI_USBPHY_DP_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_QSPI_USBPHY_DP_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_QSPI_USBPHY_DP_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_QSPI_USBPHY_DP_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_QSPI_USBPHY_DP_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_QSPI_USBPHY_DP_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_QSPI_USBPHY_DP_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_QSPI_USBPHY_DP_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_QSPI_USBPHY_DP_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_QSPI_USBPHY_DP_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_QSPI_USBPHY_DP_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_QSPI_USBPHY_DP_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_QSPI_USBPHY_DP_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_QSPI_USBPHY_DP_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_QSPI_USBPHY_DP_CTRL_FUNCSEL_Msk          = 0x1f
	IO_QSPI_USBPHY_DP_CTRL_FUNCSEL_UART1_TX     = 0x2
	IO_QSPI_USBPHY_DP_CTRL_FUNCSEL_I2C0_SDA     = 0x3
	IO_QSPI_USBPHY_DP_CTRL_FUNCSEL_SIOB_PROC_56 = 0x5
	IO_QSPI_USBPHY_DP_CTRL_FUNCSEL_NULL         = 0x1f

	// USBPHY_DM_STATUS
	// Position of IRQTOPROC field.
	IO_QSPI_USBPHY_DM_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_QSPI_USBPHY_DM_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_QSPI_USBPHY_DM_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_QSPI_USBPHY_DM_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_QSPI_USBPHY_DM_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_QSPI_USBPHY_DM_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_QSPI_USBPHY_DM_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_QSPI_USBPHY_DM_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_QSPI_USBPHY_DM_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_QSPI_USBPHY_DM_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_QSPI_USBPHY_DM_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_QSPI_USBPHY_DM_STATUS_OUTTOPAD = 0x200

	// USBPHY_DM_CTRL
	// Position of IRQOVER field.
	IO_QSPI_USBPHY_DM_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_QSPI_USBPHY_DM_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_QSPI_USBPHY_DM_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_QSPI_USBPHY_DM_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_QSPI_USBPHY_DM_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_QSPI_USBPHY_DM_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_QSPI_USBPHY_DM_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_QSPI_USBPHY_DM_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_QSPI_USBPHY_DM_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_QSPI_USBPHY_DM_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_QSPI_USBPHY_DM_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_QSPI_USBPHY_DM_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_QSPI_USBPHY_DM_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_QSPI_USBPHY_DM_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_QSPI_USBPHY_DM_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_QSPI_USBPHY_DM_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_QSPI_USBPHY_DM_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_QSPI_USBPHY_DM_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_QSPI_USBPHY_DM_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_QSPI_USBPHY_DM_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_QSPI_USBPHY_DM_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_QSPI_USBPHY_DM_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_QSPI_USBPHY_DM_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_QSPI_USBPHY_DM_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_QSPI_USBPHY_DM_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_QSPI_USBPHY_DM_CTRL_FUNCSEL_Msk          = 0x1f
	IO_QSPI_USBPHY_DM_CTRL_FUNCSEL_UART1_RX     = 0x2
	IO_QSPI_USBPHY_DM_CTRL_FUNCSEL_I2C0_SCL     = 0x3
	IO_QSPI_USBPHY_DM_CTRL_FUNCSEL_SIOB_PROC_57 = 0x5
	IO_QSPI_USBPHY_DM_CTRL_FUNCSEL_NULL         = 0x1f

	// GPIO_QSPI_SCLK_STATUS
	// Position of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_QSPI_GPIO_QSPI_SCLK_STATUS_OUTTOPAD = 0x200

	// GPIO_QSPI_SCLK_CTRL
	// Position of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_FUNCSEL_Msk          = 0x1f
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_FUNCSEL_XIP_SCLK     = 0x0
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_FUNCSEL_UART1_CTS    = 0x2
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_FUNCSEL_I2C1_SDA     = 0x3
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_FUNCSEL_SIOB_PROC_58 = 0x5
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_FUNCSEL_UART1_TX     = 0xb
	IO_QSPI_GPIO_QSPI_SCLK_CTRL_FUNCSEL_NULL         = 0x1f

	// GPIO_QSPI_SS_STATUS
	// Position of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SS_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SS_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_QSPI_GPIO_QSPI_SS_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SS_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SS_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_QSPI_GPIO_QSPI_SS_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SS_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SS_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_QSPI_GPIO_QSPI_SS_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SS_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SS_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_QSPI_GPIO_QSPI_SS_STATUS_OUTTOPAD = 0x200

	// GPIO_QSPI_SS_CTRL
	// Position of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_QSPI_GPIO_QSPI_SS_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_QSPI_GPIO_QSPI_SS_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_QSPI_GPIO_QSPI_SS_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_QSPI_GPIO_QSPI_SS_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_QSPI_GPIO_QSPI_SS_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_QSPI_GPIO_QSPI_SS_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_QSPI_GPIO_QSPI_SS_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_QSPI_GPIO_QSPI_SS_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SS_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SS_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_QSPI_GPIO_QSPI_SS_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_QSPI_GPIO_QSPI_SS_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SS_CTRL_FUNCSEL_Msk          = 0x1f
	IO_QSPI_GPIO_QSPI_SS_CTRL_FUNCSEL_XIP_SS_N_0   = 0x0
	IO_QSPI_GPIO_QSPI_SS_CTRL_FUNCSEL_UART1_RTS    = 0x2
	IO_QSPI_GPIO_QSPI_SS_CTRL_FUNCSEL_I2C1_SCL     = 0x3
	IO_QSPI_GPIO_QSPI_SS_CTRL_FUNCSEL_SIOB_PROC_59 = 0x5
	IO_QSPI_GPIO_QSPI_SS_CTRL_FUNCSEL_UART1_RX     = 0xb
	IO_QSPI_GPIO_QSPI_SS_CTRL_FUNCSEL_NULL         = 0x1f

	// GPIO_QSPI_SD0_STATUS
	// Position of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_QSPI_GPIO_QSPI_SD0_STATUS_OUTTOPAD = 0x200

	// GPIO_QSPI_SD0_CTRL
	// Position of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_QSPI_GPIO_QSPI_SD0_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_QSPI_GPIO_QSPI_SD0_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_QSPI_GPIO_QSPI_SD0_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_QSPI_GPIO_QSPI_SD0_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_QSPI_GPIO_QSPI_SD0_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_QSPI_GPIO_QSPI_SD0_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_QSPI_GPIO_QSPI_SD0_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_QSPI_GPIO_QSPI_SD0_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_QSPI_GPIO_QSPI_SD0_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SD0_CTRL_FUNCSEL_Msk          = 0x1f
	IO_QSPI_GPIO_QSPI_SD0_CTRL_FUNCSEL_XIP_SD0      = 0x0
	IO_QSPI_GPIO_QSPI_SD0_CTRL_FUNCSEL_UART0_TX     = 0x2
	IO_QSPI_GPIO_QSPI_SD0_CTRL_FUNCSEL_I2C0_SDA     = 0x3
	IO_QSPI_GPIO_QSPI_SD0_CTRL_FUNCSEL_SIOB_PROC_60 = 0x5
	IO_QSPI_GPIO_QSPI_SD0_CTRL_FUNCSEL_NULL         = 0x1f

	// GPIO_QSPI_SD1_STATUS
	// Position of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_QSPI_GPIO_QSPI_SD1_STATUS_OUTTOPAD = 0x200

	// GPIO_QSPI_SD1_CTRL
	// Position of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_QSPI_GPIO_QSPI_SD1_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_QSPI_GPIO_QSPI_SD1_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_QSPI_GPIO_QSPI_SD1_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_QSPI_GPIO_QSPI_SD1_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_QSPI_GPIO_QSPI_SD1_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_QSPI_GPIO_QSPI_SD1_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_QSPI_GPIO_QSPI_SD1_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_QSPI_GPIO_QSPI_SD1_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_QSPI_GPIO_QSPI_SD1_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SD1_CTRL_FUNCSEL_Msk          = 0x1f
	IO_QSPI_GPIO_QSPI_SD1_CTRL_FUNCSEL_XIP_SD1      = 0x0
	IO_QSPI_GPIO_QSPI_SD1_CTRL_FUNCSEL_UART0_RX     = 0x2
	IO_QSPI_GPIO_QSPI_SD1_CTRL_FUNCSEL_I2C0_SCL     = 0x3
	IO_QSPI_GPIO_QSPI_SD1_CTRL_FUNCSEL_SIOB_PROC_61 = 0x5
	IO_QSPI_GPIO_QSPI_SD1_CTRL_FUNCSEL_NULL         = 0x1f

	// GPIO_QSPI_SD2_STATUS
	// Position of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_QSPI_GPIO_QSPI_SD2_STATUS_OUTTOPAD = 0x200

	// GPIO_QSPI_SD2_CTRL
	// Position of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_QSPI_GPIO_QSPI_SD2_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_QSPI_GPIO_QSPI_SD2_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_QSPI_GPIO_QSPI_SD2_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_QSPI_GPIO_QSPI_SD2_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_QSPI_GPIO_QSPI_SD2_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_QSPI_GPIO_QSPI_SD2_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_QSPI_GPIO_QSPI_SD2_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_QSPI_GPIO_QSPI_SD2_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_QSPI_GPIO_QSPI_SD2_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SD2_CTRL_FUNCSEL_Msk          = 0x1f
	IO_QSPI_GPIO_QSPI_SD2_CTRL_FUNCSEL_XIP_SD2      = 0x0
	IO_QSPI_GPIO_QSPI_SD2_CTRL_FUNCSEL_UART0_CTS    = 0x2
	IO_QSPI_GPIO_QSPI_SD2_CTRL_FUNCSEL_I2C1_SDA     = 0x3
	IO_QSPI_GPIO_QSPI_SD2_CTRL_FUNCSEL_SIOB_PROC_62 = 0x5
	IO_QSPI_GPIO_QSPI_SD2_CTRL_FUNCSEL_UART0_TX     = 0xb
	IO_QSPI_GPIO_QSPI_SD2_CTRL_FUNCSEL_NULL         = 0x1f

	// GPIO_QSPI_SD3_STATUS
	// Position of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_QSPI_GPIO_QSPI_SD3_STATUS_OUTTOPAD = 0x200

	// GPIO_QSPI_SD3_CTRL
	// Position of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_QSPI_GPIO_QSPI_SD3_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_QSPI_GPIO_QSPI_SD3_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_QSPI_GPIO_QSPI_SD3_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_QSPI_GPIO_QSPI_SD3_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_QSPI_GPIO_QSPI_SD3_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_QSPI_GPIO_QSPI_SD3_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_QSPI_GPIO_QSPI_SD3_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_QSPI_GPIO_QSPI_SD3_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_QSPI_GPIO_QSPI_SD3_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_QSPI_GPIO_QSPI_SD3_CTRL_FUNCSEL_Msk          = 0x1f
	IO_QSPI_GPIO_QSPI_SD3_CTRL_FUNCSEL_XIP_SD3      = 0x0
	IO_QSPI_GPIO_QSPI_SD3_CTRL_FUNCSEL_UART0_RTS    = 0x2
	IO_QSPI_GPIO_QSPI_SD3_CTRL_FUNCSEL_I2C1_SCL     = 0x3
	IO_QSPI_GPIO_QSPI_SD3_CTRL_FUNCSEL_SIOB_PROC_63 = 0x5
	IO_QSPI_GPIO_QSPI_SD3_CTRL_FUNCSEL_UART0_RX     = 0xb
	IO_QSPI_GPIO_QSPI_SD3_CTRL_FUNCSEL_NULL         = 0x1f

	// IRQSUMMARY_PROC0_SECURE
	// Position of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD3_Pos = 0x7
	// Bit mask of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD3_Msk = 0x80
	// Bit GPIO_QSPI_SD3.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD3 = 0x80
	// Position of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD2_Pos = 0x6
	// Bit mask of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD2_Msk = 0x40
	// Bit GPIO_QSPI_SD2.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD2 = 0x40
	// Position of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD1_Pos = 0x5
	// Bit mask of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD1_Msk = 0x20
	// Bit GPIO_QSPI_SD1.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD1 = 0x20
	// Position of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD0_Pos = 0x4
	// Bit mask of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD0_Msk = 0x10
	// Bit GPIO_QSPI_SD0.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SD0 = 0x10
	// Position of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SS_Pos = 0x3
	// Bit mask of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SS_Msk = 0x8
	// Bit GPIO_QSPI_SS.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SS = 0x8
	// Position of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SCLK_Pos = 0x2
	// Bit mask of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SCLK_Msk = 0x4
	// Bit GPIO_QSPI_SCLK.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_GPIO_QSPI_SCLK = 0x4
	// Position of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_USBPHY_DM_Pos = 0x1
	// Bit mask of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_USBPHY_DM_Msk = 0x2
	// Bit USBPHY_DM.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_USBPHY_DM = 0x2
	// Position of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_USBPHY_DP_Pos = 0x0
	// Bit mask of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_USBPHY_DP_Msk = 0x1
	// Bit USBPHY_DP.
	IO_QSPI_IRQSUMMARY_PROC0_SECURE_USBPHY_DP = 0x1

	// IRQSUMMARY_PROC0_NONSECURE
	// Position of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD3_Pos = 0x7
	// Bit mask of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD3_Msk = 0x80
	// Bit GPIO_QSPI_SD3.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD3 = 0x80
	// Position of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD2_Pos = 0x6
	// Bit mask of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD2_Msk = 0x40
	// Bit GPIO_QSPI_SD2.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD2 = 0x40
	// Position of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD1_Pos = 0x5
	// Bit mask of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD1_Msk = 0x20
	// Bit GPIO_QSPI_SD1.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD1 = 0x20
	// Position of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD0_Pos = 0x4
	// Bit mask of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD0_Msk = 0x10
	// Bit GPIO_QSPI_SD0.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SD0 = 0x10
	// Position of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SS_Pos = 0x3
	// Bit mask of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SS_Msk = 0x8
	// Bit GPIO_QSPI_SS.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SS = 0x8
	// Position of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SCLK_Pos = 0x2
	// Bit mask of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SCLK_Msk = 0x4
	// Bit GPIO_QSPI_SCLK.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_GPIO_QSPI_SCLK = 0x4
	// Position of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_USBPHY_DM_Pos = 0x1
	// Bit mask of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_USBPHY_DM_Msk = 0x2
	// Bit USBPHY_DM.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_USBPHY_DM = 0x2
	// Position of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_USBPHY_DP_Pos = 0x0
	// Bit mask of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_USBPHY_DP_Msk = 0x1
	// Bit USBPHY_DP.
	IO_QSPI_IRQSUMMARY_PROC0_NONSECURE_USBPHY_DP = 0x1

	// IRQSUMMARY_PROC1_SECURE
	// Position of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD3_Pos = 0x7
	// Bit mask of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD3_Msk = 0x80
	// Bit GPIO_QSPI_SD3.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD3 = 0x80
	// Position of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD2_Pos = 0x6
	// Bit mask of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD2_Msk = 0x40
	// Bit GPIO_QSPI_SD2.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD2 = 0x40
	// Position of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD1_Pos = 0x5
	// Bit mask of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD1_Msk = 0x20
	// Bit GPIO_QSPI_SD1.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD1 = 0x20
	// Position of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD0_Pos = 0x4
	// Bit mask of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD0_Msk = 0x10
	// Bit GPIO_QSPI_SD0.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SD0 = 0x10
	// Position of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SS_Pos = 0x3
	// Bit mask of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SS_Msk = 0x8
	// Bit GPIO_QSPI_SS.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SS = 0x8
	// Position of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SCLK_Pos = 0x2
	// Bit mask of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SCLK_Msk = 0x4
	// Bit GPIO_QSPI_SCLK.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_GPIO_QSPI_SCLK = 0x4
	// Position of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_USBPHY_DM_Pos = 0x1
	// Bit mask of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_USBPHY_DM_Msk = 0x2
	// Bit USBPHY_DM.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_USBPHY_DM = 0x2
	// Position of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_USBPHY_DP_Pos = 0x0
	// Bit mask of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_USBPHY_DP_Msk = 0x1
	// Bit USBPHY_DP.
	IO_QSPI_IRQSUMMARY_PROC1_SECURE_USBPHY_DP = 0x1

	// IRQSUMMARY_PROC1_NONSECURE
	// Position of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD3_Pos = 0x7
	// Bit mask of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD3_Msk = 0x80
	// Bit GPIO_QSPI_SD3.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD3 = 0x80
	// Position of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD2_Pos = 0x6
	// Bit mask of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD2_Msk = 0x40
	// Bit GPIO_QSPI_SD2.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD2 = 0x40
	// Position of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD1_Pos = 0x5
	// Bit mask of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD1_Msk = 0x20
	// Bit GPIO_QSPI_SD1.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD1 = 0x20
	// Position of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD0_Pos = 0x4
	// Bit mask of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD0_Msk = 0x10
	// Bit GPIO_QSPI_SD0.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SD0 = 0x10
	// Position of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SS_Pos = 0x3
	// Bit mask of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SS_Msk = 0x8
	// Bit GPIO_QSPI_SS.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SS = 0x8
	// Position of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SCLK_Pos = 0x2
	// Bit mask of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SCLK_Msk = 0x4
	// Bit GPIO_QSPI_SCLK.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_GPIO_QSPI_SCLK = 0x4
	// Position of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_USBPHY_DM_Pos = 0x1
	// Bit mask of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_USBPHY_DM_Msk = 0x2
	// Bit USBPHY_DM.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_USBPHY_DM = 0x2
	// Position of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_USBPHY_DP_Pos = 0x0
	// Bit mask of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_USBPHY_DP_Msk = 0x1
	// Bit USBPHY_DP.
	IO_QSPI_IRQSUMMARY_PROC1_NONSECURE_USBPHY_DP = 0x1

	// IRQSUMMARY_DORMANT_WAKE_SECURE
	// Position of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD3_Pos = 0x7
	// Bit mask of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD3_Msk = 0x80
	// Bit GPIO_QSPI_SD3.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD3 = 0x80
	// Position of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD2_Pos = 0x6
	// Bit mask of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD2_Msk = 0x40
	// Bit GPIO_QSPI_SD2.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD2 = 0x40
	// Position of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD1_Pos = 0x5
	// Bit mask of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD1_Msk = 0x20
	// Bit GPIO_QSPI_SD1.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD1 = 0x20
	// Position of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD0_Pos = 0x4
	// Bit mask of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD0_Msk = 0x10
	// Bit GPIO_QSPI_SD0.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SD0 = 0x10
	// Position of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SS_Pos = 0x3
	// Bit mask of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SS_Msk = 0x8
	// Bit GPIO_QSPI_SS.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SS = 0x8
	// Position of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SCLK_Pos = 0x2
	// Bit mask of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SCLK_Msk = 0x4
	// Bit GPIO_QSPI_SCLK.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_GPIO_QSPI_SCLK = 0x4
	// Position of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DM_Pos = 0x1
	// Bit mask of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DM_Msk = 0x2
	// Bit USBPHY_DM.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DM = 0x2
	// Position of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DP_Pos = 0x0
	// Bit mask of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DP_Msk = 0x1
	// Bit USBPHY_DP.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_SECURE_USBPHY_DP = 0x1

	// IRQSUMMARY_DORMANT_WAKE_NONSECURE
	// Position of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD3_Pos = 0x7
	// Bit mask of GPIO_QSPI_SD3 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD3_Msk = 0x80
	// Bit GPIO_QSPI_SD3.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD3 = 0x80
	// Position of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD2_Pos = 0x6
	// Bit mask of GPIO_QSPI_SD2 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD2_Msk = 0x40
	// Bit GPIO_QSPI_SD2.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD2 = 0x40
	// Position of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD1_Pos = 0x5
	// Bit mask of GPIO_QSPI_SD1 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD1_Msk = 0x20
	// Bit GPIO_QSPI_SD1.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD1 = 0x20
	// Position of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD0_Pos = 0x4
	// Bit mask of GPIO_QSPI_SD0 field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD0_Msk = 0x10
	// Bit GPIO_QSPI_SD0.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SD0 = 0x10
	// Position of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SS_Pos = 0x3
	// Bit mask of GPIO_QSPI_SS field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SS_Msk = 0x8
	// Bit GPIO_QSPI_SS.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SS = 0x8
	// Position of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SCLK_Pos = 0x2
	// Bit mask of GPIO_QSPI_SCLK field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SCLK_Msk = 0x4
	// Bit GPIO_QSPI_SCLK.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_GPIO_QSPI_SCLK = 0x4
	// Position of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DM_Pos = 0x1
	// Bit mask of USBPHY_DM field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DM_Msk = 0x2
	// Bit USBPHY_DM.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DM = 0x2
	// Position of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DP_Pos = 0x0
	// Bit mask of USBPHY_DP field.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DP_Msk = 0x1
	// Bit USBPHY_DP.
	IO_QSPI_IRQSUMMARY_DORMANT_WAKE_NONSECURE_USBPHY_DP = 0x1

	// INTR: Raw Interrupts
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_INTR_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_INTR_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_INTR_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_INTR_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_INTR_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_INTR_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_INTR_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_INTR_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_INTR_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_INTR_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_INTR_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_INTR_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_INTR_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_INTR_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_INTR_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_INTR_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_INTR_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_INTR_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_INTR_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_INTR_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_INTR_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_INTR_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_INTR_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_INTR_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_INTR_USBPHY_DP_LEVEL_LOW = 0x1

	// PROC0_INTE: Interrupt Enable for proc0
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_PROC0_INTE_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_PROC0_INTE_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_PROC0_INTE_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_PROC0_INTE_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_PROC0_INTE_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC0_INTE_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_PROC0_INTE_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC0_INTE_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_PROC0_INTE_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTE_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_PROC0_INTE_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC0_INTE_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_PROC0_INTE_USBPHY_DP_LEVEL_LOW = 0x1

	// PROC0_INTF: Interrupt Force for proc0
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_PROC0_INTF_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_PROC0_INTF_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_PROC0_INTF_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_PROC0_INTF_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_PROC0_INTF_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC0_INTF_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_PROC0_INTF_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC0_INTF_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_PROC0_INTF_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTF_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_PROC0_INTF_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC0_INTF_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_PROC0_INTF_USBPHY_DP_LEVEL_LOW = 0x1

	// PROC0_INTS: Interrupt status after masking & forcing for proc0
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_PROC0_INTS_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_PROC0_INTS_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_PROC0_INTS_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_PROC0_INTS_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_PROC0_INTS_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC0_INTS_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_PROC0_INTS_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC0_INTS_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_PROC0_INTS_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC0_INTS_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_PROC0_INTS_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC0_INTS_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_PROC0_INTS_USBPHY_DP_LEVEL_LOW = 0x1

	// PROC1_INTE: Interrupt Enable for proc1
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_PROC1_INTE_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_PROC1_INTE_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_PROC1_INTE_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_PROC1_INTE_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_PROC1_INTE_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC1_INTE_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_PROC1_INTE_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC1_INTE_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_PROC1_INTE_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTE_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_PROC1_INTE_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC1_INTE_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_PROC1_INTE_USBPHY_DP_LEVEL_LOW = 0x1

	// PROC1_INTF: Interrupt Force for proc1
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_PROC1_INTF_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_PROC1_INTF_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_PROC1_INTF_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_PROC1_INTF_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_PROC1_INTF_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC1_INTF_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_PROC1_INTF_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC1_INTF_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_PROC1_INTF_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTF_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_PROC1_INTF_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC1_INTF_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_PROC1_INTF_USBPHY_DP_LEVEL_LOW = 0x1

	// PROC1_INTS: Interrupt status after masking & forcing for proc1
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_PROC1_INTS_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_PROC1_INTS_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_PROC1_INTS_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_PROC1_INTS_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_PROC1_INTS_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_PROC1_INTS_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_PROC1_INTS_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_PROC1_INTS_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_PROC1_INTS_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_PROC1_INTS_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_PROC1_INTS_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_PROC1_INTS_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_PROC1_INTS_USBPHY_DP_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTE: Interrupt Enable for dormant_wake
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTE_USBPHY_DP_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTF: Interrupt Force for dormant_wake
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTF_USBPHY_DP_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTS: Interrupt status after masking & forcing for dormant_wake
	// Position of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO_QSPI_SD3_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO_QSPI_SD3_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_HIGH = 0x80000000
	// Position of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO_QSPI_SD3_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO_QSPI_SD3_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_EDGE_LOW = 0x40000000
	// Position of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO_QSPI_SD3_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO_QSPI_SD3_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_HIGH = 0x20000000
	// Position of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO_QSPI_SD3_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO_QSPI_SD3_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD3_LEVEL_LOW = 0x10000000
	// Position of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO_QSPI_SD2_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO_QSPI_SD2_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_HIGH = 0x8000000
	// Position of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO_QSPI_SD2_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO_QSPI_SD2_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_EDGE_LOW = 0x4000000
	// Position of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO_QSPI_SD2_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO_QSPI_SD2_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_HIGH = 0x2000000
	// Position of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO_QSPI_SD2_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO_QSPI_SD2_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD2_LEVEL_LOW = 0x1000000
	// Position of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO_QSPI_SD1_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO_QSPI_SD1_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_HIGH = 0x800000
	// Position of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO_QSPI_SD1_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_LOW_Msk = 0x400000
	// Bit GPIO_QSPI_SD1_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_EDGE_LOW = 0x400000
	// Position of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO_QSPI_SD1_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO_QSPI_SD1_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_HIGH = 0x200000
	// Position of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO_QSPI_SD1_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO_QSPI_SD1_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD1_LEVEL_LOW = 0x100000
	// Position of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO_QSPI_SD0_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO_QSPI_SD0_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_HIGH = 0x80000
	// Position of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO_QSPI_SD0_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_LOW_Msk = 0x40000
	// Bit GPIO_QSPI_SD0_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_EDGE_LOW = 0x40000
	// Position of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO_QSPI_SD0_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO_QSPI_SD0_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_HIGH = 0x20000
	// Position of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO_QSPI_SD0_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO_QSPI_SD0_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SD0_LEVEL_LOW = 0x10000
	// Position of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO_QSPI_SS_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO_QSPI_SS_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_HIGH = 0x8000
	// Position of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO_QSPI_SS_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_LOW_Msk = 0x4000
	// Bit GPIO_QSPI_SS_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_EDGE_LOW = 0x4000
	// Position of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO_QSPI_SS_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO_QSPI_SS_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_HIGH = 0x2000
	// Position of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO_QSPI_SS_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO_QSPI_SS_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SS_LEVEL_LOW = 0x1000
	// Position of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO_QSPI_SCLK_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_HIGH_Msk = 0x800
	// Bit GPIO_QSPI_SCLK_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_HIGH = 0x800
	// Position of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO_QSPI_SCLK_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_LOW_Msk = 0x400
	// Bit GPIO_QSPI_SCLK_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_EDGE_LOW = 0x400
	// Position of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO_QSPI_SCLK_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_HIGH = 0x200
	// Position of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO_QSPI_SCLK_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_LOW_Msk = 0x100
	// Bit GPIO_QSPI_SCLK_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_GPIO_QSPI_SCLK_LEVEL_LOW = 0x100
	// Position of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_EDGE_HIGH_Pos = 0x7
	// Bit mask of USBPHY_DM_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_EDGE_HIGH_Msk = 0x80
	// Bit USBPHY_DM_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_EDGE_HIGH = 0x80
	// Position of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_EDGE_LOW_Pos = 0x6
	// Bit mask of USBPHY_DM_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_EDGE_LOW_Msk = 0x40
	// Bit USBPHY_DM_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_EDGE_LOW = 0x40
	// Position of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_LEVEL_HIGH_Pos = 0x5
	// Bit mask of USBPHY_DM_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_LEVEL_HIGH_Msk = 0x20
	// Bit USBPHY_DM_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_LEVEL_HIGH = 0x20
	// Position of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_LEVEL_LOW_Pos = 0x4
	// Bit mask of USBPHY_DM_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_LEVEL_LOW_Msk = 0x10
	// Bit USBPHY_DM_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DM_LEVEL_LOW = 0x10
	// Position of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_EDGE_HIGH_Pos = 0x3
	// Bit mask of USBPHY_DP_EDGE_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_EDGE_HIGH_Msk = 0x8
	// Bit USBPHY_DP_EDGE_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_EDGE_HIGH = 0x8
	// Position of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_EDGE_LOW_Pos = 0x2
	// Bit mask of USBPHY_DP_EDGE_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_EDGE_LOW_Msk = 0x4
	// Bit USBPHY_DP_EDGE_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_EDGE_LOW = 0x4
	// Position of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_LEVEL_HIGH_Pos = 0x1
	// Bit mask of USBPHY_DP_LEVEL_HIGH field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_LEVEL_HIGH_Msk = 0x2
	// Bit USBPHY_DP_LEVEL_HIGH.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_LEVEL_HIGH = 0x2
	// Position of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_LEVEL_LOW_Pos = 0x0
	// Bit mask of USBPHY_DP_LEVEL_LOW field.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_LEVEL_LOW_Msk = 0x1
	// Bit USBPHY_DP_LEVEL_LOW.
	IO_QSPI_DORMANT_WAKE_INTS_USBPHY_DP_LEVEL_LOW = 0x1
)

// Constants for IO_BANK0
const (
	// GPIO0_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO0_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO0_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO0_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO0_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO0_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO0_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO0_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO0_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO0_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO0_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO0_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO0_STATUS_OUTTOPAD = 0x200

	// GPIO0_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO0_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO0_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO0_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO0_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO0_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO0_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO0_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO0_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO0_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO0_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO0_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO0_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO0_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO0_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO0_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO0_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO0_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO0_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO0_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO0_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO0_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO0_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO0_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO0_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO0_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO0_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO0_CTRL_FUNCSEL_JTAG_TCK                   = 0x0
	IO_BANK0_GPIO0_CTRL_FUNCSEL_SPI0_RX                    = 0x1
	IO_BANK0_GPIO0_CTRL_FUNCSEL_UART0_TX                   = 0x2
	IO_BANK0_GPIO0_CTRL_FUNCSEL_I2C0_SDA                   = 0x3
	IO_BANK0_GPIO0_CTRL_FUNCSEL_PWM_A_0                    = 0x4
	IO_BANK0_GPIO0_CTRL_FUNCSEL_SIOB_PROC_0                = 0x5
	IO_BANK0_GPIO0_CTRL_FUNCSEL_PIO0_0                     = 0x6
	IO_BANK0_GPIO0_CTRL_FUNCSEL_PIO1_0                     = 0x7
	IO_BANK0_GPIO0_CTRL_FUNCSEL_PIO2_0                     = 0x8
	IO_BANK0_GPIO0_CTRL_FUNCSEL_XIP_SS_N_1                 = 0x9
	IO_BANK0_GPIO0_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO0_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO1_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO1_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO1_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO1_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO1_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO1_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO1_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO1_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO1_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO1_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO1_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO1_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO1_STATUS_OUTTOPAD = 0x200

	// GPIO1_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO1_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO1_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO1_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO1_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO1_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO1_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO1_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO1_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO1_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO1_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO1_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO1_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO1_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO1_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO1_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO1_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO1_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO1_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO1_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO1_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO1_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO1_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO1_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO1_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO1_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO1_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO1_CTRL_FUNCSEL_JTAG_TMS               = 0x0
	IO_BANK0_GPIO1_CTRL_FUNCSEL_SPI0_SS_N              = 0x1
	IO_BANK0_GPIO1_CTRL_FUNCSEL_UART0_RX               = 0x2
	IO_BANK0_GPIO1_CTRL_FUNCSEL_I2C0_SCL               = 0x3
	IO_BANK0_GPIO1_CTRL_FUNCSEL_PWM_B_0                = 0x4
	IO_BANK0_GPIO1_CTRL_FUNCSEL_SIOB_PROC_1            = 0x5
	IO_BANK0_GPIO1_CTRL_FUNCSEL_PIO0_1                 = 0x6
	IO_BANK0_GPIO1_CTRL_FUNCSEL_PIO1_1                 = 0x7
	IO_BANK0_GPIO1_CTRL_FUNCSEL_PIO2_1                 = 0x8
	IO_BANK0_GPIO1_CTRL_FUNCSEL_CORESIGHT_TRACECLK     = 0x9
	IO_BANK0_GPIO1_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO1_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO2_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO2_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO2_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO2_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO2_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO2_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO2_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO2_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO2_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO2_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO2_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO2_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO2_STATUS_OUTTOPAD = 0x200

	// GPIO2_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO2_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO2_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO2_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO2_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO2_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO2_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO2_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO2_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO2_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO2_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO2_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO2_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO2_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO2_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO2_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO2_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO2_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO2_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO2_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO2_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO2_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO2_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO2_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO2_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO2_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO2_CTRL_FUNCSEL_Msk                   = 0x1f
	IO_BANK0_GPIO2_CTRL_FUNCSEL_JTAG_TDI              = 0x0
	IO_BANK0_GPIO2_CTRL_FUNCSEL_SPI0_SCLK             = 0x1
	IO_BANK0_GPIO2_CTRL_FUNCSEL_UART0_CTS             = 0x2
	IO_BANK0_GPIO2_CTRL_FUNCSEL_I2C1_SDA              = 0x3
	IO_BANK0_GPIO2_CTRL_FUNCSEL_PWM_A_1               = 0x4
	IO_BANK0_GPIO2_CTRL_FUNCSEL_SIOB_PROC_2           = 0x5
	IO_BANK0_GPIO2_CTRL_FUNCSEL_PIO0_2                = 0x6
	IO_BANK0_GPIO2_CTRL_FUNCSEL_PIO1_2                = 0x7
	IO_BANK0_GPIO2_CTRL_FUNCSEL_PIO2_2                = 0x8
	IO_BANK0_GPIO2_CTRL_FUNCSEL_CORESIGHT_TRACEDATA_0 = 0x9
	IO_BANK0_GPIO2_CTRL_FUNCSEL_USB_MUXING_VBUS_EN    = 0xa
	IO_BANK0_GPIO2_CTRL_FUNCSEL_UART0_TX              = 0xb
	IO_BANK0_GPIO2_CTRL_FUNCSEL_NULL                  = 0x1f

	// GPIO3_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO3_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO3_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO3_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO3_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO3_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO3_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO3_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO3_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO3_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO3_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO3_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO3_STATUS_OUTTOPAD = 0x200

	// GPIO3_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO3_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO3_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO3_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO3_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO3_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO3_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO3_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO3_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO3_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO3_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO3_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO3_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO3_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO3_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO3_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO3_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO3_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO3_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO3_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO3_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO3_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO3_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO3_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO3_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO3_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO3_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO3_CTRL_FUNCSEL_JTAG_TDO                   = 0x0
	IO_BANK0_GPIO3_CTRL_FUNCSEL_SPI0_TX                    = 0x1
	IO_BANK0_GPIO3_CTRL_FUNCSEL_UART0_RTS                  = 0x2
	IO_BANK0_GPIO3_CTRL_FUNCSEL_I2C1_SCL                   = 0x3
	IO_BANK0_GPIO3_CTRL_FUNCSEL_PWM_B_1                    = 0x4
	IO_BANK0_GPIO3_CTRL_FUNCSEL_SIOB_PROC_3                = 0x5
	IO_BANK0_GPIO3_CTRL_FUNCSEL_PIO0_3                     = 0x6
	IO_BANK0_GPIO3_CTRL_FUNCSEL_PIO1_3                     = 0x7
	IO_BANK0_GPIO3_CTRL_FUNCSEL_PIO2_3                     = 0x8
	IO_BANK0_GPIO3_CTRL_FUNCSEL_CORESIGHT_TRACEDATA_1      = 0x9
	IO_BANK0_GPIO3_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO3_CTRL_FUNCSEL_UART0_RX                   = 0xb
	IO_BANK0_GPIO3_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO4_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO4_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO4_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO4_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO4_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO4_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO4_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO4_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO4_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO4_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO4_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO4_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO4_STATUS_OUTTOPAD = 0x200

	// GPIO4_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO4_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO4_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO4_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO4_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO4_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO4_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO4_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO4_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO4_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO4_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO4_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO4_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO4_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO4_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO4_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO4_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO4_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO4_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO4_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO4_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO4_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO4_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO4_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO4_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO4_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO4_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO4_CTRL_FUNCSEL_SPI0_RX                = 0x1
	IO_BANK0_GPIO4_CTRL_FUNCSEL_UART1_TX               = 0x2
	IO_BANK0_GPIO4_CTRL_FUNCSEL_I2C0_SDA               = 0x3
	IO_BANK0_GPIO4_CTRL_FUNCSEL_PWM_A_2                = 0x4
	IO_BANK0_GPIO4_CTRL_FUNCSEL_SIOB_PROC_4            = 0x5
	IO_BANK0_GPIO4_CTRL_FUNCSEL_PIO0_4                 = 0x6
	IO_BANK0_GPIO4_CTRL_FUNCSEL_PIO1_4                 = 0x7
	IO_BANK0_GPIO4_CTRL_FUNCSEL_PIO2_4                 = 0x8
	IO_BANK0_GPIO4_CTRL_FUNCSEL_CORESIGHT_TRACEDATA_2  = 0x9
	IO_BANK0_GPIO4_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO4_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO5_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO5_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO5_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO5_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO5_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO5_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO5_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO5_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO5_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO5_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO5_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO5_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO5_STATUS_OUTTOPAD = 0x200

	// GPIO5_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO5_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO5_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO5_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO5_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO5_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO5_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO5_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO5_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO5_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO5_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO5_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO5_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO5_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO5_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO5_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO5_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO5_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO5_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO5_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO5_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO5_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO5_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO5_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO5_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO5_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO5_CTRL_FUNCSEL_Msk                   = 0x1f
	IO_BANK0_GPIO5_CTRL_FUNCSEL_SPI0_SS_N             = 0x1
	IO_BANK0_GPIO5_CTRL_FUNCSEL_UART1_RX              = 0x2
	IO_BANK0_GPIO5_CTRL_FUNCSEL_I2C0_SCL              = 0x3
	IO_BANK0_GPIO5_CTRL_FUNCSEL_PWM_B_2               = 0x4
	IO_BANK0_GPIO5_CTRL_FUNCSEL_SIOB_PROC_5           = 0x5
	IO_BANK0_GPIO5_CTRL_FUNCSEL_PIO0_5                = 0x6
	IO_BANK0_GPIO5_CTRL_FUNCSEL_PIO1_5                = 0x7
	IO_BANK0_GPIO5_CTRL_FUNCSEL_PIO2_5                = 0x8
	IO_BANK0_GPIO5_CTRL_FUNCSEL_CORESIGHT_TRACEDATA_3 = 0x9
	IO_BANK0_GPIO5_CTRL_FUNCSEL_USB_MUXING_VBUS_EN    = 0xa
	IO_BANK0_GPIO5_CTRL_FUNCSEL_NULL                  = 0x1f

	// GPIO6_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO6_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO6_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO6_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO6_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO6_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO6_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO6_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO6_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO6_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO6_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO6_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO6_STATUS_OUTTOPAD = 0x200

	// GPIO6_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO6_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO6_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO6_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO6_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO6_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO6_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO6_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO6_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO6_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO6_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO6_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO6_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO6_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO6_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO6_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO6_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO6_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO6_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO6_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO6_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO6_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO6_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO6_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO6_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO6_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO6_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO6_CTRL_FUNCSEL_SPI0_SCLK                  = 0x1
	IO_BANK0_GPIO6_CTRL_FUNCSEL_UART1_CTS                  = 0x2
	IO_BANK0_GPIO6_CTRL_FUNCSEL_I2C1_SDA                   = 0x3
	IO_BANK0_GPIO6_CTRL_FUNCSEL_PWM_A_3                    = 0x4
	IO_BANK0_GPIO6_CTRL_FUNCSEL_SIOB_PROC_6                = 0x5
	IO_BANK0_GPIO6_CTRL_FUNCSEL_PIO0_6                     = 0x6
	IO_BANK0_GPIO6_CTRL_FUNCSEL_PIO1_6                     = 0x7
	IO_BANK0_GPIO6_CTRL_FUNCSEL_PIO2_6                     = 0x8
	IO_BANK0_GPIO6_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO6_CTRL_FUNCSEL_UART1_TX                   = 0xb
	IO_BANK0_GPIO6_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO7_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO7_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO7_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO7_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO7_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO7_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO7_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO7_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO7_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO7_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO7_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO7_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO7_STATUS_OUTTOPAD = 0x200

	// GPIO7_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO7_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO7_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO7_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO7_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO7_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO7_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO7_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO7_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO7_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO7_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO7_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO7_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO7_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO7_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO7_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO7_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO7_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO7_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO7_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO7_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO7_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO7_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO7_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO7_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO7_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO7_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO7_CTRL_FUNCSEL_SPI0_TX                = 0x1
	IO_BANK0_GPIO7_CTRL_FUNCSEL_UART1_RTS              = 0x2
	IO_BANK0_GPIO7_CTRL_FUNCSEL_I2C1_SCL               = 0x3
	IO_BANK0_GPIO7_CTRL_FUNCSEL_PWM_B_3                = 0x4
	IO_BANK0_GPIO7_CTRL_FUNCSEL_SIOB_PROC_7            = 0x5
	IO_BANK0_GPIO7_CTRL_FUNCSEL_PIO0_7                 = 0x6
	IO_BANK0_GPIO7_CTRL_FUNCSEL_PIO1_7                 = 0x7
	IO_BANK0_GPIO7_CTRL_FUNCSEL_PIO2_7                 = 0x8
	IO_BANK0_GPIO7_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO7_CTRL_FUNCSEL_UART1_RX               = 0xb
	IO_BANK0_GPIO7_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO8_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO8_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO8_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO8_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO8_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO8_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO8_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO8_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO8_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO8_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO8_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO8_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO8_STATUS_OUTTOPAD = 0x200

	// GPIO8_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO8_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO8_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO8_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO8_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO8_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO8_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO8_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO8_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO8_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO8_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO8_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO8_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO8_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO8_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO8_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO8_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO8_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO8_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO8_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO8_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO8_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO8_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO8_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO8_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO8_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO8_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO8_CTRL_FUNCSEL_SPI1_RX            = 0x1
	IO_BANK0_GPIO8_CTRL_FUNCSEL_UART1_TX           = 0x2
	IO_BANK0_GPIO8_CTRL_FUNCSEL_I2C0_SDA           = 0x3
	IO_BANK0_GPIO8_CTRL_FUNCSEL_PWM_A_4            = 0x4
	IO_BANK0_GPIO8_CTRL_FUNCSEL_SIOB_PROC_8        = 0x5
	IO_BANK0_GPIO8_CTRL_FUNCSEL_PIO0_8             = 0x6
	IO_BANK0_GPIO8_CTRL_FUNCSEL_PIO1_8             = 0x7
	IO_BANK0_GPIO8_CTRL_FUNCSEL_PIO2_8             = 0x8
	IO_BANK0_GPIO8_CTRL_FUNCSEL_XIP_SS_N_1         = 0x9
	IO_BANK0_GPIO8_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO8_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO9_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO9_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO9_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO9_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO9_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO9_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO9_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO9_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO9_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO9_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO9_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO9_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO9_STATUS_OUTTOPAD = 0x200

	// GPIO9_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO9_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO9_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO9_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO9_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO9_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO9_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO9_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO9_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO9_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO9_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO9_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO9_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO9_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO9_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO9_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO9_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO9_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO9_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO9_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO9_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO9_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO9_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO9_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO9_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO9_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO9_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO9_CTRL_FUNCSEL_SPI1_SS_N                  = 0x1
	IO_BANK0_GPIO9_CTRL_FUNCSEL_UART1_RX                   = 0x2
	IO_BANK0_GPIO9_CTRL_FUNCSEL_I2C0_SCL                   = 0x3
	IO_BANK0_GPIO9_CTRL_FUNCSEL_PWM_B_4                    = 0x4
	IO_BANK0_GPIO9_CTRL_FUNCSEL_SIOB_PROC_9                = 0x5
	IO_BANK0_GPIO9_CTRL_FUNCSEL_PIO0_9                     = 0x6
	IO_BANK0_GPIO9_CTRL_FUNCSEL_PIO1_9                     = 0x7
	IO_BANK0_GPIO9_CTRL_FUNCSEL_PIO2_9                     = 0x8
	IO_BANK0_GPIO9_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO9_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO10_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO10_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO10_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO10_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO10_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO10_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO10_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO10_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO10_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO10_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO10_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO10_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO10_STATUS_OUTTOPAD = 0x200

	// GPIO10_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO10_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO10_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO10_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO10_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO10_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO10_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO10_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO10_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO10_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO10_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO10_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO10_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO10_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO10_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO10_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO10_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO10_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO10_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO10_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO10_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO10_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO10_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO10_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO10_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO10_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO10_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO10_CTRL_FUNCSEL_SPI1_SCLK              = 0x1
	IO_BANK0_GPIO10_CTRL_FUNCSEL_UART1_CTS              = 0x2
	IO_BANK0_GPIO10_CTRL_FUNCSEL_I2C1_SDA               = 0x3
	IO_BANK0_GPIO10_CTRL_FUNCSEL_PWM_A_5                = 0x4
	IO_BANK0_GPIO10_CTRL_FUNCSEL_SIOB_PROC_10           = 0x5
	IO_BANK0_GPIO10_CTRL_FUNCSEL_PIO0_10                = 0x6
	IO_BANK0_GPIO10_CTRL_FUNCSEL_PIO1_10                = 0x7
	IO_BANK0_GPIO10_CTRL_FUNCSEL_PIO2_10                = 0x8
	IO_BANK0_GPIO10_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO10_CTRL_FUNCSEL_UART1_TX               = 0xb
	IO_BANK0_GPIO10_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO11_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO11_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO11_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO11_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO11_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO11_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO11_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO11_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO11_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO11_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO11_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO11_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO11_STATUS_OUTTOPAD = 0x200

	// GPIO11_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO11_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO11_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO11_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO11_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO11_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO11_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO11_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO11_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO11_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO11_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO11_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO11_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO11_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO11_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO11_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO11_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO11_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO11_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO11_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO11_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO11_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO11_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO11_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO11_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO11_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO11_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO11_CTRL_FUNCSEL_SPI1_TX            = 0x1
	IO_BANK0_GPIO11_CTRL_FUNCSEL_UART1_RTS          = 0x2
	IO_BANK0_GPIO11_CTRL_FUNCSEL_I2C1_SCL           = 0x3
	IO_BANK0_GPIO11_CTRL_FUNCSEL_PWM_B_5            = 0x4
	IO_BANK0_GPIO11_CTRL_FUNCSEL_SIOB_PROC_11       = 0x5
	IO_BANK0_GPIO11_CTRL_FUNCSEL_PIO0_11            = 0x6
	IO_BANK0_GPIO11_CTRL_FUNCSEL_PIO1_11            = 0x7
	IO_BANK0_GPIO11_CTRL_FUNCSEL_PIO2_11            = 0x8
	IO_BANK0_GPIO11_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO11_CTRL_FUNCSEL_UART1_RX           = 0xb
	IO_BANK0_GPIO11_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO12_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO12_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO12_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO12_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO12_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO12_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO12_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO12_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO12_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO12_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO12_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO12_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO12_STATUS_OUTTOPAD = 0x200

	// GPIO12_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO12_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO12_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO12_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO12_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO12_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO12_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO12_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO12_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO12_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO12_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO12_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO12_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO12_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO12_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO12_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO12_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO12_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO12_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO12_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO12_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO12_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO12_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO12_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO12_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO12_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO12_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO12_CTRL_FUNCSEL_HSTX_0                     = 0x0
	IO_BANK0_GPIO12_CTRL_FUNCSEL_SPI1_RX                    = 0x1
	IO_BANK0_GPIO12_CTRL_FUNCSEL_UART0_TX                   = 0x2
	IO_BANK0_GPIO12_CTRL_FUNCSEL_I2C0_SDA                   = 0x3
	IO_BANK0_GPIO12_CTRL_FUNCSEL_PWM_A_6                    = 0x4
	IO_BANK0_GPIO12_CTRL_FUNCSEL_SIOB_PROC_12               = 0x5
	IO_BANK0_GPIO12_CTRL_FUNCSEL_PIO0_12                    = 0x6
	IO_BANK0_GPIO12_CTRL_FUNCSEL_PIO1_12                    = 0x7
	IO_BANK0_GPIO12_CTRL_FUNCSEL_PIO2_12                    = 0x8
	IO_BANK0_GPIO12_CTRL_FUNCSEL_CLOCKS_GPIN_0              = 0x9
	IO_BANK0_GPIO12_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO12_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO13_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO13_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO13_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO13_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO13_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO13_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO13_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO13_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO13_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO13_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO13_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO13_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO13_STATUS_OUTTOPAD = 0x200

	// GPIO13_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO13_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO13_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO13_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO13_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO13_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO13_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO13_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO13_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO13_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO13_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO13_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO13_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO13_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO13_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO13_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO13_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO13_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO13_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO13_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO13_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO13_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO13_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO13_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO13_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO13_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO13_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO13_CTRL_FUNCSEL_HSTX_1                 = 0x0
	IO_BANK0_GPIO13_CTRL_FUNCSEL_SPI1_SS_N              = 0x1
	IO_BANK0_GPIO13_CTRL_FUNCSEL_UART0_RX               = 0x2
	IO_BANK0_GPIO13_CTRL_FUNCSEL_I2C0_SCL               = 0x3
	IO_BANK0_GPIO13_CTRL_FUNCSEL_PWM_B_6                = 0x4
	IO_BANK0_GPIO13_CTRL_FUNCSEL_SIOB_PROC_13           = 0x5
	IO_BANK0_GPIO13_CTRL_FUNCSEL_PIO0_13                = 0x6
	IO_BANK0_GPIO13_CTRL_FUNCSEL_PIO1_13                = 0x7
	IO_BANK0_GPIO13_CTRL_FUNCSEL_PIO2_13                = 0x8
	IO_BANK0_GPIO13_CTRL_FUNCSEL_CLOCKS_GPOUT_0         = 0x9
	IO_BANK0_GPIO13_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO13_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO14_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO14_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO14_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO14_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO14_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO14_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO14_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO14_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO14_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO14_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO14_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO14_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO14_STATUS_OUTTOPAD = 0x200

	// GPIO14_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO14_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO14_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO14_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO14_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO14_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO14_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO14_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO14_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO14_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO14_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO14_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO14_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO14_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO14_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO14_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO14_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO14_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO14_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO14_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO14_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO14_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO14_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO14_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO14_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO14_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO14_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO14_CTRL_FUNCSEL_HSTX_2             = 0x0
	IO_BANK0_GPIO14_CTRL_FUNCSEL_SPI1_SCLK          = 0x1
	IO_BANK0_GPIO14_CTRL_FUNCSEL_UART0_CTS          = 0x2
	IO_BANK0_GPIO14_CTRL_FUNCSEL_I2C1_SDA           = 0x3
	IO_BANK0_GPIO14_CTRL_FUNCSEL_PWM_A_7            = 0x4
	IO_BANK0_GPIO14_CTRL_FUNCSEL_SIOB_PROC_14       = 0x5
	IO_BANK0_GPIO14_CTRL_FUNCSEL_PIO0_14            = 0x6
	IO_BANK0_GPIO14_CTRL_FUNCSEL_PIO1_14            = 0x7
	IO_BANK0_GPIO14_CTRL_FUNCSEL_PIO2_14            = 0x8
	IO_BANK0_GPIO14_CTRL_FUNCSEL_CLOCKS_GPIN_1      = 0x9
	IO_BANK0_GPIO14_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO14_CTRL_FUNCSEL_UART0_TX           = 0xb
	IO_BANK0_GPIO14_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO15_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO15_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO15_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO15_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO15_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO15_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO15_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO15_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO15_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO15_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO15_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO15_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO15_STATUS_OUTTOPAD = 0x200

	// GPIO15_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO15_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO15_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO15_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO15_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO15_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO15_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO15_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO15_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO15_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO15_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO15_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO15_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO15_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO15_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO15_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO15_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO15_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO15_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO15_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO15_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO15_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO15_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO15_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO15_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO15_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO15_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO15_CTRL_FUNCSEL_HSTX_3                     = 0x0
	IO_BANK0_GPIO15_CTRL_FUNCSEL_SPI1_TX                    = 0x1
	IO_BANK0_GPIO15_CTRL_FUNCSEL_UART0_RTS                  = 0x2
	IO_BANK0_GPIO15_CTRL_FUNCSEL_I2C1_SCL                   = 0x3
	IO_BANK0_GPIO15_CTRL_FUNCSEL_PWM_B_7                    = 0x4
	IO_BANK0_GPIO15_CTRL_FUNCSEL_SIOB_PROC_15               = 0x5
	IO_BANK0_GPIO15_CTRL_FUNCSEL_PIO0_15                    = 0x6
	IO_BANK0_GPIO15_CTRL_FUNCSEL_PIO1_15                    = 0x7
	IO_BANK0_GPIO15_CTRL_FUNCSEL_PIO2_15                    = 0x8
	IO_BANK0_GPIO15_CTRL_FUNCSEL_CLOCKS_GPOUT_1             = 0x9
	IO_BANK0_GPIO15_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO15_CTRL_FUNCSEL_UART0_RX                   = 0xb
	IO_BANK0_GPIO15_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO16_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO16_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO16_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO16_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO16_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO16_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO16_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO16_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO16_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO16_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO16_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO16_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO16_STATUS_OUTTOPAD = 0x200

	// GPIO16_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO16_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO16_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO16_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO16_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO16_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO16_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO16_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO16_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO16_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO16_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO16_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO16_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO16_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO16_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO16_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO16_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO16_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO16_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO16_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO16_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO16_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO16_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO16_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO16_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO16_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO16_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO16_CTRL_FUNCSEL_HSTX_4                 = 0x0
	IO_BANK0_GPIO16_CTRL_FUNCSEL_SPI0_RX                = 0x1
	IO_BANK0_GPIO16_CTRL_FUNCSEL_UART0_TX               = 0x2
	IO_BANK0_GPIO16_CTRL_FUNCSEL_I2C0_SDA               = 0x3
	IO_BANK0_GPIO16_CTRL_FUNCSEL_PWM_A_0                = 0x4
	IO_BANK0_GPIO16_CTRL_FUNCSEL_SIOB_PROC_16           = 0x5
	IO_BANK0_GPIO16_CTRL_FUNCSEL_PIO0_16                = 0x6
	IO_BANK0_GPIO16_CTRL_FUNCSEL_PIO1_16                = 0x7
	IO_BANK0_GPIO16_CTRL_FUNCSEL_PIO2_16                = 0x8
	IO_BANK0_GPIO16_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO16_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO17_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO17_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO17_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO17_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO17_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO17_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO17_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO17_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO17_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO17_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO17_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO17_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO17_STATUS_OUTTOPAD = 0x200

	// GPIO17_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO17_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO17_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO17_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO17_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO17_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO17_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO17_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO17_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO17_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO17_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO17_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO17_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO17_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO17_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO17_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO17_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO17_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO17_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO17_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO17_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO17_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO17_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO17_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO17_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO17_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO17_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO17_CTRL_FUNCSEL_HSTX_5             = 0x0
	IO_BANK0_GPIO17_CTRL_FUNCSEL_SPI0_SS_N          = 0x1
	IO_BANK0_GPIO17_CTRL_FUNCSEL_UART0_RX           = 0x2
	IO_BANK0_GPIO17_CTRL_FUNCSEL_I2C0_SCL           = 0x3
	IO_BANK0_GPIO17_CTRL_FUNCSEL_PWM_B_0            = 0x4
	IO_BANK0_GPIO17_CTRL_FUNCSEL_SIOB_PROC_17       = 0x5
	IO_BANK0_GPIO17_CTRL_FUNCSEL_PIO0_17            = 0x6
	IO_BANK0_GPIO17_CTRL_FUNCSEL_PIO1_17            = 0x7
	IO_BANK0_GPIO17_CTRL_FUNCSEL_PIO2_17            = 0x8
	IO_BANK0_GPIO17_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO17_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO18_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO18_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO18_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO18_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO18_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO18_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO18_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO18_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO18_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO18_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO18_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO18_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO18_STATUS_OUTTOPAD = 0x200

	// GPIO18_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO18_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO18_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO18_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO18_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO18_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO18_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO18_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO18_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO18_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO18_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO18_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO18_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO18_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO18_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO18_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO18_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO18_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO18_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO18_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO18_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO18_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO18_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO18_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO18_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO18_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO18_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO18_CTRL_FUNCSEL_HSTX_6                     = 0x0
	IO_BANK0_GPIO18_CTRL_FUNCSEL_SPI0_SCLK                  = 0x1
	IO_BANK0_GPIO18_CTRL_FUNCSEL_UART0_CTS                  = 0x2
	IO_BANK0_GPIO18_CTRL_FUNCSEL_I2C1_SDA                   = 0x3
	IO_BANK0_GPIO18_CTRL_FUNCSEL_PWM_A_1                    = 0x4
	IO_BANK0_GPIO18_CTRL_FUNCSEL_SIOB_PROC_18               = 0x5
	IO_BANK0_GPIO18_CTRL_FUNCSEL_PIO0_18                    = 0x6
	IO_BANK0_GPIO18_CTRL_FUNCSEL_PIO1_18                    = 0x7
	IO_BANK0_GPIO18_CTRL_FUNCSEL_PIO2_18                    = 0x8
	IO_BANK0_GPIO18_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO18_CTRL_FUNCSEL_UART0_TX                   = 0xb
	IO_BANK0_GPIO18_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO19_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO19_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO19_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO19_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO19_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO19_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO19_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO19_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO19_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO19_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO19_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO19_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO19_STATUS_OUTTOPAD = 0x200

	// GPIO19_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO19_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO19_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO19_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO19_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO19_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO19_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO19_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO19_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO19_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO19_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO19_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO19_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO19_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO19_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO19_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO19_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO19_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO19_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO19_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO19_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO19_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO19_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO19_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO19_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO19_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO19_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO19_CTRL_FUNCSEL_HSTX_7                 = 0x0
	IO_BANK0_GPIO19_CTRL_FUNCSEL_SPI0_TX                = 0x1
	IO_BANK0_GPIO19_CTRL_FUNCSEL_UART0_RTS              = 0x2
	IO_BANK0_GPIO19_CTRL_FUNCSEL_I2C1_SCL               = 0x3
	IO_BANK0_GPIO19_CTRL_FUNCSEL_PWM_B_1                = 0x4
	IO_BANK0_GPIO19_CTRL_FUNCSEL_SIOB_PROC_19           = 0x5
	IO_BANK0_GPIO19_CTRL_FUNCSEL_PIO0_19                = 0x6
	IO_BANK0_GPIO19_CTRL_FUNCSEL_PIO1_19                = 0x7
	IO_BANK0_GPIO19_CTRL_FUNCSEL_PIO2_19                = 0x8
	IO_BANK0_GPIO19_CTRL_FUNCSEL_XIP_SS_N_1             = 0x9
	IO_BANK0_GPIO19_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO19_CTRL_FUNCSEL_UART0_RX               = 0xb
	IO_BANK0_GPIO19_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO20_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO20_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO20_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO20_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO20_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO20_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO20_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO20_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO20_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO20_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO20_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO20_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO20_STATUS_OUTTOPAD = 0x200

	// GPIO20_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO20_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO20_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO20_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO20_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO20_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO20_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO20_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO20_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO20_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO20_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO20_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO20_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO20_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO20_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO20_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO20_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO20_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO20_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO20_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO20_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO20_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO20_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO20_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO20_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO20_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO20_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO20_CTRL_FUNCSEL_SPI0_RX            = 0x1
	IO_BANK0_GPIO20_CTRL_FUNCSEL_UART1_TX           = 0x2
	IO_BANK0_GPIO20_CTRL_FUNCSEL_I2C0_SDA           = 0x3
	IO_BANK0_GPIO20_CTRL_FUNCSEL_PWM_A_2            = 0x4
	IO_BANK0_GPIO20_CTRL_FUNCSEL_SIOB_PROC_20       = 0x5
	IO_BANK0_GPIO20_CTRL_FUNCSEL_PIO0_20            = 0x6
	IO_BANK0_GPIO20_CTRL_FUNCSEL_PIO1_20            = 0x7
	IO_BANK0_GPIO20_CTRL_FUNCSEL_PIO2_20            = 0x8
	IO_BANK0_GPIO20_CTRL_FUNCSEL_CLOCKS_GPIN_0      = 0x9
	IO_BANK0_GPIO20_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO20_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO21_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO21_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO21_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO21_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO21_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO21_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO21_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO21_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO21_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO21_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO21_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO21_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO21_STATUS_OUTTOPAD = 0x200

	// GPIO21_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO21_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO21_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO21_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO21_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO21_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO21_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO21_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO21_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO21_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO21_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO21_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO21_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO21_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO21_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO21_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO21_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO21_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO21_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO21_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO21_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO21_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO21_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO21_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO21_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO21_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO21_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO21_CTRL_FUNCSEL_SPI0_SS_N                  = 0x1
	IO_BANK0_GPIO21_CTRL_FUNCSEL_UART1_RX                   = 0x2
	IO_BANK0_GPIO21_CTRL_FUNCSEL_I2C0_SCL                   = 0x3
	IO_BANK0_GPIO21_CTRL_FUNCSEL_PWM_B_2                    = 0x4
	IO_BANK0_GPIO21_CTRL_FUNCSEL_SIOB_PROC_21               = 0x5
	IO_BANK0_GPIO21_CTRL_FUNCSEL_PIO0_21                    = 0x6
	IO_BANK0_GPIO21_CTRL_FUNCSEL_PIO1_21                    = 0x7
	IO_BANK0_GPIO21_CTRL_FUNCSEL_PIO2_21                    = 0x8
	IO_BANK0_GPIO21_CTRL_FUNCSEL_CLOCKS_GPOUT_0             = 0x9
	IO_BANK0_GPIO21_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO21_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO22_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO22_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO22_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO22_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO22_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO22_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO22_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO22_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO22_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO22_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO22_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO22_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO22_STATUS_OUTTOPAD = 0x200

	// GPIO22_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO22_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO22_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO22_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO22_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO22_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO22_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO22_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO22_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO22_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO22_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO22_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO22_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO22_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO22_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO22_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO22_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO22_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO22_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO22_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO22_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO22_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO22_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO22_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO22_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO22_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO22_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO22_CTRL_FUNCSEL_SPI0_SCLK              = 0x1
	IO_BANK0_GPIO22_CTRL_FUNCSEL_UART1_CTS              = 0x2
	IO_BANK0_GPIO22_CTRL_FUNCSEL_I2C1_SDA               = 0x3
	IO_BANK0_GPIO22_CTRL_FUNCSEL_PWM_A_3                = 0x4
	IO_BANK0_GPIO22_CTRL_FUNCSEL_SIOB_PROC_22           = 0x5
	IO_BANK0_GPIO22_CTRL_FUNCSEL_PIO0_22                = 0x6
	IO_BANK0_GPIO22_CTRL_FUNCSEL_PIO1_22                = 0x7
	IO_BANK0_GPIO22_CTRL_FUNCSEL_PIO2_22                = 0x8
	IO_BANK0_GPIO22_CTRL_FUNCSEL_CLOCKS_GPIN_1          = 0x9
	IO_BANK0_GPIO22_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO22_CTRL_FUNCSEL_UART1_TX               = 0xb
	IO_BANK0_GPIO22_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO23_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO23_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO23_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO23_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO23_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO23_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO23_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO23_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO23_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO23_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO23_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO23_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO23_STATUS_OUTTOPAD = 0x200

	// GPIO23_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO23_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO23_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO23_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO23_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO23_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO23_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO23_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO23_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO23_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO23_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO23_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO23_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO23_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO23_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO23_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO23_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO23_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO23_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO23_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO23_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO23_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO23_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO23_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO23_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO23_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO23_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO23_CTRL_FUNCSEL_SPI0_TX            = 0x1
	IO_BANK0_GPIO23_CTRL_FUNCSEL_UART1_RTS          = 0x2
	IO_BANK0_GPIO23_CTRL_FUNCSEL_I2C1_SCL           = 0x3
	IO_BANK0_GPIO23_CTRL_FUNCSEL_PWM_B_3            = 0x4
	IO_BANK0_GPIO23_CTRL_FUNCSEL_SIOB_PROC_23       = 0x5
	IO_BANK0_GPIO23_CTRL_FUNCSEL_PIO0_23            = 0x6
	IO_BANK0_GPIO23_CTRL_FUNCSEL_PIO1_23            = 0x7
	IO_BANK0_GPIO23_CTRL_FUNCSEL_PIO2_23            = 0x8
	IO_BANK0_GPIO23_CTRL_FUNCSEL_CLOCKS_GPOUT_1     = 0x9
	IO_BANK0_GPIO23_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO23_CTRL_FUNCSEL_UART1_RX           = 0xb
	IO_BANK0_GPIO23_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO24_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO24_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO24_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO24_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO24_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO24_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO24_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO24_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO24_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO24_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO24_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO24_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO24_STATUS_OUTTOPAD = 0x200

	// GPIO24_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO24_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO24_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO24_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO24_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO24_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO24_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO24_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO24_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO24_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO24_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO24_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO24_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO24_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO24_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO24_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO24_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO24_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO24_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO24_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO24_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO24_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO24_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO24_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO24_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO24_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO24_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO24_CTRL_FUNCSEL_SPI1_RX                    = 0x1
	IO_BANK0_GPIO24_CTRL_FUNCSEL_UART1_TX                   = 0x2
	IO_BANK0_GPIO24_CTRL_FUNCSEL_I2C0_SDA                   = 0x3
	IO_BANK0_GPIO24_CTRL_FUNCSEL_PWM_A_4                    = 0x4
	IO_BANK0_GPIO24_CTRL_FUNCSEL_SIOB_PROC_24               = 0x5
	IO_BANK0_GPIO24_CTRL_FUNCSEL_PIO0_24                    = 0x6
	IO_BANK0_GPIO24_CTRL_FUNCSEL_PIO1_24                    = 0x7
	IO_BANK0_GPIO24_CTRL_FUNCSEL_PIO2_24                    = 0x8
	IO_BANK0_GPIO24_CTRL_FUNCSEL_CLOCKS_GPOUT_2             = 0x9
	IO_BANK0_GPIO24_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO24_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO25_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO25_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO25_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO25_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO25_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO25_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO25_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO25_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO25_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO25_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO25_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO25_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO25_STATUS_OUTTOPAD = 0x200

	// GPIO25_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO25_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO25_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO25_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO25_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO25_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO25_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO25_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO25_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO25_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO25_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO25_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO25_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO25_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO25_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO25_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO25_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO25_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO25_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO25_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO25_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO25_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO25_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO25_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO25_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO25_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO25_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO25_CTRL_FUNCSEL_SPI1_SS_N              = 0x1
	IO_BANK0_GPIO25_CTRL_FUNCSEL_UART1_RX               = 0x2
	IO_BANK0_GPIO25_CTRL_FUNCSEL_I2C0_SCL               = 0x3
	IO_BANK0_GPIO25_CTRL_FUNCSEL_PWM_B_4                = 0x4
	IO_BANK0_GPIO25_CTRL_FUNCSEL_SIOB_PROC_25           = 0x5
	IO_BANK0_GPIO25_CTRL_FUNCSEL_PIO0_25                = 0x6
	IO_BANK0_GPIO25_CTRL_FUNCSEL_PIO1_25                = 0x7
	IO_BANK0_GPIO25_CTRL_FUNCSEL_PIO2_25                = 0x8
	IO_BANK0_GPIO25_CTRL_FUNCSEL_CLOCKS_GPOUT_3         = 0x9
	IO_BANK0_GPIO25_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO25_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO26_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO26_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO26_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO26_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO26_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO26_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO26_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO26_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO26_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO26_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO26_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO26_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO26_STATUS_OUTTOPAD = 0x200

	// GPIO26_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO26_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO26_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO26_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO26_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO26_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO26_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO26_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO26_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO26_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO26_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO26_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO26_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO26_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO26_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO26_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO26_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO26_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO26_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO26_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO26_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO26_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO26_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO26_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO26_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO26_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO26_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO26_CTRL_FUNCSEL_SPI1_SCLK          = 0x1
	IO_BANK0_GPIO26_CTRL_FUNCSEL_UART1_CTS          = 0x2
	IO_BANK0_GPIO26_CTRL_FUNCSEL_I2C1_SDA           = 0x3
	IO_BANK0_GPIO26_CTRL_FUNCSEL_PWM_A_5            = 0x4
	IO_BANK0_GPIO26_CTRL_FUNCSEL_SIOB_PROC_26       = 0x5
	IO_BANK0_GPIO26_CTRL_FUNCSEL_PIO0_26            = 0x6
	IO_BANK0_GPIO26_CTRL_FUNCSEL_PIO1_26            = 0x7
	IO_BANK0_GPIO26_CTRL_FUNCSEL_PIO2_26            = 0x8
	IO_BANK0_GPIO26_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO26_CTRL_FUNCSEL_UART1_TX           = 0xb
	IO_BANK0_GPIO26_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO27_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO27_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO27_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO27_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO27_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO27_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO27_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO27_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO27_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO27_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO27_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO27_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO27_STATUS_OUTTOPAD = 0x200

	// GPIO27_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO27_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO27_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO27_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO27_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO27_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO27_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO27_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO27_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO27_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO27_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO27_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO27_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO27_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO27_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO27_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO27_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO27_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO27_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO27_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO27_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO27_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO27_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO27_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO27_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO27_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO27_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO27_CTRL_FUNCSEL_SPI1_TX                    = 0x1
	IO_BANK0_GPIO27_CTRL_FUNCSEL_UART1_RTS                  = 0x2
	IO_BANK0_GPIO27_CTRL_FUNCSEL_I2C1_SCL                   = 0x3
	IO_BANK0_GPIO27_CTRL_FUNCSEL_PWM_B_5                    = 0x4
	IO_BANK0_GPIO27_CTRL_FUNCSEL_SIOB_PROC_27               = 0x5
	IO_BANK0_GPIO27_CTRL_FUNCSEL_PIO0_27                    = 0x6
	IO_BANK0_GPIO27_CTRL_FUNCSEL_PIO1_27                    = 0x7
	IO_BANK0_GPIO27_CTRL_FUNCSEL_PIO2_27                    = 0x8
	IO_BANK0_GPIO27_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO27_CTRL_FUNCSEL_UART1_RX                   = 0xb
	IO_BANK0_GPIO27_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO28_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO28_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO28_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO28_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO28_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO28_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO28_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO28_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO28_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO28_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO28_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO28_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO28_STATUS_OUTTOPAD = 0x200

	// GPIO28_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO28_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO28_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO28_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO28_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO28_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO28_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO28_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO28_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO28_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO28_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO28_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO28_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO28_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO28_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO28_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO28_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO28_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO28_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO28_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO28_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO28_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO28_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO28_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO28_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO28_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO28_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO28_CTRL_FUNCSEL_SPI1_RX                = 0x1
	IO_BANK0_GPIO28_CTRL_FUNCSEL_UART0_TX               = 0x2
	IO_BANK0_GPIO28_CTRL_FUNCSEL_I2C0_SDA               = 0x3
	IO_BANK0_GPIO28_CTRL_FUNCSEL_PWM_A_6                = 0x4
	IO_BANK0_GPIO28_CTRL_FUNCSEL_SIOB_PROC_28           = 0x5
	IO_BANK0_GPIO28_CTRL_FUNCSEL_PIO0_28                = 0x6
	IO_BANK0_GPIO28_CTRL_FUNCSEL_PIO1_28                = 0x7
	IO_BANK0_GPIO28_CTRL_FUNCSEL_PIO2_28                = 0x8
	IO_BANK0_GPIO28_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO28_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO29_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO29_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO29_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO29_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO29_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO29_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO29_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO29_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO29_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO29_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO29_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO29_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO29_STATUS_OUTTOPAD = 0x200

	// GPIO29_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO29_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO29_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO29_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO29_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO29_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO29_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO29_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO29_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO29_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO29_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO29_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO29_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO29_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO29_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO29_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO29_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO29_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO29_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO29_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO29_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO29_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO29_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO29_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO29_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO29_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO29_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO29_CTRL_FUNCSEL_SPI1_SS_N          = 0x1
	IO_BANK0_GPIO29_CTRL_FUNCSEL_UART0_RX           = 0x2
	IO_BANK0_GPIO29_CTRL_FUNCSEL_I2C0_SCL           = 0x3
	IO_BANK0_GPIO29_CTRL_FUNCSEL_PWM_B_6            = 0x4
	IO_BANK0_GPIO29_CTRL_FUNCSEL_SIOB_PROC_29       = 0x5
	IO_BANK0_GPIO29_CTRL_FUNCSEL_PIO0_29            = 0x6
	IO_BANK0_GPIO29_CTRL_FUNCSEL_PIO1_29            = 0x7
	IO_BANK0_GPIO29_CTRL_FUNCSEL_PIO2_29            = 0x8
	IO_BANK0_GPIO29_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO29_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO30_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO30_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO30_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO30_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO30_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO30_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO30_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO30_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO30_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO30_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO30_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO30_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO30_STATUS_OUTTOPAD = 0x200

	// GPIO30_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO30_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO30_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO30_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO30_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO30_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO30_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO30_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO30_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO30_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO30_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO30_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO30_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO30_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO30_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO30_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO30_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO30_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO30_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO30_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO30_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO30_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO30_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO30_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO30_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO30_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO30_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO30_CTRL_FUNCSEL_SPI1_SCLK                  = 0x1
	IO_BANK0_GPIO30_CTRL_FUNCSEL_UART0_CTS                  = 0x2
	IO_BANK0_GPIO30_CTRL_FUNCSEL_I2C1_SDA                   = 0x3
	IO_BANK0_GPIO30_CTRL_FUNCSEL_PWM_A_7                    = 0x4
	IO_BANK0_GPIO30_CTRL_FUNCSEL_SIOB_PROC_30               = 0x5
	IO_BANK0_GPIO30_CTRL_FUNCSEL_PIO0_30                    = 0x6
	IO_BANK0_GPIO30_CTRL_FUNCSEL_PIO1_30                    = 0x7
	IO_BANK0_GPIO30_CTRL_FUNCSEL_PIO2_30                    = 0x8
	IO_BANK0_GPIO30_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO30_CTRL_FUNCSEL_UART0_TX                   = 0xb
	IO_BANK0_GPIO30_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO31_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO31_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO31_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO31_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO31_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO31_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO31_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO31_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO31_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO31_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO31_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO31_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO31_STATUS_OUTTOPAD = 0x200

	// GPIO31_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO31_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO31_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO31_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO31_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO31_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO31_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO31_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO31_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO31_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO31_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO31_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO31_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO31_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO31_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO31_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO31_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO31_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO31_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO31_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO31_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO31_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO31_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO31_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO31_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO31_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO31_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO31_CTRL_FUNCSEL_SPI1_TX                = 0x1
	IO_BANK0_GPIO31_CTRL_FUNCSEL_UART0_RTS              = 0x2
	IO_BANK0_GPIO31_CTRL_FUNCSEL_I2C1_SCL               = 0x3
	IO_BANK0_GPIO31_CTRL_FUNCSEL_PWM_B_7                = 0x4
	IO_BANK0_GPIO31_CTRL_FUNCSEL_SIOB_PROC_31           = 0x5
	IO_BANK0_GPIO31_CTRL_FUNCSEL_PIO0_31                = 0x6
	IO_BANK0_GPIO31_CTRL_FUNCSEL_PIO1_31                = 0x7
	IO_BANK0_GPIO31_CTRL_FUNCSEL_PIO2_31                = 0x8
	IO_BANK0_GPIO31_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO31_CTRL_FUNCSEL_UART0_RX               = 0xb
	IO_BANK0_GPIO31_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO32_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO32_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO32_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO32_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO32_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO32_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO32_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO32_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO32_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO32_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO32_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO32_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO32_STATUS_OUTTOPAD = 0x200

	// GPIO32_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO32_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO32_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO32_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO32_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO32_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO32_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO32_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO32_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO32_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO32_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO32_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO32_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO32_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO32_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO32_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO32_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO32_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO32_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO32_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO32_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO32_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO32_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO32_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO32_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO32_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO32_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO32_CTRL_FUNCSEL_SPI0_RX            = 0x1
	IO_BANK0_GPIO32_CTRL_FUNCSEL_UART0_TX           = 0x2
	IO_BANK0_GPIO32_CTRL_FUNCSEL_I2C0_SDA           = 0x3
	IO_BANK0_GPIO32_CTRL_FUNCSEL_PWM_A_8            = 0x4
	IO_BANK0_GPIO32_CTRL_FUNCSEL_SIOB_PROC_32       = 0x5
	IO_BANK0_GPIO32_CTRL_FUNCSEL_PIO0_32            = 0x6
	IO_BANK0_GPIO32_CTRL_FUNCSEL_PIO1_32            = 0x7
	IO_BANK0_GPIO32_CTRL_FUNCSEL_PIO2_32            = 0x8
	IO_BANK0_GPIO32_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO32_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO33_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO33_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO33_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO33_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO33_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO33_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO33_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO33_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO33_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO33_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO33_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO33_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO33_STATUS_OUTTOPAD = 0x200

	// GPIO33_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO33_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO33_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO33_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO33_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO33_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO33_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO33_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO33_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO33_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO33_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO33_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO33_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO33_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO33_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO33_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO33_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO33_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO33_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO33_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO33_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO33_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO33_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO33_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO33_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO33_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO33_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO33_CTRL_FUNCSEL_SPI0_SS_N                  = 0x1
	IO_BANK0_GPIO33_CTRL_FUNCSEL_UART0_RX                   = 0x2
	IO_BANK0_GPIO33_CTRL_FUNCSEL_I2C0_SCL                   = 0x3
	IO_BANK0_GPIO33_CTRL_FUNCSEL_PWM_B_8                    = 0x4
	IO_BANK0_GPIO33_CTRL_FUNCSEL_SIOB_PROC_33               = 0x5
	IO_BANK0_GPIO33_CTRL_FUNCSEL_PIO0_33                    = 0x6
	IO_BANK0_GPIO33_CTRL_FUNCSEL_PIO1_33                    = 0x7
	IO_BANK0_GPIO33_CTRL_FUNCSEL_PIO2_33                    = 0x8
	IO_BANK0_GPIO33_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO33_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO34_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO34_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO34_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO34_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO34_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO34_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO34_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO34_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO34_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO34_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO34_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO34_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO34_STATUS_OUTTOPAD = 0x200

	// GPIO34_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO34_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO34_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO34_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO34_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO34_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO34_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO34_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO34_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO34_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO34_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO34_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO34_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO34_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO34_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO34_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO34_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO34_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO34_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO34_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO34_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO34_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO34_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO34_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO34_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO34_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO34_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO34_CTRL_FUNCSEL_SPI0_SCLK              = 0x1
	IO_BANK0_GPIO34_CTRL_FUNCSEL_UART0_CTS              = 0x2
	IO_BANK0_GPIO34_CTRL_FUNCSEL_I2C1_SDA               = 0x3
	IO_BANK0_GPIO34_CTRL_FUNCSEL_PWM_A_9                = 0x4
	IO_BANK0_GPIO34_CTRL_FUNCSEL_SIOB_PROC_34           = 0x5
	IO_BANK0_GPIO34_CTRL_FUNCSEL_PIO0_34                = 0x6
	IO_BANK0_GPIO34_CTRL_FUNCSEL_PIO1_34                = 0x7
	IO_BANK0_GPIO34_CTRL_FUNCSEL_PIO2_34                = 0x8
	IO_BANK0_GPIO34_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO34_CTRL_FUNCSEL_UART0_TX               = 0xb
	IO_BANK0_GPIO34_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO35_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO35_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO35_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO35_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO35_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO35_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO35_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO35_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO35_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO35_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO35_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO35_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO35_STATUS_OUTTOPAD = 0x200

	// GPIO35_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO35_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO35_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO35_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO35_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO35_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO35_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO35_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO35_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO35_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO35_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO35_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO35_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO35_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO35_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO35_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO35_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO35_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO35_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO35_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO35_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO35_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO35_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO35_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO35_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO35_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO35_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO35_CTRL_FUNCSEL_SPI0_TX            = 0x1
	IO_BANK0_GPIO35_CTRL_FUNCSEL_UART0_RTS          = 0x2
	IO_BANK0_GPIO35_CTRL_FUNCSEL_I2C1_SCL           = 0x3
	IO_BANK0_GPIO35_CTRL_FUNCSEL_PWM_B_9            = 0x4
	IO_BANK0_GPIO35_CTRL_FUNCSEL_SIOB_PROC_35       = 0x5
	IO_BANK0_GPIO35_CTRL_FUNCSEL_PIO0_35            = 0x6
	IO_BANK0_GPIO35_CTRL_FUNCSEL_PIO1_35            = 0x7
	IO_BANK0_GPIO35_CTRL_FUNCSEL_PIO2_35            = 0x8
	IO_BANK0_GPIO35_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO35_CTRL_FUNCSEL_UART0_RX           = 0xb
	IO_BANK0_GPIO35_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO36_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO36_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO36_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO36_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO36_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO36_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO36_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO36_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO36_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO36_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO36_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO36_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO36_STATUS_OUTTOPAD = 0x200

	// GPIO36_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO36_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO36_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO36_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO36_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO36_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO36_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO36_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO36_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO36_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO36_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO36_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO36_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO36_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO36_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO36_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO36_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO36_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO36_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO36_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO36_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO36_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO36_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO36_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO36_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO36_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO36_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO36_CTRL_FUNCSEL_SPI0_RX                    = 0x1
	IO_BANK0_GPIO36_CTRL_FUNCSEL_UART1_TX                   = 0x2
	IO_BANK0_GPIO36_CTRL_FUNCSEL_I2C0_SDA                   = 0x3
	IO_BANK0_GPIO36_CTRL_FUNCSEL_PWM_A_10                   = 0x4
	IO_BANK0_GPIO36_CTRL_FUNCSEL_SIOB_PROC_36               = 0x5
	IO_BANK0_GPIO36_CTRL_FUNCSEL_PIO0_36                    = 0x6
	IO_BANK0_GPIO36_CTRL_FUNCSEL_PIO1_36                    = 0x7
	IO_BANK0_GPIO36_CTRL_FUNCSEL_PIO2_36                    = 0x8
	IO_BANK0_GPIO36_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO36_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO37_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO37_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO37_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO37_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO37_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO37_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO37_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO37_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO37_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO37_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO37_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO37_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO37_STATUS_OUTTOPAD = 0x200

	// GPIO37_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO37_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO37_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO37_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO37_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO37_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO37_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO37_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO37_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO37_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO37_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO37_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO37_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO37_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO37_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO37_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO37_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO37_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO37_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO37_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO37_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO37_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO37_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO37_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO37_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO37_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO37_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO37_CTRL_FUNCSEL_SPI0_SS_N              = 0x1
	IO_BANK0_GPIO37_CTRL_FUNCSEL_UART1_RX               = 0x2
	IO_BANK0_GPIO37_CTRL_FUNCSEL_I2C0_SCL               = 0x3
	IO_BANK0_GPIO37_CTRL_FUNCSEL_PWM_B_10               = 0x4
	IO_BANK0_GPIO37_CTRL_FUNCSEL_SIOB_PROC_37           = 0x5
	IO_BANK0_GPIO37_CTRL_FUNCSEL_PIO0_37                = 0x6
	IO_BANK0_GPIO37_CTRL_FUNCSEL_PIO1_37                = 0x7
	IO_BANK0_GPIO37_CTRL_FUNCSEL_PIO2_37                = 0x8
	IO_BANK0_GPIO37_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO37_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO38_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO38_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO38_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO38_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO38_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO38_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO38_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO38_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO38_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO38_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO38_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO38_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO38_STATUS_OUTTOPAD = 0x200

	// GPIO38_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO38_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO38_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO38_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO38_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO38_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO38_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO38_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO38_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO38_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO38_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO38_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO38_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO38_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO38_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO38_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO38_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO38_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO38_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO38_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO38_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO38_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO38_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO38_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO38_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO38_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO38_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO38_CTRL_FUNCSEL_SPI0_SCLK          = 0x1
	IO_BANK0_GPIO38_CTRL_FUNCSEL_UART1_CTS          = 0x2
	IO_BANK0_GPIO38_CTRL_FUNCSEL_I2C1_SDA           = 0x3
	IO_BANK0_GPIO38_CTRL_FUNCSEL_PWM_A_11           = 0x4
	IO_BANK0_GPIO38_CTRL_FUNCSEL_SIOB_PROC_38       = 0x5
	IO_BANK0_GPIO38_CTRL_FUNCSEL_PIO0_38            = 0x6
	IO_BANK0_GPIO38_CTRL_FUNCSEL_PIO1_38            = 0x7
	IO_BANK0_GPIO38_CTRL_FUNCSEL_PIO2_38            = 0x8
	IO_BANK0_GPIO38_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO38_CTRL_FUNCSEL_UART1_TX           = 0xb
	IO_BANK0_GPIO38_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO39_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO39_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO39_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO39_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO39_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO39_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO39_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO39_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO39_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO39_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO39_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO39_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO39_STATUS_OUTTOPAD = 0x200

	// GPIO39_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO39_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO39_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO39_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO39_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO39_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO39_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO39_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO39_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO39_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO39_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO39_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO39_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO39_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO39_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO39_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO39_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO39_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO39_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO39_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO39_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO39_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO39_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO39_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO39_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO39_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO39_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO39_CTRL_FUNCSEL_SPI0_TX                    = 0x1
	IO_BANK0_GPIO39_CTRL_FUNCSEL_UART1_RTS                  = 0x2
	IO_BANK0_GPIO39_CTRL_FUNCSEL_I2C1_SCL                   = 0x3
	IO_BANK0_GPIO39_CTRL_FUNCSEL_PWM_B_11                   = 0x4
	IO_BANK0_GPIO39_CTRL_FUNCSEL_SIOB_PROC_39               = 0x5
	IO_BANK0_GPIO39_CTRL_FUNCSEL_PIO0_39                    = 0x6
	IO_BANK0_GPIO39_CTRL_FUNCSEL_PIO1_39                    = 0x7
	IO_BANK0_GPIO39_CTRL_FUNCSEL_PIO2_39                    = 0x8
	IO_BANK0_GPIO39_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO39_CTRL_FUNCSEL_UART1_RX                   = 0xb
	IO_BANK0_GPIO39_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO40_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO40_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO40_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO40_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO40_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO40_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO40_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO40_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO40_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO40_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO40_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO40_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO40_STATUS_OUTTOPAD = 0x200

	// GPIO40_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO40_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO40_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO40_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO40_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO40_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO40_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO40_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO40_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO40_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO40_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO40_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO40_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO40_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO40_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO40_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO40_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO40_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO40_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO40_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO40_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO40_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO40_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO40_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO40_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO40_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO40_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO40_CTRL_FUNCSEL_SPI1_RX                = 0x1
	IO_BANK0_GPIO40_CTRL_FUNCSEL_UART1_TX               = 0x2
	IO_BANK0_GPIO40_CTRL_FUNCSEL_I2C0_SDA               = 0x3
	IO_BANK0_GPIO40_CTRL_FUNCSEL_PWM_A_8                = 0x4
	IO_BANK0_GPIO40_CTRL_FUNCSEL_SIOB_PROC_40           = 0x5
	IO_BANK0_GPIO40_CTRL_FUNCSEL_PIO0_40                = 0x6
	IO_BANK0_GPIO40_CTRL_FUNCSEL_PIO1_40                = 0x7
	IO_BANK0_GPIO40_CTRL_FUNCSEL_PIO2_40                = 0x8
	IO_BANK0_GPIO40_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO40_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO41_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO41_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO41_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO41_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO41_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO41_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO41_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO41_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO41_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO41_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO41_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO41_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO41_STATUS_OUTTOPAD = 0x200

	// GPIO41_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO41_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO41_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO41_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO41_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO41_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO41_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO41_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO41_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO41_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO41_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO41_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO41_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO41_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO41_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO41_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO41_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO41_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO41_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO41_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO41_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO41_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO41_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO41_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO41_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO41_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO41_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO41_CTRL_FUNCSEL_SPI1_SS_N          = 0x1
	IO_BANK0_GPIO41_CTRL_FUNCSEL_UART1_RX           = 0x2
	IO_BANK0_GPIO41_CTRL_FUNCSEL_I2C0_SCL           = 0x3
	IO_BANK0_GPIO41_CTRL_FUNCSEL_PWM_B_8            = 0x4
	IO_BANK0_GPIO41_CTRL_FUNCSEL_SIOB_PROC_41       = 0x5
	IO_BANK0_GPIO41_CTRL_FUNCSEL_PIO0_41            = 0x6
	IO_BANK0_GPIO41_CTRL_FUNCSEL_PIO1_41            = 0x7
	IO_BANK0_GPIO41_CTRL_FUNCSEL_PIO2_41            = 0x8
	IO_BANK0_GPIO41_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO41_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO42_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO42_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO42_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO42_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO42_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO42_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO42_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO42_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO42_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO42_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO42_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO42_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO42_STATUS_OUTTOPAD = 0x200

	// GPIO42_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO42_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO42_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO42_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO42_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO42_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO42_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO42_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO42_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO42_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO42_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO42_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO42_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO42_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO42_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO42_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO42_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO42_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO42_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO42_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO42_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO42_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO42_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO42_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO42_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO42_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO42_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO42_CTRL_FUNCSEL_SPI1_SCLK                  = 0x1
	IO_BANK0_GPIO42_CTRL_FUNCSEL_UART1_CTS                  = 0x2
	IO_BANK0_GPIO42_CTRL_FUNCSEL_I2C1_SDA                   = 0x3
	IO_BANK0_GPIO42_CTRL_FUNCSEL_PWM_A_9                    = 0x4
	IO_BANK0_GPIO42_CTRL_FUNCSEL_SIOB_PROC_42               = 0x5
	IO_BANK0_GPIO42_CTRL_FUNCSEL_PIO0_42                    = 0x6
	IO_BANK0_GPIO42_CTRL_FUNCSEL_PIO1_42                    = 0x7
	IO_BANK0_GPIO42_CTRL_FUNCSEL_PIO2_42                    = 0x8
	IO_BANK0_GPIO42_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO42_CTRL_FUNCSEL_UART1_TX                   = 0xb
	IO_BANK0_GPIO42_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO43_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO43_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO43_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO43_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO43_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO43_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO43_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO43_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO43_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO43_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO43_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO43_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO43_STATUS_OUTTOPAD = 0x200

	// GPIO43_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO43_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO43_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO43_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO43_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO43_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO43_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO43_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO43_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO43_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO43_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO43_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO43_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO43_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO43_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO43_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO43_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO43_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO43_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO43_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO43_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO43_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO43_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO43_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO43_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO43_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO43_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO43_CTRL_FUNCSEL_SPI1_TX                = 0x1
	IO_BANK0_GPIO43_CTRL_FUNCSEL_UART1_RTS              = 0x2
	IO_BANK0_GPIO43_CTRL_FUNCSEL_I2C1_SCL               = 0x3
	IO_BANK0_GPIO43_CTRL_FUNCSEL_PWM_B_9                = 0x4
	IO_BANK0_GPIO43_CTRL_FUNCSEL_SIOB_PROC_43           = 0x5
	IO_BANK0_GPIO43_CTRL_FUNCSEL_PIO0_43                = 0x6
	IO_BANK0_GPIO43_CTRL_FUNCSEL_PIO1_43                = 0x7
	IO_BANK0_GPIO43_CTRL_FUNCSEL_PIO2_43                = 0x8
	IO_BANK0_GPIO43_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO43_CTRL_FUNCSEL_UART1_RX               = 0xb
	IO_BANK0_GPIO43_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO44_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO44_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO44_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO44_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO44_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO44_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO44_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO44_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO44_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO44_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO44_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO44_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO44_STATUS_OUTTOPAD = 0x200

	// GPIO44_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO44_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO44_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO44_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO44_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO44_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO44_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO44_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO44_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO44_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO44_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO44_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO44_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO44_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO44_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO44_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO44_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO44_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO44_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO44_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO44_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO44_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO44_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO44_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO44_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO44_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO44_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO44_CTRL_FUNCSEL_SPI1_RX            = 0x1
	IO_BANK0_GPIO44_CTRL_FUNCSEL_UART0_TX           = 0x2
	IO_BANK0_GPIO44_CTRL_FUNCSEL_I2C0_SDA           = 0x3
	IO_BANK0_GPIO44_CTRL_FUNCSEL_PWM_A_10           = 0x4
	IO_BANK0_GPIO44_CTRL_FUNCSEL_SIOB_PROC_44       = 0x5
	IO_BANK0_GPIO44_CTRL_FUNCSEL_PIO0_44            = 0x6
	IO_BANK0_GPIO44_CTRL_FUNCSEL_PIO1_44            = 0x7
	IO_BANK0_GPIO44_CTRL_FUNCSEL_PIO2_44            = 0x8
	IO_BANK0_GPIO44_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO44_CTRL_FUNCSEL_NULL               = 0x1f

	// GPIO45_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO45_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO45_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO45_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO45_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO45_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO45_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO45_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO45_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO45_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO45_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO45_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO45_STATUS_OUTTOPAD = 0x200

	// GPIO45_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO45_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO45_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO45_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO45_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO45_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO45_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO45_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO45_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO45_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO45_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO45_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO45_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO45_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO45_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO45_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO45_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO45_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO45_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO45_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO45_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO45_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO45_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO45_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO45_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO45_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO45_CTRL_FUNCSEL_Msk                        = 0x1f
	IO_BANK0_GPIO45_CTRL_FUNCSEL_SPI1_SS_N                  = 0x1
	IO_BANK0_GPIO45_CTRL_FUNCSEL_UART0_RX                   = 0x2
	IO_BANK0_GPIO45_CTRL_FUNCSEL_I2C0_SCL                   = 0x3
	IO_BANK0_GPIO45_CTRL_FUNCSEL_PWM_B_10                   = 0x4
	IO_BANK0_GPIO45_CTRL_FUNCSEL_SIOB_PROC_45               = 0x5
	IO_BANK0_GPIO45_CTRL_FUNCSEL_PIO0_45                    = 0x6
	IO_BANK0_GPIO45_CTRL_FUNCSEL_PIO1_45                    = 0x7
	IO_BANK0_GPIO45_CTRL_FUNCSEL_PIO2_45                    = 0x8
	IO_BANK0_GPIO45_CTRL_FUNCSEL_USB_MUXING_OVERCURR_DETECT = 0xa
	IO_BANK0_GPIO45_CTRL_FUNCSEL_NULL                       = 0x1f

	// GPIO46_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO46_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO46_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO46_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO46_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO46_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO46_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO46_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO46_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO46_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO46_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO46_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO46_STATUS_OUTTOPAD = 0x200

	// GPIO46_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO46_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO46_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO46_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO46_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO46_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO46_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO46_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO46_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO46_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO46_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO46_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO46_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO46_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO46_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO46_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO46_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO46_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO46_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO46_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO46_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO46_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO46_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO46_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO46_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO46_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO46_CTRL_FUNCSEL_Msk                    = 0x1f
	IO_BANK0_GPIO46_CTRL_FUNCSEL_SPI1_SCLK              = 0x1
	IO_BANK0_GPIO46_CTRL_FUNCSEL_UART0_CTS              = 0x2
	IO_BANK0_GPIO46_CTRL_FUNCSEL_I2C1_SDA               = 0x3
	IO_BANK0_GPIO46_CTRL_FUNCSEL_PWM_A_11               = 0x4
	IO_BANK0_GPIO46_CTRL_FUNCSEL_SIOB_PROC_46           = 0x5
	IO_BANK0_GPIO46_CTRL_FUNCSEL_PIO0_46                = 0x6
	IO_BANK0_GPIO46_CTRL_FUNCSEL_PIO1_46                = 0x7
	IO_BANK0_GPIO46_CTRL_FUNCSEL_PIO2_46                = 0x8
	IO_BANK0_GPIO46_CTRL_FUNCSEL_USB_MUXING_VBUS_DETECT = 0xa
	IO_BANK0_GPIO46_CTRL_FUNCSEL_UART0_TX               = 0xb
	IO_BANK0_GPIO46_CTRL_FUNCSEL_NULL                   = 0x1f

	// GPIO47_STATUS
	// Position of IRQTOPROC field.
	IO_BANK0_GPIO47_STATUS_IRQTOPROC_Pos = 0x1a
	// Bit mask of IRQTOPROC field.
	IO_BANK0_GPIO47_STATUS_IRQTOPROC_Msk = 0x4000000
	// Bit IRQTOPROC.
	IO_BANK0_GPIO47_STATUS_IRQTOPROC = 0x4000000
	// Position of INFROMPAD field.
	IO_BANK0_GPIO47_STATUS_INFROMPAD_Pos = 0x11
	// Bit mask of INFROMPAD field.
	IO_BANK0_GPIO47_STATUS_INFROMPAD_Msk = 0x20000
	// Bit INFROMPAD.
	IO_BANK0_GPIO47_STATUS_INFROMPAD = 0x20000
	// Position of OETOPAD field.
	IO_BANK0_GPIO47_STATUS_OETOPAD_Pos = 0xd
	// Bit mask of OETOPAD field.
	IO_BANK0_GPIO47_STATUS_OETOPAD_Msk = 0x2000
	// Bit OETOPAD.
	IO_BANK0_GPIO47_STATUS_OETOPAD = 0x2000
	// Position of OUTTOPAD field.
	IO_BANK0_GPIO47_STATUS_OUTTOPAD_Pos = 0x9
	// Bit mask of OUTTOPAD field.
	IO_BANK0_GPIO47_STATUS_OUTTOPAD_Msk = 0x200
	// Bit OUTTOPAD.
	IO_BANK0_GPIO47_STATUS_OUTTOPAD = 0x200

	// GPIO47_CTRL
	// Position of IRQOVER field.
	IO_BANK0_GPIO47_CTRL_IRQOVER_Pos = 0x1c
	// Bit mask of IRQOVER field.
	IO_BANK0_GPIO47_CTRL_IRQOVER_Msk = 0x30000000
	// don't invert the interrupt
	IO_BANK0_GPIO47_CTRL_IRQOVER_NORMAL = 0x0
	// invert the interrupt
	IO_BANK0_GPIO47_CTRL_IRQOVER_INVERT = 0x1
	// drive interrupt low
	IO_BANK0_GPIO47_CTRL_IRQOVER_LOW = 0x2
	// drive interrupt high
	IO_BANK0_GPIO47_CTRL_IRQOVER_HIGH = 0x3
	// Position of INOVER field.
	IO_BANK0_GPIO47_CTRL_INOVER_Pos = 0x10
	// Bit mask of INOVER field.
	IO_BANK0_GPIO47_CTRL_INOVER_Msk = 0x30000
	// don't invert the peri input
	IO_BANK0_GPIO47_CTRL_INOVER_NORMAL = 0x0
	// invert the peri input
	IO_BANK0_GPIO47_CTRL_INOVER_INVERT = 0x1
	// drive peri input low
	IO_BANK0_GPIO47_CTRL_INOVER_LOW = 0x2
	// drive peri input high
	IO_BANK0_GPIO47_CTRL_INOVER_HIGH = 0x3
	// Position of OEOVER field.
	IO_BANK0_GPIO47_CTRL_OEOVER_Pos = 0xe
	// Bit mask of OEOVER field.
	IO_BANK0_GPIO47_CTRL_OEOVER_Msk = 0xc000
	// drive output enable from peripheral signal selected by funcsel
	IO_BANK0_GPIO47_CTRL_OEOVER_NORMAL = 0x0
	// drive output enable from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO47_CTRL_OEOVER_INVERT = 0x1
	// disable output
	IO_BANK0_GPIO47_CTRL_OEOVER_DISABLE = 0x2
	// enable output
	IO_BANK0_GPIO47_CTRL_OEOVER_ENABLE = 0x3
	// Position of OUTOVER field.
	IO_BANK0_GPIO47_CTRL_OUTOVER_Pos = 0xc
	// Bit mask of OUTOVER field.
	IO_BANK0_GPIO47_CTRL_OUTOVER_Msk = 0x3000
	// drive output from peripheral signal selected by funcsel
	IO_BANK0_GPIO47_CTRL_OUTOVER_NORMAL = 0x0
	// drive output from inverse of peripheral signal selected by funcsel
	IO_BANK0_GPIO47_CTRL_OUTOVER_INVERT = 0x1
	// drive output low
	IO_BANK0_GPIO47_CTRL_OUTOVER_LOW = 0x2
	// drive output high
	IO_BANK0_GPIO47_CTRL_OUTOVER_HIGH = 0x3
	// Position of FUNCSEL field.
	IO_BANK0_GPIO47_CTRL_FUNCSEL_Pos = 0x0
	// Bit mask of FUNCSEL field.
	IO_BANK0_GPIO47_CTRL_FUNCSEL_Msk                = 0x1f
	IO_BANK0_GPIO47_CTRL_FUNCSEL_SPI1_TX            = 0x1
	IO_BANK0_GPIO47_CTRL_FUNCSEL_UART0_RTS          = 0x2
	IO_BANK0_GPIO47_CTRL_FUNCSEL_I2C1_SCL           = 0x3
	IO_BANK0_GPIO47_CTRL_FUNCSEL_PWM_B_11           = 0x4
	IO_BANK0_GPIO47_CTRL_FUNCSEL_SIOB_PROC_47       = 0x5
	IO_BANK0_GPIO47_CTRL_FUNCSEL_PIO0_47            = 0x6
	IO_BANK0_GPIO47_CTRL_FUNCSEL_PIO1_47            = 0x7
	IO_BANK0_GPIO47_CTRL_FUNCSEL_PIO2_47            = 0x8
	IO_BANK0_GPIO47_CTRL_FUNCSEL_XIP_SS_N_1         = 0x9
	IO_BANK0_GPIO47_CTRL_FUNCSEL_USB_MUXING_VBUS_EN = 0xa
	IO_BANK0_GPIO47_CTRL_FUNCSEL_UART0_RX           = 0xb
	IO_BANK0_GPIO47_CTRL_FUNCSEL_NULL               = 0x1f

	// IRQSUMMARY_PROC0_SECURE0
	// Position of GPIO31 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO31_Pos = 0x1f
	// Bit mask of GPIO31 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO31_Msk = 0x80000000
	// Bit GPIO31.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO31 = 0x80000000
	// Position of GPIO30 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO30_Pos = 0x1e
	// Bit mask of GPIO30 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO30_Msk = 0x40000000
	// Bit GPIO30.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO30 = 0x40000000
	// Position of GPIO29 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO29_Pos = 0x1d
	// Bit mask of GPIO29 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO29_Msk = 0x20000000
	// Bit GPIO29.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO29 = 0x20000000
	// Position of GPIO28 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO28_Pos = 0x1c
	// Bit mask of GPIO28 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO28_Msk = 0x10000000
	// Bit GPIO28.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO28 = 0x10000000
	// Position of GPIO27 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO27_Pos = 0x1b
	// Bit mask of GPIO27 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO27_Msk = 0x8000000
	// Bit GPIO27.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO27 = 0x8000000
	// Position of GPIO26 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO26_Pos = 0x1a
	// Bit mask of GPIO26 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO26_Msk = 0x4000000
	// Bit GPIO26.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO26 = 0x4000000
	// Position of GPIO25 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO25_Pos = 0x19
	// Bit mask of GPIO25 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO25_Msk = 0x2000000
	// Bit GPIO25.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO25 = 0x2000000
	// Position of GPIO24 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO24_Pos = 0x18
	// Bit mask of GPIO24 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO24_Msk = 0x1000000
	// Bit GPIO24.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO24 = 0x1000000
	// Position of GPIO23 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO23_Pos = 0x17
	// Bit mask of GPIO23 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO23_Msk = 0x800000
	// Bit GPIO23.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO23 = 0x800000
	// Position of GPIO22 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO22_Pos = 0x16
	// Bit mask of GPIO22 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO22_Msk = 0x400000
	// Bit GPIO22.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO22 = 0x400000
	// Position of GPIO21 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO21_Pos = 0x15
	// Bit mask of GPIO21 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO21_Msk = 0x200000
	// Bit GPIO21.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO21 = 0x200000
	// Position of GPIO20 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO20_Pos = 0x14
	// Bit mask of GPIO20 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO20_Msk = 0x100000
	// Bit GPIO20.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO20 = 0x100000
	// Position of GPIO19 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO19_Pos = 0x13
	// Bit mask of GPIO19 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO19_Msk = 0x80000
	// Bit GPIO19.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO19 = 0x80000
	// Position of GPIO18 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO18_Pos = 0x12
	// Bit mask of GPIO18 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO18_Msk = 0x40000
	// Bit GPIO18.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO18 = 0x40000
	// Position of GPIO17 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO17_Pos = 0x11
	// Bit mask of GPIO17 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO17_Msk = 0x20000
	// Bit GPIO17.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO17 = 0x20000
	// Position of GPIO16 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO16_Pos = 0x10
	// Bit mask of GPIO16 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO16_Msk = 0x10000
	// Bit GPIO16.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO16 = 0x10000
	// Position of GPIO15 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO15_Pos = 0xf
	// Bit mask of GPIO15 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO15_Msk = 0x8000
	// Bit GPIO15.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO15 = 0x8000
	// Position of GPIO14 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO14_Pos = 0xe
	// Bit mask of GPIO14 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO14_Msk = 0x4000
	// Bit GPIO14.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO14 = 0x4000
	// Position of GPIO13 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO13_Pos = 0xd
	// Bit mask of GPIO13 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO13_Msk = 0x2000
	// Bit GPIO13.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO13 = 0x2000
	// Position of GPIO12 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO12_Pos = 0xc
	// Bit mask of GPIO12 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO12_Msk = 0x1000
	// Bit GPIO12.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO12 = 0x1000
	// Position of GPIO11 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO11_Pos = 0xb
	// Bit mask of GPIO11 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO11_Msk = 0x800
	// Bit GPIO11.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO11 = 0x800
	// Position of GPIO10 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO10_Pos = 0xa
	// Bit mask of GPIO10 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO10_Msk = 0x400
	// Bit GPIO10.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO10 = 0x400
	// Position of GPIO9 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO9_Pos = 0x9
	// Bit mask of GPIO9 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO9_Msk = 0x200
	// Bit GPIO9.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO9 = 0x200
	// Position of GPIO8 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO8_Pos = 0x8
	// Bit mask of GPIO8 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO8_Msk = 0x100
	// Bit GPIO8.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO8 = 0x100
	// Position of GPIO7 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO7_Pos = 0x7
	// Bit mask of GPIO7 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO7_Msk = 0x80
	// Bit GPIO7.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO7 = 0x80
	// Position of GPIO6 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO6_Pos = 0x6
	// Bit mask of GPIO6 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO6_Msk = 0x40
	// Bit GPIO6.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO6 = 0x40
	// Position of GPIO5 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO5_Pos = 0x5
	// Bit mask of GPIO5 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO5_Msk = 0x20
	// Bit GPIO5.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO5 = 0x20
	// Position of GPIO4 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO4_Pos = 0x4
	// Bit mask of GPIO4 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO4_Msk = 0x10
	// Bit GPIO4.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO4 = 0x10
	// Position of GPIO3 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO3_Pos = 0x3
	// Bit mask of GPIO3 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO3_Msk = 0x8
	// Bit GPIO3.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO3 = 0x8
	// Position of GPIO2 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO2_Pos = 0x2
	// Bit mask of GPIO2 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO2_Msk = 0x4
	// Bit GPIO2.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO2 = 0x4
	// Position of GPIO1 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO1_Pos = 0x1
	// Bit mask of GPIO1 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO1_Msk = 0x2
	// Bit GPIO1.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO1 = 0x2
	// Position of GPIO0 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO0_Pos = 0x0
	// Bit mask of GPIO0 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO0_Msk = 0x1
	// Bit GPIO0.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE0_GPIO0 = 0x1

	// IRQSUMMARY_PROC0_SECURE1
	// Position of GPIO47 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO47_Pos = 0xf
	// Bit mask of GPIO47 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO47_Msk = 0x8000
	// Bit GPIO47.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO47 = 0x8000
	// Position of GPIO46 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO46_Pos = 0xe
	// Bit mask of GPIO46 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO46_Msk = 0x4000
	// Bit GPIO46.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO46 = 0x4000
	// Position of GPIO45 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO45_Pos = 0xd
	// Bit mask of GPIO45 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO45_Msk = 0x2000
	// Bit GPIO45.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO45 = 0x2000
	// Position of GPIO44 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO44_Pos = 0xc
	// Bit mask of GPIO44 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO44_Msk = 0x1000
	// Bit GPIO44.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO44 = 0x1000
	// Position of GPIO43 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO43_Pos = 0xb
	// Bit mask of GPIO43 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO43_Msk = 0x800
	// Bit GPIO43.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO43 = 0x800
	// Position of GPIO42 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO42_Pos = 0xa
	// Bit mask of GPIO42 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO42_Msk = 0x400
	// Bit GPIO42.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO42 = 0x400
	// Position of GPIO41 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO41_Pos = 0x9
	// Bit mask of GPIO41 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO41_Msk = 0x200
	// Bit GPIO41.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO41 = 0x200
	// Position of GPIO40 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO40_Pos = 0x8
	// Bit mask of GPIO40 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO40_Msk = 0x100
	// Bit GPIO40.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO40 = 0x100
	// Position of GPIO39 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO39_Pos = 0x7
	// Bit mask of GPIO39 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO39_Msk = 0x80
	// Bit GPIO39.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO39 = 0x80
	// Position of GPIO38 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO38_Pos = 0x6
	// Bit mask of GPIO38 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO38_Msk = 0x40
	// Bit GPIO38.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO38 = 0x40
	// Position of GPIO37 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO37_Pos = 0x5
	// Bit mask of GPIO37 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO37_Msk = 0x20
	// Bit GPIO37.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO37 = 0x20
	// Position of GPIO36 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO36_Pos = 0x4
	// Bit mask of GPIO36 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO36_Msk = 0x10
	// Bit GPIO36.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO36 = 0x10
	// Position of GPIO35 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO35_Pos = 0x3
	// Bit mask of GPIO35 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO35_Msk = 0x8
	// Bit GPIO35.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO35 = 0x8
	// Position of GPIO34 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO34_Pos = 0x2
	// Bit mask of GPIO34 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO34_Msk = 0x4
	// Bit GPIO34.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO34 = 0x4
	// Position of GPIO33 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO33_Pos = 0x1
	// Bit mask of GPIO33 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO33_Msk = 0x2
	// Bit GPIO33.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO33 = 0x2
	// Position of GPIO32 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO32_Pos = 0x0
	// Bit mask of GPIO32 field.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO32_Msk = 0x1
	// Bit GPIO32.
	IO_BANK0_IRQSUMMARY_PROC0_SECURE1_GPIO32 = 0x1

	// IRQSUMMARY_PROC0_NONSECURE0
	// Position of GPIO31 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO31_Pos = 0x1f
	// Bit mask of GPIO31 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO31_Msk = 0x80000000
	// Bit GPIO31.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO31 = 0x80000000
	// Position of GPIO30 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO30_Pos = 0x1e
	// Bit mask of GPIO30 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO30_Msk = 0x40000000
	// Bit GPIO30.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO30 = 0x40000000
	// Position of GPIO29 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO29_Pos = 0x1d
	// Bit mask of GPIO29 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO29_Msk = 0x20000000
	// Bit GPIO29.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO29 = 0x20000000
	// Position of GPIO28 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO28_Pos = 0x1c
	// Bit mask of GPIO28 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO28_Msk = 0x10000000
	// Bit GPIO28.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO28 = 0x10000000
	// Position of GPIO27 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO27_Pos = 0x1b
	// Bit mask of GPIO27 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO27_Msk = 0x8000000
	// Bit GPIO27.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO27 = 0x8000000
	// Position of GPIO26 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO26_Pos = 0x1a
	// Bit mask of GPIO26 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO26_Msk = 0x4000000
	// Bit GPIO26.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO26 = 0x4000000
	// Position of GPIO25 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO25_Pos = 0x19
	// Bit mask of GPIO25 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO25_Msk = 0x2000000
	// Bit GPIO25.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO25 = 0x2000000
	// Position of GPIO24 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO24_Pos = 0x18
	// Bit mask of GPIO24 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO24_Msk = 0x1000000
	// Bit GPIO24.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO24 = 0x1000000
	// Position of GPIO23 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO23_Pos = 0x17
	// Bit mask of GPIO23 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO23_Msk = 0x800000
	// Bit GPIO23.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO23 = 0x800000
	// Position of GPIO22 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO22_Pos = 0x16
	// Bit mask of GPIO22 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO22_Msk = 0x400000
	// Bit GPIO22.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO22 = 0x400000
	// Position of GPIO21 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO21_Pos = 0x15
	// Bit mask of GPIO21 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO21_Msk = 0x200000
	// Bit GPIO21.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO21 = 0x200000
	// Position of GPIO20 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO20_Pos = 0x14
	// Bit mask of GPIO20 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO20_Msk = 0x100000
	// Bit GPIO20.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO20 = 0x100000
	// Position of GPIO19 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO19_Pos = 0x13
	// Bit mask of GPIO19 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO19_Msk = 0x80000
	// Bit GPIO19.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO19 = 0x80000
	// Position of GPIO18 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO18_Pos = 0x12
	// Bit mask of GPIO18 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO18_Msk = 0x40000
	// Bit GPIO18.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO18 = 0x40000
	// Position of GPIO17 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO17_Pos = 0x11
	// Bit mask of GPIO17 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO17_Msk = 0x20000
	// Bit GPIO17.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO17 = 0x20000
	// Position of GPIO16 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO16_Pos = 0x10
	// Bit mask of GPIO16 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO16_Msk = 0x10000
	// Bit GPIO16.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO16 = 0x10000
	// Position of GPIO15 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO15_Pos = 0xf
	// Bit mask of GPIO15 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO15_Msk = 0x8000
	// Bit GPIO15.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO15 = 0x8000
	// Position of GPIO14 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO14_Pos = 0xe
	// Bit mask of GPIO14 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO14_Msk = 0x4000
	// Bit GPIO14.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO14 = 0x4000
	// Position of GPIO13 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO13_Pos = 0xd
	// Bit mask of GPIO13 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO13_Msk = 0x2000
	// Bit GPIO13.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO13 = 0x2000
	// Position of GPIO12 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO12_Pos = 0xc
	// Bit mask of GPIO12 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO12_Msk = 0x1000
	// Bit GPIO12.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO12 = 0x1000
	// Position of GPIO11 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO11_Pos = 0xb
	// Bit mask of GPIO11 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO11_Msk = 0x800
	// Bit GPIO11.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO11 = 0x800
	// Position of GPIO10 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO10_Pos = 0xa
	// Bit mask of GPIO10 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO10_Msk = 0x400
	// Bit GPIO10.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO10 = 0x400
	// Position of GPIO9 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO9_Pos = 0x9
	// Bit mask of GPIO9 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO9_Msk = 0x200
	// Bit GPIO9.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO9 = 0x200
	// Position of GPIO8 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO8_Pos = 0x8
	// Bit mask of GPIO8 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO8_Msk = 0x100
	// Bit GPIO8.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO8 = 0x100
	// Position of GPIO7 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO7_Pos = 0x7
	// Bit mask of GPIO7 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO7_Msk = 0x80
	// Bit GPIO7.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO7 = 0x80
	// Position of GPIO6 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO6_Pos = 0x6
	// Bit mask of GPIO6 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO6_Msk = 0x40
	// Bit GPIO6.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO6 = 0x40
	// Position of GPIO5 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO5_Pos = 0x5
	// Bit mask of GPIO5 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO5_Msk = 0x20
	// Bit GPIO5.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO5 = 0x20
	// Position of GPIO4 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO4_Pos = 0x4
	// Bit mask of GPIO4 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO4_Msk = 0x10
	// Bit GPIO4.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO4 = 0x10
	// Position of GPIO3 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO3_Pos = 0x3
	// Bit mask of GPIO3 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO3_Msk = 0x8
	// Bit GPIO3.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO3 = 0x8
	// Position of GPIO2 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO2_Pos = 0x2
	// Bit mask of GPIO2 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO2_Msk = 0x4
	// Bit GPIO2.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO2 = 0x4
	// Position of GPIO1 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO1_Pos = 0x1
	// Bit mask of GPIO1 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO1_Msk = 0x2
	// Bit GPIO1.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO1 = 0x2
	// Position of GPIO0 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO0_Pos = 0x0
	// Bit mask of GPIO0 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO0_Msk = 0x1
	// Bit GPIO0.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE0_GPIO0 = 0x1

	// IRQSUMMARY_PROC0_NONSECURE1
	// Position of GPIO47 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO47_Pos = 0xf
	// Bit mask of GPIO47 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO47_Msk = 0x8000
	// Bit GPIO47.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO47 = 0x8000
	// Position of GPIO46 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO46_Pos = 0xe
	// Bit mask of GPIO46 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO46_Msk = 0x4000
	// Bit GPIO46.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO46 = 0x4000
	// Position of GPIO45 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO45_Pos = 0xd
	// Bit mask of GPIO45 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO45_Msk = 0x2000
	// Bit GPIO45.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO45 = 0x2000
	// Position of GPIO44 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO44_Pos = 0xc
	// Bit mask of GPIO44 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO44_Msk = 0x1000
	// Bit GPIO44.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO44 = 0x1000
	// Position of GPIO43 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO43_Pos = 0xb
	// Bit mask of GPIO43 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO43_Msk = 0x800
	// Bit GPIO43.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO43 = 0x800
	// Position of GPIO42 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO42_Pos = 0xa
	// Bit mask of GPIO42 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO42_Msk = 0x400
	// Bit GPIO42.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO42 = 0x400
	// Position of GPIO41 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO41_Pos = 0x9
	// Bit mask of GPIO41 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO41_Msk = 0x200
	// Bit GPIO41.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO41 = 0x200
	// Position of GPIO40 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO40_Pos = 0x8
	// Bit mask of GPIO40 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO40_Msk = 0x100
	// Bit GPIO40.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO40 = 0x100
	// Position of GPIO39 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO39_Pos = 0x7
	// Bit mask of GPIO39 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO39_Msk = 0x80
	// Bit GPIO39.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO39 = 0x80
	// Position of GPIO38 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO38_Pos = 0x6
	// Bit mask of GPIO38 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO38_Msk = 0x40
	// Bit GPIO38.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO38 = 0x40
	// Position of GPIO37 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO37_Pos = 0x5
	// Bit mask of GPIO37 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO37_Msk = 0x20
	// Bit GPIO37.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO37 = 0x20
	// Position of GPIO36 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO36_Pos = 0x4
	// Bit mask of GPIO36 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO36_Msk = 0x10
	// Bit GPIO36.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO36 = 0x10
	// Position of GPIO35 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO35_Pos = 0x3
	// Bit mask of GPIO35 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO35_Msk = 0x8
	// Bit GPIO35.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO35 = 0x8
	// Position of GPIO34 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO34_Pos = 0x2
	// Bit mask of GPIO34 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO34_Msk = 0x4
	// Bit GPIO34.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO34 = 0x4
	// Position of GPIO33 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO33_Pos = 0x1
	// Bit mask of GPIO33 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO33_Msk = 0x2
	// Bit GPIO33.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO33 = 0x2
	// Position of GPIO32 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO32_Pos = 0x0
	// Bit mask of GPIO32 field.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO32_Msk = 0x1
	// Bit GPIO32.
	IO_BANK0_IRQSUMMARY_PROC0_NONSECURE1_GPIO32 = 0x1

	// IRQSUMMARY_PROC1_SECURE0
	// Position of GPIO31 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO31_Pos = 0x1f
	// Bit mask of GPIO31 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO31_Msk = 0x80000000
	// Bit GPIO31.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO31 = 0x80000000
	// Position of GPIO30 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO30_Pos = 0x1e
	// Bit mask of GPIO30 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO30_Msk = 0x40000000
	// Bit GPIO30.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO30 = 0x40000000
	// Position of GPIO29 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO29_Pos = 0x1d
	// Bit mask of GPIO29 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO29_Msk = 0x20000000
	// Bit GPIO29.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO29 = 0x20000000
	// Position of GPIO28 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO28_Pos = 0x1c
	// Bit mask of GPIO28 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO28_Msk = 0x10000000
	// Bit GPIO28.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO28 = 0x10000000
	// Position of GPIO27 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO27_Pos = 0x1b
	// Bit mask of GPIO27 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO27_Msk = 0x8000000
	// Bit GPIO27.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO27 = 0x8000000
	// Position of GPIO26 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO26_Pos = 0x1a
	// Bit mask of GPIO26 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO26_Msk = 0x4000000
	// Bit GPIO26.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO26 = 0x4000000
	// Position of GPIO25 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO25_Pos = 0x19
	// Bit mask of GPIO25 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO25_Msk = 0x2000000
	// Bit GPIO25.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO25 = 0x2000000
	// Position of GPIO24 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO24_Pos = 0x18
	// Bit mask of GPIO24 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO24_Msk = 0x1000000
	// Bit GPIO24.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO24 = 0x1000000
	// Position of GPIO23 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO23_Pos = 0x17
	// Bit mask of GPIO23 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO23_Msk = 0x800000
	// Bit GPIO23.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO23 = 0x800000
	// Position of GPIO22 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO22_Pos = 0x16
	// Bit mask of GPIO22 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO22_Msk = 0x400000
	// Bit GPIO22.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO22 = 0x400000
	// Position of GPIO21 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO21_Pos = 0x15
	// Bit mask of GPIO21 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO21_Msk = 0x200000
	// Bit GPIO21.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO21 = 0x200000
	// Position of GPIO20 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO20_Pos = 0x14
	// Bit mask of GPIO20 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO20_Msk = 0x100000
	// Bit GPIO20.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO20 = 0x100000
	// Position of GPIO19 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO19_Pos = 0x13
	// Bit mask of GPIO19 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO19_Msk = 0x80000
	// Bit GPIO19.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO19 = 0x80000
	// Position of GPIO18 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO18_Pos = 0x12
	// Bit mask of GPIO18 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO18_Msk = 0x40000
	// Bit GPIO18.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO18 = 0x40000
	// Position of GPIO17 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO17_Pos = 0x11
	// Bit mask of GPIO17 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO17_Msk = 0x20000
	// Bit GPIO17.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO17 = 0x20000
	// Position of GPIO16 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO16_Pos = 0x10
	// Bit mask of GPIO16 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO16_Msk = 0x10000
	// Bit GPIO16.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO16 = 0x10000
	// Position of GPIO15 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO15_Pos = 0xf
	// Bit mask of GPIO15 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO15_Msk = 0x8000
	// Bit GPIO15.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO15 = 0x8000
	// Position of GPIO14 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO14_Pos = 0xe
	// Bit mask of GPIO14 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO14_Msk = 0x4000
	// Bit GPIO14.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO14 = 0x4000
	// Position of GPIO13 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO13_Pos = 0xd
	// Bit mask of GPIO13 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO13_Msk = 0x2000
	// Bit GPIO13.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO13 = 0x2000
	// Position of GPIO12 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO12_Pos = 0xc
	// Bit mask of GPIO12 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO12_Msk = 0x1000
	// Bit GPIO12.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO12 = 0x1000
	// Position of GPIO11 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO11_Pos = 0xb
	// Bit mask of GPIO11 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO11_Msk = 0x800
	// Bit GPIO11.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO11 = 0x800
	// Position of GPIO10 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO10_Pos = 0xa
	// Bit mask of GPIO10 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO10_Msk = 0x400
	// Bit GPIO10.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO10 = 0x400
	// Position of GPIO9 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO9_Pos = 0x9
	// Bit mask of GPIO9 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO9_Msk = 0x200
	// Bit GPIO9.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO9 = 0x200
	// Position of GPIO8 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO8_Pos = 0x8
	// Bit mask of GPIO8 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO8_Msk = 0x100
	// Bit GPIO8.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO8 = 0x100
	// Position of GPIO7 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO7_Pos = 0x7
	// Bit mask of GPIO7 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO7_Msk = 0x80
	// Bit GPIO7.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO7 = 0x80
	// Position of GPIO6 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO6_Pos = 0x6
	// Bit mask of GPIO6 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO6_Msk = 0x40
	// Bit GPIO6.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO6 = 0x40
	// Position of GPIO5 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO5_Pos = 0x5
	// Bit mask of GPIO5 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO5_Msk = 0x20
	// Bit GPIO5.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO5 = 0x20
	// Position of GPIO4 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO4_Pos = 0x4
	// Bit mask of GPIO4 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO4_Msk = 0x10
	// Bit GPIO4.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO4 = 0x10
	// Position of GPIO3 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO3_Pos = 0x3
	// Bit mask of GPIO3 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO3_Msk = 0x8
	// Bit GPIO3.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO3 = 0x8
	// Position of GPIO2 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO2_Pos = 0x2
	// Bit mask of GPIO2 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO2_Msk = 0x4
	// Bit GPIO2.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO2 = 0x4
	// Position of GPIO1 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO1_Pos = 0x1
	// Bit mask of GPIO1 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO1_Msk = 0x2
	// Bit GPIO1.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO1 = 0x2
	// Position of GPIO0 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO0_Pos = 0x0
	// Bit mask of GPIO0 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO0_Msk = 0x1
	// Bit GPIO0.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE0_GPIO0 = 0x1

	// IRQSUMMARY_PROC1_SECURE1
	// Position of GPIO47 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO47_Pos = 0xf
	// Bit mask of GPIO47 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO47_Msk = 0x8000
	// Bit GPIO47.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO47 = 0x8000
	// Position of GPIO46 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO46_Pos = 0xe
	// Bit mask of GPIO46 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO46_Msk = 0x4000
	// Bit GPIO46.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO46 = 0x4000
	// Position of GPIO45 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO45_Pos = 0xd
	// Bit mask of GPIO45 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO45_Msk = 0x2000
	// Bit GPIO45.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO45 = 0x2000
	// Position of GPIO44 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO44_Pos = 0xc
	// Bit mask of GPIO44 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO44_Msk = 0x1000
	// Bit GPIO44.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO44 = 0x1000
	// Position of GPIO43 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO43_Pos = 0xb
	// Bit mask of GPIO43 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO43_Msk = 0x800
	// Bit GPIO43.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO43 = 0x800
	// Position of GPIO42 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO42_Pos = 0xa
	// Bit mask of GPIO42 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO42_Msk = 0x400
	// Bit GPIO42.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO42 = 0x400
	// Position of GPIO41 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO41_Pos = 0x9
	// Bit mask of GPIO41 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO41_Msk = 0x200
	// Bit GPIO41.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO41 = 0x200
	// Position of GPIO40 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO40_Pos = 0x8
	// Bit mask of GPIO40 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO40_Msk = 0x100
	// Bit GPIO40.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO40 = 0x100
	// Position of GPIO39 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO39_Pos = 0x7
	// Bit mask of GPIO39 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO39_Msk = 0x80
	// Bit GPIO39.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO39 = 0x80
	// Position of GPIO38 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO38_Pos = 0x6
	// Bit mask of GPIO38 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO38_Msk = 0x40
	// Bit GPIO38.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO38 = 0x40
	// Position of GPIO37 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO37_Pos = 0x5
	// Bit mask of GPIO37 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO37_Msk = 0x20
	// Bit GPIO37.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO37 = 0x20
	// Position of GPIO36 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO36_Pos = 0x4
	// Bit mask of GPIO36 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO36_Msk = 0x10
	// Bit GPIO36.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO36 = 0x10
	// Position of GPIO35 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO35_Pos = 0x3
	// Bit mask of GPIO35 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO35_Msk = 0x8
	// Bit GPIO35.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO35 = 0x8
	// Position of GPIO34 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO34_Pos = 0x2
	// Bit mask of GPIO34 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO34_Msk = 0x4
	// Bit GPIO34.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO34 = 0x4
	// Position of GPIO33 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO33_Pos = 0x1
	// Bit mask of GPIO33 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO33_Msk = 0x2
	// Bit GPIO33.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO33 = 0x2
	// Position of GPIO32 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO32_Pos = 0x0
	// Bit mask of GPIO32 field.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO32_Msk = 0x1
	// Bit GPIO32.
	IO_BANK0_IRQSUMMARY_PROC1_SECURE1_GPIO32 = 0x1

	// IRQSUMMARY_PROC1_NONSECURE0
	// Position of GPIO31 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO31_Pos = 0x1f
	// Bit mask of GPIO31 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO31_Msk = 0x80000000
	// Bit GPIO31.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO31 = 0x80000000
	// Position of GPIO30 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO30_Pos = 0x1e
	// Bit mask of GPIO30 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO30_Msk = 0x40000000
	// Bit GPIO30.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO30 = 0x40000000
	// Position of GPIO29 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO29_Pos = 0x1d
	// Bit mask of GPIO29 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO29_Msk = 0x20000000
	// Bit GPIO29.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO29 = 0x20000000
	// Position of GPIO28 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO28_Pos = 0x1c
	// Bit mask of GPIO28 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO28_Msk = 0x10000000
	// Bit GPIO28.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO28 = 0x10000000
	// Position of GPIO27 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO27_Pos = 0x1b
	// Bit mask of GPIO27 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO27_Msk = 0x8000000
	// Bit GPIO27.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO27 = 0x8000000
	// Position of GPIO26 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO26_Pos = 0x1a
	// Bit mask of GPIO26 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO26_Msk = 0x4000000
	// Bit GPIO26.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO26 = 0x4000000
	// Position of GPIO25 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO25_Pos = 0x19
	// Bit mask of GPIO25 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO25_Msk = 0x2000000
	// Bit GPIO25.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO25 = 0x2000000
	// Position of GPIO24 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO24_Pos = 0x18
	// Bit mask of GPIO24 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO24_Msk = 0x1000000
	// Bit GPIO24.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO24 = 0x1000000
	// Position of GPIO23 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO23_Pos = 0x17
	// Bit mask of GPIO23 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO23_Msk = 0x800000
	// Bit GPIO23.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO23 = 0x800000
	// Position of GPIO22 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO22_Pos = 0x16
	// Bit mask of GPIO22 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO22_Msk = 0x400000
	// Bit GPIO22.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO22 = 0x400000
	// Position of GPIO21 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO21_Pos = 0x15
	// Bit mask of GPIO21 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO21_Msk = 0x200000
	// Bit GPIO21.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO21 = 0x200000
	// Position of GPIO20 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO20_Pos = 0x14
	// Bit mask of GPIO20 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO20_Msk = 0x100000
	// Bit GPIO20.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO20 = 0x100000
	// Position of GPIO19 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO19_Pos = 0x13
	// Bit mask of GPIO19 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO19_Msk = 0x80000
	// Bit GPIO19.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO19 = 0x80000
	// Position of GPIO18 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO18_Pos = 0x12
	// Bit mask of GPIO18 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO18_Msk = 0x40000
	// Bit GPIO18.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO18 = 0x40000
	// Position of GPIO17 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO17_Pos = 0x11
	// Bit mask of GPIO17 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO17_Msk = 0x20000
	// Bit GPIO17.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO17 = 0x20000
	// Position of GPIO16 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO16_Pos = 0x10
	// Bit mask of GPIO16 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO16_Msk = 0x10000
	// Bit GPIO16.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO16 = 0x10000
	// Position of GPIO15 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO15_Pos = 0xf
	// Bit mask of GPIO15 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO15_Msk = 0x8000
	// Bit GPIO15.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO15 = 0x8000
	// Position of GPIO14 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO14_Pos = 0xe
	// Bit mask of GPIO14 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO14_Msk = 0x4000
	// Bit GPIO14.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO14 = 0x4000
	// Position of GPIO13 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO13_Pos = 0xd
	// Bit mask of GPIO13 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO13_Msk = 0x2000
	// Bit GPIO13.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO13 = 0x2000
	// Position of GPIO12 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO12_Pos = 0xc
	// Bit mask of GPIO12 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO12_Msk = 0x1000
	// Bit GPIO12.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO12 = 0x1000
	// Position of GPIO11 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO11_Pos = 0xb
	// Bit mask of GPIO11 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO11_Msk = 0x800
	// Bit GPIO11.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO11 = 0x800
	// Position of GPIO10 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO10_Pos = 0xa
	// Bit mask of GPIO10 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO10_Msk = 0x400
	// Bit GPIO10.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO10 = 0x400
	// Position of GPIO9 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO9_Pos = 0x9
	// Bit mask of GPIO9 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO9_Msk = 0x200
	// Bit GPIO9.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO9 = 0x200
	// Position of GPIO8 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO8_Pos = 0x8
	// Bit mask of GPIO8 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO8_Msk = 0x100
	// Bit GPIO8.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO8 = 0x100
	// Position of GPIO7 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO7_Pos = 0x7
	// Bit mask of GPIO7 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO7_Msk = 0x80
	// Bit GPIO7.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO7 = 0x80
	// Position of GPIO6 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO6_Pos = 0x6
	// Bit mask of GPIO6 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO6_Msk = 0x40
	// Bit GPIO6.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO6 = 0x40
	// Position of GPIO5 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO5_Pos = 0x5
	// Bit mask of GPIO5 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO5_Msk = 0x20
	// Bit GPIO5.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO5 = 0x20
	// Position of GPIO4 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO4_Pos = 0x4
	// Bit mask of GPIO4 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO4_Msk = 0x10
	// Bit GPIO4.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO4 = 0x10
	// Position of GPIO3 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO3_Pos = 0x3
	// Bit mask of GPIO3 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO3_Msk = 0x8
	// Bit GPIO3.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO3 = 0x8
	// Position of GPIO2 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO2_Pos = 0x2
	// Bit mask of GPIO2 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO2_Msk = 0x4
	// Bit GPIO2.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO2 = 0x4
	// Position of GPIO1 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO1_Pos = 0x1
	// Bit mask of GPIO1 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO1_Msk = 0x2
	// Bit GPIO1.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO1 = 0x2
	// Position of GPIO0 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO0_Pos = 0x0
	// Bit mask of GPIO0 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO0_Msk = 0x1
	// Bit GPIO0.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE0_GPIO0 = 0x1

	// IRQSUMMARY_PROC1_NONSECURE1
	// Position of GPIO47 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO47_Pos = 0xf
	// Bit mask of GPIO47 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO47_Msk = 0x8000
	// Bit GPIO47.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO47 = 0x8000
	// Position of GPIO46 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO46_Pos = 0xe
	// Bit mask of GPIO46 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO46_Msk = 0x4000
	// Bit GPIO46.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO46 = 0x4000
	// Position of GPIO45 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO45_Pos = 0xd
	// Bit mask of GPIO45 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO45_Msk = 0x2000
	// Bit GPIO45.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO45 = 0x2000
	// Position of GPIO44 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO44_Pos = 0xc
	// Bit mask of GPIO44 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO44_Msk = 0x1000
	// Bit GPIO44.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO44 = 0x1000
	// Position of GPIO43 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO43_Pos = 0xb
	// Bit mask of GPIO43 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO43_Msk = 0x800
	// Bit GPIO43.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO43 = 0x800
	// Position of GPIO42 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO42_Pos = 0xa
	// Bit mask of GPIO42 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO42_Msk = 0x400
	// Bit GPIO42.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO42 = 0x400
	// Position of GPIO41 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO41_Pos = 0x9
	// Bit mask of GPIO41 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO41_Msk = 0x200
	// Bit GPIO41.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO41 = 0x200
	// Position of GPIO40 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO40_Pos = 0x8
	// Bit mask of GPIO40 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO40_Msk = 0x100
	// Bit GPIO40.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO40 = 0x100
	// Position of GPIO39 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO39_Pos = 0x7
	// Bit mask of GPIO39 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO39_Msk = 0x80
	// Bit GPIO39.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO39 = 0x80
	// Position of GPIO38 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO38_Pos = 0x6
	// Bit mask of GPIO38 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO38_Msk = 0x40
	// Bit GPIO38.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO38 = 0x40
	// Position of GPIO37 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO37_Pos = 0x5
	// Bit mask of GPIO37 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO37_Msk = 0x20
	// Bit GPIO37.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO37 = 0x20
	// Position of GPIO36 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO36_Pos = 0x4
	// Bit mask of GPIO36 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO36_Msk = 0x10
	// Bit GPIO36.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO36 = 0x10
	// Position of GPIO35 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO35_Pos = 0x3
	// Bit mask of GPIO35 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO35_Msk = 0x8
	// Bit GPIO35.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO35 = 0x8
	// Position of GPIO34 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO34_Pos = 0x2
	// Bit mask of GPIO34 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO34_Msk = 0x4
	// Bit GPIO34.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO34 = 0x4
	// Position of GPIO33 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO33_Pos = 0x1
	// Bit mask of GPIO33 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO33_Msk = 0x2
	// Bit GPIO33.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO33 = 0x2
	// Position of GPIO32 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO32_Pos = 0x0
	// Bit mask of GPIO32 field.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO32_Msk = 0x1
	// Bit GPIO32.
	IO_BANK0_IRQSUMMARY_PROC1_NONSECURE1_GPIO32 = 0x1

	// IRQSUMMARY_DORMANT_WAKE_SECURE0
	// Position of GPIO31 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO31_Pos = 0x1f
	// Bit mask of GPIO31 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO31_Msk = 0x80000000
	// Bit GPIO31.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO31 = 0x80000000
	// Position of GPIO30 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO30_Pos = 0x1e
	// Bit mask of GPIO30 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO30_Msk = 0x40000000
	// Bit GPIO30.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO30 = 0x40000000
	// Position of GPIO29 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO29_Pos = 0x1d
	// Bit mask of GPIO29 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO29_Msk = 0x20000000
	// Bit GPIO29.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO29 = 0x20000000
	// Position of GPIO28 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO28_Pos = 0x1c
	// Bit mask of GPIO28 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO28_Msk = 0x10000000
	// Bit GPIO28.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO28 = 0x10000000
	// Position of GPIO27 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO27_Pos = 0x1b
	// Bit mask of GPIO27 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO27_Msk = 0x8000000
	// Bit GPIO27.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO27 = 0x8000000
	// Position of GPIO26 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO26_Pos = 0x1a
	// Bit mask of GPIO26 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO26_Msk = 0x4000000
	// Bit GPIO26.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO26 = 0x4000000
	// Position of GPIO25 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO25_Pos = 0x19
	// Bit mask of GPIO25 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO25_Msk = 0x2000000
	// Bit GPIO25.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO25 = 0x2000000
	// Position of GPIO24 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO24_Pos = 0x18
	// Bit mask of GPIO24 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO24_Msk = 0x1000000
	// Bit GPIO24.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO24 = 0x1000000
	// Position of GPIO23 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO23_Pos = 0x17
	// Bit mask of GPIO23 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO23_Msk = 0x800000
	// Bit GPIO23.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO23 = 0x800000
	// Position of GPIO22 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO22_Pos = 0x16
	// Bit mask of GPIO22 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO22_Msk = 0x400000
	// Bit GPIO22.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO22 = 0x400000
	// Position of GPIO21 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO21_Pos = 0x15
	// Bit mask of GPIO21 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO21_Msk = 0x200000
	// Bit GPIO21.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO21 = 0x200000
	// Position of GPIO20 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO20_Pos = 0x14
	// Bit mask of GPIO20 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO20_Msk = 0x100000
	// Bit GPIO20.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO20 = 0x100000
	// Position of GPIO19 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO19_Pos = 0x13
	// Bit mask of GPIO19 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO19_Msk = 0x80000
	// Bit GPIO19.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO19 = 0x80000
	// Position of GPIO18 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO18_Pos = 0x12
	// Bit mask of GPIO18 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO18_Msk = 0x40000
	// Bit GPIO18.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO18 = 0x40000
	// Position of GPIO17 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO17_Pos = 0x11
	// Bit mask of GPIO17 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO17_Msk = 0x20000
	// Bit GPIO17.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO17 = 0x20000
	// Position of GPIO16 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO16_Pos = 0x10
	// Bit mask of GPIO16 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO16_Msk = 0x10000
	// Bit GPIO16.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO16 = 0x10000
	// Position of GPIO15 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO15_Pos = 0xf
	// Bit mask of GPIO15 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO15_Msk = 0x8000
	// Bit GPIO15.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO15 = 0x8000
	// Position of GPIO14 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO14_Pos = 0xe
	// Bit mask of GPIO14 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO14_Msk = 0x4000
	// Bit GPIO14.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO14 = 0x4000
	// Position of GPIO13 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO13_Pos = 0xd
	// Bit mask of GPIO13 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO13_Msk = 0x2000
	// Bit GPIO13.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO13 = 0x2000
	// Position of GPIO12 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO12_Pos = 0xc
	// Bit mask of GPIO12 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO12_Msk = 0x1000
	// Bit GPIO12.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO12 = 0x1000
	// Position of GPIO11 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO11_Pos = 0xb
	// Bit mask of GPIO11 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO11_Msk = 0x800
	// Bit GPIO11.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO11 = 0x800
	// Position of GPIO10 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO10_Pos = 0xa
	// Bit mask of GPIO10 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO10_Msk = 0x400
	// Bit GPIO10.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO10 = 0x400
	// Position of GPIO9 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO9_Pos = 0x9
	// Bit mask of GPIO9 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO9_Msk = 0x200
	// Bit GPIO9.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO9 = 0x200
	// Position of GPIO8 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO8_Pos = 0x8
	// Bit mask of GPIO8 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO8_Msk = 0x100
	// Bit GPIO8.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO8 = 0x100
	// Position of GPIO7 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO7_Pos = 0x7
	// Bit mask of GPIO7 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO7_Msk = 0x80
	// Bit GPIO7.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO7 = 0x80
	// Position of GPIO6 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO6_Pos = 0x6
	// Bit mask of GPIO6 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO6_Msk = 0x40
	// Bit GPIO6.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO6 = 0x40
	// Position of GPIO5 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO5_Pos = 0x5
	// Bit mask of GPIO5 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO5_Msk = 0x20
	// Bit GPIO5.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO5 = 0x20
	// Position of GPIO4 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO4_Pos = 0x4
	// Bit mask of GPIO4 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO4_Msk = 0x10
	// Bit GPIO4.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO4 = 0x10
	// Position of GPIO3 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO3_Pos = 0x3
	// Bit mask of GPIO3 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO3_Msk = 0x8
	// Bit GPIO3.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO3 = 0x8
	// Position of GPIO2 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO2_Pos = 0x2
	// Bit mask of GPIO2 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO2_Msk = 0x4
	// Bit GPIO2.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO2 = 0x4
	// Position of GPIO1 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO1_Pos = 0x1
	// Bit mask of GPIO1 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO1_Msk = 0x2
	// Bit GPIO1.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO1 = 0x2
	// Position of GPIO0 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO0_Pos = 0x0
	// Bit mask of GPIO0 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO0_Msk = 0x1
	// Bit GPIO0.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE0_GPIO0 = 0x1

	// IRQSUMMARY_DORMANT_WAKE_SECURE1
	// Position of GPIO47 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO47_Pos = 0xf
	// Bit mask of GPIO47 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO47_Msk = 0x8000
	// Bit GPIO47.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO47 = 0x8000
	// Position of GPIO46 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO46_Pos = 0xe
	// Bit mask of GPIO46 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO46_Msk = 0x4000
	// Bit GPIO46.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO46 = 0x4000
	// Position of GPIO45 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO45_Pos = 0xd
	// Bit mask of GPIO45 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO45_Msk = 0x2000
	// Bit GPIO45.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO45 = 0x2000
	// Position of GPIO44 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO44_Pos = 0xc
	// Bit mask of GPIO44 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO44_Msk = 0x1000
	// Bit GPIO44.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO44 = 0x1000
	// Position of GPIO43 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO43_Pos = 0xb
	// Bit mask of GPIO43 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO43_Msk = 0x800
	// Bit GPIO43.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO43 = 0x800
	// Position of GPIO42 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO42_Pos = 0xa
	// Bit mask of GPIO42 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO42_Msk = 0x400
	// Bit GPIO42.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO42 = 0x400
	// Position of GPIO41 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO41_Pos = 0x9
	// Bit mask of GPIO41 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO41_Msk = 0x200
	// Bit GPIO41.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO41 = 0x200
	// Position of GPIO40 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO40_Pos = 0x8
	// Bit mask of GPIO40 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO40_Msk = 0x100
	// Bit GPIO40.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO40 = 0x100
	// Position of GPIO39 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO39_Pos = 0x7
	// Bit mask of GPIO39 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO39_Msk = 0x80
	// Bit GPIO39.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO39 = 0x80
	// Position of GPIO38 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO38_Pos = 0x6
	// Bit mask of GPIO38 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO38_Msk = 0x40
	// Bit GPIO38.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO38 = 0x40
	// Position of GPIO37 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO37_Pos = 0x5
	// Bit mask of GPIO37 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO37_Msk = 0x20
	// Bit GPIO37.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO37 = 0x20
	// Position of GPIO36 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO36_Pos = 0x4
	// Bit mask of GPIO36 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO36_Msk = 0x10
	// Bit GPIO36.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO36 = 0x10
	// Position of GPIO35 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO35_Pos = 0x3
	// Bit mask of GPIO35 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO35_Msk = 0x8
	// Bit GPIO35.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO35 = 0x8
	// Position of GPIO34 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO34_Pos = 0x2
	// Bit mask of GPIO34 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO34_Msk = 0x4
	// Bit GPIO34.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO34 = 0x4
	// Position of GPIO33 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO33_Pos = 0x1
	// Bit mask of GPIO33 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO33_Msk = 0x2
	// Bit GPIO33.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO33 = 0x2
	// Position of GPIO32 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO32_Pos = 0x0
	// Bit mask of GPIO32 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO32_Msk = 0x1
	// Bit GPIO32.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_SECURE1_GPIO32 = 0x1

	// IRQSUMMARY_DORMANT_WAKE_NONSECURE0
	// Position of GPIO31 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO31_Pos = 0x1f
	// Bit mask of GPIO31 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO31_Msk = 0x80000000
	// Bit GPIO31.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO31 = 0x80000000
	// Position of GPIO30 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO30_Pos = 0x1e
	// Bit mask of GPIO30 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO30_Msk = 0x40000000
	// Bit GPIO30.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO30 = 0x40000000
	// Position of GPIO29 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO29_Pos = 0x1d
	// Bit mask of GPIO29 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO29_Msk = 0x20000000
	// Bit GPIO29.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO29 = 0x20000000
	// Position of GPIO28 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO28_Pos = 0x1c
	// Bit mask of GPIO28 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO28_Msk = 0x10000000
	// Bit GPIO28.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO28 = 0x10000000
	// Position of GPIO27 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO27_Pos = 0x1b
	// Bit mask of GPIO27 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO27_Msk = 0x8000000
	// Bit GPIO27.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO27 = 0x8000000
	// Position of GPIO26 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO26_Pos = 0x1a
	// Bit mask of GPIO26 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO26_Msk = 0x4000000
	// Bit GPIO26.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO26 = 0x4000000
	// Position of GPIO25 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO25_Pos = 0x19
	// Bit mask of GPIO25 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO25_Msk = 0x2000000
	// Bit GPIO25.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO25 = 0x2000000
	// Position of GPIO24 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO24_Pos = 0x18
	// Bit mask of GPIO24 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO24_Msk = 0x1000000
	// Bit GPIO24.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO24 = 0x1000000
	// Position of GPIO23 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO23_Pos = 0x17
	// Bit mask of GPIO23 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO23_Msk = 0x800000
	// Bit GPIO23.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO23 = 0x800000
	// Position of GPIO22 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO22_Pos = 0x16
	// Bit mask of GPIO22 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO22_Msk = 0x400000
	// Bit GPIO22.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO22 = 0x400000
	// Position of GPIO21 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO21_Pos = 0x15
	// Bit mask of GPIO21 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO21_Msk = 0x200000
	// Bit GPIO21.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO21 = 0x200000
	// Position of GPIO20 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO20_Pos = 0x14
	// Bit mask of GPIO20 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO20_Msk = 0x100000
	// Bit GPIO20.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO20 = 0x100000
	// Position of GPIO19 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO19_Pos = 0x13
	// Bit mask of GPIO19 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO19_Msk = 0x80000
	// Bit GPIO19.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO19 = 0x80000
	// Position of GPIO18 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO18_Pos = 0x12
	// Bit mask of GPIO18 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO18_Msk = 0x40000
	// Bit GPIO18.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO18 = 0x40000
	// Position of GPIO17 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO17_Pos = 0x11
	// Bit mask of GPIO17 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO17_Msk = 0x20000
	// Bit GPIO17.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO17 = 0x20000
	// Position of GPIO16 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO16_Pos = 0x10
	// Bit mask of GPIO16 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO16_Msk = 0x10000
	// Bit GPIO16.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO16 = 0x10000
	// Position of GPIO15 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO15_Pos = 0xf
	// Bit mask of GPIO15 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO15_Msk = 0x8000
	// Bit GPIO15.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO15 = 0x8000
	// Position of GPIO14 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO14_Pos = 0xe
	// Bit mask of GPIO14 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO14_Msk = 0x4000
	// Bit GPIO14.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO14 = 0x4000
	// Position of GPIO13 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO13_Pos = 0xd
	// Bit mask of GPIO13 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO13_Msk = 0x2000
	// Bit GPIO13.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO13 = 0x2000
	// Position of GPIO12 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO12_Pos = 0xc
	// Bit mask of GPIO12 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO12_Msk = 0x1000
	// Bit GPIO12.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO12 = 0x1000
	// Position of GPIO11 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO11_Pos = 0xb
	// Bit mask of GPIO11 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO11_Msk = 0x800
	// Bit GPIO11.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO11 = 0x800
	// Position of GPIO10 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO10_Pos = 0xa
	// Bit mask of GPIO10 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO10_Msk = 0x400
	// Bit GPIO10.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO10 = 0x400
	// Position of GPIO9 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO9_Pos = 0x9
	// Bit mask of GPIO9 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO9_Msk = 0x200
	// Bit GPIO9.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO9 = 0x200
	// Position of GPIO8 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO8_Pos = 0x8
	// Bit mask of GPIO8 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO8_Msk = 0x100
	// Bit GPIO8.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO8 = 0x100
	// Position of GPIO7 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO7_Pos = 0x7
	// Bit mask of GPIO7 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO7_Msk = 0x80
	// Bit GPIO7.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO7 = 0x80
	// Position of GPIO6 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO6_Pos = 0x6
	// Bit mask of GPIO6 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO6_Msk = 0x40
	// Bit GPIO6.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO6 = 0x40
	// Position of GPIO5 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO5_Pos = 0x5
	// Bit mask of GPIO5 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO5_Msk = 0x20
	// Bit GPIO5.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO5 = 0x20
	// Position of GPIO4 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO4_Pos = 0x4
	// Bit mask of GPIO4 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO4_Msk = 0x10
	// Bit GPIO4.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO4 = 0x10
	// Position of GPIO3 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO3_Pos = 0x3
	// Bit mask of GPIO3 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO3_Msk = 0x8
	// Bit GPIO3.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO3 = 0x8
	// Position of GPIO2 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO2_Pos = 0x2
	// Bit mask of GPIO2 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO2_Msk = 0x4
	// Bit GPIO2.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO2 = 0x4
	// Position of GPIO1 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO1_Pos = 0x1
	// Bit mask of GPIO1 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO1_Msk = 0x2
	// Bit GPIO1.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO1 = 0x2
	// Position of GPIO0 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO0_Pos = 0x0
	// Bit mask of GPIO0 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO0_Msk = 0x1
	// Bit GPIO0.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE0_GPIO0 = 0x1

	// IRQSUMMARY_DORMANT_WAKE_NONSECURE1
	// Position of GPIO47 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO47_Pos = 0xf
	// Bit mask of GPIO47 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO47_Msk = 0x8000
	// Bit GPIO47.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO47 = 0x8000
	// Position of GPIO46 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO46_Pos = 0xe
	// Bit mask of GPIO46 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO46_Msk = 0x4000
	// Bit GPIO46.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO46 = 0x4000
	// Position of GPIO45 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO45_Pos = 0xd
	// Bit mask of GPIO45 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO45_Msk = 0x2000
	// Bit GPIO45.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO45 = 0x2000
	// Position of GPIO44 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO44_Pos = 0xc
	// Bit mask of GPIO44 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO44_Msk = 0x1000
	// Bit GPIO44.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO44 = 0x1000
	// Position of GPIO43 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO43_Pos = 0xb
	// Bit mask of GPIO43 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO43_Msk = 0x800
	// Bit GPIO43.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO43 = 0x800
	// Position of GPIO42 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO42_Pos = 0xa
	// Bit mask of GPIO42 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO42_Msk = 0x400
	// Bit GPIO42.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO42 = 0x400
	// Position of GPIO41 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO41_Pos = 0x9
	// Bit mask of GPIO41 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO41_Msk = 0x200
	// Bit GPIO41.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO41 = 0x200
	// Position of GPIO40 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO40_Pos = 0x8
	// Bit mask of GPIO40 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO40_Msk = 0x100
	// Bit GPIO40.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO40 = 0x100
	// Position of GPIO39 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO39_Pos = 0x7
	// Bit mask of GPIO39 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO39_Msk = 0x80
	// Bit GPIO39.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO39 = 0x80
	// Position of GPIO38 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO38_Pos = 0x6
	// Bit mask of GPIO38 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO38_Msk = 0x40
	// Bit GPIO38.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO38 = 0x40
	// Position of GPIO37 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO37_Pos = 0x5
	// Bit mask of GPIO37 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO37_Msk = 0x20
	// Bit GPIO37.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO37 = 0x20
	// Position of GPIO36 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO36_Pos = 0x4
	// Bit mask of GPIO36 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO36_Msk = 0x10
	// Bit GPIO36.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO36 = 0x10
	// Position of GPIO35 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO35_Pos = 0x3
	// Bit mask of GPIO35 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO35_Msk = 0x8
	// Bit GPIO35.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO35 = 0x8
	// Position of GPIO34 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO34_Pos = 0x2
	// Bit mask of GPIO34 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO34_Msk = 0x4
	// Bit GPIO34.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO34 = 0x4
	// Position of GPIO33 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO33_Pos = 0x1
	// Bit mask of GPIO33 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO33_Msk = 0x2
	// Bit GPIO33.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO33 = 0x2
	// Position of GPIO32 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO32_Pos = 0x0
	// Bit mask of GPIO32 field.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO32_Msk = 0x1
	// Bit GPIO32.
	IO_BANK0_IRQSUMMARY_DORMANT_WAKE_NONSECURE1_GPIO32 = 0x1

	// INTR0: Raw Interrupts
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_INTR0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_INTR0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_INTR0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_INTR0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_INTR0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_INTR0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_INTR0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_INTR0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_INTR0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_INTR0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_INTR0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_INTR0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_INTR0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_INTR0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_INTR0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_INTR0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_INTR0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_INTR0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_INTR0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_INTR0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_INTR0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_INTR0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_INTR0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_INTR0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_INTR0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_INTR0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_INTR0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_INTR0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_INTR0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_INTR0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_INTR0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_INTR0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_INTR0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_INTR0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_INTR0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_INTR0_GPIO0_LEVEL_LOW = 0x1

	// INTR1: Raw Interrupts
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_INTR1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_INTR1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_INTR1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_INTR1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_INTR1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_INTR1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_INTR1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_INTR1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_INTR1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_INTR1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_INTR1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_INTR1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_INTR1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_INTR1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_INTR1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_INTR1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_INTR1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_INTR1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_INTR1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_INTR1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_INTR1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_INTR1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_INTR1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_INTR1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_INTR1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_INTR1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_INTR1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_INTR1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_INTR1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_INTR1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_INTR1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_INTR1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_INTR1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_INTR1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_INTR1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_INTR1_GPIO8_LEVEL_LOW = 0x1

	// INTR2: Raw Interrupts
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_INTR2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_INTR2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_INTR2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_INTR2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_INTR2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_INTR2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_INTR2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_INTR2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_INTR2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_INTR2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_INTR2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_INTR2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_INTR2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_INTR2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_INTR2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_INTR2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_INTR2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_INTR2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_INTR2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_INTR2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_INTR2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_INTR2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_INTR2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_INTR2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_INTR2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_INTR2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_INTR2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_INTR2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_INTR2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_INTR2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_INTR2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_INTR2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_INTR2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_INTR2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_INTR2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_INTR2_GPIO16_LEVEL_LOW = 0x1

	// INTR3: Raw Interrupts
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_INTR3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_INTR3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_INTR3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_INTR3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_INTR3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_INTR3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_INTR3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_INTR3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_INTR3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_INTR3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_INTR3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_INTR3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_INTR3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_INTR3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_INTR3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_INTR3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_INTR3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_INTR3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_INTR3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_INTR3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_INTR3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_INTR3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_INTR3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_INTR3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_INTR3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_INTR3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_INTR3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_INTR3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_INTR3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_INTR3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_INTR3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_INTR3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_INTR3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_INTR3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_INTR3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_INTR3_GPIO24_LEVEL_LOW = 0x1

	// INTR4: Raw Interrupts
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_INTR4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_INTR4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_INTR4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_INTR4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_INTR4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_INTR4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_INTR4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_INTR4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_INTR4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_INTR4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_INTR4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_INTR4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_INTR4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_INTR4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_INTR4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_INTR4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_INTR4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_INTR4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_INTR4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_INTR4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_INTR4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_INTR4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_INTR4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_INTR4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_INTR4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_INTR4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_INTR4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_INTR4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_INTR4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_INTR4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_INTR4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_INTR4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_INTR4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_INTR4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_INTR4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_INTR4_GPIO32_LEVEL_LOW = 0x1

	// INTR5: Raw Interrupts
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_INTR5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_INTR5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_INTR5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_INTR5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_INTR5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_INTR5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_INTR5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_INTR5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_INTR5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_INTR5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_INTR5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_INTR5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_INTR5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_INTR5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_INTR5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_INTR5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_INTR5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_INTR5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_INTR5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_INTR5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_INTR5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_INTR5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_INTR5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_INTR5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_INTR5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_INTR5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_INTR5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_INTR5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_INTR5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_INTR5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_INTR5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_INTR5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_INTR5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_INTR5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_INTR5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_INTR5_GPIO40_LEVEL_LOW = 0x1

	// PROC0_INTE0: Interrupt Enable for proc0
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_PROC0_INTE0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_PROC0_INTE0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_PROC0_INTE0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_PROC0_INTE0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_PROC0_INTE0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_PROC0_INTE0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_PROC0_INTE0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_PROC0_INTE0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_PROC0_INTE0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_PROC0_INTE0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_PROC0_INTE0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_PROC0_INTE0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_PROC0_INTE0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_PROC0_INTE0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_PROC0_INTE0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_PROC0_INTE0_GPIO0_LEVEL_LOW = 0x1

	// PROC0_INTE1: Interrupt Enable for proc0
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_PROC0_INTE1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_PROC0_INTE1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_PROC0_INTE1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_PROC0_INTE1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_PROC0_INTE1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_PROC0_INTE1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_PROC0_INTE1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_PROC0_INTE1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_PROC0_INTE1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_PROC0_INTE1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_PROC0_INTE1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_PROC0_INTE1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_PROC0_INTE1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_PROC0_INTE1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_PROC0_INTE1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_PROC0_INTE1_GPIO8_LEVEL_LOW = 0x1

	// PROC0_INTE2: Interrupt Enable for proc0
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_PROC0_INTE2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_PROC0_INTE2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_PROC0_INTE2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_PROC0_INTE2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_PROC0_INTE2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_PROC0_INTE2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_PROC0_INTE2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_PROC0_INTE2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_PROC0_INTE2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_PROC0_INTE2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_PROC0_INTE2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_PROC0_INTE2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_PROC0_INTE2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_PROC0_INTE2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_PROC0_INTE2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_PROC0_INTE2_GPIO16_LEVEL_LOW = 0x1

	// PROC0_INTE3: Interrupt Enable for proc0
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_PROC0_INTE3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_PROC0_INTE3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_PROC0_INTE3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_PROC0_INTE3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_PROC0_INTE3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_PROC0_INTE3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_PROC0_INTE3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_PROC0_INTE3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_PROC0_INTE3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_PROC0_INTE3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_PROC0_INTE3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_PROC0_INTE3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_PROC0_INTE3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_PROC0_INTE3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_PROC0_INTE3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_PROC0_INTE3_GPIO24_LEVEL_LOW = 0x1

	// PROC0_INTE4: Interrupt Enable for proc0
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_PROC0_INTE4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_PROC0_INTE4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_PROC0_INTE4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_PROC0_INTE4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_PROC0_INTE4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_PROC0_INTE4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_PROC0_INTE4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_PROC0_INTE4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_PROC0_INTE4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_PROC0_INTE4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_PROC0_INTE4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_PROC0_INTE4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_PROC0_INTE4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_PROC0_INTE4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_PROC0_INTE4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_PROC0_INTE4_GPIO32_LEVEL_LOW = 0x1

	// PROC0_INTE5: Interrupt Enable for proc0
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_PROC0_INTE5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_PROC0_INTE5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_PROC0_INTE5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_PROC0_INTE5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_PROC0_INTE5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_PROC0_INTE5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_PROC0_INTE5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_PROC0_INTE5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_PROC0_INTE5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_PROC0_INTE5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_PROC0_INTE5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_PROC0_INTE5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_PROC0_INTE5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_PROC0_INTE5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_PROC0_INTE5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTE5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_PROC0_INTE5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC0_INTE5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_PROC0_INTE5_GPIO40_LEVEL_LOW = 0x1

	// PROC0_INTF0: Interrupt Force for proc0
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_PROC0_INTF0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_PROC0_INTF0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_PROC0_INTF0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_PROC0_INTF0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_PROC0_INTF0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_PROC0_INTF0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_PROC0_INTF0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_PROC0_INTF0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_PROC0_INTF0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_PROC0_INTF0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_PROC0_INTF0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_PROC0_INTF0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_PROC0_INTF0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_PROC0_INTF0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_PROC0_INTF0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_PROC0_INTF0_GPIO0_LEVEL_LOW = 0x1

	// PROC0_INTF1: Interrupt Force for proc0
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_PROC0_INTF1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_PROC0_INTF1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_PROC0_INTF1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_PROC0_INTF1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_PROC0_INTF1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_PROC0_INTF1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_PROC0_INTF1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_PROC0_INTF1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_PROC0_INTF1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_PROC0_INTF1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_PROC0_INTF1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_PROC0_INTF1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_PROC0_INTF1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_PROC0_INTF1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_PROC0_INTF1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_PROC0_INTF1_GPIO8_LEVEL_LOW = 0x1

	// PROC0_INTF2: Interrupt Force for proc0
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_PROC0_INTF2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_PROC0_INTF2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_PROC0_INTF2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_PROC0_INTF2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_PROC0_INTF2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_PROC0_INTF2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_PROC0_INTF2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_PROC0_INTF2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_PROC0_INTF2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_PROC0_INTF2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_PROC0_INTF2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_PROC0_INTF2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_PROC0_INTF2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_PROC0_INTF2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_PROC0_INTF2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_PROC0_INTF2_GPIO16_LEVEL_LOW = 0x1

	// PROC0_INTF3: Interrupt Force for proc0
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_PROC0_INTF3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_PROC0_INTF3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_PROC0_INTF3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_PROC0_INTF3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_PROC0_INTF3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_PROC0_INTF3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_PROC0_INTF3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_PROC0_INTF3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_PROC0_INTF3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_PROC0_INTF3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_PROC0_INTF3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_PROC0_INTF3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_PROC0_INTF3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_PROC0_INTF3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_PROC0_INTF3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_PROC0_INTF3_GPIO24_LEVEL_LOW = 0x1

	// PROC0_INTF4: Interrupt Force for proc0
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_PROC0_INTF4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_PROC0_INTF4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_PROC0_INTF4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_PROC0_INTF4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_PROC0_INTF4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_PROC0_INTF4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_PROC0_INTF4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_PROC0_INTF4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_PROC0_INTF4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_PROC0_INTF4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_PROC0_INTF4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_PROC0_INTF4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_PROC0_INTF4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_PROC0_INTF4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_PROC0_INTF4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_PROC0_INTF4_GPIO32_LEVEL_LOW = 0x1

	// PROC0_INTF5: Interrupt Force for proc0
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_PROC0_INTF5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_PROC0_INTF5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_PROC0_INTF5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_PROC0_INTF5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_PROC0_INTF5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_PROC0_INTF5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_PROC0_INTF5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_PROC0_INTF5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_PROC0_INTF5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_PROC0_INTF5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_PROC0_INTF5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_PROC0_INTF5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_PROC0_INTF5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_PROC0_INTF5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_PROC0_INTF5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTF5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_PROC0_INTF5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC0_INTF5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_PROC0_INTF5_GPIO40_LEVEL_LOW = 0x1

	// PROC0_INTS0: Interrupt status after masking & forcing for proc0
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_PROC0_INTS0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_PROC0_INTS0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_PROC0_INTS0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_PROC0_INTS0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_PROC0_INTS0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_PROC0_INTS0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_PROC0_INTS0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_PROC0_INTS0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_PROC0_INTS0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_PROC0_INTS0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_PROC0_INTS0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_PROC0_INTS0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_PROC0_INTS0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_PROC0_INTS0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_PROC0_INTS0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_PROC0_INTS0_GPIO0_LEVEL_LOW = 0x1

	// PROC0_INTS1: Interrupt status after masking & forcing for proc0
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_PROC0_INTS1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_PROC0_INTS1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_PROC0_INTS1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_PROC0_INTS1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_PROC0_INTS1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_PROC0_INTS1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_PROC0_INTS1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_PROC0_INTS1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_PROC0_INTS1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_PROC0_INTS1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_PROC0_INTS1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_PROC0_INTS1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_PROC0_INTS1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_PROC0_INTS1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_PROC0_INTS1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_PROC0_INTS1_GPIO8_LEVEL_LOW = 0x1

	// PROC0_INTS2: Interrupt status after masking & forcing for proc0
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_PROC0_INTS2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_PROC0_INTS2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_PROC0_INTS2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_PROC0_INTS2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_PROC0_INTS2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_PROC0_INTS2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_PROC0_INTS2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_PROC0_INTS2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_PROC0_INTS2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_PROC0_INTS2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_PROC0_INTS2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_PROC0_INTS2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_PROC0_INTS2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_PROC0_INTS2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_PROC0_INTS2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_PROC0_INTS2_GPIO16_LEVEL_LOW = 0x1

	// PROC0_INTS3: Interrupt status after masking & forcing for proc0
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_PROC0_INTS3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_PROC0_INTS3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_PROC0_INTS3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_PROC0_INTS3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_PROC0_INTS3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_PROC0_INTS3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_PROC0_INTS3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_PROC0_INTS3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_PROC0_INTS3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_PROC0_INTS3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_PROC0_INTS3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_PROC0_INTS3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_PROC0_INTS3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_PROC0_INTS3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_PROC0_INTS3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_PROC0_INTS3_GPIO24_LEVEL_LOW = 0x1

	// PROC0_INTS4: Interrupt status after masking & forcing for proc0
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_PROC0_INTS4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_PROC0_INTS4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_PROC0_INTS4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_PROC0_INTS4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_PROC0_INTS4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_PROC0_INTS4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_PROC0_INTS4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_PROC0_INTS4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_PROC0_INTS4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_PROC0_INTS4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_PROC0_INTS4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_PROC0_INTS4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_PROC0_INTS4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_PROC0_INTS4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_PROC0_INTS4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_PROC0_INTS4_GPIO32_LEVEL_LOW = 0x1

	// PROC0_INTS5: Interrupt status after masking & forcing for proc0
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_PROC0_INTS5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_PROC0_INTS5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_PROC0_INTS5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_PROC0_INTS5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_PROC0_INTS5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_PROC0_INTS5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_PROC0_INTS5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_PROC0_INTS5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_PROC0_INTS5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_PROC0_INTS5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_PROC0_INTS5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_PROC0_INTS5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_PROC0_INTS5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_PROC0_INTS5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_PROC0_INTS5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC0_INTS5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_PROC0_INTS5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC0_INTS5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_PROC0_INTS5_GPIO40_LEVEL_LOW = 0x1

	// PROC1_INTE0: Interrupt Enable for proc1
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_PROC1_INTE0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_PROC1_INTE0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_PROC1_INTE0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_PROC1_INTE0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_PROC1_INTE0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_PROC1_INTE0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_PROC1_INTE0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_PROC1_INTE0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_PROC1_INTE0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_PROC1_INTE0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_PROC1_INTE0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_PROC1_INTE0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_PROC1_INTE0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_PROC1_INTE0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_PROC1_INTE0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_PROC1_INTE0_GPIO0_LEVEL_LOW = 0x1

	// PROC1_INTE1: Interrupt Enable for proc1
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_PROC1_INTE1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_PROC1_INTE1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_PROC1_INTE1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_PROC1_INTE1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_PROC1_INTE1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_PROC1_INTE1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_PROC1_INTE1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_PROC1_INTE1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_PROC1_INTE1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_PROC1_INTE1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_PROC1_INTE1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_PROC1_INTE1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_PROC1_INTE1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_PROC1_INTE1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_PROC1_INTE1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_PROC1_INTE1_GPIO8_LEVEL_LOW = 0x1

	// PROC1_INTE2: Interrupt Enable for proc1
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_PROC1_INTE2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_PROC1_INTE2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_PROC1_INTE2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_PROC1_INTE2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_PROC1_INTE2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_PROC1_INTE2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_PROC1_INTE2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_PROC1_INTE2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_PROC1_INTE2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_PROC1_INTE2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_PROC1_INTE2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_PROC1_INTE2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_PROC1_INTE2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_PROC1_INTE2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_PROC1_INTE2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_PROC1_INTE2_GPIO16_LEVEL_LOW = 0x1

	// PROC1_INTE3: Interrupt Enable for proc1
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_PROC1_INTE3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_PROC1_INTE3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_PROC1_INTE3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_PROC1_INTE3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_PROC1_INTE3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_PROC1_INTE3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_PROC1_INTE3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_PROC1_INTE3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_PROC1_INTE3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_PROC1_INTE3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_PROC1_INTE3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_PROC1_INTE3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_PROC1_INTE3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_PROC1_INTE3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_PROC1_INTE3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_PROC1_INTE3_GPIO24_LEVEL_LOW = 0x1

	// PROC1_INTE4: Interrupt Enable for proc1
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_PROC1_INTE4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_PROC1_INTE4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_PROC1_INTE4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_PROC1_INTE4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_PROC1_INTE4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_PROC1_INTE4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_PROC1_INTE4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_PROC1_INTE4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_PROC1_INTE4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_PROC1_INTE4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_PROC1_INTE4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_PROC1_INTE4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_PROC1_INTE4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_PROC1_INTE4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_PROC1_INTE4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_PROC1_INTE4_GPIO32_LEVEL_LOW = 0x1

	// PROC1_INTE5: Interrupt Enable for proc1
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_PROC1_INTE5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_PROC1_INTE5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_PROC1_INTE5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_PROC1_INTE5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_PROC1_INTE5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_PROC1_INTE5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_PROC1_INTE5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_PROC1_INTE5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_PROC1_INTE5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_PROC1_INTE5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_PROC1_INTE5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_PROC1_INTE5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_PROC1_INTE5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_PROC1_INTE5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_PROC1_INTE5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTE5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_PROC1_INTE5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC1_INTE5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_PROC1_INTE5_GPIO40_LEVEL_LOW = 0x1

	// PROC1_INTF0: Interrupt Force for proc1
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_PROC1_INTF0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_PROC1_INTF0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_PROC1_INTF0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_PROC1_INTF0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_PROC1_INTF0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_PROC1_INTF0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_PROC1_INTF0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_PROC1_INTF0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_PROC1_INTF0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_PROC1_INTF0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_PROC1_INTF0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_PROC1_INTF0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_PROC1_INTF0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_PROC1_INTF0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_PROC1_INTF0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_PROC1_INTF0_GPIO0_LEVEL_LOW = 0x1

	// PROC1_INTF1: Interrupt Force for proc1
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_PROC1_INTF1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_PROC1_INTF1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_PROC1_INTF1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_PROC1_INTF1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_PROC1_INTF1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_PROC1_INTF1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_PROC1_INTF1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_PROC1_INTF1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_PROC1_INTF1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_PROC1_INTF1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_PROC1_INTF1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_PROC1_INTF1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_PROC1_INTF1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_PROC1_INTF1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_PROC1_INTF1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_PROC1_INTF1_GPIO8_LEVEL_LOW = 0x1

	// PROC1_INTF2: Interrupt Force for proc1
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_PROC1_INTF2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_PROC1_INTF2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_PROC1_INTF2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_PROC1_INTF2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_PROC1_INTF2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_PROC1_INTF2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_PROC1_INTF2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_PROC1_INTF2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_PROC1_INTF2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_PROC1_INTF2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_PROC1_INTF2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_PROC1_INTF2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_PROC1_INTF2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_PROC1_INTF2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_PROC1_INTF2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_PROC1_INTF2_GPIO16_LEVEL_LOW = 0x1

	// PROC1_INTF3: Interrupt Force for proc1
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_PROC1_INTF3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_PROC1_INTF3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_PROC1_INTF3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_PROC1_INTF3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_PROC1_INTF3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_PROC1_INTF3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_PROC1_INTF3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_PROC1_INTF3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_PROC1_INTF3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_PROC1_INTF3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_PROC1_INTF3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_PROC1_INTF3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_PROC1_INTF3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_PROC1_INTF3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_PROC1_INTF3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_PROC1_INTF3_GPIO24_LEVEL_LOW = 0x1

	// PROC1_INTF4: Interrupt Force for proc1
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_PROC1_INTF4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_PROC1_INTF4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_PROC1_INTF4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_PROC1_INTF4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_PROC1_INTF4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_PROC1_INTF4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_PROC1_INTF4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_PROC1_INTF4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_PROC1_INTF4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_PROC1_INTF4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_PROC1_INTF4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_PROC1_INTF4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_PROC1_INTF4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_PROC1_INTF4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_PROC1_INTF4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_PROC1_INTF4_GPIO32_LEVEL_LOW = 0x1

	// PROC1_INTF5: Interrupt Force for proc1
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_PROC1_INTF5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_PROC1_INTF5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_PROC1_INTF5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_PROC1_INTF5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_PROC1_INTF5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_PROC1_INTF5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_PROC1_INTF5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_PROC1_INTF5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_PROC1_INTF5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_PROC1_INTF5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_PROC1_INTF5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_PROC1_INTF5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_PROC1_INTF5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_PROC1_INTF5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_PROC1_INTF5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTF5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_PROC1_INTF5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC1_INTF5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_PROC1_INTF5_GPIO40_LEVEL_LOW = 0x1

	// PROC1_INTS0: Interrupt status after masking & forcing for proc1
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_PROC1_INTS0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_PROC1_INTS0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_PROC1_INTS0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_PROC1_INTS0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_PROC1_INTS0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_PROC1_INTS0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_PROC1_INTS0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_PROC1_INTS0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_PROC1_INTS0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_PROC1_INTS0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_PROC1_INTS0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_PROC1_INTS0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_PROC1_INTS0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_PROC1_INTS0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_PROC1_INTS0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_PROC1_INTS0_GPIO0_LEVEL_LOW = 0x1

	// PROC1_INTS1: Interrupt status after masking & forcing for proc1
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_PROC1_INTS1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_PROC1_INTS1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_PROC1_INTS1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_PROC1_INTS1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_PROC1_INTS1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_PROC1_INTS1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_PROC1_INTS1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_PROC1_INTS1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_PROC1_INTS1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_PROC1_INTS1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_PROC1_INTS1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_PROC1_INTS1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_PROC1_INTS1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_PROC1_INTS1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_PROC1_INTS1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_PROC1_INTS1_GPIO8_LEVEL_LOW = 0x1

	// PROC1_INTS2: Interrupt status after masking & forcing for proc1
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_PROC1_INTS2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_PROC1_INTS2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_PROC1_INTS2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_PROC1_INTS2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_PROC1_INTS2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_PROC1_INTS2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_PROC1_INTS2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_PROC1_INTS2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_PROC1_INTS2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_PROC1_INTS2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_PROC1_INTS2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_PROC1_INTS2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_PROC1_INTS2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_PROC1_INTS2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_PROC1_INTS2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_PROC1_INTS2_GPIO16_LEVEL_LOW = 0x1

	// PROC1_INTS3: Interrupt status after masking & forcing for proc1
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_PROC1_INTS3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_PROC1_INTS3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_PROC1_INTS3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_PROC1_INTS3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_PROC1_INTS3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_PROC1_INTS3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_PROC1_INTS3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_PROC1_INTS3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_PROC1_INTS3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_PROC1_INTS3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_PROC1_INTS3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_PROC1_INTS3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_PROC1_INTS3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_PROC1_INTS3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_PROC1_INTS3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_PROC1_INTS3_GPIO24_LEVEL_LOW = 0x1

	// PROC1_INTS4: Interrupt status after masking & forcing for proc1
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_PROC1_INTS4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_PROC1_INTS4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_PROC1_INTS4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_PROC1_INTS4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_PROC1_INTS4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_PROC1_INTS4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_PROC1_INTS4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_PROC1_INTS4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_PROC1_INTS4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_PROC1_INTS4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_PROC1_INTS4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_PROC1_INTS4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_PROC1_INTS4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_PROC1_INTS4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_PROC1_INTS4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_PROC1_INTS4_GPIO32_LEVEL_LOW = 0x1

	// PROC1_INTS5: Interrupt status after masking & forcing for proc1
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_PROC1_INTS5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_PROC1_INTS5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_PROC1_INTS5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_PROC1_INTS5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_PROC1_INTS5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_PROC1_INTS5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_PROC1_INTS5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_PROC1_INTS5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_PROC1_INTS5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_PROC1_INTS5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_PROC1_INTS5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_PROC1_INTS5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_PROC1_INTS5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_PROC1_INTS5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_PROC1_INTS5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_PROC1_INTS5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_PROC1_INTS5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_PROC1_INTS5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_PROC1_INTS5_GPIO40_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTE0: Interrupt Enable for dormant_wake
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE0_GPIO0_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTE1: Interrupt Enable for dormant_wake
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE1_GPIO8_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTE2: Interrupt Enable for dormant_wake
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE2_GPIO16_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTE3: Interrupt Enable for dormant_wake
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE3_GPIO24_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTE4: Interrupt Enable for dormant_wake
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE4_GPIO32_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTE5: Interrupt Enable for dormant_wake
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTE5_GPIO40_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTF0: Interrupt Force for dormant_wake
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF0_GPIO0_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTF1: Interrupt Force for dormant_wake
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF1_GPIO8_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTF2: Interrupt Force for dormant_wake
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF2_GPIO16_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTF3: Interrupt Force for dormant_wake
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF3_GPIO24_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTF4: Interrupt Force for dormant_wake
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF4_GPIO32_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTF5: Interrupt Force for dormant_wake
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTF5_GPIO40_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTS0: Interrupt status after masking & forcing for dormant_wake
	// Position of GPIO7_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO7_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO7_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_EDGE_HIGH = 0x80000000
	// Position of GPIO7_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO7_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO7_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_EDGE_LOW = 0x40000000
	// Position of GPIO7_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO7_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO7_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_LEVEL_HIGH = 0x20000000
	// Position of GPIO7_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO7_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO7_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO7_LEVEL_LOW = 0x10000000
	// Position of GPIO6_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO6_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO6_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_EDGE_HIGH = 0x8000000
	// Position of GPIO6_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO6_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO6_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_EDGE_LOW = 0x4000000
	// Position of GPIO6_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO6_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO6_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_LEVEL_HIGH = 0x2000000
	// Position of GPIO6_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO6_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO6_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO6_LEVEL_LOW = 0x1000000
	// Position of GPIO5_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO5_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO5_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_EDGE_HIGH = 0x800000
	// Position of GPIO5_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO5_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_EDGE_LOW_Msk = 0x400000
	// Bit GPIO5_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_EDGE_LOW = 0x400000
	// Position of GPIO5_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO5_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO5_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_LEVEL_HIGH = 0x200000
	// Position of GPIO5_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO5_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO5_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO5_LEVEL_LOW = 0x100000
	// Position of GPIO4_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO4_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO4_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_EDGE_HIGH = 0x80000
	// Position of GPIO4_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO4_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_EDGE_LOW_Msk = 0x40000
	// Bit GPIO4_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_EDGE_LOW = 0x40000
	// Position of GPIO4_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO4_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO4_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_LEVEL_HIGH = 0x20000
	// Position of GPIO4_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO4_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO4_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO4_LEVEL_LOW = 0x10000
	// Position of GPIO3_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO3_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO3_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_EDGE_HIGH = 0x8000
	// Position of GPIO3_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO3_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_EDGE_LOW_Msk = 0x4000
	// Bit GPIO3_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_EDGE_LOW = 0x4000
	// Position of GPIO3_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO3_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO3_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_LEVEL_HIGH = 0x2000
	// Position of GPIO3_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO3_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO3_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO3_LEVEL_LOW = 0x1000
	// Position of GPIO2_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO2_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_EDGE_HIGH_Msk = 0x800
	// Bit GPIO2_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_EDGE_HIGH = 0x800
	// Position of GPIO2_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO2_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_EDGE_LOW_Msk = 0x400
	// Bit GPIO2_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_EDGE_LOW = 0x400
	// Position of GPIO2_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO2_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO2_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_LEVEL_HIGH = 0x200
	// Position of GPIO2_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO2_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_LEVEL_LOW_Msk = 0x100
	// Bit GPIO2_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO2_LEVEL_LOW = 0x100
	// Position of GPIO1_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO1_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_EDGE_HIGH_Msk = 0x80
	// Bit GPIO1_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_EDGE_HIGH = 0x80
	// Position of GPIO1_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO1_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_EDGE_LOW_Msk = 0x40
	// Bit GPIO1_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_EDGE_LOW = 0x40
	// Position of GPIO1_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO1_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO1_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_LEVEL_HIGH = 0x20
	// Position of GPIO1_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO1_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_LEVEL_LOW_Msk = 0x10
	// Bit GPIO1_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO1_LEVEL_LOW = 0x10
	// Position of GPIO0_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO0_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_EDGE_HIGH_Msk = 0x8
	// Bit GPIO0_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_EDGE_HIGH = 0x8
	// Position of GPIO0_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO0_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_EDGE_LOW_Msk = 0x4
	// Bit GPIO0_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_EDGE_LOW = 0x4
	// Position of GPIO0_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO0_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO0_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_LEVEL_HIGH = 0x2
	// Position of GPIO0_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO0_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_LEVEL_LOW_Msk = 0x1
	// Bit GPIO0_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS0_GPIO0_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTS1: Interrupt status after masking & forcing for dormant_wake
	// Position of GPIO15_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO15_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO15_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_EDGE_HIGH = 0x80000000
	// Position of GPIO15_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO15_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO15_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_EDGE_LOW = 0x40000000
	// Position of GPIO15_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO15_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO15_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_LEVEL_HIGH = 0x20000000
	// Position of GPIO15_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO15_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO15_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO15_LEVEL_LOW = 0x10000000
	// Position of GPIO14_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO14_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO14_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_EDGE_HIGH = 0x8000000
	// Position of GPIO14_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO14_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO14_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_EDGE_LOW = 0x4000000
	// Position of GPIO14_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO14_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO14_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_LEVEL_HIGH = 0x2000000
	// Position of GPIO14_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO14_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO14_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO14_LEVEL_LOW = 0x1000000
	// Position of GPIO13_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO13_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO13_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_EDGE_HIGH = 0x800000
	// Position of GPIO13_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO13_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_EDGE_LOW_Msk = 0x400000
	// Bit GPIO13_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_EDGE_LOW = 0x400000
	// Position of GPIO13_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO13_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO13_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_LEVEL_HIGH = 0x200000
	// Position of GPIO13_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO13_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO13_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO13_LEVEL_LOW = 0x100000
	// Position of GPIO12_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO12_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO12_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_EDGE_HIGH = 0x80000
	// Position of GPIO12_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO12_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_EDGE_LOW_Msk = 0x40000
	// Bit GPIO12_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_EDGE_LOW = 0x40000
	// Position of GPIO12_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO12_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO12_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_LEVEL_HIGH = 0x20000
	// Position of GPIO12_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO12_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO12_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO12_LEVEL_LOW = 0x10000
	// Position of GPIO11_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO11_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO11_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_EDGE_HIGH = 0x8000
	// Position of GPIO11_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO11_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_EDGE_LOW_Msk = 0x4000
	// Bit GPIO11_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_EDGE_LOW = 0x4000
	// Position of GPIO11_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO11_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO11_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_LEVEL_HIGH = 0x2000
	// Position of GPIO11_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO11_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO11_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO11_LEVEL_LOW = 0x1000
	// Position of GPIO10_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO10_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_EDGE_HIGH_Msk = 0x800
	// Bit GPIO10_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_EDGE_HIGH = 0x800
	// Position of GPIO10_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO10_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_EDGE_LOW_Msk = 0x400
	// Bit GPIO10_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_EDGE_LOW = 0x400
	// Position of GPIO10_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO10_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO10_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_LEVEL_HIGH = 0x200
	// Position of GPIO10_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO10_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_LEVEL_LOW_Msk = 0x100
	// Bit GPIO10_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO10_LEVEL_LOW = 0x100
	// Position of GPIO9_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO9_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_EDGE_HIGH_Msk = 0x80
	// Bit GPIO9_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_EDGE_HIGH = 0x80
	// Position of GPIO9_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO9_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_EDGE_LOW_Msk = 0x40
	// Bit GPIO9_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_EDGE_LOW = 0x40
	// Position of GPIO9_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO9_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO9_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_LEVEL_HIGH = 0x20
	// Position of GPIO9_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO9_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_LEVEL_LOW_Msk = 0x10
	// Bit GPIO9_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO9_LEVEL_LOW = 0x10
	// Position of GPIO8_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO8_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_EDGE_HIGH_Msk = 0x8
	// Bit GPIO8_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_EDGE_HIGH = 0x8
	// Position of GPIO8_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO8_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_EDGE_LOW_Msk = 0x4
	// Bit GPIO8_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_EDGE_LOW = 0x4
	// Position of GPIO8_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO8_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO8_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_LEVEL_HIGH = 0x2
	// Position of GPIO8_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO8_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_LEVEL_LOW_Msk = 0x1
	// Bit GPIO8_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS1_GPIO8_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTS2: Interrupt status after masking & forcing for dormant_wake
	// Position of GPIO23_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO23_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO23_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_EDGE_HIGH = 0x80000000
	// Position of GPIO23_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO23_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO23_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_EDGE_LOW = 0x40000000
	// Position of GPIO23_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO23_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO23_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_LEVEL_HIGH = 0x20000000
	// Position of GPIO23_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO23_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO23_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO23_LEVEL_LOW = 0x10000000
	// Position of GPIO22_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO22_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO22_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_EDGE_HIGH = 0x8000000
	// Position of GPIO22_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO22_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO22_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_EDGE_LOW = 0x4000000
	// Position of GPIO22_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO22_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO22_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_LEVEL_HIGH = 0x2000000
	// Position of GPIO22_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO22_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO22_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO22_LEVEL_LOW = 0x1000000
	// Position of GPIO21_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO21_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO21_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_EDGE_HIGH = 0x800000
	// Position of GPIO21_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO21_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_EDGE_LOW_Msk = 0x400000
	// Bit GPIO21_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_EDGE_LOW = 0x400000
	// Position of GPIO21_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO21_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO21_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_LEVEL_HIGH = 0x200000
	// Position of GPIO21_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO21_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO21_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO21_LEVEL_LOW = 0x100000
	// Position of GPIO20_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO20_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO20_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_EDGE_HIGH = 0x80000
	// Position of GPIO20_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO20_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_EDGE_LOW_Msk = 0x40000
	// Bit GPIO20_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_EDGE_LOW = 0x40000
	// Position of GPIO20_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO20_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO20_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_LEVEL_HIGH = 0x20000
	// Position of GPIO20_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO20_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO20_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO20_LEVEL_LOW = 0x10000
	// Position of GPIO19_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO19_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO19_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_EDGE_HIGH = 0x8000
	// Position of GPIO19_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO19_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_EDGE_LOW_Msk = 0x4000
	// Bit GPIO19_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_EDGE_LOW = 0x4000
	// Position of GPIO19_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO19_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO19_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_LEVEL_HIGH = 0x2000
	// Position of GPIO19_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO19_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO19_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO19_LEVEL_LOW = 0x1000
	// Position of GPIO18_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO18_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_EDGE_HIGH_Msk = 0x800
	// Bit GPIO18_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_EDGE_HIGH = 0x800
	// Position of GPIO18_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO18_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_EDGE_LOW_Msk = 0x400
	// Bit GPIO18_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_EDGE_LOW = 0x400
	// Position of GPIO18_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO18_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO18_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_LEVEL_HIGH = 0x200
	// Position of GPIO18_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO18_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_LEVEL_LOW_Msk = 0x100
	// Bit GPIO18_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO18_LEVEL_LOW = 0x100
	// Position of GPIO17_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO17_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_EDGE_HIGH_Msk = 0x80
	// Bit GPIO17_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_EDGE_HIGH = 0x80
	// Position of GPIO17_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO17_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_EDGE_LOW_Msk = 0x40
	// Bit GPIO17_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_EDGE_LOW = 0x40
	// Position of GPIO17_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO17_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO17_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_LEVEL_HIGH = 0x20
	// Position of GPIO17_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO17_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_LEVEL_LOW_Msk = 0x10
	// Bit GPIO17_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO17_LEVEL_LOW = 0x10
	// Position of GPIO16_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO16_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_EDGE_HIGH_Msk = 0x8
	// Bit GPIO16_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_EDGE_HIGH = 0x8
	// Position of GPIO16_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO16_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_EDGE_LOW_Msk = 0x4
	// Bit GPIO16_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_EDGE_LOW = 0x4
	// Position of GPIO16_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO16_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO16_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_LEVEL_HIGH = 0x2
	// Position of GPIO16_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO16_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_LEVEL_LOW_Msk = 0x1
	// Bit GPIO16_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS2_GPIO16_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTS3: Interrupt status after masking & forcing for dormant_wake
	// Position of GPIO31_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO31_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO31_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_EDGE_HIGH = 0x80000000
	// Position of GPIO31_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO31_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO31_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_EDGE_LOW = 0x40000000
	// Position of GPIO31_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO31_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO31_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_LEVEL_HIGH = 0x20000000
	// Position of GPIO31_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO31_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO31_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO31_LEVEL_LOW = 0x10000000
	// Position of GPIO30_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO30_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO30_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_EDGE_HIGH = 0x8000000
	// Position of GPIO30_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO30_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO30_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_EDGE_LOW = 0x4000000
	// Position of GPIO30_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO30_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO30_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_LEVEL_HIGH = 0x2000000
	// Position of GPIO30_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO30_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO30_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO30_LEVEL_LOW = 0x1000000
	// Position of GPIO29_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO29_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO29_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_EDGE_HIGH = 0x800000
	// Position of GPIO29_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO29_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_EDGE_LOW_Msk = 0x400000
	// Bit GPIO29_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_EDGE_LOW = 0x400000
	// Position of GPIO29_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO29_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO29_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_LEVEL_HIGH = 0x200000
	// Position of GPIO29_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO29_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO29_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO29_LEVEL_LOW = 0x100000
	// Position of GPIO28_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO28_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO28_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_EDGE_HIGH = 0x80000
	// Position of GPIO28_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO28_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_EDGE_LOW_Msk = 0x40000
	// Bit GPIO28_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_EDGE_LOW = 0x40000
	// Position of GPIO28_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO28_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO28_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_LEVEL_HIGH = 0x20000
	// Position of GPIO28_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO28_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO28_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO28_LEVEL_LOW = 0x10000
	// Position of GPIO27_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO27_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO27_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_EDGE_HIGH = 0x8000
	// Position of GPIO27_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO27_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_EDGE_LOW_Msk = 0x4000
	// Bit GPIO27_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_EDGE_LOW = 0x4000
	// Position of GPIO27_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO27_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO27_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_LEVEL_HIGH = 0x2000
	// Position of GPIO27_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO27_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO27_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO27_LEVEL_LOW = 0x1000
	// Position of GPIO26_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO26_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_EDGE_HIGH_Msk = 0x800
	// Bit GPIO26_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_EDGE_HIGH = 0x800
	// Position of GPIO26_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO26_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_EDGE_LOW_Msk = 0x400
	// Bit GPIO26_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_EDGE_LOW = 0x400
	// Position of GPIO26_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO26_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO26_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_LEVEL_HIGH = 0x200
	// Position of GPIO26_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO26_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_LEVEL_LOW_Msk = 0x100
	// Bit GPIO26_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO26_LEVEL_LOW = 0x100
	// Position of GPIO25_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO25_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_EDGE_HIGH_Msk = 0x80
	// Bit GPIO25_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_EDGE_HIGH = 0x80
	// Position of GPIO25_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO25_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_EDGE_LOW_Msk = 0x40
	// Bit GPIO25_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_EDGE_LOW = 0x40
	// Position of GPIO25_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO25_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO25_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_LEVEL_HIGH = 0x20
	// Position of GPIO25_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO25_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_LEVEL_LOW_Msk = 0x10
	// Bit GPIO25_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO25_LEVEL_LOW = 0x10
	// Position of GPIO24_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO24_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_EDGE_HIGH_Msk = 0x8
	// Bit GPIO24_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_EDGE_HIGH = 0x8
	// Position of GPIO24_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO24_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_EDGE_LOW_Msk = 0x4
	// Bit GPIO24_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_EDGE_LOW = 0x4
	// Position of GPIO24_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO24_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO24_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_LEVEL_HIGH = 0x2
	// Position of GPIO24_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO24_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_LEVEL_LOW_Msk = 0x1
	// Bit GPIO24_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS3_GPIO24_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTS4: Interrupt status after masking & forcing for dormant_wake
	// Position of GPIO39_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO39_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO39_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_EDGE_HIGH = 0x80000000
	// Position of GPIO39_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO39_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO39_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_EDGE_LOW = 0x40000000
	// Position of GPIO39_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO39_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO39_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_LEVEL_HIGH = 0x20000000
	// Position of GPIO39_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO39_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO39_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO39_LEVEL_LOW = 0x10000000
	// Position of GPIO38_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO38_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO38_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_EDGE_HIGH = 0x8000000
	// Position of GPIO38_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO38_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO38_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_EDGE_LOW = 0x4000000
	// Position of GPIO38_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO38_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO38_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_LEVEL_HIGH = 0x2000000
	// Position of GPIO38_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO38_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO38_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO38_LEVEL_LOW = 0x1000000
	// Position of GPIO37_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO37_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO37_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_EDGE_HIGH = 0x800000
	// Position of GPIO37_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO37_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_EDGE_LOW_Msk = 0x400000
	// Bit GPIO37_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_EDGE_LOW = 0x400000
	// Position of GPIO37_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO37_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO37_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_LEVEL_HIGH = 0x200000
	// Position of GPIO37_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO37_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO37_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO37_LEVEL_LOW = 0x100000
	// Position of GPIO36_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO36_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO36_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_EDGE_HIGH = 0x80000
	// Position of GPIO36_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO36_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_EDGE_LOW_Msk = 0x40000
	// Bit GPIO36_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_EDGE_LOW = 0x40000
	// Position of GPIO36_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO36_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO36_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_LEVEL_HIGH = 0x20000
	// Position of GPIO36_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO36_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO36_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO36_LEVEL_LOW = 0x10000
	// Position of GPIO35_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO35_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO35_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_EDGE_HIGH = 0x8000
	// Position of GPIO35_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO35_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_EDGE_LOW_Msk = 0x4000
	// Bit GPIO35_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_EDGE_LOW = 0x4000
	// Position of GPIO35_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO35_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO35_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_LEVEL_HIGH = 0x2000
	// Position of GPIO35_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO35_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO35_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO35_LEVEL_LOW = 0x1000
	// Position of GPIO34_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO34_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_EDGE_HIGH_Msk = 0x800
	// Bit GPIO34_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_EDGE_HIGH = 0x800
	// Position of GPIO34_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO34_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_EDGE_LOW_Msk = 0x400
	// Bit GPIO34_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_EDGE_LOW = 0x400
	// Position of GPIO34_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO34_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO34_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_LEVEL_HIGH = 0x200
	// Position of GPIO34_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO34_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_LEVEL_LOW_Msk = 0x100
	// Bit GPIO34_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO34_LEVEL_LOW = 0x100
	// Position of GPIO33_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO33_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_EDGE_HIGH_Msk = 0x80
	// Bit GPIO33_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_EDGE_HIGH = 0x80
	// Position of GPIO33_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO33_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_EDGE_LOW_Msk = 0x40
	// Bit GPIO33_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_EDGE_LOW = 0x40
	// Position of GPIO33_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO33_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO33_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_LEVEL_HIGH = 0x20
	// Position of GPIO33_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO33_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_LEVEL_LOW_Msk = 0x10
	// Bit GPIO33_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO33_LEVEL_LOW = 0x10
	// Position of GPIO32_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO32_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_EDGE_HIGH_Msk = 0x8
	// Bit GPIO32_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_EDGE_HIGH = 0x8
	// Position of GPIO32_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO32_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_EDGE_LOW_Msk = 0x4
	// Bit GPIO32_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_EDGE_LOW = 0x4
	// Position of GPIO32_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO32_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO32_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_LEVEL_HIGH = 0x2
	// Position of GPIO32_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO32_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_LEVEL_LOW_Msk = 0x1
	// Bit GPIO32_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS4_GPIO32_LEVEL_LOW = 0x1

	// DORMANT_WAKE_INTS5: Interrupt status after masking & forcing for dormant_wake
	// Position of GPIO47_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_EDGE_HIGH_Pos = 0x1f
	// Bit mask of GPIO47_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_EDGE_HIGH_Msk = 0x80000000
	// Bit GPIO47_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_EDGE_HIGH = 0x80000000
	// Position of GPIO47_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_EDGE_LOW_Pos = 0x1e
	// Bit mask of GPIO47_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_EDGE_LOW_Msk = 0x40000000
	// Bit GPIO47_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_EDGE_LOW = 0x40000000
	// Position of GPIO47_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_LEVEL_HIGH_Pos = 0x1d
	// Bit mask of GPIO47_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_LEVEL_HIGH_Msk = 0x20000000
	// Bit GPIO47_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_LEVEL_HIGH = 0x20000000
	// Position of GPIO47_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_LEVEL_LOW_Pos = 0x1c
	// Bit mask of GPIO47_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_LEVEL_LOW_Msk = 0x10000000
	// Bit GPIO47_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO47_LEVEL_LOW = 0x10000000
	// Position of GPIO46_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_EDGE_HIGH_Pos = 0x1b
	// Bit mask of GPIO46_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_EDGE_HIGH_Msk = 0x8000000
	// Bit GPIO46_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_EDGE_HIGH = 0x8000000
	// Position of GPIO46_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_EDGE_LOW_Pos = 0x1a
	// Bit mask of GPIO46_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_EDGE_LOW_Msk = 0x4000000
	// Bit GPIO46_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_EDGE_LOW = 0x4000000
	// Position of GPIO46_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_LEVEL_HIGH_Pos = 0x19
	// Bit mask of GPIO46_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_LEVEL_HIGH_Msk = 0x2000000
	// Bit GPIO46_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_LEVEL_HIGH = 0x2000000
	// Position of GPIO46_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_LEVEL_LOW_Pos = 0x18
	// Bit mask of GPIO46_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_LEVEL_LOW_Msk = 0x1000000
	// Bit GPIO46_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO46_LEVEL_LOW = 0x1000000
	// Position of GPIO45_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_EDGE_HIGH_Pos = 0x17
	// Bit mask of GPIO45_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_EDGE_HIGH_Msk = 0x800000
	// Bit GPIO45_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_EDGE_HIGH = 0x800000
	// Position of GPIO45_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_EDGE_LOW_Pos = 0x16
	// Bit mask of GPIO45_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_EDGE_LOW_Msk = 0x400000
	// Bit GPIO45_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_EDGE_LOW = 0x400000
	// Position of GPIO45_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_LEVEL_HIGH_Pos = 0x15
	// Bit mask of GPIO45_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_LEVEL_HIGH_Msk = 0x200000
	// Bit GPIO45_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_LEVEL_HIGH = 0x200000
	// Position of GPIO45_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_LEVEL_LOW_Pos = 0x14
	// Bit mask of GPIO45_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_LEVEL_LOW_Msk = 0x100000
	// Bit GPIO45_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO45_LEVEL_LOW = 0x100000
	// Position of GPIO44_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_EDGE_HIGH_Pos = 0x13
	// Bit mask of GPIO44_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_EDGE_HIGH_Msk = 0x80000
	// Bit GPIO44_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_EDGE_HIGH = 0x80000
	// Position of GPIO44_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_EDGE_LOW_Pos = 0x12
	// Bit mask of GPIO44_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_EDGE_LOW_Msk = 0x40000
	// Bit GPIO44_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_EDGE_LOW = 0x40000
	// Position of GPIO44_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_LEVEL_HIGH_Pos = 0x11
	// Bit mask of GPIO44_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_LEVEL_HIGH_Msk = 0x20000
	// Bit GPIO44_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_LEVEL_HIGH = 0x20000
	// Position of GPIO44_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_LEVEL_LOW_Pos = 0x10
	// Bit mask of GPIO44_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_LEVEL_LOW_Msk = 0x10000
	// Bit GPIO44_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO44_LEVEL_LOW = 0x10000
	// Position of GPIO43_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_EDGE_HIGH_Pos = 0xf
	// Bit mask of GPIO43_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_EDGE_HIGH_Msk = 0x8000
	// Bit GPIO43_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_EDGE_HIGH = 0x8000
	// Position of GPIO43_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_EDGE_LOW_Pos = 0xe
	// Bit mask of GPIO43_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_EDGE_LOW_Msk = 0x4000
	// Bit GPIO43_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_EDGE_LOW = 0x4000
	// Position of GPIO43_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_LEVEL_HIGH_Pos = 0xd
	// Bit mask of GPIO43_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_LEVEL_HIGH_Msk = 0x2000
	// Bit GPIO43_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_LEVEL_HIGH = 0x2000
	// Position of GPIO43_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_LEVEL_LOW_Pos = 0xc
	// Bit mask of GPIO43_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_LEVEL_LOW_Msk = 0x1000
	// Bit GPIO43_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO43_LEVEL_LOW = 0x1000
	// Position of GPIO42_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_EDGE_HIGH_Pos = 0xb
	// Bit mask of GPIO42_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_EDGE_HIGH_Msk = 0x800
	// Bit GPIO42_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_EDGE_HIGH = 0x800
	// Position of GPIO42_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_EDGE_LOW_Pos = 0xa
	// Bit mask of GPIO42_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_EDGE_LOW_Msk = 0x400
	// Bit GPIO42_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_EDGE_LOW = 0x400
	// Position of GPIO42_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_LEVEL_HIGH_Pos = 0x9
	// Bit mask of GPIO42_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_LEVEL_HIGH_Msk = 0x200
	// Bit GPIO42_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_LEVEL_HIGH = 0x200
	// Position of GPIO42_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_LEVEL_LOW_Pos = 0x8
	// Bit mask of GPIO42_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_LEVEL_LOW_Msk = 0x100
	// Bit GPIO42_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO42_LEVEL_LOW = 0x100
	// Position of GPIO41_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_EDGE_HIGH_Pos = 0x7
	// Bit mask of GPIO41_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_EDGE_HIGH_Msk = 0x80
	// Bit GPIO41_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_EDGE_HIGH = 0x80
	// Position of GPIO41_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_EDGE_LOW_Pos = 0x6
	// Bit mask of GPIO41_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_EDGE_LOW_Msk = 0x40
	// Bit GPIO41_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_EDGE_LOW = 0x40
	// Position of GPIO41_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_LEVEL_HIGH_Pos = 0x5
	// Bit mask of GPIO41_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_LEVEL_HIGH_Msk = 0x20
	// Bit GPIO41_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_LEVEL_HIGH = 0x20
	// Position of GPIO41_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_LEVEL_LOW_Pos = 0x4
	// Bit mask of GPIO41_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_LEVEL_LOW_Msk = 0x10
	// Bit GPIO41_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO41_LEVEL_LOW = 0x10
	// Position of GPIO40_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_EDGE_HIGH_Pos = 0x3
	// Bit mask of GPIO40_EDGE_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_EDGE_HIGH_Msk = 0x8
	// Bit GPIO40_EDGE_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_EDGE_HIGH = 0x8
	// Position of GPIO40_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_EDGE_LOW_Pos = 0x2
	// Bit mask of GPIO40_EDGE_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_EDGE_LOW_Msk = 0x4
	// Bit GPIO40_EDGE_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_EDGE_LOW = 0x4
	// Position of GPIO40_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_LEVEL_HIGH_Pos = 0x1
	// Bit mask of GPIO40_LEVEL_HIGH field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_LEVEL_HIGH_Msk = 0x2
	// Bit GPIO40_LEVEL_HIGH.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_LEVEL_HIGH = 0x2
	// Position of GPIO40_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_LEVEL_LOW_Pos = 0x0
	// Bit mask of GPIO40_LEVEL_LOW field.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_LEVEL_LOW_Msk = 0x1
	// Bit GPIO40_LEVEL_LOW.
	IO_BANK0_DORMANT_WAKE_INTS5_GPIO40_LEVEL_LOW = 0x1
)

// Constants for SYSINFO
const (
	// CHIP_ID: JEDEC JEP-106 compliant chip identifier.
	// Position of REVISION field.
	SYSINFO_CHIP_ID_REVISION_Pos = 0x1c
	// Bit mask of REVISION field.
	SYSINFO_CHIP_ID_REVISION_Msk = 0xf0000000
	// Position of PART field.
	SYSINFO_CHIP_ID_PART_Pos = 0xc
	// Bit mask of PART field.
	SYSINFO_CHIP_ID_PART_Msk = 0xffff000
	// Position of MANUFACTURER field.
	SYSINFO_CHIP_ID_MANUFACTURER_Pos = 0x1
	// Bit mask of MANUFACTURER field.
	SYSINFO_CHIP_ID_MANUFACTURER_Msk = 0xffe
	// Position of STOP_BIT field.
	SYSINFO_CHIP_ID_STOP_BIT_Pos = 0x0
	// Bit mask of STOP_BIT field.
	SYSINFO_CHIP_ID_STOP_BIT_Msk = 0x1
	// Bit STOP_BIT.
	SYSINFO_CHIP_ID_STOP_BIT = 0x1

	// PACKAGE_SEL
	// Position of PACKAGE_SEL field.
	SYSINFO_PACKAGE_SEL_PACKAGE_SEL_Pos = 0x0
	// Bit mask of PACKAGE_SEL field.
	SYSINFO_PACKAGE_SEL_PACKAGE_SEL_Msk = 0x1
	// Bit PACKAGE_SEL.
	SYSINFO_PACKAGE_SEL_PACKAGE_SEL = 0x1

	// PLATFORM: Platform register. Allows software to know what environment it is running in during pre-production development. Post-production, the PLATFORM is always ASIC, non-SIM.
	// Position of GATESIM field.
	SYSINFO_PLATFORM_GATESIM_Pos = 0x4
	// Bit mask of GATESIM field.
	SYSINFO_PLATFORM_GATESIM_Msk = 0x10
	// Bit GATESIM.
	SYSINFO_PLATFORM_GATESIM = 0x10
	// Position of BATCHSIM field.
	SYSINFO_PLATFORM_BATCHSIM_Pos = 0x3
	// Bit mask of BATCHSIM field.
	SYSINFO_PLATFORM_BATCHSIM_Msk = 0x8
	// Bit BATCHSIM.
	SYSINFO_PLATFORM_BATCHSIM = 0x8
	// Position of HDLSIM field.
	SYSINFO_PLATFORM_HDLSIM_Pos = 0x2
	// Bit mask of HDLSIM field.
	SYSINFO_PLATFORM_HDLSIM_Msk = 0x4
	// Bit HDLSIM.
	SYSINFO_PLATFORM_HDLSIM = 0x4
	// Position of ASIC field.
	SYSINFO_PLATFORM_ASIC_Pos = 0x1
	// Bit mask of ASIC field.
	SYSINFO_PLATFORM_ASIC_Msk = 0x2
	// Bit ASIC.
	SYSINFO_PLATFORM_ASIC = 0x2
	// Position of FPGA field.
	SYSINFO_PLATFORM_FPGA_Pos = 0x0
	// Bit mask of FPGA field.
	SYSINFO_PLATFORM_FPGA_Msk = 0x1
	// Bit FPGA.
	SYSINFO_PLATFORM_FPGA = 0x1

	// GITREF_RP2350: Git hash of the chip source. Used to identify chip version.
	// Position of GITREF_RP2350 field.
	SYSINFO_GITREF_RP2350_GITREF_RP2350_Pos = 0x0
	// Bit mask of GITREF_RP2350 field.
	SYSINFO_GITREF_RP2350_GITREF_RP2350_Msk = 0xffffffff
)

// Constants for SHA256: SHA-256 hash function implementation
const (
	// CSR: Control and status register
	// Position of BSWAP field.
	SHA256_CSR_BSWAP_Pos = 0xc
	// Bit mask of BSWAP field.
	SHA256_CSR_BSWAP_Msk = 0x1000
	// Bit BSWAP.
	SHA256_CSR_BSWAP = 0x1000
	// Position of DMA_SIZE field.
	SHA256_CSR_DMA_SIZE_Pos = 0x8
	// Bit mask of DMA_SIZE field.
	SHA256_CSR_DMA_SIZE_Msk   = 0x300
	SHA256_CSR_DMA_SIZE_8bit  = 0x0
	SHA256_CSR_DMA_SIZE_16bit = 0x1
	SHA256_CSR_DMA_SIZE_32bit = 0x2
	// Position of ERR_WDATA_NOT_RDY field.
	SHA256_CSR_ERR_WDATA_NOT_RDY_Pos = 0x4
	// Bit mask of ERR_WDATA_NOT_RDY field.
	SHA256_CSR_ERR_WDATA_NOT_RDY_Msk = 0x10
	// Bit ERR_WDATA_NOT_RDY.
	SHA256_CSR_ERR_WDATA_NOT_RDY = 0x10
	// Position of SUM_VLD field.
	SHA256_CSR_SUM_VLD_Pos = 0x2
	// Bit mask of SUM_VLD field.
	SHA256_CSR_SUM_VLD_Msk = 0x4
	// Bit SUM_VLD.
	SHA256_CSR_SUM_VLD = 0x4
	// Position of WDATA_RDY field.
	SHA256_CSR_WDATA_RDY_Pos = 0x1
	// Bit mask of WDATA_RDY field.
	SHA256_CSR_WDATA_RDY_Msk = 0x2
	// Bit WDATA_RDY.
	SHA256_CSR_WDATA_RDY = 0x2
	// Position of START field.
	SHA256_CSR_START_Pos = 0x0
	// Bit mask of START field.
	SHA256_CSR_START_Msk = 0x1
	// Bit START.
	SHA256_CSR_START = 0x1

	// WDATA: Write data register
	// Position of WDATA field.
	SHA256_WDATA_WDATA_Pos = 0x0
	// Bit mask of WDATA field.
	SHA256_WDATA_WDATA_Msk = 0xffffffff

	// SUM0: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
	// Position of SUM0 field.
	SHA256_SUM0_SUM0_Pos = 0x0
	// Bit mask of SUM0 field.
	SHA256_SUM0_SUM0_Msk = 0xffffffff

	// SUM1: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
	// Position of SUM1 field.
	SHA256_SUM1_SUM1_Pos = 0x0
	// Bit mask of SUM1 field.
	SHA256_SUM1_SUM1_Msk = 0xffffffff

	// SUM2: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
	// Position of SUM2 field.
	SHA256_SUM2_SUM2_Pos = 0x0
	// Bit mask of SUM2 field.
	SHA256_SUM2_SUM2_Msk = 0xffffffff

	// SUM3: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
	// Position of SUM3 field.
	SHA256_SUM3_SUM3_Pos = 0x0
	// Bit mask of SUM3 field.
	SHA256_SUM3_SUM3_Msk = 0xffffffff

	// SUM4: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
	// Position of SUM4 field.
	SHA256_SUM4_SUM4_Pos = 0x0
	// Bit mask of SUM4 field.
	SHA256_SUM4_SUM4_Msk = 0xffffffff

	// SUM5: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
	// Position of SUM5 field.
	SHA256_SUM5_SUM5_Pos = 0x0
	// Bit mask of SUM5 field.
	SHA256_SUM5_SUM5_Msk = 0xffffffff

	// SUM6: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
	// Position of SUM6 field.
	SHA256_SUM6_SUM6_Pos = 0x0
	// Bit mask of SUM6 field.
	SHA256_SUM6_SUM6_Msk = 0xffffffff

	// SUM7: 256-bit checksum result. Contents are undefined when CSR_SUM_VLD is 0.
	// Position of SUM7 field.
	SHA256_SUM7_SUM7_Pos = 0x0
	// Bit mask of SUM7 field.
	SHA256_SUM7_SUM7_Msk = 0xffffffff
)

// Constants for HSTX_FIFO: FIFO status and write access for HSTX
const (
	// STAT: FIFO status
	// Position of WOF field.
	HSTX_FIFO_STAT_WOF_Pos = 0xa
	// Bit mask of WOF field.
	HSTX_FIFO_STAT_WOF_Msk = 0x400
	// Bit WOF.
	HSTX_FIFO_STAT_WOF = 0x400
	// Position of EMPTY field.
	HSTX_FIFO_STAT_EMPTY_Pos = 0x9
	// Bit mask of EMPTY field.
	HSTX_FIFO_STAT_EMPTY_Msk = 0x200
	// Bit EMPTY.
	HSTX_FIFO_STAT_EMPTY = 0x200
	// Position of FULL field.
	HSTX_FIFO_STAT_FULL_Pos = 0x8
	// Bit mask of FULL field.
	HSTX_FIFO_STAT_FULL_Msk = 0x100
	// Bit FULL.
	HSTX_FIFO_STAT_FULL = 0x100
	// Position of LEVEL field.
	HSTX_FIFO_STAT_LEVEL_Pos = 0x0
	// Bit mask of LEVEL field.
	HSTX_FIFO_STAT_LEVEL_Msk = 0xff

	// FIFO: Write access to FIFO
	// Position of FIFO field.
	HSTX_FIFO_FIFO_FIFO_Pos = 0x0
	// Bit mask of FIFO field.
	HSTX_FIFO_FIFO_FIFO_Msk = 0xffffffff
)

// Constants for HSTX_CTRL: Control interface to HSTX. For FIFO write access and status, see the HSTX_FIFO register block.
const (
	// CSR
	// Position of CLKDIV field.
	HSTX_CTRL_CSR_CLKDIV_Pos = 0x1c
	// Bit mask of CLKDIV field.
	HSTX_CTRL_CSR_CLKDIV_Msk = 0xf0000000
	// Position of CLKPHASE field.
	HSTX_CTRL_CSR_CLKPHASE_Pos = 0x18
	// Bit mask of CLKPHASE field.
	HSTX_CTRL_CSR_CLKPHASE_Msk = 0xf000000
	// Position of N_SHIFTS field.
	HSTX_CTRL_CSR_N_SHIFTS_Pos = 0x10
	// Bit mask of N_SHIFTS field.
	HSTX_CTRL_CSR_N_SHIFTS_Msk = 0x1f0000
	// Position of SHIFT field.
	HSTX_CTRL_CSR_SHIFT_Pos = 0x8
	// Bit mask of SHIFT field.
	HSTX_CTRL_CSR_SHIFT_Msk = 0x1f00
	// Position of COUPLED_SEL field.
	HSTX_CTRL_CSR_COUPLED_SEL_Pos = 0x5
	// Bit mask of COUPLED_SEL field.
	HSTX_CTRL_CSR_COUPLED_SEL_Msk = 0x60
	// Position of COUPLED_MODE field.
	HSTX_CTRL_CSR_COUPLED_MODE_Pos = 0x4
	// Bit mask of COUPLED_MODE field.
	HSTX_CTRL_CSR_COUPLED_MODE_Msk = 0x10
	// Bit COUPLED_MODE.
	HSTX_CTRL_CSR_COUPLED_MODE = 0x10
	// Position of EXPAND_EN field.
	HSTX_CTRL_CSR_EXPAND_EN_Pos = 0x1
	// Bit mask of EXPAND_EN field.
	HSTX_CTRL_CSR_EXPAND_EN_Msk = 0x2
	// Bit EXPAND_EN.
	HSTX_CTRL_CSR_EXPAND_EN = 0x2
	// Position of EN field.
	HSTX_CTRL_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	HSTX_CTRL_CSR_EN_Msk = 0x1
	// Bit EN.
	HSTX_CTRL_CSR_EN = 0x1

	// BIT0: Data control register for output bit 0
	// Position of CLK field.
	HSTX_CTRL_BIT0_CLK_Pos = 0x11
	// Bit mask of CLK field.
	HSTX_CTRL_BIT0_CLK_Msk = 0x20000
	// Bit CLK.
	HSTX_CTRL_BIT0_CLK = 0x20000
	// Position of INV field.
	HSTX_CTRL_BIT0_INV_Pos = 0x10
	// Bit mask of INV field.
	HSTX_CTRL_BIT0_INV_Msk = 0x10000
	// Bit INV.
	HSTX_CTRL_BIT0_INV = 0x10000
	// Position of SEL_N field.
	HSTX_CTRL_BIT0_SEL_N_Pos = 0x8
	// Bit mask of SEL_N field.
	HSTX_CTRL_BIT0_SEL_N_Msk = 0x1f00
	// Position of SEL_P field.
	HSTX_CTRL_BIT0_SEL_P_Pos = 0x0
	// Bit mask of SEL_P field.
	HSTX_CTRL_BIT0_SEL_P_Msk = 0x1f

	// BIT1: Data control register for output bit 1
	// Position of CLK field.
	HSTX_CTRL_BIT1_CLK_Pos = 0x11
	// Bit mask of CLK field.
	HSTX_CTRL_BIT1_CLK_Msk = 0x20000
	// Bit CLK.
	HSTX_CTRL_BIT1_CLK = 0x20000
	// Position of INV field.
	HSTX_CTRL_BIT1_INV_Pos = 0x10
	// Bit mask of INV field.
	HSTX_CTRL_BIT1_INV_Msk = 0x10000
	// Bit INV.
	HSTX_CTRL_BIT1_INV = 0x10000
	// Position of SEL_N field.
	HSTX_CTRL_BIT1_SEL_N_Pos = 0x8
	// Bit mask of SEL_N field.
	HSTX_CTRL_BIT1_SEL_N_Msk = 0x1f00
	// Position of SEL_P field.
	HSTX_CTRL_BIT1_SEL_P_Pos = 0x0
	// Bit mask of SEL_P field.
	HSTX_CTRL_BIT1_SEL_P_Msk = 0x1f

	// BIT2: Data control register for output bit 2
	// Position of CLK field.
	HSTX_CTRL_BIT2_CLK_Pos = 0x11
	// Bit mask of CLK field.
	HSTX_CTRL_BIT2_CLK_Msk = 0x20000
	// Bit CLK.
	HSTX_CTRL_BIT2_CLK = 0x20000
	// Position of INV field.
	HSTX_CTRL_BIT2_INV_Pos = 0x10
	// Bit mask of INV field.
	HSTX_CTRL_BIT2_INV_Msk = 0x10000
	// Bit INV.
	HSTX_CTRL_BIT2_INV = 0x10000
	// Position of SEL_N field.
	HSTX_CTRL_BIT2_SEL_N_Pos = 0x8
	// Bit mask of SEL_N field.
	HSTX_CTRL_BIT2_SEL_N_Msk = 0x1f00
	// Position of SEL_P field.
	HSTX_CTRL_BIT2_SEL_P_Pos = 0x0
	// Bit mask of SEL_P field.
	HSTX_CTRL_BIT2_SEL_P_Msk = 0x1f

	// BIT3: Data control register for output bit 3
	// Position of CLK field.
	HSTX_CTRL_BIT3_CLK_Pos = 0x11
	// Bit mask of CLK field.
	HSTX_CTRL_BIT3_CLK_Msk = 0x20000
	// Bit CLK.
	HSTX_CTRL_BIT3_CLK = 0x20000
	// Position of INV field.
	HSTX_CTRL_BIT3_INV_Pos = 0x10
	// Bit mask of INV field.
	HSTX_CTRL_BIT3_INV_Msk = 0x10000
	// Bit INV.
	HSTX_CTRL_BIT3_INV = 0x10000
	// Position of SEL_N field.
	HSTX_CTRL_BIT3_SEL_N_Pos = 0x8
	// Bit mask of SEL_N field.
	HSTX_CTRL_BIT3_SEL_N_Msk = 0x1f00
	// Position of SEL_P field.
	HSTX_CTRL_BIT3_SEL_P_Pos = 0x0
	// Bit mask of SEL_P field.
	HSTX_CTRL_BIT3_SEL_P_Msk = 0x1f

	// BIT4: Data control register for output bit 4
	// Position of CLK field.
	HSTX_CTRL_BIT4_CLK_Pos = 0x11
	// Bit mask of CLK field.
	HSTX_CTRL_BIT4_CLK_Msk = 0x20000
	// Bit CLK.
	HSTX_CTRL_BIT4_CLK = 0x20000
	// Position of INV field.
	HSTX_CTRL_BIT4_INV_Pos = 0x10
	// Bit mask of INV field.
	HSTX_CTRL_BIT4_INV_Msk = 0x10000
	// Bit INV.
	HSTX_CTRL_BIT4_INV = 0x10000
	// Position of SEL_N field.
	HSTX_CTRL_BIT4_SEL_N_Pos = 0x8
	// Bit mask of SEL_N field.
	HSTX_CTRL_BIT4_SEL_N_Msk = 0x1f00
	// Position of SEL_P field.
	HSTX_CTRL_BIT4_SEL_P_Pos = 0x0
	// Bit mask of SEL_P field.
	HSTX_CTRL_BIT4_SEL_P_Msk = 0x1f

	// BIT5: Data control register for output bit 5
	// Position of CLK field.
	HSTX_CTRL_BIT5_CLK_Pos = 0x11
	// Bit mask of CLK field.
	HSTX_CTRL_BIT5_CLK_Msk = 0x20000
	// Bit CLK.
	HSTX_CTRL_BIT5_CLK = 0x20000
	// Position of INV field.
	HSTX_CTRL_BIT5_INV_Pos = 0x10
	// Bit mask of INV field.
	HSTX_CTRL_BIT5_INV_Msk = 0x10000
	// Bit INV.
	HSTX_CTRL_BIT5_INV = 0x10000
	// Position of SEL_N field.
	HSTX_CTRL_BIT5_SEL_N_Pos = 0x8
	// Bit mask of SEL_N field.
	HSTX_CTRL_BIT5_SEL_N_Msk = 0x1f00
	// Position of SEL_P field.
	HSTX_CTRL_BIT5_SEL_P_Pos = 0x0
	// Bit mask of SEL_P field.
	HSTX_CTRL_BIT5_SEL_P_Msk = 0x1f

	// BIT6: Data control register for output bit 6
	// Position of CLK field.
	HSTX_CTRL_BIT6_CLK_Pos = 0x11
	// Bit mask of CLK field.
	HSTX_CTRL_BIT6_CLK_Msk = 0x20000
	// Bit CLK.
	HSTX_CTRL_BIT6_CLK = 0x20000
	// Position of INV field.
	HSTX_CTRL_BIT6_INV_Pos = 0x10
	// Bit mask of INV field.
	HSTX_CTRL_BIT6_INV_Msk = 0x10000
	// Bit INV.
	HSTX_CTRL_BIT6_INV = 0x10000
	// Position of SEL_N field.
	HSTX_CTRL_BIT6_SEL_N_Pos = 0x8
	// Bit mask of SEL_N field.
	HSTX_CTRL_BIT6_SEL_N_Msk = 0x1f00
	// Position of SEL_P field.
	HSTX_CTRL_BIT6_SEL_P_Pos = 0x0
	// Bit mask of SEL_P field.
	HSTX_CTRL_BIT6_SEL_P_Msk = 0x1f

	// BIT7: Data control register for output bit 7
	// Position of CLK field.
	HSTX_CTRL_BIT7_CLK_Pos = 0x11
	// Bit mask of CLK field.
	HSTX_CTRL_BIT7_CLK_Msk = 0x20000
	// Bit CLK.
	HSTX_CTRL_BIT7_CLK = 0x20000
	// Position of INV field.
	HSTX_CTRL_BIT7_INV_Pos = 0x10
	// Bit mask of INV field.
	HSTX_CTRL_BIT7_INV_Msk = 0x10000
	// Bit INV.
	HSTX_CTRL_BIT7_INV = 0x10000
	// Position of SEL_N field.
	HSTX_CTRL_BIT7_SEL_N_Pos = 0x8
	// Bit mask of SEL_N field.
	HSTX_CTRL_BIT7_SEL_N_Msk = 0x1f00
	// Position of SEL_P field.
	HSTX_CTRL_BIT7_SEL_P_Pos = 0x0
	// Bit mask of SEL_P field.
	HSTX_CTRL_BIT7_SEL_P_Msk = 0x1f

	// EXPAND_SHIFT: Configure the optional shifter inside the command expander
	// Position of ENC_N_SHIFTS field.
	HSTX_CTRL_EXPAND_SHIFT_ENC_N_SHIFTS_Pos = 0x18
	// Bit mask of ENC_N_SHIFTS field.
	HSTX_CTRL_EXPAND_SHIFT_ENC_N_SHIFTS_Msk = 0x1f000000
	// Position of ENC_SHIFT field.
	HSTX_CTRL_EXPAND_SHIFT_ENC_SHIFT_Pos = 0x10
	// Bit mask of ENC_SHIFT field.
	HSTX_CTRL_EXPAND_SHIFT_ENC_SHIFT_Msk = 0x1f0000
	// Position of RAW_N_SHIFTS field.
	HSTX_CTRL_EXPAND_SHIFT_RAW_N_SHIFTS_Pos = 0x8
	// Bit mask of RAW_N_SHIFTS field.
	HSTX_CTRL_EXPAND_SHIFT_RAW_N_SHIFTS_Msk = 0x1f00
	// Position of RAW_SHIFT field.
	HSTX_CTRL_EXPAND_SHIFT_RAW_SHIFT_Pos = 0x0
	// Bit mask of RAW_SHIFT field.
	HSTX_CTRL_EXPAND_SHIFT_RAW_SHIFT_Msk = 0x1f

	// EXPAND_TMDS: Configure the optional TMDS encoder inside the command expander
	// Position of L2_NBITS field.
	HSTX_CTRL_EXPAND_TMDS_L2_NBITS_Pos = 0x15
	// Bit mask of L2_NBITS field.
	HSTX_CTRL_EXPAND_TMDS_L2_NBITS_Msk = 0xe00000
	// Position of L2_ROT field.
	HSTX_CTRL_EXPAND_TMDS_L2_ROT_Pos = 0x10
	// Bit mask of L2_ROT field.
	HSTX_CTRL_EXPAND_TMDS_L2_ROT_Msk = 0x1f0000
	// Position of L1_NBITS field.
	HSTX_CTRL_EXPAND_TMDS_L1_NBITS_Pos = 0xd
	// Bit mask of L1_NBITS field.
	HSTX_CTRL_EXPAND_TMDS_L1_NBITS_Msk = 0xe000
	// Position of L1_ROT field.
	HSTX_CTRL_EXPAND_TMDS_L1_ROT_Pos = 0x8
	// Bit mask of L1_ROT field.
	HSTX_CTRL_EXPAND_TMDS_L1_ROT_Msk = 0x1f00
	// Position of L0_NBITS field.
	HSTX_CTRL_EXPAND_TMDS_L0_NBITS_Pos = 0x5
	// Bit mask of L0_NBITS field.
	HSTX_CTRL_EXPAND_TMDS_L0_NBITS_Msk = 0xe0
	// Position of L0_ROT field.
	HSTX_CTRL_EXPAND_TMDS_L0_ROT_Pos = 0x0
	// Bit mask of L0_ROT field.
	HSTX_CTRL_EXPAND_TMDS_L0_ROT_Msk = 0x1f
)

// Constants for EPPB: Cortex-M33 EPPB vendor register block for RP2350
const (
	// NMI_MASK0: NMI mask for IRQs 0 through 31. This register is core-local, and is reset by a processor warm reset.
	// Position of NMI_MASK0 field.
	EPPB_NMI_MASK0_NMI_MASK0_Pos = 0x0
	// Bit mask of NMI_MASK0 field.
	EPPB_NMI_MASK0_NMI_MASK0_Msk = 0xffffffff

	// NMI_MASK1: NMI mask for IRQs 0 though 51. This register is core-local, and is reset by a processor warm reset.
	// Position of NMI_MASK1 field.
	EPPB_NMI_MASK1_NMI_MASK1_Pos = 0x0
	// Bit mask of NMI_MASK1 field.
	EPPB_NMI_MASK1_NMI_MASK1_Msk = 0xfffff

	// SLEEPCTRL: Nonstandard sleep control register
	// Position of WICENACK field.
	EPPB_SLEEPCTRL_WICENACK_Pos = 0x2
	// Bit mask of WICENACK field.
	EPPB_SLEEPCTRL_WICENACK_Msk = 0x4
	// Bit WICENACK.
	EPPB_SLEEPCTRL_WICENACK = 0x4
	// Position of WICENREQ field.
	EPPB_SLEEPCTRL_WICENREQ_Pos = 0x1
	// Bit mask of WICENREQ field.
	EPPB_SLEEPCTRL_WICENREQ_Msk = 0x2
	// Bit WICENREQ.
	EPPB_SLEEPCTRL_WICENREQ = 0x2
	// Position of LIGHT_SLEEP field.
	EPPB_SLEEPCTRL_LIGHT_SLEEP_Pos = 0x0
	// Bit mask of LIGHT_SLEEP field.
	EPPB_SLEEPCTRL_LIGHT_SLEEP_Msk = 0x1
	// Bit LIGHT_SLEEP.
	EPPB_SLEEPCTRL_LIGHT_SLEEP = 0x1
)

// Constants for PPB: TEAL registers accessible through the debug interface
const (
	// ITM_STIM0: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM0_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM0_STIMULUS_Msk = 0xffffffff

	// ITM_STIM1: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM1_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM1_STIMULUS_Msk = 0xffffffff

	// ITM_STIM2: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM2_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM2_STIMULUS_Msk = 0xffffffff

	// ITM_STIM3: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM3_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM3_STIMULUS_Msk = 0xffffffff

	// ITM_STIM4: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM4_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM4_STIMULUS_Msk = 0xffffffff

	// ITM_STIM5: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM5_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM5_STIMULUS_Msk = 0xffffffff

	// ITM_STIM6: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM6_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM6_STIMULUS_Msk = 0xffffffff

	// ITM_STIM7: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM7_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM7_STIMULUS_Msk = 0xffffffff

	// ITM_STIM8: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM8_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM8_STIMULUS_Msk = 0xffffffff

	// ITM_STIM9: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM9_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM9_STIMULUS_Msk = 0xffffffff

	// ITM_STIM10: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM10_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM10_STIMULUS_Msk = 0xffffffff

	// ITM_STIM11: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM11_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM11_STIMULUS_Msk = 0xffffffff

	// ITM_STIM12: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM12_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM12_STIMULUS_Msk = 0xffffffff

	// ITM_STIM13: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM13_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM13_STIMULUS_Msk = 0xffffffff

	// ITM_STIM14: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM14_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM14_STIMULUS_Msk = 0xffffffff

	// ITM_STIM15: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM15_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM15_STIMULUS_Msk = 0xffffffff

	// ITM_STIM16: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM16_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM16_STIMULUS_Msk = 0xffffffff

	// ITM_STIM17: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM17_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM17_STIMULUS_Msk = 0xffffffff

	// ITM_STIM18: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM18_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM18_STIMULUS_Msk = 0xffffffff

	// ITM_STIM19: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM19_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM19_STIMULUS_Msk = 0xffffffff

	// ITM_STIM20: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM20_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM20_STIMULUS_Msk = 0xffffffff

	// ITM_STIM21: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM21_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM21_STIMULUS_Msk = 0xffffffff

	// ITM_STIM22: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM22_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM22_STIMULUS_Msk = 0xffffffff

	// ITM_STIM23: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM23_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM23_STIMULUS_Msk = 0xffffffff

	// ITM_STIM24: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM24_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM24_STIMULUS_Msk = 0xffffffff

	// ITM_STIM25: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM25_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM25_STIMULUS_Msk = 0xffffffff

	// ITM_STIM26: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM26_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM26_STIMULUS_Msk = 0xffffffff

	// ITM_STIM27: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM27_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM27_STIMULUS_Msk = 0xffffffff

	// ITM_STIM28: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM28_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM28_STIMULUS_Msk = 0xffffffff

	// ITM_STIM29: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM29_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM29_STIMULUS_Msk = 0xffffffff

	// ITM_STIM30: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM30_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM30_STIMULUS_Msk = 0xffffffff

	// ITM_STIM31: Provides the interface for generating Instrumentation packets
	// Position of STIMULUS field.
	PPB_ITM_STIM31_STIMULUS_Pos = 0x0
	// Bit mask of STIMULUS field.
	PPB_ITM_STIM31_STIMULUS_Msk = 0xffffffff

	// ITM_TER0: Provide an individual enable bit for each ITM_STIM register
	// Position of STIMENA field.
	PPB_ITM_TER0_STIMENA_Pos = 0x0
	// Bit mask of STIMENA field.
	PPB_ITM_TER0_STIMENA_Msk = 0xffffffff

	// ITM_TPR: Controls which stimulus ports can be accessed by unprivileged code
	// Position of PRIVMASK field.
	PPB_ITM_TPR_PRIVMASK_Pos = 0x0
	// Bit mask of PRIVMASK field.
	PPB_ITM_TPR_PRIVMASK_Msk = 0xf

	// ITM_TCR: Configures and controls transfers through the ITM interface
	// Position of BUSY field.
	PPB_ITM_TCR_BUSY_Pos = 0x17
	// Bit mask of BUSY field.
	PPB_ITM_TCR_BUSY_Msk = 0x800000
	// Bit BUSY.
	PPB_ITM_TCR_BUSY = 0x800000
	// Position of TRACEBUSID field.
	PPB_ITM_TCR_TRACEBUSID_Pos = 0x10
	// Bit mask of TRACEBUSID field.
	PPB_ITM_TCR_TRACEBUSID_Msk = 0x7f0000
	// Position of GTSFREQ field.
	PPB_ITM_TCR_GTSFREQ_Pos = 0xa
	// Bit mask of GTSFREQ field.
	PPB_ITM_TCR_GTSFREQ_Msk = 0xc00
	// Position of TSPRESCALE field.
	PPB_ITM_TCR_TSPRESCALE_Pos = 0x8
	// Bit mask of TSPRESCALE field.
	PPB_ITM_TCR_TSPRESCALE_Msk = 0x300
	// Position of STALLENA field.
	PPB_ITM_TCR_STALLENA_Pos = 0x5
	// Bit mask of STALLENA field.
	PPB_ITM_TCR_STALLENA_Msk = 0x20
	// Bit STALLENA.
	PPB_ITM_TCR_STALLENA = 0x20
	// Position of SWOENA field.
	PPB_ITM_TCR_SWOENA_Pos = 0x4
	// Bit mask of SWOENA field.
	PPB_ITM_TCR_SWOENA_Msk = 0x10
	// Bit SWOENA.
	PPB_ITM_TCR_SWOENA = 0x10
	// Position of TXENA field.
	PPB_ITM_TCR_TXENA_Pos = 0x3
	// Bit mask of TXENA field.
	PPB_ITM_TCR_TXENA_Msk = 0x8
	// Bit TXENA.
	PPB_ITM_TCR_TXENA = 0x8
	// Position of SYNCENA field.
	PPB_ITM_TCR_SYNCENA_Pos = 0x2
	// Bit mask of SYNCENA field.
	PPB_ITM_TCR_SYNCENA_Msk = 0x4
	// Bit SYNCENA.
	PPB_ITM_TCR_SYNCENA = 0x4
	// Position of TSENA field.
	PPB_ITM_TCR_TSENA_Pos = 0x1
	// Bit mask of TSENA field.
	PPB_ITM_TCR_TSENA_Msk = 0x2
	// Bit TSENA.
	PPB_ITM_TCR_TSENA = 0x2
	// Position of ITMENA field.
	PPB_ITM_TCR_ITMENA_Pos = 0x0
	// Bit mask of ITMENA field.
	PPB_ITM_TCR_ITMENA_Msk = 0x1
	// Bit ITMENA.
	PPB_ITM_TCR_ITMENA = 0x1

	// INT_ATREADY: Integration Mode: Read ATB Ready
	// Position of AFVALID field.
	PPB_INT_ATREADY_AFVALID_Pos = 0x1
	// Bit mask of AFVALID field.
	PPB_INT_ATREADY_AFVALID_Msk = 0x2
	// Bit AFVALID.
	PPB_INT_ATREADY_AFVALID = 0x2
	// Position of ATREADY field.
	PPB_INT_ATREADY_ATREADY_Pos = 0x0
	// Bit mask of ATREADY field.
	PPB_INT_ATREADY_ATREADY_Msk = 0x1
	// Bit ATREADY.
	PPB_INT_ATREADY_ATREADY = 0x1

	// INT_ATVALID: Integration Mode: Write ATB Valid
	// Position of AFREADY field.
	PPB_INT_ATVALID_AFREADY_Pos = 0x1
	// Bit mask of AFREADY field.
	PPB_INT_ATVALID_AFREADY_Msk = 0x2
	// Bit AFREADY.
	PPB_INT_ATVALID_AFREADY = 0x2
	// Position of ATREADY field.
	PPB_INT_ATVALID_ATREADY_Pos = 0x0
	// Bit mask of ATREADY field.
	PPB_INT_ATVALID_ATREADY_Msk = 0x1
	// Bit ATREADY.
	PPB_INT_ATVALID_ATREADY = 0x1

	// ITM_ITCTRL: Integration Mode Control Register
	// Position of IME field.
	PPB_ITM_ITCTRL_IME_Pos = 0x0
	// Bit mask of IME field.
	PPB_ITM_ITCTRL_IME_Msk = 0x1
	// Bit IME.
	PPB_ITM_ITCTRL_IME = 0x1

	// ITM_DEVARCH: Provides CoreSight discovery information for the ITM
	// Position of ARCHITECT field.
	PPB_ITM_DEVARCH_ARCHITECT_Pos = 0x15
	// Bit mask of ARCHITECT field.
	PPB_ITM_DEVARCH_ARCHITECT_Msk = 0xffe00000
	// Position of PRESENT field.
	PPB_ITM_DEVARCH_PRESENT_Pos = 0x14
	// Bit mask of PRESENT field.
	PPB_ITM_DEVARCH_PRESENT_Msk = 0x100000
	// Bit PRESENT.
	PPB_ITM_DEVARCH_PRESENT = 0x100000
	// Position of REVISION field.
	PPB_ITM_DEVARCH_REVISION_Pos = 0x10
	// Bit mask of REVISION field.
	PPB_ITM_DEVARCH_REVISION_Msk = 0xf0000
	// Position of ARCHVER field.
	PPB_ITM_DEVARCH_ARCHVER_Pos = 0xc
	// Bit mask of ARCHVER field.
	PPB_ITM_DEVARCH_ARCHVER_Msk = 0xf000
	// Position of ARCHPART field.
	PPB_ITM_DEVARCH_ARCHPART_Pos = 0x0
	// Bit mask of ARCHPART field.
	PPB_ITM_DEVARCH_ARCHPART_Msk = 0xfff

	// ITM_DEVTYPE: Provides CoreSight discovery information for the ITM
	// Position of SUB field.
	PPB_ITM_DEVTYPE_SUB_Pos = 0x4
	// Bit mask of SUB field.
	PPB_ITM_DEVTYPE_SUB_Msk = 0xf0
	// Position of MAJOR field.
	PPB_ITM_DEVTYPE_MAJOR_Pos = 0x0
	// Bit mask of MAJOR field.
	PPB_ITM_DEVTYPE_MAJOR_Msk = 0xf

	// ITM_PIDR4: Provides CoreSight discovery information for the ITM
	// Position of SIZE field.
	PPB_ITM_PIDR4_SIZE_Pos = 0x4
	// Bit mask of SIZE field.
	PPB_ITM_PIDR4_SIZE_Msk = 0xf0
	// Position of DES_2 field.
	PPB_ITM_PIDR4_DES_2_Pos = 0x0
	// Bit mask of DES_2 field.
	PPB_ITM_PIDR4_DES_2_Msk = 0xf

	// ITM_PIDR5: Provides CoreSight discovery information for the ITM
	// Position of ITM_PIDR5 field.
	PPB_ITM_PIDR5_ITM_PIDR5_Pos = 0x0
	// Bit mask of ITM_PIDR5 field.
	PPB_ITM_PIDR5_ITM_PIDR5_Msk = 0xffffffff

	// ITM_PIDR6: Provides CoreSight discovery information for the ITM
	// Position of ITM_PIDR6 field.
	PPB_ITM_PIDR6_ITM_PIDR6_Pos = 0x0
	// Bit mask of ITM_PIDR6 field.
	PPB_ITM_PIDR6_ITM_PIDR6_Msk = 0xffffffff

	// ITM_PIDR7: Provides CoreSight discovery information for the ITM
	// Position of ITM_PIDR7 field.
	PPB_ITM_PIDR7_ITM_PIDR7_Pos = 0x0
	// Bit mask of ITM_PIDR7 field.
	PPB_ITM_PIDR7_ITM_PIDR7_Msk = 0xffffffff

	// ITM_PIDR0: Provides CoreSight discovery information for the ITM
	// Position of PART_0 field.
	PPB_ITM_PIDR0_PART_0_Pos = 0x0
	// Bit mask of PART_0 field.
	PPB_ITM_PIDR0_PART_0_Msk = 0xff

	// ITM_PIDR1: Provides CoreSight discovery information for the ITM
	// Position of DES_0 field.
	PPB_ITM_PIDR1_DES_0_Pos = 0x4
	// Bit mask of DES_0 field.
	PPB_ITM_PIDR1_DES_0_Msk = 0xf0
	// Position of PART_1 field.
	PPB_ITM_PIDR1_PART_1_Pos = 0x0
	// Bit mask of PART_1 field.
	PPB_ITM_PIDR1_PART_1_Msk = 0xf

	// ITM_PIDR2: Provides CoreSight discovery information for the ITM
	// Position of REVISION field.
	PPB_ITM_PIDR2_REVISION_Pos = 0x4
	// Bit mask of REVISION field.
	PPB_ITM_PIDR2_REVISION_Msk = 0xf0
	// Position of JEDEC field.
	PPB_ITM_PIDR2_JEDEC_Pos = 0x3
	// Bit mask of JEDEC field.
	PPB_ITM_PIDR2_JEDEC_Msk = 0x8
	// Bit JEDEC.
	PPB_ITM_PIDR2_JEDEC = 0x8
	// Position of DES_1 field.
	PPB_ITM_PIDR2_DES_1_Pos = 0x0
	// Bit mask of DES_1 field.
	PPB_ITM_PIDR2_DES_1_Msk = 0x7

	// ITM_PIDR3: Provides CoreSight discovery information for the ITM
	// Position of REVAND field.
	PPB_ITM_PIDR3_REVAND_Pos = 0x4
	// Bit mask of REVAND field.
	PPB_ITM_PIDR3_REVAND_Msk = 0xf0
	// Position of CMOD field.
	PPB_ITM_PIDR3_CMOD_Pos = 0x0
	// Bit mask of CMOD field.
	PPB_ITM_PIDR3_CMOD_Msk = 0xf

	// ITM_CIDR0: Provides CoreSight discovery information for the ITM
	// Position of PRMBL_0 field.
	PPB_ITM_CIDR0_PRMBL_0_Pos = 0x0
	// Bit mask of PRMBL_0 field.
	PPB_ITM_CIDR0_PRMBL_0_Msk = 0xff

	// ITM_CIDR1: Provides CoreSight discovery information for the ITM
	// Position of CLASS field.
	PPB_ITM_CIDR1_CLASS_Pos = 0x4
	// Bit mask of CLASS field.
	PPB_ITM_CIDR1_CLASS_Msk = 0xf0
	// Position of PRMBL_1 field.
	PPB_ITM_CIDR1_PRMBL_1_Pos = 0x0
	// Bit mask of PRMBL_1 field.
	PPB_ITM_CIDR1_PRMBL_1_Msk = 0xf

	// ITM_CIDR2: Provides CoreSight discovery information for the ITM
	// Position of PRMBL_2 field.
	PPB_ITM_CIDR2_PRMBL_2_Pos = 0x0
	// Bit mask of PRMBL_2 field.
	PPB_ITM_CIDR2_PRMBL_2_Msk = 0xff

	// ITM_CIDR3: Provides CoreSight discovery information for the ITM
	// Position of PRMBL_3 field.
	PPB_ITM_CIDR3_PRMBL_3_Pos = 0x0
	// Bit mask of PRMBL_3 field.
	PPB_ITM_CIDR3_PRMBL_3_Msk = 0xff

	// DWT_CTRL: Provides configuration and status information for the DWT unit, and used to control features of the unit
	// Position of NUMCOMP field.
	PPB_DWT_CTRL_NUMCOMP_Pos = 0x1c
	// Bit mask of NUMCOMP field.
	PPB_DWT_CTRL_NUMCOMP_Msk = 0xf0000000
	// Position of NOTRCPKT field.
	PPB_DWT_CTRL_NOTRCPKT_Pos = 0x1b
	// Bit mask of NOTRCPKT field.
	PPB_DWT_CTRL_NOTRCPKT_Msk = 0x8000000
	// Bit NOTRCPKT.
	PPB_DWT_CTRL_NOTRCPKT = 0x8000000
	// Position of NOEXTTRIG field.
	PPB_DWT_CTRL_NOEXTTRIG_Pos = 0x1a
	// Bit mask of NOEXTTRIG field.
	PPB_DWT_CTRL_NOEXTTRIG_Msk = 0x4000000
	// Bit NOEXTTRIG.
	PPB_DWT_CTRL_NOEXTTRIG = 0x4000000
	// Position of NOCYCCNT field.
	PPB_DWT_CTRL_NOCYCCNT_Pos = 0x19
	// Bit mask of NOCYCCNT field.
	PPB_DWT_CTRL_NOCYCCNT_Msk = 0x2000000
	// Bit NOCYCCNT.
	PPB_DWT_CTRL_NOCYCCNT = 0x2000000
	// Position of NOPRFCNT field.
	PPB_DWT_CTRL_NOPRFCNT_Pos = 0x18
	// Bit mask of NOPRFCNT field.
	PPB_DWT_CTRL_NOPRFCNT_Msk = 0x1000000
	// Bit NOPRFCNT.
	PPB_DWT_CTRL_NOPRFCNT = 0x1000000
	// Position of CYCDISS field.
	PPB_DWT_CTRL_CYCDISS_Pos = 0x17
	// Bit mask of CYCDISS field.
	PPB_DWT_CTRL_CYCDISS_Msk = 0x800000
	// Bit CYCDISS.
	PPB_DWT_CTRL_CYCDISS = 0x800000
	// Position of CYCEVTENA field.
	PPB_DWT_CTRL_CYCEVTENA_Pos = 0x16
	// Bit mask of CYCEVTENA field.
	PPB_DWT_CTRL_CYCEVTENA_Msk = 0x400000
	// Bit CYCEVTENA.
	PPB_DWT_CTRL_CYCEVTENA = 0x400000
	// Position of FOLDEVTENA field.
	PPB_DWT_CTRL_FOLDEVTENA_Pos = 0x15
	// Bit mask of FOLDEVTENA field.
	PPB_DWT_CTRL_FOLDEVTENA_Msk = 0x200000
	// Bit FOLDEVTENA.
	PPB_DWT_CTRL_FOLDEVTENA = 0x200000
	// Position of LSUEVTENA field.
	PPB_DWT_CTRL_LSUEVTENA_Pos = 0x14
	// Bit mask of LSUEVTENA field.
	PPB_DWT_CTRL_LSUEVTENA_Msk = 0x100000
	// Bit LSUEVTENA.
	PPB_DWT_CTRL_LSUEVTENA = 0x100000
	// Position of SLEEPEVTENA field.
	PPB_DWT_CTRL_SLEEPEVTENA_Pos = 0x13
	// Bit mask of SLEEPEVTENA field.
	PPB_DWT_CTRL_SLEEPEVTENA_Msk = 0x80000
	// Bit SLEEPEVTENA.
	PPB_DWT_CTRL_SLEEPEVTENA = 0x80000
	// Position of EXCEVTENA field.
	PPB_DWT_CTRL_EXCEVTENA_Pos = 0x12
	// Bit mask of EXCEVTENA field.
	PPB_DWT_CTRL_EXCEVTENA_Msk = 0x40000
	// Bit EXCEVTENA.
	PPB_DWT_CTRL_EXCEVTENA = 0x40000
	// Position of CPIEVTENA field.
	PPB_DWT_CTRL_CPIEVTENA_Pos = 0x11
	// Bit mask of CPIEVTENA field.
	PPB_DWT_CTRL_CPIEVTENA_Msk = 0x20000
	// Bit CPIEVTENA.
	PPB_DWT_CTRL_CPIEVTENA = 0x20000
	// Position of EXTTRCENA field.
	PPB_DWT_CTRL_EXTTRCENA_Pos = 0x10
	// Bit mask of EXTTRCENA field.
	PPB_DWT_CTRL_EXTTRCENA_Msk = 0x10000
	// Bit EXTTRCENA.
	PPB_DWT_CTRL_EXTTRCENA = 0x10000
	// Position of PCSAMPLENA field.
	PPB_DWT_CTRL_PCSAMPLENA_Pos = 0xc
	// Bit mask of PCSAMPLENA field.
	PPB_DWT_CTRL_PCSAMPLENA_Msk = 0x1000
	// Bit PCSAMPLENA.
	PPB_DWT_CTRL_PCSAMPLENA = 0x1000
	// Position of SYNCTAP field.
	PPB_DWT_CTRL_SYNCTAP_Pos = 0xa
	// Bit mask of SYNCTAP field.
	PPB_DWT_CTRL_SYNCTAP_Msk = 0xc00
	// Position of CYCTAP field.
	PPB_DWT_CTRL_CYCTAP_Pos = 0x9
	// Bit mask of CYCTAP field.
	PPB_DWT_CTRL_CYCTAP_Msk = 0x200
	// Bit CYCTAP.
	PPB_DWT_CTRL_CYCTAP = 0x200
	// Position of POSTINIT field.
	PPB_DWT_CTRL_POSTINIT_Pos = 0x5
	// Bit mask of POSTINIT field.
	PPB_DWT_CTRL_POSTINIT_Msk = 0x1e0
	// Position of POSTPRESET field.
	PPB_DWT_CTRL_POSTPRESET_Pos = 0x1
	// Bit mask of POSTPRESET field.
	PPB_DWT_CTRL_POSTPRESET_Msk = 0x1e
	// Position of CYCCNTENA field.
	PPB_DWT_CTRL_CYCCNTENA_Pos = 0x0
	// Bit mask of CYCCNTENA field.
	PPB_DWT_CTRL_CYCCNTENA_Msk = 0x1
	// Bit CYCCNTENA.
	PPB_DWT_CTRL_CYCCNTENA = 0x1

	// DWT_CYCCNT: Shows or sets the value of the processor cycle counter, CYCCNT
	// Position of CYCCNT field.
	PPB_DWT_CYCCNT_CYCCNT_Pos = 0x0
	// Bit mask of CYCCNT field.
	PPB_DWT_CYCCNT_CYCCNT_Msk = 0xffffffff

	// DWT_EXCCNT: Counts the total cycles spent in exception processing
	// Position of EXCCNT field.
	PPB_DWT_EXCCNT_EXCCNT_Pos = 0x0
	// Bit mask of EXCCNT field.
	PPB_DWT_EXCCNT_EXCCNT_Msk = 0xff

	// DWT_LSUCNT: Increments on the additional cycles required to execute all load or store instructions
	// Position of LSUCNT field.
	PPB_DWT_LSUCNT_LSUCNT_Pos = 0x0
	// Bit mask of LSUCNT field.
	PPB_DWT_LSUCNT_LSUCNT_Msk = 0xff

	// DWT_FOLDCNT: Increments on the additional cycles required to execute all load or store instructions
	// Position of FOLDCNT field.
	PPB_DWT_FOLDCNT_FOLDCNT_Pos = 0x0
	// Bit mask of FOLDCNT field.
	PPB_DWT_FOLDCNT_FOLDCNT_Msk = 0xff

	// DWT_COMP0: Provides a reference value for use by watchpoint comparator 0
	// Position of DWT_COMP0 field.
	PPB_DWT_COMP0_DWT_COMP0_Pos = 0x0
	// Bit mask of DWT_COMP0 field.
	PPB_DWT_COMP0_DWT_COMP0_Msk = 0xffffffff

	// DWT_FUNCTION0: Controls the operation of watchpoint comparator 0
	// Position of ID field.
	PPB_DWT_FUNCTION0_ID_Pos = 0x1b
	// Bit mask of ID field.
	PPB_DWT_FUNCTION0_ID_Msk = 0xf8000000
	// Position of MATCHED field.
	PPB_DWT_FUNCTION0_MATCHED_Pos = 0x18
	// Bit mask of MATCHED field.
	PPB_DWT_FUNCTION0_MATCHED_Msk = 0x1000000
	// Bit MATCHED.
	PPB_DWT_FUNCTION0_MATCHED = 0x1000000
	// Position of DATAVSIZE field.
	PPB_DWT_FUNCTION0_DATAVSIZE_Pos = 0xa
	// Bit mask of DATAVSIZE field.
	PPB_DWT_FUNCTION0_DATAVSIZE_Msk = 0xc00
	// Position of ACTION field.
	PPB_DWT_FUNCTION0_ACTION_Pos = 0x4
	// Bit mask of ACTION field.
	PPB_DWT_FUNCTION0_ACTION_Msk = 0x30
	// Position of MATCH field.
	PPB_DWT_FUNCTION0_MATCH_Pos = 0x0
	// Bit mask of MATCH field.
	PPB_DWT_FUNCTION0_MATCH_Msk = 0xf

	// DWT_COMP1: Provides a reference value for use by watchpoint comparator 1
	// Position of DWT_COMP1 field.
	PPB_DWT_COMP1_DWT_COMP1_Pos = 0x0
	// Bit mask of DWT_COMP1 field.
	PPB_DWT_COMP1_DWT_COMP1_Msk = 0xffffffff

	// DWT_FUNCTION1: Controls the operation of watchpoint comparator 1
	// Position of ID field.
	PPB_DWT_FUNCTION1_ID_Pos = 0x1b
	// Bit mask of ID field.
	PPB_DWT_FUNCTION1_ID_Msk = 0xf8000000
	// Position of MATCHED field.
	PPB_DWT_FUNCTION1_MATCHED_Pos = 0x18
	// Bit mask of MATCHED field.
	PPB_DWT_FUNCTION1_MATCHED_Msk = 0x1000000
	// Bit MATCHED.
	PPB_DWT_FUNCTION1_MATCHED = 0x1000000
	// Position of DATAVSIZE field.
	PPB_DWT_FUNCTION1_DATAVSIZE_Pos = 0xa
	// Bit mask of DATAVSIZE field.
	PPB_DWT_FUNCTION1_DATAVSIZE_Msk = 0xc00
	// Position of ACTION field.
	PPB_DWT_FUNCTION1_ACTION_Pos = 0x4
	// Bit mask of ACTION field.
	PPB_DWT_FUNCTION1_ACTION_Msk = 0x30
	// Position of MATCH field.
	PPB_DWT_FUNCTION1_MATCH_Pos = 0x0
	// Bit mask of MATCH field.
	PPB_DWT_FUNCTION1_MATCH_Msk = 0xf

	// DWT_COMP2: Provides a reference value for use by watchpoint comparator 2
	// Position of DWT_COMP2 field.
	PPB_DWT_COMP2_DWT_COMP2_Pos = 0x0
	// Bit mask of DWT_COMP2 field.
	PPB_DWT_COMP2_DWT_COMP2_Msk = 0xffffffff

	// DWT_FUNCTION2: Controls the operation of watchpoint comparator 2
	// Position of ID field.
	PPB_DWT_FUNCTION2_ID_Pos = 0x1b
	// Bit mask of ID field.
	PPB_DWT_FUNCTION2_ID_Msk = 0xf8000000
	// Position of MATCHED field.
	PPB_DWT_FUNCTION2_MATCHED_Pos = 0x18
	// Bit mask of MATCHED field.
	PPB_DWT_FUNCTION2_MATCHED_Msk = 0x1000000
	// Bit MATCHED.
	PPB_DWT_FUNCTION2_MATCHED = 0x1000000
	// Position of DATAVSIZE field.
	PPB_DWT_FUNCTION2_DATAVSIZE_Pos = 0xa
	// Bit mask of DATAVSIZE field.
	PPB_DWT_FUNCTION2_DATAVSIZE_Msk = 0xc00
	// Position of ACTION field.
	PPB_DWT_FUNCTION2_ACTION_Pos = 0x4
	// Bit mask of ACTION field.
	PPB_DWT_FUNCTION2_ACTION_Msk = 0x30
	// Position of MATCH field.
	PPB_DWT_FUNCTION2_MATCH_Pos = 0x0
	// Bit mask of MATCH field.
	PPB_DWT_FUNCTION2_MATCH_Msk = 0xf

	// DWT_COMP3: Provides a reference value for use by watchpoint comparator 3
	// Position of DWT_COMP3 field.
	PPB_DWT_COMP3_DWT_COMP3_Pos = 0x0
	// Bit mask of DWT_COMP3 field.
	PPB_DWT_COMP3_DWT_COMP3_Msk = 0xffffffff

	// DWT_FUNCTION3: Controls the operation of watchpoint comparator 3
	// Position of ID field.
	PPB_DWT_FUNCTION3_ID_Pos = 0x1b
	// Bit mask of ID field.
	PPB_DWT_FUNCTION3_ID_Msk = 0xf8000000
	// Position of MATCHED field.
	PPB_DWT_FUNCTION3_MATCHED_Pos = 0x18
	// Bit mask of MATCHED field.
	PPB_DWT_FUNCTION3_MATCHED_Msk = 0x1000000
	// Bit MATCHED.
	PPB_DWT_FUNCTION3_MATCHED = 0x1000000
	// Position of DATAVSIZE field.
	PPB_DWT_FUNCTION3_DATAVSIZE_Pos = 0xa
	// Bit mask of DATAVSIZE field.
	PPB_DWT_FUNCTION3_DATAVSIZE_Msk = 0xc00
	// Position of ACTION field.
	PPB_DWT_FUNCTION3_ACTION_Pos = 0x4
	// Bit mask of ACTION field.
	PPB_DWT_FUNCTION3_ACTION_Msk = 0x30
	// Position of MATCH field.
	PPB_DWT_FUNCTION3_MATCH_Pos = 0x0
	// Bit mask of MATCH field.
	PPB_DWT_FUNCTION3_MATCH_Msk = 0xf

	// DWT_DEVARCH: Provides CoreSight discovery information for the DWT
	// Position of ARCHITECT field.
	PPB_DWT_DEVARCH_ARCHITECT_Pos = 0x15
	// Bit mask of ARCHITECT field.
	PPB_DWT_DEVARCH_ARCHITECT_Msk = 0xffe00000
	// Position of PRESENT field.
	PPB_DWT_DEVARCH_PRESENT_Pos = 0x14
	// Bit mask of PRESENT field.
	PPB_DWT_DEVARCH_PRESENT_Msk = 0x100000
	// Bit PRESENT.
	PPB_DWT_DEVARCH_PRESENT = 0x100000
	// Position of REVISION field.
	PPB_DWT_DEVARCH_REVISION_Pos = 0x10
	// Bit mask of REVISION field.
	PPB_DWT_DEVARCH_REVISION_Msk = 0xf0000
	// Position of ARCHVER field.
	PPB_DWT_DEVARCH_ARCHVER_Pos = 0xc
	// Bit mask of ARCHVER field.
	PPB_DWT_DEVARCH_ARCHVER_Msk = 0xf000
	// Position of ARCHPART field.
	PPB_DWT_DEVARCH_ARCHPART_Pos = 0x0
	// Bit mask of ARCHPART field.
	PPB_DWT_DEVARCH_ARCHPART_Msk = 0xfff

	// DWT_DEVTYPE: Provides CoreSight discovery information for the DWT
	// Position of SUB field.
	PPB_DWT_DEVTYPE_SUB_Pos = 0x4
	// Bit mask of SUB field.
	PPB_DWT_DEVTYPE_SUB_Msk = 0xf0
	// Position of MAJOR field.
	PPB_DWT_DEVTYPE_MAJOR_Pos = 0x0
	// Bit mask of MAJOR field.
	PPB_DWT_DEVTYPE_MAJOR_Msk = 0xf

	// DWT_PIDR4: Provides CoreSight discovery information for the DWT
	// Position of SIZE field.
	PPB_DWT_PIDR4_SIZE_Pos = 0x4
	// Bit mask of SIZE field.
	PPB_DWT_PIDR4_SIZE_Msk = 0xf0
	// Position of DES_2 field.
	PPB_DWT_PIDR4_DES_2_Pos = 0x0
	// Bit mask of DES_2 field.
	PPB_DWT_PIDR4_DES_2_Msk = 0xf

	// DWT_PIDR5: Provides CoreSight discovery information for the DWT
	// Position of DWT_PIDR5 field.
	PPB_DWT_PIDR5_DWT_PIDR5_Pos = 0x0
	// Bit mask of DWT_PIDR5 field.
	PPB_DWT_PIDR5_DWT_PIDR5_Msk = 0xffffffff

	// DWT_PIDR6: Provides CoreSight discovery information for the DWT
	// Position of DWT_PIDR6 field.
	PPB_DWT_PIDR6_DWT_PIDR6_Pos = 0x0
	// Bit mask of DWT_PIDR6 field.
	PPB_DWT_PIDR6_DWT_PIDR6_Msk = 0xffffffff

	// DWT_PIDR7: Provides CoreSight discovery information for the DWT
	// Position of DWT_PIDR7 field.
	PPB_DWT_PIDR7_DWT_PIDR7_Pos = 0x0
	// Bit mask of DWT_PIDR7 field.
	PPB_DWT_PIDR7_DWT_PIDR7_Msk = 0xffffffff

	// DWT_PIDR0: Provides CoreSight discovery information for the DWT
	// Position of PART_0 field.
	PPB_DWT_PIDR0_PART_0_Pos = 0x0
	// Bit mask of PART_0 field.
	PPB_DWT_PIDR0_PART_0_Msk = 0xff

	// DWT_PIDR1: Provides CoreSight discovery information for the DWT
	// Position of DES_0 field.
	PPB_DWT_PIDR1_DES_0_Pos = 0x4
	// Bit mask of DES_0 field.
	PPB_DWT_PIDR1_DES_0_Msk = 0xf0
	// Position of PART_1 field.
	PPB_DWT_PIDR1_PART_1_Pos = 0x0
	// Bit mask of PART_1 field.
	PPB_DWT_PIDR1_PART_1_Msk = 0xf

	// DWT_PIDR2: Provides CoreSight discovery information for the DWT
	// Position of REVISION field.
	PPB_DWT_PIDR2_REVISION_Pos = 0x4
	// Bit mask of REVISION field.
	PPB_DWT_PIDR2_REVISION_Msk = 0xf0
	// Position of JEDEC field.
	PPB_DWT_PIDR2_JEDEC_Pos = 0x3
	// Bit mask of JEDEC field.
	PPB_DWT_PIDR2_JEDEC_Msk = 0x8
	// Bit JEDEC.
	PPB_DWT_PIDR2_JEDEC = 0x8
	// Position of DES_1 field.
	PPB_DWT_PIDR2_DES_1_Pos = 0x0
	// Bit mask of DES_1 field.
	PPB_DWT_PIDR2_DES_1_Msk = 0x7

	// DWT_PIDR3: Provides CoreSight discovery information for the DWT
	// Position of REVAND field.
	PPB_DWT_PIDR3_REVAND_Pos = 0x4
	// Bit mask of REVAND field.
	PPB_DWT_PIDR3_REVAND_Msk = 0xf0
	// Position of CMOD field.
	PPB_DWT_PIDR3_CMOD_Pos = 0x0
	// Bit mask of CMOD field.
	PPB_DWT_PIDR3_CMOD_Msk = 0xf

	// DWT_CIDR0: Provides CoreSight discovery information for the DWT
	// Position of PRMBL_0 field.
	PPB_DWT_CIDR0_PRMBL_0_Pos = 0x0
	// Bit mask of PRMBL_0 field.
	PPB_DWT_CIDR0_PRMBL_0_Msk = 0xff

	// DWT_CIDR1: Provides CoreSight discovery information for the DWT
	// Position of CLASS field.
	PPB_DWT_CIDR1_CLASS_Pos = 0x4
	// Bit mask of CLASS field.
	PPB_DWT_CIDR1_CLASS_Msk = 0xf0
	// Position of PRMBL_1 field.
	PPB_DWT_CIDR1_PRMBL_1_Pos = 0x0
	// Bit mask of PRMBL_1 field.
	PPB_DWT_CIDR1_PRMBL_1_Msk = 0xf

	// DWT_CIDR2: Provides CoreSight discovery information for the DWT
	// Position of PRMBL_2 field.
	PPB_DWT_CIDR2_PRMBL_2_Pos = 0x0
	// Bit mask of PRMBL_2 field.
	PPB_DWT_CIDR2_PRMBL_2_Msk = 0xff

	// DWT_CIDR3: Provides CoreSight discovery information for the DWT
	// Position of PRMBL_3 field.
	PPB_DWT_CIDR3_PRMBL_3_Pos = 0x0
	// Bit mask of PRMBL_3 field.
	PPB_DWT_CIDR3_PRMBL_3_Msk = 0xff

	// FP_CTRL: Provides FPB implementation information, and the global enable for the FPB unit
	// Position of REV field.
	PPB_FP_CTRL_REV_Pos = 0x1c
	// Bit mask of REV field.
	PPB_FP_CTRL_REV_Msk = 0xf0000000
	// Position of NUM_CODE_14_12_ field.
	PPB_FP_CTRL_NUM_CODE_14_12__Pos = 0xc
	// Bit mask of NUM_CODE_14_12_ field.
	PPB_FP_CTRL_NUM_CODE_14_12__Msk = 0x7000
	// Position of NUM_LIT field.
	PPB_FP_CTRL_NUM_LIT_Pos = 0x8
	// Bit mask of NUM_LIT field.
	PPB_FP_CTRL_NUM_LIT_Msk = 0xf00
	// Position of NUM_CODE_7_4_ field.
	PPB_FP_CTRL_NUM_CODE_7_4__Pos = 0x4
	// Bit mask of NUM_CODE_7_4_ field.
	PPB_FP_CTRL_NUM_CODE_7_4__Msk = 0xf0
	// Position of KEY field.
	PPB_FP_CTRL_KEY_Pos = 0x1
	// Bit mask of KEY field.
	PPB_FP_CTRL_KEY_Msk = 0x2
	// Bit KEY.
	PPB_FP_CTRL_KEY = 0x2
	// Position of ENABLE field.
	PPB_FP_CTRL_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	PPB_FP_CTRL_ENABLE_Msk = 0x1
	// Bit ENABLE.
	PPB_FP_CTRL_ENABLE = 0x1

	// FP_REMAP: Indicates whether the implementation supports Flash Patch remap and, if it does, holds the target address for remap
	// Position of RMPSPT field.
	PPB_FP_REMAP_RMPSPT_Pos = 0x1d
	// Bit mask of RMPSPT field.
	PPB_FP_REMAP_RMPSPT_Msk = 0x20000000
	// Bit RMPSPT.
	PPB_FP_REMAP_RMPSPT = 0x20000000
	// Position of REMAP field.
	PPB_FP_REMAP_REMAP_Pos = 0x5
	// Bit mask of REMAP field.
	PPB_FP_REMAP_REMAP_Msk = 0x1fffffe0

	// FP_COMP0: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
	// Position of BE field.
	PPB_FP_COMP0_BE_Pos = 0x0
	// Bit mask of BE field.
	PPB_FP_COMP0_BE_Msk = 0x1
	// Bit BE.
	PPB_FP_COMP0_BE = 0x1

	// FP_COMP1: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
	// Position of BE field.
	PPB_FP_COMP1_BE_Pos = 0x0
	// Bit mask of BE field.
	PPB_FP_COMP1_BE_Msk = 0x1
	// Bit BE.
	PPB_FP_COMP1_BE = 0x1

	// FP_COMP2: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
	// Position of BE field.
	PPB_FP_COMP2_BE_Pos = 0x0
	// Bit mask of BE field.
	PPB_FP_COMP2_BE_Msk = 0x1
	// Bit BE.
	PPB_FP_COMP2_BE = 0x1

	// FP_COMP3: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
	// Position of BE field.
	PPB_FP_COMP3_BE_Pos = 0x0
	// Bit mask of BE field.
	PPB_FP_COMP3_BE_Msk = 0x1
	// Bit BE.
	PPB_FP_COMP3_BE = 0x1

	// FP_COMP4: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
	// Position of BE field.
	PPB_FP_COMP4_BE_Pos = 0x0
	// Bit mask of BE field.
	PPB_FP_COMP4_BE_Msk = 0x1
	// Bit BE.
	PPB_FP_COMP4_BE = 0x1

	// FP_COMP5: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
	// Position of BE field.
	PPB_FP_COMP5_BE_Pos = 0x0
	// Bit mask of BE field.
	PPB_FP_COMP5_BE_Msk = 0x1
	// Bit BE.
	PPB_FP_COMP5_BE = 0x1

	// FP_COMP6: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
	// Position of BE field.
	PPB_FP_COMP6_BE_Pos = 0x0
	// Bit mask of BE field.
	PPB_FP_COMP6_BE_Msk = 0x1
	// Bit BE.
	PPB_FP_COMP6_BE = 0x1

	// FP_COMP7: Holds an address for comparison. The effect of the match depends on the configuration of the FPB and whether the comparator is an instruction address comparator or a literal address comparator
	// Position of BE field.
	PPB_FP_COMP7_BE_Pos = 0x0
	// Bit mask of BE field.
	PPB_FP_COMP7_BE_Msk = 0x1
	// Bit BE.
	PPB_FP_COMP7_BE = 0x1

	// FP_DEVARCH: Provides CoreSight discovery information for the FPB
	// Position of ARCHITECT field.
	PPB_FP_DEVARCH_ARCHITECT_Pos = 0x15
	// Bit mask of ARCHITECT field.
	PPB_FP_DEVARCH_ARCHITECT_Msk = 0xffe00000
	// Position of PRESENT field.
	PPB_FP_DEVARCH_PRESENT_Pos = 0x14
	// Bit mask of PRESENT field.
	PPB_FP_DEVARCH_PRESENT_Msk = 0x100000
	// Bit PRESENT.
	PPB_FP_DEVARCH_PRESENT = 0x100000
	// Position of REVISION field.
	PPB_FP_DEVARCH_REVISION_Pos = 0x10
	// Bit mask of REVISION field.
	PPB_FP_DEVARCH_REVISION_Msk = 0xf0000
	// Position of ARCHVER field.
	PPB_FP_DEVARCH_ARCHVER_Pos = 0xc
	// Bit mask of ARCHVER field.
	PPB_FP_DEVARCH_ARCHVER_Msk = 0xf000
	// Position of ARCHPART field.
	PPB_FP_DEVARCH_ARCHPART_Pos = 0x0
	// Bit mask of ARCHPART field.
	PPB_FP_DEVARCH_ARCHPART_Msk = 0xfff

	// FP_DEVTYPE: Provides CoreSight discovery information for the FPB
	// Position of SUB field.
	PPB_FP_DEVTYPE_SUB_Pos = 0x4
	// Bit mask of SUB field.
	PPB_FP_DEVTYPE_SUB_Msk = 0xf0
	// Position of MAJOR field.
	PPB_FP_DEVTYPE_MAJOR_Pos = 0x0
	// Bit mask of MAJOR field.
	PPB_FP_DEVTYPE_MAJOR_Msk = 0xf

	// FP_PIDR4: Provides CoreSight discovery information for the FP
	// Position of SIZE field.
	PPB_FP_PIDR4_SIZE_Pos = 0x4
	// Bit mask of SIZE field.
	PPB_FP_PIDR4_SIZE_Msk = 0xf0
	// Position of DES_2 field.
	PPB_FP_PIDR4_DES_2_Pos = 0x0
	// Bit mask of DES_2 field.
	PPB_FP_PIDR4_DES_2_Msk = 0xf

	// FP_PIDR5: Provides CoreSight discovery information for the FP
	// Position of FP_PIDR5 field.
	PPB_FP_PIDR5_FP_PIDR5_Pos = 0x0
	// Bit mask of FP_PIDR5 field.
	PPB_FP_PIDR5_FP_PIDR5_Msk = 0xffffffff

	// FP_PIDR6: Provides CoreSight discovery information for the FP
	// Position of FP_PIDR6 field.
	PPB_FP_PIDR6_FP_PIDR6_Pos = 0x0
	// Bit mask of FP_PIDR6 field.
	PPB_FP_PIDR6_FP_PIDR6_Msk = 0xffffffff

	// FP_PIDR7: Provides CoreSight discovery information for the FP
	// Position of FP_PIDR7 field.
	PPB_FP_PIDR7_FP_PIDR7_Pos = 0x0
	// Bit mask of FP_PIDR7 field.
	PPB_FP_PIDR7_FP_PIDR7_Msk = 0xffffffff

	// FP_PIDR0: Provides CoreSight discovery information for the FP
	// Position of PART_0 field.
	PPB_FP_PIDR0_PART_0_Pos = 0x0
	// Bit mask of PART_0 field.
	PPB_FP_PIDR0_PART_0_Msk = 0xff

	// FP_PIDR1: Provides CoreSight discovery information for the FP
	// Position of DES_0 field.
	PPB_FP_PIDR1_DES_0_Pos = 0x4
	// Bit mask of DES_0 field.
	PPB_FP_PIDR1_DES_0_Msk = 0xf0
	// Position of PART_1 field.
	PPB_FP_PIDR1_PART_1_Pos = 0x0
	// Bit mask of PART_1 field.
	PPB_FP_PIDR1_PART_1_Msk = 0xf

	// FP_PIDR2: Provides CoreSight discovery information for the FP
	// Position of REVISION field.
	PPB_FP_PIDR2_REVISION_Pos = 0x4
	// Bit mask of REVISION field.
	PPB_FP_PIDR2_REVISION_Msk = 0xf0
	// Position of JEDEC field.
	PPB_FP_PIDR2_JEDEC_Pos = 0x3
	// Bit mask of JEDEC field.
	PPB_FP_PIDR2_JEDEC_Msk = 0x8
	// Bit JEDEC.
	PPB_FP_PIDR2_JEDEC = 0x8
	// Position of DES_1 field.
	PPB_FP_PIDR2_DES_1_Pos = 0x0
	// Bit mask of DES_1 field.
	PPB_FP_PIDR2_DES_1_Msk = 0x7

	// FP_PIDR3: Provides CoreSight discovery information for the FP
	// Position of REVAND field.
	PPB_FP_PIDR3_REVAND_Pos = 0x4
	// Bit mask of REVAND field.
	PPB_FP_PIDR3_REVAND_Msk = 0xf0
	// Position of CMOD field.
	PPB_FP_PIDR3_CMOD_Pos = 0x0
	// Bit mask of CMOD field.
	PPB_FP_PIDR3_CMOD_Msk = 0xf

	// FP_CIDR0: Provides CoreSight discovery information for the FP
	// Position of PRMBL_0 field.
	PPB_FP_CIDR0_PRMBL_0_Pos = 0x0
	// Bit mask of PRMBL_0 field.
	PPB_FP_CIDR0_PRMBL_0_Msk = 0xff

	// FP_CIDR1: Provides CoreSight discovery information for the FP
	// Position of CLASS field.
	PPB_FP_CIDR1_CLASS_Pos = 0x4
	// Bit mask of CLASS field.
	PPB_FP_CIDR1_CLASS_Msk = 0xf0
	// Position of PRMBL_1 field.
	PPB_FP_CIDR1_PRMBL_1_Pos = 0x0
	// Bit mask of PRMBL_1 field.
	PPB_FP_CIDR1_PRMBL_1_Msk = 0xf

	// FP_CIDR2: Provides CoreSight discovery information for the FP
	// Position of PRMBL_2 field.
	PPB_FP_CIDR2_PRMBL_2_Pos = 0x0
	// Bit mask of PRMBL_2 field.
	PPB_FP_CIDR2_PRMBL_2_Msk = 0xff

	// FP_CIDR3: Provides CoreSight discovery information for the FP
	// Position of PRMBL_3 field.
	PPB_FP_CIDR3_PRMBL_3_Pos = 0x0
	// Bit mask of PRMBL_3 field.
	PPB_FP_CIDR3_PRMBL_3_Msk = 0xff

	// ICTR: Provides information about the interrupt controller
	// Position of INTLINESNUM field.
	PPB_ICTR_INTLINESNUM_Pos = 0x0
	// Bit mask of INTLINESNUM field.
	PPB_ICTR_INTLINESNUM_Msk = 0xf

	// ACTLR: Provides IMPLEMENTATION DEFINED configuration and control options
	// Position of EXTEXCLALL field.
	PPB_ACTLR_EXTEXCLALL_Pos = 0x1d
	// Bit mask of EXTEXCLALL field.
	PPB_ACTLR_EXTEXCLALL_Msk = 0x20000000
	// Bit EXTEXCLALL.
	PPB_ACTLR_EXTEXCLALL = 0x20000000
	// Position of DISITMATBFLUSH field.
	PPB_ACTLR_DISITMATBFLUSH_Pos = 0xc
	// Bit mask of DISITMATBFLUSH field.
	PPB_ACTLR_DISITMATBFLUSH_Msk = 0x1000
	// Bit DISITMATBFLUSH.
	PPB_ACTLR_DISITMATBFLUSH = 0x1000
	// Position of FPEXCODIS field.
	PPB_ACTLR_FPEXCODIS_Pos = 0xa
	// Bit mask of FPEXCODIS field.
	PPB_ACTLR_FPEXCODIS_Msk = 0x400
	// Bit FPEXCODIS.
	PPB_ACTLR_FPEXCODIS = 0x400
	// Position of DISOOFP field.
	PPB_ACTLR_DISOOFP_Pos = 0x9
	// Bit mask of DISOOFP field.
	PPB_ACTLR_DISOOFP_Msk = 0x200
	// Bit DISOOFP.
	PPB_ACTLR_DISOOFP = 0x200
	// Position of DISFOLD field.
	PPB_ACTLR_DISFOLD_Pos = 0x2
	// Bit mask of DISFOLD field.
	PPB_ACTLR_DISFOLD_Msk = 0x4
	// Bit DISFOLD.
	PPB_ACTLR_DISFOLD = 0x4
	// Position of DISMCYCINT field.
	PPB_ACTLR_DISMCYCINT_Pos = 0x0
	// Bit mask of DISMCYCINT field.
	PPB_ACTLR_DISMCYCINT_Msk = 0x1
	// Bit DISMCYCINT.
	PPB_ACTLR_DISMCYCINT = 0x1

	// SYST_CSR: Use the SysTick Control and Status Register to enable the SysTick features.
	// Position of COUNTFLAG field.
	PPB_SYST_CSR_COUNTFLAG_Pos = 0x10
	// Bit mask of COUNTFLAG field.
	PPB_SYST_CSR_COUNTFLAG_Msk = 0x10000
	// Bit COUNTFLAG.
	PPB_SYST_CSR_COUNTFLAG = 0x10000
	// Position of CLKSOURCE field.
	PPB_SYST_CSR_CLKSOURCE_Pos = 0x2
	// Bit mask of CLKSOURCE field.
	PPB_SYST_CSR_CLKSOURCE_Msk = 0x4
	// Bit CLKSOURCE.
	PPB_SYST_CSR_CLKSOURCE = 0x4
	// Position of TICKINT field.
	PPB_SYST_CSR_TICKINT_Pos = 0x1
	// Bit mask of TICKINT field.
	PPB_SYST_CSR_TICKINT_Msk = 0x2
	// Bit TICKINT.
	PPB_SYST_CSR_TICKINT = 0x2
	// Position of ENABLE field.
	PPB_SYST_CSR_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	PPB_SYST_CSR_ENABLE_Msk = 0x1
	// Bit ENABLE.
	PPB_SYST_CSR_ENABLE = 0x1

	// SYST_RVR: Use the SysTick Reload Value Register to specify the start value to load into the current value register when the counter reaches 0. It can be any value between 0 and 0x00FFFFFF. A start value of 0 is possible, but has no effect because the SysTick interrupt and COUNTFLAG are activated when counting from 1 to 0. The reset value of this register is UNKNOWN. To generate a multi-shot timer with a period of N processor clock cycles, use a RELOAD value of N-1. For example, if the SysTick interrupt is required every 100 clock pulses, set RELOAD to 99.
	// Position of RELOAD field.
	PPB_SYST_RVR_RELOAD_Pos = 0x0
	// Bit mask of RELOAD field.
	PPB_SYST_RVR_RELOAD_Msk = 0xffffff

	// SYST_CVR: Use the SysTick Current Value Register to find the current value in the register. The reset value of this register is UNKNOWN.
	// Position of CURRENT field.
	PPB_SYST_CVR_CURRENT_Pos = 0x0
	// Bit mask of CURRENT field.
	PPB_SYST_CVR_CURRENT_Msk = 0xffffff

	// SYST_CALIB: Use the SysTick Calibration Value Register to enable software to scale to any required speed using divide and multiply.
	// Position of NOREF field.
	PPB_SYST_CALIB_NOREF_Pos = 0x1f
	// Bit mask of NOREF field.
	PPB_SYST_CALIB_NOREF_Msk = 0x80000000
	// Bit NOREF.
	PPB_SYST_CALIB_NOREF = 0x80000000
	// Position of SKEW field.
	PPB_SYST_CALIB_SKEW_Pos = 0x1e
	// Bit mask of SKEW field.
	PPB_SYST_CALIB_SKEW_Msk = 0x40000000
	// Bit SKEW.
	PPB_SYST_CALIB_SKEW = 0x40000000
	// Position of TENMS field.
	PPB_SYST_CALIB_TENMS_Pos = 0x0
	// Bit mask of TENMS field.
	PPB_SYST_CALIB_TENMS_Msk = 0xffffff

	// NVIC_ISER0: Enables or reads the enabled state of each group of 32 interrupts
	// Position of SETENA field.
	PPB_NVIC_ISER0_SETENA_Pos = 0x0
	// Bit mask of SETENA field.
	PPB_NVIC_ISER0_SETENA_Msk = 0xffffffff

	// NVIC_ISER1: Enables or reads the enabled state of each group of 32 interrupts
	// Position of SETENA field.
	PPB_NVIC_ISER1_SETENA_Pos = 0x0
	// Bit mask of SETENA field.
	PPB_NVIC_ISER1_SETENA_Msk = 0xffffffff

	// NVIC_ICER0: Clears or reads the enabled state of each group of 32 interrupts
	// Position of CLRENA field.
	PPB_NVIC_ICER0_CLRENA_Pos = 0x0
	// Bit mask of CLRENA field.
	PPB_NVIC_ICER0_CLRENA_Msk = 0xffffffff

	// NVIC_ICER1: Clears or reads the enabled state of each group of 32 interrupts
	// Position of CLRENA field.
	PPB_NVIC_ICER1_CLRENA_Pos = 0x0
	// Bit mask of CLRENA field.
	PPB_NVIC_ICER1_CLRENA_Msk = 0xffffffff

	// NVIC_ISPR0: Enables or reads the pending state of each group of 32 interrupts
	// Position of SETPEND field.
	PPB_NVIC_ISPR0_SETPEND_Pos = 0x0
	// Bit mask of SETPEND field.
	PPB_NVIC_ISPR0_SETPEND_Msk = 0xffffffff

	// NVIC_ISPR1: Enables or reads the pending state of each group of 32 interrupts
	// Position of SETPEND field.
	PPB_NVIC_ISPR1_SETPEND_Pos = 0x0
	// Bit mask of SETPEND field.
	PPB_NVIC_ISPR1_SETPEND_Msk = 0xffffffff

	// NVIC_ICPR0: Clears or reads the pending state of each group of 32 interrupts
	// Position of CLRPEND field.
	PPB_NVIC_ICPR0_CLRPEND_Pos = 0x0
	// Bit mask of CLRPEND field.
	PPB_NVIC_ICPR0_CLRPEND_Msk = 0xffffffff

	// NVIC_ICPR1: Clears or reads the pending state of each group of 32 interrupts
	// Position of CLRPEND field.
	PPB_NVIC_ICPR1_CLRPEND_Pos = 0x0
	// Bit mask of CLRPEND field.
	PPB_NVIC_ICPR1_CLRPEND_Msk = 0xffffffff

	// NVIC_IABR0: For each group of 32 interrupts, shows the active state of each interrupt
	// Position of ACTIVE field.
	PPB_NVIC_IABR0_ACTIVE_Pos = 0x0
	// Bit mask of ACTIVE field.
	PPB_NVIC_IABR0_ACTIVE_Msk = 0xffffffff

	// NVIC_IABR1: For each group of 32 interrupts, shows the active state of each interrupt
	// Position of ACTIVE field.
	PPB_NVIC_IABR1_ACTIVE_Pos = 0x0
	// Bit mask of ACTIVE field.
	PPB_NVIC_IABR1_ACTIVE_Msk = 0xffffffff

	// NVIC_ITNS0: For each group of 32 interrupts, determines whether each interrupt targets Non-secure or Secure state
	// Position of ITNS field.
	PPB_NVIC_ITNS0_ITNS_Pos = 0x0
	// Bit mask of ITNS field.
	PPB_NVIC_ITNS0_ITNS_Msk = 0xffffffff

	// NVIC_ITNS1: For each group of 32 interrupts, determines whether each interrupt targets Non-secure or Secure state
	// Position of ITNS field.
	PPB_NVIC_ITNS1_ITNS_Pos = 0x0
	// Bit mask of ITNS field.
	PPB_NVIC_ITNS1_ITNS_Msk = 0xffffffff

	// NVIC_IPR0: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR0_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR0_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR0_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR0_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR0_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR0_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR0_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR0_PRI_N0_Msk = 0xf0

	// NVIC_IPR1: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR1_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR1_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR1_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR1_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR1_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR1_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR1_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR1_PRI_N0_Msk = 0xf0

	// NVIC_IPR2: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR2_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR2_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR2_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR2_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR2_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR2_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR2_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR2_PRI_N0_Msk = 0xf0

	// NVIC_IPR3: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR3_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR3_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR3_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR3_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR3_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR3_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR3_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR3_PRI_N0_Msk = 0xf0

	// NVIC_IPR4: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR4_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR4_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR4_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR4_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR4_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR4_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR4_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR4_PRI_N0_Msk = 0xf0

	// NVIC_IPR5: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR5_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR5_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR5_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR5_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR5_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR5_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR5_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR5_PRI_N0_Msk = 0xf0

	// NVIC_IPR6: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR6_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR6_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR6_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR6_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR6_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR6_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR6_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR6_PRI_N0_Msk = 0xf0

	// NVIC_IPR7: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR7_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR7_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR7_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR7_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR7_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR7_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR7_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR7_PRI_N0_Msk = 0xf0

	// NVIC_IPR8: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR8_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR8_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR8_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR8_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR8_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR8_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR8_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR8_PRI_N0_Msk = 0xf0

	// NVIC_IPR9: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR9_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR9_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR9_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR9_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR9_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR9_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR9_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR9_PRI_N0_Msk = 0xf0

	// NVIC_IPR10: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR10_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR10_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR10_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR10_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR10_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR10_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR10_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR10_PRI_N0_Msk = 0xf0

	// NVIC_IPR11: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR11_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR11_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR11_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR11_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR11_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR11_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR11_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR11_PRI_N0_Msk = 0xf0

	// NVIC_IPR12: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR12_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR12_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR12_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR12_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR12_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR12_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR12_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR12_PRI_N0_Msk = 0xf0

	// NVIC_IPR13: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR13_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR13_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR13_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR13_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR13_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR13_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR13_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR13_PRI_N0_Msk = 0xf0

	// NVIC_IPR14: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR14_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR14_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR14_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR14_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR14_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR14_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR14_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR14_PRI_N0_Msk = 0xf0

	// NVIC_IPR15: Sets or reads interrupt priorities
	// Position of PRI_N3 field.
	PPB_NVIC_IPR15_PRI_N3_Pos = 0x1c
	// Bit mask of PRI_N3 field.
	PPB_NVIC_IPR15_PRI_N3_Msk = 0xf0000000
	// Position of PRI_N2 field.
	PPB_NVIC_IPR15_PRI_N2_Pos = 0x14
	// Bit mask of PRI_N2 field.
	PPB_NVIC_IPR15_PRI_N2_Msk = 0xf00000
	// Position of PRI_N1 field.
	PPB_NVIC_IPR15_PRI_N1_Pos = 0xc
	// Bit mask of PRI_N1 field.
	PPB_NVIC_IPR15_PRI_N1_Msk = 0xf000
	// Position of PRI_N0 field.
	PPB_NVIC_IPR15_PRI_N0_Pos = 0x4
	// Bit mask of PRI_N0 field.
	PPB_NVIC_IPR15_PRI_N0_Msk = 0xf0

	// CPUID: Provides identification information for the PE, including an implementer code for the device and a device ID number
	// Position of IMPLEMENTER field.
	PPB_CPUID_IMPLEMENTER_Pos = 0x18
	// Bit mask of IMPLEMENTER field.
	PPB_CPUID_IMPLEMENTER_Msk = 0xff000000
	// Position of VARIANT field.
	PPB_CPUID_VARIANT_Pos = 0x14
	// Bit mask of VARIANT field.
	PPB_CPUID_VARIANT_Msk = 0xf00000
	// Position of ARCHITECTURE field.
	PPB_CPUID_ARCHITECTURE_Pos = 0x10
	// Bit mask of ARCHITECTURE field.
	PPB_CPUID_ARCHITECTURE_Msk = 0xf0000
	// Position of PARTNO field.
	PPB_CPUID_PARTNO_Pos = 0x4
	// Bit mask of PARTNO field.
	PPB_CPUID_PARTNO_Msk = 0xfff0
	// Position of REVISION field.
	PPB_CPUID_REVISION_Pos = 0x0
	// Bit mask of REVISION field.
	PPB_CPUID_REVISION_Msk = 0xf

	// ICSR: Controls and provides status information for NMI, PendSV, SysTick and interrupts
	// Position of PENDNMISET field.
	PPB_ICSR_PENDNMISET_Pos = 0x1f
	// Bit mask of PENDNMISET field.
	PPB_ICSR_PENDNMISET_Msk = 0x80000000
	// Bit PENDNMISET.
	PPB_ICSR_PENDNMISET = 0x80000000
	// Position of PENDNMICLR field.
	PPB_ICSR_PENDNMICLR_Pos = 0x1e
	// Bit mask of PENDNMICLR field.
	PPB_ICSR_PENDNMICLR_Msk = 0x40000000
	// Bit PENDNMICLR.
	PPB_ICSR_PENDNMICLR = 0x40000000
	// Position of PENDSVSET field.
	PPB_ICSR_PENDSVSET_Pos = 0x1c
	// Bit mask of PENDSVSET field.
	PPB_ICSR_PENDSVSET_Msk = 0x10000000
	// Bit PENDSVSET.
	PPB_ICSR_PENDSVSET = 0x10000000
	// Position of PENDSVCLR field.
	PPB_ICSR_PENDSVCLR_Pos = 0x1b
	// Bit mask of PENDSVCLR field.
	PPB_ICSR_PENDSVCLR_Msk = 0x8000000
	// Bit PENDSVCLR.
	PPB_ICSR_PENDSVCLR = 0x8000000
	// Position of PENDSTSET field.
	PPB_ICSR_PENDSTSET_Pos = 0x1a
	// Bit mask of PENDSTSET field.
	PPB_ICSR_PENDSTSET_Msk = 0x4000000
	// Bit PENDSTSET.
	PPB_ICSR_PENDSTSET = 0x4000000
	// Position of PENDSTCLR field.
	PPB_ICSR_PENDSTCLR_Pos = 0x19
	// Bit mask of PENDSTCLR field.
	PPB_ICSR_PENDSTCLR_Msk = 0x2000000
	// Bit PENDSTCLR.
	PPB_ICSR_PENDSTCLR = 0x2000000
	// Position of STTNS field.
	PPB_ICSR_STTNS_Pos = 0x18
	// Bit mask of STTNS field.
	PPB_ICSR_STTNS_Msk = 0x1000000
	// Bit STTNS.
	PPB_ICSR_STTNS = 0x1000000
	// Position of ISRPREEMPT field.
	PPB_ICSR_ISRPREEMPT_Pos = 0x17
	// Bit mask of ISRPREEMPT field.
	PPB_ICSR_ISRPREEMPT_Msk = 0x800000
	// Bit ISRPREEMPT.
	PPB_ICSR_ISRPREEMPT = 0x800000
	// Position of ISRPENDING field.
	PPB_ICSR_ISRPENDING_Pos = 0x16
	// Bit mask of ISRPENDING field.
	PPB_ICSR_ISRPENDING_Msk = 0x400000
	// Bit ISRPENDING.
	PPB_ICSR_ISRPENDING = 0x400000
	// Position of VECTPENDING field.
	PPB_ICSR_VECTPENDING_Pos = 0xc
	// Bit mask of VECTPENDING field.
	PPB_ICSR_VECTPENDING_Msk = 0x1ff000
	// Position of RETTOBASE field.
	PPB_ICSR_RETTOBASE_Pos = 0xb
	// Bit mask of RETTOBASE field.
	PPB_ICSR_RETTOBASE_Msk = 0x800
	// Bit RETTOBASE.
	PPB_ICSR_RETTOBASE = 0x800
	// Position of VECTACTIVE field.
	PPB_ICSR_VECTACTIVE_Pos = 0x0
	// Bit mask of VECTACTIVE field.
	PPB_ICSR_VECTACTIVE_Msk = 0x1ff

	// VTOR: The VTOR indicates the offset of the vector table base address from memory address 0x00000000.
	// Position of TBLOFF field.
	PPB_VTOR_TBLOFF_Pos = 0x7
	// Bit mask of TBLOFF field.
	PPB_VTOR_TBLOFF_Msk = 0xffffff80

	// AIRCR: Use the Application Interrupt and Reset Control Register to: determine data endianness, clear all active state information from debug halt mode, request a system reset.
	// Position of VECTKEY field.
	PPB_AIRCR_VECTKEY_Pos = 0x10
	// Bit mask of VECTKEY field.
	PPB_AIRCR_VECTKEY_Msk = 0xffff0000
	// Position of ENDIANESS field.
	PPB_AIRCR_ENDIANESS_Pos = 0xf
	// Bit mask of ENDIANESS field.
	PPB_AIRCR_ENDIANESS_Msk = 0x8000
	// Bit ENDIANESS.
	PPB_AIRCR_ENDIANESS = 0x8000
	// Position of PRIS field.
	PPB_AIRCR_PRIS_Pos = 0xe
	// Bit mask of PRIS field.
	PPB_AIRCR_PRIS_Msk = 0x4000
	// Bit PRIS.
	PPB_AIRCR_PRIS = 0x4000
	// Position of BFHFNMINS field.
	PPB_AIRCR_BFHFNMINS_Pos = 0xd
	// Bit mask of BFHFNMINS field.
	PPB_AIRCR_BFHFNMINS_Msk = 0x2000
	// Bit BFHFNMINS.
	PPB_AIRCR_BFHFNMINS = 0x2000
	// Position of PRIGROUP field.
	PPB_AIRCR_PRIGROUP_Pos = 0x8
	// Bit mask of PRIGROUP field.
	PPB_AIRCR_PRIGROUP_Msk = 0x700
	// Position of SYSRESETREQS field.
	PPB_AIRCR_SYSRESETREQS_Pos = 0x3
	// Bit mask of SYSRESETREQS field.
	PPB_AIRCR_SYSRESETREQS_Msk = 0x8
	// Bit SYSRESETREQS.
	PPB_AIRCR_SYSRESETREQS = 0x8
	// Position of SYSRESETREQ field.
	PPB_AIRCR_SYSRESETREQ_Pos = 0x2
	// Bit mask of SYSRESETREQ field.
	PPB_AIRCR_SYSRESETREQ_Msk = 0x4
	// Bit SYSRESETREQ.
	PPB_AIRCR_SYSRESETREQ = 0x4
	// Position of VECTCLRACTIVE field.
	PPB_AIRCR_VECTCLRACTIVE_Pos = 0x1
	// Bit mask of VECTCLRACTIVE field.
	PPB_AIRCR_VECTCLRACTIVE_Msk = 0x2
	// Bit VECTCLRACTIVE.
	PPB_AIRCR_VECTCLRACTIVE = 0x2

	// SCR: System Control Register. Use the System Control Register for power-management functions: signal to the system when the processor can enter a low power state, control how the processor enters and exits low power states.
	// Position of SEVONPEND field.
	PPB_SCR_SEVONPEND_Pos = 0x4
	// Bit mask of SEVONPEND field.
	PPB_SCR_SEVONPEND_Msk = 0x10
	// Bit SEVONPEND.
	PPB_SCR_SEVONPEND = 0x10
	// Position of SLEEPDEEPS field.
	PPB_SCR_SLEEPDEEPS_Pos = 0x3
	// Bit mask of SLEEPDEEPS field.
	PPB_SCR_SLEEPDEEPS_Msk = 0x8
	// Bit SLEEPDEEPS.
	PPB_SCR_SLEEPDEEPS = 0x8
	// Position of SLEEPDEEP field.
	PPB_SCR_SLEEPDEEP_Pos = 0x2
	// Bit mask of SLEEPDEEP field.
	PPB_SCR_SLEEPDEEP_Msk = 0x4
	// Bit SLEEPDEEP.
	PPB_SCR_SLEEPDEEP = 0x4
	// Position of SLEEPONEXIT field.
	PPB_SCR_SLEEPONEXIT_Pos = 0x1
	// Bit mask of SLEEPONEXIT field.
	PPB_SCR_SLEEPONEXIT_Msk = 0x2
	// Bit SLEEPONEXIT.
	PPB_SCR_SLEEPONEXIT = 0x2

	// CCR: Sets or returns configuration and control data
	// Position of BP field.
	PPB_CCR_BP_Pos = 0x12
	// Bit mask of BP field.
	PPB_CCR_BP_Msk = 0x40000
	// Bit BP.
	PPB_CCR_BP = 0x40000
	// Position of IC field.
	PPB_CCR_IC_Pos = 0x11
	// Bit mask of IC field.
	PPB_CCR_IC_Msk = 0x20000
	// Bit IC.
	PPB_CCR_IC = 0x20000
	// Position of DC field.
	PPB_CCR_DC_Pos = 0x10
	// Bit mask of DC field.
	PPB_CCR_DC_Msk = 0x10000
	// Bit DC.
	PPB_CCR_DC = 0x10000
	// Position of STKOFHFNMIGN field.
	PPB_CCR_STKOFHFNMIGN_Pos = 0xa
	// Bit mask of STKOFHFNMIGN field.
	PPB_CCR_STKOFHFNMIGN_Msk = 0x400
	// Bit STKOFHFNMIGN.
	PPB_CCR_STKOFHFNMIGN = 0x400
	// Position of RES1 field.
	PPB_CCR_RES1_Pos = 0x9
	// Bit mask of RES1 field.
	PPB_CCR_RES1_Msk = 0x200
	// Bit RES1.
	PPB_CCR_RES1 = 0x200
	// Position of BFHFNMIGN field.
	PPB_CCR_BFHFNMIGN_Pos = 0x8
	// Bit mask of BFHFNMIGN field.
	PPB_CCR_BFHFNMIGN_Msk = 0x100
	// Bit BFHFNMIGN.
	PPB_CCR_BFHFNMIGN = 0x100
	// Position of DIV_0_TRP field.
	PPB_CCR_DIV_0_TRP_Pos = 0x4
	// Bit mask of DIV_0_TRP field.
	PPB_CCR_DIV_0_TRP_Msk = 0x10
	// Bit DIV_0_TRP.
	PPB_CCR_DIV_0_TRP = 0x10
	// Position of UNALIGN_TRP field.
	PPB_CCR_UNALIGN_TRP_Pos = 0x3
	// Bit mask of UNALIGN_TRP field.
	PPB_CCR_UNALIGN_TRP_Msk = 0x8
	// Bit UNALIGN_TRP.
	PPB_CCR_UNALIGN_TRP = 0x8
	// Position of USERSETMPEND field.
	PPB_CCR_USERSETMPEND_Pos = 0x1
	// Bit mask of USERSETMPEND field.
	PPB_CCR_USERSETMPEND_Msk = 0x2
	// Bit USERSETMPEND.
	PPB_CCR_USERSETMPEND = 0x2
	// Position of RES1_1 field.
	PPB_CCR_RES1_1_Pos = 0x0
	// Bit mask of RES1_1 field.
	PPB_CCR_RES1_1_Msk = 0x1
	// Bit RES1_1.
	PPB_CCR_RES1_1 = 0x1

	// SHPR1: Sets or returns priority for system handlers 4 - 7
	// Position of PRI_7_3 field.
	PPB_SHPR1_PRI_7_3_Pos = 0x1d
	// Bit mask of PRI_7_3 field.
	PPB_SHPR1_PRI_7_3_Msk = 0xe0000000
	// Position of PRI_6_3 field.
	PPB_SHPR1_PRI_6_3_Pos = 0x15
	// Bit mask of PRI_6_3 field.
	PPB_SHPR1_PRI_6_3_Msk = 0xe00000
	// Position of PRI_5_3 field.
	PPB_SHPR1_PRI_5_3_Pos = 0xd
	// Bit mask of PRI_5_3 field.
	PPB_SHPR1_PRI_5_3_Msk = 0xe000
	// Position of PRI_4_3 field.
	PPB_SHPR1_PRI_4_3_Pos = 0x5
	// Bit mask of PRI_4_3 field.
	PPB_SHPR1_PRI_4_3_Msk = 0xe0

	// SHPR2: Sets or returns priority for system handlers 8 - 11
	// Position of PRI_11_3 field.
	PPB_SHPR2_PRI_11_3_Pos = 0x1d
	// Bit mask of PRI_11_3 field.
	PPB_SHPR2_PRI_11_3_Msk = 0xe0000000
	// Position of PRI_10 field.
	PPB_SHPR2_PRI_10_Pos = 0x10
	// Bit mask of PRI_10 field.
	PPB_SHPR2_PRI_10_Msk = 0xff0000
	// Position of PRI_9 field.
	PPB_SHPR2_PRI_9_Pos = 0x8
	// Bit mask of PRI_9 field.
	PPB_SHPR2_PRI_9_Msk = 0xff00
	// Position of PRI_8 field.
	PPB_SHPR2_PRI_8_Pos = 0x0
	// Bit mask of PRI_8 field.
	PPB_SHPR2_PRI_8_Msk = 0xff

	// SHPR3: Sets or returns priority for system handlers 12 - 15
	// Position of PRI_15_3 field.
	PPB_SHPR3_PRI_15_3_Pos = 0x1d
	// Bit mask of PRI_15_3 field.
	PPB_SHPR3_PRI_15_3_Msk = 0xe0000000
	// Position of PRI_14_3 field.
	PPB_SHPR3_PRI_14_3_Pos = 0x15
	// Bit mask of PRI_14_3 field.
	PPB_SHPR3_PRI_14_3_Msk = 0xe00000
	// Position of PRI_13 field.
	PPB_SHPR3_PRI_13_Pos = 0x8
	// Bit mask of PRI_13 field.
	PPB_SHPR3_PRI_13_Msk = 0xff00
	// Position of PRI_12_3 field.
	PPB_SHPR3_PRI_12_3_Pos = 0x5
	// Bit mask of PRI_12_3 field.
	PPB_SHPR3_PRI_12_3_Msk = 0xe0

	// SHCSR: Provides access to the active and pending status of system exceptions
	// Position of HARDFAULTPENDED field.
	PPB_SHCSR_HARDFAULTPENDED_Pos = 0x15
	// Bit mask of HARDFAULTPENDED field.
	PPB_SHCSR_HARDFAULTPENDED_Msk = 0x200000
	// Bit HARDFAULTPENDED.
	PPB_SHCSR_HARDFAULTPENDED = 0x200000
	// Position of SECUREFAULTPENDED field.
	PPB_SHCSR_SECUREFAULTPENDED_Pos = 0x14
	// Bit mask of SECUREFAULTPENDED field.
	PPB_SHCSR_SECUREFAULTPENDED_Msk = 0x100000
	// Bit SECUREFAULTPENDED.
	PPB_SHCSR_SECUREFAULTPENDED = 0x100000
	// Position of SECUREFAULTENA field.
	PPB_SHCSR_SECUREFAULTENA_Pos = 0x13
	// Bit mask of SECUREFAULTENA field.
	PPB_SHCSR_SECUREFAULTENA_Msk = 0x80000
	// Bit SECUREFAULTENA.
	PPB_SHCSR_SECUREFAULTENA = 0x80000
	// Position of USGFAULTENA field.
	PPB_SHCSR_USGFAULTENA_Pos = 0x12
	// Bit mask of USGFAULTENA field.
	PPB_SHCSR_USGFAULTENA_Msk = 0x40000
	// Bit USGFAULTENA.
	PPB_SHCSR_USGFAULTENA = 0x40000
	// Position of BUSFAULTENA field.
	PPB_SHCSR_BUSFAULTENA_Pos = 0x11
	// Bit mask of BUSFAULTENA field.
	PPB_SHCSR_BUSFAULTENA_Msk = 0x20000
	// Bit BUSFAULTENA.
	PPB_SHCSR_BUSFAULTENA = 0x20000
	// Position of MEMFAULTENA field.
	PPB_SHCSR_MEMFAULTENA_Pos = 0x10
	// Bit mask of MEMFAULTENA field.
	PPB_SHCSR_MEMFAULTENA_Msk = 0x10000
	// Bit MEMFAULTENA.
	PPB_SHCSR_MEMFAULTENA = 0x10000
	// Position of SVCALLPENDED field.
	PPB_SHCSR_SVCALLPENDED_Pos = 0xf
	// Bit mask of SVCALLPENDED field.
	PPB_SHCSR_SVCALLPENDED_Msk = 0x8000
	// Bit SVCALLPENDED.
	PPB_SHCSR_SVCALLPENDED = 0x8000
	// Position of BUSFAULTPENDED field.
	PPB_SHCSR_BUSFAULTPENDED_Pos = 0xe
	// Bit mask of BUSFAULTPENDED field.
	PPB_SHCSR_BUSFAULTPENDED_Msk = 0x4000
	// Bit BUSFAULTPENDED.
	PPB_SHCSR_BUSFAULTPENDED = 0x4000
	// Position of MEMFAULTPENDED field.
	PPB_SHCSR_MEMFAULTPENDED_Pos = 0xd
	// Bit mask of MEMFAULTPENDED field.
	PPB_SHCSR_MEMFAULTPENDED_Msk = 0x2000
	// Bit MEMFAULTPENDED.
	PPB_SHCSR_MEMFAULTPENDED = 0x2000
	// Position of USGFAULTPENDED field.
	PPB_SHCSR_USGFAULTPENDED_Pos = 0xc
	// Bit mask of USGFAULTPENDED field.
	PPB_SHCSR_USGFAULTPENDED_Msk = 0x1000
	// Bit USGFAULTPENDED.
	PPB_SHCSR_USGFAULTPENDED = 0x1000
	// Position of SYSTICKACT field.
	PPB_SHCSR_SYSTICKACT_Pos = 0xb
	// Bit mask of SYSTICKACT field.
	PPB_SHCSR_SYSTICKACT_Msk = 0x800
	// Bit SYSTICKACT.
	PPB_SHCSR_SYSTICKACT = 0x800
	// Position of PENDSVACT field.
	PPB_SHCSR_PENDSVACT_Pos = 0xa
	// Bit mask of PENDSVACT field.
	PPB_SHCSR_PENDSVACT_Msk = 0x400
	// Bit PENDSVACT.
	PPB_SHCSR_PENDSVACT = 0x400
	// Position of MONITORACT field.
	PPB_SHCSR_MONITORACT_Pos = 0x8
	// Bit mask of MONITORACT field.
	PPB_SHCSR_MONITORACT_Msk = 0x100
	// Bit MONITORACT.
	PPB_SHCSR_MONITORACT = 0x100
	// Position of SVCALLACT field.
	PPB_SHCSR_SVCALLACT_Pos = 0x7
	// Bit mask of SVCALLACT field.
	PPB_SHCSR_SVCALLACT_Msk = 0x80
	// Bit SVCALLACT.
	PPB_SHCSR_SVCALLACT = 0x80
	// Position of NMIACT field.
	PPB_SHCSR_NMIACT_Pos = 0x5
	// Bit mask of NMIACT field.
	PPB_SHCSR_NMIACT_Msk = 0x20
	// Bit NMIACT.
	PPB_SHCSR_NMIACT = 0x20
	// Position of SECUREFAULTACT field.
	PPB_SHCSR_SECUREFAULTACT_Pos = 0x4
	// Bit mask of SECUREFAULTACT field.
	PPB_SHCSR_SECUREFAULTACT_Msk = 0x10
	// Bit SECUREFAULTACT.
	PPB_SHCSR_SECUREFAULTACT = 0x10
	// Position of USGFAULTACT field.
	PPB_SHCSR_USGFAULTACT_Pos = 0x3
	// Bit mask of USGFAULTACT field.
	PPB_SHCSR_USGFAULTACT_Msk = 0x8
	// Bit USGFAULTACT.
	PPB_SHCSR_USGFAULTACT = 0x8
	// Position of HARDFAULTACT field.
	PPB_SHCSR_HARDFAULTACT_Pos = 0x2
	// Bit mask of HARDFAULTACT field.
	PPB_SHCSR_HARDFAULTACT_Msk = 0x4
	// Bit HARDFAULTACT.
	PPB_SHCSR_HARDFAULTACT = 0x4
	// Position of BUSFAULTACT field.
	PPB_SHCSR_BUSFAULTACT_Pos = 0x1
	// Bit mask of BUSFAULTACT field.
	PPB_SHCSR_BUSFAULTACT_Msk = 0x2
	// Bit BUSFAULTACT.
	PPB_SHCSR_BUSFAULTACT = 0x2
	// Position of MEMFAULTACT field.
	PPB_SHCSR_MEMFAULTACT_Pos = 0x0
	// Bit mask of MEMFAULTACT field.
	PPB_SHCSR_MEMFAULTACT_Msk = 0x1
	// Bit MEMFAULTACT.
	PPB_SHCSR_MEMFAULTACT = 0x1

	// CFSR: Contains the three Configurable Fault Status Registers. 31:16 UFSR: Provides information on UsageFault exceptions 15:8 BFSR: Provides information on BusFault exceptions 7:0 MMFSR: Provides information on MemManage exceptions
	// Position of UFSR_DIVBYZERO field.
	PPB_CFSR_UFSR_DIVBYZERO_Pos = 0x19
	// Bit mask of UFSR_DIVBYZERO field.
	PPB_CFSR_UFSR_DIVBYZERO_Msk = 0x2000000
	// Bit UFSR_DIVBYZERO.
	PPB_CFSR_UFSR_DIVBYZERO = 0x2000000
	// Position of UFSR_UNALIGNED field.
	PPB_CFSR_UFSR_UNALIGNED_Pos = 0x18
	// Bit mask of UFSR_UNALIGNED field.
	PPB_CFSR_UFSR_UNALIGNED_Msk = 0x1000000
	// Bit UFSR_UNALIGNED.
	PPB_CFSR_UFSR_UNALIGNED = 0x1000000
	// Position of UFSR_STKOF field.
	PPB_CFSR_UFSR_STKOF_Pos = 0x14
	// Bit mask of UFSR_STKOF field.
	PPB_CFSR_UFSR_STKOF_Msk = 0x100000
	// Bit UFSR_STKOF.
	PPB_CFSR_UFSR_STKOF = 0x100000
	// Position of UFSR_NOCP field.
	PPB_CFSR_UFSR_NOCP_Pos = 0x13
	// Bit mask of UFSR_NOCP field.
	PPB_CFSR_UFSR_NOCP_Msk = 0x80000
	// Bit UFSR_NOCP.
	PPB_CFSR_UFSR_NOCP = 0x80000
	// Position of UFSR_INVPC field.
	PPB_CFSR_UFSR_INVPC_Pos = 0x12
	// Bit mask of UFSR_INVPC field.
	PPB_CFSR_UFSR_INVPC_Msk = 0x40000
	// Bit UFSR_INVPC.
	PPB_CFSR_UFSR_INVPC = 0x40000
	// Position of UFSR_INVSTATE field.
	PPB_CFSR_UFSR_INVSTATE_Pos = 0x11
	// Bit mask of UFSR_INVSTATE field.
	PPB_CFSR_UFSR_INVSTATE_Msk = 0x20000
	// Bit UFSR_INVSTATE.
	PPB_CFSR_UFSR_INVSTATE = 0x20000
	// Position of UFSR_UNDEFINSTR field.
	PPB_CFSR_UFSR_UNDEFINSTR_Pos = 0x10
	// Bit mask of UFSR_UNDEFINSTR field.
	PPB_CFSR_UFSR_UNDEFINSTR_Msk = 0x10000
	// Bit UFSR_UNDEFINSTR.
	PPB_CFSR_UFSR_UNDEFINSTR = 0x10000
	// Position of BFSR_BFARVALID field.
	PPB_CFSR_BFSR_BFARVALID_Pos = 0xf
	// Bit mask of BFSR_BFARVALID field.
	PPB_CFSR_BFSR_BFARVALID_Msk = 0x8000
	// Bit BFSR_BFARVALID.
	PPB_CFSR_BFSR_BFARVALID = 0x8000
	// Position of BFSR_LSPERR field.
	PPB_CFSR_BFSR_LSPERR_Pos = 0xd
	// Bit mask of BFSR_LSPERR field.
	PPB_CFSR_BFSR_LSPERR_Msk = 0x2000
	// Bit BFSR_LSPERR.
	PPB_CFSR_BFSR_LSPERR = 0x2000
	// Position of BFSR_STKERR field.
	PPB_CFSR_BFSR_STKERR_Pos = 0xc
	// Bit mask of BFSR_STKERR field.
	PPB_CFSR_BFSR_STKERR_Msk = 0x1000
	// Bit BFSR_STKERR.
	PPB_CFSR_BFSR_STKERR = 0x1000
	// Position of BFSR_UNSTKERR field.
	PPB_CFSR_BFSR_UNSTKERR_Pos = 0xb
	// Bit mask of BFSR_UNSTKERR field.
	PPB_CFSR_BFSR_UNSTKERR_Msk = 0x800
	// Bit BFSR_UNSTKERR.
	PPB_CFSR_BFSR_UNSTKERR = 0x800
	// Position of BFSR_IMPRECISERR field.
	PPB_CFSR_BFSR_IMPRECISERR_Pos = 0xa
	// Bit mask of BFSR_IMPRECISERR field.
	PPB_CFSR_BFSR_IMPRECISERR_Msk = 0x400
	// Bit BFSR_IMPRECISERR.
	PPB_CFSR_BFSR_IMPRECISERR = 0x400
	// Position of BFSR_PRECISERR field.
	PPB_CFSR_BFSR_PRECISERR_Pos = 0x9
	// Bit mask of BFSR_PRECISERR field.
	PPB_CFSR_BFSR_PRECISERR_Msk = 0x200
	// Bit BFSR_PRECISERR.
	PPB_CFSR_BFSR_PRECISERR = 0x200
	// Position of BFSR_IBUSERR field.
	PPB_CFSR_BFSR_IBUSERR_Pos = 0x8
	// Bit mask of BFSR_IBUSERR field.
	PPB_CFSR_BFSR_IBUSERR_Msk = 0x100
	// Bit BFSR_IBUSERR.
	PPB_CFSR_BFSR_IBUSERR = 0x100
	// Position of MMFSR field.
	PPB_CFSR_MMFSR_Pos = 0x0
	// Bit mask of MMFSR field.
	PPB_CFSR_MMFSR_Msk = 0xff

	// HFSR: Shows the cause of any HardFaults
	// Position of DEBUGEVT field.
	PPB_HFSR_DEBUGEVT_Pos = 0x1f
	// Bit mask of DEBUGEVT field.
	PPB_HFSR_DEBUGEVT_Msk = 0x80000000
	// Bit DEBUGEVT.
	PPB_HFSR_DEBUGEVT = 0x80000000
	// Position of FORCED field.
	PPB_HFSR_FORCED_Pos = 0x1e
	// Bit mask of FORCED field.
	PPB_HFSR_FORCED_Msk = 0x40000000
	// Bit FORCED.
	PPB_HFSR_FORCED = 0x40000000
	// Position of VECTTBL field.
	PPB_HFSR_VECTTBL_Pos = 0x1
	// Bit mask of VECTTBL field.
	PPB_HFSR_VECTTBL_Msk = 0x2
	// Bit VECTTBL.
	PPB_HFSR_VECTTBL = 0x2

	// DFSR: Shows which debug event occurred
	// Position of EXTERNAL field.
	PPB_DFSR_EXTERNAL_Pos = 0x4
	// Bit mask of EXTERNAL field.
	PPB_DFSR_EXTERNAL_Msk = 0x10
	// Bit EXTERNAL.
	PPB_DFSR_EXTERNAL = 0x10
	// Position of VCATCH field.
	PPB_DFSR_VCATCH_Pos = 0x3
	// Bit mask of VCATCH field.
	PPB_DFSR_VCATCH_Msk = 0x8
	// Bit VCATCH.
	PPB_DFSR_VCATCH = 0x8
	// Position of DWTTRAP field.
	PPB_DFSR_DWTTRAP_Pos = 0x2
	// Bit mask of DWTTRAP field.
	PPB_DFSR_DWTTRAP_Msk = 0x4
	// Bit DWTTRAP.
	PPB_DFSR_DWTTRAP = 0x4
	// Position of BKPT field.
	PPB_DFSR_BKPT_Pos = 0x1
	// Bit mask of BKPT field.
	PPB_DFSR_BKPT_Msk = 0x2
	// Bit BKPT.
	PPB_DFSR_BKPT = 0x2
	// Position of HALTED field.
	PPB_DFSR_HALTED_Pos = 0x0
	// Bit mask of HALTED field.
	PPB_DFSR_HALTED_Msk = 0x1
	// Bit HALTED.
	PPB_DFSR_HALTED = 0x1

	// MMFAR: Shows the address of the memory location that caused an MPU fault
	// Position of ADDRESS field.
	PPB_MMFAR_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	PPB_MMFAR_ADDRESS_Msk = 0xffffffff

	// BFAR: Shows the address associated with a precise data access BusFault
	// Position of ADDRESS field.
	PPB_BFAR_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	PPB_BFAR_ADDRESS_Msk = 0xffffffff

	// ID_PFR0: Gives top-level information about the instruction set supported by the PE
	// Position of STATE1 field.
	PPB_ID_PFR0_STATE1_Pos = 0x4
	// Bit mask of STATE1 field.
	PPB_ID_PFR0_STATE1_Msk = 0xf0
	// Position of STATE0 field.
	PPB_ID_PFR0_STATE0_Pos = 0x0
	// Bit mask of STATE0 field.
	PPB_ID_PFR0_STATE0_Msk = 0xf

	// ID_PFR1: Gives information about the programmers' model and Extensions support
	// Position of MPROGMOD field.
	PPB_ID_PFR1_MPROGMOD_Pos = 0x8
	// Bit mask of MPROGMOD field.
	PPB_ID_PFR1_MPROGMOD_Msk = 0xf00
	// Position of SECURITY field.
	PPB_ID_PFR1_SECURITY_Pos = 0x4
	// Bit mask of SECURITY field.
	PPB_ID_PFR1_SECURITY_Msk = 0xf0

	// ID_DFR0: Provides top level information about the debug system
	// Position of MPROFDBG field.
	PPB_ID_DFR0_MPROFDBG_Pos = 0x14
	// Bit mask of MPROFDBG field.
	PPB_ID_DFR0_MPROFDBG_Msk = 0xf00000

	// ID_AFR0: Provides information about the IMPLEMENTATION DEFINED features of the PE
	// Position of IMPDEF3 field.
	PPB_ID_AFR0_IMPDEF3_Pos = 0xc
	// Bit mask of IMPDEF3 field.
	PPB_ID_AFR0_IMPDEF3_Msk = 0xf000
	// Position of IMPDEF2 field.
	PPB_ID_AFR0_IMPDEF2_Pos = 0x8
	// Bit mask of IMPDEF2 field.
	PPB_ID_AFR0_IMPDEF2_Msk = 0xf00
	// Position of IMPDEF1 field.
	PPB_ID_AFR0_IMPDEF1_Pos = 0x4
	// Bit mask of IMPDEF1 field.
	PPB_ID_AFR0_IMPDEF1_Msk = 0xf0
	// Position of IMPDEF0 field.
	PPB_ID_AFR0_IMPDEF0_Pos = 0x0
	// Bit mask of IMPDEF0 field.
	PPB_ID_AFR0_IMPDEF0_Msk = 0xf

	// ID_MMFR0: Provides information about the implemented memory model and memory management support
	// Position of AUXREG field.
	PPB_ID_MMFR0_AUXREG_Pos = 0x14
	// Bit mask of AUXREG field.
	PPB_ID_MMFR0_AUXREG_Msk = 0xf00000
	// Position of TCM field.
	PPB_ID_MMFR0_TCM_Pos = 0x10
	// Bit mask of TCM field.
	PPB_ID_MMFR0_TCM_Msk = 0xf0000
	// Position of SHARELVL field.
	PPB_ID_MMFR0_SHARELVL_Pos = 0xc
	// Bit mask of SHARELVL field.
	PPB_ID_MMFR0_SHARELVL_Msk = 0xf000
	// Position of OUTERSHR field.
	PPB_ID_MMFR0_OUTERSHR_Pos = 0x8
	// Bit mask of OUTERSHR field.
	PPB_ID_MMFR0_OUTERSHR_Msk = 0xf00
	// Position of PMSA field.
	PPB_ID_MMFR0_PMSA_Pos = 0x4
	// Bit mask of PMSA field.
	PPB_ID_MMFR0_PMSA_Msk = 0xf0

	// ID_MMFR1: Provides information about the implemented memory model and memory management support
	// Position of ID_MMFR1 field.
	PPB_ID_MMFR1_ID_MMFR1_Pos = 0x0
	// Bit mask of ID_MMFR1 field.
	PPB_ID_MMFR1_ID_MMFR1_Msk = 0xffffffff

	// ID_MMFR2: Provides information about the implemented memory model and memory management support
	// Position of WFISTALL field.
	PPB_ID_MMFR2_WFISTALL_Pos = 0x18
	// Bit mask of WFISTALL field.
	PPB_ID_MMFR2_WFISTALL_Msk = 0xf000000

	// ID_MMFR3: Provides information about the implemented memory model and memory management support
	// Position of BPMAINT field.
	PPB_ID_MMFR3_BPMAINT_Pos = 0x8
	// Bit mask of BPMAINT field.
	PPB_ID_MMFR3_BPMAINT_Msk = 0xf00
	// Position of CMAINTSW field.
	PPB_ID_MMFR3_CMAINTSW_Pos = 0x4
	// Bit mask of CMAINTSW field.
	PPB_ID_MMFR3_CMAINTSW_Msk = 0xf0
	// Position of CMAINTVA field.
	PPB_ID_MMFR3_CMAINTVA_Pos = 0x0
	// Bit mask of CMAINTVA field.
	PPB_ID_MMFR3_CMAINTVA_Msk = 0xf

	// ID_ISAR0: Provides information about the instruction set implemented by the PE
	// Position of DIVIDE field.
	PPB_ID_ISAR0_DIVIDE_Pos = 0x18
	// Bit mask of DIVIDE field.
	PPB_ID_ISAR0_DIVIDE_Msk = 0xf000000
	// Position of DEBUG field.
	PPB_ID_ISAR0_DEBUG_Pos = 0x14
	// Bit mask of DEBUG field.
	PPB_ID_ISAR0_DEBUG_Msk = 0xf00000
	// Position of COPROC field.
	PPB_ID_ISAR0_COPROC_Pos = 0x10
	// Bit mask of COPROC field.
	PPB_ID_ISAR0_COPROC_Msk = 0xf0000
	// Position of CMPBRANCH field.
	PPB_ID_ISAR0_CMPBRANCH_Pos = 0xc
	// Bit mask of CMPBRANCH field.
	PPB_ID_ISAR0_CMPBRANCH_Msk = 0xf000
	// Position of BITFIELD field.
	PPB_ID_ISAR0_BITFIELD_Pos = 0x8
	// Bit mask of BITFIELD field.
	PPB_ID_ISAR0_BITFIELD_Msk = 0xf00
	// Position of BITCOUNT field.
	PPB_ID_ISAR0_BITCOUNT_Pos = 0x4
	// Bit mask of BITCOUNT field.
	PPB_ID_ISAR0_BITCOUNT_Msk = 0xf0

	// ID_ISAR1: Provides information about the instruction set implemented by the PE
	// Position of INTERWORK field.
	PPB_ID_ISAR1_INTERWORK_Pos = 0x18
	// Bit mask of INTERWORK field.
	PPB_ID_ISAR1_INTERWORK_Msk = 0xf000000
	// Position of IMMEDIATE field.
	PPB_ID_ISAR1_IMMEDIATE_Pos = 0x14
	// Bit mask of IMMEDIATE field.
	PPB_ID_ISAR1_IMMEDIATE_Msk = 0xf00000
	// Position of IFTHEN field.
	PPB_ID_ISAR1_IFTHEN_Pos = 0x10
	// Bit mask of IFTHEN field.
	PPB_ID_ISAR1_IFTHEN_Msk = 0xf0000
	// Position of EXTEND field.
	PPB_ID_ISAR1_EXTEND_Pos = 0xc
	// Bit mask of EXTEND field.
	PPB_ID_ISAR1_EXTEND_Msk = 0xf000

	// ID_ISAR2: Provides information about the instruction set implemented by the PE
	// Position of REVERSAL field.
	PPB_ID_ISAR2_REVERSAL_Pos = 0x1c
	// Bit mask of REVERSAL field.
	PPB_ID_ISAR2_REVERSAL_Msk = 0xf0000000
	// Position of MULTU field.
	PPB_ID_ISAR2_MULTU_Pos = 0x14
	// Bit mask of MULTU field.
	PPB_ID_ISAR2_MULTU_Msk = 0xf00000
	// Position of MULTS field.
	PPB_ID_ISAR2_MULTS_Pos = 0x10
	// Bit mask of MULTS field.
	PPB_ID_ISAR2_MULTS_Msk = 0xf0000
	// Position of MULT field.
	PPB_ID_ISAR2_MULT_Pos = 0xc
	// Bit mask of MULT field.
	PPB_ID_ISAR2_MULT_Msk = 0xf000
	// Position of MULTIACCESSINT field.
	PPB_ID_ISAR2_MULTIACCESSINT_Pos = 0x8
	// Bit mask of MULTIACCESSINT field.
	PPB_ID_ISAR2_MULTIACCESSINT_Msk = 0xf00
	// Position of MEMHINT field.
	PPB_ID_ISAR2_MEMHINT_Pos = 0x4
	// Bit mask of MEMHINT field.
	PPB_ID_ISAR2_MEMHINT_Msk = 0xf0
	// Position of LOADSTORE field.
	PPB_ID_ISAR2_LOADSTORE_Pos = 0x0
	// Bit mask of LOADSTORE field.
	PPB_ID_ISAR2_LOADSTORE_Msk = 0xf

	// ID_ISAR3: Provides information about the instruction set implemented by the PE
	// Position of TRUENOP field.
	PPB_ID_ISAR3_TRUENOP_Pos = 0x18
	// Bit mask of TRUENOP field.
	PPB_ID_ISAR3_TRUENOP_Msk = 0xf000000
	// Position of T32COPY field.
	PPB_ID_ISAR3_T32COPY_Pos = 0x14
	// Bit mask of T32COPY field.
	PPB_ID_ISAR3_T32COPY_Msk = 0xf00000
	// Position of TABBRANCH field.
	PPB_ID_ISAR3_TABBRANCH_Pos = 0x10
	// Bit mask of TABBRANCH field.
	PPB_ID_ISAR3_TABBRANCH_Msk = 0xf0000
	// Position of SYNCHPRIM field.
	PPB_ID_ISAR3_SYNCHPRIM_Pos = 0xc
	// Bit mask of SYNCHPRIM field.
	PPB_ID_ISAR3_SYNCHPRIM_Msk = 0xf000
	// Position of SVC field.
	PPB_ID_ISAR3_SVC_Pos = 0x8
	// Bit mask of SVC field.
	PPB_ID_ISAR3_SVC_Msk = 0xf00
	// Position of SIMD field.
	PPB_ID_ISAR3_SIMD_Pos = 0x4
	// Bit mask of SIMD field.
	PPB_ID_ISAR3_SIMD_Msk = 0xf0
	// Position of SATURATE field.
	PPB_ID_ISAR3_SATURATE_Pos = 0x0
	// Bit mask of SATURATE field.
	PPB_ID_ISAR3_SATURATE_Msk = 0xf

	// ID_ISAR4: Provides information about the instruction set implemented by the PE
	// Position of PSR_M field.
	PPB_ID_ISAR4_PSR_M_Pos = 0x18
	// Bit mask of PSR_M field.
	PPB_ID_ISAR4_PSR_M_Msk = 0xf000000
	// Position of SYNCPRIM_FRAC field.
	PPB_ID_ISAR4_SYNCPRIM_FRAC_Pos = 0x14
	// Bit mask of SYNCPRIM_FRAC field.
	PPB_ID_ISAR4_SYNCPRIM_FRAC_Msk = 0xf00000
	// Position of BARRIER field.
	PPB_ID_ISAR4_BARRIER_Pos = 0x10
	// Bit mask of BARRIER field.
	PPB_ID_ISAR4_BARRIER_Msk = 0xf0000
	// Position of WRITEBACK field.
	PPB_ID_ISAR4_WRITEBACK_Pos = 0x8
	// Bit mask of WRITEBACK field.
	PPB_ID_ISAR4_WRITEBACK_Msk = 0xf00
	// Position of WITHSHIFTS field.
	PPB_ID_ISAR4_WITHSHIFTS_Pos = 0x4
	// Bit mask of WITHSHIFTS field.
	PPB_ID_ISAR4_WITHSHIFTS_Msk = 0xf0
	// Position of UNPRIV field.
	PPB_ID_ISAR4_UNPRIV_Pos = 0x0
	// Bit mask of UNPRIV field.
	PPB_ID_ISAR4_UNPRIV_Msk = 0xf

	// ID_ISAR5: Provides information about the instruction set implemented by the PE
	// Position of ID_ISAR5 field.
	PPB_ID_ISAR5_ID_ISAR5_Pos = 0x0
	// Bit mask of ID_ISAR5 field.
	PPB_ID_ISAR5_ID_ISAR5_Msk = 0xffffffff

	// CTR: Provides information about the architecture of the caches. CTR is RES0 if CLIDR is zero.
	// Position of RES1 field.
	PPB_CTR_RES1_Pos = 0x1f
	// Bit mask of RES1 field.
	PPB_CTR_RES1_Msk = 0x80000000
	// Bit RES1.
	PPB_CTR_RES1 = 0x80000000
	// Position of CWG field.
	PPB_CTR_CWG_Pos = 0x18
	// Bit mask of CWG field.
	PPB_CTR_CWG_Msk = 0xf000000
	// Position of ERG field.
	PPB_CTR_ERG_Pos = 0x14
	// Bit mask of ERG field.
	PPB_CTR_ERG_Msk = 0xf00000
	// Position of DMINLINE field.
	PPB_CTR_DMINLINE_Pos = 0x10
	// Bit mask of DMINLINE field.
	PPB_CTR_DMINLINE_Msk = 0xf0000
	// Position of RES1_1 field.
	PPB_CTR_RES1_1_Pos = 0xe
	// Bit mask of RES1_1 field.
	PPB_CTR_RES1_1_Msk = 0xc000
	// Position of IMINLINE field.
	PPB_CTR_IMINLINE_Pos = 0x0
	// Bit mask of IMINLINE field.
	PPB_CTR_IMINLINE_Msk = 0xf

	// CPACR: Specifies the access privileges for coprocessors and the FP Extension
	// Position of CP11 field.
	PPB_CPACR_CP11_Pos = 0x16
	// Bit mask of CP11 field.
	PPB_CPACR_CP11_Msk = 0xc00000
	// Position of CP10 field.
	PPB_CPACR_CP10_Pos = 0x14
	// Bit mask of CP10 field.
	PPB_CPACR_CP10_Msk = 0x300000
	// Position of CP7 field.
	PPB_CPACR_CP7_Pos = 0xe
	// Bit mask of CP7 field.
	PPB_CPACR_CP7_Msk = 0xc000
	// Position of CP6 field.
	PPB_CPACR_CP6_Pos = 0xc
	// Bit mask of CP6 field.
	PPB_CPACR_CP6_Msk = 0x3000
	// Position of CP5 field.
	PPB_CPACR_CP5_Pos = 0xa
	// Bit mask of CP5 field.
	PPB_CPACR_CP5_Msk = 0xc00
	// Position of CP4 field.
	PPB_CPACR_CP4_Pos = 0x8
	// Bit mask of CP4 field.
	PPB_CPACR_CP4_Msk = 0x300
	// Position of CP3 field.
	PPB_CPACR_CP3_Pos = 0x6
	// Bit mask of CP3 field.
	PPB_CPACR_CP3_Msk = 0xc0
	// Position of CP2 field.
	PPB_CPACR_CP2_Pos = 0x4
	// Bit mask of CP2 field.
	PPB_CPACR_CP2_Msk = 0x30
	// Position of CP1 field.
	PPB_CPACR_CP1_Pos = 0x2
	// Bit mask of CP1 field.
	PPB_CPACR_CP1_Msk = 0xc
	// Position of CP0 field.
	PPB_CPACR_CP0_Pos = 0x0
	// Bit mask of CP0 field.
	PPB_CPACR_CP0_Msk = 0x3

	// NSACR: Defines the Non-secure access permissions for both the FP Extension and coprocessors CP0 to CP7
	// Position of CP11 field.
	PPB_NSACR_CP11_Pos = 0xb
	// Bit mask of CP11 field.
	PPB_NSACR_CP11_Msk = 0x800
	// Bit CP11.
	PPB_NSACR_CP11 = 0x800
	// Position of CP10 field.
	PPB_NSACR_CP10_Pos = 0xa
	// Bit mask of CP10 field.
	PPB_NSACR_CP10_Msk = 0x400
	// Bit CP10.
	PPB_NSACR_CP10 = 0x400
	// Position of CP7 field.
	PPB_NSACR_CP7_Pos = 0x7
	// Bit mask of CP7 field.
	PPB_NSACR_CP7_Msk = 0x80
	// Bit CP7.
	PPB_NSACR_CP7 = 0x80
	// Position of CP6 field.
	PPB_NSACR_CP6_Pos = 0x6
	// Bit mask of CP6 field.
	PPB_NSACR_CP6_Msk = 0x40
	// Bit CP6.
	PPB_NSACR_CP6 = 0x40
	// Position of CP5 field.
	PPB_NSACR_CP5_Pos = 0x5
	// Bit mask of CP5 field.
	PPB_NSACR_CP5_Msk = 0x20
	// Bit CP5.
	PPB_NSACR_CP5 = 0x20
	// Position of CP4 field.
	PPB_NSACR_CP4_Pos = 0x4
	// Bit mask of CP4 field.
	PPB_NSACR_CP4_Msk = 0x10
	// Bit CP4.
	PPB_NSACR_CP4 = 0x10
	// Position of CP3 field.
	PPB_NSACR_CP3_Pos = 0x3
	// Bit mask of CP3 field.
	PPB_NSACR_CP3_Msk = 0x8
	// Bit CP3.
	PPB_NSACR_CP3 = 0x8
	// Position of CP2 field.
	PPB_NSACR_CP2_Pos = 0x2
	// Bit mask of CP2 field.
	PPB_NSACR_CP2_Msk = 0x4
	// Bit CP2.
	PPB_NSACR_CP2 = 0x4
	// Position of CP1 field.
	PPB_NSACR_CP1_Pos = 0x1
	// Bit mask of CP1 field.
	PPB_NSACR_CP1_Msk = 0x2
	// Bit CP1.
	PPB_NSACR_CP1 = 0x2
	// Position of CP0 field.
	PPB_NSACR_CP0_Pos = 0x0
	// Bit mask of CP0 field.
	PPB_NSACR_CP0_Msk = 0x1
	// Bit CP0.
	PPB_NSACR_CP0 = 0x1

	// MPU_TYPE: The MPU Type Register indicates how many regions the MPU `FTSSS supports
	// Position of DREGION field.
	PPB_MPU_TYPE_DREGION_Pos = 0x8
	// Bit mask of DREGION field.
	PPB_MPU_TYPE_DREGION_Msk = 0xff00
	// Position of SEPARATE field.
	PPB_MPU_TYPE_SEPARATE_Pos = 0x0
	// Bit mask of SEPARATE field.
	PPB_MPU_TYPE_SEPARATE_Msk = 0x1
	// Bit SEPARATE.
	PPB_MPU_TYPE_SEPARATE = 0x1

	// MPU_CTRL: Enables the MPU and, when the MPU is enabled, controls whether the default memory map is enabled as a background region for privileged accesses, and whether the MPU is enabled for HardFaults, NMIs, and exception handlers when FAULTMASK is set to 1
	// Position of PRIVDEFENA field.
	PPB_MPU_CTRL_PRIVDEFENA_Pos = 0x2
	// Bit mask of PRIVDEFENA field.
	PPB_MPU_CTRL_PRIVDEFENA_Msk = 0x4
	// Bit PRIVDEFENA.
	PPB_MPU_CTRL_PRIVDEFENA = 0x4
	// Position of HFNMIENA field.
	PPB_MPU_CTRL_HFNMIENA_Pos = 0x1
	// Bit mask of HFNMIENA field.
	PPB_MPU_CTRL_HFNMIENA_Msk = 0x2
	// Bit HFNMIENA.
	PPB_MPU_CTRL_HFNMIENA = 0x2
	// Position of ENABLE field.
	PPB_MPU_CTRL_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	PPB_MPU_CTRL_ENABLE_Msk = 0x1
	// Bit ENABLE.
	PPB_MPU_CTRL_ENABLE = 0x1

	// MPU_RNR: Selects the region currently accessed by MPU_RBAR and MPU_RLAR
	// Position of REGION field.
	PPB_MPU_RNR_REGION_Pos = 0x0
	// Bit mask of REGION field.
	PPB_MPU_RNR_REGION_Msk = 0x7

	// MPU_RBAR: Provides indirect read and write access to the base address of the currently selected MPU region `FTSSS
	// Position of BASE field.
	PPB_MPU_RBAR_BASE_Pos = 0x5
	// Bit mask of BASE field.
	PPB_MPU_RBAR_BASE_Msk = 0xffffffe0
	// Position of SH field.
	PPB_MPU_RBAR_SH_Pos = 0x3
	// Bit mask of SH field.
	PPB_MPU_RBAR_SH_Msk = 0x18
	// Position of AP field.
	PPB_MPU_RBAR_AP_Pos = 0x1
	// Bit mask of AP field.
	PPB_MPU_RBAR_AP_Msk = 0x6
	// Position of XN field.
	PPB_MPU_RBAR_XN_Pos = 0x0
	// Bit mask of XN field.
	PPB_MPU_RBAR_XN_Msk = 0x1
	// Bit XN.
	PPB_MPU_RBAR_XN = 0x1

	// MPU_RLAR: Provides indirect read and write access to the limit address of the currently selected MPU region `FTSSS
	// Position of LIMIT field.
	PPB_MPU_RLAR_LIMIT_Pos = 0x5
	// Bit mask of LIMIT field.
	PPB_MPU_RLAR_LIMIT_Msk = 0xffffffe0
	// Position of ATTRINDX field.
	PPB_MPU_RLAR_ATTRINDX_Pos = 0x1
	// Bit mask of ATTRINDX field.
	PPB_MPU_RLAR_ATTRINDX_Msk = 0xe
	// Position of EN field.
	PPB_MPU_RLAR_EN_Pos = 0x0
	// Bit mask of EN field.
	PPB_MPU_RLAR_EN_Msk = 0x1
	// Bit EN.
	PPB_MPU_RLAR_EN = 0x1

	// MPU_RBAR_A1: Provides indirect read and write access to the base address of the MPU region selected by MPU_RNR[7:2]:(1[1:0]) `FTSSS
	// Position of BASE field.
	PPB_MPU_RBAR_A1_BASE_Pos = 0x5
	// Bit mask of BASE field.
	PPB_MPU_RBAR_A1_BASE_Msk = 0xffffffe0
	// Position of SH field.
	PPB_MPU_RBAR_A1_SH_Pos = 0x3
	// Bit mask of SH field.
	PPB_MPU_RBAR_A1_SH_Msk = 0x18
	// Position of AP field.
	PPB_MPU_RBAR_A1_AP_Pos = 0x1
	// Bit mask of AP field.
	PPB_MPU_RBAR_A1_AP_Msk = 0x6
	// Position of XN field.
	PPB_MPU_RBAR_A1_XN_Pos = 0x0
	// Bit mask of XN field.
	PPB_MPU_RBAR_A1_XN_Msk = 0x1
	// Bit XN.
	PPB_MPU_RBAR_A1_XN = 0x1

	// MPU_RLAR_A1: Provides indirect read and write access to the limit address of the currently selected MPU region selected by MPU_RNR[7:2]:(1[1:0]) `FTSSS
	// Position of LIMIT field.
	PPB_MPU_RLAR_A1_LIMIT_Pos = 0x5
	// Bit mask of LIMIT field.
	PPB_MPU_RLAR_A1_LIMIT_Msk = 0xffffffe0
	// Position of ATTRINDX field.
	PPB_MPU_RLAR_A1_ATTRINDX_Pos = 0x1
	// Bit mask of ATTRINDX field.
	PPB_MPU_RLAR_A1_ATTRINDX_Msk = 0xe
	// Position of EN field.
	PPB_MPU_RLAR_A1_EN_Pos = 0x0
	// Bit mask of EN field.
	PPB_MPU_RLAR_A1_EN_Msk = 0x1
	// Bit EN.
	PPB_MPU_RLAR_A1_EN = 0x1

	// MPU_RBAR_A2: Provides indirect read and write access to the base address of the MPU region selected by MPU_RNR[7:2]:(2[1:0]) `FTSSS
	// Position of BASE field.
	PPB_MPU_RBAR_A2_BASE_Pos = 0x5
	// Bit mask of BASE field.
	PPB_MPU_RBAR_A2_BASE_Msk = 0xffffffe0
	// Position of SH field.
	PPB_MPU_RBAR_A2_SH_Pos = 0x3
	// Bit mask of SH field.
	PPB_MPU_RBAR_A2_SH_Msk = 0x18
	// Position of AP field.
	PPB_MPU_RBAR_A2_AP_Pos = 0x1
	// Bit mask of AP field.
	PPB_MPU_RBAR_A2_AP_Msk = 0x6
	// Position of XN field.
	PPB_MPU_RBAR_A2_XN_Pos = 0x0
	// Bit mask of XN field.
	PPB_MPU_RBAR_A2_XN_Msk = 0x1
	// Bit XN.
	PPB_MPU_RBAR_A2_XN = 0x1

	// MPU_RLAR_A2: Provides indirect read and write access to the limit address of the currently selected MPU region selected by MPU_RNR[7:2]:(2[1:0]) `FTSSS
	// Position of LIMIT field.
	PPB_MPU_RLAR_A2_LIMIT_Pos = 0x5
	// Bit mask of LIMIT field.
	PPB_MPU_RLAR_A2_LIMIT_Msk = 0xffffffe0
	// Position of ATTRINDX field.
	PPB_MPU_RLAR_A2_ATTRINDX_Pos = 0x1
	// Bit mask of ATTRINDX field.
	PPB_MPU_RLAR_A2_ATTRINDX_Msk = 0xe
	// Position of EN field.
	PPB_MPU_RLAR_A2_EN_Pos = 0x0
	// Bit mask of EN field.
	PPB_MPU_RLAR_A2_EN_Msk = 0x1
	// Bit EN.
	PPB_MPU_RLAR_A2_EN = 0x1

	// MPU_RBAR_A3: Provides indirect read and write access to the base address of the MPU region selected by MPU_RNR[7:2]:(3[1:0]) `FTSSS
	// Position of BASE field.
	PPB_MPU_RBAR_A3_BASE_Pos = 0x5
	// Bit mask of BASE field.
	PPB_MPU_RBAR_A3_BASE_Msk = 0xffffffe0
	// Position of SH field.
	PPB_MPU_RBAR_A3_SH_Pos = 0x3
	// Bit mask of SH field.
	PPB_MPU_RBAR_A3_SH_Msk = 0x18
	// Position of AP field.
	PPB_MPU_RBAR_A3_AP_Pos = 0x1
	// Bit mask of AP field.
	PPB_MPU_RBAR_A3_AP_Msk = 0x6
	// Position of XN field.
	PPB_MPU_RBAR_A3_XN_Pos = 0x0
	// Bit mask of XN field.
	PPB_MPU_RBAR_A3_XN_Msk = 0x1
	// Bit XN.
	PPB_MPU_RBAR_A3_XN = 0x1

	// MPU_RLAR_A3: Provides indirect read and write access to the limit address of the currently selected MPU region selected by MPU_RNR[7:2]:(3[1:0]) `FTSSS
	// Position of LIMIT field.
	PPB_MPU_RLAR_A3_LIMIT_Pos = 0x5
	// Bit mask of LIMIT field.
	PPB_MPU_RLAR_A3_LIMIT_Msk = 0xffffffe0
	// Position of ATTRINDX field.
	PPB_MPU_RLAR_A3_ATTRINDX_Pos = 0x1
	// Bit mask of ATTRINDX field.
	PPB_MPU_RLAR_A3_ATTRINDX_Msk = 0xe
	// Position of EN field.
	PPB_MPU_RLAR_A3_EN_Pos = 0x0
	// Bit mask of EN field.
	PPB_MPU_RLAR_A3_EN_Msk = 0x1
	// Bit EN.
	PPB_MPU_RLAR_A3_EN = 0x1

	// MPU_MAIR0: Along with MPU_MAIR1, provides the memory attribute encodings corresponding to the AttrIndex values
	// Position of ATTR3 field.
	PPB_MPU_MAIR0_ATTR3_Pos = 0x18
	// Bit mask of ATTR3 field.
	PPB_MPU_MAIR0_ATTR3_Msk = 0xff000000
	// Position of ATTR2 field.
	PPB_MPU_MAIR0_ATTR2_Pos = 0x10
	// Bit mask of ATTR2 field.
	PPB_MPU_MAIR0_ATTR2_Msk = 0xff0000
	// Position of ATTR1 field.
	PPB_MPU_MAIR0_ATTR1_Pos = 0x8
	// Bit mask of ATTR1 field.
	PPB_MPU_MAIR0_ATTR1_Msk = 0xff00
	// Position of ATTR0 field.
	PPB_MPU_MAIR0_ATTR0_Pos = 0x0
	// Bit mask of ATTR0 field.
	PPB_MPU_MAIR0_ATTR0_Msk = 0xff

	// MPU_MAIR1: Along with MPU_MAIR0, provides the memory attribute encodings corresponding to the AttrIndex values
	// Position of ATTR7 field.
	PPB_MPU_MAIR1_ATTR7_Pos = 0x18
	// Bit mask of ATTR7 field.
	PPB_MPU_MAIR1_ATTR7_Msk = 0xff000000
	// Position of ATTR6 field.
	PPB_MPU_MAIR1_ATTR6_Pos = 0x10
	// Bit mask of ATTR6 field.
	PPB_MPU_MAIR1_ATTR6_Msk = 0xff0000
	// Position of ATTR5 field.
	PPB_MPU_MAIR1_ATTR5_Pos = 0x8
	// Bit mask of ATTR5 field.
	PPB_MPU_MAIR1_ATTR5_Msk = 0xff00
	// Position of ATTR4 field.
	PPB_MPU_MAIR1_ATTR4_Pos = 0x0
	// Bit mask of ATTR4 field.
	PPB_MPU_MAIR1_ATTR4_Msk = 0xff

	// SAU_CTRL: Allows enabling of the Security Attribution Unit
	// Position of ALLNS field.
	PPB_SAU_CTRL_ALLNS_Pos = 0x1
	// Bit mask of ALLNS field.
	PPB_SAU_CTRL_ALLNS_Msk = 0x2
	// Bit ALLNS.
	PPB_SAU_CTRL_ALLNS = 0x2
	// Position of ENABLE field.
	PPB_SAU_CTRL_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	PPB_SAU_CTRL_ENABLE_Msk = 0x1
	// Bit ENABLE.
	PPB_SAU_CTRL_ENABLE = 0x1

	// SAU_TYPE: Indicates the number of regions implemented by the Security Attribution Unit
	// Position of SREGION field.
	PPB_SAU_TYPE_SREGION_Pos = 0x0
	// Bit mask of SREGION field.
	PPB_SAU_TYPE_SREGION_Msk = 0xff

	// SAU_RNR: Selects the region currently accessed by SAU_RBAR and SAU_RLAR
	// Position of REGION field.
	PPB_SAU_RNR_REGION_Pos = 0x0
	// Bit mask of REGION field.
	PPB_SAU_RNR_REGION_Msk = 0xff

	// SAU_RBAR: Provides indirect read and write access to the base address of the currently selected SAU region
	// Position of BADDR field.
	PPB_SAU_RBAR_BADDR_Pos = 0x5
	// Bit mask of BADDR field.
	PPB_SAU_RBAR_BADDR_Msk = 0xffffffe0

	// SAU_RLAR: Provides indirect read and write access to the limit address of the currently selected SAU region
	// Position of LADDR field.
	PPB_SAU_RLAR_LADDR_Pos = 0x5
	// Bit mask of LADDR field.
	PPB_SAU_RLAR_LADDR_Msk = 0xffffffe0
	// Position of NSC field.
	PPB_SAU_RLAR_NSC_Pos = 0x1
	// Bit mask of NSC field.
	PPB_SAU_RLAR_NSC_Msk = 0x2
	// Bit NSC.
	PPB_SAU_RLAR_NSC = 0x2
	// Position of ENABLE field.
	PPB_SAU_RLAR_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	PPB_SAU_RLAR_ENABLE_Msk = 0x1
	// Bit ENABLE.
	PPB_SAU_RLAR_ENABLE = 0x1

	// SFSR: Provides information about any security related faults
	// Position of LSERR field.
	PPB_SFSR_LSERR_Pos = 0x7
	// Bit mask of LSERR field.
	PPB_SFSR_LSERR_Msk = 0x80
	// Bit LSERR.
	PPB_SFSR_LSERR = 0x80
	// Position of SFARVALID field.
	PPB_SFSR_SFARVALID_Pos = 0x6
	// Bit mask of SFARVALID field.
	PPB_SFSR_SFARVALID_Msk = 0x40
	// Bit SFARVALID.
	PPB_SFSR_SFARVALID = 0x40
	// Position of LSPERR field.
	PPB_SFSR_LSPERR_Pos = 0x5
	// Bit mask of LSPERR field.
	PPB_SFSR_LSPERR_Msk = 0x20
	// Bit LSPERR.
	PPB_SFSR_LSPERR = 0x20
	// Position of INVTRAN field.
	PPB_SFSR_INVTRAN_Pos = 0x4
	// Bit mask of INVTRAN field.
	PPB_SFSR_INVTRAN_Msk = 0x10
	// Bit INVTRAN.
	PPB_SFSR_INVTRAN = 0x10
	// Position of AUVIOL field.
	PPB_SFSR_AUVIOL_Pos = 0x3
	// Bit mask of AUVIOL field.
	PPB_SFSR_AUVIOL_Msk = 0x8
	// Bit AUVIOL.
	PPB_SFSR_AUVIOL = 0x8
	// Position of INVER field.
	PPB_SFSR_INVER_Pos = 0x2
	// Bit mask of INVER field.
	PPB_SFSR_INVER_Msk = 0x4
	// Bit INVER.
	PPB_SFSR_INVER = 0x4
	// Position of INVIS field.
	PPB_SFSR_INVIS_Pos = 0x1
	// Bit mask of INVIS field.
	PPB_SFSR_INVIS_Msk = 0x2
	// Bit INVIS.
	PPB_SFSR_INVIS = 0x2
	// Position of INVEP field.
	PPB_SFSR_INVEP_Pos = 0x0
	// Bit mask of INVEP field.
	PPB_SFSR_INVEP_Msk = 0x1
	// Bit INVEP.
	PPB_SFSR_INVEP = 0x1

	// SFAR: Shows the address of the memory location that caused a Security violation
	// Position of ADDRESS field.
	PPB_SFAR_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	PPB_SFAR_ADDRESS_Msk = 0xffffffff

	// DHCSR: Controls halting debug
	// Position of S_RESTART_ST field.
	PPB_DHCSR_S_RESTART_ST_Pos = 0x1a
	// Bit mask of S_RESTART_ST field.
	PPB_DHCSR_S_RESTART_ST_Msk = 0x4000000
	// Bit S_RESTART_ST.
	PPB_DHCSR_S_RESTART_ST = 0x4000000
	// Position of S_RESET_ST field.
	PPB_DHCSR_S_RESET_ST_Pos = 0x19
	// Bit mask of S_RESET_ST field.
	PPB_DHCSR_S_RESET_ST_Msk = 0x2000000
	// Bit S_RESET_ST.
	PPB_DHCSR_S_RESET_ST = 0x2000000
	// Position of S_RETIRE_ST field.
	PPB_DHCSR_S_RETIRE_ST_Pos = 0x18
	// Bit mask of S_RETIRE_ST field.
	PPB_DHCSR_S_RETIRE_ST_Msk = 0x1000000
	// Bit S_RETIRE_ST.
	PPB_DHCSR_S_RETIRE_ST = 0x1000000
	// Position of S_SDE field.
	PPB_DHCSR_S_SDE_Pos = 0x14
	// Bit mask of S_SDE field.
	PPB_DHCSR_S_SDE_Msk = 0x100000
	// Bit S_SDE.
	PPB_DHCSR_S_SDE = 0x100000
	// Position of S_LOCKUP field.
	PPB_DHCSR_S_LOCKUP_Pos = 0x13
	// Bit mask of S_LOCKUP field.
	PPB_DHCSR_S_LOCKUP_Msk = 0x80000
	// Bit S_LOCKUP.
	PPB_DHCSR_S_LOCKUP = 0x80000
	// Position of S_SLEEP field.
	PPB_DHCSR_S_SLEEP_Pos = 0x12
	// Bit mask of S_SLEEP field.
	PPB_DHCSR_S_SLEEP_Msk = 0x40000
	// Bit S_SLEEP.
	PPB_DHCSR_S_SLEEP = 0x40000
	// Position of S_HALT field.
	PPB_DHCSR_S_HALT_Pos = 0x11
	// Bit mask of S_HALT field.
	PPB_DHCSR_S_HALT_Msk = 0x20000
	// Bit S_HALT.
	PPB_DHCSR_S_HALT = 0x20000
	// Position of S_REGRDY field.
	PPB_DHCSR_S_REGRDY_Pos = 0x10
	// Bit mask of S_REGRDY field.
	PPB_DHCSR_S_REGRDY_Msk = 0x10000
	// Bit S_REGRDY.
	PPB_DHCSR_S_REGRDY = 0x10000
	// Position of C_SNAPSTALL field.
	PPB_DHCSR_C_SNAPSTALL_Pos = 0x5
	// Bit mask of C_SNAPSTALL field.
	PPB_DHCSR_C_SNAPSTALL_Msk = 0x20
	// Bit C_SNAPSTALL.
	PPB_DHCSR_C_SNAPSTALL = 0x20
	// Position of C_MASKINTS field.
	PPB_DHCSR_C_MASKINTS_Pos = 0x3
	// Bit mask of C_MASKINTS field.
	PPB_DHCSR_C_MASKINTS_Msk = 0x8
	// Bit C_MASKINTS.
	PPB_DHCSR_C_MASKINTS = 0x8
	// Position of C_STEP field.
	PPB_DHCSR_C_STEP_Pos = 0x2
	// Bit mask of C_STEP field.
	PPB_DHCSR_C_STEP_Msk = 0x4
	// Bit C_STEP.
	PPB_DHCSR_C_STEP = 0x4
	// Position of C_HALT field.
	PPB_DHCSR_C_HALT_Pos = 0x1
	// Bit mask of C_HALT field.
	PPB_DHCSR_C_HALT_Msk = 0x2
	// Bit C_HALT.
	PPB_DHCSR_C_HALT = 0x2
	// Position of C_DEBUGEN field.
	PPB_DHCSR_C_DEBUGEN_Pos = 0x0
	// Bit mask of C_DEBUGEN field.
	PPB_DHCSR_C_DEBUGEN_Msk = 0x1
	// Bit C_DEBUGEN.
	PPB_DHCSR_C_DEBUGEN = 0x1

	// DCRSR: With the DCRDR, provides debug access to the general-purpose registers, special-purpose registers, and the FP extension registers. A write to the DCRSR specifies the register to transfer, whether the transfer is a read or write, and starts the transfer
	// Position of REGWNR field.
	PPB_DCRSR_REGWNR_Pos = 0x10
	// Bit mask of REGWNR field.
	PPB_DCRSR_REGWNR_Msk = 0x10000
	// Bit REGWNR.
	PPB_DCRSR_REGWNR = 0x10000
	// Position of REGSEL field.
	PPB_DCRSR_REGSEL_Pos = 0x0
	// Bit mask of REGSEL field.
	PPB_DCRSR_REGSEL_Msk = 0x7f

	// DCRDR: With the DCRSR, provides debug access to the general-purpose registers, special-purpose registers, and the FP Extension registers. If the Main Extension is implemented, it can also be used for message passing between an external debugger and a debug agent running on the PE
	// Position of DBGTMP field.
	PPB_DCRDR_DBGTMP_Pos = 0x0
	// Bit mask of DBGTMP field.
	PPB_DCRDR_DBGTMP_Msk = 0xffffffff

	// DEMCR: Manages vector catch behavior and DebugMonitor handling when debugging
	// Position of TRCENA field.
	PPB_DEMCR_TRCENA_Pos = 0x18
	// Bit mask of TRCENA field.
	PPB_DEMCR_TRCENA_Msk = 0x1000000
	// Bit TRCENA.
	PPB_DEMCR_TRCENA = 0x1000000
	// Position of SDME field.
	PPB_DEMCR_SDME_Pos = 0x14
	// Bit mask of SDME field.
	PPB_DEMCR_SDME_Msk = 0x100000
	// Bit SDME.
	PPB_DEMCR_SDME = 0x100000
	// Position of MON_REQ field.
	PPB_DEMCR_MON_REQ_Pos = 0x13
	// Bit mask of MON_REQ field.
	PPB_DEMCR_MON_REQ_Msk = 0x80000
	// Bit MON_REQ.
	PPB_DEMCR_MON_REQ = 0x80000
	// Position of MON_STEP field.
	PPB_DEMCR_MON_STEP_Pos = 0x12
	// Bit mask of MON_STEP field.
	PPB_DEMCR_MON_STEP_Msk = 0x40000
	// Bit MON_STEP.
	PPB_DEMCR_MON_STEP = 0x40000
	// Position of MON_PEND field.
	PPB_DEMCR_MON_PEND_Pos = 0x11
	// Bit mask of MON_PEND field.
	PPB_DEMCR_MON_PEND_Msk = 0x20000
	// Bit MON_PEND.
	PPB_DEMCR_MON_PEND = 0x20000
	// Position of MON_EN field.
	PPB_DEMCR_MON_EN_Pos = 0x10
	// Bit mask of MON_EN field.
	PPB_DEMCR_MON_EN_Msk = 0x10000
	// Bit MON_EN.
	PPB_DEMCR_MON_EN = 0x10000
	// Position of VC_SFERR field.
	PPB_DEMCR_VC_SFERR_Pos = 0xb
	// Bit mask of VC_SFERR field.
	PPB_DEMCR_VC_SFERR_Msk = 0x800
	// Bit VC_SFERR.
	PPB_DEMCR_VC_SFERR = 0x800
	// Position of VC_HARDERR field.
	PPB_DEMCR_VC_HARDERR_Pos = 0xa
	// Bit mask of VC_HARDERR field.
	PPB_DEMCR_VC_HARDERR_Msk = 0x400
	// Bit VC_HARDERR.
	PPB_DEMCR_VC_HARDERR = 0x400
	// Position of VC_INTERR field.
	PPB_DEMCR_VC_INTERR_Pos = 0x9
	// Bit mask of VC_INTERR field.
	PPB_DEMCR_VC_INTERR_Msk = 0x200
	// Bit VC_INTERR.
	PPB_DEMCR_VC_INTERR = 0x200
	// Position of VC_BUSERR field.
	PPB_DEMCR_VC_BUSERR_Pos = 0x8
	// Bit mask of VC_BUSERR field.
	PPB_DEMCR_VC_BUSERR_Msk = 0x100
	// Bit VC_BUSERR.
	PPB_DEMCR_VC_BUSERR = 0x100
	// Position of VC_STATERR field.
	PPB_DEMCR_VC_STATERR_Pos = 0x7
	// Bit mask of VC_STATERR field.
	PPB_DEMCR_VC_STATERR_Msk = 0x80
	// Bit VC_STATERR.
	PPB_DEMCR_VC_STATERR = 0x80
	// Position of VC_CHKERR field.
	PPB_DEMCR_VC_CHKERR_Pos = 0x6
	// Bit mask of VC_CHKERR field.
	PPB_DEMCR_VC_CHKERR_Msk = 0x40
	// Bit VC_CHKERR.
	PPB_DEMCR_VC_CHKERR = 0x40
	// Position of VC_NOCPERR field.
	PPB_DEMCR_VC_NOCPERR_Pos = 0x5
	// Bit mask of VC_NOCPERR field.
	PPB_DEMCR_VC_NOCPERR_Msk = 0x20
	// Bit VC_NOCPERR.
	PPB_DEMCR_VC_NOCPERR = 0x20
	// Position of VC_MMERR field.
	PPB_DEMCR_VC_MMERR_Pos = 0x4
	// Bit mask of VC_MMERR field.
	PPB_DEMCR_VC_MMERR_Msk = 0x10
	// Bit VC_MMERR.
	PPB_DEMCR_VC_MMERR = 0x10
	// Position of VC_CORERESET field.
	PPB_DEMCR_VC_CORERESET_Pos = 0x0
	// Bit mask of VC_CORERESET field.
	PPB_DEMCR_VC_CORERESET_Msk = 0x1
	// Bit VC_CORERESET.
	PPB_DEMCR_VC_CORERESET = 0x1

	// DSCSR: Provides control and status information for Secure debug
	// Position of CDSKEY field.
	PPB_DSCSR_CDSKEY_Pos = 0x11
	// Bit mask of CDSKEY field.
	PPB_DSCSR_CDSKEY_Msk = 0x20000
	// Bit CDSKEY.
	PPB_DSCSR_CDSKEY = 0x20000
	// Position of CDS field.
	PPB_DSCSR_CDS_Pos = 0x10
	// Bit mask of CDS field.
	PPB_DSCSR_CDS_Msk = 0x10000
	// Bit CDS.
	PPB_DSCSR_CDS = 0x10000
	// Position of SBRSEL field.
	PPB_DSCSR_SBRSEL_Pos = 0x1
	// Bit mask of SBRSEL field.
	PPB_DSCSR_SBRSEL_Msk = 0x2
	// Bit SBRSEL.
	PPB_DSCSR_SBRSEL = 0x2
	// Position of SBRSELEN field.
	PPB_DSCSR_SBRSELEN_Pos = 0x0
	// Bit mask of SBRSELEN field.
	PPB_DSCSR_SBRSELEN_Msk = 0x1
	// Bit SBRSELEN.
	PPB_DSCSR_SBRSELEN = 0x1

	// STIR: Provides a mechanism for software to generate an interrupt
	// Position of INTID field.
	PPB_STIR_INTID_Pos = 0x0
	// Bit mask of INTID field.
	PPB_STIR_INTID_Msk = 0x1ff

	// FPCCR: Holds control data for the Floating-point extension
	// Position of ASPEN field.
	PPB_FPCCR_ASPEN_Pos = 0x1f
	// Bit mask of ASPEN field.
	PPB_FPCCR_ASPEN_Msk = 0x80000000
	// Bit ASPEN.
	PPB_FPCCR_ASPEN = 0x80000000
	// Position of LSPEN field.
	PPB_FPCCR_LSPEN_Pos = 0x1e
	// Bit mask of LSPEN field.
	PPB_FPCCR_LSPEN_Msk = 0x40000000
	// Bit LSPEN.
	PPB_FPCCR_LSPEN = 0x40000000
	// Position of LSPENS field.
	PPB_FPCCR_LSPENS_Pos = 0x1d
	// Bit mask of LSPENS field.
	PPB_FPCCR_LSPENS_Msk = 0x20000000
	// Bit LSPENS.
	PPB_FPCCR_LSPENS = 0x20000000
	// Position of CLRONRET field.
	PPB_FPCCR_CLRONRET_Pos = 0x1c
	// Bit mask of CLRONRET field.
	PPB_FPCCR_CLRONRET_Msk = 0x10000000
	// Bit CLRONRET.
	PPB_FPCCR_CLRONRET = 0x10000000
	// Position of CLRONRETS field.
	PPB_FPCCR_CLRONRETS_Pos = 0x1b
	// Bit mask of CLRONRETS field.
	PPB_FPCCR_CLRONRETS_Msk = 0x8000000
	// Bit CLRONRETS.
	PPB_FPCCR_CLRONRETS = 0x8000000
	// Position of TS field.
	PPB_FPCCR_TS_Pos = 0x1a
	// Bit mask of TS field.
	PPB_FPCCR_TS_Msk = 0x4000000
	// Bit TS.
	PPB_FPCCR_TS = 0x4000000
	// Position of UFRDY field.
	PPB_FPCCR_UFRDY_Pos = 0xa
	// Bit mask of UFRDY field.
	PPB_FPCCR_UFRDY_Msk = 0x400
	// Bit UFRDY.
	PPB_FPCCR_UFRDY = 0x400
	// Position of SPLIMVIOL field.
	PPB_FPCCR_SPLIMVIOL_Pos = 0x9
	// Bit mask of SPLIMVIOL field.
	PPB_FPCCR_SPLIMVIOL_Msk = 0x200
	// Bit SPLIMVIOL.
	PPB_FPCCR_SPLIMVIOL = 0x200
	// Position of MONRDY field.
	PPB_FPCCR_MONRDY_Pos = 0x8
	// Bit mask of MONRDY field.
	PPB_FPCCR_MONRDY_Msk = 0x100
	// Bit MONRDY.
	PPB_FPCCR_MONRDY = 0x100
	// Position of SFRDY field.
	PPB_FPCCR_SFRDY_Pos = 0x7
	// Bit mask of SFRDY field.
	PPB_FPCCR_SFRDY_Msk = 0x80
	// Bit SFRDY.
	PPB_FPCCR_SFRDY = 0x80
	// Position of BFRDY field.
	PPB_FPCCR_BFRDY_Pos = 0x6
	// Bit mask of BFRDY field.
	PPB_FPCCR_BFRDY_Msk = 0x40
	// Bit BFRDY.
	PPB_FPCCR_BFRDY = 0x40
	// Position of MMRDY field.
	PPB_FPCCR_MMRDY_Pos = 0x5
	// Bit mask of MMRDY field.
	PPB_FPCCR_MMRDY_Msk = 0x20
	// Bit MMRDY.
	PPB_FPCCR_MMRDY = 0x20
	// Position of HFRDY field.
	PPB_FPCCR_HFRDY_Pos = 0x4
	// Bit mask of HFRDY field.
	PPB_FPCCR_HFRDY_Msk = 0x10
	// Bit HFRDY.
	PPB_FPCCR_HFRDY = 0x10
	// Position of THREAD field.
	PPB_FPCCR_THREAD_Pos = 0x3
	// Bit mask of THREAD field.
	PPB_FPCCR_THREAD_Msk = 0x8
	// Bit THREAD.
	PPB_FPCCR_THREAD = 0x8
	// Position of S field.
	PPB_FPCCR_S_Pos = 0x2
	// Bit mask of S field.
	PPB_FPCCR_S_Msk = 0x4
	// Bit S.
	PPB_FPCCR_S = 0x4
	// Position of USER field.
	PPB_FPCCR_USER_Pos = 0x1
	// Bit mask of USER field.
	PPB_FPCCR_USER_Msk = 0x2
	// Bit USER.
	PPB_FPCCR_USER = 0x2
	// Position of LSPACT field.
	PPB_FPCCR_LSPACT_Pos = 0x0
	// Bit mask of LSPACT field.
	PPB_FPCCR_LSPACT_Msk = 0x1
	// Bit LSPACT.
	PPB_FPCCR_LSPACT = 0x1

	// FPCAR: Holds the location of the unpopulated floating-point register space allocated on an exception stack frame
	// Position of ADDRESS field.
	PPB_FPCAR_ADDRESS_Pos = 0x3
	// Bit mask of ADDRESS field.
	PPB_FPCAR_ADDRESS_Msk = 0xfffffff8

	// FPDSCR: Holds the default values for the floating-point status control data that the PE assigns to the FPSCR when it creates a new floating-point context
	// Position of AHP field.
	PPB_FPDSCR_AHP_Pos = 0x1a
	// Bit mask of AHP field.
	PPB_FPDSCR_AHP_Msk = 0x4000000
	// Bit AHP.
	PPB_FPDSCR_AHP = 0x4000000
	// Position of DN field.
	PPB_FPDSCR_DN_Pos = 0x19
	// Bit mask of DN field.
	PPB_FPDSCR_DN_Msk = 0x2000000
	// Bit DN.
	PPB_FPDSCR_DN = 0x2000000
	// Position of FZ field.
	PPB_FPDSCR_FZ_Pos = 0x18
	// Bit mask of FZ field.
	PPB_FPDSCR_FZ_Msk = 0x1000000
	// Bit FZ.
	PPB_FPDSCR_FZ = 0x1000000
	// Position of RMODE field.
	PPB_FPDSCR_RMODE_Pos = 0x16
	// Bit mask of RMODE field.
	PPB_FPDSCR_RMODE_Msk = 0xc00000

	// MVFR0: Describes the features provided by the Floating-point Extension
	// Position of FPROUND field.
	PPB_MVFR0_FPROUND_Pos = 0x1c
	// Bit mask of FPROUND field.
	PPB_MVFR0_FPROUND_Msk = 0xf0000000
	// Position of FPSQRT field.
	PPB_MVFR0_FPSQRT_Pos = 0x14
	// Bit mask of FPSQRT field.
	PPB_MVFR0_FPSQRT_Msk = 0xf00000
	// Position of FPDIVIDE field.
	PPB_MVFR0_FPDIVIDE_Pos = 0x10
	// Bit mask of FPDIVIDE field.
	PPB_MVFR0_FPDIVIDE_Msk = 0xf0000
	// Position of FPDP field.
	PPB_MVFR0_FPDP_Pos = 0x8
	// Bit mask of FPDP field.
	PPB_MVFR0_FPDP_Msk = 0xf00
	// Position of FPSP field.
	PPB_MVFR0_FPSP_Pos = 0x4
	// Bit mask of FPSP field.
	PPB_MVFR0_FPSP_Msk = 0xf0
	// Position of SIMDREG field.
	PPB_MVFR0_SIMDREG_Pos = 0x0
	// Bit mask of SIMDREG field.
	PPB_MVFR0_SIMDREG_Msk = 0xf

	// MVFR1: Describes the features provided by the Floating-point Extension
	// Position of FMAC field.
	PPB_MVFR1_FMAC_Pos = 0x1c
	// Bit mask of FMAC field.
	PPB_MVFR1_FMAC_Msk = 0xf0000000
	// Position of FPHP field.
	PPB_MVFR1_FPHP_Pos = 0x18
	// Bit mask of FPHP field.
	PPB_MVFR1_FPHP_Msk = 0xf000000
	// Position of FPDNAN field.
	PPB_MVFR1_FPDNAN_Pos = 0x4
	// Bit mask of FPDNAN field.
	PPB_MVFR1_FPDNAN_Msk = 0xf0
	// Position of FPFTZ field.
	PPB_MVFR1_FPFTZ_Pos = 0x0
	// Bit mask of FPFTZ field.
	PPB_MVFR1_FPFTZ_Msk = 0xf

	// MVFR2: Describes the features provided by the Floating-point Extension
	// Position of FPMISC field.
	PPB_MVFR2_FPMISC_Pos = 0x4
	// Bit mask of FPMISC field.
	PPB_MVFR2_FPMISC_Msk = 0xf0

	// DDEVARCH: Provides CoreSight discovery information for the SCS
	// Position of ARCHITECT field.
	PPB_DDEVARCH_ARCHITECT_Pos = 0x15
	// Bit mask of ARCHITECT field.
	PPB_DDEVARCH_ARCHITECT_Msk = 0xffe00000
	// Position of PRESENT field.
	PPB_DDEVARCH_PRESENT_Pos = 0x14
	// Bit mask of PRESENT field.
	PPB_DDEVARCH_PRESENT_Msk = 0x100000
	// Bit PRESENT.
	PPB_DDEVARCH_PRESENT = 0x100000
	// Position of REVISION field.
	PPB_DDEVARCH_REVISION_Pos = 0x10
	// Bit mask of REVISION field.
	PPB_DDEVARCH_REVISION_Msk = 0xf0000
	// Position of ARCHVER field.
	PPB_DDEVARCH_ARCHVER_Pos = 0xc
	// Bit mask of ARCHVER field.
	PPB_DDEVARCH_ARCHVER_Msk = 0xf000
	// Position of ARCHPART field.
	PPB_DDEVARCH_ARCHPART_Pos = 0x0
	// Bit mask of ARCHPART field.
	PPB_DDEVARCH_ARCHPART_Msk = 0xfff

	// DDEVTYPE: Provides CoreSight discovery information for the SCS
	// Position of SUB field.
	PPB_DDEVTYPE_SUB_Pos = 0x4
	// Bit mask of SUB field.
	PPB_DDEVTYPE_SUB_Msk = 0xf0
	// Position of MAJOR field.
	PPB_DDEVTYPE_MAJOR_Pos = 0x0
	// Bit mask of MAJOR field.
	PPB_DDEVTYPE_MAJOR_Msk = 0xf

	// DPIDR4: Provides CoreSight discovery information for the SCS
	// Position of SIZE field.
	PPB_DPIDR4_SIZE_Pos = 0x4
	// Bit mask of SIZE field.
	PPB_DPIDR4_SIZE_Msk = 0xf0
	// Position of DES_2 field.
	PPB_DPIDR4_DES_2_Pos = 0x0
	// Bit mask of DES_2 field.
	PPB_DPIDR4_DES_2_Msk = 0xf

	// DPIDR5: Provides CoreSight discovery information for the SCS
	// Position of DPIDR5 field.
	PPB_DPIDR5_DPIDR5_Pos = 0x0
	// Bit mask of DPIDR5 field.
	PPB_DPIDR5_DPIDR5_Msk = 0xffffffff

	// DPIDR6: Provides CoreSight discovery information for the SCS
	// Position of DPIDR6 field.
	PPB_DPIDR6_DPIDR6_Pos = 0x0
	// Bit mask of DPIDR6 field.
	PPB_DPIDR6_DPIDR6_Msk = 0xffffffff

	// DPIDR7: Provides CoreSight discovery information for the SCS
	// Position of DPIDR7 field.
	PPB_DPIDR7_DPIDR7_Pos = 0x0
	// Bit mask of DPIDR7 field.
	PPB_DPIDR7_DPIDR7_Msk = 0xffffffff

	// DPIDR0: Provides CoreSight discovery information for the SCS
	// Position of PART_0 field.
	PPB_DPIDR0_PART_0_Pos = 0x0
	// Bit mask of PART_0 field.
	PPB_DPIDR0_PART_0_Msk = 0xff

	// DPIDR1: Provides CoreSight discovery information for the SCS
	// Position of DES_0 field.
	PPB_DPIDR1_DES_0_Pos = 0x4
	// Bit mask of DES_0 field.
	PPB_DPIDR1_DES_0_Msk = 0xf0
	// Position of PART_1 field.
	PPB_DPIDR1_PART_1_Pos = 0x0
	// Bit mask of PART_1 field.
	PPB_DPIDR1_PART_1_Msk = 0xf

	// DPIDR2: Provides CoreSight discovery information for the SCS
	// Position of REVISION field.
	PPB_DPIDR2_REVISION_Pos = 0x4
	// Bit mask of REVISION field.
	PPB_DPIDR2_REVISION_Msk = 0xf0
	// Position of JEDEC field.
	PPB_DPIDR2_JEDEC_Pos = 0x3
	// Bit mask of JEDEC field.
	PPB_DPIDR2_JEDEC_Msk = 0x8
	// Bit JEDEC.
	PPB_DPIDR2_JEDEC = 0x8
	// Position of DES_1 field.
	PPB_DPIDR2_DES_1_Pos = 0x0
	// Bit mask of DES_1 field.
	PPB_DPIDR2_DES_1_Msk = 0x7

	// DPIDR3: Provides CoreSight discovery information for the SCS
	// Position of REVAND field.
	PPB_DPIDR3_REVAND_Pos = 0x4
	// Bit mask of REVAND field.
	PPB_DPIDR3_REVAND_Msk = 0xf0
	// Position of CMOD field.
	PPB_DPIDR3_CMOD_Pos = 0x0
	// Bit mask of CMOD field.
	PPB_DPIDR3_CMOD_Msk = 0xf

	// DCIDR0: Provides CoreSight discovery information for the SCS
	// Position of PRMBL_0 field.
	PPB_DCIDR0_PRMBL_0_Pos = 0x0
	// Bit mask of PRMBL_0 field.
	PPB_DCIDR0_PRMBL_0_Msk = 0xff

	// DCIDR1: Provides CoreSight discovery information for the SCS
	// Position of CLASS field.
	PPB_DCIDR1_CLASS_Pos = 0x4
	// Bit mask of CLASS field.
	PPB_DCIDR1_CLASS_Msk = 0xf0
	// Position of PRMBL_1 field.
	PPB_DCIDR1_PRMBL_1_Pos = 0x0
	// Bit mask of PRMBL_1 field.
	PPB_DCIDR1_PRMBL_1_Msk = 0xf

	// DCIDR2: Provides CoreSight discovery information for the SCS
	// Position of PRMBL_2 field.
	PPB_DCIDR2_PRMBL_2_Pos = 0x0
	// Bit mask of PRMBL_2 field.
	PPB_DCIDR2_PRMBL_2_Msk = 0xff

	// DCIDR3: Provides CoreSight discovery information for the SCS
	// Position of PRMBL_3 field.
	PPB_DCIDR3_PRMBL_3_Pos = 0x0
	// Bit mask of PRMBL_3 field.
	PPB_DCIDR3_PRMBL_3_Msk = 0xff

	// TRCPRGCTLR: Programming Control Register
	// Position of EN field.
	PPB_TRCPRGCTLR_EN_Pos = 0x0
	// Bit mask of EN field.
	PPB_TRCPRGCTLR_EN_Msk = 0x1
	// Bit EN.
	PPB_TRCPRGCTLR_EN = 0x1

	// TRCSTATR: The TRCSTATR indicates the ETM-Teal status
	// Position of PMSTABLE field.
	PPB_TRCSTATR_PMSTABLE_Pos = 0x1
	// Bit mask of PMSTABLE field.
	PPB_TRCSTATR_PMSTABLE_Msk = 0x2
	// Bit PMSTABLE.
	PPB_TRCSTATR_PMSTABLE = 0x2
	// Position of IDLE field.
	PPB_TRCSTATR_IDLE_Pos = 0x0
	// Bit mask of IDLE field.
	PPB_TRCSTATR_IDLE_Msk = 0x1
	// Bit IDLE.
	PPB_TRCSTATR_IDLE = 0x1

	// TRCCONFIGR: The TRCCONFIGR sets the basic tracing options for the trace unit
	// Position of RS field.
	PPB_TRCCONFIGR_RS_Pos = 0xc
	// Bit mask of RS field.
	PPB_TRCCONFIGR_RS_Msk = 0x1000
	// Bit RS.
	PPB_TRCCONFIGR_RS = 0x1000
	// Position of TS field.
	PPB_TRCCONFIGR_TS_Pos = 0xb
	// Bit mask of TS field.
	PPB_TRCCONFIGR_TS_Msk = 0x800
	// Bit TS.
	PPB_TRCCONFIGR_TS = 0x800
	// Position of COND field.
	PPB_TRCCONFIGR_COND_Pos = 0x5
	// Bit mask of COND field.
	PPB_TRCCONFIGR_COND_Msk = 0x7e0
	// Position of CCI field.
	PPB_TRCCONFIGR_CCI_Pos = 0x4
	// Bit mask of CCI field.
	PPB_TRCCONFIGR_CCI_Msk = 0x10
	// Bit CCI.
	PPB_TRCCONFIGR_CCI = 0x10
	// Position of BB field.
	PPB_TRCCONFIGR_BB_Pos = 0x3
	// Bit mask of BB field.
	PPB_TRCCONFIGR_BB_Msk = 0x8
	// Bit BB.
	PPB_TRCCONFIGR_BB = 0x8

	// TRCEVENTCTL0R: The TRCEVENTCTL0R controls the tracing of events in the trace stream. The events also drive the ETM-Teal external outputs.
	// Position of TYPE1 field.
	PPB_TRCEVENTCTL0R_TYPE1_Pos = 0xf
	// Bit mask of TYPE1 field.
	PPB_TRCEVENTCTL0R_TYPE1_Msk = 0x8000
	// Bit TYPE1.
	PPB_TRCEVENTCTL0R_TYPE1 = 0x8000
	// Position of SEL1 field.
	PPB_TRCEVENTCTL0R_SEL1_Pos = 0x8
	// Bit mask of SEL1 field.
	PPB_TRCEVENTCTL0R_SEL1_Msk = 0x700
	// Position of TYPE0 field.
	PPB_TRCEVENTCTL0R_TYPE0_Pos = 0x7
	// Bit mask of TYPE0 field.
	PPB_TRCEVENTCTL0R_TYPE0_Msk = 0x80
	// Bit TYPE0.
	PPB_TRCEVENTCTL0R_TYPE0 = 0x80
	// Position of SEL0 field.
	PPB_TRCEVENTCTL0R_SEL0_Pos = 0x0
	// Bit mask of SEL0 field.
	PPB_TRCEVENTCTL0R_SEL0_Msk = 0x7

	// TRCEVENTCTL1R: The TRCEVENTCTL1R controls how the events selected by TRCEVENTCTL0R behave
	// Position of LPOVERRIDE field.
	PPB_TRCEVENTCTL1R_LPOVERRIDE_Pos = 0xc
	// Bit mask of LPOVERRIDE field.
	PPB_TRCEVENTCTL1R_LPOVERRIDE_Msk = 0x1000
	// Bit LPOVERRIDE.
	PPB_TRCEVENTCTL1R_LPOVERRIDE = 0x1000
	// Position of ATB field.
	PPB_TRCEVENTCTL1R_ATB_Pos = 0xb
	// Bit mask of ATB field.
	PPB_TRCEVENTCTL1R_ATB_Msk = 0x800
	// Bit ATB.
	PPB_TRCEVENTCTL1R_ATB = 0x800
	// Position of INSTEN1 field.
	PPB_TRCEVENTCTL1R_INSTEN1_Pos = 0x1
	// Bit mask of INSTEN1 field.
	PPB_TRCEVENTCTL1R_INSTEN1_Msk = 0x2
	// Bit INSTEN1.
	PPB_TRCEVENTCTL1R_INSTEN1 = 0x2
	// Position of INSTEN0 field.
	PPB_TRCEVENTCTL1R_INSTEN0_Pos = 0x0
	// Bit mask of INSTEN0 field.
	PPB_TRCEVENTCTL1R_INSTEN0_Msk = 0x1
	// Bit INSTEN0.
	PPB_TRCEVENTCTL1R_INSTEN0 = 0x1

	// TRCSTALLCTLR: The TRCSTALLCTLR enables ETM-Teal to stall the processor if the ETM-Teal FIFO goes over the programmed level to minimize risk of overflow
	// Position of INSTPRIORITY field.
	PPB_TRCSTALLCTLR_INSTPRIORITY_Pos = 0xa
	// Bit mask of INSTPRIORITY field.
	PPB_TRCSTALLCTLR_INSTPRIORITY_Msk = 0x400
	// Bit INSTPRIORITY.
	PPB_TRCSTALLCTLR_INSTPRIORITY = 0x400
	// Position of ISTALL field.
	PPB_TRCSTALLCTLR_ISTALL_Pos = 0x8
	// Bit mask of ISTALL field.
	PPB_TRCSTALLCTLR_ISTALL_Msk = 0x100
	// Bit ISTALL.
	PPB_TRCSTALLCTLR_ISTALL = 0x100
	// Position of LEVEL field.
	PPB_TRCSTALLCTLR_LEVEL_Pos = 0x2
	// Bit mask of LEVEL field.
	PPB_TRCSTALLCTLR_LEVEL_Msk = 0xc

	// TRCTSCTLR: The TRCTSCTLR controls the insertion of global timestamps into the trace stream. A timestamp is always inserted into the instruction trace stream
	// Position of TYPE0 field.
	PPB_TRCTSCTLR_TYPE0_Pos = 0x7
	// Bit mask of TYPE0 field.
	PPB_TRCTSCTLR_TYPE0_Msk = 0x80
	// Bit TYPE0.
	PPB_TRCTSCTLR_TYPE0 = 0x80
	// Position of SEL0 field.
	PPB_TRCTSCTLR_SEL0_Pos = 0x0
	// Bit mask of SEL0 field.
	PPB_TRCTSCTLR_SEL0_Msk = 0x3

	// TRCSYNCPR: The TRCSYNCPR specifies the period of trace synchronization of the trace streams. TRCSYNCPR defines a number of bytes of trace between requests for trace synchronization. This value is always a power of two
	// Position of PERIOD field.
	PPB_TRCSYNCPR_PERIOD_Pos = 0x0
	// Bit mask of PERIOD field.
	PPB_TRCSYNCPR_PERIOD_Msk = 0x1f

	// TRCCCCTLR: The TRCCCCTLR sets the threshold value for instruction trace cycle counting. The threshold represents the minimum interval between cycle count trace packets
	// Position of THRESHOLD field.
	PPB_TRCCCCTLR_THRESHOLD_Pos = 0x0
	// Bit mask of THRESHOLD field.
	PPB_TRCCCCTLR_THRESHOLD_Msk = 0xfff

	// TRCVICTLR: The TRCVICTLR controls instruction trace filtering
	// Position of EXLEVEL_S3 field.
	PPB_TRCVICTLR_EXLEVEL_S3_Pos = 0x13
	// Bit mask of EXLEVEL_S3 field.
	PPB_TRCVICTLR_EXLEVEL_S3_Msk = 0x80000
	// Bit EXLEVEL_S3.
	PPB_TRCVICTLR_EXLEVEL_S3 = 0x80000
	// Position of EXLEVEL_S0 field.
	PPB_TRCVICTLR_EXLEVEL_S0_Pos = 0x10
	// Bit mask of EXLEVEL_S0 field.
	PPB_TRCVICTLR_EXLEVEL_S0_Msk = 0x10000
	// Bit EXLEVEL_S0.
	PPB_TRCVICTLR_EXLEVEL_S0 = 0x10000
	// Position of TRCERR field.
	PPB_TRCVICTLR_TRCERR_Pos = 0xb
	// Bit mask of TRCERR field.
	PPB_TRCVICTLR_TRCERR_Msk = 0x800
	// Bit TRCERR.
	PPB_TRCVICTLR_TRCERR = 0x800
	// Position of TRCRESET field.
	PPB_TRCVICTLR_TRCRESET_Pos = 0xa
	// Bit mask of TRCRESET field.
	PPB_TRCVICTLR_TRCRESET_Msk = 0x400
	// Bit TRCRESET.
	PPB_TRCVICTLR_TRCRESET = 0x400
	// Position of SSSTATUS field.
	PPB_TRCVICTLR_SSSTATUS_Pos = 0x9
	// Bit mask of SSSTATUS field.
	PPB_TRCVICTLR_SSSTATUS_Msk = 0x200
	// Bit SSSTATUS.
	PPB_TRCVICTLR_SSSTATUS = 0x200
	// Position of TYPE0 field.
	PPB_TRCVICTLR_TYPE0_Pos = 0x7
	// Bit mask of TYPE0 field.
	PPB_TRCVICTLR_TYPE0_Msk = 0x80
	// Bit TYPE0.
	PPB_TRCVICTLR_TYPE0 = 0x80
	// Position of SEL0 field.
	PPB_TRCVICTLR_SEL0_Pos = 0x0
	// Bit mask of SEL0 field.
	PPB_TRCVICTLR_SEL0_Msk = 0x3

	// TRCCNTRLDVR0: The TRCCNTRLDVR defines the reload value for the reduced function counter
	// Position of VALUE field.
	PPB_TRCCNTRLDVR0_VALUE_Pos = 0x0
	// Bit mask of VALUE field.
	PPB_TRCCNTRLDVR0_VALUE_Msk = 0xffff

	// TRCIDR8: TRCIDR8
	// Position of MAXSPEC field.
	PPB_TRCIDR8_MAXSPEC_Pos = 0x0
	// Bit mask of MAXSPEC field.
	PPB_TRCIDR8_MAXSPEC_Msk = 0xffffffff

	// TRCIDR9: TRCIDR9
	// Position of NUMP0KEY field.
	PPB_TRCIDR9_NUMP0KEY_Pos = 0x0
	// Bit mask of NUMP0KEY field.
	PPB_TRCIDR9_NUMP0KEY_Msk = 0xffffffff

	// TRCIDR10: TRCIDR10
	// Position of NUMP1KEY field.
	PPB_TRCIDR10_NUMP1KEY_Pos = 0x0
	// Bit mask of NUMP1KEY field.
	PPB_TRCIDR10_NUMP1KEY_Msk = 0xffffffff

	// TRCIDR11: TRCIDR11
	// Position of NUMP1SPC field.
	PPB_TRCIDR11_NUMP1SPC_Pos = 0x0
	// Bit mask of NUMP1SPC field.
	PPB_TRCIDR11_NUMP1SPC_Msk = 0xffffffff

	// TRCIDR12: TRCIDR12
	// Position of NUMCONDKEY field.
	PPB_TRCIDR12_NUMCONDKEY_Pos = 0x0
	// Bit mask of NUMCONDKEY field.
	PPB_TRCIDR12_NUMCONDKEY_Msk = 0xffffffff

	// TRCIDR13: TRCIDR13
	// Position of NUMCONDSPC field.
	PPB_TRCIDR13_NUMCONDSPC_Pos = 0x0
	// Bit mask of NUMCONDSPC field.
	PPB_TRCIDR13_NUMCONDSPC_Msk = 0xffffffff

	// TRCIMSPEC: The TRCIMSPEC shows the presence of any IMPLEMENTATION SPECIFIC features, and enables any features that are provided
	// Position of SUPPORT field.
	PPB_TRCIMSPEC_SUPPORT_Pos = 0x0
	// Bit mask of SUPPORT field.
	PPB_TRCIMSPEC_SUPPORT_Msk = 0xf

	// TRCIDR0: TRCIDR0
	// Position of COMMOPT field.
	PPB_TRCIDR0_COMMOPT_Pos = 0x1d
	// Bit mask of COMMOPT field.
	PPB_TRCIDR0_COMMOPT_Msk = 0x20000000
	// Bit COMMOPT.
	PPB_TRCIDR0_COMMOPT = 0x20000000
	// Position of TSSIZE field.
	PPB_TRCIDR0_TSSIZE_Pos = 0x18
	// Bit mask of TSSIZE field.
	PPB_TRCIDR0_TSSIZE_Msk = 0x1f000000
	// Position of TRCEXDATA field.
	PPB_TRCIDR0_TRCEXDATA_Pos = 0x11
	// Bit mask of TRCEXDATA field.
	PPB_TRCIDR0_TRCEXDATA_Msk = 0x20000
	// Bit TRCEXDATA.
	PPB_TRCIDR0_TRCEXDATA = 0x20000
	// Position of QSUPP field.
	PPB_TRCIDR0_QSUPP_Pos = 0xf
	// Bit mask of QSUPP field.
	PPB_TRCIDR0_QSUPP_Msk = 0x18000
	// Position of QFILT field.
	PPB_TRCIDR0_QFILT_Pos = 0xe
	// Bit mask of QFILT field.
	PPB_TRCIDR0_QFILT_Msk = 0x4000
	// Bit QFILT.
	PPB_TRCIDR0_QFILT = 0x4000
	// Position of CONDTYPE field.
	PPB_TRCIDR0_CONDTYPE_Pos = 0xc
	// Bit mask of CONDTYPE field.
	PPB_TRCIDR0_CONDTYPE_Msk = 0x3000
	// Position of NUMEVENT field.
	PPB_TRCIDR0_NUMEVENT_Pos = 0xa
	// Bit mask of NUMEVENT field.
	PPB_TRCIDR0_NUMEVENT_Msk = 0xc00
	// Position of RETSTACK field.
	PPB_TRCIDR0_RETSTACK_Pos = 0x9
	// Bit mask of RETSTACK field.
	PPB_TRCIDR0_RETSTACK_Msk = 0x200
	// Bit RETSTACK.
	PPB_TRCIDR0_RETSTACK = 0x200
	// Position of TRCCCI field.
	PPB_TRCIDR0_TRCCCI_Pos = 0x7
	// Bit mask of TRCCCI field.
	PPB_TRCIDR0_TRCCCI_Msk = 0x80
	// Bit TRCCCI.
	PPB_TRCIDR0_TRCCCI = 0x80
	// Position of TRCCOND field.
	PPB_TRCIDR0_TRCCOND_Pos = 0x6
	// Bit mask of TRCCOND field.
	PPB_TRCIDR0_TRCCOND_Msk = 0x40
	// Bit TRCCOND.
	PPB_TRCIDR0_TRCCOND = 0x40
	// Position of TRCBB field.
	PPB_TRCIDR0_TRCBB_Pos = 0x5
	// Bit mask of TRCBB field.
	PPB_TRCIDR0_TRCBB_Msk = 0x20
	// Bit TRCBB.
	PPB_TRCIDR0_TRCBB = 0x20
	// Position of TRCDATA field.
	PPB_TRCIDR0_TRCDATA_Pos = 0x3
	// Bit mask of TRCDATA field.
	PPB_TRCIDR0_TRCDATA_Msk = 0x18
	// Position of INSTP0 field.
	PPB_TRCIDR0_INSTP0_Pos = 0x1
	// Bit mask of INSTP0 field.
	PPB_TRCIDR0_INSTP0_Msk = 0x6
	// Position of RES1 field.
	PPB_TRCIDR0_RES1_Pos = 0x0
	// Bit mask of RES1 field.
	PPB_TRCIDR0_RES1_Msk = 0x1
	// Bit RES1.
	PPB_TRCIDR0_RES1 = 0x1

	// TRCIDR1: TRCIDR1
	// Position of DESIGNER field.
	PPB_TRCIDR1_DESIGNER_Pos = 0x18
	// Bit mask of DESIGNER field.
	PPB_TRCIDR1_DESIGNER_Msk = 0xff000000
	// Position of RES1 field.
	PPB_TRCIDR1_RES1_Pos = 0xc
	// Bit mask of RES1 field.
	PPB_TRCIDR1_RES1_Msk = 0xf000
	// Position of TRCARCHMAJ field.
	PPB_TRCIDR1_TRCARCHMAJ_Pos = 0x8
	// Bit mask of TRCARCHMAJ field.
	PPB_TRCIDR1_TRCARCHMAJ_Msk = 0xf00
	// Position of TRCARCHMIN field.
	PPB_TRCIDR1_TRCARCHMIN_Pos = 0x4
	// Bit mask of TRCARCHMIN field.
	PPB_TRCIDR1_TRCARCHMIN_Msk = 0xf0
	// Position of REVISION field.
	PPB_TRCIDR1_REVISION_Pos = 0x0
	// Bit mask of REVISION field.
	PPB_TRCIDR1_REVISION_Msk = 0xf

	// TRCIDR2: TRCIDR2
	// Position of CCSIZE field.
	PPB_TRCIDR2_CCSIZE_Pos = 0x19
	// Bit mask of CCSIZE field.
	PPB_TRCIDR2_CCSIZE_Msk = 0x1e000000
	// Position of DVSIZE field.
	PPB_TRCIDR2_DVSIZE_Pos = 0x14
	// Bit mask of DVSIZE field.
	PPB_TRCIDR2_DVSIZE_Msk = 0x1f00000
	// Position of DASIZE field.
	PPB_TRCIDR2_DASIZE_Pos = 0xf
	// Bit mask of DASIZE field.
	PPB_TRCIDR2_DASIZE_Msk = 0xf8000
	// Position of VMIDSIZE field.
	PPB_TRCIDR2_VMIDSIZE_Pos = 0xa
	// Bit mask of VMIDSIZE field.
	PPB_TRCIDR2_VMIDSIZE_Msk = 0x7c00
	// Position of CIDSIZE field.
	PPB_TRCIDR2_CIDSIZE_Pos = 0x5
	// Bit mask of CIDSIZE field.
	PPB_TRCIDR2_CIDSIZE_Msk = 0x3e0
	// Position of IASIZE field.
	PPB_TRCIDR2_IASIZE_Pos = 0x0
	// Bit mask of IASIZE field.
	PPB_TRCIDR2_IASIZE_Msk = 0x1f

	// TRCIDR3: TRCIDR3
	// Position of NOOVERFLOW field.
	PPB_TRCIDR3_NOOVERFLOW_Pos = 0x1f
	// Bit mask of NOOVERFLOW field.
	PPB_TRCIDR3_NOOVERFLOW_Msk = 0x80000000
	// Bit NOOVERFLOW.
	PPB_TRCIDR3_NOOVERFLOW = 0x80000000
	// Position of NUMPROC field.
	PPB_TRCIDR3_NUMPROC_Pos = 0x1c
	// Bit mask of NUMPROC field.
	PPB_TRCIDR3_NUMPROC_Msk = 0x70000000
	// Position of SYSSTALL field.
	PPB_TRCIDR3_SYSSTALL_Pos = 0x1b
	// Bit mask of SYSSTALL field.
	PPB_TRCIDR3_SYSSTALL_Msk = 0x8000000
	// Bit SYSSTALL.
	PPB_TRCIDR3_SYSSTALL = 0x8000000
	// Position of STALLCTL field.
	PPB_TRCIDR3_STALLCTL_Pos = 0x1a
	// Bit mask of STALLCTL field.
	PPB_TRCIDR3_STALLCTL_Msk = 0x4000000
	// Bit STALLCTL.
	PPB_TRCIDR3_STALLCTL = 0x4000000
	// Position of SYNCPR field.
	PPB_TRCIDR3_SYNCPR_Pos = 0x19
	// Bit mask of SYNCPR field.
	PPB_TRCIDR3_SYNCPR_Msk = 0x2000000
	// Bit SYNCPR.
	PPB_TRCIDR3_SYNCPR = 0x2000000
	// Position of TRCERR field.
	PPB_TRCIDR3_TRCERR_Pos = 0x18
	// Bit mask of TRCERR field.
	PPB_TRCIDR3_TRCERR_Msk = 0x1000000
	// Bit TRCERR.
	PPB_TRCIDR3_TRCERR = 0x1000000
	// Position of EXLEVEL_NS field.
	PPB_TRCIDR3_EXLEVEL_NS_Pos = 0x14
	// Bit mask of EXLEVEL_NS field.
	PPB_TRCIDR3_EXLEVEL_NS_Msk = 0xf00000
	// Position of EXLEVEL_S field.
	PPB_TRCIDR3_EXLEVEL_S_Pos = 0x10
	// Bit mask of EXLEVEL_S field.
	PPB_TRCIDR3_EXLEVEL_S_Msk = 0xf0000
	// Position of CCITMIN field.
	PPB_TRCIDR3_CCITMIN_Pos = 0x0
	// Bit mask of CCITMIN field.
	PPB_TRCIDR3_CCITMIN_Msk = 0xfff

	// TRCIDR4: TRCIDR4
	// Position of NUMVMIDC field.
	PPB_TRCIDR4_NUMVMIDC_Pos = 0x1c
	// Bit mask of NUMVMIDC field.
	PPB_TRCIDR4_NUMVMIDC_Msk = 0xf0000000
	// Position of NUMCIDC field.
	PPB_TRCIDR4_NUMCIDC_Pos = 0x18
	// Bit mask of NUMCIDC field.
	PPB_TRCIDR4_NUMCIDC_Msk = 0xf000000
	// Position of NUMSSCC field.
	PPB_TRCIDR4_NUMSSCC_Pos = 0x14
	// Bit mask of NUMSSCC field.
	PPB_TRCIDR4_NUMSSCC_Msk = 0xf00000
	// Position of NUMRSPAIR field.
	PPB_TRCIDR4_NUMRSPAIR_Pos = 0x10
	// Bit mask of NUMRSPAIR field.
	PPB_TRCIDR4_NUMRSPAIR_Msk = 0xf0000
	// Position of NUMPC field.
	PPB_TRCIDR4_NUMPC_Pos = 0xc
	// Bit mask of NUMPC field.
	PPB_TRCIDR4_NUMPC_Msk = 0xf000
	// Position of SUPPDAC field.
	PPB_TRCIDR4_SUPPDAC_Pos = 0x8
	// Bit mask of SUPPDAC field.
	PPB_TRCIDR4_SUPPDAC_Msk = 0x100
	// Bit SUPPDAC.
	PPB_TRCIDR4_SUPPDAC = 0x100
	// Position of NUMDVC field.
	PPB_TRCIDR4_NUMDVC_Pos = 0x4
	// Bit mask of NUMDVC field.
	PPB_TRCIDR4_NUMDVC_Msk = 0xf0
	// Position of NUMACPAIRS field.
	PPB_TRCIDR4_NUMACPAIRS_Pos = 0x0
	// Bit mask of NUMACPAIRS field.
	PPB_TRCIDR4_NUMACPAIRS_Msk = 0xf

	// TRCIDR5: TRCIDR5
	// Position of REDFUNCNTR field.
	PPB_TRCIDR5_REDFUNCNTR_Pos = 0x1f
	// Bit mask of REDFUNCNTR field.
	PPB_TRCIDR5_REDFUNCNTR_Msk = 0x80000000
	// Bit REDFUNCNTR.
	PPB_TRCIDR5_REDFUNCNTR = 0x80000000
	// Position of NUMCNTR field.
	PPB_TRCIDR5_NUMCNTR_Pos = 0x1c
	// Bit mask of NUMCNTR field.
	PPB_TRCIDR5_NUMCNTR_Msk = 0x70000000
	// Position of NUMSEQSTATE field.
	PPB_TRCIDR5_NUMSEQSTATE_Pos = 0x19
	// Bit mask of NUMSEQSTATE field.
	PPB_TRCIDR5_NUMSEQSTATE_Msk = 0xe000000
	// Position of LPOVERRIDE field.
	PPB_TRCIDR5_LPOVERRIDE_Pos = 0x17
	// Bit mask of LPOVERRIDE field.
	PPB_TRCIDR5_LPOVERRIDE_Msk = 0x800000
	// Bit LPOVERRIDE.
	PPB_TRCIDR5_LPOVERRIDE = 0x800000
	// Position of ATBTRIG field.
	PPB_TRCIDR5_ATBTRIG_Pos = 0x16
	// Bit mask of ATBTRIG field.
	PPB_TRCIDR5_ATBTRIG_Msk = 0x400000
	// Bit ATBTRIG.
	PPB_TRCIDR5_ATBTRIG = 0x400000
	// Position of TRACEIDSIZE field.
	PPB_TRCIDR5_TRACEIDSIZE_Pos = 0x10
	// Bit mask of TRACEIDSIZE field.
	PPB_TRCIDR5_TRACEIDSIZE_Msk = 0x3f0000
	// Position of NUMEXTINSEL field.
	PPB_TRCIDR5_NUMEXTINSEL_Pos = 0x9
	// Bit mask of NUMEXTINSEL field.
	PPB_TRCIDR5_NUMEXTINSEL_Msk = 0xe00
	// Position of NUMEXTIN field.
	PPB_TRCIDR5_NUMEXTIN_Pos = 0x0
	// Bit mask of NUMEXTIN field.
	PPB_TRCIDR5_NUMEXTIN_Msk = 0x1ff

	// TRCIDR6: TRCIDR6
	// Position of TRCIDR6 field.
	PPB_TRCIDR6_TRCIDR6_Pos = 0x0
	// Bit mask of TRCIDR6 field.
	PPB_TRCIDR6_TRCIDR6_Msk = 0xffffffff

	// TRCIDR7: TRCIDR7
	// Position of TRCIDR7 field.
	PPB_TRCIDR7_TRCIDR7_Pos = 0x0
	// Bit mask of TRCIDR7 field.
	PPB_TRCIDR7_TRCIDR7_Msk = 0xffffffff

	// TRCRSCTLR2: The TRCRSCTLR controls the trace resources
	// Position of PAIRINV field.
	PPB_TRCRSCTLR2_PAIRINV_Pos = 0x15
	// Bit mask of PAIRINV field.
	PPB_TRCRSCTLR2_PAIRINV_Msk = 0x200000
	// Bit PAIRINV.
	PPB_TRCRSCTLR2_PAIRINV = 0x200000
	// Position of INV field.
	PPB_TRCRSCTLR2_INV_Pos = 0x14
	// Bit mask of INV field.
	PPB_TRCRSCTLR2_INV_Msk = 0x100000
	// Bit INV.
	PPB_TRCRSCTLR2_INV = 0x100000
	// Position of GROUP field.
	PPB_TRCRSCTLR2_GROUP_Pos = 0x10
	// Bit mask of GROUP field.
	PPB_TRCRSCTLR2_GROUP_Msk = 0x70000
	// Position of SELECT field.
	PPB_TRCRSCTLR2_SELECT_Pos = 0x0
	// Bit mask of SELECT field.
	PPB_TRCRSCTLR2_SELECT_Msk = 0xff

	// TRCRSCTLR3: The TRCRSCTLR controls the trace resources
	// Position of PAIRINV field.
	PPB_TRCRSCTLR3_PAIRINV_Pos = 0x15
	// Bit mask of PAIRINV field.
	PPB_TRCRSCTLR3_PAIRINV_Msk = 0x200000
	// Bit PAIRINV.
	PPB_TRCRSCTLR3_PAIRINV = 0x200000
	// Position of INV field.
	PPB_TRCRSCTLR3_INV_Pos = 0x14
	// Bit mask of INV field.
	PPB_TRCRSCTLR3_INV_Msk = 0x100000
	// Bit INV.
	PPB_TRCRSCTLR3_INV = 0x100000
	// Position of GROUP field.
	PPB_TRCRSCTLR3_GROUP_Pos = 0x10
	// Bit mask of GROUP field.
	PPB_TRCRSCTLR3_GROUP_Msk = 0x70000
	// Position of SELECT field.
	PPB_TRCRSCTLR3_SELECT_Pos = 0x0
	// Bit mask of SELECT field.
	PPB_TRCRSCTLR3_SELECT_Msk = 0xff

	// TRCSSCSR: Controls the corresponding single-shot comparator resource
	// Position of STATUS field.
	PPB_TRCSSCSR_STATUS_Pos = 0x1f
	// Bit mask of STATUS field.
	PPB_TRCSSCSR_STATUS_Msk = 0x80000000
	// Bit STATUS.
	PPB_TRCSSCSR_STATUS = 0x80000000
	// Position of PC field.
	PPB_TRCSSCSR_PC_Pos = 0x3
	// Bit mask of PC field.
	PPB_TRCSSCSR_PC_Msk = 0x8
	// Bit PC.
	PPB_TRCSSCSR_PC = 0x8
	// Position of DV field.
	PPB_TRCSSCSR_DV_Pos = 0x2
	// Bit mask of DV field.
	PPB_TRCSSCSR_DV_Msk = 0x4
	// Bit DV.
	PPB_TRCSSCSR_DV = 0x4
	// Position of DA field.
	PPB_TRCSSCSR_DA_Pos = 0x1
	// Bit mask of DA field.
	PPB_TRCSSCSR_DA_Msk = 0x2
	// Bit DA.
	PPB_TRCSSCSR_DA = 0x2
	// Position of INST field.
	PPB_TRCSSCSR_INST_Pos = 0x0
	// Bit mask of INST field.
	PPB_TRCSSCSR_INST_Msk = 0x1
	// Bit INST.
	PPB_TRCSSCSR_INST = 0x1

	// TRCSSPCICR: Selects the PE comparator inputs for Single-shot control
	// Position of PC field.
	PPB_TRCSSPCICR_PC_Pos = 0x0
	// Bit mask of PC field.
	PPB_TRCSSPCICR_PC_Msk = 0xf

	// TRCPDCR: Requests the system to provide power to the trace unit
	// Position of PU field.
	PPB_TRCPDCR_PU_Pos = 0x3
	// Bit mask of PU field.
	PPB_TRCPDCR_PU_Msk = 0x8
	// Bit PU.
	PPB_TRCPDCR_PU = 0x8

	// TRCPDSR: Returns the following information about the trace unit: - OS Lock status. - Core power domain status. - Power interruption status
	// Position of OSLK field.
	PPB_TRCPDSR_OSLK_Pos = 0x5
	// Bit mask of OSLK field.
	PPB_TRCPDSR_OSLK_Msk = 0x20
	// Bit OSLK.
	PPB_TRCPDSR_OSLK = 0x20
	// Position of STICKYPD field.
	PPB_TRCPDSR_STICKYPD_Pos = 0x1
	// Bit mask of STICKYPD field.
	PPB_TRCPDSR_STICKYPD_Msk = 0x2
	// Bit STICKYPD.
	PPB_TRCPDSR_STICKYPD = 0x2
	// Position of POWER field.
	PPB_TRCPDSR_POWER_Pos = 0x0
	// Bit mask of POWER field.
	PPB_TRCPDSR_POWER_Msk = 0x1
	// Bit POWER.
	PPB_TRCPDSR_POWER = 0x1

	// TRCITATBIDR: Trace Integration ATB Identification Register
	// Position of ID field.
	PPB_TRCITATBIDR_ID_Pos = 0x0
	// Bit mask of ID field.
	PPB_TRCITATBIDR_ID_Msk = 0x7f

	// TRCITIATBINR: Trace Integration Instruction ATB In Register
	// Position of AFVALIDM field.
	PPB_TRCITIATBINR_AFVALIDM_Pos = 0x1
	// Bit mask of AFVALIDM field.
	PPB_TRCITIATBINR_AFVALIDM_Msk = 0x2
	// Bit AFVALIDM.
	PPB_TRCITIATBINR_AFVALIDM = 0x2
	// Position of ATREADYM field.
	PPB_TRCITIATBINR_ATREADYM_Pos = 0x0
	// Bit mask of ATREADYM field.
	PPB_TRCITIATBINR_ATREADYM_Msk = 0x1
	// Bit ATREADYM.
	PPB_TRCITIATBINR_ATREADYM = 0x1

	// TRCITIATBOUTR: Trace Integration Instruction ATB Out Register
	// Position of AFREADY field.
	PPB_TRCITIATBOUTR_AFREADY_Pos = 0x1
	// Bit mask of AFREADY field.
	PPB_TRCITIATBOUTR_AFREADY_Msk = 0x2
	// Bit AFREADY.
	PPB_TRCITIATBOUTR_AFREADY = 0x2
	// Position of ATVALID field.
	PPB_TRCITIATBOUTR_ATVALID_Pos = 0x0
	// Bit mask of ATVALID field.
	PPB_TRCITIATBOUTR_ATVALID_Msk = 0x1
	// Bit ATVALID.
	PPB_TRCITIATBOUTR_ATVALID = 0x1

	// TRCCLAIMSET: Claim Tag Set Register
	// Position of SET3 field.
	PPB_TRCCLAIMSET_SET3_Pos = 0x3
	// Bit mask of SET3 field.
	PPB_TRCCLAIMSET_SET3_Msk = 0x8
	// Bit SET3.
	PPB_TRCCLAIMSET_SET3 = 0x8
	// Position of SET2 field.
	PPB_TRCCLAIMSET_SET2_Pos = 0x2
	// Bit mask of SET2 field.
	PPB_TRCCLAIMSET_SET2_Msk = 0x4
	// Bit SET2.
	PPB_TRCCLAIMSET_SET2 = 0x4
	// Position of SET1 field.
	PPB_TRCCLAIMSET_SET1_Pos = 0x1
	// Bit mask of SET1 field.
	PPB_TRCCLAIMSET_SET1_Msk = 0x2
	// Bit SET1.
	PPB_TRCCLAIMSET_SET1 = 0x2
	// Position of SET0 field.
	PPB_TRCCLAIMSET_SET0_Pos = 0x0
	// Bit mask of SET0 field.
	PPB_TRCCLAIMSET_SET0_Msk = 0x1
	// Bit SET0.
	PPB_TRCCLAIMSET_SET0 = 0x1

	// TRCCLAIMCLR: Claim Tag Clear Register
	// Position of CLR3 field.
	PPB_TRCCLAIMCLR_CLR3_Pos = 0x3
	// Bit mask of CLR3 field.
	PPB_TRCCLAIMCLR_CLR3_Msk = 0x8
	// Bit CLR3.
	PPB_TRCCLAIMCLR_CLR3 = 0x8
	// Position of CLR2 field.
	PPB_TRCCLAIMCLR_CLR2_Pos = 0x2
	// Bit mask of CLR2 field.
	PPB_TRCCLAIMCLR_CLR2_Msk = 0x4
	// Bit CLR2.
	PPB_TRCCLAIMCLR_CLR2 = 0x4
	// Position of CLR1 field.
	PPB_TRCCLAIMCLR_CLR1_Pos = 0x1
	// Bit mask of CLR1 field.
	PPB_TRCCLAIMCLR_CLR1_Msk = 0x2
	// Bit CLR1.
	PPB_TRCCLAIMCLR_CLR1 = 0x2
	// Position of CLR0 field.
	PPB_TRCCLAIMCLR_CLR0_Pos = 0x0
	// Bit mask of CLR0 field.
	PPB_TRCCLAIMCLR_CLR0_Msk = 0x1
	// Bit CLR0.
	PPB_TRCCLAIMCLR_CLR0 = 0x1

	// TRCAUTHSTATUS: Returns the level of tracing that the trace unit can support
	// Position of SNID field.
	PPB_TRCAUTHSTATUS_SNID_Pos = 0x6
	// Bit mask of SNID field.
	PPB_TRCAUTHSTATUS_SNID_Msk = 0xc0
	// Position of SID field.
	PPB_TRCAUTHSTATUS_SID_Pos = 0x4
	// Bit mask of SID field.
	PPB_TRCAUTHSTATUS_SID_Msk = 0x30
	// Position of NSNID field.
	PPB_TRCAUTHSTATUS_NSNID_Pos = 0x2
	// Bit mask of NSNID field.
	PPB_TRCAUTHSTATUS_NSNID_Msk = 0xc
	// Position of NSID field.
	PPB_TRCAUTHSTATUS_NSID_Pos = 0x0
	// Bit mask of NSID field.
	PPB_TRCAUTHSTATUS_NSID_Msk = 0x3

	// TRCDEVARCH: TRCDEVARCH
	// Position of ARCHITECT field.
	PPB_TRCDEVARCH_ARCHITECT_Pos = 0x15
	// Bit mask of ARCHITECT field.
	PPB_TRCDEVARCH_ARCHITECT_Msk = 0xffe00000
	// Position of PRESENT field.
	PPB_TRCDEVARCH_PRESENT_Pos = 0x14
	// Bit mask of PRESENT field.
	PPB_TRCDEVARCH_PRESENT_Msk = 0x100000
	// Bit PRESENT.
	PPB_TRCDEVARCH_PRESENT = 0x100000
	// Position of REVISION field.
	PPB_TRCDEVARCH_REVISION_Pos = 0x10
	// Bit mask of REVISION field.
	PPB_TRCDEVARCH_REVISION_Msk = 0xf0000
	// Position of ARCHID field.
	PPB_TRCDEVARCH_ARCHID_Pos = 0x0
	// Bit mask of ARCHID field.
	PPB_TRCDEVARCH_ARCHID_Msk = 0xffff

	// TRCDEVID: TRCDEVID
	// Position of TRCDEVID field.
	PPB_TRCDEVID_TRCDEVID_Pos = 0x0
	// Bit mask of TRCDEVID field.
	PPB_TRCDEVID_TRCDEVID_Msk = 0xffffffff

	// TRCDEVTYPE: TRCDEVTYPE
	// Position of SUB field.
	PPB_TRCDEVTYPE_SUB_Pos = 0x4
	// Bit mask of SUB field.
	PPB_TRCDEVTYPE_SUB_Msk = 0xf0
	// Position of MAJOR field.
	PPB_TRCDEVTYPE_MAJOR_Pos = 0x0
	// Bit mask of MAJOR field.
	PPB_TRCDEVTYPE_MAJOR_Msk = 0xf

	// TRCPIDR4: TRCPIDR4
	// Position of SIZE field.
	PPB_TRCPIDR4_SIZE_Pos = 0x4
	// Bit mask of SIZE field.
	PPB_TRCPIDR4_SIZE_Msk = 0xf0
	// Position of DES_2 field.
	PPB_TRCPIDR4_DES_2_Pos = 0x0
	// Bit mask of DES_2 field.
	PPB_TRCPIDR4_DES_2_Msk = 0xf

	// TRCPIDR5: TRCPIDR5
	// Position of TRCPIDR5 field.
	PPB_TRCPIDR5_TRCPIDR5_Pos = 0x0
	// Bit mask of TRCPIDR5 field.
	PPB_TRCPIDR5_TRCPIDR5_Msk = 0xffffffff

	// TRCPIDR6: TRCPIDR6
	// Position of TRCPIDR6 field.
	PPB_TRCPIDR6_TRCPIDR6_Pos = 0x0
	// Bit mask of TRCPIDR6 field.
	PPB_TRCPIDR6_TRCPIDR6_Msk = 0xffffffff

	// TRCPIDR7: TRCPIDR7
	// Position of TRCPIDR7 field.
	PPB_TRCPIDR7_TRCPIDR7_Pos = 0x0
	// Bit mask of TRCPIDR7 field.
	PPB_TRCPIDR7_TRCPIDR7_Msk = 0xffffffff

	// TRCPIDR0: TRCPIDR0
	// Position of PART_0 field.
	PPB_TRCPIDR0_PART_0_Pos = 0x0
	// Bit mask of PART_0 field.
	PPB_TRCPIDR0_PART_0_Msk = 0xff

	// TRCPIDR1: TRCPIDR1
	// Position of DES_0 field.
	PPB_TRCPIDR1_DES_0_Pos = 0x4
	// Bit mask of DES_0 field.
	PPB_TRCPIDR1_DES_0_Msk = 0xf0
	// Position of PART_0 field.
	PPB_TRCPIDR1_PART_0_Pos = 0x0
	// Bit mask of PART_0 field.
	PPB_TRCPIDR1_PART_0_Msk = 0xf

	// TRCPIDR2: TRCPIDR2
	// Position of REVISION field.
	PPB_TRCPIDR2_REVISION_Pos = 0x4
	// Bit mask of REVISION field.
	PPB_TRCPIDR2_REVISION_Msk = 0xf0
	// Position of JEDEC field.
	PPB_TRCPIDR2_JEDEC_Pos = 0x3
	// Bit mask of JEDEC field.
	PPB_TRCPIDR2_JEDEC_Msk = 0x8
	// Bit JEDEC.
	PPB_TRCPIDR2_JEDEC = 0x8
	// Position of DES_0 field.
	PPB_TRCPIDR2_DES_0_Pos = 0x0
	// Bit mask of DES_0 field.
	PPB_TRCPIDR2_DES_0_Msk = 0x7

	// TRCPIDR3: TRCPIDR3
	// Position of REVAND field.
	PPB_TRCPIDR3_REVAND_Pos = 0x4
	// Bit mask of REVAND field.
	PPB_TRCPIDR3_REVAND_Msk = 0xf0
	// Position of CMOD field.
	PPB_TRCPIDR3_CMOD_Pos = 0x0
	// Bit mask of CMOD field.
	PPB_TRCPIDR3_CMOD_Msk = 0xf

	// TRCCIDR0: TRCCIDR0
	// Position of PRMBL_0 field.
	PPB_TRCCIDR0_PRMBL_0_Pos = 0x0
	// Bit mask of PRMBL_0 field.
	PPB_TRCCIDR0_PRMBL_0_Msk = 0xff

	// TRCCIDR1: TRCCIDR1
	// Position of CLASS field.
	PPB_TRCCIDR1_CLASS_Pos = 0x4
	// Bit mask of CLASS field.
	PPB_TRCCIDR1_CLASS_Msk = 0xf0
	// Position of PRMBL_1 field.
	PPB_TRCCIDR1_PRMBL_1_Pos = 0x0
	// Bit mask of PRMBL_1 field.
	PPB_TRCCIDR1_PRMBL_1_Msk = 0xf

	// TRCCIDR2: TRCCIDR2
	// Position of PRMBL_2 field.
	PPB_TRCCIDR2_PRMBL_2_Pos = 0x0
	// Bit mask of PRMBL_2 field.
	PPB_TRCCIDR2_PRMBL_2_Msk = 0xff

	// TRCCIDR3: TRCCIDR3
	// Position of PRMBL_3 field.
	PPB_TRCCIDR3_PRMBL_3_Pos = 0x0
	// Bit mask of PRMBL_3 field.
	PPB_TRCCIDR3_PRMBL_3_Msk = 0xff

	// CTICONTROL: CTI Control Register
	// Position of GLBEN field.
	PPB_CTICONTROL_GLBEN_Pos = 0x0
	// Bit mask of GLBEN field.
	PPB_CTICONTROL_GLBEN_Msk = 0x1
	// Bit GLBEN.
	PPB_CTICONTROL_GLBEN = 0x1

	// CTIINTACK: CTI Interrupt Acknowledge Register
	// Position of INTACK field.
	PPB_CTIINTACK_INTACK_Pos = 0x0
	// Bit mask of INTACK field.
	PPB_CTIINTACK_INTACK_Msk = 0xff

	// CTIAPPSET: CTI Application Trigger Set Register
	// Position of APPSET field.
	PPB_CTIAPPSET_APPSET_Pos = 0x0
	// Bit mask of APPSET field.
	PPB_CTIAPPSET_APPSET_Msk = 0xf

	// CTIAPPCLEAR: CTI Application Trigger Clear Register
	// Position of APPCLEAR field.
	PPB_CTIAPPCLEAR_APPCLEAR_Pos = 0x0
	// Bit mask of APPCLEAR field.
	PPB_CTIAPPCLEAR_APPCLEAR_Msk = 0xf

	// CTIAPPPULSE: CTI Application Pulse Register
	// Position of APPULSE field.
	PPB_CTIAPPPULSE_APPULSE_Pos = 0x0
	// Bit mask of APPULSE field.
	PPB_CTIAPPPULSE_APPULSE_Msk = 0xf

	// CTIINEN0: CTI Trigger to Channel Enable Registers
	// Position of TRIGINEN field.
	PPB_CTIINEN0_TRIGINEN_Pos = 0x0
	// Bit mask of TRIGINEN field.
	PPB_CTIINEN0_TRIGINEN_Msk = 0xf

	// CTIINEN1: CTI Trigger to Channel Enable Registers
	// Position of TRIGINEN field.
	PPB_CTIINEN1_TRIGINEN_Pos = 0x0
	// Bit mask of TRIGINEN field.
	PPB_CTIINEN1_TRIGINEN_Msk = 0xf

	// CTIINEN2: CTI Trigger to Channel Enable Registers
	// Position of TRIGINEN field.
	PPB_CTIINEN2_TRIGINEN_Pos = 0x0
	// Bit mask of TRIGINEN field.
	PPB_CTIINEN2_TRIGINEN_Msk = 0xf

	// CTIINEN3: CTI Trigger to Channel Enable Registers
	// Position of TRIGINEN field.
	PPB_CTIINEN3_TRIGINEN_Pos = 0x0
	// Bit mask of TRIGINEN field.
	PPB_CTIINEN3_TRIGINEN_Msk = 0xf

	// CTIINEN4: CTI Trigger to Channel Enable Registers
	// Position of TRIGINEN field.
	PPB_CTIINEN4_TRIGINEN_Pos = 0x0
	// Bit mask of TRIGINEN field.
	PPB_CTIINEN4_TRIGINEN_Msk = 0xf

	// CTIINEN5: CTI Trigger to Channel Enable Registers
	// Position of TRIGINEN field.
	PPB_CTIINEN5_TRIGINEN_Pos = 0x0
	// Bit mask of TRIGINEN field.
	PPB_CTIINEN5_TRIGINEN_Msk = 0xf

	// CTIINEN6: CTI Trigger to Channel Enable Registers
	// Position of TRIGINEN field.
	PPB_CTIINEN6_TRIGINEN_Pos = 0x0
	// Bit mask of TRIGINEN field.
	PPB_CTIINEN6_TRIGINEN_Msk = 0xf

	// CTIINEN7: CTI Trigger to Channel Enable Registers
	// Position of TRIGINEN field.
	PPB_CTIINEN7_TRIGINEN_Pos = 0x0
	// Bit mask of TRIGINEN field.
	PPB_CTIINEN7_TRIGINEN_Msk = 0xf

	// CTIOUTEN0: CTI Trigger to Channel Enable Registers
	// Position of TRIGOUTEN field.
	PPB_CTIOUTEN0_TRIGOUTEN_Pos = 0x0
	// Bit mask of TRIGOUTEN field.
	PPB_CTIOUTEN0_TRIGOUTEN_Msk = 0xf

	// CTIOUTEN1: CTI Trigger to Channel Enable Registers
	// Position of TRIGOUTEN field.
	PPB_CTIOUTEN1_TRIGOUTEN_Pos = 0x0
	// Bit mask of TRIGOUTEN field.
	PPB_CTIOUTEN1_TRIGOUTEN_Msk = 0xf

	// CTIOUTEN2: CTI Trigger to Channel Enable Registers
	// Position of TRIGOUTEN field.
	PPB_CTIOUTEN2_TRIGOUTEN_Pos = 0x0
	// Bit mask of TRIGOUTEN field.
	PPB_CTIOUTEN2_TRIGOUTEN_Msk = 0xf

	// CTIOUTEN3: CTI Trigger to Channel Enable Registers
	// Position of TRIGOUTEN field.
	PPB_CTIOUTEN3_TRIGOUTEN_Pos = 0x0
	// Bit mask of TRIGOUTEN field.
	PPB_CTIOUTEN3_TRIGOUTEN_Msk = 0xf

	// CTIOUTEN4: CTI Trigger to Channel Enable Registers
	// Position of TRIGOUTEN field.
	PPB_CTIOUTEN4_TRIGOUTEN_Pos = 0x0
	// Bit mask of TRIGOUTEN field.
	PPB_CTIOUTEN4_TRIGOUTEN_Msk = 0xf

	// CTIOUTEN5: CTI Trigger to Channel Enable Registers
	// Position of TRIGOUTEN field.
	PPB_CTIOUTEN5_TRIGOUTEN_Pos = 0x0
	// Bit mask of TRIGOUTEN field.
	PPB_CTIOUTEN5_TRIGOUTEN_Msk = 0xf

	// CTIOUTEN6: CTI Trigger to Channel Enable Registers
	// Position of TRIGOUTEN field.
	PPB_CTIOUTEN6_TRIGOUTEN_Pos = 0x0
	// Bit mask of TRIGOUTEN field.
	PPB_CTIOUTEN6_TRIGOUTEN_Msk = 0xf

	// CTIOUTEN7: CTI Trigger to Channel Enable Registers
	// Position of TRIGOUTEN field.
	PPB_CTIOUTEN7_TRIGOUTEN_Pos = 0x0
	// Bit mask of TRIGOUTEN field.
	PPB_CTIOUTEN7_TRIGOUTEN_Msk = 0xf

	// CTITRIGINSTATUS: CTI Trigger to Channel Enable Registers
	// Position of TRIGINSTATUS field.
	PPB_CTITRIGINSTATUS_TRIGINSTATUS_Pos = 0x0
	// Bit mask of TRIGINSTATUS field.
	PPB_CTITRIGINSTATUS_TRIGINSTATUS_Msk = 0xff

	// CTITRIGOUTSTATUS: CTI Trigger In Status Register
	// Position of TRIGOUTSTATUS field.
	PPB_CTITRIGOUTSTATUS_TRIGOUTSTATUS_Pos = 0x0
	// Bit mask of TRIGOUTSTATUS field.
	PPB_CTITRIGOUTSTATUS_TRIGOUTSTATUS_Msk = 0xff

	// CTICHINSTATUS: CTI Channel In Status Register
	// Position of CTICHOUTSTATUS field.
	PPB_CTICHINSTATUS_CTICHOUTSTATUS_Pos = 0x0
	// Bit mask of CTICHOUTSTATUS field.
	PPB_CTICHINSTATUS_CTICHOUTSTATUS_Msk = 0xf

	// CTIGATE: Enable CTI Channel Gate register
	// Position of CTIGATEEN3 field.
	PPB_CTIGATE_CTIGATEEN3_Pos = 0x3
	// Bit mask of CTIGATEEN3 field.
	PPB_CTIGATE_CTIGATEEN3_Msk = 0x8
	// Bit CTIGATEEN3.
	PPB_CTIGATE_CTIGATEEN3 = 0x8
	// Position of CTIGATEEN2 field.
	PPB_CTIGATE_CTIGATEEN2_Pos = 0x2
	// Bit mask of CTIGATEEN2 field.
	PPB_CTIGATE_CTIGATEEN2_Msk = 0x4
	// Bit CTIGATEEN2.
	PPB_CTIGATE_CTIGATEEN2 = 0x4
	// Position of CTIGATEEN1 field.
	PPB_CTIGATE_CTIGATEEN1_Pos = 0x1
	// Bit mask of CTIGATEEN1 field.
	PPB_CTIGATE_CTIGATEEN1_Msk = 0x2
	// Bit CTIGATEEN1.
	PPB_CTIGATE_CTIGATEEN1 = 0x2
	// Position of CTIGATEEN0 field.
	PPB_CTIGATE_CTIGATEEN0_Pos = 0x0
	// Bit mask of CTIGATEEN0 field.
	PPB_CTIGATE_CTIGATEEN0_Msk = 0x1
	// Bit CTIGATEEN0.
	PPB_CTIGATE_CTIGATEEN0 = 0x1

	// ASICCTL: External Multiplexer Control register
	// Position of ASICCTL field.
	PPB_ASICCTL_ASICCTL_Pos = 0x0
	// Bit mask of ASICCTL field.
	PPB_ASICCTL_ASICCTL_Msk = 0xffffffff

	// ITCHOUT: Integration Test Channel Output register
	// Position of CTCHOUT field.
	PPB_ITCHOUT_CTCHOUT_Pos = 0x0
	// Bit mask of CTCHOUT field.
	PPB_ITCHOUT_CTCHOUT_Msk = 0xf

	// ITTRIGOUT: Integration Test Trigger Output register
	// Position of CTTRIGOUT field.
	PPB_ITTRIGOUT_CTTRIGOUT_Pos = 0x0
	// Bit mask of CTTRIGOUT field.
	PPB_ITTRIGOUT_CTTRIGOUT_Msk = 0xff

	// ITCHIN: Integration Test Channel Input register
	// Position of CTCHIN field.
	PPB_ITCHIN_CTCHIN_Pos = 0x0
	// Bit mask of CTCHIN field.
	PPB_ITCHIN_CTCHIN_Msk = 0xf

	// ITCTRL: Integration Mode Control register
	// Position of IME field.
	PPB_ITCTRL_IME_Pos = 0x0
	// Bit mask of IME field.
	PPB_ITCTRL_IME_Msk = 0x1
	// Bit IME.
	PPB_ITCTRL_IME = 0x1

	// DEVARCH: Device Architecture register
	// Position of ARCHITECT field.
	PPB_DEVARCH_ARCHITECT_Pos = 0x15
	// Bit mask of ARCHITECT field.
	PPB_DEVARCH_ARCHITECT_Msk = 0xffe00000
	// Position of PRESENT field.
	PPB_DEVARCH_PRESENT_Pos = 0x14
	// Bit mask of PRESENT field.
	PPB_DEVARCH_PRESENT_Msk = 0x100000
	// Bit PRESENT.
	PPB_DEVARCH_PRESENT = 0x100000
	// Position of REVISION field.
	PPB_DEVARCH_REVISION_Pos = 0x10
	// Bit mask of REVISION field.
	PPB_DEVARCH_REVISION_Msk = 0xf0000
	// Position of ARCHID field.
	PPB_DEVARCH_ARCHID_Pos = 0x0
	// Bit mask of ARCHID field.
	PPB_DEVARCH_ARCHID_Msk = 0xffff

	// DEVID: Device Configuration register
	// Position of NUMCH field.
	PPB_DEVID_NUMCH_Pos = 0x10
	// Bit mask of NUMCH field.
	PPB_DEVID_NUMCH_Msk = 0xf0000
	// Position of NUMTRIG field.
	PPB_DEVID_NUMTRIG_Pos = 0x8
	// Bit mask of NUMTRIG field.
	PPB_DEVID_NUMTRIG_Msk = 0xff00
	// Position of EXTMUXNUM field.
	PPB_DEVID_EXTMUXNUM_Pos = 0x0
	// Bit mask of EXTMUXNUM field.
	PPB_DEVID_EXTMUXNUM_Msk = 0x1f

	// DEVTYPE: Device Type Identifier register
	// Position of SUB field.
	PPB_DEVTYPE_SUB_Pos = 0x4
	// Bit mask of SUB field.
	PPB_DEVTYPE_SUB_Msk = 0xf0
	// Position of MAJOR field.
	PPB_DEVTYPE_MAJOR_Pos = 0x0
	// Bit mask of MAJOR field.
	PPB_DEVTYPE_MAJOR_Msk = 0xf

	// PIDR4: CoreSight Peripheral ID4
	// Position of SIZE field.
	PPB_PIDR4_SIZE_Pos = 0x4
	// Bit mask of SIZE field.
	PPB_PIDR4_SIZE_Msk = 0xf0
	// Position of DES_2 field.
	PPB_PIDR4_DES_2_Pos = 0x0
	// Bit mask of DES_2 field.
	PPB_PIDR4_DES_2_Msk = 0xf

	// PIDR5: CoreSight Peripheral ID5
	// Position of PIDR5 field.
	PPB_PIDR5_PIDR5_Pos = 0x0
	// Bit mask of PIDR5 field.
	PPB_PIDR5_PIDR5_Msk = 0xffffffff

	// PIDR6: CoreSight Peripheral ID6
	// Position of PIDR6 field.
	PPB_PIDR6_PIDR6_Pos = 0x0
	// Bit mask of PIDR6 field.
	PPB_PIDR6_PIDR6_Msk = 0xffffffff

	// PIDR7: CoreSight Peripheral ID7
	// Position of PIDR7 field.
	PPB_PIDR7_PIDR7_Pos = 0x0
	// Bit mask of PIDR7 field.
	PPB_PIDR7_PIDR7_Msk = 0xffffffff

	// PIDR0: CoreSight Peripheral ID0
	// Position of PART_0 field.
	PPB_PIDR0_PART_0_Pos = 0x0
	// Bit mask of PART_0 field.
	PPB_PIDR0_PART_0_Msk = 0xff

	// PIDR1: CoreSight Peripheral ID1
	// Position of DES_0 field.
	PPB_PIDR1_DES_0_Pos = 0x4
	// Bit mask of DES_0 field.
	PPB_PIDR1_DES_0_Msk = 0xf0
	// Position of PART_1 field.
	PPB_PIDR1_PART_1_Pos = 0x0
	// Bit mask of PART_1 field.
	PPB_PIDR1_PART_1_Msk = 0xf

	// PIDR2: CoreSight Peripheral ID2
	// Position of REVISION field.
	PPB_PIDR2_REVISION_Pos = 0x4
	// Bit mask of REVISION field.
	PPB_PIDR2_REVISION_Msk = 0xf0
	// Position of JEDEC field.
	PPB_PIDR2_JEDEC_Pos = 0x3
	// Bit mask of JEDEC field.
	PPB_PIDR2_JEDEC_Msk = 0x8
	// Bit JEDEC.
	PPB_PIDR2_JEDEC = 0x8
	// Position of DES_1 field.
	PPB_PIDR2_DES_1_Pos = 0x0
	// Bit mask of DES_1 field.
	PPB_PIDR2_DES_1_Msk = 0x7

	// PIDR3: CoreSight Peripheral ID3
	// Position of REVAND field.
	PPB_PIDR3_REVAND_Pos = 0x4
	// Bit mask of REVAND field.
	PPB_PIDR3_REVAND_Msk = 0xf0
	// Position of CMOD field.
	PPB_PIDR3_CMOD_Pos = 0x0
	// Bit mask of CMOD field.
	PPB_PIDR3_CMOD_Msk = 0xf

	// CIDR0: CoreSight Component ID0
	// Position of PRMBL_0 field.
	PPB_CIDR0_PRMBL_0_Pos = 0x0
	// Bit mask of PRMBL_0 field.
	PPB_CIDR0_PRMBL_0_Msk = 0xff

	// CIDR1: CoreSight Component ID1
	// Position of CLASS field.
	PPB_CIDR1_CLASS_Pos = 0x4
	// Bit mask of CLASS field.
	PPB_CIDR1_CLASS_Msk = 0xf0
	// Position of PRMBL_1 field.
	PPB_CIDR1_PRMBL_1_Pos = 0x0
	// Bit mask of PRMBL_1 field.
	PPB_CIDR1_PRMBL_1_Msk = 0xf

	// CIDR2: CoreSight Component ID2
	// Position of PRMBL_2 field.
	PPB_CIDR2_PRMBL_2_Pos = 0x0
	// Bit mask of PRMBL_2 field.
	PPB_CIDR2_PRMBL_2_Msk = 0xff

	// CIDR3: CoreSight Component ID3
	// Position of PRMBL_3 field.
	PPB_CIDR3_PRMBL_3_Pos = 0x0
	// Bit mask of PRMBL_3 field.
	PPB_CIDR3_PRMBL_3_Msk = 0xff
)

// Constants for QMI: QSPI Memory Interface. Provides a memory-mapped interface to up to two SPI/DSPI/QSPI flash or PSRAM devices. Also provides a serial interface for programming and configuration of the external device.
const (
	// DIRECT_CSR: Control and status for direct serial mode Direct serial mode allows the processor to send and receive raw serial frames, for programming, configuration and control of the external memory devices. Only SPI mode 0 (CPOL=0 CPHA=0) is supported.
	// Position of RXDELAY field.
	QMI_DIRECT_CSR_RXDELAY_Pos = 0x1e
	// Bit mask of RXDELAY field.
	QMI_DIRECT_CSR_RXDELAY_Msk = 0xc0000000
	// Position of CLKDIV field.
	QMI_DIRECT_CSR_CLKDIV_Pos = 0x16
	// Bit mask of CLKDIV field.
	QMI_DIRECT_CSR_CLKDIV_Msk = 0x3fc00000
	// Position of RXLEVEL field.
	QMI_DIRECT_CSR_RXLEVEL_Pos = 0x12
	// Bit mask of RXLEVEL field.
	QMI_DIRECT_CSR_RXLEVEL_Msk = 0x1c0000
	// Position of RXFULL field.
	QMI_DIRECT_CSR_RXFULL_Pos = 0x11
	// Bit mask of RXFULL field.
	QMI_DIRECT_CSR_RXFULL_Msk = 0x20000
	// Bit RXFULL.
	QMI_DIRECT_CSR_RXFULL = 0x20000
	// Position of RXEMPTY field.
	QMI_DIRECT_CSR_RXEMPTY_Pos = 0x10
	// Bit mask of RXEMPTY field.
	QMI_DIRECT_CSR_RXEMPTY_Msk = 0x10000
	// Bit RXEMPTY.
	QMI_DIRECT_CSR_RXEMPTY = 0x10000
	// Position of TXLEVEL field.
	QMI_DIRECT_CSR_TXLEVEL_Pos = 0xc
	// Bit mask of TXLEVEL field.
	QMI_DIRECT_CSR_TXLEVEL_Msk = 0x7000
	// Position of TXEMPTY field.
	QMI_DIRECT_CSR_TXEMPTY_Pos = 0xb
	// Bit mask of TXEMPTY field.
	QMI_DIRECT_CSR_TXEMPTY_Msk = 0x800
	// Bit TXEMPTY.
	QMI_DIRECT_CSR_TXEMPTY = 0x800
	// Position of TXFULL field.
	QMI_DIRECT_CSR_TXFULL_Pos = 0xa
	// Bit mask of TXFULL field.
	QMI_DIRECT_CSR_TXFULL_Msk = 0x400
	// Bit TXFULL.
	QMI_DIRECT_CSR_TXFULL = 0x400
	// Position of AUTO_CS1N field.
	QMI_DIRECT_CSR_AUTO_CS1N_Pos = 0x7
	// Bit mask of AUTO_CS1N field.
	QMI_DIRECT_CSR_AUTO_CS1N_Msk = 0x80
	// Bit AUTO_CS1N.
	QMI_DIRECT_CSR_AUTO_CS1N = 0x80
	// Position of AUTO_CS0N field.
	QMI_DIRECT_CSR_AUTO_CS0N_Pos = 0x6
	// Bit mask of AUTO_CS0N field.
	QMI_DIRECT_CSR_AUTO_CS0N_Msk = 0x40
	// Bit AUTO_CS0N.
	QMI_DIRECT_CSR_AUTO_CS0N = 0x40
	// Position of ASSERT_CS1N field.
	QMI_DIRECT_CSR_ASSERT_CS1N_Pos = 0x3
	// Bit mask of ASSERT_CS1N field.
	QMI_DIRECT_CSR_ASSERT_CS1N_Msk = 0x8
	// Bit ASSERT_CS1N.
	QMI_DIRECT_CSR_ASSERT_CS1N = 0x8
	// Position of ASSERT_CS0N field.
	QMI_DIRECT_CSR_ASSERT_CS0N_Pos = 0x2
	// Bit mask of ASSERT_CS0N field.
	QMI_DIRECT_CSR_ASSERT_CS0N_Msk = 0x4
	// Bit ASSERT_CS0N.
	QMI_DIRECT_CSR_ASSERT_CS0N = 0x4
	// Position of BUSY field.
	QMI_DIRECT_CSR_BUSY_Pos = 0x1
	// Bit mask of BUSY field.
	QMI_DIRECT_CSR_BUSY_Msk = 0x2
	// Bit BUSY.
	QMI_DIRECT_CSR_BUSY = 0x2
	// Position of EN field.
	QMI_DIRECT_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	QMI_DIRECT_CSR_EN_Msk = 0x1
	// Bit EN.
	QMI_DIRECT_CSR_EN = 0x1

	// DIRECT_TX: Transmit FIFO for direct mode
	// Position of NOPUSH field.
	QMI_DIRECT_TX_NOPUSH_Pos = 0x14
	// Bit mask of NOPUSH field.
	QMI_DIRECT_TX_NOPUSH_Msk = 0x100000
	// Bit NOPUSH.
	QMI_DIRECT_TX_NOPUSH = 0x100000
	// Position of OE field.
	QMI_DIRECT_TX_OE_Pos = 0x13
	// Bit mask of OE field.
	QMI_DIRECT_TX_OE_Msk = 0x80000
	// Bit OE.
	QMI_DIRECT_TX_OE = 0x80000
	// Position of DWIDTH field.
	QMI_DIRECT_TX_DWIDTH_Pos = 0x12
	// Bit mask of DWIDTH field.
	QMI_DIRECT_TX_DWIDTH_Msk = 0x40000
	// Bit DWIDTH.
	QMI_DIRECT_TX_DWIDTH = 0x40000
	// Position of IWIDTH field.
	QMI_DIRECT_TX_IWIDTH_Pos = 0x10
	// Bit mask of IWIDTH field.
	QMI_DIRECT_TX_IWIDTH_Msk = 0x30000
	// Single width
	QMI_DIRECT_TX_IWIDTH_S = 0x0
	// Dual width
	QMI_DIRECT_TX_IWIDTH_D = 0x1
	// Quad width
	QMI_DIRECT_TX_IWIDTH_Q = 0x2
	// Position of DATA field.
	QMI_DIRECT_TX_DATA_Pos = 0x0
	// Bit mask of DATA field.
	QMI_DIRECT_TX_DATA_Msk = 0xffff

	// DIRECT_RX: Receive FIFO for direct mode
	// Position of DIRECT_RX field.
	QMI_DIRECT_RX_DIRECT_RX_Pos = 0x0
	// Bit mask of DIRECT_RX field.
	QMI_DIRECT_RX_DIRECT_RX_Msk = 0xffff

	// M0_TIMING: Timing configuration register for memory address window 0.
	// Position of COOLDOWN field.
	QMI_M0_TIMING_COOLDOWN_Pos = 0x1e
	// Bit mask of COOLDOWN field.
	QMI_M0_TIMING_COOLDOWN_Msk = 0xc0000000
	// Position of PAGEBREAK field.
	QMI_M0_TIMING_PAGEBREAK_Pos = 0x1c
	// Bit mask of PAGEBREAK field.
	QMI_M0_TIMING_PAGEBREAK_Msk = 0x30000000
	// No page boundary is enforced
	QMI_M0_TIMING_PAGEBREAK_NONE = 0x0
	// Break bursts crossing a 256-byte page boundary
	QMI_M0_TIMING_PAGEBREAK_256 = 0x1
	// Break bursts crossing a 1024-byte quad-page boundary
	QMI_M0_TIMING_PAGEBREAK_1024 = 0x2
	// Break bursts crossing a 4096-byte sector boundary
	QMI_M0_TIMING_PAGEBREAK_4096 = 0x3
	// Position of SELECT_SETUP field.
	QMI_M0_TIMING_SELECT_SETUP_Pos = 0x19
	// Bit mask of SELECT_SETUP field.
	QMI_M0_TIMING_SELECT_SETUP_Msk = 0x2000000
	// Bit SELECT_SETUP.
	QMI_M0_TIMING_SELECT_SETUP = 0x2000000
	// Position of SELECT_HOLD field.
	QMI_M0_TIMING_SELECT_HOLD_Pos = 0x17
	// Bit mask of SELECT_HOLD field.
	QMI_M0_TIMING_SELECT_HOLD_Msk = 0x1800000
	// Position of MAX_SELECT field.
	QMI_M0_TIMING_MAX_SELECT_Pos = 0x11
	// Bit mask of MAX_SELECT field.
	QMI_M0_TIMING_MAX_SELECT_Msk = 0x7e0000
	// Position of MIN_DESELECT field.
	QMI_M0_TIMING_MIN_DESELECT_Pos = 0xc
	// Bit mask of MIN_DESELECT field.
	QMI_M0_TIMING_MIN_DESELECT_Msk = 0x1f000
	// Position of RXDELAY field.
	QMI_M0_TIMING_RXDELAY_Pos = 0x8
	// Bit mask of RXDELAY field.
	QMI_M0_TIMING_RXDELAY_Msk = 0x700
	// Position of CLKDIV field.
	QMI_M0_TIMING_CLKDIV_Pos = 0x0
	// Bit mask of CLKDIV field.
	QMI_M0_TIMING_CLKDIV_Msk = 0xff

	// M0_RFMT: Read transfer format configuration for memory address window 0. Configure the bus width of each transfer phase individually, and configure the length or presence of the command prefix, command suffix and dummy/turnaround transfer phases. Only 24-bit addresses are supported. The reset value of the M0_RFMT register is configured to support a basic 03h serial read transfer with no additional configuration.
	// Position of DTR field.
	QMI_M0_RFMT_DTR_Pos = 0x1c
	// Bit mask of DTR field.
	QMI_M0_RFMT_DTR_Msk = 0x10000000
	// Bit DTR.
	QMI_M0_RFMT_DTR = 0x10000000
	// Position of DUMMY_LEN field.
	QMI_M0_RFMT_DUMMY_LEN_Pos = 0x10
	// Bit mask of DUMMY_LEN field.
	QMI_M0_RFMT_DUMMY_LEN_Msk = 0x70000
	// No dummy phase
	QMI_M0_RFMT_DUMMY_LEN_NONE = 0x0
	// 4 dummy bits
	QMI_M0_RFMT_DUMMY_LEN_4 = 0x1
	// 8 dummy bits
	QMI_M0_RFMT_DUMMY_LEN_8 = 0x2
	// 12 dummy bits
	QMI_M0_RFMT_DUMMY_LEN_12 = 0x3
	// 16 dummy bits
	QMI_M0_RFMT_DUMMY_LEN_16 = 0x4
	// 20 dummy bits
	QMI_M0_RFMT_DUMMY_LEN_20 = 0x5
	// 24 dummy bits
	QMI_M0_RFMT_DUMMY_LEN_24 = 0x6
	// 28 dummy bits
	QMI_M0_RFMT_DUMMY_LEN_28 = 0x7
	// Position of SUFFIX_LEN field.
	QMI_M0_RFMT_SUFFIX_LEN_Pos = 0xe
	// Bit mask of SUFFIX_LEN field.
	QMI_M0_RFMT_SUFFIX_LEN_Msk = 0xc000
	// No suffix
	QMI_M0_RFMT_SUFFIX_LEN_NONE = 0x0
	// 8-bit suffix
	QMI_M0_RFMT_SUFFIX_LEN_8 = 0x2
	// Position of PREFIX_LEN field.
	QMI_M0_RFMT_PREFIX_LEN_Pos = 0xc
	// Bit mask of PREFIX_LEN field.
	QMI_M0_RFMT_PREFIX_LEN_Msk = 0x1000
	// Bit PREFIX_LEN.
	QMI_M0_RFMT_PREFIX_LEN = 0x1000
	// No prefix
	QMI_M0_RFMT_PREFIX_LEN_NONE = 0x0
	// 8-bit prefix
	QMI_M0_RFMT_PREFIX_LEN_8 = 0x1
	// Position of DATA_WIDTH field.
	QMI_M0_RFMT_DATA_WIDTH_Pos = 0x8
	// Bit mask of DATA_WIDTH field.
	QMI_M0_RFMT_DATA_WIDTH_Msk = 0x300
	// Single width
	QMI_M0_RFMT_DATA_WIDTH_S = 0x0
	// Dual width
	QMI_M0_RFMT_DATA_WIDTH_D = 0x1
	// Quad width
	QMI_M0_RFMT_DATA_WIDTH_Q = 0x2
	// Position of DUMMY_WIDTH field.
	QMI_M0_RFMT_DUMMY_WIDTH_Pos = 0x6
	// Bit mask of DUMMY_WIDTH field.
	QMI_M0_RFMT_DUMMY_WIDTH_Msk = 0xc0
	// Single width
	QMI_M0_RFMT_DUMMY_WIDTH_S = 0x0
	// Dual width
	QMI_M0_RFMT_DUMMY_WIDTH_D = 0x1
	// Quad width
	QMI_M0_RFMT_DUMMY_WIDTH_Q = 0x2
	// Position of SUFFIX_WIDTH field.
	QMI_M0_RFMT_SUFFIX_WIDTH_Pos = 0x4
	// Bit mask of SUFFIX_WIDTH field.
	QMI_M0_RFMT_SUFFIX_WIDTH_Msk = 0x30
	// Single width
	QMI_M0_RFMT_SUFFIX_WIDTH_S = 0x0
	// Dual width
	QMI_M0_RFMT_SUFFIX_WIDTH_D = 0x1
	// Quad width
	QMI_M0_RFMT_SUFFIX_WIDTH_Q = 0x2
	// Position of ADDR_WIDTH field.
	QMI_M0_RFMT_ADDR_WIDTH_Pos = 0x2
	// Bit mask of ADDR_WIDTH field.
	QMI_M0_RFMT_ADDR_WIDTH_Msk = 0xc
	// Single width
	QMI_M0_RFMT_ADDR_WIDTH_S = 0x0
	// Dual width
	QMI_M0_RFMT_ADDR_WIDTH_D = 0x1
	// Quad width
	QMI_M0_RFMT_ADDR_WIDTH_Q = 0x2
	// Position of PREFIX_WIDTH field.
	QMI_M0_RFMT_PREFIX_WIDTH_Pos = 0x0
	// Bit mask of PREFIX_WIDTH field.
	QMI_M0_RFMT_PREFIX_WIDTH_Msk = 0x3
	// Single width
	QMI_M0_RFMT_PREFIX_WIDTH_S = 0x0
	// Dual width
	QMI_M0_RFMT_PREFIX_WIDTH_D = 0x1
	// Quad width
	QMI_M0_RFMT_PREFIX_WIDTH_Q = 0x2

	// M0_RCMD: Command constants used for reads from memory address window 0. The reset value of the M0_RCMD register is configured to support a basic 03h serial read transfer with no additional configuration.
	// Position of SUFFIX field.
	QMI_M0_RCMD_SUFFIX_Pos = 0x8
	// Bit mask of SUFFIX field.
	QMI_M0_RCMD_SUFFIX_Msk = 0xff00
	// Position of PREFIX field.
	QMI_M0_RCMD_PREFIX_Pos = 0x0
	// Bit mask of PREFIX field.
	QMI_M0_RCMD_PREFIX_Msk = 0xff

	// M0_WFMT: Write transfer format configuration for memory address window 0. Configure the bus width of each transfer phase individually, and configure the length or presence of the command prefix, command suffix and dummy/turnaround transfer phases. Only 24-bit addresses are supported. The reset value of the M0_WFMT register is configured to support a basic 02h serial write transfer. However, writes to this window must first be enabled via the XIP_CTRL_WRITABLE_M0 bit, as XIP memory is read-only by default.
	// Position of DTR field.
	QMI_M0_WFMT_DTR_Pos = 0x1c
	// Bit mask of DTR field.
	QMI_M0_WFMT_DTR_Msk = 0x10000000
	// Bit DTR.
	QMI_M0_WFMT_DTR = 0x10000000
	// Position of DUMMY_LEN field.
	QMI_M0_WFMT_DUMMY_LEN_Pos = 0x10
	// Bit mask of DUMMY_LEN field.
	QMI_M0_WFMT_DUMMY_LEN_Msk = 0x70000
	// No dummy phase
	QMI_M0_WFMT_DUMMY_LEN_NONE = 0x0
	// 4 dummy bits
	QMI_M0_WFMT_DUMMY_LEN_4 = 0x1
	// 8 dummy bits
	QMI_M0_WFMT_DUMMY_LEN_8 = 0x2
	// 12 dummy bits
	QMI_M0_WFMT_DUMMY_LEN_12 = 0x3
	// 16 dummy bits
	QMI_M0_WFMT_DUMMY_LEN_16 = 0x4
	// 20 dummy bits
	QMI_M0_WFMT_DUMMY_LEN_20 = 0x5
	// 24 dummy bits
	QMI_M0_WFMT_DUMMY_LEN_24 = 0x6
	// 28 dummy bits
	QMI_M0_WFMT_DUMMY_LEN_28 = 0x7
	// Position of SUFFIX_LEN field.
	QMI_M0_WFMT_SUFFIX_LEN_Pos = 0xe
	// Bit mask of SUFFIX_LEN field.
	QMI_M0_WFMT_SUFFIX_LEN_Msk = 0xc000
	// No suffix
	QMI_M0_WFMT_SUFFIX_LEN_NONE = 0x0
	// 8-bit suffix
	QMI_M0_WFMT_SUFFIX_LEN_8 = 0x2
	// Position of PREFIX_LEN field.
	QMI_M0_WFMT_PREFIX_LEN_Pos = 0xc
	// Bit mask of PREFIX_LEN field.
	QMI_M0_WFMT_PREFIX_LEN_Msk = 0x1000
	// Bit PREFIX_LEN.
	QMI_M0_WFMT_PREFIX_LEN = 0x1000
	// No prefix
	QMI_M0_WFMT_PREFIX_LEN_NONE = 0x0
	// 8-bit prefix
	QMI_M0_WFMT_PREFIX_LEN_8 = 0x1
	// Position of DATA_WIDTH field.
	QMI_M0_WFMT_DATA_WIDTH_Pos = 0x8
	// Bit mask of DATA_WIDTH field.
	QMI_M0_WFMT_DATA_WIDTH_Msk = 0x300
	// Single width
	QMI_M0_WFMT_DATA_WIDTH_S = 0x0
	// Dual width
	QMI_M0_WFMT_DATA_WIDTH_D = 0x1
	// Quad width
	QMI_M0_WFMT_DATA_WIDTH_Q = 0x2
	// Position of DUMMY_WIDTH field.
	QMI_M0_WFMT_DUMMY_WIDTH_Pos = 0x6
	// Bit mask of DUMMY_WIDTH field.
	QMI_M0_WFMT_DUMMY_WIDTH_Msk = 0xc0
	// Single width
	QMI_M0_WFMT_DUMMY_WIDTH_S = 0x0
	// Dual width
	QMI_M0_WFMT_DUMMY_WIDTH_D = 0x1
	// Quad width
	QMI_M0_WFMT_DUMMY_WIDTH_Q = 0x2
	// Position of SUFFIX_WIDTH field.
	QMI_M0_WFMT_SUFFIX_WIDTH_Pos = 0x4
	// Bit mask of SUFFIX_WIDTH field.
	QMI_M0_WFMT_SUFFIX_WIDTH_Msk = 0x30
	// Single width
	QMI_M0_WFMT_SUFFIX_WIDTH_S = 0x0
	// Dual width
	QMI_M0_WFMT_SUFFIX_WIDTH_D = 0x1
	// Quad width
	QMI_M0_WFMT_SUFFIX_WIDTH_Q = 0x2
	// Position of ADDR_WIDTH field.
	QMI_M0_WFMT_ADDR_WIDTH_Pos = 0x2
	// Bit mask of ADDR_WIDTH field.
	QMI_M0_WFMT_ADDR_WIDTH_Msk = 0xc
	// Single width
	QMI_M0_WFMT_ADDR_WIDTH_S = 0x0
	// Dual width
	QMI_M0_WFMT_ADDR_WIDTH_D = 0x1
	// Quad width
	QMI_M0_WFMT_ADDR_WIDTH_Q = 0x2
	// Position of PREFIX_WIDTH field.
	QMI_M0_WFMT_PREFIX_WIDTH_Pos = 0x0
	// Bit mask of PREFIX_WIDTH field.
	QMI_M0_WFMT_PREFIX_WIDTH_Msk = 0x3
	// Single width
	QMI_M0_WFMT_PREFIX_WIDTH_S = 0x0
	// Dual width
	QMI_M0_WFMT_PREFIX_WIDTH_D = 0x1
	// Quad width
	QMI_M0_WFMT_PREFIX_WIDTH_Q = 0x2

	// M0_WCMD: Command constants used for writes to memory address window 0. The reset value of the M0_WCMD register is configured to support a basic 02h serial write transfer with no additional configuration.
	// Position of SUFFIX field.
	QMI_M0_WCMD_SUFFIX_Pos = 0x8
	// Bit mask of SUFFIX field.
	QMI_M0_WCMD_SUFFIX_Msk = 0xff00
	// Position of PREFIX field.
	QMI_M0_WCMD_PREFIX_Pos = 0x0
	// Bit mask of PREFIX field.
	QMI_M0_WCMD_PREFIX_Msk = 0xff

	// M1_TIMING: Timing configuration register for memory address window 1.
	// Position of COOLDOWN field.
	QMI_M1_TIMING_COOLDOWN_Pos = 0x1e
	// Bit mask of COOLDOWN field.
	QMI_M1_TIMING_COOLDOWN_Msk = 0xc0000000
	// Position of PAGEBREAK field.
	QMI_M1_TIMING_PAGEBREAK_Pos = 0x1c
	// Bit mask of PAGEBREAK field.
	QMI_M1_TIMING_PAGEBREAK_Msk = 0x30000000
	// No page boundary is enforced
	QMI_M1_TIMING_PAGEBREAK_NONE = 0x0
	// Break bursts crossing a 256-byte page boundary
	QMI_M1_TIMING_PAGEBREAK_256 = 0x1
	// Break bursts crossing a 1024-byte quad-page boundary
	QMI_M1_TIMING_PAGEBREAK_1024 = 0x2
	// Break bursts crossing a 4096-byte sector boundary
	QMI_M1_TIMING_PAGEBREAK_4096 = 0x3
	// Position of SELECT_SETUP field.
	QMI_M1_TIMING_SELECT_SETUP_Pos = 0x19
	// Bit mask of SELECT_SETUP field.
	QMI_M1_TIMING_SELECT_SETUP_Msk = 0x2000000
	// Bit SELECT_SETUP.
	QMI_M1_TIMING_SELECT_SETUP = 0x2000000
	// Position of SELECT_HOLD field.
	QMI_M1_TIMING_SELECT_HOLD_Pos = 0x17
	// Bit mask of SELECT_HOLD field.
	QMI_M1_TIMING_SELECT_HOLD_Msk = 0x1800000
	// Position of MAX_SELECT field.
	QMI_M1_TIMING_MAX_SELECT_Pos = 0x11
	// Bit mask of MAX_SELECT field.
	QMI_M1_TIMING_MAX_SELECT_Msk = 0x7e0000
	// Position of MIN_DESELECT field.
	QMI_M1_TIMING_MIN_DESELECT_Pos = 0xc
	// Bit mask of MIN_DESELECT field.
	QMI_M1_TIMING_MIN_DESELECT_Msk = 0x1f000
	// Position of RXDELAY field.
	QMI_M1_TIMING_RXDELAY_Pos = 0x8
	// Bit mask of RXDELAY field.
	QMI_M1_TIMING_RXDELAY_Msk = 0x700
	// Position of CLKDIV field.
	QMI_M1_TIMING_CLKDIV_Pos = 0x0
	// Bit mask of CLKDIV field.
	QMI_M1_TIMING_CLKDIV_Msk = 0xff

	// M1_RFMT: Read transfer format configuration for memory address window 1. Configure the bus width of each transfer phase individually, and configure the length or presence of the command prefix, command suffix and dummy/turnaround transfer phases. Only 24-bit addresses are supported. The reset value of the M1_RFMT register is configured to support a basic 03h serial read transfer with no additional configuration.
	// Position of DTR field.
	QMI_M1_RFMT_DTR_Pos = 0x1c
	// Bit mask of DTR field.
	QMI_M1_RFMT_DTR_Msk = 0x10000000
	// Bit DTR.
	QMI_M1_RFMT_DTR = 0x10000000
	// Position of DUMMY_LEN field.
	QMI_M1_RFMT_DUMMY_LEN_Pos = 0x10
	// Bit mask of DUMMY_LEN field.
	QMI_M1_RFMT_DUMMY_LEN_Msk = 0x70000
	// No dummy phase
	QMI_M1_RFMT_DUMMY_LEN_NONE = 0x0
	// 4 dummy bits
	QMI_M1_RFMT_DUMMY_LEN_4 = 0x1
	// 8 dummy bits
	QMI_M1_RFMT_DUMMY_LEN_8 = 0x2
	// 12 dummy bits
	QMI_M1_RFMT_DUMMY_LEN_12 = 0x3
	// 16 dummy bits
	QMI_M1_RFMT_DUMMY_LEN_16 = 0x4
	// 20 dummy bits
	QMI_M1_RFMT_DUMMY_LEN_20 = 0x5
	// 24 dummy bits
	QMI_M1_RFMT_DUMMY_LEN_24 = 0x6
	// 28 dummy bits
	QMI_M1_RFMT_DUMMY_LEN_28 = 0x7
	// Position of SUFFIX_LEN field.
	QMI_M1_RFMT_SUFFIX_LEN_Pos = 0xe
	// Bit mask of SUFFIX_LEN field.
	QMI_M1_RFMT_SUFFIX_LEN_Msk = 0xc000
	// No suffix
	QMI_M1_RFMT_SUFFIX_LEN_NONE = 0x0
	// 8-bit suffix
	QMI_M1_RFMT_SUFFIX_LEN_8 = 0x2
	// Position of PREFIX_LEN field.
	QMI_M1_RFMT_PREFIX_LEN_Pos = 0xc
	// Bit mask of PREFIX_LEN field.
	QMI_M1_RFMT_PREFIX_LEN_Msk = 0x1000
	// Bit PREFIX_LEN.
	QMI_M1_RFMT_PREFIX_LEN = 0x1000
	// No prefix
	QMI_M1_RFMT_PREFIX_LEN_NONE = 0x0
	// 8-bit prefix
	QMI_M1_RFMT_PREFIX_LEN_8 = 0x1
	// Position of DATA_WIDTH field.
	QMI_M1_RFMT_DATA_WIDTH_Pos = 0x8
	// Bit mask of DATA_WIDTH field.
	QMI_M1_RFMT_DATA_WIDTH_Msk = 0x300
	// Single width
	QMI_M1_RFMT_DATA_WIDTH_S = 0x0
	// Dual width
	QMI_M1_RFMT_DATA_WIDTH_D = 0x1
	// Quad width
	QMI_M1_RFMT_DATA_WIDTH_Q = 0x2
	// Position of DUMMY_WIDTH field.
	QMI_M1_RFMT_DUMMY_WIDTH_Pos = 0x6
	// Bit mask of DUMMY_WIDTH field.
	QMI_M1_RFMT_DUMMY_WIDTH_Msk = 0xc0
	// Single width
	QMI_M1_RFMT_DUMMY_WIDTH_S = 0x0
	// Dual width
	QMI_M1_RFMT_DUMMY_WIDTH_D = 0x1
	// Quad width
	QMI_M1_RFMT_DUMMY_WIDTH_Q = 0x2
	// Position of SUFFIX_WIDTH field.
	QMI_M1_RFMT_SUFFIX_WIDTH_Pos = 0x4
	// Bit mask of SUFFIX_WIDTH field.
	QMI_M1_RFMT_SUFFIX_WIDTH_Msk = 0x30
	// Single width
	QMI_M1_RFMT_SUFFIX_WIDTH_S = 0x0
	// Dual width
	QMI_M1_RFMT_SUFFIX_WIDTH_D = 0x1
	// Quad width
	QMI_M1_RFMT_SUFFIX_WIDTH_Q = 0x2
	// Position of ADDR_WIDTH field.
	QMI_M1_RFMT_ADDR_WIDTH_Pos = 0x2
	// Bit mask of ADDR_WIDTH field.
	QMI_M1_RFMT_ADDR_WIDTH_Msk = 0xc
	// Single width
	QMI_M1_RFMT_ADDR_WIDTH_S = 0x0
	// Dual width
	QMI_M1_RFMT_ADDR_WIDTH_D = 0x1
	// Quad width
	QMI_M1_RFMT_ADDR_WIDTH_Q = 0x2
	// Position of PREFIX_WIDTH field.
	QMI_M1_RFMT_PREFIX_WIDTH_Pos = 0x0
	// Bit mask of PREFIX_WIDTH field.
	QMI_M1_RFMT_PREFIX_WIDTH_Msk = 0x3
	// Single width
	QMI_M1_RFMT_PREFIX_WIDTH_S = 0x0
	// Dual width
	QMI_M1_RFMT_PREFIX_WIDTH_D = 0x1
	// Quad width
	QMI_M1_RFMT_PREFIX_WIDTH_Q = 0x2

	// M1_RCMD: Command constants used for reads from memory address window 1. The reset value of the M1_RCMD register is configured to support a basic 03h serial read transfer with no additional configuration.
	// Position of SUFFIX field.
	QMI_M1_RCMD_SUFFIX_Pos = 0x8
	// Bit mask of SUFFIX field.
	QMI_M1_RCMD_SUFFIX_Msk = 0xff00
	// Position of PREFIX field.
	QMI_M1_RCMD_PREFIX_Pos = 0x0
	// Bit mask of PREFIX field.
	QMI_M1_RCMD_PREFIX_Msk = 0xff

	// M1_WFMT: Write transfer format configuration for memory address window 1. Configure the bus width of each transfer phase individually, and configure the length or presence of the command prefix, command suffix and dummy/turnaround transfer phases. Only 24-bit addresses are supported. The reset value of the M1_WFMT register is configured to support a basic 02h serial write transfer. However, writes to this window must first be enabled via the XIP_CTRL_WRITABLE_M1 bit, as XIP memory is read-only by default.
	// Position of DTR field.
	QMI_M1_WFMT_DTR_Pos = 0x1c
	// Bit mask of DTR field.
	QMI_M1_WFMT_DTR_Msk = 0x10000000
	// Bit DTR.
	QMI_M1_WFMT_DTR = 0x10000000
	// Position of DUMMY_LEN field.
	QMI_M1_WFMT_DUMMY_LEN_Pos = 0x10
	// Bit mask of DUMMY_LEN field.
	QMI_M1_WFMT_DUMMY_LEN_Msk = 0x70000
	// No dummy phase
	QMI_M1_WFMT_DUMMY_LEN_NONE = 0x0
	// 4 dummy bits
	QMI_M1_WFMT_DUMMY_LEN_4 = 0x1
	// 8 dummy bits
	QMI_M1_WFMT_DUMMY_LEN_8 = 0x2
	// 12 dummy bits
	QMI_M1_WFMT_DUMMY_LEN_12 = 0x3
	// 16 dummy bits
	QMI_M1_WFMT_DUMMY_LEN_16 = 0x4
	// 20 dummy bits
	QMI_M1_WFMT_DUMMY_LEN_20 = 0x5
	// 24 dummy bits
	QMI_M1_WFMT_DUMMY_LEN_24 = 0x6
	// 28 dummy bits
	QMI_M1_WFMT_DUMMY_LEN_28 = 0x7
	// Position of SUFFIX_LEN field.
	QMI_M1_WFMT_SUFFIX_LEN_Pos = 0xe
	// Bit mask of SUFFIX_LEN field.
	QMI_M1_WFMT_SUFFIX_LEN_Msk = 0xc000
	// No suffix
	QMI_M1_WFMT_SUFFIX_LEN_NONE = 0x0
	// 8-bit suffix
	QMI_M1_WFMT_SUFFIX_LEN_8 = 0x2
	// Position of PREFIX_LEN field.
	QMI_M1_WFMT_PREFIX_LEN_Pos = 0xc
	// Bit mask of PREFIX_LEN field.
	QMI_M1_WFMT_PREFIX_LEN_Msk = 0x1000
	// Bit PREFIX_LEN.
	QMI_M1_WFMT_PREFIX_LEN = 0x1000
	// No prefix
	QMI_M1_WFMT_PREFIX_LEN_NONE = 0x0
	// 8-bit prefix
	QMI_M1_WFMT_PREFIX_LEN_8 = 0x1
	// Position of DATA_WIDTH field.
	QMI_M1_WFMT_DATA_WIDTH_Pos = 0x8
	// Bit mask of DATA_WIDTH field.
	QMI_M1_WFMT_DATA_WIDTH_Msk = 0x300
	// Single width
	QMI_M1_WFMT_DATA_WIDTH_S = 0x0
	// Dual width
	QMI_M1_WFMT_DATA_WIDTH_D = 0x1
	// Quad width
	QMI_M1_WFMT_DATA_WIDTH_Q = 0x2
	// Position of DUMMY_WIDTH field.
	QMI_M1_WFMT_DUMMY_WIDTH_Pos = 0x6
	// Bit mask of DUMMY_WIDTH field.
	QMI_M1_WFMT_DUMMY_WIDTH_Msk = 0xc0
	// Single width
	QMI_M1_WFMT_DUMMY_WIDTH_S = 0x0
	// Dual width
	QMI_M1_WFMT_DUMMY_WIDTH_D = 0x1
	// Quad width
	QMI_M1_WFMT_DUMMY_WIDTH_Q = 0x2
	// Position of SUFFIX_WIDTH field.
	QMI_M1_WFMT_SUFFIX_WIDTH_Pos = 0x4
	// Bit mask of SUFFIX_WIDTH field.
	QMI_M1_WFMT_SUFFIX_WIDTH_Msk = 0x30
	// Single width
	QMI_M1_WFMT_SUFFIX_WIDTH_S = 0x0
	// Dual width
	QMI_M1_WFMT_SUFFIX_WIDTH_D = 0x1
	// Quad width
	QMI_M1_WFMT_SUFFIX_WIDTH_Q = 0x2
	// Position of ADDR_WIDTH field.
	QMI_M1_WFMT_ADDR_WIDTH_Pos = 0x2
	// Bit mask of ADDR_WIDTH field.
	QMI_M1_WFMT_ADDR_WIDTH_Msk = 0xc
	// Single width
	QMI_M1_WFMT_ADDR_WIDTH_S = 0x0
	// Dual width
	QMI_M1_WFMT_ADDR_WIDTH_D = 0x1
	// Quad width
	QMI_M1_WFMT_ADDR_WIDTH_Q = 0x2
	// Position of PREFIX_WIDTH field.
	QMI_M1_WFMT_PREFIX_WIDTH_Pos = 0x0
	// Bit mask of PREFIX_WIDTH field.
	QMI_M1_WFMT_PREFIX_WIDTH_Msk = 0x3
	// Single width
	QMI_M1_WFMT_PREFIX_WIDTH_S = 0x0
	// Dual width
	QMI_M1_WFMT_PREFIX_WIDTH_D = 0x1
	// Quad width
	QMI_M1_WFMT_PREFIX_WIDTH_Q = 0x2

	// M1_WCMD: Command constants used for writes to memory address window 1. The reset value of the M1_WCMD register is configured to support a basic 02h serial write transfer with no additional configuration.
	// Position of SUFFIX field.
	QMI_M1_WCMD_SUFFIX_Pos = 0x8
	// Bit mask of SUFFIX field.
	QMI_M1_WCMD_SUFFIX_Msk = 0xff00
	// Position of PREFIX field.
	QMI_M1_WCMD_PREFIX_Pos = 0x0
	// Bit mask of PREFIX field.
	QMI_M1_WCMD_PREFIX_Msk = 0xff

	// ATRANS0: Configure address translation for XIP virtual addresses 0x000000 through 0x3fffff (a 4 MiB window starting at +0 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
	// Position of SIZE field.
	QMI_ATRANS0_SIZE_Pos = 0x10
	// Bit mask of SIZE field.
	QMI_ATRANS0_SIZE_Msk = 0x7ff0000
	// Position of BASE field.
	QMI_ATRANS0_BASE_Pos = 0x0
	// Bit mask of BASE field.
	QMI_ATRANS0_BASE_Msk = 0xfff

	// ATRANS1: Configure address translation for XIP virtual addresses 0x400000 through 0x7fffff (a 4 MiB window starting at +4 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
	// Position of SIZE field.
	QMI_ATRANS1_SIZE_Pos = 0x10
	// Bit mask of SIZE field.
	QMI_ATRANS1_SIZE_Msk = 0x7ff0000
	// Position of BASE field.
	QMI_ATRANS1_BASE_Pos = 0x0
	// Bit mask of BASE field.
	QMI_ATRANS1_BASE_Msk = 0xfff

	// ATRANS2: Configure address translation for XIP virtual addresses 0x800000 through 0xbfffff (a 4 MiB window starting at +8 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
	// Position of SIZE field.
	QMI_ATRANS2_SIZE_Pos = 0x10
	// Bit mask of SIZE field.
	QMI_ATRANS2_SIZE_Msk = 0x7ff0000
	// Position of BASE field.
	QMI_ATRANS2_BASE_Pos = 0x0
	// Bit mask of BASE field.
	QMI_ATRANS2_BASE_Msk = 0xfff

	// ATRANS3: Configure address translation for XIP virtual addresses 0xc00000 through 0xffffff (a 4 MiB window starting at +12 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
	// Position of SIZE field.
	QMI_ATRANS3_SIZE_Pos = 0x10
	// Bit mask of SIZE field.
	QMI_ATRANS3_SIZE_Msk = 0x7ff0000
	// Position of BASE field.
	QMI_ATRANS3_BASE_Pos = 0x0
	// Bit mask of BASE field.
	QMI_ATRANS3_BASE_Msk = 0xfff

	// ATRANS4: Configure address translation for XIP virtual addresses 0x1000000 through 0x13fffff (a 4 MiB window starting at +16 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
	// Position of SIZE field.
	QMI_ATRANS4_SIZE_Pos = 0x10
	// Bit mask of SIZE field.
	QMI_ATRANS4_SIZE_Msk = 0x7ff0000
	// Position of BASE field.
	QMI_ATRANS4_BASE_Pos = 0x0
	// Bit mask of BASE field.
	QMI_ATRANS4_BASE_Msk = 0xfff

	// ATRANS5: Configure address translation for XIP virtual addresses 0x1400000 through 0x17fffff (a 4 MiB window starting at +20 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
	// Position of SIZE field.
	QMI_ATRANS5_SIZE_Pos = 0x10
	// Bit mask of SIZE field.
	QMI_ATRANS5_SIZE_Msk = 0x7ff0000
	// Position of BASE field.
	QMI_ATRANS5_BASE_Pos = 0x0
	// Bit mask of BASE field.
	QMI_ATRANS5_BASE_Msk = 0xfff

	// ATRANS6: Configure address translation for XIP virtual addresses 0x1800000 through 0x1bfffff (a 4 MiB window starting at +24 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
	// Position of SIZE field.
	QMI_ATRANS6_SIZE_Pos = 0x10
	// Bit mask of SIZE field.
	QMI_ATRANS6_SIZE_Msk = 0x7ff0000
	// Position of BASE field.
	QMI_ATRANS6_BASE_Pos = 0x0
	// Bit mask of BASE field.
	QMI_ATRANS6_BASE_Msk = 0xfff

	// ATRANS7: Configure address translation for XIP virtual addresses 0x1c00000 through 0x1ffffff (a 4 MiB window starting at +28 MiB). Address translation allows a program image to be executed in place at multiple physical flash addresses (for example, a double-buffered flash image for over-the-air updates), without the overhead of position-independent code. At reset, the address translation registers are initialised to an identity mapping, so that they can be ignored if address translation is not required. Note that the XIP cache is fully virtually addressed, so a cache flush is required after changing the address translation.
	// Position of SIZE field.
	QMI_ATRANS7_SIZE_Pos = 0x10
	// Bit mask of SIZE field.
	QMI_ATRANS7_SIZE_Msk = 0x7ff0000
	// Position of BASE field.
	QMI_ATRANS7_BASE_Pos = 0x0
	// Bit mask of BASE field.
	QMI_ATRANS7_BASE_Msk = 0xfff
)

// Constants for XIP_CTRL: QSPI flash execute-in-place block
const (
	// CTRL: Cache control register. Read-only from a Non-secure context.
	// Position of WRITABLE_M1 field.
	XIP_CTRL_CTRL_WRITABLE_M1_Pos = 0xb
	// Bit mask of WRITABLE_M1 field.
	XIP_CTRL_CTRL_WRITABLE_M1_Msk = 0x800
	// Bit WRITABLE_M1.
	XIP_CTRL_CTRL_WRITABLE_M1 = 0x800
	// Position of WRITABLE_M0 field.
	XIP_CTRL_CTRL_WRITABLE_M0_Pos = 0xa
	// Bit mask of WRITABLE_M0 field.
	XIP_CTRL_CTRL_WRITABLE_M0_Msk = 0x400
	// Bit WRITABLE_M0.
	XIP_CTRL_CTRL_WRITABLE_M0 = 0x400
	// Position of SPLIT_WAYS field.
	XIP_CTRL_CTRL_SPLIT_WAYS_Pos = 0x9
	// Bit mask of SPLIT_WAYS field.
	XIP_CTRL_CTRL_SPLIT_WAYS_Msk = 0x200
	// Bit SPLIT_WAYS.
	XIP_CTRL_CTRL_SPLIT_WAYS = 0x200
	// Position of MAINT_NONSEC field.
	XIP_CTRL_CTRL_MAINT_NONSEC_Pos = 0x8
	// Bit mask of MAINT_NONSEC field.
	XIP_CTRL_CTRL_MAINT_NONSEC_Msk = 0x100
	// Bit MAINT_NONSEC.
	XIP_CTRL_CTRL_MAINT_NONSEC = 0x100
	// Position of NO_UNTRANSLATED_NONSEC field.
	XIP_CTRL_CTRL_NO_UNTRANSLATED_NONSEC_Pos = 0x7
	// Bit mask of NO_UNTRANSLATED_NONSEC field.
	XIP_CTRL_CTRL_NO_UNTRANSLATED_NONSEC_Msk = 0x80
	// Bit NO_UNTRANSLATED_NONSEC.
	XIP_CTRL_CTRL_NO_UNTRANSLATED_NONSEC = 0x80
	// Position of NO_UNTRANSLATED_SEC field.
	XIP_CTRL_CTRL_NO_UNTRANSLATED_SEC_Pos = 0x6
	// Bit mask of NO_UNTRANSLATED_SEC field.
	XIP_CTRL_CTRL_NO_UNTRANSLATED_SEC_Msk = 0x40
	// Bit NO_UNTRANSLATED_SEC.
	XIP_CTRL_CTRL_NO_UNTRANSLATED_SEC = 0x40
	// Position of NO_UNCACHED_NONSEC field.
	XIP_CTRL_CTRL_NO_UNCACHED_NONSEC_Pos = 0x5
	// Bit mask of NO_UNCACHED_NONSEC field.
	XIP_CTRL_CTRL_NO_UNCACHED_NONSEC_Msk = 0x20
	// Bit NO_UNCACHED_NONSEC.
	XIP_CTRL_CTRL_NO_UNCACHED_NONSEC = 0x20
	// Position of NO_UNCACHED_SEC field.
	XIP_CTRL_CTRL_NO_UNCACHED_SEC_Pos = 0x4
	// Bit mask of NO_UNCACHED_SEC field.
	XIP_CTRL_CTRL_NO_UNCACHED_SEC_Msk = 0x10
	// Bit NO_UNCACHED_SEC.
	XIP_CTRL_CTRL_NO_UNCACHED_SEC = 0x10
	// Position of POWER_DOWN field.
	XIP_CTRL_CTRL_POWER_DOWN_Pos = 0x3
	// Bit mask of POWER_DOWN field.
	XIP_CTRL_CTRL_POWER_DOWN_Msk = 0x8
	// Bit POWER_DOWN.
	XIP_CTRL_CTRL_POWER_DOWN = 0x8
	// Position of EN_NONSECURE field.
	XIP_CTRL_CTRL_EN_NONSECURE_Pos = 0x1
	// Bit mask of EN_NONSECURE field.
	XIP_CTRL_CTRL_EN_NONSECURE_Msk = 0x2
	// Bit EN_NONSECURE.
	XIP_CTRL_CTRL_EN_NONSECURE = 0x2
	// Position of EN_SECURE field.
	XIP_CTRL_CTRL_EN_SECURE_Pos = 0x0
	// Bit mask of EN_SECURE field.
	XIP_CTRL_CTRL_EN_SECURE_Msk = 0x1
	// Bit EN_SECURE.
	XIP_CTRL_CTRL_EN_SECURE = 0x1

	// STAT
	// Position of FIFO_FULL field.
	XIP_CTRL_STAT_FIFO_FULL_Pos = 0x2
	// Bit mask of FIFO_FULL field.
	XIP_CTRL_STAT_FIFO_FULL_Msk = 0x4
	// Bit FIFO_FULL.
	XIP_CTRL_STAT_FIFO_FULL = 0x4
	// Position of FIFO_EMPTY field.
	XIP_CTRL_STAT_FIFO_EMPTY_Pos = 0x1
	// Bit mask of FIFO_EMPTY field.
	XIP_CTRL_STAT_FIFO_EMPTY_Msk = 0x2
	// Bit FIFO_EMPTY.
	XIP_CTRL_STAT_FIFO_EMPTY = 0x2

	// CTR_HIT: Cache Hit counter
	// Position of CTR_HIT field.
	XIP_CTRL_CTR_HIT_CTR_HIT_Pos = 0x0
	// Bit mask of CTR_HIT field.
	XIP_CTRL_CTR_HIT_CTR_HIT_Msk = 0xffffffff

	// CTR_ACC: Cache Access counter
	// Position of CTR_ACC field.
	XIP_CTRL_CTR_ACC_CTR_ACC_Pos = 0x0
	// Bit mask of CTR_ACC field.
	XIP_CTRL_CTR_ACC_CTR_ACC_Msk = 0xffffffff

	// STREAM_ADDR: FIFO stream address
	// Position of STREAM_ADDR field.
	XIP_CTRL_STREAM_ADDR_STREAM_ADDR_Pos = 0x2
	// Bit mask of STREAM_ADDR field.
	XIP_CTRL_STREAM_ADDR_STREAM_ADDR_Msk = 0xfffffffc

	// STREAM_CTR: FIFO stream control
	// Position of STREAM_CTR field.
	XIP_CTRL_STREAM_CTR_STREAM_CTR_Pos = 0x0
	// Bit mask of STREAM_CTR field.
	XIP_CTRL_STREAM_CTR_STREAM_CTR_Msk = 0x3fffff

	// STREAM_FIFO: FIFO stream data
	// Position of STREAM_FIFO field.
	XIP_CTRL_STREAM_FIFO_STREAM_FIFO_Pos = 0x0
	// Bit mask of STREAM_FIFO field.
	XIP_CTRL_STREAM_FIFO_STREAM_FIFO_Msk = 0xffffffff
)

// Constants for XIP_AUX: Auxiliary DMA access to XIP FIFOs, via fast AHB bus access
const (
	// STREAM: Read the XIP stream FIFO (fast bus access to XIP_CTRL_STREAM_FIFO)
	// Position of STREAM field.
	XIP_AUX_STREAM_STREAM_Pos = 0x0
	// Bit mask of STREAM field.
	XIP_AUX_STREAM_STREAM_Msk = 0xffffffff

	// QMI_DIRECT_TX: Write to the QMI direct-mode TX FIFO (fast bus access to QMI_DIRECT_TX)
	// Position of NOPUSH field.
	XIP_AUX_QMI_DIRECT_TX_NOPUSH_Pos = 0x14
	// Bit mask of NOPUSH field.
	XIP_AUX_QMI_DIRECT_TX_NOPUSH_Msk = 0x100000
	// Bit NOPUSH.
	XIP_AUX_QMI_DIRECT_TX_NOPUSH = 0x100000
	// Position of OE field.
	XIP_AUX_QMI_DIRECT_TX_OE_Pos = 0x13
	// Bit mask of OE field.
	XIP_AUX_QMI_DIRECT_TX_OE_Msk = 0x80000
	// Bit OE.
	XIP_AUX_QMI_DIRECT_TX_OE = 0x80000
	// Position of DWIDTH field.
	XIP_AUX_QMI_DIRECT_TX_DWIDTH_Pos = 0x12
	// Bit mask of DWIDTH field.
	XIP_AUX_QMI_DIRECT_TX_DWIDTH_Msk = 0x40000
	// Bit DWIDTH.
	XIP_AUX_QMI_DIRECT_TX_DWIDTH = 0x40000
	// Position of IWIDTH field.
	XIP_AUX_QMI_DIRECT_TX_IWIDTH_Pos = 0x10
	// Bit mask of IWIDTH field.
	XIP_AUX_QMI_DIRECT_TX_IWIDTH_Msk = 0x30000
	// Single width
	XIP_AUX_QMI_DIRECT_TX_IWIDTH_S = 0x0
	// Dual width
	XIP_AUX_QMI_DIRECT_TX_IWIDTH_D = 0x1
	// Quad width
	XIP_AUX_QMI_DIRECT_TX_IWIDTH_Q = 0x2
	// Position of DATA field.
	XIP_AUX_QMI_DIRECT_TX_DATA_Pos = 0x0
	// Bit mask of DATA field.
	XIP_AUX_QMI_DIRECT_TX_DATA_Msk = 0xffff

	// QMI_DIRECT_RX: Read from the QMI direct-mode RX FIFO (fast bus access to QMI_DIRECT_RX)
	// Position of QMI_DIRECT_RX field.
	XIP_AUX_QMI_DIRECT_RX_QMI_DIRECT_RX_Pos = 0x0
	// Bit mask of QMI_DIRECT_RX field.
	XIP_AUX_QMI_DIRECT_RX_QMI_DIRECT_RX_Msk = 0xffff
)

// Constants for SYSCFG: Register block for various chip control signals
const (
	// PROC_CONFIG: Configuration for processors
	// Position of PROC1_HALTED field.
	SYSCFG_PROC_CONFIG_PROC1_HALTED_Pos = 0x1
	// Bit mask of PROC1_HALTED field.
	SYSCFG_PROC_CONFIG_PROC1_HALTED_Msk = 0x2
	// Bit PROC1_HALTED.
	SYSCFG_PROC_CONFIG_PROC1_HALTED = 0x2
	// Position of PROC0_HALTED field.
	SYSCFG_PROC_CONFIG_PROC0_HALTED_Pos = 0x0
	// Bit mask of PROC0_HALTED field.
	SYSCFG_PROC_CONFIG_PROC0_HALTED_Msk = 0x1
	// Bit PROC0_HALTED.
	SYSCFG_PROC_CONFIG_PROC0_HALTED = 0x1

	// PROC_IN_SYNC_BYPASS: For each bit, if 1, bypass the input synchronizer between that GPIO and the GPIO input register in the SIO. The input synchronizers should generally be unbypassed, to avoid injecting metastabilities into processors. If you're feeling brave, you can bypass to save two cycles of input latency. This register applies to GPIO 0...31.
	// Position of GPIO field.
	SYSCFG_PROC_IN_SYNC_BYPASS_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SYSCFG_PROC_IN_SYNC_BYPASS_GPIO_Msk = 0xffffffff

	// PROC_IN_SYNC_BYPASS_HI: For each bit, if 1, bypass the input synchronizer between that GPIO and the GPIO input register in the SIO. The input synchronizers should generally be unbypassed, to avoid injecting metastabilities into processors. If you're feeling brave, you can bypass to save two cycles of input latency. This register applies to GPIO 32...47. USB GPIO 56..57 QSPI GPIO 58..63
	// Position of QSPI_SD field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_USB_DM = 0x2000000
	// Position of USB_DP field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_USB_DP = 0x1000000
	// Position of GPIO field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SYSCFG_PROC_IN_SYNC_BYPASS_HI_GPIO_Msk = 0xffff

	// DBGFORCE: Directly control the chip SWD debug port
	// Position of ATTACH field.
	SYSCFG_DBGFORCE_ATTACH_Pos = 0x3
	// Bit mask of ATTACH field.
	SYSCFG_DBGFORCE_ATTACH_Msk = 0x8
	// Bit ATTACH.
	SYSCFG_DBGFORCE_ATTACH = 0x8
	// Position of SWCLK field.
	SYSCFG_DBGFORCE_SWCLK_Pos = 0x2
	// Bit mask of SWCLK field.
	SYSCFG_DBGFORCE_SWCLK_Msk = 0x4
	// Bit SWCLK.
	SYSCFG_DBGFORCE_SWCLK = 0x4
	// Position of SWDI field.
	SYSCFG_DBGFORCE_SWDI_Pos = 0x1
	// Bit mask of SWDI field.
	SYSCFG_DBGFORCE_SWDI_Msk = 0x2
	// Bit SWDI.
	SYSCFG_DBGFORCE_SWDI = 0x2
	// Position of SWDO field.
	SYSCFG_DBGFORCE_SWDO_Pos = 0x0
	// Bit mask of SWDO field.
	SYSCFG_DBGFORCE_SWDO_Msk = 0x1
	// Bit SWDO.
	SYSCFG_DBGFORCE_SWDO = 0x1

	// MEMPOWERDOWN: Control PD pins to memories. Set high to put memories to a low power state. In this state the memories will retain contents but not be accessible Use with caution
	// Position of BOOTRAM field.
	SYSCFG_MEMPOWERDOWN_BOOTRAM_Pos = 0xc
	// Bit mask of BOOTRAM field.
	SYSCFG_MEMPOWERDOWN_BOOTRAM_Msk = 0x1000
	// Bit BOOTRAM.
	SYSCFG_MEMPOWERDOWN_BOOTRAM = 0x1000
	// Position of ROM field.
	SYSCFG_MEMPOWERDOWN_ROM_Pos = 0xb
	// Bit mask of ROM field.
	SYSCFG_MEMPOWERDOWN_ROM_Msk = 0x800
	// Bit ROM.
	SYSCFG_MEMPOWERDOWN_ROM = 0x800
	// Position of USB field.
	SYSCFG_MEMPOWERDOWN_USB_Pos = 0xa
	// Bit mask of USB field.
	SYSCFG_MEMPOWERDOWN_USB_Msk = 0x400
	// Bit USB.
	SYSCFG_MEMPOWERDOWN_USB = 0x400
	// Position of SRAM9 field.
	SYSCFG_MEMPOWERDOWN_SRAM9_Pos = 0x9
	// Bit mask of SRAM9 field.
	SYSCFG_MEMPOWERDOWN_SRAM9_Msk = 0x200
	// Bit SRAM9.
	SYSCFG_MEMPOWERDOWN_SRAM9 = 0x200
	// Position of SRAM8 field.
	SYSCFG_MEMPOWERDOWN_SRAM8_Pos = 0x8
	// Bit mask of SRAM8 field.
	SYSCFG_MEMPOWERDOWN_SRAM8_Msk = 0x100
	// Bit SRAM8.
	SYSCFG_MEMPOWERDOWN_SRAM8 = 0x100
	// Position of SRAM7 field.
	SYSCFG_MEMPOWERDOWN_SRAM7_Pos = 0x7
	// Bit mask of SRAM7 field.
	SYSCFG_MEMPOWERDOWN_SRAM7_Msk = 0x80
	// Bit SRAM7.
	SYSCFG_MEMPOWERDOWN_SRAM7 = 0x80
	// Position of SRAM6 field.
	SYSCFG_MEMPOWERDOWN_SRAM6_Pos = 0x6
	// Bit mask of SRAM6 field.
	SYSCFG_MEMPOWERDOWN_SRAM6_Msk = 0x40
	// Bit SRAM6.
	SYSCFG_MEMPOWERDOWN_SRAM6 = 0x40
	// Position of SRAM5 field.
	SYSCFG_MEMPOWERDOWN_SRAM5_Pos = 0x5
	// Bit mask of SRAM5 field.
	SYSCFG_MEMPOWERDOWN_SRAM5_Msk = 0x20
	// Bit SRAM5.
	SYSCFG_MEMPOWERDOWN_SRAM5 = 0x20
	// Position of SRAM4 field.
	SYSCFG_MEMPOWERDOWN_SRAM4_Pos = 0x4
	// Bit mask of SRAM4 field.
	SYSCFG_MEMPOWERDOWN_SRAM4_Msk = 0x10
	// Bit SRAM4.
	SYSCFG_MEMPOWERDOWN_SRAM4 = 0x10
	// Position of SRAM3 field.
	SYSCFG_MEMPOWERDOWN_SRAM3_Pos = 0x3
	// Bit mask of SRAM3 field.
	SYSCFG_MEMPOWERDOWN_SRAM3_Msk = 0x8
	// Bit SRAM3.
	SYSCFG_MEMPOWERDOWN_SRAM3 = 0x8
	// Position of SRAM2 field.
	SYSCFG_MEMPOWERDOWN_SRAM2_Pos = 0x2
	// Bit mask of SRAM2 field.
	SYSCFG_MEMPOWERDOWN_SRAM2_Msk = 0x4
	// Bit SRAM2.
	SYSCFG_MEMPOWERDOWN_SRAM2 = 0x4
	// Position of SRAM1 field.
	SYSCFG_MEMPOWERDOWN_SRAM1_Pos = 0x1
	// Bit mask of SRAM1 field.
	SYSCFG_MEMPOWERDOWN_SRAM1_Msk = 0x2
	// Bit SRAM1.
	SYSCFG_MEMPOWERDOWN_SRAM1 = 0x2
	// Position of SRAM0 field.
	SYSCFG_MEMPOWERDOWN_SRAM0_Pos = 0x0
	// Bit mask of SRAM0 field.
	SYSCFG_MEMPOWERDOWN_SRAM0_Msk = 0x1
	// Bit SRAM0.
	SYSCFG_MEMPOWERDOWN_SRAM0 = 0x1

	// AUXCTRL: Auxiliary system control register
	// Position of AUXCTRL field.
	SYSCFG_AUXCTRL_AUXCTRL_Pos = 0x0
	// Bit mask of AUXCTRL field.
	SYSCFG_AUXCTRL_AUXCTRL_Msk = 0xff
)

// Constants for XOSC: Controls the crystal oscillator
const (
	// CTRL: Crystal Oscillator Control
	// Position of ENABLE field.
	XOSC_CTRL_ENABLE_Pos = 0xc
	// Bit mask of ENABLE field.
	XOSC_CTRL_ENABLE_Msk     = 0xfff000
	XOSC_CTRL_ENABLE_DISABLE = 0xd1e
	XOSC_CTRL_ENABLE_ENABLE  = 0xfab
	// Position of FREQ_RANGE field.
	XOSC_CTRL_FREQ_RANGE_Pos = 0x0
	// Bit mask of FREQ_RANGE field.
	XOSC_CTRL_FREQ_RANGE_Msk       = 0xfff
	XOSC_CTRL_FREQ_RANGE_1_15MHZ   = 0xaa0
	XOSC_CTRL_FREQ_RANGE_10_30MHZ  = 0xaa1
	XOSC_CTRL_FREQ_RANGE_25_60MHZ  = 0xaa2
	XOSC_CTRL_FREQ_RANGE_40_100MHZ = 0xaa3

	// STATUS: Crystal Oscillator Status
	// Position of STABLE field.
	XOSC_STATUS_STABLE_Pos = 0x1f
	// Bit mask of STABLE field.
	XOSC_STATUS_STABLE_Msk = 0x80000000
	// Bit STABLE.
	XOSC_STATUS_STABLE = 0x80000000
	// Position of BADWRITE field.
	XOSC_STATUS_BADWRITE_Pos = 0x18
	// Bit mask of BADWRITE field.
	XOSC_STATUS_BADWRITE_Msk = 0x1000000
	// Bit BADWRITE.
	XOSC_STATUS_BADWRITE = 0x1000000
	// Position of ENABLED field.
	XOSC_STATUS_ENABLED_Pos = 0xc
	// Bit mask of ENABLED field.
	XOSC_STATUS_ENABLED_Msk = 0x1000
	// Bit ENABLED.
	XOSC_STATUS_ENABLED = 0x1000
	// Position of FREQ_RANGE field.
	XOSC_STATUS_FREQ_RANGE_Pos = 0x0
	// Bit mask of FREQ_RANGE field.
	XOSC_STATUS_FREQ_RANGE_Msk       = 0x3
	XOSC_STATUS_FREQ_RANGE_1_15MHZ   = 0x0
	XOSC_STATUS_FREQ_RANGE_10_30MHZ  = 0x1
	XOSC_STATUS_FREQ_RANGE_25_60MHZ  = 0x2
	XOSC_STATUS_FREQ_RANGE_40_100MHZ = 0x3

	// DORMANT: Crystal Oscillator pause control
	// Position of DORMANT field.
	XOSC_DORMANT_DORMANT_Pos = 0x0
	// Bit mask of DORMANT field.
	XOSC_DORMANT_DORMANT_Msk     = 0xffffffff
	XOSC_DORMANT_DORMANT_DORMANT = 0x636f6d61
	XOSC_DORMANT_DORMANT_WAKE    = 0x77616b65

	// STARTUP: Controls the startup delay
	// Position of X4 field.
	XOSC_STARTUP_X4_Pos = 0x14
	// Bit mask of X4 field.
	XOSC_STARTUP_X4_Msk = 0x100000
	// Bit X4.
	XOSC_STARTUP_X4 = 0x100000
	// Position of DELAY field.
	XOSC_STARTUP_DELAY_Pos = 0x0
	// Bit mask of DELAY field.
	XOSC_STARTUP_DELAY_Msk = 0x3fff

	// COUNT: A down counter running at the xosc frequency which counts to zero and stops. Can be used for short software pauses when setting up time sensitive hardware. To start the counter, write a non-zero value. Reads will return 1 while the count is running and 0 when it has finished. Minimum count value is 4. Count values <4 will be treated as count value =4. Note that synchronisation to the register clock domain costs 2 register clock cycles and the counter cannot compensate for that.
	// Position of COUNT field.
	XOSC_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	XOSC_COUNT_COUNT_Msk = 0xffff
)

// Constants for PLL_SYS
const (
	// CS: Control and Status GENERAL CONSTRAINTS: Reference clock frequency min=5MHz, max=800MHz Feedback divider min=16, max=320 VCO frequency min=750MHz, max=1600MHz
	// Position of LOCK field.
	PLL_SYS_CS_LOCK_Pos = 0x1f
	// Bit mask of LOCK field.
	PLL_SYS_CS_LOCK_Msk = 0x80000000
	// Bit LOCK.
	PLL_SYS_CS_LOCK = 0x80000000
	// Position of LOCK_N field.
	PLL_SYS_CS_LOCK_N_Pos = 0x1e
	// Bit mask of LOCK_N field.
	PLL_SYS_CS_LOCK_N_Msk = 0x40000000
	// Bit LOCK_N.
	PLL_SYS_CS_LOCK_N = 0x40000000
	// Position of BYPASS field.
	PLL_SYS_CS_BYPASS_Pos = 0x8
	// Bit mask of BYPASS field.
	PLL_SYS_CS_BYPASS_Msk = 0x100
	// Bit BYPASS.
	PLL_SYS_CS_BYPASS = 0x100
	// Position of REFDIV field.
	PLL_SYS_CS_REFDIV_Pos = 0x0
	// Bit mask of REFDIV field.
	PLL_SYS_CS_REFDIV_Msk = 0x3f

	// PWR: Controls the PLL power modes.
	// Position of VCOPD field.
	PLL_SYS_PWR_VCOPD_Pos = 0x5
	// Bit mask of VCOPD field.
	PLL_SYS_PWR_VCOPD_Msk = 0x20
	// Bit VCOPD.
	PLL_SYS_PWR_VCOPD = 0x20
	// Position of POSTDIVPD field.
	PLL_SYS_PWR_POSTDIVPD_Pos = 0x3
	// Bit mask of POSTDIVPD field.
	PLL_SYS_PWR_POSTDIVPD_Msk = 0x8
	// Bit POSTDIVPD.
	PLL_SYS_PWR_POSTDIVPD = 0x8
	// Position of DSMPD field.
	PLL_SYS_PWR_DSMPD_Pos = 0x2
	// Bit mask of DSMPD field.
	PLL_SYS_PWR_DSMPD_Msk = 0x4
	// Bit DSMPD.
	PLL_SYS_PWR_DSMPD = 0x4
	// Position of PD field.
	PLL_SYS_PWR_PD_Pos = 0x0
	// Bit mask of PD field.
	PLL_SYS_PWR_PD_Msk = 0x1
	// Bit PD.
	PLL_SYS_PWR_PD = 0x1

	// FBDIV_INT: Feedback divisor (note: this PLL does not support fractional division)
	// Position of FBDIV_INT field.
	PLL_SYS_FBDIV_INT_FBDIV_INT_Pos = 0x0
	// Bit mask of FBDIV_INT field.
	PLL_SYS_FBDIV_INT_FBDIV_INT_Msk = 0xfff

	// PRIM: Controls the PLL post dividers for the primary output (note: this PLL does not have a secondary output) the primary output is driven from VCO divided by postdiv1*postdiv2
	// Position of POSTDIV1 field.
	PLL_SYS_PRIM_POSTDIV1_Pos = 0x10
	// Bit mask of POSTDIV1 field.
	PLL_SYS_PRIM_POSTDIV1_Msk = 0x70000
	// Position of POSTDIV2 field.
	PLL_SYS_PRIM_POSTDIV2_Pos = 0xc
	// Bit mask of POSTDIV2 field.
	PLL_SYS_PRIM_POSTDIV2_Msk = 0x7000

	// INTR: Raw Interrupts
	// Position of LOCK_N_STICKY field.
	PLL_SYS_INTR_LOCK_N_STICKY_Pos = 0x0
	// Bit mask of LOCK_N_STICKY field.
	PLL_SYS_INTR_LOCK_N_STICKY_Msk = 0x1
	// Bit LOCK_N_STICKY.
	PLL_SYS_INTR_LOCK_N_STICKY = 0x1

	// INTE: Interrupt Enable
	// Position of LOCK_N_STICKY field.
	PLL_SYS_INTE_LOCK_N_STICKY_Pos = 0x0
	// Bit mask of LOCK_N_STICKY field.
	PLL_SYS_INTE_LOCK_N_STICKY_Msk = 0x1
	// Bit LOCK_N_STICKY.
	PLL_SYS_INTE_LOCK_N_STICKY = 0x1

	// INTF: Interrupt Force
	// Position of LOCK_N_STICKY field.
	PLL_SYS_INTF_LOCK_N_STICKY_Pos = 0x0
	// Bit mask of LOCK_N_STICKY field.
	PLL_SYS_INTF_LOCK_N_STICKY_Msk = 0x1
	// Bit LOCK_N_STICKY.
	PLL_SYS_INTF_LOCK_N_STICKY = 0x1

	// INTS: Interrupt status after masking & forcing
	// Position of LOCK_N_STICKY field.
	PLL_SYS_INTS_LOCK_N_STICKY_Pos = 0x0
	// Bit mask of LOCK_N_STICKY field.
	PLL_SYS_INTS_LOCK_N_STICKY_Msk = 0x1
	// Bit LOCK_N_STICKY.
	PLL_SYS_INTS_LOCK_N_STICKY = 0x1
)

// Constants for ACCESSCTRL: Hardware access control registers
const (
	// LOCK: Once a LOCK bit is written to 1, ACCESSCTRL silently ignores writes from that master. LOCK is writable only by a Secure, Privileged processor or debugger. LOCK bits are only writable when their value is zero. Once set, they can never be cleared, except by a full reset of ACCESSCTRL Setting the LOCK bit does not affect whether an access raises a bus error. Unprivileged writes, or writes from the DMA, will continue to raise bus errors. All other accesses will continue not to.
	// Position of DEBUG field.
	ACCESSCTRL_LOCK_DEBUG_Pos = 0x3
	// Bit mask of DEBUG field.
	ACCESSCTRL_LOCK_DEBUG_Msk = 0x8
	// Bit DEBUG.
	ACCESSCTRL_LOCK_DEBUG = 0x8
	// Position of DMA field.
	ACCESSCTRL_LOCK_DMA_Pos = 0x2
	// Bit mask of DMA field.
	ACCESSCTRL_LOCK_DMA_Msk = 0x4
	// Bit DMA.
	ACCESSCTRL_LOCK_DMA = 0x4
	// Position of CORE1 field.
	ACCESSCTRL_LOCK_CORE1_Pos = 0x1
	// Bit mask of CORE1 field.
	ACCESSCTRL_LOCK_CORE1_Msk = 0x2
	// Bit CORE1.
	ACCESSCTRL_LOCK_CORE1 = 0x2
	// Position of CORE0 field.
	ACCESSCTRL_LOCK_CORE0_Pos = 0x0
	// Bit mask of CORE0 field.
	ACCESSCTRL_LOCK_CORE0_Msk = 0x1
	// Bit CORE0.
	ACCESSCTRL_LOCK_CORE0 = 0x1

	// FORCE_CORE_NS: Force core 1's bus accesses to always be Non-secure, no matter the core's internal state. Useful for schemes where one core is designated as the Non-secure core, since some peripherals may filter individual registers internally based on security state but not on master ID.
	// Position of CORE1 field.
	ACCESSCTRL_FORCE_CORE_NS_CORE1_Pos = 0x1
	// Bit mask of CORE1 field.
	ACCESSCTRL_FORCE_CORE_NS_CORE1_Msk = 0x2
	// Bit CORE1.
	ACCESSCTRL_FORCE_CORE_NS_CORE1 = 0x2

	// CFGRESET: Write 1 to reset all ACCESSCTRL configuration, except for the LOCK and FORCE_CORE_NS registers. This bit is used in the RP2350 bootrom to quickly restore ACCESSCTRL to a known state during the boot path. Note that, like all registers in ACCESSCTRL, this register is not writable when the writer's corresponding LOCK bit is set, therefore a master which has been locked out of ACCESSCTRL can not use the CFGRESET register to disturb its contents.
	// Position of CFGRESET field.
	ACCESSCTRL_CFGRESET_CFGRESET_Pos = 0x0
	// Bit mask of CFGRESET field.
	ACCESSCTRL_CFGRESET_CFGRESET_Msk = 0x1
	// Bit CFGRESET.
	ACCESSCTRL_CFGRESET_CFGRESET = 0x1

	// GPIO_NSMASK0: Control whether GPIO0...31 are accessible to Non-secure code. Writable only by a Secure, Privileged processor or debugger. 0 -> Secure access only 1 -> Secure + Non-secure access
	// Position of GPIO_NSMASK0 field.
	ACCESSCTRL_GPIO_NSMASK0_GPIO_NSMASK0_Pos = 0x0
	// Bit mask of GPIO_NSMASK0 field.
	ACCESSCTRL_GPIO_NSMASK0_GPIO_NSMASK0_Msk = 0xffffffff

	// GPIO_NSMASK1: Control whether GPIO32..47 are accessible to Non-secure code, and whether QSPI and USB bitbang are accessible through the Non-secure SIO. Writable only by a Secure, Privileged processor or debugger.
	// Position of QSPI_SD field.
	ACCESSCTRL_GPIO_NSMASK1_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	ACCESSCTRL_GPIO_NSMASK1_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	ACCESSCTRL_GPIO_NSMASK1_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	ACCESSCTRL_GPIO_NSMASK1_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	ACCESSCTRL_GPIO_NSMASK1_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	ACCESSCTRL_GPIO_NSMASK1_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	ACCESSCTRL_GPIO_NSMASK1_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	ACCESSCTRL_GPIO_NSMASK1_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	ACCESSCTRL_GPIO_NSMASK1_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	ACCESSCTRL_GPIO_NSMASK1_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	ACCESSCTRL_GPIO_NSMASK1_USB_DM = 0x2000000
	// Position of USB_DP field.
	ACCESSCTRL_GPIO_NSMASK1_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	ACCESSCTRL_GPIO_NSMASK1_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	ACCESSCTRL_GPIO_NSMASK1_USB_DP = 0x1000000
	// Position of GPIO field.
	ACCESSCTRL_GPIO_NSMASK1_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	ACCESSCTRL_GPIO_NSMASK1_GPIO_Msk = 0xffff

	// ROM: Control whether debugger, DMA, core 0 and core 1 can access ROM, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_ROM_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_ROM_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_ROM_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_ROM_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_ROM_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_ROM_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_ROM_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_ROM_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_ROM_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_ROM_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_ROM_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_ROM_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_ROM_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_ROM_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_ROM_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_ROM_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_ROM_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_ROM_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_ROM_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_ROM_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_ROM_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_ROM_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_ROM_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_ROM_NSU = 0x1

	// XIP_MAIN: Control whether debugger, DMA, core 0 and core 1 can access XIP_MAIN, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_XIP_MAIN_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_XIP_MAIN_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_XIP_MAIN_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_XIP_MAIN_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_XIP_MAIN_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_XIP_MAIN_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_XIP_MAIN_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_XIP_MAIN_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_XIP_MAIN_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_XIP_MAIN_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_XIP_MAIN_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_XIP_MAIN_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_XIP_MAIN_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_XIP_MAIN_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_XIP_MAIN_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_XIP_MAIN_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_XIP_MAIN_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_XIP_MAIN_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_XIP_MAIN_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_XIP_MAIN_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_XIP_MAIN_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_XIP_MAIN_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_XIP_MAIN_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_XIP_MAIN_NSU = 0x1

	// SRAM0: Control whether debugger, DMA, core 0 and core 1 can access SRAM0, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM0_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM0_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM0_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM0_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM0_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM0_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM0_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM0_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM0_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM0_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM0_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM0_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM0_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM0_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM0_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM0_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM0_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM0_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM0_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM0_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM0_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM0_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM0_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM0_NSU = 0x1

	// SRAM1: Control whether debugger, DMA, core 0 and core 1 can access SRAM1, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM1_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM1_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM1_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM1_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM1_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM1_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM1_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM1_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM1_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM1_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM1_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM1_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM1_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM1_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM1_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM1_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM1_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM1_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM1_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM1_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM1_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM1_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM1_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM1_NSU = 0x1

	// SRAM2: Control whether debugger, DMA, core 0 and core 1 can access SRAM2, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM2_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM2_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM2_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM2_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM2_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM2_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM2_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM2_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM2_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM2_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM2_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM2_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM2_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM2_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM2_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM2_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM2_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM2_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM2_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM2_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM2_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM2_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM2_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM2_NSU = 0x1

	// SRAM3: Control whether debugger, DMA, core 0 and core 1 can access SRAM3, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM3_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM3_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM3_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM3_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM3_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM3_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM3_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM3_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM3_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM3_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM3_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM3_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM3_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM3_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM3_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM3_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM3_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM3_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM3_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM3_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM3_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM3_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM3_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM3_NSU = 0x1

	// SRAM4: Control whether debugger, DMA, core 0 and core 1 can access SRAM4, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM4_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM4_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM4_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM4_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM4_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM4_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM4_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM4_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM4_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM4_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM4_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM4_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM4_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM4_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM4_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM4_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM4_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM4_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM4_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM4_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM4_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM4_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM4_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM4_NSU = 0x1

	// SRAM5: Control whether debugger, DMA, core 0 and core 1 can access SRAM5, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM5_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM5_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM5_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM5_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM5_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM5_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM5_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM5_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM5_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM5_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM5_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM5_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM5_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM5_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM5_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM5_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM5_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM5_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM5_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM5_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM5_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM5_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM5_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM5_NSU = 0x1

	// SRAM6: Control whether debugger, DMA, core 0 and core 1 can access SRAM6, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM6_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM6_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM6_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM6_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM6_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM6_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM6_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM6_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM6_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM6_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM6_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM6_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM6_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM6_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM6_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM6_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM6_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM6_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM6_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM6_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM6_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM6_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM6_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM6_NSU = 0x1

	// SRAM7: Control whether debugger, DMA, core 0 and core 1 can access SRAM7, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM7_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM7_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM7_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM7_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM7_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM7_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM7_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM7_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM7_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM7_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM7_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM7_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM7_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM7_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM7_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM7_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM7_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM7_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM7_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM7_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM7_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM7_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM7_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM7_NSU = 0x1

	// SRAM8: Control whether debugger, DMA, core 0 and core 1 can access SRAM8, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM8_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM8_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM8_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM8_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM8_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM8_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM8_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM8_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM8_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM8_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM8_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM8_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM8_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM8_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM8_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM8_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM8_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM8_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM8_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM8_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM8_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM8_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM8_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM8_NSU = 0x1

	// SRAM9: Control whether debugger, DMA, core 0 and core 1 can access SRAM9, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SRAM9_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SRAM9_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SRAM9_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SRAM9_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SRAM9_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SRAM9_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SRAM9_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SRAM9_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SRAM9_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SRAM9_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SRAM9_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SRAM9_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SRAM9_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SRAM9_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SRAM9_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SRAM9_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SRAM9_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SRAM9_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SRAM9_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SRAM9_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SRAM9_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SRAM9_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SRAM9_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SRAM9_NSU = 0x1

	// DMA: Control whether debugger, DMA, core 0 and core 1 can access DMA, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_DMA_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_DMA_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_DMA_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_DMA_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_DMA_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_DMA_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_DMA_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_DMA_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_DMA_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_DMA_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_DMA_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_DMA_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_DMA_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_DMA_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_DMA_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_DMA_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_DMA_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_DMA_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_DMA_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_DMA_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_DMA_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_DMA_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_DMA_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_DMA_NSU = 0x1

	// USBCTRL: Control whether debugger, DMA, core 0 and core 1 can access USBCTRL, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_USBCTRL_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_USBCTRL_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_USBCTRL_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_USBCTRL_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_USBCTRL_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_USBCTRL_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_USBCTRL_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_USBCTRL_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_USBCTRL_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_USBCTRL_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_USBCTRL_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_USBCTRL_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_USBCTRL_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_USBCTRL_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_USBCTRL_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_USBCTRL_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_USBCTRL_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_USBCTRL_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_USBCTRL_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_USBCTRL_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_USBCTRL_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_USBCTRL_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_USBCTRL_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_USBCTRL_NSU = 0x1

	// PIO0: Control whether debugger, DMA, core 0 and core 1 can access PIO0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_PIO0_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_PIO0_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_PIO0_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_PIO0_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_PIO0_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_PIO0_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_PIO0_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_PIO0_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_PIO0_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_PIO0_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_PIO0_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_PIO0_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_PIO0_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_PIO0_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_PIO0_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_PIO0_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_PIO0_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_PIO0_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_PIO0_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_PIO0_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_PIO0_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_PIO0_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_PIO0_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_PIO0_NSU = 0x1

	// PIO1: Control whether debugger, DMA, core 0 and core 1 can access PIO1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_PIO1_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_PIO1_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_PIO1_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_PIO1_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_PIO1_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_PIO1_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_PIO1_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_PIO1_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_PIO1_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_PIO1_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_PIO1_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_PIO1_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_PIO1_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_PIO1_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_PIO1_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_PIO1_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_PIO1_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_PIO1_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_PIO1_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_PIO1_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_PIO1_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_PIO1_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_PIO1_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_PIO1_NSU = 0x1

	// PIO2: Control whether debugger, DMA, core 0 and core 1 can access PIO2, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_PIO2_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_PIO2_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_PIO2_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_PIO2_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_PIO2_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_PIO2_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_PIO2_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_PIO2_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_PIO2_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_PIO2_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_PIO2_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_PIO2_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_PIO2_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_PIO2_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_PIO2_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_PIO2_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_PIO2_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_PIO2_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_PIO2_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_PIO2_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_PIO2_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_PIO2_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_PIO2_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_PIO2_NSU = 0x1

	// CORESIGHT_TRACE: Control whether debugger, DMA, core 0 and core 1 can access CORESIGHT_TRACE, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_CORESIGHT_TRACE_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_CORESIGHT_TRACE_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_CORESIGHT_TRACE_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_CORESIGHT_TRACE_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_CORESIGHT_TRACE_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_CORESIGHT_TRACE_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_CORESIGHT_TRACE_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_CORESIGHT_TRACE_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_CORESIGHT_TRACE_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_CORESIGHT_TRACE_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_CORESIGHT_TRACE_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_CORESIGHT_TRACE_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_CORESIGHT_TRACE_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_CORESIGHT_TRACE_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_CORESIGHT_TRACE_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_CORESIGHT_TRACE_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_CORESIGHT_TRACE_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_CORESIGHT_TRACE_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_CORESIGHT_TRACE_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_CORESIGHT_TRACE_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_CORESIGHT_TRACE_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_CORESIGHT_TRACE_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_CORESIGHT_TRACE_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_CORESIGHT_TRACE_NSU = 0x1

	// CORESIGHT_PERIPH: Control whether debugger, DMA, core 0 and core 1 can access CORESIGHT_PERIPH, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_CORESIGHT_PERIPH_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_CORESIGHT_PERIPH_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_CORESIGHT_PERIPH_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_CORESIGHT_PERIPH_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_CORESIGHT_PERIPH_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_CORESIGHT_PERIPH_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_CORESIGHT_PERIPH_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_CORESIGHT_PERIPH_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_CORESIGHT_PERIPH_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_CORESIGHT_PERIPH_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_CORESIGHT_PERIPH_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_CORESIGHT_PERIPH_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_CORESIGHT_PERIPH_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_CORESIGHT_PERIPH_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_CORESIGHT_PERIPH_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_CORESIGHT_PERIPH_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_CORESIGHT_PERIPH_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_CORESIGHT_PERIPH_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_CORESIGHT_PERIPH_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_CORESIGHT_PERIPH_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_CORESIGHT_PERIPH_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_CORESIGHT_PERIPH_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_CORESIGHT_PERIPH_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_CORESIGHT_PERIPH_NSU = 0x1

	// SYSINFO: Control whether debugger, DMA, core 0 and core 1 can access SYSINFO, and at what security/privilege levels they can do so. Defaults to fully open access. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SYSINFO_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SYSINFO_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SYSINFO_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SYSINFO_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SYSINFO_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SYSINFO_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SYSINFO_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SYSINFO_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SYSINFO_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SYSINFO_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SYSINFO_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SYSINFO_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SYSINFO_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SYSINFO_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SYSINFO_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SYSINFO_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SYSINFO_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SYSINFO_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SYSINFO_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SYSINFO_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SYSINFO_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SYSINFO_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SYSINFO_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SYSINFO_NSU = 0x1

	// RESETS: Control whether debugger, DMA, core 0 and core 1 can access RESETS, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_RESETS_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_RESETS_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_RESETS_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_RESETS_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_RESETS_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_RESETS_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_RESETS_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_RESETS_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_RESETS_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_RESETS_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_RESETS_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_RESETS_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_RESETS_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_RESETS_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_RESETS_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_RESETS_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_RESETS_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_RESETS_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_RESETS_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_RESETS_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_RESETS_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_RESETS_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_RESETS_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_RESETS_NSU = 0x1

	// IO_BANK0: Control whether debugger, DMA, core 0 and core 1 can access IO_BANK0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_IO_BANK0_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_IO_BANK0_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_IO_BANK0_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_IO_BANK0_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_IO_BANK0_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_IO_BANK0_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_IO_BANK0_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_IO_BANK0_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_IO_BANK0_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_IO_BANK0_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_IO_BANK0_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_IO_BANK0_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_IO_BANK0_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_IO_BANK0_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_IO_BANK0_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_IO_BANK0_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_IO_BANK0_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_IO_BANK0_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_IO_BANK0_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_IO_BANK0_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_IO_BANK0_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_IO_BANK0_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_IO_BANK0_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_IO_BANK0_NSU = 0x1

	// IO_BANK1: Control whether debugger, DMA, core 0 and core 1 can access IO_BANK1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_IO_BANK1_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_IO_BANK1_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_IO_BANK1_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_IO_BANK1_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_IO_BANK1_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_IO_BANK1_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_IO_BANK1_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_IO_BANK1_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_IO_BANK1_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_IO_BANK1_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_IO_BANK1_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_IO_BANK1_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_IO_BANK1_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_IO_BANK1_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_IO_BANK1_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_IO_BANK1_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_IO_BANK1_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_IO_BANK1_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_IO_BANK1_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_IO_BANK1_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_IO_BANK1_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_IO_BANK1_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_IO_BANK1_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_IO_BANK1_NSU = 0x1

	// PADS_BANK0: Control whether debugger, DMA, core 0 and core 1 can access PADS_BANK0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_PADS_BANK0_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_PADS_BANK0_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_PADS_BANK0_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_PADS_BANK0_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_PADS_BANK0_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_PADS_BANK0_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_PADS_BANK0_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_PADS_BANK0_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_PADS_BANK0_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_PADS_BANK0_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_PADS_BANK0_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_PADS_BANK0_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_PADS_BANK0_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_PADS_BANK0_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_PADS_BANK0_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_PADS_BANK0_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_PADS_BANK0_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_PADS_BANK0_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_PADS_BANK0_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_PADS_BANK0_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_PADS_BANK0_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_PADS_BANK0_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_PADS_BANK0_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_PADS_BANK0_NSU = 0x1

	// PADS_QSPI: Control whether debugger, DMA, core 0 and core 1 can access PADS_QSPI, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_PADS_QSPI_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_PADS_QSPI_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_PADS_QSPI_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_PADS_QSPI_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_PADS_QSPI_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_PADS_QSPI_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_PADS_QSPI_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_PADS_QSPI_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_PADS_QSPI_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_PADS_QSPI_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_PADS_QSPI_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_PADS_QSPI_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_PADS_QSPI_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_PADS_QSPI_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_PADS_QSPI_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_PADS_QSPI_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_PADS_QSPI_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_PADS_QSPI_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_PADS_QSPI_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_PADS_QSPI_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_PADS_QSPI_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_PADS_QSPI_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_PADS_QSPI_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_PADS_QSPI_NSU = 0x1

	// BUSCTRL: Control whether debugger, DMA, core 0 and core 1 can access BUSCTRL, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_BUSCTRL_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_BUSCTRL_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_BUSCTRL_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_BUSCTRL_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_BUSCTRL_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_BUSCTRL_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_BUSCTRL_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_BUSCTRL_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_BUSCTRL_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_BUSCTRL_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_BUSCTRL_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_BUSCTRL_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_BUSCTRL_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_BUSCTRL_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_BUSCTRL_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_BUSCTRL_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_BUSCTRL_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_BUSCTRL_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_BUSCTRL_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_BUSCTRL_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_BUSCTRL_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_BUSCTRL_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_BUSCTRL_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_BUSCTRL_NSU = 0x1

	// ADC0: Control whether debugger, DMA, core 0 and core 1 can access ADC0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_ADC0_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_ADC0_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_ADC0_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_ADC0_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_ADC0_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_ADC0_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_ADC0_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_ADC0_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_ADC0_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_ADC0_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_ADC0_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_ADC0_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_ADC0_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_ADC0_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_ADC0_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_ADC0_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_ADC0_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_ADC0_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_ADC0_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_ADC0_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_ADC0_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_ADC0_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_ADC0_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_ADC0_NSU = 0x1

	// HSTX: Control whether debugger, DMA, core 0 and core 1 can access HSTX, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_HSTX_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_HSTX_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_HSTX_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_HSTX_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_HSTX_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_HSTX_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_HSTX_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_HSTX_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_HSTX_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_HSTX_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_HSTX_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_HSTX_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_HSTX_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_HSTX_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_HSTX_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_HSTX_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_HSTX_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_HSTX_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_HSTX_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_HSTX_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_HSTX_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_HSTX_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_HSTX_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_HSTX_NSU = 0x1

	// I2C0: Control whether debugger, DMA, core 0 and core 1 can access I2C0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_I2C0_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_I2C0_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_I2C0_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_I2C0_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_I2C0_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_I2C0_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_I2C0_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_I2C0_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_I2C0_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_I2C0_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_I2C0_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_I2C0_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_I2C0_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_I2C0_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_I2C0_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_I2C0_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_I2C0_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_I2C0_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_I2C0_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_I2C0_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_I2C0_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_I2C0_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_I2C0_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_I2C0_NSU = 0x1

	// I2C1: Control whether debugger, DMA, core 0 and core 1 can access I2C1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_I2C1_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_I2C1_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_I2C1_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_I2C1_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_I2C1_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_I2C1_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_I2C1_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_I2C1_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_I2C1_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_I2C1_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_I2C1_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_I2C1_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_I2C1_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_I2C1_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_I2C1_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_I2C1_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_I2C1_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_I2C1_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_I2C1_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_I2C1_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_I2C1_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_I2C1_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_I2C1_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_I2C1_NSU = 0x1

	// PWM: Control whether debugger, DMA, core 0 and core 1 can access PWM, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_PWM_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_PWM_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_PWM_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_PWM_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_PWM_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_PWM_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_PWM_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_PWM_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_PWM_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_PWM_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_PWM_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_PWM_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_PWM_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_PWM_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_PWM_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_PWM_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_PWM_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_PWM_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_PWM_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_PWM_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_PWM_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_PWM_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_PWM_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_PWM_NSU = 0x1

	// SPI0: Control whether debugger, DMA, core 0 and core 1 can access SPI0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SPI0_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SPI0_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SPI0_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SPI0_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SPI0_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SPI0_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SPI0_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SPI0_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SPI0_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SPI0_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SPI0_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SPI0_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SPI0_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SPI0_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SPI0_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SPI0_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SPI0_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SPI0_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SPI0_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SPI0_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SPI0_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SPI0_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SPI0_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SPI0_NSU = 0x1

	// SPI1: Control whether debugger, DMA, core 0 and core 1 can access SPI1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SPI1_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SPI1_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SPI1_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SPI1_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SPI1_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SPI1_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SPI1_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SPI1_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SPI1_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SPI1_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SPI1_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SPI1_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SPI1_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SPI1_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SPI1_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SPI1_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SPI1_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SPI1_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SPI1_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SPI1_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SPI1_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SPI1_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SPI1_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SPI1_NSU = 0x1

	// TIMER0: Control whether debugger, DMA, core 0 and core 1 can access TIMER0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_TIMER0_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_TIMER0_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_TIMER0_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_TIMER0_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_TIMER0_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_TIMER0_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_TIMER0_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_TIMER0_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_TIMER0_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_TIMER0_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_TIMER0_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_TIMER0_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_TIMER0_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_TIMER0_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_TIMER0_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_TIMER0_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_TIMER0_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_TIMER0_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_TIMER0_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_TIMER0_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_TIMER0_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_TIMER0_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_TIMER0_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_TIMER0_NSU = 0x1

	// TIMER1: Control whether debugger, DMA, core 0 and core 1 can access TIMER1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_TIMER1_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_TIMER1_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_TIMER1_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_TIMER1_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_TIMER1_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_TIMER1_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_TIMER1_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_TIMER1_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_TIMER1_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_TIMER1_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_TIMER1_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_TIMER1_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_TIMER1_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_TIMER1_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_TIMER1_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_TIMER1_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_TIMER1_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_TIMER1_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_TIMER1_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_TIMER1_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_TIMER1_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_TIMER1_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_TIMER1_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_TIMER1_NSU = 0x1

	// UART0: Control whether debugger, DMA, core 0 and core 1 can access UART0, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_UART0_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_UART0_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_UART0_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_UART0_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_UART0_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_UART0_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_UART0_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_UART0_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_UART0_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_UART0_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_UART0_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_UART0_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_UART0_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_UART0_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_UART0_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_UART0_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_UART0_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_UART0_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_UART0_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_UART0_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_UART0_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_UART0_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_UART0_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_UART0_NSU = 0x1

	// UART1: Control whether debugger, DMA, core 0 and core 1 can access UART1, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_UART1_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_UART1_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_UART1_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_UART1_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_UART1_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_UART1_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_UART1_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_UART1_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_UART1_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_UART1_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_UART1_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_UART1_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_UART1_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_UART1_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_UART1_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_UART1_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_UART1_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_UART1_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_UART1_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_UART1_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_UART1_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_UART1_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_UART1_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_UART1_NSU = 0x1

	// OTP: Control whether debugger, DMA, core 0 and core 1 can access OTP, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_OTP_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_OTP_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_OTP_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_OTP_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_OTP_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_OTP_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_OTP_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_OTP_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_OTP_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_OTP_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_OTP_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_OTP_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_OTP_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_OTP_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_OTP_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_OTP_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_OTP_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_OTP_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_OTP_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_OTP_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_OTP_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_OTP_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_OTP_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_OTP_NSU = 0x1

	// TBMAN: Control whether debugger, DMA, core 0 and core 1 can access TBMAN, and at what security/privilege levels they can do so. Defaults to Secure access from any master. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_TBMAN_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_TBMAN_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_TBMAN_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_TBMAN_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_TBMAN_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_TBMAN_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_TBMAN_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_TBMAN_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_TBMAN_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_TBMAN_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_TBMAN_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_TBMAN_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_TBMAN_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_TBMAN_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_TBMAN_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_TBMAN_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_TBMAN_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_TBMAN_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_TBMAN_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_TBMAN_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_TBMAN_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_TBMAN_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_TBMAN_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_TBMAN_NSU = 0x1

	// POWMAN: Control whether debugger, DMA, core 0 and core 1 can access POWMAN, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_POWMAN_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_POWMAN_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_POWMAN_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_POWMAN_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_POWMAN_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_POWMAN_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_POWMAN_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_POWMAN_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_POWMAN_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_POWMAN_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_POWMAN_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_POWMAN_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_POWMAN_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_POWMAN_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_POWMAN_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_POWMAN_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_POWMAN_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_POWMAN_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_POWMAN_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_POWMAN_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_POWMAN_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_POWMAN_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_POWMAN_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_POWMAN_NSU = 0x1

	// TRNG: Control whether debugger, DMA, core 0 and core 1 can access TRNG, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_TRNG_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_TRNG_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_TRNG_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_TRNG_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_TRNG_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_TRNG_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_TRNG_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_TRNG_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_TRNG_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_TRNG_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_TRNG_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_TRNG_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_TRNG_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_TRNG_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_TRNG_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_TRNG_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_TRNG_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_TRNG_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_TRNG_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_TRNG_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_TRNG_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_TRNG_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_TRNG_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_TRNG_NSU = 0x1

	// SHA256: Control whether debugger, DMA, core 0 and core 1 can access SHA256, and at what security/privilege levels they can do so. Defaults to Secure, Privileged access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SHA256_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SHA256_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SHA256_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SHA256_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SHA256_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SHA256_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SHA256_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SHA256_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SHA256_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SHA256_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SHA256_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SHA256_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SHA256_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SHA256_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SHA256_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SHA256_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SHA256_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SHA256_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SHA256_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SHA256_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SHA256_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SHA256_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SHA256_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SHA256_NSU = 0x1

	// SYSCFG: Control whether debugger, DMA, core 0 and core 1 can access SYSCFG, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_SYSCFG_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_SYSCFG_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_SYSCFG_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_SYSCFG_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_SYSCFG_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_SYSCFG_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_SYSCFG_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_SYSCFG_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_SYSCFG_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_SYSCFG_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_SYSCFG_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_SYSCFG_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_SYSCFG_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_SYSCFG_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_SYSCFG_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_SYSCFG_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_SYSCFG_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_SYSCFG_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_SYSCFG_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_SYSCFG_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_SYSCFG_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_SYSCFG_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_SYSCFG_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_SYSCFG_NSU = 0x1

	// CLOCKS: Control whether debugger, DMA, core 0 and core 1 can access CLOCKS, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_CLOCKS_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_CLOCKS_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_CLOCKS_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_CLOCKS_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_CLOCKS_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_CLOCKS_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_CLOCKS_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_CLOCKS_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_CLOCKS_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_CLOCKS_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_CLOCKS_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_CLOCKS_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_CLOCKS_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_CLOCKS_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_CLOCKS_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_CLOCKS_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_CLOCKS_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_CLOCKS_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_CLOCKS_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_CLOCKS_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_CLOCKS_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_CLOCKS_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_CLOCKS_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_CLOCKS_NSU = 0x1

	// XOSC: Control whether debugger, DMA, core 0 and core 1 can access XOSC, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_XOSC_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_XOSC_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_XOSC_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_XOSC_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_XOSC_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_XOSC_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_XOSC_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_XOSC_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_XOSC_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_XOSC_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_XOSC_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_XOSC_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_XOSC_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_XOSC_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_XOSC_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_XOSC_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_XOSC_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_XOSC_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_XOSC_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_XOSC_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_XOSC_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_XOSC_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_XOSC_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_XOSC_NSU = 0x1

	// ROSC: Control whether debugger, DMA, core 0 and core 1 can access ROSC, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_ROSC_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_ROSC_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_ROSC_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_ROSC_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_ROSC_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_ROSC_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_ROSC_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_ROSC_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_ROSC_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_ROSC_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_ROSC_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_ROSC_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_ROSC_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_ROSC_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_ROSC_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_ROSC_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_ROSC_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_ROSC_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_ROSC_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_ROSC_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_ROSC_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_ROSC_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_ROSC_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_ROSC_NSU = 0x1

	// PLL_SYS: Control whether debugger, DMA, core 0 and core 1 can access PLL_SYS, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_PLL_SYS_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_PLL_SYS_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_PLL_SYS_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_PLL_SYS_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_PLL_SYS_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_PLL_SYS_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_PLL_SYS_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_PLL_SYS_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_PLL_SYS_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_PLL_SYS_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_PLL_SYS_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_PLL_SYS_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_PLL_SYS_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_PLL_SYS_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_PLL_SYS_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_PLL_SYS_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_PLL_SYS_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_PLL_SYS_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_PLL_SYS_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_PLL_SYS_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_PLL_SYS_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_PLL_SYS_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_PLL_SYS_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_PLL_SYS_NSU = 0x1

	// PLL_USB: Control whether debugger, DMA, core 0 and core 1 can access PLL_USB, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_PLL_USB_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_PLL_USB_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_PLL_USB_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_PLL_USB_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_PLL_USB_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_PLL_USB_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_PLL_USB_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_PLL_USB_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_PLL_USB_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_PLL_USB_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_PLL_USB_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_PLL_USB_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_PLL_USB_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_PLL_USB_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_PLL_USB_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_PLL_USB_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_PLL_USB_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_PLL_USB_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_PLL_USB_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_PLL_USB_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_PLL_USB_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_PLL_USB_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_PLL_USB_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_PLL_USB_NSU = 0x1

	// TICKS: Control whether debugger, DMA, core 0 and core 1 can access TICKS, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_TICKS_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_TICKS_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_TICKS_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_TICKS_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_TICKS_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_TICKS_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_TICKS_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_TICKS_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_TICKS_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_TICKS_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_TICKS_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_TICKS_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_TICKS_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_TICKS_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_TICKS_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_TICKS_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_TICKS_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_TICKS_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_TICKS_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_TICKS_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_TICKS_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_TICKS_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_TICKS_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_TICKS_NSU = 0x1

	// WATCHDOG: Control whether debugger, DMA, core 0 and core 1 can access WATCHDOG, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_WATCHDOG_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_WATCHDOG_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_WATCHDOG_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_WATCHDOG_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_WATCHDOG_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_WATCHDOG_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_WATCHDOG_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_WATCHDOG_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_WATCHDOG_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_WATCHDOG_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_WATCHDOG_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_WATCHDOG_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_WATCHDOG_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_WATCHDOG_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_WATCHDOG_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_WATCHDOG_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_WATCHDOG_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_WATCHDOG_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_WATCHDOG_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_WATCHDOG_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_WATCHDOG_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_WATCHDOG_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_WATCHDOG_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_WATCHDOG_NSU = 0x1

	// RSM: Control whether debugger, DMA, core 0 and core 1 can access RSM, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_RSM_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_RSM_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_RSM_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_RSM_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_RSM_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_RSM_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_RSM_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_RSM_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_RSM_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_RSM_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_RSM_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_RSM_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_RSM_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_RSM_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_RSM_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_RSM_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_RSM_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_RSM_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_RSM_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_RSM_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_RSM_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_RSM_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_RSM_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_RSM_NSU = 0x1

	// XIP_CTRL: Control whether debugger, DMA, core 0 and core 1 can access XIP_CTRL, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_XIP_CTRL_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_XIP_CTRL_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_XIP_CTRL_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_XIP_CTRL_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_XIP_CTRL_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_XIP_CTRL_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_XIP_CTRL_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_XIP_CTRL_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_XIP_CTRL_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_XIP_CTRL_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_XIP_CTRL_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_XIP_CTRL_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_XIP_CTRL_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_XIP_CTRL_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_XIP_CTRL_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_XIP_CTRL_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_XIP_CTRL_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_XIP_CTRL_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_XIP_CTRL_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_XIP_CTRL_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_XIP_CTRL_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_XIP_CTRL_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_XIP_CTRL_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_XIP_CTRL_NSU = 0x1

	// XIP_QMI: Control whether debugger, DMA, core 0 and core 1 can access XIP_QMI, and at what security/privilege levels they can do so. Defaults to Secure, Privileged processor or debug access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_XIP_QMI_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_XIP_QMI_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_XIP_QMI_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_XIP_QMI_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_XIP_QMI_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_XIP_QMI_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_XIP_QMI_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_XIP_QMI_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_XIP_QMI_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_XIP_QMI_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_XIP_QMI_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_XIP_QMI_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_XIP_QMI_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_XIP_QMI_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_XIP_QMI_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_XIP_QMI_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_XIP_QMI_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_XIP_QMI_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_XIP_QMI_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_XIP_QMI_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_XIP_QMI_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_XIP_QMI_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_XIP_QMI_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_XIP_QMI_NSU = 0x1

	// XIP_AUX: Control whether debugger, DMA, core 0 and core 1 can access XIP_AUX, and at what security/privilege levels they can do so. Defaults to Secure, Privileged access only. This register is writable only from a Secure, Privileged processor or debugger, with the exception of the NSU bit, which becomes Non-secure-Privileged-writable when the NSP bit is set.
	// Position of DBG field.
	ACCESSCTRL_XIP_AUX_DBG_Pos = 0x7
	// Bit mask of DBG field.
	ACCESSCTRL_XIP_AUX_DBG_Msk = 0x80
	// Bit DBG.
	ACCESSCTRL_XIP_AUX_DBG = 0x80
	// Position of DMA field.
	ACCESSCTRL_XIP_AUX_DMA_Pos = 0x6
	// Bit mask of DMA field.
	ACCESSCTRL_XIP_AUX_DMA_Msk = 0x40
	// Bit DMA.
	ACCESSCTRL_XIP_AUX_DMA = 0x40
	// Position of CORE1 field.
	ACCESSCTRL_XIP_AUX_CORE1_Pos = 0x5
	// Bit mask of CORE1 field.
	ACCESSCTRL_XIP_AUX_CORE1_Msk = 0x20
	// Bit CORE1.
	ACCESSCTRL_XIP_AUX_CORE1 = 0x20
	// Position of CORE0 field.
	ACCESSCTRL_XIP_AUX_CORE0_Pos = 0x4
	// Bit mask of CORE0 field.
	ACCESSCTRL_XIP_AUX_CORE0_Msk = 0x10
	// Bit CORE0.
	ACCESSCTRL_XIP_AUX_CORE0 = 0x10
	// Position of SP field.
	ACCESSCTRL_XIP_AUX_SP_Pos = 0x3
	// Bit mask of SP field.
	ACCESSCTRL_XIP_AUX_SP_Msk = 0x8
	// Bit SP.
	ACCESSCTRL_XIP_AUX_SP = 0x8
	// Position of SU field.
	ACCESSCTRL_XIP_AUX_SU_Pos = 0x2
	// Bit mask of SU field.
	ACCESSCTRL_XIP_AUX_SU_Msk = 0x4
	// Bit SU.
	ACCESSCTRL_XIP_AUX_SU = 0x4
	// Position of NSP field.
	ACCESSCTRL_XIP_AUX_NSP_Pos = 0x1
	// Bit mask of NSP field.
	ACCESSCTRL_XIP_AUX_NSP_Msk = 0x2
	// Bit NSP.
	ACCESSCTRL_XIP_AUX_NSP = 0x2
	// Position of NSU field.
	ACCESSCTRL_XIP_AUX_NSU_Pos = 0x0
	// Bit mask of NSU field.
	ACCESSCTRL_XIP_AUX_NSU_Msk = 0x1
	// Bit NSU.
	ACCESSCTRL_XIP_AUX_NSU = 0x1
)

// Constants for UART0
const (
	// UARTDR: Data Register, UARTDR
	// Position of OE field.
	UART0_UARTDR_OE_Pos = 0xb
	// Bit mask of OE field.
	UART0_UARTDR_OE_Msk = 0x800
	// Bit OE.
	UART0_UARTDR_OE = 0x800
	// Position of BE field.
	UART0_UARTDR_BE_Pos = 0xa
	// Bit mask of BE field.
	UART0_UARTDR_BE_Msk = 0x400
	// Bit BE.
	UART0_UARTDR_BE = 0x400
	// Position of PE field.
	UART0_UARTDR_PE_Pos = 0x9
	// Bit mask of PE field.
	UART0_UARTDR_PE_Msk = 0x200
	// Bit PE.
	UART0_UARTDR_PE = 0x200
	// Position of FE field.
	UART0_UARTDR_FE_Pos = 0x8
	// Bit mask of FE field.
	UART0_UARTDR_FE_Msk = 0x100
	// Bit FE.
	UART0_UARTDR_FE = 0x100
	// Position of DATA field.
	UART0_UARTDR_DATA_Pos = 0x0
	// Bit mask of DATA field.
	UART0_UARTDR_DATA_Msk = 0xff

	// UARTRSR: Receive Status Register/Error Clear Register, UARTRSR/UARTECR
	// Position of OE field.
	UART0_UARTRSR_OE_Pos = 0x3
	// Bit mask of OE field.
	UART0_UARTRSR_OE_Msk = 0x8
	// Bit OE.
	UART0_UARTRSR_OE = 0x8
	// Position of BE field.
	UART0_UARTRSR_BE_Pos = 0x2
	// Bit mask of BE field.
	UART0_UARTRSR_BE_Msk = 0x4
	// Bit BE.
	UART0_UARTRSR_BE = 0x4
	// Position of PE field.
	UART0_UARTRSR_PE_Pos = 0x1
	// Bit mask of PE field.
	UART0_UARTRSR_PE_Msk = 0x2
	// Bit PE.
	UART0_UARTRSR_PE = 0x2
	// Position of FE field.
	UART0_UARTRSR_FE_Pos = 0x0
	// Bit mask of FE field.
	UART0_UARTRSR_FE_Msk = 0x1
	// Bit FE.
	UART0_UARTRSR_FE = 0x1

	// UARTFR: Flag Register, UARTFR
	// Position of RI field.
	UART0_UARTFR_RI_Pos = 0x8
	// Bit mask of RI field.
	UART0_UARTFR_RI_Msk = 0x100
	// Bit RI.
	UART0_UARTFR_RI = 0x100
	// Position of TXFE field.
	UART0_UARTFR_TXFE_Pos = 0x7
	// Bit mask of TXFE field.
	UART0_UARTFR_TXFE_Msk = 0x80
	// Bit TXFE.
	UART0_UARTFR_TXFE = 0x80
	// Position of RXFF field.
	UART0_UARTFR_RXFF_Pos = 0x6
	// Bit mask of RXFF field.
	UART0_UARTFR_RXFF_Msk = 0x40
	// Bit RXFF.
	UART0_UARTFR_RXFF = 0x40
	// Position of TXFF field.
	UART0_UARTFR_TXFF_Pos = 0x5
	// Bit mask of TXFF field.
	UART0_UARTFR_TXFF_Msk = 0x20
	// Bit TXFF.
	UART0_UARTFR_TXFF = 0x20
	// Position of RXFE field.
	UART0_UARTFR_RXFE_Pos = 0x4
	// Bit mask of RXFE field.
	UART0_UARTFR_RXFE_Msk = 0x10
	// Bit RXFE.
	UART0_UARTFR_RXFE = 0x10
	// Position of BUSY field.
	UART0_UARTFR_BUSY_Pos = 0x3
	// Bit mask of BUSY field.
	UART0_UARTFR_BUSY_Msk = 0x8
	// Bit BUSY.
	UART0_UARTFR_BUSY = 0x8
	// Position of DCD field.
	UART0_UARTFR_DCD_Pos = 0x2
	// Bit mask of DCD field.
	UART0_UARTFR_DCD_Msk = 0x4
	// Bit DCD.
	UART0_UARTFR_DCD = 0x4
	// Position of DSR field.
	UART0_UARTFR_DSR_Pos = 0x1
	// Bit mask of DSR field.
	UART0_UARTFR_DSR_Msk = 0x2
	// Bit DSR.
	UART0_UARTFR_DSR = 0x2
	// Position of CTS field.
	UART0_UARTFR_CTS_Pos = 0x0
	// Bit mask of CTS field.
	UART0_UARTFR_CTS_Msk = 0x1
	// Bit CTS.
	UART0_UARTFR_CTS = 0x1

	// UARTILPR: IrDA Low-Power Counter Register, UARTILPR
	// Position of ILPDVSR field.
	UART0_UARTILPR_ILPDVSR_Pos = 0x0
	// Bit mask of ILPDVSR field.
	UART0_UARTILPR_ILPDVSR_Msk = 0xff

	// UARTIBRD: Integer Baud Rate Register, UARTIBRD
	// Position of BAUD_DIVINT field.
	UART0_UARTIBRD_BAUD_DIVINT_Pos = 0x0
	// Bit mask of BAUD_DIVINT field.
	UART0_UARTIBRD_BAUD_DIVINT_Msk = 0xffff

	// UARTFBRD: Fractional Baud Rate Register, UARTFBRD
	// Position of BAUD_DIVFRAC field.
	UART0_UARTFBRD_BAUD_DIVFRAC_Pos = 0x0
	// Bit mask of BAUD_DIVFRAC field.
	UART0_UARTFBRD_BAUD_DIVFRAC_Msk = 0x3f

	// UARTLCR_H: Line Control Register, UARTLCR_H
	// Position of SPS field.
	UART0_UARTLCR_H_SPS_Pos = 0x7
	// Bit mask of SPS field.
	UART0_UARTLCR_H_SPS_Msk = 0x80
	// Bit SPS.
	UART0_UARTLCR_H_SPS = 0x80
	// Position of WLEN field.
	UART0_UARTLCR_H_WLEN_Pos = 0x5
	// Bit mask of WLEN field.
	UART0_UARTLCR_H_WLEN_Msk = 0x60
	// Position of FEN field.
	UART0_UARTLCR_H_FEN_Pos = 0x4
	// Bit mask of FEN field.
	UART0_UARTLCR_H_FEN_Msk = 0x10
	// Bit FEN.
	UART0_UARTLCR_H_FEN = 0x10
	// Position of STP2 field.
	UART0_UARTLCR_H_STP2_Pos = 0x3
	// Bit mask of STP2 field.
	UART0_UARTLCR_H_STP2_Msk = 0x8
	// Bit STP2.
	UART0_UARTLCR_H_STP2 = 0x8
	// Position of EPS field.
	UART0_UARTLCR_H_EPS_Pos = 0x2
	// Bit mask of EPS field.
	UART0_UARTLCR_H_EPS_Msk = 0x4
	// Bit EPS.
	UART0_UARTLCR_H_EPS = 0x4
	// Position of PEN field.
	UART0_UARTLCR_H_PEN_Pos = 0x1
	// Bit mask of PEN field.
	UART0_UARTLCR_H_PEN_Msk = 0x2
	// Bit PEN.
	UART0_UARTLCR_H_PEN = 0x2
	// Position of BRK field.
	UART0_UARTLCR_H_BRK_Pos = 0x0
	// Bit mask of BRK field.
	UART0_UARTLCR_H_BRK_Msk = 0x1
	// Bit BRK.
	UART0_UARTLCR_H_BRK = 0x1

	// UARTCR: Control Register, UARTCR
	// Position of CTSEN field.
	UART0_UARTCR_CTSEN_Pos = 0xf
	// Bit mask of CTSEN field.
	UART0_UARTCR_CTSEN_Msk = 0x8000
	// Bit CTSEN.
	UART0_UARTCR_CTSEN = 0x8000
	// Position of RTSEN field.
	UART0_UARTCR_RTSEN_Pos = 0xe
	// Bit mask of RTSEN field.
	UART0_UARTCR_RTSEN_Msk = 0x4000
	// Bit RTSEN.
	UART0_UARTCR_RTSEN = 0x4000
	// Position of OUT2 field.
	UART0_UARTCR_OUT2_Pos = 0xd
	// Bit mask of OUT2 field.
	UART0_UARTCR_OUT2_Msk = 0x2000
	// Bit OUT2.
	UART0_UARTCR_OUT2 = 0x2000
	// Position of OUT1 field.
	UART0_UARTCR_OUT1_Pos = 0xc
	// Bit mask of OUT1 field.
	UART0_UARTCR_OUT1_Msk = 0x1000
	// Bit OUT1.
	UART0_UARTCR_OUT1 = 0x1000
	// Position of RTS field.
	UART0_UARTCR_RTS_Pos = 0xb
	// Bit mask of RTS field.
	UART0_UARTCR_RTS_Msk = 0x800
	// Bit RTS.
	UART0_UARTCR_RTS = 0x800
	// Position of DTR field.
	UART0_UARTCR_DTR_Pos = 0xa
	// Bit mask of DTR field.
	UART0_UARTCR_DTR_Msk = 0x400
	// Bit DTR.
	UART0_UARTCR_DTR = 0x400
	// Position of RXE field.
	UART0_UARTCR_RXE_Pos = 0x9
	// Bit mask of RXE field.
	UART0_UARTCR_RXE_Msk = 0x200
	// Bit RXE.
	UART0_UARTCR_RXE = 0x200
	// Position of TXE field.
	UART0_UARTCR_TXE_Pos = 0x8
	// Bit mask of TXE field.
	UART0_UARTCR_TXE_Msk = 0x100
	// Bit TXE.
	UART0_UARTCR_TXE = 0x100
	// Position of LBE field.
	UART0_UARTCR_LBE_Pos = 0x7
	// Bit mask of LBE field.
	UART0_UARTCR_LBE_Msk = 0x80
	// Bit LBE.
	UART0_UARTCR_LBE = 0x80
	// Position of SIRLP field.
	UART0_UARTCR_SIRLP_Pos = 0x2
	// Bit mask of SIRLP field.
	UART0_UARTCR_SIRLP_Msk = 0x4
	// Bit SIRLP.
	UART0_UARTCR_SIRLP = 0x4
	// Position of SIREN field.
	UART0_UARTCR_SIREN_Pos = 0x1
	// Bit mask of SIREN field.
	UART0_UARTCR_SIREN_Msk = 0x2
	// Bit SIREN.
	UART0_UARTCR_SIREN = 0x2
	// Position of UARTEN field.
	UART0_UARTCR_UARTEN_Pos = 0x0
	// Bit mask of UARTEN field.
	UART0_UARTCR_UARTEN_Msk = 0x1
	// Bit UARTEN.
	UART0_UARTCR_UARTEN = 0x1

	// UARTIFLS: Interrupt FIFO Level Select Register, UARTIFLS
	// Position of RXIFLSEL field.
	UART0_UARTIFLS_RXIFLSEL_Pos = 0x3
	// Bit mask of RXIFLSEL field.
	UART0_UARTIFLS_RXIFLSEL_Msk = 0x38
	// Position of TXIFLSEL field.
	UART0_UARTIFLS_TXIFLSEL_Pos = 0x0
	// Bit mask of TXIFLSEL field.
	UART0_UARTIFLS_TXIFLSEL_Msk = 0x7

	// UARTIMSC: Interrupt Mask Set/Clear Register, UARTIMSC
	// Position of OEIM field.
	UART0_UARTIMSC_OEIM_Pos = 0xa
	// Bit mask of OEIM field.
	UART0_UARTIMSC_OEIM_Msk = 0x400
	// Bit OEIM.
	UART0_UARTIMSC_OEIM = 0x400
	// Position of BEIM field.
	UART0_UARTIMSC_BEIM_Pos = 0x9
	// Bit mask of BEIM field.
	UART0_UARTIMSC_BEIM_Msk = 0x200
	// Bit BEIM.
	UART0_UARTIMSC_BEIM = 0x200
	// Position of PEIM field.
	UART0_UARTIMSC_PEIM_Pos = 0x8
	// Bit mask of PEIM field.
	UART0_UARTIMSC_PEIM_Msk = 0x100
	// Bit PEIM.
	UART0_UARTIMSC_PEIM = 0x100
	// Position of FEIM field.
	UART0_UARTIMSC_FEIM_Pos = 0x7
	// Bit mask of FEIM field.
	UART0_UARTIMSC_FEIM_Msk = 0x80
	// Bit FEIM.
	UART0_UARTIMSC_FEIM = 0x80
	// Position of RTIM field.
	UART0_UARTIMSC_RTIM_Pos = 0x6
	// Bit mask of RTIM field.
	UART0_UARTIMSC_RTIM_Msk = 0x40
	// Bit RTIM.
	UART0_UARTIMSC_RTIM = 0x40
	// Position of TXIM field.
	UART0_UARTIMSC_TXIM_Pos = 0x5
	// Bit mask of TXIM field.
	UART0_UARTIMSC_TXIM_Msk = 0x20
	// Bit TXIM.
	UART0_UARTIMSC_TXIM = 0x20
	// Position of RXIM field.
	UART0_UARTIMSC_RXIM_Pos = 0x4
	// Bit mask of RXIM field.
	UART0_UARTIMSC_RXIM_Msk = 0x10
	// Bit RXIM.
	UART0_UARTIMSC_RXIM = 0x10
	// Position of DSRMIM field.
	UART0_UARTIMSC_DSRMIM_Pos = 0x3
	// Bit mask of DSRMIM field.
	UART0_UARTIMSC_DSRMIM_Msk = 0x8
	// Bit DSRMIM.
	UART0_UARTIMSC_DSRMIM = 0x8
	// Position of DCDMIM field.
	UART0_UARTIMSC_DCDMIM_Pos = 0x2
	// Bit mask of DCDMIM field.
	UART0_UARTIMSC_DCDMIM_Msk = 0x4
	// Bit DCDMIM.
	UART0_UARTIMSC_DCDMIM = 0x4
	// Position of CTSMIM field.
	UART0_UARTIMSC_CTSMIM_Pos = 0x1
	// Bit mask of CTSMIM field.
	UART0_UARTIMSC_CTSMIM_Msk = 0x2
	// Bit CTSMIM.
	UART0_UARTIMSC_CTSMIM = 0x2
	// Position of RIMIM field.
	UART0_UARTIMSC_RIMIM_Pos = 0x0
	// Bit mask of RIMIM field.
	UART0_UARTIMSC_RIMIM_Msk = 0x1
	// Bit RIMIM.
	UART0_UARTIMSC_RIMIM = 0x1

	// UARTRIS: Raw Interrupt Status Register, UARTRIS
	// Position of OERIS field.
	UART0_UARTRIS_OERIS_Pos = 0xa
	// Bit mask of OERIS field.
	UART0_UARTRIS_OERIS_Msk = 0x400
	// Bit OERIS.
	UART0_UARTRIS_OERIS = 0x400
	// Position of BERIS field.
	UART0_UARTRIS_BERIS_Pos = 0x9
	// Bit mask of BERIS field.
	UART0_UARTRIS_BERIS_Msk = 0x200
	// Bit BERIS.
	UART0_UARTRIS_BERIS = 0x200
	// Position of PERIS field.
	UART0_UARTRIS_PERIS_Pos = 0x8
	// Bit mask of PERIS field.
	UART0_UARTRIS_PERIS_Msk = 0x100
	// Bit PERIS.
	UART0_UARTRIS_PERIS = 0x100
	// Position of FERIS field.
	UART0_UARTRIS_FERIS_Pos = 0x7
	// Bit mask of FERIS field.
	UART0_UARTRIS_FERIS_Msk = 0x80
	// Bit FERIS.
	UART0_UARTRIS_FERIS = 0x80
	// Position of RTRIS field.
	UART0_UARTRIS_RTRIS_Pos = 0x6
	// Bit mask of RTRIS field.
	UART0_UARTRIS_RTRIS_Msk = 0x40
	// Bit RTRIS.
	UART0_UARTRIS_RTRIS = 0x40
	// Position of TXRIS field.
	UART0_UARTRIS_TXRIS_Pos = 0x5
	// Bit mask of TXRIS field.
	UART0_UARTRIS_TXRIS_Msk = 0x20
	// Bit TXRIS.
	UART0_UARTRIS_TXRIS = 0x20
	// Position of RXRIS field.
	UART0_UARTRIS_RXRIS_Pos = 0x4
	// Bit mask of RXRIS field.
	UART0_UARTRIS_RXRIS_Msk = 0x10
	// Bit RXRIS.
	UART0_UARTRIS_RXRIS = 0x10
	// Position of DSRRMIS field.
	UART0_UARTRIS_DSRRMIS_Pos = 0x3
	// Bit mask of DSRRMIS field.
	UART0_UARTRIS_DSRRMIS_Msk = 0x8
	// Bit DSRRMIS.
	UART0_UARTRIS_DSRRMIS = 0x8
	// Position of DCDRMIS field.
	UART0_UARTRIS_DCDRMIS_Pos = 0x2
	// Bit mask of DCDRMIS field.
	UART0_UARTRIS_DCDRMIS_Msk = 0x4
	// Bit DCDRMIS.
	UART0_UARTRIS_DCDRMIS = 0x4
	// Position of CTSRMIS field.
	UART0_UARTRIS_CTSRMIS_Pos = 0x1
	// Bit mask of CTSRMIS field.
	UART0_UARTRIS_CTSRMIS_Msk = 0x2
	// Bit CTSRMIS.
	UART0_UARTRIS_CTSRMIS = 0x2
	// Position of RIRMIS field.
	UART0_UARTRIS_RIRMIS_Pos = 0x0
	// Bit mask of RIRMIS field.
	UART0_UARTRIS_RIRMIS_Msk = 0x1
	// Bit RIRMIS.
	UART0_UARTRIS_RIRMIS = 0x1

	// UARTMIS: Masked Interrupt Status Register, UARTMIS
	// Position of OEMIS field.
	UART0_UARTMIS_OEMIS_Pos = 0xa
	// Bit mask of OEMIS field.
	UART0_UARTMIS_OEMIS_Msk = 0x400
	// Bit OEMIS.
	UART0_UARTMIS_OEMIS = 0x400
	// Position of BEMIS field.
	UART0_UARTMIS_BEMIS_Pos = 0x9
	// Bit mask of BEMIS field.
	UART0_UARTMIS_BEMIS_Msk = 0x200
	// Bit BEMIS.
	UART0_UARTMIS_BEMIS = 0x200
	// Position of PEMIS field.
	UART0_UARTMIS_PEMIS_Pos = 0x8
	// Bit mask of PEMIS field.
	UART0_UARTMIS_PEMIS_Msk = 0x100
	// Bit PEMIS.
	UART0_UARTMIS_PEMIS = 0x100
	// Position of FEMIS field.
	UART0_UARTMIS_FEMIS_Pos = 0x7
	// Bit mask of FEMIS field.
	UART0_UARTMIS_FEMIS_Msk = 0x80
	// Bit FEMIS.
	UART0_UARTMIS_FEMIS = 0x80
	// Position of RTMIS field.
	UART0_UARTMIS_RTMIS_Pos = 0x6
	// Bit mask of RTMIS field.
	UART0_UARTMIS_RTMIS_Msk = 0x40
	// Bit RTMIS.
	UART0_UARTMIS_RTMIS = 0x40
	// Position of TXMIS field.
	UART0_UARTMIS_TXMIS_Pos = 0x5
	// Bit mask of TXMIS field.
	UART0_UARTMIS_TXMIS_Msk = 0x20
	// Bit TXMIS.
	UART0_UARTMIS_TXMIS = 0x20
	// Position of RXMIS field.
	UART0_UARTMIS_RXMIS_Pos = 0x4
	// Bit mask of RXMIS field.
	UART0_UARTMIS_RXMIS_Msk = 0x10
	// Bit RXMIS.
	UART0_UARTMIS_RXMIS = 0x10
	// Position of DSRMMIS field.
	UART0_UARTMIS_DSRMMIS_Pos = 0x3
	// Bit mask of DSRMMIS field.
	UART0_UARTMIS_DSRMMIS_Msk = 0x8
	// Bit DSRMMIS.
	UART0_UARTMIS_DSRMMIS = 0x8
	// Position of DCDMMIS field.
	UART0_UARTMIS_DCDMMIS_Pos = 0x2
	// Bit mask of DCDMMIS field.
	UART0_UARTMIS_DCDMMIS_Msk = 0x4
	// Bit DCDMMIS.
	UART0_UARTMIS_DCDMMIS = 0x4
	// Position of CTSMMIS field.
	UART0_UARTMIS_CTSMMIS_Pos = 0x1
	// Bit mask of CTSMMIS field.
	UART0_UARTMIS_CTSMMIS_Msk = 0x2
	// Bit CTSMMIS.
	UART0_UARTMIS_CTSMMIS = 0x2
	// Position of RIMMIS field.
	UART0_UARTMIS_RIMMIS_Pos = 0x0
	// Bit mask of RIMMIS field.
	UART0_UARTMIS_RIMMIS_Msk = 0x1
	// Bit RIMMIS.
	UART0_UARTMIS_RIMMIS = 0x1

	// UARTICR: Interrupt Clear Register, UARTICR
	// Position of OEIC field.
	UART0_UARTICR_OEIC_Pos = 0xa
	// Bit mask of OEIC field.
	UART0_UARTICR_OEIC_Msk = 0x400
	// Bit OEIC.
	UART0_UARTICR_OEIC = 0x400
	// Position of BEIC field.
	UART0_UARTICR_BEIC_Pos = 0x9
	// Bit mask of BEIC field.
	UART0_UARTICR_BEIC_Msk = 0x200
	// Bit BEIC.
	UART0_UARTICR_BEIC = 0x200
	// Position of PEIC field.
	UART0_UARTICR_PEIC_Pos = 0x8
	// Bit mask of PEIC field.
	UART0_UARTICR_PEIC_Msk = 0x100
	// Bit PEIC.
	UART0_UARTICR_PEIC = 0x100
	// Position of FEIC field.
	UART0_UARTICR_FEIC_Pos = 0x7
	// Bit mask of FEIC field.
	UART0_UARTICR_FEIC_Msk = 0x80
	// Bit FEIC.
	UART0_UARTICR_FEIC = 0x80
	// Position of RTIC field.
	UART0_UARTICR_RTIC_Pos = 0x6
	// Bit mask of RTIC field.
	UART0_UARTICR_RTIC_Msk = 0x40
	// Bit RTIC.
	UART0_UARTICR_RTIC = 0x40
	// Position of TXIC field.
	UART0_UARTICR_TXIC_Pos = 0x5
	// Bit mask of TXIC field.
	UART0_UARTICR_TXIC_Msk = 0x20
	// Bit TXIC.
	UART0_UARTICR_TXIC = 0x20
	// Position of RXIC field.
	UART0_UARTICR_RXIC_Pos = 0x4
	// Bit mask of RXIC field.
	UART0_UARTICR_RXIC_Msk = 0x10
	// Bit RXIC.
	UART0_UARTICR_RXIC = 0x10
	// Position of DSRMIC field.
	UART0_UARTICR_DSRMIC_Pos = 0x3
	// Bit mask of DSRMIC field.
	UART0_UARTICR_DSRMIC_Msk = 0x8
	// Bit DSRMIC.
	UART0_UARTICR_DSRMIC = 0x8
	// Position of DCDMIC field.
	UART0_UARTICR_DCDMIC_Pos = 0x2
	// Bit mask of DCDMIC field.
	UART0_UARTICR_DCDMIC_Msk = 0x4
	// Bit DCDMIC.
	UART0_UARTICR_DCDMIC = 0x4
	// Position of CTSMIC field.
	UART0_UARTICR_CTSMIC_Pos = 0x1
	// Bit mask of CTSMIC field.
	UART0_UARTICR_CTSMIC_Msk = 0x2
	// Bit CTSMIC.
	UART0_UARTICR_CTSMIC = 0x2
	// Position of RIMIC field.
	UART0_UARTICR_RIMIC_Pos = 0x0
	// Bit mask of RIMIC field.
	UART0_UARTICR_RIMIC_Msk = 0x1
	// Bit RIMIC.
	UART0_UARTICR_RIMIC = 0x1

	// UARTDMACR: DMA Control Register, UARTDMACR
	// Position of DMAONERR field.
	UART0_UARTDMACR_DMAONERR_Pos = 0x2
	// Bit mask of DMAONERR field.
	UART0_UARTDMACR_DMAONERR_Msk = 0x4
	// Bit DMAONERR.
	UART0_UARTDMACR_DMAONERR = 0x4
	// Position of TXDMAE field.
	UART0_UARTDMACR_TXDMAE_Pos = 0x1
	// Bit mask of TXDMAE field.
	UART0_UARTDMACR_TXDMAE_Msk = 0x2
	// Bit TXDMAE.
	UART0_UARTDMACR_TXDMAE = 0x2
	// Position of RXDMAE field.
	UART0_UARTDMACR_RXDMAE_Pos = 0x0
	// Bit mask of RXDMAE field.
	UART0_UARTDMACR_RXDMAE_Msk = 0x1
	// Bit RXDMAE.
	UART0_UARTDMACR_RXDMAE = 0x1

	// UARTPERIPHID0: UARTPeriphID0 Register
	// Position of PARTNUMBER0 field.
	UART0_UARTPERIPHID0_PARTNUMBER0_Pos = 0x0
	// Bit mask of PARTNUMBER0 field.
	UART0_UARTPERIPHID0_PARTNUMBER0_Msk = 0xff

	// UARTPERIPHID1: UARTPeriphID1 Register
	// Position of DESIGNER0 field.
	UART0_UARTPERIPHID1_DESIGNER0_Pos = 0x4
	// Bit mask of DESIGNER0 field.
	UART0_UARTPERIPHID1_DESIGNER0_Msk = 0xf0
	// Position of PARTNUMBER1 field.
	UART0_UARTPERIPHID1_PARTNUMBER1_Pos = 0x0
	// Bit mask of PARTNUMBER1 field.
	UART0_UARTPERIPHID1_PARTNUMBER1_Msk = 0xf

	// UARTPERIPHID2: UARTPeriphID2 Register
	// Position of REVISION field.
	UART0_UARTPERIPHID2_REVISION_Pos = 0x4
	// Bit mask of REVISION field.
	UART0_UARTPERIPHID2_REVISION_Msk = 0xf0
	// Position of DESIGNER1 field.
	UART0_UARTPERIPHID2_DESIGNER1_Pos = 0x0
	// Bit mask of DESIGNER1 field.
	UART0_UARTPERIPHID2_DESIGNER1_Msk = 0xf

	// UARTPERIPHID3: UARTPeriphID3 Register
	// Position of CONFIGURATION field.
	UART0_UARTPERIPHID3_CONFIGURATION_Pos = 0x0
	// Bit mask of CONFIGURATION field.
	UART0_UARTPERIPHID3_CONFIGURATION_Msk = 0xff

	// UARTPCELLID0: UARTPCellID0 Register
	// Position of UARTPCELLID0 field.
	UART0_UARTPCELLID0_UARTPCELLID0_Pos = 0x0
	// Bit mask of UARTPCELLID0 field.
	UART0_UARTPCELLID0_UARTPCELLID0_Msk = 0xff

	// UARTPCELLID1: UARTPCellID1 Register
	// Position of UARTPCELLID1 field.
	UART0_UARTPCELLID1_UARTPCELLID1_Pos = 0x0
	// Bit mask of UARTPCELLID1 field.
	UART0_UARTPCELLID1_UARTPCELLID1_Msk = 0xff

	// UARTPCELLID2: UARTPCellID2 Register
	// Position of UARTPCELLID2 field.
	UART0_UARTPCELLID2_UARTPCELLID2_Pos = 0x0
	// Bit mask of UARTPCELLID2 field.
	UART0_UARTPCELLID2_UARTPCELLID2_Msk = 0xff

	// UARTPCELLID3: UARTPCellID3 Register
	// Position of UARTPCELLID3 field.
	UART0_UARTPCELLID3_UARTPCELLID3_Pos = 0x0
	// Bit mask of UARTPCELLID3 field.
	UART0_UARTPCELLID3_UARTPCELLID3_Msk = 0xff
)

// Constants for ROSC
const (
	// CTRL: Ring Oscillator control
	// Position of ENABLE field.
	ROSC_CTRL_ENABLE_Pos = 0xc
	// Bit mask of ENABLE field.
	ROSC_CTRL_ENABLE_Msk     = 0xfff000
	ROSC_CTRL_ENABLE_DISABLE = 0xd1e
	ROSC_CTRL_ENABLE_ENABLE  = 0xfab
	// Position of FREQ_RANGE field.
	ROSC_CTRL_FREQ_RANGE_Pos = 0x0
	// Bit mask of FREQ_RANGE field.
	ROSC_CTRL_FREQ_RANGE_Msk     = 0xfff
	ROSC_CTRL_FREQ_RANGE_LOW     = 0xfa4
	ROSC_CTRL_FREQ_RANGE_MEDIUM  = 0xfa5
	ROSC_CTRL_FREQ_RANGE_HIGH    = 0xfa7
	ROSC_CTRL_FREQ_RANGE_TOOHIGH = 0xfa6

	// FREQA: The FREQA & FREQB registers control the frequency by controlling the drive strength of each stage The drive strength has 4 levels determined by the number of bits set Increasing the number of bits set increases the drive strength and increases the oscillation frequency 0 bits set is the default drive strength 1 bit set doubles the drive strength 2 bits set triples drive strength 3 bits set quadruples drive strength For frequency randomisation set both DS0_RANDOM=1 & DS1_RANDOM=1
	// Position of PASSWD field.
	ROSC_FREQA_PASSWD_Pos = 0x10
	// Bit mask of PASSWD field.
	ROSC_FREQA_PASSWD_Msk  = 0xffff0000
	ROSC_FREQA_PASSWD_PASS = 0x9696
	// Position of DS3 field.
	ROSC_FREQA_DS3_Pos = 0xc
	// Bit mask of DS3 field.
	ROSC_FREQA_DS3_Msk = 0x7000
	// Position of DS2 field.
	ROSC_FREQA_DS2_Pos = 0x8
	// Bit mask of DS2 field.
	ROSC_FREQA_DS2_Msk = 0x700
	// Position of DS1_RANDOM field.
	ROSC_FREQA_DS1_RANDOM_Pos = 0x7
	// Bit mask of DS1_RANDOM field.
	ROSC_FREQA_DS1_RANDOM_Msk = 0x80
	// Bit DS1_RANDOM.
	ROSC_FREQA_DS1_RANDOM = 0x80
	// Position of DS1 field.
	ROSC_FREQA_DS1_Pos = 0x4
	// Bit mask of DS1 field.
	ROSC_FREQA_DS1_Msk = 0x70
	// Position of DS0_RANDOM field.
	ROSC_FREQA_DS0_RANDOM_Pos = 0x3
	// Bit mask of DS0_RANDOM field.
	ROSC_FREQA_DS0_RANDOM_Msk = 0x8
	// Bit DS0_RANDOM.
	ROSC_FREQA_DS0_RANDOM = 0x8
	// Position of DS0 field.
	ROSC_FREQA_DS0_Pos = 0x0
	// Bit mask of DS0 field.
	ROSC_FREQA_DS0_Msk = 0x7

	// FREQB: For a detailed description see freqa register
	// Position of PASSWD field.
	ROSC_FREQB_PASSWD_Pos = 0x10
	// Bit mask of PASSWD field.
	ROSC_FREQB_PASSWD_Msk  = 0xffff0000
	ROSC_FREQB_PASSWD_PASS = 0x9696
	// Position of DS7 field.
	ROSC_FREQB_DS7_Pos = 0xc
	// Bit mask of DS7 field.
	ROSC_FREQB_DS7_Msk = 0x7000
	// Position of DS6 field.
	ROSC_FREQB_DS6_Pos = 0x8
	// Bit mask of DS6 field.
	ROSC_FREQB_DS6_Msk = 0x700
	// Position of DS5 field.
	ROSC_FREQB_DS5_Pos = 0x4
	// Bit mask of DS5 field.
	ROSC_FREQB_DS5_Msk = 0x70
	// Position of DS4 field.
	ROSC_FREQB_DS4_Pos = 0x0
	// Bit mask of DS4 field.
	ROSC_FREQB_DS4_Msk = 0x7

	// RANDOM: Loads a value to the LFSR randomiser
	// Position of SEED field.
	ROSC_RANDOM_SEED_Pos = 0x0
	// Bit mask of SEED field.
	ROSC_RANDOM_SEED_Msk = 0xffffffff

	// DORMANT: Ring Oscillator pause control
	// Position of DORMANT field.
	ROSC_DORMANT_DORMANT_Pos = 0x0
	// Bit mask of DORMANT field.
	ROSC_DORMANT_DORMANT_Msk     = 0xffffffff
	ROSC_DORMANT_DORMANT_DORMANT = 0x636f6d61
	ROSC_DORMANT_DORMANT_WAKE    = 0x77616b65

	// DIV: Controls the output divider
	// Position of DIV field.
	ROSC_DIV_DIV_Pos = 0x0
	// Bit mask of DIV field.
	ROSC_DIV_DIV_Msk  = 0xffff
	ROSC_DIV_DIV_PASS = 0xaa00

	// PHASE: Controls the phase shifted output
	// Position of PASSWD field.
	ROSC_PHASE_PASSWD_Pos = 0x4
	// Bit mask of PASSWD field.
	ROSC_PHASE_PASSWD_Msk = 0xff0
	// Position of ENABLE field.
	ROSC_PHASE_ENABLE_Pos = 0x3
	// Bit mask of ENABLE field.
	ROSC_PHASE_ENABLE_Msk = 0x8
	// Bit ENABLE.
	ROSC_PHASE_ENABLE = 0x8
	// Position of FLIP field.
	ROSC_PHASE_FLIP_Pos = 0x2
	// Bit mask of FLIP field.
	ROSC_PHASE_FLIP_Msk = 0x4
	// Bit FLIP.
	ROSC_PHASE_FLIP = 0x4
	// Position of SHIFT field.
	ROSC_PHASE_SHIFT_Pos = 0x0
	// Bit mask of SHIFT field.
	ROSC_PHASE_SHIFT_Msk = 0x3

	// STATUS: Ring Oscillator Status
	// Position of STABLE field.
	ROSC_STATUS_STABLE_Pos = 0x1f
	// Bit mask of STABLE field.
	ROSC_STATUS_STABLE_Msk = 0x80000000
	// Bit STABLE.
	ROSC_STATUS_STABLE = 0x80000000
	// Position of BADWRITE field.
	ROSC_STATUS_BADWRITE_Pos = 0x18
	// Bit mask of BADWRITE field.
	ROSC_STATUS_BADWRITE_Msk = 0x1000000
	// Bit BADWRITE.
	ROSC_STATUS_BADWRITE = 0x1000000
	// Position of DIV_RUNNING field.
	ROSC_STATUS_DIV_RUNNING_Pos = 0x10
	// Bit mask of DIV_RUNNING field.
	ROSC_STATUS_DIV_RUNNING_Msk = 0x10000
	// Bit DIV_RUNNING.
	ROSC_STATUS_DIV_RUNNING = 0x10000
	// Position of ENABLED field.
	ROSC_STATUS_ENABLED_Pos = 0xc
	// Bit mask of ENABLED field.
	ROSC_STATUS_ENABLED_Msk = 0x1000
	// Bit ENABLED.
	ROSC_STATUS_ENABLED = 0x1000

	// RANDOMBIT: This just reads the state of the oscillator output so randomness is compromised if the ring oscillator is stopped or run at a harmonic of the bus frequency
	// Position of RANDOMBIT field.
	ROSC_RANDOMBIT_RANDOMBIT_Pos = 0x0
	// Bit mask of RANDOMBIT field.
	ROSC_RANDOMBIT_RANDOMBIT_Msk = 0x1
	// Bit RANDOMBIT.
	ROSC_RANDOMBIT_RANDOMBIT = 0x1

	// COUNT: A down counter running at the ROSC frequency which counts to zero and stops. To start the counter write a non-zero value. Can be used for short software pauses when setting up time sensitive hardware.
	// Position of COUNT field.
	ROSC_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	ROSC_COUNT_COUNT_Msk = 0xffff
)

// Constants for POWMAN: Controls vreg, bor, lposc, chip resets & xosc startup, powman and provides scratch register for general use and for bootcode use
const (
	// BADPASSWD: Indicates a bad password has been used
	// Position of BADPASSWD field.
	POWMAN_BADPASSWD_BADPASSWD_Pos = 0x0
	// Bit mask of BADPASSWD field.
	POWMAN_BADPASSWD_BADPASSWD_Msk = 0x1
	// Bit BADPASSWD.
	POWMAN_BADPASSWD_BADPASSWD = 0x1

	// VREG_CTRL: Voltage Regulator Control
	// Position of RST_N field.
	POWMAN_VREG_CTRL_RST_N_Pos = 0xf
	// Bit mask of RST_N field.
	POWMAN_VREG_CTRL_RST_N_Msk = 0x8000
	// Bit RST_N.
	POWMAN_VREG_CTRL_RST_N = 0x8000
	// Position of UNLOCK field.
	POWMAN_VREG_CTRL_UNLOCK_Pos = 0xd
	// Bit mask of UNLOCK field.
	POWMAN_VREG_CTRL_UNLOCK_Msk = 0x2000
	// Bit UNLOCK.
	POWMAN_VREG_CTRL_UNLOCK = 0x2000
	// Position of ISOLATE field.
	POWMAN_VREG_CTRL_ISOLATE_Pos = 0xc
	// Bit mask of ISOLATE field.
	POWMAN_VREG_CTRL_ISOLATE_Msk = 0x1000
	// Bit ISOLATE.
	POWMAN_VREG_CTRL_ISOLATE = 0x1000
	// Position of DISABLE_VOLTAGE_LIMIT field.
	POWMAN_VREG_CTRL_DISABLE_VOLTAGE_LIMIT_Pos = 0x8
	// Bit mask of DISABLE_VOLTAGE_LIMIT field.
	POWMAN_VREG_CTRL_DISABLE_VOLTAGE_LIMIT_Msk = 0x100
	// Bit DISABLE_VOLTAGE_LIMIT.
	POWMAN_VREG_CTRL_DISABLE_VOLTAGE_LIMIT = 0x100
	// Position of HT_TH field.
	POWMAN_VREG_CTRL_HT_TH_Pos = 0x4
	// Bit mask of HT_TH field.
	POWMAN_VREG_CTRL_HT_TH_Msk = 0x70

	// VREG_STS: Voltage Regulator Status
	// Position of VOUT_OK field.
	POWMAN_VREG_STS_VOUT_OK_Pos = 0x4
	// Bit mask of VOUT_OK field.
	POWMAN_VREG_STS_VOUT_OK_Msk = 0x10
	// Bit VOUT_OK.
	POWMAN_VREG_STS_VOUT_OK = 0x10
	// Position of STARTUP field.
	POWMAN_VREG_STS_STARTUP_Pos = 0x0
	// Bit mask of STARTUP field.
	POWMAN_VREG_STS_STARTUP_Msk = 0x1
	// Bit STARTUP.
	POWMAN_VREG_STS_STARTUP = 0x1

	// VREG: Voltage Regulator Settings
	// Position of UPDATE_IN_PROGRESS field.
	POWMAN_VREG_UPDATE_IN_PROGRESS_Pos = 0xf
	// Bit mask of UPDATE_IN_PROGRESS field.
	POWMAN_VREG_UPDATE_IN_PROGRESS_Msk = 0x8000
	// Bit UPDATE_IN_PROGRESS.
	POWMAN_VREG_UPDATE_IN_PROGRESS = 0x8000
	// Position of VSEL field.
	POWMAN_VREG_VSEL_Pos = 0x4
	// Bit mask of VSEL field.
	POWMAN_VREG_VSEL_Msk = 0x1f0
	// Position of HIZ field.
	POWMAN_VREG_HIZ_Pos = 0x1
	// Bit mask of HIZ field.
	POWMAN_VREG_HIZ_Msk = 0x2
	// Bit HIZ.
	POWMAN_VREG_HIZ = 0x2

	// VREG_LP_ENTRY: Voltage Regulator Low Power Entry Settings
	// Position of VSEL field.
	POWMAN_VREG_LP_ENTRY_VSEL_Pos = 0x4
	// Bit mask of VSEL field.
	POWMAN_VREG_LP_ENTRY_VSEL_Msk = 0x1f0
	// Position of MODE field.
	POWMAN_VREG_LP_ENTRY_MODE_Pos = 0x2
	// Bit mask of MODE field.
	POWMAN_VREG_LP_ENTRY_MODE_Msk = 0x4
	// Bit MODE.
	POWMAN_VREG_LP_ENTRY_MODE = 0x4
	// Position of HIZ field.
	POWMAN_VREG_LP_ENTRY_HIZ_Pos = 0x1
	// Bit mask of HIZ field.
	POWMAN_VREG_LP_ENTRY_HIZ_Msk = 0x2
	// Bit HIZ.
	POWMAN_VREG_LP_ENTRY_HIZ = 0x2

	// VREG_LP_EXIT: Voltage Regulator Low Power Exit Settings
	// Position of VSEL field.
	POWMAN_VREG_LP_EXIT_VSEL_Pos = 0x4
	// Bit mask of VSEL field.
	POWMAN_VREG_LP_EXIT_VSEL_Msk = 0x1f0
	// Position of MODE field.
	POWMAN_VREG_LP_EXIT_MODE_Pos = 0x2
	// Bit mask of MODE field.
	POWMAN_VREG_LP_EXIT_MODE_Msk = 0x4
	// Bit MODE.
	POWMAN_VREG_LP_EXIT_MODE = 0x4
	// Position of HIZ field.
	POWMAN_VREG_LP_EXIT_HIZ_Pos = 0x1
	// Bit mask of HIZ field.
	POWMAN_VREG_LP_EXIT_HIZ_Msk = 0x2
	// Bit HIZ.
	POWMAN_VREG_LP_EXIT_HIZ = 0x2

	// BOD_CTRL: Brown-out Detection Control
	// Position of ISOLATE field.
	POWMAN_BOD_CTRL_ISOLATE_Pos = 0xc
	// Bit mask of ISOLATE field.
	POWMAN_BOD_CTRL_ISOLATE_Msk = 0x1000
	// Bit ISOLATE.
	POWMAN_BOD_CTRL_ISOLATE = 0x1000

	// BOD: Brown-out Detection Settings
	// Position of VSEL field.
	POWMAN_BOD_VSEL_Pos = 0x4
	// Bit mask of VSEL field.
	POWMAN_BOD_VSEL_Msk = 0x1f0
	// Position of EN field.
	POWMAN_BOD_EN_Pos = 0x0
	// Bit mask of EN field.
	POWMAN_BOD_EN_Msk = 0x1
	// Bit EN.
	POWMAN_BOD_EN = 0x1

	// BOD_LP_ENTRY: Brown-out Detection Low Power Entry Settings
	// Position of VSEL field.
	POWMAN_BOD_LP_ENTRY_VSEL_Pos = 0x4
	// Bit mask of VSEL field.
	POWMAN_BOD_LP_ENTRY_VSEL_Msk = 0x1f0
	// Position of EN field.
	POWMAN_BOD_LP_ENTRY_EN_Pos = 0x0
	// Bit mask of EN field.
	POWMAN_BOD_LP_ENTRY_EN_Msk = 0x1
	// Bit EN.
	POWMAN_BOD_LP_ENTRY_EN = 0x1

	// BOD_LP_EXIT: Brown-out Detection Low Power Exit Settings
	// Position of VSEL field.
	POWMAN_BOD_LP_EXIT_VSEL_Pos = 0x4
	// Bit mask of VSEL field.
	POWMAN_BOD_LP_EXIT_VSEL_Msk = 0x1f0
	// Position of EN field.
	POWMAN_BOD_LP_EXIT_EN_Pos = 0x0
	// Bit mask of EN field.
	POWMAN_BOD_LP_EXIT_EN_Msk = 0x1
	// Bit EN.
	POWMAN_BOD_LP_EXIT_EN = 0x1

	// LPOSC: Low power oscillator control register.
	// Position of TRIM field.
	POWMAN_LPOSC_TRIM_Pos = 0x4
	// Bit mask of TRIM field.
	POWMAN_LPOSC_TRIM_Msk = 0x3f0
	// Position of MODE field.
	POWMAN_LPOSC_MODE_Pos = 0x0
	// Bit mask of MODE field.
	POWMAN_LPOSC_MODE_Msk = 0x3

	// CHIP_RESET: Chip reset control and status
	// Position of HAD_WATCHDOG_RESET_RSM field.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_RSM_Pos = 0x1c
	// Bit mask of HAD_WATCHDOG_RESET_RSM field.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_RSM_Msk = 0x10000000
	// Bit HAD_WATCHDOG_RESET_RSM.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_RSM = 0x10000000
	// Position of HAD_HZD_SYS_RESET_REQ field.
	POWMAN_CHIP_RESET_HAD_HZD_SYS_RESET_REQ_Pos = 0x1b
	// Bit mask of HAD_HZD_SYS_RESET_REQ field.
	POWMAN_CHIP_RESET_HAD_HZD_SYS_RESET_REQ_Msk = 0x8000000
	// Bit HAD_HZD_SYS_RESET_REQ.
	POWMAN_CHIP_RESET_HAD_HZD_SYS_RESET_REQ = 0x8000000
	// Position of HAD_GLITCH_DETECT field.
	POWMAN_CHIP_RESET_HAD_GLITCH_DETECT_Pos = 0x1a
	// Bit mask of HAD_GLITCH_DETECT field.
	POWMAN_CHIP_RESET_HAD_GLITCH_DETECT_Msk = 0x4000000
	// Bit HAD_GLITCH_DETECT.
	POWMAN_CHIP_RESET_HAD_GLITCH_DETECT = 0x4000000
	// Position of HAD_SWCORE_PD field.
	POWMAN_CHIP_RESET_HAD_SWCORE_PD_Pos = 0x19
	// Bit mask of HAD_SWCORE_PD field.
	POWMAN_CHIP_RESET_HAD_SWCORE_PD_Msk = 0x2000000
	// Bit HAD_SWCORE_PD.
	POWMAN_CHIP_RESET_HAD_SWCORE_PD = 0x2000000
	// Position of HAD_WATCHDOG_RESET_SWCORE field.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_SWCORE_Pos = 0x18
	// Bit mask of HAD_WATCHDOG_RESET_SWCORE field.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_SWCORE_Msk = 0x1000000
	// Bit HAD_WATCHDOG_RESET_SWCORE.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_SWCORE = 0x1000000
	// Position of HAD_WATCHDOG_RESET_POWMAN field.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_POWMAN_Pos = 0x17
	// Bit mask of HAD_WATCHDOG_RESET_POWMAN field.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_POWMAN_Msk = 0x800000
	// Bit HAD_WATCHDOG_RESET_POWMAN.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_POWMAN = 0x800000
	// Position of HAD_WATCHDOG_RESET_POWMAN_ASYNC field.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_POWMAN_ASYNC_Pos = 0x16
	// Bit mask of HAD_WATCHDOG_RESET_POWMAN_ASYNC field.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_POWMAN_ASYNC_Msk = 0x400000
	// Bit HAD_WATCHDOG_RESET_POWMAN_ASYNC.
	POWMAN_CHIP_RESET_HAD_WATCHDOG_RESET_POWMAN_ASYNC = 0x400000
	// Position of HAD_RESCUE field.
	POWMAN_CHIP_RESET_HAD_RESCUE_Pos = 0x15
	// Bit mask of HAD_RESCUE field.
	POWMAN_CHIP_RESET_HAD_RESCUE_Msk = 0x200000
	// Bit HAD_RESCUE.
	POWMAN_CHIP_RESET_HAD_RESCUE = 0x200000
	// Position of HAD_DP_RESET_REQ field.
	POWMAN_CHIP_RESET_HAD_DP_RESET_REQ_Pos = 0x13
	// Bit mask of HAD_DP_RESET_REQ field.
	POWMAN_CHIP_RESET_HAD_DP_RESET_REQ_Msk = 0x80000
	// Bit HAD_DP_RESET_REQ.
	POWMAN_CHIP_RESET_HAD_DP_RESET_REQ = 0x80000
	// Position of HAD_RUN_LOW field.
	POWMAN_CHIP_RESET_HAD_RUN_LOW_Pos = 0x12
	// Bit mask of HAD_RUN_LOW field.
	POWMAN_CHIP_RESET_HAD_RUN_LOW_Msk = 0x40000
	// Bit HAD_RUN_LOW.
	POWMAN_CHIP_RESET_HAD_RUN_LOW = 0x40000
	// Position of HAD_BOR field.
	POWMAN_CHIP_RESET_HAD_BOR_Pos = 0x11
	// Bit mask of HAD_BOR field.
	POWMAN_CHIP_RESET_HAD_BOR_Msk = 0x20000
	// Bit HAD_BOR.
	POWMAN_CHIP_RESET_HAD_BOR = 0x20000
	// Position of HAD_POR field.
	POWMAN_CHIP_RESET_HAD_POR_Pos = 0x10
	// Bit mask of HAD_POR field.
	POWMAN_CHIP_RESET_HAD_POR_Msk = 0x10000
	// Bit HAD_POR.
	POWMAN_CHIP_RESET_HAD_POR = 0x10000
	// Position of RESCUE_FLAG field.
	POWMAN_CHIP_RESET_RESCUE_FLAG_Pos = 0x4
	// Bit mask of RESCUE_FLAG field.
	POWMAN_CHIP_RESET_RESCUE_FLAG_Msk = 0x10
	// Bit RESCUE_FLAG.
	POWMAN_CHIP_RESET_RESCUE_FLAG = 0x10
	// Position of DOUBLE_TAP field.
	POWMAN_CHIP_RESET_DOUBLE_TAP_Pos = 0x0
	// Bit mask of DOUBLE_TAP field.
	POWMAN_CHIP_RESET_DOUBLE_TAP_Msk = 0x1
	// Bit DOUBLE_TAP.
	POWMAN_CHIP_RESET_DOUBLE_TAP = 0x1

	// WDSEL: Allows a watchdog reset to reset the internal state of powman in addition to the power-on state machine (PSM). Note that powman ignores watchdog resets that do not select at least the CLOCKS stage or earlier stages in the PSM. If using these bits, it's recommended to set PSM_WDSEL to all-ones in addition to the desired bits in this register. Failing to select CLOCKS or earlier will result in the POWMAN_WDSEL register having no effect.
	// Position of RESET_RSM field.
	POWMAN_WDSEL_RESET_RSM_Pos = 0xc
	// Bit mask of RESET_RSM field.
	POWMAN_WDSEL_RESET_RSM_Msk = 0x1000
	// Bit RESET_RSM.
	POWMAN_WDSEL_RESET_RSM = 0x1000
	// Position of RESET_SWCORE field.
	POWMAN_WDSEL_RESET_SWCORE_Pos = 0x8
	// Bit mask of RESET_SWCORE field.
	POWMAN_WDSEL_RESET_SWCORE_Msk = 0x100
	// Bit RESET_SWCORE.
	POWMAN_WDSEL_RESET_SWCORE = 0x100
	// Position of RESET_POWMAN field.
	POWMAN_WDSEL_RESET_POWMAN_Pos = 0x4
	// Bit mask of RESET_POWMAN field.
	POWMAN_WDSEL_RESET_POWMAN_Msk = 0x10
	// Bit RESET_POWMAN.
	POWMAN_WDSEL_RESET_POWMAN = 0x10
	// Position of RESET_POWMAN_ASYNC field.
	POWMAN_WDSEL_RESET_POWMAN_ASYNC_Pos = 0x0
	// Bit mask of RESET_POWMAN_ASYNC field.
	POWMAN_WDSEL_RESET_POWMAN_ASYNC_Msk = 0x1
	// Bit RESET_POWMAN_ASYNC.
	POWMAN_WDSEL_RESET_POWMAN_ASYNC = 0x1

	// SEQ_CFG: For configuration of the power sequencer Writes are ignored while POWMAN_STATE_CHANGING=1
	// Position of USING_FAST_POWCK field.
	POWMAN_SEQ_CFG_USING_FAST_POWCK_Pos = 0x14
	// Bit mask of USING_FAST_POWCK field.
	POWMAN_SEQ_CFG_USING_FAST_POWCK_Msk = 0x100000
	// Bit USING_FAST_POWCK.
	POWMAN_SEQ_CFG_USING_FAST_POWCK = 0x100000
	// Position of USING_BOD_LP field.
	POWMAN_SEQ_CFG_USING_BOD_LP_Pos = 0x11
	// Bit mask of USING_BOD_LP field.
	POWMAN_SEQ_CFG_USING_BOD_LP_Msk = 0x20000
	// Bit USING_BOD_LP.
	POWMAN_SEQ_CFG_USING_BOD_LP = 0x20000
	// Position of USING_VREG_LP field.
	POWMAN_SEQ_CFG_USING_VREG_LP_Pos = 0x10
	// Bit mask of USING_VREG_LP field.
	POWMAN_SEQ_CFG_USING_VREG_LP_Msk = 0x10000
	// Bit USING_VREG_LP.
	POWMAN_SEQ_CFG_USING_VREG_LP = 0x10000
	// Position of USE_FAST_POWCK field.
	POWMAN_SEQ_CFG_USE_FAST_POWCK_Pos = 0xc
	// Bit mask of USE_FAST_POWCK field.
	POWMAN_SEQ_CFG_USE_FAST_POWCK_Msk = 0x1000
	// Bit USE_FAST_POWCK.
	POWMAN_SEQ_CFG_USE_FAST_POWCK = 0x1000
	// Position of RUN_LPOSC_IN_LP field.
	POWMAN_SEQ_CFG_RUN_LPOSC_IN_LP_Pos = 0x8
	// Bit mask of RUN_LPOSC_IN_LP field.
	POWMAN_SEQ_CFG_RUN_LPOSC_IN_LP_Msk = 0x100
	// Bit RUN_LPOSC_IN_LP.
	POWMAN_SEQ_CFG_RUN_LPOSC_IN_LP = 0x100
	// Position of USE_BOD_HP field.
	POWMAN_SEQ_CFG_USE_BOD_HP_Pos = 0x7
	// Bit mask of USE_BOD_HP field.
	POWMAN_SEQ_CFG_USE_BOD_HP_Msk = 0x80
	// Bit USE_BOD_HP.
	POWMAN_SEQ_CFG_USE_BOD_HP = 0x80
	// Position of USE_BOD_LP field.
	POWMAN_SEQ_CFG_USE_BOD_LP_Pos = 0x6
	// Bit mask of USE_BOD_LP field.
	POWMAN_SEQ_CFG_USE_BOD_LP_Msk = 0x40
	// Bit USE_BOD_LP.
	POWMAN_SEQ_CFG_USE_BOD_LP = 0x40
	// Position of USE_VREG_HP field.
	POWMAN_SEQ_CFG_USE_VREG_HP_Pos = 0x5
	// Bit mask of USE_VREG_HP field.
	POWMAN_SEQ_CFG_USE_VREG_HP_Msk = 0x20
	// Bit USE_VREG_HP.
	POWMAN_SEQ_CFG_USE_VREG_HP = 0x20
	// Position of USE_VREG_LP field.
	POWMAN_SEQ_CFG_USE_VREG_LP_Pos = 0x4
	// Bit mask of USE_VREG_LP field.
	POWMAN_SEQ_CFG_USE_VREG_LP_Msk = 0x10
	// Bit USE_VREG_LP.
	POWMAN_SEQ_CFG_USE_VREG_LP = 0x10
	// Position of HW_PWRUP_SRAM0 field.
	POWMAN_SEQ_CFG_HW_PWRUP_SRAM0_Pos = 0x1
	// Bit mask of HW_PWRUP_SRAM0 field.
	POWMAN_SEQ_CFG_HW_PWRUP_SRAM0_Msk = 0x2
	// Bit HW_PWRUP_SRAM0.
	POWMAN_SEQ_CFG_HW_PWRUP_SRAM0 = 0x2
	// Position of HW_PWRUP_SRAM1 field.
	POWMAN_SEQ_CFG_HW_PWRUP_SRAM1_Pos = 0x0
	// Bit mask of HW_PWRUP_SRAM1 field.
	POWMAN_SEQ_CFG_HW_PWRUP_SRAM1_Msk = 0x1
	// Bit HW_PWRUP_SRAM1.
	POWMAN_SEQ_CFG_HW_PWRUP_SRAM1 = 0x1

	// STATE: This register controls the power state of the 4 power domains. The current power state is indicated in POWMAN_STATE_CURRENT which is read-only. To change the state, write to POWMAN_STATE_REQ. The coding of POWMAN_STATE_CURRENT & POWMAN_STATE_REQ corresponds to the power states defined in the datasheet: bit 3 = SWCORE bit 2 = XIP cache bit 1 = SRAM0 bit 0 = SRAM1 0 = powered up 1 = powered down When POWMAN_STATE_REQ is written, the POWMAN_STATE_WAITING flag is set while the Power Manager determines what is required. If an invalid transition is requested the Power Manager will still register the request in POWMAN_STATE_REQ but will also set the POWMAN_BAD_REQ flag. It will then implement the power-up requests and ignore the power down requests. To do nothing would risk entering an unrecoverable lock-up state. Invalid requests are: any combination of power up and power down requests any request that results in swcore boing powered and xip unpowered If the request is to power down the switched-core domain then POWMAN_STATE_WAITING stays active until the processors halt. During this time the POWMAN_STATE_REQ field can be re-written to change or cancel the request. When the power state transition begins the POWMAN_STATE_WAITING_flag is cleared, the POWMAN_STATE_CHANGING flag is set and POWMAN register writes are ignored until the transition completes.
	// Position of CHANGING field.
	POWMAN_STATE_CHANGING_Pos = 0xd
	// Bit mask of CHANGING field.
	POWMAN_STATE_CHANGING_Msk = 0x2000
	// Bit CHANGING.
	POWMAN_STATE_CHANGING = 0x2000
	// Position of WAITING field.
	POWMAN_STATE_WAITING_Pos = 0xc
	// Bit mask of WAITING field.
	POWMAN_STATE_WAITING_Msk = 0x1000
	// Bit WAITING.
	POWMAN_STATE_WAITING = 0x1000
	// Position of BAD_HW_REQ field.
	POWMAN_STATE_BAD_HW_REQ_Pos = 0xb
	// Bit mask of BAD_HW_REQ field.
	POWMAN_STATE_BAD_HW_REQ_Msk = 0x800
	// Bit BAD_HW_REQ.
	POWMAN_STATE_BAD_HW_REQ = 0x800
	// Position of BAD_SW_REQ field.
	POWMAN_STATE_BAD_SW_REQ_Pos = 0xa
	// Bit mask of BAD_SW_REQ field.
	POWMAN_STATE_BAD_SW_REQ_Msk = 0x400
	// Bit BAD_SW_REQ.
	POWMAN_STATE_BAD_SW_REQ = 0x400
	// Position of PWRUP_WHILE_WAITING field.
	POWMAN_STATE_PWRUP_WHILE_WAITING_Pos = 0x9
	// Bit mask of PWRUP_WHILE_WAITING field.
	POWMAN_STATE_PWRUP_WHILE_WAITING_Msk = 0x200
	// Bit PWRUP_WHILE_WAITING.
	POWMAN_STATE_PWRUP_WHILE_WAITING = 0x200
	// Position of REQ_IGNORED field.
	POWMAN_STATE_REQ_IGNORED_Pos = 0x8
	// Bit mask of REQ_IGNORED field.
	POWMAN_STATE_REQ_IGNORED_Msk = 0x100
	// Bit REQ_IGNORED.
	POWMAN_STATE_REQ_IGNORED = 0x100
	// Position of REQ field.
	POWMAN_STATE_REQ_Pos = 0x4
	// Bit mask of REQ field.
	POWMAN_STATE_REQ_Msk = 0xf0
	// Position of CURRENT field.
	POWMAN_STATE_CURRENT_Pos = 0x0
	// Bit mask of CURRENT field.
	POWMAN_STATE_CURRENT_Msk = 0xf

	// POW_FASTDIV
	// Position of POW_FASTDIV field.
	POWMAN_POW_FASTDIV_POW_FASTDIV_Pos = 0x0
	// Bit mask of POW_FASTDIV field.
	POWMAN_POW_FASTDIV_POW_FASTDIV_Msk = 0x7ff

	// POW_DELAY: power state machine delays
	// Position of SRAM_STEP field.
	POWMAN_POW_DELAY_SRAM_STEP_Pos = 0x8
	// Bit mask of SRAM_STEP field.
	POWMAN_POW_DELAY_SRAM_STEP_Msk = 0xff00
	// Position of XIP_STEP field.
	POWMAN_POW_DELAY_XIP_STEP_Pos = 0x4
	// Bit mask of XIP_STEP field.
	POWMAN_POW_DELAY_XIP_STEP_Msk = 0xf0
	// Position of SWCORE_STEP field.
	POWMAN_POW_DELAY_SWCORE_STEP_Pos = 0x0
	// Bit mask of SWCORE_STEP field.
	POWMAN_POW_DELAY_SWCORE_STEP_Msk = 0xf

	// EXT_CTRL0: Configures a gpio as a power mode aware control output
	// Position of LP_EXIT_STATE field.
	POWMAN_EXT_CTRL0_LP_EXIT_STATE_Pos = 0xe
	// Bit mask of LP_EXIT_STATE field.
	POWMAN_EXT_CTRL0_LP_EXIT_STATE_Msk = 0x4000
	// Bit LP_EXIT_STATE.
	POWMAN_EXT_CTRL0_LP_EXIT_STATE = 0x4000
	// Position of LP_ENTRY_STATE field.
	POWMAN_EXT_CTRL0_LP_ENTRY_STATE_Pos = 0xd
	// Bit mask of LP_ENTRY_STATE field.
	POWMAN_EXT_CTRL0_LP_ENTRY_STATE_Msk = 0x2000
	// Bit LP_ENTRY_STATE.
	POWMAN_EXT_CTRL0_LP_ENTRY_STATE = 0x2000
	// Position of INIT_STATE field.
	POWMAN_EXT_CTRL0_INIT_STATE_Pos = 0xc
	// Bit mask of INIT_STATE field.
	POWMAN_EXT_CTRL0_INIT_STATE_Msk = 0x1000
	// Bit INIT_STATE.
	POWMAN_EXT_CTRL0_INIT_STATE = 0x1000
	// Position of INIT field.
	POWMAN_EXT_CTRL0_INIT_Pos = 0x8
	// Bit mask of INIT field.
	POWMAN_EXT_CTRL0_INIT_Msk = 0x100
	// Bit INIT.
	POWMAN_EXT_CTRL0_INIT = 0x100
	// Position of GPIO_SELECT field.
	POWMAN_EXT_CTRL0_GPIO_SELECT_Pos = 0x0
	// Bit mask of GPIO_SELECT field.
	POWMAN_EXT_CTRL0_GPIO_SELECT_Msk = 0x3f

	// EXT_CTRL1: Configures a gpio as a power mode aware control output
	// Position of LP_EXIT_STATE field.
	POWMAN_EXT_CTRL1_LP_EXIT_STATE_Pos = 0xe
	// Bit mask of LP_EXIT_STATE field.
	POWMAN_EXT_CTRL1_LP_EXIT_STATE_Msk = 0x4000
	// Bit LP_EXIT_STATE.
	POWMAN_EXT_CTRL1_LP_EXIT_STATE = 0x4000
	// Position of LP_ENTRY_STATE field.
	POWMAN_EXT_CTRL1_LP_ENTRY_STATE_Pos = 0xd
	// Bit mask of LP_ENTRY_STATE field.
	POWMAN_EXT_CTRL1_LP_ENTRY_STATE_Msk = 0x2000
	// Bit LP_ENTRY_STATE.
	POWMAN_EXT_CTRL1_LP_ENTRY_STATE = 0x2000
	// Position of INIT_STATE field.
	POWMAN_EXT_CTRL1_INIT_STATE_Pos = 0xc
	// Bit mask of INIT_STATE field.
	POWMAN_EXT_CTRL1_INIT_STATE_Msk = 0x1000
	// Bit INIT_STATE.
	POWMAN_EXT_CTRL1_INIT_STATE = 0x1000
	// Position of INIT field.
	POWMAN_EXT_CTRL1_INIT_Pos = 0x8
	// Bit mask of INIT field.
	POWMAN_EXT_CTRL1_INIT_Msk = 0x100
	// Bit INIT.
	POWMAN_EXT_CTRL1_INIT = 0x100
	// Position of GPIO_SELECT field.
	POWMAN_EXT_CTRL1_GPIO_SELECT_Pos = 0x0
	// Bit mask of GPIO_SELECT field.
	POWMAN_EXT_CTRL1_GPIO_SELECT_Msk = 0x3f

	// EXT_TIME_REF: Select a GPIO to use as a time reference, the source can be used to drive the low power clock at 32kHz, or to provide a 1ms tick to the timer, or provide a 1Hz tick to the timer. The tick selection is controlled by the POWMAN_TIMER register.
	// Position of DRIVE_LPCK field.
	POWMAN_EXT_TIME_REF_DRIVE_LPCK_Pos = 0x4
	// Bit mask of DRIVE_LPCK field.
	POWMAN_EXT_TIME_REF_DRIVE_LPCK_Msk = 0x10
	// Bit DRIVE_LPCK.
	POWMAN_EXT_TIME_REF_DRIVE_LPCK = 0x10
	// Position of SOURCE_SEL field.
	POWMAN_EXT_TIME_REF_SOURCE_SEL_Pos = 0x0
	// Bit mask of SOURCE_SEL field.
	POWMAN_EXT_TIME_REF_SOURCE_SEL_Msk = 0x3

	// LPOSC_FREQ_KHZ_INT: Informs the AON Timer of the integer component of the clock frequency when running off the LPOSC.
	// Position of LPOSC_FREQ_KHZ_INT field.
	POWMAN_LPOSC_FREQ_KHZ_INT_LPOSC_FREQ_KHZ_INT_Pos = 0x0
	// Bit mask of LPOSC_FREQ_KHZ_INT field.
	POWMAN_LPOSC_FREQ_KHZ_INT_LPOSC_FREQ_KHZ_INT_Msk = 0x3f

	// LPOSC_FREQ_KHZ_FRAC: Informs the AON Timer of the fractional component of the clock frequency when running off the LPOSC.
	// Position of LPOSC_FREQ_KHZ_FRAC field.
	POWMAN_LPOSC_FREQ_KHZ_FRAC_LPOSC_FREQ_KHZ_FRAC_Pos = 0x0
	// Bit mask of LPOSC_FREQ_KHZ_FRAC field.
	POWMAN_LPOSC_FREQ_KHZ_FRAC_LPOSC_FREQ_KHZ_FRAC_Msk = 0xffff

	// XOSC_FREQ_KHZ_INT: Informs the AON Timer of the integer component of the clock frequency when running off the XOSC.
	// Position of XOSC_FREQ_KHZ_INT field.
	POWMAN_XOSC_FREQ_KHZ_INT_XOSC_FREQ_KHZ_INT_Pos = 0x0
	// Bit mask of XOSC_FREQ_KHZ_INT field.
	POWMAN_XOSC_FREQ_KHZ_INT_XOSC_FREQ_KHZ_INT_Msk = 0xffff

	// XOSC_FREQ_KHZ_FRAC: Informs the AON Timer of the fractional component of the clock frequency when running off the XOSC.
	// Position of XOSC_FREQ_KHZ_FRAC field.
	POWMAN_XOSC_FREQ_KHZ_FRAC_XOSC_FREQ_KHZ_FRAC_Pos = 0x0
	// Bit mask of XOSC_FREQ_KHZ_FRAC field.
	POWMAN_XOSC_FREQ_KHZ_FRAC_XOSC_FREQ_KHZ_FRAC_Msk = 0xffff

	// SET_TIME_63TO48
	// Position of SET_TIME_63TO48 field.
	POWMAN_SET_TIME_63TO48_SET_TIME_63TO48_Pos = 0x0
	// Bit mask of SET_TIME_63TO48 field.
	POWMAN_SET_TIME_63TO48_SET_TIME_63TO48_Msk = 0xffff

	// SET_TIME_47TO32
	// Position of SET_TIME_47TO32 field.
	POWMAN_SET_TIME_47TO32_SET_TIME_47TO32_Pos = 0x0
	// Bit mask of SET_TIME_47TO32 field.
	POWMAN_SET_TIME_47TO32_SET_TIME_47TO32_Msk = 0xffff

	// SET_TIME_31TO16
	// Position of SET_TIME_31TO16 field.
	POWMAN_SET_TIME_31TO16_SET_TIME_31TO16_Pos = 0x0
	// Bit mask of SET_TIME_31TO16 field.
	POWMAN_SET_TIME_31TO16_SET_TIME_31TO16_Msk = 0xffff

	// SET_TIME_15TO0
	// Position of SET_TIME_15TO0 field.
	POWMAN_SET_TIME_15TO0_SET_TIME_15TO0_Pos = 0x0
	// Bit mask of SET_TIME_15TO0 field.
	POWMAN_SET_TIME_15TO0_SET_TIME_15TO0_Msk = 0xffff

	// READ_TIME_UPPER
	// Position of READ_TIME_UPPER field.
	POWMAN_READ_TIME_UPPER_READ_TIME_UPPER_Pos = 0x0
	// Bit mask of READ_TIME_UPPER field.
	POWMAN_READ_TIME_UPPER_READ_TIME_UPPER_Msk = 0xffffffff

	// READ_TIME_LOWER
	// Position of READ_TIME_LOWER field.
	POWMAN_READ_TIME_LOWER_READ_TIME_LOWER_Pos = 0x0
	// Bit mask of READ_TIME_LOWER field.
	POWMAN_READ_TIME_LOWER_READ_TIME_LOWER_Msk = 0xffffffff

	// ALARM_TIME_63TO48
	// Position of ALARM_TIME_63TO48 field.
	POWMAN_ALARM_TIME_63TO48_ALARM_TIME_63TO48_Pos = 0x0
	// Bit mask of ALARM_TIME_63TO48 field.
	POWMAN_ALARM_TIME_63TO48_ALARM_TIME_63TO48_Msk = 0xffff

	// ALARM_TIME_47TO32
	// Position of ALARM_TIME_47TO32 field.
	POWMAN_ALARM_TIME_47TO32_ALARM_TIME_47TO32_Pos = 0x0
	// Bit mask of ALARM_TIME_47TO32 field.
	POWMAN_ALARM_TIME_47TO32_ALARM_TIME_47TO32_Msk = 0xffff

	// ALARM_TIME_31TO16
	// Position of ALARM_TIME_31TO16 field.
	POWMAN_ALARM_TIME_31TO16_ALARM_TIME_31TO16_Pos = 0x0
	// Bit mask of ALARM_TIME_31TO16 field.
	POWMAN_ALARM_TIME_31TO16_ALARM_TIME_31TO16_Msk = 0xffff

	// ALARM_TIME_15TO0
	// Position of ALARM_TIME_15TO0 field.
	POWMAN_ALARM_TIME_15TO0_ALARM_TIME_15TO0_Pos = 0x0
	// Bit mask of ALARM_TIME_15TO0 field.
	POWMAN_ALARM_TIME_15TO0_ALARM_TIME_15TO0_Msk = 0xffff

	// TIMER
	// Position of USING_GPIO_1HZ field.
	POWMAN_TIMER_USING_GPIO_1HZ_Pos = 0x13
	// Bit mask of USING_GPIO_1HZ field.
	POWMAN_TIMER_USING_GPIO_1HZ_Msk = 0x80000
	// Bit USING_GPIO_1HZ.
	POWMAN_TIMER_USING_GPIO_1HZ = 0x80000
	// Position of USING_GPIO_1KHZ field.
	POWMAN_TIMER_USING_GPIO_1KHZ_Pos = 0x12
	// Bit mask of USING_GPIO_1KHZ field.
	POWMAN_TIMER_USING_GPIO_1KHZ_Msk = 0x40000
	// Bit USING_GPIO_1KHZ.
	POWMAN_TIMER_USING_GPIO_1KHZ = 0x40000
	// Position of USING_LPOSC field.
	POWMAN_TIMER_USING_LPOSC_Pos = 0x11
	// Bit mask of USING_LPOSC field.
	POWMAN_TIMER_USING_LPOSC_Msk = 0x20000
	// Bit USING_LPOSC.
	POWMAN_TIMER_USING_LPOSC = 0x20000
	// Position of USING_XOSC field.
	POWMAN_TIMER_USING_XOSC_Pos = 0x10
	// Bit mask of USING_XOSC field.
	POWMAN_TIMER_USING_XOSC_Msk = 0x10000
	// Bit USING_XOSC.
	POWMAN_TIMER_USING_XOSC = 0x10000
	// Position of USE_GPIO_1HZ field.
	POWMAN_TIMER_USE_GPIO_1HZ_Pos = 0xd
	// Bit mask of USE_GPIO_1HZ field.
	POWMAN_TIMER_USE_GPIO_1HZ_Msk = 0x2000
	// Bit USE_GPIO_1HZ.
	POWMAN_TIMER_USE_GPIO_1HZ = 0x2000
	// Position of USE_GPIO_1KHZ field.
	POWMAN_TIMER_USE_GPIO_1KHZ_Pos = 0xa
	// Bit mask of USE_GPIO_1KHZ field.
	POWMAN_TIMER_USE_GPIO_1KHZ_Msk = 0x400
	// Bit USE_GPIO_1KHZ.
	POWMAN_TIMER_USE_GPIO_1KHZ = 0x400
	// Position of USE_XOSC field.
	POWMAN_TIMER_USE_XOSC_Pos = 0x9
	// Bit mask of USE_XOSC field.
	POWMAN_TIMER_USE_XOSC_Msk = 0x200
	// Bit USE_XOSC.
	POWMAN_TIMER_USE_XOSC = 0x200
	// Position of USE_LPOSC field.
	POWMAN_TIMER_USE_LPOSC_Pos = 0x8
	// Bit mask of USE_LPOSC field.
	POWMAN_TIMER_USE_LPOSC_Msk = 0x100
	// Bit USE_LPOSC.
	POWMAN_TIMER_USE_LPOSC = 0x100
	// Position of ALARM field.
	POWMAN_TIMER_ALARM_Pos = 0x6
	// Bit mask of ALARM field.
	POWMAN_TIMER_ALARM_Msk = 0x40
	// Bit ALARM.
	POWMAN_TIMER_ALARM = 0x40
	// Position of PWRUP_ON_ALARM field.
	POWMAN_TIMER_PWRUP_ON_ALARM_Pos = 0x5
	// Bit mask of PWRUP_ON_ALARM field.
	POWMAN_TIMER_PWRUP_ON_ALARM_Msk = 0x20
	// Bit PWRUP_ON_ALARM.
	POWMAN_TIMER_PWRUP_ON_ALARM = 0x20
	// Position of ALARM_ENAB field.
	POWMAN_TIMER_ALARM_ENAB_Pos = 0x4
	// Bit mask of ALARM_ENAB field.
	POWMAN_TIMER_ALARM_ENAB_Msk = 0x10
	// Bit ALARM_ENAB.
	POWMAN_TIMER_ALARM_ENAB = 0x10
	// Position of CLEAR field.
	POWMAN_TIMER_CLEAR_Pos = 0x2
	// Bit mask of CLEAR field.
	POWMAN_TIMER_CLEAR_Msk = 0x4
	// Bit CLEAR.
	POWMAN_TIMER_CLEAR = 0x4
	// Position of RUN field.
	POWMAN_TIMER_RUN_Pos = 0x1
	// Bit mask of RUN field.
	POWMAN_TIMER_RUN_Msk = 0x2
	// Bit RUN.
	POWMAN_TIMER_RUN = 0x2
	// Position of NONSEC_WRITE field.
	POWMAN_TIMER_NONSEC_WRITE_Pos = 0x0
	// Bit mask of NONSEC_WRITE field.
	POWMAN_TIMER_NONSEC_WRITE_Msk = 0x1
	// Bit NONSEC_WRITE.
	POWMAN_TIMER_NONSEC_WRITE = 0x1

	// PWRUP0: 4 GPIO powerup events can be configured to wake the chip up from a low power state. The pwrups are level/edge sensitive and can be set to trigger on a high/rising or low/falling event The number of gpios available depends on the package option. An invalid selection will be ignored source = 0 selects gpio0 . . source = 47 selects gpio47 source = 48 selects qspi_ss source = 49 selects qspi_sd0 source = 50 selects qspi_sd1 source = 51 selects qspi_sd2 source = 52 selects qspi_sd3 source = 53 selects qspi_sclk level = 0 triggers the pwrup when the source is low level = 1 triggers the pwrup when the source is high
	// Position of RAW_STATUS field.
	POWMAN_PWRUP0_RAW_STATUS_Pos = 0xa
	// Bit mask of RAW_STATUS field.
	POWMAN_PWRUP0_RAW_STATUS_Msk = 0x400
	// Bit RAW_STATUS.
	POWMAN_PWRUP0_RAW_STATUS = 0x400
	// Position of STATUS field.
	POWMAN_PWRUP0_STATUS_Pos = 0x9
	// Bit mask of STATUS field.
	POWMAN_PWRUP0_STATUS_Msk = 0x200
	// Bit STATUS.
	POWMAN_PWRUP0_STATUS = 0x200
	// Position of MODE field.
	POWMAN_PWRUP0_MODE_Pos = 0x8
	// Bit mask of MODE field.
	POWMAN_PWRUP0_MODE_Msk = 0x100
	// Bit MODE.
	POWMAN_PWRUP0_MODE       = 0x100
	POWMAN_PWRUP0_MODE_LEVEL = 0x0
	POWMAN_PWRUP0_MODE_EDGE  = 0x1
	// Position of DIRECTION field.
	POWMAN_PWRUP0_DIRECTION_Pos = 0x7
	// Bit mask of DIRECTION field.
	POWMAN_PWRUP0_DIRECTION_Msk = 0x80
	// Bit DIRECTION.
	POWMAN_PWRUP0_DIRECTION             = 0x80
	POWMAN_PWRUP0_DIRECTION_LOW_FALLING = 0x0
	POWMAN_PWRUP0_DIRECTION_HIGH_RISING = 0x1
	// Position of ENABLE field.
	POWMAN_PWRUP0_ENABLE_Pos = 0x6
	// Bit mask of ENABLE field.
	POWMAN_PWRUP0_ENABLE_Msk = 0x40
	// Bit ENABLE.
	POWMAN_PWRUP0_ENABLE = 0x40
	// Position of SOURCE field.
	POWMAN_PWRUP0_SOURCE_Pos = 0x0
	// Bit mask of SOURCE field.
	POWMAN_PWRUP0_SOURCE_Msk = 0x3f

	// PWRUP1: 4 GPIO powerup events can be configured to wake the chip up from a low power state. The pwrups are level/edge sensitive and can be set to trigger on a high/rising or low/falling event The number of gpios available depends on the package option. An invalid selection will be ignored source = 0 selects gpio0 . . source = 47 selects gpio47 source = 48 selects qspi_ss source = 49 selects qspi_sd0 source = 50 selects qspi_sd1 source = 51 selects qspi_sd2 source = 52 selects qspi_sd3 source = 53 selects qspi_sclk level = 0 triggers the pwrup when the source is low level = 1 triggers the pwrup when the source is high
	// Position of RAW_STATUS field.
	POWMAN_PWRUP1_RAW_STATUS_Pos = 0xa
	// Bit mask of RAW_STATUS field.
	POWMAN_PWRUP1_RAW_STATUS_Msk = 0x400
	// Bit RAW_STATUS.
	POWMAN_PWRUP1_RAW_STATUS = 0x400
	// Position of STATUS field.
	POWMAN_PWRUP1_STATUS_Pos = 0x9
	// Bit mask of STATUS field.
	POWMAN_PWRUP1_STATUS_Msk = 0x200
	// Bit STATUS.
	POWMAN_PWRUP1_STATUS = 0x200
	// Position of MODE field.
	POWMAN_PWRUP1_MODE_Pos = 0x8
	// Bit mask of MODE field.
	POWMAN_PWRUP1_MODE_Msk = 0x100
	// Bit MODE.
	POWMAN_PWRUP1_MODE       = 0x100
	POWMAN_PWRUP1_MODE_LEVEL = 0x0
	POWMAN_PWRUP1_MODE_EDGE  = 0x1
	// Position of DIRECTION field.
	POWMAN_PWRUP1_DIRECTION_Pos = 0x7
	// Bit mask of DIRECTION field.
	POWMAN_PWRUP1_DIRECTION_Msk = 0x80
	// Bit DIRECTION.
	POWMAN_PWRUP1_DIRECTION             = 0x80
	POWMAN_PWRUP1_DIRECTION_LOW_FALLING = 0x0
	POWMAN_PWRUP1_DIRECTION_HIGH_RISING = 0x1
	// Position of ENABLE field.
	POWMAN_PWRUP1_ENABLE_Pos = 0x6
	// Bit mask of ENABLE field.
	POWMAN_PWRUP1_ENABLE_Msk = 0x40
	// Bit ENABLE.
	POWMAN_PWRUP1_ENABLE = 0x40
	// Position of SOURCE field.
	POWMAN_PWRUP1_SOURCE_Pos = 0x0
	// Bit mask of SOURCE field.
	POWMAN_PWRUP1_SOURCE_Msk = 0x3f

	// PWRUP2: 4 GPIO powerup events can be configured to wake the chip up from a low power state. The pwrups are level/edge sensitive and can be set to trigger on a high/rising or low/falling event The number of gpios available depends on the package option. An invalid selection will be ignored source = 0 selects gpio0 . . source = 47 selects gpio47 source = 48 selects qspi_ss source = 49 selects qspi_sd0 source = 50 selects qspi_sd1 source = 51 selects qspi_sd2 source = 52 selects qspi_sd3 source = 53 selects qspi_sclk level = 0 triggers the pwrup when the source is low level = 1 triggers the pwrup when the source is high
	// Position of RAW_STATUS field.
	POWMAN_PWRUP2_RAW_STATUS_Pos = 0xa
	// Bit mask of RAW_STATUS field.
	POWMAN_PWRUP2_RAW_STATUS_Msk = 0x400
	// Bit RAW_STATUS.
	POWMAN_PWRUP2_RAW_STATUS = 0x400
	// Position of STATUS field.
	POWMAN_PWRUP2_STATUS_Pos = 0x9
	// Bit mask of STATUS field.
	POWMAN_PWRUP2_STATUS_Msk = 0x200
	// Bit STATUS.
	POWMAN_PWRUP2_STATUS = 0x200
	// Position of MODE field.
	POWMAN_PWRUP2_MODE_Pos = 0x8
	// Bit mask of MODE field.
	POWMAN_PWRUP2_MODE_Msk = 0x100
	// Bit MODE.
	POWMAN_PWRUP2_MODE       = 0x100
	POWMAN_PWRUP2_MODE_LEVEL = 0x0
	POWMAN_PWRUP2_MODE_EDGE  = 0x1
	// Position of DIRECTION field.
	POWMAN_PWRUP2_DIRECTION_Pos = 0x7
	// Bit mask of DIRECTION field.
	POWMAN_PWRUP2_DIRECTION_Msk = 0x80
	// Bit DIRECTION.
	POWMAN_PWRUP2_DIRECTION             = 0x80
	POWMAN_PWRUP2_DIRECTION_LOW_FALLING = 0x0
	POWMAN_PWRUP2_DIRECTION_HIGH_RISING = 0x1
	// Position of ENABLE field.
	POWMAN_PWRUP2_ENABLE_Pos = 0x6
	// Bit mask of ENABLE field.
	POWMAN_PWRUP2_ENABLE_Msk = 0x40
	// Bit ENABLE.
	POWMAN_PWRUP2_ENABLE = 0x40
	// Position of SOURCE field.
	POWMAN_PWRUP2_SOURCE_Pos = 0x0
	// Bit mask of SOURCE field.
	POWMAN_PWRUP2_SOURCE_Msk = 0x3f

	// PWRUP3: 4 GPIO powerup events can be configured to wake the chip up from a low power state. The pwrups are level/edge sensitive and can be set to trigger on a high/rising or low/falling event The number of gpios available depends on the package option. An invalid selection will be ignored source = 0 selects gpio0 . . source = 47 selects gpio47 source = 48 selects qspi_ss source = 49 selects qspi_sd0 source = 50 selects qspi_sd1 source = 51 selects qspi_sd2 source = 52 selects qspi_sd3 source = 53 selects qspi_sclk level = 0 triggers the pwrup when the source is low level = 1 triggers the pwrup when the source is high
	// Position of RAW_STATUS field.
	POWMAN_PWRUP3_RAW_STATUS_Pos = 0xa
	// Bit mask of RAW_STATUS field.
	POWMAN_PWRUP3_RAW_STATUS_Msk = 0x400
	// Bit RAW_STATUS.
	POWMAN_PWRUP3_RAW_STATUS = 0x400
	// Position of STATUS field.
	POWMAN_PWRUP3_STATUS_Pos = 0x9
	// Bit mask of STATUS field.
	POWMAN_PWRUP3_STATUS_Msk = 0x200
	// Bit STATUS.
	POWMAN_PWRUP3_STATUS = 0x200
	// Position of MODE field.
	POWMAN_PWRUP3_MODE_Pos = 0x8
	// Bit mask of MODE field.
	POWMAN_PWRUP3_MODE_Msk = 0x100
	// Bit MODE.
	POWMAN_PWRUP3_MODE       = 0x100
	POWMAN_PWRUP3_MODE_LEVEL = 0x0
	POWMAN_PWRUP3_MODE_EDGE  = 0x1
	// Position of DIRECTION field.
	POWMAN_PWRUP3_DIRECTION_Pos = 0x7
	// Bit mask of DIRECTION field.
	POWMAN_PWRUP3_DIRECTION_Msk = 0x80
	// Bit DIRECTION.
	POWMAN_PWRUP3_DIRECTION             = 0x80
	POWMAN_PWRUP3_DIRECTION_LOW_FALLING = 0x0
	POWMAN_PWRUP3_DIRECTION_HIGH_RISING = 0x1
	// Position of ENABLE field.
	POWMAN_PWRUP3_ENABLE_Pos = 0x6
	// Bit mask of ENABLE field.
	POWMAN_PWRUP3_ENABLE_Msk = 0x40
	// Bit ENABLE.
	POWMAN_PWRUP3_ENABLE = 0x40
	// Position of SOURCE field.
	POWMAN_PWRUP3_SOURCE_Pos = 0x0
	// Bit mask of SOURCE field.
	POWMAN_PWRUP3_SOURCE_Msk = 0x3f

	// CURRENT_PWRUP_REQ: Indicates current powerup request state pwrup events can be cleared by removing the enable from the pwrup register. The alarm pwrup req can be cleared by clearing timer.alarm_enab 0 = chip reset, for the source of the last reset see POWMAN_CHIP_RESET 1 = pwrup0 2 = pwrup1 3 = pwrup2 4 = pwrup3 5 = coresight_pwrup 6 = alarm_pwrup
	// Position of CURRENT_PWRUP_REQ field.
	POWMAN_CURRENT_PWRUP_REQ_CURRENT_PWRUP_REQ_Pos = 0x0
	// Bit mask of CURRENT_PWRUP_REQ field.
	POWMAN_CURRENT_PWRUP_REQ_CURRENT_PWRUP_REQ_Msk = 0x7f

	// LAST_SWCORE_PWRUP: Indicates which pwrup source triggered the last switched-core power up 0 = chip reset, for the source of the last reset see POWMAN_CHIP_RESET 1 = pwrup0 2 = pwrup1 3 = pwrup2 4 = pwrup3 5 = coresight_pwrup 6 = alarm_pwrup
	// Position of LAST_SWCORE_PWRUP field.
	POWMAN_LAST_SWCORE_PWRUP_LAST_SWCORE_PWRUP_Pos = 0x0
	// Bit mask of LAST_SWCORE_PWRUP field.
	POWMAN_LAST_SWCORE_PWRUP_LAST_SWCORE_PWRUP_Msk = 0x7f

	// DBG_PWRCFG
	// Position of IGNORE field.
	POWMAN_DBG_PWRCFG_IGNORE_Pos = 0x0
	// Bit mask of IGNORE field.
	POWMAN_DBG_PWRCFG_IGNORE_Msk = 0x1
	// Bit IGNORE.
	POWMAN_DBG_PWRCFG_IGNORE = 0x1

	// BOOTDIS: Tell the bootrom to ignore the BOOT0..3 registers following the next RSM reset (e.g. the next core power down/up). If an early boot stage has soft-locked some OTP pages in order to protect their contents from later stages, there is a risk that Secure code running at a later stage can unlock the pages by powering the core up and down. This register can be used to ensure that the bootloader runs as normal on the next power up, preventing Secure code at a later stage from accessing OTP in its unlocked state. Should be used in conjunction with the OTP BOOTDIS register.
	// Position of NEXT field.
	POWMAN_BOOTDIS_NEXT_Pos = 0x1
	// Bit mask of NEXT field.
	POWMAN_BOOTDIS_NEXT_Msk = 0x2
	// Bit NEXT.
	POWMAN_BOOTDIS_NEXT = 0x2
	// Position of NOW field.
	POWMAN_BOOTDIS_NOW_Pos = 0x0
	// Bit mask of NOW field.
	POWMAN_BOOTDIS_NOW_Msk = 0x1
	// Bit NOW.
	POWMAN_BOOTDIS_NOW = 0x1

	// DBGCONFIG
	// Position of DP_INSTID field.
	POWMAN_DBGCONFIG_DP_INSTID_Pos = 0x0
	// Bit mask of DP_INSTID field.
	POWMAN_DBGCONFIG_DP_INSTID_Msk = 0xf

	// SCRATCH0: Scratch register. Information persists in low power mode
	// Position of SCRATCH0 field.
	POWMAN_SCRATCH0_SCRATCH0_Pos = 0x0
	// Bit mask of SCRATCH0 field.
	POWMAN_SCRATCH0_SCRATCH0_Msk = 0xffffffff

	// SCRATCH1: Scratch register. Information persists in low power mode
	// Position of SCRATCH1 field.
	POWMAN_SCRATCH1_SCRATCH1_Pos = 0x0
	// Bit mask of SCRATCH1 field.
	POWMAN_SCRATCH1_SCRATCH1_Msk = 0xffffffff

	// SCRATCH2: Scratch register. Information persists in low power mode
	// Position of SCRATCH2 field.
	POWMAN_SCRATCH2_SCRATCH2_Pos = 0x0
	// Bit mask of SCRATCH2 field.
	POWMAN_SCRATCH2_SCRATCH2_Msk = 0xffffffff

	// SCRATCH3: Scratch register. Information persists in low power mode
	// Position of SCRATCH3 field.
	POWMAN_SCRATCH3_SCRATCH3_Pos = 0x0
	// Bit mask of SCRATCH3 field.
	POWMAN_SCRATCH3_SCRATCH3_Msk = 0xffffffff

	// SCRATCH4: Scratch register. Information persists in low power mode
	// Position of SCRATCH4 field.
	POWMAN_SCRATCH4_SCRATCH4_Pos = 0x0
	// Bit mask of SCRATCH4 field.
	POWMAN_SCRATCH4_SCRATCH4_Msk = 0xffffffff

	// SCRATCH5: Scratch register. Information persists in low power mode
	// Position of SCRATCH5 field.
	POWMAN_SCRATCH5_SCRATCH5_Pos = 0x0
	// Bit mask of SCRATCH5 field.
	POWMAN_SCRATCH5_SCRATCH5_Msk = 0xffffffff

	// SCRATCH6: Scratch register. Information persists in low power mode
	// Position of SCRATCH6 field.
	POWMAN_SCRATCH6_SCRATCH6_Pos = 0x0
	// Bit mask of SCRATCH6 field.
	POWMAN_SCRATCH6_SCRATCH6_Msk = 0xffffffff

	// SCRATCH7: Scratch register. Information persists in low power mode
	// Position of SCRATCH7 field.
	POWMAN_SCRATCH7_SCRATCH7_Pos = 0x0
	// Bit mask of SCRATCH7 field.
	POWMAN_SCRATCH7_SCRATCH7_Msk = 0xffffffff

	// BOOT0: Scratch register. Information persists in low power mode
	// Position of BOOT0 field.
	POWMAN_BOOT0_BOOT0_Pos = 0x0
	// Bit mask of BOOT0 field.
	POWMAN_BOOT0_BOOT0_Msk = 0xffffffff

	// BOOT1: Scratch register. Information persists in low power mode
	// Position of BOOT1 field.
	POWMAN_BOOT1_BOOT1_Pos = 0x0
	// Bit mask of BOOT1 field.
	POWMAN_BOOT1_BOOT1_Msk = 0xffffffff

	// BOOT2: Scratch register. Information persists in low power mode
	// Position of BOOT2 field.
	POWMAN_BOOT2_BOOT2_Pos = 0x0
	// Bit mask of BOOT2 field.
	POWMAN_BOOT2_BOOT2_Msk = 0xffffffff

	// BOOT3: Scratch register. Information persists in low power mode
	// Position of BOOT3 field.
	POWMAN_BOOT3_BOOT3_Pos = 0x0
	// Bit mask of BOOT3 field.
	POWMAN_BOOT3_BOOT3_Msk = 0xffffffff

	// INTR: Raw Interrupts
	// Position of PWRUP_WHILE_WAITING field.
	POWMAN_INTR_PWRUP_WHILE_WAITING_Pos = 0x3
	// Bit mask of PWRUP_WHILE_WAITING field.
	POWMAN_INTR_PWRUP_WHILE_WAITING_Msk = 0x8
	// Bit PWRUP_WHILE_WAITING.
	POWMAN_INTR_PWRUP_WHILE_WAITING = 0x8
	// Position of STATE_REQ_IGNORED field.
	POWMAN_INTR_STATE_REQ_IGNORED_Pos = 0x2
	// Bit mask of STATE_REQ_IGNORED field.
	POWMAN_INTR_STATE_REQ_IGNORED_Msk = 0x4
	// Bit STATE_REQ_IGNORED.
	POWMAN_INTR_STATE_REQ_IGNORED = 0x4
	// Position of TIMER field.
	POWMAN_INTR_TIMER_Pos = 0x1
	// Bit mask of TIMER field.
	POWMAN_INTR_TIMER_Msk = 0x2
	// Bit TIMER.
	POWMAN_INTR_TIMER = 0x2
	// Position of VREG_OUTPUT_LOW field.
	POWMAN_INTR_VREG_OUTPUT_LOW_Pos = 0x0
	// Bit mask of VREG_OUTPUT_LOW field.
	POWMAN_INTR_VREG_OUTPUT_LOW_Msk = 0x1
	// Bit VREG_OUTPUT_LOW.
	POWMAN_INTR_VREG_OUTPUT_LOW = 0x1

	// INTE: Interrupt Enable
	// Position of PWRUP_WHILE_WAITING field.
	POWMAN_INTE_PWRUP_WHILE_WAITING_Pos = 0x3
	// Bit mask of PWRUP_WHILE_WAITING field.
	POWMAN_INTE_PWRUP_WHILE_WAITING_Msk = 0x8
	// Bit PWRUP_WHILE_WAITING.
	POWMAN_INTE_PWRUP_WHILE_WAITING = 0x8
	// Position of STATE_REQ_IGNORED field.
	POWMAN_INTE_STATE_REQ_IGNORED_Pos = 0x2
	// Bit mask of STATE_REQ_IGNORED field.
	POWMAN_INTE_STATE_REQ_IGNORED_Msk = 0x4
	// Bit STATE_REQ_IGNORED.
	POWMAN_INTE_STATE_REQ_IGNORED = 0x4
	// Position of TIMER field.
	POWMAN_INTE_TIMER_Pos = 0x1
	// Bit mask of TIMER field.
	POWMAN_INTE_TIMER_Msk = 0x2
	// Bit TIMER.
	POWMAN_INTE_TIMER = 0x2
	// Position of VREG_OUTPUT_LOW field.
	POWMAN_INTE_VREG_OUTPUT_LOW_Pos = 0x0
	// Bit mask of VREG_OUTPUT_LOW field.
	POWMAN_INTE_VREG_OUTPUT_LOW_Msk = 0x1
	// Bit VREG_OUTPUT_LOW.
	POWMAN_INTE_VREG_OUTPUT_LOW = 0x1

	// INTF: Interrupt Force
	// Position of PWRUP_WHILE_WAITING field.
	POWMAN_INTF_PWRUP_WHILE_WAITING_Pos = 0x3
	// Bit mask of PWRUP_WHILE_WAITING field.
	POWMAN_INTF_PWRUP_WHILE_WAITING_Msk = 0x8
	// Bit PWRUP_WHILE_WAITING.
	POWMAN_INTF_PWRUP_WHILE_WAITING = 0x8
	// Position of STATE_REQ_IGNORED field.
	POWMAN_INTF_STATE_REQ_IGNORED_Pos = 0x2
	// Bit mask of STATE_REQ_IGNORED field.
	POWMAN_INTF_STATE_REQ_IGNORED_Msk = 0x4
	// Bit STATE_REQ_IGNORED.
	POWMAN_INTF_STATE_REQ_IGNORED = 0x4
	// Position of TIMER field.
	POWMAN_INTF_TIMER_Pos = 0x1
	// Bit mask of TIMER field.
	POWMAN_INTF_TIMER_Msk = 0x2
	// Bit TIMER.
	POWMAN_INTF_TIMER = 0x2
	// Position of VREG_OUTPUT_LOW field.
	POWMAN_INTF_VREG_OUTPUT_LOW_Pos = 0x0
	// Bit mask of VREG_OUTPUT_LOW field.
	POWMAN_INTF_VREG_OUTPUT_LOW_Msk = 0x1
	// Bit VREG_OUTPUT_LOW.
	POWMAN_INTF_VREG_OUTPUT_LOW = 0x1

	// INTS: Interrupt status after masking & forcing
	// Position of PWRUP_WHILE_WAITING field.
	POWMAN_INTS_PWRUP_WHILE_WAITING_Pos = 0x3
	// Bit mask of PWRUP_WHILE_WAITING field.
	POWMAN_INTS_PWRUP_WHILE_WAITING_Msk = 0x8
	// Bit PWRUP_WHILE_WAITING.
	POWMAN_INTS_PWRUP_WHILE_WAITING = 0x8
	// Position of STATE_REQ_IGNORED field.
	POWMAN_INTS_STATE_REQ_IGNORED_Pos = 0x2
	// Bit mask of STATE_REQ_IGNORED field.
	POWMAN_INTS_STATE_REQ_IGNORED_Msk = 0x4
	// Bit STATE_REQ_IGNORED.
	POWMAN_INTS_STATE_REQ_IGNORED = 0x4
	// Position of TIMER field.
	POWMAN_INTS_TIMER_Pos = 0x1
	// Bit mask of TIMER field.
	POWMAN_INTS_TIMER_Msk = 0x2
	// Bit TIMER.
	POWMAN_INTS_TIMER = 0x2
	// Position of VREG_OUTPUT_LOW field.
	POWMAN_INTS_VREG_OUTPUT_LOW_Pos = 0x0
	// Bit mask of VREG_OUTPUT_LOW field.
	POWMAN_INTS_VREG_OUTPUT_LOW_Msk = 0x1
	// Bit VREG_OUTPUT_LOW.
	POWMAN_INTS_VREG_OUTPUT_LOW = 0x1
)

// Constants for WATCHDOG
const (
	// CTRL: Watchdog control The rst_wdsel register determines which subsystems are reset when the watchdog is triggered. The watchdog can be triggered in software.
	// Position of TRIGGER field.
	WATCHDOG_CTRL_TRIGGER_Pos = 0x1f
	// Bit mask of TRIGGER field.
	WATCHDOG_CTRL_TRIGGER_Msk = 0x80000000
	// Bit TRIGGER.
	WATCHDOG_CTRL_TRIGGER = 0x80000000
	// Position of ENABLE field.
	WATCHDOG_CTRL_ENABLE_Pos = 0x1e
	// Bit mask of ENABLE field.
	WATCHDOG_CTRL_ENABLE_Msk = 0x40000000
	// Bit ENABLE.
	WATCHDOG_CTRL_ENABLE = 0x40000000
	// Position of PAUSE_DBG1 field.
	WATCHDOG_CTRL_PAUSE_DBG1_Pos = 0x1a
	// Bit mask of PAUSE_DBG1 field.
	WATCHDOG_CTRL_PAUSE_DBG1_Msk = 0x4000000
	// Bit PAUSE_DBG1.
	WATCHDOG_CTRL_PAUSE_DBG1 = 0x4000000
	// Position of PAUSE_DBG0 field.
	WATCHDOG_CTRL_PAUSE_DBG0_Pos = 0x19
	// Bit mask of PAUSE_DBG0 field.
	WATCHDOG_CTRL_PAUSE_DBG0_Msk = 0x2000000
	// Bit PAUSE_DBG0.
	WATCHDOG_CTRL_PAUSE_DBG0 = 0x2000000
	// Position of PAUSE_JTAG field.
	WATCHDOG_CTRL_PAUSE_JTAG_Pos = 0x18
	// Bit mask of PAUSE_JTAG field.
	WATCHDOG_CTRL_PAUSE_JTAG_Msk = 0x1000000
	// Bit PAUSE_JTAG.
	WATCHDOG_CTRL_PAUSE_JTAG = 0x1000000
	// Position of TIME field.
	WATCHDOG_CTRL_TIME_Pos = 0x0
	// Bit mask of TIME field.
	WATCHDOG_CTRL_TIME_Msk = 0xffffff

	// LOAD: Load the watchdog timer. The maximum setting is 0xffffff which corresponds to approximately 16 seconds.
	// Position of LOAD field.
	WATCHDOG_LOAD_LOAD_Pos = 0x0
	// Bit mask of LOAD field.
	WATCHDOG_LOAD_LOAD_Msk = 0xffffff

	// REASON: Logs the reason for the last reset. Both bits are zero for the case of a hardware reset. Additionally, as of RP2350, a debugger warm reset of either core (SYSRESETREQ or hartreset) will also clear the watchdog reason register, so that software loaded under the debugger following a watchdog timeout will not continue to see the timeout condition.
	// Position of FORCE field.
	WATCHDOG_REASON_FORCE_Pos = 0x1
	// Bit mask of FORCE field.
	WATCHDOG_REASON_FORCE_Msk = 0x2
	// Bit FORCE.
	WATCHDOG_REASON_FORCE = 0x2
	// Position of TIMER field.
	WATCHDOG_REASON_TIMER_Pos = 0x0
	// Bit mask of TIMER field.
	WATCHDOG_REASON_TIMER_Msk = 0x1
	// Bit TIMER.
	WATCHDOG_REASON_TIMER = 0x1

	// SCRATCH0: Scratch register. Information persists through soft reset of the chip.
	// Position of SCRATCH0 field.
	WATCHDOG_SCRATCH0_SCRATCH0_Pos = 0x0
	// Bit mask of SCRATCH0 field.
	WATCHDOG_SCRATCH0_SCRATCH0_Msk = 0xffffffff

	// SCRATCH1: Scratch register. Information persists through soft reset of the chip.
	// Position of SCRATCH1 field.
	WATCHDOG_SCRATCH1_SCRATCH1_Pos = 0x0
	// Bit mask of SCRATCH1 field.
	WATCHDOG_SCRATCH1_SCRATCH1_Msk = 0xffffffff

	// SCRATCH2: Scratch register. Information persists through soft reset of the chip.
	// Position of SCRATCH2 field.
	WATCHDOG_SCRATCH2_SCRATCH2_Pos = 0x0
	// Bit mask of SCRATCH2 field.
	WATCHDOG_SCRATCH2_SCRATCH2_Msk = 0xffffffff

	// SCRATCH3: Scratch register. Information persists through soft reset of the chip.
	// Position of SCRATCH3 field.
	WATCHDOG_SCRATCH3_SCRATCH3_Pos = 0x0
	// Bit mask of SCRATCH3 field.
	WATCHDOG_SCRATCH3_SCRATCH3_Msk = 0xffffffff

	// SCRATCH4: Scratch register. Information persists through soft reset of the chip.
	// Position of SCRATCH4 field.
	WATCHDOG_SCRATCH4_SCRATCH4_Pos = 0x0
	// Bit mask of SCRATCH4 field.
	WATCHDOG_SCRATCH4_SCRATCH4_Msk = 0xffffffff

	// SCRATCH5: Scratch register. Information persists through soft reset of the chip.
	// Position of SCRATCH5 field.
	WATCHDOG_SCRATCH5_SCRATCH5_Pos = 0x0
	// Bit mask of SCRATCH5 field.
	WATCHDOG_SCRATCH5_SCRATCH5_Msk = 0xffffffff

	// SCRATCH6: Scratch register. Information persists through soft reset of the chip.
	// Position of SCRATCH6 field.
	WATCHDOG_SCRATCH6_SCRATCH6_Pos = 0x0
	// Bit mask of SCRATCH6 field.
	WATCHDOG_SCRATCH6_SCRATCH6_Msk = 0xffffffff

	// SCRATCH7: Scratch register. Information persists through soft reset of the chip.
	// Position of SCRATCH7 field.
	WATCHDOG_SCRATCH7_SCRATCH7_Pos = 0x0
	// Bit mask of SCRATCH7 field.
	WATCHDOG_SCRATCH7_SCRATCH7_Msk = 0xffffffff
)

// Constants for DMA: DMA with separate read and write masters
const (
	// CH0_READ_ADDR: DMA Channel 0 Read Address pointer
	// Position of CH0_READ_ADDR field.
	DMA_CH0_READ_ADDR_CH0_READ_ADDR_Pos = 0x0
	// Bit mask of CH0_READ_ADDR field.
	DMA_CH0_READ_ADDR_CH0_READ_ADDR_Msk = 0xffffffff

	// CH0_WRITE_ADDR: DMA Channel 0 Write Address pointer
	// Position of CH0_WRITE_ADDR field.
	DMA_CH0_WRITE_ADDR_CH0_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH0_WRITE_ADDR field.
	DMA_CH0_WRITE_ADDR_CH0_WRITE_ADDR_Msk = 0xffffffff

	// CH0_TRANS_COUNT: DMA Channel 0 Transfer Count
	// Position of MODE field.
	DMA_CH0_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH0_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH0_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH0_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH0_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH0_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH0_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH0_CTRL_TRIG: DMA Channel 0 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH0_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH0_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH0_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH0_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH0_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH0_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH0_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH0_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH0_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH0_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH0_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH0_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH0_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH0_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH0_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH0_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH0_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH0_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH0_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH0_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH0_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH0_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH0_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH0_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH0_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH0_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH0_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH0_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH0_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH0_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH0_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH0_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH0_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH0_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH0_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH0_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH0_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH0_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH0_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH0_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH0_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH0_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH0_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH0_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH0_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH0_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH0_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH0_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH0_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH0_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH0_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH0_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH0_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH0_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH0_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH0_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH0_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH0_CTRL_TRIG_EN = 0x1

	// CH0_AL1_CTRL: Alias for channel 0 CTRL register
	// Position of CH0_AL1_CTRL field.
	DMA_CH0_AL1_CTRL_CH0_AL1_CTRL_Pos = 0x0
	// Bit mask of CH0_AL1_CTRL field.
	DMA_CH0_AL1_CTRL_CH0_AL1_CTRL_Msk = 0xffffffff

	// CH0_AL1_READ_ADDR: Alias for channel 0 READ_ADDR register
	// Position of CH0_AL1_READ_ADDR field.
	DMA_CH0_AL1_READ_ADDR_CH0_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH0_AL1_READ_ADDR field.
	DMA_CH0_AL1_READ_ADDR_CH0_AL1_READ_ADDR_Msk = 0xffffffff

	// CH0_AL1_WRITE_ADDR: Alias for channel 0 WRITE_ADDR register
	// Position of CH0_AL1_WRITE_ADDR field.
	DMA_CH0_AL1_WRITE_ADDR_CH0_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH0_AL1_WRITE_ADDR field.
	DMA_CH0_AL1_WRITE_ADDR_CH0_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH0_AL1_TRANS_COUNT_TRIG: Alias for channel 0 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH0_AL1_TRANS_COUNT_TRIG field.
	DMA_CH0_AL1_TRANS_COUNT_TRIG_CH0_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH0_AL1_TRANS_COUNT_TRIG field.
	DMA_CH0_AL1_TRANS_COUNT_TRIG_CH0_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH0_AL2_CTRL: Alias for channel 0 CTRL register
	// Position of CH0_AL2_CTRL field.
	DMA_CH0_AL2_CTRL_CH0_AL2_CTRL_Pos = 0x0
	// Bit mask of CH0_AL2_CTRL field.
	DMA_CH0_AL2_CTRL_CH0_AL2_CTRL_Msk = 0xffffffff

	// CH0_AL2_TRANS_COUNT: Alias for channel 0 TRANS_COUNT register
	// Position of CH0_AL2_TRANS_COUNT field.
	DMA_CH0_AL2_TRANS_COUNT_CH0_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH0_AL2_TRANS_COUNT field.
	DMA_CH0_AL2_TRANS_COUNT_CH0_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH0_AL2_READ_ADDR: Alias for channel 0 READ_ADDR register
	// Position of CH0_AL2_READ_ADDR field.
	DMA_CH0_AL2_READ_ADDR_CH0_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH0_AL2_READ_ADDR field.
	DMA_CH0_AL2_READ_ADDR_CH0_AL2_READ_ADDR_Msk = 0xffffffff

	// CH0_AL2_WRITE_ADDR_TRIG: Alias for channel 0 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH0_AL2_WRITE_ADDR_TRIG field.
	DMA_CH0_AL2_WRITE_ADDR_TRIG_CH0_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH0_AL2_WRITE_ADDR_TRIG field.
	DMA_CH0_AL2_WRITE_ADDR_TRIG_CH0_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH0_AL3_CTRL: Alias for channel 0 CTRL register
	// Position of CH0_AL3_CTRL field.
	DMA_CH0_AL3_CTRL_CH0_AL3_CTRL_Pos = 0x0
	// Bit mask of CH0_AL3_CTRL field.
	DMA_CH0_AL3_CTRL_CH0_AL3_CTRL_Msk = 0xffffffff

	// CH0_AL3_WRITE_ADDR: Alias for channel 0 WRITE_ADDR register
	// Position of CH0_AL3_WRITE_ADDR field.
	DMA_CH0_AL3_WRITE_ADDR_CH0_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH0_AL3_WRITE_ADDR field.
	DMA_CH0_AL3_WRITE_ADDR_CH0_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH0_AL3_TRANS_COUNT: Alias for channel 0 TRANS_COUNT register
	// Position of CH0_AL3_TRANS_COUNT field.
	DMA_CH0_AL3_TRANS_COUNT_CH0_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH0_AL3_TRANS_COUNT field.
	DMA_CH0_AL3_TRANS_COUNT_CH0_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH0_AL3_READ_ADDR_TRIG: Alias for channel 0 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH0_AL3_READ_ADDR_TRIG field.
	DMA_CH0_AL3_READ_ADDR_TRIG_CH0_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH0_AL3_READ_ADDR_TRIG field.
	DMA_CH0_AL3_READ_ADDR_TRIG_CH0_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH1_READ_ADDR: DMA Channel 1 Read Address pointer
	// Position of CH1_READ_ADDR field.
	DMA_CH1_READ_ADDR_CH1_READ_ADDR_Pos = 0x0
	// Bit mask of CH1_READ_ADDR field.
	DMA_CH1_READ_ADDR_CH1_READ_ADDR_Msk = 0xffffffff

	// CH1_WRITE_ADDR: DMA Channel 1 Write Address pointer
	// Position of CH1_WRITE_ADDR field.
	DMA_CH1_WRITE_ADDR_CH1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH1_WRITE_ADDR field.
	DMA_CH1_WRITE_ADDR_CH1_WRITE_ADDR_Msk = 0xffffffff

	// CH1_TRANS_COUNT: DMA Channel 1 Transfer Count
	// Position of MODE field.
	DMA_CH1_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH1_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH1_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH1_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH1_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH1_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH1_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH1_CTRL_TRIG: DMA Channel 1 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH1_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH1_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH1_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH1_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH1_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH1_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH1_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH1_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH1_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH1_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH1_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH1_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH1_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH1_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH1_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH1_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH1_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH1_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH1_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH1_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH1_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH1_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH1_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH1_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH1_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH1_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH1_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH1_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH1_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH1_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH1_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH1_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH1_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH1_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH1_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH1_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH1_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH1_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH1_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH1_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH1_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH1_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH1_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH1_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH1_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH1_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH1_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH1_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH1_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH1_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH1_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH1_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH1_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH1_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH1_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH1_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH1_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH1_CTRL_TRIG_EN = 0x1

	// CH1_AL1_CTRL: Alias for channel 1 CTRL register
	// Position of CH1_AL1_CTRL field.
	DMA_CH1_AL1_CTRL_CH1_AL1_CTRL_Pos = 0x0
	// Bit mask of CH1_AL1_CTRL field.
	DMA_CH1_AL1_CTRL_CH1_AL1_CTRL_Msk = 0xffffffff

	// CH1_AL1_READ_ADDR: Alias for channel 1 READ_ADDR register
	// Position of CH1_AL1_READ_ADDR field.
	DMA_CH1_AL1_READ_ADDR_CH1_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH1_AL1_READ_ADDR field.
	DMA_CH1_AL1_READ_ADDR_CH1_AL1_READ_ADDR_Msk = 0xffffffff

	// CH1_AL1_WRITE_ADDR: Alias for channel 1 WRITE_ADDR register
	// Position of CH1_AL1_WRITE_ADDR field.
	DMA_CH1_AL1_WRITE_ADDR_CH1_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH1_AL1_WRITE_ADDR field.
	DMA_CH1_AL1_WRITE_ADDR_CH1_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH1_AL1_TRANS_COUNT_TRIG: Alias for channel 1 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH1_AL1_TRANS_COUNT_TRIG field.
	DMA_CH1_AL1_TRANS_COUNT_TRIG_CH1_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH1_AL1_TRANS_COUNT_TRIG field.
	DMA_CH1_AL1_TRANS_COUNT_TRIG_CH1_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH1_AL2_CTRL: Alias for channel 1 CTRL register
	// Position of CH1_AL2_CTRL field.
	DMA_CH1_AL2_CTRL_CH1_AL2_CTRL_Pos = 0x0
	// Bit mask of CH1_AL2_CTRL field.
	DMA_CH1_AL2_CTRL_CH1_AL2_CTRL_Msk = 0xffffffff

	// CH1_AL2_TRANS_COUNT: Alias for channel 1 TRANS_COUNT register
	// Position of CH1_AL2_TRANS_COUNT field.
	DMA_CH1_AL2_TRANS_COUNT_CH1_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH1_AL2_TRANS_COUNT field.
	DMA_CH1_AL2_TRANS_COUNT_CH1_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH1_AL2_READ_ADDR: Alias for channel 1 READ_ADDR register
	// Position of CH1_AL2_READ_ADDR field.
	DMA_CH1_AL2_READ_ADDR_CH1_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH1_AL2_READ_ADDR field.
	DMA_CH1_AL2_READ_ADDR_CH1_AL2_READ_ADDR_Msk = 0xffffffff

	// CH1_AL2_WRITE_ADDR_TRIG: Alias for channel 1 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH1_AL2_WRITE_ADDR_TRIG field.
	DMA_CH1_AL2_WRITE_ADDR_TRIG_CH1_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH1_AL2_WRITE_ADDR_TRIG field.
	DMA_CH1_AL2_WRITE_ADDR_TRIG_CH1_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH1_AL3_CTRL: Alias for channel 1 CTRL register
	// Position of CH1_AL3_CTRL field.
	DMA_CH1_AL3_CTRL_CH1_AL3_CTRL_Pos = 0x0
	// Bit mask of CH1_AL3_CTRL field.
	DMA_CH1_AL3_CTRL_CH1_AL3_CTRL_Msk = 0xffffffff

	// CH1_AL3_WRITE_ADDR: Alias for channel 1 WRITE_ADDR register
	// Position of CH1_AL3_WRITE_ADDR field.
	DMA_CH1_AL3_WRITE_ADDR_CH1_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH1_AL3_WRITE_ADDR field.
	DMA_CH1_AL3_WRITE_ADDR_CH1_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH1_AL3_TRANS_COUNT: Alias for channel 1 TRANS_COUNT register
	// Position of CH1_AL3_TRANS_COUNT field.
	DMA_CH1_AL3_TRANS_COUNT_CH1_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH1_AL3_TRANS_COUNT field.
	DMA_CH1_AL3_TRANS_COUNT_CH1_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH1_AL3_READ_ADDR_TRIG: Alias for channel 1 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH1_AL3_READ_ADDR_TRIG field.
	DMA_CH1_AL3_READ_ADDR_TRIG_CH1_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH1_AL3_READ_ADDR_TRIG field.
	DMA_CH1_AL3_READ_ADDR_TRIG_CH1_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH2_READ_ADDR: DMA Channel 2 Read Address pointer
	// Position of CH2_READ_ADDR field.
	DMA_CH2_READ_ADDR_CH2_READ_ADDR_Pos = 0x0
	// Bit mask of CH2_READ_ADDR field.
	DMA_CH2_READ_ADDR_CH2_READ_ADDR_Msk = 0xffffffff

	// CH2_WRITE_ADDR: DMA Channel 2 Write Address pointer
	// Position of CH2_WRITE_ADDR field.
	DMA_CH2_WRITE_ADDR_CH2_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH2_WRITE_ADDR field.
	DMA_CH2_WRITE_ADDR_CH2_WRITE_ADDR_Msk = 0xffffffff

	// CH2_TRANS_COUNT: DMA Channel 2 Transfer Count
	// Position of MODE field.
	DMA_CH2_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH2_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH2_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH2_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH2_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH2_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH2_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH2_CTRL_TRIG: DMA Channel 2 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH2_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH2_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH2_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH2_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH2_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH2_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH2_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH2_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH2_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH2_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH2_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH2_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH2_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH2_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH2_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH2_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH2_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH2_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH2_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH2_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH2_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH2_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH2_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH2_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH2_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH2_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH2_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH2_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH2_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH2_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH2_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH2_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH2_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH2_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH2_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH2_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH2_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH2_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH2_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH2_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH2_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH2_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH2_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH2_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH2_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH2_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH2_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH2_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH2_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH2_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH2_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH2_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH2_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH2_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH2_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH2_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH2_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH2_CTRL_TRIG_EN = 0x1

	// CH2_AL1_CTRL: Alias for channel 2 CTRL register
	// Position of CH2_AL1_CTRL field.
	DMA_CH2_AL1_CTRL_CH2_AL1_CTRL_Pos = 0x0
	// Bit mask of CH2_AL1_CTRL field.
	DMA_CH2_AL1_CTRL_CH2_AL1_CTRL_Msk = 0xffffffff

	// CH2_AL1_READ_ADDR: Alias for channel 2 READ_ADDR register
	// Position of CH2_AL1_READ_ADDR field.
	DMA_CH2_AL1_READ_ADDR_CH2_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH2_AL1_READ_ADDR field.
	DMA_CH2_AL1_READ_ADDR_CH2_AL1_READ_ADDR_Msk = 0xffffffff

	// CH2_AL1_WRITE_ADDR: Alias for channel 2 WRITE_ADDR register
	// Position of CH2_AL1_WRITE_ADDR field.
	DMA_CH2_AL1_WRITE_ADDR_CH2_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH2_AL1_WRITE_ADDR field.
	DMA_CH2_AL1_WRITE_ADDR_CH2_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH2_AL1_TRANS_COUNT_TRIG: Alias for channel 2 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH2_AL1_TRANS_COUNT_TRIG field.
	DMA_CH2_AL1_TRANS_COUNT_TRIG_CH2_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH2_AL1_TRANS_COUNT_TRIG field.
	DMA_CH2_AL1_TRANS_COUNT_TRIG_CH2_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH2_AL2_CTRL: Alias for channel 2 CTRL register
	// Position of CH2_AL2_CTRL field.
	DMA_CH2_AL2_CTRL_CH2_AL2_CTRL_Pos = 0x0
	// Bit mask of CH2_AL2_CTRL field.
	DMA_CH2_AL2_CTRL_CH2_AL2_CTRL_Msk = 0xffffffff

	// CH2_AL2_TRANS_COUNT: Alias for channel 2 TRANS_COUNT register
	// Position of CH2_AL2_TRANS_COUNT field.
	DMA_CH2_AL2_TRANS_COUNT_CH2_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH2_AL2_TRANS_COUNT field.
	DMA_CH2_AL2_TRANS_COUNT_CH2_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH2_AL2_READ_ADDR: Alias for channel 2 READ_ADDR register
	// Position of CH2_AL2_READ_ADDR field.
	DMA_CH2_AL2_READ_ADDR_CH2_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH2_AL2_READ_ADDR field.
	DMA_CH2_AL2_READ_ADDR_CH2_AL2_READ_ADDR_Msk = 0xffffffff

	// CH2_AL2_WRITE_ADDR_TRIG: Alias for channel 2 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH2_AL2_WRITE_ADDR_TRIG field.
	DMA_CH2_AL2_WRITE_ADDR_TRIG_CH2_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH2_AL2_WRITE_ADDR_TRIG field.
	DMA_CH2_AL2_WRITE_ADDR_TRIG_CH2_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH2_AL3_CTRL: Alias for channel 2 CTRL register
	// Position of CH2_AL3_CTRL field.
	DMA_CH2_AL3_CTRL_CH2_AL3_CTRL_Pos = 0x0
	// Bit mask of CH2_AL3_CTRL field.
	DMA_CH2_AL3_CTRL_CH2_AL3_CTRL_Msk = 0xffffffff

	// CH2_AL3_WRITE_ADDR: Alias for channel 2 WRITE_ADDR register
	// Position of CH2_AL3_WRITE_ADDR field.
	DMA_CH2_AL3_WRITE_ADDR_CH2_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH2_AL3_WRITE_ADDR field.
	DMA_CH2_AL3_WRITE_ADDR_CH2_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH2_AL3_TRANS_COUNT: Alias for channel 2 TRANS_COUNT register
	// Position of CH2_AL3_TRANS_COUNT field.
	DMA_CH2_AL3_TRANS_COUNT_CH2_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH2_AL3_TRANS_COUNT field.
	DMA_CH2_AL3_TRANS_COUNT_CH2_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH2_AL3_READ_ADDR_TRIG: Alias for channel 2 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH2_AL3_READ_ADDR_TRIG field.
	DMA_CH2_AL3_READ_ADDR_TRIG_CH2_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH2_AL3_READ_ADDR_TRIG field.
	DMA_CH2_AL3_READ_ADDR_TRIG_CH2_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH3_READ_ADDR: DMA Channel 3 Read Address pointer
	// Position of CH3_READ_ADDR field.
	DMA_CH3_READ_ADDR_CH3_READ_ADDR_Pos = 0x0
	// Bit mask of CH3_READ_ADDR field.
	DMA_CH3_READ_ADDR_CH3_READ_ADDR_Msk = 0xffffffff

	// CH3_WRITE_ADDR: DMA Channel 3 Write Address pointer
	// Position of CH3_WRITE_ADDR field.
	DMA_CH3_WRITE_ADDR_CH3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH3_WRITE_ADDR field.
	DMA_CH3_WRITE_ADDR_CH3_WRITE_ADDR_Msk = 0xffffffff

	// CH3_TRANS_COUNT: DMA Channel 3 Transfer Count
	// Position of MODE field.
	DMA_CH3_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH3_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH3_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH3_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH3_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH3_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH3_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH3_CTRL_TRIG: DMA Channel 3 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH3_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH3_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH3_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH3_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH3_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH3_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH3_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH3_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH3_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH3_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH3_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH3_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH3_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH3_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH3_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH3_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH3_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH3_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH3_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH3_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH3_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH3_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH3_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH3_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH3_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH3_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH3_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH3_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH3_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH3_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH3_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH3_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH3_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH3_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH3_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH3_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH3_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH3_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH3_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH3_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH3_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH3_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH3_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH3_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH3_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH3_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH3_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH3_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH3_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH3_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH3_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH3_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH3_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH3_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH3_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH3_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH3_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH3_CTRL_TRIG_EN = 0x1

	// CH3_AL1_CTRL: Alias for channel 3 CTRL register
	// Position of CH3_AL1_CTRL field.
	DMA_CH3_AL1_CTRL_CH3_AL1_CTRL_Pos = 0x0
	// Bit mask of CH3_AL1_CTRL field.
	DMA_CH3_AL1_CTRL_CH3_AL1_CTRL_Msk = 0xffffffff

	// CH3_AL1_READ_ADDR: Alias for channel 3 READ_ADDR register
	// Position of CH3_AL1_READ_ADDR field.
	DMA_CH3_AL1_READ_ADDR_CH3_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH3_AL1_READ_ADDR field.
	DMA_CH3_AL1_READ_ADDR_CH3_AL1_READ_ADDR_Msk = 0xffffffff

	// CH3_AL1_WRITE_ADDR: Alias for channel 3 WRITE_ADDR register
	// Position of CH3_AL1_WRITE_ADDR field.
	DMA_CH3_AL1_WRITE_ADDR_CH3_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH3_AL1_WRITE_ADDR field.
	DMA_CH3_AL1_WRITE_ADDR_CH3_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH3_AL1_TRANS_COUNT_TRIG: Alias for channel 3 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH3_AL1_TRANS_COUNT_TRIG field.
	DMA_CH3_AL1_TRANS_COUNT_TRIG_CH3_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH3_AL1_TRANS_COUNT_TRIG field.
	DMA_CH3_AL1_TRANS_COUNT_TRIG_CH3_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH3_AL2_CTRL: Alias for channel 3 CTRL register
	// Position of CH3_AL2_CTRL field.
	DMA_CH3_AL2_CTRL_CH3_AL2_CTRL_Pos = 0x0
	// Bit mask of CH3_AL2_CTRL field.
	DMA_CH3_AL2_CTRL_CH3_AL2_CTRL_Msk = 0xffffffff

	// CH3_AL2_TRANS_COUNT: Alias for channel 3 TRANS_COUNT register
	// Position of CH3_AL2_TRANS_COUNT field.
	DMA_CH3_AL2_TRANS_COUNT_CH3_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH3_AL2_TRANS_COUNT field.
	DMA_CH3_AL2_TRANS_COUNT_CH3_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH3_AL2_READ_ADDR: Alias for channel 3 READ_ADDR register
	// Position of CH3_AL2_READ_ADDR field.
	DMA_CH3_AL2_READ_ADDR_CH3_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH3_AL2_READ_ADDR field.
	DMA_CH3_AL2_READ_ADDR_CH3_AL2_READ_ADDR_Msk = 0xffffffff

	// CH3_AL2_WRITE_ADDR_TRIG: Alias for channel 3 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH3_AL2_WRITE_ADDR_TRIG field.
	DMA_CH3_AL2_WRITE_ADDR_TRIG_CH3_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH3_AL2_WRITE_ADDR_TRIG field.
	DMA_CH3_AL2_WRITE_ADDR_TRIG_CH3_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH3_AL3_CTRL: Alias for channel 3 CTRL register
	// Position of CH3_AL3_CTRL field.
	DMA_CH3_AL3_CTRL_CH3_AL3_CTRL_Pos = 0x0
	// Bit mask of CH3_AL3_CTRL field.
	DMA_CH3_AL3_CTRL_CH3_AL3_CTRL_Msk = 0xffffffff

	// CH3_AL3_WRITE_ADDR: Alias for channel 3 WRITE_ADDR register
	// Position of CH3_AL3_WRITE_ADDR field.
	DMA_CH3_AL3_WRITE_ADDR_CH3_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH3_AL3_WRITE_ADDR field.
	DMA_CH3_AL3_WRITE_ADDR_CH3_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH3_AL3_TRANS_COUNT: Alias for channel 3 TRANS_COUNT register
	// Position of CH3_AL3_TRANS_COUNT field.
	DMA_CH3_AL3_TRANS_COUNT_CH3_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH3_AL3_TRANS_COUNT field.
	DMA_CH3_AL3_TRANS_COUNT_CH3_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH3_AL3_READ_ADDR_TRIG: Alias for channel 3 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH3_AL3_READ_ADDR_TRIG field.
	DMA_CH3_AL3_READ_ADDR_TRIG_CH3_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH3_AL3_READ_ADDR_TRIG field.
	DMA_CH3_AL3_READ_ADDR_TRIG_CH3_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH4_READ_ADDR: DMA Channel 4 Read Address pointer
	// Position of CH4_READ_ADDR field.
	DMA_CH4_READ_ADDR_CH4_READ_ADDR_Pos = 0x0
	// Bit mask of CH4_READ_ADDR field.
	DMA_CH4_READ_ADDR_CH4_READ_ADDR_Msk = 0xffffffff

	// CH4_WRITE_ADDR: DMA Channel 4 Write Address pointer
	// Position of CH4_WRITE_ADDR field.
	DMA_CH4_WRITE_ADDR_CH4_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH4_WRITE_ADDR field.
	DMA_CH4_WRITE_ADDR_CH4_WRITE_ADDR_Msk = 0xffffffff

	// CH4_TRANS_COUNT: DMA Channel 4 Transfer Count
	// Position of MODE field.
	DMA_CH4_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH4_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH4_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH4_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH4_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH4_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH4_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH4_CTRL_TRIG: DMA Channel 4 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH4_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH4_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH4_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH4_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH4_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH4_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH4_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH4_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH4_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH4_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH4_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH4_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH4_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH4_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH4_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH4_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH4_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH4_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH4_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH4_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH4_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH4_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH4_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH4_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH4_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH4_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH4_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH4_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH4_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH4_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH4_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH4_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH4_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH4_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH4_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH4_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH4_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH4_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH4_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH4_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH4_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH4_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH4_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH4_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH4_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH4_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH4_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH4_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH4_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH4_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH4_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH4_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH4_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH4_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH4_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH4_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH4_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH4_CTRL_TRIG_EN = 0x1

	// CH4_AL1_CTRL: Alias for channel 4 CTRL register
	// Position of CH4_AL1_CTRL field.
	DMA_CH4_AL1_CTRL_CH4_AL1_CTRL_Pos = 0x0
	// Bit mask of CH4_AL1_CTRL field.
	DMA_CH4_AL1_CTRL_CH4_AL1_CTRL_Msk = 0xffffffff

	// CH4_AL1_READ_ADDR: Alias for channel 4 READ_ADDR register
	// Position of CH4_AL1_READ_ADDR field.
	DMA_CH4_AL1_READ_ADDR_CH4_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH4_AL1_READ_ADDR field.
	DMA_CH4_AL1_READ_ADDR_CH4_AL1_READ_ADDR_Msk = 0xffffffff

	// CH4_AL1_WRITE_ADDR: Alias for channel 4 WRITE_ADDR register
	// Position of CH4_AL1_WRITE_ADDR field.
	DMA_CH4_AL1_WRITE_ADDR_CH4_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH4_AL1_WRITE_ADDR field.
	DMA_CH4_AL1_WRITE_ADDR_CH4_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH4_AL1_TRANS_COUNT_TRIG: Alias for channel 4 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH4_AL1_TRANS_COUNT_TRIG field.
	DMA_CH4_AL1_TRANS_COUNT_TRIG_CH4_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH4_AL1_TRANS_COUNT_TRIG field.
	DMA_CH4_AL1_TRANS_COUNT_TRIG_CH4_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH4_AL2_CTRL: Alias for channel 4 CTRL register
	// Position of CH4_AL2_CTRL field.
	DMA_CH4_AL2_CTRL_CH4_AL2_CTRL_Pos = 0x0
	// Bit mask of CH4_AL2_CTRL field.
	DMA_CH4_AL2_CTRL_CH4_AL2_CTRL_Msk = 0xffffffff

	// CH4_AL2_TRANS_COUNT: Alias for channel 4 TRANS_COUNT register
	// Position of CH4_AL2_TRANS_COUNT field.
	DMA_CH4_AL2_TRANS_COUNT_CH4_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH4_AL2_TRANS_COUNT field.
	DMA_CH4_AL2_TRANS_COUNT_CH4_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH4_AL2_READ_ADDR: Alias for channel 4 READ_ADDR register
	// Position of CH4_AL2_READ_ADDR field.
	DMA_CH4_AL2_READ_ADDR_CH4_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH4_AL2_READ_ADDR field.
	DMA_CH4_AL2_READ_ADDR_CH4_AL2_READ_ADDR_Msk = 0xffffffff

	// CH4_AL2_WRITE_ADDR_TRIG: Alias for channel 4 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH4_AL2_WRITE_ADDR_TRIG field.
	DMA_CH4_AL2_WRITE_ADDR_TRIG_CH4_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH4_AL2_WRITE_ADDR_TRIG field.
	DMA_CH4_AL2_WRITE_ADDR_TRIG_CH4_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH4_AL3_CTRL: Alias for channel 4 CTRL register
	// Position of CH4_AL3_CTRL field.
	DMA_CH4_AL3_CTRL_CH4_AL3_CTRL_Pos = 0x0
	// Bit mask of CH4_AL3_CTRL field.
	DMA_CH4_AL3_CTRL_CH4_AL3_CTRL_Msk = 0xffffffff

	// CH4_AL3_WRITE_ADDR: Alias for channel 4 WRITE_ADDR register
	// Position of CH4_AL3_WRITE_ADDR field.
	DMA_CH4_AL3_WRITE_ADDR_CH4_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH4_AL3_WRITE_ADDR field.
	DMA_CH4_AL3_WRITE_ADDR_CH4_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH4_AL3_TRANS_COUNT: Alias for channel 4 TRANS_COUNT register
	// Position of CH4_AL3_TRANS_COUNT field.
	DMA_CH4_AL3_TRANS_COUNT_CH4_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH4_AL3_TRANS_COUNT field.
	DMA_CH4_AL3_TRANS_COUNT_CH4_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH4_AL3_READ_ADDR_TRIG: Alias for channel 4 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH4_AL3_READ_ADDR_TRIG field.
	DMA_CH4_AL3_READ_ADDR_TRIG_CH4_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH4_AL3_READ_ADDR_TRIG field.
	DMA_CH4_AL3_READ_ADDR_TRIG_CH4_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH5_READ_ADDR: DMA Channel 5 Read Address pointer
	// Position of CH5_READ_ADDR field.
	DMA_CH5_READ_ADDR_CH5_READ_ADDR_Pos = 0x0
	// Bit mask of CH5_READ_ADDR field.
	DMA_CH5_READ_ADDR_CH5_READ_ADDR_Msk = 0xffffffff

	// CH5_WRITE_ADDR: DMA Channel 5 Write Address pointer
	// Position of CH5_WRITE_ADDR field.
	DMA_CH5_WRITE_ADDR_CH5_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH5_WRITE_ADDR field.
	DMA_CH5_WRITE_ADDR_CH5_WRITE_ADDR_Msk = 0xffffffff

	// CH5_TRANS_COUNT: DMA Channel 5 Transfer Count
	// Position of MODE field.
	DMA_CH5_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH5_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH5_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH5_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH5_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH5_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH5_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH5_CTRL_TRIG: DMA Channel 5 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH5_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH5_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH5_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH5_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH5_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH5_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH5_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH5_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH5_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH5_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH5_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH5_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH5_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH5_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH5_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH5_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH5_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH5_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH5_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH5_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH5_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH5_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH5_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH5_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH5_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH5_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH5_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH5_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH5_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH5_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH5_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH5_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH5_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH5_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH5_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH5_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH5_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH5_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH5_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH5_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH5_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH5_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH5_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH5_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH5_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH5_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH5_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH5_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH5_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH5_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH5_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH5_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH5_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH5_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH5_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH5_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH5_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH5_CTRL_TRIG_EN = 0x1

	// CH5_AL1_CTRL: Alias for channel 5 CTRL register
	// Position of CH5_AL1_CTRL field.
	DMA_CH5_AL1_CTRL_CH5_AL1_CTRL_Pos = 0x0
	// Bit mask of CH5_AL1_CTRL field.
	DMA_CH5_AL1_CTRL_CH5_AL1_CTRL_Msk = 0xffffffff

	// CH5_AL1_READ_ADDR: Alias for channel 5 READ_ADDR register
	// Position of CH5_AL1_READ_ADDR field.
	DMA_CH5_AL1_READ_ADDR_CH5_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH5_AL1_READ_ADDR field.
	DMA_CH5_AL1_READ_ADDR_CH5_AL1_READ_ADDR_Msk = 0xffffffff

	// CH5_AL1_WRITE_ADDR: Alias for channel 5 WRITE_ADDR register
	// Position of CH5_AL1_WRITE_ADDR field.
	DMA_CH5_AL1_WRITE_ADDR_CH5_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH5_AL1_WRITE_ADDR field.
	DMA_CH5_AL1_WRITE_ADDR_CH5_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH5_AL1_TRANS_COUNT_TRIG: Alias for channel 5 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH5_AL1_TRANS_COUNT_TRIG field.
	DMA_CH5_AL1_TRANS_COUNT_TRIG_CH5_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH5_AL1_TRANS_COUNT_TRIG field.
	DMA_CH5_AL1_TRANS_COUNT_TRIG_CH5_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH5_AL2_CTRL: Alias for channel 5 CTRL register
	// Position of CH5_AL2_CTRL field.
	DMA_CH5_AL2_CTRL_CH5_AL2_CTRL_Pos = 0x0
	// Bit mask of CH5_AL2_CTRL field.
	DMA_CH5_AL2_CTRL_CH5_AL2_CTRL_Msk = 0xffffffff

	// CH5_AL2_TRANS_COUNT: Alias for channel 5 TRANS_COUNT register
	// Position of CH5_AL2_TRANS_COUNT field.
	DMA_CH5_AL2_TRANS_COUNT_CH5_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH5_AL2_TRANS_COUNT field.
	DMA_CH5_AL2_TRANS_COUNT_CH5_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH5_AL2_READ_ADDR: Alias for channel 5 READ_ADDR register
	// Position of CH5_AL2_READ_ADDR field.
	DMA_CH5_AL2_READ_ADDR_CH5_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH5_AL2_READ_ADDR field.
	DMA_CH5_AL2_READ_ADDR_CH5_AL2_READ_ADDR_Msk = 0xffffffff

	// CH5_AL2_WRITE_ADDR_TRIG: Alias for channel 5 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH5_AL2_WRITE_ADDR_TRIG field.
	DMA_CH5_AL2_WRITE_ADDR_TRIG_CH5_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH5_AL2_WRITE_ADDR_TRIG field.
	DMA_CH5_AL2_WRITE_ADDR_TRIG_CH5_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH5_AL3_CTRL: Alias for channel 5 CTRL register
	// Position of CH5_AL3_CTRL field.
	DMA_CH5_AL3_CTRL_CH5_AL3_CTRL_Pos = 0x0
	// Bit mask of CH5_AL3_CTRL field.
	DMA_CH5_AL3_CTRL_CH5_AL3_CTRL_Msk = 0xffffffff

	// CH5_AL3_WRITE_ADDR: Alias for channel 5 WRITE_ADDR register
	// Position of CH5_AL3_WRITE_ADDR field.
	DMA_CH5_AL3_WRITE_ADDR_CH5_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH5_AL3_WRITE_ADDR field.
	DMA_CH5_AL3_WRITE_ADDR_CH5_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH5_AL3_TRANS_COUNT: Alias for channel 5 TRANS_COUNT register
	// Position of CH5_AL3_TRANS_COUNT field.
	DMA_CH5_AL3_TRANS_COUNT_CH5_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH5_AL3_TRANS_COUNT field.
	DMA_CH5_AL3_TRANS_COUNT_CH5_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH5_AL3_READ_ADDR_TRIG: Alias for channel 5 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH5_AL3_READ_ADDR_TRIG field.
	DMA_CH5_AL3_READ_ADDR_TRIG_CH5_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH5_AL3_READ_ADDR_TRIG field.
	DMA_CH5_AL3_READ_ADDR_TRIG_CH5_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH6_READ_ADDR: DMA Channel 6 Read Address pointer
	// Position of CH6_READ_ADDR field.
	DMA_CH6_READ_ADDR_CH6_READ_ADDR_Pos = 0x0
	// Bit mask of CH6_READ_ADDR field.
	DMA_CH6_READ_ADDR_CH6_READ_ADDR_Msk = 0xffffffff

	// CH6_WRITE_ADDR: DMA Channel 6 Write Address pointer
	// Position of CH6_WRITE_ADDR field.
	DMA_CH6_WRITE_ADDR_CH6_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH6_WRITE_ADDR field.
	DMA_CH6_WRITE_ADDR_CH6_WRITE_ADDR_Msk = 0xffffffff

	// CH6_TRANS_COUNT: DMA Channel 6 Transfer Count
	// Position of MODE field.
	DMA_CH6_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH6_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH6_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH6_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH6_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH6_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH6_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH6_CTRL_TRIG: DMA Channel 6 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH6_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH6_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH6_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH6_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH6_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH6_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH6_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH6_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH6_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH6_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH6_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH6_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH6_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH6_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH6_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH6_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH6_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH6_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH6_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH6_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH6_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH6_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH6_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH6_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH6_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH6_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH6_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH6_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH6_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH6_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH6_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH6_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH6_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH6_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH6_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH6_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH6_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH6_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH6_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH6_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH6_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH6_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH6_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH6_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH6_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH6_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH6_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH6_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH6_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH6_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH6_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH6_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH6_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH6_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH6_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH6_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH6_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH6_CTRL_TRIG_EN = 0x1

	// CH6_AL1_CTRL: Alias for channel 6 CTRL register
	// Position of CH6_AL1_CTRL field.
	DMA_CH6_AL1_CTRL_CH6_AL1_CTRL_Pos = 0x0
	// Bit mask of CH6_AL1_CTRL field.
	DMA_CH6_AL1_CTRL_CH6_AL1_CTRL_Msk = 0xffffffff

	// CH6_AL1_READ_ADDR: Alias for channel 6 READ_ADDR register
	// Position of CH6_AL1_READ_ADDR field.
	DMA_CH6_AL1_READ_ADDR_CH6_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH6_AL1_READ_ADDR field.
	DMA_CH6_AL1_READ_ADDR_CH6_AL1_READ_ADDR_Msk = 0xffffffff

	// CH6_AL1_WRITE_ADDR: Alias for channel 6 WRITE_ADDR register
	// Position of CH6_AL1_WRITE_ADDR field.
	DMA_CH6_AL1_WRITE_ADDR_CH6_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH6_AL1_WRITE_ADDR field.
	DMA_CH6_AL1_WRITE_ADDR_CH6_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH6_AL1_TRANS_COUNT_TRIG: Alias for channel 6 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH6_AL1_TRANS_COUNT_TRIG field.
	DMA_CH6_AL1_TRANS_COUNT_TRIG_CH6_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH6_AL1_TRANS_COUNT_TRIG field.
	DMA_CH6_AL1_TRANS_COUNT_TRIG_CH6_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH6_AL2_CTRL: Alias for channel 6 CTRL register
	// Position of CH6_AL2_CTRL field.
	DMA_CH6_AL2_CTRL_CH6_AL2_CTRL_Pos = 0x0
	// Bit mask of CH6_AL2_CTRL field.
	DMA_CH6_AL2_CTRL_CH6_AL2_CTRL_Msk = 0xffffffff

	// CH6_AL2_TRANS_COUNT: Alias for channel 6 TRANS_COUNT register
	// Position of CH6_AL2_TRANS_COUNT field.
	DMA_CH6_AL2_TRANS_COUNT_CH6_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH6_AL2_TRANS_COUNT field.
	DMA_CH6_AL2_TRANS_COUNT_CH6_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH6_AL2_READ_ADDR: Alias for channel 6 READ_ADDR register
	// Position of CH6_AL2_READ_ADDR field.
	DMA_CH6_AL2_READ_ADDR_CH6_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH6_AL2_READ_ADDR field.
	DMA_CH6_AL2_READ_ADDR_CH6_AL2_READ_ADDR_Msk = 0xffffffff

	// CH6_AL2_WRITE_ADDR_TRIG: Alias for channel 6 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH6_AL2_WRITE_ADDR_TRIG field.
	DMA_CH6_AL2_WRITE_ADDR_TRIG_CH6_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH6_AL2_WRITE_ADDR_TRIG field.
	DMA_CH6_AL2_WRITE_ADDR_TRIG_CH6_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH6_AL3_CTRL: Alias for channel 6 CTRL register
	// Position of CH6_AL3_CTRL field.
	DMA_CH6_AL3_CTRL_CH6_AL3_CTRL_Pos = 0x0
	// Bit mask of CH6_AL3_CTRL field.
	DMA_CH6_AL3_CTRL_CH6_AL3_CTRL_Msk = 0xffffffff

	// CH6_AL3_WRITE_ADDR: Alias for channel 6 WRITE_ADDR register
	// Position of CH6_AL3_WRITE_ADDR field.
	DMA_CH6_AL3_WRITE_ADDR_CH6_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH6_AL3_WRITE_ADDR field.
	DMA_CH6_AL3_WRITE_ADDR_CH6_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH6_AL3_TRANS_COUNT: Alias for channel 6 TRANS_COUNT register
	// Position of CH6_AL3_TRANS_COUNT field.
	DMA_CH6_AL3_TRANS_COUNT_CH6_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH6_AL3_TRANS_COUNT field.
	DMA_CH6_AL3_TRANS_COUNT_CH6_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH6_AL3_READ_ADDR_TRIG: Alias for channel 6 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH6_AL3_READ_ADDR_TRIG field.
	DMA_CH6_AL3_READ_ADDR_TRIG_CH6_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH6_AL3_READ_ADDR_TRIG field.
	DMA_CH6_AL3_READ_ADDR_TRIG_CH6_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH7_READ_ADDR: DMA Channel 7 Read Address pointer
	// Position of CH7_READ_ADDR field.
	DMA_CH7_READ_ADDR_CH7_READ_ADDR_Pos = 0x0
	// Bit mask of CH7_READ_ADDR field.
	DMA_CH7_READ_ADDR_CH7_READ_ADDR_Msk = 0xffffffff

	// CH7_WRITE_ADDR: DMA Channel 7 Write Address pointer
	// Position of CH7_WRITE_ADDR field.
	DMA_CH7_WRITE_ADDR_CH7_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH7_WRITE_ADDR field.
	DMA_CH7_WRITE_ADDR_CH7_WRITE_ADDR_Msk = 0xffffffff

	// CH7_TRANS_COUNT: DMA Channel 7 Transfer Count
	// Position of MODE field.
	DMA_CH7_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH7_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH7_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH7_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH7_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH7_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH7_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH7_CTRL_TRIG: DMA Channel 7 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH7_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH7_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH7_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH7_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH7_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH7_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH7_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH7_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH7_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH7_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH7_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH7_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH7_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH7_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH7_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH7_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH7_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH7_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH7_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH7_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH7_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH7_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH7_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH7_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH7_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH7_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH7_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH7_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH7_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH7_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH7_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH7_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH7_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH7_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH7_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH7_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH7_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH7_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH7_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH7_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH7_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH7_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH7_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH7_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH7_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH7_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH7_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH7_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH7_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH7_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH7_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH7_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH7_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH7_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH7_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH7_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH7_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH7_CTRL_TRIG_EN = 0x1

	// CH7_AL1_CTRL: Alias for channel 7 CTRL register
	// Position of CH7_AL1_CTRL field.
	DMA_CH7_AL1_CTRL_CH7_AL1_CTRL_Pos = 0x0
	// Bit mask of CH7_AL1_CTRL field.
	DMA_CH7_AL1_CTRL_CH7_AL1_CTRL_Msk = 0xffffffff

	// CH7_AL1_READ_ADDR: Alias for channel 7 READ_ADDR register
	// Position of CH7_AL1_READ_ADDR field.
	DMA_CH7_AL1_READ_ADDR_CH7_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH7_AL1_READ_ADDR field.
	DMA_CH7_AL1_READ_ADDR_CH7_AL1_READ_ADDR_Msk = 0xffffffff

	// CH7_AL1_WRITE_ADDR: Alias for channel 7 WRITE_ADDR register
	// Position of CH7_AL1_WRITE_ADDR field.
	DMA_CH7_AL1_WRITE_ADDR_CH7_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH7_AL1_WRITE_ADDR field.
	DMA_CH7_AL1_WRITE_ADDR_CH7_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH7_AL1_TRANS_COUNT_TRIG: Alias for channel 7 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH7_AL1_TRANS_COUNT_TRIG field.
	DMA_CH7_AL1_TRANS_COUNT_TRIG_CH7_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH7_AL1_TRANS_COUNT_TRIG field.
	DMA_CH7_AL1_TRANS_COUNT_TRIG_CH7_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH7_AL2_CTRL: Alias for channel 7 CTRL register
	// Position of CH7_AL2_CTRL field.
	DMA_CH7_AL2_CTRL_CH7_AL2_CTRL_Pos = 0x0
	// Bit mask of CH7_AL2_CTRL field.
	DMA_CH7_AL2_CTRL_CH7_AL2_CTRL_Msk = 0xffffffff

	// CH7_AL2_TRANS_COUNT: Alias for channel 7 TRANS_COUNT register
	// Position of CH7_AL2_TRANS_COUNT field.
	DMA_CH7_AL2_TRANS_COUNT_CH7_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH7_AL2_TRANS_COUNT field.
	DMA_CH7_AL2_TRANS_COUNT_CH7_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH7_AL2_READ_ADDR: Alias for channel 7 READ_ADDR register
	// Position of CH7_AL2_READ_ADDR field.
	DMA_CH7_AL2_READ_ADDR_CH7_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH7_AL2_READ_ADDR field.
	DMA_CH7_AL2_READ_ADDR_CH7_AL2_READ_ADDR_Msk = 0xffffffff

	// CH7_AL2_WRITE_ADDR_TRIG: Alias for channel 7 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH7_AL2_WRITE_ADDR_TRIG field.
	DMA_CH7_AL2_WRITE_ADDR_TRIG_CH7_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH7_AL2_WRITE_ADDR_TRIG field.
	DMA_CH7_AL2_WRITE_ADDR_TRIG_CH7_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH7_AL3_CTRL: Alias for channel 7 CTRL register
	// Position of CH7_AL3_CTRL field.
	DMA_CH7_AL3_CTRL_CH7_AL3_CTRL_Pos = 0x0
	// Bit mask of CH7_AL3_CTRL field.
	DMA_CH7_AL3_CTRL_CH7_AL3_CTRL_Msk = 0xffffffff

	// CH7_AL3_WRITE_ADDR: Alias for channel 7 WRITE_ADDR register
	// Position of CH7_AL3_WRITE_ADDR field.
	DMA_CH7_AL3_WRITE_ADDR_CH7_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH7_AL3_WRITE_ADDR field.
	DMA_CH7_AL3_WRITE_ADDR_CH7_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH7_AL3_TRANS_COUNT: Alias for channel 7 TRANS_COUNT register
	// Position of CH7_AL3_TRANS_COUNT field.
	DMA_CH7_AL3_TRANS_COUNT_CH7_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH7_AL3_TRANS_COUNT field.
	DMA_CH7_AL3_TRANS_COUNT_CH7_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH7_AL3_READ_ADDR_TRIG: Alias for channel 7 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH7_AL3_READ_ADDR_TRIG field.
	DMA_CH7_AL3_READ_ADDR_TRIG_CH7_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH7_AL3_READ_ADDR_TRIG field.
	DMA_CH7_AL3_READ_ADDR_TRIG_CH7_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH8_READ_ADDR: DMA Channel 8 Read Address pointer
	// Position of CH8_READ_ADDR field.
	DMA_CH8_READ_ADDR_CH8_READ_ADDR_Pos = 0x0
	// Bit mask of CH8_READ_ADDR field.
	DMA_CH8_READ_ADDR_CH8_READ_ADDR_Msk = 0xffffffff

	// CH8_WRITE_ADDR: DMA Channel 8 Write Address pointer
	// Position of CH8_WRITE_ADDR field.
	DMA_CH8_WRITE_ADDR_CH8_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH8_WRITE_ADDR field.
	DMA_CH8_WRITE_ADDR_CH8_WRITE_ADDR_Msk = 0xffffffff

	// CH8_TRANS_COUNT: DMA Channel 8 Transfer Count
	// Position of MODE field.
	DMA_CH8_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH8_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH8_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH8_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH8_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH8_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH8_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH8_CTRL_TRIG: DMA Channel 8 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH8_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH8_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH8_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH8_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH8_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH8_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH8_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH8_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH8_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH8_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH8_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH8_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH8_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH8_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH8_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH8_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH8_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH8_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH8_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH8_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH8_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH8_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH8_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH8_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH8_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH8_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH8_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH8_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH8_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH8_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH8_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH8_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH8_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH8_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH8_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH8_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH8_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH8_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH8_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH8_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH8_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH8_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH8_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH8_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH8_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH8_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH8_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH8_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH8_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH8_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH8_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH8_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH8_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH8_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH8_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH8_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH8_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH8_CTRL_TRIG_EN = 0x1

	// CH8_AL1_CTRL: Alias for channel 8 CTRL register
	// Position of CH8_AL1_CTRL field.
	DMA_CH8_AL1_CTRL_CH8_AL1_CTRL_Pos = 0x0
	// Bit mask of CH8_AL1_CTRL field.
	DMA_CH8_AL1_CTRL_CH8_AL1_CTRL_Msk = 0xffffffff

	// CH8_AL1_READ_ADDR: Alias for channel 8 READ_ADDR register
	// Position of CH8_AL1_READ_ADDR field.
	DMA_CH8_AL1_READ_ADDR_CH8_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH8_AL1_READ_ADDR field.
	DMA_CH8_AL1_READ_ADDR_CH8_AL1_READ_ADDR_Msk = 0xffffffff

	// CH8_AL1_WRITE_ADDR: Alias for channel 8 WRITE_ADDR register
	// Position of CH8_AL1_WRITE_ADDR field.
	DMA_CH8_AL1_WRITE_ADDR_CH8_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH8_AL1_WRITE_ADDR field.
	DMA_CH8_AL1_WRITE_ADDR_CH8_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH8_AL1_TRANS_COUNT_TRIG: Alias for channel 8 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH8_AL1_TRANS_COUNT_TRIG field.
	DMA_CH8_AL1_TRANS_COUNT_TRIG_CH8_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH8_AL1_TRANS_COUNT_TRIG field.
	DMA_CH8_AL1_TRANS_COUNT_TRIG_CH8_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH8_AL2_CTRL: Alias for channel 8 CTRL register
	// Position of CH8_AL2_CTRL field.
	DMA_CH8_AL2_CTRL_CH8_AL2_CTRL_Pos = 0x0
	// Bit mask of CH8_AL2_CTRL field.
	DMA_CH8_AL2_CTRL_CH8_AL2_CTRL_Msk = 0xffffffff

	// CH8_AL2_TRANS_COUNT: Alias for channel 8 TRANS_COUNT register
	// Position of CH8_AL2_TRANS_COUNT field.
	DMA_CH8_AL2_TRANS_COUNT_CH8_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH8_AL2_TRANS_COUNT field.
	DMA_CH8_AL2_TRANS_COUNT_CH8_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH8_AL2_READ_ADDR: Alias for channel 8 READ_ADDR register
	// Position of CH8_AL2_READ_ADDR field.
	DMA_CH8_AL2_READ_ADDR_CH8_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH8_AL2_READ_ADDR field.
	DMA_CH8_AL2_READ_ADDR_CH8_AL2_READ_ADDR_Msk = 0xffffffff

	// CH8_AL2_WRITE_ADDR_TRIG: Alias for channel 8 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH8_AL2_WRITE_ADDR_TRIG field.
	DMA_CH8_AL2_WRITE_ADDR_TRIG_CH8_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH8_AL2_WRITE_ADDR_TRIG field.
	DMA_CH8_AL2_WRITE_ADDR_TRIG_CH8_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH8_AL3_CTRL: Alias for channel 8 CTRL register
	// Position of CH8_AL3_CTRL field.
	DMA_CH8_AL3_CTRL_CH8_AL3_CTRL_Pos = 0x0
	// Bit mask of CH8_AL3_CTRL field.
	DMA_CH8_AL3_CTRL_CH8_AL3_CTRL_Msk = 0xffffffff

	// CH8_AL3_WRITE_ADDR: Alias for channel 8 WRITE_ADDR register
	// Position of CH8_AL3_WRITE_ADDR field.
	DMA_CH8_AL3_WRITE_ADDR_CH8_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH8_AL3_WRITE_ADDR field.
	DMA_CH8_AL3_WRITE_ADDR_CH8_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH8_AL3_TRANS_COUNT: Alias for channel 8 TRANS_COUNT register
	// Position of CH8_AL3_TRANS_COUNT field.
	DMA_CH8_AL3_TRANS_COUNT_CH8_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH8_AL3_TRANS_COUNT field.
	DMA_CH8_AL3_TRANS_COUNT_CH8_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH8_AL3_READ_ADDR_TRIG: Alias for channel 8 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH8_AL3_READ_ADDR_TRIG field.
	DMA_CH8_AL3_READ_ADDR_TRIG_CH8_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH8_AL3_READ_ADDR_TRIG field.
	DMA_CH8_AL3_READ_ADDR_TRIG_CH8_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH9_READ_ADDR: DMA Channel 9 Read Address pointer
	// Position of CH9_READ_ADDR field.
	DMA_CH9_READ_ADDR_CH9_READ_ADDR_Pos = 0x0
	// Bit mask of CH9_READ_ADDR field.
	DMA_CH9_READ_ADDR_CH9_READ_ADDR_Msk = 0xffffffff

	// CH9_WRITE_ADDR: DMA Channel 9 Write Address pointer
	// Position of CH9_WRITE_ADDR field.
	DMA_CH9_WRITE_ADDR_CH9_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH9_WRITE_ADDR field.
	DMA_CH9_WRITE_ADDR_CH9_WRITE_ADDR_Msk = 0xffffffff

	// CH9_TRANS_COUNT: DMA Channel 9 Transfer Count
	// Position of MODE field.
	DMA_CH9_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH9_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH9_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH9_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH9_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH9_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH9_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH9_CTRL_TRIG: DMA Channel 9 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH9_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH9_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH9_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH9_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH9_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH9_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH9_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH9_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH9_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH9_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH9_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH9_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH9_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH9_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH9_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH9_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH9_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH9_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH9_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH9_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH9_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH9_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH9_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH9_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH9_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH9_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH9_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH9_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH9_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH9_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH9_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH9_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH9_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH9_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH9_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH9_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH9_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH9_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH9_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH9_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH9_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH9_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH9_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH9_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH9_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH9_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH9_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH9_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH9_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH9_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH9_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH9_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH9_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH9_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH9_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH9_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH9_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH9_CTRL_TRIG_EN = 0x1

	// CH9_AL1_CTRL: Alias for channel 9 CTRL register
	// Position of CH9_AL1_CTRL field.
	DMA_CH9_AL1_CTRL_CH9_AL1_CTRL_Pos = 0x0
	// Bit mask of CH9_AL1_CTRL field.
	DMA_CH9_AL1_CTRL_CH9_AL1_CTRL_Msk = 0xffffffff

	// CH9_AL1_READ_ADDR: Alias for channel 9 READ_ADDR register
	// Position of CH9_AL1_READ_ADDR field.
	DMA_CH9_AL1_READ_ADDR_CH9_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH9_AL1_READ_ADDR field.
	DMA_CH9_AL1_READ_ADDR_CH9_AL1_READ_ADDR_Msk = 0xffffffff

	// CH9_AL1_WRITE_ADDR: Alias for channel 9 WRITE_ADDR register
	// Position of CH9_AL1_WRITE_ADDR field.
	DMA_CH9_AL1_WRITE_ADDR_CH9_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH9_AL1_WRITE_ADDR field.
	DMA_CH9_AL1_WRITE_ADDR_CH9_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH9_AL1_TRANS_COUNT_TRIG: Alias for channel 9 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH9_AL1_TRANS_COUNT_TRIG field.
	DMA_CH9_AL1_TRANS_COUNT_TRIG_CH9_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH9_AL1_TRANS_COUNT_TRIG field.
	DMA_CH9_AL1_TRANS_COUNT_TRIG_CH9_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH9_AL2_CTRL: Alias for channel 9 CTRL register
	// Position of CH9_AL2_CTRL field.
	DMA_CH9_AL2_CTRL_CH9_AL2_CTRL_Pos = 0x0
	// Bit mask of CH9_AL2_CTRL field.
	DMA_CH9_AL2_CTRL_CH9_AL2_CTRL_Msk = 0xffffffff

	// CH9_AL2_TRANS_COUNT: Alias for channel 9 TRANS_COUNT register
	// Position of CH9_AL2_TRANS_COUNT field.
	DMA_CH9_AL2_TRANS_COUNT_CH9_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH9_AL2_TRANS_COUNT field.
	DMA_CH9_AL2_TRANS_COUNT_CH9_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH9_AL2_READ_ADDR: Alias for channel 9 READ_ADDR register
	// Position of CH9_AL2_READ_ADDR field.
	DMA_CH9_AL2_READ_ADDR_CH9_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH9_AL2_READ_ADDR field.
	DMA_CH9_AL2_READ_ADDR_CH9_AL2_READ_ADDR_Msk = 0xffffffff

	// CH9_AL2_WRITE_ADDR_TRIG: Alias for channel 9 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH9_AL2_WRITE_ADDR_TRIG field.
	DMA_CH9_AL2_WRITE_ADDR_TRIG_CH9_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH9_AL2_WRITE_ADDR_TRIG field.
	DMA_CH9_AL2_WRITE_ADDR_TRIG_CH9_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH9_AL3_CTRL: Alias for channel 9 CTRL register
	// Position of CH9_AL3_CTRL field.
	DMA_CH9_AL3_CTRL_CH9_AL3_CTRL_Pos = 0x0
	// Bit mask of CH9_AL3_CTRL field.
	DMA_CH9_AL3_CTRL_CH9_AL3_CTRL_Msk = 0xffffffff

	// CH9_AL3_WRITE_ADDR: Alias for channel 9 WRITE_ADDR register
	// Position of CH9_AL3_WRITE_ADDR field.
	DMA_CH9_AL3_WRITE_ADDR_CH9_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH9_AL3_WRITE_ADDR field.
	DMA_CH9_AL3_WRITE_ADDR_CH9_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH9_AL3_TRANS_COUNT: Alias for channel 9 TRANS_COUNT register
	// Position of CH9_AL3_TRANS_COUNT field.
	DMA_CH9_AL3_TRANS_COUNT_CH9_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH9_AL3_TRANS_COUNT field.
	DMA_CH9_AL3_TRANS_COUNT_CH9_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH9_AL3_READ_ADDR_TRIG: Alias for channel 9 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH9_AL3_READ_ADDR_TRIG field.
	DMA_CH9_AL3_READ_ADDR_TRIG_CH9_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH9_AL3_READ_ADDR_TRIG field.
	DMA_CH9_AL3_READ_ADDR_TRIG_CH9_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH10_READ_ADDR: DMA Channel 10 Read Address pointer
	// Position of CH10_READ_ADDR field.
	DMA_CH10_READ_ADDR_CH10_READ_ADDR_Pos = 0x0
	// Bit mask of CH10_READ_ADDR field.
	DMA_CH10_READ_ADDR_CH10_READ_ADDR_Msk = 0xffffffff

	// CH10_WRITE_ADDR: DMA Channel 10 Write Address pointer
	// Position of CH10_WRITE_ADDR field.
	DMA_CH10_WRITE_ADDR_CH10_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH10_WRITE_ADDR field.
	DMA_CH10_WRITE_ADDR_CH10_WRITE_ADDR_Msk = 0xffffffff

	// CH10_TRANS_COUNT: DMA Channel 10 Transfer Count
	// Position of MODE field.
	DMA_CH10_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH10_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH10_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH10_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH10_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH10_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH10_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH10_CTRL_TRIG: DMA Channel 10 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH10_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH10_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH10_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH10_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH10_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH10_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH10_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH10_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH10_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH10_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH10_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH10_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH10_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH10_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH10_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH10_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH10_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH10_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH10_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH10_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH10_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH10_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH10_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH10_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH10_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH10_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH10_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH10_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH10_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH10_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH10_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH10_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH10_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH10_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH10_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH10_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH10_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH10_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH10_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH10_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH10_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH10_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH10_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH10_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH10_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH10_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH10_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH10_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH10_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH10_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH10_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH10_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH10_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH10_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH10_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH10_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH10_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH10_CTRL_TRIG_EN = 0x1

	// CH10_AL1_CTRL: Alias for channel 10 CTRL register
	// Position of CH10_AL1_CTRL field.
	DMA_CH10_AL1_CTRL_CH10_AL1_CTRL_Pos = 0x0
	// Bit mask of CH10_AL1_CTRL field.
	DMA_CH10_AL1_CTRL_CH10_AL1_CTRL_Msk = 0xffffffff

	// CH10_AL1_READ_ADDR: Alias for channel 10 READ_ADDR register
	// Position of CH10_AL1_READ_ADDR field.
	DMA_CH10_AL1_READ_ADDR_CH10_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH10_AL1_READ_ADDR field.
	DMA_CH10_AL1_READ_ADDR_CH10_AL1_READ_ADDR_Msk = 0xffffffff

	// CH10_AL1_WRITE_ADDR: Alias for channel 10 WRITE_ADDR register
	// Position of CH10_AL1_WRITE_ADDR field.
	DMA_CH10_AL1_WRITE_ADDR_CH10_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH10_AL1_WRITE_ADDR field.
	DMA_CH10_AL1_WRITE_ADDR_CH10_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH10_AL1_TRANS_COUNT_TRIG: Alias for channel 10 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH10_AL1_TRANS_COUNT_TRIG field.
	DMA_CH10_AL1_TRANS_COUNT_TRIG_CH10_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH10_AL1_TRANS_COUNT_TRIG field.
	DMA_CH10_AL1_TRANS_COUNT_TRIG_CH10_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH10_AL2_CTRL: Alias for channel 10 CTRL register
	// Position of CH10_AL2_CTRL field.
	DMA_CH10_AL2_CTRL_CH10_AL2_CTRL_Pos = 0x0
	// Bit mask of CH10_AL2_CTRL field.
	DMA_CH10_AL2_CTRL_CH10_AL2_CTRL_Msk = 0xffffffff

	// CH10_AL2_TRANS_COUNT: Alias for channel 10 TRANS_COUNT register
	// Position of CH10_AL2_TRANS_COUNT field.
	DMA_CH10_AL2_TRANS_COUNT_CH10_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH10_AL2_TRANS_COUNT field.
	DMA_CH10_AL2_TRANS_COUNT_CH10_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH10_AL2_READ_ADDR: Alias for channel 10 READ_ADDR register
	// Position of CH10_AL2_READ_ADDR field.
	DMA_CH10_AL2_READ_ADDR_CH10_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH10_AL2_READ_ADDR field.
	DMA_CH10_AL2_READ_ADDR_CH10_AL2_READ_ADDR_Msk = 0xffffffff

	// CH10_AL2_WRITE_ADDR_TRIG: Alias for channel 10 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH10_AL2_WRITE_ADDR_TRIG field.
	DMA_CH10_AL2_WRITE_ADDR_TRIG_CH10_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH10_AL2_WRITE_ADDR_TRIG field.
	DMA_CH10_AL2_WRITE_ADDR_TRIG_CH10_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH10_AL3_CTRL: Alias for channel 10 CTRL register
	// Position of CH10_AL3_CTRL field.
	DMA_CH10_AL3_CTRL_CH10_AL3_CTRL_Pos = 0x0
	// Bit mask of CH10_AL3_CTRL field.
	DMA_CH10_AL3_CTRL_CH10_AL3_CTRL_Msk = 0xffffffff

	// CH10_AL3_WRITE_ADDR: Alias for channel 10 WRITE_ADDR register
	// Position of CH10_AL3_WRITE_ADDR field.
	DMA_CH10_AL3_WRITE_ADDR_CH10_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH10_AL3_WRITE_ADDR field.
	DMA_CH10_AL3_WRITE_ADDR_CH10_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH10_AL3_TRANS_COUNT: Alias for channel 10 TRANS_COUNT register
	// Position of CH10_AL3_TRANS_COUNT field.
	DMA_CH10_AL3_TRANS_COUNT_CH10_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH10_AL3_TRANS_COUNT field.
	DMA_CH10_AL3_TRANS_COUNT_CH10_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH10_AL3_READ_ADDR_TRIG: Alias for channel 10 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH10_AL3_READ_ADDR_TRIG field.
	DMA_CH10_AL3_READ_ADDR_TRIG_CH10_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH10_AL3_READ_ADDR_TRIG field.
	DMA_CH10_AL3_READ_ADDR_TRIG_CH10_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH11_READ_ADDR: DMA Channel 11 Read Address pointer
	// Position of CH11_READ_ADDR field.
	DMA_CH11_READ_ADDR_CH11_READ_ADDR_Pos = 0x0
	// Bit mask of CH11_READ_ADDR field.
	DMA_CH11_READ_ADDR_CH11_READ_ADDR_Msk = 0xffffffff

	// CH11_WRITE_ADDR: DMA Channel 11 Write Address pointer
	// Position of CH11_WRITE_ADDR field.
	DMA_CH11_WRITE_ADDR_CH11_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH11_WRITE_ADDR field.
	DMA_CH11_WRITE_ADDR_CH11_WRITE_ADDR_Msk = 0xffffffff

	// CH11_TRANS_COUNT: DMA Channel 11 Transfer Count
	// Position of MODE field.
	DMA_CH11_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH11_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH11_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH11_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH11_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH11_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH11_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH11_CTRL_TRIG: DMA Channel 11 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH11_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH11_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH11_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH11_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH11_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH11_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH11_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH11_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH11_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH11_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH11_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH11_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH11_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH11_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH11_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH11_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH11_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH11_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH11_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH11_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH11_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH11_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH11_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH11_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH11_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH11_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH11_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH11_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH11_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH11_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH11_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH11_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH11_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH11_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH11_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH11_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH11_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH11_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH11_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH11_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH11_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH11_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH11_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH11_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH11_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH11_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH11_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH11_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH11_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH11_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH11_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH11_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH11_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH11_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH11_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH11_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH11_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH11_CTRL_TRIG_EN = 0x1

	// CH11_AL1_CTRL: Alias for channel 11 CTRL register
	// Position of CH11_AL1_CTRL field.
	DMA_CH11_AL1_CTRL_CH11_AL1_CTRL_Pos = 0x0
	// Bit mask of CH11_AL1_CTRL field.
	DMA_CH11_AL1_CTRL_CH11_AL1_CTRL_Msk = 0xffffffff

	// CH11_AL1_READ_ADDR: Alias for channel 11 READ_ADDR register
	// Position of CH11_AL1_READ_ADDR field.
	DMA_CH11_AL1_READ_ADDR_CH11_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH11_AL1_READ_ADDR field.
	DMA_CH11_AL1_READ_ADDR_CH11_AL1_READ_ADDR_Msk = 0xffffffff

	// CH11_AL1_WRITE_ADDR: Alias for channel 11 WRITE_ADDR register
	// Position of CH11_AL1_WRITE_ADDR field.
	DMA_CH11_AL1_WRITE_ADDR_CH11_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH11_AL1_WRITE_ADDR field.
	DMA_CH11_AL1_WRITE_ADDR_CH11_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH11_AL1_TRANS_COUNT_TRIG: Alias for channel 11 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH11_AL1_TRANS_COUNT_TRIG field.
	DMA_CH11_AL1_TRANS_COUNT_TRIG_CH11_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH11_AL1_TRANS_COUNT_TRIG field.
	DMA_CH11_AL1_TRANS_COUNT_TRIG_CH11_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH11_AL2_CTRL: Alias for channel 11 CTRL register
	// Position of CH11_AL2_CTRL field.
	DMA_CH11_AL2_CTRL_CH11_AL2_CTRL_Pos = 0x0
	// Bit mask of CH11_AL2_CTRL field.
	DMA_CH11_AL2_CTRL_CH11_AL2_CTRL_Msk = 0xffffffff

	// CH11_AL2_TRANS_COUNT: Alias for channel 11 TRANS_COUNT register
	// Position of CH11_AL2_TRANS_COUNT field.
	DMA_CH11_AL2_TRANS_COUNT_CH11_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH11_AL2_TRANS_COUNT field.
	DMA_CH11_AL2_TRANS_COUNT_CH11_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH11_AL2_READ_ADDR: Alias for channel 11 READ_ADDR register
	// Position of CH11_AL2_READ_ADDR field.
	DMA_CH11_AL2_READ_ADDR_CH11_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH11_AL2_READ_ADDR field.
	DMA_CH11_AL2_READ_ADDR_CH11_AL2_READ_ADDR_Msk = 0xffffffff

	// CH11_AL2_WRITE_ADDR_TRIG: Alias for channel 11 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH11_AL2_WRITE_ADDR_TRIG field.
	DMA_CH11_AL2_WRITE_ADDR_TRIG_CH11_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH11_AL2_WRITE_ADDR_TRIG field.
	DMA_CH11_AL2_WRITE_ADDR_TRIG_CH11_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH11_AL3_CTRL: Alias for channel 11 CTRL register
	// Position of CH11_AL3_CTRL field.
	DMA_CH11_AL3_CTRL_CH11_AL3_CTRL_Pos = 0x0
	// Bit mask of CH11_AL3_CTRL field.
	DMA_CH11_AL3_CTRL_CH11_AL3_CTRL_Msk = 0xffffffff

	// CH11_AL3_WRITE_ADDR: Alias for channel 11 WRITE_ADDR register
	// Position of CH11_AL3_WRITE_ADDR field.
	DMA_CH11_AL3_WRITE_ADDR_CH11_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH11_AL3_WRITE_ADDR field.
	DMA_CH11_AL3_WRITE_ADDR_CH11_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH11_AL3_TRANS_COUNT: Alias for channel 11 TRANS_COUNT register
	// Position of CH11_AL3_TRANS_COUNT field.
	DMA_CH11_AL3_TRANS_COUNT_CH11_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH11_AL3_TRANS_COUNT field.
	DMA_CH11_AL3_TRANS_COUNT_CH11_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH11_AL3_READ_ADDR_TRIG: Alias for channel 11 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH11_AL3_READ_ADDR_TRIG field.
	DMA_CH11_AL3_READ_ADDR_TRIG_CH11_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH11_AL3_READ_ADDR_TRIG field.
	DMA_CH11_AL3_READ_ADDR_TRIG_CH11_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH12_READ_ADDR: DMA Channel 12 Read Address pointer
	// Position of CH12_READ_ADDR field.
	DMA_CH12_READ_ADDR_CH12_READ_ADDR_Pos = 0x0
	// Bit mask of CH12_READ_ADDR field.
	DMA_CH12_READ_ADDR_CH12_READ_ADDR_Msk = 0xffffffff

	// CH12_WRITE_ADDR: DMA Channel 12 Write Address pointer
	// Position of CH12_WRITE_ADDR field.
	DMA_CH12_WRITE_ADDR_CH12_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH12_WRITE_ADDR field.
	DMA_CH12_WRITE_ADDR_CH12_WRITE_ADDR_Msk = 0xffffffff

	// CH12_TRANS_COUNT: DMA Channel 12 Transfer Count
	// Position of MODE field.
	DMA_CH12_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH12_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH12_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH12_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH12_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH12_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH12_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH12_CTRL_TRIG: DMA Channel 12 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH12_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH12_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH12_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH12_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH12_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH12_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH12_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH12_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH12_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH12_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH12_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH12_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH12_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH12_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH12_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH12_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH12_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH12_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH12_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH12_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH12_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH12_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH12_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH12_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH12_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH12_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH12_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH12_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH12_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH12_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH12_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH12_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH12_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH12_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH12_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH12_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH12_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH12_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH12_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH12_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH12_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH12_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH12_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH12_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH12_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH12_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH12_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH12_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH12_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH12_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH12_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH12_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH12_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH12_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH12_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH12_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH12_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH12_CTRL_TRIG_EN = 0x1

	// CH12_AL1_CTRL: Alias for channel 12 CTRL register
	// Position of CH12_AL1_CTRL field.
	DMA_CH12_AL1_CTRL_CH12_AL1_CTRL_Pos = 0x0
	// Bit mask of CH12_AL1_CTRL field.
	DMA_CH12_AL1_CTRL_CH12_AL1_CTRL_Msk = 0xffffffff

	// CH12_AL1_READ_ADDR: Alias for channel 12 READ_ADDR register
	// Position of CH12_AL1_READ_ADDR field.
	DMA_CH12_AL1_READ_ADDR_CH12_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH12_AL1_READ_ADDR field.
	DMA_CH12_AL1_READ_ADDR_CH12_AL1_READ_ADDR_Msk = 0xffffffff

	// CH12_AL1_WRITE_ADDR: Alias for channel 12 WRITE_ADDR register
	// Position of CH12_AL1_WRITE_ADDR field.
	DMA_CH12_AL1_WRITE_ADDR_CH12_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH12_AL1_WRITE_ADDR field.
	DMA_CH12_AL1_WRITE_ADDR_CH12_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH12_AL1_TRANS_COUNT_TRIG: Alias for channel 12 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH12_AL1_TRANS_COUNT_TRIG field.
	DMA_CH12_AL1_TRANS_COUNT_TRIG_CH12_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH12_AL1_TRANS_COUNT_TRIG field.
	DMA_CH12_AL1_TRANS_COUNT_TRIG_CH12_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH12_AL2_CTRL: Alias for channel 12 CTRL register
	// Position of CH12_AL2_CTRL field.
	DMA_CH12_AL2_CTRL_CH12_AL2_CTRL_Pos = 0x0
	// Bit mask of CH12_AL2_CTRL field.
	DMA_CH12_AL2_CTRL_CH12_AL2_CTRL_Msk = 0xffffffff

	// CH12_AL2_TRANS_COUNT: Alias for channel 12 TRANS_COUNT register
	// Position of CH12_AL2_TRANS_COUNT field.
	DMA_CH12_AL2_TRANS_COUNT_CH12_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH12_AL2_TRANS_COUNT field.
	DMA_CH12_AL2_TRANS_COUNT_CH12_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH12_AL2_READ_ADDR: Alias for channel 12 READ_ADDR register
	// Position of CH12_AL2_READ_ADDR field.
	DMA_CH12_AL2_READ_ADDR_CH12_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH12_AL2_READ_ADDR field.
	DMA_CH12_AL2_READ_ADDR_CH12_AL2_READ_ADDR_Msk = 0xffffffff

	// CH12_AL2_WRITE_ADDR_TRIG: Alias for channel 12 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH12_AL2_WRITE_ADDR_TRIG field.
	DMA_CH12_AL2_WRITE_ADDR_TRIG_CH12_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH12_AL2_WRITE_ADDR_TRIG field.
	DMA_CH12_AL2_WRITE_ADDR_TRIG_CH12_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH12_AL3_CTRL: Alias for channel 12 CTRL register
	// Position of CH12_AL3_CTRL field.
	DMA_CH12_AL3_CTRL_CH12_AL3_CTRL_Pos = 0x0
	// Bit mask of CH12_AL3_CTRL field.
	DMA_CH12_AL3_CTRL_CH12_AL3_CTRL_Msk = 0xffffffff

	// CH12_AL3_WRITE_ADDR: Alias for channel 12 WRITE_ADDR register
	// Position of CH12_AL3_WRITE_ADDR field.
	DMA_CH12_AL3_WRITE_ADDR_CH12_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH12_AL3_WRITE_ADDR field.
	DMA_CH12_AL3_WRITE_ADDR_CH12_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH12_AL3_TRANS_COUNT: Alias for channel 12 TRANS_COUNT register
	// Position of CH12_AL3_TRANS_COUNT field.
	DMA_CH12_AL3_TRANS_COUNT_CH12_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH12_AL3_TRANS_COUNT field.
	DMA_CH12_AL3_TRANS_COUNT_CH12_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH12_AL3_READ_ADDR_TRIG: Alias for channel 12 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH12_AL3_READ_ADDR_TRIG field.
	DMA_CH12_AL3_READ_ADDR_TRIG_CH12_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH12_AL3_READ_ADDR_TRIG field.
	DMA_CH12_AL3_READ_ADDR_TRIG_CH12_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH13_READ_ADDR: DMA Channel 13 Read Address pointer
	// Position of CH13_READ_ADDR field.
	DMA_CH13_READ_ADDR_CH13_READ_ADDR_Pos = 0x0
	// Bit mask of CH13_READ_ADDR field.
	DMA_CH13_READ_ADDR_CH13_READ_ADDR_Msk = 0xffffffff

	// CH13_WRITE_ADDR: DMA Channel 13 Write Address pointer
	// Position of CH13_WRITE_ADDR field.
	DMA_CH13_WRITE_ADDR_CH13_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH13_WRITE_ADDR field.
	DMA_CH13_WRITE_ADDR_CH13_WRITE_ADDR_Msk = 0xffffffff

	// CH13_TRANS_COUNT: DMA Channel 13 Transfer Count
	// Position of MODE field.
	DMA_CH13_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH13_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH13_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH13_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH13_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH13_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH13_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH13_CTRL_TRIG: DMA Channel 13 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH13_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH13_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH13_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH13_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH13_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH13_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH13_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH13_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH13_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH13_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH13_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH13_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH13_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH13_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH13_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH13_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH13_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH13_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH13_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH13_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH13_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH13_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH13_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH13_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH13_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH13_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH13_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH13_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH13_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH13_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH13_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH13_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH13_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH13_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH13_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH13_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH13_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH13_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH13_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH13_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH13_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH13_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH13_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH13_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH13_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH13_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH13_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH13_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH13_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH13_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH13_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH13_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH13_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH13_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH13_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH13_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH13_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH13_CTRL_TRIG_EN = 0x1

	// CH13_AL1_CTRL: Alias for channel 13 CTRL register
	// Position of CH13_AL1_CTRL field.
	DMA_CH13_AL1_CTRL_CH13_AL1_CTRL_Pos = 0x0
	// Bit mask of CH13_AL1_CTRL field.
	DMA_CH13_AL1_CTRL_CH13_AL1_CTRL_Msk = 0xffffffff

	// CH13_AL1_READ_ADDR: Alias for channel 13 READ_ADDR register
	// Position of CH13_AL1_READ_ADDR field.
	DMA_CH13_AL1_READ_ADDR_CH13_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH13_AL1_READ_ADDR field.
	DMA_CH13_AL1_READ_ADDR_CH13_AL1_READ_ADDR_Msk = 0xffffffff

	// CH13_AL1_WRITE_ADDR: Alias for channel 13 WRITE_ADDR register
	// Position of CH13_AL1_WRITE_ADDR field.
	DMA_CH13_AL1_WRITE_ADDR_CH13_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH13_AL1_WRITE_ADDR field.
	DMA_CH13_AL1_WRITE_ADDR_CH13_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH13_AL1_TRANS_COUNT_TRIG: Alias for channel 13 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH13_AL1_TRANS_COUNT_TRIG field.
	DMA_CH13_AL1_TRANS_COUNT_TRIG_CH13_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH13_AL1_TRANS_COUNT_TRIG field.
	DMA_CH13_AL1_TRANS_COUNT_TRIG_CH13_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH13_AL2_CTRL: Alias for channel 13 CTRL register
	// Position of CH13_AL2_CTRL field.
	DMA_CH13_AL2_CTRL_CH13_AL2_CTRL_Pos = 0x0
	// Bit mask of CH13_AL2_CTRL field.
	DMA_CH13_AL2_CTRL_CH13_AL2_CTRL_Msk = 0xffffffff

	// CH13_AL2_TRANS_COUNT: Alias for channel 13 TRANS_COUNT register
	// Position of CH13_AL2_TRANS_COUNT field.
	DMA_CH13_AL2_TRANS_COUNT_CH13_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH13_AL2_TRANS_COUNT field.
	DMA_CH13_AL2_TRANS_COUNT_CH13_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH13_AL2_READ_ADDR: Alias for channel 13 READ_ADDR register
	// Position of CH13_AL2_READ_ADDR field.
	DMA_CH13_AL2_READ_ADDR_CH13_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH13_AL2_READ_ADDR field.
	DMA_CH13_AL2_READ_ADDR_CH13_AL2_READ_ADDR_Msk = 0xffffffff

	// CH13_AL2_WRITE_ADDR_TRIG: Alias for channel 13 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH13_AL2_WRITE_ADDR_TRIG field.
	DMA_CH13_AL2_WRITE_ADDR_TRIG_CH13_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH13_AL2_WRITE_ADDR_TRIG field.
	DMA_CH13_AL2_WRITE_ADDR_TRIG_CH13_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH13_AL3_CTRL: Alias for channel 13 CTRL register
	// Position of CH13_AL3_CTRL field.
	DMA_CH13_AL3_CTRL_CH13_AL3_CTRL_Pos = 0x0
	// Bit mask of CH13_AL3_CTRL field.
	DMA_CH13_AL3_CTRL_CH13_AL3_CTRL_Msk = 0xffffffff

	// CH13_AL3_WRITE_ADDR: Alias for channel 13 WRITE_ADDR register
	// Position of CH13_AL3_WRITE_ADDR field.
	DMA_CH13_AL3_WRITE_ADDR_CH13_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH13_AL3_WRITE_ADDR field.
	DMA_CH13_AL3_WRITE_ADDR_CH13_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH13_AL3_TRANS_COUNT: Alias for channel 13 TRANS_COUNT register
	// Position of CH13_AL3_TRANS_COUNT field.
	DMA_CH13_AL3_TRANS_COUNT_CH13_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH13_AL3_TRANS_COUNT field.
	DMA_CH13_AL3_TRANS_COUNT_CH13_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH13_AL3_READ_ADDR_TRIG: Alias for channel 13 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH13_AL3_READ_ADDR_TRIG field.
	DMA_CH13_AL3_READ_ADDR_TRIG_CH13_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH13_AL3_READ_ADDR_TRIG field.
	DMA_CH13_AL3_READ_ADDR_TRIG_CH13_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH14_READ_ADDR: DMA Channel 14 Read Address pointer
	// Position of CH14_READ_ADDR field.
	DMA_CH14_READ_ADDR_CH14_READ_ADDR_Pos = 0x0
	// Bit mask of CH14_READ_ADDR field.
	DMA_CH14_READ_ADDR_CH14_READ_ADDR_Msk = 0xffffffff

	// CH14_WRITE_ADDR: DMA Channel 14 Write Address pointer
	// Position of CH14_WRITE_ADDR field.
	DMA_CH14_WRITE_ADDR_CH14_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH14_WRITE_ADDR field.
	DMA_CH14_WRITE_ADDR_CH14_WRITE_ADDR_Msk = 0xffffffff

	// CH14_TRANS_COUNT: DMA Channel 14 Transfer Count
	// Position of MODE field.
	DMA_CH14_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH14_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH14_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH14_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH14_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH14_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH14_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH14_CTRL_TRIG: DMA Channel 14 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH14_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH14_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH14_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH14_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH14_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH14_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH14_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH14_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH14_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH14_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH14_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH14_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH14_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH14_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH14_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH14_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH14_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH14_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH14_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH14_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH14_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH14_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH14_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH14_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH14_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH14_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH14_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH14_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH14_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH14_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH14_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH14_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH14_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH14_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH14_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH14_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH14_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH14_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH14_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH14_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH14_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH14_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH14_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH14_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH14_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH14_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH14_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH14_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH14_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH14_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH14_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH14_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH14_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH14_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH14_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH14_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH14_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH14_CTRL_TRIG_EN = 0x1

	// CH14_AL1_CTRL: Alias for channel 14 CTRL register
	// Position of CH14_AL1_CTRL field.
	DMA_CH14_AL1_CTRL_CH14_AL1_CTRL_Pos = 0x0
	// Bit mask of CH14_AL1_CTRL field.
	DMA_CH14_AL1_CTRL_CH14_AL1_CTRL_Msk = 0xffffffff

	// CH14_AL1_READ_ADDR: Alias for channel 14 READ_ADDR register
	// Position of CH14_AL1_READ_ADDR field.
	DMA_CH14_AL1_READ_ADDR_CH14_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH14_AL1_READ_ADDR field.
	DMA_CH14_AL1_READ_ADDR_CH14_AL1_READ_ADDR_Msk = 0xffffffff

	// CH14_AL1_WRITE_ADDR: Alias for channel 14 WRITE_ADDR register
	// Position of CH14_AL1_WRITE_ADDR field.
	DMA_CH14_AL1_WRITE_ADDR_CH14_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH14_AL1_WRITE_ADDR field.
	DMA_CH14_AL1_WRITE_ADDR_CH14_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH14_AL1_TRANS_COUNT_TRIG: Alias for channel 14 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH14_AL1_TRANS_COUNT_TRIG field.
	DMA_CH14_AL1_TRANS_COUNT_TRIG_CH14_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH14_AL1_TRANS_COUNT_TRIG field.
	DMA_CH14_AL1_TRANS_COUNT_TRIG_CH14_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH14_AL2_CTRL: Alias for channel 14 CTRL register
	// Position of CH14_AL2_CTRL field.
	DMA_CH14_AL2_CTRL_CH14_AL2_CTRL_Pos = 0x0
	// Bit mask of CH14_AL2_CTRL field.
	DMA_CH14_AL2_CTRL_CH14_AL2_CTRL_Msk = 0xffffffff

	// CH14_AL2_TRANS_COUNT: Alias for channel 14 TRANS_COUNT register
	// Position of CH14_AL2_TRANS_COUNT field.
	DMA_CH14_AL2_TRANS_COUNT_CH14_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH14_AL2_TRANS_COUNT field.
	DMA_CH14_AL2_TRANS_COUNT_CH14_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH14_AL2_READ_ADDR: Alias for channel 14 READ_ADDR register
	// Position of CH14_AL2_READ_ADDR field.
	DMA_CH14_AL2_READ_ADDR_CH14_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH14_AL2_READ_ADDR field.
	DMA_CH14_AL2_READ_ADDR_CH14_AL2_READ_ADDR_Msk = 0xffffffff

	// CH14_AL2_WRITE_ADDR_TRIG: Alias for channel 14 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH14_AL2_WRITE_ADDR_TRIG field.
	DMA_CH14_AL2_WRITE_ADDR_TRIG_CH14_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH14_AL2_WRITE_ADDR_TRIG field.
	DMA_CH14_AL2_WRITE_ADDR_TRIG_CH14_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH14_AL3_CTRL: Alias for channel 14 CTRL register
	// Position of CH14_AL3_CTRL field.
	DMA_CH14_AL3_CTRL_CH14_AL3_CTRL_Pos = 0x0
	// Bit mask of CH14_AL3_CTRL field.
	DMA_CH14_AL3_CTRL_CH14_AL3_CTRL_Msk = 0xffffffff

	// CH14_AL3_WRITE_ADDR: Alias for channel 14 WRITE_ADDR register
	// Position of CH14_AL3_WRITE_ADDR field.
	DMA_CH14_AL3_WRITE_ADDR_CH14_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH14_AL3_WRITE_ADDR field.
	DMA_CH14_AL3_WRITE_ADDR_CH14_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH14_AL3_TRANS_COUNT: Alias for channel 14 TRANS_COUNT register
	// Position of CH14_AL3_TRANS_COUNT field.
	DMA_CH14_AL3_TRANS_COUNT_CH14_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH14_AL3_TRANS_COUNT field.
	DMA_CH14_AL3_TRANS_COUNT_CH14_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH14_AL3_READ_ADDR_TRIG: Alias for channel 14 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH14_AL3_READ_ADDR_TRIG field.
	DMA_CH14_AL3_READ_ADDR_TRIG_CH14_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH14_AL3_READ_ADDR_TRIG field.
	DMA_CH14_AL3_READ_ADDR_TRIG_CH14_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// CH15_READ_ADDR: DMA Channel 15 Read Address pointer
	// Position of CH15_READ_ADDR field.
	DMA_CH15_READ_ADDR_CH15_READ_ADDR_Pos = 0x0
	// Bit mask of CH15_READ_ADDR field.
	DMA_CH15_READ_ADDR_CH15_READ_ADDR_Msk = 0xffffffff

	// CH15_WRITE_ADDR: DMA Channel 15 Write Address pointer
	// Position of CH15_WRITE_ADDR field.
	DMA_CH15_WRITE_ADDR_CH15_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH15_WRITE_ADDR field.
	DMA_CH15_WRITE_ADDR_CH15_WRITE_ADDR_Msk = 0xffffffff

	// CH15_TRANS_COUNT: DMA Channel 15 Transfer Count
	// Position of MODE field.
	DMA_CH15_TRANS_COUNT_MODE_Pos = 0x1c
	// Bit mask of MODE field.
	DMA_CH15_TRANS_COUNT_MODE_Msk          = 0xf0000000
	DMA_CH15_TRANS_COUNT_MODE_NORMAL       = 0x0
	DMA_CH15_TRANS_COUNT_MODE_TRIGGER_SELF = 0x1
	DMA_CH15_TRANS_COUNT_MODE_ENDLESS      = 0xf
	// Position of COUNT field.
	DMA_CH15_TRANS_COUNT_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	DMA_CH15_TRANS_COUNT_COUNT_Msk = 0xfffffff

	// CH15_CTRL_TRIG: DMA Channel 15 Control and Status
	// Position of AHB_ERROR field.
	DMA_CH15_CTRL_TRIG_AHB_ERROR_Pos = 0x1f
	// Bit mask of AHB_ERROR field.
	DMA_CH15_CTRL_TRIG_AHB_ERROR_Msk = 0x80000000
	// Bit AHB_ERROR.
	DMA_CH15_CTRL_TRIG_AHB_ERROR = 0x80000000
	// Position of READ_ERROR field.
	DMA_CH15_CTRL_TRIG_READ_ERROR_Pos = 0x1e
	// Bit mask of READ_ERROR field.
	DMA_CH15_CTRL_TRIG_READ_ERROR_Msk = 0x40000000
	// Bit READ_ERROR.
	DMA_CH15_CTRL_TRIG_READ_ERROR = 0x40000000
	// Position of WRITE_ERROR field.
	DMA_CH15_CTRL_TRIG_WRITE_ERROR_Pos = 0x1d
	// Bit mask of WRITE_ERROR field.
	DMA_CH15_CTRL_TRIG_WRITE_ERROR_Msk = 0x20000000
	// Bit WRITE_ERROR.
	DMA_CH15_CTRL_TRIG_WRITE_ERROR = 0x20000000
	// Position of BUSY field.
	DMA_CH15_CTRL_TRIG_BUSY_Pos = 0x1a
	// Bit mask of BUSY field.
	DMA_CH15_CTRL_TRIG_BUSY_Msk = 0x4000000
	// Bit BUSY.
	DMA_CH15_CTRL_TRIG_BUSY = 0x4000000
	// Position of SNIFF_EN field.
	DMA_CH15_CTRL_TRIG_SNIFF_EN_Pos = 0x19
	// Bit mask of SNIFF_EN field.
	DMA_CH15_CTRL_TRIG_SNIFF_EN_Msk = 0x2000000
	// Bit SNIFF_EN.
	DMA_CH15_CTRL_TRIG_SNIFF_EN = 0x2000000
	// Position of BSWAP field.
	DMA_CH15_CTRL_TRIG_BSWAP_Pos = 0x18
	// Bit mask of BSWAP field.
	DMA_CH15_CTRL_TRIG_BSWAP_Msk = 0x1000000
	// Bit BSWAP.
	DMA_CH15_CTRL_TRIG_BSWAP = 0x1000000
	// Position of IRQ_QUIET field.
	DMA_CH15_CTRL_TRIG_IRQ_QUIET_Pos = 0x17
	// Bit mask of IRQ_QUIET field.
	DMA_CH15_CTRL_TRIG_IRQ_QUIET_Msk = 0x800000
	// Bit IRQ_QUIET.
	DMA_CH15_CTRL_TRIG_IRQ_QUIET = 0x800000
	// Position of TREQ_SEL field.
	DMA_CH15_CTRL_TRIG_TREQ_SEL_Pos = 0x11
	// Bit mask of TREQ_SEL field.
	DMA_CH15_CTRL_TRIG_TREQ_SEL_Msk = 0x7e0000
	// Select PIO0's TX FIFO 0 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO0_TX0 = 0x0
	// Select PIO0's TX FIFO 1 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO0_TX1 = 0x1
	// Select PIO0's TX FIFO 2 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO0_TX2 = 0x2
	// Select PIO0's TX FIFO 3 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO0_TX3 = 0x3
	// Select PIO0's RX FIFO 0 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO0_RX0 = 0x4
	// Select PIO0's RX FIFO 1 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO0_RX1 = 0x5
	// Select PIO0's RX FIFO 2 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO0_RX2 = 0x6
	// Select PIO0's RX FIFO 3 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO0_RX3 = 0x7
	// Select PIO1's TX FIFO 0 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO1_TX0 = 0x8
	// Select PIO1's TX FIFO 1 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO1_TX1 = 0x9
	// Select PIO1's TX FIFO 2 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO1_TX2 = 0xa
	// Select PIO1's TX FIFO 3 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO1_TX3 = 0xb
	// Select PIO1's RX FIFO 0 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO1_RX0 = 0xc
	// Select PIO1's RX FIFO 1 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO1_RX1 = 0xd
	// Select PIO1's RX FIFO 2 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO1_RX2 = 0xe
	// Select PIO1's RX FIFO 3 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO1_RX3 = 0xf
	// Select PIO2's TX FIFO 0 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO2_TX0 = 0x10
	// Select PIO2's TX FIFO 1 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO2_TX1 = 0x11
	// Select PIO2's TX FIFO 2 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO2_TX2 = 0x12
	// Select PIO2's TX FIFO 3 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO2_TX3 = 0x13
	// Select PIO2's RX FIFO 0 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO2_RX0 = 0x14
	// Select PIO2's RX FIFO 1 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO2_RX1 = 0x15
	// Select PIO2's RX FIFO 2 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO2_RX2 = 0x16
	// Select PIO2's RX FIFO 3 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PIO2_RX3 = 0x17
	// Select SPI0's TX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_SPI0_TX = 0x18
	// Select SPI0's RX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_SPI0_RX = 0x19
	// Select SPI1's TX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_SPI1_TX = 0x1a
	// Select SPI1's RX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_SPI1_RX = 0x1b
	// Select UART0's TX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_UART0_TX = 0x1c
	// Select UART0's RX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_UART0_RX = 0x1d
	// Select UART1's TX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_UART1_TX = 0x1e
	// Select UART1's RX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_UART1_RX = 0x1f
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP0 = 0x20
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP1 = 0x21
	// Select PWM Counter 2's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP2 = 0x22
	// Select PWM Counter 3's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP3 = 0x23
	// Select PWM Counter 4's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP4 = 0x24
	// Select PWM Counter 5's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP5 = 0x25
	// Select PWM Counter 6's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP6 = 0x26
	// Select PWM Counter 7's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP7 = 0x27
	// Select PWM Counter 8's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP8 = 0x28
	// Select PWM Counter 9's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP9 = 0x29
	// Select PWM Counter 0's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP10 = 0x2a
	// Select PWM Counter 1's Wrap Value as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PWM_WRAP11 = 0x2b
	// Select I2C0's TX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_I2C0_TX = 0x2c
	// Select I2C0's RX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_I2C0_RX = 0x2d
	// Select I2C1's TX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_I2C1_TX = 0x2e
	// Select I2C1's RX FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_I2C1_RX = 0x2f
	// Select the ADC as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_ADC = 0x30
	// Select the XIP Streaming FIFO as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_XIP_STREAM = 0x31
	// Select XIP_QMITX as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_XIP_QMITX = 0x32
	// Select XIP_QMIRX as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_XIP_QMIRX = 0x33
	// Select HSTX as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_HSTX = 0x34
	// Select CORESIGHT as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_CORESIGHT = 0x35
	// Select SHA256 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_SHA256 = 0x36
	// Select Timer 0 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_TIMER0 = 0x3b
	// Select Timer 1 as TREQ
	DMA_CH15_CTRL_TRIG_TREQ_SEL_TIMER1 = 0x3c
	// Select Timer 2 as TREQ (Optional)
	DMA_CH15_CTRL_TRIG_TREQ_SEL_TIMER2 = 0x3d
	// Select Timer 3 as TREQ (Optional)
	DMA_CH15_CTRL_TRIG_TREQ_SEL_TIMER3 = 0x3e
	// Permanent request, for unpaced transfers.
	DMA_CH15_CTRL_TRIG_TREQ_SEL_PERMANENT = 0x3f
	// Position of CHAIN_TO field.
	DMA_CH15_CTRL_TRIG_CHAIN_TO_Pos = 0xd
	// Bit mask of CHAIN_TO field.
	DMA_CH15_CTRL_TRIG_CHAIN_TO_Msk = 0x1e000
	// Position of RING_SEL field.
	DMA_CH15_CTRL_TRIG_RING_SEL_Pos = 0xc
	// Bit mask of RING_SEL field.
	DMA_CH15_CTRL_TRIG_RING_SEL_Msk = 0x1000
	// Bit RING_SEL.
	DMA_CH15_CTRL_TRIG_RING_SEL = 0x1000
	// Position of RING_SIZE field.
	DMA_CH15_CTRL_TRIG_RING_SIZE_Pos = 0x8
	// Bit mask of RING_SIZE field.
	DMA_CH15_CTRL_TRIG_RING_SIZE_Msk       = 0xf00
	DMA_CH15_CTRL_TRIG_RING_SIZE_RING_NONE = 0x0
	// Position of INCR_WRITE_REV field.
	DMA_CH15_CTRL_TRIG_INCR_WRITE_REV_Pos = 0x7
	// Bit mask of INCR_WRITE_REV field.
	DMA_CH15_CTRL_TRIG_INCR_WRITE_REV_Msk = 0x80
	// Bit INCR_WRITE_REV.
	DMA_CH15_CTRL_TRIG_INCR_WRITE_REV = 0x80
	// Position of INCR_WRITE field.
	DMA_CH15_CTRL_TRIG_INCR_WRITE_Pos = 0x6
	// Bit mask of INCR_WRITE field.
	DMA_CH15_CTRL_TRIG_INCR_WRITE_Msk = 0x40
	// Bit INCR_WRITE.
	DMA_CH15_CTRL_TRIG_INCR_WRITE = 0x40
	// Position of INCR_READ_REV field.
	DMA_CH15_CTRL_TRIG_INCR_READ_REV_Pos = 0x5
	// Bit mask of INCR_READ_REV field.
	DMA_CH15_CTRL_TRIG_INCR_READ_REV_Msk = 0x20
	// Bit INCR_READ_REV.
	DMA_CH15_CTRL_TRIG_INCR_READ_REV = 0x20
	// Position of INCR_READ field.
	DMA_CH15_CTRL_TRIG_INCR_READ_Pos = 0x4
	// Bit mask of INCR_READ field.
	DMA_CH15_CTRL_TRIG_INCR_READ_Msk = 0x10
	// Bit INCR_READ.
	DMA_CH15_CTRL_TRIG_INCR_READ = 0x10
	// Position of DATA_SIZE field.
	DMA_CH15_CTRL_TRIG_DATA_SIZE_Pos = 0x2
	// Bit mask of DATA_SIZE field.
	DMA_CH15_CTRL_TRIG_DATA_SIZE_Msk           = 0xc
	DMA_CH15_CTRL_TRIG_DATA_SIZE_SIZE_BYTE     = 0x0
	DMA_CH15_CTRL_TRIG_DATA_SIZE_SIZE_HALFWORD = 0x1
	DMA_CH15_CTRL_TRIG_DATA_SIZE_SIZE_WORD     = 0x2
	// Position of HIGH_PRIORITY field.
	DMA_CH15_CTRL_TRIG_HIGH_PRIORITY_Pos = 0x1
	// Bit mask of HIGH_PRIORITY field.
	DMA_CH15_CTRL_TRIG_HIGH_PRIORITY_Msk = 0x2
	// Bit HIGH_PRIORITY.
	DMA_CH15_CTRL_TRIG_HIGH_PRIORITY = 0x2
	// Position of EN field.
	DMA_CH15_CTRL_TRIG_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_CH15_CTRL_TRIG_EN_Msk = 0x1
	// Bit EN.
	DMA_CH15_CTRL_TRIG_EN = 0x1

	// CH15_AL1_CTRL: Alias for channel 15 CTRL register
	// Position of CH15_AL1_CTRL field.
	DMA_CH15_AL1_CTRL_CH15_AL1_CTRL_Pos = 0x0
	// Bit mask of CH15_AL1_CTRL field.
	DMA_CH15_AL1_CTRL_CH15_AL1_CTRL_Msk = 0xffffffff

	// CH15_AL1_READ_ADDR: Alias for channel 15 READ_ADDR register
	// Position of CH15_AL1_READ_ADDR field.
	DMA_CH15_AL1_READ_ADDR_CH15_AL1_READ_ADDR_Pos = 0x0
	// Bit mask of CH15_AL1_READ_ADDR field.
	DMA_CH15_AL1_READ_ADDR_CH15_AL1_READ_ADDR_Msk = 0xffffffff

	// CH15_AL1_WRITE_ADDR: Alias for channel 15 WRITE_ADDR register
	// Position of CH15_AL1_WRITE_ADDR field.
	DMA_CH15_AL1_WRITE_ADDR_CH15_AL1_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH15_AL1_WRITE_ADDR field.
	DMA_CH15_AL1_WRITE_ADDR_CH15_AL1_WRITE_ADDR_Msk = 0xffffffff

	// CH15_AL1_TRANS_COUNT_TRIG: Alias for channel 15 TRANS_COUNT register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH15_AL1_TRANS_COUNT_TRIG field.
	DMA_CH15_AL1_TRANS_COUNT_TRIG_CH15_AL1_TRANS_COUNT_TRIG_Pos = 0x0
	// Bit mask of CH15_AL1_TRANS_COUNT_TRIG field.
	DMA_CH15_AL1_TRANS_COUNT_TRIG_CH15_AL1_TRANS_COUNT_TRIG_Msk = 0xffffffff

	// CH15_AL2_CTRL: Alias for channel 15 CTRL register
	// Position of CH15_AL2_CTRL field.
	DMA_CH15_AL2_CTRL_CH15_AL2_CTRL_Pos = 0x0
	// Bit mask of CH15_AL2_CTRL field.
	DMA_CH15_AL2_CTRL_CH15_AL2_CTRL_Msk = 0xffffffff

	// CH15_AL2_TRANS_COUNT: Alias for channel 15 TRANS_COUNT register
	// Position of CH15_AL2_TRANS_COUNT field.
	DMA_CH15_AL2_TRANS_COUNT_CH15_AL2_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH15_AL2_TRANS_COUNT field.
	DMA_CH15_AL2_TRANS_COUNT_CH15_AL2_TRANS_COUNT_Msk = 0xffffffff

	// CH15_AL2_READ_ADDR: Alias for channel 15 READ_ADDR register
	// Position of CH15_AL2_READ_ADDR field.
	DMA_CH15_AL2_READ_ADDR_CH15_AL2_READ_ADDR_Pos = 0x0
	// Bit mask of CH15_AL2_READ_ADDR field.
	DMA_CH15_AL2_READ_ADDR_CH15_AL2_READ_ADDR_Msk = 0xffffffff

	// CH15_AL2_WRITE_ADDR_TRIG: Alias for channel 15 WRITE_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH15_AL2_WRITE_ADDR_TRIG field.
	DMA_CH15_AL2_WRITE_ADDR_TRIG_CH15_AL2_WRITE_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH15_AL2_WRITE_ADDR_TRIG field.
	DMA_CH15_AL2_WRITE_ADDR_TRIG_CH15_AL2_WRITE_ADDR_TRIG_Msk = 0xffffffff

	// CH15_AL3_CTRL: Alias for channel 15 CTRL register
	// Position of CH15_AL3_CTRL field.
	DMA_CH15_AL3_CTRL_CH15_AL3_CTRL_Pos = 0x0
	// Bit mask of CH15_AL3_CTRL field.
	DMA_CH15_AL3_CTRL_CH15_AL3_CTRL_Msk = 0xffffffff

	// CH15_AL3_WRITE_ADDR: Alias for channel 15 WRITE_ADDR register
	// Position of CH15_AL3_WRITE_ADDR field.
	DMA_CH15_AL3_WRITE_ADDR_CH15_AL3_WRITE_ADDR_Pos = 0x0
	// Bit mask of CH15_AL3_WRITE_ADDR field.
	DMA_CH15_AL3_WRITE_ADDR_CH15_AL3_WRITE_ADDR_Msk = 0xffffffff

	// CH15_AL3_TRANS_COUNT: Alias for channel 15 TRANS_COUNT register
	// Position of CH15_AL3_TRANS_COUNT field.
	DMA_CH15_AL3_TRANS_COUNT_CH15_AL3_TRANS_COUNT_Pos = 0x0
	// Bit mask of CH15_AL3_TRANS_COUNT field.
	DMA_CH15_AL3_TRANS_COUNT_CH15_AL3_TRANS_COUNT_Msk = 0xffffffff

	// CH15_AL3_READ_ADDR_TRIG: Alias for channel 15 READ_ADDR register This is a trigger register (0xc). Writing a nonzero value will reload the channel counter and start the channel.
	// Position of CH15_AL3_READ_ADDR_TRIG field.
	DMA_CH15_AL3_READ_ADDR_TRIG_CH15_AL3_READ_ADDR_TRIG_Pos = 0x0
	// Bit mask of CH15_AL3_READ_ADDR_TRIG field.
	DMA_CH15_AL3_READ_ADDR_TRIG_CH15_AL3_READ_ADDR_TRIG_Msk = 0xffffffff

	// INTR: Interrupt Status (raw)
	// Position of INTR field.
	DMA_INTR_INTR_Pos = 0x0
	// Bit mask of INTR field.
	DMA_INTR_INTR_Msk = 0xffff

	// INTE0: Interrupt Enables for IRQ 0
	// Position of INTE0 field.
	DMA_INTE0_INTE0_Pos = 0x0
	// Bit mask of INTE0 field.
	DMA_INTE0_INTE0_Msk = 0xffff

	// INTF0: Force Interrupts
	// Position of INTF0 field.
	DMA_INTF0_INTF0_Pos = 0x0
	// Bit mask of INTF0 field.
	DMA_INTF0_INTF0_Msk = 0xffff

	// INTS0: Interrupt Status for IRQ 0
	// Position of INTS0 field.
	DMA_INTS0_INTS0_Pos = 0x0
	// Bit mask of INTS0 field.
	DMA_INTS0_INTS0_Msk = 0xffff

	// INTR1: Interrupt Status (raw)
	// Position of INTR1 field.
	DMA_INTR1_INTR1_Pos = 0x0
	// Bit mask of INTR1 field.
	DMA_INTR1_INTR1_Msk = 0xffff

	// INTE1: Interrupt Enables for IRQ 1
	// Position of INTE1 field.
	DMA_INTE1_INTE1_Pos = 0x0
	// Bit mask of INTE1 field.
	DMA_INTE1_INTE1_Msk = 0xffff

	// INTF1: Force Interrupts
	// Position of INTF1 field.
	DMA_INTF1_INTF1_Pos = 0x0
	// Bit mask of INTF1 field.
	DMA_INTF1_INTF1_Msk = 0xffff

	// INTS1: Interrupt Status for IRQ 1
	// Position of INTS1 field.
	DMA_INTS1_INTS1_Pos = 0x0
	// Bit mask of INTS1 field.
	DMA_INTS1_INTS1_Msk = 0xffff

	// INTR2: Interrupt Status (raw)
	// Position of INTR2 field.
	DMA_INTR2_INTR2_Pos = 0x0
	// Bit mask of INTR2 field.
	DMA_INTR2_INTR2_Msk = 0xffff

	// INTE2: Interrupt Enables for IRQ 2
	// Position of INTE2 field.
	DMA_INTE2_INTE2_Pos = 0x0
	// Bit mask of INTE2 field.
	DMA_INTE2_INTE2_Msk = 0xffff

	// INTF2: Force Interrupts
	// Position of INTF2 field.
	DMA_INTF2_INTF2_Pos = 0x0
	// Bit mask of INTF2 field.
	DMA_INTF2_INTF2_Msk = 0xffff

	// INTS2: Interrupt Status for IRQ 2
	// Position of INTS2 field.
	DMA_INTS2_INTS2_Pos = 0x0
	// Bit mask of INTS2 field.
	DMA_INTS2_INTS2_Msk = 0xffff

	// INTR3: Interrupt Status (raw)
	// Position of INTR3 field.
	DMA_INTR3_INTR3_Pos = 0x0
	// Bit mask of INTR3 field.
	DMA_INTR3_INTR3_Msk = 0xffff

	// INTE3: Interrupt Enables for IRQ 3
	// Position of INTE3 field.
	DMA_INTE3_INTE3_Pos = 0x0
	// Bit mask of INTE3 field.
	DMA_INTE3_INTE3_Msk = 0xffff

	// INTF3: Force Interrupts
	// Position of INTF3 field.
	DMA_INTF3_INTF3_Pos = 0x0
	// Bit mask of INTF3 field.
	DMA_INTF3_INTF3_Msk = 0xffff

	// INTS3: Interrupt Status for IRQ 3
	// Position of INTS3 field.
	DMA_INTS3_INTS3_Pos = 0x0
	// Bit mask of INTS3 field.
	DMA_INTS3_INTS3_Msk = 0xffff

	// TIMER0: Pacing (X/Y) fractional timer The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
	// Position of X field.
	DMA_TIMER0_X_Pos = 0x10
	// Bit mask of X field.
	DMA_TIMER0_X_Msk = 0xffff0000
	// Position of Y field.
	DMA_TIMER0_Y_Pos = 0x0
	// Bit mask of Y field.
	DMA_TIMER0_Y_Msk = 0xffff

	// TIMER1: Pacing (X/Y) fractional timer The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
	// Position of X field.
	DMA_TIMER1_X_Pos = 0x10
	// Bit mask of X field.
	DMA_TIMER1_X_Msk = 0xffff0000
	// Position of Y field.
	DMA_TIMER1_Y_Pos = 0x0
	// Bit mask of Y field.
	DMA_TIMER1_Y_Msk = 0xffff

	// TIMER2: Pacing (X/Y) fractional timer The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
	// Position of X field.
	DMA_TIMER2_X_Pos = 0x10
	// Bit mask of X field.
	DMA_TIMER2_X_Msk = 0xffff0000
	// Position of Y field.
	DMA_TIMER2_Y_Pos = 0x0
	// Bit mask of Y field.
	DMA_TIMER2_Y_Msk = 0xffff

	// TIMER3: Pacing (X/Y) fractional timer The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
	// Position of X field.
	DMA_TIMER3_X_Pos = 0x10
	// Bit mask of X field.
	DMA_TIMER3_X_Msk = 0xffff0000
	// Position of Y field.
	DMA_TIMER3_Y_Pos = 0x0
	// Bit mask of Y field.
	DMA_TIMER3_Y_Msk = 0xffff

	// MULTI_CHAN_TRIGGER: Trigger one or more channels simultaneously
	// Position of MULTI_CHAN_TRIGGER field.
	DMA_MULTI_CHAN_TRIGGER_MULTI_CHAN_TRIGGER_Pos = 0x0
	// Bit mask of MULTI_CHAN_TRIGGER field.
	DMA_MULTI_CHAN_TRIGGER_MULTI_CHAN_TRIGGER_Msk = 0xffff

	// SNIFF_CTRL: Sniffer Control
	// Position of OUT_INV field.
	DMA_SNIFF_CTRL_OUT_INV_Pos = 0xb
	// Bit mask of OUT_INV field.
	DMA_SNIFF_CTRL_OUT_INV_Msk = 0x800
	// Bit OUT_INV.
	DMA_SNIFF_CTRL_OUT_INV = 0x800
	// Position of OUT_REV field.
	DMA_SNIFF_CTRL_OUT_REV_Pos = 0xa
	// Bit mask of OUT_REV field.
	DMA_SNIFF_CTRL_OUT_REV_Msk = 0x400
	// Bit OUT_REV.
	DMA_SNIFF_CTRL_OUT_REV = 0x400
	// Position of BSWAP field.
	DMA_SNIFF_CTRL_BSWAP_Pos = 0x9
	// Bit mask of BSWAP field.
	DMA_SNIFF_CTRL_BSWAP_Msk = 0x200
	// Bit BSWAP.
	DMA_SNIFF_CTRL_BSWAP = 0x200
	// Position of CALC field.
	DMA_SNIFF_CTRL_CALC_Pos = 0x5
	// Bit mask of CALC field.
	DMA_SNIFF_CTRL_CALC_Msk = 0x1e0
	// Calculate a CRC-32 (IEEE802.3 polynomial)
	DMA_SNIFF_CTRL_CALC_CRC32 = 0x0
	// Calculate a CRC-32 (IEEE802.3 polynomial) with bit reversed data
	DMA_SNIFF_CTRL_CALC_CRC32R = 0x1
	// Calculate a CRC-16-CCITT
	DMA_SNIFF_CTRL_CALC_CRC16 = 0x2
	// Calculate a CRC-16-CCITT with bit reversed data
	DMA_SNIFF_CTRL_CALC_CRC16R = 0x3
	// XOR reduction over all data. == 1 if the total 1 population count is odd.
	DMA_SNIFF_CTRL_CALC_EVEN = 0xe
	// Calculate a simple 32-bit checksum (addition with a 32 bit accumulator)
	DMA_SNIFF_CTRL_CALC_SUM = 0xf
	// Position of DMACH field.
	DMA_SNIFF_CTRL_DMACH_Pos = 0x1
	// Bit mask of DMACH field.
	DMA_SNIFF_CTRL_DMACH_Msk = 0x1e
	// Position of EN field.
	DMA_SNIFF_CTRL_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_SNIFF_CTRL_EN_Msk = 0x1
	// Bit EN.
	DMA_SNIFF_CTRL_EN = 0x1

	// SNIFF_DATA: Data accumulator for sniff hardware
	// Position of SNIFF_DATA field.
	DMA_SNIFF_DATA_SNIFF_DATA_Pos = 0x0
	// Bit mask of SNIFF_DATA field.
	DMA_SNIFF_DATA_SNIFF_DATA_Msk = 0xffffffff

	// FIFO_LEVELS: Debug RAF, WAF, TDF levels
	// Position of RAF_LVL field.
	DMA_FIFO_LEVELS_RAF_LVL_Pos = 0x10
	// Bit mask of RAF_LVL field.
	DMA_FIFO_LEVELS_RAF_LVL_Msk = 0xff0000
	// Position of WAF_LVL field.
	DMA_FIFO_LEVELS_WAF_LVL_Pos = 0x8
	// Bit mask of WAF_LVL field.
	DMA_FIFO_LEVELS_WAF_LVL_Msk = 0xff00
	// Position of TDF_LVL field.
	DMA_FIFO_LEVELS_TDF_LVL_Pos = 0x0
	// Bit mask of TDF_LVL field.
	DMA_FIFO_LEVELS_TDF_LVL_Msk = 0xff

	// CHAN_ABORT: Abort an in-progress transfer sequence on one or more channels
	// Position of CHAN_ABORT field.
	DMA_CHAN_ABORT_CHAN_ABORT_Pos = 0x0
	// Bit mask of CHAN_ABORT field.
	DMA_CHAN_ABORT_CHAN_ABORT_Msk = 0xffff

	// N_CHANNELS: The number of channels this DMA instance is equipped with. This DMA supports up to 16 hardware channels, but can be configured with as few as one, to minimise silicon area.
	// Position of N_CHANNELS field.
	DMA_N_CHANNELS_N_CHANNELS_Pos = 0x0
	// Bit mask of N_CHANNELS field.
	DMA_N_CHANNELS_N_CHANNELS_Msk = 0x1f

	// SECCFG_CH0: Security configuration for channel 0. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH0_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH0_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH0_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH0_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH0_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH0_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH0_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH0_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH0_P = 0x1

	// SECCFG_CH1: Security configuration for channel 1. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH1_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH1_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH1_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH1_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH1_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH1_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH1_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH1_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH1_P = 0x1

	// SECCFG_CH2: Security configuration for channel 2. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH2_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH2_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH2_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH2_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH2_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH2_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH2_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH2_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH2_P = 0x1

	// SECCFG_CH3: Security configuration for channel 3. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH3_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH3_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH3_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH3_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH3_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH3_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH3_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH3_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH3_P = 0x1

	// SECCFG_CH4: Security configuration for channel 4. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH4_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH4_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH4_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH4_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH4_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH4_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH4_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH4_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH4_P = 0x1

	// SECCFG_CH5: Security configuration for channel 5. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH5_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH5_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH5_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH5_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH5_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH5_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH5_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH5_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH5_P = 0x1

	// SECCFG_CH6: Security configuration for channel 6. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH6_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH6_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH6_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH6_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH6_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH6_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH6_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH6_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH6_P = 0x1

	// SECCFG_CH7: Security configuration for channel 7. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH7_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH7_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH7_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH7_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH7_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH7_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH7_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH7_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH7_P = 0x1

	// SECCFG_CH8: Security configuration for channel 8. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH8_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH8_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH8_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH8_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH8_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH8_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH8_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH8_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH8_P = 0x1

	// SECCFG_CH9: Security configuration for channel 9. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH9_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH9_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH9_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH9_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH9_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH9_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH9_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH9_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH9_P = 0x1

	// SECCFG_CH10: Security configuration for channel 10. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH10_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH10_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH10_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH10_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH10_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH10_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH10_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH10_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH10_P = 0x1

	// SECCFG_CH11: Security configuration for channel 11. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH11_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH11_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH11_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH11_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH11_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH11_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH11_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH11_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH11_P = 0x1

	// SECCFG_CH12: Security configuration for channel 12. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH12_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH12_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH12_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH12_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH12_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH12_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH12_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH12_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH12_P = 0x1

	// SECCFG_CH13: Security configuration for channel 13. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH13_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH13_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH13_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH13_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH13_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH13_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH13_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH13_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH13_P = 0x1

	// SECCFG_CH14: Security configuration for channel 14. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH14_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH14_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH14_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH14_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH14_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH14_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH14_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH14_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH14_P = 0x1

	// SECCFG_CH15: Security configuration for channel 15. Control whether this channel performs Secure/Non-secure and Privileged/Unprivileged bus accesses. If this channel generates bus accesses of some security level, an access of at least that level (in the order S+P > S+U > NS+P > NS+U) is required to program, trigger, abort, check the status of, interrupt on or acknowledge the interrupt of this channel. This register automatically locks down (becomes read-only) once software starts to configure the channel. This register is world-readable, but is writable only from a Secure, Privileged context.
	// Position of LOCK field.
	DMA_SECCFG_CH15_LOCK_Pos = 0x2
	// Bit mask of LOCK field.
	DMA_SECCFG_CH15_LOCK_Msk = 0x4
	// Bit LOCK.
	DMA_SECCFG_CH15_LOCK = 0x4
	// Position of S field.
	DMA_SECCFG_CH15_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_CH15_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_CH15_S = 0x2
	// Position of P field.
	DMA_SECCFG_CH15_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_CH15_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_CH15_P = 0x1

	// SECCFG_IRQ0: Security configuration for IRQ 0. Control whether the IRQ permits configuration by Non-secure/Unprivileged contexts, and whether it can observe Secure/Privileged channel interrupt flags.
	// Position of S field.
	DMA_SECCFG_IRQ0_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_IRQ0_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_IRQ0_S = 0x2
	// Position of P field.
	DMA_SECCFG_IRQ0_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_IRQ0_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_IRQ0_P = 0x1

	// SECCFG_IRQ1: Security configuration for IRQ 1. Control whether the IRQ permits configuration by Non-secure/Unprivileged contexts, and whether it can observe Secure/Privileged channel interrupt flags.
	// Position of S field.
	DMA_SECCFG_IRQ1_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_IRQ1_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_IRQ1_S = 0x2
	// Position of P field.
	DMA_SECCFG_IRQ1_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_IRQ1_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_IRQ1_P = 0x1

	// SECCFG_IRQ2: Security configuration for IRQ 2. Control whether the IRQ permits configuration by Non-secure/Unprivileged contexts, and whether it can observe Secure/Privileged channel interrupt flags.
	// Position of S field.
	DMA_SECCFG_IRQ2_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_IRQ2_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_IRQ2_S = 0x2
	// Position of P field.
	DMA_SECCFG_IRQ2_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_IRQ2_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_IRQ2_P = 0x1

	// SECCFG_IRQ3: Security configuration for IRQ 3. Control whether the IRQ permits configuration by Non-secure/Unprivileged contexts, and whether it can observe Secure/Privileged channel interrupt flags.
	// Position of S field.
	DMA_SECCFG_IRQ3_S_Pos = 0x1
	// Bit mask of S field.
	DMA_SECCFG_IRQ3_S_Msk = 0x2
	// Bit S.
	DMA_SECCFG_IRQ3_S = 0x2
	// Position of P field.
	DMA_SECCFG_IRQ3_P_Pos = 0x0
	// Bit mask of P field.
	DMA_SECCFG_IRQ3_P_Msk = 0x1
	// Bit P.
	DMA_SECCFG_IRQ3_P = 0x1

	// SECCFG_MISC: Miscellaneous security configuration
	// Position of TIMER3_S field.
	DMA_SECCFG_MISC_TIMER3_S_Pos = 0x9
	// Bit mask of TIMER3_S field.
	DMA_SECCFG_MISC_TIMER3_S_Msk = 0x200
	// Bit TIMER3_S.
	DMA_SECCFG_MISC_TIMER3_S = 0x200
	// Position of TIMER3_P field.
	DMA_SECCFG_MISC_TIMER3_P_Pos = 0x8
	// Bit mask of TIMER3_P field.
	DMA_SECCFG_MISC_TIMER3_P_Msk = 0x100
	// Bit TIMER3_P.
	DMA_SECCFG_MISC_TIMER3_P = 0x100
	// Position of TIMER2_S field.
	DMA_SECCFG_MISC_TIMER2_S_Pos = 0x7
	// Bit mask of TIMER2_S field.
	DMA_SECCFG_MISC_TIMER2_S_Msk = 0x80
	// Bit TIMER2_S.
	DMA_SECCFG_MISC_TIMER2_S = 0x80
	// Position of TIMER2_P field.
	DMA_SECCFG_MISC_TIMER2_P_Pos = 0x6
	// Bit mask of TIMER2_P field.
	DMA_SECCFG_MISC_TIMER2_P_Msk = 0x40
	// Bit TIMER2_P.
	DMA_SECCFG_MISC_TIMER2_P = 0x40
	// Position of TIMER1_S field.
	DMA_SECCFG_MISC_TIMER1_S_Pos = 0x5
	// Bit mask of TIMER1_S field.
	DMA_SECCFG_MISC_TIMER1_S_Msk = 0x20
	// Bit TIMER1_S.
	DMA_SECCFG_MISC_TIMER1_S = 0x20
	// Position of TIMER1_P field.
	DMA_SECCFG_MISC_TIMER1_P_Pos = 0x4
	// Bit mask of TIMER1_P field.
	DMA_SECCFG_MISC_TIMER1_P_Msk = 0x10
	// Bit TIMER1_P.
	DMA_SECCFG_MISC_TIMER1_P = 0x10
	// Position of TIMER0_S field.
	DMA_SECCFG_MISC_TIMER0_S_Pos = 0x3
	// Bit mask of TIMER0_S field.
	DMA_SECCFG_MISC_TIMER0_S_Msk = 0x8
	// Bit TIMER0_S.
	DMA_SECCFG_MISC_TIMER0_S = 0x8
	// Position of TIMER0_P field.
	DMA_SECCFG_MISC_TIMER0_P_Pos = 0x2
	// Bit mask of TIMER0_P field.
	DMA_SECCFG_MISC_TIMER0_P_Msk = 0x4
	// Bit TIMER0_P.
	DMA_SECCFG_MISC_TIMER0_P = 0x4
	// Position of SNIFF_S field.
	DMA_SECCFG_MISC_SNIFF_S_Pos = 0x1
	// Bit mask of SNIFF_S field.
	DMA_SECCFG_MISC_SNIFF_S_Msk = 0x2
	// Bit SNIFF_S.
	DMA_SECCFG_MISC_SNIFF_S = 0x2
	// Position of SNIFF_P field.
	DMA_SECCFG_MISC_SNIFF_P_Pos = 0x0
	// Bit mask of SNIFF_P field.
	DMA_SECCFG_MISC_SNIFF_P_Msk = 0x1
	// Bit SNIFF_P.
	DMA_SECCFG_MISC_SNIFF_P = 0x1

	// MPU_CTRL: Control register for DMA MPU. Accessible only from a Privileged context.
	// Position of NS_HIDE_ADDR field.
	DMA_MPU_CTRL_NS_HIDE_ADDR_Pos = 0x3
	// Bit mask of NS_HIDE_ADDR field.
	DMA_MPU_CTRL_NS_HIDE_ADDR_Msk = 0x8
	// Bit NS_HIDE_ADDR.
	DMA_MPU_CTRL_NS_HIDE_ADDR = 0x8
	// Position of S field.
	DMA_MPU_CTRL_S_Pos = 0x2
	// Bit mask of S field.
	DMA_MPU_CTRL_S_Msk = 0x4
	// Bit S.
	DMA_MPU_CTRL_S = 0x4
	// Position of P field.
	DMA_MPU_CTRL_P_Pos = 0x1
	// Bit mask of P field.
	DMA_MPU_CTRL_P_Msk = 0x2
	// Bit P.
	DMA_MPU_CTRL_P = 0x2

	// MPU_BAR0: Base address register for MPU region 0. Writable only from a Secure, Privileged context.
	// Position of ADDR field.
	DMA_MPU_BAR0_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_BAR0_ADDR_Msk = 0xffffffe0

	// MPU_LAR0: Limit address register for MPU region 0. Writable only from a Secure, Privileged context, with the exception of the P bit.
	// Position of ADDR field.
	DMA_MPU_LAR0_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_LAR0_ADDR_Msk = 0xffffffe0
	// Position of S field.
	DMA_MPU_LAR0_S_Pos = 0x2
	// Bit mask of S field.
	DMA_MPU_LAR0_S_Msk = 0x4
	// Bit S.
	DMA_MPU_LAR0_S = 0x4
	// Position of P field.
	DMA_MPU_LAR0_P_Pos = 0x1
	// Bit mask of P field.
	DMA_MPU_LAR0_P_Msk = 0x2
	// Bit P.
	DMA_MPU_LAR0_P = 0x2
	// Position of EN field.
	DMA_MPU_LAR0_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_MPU_LAR0_EN_Msk = 0x1
	// Bit EN.
	DMA_MPU_LAR0_EN = 0x1

	// MPU_BAR1: Base address register for MPU region 1. Writable only from a Secure, Privileged context.
	// Position of ADDR field.
	DMA_MPU_BAR1_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_BAR1_ADDR_Msk = 0xffffffe0

	// MPU_LAR1: Limit address register for MPU region 1. Writable only from a Secure, Privileged context, with the exception of the P bit.
	// Position of ADDR field.
	DMA_MPU_LAR1_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_LAR1_ADDR_Msk = 0xffffffe0
	// Position of S field.
	DMA_MPU_LAR1_S_Pos = 0x2
	// Bit mask of S field.
	DMA_MPU_LAR1_S_Msk = 0x4
	// Bit S.
	DMA_MPU_LAR1_S = 0x4
	// Position of P field.
	DMA_MPU_LAR1_P_Pos = 0x1
	// Bit mask of P field.
	DMA_MPU_LAR1_P_Msk = 0x2
	// Bit P.
	DMA_MPU_LAR1_P = 0x2
	// Position of EN field.
	DMA_MPU_LAR1_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_MPU_LAR1_EN_Msk = 0x1
	// Bit EN.
	DMA_MPU_LAR1_EN = 0x1

	// MPU_BAR2: Base address register for MPU region 2. Writable only from a Secure, Privileged context.
	// Position of ADDR field.
	DMA_MPU_BAR2_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_BAR2_ADDR_Msk = 0xffffffe0

	// MPU_LAR2: Limit address register for MPU region 2. Writable only from a Secure, Privileged context, with the exception of the P bit.
	// Position of ADDR field.
	DMA_MPU_LAR2_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_LAR2_ADDR_Msk = 0xffffffe0
	// Position of S field.
	DMA_MPU_LAR2_S_Pos = 0x2
	// Bit mask of S field.
	DMA_MPU_LAR2_S_Msk = 0x4
	// Bit S.
	DMA_MPU_LAR2_S = 0x4
	// Position of P field.
	DMA_MPU_LAR2_P_Pos = 0x1
	// Bit mask of P field.
	DMA_MPU_LAR2_P_Msk = 0x2
	// Bit P.
	DMA_MPU_LAR2_P = 0x2
	// Position of EN field.
	DMA_MPU_LAR2_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_MPU_LAR2_EN_Msk = 0x1
	// Bit EN.
	DMA_MPU_LAR2_EN = 0x1

	// MPU_BAR3: Base address register for MPU region 3. Writable only from a Secure, Privileged context.
	// Position of ADDR field.
	DMA_MPU_BAR3_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_BAR3_ADDR_Msk = 0xffffffe0

	// MPU_LAR3: Limit address register for MPU region 3. Writable only from a Secure, Privileged context, with the exception of the P bit.
	// Position of ADDR field.
	DMA_MPU_LAR3_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_LAR3_ADDR_Msk = 0xffffffe0
	// Position of S field.
	DMA_MPU_LAR3_S_Pos = 0x2
	// Bit mask of S field.
	DMA_MPU_LAR3_S_Msk = 0x4
	// Bit S.
	DMA_MPU_LAR3_S = 0x4
	// Position of P field.
	DMA_MPU_LAR3_P_Pos = 0x1
	// Bit mask of P field.
	DMA_MPU_LAR3_P_Msk = 0x2
	// Bit P.
	DMA_MPU_LAR3_P = 0x2
	// Position of EN field.
	DMA_MPU_LAR3_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_MPU_LAR3_EN_Msk = 0x1
	// Bit EN.
	DMA_MPU_LAR3_EN = 0x1

	// MPU_BAR4: Base address register for MPU region 4. Writable only from a Secure, Privileged context.
	// Position of ADDR field.
	DMA_MPU_BAR4_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_BAR4_ADDR_Msk = 0xffffffe0

	// MPU_LAR4: Limit address register for MPU region 4. Writable only from a Secure, Privileged context, with the exception of the P bit.
	// Position of ADDR field.
	DMA_MPU_LAR4_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_LAR4_ADDR_Msk = 0xffffffe0
	// Position of S field.
	DMA_MPU_LAR4_S_Pos = 0x2
	// Bit mask of S field.
	DMA_MPU_LAR4_S_Msk = 0x4
	// Bit S.
	DMA_MPU_LAR4_S = 0x4
	// Position of P field.
	DMA_MPU_LAR4_P_Pos = 0x1
	// Bit mask of P field.
	DMA_MPU_LAR4_P_Msk = 0x2
	// Bit P.
	DMA_MPU_LAR4_P = 0x2
	// Position of EN field.
	DMA_MPU_LAR4_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_MPU_LAR4_EN_Msk = 0x1
	// Bit EN.
	DMA_MPU_LAR4_EN = 0x1

	// MPU_BAR5: Base address register for MPU region 5. Writable only from a Secure, Privileged context.
	// Position of ADDR field.
	DMA_MPU_BAR5_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_BAR5_ADDR_Msk = 0xffffffe0

	// MPU_LAR5: Limit address register for MPU region 5. Writable only from a Secure, Privileged context, with the exception of the P bit.
	// Position of ADDR field.
	DMA_MPU_LAR5_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_LAR5_ADDR_Msk = 0xffffffe0
	// Position of S field.
	DMA_MPU_LAR5_S_Pos = 0x2
	// Bit mask of S field.
	DMA_MPU_LAR5_S_Msk = 0x4
	// Bit S.
	DMA_MPU_LAR5_S = 0x4
	// Position of P field.
	DMA_MPU_LAR5_P_Pos = 0x1
	// Bit mask of P field.
	DMA_MPU_LAR5_P_Msk = 0x2
	// Bit P.
	DMA_MPU_LAR5_P = 0x2
	// Position of EN field.
	DMA_MPU_LAR5_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_MPU_LAR5_EN_Msk = 0x1
	// Bit EN.
	DMA_MPU_LAR5_EN = 0x1

	// MPU_BAR6: Base address register for MPU region 6. Writable only from a Secure, Privileged context.
	// Position of ADDR field.
	DMA_MPU_BAR6_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_BAR6_ADDR_Msk = 0xffffffe0

	// MPU_LAR6: Limit address register for MPU region 6. Writable only from a Secure, Privileged context, with the exception of the P bit.
	// Position of ADDR field.
	DMA_MPU_LAR6_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_LAR6_ADDR_Msk = 0xffffffe0
	// Position of S field.
	DMA_MPU_LAR6_S_Pos = 0x2
	// Bit mask of S field.
	DMA_MPU_LAR6_S_Msk = 0x4
	// Bit S.
	DMA_MPU_LAR6_S = 0x4
	// Position of P field.
	DMA_MPU_LAR6_P_Pos = 0x1
	// Bit mask of P field.
	DMA_MPU_LAR6_P_Msk = 0x2
	// Bit P.
	DMA_MPU_LAR6_P = 0x2
	// Position of EN field.
	DMA_MPU_LAR6_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_MPU_LAR6_EN_Msk = 0x1
	// Bit EN.
	DMA_MPU_LAR6_EN = 0x1

	// MPU_BAR7: Base address register for MPU region 7. Writable only from a Secure, Privileged context.
	// Position of ADDR field.
	DMA_MPU_BAR7_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_BAR7_ADDR_Msk = 0xffffffe0

	// MPU_LAR7: Limit address register for MPU region 7. Writable only from a Secure, Privileged context, with the exception of the P bit.
	// Position of ADDR field.
	DMA_MPU_LAR7_ADDR_Pos = 0x5
	// Bit mask of ADDR field.
	DMA_MPU_LAR7_ADDR_Msk = 0xffffffe0
	// Position of S field.
	DMA_MPU_LAR7_S_Pos = 0x2
	// Bit mask of S field.
	DMA_MPU_LAR7_S_Msk = 0x4
	// Bit S.
	DMA_MPU_LAR7_S = 0x4
	// Position of P field.
	DMA_MPU_LAR7_P_Pos = 0x1
	// Bit mask of P field.
	DMA_MPU_LAR7_P_Msk = 0x2
	// Bit P.
	DMA_MPU_LAR7_P = 0x2
	// Position of EN field.
	DMA_MPU_LAR7_EN_Pos = 0x0
	// Bit mask of EN field.
	DMA_MPU_LAR7_EN_Msk = 0x1
	// Bit EN.
	DMA_MPU_LAR7_EN = 0x1

	// CH0_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH0_DBG_CTDREQ field.
	DMA_CH0_DBG_CTDREQ_CH0_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH0_DBG_CTDREQ field.
	DMA_CH0_DBG_CTDREQ_CH0_DBG_CTDREQ_Msk = 0x3f

	// CH0_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH0_DBG_TCR field.
	DMA_CH0_DBG_TCR_CH0_DBG_TCR_Pos = 0x0
	// Bit mask of CH0_DBG_TCR field.
	DMA_CH0_DBG_TCR_CH0_DBG_TCR_Msk = 0xffffffff

	// CH1_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH1_DBG_CTDREQ field.
	DMA_CH1_DBG_CTDREQ_CH1_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH1_DBG_CTDREQ field.
	DMA_CH1_DBG_CTDREQ_CH1_DBG_CTDREQ_Msk = 0x3f

	// CH1_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH1_DBG_TCR field.
	DMA_CH1_DBG_TCR_CH1_DBG_TCR_Pos = 0x0
	// Bit mask of CH1_DBG_TCR field.
	DMA_CH1_DBG_TCR_CH1_DBG_TCR_Msk = 0xffffffff

	// CH2_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH2_DBG_CTDREQ field.
	DMA_CH2_DBG_CTDREQ_CH2_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH2_DBG_CTDREQ field.
	DMA_CH2_DBG_CTDREQ_CH2_DBG_CTDREQ_Msk = 0x3f

	// CH2_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH2_DBG_TCR field.
	DMA_CH2_DBG_TCR_CH2_DBG_TCR_Pos = 0x0
	// Bit mask of CH2_DBG_TCR field.
	DMA_CH2_DBG_TCR_CH2_DBG_TCR_Msk = 0xffffffff

	// CH3_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH3_DBG_CTDREQ field.
	DMA_CH3_DBG_CTDREQ_CH3_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH3_DBG_CTDREQ field.
	DMA_CH3_DBG_CTDREQ_CH3_DBG_CTDREQ_Msk = 0x3f

	// CH3_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH3_DBG_TCR field.
	DMA_CH3_DBG_TCR_CH3_DBG_TCR_Pos = 0x0
	// Bit mask of CH3_DBG_TCR field.
	DMA_CH3_DBG_TCR_CH3_DBG_TCR_Msk = 0xffffffff

	// CH4_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH4_DBG_CTDREQ field.
	DMA_CH4_DBG_CTDREQ_CH4_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH4_DBG_CTDREQ field.
	DMA_CH4_DBG_CTDREQ_CH4_DBG_CTDREQ_Msk = 0x3f

	// CH4_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH4_DBG_TCR field.
	DMA_CH4_DBG_TCR_CH4_DBG_TCR_Pos = 0x0
	// Bit mask of CH4_DBG_TCR field.
	DMA_CH4_DBG_TCR_CH4_DBG_TCR_Msk = 0xffffffff

	// CH5_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH5_DBG_CTDREQ field.
	DMA_CH5_DBG_CTDREQ_CH5_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH5_DBG_CTDREQ field.
	DMA_CH5_DBG_CTDREQ_CH5_DBG_CTDREQ_Msk = 0x3f

	// CH5_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH5_DBG_TCR field.
	DMA_CH5_DBG_TCR_CH5_DBG_TCR_Pos = 0x0
	// Bit mask of CH5_DBG_TCR field.
	DMA_CH5_DBG_TCR_CH5_DBG_TCR_Msk = 0xffffffff

	// CH6_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH6_DBG_CTDREQ field.
	DMA_CH6_DBG_CTDREQ_CH6_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH6_DBG_CTDREQ field.
	DMA_CH6_DBG_CTDREQ_CH6_DBG_CTDREQ_Msk = 0x3f

	// CH6_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH6_DBG_TCR field.
	DMA_CH6_DBG_TCR_CH6_DBG_TCR_Pos = 0x0
	// Bit mask of CH6_DBG_TCR field.
	DMA_CH6_DBG_TCR_CH6_DBG_TCR_Msk = 0xffffffff

	// CH7_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH7_DBG_CTDREQ field.
	DMA_CH7_DBG_CTDREQ_CH7_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH7_DBG_CTDREQ field.
	DMA_CH7_DBG_CTDREQ_CH7_DBG_CTDREQ_Msk = 0x3f

	// CH7_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH7_DBG_TCR field.
	DMA_CH7_DBG_TCR_CH7_DBG_TCR_Pos = 0x0
	// Bit mask of CH7_DBG_TCR field.
	DMA_CH7_DBG_TCR_CH7_DBG_TCR_Msk = 0xffffffff

	// CH8_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH8_DBG_CTDREQ field.
	DMA_CH8_DBG_CTDREQ_CH8_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH8_DBG_CTDREQ field.
	DMA_CH8_DBG_CTDREQ_CH8_DBG_CTDREQ_Msk = 0x3f

	// CH8_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH8_DBG_TCR field.
	DMA_CH8_DBG_TCR_CH8_DBG_TCR_Pos = 0x0
	// Bit mask of CH8_DBG_TCR field.
	DMA_CH8_DBG_TCR_CH8_DBG_TCR_Msk = 0xffffffff

	// CH9_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH9_DBG_CTDREQ field.
	DMA_CH9_DBG_CTDREQ_CH9_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH9_DBG_CTDREQ field.
	DMA_CH9_DBG_CTDREQ_CH9_DBG_CTDREQ_Msk = 0x3f

	// CH9_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH9_DBG_TCR field.
	DMA_CH9_DBG_TCR_CH9_DBG_TCR_Pos = 0x0
	// Bit mask of CH9_DBG_TCR field.
	DMA_CH9_DBG_TCR_CH9_DBG_TCR_Msk = 0xffffffff

	// CH10_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH10_DBG_CTDREQ field.
	DMA_CH10_DBG_CTDREQ_CH10_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH10_DBG_CTDREQ field.
	DMA_CH10_DBG_CTDREQ_CH10_DBG_CTDREQ_Msk = 0x3f

	// CH10_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH10_DBG_TCR field.
	DMA_CH10_DBG_TCR_CH10_DBG_TCR_Pos = 0x0
	// Bit mask of CH10_DBG_TCR field.
	DMA_CH10_DBG_TCR_CH10_DBG_TCR_Msk = 0xffffffff

	// CH11_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH11_DBG_CTDREQ field.
	DMA_CH11_DBG_CTDREQ_CH11_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH11_DBG_CTDREQ field.
	DMA_CH11_DBG_CTDREQ_CH11_DBG_CTDREQ_Msk = 0x3f

	// CH11_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH11_DBG_TCR field.
	DMA_CH11_DBG_TCR_CH11_DBG_TCR_Pos = 0x0
	// Bit mask of CH11_DBG_TCR field.
	DMA_CH11_DBG_TCR_CH11_DBG_TCR_Msk = 0xffffffff

	// CH12_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH12_DBG_CTDREQ field.
	DMA_CH12_DBG_CTDREQ_CH12_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH12_DBG_CTDREQ field.
	DMA_CH12_DBG_CTDREQ_CH12_DBG_CTDREQ_Msk = 0x3f

	// CH12_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH12_DBG_TCR field.
	DMA_CH12_DBG_TCR_CH12_DBG_TCR_Pos = 0x0
	// Bit mask of CH12_DBG_TCR field.
	DMA_CH12_DBG_TCR_CH12_DBG_TCR_Msk = 0xffffffff

	// CH13_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH13_DBG_CTDREQ field.
	DMA_CH13_DBG_CTDREQ_CH13_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH13_DBG_CTDREQ field.
	DMA_CH13_DBG_CTDREQ_CH13_DBG_CTDREQ_Msk = 0x3f

	// CH13_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH13_DBG_TCR field.
	DMA_CH13_DBG_TCR_CH13_DBG_TCR_Pos = 0x0
	// Bit mask of CH13_DBG_TCR field.
	DMA_CH13_DBG_TCR_CH13_DBG_TCR_Msk = 0xffffffff

	// CH14_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH14_DBG_CTDREQ field.
	DMA_CH14_DBG_CTDREQ_CH14_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH14_DBG_CTDREQ field.
	DMA_CH14_DBG_CTDREQ_CH14_DBG_CTDREQ_Msk = 0x3f

	// CH14_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH14_DBG_TCR field.
	DMA_CH14_DBG_TCR_CH14_DBG_TCR_Pos = 0x0
	// Bit mask of CH14_DBG_TCR field.
	DMA_CH14_DBG_TCR_CH14_DBG_TCR_Msk = 0xffffffff

	// CH15_DBG_CTDREQ: Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
	// Position of CH15_DBG_CTDREQ field.
	DMA_CH15_DBG_CTDREQ_CH15_DBG_CTDREQ_Pos = 0x0
	// Bit mask of CH15_DBG_CTDREQ field.
	DMA_CH15_DBG_CTDREQ_CH15_DBG_CTDREQ_Msk = 0x3f

	// CH15_DBG_TCR: Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
	// Position of CH15_DBG_TCR field.
	DMA_CH15_DBG_TCR_CH15_DBG_TCR_Pos = 0x0
	// Bit mask of CH15_DBG_TCR field.
	DMA_CH15_DBG_TCR_CH15_DBG_TCR_Msk = 0xffffffff
)

// Constants for TIMER0: Controls time and alarms time is a 64 bit value indicating the time since power-on timeh is the top 32 bits of time & timel is the bottom 32 bits to change time write to timelw before timehw to read time read from timelr before timehr An alarm is set by setting alarm_enable and writing to the corresponding alarm register When an alarm is pending, the corresponding alarm_running signal will be high An alarm can be cancelled before it has finished by clearing the alarm_enable When an alarm fires, the corresponding alarm_irq is set and alarm_running is cleared To clear the interrupt write a 1 to the corresponding alarm_irq The timer can be locked to prevent writing
const (
	// TIMEHW: Write to bits 63:32 of time always write timelw before timehw
	// Position of TIMEHW field.
	TIMER0_TIMEHW_TIMEHW_Pos = 0x0
	// Bit mask of TIMEHW field.
	TIMER0_TIMEHW_TIMEHW_Msk = 0xffffffff

	// TIMELW: Write to bits 31:0 of time writes do not get copied to time until timehw is written
	// Position of TIMELW field.
	TIMER0_TIMELW_TIMELW_Pos = 0x0
	// Bit mask of TIMELW field.
	TIMER0_TIMELW_TIMELW_Msk = 0xffffffff

	// TIMEHR: Read from bits 63:32 of time always read timelr before timehr
	// Position of TIMEHR field.
	TIMER0_TIMEHR_TIMEHR_Pos = 0x0
	// Bit mask of TIMEHR field.
	TIMER0_TIMEHR_TIMEHR_Msk = 0xffffffff

	// TIMELR: Read from bits 31:0 of time
	// Position of TIMELR field.
	TIMER0_TIMELR_TIMELR_Pos = 0x0
	// Bit mask of TIMELR field.
	TIMER0_TIMELR_TIMELR_Msk = 0xffffffff

	// ALARM0: Arm alarm 0, and configure the time it will fire. Once armed, the alarm fires when TIMER_ALARM0 == TIMELR. The alarm will disarm itself once it fires, and can be disarmed early using the ARMED status register.
	// Position of ALARM0 field.
	TIMER0_ALARM0_ALARM0_Pos = 0x0
	// Bit mask of ALARM0 field.
	TIMER0_ALARM0_ALARM0_Msk = 0xffffffff

	// ALARM1: Arm alarm 1, and configure the time it will fire. Once armed, the alarm fires when TIMER_ALARM1 == TIMELR. The alarm will disarm itself once it fires, and can be disarmed early using the ARMED status register.
	// Position of ALARM1 field.
	TIMER0_ALARM1_ALARM1_Pos = 0x0
	// Bit mask of ALARM1 field.
	TIMER0_ALARM1_ALARM1_Msk = 0xffffffff

	// ALARM2: Arm alarm 2, and configure the time it will fire. Once armed, the alarm fires when TIMER_ALARM2 == TIMELR. The alarm will disarm itself once it fires, and can be disarmed early using the ARMED status register.
	// Position of ALARM2 field.
	TIMER0_ALARM2_ALARM2_Pos = 0x0
	// Bit mask of ALARM2 field.
	TIMER0_ALARM2_ALARM2_Msk = 0xffffffff

	// ALARM3: Arm alarm 3, and configure the time it will fire. Once armed, the alarm fires when TIMER_ALARM3 == TIMELR. The alarm will disarm itself once it fires, and can be disarmed early using the ARMED status register.
	// Position of ALARM3 field.
	TIMER0_ALARM3_ALARM3_Pos = 0x0
	// Bit mask of ALARM3 field.
	TIMER0_ALARM3_ALARM3_Msk = 0xffffffff

	// ARMED: Indicates the armed/disarmed status of each alarm. A write to the corresponding ALARMx register arms the alarm. Alarms automatically disarm upon firing, but writing ones here will disarm immediately without waiting to fire.
	// Position of ARMED field.
	TIMER0_ARMED_ARMED_Pos = 0x0
	// Bit mask of ARMED field.
	TIMER0_ARMED_ARMED_Msk = 0xf

	// TIMERAWH: Raw read from bits 63:32 of time (no side effects)
	// Position of TIMERAWH field.
	TIMER0_TIMERAWH_TIMERAWH_Pos = 0x0
	// Bit mask of TIMERAWH field.
	TIMER0_TIMERAWH_TIMERAWH_Msk = 0xffffffff

	// TIMERAWL: Raw read from bits 31:0 of time (no side effects)
	// Position of TIMERAWL field.
	TIMER0_TIMERAWL_TIMERAWL_Pos = 0x0
	// Bit mask of TIMERAWL field.
	TIMER0_TIMERAWL_TIMERAWL_Msk = 0xffffffff

	// DBGPAUSE: Set bits high to enable pause when the corresponding debug ports are active
	// Position of DBG1 field.
	TIMER0_DBGPAUSE_DBG1_Pos = 0x2
	// Bit mask of DBG1 field.
	TIMER0_DBGPAUSE_DBG1_Msk = 0x4
	// Bit DBG1.
	TIMER0_DBGPAUSE_DBG1 = 0x4
	// Position of DBG0 field.
	TIMER0_DBGPAUSE_DBG0_Pos = 0x1
	// Bit mask of DBG0 field.
	TIMER0_DBGPAUSE_DBG0_Msk = 0x2
	// Bit DBG0.
	TIMER0_DBGPAUSE_DBG0 = 0x2

	// PAUSE: Set high to pause the timer
	// Position of PAUSE field.
	TIMER0_PAUSE_PAUSE_Pos = 0x0
	// Bit mask of PAUSE field.
	TIMER0_PAUSE_PAUSE_Msk = 0x1
	// Bit PAUSE.
	TIMER0_PAUSE_PAUSE = 0x1

	// LOCKED: Set locked bit to disable write access to timer Once set, cannot be cleared (without a reset)
	// Position of LOCKED field.
	TIMER0_LOCKED_LOCKED_Pos = 0x0
	// Bit mask of LOCKED field.
	TIMER0_LOCKED_LOCKED_Msk = 0x1
	// Bit LOCKED.
	TIMER0_LOCKED_LOCKED = 0x1

	// SOURCE: Selects the source for the timer. Defaults to the normal tick configured in the ticks block (typically configured to 1 microsecond). Writing to 1 will ignore the tick and count clk_sys cycles instead.
	// Position of CLK_SYS field.
	TIMER0_SOURCE_CLK_SYS_Pos = 0x0
	// Bit mask of CLK_SYS field.
	TIMER0_SOURCE_CLK_SYS_Msk = 0x1
	// Bit CLK_SYS.
	TIMER0_SOURCE_CLK_SYS         = 0x1
	TIMER0_SOURCE_CLK_SYS_TICK    = 0x0
	TIMER0_SOURCE_CLK_SYS_CLK_SYS = 0x1

	// INTR: Raw Interrupts
	// Position of ALARM_3 field.
	TIMER0_INTR_ALARM_3_Pos = 0x3
	// Bit mask of ALARM_3 field.
	TIMER0_INTR_ALARM_3_Msk = 0x8
	// Bit ALARM_3.
	TIMER0_INTR_ALARM_3 = 0x8
	// Position of ALARM_2 field.
	TIMER0_INTR_ALARM_2_Pos = 0x2
	// Bit mask of ALARM_2 field.
	TIMER0_INTR_ALARM_2_Msk = 0x4
	// Bit ALARM_2.
	TIMER0_INTR_ALARM_2 = 0x4
	// Position of ALARM_1 field.
	TIMER0_INTR_ALARM_1_Pos = 0x1
	// Bit mask of ALARM_1 field.
	TIMER0_INTR_ALARM_1_Msk = 0x2
	// Bit ALARM_1.
	TIMER0_INTR_ALARM_1 = 0x2
	// Position of ALARM_0 field.
	TIMER0_INTR_ALARM_0_Pos = 0x0
	// Bit mask of ALARM_0 field.
	TIMER0_INTR_ALARM_0_Msk = 0x1
	// Bit ALARM_0.
	TIMER0_INTR_ALARM_0 = 0x1

	// INTE: Interrupt Enable
	// Position of ALARM_3 field.
	TIMER0_INTE_ALARM_3_Pos = 0x3
	// Bit mask of ALARM_3 field.
	TIMER0_INTE_ALARM_3_Msk = 0x8
	// Bit ALARM_3.
	TIMER0_INTE_ALARM_3 = 0x8
	// Position of ALARM_2 field.
	TIMER0_INTE_ALARM_2_Pos = 0x2
	// Bit mask of ALARM_2 field.
	TIMER0_INTE_ALARM_2_Msk = 0x4
	// Bit ALARM_2.
	TIMER0_INTE_ALARM_2 = 0x4
	// Position of ALARM_1 field.
	TIMER0_INTE_ALARM_1_Pos = 0x1
	// Bit mask of ALARM_1 field.
	TIMER0_INTE_ALARM_1_Msk = 0x2
	// Bit ALARM_1.
	TIMER0_INTE_ALARM_1 = 0x2
	// Position of ALARM_0 field.
	TIMER0_INTE_ALARM_0_Pos = 0x0
	// Bit mask of ALARM_0 field.
	TIMER0_INTE_ALARM_0_Msk = 0x1
	// Bit ALARM_0.
	TIMER0_INTE_ALARM_0 = 0x1

	// INTF: Interrupt Force
	// Position of ALARM_3 field.
	TIMER0_INTF_ALARM_3_Pos = 0x3
	// Bit mask of ALARM_3 field.
	TIMER0_INTF_ALARM_3_Msk = 0x8
	// Bit ALARM_3.
	TIMER0_INTF_ALARM_3 = 0x8
	// Position of ALARM_2 field.
	TIMER0_INTF_ALARM_2_Pos = 0x2
	// Bit mask of ALARM_2 field.
	TIMER0_INTF_ALARM_2_Msk = 0x4
	// Bit ALARM_2.
	TIMER0_INTF_ALARM_2 = 0x4
	// Position of ALARM_1 field.
	TIMER0_INTF_ALARM_1_Pos = 0x1
	// Bit mask of ALARM_1 field.
	TIMER0_INTF_ALARM_1_Msk = 0x2
	// Bit ALARM_1.
	TIMER0_INTF_ALARM_1 = 0x2
	// Position of ALARM_0 field.
	TIMER0_INTF_ALARM_0_Pos = 0x0
	// Bit mask of ALARM_0 field.
	TIMER0_INTF_ALARM_0_Msk = 0x1
	// Bit ALARM_0.
	TIMER0_INTF_ALARM_0 = 0x1

	// INTS: Interrupt status after masking & forcing
	// Position of ALARM_3 field.
	TIMER0_INTS_ALARM_3_Pos = 0x3
	// Bit mask of ALARM_3 field.
	TIMER0_INTS_ALARM_3_Msk = 0x8
	// Bit ALARM_3.
	TIMER0_INTS_ALARM_3 = 0x8
	// Position of ALARM_2 field.
	TIMER0_INTS_ALARM_2_Pos = 0x2
	// Bit mask of ALARM_2 field.
	TIMER0_INTS_ALARM_2_Msk = 0x4
	// Bit ALARM_2.
	TIMER0_INTS_ALARM_2 = 0x4
	// Position of ALARM_1 field.
	TIMER0_INTS_ALARM_1_Pos = 0x1
	// Bit mask of ALARM_1 field.
	TIMER0_INTS_ALARM_1_Msk = 0x2
	// Bit ALARM_1.
	TIMER0_INTS_ALARM_1 = 0x2
	// Position of ALARM_0 field.
	TIMER0_INTS_ALARM_0_Pos = 0x0
	// Bit mask of ALARM_0 field.
	TIMER0_INTS_ALARM_0_Msk = 0x1
	// Bit ALARM_0.
	TIMER0_INTS_ALARM_0 = 0x1
)

// Constants for PWM: Simple PWM
const (
	// CH0_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH0_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH0_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH0_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH0_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH0_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH0_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH0_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH0_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH0_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH0_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH0_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH0_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH0_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH0_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH0_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH0_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH0_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH0_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH0_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH0_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH0_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH0_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH0_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH0_CSR_EN = 0x1

	// CH0_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH0_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH0_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH0_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH0_DIV_FRAC_Msk = 0xf

	// CH0_CTR: Direct access to the PWM counter
	// Position of CH0_CTR field.
	PWM_CH0_CTR_CH0_CTR_Pos = 0x0
	// Bit mask of CH0_CTR field.
	PWM_CH0_CTR_CH0_CTR_Msk = 0xffff

	// CH0_CC: Counter compare values
	// Position of B field.
	PWM_CH0_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH0_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH0_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH0_CC_A_Msk = 0xffff

	// CH0_TOP: Counter wrap value
	// Position of CH0_TOP field.
	PWM_CH0_TOP_CH0_TOP_Pos = 0x0
	// Bit mask of CH0_TOP field.
	PWM_CH0_TOP_CH0_TOP_Msk = 0xffff

	// CH1_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH1_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH1_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH1_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH1_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH1_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH1_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH1_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH1_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH1_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH1_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH1_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH1_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH1_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH1_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH1_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH1_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH1_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH1_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH1_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH1_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH1_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH1_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH1_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH1_CSR_EN = 0x1

	// CH1_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH1_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH1_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH1_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH1_DIV_FRAC_Msk = 0xf

	// CH1_CTR: Direct access to the PWM counter
	// Position of CH1_CTR field.
	PWM_CH1_CTR_CH1_CTR_Pos = 0x0
	// Bit mask of CH1_CTR field.
	PWM_CH1_CTR_CH1_CTR_Msk = 0xffff

	// CH1_CC: Counter compare values
	// Position of B field.
	PWM_CH1_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH1_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH1_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH1_CC_A_Msk = 0xffff

	// CH1_TOP: Counter wrap value
	// Position of CH1_TOP field.
	PWM_CH1_TOP_CH1_TOP_Pos = 0x0
	// Bit mask of CH1_TOP field.
	PWM_CH1_TOP_CH1_TOP_Msk = 0xffff

	// CH2_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH2_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH2_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH2_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH2_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH2_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH2_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH2_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH2_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH2_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH2_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH2_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH2_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH2_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH2_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH2_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH2_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH2_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH2_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH2_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH2_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH2_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH2_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH2_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH2_CSR_EN = 0x1

	// CH2_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH2_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH2_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH2_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH2_DIV_FRAC_Msk = 0xf

	// CH2_CTR: Direct access to the PWM counter
	// Position of CH2_CTR field.
	PWM_CH2_CTR_CH2_CTR_Pos = 0x0
	// Bit mask of CH2_CTR field.
	PWM_CH2_CTR_CH2_CTR_Msk = 0xffff

	// CH2_CC: Counter compare values
	// Position of B field.
	PWM_CH2_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH2_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH2_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH2_CC_A_Msk = 0xffff

	// CH2_TOP: Counter wrap value
	// Position of CH2_TOP field.
	PWM_CH2_TOP_CH2_TOP_Pos = 0x0
	// Bit mask of CH2_TOP field.
	PWM_CH2_TOP_CH2_TOP_Msk = 0xffff

	// CH3_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH3_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH3_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH3_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH3_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH3_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH3_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH3_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH3_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH3_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH3_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH3_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH3_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH3_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH3_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH3_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH3_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH3_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH3_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH3_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH3_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH3_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH3_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH3_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH3_CSR_EN = 0x1

	// CH3_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH3_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH3_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH3_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH3_DIV_FRAC_Msk = 0xf

	// CH3_CTR: Direct access to the PWM counter
	// Position of CH3_CTR field.
	PWM_CH3_CTR_CH3_CTR_Pos = 0x0
	// Bit mask of CH3_CTR field.
	PWM_CH3_CTR_CH3_CTR_Msk = 0xffff

	// CH3_CC: Counter compare values
	// Position of B field.
	PWM_CH3_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH3_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH3_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH3_CC_A_Msk = 0xffff

	// CH3_TOP: Counter wrap value
	// Position of CH3_TOP field.
	PWM_CH3_TOP_CH3_TOP_Pos = 0x0
	// Bit mask of CH3_TOP field.
	PWM_CH3_TOP_CH3_TOP_Msk = 0xffff

	// CH4_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH4_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH4_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH4_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH4_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH4_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH4_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH4_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH4_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH4_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH4_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH4_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH4_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH4_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH4_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH4_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH4_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH4_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH4_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH4_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH4_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH4_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH4_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH4_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH4_CSR_EN = 0x1

	// CH4_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH4_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH4_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH4_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH4_DIV_FRAC_Msk = 0xf

	// CH4_CTR: Direct access to the PWM counter
	// Position of CH4_CTR field.
	PWM_CH4_CTR_CH4_CTR_Pos = 0x0
	// Bit mask of CH4_CTR field.
	PWM_CH4_CTR_CH4_CTR_Msk = 0xffff

	// CH4_CC: Counter compare values
	// Position of B field.
	PWM_CH4_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH4_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH4_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH4_CC_A_Msk = 0xffff

	// CH4_TOP: Counter wrap value
	// Position of CH4_TOP field.
	PWM_CH4_TOP_CH4_TOP_Pos = 0x0
	// Bit mask of CH4_TOP field.
	PWM_CH4_TOP_CH4_TOP_Msk = 0xffff

	// CH5_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH5_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH5_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH5_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH5_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH5_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH5_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH5_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH5_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH5_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH5_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH5_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH5_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH5_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH5_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH5_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH5_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH5_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH5_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH5_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH5_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH5_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH5_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH5_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH5_CSR_EN = 0x1

	// CH5_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH5_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH5_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH5_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH5_DIV_FRAC_Msk = 0xf

	// CH5_CTR: Direct access to the PWM counter
	// Position of CH5_CTR field.
	PWM_CH5_CTR_CH5_CTR_Pos = 0x0
	// Bit mask of CH5_CTR field.
	PWM_CH5_CTR_CH5_CTR_Msk = 0xffff

	// CH5_CC: Counter compare values
	// Position of B field.
	PWM_CH5_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH5_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH5_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH5_CC_A_Msk = 0xffff

	// CH5_TOP: Counter wrap value
	// Position of CH5_TOP field.
	PWM_CH5_TOP_CH5_TOP_Pos = 0x0
	// Bit mask of CH5_TOP field.
	PWM_CH5_TOP_CH5_TOP_Msk = 0xffff

	// CH6_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH6_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH6_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH6_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH6_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH6_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH6_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH6_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH6_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH6_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH6_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH6_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH6_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH6_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH6_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH6_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH6_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH6_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH6_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH6_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH6_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH6_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH6_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH6_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH6_CSR_EN = 0x1

	// CH6_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH6_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH6_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH6_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH6_DIV_FRAC_Msk = 0xf

	// CH6_CTR: Direct access to the PWM counter
	// Position of CH6_CTR field.
	PWM_CH6_CTR_CH6_CTR_Pos = 0x0
	// Bit mask of CH6_CTR field.
	PWM_CH6_CTR_CH6_CTR_Msk = 0xffff

	// CH6_CC: Counter compare values
	// Position of B field.
	PWM_CH6_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH6_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH6_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH6_CC_A_Msk = 0xffff

	// CH6_TOP: Counter wrap value
	// Position of CH6_TOP field.
	PWM_CH6_TOP_CH6_TOP_Pos = 0x0
	// Bit mask of CH6_TOP field.
	PWM_CH6_TOP_CH6_TOP_Msk = 0xffff

	// CH7_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH7_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH7_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH7_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH7_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH7_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH7_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH7_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH7_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH7_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH7_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH7_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH7_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH7_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH7_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH7_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH7_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH7_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH7_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH7_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH7_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH7_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH7_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH7_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH7_CSR_EN = 0x1

	// CH7_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH7_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH7_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH7_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH7_DIV_FRAC_Msk = 0xf

	// CH7_CTR: Direct access to the PWM counter
	// Position of CH7_CTR field.
	PWM_CH7_CTR_CH7_CTR_Pos = 0x0
	// Bit mask of CH7_CTR field.
	PWM_CH7_CTR_CH7_CTR_Msk = 0xffff

	// CH7_CC: Counter compare values
	// Position of B field.
	PWM_CH7_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH7_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH7_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH7_CC_A_Msk = 0xffff

	// CH7_TOP: Counter wrap value
	// Position of CH7_TOP field.
	PWM_CH7_TOP_CH7_TOP_Pos = 0x0
	// Bit mask of CH7_TOP field.
	PWM_CH7_TOP_CH7_TOP_Msk = 0xffff

	// CH8_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH8_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH8_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH8_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH8_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH8_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH8_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH8_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH8_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH8_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH8_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH8_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH8_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH8_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH8_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH8_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH8_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH8_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH8_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH8_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH8_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH8_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH8_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH8_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH8_CSR_EN = 0x1

	// CH8_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH8_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH8_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH8_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH8_DIV_FRAC_Msk = 0xf

	// CH8_CTR: Direct access to the PWM counter
	// Position of CH8_CTR field.
	PWM_CH8_CTR_CH8_CTR_Pos = 0x0
	// Bit mask of CH8_CTR field.
	PWM_CH8_CTR_CH8_CTR_Msk = 0xffff

	// CH8_CC: Counter compare values
	// Position of B field.
	PWM_CH8_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH8_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH8_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH8_CC_A_Msk = 0xffff

	// CH8_TOP: Counter wrap value
	// Position of CH8_TOP field.
	PWM_CH8_TOP_CH8_TOP_Pos = 0x0
	// Bit mask of CH8_TOP field.
	PWM_CH8_TOP_CH8_TOP_Msk = 0xffff

	// CH9_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH9_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH9_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH9_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH9_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH9_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH9_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH9_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH9_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH9_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH9_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH9_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH9_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH9_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH9_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH9_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH9_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH9_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH9_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH9_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH9_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH9_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH9_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH9_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH9_CSR_EN = 0x1

	// CH9_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH9_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH9_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH9_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH9_DIV_FRAC_Msk = 0xf

	// CH9_CTR: Direct access to the PWM counter
	// Position of CH9_CTR field.
	PWM_CH9_CTR_CH9_CTR_Pos = 0x0
	// Bit mask of CH9_CTR field.
	PWM_CH9_CTR_CH9_CTR_Msk = 0xffff

	// CH9_CC: Counter compare values
	// Position of B field.
	PWM_CH9_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH9_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH9_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH9_CC_A_Msk = 0xffff

	// CH9_TOP: Counter wrap value
	// Position of CH9_TOP field.
	PWM_CH9_TOP_CH9_TOP_Pos = 0x0
	// Bit mask of CH9_TOP field.
	PWM_CH9_TOP_CH9_TOP_Msk = 0xffff

	// CH10_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH10_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH10_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH10_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH10_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH10_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH10_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH10_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH10_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH10_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH10_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH10_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH10_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH10_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH10_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH10_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH10_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH10_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH10_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH10_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH10_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH10_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH10_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH10_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH10_CSR_EN = 0x1

	// CH10_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH10_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH10_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH10_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH10_DIV_FRAC_Msk = 0xf

	// CH10_CTR: Direct access to the PWM counter
	// Position of CH10_CTR field.
	PWM_CH10_CTR_CH10_CTR_Pos = 0x0
	// Bit mask of CH10_CTR field.
	PWM_CH10_CTR_CH10_CTR_Msk = 0xffff

	// CH10_CC: Counter compare values
	// Position of B field.
	PWM_CH10_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH10_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH10_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH10_CC_A_Msk = 0xffff

	// CH10_TOP: Counter wrap value
	// Position of CH10_TOP field.
	PWM_CH10_TOP_CH10_TOP_Pos = 0x0
	// Bit mask of CH10_TOP field.
	PWM_CH10_TOP_CH10_TOP_Msk = 0xffff

	// CH11_CSR: Control and status register
	// Position of PH_ADV field.
	PWM_CH11_CSR_PH_ADV_Pos = 0x7
	// Bit mask of PH_ADV field.
	PWM_CH11_CSR_PH_ADV_Msk = 0x80
	// Bit PH_ADV.
	PWM_CH11_CSR_PH_ADV = 0x80
	// Position of PH_RET field.
	PWM_CH11_CSR_PH_RET_Pos = 0x6
	// Bit mask of PH_RET field.
	PWM_CH11_CSR_PH_RET_Msk = 0x40
	// Bit PH_RET.
	PWM_CH11_CSR_PH_RET = 0x40
	// Position of DIVMODE field.
	PWM_CH11_CSR_DIVMODE_Pos = 0x4
	// Bit mask of DIVMODE field.
	PWM_CH11_CSR_DIVMODE_Msk = 0x30
	// Free-running counting at rate dictated by fractional divider
	PWM_CH11_CSR_DIVMODE_DIV = 0x0
	// Fractional divider operation is gated by the PWM B pin.
	PWM_CH11_CSR_DIVMODE_LEVEL = 0x1
	// Counter advances with each rising edge of the PWM B pin.
	PWM_CH11_CSR_DIVMODE_RISE = 0x2
	// Counter advances with each falling edge of the PWM B pin.
	PWM_CH11_CSR_DIVMODE_FALL = 0x3
	// Position of B_INV field.
	PWM_CH11_CSR_B_INV_Pos = 0x3
	// Bit mask of B_INV field.
	PWM_CH11_CSR_B_INV_Msk = 0x8
	// Bit B_INV.
	PWM_CH11_CSR_B_INV = 0x8
	// Position of A_INV field.
	PWM_CH11_CSR_A_INV_Pos = 0x2
	// Bit mask of A_INV field.
	PWM_CH11_CSR_A_INV_Msk = 0x4
	// Bit A_INV.
	PWM_CH11_CSR_A_INV = 0x4
	// Position of PH_CORRECT field.
	PWM_CH11_CSR_PH_CORRECT_Pos = 0x1
	// Bit mask of PH_CORRECT field.
	PWM_CH11_CSR_PH_CORRECT_Msk = 0x2
	// Bit PH_CORRECT.
	PWM_CH11_CSR_PH_CORRECT = 0x2
	// Position of EN field.
	PWM_CH11_CSR_EN_Pos = 0x0
	// Bit mask of EN field.
	PWM_CH11_CSR_EN_Msk = 0x1
	// Bit EN.
	PWM_CH11_CSR_EN = 0x1

	// CH11_DIV: INT and FRAC form a fixed-point fractional number. Counting rate is system clock frequency divided by this number. Fractional division uses simple 1st-order sigma-delta.
	// Position of INT field.
	PWM_CH11_DIV_INT_Pos = 0x4
	// Bit mask of INT field.
	PWM_CH11_DIV_INT_Msk = 0xff0
	// Position of FRAC field.
	PWM_CH11_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	PWM_CH11_DIV_FRAC_Msk = 0xf

	// CH11_CTR: Direct access to the PWM counter
	// Position of CH11_CTR field.
	PWM_CH11_CTR_CH11_CTR_Pos = 0x0
	// Bit mask of CH11_CTR field.
	PWM_CH11_CTR_CH11_CTR_Msk = 0xffff

	// CH11_CC: Counter compare values
	// Position of B field.
	PWM_CH11_CC_B_Pos = 0x10
	// Bit mask of B field.
	PWM_CH11_CC_B_Msk = 0xffff0000
	// Position of A field.
	PWM_CH11_CC_A_Pos = 0x0
	// Bit mask of A field.
	PWM_CH11_CC_A_Msk = 0xffff

	// CH11_TOP: Counter wrap value
	// Position of CH11_TOP field.
	PWM_CH11_TOP_CH11_TOP_Pos = 0x0
	// Bit mask of CH11_TOP field.
	PWM_CH11_TOP_CH11_TOP_Msk = 0xffff

	// EN: This register aliases the CSR_EN bits for all channels. Writing to this register allows multiple channels to be enabled or disabled simultaneously, so they can run in perfect sync. For each channel, there is only one physical EN register bit, which can be accessed through here or CHx_CSR.
	// Position of CH11 field.
	PWM_EN_CH11_Pos = 0xb
	// Bit mask of CH11 field.
	PWM_EN_CH11_Msk = 0x800
	// Bit CH11.
	PWM_EN_CH11 = 0x800
	// Position of CH10 field.
	PWM_EN_CH10_Pos = 0xa
	// Bit mask of CH10 field.
	PWM_EN_CH10_Msk = 0x400
	// Bit CH10.
	PWM_EN_CH10 = 0x400
	// Position of CH9 field.
	PWM_EN_CH9_Pos = 0x9
	// Bit mask of CH9 field.
	PWM_EN_CH9_Msk = 0x200
	// Bit CH9.
	PWM_EN_CH9 = 0x200
	// Position of CH8 field.
	PWM_EN_CH8_Pos = 0x8
	// Bit mask of CH8 field.
	PWM_EN_CH8_Msk = 0x100
	// Bit CH8.
	PWM_EN_CH8 = 0x100
	// Position of CH7 field.
	PWM_EN_CH7_Pos = 0x7
	// Bit mask of CH7 field.
	PWM_EN_CH7_Msk = 0x80
	// Bit CH7.
	PWM_EN_CH7 = 0x80
	// Position of CH6 field.
	PWM_EN_CH6_Pos = 0x6
	// Bit mask of CH6 field.
	PWM_EN_CH6_Msk = 0x40
	// Bit CH6.
	PWM_EN_CH6 = 0x40
	// Position of CH5 field.
	PWM_EN_CH5_Pos = 0x5
	// Bit mask of CH5 field.
	PWM_EN_CH5_Msk = 0x20
	// Bit CH5.
	PWM_EN_CH5 = 0x20
	// Position of CH4 field.
	PWM_EN_CH4_Pos = 0x4
	// Bit mask of CH4 field.
	PWM_EN_CH4_Msk = 0x10
	// Bit CH4.
	PWM_EN_CH4 = 0x10
	// Position of CH3 field.
	PWM_EN_CH3_Pos = 0x3
	// Bit mask of CH3 field.
	PWM_EN_CH3_Msk = 0x8
	// Bit CH3.
	PWM_EN_CH3 = 0x8
	// Position of CH2 field.
	PWM_EN_CH2_Pos = 0x2
	// Bit mask of CH2 field.
	PWM_EN_CH2_Msk = 0x4
	// Bit CH2.
	PWM_EN_CH2 = 0x4
	// Position of CH1 field.
	PWM_EN_CH1_Pos = 0x1
	// Bit mask of CH1 field.
	PWM_EN_CH1_Msk = 0x2
	// Bit CH1.
	PWM_EN_CH1 = 0x2
	// Position of CH0 field.
	PWM_EN_CH0_Pos = 0x0
	// Bit mask of CH0 field.
	PWM_EN_CH0_Msk = 0x1
	// Bit CH0.
	PWM_EN_CH0 = 0x1

	// INTR: Raw Interrupts
	// Position of CH11 field.
	PWM_INTR_CH11_Pos = 0xb
	// Bit mask of CH11 field.
	PWM_INTR_CH11_Msk = 0x800
	// Bit CH11.
	PWM_INTR_CH11 = 0x800
	// Position of CH10 field.
	PWM_INTR_CH10_Pos = 0xa
	// Bit mask of CH10 field.
	PWM_INTR_CH10_Msk = 0x400
	// Bit CH10.
	PWM_INTR_CH10 = 0x400
	// Position of CH9 field.
	PWM_INTR_CH9_Pos = 0x9
	// Bit mask of CH9 field.
	PWM_INTR_CH9_Msk = 0x200
	// Bit CH9.
	PWM_INTR_CH9 = 0x200
	// Position of CH8 field.
	PWM_INTR_CH8_Pos = 0x8
	// Bit mask of CH8 field.
	PWM_INTR_CH8_Msk = 0x100
	// Bit CH8.
	PWM_INTR_CH8 = 0x100
	// Position of CH7 field.
	PWM_INTR_CH7_Pos = 0x7
	// Bit mask of CH7 field.
	PWM_INTR_CH7_Msk = 0x80
	// Bit CH7.
	PWM_INTR_CH7 = 0x80
	// Position of CH6 field.
	PWM_INTR_CH6_Pos = 0x6
	// Bit mask of CH6 field.
	PWM_INTR_CH6_Msk = 0x40
	// Bit CH6.
	PWM_INTR_CH6 = 0x40
	// Position of CH5 field.
	PWM_INTR_CH5_Pos = 0x5
	// Bit mask of CH5 field.
	PWM_INTR_CH5_Msk = 0x20
	// Bit CH5.
	PWM_INTR_CH5 = 0x20
	// Position of CH4 field.
	PWM_INTR_CH4_Pos = 0x4
	// Bit mask of CH4 field.
	PWM_INTR_CH4_Msk = 0x10
	// Bit CH4.
	PWM_INTR_CH4 = 0x10
	// Position of CH3 field.
	PWM_INTR_CH3_Pos = 0x3
	// Bit mask of CH3 field.
	PWM_INTR_CH3_Msk = 0x8
	// Bit CH3.
	PWM_INTR_CH3 = 0x8
	// Position of CH2 field.
	PWM_INTR_CH2_Pos = 0x2
	// Bit mask of CH2 field.
	PWM_INTR_CH2_Msk = 0x4
	// Bit CH2.
	PWM_INTR_CH2 = 0x4
	// Position of CH1 field.
	PWM_INTR_CH1_Pos = 0x1
	// Bit mask of CH1 field.
	PWM_INTR_CH1_Msk = 0x2
	// Bit CH1.
	PWM_INTR_CH1 = 0x2
	// Position of CH0 field.
	PWM_INTR_CH0_Pos = 0x0
	// Bit mask of CH0 field.
	PWM_INTR_CH0_Msk = 0x1
	// Bit CH0.
	PWM_INTR_CH0 = 0x1

	// IRQ0_INTE: Interrupt Enable for irq0
	// Position of CH11 field.
	PWM_IRQ0_INTE_CH11_Pos = 0xb
	// Bit mask of CH11 field.
	PWM_IRQ0_INTE_CH11_Msk = 0x800
	// Bit CH11.
	PWM_IRQ0_INTE_CH11 = 0x800
	// Position of CH10 field.
	PWM_IRQ0_INTE_CH10_Pos = 0xa
	// Bit mask of CH10 field.
	PWM_IRQ0_INTE_CH10_Msk = 0x400
	// Bit CH10.
	PWM_IRQ0_INTE_CH10 = 0x400
	// Position of CH9 field.
	PWM_IRQ0_INTE_CH9_Pos = 0x9
	// Bit mask of CH9 field.
	PWM_IRQ0_INTE_CH9_Msk = 0x200
	// Bit CH9.
	PWM_IRQ0_INTE_CH9 = 0x200
	// Position of CH8 field.
	PWM_IRQ0_INTE_CH8_Pos = 0x8
	// Bit mask of CH8 field.
	PWM_IRQ0_INTE_CH8_Msk = 0x100
	// Bit CH8.
	PWM_IRQ0_INTE_CH8 = 0x100
	// Position of CH7 field.
	PWM_IRQ0_INTE_CH7_Pos = 0x7
	// Bit mask of CH7 field.
	PWM_IRQ0_INTE_CH7_Msk = 0x80
	// Bit CH7.
	PWM_IRQ0_INTE_CH7 = 0x80
	// Position of CH6 field.
	PWM_IRQ0_INTE_CH6_Pos = 0x6
	// Bit mask of CH6 field.
	PWM_IRQ0_INTE_CH6_Msk = 0x40
	// Bit CH6.
	PWM_IRQ0_INTE_CH6 = 0x40
	// Position of CH5 field.
	PWM_IRQ0_INTE_CH5_Pos = 0x5
	// Bit mask of CH5 field.
	PWM_IRQ0_INTE_CH5_Msk = 0x20
	// Bit CH5.
	PWM_IRQ0_INTE_CH5 = 0x20
	// Position of CH4 field.
	PWM_IRQ0_INTE_CH4_Pos = 0x4
	// Bit mask of CH4 field.
	PWM_IRQ0_INTE_CH4_Msk = 0x10
	// Bit CH4.
	PWM_IRQ0_INTE_CH4 = 0x10
	// Position of CH3 field.
	PWM_IRQ0_INTE_CH3_Pos = 0x3
	// Bit mask of CH3 field.
	PWM_IRQ0_INTE_CH3_Msk = 0x8
	// Bit CH3.
	PWM_IRQ0_INTE_CH3 = 0x8
	// Position of CH2 field.
	PWM_IRQ0_INTE_CH2_Pos = 0x2
	// Bit mask of CH2 field.
	PWM_IRQ0_INTE_CH2_Msk = 0x4
	// Bit CH2.
	PWM_IRQ0_INTE_CH2 = 0x4
	// Position of CH1 field.
	PWM_IRQ0_INTE_CH1_Pos = 0x1
	// Bit mask of CH1 field.
	PWM_IRQ0_INTE_CH1_Msk = 0x2
	// Bit CH1.
	PWM_IRQ0_INTE_CH1 = 0x2
	// Position of CH0 field.
	PWM_IRQ0_INTE_CH0_Pos = 0x0
	// Bit mask of CH0 field.
	PWM_IRQ0_INTE_CH0_Msk = 0x1
	// Bit CH0.
	PWM_IRQ0_INTE_CH0 = 0x1

	// IRQ0_INTF: Interrupt Force for irq0
	// Position of CH11 field.
	PWM_IRQ0_INTF_CH11_Pos = 0xb
	// Bit mask of CH11 field.
	PWM_IRQ0_INTF_CH11_Msk = 0x800
	// Bit CH11.
	PWM_IRQ0_INTF_CH11 = 0x800
	// Position of CH10 field.
	PWM_IRQ0_INTF_CH10_Pos = 0xa
	// Bit mask of CH10 field.
	PWM_IRQ0_INTF_CH10_Msk = 0x400
	// Bit CH10.
	PWM_IRQ0_INTF_CH10 = 0x400
	// Position of CH9 field.
	PWM_IRQ0_INTF_CH9_Pos = 0x9
	// Bit mask of CH9 field.
	PWM_IRQ0_INTF_CH9_Msk = 0x200
	// Bit CH9.
	PWM_IRQ0_INTF_CH9 = 0x200
	// Position of CH8 field.
	PWM_IRQ0_INTF_CH8_Pos = 0x8
	// Bit mask of CH8 field.
	PWM_IRQ0_INTF_CH8_Msk = 0x100
	// Bit CH8.
	PWM_IRQ0_INTF_CH8 = 0x100
	// Position of CH7 field.
	PWM_IRQ0_INTF_CH7_Pos = 0x7
	// Bit mask of CH7 field.
	PWM_IRQ0_INTF_CH7_Msk = 0x80
	// Bit CH7.
	PWM_IRQ0_INTF_CH7 = 0x80
	// Position of CH6 field.
	PWM_IRQ0_INTF_CH6_Pos = 0x6
	// Bit mask of CH6 field.
	PWM_IRQ0_INTF_CH6_Msk = 0x40
	// Bit CH6.
	PWM_IRQ0_INTF_CH6 = 0x40
	// Position of CH5 field.
	PWM_IRQ0_INTF_CH5_Pos = 0x5
	// Bit mask of CH5 field.
	PWM_IRQ0_INTF_CH5_Msk = 0x20
	// Bit CH5.
	PWM_IRQ0_INTF_CH5 = 0x20
	// Position of CH4 field.
	PWM_IRQ0_INTF_CH4_Pos = 0x4
	// Bit mask of CH4 field.
	PWM_IRQ0_INTF_CH4_Msk = 0x10
	// Bit CH4.
	PWM_IRQ0_INTF_CH4 = 0x10
	// Position of CH3 field.
	PWM_IRQ0_INTF_CH3_Pos = 0x3
	// Bit mask of CH3 field.
	PWM_IRQ0_INTF_CH3_Msk = 0x8
	// Bit CH3.
	PWM_IRQ0_INTF_CH3 = 0x8
	// Position of CH2 field.
	PWM_IRQ0_INTF_CH2_Pos = 0x2
	// Bit mask of CH2 field.
	PWM_IRQ0_INTF_CH2_Msk = 0x4
	// Bit CH2.
	PWM_IRQ0_INTF_CH2 = 0x4
	// Position of CH1 field.
	PWM_IRQ0_INTF_CH1_Pos = 0x1
	// Bit mask of CH1 field.
	PWM_IRQ0_INTF_CH1_Msk = 0x2
	// Bit CH1.
	PWM_IRQ0_INTF_CH1 = 0x2
	// Position of CH0 field.
	PWM_IRQ0_INTF_CH0_Pos = 0x0
	// Bit mask of CH0 field.
	PWM_IRQ0_INTF_CH0_Msk = 0x1
	// Bit CH0.
	PWM_IRQ0_INTF_CH0 = 0x1

	// IRQ0_INTS: Interrupt status after masking & forcing for irq0
	// Position of CH11 field.
	PWM_IRQ0_INTS_CH11_Pos = 0xb
	// Bit mask of CH11 field.
	PWM_IRQ0_INTS_CH11_Msk = 0x800
	// Bit CH11.
	PWM_IRQ0_INTS_CH11 = 0x800
	// Position of CH10 field.
	PWM_IRQ0_INTS_CH10_Pos = 0xa
	// Bit mask of CH10 field.
	PWM_IRQ0_INTS_CH10_Msk = 0x400
	// Bit CH10.
	PWM_IRQ0_INTS_CH10 = 0x400
	// Position of CH9 field.
	PWM_IRQ0_INTS_CH9_Pos = 0x9
	// Bit mask of CH9 field.
	PWM_IRQ0_INTS_CH9_Msk = 0x200
	// Bit CH9.
	PWM_IRQ0_INTS_CH9 = 0x200
	// Position of CH8 field.
	PWM_IRQ0_INTS_CH8_Pos = 0x8
	// Bit mask of CH8 field.
	PWM_IRQ0_INTS_CH8_Msk = 0x100
	// Bit CH8.
	PWM_IRQ0_INTS_CH8 = 0x100
	// Position of CH7 field.
	PWM_IRQ0_INTS_CH7_Pos = 0x7
	// Bit mask of CH7 field.
	PWM_IRQ0_INTS_CH7_Msk = 0x80
	// Bit CH7.
	PWM_IRQ0_INTS_CH7 = 0x80
	// Position of CH6 field.
	PWM_IRQ0_INTS_CH6_Pos = 0x6
	// Bit mask of CH6 field.
	PWM_IRQ0_INTS_CH6_Msk = 0x40
	// Bit CH6.
	PWM_IRQ0_INTS_CH6 = 0x40
	// Position of CH5 field.
	PWM_IRQ0_INTS_CH5_Pos = 0x5
	// Bit mask of CH5 field.
	PWM_IRQ0_INTS_CH5_Msk = 0x20
	// Bit CH5.
	PWM_IRQ0_INTS_CH5 = 0x20
	// Position of CH4 field.
	PWM_IRQ0_INTS_CH4_Pos = 0x4
	// Bit mask of CH4 field.
	PWM_IRQ0_INTS_CH4_Msk = 0x10
	// Bit CH4.
	PWM_IRQ0_INTS_CH4 = 0x10
	// Position of CH3 field.
	PWM_IRQ0_INTS_CH3_Pos = 0x3
	// Bit mask of CH3 field.
	PWM_IRQ0_INTS_CH3_Msk = 0x8
	// Bit CH3.
	PWM_IRQ0_INTS_CH3 = 0x8
	// Position of CH2 field.
	PWM_IRQ0_INTS_CH2_Pos = 0x2
	// Bit mask of CH2 field.
	PWM_IRQ0_INTS_CH2_Msk = 0x4
	// Bit CH2.
	PWM_IRQ0_INTS_CH2 = 0x4
	// Position of CH1 field.
	PWM_IRQ0_INTS_CH1_Pos = 0x1
	// Bit mask of CH1 field.
	PWM_IRQ0_INTS_CH1_Msk = 0x2
	// Bit CH1.
	PWM_IRQ0_INTS_CH1 = 0x2
	// Position of CH0 field.
	PWM_IRQ0_INTS_CH0_Pos = 0x0
	// Bit mask of CH0 field.
	PWM_IRQ0_INTS_CH0_Msk = 0x1
	// Bit CH0.
	PWM_IRQ0_INTS_CH0 = 0x1

	// IRQ1_INTE: Interrupt Enable for irq1
	// Position of CH11 field.
	PWM_IRQ1_INTE_CH11_Pos = 0xb
	// Bit mask of CH11 field.
	PWM_IRQ1_INTE_CH11_Msk = 0x800
	// Bit CH11.
	PWM_IRQ1_INTE_CH11 = 0x800
	// Position of CH10 field.
	PWM_IRQ1_INTE_CH10_Pos = 0xa
	// Bit mask of CH10 field.
	PWM_IRQ1_INTE_CH10_Msk = 0x400
	// Bit CH10.
	PWM_IRQ1_INTE_CH10 = 0x400
	// Position of CH9 field.
	PWM_IRQ1_INTE_CH9_Pos = 0x9
	// Bit mask of CH9 field.
	PWM_IRQ1_INTE_CH9_Msk = 0x200
	// Bit CH9.
	PWM_IRQ1_INTE_CH9 = 0x200
	// Position of CH8 field.
	PWM_IRQ1_INTE_CH8_Pos = 0x8
	// Bit mask of CH8 field.
	PWM_IRQ1_INTE_CH8_Msk = 0x100
	// Bit CH8.
	PWM_IRQ1_INTE_CH8 = 0x100
	// Position of CH7 field.
	PWM_IRQ1_INTE_CH7_Pos = 0x7
	// Bit mask of CH7 field.
	PWM_IRQ1_INTE_CH7_Msk = 0x80
	// Bit CH7.
	PWM_IRQ1_INTE_CH7 = 0x80
	// Position of CH6 field.
	PWM_IRQ1_INTE_CH6_Pos = 0x6
	// Bit mask of CH6 field.
	PWM_IRQ1_INTE_CH6_Msk = 0x40
	// Bit CH6.
	PWM_IRQ1_INTE_CH6 = 0x40
	// Position of CH5 field.
	PWM_IRQ1_INTE_CH5_Pos = 0x5
	// Bit mask of CH5 field.
	PWM_IRQ1_INTE_CH5_Msk = 0x20
	// Bit CH5.
	PWM_IRQ1_INTE_CH5 = 0x20
	// Position of CH4 field.
	PWM_IRQ1_INTE_CH4_Pos = 0x4
	// Bit mask of CH4 field.
	PWM_IRQ1_INTE_CH4_Msk = 0x10
	// Bit CH4.
	PWM_IRQ1_INTE_CH4 = 0x10
	// Position of CH3 field.
	PWM_IRQ1_INTE_CH3_Pos = 0x3
	// Bit mask of CH3 field.
	PWM_IRQ1_INTE_CH3_Msk = 0x8
	// Bit CH3.
	PWM_IRQ1_INTE_CH3 = 0x8
	// Position of CH2 field.
	PWM_IRQ1_INTE_CH2_Pos = 0x2
	// Bit mask of CH2 field.
	PWM_IRQ1_INTE_CH2_Msk = 0x4
	// Bit CH2.
	PWM_IRQ1_INTE_CH2 = 0x4
	// Position of CH1 field.
	PWM_IRQ1_INTE_CH1_Pos = 0x1
	// Bit mask of CH1 field.
	PWM_IRQ1_INTE_CH1_Msk = 0x2
	// Bit CH1.
	PWM_IRQ1_INTE_CH1 = 0x2
	// Position of CH0 field.
	PWM_IRQ1_INTE_CH0_Pos = 0x0
	// Bit mask of CH0 field.
	PWM_IRQ1_INTE_CH0_Msk = 0x1
	// Bit CH0.
	PWM_IRQ1_INTE_CH0 = 0x1

	// IRQ1_INTF: Interrupt Force for irq1
	// Position of CH11 field.
	PWM_IRQ1_INTF_CH11_Pos = 0xb
	// Bit mask of CH11 field.
	PWM_IRQ1_INTF_CH11_Msk = 0x800
	// Bit CH11.
	PWM_IRQ1_INTF_CH11 = 0x800
	// Position of CH10 field.
	PWM_IRQ1_INTF_CH10_Pos = 0xa
	// Bit mask of CH10 field.
	PWM_IRQ1_INTF_CH10_Msk = 0x400
	// Bit CH10.
	PWM_IRQ1_INTF_CH10 = 0x400
	// Position of CH9 field.
	PWM_IRQ1_INTF_CH9_Pos = 0x9
	// Bit mask of CH9 field.
	PWM_IRQ1_INTF_CH9_Msk = 0x200
	// Bit CH9.
	PWM_IRQ1_INTF_CH9 = 0x200
	// Position of CH8 field.
	PWM_IRQ1_INTF_CH8_Pos = 0x8
	// Bit mask of CH8 field.
	PWM_IRQ1_INTF_CH8_Msk = 0x100
	// Bit CH8.
	PWM_IRQ1_INTF_CH8 = 0x100
	// Position of CH7 field.
	PWM_IRQ1_INTF_CH7_Pos = 0x7
	// Bit mask of CH7 field.
	PWM_IRQ1_INTF_CH7_Msk = 0x80
	// Bit CH7.
	PWM_IRQ1_INTF_CH7 = 0x80
	// Position of CH6 field.
	PWM_IRQ1_INTF_CH6_Pos = 0x6
	// Bit mask of CH6 field.
	PWM_IRQ1_INTF_CH6_Msk = 0x40
	// Bit CH6.
	PWM_IRQ1_INTF_CH6 = 0x40
	// Position of CH5 field.
	PWM_IRQ1_INTF_CH5_Pos = 0x5
	// Bit mask of CH5 field.
	PWM_IRQ1_INTF_CH5_Msk = 0x20
	// Bit CH5.
	PWM_IRQ1_INTF_CH5 = 0x20
	// Position of CH4 field.
	PWM_IRQ1_INTF_CH4_Pos = 0x4
	// Bit mask of CH4 field.
	PWM_IRQ1_INTF_CH4_Msk = 0x10
	// Bit CH4.
	PWM_IRQ1_INTF_CH4 = 0x10
	// Position of CH3 field.
	PWM_IRQ1_INTF_CH3_Pos = 0x3
	// Bit mask of CH3 field.
	PWM_IRQ1_INTF_CH3_Msk = 0x8
	// Bit CH3.
	PWM_IRQ1_INTF_CH3 = 0x8
	// Position of CH2 field.
	PWM_IRQ1_INTF_CH2_Pos = 0x2
	// Bit mask of CH2 field.
	PWM_IRQ1_INTF_CH2_Msk = 0x4
	// Bit CH2.
	PWM_IRQ1_INTF_CH2 = 0x4
	// Position of CH1 field.
	PWM_IRQ1_INTF_CH1_Pos = 0x1
	// Bit mask of CH1 field.
	PWM_IRQ1_INTF_CH1_Msk = 0x2
	// Bit CH1.
	PWM_IRQ1_INTF_CH1 = 0x2
	// Position of CH0 field.
	PWM_IRQ1_INTF_CH0_Pos = 0x0
	// Bit mask of CH0 field.
	PWM_IRQ1_INTF_CH0_Msk = 0x1
	// Bit CH0.
	PWM_IRQ1_INTF_CH0 = 0x1

	// IRQ1_INTS: Interrupt status after masking & forcing for irq1
	// Position of CH11 field.
	PWM_IRQ1_INTS_CH11_Pos = 0xb
	// Bit mask of CH11 field.
	PWM_IRQ1_INTS_CH11_Msk = 0x800
	// Bit CH11.
	PWM_IRQ1_INTS_CH11 = 0x800
	// Position of CH10 field.
	PWM_IRQ1_INTS_CH10_Pos = 0xa
	// Bit mask of CH10 field.
	PWM_IRQ1_INTS_CH10_Msk = 0x400
	// Bit CH10.
	PWM_IRQ1_INTS_CH10 = 0x400
	// Position of CH9 field.
	PWM_IRQ1_INTS_CH9_Pos = 0x9
	// Bit mask of CH9 field.
	PWM_IRQ1_INTS_CH9_Msk = 0x200
	// Bit CH9.
	PWM_IRQ1_INTS_CH9 = 0x200
	// Position of CH8 field.
	PWM_IRQ1_INTS_CH8_Pos = 0x8
	// Bit mask of CH8 field.
	PWM_IRQ1_INTS_CH8_Msk = 0x100
	// Bit CH8.
	PWM_IRQ1_INTS_CH8 = 0x100
	// Position of CH7 field.
	PWM_IRQ1_INTS_CH7_Pos = 0x7
	// Bit mask of CH7 field.
	PWM_IRQ1_INTS_CH7_Msk = 0x80
	// Bit CH7.
	PWM_IRQ1_INTS_CH7 = 0x80
	// Position of CH6 field.
	PWM_IRQ1_INTS_CH6_Pos = 0x6
	// Bit mask of CH6 field.
	PWM_IRQ1_INTS_CH6_Msk = 0x40
	// Bit CH6.
	PWM_IRQ1_INTS_CH6 = 0x40
	// Position of CH5 field.
	PWM_IRQ1_INTS_CH5_Pos = 0x5
	// Bit mask of CH5 field.
	PWM_IRQ1_INTS_CH5_Msk = 0x20
	// Bit CH5.
	PWM_IRQ1_INTS_CH5 = 0x20
	// Position of CH4 field.
	PWM_IRQ1_INTS_CH4_Pos = 0x4
	// Bit mask of CH4 field.
	PWM_IRQ1_INTS_CH4_Msk = 0x10
	// Bit CH4.
	PWM_IRQ1_INTS_CH4 = 0x10
	// Position of CH3 field.
	PWM_IRQ1_INTS_CH3_Pos = 0x3
	// Bit mask of CH3 field.
	PWM_IRQ1_INTS_CH3_Msk = 0x8
	// Bit CH3.
	PWM_IRQ1_INTS_CH3 = 0x8
	// Position of CH2 field.
	PWM_IRQ1_INTS_CH2_Pos = 0x2
	// Bit mask of CH2 field.
	PWM_IRQ1_INTS_CH2_Msk = 0x4
	// Bit CH2.
	PWM_IRQ1_INTS_CH2 = 0x4
	// Position of CH1 field.
	PWM_IRQ1_INTS_CH1_Pos = 0x1
	// Bit mask of CH1 field.
	PWM_IRQ1_INTS_CH1_Msk = 0x2
	// Bit CH1.
	PWM_IRQ1_INTS_CH1 = 0x2
	// Position of CH0 field.
	PWM_IRQ1_INTS_CH0_Pos = 0x0
	// Bit mask of CH0 field.
	PWM_IRQ1_INTS_CH0_Msk = 0x1
	// Bit CH0.
	PWM_IRQ1_INTS_CH0 = 0x1
)

// Constants for ADC: Control and data interface to SAR ADC
const (
	// CS: ADC Control and Status
	// Position of RROBIN field.
	ADC_CS_RROBIN_Pos = 0x10
	// Bit mask of RROBIN field.
	ADC_CS_RROBIN_Msk = 0x1ff0000
	// Position of AINSEL field.
	ADC_CS_AINSEL_Pos = 0xc
	// Bit mask of AINSEL field.
	ADC_CS_AINSEL_Msk = 0xf000
	// Position of ERR_STICKY field.
	ADC_CS_ERR_STICKY_Pos = 0xa
	// Bit mask of ERR_STICKY field.
	ADC_CS_ERR_STICKY_Msk = 0x400
	// Bit ERR_STICKY.
	ADC_CS_ERR_STICKY = 0x400
	// Position of ERR field.
	ADC_CS_ERR_Pos = 0x9
	// Bit mask of ERR field.
	ADC_CS_ERR_Msk = 0x200
	// Bit ERR.
	ADC_CS_ERR = 0x200
	// Position of READY field.
	ADC_CS_READY_Pos = 0x8
	// Bit mask of READY field.
	ADC_CS_READY_Msk = 0x100
	// Bit READY.
	ADC_CS_READY = 0x100
	// Position of START_MANY field.
	ADC_CS_START_MANY_Pos = 0x3
	// Bit mask of START_MANY field.
	ADC_CS_START_MANY_Msk = 0x8
	// Bit START_MANY.
	ADC_CS_START_MANY = 0x8
	// Position of START_ONCE field.
	ADC_CS_START_ONCE_Pos = 0x2
	// Bit mask of START_ONCE field.
	ADC_CS_START_ONCE_Msk = 0x4
	// Bit START_ONCE.
	ADC_CS_START_ONCE = 0x4
	// Position of TS_EN field.
	ADC_CS_TS_EN_Pos = 0x1
	// Bit mask of TS_EN field.
	ADC_CS_TS_EN_Msk = 0x2
	// Bit TS_EN.
	ADC_CS_TS_EN = 0x2
	// Position of EN field.
	ADC_CS_EN_Pos = 0x0
	// Bit mask of EN field.
	ADC_CS_EN_Msk = 0x1
	// Bit EN.
	ADC_CS_EN = 0x1

	// RESULT: Result of most recent ADC conversion
	// Position of RESULT field.
	ADC_RESULT_RESULT_Pos = 0x0
	// Bit mask of RESULT field.
	ADC_RESULT_RESULT_Msk = 0xfff

	// FCS: FIFO control and status
	// Position of THRESH field.
	ADC_FCS_THRESH_Pos = 0x18
	// Bit mask of THRESH field.
	ADC_FCS_THRESH_Msk = 0xf000000
	// Position of LEVEL field.
	ADC_FCS_LEVEL_Pos = 0x10
	// Bit mask of LEVEL field.
	ADC_FCS_LEVEL_Msk = 0xf0000
	// Position of OVER field.
	ADC_FCS_OVER_Pos = 0xb
	// Bit mask of OVER field.
	ADC_FCS_OVER_Msk = 0x800
	// Bit OVER.
	ADC_FCS_OVER = 0x800
	// Position of UNDER field.
	ADC_FCS_UNDER_Pos = 0xa
	// Bit mask of UNDER field.
	ADC_FCS_UNDER_Msk = 0x400
	// Bit UNDER.
	ADC_FCS_UNDER = 0x400
	// Position of FULL field.
	ADC_FCS_FULL_Pos = 0x9
	// Bit mask of FULL field.
	ADC_FCS_FULL_Msk = 0x200
	// Bit FULL.
	ADC_FCS_FULL = 0x200
	// Position of EMPTY field.
	ADC_FCS_EMPTY_Pos = 0x8
	// Bit mask of EMPTY field.
	ADC_FCS_EMPTY_Msk = 0x100
	// Bit EMPTY.
	ADC_FCS_EMPTY = 0x100
	// Position of DREQ_EN field.
	ADC_FCS_DREQ_EN_Pos = 0x3
	// Bit mask of DREQ_EN field.
	ADC_FCS_DREQ_EN_Msk = 0x8
	// Bit DREQ_EN.
	ADC_FCS_DREQ_EN = 0x8
	// Position of ERR field.
	ADC_FCS_ERR_Pos = 0x2
	// Bit mask of ERR field.
	ADC_FCS_ERR_Msk = 0x4
	// Bit ERR.
	ADC_FCS_ERR = 0x4
	// Position of SHIFT field.
	ADC_FCS_SHIFT_Pos = 0x1
	// Bit mask of SHIFT field.
	ADC_FCS_SHIFT_Msk = 0x2
	// Bit SHIFT.
	ADC_FCS_SHIFT = 0x2
	// Position of EN field.
	ADC_FCS_EN_Pos = 0x0
	// Bit mask of EN field.
	ADC_FCS_EN_Msk = 0x1
	// Bit EN.
	ADC_FCS_EN = 0x1

	// FIFO: Conversion result FIFO
	// Position of ERR field.
	ADC_FIFO_ERR_Pos = 0xf
	// Bit mask of ERR field.
	ADC_FIFO_ERR_Msk = 0x8000
	// Bit ERR.
	ADC_FIFO_ERR = 0x8000
	// Position of VAL field.
	ADC_FIFO_VAL_Pos = 0x0
	// Bit mask of VAL field.
	ADC_FIFO_VAL_Msk = 0xfff

	// DIV: Clock divider. If non-zero, CS_START_MANY will start conversions at regular intervals rather than back-to-back. The divider is reset when either of these fields are written. Total period is 1 + INT + FRAC / 256
	// Position of INT field.
	ADC_DIV_INT_Pos = 0x8
	// Bit mask of INT field.
	ADC_DIV_INT_Msk = 0xffff00
	// Position of FRAC field.
	ADC_DIV_FRAC_Pos = 0x0
	// Bit mask of FRAC field.
	ADC_DIV_FRAC_Msk = 0xff

	// INTR: Raw Interrupts
	// Position of FIFO field.
	ADC_INTR_FIFO_Pos = 0x0
	// Bit mask of FIFO field.
	ADC_INTR_FIFO_Msk = 0x1
	// Bit FIFO.
	ADC_INTR_FIFO = 0x1

	// INTE: Interrupt Enable
	// Position of FIFO field.
	ADC_INTE_FIFO_Pos = 0x0
	// Bit mask of FIFO field.
	ADC_INTE_FIFO_Msk = 0x1
	// Bit FIFO.
	ADC_INTE_FIFO = 0x1

	// INTF: Interrupt Force
	// Position of FIFO field.
	ADC_INTF_FIFO_Pos = 0x0
	// Bit mask of FIFO field.
	ADC_INTF_FIFO_Msk = 0x1
	// Bit FIFO.
	ADC_INTF_FIFO = 0x1

	// INTS: Interrupt status after masking & forcing
	// Position of FIFO field.
	ADC_INTS_FIFO_Pos = 0x0
	// Bit mask of FIFO field.
	ADC_INTS_FIFO_Msk = 0x1
	// Bit FIFO.
	ADC_INTS_FIFO = 0x1
)

// Constants for I2C0: DW_apb_i2c address block List of configuration constants for the Synopsys I2C hardware (you may see references to these in I2C register header; these are *fixed* values, set at hardware design time): IC_ULTRA_FAST_MODE ................ 0x0 IC_UFM_TBUF_CNT_DEFAULT ........... 0x8 IC_UFM_SCL_LOW_COUNT .............. 0x0008 IC_UFM_SCL_HIGH_COUNT ............. 0x0006 IC_TX_TL .......................... 0x0 IC_TX_CMD_BLOCK ................... 0x1 IC_HAS_DMA ........................ 0x1 IC_HAS_ASYNC_FIFO ................. 0x0 IC_SMBUS_ARP ...................... 0x0 IC_FIRST_DATA_BYTE_STATUS ......... 0x1 IC_INTR_IO ........................ 0x1 IC_MASTER_MODE .................... 0x1 IC_DEFAULT_ACK_GENERAL_CALL ....... 0x1 IC_INTR_POL ....................... 0x1 IC_OPTIONAL_SAR ................... 0x0 IC_DEFAULT_TAR_SLAVE_ADDR ......... 0x055 IC_DEFAULT_SLAVE_ADDR ............. 0x055 IC_DEFAULT_HS_SPKLEN .............. 0x1 IC_FS_SCL_HIGH_COUNT .............. 0x0006 IC_HS_SCL_LOW_COUNT ............... 0x0008 IC_DEVICE_ID_VALUE ................ 0x0 IC_10BITADDR_MASTER ............... 0x0 IC_CLK_FREQ_OPTIMIZATION .......... 0x0 IC_DEFAULT_FS_SPKLEN .............. 0x7 IC_ADD_ENCODED_PARAMS ............. 0x0 IC_DEFAULT_SDA_HOLD ............... 0x000001 IC_DEFAULT_SDA_SETUP .............. 0x64 IC_AVOID_RX_FIFO_FLUSH_ON_TX_ABRT . 0x0 IC_CLOCK_PERIOD ................... 100 IC_EMPTYFIFO_HOLD_MASTER_EN ....... 1 IC_RESTART_EN ..................... 0x1 IC_TX_CMD_BLOCK_DEFAULT ........... 0x0 IC_BUS_CLEAR_FEATURE .............. 0x0 IC_CAP_LOADING .................... 100 IC_FS_SCL_LOW_COUNT ............... 0x000d APB_DATA_WIDTH .................... 32 IC_SDA_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_SLV_DATA_NACK_ONLY ............. 0x1 IC_10BITADDR_SLAVE ................ 0x0 IC_CLK_TYPE ....................... 0x0 IC_SMBUS_UDID_MSB ................. 0x0 IC_SMBUS_SUSPEND_ALERT ............ 0x0 IC_HS_SCL_HIGH_COUNT .............. 0x0006 IC_SLV_RESTART_DET_EN ............. 0x1 IC_SMBUS .......................... 0x0 IC_OPTIONAL_SAR_DEFAULT ........... 0x0 IC_PERSISTANT_SLV_ADDR_DEFAULT .... 0x0 IC_USE_COUNTS ..................... 0x0 IC_RX_BUFFER_DEPTH ................ 16 IC_SCL_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff IC_RX_FULL_HLD_BUS_EN ............. 0x1 IC_SLAVE_DISABLE .................. 0x1 IC_RX_TL .......................... 0x0 IC_DEVICE_ID ...................... 0x0 IC_HC_COUNT_VALUES ................ 0x0 I2C_DYNAMIC_TAR_UPDATE ............ 0 IC_SMBUS_CLK_LOW_MEXT_DEFAULT ..... 0xffffffff IC_SMBUS_CLK_LOW_SEXT_DEFAULT ..... 0xffffffff IC_HS_MASTER_CODE ................. 0x1 IC_SMBUS_RST_IDLE_CNT_DEFAULT ..... 0xffff IC_SMBUS_UDID_LSB_DEFAULT ......... 0xffffffff IC_SS_SCL_HIGH_COUNT .............. 0x0028 IC_SS_SCL_LOW_COUNT ............... 0x002f IC_MAX_SPEED_MODE ................. 0x2 IC_STAT_FOR_CLK_STRETCH ........... 0x0 IC_STOP_DET_IF_MASTER_ACTIVE ...... 0x0 IC_DEFAULT_UFM_SPKLEN ............. 0x1 IC_TX_BUFFER_DEPTH ................ 16
const (
	// IC_CON: I2C Control Register. This register can be written only when the DW_apb_i2c is disabled, which corresponds to the IC_ENABLE[0] register being set to 0. Writes at other times have no effect. Read/Write Access: - bit 10 is read only. - bit 11 is read only - bit 16 is read only - bit 17 is read only - bits 18 and 19 are read only.
	// Position of STOP_DET_IF_MASTER_ACTIVE field.
	I2C0_IC_CON_STOP_DET_IF_MASTER_ACTIVE_Pos = 0xa
	// Bit mask of STOP_DET_IF_MASTER_ACTIVE field.
	I2C0_IC_CON_STOP_DET_IF_MASTER_ACTIVE_Msk = 0x400
	// Bit STOP_DET_IF_MASTER_ACTIVE.
	I2C0_IC_CON_STOP_DET_IF_MASTER_ACTIVE = 0x400
	// Position of RX_FIFO_FULL_HLD_CTRL field.
	I2C0_IC_CON_RX_FIFO_FULL_HLD_CTRL_Pos = 0x9
	// Bit mask of RX_FIFO_FULL_HLD_CTRL field.
	I2C0_IC_CON_RX_FIFO_FULL_HLD_CTRL_Msk = 0x200
	// Bit RX_FIFO_FULL_HLD_CTRL.
	I2C0_IC_CON_RX_FIFO_FULL_HLD_CTRL = 0x200
	// Overflow when RX_FIFO is full
	I2C0_IC_CON_RX_FIFO_FULL_HLD_CTRL_DISABLED = 0x0
	// Hold bus when RX_FIFO is full
	I2C0_IC_CON_RX_FIFO_FULL_HLD_CTRL_ENABLED = 0x1
	// Position of TX_EMPTY_CTRL field.
	I2C0_IC_CON_TX_EMPTY_CTRL_Pos = 0x8
	// Bit mask of TX_EMPTY_CTRL field.
	I2C0_IC_CON_TX_EMPTY_CTRL_Msk = 0x100
	// Bit TX_EMPTY_CTRL.
	I2C0_IC_CON_TX_EMPTY_CTRL = 0x100
	// Default behaviour of TX_EMPTY interrupt
	I2C0_IC_CON_TX_EMPTY_CTRL_DISABLED = 0x0
	// Controlled generation of TX_EMPTY interrupt
	I2C0_IC_CON_TX_EMPTY_CTRL_ENABLED = 0x1
	// Position of STOP_DET_IFADDRESSED field.
	I2C0_IC_CON_STOP_DET_IFADDRESSED_Pos = 0x7
	// Bit mask of STOP_DET_IFADDRESSED field.
	I2C0_IC_CON_STOP_DET_IFADDRESSED_Msk = 0x80
	// Bit STOP_DET_IFADDRESSED.
	I2C0_IC_CON_STOP_DET_IFADDRESSED = 0x80
	// slave issues STOP_DET intr always
	I2C0_IC_CON_STOP_DET_IFADDRESSED_DISABLED = 0x0
	// slave issues STOP_DET intr only if addressed
	I2C0_IC_CON_STOP_DET_IFADDRESSED_ENABLED = 0x1
	// Position of IC_SLAVE_DISABLE field.
	I2C0_IC_CON_IC_SLAVE_DISABLE_Pos = 0x6
	// Bit mask of IC_SLAVE_DISABLE field.
	I2C0_IC_CON_IC_SLAVE_DISABLE_Msk = 0x40
	// Bit IC_SLAVE_DISABLE.
	I2C0_IC_CON_IC_SLAVE_DISABLE = 0x40
	// Slave mode is enabled
	I2C0_IC_CON_IC_SLAVE_DISABLE_SLAVE_ENABLED = 0x0
	// Slave mode is disabled
	I2C0_IC_CON_IC_SLAVE_DISABLE_SLAVE_DISABLED = 0x1
	// Position of IC_RESTART_EN field.
	I2C0_IC_CON_IC_RESTART_EN_Pos = 0x5
	// Bit mask of IC_RESTART_EN field.
	I2C0_IC_CON_IC_RESTART_EN_Msk = 0x20
	// Bit IC_RESTART_EN.
	I2C0_IC_CON_IC_RESTART_EN = 0x20
	// Master restart disabled
	I2C0_IC_CON_IC_RESTART_EN_DISABLED = 0x0
	// Master restart enabled
	I2C0_IC_CON_IC_RESTART_EN_ENABLED = 0x1
	// Position of IC_10BITADDR_MASTER field.
	I2C0_IC_CON_IC_10BITADDR_MASTER_Pos = 0x4
	// Bit mask of IC_10BITADDR_MASTER field.
	I2C0_IC_CON_IC_10BITADDR_MASTER_Msk = 0x10
	// Bit IC_10BITADDR_MASTER.
	I2C0_IC_CON_IC_10BITADDR_MASTER = 0x10
	// Master 7Bit addressing mode
	I2C0_IC_CON_IC_10BITADDR_MASTER_ADDR_7BITS = 0x0
	// Master 10Bit addressing mode
	I2C0_IC_CON_IC_10BITADDR_MASTER_ADDR_10BITS = 0x1
	// Position of IC_10BITADDR_SLAVE field.
	I2C0_IC_CON_IC_10BITADDR_SLAVE_Pos = 0x3
	// Bit mask of IC_10BITADDR_SLAVE field.
	I2C0_IC_CON_IC_10BITADDR_SLAVE_Msk = 0x8
	// Bit IC_10BITADDR_SLAVE.
	I2C0_IC_CON_IC_10BITADDR_SLAVE = 0x8
	// Slave 7Bit addressing
	I2C0_IC_CON_IC_10BITADDR_SLAVE_ADDR_7BITS = 0x0
	// Slave 10Bit addressing
	I2C0_IC_CON_IC_10BITADDR_SLAVE_ADDR_10BITS = 0x1
	// Position of SPEED field.
	I2C0_IC_CON_SPEED_Pos = 0x1
	// Bit mask of SPEED field.
	I2C0_IC_CON_SPEED_Msk = 0x6
	// Standard Speed mode of operation
	I2C0_IC_CON_SPEED_STANDARD = 0x1
	// Fast or Fast Plus mode of operation
	I2C0_IC_CON_SPEED_FAST = 0x2
	// High Speed mode of operation
	I2C0_IC_CON_SPEED_HIGH = 0x3
	// Position of MASTER_MODE field.
	I2C0_IC_CON_MASTER_MODE_Pos = 0x0
	// Bit mask of MASTER_MODE field.
	I2C0_IC_CON_MASTER_MODE_Msk = 0x1
	// Bit MASTER_MODE.
	I2C0_IC_CON_MASTER_MODE = 0x1
	// Master mode is disabled
	I2C0_IC_CON_MASTER_MODE_DISABLED = 0x0
	// Master mode is enabled
	I2C0_IC_CON_MASTER_MODE_ENABLED = 0x1

	// IC_TAR: I2C Target Address Register This register is 12 bits wide, and bits 31:12 are reserved. This register can be written to only when IC_ENABLE[0] is set to 0. Note: If the software or application is aware that the DW_apb_i2c is not using the TAR address for the pending commands in the Tx FIFO, then it is possible to update the TAR address even while the Tx FIFO has entries (IC_STATUS[2]= 0). - It is not necessary to perform any write to this register if DW_apb_i2c is enabled as an I2C slave only.
	// Position of SPECIAL field.
	I2C0_IC_TAR_SPECIAL_Pos = 0xb
	// Bit mask of SPECIAL field.
	I2C0_IC_TAR_SPECIAL_Msk = 0x800
	// Bit SPECIAL.
	I2C0_IC_TAR_SPECIAL = 0x800
	// Disables programming of GENERAL_CALL or START_BYTE transmission
	I2C0_IC_TAR_SPECIAL_DISABLED = 0x0
	// Enables programming of GENERAL_CALL or START_BYTE transmission
	I2C0_IC_TAR_SPECIAL_ENABLED = 0x1
	// Position of GC_OR_START field.
	I2C0_IC_TAR_GC_OR_START_Pos = 0xa
	// Bit mask of GC_OR_START field.
	I2C0_IC_TAR_GC_OR_START_Msk = 0x400
	// Bit GC_OR_START.
	I2C0_IC_TAR_GC_OR_START = 0x400
	// GENERAL_CALL byte transmission
	I2C0_IC_TAR_GC_OR_START_GENERAL_CALL = 0x0
	// START byte transmission
	I2C0_IC_TAR_GC_OR_START_START_BYTE = 0x1
	// Position of IC_TAR field.
	I2C0_IC_TAR_IC_TAR_Pos = 0x0
	// Bit mask of IC_TAR field.
	I2C0_IC_TAR_IC_TAR_Msk = 0x3ff

	// IC_SAR: I2C Slave Address Register
	// Position of IC_SAR field.
	I2C0_IC_SAR_IC_SAR_Pos = 0x0
	// Bit mask of IC_SAR field.
	I2C0_IC_SAR_IC_SAR_Msk = 0x3ff

	// IC_DATA_CMD: I2C Rx/Tx Data Buffer and Command Register; this is the register the CPU writes to when filling the TX FIFO and the CPU reads from when retrieving bytes from RX FIFO. The size of the register changes as follows: Write: - 11 bits when IC_EMPTYFIFO_HOLD_MASTER_EN=1 - 9 bits when IC_EMPTYFIFO_HOLD_MASTER_EN=0 Read: - 12 bits when IC_FIRST_DATA_BYTE_STATUS = 1 - 8 bits when IC_FIRST_DATA_BYTE_STATUS = 0 Note: In order for the DW_apb_i2c to continue acknowledging reads, a read command should be written for every byte that is to be received; otherwise the DW_apb_i2c will stop acknowledging.
	// Position of FIRST_DATA_BYTE field.
	I2C0_IC_DATA_CMD_FIRST_DATA_BYTE_Pos = 0xb
	// Bit mask of FIRST_DATA_BYTE field.
	I2C0_IC_DATA_CMD_FIRST_DATA_BYTE_Msk = 0x800
	// Bit FIRST_DATA_BYTE.
	I2C0_IC_DATA_CMD_FIRST_DATA_BYTE = 0x800
	// Sequential data byte received
	I2C0_IC_DATA_CMD_FIRST_DATA_BYTE_INACTIVE = 0x0
	// Non sequential data byte received
	I2C0_IC_DATA_CMD_FIRST_DATA_BYTE_ACTIVE = 0x1
	// Position of RESTART field.
	I2C0_IC_DATA_CMD_RESTART_Pos = 0xa
	// Bit mask of RESTART field.
	I2C0_IC_DATA_CMD_RESTART_Msk = 0x400
	// Bit RESTART.
	I2C0_IC_DATA_CMD_RESTART = 0x400
	// Don't Issue RESTART before this command
	I2C0_IC_DATA_CMD_RESTART_DISABLE = 0x0
	// Issue RESTART before this command
	I2C0_IC_DATA_CMD_RESTART_ENABLE = 0x1
	// Position of STOP field.
	I2C0_IC_DATA_CMD_STOP_Pos = 0x9
	// Bit mask of STOP field.
	I2C0_IC_DATA_CMD_STOP_Msk = 0x200
	// Bit STOP.
	I2C0_IC_DATA_CMD_STOP = 0x200
	// Don't Issue STOP after this command
	I2C0_IC_DATA_CMD_STOP_DISABLE = 0x0
	// Issue STOP after this command
	I2C0_IC_DATA_CMD_STOP_ENABLE = 0x1
	// Position of CMD field.
	I2C0_IC_DATA_CMD_CMD_Pos = 0x8
	// Bit mask of CMD field.
	I2C0_IC_DATA_CMD_CMD_Msk = 0x100
	// Bit CMD.
	I2C0_IC_DATA_CMD_CMD = 0x100
	// Master Write Command
	I2C0_IC_DATA_CMD_CMD_WRITE = 0x0
	// Master Read Command
	I2C0_IC_DATA_CMD_CMD_READ = 0x1
	// Position of DAT field.
	I2C0_IC_DATA_CMD_DAT_Pos = 0x0
	// Bit mask of DAT field.
	I2C0_IC_DATA_CMD_DAT_Msk = 0xff

	// IC_SS_SCL_HCNT: Standard Speed I2C Clock SCL High Count Register
	// Position of IC_SS_SCL_HCNT field.
	I2C0_IC_SS_SCL_HCNT_IC_SS_SCL_HCNT_Pos = 0x0
	// Bit mask of IC_SS_SCL_HCNT field.
	I2C0_IC_SS_SCL_HCNT_IC_SS_SCL_HCNT_Msk = 0xffff

	// IC_SS_SCL_LCNT: Standard Speed I2C Clock SCL Low Count Register
	// Position of IC_SS_SCL_LCNT field.
	I2C0_IC_SS_SCL_LCNT_IC_SS_SCL_LCNT_Pos = 0x0
	// Bit mask of IC_SS_SCL_LCNT field.
	I2C0_IC_SS_SCL_LCNT_IC_SS_SCL_LCNT_Msk = 0xffff

	// IC_FS_SCL_HCNT: Fast Mode or Fast Mode Plus I2C Clock SCL High Count Register
	// Position of IC_FS_SCL_HCNT field.
	I2C0_IC_FS_SCL_HCNT_IC_FS_SCL_HCNT_Pos = 0x0
	// Bit mask of IC_FS_SCL_HCNT field.
	I2C0_IC_FS_SCL_HCNT_IC_FS_SCL_HCNT_Msk = 0xffff

	// IC_FS_SCL_LCNT: Fast Mode or Fast Mode Plus I2C Clock SCL Low Count Register
	// Position of IC_FS_SCL_LCNT field.
	I2C0_IC_FS_SCL_LCNT_IC_FS_SCL_LCNT_Pos = 0x0
	// Bit mask of IC_FS_SCL_LCNT field.
	I2C0_IC_FS_SCL_LCNT_IC_FS_SCL_LCNT_Msk = 0xffff

	// IC_INTR_STAT: I2C Interrupt Status Register Each bit in this register has a corresponding mask bit in the IC_INTR_MASK register. These bits are cleared by reading the matching interrupt clear register. The unmasked raw versions of these bits are available in the IC_RAW_INTR_STAT register.
	// Position of R_RESTART_DET field.
	I2C0_IC_INTR_STAT_R_RESTART_DET_Pos = 0xc
	// Bit mask of R_RESTART_DET field.
	I2C0_IC_INTR_STAT_R_RESTART_DET_Msk = 0x1000
	// Bit R_RESTART_DET.
	I2C0_IC_INTR_STAT_R_RESTART_DET = 0x1000
	// R_RESTART_DET interrupt is inactive
	I2C0_IC_INTR_STAT_R_RESTART_DET_INACTIVE = 0x0
	// R_RESTART_DET interrupt is active
	I2C0_IC_INTR_STAT_R_RESTART_DET_ACTIVE = 0x1
	// Position of R_GEN_CALL field.
	I2C0_IC_INTR_STAT_R_GEN_CALL_Pos = 0xb
	// Bit mask of R_GEN_CALL field.
	I2C0_IC_INTR_STAT_R_GEN_CALL_Msk = 0x800
	// Bit R_GEN_CALL.
	I2C0_IC_INTR_STAT_R_GEN_CALL = 0x800
	// R_GEN_CALL interrupt is inactive
	I2C0_IC_INTR_STAT_R_GEN_CALL_INACTIVE = 0x0
	// R_GEN_CALL interrupt is active
	I2C0_IC_INTR_STAT_R_GEN_CALL_ACTIVE = 0x1
	// Position of R_START_DET field.
	I2C0_IC_INTR_STAT_R_START_DET_Pos = 0xa
	// Bit mask of R_START_DET field.
	I2C0_IC_INTR_STAT_R_START_DET_Msk = 0x400
	// Bit R_START_DET.
	I2C0_IC_INTR_STAT_R_START_DET = 0x400
	// R_START_DET interrupt is inactive
	I2C0_IC_INTR_STAT_R_START_DET_INACTIVE = 0x0
	// R_START_DET interrupt is active
	I2C0_IC_INTR_STAT_R_START_DET_ACTIVE = 0x1
	// Position of R_STOP_DET field.
	I2C0_IC_INTR_STAT_R_STOP_DET_Pos = 0x9
	// Bit mask of R_STOP_DET field.
	I2C0_IC_INTR_STAT_R_STOP_DET_Msk = 0x200
	// Bit R_STOP_DET.
	I2C0_IC_INTR_STAT_R_STOP_DET = 0x200
	// R_STOP_DET interrupt is inactive
	I2C0_IC_INTR_STAT_R_STOP_DET_INACTIVE = 0x0
	// R_STOP_DET interrupt is active
	I2C0_IC_INTR_STAT_R_STOP_DET_ACTIVE = 0x1
	// Position of R_ACTIVITY field.
	I2C0_IC_INTR_STAT_R_ACTIVITY_Pos = 0x8
	// Bit mask of R_ACTIVITY field.
	I2C0_IC_INTR_STAT_R_ACTIVITY_Msk = 0x100
	// Bit R_ACTIVITY.
	I2C0_IC_INTR_STAT_R_ACTIVITY = 0x100
	// R_ACTIVITY interrupt is inactive
	I2C0_IC_INTR_STAT_R_ACTIVITY_INACTIVE = 0x0
	// R_ACTIVITY interrupt is active
	I2C0_IC_INTR_STAT_R_ACTIVITY_ACTIVE = 0x1
	// Position of R_RX_DONE field.
	I2C0_IC_INTR_STAT_R_RX_DONE_Pos = 0x7
	// Bit mask of R_RX_DONE field.
	I2C0_IC_INTR_STAT_R_RX_DONE_Msk = 0x80
	// Bit R_RX_DONE.
	I2C0_IC_INTR_STAT_R_RX_DONE = 0x80
	// R_RX_DONE interrupt is inactive
	I2C0_IC_INTR_STAT_R_RX_DONE_INACTIVE = 0x0
	// R_RX_DONE interrupt is active
	I2C0_IC_INTR_STAT_R_RX_DONE_ACTIVE = 0x1
	// Position of R_TX_ABRT field.
	I2C0_IC_INTR_STAT_R_TX_ABRT_Pos = 0x6
	// Bit mask of R_TX_ABRT field.
	I2C0_IC_INTR_STAT_R_TX_ABRT_Msk = 0x40
	// Bit R_TX_ABRT.
	I2C0_IC_INTR_STAT_R_TX_ABRT = 0x40
	// R_TX_ABRT interrupt is inactive
	I2C0_IC_INTR_STAT_R_TX_ABRT_INACTIVE = 0x0
	// R_TX_ABRT interrupt is active
	I2C0_IC_INTR_STAT_R_TX_ABRT_ACTIVE = 0x1
	// Position of R_RD_REQ field.
	I2C0_IC_INTR_STAT_R_RD_REQ_Pos = 0x5
	// Bit mask of R_RD_REQ field.
	I2C0_IC_INTR_STAT_R_RD_REQ_Msk = 0x20
	// Bit R_RD_REQ.
	I2C0_IC_INTR_STAT_R_RD_REQ = 0x20
	// R_RD_REQ interrupt is inactive
	I2C0_IC_INTR_STAT_R_RD_REQ_INACTIVE = 0x0
	// R_RD_REQ interrupt is active
	I2C0_IC_INTR_STAT_R_RD_REQ_ACTIVE = 0x1
	// Position of R_TX_EMPTY field.
	I2C0_IC_INTR_STAT_R_TX_EMPTY_Pos = 0x4
	// Bit mask of R_TX_EMPTY field.
	I2C0_IC_INTR_STAT_R_TX_EMPTY_Msk = 0x10
	// Bit R_TX_EMPTY.
	I2C0_IC_INTR_STAT_R_TX_EMPTY = 0x10
	// R_TX_EMPTY interrupt is inactive
	I2C0_IC_INTR_STAT_R_TX_EMPTY_INACTIVE = 0x0
	// R_TX_EMPTY interrupt is active
	I2C0_IC_INTR_STAT_R_TX_EMPTY_ACTIVE = 0x1
	// Position of R_TX_OVER field.
	I2C0_IC_INTR_STAT_R_TX_OVER_Pos = 0x3
	// Bit mask of R_TX_OVER field.
	I2C0_IC_INTR_STAT_R_TX_OVER_Msk = 0x8
	// Bit R_TX_OVER.
	I2C0_IC_INTR_STAT_R_TX_OVER = 0x8
	// R_TX_OVER interrupt is inactive
	I2C0_IC_INTR_STAT_R_TX_OVER_INACTIVE = 0x0
	// R_TX_OVER interrupt is active
	I2C0_IC_INTR_STAT_R_TX_OVER_ACTIVE = 0x1
	// Position of R_RX_FULL field.
	I2C0_IC_INTR_STAT_R_RX_FULL_Pos = 0x2
	// Bit mask of R_RX_FULL field.
	I2C0_IC_INTR_STAT_R_RX_FULL_Msk = 0x4
	// Bit R_RX_FULL.
	I2C0_IC_INTR_STAT_R_RX_FULL = 0x4
	// R_RX_FULL interrupt is inactive
	I2C0_IC_INTR_STAT_R_RX_FULL_INACTIVE = 0x0
	// R_RX_FULL interrupt is active
	I2C0_IC_INTR_STAT_R_RX_FULL_ACTIVE = 0x1
	// Position of R_RX_OVER field.
	I2C0_IC_INTR_STAT_R_RX_OVER_Pos = 0x1
	// Bit mask of R_RX_OVER field.
	I2C0_IC_INTR_STAT_R_RX_OVER_Msk = 0x2
	// Bit R_RX_OVER.
	I2C0_IC_INTR_STAT_R_RX_OVER = 0x2
	// R_RX_OVER interrupt is inactive
	I2C0_IC_INTR_STAT_R_RX_OVER_INACTIVE = 0x0
	// R_RX_OVER interrupt is active
	I2C0_IC_INTR_STAT_R_RX_OVER_ACTIVE = 0x1
	// Position of R_RX_UNDER field.
	I2C0_IC_INTR_STAT_R_RX_UNDER_Pos = 0x0
	// Bit mask of R_RX_UNDER field.
	I2C0_IC_INTR_STAT_R_RX_UNDER_Msk = 0x1
	// Bit R_RX_UNDER.
	I2C0_IC_INTR_STAT_R_RX_UNDER = 0x1
	// RX_UNDER interrupt is inactive
	I2C0_IC_INTR_STAT_R_RX_UNDER_INACTIVE = 0x0
	// RX_UNDER interrupt is active
	I2C0_IC_INTR_STAT_R_RX_UNDER_ACTIVE = 0x1

	// IC_INTR_MASK: I2C Interrupt Mask Register. These bits mask their corresponding interrupt status bits. This register is active low; a value of 0 masks the interrupt, whereas a value of 1 unmasks the interrupt.
	// Position of M_RESTART_DET field.
	I2C0_IC_INTR_MASK_M_RESTART_DET_Pos = 0xc
	// Bit mask of M_RESTART_DET field.
	I2C0_IC_INTR_MASK_M_RESTART_DET_Msk = 0x1000
	// Bit M_RESTART_DET.
	I2C0_IC_INTR_MASK_M_RESTART_DET = 0x1000
	// RESTART_DET interrupt is masked
	I2C0_IC_INTR_MASK_M_RESTART_DET_ENABLED = 0x0
	// RESTART_DET interrupt is unmasked
	I2C0_IC_INTR_MASK_M_RESTART_DET_DISABLED = 0x1
	// Position of M_GEN_CALL field.
	I2C0_IC_INTR_MASK_M_GEN_CALL_Pos = 0xb
	// Bit mask of M_GEN_CALL field.
	I2C0_IC_INTR_MASK_M_GEN_CALL_Msk = 0x800
	// Bit M_GEN_CALL.
	I2C0_IC_INTR_MASK_M_GEN_CALL = 0x800
	// GEN_CALL interrupt is masked
	I2C0_IC_INTR_MASK_M_GEN_CALL_ENABLED = 0x0
	// GEN_CALL interrupt is unmasked
	I2C0_IC_INTR_MASK_M_GEN_CALL_DISABLED = 0x1
	// Position of M_START_DET field.
	I2C0_IC_INTR_MASK_M_START_DET_Pos = 0xa
	// Bit mask of M_START_DET field.
	I2C0_IC_INTR_MASK_M_START_DET_Msk = 0x400
	// Bit M_START_DET.
	I2C0_IC_INTR_MASK_M_START_DET = 0x400
	// START_DET interrupt is masked
	I2C0_IC_INTR_MASK_M_START_DET_ENABLED = 0x0
	// START_DET interrupt is unmasked
	I2C0_IC_INTR_MASK_M_START_DET_DISABLED = 0x1
	// Position of M_STOP_DET field.
	I2C0_IC_INTR_MASK_M_STOP_DET_Pos = 0x9
	// Bit mask of M_STOP_DET field.
	I2C0_IC_INTR_MASK_M_STOP_DET_Msk = 0x200
	// Bit M_STOP_DET.
	I2C0_IC_INTR_MASK_M_STOP_DET = 0x200
	// STOP_DET interrupt is masked
	I2C0_IC_INTR_MASK_M_STOP_DET_ENABLED = 0x0
	// STOP_DET interrupt is unmasked
	I2C0_IC_INTR_MASK_M_STOP_DET_DISABLED = 0x1
	// Position of M_ACTIVITY field.
	I2C0_IC_INTR_MASK_M_ACTIVITY_Pos = 0x8
	// Bit mask of M_ACTIVITY field.
	I2C0_IC_INTR_MASK_M_ACTIVITY_Msk = 0x100
	// Bit M_ACTIVITY.
	I2C0_IC_INTR_MASK_M_ACTIVITY = 0x100
	// ACTIVITY interrupt is masked
	I2C0_IC_INTR_MASK_M_ACTIVITY_ENABLED = 0x0
	// ACTIVITY interrupt is unmasked
	I2C0_IC_INTR_MASK_M_ACTIVITY_DISABLED = 0x1
	// Position of M_RX_DONE field.
	I2C0_IC_INTR_MASK_M_RX_DONE_Pos = 0x7
	// Bit mask of M_RX_DONE field.
	I2C0_IC_INTR_MASK_M_RX_DONE_Msk = 0x80
	// Bit M_RX_DONE.
	I2C0_IC_INTR_MASK_M_RX_DONE = 0x80
	// RX_DONE interrupt is masked
	I2C0_IC_INTR_MASK_M_RX_DONE_ENABLED = 0x0
	// RX_DONE interrupt is unmasked
	I2C0_IC_INTR_MASK_M_RX_DONE_DISABLED = 0x1
	// Position of M_TX_ABRT field.
	I2C0_IC_INTR_MASK_M_TX_ABRT_Pos = 0x6
	// Bit mask of M_TX_ABRT field.
	I2C0_IC_INTR_MASK_M_TX_ABRT_Msk = 0x40
	// Bit M_TX_ABRT.
	I2C0_IC_INTR_MASK_M_TX_ABRT = 0x40
	// TX_ABORT interrupt is masked
	I2C0_IC_INTR_MASK_M_TX_ABRT_ENABLED = 0x0
	// TX_ABORT interrupt is unmasked
	I2C0_IC_INTR_MASK_M_TX_ABRT_DISABLED = 0x1
	// Position of M_RD_REQ field.
	I2C0_IC_INTR_MASK_M_RD_REQ_Pos = 0x5
	// Bit mask of M_RD_REQ field.
	I2C0_IC_INTR_MASK_M_RD_REQ_Msk = 0x20
	// Bit M_RD_REQ.
	I2C0_IC_INTR_MASK_M_RD_REQ = 0x20
	// RD_REQ interrupt is masked
	I2C0_IC_INTR_MASK_M_RD_REQ_ENABLED = 0x0
	// RD_REQ interrupt is unmasked
	I2C0_IC_INTR_MASK_M_RD_REQ_DISABLED = 0x1
	// Position of M_TX_EMPTY field.
	I2C0_IC_INTR_MASK_M_TX_EMPTY_Pos = 0x4
	// Bit mask of M_TX_EMPTY field.
	I2C0_IC_INTR_MASK_M_TX_EMPTY_Msk = 0x10
	// Bit M_TX_EMPTY.
	I2C0_IC_INTR_MASK_M_TX_EMPTY = 0x10
	// TX_EMPTY interrupt is masked
	I2C0_IC_INTR_MASK_M_TX_EMPTY_ENABLED = 0x0
	// TX_EMPTY interrupt is unmasked
	I2C0_IC_INTR_MASK_M_TX_EMPTY_DISABLED = 0x1
	// Position of M_TX_OVER field.
	I2C0_IC_INTR_MASK_M_TX_OVER_Pos = 0x3
	// Bit mask of M_TX_OVER field.
	I2C0_IC_INTR_MASK_M_TX_OVER_Msk = 0x8
	// Bit M_TX_OVER.
	I2C0_IC_INTR_MASK_M_TX_OVER = 0x8
	// TX_OVER interrupt is masked
	I2C0_IC_INTR_MASK_M_TX_OVER_ENABLED = 0x0
	// TX_OVER interrupt is unmasked
	I2C0_IC_INTR_MASK_M_TX_OVER_DISABLED = 0x1
	// Position of M_RX_FULL field.
	I2C0_IC_INTR_MASK_M_RX_FULL_Pos = 0x2
	// Bit mask of M_RX_FULL field.
	I2C0_IC_INTR_MASK_M_RX_FULL_Msk = 0x4
	// Bit M_RX_FULL.
	I2C0_IC_INTR_MASK_M_RX_FULL = 0x4
	// RX_FULL interrupt is masked
	I2C0_IC_INTR_MASK_M_RX_FULL_ENABLED = 0x0
	// RX_FULL interrupt is unmasked
	I2C0_IC_INTR_MASK_M_RX_FULL_DISABLED = 0x1
	// Position of M_RX_OVER field.
	I2C0_IC_INTR_MASK_M_RX_OVER_Pos = 0x1
	// Bit mask of M_RX_OVER field.
	I2C0_IC_INTR_MASK_M_RX_OVER_Msk = 0x2
	// Bit M_RX_OVER.
	I2C0_IC_INTR_MASK_M_RX_OVER = 0x2
	// RX_OVER interrupt is masked
	I2C0_IC_INTR_MASK_M_RX_OVER_ENABLED = 0x0
	// RX_OVER interrupt is unmasked
	I2C0_IC_INTR_MASK_M_RX_OVER_DISABLED = 0x1
	// Position of M_RX_UNDER field.
	I2C0_IC_INTR_MASK_M_RX_UNDER_Pos = 0x0
	// Bit mask of M_RX_UNDER field.
	I2C0_IC_INTR_MASK_M_RX_UNDER_Msk = 0x1
	// Bit M_RX_UNDER.
	I2C0_IC_INTR_MASK_M_RX_UNDER = 0x1
	// RX_UNDER interrupt is masked
	I2C0_IC_INTR_MASK_M_RX_UNDER_ENABLED = 0x0
	// RX_UNDER interrupt is unmasked
	I2C0_IC_INTR_MASK_M_RX_UNDER_DISABLED = 0x1

	// IC_RAW_INTR_STAT: I2C Raw Interrupt Status Register Unlike the IC_INTR_STAT register, these bits are not masked so they always show the true status of the DW_apb_i2c.
	// Position of RESTART_DET field.
	I2C0_IC_RAW_INTR_STAT_RESTART_DET_Pos = 0xc
	// Bit mask of RESTART_DET field.
	I2C0_IC_RAW_INTR_STAT_RESTART_DET_Msk = 0x1000
	// Bit RESTART_DET.
	I2C0_IC_RAW_INTR_STAT_RESTART_DET = 0x1000
	// RESTART_DET interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_RESTART_DET_INACTIVE = 0x0
	// RESTART_DET interrupt is active
	I2C0_IC_RAW_INTR_STAT_RESTART_DET_ACTIVE = 0x1
	// Position of GEN_CALL field.
	I2C0_IC_RAW_INTR_STAT_GEN_CALL_Pos = 0xb
	// Bit mask of GEN_CALL field.
	I2C0_IC_RAW_INTR_STAT_GEN_CALL_Msk = 0x800
	// Bit GEN_CALL.
	I2C0_IC_RAW_INTR_STAT_GEN_CALL = 0x800
	// GEN_CALL interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_GEN_CALL_INACTIVE = 0x0
	// GEN_CALL interrupt is active
	I2C0_IC_RAW_INTR_STAT_GEN_CALL_ACTIVE = 0x1
	// Position of START_DET field.
	I2C0_IC_RAW_INTR_STAT_START_DET_Pos = 0xa
	// Bit mask of START_DET field.
	I2C0_IC_RAW_INTR_STAT_START_DET_Msk = 0x400
	// Bit START_DET.
	I2C0_IC_RAW_INTR_STAT_START_DET = 0x400
	// START_DET interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_START_DET_INACTIVE = 0x0
	// START_DET interrupt is active
	I2C0_IC_RAW_INTR_STAT_START_DET_ACTIVE = 0x1
	// Position of STOP_DET field.
	I2C0_IC_RAW_INTR_STAT_STOP_DET_Pos = 0x9
	// Bit mask of STOP_DET field.
	I2C0_IC_RAW_INTR_STAT_STOP_DET_Msk = 0x200
	// Bit STOP_DET.
	I2C0_IC_RAW_INTR_STAT_STOP_DET = 0x200
	// STOP_DET interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_STOP_DET_INACTIVE = 0x0
	// STOP_DET interrupt is active
	I2C0_IC_RAW_INTR_STAT_STOP_DET_ACTIVE = 0x1
	// Position of ACTIVITY field.
	I2C0_IC_RAW_INTR_STAT_ACTIVITY_Pos = 0x8
	// Bit mask of ACTIVITY field.
	I2C0_IC_RAW_INTR_STAT_ACTIVITY_Msk = 0x100
	// Bit ACTIVITY.
	I2C0_IC_RAW_INTR_STAT_ACTIVITY = 0x100
	// RAW_INTR_ACTIVITY interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_ACTIVITY_INACTIVE = 0x0
	// RAW_INTR_ACTIVITY interrupt is active
	I2C0_IC_RAW_INTR_STAT_ACTIVITY_ACTIVE = 0x1
	// Position of RX_DONE field.
	I2C0_IC_RAW_INTR_STAT_RX_DONE_Pos = 0x7
	// Bit mask of RX_DONE field.
	I2C0_IC_RAW_INTR_STAT_RX_DONE_Msk = 0x80
	// Bit RX_DONE.
	I2C0_IC_RAW_INTR_STAT_RX_DONE = 0x80
	// RX_DONE interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_RX_DONE_INACTIVE = 0x0
	// RX_DONE interrupt is active
	I2C0_IC_RAW_INTR_STAT_RX_DONE_ACTIVE = 0x1
	// Position of TX_ABRT field.
	I2C0_IC_RAW_INTR_STAT_TX_ABRT_Pos = 0x6
	// Bit mask of TX_ABRT field.
	I2C0_IC_RAW_INTR_STAT_TX_ABRT_Msk = 0x40
	// Bit TX_ABRT.
	I2C0_IC_RAW_INTR_STAT_TX_ABRT = 0x40
	// TX_ABRT interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_TX_ABRT_INACTIVE = 0x0
	// TX_ABRT interrupt is active
	I2C0_IC_RAW_INTR_STAT_TX_ABRT_ACTIVE = 0x1
	// Position of RD_REQ field.
	I2C0_IC_RAW_INTR_STAT_RD_REQ_Pos = 0x5
	// Bit mask of RD_REQ field.
	I2C0_IC_RAW_INTR_STAT_RD_REQ_Msk = 0x20
	// Bit RD_REQ.
	I2C0_IC_RAW_INTR_STAT_RD_REQ = 0x20
	// RD_REQ interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_RD_REQ_INACTIVE = 0x0
	// RD_REQ interrupt is active
	I2C0_IC_RAW_INTR_STAT_RD_REQ_ACTIVE = 0x1
	// Position of TX_EMPTY field.
	I2C0_IC_RAW_INTR_STAT_TX_EMPTY_Pos = 0x4
	// Bit mask of TX_EMPTY field.
	I2C0_IC_RAW_INTR_STAT_TX_EMPTY_Msk = 0x10
	// Bit TX_EMPTY.
	I2C0_IC_RAW_INTR_STAT_TX_EMPTY = 0x10
	// TX_EMPTY interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_TX_EMPTY_INACTIVE = 0x0
	// TX_EMPTY interrupt is active
	I2C0_IC_RAW_INTR_STAT_TX_EMPTY_ACTIVE = 0x1
	// Position of TX_OVER field.
	I2C0_IC_RAW_INTR_STAT_TX_OVER_Pos = 0x3
	// Bit mask of TX_OVER field.
	I2C0_IC_RAW_INTR_STAT_TX_OVER_Msk = 0x8
	// Bit TX_OVER.
	I2C0_IC_RAW_INTR_STAT_TX_OVER = 0x8
	// TX_OVER interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_TX_OVER_INACTIVE = 0x0
	// TX_OVER interrupt is active
	I2C0_IC_RAW_INTR_STAT_TX_OVER_ACTIVE = 0x1
	// Position of RX_FULL field.
	I2C0_IC_RAW_INTR_STAT_RX_FULL_Pos = 0x2
	// Bit mask of RX_FULL field.
	I2C0_IC_RAW_INTR_STAT_RX_FULL_Msk = 0x4
	// Bit RX_FULL.
	I2C0_IC_RAW_INTR_STAT_RX_FULL = 0x4
	// RX_FULL interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_RX_FULL_INACTIVE = 0x0
	// RX_FULL interrupt is active
	I2C0_IC_RAW_INTR_STAT_RX_FULL_ACTIVE = 0x1
	// Position of RX_OVER field.
	I2C0_IC_RAW_INTR_STAT_RX_OVER_Pos = 0x1
	// Bit mask of RX_OVER field.
	I2C0_IC_RAW_INTR_STAT_RX_OVER_Msk = 0x2
	// Bit RX_OVER.
	I2C0_IC_RAW_INTR_STAT_RX_OVER = 0x2
	// RX_OVER interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_RX_OVER_INACTIVE = 0x0
	// RX_OVER interrupt is active
	I2C0_IC_RAW_INTR_STAT_RX_OVER_ACTIVE = 0x1
	// Position of RX_UNDER field.
	I2C0_IC_RAW_INTR_STAT_RX_UNDER_Pos = 0x0
	// Bit mask of RX_UNDER field.
	I2C0_IC_RAW_INTR_STAT_RX_UNDER_Msk = 0x1
	// Bit RX_UNDER.
	I2C0_IC_RAW_INTR_STAT_RX_UNDER = 0x1
	// RX_UNDER interrupt is inactive
	I2C0_IC_RAW_INTR_STAT_RX_UNDER_INACTIVE = 0x0
	// RX_UNDER interrupt is active
	I2C0_IC_RAW_INTR_STAT_RX_UNDER_ACTIVE = 0x1

	// IC_RX_TL: I2C Receive FIFO Threshold Register
	// Position of RX_TL field.
	I2C0_IC_RX_TL_RX_TL_Pos = 0x0
	// Bit mask of RX_TL field.
	I2C0_IC_RX_TL_RX_TL_Msk = 0xff

	// IC_TX_TL: I2C Transmit FIFO Threshold Register
	// Position of TX_TL field.
	I2C0_IC_TX_TL_TX_TL_Pos = 0x0
	// Bit mask of TX_TL field.
	I2C0_IC_TX_TL_TX_TL_Msk = 0xff

	// IC_CLR_INTR: Clear Combined and Individual Interrupt Register
	// Position of CLR_INTR field.
	I2C0_IC_CLR_INTR_CLR_INTR_Pos = 0x0
	// Bit mask of CLR_INTR field.
	I2C0_IC_CLR_INTR_CLR_INTR_Msk = 0x1
	// Bit CLR_INTR.
	I2C0_IC_CLR_INTR_CLR_INTR = 0x1

	// IC_CLR_RX_UNDER: Clear RX_UNDER Interrupt Register
	// Position of CLR_RX_UNDER field.
	I2C0_IC_CLR_RX_UNDER_CLR_RX_UNDER_Pos = 0x0
	// Bit mask of CLR_RX_UNDER field.
	I2C0_IC_CLR_RX_UNDER_CLR_RX_UNDER_Msk = 0x1
	// Bit CLR_RX_UNDER.
	I2C0_IC_CLR_RX_UNDER_CLR_RX_UNDER = 0x1

	// IC_CLR_RX_OVER: Clear RX_OVER Interrupt Register
	// Position of CLR_RX_OVER field.
	I2C0_IC_CLR_RX_OVER_CLR_RX_OVER_Pos = 0x0
	// Bit mask of CLR_RX_OVER field.
	I2C0_IC_CLR_RX_OVER_CLR_RX_OVER_Msk = 0x1
	// Bit CLR_RX_OVER.
	I2C0_IC_CLR_RX_OVER_CLR_RX_OVER = 0x1

	// IC_CLR_TX_OVER: Clear TX_OVER Interrupt Register
	// Position of CLR_TX_OVER field.
	I2C0_IC_CLR_TX_OVER_CLR_TX_OVER_Pos = 0x0
	// Bit mask of CLR_TX_OVER field.
	I2C0_IC_CLR_TX_OVER_CLR_TX_OVER_Msk = 0x1
	// Bit CLR_TX_OVER.
	I2C0_IC_CLR_TX_OVER_CLR_TX_OVER = 0x1

	// IC_CLR_RD_REQ: Clear RD_REQ Interrupt Register
	// Position of CLR_RD_REQ field.
	I2C0_IC_CLR_RD_REQ_CLR_RD_REQ_Pos = 0x0
	// Bit mask of CLR_RD_REQ field.
	I2C0_IC_CLR_RD_REQ_CLR_RD_REQ_Msk = 0x1
	// Bit CLR_RD_REQ.
	I2C0_IC_CLR_RD_REQ_CLR_RD_REQ = 0x1

	// IC_CLR_TX_ABRT: Clear TX_ABRT Interrupt Register
	// Position of CLR_TX_ABRT field.
	I2C0_IC_CLR_TX_ABRT_CLR_TX_ABRT_Pos = 0x0
	// Bit mask of CLR_TX_ABRT field.
	I2C0_IC_CLR_TX_ABRT_CLR_TX_ABRT_Msk = 0x1
	// Bit CLR_TX_ABRT.
	I2C0_IC_CLR_TX_ABRT_CLR_TX_ABRT = 0x1

	// IC_CLR_RX_DONE: Clear RX_DONE Interrupt Register
	// Position of CLR_RX_DONE field.
	I2C0_IC_CLR_RX_DONE_CLR_RX_DONE_Pos = 0x0
	// Bit mask of CLR_RX_DONE field.
	I2C0_IC_CLR_RX_DONE_CLR_RX_DONE_Msk = 0x1
	// Bit CLR_RX_DONE.
	I2C0_IC_CLR_RX_DONE_CLR_RX_DONE = 0x1

	// IC_CLR_ACTIVITY: Clear ACTIVITY Interrupt Register
	// Position of CLR_ACTIVITY field.
	I2C0_IC_CLR_ACTIVITY_CLR_ACTIVITY_Pos = 0x0
	// Bit mask of CLR_ACTIVITY field.
	I2C0_IC_CLR_ACTIVITY_CLR_ACTIVITY_Msk = 0x1
	// Bit CLR_ACTIVITY.
	I2C0_IC_CLR_ACTIVITY_CLR_ACTIVITY = 0x1

	// IC_CLR_STOP_DET: Clear STOP_DET Interrupt Register
	// Position of CLR_STOP_DET field.
	I2C0_IC_CLR_STOP_DET_CLR_STOP_DET_Pos = 0x0
	// Bit mask of CLR_STOP_DET field.
	I2C0_IC_CLR_STOP_DET_CLR_STOP_DET_Msk = 0x1
	// Bit CLR_STOP_DET.
	I2C0_IC_CLR_STOP_DET_CLR_STOP_DET = 0x1

	// IC_CLR_START_DET: Clear START_DET Interrupt Register
	// Position of CLR_START_DET field.
	I2C0_IC_CLR_START_DET_CLR_START_DET_Pos = 0x0
	// Bit mask of CLR_START_DET field.
	I2C0_IC_CLR_START_DET_CLR_START_DET_Msk = 0x1
	// Bit CLR_START_DET.
	I2C0_IC_CLR_START_DET_CLR_START_DET = 0x1

	// IC_CLR_GEN_CALL: Clear GEN_CALL Interrupt Register
	// Position of CLR_GEN_CALL field.
	I2C0_IC_CLR_GEN_CALL_CLR_GEN_CALL_Pos = 0x0
	// Bit mask of CLR_GEN_CALL field.
	I2C0_IC_CLR_GEN_CALL_CLR_GEN_CALL_Msk = 0x1
	// Bit CLR_GEN_CALL.
	I2C0_IC_CLR_GEN_CALL_CLR_GEN_CALL = 0x1

	// IC_ENABLE: I2C Enable Register
	// Position of TX_CMD_BLOCK field.
	I2C0_IC_ENABLE_TX_CMD_BLOCK_Pos = 0x2
	// Bit mask of TX_CMD_BLOCK field.
	I2C0_IC_ENABLE_TX_CMD_BLOCK_Msk = 0x4
	// Bit TX_CMD_BLOCK.
	I2C0_IC_ENABLE_TX_CMD_BLOCK = 0x4
	// Tx Command execution not blocked
	I2C0_IC_ENABLE_TX_CMD_BLOCK_NOT_BLOCKED = 0x0
	// Tx Command execution blocked
	I2C0_IC_ENABLE_TX_CMD_BLOCK_BLOCKED = 0x1
	// Position of ABORT field.
	I2C0_IC_ENABLE_ABORT_Pos = 0x1
	// Bit mask of ABORT field.
	I2C0_IC_ENABLE_ABORT_Msk = 0x2
	// Bit ABORT.
	I2C0_IC_ENABLE_ABORT = 0x2
	// ABORT operation not in progress
	I2C0_IC_ENABLE_ABORT_DISABLE = 0x0
	// ABORT operation in progress
	I2C0_IC_ENABLE_ABORT_ENABLED = 0x1
	// Position of ENABLE field.
	I2C0_IC_ENABLE_ENABLE_Pos = 0x0
	// Bit mask of ENABLE field.
	I2C0_IC_ENABLE_ENABLE_Msk = 0x1
	// Bit ENABLE.
	I2C0_IC_ENABLE_ENABLE = 0x1
	// I2C is disabled
	I2C0_IC_ENABLE_ENABLE_DISABLED = 0x0
	// I2C is enabled
	I2C0_IC_ENABLE_ENABLE_ENABLED = 0x1

	// IC_STATUS: I2C Status Register This is a read-only register used to indicate the current transfer status and FIFO status. The status register may be read at any time. None of the bits in this register request an interrupt. When the I2C is disabled by writing 0 in bit 0 of the IC_ENABLE register: - Bits 1 and 2 are set to 1 - Bits 3 and 10 are set to 0 When the master or slave state machines goes to idle and ic_en=0: - Bits 5 and 6 are set to 0
	// Position of SLV_ACTIVITY field.
	I2C0_IC_STATUS_SLV_ACTIVITY_Pos = 0x6
	// Bit mask of SLV_ACTIVITY field.
	I2C0_IC_STATUS_SLV_ACTIVITY_Msk = 0x40
	// Bit SLV_ACTIVITY.
	I2C0_IC_STATUS_SLV_ACTIVITY = 0x40
	// Slave is idle
	I2C0_IC_STATUS_SLV_ACTIVITY_IDLE = 0x0
	// Slave not idle
	I2C0_IC_STATUS_SLV_ACTIVITY_ACTIVE = 0x1
	// Position of MST_ACTIVITY field.
	I2C0_IC_STATUS_MST_ACTIVITY_Pos = 0x5
	// Bit mask of MST_ACTIVITY field.
	I2C0_IC_STATUS_MST_ACTIVITY_Msk = 0x20
	// Bit MST_ACTIVITY.
	I2C0_IC_STATUS_MST_ACTIVITY = 0x20
	// Master is idle
	I2C0_IC_STATUS_MST_ACTIVITY_IDLE = 0x0
	// Master not idle
	I2C0_IC_STATUS_MST_ACTIVITY_ACTIVE = 0x1
	// Position of RFF field.
	I2C0_IC_STATUS_RFF_Pos = 0x4
	// Bit mask of RFF field.
	I2C0_IC_STATUS_RFF_Msk = 0x10
	// Bit RFF.
	I2C0_IC_STATUS_RFF = 0x10
	// Rx FIFO not full
	I2C0_IC_STATUS_RFF_NOT_FULL = 0x0
	// Rx FIFO is full
	I2C0_IC_STATUS_RFF_FULL = 0x1
	// Position of RFNE field.
	I2C0_IC_STATUS_RFNE_Pos = 0x3
	// Bit mask of RFNE field.
	I2C0_IC_STATUS_RFNE_Msk = 0x8
	// Bit RFNE.
	I2C0_IC_STATUS_RFNE = 0x8
	// Rx FIFO is empty
	I2C0_IC_STATUS_RFNE_EMPTY = 0x0
	// Rx FIFO not empty
	I2C0_IC_STATUS_RFNE_NOT_EMPTY = 0x1
	// Position of TFE field.
	I2C0_IC_STATUS_TFE_Pos = 0x2
	// Bit mask of TFE field.
	I2C0_IC_STATUS_TFE_Msk = 0x4
	// Bit TFE.
	I2C0_IC_STATUS_TFE = 0x4
	// Tx FIFO not empty
	I2C0_IC_STATUS_TFE_NON_EMPTY = 0x0
	// Tx FIFO is empty
	I2C0_IC_STATUS_TFE_EMPTY = 0x1
	// Position of TFNF field.
	I2C0_IC_STATUS_TFNF_Pos = 0x1
	// Bit mask of TFNF field.
	I2C0_IC_STATUS_TFNF_Msk = 0x2
	// Bit TFNF.
	I2C0_IC_STATUS_TFNF = 0x2
	// Tx FIFO is full
	I2C0_IC_STATUS_TFNF_FULL = 0x0
	// Tx FIFO not full
	I2C0_IC_STATUS_TFNF_NOT_FULL = 0x1
	// Position of ACTIVITY field.
	I2C0_IC_STATUS_ACTIVITY_Pos = 0x0
	// Bit mask of ACTIVITY field.
	I2C0_IC_STATUS_ACTIVITY_Msk = 0x1
	// Bit ACTIVITY.
	I2C0_IC_STATUS_ACTIVITY = 0x1
	// I2C is idle
	I2C0_IC_STATUS_ACTIVITY_INACTIVE = 0x0
	// I2C is active
	I2C0_IC_STATUS_ACTIVITY_ACTIVE = 0x1

	// IC_TXFLR: I2C Transmit FIFO Level Register This register contains the number of valid data entries in the transmit FIFO buffer. It is cleared whenever: - The I2C is disabled - There is a transmit abort - that is, TX_ABRT bit is set in the IC_RAW_INTR_STAT register - The slave bulk transmit mode is aborted The register increments whenever data is placed into the transmit FIFO and decrements when data is taken from the transmit FIFO.
	// Position of TXFLR field.
	I2C0_IC_TXFLR_TXFLR_Pos = 0x0
	// Bit mask of TXFLR field.
	I2C0_IC_TXFLR_TXFLR_Msk = 0x1f

	// IC_RXFLR: I2C Receive FIFO Level Register This register contains the number of valid data entries in the receive FIFO buffer. It is cleared whenever: - The I2C is disabled - Whenever there is a transmit abort caused by any of the events tracked in IC_TX_ABRT_SOURCE The register increments whenever data is placed into the receive FIFO and decrements when data is taken from the receive FIFO.
	// Position of RXFLR field.
	I2C0_IC_RXFLR_RXFLR_Pos = 0x0
	// Bit mask of RXFLR field.
	I2C0_IC_RXFLR_RXFLR_Msk = 0x1f

	// IC_SDA_HOLD: I2C SDA Hold Time Length Register The bits [15:0] of this register are used to control the hold time of SDA during transmit in both slave and master mode (after SCL goes from HIGH to LOW). The bits [23:16] of this register are used to extend the SDA transition (if any) whenever SCL is HIGH in the receiver in either master or slave mode. Writes to this register succeed only when IC_ENABLE[0]=0. The values in this register are in units of ic_clk period. The value programmed in IC_SDA_TX_HOLD must be greater than the minimum hold time in each mode (one cycle in master mode, seven cycles in slave mode) for the value to be implemented. The programmed SDA hold time during transmit (IC_SDA_TX_HOLD) cannot exceed at any time the duration of the low part of scl. Therefore the programmed value cannot be larger than N_SCL_LOW-2, where N_SCL_LOW is the duration of the low part of the scl period measured in ic_clk cycles.
	// Position of IC_SDA_RX_HOLD field.
	I2C0_IC_SDA_HOLD_IC_SDA_RX_HOLD_Pos = 0x10
	// Bit mask of IC_SDA_RX_HOLD field.
	I2C0_IC_SDA_HOLD_IC_SDA_RX_HOLD_Msk = 0xff0000
	// Position of IC_SDA_TX_HOLD field.
	I2C0_IC_SDA_HOLD_IC_SDA_TX_HOLD_Pos = 0x0
	// Bit mask of IC_SDA_TX_HOLD field.
	I2C0_IC_SDA_HOLD_IC_SDA_TX_HOLD_Msk = 0xffff

	// IC_TX_ABRT_SOURCE: I2C Transmit Abort Source Register This register has 32 bits that indicate the source of the TX_ABRT bit. Except for Bit 9, this register is cleared whenever the IC_CLR_TX_ABRT register or the IC_CLR_INTR register is read. To clear Bit 9, the source of the ABRT_SBYTE_NORSTRT must be fixed first; RESTART must be enabled (IC_CON[5]=1), the SPECIAL bit must be cleared (IC_TAR[11]), or the GC_OR_START bit must be cleared (IC_TAR[10]). Once the source of the ABRT_SBYTE_NORSTRT is fixed, then this bit can be cleared in the same manner as other bits in this register. If the source of the ABRT_SBYTE_NORSTRT is not fixed before attempting to clear this bit, Bit 9 clears for one cycle and is then re-asserted.
	// Position of TX_FLUSH_CNT field.
	I2C0_IC_TX_ABRT_SOURCE_TX_FLUSH_CNT_Pos = 0x17
	// Bit mask of TX_FLUSH_CNT field.
	I2C0_IC_TX_ABRT_SOURCE_TX_FLUSH_CNT_Msk = 0xff800000
	// Position of ABRT_USER_ABRT field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_USER_ABRT_Pos = 0x10
	// Bit mask of ABRT_USER_ABRT field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_USER_ABRT_Msk = 0x10000
	// Bit ABRT_USER_ABRT.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_USER_ABRT = 0x10000
	// Transfer abort detected by master- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_USER_ABRT_ABRT_USER_ABRT_VOID = 0x0
	// Transfer abort detected by master
	I2C0_IC_TX_ABRT_SOURCE_ABRT_USER_ABRT_ABRT_USER_ABRT_GENERATED = 0x1
	// Position of ABRT_SLVRD_INTX field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVRD_INTX_Pos = 0xf
	// Bit mask of ABRT_SLVRD_INTX field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVRD_INTX_Msk = 0x8000
	// Bit ABRT_SLVRD_INTX.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVRD_INTX = 0x8000
	// Slave trying to transmit to remote master in read mode- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVRD_INTX_ABRT_SLVRD_INTX_VOID = 0x0
	// Slave trying to transmit to remote master in read mode
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVRD_INTX_ABRT_SLVRD_INTX_GENERATED = 0x1
	// Position of ABRT_SLV_ARBLOST field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLV_ARBLOST_Pos = 0xe
	// Bit mask of ABRT_SLV_ARBLOST field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLV_ARBLOST_Msk = 0x4000
	// Bit ABRT_SLV_ARBLOST.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLV_ARBLOST = 0x4000
	// Slave lost arbitration to remote master- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLV_ARBLOST_ABRT_SLV_ARBLOST_VOID = 0x0
	// Slave lost arbitration to remote master
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLV_ARBLOST_ABRT_SLV_ARBLOST_GENERATED = 0x1
	// Position of ABRT_SLVFLUSH_TXFIFO field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVFLUSH_TXFIFO_Pos = 0xd
	// Bit mask of ABRT_SLVFLUSH_TXFIFO field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVFLUSH_TXFIFO_Msk = 0x2000
	// Bit ABRT_SLVFLUSH_TXFIFO.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVFLUSH_TXFIFO = 0x2000
	// Slave flushes existing data in TX-FIFO upon getting read command- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVFLUSH_TXFIFO_ABRT_SLVFLUSH_TXFIFO_VOID = 0x0
	// Slave flushes existing data in TX-FIFO upon getting read command
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SLVFLUSH_TXFIFO_ABRT_SLVFLUSH_TXFIFO_GENERATED = 0x1
	// Position of ARB_LOST field.
	I2C0_IC_TX_ABRT_SOURCE_ARB_LOST_Pos = 0xc
	// Bit mask of ARB_LOST field.
	I2C0_IC_TX_ABRT_SOURCE_ARB_LOST_Msk = 0x1000
	// Bit ARB_LOST.
	I2C0_IC_TX_ABRT_SOURCE_ARB_LOST = 0x1000
	// Master or Slave-Transmitter lost arbitration- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ARB_LOST_ABRT_LOST_VOID = 0x0
	// Master or Slave-Transmitter lost arbitration
	I2C0_IC_TX_ABRT_SOURCE_ARB_LOST_ABRT_LOST_GENERATED = 0x1
	// Position of ABRT_MASTER_DIS field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_MASTER_DIS_Pos = 0xb
	// Bit mask of ABRT_MASTER_DIS field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_MASTER_DIS_Msk = 0x800
	// Bit ABRT_MASTER_DIS.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_MASTER_DIS = 0x800
	// User initiating master operation when MASTER disabled- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_MASTER_DIS_ABRT_MASTER_DIS_VOID = 0x0
	// User initiating master operation when MASTER disabled
	I2C0_IC_TX_ABRT_SOURCE_ABRT_MASTER_DIS_ABRT_MASTER_DIS_GENERATED = 0x1
	// Position of ABRT_10B_RD_NORSTRT field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10B_RD_NORSTRT_Pos = 0xa
	// Bit mask of ABRT_10B_RD_NORSTRT field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10B_RD_NORSTRT_Msk = 0x400
	// Bit ABRT_10B_RD_NORSTRT.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10B_RD_NORSTRT = 0x400
	// Master not trying to read in 10Bit addressing mode when RESTART disabled
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10B_RD_NORSTRT_ABRT_10B_RD_VOID = 0x0
	// Master trying to read in 10Bit addressing mode when RESTART disabled
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10B_RD_NORSTRT_ABRT_10B_RD_GENERATED = 0x1
	// Position of ABRT_SBYTE_NORSTRT field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_NORSTRT_Pos = 0x9
	// Bit mask of ABRT_SBYTE_NORSTRT field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_NORSTRT_Msk = 0x200
	// Bit ABRT_SBYTE_NORSTRT.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_NORSTRT = 0x200
	// User trying to send START byte when RESTART disabled- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_NORSTRT_ABRT_SBYTE_NORSTRT_VOID = 0x0
	// User trying to send START byte when RESTART disabled
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_NORSTRT_ABRT_SBYTE_NORSTRT_GENERATED = 0x1
	// Position of ABRT_HS_NORSTRT field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_NORSTRT_Pos = 0x8
	// Bit mask of ABRT_HS_NORSTRT field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_NORSTRT_Msk = 0x100
	// Bit ABRT_HS_NORSTRT.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_NORSTRT = 0x100
	// User trying to switch Master to HS mode when RESTART disabled- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_NORSTRT_ABRT_HS_NORSTRT_VOID = 0x0
	// User trying to switch Master to HS mode when RESTART disabled
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_NORSTRT_ABRT_HS_NORSTRT_GENERATED = 0x1
	// Position of ABRT_SBYTE_ACKDET field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_ACKDET_Pos = 0x7
	// Bit mask of ABRT_SBYTE_ACKDET field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_ACKDET_Msk = 0x80
	// Bit ABRT_SBYTE_ACKDET.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_ACKDET = 0x80
	// ACK detected for START byte- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_ACKDET_ABRT_SBYTE_ACKDET_VOID = 0x0
	// ACK detected for START byte
	I2C0_IC_TX_ABRT_SOURCE_ABRT_SBYTE_ACKDET_ABRT_SBYTE_ACKDET_GENERATED = 0x1
	// Position of ABRT_HS_ACKDET field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_ACKDET_Pos = 0x6
	// Bit mask of ABRT_HS_ACKDET field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_ACKDET_Msk = 0x40
	// Bit ABRT_HS_ACKDET.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_ACKDET = 0x40
	// HS Master code ACKed in HS Mode- scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_ACKDET_ABRT_HS_ACK_VOID = 0x0
	// HS Master code ACKed in HS Mode
	I2C0_IC_TX_ABRT_SOURCE_ABRT_HS_ACKDET_ABRT_HS_ACK_GENERATED = 0x1
	// Position of ABRT_GCALL_READ field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_READ_Pos = 0x5
	// Bit mask of ABRT_GCALL_READ field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_READ_Msk = 0x20
	// Bit ABRT_GCALL_READ.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_READ = 0x20
	// GCALL is followed by read from bus-scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_READ_ABRT_GCALL_READ_VOID = 0x0
	// GCALL is followed by read from bus
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_READ_ABRT_GCALL_READ_GENERATED = 0x1
	// Position of ABRT_GCALL_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_NOACK_Pos = 0x4
	// Bit mask of ABRT_GCALL_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_NOACK_Msk = 0x10
	// Bit ABRT_GCALL_NOACK.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_NOACK = 0x10
	// GCALL not ACKed by any slave-scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_NOACK_ABRT_GCALL_NOACK_VOID = 0x0
	// GCALL not ACKed by any slave
	I2C0_IC_TX_ABRT_SOURCE_ABRT_GCALL_NOACK_ABRT_GCALL_NOACK_GENERATED = 0x1
	// Position of ABRT_TXDATA_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_TXDATA_NOACK_Pos = 0x3
	// Bit mask of ABRT_TXDATA_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_TXDATA_NOACK_Msk = 0x8
	// Bit ABRT_TXDATA_NOACK.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_TXDATA_NOACK = 0x8
	// Transmitted data non-ACKed by addressed slave-scenario not present
	I2C0_IC_TX_ABRT_SOURCE_ABRT_TXDATA_NOACK_ABRT_TXDATA_NOACK_VOID = 0x0
	// Transmitted data not ACKed by addressed slave
	I2C0_IC_TX_ABRT_SOURCE_ABRT_TXDATA_NOACK_ABRT_TXDATA_NOACK_GENERATED = 0x1
	// Position of ABRT_10ADDR2_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR2_NOACK_Pos = 0x2
	// Bit mask of ABRT_10ADDR2_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR2_NOACK_Msk = 0x4
	// Bit ABRT_10ADDR2_NOACK.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR2_NOACK = 0x4
	// This abort is not generated
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR2_NOACK_INACTIVE = 0x0
	// Byte 2 of 10Bit Address not ACKed by any slave
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR2_NOACK_ACTIVE = 0x1
	// Position of ABRT_10ADDR1_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR1_NOACK_Pos = 0x1
	// Bit mask of ABRT_10ADDR1_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR1_NOACK_Msk = 0x2
	// Bit ABRT_10ADDR1_NOACK.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR1_NOACK = 0x2
	// This abort is not generated
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR1_NOACK_INACTIVE = 0x0
	// Byte 1 of 10Bit Address not ACKed by any slave
	I2C0_IC_TX_ABRT_SOURCE_ABRT_10ADDR1_NOACK_ACTIVE = 0x1
	// Position of ABRT_7B_ADDR_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_7B_ADDR_NOACK_Pos = 0x0
	// Bit mask of ABRT_7B_ADDR_NOACK field.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_7B_ADDR_NOACK_Msk = 0x1
	// Bit ABRT_7B_ADDR_NOACK.
	I2C0_IC_TX_ABRT_SOURCE_ABRT_7B_ADDR_NOACK = 0x1
	// This abort is not generated
	I2C0_IC_TX_ABRT_SOURCE_ABRT_7B_ADDR_NOACK_INACTIVE = 0x0
	// This abort is generated because of NOACK for 7-bit address
	I2C0_IC_TX_ABRT_SOURCE_ABRT_7B_ADDR_NOACK_ACTIVE = 0x1

	// IC_SLV_DATA_NACK_ONLY: Generate Slave Data NACK Register The register is used to generate a NACK for the data part of a transfer when DW_apb_i2c is acting as a slave-receiver. This register only exists when the IC_SLV_DATA_NACK_ONLY parameter is set to 1. When this parameter disabled, this register does not exist and writing to the register's address has no effect. A write can occur on this register if both of the following conditions are met: - DW_apb_i2c is disabled (IC_ENABLE[0] = 0) - Slave part is inactive (IC_STATUS[6] = 0) Note: The IC_STATUS[6] is a register read-back location for the internal slv_activity signal; the user should poll this before writing the ic_slv_data_nack_only bit.
	// Position of NACK field.
	I2C0_IC_SLV_DATA_NACK_ONLY_NACK_Pos = 0x0
	// Bit mask of NACK field.
	I2C0_IC_SLV_DATA_NACK_ONLY_NACK_Msk = 0x1
	// Bit NACK.
	I2C0_IC_SLV_DATA_NACK_ONLY_NACK = 0x1
	// Slave receiver generates NACK normally
	I2C0_IC_SLV_DATA_NACK_ONLY_NACK_DISABLED = 0x0
	// Slave receiver generates NACK upon data reception only
	I2C0_IC_SLV_DATA_NACK_ONLY_NACK_ENABLED = 0x1

	// IC_DMA_CR: DMA Control Register The register is used to enable the DMA Controller interface operation. There is a separate bit for transmit and receive. This can be programmed regardless of the state of IC_ENABLE.
	// Position of TDMAE field.
	I2C0_IC_DMA_CR_TDMAE_Pos = 0x1
	// Bit mask of TDMAE field.
	I2C0_IC_DMA_CR_TDMAE_Msk = 0x2
	// Bit TDMAE.
	I2C0_IC_DMA_CR_TDMAE = 0x2
	// transmit FIFO DMA channel disabled
	I2C0_IC_DMA_CR_TDMAE_DISABLED = 0x0
	// Transmit FIFO DMA channel enabled
	I2C0_IC_DMA_CR_TDMAE_ENABLED = 0x1
	// Position of RDMAE field.
	I2C0_IC_DMA_CR_RDMAE_Pos = 0x0
	// Bit mask of RDMAE field.
	I2C0_IC_DMA_CR_RDMAE_Msk = 0x1
	// Bit RDMAE.
	I2C0_IC_DMA_CR_RDMAE = 0x1
	// Receive FIFO DMA channel disabled
	I2C0_IC_DMA_CR_RDMAE_DISABLED = 0x0
	// Receive FIFO DMA channel enabled
	I2C0_IC_DMA_CR_RDMAE_ENABLED = 0x1

	// IC_DMA_TDLR: DMA Transmit Data Level Register
	// Position of DMATDL field.
	I2C0_IC_DMA_TDLR_DMATDL_Pos = 0x0
	// Bit mask of DMATDL field.
	I2C0_IC_DMA_TDLR_DMATDL_Msk = 0xf

	// IC_DMA_RDLR: I2C Receive Data Level Register
	// Position of DMARDL field.
	I2C0_IC_DMA_RDLR_DMARDL_Pos = 0x0
	// Bit mask of DMARDL field.
	I2C0_IC_DMA_RDLR_DMARDL_Msk = 0xf

	// IC_SDA_SETUP: I2C SDA Setup Register This register controls the amount of time delay (in terms of number of ic_clk clock periods) introduced in the rising edge of SCL - relative to SDA changing - when DW_apb_i2c services a read request in a slave-transmitter operation. The relevant I2C requirement is tSU:DAT (note 4) as detailed in the I2C Bus Specification. This register must be programmed with a value equal to or greater than 2. Writes to this register succeed only when IC_ENABLE[0] = 0. Note: The length of setup time is calculated using [(IC_SDA_SETUP - 1) * (ic_clk_period)], so if the user requires 10 ic_clk periods of setup time, they should program a value of 11. The IC_SDA_SETUP register is only used by the DW_apb_i2c when operating as a slave transmitter.
	// Position of SDA_SETUP field.
	I2C0_IC_SDA_SETUP_SDA_SETUP_Pos = 0x0
	// Bit mask of SDA_SETUP field.
	I2C0_IC_SDA_SETUP_SDA_SETUP_Msk = 0xff

	// IC_ACK_GENERAL_CALL: I2C ACK General Call Register The register controls whether DW_apb_i2c responds with a ACK or NACK when it receives an I2C General Call address. This register is applicable only when the DW_apb_i2c is in slave mode.
	// Position of ACK_GEN_CALL field.
	I2C0_IC_ACK_GENERAL_CALL_ACK_GEN_CALL_Pos = 0x0
	// Bit mask of ACK_GEN_CALL field.
	I2C0_IC_ACK_GENERAL_CALL_ACK_GEN_CALL_Msk = 0x1
	// Bit ACK_GEN_CALL.
	I2C0_IC_ACK_GENERAL_CALL_ACK_GEN_CALL = 0x1
	// Generate NACK for a General Call
	I2C0_IC_ACK_GENERAL_CALL_ACK_GEN_CALL_DISABLED = 0x0
	// Generate ACK for a General Call
	I2C0_IC_ACK_GENERAL_CALL_ACK_GEN_CALL_ENABLED = 0x1

	// IC_ENABLE_STATUS: I2C Enable Status Register The register is used to report the DW_apb_i2c hardware status when the IC_ENABLE[0] register is set from 1 to 0; that is, when DW_apb_i2c is disabled. If IC_ENABLE[0] has been set to 1, bits 2:1 are forced to 0, and bit 0 is forced to 1. If IC_ENABLE[0] has been set to 0, bits 2:1 is only be valid as soon as bit 0 is read as '0'. Note: When IC_ENABLE[0] has been set to 0, a delay occurs for bit 0 to be read as 0 because disabling the DW_apb_i2c depends on I2C bus activities.
	// Position of SLV_RX_DATA_LOST field.
	I2C0_IC_ENABLE_STATUS_SLV_RX_DATA_LOST_Pos = 0x2
	// Bit mask of SLV_RX_DATA_LOST field.
	I2C0_IC_ENABLE_STATUS_SLV_RX_DATA_LOST_Msk = 0x4
	// Bit SLV_RX_DATA_LOST.
	I2C0_IC_ENABLE_STATUS_SLV_RX_DATA_LOST = 0x4
	// Slave RX Data is not lost
	I2C0_IC_ENABLE_STATUS_SLV_RX_DATA_LOST_INACTIVE = 0x0
	// Slave RX Data is lost
	I2C0_IC_ENABLE_STATUS_SLV_RX_DATA_LOST_ACTIVE = 0x1
	// Position of SLV_DISABLED_WHILE_BUSY field.
	I2C0_IC_ENABLE_STATUS_SLV_DISABLED_WHILE_BUSY_Pos = 0x1
	// Bit mask of SLV_DISABLED_WHILE_BUSY field.
	I2C0_IC_ENABLE_STATUS_SLV_DISABLED_WHILE_BUSY_Msk = 0x2
	// Bit SLV_DISABLED_WHILE_BUSY.
	I2C0_IC_ENABLE_STATUS_SLV_DISABLED_WHILE_BUSY = 0x2
	// Slave is disabled when it is idle
	I2C0_IC_ENABLE_STATUS_SLV_DISABLED_WHILE_BUSY_INACTIVE = 0x0
	// Slave is disabled when it is active
	I2C0_IC_ENABLE_STATUS_SLV_DISABLED_WHILE_BUSY_ACTIVE = 0x1
	// Position of IC_EN field.
	I2C0_IC_ENABLE_STATUS_IC_EN_Pos = 0x0
	// Bit mask of IC_EN field.
	I2C0_IC_ENABLE_STATUS_IC_EN_Msk = 0x1
	// Bit IC_EN.
	I2C0_IC_ENABLE_STATUS_IC_EN = 0x1
	// I2C disabled
	I2C0_IC_ENABLE_STATUS_IC_EN_DISABLED = 0x0
	// I2C enabled
	I2C0_IC_ENABLE_STATUS_IC_EN_ENABLED = 0x1

	// IC_FS_SPKLEN: I2C SS, FS or FM+ spike suppression limit This register is used to store the duration, measured in ic_clk cycles, of the longest spike that is filtered out by the spike suppression logic when the component is operating in SS, FS or FM+ modes. The relevant I2C requirement is tSP (table 4) as detailed in the I2C Bus Specification. This register must be programmed with a minimum value of 1.
	// Position of IC_FS_SPKLEN field.
	I2C0_IC_FS_SPKLEN_IC_FS_SPKLEN_Pos = 0x0
	// Bit mask of IC_FS_SPKLEN field.
	I2C0_IC_FS_SPKLEN_IC_FS_SPKLEN_Msk = 0xff

	// IC_CLR_RESTART_DET: Clear RESTART_DET Interrupt Register
	// Position of CLR_RESTART_DET field.
	I2C0_IC_CLR_RESTART_DET_CLR_RESTART_DET_Pos = 0x0
	// Bit mask of CLR_RESTART_DET field.
	I2C0_IC_CLR_RESTART_DET_CLR_RESTART_DET_Msk = 0x1
	// Bit CLR_RESTART_DET.
	I2C0_IC_CLR_RESTART_DET_CLR_RESTART_DET = 0x1

	// IC_COMP_PARAM_1: Component Parameter Register 1 Note This register is not implemented and therefore reads as 0. If it was implemented it would be a constant read-only register that contains encoded information about the component's parameter settings. Fields shown below are the settings for those parameters
	// Position of TX_BUFFER_DEPTH field.
	I2C0_IC_COMP_PARAM_1_TX_BUFFER_DEPTH_Pos = 0x10
	// Bit mask of TX_BUFFER_DEPTH field.
	I2C0_IC_COMP_PARAM_1_TX_BUFFER_DEPTH_Msk = 0xff0000
	// Position of RX_BUFFER_DEPTH field.
	I2C0_IC_COMP_PARAM_1_RX_BUFFER_DEPTH_Pos = 0x8
	// Bit mask of RX_BUFFER_DEPTH field.
	I2C0_IC_COMP_PARAM_1_RX_BUFFER_DEPTH_Msk = 0xff00
	// Position of ADD_ENCODED_PARAMS field.
	I2C0_IC_COMP_PARAM_1_ADD_ENCODED_PARAMS_Pos = 0x7
	// Bit mask of ADD_ENCODED_PARAMS field.
	I2C0_IC_COMP_PARAM_1_ADD_ENCODED_PARAMS_Msk = 0x80
	// Bit ADD_ENCODED_PARAMS.
	I2C0_IC_COMP_PARAM_1_ADD_ENCODED_PARAMS = 0x80
	// Position of HAS_DMA field.
	I2C0_IC_COMP_PARAM_1_HAS_DMA_Pos = 0x6
	// Bit mask of HAS_DMA field.
	I2C0_IC_COMP_PARAM_1_HAS_DMA_Msk = 0x40
	// Bit HAS_DMA.
	I2C0_IC_COMP_PARAM_1_HAS_DMA = 0x40
	// Position of INTR_IO field.
	I2C0_IC_COMP_PARAM_1_INTR_IO_Pos = 0x5
	// Bit mask of INTR_IO field.
	I2C0_IC_COMP_PARAM_1_INTR_IO_Msk = 0x20
	// Bit INTR_IO.
	I2C0_IC_COMP_PARAM_1_INTR_IO = 0x20
	// Position of HC_COUNT_VALUES field.
	I2C0_IC_COMP_PARAM_1_HC_COUNT_VALUES_Pos = 0x4
	// Bit mask of HC_COUNT_VALUES field.
	I2C0_IC_COMP_PARAM_1_HC_COUNT_VALUES_Msk = 0x10
	// Bit HC_COUNT_VALUES.
	I2C0_IC_COMP_PARAM_1_HC_COUNT_VALUES = 0x10
	// Position of MAX_SPEED_MODE field.
	I2C0_IC_COMP_PARAM_1_MAX_SPEED_MODE_Pos = 0x2
	// Bit mask of MAX_SPEED_MODE field.
	I2C0_IC_COMP_PARAM_1_MAX_SPEED_MODE_Msk = 0xc
	// Position of APB_DATA_WIDTH field.
	I2C0_IC_COMP_PARAM_1_APB_DATA_WIDTH_Pos = 0x0
	// Bit mask of APB_DATA_WIDTH field.
	I2C0_IC_COMP_PARAM_1_APB_DATA_WIDTH_Msk = 0x3

	// IC_COMP_VERSION: I2C Component Version Register
	// Position of IC_COMP_VERSION field.
	I2C0_IC_COMP_VERSION_IC_COMP_VERSION_Pos = 0x0
	// Bit mask of IC_COMP_VERSION field.
	I2C0_IC_COMP_VERSION_IC_COMP_VERSION_Msk = 0xffffffff

	// IC_COMP_TYPE: I2C Component Type Register
	// Position of IC_COMP_TYPE field.
	I2C0_IC_COMP_TYPE_IC_COMP_TYPE_Pos = 0x0
	// Bit mask of IC_COMP_TYPE field.
	I2C0_IC_COMP_TYPE_IC_COMP_TYPE_Msk = 0xffffffff
)

// Constants for SPI0
const (
	// SSPCR0: Control register 0, SSPCR0 on page 3-4
	// Position of SCR field.
	SPI0_SSPCR0_SCR_Pos = 0x8
	// Bit mask of SCR field.
	SPI0_SSPCR0_SCR_Msk = 0xff00
	// Position of SPH field.
	SPI0_SSPCR0_SPH_Pos = 0x7
	// Bit mask of SPH field.
	SPI0_SSPCR0_SPH_Msk = 0x80
	// Bit SPH.
	SPI0_SSPCR0_SPH = 0x80
	// Position of SPO field.
	SPI0_SSPCR0_SPO_Pos = 0x6
	// Bit mask of SPO field.
	SPI0_SSPCR0_SPO_Msk = 0x40
	// Bit SPO.
	SPI0_SSPCR0_SPO = 0x40
	// Position of FRF field.
	SPI0_SSPCR0_FRF_Pos = 0x4
	// Bit mask of FRF field.
	SPI0_SSPCR0_FRF_Msk = 0x30
	// Position of DSS field.
	SPI0_SSPCR0_DSS_Pos = 0x0
	// Bit mask of DSS field.
	SPI0_SSPCR0_DSS_Msk = 0xf

	// SSPCR1: Control register 1, SSPCR1 on page 3-5
	// Position of SOD field.
	SPI0_SSPCR1_SOD_Pos = 0x3
	// Bit mask of SOD field.
	SPI0_SSPCR1_SOD_Msk = 0x8
	// Bit SOD.
	SPI0_SSPCR1_SOD = 0x8
	// Position of MS field.
	SPI0_SSPCR1_MS_Pos = 0x2
	// Bit mask of MS field.
	SPI0_SSPCR1_MS_Msk = 0x4
	// Bit MS.
	SPI0_SSPCR1_MS = 0x4
	// Position of SSE field.
	SPI0_SSPCR1_SSE_Pos = 0x1
	// Bit mask of SSE field.
	SPI0_SSPCR1_SSE_Msk = 0x2
	// Bit SSE.
	SPI0_SSPCR1_SSE = 0x2
	// Position of LBM field.
	SPI0_SSPCR1_LBM_Pos = 0x0
	// Bit mask of LBM field.
	SPI0_SSPCR1_LBM_Msk = 0x1
	// Bit LBM.
	SPI0_SSPCR1_LBM = 0x1

	// SSPDR: Data register, SSPDR on page 3-6
	// Position of DATA field.
	SPI0_SSPDR_DATA_Pos = 0x0
	// Bit mask of DATA field.
	SPI0_SSPDR_DATA_Msk = 0xffff

	// SSPSR: Status register, SSPSR on page 3-7
	// Position of BSY field.
	SPI0_SSPSR_BSY_Pos = 0x4
	// Bit mask of BSY field.
	SPI0_SSPSR_BSY_Msk = 0x10
	// Bit BSY.
	SPI0_SSPSR_BSY = 0x10
	// Position of RFF field.
	SPI0_SSPSR_RFF_Pos = 0x3
	// Bit mask of RFF field.
	SPI0_SSPSR_RFF_Msk = 0x8
	// Bit RFF.
	SPI0_SSPSR_RFF = 0x8
	// Position of RNE field.
	SPI0_SSPSR_RNE_Pos = 0x2
	// Bit mask of RNE field.
	SPI0_SSPSR_RNE_Msk = 0x4
	// Bit RNE.
	SPI0_SSPSR_RNE = 0x4
	// Position of TNF field.
	SPI0_SSPSR_TNF_Pos = 0x1
	// Bit mask of TNF field.
	SPI0_SSPSR_TNF_Msk = 0x2
	// Bit TNF.
	SPI0_SSPSR_TNF = 0x2
	// Position of TFE field.
	SPI0_SSPSR_TFE_Pos = 0x0
	// Bit mask of TFE field.
	SPI0_SSPSR_TFE_Msk = 0x1
	// Bit TFE.
	SPI0_SSPSR_TFE = 0x1

	// SSPCPSR: Clock prescale register, SSPCPSR on page 3-8
	// Position of CPSDVSR field.
	SPI0_SSPCPSR_CPSDVSR_Pos = 0x0
	// Bit mask of CPSDVSR field.
	SPI0_SSPCPSR_CPSDVSR_Msk = 0xff

	// SSPIMSC: Interrupt mask set or clear register, SSPIMSC on page 3-9
	// Position of TXIM field.
	SPI0_SSPIMSC_TXIM_Pos = 0x3
	// Bit mask of TXIM field.
	SPI0_SSPIMSC_TXIM_Msk = 0x8
	// Bit TXIM.
	SPI0_SSPIMSC_TXIM = 0x8
	// Position of RXIM field.
	SPI0_SSPIMSC_RXIM_Pos = 0x2
	// Bit mask of RXIM field.
	SPI0_SSPIMSC_RXIM_Msk = 0x4
	// Bit RXIM.
	SPI0_SSPIMSC_RXIM = 0x4
	// Position of RTIM field.
	SPI0_SSPIMSC_RTIM_Pos = 0x1
	// Bit mask of RTIM field.
	SPI0_SSPIMSC_RTIM_Msk = 0x2
	// Bit RTIM.
	SPI0_SSPIMSC_RTIM = 0x2
	// Position of RORIM field.
	SPI0_SSPIMSC_RORIM_Pos = 0x0
	// Bit mask of RORIM field.
	SPI0_SSPIMSC_RORIM_Msk = 0x1
	// Bit RORIM.
	SPI0_SSPIMSC_RORIM = 0x1

	// SSPRIS: Raw interrupt status register, SSPRIS on page 3-10
	// Position of TXRIS field.
	SPI0_SSPRIS_TXRIS_Pos = 0x3
	// Bit mask of TXRIS field.
	SPI0_SSPRIS_TXRIS_Msk = 0x8
	// Bit TXRIS.
	SPI0_SSPRIS_TXRIS = 0x8
	// Position of RXRIS field.
	SPI0_SSPRIS_RXRIS_Pos = 0x2
	// Bit mask of RXRIS field.
	SPI0_SSPRIS_RXRIS_Msk = 0x4
	// Bit RXRIS.
	SPI0_SSPRIS_RXRIS = 0x4
	// Position of RTRIS field.
	SPI0_SSPRIS_RTRIS_Pos = 0x1
	// Bit mask of RTRIS field.
	SPI0_SSPRIS_RTRIS_Msk = 0x2
	// Bit RTRIS.
	SPI0_SSPRIS_RTRIS = 0x2
	// Position of RORRIS field.
	SPI0_SSPRIS_RORRIS_Pos = 0x0
	// Bit mask of RORRIS field.
	SPI0_SSPRIS_RORRIS_Msk = 0x1
	// Bit RORRIS.
	SPI0_SSPRIS_RORRIS = 0x1

	// SSPMIS: Masked interrupt status register, SSPMIS on page 3-11
	// Position of TXMIS field.
	SPI0_SSPMIS_TXMIS_Pos = 0x3
	// Bit mask of TXMIS field.
	SPI0_SSPMIS_TXMIS_Msk = 0x8
	// Bit TXMIS.
	SPI0_SSPMIS_TXMIS = 0x8
	// Position of RXMIS field.
	SPI0_SSPMIS_RXMIS_Pos = 0x2
	// Bit mask of RXMIS field.
	SPI0_SSPMIS_RXMIS_Msk = 0x4
	// Bit RXMIS.
	SPI0_SSPMIS_RXMIS = 0x4
	// Position of RTMIS field.
	SPI0_SSPMIS_RTMIS_Pos = 0x1
	// Bit mask of RTMIS field.
	SPI0_SSPMIS_RTMIS_Msk = 0x2
	// Bit RTMIS.
	SPI0_SSPMIS_RTMIS = 0x2
	// Position of RORMIS field.
	SPI0_SSPMIS_RORMIS_Pos = 0x0
	// Bit mask of RORMIS field.
	SPI0_SSPMIS_RORMIS_Msk = 0x1
	// Bit RORMIS.
	SPI0_SSPMIS_RORMIS = 0x1

	// SSPICR: Interrupt clear register, SSPICR on page 3-11
	// Position of RTIC field.
	SPI0_SSPICR_RTIC_Pos = 0x1
	// Bit mask of RTIC field.
	SPI0_SSPICR_RTIC_Msk = 0x2
	// Bit RTIC.
	SPI0_SSPICR_RTIC = 0x2
	// Position of RORIC field.
	SPI0_SSPICR_RORIC_Pos = 0x0
	// Bit mask of RORIC field.
	SPI0_SSPICR_RORIC_Msk = 0x1
	// Bit RORIC.
	SPI0_SSPICR_RORIC = 0x1

	// SSPDMACR: DMA control register, SSPDMACR on page 3-12
	// Position of TXDMAE field.
	SPI0_SSPDMACR_TXDMAE_Pos = 0x1
	// Bit mask of TXDMAE field.
	SPI0_SSPDMACR_TXDMAE_Msk = 0x2
	// Bit TXDMAE.
	SPI0_SSPDMACR_TXDMAE = 0x2
	// Position of RXDMAE field.
	SPI0_SSPDMACR_RXDMAE_Pos = 0x0
	// Bit mask of RXDMAE field.
	SPI0_SSPDMACR_RXDMAE_Msk = 0x1
	// Bit RXDMAE.
	SPI0_SSPDMACR_RXDMAE = 0x1

	// SSPPERIPHID0: Peripheral identification registers, SSPPeriphID0-3 on page 3-13
	// Position of PARTNUMBER0 field.
	SPI0_SSPPERIPHID0_PARTNUMBER0_Pos = 0x0
	// Bit mask of PARTNUMBER0 field.
	SPI0_SSPPERIPHID0_PARTNUMBER0_Msk = 0xff

	// SSPPERIPHID1: Peripheral identification registers, SSPPeriphID0-3 on page 3-13
	// Position of DESIGNER0 field.
	SPI0_SSPPERIPHID1_DESIGNER0_Pos = 0x4
	// Bit mask of DESIGNER0 field.
	SPI0_SSPPERIPHID1_DESIGNER0_Msk = 0xf0
	// Position of PARTNUMBER1 field.
	SPI0_SSPPERIPHID1_PARTNUMBER1_Pos = 0x0
	// Bit mask of PARTNUMBER1 field.
	SPI0_SSPPERIPHID1_PARTNUMBER1_Msk = 0xf

	// SSPPERIPHID2: Peripheral identification registers, SSPPeriphID0-3 on page 3-13
	// Position of REVISION field.
	SPI0_SSPPERIPHID2_REVISION_Pos = 0x4
	// Bit mask of REVISION field.
	SPI0_SSPPERIPHID2_REVISION_Msk = 0xf0
	// Position of DESIGNER1 field.
	SPI0_SSPPERIPHID2_DESIGNER1_Pos = 0x0
	// Bit mask of DESIGNER1 field.
	SPI0_SSPPERIPHID2_DESIGNER1_Msk = 0xf

	// SSPPERIPHID3: Peripheral identification registers, SSPPeriphID0-3 on page 3-13
	// Position of CONFIGURATION field.
	SPI0_SSPPERIPHID3_CONFIGURATION_Pos = 0x0
	// Bit mask of CONFIGURATION field.
	SPI0_SSPPERIPHID3_CONFIGURATION_Msk = 0xff

	// SSPPCELLID0: PrimeCell identification registers, SSPPCellID0-3 on page 3-16
	// Position of SSPPCELLID0 field.
	SPI0_SSPPCELLID0_SSPPCELLID0_Pos = 0x0
	// Bit mask of SSPPCELLID0 field.
	SPI0_SSPPCELLID0_SSPPCELLID0_Msk = 0xff

	// SSPPCELLID1: PrimeCell identification registers, SSPPCellID0-3 on page 3-16
	// Position of SSPPCELLID1 field.
	SPI0_SSPPCELLID1_SSPPCELLID1_Pos = 0x0
	// Bit mask of SSPPCELLID1 field.
	SPI0_SSPPCELLID1_SSPPCELLID1_Msk = 0xff

	// SSPPCELLID2: PrimeCell identification registers, SSPPCellID0-3 on page 3-16
	// Position of SSPPCELLID2 field.
	SPI0_SSPPCELLID2_SSPPCELLID2_Pos = 0x0
	// Bit mask of SSPPCELLID2 field.
	SPI0_SSPPCELLID2_SSPPCELLID2_Msk = 0xff

	// SSPPCELLID3: PrimeCell identification registers, SSPPCellID0-3 on page 3-16
	// Position of SSPPCELLID3 field.
	SPI0_SSPPCELLID3_SSPPCELLID3_Pos = 0x0
	// Bit mask of SSPPCELLID3 field.
	SPI0_SSPPCELLID3_SSPPCELLID3_Msk = 0xff
)

// Constants for PIO0: Programmable IO block
const (
	// CTRL: PIO control register
	// Position of NEXTPREV_CLKDIV_RESTART field.
	PIO0_CTRL_NEXTPREV_CLKDIV_RESTART_Pos = 0x1a
	// Bit mask of NEXTPREV_CLKDIV_RESTART field.
	PIO0_CTRL_NEXTPREV_CLKDIV_RESTART_Msk = 0x4000000
	// Bit NEXTPREV_CLKDIV_RESTART.
	PIO0_CTRL_NEXTPREV_CLKDIV_RESTART = 0x4000000
	// Position of NEXTPREV_SM_DISABLE field.
	PIO0_CTRL_NEXTPREV_SM_DISABLE_Pos = 0x19
	// Bit mask of NEXTPREV_SM_DISABLE field.
	PIO0_CTRL_NEXTPREV_SM_DISABLE_Msk = 0x2000000
	// Bit NEXTPREV_SM_DISABLE.
	PIO0_CTRL_NEXTPREV_SM_DISABLE = 0x2000000
	// Position of NEXTPREV_SM_ENABLE field.
	PIO0_CTRL_NEXTPREV_SM_ENABLE_Pos = 0x18
	// Bit mask of NEXTPREV_SM_ENABLE field.
	PIO0_CTRL_NEXTPREV_SM_ENABLE_Msk = 0x1000000
	// Bit NEXTPREV_SM_ENABLE.
	PIO0_CTRL_NEXTPREV_SM_ENABLE = 0x1000000
	// Position of NEXT_PIO_MASK field.
	PIO0_CTRL_NEXT_PIO_MASK_Pos = 0x14
	// Bit mask of NEXT_PIO_MASK field.
	PIO0_CTRL_NEXT_PIO_MASK_Msk = 0xf00000
	// Position of PREV_PIO_MASK field.
	PIO0_CTRL_PREV_PIO_MASK_Pos = 0x10
	// Bit mask of PREV_PIO_MASK field.
	PIO0_CTRL_PREV_PIO_MASK_Msk = 0xf0000
	// Position of CLKDIV_RESTART field.
	PIO0_CTRL_CLKDIV_RESTART_Pos = 0x8
	// Bit mask of CLKDIV_RESTART field.
	PIO0_CTRL_CLKDIV_RESTART_Msk = 0xf00
	// Position of SM_RESTART field.
	PIO0_CTRL_SM_RESTART_Pos = 0x4
	// Bit mask of SM_RESTART field.
	PIO0_CTRL_SM_RESTART_Msk = 0xf0
	// Position of SM_ENABLE field.
	PIO0_CTRL_SM_ENABLE_Pos = 0x0
	// Bit mask of SM_ENABLE field.
	PIO0_CTRL_SM_ENABLE_Msk = 0xf

	// FSTAT: FIFO status register
	// Position of TXEMPTY field.
	PIO0_FSTAT_TXEMPTY_Pos = 0x18
	// Bit mask of TXEMPTY field.
	PIO0_FSTAT_TXEMPTY_Msk = 0xf000000
	// Position of TXFULL field.
	PIO0_FSTAT_TXFULL_Pos = 0x10
	// Bit mask of TXFULL field.
	PIO0_FSTAT_TXFULL_Msk = 0xf0000
	// Position of RXEMPTY field.
	PIO0_FSTAT_RXEMPTY_Pos = 0x8
	// Bit mask of RXEMPTY field.
	PIO0_FSTAT_RXEMPTY_Msk = 0xf00
	// Position of RXFULL field.
	PIO0_FSTAT_RXFULL_Pos = 0x0
	// Bit mask of RXFULL field.
	PIO0_FSTAT_RXFULL_Msk = 0xf

	// FDEBUG: FIFO debug register
	// Position of TXSTALL field.
	PIO0_FDEBUG_TXSTALL_Pos = 0x18
	// Bit mask of TXSTALL field.
	PIO0_FDEBUG_TXSTALL_Msk = 0xf000000
	// Position of TXOVER field.
	PIO0_FDEBUG_TXOVER_Pos = 0x10
	// Bit mask of TXOVER field.
	PIO0_FDEBUG_TXOVER_Msk = 0xf0000
	// Position of RXUNDER field.
	PIO0_FDEBUG_RXUNDER_Pos = 0x8
	// Bit mask of RXUNDER field.
	PIO0_FDEBUG_RXUNDER_Msk = 0xf00
	// Position of RXSTALL field.
	PIO0_FDEBUG_RXSTALL_Pos = 0x0
	// Bit mask of RXSTALL field.
	PIO0_FDEBUG_RXSTALL_Msk = 0xf

	// FLEVEL: FIFO levels
	// Position of RX3 field.
	PIO0_FLEVEL_RX3_Pos = 0x1c
	// Bit mask of RX3 field.
	PIO0_FLEVEL_RX3_Msk = 0xf0000000
	// Position of TX3 field.
	PIO0_FLEVEL_TX3_Pos = 0x18
	// Bit mask of TX3 field.
	PIO0_FLEVEL_TX3_Msk = 0xf000000
	// Position of RX2 field.
	PIO0_FLEVEL_RX2_Pos = 0x14
	// Bit mask of RX2 field.
	PIO0_FLEVEL_RX2_Msk = 0xf00000
	// Position of TX2 field.
	PIO0_FLEVEL_TX2_Pos = 0x10
	// Bit mask of TX2 field.
	PIO0_FLEVEL_TX2_Msk = 0xf0000
	// Position of RX1 field.
	PIO0_FLEVEL_RX1_Pos = 0xc
	// Bit mask of RX1 field.
	PIO0_FLEVEL_RX1_Msk = 0xf000
	// Position of TX1 field.
	PIO0_FLEVEL_TX1_Pos = 0x8
	// Bit mask of TX1 field.
	PIO0_FLEVEL_TX1_Msk = 0xf00
	// Position of RX0 field.
	PIO0_FLEVEL_RX0_Pos = 0x4
	// Bit mask of RX0 field.
	PIO0_FLEVEL_RX0_Msk = 0xf0
	// Position of TX0 field.
	PIO0_FLEVEL_TX0_Pos = 0x0
	// Bit mask of TX0 field.
	PIO0_FLEVEL_TX0_Msk = 0xf

	// TXF0: Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
	// Position of TXF0 field.
	PIO0_TXF0_TXF0_Pos = 0x0
	// Bit mask of TXF0 field.
	PIO0_TXF0_TXF0_Msk = 0xffffffff

	// TXF1: Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
	// Position of TXF1 field.
	PIO0_TXF1_TXF1_Pos = 0x0
	// Bit mask of TXF1 field.
	PIO0_TXF1_TXF1_Msk = 0xffffffff

	// TXF2: Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
	// Position of TXF2 field.
	PIO0_TXF2_TXF2_Pos = 0x0
	// Bit mask of TXF2 field.
	PIO0_TXF2_TXF2_Msk = 0xffffffff

	// TXF3: Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
	// Position of TXF3 field.
	PIO0_TXF3_TXF3_Pos = 0x0
	// Bit mask of TXF3 field.
	PIO0_TXF3_TXF3_Msk = 0xffffffff

	// RXF0: Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
	// Position of RXF0 field.
	PIO0_RXF0_RXF0_Pos = 0x0
	// Bit mask of RXF0 field.
	PIO0_RXF0_RXF0_Msk = 0xffffffff

	// RXF1: Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
	// Position of RXF1 field.
	PIO0_RXF1_RXF1_Pos = 0x0
	// Bit mask of RXF1 field.
	PIO0_RXF1_RXF1_Msk = 0xffffffff

	// RXF2: Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
	// Position of RXF2 field.
	PIO0_RXF2_RXF2_Pos = 0x0
	// Bit mask of RXF2 field.
	PIO0_RXF2_RXF2_Msk = 0xffffffff

	// RXF3: Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
	// Position of RXF3 field.
	PIO0_RXF3_RXF3_Pos = 0x0
	// Bit mask of RXF3 field.
	PIO0_RXF3_RXF3_Msk = 0xffffffff

	// IRQ: State machine IRQ flags register. Write 1 to clear. There are eight state machine IRQ flags, which can be set, cleared, and waited on by the state machines. There's no fixed association between flags and state machines -- any state machine can use any flag. Any of the eight flags can be used for timing synchronisation between state machines, using IRQ and WAIT instructions. Any combination of the eight flags can also routed out to either of the two system-level interrupt requests, alongside FIFO status interrupts -- see e.g. IRQ0_INTE.
	// Position of IRQ field.
	PIO0_IRQ_IRQ_Pos = 0x0
	// Bit mask of IRQ field.
	PIO0_IRQ_IRQ_Msk = 0xff

	// IRQ_FORCE: Writing a 1 to each of these bits will forcibly assert the corresponding IRQ. Note this is different to the INTF register: writing here affects PIO internal state. INTF just asserts the processor-facing IRQ signal for testing ISRs, and is not visible to the state machines.
	// Position of IRQ_FORCE field.
	PIO0_IRQ_FORCE_IRQ_FORCE_Pos = 0x0
	// Bit mask of IRQ_FORCE field.
	PIO0_IRQ_FORCE_IRQ_FORCE_Msk = 0xff

	// INPUT_SYNC_BYPASS: There is a 2-flipflop synchronizer on each GPIO input, which protects PIO logic from metastabilities. This increases input delay, and for fast synchronous IO (e.g. SPI) these synchronizers may need to be bypassed. Each bit in this register corresponds to one GPIO. 0 -> input is synchronized (default) 1 -> synchronizer is bypassed If in doubt, leave this register as all zeroes.
	// Position of INPUT_SYNC_BYPASS field.
	PIO0_INPUT_SYNC_BYPASS_INPUT_SYNC_BYPASS_Pos = 0x0
	// Bit mask of INPUT_SYNC_BYPASS field.
	PIO0_INPUT_SYNC_BYPASS_INPUT_SYNC_BYPASS_Msk = 0xffffffff

	// DBG_PADOUT: Read to sample the pad output values PIO is currently driving to the GPIOs. On RP2040 there are 30 GPIOs, so the two most significant bits are hardwired to 0.
	// Position of DBG_PADOUT field.
	PIO0_DBG_PADOUT_DBG_PADOUT_Pos = 0x0
	// Bit mask of DBG_PADOUT field.
	PIO0_DBG_PADOUT_DBG_PADOUT_Msk = 0xffffffff

	// DBG_PADOE: Read to sample the pad output enables (direction) PIO is currently driving to the GPIOs. On RP2040 there are 30 GPIOs, so the two most significant bits are hardwired to 0.
	// Position of DBG_PADOE field.
	PIO0_DBG_PADOE_DBG_PADOE_Pos = 0x0
	// Bit mask of DBG_PADOE field.
	PIO0_DBG_PADOE_DBG_PADOE_Msk = 0xffffffff

	// DBG_CFGINFO: The PIO hardware has some free parameters that may vary between chip products. These should be provided in the chip datasheet, but are also exposed here.
	// Position of VERSION field.
	PIO0_DBG_CFGINFO_VERSION_Pos = 0x1c
	// Bit mask of VERSION field.
	PIO0_DBG_CFGINFO_VERSION_Msk = 0xf0000000
	// Version 0 (RP2040)
	PIO0_DBG_CFGINFO_VERSION_V0 = 0x0
	// Version 1 (RP2350)
	PIO0_DBG_CFGINFO_VERSION_V1 = 0x1
	// Position of IMEM_SIZE field.
	PIO0_DBG_CFGINFO_IMEM_SIZE_Pos = 0x10
	// Bit mask of IMEM_SIZE field.
	PIO0_DBG_CFGINFO_IMEM_SIZE_Msk = 0x3f0000
	// Position of SM_COUNT field.
	PIO0_DBG_CFGINFO_SM_COUNT_Pos = 0x8
	// Bit mask of SM_COUNT field.
	PIO0_DBG_CFGINFO_SM_COUNT_Msk = 0xf00
	// Position of FIFO_DEPTH field.
	PIO0_DBG_CFGINFO_FIFO_DEPTH_Pos = 0x0
	// Bit mask of FIFO_DEPTH field.
	PIO0_DBG_CFGINFO_FIFO_DEPTH_Msk = 0x3f

	// INSTR_MEM0: Write-only access to instruction memory location 0
	// Position of INSTR_MEM0 field.
	PIO0_INSTR_MEM0_INSTR_MEM0_Pos = 0x0
	// Bit mask of INSTR_MEM0 field.
	PIO0_INSTR_MEM0_INSTR_MEM0_Msk = 0xffff

	// INSTR_MEM1: Write-only access to instruction memory location 1
	// Position of INSTR_MEM1 field.
	PIO0_INSTR_MEM1_INSTR_MEM1_Pos = 0x0
	// Bit mask of INSTR_MEM1 field.
	PIO0_INSTR_MEM1_INSTR_MEM1_Msk = 0xffff

	// INSTR_MEM2: Write-only access to instruction memory location 2
	// Position of INSTR_MEM2 field.
	PIO0_INSTR_MEM2_INSTR_MEM2_Pos = 0x0
	// Bit mask of INSTR_MEM2 field.
	PIO0_INSTR_MEM2_INSTR_MEM2_Msk = 0xffff

	// INSTR_MEM3: Write-only access to instruction memory location 3
	// Position of INSTR_MEM3 field.
	PIO0_INSTR_MEM3_INSTR_MEM3_Pos = 0x0
	// Bit mask of INSTR_MEM3 field.
	PIO0_INSTR_MEM3_INSTR_MEM3_Msk = 0xffff

	// INSTR_MEM4: Write-only access to instruction memory location 4
	// Position of INSTR_MEM4 field.
	PIO0_INSTR_MEM4_INSTR_MEM4_Pos = 0x0
	// Bit mask of INSTR_MEM4 field.
	PIO0_INSTR_MEM4_INSTR_MEM4_Msk = 0xffff

	// INSTR_MEM5: Write-only access to instruction memory location 5
	// Position of INSTR_MEM5 field.
	PIO0_INSTR_MEM5_INSTR_MEM5_Pos = 0x0
	// Bit mask of INSTR_MEM5 field.
	PIO0_INSTR_MEM5_INSTR_MEM5_Msk = 0xffff

	// INSTR_MEM6: Write-only access to instruction memory location 6
	// Position of INSTR_MEM6 field.
	PIO0_INSTR_MEM6_INSTR_MEM6_Pos = 0x0
	// Bit mask of INSTR_MEM6 field.
	PIO0_INSTR_MEM6_INSTR_MEM6_Msk = 0xffff

	// INSTR_MEM7: Write-only access to instruction memory location 7
	// Position of INSTR_MEM7 field.
	PIO0_INSTR_MEM7_INSTR_MEM7_Pos = 0x0
	// Bit mask of INSTR_MEM7 field.
	PIO0_INSTR_MEM7_INSTR_MEM7_Msk = 0xffff

	// INSTR_MEM8: Write-only access to instruction memory location 8
	// Position of INSTR_MEM8 field.
	PIO0_INSTR_MEM8_INSTR_MEM8_Pos = 0x0
	// Bit mask of INSTR_MEM8 field.
	PIO0_INSTR_MEM8_INSTR_MEM8_Msk = 0xffff

	// INSTR_MEM9: Write-only access to instruction memory location 9
	// Position of INSTR_MEM9 field.
	PIO0_INSTR_MEM9_INSTR_MEM9_Pos = 0x0
	// Bit mask of INSTR_MEM9 field.
	PIO0_INSTR_MEM9_INSTR_MEM9_Msk = 0xffff

	// INSTR_MEM10: Write-only access to instruction memory location 10
	// Position of INSTR_MEM10 field.
	PIO0_INSTR_MEM10_INSTR_MEM10_Pos = 0x0
	// Bit mask of INSTR_MEM10 field.
	PIO0_INSTR_MEM10_INSTR_MEM10_Msk = 0xffff

	// INSTR_MEM11: Write-only access to instruction memory location 11
	// Position of INSTR_MEM11 field.
	PIO0_INSTR_MEM11_INSTR_MEM11_Pos = 0x0
	// Bit mask of INSTR_MEM11 field.
	PIO0_INSTR_MEM11_INSTR_MEM11_Msk = 0xffff

	// INSTR_MEM12: Write-only access to instruction memory location 12
	// Position of INSTR_MEM12 field.
	PIO0_INSTR_MEM12_INSTR_MEM12_Pos = 0x0
	// Bit mask of INSTR_MEM12 field.
	PIO0_INSTR_MEM12_INSTR_MEM12_Msk = 0xffff

	// INSTR_MEM13: Write-only access to instruction memory location 13
	// Position of INSTR_MEM13 field.
	PIO0_INSTR_MEM13_INSTR_MEM13_Pos = 0x0
	// Bit mask of INSTR_MEM13 field.
	PIO0_INSTR_MEM13_INSTR_MEM13_Msk = 0xffff

	// INSTR_MEM14: Write-only access to instruction memory location 14
	// Position of INSTR_MEM14 field.
	PIO0_INSTR_MEM14_INSTR_MEM14_Pos = 0x0
	// Bit mask of INSTR_MEM14 field.
	PIO0_INSTR_MEM14_INSTR_MEM14_Msk = 0xffff

	// INSTR_MEM15: Write-only access to instruction memory location 15
	// Position of INSTR_MEM15 field.
	PIO0_INSTR_MEM15_INSTR_MEM15_Pos = 0x0
	// Bit mask of INSTR_MEM15 field.
	PIO0_INSTR_MEM15_INSTR_MEM15_Msk = 0xffff

	// INSTR_MEM16: Write-only access to instruction memory location 16
	// Position of INSTR_MEM16 field.
	PIO0_INSTR_MEM16_INSTR_MEM16_Pos = 0x0
	// Bit mask of INSTR_MEM16 field.
	PIO0_INSTR_MEM16_INSTR_MEM16_Msk = 0xffff

	// INSTR_MEM17: Write-only access to instruction memory location 17
	// Position of INSTR_MEM17 field.
	PIO0_INSTR_MEM17_INSTR_MEM17_Pos = 0x0
	// Bit mask of INSTR_MEM17 field.
	PIO0_INSTR_MEM17_INSTR_MEM17_Msk = 0xffff

	// INSTR_MEM18: Write-only access to instruction memory location 18
	// Position of INSTR_MEM18 field.
	PIO0_INSTR_MEM18_INSTR_MEM18_Pos = 0x0
	// Bit mask of INSTR_MEM18 field.
	PIO0_INSTR_MEM18_INSTR_MEM18_Msk = 0xffff

	// INSTR_MEM19: Write-only access to instruction memory location 19
	// Position of INSTR_MEM19 field.
	PIO0_INSTR_MEM19_INSTR_MEM19_Pos = 0x0
	// Bit mask of INSTR_MEM19 field.
	PIO0_INSTR_MEM19_INSTR_MEM19_Msk = 0xffff

	// INSTR_MEM20: Write-only access to instruction memory location 20
	// Position of INSTR_MEM20 field.
	PIO0_INSTR_MEM20_INSTR_MEM20_Pos = 0x0
	// Bit mask of INSTR_MEM20 field.
	PIO0_INSTR_MEM20_INSTR_MEM20_Msk = 0xffff

	// INSTR_MEM21: Write-only access to instruction memory location 21
	// Position of INSTR_MEM21 field.
	PIO0_INSTR_MEM21_INSTR_MEM21_Pos = 0x0
	// Bit mask of INSTR_MEM21 field.
	PIO0_INSTR_MEM21_INSTR_MEM21_Msk = 0xffff

	// INSTR_MEM22: Write-only access to instruction memory location 22
	// Position of INSTR_MEM22 field.
	PIO0_INSTR_MEM22_INSTR_MEM22_Pos = 0x0
	// Bit mask of INSTR_MEM22 field.
	PIO0_INSTR_MEM22_INSTR_MEM22_Msk = 0xffff

	// INSTR_MEM23: Write-only access to instruction memory location 23
	// Position of INSTR_MEM23 field.
	PIO0_INSTR_MEM23_INSTR_MEM23_Pos = 0x0
	// Bit mask of INSTR_MEM23 field.
	PIO0_INSTR_MEM23_INSTR_MEM23_Msk = 0xffff

	// INSTR_MEM24: Write-only access to instruction memory location 24
	// Position of INSTR_MEM24 field.
	PIO0_INSTR_MEM24_INSTR_MEM24_Pos = 0x0
	// Bit mask of INSTR_MEM24 field.
	PIO0_INSTR_MEM24_INSTR_MEM24_Msk = 0xffff

	// INSTR_MEM25: Write-only access to instruction memory location 25
	// Position of INSTR_MEM25 field.
	PIO0_INSTR_MEM25_INSTR_MEM25_Pos = 0x0
	// Bit mask of INSTR_MEM25 field.
	PIO0_INSTR_MEM25_INSTR_MEM25_Msk = 0xffff

	// INSTR_MEM26: Write-only access to instruction memory location 26
	// Position of INSTR_MEM26 field.
	PIO0_INSTR_MEM26_INSTR_MEM26_Pos = 0x0
	// Bit mask of INSTR_MEM26 field.
	PIO0_INSTR_MEM26_INSTR_MEM26_Msk = 0xffff

	// INSTR_MEM27: Write-only access to instruction memory location 27
	// Position of INSTR_MEM27 field.
	PIO0_INSTR_MEM27_INSTR_MEM27_Pos = 0x0
	// Bit mask of INSTR_MEM27 field.
	PIO0_INSTR_MEM27_INSTR_MEM27_Msk = 0xffff

	// INSTR_MEM28: Write-only access to instruction memory location 28
	// Position of INSTR_MEM28 field.
	PIO0_INSTR_MEM28_INSTR_MEM28_Pos = 0x0
	// Bit mask of INSTR_MEM28 field.
	PIO0_INSTR_MEM28_INSTR_MEM28_Msk = 0xffff

	// INSTR_MEM29: Write-only access to instruction memory location 29
	// Position of INSTR_MEM29 field.
	PIO0_INSTR_MEM29_INSTR_MEM29_Pos = 0x0
	// Bit mask of INSTR_MEM29 field.
	PIO0_INSTR_MEM29_INSTR_MEM29_Msk = 0xffff

	// INSTR_MEM30: Write-only access to instruction memory location 30
	// Position of INSTR_MEM30 field.
	PIO0_INSTR_MEM30_INSTR_MEM30_Pos = 0x0
	// Bit mask of INSTR_MEM30 field.
	PIO0_INSTR_MEM30_INSTR_MEM30_Msk = 0xffff

	// INSTR_MEM31: Write-only access to instruction memory location 31
	// Position of INSTR_MEM31 field.
	PIO0_INSTR_MEM31_INSTR_MEM31_Pos = 0x0
	// Bit mask of INSTR_MEM31 field.
	PIO0_INSTR_MEM31_INSTR_MEM31_Msk = 0xffff

	// SM0_CLKDIV: Clock divisor register for state machine 0 Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
	// Position of INT field.
	PIO0_SM0_CLKDIV_INT_Pos = 0x10
	// Bit mask of INT field.
	PIO0_SM0_CLKDIV_INT_Msk = 0xffff0000
	// Position of FRAC field.
	PIO0_SM0_CLKDIV_FRAC_Pos = 0x8
	// Bit mask of FRAC field.
	PIO0_SM0_CLKDIV_FRAC_Msk = 0xff00

	// SM0_EXECCTRL: Execution/behavioural settings for state machine 0
	// Position of EXEC_STALLED field.
	PIO0_SM0_EXECCTRL_EXEC_STALLED_Pos = 0x1f
	// Bit mask of EXEC_STALLED field.
	PIO0_SM0_EXECCTRL_EXEC_STALLED_Msk = 0x80000000
	// Bit EXEC_STALLED.
	PIO0_SM0_EXECCTRL_EXEC_STALLED = 0x80000000
	// Position of SIDE_EN field.
	PIO0_SM0_EXECCTRL_SIDE_EN_Pos = 0x1e
	// Bit mask of SIDE_EN field.
	PIO0_SM0_EXECCTRL_SIDE_EN_Msk = 0x40000000
	// Bit SIDE_EN.
	PIO0_SM0_EXECCTRL_SIDE_EN = 0x40000000
	// Position of SIDE_PINDIR field.
	PIO0_SM0_EXECCTRL_SIDE_PINDIR_Pos = 0x1d
	// Bit mask of SIDE_PINDIR field.
	PIO0_SM0_EXECCTRL_SIDE_PINDIR_Msk = 0x20000000
	// Bit SIDE_PINDIR.
	PIO0_SM0_EXECCTRL_SIDE_PINDIR = 0x20000000
	// Position of JMP_PIN field.
	PIO0_SM0_EXECCTRL_JMP_PIN_Pos = 0x18
	// Bit mask of JMP_PIN field.
	PIO0_SM0_EXECCTRL_JMP_PIN_Msk = 0x1f000000
	// Position of OUT_EN_SEL field.
	PIO0_SM0_EXECCTRL_OUT_EN_SEL_Pos = 0x13
	// Bit mask of OUT_EN_SEL field.
	PIO0_SM0_EXECCTRL_OUT_EN_SEL_Msk = 0xf80000
	// Position of INLINE_OUT_EN field.
	PIO0_SM0_EXECCTRL_INLINE_OUT_EN_Pos = 0x12
	// Bit mask of INLINE_OUT_EN field.
	PIO0_SM0_EXECCTRL_INLINE_OUT_EN_Msk = 0x40000
	// Bit INLINE_OUT_EN.
	PIO0_SM0_EXECCTRL_INLINE_OUT_EN = 0x40000
	// Position of OUT_STICKY field.
	PIO0_SM0_EXECCTRL_OUT_STICKY_Pos = 0x11
	// Bit mask of OUT_STICKY field.
	PIO0_SM0_EXECCTRL_OUT_STICKY_Msk = 0x20000
	// Bit OUT_STICKY.
	PIO0_SM0_EXECCTRL_OUT_STICKY = 0x20000
	// Position of WRAP_TOP field.
	PIO0_SM0_EXECCTRL_WRAP_TOP_Pos = 0xc
	// Bit mask of WRAP_TOP field.
	PIO0_SM0_EXECCTRL_WRAP_TOP_Msk = 0x1f000
	// Position of WRAP_BOTTOM field.
	PIO0_SM0_EXECCTRL_WRAP_BOTTOM_Pos = 0x7
	// Bit mask of WRAP_BOTTOM field.
	PIO0_SM0_EXECCTRL_WRAP_BOTTOM_Msk = 0xf80
	// Position of STATUS_SEL field.
	PIO0_SM0_EXECCTRL_STATUS_SEL_Pos = 0x5
	// Bit mask of STATUS_SEL field.
	PIO0_SM0_EXECCTRL_STATUS_SEL_Msk = 0x60
	// All-ones if TX FIFO level < N, otherwise all-zeroes
	PIO0_SM0_EXECCTRL_STATUS_SEL_TXLEVEL = 0x0
	// All-ones if RX FIFO level < N, otherwise all-zeroes
	PIO0_SM0_EXECCTRL_STATUS_SEL_RXLEVEL = 0x1
	// All-ones if the indexed IRQ flag is raised, otherwise all-zeroes
	PIO0_SM0_EXECCTRL_STATUS_SEL_IRQ = 0x2
	// Position of STATUS_N field.
	PIO0_SM0_EXECCTRL_STATUS_N_Pos = 0x0
	// Bit mask of STATUS_N field.
	PIO0_SM0_EXECCTRL_STATUS_N_Msk = 0x1f
	// Index 0-7 of an IRQ flag in this PIO block
	PIO0_SM0_EXECCTRL_STATUS_N_IRQ = 0x0
	// Index 0-7 of an IRQ flag in the next lower-numbered PIO block
	PIO0_SM0_EXECCTRL_STATUS_N_IRQ_PREVPIO = 0x8
	// Index 0-7 of an IRQ flag in the next higher-numbered PIO block
	PIO0_SM0_EXECCTRL_STATUS_N_IRQ_NEXTPIO = 0x10

	// SM0_SHIFTCTRL: Control behaviour of the input/output shift registers for state machine 0
	// Position of FJOIN_RX field.
	PIO0_SM0_SHIFTCTRL_FJOIN_RX_Pos = 0x1f
	// Bit mask of FJOIN_RX field.
	PIO0_SM0_SHIFTCTRL_FJOIN_RX_Msk = 0x80000000
	// Bit FJOIN_RX.
	PIO0_SM0_SHIFTCTRL_FJOIN_RX = 0x80000000
	// Position of FJOIN_TX field.
	PIO0_SM0_SHIFTCTRL_FJOIN_TX_Pos = 0x1e
	// Bit mask of FJOIN_TX field.
	PIO0_SM0_SHIFTCTRL_FJOIN_TX_Msk = 0x40000000
	// Bit FJOIN_TX.
	PIO0_SM0_SHIFTCTRL_FJOIN_TX = 0x40000000
	// Position of PULL_THRESH field.
	PIO0_SM0_SHIFTCTRL_PULL_THRESH_Pos = 0x19
	// Bit mask of PULL_THRESH field.
	PIO0_SM0_SHIFTCTRL_PULL_THRESH_Msk = 0x3e000000
	// Position of PUSH_THRESH field.
	PIO0_SM0_SHIFTCTRL_PUSH_THRESH_Pos = 0x14
	// Bit mask of PUSH_THRESH field.
	PIO0_SM0_SHIFTCTRL_PUSH_THRESH_Msk = 0x1f00000
	// Position of OUT_SHIFTDIR field.
	PIO0_SM0_SHIFTCTRL_OUT_SHIFTDIR_Pos = 0x13
	// Bit mask of OUT_SHIFTDIR field.
	PIO0_SM0_SHIFTCTRL_OUT_SHIFTDIR_Msk = 0x80000
	// Bit OUT_SHIFTDIR.
	PIO0_SM0_SHIFTCTRL_OUT_SHIFTDIR = 0x80000
	// Position of IN_SHIFTDIR field.
	PIO0_SM0_SHIFTCTRL_IN_SHIFTDIR_Pos = 0x12
	// Bit mask of IN_SHIFTDIR field.
	PIO0_SM0_SHIFTCTRL_IN_SHIFTDIR_Msk = 0x40000
	// Bit IN_SHIFTDIR.
	PIO0_SM0_SHIFTCTRL_IN_SHIFTDIR = 0x40000
	// Position of AUTOPULL field.
	PIO0_SM0_SHIFTCTRL_AUTOPULL_Pos = 0x11
	// Bit mask of AUTOPULL field.
	PIO0_SM0_SHIFTCTRL_AUTOPULL_Msk = 0x20000
	// Bit AUTOPULL.
	PIO0_SM0_SHIFTCTRL_AUTOPULL = 0x20000
	// Position of AUTOPUSH field.
	PIO0_SM0_SHIFTCTRL_AUTOPUSH_Pos = 0x10
	// Bit mask of AUTOPUSH field.
	PIO0_SM0_SHIFTCTRL_AUTOPUSH_Msk = 0x10000
	// Bit AUTOPUSH.
	PIO0_SM0_SHIFTCTRL_AUTOPUSH = 0x10000
	// Position of FJOIN_RX_PUT field.
	PIO0_SM0_SHIFTCTRL_FJOIN_RX_PUT_Pos = 0xf
	// Bit mask of FJOIN_RX_PUT field.
	PIO0_SM0_SHIFTCTRL_FJOIN_RX_PUT_Msk = 0x8000
	// Bit FJOIN_RX_PUT.
	PIO0_SM0_SHIFTCTRL_FJOIN_RX_PUT = 0x8000
	// Position of FJOIN_RX_GET field.
	PIO0_SM0_SHIFTCTRL_FJOIN_RX_GET_Pos = 0xe
	// Bit mask of FJOIN_RX_GET field.
	PIO0_SM0_SHIFTCTRL_FJOIN_RX_GET_Msk = 0x4000
	// Bit FJOIN_RX_GET.
	PIO0_SM0_SHIFTCTRL_FJOIN_RX_GET = 0x4000
	// Position of IN_COUNT field.
	PIO0_SM0_SHIFTCTRL_IN_COUNT_Pos = 0x0
	// Bit mask of IN_COUNT field.
	PIO0_SM0_SHIFTCTRL_IN_COUNT_Msk = 0x1f

	// SM0_ADDR: Current instruction address of state machine 0
	// Position of SM0_ADDR field.
	PIO0_SM0_ADDR_SM0_ADDR_Pos = 0x0
	// Bit mask of SM0_ADDR field.
	PIO0_SM0_ADDR_SM0_ADDR_Msk = 0x1f

	// SM0_INSTR: Read to see the instruction currently addressed by state machine 0's program counter Write to execute an instruction immediately (including jumps) and then resume execution.
	// Position of SM0_INSTR field.
	PIO0_SM0_INSTR_SM0_INSTR_Pos = 0x0
	// Bit mask of SM0_INSTR field.
	PIO0_SM0_INSTR_SM0_INSTR_Msk = 0xffff

	// SM0_PINCTRL: State machine pin control
	// Position of SIDESET_COUNT field.
	PIO0_SM0_PINCTRL_SIDESET_COUNT_Pos = 0x1d
	// Bit mask of SIDESET_COUNT field.
	PIO0_SM0_PINCTRL_SIDESET_COUNT_Msk = 0xe0000000
	// Position of SET_COUNT field.
	PIO0_SM0_PINCTRL_SET_COUNT_Pos = 0x1a
	// Bit mask of SET_COUNT field.
	PIO0_SM0_PINCTRL_SET_COUNT_Msk = 0x1c000000
	// Position of OUT_COUNT field.
	PIO0_SM0_PINCTRL_OUT_COUNT_Pos = 0x14
	// Bit mask of OUT_COUNT field.
	PIO0_SM0_PINCTRL_OUT_COUNT_Msk = 0x3f00000
	// Position of IN_BASE field.
	PIO0_SM0_PINCTRL_IN_BASE_Pos = 0xf
	// Bit mask of IN_BASE field.
	PIO0_SM0_PINCTRL_IN_BASE_Msk = 0xf8000
	// Position of SIDESET_BASE field.
	PIO0_SM0_PINCTRL_SIDESET_BASE_Pos = 0xa
	// Bit mask of SIDESET_BASE field.
	PIO0_SM0_PINCTRL_SIDESET_BASE_Msk = 0x7c00
	// Position of SET_BASE field.
	PIO0_SM0_PINCTRL_SET_BASE_Pos = 0x5
	// Bit mask of SET_BASE field.
	PIO0_SM0_PINCTRL_SET_BASE_Msk = 0x3e0
	// Position of OUT_BASE field.
	PIO0_SM0_PINCTRL_OUT_BASE_Pos = 0x0
	// Bit mask of OUT_BASE field.
	PIO0_SM0_PINCTRL_OUT_BASE_Msk = 0x1f

	// SM1_CLKDIV: Clock divisor register for state machine 1 Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
	// Position of INT field.
	PIO0_SM1_CLKDIV_INT_Pos = 0x10
	// Bit mask of INT field.
	PIO0_SM1_CLKDIV_INT_Msk = 0xffff0000
	// Position of FRAC field.
	PIO0_SM1_CLKDIV_FRAC_Pos = 0x8
	// Bit mask of FRAC field.
	PIO0_SM1_CLKDIV_FRAC_Msk = 0xff00

	// SM1_EXECCTRL: Execution/behavioural settings for state machine 1
	// Position of EXEC_STALLED field.
	PIO0_SM1_EXECCTRL_EXEC_STALLED_Pos = 0x1f
	// Bit mask of EXEC_STALLED field.
	PIO0_SM1_EXECCTRL_EXEC_STALLED_Msk = 0x80000000
	// Bit EXEC_STALLED.
	PIO0_SM1_EXECCTRL_EXEC_STALLED = 0x80000000
	// Position of SIDE_EN field.
	PIO0_SM1_EXECCTRL_SIDE_EN_Pos = 0x1e
	// Bit mask of SIDE_EN field.
	PIO0_SM1_EXECCTRL_SIDE_EN_Msk = 0x40000000
	// Bit SIDE_EN.
	PIO0_SM1_EXECCTRL_SIDE_EN = 0x40000000
	// Position of SIDE_PINDIR field.
	PIO0_SM1_EXECCTRL_SIDE_PINDIR_Pos = 0x1d
	// Bit mask of SIDE_PINDIR field.
	PIO0_SM1_EXECCTRL_SIDE_PINDIR_Msk = 0x20000000
	// Bit SIDE_PINDIR.
	PIO0_SM1_EXECCTRL_SIDE_PINDIR = 0x20000000
	// Position of JMP_PIN field.
	PIO0_SM1_EXECCTRL_JMP_PIN_Pos = 0x18
	// Bit mask of JMP_PIN field.
	PIO0_SM1_EXECCTRL_JMP_PIN_Msk = 0x1f000000
	// Position of OUT_EN_SEL field.
	PIO0_SM1_EXECCTRL_OUT_EN_SEL_Pos = 0x13
	// Bit mask of OUT_EN_SEL field.
	PIO0_SM1_EXECCTRL_OUT_EN_SEL_Msk = 0xf80000
	// Position of INLINE_OUT_EN field.
	PIO0_SM1_EXECCTRL_INLINE_OUT_EN_Pos = 0x12
	// Bit mask of INLINE_OUT_EN field.
	PIO0_SM1_EXECCTRL_INLINE_OUT_EN_Msk = 0x40000
	// Bit INLINE_OUT_EN.
	PIO0_SM1_EXECCTRL_INLINE_OUT_EN = 0x40000
	// Position of OUT_STICKY field.
	PIO0_SM1_EXECCTRL_OUT_STICKY_Pos = 0x11
	// Bit mask of OUT_STICKY field.
	PIO0_SM1_EXECCTRL_OUT_STICKY_Msk = 0x20000
	// Bit OUT_STICKY.
	PIO0_SM1_EXECCTRL_OUT_STICKY = 0x20000
	// Position of WRAP_TOP field.
	PIO0_SM1_EXECCTRL_WRAP_TOP_Pos = 0xc
	// Bit mask of WRAP_TOP field.
	PIO0_SM1_EXECCTRL_WRAP_TOP_Msk = 0x1f000
	// Position of WRAP_BOTTOM field.
	PIO0_SM1_EXECCTRL_WRAP_BOTTOM_Pos = 0x7
	// Bit mask of WRAP_BOTTOM field.
	PIO0_SM1_EXECCTRL_WRAP_BOTTOM_Msk = 0xf80
	// Position of STATUS_SEL field.
	PIO0_SM1_EXECCTRL_STATUS_SEL_Pos = 0x5
	// Bit mask of STATUS_SEL field.
	PIO0_SM1_EXECCTRL_STATUS_SEL_Msk = 0x60
	// All-ones if TX FIFO level < N, otherwise all-zeroes
	PIO0_SM1_EXECCTRL_STATUS_SEL_TXLEVEL = 0x0
	// All-ones if RX FIFO level < N, otherwise all-zeroes
	PIO0_SM1_EXECCTRL_STATUS_SEL_RXLEVEL = 0x1
	// All-ones if the indexed IRQ flag is raised, otherwise all-zeroes
	PIO0_SM1_EXECCTRL_STATUS_SEL_IRQ = 0x2
	// Position of STATUS_N field.
	PIO0_SM1_EXECCTRL_STATUS_N_Pos = 0x0
	// Bit mask of STATUS_N field.
	PIO0_SM1_EXECCTRL_STATUS_N_Msk = 0x1f
	// Index 0-7 of an IRQ flag in this PIO block
	PIO0_SM1_EXECCTRL_STATUS_N_IRQ = 0x0
	// Index 0-7 of an IRQ flag in the next lower-numbered PIO block
	PIO0_SM1_EXECCTRL_STATUS_N_IRQ_PREVPIO = 0x8
	// Index 0-7 of an IRQ flag in the next higher-numbered PIO block
	PIO0_SM1_EXECCTRL_STATUS_N_IRQ_NEXTPIO = 0x10

	// SM1_SHIFTCTRL: Control behaviour of the input/output shift registers for state machine 1
	// Position of FJOIN_RX field.
	PIO0_SM1_SHIFTCTRL_FJOIN_RX_Pos = 0x1f
	// Bit mask of FJOIN_RX field.
	PIO0_SM1_SHIFTCTRL_FJOIN_RX_Msk = 0x80000000
	// Bit FJOIN_RX.
	PIO0_SM1_SHIFTCTRL_FJOIN_RX = 0x80000000
	// Position of FJOIN_TX field.
	PIO0_SM1_SHIFTCTRL_FJOIN_TX_Pos = 0x1e
	// Bit mask of FJOIN_TX field.
	PIO0_SM1_SHIFTCTRL_FJOIN_TX_Msk = 0x40000000
	// Bit FJOIN_TX.
	PIO0_SM1_SHIFTCTRL_FJOIN_TX = 0x40000000
	// Position of PULL_THRESH field.
	PIO0_SM1_SHIFTCTRL_PULL_THRESH_Pos = 0x19
	// Bit mask of PULL_THRESH field.
	PIO0_SM1_SHIFTCTRL_PULL_THRESH_Msk = 0x3e000000
	// Position of PUSH_THRESH field.
	PIO0_SM1_SHIFTCTRL_PUSH_THRESH_Pos = 0x14
	// Bit mask of PUSH_THRESH field.
	PIO0_SM1_SHIFTCTRL_PUSH_THRESH_Msk = 0x1f00000
	// Position of OUT_SHIFTDIR field.
	PIO0_SM1_SHIFTCTRL_OUT_SHIFTDIR_Pos = 0x13
	// Bit mask of OUT_SHIFTDIR field.
	PIO0_SM1_SHIFTCTRL_OUT_SHIFTDIR_Msk = 0x80000
	// Bit OUT_SHIFTDIR.
	PIO0_SM1_SHIFTCTRL_OUT_SHIFTDIR = 0x80000
	// Position of IN_SHIFTDIR field.
	PIO0_SM1_SHIFTCTRL_IN_SHIFTDIR_Pos = 0x12
	// Bit mask of IN_SHIFTDIR field.
	PIO0_SM1_SHIFTCTRL_IN_SHIFTDIR_Msk = 0x40000
	// Bit IN_SHIFTDIR.
	PIO0_SM1_SHIFTCTRL_IN_SHIFTDIR = 0x40000
	// Position of AUTOPULL field.
	PIO0_SM1_SHIFTCTRL_AUTOPULL_Pos = 0x11
	// Bit mask of AUTOPULL field.
	PIO0_SM1_SHIFTCTRL_AUTOPULL_Msk = 0x20000
	// Bit AUTOPULL.
	PIO0_SM1_SHIFTCTRL_AUTOPULL = 0x20000
	// Position of AUTOPUSH field.
	PIO0_SM1_SHIFTCTRL_AUTOPUSH_Pos = 0x10
	// Bit mask of AUTOPUSH field.
	PIO0_SM1_SHIFTCTRL_AUTOPUSH_Msk = 0x10000
	// Bit AUTOPUSH.
	PIO0_SM1_SHIFTCTRL_AUTOPUSH = 0x10000
	// Position of FJOIN_RX_PUT field.
	PIO0_SM1_SHIFTCTRL_FJOIN_RX_PUT_Pos = 0xf
	// Bit mask of FJOIN_RX_PUT field.
	PIO0_SM1_SHIFTCTRL_FJOIN_RX_PUT_Msk = 0x8000
	// Bit FJOIN_RX_PUT.
	PIO0_SM1_SHIFTCTRL_FJOIN_RX_PUT = 0x8000
	// Position of FJOIN_RX_GET field.
	PIO0_SM1_SHIFTCTRL_FJOIN_RX_GET_Pos = 0xe
	// Bit mask of FJOIN_RX_GET field.
	PIO0_SM1_SHIFTCTRL_FJOIN_RX_GET_Msk = 0x4000
	// Bit FJOIN_RX_GET.
	PIO0_SM1_SHIFTCTRL_FJOIN_RX_GET = 0x4000
	// Position of IN_COUNT field.
	PIO0_SM1_SHIFTCTRL_IN_COUNT_Pos = 0x0
	// Bit mask of IN_COUNT field.
	PIO0_SM1_SHIFTCTRL_IN_COUNT_Msk = 0x1f

	// SM1_ADDR: Current instruction address of state machine 1
	// Position of SM1_ADDR field.
	PIO0_SM1_ADDR_SM1_ADDR_Pos = 0x0
	// Bit mask of SM1_ADDR field.
	PIO0_SM1_ADDR_SM1_ADDR_Msk = 0x1f

	// SM1_INSTR: Read to see the instruction currently addressed by state machine 1's program counter Write to execute an instruction immediately (including jumps) and then resume execution.
	// Position of SM1_INSTR field.
	PIO0_SM1_INSTR_SM1_INSTR_Pos = 0x0
	// Bit mask of SM1_INSTR field.
	PIO0_SM1_INSTR_SM1_INSTR_Msk = 0xffff

	// SM1_PINCTRL: State machine pin control
	// Position of SIDESET_COUNT field.
	PIO0_SM1_PINCTRL_SIDESET_COUNT_Pos = 0x1d
	// Bit mask of SIDESET_COUNT field.
	PIO0_SM1_PINCTRL_SIDESET_COUNT_Msk = 0xe0000000
	// Position of SET_COUNT field.
	PIO0_SM1_PINCTRL_SET_COUNT_Pos = 0x1a
	// Bit mask of SET_COUNT field.
	PIO0_SM1_PINCTRL_SET_COUNT_Msk = 0x1c000000
	// Position of OUT_COUNT field.
	PIO0_SM1_PINCTRL_OUT_COUNT_Pos = 0x14
	// Bit mask of OUT_COUNT field.
	PIO0_SM1_PINCTRL_OUT_COUNT_Msk = 0x3f00000
	// Position of IN_BASE field.
	PIO0_SM1_PINCTRL_IN_BASE_Pos = 0xf
	// Bit mask of IN_BASE field.
	PIO0_SM1_PINCTRL_IN_BASE_Msk = 0xf8000
	// Position of SIDESET_BASE field.
	PIO0_SM1_PINCTRL_SIDESET_BASE_Pos = 0xa
	// Bit mask of SIDESET_BASE field.
	PIO0_SM1_PINCTRL_SIDESET_BASE_Msk = 0x7c00
	// Position of SET_BASE field.
	PIO0_SM1_PINCTRL_SET_BASE_Pos = 0x5
	// Bit mask of SET_BASE field.
	PIO0_SM1_PINCTRL_SET_BASE_Msk = 0x3e0
	// Position of OUT_BASE field.
	PIO0_SM1_PINCTRL_OUT_BASE_Pos = 0x0
	// Bit mask of OUT_BASE field.
	PIO0_SM1_PINCTRL_OUT_BASE_Msk = 0x1f

	// SM2_CLKDIV: Clock divisor register for state machine 2 Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
	// Position of INT field.
	PIO0_SM2_CLKDIV_INT_Pos = 0x10
	// Bit mask of INT field.
	PIO0_SM2_CLKDIV_INT_Msk = 0xffff0000
	// Position of FRAC field.
	PIO0_SM2_CLKDIV_FRAC_Pos = 0x8
	// Bit mask of FRAC field.
	PIO0_SM2_CLKDIV_FRAC_Msk = 0xff00

	// SM2_EXECCTRL: Execution/behavioural settings for state machine 2
	// Position of EXEC_STALLED field.
	PIO0_SM2_EXECCTRL_EXEC_STALLED_Pos = 0x1f
	// Bit mask of EXEC_STALLED field.
	PIO0_SM2_EXECCTRL_EXEC_STALLED_Msk = 0x80000000
	// Bit EXEC_STALLED.
	PIO0_SM2_EXECCTRL_EXEC_STALLED = 0x80000000
	// Position of SIDE_EN field.
	PIO0_SM2_EXECCTRL_SIDE_EN_Pos = 0x1e
	// Bit mask of SIDE_EN field.
	PIO0_SM2_EXECCTRL_SIDE_EN_Msk = 0x40000000
	// Bit SIDE_EN.
	PIO0_SM2_EXECCTRL_SIDE_EN = 0x40000000
	// Position of SIDE_PINDIR field.
	PIO0_SM2_EXECCTRL_SIDE_PINDIR_Pos = 0x1d
	// Bit mask of SIDE_PINDIR field.
	PIO0_SM2_EXECCTRL_SIDE_PINDIR_Msk = 0x20000000
	// Bit SIDE_PINDIR.
	PIO0_SM2_EXECCTRL_SIDE_PINDIR = 0x20000000
	// Position of JMP_PIN field.
	PIO0_SM2_EXECCTRL_JMP_PIN_Pos = 0x18
	// Bit mask of JMP_PIN field.
	PIO0_SM2_EXECCTRL_JMP_PIN_Msk = 0x1f000000
	// Position of OUT_EN_SEL field.
	PIO0_SM2_EXECCTRL_OUT_EN_SEL_Pos = 0x13
	// Bit mask of OUT_EN_SEL field.
	PIO0_SM2_EXECCTRL_OUT_EN_SEL_Msk = 0xf80000
	// Position of INLINE_OUT_EN field.
	PIO0_SM2_EXECCTRL_INLINE_OUT_EN_Pos = 0x12
	// Bit mask of INLINE_OUT_EN field.
	PIO0_SM2_EXECCTRL_INLINE_OUT_EN_Msk = 0x40000
	// Bit INLINE_OUT_EN.
	PIO0_SM2_EXECCTRL_INLINE_OUT_EN = 0x40000
	// Position of OUT_STICKY field.
	PIO0_SM2_EXECCTRL_OUT_STICKY_Pos = 0x11
	// Bit mask of OUT_STICKY field.
	PIO0_SM2_EXECCTRL_OUT_STICKY_Msk = 0x20000
	// Bit OUT_STICKY.
	PIO0_SM2_EXECCTRL_OUT_STICKY = 0x20000
	// Position of WRAP_TOP field.
	PIO0_SM2_EXECCTRL_WRAP_TOP_Pos = 0xc
	// Bit mask of WRAP_TOP field.
	PIO0_SM2_EXECCTRL_WRAP_TOP_Msk = 0x1f000
	// Position of WRAP_BOTTOM field.
	PIO0_SM2_EXECCTRL_WRAP_BOTTOM_Pos = 0x7
	// Bit mask of WRAP_BOTTOM field.
	PIO0_SM2_EXECCTRL_WRAP_BOTTOM_Msk = 0xf80
	// Position of STATUS_SEL field.
	PIO0_SM2_EXECCTRL_STATUS_SEL_Pos = 0x5
	// Bit mask of STATUS_SEL field.
	PIO0_SM2_EXECCTRL_STATUS_SEL_Msk = 0x60
	// All-ones if TX FIFO level < N, otherwise all-zeroes
	PIO0_SM2_EXECCTRL_STATUS_SEL_TXLEVEL = 0x0
	// All-ones if RX FIFO level < N, otherwise all-zeroes
	PIO0_SM2_EXECCTRL_STATUS_SEL_RXLEVEL = 0x1
	// All-ones if the indexed IRQ flag is raised, otherwise all-zeroes
	PIO0_SM2_EXECCTRL_STATUS_SEL_IRQ = 0x2
	// Position of STATUS_N field.
	PIO0_SM2_EXECCTRL_STATUS_N_Pos = 0x0
	// Bit mask of STATUS_N field.
	PIO0_SM2_EXECCTRL_STATUS_N_Msk = 0x1f
	// Index 0-7 of an IRQ flag in this PIO block
	PIO0_SM2_EXECCTRL_STATUS_N_IRQ = 0x0
	// Index 0-7 of an IRQ flag in the next lower-numbered PIO block
	PIO0_SM2_EXECCTRL_STATUS_N_IRQ_PREVPIO = 0x8
	// Index 0-7 of an IRQ flag in the next higher-numbered PIO block
	PIO0_SM2_EXECCTRL_STATUS_N_IRQ_NEXTPIO = 0x10

	// SM2_SHIFTCTRL: Control behaviour of the input/output shift registers for state machine 2
	// Position of FJOIN_RX field.
	PIO0_SM2_SHIFTCTRL_FJOIN_RX_Pos = 0x1f
	// Bit mask of FJOIN_RX field.
	PIO0_SM2_SHIFTCTRL_FJOIN_RX_Msk = 0x80000000
	// Bit FJOIN_RX.
	PIO0_SM2_SHIFTCTRL_FJOIN_RX = 0x80000000
	// Position of FJOIN_TX field.
	PIO0_SM2_SHIFTCTRL_FJOIN_TX_Pos = 0x1e
	// Bit mask of FJOIN_TX field.
	PIO0_SM2_SHIFTCTRL_FJOIN_TX_Msk = 0x40000000
	// Bit FJOIN_TX.
	PIO0_SM2_SHIFTCTRL_FJOIN_TX = 0x40000000
	// Position of PULL_THRESH field.
	PIO0_SM2_SHIFTCTRL_PULL_THRESH_Pos = 0x19
	// Bit mask of PULL_THRESH field.
	PIO0_SM2_SHIFTCTRL_PULL_THRESH_Msk = 0x3e000000
	// Position of PUSH_THRESH field.
	PIO0_SM2_SHIFTCTRL_PUSH_THRESH_Pos = 0x14
	// Bit mask of PUSH_THRESH field.
	PIO0_SM2_SHIFTCTRL_PUSH_THRESH_Msk = 0x1f00000
	// Position of OUT_SHIFTDIR field.
	PIO0_SM2_SHIFTCTRL_OUT_SHIFTDIR_Pos = 0x13
	// Bit mask of OUT_SHIFTDIR field.
	PIO0_SM2_SHIFTCTRL_OUT_SHIFTDIR_Msk = 0x80000
	// Bit OUT_SHIFTDIR.
	PIO0_SM2_SHIFTCTRL_OUT_SHIFTDIR = 0x80000
	// Position of IN_SHIFTDIR field.
	PIO0_SM2_SHIFTCTRL_IN_SHIFTDIR_Pos = 0x12
	// Bit mask of IN_SHIFTDIR field.
	PIO0_SM2_SHIFTCTRL_IN_SHIFTDIR_Msk = 0x40000
	// Bit IN_SHIFTDIR.
	PIO0_SM2_SHIFTCTRL_IN_SHIFTDIR = 0x40000
	// Position of AUTOPULL field.
	PIO0_SM2_SHIFTCTRL_AUTOPULL_Pos = 0x11
	// Bit mask of AUTOPULL field.
	PIO0_SM2_SHIFTCTRL_AUTOPULL_Msk = 0x20000
	// Bit AUTOPULL.
	PIO0_SM2_SHIFTCTRL_AUTOPULL = 0x20000
	// Position of AUTOPUSH field.
	PIO0_SM2_SHIFTCTRL_AUTOPUSH_Pos = 0x10
	// Bit mask of AUTOPUSH field.
	PIO0_SM2_SHIFTCTRL_AUTOPUSH_Msk = 0x10000
	// Bit AUTOPUSH.
	PIO0_SM2_SHIFTCTRL_AUTOPUSH = 0x10000
	// Position of FJOIN_RX_PUT field.
	PIO0_SM2_SHIFTCTRL_FJOIN_RX_PUT_Pos = 0xf
	// Bit mask of FJOIN_RX_PUT field.
	PIO0_SM2_SHIFTCTRL_FJOIN_RX_PUT_Msk = 0x8000
	// Bit FJOIN_RX_PUT.
	PIO0_SM2_SHIFTCTRL_FJOIN_RX_PUT = 0x8000
	// Position of FJOIN_RX_GET field.
	PIO0_SM2_SHIFTCTRL_FJOIN_RX_GET_Pos = 0xe
	// Bit mask of FJOIN_RX_GET field.
	PIO0_SM2_SHIFTCTRL_FJOIN_RX_GET_Msk = 0x4000
	// Bit FJOIN_RX_GET.
	PIO0_SM2_SHIFTCTRL_FJOIN_RX_GET = 0x4000
	// Position of IN_COUNT field.
	PIO0_SM2_SHIFTCTRL_IN_COUNT_Pos = 0x0
	// Bit mask of IN_COUNT field.
	PIO0_SM2_SHIFTCTRL_IN_COUNT_Msk = 0x1f

	// SM2_ADDR: Current instruction address of state machine 2
	// Position of SM2_ADDR field.
	PIO0_SM2_ADDR_SM2_ADDR_Pos = 0x0
	// Bit mask of SM2_ADDR field.
	PIO0_SM2_ADDR_SM2_ADDR_Msk = 0x1f

	// SM2_INSTR: Read to see the instruction currently addressed by state machine 2's program counter Write to execute an instruction immediately (including jumps) and then resume execution.
	// Position of SM2_INSTR field.
	PIO0_SM2_INSTR_SM2_INSTR_Pos = 0x0
	// Bit mask of SM2_INSTR field.
	PIO0_SM2_INSTR_SM2_INSTR_Msk = 0xffff

	// SM2_PINCTRL: State machine pin control
	// Position of SIDESET_COUNT field.
	PIO0_SM2_PINCTRL_SIDESET_COUNT_Pos = 0x1d
	// Bit mask of SIDESET_COUNT field.
	PIO0_SM2_PINCTRL_SIDESET_COUNT_Msk = 0xe0000000
	// Position of SET_COUNT field.
	PIO0_SM2_PINCTRL_SET_COUNT_Pos = 0x1a
	// Bit mask of SET_COUNT field.
	PIO0_SM2_PINCTRL_SET_COUNT_Msk = 0x1c000000
	// Position of OUT_COUNT field.
	PIO0_SM2_PINCTRL_OUT_COUNT_Pos = 0x14
	// Bit mask of OUT_COUNT field.
	PIO0_SM2_PINCTRL_OUT_COUNT_Msk = 0x3f00000
	// Position of IN_BASE field.
	PIO0_SM2_PINCTRL_IN_BASE_Pos = 0xf
	// Bit mask of IN_BASE field.
	PIO0_SM2_PINCTRL_IN_BASE_Msk = 0xf8000
	// Position of SIDESET_BASE field.
	PIO0_SM2_PINCTRL_SIDESET_BASE_Pos = 0xa
	// Bit mask of SIDESET_BASE field.
	PIO0_SM2_PINCTRL_SIDESET_BASE_Msk = 0x7c00
	// Position of SET_BASE field.
	PIO0_SM2_PINCTRL_SET_BASE_Pos = 0x5
	// Bit mask of SET_BASE field.
	PIO0_SM2_PINCTRL_SET_BASE_Msk = 0x3e0
	// Position of OUT_BASE field.
	PIO0_SM2_PINCTRL_OUT_BASE_Pos = 0x0
	// Bit mask of OUT_BASE field.
	PIO0_SM2_PINCTRL_OUT_BASE_Msk = 0x1f

	// SM3_CLKDIV: Clock divisor register for state machine 3 Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
	// Position of INT field.
	PIO0_SM3_CLKDIV_INT_Pos = 0x10
	// Bit mask of INT field.
	PIO0_SM3_CLKDIV_INT_Msk = 0xffff0000
	// Position of FRAC field.
	PIO0_SM3_CLKDIV_FRAC_Pos = 0x8
	// Bit mask of FRAC field.
	PIO0_SM3_CLKDIV_FRAC_Msk = 0xff00

	// SM3_EXECCTRL: Execution/behavioural settings for state machine 3
	// Position of EXEC_STALLED field.
	PIO0_SM3_EXECCTRL_EXEC_STALLED_Pos = 0x1f
	// Bit mask of EXEC_STALLED field.
	PIO0_SM3_EXECCTRL_EXEC_STALLED_Msk = 0x80000000
	// Bit EXEC_STALLED.
	PIO0_SM3_EXECCTRL_EXEC_STALLED = 0x80000000
	// Position of SIDE_EN field.
	PIO0_SM3_EXECCTRL_SIDE_EN_Pos = 0x1e
	// Bit mask of SIDE_EN field.
	PIO0_SM3_EXECCTRL_SIDE_EN_Msk = 0x40000000
	// Bit SIDE_EN.
	PIO0_SM3_EXECCTRL_SIDE_EN = 0x40000000
	// Position of SIDE_PINDIR field.
	PIO0_SM3_EXECCTRL_SIDE_PINDIR_Pos = 0x1d
	// Bit mask of SIDE_PINDIR field.
	PIO0_SM3_EXECCTRL_SIDE_PINDIR_Msk = 0x20000000
	// Bit SIDE_PINDIR.
	PIO0_SM3_EXECCTRL_SIDE_PINDIR = 0x20000000
	// Position of JMP_PIN field.
	PIO0_SM3_EXECCTRL_JMP_PIN_Pos = 0x18
	// Bit mask of JMP_PIN field.
	PIO0_SM3_EXECCTRL_JMP_PIN_Msk = 0x1f000000
	// Position of OUT_EN_SEL field.
	PIO0_SM3_EXECCTRL_OUT_EN_SEL_Pos = 0x13
	// Bit mask of OUT_EN_SEL field.
	PIO0_SM3_EXECCTRL_OUT_EN_SEL_Msk = 0xf80000
	// Position of INLINE_OUT_EN field.
	PIO0_SM3_EXECCTRL_INLINE_OUT_EN_Pos = 0x12
	// Bit mask of INLINE_OUT_EN field.
	PIO0_SM3_EXECCTRL_INLINE_OUT_EN_Msk = 0x40000
	// Bit INLINE_OUT_EN.
	PIO0_SM3_EXECCTRL_INLINE_OUT_EN = 0x40000
	// Position of OUT_STICKY field.
	PIO0_SM3_EXECCTRL_OUT_STICKY_Pos = 0x11
	// Bit mask of OUT_STICKY field.
	PIO0_SM3_EXECCTRL_OUT_STICKY_Msk = 0x20000
	// Bit OUT_STICKY.
	PIO0_SM3_EXECCTRL_OUT_STICKY = 0x20000
	// Position of WRAP_TOP field.
	PIO0_SM3_EXECCTRL_WRAP_TOP_Pos = 0xc
	// Bit mask of WRAP_TOP field.
	PIO0_SM3_EXECCTRL_WRAP_TOP_Msk = 0x1f000
	// Position of WRAP_BOTTOM field.
	PIO0_SM3_EXECCTRL_WRAP_BOTTOM_Pos = 0x7
	// Bit mask of WRAP_BOTTOM field.
	PIO0_SM3_EXECCTRL_WRAP_BOTTOM_Msk = 0xf80
	// Position of STATUS_SEL field.
	PIO0_SM3_EXECCTRL_STATUS_SEL_Pos = 0x5
	// Bit mask of STATUS_SEL field.
	PIO0_SM3_EXECCTRL_STATUS_SEL_Msk = 0x60
	// All-ones if TX FIFO level < N, otherwise all-zeroes
	PIO0_SM3_EXECCTRL_STATUS_SEL_TXLEVEL = 0x0
	// All-ones if RX FIFO level < N, otherwise all-zeroes
	PIO0_SM3_EXECCTRL_STATUS_SEL_RXLEVEL = 0x1
	// All-ones if the indexed IRQ flag is raised, otherwise all-zeroes
	PIO0_SM3_EXECCTRL_STATUS_SEL_IRQ = 0x2
	// Position of STATUS_N field.
	PIO0_SM3_EXECCTRL_STATUS_N_Pos = 0x0
	// Bit mask of STATUS_N field.
	PIO0_SM3_EXECCTRL_STATUS_N_Msk = 0x1f
	// Index 0-7 of an IRQ flag in this PIO block
	PIO0_SM3_EXECCTRL_STATUS_N_IRQ = 0x0
	// Index 0-7 of an IRQ flag in the next lower-numbered PIO block
	PIO0_SM3_EXECCTRL_STATUS_N_IRQ_PREVPIO = 0x8
	// Index 0-7 of an IRQ flag in the next higher-numbered PIO block
	PIO0_SM3_EXECCTRL_STATUS_N_IRQ_NEXTPIO = 0x10

	// SM3_SHIFTCTRL: Control behaviour of the input/output shift registers for state machine 3
	// Position of FJOIN_RX field.
	PIO0_SM3_SHIFTCTRL_FJOIN_RX_Pos = 0x1f
	// Bit mask of FJOIN_RX field.
	PIO0_SM3_SHIFTCTRL_FJOIN_RX_Msk = 0x80000000
	// Bit FJOIN_RX.
	PIO0_SM3_SHIFTCTRL_FJOIN_RX = 0x80000000
	// Position of FJOIN_TX field.
	PIO0_SM3_SHIFTCTRL_FJOIN_TX_Pos = 0x1e
	// Bit mask of FJOIN_TX field.
	PIO0_SM3_SHIFTCTRL_FJOIN_TX_Msk = 0x40000000
	// Bit FJOIN_TX.
	PIO0_SM3_SHIFTCTRL_FJOIN_TX = 0x40000000
	// Position of PULL_THRESH field.
	PIO0_SM3_SHIFTCTRL_PULL_THRESH_Pos = 0x19
	// Bit mask of PULL_THRESH field.
	PIO0_SM3_SHIFTCTRL_PULL_THRESH_Msk = 0x3e000000
	// Position of PUSH_THRESH field.
	PIO0_SM3_SHIFTCTRL_PUSH_THRESH_Pos = 0x14
	// Bit mask of PUSH_THRESH field.
	PIO0_SM3_SHIFTCTRL_PUSH_THRESH_Msk = 0x1f00000
	// Position of OUT_SHIFTDIR field.
	PIO0_SM3_SHIFTCTRL_OUT_SHIFTDIR_Pos = 0x13
	// Bit mask of OUT_SHIFTDIR field.
	PIO0_SM3_SHIFTCTRL_OUT_SHIFTDIR_Msk = 0x80000
	// Bit OUT_SHIFTDIR.
	PIO0_SM3_SHIFTCTRL_OUT_SHIFTDIR = 0x80000
	// Position of IN_SHIFTDIR field.
	PIO0_SM3_SHIFTCTRL_IN_SHIFTDIR_Pos = 0x12
	// Bit mask of IN_SHIFTDIR field.
	PIO0_SM3_SHIFTCTRL_IN_SHIFTDIR_Msk = 0x40000
	// Bit IN_SHIFTDIR.
	PIO0_SM3_SHIFTCTRL_IN_SHIFTDIR = 0x40000
	// Position of AUTOPULL field.
	PIO0_SM3_SHIFTCTRL_AUTOPULL_Pos = 0x11
	// Bit mask of AUTOPULL field.
	PIO0_SM3_SHIFTCTRL_AUTOPULL_Msk = 0x20000
	// Bit AUTOPULL.
	PIO0_SM3_SHIFTCTRL_AUTOPULL = 0x20000
	// Position of AUTOPUSH field.
	PIO0_SM3_SHIFTCTRL_AUTOPUSH_Pos = 0x10
	// Bit mask of AUTOPUSH field.
	PIO0_SM3_SHIFTCTRL_AUTOPUSH_Msk = 0x10000
	// Bit AUTOPUSH.
	PIO0_SM3_SHIFTCTRL_AUTOPUSH = 0x10000
	// Position of FJOIN_RX_PUT field.
	PIO0_SM3_SHIFTCTRL_FJOIN_RX_PUT_Pos = 0xf
	// Bit mask of FJOIN_RX_PUT field.
	PIO0_SM3_SHIFTCTRL_FJOIN_RX_PUT_Msk = 0x8000
	// Bit FJOIN_RX_PUT.
	PIO0_SM3_SHIFTCTRL_FJOIN_RX_PUT = 0x8000
	// Position of FJOIN_RX_GET field.
	PIO0_SM3_SHIFTCTRL_FJOIN_RX_GET_Pos = 0xe
	// Bit mask of FJOIN_RX_GET field.
	PIO0_SM3_SHIFTCTRL_FJOIN_RX_GET_Msk = 0x4000
	// Bit FJOIN_RX_GET.
	PIO0_SM3_SHIFTCTRL_FJOIN_RX_GET = 0x4000
	// Position of IN_COUNT field.
	PIO0_SM3_SHIFTCTRL_IN_COUNT_Pos = 0x0
	// Bit mask of IN_COUNT field.
	PIO0_SM3_SHIFTCTRL_IN_COUNT_Msk = 0x1f

	// SM3_ADDR: Current instruction address of state machine 3
	// Position of SM3_ADDR field.
	PIO0_SM3_ADDR_SM3_ADDR_Pos = 0x0
	// Bit mask of SM3_ADDR field.
	PIO0_SM3_ADDR_SM3_ADDR_Msk = 0x1f

	// SM3_INSTR: Read to see the instruction currently addressed by state machine 3's program counter Write to execute an instruction immediately (including jumps) and then resume execution.
	// Position of SM3_INSTR field.
	PIO0_SM3_INSTR_SM3_INSTR_Pos = 0x0
	// Bit mask of SM3_INSTR field.
	PIO0_SM3_INSTR_SM3_INSTR_Msk = 0xffff

	// SM3_PINCTRL: State machine pin control
	// Position of SIDESET_COUNT field.
	PIO0_SM3_PINCTRL_SIDESET_COUNT_Pos = 0x1d
	// Bit mask of SIDESET_COUNT field.
	PIO0_SM3_PINCTRL_SIDESET_COUNT_Msk = 0xe0000000
	// Position of SET_COUNT field.
	PIO0_SM3_PINCTRL_SET_COUNT_Pos = 0x1a
	// Bit mask of SET_COUNT field.
	PIO0_SM3_PINCTRL_SET_COUNT_Msk = 0x1c000000
	// Position of OUT_COUNT field.
	PIO0_SM3_PINCTRL_OUT_COUNT_Pos = 0x14
	// Bit mask of OUT_COUNT field.
	PIO0_SM3_PINCTRL_OUT_COUNT_Msk = 0x3f00000
	// Position of IN_BASE field.
	PIO0_SM3_PINCTRL_IN_BASE_Pos = 0xf
	// Bit mask of IN_BASE field.
	PIO0_SM3_PINCTRL_IN_BASE_Msk = 0xf8000
	// Position of SIDESET_BASE field.
	PIO0_SM3_PINCTRL_SIDESET_BASE_Pos = 0xa
	// Bit mask of SIDESET_BASE field.
	PIO0_SM3_PINCTRL_SIDESET_BASE_Msk = 0x7c00
	// Position of SET_BASE field.
	PIO0_SM3_PINCTRL_SET_BASE_Pos = 0x5
	// Bit mask of SET_BASE field.
	PIO0_SM3_PINCTRL_SET_BASE_Msk = 0x3e0
	// Position of OUT_BASE field.
	PIO0_SM3_PINCTRL_OUT_BASE_Pos = 0x0
	// Bit mask of OUT_BASE field.
	PIO0_SM3_PINCTRL_OUT_BASE_Msk = 0x1f

	// RXF0_PUTGET0: Direct read/write access to entry 0 of SM0's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF0_PUTGET0 field.
	PIO0_RXF0_PUTGET0_RXF0_PUTGET0_Pos = 0x0
	// Bit mask of RXF0_PUTGET0 field.
	PIO0_RXF0_PUTGET0_RXF0_PUTGET0_Msk = 0xffffffff

	// RXF0_PUTGET1: Direct read/write access to entry 1 of SM0's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF0_PUTGET1 field.
	PIO0_RXF0_PUTGET1_RXF0_PUTGET1_Pos = 0x0
	// Bit mask of RXF0_PUTGET1 field.
	PIO0_RXF0_PUTGET1_RXF0_PUTGET1_Msk = 0xffffffff

	// RXF0_PUTGET2: Direct read/write access to entry 2 of SM0's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF0_PUTGET2 field.
	PIO0_RXF0_PUTGET2_RXF0_PUTGET2_Pos = 0x0
	// Bit mask of RXF0_PUTGET2 field.
	PIO0_RXF0_PUTGET2_RXF0_PUTGET2_Msk = 0xffffffff

	// RXF0_PUTGET3: Direct read/write access to entry 3 of SM0's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF0_PUTGET3 field.
	PIO0_RXF0_PUTGET3_RXF0_PUTGET3_Pos = 0x0
	// Bit mask of RXF0_PUTGET3 field.
	PIO0_RXF0_PUTGET3_RXF0_PUTGET3_Msk = 0xffffffff

	// RXF1_PUTGET0: Direct read/write access to entry 0 of SM1's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF1_PUTGET0 field.
	PIO0_RXF1_PUTGET0_RXF1_PUTGET0_Pos = 0x0
	// Bit mask of RXF1_PUTGET0 field.
	PIO0_RXF1_PUTGET0_RXF1_PUTGET0_Msk = 0xffffffff

	// RXF1_PUTGET1: Direct read/write access to entry 1 of SM1's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF1_PUTGET1 field.
	PIO0_RXF1_PUTGET1_RXF1_PUTGET1_Pos = 0x0
	// Bit mask of RXF1_PUTGET1 field.
	PIO0_RXF1_PUTGET1_RXF1_PUTGET1_Msk = 0xffffffff

	// RXF1_PUTGET2: Direct read/write access to entry 2 of SM1's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF1_PUTGET2 field.
	PIO0_RXF1_PUTGET2_RXF1_PUTGET2_Pos = 0x0
	// Bit mask of RXF1_PUTGET2 field.
	PIO0_RXF1_PUTGET2_RXF1_PUTGET2_Msk = 0xffffffff

	// RXF1_PUTGET3: Direct read/write access to entry 3 of SM1's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF1_PUTGET3 field.
	PIO0_RXF1_PUTGET3_RXF1_PUTGET3_Pos = 0x0
	// Bit mask of RXF1_PUTGET3 field.
	PIO0_RXF1_PUTGET3_RXF1_PUTGET3_Msk = 0xffffffff

	// RXF2_PUTGET0: Direct read/write access to entry 0 of SM2's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF2_PUTGET0 field.
	PIO0_RXF2_PUTGET0_RXF2_PUTGET0_Pos = 0x0
	// Bit mask of RXF2_PUTGET0 field.
	PIO0_RXF2_PUTGET0_RXF2_PUTGET0_Msk = 0xffffffff

	// RXF2_PUTGET1: Direct read/write access to entry 1 of SM2's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF2_PUTGET1 field.
	PIO0_RXF2_PUTGET1_RXF2_PUTGET1_Pos = 0x0
	// Bit mask of RXF2_PUTGET1 field.
	PIO0_RXF2_PUTGET1_RXF2_PUTGET1_Msk = 0xffffffff

	// RXF2_PUTGET2: Direct read/write access to entry 2 of SM2's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF2_PUTGET2 field.
	PIO0_RXF2_PUTGET2_RXF2_PUTGET2_Pos = 0x0
	// Bit mask of RXF2_PUTGET2 field.
	PIO0_RXF2_PUTGET2_RXF2_PUTGET2_Msk = 0xffffffff

	// RXF2_PUTGET3: Direct read/write access to entry 3 of SM2's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF2_PUTGET3 field.
	PIO0_RXF2_PUTGET3_RXF2_PUTGET3_Pos = 0x0
	// Bit mask of RXF2_PUTGET3 field.
	PIO0_RXF2_PUTGET3_RXF2_PUTGET3_Msk = 0xffffffff

	// RXF3_PUTGET0: Direct read/write access to entry 0 of SM3's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF3_PUTGET0 field.
	PIO0_RXF3_PUTGET0_RXF3_PUTGET0_Pos = 0x0
	// Bit mask of RXF3_PUTGET0 field.
	PIO0_RXF3_PUTGET0_RXF3_PUTGET0_Msk = 0xffffffff

	// RXF3_PUTGET1: Direct read/write access to entry 1 of SM3's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF3_PUTGET1 field.
	PIO0_RXF3_PUTGET1_RXF3_PUTGET1_Pos = 0x0
	// Bit mask of RXF3_PUTGET1 field.
	PIO0_RXF3_PUTGET1_RXF3_PUTGET1_Msk = 0xffffffff

	// RXF3_PUTGET2: Direct read/write access to entry 2 of SM3's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF3_PUTGET2 field.
	PIO0_RXF3_PUTGET2_RXF3_PUTGET2_Pos = 0x0
	// Bit mask of RXF3_PUTGET2 field.
	PIO0_RXF3_PUTGET2_RXF3_PUTGET2_Msk = 0xffffffff

	// RXF3_PUTGET3: Direct read/write access to entry 3 of SM3's RX FIFO, if SHIFTCTRL_FJOIN_RX_PUT xor SHIFTCTRL_FJOIN_RX_GET is set.
	// Position of RXF3_PUTGET3 field.
	PIO0_RXF3_PUTGET3_RXF3_PUTGET3_Pos = 0x0
	// Bit mask of RXF3_PUTGET3 field.
	PIO0_RXF3_PUTGET3_RXF3_PUTGET3_Msk = 0xffffffff

	// GPIOBASE: Relocate GPIO 0 (from PIO's point of view) in the system GPIO numbering, to access more than 32 GPIOs from PIO. Only the values 0 and 16 are supported (only bit 4 is writable).
	// Position of GPIOBASE field.
	PIO0_GPIOBASE_GPIOBASE_Pos = 0x4
	// Bit mask of GPIOBASE field.
	PIO0_GPIOBASE_GPIOBASE_Msk = 0x10
	// Bit GPIOBASE.
	PIO0_GPIOBASE_GPIOBASE = 0x10

	// INTR: Raw Interrupts
	// Position of SM7 field.
	PIO0_INTR_SM7_Pos = 0xf
	// Bit mask of SM7 field.
	PIO0_INTR_SM7_Msk = 0x8000
	// Bit SM7.
	PIO0_INTR_SM7 = 0x8000
	// Position of SM6 field.
	PIO0_INTR_SM6_Pos = 0xe
	// Bit mask of SM6 field.
	PIO0_INTR_SM6_Msk = 0x4000
	// Bit SM6.
	PIO0_INTR_SM6 = 0x4000
	// Position of SM5 field.
	PIO0_INTR_SM5_Pos = 0xd
	// Bit mask of SM5 field.
	PIO0_INTR_SM5_Msk = 0x2000
	// Bit SM5.
	PIO0_INTR_SM5 = 0x2000
	// Position of SM4 field.
	PIO0_INTR_SM4_Pos = 0xc
	// Bit mask of SM4 field.
	PIO0_INTR_SM4_Msk = 0x1000
	// Bit SM4.
	PIO0_INTR_SM4 = 0x1000
	// Position of SM3 field.
	PIO0_INTR_SM3_Pos = 0xb
	// Bit mask of SM3 field.
	PIO0_INTR_SM3_Msk = 0x800
	// Bit SM3.
	PIO0_INTR_SM3 = 0x800
	// Position of SM2 field.
	PIO0_INTR_SM2_Pos = 0xa
	// Bit mask of SM2 field.
	PIO0_INTR_SM2_Msk = 0x400
	// Bit SM2.
	PIO0_INTR_SM2 = 0x400
	// Position of SM1 field.
	PIO0_INTR_SM1_Pos = 0x9
	// Bit mask of SM1 field.
	PIO0_INTR_SM1_Msk = 0x200
	// Bit SM1.
	PIO0_INTR_SM1 = 0x200
	// Position of SM0 field.
	PIO0_INTR_SM0_Pos = 0x8
	// Bit mask of SM0 field.
	PIO0_INTR_SM0_Msk = 0x100
	// Bit SM0.
	PIO0_INTR_SM0 = 0x100
	// Position of SM3_TXNFULL field.
	PIO0_INTR_SM3_TXNFULL_Pos = 0x7
	// Bit mask of SM3_TXNFULL field.
	PIO0_INTR_SM3_TXNFULL_Msk = 0x80
	// Bit SM3_TXNFULL.
	PIO0_INTR_SM3_TXNFULL = 0x80
	// Position of SM2_TXNFULL field.
	PIO0_INTR_SM2_TXNFULL_Pos = 0x6
	// Bit mask of SM2_TXNFULL field.
	PIO0_INTR_SM2_TXNFULL_Msk = 0x40
	// Bit SM2_TXNFULL.
	PIO0_INTR_SM2_TXNFULL = 0x40
	// Position of SM1_TXNFULL field.
	PIO0_INTR_SM1_TXNFULL_Pos = 0x5
	// Bit mask of SM1_TXNFULL field.
	PIO0_INTR_SM1_TXNFULL_Msk = 0x20
	// Bit SM1_TXNFULL.
	PIO0_INTR_SM1_TXNFULL = 0x20
	// Position of SM0_TXNFULL field.
	PIO0_INTR_SM0_TXNFULL_Pos = 0x4
	// Bit mask of SM0_TXNFULL field.
	PIO0_INTR_SM0_TXNFULL_Msk = 0x10
	// Bit SM0_TXNFULL.
	PIO0_INTR_SM0_TXNFULL = 0x10
	// Position of SM3_RXNEMPTY field.
	PIO0_INTR_SM3_RXNEMPTY_Pos = 0x3
	// Bit mask of SM3_RXNEMPTY field.
	PIO0_INTR_SM3_RXNEMPTY_Msk = 0x8
	// Bit SM3_RXNEMPTY.
	PIO0_INTR_SM3_RXNEMPTY = 0x8
	// Position of SM2_RXNEMPTY field.
	PIO0_INTR_SM2_RXNEMPTY_Pos = 0x2
	// Bit mask of SM2_RXNEMPTY field.
	PIO0_INTR_SM2_RXNEMPTY_Msk = 0x4
	// Bit SM2_RXNEMPTY.
	PIO0_INTR_SM2_RXNEMPTY = 0x4
	// Position of SM1_RXNEMPTY field.
	PIO0_INTR_SM1_RXNEMPTY_Pos = 0x1
	// Bit mask of SM1_RXNEMPTY field.
	PIO0_INTR_SM1_RXNEMPTY_Msk = 0x2
	// Bit SM1_RXNEMPTY.
	PIO0_INTR_SM1_RXNEMPTY = 0x2
	// Position of SM0_RXNEMPTY field.
	PIO0_INTR_SM0_RXNEMPTY_Pos = 0x0
	// Bit mask of SM0_RXNEMPTY field.
	PIO0_INTR_SM0_RXNEMPTY_Msk = 0x1
	// Bit SM0_RXNEMPTY.
	PIO0_INTR_SM0_RXNEMPTY = 0x1

	// IRQ0_INTE: Interrupt Enable for irq0
	// Position of SM7 field.
	PIO0_IRQ0_INTE_SM7_Pos = 0xf
	// Bit mask of SM7 field.
	PIO0_IRQ0_INTE_SM7_Msk = 0x8000
	// Bit SM7.
	PIO0_IRQ0_INTE_SM7 = 0x8000
	// Position of SM6 field.
	PIO0_IRQ0_INTE_SM6_Pos = 0xe
	// Bit mask of SM6 field.
	PIO0_IRQ0_INTE_SM6_Msk = 0x4000
	// Bit SM6.
	PIO0_IRQ0_INTE_SM6 = 0x4000
	// Position of SM5 field.
	PIO0_IRQ0_INTE_SM5_Pos = 0xd
	// Bit mask of SM5 field.
	PIO0_IRQ0_INTE_SM5_Msk = 0x2000
	// Bit SM5.
	PIO0_IRQ0_INTE_SM5 = 0x2000
	// Position of SM4 field.
	PIO0_IRQ0_INTE_SM4_Pos = 0xc
	// Bit mask of SM4 field.
	PIO0_IRQ0_INTE_SM4_Msk = 0x1000
	// Bit SM4.
	PIO0_IRQ0_INTE_SM4 = 0x1000
	// Position of SM3 field.
	PIO0_IRQ0_INTE_SM3_Pos = 0xb
	// Bit mask of SM3 field.
	PIO0_IRQ0_INTE_SM3_Msk = 0x800
	// Bit SM3.
	PIO0_IRQ0_INTE_SM3 = 0x800
	// Position of SM2 field.
	PIO0_IRQ0_INTE_SM2_Pos = 0xa
	// Bit mask of SM2 field.
	PIO0_IRQ0_INTE_SM2_Msk = 0x400
	// Bit SM2.
	PIO0_IRQ0_INTE_SM2 = 0x400
	// Position of SM1 field.
	PIO0_IRQ0_INTE_SM1_Pos = 0x9
	// Bit mask of SM1 field.
	PIO0_IRQ0_INTE_SM1_Msk = 0x200
	// Bit SM1.
	PIO0_IRQ0_INTE_SM1 = 0x200
	// Position of SM0 field.
	PIO0_IRQ0_INTE_SM0_Pos = 0x8
	// Bit mask of SM0 field.
	PIO0_IRQ0_INTE_SM0_Msk = 0x100
	// Bit SM0.
	PIO0_IRQ0_INTE_SM0 = 0x100
	// Position of SM3_TXNFULL field.
	PIO0_IRQ0_INTE_SM3_TXNFULL_Pos = 0x7
	// Bit mask of SM3_TXNFULL field.
	PIO0_IRQ0_INTE_SM3_TXNFULL_Msk = 0x80
	// Bit SM3_TXNFULL.
	PIO0_IRQ0_INTE_SM3_TXNFULL = 0x80
	// Position of SM2_TXNFULL field.
	PIO0_IRQ0_INTE_SM2_TXNFULL_Pos = 0x6
	// Bit mask of SM2_TXNFULL field.
	PIO0_IRQ0_INTE_SM2_TXNFULL_Msk = 0x40
	// Bit SM2_TXNFULL.
	PIO0_IRQ0_INTE_SM2_TXNFULL = 0x40
	// Position of SM1_TXNFULL field.
	PIO0_IRQ0_INTE_SM1_TXNFULL_Pos = 0x5
	// Bit mask of SM1_TXNFULL field.
	PIO0_IRQ0_INTE_SM1_TXNFULL_Msk = 0x20
	// Bit SM1_TXNFULL.
	PIO0_IRQ0_INTE_SM1_TXNFULL = 0x20
	// Position of SM0_TXNFULL field.
	PIO0_IRQ0_INTE_SM0_TXNFULL_Pos = 0x4
	// Bit mask of SM0_TXNFULL field.
	PIO0_IRQ0_INTE_SM0_TXNFULL_Msk = 0x10
	// Bit SM0_TXNFULL.
	PIO0_IRQ0_INTE_SM0_TXNFULL = 0x10
	// Position of SM3_RXNEMPTY field.
	PIO0_IRQ0_INTE_SM3_RXNEMPTY_Pos = 0x3
	// Bit mask of SM3_RXNEMPTY field.
	PIO0_IRQ0_INTE_SM3_RXNEMPTY_Msk = 0x8
	// Bit SM3_RXNEMPTY.
	PIO0_IRQ0_INTE_SM3_RXNEMPTY = 0x8
	// Position of SM2_RXNEMPTY field.
	PIO0_IRQ0_INTE_SM2_RXNEMPTY_Pos = 0x2
	// Bit mask of SM2_RXNEMPTY field.
	PIO0_IRQ0_INTE_SM2_RXNEMPTY_Msk = 0x4
	// Bit SM2_RXNEMPTY.
	PIO0_IRQ0_INTE_SM2_RXNEMPTY = 0x4
	// Position of SM1_RXNEMPTY field.
	PIO0_IRQ0_INTE_SM1_RXNEMPTY_Pos = 0x1
	// Bit mask of SM1_RXNEMPTY field.
	PIO0_IRQ0_INTE_SM1_RXNEMPTY_Msk = 0x2
	// Bit SM1_RXNEMPTY.
	PIO0_IRQ0_INTE_SM1_RXNEMPTY = 0x2
	// Position of SM0_RXNEMPTY field.
	PIO0_IRQ0_INTE_SM0_RXNEMPTY_Pos = 0x0
	// Bit mask of SM0_RXNEMPTY field.
	PIO0_IRQ0_INTE_SM0_RXNEMPTY_Msk = 0x1
	// Bit SM0_RXNEMPTY.
	PIO0_IRQ0_INTE_SM0_RXNEMPTY = 0x1

	// IRQ0_INTF: Interrupt Force for irq0
	// Position of SM7 field.
	PIO0_IRQ0_INTF_SM7_Pos = 0xf
	// Bit mask of SM7 field.
	PIO0_IRQ0_INTF_SM7_Msk = 0x8000
	// Bit SM7.
	PIO0_IRQ0_INTF_SM7 = 0x8000
	// Position of SM6 field.
	PIO0_IRQ0_INTF_SM6_Pos = 0xe
	// Bit mask of SM6 field.
	PIO0_IRQ0_INTF_SM6_Msk = 0x4000
	// Bit SM6.
	PIO0_IRQ0_INTF_SM6 = 0x4000
	// Position of SM5 field.
	PIO0_IRQ0_INTF_SM5_Pos = 0xd
	// Bit mask of SM5 field.
	PIO0_IRQ0_INTF_SM5_Msk = 0x2000
	// Bit SM5.
	PIO0_IRQ0_INTF_SM5 = 0x2000
	// Position of SM4 field.
	PIO0_IRQ0_INTF_SM4_Pos = 0xc
	// Bit mask of SM4 field.
	PIO0_IRQ0_INTF_SM4_Msk = 0x1000
	// Bit SM4.
	PIO0_IRQ0_INTF_SM4 = 0x1000
	// Position of SM3 field.
	PIO0_IRQ0_INTF_SM3_Pos = 0xb
	// Bit mask of SM3 field.
	PIO0_IRQ0_INTF_SM3_Msk = 0x800
	// Bit SM3.
	PIO0_IRQ0_INTF_SM3 = 0x800
	// Position of SM2 field.
	PIO0_IRQ0_INTF_SM2_Pos = 0xa
	// Bit mask of SM2 field.
	PIO0_IRQ0_INTF_SM2_Msk = 0x400
	// Bit SM2.
	PIO0_IRQ0_INTF_SM2 = 0x400
	// Position of SM1 field.
	PIO0_IRQ0_INTF_SM1_Pos = 0x9
	// Bit mask of SM1 field.
	PIO0_IRQ0_INTF_SM1_Msk = 0x200
	// Bit SM1.
	PIO0_IRQ0_INTF_SM1 = 0x200
	// Position of SM0 field.
	PIO0_IRQ0_INTF_SM0_Pos = 0x8
	// Bit mask of SM0 field.
	PIO0_IRQ0_INTF_SM0_Msk = 0x100
	// Bit SM0.
	PIO0_IRQ0_INTF_SM0 = 0x100
	// Position of SM3_TXNFULL field.
	PIO0_IRQ0_INTF_SM3_TXNFULL_Pos = 0x7
	// Bit mask of SM3_TXNFULL field.
	PIO0_IRQ0_INTF_SM3_TXNFULL_Msk = 0x80
	// Bit SM3_TXNFULL.
	PIO0_IRQ0_INTF_SM3_TXNFULL = 0x80
	// Position of SM2_TXNFULL field.
	PIO0_IRQ0_INTF_SM2_TXNFULL_Pos = 0x6
	// Bit mask of SM2_TXNFULL field.
	PIO0_IRQ0_INTF_SM2_TXNFULL_Msk = 0x40
	// Bit SM2_TXNFULL.
	PIO0_IRQ0_INTF_SM2_TXNFULL = 0x40
	// Position of SM1_TXNFULL field.
	PIO0_IRQ0_INTF_SM1_TXNFULL_Pos = 0x5
	// Bit mask of SM1_TXNFULL field.
	PIO0_IRQ0_INTF_SM1_TXNFULL_Msk = 0x20
	// Bit SM1_TXNFULL.
	PIO0_IRQ0_INTF_SM1_TXNFULL = 0x20
	// Position of SM0_TXNFULL field.
	PIO0_IRQ0_INTF_SM0_TXNFULL_Pos = 0x4
	// Bit mask of SM0_TXNFULL field.
	PIO0_IRQ0_INTF_SM0_TXNFULL_Msk = 0x10
	// Bit SM0_TXNFULL.
	PIO0_IRQ0_INTF_SM0_TXNFULL = 0x10
	// Position of SM3_RXNEMPTY field.
	PIO0_IRQ0_INTF_SM3_RXNEMPTY_Pos = 0x3
	// Bit mask of SM3_RXNEMPTY field.
	PIO0_IRQ0_INTF_SM3_RXNEMPTY_Msk = 0x8
	// Bit SM3_RXNEMPTY.
	PIO0_IRQ0_INTF_SM3_RXNEMPTY = 0x8
	// Position of SM2_RXNEMPTY field.
	PIO0_IRQ0_INTF_SM2_RXNEMPTY_Pos = 0x2
	// Bit mask of SM2_RXNEMPTY field.
	PIO0_IRQ0_INTF_SM2_RXNEMPTY_Msk = 0x4
	// Bit SM2_RXNEMPTY.
	PIO0_IRQ0_INTF_SM2_RXNEMPTY = 0x4
	// Position of SM1_RXNEMPTY field.
	PIO0_IRQ0_INTF_SM1_RXNEMPTY_Pos = 0x1
	// Bit mask of SM1_RXNEMPTY field.
	PIO0_IRQ0_INTF_SM1_RXNEMPTY_Msk = 0x2
	// Bit SM1_RXNEMPTY.
	PIO0_IRQ0_INTF_SM1_RXNEMPTY = 0x2
	// Position of SM0_RXNEMPTY field.
	PIO0_IRQ0_INTF_SM0_RXNEMPTY_Pos = 0x0
	// Bit mask of SM0_RXNEMPTY field.
	PIO0_IRQ0_INTF_SM0_RXNEMPTY_Msk = 0x1
	// Bit SM0_RXNEMPTY.
	PIO0_IRQ0_INTF_SM0_RXNEMPTY = 0x1

	// IRQ0_INTS: Interrupt status after masking & forcing for irq0
	// Position of SM7 field.
	PIO0_IRQ0_INTS_SM7_Pos = 0xf
	// Bit mask of SM7 field.
	PIO0_IRQ0_INTS_SM7_Msk = 0x8000
	// Bit SM7.
	PIO0_IRQ0_INTS_SM7 = 0x8000
	// Position of SM6 field.
	PIO0_IRQ0_INTS_SM6_Pos = 0xe
	// Bit mask of SM6 field.
	PIO0_IRQ0_INTS_SM6_Msk = 0x4000
	// Bit SM6.
	PIO0_IRQ0_INTS_SM6 = 0x4000
	// Position of SM5 field.
	PIO0_IRQ0_INTS_SM5_Pos = 0xd
	// Bit mask of SM5 field.
	PIO0_IRQ0_INTS_SM5_Msk = 0x2000
	// Bit SM5.
	PIO0_IRQ0_INTS_SM5 = 0x2000
	// Position of SM4 field.
	PIO0_IRQ0_INTS_SM4_Pos = 0xc
	// Bit mask of SM4 field.
	PIO0_IRQ0_INTS_SM4_Msk = 0x1000
	// Bit SM4.
	PIO0_IRQ0_INTS_SM4 = 0x1000
	// Position of SM3 field.
	PIO0_IRQ0_INTS_SM3_Pos = 0xb
	// Bit mask of SM3 field.
	PIO0_IRQ0_INTS_SM3_Msk = 0x800
	// Bit SM3.
	PIO0_IRQ0_INTS_SM3 = 0x800
	// Position of SM2 field.
	PIO0_IRQ0_INTS_SM2_Pos = 0xa
	// Bit mask of SM2 field.
	PIO0_IRQ0_INTS_SM2_Msk = 0x400
	// Bit SM2.
	PIO0_IRQ0_INTS_SM2 = 0x400
	// Position of SM1 field.
	PIO0_IRQ0_INTS_SM1_Pos = 0x9
	// Bit mask of SM1 field.
	PIO0_IRQ0_INTS_SM1_Msk = 0x200
	// Bit SM1.
	PIO0_IRQ0_INTS_SM1 = 0x200
	// Position of SM0 field.
	PIO0_IRQ0_INTS_SM0_Pos = 0x8
	// Bit mask of SM0 field.
	PIO0_IRQ0_INTS_SM0_Msk = 0x100
	// Bit SM0.
	PIO0_IRQ0_INTS_SM0 = 0x100
	// Position of SM3_TXNFULL field.
	PIO0_IRQ0_INTS_SM3_TXNFULL_Pos = 0x7
	// Bit mask of SM3_TXNFULL field.
	PIO0_IRQ0_INTS_SM3_TXNFULL_Msk = 0x80
	// Bit SM3_TXNFULL.
	PIO0_IRQ0_INTS_SM3_TXNFULL = 0x80
	// Position of SM2_TXNFULL field.
	PIO0_IRQ0_INTS_SM2_TXNFULL_Pos = 0x6
	// Bit mask of SM2_TXNFULL field.
	PIO0_IRQ0_INTS_SM2_TXNFULL_Msk = 0x40
	// Bit SM2_TXNFULL.
	PIO0_IRQ0_INTS_SM2_TXNFULL = 0x40
	// Position of SM1_TXNFULL field.
	PIO0_IRQ0_INTS_SM1_TXNFULL_Pos = 0x5
	// Bit mask of SM1_TXNFULL field.
	PIO0_IRQ0_INTS_SM1_TXNFULL_Msk = 0x20
	// Bit SM1_TXNFULL.
	PIO0_IRQ0_INTS_SM1_TXNFULL = 0x20
	// Position of SM0_TXNFULL field.
	PIO0_IRQ0_INTS_SM0_TXNFULL_Pos = 0x4
	// Bit mask of SM0_TXNFULL field.
	PIO0_IRQ0_INTS_SM0_TXNFULL_Msk = 0x10
	// Bit SM0_TXNFULL.
	PIO0_IRQ0_INTS_SM0_TXNFULL = 0x10
	// Position of SM3_RXNEMPTY field.
	PIO0_IRQ0_INTS_SM3_RXNEMPTY_Pos = 0x3
	// Bit mask of SM3_RXNEMPTY field.
	PIO0_IRQ0_INTS_SM3_RXNEMPTY_Msk = 0x8
	// Bit SM3_RXNEMPTY.
	PIO0_IRQ0_INTS_SM3_RXNEMPTY = 0x8
	// Position of SM2_RXNEMPTY field.
	PIO0_IRQ0_INTS_SM2_RXNEMPTY_Pos = 0x2
	// Bit mask of SM2_RXNEMPTY field.
	PIO0_IRQ0_INTS_SM2_RXNEMPTY_Msk = 0x4
	// Bit SM2_RXNEMPTY.
	PIO0_IRQ0_INTS_SM2_RXNEMPTY = 0x4
	// Position of SM1_RXNEMPTY field.
	PIO0_IRQ0_INTS_SM1_RXNEMPTY_Pos = 0x1
	// Bit mask of SM1_RXNEMPTY field.
	PIO0_IRQ0_INTS_SM1_RXNEMPTY_Msk = 0x2
	// Bit SM1_RXNEMPTY.
	PIO0_IRQ0_INTS_SM1_RXNEMPTY = 0x2
	// Position of SM0_RXNEMPTY field.
	PIO0_IRQ0_INTS_SM0_RXNEMPTY_Pos = 0x0
	// Bit mask of SM0_RXNEMPTY field.
	PIO0_IRQ0_INTS_SM0_RXNEMPTY_Msk = 0x1
	// Bit SM0_RXNEMPTY.
	PIO0_IRQ0_INTS_SM0_RXNEMPTY = 0x1

	// IRQ1_INTE: Interrupt Enable for irq1
	// Position of SM7 field.
	PIO0_IRQ1_INTE_SM7_Pos = 0xf
	// Bit mask of SM7 field.
	PIO0_IRQ1_INTE_SM7_Msk = 0x8000
	// Bit SM7.
	PIO0_IRQ1_INTE_SM7 = 0x8000
	// Position of SM6 field.
	PIO0_IRQ1_INTE_SM6_Pos = 0xe
	// Bit mask of SM6 field.
	PIO0_IRQ1_INTE_SM6_Msk = 0x4000
	// Bit SM6.
	PIO0_IRQ1_INTE_SM6 = 0x4000
	// Position of SM5 field.
	PIO0_IRQ1_INTE_SM5_Pos = 0xd
	// Bit mask of SM5 field.
	PIO0_IRQ1_INTE_SM5_Msk = 0x2000
	// Bit SM5.
	PIO0_IRQ1_INTE_SM5 = 0x2000
	// Position of SM4 field.
	PIO0_IRQ1_INTE_SM4_Pos = 0xc
	// Bit mask of SM4 field.
	PIO0_IRQ1_INTE_SM4_Msk = 0x1000
	// Bit SM4.
	PIO0_IRQ1_INTE_SM4 = 0x1000
	// Position of SM3 field.
	PIO0_IRQ1_INTE_SM3_Pos = 0xb
	// Bit mask of SM3 field.
	PIO0_IRQ1_INTE_SM3_Msk = 0x800
	// Bit SM3.
	PIO0_IRQ1_INTE_SM3 = 0x800
	// Position of SM2 field.
	PIO0_IRQ1_INTE_SM2_Pos = 0xa
	// Bit mask of SM2 field.
	PIO0_IRQ1_INTE_SM2_Msk = 0x400
	// Bit SM2.
	PIO0_IRQ1_INTE_SM2 = 0x400
	// Position of SM1 field.
	PIO0_IRQ1_INTE_SM1_Pos = 0x9
	// Bit mask of SM1 field.
	PIO0_IRQ1_INTE_SM1_Msk = 0x200
	// Bit SM1.
	PIO0_IRQ1_INTE_SM1 = 0x200
	// Position of SM0 field.
	PIO0_IRQ1_INTE_SM0_Pos = 0x8
	// Bit mask of SM0 field.
	PIO0_IRQ1_INTE_SM0_Msk = 0x100
	// Bit SM0.
	PIO0_IRQ1_INTE_SM0 = 0x100
	// Position of SM3_TXNFULL field.
	PIO0_IRQ1_INTE_SM3_TXNFULL_Pos = 0x7
	// Bit mask of SM3_TXNFULL field.
	PIO0_IRQ1_INTE_SM3_TXNFULL_Msk = 0x80
	// Bit SM3_TXNFULL.
	PIO0_IRQ1_INTE_SM3_TXNFULL = 0x80
	// Position of SM2_TXNFULL field.
	PIO0_IRQ1_INTE_SM2_TXNFULL_Pos = 0x6
	// Bit mask of SM2_TXNFULL field.
	PIO0_IRQ1_INTE_SM2_TXNFULL_Msk = 0x40
	// Bit SM2_TXNFULL.
	PIO0_IRQ1_INTE_SM2_TXNFULL = 0x40
	// Position of SM1_TXNFULL field.
	PIO0_IRQ1_INTE_SM1_TXNFULL_Pos = 0x5
	// Bit mask of SM1_TXNFULL field.
	PIO0_IRQ1_INTE_SM1_TXNFULL_Msk = 0x20
	// Bit SM1_TXNFULL.
	PIO0_IRQ1_INTE_SM1_TXNFULL = 0x20
	// Position of SM0_TXNFULL field.
	PIO0_IRQ1_INTE_SM0_TXNFULL_Pos = 0x4
	// Bit mask of SM0_TXNFULL field.
	PIO0_IRQ1_INTE_SM0_TXNFULL_Msk = 0x10
	// Bit SM0_TXNFULL.
	PIO0_IRQ1_INTE_SM0_TXNFULL = 0x10
	// Position of SM3_RXNEMPTY field.
	PIO0_IRQ1_INTE_SM3_RXNEMPTY_Pos = 0x3
	// Bit mask of SM3_RXNEMPTY field.
	PIO0_IRQ1_INTE_SM3_RXNEMPTY_Msk = 0x8
	// Bit SM3_RXNEMPTY.
	PIO0_IRQ1_INTE_SM3_RXNEMPTY = 0x8
	// Position of SM2_RXNEMPTY field.
	PIO0_IRQ1_INTE_SM2_RXNEMPTY_Pos = 0x2
	// Bit mask of SM2_RXNEMPTY field.
	PIO0_IRQ1_INTE_SM2_RXNEMPTY_Msk = 0x4
	// Bit SM2_RXNEMPTY.
	PIO0_IRQ1_INTE_SM2_RXNEMPTY = 0x4
	// Position of SM1_RXNEMPTY field.
	PIO0_IRQ1_INTE_SM1_RXNEMPTY_Pos = 0x1
	// Bit mask of SM1_RXNEMPTY field.
	PIO0_IRQ1_INTE_SM1_RXNEMPTY_Msk = 0x2
	// Bit SM1_RXNEMPTY.
	PIO0_IRQ1_INTE_SM1_RXNEMPTY = 0x2
	// Position of SM0_RXNEMPTY field.
	PIO0_IRQ1_INTE_SM0_RXNEMPTY_Pos = 0x0
	// Bit mask of SM0_RXNEMPTY field.
	PIO0_IRQ1_INTE_SM0_RXNEMPTY_Msk = 0x1
	// Bit SM0_RXNEMPTY.
	PIO0_IRQ1_INTE_SM0_RXNEMPTY = 0x1

	// IRQ1_INTF: Interrupt Force for irq1
	// Position of SM7 field.
	PIO0_IRQ1_INTF_SM7_Pos = 0xf
	// Bit mask of SM7 field.
	PIO0_IRQ1_INTF_SM7_Msk = 0x8000
	// Bit SM7.
	PIO0_IRQ1_INTF_SM7 = 0x8000
	// Position of SM6 field.
	PIO0_IRQ1_INTF_SM6_Pos = 0xe
	// Bit mask of SM6 field.
	PIO0_IRQ1_INTF_SM6_Msk = 0x4000
	// Bit SM6.
	PIO0_IRQ1_INTF_SM6 = 0x4000
	// Position of SM5 field.
	PIO0_IRQ1_INTF_SM5_Pos = 0xd
	// Bit mask of SM5 field.
	PIO0_IRQ1_INTF_SM5_Msk = 0x2000
	// Bit SM5.
	PIO0_IRQ1_INTF_SM5 = 0x2000
	// Position of SM4 field.
	PIO0_IRQ1_INTF_SM4_Pos = 0xc
	// Bit mask of SM4 field.
	PIO0_IRQ1_INTF_SM4_Msk = 0x1000
	// Bit SM4.
	PIO0_IRQ1_INTF_SM4 = 0x1000
	// Position of SM3 field.
	PIO0_IRQ1_INTF_SM3_Pos = 0xb
	// Bit mask of SM3 field.
	PIO0_IRQ1_INTF_SM3_Msk = 0x800
	// Bit SM3.
	PIO0_IRQ1_INTF_SM3 = 0x800
	// Position of SM2 field.
	PIO0_IRQ1_INTF_SM2_Pos = 0xa
	// Bit mask of SM2 field.
	PIO0_IRQ1_INTF_SM2_Msk = 0x400
	// Bit SM2.
	PIO0_IRQ1_INTF_SM2 = 0x400
	// Position of SM1 field.
	PIO0_IRQ1_INTF_SM1_Pos = 0x9
	// Bit mask of SM1 field.
	PIO0_IRQ1_INTF_SM1_Msk = 0x200
	// Bit SM1.
	PIO0_IRQ1_INTF_SM1 = 0x200
	// Position of SM0 field.
	PIO0_IRQ1_INTF_SM0_Pos = 0x8
	// Bit mask of SM0 field.
	PIO0_IRQ1_INTF_SM0_Msk = 0x100
	// Bit SM0.
	PIO0_IRQ1_INTF_SM0 = 0x100
	// Position of SM3_TXNFULL field.
	PIO0_IRQ1_INTF_SM3_TXNFULL_Pos = 0x7
	// Bit mask of SM3_TXNFULL field.
	PIO0_IRQ1_INTF_SM3_TXNFULL_Msk = 0x80
	// Bit SM3_TXNFULL.
	PIO0_IRQ1_INTF_SM3_TXNFULL = 0x80
	// Position of SM2_TXNFULL field.
	PIO0_IRQ1_INTF_SM2_TXNFULL_Pos = 0x6
	// Bit mask of SM2_TXNFULL field.
	PIO0_IRQ1_INTF_SM2_TXNFULL_Msk = 0x40
	// Bit SM2_TXNFULL.
	PIO0_IRQ1_INTF_SM2_TXNFULL = 0x40
	// Position of SM1_TXNFULL field.
	PIO0_IRQ1_INTF_SM1_TXNFULL_Pos = 0x5
	// Bit mask of SM1_TXNFULL field.
	PIO0_IRQ1_INTF_SM1_TXNFULL_Msk = 0x20
	// Bit SM1_TXNFULL.
	PIO0_IRQ1_INTF_SM1_TXNFULL = 0x20
	// Position of SM0_TXNFULL field.
	PIO0_IRQ1_INTF_SM0_TXNFULL_Pos = 0x4
	// Bit mask of SM0_TXNFULL field.
	PIO0_IRQ1_INTF_SM0_TXNFULL_Msk = 0x10
	// Bit SM0_TXNFULL.
	PIO0_IRQ1_INTF_SM0_TXNFULL = 0x10
	// Position of SM3_RXNEMPTY field.
	PIO0_IRQ1_INTF_SM3_RXNEMPTY_Pos = 0x3
	// Bit mask of SM3_RXNEMPTY field.
	PIO0_IRQ1_INTF_SM3_RXNEMPTY_Msk = 0x8
	// Bit SM3_RXNEMPTY.
	PIO0_IRQ1_INTF_SM3_RXNEMPTY = 0x8
	// Position of SM2_RXNEMPTY field.
	PIO0_IRQ1_INTF_SM2_RXNEMPTY_Pos = 0x2
	// Bit mask of SM2_RXNEMPTY field.
	PIO0_IRQ1_INTF_SM2_RXNEMPTY_Msk = 0x4
	// Bit SM2_RXNEMPTY.
	PIO0_IRQ1_INTF_SM2_RXNEMPTY = 0x4
	// Position of SM1_RXNEMPTY field.
	PIO0_IRQ1_INTF_SM1_RXNEMPTY_Pos = 0x1
	// Bit mask of SM1_RXNEMPTY field.
	PIO0_IRQ1_INTF_SM1_RXNEMPTY_Msk = 0x2
	// Bit SM1_RXNEMPTY.
	PIO0_IRQ1_INTF_SM1_RXNEMPTY = 0x2
	// Position of SM0_RXNEMPTY field.
	PIO0_IRQ1_INTF_SM0_RXNEMPTY_Pos = 0x0
	// Bit mask of SM0_RXNEMPTY field.
	PIO0_IRQ1_INTF_SM0_RXNEMPTY_Msk = 0x1
	// Bit SM0_RXNEMPTY.
	PIO0_IRQ1_INTF_SM0_RXNEMPTY = 0x1

	// IRQ1_INTS: Interrupt status after masking & forcing for irq1
	// Position of SM7 field.
	PIO0_IRQ1_INTS_SM7_Pos = 0xf
	// Bit mask of SM7 field.
	PIO0_IRQ1_INTS_SM7_Msk = 0x8000
	// Bit SM7.
	PIO0_IRQ1_INTS_SM7 = 0x8000
	// Position of SM6 field.
	PIO0_IRQ1_INTS_SM6_Pos = 0xe
	// Bit mask of SM6 field.
	PIO0_IRQ1_INTS_SM6_Msk = 0x4000
	// Bit SM6.
	PIO0_IRQ1_INTS_SM6 = 0x4000
	// Position of SM5 field.
	PIO0_IRQ1_INTS_SM5_Pos = 0xd
	// Bit mask of SM5 field.
	PIO0_IRQ1_INTS_SM5_Msk = 0x2000
	// Bit SM5.
	PIO0_IRQ1_INTS_SM5 = 0x2000
	// Position of SM4 field.
	PIO0_IRQ1_INTS_SM4_Pos = 0xc
	// Bit mask of SM4 field.
	PIO0_IRQ1_INTS_SM4_Msk = 0x1000
	// Bit SM4.
	PIO0_IRQ1_INTS_SM4 = 0x1000
	// Position of SM3 field.
	PIO0_IRQ1_INTS_SM3_Pos = 0xb
	// Bit mask of SM3 field.
	PIO0_IRQ1_INTS_SM3_Msk = 0x800
	// Bit SM3.
	PIO0_IRQ1_INTS_SM3 = 0x800
	// Position of SM2 field.
	PIO0_IRQ1_INTS_SM2_Pos = 0xa
	// Bit mask of SM2 field.
	PIO0_IRQ1_INTS_SM2_Msk = 0x400
	// Bit SM2.
	PIO0_IRQ1_INTS_SM2 = 0x400
	// Position of SM1 field.
	PIO0_IRQ1_INTS_SM1_Pos = 0x9
	// Bit mask of SM1 field.
	PIO0_IRQ1_INTS_SM1_Msk = 0x200
	// Bit SM1.
	PIO0_IRQ1_INTS_SM1 = 0x200
	// Position of SM0 field.
	PIO0_IRQ1_INTS_SM0_Pos = 0x8
	// Bit mask of SM0 field.
	PIO0_IRQ1_INTS_SM0_Msk = 0x100
	// Bit SM0.
	PIO0_IRQ1_INTS_SM0 = 0x100
	// Position of SM3_TXNFULL field.
	PIO0_IRQ1_INTS_SM3_TXNFULL_Pos = 0x7
	// Bit mask of SM3_TXNFULL field.
	PIO0_IRQ1_INTS_SM3_TXNFULL_Msk = 0x80
	// Bit SM3_TXNFULL.
	PIO0_IRQ1_INTS_SM3_TXNFULL = 0x80
	// Position of SM2_TXNFULL field.
	PIO0_IRQ1_INTS_SM2_TXNFULL_Pos = 0x6
	// Bit mask of SM2_TXNFULL field.
	PIO0_IRQ1_INTS_SM2_TXNFULL_Msk = 0x40
	// Bit SM2_TXNFULL.
	PIO0_IRQ1_INTS_SM2_TXNFULL = 0x40
	// Position of SM1_TXNFULL field.
	PIO0_IRQ1_INTS_SM1_TXNFULL_Pos = 0x5
	// Bit mask of SM1_TXNFULL field.
	PIO0_IRQ1_INTS_SM1_TXNFULL_Msk = 0x20
	// Bit SM1_TXNFULL.
	PIO0_IRQ1_INTS_SM1_TXNFULL = 0x20
	// Position of SM0_TXNFULL field.
	PIO0_IRQ1_INTS_SM0_TXNFULL_Pos = 0x4
	// Bit mask of SM0_TXNFULL field.
	PIO0_IRQ1_INTS_SM0_TXNFULL_Msk = 0x10
	// Bit SM0_TXNFULL.
	PIO0_IRQ1_INTS_SM0_TXNFULL = 0x10
	// Position of SM3_RXNEMPTY field.
	PIO0_IRQ1_INTS_SM3_RXNEMPTY_Pos = 0x3
	// Bit mask of SM3_RXNEMPTY field.
	PIO0_IRQ1_INTS_SM3_RXNEMPTY_Msk = 0x8
	// Bit SM3_RXNEMPTY.
	PIO0_IRQ1_INTS_SM3_RXNEMPTY = 0x8
	// Position of SM2_RXNEMPTY field.
	PIO0_IRQ1_INTS_SM2_RXNEMPTY_Pos = 0x2
	// Bit mask of SM2_RXNEMPTY field.
	PIO0_IRQ1_INTS_SM2_RXNEMPTY_Msk = 0x4
	// Bit SM2_RXNEMPTY.
	PIO0_IRQ1_INTS_SM2_RXNEMPTY = 0x4
	// Position of SM1_RXNEMPTY field.
	PIO0_IRQ1_INTS_SM1_RXNEMPTY_Pos = 0x1
	// Bit mask of SM1_RXNEMPTY field.
	PIO0_IRQ1_INTS_SM1_RXNEMPTY_Msk = 0x2
	// Bit SM1_RXNEMPTY.
	PIO0_IRQ1_INTS_SM1_RXNEMPTY = 0x2
	// Position of SM0_RXNEMPTY field.
	PIO0_IRQ1_INTS_SM0_RXNEMPTY_Pos = 0x0
	// Bit mask of SM0_RXNEMPTY field.
	PIO0_IRQ1_INTS_SM0_RXNEMPTY_Msk = 0x1
	// Bit SM0_RXNEMPTY.
	PIO0_IRQ1_INTS_SM0_RXNEMPTY = 0x1
)

// Constants for BUSCTRL: Register block for busfabric control signals and performance counters
const (
	// BUS_PRIORITY: Set the priority of each master for bus arbitration.
	// Position of DMA_W field.
	BUSCTRL_BUS_PRIORITY_DMA_W_Pos = 0xc
	// Bit mask of DMA_W field.
	BUSCTRL_BUS_PRIORITY_DMA_W_Msk = 0x1000
	// Bit DMA_W.
	BUSCTRL_BUS_PRIORITY_DMA_W = 0x1000
	// Position of DMA_R field.
	BUSCTRL_BUS_PRIORITY_DMA_R_Pos = 0x8
	// Bit mask of DMA_R field.
	BUSCTRL_BUS_PRIORITY_DMA_R_Msk = 0x100
	// Bit DMA_R.
	BUSCTRL_BUS_PRIORITY_DMA_R = 0x100
	// Position of PROC1 field.
	BUSCTRL_BUS_PRIORITY_PROC1_Pos = 0x4
	// Bit mask of PROC1 field.
	BUSCTRL_BUS_PRIORITY_PROC1_Msk = 0x10
	// Bit PROC1.
	BUSCTRL_BUS_PRIORITY_PROC1 = 0x10
	// Position of PROC0 field.
	BUSCTRL_BUS_PRIORITY_PROC0_Pos = 0x0
	// Bit mask of PROC0 field.
	BUSCTRL_BUS_PRIORITY_PROC0_Msk = 0x1
	// Bit PROC0.
	BUSCTRL_BUS_PRIORITY_PROC0 = 0x1

	// BUS_PRIORITY_ACK: Bus priority acknowledge
	// Position of BUS_PRIORITY_ACK field.
	BUSCTRL_BUS_PRIORITY_ACK_BUS_PRIORITY_ACK_Pos = 0x0
	// Bit mask of BUS_PRIORITY_ACK field.
	BUSCTRL_BUS_PRIORITY_ACK_BUS_PRIORITY_ACK_Msk = 0x1
	// Bit BUS_PRIORITY_ACK.
	BUSCTRL_BUS_PRIORITY_ACK_BUS_PRIORITY_ACK = 0x1

	// PERFCTR_EN: Enable the performance counters. If 0, the performance counters do not increment. This can be used to precisely start/stop event sampling around the profiled section of code. The performance counters are initially disabled, to save energy.
	// Position of PERFCTR_EN field.
	BUSCTRL_PERFCTR_EN_PERFCTR_EN_Pos = 0x0
	// Bit mask of PERFCTR_EN field.
	BUSCTRL_PERFCTR_EN_PERFCTR_EN_Msk = 0x1
	// Bit PERFCTR_EN.
	BUSCTRL_PERFCTR_EN_PERFCTR_EN = 0x1

	// PERFCTR0: Bus fabric performance counter 0
	// Position of PERFCTR0 field.
	BUSCTRL_PERFCTR0_PERFCTR0_Pos = 0x0
	// Bit mask of PERFCTR0 field.
	BUSCTRL_PERFCTR0_PERFCTR0_Msk = 0xffffff

	// PERFSEL0: Bus fabric performance event select for PERFCTR0
	// Position of PERFSEL0 field.
	BUSCTRL_PERFSEL0_PERFSEL0_Pos = 0x0
	// Bit mask of PERFSEL0 field.
	BUSCTRL_PERFSEL0_PERFSEL0_Msk                         = 0x7f
	BUSCTRL_PERFSEL0_PERFSEL0_SIOB_PROC1_STALL_UPSTREAM   = 0x0
	BUSCTRL_PERFSEL0_PERFSEL0_SIOB_PROC1_STALL_DOWNSTREAM = 0x1
	BUSCTRL_PERFSEL0_PERFSEL0_SIOB_PROC1_ACCESS_CONTESTED = 0x2
	BUSCTRL_PERFSEL0_PERFSEL0_SIOB_PROC1_ACCESS           = 0x3
	BUSCTRL_PERFSEL0_PERFSEL0_SIOB_PROC0_STALL_UPSTREAM   = 0x4
	BUSCTRL_PERFSEL0_PERFSEL0_SIOB_PROC0_STALL_DOWNSTREAM = 0x5
	BUSCTRL_PERFSEL0_PERFSEL0_SIOB_PROC0_ACCESS_CONTESTED = 0x6
	BUSCTRL_PERFSEL0_PERFSEL0_SIOB_PROC0_ACCESS           = 0x7
	BUSCTRL_PERFSEL0_PERFSEL0_APB_STALL_UPSTREAM          = 0x8
	BUSCTRL_PERFSEL0_PERFSEL0_APB_STALL_DOWNSTREAM        = 0x9
	BUSCTRL_PERFSEL0_PERFSEL0_APB_ACCESS_CONTESTED        = 0xa
	BUSCTRL_PERFSEL0_PERFSEL0_APB_ACCESS                  = 0xb
	BUSCTRL_PERFSEL0_PERFSEL0_FASTPERI_STALL_UPSTREAM     = 0xc
	BUSCTRL_PERFSEL0_PERFSEL0_FASTPERI_STALL_DOWNSTREAM   = 0xd
	BUSCTRL_PERFSEL0_PERFSEL0_FASTPERI_ACCESS_CONTESTED   = 0xe
	BUSCTRL_PERFSEL0_PERFSEL0_FASTPERI_ACCESS             = 0xf
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM9_STALL_UPSTREAM        = 0x10
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM9_STALL_DOWNSTREAM      = 0x11
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM9_ACCESS_CONTESTED      = 0x12
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM9_ACCESS                = 0x13
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM8_STALL_UPSTREAM        = 0x14
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM8_STALL_DOWNSTREAM      = 0x15
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM8_ACCESS_CONTESTED      = 0x16
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM8_ACCESS                = 0x17
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM7_STALL_UPSTREAM        = 0x18
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM7_STALL_DOWNSTREAM      = 0x19
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM7_ACCESS_CONTESTED      = 0x1a
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM7_ACCESS                = 0x1b
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM6_STALL_UPSTREAM        = 0x1c
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM6_STALL_DOWNSTREAM      = 0x1d
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM6_ACCESS_CONTESTED      = 0x1e
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM6_ACCESS                = 0x1f
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM5_STALL_UPSTREAM        = 0x20
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM5_STALL_DOWNSTREAM      = 0x21
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM5_ACCESS_CONTESTED      = 0x22
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM5_ACCESS                = 0x23
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM4_STALL_UPSTREAM        = 0x24
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM4_STALL_DOWNSTREAM      = 0x25
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM4_ACCESS_CONTESTED      = 0x26
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM4_ACCESS                = 0x27
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM3_STALL_UPSTREAM        = 0x28
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM3_STALL_DOWNSTREAM      = 0x29
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM3_ACCESS_CONTESTED      = 0x2a
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM3_ACCESS                = 0x2b
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM2_STALL_UPSTREAM        = 0x2c
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM2_STALL_DOWNSTREAM      = 0x2d
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM2_ACCESS_CONTESTED      = 0x2e
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM2_ACCESS                = 0x2f
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM1_STALL_UPSTREAM        = 0x30
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM1_STALL_DOWNSTREAM      = 0x31
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM1_ACCESS_CONTESTED      = 0x32
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM1_ACCESS                = 0x33
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM0_STALL_UPSTREAM        = 0x34
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM0_STALL_DOWNSTREAM      = 0x35
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM0_ACCESS_CONTESTED      = 0x36
	BUSCTRL_PERFSEL0_PERFSEL0_SRAM0_ACCESS                = 0x37
	BUSCTRL_PERFSEL0_PERFSEL0_XIP_MAIN1_STALL_UPSTREAM    = 0x38
	BUSCTRL_PERFSEL0_PERFSEL0_XIP_MAIN1_STALL_DOWNSTREAM  = 0x39
	BUSCTRL_PERFSEL0_PERFSEL0_XIP_MAIN1_ACCESS_CONTESTED  = 0x3a
	BUSCTRL_PERFSEL0_PERFSEL0_XIP_MAIN1_ACCESS            = 0x3b
	BUSCTRL_PERFSEL0_PERFSEL0_XIP_MAIN0_STALL_UPSTREAM    = 0x3c
	BUSCTRL_PERFSEL0_PERFSEL0_XIP_MAIN0_STALL_DOWNSTREAM  = 0x3d
	BUSCTRL_PERFSEL0_PERFSEL0_XIP_MAIN0_ACCESS_CONTESTED  = 0x3e
	BUSCTRL_PERFSEL0_PERFSEL0_XIP_MAIN0_ACCESS            = 0x3f
	BUSCTRL_PERFSEL0_PERFSEL0_ROM_STALL_UPSTREAM          = 0x40
	BUSCTRL_PERFSEL0_PERFSEL0_ROM_STALL_DOWNSTREAM        = 0x41
	BUSCTRL_PERFSEL0_PERFSEL0_ROM_ACCESS_CONTESTED        = 0x42
	BUSCTRL_PERFSEL0_PERFSEL0_ROM_ACCESS                  = 0x43

	// PERFCTR1: Bus fabric performance counter 1
	// Position of PERFCTR1 field.
	BUSCTRL_PERFCTR1_PERFCTR1_Pos = 0x0
	// Bit mask of PERFCTR1 field.
	BUSCTRL_PERFCTR1_PERFCTR1_Msk = 0xffffff

	// PERFSEL1: Bus fabric performance event select for PERFCTR1
	// Position of PERFSEL1 field.
	BUSCTRL_PERFSEL1_PERFSEL1_Pos = 0x0
	// Bit mask of PERFSEL1 field.
	BUSCTRL_PERFSEL1_PERFSEL1_Msk                         = 0x7f
	BUSCTRL_PERFSEL1_PERFSEL1_SIOB_PROC1_STALL_UPSTREAM   = 0x0
	BUSCTRL_PERFSEL1_PERFSEL1_SIOB_PROC1_STALL_DOWNSTREAM = 0x1
	BUSCTRL_PERFSEL1_PERFSEL1_SIOB_PROC1_ACCESS_CONTESTED = 0x2
	BUSCTRL_PERFSEL1_PERFSEL1_SIOB_PROC1_ACCESS           = 0x3
	BUSCTRL_PERFSEL1_PERFSEL1_SIOB_PROC0_STALL_UPSTREAM   = 0x4
	BUSCTRL_PERFSEL1_PERFSEL1_SIOB_PROC0_STALL_DOWNSTREAM = 0x5
	BUSCTRL_PERFSEL1_PERFSEL1_SIOB_PROC0_ACCESS_CONTESTED = 0x6
	BUSCTRL_PERFSEL1_PERFSEL1_SIOB_PROC0_ACCESS           = 0x7
	BUSCTRL_PERFSEL1_PERFSEL1_APB_STALL_UPSTREAM          = 0x8
	BUSCTRL_PERFSEL1_PERFSEL1_APB_STALL_DOWNSTREAM        = 0x9
	BUSCTRL_PERFSEL1_PERFSEL1_APB_ACCESS_CONTESTED        = 0xa
	BUSCTRL_PERFSEL1_PERFSEL1_APB_ACCESS                  = 0xb
	BUSCTRL_PERFSEL1_PERFSEL1_FASTPERI_STALL_UPSTREAM     = 0xc
	BUSCTRL_PERFSEL1_PERFSEL1_FASTPERI_STALL_DOWNSTREAM   = 0xd
	BUSCTRL_PERFSEL1_PERFSEL1_FASTPERI_ACCESS_CONTESTED   = 0xe
	BUSCTRL_PERFSEL1_PERFSEL1_FASTPERI_ACCESS             = 0xf
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM9_STALL_UPSTREAM        = 0x10
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM9_STALL_DOWNSTREAM      = 0x11
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM9_ACCESS_CONTESTED      = 0x12
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM9_ACCESS                = 0x13
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM8_STALL_UPSTREAM        = 0x14
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM8_STALL_DOWNSTREAM      = 0x15
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM8_ACCESS_CONTESTED      = 0x16
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM8_ACCESS                = 0x17
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM7_STALL_UPSTREAM        = 0x18
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM7_STALL_DOWNSTREAM      = 0x19
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM7_ACCESS_CONTESTED      = 0x1a
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM7_ACCESS                = 0x1b
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM6_STALL_UPSTREAM        = 0x1c
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM6_STALL_DOWNSTREAM      = 0x1d
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM6_ACCESS_CONTESTED      = 0x1e
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM6_ACCESS                = 0x1f
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM5_STALL_UPSTREAM        = 0x20
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM5_STALL_DOWNSTREAM      = 0x21
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM5_ACCESS_CONTESTED      = 0x22
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM5_ACCESS                = 0x23
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM4_STALL_UPSTREAM        = 0x24
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM4_STALL_DOWNSTREAM      = 0x25
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM4_ACCESS_CONTESTED      = 0x26
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM4_ACCESS                = 0x27
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM3_STALL_UPSTREAM        = 0x28
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM3_STALL_DOWNSTREAM      = 0x29
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM3_ACCESS_CONTESTED      = 0x2a
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM3_ACCESS                = 0x2b
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM2_STALL_UPSTREAM        = 0x2c
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM2_STALL_DOWNSTREAM      = 0x2d
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM2_ACCESS_CONTESTED      = 0x2e
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM2_ACCESS                = 0x2f
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM1_STALL_UPSTREAM        = 0x30
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM1_STALL_DOWNSTREAM      = 0x31
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM1_ACCESS_CONTESTED      = 0x32
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM1_ACCESS                = 0x33
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM0_STALL_UPSTREAM        = 0x34
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM0_STALL_DOWNSTREAM      = 0x35
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM0_ACCESS_CONTESTED      = 0x36
	BUSCTRL_PERFSEL1_PERFSEL1_SRAM0_ACCESS                = 0x37
	BUSCTRL_PERFSEL1_PERFSEL1_XIP_MAIN1_STALL_UPSTREAM    = 0x38
	BUSCTRL_PERFSEL1_PERFSEL1_XIP_MAIN1_STALL_DOWNSTREAM  = 0x39
	BUSCTRL_PERFSEL1_PERFSEL1_XIP_MAIN1_ACCESS_CONTESTED  = 0x3a
	BUSCTRL_PERFSEL1_PERFSEL1_XIP_MAIN1_ACCESS            = 0x3b
	BUSCTRL_PERFSEL1_PERFSEL1_XIP_MAIN0_STALL_UPSTREAM    = 0x3c
	BUSCTRL_PERFSEL1_PERFSEL1_XIP_MAIN0_STALL_DOWNSTREAM  = 0x3d
	BUSCTRL_PERFSEL1_PERFSEL1_XIP_MAIN0_ACCESS_CONTESTED  = 0x3e
	BUSCTRL_PERFSEL1_PERFSEL1_XIP_MAIN0_ACCESS            = 0x3f
	BUSCTRL_PERFSEL1_PERFSEL1_ROM_STALL_UPSTREAM          = 0x40
	BUSCTRL_PERFSEL1_PERFSEL1_ROM_STALL_DOWNSTREAM        = 0x41
	BUSCTRL_PERFSEL1_PERFSEL1_ROM_ACCESS_CONTESTED        = 0x42
	BUSCTRL_PERFSEL1_PERFSEL1_ROM_ACCESS                  = 0x43

	// PERFCTR2: Bus fabric performance counter 2
	// Position of PERFCTR2 field.
	BUSCTRL_PERFCTR2_PERFCTR2_Pos = 0x0
	// Bit mask of PERFCTR2 field.
	BUSCTRL_PERFCTR2_PERFCTR2_Msk = 0xffffff

	// PERFSEL2: Bus fabric performance event select for PERFCTR2
	// Position of PERFSEL2 field.
	BUSCTRL_PERFSEL2_PERFSEL2_Pos = 0x0
	// Bit mask of PERFSEL2 field.
	BUSCTRL_PERFSEL2_PERFSEL2_Msk                         = 0x7f
	BUSCTRL_PERFSEL2_PERFSEL2_SIOB_PROC1_STALL_UPSTREAM   = 0x0
	BUSCTRL_PERFSEL2_PERFSEL2_SIOB_PROC1_STALL_DOWNSTREAM = 0x1
	BUSCTRL_PERFSEL2_PERFSEL2_SIOB_PROC1_ACCESS_CONTESTED = 0x2
	BUSCTRL_PERFSEL2_PERFSEL2_SIOB_PROC1_ACCESS           = 0x3
	BUSCTRL_PERFSEL2_PERFSEL2_SIOB_PROC0_STALL_UPSTREAM   = 0x4
	BUSCTRL_PERFSEL2_PERFSEL2_SIOB_PROC0_STALL_DOWNSTREAM = 0x5
	BUSCTRL_PERFSEL2_PERFSEL2_SIOB_PROC0_ACCESS_CONTESTED = 0x6
	BUSCTRL_PERFSEL2_PERFSEL2_SIOB_PROC0_ACCESS           = 0x7
	BUSCTRL_PERFSEL2_PERFSEL2_APB_STALL_UPSTREAM          = 0x8
	BUSCTRL_PERFSEL2_PERFSEL2_APB_STALL_DOWNSTREAM        = 0x9
	BUSCTRL_PERFSEL2_PERFSEL2_APB_ACCESS_CONTESTED        = 0xa
	BUSCTRL_PERFSEL2_PERFSEL2_APB_ACCESS                  = 0xb
	BUSCTRL_PERFSEL2_PERFSEL2_FASTPERI_STALL_UPSTREAM     = 0xc
	BUSCTRL_PERFSEL2_PERFSEL2_FASTPERI_STALL_DOWNSTREAM   = 0xd
	BUSCTRL_PERFSEL2_PERFSEL2_FASTPERI_ACCESS_CONTESTED   = 0xe
	BUSCTRL_PERFSEL2_PERFSEL2_FASTPERI_ACCESS             = 0xf
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM9_STALL_UPSTREAM        = 0x10
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM9_STALL_DOWNSTREAM      = 0x11
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM9_ACCESS_CONTESTED      = 0x12
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM9_ACCESS                = 0x13
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM8_STALL_UPSTREAM        = 0x14
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM8_STALL_DOWNSTREAM      = 0x15
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM8_ACCESS_CONTESTED      = 0x16
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM8_ACCESS                = 0x17
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM7_STALL_UPSTREAM        = 0x18
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM7_STALL_DOWNSTREAM      = 0x19
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM7_ACCESS_CONTESTED      = 0x1a
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM7_ACCESS                = 0x1b
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM6_STALL_UPSTREAM        = 0x1c
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM6_STALL_DOWNSTREAM      = 0x1d
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM6_ACCESS_CONTESTED      = 0x1e
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM6_ACCESS                = 0x1f
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM5_STALL_UPSTREAM        = 0x20
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM5_STALL_DOWNSTREAM      = 0x21
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM5_ACCESS_CONTESTED      = 0x22
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM5_ACCESS                = 0x23
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM4_STALL_UPSTREAM        = 0x24
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM4_STALL_DOWNSTREAM      = 0x25
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM4_ACCESS_CONTESTED      = 0x26
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM4_ACCESS                = 0x27
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM3_STALL_UPSTREAM        = 0x28
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM3_STALL_DOWNSTREAM      = 0x29
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM3_ACCESS_CONTESTED      = 0x2a
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM3_ACCESS                = 0x2b
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM2_STALL_UPSTREAM        = 0x2c
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM2_STALL_DOWNSTREAM      = 0x2d
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM2_ACCESS_CONTESTED      = 0x2e
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM2_ACCESS                = 0x2f
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM1_STALL_UPSTREAM        = 0x30
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM1_STALL_DOWNSTREAM      = 0x31
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM1_ACCESS_CONTESTED      = 0x32
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM1_ACCESS                = 0x33
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM0_STALL_UPSTREAM        = 0x34
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM0_STALL_DOWNSTREAM      = 0x35
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM0_ACCESS_CONTESTED      = 0x36
	BUSCTRL_PERFSEL2_PERFSEL2_SRAM0_ACCESS                = 0x37
	BUSCTRL_PERFSEL2_PERFSEL2_XIP_MAIN1_STALL_UPSTREAM    = 0x38
	BUSCTRL_PERFSEL2_PERFSEL2_XIP_MAIN1_STALL_DOWNSTREAM  = 0x39
	BUSCTRL_PERFSEL2_PERFSEL2_XIP_MAIN1_ACCESS_CONTESTED  = 0x3a
	BUSCTRL_PERFSEL2_PERFSEL2_XIP_MAIN1_ACCESS            = 0x3b
	BUSCTRL_PERFSEL2_PERFSEL2_XIP_MAIN0_STALL_UPSTREAM    = 0x3c
	BUSCTRL_PERFSEL2_PERFSEL2_XIP_MAIN0_STALL_DOWNSTREAM  = 0x3d
	BUSCTRL_PERFSEL2_PERFSEL2_XIP_MAIN0_ACCESS_CONTESTED  = 0x3e
	BUSCTRL_PERFSEL2_PERFSEL2_XIP_MAIN0_ACCESS            = 0x3f
	BUSCTRL_PERFSEL2_PERFSEL2_ROM_STALL_UPSTREAM          = 0x40
	BUSCTRL_PERFSEL2_PERFSEL2_ROM_STALL_DOWNSTREAM        = 0x41
	BUSCTRL_PERFSEL2_PERFSEL2_ROM_ACCESS_CONTESTED        = 0x42
	BUSCTRL_PERFSEL2_PERFSEL2_ROM_ACCESS                  = 0x43

	// PERFCTR3: Bus fabric performance counter 3
	// Position of PERFCTR3 field.
	BUSCTRL_PERFCTR3_PERFCTR3_Pos = 0x0
	// Bit mask of PERFCTR3 field.
	BUSCTRL_PERFCTR3_PERFCTR3_Msk = 0xffffff

	// PERFSEL3: Bus fabric performance event select for PERFCTR3
	// Position of PERFSEL3 field.
	BUSCTRL_PERFSEL3_PERFSEL3_Pos = 0x0
	// Bit mask of PERFSEL3 field.
	BUSCTRL_PERFSEL3_PERFSEL3_Msk                         = 0x7f
	BUSCTRL_PERFSEL3_PERFSEL3_SIOB_PROC1_STALL_UPSTREAM   = 0x0
	BUSCTRL_PERFSEL3_PERFSEL3_SIOB_PROC1_STALL_DOWNSTREAM = 0x1
	BUSCTRL_PERFSEL3_PERFSEL3_SIOB_PROC1_ACCESS_CONTESTED = 0x2
	BUSCTRL_PERFSEL3_PERFSEL3_SIOB_PROC1_ACCESS           = 0x3
	BUSCTRL_PERFSEL3_PERFSEL3_SIOB_PROC0_STALL_UPSTREAM   = 0x4
	BUSCTRL_PERFSEL3_PERFSEL3_SIOB_PROC0_STALL_DOWNSTREAM = 0x5
	BUSCTRL_PERFSEL3_PERFSEL3_SIOB_PROC0_ACCESS_CONTESTED = 0x6
	BUSCTRL_PERFSEL3_PERFSEL3_SIOB_PROC0_ACCESS           = 0x7
	BUSCTRL_PERFSEL3_PERFSEL3_APB_STALL_UPSTREAM          = 0x8
	BUSCTRL_PERFSEL3_PERFSEL3_APB_STALL_DOWNSTREAM        = 0x9
	BUSCTRL_PERFSEL3_PERFSEL3_APB_ACCESS_CONTESTED        = 0xa
	BUSCTRL_PERFSEL3_PERFSEL3_APB_ACCESS                  = 0xb
	BUSCTRL_PERFSEL3_PERFSEL3_FASTPERI_STALL_UPSTREAM     = 0xc
	BUSCTRL_PERFSEL3_PERFSEL3_FASTPERI_STALL_DOWNSTREAM   = 0xd
	BUSCTRL_PERFSEL3_PERFSEL3_FASTPERI_ACCESS_CONTESTED   = 0xe
	BUSCTRL_PERFSEL3_PERFSEL3_FASTPERI_ACCESS             = 0xf
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM9_STALL_UPSTREAM        = 0x10
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM9_STALL_DOWNSTREAM      = 0x11
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM9_ACCESS_CONTESTED      = 0x12
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM9_ACCESS                = 0x13
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM8_STALL_UPSTREAM        = 0x14
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM8_STALL_DOWNSTREAM      = 0x15
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM8_ACCESS_CONTESTED      = 0x16
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM8_ACCESS                = 0x17
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM7_STALL_UPSTREAM        = 0x18
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM7_STALL_DOWNSTREAM      = 0x19
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM7_ACCESS_CONTESTED      = 0x1a
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM7_ACCESS                = 0x1b
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM6_STALL_UPSTREAM        = 0x1c
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM6_STALL_DOWNSTREAM      = 0x1d
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM6_ACCESS_CONTESTED      = 0x1e
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM6_ACCESS                = 0x1f
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM5_STALL_UPSTREAM        = 0x20
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM5_STALL_DOWNSTREAM      = 0x21
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM5_ACCESS_CONTESTED      = 0x22
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM5_ACCESS                = 0x23
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM4_STALL_UPSTREAM        = 0x24
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM4_STALL_DOWNSTREAM      = 0x25
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM4_ACCESS_CONTESTED      = 0x26
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM4_ACCESS                = 0x27
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM3_STALL_UPSTREAM        = 0x28
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM3_STALL_DOWNSTREAM      = 0x29
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM3_ACCESS_CONTESTED      = 0x2a
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM3_ACCESS                = 0x2b
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM2_STALL_UPSTREAM        = 0x2c
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM2_STALL_DOWNSTREAM      = 0x2d
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM2_ACCESS_CONTESTED      = 0x2e
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM2_ACCESS                = 0x2f
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM1_STALL_UPSTREAM        = 0x30
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM1_STALL_DOWNSTREAM      = 0x31
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM1_ACCESS_CONTESTED      = 0x32
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM1_ACCESS                = 0x33
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM0_STALL_UPSTREAM        = 0x34
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM0_STALL_DOWNSTREAM      = 0x35
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM0_ACCESS_CONTESTED      = 0x36
	BUSCTRL_PERFSEL3_PERFSEL3_SRAM0_ACCESS                = 0x37
	BUSCTRL_PERFSEL3_PERFSEL3_XIP_MAIN1_STALL_UPSTREAM    = 0x38
	BUSCTRL_PERFSEL3_PERFSEL3_XIP_MAIN1_STALL_DOWNSTREAM  = 0x39
	BUSCTRL_PERFSEL3_PERFSEL3_XIP_MAIN1_ACCESS_CONTESTED  = 0x3a
	BUSCTRL_PERFSEL3_PERFSEL3_XIP_MAIN1_ACCESS            = 0x3b
	BUSCTRL_PERFSEL3_PERFSEL3_XIP_MAIN0_STALL_UPSTREAM    = 0x3c
	BUSCTRL_PERFSEL3_PERFSEL3_XIP_MAIN0_STALL_DOWNSTREAM  = 0x3d
	BUSCTRL_PERFSEL3_PERFSEL3_XIP_MAIN0_ACCESS_CONTESTED  = 0x3e
	BUSCTRL_PERFSEL3_PERFSEL3_XIP_MAIN0_ACCESS            = 0x3f
	BUSCTRL_PERFSEL3_PERFSEL3_ROM_STALL_UPSTREAM          = 0x40
	BUSCTRL_PERFSEL3_PERFSEL3_ROM_STALL_DOWNSTREAM        = 0x41
	BUSCTRL_PERFSEL3_PERFSEL3_ROM_ACCESS_CONTESTED        = 0x42
	BUSCTRL_PERFSEL3_PERFSEL3_ROM_ACCESS                  = 0x43
)

// Constants for SIO: Single-cycle IO block Provides core-local and inter-core hardware for the two processors, with single-cycle access.
const (
	// CPUID: Processor core identifier
	// Position of CPUID field.
	SIO_CPUID_CPUID_Pos = 0x0
	// Bit mask of CPUID field.
	SIO_CPUID_CPUID_Msk = 0xffffffff

	// GPIO_IN: Input value for GPIO0...31. In the Non-secure SIO, Secure-only GPIOs (as per ACCESSCTRL) appear as zero.
	// Position of GPIO_IN field.
	SIO_GPIO_IN_GPIO_IN_Pos = 0x0
	// Bit mask of GPIO_IN field.
	SIO_GPIO_IN_GPIO_IN_Msk = 0xffffffff

	// GPIO_HI_IN: Input value on GPIO32...47, QSPI IOs and USB pins In the Non-secure SIO, Secure-only GPIOs (as per ACCESSCTRL) appear as zero.
	// Position of QSPI_SD field.
	SIO_GPIO_HI_IN_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SIO_GPIO_HI_IN_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SIO_GPIO_HI_IN_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SIO_GPIO_HI_IN_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SIO_GPIO_HI_IN_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SIO_GPIO_HI_IN_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SIO_GPIO_HI_IN_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SIO_GPIO_HI_IN_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SIO_GPIO_HI_IN_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SIO_GPIO_HI_IN_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SIO_GPIO_HI_IN_USB_DM = 0x2000000
	// Position of USB_DP field.
	SIO_GPIO_HI_IN_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SIO_GPIO_HI_IN_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SIO_GPIO_HI_IN_USB_DP = 0x1000000
	// Position of GPIO field.
	SIO_GPIO_HI_IN_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SIO_GPIO_HI_IN_GPIO_Msk = 0xffff

	// GPIO_OUT: GPIO0...31 output value
	// Position of GPIO_OUT field.
	SIO_GPIO_OUT_GPIO_OUT_Pos = 0x0
	// Bit mask of GPIO_OUT field.
	SIO_GPIO_OUT_GPIO_OUT_Msk = 0xffffffff

	// GPIO_HI_OUT: Output value for GPIO32...47, QSPI IOs and USB pins. Write to set output level (1/0 -> high/low). Reading back gives the last value written, NOT the input value from the pins. If core 0 and core 1 both write to GPIO_HI_OUT simultaneously (or to a SET/CLR/XOR alias), the result is as though the write from core 0 took place first, and the write from core 1 was then applied to that intermediate result. In the Non-secure SIO, Secure-only GPIOs (as per ACCESSCTRL) ignore writes, and their output status reads back as zero. This is also true for SET/CLR/XOR aliases of this register.
	// Position of QSPI_SD field.
	SIO_GPIO_HI_OUT_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SIO_GPIO_HI_OUT_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SIO_GPIO_HI_OUT_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SIO_GPIO_HI_OUT_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SIO_GPIO_HI_OUT_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SIO_GPIO_HI_OUT_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SIO_GPIO_HI_OUT_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SIO_GPIO_HI_OUT_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SIO_GPIO_HI_OUT_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SIO_GPIO_HI_OUT_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SIO_GPIO_HI_OUT_USB_DM = 0x2000000
	// Position of USB_DP field.
	SIO_GPIO_HI_OUT_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SIO_GPIO_HI_OUT_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SIO_GPIO_HI_OUT_USB_DP = 0x1000000
	// Position of GPIO field.
	SIO_GPIO_HI_OUT_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SIO_GPIO_HI_OUT_GPIO_Msk = 0xffff

	// GPIO_OUT_SET: GPIO0...31 output value set
	// Position of GPIO_OUT_SET field.
	SIO_GPIO_OUT_SET_GPIO_OUT_SET_Pos = 0x0
	// Bit mask of GPIO_OUT_SET field.
	SIO_GPIO_OUT_SET_GPIO_OUT_SET_Msk = 0xffffffff

	// GPIO_HI_OUT_SET: Output value set for GPIO32..47, QSPI IOs and USB pins. Perform an atomic bit-set on GPIO_HI_OUT, i.e. `GPIO_HI_OUT |= wdata`
	// Position of QSPI_SD field.
	SIO_GPIO_HI_OUT_SET_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SIO_GPIO_HI_OUT_SET_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SIO_GPIO_HI_OUT_SET_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SIO_GPIO_HI_OUT_SET_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SIO_GPIO_HI_OUT_SET_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SIO_GPIO_HI_OUT_SET_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SIO_GPIO_HI_OUT_SET_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SIO_GPIO_HI_OUT_SET_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SIO_GPIO_HI_OUT_SET_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SIO_GPIO_HI_OUT_SET_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SIO_GPIO_HI_OUT_SET_USB_DM = 0x2000000
	// Position of USB_DP field.
	SIO_GPIO_HI_OUT_SET_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SIO_GPIO_HI_OUT_SET_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SIO_GPIO_HI_OUT_SET_USB_DP = 0x1000000
	// Position of GPIO field.
	SIO_GPIO_HI_OUT_SET_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SIO_GPIO_HI_OUT_SET_GPIO_Msk = 0xffff

	// GPIO_OUT_CLR: GPIO0...31 output value clear
	// Position of GPIO_OUT_CLR field.
	SIO_GPIO_OUT_CLR_GPIO_OUT_CLR_Pos = 0x0
	// Bit mask of GPIO_OUT_CLR field.
	SIO_GPIO_OUT_CLR_GPIO_OUT_CLR_Msk = 0xffffffff

	// GPIO_HI_OUT_CLR: Output value clear for GPIO32..47, QSPI IOs and USB pins. Perform an atomic bit-clear on GPIO_HI_OUT, i.e. `GPIO_HI_OUT &= ~wdata`
	// Position of QSPI_SD field.
	SIO_GPIO_HI_OUT_CLR_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SIO_GPIO_HI_OUT_CLR_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SIO_GPIO_HI_OUT_CLR_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SIO_GPIO_HI_OUT_CLR_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SIO_GPIO_HI_OUT_CLR_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SIO_GPIO_HI_OUT_CLR_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SIO_GPIO_HI_OUT_CLR_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SIO_GPIO_HI_OUT_CLR_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SIO_GPIO_HI_OUT_CLR_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SIO_GPIO_HI_OUT_CLR_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SIO_GPIO_HI_OUT_CLR_USB_DM = 0x2000000
	// Position of USB_DP field.
	SIO_GPIO_HI_OUT_CLR_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SIO_GPIO_HI_OUT_CLR_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SIO_GPIO_HI_OUT_CLR_USB_DP = 0x1000000
	// Position of GPIO field.
	SIO_GPIO_HI_OUT_CLR_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SIO_GPIO_HI_OUT_CLR_GPIO_Msk = 0xffff

	// GPIO_OUT_XOR: GPIO0...31 output value XOR
	// Position of GPIO_OUT_XOR field.
	SIO_GPIO_OUT_XOR_GPIO_OUT_XOR_Pos = 0x0
	// Bit mask of GPIO_OUT_XOR field.
	SIO_GPIO_OUT_XOR_GPIO_OUT_XOR_Msk = 0xffffffff

	// GPIO_HI_OUT_XOR: Output value XOR for GPIO32..47, QSPI IOs and USB pins. Perform an atomic bitwise XOR on GPIO_HI_OUT, i.e. `GPIO_HI_OUT ^= wdata`
	// Position of QSPI_SD field.
	SIO_GPIO_HI_OUT_XOR_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SIO_GPIO_HI_OUT_XOR_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SIO_GPIO_HI_OUT_XOR_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SIO_GPIO_HI_OUT_XOR_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SIO_GPIO_HI_OUT_XOR_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SIO_GPIO_HI_OUT_XOR_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SIO_GPIO_HI_OUT_XOR_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SIO_GPIO_HI_OUT_XOR_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SIO_GPIO_HI_OUT_XOR_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SIO_GPIO_HI_OUT_XOR_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SIO_GPIO_HI_OUT_XOR_USB_DM = 0x2000000
	// Position of USB_DP field.
	SIO_GPIO_HI_OUT_XOR_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SIO_GPIO_HI_OUT_XOR_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SIO_GPIO_HI_OUT_XOR_USB_DP = 0x1000000
	// Position of GPIO field.
	SIO_GPIO_HI_OUT_XOR_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SIO_GPIO_HI_OUT_XOR_GPIO_Msk = 0xffff

	// GPIO_OE: GPIO0...31 output enable
	// Position of GPIO_OE field.
	SIO_GPIO_OE_GPIO_OE_Pos = 0x0
	// Bit mask of GPIO_OE field.
	SIO_GPIO_OE_GPIO_OE_Msk = 0xffffffff

	// GPIO_HI_OE: Output enable value for GPIO32...47, QSPI IOs and USB pins. Write output enable (1/0 -> output/input). Reading back gives the last value written. If core 0 and core 1 both write to GPIO_HI_OE simultaneously (or to a SET/CLR/XOR alias), the result is as though the write from core 0 took place first, and the write from core 1 was then applied to that intermediate result. In the Non-secure SIO, Secure-only GPIOs (as per ACCESSCTRL) ignore writes, and their output status reads back as zero. This is also true for SET/CLR/XOR aliases of this register.
	// Position of QSPI_SD field.
	SIO_GPIO_HI_OE_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SIO_GPIO_HI_OE_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SIO_GPIO_HI_OE_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SIO_GPIO_HI_OE_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SIO_GPIO_HI_OE_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SIO_GPIO_HI_OE_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SIO_GPIO_HI_OE_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SIO_GPIO_HI_OE_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SIO_GPIO_HI_OE_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SIO_GPIO_HI_OE_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SIO_GPIO_HI_OE_USB_DM = 0x2000000
	// Position of USB_DP field.
	SIO_GPIO_HI_OE_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SIO_GPIO_HI_OE_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SIO_GPIO_HI_OE_USB_DP = 0x1000000
	// Position of GPIO field.
	SIO_GPIO_HI_OE_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SIO_GPIO_HI_OE_GPIO_Msk = 0xffff

	// GPIO_OE_SET: GPIO0...31 output enable set
	// Position of GPIO_OE_SET field.
	SIO_GPIO_OE_SET_GPIO_OE_SET_Pos = 0x0
	// Bit mask of GPIO_OE_SET field.
	SIO_GPIO_OE_SET_GPIO_OE_SET_Msk = 0xffffffff

	// GPIO_HI_OE_SET: Output enable set for GPIO32...47, QSPI IOs and USB pins. Perform an atomic bit-set on GPIO_HI_OE, i.e. `GPIO_HI_OE |= wdata`
	// Position of QSPI_SD field.
	SIO_GPIO_HI_OE_SET_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SIO_GPIO_HI_OE_SET_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SIO_GPIO_HI_OE_SET_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SIO_GPIO_HI_OE_SET_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SIO_GPIO_HI_OE_SET_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SIO_GPIO_HI_OE_SET_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SIO_GPIO_HI_OE_SET_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SIO_GPIO_HI_OE_SET_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SIO_GPIO_HI_OE_SET_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SIO_GPIO_HI_OE_SET_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SIO_GPIO_HI_OE_SET_USB_DM = 0x2000000
	// Position of USB_DP field.
	SIO_GPIO_HI_OE_SET_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SIO_GPIO_HI_OE_SET_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SIO_GPIO_HI_OE_SET_USB_DP = 0x1000000
	// Position of GPIO field.
	SIO_GPIO_HI_OE_SET_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SIO_GPIO_HI_OE_SET_GPIO_Msk = 0xffff

	// GPIO_OE_CLR: GPIO0...31 output enable clear
	// Position of GPIO_OE_CLR field.
	SIO_GPIO_OE_CLR_GPIO_OE_CLR_Pos = 0x0
	// Bit mask of GPIO_OE_CLR field.
	SIO_GPIO_OE_CLR_GPIO_OE_CLR_Msk = 0xffffffff

	// GPIO_HI_OE_CLR: Output enable clear for GPIO32...47, QSPI IOs and USB pins. Perform an atomic bit-clear on GPIO_HI_OE, i.e. `GPIO_HI_OE &= ~wdata`
	// Position of QSPI_SD field.
	SIO_GPIO_HI_OE_CLR_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SIO_GPIO_HI_OE_CLR_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SIO_GPIO_HI_OE_CLR_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SIO_GPIO_HI_OE_CLR_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SIO_GPIO_HI_OE_CLR_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SIO_GPIO_HI_OE_CLR_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SIO_GPIO_HI_OE_CLR_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SIO_GPIO_HI_OE_CLR_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SIO_GPIO_HI_OE_CLR_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SIO_GPIO_HI_OE_CLR_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SIO_GPIO_HI_OE_CLR_USB_DM = 0x2000000
	// Position of USB_DP field.
	SIO_GPIO_HI_OE_CLR_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SIO_GPIO_HI_OE_CLR_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SIO_GPIO_HI_OE_CLR_USB_DP = 0x1000000
	// Position of GPIO field.
	SIO_GPIO_HI_OE_CLR_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SIO_GPIO_HI_OE_CLR_GPIO_Msk = 0xffff

	// GPIO_OE_XOR: GPIO0...31 output enable XOR
	// Position of GPIO_OE_XOR field.
	SIO_GPIO_OE_XOR_GPIO_OE_XOR_Pos = 0x0
	// Bit mask of GPIO_OE_XOR field.
	SIO_GPIO_OE_XOR_GPIO_OE_XOR_Msk = 0xffffffff

	// GPIO_HI_OE_XOR: Output enable XOR for GPIO32...47, QSPI IOs and USB pins. Perform an atomic bitwise XOR on GPIO_HI_OE, i.e. `GPIO_HI_OE ^= wdata`
	// Position of QSPI_SD field.
	SIO_GPIO_HI_OE_XOR_QSPI_SD_Pos = 0x1c
	// Bit mask of QSPI_SD field.
	SIO_GPIO_HI_OE_XOR_QSPI_SD_Msk = 0xf0000000
	// Position of QSPI_CSN field.
	SIO_GPIO_HI_OE_XOR_QSPI_CSN_Pos = 0x1b
	// Bit mask of QSPI_CSN field.
	SIO_GPIO_HI_OE_XOR_QSPI_CSN_Msk = 0x8000000
	// Bit QSPI_CSN.
	SIO_GPIO_HI_OE_XOR_QSPI_CSN = 0x8000000
	// Position of QSPI_SCK field.
	SIO_GPIO_HI_OE_XOR_QSPI_SCK_Pos = 0x1a
	// Bit mask of QSPI_SCK field.
	SIO_GPIO_HI_OE_XOR_QSPI_SCK_Msk = 0x4000000
	// Bit QSPI_SCK.
	SIO_GPIO_HI_OE_XOR_QSPI_SCK = 0x4000000
	// Position of USB_DM field.
	SIO_GPIO_HI_OE_XOR_USB_DM_Pos = 0x19
	// Bit mask of USB_DM field.
	SIO_GPIO_HI_OE_XOR_USB_DM_Msk = 0x2000000
	// Bit USB_DM.
	SIO_GPIO_HI_OE_XOR_USB_DM = 0x2000000
	// Position of USB_DP field.
	SIO_GPIO_HI_OE_XOR_USB_DP_Pos = 0x18
	// Bit mask of USB_DP field.
	SIO_GPIO_HI_OE_XOR_USB_DP_Msk = 0x1000000
	// Bit USB_DP.
	SIO_GPIO_HI_OE_XOR_USB_DP = 0x1000000
	// Position of GPIO field.
	SIO_GPIO_HI_OE_XOR_GPIO_Pos = 0x0
	// Bit mask of GPIO field.
	SIO_GPIO_HI_OE_XOR_GPIO_Msk = 0xffff

	// FIFO_ST: Status register for inter-core FIFOs (mailboxes). There is one FIFO in the core 0 -> core 1 direction, and one core 1 -> core 0. Both are 32 bits wide and 8 words deep. Core 0 can see the read side of the 1->0 FIFO (RX), and the write side of 0->1 FIFO (TX). Core 1 can see the read side of the 0->1 FIFO (RX), and the write side of 1->0 FIFO (TX). The SIO IRQ for each core is the logical OR of the VLD, WOF and ROE fields of its FIFO_ST register.
	// Position of ROE field.
	SIO_FIFO_ST_ROE_Pos = 0x3
	// Bit mask of ROE field.
	SIO_FIFO_ST_ROE_Msk = 0x8
	// Bit ROE.
	SIO_FIFO_ST_ROE = 0x8
	// Position of WOF field.
	SIO_FIFO_ST_WOF_Pos = 0x2
	// Bit mask of WOF field.
	SIO_FIFO_ST_WOF_Msk = 0x4
	// Bit WOF.
	SIO_FIFO_ST_WOF = 0x4
	// Position of RDY field.
	SIO_FIFO_ST_RDY_Pos = 0x1
	// Bit mask of RDY field.
	SIO_FIFO_ST_RDY_Msk = 0x2
	// Bit RDY.
	SIO_FIFO_ST_RDY = 0x2
	// Position of VLD field.
	SIO_FIFO_ST_VLD_Pos = 0x0
	// Bit mask of VLD field.
	SIO_FIFO_ST_VLD_Msk = 0x1
	// Bit VLD.
	SIO_FIFO_ST_VLD = 0x1

	// FIFO_WR: Write access to this core's TX FIFO
	// Position of FIFO_WR field.
	SIO_FIFO_WR_FIFO_WR_Pos = 0x0
	// Bit mask of FIFO_WR field.
	SIO_FIFO_WR_FIFO_WR_Msk = 0xffffffff

	// FIFO_RD: Read access to this core's RX FIFO
	// Position of FIFO_RD field.
	SIO_FIFO_RD_FIFO_RD_Pos = 0x0
	// Bit mask of FIFO_RD field.
	SIO_FIFO_RD_FIFO_RD_Msk = 0xffffffff

	// SPINLOCK_ST: Spinlock state A bitmap containing the state of all 32 spinlocks (1=locked). Mainly intended for debugging.
	// Position of SPINLOCK_ST field.
	SIO_SPINLOCK_ST_SPINLOCK_ST_Pos = 0x0
	// Bit mask of SPINLOCK_ST field.
	SIO_SPINLOCK_ST_SPINLOCK_ST_Msk = 0xffffffff

	// INTERP0_ACCUM0: Read/write access to accumulator 0
	// Position of INTERP0_ACCUM0 field.
	SIO_INTERP0_ACCUM0_INTERP0_ACCUM0_Pos = 0x0
	// Bit mask of INTERP0_ACCUM0 field.
	SIO_INTERP0_ACCUM0_INTERP0_ACCUM0_Msk = 0xffffffff

	// INTERP0_ACCUM1: Read/write access to accumulator 1
	// Position of INTERP0_ACCUM1 field.
	SIO_INTERP0_ACCUM1_INTERP0_ACCUM1_Pos = 0x0
	// Bit mask of INTERP0_ACCUM1 field.
	SIO_INTERP0_ACCUM1_INTERP0_ACCUM1_Msk = 0xffffffff

	// INTERP0_BASE0: Read/write access to BASE0 register.
	// Position of INTERP0_BASE0 field.
	SIO_INTERP0_BASE0_INTERP0_BASE0_Pos = 0x0
	// Bit mask of INTERP0_BASE0 field.
	SIO_INTERP0_BASE0_INTERP0_BASE0_Msk = 0xffffffff

	// INTERP0_BASE1: Read/write access to BASE1 register.
	// Position of INTERP0_BASE1 field.
	SIO_INTERP0_BASE1_INTERP0_BASE1_Pos = 0x0
	// Bit mask of INTERP0_BASE1 field.
	SIO_INTERP0_BASE1_INTERP0_BASE1_Msk = 0xffffffff

	// INTERP0_BASE2: Read/write access to BASE2 register.
	// Position of INTERP0_BASE2 field.
	SIO_INTERP0_BASE2_INTERP0_BASE2_Pos = 0x0
	// Bit mask of INTERP0_BASE2 field.
	SIO_INTERP0_BASE2_INTERP0_BASE2_Msk = 0xffffffff

	// INTERP0_POP_LANE0: Read LANE0 result, and simultaneously write lane results to both accumulators (POP).
	// Position of INTERP0_POP_LANE0 field.
	SIO_INTERP0_POP_LANE0_INTERP0_POP_LANE0_Pos = 0x0
	// Bit mask of INTERP0_POP_LANE0 field.
	SIO_INTERP0_POP_LANE0_INTERP0_POP_LANE0_Msk = 0xffffffff

	// INTERP0_POP_LANE1: Read LANE1 result, and simultaneously write lane results to both accumulators (POP).
	// Position of INTERP0_POP_LANE1 field.
	SIO_INTERP0_POP_LANE1_INTERP0_POP_LANE1_Pos = 0x0
	// Bit mask of INTERP0_POP_LANE1 field.
	SIO_INTERP0_POP_LANE1_INTERP0_POP_LANE1_Msk = 0xffffffff

	// INTERP0_POP_FULL: Read FULL result, and simultaneously write lane results to both accumulators (POP).
	// Position of INTERP0_POP_FULL field.
	SIO_INTERP0_POP_FULL_INTERP0_POP_FULL_Pos = 0x0
	// Bit mask of INTERP0_POP_FULL field.
	SIO_INTERP0_POP_FULL_INTERP0_POP_FULL_Msk = 0xffffffff

	// INTERP0_PEEK_LANE0: Read LANE0 result, without altering any internal state (PEEK).
	// Position of INTERP0_PEEK_LANE0 field.
	SIO_INTERP0_PEEK_LANE0_INTERP0_PEEK_LANE0_Pos = 0x0
	// Bit mask of INTERP0_PEEK_LANE0 field.
	SIO_INTERP0_PEEK_LANE0_INTERP0_PEEK_LANE0_Msk = 0xffffffff

	// INTERP0_PEEK_LANE1: Read LANE1 result, without altering any internal state (PEEK).
	// Position of INTERP0_PEEK_LANE1 field.
	SIO_INTERP0_PEEK_LANE1_INTERP0_PEEK_LANE1_Pos = 0x0
	// Bit mask of INTERP0_PEEK_LANE1 field.
	SIO_INTERP0_PEEK_LANE1_INTERP0_PEEK_LANE1_Msk = 0xffffffff

	// INTERP0_PEEK_FULL: Read FULL result, without altering any internal state (PEEK).
	// Position of INTERP0_PEEK_FULL field.
	SIO_INTERP0_PEEK_FULL_INTERP0_PEEK_FULL_Pos = 0x0
	// Bit mask of INTERP0_PEEK_FULL field.
	SIO_INTERP0_PEEK_FULL_INTERP0_PEEK_FULL_Msk = 0xffffffff

	// INTERP0_CTRL_LANE0: Control register for lane 0
	// Position of OVERF field.
	SIO_INTERP0_CTRL_LANE0_OVERF_Pos = 0x19
	// Bit mask of OVERF field.
	SIO_INTERP0_CTRL_LANE0_OVERF_Msk = 0x2000000
	// Bit OVERF.
	SIO_INTERP0_CTRL_LANE0_OVERF = 0x2000000
	// Position of OVERF1 field.
	SIO_INTERP0_CTRL_LANE0_OVERF1_Pos = 0x18
	// Bit mask of OVERF1 field.
	SIO_INTERP0_CTRL_LANE0_OVERF1_Msk = 0x1000000
	// Bit OVERF1.
	SIO_INTERP0_CTRL_LANE0_OVERF1 = 0x1000000
	// Position of OVERF0 field.
	SIO_INTERP0_CTRL_LANE0_OVERF0_Pos = 0x17
	// Bit mask of OVERF0 field.
	SIO_INTERP0_CTRL_LANE0_OVERF0_Msk = 0x800000
	// Bit OVERF0.
	SIO_INTERP0_CTRL_LANE0_OVERF0 = 0x800000
	// Position of BLEND field.
	SIO_INTERP0_CTRL_LANE0_BLEND_Pos = 0x15
	// Bit mask of BLEND field.
	SIO_INTERP0_CTRL_LANE0_BLEND_Msk = 0x200000
	// Bit BLEND.
	SIO_INTERP0_CTRL_LANE0_BLEND = 0x200000
	// Position of FORCE_MSB field.
	SIO_INTERP0_CTRL_LANE0_FORCE_MSB_Pos = 0x13
	// Bit mask of FORCE_MSB field.
	SIO_INTERP0_CTRL_LANE0_FORCE_MSB_Msk = 0x180000
	// Position of ADD_RAW field.
	SIO_INTERP0_CTRL_LANE0_ADD_RAW_Pos = 0x12
	// Bit mask of ADD_RAW field.
	SIO_INTERP0_CTRL_LANE0_ADD_RAW_Msk = 0x40000
	// Bit ADD_RAW.
	SIO_INTERP0_CTRL_LANE0_ADD_RAW = 0x40000
	// Position of CROSS_RESULT field.
	SIO_INTERP0_CTRL_LANE0_CROSS_RESULT_Pos = 0x11
	// Bit mask of CROSS_RESULT field.
	SIO_INTERP0_CTRL_LANE0_CROSS_RESULT_Msk = 0x20000
	// Bit CROSS_RESULT.
	SIO_INTERP0_CTRL_LANE0_CROSS_RESULT = 0x20000
	// Position of CROSS_INPUT field.
	SIO_INTERP0_CTRL_LANE0_CROSS_INPUT_Pos = 0x10
	// Bit mask of CROSS_INPUT field.
	SIO_INTERP0_CTRL_LANE0_CROSS_INPUT_Msk = 0x10000
	// Bit CROSS_INPUT.
	SIO_INTERP0_CTRL_LANE0_CROSS_INPUT = 0x10000
	// Position of SIGNED field.
	SIO_INTERP0_CTRL_LANE0_SIGNED_Pos = 0xf
	// Bit mask of SIGNED field.
	SIO_INTERP0_CTRL_LANE0_SIGNED_Msk = 0x8000
	// Bit SIGNED.
	SIO_INTERP0_CTRL_LANE0_SIGNED = 0x8000
	// Position of MASK_MSB field.
	SIO_INTERP0_CTRL_LANE0_MASK_MSB_Pos = 0xa
	// Bit mask of MASK_MSB field.
	SIO_INTERP0_CTRL_LANE0_MASK_MSB_Msk = 0x7c00
	// Position of MASK_LSB field.
	SIO_INTERP0_CTRL_LANE0_MASK_LSB_Pos = 0x5
	// Bit mask of MASK_LSB field.
	SIO_INTERP0_CTRL_LANE0_MASK_LSB_Msk = 0x3e0
	// Position of SHIFT field.
	SIO_INTERP0_CTRL_LANE0_SHIFT_Pos = 0x0
	// Bit mask of SHIFT field.
	SIO_INTERP0_CTRL_LANE0_SHIFT_Msk = 0x1f

	// INTERP0_CTRL_LANE1: Control register for lane 1
	// Position of FORCE_MSB field.
	SIO_INTERP0_CTRL_LANE1_FORCE_MSB_Pos = 0x13
	// Bit mask of FORCE_MSB field.
	SIO_INTERP0_CTRL_LANE1_FORCE_MSB_Msk = 0x180000
	// Position of ADD_RAW field.
	SIO_INTERP0_CTRL_LANE1_ADD_RAW_Pos = 0x12
	// Bit mask of ADD_RAW field.
	SIO_INTERP0_CTRL_LANE1_ADD_RAW_Msk = 0x40000
	// Bit ADD_RAW.
	SIO_INTERP0_CTRL_LANE1_ADD_RAW = 0x40000
	// Position of CROSS_RESULT field.
	SIO_INTERP0_CTRL_LANE1_CROSS_RESULT_Pos = 0x11
	// Bit mask of CROSS_RESULT field.
	SIO_INTERP0_CTRL_LANE1_CROSS_RESULT_Msk = 0x20000
	// Bit CROSS_RESULT.
	SIO_INTERP0_CTRL_LANE1_CROSS_RESULT = 0x20000
	// Position of CROSS_INPUT field.
	SIO_INTERP0_CTRL_LANE1_CROSS_INPUT_Pos = 0x10
	// Bit mask of CROSS_INPUT field.
	SIO_INTERP0_CTRL_LANE1_CROSS_INPUT_Msk = 0x10000
	// Bit CROSS_INPUT.
	SIO_INTERP0_CTRL_LANE1_CROSS_INPUT = 0x10000
	// Position of SIGNED field.
	SIO_INTERP0_CTRL_LANE1_SIGNED_Pos = 0xf
	// Bit mask of SIGNED field.
	SIO_INTERP0_CTRL_LANE1_SIGNED_Msk = 0x8000
	// Bit SIGNED.
	SIO_INTERP0_CTRL_LANE1_SIGNED = 0x8000
	// Position of MASK_MSB field.
	SIO_INTERP0_CTRL_LANE1_MASK_MSB_Pos = 0xa
	// Bit mask of MASK_MSB field.
	SIO_INTERP0_CTRL_LANE1_MASK_MSB_Msk = 0x7c00
	// Position of MASK_LSB field.
	SIO_INTERP0_CTRL_LANE1_MASK_LSB_Pos = 0x5
	// Bit mask of MASK_LSB field.
	SIO_INTERP0_CTRL_LANE1_MASK_LSB_Msk = 0x3e0
	// Position of SHIFT field.
	SIO_INTERP0_CTRL_LANE1_SHIFT_Pos = 0x0
	// Bit mask of SHIFT field.
	SIO_INTERP0_CTRL_LANE1_SHIFT_Msk = 0x1f

	// INTERP0_ACCUM0_ADD: Values written here are atomically added to ACCUM0 Reading yields lane 0's raw shift and mask value (BASE0 not added).
	// Position of INTERP0_ACCUM0_ADD field.
	SIO_INTERP0_ACCUM0_ADD_INTERP0_ACCUM0_ADD_Pos = 0x0
	// Bit mask of INTERP0_ACCUM0_ADD field.
	SIO_INTERP0_ACCUM0_ADD_INTERP0_ACCUM0_ADD_Msk = 0xffffff

	// INTERP0_ACCUM1_ADD: Values written here are atomically added to ACCUM1 Reading yields lane 1's raw shift and mask value (BASE1 not added).
	// Position of INTERP0_ACCUM1_ADD field.
	SIO_INTERP0_ACCUM1_ADD_INTERP0_ACCUM1_ADD_Pos = 0x0
	// Bit mask of INTERP0_ACCUM1_ADD field.
	SIO_INTERP0_ACCUM1_ADD_INTERP0_ACCUM1_ADD_Msk = 0xffffff

	// INTERP0_BASE_1AND0: On write, the lower 16 bits go to BASE0, upper bits to BASE1 simultaneously. Each half is sign-extended to 32 bits if that lane's SIGNED flag is set.
	// Position of INTERP0_BASE_1AND0 field.
	SIO_INTERP0_BASE_1AND0_INTERP0_BASE_1AND0_Pos = 0x0
	// Bit mask of INTERP0_BASE_1AND0 field.
	SIO_INTERP0_BASE_1AND0_INTERP0_BASE_1AND0_Msk = 0xffffffff

	// INTERP1_ACCUM0: Read/write access to accumulator 0
	// Position of INTERP1_ACCUM0 field.
	SIO_INTERP1_ACCUM0_INTERP1_ACCUM0_Pos = 0x0
	// Bit mask of INTERP1_ACCUM0 field.
	SIO_INTERP1_ACCUM0_INTERP1_ACCUM0_Msk = 0xffffffff

	// INTERP1_ACCUM1: Read/write access to accumulator 1
	// Position of INTERP1_ACCUM1 field.
	SIO_INTERP1_ACCUM1_INTERP1_ACCUM1_Pos = 0x0
	// Bit mask of INTERP1_ACCUM1 field.
	SIO_INTERP1_ACCUM1_INTERP1_ACCUM1_Msk = 0xffffffff

	// INTERP1_BASE0: Read/write access to BASE0 register.
	// Position of INTERP1_BASE0 field.
	SIO_INTERP1_BASE0_INTERP1_BASE0_Pos = 0x0
	// Bit mask of INTERP1_BASE0 field.
	SIO_INTERP1_BASE0_INTERP1_BASE0_Msk = 0xffffffff

	// INTERP1_BASE1: Read/write access to BASE1 register.
	// Position of INTERP1_BASE1 field.
	SIO_INTERP1_BASE1_INTERP1_BASE1_Pos = 0x0
	// Bit mask of INTERP1_BASE1 field.
	SIO_INTERP1_BASE1_INTERP1_BASE1_Msk = 0xffffffff

	// INTERP1_BASE2: Read/write access to BASE2 register.
	// Position of INTERP1_BASE2 field.
	SIO_INTERP1_BASE2_INTERP1_BASE2_Pos = 0x0
	// Bit mask of INTERP1_BASE2 field.
	SIO_INTERP1_BASE2_INTERP1_BASE2_Msk = 0xffffffff

	// INTERP1_POP_LANE0: Read LANE0 result, and simultaneously write lane results to both accumulators (POP).
	// Position of INTERP1_POP_LANE0 field.
	SIO_INTERP1_POP_LANE0_INTERP1_POP_LANE0_Pos = 0x0
	// Bit mask of INTERP1_POP_LANE0 field.
	SIO_INTERP1_POP_LANE0_INTERP1_POP_LANE0_Msk = 0xffffffff

	// INTERP1_POP_LANE1: Read LANE1 result, and simultaneously write lane results to both accumulators (POP).
	// Position of INTERP1_POP_LANE1 field.
	SIO_INTERP1_POP_LANE1_INTERP1_POP_LANE1_Pos = 0x0
	// Bit mask of INTERP1_POP_LANE1 field.
	SIO_INTERP1_POP_LANE1_INTERP1_POP_LANE1_Msk = 0xffffffff

	// INTERP1_POP_FULL: Read FULL result, and simultaneously write lane results to both accumulators (POP).
	// Position of INTERP1_POP_FULL field.
	SIO_INTERP1_POP_FULL_INTERP1_POP_FULL_Pos = 0x0
	// Bit mask of INTERP1_POP_FULL field.
	SIO_INTERP1_POP_FULL_INTERP1_POP_FULL_Msk = 0xffffffff

	// INTERP1_PEEK_LANE0: Read LANE0 result, without altering any internal state (PEEK).
	// Position of INTERP1_PEEK_LANE0 field.
	SIO_INTERP1_PEEK_LANE0_INTERP1_PEEK_LANE0_Pos = 0x0
	// Bit mask of INTERP1_PEEK_LANE0 field.
	SIO_INTERP1_PEEK_LANE0_INTERP1_PEEK_LANE0_Msk = 0xffffffff

	// INTERP1_PEEK_LANE1: Read LANE1 result, without altering any internal state (PEEK).
	// Position of INTERP1_PEEK_LANE1 field.
	SIO_INTERP1_PEEK_LANE1_INTERP1_PEEK_LANE1_Pos = 0x0
	// Bit mask of INTERP1_PEEK_LANE1 field.
	SIO_INTERP1_PEEK_LANE1_INTERP1_PEEK_LANE1_Msk = 0xffffffff

	// INTERP1_PEEK_FULL: Read FULL result, without altering any internal state (PEEK).
	// Position of INTERP1_PEEK_FULL field.
	SIO_INTERP1_PEEK_FULL_INTERP1_PEEK_FULL_Pos = 0x0
	// Bit mask of INTERP1_PEEK_FULL field.
	SIO_INTERP1_PEEK_FULL_INTERP1_PEEK_FULL_Msk = 0xffffffff

	// INTERP1_CTRL_LANE0: Control register for lane 0
	// Position of OVERF field.
	SIO_INTERP1_CTRL_LANE0_OVERF_Pos = 0x19
	// Bit mask of OVERF field.
	SIO_INTERP1_CTRL_LANE0_OVERF_Msk = 0x2000000
	// Bit OVERF.
	SIO_INTERP1_CTRL_LANE0_OVERF = 0x2000000
	// Position of OVERF1 field.
	SIO_INTERP1_CTRL_LANE0_OVERF1_Pos = 0x18
	// Bit mask of OVERF1 field.
	SIO_INTERP1_CTRL_LANE0_OVERF1_Msk = 0x1000000
	// Bit OVERF1.
	SIO_INTERP1_CTRL_LANE0_OVERF1 = 0x1000000
	// Position of OVERF0 field.
	SIO_INTERP1_CTRL_LANE0_OVERF0_Pos = 0x17
	// Bit mask of OVERF0 field.
	SIO_INTERP1_CTRL_LANE0_OVERF0_Msk = 0x800000
	// Bit OVERF0.
	SIO_INTERP1_CTRL_LANE0_OVERF0 = 0x800000
	// Position of CLAMP field.
	SIO_INTERP1_CTRL_LANE0_CLAMP_Pos = 0x16
	// Bit mask of CLAMP field.
	SIO_INTERP1_CTRL_LANE0_CLAMP_Msk = 0x400000
	// Bit CLAMP.
	SIO_INTERP1_CTRL_LANE0_CLAMP = 0x400000
	// Position of FORCE_MSB field.
	SIO_INTERP1_CTRL_LANE0_FORCE_MSB_Pos = 0x13
	// Bit mask of FORCE_MSB field.
	SIO_INTERP1_CTRL_LANE0_FORCE_MSB_Msk = 0x180000
	// Position of ADD_RAW field.
	SIO_INTERP1_CTRL_LANE0_ADD_RAW_Pos = 0x12
	// Bit mask of ADD_RAW field.
	SIO_INTERP1_CTRL_LANE0_ADD_RAW_Msk = 0x40000
	// Bit ADD_RAW.
	SIO_INTERP1_CTRL_LANE0_ADD_RAW = 0x40000
	// Position of CROSS_RESULT field.
	SIO_INTERP1_CTRL_LANE0_CROSS_RESULT_Pos = 0x11
	// Bit mask of CROSS_RESULT field.
	SIO_INTERP1_CTRL_LANE0_CROSS_RESULT_Msk = 0x20000
	// Bit CROSS_RESULT.
	SIO_INTERP1_CTRL_LANE0_CROSS_RESULT = 0x20000
	// Position of CROSS_INPUT field.
	SIO_INTERP1_CTRL_LANE0_CROSS_INPUT_Pos = 0x10
	// Bit mask of CROSS_INPUT field.
	SIO_INTERP1_CTRL_LANE0_CROSS_INPUT_Msk = 0x10000
	// Bit CROSS_INPUT.
	SIO_INTERP1_CTRL_LANE0_CROSS_INPUT = 0x10000
	// Position of SIGNED field.
	SIO_INTERP1_CTRL_LANE0_SIGNED_Pos = 0xf
	// Bit mask of SIGNED field.
	SIO_INTERP1_CTRL_LANE0_SIGNED_Msk = 0x8000
	// Bit SIGNED.
	SIO_INTERP1_CTRL_LANE0_SIGNED = 0x8000
	// Position of MASK_MSB field.
	SIO_INTERP1_CTRL_LANE0_MASK_MSB_Pos = 0xa
	// Bit mask of MASK_MSB field.
	SIO_INTERP1_CTRL_LANE0_MASK_MSB_Msk = 0x7c00
	// Position of MASK_LSB field.
	SIO_INTERP1_CTRL_LANE0_MASK_LSB_Pos = 0x5
	// Bit mask of MASK_LSB field.
	SIO_INTERP1_CTRL_LANE0_MASK_LSB_Msk = 0x3e0
	// Position of SHIFT field.
	SIO_INTERP1_CTRL_LANE0_SHIFT_Pos = 0x0
	// Bit mask of SHIFT field.
	SIO_INTERP1_CTRL_LANE0_SHIFT_Msk = 0x1f

	// INTERP1_CTRL_LANE1: Control register for lane 1
	// Position of FORCE_MSB field.
	SIO_INTERP1_CTRL_LANE1_FORCE_MSB_Pos = 0x13
	// Bit mask of FORCE_MSB field.
	SIO_INTERP1_CTRL_LANE1_FORCE_MSB_Msk = 0x180000
	// Position of ADD_RAW field.
	SIO_INTERP1_CTRL_LANE1_ADD_RAW_Pos = 0x12
	// Bit mask of ADD_RAW field.
	SIO_INTERP1_CTRL_LANE1_ADD_RAW_Msk = 0x40000
	// Bit ADD_RAW.
	SIO_INTERP1_CTRL_LANE1_ADD_RAW = 0x40000
	// Position of CROSS_RESULT field.
	SIO_INTERP1_CTRL_LANE1_CROSS_RESULT_Pos = 0x11
	// Bit mask of CROSS_RESULT field.
	SIO_INTERP1_CTRL_LANE1_CROSS_RESULT_Msk = 0x20000
	// Bit CROSS_RESULT.
	SIO_INTERP1_CTRL_LANE1_CROSS_RESULT = 0x20000
	// Position of CROSS_INPUT field.
	SIO_INTERP1_CTRL_LANE1_CROSS_INPUT_Pos = 0x10
	// Bit mask of CROSS_INPUT field.
	SIO_INTERP1_CTRL_LANE1_CROSS_INPUT_Msk = 0x10000
	// Bit CROSS_INPUT.
	SIO_INTERP1_CTRL_LANE1_CROSS_INPUT = 0x10000
	// Position of SIGNED field.
	SIO_INTERP1_CTRL_LANE1_SIGNED_Pos = 0xf
	// Bit mask of SIGNED field.
	SIO_INTERP1_CTRL_LANE1_SIGNED_Msk = 0x8000
	// Bit SIGNED.
	SIO_INTERP1_CTRL_LANE1_SIGNED = 0x8000
	// Position of MASK_MSB field.
	SIO_INTERP1_CTRL_LANE1_MASK_MSB_Pos = 0xa
	// Bit mask of MASK_MSB field.
	SIO_INTERP1_CTRL_LANE1_MASK_MSB_Msk = 0x7c00
	// Position of MASK_LSB field.
	SIO_INTERP1_CTRL_LANE1_MASK_LSB_Pos = 0x5
	// Bit mask of MASK_LSB field.
	SIO_INTERP1_CTRL_LANE1_MASK_LSB_Msk = 0x3e0
	// Position of SHIFT field.
	SIO_INTERP1_CTRL_LANE1_SHIFT_Pos = 0x0
	// Bit mask of SHIFT field.
	SIO_INTERP1_CTRL_LANE1_SHIFT_Msk = 0x1f

	// INTERP1_ACCUM0_ADD: Values written here are atomically added to ACCUM0 Reading yields lane 0's raw shift and mask value (BASE0 not added).
	// Position of INTERP1_ACCUM0_ADD field.
	SIO_INTERP1_ACCUM0_ADD_INTERP1_ACCUM0_ADD_Pos = 0x0
	// Bit mask of INTERP1_ACCUM0_ADD field.
	SIO_INTERP1_ACCUM0_ADD_INTERP1_ACCUM0_ADD_Msk = 0xffffff

	// INTERP1_ACCUM1_ADD: Values written here are atomically added to ACCUM1 Reading yields lane 1's raw shift and mask value (BASE1 not added).
	// Position of INTERP1_ACCUM1_ADD field.
	SIO_INTERP1_ACCUM1_ADD_INTERP1_ACCUM1_ADD_Pos = 0x0
	// Bit mask of INTERP1_ACCUM1_ADD field.
	SIO_INTERP1_ACCUM1_ADD_INTERP1_ACCUM1_ADD_Msk = 0xffffff

	// INTERP1_BASE_1AND0: On write, the lower 16 bits go to BASE0, upper bits to BASE1 simultaneously. Each half is sign-extended to 32 bits if that lane's SIGNED flag is set.
	// Position of INTERP1_BASE_1AND0 field.
	SIO_INTERP1_BASE_1AND0_INTERP1_BASE_1AND0_Pos = 0x0
	// Bit mask of INTERP1_BASE_1AND0 field.
	SIO_INTERP1_BASE_1AND0_INTERP1_BASE_1AND0_Msk = 0xffffffff

	// SPINLOCK0: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK0 field.
	SIO_SPINLOCK0_SPINLOCK0_Pos = 0x0
	// Bit mask of SPINLOCK0 field.
	SIO_SPINLOCK0_SPINLOCK0_Msk = 0xffffffff

	// SPINLOCK1: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK1 field.
	SIO_SPINLOCK1_SPINLOCK1_Pos = 0x0
	// Bit mask of SPINLOCK1 field.
	SIO_SPINLOCK1_SPINLOCK1_Msk = 0xffffffff

	// SPINLOCK2: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK2 field.
	SIO_SPINLOCK2_SPINLOCK2_Pos = 0x0
	// Bit mask of SPINLOCK2 field.
	SIO_SPINLOCK2_SPINLOCK2_Msk = 0xffffffff

	// SPINLOCK3: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK3 field.
	SIO_SPINLOCK3_SPINLOCK3_Pos = 0x0
	// Bit mask of SPINLOCK3 field.
	SIO_SPINLOCK3_SPINLOCK3_Msk = 0xffffffff

	// SPINLOCK4: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK4 field.
	SIO_SPINLOCK4_SPINLOCK4_Pos = 0x0
	// Bit mask of SPINLOCK4 field.
	SIO_SPINLOCK4_SPINLOCK4_Msk = 0xffffffff

	// SPINLOCK5: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK5 field.
	SIO_SPINLOCK5_SPINLOCK5_Pos = 0x0
	// Bit mask of SPINLOCK5 field.
	SIO_SPINLOCK5_SPINLOCK5_Msk = 0xffffffff

	// SPINLOCK6: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK6 field.
	SIO_SPINLOCK6_SPINLOCK6_Pos = 0x0
	// Bit mask of SPINLOCK6 field.
	SIO_SPINLOCK6_SPINLOCK6_Msk = 0xffffffff

	// SPINLOCK7: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK7 field.
	SIO_SPINLOCK7_SPINLOCK7_Pos = 0x0
	// Bit mask of SPINLOCK7 field.
	SIO_SPINLOCK7_SPINLOCK7_Msk = 0xffffffff

	// SPINLOCK8: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK8 field.
	SIO_SPINLOCK8_SPINLOCK8_Pos = 0x0
	// Bit mask of SPINLOCK8 field.
	SIO_SPINLOCK8_SPINLOCK8_Msk = 0xffffffff

	// SPINLOCK9: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK9 field.
	SIO_SPINLOCK9_SPINLOCK9_Pos = 0x0
	// Bit mask of SPINLOCK9 field.
	SIO_SPINLOCK9_SPINLOCK9_Msk = 0xffffffff

	// SPINLOCK10: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK10 field.
	SIO_SPINLOCK10_SPINLOCK10_Pos = 0x0
	// Bit mask of SPINLOCK10 field.
	SIO_SPINLOCK10_SPINLOCK10_Msk = 0xffffffff

	// SPINLOCK11: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK11 field.
	SIO_SPINLOCK11_SPINLOCK11_Pos = 0x0
	// Bit mask of SPINLOCK11 field.
	SIO_SPINLOCK11_SPINLOCK11_Msk = 0xffffffff

	// SPINLOCK12: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK12 field.
	SIO_SPINLOCK12_SPINLOCK12_Pos = 0x0
	// Bit mask of SPINLOCK12 field.
	SIO_SPINLOCK12_SPINLOCK12_Msk = 0xffffffff

	// SPINLOCK13: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK13 field.
	SIO_SPINLOCK13_SPINLOCK13_Pos = 0x0
	// Bit mask of SPINLOCK13 field.
	SIO_SPINLOCK13_SPINLOCK13_Msk = 0xffffffff

	// SPINLOCK14: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK14 field.
	SIO_SPINLOCK14_SPINLOCK14_Pos = 0x0
	// Bit mask of SPINLOCK14 field.
	SIO_SPINLOCK14_SPINLOCK14_Msk = 0xffffffff

	// SPINLOCK15: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK15 field.
	SIO_SPINLOCK15_SPINLOCK15_Pos = 0x0
	// Bit mask of SPINLOCK15 field.
	SIO_SPINLOCK15_SPINLOCK15_Msk = 0xffffffff

	// SPINLOCK16: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK16 field.
	SIO_SPINLOCK16_SPINLOCK16_Pos = 0x0
	// Bit mask of SPINLOCK16 field.
	SIO_SPINLOCK16_SPINLOCK16_Msk = 0xffffffff

	// SPINLOCK17: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK17 field.
	SIO_SPINLOCK17_SPINLOCK17_Pos = 0x0
	// Bit mask of SPINLOCK17 field.
	SIO_SPINLOCK17_SPINLOCK17_Msk = 0xffffffff

	// SPINLOCK18: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK18 field.
	SIO_SPINLOCK18_SPINLOCK18_Pos = 0x0
	// Bit mask of SPINLOCK18 field.
	SIO_SPINLOCK18_SPINLOCK18_Msk = 0xffffffff

	// SPINLOCK19: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK19 field.
	SIO_SPINLOCK19_SPINLOCK19_Pos = 0x0
	// Bit mask of SPINLOCK19 field.
	SIO_SPINLOCK19_SPINLOCK19_Msk = 0xffffffff

	// SPINLOCK20: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK20 field.
	SIO_SPINLOCK20_SPINLOCK20_Pos = 0x0
	// Bit mask of SPINLOCK20 field.
	SIO_SPINLOCK20_SPINLOCK20_Msk = 0xffffffff

	// SPINLOCK21: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK21 field.
	SIO_SPINLOCK21_SPINLOCK21_Pos = 0x0
	// Bit mask of SPINLOCK21 field.
	SIO_SPINLOCK21_SPINLOCK21_Msk = 0xffffffff

	// SPINLOCK22: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK22 field.
	SIO_SPINLOCK22_SPINLOCK22_Pos = 0x0
	// Bit mask of SPINLOCK22 field.
	SIO_SPINLOCK22_SPINLOCK22_Msk = 0xffffffff

	// SPINLOCK23: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK23 field.
	SIO_SPINLOCK23_SPINLOCK23_Pos = 0x0
	// Bit mask of SPINLOCK23 field.
	SIO_SPINLOCK23_SPINLOCK23_Msk = 0xffffffff

	// SPINLOCK24: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK24 field.
	SIO_SPINLOCK24_SPINLOCK24_Pos = 0x0
	// Bit mask of SPINLOCK24 field.
	SIO_SPINLOCK24_SPINLOCK24_Msk = 0xffffffff

	// SPINLOCK25: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK25 field.
	SIO_SPINLOCK25_SPINLOCK25_Pos = 0x0
	// Bit mask of SPINLOCK25 field.
	SIO_SPINLOCK25_SPINLOCK25_Msk = 0xffffffff

	// SPINLOCK26: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK26 field.
	SIO_SPINLOCK26_SPINLOCK26_Pos = 0x0
	// Bit mask of SPINLOCK26 field.
	SIO_SPINLOCK26_SPINLOCK26_Msk = 0xffffffff

	// SPINLOCK27: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK27 field.
	SIO_SPINLOCK27_SPINLOCK27_Pos = 0x0
	// Bit mask of SPINLOCK27 field.
	SIO_SPINLOCK27_SPINLOCK27_Msk = 0xffffffff

	// SPINLOCK28: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK28 field.
	SIO_SPINLOCK28_SPINLOCK28_Pos = 0x0
	// Bit mask of SPINLOCK28 field.
	SIO_SPINLOCK28_SPINLOCK28_Msk = 0xffffffff

	// SPINLOCK29: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK29 field.
	SIO_SPINLOCK29_SPINLOCK29_Pos = 0x0
	// Bit mask of SPINLOCK29 field.
	SIO_SPINLOCK29_SPINLOCK29_Msk = 0xffffffff

	// SPINLOCK30: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK30 field.
	SIO_SPINLOCK30_SPINLOCK30_Pos = 0x0
	// Bit mask of SPINLOCK30 field.
	SIO_SPINLOCK30_SPINLOCK30_Msk = 0xffffffff

	// SPINLOCK31: Reading from a spinlock address will: - Return 0 if lock is already locked - Otherwise return nonzero, and simultaneously claim the lock Writing (any value) releases the lock. If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins. The value returned on success is 0x1 << lock number.
	// Position of SPINLOCK31 field.
	SIO_SPINLOCK31_SPINLOCK31_Pos = 0x0
	// Bit mask of SPINLOCK31 field.
	SIO_SPINLOCK31_SPINLOCK31_Msk = 0xffffffff

	// DOORBELL_OUT_SET: Trigger a doorbell interrupt on the opposite core. Write 1 to a bit to set the corresponding bit in DOORBELL_IN on the opposite core. This raises the opposite core's doorbell interrupt. Read to get the status of the doorbells currently asserted on the opposite core. This is equivalent to that core reading its own DOORBELL_IN status.
	// Position of DOORBELL_OUT_SET field.
	SIO_DOORBELL_OUT_SET_DOORBELL_OUT_SET_Pos = 0x0
	// Bit mask of DOORBELL_OUT_SET field.
	SIO_DOORBELL_OUT_SET_DOORBELL_OUT_SET_Msk = 0xff

	// DOORBELL_OUT_CLR: Clear doorbells which have been posted to the opposite core. This register is intended for debugging and initialisation purposes. Writing 1 to a bit in DOORBELL_OUT_CLR clears the corresponding bit in DOORBELL_IN on the opposite core. Clearing all bits will cause that core's doorbell interrupt to deassert. Since the usual order of events is for software to send events using DOORBELL_OUT_SET, and acknowledge incoming events by writing to DOORBELL_IN_CLR, this register should be used with caution to avoid race conditions. Reading returns the status of the doorbells currently asserted on the other core, i.e. is equivalent to that core reading its own DOORBELL_IN status.
	// Position of DOORBELL_OUT_CLR field.
	SIO_DOORBELL_OUT_CLR_DOORBELL_OUT_CLR_Pos = 0x0
	// Bit mask of DOORBELL_OUT_CLR field.
	SIO_DOORBELL_OUT_CLR_DOORBELL_OUT_CLR_Msk = 0xff

	// DOORBELL_IN_SET: Write 1s to trigger doorbell interrupts on this core. Read to get status of doorbells currently asserted on this core.
	// Position of DOORBELL_IN_SET field.
	SIO_DOORBELL_IN_SET_DOORBELL_IN_SET_Pos = 0x0
	// Bit mask of DOORBELL_IN_SET field.
	SIO_DOORBELL_IN_SET_DOORBELL_IN_SET_Msk = 0xff

	// DOORBELL_IN_CLR: Check and acknowledge doorbells posted to this core. This core's doorbell interrupt is asserted when any bit in this register is 1. Write 1 to each bit to clear that bit. The doorbell interrupt deasserts once all bits are cleared. Read to get status of doorbells currently asserted on this core.
	// Position of DOORBELL_IN_CLR field.
	SIO_DOORBELL_IN_CLR_DOORBELL_IN_CLR_Pos = 0x0
	// Bit mask of DOORBELL_IN_CLR field.
	SIO_DOORBELL_IN_CLR_DOORBELL_IN_CLR_Msk = 0xff

	// PERI_NONSEC: Detach certain core-local peripherals from Secure SIO, and attach them to Non-secure SIO, so that Non-secure software can use them. Attempting to access one of these peripherals from the Secure SIO when it is attached to the Non-secure SIO, or vice versa, will generate a bus error. This register is per-core, and is only present on the Secure SIO. Most SIO hardware is duplicated across the Secure and Non-secure SIO, so is not listed in this register.
	// Position of TMDS field.
	SIO_PERI_NONSEC_TMDS_Pos = 0x5
	// Bit mask of TMDS field.
	SIO_PERI_NONSEC_TMDS_Msk = 0x20
	// Bit TMDS.
	SIO_PERI_NONSEC_TMDS = 0x20
	// Position of INTERP1 field.
	SIO_PERI_NONSEC_INTERP1_Pos = 0x1
	// Bit mask of INTERP1 field.
	SIO_PERI_NONSEC_INTERP1_Msk = 0x2
	// Bit INTERP1.
	SIO_PERI_NONSEC_INTERP1 = 0x2
	// Position of INTERP0 field.
	SIO_PERI_NONSEC_INTERP0_Pos = 0x0
	// Bit mask of INTERP0 field.
	SIO_PERI_NONSEC_INTERP0_Msk = 0x1
	// Bit INTERP0.
	SIO_PERI_NONSEC_INTERP0 = 0x1

	// RISCV_SOFTIRQ: Control the assertion of the standard software interrupt (MIP.MSIP) on the RISC-V cores. Unlike the RISC-V timer, this interrupt is not routed to a normal system-level interrupt line, so can not be used by the Arm cores. It is safe for both cores to write to this register on the same cycle. The set/clear effect is accumulated across both cores, and then applied. If a flag is both set and cleared on the same cycle, only the set takes effect.
	// Position of CORE1_CLR field.
	SIO_RISCV_SOFTIRQ_CORE1_CLR_Pos = 0x9
	// Bit mask of CORE1_CLR field.
	SIO_RISCV_SOFTIRQ_CORE1_CLR_Msk = 0x200
	// Bit CORE1_CLR.
	SIO_RISCV_SOFTIRQ_CORE1_CLR = 0x200
	// Position of CORE0_CLR field.
	SIO_RISCV_SOFTIRQ_CORE0_CLR_Pos = 0x8
	// Bit mask of CORE0_CLR field.
	SIO_RISCV_SOFTIRQ_CORE0_CLR_Msk = 0x100
	// Bit CORE0_CLR.
	SIO_RISCV_SOFTIRQ_CORE0_CLR = 0x100
	// Position of CORE1_SET field.
	SIO_RISCV_SOFTIRQ_CORE1_SET_Pos = 0x1
	// Bit mask of CORE1_SET field.
	SIO_RISCV_SOFTIRQ_CORE1_SET_Msk = 0x2
	// Bit CORE1_SET.
	SIO_RISCV_SOFTIRQ_CORE1_SET = 0x2
	// Position of CORE0_SET field.
	SIO_RISCV_SOFTIRQ_CORE0_SET_Pos = 0x0
	// Bit mask of CORE0_SET field.
	SIO_RISCV_SOFTIRQ_CORE0_SET_Msk = 0x1
	// Bit CORE0_SET.
	SIO_RISCV_SOFTIRQ_CORE0_SET = 0x1

	// MTIME_CTRL: Control register for the RISC-V 64-bit Machine-mode timer. This timer is only present in the Secure SIO, so is only accessible to an Arm core in Secure mode or a RISC-V core in Machine mode. Note whilst this timer follows the RISC-V privileged specification, it is equally usable by the Arm cores. The interrupts are routed to normal system-level interrupt lines as well as to the MIP.MTIP inputs on the RISC-V cores.
	// Position of DBGPAUSE_CORE1 field.
	SIO_MTIME_CTRL_DBGPAUSE_CORE1_Pos = 0x3
	// Bit mask of DBGPAUSE_CORE1 field.
	SIO_MTIME_CTRL_DBGPAUSE_CORE1_Msk = 0x8
	// Bit DBGPAUSE_CORE1.
	SIO_MTIME_CTRL_DBGPAUSE_CORE1 = 0x8
	// Position of DBGPAUSE_CORE0 field.
	SIO_MTIME_CTRL_DBGPAUSE_CORE0_Pos = 0x2
	// Bit mask of DBGPAUSE_CORE0 field.
	SIO_MTIME_CTRL_DBGPAUSE_CORE0_Msk = 0x4
	// Bit DBGPAUSE_CORE0.
	SIO_MTIME_CTRL_DBGPAUSE_CORE0 = 0x4
	// Position of FULLSPEED field.
	SIO_MTIME_CTRL_FULLSPEED_Pos = 0x1
	// Bit mask of FULLSPEED field.
	SIO_MTIME_CTRL_FULLSPEED_Msk = 0x2
	// Bit FULLSPEED.
	SIO_MTIME_CTRL_FULLSPEED = 0x2
	// Position of EN field.
	SIO_MTIME_CTRL_EN_Pos = 0x0
	// Bit mask of EN field.
	SIO_MTIME_CTRL_EN_Msk = 0x1
	// Bit EN.
	SIO_MTIME_CTRL_EN = 0x1

	// MTIME: Read/write access to the high half of RISC-V Machine-mode timer. This register is shared between both cores. If both cores write on the same cycle, core 1 takes precedence.
	// Position of MTIME field.
	SIO_MTIME_MTIME_Pos = 0x0
	// Bit mask of MTIME field.
	SIO_MTIME_MTIME_Msk = 0xffffffff

	// MTIMEH: Read/write access to the high half of RISC-V Machine-mode timer. This register is shared between both cores. If both cores write on the same cycle, core 1 takes precedence.
	// Position of MTIMEH field.
	SIO_MTIMEH_MTIMEH_Pos = 0x0
	// Bit mask of MTIMEH field.
	SIO_MTIMEH_MTIMEH_Msk = 0xffffffff

	// MTIMECMP: Low half of RISC-V Machine-mode timer comparator. This register is core-local, i.e., each core gets a copy of this register, with the comparison result routed to its own interrupt line. The timer interrupt is asserted whenever MTIME is greater than or equal to MTIMECMP. This comparison is unsigned, and performed on the full 64-bit values.
	// Position of MTIMECMP field.
	SIO_MTIMECMP_MTIMECMP_Pos = 0x0
	// Bit mask of MTIMECMP field.
	SIO_MTIMECMP_MTIMECMP_Msk = 0xffffffff

	// MTIMECMPH: High half of RISC-V Machine-mode timer comparator. This register is core-local. The timer interrupt is asserted whenever MTIME is greater than or equal to MTIMECMP. This comparison is unsigned, and performed on the full 64-bit values.
	// Position of MTIMECMPH field.
	SIO_MTIMECMPH_MTIMECMPH_Pos = 0x0
	// Bit mask of MTIMECMPH field.
	SIO_MTIMECMPH_MTIMECMPH_Msk = 0xffffffff

	// TMDS_CTRL: Control register for TMDS encoder.
	// Position of CLEAR_BALANCE field.
	SIO_TMDS_CTRL_CLEAR_BALANCE_Pos = 0x1c
	// Bit mask of CLEAR_BALANCE field.
	SIO_TMDS_CTRL_CLEAR_BALANCE_Msk = 0x10000000
	// Bit CLEAR_BALANCE.
	SIO_TMDS_CTRL_CLEAR_BALANCE = 0x10000000
	// Position of PIX2_NOSHIFT field.
	SIO_TMDS_CTRL_PIX2_NOSHIFT_Pos = 0x1b
	// Bit mask of PIX2_NOSHIFT field.
	SIO_TMDS_CTRL_PIX2_NOSHIFT_Msk = 0x8000000
	// Bit PIX2_NOSHIFT.
	SIO_TMDS_CTRL_PIX2_NOSHIFT = 0x8000000
	// Position of PIX_SHIFT field.
	SIO_TMDS_CTRL_PIX_SHIFT_Pos = 0x18
	// Bit mask of PIX_SHIFT field.
	SIO_TMDS_CTRL_PIX_SHIFT_Msk = 0x7000000
	// Do not shift the colour data register.
	SIO_TMDS_CTRL_PIX_SHIFT_0 = 0x0
	// Shift the colour data register by 1 bit
	SIO_TMDS_CTRL_PIX_SHIFT_1 = 0x1
	// Shift the colour data register by 2 bits
	SIO_TMDS_CTRL_PIX_SHIFT_2 = 0x2
	// Shift the colour data register by 4 bits
	SIO_TMDS_CTRL_PIX_SHIFT_4 = 0x3
	// Shift the colour data register by 8 bits
	SIO_TMDS_CTRL_PIX_SHIFT_8 = 0x4
	// Shift the colour data register by 16 bits
	SIO_TMDS_CTRL_PIX_SHIFT_16 = 0x5
	// Position of INTERLEAVE field.
	SIO_TMDS_CTRL_INTERLEAVE_Pos = 0x17
	// Bit mask of INTERLEAVE field.
	SIO_TMDS_CTRL_INTERLEAVE_Msk = 0x800000
	// Bit INTERLEAVE.
	SIO_TMDS_CTRL_INTERLEAVE = 0x800000
	// Position of L2_NBITS field.
	SIO_TMDS_CTRL_L2_NBITS_Pos = 0x12
	// Bit mask of L2_NBITS field.
	SIO_TMDS_CTRL_L2_NBITS_Msk = 0x1c0000
	// Position of L1_NBITS field.
	SIO_TMDS_CTRL_L1_NBITS_Pos = 0xf
	// Bit mask of L1_NBITS field.
	SIO_TMDS_CTRL_L1_NBITS_Msk = 0x38000
	// Position of L0_NBITS field.
	SIO_TMDS_CTRL_L0_NBITS_Pos = 0xc
	// Bit mask of L0_NBITS field.
	SIO_TMDS_CTRL_L0_NBITS_Msk = 0x7000
	// Position of L2_ROT field.
	SIO_TMDS_CTRL_L2_ROT_Pos = 0x8
	// Bit mask of L2_ROT field.
	SIO_TMDS_CTRL_L2_ROT_Msk = 0xf00
	// Position of L1_ROT field.
	SIO_TMDS_CTRL_L1_ROT_Pos = 0x4
	// Bit mask of L1_ROT field.
	SIO_TMDS_CTRL_L1_ROT_Msk = 0xf0
	// Position of L0_ROT field.
	SIO_TMDS_CTRL_L0_ROT_Pos = 0x0
	// Bit mask of L0_ROT field.
	SIO_TMDS_CTRL_L0_ROT_Msk = 0xf

	// TMDS_WDATA: Write-only access to the TMDS colour data register.
	// Position of TMDS_WDATA field.
	SIO_TMDS_WDATA_TMDS_WDATA_Pos = 0x0
	// Bit mask of TMDS_WDATA field.
	SIO_TMDS_WDATA_TMDS_WDATA_Msk = 0xffffffff

	// TMDS_PEEK_SINGLE: Get the encoding of one pixel's worth of colour data, packed into a 32-bit value (3x10-bit symbols). The PEEK alias does not shift the colour register when read, but still advances the running DC balance state of each encoder. This is useful for pixel doubling.
	// Position of TMDS_PEEK_SINGLE field.
	SIO_TMDS_PEEK_SINGLE_TMDS_PEEK_SINGLE_Pos = 0x0
	// Bit mask of TMDS_PEEK_SINGLE field.
	SIO_TMDS_PEEK_SINGLE_TMDS_PEEK_SINGLE_Msk = 0xffffffff

	// TMDS_POP_SINGLE: Get the encoding of one pixel's worth of colour data, packed into a 32-bit value. The packing is 5 chunks of 3 lanes times 2 bits (30 bits total). Each chunk contains two bits of a TMDS symbol per lane. This format is intended for shifting out with the HSTX peripheral on RP2350. The POP alias shifts the colour register when read, as well as advancing the running DC balance state of each encoder.
	// Position of TMDS_POP_SINGLE field.
	SIO_TMDS_POP_SINGLE_TMDS_POP_SINGLE_Pos = 0x0
	// Bit mask of TMDS_POP_SINGLE field.
	SIO_TMDS_POP_SINGLE_TMDS_POP_SINGLE_Msk = 0xffffffff

	// TMDS_PEEK_DOUBLE_L0: Get lane 0 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The PEEK alias does not shift the colour register when read, but still advances the lane 0 DC balance state. This is useful if all 3 lanes' worth of encode are to be read at once, rather than processing the entire scanline for one lane before moving to the next lane.
	// Position of TMDS_PEEK_DOUBLE_L0 field.
	SIO_TMDS_PEEK_DOUBLE_L0_TMDS_PEEK_DOUBLE_L0_Pos = 0x0
	// Bit mask of TMDS_PEEK_DOUBLE_L0 field.
	SIO_TMDS_PEEK_DOUBLE_L0_TMDS_PEEK_DOUBLE_L0_Msk = 0xffffffff

	// TMDS_POP_DOUBLE_L0: Get lane 0 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The POP alias shifts the colour register when read, according to the values of PIX_SHIFT and PIX2_NOSHIFT.
	// Position of TMDS_POP_DOUBLE_L0 field.
	SIO_TMDS_POP_DOUBLE_L0_TMDS_POP_DOUBLE_L0_Pos = 0x0
	// Bit mask of TMDS_POP_DOUBLE_L0 field.
	SIO_TMDS_POP_DOUBLE_L0_TMDS_POP_DOUBLE_L0_Msk = 0xffffffff

	// TMDS_PEEK_DOUBLE_L1: Get lane 1 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The PEEK alias does not shift the colour register when read, but still advances the lane 1 DC balance state. This is useful if all 3 lanes' worth of encode are to be read at once, rather than processing the entire scanline for one lane before moving to the next lane.
	// Position of TMDS_PEEK_DOUBLE_L1 field.
	SIO_TMDS_PEEK_DOUBLE_L1_TMDS_PEEK_DOUBLE_L1_Pos = 0x0
	// Bit mask of TMDS_PEEK_DOUBLE_L1 field.
	SIO_TMDS_PEEK_DOUBLE_L1_TMDS_PEEK_DOUBLE_L1_Msk = 0xffffffff

	// TMDS_POP_DOUBLE_L1: Get lane 1 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The POP alias shifts the colour register when read, according to the values of PIX_SHIFT and PIX2_NOSHIFT.
	// Position of TMDS_POP_DOUBLE_L1 field.
	SIO_TMDS_POP_DOUBLE_L1_TMDS_POP_DOUBLE_L1_Pos = 0x0
	// Bit mask of TMDS_POP_DOUBLE_L1 field.
	SIO_TMDS_POP_DOUBLE_L1_TMDS_POP_DOUBLE_L1_Msk = 0xffffffff

	// TMDS_PEEK_DOUBLE_L2: Get lane 2 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The PEEK alias does not shift the colour register when read, but still advances the lane 2 DC balance state. This is useful if all 3 lanes' worth of encode are to be read at once, rather than processing the entire scanline for one lane before moving to the next lane.
	// Position of TMDS_PEEK_DOUBLE_L2 field.
	SIO_TMDS_PEEK_DOUBLE_L2_TMDS_PEEK_DOUBLE_L2_Pos = 0x0
	// Bit mask of TMDS_PEEK_DOUBLE_L2 field.
	SIO_TMDS_PEEK_DOUBLE_L2_TMDS_PEEK_DOUBLE_L2_Msk = 0xffffffff

	// TMDS_POP_DOUBLE_L2: Get lane 2 of the encoding of two pixels' worth of colour data. Two 10-bit TMDS symbols are packed at the bottom of a 32-bit word. The POP alias shifts the colour register when read, according to the values of PIX_SHIFT and PIX2_NOSHIFT.
	// Position of TMDS_POP_DOUBLE_L2 field.
	SIO_TMDS_POP_DOUBLE_L2_TMDS_POP_DOUBLE_L2_Pos = 0x0
	// Bit mask of TMDS_POP_DOUBLE_L2 field.
	SIO_TMDS_POP_DOUBLE_L2_TMDS_POP_DOUBLE_L2_Msk = 0xffffffff
)

// Constants for BOOTRAM: Additional registers mapped adjacent to the bootram, for use by the bootrom.
const (
	// WRITE_ONCE0: This registers always ORs writes into its current contents. Once a bit is set, it can only be cleared by a reset.
	// Position of WRITE_ONCE0 field.
	BOOTRAM_WRITE_ONCE0_WRITE_ONCE0_Pos = 0x0
	// Bit mask of WRITE_ONCE0 field.
	BOOTRAM_WRITE_ONCE0_WRITE_ONCE0_Msk = 0xffffffff

	// WRITE_ONCE1: This registers always ORs writes into its current contents. Once a bit is set, it can only be cleared by a reset.
	// Position of WRITE_ONCE1 field.
	BOOTRAM_WRITE_ONCE1_WRITE_ONCE1_Pos = 0x0
	// Bit mask of WRITE_ONCE1 field.
	BOOTRAM_WRITE_ONCE1_WRITE_ONCE1_Msk = 0xffffffff

	// BOOTLOCK_STAT: Bootlock status register. 1=unclaimed, 0=claimed. These locks function identically to the SIO spinlocks, but are reserved for bootrom use.
	// Position of BOOTLOCK_STAT field.
	BOOTRAM_BOOTLOCK_STAT_BOOTLOCK_STAT_Pos = 0x0
	// Bit mask of BOOTLOCK_STAT field.
	BOOTRAM_BOOTLOCK_STAT_BOOTLOCK_STAT_Msk = 0xff

	// BOOTLOCK0: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
	// Position of BOOTLOCK0 field.
	BOOTRAM_BOOTLOCK0_BOOTLOCK0_Pos = 0x0
	// Bit mask of BOOTLOCK0 field.
	BOOTRAM_BOOTLOCK0_BOOTLOCK0_Msk = 0xffffffff

	// BOOTLOCK1: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
	// Position of BOOTLOCK1 field.
	BOOTRAM_BOOTLOCK1_BOOTLOCK1_Pos = 0x0
	// Bit mask of BOOTLOCK1 field.
	BOOTRAM_BOOTLOCK1_BOOTLOCK1_Msk = 0xffffffff

	// BOOTLOCK2: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
	// Position of BOOTLOCK2 field.
	BOOTRAM_BOOTLOCK2_BOOTLOCK2_Pos = 0x0
	// Bit mask of BOOTLOCK2 field.
	BOOTRAM_BOOTLOCK2_BOOTLOCK2_Msk = 0xffffffff

	// BOOTLOCK3: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
	// Position of BOOTLOCK3 field.
	BOOTRAM_BOOTLOCK3_BOOTLOCK3_Pos = 0x0
	// Bit mask of BOOTLOCK3 field.
	BOOTRAM_BOOTLOCK3_BOOTLOCK3_Msk = 0xffffffff

	// BOOTLOCK4: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
	// Position of BOOTLOCK4 field.
	BOOTRAM_BOOTLOCK4_BOOTLOCK4_Pos = 0x0
	// Bit mask of BOOTLOCK4 field.
	BOOTRAM_BOOTLOCK4_BOOTLOCK4_Msk = 0xffffffff

	// BOOTLOCK5: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
	// Position of BOOTLOCK5 field.
	BOOTRAM_BOOTLOCK5_BOOTLOCK5_Pos = 0x0
	// Bit mask of BOOTLOCK5 field.
	BOOTRAM_BOOTLOCK5_BOOTLOCK5_Msk = 0xffffffff

	// BOOTLOCK6: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
	// Position of BOOTLOCK6 field.
	BOOTRAM_BOOTLOCK6_BOOTLOCK6_Pos = 0x0
	// Bit mask of BOOTLOCK6 field.
	BOOTRAM_BOOTLOCK6_BOOTLOCK6_Msk = 0xffffffff

	// BOOTLOCK7: Read to claim and check. Write to unclaim. The value returned on successful claim is 1 << n, and on failed claim is zero.
	// Position of BOOTLOCK7 field.
	BOOTRAM_BOOTLOCK7_BOOTLOCK7_Pos = 0x0
	// Bit mask of BOOTLOCK7 field.
	BOOTRAM_BOOTLOCK7_BOOTLOCK7_Msk = 0xffffffff
)

// Constants for CORESIGHT_TRACE: Coresight block - RP specific registers
const (
	// CTRL_STATUS: Control and status register
	// Position of TRACE_CAPTURE_FIFO_OVERFLOW field.
	CORESIGHT_TRACE_CTRL_STATUS_TRACE_CAPTURE_FIFO_OVERFLOW_Pos = 0x1
	// Bit mask of TRACE_CAPTURE_FIFO_OVERFLOW field.
	CORESIGHT_TRACE_CTRL_STATUS_TRACE_CAPTURE_FIFO_OVERFLOW_Msk = 0x2
	// Bit TRACE_CAPTURE_FIFO_OVERFLOW.
	CORESIGHT_TRACE_CTRL_STATUS_TRACE_CAPTURE_FIFO_OVERFLOW = 0x2
	// Position of TRACE_CAPTURE_FIFO_FLUSH field.
	CORESIGHT_TRACE_CTRL_STATUS_TRACE_CAPTURE_FIFO_FLUSH_Pos = 0x0
	// Bit mask of TRACE_CAPTURE_FIFO_FLUSH field.
	CORESIGHT_TRACE_CTRL_STATUS_TRACE_CAPTURE_FIFO_FLUSH_Msk = 0x1
	// Bit TRACE_CAPTURE_FIFO_FLUSH.
	CORESIGHT_TRACE_CTRL_STATUS_TRACE_CAPTURE_FIFO_FLUSH = 0x1

	// TRACE_CAPTURE_FIFO: FIFO for trace data captured from the TPIU
	// Position of RDATA field.
	CORESIGHT_TRACE_TRACE_CAPTURE_FIFO_RDATA_Pos = 0x0
	// Bit mask of RDATA field.
	CORESIGHT_TRACE_TRACE_CAPTURE_FIFO_RDATA_Msk = 0xffffffff
)

// Constants for USB: USB FS/LS controller device registers
const (
	// ADDR_ENDP: Device address and endpoint control
	// Position of ENDPOINT field.
	USB_ADDR_ENDP_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP_ADDRESS_Msk = 0x7f

	// ADDR_ENDP1: Interrupt endpoint 1. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP1_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP1_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP1_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP1_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP1_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP1_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP1_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP1_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP1_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP1_ADDRESS_Msk = 0x7f

	// ADDR_ENDP2: Interrupt endpoint 2. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP2_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP2_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP2_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP2_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP2_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP2_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP2_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP2_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP2_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP2_ADDRESS_Msk = 0x7f

	// ADDR_ENDP3: Interrupt endpoint 3. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP3_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP3_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP3_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP3_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP3_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP3_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP3_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP3_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP3_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP3_ADDRESS_Msk = 0x7f

	// ADDR_ENDP4: Interrupt endpoint 4. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP4_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP4_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP4_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP4_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP4_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP4_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP4_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP4_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP4_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP4_ADDRESS_Msk = 0x7f

	// ADDR_ENDP5: Interrupt endpoint 5. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP5_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP5_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP5_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP5_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP5_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP5_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP5_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP5_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP5_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP5_ADDRESS_Msk = 0x7f

	// ADDR_ENDP6: Interrupt endpoint 6. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP6_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP6_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP6_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP6_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP6_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP6_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP6_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP6_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP6_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP6_ADDRESS_Msk = 0x7f

	// ADDR_ENDP7: Interrupt endpoint 7. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP7_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP7_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP7_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP7_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP7_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP7_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP7_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP7_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP7_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP7_ADDRESS_Msk = 0x7f

	// ADDR_ENDP8: Interrupt endpoint 8. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP8_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP8_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP8_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP8_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP8_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP8_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP8_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP8_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP8_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP8_ADDRESS_Msk = 0x7f

	// ADDR_ENDP9: Interrupt endpoint 9. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP9_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP9_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP9_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP9_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP9_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP9_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP9_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP9_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP9_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP9_ADDRESS_Msk = 0x7f

	// ADDR_ENDP10: Interrupt endpoint 10. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP10_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP10_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP10_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP10_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP10_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP10_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP10_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP10_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP10_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP10_ADDRESS_Msk = 0x7f

	// ADDR_ENDP11: Interrupt endpoint 11. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP11_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP11_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP11_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP11_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP11_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP11_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP11_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP11_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP11_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP11_ADDRESS_Msk = 0x7f

	// ADDR_ENDP12: Interrupt endpoint 12. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP12_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP12_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP12_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP12_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP12_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP12_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP12_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP12_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP12_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP12_ADDRESS_Msk = 0x7f

	// ADDR_ENDP13: Interrupt endpoint 13. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP13_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP13_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP13_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP13_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP13_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP13_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP13_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP13_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP13_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP13_ADDRESS_Msk = 0x7f

	// ADDR_ENDP14: Interrupt endpoint 14. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP14_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP14_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP14_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP14_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP14_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP14_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP14_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP14_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP14_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP14_ADDRESS_Msk = 0x7f

	// ADDR_ENDP15: Interrupt endpoint 15. Only valid for HOST mode.
	// Position of INTEP_PREAMBLE field.
	USB_ADDR_ENDP15_INTEP_PREAMBLE_Pos = 0x1a
	// Bit mask of INTEP_PREAMBLE field.
	USB_ADDR_ENDP15_INTEP_PREAMBLE_Msk = 0x4000000
	// Bit INTEP_PREAMBLE.
	USB_ADDR_ENDP15_INTEP_PREAMBLE = 0x4000000
	// Position of INTEP_DIR field.
	USB_ADDR_ENDP15_INTEP_DIR_Pos = 0x19
	// Bit mask of INTEP_DIR field.
	USB_ADDR_ENDP15_INTEP_DIR_Msk = 0x2000000
	// Bit INTEP_DIR.
	USB_ADDR_ENDP15_INTEP_DIR = 0x2000000
	// Position of ENDPOINT field.
	USB_ADDR_ENDP15_ENDPOINT_Pos = 0x10
	// Bit mask of ENDPOINT field.
	USB_ADDR_ENDP15_ENDPOINT_Msk = 0xf0000
	// Position of ADDRESS field.
	USB_ADDR_ENDP15_ADDRESS_Pos = 0x0
	// Bit mask of ADDRESS field.
	USB_ADDR_ENDP15_ADDRESS_Msk = 0x7f

	// MAIN_CTRL: Main control register
	// Position of SIM_TIMING field.
	USB_MAIN_CTRL_SIM_TIMING_Pos = 0x1f
	// Bit mask of SIM_TIMING field.
	USB_MAIN_CTRL_SIM_TIMING_Msk = 0x80000000
	// Bit SIM_TIMING.
	USB_MAIN_CTRL_SIM_TIMING = 0x80000000
	// Position of PHY_ISO field.
	USB_MAIN_CTRL_PHY_ISO_Pos = 0x2
	// Bit mask of PHY_ISO field.
	USB_MAIN_CTRL_PHY_ISO_Msk = 0x4
	// Bit PHY_ISO.
	USB_MAIN_CTRL_PHY_ISO = 0x4
	// Position of HOST_NDEVICE field.
	USB_MAIN_CTRL_HOST_NDEVICE_Pos = 0x1
	// Bit mask of HOST_NDEVICE field.
	USB_MAIN_CTRL_HOST_NDEVICE_Msk = 0x2
	// Bit HOST_NDEVICE.
	USB_MAIN_CTRL_HOST_NDEVICE = 0x2
	// Position of CONTROLLER_EN field.
	USB_MAIN_CTRL_CONTROLLER_EN_Pos = 0x0
	// Bit mask of CONTROLLER_EN field.
	USB_MAIN_CTRL_CONTROLLER_EN_Msk = 0x1
	// Bit CONTROLLER_EN.
	USB_MAIN_CTRL_CONTROLLER_EN = 0x1

	// SOF_WR: Set the SOF (Start of Frame) frame number in the host controller. The SOF packet is sent every 1ms and the host will increment the frame number by 1 each time.
	// Position of COUNT field.
	USB_SOF_WR_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	USB_SOF_WR_COUNT_Msk = 0x7ff

	// SOF_RD: Read the last SOF (Start of Frame) frame number seen. In device mode the last SOF received from the host. In host mode the last SOF sent by the host.
	// Position of COUNT field.
	USB_SOF_RD_COUNT_Pos = 0x0
	// Bit mask of COUNT field.
	USB_SOF_RD_COUNT_Msk = 0x7ff

	// SIE_CTRL: SIE control register
	// Position of EP0_INT_STALL field.
	USB_SIE_CTRL_EP0_INT_STALL_Pos = 0x1f
	// Bit mask of EP0_INT_STALL field.
	USB_SIE_CTRL_EP0_INT_STALL_Msk = 0x80000000
	// Bit EP0_INT_STALL.
	USB_SIE_CTRL_EP0_INT_STALL = 0x80000000
	// Position of EP0_DOUBLE_BUF field.
	USB_SIE_CTRL_EP0_DOUBLE_BUF_Pos = 0x1e
	// Bit mask of EP0_DOUBLE_BUF field.
	USB_SIE_CTRL_EP0_DOUBLE_BUF_Msk = 0x40000000
	// Bit EP0_DOUBLE_BUF.
	USB_SIE_CTRL_EP0_DOUBLE_BUF = 0x40000000
	// Position of EP0_INT_1BUF field.
	USB_SIE_CTRL_EP0_INT_1BUF_Pos = 0x1d
	// Bit mask of EP0_INT_1BUF field.
	USB_SIE_CTRL_EP0_INT_1BUF_Msk = 0x20000000
	// Bit EP0_INT_1BUF.
	USB_SIE_CTRL_EP0_INT_1BUF = 0x20000000
	// Position of EP0_INT_2BUF field.
	USB_SIE_CTRL_EP0_INT_2BUF_Pos = 0x1c
	// Bit mask of EP0_INT_2BUF field.
	USB_SIE_CTRL_EP0_INT_2BUF_Msk = 0x10000000
	// Bit EP0_INT_2BUF.
	USB_SIE_CTRL_EP0_INT_2BUF = 0x10000000
	// Position of EP0_INT_NAK field.
	USB_SIE_CTRL_EP0_INT_NAK_Pos = 0x1b
	// Bit mask of EP0_INT_NAK field.
	USB_SIE_CTRL_EP0_INT_NAK_Msk = 0x8000000
	// Bit EP0_INT_NAK.
	USB_SIE_CTRL_EP0_INT_NAK = 0x8000000
	// Position of DIRECT_EN field.
	USB_SIE_CTRL_DIRECT_EN_Pos = 0x1a
	// Bit mask of DIRECT_EN field.
	USB_SIE_CTRL_DIRECT_EN_Msk = 0x4000000
	// Bit DIRECT_EN.
	USB_SIE_CTRL_DIRECT_EN = 0x4000000
	// Position of DIRECT_DP field.
	USB_SIE_CTRL_DIRECT_DP_Pos = 0x19
	// Bit mask of DIRECT_DP field.
	USB_SIE_CTRL_DIRECT_DP_Msk = 0x2000000
	// Bit DIRECT_DP.
	USB_SIE_CTRL_DIRECT_DP = 0x2000000
	// Position of DIRECT_DM field.
	USB_SIE_CTRL_DIRECT_DM_Pos = 0x18
	// Bit mask of DIRECT_DM field.
	USB_SIE_CTRL_DIRECT_DM_Msk = 0x1000000
	// Bit DIRECT_DM.
	USB_SIE_CTRL_DIRECT_DM = 0x1000000
	// Position of EP0_STOP_ON_SHORT_PACKET field.
	USB_SIE_CTRL_EP0_STOP_ON_SHORT_PACKET_Pos = 0x13
	// Bit mask of EP0_STOP_ON_SHORT_PACKET field.
	USB_SIE_CTRL_EP0_STOP_ON_SHORT_PACKET_Msk = 0x80000
	// Bit EP0_STOP_ON_SHORT_PACKET.
	USB_SIE_CTRL_EP0_STOP_ON_SHORT_PACKET = 0x80000
	// Position of TRANSCEIVER_PD field.
	USB_SIE_CTRL_TRANSCEIVER_PD_Pos = 0x12
	// Bit mask of TRANSCEIVER_PD field.
	USB_SIE_CTRL_TRANSCEIVER_PD_Msk = 0x40000
	// Bit TRANSCEIVER_PD.
	USB_SIE_CTRL_TRANSCEIVER_PD = 0x40000
	// Position of RPU_OPT field.
	USB_SIE_CTRL_RPU_OPT_Pos = 0x11
	// Bit mask of RPU_OPT field.
	USB_SIE_CTRL_RPU_OPT_Msk = 0x20000
	// Bit RPU_OPT.
	USB_SIE_CTRL_RPU_OPT = 0x20000
	// Position of PULLUP_EN field.
	USB_SIE_CTRL_PULLUP_EN_Pos = 0x10
	// Bit mask of PULLUP_EN field.
	USB_SIE_CTRL_PULLUP_EN_Msk = 0x10000
	// Bit PULLUP_EN.
	USB_SIE_CTRL_PULLUP_EN = 0x10000
	// Position of PULLDOWN_EN field.
	USB_SIE_CTRL_PULLDOWN_EN_Pos = 0xf
	// Bit mask of PULLDOWN_EN field.
	USB_SIE_CTRL_PULLDOWN_EN_Msk = 0x8000
	// Bit PULLDOWN_EN.
	USB_SIE_CTRL_PULLDOWN_EN = 0x8000
	// Position of RESET_BUS field.
	USB_SIE_CTRL_RESET_BUS_Pos = 0xd
	// Bit mask of RESET_BUS field.
	USB_SIE_CTRL_RESET_BUS_Msk = 0x2000
	// Bit RESET_BUS.
	USB_SIE_CTRL_RESET_BUS = 0x2000
	// Position of RESUME field.
	USB_SIE_CTRL_RESUME_Pos = 0xc
	// Bit mask of RESUME field.
	USB_SIE_CTRL_RESUME_Msk = 0x1000
	// Bit RESUME.
	USB_SIE_CTRL_RESUME = 0x1000
	// Position of VBUS_EN field.
	USB_SIE_CTRL_VBUS_EN_Pos = 0xb
	// Bit mask of VBUS_EN field.
	USB_SIE_CTRL_VBUS_EN_Msk = 0x800
	// Bit VBUS_EN.
	USB_SIE_CTRL_VBUS_EN = 0x800
	// Position of KEEP_ALIVE_EN field.
	USB_SIE_CTRL_KEEP_ALIVE_EN_Pos = 0xa
	// Bit mask of KEEP_ALIVE_EN field.
	USB_SIE_CTRL_KEEP_ALIVE_EN_Msk = 0x400
	// Bit KEEP_ALIVE_EN.
	USB_SIE_CTRL_KEEP_ALIVE_EN = 0x400
	// Position of SOF_EN field.
	USB_SIE_CTRL_SOF_EN_Pos = 0x9
	// Bit mask of SOF_EN field.
	USB_SIE_CTRL_SOF_EN_Msk = 0x200
	// Bit SOF_EN.
	USB_SIE_CTRL_SOF_EN = 0x200
	// Position of SOF_SYNC field.
	USB_SIE_CTRL_SOF_SYNC_Pos = 0x8
	// Bit mask of SOF_SYNC field.
	USB_SIE_CTRL_SOF_SYNC_Msk = 0x100
	// Bit SOF_SYNC.
	USB_SIE_CTRL_SOF_SYNC = 0x100
	// Position of PREAMBLE_EN field.
	USB_SIE_CTRL_PREAMBLE_EN_Pos = 0x6
	// Bit mask of PREAMBLE_EN field.
	USB_SIE_CTRL_PREAMBLE_EN_Msk = 0x40
	// Bit PREAMBLE_EN.
	USB_SIE_CTRL_PREAMBLE_EN = 0x40
	// Position of STOP_TRANS field.
	USB_SIE_CTRL_STOP_TRANS_Pos = 0x4
	// Bit mask of STOP_TRANS field.
	USB_SIE_CTRL_STOP_TRANS_Msk = 0x10
	// Bit STOP_TRANS.
	USB_SIE_CTRL_STOP_TRANS = 0x10
	// Position of RECEIVE_DATA field.
	USB_SIE_CTRL_RECEIVE_DATA_Pos = 0x3
	// Bit mask of RECEIVE_DATA field.
	USB_SIE_CTRL_RECEIVE_DATA_Msk = 0x8
	// Bit RECEIVE_DATA.
	USB_SIE_CTRL_RECEIVE_DATA = 0x8
	// Position of SEND_DATA field.
	USB_SIE_CTRL_SEND_DATA_Pos = 0x2
	// Bit mask of SEND_DATA field.
	USB_SIE_CTRL_SEND_DATA_Msk = 0x4
	// Bit SEND_DATA.
	USB_SIE_CTRL_SEND_DATA = 0x4
	// Position of SEND_SETUP field.
	USB_SIE_CTRL_SEND_SETUP_Pos = 0x1
	// Bit mask of SEND_SETUP field.
	USB_SIE_CTRL_SEND_SETUP_Msk = 0x2
	// Bit SEND_SETUP.
	USB_SIE_CTRL_SEND_SETUP = 0x2
	// Position of START_TRANS field.
	USB_SIE_CTRL_START_TRANS_Pos = 0x0
	// Bit mask of START_TRANS field.
	USB_SIE_CTRL_START_TRANS_Msk = 0x1
	// Bit START_TRANS.
	USB_SIE_CTRL_START_TRANS = 0x1

	// SIE_STATUS: SIE status register
	// Position of DATA_SEQ_ERROR field.
	USB_SIE_STATUS_DATA_SEQ_ERROR_Pos = 0x1f
	// Bit mask of DATA_SEQ_ERROR field.
	USB_SIE_STATUS_DATA_SEQ_ERROR_Msk = 0x80000000
	// Bit DATA_SEQ_ERROR.
	USB_SIE_STATUS_DATA_SEQ_ERROR = 0x80000000
	// Position of ACK_REC field.
	USB_SIE_STATUS_ACK_REC_Pos = 0x1e
	// Bit mask of ACK_REC field.
	USB_SIE_STATUS_ACK_REC_Msk = 0x40000000
	// Bit ACK_REC.
	USB_SIE_STATUS_ACK_REC = 0x40000000
	// Position of STALL_REC field.
	USB_SIE_STATUS_STALL_REC_Pos = 0x1d
	// Bit mask of STALL_REC field.
	USB_SIE_STATUS_STALL_REC_Msk = 0x20000000
	// Bit STALL_REC.
	USB_SIE_STATUS_STALL_REC = 0x20000000
	// Position of NAK_REC field.
	USB_SIE_STATUS_NAK_REC_Pos = 0x1c
	// Bit mask of NAK_REC field.
	USB_SIE_STATUS_NAK_REC_Msk = 0x10000000
	// Bit NAK_REC.
	USB_SIE_STATUS_NAK_REC = 0x10000000
	// Position of RX_TIMEOUT field.
	USB_SIE_STATUS_RX_TIMEOUT_Pos = 0x1b
	// Bit mask of RX_TIMEOUT field.
	USB_SIE_STATUS_RX_TIMEOUT_Msk = 0x8000000
	// Bit RX_TIMEOUT.
	USB_SIE_STATUS_RX_TIMEOUT = 0x8000000
	// Position of RX_OVERFLOW field.
	USB_SIE_STATUS_RX_OVERFLOW_Pos = 0x1a
	// Bit mask of RX_OVERFLOW field.
	USB_SIE_STATUS_RX_OVERFLOW_Msk = 0x4000000
	// Bit RX_OVERFLOW.
	USB_SIE_STATUS_RX_OVERFLOW = 0x4000000
	// Position of BIT_STUFF_ERROR field.
	USB_SIE_STATUS_BIT_STUFF_ERROR_Pos = 0x19
	// Bit mask of BIT_STUFF_ERROR field.
	USB_SIE_STATUS_BIT_STUFF_ERROR_Msk = 0x2000000
	// Bit BIT_STUFF_ERROR.
	USB_SIE_STATUS_BIT_STUFF_ERROR = 0x2000000
	// Position of CRC_ERROR field.
	USB_SIE_STATUS_CRC_ERROR_Pos = 0x18
	// Bit mask of CRC_ERROR field.
	USB_SIE_STATUS_CRC_ERROR_Msk = 0x1000000
	// Bit CRC_ERROR.
	USB_SIE_STATUS_CRC_ERROR = 0x1000000
	// Position of ENDPOINT_ERROR field.
	USB_SIE_STATUS_ENDPOINT_ERROR_Pos = 0x17
	// Bit mask of ENDPOINT_ERROR field.
	USB_SIE_STATUS_ENDPOINT_ERROR_Msk = 0x800000
	// Bit ENDPOINT_ERROR.
	USB_SIE_STATUS_ENDPOINT_ERROR = 0x800000
	// Position of BUS_RESET field.
	USB_SIE_STATUS_BUS_RESET_Pos = 0x13
	// Bit mask of BUS_RESET field.
	USB_SIE_STATUS_BUS_RESET_Msk = 0x80000
	// Bit BUS_RESET.
	USB_SIE_STATUS_BUS_RESET = 0x80000
	// Position of TRANS_COMPLETE field.
	USB_SIE_STATUS_TRANS_COMPLETE_Pos = 0x12
	// Bit mask of TRANS_COMPLETE field.
	USB_SIE_STATUS_TRANS_COMPLETE_Msk = 0x40000
	// Bit TRANS_COMPLETE.
	USB_SIE_STATUS_TRANS_COMPLETE = 0x40000
	// Position of SETUP_REC field.
	USB_SIE_STATUS_SETUP_REC_Pos = 0x11
	// Bit mask of SETUP_REC field.
	USB_SIE_STATUS_SETUP_REC_Msk = 0x20000
	// Bit SETUP_REC.
	USB_SIE_STATUS_SETUP_REC = 0x20000
	// Position of CONNECTED field.
	USB_SIE_STATUS_CONNECTED_Pos = 0x10
	// Bit mask of CONNECTED field.
	USB_SIE_STATUS_CONNECTED_Msk = 0x10000
	// Bit CONNECTED.
	USB_SIE_STATUS_CONNECTED = 0x10000
	// Position of RX_SHORT_PACKET field.
	USB_SIE_STATUS_RX_SHORT_PACKET_Pos = 0xc
	// Bit mask of RX_SHORT_PACKET field.
	USB_SIE_STATUS_RX_SHORT_PACKET_Msk = 0x1000
	// Bit RX_SHORT_PACKET.
	USB_SIE_STATUS_RX_SHORT_PACKET = 0x1000
	// Position of RESUME field.
	USB_SIE_STATUS_RESUME_Pos = 0xb
	// Bit mask of RESUME field.
	USB_SIE_STATUS_RESUME_Msk = 0x800
	// Bit RESUME.
	USB_SIE_STATUS_RESUME = 0x800
	// Position of VBUS_OVER_CURR field.
	USB_SIE_STATUS_VBUS_OVER_CURR_Pos = 0xa
	// Bit mask of VBUS_OVER_CURR field.
	USB_SIE_STATUS_VBUS_OVER_CURR_Msk = 0x400
	// Bit VBUS_OVER_CURR.
	USB_SIE_STATUS_VBUS_OVER_CURR = 0x400
	// Position of SPEED field.
	USB_SIE_STATUS_SPEED_Pos = 0x8
	// Bit mask of SPEED field.
	USB_SIE_STATUS_SPEED_Msk = 0x300
	// Position of SUSPENDED field.
	USB_SIE_STATUS_SUSPENDED_Pos = 0x4
	// Bit mask of SUSPENDED field.
	USB_SIE_STATUS_SUSPENDED_Msk = 0x10
	// Bit SUSPENDED.
	USB_SIE_STATUS_SUSPENDED = 0x10
	// Position of LINE_STATE field.
	USB_SIE_STATUS_LINE_STATE_Pos = 0x2
	// Bit mask of LINE_STATE field.
	USB_SIE_STATUS_LINE_STATE_Msk = 0xc
	// Position of VBUS_DETECTED field.
	USB_SIE_STATUS_VBUS_DETECTED_Pos = 0x0
	// Bit mask of VBUS_DETECTED field.
	USB_SIE_STATUS_VBUS_DETECTED_Msk = 0x1
	// Bit VBUS_DETECTED.
	USB_SIE_STATUS_VBUS_DETECTED = 0x1

	// INT_EP_CTRL: interrupt endpoint control register
	// Position of INT_EP_ACTIVE field.
	USB_INT_EP_CTRL_INT_EP_ACTIVE_Pos = 0x1
	// Bit mask of INT_EP_ACTIVE field.
	USB_INT_EP_CTRL_INT_EP_ACTIVE_Msk = 0xfffe

	// BUFF_STATUS: Buffer status register. A bit set here indicates that a buffer has completed on the endpoint (if the buffer interrupt is enabled). It is possible for 2 buffers to be completed, so clearing the buffer status bit may instantly re set it on the next clock cycle.
	// Position of EP15_OUT field.
	USB_BUFF_STATUS_EP15_OUT_Pos = 0x1f
	// Bit mask of EP15_OUT field.
	USB_BUFF_STATUS_EP15_OUT_Msk = 0x80000000
	// Bit EP15_OUT.
	USB_BUFF_STATUS_EP15_OUT = 0x80000000
	// Position of EP15_IN field.
	USB_BUFF_STATUS_EP15_IN_Pos = 0x1e
	// Bit mask of EP15_IN field.
	USB_BUFF_STATUS_EP15_IN_Msk = 0x40000000
	// Bit EP15_IN.
	USB_BUFF_STATUS_EP15_IN = 0x40000000
	// Position of EP14_OUT field.
	USB_BUFF_STATUS_EP14_OUT_Pos = 0x1d
	// Bit mask of EP14_OUT field.
	USB_BUFF_STATUS_EP14_OUT_Msk = 0x20000000
	// Bit EP14_OUT.
	USB_BUFF_STATUS_EP14_OUT = 0x20000000
	// Position of EP14_IN field.
	USB_BUFF_STATUS_EP14_IN_Pos = 0x1c
	// Bit mask of EP14_IN field.
	USB_BUFF_STATUS_EP14_IN_Msk = 0x10000000
	// Bit EP14_IN.
	USB_BUFF_STATUS_EP14_IN = 0x10000000
	// Position of EP13_OUT field.
	USB_BUFF_STATUS_EP13_OUT_Pos = 0x1b
	// Bit mask of EP13_OUT field.
	USB_BUFF_STATUS_EP13_OUT_Msk = 0x8000000
	// Bit EP13_OUT.
	USB_BUFF_STATUS_EP13_OUT = 0x8000000
	// Position of EP13_IN field.
	USB_BUFF_STATUS_EP13_IN_Pos = 0x1a
	// Bit mask of EP13_IN field.
	USB_BUFF_STATUS_EP13_IN_Msk = 0x4000000
	// Bit EP13_IN.
	USB_BUFF_STATUS_EP13_IN = 0x4000000
	// Position of EP12_OUT field.
	USB_BUFF_STATUS_EP12_OUT_Pos = 0x19
	// Bit mask of EP12_OUT field.
	USB_BUFF_STATUS_EP12_OUT_Msk = 0x2000000
	// Bit EP12_OUT.
	USB_BUFF_STATUS_EP12_OUT = 0x2000000
	// Position of EP12_IN field.
	USB_BUFF_STATUS_EP12_IN_Pos = 0x18
	// Bit mask of EP12_IN field.
	USB_BUFF_STATUS_EP12_IN_Msk = 0x1000000
	// Bit EP12_IN.
	USB_BUFF_STATUS_EP12_IN = 0x1000000
	// Position of EP11_OUT field.
	USB_BUFF_STATUS_EP11_OUT_Pos = 0x17
	// Bit mask of EP11_OUT field.
	USB_BUFF_STATUS_EP11_OUT_Msk = 0x800000
	// Bit EP11_OUT.
	USB_BUFF_STATUS_EP11_OUT = 0x800000
	// Position of EP11_IN field.
	USB_BUFF_STATUS_EP11_IN_Pos = 0x16
	// Bit mask of EP11_IN field.
	USB_BUFF_STATUS_EP11_IN_Msk = 0x400000
	// Bit EP11_IN.
	USB_BUFF_STATUS_EP11_IN = 0x400000
	// Position of EP10_OUT field.
	USB_BUFF_STATUS_EP10_OUT_Pos = 0x15
	// Bit mask of EP10_OUT field.
	USB_BUFF_STATUS_EP10_OUT_Msk = 0x200000
	// Bit EP10_OUT.
	USB_BUFF_STATUS_EP10_OUT = 0x200000
	// Position of EP10_IN field.
	USB_BUFF_STATUS_EP10_IN_Pos = 0x14
	// Bit mask of EP10_IN field.
	USB_BUFF_STATUS_EP10_IN_Msk = 0x100000
	// Bit EP10_IN.
	USB_BUFF_STATUS_EP10_IN = 0x100000
	// Position of EP9_OUT field.
	USB_BUFF_STATUS_EP9_OUT_Pos = 0x13
	// Bit mask of EP9_OUT field.
	USB_BUFF_STATUS_EP9_OUT_Msk = 0x80000
	// Bit EP9_OUT.
	USB_BUFF_STATUS_EP9_OUT = 0x80000
	// Position of EP9_IN field.
	USB_BUFF_STATUS_EP9_IN_Pos = 0x12
	// Bit mask of EP9_IN field.
	USB_BUFF_STATUS_EP9_IN_Msk = 0x40000
	// Bit EP9_IN.
	USB_BUFF_STATUS_EP9_IN = 0x40000
	// Position of EP8_OUT field.
	USB_BUFF_STATUS_EP8_OUT_Pos = 0x11
	// Bit mask of EP8_OUT field.
	USB_BUFF_STATUS_EP8_OUT_Msk = 0x20000
	// Bit EP8_OUT.
	USB_BUFF_STATUS_EP8_OUT = 0x20000
	// Position of EP8_IN field.
	USB_BUFF_STATUS_EP8_IN_Pos = 0x10
	// Bit mask of EP8_IN field.
	USB_BUFF_STATUS_EP8_IN_Msk = 0x10000
	// Bit EP8_IN.
	USB_BUFF_STATUS_EP8_IN = 0x10000
	// Position of EP7_OUT field.
	USB_BUFF_STATUS_EP7_OUT_Pos = 0xf
	// Bit mask of EP7_OUT field.
	USB_BUFF_STATUS_EP7_OUT_Msk = 0x8000
	// Bit EP7_OUT.
	USB_BUFF_STATUS_EP7_OUT = 0x8000
	// Position of EP7_IN field.
	USB_BUFF_STATUS_EP7_IN_Pos = 0xe
	// Bit mask of EP7_IN field.
	USB_BUFF_STATUS_EP7_IN_Msk = 0x4000
	// Bit EP7_IN.
	USB_BUFF_STATUS_EP7_IN = 0x4000
	// Position of EP6_OUT field.
	USB_BUFF_STATUS_EP6_OUT_Pos = 0xd
	// Bit mask of EP6_OUT field.
	USB_BUFF_STATUS_EP6_OUT_Msk = 0x2000
	// Bit EP6_OUT.
	USB_BUFF_STATUS_EP6_OUT = 0x2000
	// Position of EP6_IN field.
	USB_BUFF_STATUS_EP6_IN_Pos = 0xc
	// Bit mask of EP6_IN field.
	USB_BUFF_STATUS_EP6_IN_Msk = 0x1000
	// Bit EP6_IN.
	USB_BUFF_STATUS_EP6_IN = 0x1000
	// Position of EP5_OUT field.
	USB_BUFF_STATUS_EP5_OUT_Pos = 0xb
	// Bit mask of EP5_OUT field.
	USB_BUFF_STATUS_EP5_OUT_Msk = 0x800
	// Bit EP5_OUT.
	USB_BUFF_STATUS_EP5_OUT = 0x800
	// Position of EP5_IN field.
	USB_BUFF_STATUS_EP5_IN_Pos = 0xa
	// Bit mask of EP5_IN field.
	USB_BUFF_STATUS_EP5_IN_Msk = 0x400
	// Bit EP5_IN.
	USB_BUFF_STATUS_EP5_IN = 0x400
	// Position of EP4_OUT field.
	USB_BUFF_STATUS_EP4_OUT_Pos = 0x9
	// Bit mask of EP4_OUT field.
	USB_BUFF_STATUS_EP4_OUT_Msk = 0x200
	// Bit EP4_OUT.
	USB_BUFF_STATUS_EP4_OUT = 0x200
	// Position of EP4_IN field.
	USB_BUFF_STATUS_EP4_IN_Pos = 0x8
	// Bit mask of EP4_IN field.
	USB_BUFF_STATUS_EP4_IN_Msk = 0x100
	// Bit EP4_IN.
	USB_BUFF_STATUS_EP4_IN = 0x100
	// Position of EP3_OUT field.
	USB_BUFF_STATUS_EP3_OUT_Pos = 0x7
	// Bit mask of EP3_OUT field.
	USB_BUFF_STATUS_EP3_OUT_Msk = 0x80
	// Bit EP3_OUT.
	USB_BUFF_STATUS_EP3_OUT = 0x80
	// Position of EP3_IN field.
	USB_BUFF_STATUS_EP3_IN_Pos = 0x6
	// Bit mask of EP3_IN field.
	USB_BUFF_STATUS_EP3_IN_Msk = 0x40
	// Bit EP3_IN.
	USB_BUFF_STATUS_EP3_IN = 0x40
	// Position of EP2_OUT field.
	USB_BUFF_STATUS_EP2_OUT_Pos = 0x5
	// Bit mask of EP2_OUT field.
	USB_BUFF_STATUS_EP2_OUT_Msk = 0x20
	// Bit EP2_OUT.
	USB_BUFF_STATUS_EP2_OUT = 0x20
	// Position of EP2_IN field.
	USB_BUFF_STATUS_EP2_IN_Pos = 0x4
	// Bit mask of EP2_IN field.
	USB_BUFF_STATUS_EP2_IN_Msk = 0x10
	// Bit EP2_IN.
	USB_BUFF_STATUS_EP2_IN = 0x10
	// Position of EP1_OUT field.
	USB_BUFF_STATUS_EP1_OUT_Pos = 0x3
	// Bit mask of EP1_OUT field.
	USB_BUFF_STATUS_EP1_OUT_Msk = 0x8
	// Bit EP1_OUT.
	USB_BUFF_STATUS_EP1_OUT = 0x8
	// Position of EP1_IN field.
	USB_BUFF_STATUS_EP1_IN_Pos = 0x2
	// Bit mask of EP1_IN field.
	USB_BUFF_STATUS_EP1_IN_Msk = 0x4
	// Bit EP1_IN.
	USB_BUFF_STATUS_EP1_IN = 0x4
	// Position of EP0_OUT field.
	USB_BUFF_STATUS_EP0_OUT_Pos = 0x1
	// Bit mask of EP0_OUT field.
	USB_BUFF_STATUS_EP0_OUT_Msk = 0x2
	// Bit EP0_OUT.
	USB_BUFF_STATUS_EP0_OUT = 0x2
	// Position of EP0_IN field.
	USB_BUFF_STATUS_EP0_IN_Pos = 0x0
	// Bit mask of EP0_IN field.
	USB_BUFF_STATUS_EP0_IN_Msk = 0x1
	// Bit EP0_IN.
	USB_BUFF_STATUS_EP0_IN = 0x1

	// BUFF_CPU_SHOULD_HANDLE: Which of the double buffers should be handled. Only valid if using an interrupt per buffer (i.e. not per 2 buffers). Not valid for host interrupt endpoint polling because they are only single buffered.
	// Position of EP15_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP15_OUT_Pos = 0x1f
	// Bit mask of EP15_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP15_OUT_Msk = 0x80000000
	// Bit EP15_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP15_OUT = 0x80000000
	// Position of EP15_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP15_IN_Pos = 0x1e
	// Bit mask of EP15_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP15_IN_Msk = 0x40000000
	// Bit EP15_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP15_IN = 0x40000000
	// Position of EP14_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP14_OUT_Pos = 0x1d
	// Bit mask of EP14_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP14_OUT_Msk = 0x20000000
	// Bit EP14_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP14_OUT = 0x20000000
	// Position of EP14_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP14_IN_Pos = 0x1c
	// Bit mask of EP14_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP14_IN_Msk = 0x10000000
	// Bit EP14_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP14_IN = 0x10000000
	// Position of EP13_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP13_OUT_Pos = 0x1b
	// Bit mask of EP13_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP13_OUT_Msk = 0x8000000
	// Bit EP13_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP13_OUT = 0x8000000
	// Position of EP13_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP13_IN_Pos = 0x1a
	// Bit mask of EP13_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP13_IN_Msk = 0x4000000
	// Bit EP13_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP13_IN = 0x4000000
	// Position of EP12_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP12_OUT_Pos = 0x19
	// Bit mask of EP12_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP12_OUT_Msk = 0x2000000
	// Bit EP12_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP12_OUT = 0x2000000
	// Position of EP12_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP12_IN_Pos = 0x18
	// Bit mask of EP12_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP12_IN_Msk = 0x1000000
	// Bit EP12_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP12_IN = 0x1000000
	// Position of EP11_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP11_OUT_Pos = 0x17
	// Bit mask of EP11_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP11_OUT_Msk = 0x800000
	// Bit EP11_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP11_OUT = 0x800000
	// Position of EP11_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP11_IN_Pos = 0x16
	// Bit mask of EP11_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP11_IN_Msk = 0x400000
	// Bit EP11_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP11_IN = 0x400000
	// Position of EP10_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP10_OUT_Pos = 0x15
	// Bit mask of EP10_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP10_OUT_Msk = 0x200000
	// Bit EP10_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP10_OUT = 0x200000
	// Position of EP10_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP10_IN_Pos = 0x14
	// Bit mask of EP10_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP10_IN_Msk = 0x100000
	// Bit EP10_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP10_IN = 0x100000
	// Position of EP9_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP9_OUT_Pos = 0x13
	// Bit mask of EP9_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP9_OUT_Msk = 0x80000
	// Bit EP9_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP9_OUT = 0x80000
	// Position of EP9_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP9_IN_Pos = 0x12
	// Bit mask of EP9_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP9_IN_Msk = 0x40000
	// Bit EP9_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP9_IN = 0x40000
	// Position of EP8_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP8_OUT_Pos = 0x11
	// Bit mask of EP8_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP8_OUT_Msk = 0x20000
	// Bit EP8_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP8_OUT = 0x20000
	// Position of EP8_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP8_IN_Pos = 0x10
	// Bit mask of EP8_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP8_IN_Msk = 0x10000
	// Bit EP8_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP8_IN = 0x10000
	// Position of EP7_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP7_OUT_Pos = 0xf
	// Bit mask of EP7_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP7_OUT_Msk = 0x8000
	// Bit EP7_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP7_OUT = 0x8000
	// Position of EP7_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP7_IN_Pos = 0xe
	// Bit mask of EP7_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP7_IN_Msk = 0x4000
	// Bit EP7_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP7_IN = 0x4000
	// Position of EP6_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP6_OUT_Pos = 0xd
	// Bit mask of EP6_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP6_OUT_Msk = 0x2000
	// Bit EP6_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP6_OUT = 0x2000
	// Position of EP6_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP6_IN_Pos = 0xc
	// Bit mask of EP6_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP6_IN_Msk = 0x1000
	// Bit EP6_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP6_IN = 0x1000
	// Position of EP5_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP5_OUT_Pos = 0xb
	// Bit mask of EP5_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP5_OUT_Msk = 0x800
	// Bit EP5_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP5_OUT = 0x800
	// Position of EP5_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP5_IN_Pos = 0xa
	// Bit mask of EP5_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP5_IN_Msk = 0x400
	// Bit EP5_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP5_IN = 0x400
	// Position of EP4_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP4_OUT_Pos = 0x9
	// Bit mask of EP4_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP4_OUT_Msk = 0x200
	// Bit EP4_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP4_OUT = 0x200
	// Position of EP4_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP4_IN_Pos = 0x8
	// Bit mask of EP4_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP4_IN_Msk = 0x100
	// Bit EP4_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP4_IN = 0x100
	// Position of EP3_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP3_OUT_Pos = 0x7
	// Bit mask of EP3_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP3_OUT_Msk = 0x80
	// Bit EP3_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP3_OUT = 0x80
	// Position of EP3_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP3_IN_Pos = 0x6
	// Bit mask of EP3_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP3_IN_Msk = 0x40
	// Bit EP3_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP3_IN = 0x40
	// Position of EP2_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP2_OUT_Pos = 0x5
	// Bit mask of EP2_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP2_OUT_Msk = 0x20
	// Bit EP2_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP2_OUT = 0x20
	// Position of EP2_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP2_IN_Pos = 0x4
	// Bit mask of EP2_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP2_IN_Msk = 0x10
	// Bit EP2_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP2_IN = 0x10
	// Position of EP1_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP1_OUT_Pos = 0x3
	// Bit mask of EP1_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP1_OUT_Msk = 0x8
	// Bit EP1_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP1_OUT = 0x8
	// Position of EP1_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP1_IN_Pos = 0x2
	// Bit mask of EP1_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP1_IN_Msk = 0x4
	// Bit EP1_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP1_IN = 0x4
	// Position of EP0_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP0_OUT_Pos = 0x1
	// Bit mask of EP0_OUT field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP0_OUT_Msk = 0x2
	// Bit EP0_OUT.
	USB_BUFF_CPU_SHOULD_HANDLE_EP0_OUT = 0x2
	// Position of EP0_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP0_IN_Pos = 0x0
	// Bit mask of EP0_IN field.
	USB_BUFF_CPU_SHOULD_HANDLE_EP0_IN_Msk = 0x1
	// Bit EP0_IN.
	USB_BUFF_CPU_SHOULD_HANDLE_EP0_IN = 0x1

	// EP_ABORT: Device only: Can be set to ignore the buffer control register for this endpoint in case you would like to revoke a buffer. A NAK will be sent for every access to the endpoint until this bit is cleared. A corresponding bit in `EP_ABORT_DONE` is set when it is safe to modify the buffer control register.
	// Position of EP15_OUT field.
	USB_EP_ABORT_EP15_OUT_Pos = 0x1f
	// Bit mask of EP15_OUT field.
	USB_EP_ABORT_EP15_OUT_Msk = 0x80000000
	// Bit EP15_OUT.
	USB_EP_ABORT_EP15_OUT = 0x80000000
	// Position of EP15_IN field.
	USB_EP_ABORT_EP15_IN_Pos = 0x1e
	// Bit mask of EP15_IN field.
	USB_EP_ABORT_EP15_IN_Msk = 0x40000000
	// Bit EP15_IN.
	USB_EP_ABORT_EP15_IN = 0x40000000
	// Position of EP14_OUT field.
	USB_EP_ABORT_EP14_OUT_Pos = 0x1d
	// Bit mask of EP14_OUT field.
	USB_EP_ABORT_EP14_OUT_Msk = 0x20000000
	// Bit EP14_OUT.
	USB_EP_ABORT_EP14_OUT = 0x20000000
	// Position of EP14_IN field.
	USB_EP_ABORT_EP14_IN_Pos = 0x1c
	// Bit mask of EP14_IN field.
	USB_EP_ABORT_EP14_IN_Msk = 0x10000000
	// Bit EP14_IN.
	USB_EP_ABORT_EP14_IN = 0x10000000
	// Position of EP13_OUT field.
	USB_EP_ABORT_EP13_OUT_Pos = 0x1b
	// Bit mask of EP13_OUT field.
	USB_EP_ABORT_EP13_OUT_Msk = 0x8000000
	// Bit EP13_OUT.
	USB_EP_ABORT_EP13_OUT = 0x8000000
	// Position of EP13_IN field.
	USB_EP_ABORT_EP13_IN_Pos = 0x1a
	// Bit mask of EP13_IN field.
	USB_EP_ABORT_EP13_IN_Msk = 0x4000000
	// Bit EP13_IN.
	USB_EP_ABORT_EP13_IN = 0x4000000
	// Position of EP12_OUT field.
	USB_EP_ABORT_EP12_OUT_Pos = 0x19
	// Bit mask of EP12_OUT field.
	USB_EP_ABORT_EP12_OUT_Msk = 0x2000000
	// Bit EP12_OUT.
	USB_EP_ABORT_EP12_OUT = 0x2000000
	// Position of EP12_IN field.
	USB_EP_ABORT_EP12_IN_Pos = 0x18
	// Bit mask of EP12_IN field.
	USB_EP_ABORT_EP12_IN_Msk = 0x1000000
	// Bit EP12_IN.
	USB_EP_ABORT_EP12_IN = 0x1000000
	// Position of EP11_OUT field.
	USB_EP_ABORT_EP11_OUT_Pos = 0x17
	// Bit mask of EP11_OUT field.
	USB_EP_ABORT_EP11_OUT_Msk = 0x800000
	// Bit EP11_OUT.
	USB_EP_ABORT_EP11_OUT = 0x800000
	// Position of EP11_IN field.
	USB_EP_ABORT_EP11_IN_Pos = 0x16
	// Bit mask of EP11_IN field.
	USB_EP_ABORT_EP11_IN_Msk = 0x400000
	// Bit EP11_IN.
	USB_EP_ABORT_EP11_IN = 0x400000
	// Position of EP10_OUT field.
	USB_EP_ABORT_EP10_OUT_Pos = 0x15
	// Bit mask of EP10_OUT field.
	USB_EP_ABORT_EP10_OUT_Msk = 0x200000
	// Bit EP10_OUT.
	USB_EP_ABORT_EP10_OUT = 0x200000
	// Position of EP10_IN field.
	USB_EP_ABORT_EP10_IN_Pos = 0x14
	// Bit mask of EP10_IN field.
	USB_EP_ABORT_EP10_IN_Msk = 0x100000
	// Bit EP10_IN.
	USB_EP_ABORT_EP10_IN = 0x100000
	// Position of EP9_OUT field.
	USB_EP_ABORT_EP9_OUT_Pos = 0x13
	// Bit mask of EP9_OUT field.
	USB_EP_ABORT_EP9_OUT_Msk = 0x80000
	// Bit EP9_OUT.
	USB_EP_ABORT_EP9_OUT = 0x80000
	// Position of EP9_IN field.
	USB_EP_ABORT_EP9_IN_Pos = 0x12
	// Bit mask of EP9_IN field.
	USB_EP_ABORT_EP9_IN_Msk = 0x40000
	// Bit EP9_IN.
	USB_EP_ABORT_EP9_IN = 0x40000
	// Position of EP8_OUT field.
	USB_EP_ABORT_EP8_OUT_Pos = 0x11
	// Bit mask of EP8_OUT field.
	USB_EP_ABORT_EP8_OUT_Msk = 0x20000
	// Bit EP8_OUT.
	USB_EP_ABORT_EP8_OUT = 0x20000
	// Position of EP8_IN field.
	USB_EP_ABORT_EP8_IN_Pos = 0x10
	// Bit mask of EP8_IN field.
	USB_EP_ABORT_EP8_IN_Msk = 0x10000
	// Bit EP8_IN.
	USB_EP_ABORT_EP8_IN = 0x10000
	// Position of EP7_OUT field.
	USB_EP_ABORT_EP7_OUT_Pos = 0xf
	// Bit mask of EP7_OUT field.
	USB_EP_ABORT_EP7_OUT_Msk = 0x8000
	// Bit EP7_OUT.
	USB_EP_ABORT_EP7_OUT = 0x8000
	// Position of EP7_IN field.
	USB_EP_ABORT_EP7_IN_Pos = 0xe
	// Bit mask of EP7_IN field.
	USB_EP_ABORT_EP7_IN_Msk = 0x4000
	// Bit EP7_IN.
	USB_EP_ABORT_EP7_IN = 0x4000
	// Position of EP6_OUT field.
	USB_EP_ABORT_EP6_OUT_Pos = 0xd
	// Bit mask of EP6_OUT field.
	USB_EP_ABORT_EP6_OUT_Msk = 0x2000
	// Bit EP6_OUT.
	USB_EP_ABORT_EP6_OUT = 0x2000
	// Position of EP6_IN field.
	USB_EP_ABORT_EP6_IN_Pos = 0xc
	// Bit mask of EP6_IN field.
	USB_EP_ABORT_EP6_IN_Msk = 0x1000
	// Bit EP6_IN.
	USB_EP_ABORT_EP6_IN = 0x1000
	// Position of EP5_OUT field.
	USB_EP_ABORT_EP5_OUT_Pos = 0xb
	// Bit mask of EP5_OUT field.
	USB_EP_ABORT_EP5_OUT_Msk = 0x800
	// Bit EP5_OUT.
	USB_EP_ABORT_EP5_OUT = 0x800
	// Position of EP5_IN field.
	USB_EP_ABORT_EP5_IN_Pos = 0xa
	// Bit mask of EP5_IN field.
	USB_EP_ABORT_EP5_IN_Msk = 0x400
	// Bit EP5_IN.
	USB_EP_ABORT_EP5_IN = 0x400
	// Position of EP4_OUT field.
	USB_EP_ABORT_EP4_OUT_Pos = 0x9
	// Bit mask of EP4_OUT field.
	USB_EP_ABORT_EP4_OUT_Msk = 0x200
	// Bit EP4_OUT.
	USB_EP_ABORT_EP4_OUT = 0x200
	// Position of EP4_IN field.
	USB_EP_ABORT_EP4_IN_Pos = 0x8
	// Bit mask of EP4_IN field.
	USB_EP_ABORT_EP4_IN_Msk = 0x100
	// Bit EP4_IN.
	USB_EP_ABORT_EP4_IN = 0x100
	// Position of EP3_OUT field.
	USB_EP_ABORT_EP3_OUT_Pos = 0x7
	// Bit mask of EP3_OUT field.
	USB_EP_ABORT_EP3_OUT_Msk = 0x80
	// Bit EP3_OUT.
	USB_EP_ABORT_EP3_OUT = 0x80
	// Position of EP3_IN field.
	USB_EP_ABORT_EP3_IN_Pos = 0x6
	// Bit mask of EP3_IN field.
	USB_EP_ABORT_EP3_IN_Msk = 0x40
	// Bit EP3_IN.
	USB_EP_ABORT_EP3_IN = 0x40
	// Position of EP2_OUT field.
	USB_EP_ABORT_EP2_OUT_Pos = 0x5
	// Bit mask of EP2_OUT field.
	USB_EP_ABORT_EP2_OUT_Msk = 0x20
	// Bit EP2_OUT.
	USB_EP_ABORT_EP2_OUT = 0x20
	// Position of EP2_IN field.
	USB_EP_ABORT_EP2_IN_Pos = 0x4
	// Bit mask of EP2_IN field.
	USB_EP_ABORT_EP2_IN_Msk = 0x10
	// Bit EP2_IN.
	USB_EP_ABORT_EP2_IN = 0x10
	// Position of EP1_OUT field.
	USB_EP_ABORT_EP1_OUT_Pos = 0x3
	// Bit mask of EP1_OUT field.
	USB_EP_ABORT_EP1_OUT_Msk = 0x8
	// Bit EP1_OUT.
	USB_EP_ABORT_EP1_OUT = 0x8
	// Position of EP1_IN field.
	USB_EP_ABORT_EP1_IN_Pos = 0x2
	// Bit mask of EP1_IN field.
	USB_EP_ABORT_EP1_IN_Msk = 0x4
	// Bit EP1_IN.
	USB_EP_ABORT_EP1_IN = 0x4
	// Position of EP0_OUT field.
	USB_EP_ABORT_EP0_OUT_Pos = 0x1
	// Bit mask of EP0_OUT field.
	USB_EP_ABORT_EP0_OUT_Msk = 0x2
	// Bit EP0_OUT.
	USB_EP_ABORT_EP0_OUT = 0x2
	// Position of EP0_IN field.
	USB_EP_ABORT_EP0_IN_Pos = 0x0
	// Bit mask of EP0_IN field.
	USB_EP_ABORT_EP0_IN_Msk = 0x1
	// Bit EP0_IN.
	USB_EP_ABORT_EP0_IN = 0x1

	// EP_ABORT_DONE: Device only: Used in conjunction with `EP_ABORT`. Set once an endpoint is idle so the programmer knows it is safe to modify the buffer control register.
	// Position of EP15_OUT field.
	USB_EP_ABORT_DONE_EP15_OUT_Pos = 0x1f
	// Bit mask of EP15_OUT field.
	USB_EP_ABORT_DONE_EP15_OUT_Msk = 0x80000000
	// Bit EP15_OUT.
	USB_EP_ABORT_DONE_EP15_OUT = 0x80000000
	// Position of EP15_IN field.
	USB_EP_ABORT_DONE_EP15_IN_Pos = 0x1e
	// Bit mask of EP15_IN field.
	USB_EP_ABORT_DONE_EP15_IN_Msk = 0x40000000
	// Bit EP15_IN.
	USB_EP_ABORT_DONE_EP15_IN = 0x40000000
	// Position of EP14_OUT field.
	USB_EP_ABORT_DONE_EP14_OUT_Pos = 0x1d
	// Bit mask of EP14_OUT field.
	USB_EP_ABORT_DONE_EP14_OUT_Msk = 0x20000000
	// Bit EP14_OUT.
	USB_EP_ABORT_DONE_EP14_OUT = 0x20000000
	// Position of EP14_IN field.
	USB_EP_ABORT_DONE_EP14_IN_Pos = 0x1c
	// Bit mask of EP14_IN field.
	USB_EP_ABORT_DONE_EP14_IN_Msk = 0x10000000
	// Bit EP14_IN.
	USB_EP_ABORT_DONE_EP14_IN = 0x10000000
	// Position of EP13_OUT field.
	USB_EP_ABORT_DONE_EP13_OUT_Pos = 0x1b
	// Bit mask of EP13_OUT field.
	USB_EP_ABORT_DONE_EP13_OUT_Msk = 0x8000000
	// Bit EP13_OUT.
	USB_EP_ABORT_DONE_EP13_OUT = 0x8000000
	// Position of EP13_IN field.
	USB_EP_ABORT_DONE_EP13_IN_Pos = 0x1a
	// Bit mask of EP13_IN field.
	USB_EP_ABORT_DONE_EP13_IN_Msk = 0x4000000
	// Bit EP13_IN.
	USB_EP_ABORT_DONE_EP13_IN = 0x4000000
	// Position of EP12_OUT field.
	USB_EP_ABORT_DONE_EP12_OUT_Pos = 0x19
	// Bit mask of EP12_OUT field.
	USB_EP_ABORT_DONE_EP12_OUT_Msk = 0x2000000
	// Bit EP12_OUT.
	USB_EP_ABORT_DONE_EP12_OUT = 0x2000000
	// Position of EP12_IN field.
	USB_EP_ABORT_DONE_EP12_IN_Pos = 0x18
	// Bit mask of EP12_IN field.
	USB_EP_ABORT_DONE_EP12_IN_Msk = 0x1000000
	// Bit EP12_IN.
	USB_EP_ABORT_DONE_EP12_IN = 0x1000000
	// Position of EP11_OUT field.
	USB_EP_ABORT_DONE_EP11_OUT_Pos = 0x17
	// Bit mask of EP11_OUT field.
	USB_EP_ABORT_DONE_EP11_OUT_Msk = 0x800000
	// Bit EP11_OUT.
	USB_EP_ABORT_DONE_EP11_OUT = 0x800000
	// Position of EP11_IN field.
	USB_EP_ABORT_DONE_EP11_IN_Pos = 0x16
	// Bit mask of EP11_IN field.
	USB_EP_ABORT_DONE_EP11_IN_Msk = 0x400000
	// Bit EP11_IN.
	USB_EP_ABORT_DONE_EP11_IN = 0x400000
	// Position of EP10_OUT field.
	USB_EP_ABORT_DONE_EP10_OUT_Pos = 0x15
	// Bit mask of EP10_OUT field.
	USB_EP_ABORT_DONE_EP10_OUT_Msk = 0x200000
	// Bit EP10_OUT.
	USB_EP_ABORT_DONE_EP10_OUT = 0x200000
	// Position of EP10_IN field.
	USB_EP_ABORT_DONE_EP10_IN_Pos = 0x14
	// Bit mask of EP10_IN field.
	USB_EP_ABORT_DONE_EP10_IN_Msk = 0x100000
	// Bit EP10_IN.
	USB_EP_ABORT_DONE_EP10_IN = 0x100000
	// Position of EP9_OUT field.
	USB_EP_ABORT_DONE_EP9_OUT_Pos = 0x13
	// Bit mask of EP9_OUT field.
	USB_EP_ABORT_DONE_EP9_OUT_Msk = 0x80000
	// Bit EP9_OUT.
	USB_EP_ABORT_DONE_EP9_OUT = 0x80000
	// Position of EP9_IN field.
	USB_EP_ABORT_DONE_EP9_IN_Pos = 0x12
	// Bit mask of EP9_IN field.
	USB_EP_ABORT_DONE_EP9_IN_Msk = 0x40000
	// Bit EP9_IN.
	USB_EP_ABORT_DONE_EP9_IN = 0x40000
	// Position of EP8_OUT field.
	USB_EP_ABORT_DONE_EP8_OUT_Pos = 0x11
	// Bit mask of EP8_OUT field.
	USB_EP_ABORT_DONE_EP8_OUT_Msk = 0x20000
	// Bit EP8_OUT.
	USB_EP_ABORT_DONE_EP8_OUT = 0x20000
	// Position of EP8_IN field.
	USB_EP_ABORT_DONE_EP8_IN_Pos = 0x10
	// Bit mask of EP8_IN field.
	USB_EP_ABORT_DONE_EP8_IN_Msk = 0x10000
	// Bit EP8_IN.
	USB_EP_ABORT_DONE_EP8_IN = 0x10000
	// Position of EP7_OUT field.
	USB_EP_ABORT_DONE_EP7_OUT_Pos = 0xf
	// Bit mask of EP7_OUT field.
	USB_EP_ABORT_DONE_EP7_OUT_Msk = 0x8000
	// Bit EP7_OUT.
	USB_EP_ABORT_DONE_EP7_OUT = 0x8000
	// Position of EP7_IN field.
	USB_EP_ABORT_DONE_EP7_IN_Pos = 0xe
	// Bit mask of EP7_IN field.
	USB_EP_ABORT_DONE_EP7_IN_Msk = 0x4000
	// Bit EP7_IN.
	USB_EP_ABORT_DONE_EP7_IN = 0x4000
	// Position of EP6_OUT field.
	USB_EP_ABORT_DONE_EP6_OUT_Pos = 0xd
	// Bit mask of EP6_OUT field.
	USB_EP_ABORT_DONE_EP6_OUT_Msk = 0x2000
	// Bit EP6_OUT.
	USB_EP_ABORT_DONE_EP6_OUT = 0x2000
	// Position of EP6_IN field.
	USB_EP_ABORT_DONE_EP6_IN_Pos = 0xc
	// Bit mask of EP6_IN field.
	USB_EP_ABORT_DONE_EP6_IN_Msk = 0x1000
	// Bit EP6_IN.
	USB_EP_ABORT_DONE_EP6_IN = 0x1000
	// Position of EP5_OUT field.
	USB_EP_ABORT_DONE_EP5_OUT_Pos = 0xb
	// Bit mask of EP5_OUT field.
	USB_EP_ABORT_DONE_EP5_OUT_Msk = 0x800
	// Bit EP5_OUT.
	USB_EP_ABORT_DONE_EP5_OUT = 0x800
	// Position of EP5_IN field.
	USB_EP_ABORT_DONE_EP5_IN_Pos = 0xa
	// Bit mask of EP5_IN field.
	USB_EP_ABORT_DONE_EP5_IN_Msk = 0x400
	// Bit EP5_IN.
	USB_EP_ABORT_DONE_EP5_IN = 0x400
	// Position of EP4_OUT field.
	USB_EP_ABORT_DONE_EP4_OUT_Pos = 0x9
	// Bit mask of EP4_OUT field.
	USB_EP_ABORT_DONE_EP4_OUT_Msk = 0x200
	// Bit EP4_OUT.
	USB_EP_ABORT_DONE_EP4_OUT = 0x200
	// Position of EP4_IN field.
	USB_EP_ABORT_DONE_EP4_IN_Pos = 0x8
	// Bit mask of EP4_IN field.
	USB_EP_ABORT_DONE_EP4_IN_Msk = 0x100
	// Bit EP4_IN.
	USB_EP_ABORT_DONE_EP4_IN = 0x100
	// Position of EP3_OUT field.
	USB_EP_ABORT_DONE_EP3_OUT_Pos = 0x7
	// Bit mask of EP3_OUT field.
	USB_EP_ABORT_DONE_EP3_OUT_Msk = 0x80
	// Bit EP3_OUT.
	USB_EP_ABORT_DONE_EP3_OUT = 0x80
	// Position of EP3_IN field.
	USB_EP_ABORT_DONE_EP3_IN_Pos = 0x6
	// Bit mask of EP3_IN field.
	USB_EP_ABORT_DONE_EP3_IN_Msk = 0x40
	// Bit EP3_IN.
	USB_EP_ABORT_DONE_EP3_IN = 0x40
	// Position of EP2_OUT field.
	USB_EP_ABORT_DONE_EP2_OUT_Pos = 0x5
	// Bit mask of EP2_OUT field.
	USB_EP_ABORT_DONE_EP2_OUT_Msk = 0x20
	// Bit EP2_OUT.
	USB_EP_ABORT_DONE_EP2_OUT = 0x20
	// Position of EP2_IN field.
	USB_EP_ABORT_DONE_EP2_IN_Pos = 0x4
	// Bit mask of EP2_IN field.
	USB_EP_ABORT_DONE_EP2_IN_Msk = 0x10
	// Bit EP2_IN.
	USB_EP_ABORT_DONE_EP2_IN = 0x10
	// Position of EP1_OUT field.
	USB_EP_ABORT_DONE_EP1_OUT_Pos = 0x3
	// Bit mask of EP1_OUT field.
	USB_EP_ABORT_DONE_EP1_OUT_Msk = 0x8
	// Bit EP1_OUT.
	USB_EP_ABORT_DONE_EP1_OUT = 0x8
	// Position of EP1_IN field.
	USB_EP_ABORT_DONE_EP1_IN_Pos = 0x2
	// Bit mask of EP1_IN field.
	USB_EP_ABORT_DONE_EP1_IN_Msk = 0x4
	// Bit EP1_IN.
	USB_EP_ABORT_DONE_EP1_IN = 0x4
	// Position of EP0_OUT field.
	USB_EP_ABORT_DONE_EP0_OUT_Pos = 0x1
	// Bit mask of EP0_OUT field.
	USB_EP_ABORT_DONE_EP0_OUT_Msk = 0x2
	// Bit EP0_OUT.
	USB_EP_ABORT_DONE_EP0_OUT = 0x2
	// Position of EP0_IN field.
	USB_EP_ABORT_DONE_EP0_IN_Pos = 0x0
	// Bit mask of EP0_IN field.
	USB_EP_ABORT_DONE_EP0_IN_Msk = 0x1
	// Bit EP0_IN.
	USB_EP_ABORT_DONE_EP0_IN = 0x1

	// EP_STALL_ARM: Device: this bit must be set in conjunction with the `STALL` bit in the buffer control register to send a STALL on EP0. The device controller clears these bits when a SETUP packet is received because the USB spec requires that a STALL condition is cleared when a SETUP packet is received.
	// Position of EP0_OUT field.
	USB_EP_STALL_ARM_EP0_OUT_Pos = 0x1
	// Bit mask of EP0_OUT field.
	USB_EP_STALL_ARM_EP0_OUT_Msk = 0x2
	// Bit EP0_OUT.
	USB_EP_STALL_ARM_EP0_OUT = 0x2
	// Position of EP0_IN field.
	USB_EP_STALL_ARM_EP0_IN_Pos = 0x0
	// Bit mask of EP0_IN field.
	USB_EP_STALL_ARM_EP0_IN_Msk = 0x1
	// Bit EP0_IN.
	USB_EP_STALL_ARM_EP0_IN = 0x1

	// NAK_POLL: Used by the host controller. Sets the wait time in microseconds before trying again if the device replies with a NAK.
	// Position of RETRY_COUNT_HI field.
	USB_NAK_POLL_RETRY_COUNT_HI_Pos = 0x1c
	// Bit mask of RETRY_COUNT_HI field.
	USB_NAK_POLL_RETRY_COUNT_HI_Msk = 0xf0000000
	// Position of EPX_STOPPED_ON_NAK field.
	USB_NAK_POLL_EPX_STOPPED_ON_NAK_Pos = 0x1b
	// Bit mask of EPX_STOPPED_ON_NAK field.
	USB_NAK_POLL_EPX_STOPPED_ON_NAK_Msk = 0x8000000
	// Bit EPX_STOPPED_ON_NAK.
	USB_NAK_POLL_EPX_STOPPED_ON_NAK = 0x8000000
	// Position of STOP_EPX_ON_NAK field.
	USB_NAK_POLL_STOP_EPX_ON_NAK_Pos = 0x1a
	// Bit mask of STOP_EPX_ON_NAK field.
	USB_NAK_POLL_STOP_EPX_ON_NAK_Msk = 0x4000000
	// Bit STOP_EPX_ON_NAK.
	USB_NAK_POLL_STOP_EPX_ON_NAK = 0x4000000
	// Position of DELAY_FS field.
	USB_NAK_POLL_DELAY_FS_Pos = 0x10
	// Bit mask of DELAY_FS field.
	USB_NAK_POLL_DELAY_FS_Msk = 0x3ff0000
	// Position of RETRY_COUNT_LO field.
	USB_NAK_POLL_RETRY_COUNT_LO_Pos = 0xa
	// Bit mask of RETRY_COUNT_LO field.
	USB_NAK_POLL_RETRY_COUNT_LO_Msk = 0xfc00
	// Position of DELAY_LS field.
	USB_NAK_POLL_DELAY_LS_Pos = 0x0
	// Bit mask of DELAY_LS field.
	USB_NAK_POLL_DELAY_LS_Msk = 0x3ff

	// EP_STATUS_STALL_NAK: Device: bits are set when the `IRQ_ON_NAK` or `IRQ_ON_STALL` bits are set. For EP0 this comes from `SIE_CTRL`. For all other endpoints it comes from the endpoint control register.
	// Position of EP15_OUT field.
	USB_EP_STATUS_STALL_NAK_EP15_OUT_Pos = 0x1f
	// Bit mask of EP15_OUT field.
	USB_EP_STATUS_STALL_NAK_EP15_OUT_Msk = 0x80000000
	// Bit EP15_OUT.
	USB_EP_STATUS_STALL_NAK_EP15_OUT = 0x80000000
	// Position of EP15_IN field.
	USB_EP_STATUS_STALL_NAK_EP15_IN_Pos = 0x1e
	// Bit mask of EP15_IN field.
	USB_EP_STATUS_STALL_NAK_EP15_IN_Msk = 0x40000000
	// Bit EP15_IN.
	USB_EP_STATUS_STALL_NAK_EP15_IN = 0x40000000
	// Position of EP14_OUT field.
	USB_EP_STATUS_STALL_NAK_EP14_OUT_Pos = 0x1d
	// Bit mask of EP14_OUT field.
	USB_EP_STATUS_STALL_NAK_EP14_OUT_Msk = 0x20000000
	// Bit EP14_OUT.
	USB_EP_STATUS_STALL_NAK_EP14_OUT = 0x20000000
	// Position of EP14_IN field.
	USB_EP_STATUS_STALL_NAK_EP14_IN_Pos = 0x1c
	// Bit mask of EP14_IN field.
	USB_EP_STATUS_STALL_NAK_EP14_IN_Msk = 0x10000000
	// Bit EP14_IN.
	USB_EP_STATUS_STALL_NAK_EP14_IN = 0x10000000
	// Position of EP13_OUT field.
	USB_EP_STATUS_STALL_NAK_EP13_OUT_Pos = 0x1b
	// Bit mask of EP13_OUT field.
	USB_EP_STATUS_STALL_NAK_EP13_OUT_Msk = 0x8000000
	// Bit EP13_OUT.
	USB_EP_STATUS_STALL_NAK_EP13_OUT = 0x8000000
	// Position of EP13_IN field.
	USB_EP_STATUS_STALL_NAK_EP13_IN_Pos = 0x1a
	// Bit mask of EP13_IN field.
	USB_EP_STATUS_STALL_NAK_EP13_IN_Msk = 0x4000000
	// Bit EP13_IN.
	USB_EP_STATUS_STALL_NAK_EP13_IN = 0x4000000
	// Position of EP12_OUT field.
	USB_EP_STATUS_STALL_NAK_EP12_OUT_Pos = 0x19
	// Bit mask of EP12_OUT field.
	USB_EP_STATUS_STALL_NAK_EP12_OUT_Msk = 0x2000000
	// Bit EP12_OUT.
	USB_EP_STATUS_STALL_NAK_EP12_OUT = 0x2000000
	// Position of EP12_IN field.
	USB_EP_STATUS_STALL_NAK_EP12_IN_Pos = 0x18
	// Bit mask of EP12_IN field.
	USB_EP_STATUS_STALL_NAK_EP12_IN_Msk = 0x1000000
	// Bit EP12_IN.
	USB_EP_STATUS_STALL_NAK_EP12_IN = 0x1000000
	// Position of EP11_OUT field.
	USB_EP_STATUS_STALL_NAK_EP11_OUT_Pos = 0x17
	// Bit mask of EP11_OUT field.
	USB_EP_STATUS_STALL_NAK_EP11_OUT_Msk = 0x800000
	// Bit EP11_OUT.
	USB_EP_STATUS_STALL_NAK_EP11_OUT = 0x800000
	// Position of EP11_IN field.
	USB_EP_STATUS_STALL_NAK_EP11_IN_Pos = 0x16
	// Bit mask of EP11_IN field.
	USB_EP_STATUS_STALL_NAK_EP11_IN_Msk = 0x400000
	// Bit EP11_IN.
	USB_EP_STATUS_STALL_NAK_EP11_IN = 0x400000
	// Position of EP10_OUT field.
	USB_EP_STATUS_STALL_NAK_EP10_OUT_Pos = 0x15
	// Bit mask of EP10_OUT field.
	USB_EP_STATUS_STALL_NAK_EP10_OUT_Msk = 0x200000
	// Bit EP10_OUT.
	USB_EP_STATUS_STALL_NAK_EP10_OUT = 0x200000
	// Position of EP10_IN field.
	USB_EP_STATUS_STALL_NAK_EP10_IN_Pos = 0x14
	// Bit mask of EP10_IN field.
	USB_EP_STATUS_STALL_NAK_EP10_IN_Msk = 0x100000
	// Bit EP10_IN.
	USB_EP_STATUS_STALL_NAK_EP10_IN = 0x100000
	// Position of EP9_OUT field.
	USB_EP_STATUS_STALL_NAK_EP9_OUT_Pos = 0x13
	// Bit mask of EP9_OUT field.
	USB_EP_STATUS_STALL_NAK_EP9_OUT_Msk = 0x80000
	// Bit EP9_OUT.
	USB_EP_STATUS_STALL_NAK_EP9_OUT = 0x80000
	// Position of EP9_IN field.
	USB_EP_STATUS_STALL_NAK_EP9_IN_Pos = 0x12
	// Bit mask of EP9_IN field.
	USB_EP_STATUS_STALL_NAK_EP9_IN_Msk = 0x40000
	// Bit EP9_IN.
	USB_EP_STATUS_STALL_NAK_EP9_IN = 0x40000
	// Position of EP8_OUT field.
	USB_EP_STATUS_STALL_NAK_EP8_OUT_Pos = 0x11
	// Bit mask of EP8_OUT field.
	USB_EP_STATUS_STALL_NAK_EP8_OUT_Msk = 0x20000
	// Bit EP8_OUT.
	USB_EP_STATUS_STALL_NAK_EP8_OUT = 0x20000
	// Position of EP8_IN field.
	USB_EP_STATUS_STALL_NAK_EP8_IN_Pos = 0x10
	// Bit mask of EP8_IN field.
	USB_EP_STATUS_STALL_NAK_EP8_IN_Msk = 0x10000
	// Bit EP8_IN.
	USB_EP_STATUS_STALL_NAK_EP8_IN = 0x10000
	// Position of EP7_OUT field.
	USB_EP_STATUS_STALL_NAK_EP7_OUT_Pos = 0xf
	// Bit mask of EP7_OUT field.
	USB_EP_STATUS_STALL_NAK_EP7_OUT_Msk = 0x8000
	// Bit EP7_OUT.
	USB_EP_STATUS_STALL_NAK_EP7_OUT = 0x8000
	// Position of EP7_IN field.
	USB_EP_STATUS_STALL_NAK_EP7_IN_Pos = 0xe
	// Bit mask of EP7_IN field.
	USB_EP_STATUS_STALL_NAK_EP7_IN_Msk = 0x4000
	// Bit EP7_IN.
	USB_EP_STATUS_STALL_NAK_EP7_IN = 0x4000
	// Position of EP6_OUT field.
	USB_EP_STATUS_STALL_NAK_EP6_OUT_Pos = 0xd
	// Bit mask of EP6_OUT field.
	USB_EP_STATUS_STALL_NAK_EP6_OUT_Msk = 0x2000
	// Bit EP6_OUT.
	USB_EP_STATUS_STALL_NAK_EP6_OUT = 0x2000
	// Position of EP6_IN field.
	USB_EP_STATUS_STALL_NAK_EP6_IN_Pos = 0xc
	// Bit mask of EP6_IN field.
	USB_EP_STATUS_STALL_NAK_EP6_IN_Msk = 0x1000
	// Bit EP6_IN.
	USB_EP_STATUS_STALL_NAK_EP6_IN = 0x1000
	// Position of EP5_OUT field.
	USB_EP_STATUS_STALL_NAK_EP5_OUT_Pos = 0xb
	// Bit mask of EP5_OUT field.
	USB_EP_STATUS_STALL_NAK_EP5_OUT_Msk = 0x800
	// Bit EP5_OUT.
	USB_EP_STATUS_STALL_NAK_EP5_OUT = 0x800
	// Position of EP5_IN field.
	USB_EP_STATUS_STALL_NAK_EP5_IN_Pos = 0xa
	// Bit mask of EP5_IN field.
	USB_EP_STATUS_STALL_NAK_EP5_IN_Msk = 0x400
	// Bit EP5_IN.
	USB_EP_STATUS_STALL_NAK_EP5_IN = 0x400
	// Position of EP4_OUT field.
	USB_EP_STATUS_STALL_NAK_EP4_OUT_Pos = 0x9
	// Bit mask of EP4_OUT field.
	USB_EP_STATUS_STALL_NAK_EP4_OUT_Msk = 0x200
	// Bit EP4_OUT.
	USB_EP_STATUS_STALL_NAK_EP4_OUT = 0x200
	// Position of EP4_IN field.
	USB_EP_STATUS_STALL_NAK_EP4_IN_Pos = 0x8
	// Bit mask of EP4_IN field.
	USB_EP_STATUS_STALL_NAK_EP4_IN_Msk = 0x100
	// Bit EP4_IN.
	USB_EP_STATUS_STALL_NAK_EP4_IN = 0x100
	// Position of EP3_OUT field.
	USB_EP_STATUS_STALL_NAK_EP3_OUT_Pos = 0x7
	// Bit mask of EP3_OUT field.
	USB_EP_STATUS_STALL_NAK_EP3_OUT_Msk = 0x80
	// Bit EP3_OUT.
	USB_EP_STATUS_STALL_NAK_EP3_OUT = 0x80
	// Position of EP3_IN field.
	USB_EP_STATUS_STALL_NAK_EP3_IN_Pos = 0x6
	// Bit mask of EP3_IN field.
	USB_EP_STATUS_STALL_NAK_EP3_IN_Msk = 0x40
	// Bit EP3_IN.
	USB_EP_STATUS_STALL_NAK_EP3_IN = 0x40
	// Position of EP2_OUT field.
	USB_EP_STATUS_STALL_NAK_EP2_OUT_Pos = 0x5
	// Bit mask of EP2_OUT field.
	USB_EP_STATUS_STALL_NAK_EP2_OUT_Msk = 0x20
	// Bit EP2_OUT.
	USB_EP_STATUS_STALL_NAK_EP2_OUT = 0x20
	// Position of EP2_IN field.
	USB_EP_STATUS_STALL_NAK_EP2_IN_Pos = 0x4
	// Bit mask of EP2_IN field.
	USB_EP_STATUS_STALL_NAK_EP2_IN_Msk = 0x10
	// Bit EP2_IN.
	USB_EP_STATUS_STALL_NAK_EP2_IN = 0x10
	// Position of EP1_OUT field.
	USB_EP_STATUS_STALL_NAK_EP1_OUT_Pos = 0x3
	// Bit mask of EP1_OUT field.
	USB_EP_STATUS_STALL_NAK_EP1_OUT_Msk = 0x8
	// Bit EP1_OUT.
	USB_EP_STATUS_STALL_NAK_EP1_OUT = 0x8
	// Position of EP1_IN field.
	USB_EP_STATUS_STALL_NAK_EP1_IN_Pos = 0x2
	// Bit mask of EP1_IN field.
	USB_EP_STATUS_STALL_NAK_EP1_IN_Msk = 0x4
	// Bit EP1_IN.
	USB_EP_STATUS_STALL_NAK_EP1_IN = 0x4
	// Position of EP0_OUT field.
	USB_EP_STATUS_STALL_NAK_EP0_OUT_Pos = 0x1
	// Bit mask of EP0_OUT field.
	USB_EP_STATUS_STALL_NAK_EP0_OUT_Msk = 0x2
	// Bit EP0_OUT.
	USB_EP_STATUS_STALL_NAK_EP0_OUT = 0x2
	// Position of EP0_IN field.
	USB_EP_STATUS_STALL_NAK_EP0_IN_Pos = 0x0
	// Bit mask of EP0_IN field.
	USB_EP_STATUS_STALL_NAK_EP0_IN_Msk = 0x1
	// Bit EP0_IN.
	USB_EP_STATUS_STALL_NAK_EP0_IN = 0x1

	// USB_MUXING: Where to connect the USB controller. Should be to_phy by default.
	// Position of SWAP_DPDM field.
	USB_USB_MUXING_SWAP_DPDM_Pos = 0x1f
	// Bit mask of SWAP_DPDM field.
	USB_USB_MUXING_SWAP_DPDM_Msk = 0x80000000
	// Bit SWAP_DPDM.
	USB_USB_MUXING_SWAP_DPDM = 0x80000000
	// Position of USBPHY_AS_GPIO field.
	USB_USB_MUXING_USBPHY_AS_GPIO_Pos = 0x4
	// Bit mask of USBPHY_AS_GPIO field.
	USB_USB_MUXING_USBPHY_AS_GPIO_Msk = 0x10
	// Bit USBPHY_AS_GPIO.
	USB_USB_MUXING_USBPHY_AS_GPIO = 0x10
	// Position of SOFTCON field.
	USB_USB_MUXING_SOFTCON_Pos = 0x3
	// Bit mask of SOFTCON field.
	USB_USB_MUXING_SOFTCON_Msk = 0x8
	// Bit SOFTCON.
	USB_USB_MUXING_SOFTCON = 0x8
	// Position of TO_DIGITAL_PAD field.
	USB_USB_MUXING_TO_DIGITAL_PAD_Pos = 0x2
	// Bit mask of TO_DIGITAL_PAD field.
	USB_USB_MUXING_TO_DIGITAL_PAD_Msk = 0x4
	// Bit TO_DIGITAL_PAD.
	USB_USB_MUXING_TO_DIGITAL_PAD = 0x4
	// Position of TO_EXTPHY field.
	USB_USB_MUXING_TO_EXTPHY_Pos = 0x1
	// Bit mask of TO_EXTPHY field.
	USB_USB_MUXING_TO_EXTPHY_Msk = 0x2
	// Bit TO_EXTPHY.
	USB_USB_MUXING_TO_EXTPHY = 0x2
	// Position of TO_PHY field.
	USB_USB_MUXING_TO_PHY_Pos = 0x0
	// Bit mask of TO_PHY field.
	USB_USB_MUXING_TO_PHY_Msk = 0x1
	// Bit TO_PHY.
	USB_USB_MUXING_TO_PHY = 0x1

	// USB_PWR: Overrides for the power signals in the event that the VBUS signals are not hooked up to GPIO. Set the value of the override and then the override enable to switch over to the override value.
	// Position of OVERCURR_DETECT_EN field.
	USB_USB_PWR_OVERCURR_DETECT_EN_Pos = 0x5
	// Bit mask of OVERCURR_DETECT_EN field.
	USB_USB_PWR_OVERCURR_DETECT_EN_Msk = 0x20
	// Bit OVERCURR_DETECT_EN.
	USB_USB_PWR_OVERCURR_DETECT_EN = 0x20
	// Position of OVERCURR_DETECT field.
	USB_USB_PWR_OVERCURR_DETECT_Pos = 0x4
	// Bit mask of OVERCURR_DETECT field.
	USB_USB_PWR_OVERCURR_DETECT_Msk = 0x10
	// Bit OVERCURR_DETECT.
	USB_USB_PWR_OVERCURR_DETECT = 0x10
	// Position of VBUS_DETECT_OVERRIDE_EN field.
	USB_USB_PWR_VBUS_DETECT_OVERRIDE_EN_Pos = 0x3
	// Bit mask of VBUS_DETECT_OVERRIDE_EN field.
	USB_USB_PWR_VBUS_DETECT_OVERRIDE_EN_Msk = 0x8
	// Bit VBUS_DETECT_OVERRIDE_EN.
	USB_USB_PWR_VBUS_DETECT_OVERRIDE_EN = 0x8
	// Position of VBUS_DETECT field.
	USB_USB_PWR_VBUS_DETECT_Pos = 0x2
	// Bit mask of VBUS_DETECT field.
	USB_USB_PWR_VBUS_DETECT_Msk = 0x4
	// Bit VBUS_DETECT.
	USB_USB_PWR_VBUS_DETECT = 0x4
	// Position of VBUS_EN_OVERRIDE_EN field.
	USB_USB_PWR_VBUS_EN_OVERRIDE_EN_Pos = 0x1
	// Bit mask of VBUS_EN_OVERRIDE_EN field.
	USB_USB_PWR_VBUS_EN_OVERRIDE_EN_Msk = 0x2
	// Bit VBUS_EN_OVERRIDE_EN.
	USB_USB_PWR_VBUS_EN_OVERRIDE_EN = 0x2
	// Position of VBUS_EN field.
	USB_USB_PWR_VBUS_EN_Pos = 0x0
	// Bit mask of VBUS_EN field.
	USB_USB_PWR_VBUS_EN_Msk = 0x1
	// Bit VBUS_EN.
	USB_USB_PWR_VBUS_EN = 0x1

	// USBPHY_DIRECT: This register allows for direct control of the USB phy. Use in conjunction with usbphy_direct_override register to enable each override bit.
	// Position of RX_DM_OVERRIDE field.
	USB_USBPHY_DIRECT_RX_DM_OVERRIDE_Pos = 0x19
	// Bit mask of RX_DM_OVERRIDE field.
	USB_USBPHY_DIRECT_RX_DM_OVERRIDE_Msk = 0x2000000
	// Bit RX_DM_OVERRIDE.
	USB_USBPHY_DIRECT_RX_DM_OVERRIDE = 0x2000000
	// Position of RX_DP_OVERRIDE field.
	USB_USBPHY_DIRECT_RX_DP_OVERRIDE_Pos = 0x18
	// Bit mask of RX_DP_OVERRIDE field.
	USB_USBPHY_DIRECT_RX_DP_OVERRIDE_Msk = 0x1000000
	// Bit RX_DP_OVERRIDE.
	USB_USBPHY_DIRECT_RX_DP_OVERRIDE = 0x1000000
	// Position of RX_DD_OVERRIDE field.
	USB_USBPHY_DIRECT_RX_DD_OVERRIDE_Pos = 0x17
	// Bit mask of RX_DD_OVERRIDE field.
	USB_USBPHY_DIRECT_RX_DD_OVERRIDE_Msk = 0x800000
	// Bit RX_DD_OVERRIDE.
	USB_USBPHY_DIRECT_RX_DD_OVERRIDE = 0x800000
	// Position of DM_OVV field.
	USB_USBPHY_DIRECT_DM_OVV_Pos = 0x16
	// Bit mask of DM_OVV field.
	USB_USBPHY_DIRECT_DM_OVV_Msk = 0x400000
	// Bit DM_OVV.
	USB_USBPHY_DIRECT_DM_OVV = 0x400000
	// Position of DP_OVV field.
	USB_USBPHY_DIRECT_DP_OVV_Pos = 0x15
	// Bit mask of DP_OVV field.
	USB_USBPHY_DIRECT_DP_OVV_Msk = 0x200000
	// Bit DP_OVV.
	USB_USBPHY_DIRECT_DP_OVV = 0x200000
	// Position of DM_OVCN field.
	USB_USBPHY_DIRECT_DM_OVCN_Pos = 0x14
	// Bit mask of DM_OVCN field.
	USB_USBPHY_DIRECT_DM_OVCN_Msk = 0x100000
	// Bit DM_OVCN.
	USB_USBPHY_DIRECT_DM_OVCN = 0x100000
	// Position of DP_OVCN field.
	USB_USBPHY_DIRECT_DP_OVCN_Pos = 0x13
	// Bit mask of DP_OVCN field.
	USB_USBPHY_DIRECT_DP_OVCN_Msk = 0x80000
	// Bit DP_OVCN.
	USB_USBPHY_DIRECT_DP_OVCN = 0x80000
	// Position of RX_DM field.
	USB_USBPHY_DIRECT_RX_DM_Pos = 0x12
	// Bit mask of RX_DM field.
	USB_USBPHY_DIRECT_RX_DM_Msk = 0x40000
	// Bit RX_DM.
	USB_USBPHY_DIRECT_RX_DM = 0x40000
	// Position of RX_DP field.
	USB_USBPHY_DIRECT_RX_DP_Pos = 0x11
	// Bit mask of RX_DP field.
	USB_USBPHY_DIRECT_RX_DP_Msk = 0x20000
	// Bit RX_DP.
	USB_USBPHY_DIRECT_RX_DP = 0x20000
	// Position of RX_DD field.
	USB_USBPHY_DIRECT_RX_DD_Pos = 0x10
	// Bit mask of RX_DD field.
	USB_USBPHY_DIRECT_RX_DD_Msk = 0x10000
	// Bit RX_DD.
	USB_USBPHY_DIRECT_RX_DD = 0x10000
	// Position of TX_DIFFMODE field.
	USB_USBPHY_DIRECT_TX_DIFFMODE_Pos = 0xf
	// Bit mask of TX_DIFFMODE field.
	USB_USBPHY_DIRECT_TX_DIFFMODE_Msk = 0x8000
	// Bit TX_DIFFMODE.
	USB_USBPHY_DIRECT_TX_DIFFMODE = 0x8000
	// Position of TX_FSSLEW field.
	USB_USBPHY_DIRECT_TX_FSSLEW_Pos = 0xe
	// Bit mask of TX_FSSLEW field.
	USB_USBPHY_DIRECT_TX_FSSLEW_Msk = 0x4000
	// Bit TX_FSSLEW.
	USB_USBPHY_DIRECT_TX_FSSLEW = 0x4000
	// Position of TX_PD field.
	USB_USBPHY_DIRECT_TX_PD_Pos = 0xd
	// Bit mask of TX_PD field.
	USB_USBPHY_DIRECT_TX_PD_Msk = 0x2000
	// Bit TX_PD.
	USB_USBPHY_DIRECT_TX_PD = 0x2000
	// Position of RX_PD field.
	USB_USBPHY_DIRECT_RX_PD_Pos = 0xc
	// Bit mask of RX_PD field.
	USB_USBPHY_DIRECT_RX_PD_Msk = 0x1000
	// Bit RX_PD.
	USB_USBPHY_DIRECT_RX_PD = 0x1000
	// Position of TX_DM field.
	USB_USBPHY_DIRECT_TX_DM_Pos = 0xb
	// Bit mask of TX_DM field.
	USB_USBPHY_DIRECT_TX_DM_Msk = 0x800
	// Bit TX_DM.
	USB_USBPHY_DIRECT_TX_DM = 0x800
	// Position of TX_DP field.
	USB_USBPHY_DIRECT_TX_DP_Pos = 0xa
	// Bit mask of TX_DP field.
	USB_USBPHY_DIRECT_TX_DP_Msk = 0x400
	// Bit TX_DP.
	USB_USBPHY_DIRECT_TX_DP = 0x400
	// Position of TX_DM_OE field.
	USB_USBPHY_DIRECT_TX_DM_OE_Pos = 0x9
	// Bit mask of TX_DM_OE field.
	USB_USBPHY_DIRECT_TX_DM_OE_Msk = 0x200
	// Bit TX_DM_OE.
	USB_USBPHY_DIRECT_TX_DM_OE = 0x200
	// Position of TX_DP_OE field.
	USB_USBPHY_DIRECT_TX_DP_OE_Pos = 0x8
	// Bit mask of TX_DP_OE field.
	USB_USBPHY_DIRECT_TX_DP_OE_Msk = 0x100
	// Bit TX_DP_OE.
	USB_USBPHY_DIRECT_TX_DP_OE = 0x100
	// Position of DM_PULLDN_EN field.
	USB_USBPHY_DIRECT_DM_PULLDN_EN_Pos = 0x6
	// Bit mask of DM_PULLDN_EN field.
	USB_USBPHY_DIRECT_DM_PULLDN_EN_Msk = 0x40
	// Bit DM_PULLDN_EN.
	USB_USBPHY_DIRECT_DM_PULLDN_EN = 0x40
	// Position of DM_PULLUP_EN field.
	USB_USBPHY_DIRECT_DM_PULLUP_EN_Pos = 0x5
	// Bit mask of DM_PULLUP_EN field.
	USB_USBPHY_DIRECT_DM_PULLUP_EN_Msk = 0x20
	// Bit DM_PULLUP_EN.
	USB_USBPHY_DIRECT_DM_PULLUP_EN = 0x20
	// Position of DM_PULLUP_HISEL field.
	USB_USBPHY_DIRECT_DM_PULLUP_HISEL_Pos = 0x4
	// Bit mask of DM_PULLUP_HISEL field.
	USB_USBPHY_DIRECT_DM_PULLUP_HISEL_Msk = 0x10
	// Bit DM_PULLUP_HISEL.
	USB_USBPHY_DIRECT_DM_PULLUP_HISEL = 0x10
	// Position of DP_PULLDN_EN field.
	USB_USBPHY_DIRECT_DP_PULLDN_EN_Pos = 0x2
	// Bit mask of DP_PULLDN_EN field.
	USB_USBPHY_DIRECT_DP_PULLDN_EN_Msk = 0x4
	// Bit DP_PULLDN_EN.
	USB_USBPHY_DIRECT_DP_PULLDN_EN = 0x4
	// Position of DP_PULLUP_EN field.
	USB_USBPHY_DIRECT_DP_PULLUP_EN_Pos = 0x1
	// Bit mask of DP_PULLUP_EN field.
	USB_USBPHY_DIRECT_DP_PULLUP_EN_Msk = 0x2
	// Bit DP_PULLUP_EN.
	USB_USBPHY_DIRECT_DP_PULLUP_EN = 0x2
	// Position of DP_PULLUP_HISEL field.
	USB_USBPHY_DIRECT_DP_PULLUP_HISEL_Pos = 0x0
	// Bit mask of DP_PULLUP_HISEL field.
	USB_USBPHY_DIRECT_DP_PULLUP_HISEL_Msk = 0x1
	// Bit DP_PULLUP_HISEL.
	USB_USBPHY_DIRECT_DP_PULLUP_HISEL = 0x1

	// USBPHY_DIRECT_OVERRIDE: Override enable for each control in usbphy_direct
	// Position of RX_DM_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_RX_DM_OVERRIDE_EN_Pos = 0x12
	// Bit mask of RX_DM_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_RX_DM_OVERRIDE_EN_Msk = 0x40000
	// Bit RX_DM_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_RX_DM_OVERRIDE_EN = 0x40000
	// Position of RX_DP_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_RX_DP_OVERRIDE_EN_Pos = 0x11
	// Bit mask of RX_DP_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_RX_DP_OVERRIDE_EN_Msk = 0x20000
	// Bit RX_DP_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_RX_DP_OVERRIDE_EN = 0x20000
	// Position of RX_DD_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_RX_DD_OVERRIDE_EN_Pos = 0x10
	// Bit mask of RX_DD_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_RX_DD_OVERRIDE_EN_Msk = 0x10000
	// Bit RX_DD_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_RX_DD_OVERRIDE_EN = 0x10000
	// Position of TX_DIFFMODE_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DIFFMODE_OVERRIDE_EN_Pos = 0xf
	// Bit mask of TX_DIFFMODE_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DIFFMODE_OVERRIDE_EN_Msk = 0x8000
	// Bit TX_DIFFMODE_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DIFFMODE_OVERRIDE_EN = 0x8000
	// Position of DM_PULLUP_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DM_PULLUP_OVERRIDE_EN_Pos = 0xc
	// Bit mask of DM_PULLUP_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DM_PULLUP_OVERRIDE_EN_Msk = 0x1000
	// Bit DM_PULLUP_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_DM_PULLUP_OVERRIDE_EN = 0x1000
	// Position of TX_FSSLEW_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_FSSLEW_OVERRIDE_EN_Pos = 0xb
	// Bit mask of TX_FSSLEW_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_FSSLEW_OVERRIDE_EN_Msk = 0x800
	// Bit TX_FSSLEW_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_TX_FSSLEW_OVERRIDE_EN = 0x800
	// Position of TX_PD_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_PD_OVERRIDE_EN_Pos = 0xa
	// Bit mask of TX_PD_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_PD_OVERRIDE_EN_Msk = 0x400
	// Bit TX_PD_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_TX_PD_OVERRIDE_EN = 0x400
	// Position of RX_PD_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_RX_PD_OVERRIDE_EN_Pos = 0x9
	// Bit mask of RX_PD_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_RX_PD_OVERRIDE_EN_Msk = 0x200
	// Bit RX_PD_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_RX_PD_OVERRIDE_EN = 0x200
	// Position of TX_DM_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DM_OVERRIDE_EN_Pos = 0x8
	// Bit mask of TX_DM_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DM_OVERRIDE_EN_Msk = 0x100
	// Bit TX_DM_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DM_OVERRIDE_EN = 0x100
	// Position of TX_DP_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DP_OVERRIDE_EN_Pos = 0x7
	// Bit mask of TX_DP_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DP_OVERRIDE_EN_Msk = 0x80
	// Bit TX_DP_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DP_OVERRIDE_EN = 0x80
	// Position of TX_DM_OE_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DM_OE_OVERRIDE_EN_Pos = 0x6
	// Bit mask of TX_DM_OE_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DM_OE_OVERRIDE_EN_Msk = 0x40
	// Bit TX_DM_OE_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DM_OE_OVERRIDE_EN = 0x40
	// Position of TX_DP_OE_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DP_OE_OVERRIDE_EN_Pos = 0x5
	// Bit mask of TX_DP_OE_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DP_OE_OVERRIDE_EN_Msk = 0x20
	// Bit TX_DP_OE_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_TX_DP_OE_OVERRIDE_EN = 0x20
	// Position of DM_PULLDN_EN_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DM_PULLDN_EN_OVERRIDE_EN_Pos = 0x4
	// Bit mask of DM_PULLDN_EN_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DM_PULLDN_EN_OVERRIDE_EN_Msk = 0x10
	// Bit DM_PULLDN_EN_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_DM_PULLDN_EN_OVERRIDE_EN = 0x10
	// Position of DP_PULLDN_EN_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DP_PULLDN_EN_OVERRIDE_EN_Pos = 0x3
	// Bit mask of DP_PULLDN_EN_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DP_PULLDN_EN_OVERRIDE_EN_Msk = 0x8
	// Bit DP_PULLDN_EN_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_DP_PULLDN_EN_OVERRIDE_EN = 0x8
	// Position of DP_PULLUP_EN_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DP_PULLUP_EN_OVERRIDE_EN_Pos = 0x2
	// Bit mask of DP_PULLUP_EN_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DP_PULLUP_EN_OVERRIDE_EN_Msk = 0x4
	// Bit DP_PULLUP_EN_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_DP_PULLUP_EN_OVERRIDE_EN = 0x4
	// Position of DM_PULLUP_HISEL_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DM_PULLUP_HISEL_OVERRIDE_EN_Pos = 0x1
	// Bit mask of DM_PULLUP_HISEL_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DM_PULLUP_HISEL_OVERRIDE_EN_Msk = 0x2
	// Bit DM_PULLUP_HISEL_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_DM_PULLUP_HISEL_OVERRIDE_EN = 0x2
	// Position of DP_PULLUP_HISEL_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DP_PULLUP_HISEL_OVERRIDE_EN_Pos = 0x0
	// Bit mask of DP_PULLUP_HISEL_OVERRIDE_EN field.
	USB_USBPHY_DIRECT_OVERRIDE_DP_PULLUP_HISEL_OVERRIDE_EN_Msk = 0x1
	// Bit DP_PULLUP_HISEL_OVERRIDE_EN.
	USB_USBPHY_DIRECT_OVERRIDE_DP_PULLUP_HISEL_OVERRIDE_EN = 0x1

	// USBPHY_TRIM: Used to adjust trim values of USB phy pull down resistors.
	// Position of DM_PULLDN_TRIM field.
	USB_USBPHY_TRIM_DM_PULLDN_TRIM_Pos = 0x8
	// Bit mask of DM_PULLDN_TRIM field.
	USB_USBPHY_TRIM_DM_PULLDN_TRIM_Msk = 0x1f00
	// Position of DP_PULLDN_TRIM field.
	USB_USBPHY_TRIM_DP_PULLDN_TRIM_Pos = 0x0
	// Bit mask of DP_PULLDN_TRIM field.
	USB_USBPHY_TRIM_DP_PULLDN_TRIM_Msk = 0x1f

	// LINESTATE_TUNING: Used for debug only.
	// Position of SPARE_FIX field.
	USB_LINESTATE_TUNING_SPARE_FIX_Pos = 0x8
	// Bit mask of SPARE_FIX field.
	USB_LINESTATE_TUNING_SPARE_FIX_Msk = 0xf00
	// Position of DEV_LS_WAKE_FIX field.
	USB_LINESTATE_TUNING_DEV_LS_WAKE_FIX_Pos = 0x7
	// Bit mask of DEV_LS_WAKE_FIX field.
	USB_LINESTATE_TUNING_DEV_LS_WAKE_FIX_Msk = 0x80
	// Bit DEV_LS_WAKE_FIX.
	USB_LINESTATE_TUNING_DEV_LS_WAKE_FIX = 0x80
	// Position of DEV_RX_ERR_QUIESCE field.
	USB_LINESTATE_TUNING_DEV_RX_ERR_QUIESCE_Pos = 0x6
	// Bit mask of DEV_RX_ERR_QUIESCE field.
	USB_LINESTATE_TUNING_DEV_RX_ERR_QUIESCE_Msk = 0x40
	// Bit DEV_RX_ERR_QUIESCE.
	USB_LINESTATE_TUNING_DEV_RX_ERR_QUIESCE = 0x40
	// Position of SIE_RX_CHATTER_SE0_FIX field.
	USB_LINESTATE_TUNING_SIE_RX_CHATTER_SE0_FIX_Pos = 0x5
	// Bit mask of SIE_RX_CHATTER_SE0_FIX field.
	USB_LINESTATE_TUNING_SIE_RX_CHATTER_SE0_FIX_Msk = 0x20
	// Bit SIE_RX_CHATTER_SE0_FIX.
	USB_LINESTATE_TUNING_SIE_RX_CHATTER_SE0_FIX = 0x20
	// Position of SIE_RX_BITSTUFF_FIX field.
	USB_LINESTATE_TUNING_SIE_RX_BITSTUFF_FIX_Pos = 0x4
	// Bit mask of SIE_RX_BITSTUFF_FIX field.
	USB_LINESTATE_TUNING_SIE_RX_BITSTUFF_FIX_Msk = 0x10
	// Bit SIE_RX_BITSTUFF_FIX.
	USB_LINESTATE_TUNING_SIE_RX_BITSTUFF_FIX = 0x10
	// Position of DEV_BUFF_CONTROL_DOUBLE_READ_FIX field.
	USB_LINESTATE_TUNING_DEV_BUFF_CONTROL_DOUBLE_READ_FIX_Pos = 0x3
	// Bit mask of DEV_BUFF_CONTROL_DOUBLE_READ_FIX field.
	USB_LINESTATE_TUNING_DEV_BUFF_CONTROL_DOUBLE_READ_FIX_Msk = 0x8
	// Bit DEV_BUFF_CONTROL_DOUBLE_READ_FIX.
	USB_LINESTATE_TUNING_DEV_BUFF_CONTROL_DOUBLE_READ_FIX = 0x8
	// Position of MULTI_HUB_FIX field.
	USB_LINESTATE_TUNING_MULTI_HUB_FIX_Pos = 0x2
	// Bit mask of MULTI_HUB_FIX field.
	USB_LINESTATE_TUNING_MULTI_HUB_FIX_Msk = 0x4
	// Bit MULTI_HUB_FIX.
	USB_LINESTATE_TUNING_MULTI_HUB_FIX = 0x4
	// Position of LINESTATE_DELAY field.
	USB_LINESTATE_TUNING_LINESTATE_DELAY_Pos = 0x1
	// Bit mask of LINESTATE_DELAY field.
	USB_LINESTATE_TUNING_LINESTATE_DELAY_Msk = 0x2
	// Bit LINESTATE_DELAY.
	USB_LINESTATE_TUNING_LINESTATE_DELAY = 0x2
	// Position of RCV_DELAY field.
	USB_LINESTATE_TUNING_RCV_DELAY_Pos = 0x0
	// Bit mask of RCV_DELAY field.
	USB_LINESTATE_TUNING_RCV_DELAY_Msk = 0x1
	// Bit RCV_DELAY.
	USB_LINESTATE_TUNING_RCV_DELAY = 0x1

	// INTR: Raw Interrupts
	// Position of EPX_STOPPED_ON_NAK field.
	USB_INTR_EPX_STOPPED_ON_NAK_Pos = 0x17
	// Bit mask of EPX_STOPPED_ON_NAK field.
	USB_INTR_EPX_STOPPED_ON_NAK_Msk = 0x800000
	// Bit EPX_STOPPED_ON_NAK.
	USB_INTR_EPX_STOPPED_ON_NAK = 0x800000
	// Position of DEV_SM_WATCHDOG_FIRED field.
	USB_INTR_DEV_SM_WATCHDOG_FIRED_Pos = 0x16
	// Bit mask of DEV_SM_WATCHDOG_FIRED field.
	USB_INTR_DEV_SM_WATCHDOG_FIRED_Msk = 0x400000
	// Bit DEV_SM_WATCHDOG_FIRED.
	USB_INTR_DEV_SM_WATCHDOG_FIRED = 0x400000
	// Position of ENDPOINT_ERROR field.
	USB_INTR_ENDPOINT_ERROR_Pos = 0x15
	// Bit mask of ENDPOINT_ERROR field.
	USB_INTR_ENDPOINT_ERROR_Msk = 0x200000
	// Bit ENDPOINT_ERROR.
	USB_INTR_ENDPOINT_ERROR = 0x200000
	// Position of RX_SHORT_PACKET field.
	USB_INTR_RX_SHORT_PACKET_Pos = 0x14
	// Bit mask of RX_SHORT_PACKET field.
	USB_INTR_RX_SHORT_PACKET_Msk = 0x100000
	// Bit RX_SHORT_PACKET.
	USB_INTR_RX_SHORT_PACKET = 0x100000
	// Position of EP_STALL_NAK field.
	USB_INTR_EP_STALL_NAK_Pos = 0x13
	// Bit mask of EP_STALL_NAK field.
	USB_INTR_EP_STALL_NAK_Msk = 0x80000
	// Bit EP_STALL_NAK.
	USB_INTR_EP_STALL_NAK = 0x80000
	// Position of ABORT_DONE field.
	USB_INTR_ABORT_DONE_Pos = 0x12
	// Bit mask of ABORT_DONE field.
	USB_INTR_ABORT_DONE_Msk = 0x40000
	// Bit ABORT_DONE.
	USB_INTR_ABORT_DONE = 0x40000
	// Position of DEV_SOF field.
	USB_INTR_DEV_SOF_Pos = 0x11
	// Bit mask of DEV_SOF field.
	USB_INTR_DEV_SOF_Msk = 0x20000
	// Bit DEV_SOF.
	USB_INTR_DEV_SOF = 0x20000
	// Position of SETUP_REQ field.
	USB_INTR_SETUP_REQ_Pos = 0x10
	// Bit mask of SETUP_REQ field.
	USB_INTR_SETUP_REQ_Msk = 0x10000
	// Bit SETUP_REQ.
	USB_INTR_SETUP_REQ = 0x10000
	// Position of DEV_RESUME_FROM_HOST field.
	USB_INTR_DEV_RESUME_FROM_HOST_Pos = 0xf
	// Bit mask of DEV_RESUME_FROM_HOST field.
	USB_INTR_DEV_RESUME_FROM_HOST_Msk = 0x8000
	// Bit DEV_RESUME_FROM_HOST.
	USB_INTR_DEV_RESUME_FROM_HOST = 0x8000
	// Position of DEV_SUSPEND field.
	USB_INTR_DEV_SUSPEND_Pos = 0xe
	// Bit mask of DEV_SUSPEND field.
	USB_INTR_DEV_SUSPEND_Msk = 0x4000
	// Bit DEV_SUSPEND.
	USB_INTR_DEV_SUSPEND = 0x4000
	// Position of DEV_CONN_DIS field.
	USB_INTR_DEV_CONN_DIS_Pos = 0xd
	// Bit mask of DEV_CONN_DIS field.
	USB_INTR_DEV_CONN_DIS_Msk = 0x2000
	// Bit DEV_CONN_DIS.
	USB_INTR_DEV_CONN_DIS = 0x2000
	// Position of BUS_RESET field.
	USB_INTR_BUS_RESET_Pos = 0xc
	// Bit mask of BUS_RESET field.
	USB_INTR_BUS_RESET_Msk = 0x1000
	// Bit BUS_RESET.
	USB_INTR_BUS_RESET = 0x1000
	// Position of VBUS_DETECT field.
	USB_INTR_VBUS_DETECT_Pos = 0xb
	// Bit mask of VBUS_DETECT field.
	USB_INTR_VBUS_DETECT_Msk = 0x800
	// Bit VBUS_DETECT.
	USB_INTR_VBUS_DETECT = 0x800
	// Position of STALL field.
	USB_INTR_STALL_Pos = 0xa
	// Bit mask of STALL field.
	USB_INTR_STALL_Msk = 0x400
	// Bit STALL.
	USB_INTR_STALL = 0x400
	// Position of ERROR_CRC field.
	USB_INTR_ERROR_CRC_Pos = 0x9
	// Bit mask of ERROR_CRC field.
	USB_INTR_ERROR_CRC_Msk = 0x200
	// Bit ERROR_CRC.
	USB_INTR_ERROR_CRC = 0x200
	// Position of ERROR_BIT_STUFF field.
	USB_INTR_ERROR_BIT_STUFF_Pos = 0x8
	// Bit mask of ERROR_BIT_STUFF field.
	USB_INTR_ERROR_BIT_STUFF_Msk = 0x100
	// Bit ERROR_BIT_STUFF.
	USB_INTR_ERROR_BIT_STUFF = 0x100
	// Position of ERROR_RX_OVERFLOW field.
	USB_INTR_ERROR_RX_OVERFLOW_Pos = 0x7
	// Bit mask of ERROR_RX_OVERFLOW field.
	USB_INTR_ERROR_RX_OVERFLOW_Msk = 0x80
	// Bit ERROR_RX_OVERFLOW.
	USB_INTR_ERROR_RX_OVERFLOW = 0x80
	// Position of ERROR_RX_TIMEOUT field.
	USB_INTR_ERROR_RX_TIMEOUT_Pos = 0x6
	// Bit mask of ERROR_RX_TIMEOUT field.
	USB_INTR_ERROR_RX_TIMEOUT_Msk = 0x40
	// Bit ERROR_RX_TIMEOUT.
	USB_INTR_ERROR_RX_TIMEOUT = 0x40
	// Position of ERROR_DATA_SEQ field.
	USB_INTR_ERROR_DATA_SEQ_Pos = 0x5
	// Bit mask of ERROR_DATA_SEQ field.
	USB_INTR_ERROR_DATA_SEQ_Msk = 0x20
	// Bit ERROR_DATA_SEQ.
	USB_INTR_ERROR_DATA_SEQ = 0x20
	// Position of BUFF_STATUS field.
	USB_INTR_BUFF_STATUS_Pos = 0x4
	// Bit mask of BUFF_STATUS field.
	USB_INTR_BUFF_STATUS_Msk = 0x10
	// Bit BUFF_STATUS.
	USB_INTR_BUFF_STATUS = 0x10
	// Position of TRANS_COMPLETE field.
	USB_INTR_TRANS_COMPLETE_Pos = 0x3
	// Bit mask of TRANS_COMPLETE field.
	USB_INTR_TRANS_COMPLETE_Msk = 0x8
	// Bit TRANS_COMPLETE.
	USB_INTR_TRANS_COMPLETE = 0x8
	// Position of HOST_SOF field.
	USB_INTR_HOST_SOF_Pos = 0x2
	// Bit mask of HOST_SOF field.
	USB_INTR_HOST_SOF_Msk = 0x4
	// Bit HOST_SOF.
	USB_INTR_HOST_SOF = 0x4
	// Position of HOST_RESUME field.
	USB_INTR_HOST_RESUME_Pos = 0x1
	// Bit mask of HOST_RESUME field.
	USB_INTR_HOST_RESUME_Msk = 0x2
	// Bit HOST_RESUME.
	USB_INTR_HOST_RESUME = 0x2
	// Position of HOST_CONN_DIS field.
	USB_INTR_HOST_CONN_DIS_Pos = 0x0
	// Bit mask of HOST_CONN_DIS field.
	USB_INTR_HOST_CONN_DIS_Msk = 0x1
	// Bit HOST_CONN_DIS.
	USB_INTR_HOST_CONN_DIS = 0x1

	// INTE: Interrupt Enable
	// Position of EPX_STOPPED_ON_NAK field.
	USB_INTE_EPX_STOPPED_ON_NAK_Pos = 0x17
	// Bit mask of EPX_STOPPED_ON_NAK field.
	USB_INTE_EPX_STOPPED_ON_NAK_Msk = 0x800000
	// Bit EPX_STOPPED_ON_NAK.
	USB_INTE_EPX_STOPPED_ON_NAK = 0x800000
	// Position of DEV_SM_WATCHDOG_FIRED field.
	USB_INTE_DEV_SM_WATCHDOG_FIRED_Pos = 0x16
	// Bit mask of DEV_SM_WATCHDOG_FIRED field.
	USB_INTE_DEV_SM_WATCHDOG_FIRED_Msk = 0x400000
	// Bit DEV_SM_WATCHDOG_FIRED.
	USB_INTE_DEV_SM_WATCHDOG_FIRED = 0x400000
	// Position of ENDPOINT_ERROR field.
	USB_INTE_ENDPOINT_ERROR_Pos = 0x15
	// Bit mask of ENDPOINT_ERROR field.
	USB_INTE_ENDPOINT_ERROR_Msk = 0x200000
	// Bit ENDPOINT_ERROR.
	USB_INTE_ENDPOINT_ERROR = 0x200000
	// Position of RX_SHORT_PACKET field.
	USB_INTE_RX_SHORT_PACKET_Pos = 0x14
	// Bit mask of RX_SHORT_PACKET field.
	USB_INTE_RX_SHORT_PACKET_Msk = 0x100000
	// Bit RX_SHORT_PACKET.
	USB_INTE_RX_SHORT_PACKET = 0x100000
	// Position of EP_STALL_NAK field.
	USB_INTE_EP_STALL_NAK_Pos = 0x13
	// Bit mask of EP_STALL_NAK field.
	USB_INTE_EP_STALL_NAK_Msk = 0x80000
	// Bit EP_STALL_NAK.
	USB_INTE_EP_STALL_NAK = 0x80000
	// Position of ABORT_DONE field.
	USB_INTE_ABORT_DONE_Pos = 0x12
	// Bit mask of ABORT_DONE field.
	USB_INTE_ABORT_DONE_Msk = 0x40000
	// Bit ABORT_DONE.
	USB_INTE_ABORT_DONE = 0x40000
	// Position of DEV_SOF field.
	USB_INTE_DEV_SOF_Pos = 0x11
	// Bit mask of DEV_SOF field.
	USB_INTE_DEV_SOF_Msk = 0x20000
	// Bit DEV_SOF.
	USB_INTE_DEV_SOF = 0x20000
	// Position of SETUP_REQ field.
	USB_INTE_SETUP_REQ_Pos = 0x10
	// Bit mask of SETUP_REQ field.
	USB_INTE_SETUP_REQ_Msk = 0x10000
	// Bit SETUP_REQ.
	USB_INTE_SETUP_REQ = 0x10000
	// Position of DEV_RESUME_FROM_HOST field.
	USB_INTE_DEV_RESUME_FROM_HOST_Pos = 0xf
	// Bit mask of DEV_RESUME_FROM_HOST field.
	USB_INTE_DEV_RESUME_FROM_HOST_Msk = 0x8000
	// Bit DEV_RESUME_FROM_HOST.
	USB_INTE_DEV_RESUME_FROM_HOST = 0x8000
	// Position of DEV_SUSPEND field.
	USB_INTE_DEV_SUSPEND_Pos = 0xe
	// Bit mask of DEV_SUSPEND field.
	USB_INTE_DEV_SUSPEND_Msk = 0x4000
	// Bit DEV_SUSPEND.
	USB_INTE_DEV_SUSPEND = 0x4000
	// Position of DEV_CONN_DIS field.
	USB_INTE_DEV_CONN_DIS_Pos = 0xd
	// Bit mask of DEV_CONN_DIS field.
	USB_INTE_DEV_CONN_DIS_Msk = 0x2000
	// Bit DEV_CONN_DIS.
	USB_INTE_DEV_CONN_DIS = 0x2000
	// Position of BUS_RESET field.
	USB_INTE_BUS_RESET_Pos = 0xc
	// Bit mask of BUS_RESET field.
	USB_INTE_BUS_RESET_Msk = 0x1000
	// Bit BUS_RESET.
	USB_INTE_BUS_RESET = 0x1000
	// Position of VBUS_DETECT field.
	USB_INTE_VBUS_DETECT_Pos = 0xb
	// Bit mask of VBUS_DETECT field.
	USB_INTE_VBUS_DETECT_Msk = 0x800
	// Bit VBUS_DETECT.
	USB_INTE_VBUS_DETECT = 0x800
	// Position of STALL field.
	USB_INTE_STALL_Pos = 0xa
	// Bit mask of STALL field.
	USB_INTE_STALL_Msk = 0x400
	// Bit STALL.
	USB_INTE_STALL = 0x400
	// Position of ERROR_CRC field.
	USB_INTE_ERROR_CRC_Pos = 0x9
	// Bit mask of ERROR_CRC field.
	USB_INTE_ERROR_CRC_Msk = 0x200
	// Bit ERROR_CRC.
	USB_INTE_ERROR_CRC = 0x200
	// Position of ERROR_BIT_STUFF field.
	USB_INTE_ERROR_BIT_STUFF_Pos = 0x8
	// Bit mask of ERROR_BIT_STUFF field.
	USB_INTE_ERROR_BIT_STUFF_Msk = 0x100
	// Bit ERROR_BIT_STUFF.
	USB_INTE_ERROR_BIT_STUFF = 0x100
	// Position of ERROR_RX_OVERFLOW field.
	USB_INTE_ERROR_RX_OVERFLOW_Pos = 0x7
	// Bit mask of ERROR_RX_OVERFLOW field.
	USB_INTE_ERROR_RX_OVERFLOW_Msk = 0x80
	// Bit ERROR_RX_OVERFLOW.
	USB_INTE_ERROR_RX_OVERFLOW = 0x80
	// Position of ERROR_RX_TIMEOUT field.
	USB_INTE_ERROR_RX_TIMEOUT_Pos = 0x6
	// Bit mask of ERROR_RX_TIMEOUT field.
	USB_INTE_ERROR_RX_TIMEOUT_Msk = 0x40
	// Bit ERROR_RX_TIMEOUT.
	USB_INTE_ERROR_RX_TIMEOUT = 0x40
	// Position of ERROR_DATA_SEQ field.
	USB_INTE_ERROR_DATA_SEQ_Pos = 0x5
	// Bit mask of ERROR_DATA_SEQ field.
	USB_INTE_ERROR_DATA_SEQ_Msk = 0x20
	// Bit ERROR_DATA_SEQ.
	USB_INTE_ERROR_DATA_SEQ = 0x20
	// Position of BUFF_STATUS field.
	USB_INTE_BUFF_STATUS_Pos = 0x4
	// Bit mask of BUFF_STATUS field.
	USB_INTE_BUFF_STATUS_Msk = 0x10
	// Bit BUFF_STATUS.
	USB_INTE_BUFF_STATUS = 0x10
	// Position of TRANS_COMPLETE field.
	USB_INTE_TRANS_COMPLETE_Pos = 0x3
	// Bit mask of TRANS_COMPLETE field.
	USB_INTE_TRANS_COMPLETE_Msk = 0x8
	// Bit TRANS_COMPLETE.
	USB_INTE_TRANS_COMPLETE = 0x8
	// Position of HOST_SOF field.
	USB_INTE_HOST_SOF_Pos = 0x2
	// Bit mask of HOST_SOF field.
	USB_INTE_HOST_SOF_Msk = 0x4
	// Bit HOST_SOF.
	USB_INTE_HOST_SOF = 0x4
	// Position of HOST_RESUME field.
	USB_INTE_HOST_RESUME_Pos = 0x1
	// Bit mask of HOST_RESUME field.
	USB_INTE_HOST_RESUME_Msk = 0x2
	// Bit HOST_RESUME.
	USB_INTE_HOST_RESUME = 0x2
	// Position of HOST_CONN_DIS field.
	USB_INTE_HOST_CONN_DIS_Pos = 0x0
	// Bit mask of HOST_CONN_DIS field.
	USB_INTE_HOST_CONN_DIS_Msk = 0x1
	// Bit HOST_CONN_DIS.
	USB_INTE_HOST_CONN_DIS = 0x1

	// INTF: Interrupt Force
	// Position of EPX_STOPPED_ON_NAK field.
	USB_INTF_EPX_STOPPED_ON_NAK_Pos = 0x17
	// Bit mask of EPX_STOPPED_ON_NAK field.
	USB_INTF_EPX_STOPPED_ON_NAK_Msk = 0x800000
	// Bit EPX_STOPPED_ON_NAK.
	USB_INTF_EPX_STOPPED_ON_NAK = 0x800000
	// Position of DEV_SM_WATCHDOG_FIRED field.
	USB_INTF_DEV_SM_WATCHDOG_FIRED_Pos = 0x16
	// Bit mask of DEV_SM_WATCHDOG_FIRED field.
	USB_INTF_DEV_SM_WATCHDOG_FIRED_Msk = 0x400000
	// Bit DEV_SM_WATCHDOG_FIRED.
	USB_INTF_DEV_SM_WATCHDOG_FIRED = 0x400000
	// Position of ENDPOINT_ERROR field.
	USB_INTF_ENDPOINT_ERROR_Pos = 0x15
	// Bit mask of ENDPOINT_ERROR field.
	USB_INTF_ENDPOINT_ERROR_Msk = 0x200000
	// Bit ENDPOINT_ERROR.
	USB_INTF_ENDPOINT_ERROR = 0x200000
	// Position of RX_SHORT_PACKET field.
	USB_INTF_RX_SHORT_PACKET_Pos = 0x14
	// Bit mask of RX_SHORT_PACKET field.
	USB_INTF_RX_SHORT_PACKET_Msk = 0x100000
	// Bit RX_SHORT_PACKET.
	USB_INTF_RX_SHORT_PACKET = 0x100000
	// Position of EP_STALL_NAK field.
	USB_INTF_EP_STALL_NAK_Pos = 0x13
	// Bit mask of EP_STALL_NAK field.
	USB_INTF_EP_STALL_NAK_Msk = 0x80000
	// Bit EP_STALL_NAK.
	USB_INTF_EP_STALL_NAK = 0x80000
	// Position of ABORT_DONE field.
	USB_INTF_ABORT_DONE_Pos = 0x12
	// Bit mask of ABORT_DONE field.
	USB_INTF_ABORT_DONE_Msk = 0x40000
	// Bit ABORT_DONE.
	USB_INTF_ABORT_DONE = 0x40000
	// Position of DEV_SOF field.
	USB_INTF_DEV_SOF_Pos = 0x11
	// Bit mask of DEV_SOF field.
	USB_INTF_DEV_SOF_Msk = 0x20000
	// Bit DEV_SOF.
	USB_INTF_DEV_SOF = 0x20000
	// Position of SETUP_REQ field.
	USB_INTF_SETUP_REQ_Pos = 0x10
	// Bit mask of SETUP_REQ field.
	USB_INTF_SETUP_REQ_Msk = 0x10000
	// Bit SETUP_REQ.
	USB_INTF_SETUP_REQ = 0x10000
	// Position of DEV_RESUME_FROM_HOST field.
	USB_INTF_DEV_RESUME_FROM_HOST_Pos = 0xf
	// Bit mask of DEV_RESUME_FROM_HOST field.
	USB_INTF_DEV_RESUME_FROM_HOST_Msk = 0x8000
	// Bit DEV_RESUME_FROM_HOST.
	USB_INTF_DEV_RESUME_FROM_HOST = 0x8000
	// Position of DEV_SUSPEND field.
	USB_INTF_DEV_SUSPEND_Pos = 0xe
	// Bit mask of DEV_SUSPEND field.
	USB_INTF_DEV_SUSPEND_Msk = 0x4000
	// Bit DEV_SUSPEND.
	USB_INTF_DEV_SUSPEND = 0x4000
	// Position of DEV_CONN_DIS field.
	USB_INTF_DEV_CONN_DIS_Pos = 0xd
	// Bit mask of DEV_CONN_DIS field.
	USB_INTF_DEV_CONN_DIS_Msk = 0x2000
	// Bit DEV_CONN_DIS.
	USB_INTF_DEV_CONN_DIS = 0x2000
	// Position of BUS_RESET field.
	USB_INTF_BUS_RESET_Pos = 0xc
	// Bit mask of BUS_RESET field.
	USB_INTF_BUS_RESET_Msk = 0x1000
	// Bit BUS_RESET.
	USB_INTF_BUS_RESET = 0x1000
	// Position of VBUS_DETECT field.
	USB_INTF_VBUS_DETECT_Pos = 0xb
	// Bit mask of VBUS_DETECT field.
	USB_INTF_VBUS_DETECT_Msk = 0x800
	// Bit VBUS_DETECT.
	USB_INTF_VBUS_DETECT = 0x800
	// Position of STALL field.
	USB_INTF_STALL_Pos = 0xa
	// Bit mask of STALL field.
	USB_INTF_STALL_Msk = 0x400
	// Bit STALL.
	USB_INTF_STALL = 0x400
	// Position of ERROR_CRC field.
	USB_INTF_ERROR_CRC_Pos = 0x9
	// Bit mask of ERROR_CRC field.
	USB_INTF_ERROR_CRC_Msk = 0x200
	// Bit ERROR_CRC.
	USB_INTF_ERROR_CRC = 0x200
	// Position of ERROR_BIT_STUFF field.
	USB_INTF_ERROR_BIT_STUFF_Pos = 0x8
	// Bit mask of ERROR_BIT_STUFF field.
	USB_INTF_ERROR_BIT_STUFF_Msk = 0x100
	// Bit ERROR_BIT_STUFF.
	USB_INTF_ERROR_BIT_STUFF = 0x100
	// Position of ERROR_RX_OVERFLOW field.
	USB_INTF_ERROR_RX_OVERFLOW_Pos = 0x7
	// Bit mask of ERROR_RX_OVERFLOW field.
	USB_INTF_ERROR_RX_OVERFLOW_Msk = 0x80
	// Bit ERROR_RX_OVERFLOW.
	USB_INTF_ERROR_RX_OVERFLOW = 0x80
	// Position of ERROR_RX_TIMEOUT field.
	USB_INTF_ERROR_RX_TIMEOUT_Pos = 0x6
	// Bit mask of ERROR_RX_TIMEOUT field.
	USB_INTF_ERROR_RX_TIMEOUT_Msk = 0x40
	// Bit ERROR_RX_TIMEOUT.
	USB_INTF_ERROR_RX_TIMEOUT = 0x40
	// Position of ERROR_DATA_SEQ field.
	USB_INTF_ERROR_DATA_SEQ_Pos = 0x5
	// Bit mask of ERROR_DATA_SEQ field.
	USB_INTF_ERROR_DATA_SEQ_Msk = 0x20
	// Bit ERROR_DATA_SEQ.
	USB_INTF_ERROR_DATA_SEQ = 0x20
	// Position of BUFF_STATUS field.
	USB_INTF_BUFF_STATUS_Pos = 0x4
	// Bit mask of BUFF_STATUS field.
	USB_INTF_BUFF_STATUS_Msk = 0x10
	// Bit BUFF_STATUS.
	USB_INTF_BUFF_STATUS = 0x10
	// Position of TRANS_COMPLETE field.
	USB_INTF_TRANS_COMPLETE_Pos = 0x3
	// Bit mask of TRANS_COMPLETE field.
	USB_INTF_TRANS_COMPLETE_Msk = 0x8
	// Bit TRANS_COMPLETE.
	USB_INTF_TRANS_COMPLETE = 0x8
	// Position of HOST_SOF field.
	USB_INTF_HOST_SOF_Pos = 0x2
	// Bit mask of HOST_SOF field.
	USB_INTF_HOST_SOF_Msk = 0x4
	// Bit HOST_SOF.
	USB_INTF_HOST_SOF = 0x4
	// Position of HOST_RESUME field.
	USB_INTF_HOST_RESUME_Pos = 0x1
	// Bit mask of HOST_RESUME field.
	USB_INTF_HOST_RESUME_Msk = 0x2
	// Bit HOST_RESUME.
	USB_INTF_HOST_RESUME = 0x2
	// Position of HOST_CONN_DIS field.
	USB_INTF_HOST_CONN_DIS_Pos = 0x0
	// Bit mask of HOST_CONN_DIS field.
	USB_INTF_HOST_CONN_DIS_Msk = 0x1
	// Bit HOST_CONN_DIS.
	USB_INTF_HOST_CONN_DIS = 0x1

	// INTS: Interrupt status after masking & forcing
	// Position of EPX_STOPPED_ON_NAK field.
	USB_INTS_EPX_STOPPED_ON_NAK_Pos = 0x17
	// Bit mask of EPX_STOPPED_ON_NAK field.
	USB_INTS_EPX_STOPPED_ON_NAK_Msk = 0x800000
	// Bit EPX_STOPPED_ON_NAK.
	USB_INTS_EPX_STOPPED_ON_NAK = 0x800000
	// Position of DEV_SM_WATCHDOG_FIRED field.
	USB_INTS_DEV_SM_WATCHDOG_FIRED_Pos = 0x16
	// Bit mask of DEV_SM_WATCHDOG_FIRED field.
	USB_INTS_DEV_SM_WATCHDOG_FIRED_Msk = 0x400000
	// Bit DEV_SM_WATCHDOG_FIRED.
	USB_INTS_DEV_SM_WATCHDOG_FIRED = 0x400000
	// Position of ENDPOINT_ERROR field.
	USB_INTS_ENDPOINT_ERROR_Pos = 0x15
	// Bit mask of ENDPOINT_ERROR field.
	USB_INTS_ENDPOINT_ERROR_Msk = 0x200000
	// Bit ENDPOINT_ERROR.
	USB_INTS_ENDPOINT_ERROR = 0x200000
	// Position of RX_SHORT_PACKET field.
	USB_INTS_RX_SHORT_PACKET_Pos = 0x14
	// Bit mask of RX_SHORT_PACKET field.
	USB_INTS_RX_SHORT_PACKET_Msk = 0x100000
	// Bit RX_SHORT_PACKET.
	USB_INTS_RX_SHORT_PACKET = 0x100000
	// Position of EP_STALL_NAK field.
	USB_INTS_EP_STALL_NAK_Pos = 0x13
	// Bit mask of EP_STALL_NAK field.
	USB_INTS_EP_STALL_NAK_Msk = 0x80000
	// Bit EP_STALL_NAK.
	USB_INTS_EP_STALL_NAK = 0x80000
	// Position of ABORT_DONE field.
	USB_INTS_ABORT_DONE_Pos = 0x12
	// Bit mask of ABORT_DONE field.
	USB_INTS_ABORT_DONE_Msk = 0x40000
	// Bit ABORT_DONE.
	USB_INTS_ABORT_DONE = 0x40000
	// Position of DEV_SOF field.
	USB_INTS_DEV_SOF_Pos = 0x11
	// Bit mask of DEV_SOF field.
	USB_INTS_DEV_SOF_Msk = 0x20000
	// Bit DEV_SOF.
	USB_INTS_DEV_SOF = 0x20000
	// Position of SETUP_REQ field.
	USB_INTS_SETUP_REQ_Pos = 0x10
	// Bit mask of SETUP_REQ field.
	USB_INTS_SETUP_REQ_Msk = 0x10000
	// Bit SETUP_REQ.
	USB_INTS_SETUP_REQ = 0x10000
	// Position of DEV_RESUME_FROM_HOST field.
	USB_INTS_DEV_RESUME_FROM_HOST_Pos = 0xf
	// Bit mask of DEV_RESUME_FROM_HOST field.
	USB_INTS_DEV_RESUME_FROM_HOST_Msk = 0x8000
	// Bit DEV_RESUME_FROM_HOST.
	USB_INTS_DEV_RESUME_FROM_HOST = 0x8000
	// Position of DEV_SUSPEND field.
	USB_INTS_DEV_SUSPEND_Pos = 0xe
	// Bit mask of DEV_SUSPEND field.
	USB_INTS_DEV_SUSPEND_Msk = 0x4000
	// Bit DEV_SUSPEND.
	USB_INTS_DEV_SUSPEND = 0x4000
	// Position of DEV_CONN_DIS field.
	USB_INTS_DEV_CONN_DIS_Pos = 0xd
	// Bit mask of DEV_CONN_DIS field.
	USB_INTS_DEV_CONN_DIS_Msk = 0x2000
	// Bit DEV_CONN_DIS.
	USB_INTS_DEV_CONN_DIS = 0x2000
	// Position of BUS_RESET field.
	USB_INTS_BUS_RESET_Pos = 0xc
	// Bit mask of BUS_RESET field.
	USB_INTS_BUS_RESET_Msk = 0x1000
	// Bit BUS_RESET.
	USB_INTS_BUS_RESET = 0x1000
	// Position of VBUS_DETECT field.
	USB_INTS_VBUS_DETECT_Pos = 0xb
	// Bit mask of VBUS_DETECT field.
	USB_INTS_VBUS_DETECT_Msk = 0x800
	// Bit VBUS_DETECT.
	USB_INTS_VBUS_DETECT = 0x800
	// Position of STALL field.
	USB_INTS_STALL_Pos = 0xa
	// Bit mask of STALL field.
	USB_INTS_STALL_Msk = 0x400
	// Bit STALL.
	USB_INTS_STALL = 0x400
	// Position of ERROR_CRC field.
	USB_INTS_ERROR_CRC_Pos = 0x9
	// Bit mask of ERROR_CRC field.
	USB_INTS_ERROR_CRC_Msk = 0x200
	// Bit ERROR_CRC.
	USB_INTS_ERROR_CRC = 0x200
	// Position of ERROR_BIT_STUFF field.
	USB_INTS_ERROR_BIT_STUFF_Pos = 0x8
	// Bit mask of ERROR_BIT_STUFF field.
	USB_INTS_ERROR_BIT_STUFF_Msk = 0x100
	// Bit ERROR_BIT_STUFF.
	USB_INTS_ERROR_BIT_STUFF = 0x100
	// Position of ERROR_RX_OVERFLOW field.
	USB_INTS_ERROR_RX_OVERFLOW_Pos = 0x7
	// Bit mask of ERROR_RX_OVERFLOW field.
	USB_INTS_ERROR_RX_OVERFLOW_Msk = 0x80
	// Bit ERROR_RX_OVERFLOW.
	USB_INTS_ERROR_RX_OVERFLOW = 0x80
	// Position of ERROR_RX_TIMEOUT field.
	USB_INTS_ERROR_RX_TIMEOUT_Pos = 0x6
	// Bit mask of ERROR_RX_TIMEOUT field.
	USB_INTS_ERROR_RX_TIMEOUT_Msk = 0x40
	// Bit ERROR_RX_TIMEOUT.
	USB_INTS_ERROR_RX_TIMEOUT = 0x40
	// Position of ERROR_DATA_SEQ field.
	USB_INTS_ERROR_DATA_SEQ_Pos = 0x5
	// Bit mask of ERROR_DATA_SEQ field.
	USB_INTS_ERROR_DATA_SEQ_Msk = 0x20
	// Bit ERROR_DATA_SEQ.
	USB_INTS_ERROR_DATA_SEQ = 0x20
	// Position of BUFF_STATUS field.
	USB_INTS_BUFF_STATUS_Pos = 0x4
	// Bit mask of BUFF_STATUS field.
	USB_INTS_BUFF_STATUS_Msk = 0x10
	// Bit BUFF_STATUS.
	USB_INTS_BUFF_STATUS = 0x10
	// Position of TRANS_COMPLETE field.
	USB_INTS_TRANS_COMPLETE_Pos = 0x3
	// Bit mask of TRANS_COMPLETE field.
	USB_INTS_TRANS_COMPLETE_Msk = 0x8
	// Bit TRANS_COMPLETE.
	USB_INTS_TRANS_COMPLETE = 0x8
	// Position of HOST_SOF field.
	USB_INTS_HOST_SOF_Pos = 0x2
	// Bit mask of HOST_SOF field.
	USB_INTS_HOST_SOF_Msk = 0x4
	// Bit HOST_SOF.
	USB_INTS_HOST_SOF = 0x4
	// Position of HOST_RESUME field.
	USB_INTS_HOST_RESUME_Pos = 0x1
	// Bit mask of HOST_RESUME field.
	USB_INTS_HOST_RESUME_Msk = 0x2
	// Bit HOST_RESUME.
	USB_INTS_HOST_RESUME = 0x2
	// Position of HOST_CONN_DIS field.
	USB_INTS_HOST_CONN_DIS_Pos = 0x0
	// Bit mask of HOST_CONN_DIS field.
	USB_INTS_HOST_CONN_DIS_Msk = 0x1
	// Bit HOST_CONN_DIS.
	USB_INTS_HOST_CONN_DIS = 0x1

	// SOF_TIMESTAMP_RAW: Device only. Raw value of free-running PHY clock counter @48MHz. Used to calculate time between SOF events.
	// Position of SOF_TIMESTAMP_RAW field.
	USB_SOF_TIMESTAMP_RAW_SOF_TIMESTAMP_RAW_Pos = 0x0
	// Bit mask of SOF_TIMESTAMP_RAW field.
	USB_SOF_TIMESTAMP_RAW_SOF_TIMESTAMP_RAW_Msk = 0x1fffff

	// SOF_TIMESTAMP_LAST: Device only. Value of free-running PHY clock counter @48MHz when last SOF event occurred.
	// Position of SOF_TIMESTAMP_LAST field.
	USB_SOF_TIMESTAMP_LAST_SOF_TIMESTAMP_LAST_Pos = 0x0
	// Bit mask of SOF_TIMESTAMP_LAST field.
	USB_SOF_TIMESTAMP_LAST_SOF_TIMESTAMP_LAST_Msk = 0x1fffff

	// SM_STATE
	// Position of RX_DASM field.
	USB_SM_STATE_RX_DASM_Pos = 0x8
	// Bit mask of RX_DASM field.
	USB_SM_STATE_RX_DASM_Msk = 0xf00
	// Position of BC_STATE field.
	USB_SM_STATE_BC_STATE_Pos = 0x5
	// Bit mask of BC_STATE field.
	USB_SM_STATE_BC_STATE_Msk = 0xe0
	// Position of STATE field.
	USB_SM_STATE_STATE_Pos = 0x0
	// Bit mask of STATE field.
	USB_SM_STATE_STATE_Msk = 0x1f

	// EP_TX_ERROR: TX error count for each endpoint. Write to each field to reset the counter to 0.
	// Position of EP15 field.
	USB_EP_TX_ERROR_EP15_Pos = 0x1e
	// Bit mask of EP15 field.
	USB_EP_TX_ERROR_EP15_Msk = 0xc0000000
	// Position of EP14 field.
	USB_EP_TX_ERROR_EP14_Pos = 0x1c
	// Bit mask of EP14 field.
	USB_EP_TX_ERROR_EP14_Msk = 0x30000000
	// Position of EP13 field.
	USB_EP_TX_ERROR_EP13_Pos = 0x1a
	// Bit mask of EP13 field.
	USB_EP_TX_ERROR_EP13_Msk = 0xc000000
	// Position of EP12 field.
	USB_EP_TX_ERROR_EP12_Pos = 0x18
	// Bit mask of EP12 field.
	USB_EP_TX_ERROR_EP12_Msk = 0x3000000
	// Position of EP11 field.
	USB_EP_TX_ERROR_EP11_Pos = 0x16
	// Bit mask of EP11 field.
	USB_EP_TX_ERROR_EP11_Msk = 0xc00000
	// Position of EP10 field.
	USB_EP_TX_ERROR_EP10_Pos = 0x14
	// Bit mask of EP10 field.
	USB_EP_TX_ERROR_EP10_Msk = 0x300000
	// Position of EP9 field.
	USB_EP_TX_ERROR_EP9_Pos = 0x12
	// Bit mask of EP9 field.
	USB_EP_TX_ERROR_EP9_Msk = 0xc0000
	// Position of EP8 field.
	USB_EP_TX_ERROR_EP8_Pos = 0x10
	// Bit mask of EP8 field.
	USB_EP_TX_ERROR_EP8_Msk = 0x30000
	// Position of EP7 field.
	USB_EP_TX_ERROR_EP7_Pos = 0xe
	// Bit mask of EP7 field.
	USB_EP_TX_ERROR_EP7_Msk = 0xc000
	// Position of EP6 field.
	USB_EP_TX_ERROR_EP6_Pos = 0xc
	// Bit mask of EP6 field.
	USB_EP_TX_ERROR_EP6_Msk = 0x3000
	// Position of EP5 field.
	USB_EP_TX_ERROR_EP5_Pos = 0xa
	// Bit mask of EP5 field.
	USB_EP_TX_ERROR_EP5_Msk = 0xc00
	// Position of EP4 field.
	USB_EP_TX_ERROR_EP4_Pos = 0x8
	// Bit mask of EP4 field.
	USB_EP_TX_ERROR_EP4_Msk = 0x300
	// Position of EP3 field.
	USB_EP_TX_ERROR_EP3_Pos = 0x6
	// Bit mask of EP3 field.
	USB_EP_TX_ERROR_EP3_Msk = 0xc0
	// Position of EP2 field.
	USB_EP_TX_ERROR_EP2_Pos = 0x4
	// Bit mask of EP2 field.
	USB_EP_TX_ERROR_EP2_Msk = 0x30
	// Position of EP1 field.
	USB_EP_TX_ERROR_EP1_Pos = 0x2
	// Bit mask of EP1 field.
	USB_EP_TX_ERROR_EP1_Msk = 0xc
	// Position of EP0 field.
	USB_EP_TX_ERROR_EP0_Pos = 0x0
	// Bit mask of EP0 field.
	USB_EP_TX_ERROR_EP0_Msk = 0x3

	// EP_RX_ERROR: RX error count for each endpoint. Write to each field to reset the counter to 0.
	// Position of EP15_SEQ field.
	USB_EP_RX_ERROR_EP15_SEQ_Pos = 0x1f
	// Bit mask of EP15_SEQ field.
	USB_EP_RX_ERROR_EP15_SEQ_Msk = 0x80000000
	// Bit EP15_SEQ.
	USB_EP_RX_ERROR_EP15_SEQ = 0x80000000
	// Position of EP15_TRANSACTION field.
	USB_EP_RX_ERROR_EP15_TRANSACTION_Pos = 0x1e
	// Bit mask of EP15_TRANSACTION field.
	USB_EP_RX_ERROR_EP15_TRANSACTION_Msk = 0x40000000
	// Bit EP15_TRANSACTION.
	USB_EP_RX_ERROR_EP15_TRANSACTION = 0x40000000
	// Position of EP14_SEQ field.
	USB_EP_RX_ERROR_EP14_SEQ_Pos = 0x1d
	// Bit mask of EP14_SEQ field.
	USB_EP_RX_ERROR_EP14_SEQ_Msk = 0x20000000
	// Bit EP14_SEQ.
	USB_EP_RX_ERROR_EP14_SEQ = 0x20000000
	// Position of EP14_TRANSACTION field.
	USB_EP_RX_ERROR_EP14_TRANSACTION_Pos = 0x1c
	// Bit mask of EP14_TRANSACTION field.
	USB_EP_RX_ERROR_EP14_TRANSACTION_Msk = 0x10000000
	// Bit EP14_TRANSACTION.
	USB_EP_RX_ERROR_EP14_TRANSACTION = 0x10000000
	// Position of EP13_SEQ field.
	USB_EP_RX_ERROR_EP13_SEQ_Pos = 0x1b
	// Bit mask of EP13_SEQ field.
	USB_EP_RX_ERROR_EP13_SEQ_Msk = 0x8000000
	// Bit EP13_SEQ.
	USB_EP_RX_ERROR_EP13_SEQ = 0x8000000
	// Position of EP13_TRANSACTION field.
	USB_EP_RX_ERROR_EP13_TRANSACTION_Pos = 0x1a
	// Bit mask of EP13_TRANSACTION field.
	USB_EP_RX_ERROR_EP13_TRANSACTION_Msk = 0x4000000
	// Bit EP13_TRANSACTION.
	USB_EP_RX_ERROR_EP13_TRANSACTION = 0x4000000
	// Position of EP12_SEQ field.
	USB_EP_RX_ERROR_EP12_SEQ_Pos = 0x19
	// Bit mask of EP12_SEQ field.
	USB_EP_RX_ERROR_EP12_SEQ_Msk = 0x2000000
	// Bit EP12_SEQ.
	USB_EP_RX_ERROR_EP12_SEQ = 0x2000000
	// Position of EP12_TRANSACTION field.
	USB_EP_RX_ERROR_EP12_TRANSACTION_Pos = 0x18
	// Bit mask of EP12_TRANSACTION field.
	USB_EP_RX_ERROR_EP12_TRANSACTION_Msk = 0x1000000
	// Bit EP12_TRANSACTION.
	USB_EP_RX_ERROR_EP12_TRANSACTION = 0x1000000
	// Position of EP11_SEQ field.
	USB_EP_RX_ERROR_EP11_SEQ_Pos = 0x17
	// Bit mask of EP11_SEQ field.
	USB_EP_RX_ERROR_EP11_SEQ_Msk = 0x800000
	// Bit EP11_SEQ.
	USB_EP_RX_ERROR_EP11_SEQ = 0x800000
	// Position of EP11_TRANSACTION field.
	USB_EP_RX_ERROR_EP11_TRANSACTION_Pos = 0x16
	// Bit mask of EP11_TRANSACTION field.
	USB_EP_RX_ERROR_EP11_TRANSACTION_Msk = 0x400000
	// Bit EP11_TRANSACTION.
	USB_EP_RX_ERROR_EP11_TRANSACTION = 0x400000
	// Position of EP10_SEQ field.
	USB_EP_RX_ERROR_EP10_SEQ_Pos = 0x15
	// Bit mask of EP10_SEQ field.
	USB_EP_RX_ERROR_EP10_SEQ_Msk = 0x200000
	// Bit EP10_SEQ.
	USB_EP_RX_ERROR_EP10_SEQ = 0x200000
	// Position of EP10_TRANSACTION field.
	USB_EP_RX_ERROR_EP10_TRANSACTION_Pos = 0x14
	// Bit mask of EP10_TRANSACTION field.
	USB_EP_RX_ERROR_EP10_TRANSACTION_Msk = 0x100000
	// Bit EP10_TRANSACTION.
	USB_EP_RX_ERROR_EP10_TRANSACTION = 0x100000
	// Position of EP9_SEQ field.
	USB_EP_RX_ERROR_EP9_SEQ_Pos = 0x13
	// Bit mask of EP9_SEQ field.
	USB_EP_RX_ERROR_EP9_SEQ_Msk = 0x80000
	// Bit EP9_SEQ.
	USB_EP_RX_ERROR_EP9_SEQ = 0x80000
	// Position of EP9_TRANSACTION field.
	USB_EP_RX_ERROR_EP9_TRANSACTION_Pos = 0x12
	// Bit mask of EP9_TRANSACTION field.
	USB_EP_RX_ERROR_EP9_TRANSACTION_Msk = 0x40000
	// Bit EP9_TRANSACTION.
	USB_EP_RX_ERROR_EP9_TRANSACTION = 0x40000
	// Position of EP8_SEQ field.
	USB_EP_RX_ERROR_EP8_SEQ_Pos = 0x11
	// Bit mask of EP8_SEQ field.
	USB_EP_RX_ERROR_EP8_SEQ_Msk = 0x20000
	// Bit EP8_SEQ.
	USB_EP_RX_ERROR_EP8_SEQ = 0x20000
	// Position of EP8_TRANSACTION field.
	USB_EP_RX_ERROR_EP8_TRANSACTION_Pos = 0x10
	// Bit mask of EP8_TRANSACTION field.
	USB_EP_RX_ERROR_EP8_TRANSACTION_Msk = 0x10000
	// Bit EP8_TRANSACTION.
	USB_EP_RX_ERROR_EP8_TRANSACTION = 0x10000
	// Position of EP7_SEQ field.
	USB_EP_RX_ERROR_EP7_SEQ_Pos = 0xf
	// Bit mask of EP7_SEQ field.
	USB_EP_RX_ERROR_EP7_SEQ_Msk = 0x8000
	// Bit EP7_SEQ.
	USB_EP_RX_ERROR_EP7_SEQ = 0x8000
	// Position of EP7_TRANSACTION field.
	USB_EP_RX_ERROR_EP7_TRANSACTION_Pos = 0xe
	// Bit mask of EP7_TRANSACTION field.
	USB_EP_RX_ERROR_EP7_TRANSACTION_Msk = 0x4000
	// Bit EP7_TRANSACTION.
	USB_EP_RX_ERROR_EP7_TRANSACTION = 0x4000
	// Position of EP6_SEQ field.
	USB_EP_RX_ERROR_EP6_SEQ_Pos = 0xd
	// Bit mask of EP6_SEQ field.
	USB_EP_RX_ERROR_EP6_SEQ_Msk = 0x2000
	// Bit EP6_SEQ.
	USB_EP_RX_ERROR_EP6_SEQ = 0x2000
	// Position of EP6_TRANSACTION field.
	USB_EP_RX_ERROR_EP6_TRANSACTION_Pos = 0xc
	// Bit mask of EP6_TRANSACTION field.
	USB_EP_RX_ERROR_EP6_TRANSACTION_Msk = 0x1000
	// Bit EP6_TRANSACTION.
	USB_EP_RX_ERROR_EP6_TRANSACTION = 0x1000
	// Position of EP5_SEQ field.
	USB_EP_RX_ERROR_EP5_SEQ_Pos = 0xb
	// Bit mask of EP5_SEQ field.
	USB_EP_RX_ERROR_EP5_SEQ_Msk = 0x800
	// Bit EP5_SEQ.
	USB_EP_RX_ERROR_EP5_SEQ = 0x800
	// Position of EP5_TRANSACTION field.
	USB_EP_RX_ERROR_EP5_TRANSACTION_Pos = 0xa
	// Bit mask of EP5_TRANSACTION field.
	USB_EP_RX_ERROR_EP5_TRANSACTION_Msk = 0x400
	// Bit EP5_TRANSACTION.
	USB_EP_RX_ERROR_EP5_TRANSACTION = 0x400
	// Position of EP4_SEQ field.
	USB_EP_RX_ERROR_EP4_SEQ_Pos = 0x9
	// Bit mask of EP4_SEQ field.
	USB_EP_RX_ERROR_EP4_SEQ_Msk = 0x200
	// Bit EP4_SEQ.
	USB_EP_RX_ERROR_EP4_SEQ = 0x200
	// Position of EP4_TRANSACTION field.
	USB_EP_RX_ERROR_EP4_TRANSACTION_Pos = 0x8
	// Bit mask of EP4_TRANSACTION field.
	USB_EP_RX_ERROR_EP4_TRANSACTION_Msk = 0x100
	// Bit EP4_TRANSACTION.
	USB_EP_RX_ERROR_EP4_TRANSACTION = 0x100
	// Position of EP3_SEQ field.
	USB_EP_RX_ERROR_EP3_SEQ_Pos = 0x7
	// Bit mask of EP3_SEQ field.
	USB_EP_RX_ERROR_EP3_SEQ_Msk = 0x80
	// Bit EP3_SEQ.
	USB_EP_RX_ERROR_EP3_SEQ = 0x80
	// Position of EP3_TRANSACTION field.
	USB_EP_RX_ERROR_EP3_TRANSACTION_Pos = 0x6
	// Bit mask of EP3_TRANSACTION field.
	USB_EP_RX_ERROR_EP3_TRANSACTION_Msk = 0x40
	// Bit EP3_TRANSACTION.
	USB_EP_RX_ERROR_EP3_TRANSACTION = 0x40
	// Position of EP2_SEQ field.
	USB_EP_RX_ERROR_EP2_SEQ_Pos = 0x5
	// Bit mask of EP2_SEQ field.
	USB_EP_RX_ERROR_EP2_SEQ_Msk = 0x20
	// Bit EP2_SEQ.
	USB_EP_RX_ERROR_EP2_SEQ = 0x20
	// Position of EP2_TRANSACTION field.
	USB_EP_RX_ERROR_EP2_TRANSACTION_Pos = 0x4
	// Bit mask of EP2_TRANSACTION field.
	USB_EP_RX_ERROR_EP2_TRANSACTION_Msk = 0x10
	// Bit EP2_TRANSACTION.
	USB_EP_RX_ERROR_EP2_TRANSACTION = 0x10
	// Position of EP1_SEQ field.
	USB_EP_RX_ERROR_EP1_SEQ_Pos = 0x3
	// Bit mask of EP1_SEQ field.
	USB_EP_RX_ERROR_EP1_SEQ_Msk = 0x8
	// Bit EP1_SEQ.
	USB_EP_RX_ERROR_EP1_SEQ = 0x8
	// Position of EP1_TRANSACTION field.
	USB_EP_RX_ERROR_EP1_TRANSACTION_Pos = 0x2
	// Bit mask of EP1_TRANSACTION field.
	USB_EP_RX_ERROR_EP1_TRANSACTION_Msk = 0x4
	// Bit EP1_TRANSACTION.
	USB_EP_RX_ERROR_EP1_TRANSACTION = 0x4
	// Position of EP0_SEQ field.
	USB_EP_RX_ERROR_EP0_SEQ_Pos = 0x1
	// Bit mask of EP0_SEQ field.
	USB_EP_RX_ERROR_EP0_SEQ_Msk = 0x2
	// Bit EP0_SEQ.
	USB_EP_RX_ERROR_EP0_SEQ = 0x2
	// Position of EP0_TRANSACTION field.
	USB_EP_RX_ERROR_EP0_TRANSACTION_Pos = 0x0
	// Bit mask of EP0_TRANSACTION field.
	USB_EP_RX_ERROR_EP0_TRANSACTION_Msk = 0x1
	// Bit EP0_TRANSACTION.
	USB_EP_RX_ERROR_EP0_TRANSACTION = 0x1

	// DEV_SM_WATCHDOG: Watchdog that forces the device state machine to idle and raises an interrupt if the device stays in a state that isn't idle for the configured limit. The counter is reset on every state transition. Set limit while enable is low and then set the enable.
	// Position of FIRED field.
	USB_DEV_SM_WATCHDOG_FIRED_Pos = 0x14
	// Bit mask of FIRED field.
	USB_DEV_SM_WATCHDOG_FIRED_Msk = 0x100000
	// Bit FIRED.
	USB_DEV_SM_WATCHDOG_FIRED = 0x100000
	// Position of RESET field.
	USB_DEV_SM_WATCHDOG_RESET_Pos = 0x13
	// Bit mask of RESET field.
	USB_DEV_SM_WATCHDOG_RESET_Msk = 0x80000
	// Bit RESET.
	USB_DEV_SM_WATCHDOG_RESET = 0x80000
	// Position of ENABLE field.
	USB_DEV_SM_WATCHDOG_ENABLE_Pos = 0x12
	// Bit mask of ENABLE field.
	USB_DEV_SM_WATCHDOG_ENABLE_Msk = 0x40000
	// Bit ENABLE.
	USB_DEV_SM_WATCHDOG_ENABLE = 0x40000
	// Position of LIMIT field.
	USB_DEV_SM_WATCHDOG_LIMIT_Pos = 0x0
	// Bit mask of LIMIT field.
	USB_DEV_SM_WATCHDOG_LIMIT_Msk = 0x3ffff
)

// Constants for TRNG: ARM TrustZone RNG register block
const (
	// RNG_IMR: Interrupt masking.
	// Position of RESERVED field.
	TRNG_RNG_IMR_RESERVED_Pos = 0x4
	// Bit mask of RESERVED field.
	TRNG_RNG_IMR_RESERVED_Msk = 0xfffffff0
	// Position of VN_ERR_INT_MASK field.
	TRNG_RNG_IMR_VN_ERR_INT_MASK_Pos = 0x3
	// Bit mask of VN_ERR_INT_MASK field.
	TRNG_RNG_IMR_VN_ERR_INT_MASK_Msk = 0x8
	// Bit VN_ERR_INT_MASK.
	TRNG_RNG_IMR_VN_ERR_INT_MASK = 0x8
	// Position of CRNGT_ERR_INT_MASK field.
	TRNG_RNG_IMR_CRNGT_ERR_INT_MASK_Pos = 0x2
	// Bit mask of CRNGT_ERR_INT_MASK field.
	TRNG_RNG_IMR_CRNGT_ERR_INT_MASK_Msk = 0x4
	// Bit CRNGT_ERR_INT_MASK.
	TRNG_RNG_IMR_CRNGT_ERR_INT_MASK = 0x4
	// Position of AUTOCORR_ERR_INT_MASK field.
	TRNG_RNG_IMR_AUTOCORR_ERR_INT_MASK_Pos = 0x1
	// Bit mask of AUTOCORR_ERR_INT_MASK field.
	TRNG_RNG_IMR_AUTOCORR_ERR_INT_MASK_Msk = 0x2
	// Bit AUTOCORR_ERR_INT_MASK.
	TRNG_RNG_IMR_AUTOCORR_ERR_INT_MASK = 0x2
	// Position of EHR_VALID_INT_MASK field.
	TRNG_RNG_IMR_EHR_VALID_INT_MASK_Pos = 0x0
	// Bit mask of EHR_VALID_INT_MASK field.
	TRNG_RNG_IMR_EHR_VALID_INT_MASK_Msk = 0x1
	// Bit EHR_VALID_INT_MASK.
	TRNG_RNG_IMR_EHR_VALID_INT_MASK = 0x1

	// RNG_ISR: RNG status register. If corresponding RNG_IMR bit is unmasked, an interrupt will be generated.
	// Position of RESERVED field.
	TRNG_RNG_ISR_RESERVED_Pos = 0x4
	// Bit mask of RESERVED field.
	TRNG_RNG_ISR_RESERVED_Msk = 0xfffffff0
	// Position of VN_ERR field.
	TRNG_RNG_ISR_VN_ERR_Pos = 0x3
	// Bit mask of VN_ERR field.
	TRNG_RNG_ISR_VN_ERR_Msk = 0x8
	// Bit VN_ERR.
	TRNG_RNG_ISR_VN_ERR = 0x8
	// Position of CRNGT_ERR field.
	TRNG_RNG_ISR_CRNGT_ERR_Pos = 0x2
	// Bit mask of CRNGT_ERR field.
	TRNG_RNG_ISR_CRNGT_ERR_Msk = 0x4
	// Bit CRNGT_ERR.
	TRNG_RNG_ISR_CRNGT_ERR = 0x4
	// Position of AUTOCORR_ERR field.
	TRNG_RNG_ISR_AUTOCORR_ERR_Pos = 0x1
	// Bit mask of AUTOCORR_ERR field.
	TRNG_RNG_ISR_AUTOCORR_ERR_Msk = 0x2
	// Bit AUTOCORR_ERR.
	TRNG_RNG_ISR_AUTOCORR_ERR = 0x2
	// Position of EHR_VALID field.
	TRNG_RNG_ISR_EHR_VALID_Pos = 0x0
	// Bit mask of EHR_VALID field.
	TRNG_RNG_ISR_EHR_VALID_Msk = 0x1
	// Bit EHR_VALID.
	TRNG_RNG_ISR_EHR_VALID = 0x1

	// RNG_ICR: Interrupt/status bit clear Register.
	// Position of RESERVED field.
	TRNG_RNG_ICR_RESERVED_Pos = 0x4
	// Bit mask of RESERVED field.
	TRNG_RNG_ICR_RESERVED_Msk = 0xfffffff0
	// Position of VN_ERR field.
	TRNG_RNG_ICR_VN_ERR_Pos = 0x3
	// Bit mask of VN_ERR field.
	TRNG_RNG_ICR_VN_ERR_Msk = 0x8
	// Bit VN_ERR.
	TRNG_RNG_ICR_VN_ERR = 0x8
	// Position of CRNGT_ERR field.
	TRNG_RNG_ICR_CRNGT_ERR_Pos = 0x2
	// Bit mask of CRNGT_ERR field.
	TRNG_RNG_ICR_CRNGT_ERR_Msk = 0x4
	// Bit CRNGT_ERR.
	TRNG_RNG_ICR_CRNGT_ERR = 0x4
	// Position of AUTOCORR_ERR field.
	TRNG_RNG_ICR_AUTOCORR_ERR_Pos = 0x1
	// Bit mask of AUTOCORR_ERR field.
	TRNG_RNG_ICR_AUTOCORR_ERR_Msk = 0x2
	// Bit AUTOCORR_ERR.
	TRNG_RNG_ICR_AUTOCORR_ERR = 0x2
	// Position of EHR_VALID field.
	TRNG_RNG_ICR_EHR_VALID_Pos = 0x0
	// Bit mask of EHR_VALID field.
	TRNG_RNG_ICR_EHR_VALID_Msk = 0x1
	// Bit EHR_VALID.
	TRNG_RNG_ICR_EHR_VALID = 0x1

	// TRNG_CONFIG: Selecting the inverter-chain length.
	// Position of RESERVED field.
	TRNG_TRNG_CONFIG_RESERVED_Pos = 0x2
	// Bit mask of RESERVED field.
	TRNG_TRNG_CONFIG_RESERVED_Msk = 0xfffffffc
	// Position of RND_SRC_SEL field.
	TRNG_TRNG_CONFIG_RND_SRC_SEL_Pos = 0x0
	// Bit mask of RND_SRC_SEL field.
	TRNG_TRNG_CONFIG_RND_SRC_SEL_Msk = 0x3

	// TRNG_VALID: 192 bit collection indication.
	// Position of RESERVED field.
	TRNG_TRNG_VALID_RESERVED_Pos = 0x1
	// Bit mask of RESERVED field.
	TRNG_TRNG_VALID_RESERVED_Msk = 0xfffffffe
	// Position of EHR_VALID field.
	TRNG_TRNG_VALID_EHR_VALID_Pos = 0x0
	// Bit mask of EHR_VALID field.
	TRNG_TRNG_VALID_EHR_VALID_Msk = 0x1
	// Bit EHR_VALID.
	TRNG_TRNG_VALID_EHR_VALID = 0x1

	// EHR_DATA0: RNG collected bits.
	// Position of EHR_DATA0 field.
	TRNG_EHR_DATA0_EHR_DATA0_Pos = 0x0
	// Bit mask of EHR_DATA0 field.
	TRNG_EHR_DATA0_EHR_DATA0_Msk = 0xffffffff

	// EHR_DATA1: RNG collected bits.
	// Position of EHR_DATA1 field.
	TRNG_EHR_DATA1_EHR_DATA1_Pos = 0x0
	// Bit mask of EHR_DATA1 field.
	TRNG_EHR_DATA1_EHR_DATA1_Msk = 0xffffffff

	// EHR_DATA2: RNG collected bits.
	// Position of EHR_DATA2 field.
	TRNG_EHR_DATA2_EHR_DATA2_Pos = 0x0
	// Bit mask of EHR_DATA2 field.
	TRNG_EHR_DATA2_EHR_DATA2_Msk = 0xffffffff

	// EHR_DATA3: RNG collected bits.
	// Position of EHR_DATA3 field.
	TRNG_EHR_DATA3_EHR_DATA3_Pos = 0x0
	// Bit mask of EHR_DATA3 field.
	TRNG_EHR_DATA3_EHR_DATA3_Msk = 0xffffffff

	// EHR_DATA4: RNG collected bits.
	// Position of EHR_DATA4 field.
	TRNG_EHR_DATA4_EHR_DATA4_Pos = 0x0
	// Bit mask of EHR_DATA4 field.
	TRNG_EHR_DATA4_EHR_DATA4_Msk = 0xffffffff

	// EHR_DATA5: RNG collected bits.
	// Position of EHR_DATA5 field.
	TRNG_EHR_DATA5_EHR_DATA5_Pos = 0x0
	// Bit mask of EHR_DATA5 field.
	TRNG_EHR_DATA5_EHR_DATA5_Msk = 0xffffffff

	// RND_SOURCE_ENABLE: Enable signal for the random source.
	// Position of RESERVED field.
	TRNG_RND_SOURCE_ENABLE_RESERVED_Pos = 0x1
	// Bit mask of RESERVED field.
	TRNG_RND_SOURCE_ENABLE_RESERVED_Msk = 0xfffffffe
	// Position of RND_SRC_EN field.
	TRNG_RND_SOURCE_ENABLE_RND_SRC_EN_Pos = 0x0
	// Bit mask of RND_SRC_EN field.
	TRNG_RND_SOURCE_ENABLE_RND_SRC_EN_Msk = 0x1
	// Bit RND_SRC_EN.
	TRNG_RND_SOURCE_ENABLE_RND_SRC_EN = 0x1

	// SAMPLE_CNT1: Counts clocks between sampling of random bit.
	// Position of SAMPLE_CNTR1 field.
	TRNG_SAMPLE_CNT1_SAMPLE_CNTR1_Pos = 0x0
	// Bit mask of SAMPLE_CNTR1 field.
	TRNG_SAMPLE_CNT1_SAMPLE_CNTR1_Msk = 0xffffffff

	// AUTOCORR_STATISTIC: Statistic about Autocorrelation test activations.
	// Position of RESERVED field.
	TRNG_AUTOCORR_STATISTIC_RESERVED_Pos = 0x16
	// Bit mask of RESERVED field.
	TRNG_AUTOCORR_STATISTIC_RESERVED_Msk = 0xffc00000
	// Position of AUTOCORR_FAILS field.
	TRNG_AUTOCORR_STATISTIC_AUTOCORR_FAILS_Pos = 0xe
	// Bit mask of AUTOCORR_FAILS field.
	TRNG_AUTOCORR_STATISTIC_AUTOCORR_FAILS_Msk = 0x3fc000
	// Position of AUTOCORR_TRYS field.
	TRNG_AUTOCORR_STATISTIC_AUTOCORR_TRYS_Pos = 0x0
	// Bit mask of AUTOCORR_TRYS field.
	TRNG_AUTOCORR_STATISTIC_AUTOCORR_TRYS_Msk = 0x3fff

	// TRNG_DEBUG_CONTROL: Debug register.
	// Position of AUTO_CORRELATE_BYPASS field.
	TRNG_TRNG_DEBUG_CONTROL_AUTO_CORRELATE_BYPASS_Pos = 0x3
	// Bit mask of AUTO_CORRELATE_BYPASS field.
	TRNG_TRNG_DEBUG_CONTROL_AUTO_CORRELATE_BYPASS_Msk = 0x8
	// Bit AUTO_CORRELATE_BYPASS.
	TRNG_TRNG_DEBUG_CONTROL_AUTO_CORRELATE_BYPASS = 0x8
	// Position of TRNG_CRNGT_BYPASS field.
	TRNG_TRNG_DEBUG_CONTROL_TRNG_CRNGT_BYPASS_Pos = 0x2
	// Bit mask of TRNG_CRNGT_BYPASS field.
	TRNG_TRNG_DEBUG_CONTROL_TRNG_CRNGT_BYPASS_Msk = 0x4
	// Bit TRNG_CRNGT_BYPASS.
	TRNG_TRNG_DEBUG_CONTROL_TRNG_CRNGT_BYPASS = 0x4
	// Position of VNC_BYPASS field.
	TRNG_TRNG_DEBUG_CONTROL_VNC_BYPASS_Pos = 0x1
	// Bit mask of VNC_BYPASS field.
	TRNG_TRNG_DEBUG_CONTROL_VNC_BYPASS_Msk = 0x2
	// Bit VNC_BYPASS.
	TRNG_TRNG_DEBUG_CONTROL_VNC_BYPASS = 0x2
	// Position of RESERVED field.
	TRNG_TRNG_DEBUG_CONTROL_RESERVED_Pos = 0x0
	// Bit mask of RESERVED field.
	TRNG_TRNG_DEBUG_CONTROL_RESERVED_Msk = 0x1
	// Bit RESERVED.
	TRNG_TRNG_DEBUG_CONTROL_RESERVED = 0x1

	// TRNG_SW_RESET: Generate internal SW reset within the RNG block.
	// Position of RESERVED field.
	TRNG_TRNG_SW_RESET_RESERVED_Pos = 0x1
	// Bit mask of RESERVED field.
	TRNG_TRNG_SW_RESET_RESERVED_Msk = 0xfffffffe
	// Position of TRNG_SW_RESET field.
	TRNG_TRNG_SW_RESET_TRNG_SW_RESET_Pos = 0x0
	// Bit mask of TRNG_SW_RESET field.
	TRNG_TRNG_SW_RESET_TRNG_SW_RESET_Msk = 0x1
	// Bit TRNG_SW_RESET.
	TRNG_TRNG_SW_RESET_TRNG_SW_RESET = 0x1

	// RNG_DEBUG_EN_INPUT: Enable the RNG debug mode
	// Position of RESERVED field.
	TRNG_RNG_DEBUG_EN_INPUT_RESERVED_Pos = 0x1
	// Bit mask of RESERVED field.
	TRNG_RNG_DEBUG_EN_INPUT_RESERVED_Msk = 0xfffffffe
	// Position of RNG_DEBUG_EN field.
	TRNG_RNG_DEBUG_EN_INPUT_RNG_DEBUG_EN_Pos = 0x0
	// Bit mask of RNG_DEBUG_EN field.
	TRNG_RNG_DEBUG_EN_INPUT_RNG_DEBUG_EN_Msk = 0x1
	// Bit RNG_DEBUG_EN.
	TRNG_RNG_DEBUG_EN_INPUT_RNG_DEBUG_EN = 0x1

	// TRNG_BUSY: RNG Busy indication.
	// Position of RESERVED field.
	TRNG_TRNG_BUSY_RESERVED_Pos = 0x1
	// Bit mask of RESERVED field.
	TRNG_TRNG_BUSY_RESERVED_Msk = 0xfffffffe
	// Position of TRNG_BUSY field.
	TRNG_TRNG_BUSY_TRNG_BUSY_Pos = 0x0
	// Bit mask of TRNG_BUSY field.
	TRNG_TRNG_BUSY_TRNG_BUSY_Msk = 0x1
	// Bit TRNG_BUSY.
	TRNG_TRNG_BUSY_TRNG_BUSY = 0x1

	// RST_BITS_COUNTER: Reset the counter of collected bits in the RNG.
	// Position of RESERVED field.
	TRNG_RST_BITS_COUNTER_RESERVED_Pos = 0x1
	// Bit mask of RESERVED field.
	TRNG_RST_BITS_COUNTER_RESERVED_Msk = 0xfffffffe
	// Position of RST_BITS_COUNTER field.
	TRNG_RST_BITS_COUNTER_RST_BITS_COUNTER_Pos = 0x0
	// Bit mask of RST_BITS_COUNTER field.
	TRNG_RST_BITS_COUNTER_RST_BITS_COUNTER_Msk = 0x1
	// Bit RST_BITS_COUNTER.
	TRNG_RST_BITS_COUNTER_RST_BITS_COUNTER = 0x1

	// RNG_VERSION: Displays the version settings of the TRNG.
	// Position of RESERVED field.
	TRNG_RNG_VERSION_RESERVED_Pos = 0x8
	// Bit mask of RESERVED field.
	TRNG_RNG_VERSION_RESERVED_Msk = 0xffffff00
	// Position of RNG_USE_5_SBOXES field.
	TRNG_RNG_VERSION_RNG_USE_5_SBOXES_Pos = 0x7
	// Bit mask of RNG_USE_5_SBOXES field.
	TRNG_RNG_VERSION_RNG_USE_5_SBOXES_Msk = 0x80
	// Bit RNG_USE_5_SBOXES.
	TRNG_RNG_VERSION_RNG_USE_5_SBOXES = 0x80
	// Position of RESEEDING_EXISTS field.
	TRNG_RNG_VERSION_RESEEDING_EXISTS_Pos = 0x6
	// Bit mask of RESEEDING_EXISTS field.
	TRNG_RNG_VERSION_RESEEDING_EXISTS_Msk = 0x40
	// Bit RESEEDING_EXISTS.
	TRNG_RNG_VERSION_RESEEDING_EXISTS = 0x40
	// Position of KAT_EXISTS field.
	TRNG_RNG_VERSION_KAT_EXISTS_Pos = 0x5
	// Bit mask of KAT_EXISTS field.
	TRNG_RNG_VERSION_KAT_EXISTS_Msk = 0x20
	// Bit KAT_EXISTS.
	TRNG_RNG_VERSION_KAT_EXISTS = 0x20
	// Position of PRNG_EXISTS field.
	TRNG_RNG_VERSION_PRNG_EXISTS_Pos = 0x4
	// Bit mask of PRNG_EXISTS field.
	TRNG_RNG_VERSION_PRNG_EXISTS_Msk = 0x10
	// Bit PRNG_EXISTS.
	TRNG_RNG_VERSION_PRNG_EXISTS = 0x10
	// Position of TRNG_TESTS_BYPASS_EN field.
	TRNG_RNG_VERSION_TRNG_TESTS_BYPASS_EN_Pos = 0x3
	// Bit mask of TRNG_TESTS_BYPASS_EN field.
	TRNG_RNG_VERSION_TRNG_TESTS_BYPASS_EN_Msk = 0x8
	// Bit TRNG_TESTS_BYPASS_EN.
	TRNG_RNG_VERSION_TRNG_TESTS_BYPASS_EN = 0x8
	// Position of AUTOCORR_EXISTS field.
	TRNG_RNG_VERSION_AUTOCORR_EXISTS_Pos = 0x2
	// Bit mask of AUTOCORR_EXISTS field.
	TRNG_RNG_VERSION_AUTOCORR_EXISTS_Msk = 0x4
	// Bit AUTOCORR_EXISTS.
	TRNG_RNG_VERSION_AUTOCORR_EXISTS = 0x4
	// Position of CRNGT_EXISTS field.
	TRNG_RNG_VERSION_CRNGT_EXISTS_Pos = 0x1
	// Bit mask of CRNGT_EXISTS field.
	TRNG_RNG_VERSION_CRNGT_EXISTS_Msk = 0x2
	// Bit CRNGT_EXISTS.
	TRNG_RNG_VERSION_CRNGT_EXISTS = 0x2
	// Position of EHR_WIDTH_192 field.
	TRNG_RNG_VERSION_EHR_WIDTH_192_Pos = 0x0
	// Bit mask of EHR_WIDTH_192 field.
	TRNG_RNG_VERSION_EHR_WIDTH_192_Msk = 0x1
	// Bit EHR_WIDTH_192.
	TRNG_RNG_VERSION_EHR_WIDTH_192 = 0x1

	// RNG_BIST_CNTR_0: Collected BIST results.
	// Position of RESERVED field.
	TRNG_RNG_BIST_CNTR_0_RESERVED_Pos = 0x16
	// Bit mask of RESERVED field.
	TRNG_RNG_BIST_CNTR_0_RESERVED_Msk = 0xffc00000
	// Position of ROSC_CNTR_VAL field.
	TRNG_RNG_BIST_CNTR_0_ROSC_CNTR_VAL_Pos = 0x0
	// Bit mask of ROSC_CNTR_VAL field.
	TRNG_RNG_BIST_CNTR_0_ROSC_CNTR_VAL_Msk = 0x3fffff

	// RNG_BIST_CNTR_1: Collected BIST results.
	// Position of RESERVED field.
	TRNG_RNG_BIST_CNTR_1_RESERVED_Pos = 0x16
	// Bit mask of RESERVED field.
	TRNG_RNG_BIST_CNTR_1_RESERVED_Msk = 0xffc00000
	// Position of ROSC_CNTR_VAL field.
	TRNG_RNG_BIST_CNTR_1_ROSC_CNTR_VAL_Pos = 0x0
	// Bit mask of ROSC_CNTR_VAL field.
	TRNG_RNG_BIST_CNTR_1_ROSC_CNTR_VAL_Msk = 0x3fffff

	// RNG_BIST_CNTR_2: Collected BIST results.
	// Position of RESERVED field.
	TRNG_RNG_BIST_CNTR_2_RESERVED_Pos = 0x16
	// Bit mask of RESERVED field.
	TRNG_RNG_BIST_CNTR_2_RESERVED_Msk = 0xffc00000
	// Position of ROSC_CNTR_VAL field.
	TRNG_RNG_BIST_CNTR_2_ROSC_CNTR_VAL_Pos = 0x0
	// Bit mask of ROSC_CNTR_VAL field.
	TRNG_RNG_BIST_CNTR_2_ROSC_CNTR_VAL_Msk = 0x3fffff
)

// Constants for GLITCH_DETECTOR: Glitch detector controls
const (
	// ARM: Forcibly arm the glitch detectors, if they are not already armed by OTP. When armed, any individual detector trigger will cause a restart of the switched core power domain's power-on reset state machine. Glitch detector triggers are recorded accumulatively in TRIG_STATUS. If the system is reset by a glitch detector trigger, this is recorded in POWMAN_CHIP_RESET. This register is Secure read/write only.
	// Position of ARM field.
	GLITCH_DETECTOR_ARM_ARM_Pos = 0x0
	// Bit mask of ARM field.
	GLITCH_DETECTOR_ARM_ARM_Msk = 0xffff
	// Do not force the glitch detectors to be armed
	GLITCH_DETECTOR_ARM_ARM_NO = 0x5bad
	// Force the glitch detectors to be armed. (Any value other than ARM_NO counts as YES)
	GLITCH_DETECTOR_ARM_ARM_YES = 0x0

	// DISARM
	// Position of DISARM field.
	GLITCH_DETECTOR_DISARM_DISARM_Pos = 0x0
	// Bit mask of DISARM field.
	GLITCH_DETECTOR_DISARM_DISARM_Msk = 0xffff
	// Do not disarm the glitch detectors. (Any value other than DISARM_YES counts as NO)
	GLITCH_DETECTOR_DISARM_DISARM_NO = 0x0
	// Disarm the glitch detectors
	GLITCH_DETECTOR_DISARM_DISARM_YES = 0xdcaf

	// SENSITIVITY: Adjust the sensitivity of glitch detectors to values other than their OTP-provided defaults. This register is Secure read/write only.
	// Position of DEFAULT field.
	GLITCH_DETECTOR_SENSITIVITY_DEFAULT_Pos = 0x18
	// Bit mask of DEFAULT field.
	GLITCH_DETECTOR_SENSITIVITY_DEFAULT_Msk = 0xff000000
	// Use the default sensitivity configured in OTP for all detectors. (Any value other than DEFAULT_NO counts as YES)
	GLITCH_DETECTOR_SENSITIVITY_DEFAULT_YES = 0x0
	// Do not use the default sensitivity configured in OTP. Instead use the value from this register.
	GLITCH_DETECTOR_SENSITIVITY_DEFAULT_NO = 0xde
	// Position of DET3_INV field.
	GLITCH_DETECTOR_SENSITIVITY_DET3_INV_Pos = 0xe
	// Bit mask of DET3_INV field.
	GLITCH_DETECTOR_SENSITIVITY_DET3_INV_Msk = 0xc000
	// Position of DET2_INV field.
	GLITCH_DETECTOR_SENSITIVITY_DET2_INV_Pos = 0xc
	// Bit mask of DET2_INV field.
	GLITCH_DETECTOR_SENSITIVITY_DET2_INV_Msk = 0x3000
	// Position of DET1_INV field.
	GLITCH_DETECTOR_SENSITIVITY_DET1_INV_Pos = 0xa
	// Bit mask of DET1_INV field.
	GLITCH_DETECTOR_SENSITIVITY_DET1_INV_Msk = 0xc00
	// Position of DET0_INV field.
	GLITCH_DETECTOR_SENSITIVITY_DET0_INV_Pos = 0x8
	// Bit mask of DET0_INV field.
	GLITCH_DETECTOR_SENSITIVITY_DET0_INV_Msk = 0x300
	// Position of DET3 field.
	GLITCH_DETECTOR_SENSITIVITY_DET3_Pos = 0x6
	// Bit mask of DET3 field.
	GLITCH_DETECTOR_SENSITIVITY_DET3_Msk = 0xc0
	// Position of DET2 field.
	GLITCH_DETECTOR_SENSITIVITY_DET2_Pos = 0x4
	// Bit mask of DET2 field.
	GLITCH_DETECTOR_SENSITIVITY_DET2_Msk = 0x30
	// Position of DET1 field.
	GLITCH_DETECTOR_SENSITIVITY_DET1_Pos = 0x2
	// Bit mask of DET1 field.
	GLITCH_DETECTOR_SENSITIVITY_DET1_Msk = 0xc
	// Position of DET0 field.
	GLITCH_DETECTOR_SENSITIVITY_DET0_Pos = 0x0
	// Bit mask of DET0 field.
	GLITCH_DETECTOR_SENSITIVITY_DET0_Msk = 0x3

	// LOCK
	// Position of LOCK field.
	GLITCH_DETECTOR_LOCK_LOCK_Pos = 0x0
	// Bit mask of LOCK field.
	GLITCH_DETECTOR_LOCK_LOCK_Msk = 0xff

	// TRIG_STATUS: Set when a detector output triggers. Write-1-clear. (May immediately return high if the detector remains in a failed state. Detectors can only be cleared by a full reset of the switched core power domain.) This register is Secure read/write only.
	// Position of DET3 field.
	GLITCH_DETECTOR_TRIG_STATUS_DET3_Pos = 0x3
	// Bit mask of DET3 field.
	GLITCH_DETECTOR_TRIG_STATUS_DET3_Msk = 0x8
	// Bit DET3.
	GLITCH_DETECTOR_TRIG_STATUS_DET3 = 0x8
	// Position of DET2 field.
	GLITCH_DETECTOR_TRIG_STATUS_DET2_Pos = 0x2
	// Bit mask of DET2 field.
	GLITCH_DETECTOR_TRIG_STATUS_DET2_Msk = 0x4
	// Bit DET2.
	GLITCH_DETECTOR_TRIG_STATUS_DET2 = 0x4
	// Position of DET1 field.
	GLITCH_DETECTOR_TRIG_STATUS_DET1_Pos = 0x1
	// Bit mask of DET1 field.
	GLITCH_DETECTOR_TRIG_STATUS_DET1_Msk = 0x2
	// Bit DET1.
	GLITCH_DETECTOR_TRIG_STATUS_DET1 = 0x2
	// Position of DET0 field.
	GLITCH_DETECTOR_TRIG_STATUS_DET0_Pos = 0x0
	// Bit mask of DET0 field.
	GLITCH_DETECTOR_TRIG_STATUS_DET0_Msk = 0x1
	// Bit DET0.
	GLITCH_DETECTOR_TRIG_STATUS_DET0 = 0x1

	// TRIG_FORCE: Simulate the firing of one or more detectors. Writing ones to this register will set the matching bits in STATUS_TRIG. If the glitch detectors are currently armed, writing ones will also immediately reset the switched core power domain, and set the reset reason latches in POWMAN_CHIP_RESET to indicate a glitch detector resets. This register is Secure read/write only.
	// Position of TRIG_FORCE field.
	GLITCH_DETECTOR_TRIG_FORCE_TRIG_FORCE_Pos = 0x0
	// Bit mask of TRIG_FORCE field.
	GLITCH_DETECTOR_TRIG_FORCE_TRIG_FORCE_Msk = 0xf
)

// Constants for OTP: SNPS OTP control IF (SBPI and RPi wrapper control)
const (
	// SW_LOCK0: Software lock register for page 0. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK0_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK0_NSEC_Msk          = 0xc
	OTP_SW_LOCK0_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK0_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK0_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK0_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK0_SEC_Msk          = 0x3
	OTP_SW_LOCK0_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK0_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK0_SEC_INACCESSIBLE = 0x3

	// SW_LOCK1: Software lock register for page 1. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK1_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK1_NSEC_Msk          = 0xc
	OTP_SW_LOCK1_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK1_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK1_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK1_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK1_SEC_Msk          = 0x3
	OTP_SW_LOCK1_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK1_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK1_SEC_INACCESSIBLE = 0x3

	// SW_LOCK2: Software lock register for page 2. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK2_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK2_NSEC_Msk          = 0xc
	OTP_SW_LOCK2_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK2_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK2_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK2_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK2_SEC_Msk          = 0x3
	OTP_SW_LOCK2_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK2_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK2_SEC_INACCESSIBLE = 0x3

	// SW_LOCK3: Software lock register for page 3. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK3_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK3_NSEC_Msk          = 0xc
	OTP_SW_LOCK3_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK3_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK3_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK3_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK3_SEC_Msk          = 0x3
	OTP_SW_LOCK3_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK3_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK3_SEC_INACCESSIBLE = 0x3

	// SW_LOCK4: Software lock register for page 4. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK4_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK4_NSEC_Msk          = 0xc
	OTP_SW_LOCK4_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK4_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK4_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK4_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK4_SEC_Msk          = 0x3
	OTP_SW_LOCK4_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK4_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK4_SEC_INACCESSIBLE = 0x3

	// SW_LOCK5: Software lock register for page 5. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK5_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK5_NSEC_Msk          = 0xc
	OTP_SW_LOCK5_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK5_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK5_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK5_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK5_SEC_Msk          = 0x3
	OTP_SW_LOCK5_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK5_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK5_SEC_INACCESSIBLE = 0x3

	// SW_LOCK6: Software lock register for page 6. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK6_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK6_NSEC_Msk          = 0xc
	OTP_SW_LOCK6_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK6_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK6_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK6_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK6_SEC_Msk          = 0x3
	OTP_SW_LOCK6_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK6_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK6_SEC_INACCESSIBLE = 0x3

	// SW_LOCK7: Software lock register for page 7. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK7_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK7_NSEC_Msk          = 0xc
	OTP_SW_LOCK7_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK7_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK7_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK7_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK7_SEC_Msk          = 0x3
	OTP_SW_LOCK7_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK7_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK7_SEC_INACCESSIBLE = 0x3

	// SW_LOCK8: Software lock register for page 8. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK8_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK8_NSEC_Msk          = 0xc
	OTP_SW_LOCK8_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK8_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK8_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK8_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK8_SEC_Msk          = 0x3
	OTP_SW_LOCK8_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK8_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK8_SEC_INACCESSIBLE = 0x3

	// SW_LOCK9: Software lock register for page 9. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK9_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK9_NSEC_Msk          = 0xc
	OTP_SW_LOCK9_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK9_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK9_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK9_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK9_SEC_Msk          = 0x3
	OTP_SW_LOCK9_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK9_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK9_SEC_INACCESSIBLE = 0x3

	// SW_LOCK10: Software lock register for page 10. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK10_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK10_NSEC_Msk          = 0xc
	OTP_SW_LOCK10_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK10_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK10_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK10_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK10_SEC_Msk          = 0x3
	OTP_SW_LOCK10_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK10_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK10_SEC_INACCESSIBLE = 0x3

	// SW_LOCK11: Software lock register for page 11. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK11_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK11_NSEC_Msk          = 0xc
	OTP_SW_LOCK11_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK11_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK11_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK11_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK11_SEC_Msk          = 0x3
	OTP_SW_LOCK11_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK11_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK11_SEC_INACCESSIBLE = 0x3

	// SW_LOCK12: Software lock register for page 12. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK12_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK12_NSEC_Msk          = 0xc
	OTP_SW_LOCK12_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK12_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK12_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK12_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK12_SEC_Msk          = 0x3
	OTP_SW_LOCK12_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK12_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK12_SEC_INACCESSIBLE = 0x3

	// SW_LOCK13: Software lock register for page 13. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK13_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK13_NSEC_Msk          = 0xc
	OTP_SW_LOCK13_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK13_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK13_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK13_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK13_SEC_Msk          = 0x3
	OTP_SW_LOCK13_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK13_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK13_SEC_INACCESSIBLE = 0x3

	// SW_LOCK14: Software lock register for page 14. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK14_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK14_NSEC_Msk          = 0xc
	OTP_SW_LOCK14_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK14_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK14_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK14_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK14_SEC_Msk          = 0x3
	OTP_SW_LOCK14_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK14_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK14_SEC_INACCESSIBLE = 0x3

	// SW_LOCK15: Software lock register for page 15. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK15_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK15_NSEC_Msk          = 0xc
	OTP_SW_LOCK15_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK15_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK15_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK15_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK15_SEC_Msk          = 0x3
	OTP_SW_LOCK15_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK15_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK15_SEC_INACCESSIBLE = 0x3

	// SW_LOCK16: Software lock register for page 16. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK16_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK16_NSEC_Msk          = 0xc
	OTP_SW_LOCK16_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK16_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK16_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK16_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK16_SEC_Msk          = 0x3
	OTP_SW_LOCK16_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK16_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK16_SEC_INACCESSIBLE = 0x3

	// SW_LOCK17: Software lock register for page 17. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK17_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK17_NSEC_Msk          = 0xc
	OTP_SW_LOCK17_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK17_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK17_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK17_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK17_SEC_Msk          = 0x3
	OTP_SW_LOCK17_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK17_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK17_SEC_INACCESSIBLE = 0x3

	// SW_LOCK18: Software lock register for page 18. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK18_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK18_NSEC_Msk          = 0xc
	OTP_SW_LOCK18_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK18_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK18_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK18_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK18_SEC_Msk          = 0x3
	OTP_SW_LOCK18_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK18_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK18_SEC_INACCESSIBLE = 0x3

	// SW_LOCK19: Software lock register for page 19. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK19_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK19_NSEC_Msk          = 0xc
	OTP_SW_LOCK19_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK19_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK19_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK19_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK19_SEC_Msk          = 0x3
	OTP_SW_LOCK19_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK19_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK19_SEC_INACCESSIBLE = 0x3

	// SW_LOCK20: Software lock register for page 20. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK20_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK20_NSEC_Msk          = 0xc
	OTP_SW_LOCK20_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK20_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK20_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK20_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK20_SEC_Msk          = 0x3
	OTP_SW_LOCK20_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK20_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK20_SEC_INACCESSIBLE = 0x3

	// SW_LOCK21: Software lock register for page 21. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK21_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK21_NSEC_Msk          = 0xc
	OTP_SW_LOCK21_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK21_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK21_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK21_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK21_SEC_Msk          = 0x3
	OTP_SW_LOCK21_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK21_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK21_SEC_INACCESSIBLE = 0x3

	// SW_LOCK22: Software lock register for page 22. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK22_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK22_NSEC_Msk          = 0xc
	OTP_SW_LOCK22_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK22_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK22_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK22_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK22_SEC_Msk          = 0x3
	OTP_SW_LOCK22_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK22_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK22_SEC_INACCESSIBLE = 0x3

	// SW_LOCK23: Software lock register for page 23. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK23_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK23_NSEC_Msk          = 0xc
	OTP_SW_LOCK23_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK23_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK23_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK23_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK23_SEC_Msk          = 0x3
	OTP_SW_LOCK23_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK23_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK23_SEC_INACCESSIBLE = 0x3

	// SW_LOCK24: Software lock register for page 24. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK24_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK24_NSEC_Msk          = 0xc
	OTP_SW_LOCK24_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK24_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK24_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK24_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK24_SEC_Msk          = 0x3
	OTP_SW_LOCK24_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK24_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK24_SEC_INACCESSIBLE = 0x3

	// SW_LOCK25: Software lock register for page 25. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK25_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK25_NSEC_Msk          = 0xc
	OTP_SW_LOCK25_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK25_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK25_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK25_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK25_SEC_Msk          = 0x3
	OTP_SW_LOCK25_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK25_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK25_SEC_INACCESSIBLE = 0x3

	// SW_LOCK26: Software lock register for page 26. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK26_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK26_NSEC_Msk          = 0xc
	OTP_SW_LOCK26_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK26_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK26_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK26_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK26_SEC_Msk          = 0x3
	OTP_SW_LOCK26_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK26_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK26_SEC_INACCESSIBLE = 0x3

	// SW_LOCK27: Software lock register for page 27. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK27_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK27_NSEC_Msk          = 0xc
	OTP_SW_LOCK27_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK27_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK27_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK27_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK27_SEC_Msk          = 0x3
	OTP_SW_LOCK27_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK27_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK27_SEC_INACCESSIBLE = 0x3

	// SW_LOCK28: Software lock register for page 28. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK28_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK28_NSEC_Msk          = 0xc
	OTP_SW_LOCK28_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK28_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK28_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK28_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK28_SEC_Msk          = 0x3
	OTP_SW_LOCK28_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK28_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK28_SEC_INACCESSIBLE = 0x3

	// SW_LOCK29: Software lock register for page 29. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK29_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK29_NSEC_Msk          = 0xc
	OTP_SW_LOCK29_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK29_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK29_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK29_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK29_SEC_Msk          = 0x3
	OTP_SW_LOCK29_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK29_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK29_SEC_INACCESSIBLE = 0x3

	// SW_LOCK30: Software lock register for page 30. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK30_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK30_NSEC_Msk          = 0xc
	OTP_SW_LOCK30_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK30_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK30_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK30_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK30_SEC_Msk          = 0x3
	OTP_SW_LOCK30_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK30_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK30_SEC_INACCESSIBLE = 0x3

	// SW_LOCK31: Software lock register for page 31. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK31_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK31_NSEC_Msk          = 0xc
	OTP_SW_LOCK31_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK31_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK31_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK31_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK31_SEC_Msk          = 0x3
	OTP_SW_LOCK31_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK31_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK31_SEC_INACCESSIBLE = 0x3

	// SW_LOCK32: Software lock register for page 32. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK32_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK32_NSEC_Msk          = 0xc
	OTP_SW_LOCK32_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK32_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK32_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK32_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK32_SEC_Msk          = 0x3
	OTP_SW_LOCK32_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK32_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK32_SEC_INACCESSIBLE = 0x3

	// SW_LOCK33: Software lock register for page 33. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK33_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK33_NSEC_Msk          = 0xc
	OTP_SW_LOCK33_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK33_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK33_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK33_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK33_SEC_Msk          = 0x3
	OTP_SW_LOCK33_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK33_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK33_SEC_INACCESSIBLE = 0x3

	// SW_LOCK34: Software lock register for page 34. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK34_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK34_NSEC_Msk          = 0xc
	OTP_SW_LOCK34_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK34_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK34_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK34_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK34_SEC_Msk          = 0x3
	OTP_SW_LOCK34_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK34_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK34_SEC_INACCESSIBLE = 0x3

	// SW_LOCK35: Software lock register for page 35. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK35_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK35_NSEC_Msk          = 0xc
	OTP_SW_LOCK35_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK35_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK35_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK35_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK35_SEC_Msk          = 0x3
	OTP_SW_LOCK35_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK35_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK35_SEC_INACCESSIBLE = 0x3

	// SW_LOCK36: Software lock register for page 36. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK36_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK36_NSEC_Msk          = 0xc
	OTP_SW_LOCK36_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK36_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK36_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK36_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK36_SEC_Msk          = 0x3
	OTP_SW_LOCK36_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK36_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK36_SEC_INACCESSIBLE = 0x3

	// SW_LOCK37: Software lock register for page 37. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK37_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK37_NSEC_Msk          = 0xc
	OTP_SW_LOCK37_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK37_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK37_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK37_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK37_SEC_Msk          = 0x3
	OTP_SW_LOCK37_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK37_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK37_SEC_INACCESSIBLE = 0x3

	// SW_LOCK38: Software lock register for page 38. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK38_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK38_NSEC_Msk          = 0xc
	OTP_SW_LOCK38_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK38_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK38_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK38_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK38_SEC_Msk          = 0x3
	OTP_SW_LOCK38_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK38_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK38_SEC_INACCESSIBLE = 0x3

	// SW_LOCK39: Software lock register for page 39. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK39_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK39_NSEC_Msk          = 0xc
	OTP_SW_LOCK39_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK39_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK39_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK39_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK39_SEC_Msk          = 0x3
	OTP_SW_LOCK39_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK39_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK39_SEC_INACCESSIBLE = 0x3

	// SW_LOCK40: Software lock register for page 40. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK40_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK40_NSEC_Msk          = 0xc
	OTP_SW_LOCK40_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK40_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK40_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK40_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK40_SEC_Msk          = 0x3
	OTP_SW_LOCK40_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK40_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK40_SEC_INACCESSIBLE = 0x3

	// SW_LOCK41: Software lock register for page 41. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK41_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK41_NSEC_Msk          = 0xc
	OTP_SW_LOCK41_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK41_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK41_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK41_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK41_SEC_Msk          = 0x3
	OTP_SW_LOCK41_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK41_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK41_SEC_INACCESSIBLE = 0x3

	// SW_LOCK42: Software lock register for page 42. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK42_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK42_NSEC_Msk          = 0xc
	OTP_SW_LOCK42_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK42_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK42_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK42_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK42_SEC_Msk          = 0x3
	OTP_SW_LOCK42_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK42_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK42_SEC_INACCESSIBLE = 0x3

	// SW_LOCK43: Software lock register for page 43. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK43_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK43_NSEC_Msk          = 0xc
	OTP_SW_LOCK43_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK43_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK43_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK43_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK43_SEC_Msk          = 0x3
	OTP_SW_LOCK43_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK43_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK43_SEC_INACCESSIBLE = 0x3

	// SW_LOCK44: Software lock register for page 44. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK44_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK44_NSEC_Msk          = 0xc
	OTP_SW_LOCK44_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK44_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK44_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK44_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK44_SEC_Msk          = 0x3
	OTP_SW_LOCK44_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK44_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK44_SEC_INACCESSIBLE = 0x3

	// SW_LOCK45: Software lock register for page 45. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK45_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK45_NSEC_Msk          = 0xc
	OTP_SW_LOCK45_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK45_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK45_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK45_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK45_SEC_Msk          = 0x3
	OTP_SW_LOCK45_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK45_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK45_SEC_INACCESSIBLE = 0x3

	// SW_LOCK46: Software lock register for page 46. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK46_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK46_NSEC_Msk          = 0xc
	OTP_SW_LOCK46_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK46_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK46_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK46_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK46_SEC_Msk          = 0x3
	OTP_SW_LOCK46_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK46_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK46_SEC_INACCESSIBLE = 0x3

	// SW_LOCK47: Software lock register for page 47. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK47_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK47_NSEC_Msk          = 0xc
	OTP_SW_LOCK47_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK47_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK47_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK47_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK47_SEC_Msk          = 0x3
	OTP_SW_LOCK47_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK47_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK47_SEC_INACCESSIBLE = 0x3

	// SW_LOCK48: Software lock register for page 48. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK48_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK48_NSEC_Msk          = 0xc
	OTP_SW_LOCK48_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK48_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK48_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK48_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK48_SEC_Msk          = 0x3
	OTP_SW_LOCK48_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK48_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK48_SEC_INACCESSIBLE = 0x3

	// SW_LOCK49: Software lock register for page 49. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK49_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK49_NSEC_Msk          = 0xc
	OTP_SW_LOCK49_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK49_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK49_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK49_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK49_SEC_Msk          = 0x3
	OTP_SW_LOCK49_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK49_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK49_SEC_INACCESSIBLE = 0x3

	// SW_LOCK50: Software lock register for page 50. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK50_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK50_NSEC_Msk          = 0xc
	OTP_SW_LOCK50_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK50_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK50_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK50_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK50_SEC_Msk          = 0x3
	OTP_SW_LOCK50_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK50_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK50_SEC_INACCESSIBLE = 0x3

	// SW_LOCK51: Software lock register for page 51. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK51_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK51_NSEC_Msk          = 0xc
	OTP_SW_LOCK51_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK51_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK51_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK51_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK51_SEC_Msk          = 0x3
	OTP_SW_LOCK51_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK51_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK51_SEC_INACCESSIBLE = 0x3

	// SW_LOCK52: Software lock register for page 52. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK52_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK52_NSEC_Msk          = 0xc
	OTP_SW_LOCK52_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK52_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK52_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK52_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK52_SEC_Msk          = 0x3
	OTP_SW_LOCK52_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK52_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK52_SEC_INACCESSIBLE = 0x3

	// SW_LOCK53: Software lock register for page 53. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK53_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK53_NSEC_Msk          = 0xc
	OTP_SW_LOCK53_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK53_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK53_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK53_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK53_SEC_Msk          = 0x3
	OTP_SW_LOCK53_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK53_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK53_SEC_INACCESSIBLE = 0x3

	// SW_LOCK54: Software lock register for page 54. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK54_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK54_NSEC_Msk          = 0xc
	OTP_SW_LOCK54_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK54_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK54_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK54_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK54_SEC_Msk          = 0x3
	OTP_SW_LOCK54_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK54_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK54_SEC_INACCESSIBLE = 0x3

	// SW_LOCK55: Software lock register for page 55. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK55_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK55_NSEC_Msk          = 0xc
	OTP_SW_LOCK55_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK55_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK55_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK55_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK55_SEC_Msk          = 0x3
	OTP_SW_LOCK55_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK55_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK55_SEC_INACCESSIBLE = 0x3

	// SW_LOCK56: Software lock register for page 56. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK56_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK56_NSEC_Msk          = 0xc
	OTP_SW_LOCK56_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK56_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK56_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK56_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK56_SEC_Msk          = 0x3
	OTP_SW_LOCK56_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK56_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK56_SEC_INACCESSIBLE = 0x3

	// SW_LOCK57: Software lock register for page 57. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK57_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK57_NSEC_Msk          = 0xc
	OTP_SW_LOCK57_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK57_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK57_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK57_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK57_SEC_Msk          = 0x3
	OTP_SW_LOCK57_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK57_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK57_SEC_INACCESSIBLE = 0x3

	// SW_LOCK58: Software lock register for page 58. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK58_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK58_NSEC_Msk          = 0xc
	OTP_SW_LOCK58_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK58_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK58_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK58_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK58_SEC_Msk          = 0x3
	OTP_SW_LOCK58_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK58_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK58_SEC_INACCESSIBLE = 0x3

	// SW_LOCK59: Software lock register for page 59. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK59_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK59_NSEC_Msk          = 0xc
	OTP_SW_LOCK59_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK59_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK59_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK59_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK59_SEC_Msk          = 0x3
	OTP_SW_LOCK59_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK59_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK59_SEC_INACCESSIBLE = 0x3

	// SW_LOCK60: Software lock register for page 60. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK60_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK60_NSEC_Msk          = 0xc
	OTP_SW_LOCK60_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK60_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK60_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK60_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK60_SEC_Msk          = 0x3
	OTP_SW_LOCK60_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK60_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK60_SEC_INACCESSIBLE = 0x3

	// SW_LOCK61: Software lock register for page 61. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK61_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK61_NSEC_Msk          = 0xc
	OTP_SW_LOCK61_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK61_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK61_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK61_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK61_SEC_Msk          = 0x3
	OTP_SW_LOCK61_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK61_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK61_SEC_INACCESSIBLE = 0x3

	// SW_LOCK62: Software lock register for page 62. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK62_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK62_NSEC_Msk          = 0xc
	OTP_SW_LOCK62_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK62_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK62_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK62_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK62_SEC_Msk          = 0x3
	OTP_SW_LOCK62_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK62_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK62_SEC_INACCESSIBLE = 0x3

	// SW_LOCK63: Software lock register for page 63. Locks are initialised from the OTP lock pages at reset. This register can be written to further advance the lock state of each page (until next reset), and read to check the current lock state of a page.
	// Position of NSEC field.
	OTP_SW_LOCK63_NSEC_Pos = 0x2
	// Bit mask of NSEC field.
	OTP_SW_LOCK63_NSEC_Msk          = 0xc
	OTP_SW_LOCK63_NSEC_READ_WRITE   = 0x0
	OTP_SW_LOCK63_NSEC_READ_ONLY    = 0x1
	OTP_SW_LOCK63_NSEC_INACCESSIBLE = 0x3
	// Position of SEC field.
	OTP_SW_LOCK63_SEC_Pos = 0x0
	// Bit mask of SEC field.
	OTP_SW_LOCK63_SEC_Msk          = 0x3
	OTP_SW_LOCK63_SEC_READ_WRITE   = 0x0
	OTP_SW_LOCK63_SEC_READ_ONLY    = 0x1
	OTP_SW_LOCK63_SEC_INACCESSIBLE = 0x3

	// SBPI_INSTR: Dispatch instructions to the SBPI interface, used for programming the OTP fuses.
	// Position of EXEC field.
	OTP_SBPI_INSTR_EXEC_Pos = 0x1e
	// Bit mask of EXEC field.
	OTP_SBPI_INSTR_EXEC_Msk = 0x40000000
	// Bit EXEC.
	OTP_SBPI_INSTR_EXEC = 0x40000000
	// Position of IS_WR field.
	OTP_SBPI_INSTR_IS_WR_Pos = 0x1d
	// Bit mask of IS_WR field.
	OTP_SBPI_INSTR_IS_WR_Msk = 0x20000000
	// Bit IS_WR.
	OTP_SBPI_INSTR_IS_WR = 0x20000000
	// Position of HAS_PAYLOAD field.
	OTP_SBPI_INSTR_HAS_PAYLOAD_Pos = 0x1c
	// Bit mask of HAS_PAYLOAD field.
	OTP_SBPI_INSTR_HAS_PAYLOAD_Msk = 0x10000000
	// Bit HAS_PAYLOAD.
	OTP_SBPI_INSTR_HAS_PAYLOAD = 0x10000000
	// Position of PAYLOAD_SIZE_M1 field.
	OTP_SBPI_INSTR_PAYLOAD_SIZE_M1_Pos = 0x18
	// Bit mask of PAYLOAD_SIZE_M1 field.
	OTP_SBPI_INSTR_PAYLOAD_SIZE_M1_Msk = 0xf000000
	// Position of TARGET field.
	OTP_SBPI_INSTR_TARGET_Pos = 0x10
	// Bit mask of TARGET field.
	OTP_SBPI_INSTR_TARGET_Msk = 0xff0000
	// Position of CMD field.
	OTP_SBPI_INSTR_CMD_Pos = 0x8
	// Bit mask of CMD field.
	OTP_SBPI_INSTR_CMD_Msk = 0xff00
	// Position of SHORT_WDATA field.
	OTP_SBPI_INSTR_SHORT_WDATA_Pos = 0x0
	// Bit mask of SHORT_WDATA field.
	OTP_SBPI_INSTR_SHORT_WDATA_Msk = 0xff

	// SBPI_WDATA_0: SBPI write payload bytes 3..0
	// Position of SBPI_WDATA_0 field.
	OTP_SBPI_WDATA_0_SBPI_WDATA_0_Pos = 0x0
	// Bit mask of SBPI_WDATA_0 field.
	OTP_SBPI_WDATA_0_SBPI_WDATA_0_Msk = 0xffffffff

	// SBPI_WDATA_1: SBPI write payload bytes 7..4
	// Position of SBPI_WDATA_1 field.
	OTP_SBPI_WDATA_1_SBPI_WDATA_1_Pos = 0x0
	// Bit mask of SBPI_WDATA_1 field.
	OTP_SBPI_WDATA_1_SBPI_WDATA_1_Msk = 0xffffffff

	// SBPI_WDATA_2: SBPI write payload bytes 11..8
	// Position of SBPI_WDATA_2 field.
	OTP_SBPI_WDATA_2_SBPI_WDATA_2_Pos = 0x0
	// Bit mask of SBPI_WDATA_2 field.
	OTP_SBPI_WDATA_2_SBPI_WDATA_2_Msk = 0xffffffff

	// SBPI_WDATA_3: SBPI write payload bytes 15..12
	// Position of SBPI_WDATA_3 field.
	OTP_SBPI_WDATA_3_SBPI_WDATA_3_Pos = 0x0
	// Bit mask of SBPI_WDATA_3 field.
	OTP_SBPI_WDATA_3_SBPI_WDATA_3_Msk = 0xffffffff

	// SBPI_RDATA_0: Read payload bytes 3..0. Once read, the data in the register will automatically clear to 0.
	// Position of SBPI_RDATA_0 field.
	OTP_SBPI_RDATA_0_SBPI_RDATA_0_Pos = 0x0
	// Bit mask of SBPI_RDATA_0 field.
	OTP_SBPI_RDATA_0_SBPI_RDATA_0_Msk = 0xffffffff

	// SBPI_RDATA_1: Read payload bytes 7..4. Once read, the data in the register will automatically clear to 0.
	// Position of SBPI_RDATA_1 field.
	OTP_SBPI_RDATA_1_SBPI_RDATA_1_Pos = 0x0
	// Bit mask of SBPI_RDATA_1 field.
	OTP_SBPI_RDATA_1_SBPI_RDATA_1_Msk = 0xffffffff

	// SBPI_RDATA_2: Read payload bytes 11..8. Once read, the data in the register will automatically clear to 0.
	// Position of SBPI_RDATA_2 field.
	OTP_SBPI_RDATA_2_SBPI_RDATA_2_Pos = 0x0
	// Bit mask of SBPI_RDATA_2 field.
	OTP_SBPI_RDATA_2_SBPI_RDATA_2_Msk = 0xffffffff

	// SBPI_RDATA_3: Read payload bytes 15..12. Once read, the data in the register will automatically clear to 0.
	// Position of SBPI_RDATA_3 field.
	OTP_SBPI_RDATA_3_SBPI_RDATA_3_Pos = 0x0
	// Bit mask of SBPI_RDATA_3 field.
	OTP_SBPI_RDATA_3_SBPI_RDATA_3_Msk = 0xffffffff

	// SBPI_STATUS
	// Position of MISO field.
	OTP_SBPI_STATUS_MISO_Pos = 0x10
	// Bit mask of MISO field.
	OTP_SBPI_STATUS_MISO_Msk = 0xff0000
	// Position of FLAG field.
	OTP_SBPI_STATUS_FLAG_Pos = 0xc
	// Bit mask of FLAG field.
	OTP_SBPI_STATUS_FLAG_Msk = 0x1000
	// Bit FLAG.
	OTP_SBPI_STATUS_FLAG = 0x1000
	// Position of INSTR_MISS field.
	OTP_SBPI_STATUS_INSTR_MISS_Pos = 0x8
	// Bit mask of INSTR_MISS field.
	OTP_SBPI_STATUS_INSTR_MISS_Msk = 0x100
	// Bit INSTR_MISS.
	OTP_SBPI_STATUS_INSTR_MISS = 0x100
	// Position of INSTR_DONE field.
	OTP_SBPI_STATUS_INSTR_DONE_Pos = 0x4
	// Bit mask of INSTR_DONE field.
	OTP_SBPI_STATUS_INSTR_DONE_Msk = 0x10
	// Bit INSTR_DONE.
	OTP_SBPI_STATUS_INSTR_DONE = 0x10
	// Position of RDATA_VLD field.
	OTP_SBPI_STATUS_RDATA_VLD_Pos = 0x0
	// Bit mask of RDATA_VLD field.
	OTP_SBPI_STATUS_RDATA_VLD_Msk = 0x1
	// Bit RDATA_VLD.
	OTP_SBPI_STATUS_RDATA_VLD = 0x1

	// USR: Controls for APB data read interface (USER interface)
	// Position of PD field.
	OTP_USR_PD_Pos = 0x4
	// Bit mask of PD field.
	OTP_USR_PD_Msk = 0x10
	// Bit PD.
	OTP_USR_PD = 0x10
	// Position of DCTRL field.
	OTP_USR_DCTRL_Pos = 0x0
	// Bit mask of DCTRL field.
	OTP_USR_DCTRL_Msk = 0x1
	// Bit DCTRL.
	OTP_USR_DCTRL = 0x1

	// DBG: Debug for OTP power-on state machine
	// Position of CUSTOMER_RMA_FLAG field.
	OTP_DBG_CUSTOMER_RMA_FLAG_Pos = 0xc
	// Bit mask of CUSTOMER_RMA_FLAG field.
	OTP_DBG_CUSTOMER_RMA_FLAG_Msk = 0x1000
	// Bit CUSTOMER_RMA_FLAG.
	OTP_DBG_CUSTOMER_RMA_FLAG = 0x1000
	// Position of PSM_STATE field.
	OTP_DBG_PSM_STATE_Pos = 0x4
	// Bit mask of PSM_STATE field.
	OTP_DBG_PSM_STATE_Msk = 0xf0
	// Position of ROSC_UP field.
	OTP_DBG_ROSC_UP_Pos = 0x3
	// Bit mask of ROSC_UP field.
	OTP_DBG_ROSC_UP_Msk = 0x8
	// Bit ROSC_UP.
	OTP_DBG_ROSC_UP = 0x8
	// Position of ROSC_UP_SEEN field.
	OTP_DBG_ROSC_UP_SEEN_Pos = 0x2
	// Bit mask of ROSC_UP_SEEN field.
	OTP_DBG_ROSC_UP_SEEN_Msk = 0x4
	// Bit ROSC_UP_SEEN.
	OTP_DBG_ROSC_UP_SEEN = 0x4
	// Position of BOOT_DONE field.
	OTP_DBG_BOOT_DONE_Pos = 0x1
	// Bit mask of BOOT_DONE field.
	OTP_DBG_BOOT_DONE_Msk = 0x2
	// Bit BOOT_DONE.
	OTP_DBG_BOOT_DONE = 0x2
	// Position of PSM_DONE field.
	OTP_DBG_PSM_DONE_Pos = 0x0
	// Bit mask of PSM_DONE field.
	OTP_DBG_PSM_DONE_Msk = 0x1
	// Bit PSM_DONE.
	OTP_DBG_PSM_DONE = 0x1

	// BIST: During BIST, count address locations that have at least one leaky bit
	// Position of CNT_FAIL field.
	OTP_BIST_CNT_FAIL_Pos = 0x1e
	// Bit mask of CNT_FAIL field.
	OTP_BIST_CNT_FAIL_Msk = 0x40000000
	// Bit CNT_FAIL.
	OTP_BIST_CNT_FAIL = 0x40000000
	// Position of CNT_CLR field.
	OTP_BIST_CNT_CLR_Pos = 0x1d
	// Bit mask of CNT_CLR field.
	OTP_BIST_CNT_CLR_Msk = 0x20000000
	// Bit CNT_CLR.
	OTP_BIST_CNT_CLR = 0x20000000
	// Position of CNT_ENA field.
	OTP_BIST_CNT_ENA_Pos = 0x1c
	// Bit mask of CNT_ENA field.
	OTP_BIST_CNT_ENA_Msk = 0x10000000
	// Bit CNT_ENA.
	OTP_BIST_CNT_ENA = 0x10000000
	// Position of CNT_MAX field.
	OTP_BIST_CNT_MAX_Pos = 0x10
	// Bit mask of CNT_MAX field.
	OTP_BIST_CNT_MAX_Msk = 0xfff0000
	// Position of CNT field.
	OTP_BIST_CNT_Pos = 0x0
	// Bit mask of CNT field.
	OTP_BIST_CNT_Msk = 0x1fff

	// CRT_KEY_W0: Word 0 (bits 31..0) of the key. Write only, read returns 0x0
	// Position of CRT_KEY_W0 field.
	OTP_CRT_KEY_W0_CRT_KEY_W0_Pos = 0x0
	// Bit mask of CRT_KEY_W0 field.
	OTP_CRT_KEY_W0_CRT_KEY_W0_Msk = 0xffffffff

	// CRT_KEY_W1: Word 1 (bits 63..32) of the key. Write only, read returns 0x0
	// Position of CRT_KEY_W1 field.
	OTP_CRT_KEY_W1_CRT_KEY_W1_Pos = 0x0
	// Bit mask of CRT_KEY_W1 field.
	OTP_CRT_KEY_W1_CRT_KEY_W1_Msk = 0xffffffff

	// CRT_KEY_W2: Word 2 (bits 95..64) of the key. Write only, read returns 0x0
	// Position of CRT_KEY_W2 field.
	OTP_CRT_KEY_W2_CRT_KEY_W2_Pos = 0x0
	// Bit mask of CRT_KEY_W2 field.
	OTP_CRT_KEY_W2_CRT_KEY_W2_Msk = 0xffffffff

	// CRT_KEY_W3: Word 3 (bits 127..96) of the key. Write only, read returns 0x0
	// Position of CRT_KEY_W3 field.
	OTP_CRT_KEY_W3_CRT_KEY_W3_Pos = 0x0
	// Bit mask of CRT_KEY_W3 field.
	OTP_CRT_KEY_W3_CRT_KEY_W3_Msk = 0xffffffff

	// CRITICAL: Quickly check values of critical flags read during boot up
	// Position of RISCV_DISABLE field.
	OTP_CRITICAL_RISCV_DISABLE_Pos = 0x11
	// Bit mask of RISCV_DISABLE field.
	OTP_CRITICAL_RISCV_DISABLE_Msk = 0x20000
	// Bit RISCV_DISABLE.
	OTP_CRITICAL_RISCV_DISABLE = 0x20000
	// Position of ARM_DISABLE field.
	OTP_CRITICAL_ARM_DISABLE_Pos = 0x10
	// Bit mask of ARM_DISABLE field.
	OTP_CRITICAL_ARM_DISABLE_Msk = 0x10000
	// Bit ARM_DISABLE.
	OTP_CRITICAL_ARM_DISABLE = 0x10000
	// Position of GLITCH_DETECTOR_SENS field.
	OTP_CRITICAL_GLITCH_DETECTOR_SENS_Pos = 0x5
	// Bit mask of GLITCH_DETECTOR_SENS field.
	OTP_CRITICAL_GLITCH_DETECTOR_SENS_Msk = 0x60
	// Position of GLITCH_DETECTOR_ENABLE field.
	OTP_CRITICAL_GLITCH_DETECTOR_ENABLE_Pos = 0x4
	// Bit mask of GLITCH_DETECTOR_ENABLE field.
	OTP_CRITICAL_GLITCH_DETECTOR_ENABLE_Msk = 0x10
	// Bit GLITCH_DETECTOR_ENABLE.
	OTP_CRITICAL_GLITCH_DETECTOR_ENABLE = 0x10
	// Position of DEFAULT_ARCHSEL field.
	OTP_CRITICAL_DEFAULT_ARCHSEL_Pos = 0x3
	// Bit mask of DEFAULT_ARCHSEL field.
	OTP_CRITICAL_DEFAULT_ARCHSEL_Msk = 0x8
	// Bit DEFAULT_ARCHSEL.
	OTP_CRITICAL_DEFAULT_ARCHSEL = 0x8
	// Position of DEBUG_DISABLE field.
	OTP_CRITICAL_DEBUG_DISABLE_Pos = 0x2
	// Bit mask of DEBUG_DISABLE field.
	OTP_CRITICAL_DEBUG_DISABLE_Msk = 0x4
	// Bit DEBUG_DISABLE.
	OTP_CRITICAL_DEBUG_DISABLE = 0x4
	// Position of SECURE_DEBUG_DISABLE field.
	OTP_CRITICAL_SECURE_DEBUG_DISABLE_Pos = 0x1
	// Bit mask of SECURE_DEBUG_DISABLE field.
	OTP_CRITICAL_SECURE_DEBUG_DISABLE_Msk = 0x2
	// Bit SECURE_DEBUG_DISABLE.
	OTP_CRITICAL_SECURE_DEBUG_DISABLE = 0x2
	// Position of SECURE_BOOT_ENABLE field.
	OTP_CRITICAL_SECURE_BOOT_ENABLE_Pos = 0x0
	// Bit mask of SECURE_BOOT_ENABLE field.
	OTP_CRITICAL_SECURE_BOOT_ENABLE_Msk = 0x1
	// Bit SECURE_BOOT_ENABLE.
	OTP_CRITICAL_SECURE_BOOT_ENABLE = 0x1

	// KEY_VALID: Which keys were valid (enrolled) at boot time
	// Position of KEY_VALID field.
	OTP_KEY_VALID_KEY_VALID_Pos = 0x0
	// Bit mask of KEY_VALID field.
	OTP_KEY_VALID_KEY_VALID_Msk = 0xff

	// DEBUGEN: Enable a debug feature that has been disabled. Debug features are disabled if one of the relevant critical boot flags is set in OTP (DEBUG_DISABLE or SECURE_DEBUG_DISABLE), OR if a debug key is marked valid in OTP, and the matching key value has not been supplied over SWD. Specifically: - The DEBUG_DISABLE flag disables all debug features. This can be fully overridden by setting all bits of this register. - The SECURE_DEBUG_DISABLE flag disables secure processor debug. This can be fully overridden by setting the PROC0_SECURE and PROC1_SECURE bits of this register. - If a single debug key has been registered, and no matching key value has been supplied over SWD, then all debug features are disabled. This can be fully overridden by setting all bits of this register. - If both debug keys have been registered, and the Non-secure key's value (key 6) has been supplied over SWD, secure processor debug is disabled. This can be fully overridden by setting the PROC0_SECURE and PROC1_SECURE bits of this register. - If both debug keys have been registered, and the Secure key's value (key 5) has been supplied over SWD, then no debug features are disabled by the key mechanism. However, note that in this case debug features may still be disabled by the critical boot flags.
	// Position of MISC field.
	OTP_DEBUGEN_MISC_Pos = 0x8
	// Bit mask of MISC field.
	OTP_DEBUGEN_MISC_Msk = 0x100
	// Bit MISC.
	OTP_DEBUGEN_MISC = 0x100
	// Position of PROC1_SECURE field.
	OTP_DEBUGEN_PROC1_SECURE_Pos = 0x3
	// Bit mask of PROC1_SECURE field.
	OTP_DEBUGEN_PROC1_SECURE_Msk = 0x8
	// Bit PROC1_SECURE.
	OTP_DEBUGEN_PROC1_SECURE = 0x8
	// Position of PROC1 field.
	OTP_DEBUGEN_PROC1_Pos = 0x2
	// Bit mask of PROC1 field.
	OTP_DEBUGEN_PROC1_Msk = 0x4
	// Bit PROC1.
	OTP_DEBUGEN_PROC1 = 0x4
	// Position of PROC0_SECURE field.
	OTP_DEBUGEN_PROC0_SECURE_Pos = 0x1
	// Bit mask of PROC0_SECURE field.
	OTP_DEBUGEN_PROC0_SECURE_Msk = 0x2
	// Bit PROC0_SECURE.
	OTP_DEBUGEN_PROC0_SECURE = 0x2
	// Position of PROC0 field.
	OTP_DEBUGEN_PROC0_Pos = 0x0
	// Bit mask of PROC0 field.
	OTP_DEBUGEN_PROC0_Msk = 0x1
	// Bit PROC0.
	OTP_DEBUGEN_PROC0 = 0x1

	// DEBUGEN_LOCK: Write 1s to lock corresponding bits in DEBUGEN. This register is reset by the processor cold reset.
	// Position of MISC field.
	OTP_DEBUGEN_LOCK_MISC_Pos = 0x8
	// Bit mask of MISC field.
	OTP_DEBUGEN_LOCK_MISC_Msk = 0x100
	// Bit MISC.
	OTP_DEBUGEN_LOCK_MISC = 0x100
	// Position of PROC1_SECURE field.
	OTP_DEBUGEN_LOCK_PROC1_SECURE_Pos = 0x3
	// Bit mask of PROC1_SECURE field.
	OTP_DEBUGEN_LOCK_PROC1_SECURE_Msk = 0x8
	// Bit PROC1_SECURE.
	OTP_DEBUGEN_LOCK_PROC1_SECURE = 0x8
	// Position of PROC1 field.
	OTP_DEBUGEN_LOCK_PROC1_Pos = 0x2
	// Bit mask of PROC1 field.
	OTP_DEBUGEN_LOCK_PROC1_Msk = 0x4
	// Bit PROC1.
	OTP_DEBUGEN_LOCK_PROC1 = 0x4
	// Position of PROC0_SECURE field.
	OTP_DEBUGEN_LOCK_PROC0_SECURE_Pos = 0x1
	// Bit mask of PROC0_SECURE field.
	OTP_DEBUGEN_LOCK_PROC0_SECURE_Msk = 0x2
	// Bit PROC0_SECURE.
	OTP_DEBUGEN_LOCK_PROC0_SECURE = 0x2
	// Position of PROC0 field.
	OTP_DEBUGEN_LOCK_PROC0_Pos = 0x0
	// Bit mask of PROC0 field.
	OTP_DEBUGEN_LOCK_PROC0_Msk = 0x1
	// Bit PROC0.
	OTP_DEBUGEN_LOCK_PROC0 = 0x1

	// ARCHSEL: Architecture select (Arm/RISC-V). The default and allowable values of this register are constrained by the critical boot flags. This register is reset by the earliest reset in the switched core power domain (before a processor cold reset). Cores sample their architecture select signal on a warm reset. The source of the warm reset could be the system power-up state machine, the watchdog timer, Arm SYSRESETREQ or from RISC-V hartresetreq. Note that when an Arm core is deselected, its cold reset domain is also held in reset, since in particular the SYSRESETREQ bit becomes inaccessible once the core is deselected. Note also the RISC-V cores do not have a cold reset domain, since their corresponding controls are located in the Debug Module.
	// Position of CORE1 field.
	OTP_ARCHSEL_CORE1_Pos = 0x1
	// Bit mask of CORE1 field.
	OTP_ARCHSEL_CORE1_Msk = 0x2
	// Bit CORE1.
	OTP_ARCHSEL_CORE1 = 0x2
	// Switch core 1 to Arm (Cortex-M33)
	OTP_ARCHSEL_CORE1_ARM = 0x0
	// Switch core 1 to RISC-V (Hazard3)
	OTP_ARCHSEL_CORE1_RISCV = 0x1
	// Position of CORE0 field.
	OTP_ARCHSEL_CORE0_Pos = 0x0
	// Bit mask of CORE0 field.
	OTP_ARCHSEL_CORE0_Msk = 0x1
	// Bit CORE0.
	OTP_ARCHSEL_CORE0 = 0x1
	// Switch core 0 to Arm (Cortex-M33)
	OTP_ARCHSEL_CORE0_ARM = 0x0
	// Switch core 0 to RISC-V (Hazard3)
	OTP_ARCHSEL_CORE0_RISCV = 0x1

	// ARCHSEL_STATUS: Get the current architecture select state of each core. Cores sample the current value of the ARCHSEL register when their warm reset is released, at which point the corresponding bit in this register will also update.
	// Position of CORE1 field.
	OTP_ARCHSEL_STATUS_CORE1_Pos = 0x1
	// Bit mask of CORE1 field.
	OTP_ARCHSEL_STATUS_CORE1_Msk = 0x2
	// Bit CORE1.
	OTP_ARCHSEL_STATUS_CORE1 = 0x2
	// Core 1 is currently Arm (Cortex-M33)
	OTP_ARCHSEL_STATUS_CORE1_ARM = 0x0
	// Core 1 is currently RISC-V (Hazard3)
	OTP_ARCHSEL_STATUS_CORE1_RISCV = 0x1
	// Position of CORE0 field.
	OTP_ARCHSEL_STATUS_CORE0_Pos = 0x0
	// Bit mask of CORE0 field.
	OTP_ARCHSEL_STATUS_CORE0_Msk = 0x1
	// Bit CORE0.
	OTP_ARCHSEL_STATUS_CORE0 = 0x1
	// Core 0 is currently Arm (Cortex-M33)
	OTP_ARCHSEL_STATUS_CORE0_ARM = 0x0
	// Core 0 is currently RISC-V (Hazard3)
	OTP_ARCHSEL_STATUS_CORE0_RISCV = 0x1

	// BOOTDIS: Tell the bootrom to ignore scratch register boot vectors (both power manager and watchdog) on the next power up. If an early boot stage has soft-locked some OTP pages in order to protect their contents from later stages, there is a risk that Secure code running at a later stage can unlock the pages by performing a watchdog reset that resets the OTP. This register can be used to ensure that the bootloader runs as normal on the next power up, preventing Secure code at a later stage from accessing OTP in its unlocked state. Should be used in conjunction with the power manager BOOTDIS register.
	// Position of NEXT field.
	OTP_BOOTDIS_NEXT_Pos = 0x1
	// Bit mask of NEXT field.
	OTP_BOOTDIS_NEXT_Msk = 0x2
	// Bit NEXT.
	OTP_BOOTDIS_NEXT = 0x2
	// Position of NOW field.
	OTP_BOOTDIS_NOW_Pos = 0x0
	// Bit mask of NOW field.
	OTP_BOOTDIS_NOW_Msk = 0x1
	// Bit NOW.
	OTP_BOOTDIS_NOW = 0x1

	// INTR: Raw Interrupts
	// Position of APB_RD_NSEC_FAIL field.
	OTP_INTR_APB_RD_NSEC_FAIL_Pos = 0x4
	// Bit mask of APB_RD_NSEC_FAIL field.
	OTP_INTR_APB_RD_NSEC_FAIL_Msk = 0x10
	// Bit APB_RD_NSEC_FAIL.
	OTP_INTR_APB_RD_NSEC_FAIL = 0x10
	// Position of APB_RD_SEC_FAIL field.
	OTP_INTR_APB_RD_SEC_FAIL_Pos = 0x3
	// Bit mask of APB_RD_SEC_FAIL field.
	OTP_INTR_APB_RD_SEC_FAIL_Msk = 0x8
	// Bit APB_RD_SEC_FAIL.
	OTP_INTR_APB_RD_SEC_FAIL = 0x8
	// Position of APB_DCTRL_FAIL field.
	OTP_INTR_APB_DCTRL_FAIL_Pos = 0x2
	// Bit mask of APB_DCTRL_FAIL field.
	OTP_INTR_APB_DCTRL_FAIL_Msk = 0x4
	// Bit APB_DCTRL_FAIL.
	OTP_INTR_APB_DCTRL_FAIL = 0x4
	// Position of SBPI_WR_FAIL field.
	OTP_INTR_SBPI_WR_FAIL_Pos = 0x1
	// Bit mask of SBPI_WR_FAIL field.
	OTP_INTR_SBPI_WR_FAIL_Msk = 0x2
	// Bit SBPI_WR_FAIL.
	OTP_INTR_SBPI_WR_FAIL = 0x2
	// Position of SBPI_FLAG_N field.
	OTP_INTR_SBPI_FLAG_N_Pos = 0x0
	// Bit mask of SBPI_FLAG_N field.
	OTP_INTR_SBPI_FLAG_N_Msk = 0x1
	// Bit SBPI_FLAG_N.
	OTP_INTR_SBPI_FLAG_N = 0x1

	// INTE: Interrupt Enable
	// Position of APB_RD_NSEC_FAIL field.
	OTP_INTE_APB_RD_NSEC_FAIL_Pos = 0x4
	// Bit mask of APB_RD_NSEC_FAIL field.
	OTP_INTE_APB_RD_NSEC_FAIL_Msk = 0x10
	// Bit APB_RD_NSEC_FAIL.
	OTP_INTE_APB_RD_NSEC_FAIL = 0x10
	// Position of APB_RD_SEC_FAIL field.
	OTP_INTE_APB_RD_SEC_FAIL_Pos = 0x3
	// Bit mask of APB_RD_SEC_FAIL field.
	OTP_INTE_APB_RD_SEC_FAIL_Msk = 0x8
	// Bit APB_RD_SEC_FAIL.
	OTP_INTE_APB_RD_SEC_FAIL = 0x8
	// Position of APB_DCTRL_FAIL field.
	OTP_INTE_APB_DCTRL_FAIL_Pos = 0x2
	// Bit mask of APB_DCTRL_FAIL field.
	OTP_INTE_APB_DCTRL_FAIL_Msk = 0x4
	// Bit APB_DCTRL_FAIL.
	OTP_INTE_APB_DCTRL_FAIL = 0x4
	// Position of SBPI_WR_FAIL field.
	OTP_INTE_SBPI_WR_FAIL_Pos = 0x1
	// Bit mask of SBPI_WR_FAIL field.
	OTP_INTE_SBPI_WR_FAIL_Msk = 0x2
	// Bit SBPI_WR_FAIL.
	OTP_INTE_SBPI_WR_FAIL = 0x2
	// Position of SBPI_FLAG_N field.
	OTP_INTE_SBPI_FLAG_N_Pos = 0x0
	// Bit mask of SBPI_FLAG_N field.
	OTP_INTE_SBPI_FLAG_N_Msk = 0x1
	// Bit SBPI_FLAG_N.
	OTP_INTE_SBPI_FLAG_N = 0x1

	// INTF: Interrupt Force
	// Position of APB_RD_NSEC_FAIL field.
	OTP_INTF_APB_RD_NSEC_FAIL_Pos = 0x4
	// Bit mask of APB_RD_NSEC_FAIL field.
	OTP_INTF_APB_RD_NSEC_FAIL_Msk = 0x10
	// Bit APB_RD_NSEC_FAIL.
	OTP_INTF_APB_RD_NSEC_FAIL = 0x10
	// Position of APB_RD_SEC_FAIL field.
	OTP_INTF_APB_RD_SEC_FAIL_Pos = 0x3
	// Bit mask of APB_RD_SEC_FAIL field.
	OTP_INTF_APB_RD_SEC_FAIL_Msk = 0x8
	// Bit APB_RD_SEC_FAIL.
	OTP_INTF_APB_RD_SEC_FAIL = 0x8
	// Position of APB_DCTRL_FAIL field.
	OTP_INTF_APB_DCTRL_FAIL_Pos = 0x2
	// Bit mask of APB_DCTRL_FAIL field.
	OTP_INTF_APB_DCTRL_FAIL_Msk = 0x4
	// Bit APB_DCTRL_FAIL.
	OTP_INTF_APB_DCTRL_FAIL = 0x4
	// Position of SBPI_WR_FAIL field.
	OTP_INTF_SBPI_WR_FAIL_Pos = 0x1
	// Bit mask of SBPI_WR_FAIL field.
	OTP_INTF_SBPI_WR_FAIL_Msk = 0x2
	// Bit SBPI_WR_FAIL.
	OTP_INTF_SBPI_WR_FAIL = 0x2
	// Position of SBPI_FLAG_N field.
	OTP_INTF_SBPI_FLAG_N_Pos = 0x0
	// Bit mask of SBPI_FLAG_N field.
	OTP_INTF_SBPI_FLAG_N_Msk = 0x1
	// Bit SBPI_FLAG_N.
	OTP_INTF_SBPI_FLAG_N = 0x1

	// INTS: Interrupt status after masking & forcing
	// Position of APB_RD_NSEC_FAIL field.
	OTP_INTS_APB_RD_NSEC_FAIL_Pos = 0x4
	// Bit mask of APB_RD_NSEC_FAIL field.
	OTP_INTS_APB_RD_NSEC_FAIL_Msk = 0x10
	// Bit APB_RD_NSEC_FAIL.
	OTP_INTS_APB_RD_NSEC_FAIL = 0x10
	// Position of APB_RD_SEC_FAIL field.
	OTP_INTS_APB_RD_SEC_FAIL_Pos = 0x3
	// Bit mask of APB_RD_SEC_FAIL field.
	OTP_INTS_APB_RD_SEC_FAIL_Msk = 0x8
	// Bit APB_RD_SEC_FAIL.
	OTP_INTS_APB_RD_SEC_FAIL = 0x8
	// Position of APB_DCTRL_FAIL field.
	OTP_INTS_APB_DCTRL_FAIL_Pos = 0x2
	// Bit mask of APB_DCTRL_FAIL field.
	OTP_INTS_APB_DCTRL_FAIL_Msk = 0x4
	// Bit APB_DCTRL_FAIL.
	OTP_INTS_APB_DCTRL_FAIL = 0x4
	// Position of SBPI_WR_FAIL field.
	OTP_INTS_SBPI_WR_FAIL_Pos = 0x1
	// Bit mask of SBPI_WR_FAIL field.
	OTP_INTS_SBPI_WR_FAIL_Msk = 0x2
	// Bit SBPI_WR_FAIL.
	OTP_INTS_SBPI_WR_FAIL = 0x2
	// Position of SBPI_FLAG_N field.
	OTP_INTS_SBPI_FLAG_N_Pos = 0x0
	// Bit mask of SBPI_FLAG_N field.
	OTP_INTS_SBPI_FLAG_N_Msk = 0x1
	// Bit SBPI_FLAG_N.
	OTP_INTS_SBPI_FLAG_N = 0x1
)

// Constants for OTP_DATA: Predefined OTP data layout for RP2350
const (
	// CHIPID0: Bits 15:0 of public device ID. (ECC) The CHIPID0..3 rows contain a 64-bit random identifier for this chip, which can be read from the USB bootloader PICOBOOT interface or from the get_sys_info ROM API. The number of random bits makes the occurrence of twins exceedingly unlikely: for example, a fleet of a hundred million devices has a 99.97% probability of no twinned IDs. This is estimated to be lower than the occurrence of process errors in the assignment of sequential random IDs, and for practical purposes CHIPID may be treated as unique.
	// Position of CHIPID0 field.
	OTP_DATA_CHIPID0_CHIPID0_Pos = 0x0
	// Bit mask of CHIPID0 field.
	OTP_DATA_CHIPID0_CHIPID0_Msk = 0xffff

	// CHIPID1: Bits 31:16 of public device ID (ECC)
	// Position of CHIPID1 field.
	OTP_DATA_CHIPID1_CHIPID1_Pos = 0x0
	// Bit mask of CHIPID1 field.
	OTP_DATA_CHIPID1_CHIPID1_Msk = 0xffff

	// CHIPID2: Bits 47:32 of public device ID (ECC)
	// Position of CHIPID2 field.
	OTP_DATA_CHIPID2_CHIPID2_Pos = 0x0
	// Bit mask of CHIPID2 field.
	OTP_DATA_CHIPID2_CHIPID2_Msk = 0xffff

	// CHIPID3: Bits 63:48 of public device ID (ECC)
	// Position of CHIPID3 field.
	OTP_DATA_CHIPID3_CHIPID3_Pos = 0x0
	// Bit mask of CHIPID3 field.
	OTP_DATA_CHIPID3_CHIPID3_Msk = 0xffff

	// RANDID0: Bits 15:0 of private per-device random number (ECC) The RANDID0..7 rows form a 128-bit random number generated during device test. This ID is not exposed through the USB PICOBOOT GET_INFO command or the ROM `get_sys_info()` API. However note that the USB PICOBOOT OTP access point can read the entirety of page 0, so this value is not meaningfully private unless the USB PICOBOOT interface is disabled via the DISABLE_BOOTSEL_USB_PICOBOOT_IFC flag in BOOT_FLAGS0.
	// Position of RANDID0 field.
	OTP_DATA_RANDID0_RANDID0_Pos = 0x0
	// Bit mask of RANDID0 field.
	OTP_DATA_RANDID0_RANDID0_Msk = 0xffff

	// RANDID1: Bits 31:16 of private per-device random number (ECC)
	// Position of RANDID1 field.
	OTP_DATA_RANDID1_RANDID1_Pos = 0x0
	// Bit mask of RANDID1 field.
	OTP_DATA_RANDID1_RANDID1_Msk = 0xffff

	// RANDID2: Bits 47:32 of private per-device random number (ECC)
	// Position of RANDID2 field.
	OTP_DATA_RANDID2_RANDID2_Pos = 0x0
	// Bit mask of RANDID2 field.
	OTP_DATA_RANDID2_RANDID2_Msk = 0xffff

	// RANDID3: Bits 63:48 of private per-device random number (ECC)
	// Position of RANDID3 field.
	OTP_DATA_RANDID3_RANDID3_Pos = 0x0
	// Bit mask of RANDID3 field.
	OTP_DATA_RANDID3_RANDID3_Msk = 0xffff

	// RANDID4: Bits 79:64 of private per-device random number (ECC)
	// Position of RANDID4 field.
	OTP_DATA_RANDID4_RANDID4_Pos = 0x0
	// Bit mask of RANDID4 field.
	OTP_DATA_RANDID4_RANDID4_Msk = 0xffff

	// RANDID5: Bits 95:80 of private per-device random number (ECC)
	// Position of RANDID5 field.
	OTP_DATA_RANDID5_RANDID5_Pos = 0x0
	// Bit mask of RANDID5 field.
	OTP_DATA_RANDID5_RANDID5_Msk = 0xffff

	// RANDID6: Bits 111:96 of private per-device random number (ECC)
	// Position of RANDID6 field.
	OTP_DATA_RANDID6_RANDID6_Pos = 0x0
	// Bit mask of RANDID6 field.
	OTP_DATA_RANDID6_RANDID6_Msk = 0xffff

	// RANDID7: Bits 127:112 of private per-device random number (ECC)
	// Position of RANDID7 field.
	OTP_DATA_RANDID7_RANDID7_Pos = 0x0
	// Bit mask of RANDID7 field.
	OTP_DATA_RANDID7_RANDID7_Msk = 0xffff

	// ROSC_CALIB: Ring oscillator frequency in kHz, measured during manufacturing (ECC) This is measured at 1.1 V, at room temperature, with the ROSC configuration registers in their reset state.
	// Position of ROSC_CALIB field.
	OTP_DATA_ROSC_CALIB_ROSC_CALIB_Pos = 0x0
	// Bit mask of ROSC_CALIB field.
	OTP_DATA_ROSC_CALIB_ROSC_CALIB_Msk = 0xffff

	// LPOSC_CALIB: Low-power oscillator frequency in Hz, measured during manufacturing (ECC) This is measured at 1.1V, at room temperature, with the LPOSC trim register in its reset state.
	// Position of LPOSC_CALIB field.
	OTP_DATA_LPOSC_CALIB_LPOSC_CALIB_Pos = 0x0
	// Bit mask of LPOSC_CALIB field.
	OTP_DATA_LPOSC_CALIB_LPOSC_CALIB_Msk = 0xffff

	// NUM_GPIOS: The number of main user GPIOs (bank 0). Should read 48 in the QFN80 package, and 30 in the QFN60 package. (ECC)
	// Position of NUM_GPIOS field.
	OTP_DATA_NUM_GPIOS_NUM_GPIOS_Pos = 0x0
	// Bit mask of NUM_GPIOS field.
	OTP_DATA_NUM_GPIOS_NUM_GPIOS_Msk = 0xff

	// INFO_CRC0: Lower 16 bits of CRC32 of OTP addresses 0x00 through 0x6b (polynomial 0x4c11db7, input reflected, output reflected, seed all-ones, final XOR all-ones) (ECC)
	// Position of INFO_CRC0 field.
	OTP_DATA_INFO_CRC0_INFO_CRC0_Pos = 0x0
	// Bit mask of INFO_CRC0 field.
	OTP_DATA_INFO_CRC0_INFO_CRC0_Msk = 0xffff

	// INFO_CRC1: Upper 16 bits of CRC32 of OTP addresses 0x00 through 0x6b (ECC)
	// Position of INFO_CRC1 field.
	OTP_DATA_INFO_CRC1_INFO_CRC1_Pos = 0x0
	// Bit mask of INFO_CRC1 field.
	OTP_DATA_INFO_CRC1_INFO_CRC1_Msk = 0xffff

	// FLASH_DEVINFO: Stores information about external flash device(s). (ECC) Assumed to be valid if BOOT_FLAGS0_FLASH_DEVINFO_ENABLE is set.
	// Position of CS1_SIZE field.
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_Pos = 0xc
	// Bit mask of CS1_SIZE field.
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_Msk  = 0xf000
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_NONE = 0x0
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_8K   = 0x1
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_16K  = 0x2
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_32K  = 0x3
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_64k  = 0x4
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_128K = 0x5
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_256K = 0x6
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_512K = 0x7
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_1M   = 0x8
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_2M   = 0x9
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_4M   = 0xa
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_8M   = 0xb
	OTP_DATA_FLASH_DEVINFO_CS1_SIZE_16M  = 0xc
	// Position of CS0_SIZE field.
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_Pos = 0x8
	// Bit mask of CS0_SIZE field.
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_Msk  = 0xf00
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_NONE = 0x0
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_8K   = 0x1
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_16K  = 0x2
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_32K  = 0x3
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_64k  = 0x4
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_128K = 0x5
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_256K = 0x6
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_512K = 0x7
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_1M   = 0x8
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_2M   = 0x9
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_4M   = 0xa
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_8M   = 0xb
	OTP_DATA_FLASH_DEVINFO_CS0_SIZE_16M  = 0xc
	// Position of D8H_ERASE_SUPPORTED field.
	OTP_DATA_FLASH_DEVINFO_D8H_ERASE_SUPPORTED_Pos = 0x7
	// Bit mask of D8H_ERASE_SUPPORTED field.
	OTP_DATA_FLASH_DEVINFO_D8H_ERASE_SUPPORTED_Msk = 0x80
	// Bit D8H_ERASE_SUPPORTED.
	OTP_DATA_FLASH_DEVINFO_D8H_ERASE_SUPPORTED = 0x80
	// Position of CS1_GPIO field.
	OTP_DATA_FLASH_DEVINFO_CS1_GPIO_Pos = 0x0
	// Bit mask of CS1_GPIO field.
	OTP_DATA_FLASH_DEVINFO_CS1_GPIO_Msk = 0x3f

	// FLASH_PARTITION_SLOT_SIZE: Gap between partition table slot 0 and slot 1 at the start of flash (the default size is 4096 bytes) (ECC) Enabled by the OVERRIDE_FLASH_PARTITION_SLOT_SIZE bit in BOOT_FLAGS, the size is 4096 * (value + 1)
	// Position of FLASH_PARTITION_SLOT_SIZE field.
	OTP_DATA_FLASH_PARTITION_SLOT_SIZE_FLASH_PARTITION_SLOT_SIZE_Pos = 0x0
	// Bit mask of FLASH_PARTITION_SLOT_SIZE field.
	OTP_DATA_FLASH_PARTITION_SLOT_SIZE_FLASH_PARTITION_SLOT_SIZE_Msk = 0xffff

	// BOOTSEL_LED_CFG: Pin configuration for LED status, used by USB bootloader. (ECC) Must be valid if BOOT_FLAGS0_ENABLE_BOOTSEL_LED is set.
	// Position of ACTIVELOW field.
	OTP_DATA_BOOTSEL_LED_CFG_ACTIVELOW_Pos = 0x8
	// Bit mask of ACTIVELOW field.
	OTP_DATA_BOOTSEL_LED_CFG_ACTIVELOW_Msk = 0x100
	// Bit ACTIVELOW.
	OTP_DATA_BOOTSEL_LED_CFG_ACTIVELOW = 0x100
	// Position of PIN field.
	OTP_DATA_BOOTSEL_LED_CFG_PIN_Pos = 0x0
	// Bit mask of PIN field.
	OTP_DATA_BOOTSEL_LED_CFG_PIN_Msk = 0x3f

	// BOOTSEL_PLL_CFG: Optional PLL configuration for BOOTSEL mode. (ECC) This should be configured to produce an exact 48 MHz based on the crystal oscillator frequency. User mode software may also use this value to calculate the expected crystal frequency based on an assumed 48 MHz PLL output. If no configuration is given, the crystal is assumed to be 12 MHz. The PLL frequency can be calculated as: PLL out = (XOSC frequency / (REFDIV+1)) x FBDIV / (POSTDIV1 x POSTDIV2) Conversely the crystal frequency can be calculated as: XOSC frequency = 48 MHz x (REFDIV+1) x (POSTDIV1 x POSTDIV2) / FBDIV (Note the +1 on REFDIV is because the value stored in this OTP location is the actual divisor value minus one.) Used if and only if ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG is set in BOOT_FLAGS0. That bit should be set only after this row and BOOTSEL_XOSC_CFG are both correctly programmed.
	// Position of REFDIV field.
	OTP_DATA_BOOTSEL_PLL_CFG_REFDIV_Pos = 0xf
	// Bit mask of REFDIV field.
	OTP_DATA_BOOTSEL_PLL_CFG_REFDIV_Msk = 0x8000
	// Bit REFDIV.
	OTP_DATA_BOOTSEL_PLL_CFG_REFDIV = 0x8000
	// Position of POSTDIV2 field.
	OTP_DATA_BOOTSEL_PLL_CFG_POSTDIV2_Pos = 0xc
	// Bit mask of POSTDIV2 field.
	OTP_DATA_BOOTSEL_PLL_CFG_POSTDIV2_Msk = 0x7000
	// Position of POSTDIV1 field.
	OTP_DATA_BOOTSEL_PLL_CFG_POSTDIV1_Pos = 0x9
	// Bit mask of POSTDIV1 field.
	OTP_DATA_BOOTSEL_PLL_CFG_POSTDIV1_Msk = 0xe00
	// Position of FBDIV field.
	OTP_DATA_BOOTSEL_PLL_CFG_FBDIV_Pos = 0x0
	// Bit mask of FBDIV field.
	OTP_DATA_BOOTSEL_PLL_CFG_FBDIV_Msk = 0x1ff

	// BOOTSEL_XOSC_CFG: Non-default crystal oscillator configuration for the USB bootloader. (ECC) These values may also be used by user code configuring the crystal oscillator. Used if and only if ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG is set in BOOT_FLAGS0. That bit should be set only after this row and BOOTSEL_PLL_CFG are both correctly programmed.
	// Position of RANGE field.
	OTP_DATA_BOOTSEL_XOSC_CFG_RANGE_Pos = 0xe
	// Bit mask of RANGE field.
	OTP_DATA_BOOTSEL_XOSC_CFG_RANGE_Msk       = 0xc000
	OTP_DATA_BOOTSEL_XOSC_CFG_RANGE_1_15MHZ   = 0x0
	OTP_DATA_BOOTSEL_XOSC_CFG_RANGE_10_30MHZ  = 0x1
	OTP_DATA_BOOTSEL_XOSC_CFG_RANGE_25_60MHZ  = 0x2
	OTP_DATA_BOOTSEL_XOSC_CFG_RANGE_40_100MHZ = 0x3
	// Position of STARTUP field.
	OTP_DATA_BOOTSEL_XOSC_CFG_STARTUP_Pos = 0x0
	// Bit mask of STARTUP field.
	OTP_DATA_BOOTSEL_XOSC_CFG_STARTUP_Msk = 0x3fff

	// USB_WHITE_LABEL_ADDR: Row index of the USB_WHITE_LABEL structure within OTP (ECC) The table has 16 rows, each of which are also ECC and marked valid by the corresponding valid bit in USB_BOOT_FLAGS (ECC). The entries are either _VALUEs where the 16 bit value is used as is, or _STRDEFs which acts as a pointers to a string value. The value stored in a _STRDEF is two separate bytes: The low seven bits of the first (LSB) byte indicates the number of characters in the string, and the top bit of the first (LSB) byte if set to indicate that each character in the string is two bytes (Unicode) versus one byte if unset. The second (MSB) byte represents the location of the string data, and is encoded as the number of rows from this USB_WHITE_LABEL_ADDR; i.e. the row of the start of the string is USB_WHITE_LABEL_ADDR value + msb_byte. In each case, the corresponding valid bit enables replacing the default value for the corresponding item provided by the boot rom. Note that Unicode _STRDEFs are only supported for USB_DEVICE_PRODUCT_STRDEF, USB_DEVICE_SERIAL_NUMBER_STRDEF and USB_DEVICE_MANUFACTURER_STRDEF. Unicode values will be ignored if specified for other fields, and non-unicode values for these three items will be converted to Unicode characters by setting the upper 8 bits to zero. Note that if the USB_WHITE_LABEL structure or the corresponding strings are not readable by BOOTSEL mode based on OTP permissions, or if alignment requirements are not met, then the corresponding default values are used. The index values indicate where each field is located (row USB_WHITE_LABEL_ADDR value + index):
	// Position of USB_WHITE_LABEL_ADDR field.
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_Pos = 0x0
	// Bit mask of USB_WHITE_LABEL_ADDR field.
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_Msk                                          = 0xffff
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_VID_VALUE                   = 0x0
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_PID_VALUE                   = 0x1
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_BCD_DEVICE_VALUE            = 0x2
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_LANG_ID_VALUE               = 0x3
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_MANUFACTURER_STRDEF         = 0x4
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_PRODUCT_STRDEF              = 0x5
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_SERIAL_NUMBER_STRDEF        = 0x6
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES = 0x7
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_VOLUME_LABEL_STRDEF                    = 0x8
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_SCSI_INQUIRY_VENDOR_STRDEF             = 0x9
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_SCSI_INQUIRY_PRODUCT_STRDEF            = 0xa
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_SCSI_INQUIRY_VERSION_STRDEF            = 0xb
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_INDEX_HTM_REDIRECT_URL_STRDEF          = 0xc
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_INDEX_HTM_REDIRECT_NAME_STRDEF         = 0xd
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_INFO_UF2_TXT_MODEL_STRDEF              = 0xe
	OTP_DATA_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_INFO_UF2_TXT_BOARD_ID_STRDEF           = 0xf

	// OTPBOOT_SRC: OTP start row for the OTP boot image. (ECC) If OTP boot is enabled, the bootrom will load from this location into SRAM and then directly enter the loaded image. Note that the image must be signed if SECURE_BOOT_ENABLE is set. The image itself is assumed to be ECC-protected. This must be an even number. Equivalently, the OTP boot image must start at a word-aligned location in the ECC read data address window.
	// Position of OTPBOOT_SRC field.
	OTP_DATA_OTPBOOT_SRC_OTPBOOT_SRC_Pos = 0x0
	// Bit mask of OTPBOOT_SRC field.
	OTP_DATA_OTPBOOT_SRC_OTPBOOT_SRC_Msk = 0xffff

	// OTPBOOT_LEN: Length in rows of the OTP boot image. (ECC) OTPBOOT_LEN must be even. The total image size must be a multiple of 4 bytes (32 bits).
	// Position of OTPBOOT_LEN field.
	OTP_DATA_OTPBOOT_LEN_OTPBOOT_LEN_Pos = 0x0
	// Bit mask of OTPBOOT_LEN field.
	OTP_DATA_OTPBOOT_LEN_OTPBOOT_LEN_Msk = 0xffff

	// OTPBOOT_DST0: Bits 15:0 of the OTP boot image load destination (and entry point). (ECC) This must be a location in main SRAM (main SRAM is addresses 0x20000000 through 0x20082000) and must be word-aligned.
	// Position of OTPBOOT_DST0 field.
	OTP_DATA_OTPBOOT_DST0_OTPBOOT_DST0_Pos = 0x0
	// Bit mask of OTPBOOT_DST0 field.
	OTP_DATA_OTPBOOT_DST0_OTPBOOT_DST0_Msk = 0xffff

	// OTPBOOT_DST1: Bits 31:16 of the OTP boot image load destination (and entry point). (ECC) This must be a location in main SRAM (main SRAM is addresses 0x20000000 through 0x20082000) and must be word-aligned.
	// Position of OTPBOOT_DST1 field.
	OTP_DATA_OTPBOOT_DST1_OTPBOOT_DST1_Pos = 0x0
	// Bit mask of OTPBOOT_DST1 field.
	OTP_DATA_OTPBOOT_DST1_OTPBOOT_DST1_Msk = 0xffff

	// BOOTKEY0_0: Bits 15:0 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_0 field.
	OTP_DATA_BOOTKEY0_0_BOOTKEY0_0_Pos = 0x0
	// Bit mask of BOOTKEY0_0 field.
	OTP_DATA_BOOTKEY0_0_BOOTKEY0_0_Msk = 0xffff

	// BOOTKEY0_1: Bits 31:16 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_1 field.
	OTP_DATA_BOOTKEY0_1_BOOTKEY0_1_Pos = 0x0
	// Bit mask of BOOTKEY0_1 field.
	OTP_DATA_BOOTKEY0_1_BOOTKEY0_1_Msk = 0xffff

	// BOOTKEY0_2: Bits 47:32 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_2 field.
	OTP_DATA_BOOTKEY0_2_BOOTKEY0_2_Pos = 0x0
	// Bit mask of BOOTKEY0_2 field.
	OTP_DATA_BOOTKEY0_2_BOOTKEY0_2_Msk = 0xffff

	// BOOTKEY0_3: Bits 63:48 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_3 field.
	OTP_DATA_BOOTKEY0_3_BOOTKEY0_3_Pos = 0x0
	// Bit mask of BOOTKEY0_3 field.
	OTP_DATA_BOOTKEY0_3_BOOTKEY0_3_Msk = 0xffff

	// BOOTKEY0_4: Bits 79:64 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_4 field.
	OTP_DATA_BOOTKEY0_4_BOOTKEY0_4_Pos = 0x0
	// Bit mask of BOOTKEY0_4 field.
	OTP_DATA_BOOTKEY0_4_BOOTKEY0_4_Msk = 0xffff

	// BOOTKEY0_5: Bits 95:80 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_5 field.
	OTP_DATA_BOOTKEY0_5_BOOTKEY0_5_Pos = 0x0
	// Bit mask of BOOTKEY0_5 field.
	OTP_DATA_BOOTKEY0_5_BOOTKEY0_5_Msk = 0xffff

	// BOOTKEY0_6: Bits 111:96 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_6 field.
	OTP_DATA_BOOTKEY0_6_BOOTKEY0_6_Pos = 0x0
	// Bit mask of BOOTKEY0_6 field.
	OTP_DATA_BOOTKEY0_6_BOOTKEY0_6_Msk = 0xffff

	// BOOTKEY0_7: Bits 127:112 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_7 field.
	OTP_DATA_BOOTKEY0_7_BOOTKEY0_7_Pos = 0x0
	// Bit mask of BOOTKEY0_7 field.
	OTP_DATA_BOOTKEY0_7_BOOTKEY0_7_Msk = 0xffff

	// BOOTKEY0_8: Bits 143:128 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_8 field.
	OTP_DATA_BOOTKEY0_8_BOOTKEY0_8_Pos = 0x0
	// Bit mask of BOOTKEY0_8 field.
	OTP_DATA_BOOTKEY0_8_BOOTKEY0_8_Msk = 0xffff

	// BOOTKEY0_9: Bits 159:144 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_9 field.
	OTP_DATA_BOOTKEY0_9_BOOTKEY0_9_Pos = 0x0
	// Bit mask of BOOTKEY0_9 field.
	OTP_DATA_BOOTKEY0_9_BOOTKEY0_9_Msk = 0xffff

	// BOOTKEY0_10: Bits 175:160 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_10 field.
	OTP_DATA_BOOTKEY0_10_BOOTKEY0_10_Pos = 0x0
	// Bit mask of BOOTKEY0_10 field.
	OTP_DATA_BOOTKEY0_10_BOOTKEY0_10_Msk = 0xffff

	// BOOTKEY0_11: Bits 191:176 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_11 field.
	OTP_DATA_BOOTKEY0_11_BOOTKEY0_11_Pos = 0x0
	// Bit mask of BOOTKEY0_11 field.
	OTP_DATA_BOOTKEY0_11_BOOTKEY0_11_Msk = 0xffff

	// BOOTKEY0_12: Bits 207:192 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_12 field.
	OTP_DATA_BOOTKEY0_12_BOOTKEY0_12_Pos = 0x0
	// Bit mask of BOOTKEY0_12 field.
	OTP_DATA_BOOTKEY0_12_BOOTKEY0_12_Msk = 0xffff

	// BOOTKEY0_13: Bits 223:208 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_13 field.
	OTP_DATA_BOOTKEY0_13_BOOTKEY0_13_Pos = 0x0
	// Bit mask of BOOTKEY0_13 field.
	OTP_DATA_BOOTKEY0_13_BOOTKEY0_13_Msk = 0xffff

	// BOOTKEY0_14: Bits 239:224 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_14 field.
	OTP_DATA_BOOTKEY0_14_BOOTKEY0_14_Pos = 0x0
	// Bit mask of BOOTKEY0_14 field.
	OTP_DATA_BOOTKEY0_14_BOOTKEY0_14_Msk = 0xffff

	// BOOTKEY0_15: Bits 255:240 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_15 field.
	OTP_DATA_BOOTKEY0_15_BOOTKEY0_15_Pos = 0x0
	// Bit mask of BOOTKEY0_15 field.
	OTP_DATA_BOOTKEY0_15_BOOTKEY0_15_Msk = 0xffff

	// BOOTKEY1_0: Bits 15:0 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_0 field.
	OTP_DATA_BOOTKEY1_0_BOOTKEY1_0_Pos = 0x0
	// Bit mask of BOOTKEY1_0 field.
	OTP_DATA_BOOTKEY1_0_BOOTKEY1_0_Msk = 0xffff

	// BOOTKEY1_1: Bits 31:16 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_1 field.
	OTP_DATA_BOOTKEY1_1_BOOTKEY1_1_Pos = 0x0
	// Bit mask of BOOTKEY1_1 field.
	OTP_DATA_BOOTKEY1_1_BOOTKEY1_1_Msk = 0xffff

	// BOOTKEY1_2: Bits 47:32 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_2 field.
	OTP_DATA_BOOTKEY1_2_BOOTKEY1_2_Pos = 0x0
	// Bit mask of BOOTKEY1_2 field.
	OTP_DATA_BOOTKEY1_2_BOOTKEY1_2_Msk = 0xffff

	// BOOTKEY1_3: Bits 63:48 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_3 field.
	OTP_DATA_BOOTKEY1_3_BOOTKEY1_3_Pos = 0x0
	// Bit mask of BOOTKEY1_3 field.
	OTP_DATA_BOOTKEY1_3_BOOTKEY1_3_Msk = 0xffff

	// BOOTKEY1_4: Bits 79:64 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_4 field.
	OTP_DATA_BOOTKEY1_4_BOOTKEY1_4_Pos = 0x0
	// Bit mask of BOOTKEY1_4 field.
	OTP_DATA_BOOTKEY1_4_BOOTKEY1_4_Msk = 0xffff

	// BOOTKEY1_5: Bits 95:80 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_5 field.
	OTP_DATA_BOOTKEY1_5_BOOTKEY1_5_Pos = 0x0
	// Bit mask of BOOTKEY1_5 field.
	OTP_DATA_BOOTKEY1_5_BOOTKEY1_5_Msk = 0xffff

	// BOOTKEY1_6: Bits 111:96 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_6 field.
	OTP_DATA_BOOTKEY1_6_BOOTKEY1_6_Pos = 0x0
	// Bit mask of BOOTKEY1_6 field.
	OTP_DATA_BOOTKEY1_6_BOOTKEY1_6_Msk = 0xffff

	// BOOTKEY1_7: Bits 127:112 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_7 field.
	OTP_DATA_BOOTKEY1_7_BOOTKEY1_7_Pos = 0x0
	// Bit mask of BOOTKEY1_7 field.
	OTP_DATA_BOOTKEY1_7_BOOTKEY1_7_Msk = 0xffff

	// BOOTKEY1_8: Bits 143:128 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_8 field.
	OTP_DATA_BOOTKEY1_8_BOOTKEY1_8_Pos = 0x0
	// Bit mask of BOOTKEY1_8 field.
	OTP_DATA_BOOTKEY1_8_BOOTKEY1_8_Msk = 0xffff

	// BOOTKEY1_9: Bits 159:144 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_9 field.
	OTP_DATA_BOOTKEY1_9_BOOTKEY1_9_Pos = 0x0
	// Bit mask of BOOTKEY1_9 field.
	OTP_DATA_BOOTKEY1_9_BOOTKEY1_9_Msk = 0xffff

	// BOOTKEY1_10: Bits 175:160 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_10 field.
	OTP_DATA_BOOTKEY1_10_BOOTKEY1_10_Pos = 0x0
	// Bit mask of BOOTKEY1_10 field.
	OTP_DATA_BOOTKEY1_10_BOOTKEY1_10_Msk = 0xffff

	// BOOTKEY1_11: Bits 191:176 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_11 field.
	OTP_DATA_BOOTKEY1_11_BOOTKEY1_11_Pos = 0x0
	// Bit mask of BOOTKEY1_11 field.
	OTP_DATA_BOOTKEY1_11_BOOTKEY1_11_Msk = 0xffff

	// BOOTKEY1_12: Bits 207:192 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_12 field.
	OTP_DATA_BOOTKEY1_12_BOOTKEY1_12_Pos = 0x0
	// Bit mask of BOOTKEY1_12 field.
	OTP_DATA_BOOTKEY1_12_BOOTKEY1_12_Msk = 0xffff

	// BOOTKEY1_13: Bits 223:208 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_13 field.
	OTP_DATA_BOOTKEY1_13_BOOTKEY1_13_Pos = 0x0
	// Bit mask of BOOTKEY1_13 field.
	OTP_DATA_BOOTKEY1_13_BOOTKEY1_13_Msk = 0xffff

	// BOOTKEY1_14: Bits 239:224 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_14 field.
	OTP_DATA_BOOTKEY1_14_BOOTKEY1_14_Pos = 0x0
	// Bit mask of BOOTKEY1_14 field.
	OTP_DATA_BOOTKEY1_14_BOOTKEY1_14_Msk = 0xffff

	// BOOTKEY1_15: Bits 255:240 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_15 field.
	OTP_DATA_BOOTKEY1_15_BOOTKEY1_15_Pos = 0x0
	// Bit mask of BOOTKEY1_15 field.
	OTP_DATA_BOOTKEY1_15_BOOTKEY1_15_Msk = 0xffff

	// BOOTKEY2_0: Bits 15:0 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_0 field.
	OTP_DATA_BOOTKEY2_0_BOOTKEY2_0_Pos = 0x0
	// Bit mask of BOOTKEY2_0 field.
	OTP_DATA_BOOTKEY2_0_BOOTKEY2_0_Msk = 0xffff

	// BOOTKEY2_1: Bits 31:16 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_1 field.
	OTP_DATA_BOOTKEY2_1_BOOTKEY2_1_Pos = 0x0
	// Bit mask of BOOTKEY2_1 field.
	OTP_DATA_BOOTKEY2_1_BOOTKEY2_1_Msk = 0xffff

	// BOOTKEY2_2: Bits 47:32 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_2 field.
	OTP_DATA_BOOTKEY2_2_BOOTKEY2_2_Pos = 0x0
	// Bit mask of BOOTKEY2_2 field.
	OTP_DATA_BOOTKEY2_2_BOOTKEY2_2_Msk = 0xffff

	// BOOTKEY2_3: Bits 63:48 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_3 field.
	OTP_DATA_BOOTKEY2_3_BOOTKEY2_3_Pos = 0x0
	// Bit mask of BOOTKEY2_3 field.
	OTP_DATA_BOOTKEY2_3_BOOTKEY2_3_Msk = 0xffff

	// BOOTKEY2_4: Bits 79:64 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_4 field.
	OTP_DATA_BOOTKEY2_4_BOOTKEY2_4_Pos = 0x0
	// Bit mask of BOOTKEY2_4 field.
	OTP_DATA_BOOTKEY2_4_BOOTKEY2_4_Msk = 0xffff

	// BOOTKEY2_5: Bits 95:80 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_5 field.
	OTP_DATA_BOOTKEY2_5_BOOTKEY2_5_Pos = 0x0
	// Bit mask of BOOTKEY2_5 field.
	OTP_DATA_BOOTKEY2_5_BOOTKEY2_5_Msk = 0xffff

	// BOOTKEY2_6: Bits 111:96 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_6 field.
	OTP_DATA_BOOTKEY2_6_BOOTKEY2_6_Pos = 0x0
	// Bit mask of BOOTKEY2_6 field.
	OTP_DATA_BOOTKEY2_6_BOOTKEY2_6_Msk = 0xffff

	// BOOTKEY2_7: Bits 127:112 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_7 field.
	OTP_DATA_BOOTKEY2_7_BOOTKEY2_7_Pos = 0x0
	// Bit mask of BOOTKEY2_7 field.
	OTP_DATA_BOOTKEY2_7_BOOTKEY2_7_Msk = 0xffff

	// BOOTKEY2_8: Bits 143:128 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_8 field.
	OTP_DATA_BOOTKEY2_8_BOOTKEY2_8_Pos = 0x0
	// Bit mask of BOOTKEY2_8 field.
	OTP_DATA_BOOTKEY2_8_BOOTKEY2_8_Msk = 0xffff

	// BOOTKEY2_9: Bits 159:144 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_9 field.
	OTP_DATA_BOOTKEY2_9_BOOTKEY2_9_Pos = 0x0
	// Bit mask of BOOTKEY2_9 field.
	OTP_DATA_BOOTKEY2_9_BOOTKEY2_9_Msk = 0xffff

	// BOOTKEY2_10: Bits 175:160 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_10 field.
	OTP_DATA_BOOTKEY2_10_BOOTKEY2_10_Pos = 0x0
	// Bit mask of BOOTKEY2_10 field.
	OTP_DATA_BOOTKEY2_10_BOOTKEY2_10_Msk = 0xffff

	// BOOTKEY2_11: Bits 191:176 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_11 field.
	OTP_DATA_BOOTKEY2_11_BOOTKEY2_11_Pos = 0x0
	// Bit mask of BOOTKEY2_11 field.
	OTP_DATA_BOOTKEY2_11_BOOTKEY2_11_Msk = 0xffff

	// BOOTKEY2_12: Bits 207:192 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_12 field.
	OTP_DATA_BOOTKEY2_12_BOOTKEY2_12_Pos = 0x0
	// Bit mask of BOOTKEY2_12 field.
	OTP_DATA_BOOTKEY2_12_BOOTKEY2_12_Msk = 0xffff

	// BOOTKEY2_13: Bits 223:208 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_13 field.
	OTP_DATA_BOOTKEY2_13_BOOTKEY2_13_Pos = 0x0
	// Bit mask of BOOTKEY2_13 field.
	OTP_DATA_BOOTKEY2_13_BOOTKEY2_13_Msk = 0xffff

	// BOOTKEY2_14: Bits 239:224 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_14 field.
	OTP_DATA_BOOTKEY2_14_BOOTKEY2_14_Pos = 0x0
	// Bit mask of BOOTKEY2_14 field.
	OTP_DATA_BOOTKEY2_14_BOOTKEY2_14_Msk = 0xffff

	// BOOTKEY2_15: Bits 255:240 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_15 field.
	OTP_DATA_BOOTKEY2_15_BOOTKEY2_15_Pos = 0x0
	// Bit mask of BOOTKEY2_15 field.
	OTP_DATA_BOOTKEY2_15_BOOTKEY2_15_Msk = 0xffff

	// BOOTKEY3_0: Bits 15:0 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_0 field.
	OTP_DATA_BOOTKEY3_0_BOOTKEY3_0_Pos = 0x0
	// Bit mask of BOOTKEY3_0 field.
	OTP_DATA_BOOTKEY3_0_BOOTKEY3_0_Msk = 0xffff

	// BOOTKEY3_1: Bits 31:16 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_1 field.
	OTP_DATA_BOOTKEY3_1_BOOTKEY3_1_Pos = 0x0
	// Bit mask of BOOTKEY3_1 field.
	OTP_DATA_BOOTKEY3_1_BOOTKEY3_1_Msk = 0xffff

	// BOOTKEY3_2: Bits 47:32 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_2 field.
	OTP_DATA_BOOTKEY3_2_BOOTKEY3_2_Pos = 0x0
	// Bit mask of BOOTKEY3_2 field.
	OTP_DATA_BOOTKEY3_2_BOOTKEY3_2_Msk = 0xffff

	// BOOTKEY3_3: Bits 63:48 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_3 field.
	OTP_DATA_BOOTKEY3_3_BOOTKEY3_3_Pos = 0x0
	// Bit mask of BOOTKEY3_3 field.
	OTP_DATA_BOOTKEY3_3_BOOTKEY3_3_Msk = 0xffff

	// BOOTKEY3_4: Bits 79:64 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_4 field.
	OTP_DATA_BOOTKEY3_4_BOOTKEY3_4_Pos = 0x0
	// Bit mask of BOOTKEY3_4 field.
	OTP_DATA_BOOTKEY3_4_BOOTKEY3_4_Msk = 0xffff

	// BOOTKEY3_5: Bits 95:80 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_5 field.
	OTP_DATA_BOOTKEY3_5_BOOTKEY3_5_Pos = 0x0
	// Bit mask of BOOTKEY3_5 field.
	OTP_DATA_BOOTKEY3_5_BOOTKEY3_5_Msk = 0xffff

	// BOOTKEY3_6: Bits 111:96 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_6 field.
	OTP_DATA_BOOTKEY3_6_BOOTKEY3_6_Pos = 0x0
	// Bit mask of BOOTKEY3_6 field.
	OTP_DATA_BOOTKEY3_6_BOOTKEY3_6_Msk = 0xffff

	// BOOTKEY3_7: Bits 127:112 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_7 field.
	OTP_DATA_BOOTKEY3_7_BOOTKEY3_7_Pos = 0x0
	// Bit mask of BOOTKEY3_7 field.
	OTP_DATA_BOOTKEY3_7_BOOTKEY3_7_Msk = 0xffff

	// BOOTKEY3_8: Bits 143:128 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_8 field.
	OTP_DATA_BOOTKEY3_8_BOOTKEY3_8_Pos = 0x0
	// Bit mask of BOOTKEY3_8 field.
	OTP_DATA_BOOTKEY3_8_BOOTKEY3_8_Msk = 0xffff

	// BOOTKEY3_9: Bits 159:144 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_9 field.
	OTP_DATA_BOOTKEY3_9_BOOTKEY3_9_Pos = 0x0
	// Bit mask of BOOTKEY3_9 field.
	OTP_DATA_BOOTKEY3_9_BOOTKEY3_9_Msk = 0xffff

	// BOOTKEY3_10: Bits 175:160 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_10 field.
	OTP_DATA_BOOTKEY3_10_BOOTKEY3_10_Pos = 0x0
	// Bit mask of BOOTKEY3_10 field.
	OTP_DATA_BOOTKEY3_10_BOOTKEY3_10_Msk = 0xffff

	// BOOTKEY3_11: Bits 191:176 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_11 field.
	OTP_DATA_BOOTKEY3_11_BOOTKEY3_11_Pos = 0x0
	// Bit mask of BOOTKEY3_11 field.
	OTP_DATA_BOOTKEY3_11_BOOTKEY3_11_Msk = 0xffff

	// BOOTKEY3_12: Bits 207:192 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_12 field.
	OTP_DATA_BOOTKEY3_12_BOOTKEY3_12_Pos = 0x0
	// Bit mask of BOOTKEY3_12 field.
	OTP_DATA_BOOTKEY3_12_BOOTKEY3_12_Msk = 0xffff

	// BOOTKEY3_13: Bits 223:208 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_13 field.
	OTP_DATA_BOOTKEY3_13_BOOTKEY3_13_Pos = 0x0
	// Bit mask of BOOTKEY3_13 field.
	OTP_DATA_BOOTKEY3_13_BOOTKEY3_13_Msk = 0xffff

	// BOOTKEY3_14: Bits 239:224 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_14 field.
	OTP_DATA_BOOTKEY3_14_BOOTKEY3_14_Pos = 0x0
	// Bit mask of BOOTKEY3_14 field.
	OTP_DATA_BOOTKEY3_14_BOOTKEY3_14_Msk = 0xffff

	// BOOTKEY3_15: Bits 255:240 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_15 field.
	OTP_DATA_BOOTKEY3_15_BOOTKEY3_15_Pos = 0x0
	// Bit mask of BOOTKEY3_15 field.
	OTP_DATA_BOOTKEY3_15_BOOTKEY3_15_Msk = 0xffff

	// KEY1_0: Bits 15:0 of OTP access key 1 (ECC)
	// Position of KEY1_0 field.
	OTP_DATA_KEY1_0_KEY1_0_Pos = 0x0
	// Bit mask of KEY1_0 field.
	OTP_DATA_KEY1_0_KEY1_0_Msk = 0xffff

	// KEY1_1: Bits 31:16 of OTP access key 1 (ECC)
	// Position of KEY1_1 field.
	OTP_DATA_KEY1_1_KEY1_1_Pos = 0x0
	// Bit mask of KEY1_1 field.
	OTP_DATA_KEY1_1_KEY1_1_Msk = 0xffff

	// KEY1_2: Bits 47:32 of OTP access key 1 (ECC)
	// Position of KEY1_2 field.
	OTP_DATA_KEY1_2_KEY1_2_Pos = 0x0
	// Bit mask of KEY1_2 field.
	OTP_DATA_KEY1_2_KEY1_2_Msk = 0xffff

	// KEY1_3: Bits 63:48 of OTP access key 1 (ECC)
	// Position of KEY1_3 field.
	OTP_DATA_KEY1_3_KEY1_3_Pos = 0x0
	// Bit mask of KEY1_3 field.
	OTP_DATA_KEY1_3_KEY1_3_Msk = 0xffff

	// KEY1_4: Bits 79:64 of OTP access key 1 (ECC)
	// Position of KEY1_4 field.
	OTP_DATA_KEY1_4_KEY1_4_Pos = 0x0
	// Bit mask of KEY1_4 field.
	OTP_DATA_KEY1_4_KEY1_4_Msk = 0xffff

	// KEY1_5: Bits 95:80 of OTP access key 1 (ECC)
	// Position of KEY1_5 field.
	OTP_DATA_KEY1_5_KEY1_5_Pos = 0x0
	// Bit mask of KEY1_5 field.
	OTP_DATA_KEY1_5_KEY1_5_Msk = 0xffff

	// KEY1_6: Bits 111:96 of OTP access key 1 (ECC)
	// Position of KEY1_6 field.
	OTP_DATA_KEY1_6_KEY1_6_Pos = 0x0
	// Bit mask of KEY1_6 field.
	OTP_DATA_KEY1_6_KEY1_6_Msk = 0xffff

	// KEY1_7: Bits 127:112 of OTP access key 1 (ECC)
	// Position of KEY1_7 field.
	OTP_DATA_KEY1_7_KEY1_7_Pos = 0x0
	// Bit mask of KEY1_7 field.
	OTP_DATA_KEY1_7_KEY1_7_Msk = 0xffff

	// KEY2_0: Bits 15:0 of OTP access key 2 (ECC)
	// Position of KEY2_0 field.
	OTP_DATA_KEY2_0_KEY2_0_Pos = 0x0
	// Bit mask of KEY2_0 field.
	OTP_DATA_KEY2_0_KEY2_0_Msk = 0xffff

	// KEY2_1: Bits 31:16 of OTP access key 2 (ECC)
	// Position of KEY2_1 field.
	OTP_DATA_KEY2_1_KEY2_1_Pos = 0x0
	// Bit mask of KEY2_1 field.
	OTP_DATA_KEY2_1_KEY2_1_Msk = 0xffff

	// KEY2_2: Bits 47:32 of OTP access key 2 (ECC)
	// Position of KEY2_2 field.
	OTP_DATA_KEY2_2_KEY2_2_Pos = 0x0
	// Bit mask of KEY2_2 field.
	OTP_DATA_KEY2_2_KEY2_2_Msk = 0xffff

	// KEY2_3: Bits 63:48 of OTP access key 2 (ECC)
	// Position of KEY2_3 field.
	OTP_DATA_KEY2_3_KEY2_3_Pos = 0x0
	// Bit mask of KEY2_3 field.
	OTP_DATA_KEY2_3_KEY2_3_Msk = 0xffff

	// KEY2_4: Bits 79:64 of OTP access key 2 (ECC)
	// Position of KEY2_4 field.
	OTP_DATA_KEY2_4_KEY2_4_Pos = 0x0
	// Bit mask of KEY2_4 field.
	OTP_DATA_KEY2_4_KEY2_4_Msk = 0xffff

	// KEY2_5: Bits 95:80 of OTP access key 2 (ECC)
	// Position of KEY2_5 field.
	OTP_DATA_KEY2_5_KEY2_5_Pos = 0x0
	// Bit mask of KEY2_5 field.
	OTP_DATA_KEY2_5_KEY2_5_Msk = 0xffff

	// KEY2_6: Bits 111:96 of OTP access key 2 (ECC)
	// Position of KEY2_6 field.
	OTP_DATA_KEY2_6_KEY2_6_Pos = 0x0
	// Bit mask of KEY2_6 field.
	OTP_DATA_KEY2_6_KEY2_6_Msk = 0xffff

	// KEY2_7: Bits 127:112 of OTP access key 2 (ECC)
	// Position of KEY2_7 field.
	OTP_DATA_KEY2_7_KEY2_7_Pos = 0x0
	// Bit mask of KEY2_7 field.
	OTP_DATA_KEY2_7_KEY2_7_Msk = 0xffff

	// KEY3_0: Bits 15:0 of OTP access key 3 (ECC)
	// Position of KEY3_0 field.
	OTP_DATA_KEY3_0_KEY3_0_Pos = 0x0
	// Bit mask of KEY3_0 field.
	OTP_DATA_KEY3_0_KEY3_0_Msk = 0xffff

	// KEY3_1: Bits 31:16 of OTP access key 3 (ECC)
	// Position of KEY3_1 field.
	OTP_DATA_KEY3_1_KEY3_1_Pos = 0x0
	// Bit mask of KEY3_1 field.
	OTP_DATA_KEY3_1_KEY3_1_Msk = 0xffff

	// KEY3_2: Bits 47:32 of OTP access key 3 (ECC)
	// Position of KEY3_2 field.
	OTP_DATA_KEY3_2_KEY3_2_Pos = 0x0
	// Bit mask of KEY3_2 field.
	OTP_DATA_KEY3_2_KEY3_2_Msk = 0xffff

	// KEY3_3: Bits 63:48 of OTP access key 3 (ECC)
	// Position of KEY3_3 field.
	OTP_DATA_KEY3_3_KEY3_3_Pos = 0x0
	// Bit mask of KEY3_3 field.
	OTP_DATA_KEY3_3_KEY3_3_Msk = 0xffff

	// KEY3_4: Bits 79:64 of OTP access key 3 (ECC)
	// Position of KEY3_4 field.
	OTP_DATA_KEY3_4_KEY3_4_Pos = 0x0
	// Bit mask of KEY3_4 field.
	OTP_DATA_KEY3_4_KEY3_4_Msk = 0xffff

	// KEY3_5: Bits 95:80 of OTP access key 3 (ECC)
	// Position of KEY3_5 field.
	OTP_DATA_KEY3_5_KEY3_5_Pos = 0x0
	// Bit mask of KEY3_5 field.
	OTP_DATA_KEY3_5_KEY3_5_Msk = 0xffff

	// KEY3_6: Bits 111:96 of OTP access key 3 (ECC)
	// Position of KEY3_6 field.
	OTP_DATA_KEY3_6_KEY3_6_Pos = 0x0
	// Bit mask of KEY3_6 field.
	OTP_DATA_KEY3_6_KEY3_6_Msk = 0xffff

	// KEY3_7: Bits 127:112 of OTP access key 3 (ECC)
	// Position of KEY3_7 field.
	OTP_DATA_KEY3_7_KEY3_7_Pos = 0x0
	// Bit mask of KEY3_7 field.
	OTP_DATA_KEY3_7_KEY3_7_Msk = 0xffff

	// KEY4_0: Bits 15:0 of OTP access key 4 (ECC)
	// Position of KEY4_0 field.
	OTP_DATA_KEY4_0_KEY4_0_Pos = 0x0
	// Bit mask of KEY4_0 field.
	OTP_DATA_KEY4_0_KEY4_0_Msk = 0xffff

	// KEY4_1: Bits 31:16 of OTP access key 4 (ECC)
	// Position of KEY4_1 field.
	OTP_DATA_KEY4_1_KEY4_1_Pos = 0x0
	// Bit mask of KEY4_1 field.
	OTP_DATA_KEY4_1_KEY4_1_Msk = 0xffff

	// KEY4_2: Bits 47:32 of OTP access key 4 (ECC)
	// Position of KEY4_2 field.
	OTP_DATA_KEY4_2_KEY4_2_Pos = 0x0
	// Bit mask of KEY4_2 field.
	OTP_DATA_KEY4_2_KEY4_2_Msk = 0xffff

	// KEY4_3: Bits 63:48 of OTP access key 4 (ECC)
	// Position of KEY4_3 field.
	OTP_DATA_KEY4_3_KEY4_3_Pos = 0x0
	// Bit mask of KEY4_3 field.
	OTP_DATA_KEY4_3_KEY4_3_Msk = 0xffff

	// KEY4_4: Bits 79:64 of OTP access key 4 (ECC)
	// Position of KEY4_4 field.
	OTP_DATA_KEY4_4_KEY4_4_Pos = 0x0
	// Bit mask of KEY4_4 field.
	OTP_DATA_KEY4_4_KEY4_4_Msk = 0xffff

	// KEY4_5: Bits 95:80 of OTP access key 4 (ECC)
	// Position of KEY4_5 field.
	OTP_DATA_KEY4_5_KEY4_5_Pos = 0x0
	// Bit mask of KEY4_5 field.
	OTP_DATA_KEY4_5_KEY4_5_Msk = 0xffff

	// KEY4_6: Bits 111:96 of OTP access key 4 (ECC)
	// Position of KEY4_6 field.
	OTP_DATA_KEY4_6_KEY4_6_Pos = 0x0
	// Bit mask of KEY4_6 field.
	OTP_DATA_KEY4_6_KEY4_6_Msk = 0xffff

	// KEY4_7: Bits 127:112 of OTP access key 4 (ECC)
	// Position of KEY4_7 field.
	OTP_DATA_KEY4_7_KEY4_7_Pos = 0x0
	// Bit mask of KEY4_7 field.
	OTP_DATA_KEY4_7_KEY4_7_Msk = 0xffff

	// KEY5_0: Bits 15:0 of OTP access key 5 (ECC)
	// Position of KEY5_0 field.
	OTP_DATA_KEY5_0_KEY5_0_Pos = 0x0
	// Bit mask of KEY5_0 field.
	OTP_DATA_KEY5_0_KEY5_0_Msk = 0xffff

	// KEY5_1: Bits 31:16 of OTP access key 5 (ECC)
	// Position of KEY5_1 field.
	OTP_DATA_KEY5_1_KEY5_1_Pos = 0x0
	// Bit mask of KEY5_1 field.
	OTP_DATA_KEY5_1_KEY5_1_Msk = 0xffff

	// KEY5_2: Bits 47:32 of OTP access key 5 (ECC)
	// Position of KEY5_2 field.
	OTP_DATA_KEY5_2_KEY5_2_Pos = 0x0
	// Bit mask of KEY5_2 field.
	OTP_DATA_KEY5_2_KEY5_2_Msk = 0xffff

	// KEY5_3: Bits 63:48 of OTP access key 5 (ECC)
	// Position of KEY5_3 field.
	OTP_DATA_KEY5_3_KEY5_3_Pos = 0x0
	// Bit mask of KEY5_3 field.
	OTP_DATA_KEY5_3_KEY5_3_Msk = 0xffff

	// KEY5_4: Bits 79:64 of OTP access key 5 (ECC)
	// Position of KEY5_4 field.
	OTP_DATA_KEY5_4_KEY5_4_Pos = 0x0
	// Bit mask of KEY5_4 field.
	OTP_DATA_KEY5_4_KEY5_4_Msk = 0xffff

	// KEY5_5: Bits 95:80 of OTP access key 5 (ECC)
	// Position of KEY5_5 field.
	OTP_DATA_KEY5_5_KEY5_5_Pos = 0x0
	// Bit mask of KEY5_5 field.
	OTP_DATA_KEY5_5_KEY5_5_Msk = 0xffff

	// KEY5_6: Bits 111:96 of OTP access key 5 (ECC)
	// Position of KEY5_6 field.
	OTP_DATA_KEY5_6_KEY5_6_Pos = 0x0
	// Bit mask of KEY5_6 field.
	OTP_DATA_KEY5_6_KEY5_6_Msk = 0xffff

	// KEY5_7: Bits 127:112 of OTP access key 5 (ECC)
	// Position of KEY5_7 field.
	OTP_DATA_KEY5_7_KEY5_7_Pos = 0x0
	// Bit mask of KEY5_7 field.
	OTP_DATA_KEY5_7_KEY5_7_Msk = 0xffff

	// KEY6_0: Bits 15:0 of OTP access key 6 (ECC)
	// Position of KEY6_0 field.
	OTP_DATA_KEY6_0_KEY6_0_Pos = 0x0
	// Bit mask of KEY6_0 field.
	OTP_DATA_KEY6_0_KEY6_0_Msk = 0xffff

	// KEY6_1: Bits 31:16 of OTP access key 6 (ECC)
	// Position of KEY6_1 field.
	OTP_DATA_KEY6_1_KEY6_1_Pos = 0x0
	// Bit mask of KEY6_1 field.
	OTP_DATA_KEY6_1_KEY6_1_Msk = 0xffff

	// KEY6_2: Bits 47:32 of OTP access key 6 (ECC)
	// Position of KEY6_2 field.
	OTP_DATA_KEY6_2_KEY6_2_Pos = 0x0
	// Bit mask of KEY6_2 field.
	OTP_DATA_KEY6_2_KEY6_2_Msk = 0xffff

	// KEY6_3: Bits 63:48 of OTP access key 6 (ECC)
	// Position of KEY6_3 field.
	OTP_DATA_KEY6_3_KEY6_3_Pos = 0x0
	// Bit mask of KEY6_3 field.
	OTP_DATA_KEY6_3_KEY6_3_Msk = 0xffff

	// KEY6_4: Bits 79:64 of OTP access key 6 (ECC)
	// Position of KEY6_4 field.
	OTP_DATA_KEY6_4_KEY6_4_Pos = 0x0
	// Bit mask of KEY6_4 field.
	OTP_DATA_KEY6_4_KEY6_4_Msk = 0xffff

	// KEY6_5: Bits 95:80 of OTP access key 6 (ECC)
	// Position of KEY6_5 field.
	OTP_DATA_KEY6_5_KEY6_5_Pos = 0x0
	// Bit mask of KEY6_5 field.
	OTP_DATA_KEY6_5_KEY6_5_Msk = 0xffff

	// KEY6_6: Bits 111:96 of OTP access key 6 (ECC)
	// Position of KEY6_6 field.
	OTP_DATA_KEY6_6_KEY6_6_Pos = 0x0
	// Bit mask of KEY6_6 field.
	OTP_DATA_KEY6_6_KEY6_6_Msk = 0xffff

	// KEY6_7: Bits 127:112 of OTP access key 6 (ECC)
	// Position of KEY6_7 field.
	OTP_DATA_KEY6_7_KEY6_7_Pos = 0x0
	// Bit mask of KEY6_7 field.
	OTP_DATA_KEY6_7_KEY6_7_Msk = 0xffff
)

// Constants for OTP_DATA_RAW: Predefined OTP data layout for RP2350
const (
	// CHIPID0: Bits 15:0 of public device ID. (ECC) The CHIPID0..3 rows contain a 64-bit random identifier for this chip, which can be read from the USB bootloader PICOBOOT interface or from the get_sys_info ROM API. The number of random bits makes the occurrence of twins exceedingly unlikely: for example, a fleet of a hundred million devices has a 99.97% probability of no twinned IDs. This is estimated to be lower than the occurrence of process errors in the assignment of sequential random IDs, and for practical purposes CHIPID may be treated as unique.
	// Position of CHIPID0 field.
	OTP_DATA_RAW_CHIPID0_CHIPID0_Pos = 0x0
	// Bit mask of CHIPID0 field.
	OTP_DATA_RAW_CHIPID0_CHIPID0_Msk = 0xffff

	// CHIPID1: Bits 31:16 of public device ID (ECC)
	// Position of CHIPID1 field.
	OTP_DATA_RAW_CHIPID1_CHIPID1_Pos = 0x0
	// Bit mask of CHIPID1 field.
	OTP_DATA_RAW_CHIPID1_CHIPID1_Msk = 0xffff

	// CHIPID2: Bits 47:32 of public device ID (ECC)
	// Position of CHIPID2 field.
	OTP_DATA_RAW_CHIPID2_CHIPID2_Pos = 0x0
	// Bit mask of CHIPID2 field.
	OTP_DATA_RAW_CHIPID2_CHIPID2_Msk = 0xffff

	// CHIPID3: Bits 63:48 of public device ID (ECC)
	// Position of CHIPID3 field.
	OTP_DATA_RAW_CHIPID3_CHIPID3_Pos = 0x0
	// Bit mask of CHIPID3 field.
	OTP_DATA_RAW_CHIPID3_CHIPID3_Msk = 0xffff

	// RANDID0: Bits 15:0 of private per-device random number (ECC) The RANDID0..7 rows form a 128-bit random number generated during device test. This ID is not exposed through the USB PICOBOOT GET_INFO command or the ROM `get_sys_info()` API. However note that the USB PICOBOOT OTP access point can read the entirety of page 0, so this value is not meaningfully private unless the USB PICOBOOT interface is disabled via the DISABLE_BOOTSEL_USB_PICOBOOT_IFC flag in BOOT_FLAGS0.
	// Position of RANDID0 field.
	OTP_DATA_RAW_RANDID0_RANDID0_Pos = 0x0
	// Bit mask of RANDID0 field.
	OTP_DATA_RAW_RANDID0_RANDID0_Msk = 0xffff

	// RANDID1: Bits 31:16 of private per-device random number (ECC)
	// Position of RANDID1 field.
	OTP_DATA_RAW_RANDID1_RANDID1_Pos = 0x0
	// Bit mask of RANDID1 field.
	OTP_DATA_RAW_RANDID1_RANDID1_Msk = 0xffff

	// RANDID2: Bits 47:32 of private per-device random number (ECC)
	// Position of RANDID2 field.
	OTP_DATA_RAW_RANDID2_RANDID2_Pos = 0x0
	// Bit mask of RANDID2 field.
	OTP_DATA_RAW_RANDID2_RANDID2_Msk = 0xffff

	// RANDID3: Bits 63:48 of private per-device random number (ECC)
	// Position of RANDID3 field.
	OTP_DATA_RAW_RANDID3_RANDID3_Pos = 0x0
	// Bit mask of RANDID3 field.
	OTP_DATA_RAW_RANDID3_RANDID3_Msk = 0xffff

	// RANDID4: Bits 79:64 of private per-device random number (ECC)
	// Position of RANDID4 field.
	OTP_DATA_RAW_RANDID4_RANDID4_Pos = 0x0
	// Bit mask of RANDID4 field.
	OTP_DATA_RAW_RANDID4_RANDID4_Msk = 0xffff

	// RANDID5: Bits 95:80 of private per-device random number (ECC)
	// Position of RANDID5 field.
	OTP_DATA_RAW_RANDID5_RANDID5_Pos = 0x0
	// Bit mask of RANDID5 field.
	OTP_DATA_RAW_RANDID5_RANDID5_Msk = 0xffff

	// RANDID6: Bits 111:96 of private per-device random number (ECC)
	// Position of RANDID6 field.
	OTP_DATA_RAW_RANDID6_RANDID6_Pos = 0x0
	// Bit mask of RANDID6 field.
	OTP_DATA_RAW_RANDID6_RANDID6_Msk = 0xffff

	// RANDID7: Bits 127:112 of private per-device random number (ECC)
	// Position of RANDID7 field.
	OTP_DATA_RAW_RANDID7_RANDID7_Pos = 0x0
	// Bit mask of RANDID7 field.
	OTP_DATA_RAW_RANDID7_RANDID7_Msk = 0xffff

	// ROSC_CALIB: Ring oscillator frequency in kHz, measured during manufacturing (ECC) This is measured at 1.1 V, at room temperature, with the ROSC configuration registers in their reset state.
	// Position of ROSC_CALIB field.
	OTP_DATA_RAW_ROSC_CALIB_ROSC_CALIB_Pos = 0x0
	// Bit mask of ROSC_CALIB field.
	OTP_DATA_RAW_ROSC_CALIB_ROSC_CALIB_Msk = 0xffff

	// LPOSC_CALIB: Low-power oscillator frequency in Hz, measured during manufacturing (ECC) This is measured at 1.1V, at room temperature, with the LPOSC trim register in its reset state.
	// Position of LPOSC_CALIB field.
	OTP_DATA_RAW_LPOSC_CALIB_LPOSC_CALIB_Pos = 0x0
	// Bit mask of LPOSC_CALIB field.
	OTP_DATA_RAW_LPOSC_CALIB_LPOSC_CALIB_Msk = 0xffff

	// NUM_GPIOS: The number of main user GPIOs (bank 0). Should read 48 in the QFN80 package, and 30 in the QFN60 package. (ECC)
	// Position of NUM_GPIOS field.
	OTP_DATA_RAW_NUM_GPIOS_NUM_GPIOS_Pos = 0x0
	// Bit mask of NUM_GPIOS field.
	OTP_DATA_RAW_NUM_GPIOS_NUM_GPIOS_Msk = 0xff

	// INFO_CRC0: Lower 16 bits of CRC32 of OTP addresses 0x00 through 0x6b (polynomial 0x4c11db7, input reflected, output reflected, seed all-ones, final XOR all-ones) (ECC)
	// Position of INFO_CRC0 field.
	OTP_DATA_RAW_INFO_CRC0_INFO_CRC0_Pos = 0x0
	// Bit mask of INFO_CRC0 field.
	OTP_DATA_RAW_INFO_CRC0_INFO_CRC0_Msk = 0xffff

	// INFO_CRC1: Upper 16 bits of CRC32 of OTP addresses 0x00 through 0x6b (ECC)
	// Position of INFO_CRC1 field.
	OTP_DATA_RAW_INFO_CRC1_INFO_CRC1_Pos = 0x0
	// Bit mask of INFO_CRC1 field.
	OTP_DATA_RAW_INFO_CRC1_INFO_CRC1_Msk = 0xffff

	// CRIT0: Page 0 critical boot flags (RBIT-8)
	// Position of RISCV_DISABLE field.
	OTP_DATA_RAW_CRIT0_RISCV_DISABLE_Pos = 0x1
	// Bit mask of RISCV_DISABLE field.
	OTP_DATA_RAW_CRIT0_RISCV_DISABLE_Msk = 0x2
	// Bit RISCV_DISABLE.
	OTP_DATA_RAW_CRIT0_RISCV_DISABLE = 0x2
	// Position of ARM_DISABLE field.
	OTP_DATA_RAW_CRIT0_ARM_DISABLE_Pos = 0x0
	// Bit mask of ARM_DISABLE field.
	OTP_DATA_RAW_CRIT0_ARM_DISABLE_Msk = 0x1
	// Bit ARM_DISABLE.
	OTP_DATA_RAW_CRIT0_ARM_DISABLE = 0x1

	// CRIT0_R1: Redundant copy of CRIT0
	// Position of CRIT0_R1 field.
	OTP_DATA_RAW_CRIT0_R1_CRIT0_R1_Pos = 0x0
	// Bit mask of CRIT0_R1 field.
	OTP_DATA_RAW_CRIT0_R1_CRIT0_R1_Msk = 0xffffff

	// CRIT0_R2: Redundant copy of CRIT0
	// Position of CRIT0_R2 field.
	OTP_DATA_RAW_CRIT0_R2_CRIT0_R2_Pos = 0x0
	// Bit mask of CRIT0_R2 field.
	OTP_DATA_RAW_CRIT0_R2_CRIT0_R2_Msk = 0xffffff

	// CRIT0_R3: Redundant copy of CRIT0
	// Position of CRIT0_R3 field.
	OTP_DATA_RAW_CRIT0_R3_CRIT0_R3_Pos = 0x0
	// Bit mask of CRIT0_R3 field.
	OTP_DATA_RAW_CRIT0_R3_CRIT0_R3_Msk = 0xffffff

	// CRIT0_R4: Redundant copy of CRIT0
	// Position of CRIT0_R4 field.
	OTP_DATA_RAW_CRIT0_R4_CRIT0_R4_Pos = 0x0
	// Bit mask of CRIT0_R4 field.
	OTP_DATA_RAW_CRIT0_R4_CRIT0_R4_Msk = 0xffffff

	// CRIT0_R5: Redundant copy of CRIT0
	// Position of CRIT0_R5 field.
	OTP_DATA_RAW_CRIT0_R5_CRIT0_R5_Pos = 0x0
	// Bit mask of CRIT0_R5 field.
	OTP_DATA_RAW_CRIT0_R5_CRIT0_R5_Msk = 0xffffff

	// CRIT0_R6: Redundant copy of CRIT0
	// Position of CRIT0_R6 field.
	OTP_DATA_RAW_CRIT0_R6_CRIT0_R6_Pos = 0x0
	// Bit mask of CRIT0_R6 field.
	OTP_DATA_RAW_CRIT0_R6_CRIT0_R6_Msk = 0xffffff

	// CRIT0_R7: Redundant copy of CRIT0
	// Position of CRIT0_R7 field.
	OTP_DATA_RAW_CRIT0_R7_CRIT0_R7_Pos = 0x0
	// Bit mask of CRIT0_R7 field.
	OTP_DATA_RAW_CRIT0_R7_CRIT0_R7_Msk = 0xffffff

	// CRIT1: Page 1 critical boot flags (RBIT-8)
	// Position of GLITCH_DETECTOR_SENS field.
	OTP_DATA_RAW_CRIT1_GLITCH_DETECTOR_SENS_Pos = 0x5
	// Bit mask of GLITCH_DETECTOR_SENS field.
	OTP_DATA_RAW_CRIT1_GLITCH_DETECTOR_SENS_Msk = 0x60
	// Position of GLITCH_DETECTOR_ENABLE field.
	OTP_DATA_RAW_CRIT1_GLITCH_DETECTOR_ENABLE_Pos = 0x4
	// Bit mask of GLITCH_DETECTOR_ENABLE field.
	OTP_DATA_RAW_CRIT1_GLITCH_DETECTOR_ENABLE_Msk = 0x10
	// Bit GLITCH_DETECTOR_ENABLE.
	OTP_DATA_RAW_CRIT1_GLITCH_DETECTOR_ENABLE = 0x10
	// Position of BOOT_ARCH field.
	OTP_DATA_RAW_CRIT1_BOOT_ARCH_Pos = 0x3
	// Bit mask of BOOT_ARCH field.
	OTP_DATA_RAW_CRIT1_BOOT_ARCH_Msk = 0x8
	// Bit BOOT_ARCH.
	OTP_DATA_RAW_CRIT1_BOOT_ARCH = 0x8
	// Position of DEBUG_DISABLE field.
	OTP_DATA_RAW_CRIT1_DEBUG_DISABLE_Pos = 0x2
	// Bit mask of DEBUG_DISABLE field.
	OTP_DATA_RAW_CRIT1_DEBUG_DISABLE_Msk = 0x4
	// Bit DEBUG_DISABLE.
	OTP_DATA_RAW_CRIT1_DEBUG_DISABLE = 0x4
	// Position of SECURE_DEBUG_DISABLE field.
	OTP_DATA_RAW_CRIT1_SECURE_DEBUG_DISABLE_Pos = 0x1
	// Bit mask of SECURE_DEBUG_DISABLE field.
	OTP_DATA_RAW_CRIT1_SECURE_DEBUG_DISABLE_Msk = 0x2
	// Bit SECURE_DEBUG_DISABLE.
	OTP_DATA_RAW_CRIT1_SECURE_DEBUG_DISABLE = 0x2
	// Position of SECURE_BOOT_ENABLE field.
	OTP_DATA_RAW_CRIT1_SECURE_BOOT_ENABLE_Pos = 0x0
	// Bit mask of SECURE_BOOT_ENABLE field.
	OTP_DATA_RAW_CRIT1_SECURE_BOOT_ENABLE_Msk = 0x1
	// Bit SECURE_BOOT_ENABLE.
	OTP_DATA_RAW_CRIT1_SECURE_BOOT_ENABLE = 0x1

	// CRIT1_R1: Redundant copy of CRIT1
	// Position of CRIT1_R1 field.
	OTP_DATA_RAW_CRIT1_R1_CRIT1_R1_Pos = 0x0
	// Bit mask of CRIT1_R1 field.
	OTP_DATA_RAW_CRIT1_R1_CRIT1_R1_Msk = 0xffffff

	// CRIT1_R2: Redundant copy of CRIT1
	// Position of CRIT1_R2 field.
	OTP_DATA_RAW_CRIT1_R2_CRIT1_R2_Pos = 0x0
	// Bit mask of CRIT1_R2 field.
	OTP_DATA_RAW_CRIT1_R2_CRIT1_R2_Msk = 0xffffff

	// CRIT1_R3: Redundant copy of CRIT1
	// Position of CRIT1_R3 field.
	OTP_DATA_RAW_CRIT1_R3_CRIT1_R3_Pos = 0x0
	// Bit mask of CRIT1_R3 field.
	OTP_DATA_RAW_CRIT1_R3_CRIT1_R3_Msk = 0xffffff

	// CRIT1_R4: Redundant copy of CRIT1
	// Position of CRIT1_R4 field.
	OTP_DATA_RAW_CRIT1_R4_CRIT1_R4_Pos = 0x0
	// Bit mask of CRIT1_R4 field.
	OTP_DATA_RAW_CRIT1_R4_CRIT1_R4_Msk = 0xffffff

	// CRIT1_R5: Redundant copy of CRIT1
	// Position of CRIT1_R5 field.
	OTP_DATA_RAW_CRIT1_R5_CRIT1_R5_Pos = 0x0
	// Bit mask of CRIT1_R5 field.
	OTP_DATA_RAW_CRIT1_R5_CRIT1_R5_Msk = 0xffffff

	// CRIT1_R6: Redundant copy of CRIT1
	// Position of CRIT1_R6 field.
	OTP_DATA_RAW_CRIT1_R6_CRIT1_R6_Pos = 0x0
	// Bit mask of CRIT1_R6 field.
	OTP_DATA_RAW_CRIT1_R6_CRIT1_R6_Msk = 0xffffff

	// CRIT1_R7: Redundant copy of CRIT1
	// Position of CRIT1_R7 field.
	OTP_DATA_RAW_CRIT1_R7_CRIT1_R7_Pos = 0x0
	// Bit mask of CRIT1_R7 field.
	OTP_DATA_RAW_CRIT1_R7_CRIT1_R7_Msk = 0xffffff

	// BOOT_FLAGS0: Disable/Enable boot paths/features in the RP2350 mask ROM. Disables always supersede enables. Enables are provided where there are other configurations in OTP that must be valid. (RBIT-3)
	// Position of DISABLE_SRAM_WINDOW_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_SRAM_WINDOW_BOOT_Pos = 0x15
	// Bit mask of DISABLE_SRAM_WINDOW_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_SRAM_WINDOW_BOOT_Msk = 0x200000
	// Bit DISABLE_SRAM_WINDOW_BOOT.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_SRAM_WINDOW_BOOT = 0x200000
	// Position of DISABLE_XIP_ACCESS_ON_SRAM_ENTRY field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_XIP_ACCESS_ON_SRAM_ENTRY_Pos = 0x14
	// Bit mask of DISABLE_XIP_ACCESS_ON_SRAM_ENTRY field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_XIP_ACCESS_ON_SRAM_ENTRY_Msk = 0x100000
	// Bit DISABLE_XIP_ACCESS_ON_SRAM_ENTRY.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_XIP_ACCESS_ON_SRAM_ENTRY = 0x100000
	// Position of DISABLE_BOOTSEL_UART_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_UART_BOOT_Pos = 0x13
	// Bit mask of DISABLE_BOOTSEL_UART_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_UART_BOOT_Msk = 0x80000
	// Bit DISABLE_BOOTSEL_UART_BOOT.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_UART_BOOT = 0x80000
	// Position of DISABLE_BOOTSEL_USB_PICOBOOT_IFC field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_USB_PICOBOOT_IFC_Pos = 0x12
	// Bit mask of DISABLE_BOOTSEL_USB_PICOBOOT_IFC field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_USB_PICOBOOT_IFC_Msk = 0x40000
	// Bit DISABLE_BOOTSEL_USB_PICOBOOT_IFC.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_USB_PICOBOOT_IFC = 0x40000
	// Position of DISABLE_BOOTSEL_USB_MSD_IFC field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_USB_MSD_IFC_Pos = 0x11
	// Bit mask of DISABLE_BOOTSEL_USB_MSD_IFC field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_USB_MSD_IFC_Msk = 0x20000
	// Bit DISABLE_BOOTSEL_USB_MSD_IFC.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_USB_MSD_IFC = 0x20000
	// Position of DISABLE_WATCHDOG_SCRATCH field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_WATCHDOG_SCRATCH_Pos = 0x10
	// Bit mask of DISABLE_WATCHDOG_SCRATCH field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_WATCHDOG_SCRATCH_Msk = 0x10000
	// Bit DISABLE_WATCHDOG_SCRATCH.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_WATCHDOG_SCRATCH = 0x10000
	// Position of DISABLE_POWER_SCRATCH field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_POWER_SCRATCH_Pos = 0xf
	// Bit mask of DISABLE_POWER_SCRATCH field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_POWER_SCRATCH_Msk = 0x8000
	// Bit DISABLE_POWER_SCRATCH.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_POWER_SCRATCH = 0x8000
	// Position of ENABLE_OTP_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_ENABLE_OTP_BOOT_Pos = 0xe
	// Bit mask of ENABLE_OTP_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_ENABLE_OTP_BOOT_Msk = 0x4000
	// Bit ENABLE_OTP_BOOT.
	OTP_DATA_RAW_BOOT_FLAGS0_ENABLE_OTP_BOOT = 0x4000
	// Position of DISABLE_OTP_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_OTP_BOOT_Pos = 0xd
	// Bit mask of DISABLE_OTP_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_OTP_BOOT_Msk = 0x2000
	// Bit DISABLE_OTP_BOOT.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_OTP_BOOT = 0x2000
	// Position of DISABLE_FLASH_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_FLASH_BOOT_Pos = 0xc
	// Bit mask of DISABLE_FLASH_BOOT field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_FLASH_BOOT_Msk = 0x1000
	// Bit DISABLE_FLASH_BOOT.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_FLASH_BOOT = 0x1000
	// Position of ROLLBACK_REQUIRED field.
	OTP_DATA_RAW_BOOT_FLAGS0_ROLLBACK_REQUIRED_Pos = 0xb
	// Bit mask of ROLLBACK_REQUIRED field.
	OTP_DATA_RAW_BOOT_FLAGS0_ROLLBACK_REQUIRED_Msk = 0x800
	// Bit ROLLBACK_REQUIRED.
	OTP_DATA_RAW_BOOT_FLAGS0_ROLLBACK_REQUIRED = 0x800
	// Position of HASHED_PARTITION_TABLE field.
	OTP_DATA_RAW_BOOT_FLAGS0_HASHED_PARTITION_TABLE_Pos = 0xa
	// Bit mask of HASHED_PARTITION_TABLE field.
	OTP_DATA_RAW_BOOT_FLAGS0_HASHED_PARTITION_TABLE_Msk = 0x400
	// Bit HASHED_PARTITION_TABLE.
	OTP_DATA_RAW_BOOT_FLAGS0_HASHED_PARTITION_TABLE = 0x400
	// Position of SECURE_PARTITION_TABLE field.
	OTP_DATA_RAW_BOOT_FLAGS0_SECURE_PARTITION_TABLE_Pos = 0x9
	// Bit mask of SECURE_PARTITION_TABLE field.
	OTP_DATA_RAW_BOOT_FLAGS0_SECURE_PARTITION_TABLE_Msk = 0x200
	// Bit SECURE_PARTITION_TABLE.
	OTP_DATA_RAW_BOOT_FLAGS0_SECURE_PARTITION_TABLE = 0x200
	// Position of DISABLE_AUTO_SWITCH_ARCH field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_AUTO_SWITCH_ARCH_Pos = 0x8
	// Bit mask of DISABLE_AUTO_SWITCH_ARCH field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_AUTO_SWITCH_ARCH_Msk = 0x100
	// Bit DISABLE_AUTO_SWITCH_ARCH.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_AUTO_SWITCH_ARCH = 0x100
	// Position of SINGLE_FLASH_BINARY field.
	OTP_DATA_RAW_BOOT_FLAGS0_SINGLE_FLASH_BINARY_Pos = 0x7
	// Bit mask of SINGLE_FLASH_BINARY field.
	OTP_DATA_RAW_BOOT_FLAGS0_SINGLE_FLASH_BINARY_Msk = 0x80
	// Bit SINGLE_FLASH_BINARY.
	OTP_DATA_RAW_BOOT_FLAGS0_SINGLE_FLASH_BINARY = 0x80
	// Position of OVERRIDE_FLASH_PARTITION_SLOT_SIZE field.
	OTP_DATA_RAW_BOOT_FLAGS0_OVERRIDE_FLASH_PARTITION_SLOT_SIZE_Pos = 0x6
	// Bit mask of OVERRIDE_FLASH_PARTITION_SLOT_SIZE field.
	OTP_DATA_RAW_BOOT_FLAGS0_OVERRIDE_FLASH_PARTITION_SLOT_SIZE_Msk = 0x40
	// Bit OVERRIDE_FLASH_PARTITION_SLOT_SIZE.
	OTP_DATA_RAW_BOOT_FLAGS0_OVERRIDE_FLASH_PARTITION_SLOT_SIZE = 0x40
	// Position of FLASH_DEVINFO_ENABLE field.
	OTP_DATA_RAW_BOOT_FLAGS0_FLASH_DEVINFO_ENABLE_Pos = 0x5
	// Bit mask of FLASH_DEVINFO_ENABLE field.
	OTP_DATA_RAW_BOOT_FLAGS0_FLASH_DEVINFO_ENABLE_Msk = 0x20
	// Bit FLASH_DEVINFO_ENABLE.
	OTP_DATA_RAW_BOOT_FLAGS0_FLASH_DEVINFO_ENABLE = 0x20
	// Position of FAST_SIGCHECK_ROSC_DIV field.
	OTP_DATA_RAW_BOOT_FLAGS0_FAST_SIGCHECK_ROSC_DIV_Pos = 0x4
	// Bit mask of FAST_SIGCHECK_ROSC_DIV field.
	OTP_DATA_RAW_BOOT_FLAGS0_FAST_SIGCHECK_ROSC_DIV_Msk = 0x10
	// Bit FAST_SIGCHECK_ROSC_DIV.
	OTP_DATA_RAW_BOOT_FLAGS0_FAST_SIGCHECK_ROSC_DIV = 0x10
	// Position of FLASH_IO_VOLTAGE_1V8 field.
	OTP_DATA_RAW_BOOT_FLAGS0_FLASH_IO_VOLTAGE_1V8_Pos = 0x3
	// Bit mask of FLASH_IO_VOLTAGE_1V8 field.
	OTP_DATA_RAW_BOOT_FLAGS0_FLASH_IO_VOLTAGE_1V8_Msk = 0x8
	// Bit FLASH_IO_VOLTAGE_1V8.
	OTP_DATA_RAW_BOOT_FLAGS0_FLASH_IO_VOLTAGE_1V8 = 0x8
	// Position of ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG field.
	OTP_DATA_RAW_BOOT_FLAGS0_ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG_Pos = 0x2
	// Bit mask of ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG field.
	OTP_DATA_RAW_BOOT_FLAGS0_ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG_Msk = 0x4
	// Bit ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG.
	OTP_DATA_RAW_BOOT_FLAGS0_ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG = 0x4
	// Position of ENABLE_BOOTSEL_LED field.
	OTP_DATA_RAW_BOOT_FLAGS0_ENABLE_BOOTSEL_LED_Pos = 0x1
	// Bit mask of ENABLE_BOOTSEL_LED field.
	OTP_DATA_RAW_BOOT_FLAGS0_ENABLE_BOOTSEL_LED_Msk = 0x2
	// Bit ENABLE_BOOTSEL_LED.
	OTP_DATA_RAW_BOOT_FLAGS0_ENABLE_BOOTSEL_LED = 0x2
	// Position of DISABLE_BOOTSEL_EXEC2 field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_EXEC2_Pos = 0x0
	// Bit mask of DISABLE_BOOTSEL_EXEC2 field.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_EXEC2_Msk = 0x1
	// Bit DISABLE_BOOTSEL_EXEC2.
	OTP_DATA_RAW_BOOT_FLAGS0_DISABLE_BOOTSEL_EXEC2 = 0x1

	// BOOT_FLAGS0_R1: Redundant copy of BOOT_FLAGS0
	// Position of BOOT_FLAGS0_R1 field.
	OTP_DATA_RAW_BOOT_FLAGS0_R1_BOOT_FLAGS0_R1_Pos = 0x0
	// Bit mask of BOOT_FLAGS0_R1 field.
	OTP_DATA_RAW_BOOT_FLAGS0_R1_BOOT_FLAGS0_R1_Msk = 0xffffff

	// BOOT_FLAGS0_R2: Redundant copy of BOOT_FLAGS0
	// Position of BOOT_FLAGS0_R2 field.
	OTP_DATA_RAW_BOOT_FLAGS0_R2_BOOT_FLAGS0_R2_Pos = 0x0
	// Bit mask of BOOT_FLAGS0_R2 field.
	OTP_DATA_RAW_BOOT_FLAGS0_R2_BOOT_FLAGS0_R2_Msk = 0xffffff

	// BOOT_FLAGS1: Disable/Enable boot paths/features in the RP2350 mask ROM. Disables always supersede enables. Enables are provided where there are other configurations in OTP that must be valid. (RBIT-3)
	// Position of DOUBLE_TAP field.
	OTP_DATA_RAW_BOOT_FLAGS1_DOUBLE_TAP_Pos = 0x13
	// Bit mask of DOUBLE_TAP field.
	OTP_DATA_RAW_BOOT_FLAGS1_DOUBLE_TAP_Msk = 0x80000
	// Bit DOUBLE_TAP.
	OTP_DATA_RAW_BOOT_FLAGS1_DOUBLE_TAP = 0x80000
	// Position of DOUBLE_TAP_DELAY field.
	OTP_DATA_RAW_BOOT_FLAGS1_DOUBLE_TAP_DELAY_Pos = 0x10
	// Bit mask of DOUBLE_TAP_DELAY field.
	OTP_DATA_RAW_BOOT_FLAGS1_DOUBLE_TAP_DELAY_Msk = 0x70000
	// Position of KEY_INVALID field.
	OTP_DATA_RAW_BOOT_FLAGS1_KEY_INVALID_Pos = 0x8
	// Bit mask of KEY_INVALID field.
	OTP_DATA_RAW_BOOT_FLAGS1_KEY_INVALID_Msk = 0xf00
	// Position of KEY_VALID field.
	OTP_DATA_RAW_BOOT_FLAGS1_KEY_VALID_Pos = 0x0
	// Bit mask of KEY_VALID field.
	OTP_DATA_RAW_BOOT_FLAGS1_KEY_VALID_Msk = 0xf

	// BOOT_FLAGS1_R1: Redundant copy of BOOT_FLAGS1
	// Position of BOOT_FLAGS1_R1 field.
	OTP_DATA_RAW_BOOT_FLAGS1_R1_BOOT_FLAGS1_R1_Pos = 0x0
	// Bit mask of BOOT_FLAGS1_R1 field.
	OTP_DATA_RAW_BOOT_FLAGS1_R1_BOOT_FLAGS1_R1_Msk = 0xffffff

	// BOOT_FLAGS1_R2: Redundant copy of BOOT_FLAGS1
	// Position of BOOT_FLAGS1_R2 field.
	OTP_DATA_RAW_BOOT_FLAGS1_R2_BOOT_FLAGS1_R2_Pos = 0x0
	// Bit mask of BOOT_FLAGS1_R2 field.
	OTP_DATA_RAW_BOOT_FLAGS1_R2_BOOT_FLAGS1_R2_Msk = 0xffffff

	// DEFAULT_BOOT_VERSION0: Default boot version thermometer counter, bits 23:0 (RBIT-3)
	// Position of DEFAULT_BOOT_VERSION0 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION0_DEFAULT_BOOT_VERSION0_Pos = 0x0
	// Bit mask of DEFAULT_BOOT_VERSION0 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION0_DEFAULT_BOOT_VERSION0_Msk = 0xffffff

	// DEFAULT_BOOT_VERSION0_R1: Redundant copy of DEFAULT_BOOT_VERSION0
	// Position of DEFAULT_BOOT_VERSION0_R1 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION0_R1_DEFAULT_BOOT_VERSION0_R1_Pos = 0x0
	// Bit mask of DEFAULT_BOOT_VERSION0_R1 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION0_R1_DEFAULT_BOOT_VERSION0_R1_Msk = 0xffffff

	// DEFAULT_BOOT_VERSION0_R2: Redundant copy of DEFAULT_BOOT_VERSION0
	// Position of DEFAULT_BOOT_VERSION0_R2 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION0_R2_DEFAULT_BOOT_VERSION0_R2_Pos = 0x0
	// Bit mask of DEFAULT_BOOT_VERSION0_R2 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION0_R2_DEFAULT_BOOT_VERSION0_R2_Msk = 0xffffff

	// DEFAULT_BOOT_VERSION1: Default boot version thermometer counter, bits 47:24 (RBIT-3)
	// Position of DEFAULT_BOOT_VERSION1 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION1_DEFAULT_BOOT_VERSION1_Pos = 0x0
	// Bit mask of DEFAULT_BOOT_VERSION1 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION1_DEFAULT_BOOT_VERSION1_Msk = 0xffffff

	// DEFAULT_BOOT_VERSION1_R1: Redundant copy of DEFAULT_BOOT_VERSION1
	// Position of DEFAULT_BOOT_VERSION1_R1 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION1_R1_DEFAULT_BOOT_VERSION1_R1_Pos = 0x0
	// Bit mask of DEFAULT_BOOT_VERSION1_R1 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION1_R1_DEFAULT_BOOT_VERSION1_R1_Msk = 0xffffff

	// DEFAULT_BOOT_VERSION1_R2: Redundant copy of DEFAULT_BOOT_VERSION1
	// Position of DEFAULT_BOOT_VERSION1_R2 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION1_R2_DEFAULT_BOOT_VERSION1_R2_Pos = 0x0
	// Bit mask of DEFAULT_BOOT_VERSION1_R2 field.
	OTP_DATA_RAW_DEFAULT_BOOT_VERSION1_R2_DEFAULT_BOOT_VERSION1_R2_Msk = 0xffffff

	// FLASH_DEVINFO: Stores information about external flash device(s). (ECC) Assumed to be valid if BOOT_FLAGS0_FLASH_DEVINFO_ENABLE is set.
	// Position of CS1_SIZE field.
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_Pos = 0xc
	// Bit mask of CS1_SIZE field.
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_Msk  = 0xf000
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_NONE = 0x0
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_8K   = 0x1
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_16K  = 0x2
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_32K  = 0x3
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_64k  = 0x4
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_128K = 0x5
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_256K = 0x6
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_512K = 0x7
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_1M   = 0x8
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_2M   = 0x9
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_4M   = 0xa
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_8M   = 0xb
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_SIZE_16M  = 0xc
	// Position of CS0_SIZE field.
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_Pos = 0x8
	// Bit mask of CS0_SIZE field.
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_Msk  = 0xf00
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_NONE = 0x0
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_8K   = 0x1
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_16K  = 0x2
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_32K  = 0x3
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_64k  = 0x4
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_128K = 0x5
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_256K = 0x6
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_512K = 0x7
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_1M   = 0x8
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_2M   = 0x9
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_4M   = 0xa
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_8M   = 0xb
	OTP_DATA_RAW_FLASH_DEVINFO_CS0_SIZE_16M  = 0xc
	// Position of D8H_ERASE_SUPPORTED field.
	OTP_DATA_RAW_FLASH_DEVINFO_D8H_ERASE_SUPPORTED_Pos = 0x7
	// Bit mask of D8H_ERASE_SUPPORTED field.
	OTP_DATA_RAW_FLASH_DEVINFO_D8H_ERASE_SUPPORTED_Msk = 0x80
	// Bit D8H_ERASE_SUPPORTED.
	OTP_DATA_RAW_FLASH_DEVINFO_D8H_ERASE_SUPPORTED = 0x80
	// Position of CS1_GPIO field.
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_GPIO_Pos = 0x0
	// Bit mask of CS1_GPIO field.
	OTP_DATA_RAW_FLASH_DEVINFO_CS1_GPIO_Msk = 0x3f

	// FLASH_PARTITION_SLOT_SIZE: Gap between partition table slot 0 and slot 1 at the start of flash (the default size is 4096 bytes) (ECC) Enabled by the OVERRIDE_FLASH_PARTITION_SLOT_SIZE bit in BOOT_FLAGS, the size is 4096 * (value + 1)
	// Position of FLASH_PARTITION_SLOT_SIZE field.
	OTP_DATA_RAW_FLASH_PARTITION_SLOT_SIZE_FLASH_PARTITION_SLOT_SIZE_Pos = 0x0
	// Bit mask of FLASH_PARTITION_SLOT_SIZE field.
	OTP_DATA_RAW_FLASH_PARTITION_SLOT_SIZE_FLASH_PARTITION_SLOT_SIZE_Msk = 0xffff

	// BOOTSEL_LED_CFG: Pin configuration for LED status, used by USB bootloader. (ECC) Must be valid if BOOT_FLAGS0_ENABLE_BOOTSEL_LED is set.
	// Position of ACTIVELOW field.
	OTP_DATA_RAW_BOOTSEL_LED_CFG_ACTIVELOW_Pos = 0x8
	// Bit mask of ACTIVELOW field.
	OTP_DATA_RAW_BOOTSEL_LED_CFG_ACTIVELOW_Msk = 0x100
	// Bit ACTIVELOW.
	OTP_DATA_RAW_BOOTSEL_LED_CFG_ACTIVELOW = 0x100
	// Position of PIN field.
	OTP_DATA_RAW_BOOTSEL_LED_CFG_PIN_Pos = 0x0
	// Bit mask of PIN field.
	OTP_DATA_RAW_BOOTSEL_LED_CFG_PIN_Msk = 0x3f

	// BOOTSEL_PLL_CFG: Optional PLL configuration for BOOTSEL mode. (ECC) This should be configured to produce an exact 48 MHz based on the crystal oscillator frequency. User mode software may also use this value to calculate the expected crystal frequency based on an assumed 48 MHz PLL output. If no configuration is given, the crystal is assumed to be 12 MHz. The PLL frequency can be calculated as: PLL out = (XOSC frequency / (REFDIV+1)) x FBDIV / (POSTDIV1 x POSTDIV2) Conversely the crystal frequency can be calculated as: XOSC frequency = 48 MHz x (REFDIV+1) x (POSTDIV1 x POSTDIV2) / FBDIV (Note the +1 on REFDIV is because the value stored in this OTP location is the actual divisor value minus one.) Used if and only if ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG is set in BOOT_FLAGS0. That bit should be set only after this row and BOOTSEL_XOSC_CFG are both correctly programmed.
	// Position of REFDIV field.
	OTP_DATA_RAW_BOOTSEL_PLL_CFG_REFDIV_Pos = 0xf
	// Bit mask of REFDIV field.
	OTP_DATA_RAW_BOOTSEL_PLL_CFG_REFDIV_Msk = 0x8000
	// Bit REFDIV.
	OTP_DATA_RAW_BOOTSEL_PLL_CFG_REFDIV = 0x8000
	// Position of POSTDIV2 field.
	OTP_DATA_RAW_BOOTSEL_PLL_CFG_POSTDIV2_Pos = 0xc
	// Bit mask of POSTDIV2 field.
	OTP_DATA_RAW_BOOTSEL_PLL_CFG_POSTDIV2_Msk = 0x7000
	// Position of POSTDIV1 field.
	OTP_DATA_RAW_BOOTSEL_PLL_CFG_POSTDIV1_Pos = 0x9
	// Bit mask of POSTDIV1 field.
	OTP_DATA_RAW_BOOTSEL_PLL_CFG_POSTDIV1_Msk = 0xe00
	// Position of FBDIV field.
	OTP_DATA_RAW_BOOTSEL_PLL_CFG_FBDIV_Pos = 0x0
	// Bit mask of FBDIV field.
	OTP_DATA_RAW_BOOTSEL_PLL_CFG_FBDIV_Msk = 0x1ff

	// BOOTSEL_XOSC_CFG: Non-default crystal oscillator configuration for the USB bootloader. (ECC) These values may also be used by user code configuring the crystal oscillator. Used if and only if ENABLE_BOOTSEL_NON_DEFAULT_PLL_XOSC_CFG is set in BOOT_FLAGS0. That bit should be set only after this row and BOOTSEL_PLL_CFG are both correctly programmed.
	// Position of RANGE field.
	OTP_DATA_RAW_BOOTSEL_XOSC_CFG_RANGE_Pos = 0xe
	// Bit mask of RANGE field.
	OTP_DATA_RAW_BOOTSEL_XOSC_CFG_RANGE_Msk       = 0xc000
	OTP_DATA_RAW_BOOTSEL_XOSC_CFG_RANGE_1_15MHZ   = 0x0
	OTP_DATA_RAW_BOOTSEL_XOSC_CFG_RANGE_10_30MHZ  = 0x1
	OTP_DATA_RAW_BOOTSEL_XOSC_CFG_RANGE_25_60MHZ  = 0x2
	OTP_DATA_RAW_BOOTSEL_XOSC_CFG_RANGE_40_100MHZ = 0x3
	// Position of STARTUP field.
	OTP_DATA_RAW_BOOTSEL_XOSC_CFG_STARTUP_Pos = 0x0
	// Bit mask of STARTUP field.
	OTP_DATA_RAW_BOOTSEL_XOSC_CFG_STARTUP_Msk = 0x3fff

	// USB_BOOT_FLAGS: USB boot specific feature flags (RBIT-3)
	// Position of DP_DM_SWAP field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_DP_DM_SWAP_Pos = 0x17
	// Bit mask of DP_DM_SWAP field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_DP_DM_SWAP_Msk = 0x800000
	// Bit DP_DM_SWAP.
	OTP_DATA_RAW_USB_BOOT_FLAGS_DP_DM_SWAP = 0x800000
	// Position of WHITE_LABEL_ADDR_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WHITE_LABEL_ADDR_VALID_Pos = 0x16
	// Bit mask of WHITE_LABEL_ADDR_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WHITE_LABEL_ADDR_VALID_Msk = 0x400000
	// Bit WHITE_LABEL_ADDR_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WHITE_LABEL_ADDR_VALID = 0x400000
	// Position of WL_INFO_UF2_TXT_BOARD_ID_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INFO_UF2_TXT_BOARD_ID_STRDEF_VALID_Pos = 0xf
	// Bit mask of WL_INFO_UF2_TXT_BOARD_ID_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INFO_UF2_TXT_BOARD_ID_STRDEF_VALID_Msk = 0x8000
	// Bit WL_INFO_UF2_TXT_BOARD_ID_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INFO_UF2_TXT_BOARD_ID_STRDEF_VALID = 0x8000
	// Position of WL_INFO_UF2_TXT_MODEL_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INFO_UF2_TXT_MODEL_STRDEF_VALID_Pos = 0xe
	// Bit mask of WL_INFO_UF2_TXT_MODEL_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INFO_UF2_TXT_MODEL_STRDEF_VALID_Msk = 0x4000
	// Bit WL_INFO_UF2_TXT_MODEL_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INFO_UF2_TXT_MODEL_STRDEF_VALID = 0x4000
	// Position of WL_INDEX_HTM_REDIRECT_NAME_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_NAME_STRDEF_VALID_Pos = 0xd
	// Bit mask of WL_INDEX_HTM_REDIRECT_NAME_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_NAME_STRDEF_VALID_Msk = 0x2000
	// Bit WL_INDEX_HTM_REDIRECT_NAME_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_NAME_STRDEF_VALID = 0x2000
	// Position of WL_INDEX_HTM_REDIRECT_URL_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_URL_STRDEF_VALID_Pos = 0xc
	// Bit mask of WL_INDEX_HTM_REDIRECT_URL_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_URL_STRDEF_VALID_Msk = 0x1000
	// Bit WL_INDEX_HTM_REDIRECT_URL_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_INDEX_HTM_REDIRECT_URL_STRDEF_VALID = 0x1000
	// Position of WL_SCSI_INQUIRY_VERSION_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_SCSI_INQUIRY_VERSION_STRDEF_VALID_Pos = 0xb
	// Bit mask of WL_SCSI_INQUIRY_VERSION_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_SCSI_INQUIRY_VERSION_STRDEF_VALID_Msk = 0x800
	// Bit WL_SCSI_INQUIRY_VERSION_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_SCSI_INQUIRY_VERSION_STRDEF_VALID = 0x800
	// Position of WL_SCSI_INQUIRY_PRODUCT_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_SCSI_INQUIRY_PRODUCT_STRDEF_VALID_Pos = 0xa
	// Bit mask of WL_SCSI_INQUIRY_PRODUCT_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_SCSI_INQUIRY_PRODUCT_STRDEF_VALID_Msk = 0x400
	// Bit WL_SCSI_INQUIRY_PRODUCT_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_SCSI_INQUIRY_PRODUCT_STRDEF_VALID = 0x400
	// Position of WL_SCSI_INQUIRY_VENDOR_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_SCSI_INQUIRY_VENDOR_STRDEF_VALID_Pos = 0x9
	// Bit mask of WL_SCSI_INQUIRY_VENDOR_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_SCSI_INQUIRY_VENDOR_STRDEF_VALID_Msk = 0x200
	// Bit WL_SCSI_INQUIRY_VENDOR_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_SCSI_INQUIRY_VENDOR_STRDEF_VALID = 0x200
	// Position of WL_VOLUME_LABEL_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_VOLUME_LABEL_STRDEF_VALID_Pos = 0x8
	// Bit mask of WL_VOLUME_LABEL_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_VOLUME_LABEL_STRDEF_VALID_Msk = 0x100
	// Bit WL_VOLUME_LABEL_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_VOLUME_LABEL_STRDEF_VALID = 0x100
	// Position of WL_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES_VALID_Pos = 0x7
	// Bit mask of WL_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES_VALID_Msk = 0x80
	// Bit WL_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES_VALID = 0x80
	// Position of WL_USB_DEVICE_SERIAL_NUMBER_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_STRDEF_VALID_Pos = 0x6
	// Bit mask of WL_USB_DEVICE_SERIAL_NUMBER_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_STRDEF_VALID_Msk = 0x40
	// Bit WL_USB_DEVICE_SERIAL_NUMBER_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_STRDEF_VALID = 0x40
	// Position of WL_USB_DEVICE_PRODUCT_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_PRODUCT_STRDEF_VALID_Pos = 0x5
	// Bit mask of WL_USB_DEVICE_PRODUCT_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_PRODUCT_STRDEF_VALID_Msk = 0x20
	// Bit WL_USB_DEVICE_PRODUCT_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_PRODUCT_STRDEF_VALID = 0x20
	// Position of WL_USB_DEVICE_MANUFACTURER_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_MANUFACTURER_STRDEF_VALID_Pos = 0x4
	// Bit mask of WL_USB_DEVICE_MANUFACTURER_STRDEF_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_MANUFACTURER_STRDEF_VALID_Msk = 0x10
	// Bit WL_USB_DEVICE_MANUFACTURER_STRDEF_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_MANUFACTURER_STRDEF_VALID = 0x10
	// Position of WL_USB_DEVICE_LANG_ID_VALUE_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_LANG_ID_VALUE_VALID_Pos = 0x3
	// Bit mask of WL_USB_DEVICE_LANG_ID_VALUE_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_LANG_ID_VALUE_VALID_Msk = 0x8
	// Bit WL_USB_DEVICE_LANG_ID_VALUE_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_LANG_ID_VALUE_VALID = 0x8
	// Position of WL_USB_DEVICE_SERIAL_NUMBER_VALUE_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_VALUE_VALID_Pos = 0x2
	// Bit mask of WL_USB_DEVICE_SERIAL_NUMBER_VALUE_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_VALUE_VALID_Msk = 0x4
	// Bit WL_USB_DEVICE_SERIAL_NUMBER_VALUE_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_SERIAL_NUMBER_VALUE_VALID = 0x4
	// Position of WL_USB_DEVICE_PID_VALUE_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_PID_VALUE_VALID_Pos = 0x1
	// Bit mask of WL_USB_DEVICE_PID_VALUE_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_PID_VALUE_VALID_Msk = 0x2
	// Bit WL_USB_DEVICE_PID_VALUE_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_PID_VALUE_VALID = 0x2
	// Position of WL_USB_DEVICE_VID_VALUE_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_VID_VALUE_VALID_Pos = 0x0
	// Bit mask of WL_USB_DEVICE_VID_VALUE_VALID field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_VID_VALUE_VALID_Msk = 0x1
	// Bit WL_USB_DEVICE_VID_VALUE_VALID.
	OTP_DATA_RAW_USB_BOOT_FLAGS_WL_USB_DEVICE_VID_VALUE_VALID = 0x1

	// USB_BOOT_FLAGS_R1: Redundant copy of USB_BOOT_FLAGS
	// Position of USB_BOOT_FLAGS_R1 field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_R1_USB_BOOT_FLAGS_R1_Pos = 0x0
	// Bit mask of USB_BOOT_FLAGS_R1 field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_R1_USB_BOOT_FLAGS_R1_Msk = 0xffffff

	// USB_BOOT_FLAGS_R2: Redundant copy of USB_BOOT_FLAGS
	// Position of USB_BOOT_FLAGS_R2 field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_R2_USB_BOOT_FLAGS_R2_Pos = 0x0
	// Bit mask of USB_BOOT_FLAGS_R2 field.
	OTP_DATA_RAW_USB_BOOT_FLAGS_R2_USB_BOOT_FLAGS_R2_Msk = 0xffffff

	// USB_WHITE_LABEL_ADDR: Row index of the USB_WHITE_LABEL structure within OTP (ECC) The table has 16 rows, each of which are also ECC and marked valid by the corresponding valid bit in USB_BOOT_FLAGS (ECC). The entries are either _VALUEs where the 16 bit value is used as is, or _STRDEFs which acts as a pointers to a string value. The value stored in a _STRDEF is two separate bytes: The low seven bits of the first (LSB) byte indicates the number of characters in the string, and the top bit of the first (LSB) byte if set to indicate that each character in the string is two bytes (Unicode) versus one byte if unset. The second (MSB) byte represents the location of the string data, and is encoded as the number of rows from this USB_WHITE_LABEL_ADDR; i.e. the row of the start of the string is USB_WHITE_LABEL_ADDR value + msb_byte. In each case, the corresponding valid bit enables replacing the default value for the corresponding item provided by the boot rom. Note that Unicode _STRDEFs are only supported for USB_DEVICE_PRODUCT_STRDEF, USB_DEVICE_SERIAL_NUMBER_STRDEF and USB_DEVICE_MANUFACTURER_STRDEF. Unicode values will be ignored if specified for other fields, and non-unicode values for these three items will be converted to Unicode characters by setting the upper 8 bits to zero. Note that if the USB_WHITE_LABEL structure or the corresponding strings are not readable by BOOTSEL mode based on OTP permissions, or if alignment requirements are not met, then the corresponding default values are used. The index values indicate where each field is located (row USB_WHITE_LABEL_ADDR value + index):
	// Position of USB_WHITE_LABEL_ADDR field.
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_Pos = 0x0
	// Bit mask of USB_WHITE_LABEL_ADDR field.
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_Msk                                          = 0xffff
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_VID_VALUE                   = 0x0
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_PID_VALUE                   = 0x1
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_BCD_DEVICE_VALUE            = 0x2
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_LANG_ID_VALUE               = 0x3
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_MANUFACTURER_STRDEF         = 0x4
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_PRODUCT_STRDEF              = 0x5
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_DEVICE_SERIAL_NUMBER_STRDEF        = 0x6
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_USB_CONFIG_ATTRIBUTES_MAX_POWER_VALUES = 0x7
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_VOLUME_LABEL_STRDEF                    = 0x8
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_SCSI_INQUIRY_VENDOR_STRDEF             = 0x9
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_SCSI_INQUIRY_PRODUCT_STRDEF            = 0xa
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_SCSI_INQUIRY_VERSION_STRDEF            = 0xb
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_INDEX_HTM_REDIRECT_URL_STRDEF          = 0xc
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_INDEX_HTM_REDIRECT_NAME_STRDEF         = 0xd
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_INFO_UF2_TXT_MODEL_STRDEF              = 0xe
	OTP_DATA_RAW_USB_WHITE_LABEL_ADDR_USB_WHITE_LABEL_ADDR_INDEX_INFO_UF2_TXT_BOARD_ID_STRDEF           = 0xf

	// OTPBOOT_SRC: OTP start row for the OTP boot image. (ECC) If OTP boot is enabled, the bootrom will load from this location into SRAM and then directly enter the loaded image. Note that the image must be signed if SECURE_BOOT_ENABLE is set. The image itself is assumed to be ECC-protected. This must be an even number. Equivalently, the OTP boot image must start at a word-aligned location in the ECC read data address window.
	// Position of OTPBOOT_SRC field.
	OTP_DATA_RAW_OTPBOOT_SRC_OTPBOOT_SRC_Pos = 0x0
	// Bit mask of OTPBOOT_SRC field.
	OTP_DATA_RAW_OTPBOOT_SRC_OTPBOOT_SRC_Msk = 0xffff

	// OTPBOOT_LEN: Length in rows of the OTP boot image. (ECC) OTPBOOT_LEN must be even. The total image size must be a multiple of 4 bytes (32 bits).
	// Position of OTPBOOT_LEN field.
	OTP_DATA_RAW_OTPBOOT_LEN_OTPBOOT_LEN_Pos = 0x0
	// Bit mask of OTPBOOT_LEN field.
	OTP_DATA_RAW_OTPBOOT_LEN_OTPBOOT_LEN_Msk = 0xffff

	// OTPBOOT_DST0: Bits 15:0 of the OTP boot image load destination (and entry point). (ECC) This must be a location in main SRAM (main SRAM is addresses 0x20000000 through 0x20082000) and must be word-aligned.
	// Position of OTPBOOT_DST0 field.
	OTP_DATA_RAW_OTPBOOT_DST0_OTPBOOT_DST0_Pos = 0x0
	// Bit mask of OTPBOOT_DST0 field.
	OTP_DATA_RAW_OTPBOOT_DST0_OTPBOOT_DST0_Msk = 0xffff

	// OTPBOOT_DST1: Bits 31:16 of the OTP boot image load destination (and entry point). (ECC) This must be a location in main SRAM (main SRAM is addresses 0x20000000 through 0x20082000) and must be word-aligned.
	// Position of OTPBOOT_DST1 field.
	OTP_DATA_RAW_OTPBOOT_DST1_OTPBOOT_DST1_Pos = 0x0
	// Bit mask of OTPBOOT_DST1 field.
	OTP_DATA_RAW_OTPBOOT_DST1_OTPBOOT_DST1_Msk = 0xffff

	// BOOTKEY0_0: Bits 15:0 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_0 field.
	OTP_DATA_RAW_BOOTKEY0_0_BOOTKEY0_0_Pos = 0x0
	// Bit mask of BOOTKEY0_0 field.
	OTP_DATA_RAW_BOOTKEY0_0_BOOTKEY0_0_Msk = 0xffff

	// BOOTKEY0_1: Bits 31:16 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_1 field.
	OTP_DATA_RAW_BOOTKEY0_1_BOOTKEY0_1_Pos = 0x0
	// Bit mask of BOOTKEY0_1 field.
	OTP_DATA_RAW_BOOTKEY0_1_BOOTKEY0_1_Msk = 0xffff

	// BOOTKEY0_2: Bits 47:32 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_2 field.
	OTP_DATA_RAW_BOOTKEY0_2_BOOTKEY0_2_Pos = 0x0
	// Bit mask of BOOTKEY0_2 field.
	OTP_DATA_RAW_BOOTKEY0_2_BOOTKEY0_2_Msk = 0xffff

	// BOOTKEY0_3: Bits 63:48 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_3 field.
	OTP_DATA_RAW_BOOTKEY0_3_BOOTKEY0_3_Pos = 0x0
	// Bit mask of BOOTKEY0_3 field.
	OTP_DATA_RAW_BOOTKEY0_3_BOOTKEY0_3_Msk = 0xffff

	// BOOTKEY0_4: Bits 79:64 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_4 field.
	OTP_DATA_RAW_BOOTKEY0_4_BOOTKEY0_4_Pos = 0x0
	// Bit mask of BOOTKEY0_4 field.
	OTP_DATA_RAW_BOOTKEY0_4_BOOTKEY0_4_Msk = 0xffff

	// BOOTKEY0_5: Bits 95:80 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_5 field.
	OTP_DATA_RAW_BOOTKEY0_5_BOOTKEY0_5_Pos = 0x0
	// Bit mask of BOOTKEY0_5 field.
	OTP_DATA_RAW_BOOTKEY0_5_BOOTKEY0_5_Msk = 0xffff

	// BOOTKEY0_6: Bits 111:96 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_6 field.
	OTP_DATA_RAW_BOOTKEY0_6_BOOTKEY0_6_Pos = 0x0
	// Bit mask of BOOTKEY0_6 field.
	OTP_DATA_RAW_BOOTKEY0_6_BOOTKEY0_6_Msk = 0xffff

	// BOOTKEY0_7: Bits 127:112 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_7 field.
	OTP_DATA_RAW_BOOTKEY0_7_BOOTKEY0_7_Pos = 0x0
	// Bit mask of BOOTKEY0_7 field.
	OTP_DATA_RAW_BOOTKEY0_7_BOOTKEY0_7_Msk = 0xffff

	// BOOTKEY0_8: Bits 143:128 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_8 field.
	OTP_DATA_RAW_BOOTKEY0_8_BOOTKEY0_8_Pos = 0x0
	// Bit mask of BOOTKEY0_8 field.
	OTP_DATA_RAW_BOOTKEY0_8_BOOTKEY0_8_Msk = 0xffff

	// BOOTKEY0_9: Bits 159:144 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_9 field.
	OTP_DATA_RAW_BOOTKEY0_9_BOOTKEY0_9_Pos = 0x0
	// Bit mask of BOOTKEY0_9 field.
	OTP_DATA_RAW_BOOTKEY0_9_BOOTKEY0_9_Msk = 0xffff

	// BOOTKEY0_10: Bits 175:160 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_10 field.
	OTP_DATA_RAW_BOOTKEY0_10_BOOTKEY0_10_Pos = 0x0
	// Bit mask of BOOTKEY0_10 field.
	OTP_DATA_RAW_BOOTKEY0_10_BOOTKEY0_10_Msk = 0xffff

	// BOOTKEY0_11: Bits 191:176 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_11 field.
	OTP_DATA_RAW_BOOTKEY0_11_BOOTKEY0_11_Pos = 0x0
	// Bit mask of BOOTKEY0_11 field.
	OTP_DATA_RAW_BOOTKEY0_11_BOOTKEY0_11_Msk = 0xffff

	// BOOTKEY0_12: Bits 207:192 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_12 field.
	OTP_DATA_RAW_BOOTKEY0_12_BOOTKEY0_12_Pos = 0x0
	// Bit mask of BOOTKEY0_12 field.
	OTP_DATA_RAW_BOOTKEY0_12_BOOTKEY0_12_Msk = 0xffff

	// BOOTKEY0_13: Bits 223:208 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_13 field.
	OTP_DATA_RAW_BOOTKEY0_13_BOOTKEY0_13_Pos = 0x0
	// Bit mask of BOOTKEY0_13 field.
	OTP_DATA_RAW_BOOTKEY0_13_BOOTKEY0_13_Msk = 0xffff

	// BOOTKEY0_14: Bits 239:224 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_14 field.
	OTP_DATA_RAW_BOOTKEY0_14_BOOTKEY0_14_Pos = 0x0
	// Bit mask of BOOTKEY0_14 field.
	OTP_DATA_RAW_BOOTKEY0_14_BOOTKEY0_14_Msk = 0xffff

	// BOOTKEY0_15: Bits 255:240 of SHA-256 hash of boot key 0 (ECC)
	// Position of BOOTKEY0_15 field.
	OTP_DATA_RAW_BOOTKEY0_15_BOOTKEY0_15_Pos = 0x0
	// Bit mask of BOOTKEY0_15 field.
	OTP_DATA_RAW_BOOTKEY0_15_BOOTKEY0_15_Msk = 0xffff

	// BOOTKEY1_0: Bits 15:0 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_0 field.
	OTP_DATA_RAW_BOOTKEY1_0_BOOTKEY1_0_Pos = 0x0
	// Bit mask of BOOTKEY1_0 field.
	OTP_DATA_RAW_BOOTKEY1_0_BOOTKEY1_0_Msk = 0xffff

	// BOOTKEY1_1: Bits 31:16 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_1 field.
	OTP_DATA_RAW_BOOTKEY1_1_BOOTKEY1_1_Pos = 0x0
	// Bit mask of BOOTKEY1_1 field.
	OTP_DATA_RAW_BOOTKEY1_1_BOOTKEY1_1_Msk = 0xffff

	// BOOTKEY1_2: Bits 47:32 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_2 field.
	OTP_DATA_RAW_BOOTKEY1_2_BOOTKEY1_2_Pos = 0x0
	// Bit mask of BOOTKEY1_2 field.
	OTP_DATA_RAW_BOOTKEY1_2_BOOTKEY1_2_Msk = 0xffff

	// BOOTKEY1_3: Bits 63:48 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_3 field.
	OTP_DATA_RAW_BOOTKEY1_3_BOOTKEY1_3_Pos = 0x0
	// Bit mask of BOOTKEY1_3 field.
	OTP_DATA_RAW_BOOTKEY1_3_BOOTKEY1_3_Msk = 0xffff

	// BOOTKEY1_4: Bits 79:64 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_4 field.
	OTP_DATA_RAW_BOOTKEY1_4_BOOTKEY1_4_Pos = 0x0
	// Bit mask of BOOTKEY1_4 field.
	OTP_DATA_RAW_BOOTKEY1_4_BOOTKEY1_4_Msk = 0xffff

	// BOOTKEY1_5: Bits 95:80 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_5 field.
	OTP_DATA_RAW_BOOTKEY1_5_BOOTKEY1_5_Pos = 0x0
	// Bit mask of BOOTKEY1_5 field.
	OTP_DATA_RAW_BOOTKEY1_5_BOOTKEY1_5_Msk = 0xffff

	// BOOTKEY1_6: Bits 111:96 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_6 field.
	OTP_DATA_RAW_BOOTKEY1_6_BOOTKEY1_6_Pos = 0x0
	// Bit mask of BOOTKEY1_6 field.
	OTP_DATA_RAW_BOOTKEY1_6_BOOTKEY1_6_Msk = 0xffff

	// BOOTKEY1_7: Bits 127:112 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_7 field.
	OTP_DATA_RAW_BOOTKEY1_7_BOOTKEY1_7_Pos = 0x0
	// Bit mask of BOOTKEY1_7 field.
	OTP_DATA_RAW_BOOTKEY1_7_BOOTKEY1_7_Msk = 0xffff

	// BOOTKEY1_8: Bits 143:128 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_8 field.
	OTP_DATA_RAW_BOOTKEY1_8_BOOTKEY1_8_Pos = 0x0
	// Bit mask of BOOTKEY1_8 field.
	OTP_DATA_RAW_BOOTKEY1_8_BOOTKEY1_8_Msk = 0xffff

	// BOOTKEY1_9: Bits 159:144 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_9 field.
	OTP_DATA_RAW_BOOTKEY1_9_BOOTKEY1_9_Pos = 0x0
	// Bit mask of BOOTKEY1_9 field.
	OTP_DATA_RAW_BOOTKEY1_9_BOOTKEY1_9_Msk = 0xffff

	// BOOTKEY1_10: Bits 175:160 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_10 field.
	OTP_DATA_RAW_BOOTKEY1_10_BOOTKEY1_10_Pos = 0x0
	// Bit mask of BOOTKEY1_10 field.
	OTP_DATA_RAW_BOOTKEY1_10_BOOTKEY1_10_Msk = 0xffff

	// BOOTKEY1_11: Bits 191:176 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_11 field.
	OTP_DATA_RAW_BOOTKEY1_11_BOOTKEY1_11_Pos = 0x0
	// Bit mask of BOOTKEY1_11 field.
	OTP_DATA_RAW_BOOTKEY1_11_BOOTKEY1_11_Msk = 0xffff

	// BOOTKEY1_12: Bits 207:192 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_12 field.
	OTP_DATA_RAW_BOOTKEY1_12_BOOTKEY1_12_Pos = 0x0
	// Bit mask of BOOTKEY1_12 field.
	OTP_DATA_RAW_BOOTKEY1_12_BOOTKEY1_12_Msk = 0xffff

	// BOOTKEY1_13: Bits 223:208 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_13 field.
	OTP_DATA_RAW_BOOTKEY1_13_BOOTKEY1_13_Pos = 0x0
	// Bit mask of BOOTKEY1_13 field.
	OTP_DATA_RAW_BOOTKEY1_13_BOOTKEY1_13_Msk = 0xffff

	// BOOTKEY1_14: Bits 239:224 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_14 field.
	OTP_DATA_RAW_BOOTKEY1_14_BOOTKEY1_14_Pos = 0x0
	// Bit mask of BOOTKEY1_14 field.
	OTP_DATA_RAW_BOOTKEY1_14_BOOTKEY1_14_Msk = 0xffff

	// BOOTKEY1_15: Bits 255:240 of SHA-256 hash of boot key 1 (ECC)
	// Position of BOOTKEY1_15 field.
	OTP_DATA_RAW_BOOTKEY1_15_BOOTKEY1_15_Pos = 0x0
	// Bit mask of BOOTKEY1_15 field.
	OTP_DATA_RAW_BOOTKEY1_15_BOOTKEY1_15_Msk = 0xffff

	// BOOTKEY2_0: Bits 15:0 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_0 field.
	OTP_DATA_RAW_BOOTKEY2_0_BOOTKEY2_0_Pos = 0x0
	// Bit mask of BOOTKEY2_0 field.
	OTP_DATA_RAW_BOOTKEY2_0_BOOTKEY2_0_Msk = 0xffff

	// BOOTKEY2_1: Bits 31:16 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_1 field.
	OTP_DATA_RAW_BOOTKEY2_1_BOOTKEY2_1_Pos = 0x0
	// Bit mask of BOOTKEY2_1 field.
	OTP_DATA_RAW_BOOTKEY2_1_BOOTKEY2_1_Msk = 0xffff

	// BOOTKEY2_2: Bits 47:32 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_2 field.
	OTP_DATA_RAW_BOOTKEY2_2_BOOTKEY2_2_Pos = 0x0
	// Bit mask of BOOTKEY2_2 field.
	OTP_DATA_RAW_BOOTKEY2_2_BOOTKEY2_2_Msk = 0xffff

	// BOOTKEY2_3: Bits 63:48 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_3 field.
	OTP_DATA_RAW_BOOTKEY2_3_BOOTKEY2_3_Pos = 0x0
	// Bit mask of BOOTKEY2_3 field.
	OTP_DATA_RAW_BOOTKEY2_3_BOOTKEY2_3_Msk = 0xffff

	// BOOTKEY2_4: Bits 79:64 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_4 field.
	OTP_DATA_RAW_BOOTKEY2_4_BOOTKEY2_4_Pos = 0x0
	// Bit mask of BOOTKEY2_4 field.
	OTP_DATA_RAW_BOOTKEY2_4_BOOTKEY2_4_Msk = 0xffff

	// BOOTKEY2_5: Bits 95:80 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_5 field.
	OTP_DATA_RAW_BOOTKEY2_5_BOOTKEY2_5_Pos = 0x0
	// Bit mask of BOOTKEY2_5 field.
	OTP_DATA_RAW_BOOTKEY2_5_BOOTKEY2_5_Msk = 0xffff

	// BOOTKEY2_6: Bits 111:96 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_6 field.
	OTP_DATA_RAW_BOOTKEY2_6_BOOTKEY2_6_Pos = 0x0
	// Bit mask of BOOTKEY2_6 field.
	OTP_DATA_RAW_BOOTKEY2_6_BOOTKEY2_6_Msk = 0xffff

	// BOOTKEY2_7: Bits 127:112 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_7 field.
	OTP_DATA_RAW_BOOTKEY2_7_BOOTKEY2_7_Pos = 0x0
	// Bit mask of BOOTKEY2_7 field.
	OTP_DATA_RAW_BOOTKEY2_7_BOOTKEY2_7_Msk = 0xffff

	// BOOTKEY2_8: Bits 143:128 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_8 field.
	OTP_DATA_RAW_BOOTKEY2_8_BOOTKEY2_8_Pos = 0x0
	// Bit mask of BOOTKEY2_8 field.
	OTP_DATA_RAW_BOOTKEY2_8_BOOTKEY2_8_Msk = 0xffff

	// BOOTKEY2_9: Bits 159:144 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_9 field.
	OTP_DATA_RAW_BOOTKEY2_9_BOOTKEY2_9_Pos = 0x0
	// Bit mask of BOOTKEY2_9 field.
	OTP_DATA_RAW_BOOTKEY2_9_BOOTKEY2_9_Msk = 0xffff

	// BOOTKEY2_10: Bits 175:160 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_10 field.
	OTP_DATA_RAW_BOOTKEY2_10_BOOTKEY2_10_Pos = 0x0
	// Bit mask of BOOTKEY2_10 field.
	OTP_DATA_RAW_BOOTKEY2_10_BOOTKEY2_10_Msk = 0xffff

	// BOOTKEY2_11: Bits 191:176 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_11 field.
	OTP_DATA_RAW_BOOTKEY2_11_BOOTKEY2_11_Pos = 0x0
	// Bit mask of BOOTKEY2_11 field.
	OTP_DATA_RAW_BOOTKEY2_11_BOOTKEY2_11_Msk = 0xffff

	// BOOTKEY2_12: Bits 207:192 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_12 field.
	OTP_DATA_RAW_BOOTKEY2_12_BOOTKEY2_12_Pos = 0x0
	// Bit mask of BOOTKEY2_12 field.
	OTP_DATA_RAW_BOOTKEY2_12_BOOTKEY2_12_Msk = 0xffff

	// BOOTKEY2_13: Bits 223:208 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_13 field.
	OTP_DATA_RAW_BOOTKEY2_13_BOOTKEY2_13_Pos = 0x0
	// Bit mask of BOOTKEY2_13 field.
	OTP_DATA_RAW_BOOTKEY2_13_BOOTKEY2_13_Msk = 0xffff

	// BOOTKEY2_14: Bits 239:224 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_14 field.
	OTP_DATA_RAW_BOOTKEY2_14_BOOTKEY2_14_Pos = 0x0
	// Bit mask of BOOTKEY2_14 field.
	OTP_DATA_RAW_BOOTKEY2_14_BOOTKEY2_14_Msk = 0xffff

	// BOOTKEY2_15: Bits 255:240 of SHA-256 hash of boot key 2 (ECC)
	// Position of BOOTKEY2_15 field.
	OTP_DATA_RAW_BOOTKEY2_15_BOOTKEY2_15_Pos = 0x0
	// Bit mask of BOOTKEY2_15 field.
	OTP_DATA_RAW_BOOTKEY2_15_BOOTKEY2_15_Msk = 0xffff

	// BOOTKEY3_0: Bits 15:0 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_0 field.
	OTP_DATA_RAW_BOOTKEY3_0_BOOTKEY3_0_Pos = 0x0
	// Bit mask of BOOTKEY3_0 field.
	OTP_DATA_RAW_BOOTKEY3_0_BOOTKEY3_0_Msk = 0xffff

	// BOOTKEY3_1: Bits 31:16 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_1 field.
	OTP_DATA_RAW_BOOTKEY3_1_BOOTKEY3_1_Pos = 0x0
	// Bit mask of BOOTKEY3_1 field.
	OTP_DATA_RAW_BOOTKEY3_1_BOOTKEY3_1_Msk = 0xffff

	// BOOTKEY3_2: Bits 47:32 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_2 field.
	OTP_DATA_RAW_BOOTKEY3_2_BOOTKEY3_2_Pos = 0x0
	// Bit mask of BOOTKEY3_2 field.
	OTP_DATA_RAW_BOOTKEY3_2_BOOTKEY3_2_Msk = 0xffff

	// BOOTKEY3_3: Bits 63:48 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_3 field.
	OTP_DATA_RAW_BOOTKEY3_3_BOOTKEY3_3_Pos = 0x0
	// Bit mask of BOOTKEY3_3 field.
	OTP_DATA_RAW_BOOTKEY3_3_BOOTKEY3_3_Msk = 0xffff

	// BOOTKEY3_4: Bits 79:64 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_4 field.
	OTP_DATA_RAW_BOOTKEY3_4_BOOTKEY3_4_Pos = 0x0
	// Bit mask of BOOTKEY3_4 field.
	OTP_DATA_RAW_BOOTKEY3_4_BOOTKEY3_4_Msk = 0xffff

	// BOOTKEY3_5: Bits 95:80 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_5 field.
	OTP_DATA_RAW_BOOTKEY3_5_BOOTKEY3_5_Pos = 0x0
	// Bit mask of BOOTKEY3_5 field.
	OTP_DATA_RAW_BOOTKEY3_5_BOOTKEY3_5_Msk = 0xffff

	// BOOTKEY3_6: Bits 111:96 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_6 field.
	OTP_DATA_RAW_BOOTKEY3_6_BOOTKEY3_6_Pos = 0x0
	// Bit mask of BOOTKEY3_6 field.
	OTP_DATA_RAW_BOOTKEY3_6_BOOTKEY3_6_Msk = 0xffff

	// BOOTKEY3_7: Bits 127:112 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_7 field.
	OTP_DATA_RAW_BOOTKEY3_7_BOOTKEY3_7_Pos = 0x0
	// Bit mask of BOOTKEY3_7 field.
	OTP_DATA_RAW_BOOTKEY3_7_BOOTKEY3_7_Msk = 0xffff

	// BOOTKEY3_8: Bits 143:128 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_8 field.
	OTP_DATA_RAW_BOOTKEY3_8_BOOTKEY3_8_Pos = 0x0
	// Bit mask of BOOTKEY3_8 field.
	OTP_DATA_RAW_BOOTKEY3_8_BOOTKEY3_8_Msk = 0xffff

	// BOOTKEY3_9: Bits 159:144 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_9 field.
	OTP_DATA_RAW_BOOTKEY3_9_BOOTKEY3_9_Pos = 0x0
	// Bit mask of BOOTKEY3_9 field.
	OTP_DATA_RAW_BOOTKEY3_9_BOOTKEY3_9_Msk = 0xffff

	// BOOTKEY3_10: Bits 175:160 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_10 field.
	OTP_DATA_RAW_BOOTKEY3_10_BOOTKEY3_10_Pos = 0x0
	// Bit mask of BOOTKEY3_10 field.
	OTP_DATA_RAW_BOOTKEY3_10_BOOTKEY3_10_Msk = 0xffff

	// BOOTKEY3_11: Bits 191:176 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_11 field.
	OTP_DATA_RAW_BOOTKEY3_11_BOOTKEY3_11_Pos = 0x0
	// Bit mask of BOOTKEY3_11 field.
	OTP_DATA_RAW_BOOTKEY3_11_BOOTKEY3_11_Msk = 0xffff

	// BOOTKEY3_12: Bits 207:192 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_12 field.
	OTP_DATA_RAW_BOOTKEY3_12_BOOTKEY3_12_Pos = 0x0
	// Bit mask of BOOTKEY3_12 field.
	OTP_DATA_RAW_BOOTKEY3_12_BOOTKEY3_12_Msk = 0xffff

	// BOOTKEY3_13: Bits 223:208 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_13 field.
	OTP_DATA_RAW_BOOTKEY3_13_BOOTKEY3_13_Pos = 0x0
	// Bit mask of BOOTKEY3_13 field.
	OTP_DATA_RAW_BOOTKEY3_13_BOOTKEY3_13_Msk = 0xffff

	// BOOTKEY3_14: Bits 239:224 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_14 field.
	OTP_DATA_RAW_BOOTKEY3_14_BOOTKEY3_14_Pos = 0x0
	// Bit mask of BOOTKEY3_14 field.
	OTP_DATA_RAW_BOOTKEY3_14_BOOTKEY3_14_Msk = 0xffff

	// BOOTKEY3_15: Bits 255:240 of SHA-256 hash of boot key 3 (ECC)
	// Position of BOOTKEY3_15 field.
	OTP_DATA_RAW_BOOTKEY3_15_BOOTKEY3_15_Pos = 0x0
	// Bit mask of BOOTKEY3_15 field.
	OTP_DATA_RAW_BOOTKEY3_15_BOOTKEY3_15_Msk = 0xffff

	// KEY1_0: Bits 15:0 of OTP access key 1 (ECC)
	// Position of KEY1_0 field.
	OTP_DATA_RAW_KEY1_0_KEY1_0_Pos = 0x0
	// Bit mask of KEY1_0 field.
	OTP_DATA_RAW_KEY1_0_KEY1_0_Msk = 0xffff

	// KEY1_1: Bits 31:16 of OTP access key 1 (ECC)
	// Position of KEY1_1 field.
	OTP_DATA_RAW_KEY1_1_KEY1_1_Pos = 0x0
	// Bit mask of KEY1_1 field.
	OTP_DATA_RAW_KEY1_1_KEY1_1_Msk = 0xffff

	// KEY1_2: Bits 47:32 of OTP access key 1 (ECC)
	// Position of KEY1_2 field.
	OTP_DATA_RAW_KEY1_2_KEY1_2_Pos = 0x0
	// Bit mask of KEY1_2 field.
	OTP_DATA_RAW_KEY1_2_KEY1_2_Msk = 0xffff

	// KEY1_3: Bits 63:48 of OTP access key 1 (ECC)
	// Position of KEY1_3 field.
	OTP_DATA_RAW_KEY1_3_KEY1_3_Pos = 0x0
	// Bit mask of KEY1_3 field.
	OTP_DATA_RAW_KEY1_3_KEY1_3_Msk = 0xffff

	// KEY1_4: Bits 79:64 of OTP access key 1 (ECC)
	// Position of KEY1_4 field.
	OTP_DATA_RAW_KEY1_4_KEY1_4_Pos = 0x0
	// Bit mask of KEY1_4 field.
	OTP_DATA_RAW_KEY1_4_KEY1_4_Msk = 0xffff

	// KEY1_5: Bits 95:80 of OTP access key 1 (ECC)
	// Position of KEY1_5 field.
	OTP_DATA_RAW_KEY1_5_KEY1_5_Pos = 0x0
	// Bit mask of KEY1_5 field.
	OTP_DATA_RAW_KEY1_5_KEY1_5_Msk = 0xffff

	// KEY1_6: Bits 111:96 of OTP access key 1 (ECC)
	// Position of KEY1_6 field.
	OTP_DATA_RAW_KEY1_6_KEY1_6_Pos = 0x0
	// Bit mask of KEY1_6 field.
	OTP_DATA_RAW_KEY1_6_KEY1_6_Msk = 0xffff

	// KEY1_7: Bits 127:112 of OTP access key 1 (ECC)
	// Position of KEY1_7 field.
	OTP_DATA_RAW_KEY1_7_KEY1_7_Pos = 0x0
	// Bit mask of KEY1_7 field.
	OTP_DATA_RAW_KEY1_7_KEY1_7_Msk = 0xffff

	// KEY2_0: Bits 15:0 of OTP access key 2 (ECC)
	// Position of KEY2_0 field.
	OTP_DATA_RAW_KEY2_0_KEY2_0_Pos = 0x0
	// Bit mask of KEY2_0 field.
	OTP_DATA_RAW_KEY2_0_KEY2_0_Msk = 0xffff

	// KEY2_1: Bits 31:16 of OTP access key 2 (ECC)
	// Position of KEY2_1 field.
	OTP_DATA_RAW_KEY2_1_KEY2_1_Pos = 0x0
	// Bit mask of KEY2_1 field.
	OTP_DATA_RAW_KEY2_1_KEY2_1_Msk = 0xffff

	// KEY2_2: Bits 47:32 of OTP access key 2 (ECC)
	// Position of KEY2_2 field.
	OTP_DATA_RAW_KEY2_2_KEY2_2_Pos = 0x0
	// Bit mask of KEY2_2 field.
	OTP_DATA_RAW_KEY2_2_KEY2_2_Msk = 0xffff

	// KEY2_3: Bits 63:48 of OTP access key 2 (ECC)
	// Position of KEY2_3 field.
	OTP_DATA_RAW_KEY2_3_KEY2_3_Pos = 0x0
	// Bit mask of KEY2_3 field.
	OTP_DATA_RAW_KEY2_3_KEY2_3_Msk = 0xffff

	// KEY2_4: Bits 79:64 of OTP access key 2 (ECC)
	// Position of KEY2_4 field.
	OTP_DATA_RAW_KEY2_4_KEY2_4_Pos = 0x0
	// Bit mask of KEY2_4 field.
	OTP_DATA_RAW_KEY2_4_KEY2_4_Msk = 0xffff

	// KEY2_5: Bits 95:80 of OTP access key 2 (ECC)
	// Position of KEY2_5 field.
	OTP_DATA_RAW_KEY2_5_KEY2_5_Pos = 0x0
	// Bit mask of KEY2_5 field.
	OTP_DATA_RAW_KEY2_5_KEY2_5_Msk = 0xffff

	// KEY2_6: Bits 111:96 of OTP access key 2 (ECC)
	// Position of KEY2_6 field.
	OTP_DATA_RAW_KEY2_6_KEY2_6_Pos = 0x0
	// Bit mask of KEY2_6 field.
	OTP_DATA_RAW_KEY2_6_KEY2_6_Msk = 0xffff

	// KEY2_7: Bits 127:112 of OTP access key 2 (ECC)
	// Position of KEY2_7 field.
	OTP_DATA_RAW_KEY2_7_KEY2_7_Pos = 0x0
	// Bit mask of KEY2_7 field.
	OTP_DATA_RAW_KEY2_7_KEY2_7_Msk = 0xffff

	// KEY3_0: Bits 15:0 of OTP access key 3 (ECC)
	// Position of KEY3_0 field.
	OTP_DATA_RAW_KEY3_0_KEY3_0_Pos = 0x0
	// Bit mask of KEY3_0 field.
	OTP_DATA_RAW_KEY3_0_KEY3_0_Msk = 0xffff

	// KEY3_1: Bits 31:16 of OTP access key 3 (ECC)
	// Position of KEY3_1 field.
	OTP_DATA_RAW_KEY3_1_KEY3_1_Pos = 0x0
	// Bit mask of KEY3_1 field.
	OTP_DATA_RAW_KEY3_1_KEY3_1_Msk = 0xffff

	// KEY3_2: Bits 47:32 of OTP access key 3 (ECC)
	// Position of KEY3_2 field.
	OTP_DATA_RAW_KEY3_2_KEY3_2_Pos = 0x0
	// Bit mask of KEY3_2 field.
	OTP_DATA_RAW_KEY3_2_KEY3_2_Msk = 0xffff

	// KEY3_3: Bits 63:48 of OTP access key 3 (ECC)
	// Position of KEY3_3 field.
	OTP_DATA_RAW_KEY3_3_KEY3_3_Pos = 0x0
	// Bit mask of KEY3_3 field.
	OTP_DATA_RAW_KEY3_3_KEY3_3_Msk = 0xffff

	// KEY3_4: Bits 79:64 of OTP access key 3 (ECC)
	// Position of KEY3_4 field.
	OTP_DATA_RAW_KEY3_4_KEY3_4_Pos = 0x0
	// Bit mask of KEY3_4 field.
	OTP_DATA_RAW_KEY3_4_KEY3_4_Msk = 0xffff

	// KEY3_5: Bits 95:80 of OTP access key 3 (ECC)
	// Position of KEY3_5 field.
	OTP_DATA_RAW_KEY3_5_KEY3_5_Pos = 0x0
	// Bit mask of KEY3_5 field.
	OTP_DATA_RAW_KEY3_5_KEY3_5_Msk = 0xffff

	// KEY3_6: Bits 111:96 of OTP access key 3 (ECC)
	// Position of KEY3_6 field.
	OTP_DATA_RAW_KEY3_6_KEY3_6_Pos = 0x0
	// Bit mask of KEY3_6 field.
	OTP_DATA_RAW_KEY3_6_KEY3_6_Msk = 0xffff

	// KEY3_7: Bits 127:112 of OTP access key 3 (ECC)
	// Position of KEY3_7 field.
	OTP_DATA_RAW_KEY3_7_KEY3_7_Pos = 0x0
	// Bit mask of KEY3_7 field.
	OTP_DATA_RAW_KEY3_7_KEY3_7_Msk = 0xffff

	// KEY4_0: Bits 15:0 of OTP access key 4 (ECC)
	// Position of KEY4_0 field.
	OTP_DATA_RAW_KEY4_0_KEY4_0_Pos = 0x0
	// Bit mask of KEY4_0 field.
	OTP_DATA_RAW_KEY4_0_KEY4_0_Msk = 0xffff

	// KEY4_1: Bits 31:16 of OTP access key 4 (ECC)
	// Position of KEY4_1 field.
	OTP_DATA_RAW_KEY4_1_KEY4_1_Pos = 0x0
	// Bit mask of KEY4_1 field.
	OTP_DATA_RAW_KEY4_1_KEY4_1_Msk = 0xffff

	// KEY4_2: Bits 47:32 of OTP access key 4 (ECC)
	// Position of KEY4_2 field.
	OTP_DATA_RAW_KEY4_2_KEY4_2_Pos = 0x0
	// Bit mask of KEY4_2 field.
	OTP_DATA_RAW_KEY4_2_KEY4_2_Msk = 0xffff

	// KEY4_3: Bits 63:48 of OTP access key 4 (ECC)
	// Position of KEY4_3 field.
	OTP_DATA_RAW_KEY4_3_KEY4_3_Pos = 0x0
	// Bit mask of KEY4_3 field.
	OTP_DATA_RAW_KEY4_3_KEY4_3_Msk = 0xffff

	// KEY4_4: Bits 79:64 of OTP access key 4 (ECC)
	// Position of KEY4_4 field.
	OTP_DATA_RAW_KEY4_4_KEY4_4_Pos = 0x0
	// Bit mask of KEY4_4 field.
	OTP_DATA_RAW_KEY4_4_KEY4_4_Msk = 0xffff

	// KEY4_5: Bits 95:80 of OTP access key 4 (ECC)
	// Position of KEY4_5 field.
	OTP_DATA_RAW_KEY4_5_KEY4_5_Pos = 0x0
	// Bit mask of KEY4_5 field.
	OTP_DATA_RAW_KEY4_5_KEY4_5_Msk = 0xffff

	// KEY4_6: Bits 111:96 of OTP access key 4 (ECC)
	// Position of KEY4_6 field.
	OTP_DATA_RAW_KEY4_6_KEY4_6_Pos = 0x0
	// Bit mask of KEY4_6 field.
	OTP_DATA_RAW_KEY4_6_KEY4_6_Msk = 0xffff

	// KEY4_7: Bits 127:112 of OTP access key 4 (ECC)
	// Position of KEY4_7 field.
	OTP_DATA_RAW_KEY4_7_KEY4_7_Pos = 0x0
	// Bit mask of KEY4_7 field.
	OTP_DATA_RAW_KEY4_7_KEY4_7_Msk = 0xffff

	// KEY5_0: Bits 15:0 of OTP access key 5 (ECC)
	// Position of KEY5_0 field.
	OTP_DATA_RAW_KEY5_0_KEY5_0_Pos = 0x0
	// Bit mask of KEY5_0 field.
	OTP_DATA_RAW_KEY5_0_KEY5_0_Msk = 0xffff

	// KEY5_1: Bits 31:16 of OTP access key 5 (ECC)
	// Position of KEY5_1 field.
	OTP_DATA_RAW_KEY5_1_KEY5_1_Pos = 0x0
	// Bit mask of KEY5_1 field.
	OTP_DATA_RAW_KEY5_1_KEY5_1_Msk = 0xffff

	// KEY5_2: Bits 47:32 of OTP access key 5 (ECC)
	// Position of KEY5_2 field.
	OTP_DATA_RAW_KEY5_2_KEY5_2_Pos = 0x0
	// Bit mask of KEY5_2 field.
	OTP_DATA_RAW_KEY5_2_KEY5_2_Msk = 0xffff

	// KEY5_3: Bits 63:48 of OTP access key 5 (ECC)
	// Position of KEY5_3 field.
	OTP_DATA_RAW_KEY5_3_KEY5_3_Pos = 0x0
	// Bit mask of KEY5_3 field.
	OTP_DATA_RAW_KEY5_3_KEY5_3_Msk = 0xffff

	// KEY5_4: Bits 79:64 of OTP access key 5 (ECC)
	// Position of KEY5_4 field.
	OTP_DATA_RAW_KEY5_4_KEY5_4_Pos = 0x0
	// Bit mask of KEY5_4 field.
	OTP_DATA_RAW_KEY5_4_KEY5_4_Msk = 0xffff

	// KEY5_5: Bits 95:80 of OTP access key 5 (ECC)
	// Position of KEY5_5 field.
	OTP_DATA_RAW_KEY5_5_KEY5_5_Pos = 0x0
	// Bit mask of KEY5_5 field.
	OTP_DATA_RAW_KEY5_5_KEY5_5_Msk = 0xffff

	// KEY5_6: Bits 111:96 of OTP access key 5 (ECC)
	// Position of KEY5_6 field.
	OTP_DATA_RAW_KEY5_6_KEY5_6_Pos = 0x0
	// Bit mask of KEY5_6 field.
	OTP_DATA_RAW_KEY5_6_KEY5_6_Msk = 0xffff

	// KEY5_7: Bits 127:112 of OTP access key 5 (ECC)
	// Position of KEY5_7 field.
	OTP_DATA_RAW_KEY5_7_KEY5_7_Pos = 0x0
	// Bit mask of KEY5_7 field.
	OTP_DATA_RAW_KEY5_7_KEY5_7_Msk = 0xffff

	// KEY6_0: Bits 15:0 of OTP access key 6 (ECC)
	// Position of KEY6_0 field.
	OTP_DATA_RAW_KEY6_0_KEY6_0_Pos = 0x0
	// Bit mask of KEY6_0 field.
	OTP_DATA_RAW_KEY6_0_KEY6_0_Msk = 0xffff

	// KEY6_1: Bits 31:16 of OTP access key 6 (ECC)
	// Position of KEY6_1 field.
	OTP_DATA_RAW_KEY6_1_KEY6_1_Pos = 0x0
	// Bit mask of KEY6_1 field.
	OTP_DATA_RAW_KEY6_1_KEY6_1_Msk = 0xffff

	// KEY6_2: Bits 47:32 of OTP access key 6 (ECC)
	// Position of KEY6_2 field.
	OTP_DATA_RAW_KEY6_2_KEY6_2_Pos = 0x0
	// Bit mask of KEY6_2 field.
	OTP_DATA_RAW_KEY6_2_KEY6_2_Msk = 0xffff

	// KEY6_3: Bits 63:48 of OTP access key 6 (ECC)
	// Position of KEY6_3 field.
	OTP_DATA_RAW_KEY6_3_KEY6_3_Pos = 0x0
	// Bit mask of KEY6_3 field.
	OTP_DATA_RAW_KEY6_3_KEY6_3_Msk = 0xffff

	// KEY6_4: Bits 79:64 of OTP access key 6 (ECC)
	// Position of KEY6_4 field.
	OTP_DATA_RAW_KEY6_4_KEY6_4_Pos = 0x0
	// Bit mask of KEY6_4 field.
	OTP_DATA_RAW_KEY6_4_KEY6_4_Msk = 0xffff

	// KEY6_5: Bits 95:80 of OTP access key 6 (ECC)
	// Position of KEY6_5 field.
	OTP_DATA_RAW_KEY6_5_KEY6_5_Pos = 0x0
	// Bit mask of KEY6_5 field.
	OTP_DATA_RAW_KEY6_5_KEY6_5_Msk = 0xffff

	// KEY6_6: Bits 111:96 of OTP access key 6 (ECC)
	// Position of KEY6_6 field.
	OTP_DATA_RAW_KEY6_6_KEY6_6_Pos = 0x0
	// Bit mask of KEY6_6 field.
	OTP_DATA_RAW_KEY6_6_KEY6_6_Msk = 0xffff

	// KEY6_7: Bits 127:112 of OTP access key 6 (ECC)
	// Position of KEY6_7 field.
	OTP_DATA_RAW_KEY6_7_KEY6_7_Pos = 0x0
	// Bit mask of KEY6_7 field.
	OTP_DATA_RAW_KEY6_7_KEY6_7_Msk = 0xffff

	// KEY1_VALID: Valid flag for key 1. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
	// Position of VALID_R2 field.
	OTP_DATA_RAW_KEY1_VALID_VALID_R2_Pos = 0x10
	// Bit mask of VALID_R2 field.
	OTP_DATA_RAW_KEY1_VALID_VALID_R2_Msk = 0x10000
	// Bit VALID_R2.
	OTP_DATA_RAW_KEY1_VALID_VALID_R2 = 0x10000
	// Position of VALID_R1 field.
	OTP_DATA_RAW_KEY1_VALID_VALID_R1_Pos = 0x8
	// Bit mask of VALID_R1 field.
	OTP_DATA_RAW_KEY1_VALID_VALID_R1_Msk = 0x100
	// Bit VALID_R1.
	OTP_DATA_RAW_KEY1_VALID_VALID_R1 = 0x100
	// Position of VALID field.
	OTP_DATA_RAW_KEY1_VALID_VALID_Pos = 0x0
	// Bit mask of VALID field.
	OTP_DATA_RAW_KEY1_VALID_VALID_Msk = 0x1
	// Bit VALID.
	OTP_DATA_RAW_KEY1_VALID_VALID = 0x1

	// KEY2_VALID: Valid flag for key 2. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
	// Position of VALID_R2 field.
	OTP_DATA_RAW_KEY2_VALID_VALID_R2_Pos = 0x10
	// Bit mask of VALID_R2 field.
	OTP_DATA_RAW_KEY2_VALID_VALID_R2_Msk = 0x10000
	// Bit VALID_R2.
	OTP_DATA_RAW_KEY2_VALID_VALID_R2 = 0x10000
	// Position of VALID_R1 field.
	OTP_DATA_RAW_KEY2_VALID_VALID_R1_Pos = 0x8
	// Bit mask of VALID_R1 field.
	OTP_DATA_RAW_KEY2_VALID_VALID_R1_Msk = 0x100
	// Bit VALID_R1.
	OTP_DATA_RAW_KEY2_VALID_VALID_R1 = 0x100
	// Position of VALID field.
	OTP_DATA_RAW_KEY2_VALID_VALID_Pos = 0x0
	// Bit mask of VALID field.
	OTP_DATA_RAW_KEY2_VALID_VALID_Msk = 0x1
	// Bit VALID.
	OTP_DATA_RAW_KEY2_VALID_VALID = 0x1

	// KEY3_VALID: Valid flag for key 3. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
	// Position of VALID_R2 field.
	OTP_DATA_RAW_KEY3_VALID_VALID_R2_Pos = 0x10
	// Bit mask of VALID_R2 field.
	OTP_DATA_RAW_KEY3_VALID_VALID_R2_Msk = 0x10000
	// Bit VALID_R2.
	OTP_DATA_RAW_KEY3_VALID_VALID_R2 = 0x10000
	// Position of VALID_R1 field.
	OTP_DATA_RAW_KEY3_VALID_VALID_R1_Pos = 0x8
	// Bit mask of VALID_R1 field.
	OTP_DATA_RAW_KEY3_VALID_VALID_R1_Msk = 0x100
	// Bit VALID_R1.
	OTP_DATA_RAW_KEY3_VALID_VALID_R1 = 0x100
	// Position of VALID field.
	OTP_DATA_RAW_KEY3_VALID_VALID_Pos = 0x0
	// Bit mask of VALID field.
	OTP_DATA_RAW_KEY3_VALID_VALID_Msk = 0x1
	// Bit VALID.
	OTP_DATA_RAW_KEY3_VALID_VALID = 0x1

	// KEY4_VALID: Valid flag for key 4. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
	// Position of VALID_R2 field.
	OTP_DATA_RAW_KEY4_VALID_VALID_R2_Pos = 0x10
	// Bit mask of VALID_R2 field.
	OTP_DATA_RAW_KEY4_VALID_VALID_R2_Msk = 0x10000
	// Bit VALID_R2.
	OTP_DATA_RAW_KEY4_VALID_VALID_R2 = 0x10000
	// Position of VALID_R1 field.
	OTP_DATA_RAW_KEY4_VALID_VALID_R1_Pos = 0x8
	// Bit mask of VALID_R1 field.
	OTP_DATA_RAW_KEY4_VALID_VALID_R1_Msk = 0x100
	// Bit VALID_R1.
	OTP_DATA_RAW_KEY4_VALID_VALID_R1 = 0x100
	// Position of VALID field.
	OTP_DATA_RAW_KEY4_VALID_VALID_Pos = 0x0
	// Bit mask of VALID field.
	OTP_DATA_RAW_KEY4_VALID_VALID_Msk = 0x1
	// Bit VALID.
	OTP_DATA_RAW_KEY4_VALID_VALID = 0x1

	// KEY5_VALID: Valid flag for key 5. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
	// Position of VALID_R2 field.
	OTP_DATA_RAW_KEY5_VALID_VALID_R2_Pos = 0x10
	// Bit mask of VALID_R2 field.
	OTP_DATA_RAW_KEY5_VALID_VALID_R2_Msk = 0x10000
	// Bit VALID_R2.
	OTP_DATA_RAW_KEY5_VALID_VALID_R2 = 0x10000
	// Position of VALID_R1 field.
	OTP_DATA_RAW_KEY5_VALID_VALID_R1_Pos = 0x8
	// Bit mask of VALID_R1 field.
	OTP_DATA_RAW_KEY5_VALID_VALID_R1_Msk = 0x100
	// Bit VALID_R1.
	OTP_DATA_RAW_KEY5_VALID_VALID_R1 = 0x100
	// Position of VALID field.
	OTP_DATA_RAW_KEY5_VALID_VALID_Pos = 0x0
	// Bit mask of VALID field.
	OTP_DATA_RAW_KEY5_VALID_VALID_Msk = 0x1
	// Bit VALID.
	OTP_DATA_RAW_KEY5_VALID_VALID = 0x1

	// KEY6_VALID: Valid flag for key 6. Once the valid flag is set, the key can no longer be read or written, and becomes a valid fixed key for protecting OTP pages.
	// Position of VALID_R2 field.
	OTP_DATA_RAW_KEY6_VALID_VALID_R2_Pos = 0x10
	// Bit mask of VALID_R2 field.
	OTP_DATA_RAW_KEY6_VALID_VALID_R2_Msk = 0x10000
	// Bit VALID_R2.
	OTP_DATA_RAW_KEY6_VALID_VALID_R2 = 0x10000
	// Position of VALID_R1 field.
	OTP_DATA_RAW_KEY6_VALID_VALID_R1_Pos = 0x8
	// Bit mask of VALID_R1 field.
	OTP_DATA_RAW_KEY6_VALID_VALID_R1_Msk = 0x100
	// Bit VALID_R1.
	OTP_DATA_RAW_KEY6_VALID_VALID_R1 = 0x100
	// Position of VALID field.
	OTP_DATA_RAW_KEY6_VALID_VALID_Pos = 0x0
	// Bit mask of VALID field.
	OTP_DATA_RAW_KEY6_VALID_VALID_Msk = 0x1
	// Bit VALID.
	OTP_DATA_RAW_KEY6_VALID_VALID = 0x1

	// PAGE0_LOCK0: Lock configuration LSBs for page 0 (rows 0x0 through 0x3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE0_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE0_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE0_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE0_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE0_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE0_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE0_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE0_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE0_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE0_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE0_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE0_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE0_LOCK0_KEY_W_Msk = 0x7

	// PAGE0_LOCK1: Lock configuration MSBs for page 0 (rows 0x0 through 0x3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE0_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE0_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE0_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE0_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE0_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE1_LOCK0: Lock configuration LSBs for page 1 (rows 0x40 through 0x7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE1_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE1_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE1_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE1_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE1_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE1_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE1_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE1_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE1_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE1_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE1_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE1_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE1_LOCK0_KEY_W_Msk = 0x7

	// PAGE1_LOCK1: Lock configuration MSBs for page 1 (rows 0x40 through 0x7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE1_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE1_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE1_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE1_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE1_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE2_LOCK0: Lock configuration LSBs for page 2 (rows 0x80 through 0xbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE2_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE2_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE2_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE2_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE2_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE2_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE2_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE2_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE2_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE2_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE2_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE2_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE2_LOCK0_KEY_W_Msk = 0x7

	// PAGE2_LOCK1: Lock configuration MSBs for page 2 (rows 0x80 through 0xbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE2_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE2_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE2_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE2_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE2_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE3_LOCK0: Lock configuration LSBs for page 3 (rows 0xc0 through 0xff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE3_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE3_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE3_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE3_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE3_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE3_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE3_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE3_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE3_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE3_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE3_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE3_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE3_LOCK0_KEY_W_Msk = 0x7

	// PAGE3_LOCK1: Lock configuration MSBs for page 3 (rows 0xc0 through 0xff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE3_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE3_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE3_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE3_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE3_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE4_LOCK0: Lock configuration LSBs for page 4 (rows 0x100 through 0x13f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE4_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE4_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE4_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE4_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE4_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE4_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE4_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE4_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE4_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE4_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE4_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE4_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE4_LOCK0_KEY_W_Msk = 0x7

	// PAGE4_LOCK1: Lock configuration MSBs for page 4 (rows 0x100 through 0x13f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE4_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE4_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE4_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE4_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE4_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE5_LOCK0: Lock configuration LSBs for page 5 (rows 0x140 through 0x17f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE5_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE5_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE5_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE5_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE5_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE5_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE5_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE5_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE5_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE5_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE5_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE5_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE5_LOCK0_KEY_W_Msk = 0x7

	// PAGE5_LOCK1: Lock configuration MSBs for page 5 (rows 0x140 through 0x17f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE5_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE5_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE5_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE5_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE5_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE6_LOCK0: Lock configuration LSBs for page 6 (rows 0x180 through 0x1bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE6_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE6_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE6_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE6_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE6_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE6_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE6_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE6_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE6_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE6_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE6_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE6_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE6_LOCK0_KEY_W_Msk = 0x7

	// PAGE6_LOCK1: Lock configuration MSBs for page 6 (rows 0x180 through 0x1bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE6_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE6_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE6_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE6_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE6_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE7_LOCK0: Lock configuration LSBs for page 7 (rows 0x1c0 through 0x1ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE7_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE7_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE7_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE7_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE7_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE7_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE7_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE7_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE7_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE7_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE7_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE7_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE7_LOCK0_KEY_W_Msk = 0x7

	// PAGE7_LOCK1: Lock configuration MSBs for page 7 (rows 0x1c0 through 0x1ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE7_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE7_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE7_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE7_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE7_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE8_LOCK0: Lock configuration LSBs for page 8 (rows 0x200 through 0x23f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE8_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE8_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE8_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE8_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE8_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE8_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE8_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE8_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE8_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE8_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE8_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE8_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE8_LOCK0_KEY_W_Msk = 0x7

	// PAGE8_LOCK1: Lock configuration MSBs for page 8 (rows 0x200 through 0x23f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE8_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE8_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE8_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE8_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE8_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE9_LOCK0: Lock configuration LSBs for page 9 (rows 0x240 through 0x27f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE9_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE9_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE9_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE9_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE9_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE9_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE9_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE9_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE9_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE9_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE9_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE9_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE9_LOCK0_KEY_W_Msk = 0x7

	// PAGE9_LOCK1: Lock configuration MSBs for page 9 (rows 0x240 through 0x27f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE9_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE9_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE9_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE9_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE9_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE10_LOCK0: Lock configuration LSBs for page 10 (rows 0x280 through 0x2bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE10_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE10_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE10_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE10_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE10_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE10_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE10_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE10_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE10_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE10_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE10_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE10_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE10_LOCK0_KEY_W_Msk = 0x7

	// PAGE10_LOCK1: Lock configuration MSBs for page 10 (rows 0x280 through 0x2bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE10_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE10_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE10_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE10_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE10_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE11_LOCK0: Lock configuration LSBs for page 11 (rows 0x2c0 through 0x2ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE11_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE11_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE11_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE11_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE11_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE11_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE11_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE11_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE11_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE11_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE11_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE11_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE11_LOCK0_KEY_W_Msk = 0x7

	// PAGE11_LOCK1: Lock configuration MSBs for page 11 (rows 0x2c0 through 0x2ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE11_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE11_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE11_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE11_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE11_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE12_LOCK0: Lock configuration LSBs for page 12 (rows 0x300 through 0x33f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE12_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE12_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE12_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE12_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE12_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE12_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE12_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE12_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE12_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE12_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE12_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE12_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE12_LOCK0_KEY_W_Msk = 0x7

	// PAGE12_LOCK1: Lock configuration MSBs for page 12 (rows 0x300 through 0x33f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE12_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE12_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE12_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE12_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE12_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE13_LOCK0: Lock configuration LSBs for page 13 (rows 0x340 through 0x37f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE13_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE13_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE13_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE13_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE13_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE13_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE13_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE13_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE13_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE13_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE13_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE13_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE13_LOCK0_KEY_W_Msk = 0x7

	// PAGE13_LOCK1: Lock configuration MSBs for page 13 (rows 0x340 through 0x37f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE13_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE13_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE13_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE13_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE13_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE14_LOCK0: Lock configuration LSBs for page 14 (rows 0x380 through 0x3bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE14_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE14_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE14_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE14_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE14_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE14_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE14_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE14_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE14_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE14_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE14_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE14_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE14_LOCK0_KEY_W_Msk = 0x7

	// PAGE14_LOCK1: Lock configuration MSBs for page 14 (rows 0x380 through 0x3bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE14_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE14_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE14_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE14_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE14_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE15_LOCK0: Lock configuration LSBs for page 15 (rows 0x3c0 through 0x3ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE15_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE15_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE15_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE15_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE15_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE15_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE15_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE15_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE15_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE15_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE15_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE15_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE15_LOCK0_KEY_W_Msk = 0x7

	// PAGE15_LOCK1: Lock configuration MSBs for page 15 (rows 0x3c0 through 0x3ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE15_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE15_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE15_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE15_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE15_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE16_LOCK0: Lock configuration LSBs for page 16 (rows 0x400 through 0x43f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE16_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE16_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE16_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE16_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE16_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE16_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE16_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE16_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE16_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE16_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE16_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE16_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE16_LOCK0_KEY_W_Msk = 0x7

	// PAGE16_LOCK1: Lock configuration MSBs for page 16 (rows 0x400 through 0x43f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE16_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE16_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE16_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE16_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE16_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE17_LOCK0: Lock configuration LSBs for page 17 (rows 0x440 through 0x47f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE17_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE17_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE17_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE17_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE17_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE17_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE17_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE17_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE17_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE17_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE17_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE17_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE17_LOCK0_KEY_W_Msk = 0x7

	// PAGE17_LOCK1: Lock configuration MSBs for page 17 (rows 0x440 through 0x47f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE17_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE17_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE17_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE17_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE17_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE18_LOCK0: Lock configuration LSBs for page 18 (rows 0x480 through 0x4bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE18_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE18_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE18_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE18_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE18_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE18_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE18_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE18_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE18_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE18_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE18_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE18_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE18_LOCK0_KEY_W_Msk = 0x7

	// PAGE18_LOCK1: Lock configuration MSBs for page 18 (rows 0x480 through 0x4bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE18_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE18_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE18_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE18_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE18_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE19_LOCK0: Lock configuration LSBs for page 19 (rows 0x4c0 through 0x4ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE19_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE19_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE19_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE19_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE19_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE19_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE19_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE19_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE19_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE19_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE19_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE19_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE19_LOCK0_KEY_W_Msk = 0x7

	// PAGE19_LOCK1: Lock configuration MSBs for page 19 (rows 0x4c0 through 0x4ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE19_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE19_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE19_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE19_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE19_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE20_LOCK0: Lock configuration LSBs for page 20 (rows 0x500 through 0x53f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE20_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE20_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE20_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE20_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE20_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE20_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE20_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE20_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE20_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE20_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE20_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE20_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE20_LOCK0_KEY_W_Msk = 0x7

	// PAGE20_LOCK1: Lock configuration MSBs for page 20 (rows 0x500 through 0x53f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE20_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE20_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE20_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE20_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE20_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE21_LOCK0: Lock configuration LSBs for page 21 (rows 0x540 through 0x57f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE21_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE21_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE21_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE21_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE21_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE21_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE21_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE21_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE21_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE21_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE21_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE21_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE21_LOCK0_KEY_W_Msk = 0x7

	// PAGE21_LOCK1: Lock configuration MSBs for page 21 (rows 0x540 through 0x57f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE21_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE21_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE21_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE21_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE21_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE22_LOCK0: Lock configuration LSBs for page 22 (rows 0x580 through 0x5bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE22_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE22_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE22_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE22_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE22_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE22_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE22_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE22_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE22_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE22_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE22_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE22_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE22_LOCK0_KEY_W_Msk = 0x7

	// PAGE22_LOCK1: Lock configuration MSBs for page 22 (rows 0x580 through 0x5bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE22_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE22_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE22_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE22_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE22_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE23_LOCK0: Lock configuration LSBs for page 23 (rows 0x5c0 through 0x5ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE23_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE23_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE23_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE23_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE23_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE23_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE23_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE23_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE23_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE23_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE23_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE23_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE23_LOCK0_KEY_W_Msk = 0x7

	// PAGE23_LOCK1: Lock configuration MSBs for page 23 (rows 0x5c0 through 0x5ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE23_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE23_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE23_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE23_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE23_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE24_LOCK0: Lock configuration LSBs for page 24 (rows 0x600 through 0x63f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE24_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE24_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE24_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE24_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE24_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE24_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE24_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE24_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE24_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE24_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE24_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE24_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE24_LOCK0_KEY_W_Msk = 0x7

	// PAGE24_LOCK1: Lock configuration MSBs for page 24 (rows 0x600 through 0x63f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE24_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE24_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE24_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE24_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE24_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE25_LOCK0: Lock configuration LSBs for page 25 (rows 0x640 through 0x67f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE25_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE25_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE25_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE25_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE25_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE25_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE25_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE25_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE25_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE25_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE25_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE25_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE25_LOCK0_KEY_W_Msk = 0x7

	// PAGE25_LOCK1: Lock configuration MSBs for page 25 (rows 0x640 through 0x67f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE25_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE25_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE25_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE25_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE25_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE26_LOCK0: Lock configuration LSBs for page 26 (rows 0x680 through 0x6bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE26_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE26_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE26_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE26_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE26_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE26_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE26_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE26_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE26_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE26_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE26_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE26_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE26_LOCK0_KEY_W_Msk = 0x7

	// PAGE26_LOCK1: Lock configuration MSBs for page 26 (rows 0x680 through 0x6bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE26_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE26_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE26_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE26_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE26_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE27_LOCK0: Lock configuration LSBs for page 27 (rows 0x6c0 through 0x6ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE27_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE27_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE27_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE27_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE27_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE27_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE27_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE27_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE27_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE27_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE27_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE27_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE27_LOCK0_KEY_W_Msk = 0x7

	// PAGE27_LOCK1: Lock configuration MSBs for page 27 (rows 0x6c0 through 0x6ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE27_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE27_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE27_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE27_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE27_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE28_LOCK0: Lock configuration LSBs for page 28 (rows 0x700 through 0x73f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE28_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE28_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE28_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE28_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE28_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE28_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE28_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE28_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE28_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE28_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE28_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE28_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE28_LOCK0_KEY_W_Msk = 0x7

	// PAGE28_LOCK1: Lock configuration MSBs for page 28 (rows 0x700 through 0x73f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE28_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE28_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE28_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE28_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE28_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE29_LOCK0: Lock configuration LSBs for page 29 (rows 0x740 through 0x77f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE29_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE29_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE29_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE29_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE29_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE29_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE29_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE29_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE29_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE29_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE29_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE29_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE29_LOCK0_KEY_W_Msk = 0x7

	// PAGE29_LOCK1: Lock configuration MSBs for page 29 (rows 0x740 through 0x77f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE29_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE29_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE29_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE29_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE29_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE30_LOCK0: Lock configuration LSBs for page 30 (rows 0x780 through 0x7bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE30_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE30_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE30_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE30_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE30_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE30_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE30_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE30_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE30_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE30_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE30_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE30_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE30_LOCK0_KEY_W_Msk = 0x7

	// PAGE30_LOCK1: Lock configuration MSBs for page 30 (rows 0x780 through 0x7bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE30_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE30_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE30_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE30_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE30_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE31_LOCK0: Lock configuration LSBs for page 31 (rows 0x7c0 through 0x7ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE31_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE31_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE31_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE31_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE31_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE31_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE31_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE31_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE31_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE31_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE31_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE31_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE31_LOCK0_KEY_W_Msk = 0x7

	// PAGE31_LOCK1: Lock configuration MSBs for page 31 (rows 0x7c0 through 0x7ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE31_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE31_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE31_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE31_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE31_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE32_LOCK0: Lock configuration LSBs for page 32 (rows 0x800 through 0x83f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE32_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE32_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE32_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE32_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE32_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE32_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE32_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE32_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE32_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE32_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE32_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE32_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE32_LOCK0_KEY_W_Msk = 0x7

	// PAGE32_LOCK1: Lock configuration MSBs for page 32 (rows 0x800 through 0x83f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE32_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE32_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE32_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE32_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE32_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE33_LOCK0: Lock configuration LSBs for page 33 (rows 0x840 through 0x87f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE33_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE33_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE33_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE33_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE33_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE33_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE33_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE33_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE33_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE33_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE33_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE33_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE33_LOCK0_KEY_W_Msk = 0x7

	// PAGE33_LOCK1: Lock configuration MSBs for page 33 (rows 0x840 through 0x87f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE33_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE33_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE33_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE33_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE33_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE34_LOCK0: Lock configuration LSBs for page 34 (rows 0x880 through 0x8bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE34_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE34_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE34_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE34_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE34_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE34_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE34_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE34_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE34_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE34_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE34_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE34_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE34_LOCK0_KEY_W_Msk = 0x7

	// PAGE34_LOCK1: Lock configuration MSBs for page 34 (rows 0x880 through 0x8bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE34_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE34_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE34_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE34_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE34_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE35_LOCK0: Lock configuration LSBs for page 35 (rows 0x8c0 through 0x8ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE35_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE35_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE35_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE35_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE35_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE35_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE35_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE35_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE35_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE35_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE35_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE35_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE35_LOCK0_KEY_W_Msk = 0x7

	// PAGE35_LOCK1: Lock configuration MSBs for page 35 (rows 0x8c0 through 0x8ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE35_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE35_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE35_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE35_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE35_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE36_LOCK0: Lock configuration LSBs for page 36 (rows 0x900 through 0x93f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE36_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE36_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE36_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE36_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE36_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE36_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE36_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE36_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE36_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE36_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE36_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE36_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE36_LOCK0_KEY_W_Msk = 0x7

	// PAGE36_LOCK1: Lock configuration MSBs for page 36 (rows 0x900 through 0x93f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE36_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE36_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE36_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE36_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE36_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE37_LOCK0: Lock configuration LSBs for page 37 (rows 0x940 through 0x97f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE37_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE37_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE37_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE37_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE37_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE37_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE37_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE37_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE37_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE37_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE37_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE37_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE37_LOCK0_KEY_W_Msk = 0x7

	// PAGE37_LOCK1: Lock configuration MSBs for page 37 (rows 0x940 through 0x97f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE37_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE37_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE37_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE37_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE37_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE38_LOCK0: Lock configuration LSBs for page 38 (rows 0x980 through 0x9bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE38_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE38_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE38_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE38_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE38_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE38_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE38_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE38_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE38_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE38_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE38_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE38_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE38_LOCK0_KEY_W_Msk = 0x7

	// PAGE38_LOCK1: Lock configuration MSBs for page 38 (rows 0x980 through 0x9bf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE38_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE38_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE38_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE38_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE38_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE39_LOCK0: Lock configuration LSBs for page 39 (rows 0x9c0 through 0x9ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE39_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE39_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE39_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE39_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE39_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE39_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE39_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE39_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE39_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE39_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE39_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE39_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE39_LOCK0_KEY_W_Msk = 0x7

	// PAGE39_LOCK1: Lock configuration MSBs for page 39 (rows 0x9c0 through 0x9ff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE39_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE39_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE39_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE39_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE39_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE40_LOCK0: Lock configuration LSBs for page 40 (rows 0xa00 through 0xa3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE40_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE40_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE40_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE40_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE40_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE40_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE40_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE40_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE40_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE40_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE40_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE40_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE40_LOCK0_KEY_W_Msk = 0x7

	// PAGE40_LOCK1: Lock configuration MSBs for page 40 (rows 0xa00 through 0xa3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE40_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE40_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE40_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE40_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE40_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE41_LOCK0: Lock configuration LSBs for page 41 (rows 0xa40 through 0xa7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE41_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE41_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE41_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE41_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE41_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE41_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE41_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE41_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE41_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE41_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE41_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE41_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE41_LOCK0_KEY_W_Msk = 0x7

	// PAGE41_LOCK1: Lock configuration MSBs for page 41 (rows 0xa40 through 0xa7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE41_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE41_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE41_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE41_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE41_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE42_LOCK0: Lock configuration LSBs for page 42 (rows 0xa80 through 0xabf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE42_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE42_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE42_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE42_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE42_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE42_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE42_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE42_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE42_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE42_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE42_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE42_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE42_LOCK0_KEY_W_Msk = 0x7

	// PAGE42_LOCK1: Lock configuration MSBs for page 42 (rows 0xa80 through 0xabf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE42_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE42_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE42_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE42_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE42_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE43_LOCK0: Lock configuration LSBs for page 43 (rows 0xac0 through 0xaff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE43_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE43_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE43_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE43_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE43_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE43_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE43_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE43_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE43_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE43_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE43_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE43_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE43_LOCK0_KEY_W_Msk = 0x7

	// PAGE43_LOCK1: Lock configuration MSBs for page 43 (rows 0xac0 through 0xaff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE43_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE43_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE43_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE43_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE43_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE44_LOCK0: Lock configuration LSBs for page 44 (rows 0xb00 through 0xb3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE44_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE44_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE44_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE44_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE44_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE44_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE44_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE44_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE44_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE44_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE44_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE44_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE44_LOCK0_KEY_W_Msk = 0x7

	// PAGE44_LOCK1: Lock configuration MSBs for page 44 (rows 0xb00 through 0xb3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE44_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE44_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE44_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE44_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE44_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE45_LOCK0: Lock configuration LSBs for page 45 (rows 0xb40 through 0xb7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE45_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE45_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE45_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE45_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE45_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE45_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE45_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE45_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE45_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE45_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE45_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE45_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE45_LOCK0_KEY_W_Msk = 0x7

	// PAGE45_LOCK1: Lock configuration MSBs for page 45 (rows 0xb40 through 0xb7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE45_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE45_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE45_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE45_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE45_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE46_LOCK0: Lock configuration LSBs for page 46 (rows 0xb80 through 0xbbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE46_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE46_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE46_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE46_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE46_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE46_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE46_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE46_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE46_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE46_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE46_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE46_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE46_LOCK0_KEY_W_Msk = 0x7

	// PAGE46_LOCK1: Lock configuration MSBs for page 46 (rows 0xb80 through 0xbbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE46_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE46_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE46_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE46_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE46_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE47_LOCK0: Lock configuration LSBs for page 47 (rows 0xbc0 through 0xbff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE47_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE47_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE47_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE47_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE47_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE47_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE47_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE47_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE47_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE47_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE47_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE47_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE47_LOCK0_KEY_W_Msk = 0x7

	// PAGE47_LOCK1: Lock configuration MSBs for page 47 (rows 0xbc0 through 0xbff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE47_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE47_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE47_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE47_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE47_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE48_LOCK0: Lock configuration LSBs for page 48 (rows 0xc00 through 0xc3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE48_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE48_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE48_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE48_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE48_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE48_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE48_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE48_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE48_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE48_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE48_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE48_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE48_LOCK0_KEY_W_Msk = 0x7

	// PAGE48_LOCK1: Lock configuration MSBs for page 48 (rows 0xc00 through 0xc3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE48_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE48_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE48_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE48_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE48_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE49_LOCK0: Lock configuration LSBs for page 49 (rows 0xc40 through 0xc7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE49_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE49_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE49_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE49_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE49_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE49_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE49_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE49_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE49_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE49_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE49_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE49_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE49_LOCK0_KEY_W_Msk = 0x7

	// PAGE49_LOCK1: Lock configuration MSBs for page 49 (rows 0xc40 through 0xc7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE49_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE49_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE49_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE49_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE49_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE50_LOCK0: Lock configuration LSBs for page 50 (rows 0xc80 through 0xcbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE50_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE50_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE50_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE50_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE50_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE50_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE50_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE50_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE50_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE50_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE50_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE50_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE50_LOCK0_KEY_W_Msk = 0x7

	// PAGE50_LOCK1: Lock configuration MSBs for page 50 (rows 0xc80 through 0xcbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE50_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE50_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE50_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE50_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE50_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE51_LOCK0: Lock configuration LSBs for page 51 (rows 0xcc0 through 0xcff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE51_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE51_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE51_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE51_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE51_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE51_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE51_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE51_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE51_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE51_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE51_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE51_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE51_LOCK0_KEY_W_Msk = 0x7

	// PAGE51_LOCK1: Lock configuration MSBs for page 51 (rows 0xcc0 through 0xcff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE51_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE51_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE51_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE51_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE51_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE52_LOCK0: Lock configuration LSBs for page 52 (rows 0xd00 through 0xd3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE52_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE52_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE52_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE52_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE52_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE52_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE52_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE52_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE52_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE52_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE52_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE52_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE52_LOCK0_KEY_W_Msk = 0x7

	// PAGE52_LOCK1: Lock configuration MSBs for page 52 (rows 0xd00 through 0xd3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE52_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE52_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE52_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE52_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE52_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE53_LOCK0: Lock configuration LSBs for page 53 (rows 0xd40 through 0xd7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE53_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE53_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE53_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE53_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE53_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE53_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE53_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE53_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE53_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE53_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE53_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE53_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE53_LOCK0_KEY_W_Msk = 0x7

	// PAGE53_LOCK1: Lock configuration MSBs for page 53 (rows 0xd40 through 0xd7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE53_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE53_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE53_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE53_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE53_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE54_LOCK0: Lock configuration LSBs for page 54 (rows 0xd80 through 0xdbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE54_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE54_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE54_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE54_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE54_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE54_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE54_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE54_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE54_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE54_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE54_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE54_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE54_LOCK0_KEY_W_Msk = 0x7

	// PAGE54_LOCK1: Lock configuration MSBs for page 54 (rows 0xd80 through 0xdbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE54_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE54_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE54_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE54_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE54_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE55_LOCK0: Lock configuration LSBs for page 55 (rows 0xdc0 through 0xdff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE55_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE55_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE55_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE55_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE55_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE55_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE55_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE55_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE55_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE55_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE55_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE55_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE55_LOCK0_KEY_W_Msk = 0x7

	// PAGE55_LOCK1: Lock configuration MSBs for page 55 (rows 0xdc0 through 0xdff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE55_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE55_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE55_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE55_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE55_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE56_LOCK0: Lock configuration LSBs for page 56 (rows 0xe00 through 0xe3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE56_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE56_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE56_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE56_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE56_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE56_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE56_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE56_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE56_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE56_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE56_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE56_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE56_LOCK0_KEY_W_Msk = 0x7

	// PAGE56_LOCK1: Lock configuration MSBs for page 56 (rows 0xe00 through 0xe3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE56_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE56_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE56_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE56_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE56_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE57_LOCK0: Lock configuration LSBs for page 57 (rows 0xe40 through 0xe7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE57_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE57_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE57_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE57_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE57_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE57_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE57_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE57_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE57_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE57_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE57_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE57_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE57_LOCK0_KEY_W_Msk = 0x7

	// PAGE57_LOCK1: Lock configuration MSBs for page 57 (rows 0xe40 through 0xe7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE57_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE57_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE57_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE57_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE57_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE58_LOCK0: Lock configuration LSBs for page 58 (rows 0xe80 through 0xebf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE58_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE58_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE58_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE58_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE58_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE58_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE58_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE58_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE58_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE58_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE58_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE58_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE58_LOCK0_KEY_W_Msk = 0x7

	// PAGE58_LOCK1: Lock configuration MSBs for page 58 (rows 0xe80 through 0xebf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE58_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE58_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE58_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE58_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE58_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE59_LOCK0: Lock configuration LSBs for page 59 (rows 0xec0 through 0xeff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE59_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE59_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE59_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE59_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE59_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE59_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE59_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE59_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE59_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE59_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE59_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE59_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE59_LOCK0_KEY_W_Msk = 0x7

	// PAGE59_LOCK1: Lock configuration MSBs for page 59 (rows 0xec0 through 0xeff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE59_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE59_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE59_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE59_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE59_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE60_LOCK0: Lock configuration LSBs for page 60 (rows 0xf00 through 0xf3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE60_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE60_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE60_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE60_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE60_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE60_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE60_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE60_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE60_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE60_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE60_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE60_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE60_LOCK0_KEY_W_Msk = 0x7

	// PAGE60_LOCK1: Lock configuration MSBs for page 60 (rows 0xf00 through 0xf3f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE60_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE60_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE60_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE60_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE60_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE61_LOCK0: Lock configuration LSBs for page 61 (rows 0xf40 through 0xf7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE61_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE61_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE61_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE61_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE61_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE61_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE61_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE61_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE61_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE61_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE61_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE61_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE61_LOCK0_KEY_W_Msk = 0x7

	// PAGE61_LOCK1: Lock configuration MSBs for page 61 (rows 0xf40 through 0xf7f). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE61_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE61_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE61_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE61_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE61_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE62_LOCK0: Lock configuration LSBs for page 62 (rows 0xf80 through 0xfbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE62_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE62_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE62_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE62_LOCK0_R1_Msk = 0xff00
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE62_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE62_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE62_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE62_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE62_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE62_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE62_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE62_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE62_LOCK0_KEY_W_Msk = 0x7

	// PAGE62_LOCK1: Lock configuration MSBs for page 62 (rows 0xf80 through 0xfbf). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE62_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE62_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE62_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE62_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE62_LOCK1_LOCK_S_INACCESSIBLE = 0x3

	// PAGE63_LOCK0: Lock configuration LSBs for page 63 (rows 0xfc0 through 0xfff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE63_LOCK0_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE63_LOCK0_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE63_LOCK0_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE63_LOCK0_R1_Msk = 0xff00
	// Position of RMA field.
	OTP_DATA_RAW_PAGE63_LOCK0_RMA_Pos = 0x7
	// Bit mask of RMA field.
	OTP_DATA_RAW_PAGE63_LOCK0_RMA_Msk = 0x80
	// Bit RMA.
	OTP_DATA_RAW_PAGE63_LOCK0_RMA = 0x80
	// Position of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE63_LOCK0_NO_KEY_STATE_Pos = 0x6
	// Bit mask of NO_KEY_STATE field.
	OTP_DATA_RAW_PAGE63_LOCK0_NO_KEY_STATE_Msk = 0x40
	// Bit NO_KEY_STATE.
	OTP_DATA_RAW_PAGE63_LOCK0_NO_KEY_STATE              = 0x40
	OTP_DATA_RAW_PAGE63_LOCK0_NO_KEY_STATE_READ_ONLY    = 0x0
	OTP_DATA_RAW_PAGE63_LOCK0_NO_KEY_STATE_INACCESSIBLE = 0x1
	// Position of KEY_R field.
	OTP_DATA_RAW_PAGE63_LOCK0_KEY_R_Pos = 0x3
	// Bit mask of KEY_R field.
	OTP_DATA_RAW_PAGE63_LOCK0_KEY_R_Msk = 0x38
	// Position of KEY_W field.
	OTP_DATA_RAW_PAGE63_LOCK0_KEY_W_Pos = 0x0
	// Bit mask of KEY_W field.
	OTP_DATA_RAW_PAGE63_LOCK0_KEY_W_Msk = 0x7

	// PAGE63_LOCK1: Lock configuration MSBs for page 63 (rows 0xfc0 through 0xfff). Locks are stored with 3-way majority vote encoding, so that bits can be set independently. This OTP location is always readable, and is write-protected by its own permissions.
	// Position of R2 field.
	OTP_DATA_RAW_PAGE63_LOCK1_R2_Pos = 0x10
	// Bit mask of R2 field.
	OTP_DATA_RAW_PAGE63_LOCK1_R2_Msk = 0xff0000
	// Position of R1 field.
	OTP_DATA_RAW_PAGE63_LOCK1_R1_Pos = 0x8
	// Bit mask of R1 field.
	OTP_DATA_RAW_PAGE63_LOCK1_R1_Msk = 0xff00
	// Position of LOCK_BL field.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_BL_Pos = 0x4
	// Bit mask of LOCK_BL field.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_BL_Msk = 0x30
	// Bootloader permits user reads and writes to this page
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_BL_READ_WRITE = 0x0
	// Bootloader permits user reads of this page
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_BL_READ_ONLY = 0x1
	// Bootloader does not permit user access to this page
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_BL_INACCESSIBLE = 0x3
	// Position of LOCK_NS field.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_NS_Pos = 0x2
	// Bit mask of LOCK_NS field.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_NS_Msk = 0xc
	// Page can be read by Non-secure software, and Secure software may permit Non-secure writes.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_NS_READ_WRITE = 0x0
	// Page can be read by Non-secure software
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_NS_READ_ONLY = 0x1
	// Page can not be accessed by Non-secure software.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_NS_INACCESSIBLE = 0x3
	// Position of LOCK_S field.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_S_Pos = 0x0
	// Bit mask of LOCK_S field.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_S_Msk = 0x3
	// Page is fully accessible by Secure software.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_S_READ_WRITE = 0x0
	// Page can be read by Secure software, but can not be written.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_S_READ_ONLY = 0x1
	// Page can not be accessed by Secure software.
	OTP_DATA_RAW_PAGE63_LOCK1_LOCK_S_INACCESSIBLE = 0x3
)

// Constants for TBMAN: For managing simulation testbenches
const (
	// PLATFORM: Indicates the type of platform in use
	// Position of HDLSIM field.
	TBMAN_PLATFORM_HDLSIM_Pos = 0x2
	// Bit mask of HDLSIM field.
	TBMAN_PLATFORM_HDLSIM_Msk = 0x4
	// Bit HDLSIM.
	TBMAN_PLATFORM_HDLSIM = 0x4
	// Position of FPGA field.
	TBMAN_PLATFORM_FPGA_Pos = 0x1
	// Bit mask of FPGA field.
	TBMAN_PLATFORM_FPGA_Msk = 0x2
	// Bit FPGA.
	TBMAN_PLATFORM_FPGA = 0x2
	// Position of ASIC field.
	TBMAN_PLATFORM_ASIC_Pos = 0x0
	// Bit mask of ASIC field.
	TBMAN_PLATFORM_ASIC_Msk = 0x1
	// Bit ASIC.
	TBMAN_PLATFORM_ASIC = 0x1
)

// Constants for USB_DPRAM: DPRAM layout for USB device.
const (
	// SETUP_PACKET_LOW: Bytes 0-3 of the SETUP packet from the host.
	// Position of WVALUE field.
	USB_DPRAM_SETUP_PACKET_LOW_WVALUE_Pos = 0x10
	// Bit mask of WVALUE field.
	USB_DPRAM_SETUP_PACKET_LOW_WVALUE_Msk = 0xffff0000
	// Position of BREQUEST field.
	USB_DPRAM_SETUP_PACKET_LOW_BREQUEST_Pos = 0x8
	// Bit mask of BREQUEST field.
	USB_DPRAM_SETUP_PACKET_LOW_BREQUEST_Msk = 0xff00
	// Position of BMREQUESTTYPE field.
	USB_DPRAM_SETUP_PACKET_LOW_BMREQUESTTYPE_Pos = 0x0
	// Bit mask of BMREQUESTTYPE field.
	USB_DPRAM_SETUP_PACKET_LOW_BMREQUESTTYPE_Msk = 0xff

	// SETUP_PACKET_HIGH: Bytes 4-7 of the setup packet from the host.
	// Position of WLENGTH field.
	USB_DPRAM_SETUP_PACKET_HIGH_WLENGTH_Pos = 0x10
	// Bit mask of WLENGTH field.
	USB_DPRAM_SETUP_PACKET_HIGH_WLENGTH_Msk = 0xffff0000
	// Position of WINDEX field.
	USB_DPRAM_SETUP_PACKET_HIGH_WINDEX_Pos = 0x0
	// Bit mask of WINDEX field.
	USB_DPRAM_SETUP_PACKET_HIGH_WINDEX_Msk = 0xffff

	// EP1_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP1_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP1_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP1_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP1_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP1_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP1_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP1_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP1_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP1_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP1_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP1_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP1_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP1_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP1_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP1_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP1_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP1_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP1_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP1_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP1_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP1_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP1_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP1_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP1_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP1_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP1_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP1_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP1_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP1_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP1_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP1_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP2_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP2_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP2_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP2_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP2_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP2_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP2_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP2_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP2_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP2_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP2_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP2_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP2_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP2_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP2_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP2_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP2_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP2_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP2_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP2_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP2_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP2_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP2_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP2_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP2_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP2_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP2_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP2_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP2_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP2_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP2_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP2_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP3_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP3_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP3_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP3_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP3_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP3_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP3_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP3_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP3_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP3_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP3_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP3_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP3_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP3_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP3_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP3_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP3_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP3_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP3_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP3_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP3_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP3_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP3_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP3_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP3_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP3_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP3_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP3_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP3_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP3_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP3_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP3_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP4_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP4_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP4_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP4_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP4_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP4_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP4_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP4_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP4_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP4_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP4_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP4_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP4_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP4_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP4_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP4_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP4_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP4_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP4_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP4_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP4_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP4_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP4_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP4_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP4_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP4_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP4_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP4_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP4_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP4_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP4_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP4_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP5_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP5_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP5_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP5_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP5_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP5_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP5_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP5_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP5_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP5_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP5_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP5_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP5_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP5_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP5_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP5_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP5_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP5_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP5_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP5_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP5_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP5_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP5_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP5_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP5_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP5_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP5_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP5_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP5_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP5_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP5_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP5_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP6_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP6_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP6_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP6_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP6_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP6_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP6_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP6_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP6_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP6_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP6_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP6_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP6_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP6_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP6_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP6_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP6_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP6_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP6_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP6_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP6_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP6_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP6_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP6_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP6_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP6_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP6_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP6_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP6_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP6_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP6_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP6_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP7_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP7_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP7_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP7_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP7_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP7_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP7_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP7_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP7_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP7_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP7_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP7_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP7_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP7_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP7_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP7_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP7_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP7_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP7_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP7_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP7_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP7_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP7_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP7_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP7_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP7_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP7_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP7_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP7_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP7_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP7_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP7_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP8_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP8_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP8_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP8_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP8_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP8_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP8_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP8_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP8_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP8_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP8_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP8_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP8_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP8_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP8_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP8_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP8_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP8_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP8_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP8_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP8_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP8_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP8_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP8_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP8_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP8_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP8_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP8_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP8_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP8_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP8_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP8_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP9_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP9_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP9_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP9_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP9_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP9_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP9_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP9_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP9_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP9_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP9_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP9_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP9_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP9_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP9_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP9_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP9_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP9_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP9_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP9_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP9_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP9_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP9_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP9_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP9_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP9_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP9_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP9_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP9_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP9_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP9_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP9_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP10_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP10_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP10_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP10_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP10_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP10_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP10_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP10_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP10_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP10_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP10_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP10_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP10_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP10_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP10_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP10_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP10_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP10_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP10_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP10_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP10_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP10_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP10_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP10_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP10_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP10_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP10_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP10_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP10_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP10_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP10_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP10_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP11_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP11_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP11_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP11_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP11_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP11_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP11_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP11_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP11_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP11_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP11_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP11_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP11_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP11_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP11_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP11_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP11_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP11_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP11_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP11_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP11_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP11_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP11_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP11_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP11_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP11_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP11_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP11_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP11_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP11_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP11_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP11_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP12_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP12_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP12_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP12_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP12_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP12_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP12_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP12_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP12_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP12_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP12_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP12_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP12_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP12_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP12_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP12_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP12_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP12_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP12_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP12_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP12_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP12_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP12_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP12_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP12_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP12_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP12_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP12_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP12_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP12_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP12_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP12_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP13_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP13_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP13_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP13_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP13_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP13_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP13_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP13_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP13_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP13_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP13_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP13_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP13_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP13_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP13_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP13_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP13_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP13_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP13_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP13_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP13_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP13_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP13_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP13_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP13_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP13_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP13_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP13_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP13_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP13_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP13_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP13_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP14_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP14_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP14_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP14_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP14_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP14_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP14_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP14_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP14_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP14_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP14_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP14_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP14_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP14_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP14_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP14_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP14_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP14_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP14_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP14_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP14_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP14_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP14_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP14_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP14_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP14_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP14_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP14_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP14_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP14_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP14_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP14_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP15_IN_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP15_IN_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP15_IN_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP15_IN_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP15_IN_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP15_IN_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP15_IN_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP15_IN_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP15_IN_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP15_IN_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP15_IN_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP15_IN_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP15_IN_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP15_IN_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP15_IN_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP15_IN_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP15_OUT_CONTROL
	// Position of ENABLE field.
	USB_DPRAM_EP15_OUT_CONTROL_ENABLE_Pos = 0x1f
	// Bit mask of ENABLE field.
	USB_DPRAM_EP15_OUT_CONTROL_ENABLE_Msk = 0x80000000
	// Bit ENABLE.
	USB_DPRAM_EP15_OUT_CONTROL_ENABLE = 0x80000000
	// Position of DOUBLE_BUFFERED field.
	USB_DPRAM_EP15_OUT_CONTROL_DOUBLE_BUFFERED_Pos = 0x1e
	// Bit mask of DOUBLE_BUFFERED field.
	USB_DPRAM_EP15_OUT_CONTROL_DOUBLE_BUFFERED_Msk = 0x40000000
	// Bit DOUBLE_BUFFERED.
	USB_DPRAM_EP15_OUT_CONTROL_DOUBLE_BUFFERED = 0x40000000
	// Position of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_PER_BUFF_Pos = 0x1d
	// Bit mask of INTERRUPT_PER_BUFF field.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_PER_BUFF_Msk = 0x20000000
	// Bit INTERRUPT_PER_BUFF.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_PER_BUFF = 0x20000000
	// Position of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Pos = 0x1c
	// Bit mask of INTERRUPT_PER_DOUBLE_BUFF field.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF_Msk = 0x10000000
	// Bit INTERRUPT_PER_DOUBLE_BUFF.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_PER_DOUBLE_BUFF = 0x10000000
	// Position of ENDPOINT_TYPE field.
	USB_DPRAM_EP15_OUT_CONTROL_ENDPOINT_TYPE_Pos = 0x1a
	// Bit mask of ENDPOINT_TYPE field.
	USB_DPRAM_EP15_OUT_CONTROL_ENDPOINT_TYPE_Msk         = 0xc000000
	USB_DPRAM_EP15_OUT_CONTROL_ENDPOINT_TYPE_Control     = 0x0
	USB_DPRAM_EP15_OUT_CONTROL_ENDPOINT_TYPE_Isochronous = 0x1
	USB_DPRAM_EP15_OUT_CONTROL_ENDPOINT_TYPE_Bulk        = 0x2
	USB_DPRAM_EP15_OUT_CONTROL_ENDPOINT_TYPE_Interrupt   = 0x3
	// Position of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_ON_STALL_Pos = 0x11
	// Bit mask of INTERRUPT_ON_STALL field.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_ON_STALL_Msk = 0x20000
	// Bit INTERRUPT_ON_STALL.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_ON_STALL = 0x20000
	// Position of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_ON_NAK_Pos = 0x10
	// Bit mask of INTERRUPT_ON_NAK field.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_ON_NAK_Msk = 0x10000
	// Bit INTERRUPT_ON_NAK.
	USB_DPRAM_EP15_OUT_CONTROL_INTERRUPT_ON_NAK = 0x10000
	// Position of BUFFER_ADDRESS field.
	USB_DPRAM_EP15_OUT_CONTROL_BUFFER_ADDRESS_Pos = 0x0
	// Bit mask of BUFFER_ADDRESS field.
	USB_DPRAM_EP15_OUT_CONTROL_BUFFER_ADDRESS_Msk = 0xffff

	// EP0_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP0_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP0_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP0_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP1_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP1_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP1_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP1_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP2_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP2_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP2_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP2_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP3_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP3_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP3_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP3_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP4_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP4_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP4_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP4_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP5_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP5_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP5_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP5_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP6_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP6_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP6_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP6_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP7_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP7_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP7_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP7_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP8_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP8_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP8_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP8_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP9_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP9_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP9_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP9_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP10_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP10_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP10_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP10_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP11_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP11_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP11_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP11_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP12_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP12_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP12_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP12_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP13_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP13_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP13_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP13_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP14_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP14_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP14_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP14_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP15_IN_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP15_IN_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff

	// EP15_OUT_BUFFER_CONTROL: Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1. Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
	// Position of FULL_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_FULL_1_Pos = 0x1f
	// Bit mask of FULL_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_FULL_1_Msk = 0x80000000
	// Bit FULL_1.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_FULL_1 = 0x80000000
	// Position of LAST_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LAST_1_Pos = 0x1e
	// Bit mask of LAST_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LAST_1_Msk = 0x40000000
	// Bit LAST_1.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LAST_1 = 0x40000000
	// Position of PID_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_PID_1_Pos = 0x1d
	// Bit mask of PID_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_PID_1_Msk = 0x20000000
	// Bit PID_1.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_PID_1 = 0x20000000
	// Position of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Pos = 0x1b
	// Bit mask of DOUBLE_BUFFER_ISO_OFFSET field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_Msk  = 0x18000000
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_128  = 0x0
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_256  = 0x1
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_512  = 0x2
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_DOUBLE_BUFFER_ISO_OFFSET_1024 = 0x3
	// Position of AVAILABLE_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_AVAILABLE_1_Pos = 0x1a
	// Bit mask of AVAILABLE_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_AVAILABLE_1_Msk = 0x4000000
	// Bit AVAILABLE_1.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_AVAILABLE_1 = 0x4000000
	// Position of LENGTH_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LENGTH_1_Pos = 0x10
	// Bit mask of LENGTH_1 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LENGTH_1_Msk = 0x3ff0000
	// Position of FULL_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_FULL_0_Pos = 0xf
	// Bit mask of FULL_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_FULL_0_Msk = 0x8000
	// Bit FULL_0.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_FULL_0 = 0x8000
	// Position of LAST_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LAST_0_Pos = 0xe
	// Bit mask of LAST_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LAST_0_Msk = 0x4000
	// Bit LAST_0.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LAST_0 = 0x4000
	// Position of PID_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_PID_0_Pos = 0xd
	// Bit mask of PID_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_PID_0_Msk = 0x2000
	// Bit PID_0.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_PID_0 = 0x2000
	// Position of RESET field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_RESET_Pos = 0xc
	// Bit mask of RESET field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_RESET_Msk = 0x1000
	// Bit RESET.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_RESET = 0x1000
	// Position of STALL field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_STALL_Pos = 0xb
	// Bit mask of STALL field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_STALL_Msk = 0x800
	// Bit STALL.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_STALL = 0x800
	// Position of AVAILABLE_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_AVAILABLE_0_Pos = 0xa
	// Bit mask of AVAILABLE_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_AVAILABLE_0_Msk = 0x400
	// Bit AVAILABLE_0.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_AVAILABLE_0 = 0x400
	// Position of LENGTH_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LENGTH_0_Pos = 0x0
	// Bit mask of LENGTH_0 field.
	USB_DPRAM_EP15_OUT_BUFFER_CONTROL_LENGTH_0_Msk = 0x3ff
)
